# -*- encoding: utf-8 -*-


import subprocess
import logging
import os
import re
import inspect

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.tools.misc import ustr

import pessoa
import model
import control
import wizard
import inherit

import monitor


_inc = 0


MOD_SIGUTILS, MOD_PYNFE, MOD_PYSPED = range(3)
MOD_D = {
    MOD_SIGUTILS: 'SigUtils',
    MOD_PYNFE: 'pynfe',
    MOD_PYSPED: 'pythonsped'
}

logger = logging.getLogger('adm.utils')


def print_log(str, tipo='I'):
    """ Tipos:
    I = Info
    D = Debug
    W = Warning
    E = Error
    """
    logger = logging.getLogger('SiG-ERP')
    tipos = {
        'I': logger.info,
        'D': logger.debug,
        'W': logger.warning,
        'E': logger.error,
        'C': logger.critical
    }
    if tipo not in tipos.keys():
        logger.exception(_("Tipo: %s, passado na funcao 'print_log' nao e valido.") % tipo)
        return False
    else:
        log = tipos.get(tipo)
        log(str)
    return str


def Log(fn):
    varList, _, _, default = inspect.getargspec(fn)
    d = {}
    if default is not None:
        d = dict((varList[-len(default):][i], v) for i, v in enumerate(default))
    file_func = os.path.basename(fn.__globals__.get('__file__', '/'))

    def f(*argt, **argd):
        global _inc
        _inc += 1
        inc = _inc
        d.update(dict((varList[i], v) for i, v in enumerate(argt)))
        d.update(argd)
        params_normal = (['%s=%s' % (x, d[x]) for x in varList])
        class_name = ''
        if argt:
            class_name = argt[0]._name
        msg_normal = '[%06d] >>> ' % inc + '('+class_name+').'+fn.func_name + '(' + ', '.join(params_normal) + ')'
        msg_normal = re.sub(r'self=<.*>, ', r'', msg_normal)
        print_log( 'LogE:' + str(msg_normal))
        ret = fn(*argt)
        msg_ret_normal = '[%06d] <<< %r' % (inc, ret)
        print_log( 'LogS:' + str(msg_ret_normal))
        return ret
    return f


def checar_dependencias(mods):
    msg = ''
    msg_ver = _('Versoes -> ')
    for mod in mods:
        if mod[0] == MOD_SIGUTILS:
            try:
                i = True
                import SigUtils
            except Exception as e:
                msg = _('%s -> Nao foi possivel carregar o Modulo SigUtils') % e
                i = False
            if i:
                if not [int(x) for x in SigUtils.__version__.split('.')] >= \
                       [int(x) for x in mod[1].split('.')]:
                    msg = _('A versao do SigUtils deve ser maior ou igual a %s! '
                            '\nVersao atual: %s') % (mod[1], SigUtils.__version__)
                else:
                    msg_ver += _("SigUtils: %s, " % SigUtils.__version__)
        elif mod[0] == MOD_PYSPED:
            try:
                i = True
                import pythonsped
            except Exception as e:
                msg = _('%s -> Nao foi possivel carregar o Modulo python-sped') % e
                i = False
            else:
                if not [int(x) for x in pythonsped.__version__.split('.')] >= \
                       [int(x) for x in mod[1].split('.')]:
                    msg = _('A versao do python-sped deve ser maior ou igual a %s! '
                            '\nVersao atual: %s') % (mod[1], pythonsped.__version__)
                else:
                    msg_ver += _("Sped: %s, " % pythonsped.__version__)
        elif mod[0] == MOD_PYNFE:
            try:
                import pynfe
            except Exception as e:
                msg = _('%s -> Nao foi possivel carregar o Modulo python-nfe') % e
            else:
                if not [int(x) for x in pynfe.__version__.split('.')] >= \
                       [int(x) for x in mod[1].split('.')]:
                    msg = _('A versao do python-nfe deve ser maior ou igual a %s! '
                            '\nVersao atual: %s') % (mod[1], pynfe.__version__)
                else:
                    msg_ver += _("PyNFe: %s" % pynfe.__version__)
    logger.info(msg_ver)
    if msg:
        raise osv.except_osv(_('Atencao!'), msg)
    return True


def check_sigutils_ver(v, execao=False):
    """Compara a versao do SigUtils ser menor que a versao passada no parametro,
    exibi uma mensagem e sai encerra o server"""
    execao = True
    try:
        import SigUtils
    except Exception as err:
        if execao:
            raise err
        else:
            print_log('%s: %s' % (str(err), 'SigUtils nao instalado.'), 'C')
            os._exit(0)
    if not SigUtils.__version__ >= v:
        msg = 'A versao do SigUtils deve ser maior ou igual a %s! Versao Atual: %s' % (v, SigUtils.__version__)

        if execao:
            raise osv.except_osv(_('Mensagem:'), _('Esta Forma de Pagamento esta bloqueada para exclusao!'))
        else:
            print_log(msg, 'C')
            #os._exit(0)
    return True


def dpkg_state(packagename):
    """
    Verifica se um determinado pacote esta instalado no sistema
    @param packagename: Nome do pacote
    @return: @raise or @bool:
    """
    logger.info(ustr(_("Verificando se o pacote '{0}' esta instalado no sistema.").format(packagename)))

    devnull = open(os.devnull, "w")
    #retval = subprocess.call(["dpkg", "-s", packagename], stdout=devnull, stderr=subprocess.STDOUT)
    # devnull.close()
    # if retval != 0:
    #     msg = _("Tente: apt-get install {0}").format(packagename)
    #     raise osv.except_osv(_(ustr(_('Pacote de sistema nao esta instalado.'))), _(ustr(msg)))
    return True


def messagebox(message, context=None):
    context = context and context.copy() or {}
    context.update(({'mensagem': message}))

    value = {
        'view_type': 'form',
        'view_mode': 'form',
        'res_model': 'adm.messagebox',
        'type': 'ir.actions.act_window',
        'target': 'new',
        'context': context
    }
    if context.get('id_messagebox'):
        value.update({'res_id': context['id_messagebox']})
    return value

def capitalize_all( texto ):
    lista = texto.lower().split()
    lista_capitalize = []
    for a in lista:
        lista_capitalize.append( a.capitalize() )
    b = ' '
    return b.join( lista_capitalize )



checar_dependencias([(MOD_SIGUTILS, '2.4.11')])

