# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#############################################################################

import datetime
from openerp.osv import osv
from SigUtils import datas


YEAR = datetime.date.today().year


class adm_calendario_feriados(osv.Model):
    """
    Tabela contendo os feriados municipais, estaduais e federais
    """
    _name = 'adm.calendario.feriados'
    _inherit = 'adm.calendario.feriados'

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'admcfe_data' in vals:
            vals['admcfe_dia'] = int(vals['admcfe_data'][8:10])
            vals['admcfe_mes'] = int(vals['admcfe_data'][5:7])
        res = super(adm_calendario_feriados, self).write(cr, uid, ids, vals, context=context)
        return res

    def afmcfe_identificar_data(self, cr, uid, ids, data, municipio=None, uf=None, pais=None,
                                context=None):
        """
        Função para verificar se a data informada é um feriado dia util ou final de semana
        :param cr:
        :param uid:
        :param ids:
        :param data: data no formato 'yyyy-mm-dd'
        :param municipio: ID do Municipio para verificar feriados municipais, se não informado será usado o da propria empresa
        :param uf: ID do Estado(UF) para verificar feriados estaduais, se não informado será usado o da propria empresa
        :param pais: ID do Pais para verificar feriados federais, se não informado será usado o da propria empresa
        :param context:
        :return: U para dia Util, F para Feriado e A para Final de semana(Sabado e domingo)
        """
        feriado_ids = self.search(cr, uid, [], context=context)
        _data_str = datas.sigDatas(data).getDataNoFormato('I', 'S', '-')
        _data_date = datetime.datetime.strptime(_data_str, "%Y-%m-%d")
        if _data_date.weekday() > 4:  #Sabado ou Domingo(Final de semana)
            res = 'A'
        else:
            res = 'U'
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
            _company_browse = self.pool.get('res.company').browse(cr, uid, company_id)
            _pessoa_id = _company_browse.rescon_pessoa_id.id
            _pais_id = self.pool.get('adm.pessoa')._get_base_endereco(cr, uid, [_pessoa_id], None,
                                                                      None,
                                                                      tipo='pais',
                                                                      name_id='id').get(_pessoa_id,
                                                                                        False)
            _uf_id = self.pool.get('adm.pessoa')._get_base_endereco(cr, uid, [_pessoa_id], None,
                                                                      None,
                                                                      tipo='estado',
                                                                      name_id='id').get(_pessoa_id,
                                                                                        False)
            _cidade_id = self.pool.get('adm.pessoa')._get_base_endereco(cr, uid, [_pessoa_id], None,
                                                                      None,
                                                                      tipo='cidade',
                                                                      name_id='id').get(_pessoa_id,
                                                                                        False)
            for x in self.browse(cr, uid, feriado_ids):
                if x.admcfe_data != _data_str:
                    continue
                if x.admcfe_tipo == 'I':
                    res = 'F'
                    break
                if x.admcfe_tipo == 'F':
                    if x.admcfe_pais_id.id != _pais_id:
                        continue
                    res = 'F'
                if x.admcfe_tipo == 'E':
                    if x.admcfe_uf_id.id != _uf_id:
                        continue
                    res = 'F'
                if x.admcfe_tipo == 'M':
                    if x.admcfe_cidade_id.id != _cidade_id:
                        continue
                    res = 'F'
        return res