# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class adm_forma_pgto(osv.Model):
    _inherit = 'adm.forma.pgto'  # admfpg

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        """
        Metodo utilizado para localizar o nome da forma de pagamento pela sigla
        se houver.
        """
        context = context and context.copy() or {}
        ids = []
        if len(name) in (1, 2, 3):
            ids = self.search(cr, uid, [('sigla', '=', name.upper())] + args, limit=limit, context=context)
        if not ids:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def create(self, cr, uid, vals, context=None):
        """
        Metodo utilizado para no momento da criacao converter a sigla para
        maiusculos.
        """
        context = context and context.copy() or {}
        if vals.get('sigla'):
            vals.update({'sigla': vals['sigla'].upper()})

        result = super(adm_forma_pgto, self).create(cr, uid, vals, context)
        return result

    def unlink(self, cr, uid, ids, context=None):
        """
        Metodo utilizado para no momento da exclusao verificar se a forma de pgto
        esta bloqueada, se estiver, nao permitir excluir.
        """
        context = context and context.copy() or {}
        if self.read(cr, uid, ids, ['bloqueio'], context=context)[0]['bloqueio']:
            raise osv.except_osv(_('Mensagem:'),
                                 _('Esta Forma de Pagamento esta bloqueada para exclusao!'))
        return super(adm_forma_pgto, self).unlink(cr, uid, ids, context=context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('contrato_forma_pagamento'):
            #Quando esta em um contrato permite exibir somente formas de pagamento com uma parcela
            obj_condicao_forma = self.pool.get('adm.condicao.forma')
            obj_adm_condicao = self.pool.get('adm.condicao')
            condicoes_a_vista_ids = obj_adm_condicao.search(cr,uid,[('qtde_parcelas','=',1)], context=context)
            condicao_ids = obj_condicao_forma.search(cr,uid,
                                                     [('adm_condicao_id','in',condicoes_a_vista_ids),
                                                     ], context=context)
            args.append(('condicao_forma_ids','in',condicao_ids))

        res = super(adm_forma_pgto, self).search(cr, uid, args, offset, limit, order, context,
                                                  count)
        return res

    def admfpg_forma_pgto_tipo_cartao_credito(self, cr, uid, forma, context=None):
        """
        Verifica se a forma de pagamento eh cartao de credito.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param forma: ID da forma de pagamento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type forma: long
        :type context: dict
        :return: verdadeiro se a forma de pagamento for cartao de credito
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        return self.read(cr, uid, forma, ['tipo_cartao'], context).get('tipo_cartao') == 'C'

    def admfpg_forma_pgto_tipo_cartao_debito(self, cr, uid, forma, context=None):
        """
        Verifica se a forma de pagamento eh cartao de debito.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param forma: ID da forma de pagamento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type forma: long
        :type context: dict
        :return: verdadeiro se a forma de pagamento for cartao de credito.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        return self.read(cr, uid, forma, ['tipo_cartao'], context).get('tipo_cartao') == 'D'

    def admfpg_forma_pgto_tipo_cartao(self, cr, uid, forma, context=None):
        """
        Verifica se a forma de pagamento eh cartao de credito ou debito.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param forma: ID da forma de pagamento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type forma: long
        :type context: dict
        :return: verdadeiro se a forma de pagamento for cartao de debito.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        cc = self.admfpg_forma_pgto_tipo_cartao_credito(cr, uid, forma, context)
        cd = self.admfpg_forma_pgto_tipo_cartao_debito(cr, uid, forma, context)

        return cc or cd

    def admfpg_forma_pgto_tipo_cheque(self, cr, uid, forma, context=None):
        """
        Verifica se a forma de pagamento eh cheque

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param forma: ID da forma de pagamento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type forma: long
        :type context: dict
        :return: verdadeiro se a forma de pagamento for cheque.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        return self.read(cr, uid, forma, ['sigla'], context).get('sigla') == 'CH'

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

