# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class adm_transacao_capa(osv.Model):
    _name = 'adm.transacao.capa'
    _inherit = 'adm.transacao.capa'

    def _admtrc_inserir_transacao(self, cr, uid, trs_operacao, trs_objeto, trs_id, trs_capa_id, context):
        """
        Inserir Capa e itens da transacao
           @ trs_operacao = uma das opcoes de adm_operacao
           @ trs_objeto = classe ou objeto da transacao(documento:orcamento, pedido, etc)
           @ trs_id = id do documento
           @ trs_capa_id, id da capa da transacao, quando estiver grando outros documentos da mesma transacao
        """
        if trs_capa_id == 0:
            capa_id = self.create(cr, uid, {'admtrc_operacao': trs_operacao}, context)
        else:
            capa_id = trs_capa_id
        trs_ref = trs_objeto.strip() + ',' + str(trs_id)
        item_id = self.pool.get('adm.transacao.itens').create(cr, uid,
                                                              {'admtri_capa_id': capa_id, 'admtri_objeto_ref': trs_ref},
                                                              context)
        return item_id

    def _admtrc_pedido(self, cr, uid, trs_operacao, trs_objeto, trs_id, context):
        if not self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'orcamentos', context):
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context)
        obj_adm_transacao_itens = self.pool.get('adm.transacao.itens')
        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        dados_pedido = obj_estoquepedidos_pedido_capa.read(cr, uid, trs_id, ['orcamento_id'], context=context)
        if isinstance(dados_pedido, list):
            dados_pedido = dados_pedido[0]
        orcamento_id = dados_pedido['orcamento_id']
        if not orcamento_id:
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)
        _ref = 'orcamentos.orcamento.capa,' + str(orcamento_id[0])
        adm_transacao_itens_id = obj_adm_transacao_itens.search(cr, uid, [('admtri_objeto_ref', '=', _ref)], context=context)
        if adm_transacao_itens_id:
            dados_adm_transacao_itens = obj_adm_transacao_itens.read(cr, uid, adm_transacao_itens_id[0],
                                                                     ['admtri_capa_id'], context=context)
            if dados_adm_transacao_itens:
                return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id,
                                                      dados_adm_transacao_itens['admtri_capa_id'][0], context=context)
        return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)

    def _admtrc_nota(self, cr, uid, trs_operacao, trs_objeto, trs_id, context):
        obj_adm_transacao_itens = self.pool.get('adm.transacao.itens')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        dados_nf = obj_fiscal_nf_capa.read(cr, uid, trs_id, ['fisnfc_pedido_ids'], context=context)
        if isinstance(dados_nf, list):
            dados_nf = dados_nf[0]
        nf_pedido_ids = dados_nf['fisnfc_pedido_ids']
        if not nf_pedido_ids:
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)
        lista_pedidos = ''
        for x in nf_pedido_ids:
            if not lista_pedidos:
                lista_pedidos += str(x)
            else:
                lista_pedidos += ', ' + str(x)
        _ref = 'estoquepedidos.pedido.capa,' + lista_pedidos
        adm_transacao_itens_id = obj_adm_transacao_itens.search(cr, uid, [('admtri_objeto_ref', '=', _ref)], context=context)
        if adm_transacao_itens_id:
            dados_adm_transacao_itens = obj_adm_transacao_itens.read(cr, uid, adm_transacao_itens_id[0],
                                                                     ['admtri_capa_id'], context=context)
            if dados_adm_transacao_itens:
                return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id,
                                                      dados_adm_transacao_itens['admtri_capa_id'][0], context=context)
        return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)

    def _admtrc_fatura(self, cr, uid, trs_operacao, trs_objeto, trs_id, context):
        if not self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context=context):
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)
        obj_adm_transacao_itens = self.pool.get('adm.transacao.itens')
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')
        dados_financeiro_fatura = obj_financeiro_fatura.read(cr, uid, trs_id, ['nf_id'], context=context)
        if isinstance(dados_financeiro_fatura, list):
            dados_financeiro_fatura = dados_financeiro_fatura[0]
        _nf_id = dados_financeiro_fatura['nf_id']
        if not _nf_id:
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)
        _ref = 'fiscal.nf.capa,' + str(_nf_id[0])
        adm_transacao_itens_id = obj_adm_transacao_itens.search(cr, uid, [('admtri_objeto_ref', '=', _ref)], context=context)
        if adm_transacao_itens_id:
            dados_adm_transacao_itens = obj_adm_transacao_itens.read(cr, uid, adm_transacao_itens_id[0],
                                                                     ['admtri_capa_id'], context=context)
            if dados_adm_transacao_itens:
                return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id,
                                                      dados_adm_transacao_itens['admtri_capa_id'][0], context=context)
        return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)

    def _admtrc_titulo(self, cr, uid, trs_operacao, trs_objeto, trs_id, context):
        obj_adm_transacao_itens = self.pool.get('adm.transacao.itens')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        dados_financeiro_titulo = obj_financeiro_titulo.read(cr, uid, trs_id, ['fatura_id'], context=context)
        if isinstance(dados_financeiro_titulo, list):
            dados_financeiro_titulo = dados_financeiro_titulo[0]
        _fatura_id = dados_financeiro_titulo['fatura_id']
        if not _fatura_id:
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)
        _ref = 'financeiro.fatura,' + str(_fatura_id[0])
        adm_transacao_itens_id = obj_adm_transacao_itens.search(cr, uid, [('admtri_objeto_ref', '=', _ref)], context=context)
        if adm_transacao_itens_id:
            dados_adm_transacao_itens = obj_adm_transacao_itens.read(cr, uid, adm_transacao_itens_id[0],
                                                                     ['admtri_capa_id'], context=context)
            if dados_adm_transacao_itens:
                return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id,
                                                      dados_adm_transacao_itens['admtri_capa_id'][0], context=context)
        return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)

    def _admtrc_tesouraria(self, cr, uid, trs_operacao, trs_objeto, trs_id, context):
        obj_adm_transacao_itens = self.pool.get('adm.transacao.itens')
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        dados_financeiro_tesouraria_movimento = obj_financeiro_tesouraria_movimento.read(
            cr, uid, trs_id, ['fintem_titulo_ids'], context=context)
        if isinstance(dados_financeiro_tesouraria_movimento, list):
            dados_financeiro_tesouraria_movimento = dados_financeiro_tesouraria_movimento[0]
        _titulo_ids = dados_financeiro_tesouraria_movimento.get('fintem_titulo_ids')
        if not _titulo_ids:
            return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0,
                                                  context=context)
        _ref = 'financeiro.titulo,' + str(_titulo_ids[0])
        adm_transacao_itens_id = obj_adm_transacao_itens.search(cr, uid, [('admtri_objeto_ref', '=', _ref)], context=context)
        if adm_transacao_itens_id:
            dados_adm_transacao_itens = obj_adm_transacao_itens.read(cr, uid, adm_transacao_itens_id[0],
                                                                     ['admtri_capa_id'], context=context)
            if dados_adm_transacao_itens:
                return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id,
                                                      dados_adm_transacao_itens['admtri_capa_id'][0], context=context)
        return self._admtrc_inserir_transacao(cr, uid, trs_operacao, trs_objeto, trs_id, 0, context=context)

    def admtrc_criar_transacao(self, cr, uid, trs_operacao, trs_id, context=None):
        context = context and context.copy() or {}
        if trs_operacao == 'orca':
            self._admtrc_inserir_transacao(cr, uid, trs_operacao, 'orcamentos.orcamento.capa', trs_id, 0, context)
            return True
        if trs_operacao == 'pedi':
            self._admtrc_pedido(cr, uid, trs_operacao, 'estoquepedidos.pedido.capa', trs_id, context)
            return True
        if trs_operacao == 'nota':
            self._admtrc_nota(cr, uid, trs_operacao, 'fiscal.nf.capa', trs_id, context)
            return True
        if trs_operacao == 'fatu':
            self._admtrc_fatura(cr, uid, trs_operacao, 'financeiro.fatura', trs_id, context)
            return True
        if trs_operacao == 'titu':
            self._admtrc_titulo(cr, uid, trs_operacao, 'financeiro.titulo', trs_id, context)
            return True
        if trs_operacao == 'almx':
            self._admtrc_inserir_transacao(cr, uid, trs_operacao, 'almoxarifado.requisicao.capa', trs_id, 0, context)
            return True
        if trs_operacao == 'teso':
            self._admtrc_tesouraria(cr, uid, trs_operacao, 'financeiro.tesouraria.movimento', trs_id, context)
            return True
