# -*- coding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools import config
from openerp.tools.translate import _
from openerp import SUPERUSER_ID
from unicodedata import normalize
from SigUtils import cpf, cnpj, sigerp
from SigUtils.numero import getNumeroEditado
from decimal import Decimal
import locale
import re
import pytz
import math
from functools import wraps
from logging import getLogger
import time
from dateutil.parser import parse
import string
import ConfigParser


READ_LOAD = '_classic_write'

_qtd_exec = {}
_tempo_exec = []
_logger_tempo_exec = getLogger("[CALCULO DE TEMPO DE EXCUCAO]")
_logger_quantidade = getLogger("[CALCULA QUANTIDADE DE EXEC ]")


def _links_get(self, cr, uid, context=None):
    context = context and context.copy() or {}
    obj = self.pool.get('res.request.link')
    ids = obj.search(cr, uid, [], context=context)
    res = obj.read(cr, uid, ids, ['object', 'name'], context=context)
    return [(r['object'], r['name']) for r in res]


def capitalize_all( texto ):
    lista = texto.lower().split()
    lista_capitalize = []
    for a in lista:
        lista_capitalize.append( a.capitalize() )
    b = ' '
    return b.join( lista_capitalize )


def log_debug(func):
    from openerp.tools import config as openerp_config
    from datetime import datetime
    import logging
    import time
    no_logger = ['adm.utils']

    def wrapper(*args, **kwargs):
        if args[0]._name in no_logger and openerp_config.get('log_level').upper() == 'DEBUG':
            _logger = logging.getLogger(__name__)
            _logger.setLevel(logging.DEBUG)  # Enable DEBUG mode
        elif openerp_config.get('log_level').upper() == 'DEBUG':
            _logger = args[0]._logger
        else:
            return func(*args, **kwargs)
        start_time = time.time()
        result = func(*args, **kwargs)
        if _logger.isEnabledFor(logging.DEBUG):
            end_time = time.time()
            _logger.debug('%s :: %s@%s|%s (%.3fs)' % (
                func.__doc__ and func.__doc__.replace(
                    '\n', '', 1).strip().split('\n')[0] or '', func.func_name,
                datetime.fromtimestamp(start_time).strftime('%H:%M:%S'),
                datetime.fromtimestamp(end_time).strftime('%H:%M:%S'), end_time - start_time))

        return result

    return wrapper

class admConfiguracao(object):
    """
    Classe para manipulacao de arquivos de configuracao
    A pasta para arquvios de configuraqcao está definida no parametro do erp no
       atributo config_dir e na falta deste em addons_path

    """
    def __init__(self, nomeArquivo, secao=None, cripto=False, diretorio=None):
        """
        metodo de inicializacao da classe
        :param nomeArquivo: Nome do arquivo de configuracao, sem diretorio e sem sufixo
        :type nomeArquivo: str
        :param secao: seção dos parametros dentro do arquivo, é o que fica entre colchetes [ xxx ]
        :type secao: str
        :param cripto: Identifica se os dados ficaram criptografados ou nao dentro do arquivo
        :type cripto: bool
        :param diretorio: diretorio para arquivo de configuracao, não informando usar o addons_path
        """
        self.secao = secao
        self.cripto = cripto
        if diretorio:
            _dir = diretorio
        else:
            _dir = config.get('config_dir') or config.get('addons_path')
        self.arq = '%s/%s.config' % (_dir, nomeArquivo)
        self.cfg = ConfigParser.ConfigParser()
        self.parametros = []
        self.cfg.read(self.arq)
        if self.secao:
            if not self.cfg.has_section(self.secao):
                self.cfg.add_section(self.secao)
            for parametro in self.cfg.options(self.secao):
                _dado = self.cfg.get(self.secao, parametro)
                if self.cripto:
                    _dado = sigerp._crypto_sig('d', _dado)
                _dado = _dado.decode('utf-8')
                exec("self.%s = '%s'" % (parametro, _dado))
        return

    def addParametro(self, parametro, valor):
        """
        Metodo para inclusao de parametro no arquivo de configuracao
        :param parametro: nome do parametro a ser incluido dentro da secao
        :type parametro: str
        :param valor: Valor do parametro a ser incluido
        :type valor: str
        """
        if not self.secao:
            return False
        if not self.cfg.has_section(self.secao):
            self.cfg.add_section(self.secao)
        if isinstance(valor, (list, tuple, int, float)):
            valor = str(valor)
        _dado = valor.encode('utf-8')
        if self.cripto:
            _dado = sigerp._crypto_sig('e', _dado)
        self.cfg.set(self.secao, parametro, _dado)
        exec("self.%s = '%s'" % (parametro, valor))
        return

    def gravarConfiguracao(self):
        """
        Salva os parametros incluidos no arquivo
        """
        f = open(self.arq, 'w')
        self.cfg.write(f)
        f.close()
        return

# a = admConfiguracao('teste', 'luiz')
# a.addParametro('tamanho', '4')
# a.addParametro('idade', '59')
# a.tamanho
# a.gravarConfiguracao()


class adm_utils(osv.Model):
    _name = 'adm.utils'
    _auto = False


    def admutl_capitalizar_adm_nome(self, cr, uid, texto, context=None):
        context = context and context.copy() or {}
        adm_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'adm', 'admpar_capitaliza_nomes', context=context)
        _capitalizar = adm_parametro_dados.get('admpar_capitaliza_nomes')
        if _capitalizar:
            texto = capitalize_all(texto)
        return texto

    def admutl_get_dados_classe(self, cr, uid, classe, ids, campos=[], context=None, load=''):
        """
        Retorna um dicionario onde a chave eh o id do registro e o valor eh os dados do registro
        """
        context = context and context.copy() or {}
        res = {}
        if not isinstance(ids, list):
            ids = [ids]

        if not ids:
            return res

        obj = self.pool.get(classe)
        map(lambda x: res.update({x.get('id'): x}), obj.read(cr, uid, ids, campos, context, load))
        return res

    @staticmethod
    def admutl_converte_date_tz(data, tz="UTC", formato_retorno='S'):
        """
        Converte data com timezone para o timezone desejado
        @data: no formato string no formato "AAAA-MM-DDTHH:MM:SS-03:00"
        @tz: timezone a ser utilizado
        @formato_retorno: S = string, D = datetime
        """
        _dt = parse(data)
        local = pytz.timezone(tz)
        _new = _dt.astimezone(local)
        if formato_retorno == 'D':
            return _new
        else:
            return str(_new)


    def admutl_converte_tz(self, cr, uid, data, tz_origem=None, tz_destino=None, context=None):
        """
        Função para converter data de um timezone para outro
        @param data: data no formato datetime para ser convertida
        @param tz_origem: Timezone em que a data se encontra. Se None, vai buscar timezone do usuário
        @param tz_destino: Timezne em que a data será convertida. Se None, vai buscar timezone do usuário
        @return: data no formato datetime convertida para o novo timezone
        """

        context = context and context.copy() or {}
        def _buscar_tz():
            if context and context.get('tz'):
                tz_name = context['tz']
            else:
                tz_name = self.pool.get('res.users').read(cr, uid, uid, ['tz'], context=context)['tz']
            obj = pytz.timezone(tz_name)
            return obj

        context = context and context.copy() or {}
        if tz_origem:
            obj_origem = pytz.timezone(tz_origem)
        else:
            obj_origem = _buscar_tz()
        if tz_destino:
            obj_destino = pytz.timezone(tz_destino)
        else:
            obj_destino = _buscar_tz()
        x = obj_origem.localize(data, is_dst=False)
        return x.astimezone(obj_destino)

    def admutl_datetime_to_timezone(self, cr, uid, data, formato=False):
        """
        Transforma a data do timezone UTC(Timezone padrão armazenado na base do open)
        para o timezone do usuário(formato de apresentação)
        @param cr:
        @param uid:
        @param data: {datetime}  data recebida direto de um campo datetime de uma classe do open
        @return: {string} data no formato YYYY-MM-DD HH:MM:SS
        """
        if not formato:
            format = str(locale.nl_langinfo(locale.D_FMT).replace('%y', '%Y')) + ' ' + str(
                locale.nl_langinfo(locale.T_FMT))
        else:
            format = formato
        _dt = fields.datetime.context_timestamp(cr, uid, data).strftime(format).replace('/', '-')
        return _dt

    def get_sequence(self, cr, uid, classe, code, context=None):
        context = context and context.copy() or {}
        valor = self.pool.get('ir.sequence').next_by_code(cr, uid, code, context)
        if not valor:
            self.pool.get(classe).sequence_criar(cr, uid)
            valor = self.pool.get('ir.sequence').next_by_code(cr, uid, code, context)
        return valor

    def get_controle(self, cr, uid, name, parent_id=False, company_id=False, context=None):
        """
        Metodo utilizado para gerar o codigo de controle onde haja os atributos
        parent_id, controle
        return: integer
        """
        context = context and context.copy() or {}
        obj = self.pool.get(name)
        table_name = obj.__class__.__name__
        table_name = table_name.replace('.', '_')
        if obj.__class__._table:
            table_name = obj.__class__._table
        id_s = []
        clause = '= %s' % str(parent_id) if parent_id else 'IS NULL'
        clause_company = ''
        if company_id:
            clause_company = ' AND company_id = %s ' % str(company_id)
        clause += clause_company
        sql = """
            SELECT MAX(controle)
            FROM %s
            WHERE parent_id %s
        """ % (table_name, clause)
        cr.execute(sql)
        next_val = cr.fetchall()[0][0]
        next_val = next_val + 1 if next_val else 1
        if parent_id:
            level = 100
            ids = [parent_id]
            while len(ids):
                cr.execute(
                    'SELECT DISTINCT parent_id FROM ' + table_name + ' WHERE id in (' + ','.join(
                        map(str, ids)) + ')')
                ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                if ids:
                    id_s.append(ids[0])
                if not level:
                    return False
                level -= 1
        return next_val

    def get_nome_conta(self, cr, uid, ids, name, company_id=False, context=None):
        """
        Metodo utilizado para obter o codigo da conta efetuando uma busca recursiva, usando
        o atributo controle para montar o codigo
        return: tupla {id: codigo}
        """
        context = context and context.copy() or {}
        obj = self.pool.get(name)
        table_name = obj.__class__.__name__

        # O recurso abaixo foi necessario pelo fato deste metodo ser chamado a maioria das vezes por
        # campos funcionais. O campo funcional sempre e executado com o usuarui admin, entao nao
        # estava sendo respeitado a company que o usuario estava logado. Atividade 101124
        if not company_id and context.get('force_company') \
                and filter(lambda i: i == 'company_id', obj._all_columns):
            company_id = context.get('force_company')

        table_name = table_name.replace('.', '_')
        if obj.__class__._table:
            table_name = obj.__class__._table
        res = {}
        sql = """
            SELECT
                {company}
                tbl1.id        AS id,
                tbl1.parent_id AS parent_id,
                tbl1.controle  AS controle,
                tbl1.{campo}   AS conta,
                tbl2.controle  AS controle_parent
            FROM
                {table_name} tbl1
            LEFT JOIN {table_name} tbl2
                ON tbl1.parent_id = tbl2.id
            WHERE tbl1.id IN ({obj_ids})
        """.format(table_name=table_name, obj_ids=', '.join(map(lambda i: str(i), ids)),
                   company='tbl1.company_id AS company_id,' if obj._columns.get('company_id')
                   else '',
                   campo='conta' if table_name == 'estoque_categoria' else 'codigo')
        cr.execute(sql)
        obj_dados = cr.dictfetchall()
        for dado in obj_dados:
            name = ''
            id_s = [dado.get('id')]
            if dado.get('parent_id') and dado.get('controle') in (10, 100, 1000, 10000):
                sql = """
                    SELECT id, {campo} AS conta, controle
                    FROM {table_name}
                    WHERE
                        parent_id = {parent_id}
                        AND {campo} IS NOT NULL
                """.format(table_name=table_name, parent_id=dado.get('parent_id'),
                           campo='conta' if table_name == 'estoque_categoria' else 'codigo')
                cr.execute(sql)
                for dado2 in cr.dictfetchall():
                    conta = dado2.get('conta').split('.')
                    if dado.get('controle') == 10:
                        controle = '.%02d' % dado2.get('controle')
                    elif dado.get('controle') == 100:
                        controle = '.%03d' % dado2.get('controle')
                    elif dado.get('controle') == 1000:
                        controle = '.%04d' % dado2.get('controle')
                    else:
                        controle = '.%05d' % dado2.get('controle')
                    res[dado2.get('id')] = '.'.join(conta[:-1]) + controle
            parent_ids = id_s
            while parent_ids:
                cr.execute(
                    'SELECT DISTINCT parent_id FROM ' + table_name + ' WHERE id in (' +
                    ','.join(map(str, parent_ids)) + ')')
                parent_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                if parent_ids:
                    id_s.append(parent_ids[0])
            id_s.reverse()
            sql = """
                SELECT
                    {company}
                    tbl2.parent_id  AS parent_id,
                    tbl2.{campo}   AS controle
                FROM
                    {table_name} tbl1
                LEFT JOIN {table_name} tbl2
                    ON tbl1.parent_id = tbl2.id
                WHERE tbl1.id IN ({obj_ids})
                ORDER BY tbl1.id
            """.format(table_name=table_name, obj_ids=', '.join(map(lambda i: str(i), id_s)),
                       company='tbl1.company_id AS company_id,' if obj._columns.get('company_id')
                       else '', campo='conta' if table_name == 'estoque_categoria' else 'codigo')
            cr.execute(sql)
            obj_dados1 = cr.dictfetchall()
            if dado.get('conta') and table_name != 'gerencial_evento':
                dado['controle'] = str(dado.get('conta')).split('.')[-1]
            obj_dados1.append(dado)
            for dado1 in obj_dados1:
                if not dado1.get('controle'):
                    continue
                controle = str(dado1.get('controle')).split('.')[-1]
                if int(controle) == 10:
                    controle = '%02d' % int(controle)
                elif int(controle) == 100:
                    controle = '%03d' % int(controle)
                elif int(controle) == 1000:
                    controle = '%04d' % int(controle)
                elif int(controle) == 10000:
                    controle = '%05d' % int(controle)
                name += str(controle) + '.'
            res[dado.get('id')] = name[:-1]
        return res

    def ultima_data_pedido(self, cr, uid, ids, produto_id, pessoa_id=False, context=None):
        context = context and context.copy() or {}
        if pessoa_id == False:
            spessoa = 'c.pessoa_id'
        else:
            spessoa = str(pessoa_id)
        try:
            cr.execute("""
                select max(data_emissao)  as data_emissao
                  from fiscal_nf_capa fnc,
                       estoque_movimento_itens emi
                 where fnc.id = emi.nf_id
                   and fnc.tipo = 'E'
                   and fnc.state = '3'
                   and fnc.destinatario_id = %s
                   and emi.produto_id = %s
                """ % (spessoa, produto_id))
            res = cr.fetchone()[0]
        except:
            res = ""
        return res

    def admutl_convert_2_decimal(self, value, places=2):
        res = value
        places = Decimal(10) ** (places * -1)
        if isinstance(value, (int, long, float)):
            valor = Decimal(str(value))
            res = valor.quantize(exp=places)
        elif isinstance(value, Decimal):
            res = value.quantize(exp=places)
        return res

    @log_debug
    def efetuar_rateio(self, valor_total, elementos=[]):

        """
        Funcao generica para rateio.
        Passar lista de lista ex:[[1,10],[2,20],[3,30]], retornara a mesma lista mas com os valores do segundo indice rateados
        """
        float_round = fields.float_round
        if elementos:
            vr_total_elementos = sum(map(lambda x: x[1], elementos)) or 0
            if vr_total_elementos > 0:
                percentual = float(valor_total) / float(
                    vr_total_elementos)  # calcula o percentual sobre o valor informado no segundo elemento lista/tupla
                maior_elemento = [0, 0.0]
                for valor in elementos:
                    if isinstance(valor,
                                  list):  #verifica se os elementos contidos sao tuplas ou listas,
                        valor_rateado = float_round((valor[1] * percentual), 2)  #calcula o valor rateado
                        valor[1] = valor_rateado
                        if valor[1] > maior_elemento[
                            1]:  #pega maior elemento para fazer o tratamento de diferenca de valores
                            maior_elemento = valor
                vr_total_diferenca = sum(map(lambda x: x[1], elementos)) or 0
                if maior_elemento != [0, 0.0]:
                    indice = elementos.index(maior_elemento)
                else:
                    indice = elementos.index(elementos[0])
                if valor_total > vr_total_diferenca:  #verifica e ajusta diferencas
                    diff = float_round((valor_total - vr_total_diferenca), 2)
                    elementos[indice][1] = float_round(elementos[indice][1] + diff, 2)
                if valor_total < vr_total_diferenca:
                    diff = float_round((vr_total_diferenca - valor_total), 2)
                    elementos[indice][1] = float_round(elementos[indice][1] - diff, 2)
            # SE A QUANTIDADE PRESENTE NOS ELEMENTOS FOR ZERO, REALIZA O RATEIO DO VALOR TOTAL PARA CADA ELEMENTO
            elif vr_total_elementos == 0:
                len_elementos = len(elementos) or 1

                dec_valor_total = self.admutl_convert_2_decimal(valor_total, places=2)
                dec_valor_por_elemento = self.admutl_convert_2_decimal(
                    dec_valor_total / len_elementos, places=5)

                total_inserido = 0
                total = 0
                cont = 1

                for i in elementos:
                    total += dec_valor_por_elemento
                    if cont == len_elementos:
                        res = total - (total_inserido + dec_valor_por_elemento)
                        dec_valor_por_elemento += res

                    dec_total_por_elemento = self.admutl_convert_2_decimal(dec_valor_por_elemento,
                                                                           places=2)
                    i[1] = float_round(float(dec_total_por_elemento), 2)
                    total_inserido += dec_total_por_elemento
                    cont += 1

        return elementos

    def get_fonema(self, string, tamanho_fonema=5):
        def _calc_fonema(nome, _tam):
            nome = nome + '     '
            inicio = ''
            fim = ''
            #Tratar inicio
            if nome[:1] in ('A', 'E', 'I', 'O', 'U'):
                inicio = nome[:1]
                nome = nome[1:]
            else:
                if nome[:2] in ('HA', 'HE', 'HI', 'HO', 'HU'):
                    inicio = nome[1:2]
                    nome = nome[2:]
                else:
                    if nome[:1] == 'H':
                        nome = nome[1:]
                    else:
                        if nome[:3] in ('EX', 'EXE', 'EXI', 'EXO', 'EXHU'):
                            inicio = 'Z'
                            nome = nome[3:]
                            # Tratar segunda posicao
            if not inicio:
                if nome[1:2] == 'R' and nome[2] not in ('A', 'E', 'I', 'O', 'U'):
                    nome = nome[:1] + 'R' + nome[3:]
                else:
                    if nome[1:3] in ('EXE', 'EXI', 'EXA', 'EXO', 'EXU'):
                        nome = nome[:1] + 'X' + nome[4:]
            nome = nome.strip()
            # Tratar ultima posicao
            if nome[-1:] == 'C':
                fim = 'K'
                nome = nome[:-1]
            else:
                if nome[-1:] == 'N':
                    fim = 'M'
                    nome = nome[:-1]
                else:
                    if nome[-1:] == 'Z':
                        fim = 'S'
                        nome = nome[:-1]
                    else:
                        if nome[-1:] == 'R':
                            fim = '2'
                            nome = nome[:-1]
                        else:
                            if nome[-1:] == 'X':
                                fim = 'X'
                                nome = nome[:-1]
                                # Tratar penultima posicao
            if not fim:
                if nome[-3:-1] == 'EX':
                    nome = nome[:-3] + 'ks' + nome[-1:]

            nome = re.sub('SC[AOU]', 'sk', nome)
            nome = re.sub('C[AOU]', 'k', nome)
            nome = re.sub('SC[EI]', 's', nome)
            nome = re.sub('C[EI]', 's', nome)
            nome = nome.replace('CHR', 'k')
            nome = nome.replace('CH', 'x')
            nome = re.sub('C[^AEIOU]', 'k', nome)  # C com consoante
            nome = re.sub('G[AOU]', 'g', nome)
            nome = re.sub('G[EI]', 'j', nome)
            nome = re.sub('GH[^AEIOU]', 'g', nome)  # GH com consoante
            nome = nome.replace('LH', '1')
            nome = re.sub('L[AEIOU]', 'l', nome)
            nome = nome.replace('NH', '3')
            nome = nome.replace('PH', 'f')
            nome = nome.replace('Q', 'k')
            nome = nome.replace('RR', '2')
            nome = re.sub('[AEIOU]R[AEIOU]', 'r', nome)  # vogal com R com vogal
            nome = re.sub('[^AEIOU]R[AEIOU]', 'r', nome)  # consoante com R com vogal
            nome = nome.replace('SS', 's')
            nome = nome.replace('SH', 's')
            nome = nome.replace('TH', 't')
            nome = re.sub('W[AEIOU]', 'v', nome)
            nome = re.sub('W[^AEIOU]', '', nome)
            nome = re.sub('EX[EI]', 'x', nome)
            nome = re.sub('EX[AOU]', 'ks', nome)
            nome = re.sub('[AEIOU][CKGLRX]X', 's', nome)
            nome = re.sub('[DFMNPQSTVZ][AIOU]X', 'ks', nome)
            nome = nome.replace('SI', 'zi')
            nome = nome.replace('ZI', 'zi')
            nome = nome.replace('SA', 'za')
            nome = nome.replace('ZA', 'za')
            nome = nome.replace('Y', 'i')
            nome = nome.replace('H', '')
            # translate alpha chars in nome to soundex digits
            nome = nome.strip(' ')
            nome = inicio + nome + fim
            tam = len(nome)
            if tam > _tam:
                nome = nome[:_tam]
            else:
                if tam < _tam:
                    nome = nome + ('0' * (_tam - tam))
            return nome.upper()

        try:
            string = normalize('NFKD', string.decode('utf-8')).encode('ASCII', 'ignore')
        except:
            string = normalize('NFKD', string).encode('ASCII', 'ignore')
        string = string.upper()
        string = string.replace('.', ' ')
        string = string.replace(' DE ', ' ').replace(' DA ', ' ').replace(' DAS ', ' ').replace(
            ' DO ', ' ').replace(
            ' DOS ', ' ').replace(' E ', '').replace(' & ', ' ').replace(' LTDA', ' ')
        snd = ''
        inc = 0
        lst = string.split(' ')
        if lst:
            lst.sort()
        for x in lst:
            if len(x.strip()) < 2:  #nao gerar fonema para apenas uma letra
                continue
            snd += _calc_fonema(x, tamanho_fonema)
            inc += 1
            if not inc == len(lst):
                snd += ';'
        return snd

    def admutl_cpf_cnpj_testar(self, pessoa_tipo, cpf_cnpj, formatar=False):
        """
        Valida o cpf/cnpj
        @pessoa_tipo - Tipo de pessoa F=Fisica, J=Juridica
        @cpf_cnpj - Cpf ou Cnpj a ser testado
        @Return - Retorna False se cpf/cnpj estiverem errados
                  Se corretos retorna o proprio cpf/cnpj
                  Caso a primeira letra do cpf/cnpj seja "f", qualquer que seja o cpf ou cnpj ele sempre vai
                  retornar o mesmo com o digito certo.
        """
        if not cpf_cnpj:
            return False
        if not pessoa_tipo in ('F', 'J'):
            return False
        _cpf_cnpj_forcar = False
        if len(cpf_cnpj) > 1:
            if cpf_cnpj[0].lower() == 'f':
                _cpf_cnpj_forcar = True
        cpf_cnpj = re.sub('[^0-9]', '', cpf_cnpj)
        if pessoa_tipo == 'F':
            nr = cpf.Cpf()
            while len(cpf_cnpj) < 11:
                cpf_cnpj = '0' + cpf_cnpj
            if len(cpf_cnpj) > 11:
                cpf_cnpj = cpf_cnpj[:11]
            if not int(cpf_cnpj):
                return False
        elif pessoa_tipo == 'J':
            nr = cnpj.Cnpj()
            while len(cpf_cnpj) < 14:
                cpf_cnpj = '0' + cpf_cnpj
            if len(cpf_cnpj) > 14:
                cpf_cnpj = cpf_cnpj[:14]
        res = nr.validar(cpf_cnpj, _cpf_cnpj_forcar)
        if res:
            if _cpf_cnpj_forcar:
                if formatar:
                    res = nr.format(res)
            else:
                if formatar:
                    res = nr.format(cpf_cnpj)
                else:
                    res = cpf_cnpj
        return res

    def admutl_get_parametros(self, cr, uid, tabela, campos, company_id=False, context=None):
        """
        Ler campos dos arquivos de parametros
        @param cr:
        @param uid:
        @param tabela:  Tabela de parametro que deseja ser lida ou:
                           - 'adm' para adm_parametro,
                           - 'estoque' para estoque_parametro,
                           - 'financeiro' para financeiro_parametros
                           - 'fiscal' para fiscal_parametros
                           - 'rh' para rh_parametro
        @param campos: Campos da tabela que deseja que sejam lidos
        @param company_id: o id da empresa deve ser passado para evitar que o cron busque o
                           parametro da empresa correta, evitando buscar o id da empresa do admin
        @param company_id: Identifica a empresa se não informado usar o default
        @return: Se estiver tudo ok, retorna um dicionario com os campos, se houver algum problema
                 retorna False
        """
        context = context and context.copy() or {}
        if not isinstance(campos, list):
            campos = [campos]

        if tabela == 'adm':
            tabela = 'adm_parametro'
        elif tabela == 'estoque':
            tabela = 'estoque_parametro'
        elif tabela == 'financeiro':
            tabela = 'financeiro_parametros'
        elif tabela == 'fiscal':
            tabela = 'fiscal_parametro'
        elif tabela == 'rh':
            tabela = 'rh_parametro'
        tabela = tabela.replace('_', '.')
        args = []
        if company_id:
            args.append(('company_id', '=', company_id))
        try:
            args = company_id and [('company_id', '=', company_id)] or []
            parametro_ids = self.pool.get(tabela).search(cr, uid, args, context=context)
        except:
            return {}
        if not parametro_ids:
            return {}
        parametro_dados = self.pool.get(tabela).read(cr, uid, parametro_ids, campos, context=context,
                                                     load=READ_LOAD)
        if parametro_dados and isinstance(parametro_dados, list):
            parametro_dados = parametro_dados[0]
        return parametro_dados

    def admutl_check_modulo_instalado(self, cr, uid, modulo, context):
        args = [('name', '=', modulo), ('state', 'in', ('installed', 'to upgrade'))]
        modulo_ids = self.pool.get('ir.module.module').search(cr, uid, args, context=context)
        if modulo_ids:
            return True
        else:
            return False

    def admutl_converts_fetchall_to_dictfetchall(self, _cr):

        """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
         para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
         conversao."""

        # testa resultados armazenados
        results = _cr.fetchall()
        if results < 1:
            return results

        # substitui resultados das listas por dicionarios
        rownum = 0
        for row in results:
            dictrow = {}
            dictnum = 0
            for col in _cr.description:
                dictrow[col[0]] = row[dictnum]
                dictnum += 1

            results[rownum] = dictrow
            rownum += 1

        return results

    def admutl_remover_acentuacao(self, cr, uid, text, cod='utf-8'):
        """
        Funcao Wrapper para remover acentuacao
        :param text:texto onde sera feita a remocao da acentuacao
        :param cod:Tipo de codificacao
        return:string
        """

        from unicodedata import normalize, category

        try:
            unicode_string = str(text)
        except UnicodeEncodeError:
            return normalize('NFKD', text).encode('ASCII', 'ignore')

        return unicode(filter(lambda c: category(c) != 'Mn',
                              normalize('NFKD', unicode_string.decode(cod))))

    def admutl_float_to_time(self, valor):

        """
        O metodo recebe um valor inteiro ou float.
        Esse valor representa uma quantidade de tempo em horas, como por exemplo: 2.57(h).
        Retorna uma string formatada ('0:00') que representa a quantidade de tempo
        (horas e  minuto), de acordo com o valor informado.
        Ex: valor = 1.5. Retorno: '1:30'. Uma hora e trinta minutos
            valor = 2.57. Retorno: '2:34'. Duas horas, trinta e quatro minutos
        @valor: float ou inteiro
        @return: string no formato de tempo '0:00'
        """
        fator = valor < 0 and -1 or 1
        valor = abs(valor)
        res = "%d:%02d" % (fator * int(math.floor(valor)), int(round((valor % 1) * 60)))
        if fator == -1 and not '-' in res:
            res = '-' + res
        return res

    def admutl_get_id_nome_externo(self, cr, uid, modulo, nome_externo):
        """
        Esta função retorna o id de um dado inserido via xml, usando o nome externo(id do dado)
        como referencia
        @param cr:
        @param uid:
        @param modulo: Nome do modulo onde o dado foi inserido
        @param nome_externo: Campo id do xml do dado desejado
        @return: integer (id do dado) ou False
        """
        mod_obj = self.pool.get('ir.model.data')
        result = mod_obj.get_object_reference(cr, uid, modulo,
                                              nome_externo)
        return result and result[1] or False

    def admutl_valida_superuser(self, cr, uid, context=None):

        """
                Verifica permissoes

                :param cr:
                :param uid:
                :param context:
                :return: dict
             """
        context = context and context.copy() or {}
        res = {'uid':uid}

        if uid != SUPERUSER_ID:
            obj_ir_model_access = self.pool.get('ir.model.access')
            permite_acesso = obj_ir_model_access. \
                check_groups(cr, uid, 'base.group_erp_manager')
            if permite_acesso:
                res.update({'uid': SUPERUSER_ID})
        return res

    @staticmethod
    def admutl_remove_caracter(caracteres, letras=True, numeros=True, espaco=True, outros=None,
                               substituto=''):
        """
        @Parametros: caracteres - Eh a palavra ou frase que deseja remover caracters
                                  especiais - Tipo: string
                     letras     - Se True filtra letras da sequencia informada - Tipo boolean
                     numeros    - Se True filtra os digitos da sequencia de caracteres
                                  informados - Tipo: boolean
                     espaco     - Se True filtra os espacos em branco - Tipo: boolean
                     outros     - uma lista de outros caracteres que deseja filtrar - Tipo: lista
                     substituto - caracter que substituira o caracter especial removido
        @Funcao: Remover caracters especiais de uma sequencia de caracteres
        @Retorno: Retorna uma string tratada de acordo com as informacoes passadas no
                  parametro - Tipo: string
        """
        if not caracteres:
            return ''
        if not isinstance(outros, list):
            outros = []
        char_validos = ''
        if letras:
            char_validos += string.letters
        if numeros:
            char_validos += string.digits
        if espaco:
            char_validos += ' '
        for i in outros:
            char_validos += i
        new_caracters = ''.join([char if char in char_validos else substituto for char in
                                 caracteres])
        return new_caracters

    @staticmethod
    def admutl_get_valor_formatado(valor):
        """
        O metodo converte um valor float para uma string formatada
        :param valor: float
        :return: string, representa um inteiro ou um float com duas casas decimais
        """
        if not valor:
            return valor
        novo_valor = int(valor)
        if valor != novo_valor:
            valor = getNumeroEditado(valor, 2)
        else:
            valor = str(novo_valor)
        return valor

    @staticmethod
    def admutl_quebra_string(texto, tamanho_primeira_linha, tamanho_demais_linhas=None):
        """
        Metodo para quebrar um texto em linhas tentando nao dividir palavras
        :param texto: Texto a ser quebrado
        :type texto: string
        :param tamanho_primeira_linha: Tamanho máximo da primeira linhas
        :type tamanho_primeira_linha: int
        :param tamanho_demais_linhas: Tamanho máximo das demais linhas
        :type tamanho_demais_linhas: int
        """
        lista = []
        texto = texto.strip()
        if not tamanho_demais_linhas:
            tamanho_demais_linhas = tamanho_primeira_linha
        tamanho_linha = tamanho_primeira_linha
        if texto.__len__() <= tamanho_linha:
            lista.append(texto)
            return lista
        resto = texto
        _quebra = (' ', '.', ',', '-')
        _primeira = True
        while resto.__len__() > tamanho_linha:
            if _primeira:
                _primeira = False
            else:
                tamanho_linha = tamanho_demais_linhas
            linha = resto[:tamanho_linha].strip()
            resto = resto[tamanho_linha:]
            if linha.__len__() < tamanho_linha or resto[:1] in _quebra:
                lista.append(linha)
                continue
            i = tamanho_linha
            while True:
                if linha[i-1:i] in _quebra:
                    break
                i -= 1
                if i < 0:
                    break
            if i < 0:
                lista.append(linha)
                continue
            lista.append(linha[:i])
            resto = linha[i:] + resto
        if resto:
            lista.append(resto)
        return lista
    @staticmethod
    def admutl_converte_hora_float_to_str(horario):
        """
        Metodo para converter horario no formato float para String no formato HH:MM
        :param horario: Horario a ser convertido Ex.: 1.5
        :type horario: float
        :return string: horario convertido para string ex.: '01:30'
        """
        _horas, _min = str(horario).split('.')
        _min = int(((horario * 100) - (int(horario) * 100)) * 60 /100)
        res = "%02d:%02d" % (int(_horas), int(_min))
        return res

    @staticmethod
    def admutl_converte_hora_str_to_float(horario):
        """
        Metodo para converter horario no formato str("HH:MM") para float
        :param horario: Horario a ser convertido Ex.: '01:30'
        :type horario: string
        :return float: horario convertido para float ex.: '1.5'
        """
        horas, minutos = map(int, horario.split(':'))
        res = '%s.%s' % (horas, (minutos * 100 / 60))
        return float(res)

    @staticmethod
    def tempo_exec(method):
        """
        Metodo calcula o tempo de execucao de um metodo,
        basta no metodo que deseja calcular colocar este
        como decorator antes do metodo.
        E resultado eh mostrado no log do terminal
        Obs: Utilizar apenas para fim de testes no desenvolvimento
        """
        @wraps(method)
        def wrapper(*args, **kwargs):
            _logger_tempo_exec.warning('EXECUTANDO METODO: ' + method.__module__ + ' ' +
                                       method.__name__ + '...')
            inicio = time.time()
            r = method(*args, **kwargs)
            fim = time.time()
            tempo = '%f' % (fim - inicio)
            msg = 'Modulo: {modulo} \tMetodo: {metodo} \tTempo: {tempo}'
            msg = msg.format(modulo=method.__module__, metodo=method.__name__, tempo=tempo)
            dado = {'modulo': method.__module__, 'metodo': method.__name__, 'tempo': tempo}
            _tempo_exec.append(dado)
            _logger_tempo_exec.warning(msg)
            return r
        return wrapper

    @staticmethod
    def quantidade_exec(method):
        """
        Metodo calcula o tempo de execucao de um metodo,
        basta no metodo que deseja calcular colocar este
        como decorator antes do metodo.
        E resultado eh mostrado no log do terminal
        Obs: Utilizar apenas para fim de testes no desenvolvimento
        """
        @wraps(method)
        def wrapper(*args, **kwargs):
            chave = method.__module__ + method.__name__
            qtd = _qtd_exec.get(chave) or 0
            _qtd_exec.update({chave: qtd + 1})
            r = method(*args, **kwargs)
            msg = 'Metodo: {metodo} \tQuantidade: {quantidade}'
            msg = msg.format(metodo=method.__module__ + " - " + method.__name__,
                             quantidade=_qtd_exec.get(chave))
            _logger_quantidade.warning(msg)
            return r
        return wrapper

    # TODO: Mover o metodo para a classe ´adm.forma.pagto´
    def admutl_forma_pgto_tipo_cheque(self, cr, uid, forma_pagamento_id, context={}):
        """
        Funcao verifica:
        * Se a sigla do pagamento e cheque.
        True - Tipo Cheque
        False - Nao e cheque
        """
        cheque = False
        if forma_pagamento_id:
            forma_pgto_dados = self.pool.get('adm.forma.pgto').read(
                cr, uid, forma_pagamento_id, ['sigla', 'condicao_forma_ids'], context)
            if forma_pgto_dados and isinstance(forma_pgto_dados, list):
                forma_pgto_dados = forma_pgto_dados[0]
            if isinstance(forma_pgto_dados, dict):
                cheque = forma_pgto_dados.get('sigla') == 'CH'
        return cheque

    def admutl_get_objeto_ids_sem_fonema(self, cr, uid, nome_tabela, descricao='', tabela_ids=[],
                                         nome_coluna='name', company_id=False, limite=80,
                                         ordem=False, offset=0, context=None):
        """
        O metodo realiza a pesquisa da descricao informada na coluna de uma tabela, exatamente na
        ordem em que foi passada, ou seja, sera encontrada primeiro todas as palavras que se
        iniciam com aquela descricao. Palavras que nao iniciam com a descricao tambem serao
        encontradas, porem, estarao ordenas posteriormente.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param descricao: string a ser pesquisada
        :param tabela_ids: conjunto de id tabela (nome_tabela).
        :param nome_tabela: string com o nome da tabela no banco de dados
        :param nome_coluna: string com o nome da coluna da tabela em que sera realizada a pesquisa
        :param company_id: id da tabela res_company, identificador da empresa
        :param limite: inteiro com quantidade maxima de registro de retorno
        :param ordem: string com a ordem em que os registros devem ser recuperados
        :param offset: quantidade de registros fora do conjunto
        :type offset: int
        :param context: dicionario com valores contextuais
        :return: lista de id da tabela informada em nome_tabela
        """
        context = context and context.copy() or {}
        from collections import OrderedDict

        if not any([descricao, tabela_ids]) or not nome_tabela:
            return []
        dado_local = ''
        if context.get('local_id') and nome_tabela == 'estoque_produto':
            local_id = context.get('local_id')
            dado_local = """
            AND CASE
                    WHEN tipo = 'P' THEN
                        id IN (SELECT produto_id
                               FROM estoque_local_locacao
                               WHERE local_id = %(local_id)s
                                     AND active = True
                                     AND company_id = %(company_id)s)
                    ELSE TRUE
                END
            """ % locals()

        cond_kit = ''
        if context.get('nao_pesquisa_kit') and self.admutl_check_modulo_instalado(cr, uid, 'kit',
                                                                                  context):
            cond_kit = 'AND kit_de_produtos = False'

        pesquisa = ''
        if tabela_ids:
            pesquisa += """AND id in (%s)""" % ', '.join(map(lambda ti: str(ti), tabela_ids))
        if descricao:
            pesquisa += """AND     %(nome_coluna)s ILIKE $$%(descricao)s%%$$""" % locals()

        sql = """
        -- noinspection SqlDialectInspection
        SELECT  id
        FROM    %(nome_tabela)s
        WHERE   active IS TRUE
        %(dado_local)s
        %(pesquisa)s
        %(cond_kit)s"""

        if company_id:
            sql += """
        AND     company_id = %(company_id)s"""

        if ordem:
            sql += """
        ORDER   BY %(ordem)s """
        else:
            # Obedece a ordem original da classe (tabela)
            sql += """ ORDER   BY %s """ % self.pool.get(nome_tabela.replace('_', '.'))._order or \
                'id'

        if limite:
            sql += """
            LIMIT %(limite)s"""

        if offset:
            sql += """
            OFFSET %(offset)s
            """

        sql = sql % locals()
        cr.execute(sql)
        ids = map(lambda i: i.get('id'), cr.dictfetchall())

        sql = sql.replace("""AND     %(nome_coluna)s ILIKE $$%(descricao)s%%$$""" % locals(),
                          """AND     %(nome_coluna)s ILIKE $$%(descricao)s %%$$""" % locals())
        cr.execute(sql)
        ids.extend(map(lambda i: i.get('id'), cr.dictfetchall()))

        pesquisa = ''
        if tabela_ids:
            pesquisa += """AND id in (%s)""" % ', '.join(map(lambda ti: str(ti), tabela_ids))
        if descricao:
            pesquisa += """AND    %(nome_coluna)s ILIKE $$%%%(descricao)s%%$$""" % locals()

        sql = """
        -- noinspection SqlDialectInspection
        SELECT id
        FROM   %(nome_tabela)s
        WHERE  active IS TRUE
        %(dado_local)s
        %(pesquisa)s
        %(cond_kit)s"""

        if company_id:
            sql += """
        AND company_id = %(company_id)s"""

        if ordem:
            sql += """ ORDER   BY %(ordem)s """
        else:
            # Obedece a ordem original da classe (tabela)
            sql += """ ORDER   BY %s """ % self.pool.get(nome_tabela.replace('_', '.'))._order or \
                'id'

        if limite:
            sql += """
            LIMIT %(limite)s"""

        if offset:
            sql += """
            OFFSET %(offset)s
            """

        cr.execute(sql % locals())
        ids.extend(map(lambda i: i.get('id'), cr.dictfetchall()))

        # Retorna os IDs sem repeticao e sem alterar a ordem
        return list(OrderedDict.fromkeys(ids))[:limite] if limite else \
            list(OrderedDict.fromkeys(ids))

    def admutl_send_email(self, cr, uid, email, assunto, corpo, destinatarios='', titulo_dados='',
                          anexo='', nome_anexo='Documento', mail_server_id=False, context=None):
        """
        O metodo realiza a criação dos emails de acordo com os parametros recebidos e padroes do
        modulo 'mail'. Esse modulo que controla o envio deles.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param email: char com a descricao do e-mail do destinatario.
        :type: char
        :param assunto: char com a descricao do assunto do e-mail.
        :type: char
        :param corpo: text com a descricao do e-mail.
        :type: text
        :param destinatarios: char com a descricao dos e-mails de demais destinatarios.
        :type: char
        :param titulo_dados: text com as informacoes dos titulos que se encontram em atraso.
        :type: text
        :param anexo: arquivo binario que sera anexo ao e-mail.
        :type: binary
        :param nome_anexo: char com a descricao que o anexo recebera.
        :type nome_anexo: char
        :param mail_server_id: id da tabela ir_mail_server, identificador do servidor de e-mails.
        :type mail_server_id: int
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: id da tabela mail_mail.
        :rtype: integer
        """
        context = context and context.copy() or {}
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context)
        emailfrom = brw_res_users.pessoa_id.email
        assinatura = brw_res_users.signature
        if not emailfrom:
            raise osv.except_osv(_('Atencao'),
                                 _('Cadastre um e-mail para realizar essa operacao!'))
        obj_mail_message = self.pool.get('mail.message')
        obj_mail_mail = self.pool.get('mail.mail')
        if not mail_server_id:
            mail_server_ids = self.pool.get('ir.mail_server').search(cr, uid, [], context=context)
            if mail_server_ids:
                mail_server_id = mail_server_ids[0]
        body = corpo or ''
        body += '\n' + titulo_dados
        body = body.replace('\n', '<br />')
        if assinatura:
            signature = assinatura.replace('\n', '<br />')
            body += '<br /><br />%s' % signature
        mail_message_id = obj_mail_message.create(
            cr, uid, {'email_from': emailfrom, 'subject': assunto}, context)
        vals_mail = {
            'mail_message_id': mail_message_id,
            'mail_server_id': mail_server_id,
            'email_from': emailfrom,
            'email_to': email,
            'body_html': body,
        }
        if destinatarios:
            vals_mail.update({'email_cc': destinatarios})
        mail_id = obj_mail_mail.create(cr, uid, vals_mail, context)
        if anexo:
            anexo_id = self.pool.get('ir.attachment').create(
                cr, uid, {'name': nome_anexo, 'datas_fname': '%s.pdf' % nome_anexo, 'datas': anexo,
                          'res_model': obj_mail_mail._name, 'res_id': mail_id}, context)
            if anexo_id:
                obj_mail_mail.write(
                    cr, uid, mail_id, {'attachment_ids': [(6, 0, [anexo_id])]}, context=context)
        return mail_id

    def admutl_read_browse(self, cr, uid, ids, classe, campos_classes, context=None):
        '''
        Esta funcao tem o objetivo de melhorar performance em varios locais onde sao lidas classe
         que relacionam com outras, como se fosse um brwose com retornos de read 
        :param cr: 
        :param uid: 
        :param ids: 
        :param classe: string com o nome da classe
        :param campos_classes: lista com os campos a serem lidos ou 
            dict onde as chaves sao os campos da classe e os valores sao:
                lista com os campos da classe relacionada a serem lidos
                dict para acessar outros campos many2one/one2many
            Lista vazia para os demis tipos de campos(int, char, ...)
        :param context: 
        :return: retorna uma lista de dicts onde as chaves sao os campos da classe passados e os valores sao outro dict 
        tendo a cave como o nome do campo e os valores ods dados da classe relacionada
        ex: ids=[1, ...];
            classe= 'estoque.movimento.itens';  
            campos_classes={'item_cancelado': [],
                            'produto_id': ['grade_id', 'name', 'categoria_id', ...]
                            'tributacoes_ids':{'vr_encargo': [],
                                               'cst_id' : ['tipo'],
                                               'encargo_id': ['fisenc_evento_id', 'name'],                                        
                           }
        retorno [{'item_cancelado': False,
                     'produto_id': [{'id': 3, 
                                    'grade_id': 12, 
                                    'name': 'Detergente Ype',
                                    'categoria_id': 10,
                                    ...}]
                     'tributacoes_ids':{'vr_encargo': 1.15,
                                        'cst_id' : {'tipo': 'T'},
                                        'encargo_id': {'fisenc_evento_id': 15, 
                                                       'name': 'PIS'},
                     ...
                    },
                ...                                    
                }
        obs: eh como se fosse um browse mas acessa o elemento com '.get' ao inves de somente '.'
        :author: Felipe Frenca <felipe.franca[at]siginformatica.com.br>
        '''
        context = context and context.copy() or {}
        if not ids:
            return []

        load = '_classic_write'
        res = {}
        ids = isinstance(ids, list) and ids or [ids]
        campos = isinstance(campos_classes, dict) and campos_classes.keys() or campos_classes
        dados = self.pool.get(classe).read(cr, uid, ids, campos, context=context, load=load)
        ids_campos_ids = {}
        campos_ids = {}
        for dado in dados:
            ids_campos_ids[dado.get('id')] = dado
            if not isinstance(campos_classes, dict):
                continue

            for campo, valor in campos_classes.items():
                if valor and dado.get(campo):
                    if not campos_ids.get(campo):
                        campos_ids[campo] = []

                    if isinstance(dado.get(campo), list):
                        campos_ids[campo].extend(dado.get(campo))
                    else:
                        campos_ids[campo].append(dado.get(campo))

        campos_ids_dados = {}
        for campo, valor in campos_ids.items():
            campos_ids_dados[campo] = {}
            classe_do_campo = self.pool.get(classe)._columns.get(campo)._obj
            campos_do_campo = campos_classes.get(campo)
            if isinstance(campos_do_campo, dict):
                campo_dados = self.admutl_read_browse(cr, uid, valor, classe_do_campo,
                                                                   campos_do_campo, context=context)
            else:
                campo_dados = self.pool.get(classe_do_campo).read(cr, uid, valor, campos_do_campo,
                                                               context=context, load=load)
            map(lambda x: campos_ids_dados[campo].update({x.get('id'): x}), campo_dados)

        for dado in dados:
            res[dado.get('id')] = dado
            for campo, dados_campo in campos_ids_dados.items():
                dados_aux = []
                for id, valor in dados_campo.items():
                    if isinstance(dado.get(campo), list):
                        if id not in dado.get(campo):
                            continue
                    else:
                        if id != dado.get(campo):
                            continue

                    if self.pool.get(classe)._columns.get(campo)._type == 'one2many':
                        dados_aux.append(valor)
                    else:
                        dado[campo] = valor

                dado[campo] = dados_aux or dado[campo]

        return dados # retorna lista
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
