# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class adm_pessoa_documento_tipo(osv.Model):
    _name = 'adm.pessoa.documento.tipo'
    _inherit = 'adm.pessoa.documento.tipo'

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        """
        Metodo para pesquisa do tipo de documento
        Pode-se informar parte de um nome ou a sigla do documento
        """
        if not args:
            args = []
        context = context and context.copy() or {}
        ids = self.search(cr, user, [('sigla', '=', name.upper())] + args, limit=limit,
                          context=context)
        if not ids:
            ids = self.search(cr, user, [('name', operator, name)] + args,
                              limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    def unlink(self, cr, uid, ids, context=None):
        """
        Metodo para exclusao dos tipos de documento
        Nao permite a exclusao de tipos de documentos que possue o atributo bloqueio = True
        """
        context = context and context.copy() or {}
        for documento_tipo in self.browse(cr, uid, ids):
            if documento_tipo.bloqueio == True:
                raise osv.except_osv(_('Mensagem:'), _('Este tipo de documento nao pode ser excluido !'))
        return super(adm_pessoa_documento_tipo, self).unlink(cr, uid, ids, context=context)

    def create(self, cr, uid, vals, context=None):
        """
        Metodo para inclusao de tipos de documento
        Antes de efetivar a gravacao, converte a sigla para maiusculas
        """
        context = context and context.copy() or {}
        if vals.get('sigla'):
            vals['sigla'] = vals['sigla'].upper()
        result = super(adm_pessoa_documento_tipo, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        """
        Metodo para atualizar os tipos de documentos
        Nao permite atualizar um tipo de documento que tenha o bloqueio = True
        """
        context = context and context.copy() or {}
        if vals.get('sigla'):
            vals['sigla'] = vals['sigla'].upper()
        for documento_tipo in self.browse(cr, uid, ids):
            if documento_tipo.bloqueio == True:
                if vals.get('sigla'):
                    if documento_tipo.sigla != vals['sigla']:
                        raise osv.except_osv(_('Mensagem:'),
                                             _('A sigla para este tipo de documento nao pode ser alterada !'))
        result = super(adm_pessoa_documento_tipo, self).write(cr, uid, ids, vals, context)
        return result
