# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
import re
from SigUtils import email
import logging

class adm_pessoa_email(osv.Model):
    _name = 'adm.pessoa.email'
    _inherit = 'adm.pessoa.email'
    _logger = logging.getLogger(_name)

    def _valida_email(self, word):
        # caracteres nao-alfanumericos aceitos na parte local
        local_char = r'\!\$\*\+\-\?\^\_\{\|\}\=#/\'~\%&`'
        # parte local
        local_word = r'[a-z0-9%(char)s][a-z0-9%(char)s.]{0,62}[a-z0-9%(char)s]?' % {'char': local_char}
        domain_label = r'[a-z0-9][a-z0-9\-]{0,61}[a-z0-9]?\.'
        # parte dominio (que deve ser um hostname valido)
        domain_word = r'(%s)+\w{2,4}' % domain_label
        # o conjunto
        email_regex = '^%s@%s$' % (local_word, domain_word)
        # flag CASEINSENSITIVE adicionada
        email_c = re.compile(email_regex, re.I)
        # 'Devolve True se o email e valido. False caso contrario.'
        if email_c.match(word) is not None:
            parts = word.split('@')
            local_part, domain_part = parts[0], parts[1]
            if '..' in word or len(word) > 256:
                return False
            return True
        return False

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('email_cobranca', False):
            obj_pessoa = self.pool.get('adm.pessoa')
            pessoa = obj_pessoa.read(cr, uid, context.get('active_id'), ['relacionamento_ids'], context=context)
            dependentes_ids = []
            if pessoa:
                pessoa = pessoa[0]
                obj_dependente = self.pool.get('adm.pessoa.relacionamento')
                dependentes = obj_dependente.read(cr, uid, pessoa.get('relacionamento_ids', [], context=context), ['dependente_id'])
                if dependentes:
                    for i in dependentes:
                        dependentes_ids.append(i['dependente_id'][0])
                if pessoa.get('id'):
                    dependentes_ids.append(pessoa['id'])
                if dependentes_ids:
                    args += [('pessoa_id', 'in', dependentes_ids)]
        return super(adm_pessoa_email, self).search(cr, uid, args, offset, limit, order, context=context, count=count)

    def onchange_veridica_email(self, cr, uid, ids, name, context=None):
        context = context and context.copy() or {}
        res = {}
        if name:
            if not email.validar_email(name):
                res['warning'] = {'title': _('Atencao'), 'message': _('Email incorreto!')}
                res['value'] = {'name': ''}
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            # self._logger.info('===> Usuario remoto criando emails:' + str(vals))
            return True # Usuario remoto não atualiza mais
        result = super(adm_pessoa_email, self).create(cr, uid, vals, context)
        _pessoa_id = vals.get('pessoa_id', False)
        if _pessoa_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoa_id, {}, context=context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        result = super(adm_pessoa_email, self).write(cr, uid, ids, vals, context=context)
        _pessoa_id = vals.get('pessoa_id', False)
        if _pessoa_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoa_id, {}, context=context)
        return result

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        _pessoas_id = []
        for x in self.read(cr, uid, ids, ['pessoa_id'], context=context):
            _pessoas_id.append(x['pessoa_id'][0])
        result = super(adm_pessoa_email, self).unlink(cr, uid, ids, context=context)
        if _pessoas_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoas_id, {}, context=context)
        return result