# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from SigUtils.cep import Cep, Correios
from openerp.tools import config
import logging

def capitalize_all(texto):
    lista = texto.lower().split()
    lista_capitalize = []
    for a in lista:
        lista_capitalize.append(a.capitalize())
    b = ' '
    return b.join(lista_capitalize)


class adm_pessoa_endereco(osv.Model):
    _name = 'adm.pessoa.endereco'
    _inherit = 'adm.pessoa.endereco'
    _logger = logging.getLogger(_name)

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.financeiro = None
            self.fiscal = None
            self.estoque = None
            self.adm = None
            self.rh = None
            self.__dict__.update(entries)

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or dict()
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in ['financeiro', 'fiscal', 'estoque', 'adm',
                                                         'rh']])
        return self.ParamToObject(**parametros)

    def valida_endereco_nf(self, cr, uid, id, context=None):
        context = context and context.copy() or {}
        falta = list()
        endereco = self.pool.get('adm.pessoa.endereco').read(
            cr, uid, id, ['pais_id', 'logradouro_id', 'cidade_id', 'cep', 'bairro_id', 'numero',
                          'uf_id'], context=context)

        if endereco['pais_id'][1] == 'BRASIL':
            if not endereco.get('cep', False):
                falta.append(_('CEP'))
            if not endereco.get('bairro_id', False):
                falta.append(_('bairro'))
            if not endereco.get('numero', False):
                falta.append(_('numero'))
            if not endereco.get('uf_id', False):
                falta.append(_('UF'))
            if not endereco.get('cidade_id', False):
                falta.append(_('cidade'))
            else:
                cidade_id = endereco.get('cidade_id', False)[0]
                codigo_ibge = self.pool.get('adm.cidade').read(
                    cr, uid, cidade_id, ['codigo_ibge'], context=context)['codigo_ibge']

                if not codigo_ibge:
                    falta.append(_('codigo IBGE'))

            if not endereco.get('logradouro_id', False):
                falta.append(_('logradouro'))
        else:
            if not endereco.get('cidade_id', False):
                falta.append(_('cidade'))
            if not endereco.get('logradouro_id', False):
                falta.append(_('logradouro'))

        if falta:
            falta = _('Endereco incompleto para nota fiscal. \n'
                      'Informacoes faltantes: %s') % ', '.join(falta)

        return falta

    def onchange_cep(self, cr, uid, ids, cep, bairro_id, logradouro_id, context=None):
        """
        Metodo para validar o cep na perda de foco da tela
        """
        context = context and context.copy() or {}
        if not cep:
            return {'value': {}.fromkeys(['cidade_id', 'uf_id', 'bairro_id', 'logradouro_id',
                                          'pais_id'], '')}
        return self.admend_get_cep(cr, uid, cep, context)

    def admend_get_cep(self, cr, user, cep, context=None):
        """
        Recupera os dados do CEP.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param cep: o codigo de enderecamento postal CEP
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type cep: str
        :type context: dict
        :return: dados do CEP
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        obj_cep = Cep(cep)
        if not obj_cep.validar():
            return {'warning': {'title': _('Aviso'), 'message': _('CEP Invalido.')}}
        parametro = self._parametros(cr, user)
        _busca_cep = any(parametro.adm.__dict__.values()) and parametro.adm.busca_cep or 'A'
        if _busca_cep == 'B':
            return self._admend_get_cep_base(cr, user, obj_cep.getCepSemFormatar(), context)
        elif _busca_cep == 'C':
            return self._admend_get_cep_correios(cr, user, obj_cep.getCepSemFormatar(), context)

        return self._admend_get_cep_base(
            cr, user, obj_cep.getCepSemFormatar(), context) or self._admend_get_cep_correios(
            cr, user, obj_cep.getCepSemFormatar(), context)

    def _admend_get_cep_base(self, cr, user, cep, context=None):
        """
        Retorna os dados do CEP baseado nos valores armazenados na base de dados do ERP.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param cep: o codigo de enderecamento postal CEP
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type cep: str
        :type context: dict
        :return: os dados do CEP.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        adm_cep_ids = self.pool.get('adm.cep').search(
            cr, user, [('cep', '=', cep)], context=context)
        adm_cep_id = adm_cep_ids and adm_cep_ids[0]
        fields = ['cep', 'pessoa_pais_id', 'pessoa_uf_id', 'pessoa_cidade_id',
                  'pessoa_bairro_id', 'pessoa_logradouro_id']
        adm_cep_dados = adm_cep_id and self.pool.get('adm.cep').read(
            cr, user, adm_cep_id, fields, context=context)
        adm_cep_dados = adm_cep_dados and adm_cep_dados[0] or dict()

        return adm_cep_dados and {'value': dict(
            cep=adm_cep_dados.get('cep') or '',
            pais_id=adm_cep_dados.get('pessoa_pais_id') or False,
            uf_id=adm_cep_dados.get('pessoa_uf_id') or False,
            cidade_id=adm_cep_dados.get('pessoa_cidade_id') or False,
            bairro_id=adm_cep_dados.get('pessoa_bairro_id') or False,
            logradouro_id=adm_cep_dados.get('pessoa_logradouro_id') or False)
        } or dict()

    def _admend_get_cep_correios(self, cr, user, cep, context=None):
        """
        Retorna os dados do CEP baseado nos valores armazenados na base de dados dos Correios.
        Se o CEP for encontrado, os dados serao registrados na base da dados do ERP.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param cep: o codigo de enderecamento postal CEP
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type cep: str
        :type context: dict
        :return: os dados do CEP.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        try:
            dados_cep = Correios(config.options.get('http_proxy')).consulta(cep)
        except:
            return {
                'value': dict(),
                'warning': {
                    'title': _('Atencao!'),
                    'message': _('Nao foi possivel conectar-se ao site dos Correios!')
                }
            }
        if not dados_cep:
            return {
                'value': dict(),
                'warning': {'title': _('Aviso'), 'message': _('CEP nao encontrado.')}
            }
        else:
            # TODO: se houver mais de um resultado o correto sera tratar cada um individualmente
            dados_cep = dados_cep[0] if isinstance(dados_cep, list) else dados_cep

            _localidade = (dados_cep.get('localidade') or '').replace('\n', '')
            _bairro = (dados_cep.get('bairro') or '').replace('\n', '')
            _estado = (dados_cep.get('uf') or '').replace('\n', '')
            _logradouro = (dados_cep.get('logradouroDNEC') or '').replace('\n', '')
            _titulo = _logradouro.split() and _logradouro.split()[0] or ''
            _logradouro = ' '.join(_logradouro.split()[1:])  # Agora sem o titulo (Rua, Av., etc)
            logradouro_id = None
            if _logradouro:
                logradouro_ids = self.pool.get('adm.logradouro').search(
                    cr, user, [('descricao', '=', _logradouro)], context=context)
                logradouro_id = logradouro_ids and logradouro_ids[0] or \
                                self.pool.get('adm.logradouro').create(
                                    cr, user, {'descricao': _logradouro, 'titulo': _titulo}, context)
            bairro_id = None
            if _bairro:
                bairro_ids = self.pool.get('adm.bairro').search(
                    cr, user, [('name', '=', _bairro)], context=context)
                bairro_id = bairro_ids and bairro_ids[0] or self.pool.get('adm.bairro').create(
                    cr, user, {'name': _bairro}, context)
            pais_id = None
            pais_ids = self.pool.get('adm.pais').search(cr, user, [('name', '=', 'Brasil')],
                                                        context=context)
            if pais_ids:
                pais_id = pais_ids and pais_ids[0] or False

            uf_id = None
            if _estado:
                uf_ids = self.pool.get('adm.uf').search(cr, user, [('sigla', '=', _estado)],
                                                        context=context)
                uf_id = uf_ids and uf_ids[0] or self.pool.get('adm.uf').create(cr, user,
                                   {'pais_id': pais_id, 'sigla': _estado, 'name': _estado}, context)
            localidade_id = None
            if _localidade:
                localidade_ids = self.pool.get('adm.cidade').search(
                    cr, user, [('name', '=', _localidade), ('uf_id', '=', uf_id)],
                    context={'not_simple_unaccent': True})
                localidade_id = localidade_ids and localidade_ids[0] or \
                                self.pool.get('adm.cidade').create(
                                    cr, user, {'name': _localidade, 'uf_id': uf_id}, context)
            cep_ids = self.pool.get('adm.cep').search(cr, user, [('cep', '=', cep)],
                                                      context=context)
            if not cep_ids and (pais_id and uf_id and localidade_id):
                self.pool.get('adm.cep').create(
                    cr, user, dict(cep=cep, pessoa_pais_id=pais_id, pessoa_uf_id=uf_id,
                                   pessoa_cidade_id=localidade_id, pessoa_bairro_id=bairro_id,
                                   pessoa_logradouro_id=logradouro_id), context)

        return {'value': dict(cep='{}{}{}{}{}-{}{}{}'.format(*cep), pais_id=pais_id,
                              uf_id=uf_id, cidade_id=localidade_id, bairro_id=bairro_id,
                              logradouro_id=logradouro_id)}

    def str2address(self, cr, uid, IdPessoa, StrPais, StrUF, StrCidade, StrBairro, StrLogradouro, StrCEP=None,
                    StrNum=None, StrComplemento=None, StrTipo='Normal', IntCodigoIBGE=0, context=None):
        """
        Funcao utilizada para cadastramento de enderenco. Deve ser passado os parametros como String e sera
        feito uma busca para retornar os ids se existir, senao, efetua um cadastro com os parametros passados.
        return = endereco_id
        OBS: Criado para uso na Importacao da NFe
        By Christian
        """
        # UNIDADES FEDERATIVAS
        msg = IdPessoa, StrPais, StrUF, StrCidade, StrBairro, StrLogradouro, StrTipo, StrNum, StrCEP, StrComplemento
        _UFs = {'AC': 'ACRE', 'AL': 'ALAGOAS', 'AP': 'AMAPA', 'AM': 'AMAZONAS', 'BA': 'BAHIA', 'CE': 'CEARA',
                'DF': 'DISTRITO FEDERAL', 'ES': 'ESPIRITO SANTO', 'RR': 'RORAIMA', 'GO': 'GOIAS', 'MA': 'MARANHAO',
                'MT': 'MATO GROSSO', 'MS': 'MATO GROSSO DO SUL', 'MG': 'MINAS GERAIS', 'PA': 'PARA', 'PB': 'PARAIBA',
                'PR': 'PARANA', 'PE': 'PERNAMBUCO', 'PI': 'PIAUI', 'RJ': 'RIO DE JANEIRO', 'RN': 'RIO GRANDE DO NORTE',
                'RS': 'RIO GRANDE DO SUL', 'RO': 'RONDONIA', 'TO': 'TOCANTINS', 'SC': 'SANTA CATARINA',
                'SP': 'SAO PAULO', 'EX': 'EXTERIOR',
                'SE': 'SERGIPE'}

        if StrUF[:2] not in _UFs:
            raise osv.except_osv(_('Atencao'), _('A Sigla (%s) da Unidade Federativa e invalida.') % StrUF)
        else:
            UFName = _UFs[StrUF[:2]]
            # INSTANCIAR OS OBJETOS
        ObjPessoa = self.pool.get('adm.pessoa')
        ObjPais = self.pool.get('adm.pais')
        ObjUF = self.pool.get('adm.uf')
        ObjCidade = self.pool.get('adm.cidade')
        ObjBairro = self.pool.get('adm.bairro')
        ObjLogradouro = self.pool.get('adm.logradouro')
        ObjTipo = self.pool.get('adm.endereco.tipo')
        ### BUSCA PELO ID DOS OBJETOS, SENAO EXISTIR, CRIAR.
        # PAIS
        cap_strPais = StrPais.capitalize()
        capt_strPais = capitalize_all(StrPais)
        upp_strPais = StrPais.upper()
        low_strPais = StrPais.lower()
        IdPais = ObjPais.search(cr, uid,
                                [('name', 'in', (cap_strPais, capt_strPais, upp_strPais, low_strPais)),
                                  ('codigo_bc', '!=', False)],
                                context=context)
        if not IdPais:
            IdPais = ObjPais.create(cr, uid, {'name': StrPais}, context)
        else:
            IdPais = IdPais[0]
            # UF
        IdUF = ObjUF.search(cr, uid, [('pais_id', '=', IdPais), ('sigla', '=', StrUF)], context=context)
        if not IdUF:
            IdUF = ObjUF.create(cr, uid, {'name': UFName, 'pais_id': IdPais, 'sigla': StrUF}, context)
        else:
            IdUF = IdUF[0]
            # Cidade
        if IntCodigoIBGE:
            IdCidade = ObjCidade.search(cr, uid, [('codigo_ibge', '=', IntCodigoIBGE)], context=context)
        else:
            IdCidade = ObjCidade.search(cr, uid,
                                        [('pais_id', '=', IdPais), ('uf_id', '=', IdUF), ('name', '=', StrCidade)],
                                        context=context)
        if not IdCidade:
            IdCidade = ObjCidade.create(cr, uid, {'pais_id': IdPais, 'uf_id': IdUF, 'name': StrCidade,
                                                  'codigo_ibge': IntCodigoIBGE}, context)
        else:
            IdCidade = IdCidade[0]
            if IntCodigoIBGE:
                if ObjCidade.browse(cr, uid, IdCidade).codigo_ibge == 0:
                    ObjCidade.write(cr, uid, IdCidade, {'codigo_ibge': IntCodigoIBGE}, context=context)
                    # Bairro
        IdBairro = ObjBairro.search(cr, uid, [('name', '=', StrBairro)], context=context)
        if not IdBairro:
            IdBairro = ObjBairro.create(cr, uid, {'name': StrBairro}, context)
        else:
            IdBairro = IdBairro[0]
            # Logradouro
        IdLogradouro = ObjLogradouro.search(cr, uid, [('descricao', '=', StrLogradouro)], context=context)
        if not IdLogradouro:
            IdLogradouro = ObjLogradouro.create(cr, uid, {'titulo': 'Rua', 'descricao': StrLogradouro}, context)
        else:
            IdLogradouro = IdLogradouro[0]
            # Endereco Tipo
        IdTipo = ObjTipo.search(cr, uid, [('name', '=', StrTipo)], context=context)
        if not IdTipo:
            IdTipo = ObjTipo.create(cr, uid, {'name': StrTipo}, context)
        else:
            IdTipo = IdTipo[0]
            # PARAMETROS DO SEARCH
        args_search = [('pessoa_id', '=', IdPessoa), ('pais_id', '=', IdPais), ('uf_id', '=', IdUF),
                       ('cidade_id', '=', IdCidade),
                       ('logradouro_id', '=', IdLogradouro), ('endereco_tipo_id', '=', IdTipo),
                       ('bairro_id', '=', IdBairro)]
        if StrCEP:
            args_search.append(('cep', '=', StrCEP))
        if StrNum:
            args_search.append(('numero', '=', StrNum))
        if StrComplemento:
            args_search.append(('complemento', '=', StrComplemento))
        res = self.search(cr, uid, args_search, context=context)
        if not res:
            vals_create = {'pessoa_id': IdPessoa, 'pais_id': IdPais, 'uf_id': IdUF, 'cidade_id': IdCidade,
                           'cep': StrCEP, 'bairro_id': IdBairro,
                           'logradouro_id': IdLogradouro, 'endereco_tipo_id': IdTipo, 'numero': StrNum,
                           'complemento': StrComplemento}
            res = self.create(cr, uid, vals_create, context)
        else:
            res = res[0]
        return res

    def create(self, cr, uid, vals, context=None):
        """
        Metodo para inclusao de registros
        Valida o CEP antes da inclusao
        """
        context = context and context.copy() or {}
        import re

        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        # _usuarioRemoto = False
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            # _usuarioRemoto = True
            # self._logger.info('===> Usuario remoto criando endereco:' + str(vals))
            return True  # Usuario remoto não atualiza mais
        if vals.get('cep', False):
            vals['cep'] = re.sub('[^0-9]', '', vals['cep'])
        if vals.get('cep') and vals.get('uf_id') and vals.get('cidade_id'):
            cep = vals['cep'].replace('-', '').replace('.', '')
            obj_adm_cep = self.pool.get('adm.cep')
            adm_cep_ids = obj_adm_cep.search(cr, uid, [('cep', '=', cep)], context=context)
            if not adm_cep_ids:
                pais_id = vals['pais_id']
                uf_id = vals['uf_id']
                cidade_id = vals['cidade_id']
                bairro_id = vals.get('bairro_id') or 0
                logradouro_id = vals.get('logradouro_id') or 0
                cep_data = {'cep': cep, 'pessoa_uf_id': uf_id, 'pessoa_cidade_id': cidade_id, 'pessoa_pais_id': pais_id}
                if bairro_id:
                    cep_data['pessoa_bairro_id'] = bairro_id
                if logradouro_id:
                    cep_data['pessoa_logradouro_id'] = logradouro_id
                obj_adm_cep.create(cr, uid, cep_data, context=context)
        _pessoa_id = vals.get('pessoa_id', False)
        result = super(adm_pessoa_endereco, self).create(cr, uid, vals, context)

        if _pessoa_id:
            if 'sem_endereco_nf_cliente' in context:
                cliente_ids = self.pool.get('financeiro.cliente').search(cr, uid,
                                                                         [('pessoa_id', '=',
                                                                           _pessoa_id)], context=context)
                if cliente_ids:
                    self.pool.get('financeiro.cliente').write(cr, uid, cliente_ids[0],
                                                              {'endereco_nf_id': result}, context=context)
            if 'sem_endereco_nf_pessoa' in context:
                pessoa_ids = self.pool.get('adm.pessoa').search(cr, uid, [('id', '=', _pessoa_id)], context=context)
                if pessoa_ids:
                    self.pool.get('adm.pessoa').write(cr, uid, pessoa_ids[0],
                                                      {'endereco_nf_id': result}, context=context)

            self.pool.get('adm.pessoa').write(cr, uid, _pessoa_id, {}, context=context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        """
        Metodo para atualizacao de registros
        Valida o CEP antes da atualizacao
        """
        context = context and context.copy() or {}
        import re

        if vals.get('cep'):
            if vals['cep']:
                vals['cep'] = re.sub('[^0-9]', '', vals['cep'])
        result = super(adm_pessoa_endereco, self).write(cr, uid, ids, vals, context)
        _pessoa_id = vals.get('pessoa_id', False)
        if _pessoa_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoa_id, {}, context=context)
        return result

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Metodo para leitura de registros
        apos a leitura formata o cep no padrao 00.000-000
        """
        context = context and context.copy() or {}
        results = super(adm_pessoa_endereco, self).read(cr, uid, ids, fields, context, load)
        for result in results if isinstance(results, list) else [results]:
            if not isinstance(result, dict):
                continue
            cep = result.get('cep') or ''
            if len(cep) == 8:
                result['cep'] = '%s.%s-%s' % (cep[0:2], cep[2:5], cep[5:8])

        return results

    def open_map(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        endereco_obj = self.pool.get('adm.pessoa.endereco')
        endereco_dados = endereco_obj.browse(cr, uid, ids, context=context)[0]
        logradouro_id = endereco_dados.logradouro_id.id or False
        cidade_id = endereco_dados.cidade_id.id or False
        uf_id = endereco_dados.uf_id.id or False
        pais_id = endereco_dados.pais_id.id or False
        numero = endereco_dados.numero or False
        cep = endereco_dados.cep or False
        url = "http://maps.google.com/maps?oi=map&q="
        if logradouro_id:
            logradouro = self.pool.get('adm.logradouro').browse(cr, uid, logradouro_id)
            if logradouro:
                url += logradouro.name.replace(' ', '+')
        if numero:
            url += '+' + numero.replace(' ', '+')
        if cidade_id:
            cidade = self.pool.get('adm.cidade').browse(cr, uid, cidade_id)
            if cidade:
                url += '+' + cidade.name.replace(' ', '+')
        if uf_id:
            uf = self.pool.get('adm.uf').browse(cr, uid, uf_id)
            if uf:
                url += '+' + uf.name.replace(' ', '+')
        if pais_id:
            pais = self.pool.get('adm.pais').browse(cr, uid, pais_id)
            if pais:
                url += '+' + pais.name.replace(' ', '+')
        if cep:
            url += '+' + cep.replace(' ', '+').replace('.', '')
        return {
            'type': 'ir.actions.act_url',
            'url': url,
            'target': 'new'
        }

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        _pessoas_id = []
        for x in self.read(cr, uid, ids, ['pessoa_id'], context=context):
            _pessoas_id.append(x['pessoa_id'][0])
        result = super(adm_pessoa_endereco, self).unlink(cr, uid, ids, context=context)
        if _pessoas_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoas_id, {}, context=context)
        return result
