# -*- coding: utf-8 -*-

##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp import pooler, tools
from openerp.tools.translate import _
from openerp.tools import config as openerp_config
from openerp import SUPERUSER_ID
import re
from lxml import etree
from SigUtils import inscricao_estadual, cpf, cnpj, formatastring
import logging
import xmlrpclib, socket


class adm_pessoa(osv.Model):
    """
    Heranca da classe ´adm_pessoa´.
    """

    _inherit = 'adm.pessoa'
    _logger = logging.getLogger(_inherit)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    def admpes_validar_ie(self, cr, uid, ids, admpes_jur_ie):
        res = True
        estado = False
        for partner in self.browse(cr, uid, ids):
            estado = partner.uf.upper()
        if admpes_jur_ie and admpes_jur_ie.isdigit() and estado:
            res = inscricao_estadual.validar(estado, admpes_jur_ie)
        return res, estado

    def admpes_prepara_onchange_pessoa(self, cr, uid, brw_pessoa):
        """
        Este método prepara um dicionário para retorno de métodos onchange de pessoas
        @param brw_pessoa: {browse} Este parametro deve ser um objeto do tipo browse
                                    Porem, não pode ser uma lista de browse, tem que ter feito o
        @return: {dict} Dicionario com os valores de pessoa montado, pronto para o retorno de
                        um método onchange. Formato: {'value':{}}
        """
        endereco_ids = []
        for dados_endereco in brw_pessoa.endereco_ids:
            enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                         'active': dados_endereco.active,
                         "cep": dados_endereco.cep,
                         "pais_id": dados_endereco.pais_id.id,
                         "uf_id": dados_endereco.uf_id.id,
                         "cidade_id": dados_endereco.cidade_id.id,
                         "bairro_id": dados_endereco.bairro_id.id,
                         "logradouro_id": dados_endereco.logradouro_id.id,
                         "numero": dados_endereco.numero,
                         "complemento": dados_endereco.complemento,
                         "observacao": dados_endereco.observacao,
            }
            endereco_ids.append((1, dados_endereco.id, enderecos))
        email_ids = []
        for dados_emails in brw_pessoa.email_ids:
            emails = {
                "email_tipo_id": dados_emails.email_tipo_id.id,
                "name": dados_emails.name,
                "observacao": dados_emails.observacao,
            }
            email_ids.append((1, dados_emails.id, emails))
        telefones_ids = []
        for dados_telefones in brw_pessoa.telefones_ids:
            telefones = {
                "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                "ddi": dados_telefones.ddi,
                "ddd": dados_telefones.ddd,
                "admptel_operadora_id": dados_telefones.admptel_operadora_id.id,
                "numero": dados_telefones.numero,
                "observacoes": dados_telefones.observacoes,
            }
            telefones_ids.append((1, dados_telefones.id, telefones))
        categoria_ids = []
        categ_ids = []
        for dados_categoria in brw_pessoa.categoria_ids:
            categ_ids.append(dados_categoria.id)
            categoria_ids.append((6, False, categ_ids))

        modulo_fiscal_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context={})
        endereco_nf_id = brw_pessoa.endereco_nf_id.id if modulo_fiscal_instalado else False

        res = {'value': {
            'name': brw_pessoa.name,
            'fantasia_apelido': brw_pessoa.fantasia_apelido,
            'pessoa_tipo': brw_pessoa.pessoa_tipo,
            'cpf_cnpj': brw_pessoa.cpf_cnpj,
            'codigo': brw_pessoa.codigo,
            'active': brw_pessoa.active,
            'controle': brw_pessoa.controle,
            'admpes_jur_data_fundacao': brw_pessoa.admpes_jur_data_fundacao,
            'admpes_jur_tipo_empresa': brw_pessoa.admpes_jur_tipo_empresa,
            'admpes_jur_suframa': brw_pessoa.admpes_jur_suframa,
            'admpes_jur_regime_tributario': brw_pessoa.admpes_jur_regime_tributario,
            'admpes_jur_tipo_atividade': brw_pessoa.admpes_jur_tipo_atividade,
            'admpes_jur_ie': brw_pessoa.admpes_jur_ie,
            'admpes_jur_cnae_id': brw_pessoa.admpes_jur_cnae_id.id,
            "foto": brw_pessoa.foto,
            'admpes_fis_sexo': brw_pessoa.admpes_fis_sexo,
            'admpes_fis_estado_civil': brw_pessoa.admpes_fis_estado_civil,
            'admpes_fis_data_nascimento': brw_pessoa.admpes_fis_data_nascimento,
            'admpes_fis_aniversario': brw_pessoa.admpes_fis_aniversario,
            'admpes_fis_produtor_rural': brw_pessoa.admpes_fis_produtor_rural,
            'admpes_fis_rg': brw_pessoa.admpes_fis_rg,
            'email': brw_pessoa.email,
            'telefone': brw_pessoa.telefone,
            'observacoes': brw_pessoa.observacoes,
            'create_date': brw_pessoa.create_date,
            'admpes_fis_cnh_nro': brw_pessoa.admpes_fis_cnh_nro,
            'admpes_fis_cnh_categoria': brw_pessoa.admpes_fis_cnh_categoria,
            'admpes_fis_cnh_validade': brw_pessoa.admpes_fis_cnh_validade,
            'admpes_fis_cnh_orgao_emissor': brw_pessoa.admpes_fis_cnh_orgao_emissor,
            'admpes_fis_rg_orgao_emissor': brw_pessoa.admpes_fis_rg_orgao_emissor,
            'admpes_fis_rg_data_emisssao': brw_pessoa.admpes_fis_rg_data_emisssao,
            'endereco_ids': endereco_ids,
            'email_ids': email_ids,
            'telefones_ids': telefones_ids,
            'endereco_nf_id': endereco_nf_id,
            'categoria_ids': categoria_ids,
            }
        }
        return res

    def _auto_init(self, cr, context=None):
        # Implementacao feita em 16/01/2.013 na versao 3A2 qdo foi implementado a heranca do res.partner para o adm.pessoa
        context = context and context.copy() or {}
        try:
            super(adm_pessoa, self)._auto_init(cr, context)
            cr.execute('select * from adm_pessoa where partner_id is null')
            pessoa_dados = cr.dictfetchall()
            if pessoa_dados:
                for pessoa in pessoa_dados:
                    _pes_name = pessoa.get('name') or False
                    _pes_id = pessoa.get('id') or 0
                    _sql = "insert into res_partner (name, notification_email_send) values ('%s', 'comment') returning id" % _pes_name
                    cr.execute(_sql)
                    partner_id = cr.fetchone()[0]
                    _sql = "update adm_pessoa set partner_id = %s where id = %s" % (
                        partner_id, _pes_id)
                    cr.execute(_sql)
                cr.execute('alter table adm_pessoa drop column name')
                cr.execute('ALTER TABLE adm_pessoa ALTER COLUMN partner_id SET NOT NULL')
                cr.commit()
        except:
            pass

    def admpes_button_salvar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True

    def admpes_endereco_fiscal(self, cr, uid, ids, context=None):
        """
        Obtem o endereco fiscal.
        Verifica se tem endereco de nf, senao pega o primeiro normal, senal o primeiro endereco
        ret = {113: {'complemento': False, 'bairro': u'Nossa Senhora do Carmo', 'cidade': u'Sete Lagoas',
                     'logradouro': u'Rua Terenas', 'uf_sigla': u'MG', 'numero': u'47',
                     'endereco': u'Rua Terenas, 47 - Nossa Senhora do Carmo - Sete Lagoas/MG', 'pais': u'Brasil',
                     'uf': u'MINAS GERAIS'},
               1: {'complemento': u'loja 01', 'bairro': u'Centro', 'cidade': u'Sete Lagoas',
                   'logradouro': u'Avenida Antonio Olinto', 'uf_sigla': u'MG', 'numero': u'281',
                   'endereco': u'Avenida Antonio Olinto, 281 - Centro - Sete Lagoas/MG', 'pais': u'BRASIL',
                   'uf': u'MINAS GERAIS'}}
        """
        context = context and context.copy() or {}
        retval = {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        obj_adm_uf = self.pool.get('adm.uf')
        context['super_read_adm_pessoa'] = True
        enderecos = self.read(cr, uid, ids, ['id', 'endereco_nf_id', 'endereco_ids'], context=context)
        for end in enderecos:
            endereco_id = False
            if end.get('endereco_nf_id'):
                endereco_id = end.get('endereco_nf_id')[0]
            elif not end.get('endereco_nf_id') and end.get('endereco_ids'):
                tipo_normal_id = self.pool.get('adm.endereco.tipo').search(cr, uid, [
                    ('name', '=', 'Normal')], context=context)
                if tipo_normal_id:
                    args = [
                        ('id', 'in', end.get('endereco_ids')),
                        ('endereco_tipo_id', '=', tipo_normal_id[0])
                    ]
                    endereco_id = obj_adm_pessoa_endereco.search(cr, uid, args, limit=1, context=context)
                else:
                    if not endereco_id:
                        endereco_id = [end.get('endereco_ids')[0]]
            else:
                retval.update({end.get('id'): False})
            if endereco_id:
                if isinstance(endereco_id, (list, tuple)):
                    endereco_id = endereco_id[0]
                endereco_data = obj_adm_pessoa_endereco.browse(cr, uid, endereco_id, context)
                pais = endereco_data.pais_id.name or False
                codigo_pais = endereco_data.pais_id.codigo_bc or False
                cidade = endereco_data.cidade_id.name or False
                bairro = endereco_data.bairro_id.name or False
                uf = endereco_data.uf_id.name or False
                uf_sigla = endereco_data.uf_id.sigla or False
                logradouro = endereco_data.logradouro_id.name or False
                complemento = endereco_data.complemento
                numero = endereco_data.numero
                endereco = endereco_data.name
                codigo_ibge = endereco_data.cidade_id.codigo_ibge or False
                retval.update({endereco_data.pessoa_id.id: dict(
                    pais=pais, cidade=cidade, bairro=bairro, uf=uf, uf_sigla=uf_sigla,
                    logradouro=logradouro,
                    complemento=complemento, numero=numero, endereco=endereco,
                    cep=endereco_data.cep,
                    codigo_ibge=codigo_ibge,
                    codigo_pais=codigo_pais,
                )}
                )
        return retval

    # noinspection PyMethodMayBeStatic
    def verificar_empresa_simples_nacional(self, cr, uid):
        '''
        Verfica se a empresa corrente e simples nacional retornando True or False.
        '''
        sql = '''
                SELECT CASE WHEN ap.admpes_jur_regime_tributario = 'S' 
                            THEN TRUE
                            ELSE FALSE
                       END AS simples_nacional
                FROM res_users as ru
                LEFT JOIN res_company AS rc ON rc.id = ru.company_id
                LEFT JOIN adm_pessoa AS ap ON ap.id = rc.rescon_pessoa_id
                WHERE ru.id = %s
              ''' % uid
        cr.execute(sql)

        return cr.dictfetchall()[0]['simples_nacional']

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = 1
        __code = 'pessoa.codigo'
        __name = 'Codigo de pessoa'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 9,
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence.type').search(cr, uid, [('code', '=', __code)], context=context)
        if not consulta:
            self.pool.get('ir.sequence.type').create(cr, uid, {'name': __name, 'code': __code}, context)
            #todo: verificar se e possivel isolar a transacao para apenas comitar no ir.sequence
            cr.commit()
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        #todo: verificar se e possivel isolar a transacao para apenas comitar no ir.sequence
        cr.commit()
        return True

    def get_dados_da_sig(self, cr, uid, cnpj, context=None):
        context = context and context.copy() or {}
        _obj_config_parameter = self.pool.get('ir.config_parameter')
        servidor = _obj_config_parameter.get_sig_server(cr, uid, context=context).split(',')
        try:
            host = servidor[0]
            db = servidor[1]
            porta = servidor[2]
            user = servidor[3]
            senha = servidor[4]
            sock_common = xmlrpclib.ServerProxy(
                'http://' + host + ':' + str(porta) + '/xmlrpc/common')
            socket.setdefaulttimeout(3)
            uid = sock_common.login(db, user, senha)
            sock = xmlrpclib.ServerProxy('http://' + host + ':' + str(porta) + '/xmlrpc/object')
            ids = sock.execute(db, uid, senha, 'adm.pessoa', 'search', [('cpf_cnpj', '=', cnpj)])
            vals = {}
            vals['pessoa'] = sock.execute(db, uid, senha, 'adm.pessoa', 'read', ids, [])[0]
            vals['email'] = []
            vals['fone'] = []
            vals['end'] = []
            ids_email = vals['pessoa']['email_ids']
            if ids_email:
                vals['email'] = sock.execute(db, uid, senha, 'adm.pessoa.email', 'read', ids_email,
                    [])
            ids_fones = vals['pessoa']['telefones_ids']
            if ids_fones:
                vals['fone'] = sock.execute(db, uid, senha, 'adm.pessoa.telefone', 'read',
                                            ids_fones, [])
            ids_end = vals['pessoa']['endereco_ids']
            if ids_end:
                vals['end'] = sock.execute(db, uid, senha, 'adm.pessoa.endereco', 'read', ids_end,
                    [])
        except Exception as err:
            self._logger.error(_('Erro comunicando com servidor da SIG!'))
            # Não conseguindo a conecção, remover o servidor da configuração para que possar ser gerado novamente,
            # talvez ele esteja errado
            server_id = self.pool.get('ir.config_parameter').search(cr, SUPERUSER_ID,
                                                                    [('key', '=', 'server_sig')], context=context)
            self.pool.get('ir.config_parameter').unlink(cr, SUPERUSER_ID, server_id)

            return False
        return vals

    def onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}

        if not cpf_cnpj or cpf_cnpj == 'f':
            return {'warning': warning, 'value': {}}
        if not pessoa_tipo:
            warning['title'] = _('Atencao')
            warning['message'] = _('Tipo de pessoa nao informado!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj_digitado = cpf_cnpj
            cpf_cnpj = res
            id_pessoa = self.pool.get('adm.pessoa').search(cr, uid,
                                                           [('cpf_cnpj', '=', cpf_cnpj_digitado)], context=context)
            if id_pessoa and id_pessoa != ids and not context.get('convenio_pessoa'):
                warning['title'] = _('Atencao')
                warning['message'] = _('CPF/CNPJ ja cadastrado para outra pessoa')
                return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _dados_sig = False
        if context.get('cadastro_dados_empresa'):
            _dados_sig = self.get_dados_da_sig(cr, uid, cpf_cnpj, context=context)
        if _dados_sig:
            vals_sig = {
                'pessoa_tipo': _dados_sig['pessoa']['pessoa_tipo'],
                'name': _dados_sig['pessoa']['name'],
                'fantasia_apelido': _dados_sig['pessoa']['fantasia_apelido'],
                'admpes_jur_tipo_atividade': _dados_sig['pessoa']['admpes_jur_tipo_atividade'],
                'admpes_fis_produtor_rural': _dados_sig['pessoa']['admpes_fis_produtor_rural'],
                'admpes_jur_tipo_empresa': _dados_sig['pessoa']['admpes_jur_tipo_empresa'],
                'admpes_jur_suframa': _dados_sig['pessoa']['admpes_jur_suframa'],
                'admpes_jur_data_fundacao': _dados_sig['pessoa']['admpes_jur_data_fundacao'],
                'admpes_jur_regime_tributario': _dados_sig['pessoa']['admpes_jur_regime_tributario'],
                'admpes_jur_ie': _dados_sig['pessoa']['admpes_jur_ie'],
                'admpes_fis_rg_orgao_emissor': _dados_sig['pessoa']['admpes_fis_rg_orgao_emissor'],
                'admpes_fis_rg_data_emisssao': _dados_sig['pessoa']['admpes_fis_rg_data_emisssao'],
                'admpes_fis_cnh_nro': _dados_sig['pessoa']['admpes_fis_cnh_nro'],
                'admpes_fis_cnh_orgao_emissor': _dados_sig['pessoa'][
                    'admpes_fis_cnh_orgao_emissor'],
                'admpes_fis_cnh_categoria': _dados_sig['pessoa']['admpes_fis_cnh_categoria'],
                'admpes_fis_cnh_validade': _dados_sig['pessoa']['admpes_fis_cnh_validade'],
                'admpes_fis_data_nascimento': _dados_sig['pessoa']['admpes_fis_data_nascimento'],
                'admpes_fis_sexo': _dados_sig['pessoa']['admpes_fis_sexo'],
                'admpes_fis_rg': _dados_sig['pessoa']['admpes_fis_rg'],
                'admpes_fis_estado_civil': _dados_sig['pessoa']['admpes_fis_estado_civil'],
                'email_ids': [],
                'telefones_ids': [],
                'endereco_ids': [],
            }
            for x in _dados_sig['email']:
                _ids = self.pool.get('adm.email.tipo').search(cr, uid, [
                    ('name', 'ilike', x['email_tipo_id'][1])], context=context)
                if _ids:
                    vals_sig['email_ids'].append((0, 0, {'email_tipo_id': _ids[0],
                                                         'pessoa_id': ids[0],
                                                         'name': x['name'],
                    }))
            for x in _dados_sig['fone']:
                _ids = self.pool.get('adm.telefone.tipo').search(cr, uid, [
                    ('name', 'ilike', x['telefone_tipo_id'][1])], context=context)
                if _ids:
                    vals_sig['telefones_ids'].append((0, 0, {'pessoa_id': ids[0],
                                                             'telefone_tipo_id': _ids[0],
                                                             'ddi': x['ddi'],
                                                             'ddd': x['ddd'],
                                                             'numero': x['numero'],
                    }))
            for x in _dados_sig['end']:
                _tipo_ids = self.pool.get('adm.endereco.tipo').search(cr, uid, [
                    ('name', 'ilike', x['endereco_tipo_id'][1])
                ], context=context)
                _pais_ids = self.pool.get('adm.pais').search(cr, uid, [
                    ('name', 'ilike', x['pais_id'][1])
                ], context=context)
                _uf_ids = self.pool.get('adm.uf').search(cr, uid, [
                    ('name', 'ilike', x['uf_id'][1])
                ], context=context)
                _cidade_ids = self.pool.get('adm.cidade').search(cr, uid, [
                    ('name', 'ilike', x['cidade_id'][1])
                ], context=context)
                _bairro_ids = self.pool.get('adm.bairro').search(cr, uid, [
                    ('name', 'ilike', x['bairro_id'][1])
                ], context=context)
                if _bairro_ids:
                    _bairro_id = _bairro_ids[0]
                else:
                    _bairro_id = self.pool.get('adm.bairro').create(cr, uid,
                                                                    {'name': x['bairro_id'][1]}, context)
                _lista = x['logradouro_id'][1].split()
                _tit = _lista[0]
                _lista.pop(0)
                _rua = ' '.join(_lista)
                _logradouro_ids = self.pool.get('adm.logradouro').search(cr, uid, [
                    ('name', 'ilike', _rua)
                ], context=context)
                if _logradouro_ids:
                    _logradouro_id = _logradouro_ids[0]
                else:
                    _logradouro_id = self.pool.get('adm.logradouro').create(cr, uid,
                                                                            {'titulo': _tit,
                                                                             'descricao': _rua}, context)
                vals_sig['endereco_ids'].append((0, 0, {'pessoa_id': ids[0],
                                                        'endereco_tipo_id': _tipo_ids[0],
                                                        'pais_id': _pais_ids[0],
                                                        'uf_id': _uf_ids[0],
                                                        'cidade_id': _cidade_ids[0],
                                                        'bairro_id': _bairro_id,
                                                        'logradouro_id': _logradouro_id,
                                                        'numero': x['numero'],
                                                        'complemento': x['complemento'],
                                                        'cep': x['cep'],
                }))
            return {'warning': warning, 'value': vals_sig}
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}

    def onchange_ie(self, cr, uid, ids, admpes_jur_ie):
        admpes_jur_ie = (admpes_jur_ie or '').upper()
        res, estado = self.admpes_validar_ie(cr, uid, ids, admpes_jur_ie)
        if not res:
            return {'warning': {'title': _('Atencao'),
                                'message': _(
                                    'Inscricao Estadual invalida para o seguinte estado: %s') % estado}}
        return {'value': {'admpes_jur_ie': admpes_jur_ie}}

    def fields_view_get(self, cr, user, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        if context.get('pessoa_id_oportunidades') and view_type == 'form':
            obj_model_data = self.pool.get('ir.model.data')
            reference = obj_model_data.get_object_reference(cr, user, 'adm', 'view_adm_pessoa_form')
            view_id = reference and reference[1] or False
        res = super(adm_pessoa, self).fields_view_get(cr, user, view_id, view_type, context,
                                                      toolbar, submenu)
        if context.get('simple_view', False):
            if res['type'] == 'form':
                if not context.get('client', False) == 'web':
                    res['toolbar'] = True
                res['arch'] = """
                    <form string="Pessoa">
                        <group>
                        <group>
                            <field name="name" colspan="4"/>
                            <field name="pessoa_tipo"/>
                            <field name="email_ids" colspan="4" nolabel="1"/>
                            <field name="telefones_ids" colspan="4" nolabel="1"/>
                        </group>
                        </group>
                    </form>
                """
        if res.get('type') == 'form' and context.get('funcionario'):
            doc = etree.XML(res['arch'])
            nodes = doc.xpath("//field[@name='admpes_fis_sexo']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_estado_civil']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_data_nascimento']")[0].set('attrs',
                                                                             "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='cpf_cnpj']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_rg']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='endereco_ids']")[0].set('attrs',
                                                                      "{'required':[('pessoa_tipo','!=',False)]}")
            res['arch'] = etree.tostring(doc)

        if res.get('type') == 'form' and context.get('comprador'):
            doc = etree.XML(res['arch'])
            nodes = doc.xpath("//field[@name='admpes_jur_data_fundacao']")[0].set('attrs',
                                                                           "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_ie']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_regime_tributario']")[0].set('attrs',
                                                                               "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_empresa']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_suframa']")[0].set('attrs',
                                                                     "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_atividade']")[0].set('attrs',
                                                                            "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_cnae_id']")[0].set('attrs',
                                                                     "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_rg']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_sexo']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_estado_civil']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_data_nascimento']")[0].set('attrs',
                                                                             "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='cpf_cnpj']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','!=',False)]}")
            nodes = doc.xpath("//field[@name='endereco_ids']")[0].set('attrs',
                                                                      "{'required':[('pessoa_tipo','!=',False)]}")
            res['arch'] = etree.tostring(doc)

        if res.get('type') == 'form' and context.get('vendedor'):
            doc = etree.XML(res['arch'])
            nodes = doc.xpath("//field[@name='admpes_jur_data_fundacao']")[0].set('attrs',
                                                                           "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_ie']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_regime_tributario']")[0].set('attrs',
                                                                               "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_empresa']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_suframa']")[0].set('attrs',
                                                                     "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_atividade']")[0].set('attrs',
                                                                            "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_cnae_id']")[0].set('attrs',
                                                                     "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_rg']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_sexo']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_estado_civil']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_data_nascimento']")[0].set('attrs',
                                                                             "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='cpf_cnpj']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','!=',False)]}")
            nodes = doc.xpath("//field[@name='endereco_ids']")[0].set('attrs',
                                                                      "{'required':[('pessoa_tipo','!=',False)]}")
            res['arch'] = etree.tostring(doc)

        if res.get('type') == 'form' and context.get('fornecedor'):
            doc = etree.XML(res['arch'])
            nodes = doc.xpath("//field[@name='admpes_jur_ie']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_regime_tributario']")[0].set('attrs',
                                                                               "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_empresa']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_atividade']")[0].set('attrs',
                                                                            "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_rg']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_sexo']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='cpf_cnpj']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','!=',False)]}")
            nodes = doc.xpath("//field[@name='endereco_ids']")[0].set('attrs',
                                                                      "{'required':[('pessoa_tipo','!=',False)]}")
            res['arch'] = etree.tostring(doc)

        if res.get('type') == 'form' and context.get('cliente'):
            doc = etree.XML(res['arch'])
            nodes = doc.xpath("//field[@name='admpes_jur_data_fundacao']")[0].set('attrs',
                                                                           "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_ie']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_regime_tributario']")[0].set('attrs',
                                                                               "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_empresa']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_jur_tipo_atividade']")[0].set('attrs',
                                                                            "{'required':[('pessoa_tipo','=','J')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_rg']")[0].set('attrs',
                                                                "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_sexo']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_estado_civil']")[0].set('attrs',
                                                                          "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='admpes_fis_data_nascimento']")[0].set('attrs',
                                                                             "{'required':[('pessoa_tipo','=','F')]}")
            nodes = doc.xpath("//field[@name='cpf_cnpj']")[0].set('attrs',
                                                                  "{'required':[('pessoa_tipo','!=',False)]}")
            nodes = doc.xpath("//field[@name='endereco_ids']")[0].set('attrs',
                                                                      "{'required':[('pessoa_tipo','!=',False)]}")
            res['arch'] = etree.tostring(doc)
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Le os dados de uma pessoa, e devolve-os com o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        # este contexto foi colocado para melhorar a performace ao criar uma nota com MUITOS itens
        if context.get('super_read_adm_pessoa'):
            return super(adm_pessoa, self).read(cr, uid, ids, fields, context=context, load=load)

        fields = [] if not fields else fields

        if fields:
            if not 'pessoa_tipo' in fields:
                fields.append('pessoa_tipo')
            if not 'fornecedor_ids' in fields:
                fields.append('fornecedor_ids')
            if not 'cliente_ids' in fields:
                fields.append('cliente_ids')
            if not 'admpes_cpf_cnpj_obrigatorio' in fields:
                fields.append('admpes_cpf_cnpj_obrigatorio')

        res = ids and super(adm_pessoa, self).read(cr, uid, ids, fields, context, load) or []
        cpf_cnpj_obrigatorio = False

        for pessoa in res if isinstance(res, list) else [res]:
            if not (pessoa.get('cliente_ids', False) or pessoa.get('fornecedor_ids', False)):
                cpf_cnpj_obrigatorio = pessoa.get('admpes_cpf_cnpj_obrigatorio')

            if pessoa.get('admpes_fis_aniversario', False) and pessoa['admpes_fis_aniversario'] and \
                            context.get('controle') == 'pessoa':
                _aniver = formatastring.formataString(pessoa['admpes_fis_aniversario'], 4, '0', 'E')

                if pessoa['admpes_fis_aniversario'] == 0:
                    pessoa['admpes_fis_aniversario'] = False
                else:
                    pessoa['admpes_fis_aniversario'] = _aniver[2:] + _aniver[:2]
            else:
                pessoa['admpes_fis_aniversario'] = False

            if pessoa and pessoa.get('cpf_cnpj', False) and pessoa['cpf_cnpj']:
                if pessoa.get('cpf_cnpj', False) and pessoa['cpf_cnpj']:
                    _cpf_cnpj = pessoa['cpf_cnpj']
                else:
                    _cpf_cnpj = ''

                if len(_cpf_cnpj) > 10:
                    val = re.sub('[^0-9]', '', _cpf_cnpj)
                    _pessoa_tipo = pessoa['pessoa_tipo']

                    if _pessoa_tipo == 'J' and len(val) == 14:
                        pessoa['cpf_cnpj'] = "%s.%s.%s/%s-%s" % (
                            val[0:2], val[2:5], val[5:8], val[8:12], val[12:14])
                    elif _pessoa_tipo == 'F' and len(val) == 11:
                        pessoa['cpf_cnpj'] = "%s.%s.%s-%s" % (
                            val[0:3], val[3:6], val[6:9], val[9:11])

            if context.get('controle'):
                pessoa['controle'] = context['controle']

            pessoa['admpes_cpf_cnpj_obrigatorio'] = cpf_cnpj_obrigatorio

        return res

    def create(self, cr, uid, vals, context=None):
        """
        Inclui uma pessoa no cadastro, retira a formatacao do CPF/CNPJ e
        verifica a validade do mesmo.
        Caso o usuario nao tenha informado um codigo de cliente, atribui
        um valor para ele.
        """
        _spg = self.pool.get
        context = context and context.copy() or {}
        brw_res_users = _spg('res.users').browse(cr, uid, uid, context=context)
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            return True # Usuario remoto nao atualiza mais
        if 'name' in vals:
            vals['name'] = _spg('adm.utils').admutl_capitalizar_adm_nome(cr, uid, vals.get('name'))
        if vals.get('codigo'):
            del vals['codigo']
        if 'foto' in vals:
            vals['foto'] = tools.image_resize_image(vals.get('foto'), (150, 150))
        res = True
        _force_cpf_cnpj = False
        if vals.get('cpf_cnpj') == 'f':
            _force_cpf_cnpj = True
            vals['cpf_cnpj'] = ''
        if vals.get('cpf_cnpj') and vals['cpf_cnpj']:
            vals['cpf_cnpj'] = re.sub('[^0-9]', '', vals['cpf_cnpj'])
            if vals['pessoa_tipo'] == 'F':
                nr = cpf.Cpf()
                while len(vals['cpf_cnpj']) < 11:
                    vals['cpf_cnpj'] = '0' + vals['cpf_cnpj']
                res = nr.validar(vals['cpf_cnpj'])
            elif vals['pessoa_tipo'] == 'J':
                nr = cnpj.Cnpj()
                while len(vals['cpf_cnpj']) < 14:
                    vals['cpf_cnpj'] = '0' + vals['cpf_cnpj']
                res = nr.validar(vals['cpf_cnpj'])
            if not res:
                raise osv.except_osv(_('Mensagem:'),
                                     _('CPF/CNPJ Invalido!\n' + vals['cpf_cnpj']))
            elif not context.get('controle') == 'fornecedor':
                cpf_cnpj_existente = self.search(cr, uid,
                                                 [('cpf_cnpj', '=', vals.get('cpf_cnpj', False))], context=context)
                if cpf_cnpj_existente:
                    codigo = self.read(cr, uid, cpf_cnpj_existente, ['codigo'], context=context)
                    raise osv.except_osv(_('Mensagem:'),
                                         _("CPF/CNPJ Ja cadastrado para outra pessoa (codigo: %s)" %
                                           codigo[0][
                                               'codigo']))
        if vals.get('admpes_jur_ie') and vals.get('uf'):
            if vals.get('admpes_jur_ie').isdigit():
                ie = inscricao_estadual.validar(vals.get('uf'), vals.get('admpes_jur_ie'))
                if not ie:
                    raise osv.except_osv(_('Atencao'),
                                         _('Inscricao Estadual invalida para o seguinte estado: %s') % vals.get(
                                             'uf').upper())
            else:
                vals['admpes_jur_ie'] = vals.get('admpes_jur_ie').upper()
        if vals.get('admpes_fis_cnh_nro'):
            vals['admpes_fis_cnh_nro'] = re.sub('[^0-9]', '', vals.get('admpes_fis_cnh_nro'))
        if vals.get('codigo') and vals['codigo'] != 0:
            pass
        else:
            vals['codigo'] = _spg('adm.utils').get_sequence(cr, uid, self._name,
                                                                     'pessoa.codigo', context)
        if 'name' in vals and vals.get('name'):
            name = vals.get('name')
            if len(name.split()) <= 1 and len(name.split('-')) <= 1 and len(name.split('.')) <= 1:
                raise osv.except_osv(_('Atencao'), _('O nome da pessoa deve ser composto por mais '
                                                     'de uma palavra'))
        if _force_cpf_cnpj:
            _cpf_cnpj = 'f%s00' % vals.get('codigo')
            _cpf_cnpj = _spg('adm.utils').admutl_cpf_cnpj_testar(
                                            vals.get('pessoa_tipo'),
                                            _cpf_cnpj, True)
            if _cpf_cnpj:
                vals['cpf_cnpj'] = _cpf_cnpj
            vals['cpf_cnpj'] = re.sub('[^0-9]', '', vals['cpf_cnpj'])
        result = super(adm_pessoa, self).create(cr, uid, vals, context)
        return result

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        _args = args[:]
        if '!' in args:
            args.remove('!')
        res = []
        obj_adm_utils = self.pool.get('adm.utils')

        def _admpes_search_fiscal(cr, uid, args, context):
            ids = []
            if context.get('fornecedor_deducao', False):
                obj_ded = self.pool.get('fiscal.deducoes.fornecedor')
                ded_ids = obj_ded.search(cr, uid, [], context=context)
                for i in obj_ded.browse(cr, uid, ded_ids):
                    ids.append(i.pessoa_id.id)
                args.append(('id', 'in', ids))
            return args

        def _admpes_search_rat(cr, uid, args, context):
            if context.get('Rat'):
                args = filter(lambda i: i[0] != 'id', args)
                sql = """
                    select
                        distinct pp.pessoa_id
                    from
                        project_br_project_br pp,
                        project_br_task pt
                    where
                        pp.state in ('open','pending') and
                        pp.pessoa_id is not null and
                        pt.project_br_id = pp.id and
                        pt.state in ('open','pending','draft')
                """
                rat_user_id = context.get('Rat_user_id')
                if rat_user_id:
                    res_users_dados = self.pool.get('res.users').read(cr, uid, rat_user_id,
                                                                      ['pessoa_id'], context=context)
                    if res_users_dados and isinstance(res_users_dados, list):
                        res_users_dados = res_users_dados[0]
                    if isinstance(res_users_dados, dict):
                        pessoa_id = res_users_dados.get('pessoa_id') and res_users_dados['pessoa_id'][
                            0] or False
                        if pessoa_id:
                            sql += """ and pt.user_id = %s""" % str(pessoa_id)

                cr.execute(sql)
                result = cr.fetchall()
                if result:
                    list_result = map(lambda i: int(i[0]), result)
                    if len(list_result) == 1:
                        args.append(('id', '=', list_result[0]))
                    elif len(list_result) > 1:
                        args.append(('id', 'in', list_result))
                    else:
                        args.append(('id', '=', 0))
            return args

        # contexto para identificar os contatos de uma pessoa
        if context.get('contatos_de'):
            _pessoa_id = context.get('contatos_de')
            obj_adm_pessoa_relacionamento = self.pool.get('adm.pessoa.relacionamento')
            relac_ids = obj_adm_pessoa_relacionamento.search(cr, uid, ['|',
                                                                       ('dependente_id', '=', _pessoa_id),
                                                                       ('pessoa_id', '=', _pessoa_id)], context=context)
            if relac_ids:
                contatos = {}
                for relacionamentos in obj_adm_pessoa_relacionamento.browse(cr, uid, relac_ids):
                    if _pessoa_id == relacionamentos.pessoa_id.id:
                        contatos[relacionamentos.dependente_id.id] = True
                    else:
                        contatos[relacionamentos.pessoa_id.id] = True
                contato_ids = contatos.keys()
                args = filter(lambda a: 'id' not in a, args)
                args.append(['id', 'in', contato_ids])
                contato_ids = super(adm_pessoa, self).search(cr, uid, args, context=context)
                return contato_ids
            else:
                return []

        if context.get('pessoa_convenio'):
            limit = None
        if not args:
            args = []
        context = context and context.copy() or {}
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context):
            args = _admpes_search_fiscal(cr, uid, args, context)
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'rat', context):
            args = _admpes_search_rat(cr, uid, args, context)
        i = 0
        args1 = []
        pessoa_ids = []

        adm_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'adm', 'admpar_nao_usar_soundex', context=context)
        nao_usar_soundex = adm_parametro_dados.get('admpar_nao_usar_soundex')

        for x in args:
            if 'name' in x:
                #args.pop(i)
                if nao_usar_soundex and len(args) == 1:
                    res = obj_adm_utils.admutl_get_objeto_ids_sem_fonema(
                        cr, uid, descricao=x[-1], nome_tabela='adm_pessoa', nome_coluna='nome',
                        context=context)
                elif nao_usar_soundex:
                    args1.append(x)
                else:
                    sndx = obj_adm_utils.get_fonema(x[-1])
                    if sndx != '00000':
                        args1.append(['soundex', 'like', '%' + sndx.replace(';', '%') + '%'])
            elif 'codigo' in x:
                try:
                    int(x[-1])
                    args1.append(['codigo', '=', x[-1]])
                    context['active_test'] = False
                except:
                    pass
            #Recurso para ler uma pessoa pelo nome(identico). Troca o atributo nome_igual para name
            elif 'nome_igual' in x:
                x1 = ['name', x[1], x[2]]
                args1.append(x1)
            elif 'cpf_cnpj' in x:
                try:
                    _cpfCnpj = re.sub('[^0-9]', '', x[-1])
                except:
                    _cpfCnpj = False
                args1.append(['cpf_cnpj', x[1], _cpfCnpj])
                context['active_test'] = False
            elif 'admpes_fis_aniversario' in x:
                try:
                    _aniver = formatastring.formataString(int(x[-1]), 4, '0', 'E')
                    _aniver = _aniver[2:] + _aniver[:2]
                    args1.append(['admpes_fis_aniversario', x[1], _aniver])
                except:
                    args1.append(x)
            elif 'id' and 'in' in x:
                pessoa_ids = x[-1]
            else:
                args1.append(x)
            i += 1
        list_context = [context.get('contrecf_vendas'), context.get('Rat_user_id'),
                        context.get('cliente_rat_id'), context.get('modal_rodoviario')]
        args = args1 if not any(list_context) else args
        args = _args if context.get('modal_rodoviario') else args
        if pessoa_ids and not any(list_context):
            pessoa_ids += res
            res = pessoa_ids
        if not res:
            res = super(adm_pessoa, self).search(cr, uid, args, offset, limit, order, context, count)
        if context.get('transportador', False):
            res = []
            obj_trans = self.pool.get('adm.transportador')
            trans_ids = obj_trans.search(cr, uid, [], context=context)
            for x in obj_trans.browse(cr, uid, trans_ids):
                res.append(x.pessoa_id.id)
        return res

    def _fincli_verifica_existencia_cliente(self, cr, uid, ids, vals, context=None):
        """
        Verifica se a pessoa ja esta gravada como cliente numa empresa epecifica.
        Uma pessoa somente pode estar como cliente em uma mesma empresa uma unica vez
        """
        context = context and context.copy() or {}
        cliente = self.pool.get('financeiro.cliente')
        vals_cliente_ids = vals.get('cliente_ids')
        if vals_cliente_ids:
            cliente_ids_apagados = []
            for i in vals_cliente_ids:
                if not isinstance(i, int):
                    if i[0] == 2:
                        cliente_ids_apagados.append(i[1])
            for i in vals_cliente_ids:
                if not isinstance(i, int):
                    if i[2]:
                        company_id = i[2].get('company_id')
                        pessoa_id = ids[0]
                        cliente_ids = cliente.search(cr, uid, [('pessoa_id', '=', pessoa_id),
                                                               ('company_id', '=', company_id)], context=context)
                        if cliente_ids:
                            cliente_ids = cliente_ids[0]
                        if cliente_ids and cliente_ids not in cliente_ids_apagados and cliente_ids != \
                                i[1]:
                            raise osv.except_osv(_('Atencao'),
                                                 _('Cliente ja cadastrado para esta empresa'))

    def write(self, cr, uid, ids, vals, context=None):
        """
        Altera os dados no cadastro de pessoa.
        Valida o CPF/CNPJ, e retira a formatacao do mesmo.
        """
        # Para usuario remoto, se vier "codigo" no "vals", retirar
        # É atualização de cliente na base da Sig
        context = context and context.copy() or {}
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            return True # nao deixar mais o usuario remoto atualizar o cadastro da empresa
        self._fincli_verifica_existencia_cliente(cr, uid, ids, vals, context=context)
        res = True
        if 'foto' in vals:
            vals['foto'] = tools.image_resize_image(vals.get('foto'), (150, 150))
        if vals.get('cpf_cnpj') and vals['cpf_cnpj']:
            if vals.get('pessoa_tipo'):
                _pessoa_tipo = vals['pessoa_tipo']
            else:
                for individuo in pooler.get_pool(cr.dbname).get('adm.pessoa').read(cr, uid, ids,
                                                                                   ['pessoa_tipo',
                                                                                    'codigo'], context=context):
                    _pessoa_tipo = individuo['pessoa_tipo']
                    _codigo = individuo['codigo']
            if vals['cpf_cnpj'] != '':
                vals['cpf_cnpj'] = re.sub('[^0-9]', '', vals['cpf_cnpj'])
            res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(_pessoa_tipo, vals['cpf_cnpj'])
            if not res:
                raise osv.except_osv(_('Mensagem:'),
                                     _('CPF/CNPJ Invalido!'))
            else:
                if not isinstance(ids, (list, tuple)):
                    ids = [ids]
                cpf_cnpj_existente = self.search(cr, uid,
                                                 [('cpf_cnpj', '=', vals.get('cpf_cnpj', False))], context=context)
                if cpf_cnpj_existente and cpf_cnpj_existente[0] != ids[0]:
                    codigo = self.read(cr, uid, cpf_cnpj_existente, ['codigo'], context=context)
                    raise osv.except_osv(_('Mensagem:'), _(
                        "CPF/CNPJ Ja cadastrado para outra pessoa ( codigo: %s)" % codigo[0][
                            'codigo']))
        if vals.get('admpes_jur_ie'):
            if vals.get('admpes_jur_ie').isdigit():
                if vals.get('admpes_jur_ie') and vals.get('uf'):
                    ie = inscricao_estadual.validar(vals.get('uf'), vals.get('admpes_jur_ie'))
                    if not ie:
                        raise osv.except_osv(_('Atencao'),
                                             _('Inscricao Estadual invalida para o seguinte estado: %s') % vals.get(
                                                 'uf').upper())
                else:
                    for i in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
                        estado = i.uf
                    if vals.get('admpes_jur_ie') and estado:
                        ie = inscricao_estadual.validar(estado, vals.get('admpes_jur_ie'))
                        if not ie:
                            raise osv.except_osv(_('Atencao'),
                                                 _('Inscricao Estadual invalida para o seguinte estado: %s') % estado.upper())
            else:
                vals['admpes_jur_ie'] = vals.get('admpes_jur_ie').upper()
        if vals.get('admpes_fis_cnh_nro'):
            vals['admpes_fis_cnh_nro'] = re.sub('[^0-9]', '', vals.get('admpes_fis_cnh_nro'))

        if 'name' in vals and vals.get('name'):
            vals['name'] = self.pool.get('adm.utils').admutl_capitalizar_adm_nome(cr, uid,vals.get('name'), context=context)
            if len(vals.get('name').split()) <= 1:
                raise osv.except_osv(_('Atencao'), _('O nome da pessoa deve ser composto por mais '
                                                     'de uma palavra'))
        result = super(adm_pessoa, self).write(cr, uid, ids, vals, context)
        return result

    # retorna 'pessoa_id' de todos os clientes ativos
    def get_cliente_ids(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        if not ids:
            return ids
        if not isinstance(ids, list):
            ids = [ids]
        clientes_ids = self.pool.get('financeiro.cliente').search(cr, user, [('pessoa_id', 'in', ids), ('active','=',True)],
                                                                  context=context)
        dados_clientes = self.pool.get('financeiro.cliente').read(cr, user, clientes_ids, ['pessoa_id'], context=context)
        ids = []
        for i in dados_clientes:
            ids.append(i.get('pessoa_id')[0])
        return ids

    # retorna 'pessoa_id' de todos os funcionarios ativos
    def get_funcionario_ids(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        if not ids:
            return ids
        if not isinstance(ids, list):
            ids = [ids]
        funcionario_ids = self.pool.get('rh.funcionario').search(cr, user, [('pessoa', 'in', ids), ('active','=',True)],
                                                                 context=context)
        dados_funcionarios = self.pool.get('rh.funcionario').read(cr, user, funcionario_ids, ['pessoa'], context=context)
        ids = []
        for i in dados_funcionarios:
            ids.append(i.get('pessoa')[0])
        return ids

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        """
        Metodo para pesquisa de pessoas
        Para pesquisar pessoas existem algumas opcoes:
          - Informe o nome ou parte de um nome de uma pessoa, ou
          - qdo o nome comecar por 'a=', pesquisar por apelido/fantasia
          - qdo o nome comecar por 'f=', pesquisar por telefone
          - qdo o nome comecar por 'e=', pesquisar por email
          - qdo o nome comecar por 'd=', pesquisar por data de nascimento
          - qdo o nome comecar por 'ex=', pesquisar por codigo externo
          - qdo no nome contiver apenas numeros, se a quantidade de digitos for:
            - igual a 11 ou 14, pesquisar por cpf/cnpj
            - qualquer outra quantidade, pesquisar por codigo
        """
        if not args:
            args = []
        context = context and context.copy() or {}
        obj_adm_util = self.pool.get('adm.utils')
        cliente = False
        if context.get('cliente'): # contextp avisando que está buscando um cliente
            cliente = True
        funcionario = False
        if context.get('funcionario'): # contextp avisando que está buscando um cliente
            funcionario = True
        if len(name) > 2:
            if name[:2].lower() == 'a=': # buscar pessoa por apelido fantasia
                _apelido = name[2:]
                ids = self.search(cr, user, [('fantasia_apelido', operator, _apelido)] + args,
                                  limit=limit,
                                  context=context)
                return self.name_get(cr, user, ids, context)
            if name[:2].lower() == 'f=': # buscar pessoa por telefone
                _numero = re.sub('[^0-9]', '', name[2:])
                fone_ids = pooler.get_pool(cr.dbname).get('adm.pessoa.telefone').search(cr, user,
                                                                                        [('numero',
                                                                                          '=',
                                                                                          _numero)],
                                                                                        limit=limit, context=context)
                lido = pooler.get_pool(cr.dbname).get('adm.pessoa.telefone').read(cr, user,
                                                                                  fone_ids,
                                                                                  ['pessoa_id'], context=context)
                ids = []
                for x in lido:
                    ids.append(x['pessoa_id'][0])
                if ids:
                    if cliente:
                        ids = self.get_cliente_ids(cr, user, ids, context)
                    if funcionario:
                        ids = self.get_funcionario_ids(cr, user, ids, context)
                    return self.name_get(cr, user, ids, context)
            if name[:2].lower() == 'd=': # buscar pessoa por data de nascimento
                _temp = str(re.sub('[^0-9]', '', name[2:]))
                ids = []
                if len(_temp) == 8:
                    _data = _temp[4:] + '-' + _temp[2:4] + '-' + _temp[0:2]
                    ids = self.search(cr, user, [('admpes_fis_data_nascimento', '=', _data)] + args,
                                      limit=limit,
                                      context=context)
                return self.name_get(cr, user, ids, context)
            if name[:2].lower() == 'e=': # buscar pessoa por email
                _email = name[2:]
                email_ids = pooler.get_pool(cr.dbname).get('adm.pessoa.email').search(cr, user, [
                    ('name', '=', _email)],
                                                                                      limit=limit, context=context)
                lido = pooler.get_pool(cr.dbname).get('adm.pessoa.email').read(cr, user, email_ids,
                                                                               ['pessoa_id'], context=context)
                ids = []
                for x in lido:
                    ids.append(x['pessoa_id'][0])
                if ids:
                    if cliente:
                        ids = self.get_cliente_ids(cr, user, ids, context)
                    if funcionario:
                        ids = self.get_funcionario_ids(cr, user, ids, context)
                    return self.name_get(cr, user, ids, context)
            if name[:2].lower() == 'c=':  # buscar pessoa pelo cep
                _cep = name[2:]
                cep_ids = pooler.get_pool(cr.dbname).get('adm.pessoa.endereco').search(
                    cr, user, [('cep', '=', _cep)], limit=limit, context=context)
                lido = pooler.get_pool(cr.dbname).get('adm.pessoa.endereco').read(cr, user, cep_ids,
                                                                                  ['pessoa_id'], context=context)
                ids = []
                for x in lido:
                    ids.append(x['pessoa_id'][0])
                if ids:
                    if cliente:
                        ids = self.get_cliente_ids(cr, user, ids, context)
                    if funcionario:
                        ids = self.get_funcionario_ids(cr, user, ids, context)
                    return self.name_get(cr, user, ids, context)
            if name[:3].lower() == 'ex=':  # buscar pessoa pelo id externo
                _apelido = name[3:]
                operator = '='
                ids = self.search(cr, user, [('admpes_id_externo', operator, _apelido)] + args,
                                  limit=limit, context=context)
                return self.name_get(cr, user, ids, context)
            if name[:3].lower() == 'ce=':  # buscar pessoa por codigo externo
                _apelido = name[3:]
                ids = self.search(cr, user, [('admpes_id_externo', operator, _apelido)] + args,
                                  limit=limit, context=context)
                return self.name_get(cr, user, ids, context)
        ids_sem_fonema = list()
        if name.isdigit(): # Verifica se e CPF/CNPJ
            if len(name) == 14 or len(name) == 11:
                filtro = [('cpf_cnpj', '=', name)]
            else:
                # Pesquisa pelo codigo interno
                filtro = [('codigo', '=', name)]
            ids = self.search(cr, user, filtro + args, limit=limit, context=context)
        else:
            adm_parametro_dados = obj_adm_util.admutl_get_parametros(
                cr, user, 'adm', 'admpar_nao_usar_soundex', context=context)
            nao_usar_soundex = adm_parametro_dados.get('admpar_nao_usar_soundex')
            if nao_usar_soundex and name:
                ids = obj_adm_util.admutl_get_objeto_ids_sem_fonema(
                    cr, user, descricao=name, nome_tabela='adm_pessoa', nome_coluna='nome',
                    context=context)
                ids_sem_fonema = ids
            else:
                limite = 80
                if context.get('funcionario'):
                    limite = None
                ids = self.search(cr, user, [('name', operator, name)] + args, context=context,
                                  limit=limite)
        if cliente:
            ids = self.get_cliente_ids(cr, user, ids, context)
            if len(ids) > limit:
                ids = ids[:limit]
        if funcionario:
            ids = self.get_funcionario_ids(cr, user, ids, context)
            if len(ids) > limit:
                ids = ids[:limit]
        if ids_sem_fonema:
            return self.name_get_sem_fonema(cr, user, ids, context)
        else:
            return self.name_get(cr, user, ids, context)

    def name_get_sem_fonema(self, cr, uid, ids, context=None):
        """
        Metodo name_get especifico para montagem da descricao da pessoa quando a busca e realizada
        sem fonema (admutl_get_objeto_ids_sem_fonema).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados com valores contextuais.
        :type context: dict
        :return: lista de tuplas com o id e a descricao para esse.
        :rtype: list
        """
        res = []
        context = context and context.copy() or {}
        if not len(ids):
            return []
        # AQUI NAO PODE SER UTILIZADO O READ POIS ESSE ALTERA A ORDEM DOS IDS RECEBIDOS E ATRAPALHA
        # A SEQUENCIA DAS DESCRICOES PROCESSADAS PELO METODO 'admutl_get_objeto_ids_sem_fonema'.
        for pessoa in self.browse(cr, uid, ids, context):
            codigo = str(pessoa.codigo or '')
            if context.get('admpes_retorno_sem_codigo'):
                codigo = ''

            result = "%s%s%s" % (codigo, ' - ' if codigo else '', pessoa.nome)
            res.append((pessoa.id, result))

        return res

    def name_get(self, cr, uid, ids, context=None):
        res = []
        if context is None:
            context = []
        if not len(ids):
            return []

        context['super_read_adm_pessoa'] = True
        for pessoa in self.read(cr, uid, ids, ['codigo', 'nome'], context):
            codigo = str(pessoa.get('codigo') or '')
            if context.get('admpes_retorno_sem_codigo'):
                codigo = ''

            result = "%s%s%s" % (codigo, ' - ' if codigo else '', pessoa.get('nome'))
            res.append((pessoa.get('id'), result))

        return res

    def admpes_get_endereco_separado(self, cr, uid, pessoa_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if not pessoa_id:
            return {}
        obj = self.pool.get('adm.pessoa.endereco')
        endereco_nf_id = self.read(cr, uid, pessoa_id, ['endereco_nf_id'], context, '_classic_write')
        args = [('pessoa_id', '=', pessoa_id)]
        endereco_ids = obj.search(cr, uid, args, context=context)
        if endereco_ids:
            for endereco_id in endereco_ids:
                endereco_brw = obj.browse(cr, uid, endereco_id)
                res = {'endereco': self._get_endereco(endereco_brw, context),
                       'endereco_rua': endereco_brw.logradouro_id.name,
                       'endereco_nro': endereco_brw.numero or 'S/N',
                       'endereco_compl': endereco_brw.complemento or '',
                       'cidade': endereco_brw.cidade_id.name or '',
                       'bairro': endereco_brw.bairro_id.name or '',
                       'uf': endereco_brw.uf_id.sigla or '',
                       'pais': endereco_brw.pais_id.name or '',
                       'cep': endereco_brw.cep or '',
                       }
                if endereco_id == endereco_nf_id or not endereco_nf_id:
                    break
                continue
        return res

    def _get_endereco(self, endereco_brw, context=None):
        context = context and context.copy() or {}
        nro = endereco_brw.numero or 'S/N'
        compl = endereco_brw.complemento
        endereco = endereco_brw.logradouro_id.name or ''
        if nro:
            endereco = endereco + ', ' + nro

        if compl:
            endereco = endereco + '-' + compl

        return endereco

    def _get_base_endereco(self, cr, uid, ids, name, arg, context=None, tipo=None, name_id='name'):
        context = context and context.copy() or {}
        res = {}
        res_sem_tipo = {}
        if not len(ids):
            return {}
        obj = self.pool.get('adm.pessoa.endereco')
        dict_pessoa_end = {}
        map(lambda x: dict_pessoa_end.update({x.get('id'): x.get('endereco_nf_id')}),
            self.read(cr, uid, ids, ['endereco_nf_id'], context, '_classic_write'))
        for id in ids:
            endereco_nf_id = dict_pessoa_end.get(id) or False
            args = [('pessoa_id', '=', id)]
            endereco_ids = obj.search(cr, uid, args, context=context)
            if endereco_ids:
                for endereco_id in endereco_ids:
                    endereco_brw = obj.browse(cr, uid, endereco_id)
                    if not tipo:
                        res_sem_tipo[id] = {'endereco': self._get_endereco(endereco_brw, context),
                                            'cidade': endereco_brw.cidade_id.name or '',
                                            'bairro': endereco_brw.bairro_id.name or '',
                                            'uf': endereco_brw.uf_id.sigla or '',
                                            'pais': endereco_brw.pais_id.name or '',
                                            'cep': endereco_brw.cep or '',
                                            'codigo_ibge': endereco_brw.cidade_id.codigo_ibge or 0,
                                            }
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue

                    if tipo == 'endereco':
                        res[id] = self._get_endereco(endereco_brw, context)
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue

                    if tipo == 'cidade':
                        if name_id == 'id':
                            res[id] = endereco_brw.cidade_id.id
                        else:
                            res[id] = endereco_brw.cidade_id.name
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue
                    if tipo == 'codigo_ibge':
                        res[id] = endereco_brw.cidade_id.codigo_ibge or 0
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue
                    if tipo == 'bairro':
                        if name_id == 'id':
                            res[id] = endereco_brw.bairro_id.id
                        else:
                            res[id] = endereco_brw.bairro_id.name
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue
                    if tipo == 'estado':
                        if name_id == 'id':
                            res[id] = endereco_brw.uf_id.id
                        else:
                            res[id] = endereco_brw.uf_id.sigla
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue
                    if tipo == 'pais':
                        if name_id == 'id':
                            res[id] = endereco_brw.pais_id.id
                        else:
                            res[id] = endereco_brw.pais_id.name
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue
                    if tipo == 'cep':
                        cep = endereco_brw.cep
                        res[id] = cep
                        if endereco_id == endereco_nf_id or not endereco_nf_id:
                            break
                        continue
                    res[id] = ''
            else:
                res[id] = ''
                if not tipo:
                    res_sem_tipo[id] = {'endereco': '',
                                        'cidade': '',
                                        'bairro': '',
                                        'uf': '',
                                        'pais': '',
                                        'cep': '',
                                        'codigo_ibge': ''
                                        }
        if not tipo and res_sem_tipo:
            return res_sem_tipo

        return res

    def open_map(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        endereco_obj = self.pool.get('adm.pessoa')
        pessoa = endereco_obj.browse(cr, uid, ids, context=context)[0]
        url = "http://maps.google.com/maps?oi=map&q="
        if pessoa.endereco:
            url += pessoa.endereco.replace(' ', '+')
        if pessoa.cidade:
            url += '+' + pessoa.cidade.replace(' ', '+')
        if pessoa.uf:
            url += '+' + pessoa.uf.replace(' ', '+')
        if pessoa.pais:
            url += '+' + pessoa.pais.replace(' ', '+')
        if pessoa.cep:
            url += '+' + pessoa.cep.replace(' ', '+').replace('.', '')
        return {
            'type': 'ir.actions.act_url',
            'url': url,
            'target': 'new'
        }

    def get_telefones(self, cr, uid, ids, context=None):
        """Retorna os Telefones da Pessoa agrupados por tipo

        :param cr: database cursor
        :param uid: current user id
        :param ids: list of ids
        :param context: context arguments, like lang, time zone
        :rtype: {1: {u'Fax': [u'313652-8785'],
                     u'Comercial': [u'311254-5896', u'313771-2990'],
                     u'Celular': [u'313586-9856']
                    },

                }
        ex:
            fones = obj_adm_pessoa.get_telefones(cr, uid, pessoa_id).get(pessoa_id)
            if not fones:
                raise except_osv('Atencao', 'Numero do fax do estabelecimento informante nao cadastrado')
            fax = fones.get('Fax') or fones.get('Comercial') or ['']
            fax = int(re.sub('[^0-9]', '', fax[0]))
        """

        class groupby(dict):
            def __init__(self, seq, key=lambda x: x):
                for value in seq:
                    k = key(value)
                    self.setdefault(k, []).append(value[1])

            __iter__ = dict.iteritems

        context = context and context.copy() or {}
        data_adm_pessoa = self.read(cr, uid, ids, ['telefones_ids', 'id'], context=context)
        obj_adm_pessoa_telefone = self.pool.get('adm.pessoa.telefone')
        retval = {}
        for data in data_adm_pessoa:
            data_adm_pessoa_telefone = obj_adm_pessoa_telefone.read(cr, uid,
                                                                    data.get('telefones_ids'), context=context)
            lst = map(lambda x: (
                x.get('telefone_tipo_id')[1], '(%s)%s' % (x.get('ddd') or '', x.get('numero'))),
                      data_adm_pessoa_telefone)
            retval.update({data.get('id'): groupby(lst, key=lambda r: r[0])})
        return retval

    def pessoa_ativa(self, cr, uid, ids, context=None):
        """
        Verifica se a pessoa esta ou nao ativa.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se a pessoa estiver ativa.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        context['active_test'] = False  # Permite recuperar os registros inativos
        ids = [ids] if not isinstance(ids, (list, tuple)) else ids
        dados_pessoa = self.read(cr, uid, ids or 0, ['active'], context=context)

        if dados_pessoa and isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]

        # Se nao foi encontrada, nao ha como estar inativa (retorna verdadeiro)
        return dados_pessoa.get('active', True) if dados_pessoa else True

    # TODO: Alterar para utilizar o metodo ´admpes_get_tipo_consumidor´
    def admpes_get_tipo_consumidor_final(self, cr, uid, pessoa_id, context=None):
        """
        Este metodo verifica se a pessoa e um contribuinte ou consumidor final
        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param pessoa_id: ID da pessoa a ser verificada
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type pessoa_id: int|long
        :type context: dict
        :rtype: boolean
        :return: Retorna True se consumidor final
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        import logging
        logger = logging.getLogger(self._name)
        logger.warning(
            'O uso do metodo admpes_get_tipo_consumidor_final() deve ser substituido pelo metodo '
            'admpes_get_tipo_consumidor().')
        context = context and context.copy() or {}
        consumidor_final = True
        context['super_read_adm_pessoa'] = True
        dados_pessoa = self.read(cr, uid, pessoa_id, ['pessoa_tipo', 'admpes_jur_ie'],
                                 context=context)
        if dados_pessoa.get('pessoa_tipo') == 'J' and dados_pessoa.get('admpes_jur_ie') and \
                        dados_pessoa.get('admpes_jur_ie')  not in ('ISENTO', 'ISENTA'):
            consumidor_final = False

        return consumidor_final

    def admpes_get_tipo_consumidor(self, cr, user, pessoa_id, context=None):
        """
        Retorna um dict onde a chave eh o id da pessoa e o valor eh o tipo da pessoa:
         ´C´ se for contribuinte, ´F´ se for consumidor final ou ´P´ se
        for produtor rural.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param pessoa_id: ID da pessoa a ser verificada
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type pessoa_id: long
        :type context: dict
        :rtype: str
        :return: ´C´ se for contribuinte, ´F´ se for consumidor final ou ´P´ se for produtor rural.
        """
        context = context and context.copy() or {}
        context['super_read_adm_pessoa'] = True
        pessoa_dados = pessoa_id and self.read(
            cr, user, pessoa_id, ['pessoa_tipo', 'admpes_jur_ie', 'admpes_fis_produtor_rural'],
            context=context) or {}
        tipo = 'F'  # Assume como pessoa fisica (consumidor final)
        if pessoa_dados:
            IE = pessoa_dados.get('admpes_jur_ie') or ''
            IE = IE.strip().upper() or 'ISENTO'
            if pessoa_dados.get('admpes_fis_produto_rural'):
                tipo = 'P' if not IE == 'ISENTO' else 'C'
            if pessoa_dados.get('pessoa_tipo') == 'J' and IE not in ('ISENTO', 'ISENTA'):
                tipo = 'C'  # Contribuinte

        return tipo

    def admpes_onchange_tipo_atividade(self, cr, uid, ids, pessoa_tipo, tipo_atividade,
                                       context=None):
        """
        Metodo chamado na saida nos atributos pessoa_tipo e admpes_jur_tipo_atividade. Valida a
        natureza da atividade para pessoas fisicas e juridica.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param pessoa_tipo: string que representa a pessoa fisica ('F') ou juridica ('J').
        :type pessoa_tipo: str
        :param tipo_atividade: string que representa a natureza da atividade exercida pela empresa.
        :type tipo_atividade: str
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados dos atributos com seus respectivos valores e/ou mensagem de alerta ao
        usuario.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not tipo_atividade:
            return res
        if pessoa_tipo == 'F' and tipo_atividade != 'T':
            res['value'] = {'admpes_jur_tipo_atividade': ''}
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Natureza da atividade invalida para pessoa fisica')}
        return res

    def admpes_get_cobranca_endereco_id(self, cr, uid, pessoa_id, context=None):
        """
        O método encontra o registro (id) da tabela adm_pessoa_endereco para a pessoa_id passada, na
        seguite ordem: 1 - Endereco tipo cobranca, 2 - Primeiro encontrado. Com o modulo fiscal
        instalado (tratamento feito via heranca), a ordem eh a seguinte: 1 - Endereco tipo cobranca,
        2 - Endereco para NF e 3 - Primeiro encontrado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: id da tabela adm_pessoa_endereco, caso encontre o registro. Senao, Falso.
        :rtype: int
        """
        context = context and context.copy() or {}
        brw_adm_pessoa_endereco = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id,
                                                                     context).endereco_ids
        cobranca_endereco_ids = filter(lambda end: end.endereco_tipo_id.name == 'Cobranca',
                                       brw_adm_pessoa_endereco)
        endereco_id = False
        if cobranca_endereco_ids:
            endereco_id = cobranca_endereco_ids[0].id
        elif brw_adm_pessoa_endereco:
            endereco_id = brw_adm_pessoa_endereco[0].id
        return endereco_id

    def admpes_incluir_pessoa(self, cr, uid, context=None, **dadosRec):
        """
        Metodo para incluir pessoa
        Dados fornecidos:
           - Os dados fornecidos devem ser os nomes dos atributos da classe
           - Alem dos atributos da classe, temos tambem alguns campos especiais que são:
             - nome: nome da pessoa, não está na classe, está no partner
        return {'status': True/False, 'mensagem':'', pessoa_id: id_da_pessoa_criada}

        """
        _dados = dadosRec.copy()
        if _dados.get('partner_id'):
            _partner_id = _dados.get('partner_id')
            _dados.pop('partner_id')
        else:
            if not _dados.get('nome'):
                return {'status': False, 'mensagem': _('Nao foi informado nome da pessoa!')}
            _partner_id = self.pool.get('res.partner').create(cr, uid, {'name': _dados.get('nome')}, context=context)
            _dados.pop('nome')
        if _dados.get('pessoa_tipo') and _dados.get('pessoa_tipo').upper() in ('F', 'J'):
            _pessoa_tipo = _dados.get('pessoa_tipo').upper()
            _dados.pop('pessoa_tipo')
        else:
            if _dados.get('pessoa_tipo'):
                return {'status': False, 'mensagem': _('pessoa_tipo informado e invalido!')}
        if not _dados.get('pessoa_tipo'):
            _pessoa_tipo = 'F'
            if _dados.get('cpf_cnpj'):
                if len(_dados.get('cpf_cnpj')) > 11:
                    _pessoa_tipo = 'J'
        _vals_pessoa = {
            'partner_id': _partner_id,
            'pessoa_tipo': _pessoa_tipo,
        }
        for i in _dados.keys():
            _vals_pessoa[i] = _dados.get(i)
        _pessoa_id = self.create(cr, uid, _vals_pessoa, context=context)

        # _email_tipo_ids = self.pool.get('adm.email.tipo').search(cr, uid, [('name', '=', tipo_email)],
        #                                                          context=context)
        # if ie:
        #     _vals_pessoa['admpes_jur_ie'] = ie
        # if _email_tipo_ids and email:
        #     self.__mkputl_cadastrar_email(cr, uid, _pessoa_id, _email_tipo_ids[0], email, context=context)
        # if fone1:
        #     self.__mkputl_cadastrar_fone(cr, uid, _pessoa_id, ddd1, fone1, context=context)
        # if fone2:
        #     self.__mkputl_cadastrar_fone(cr, uid, _pessoa_id, ddd2, fone2, context=context)
        # _pais_id, _estado_id, _localidade_id, _bairro_id, _logradouro_id = self.__mkputl_cadastrar_cep(cr, uid, cep,
        #                                                                                                context=context)
        # if not _logradouro_id and rua:
        #     _logradouro_id = self.__mkputl_cadastrar_logradouro(cr, uid, rua, context=context)
        # if not _pais_id and outros_dados.get('pais'):
        #     _pais_id = self.mkputl_orders_pesquisar_incluir_pais(cr, uid, outros_dados.get('pais'), context=context)
        # if outros_dados.get('estado'):
        #     _estado_id = self.mkputl_orders_pesquisar_incluir_uf(cr, uid, _pais_id, outros_dados.get('estado'),
        #                                                          context=context)
        # if outros_dados.get('cidade') and not '***' in outros_dados.get('cidade'):
        #     _localidade_id = self.mkputl_orders_pesquisar_incluir_localidade(cr, uid, outros_dados.get('cidade'),
        #                                                                      _estado_id, context=context)
        # if outros_dados.get('bairro') and not "***" in outros_dados.get('bairro'):
        #     _bairro_id = self.mkputl_orders_pesquisar_incluir_bairro(cr, uid, outros_dados.get('bairro'),
        #                                                              context=context)
        #
        # _endereco_tipo_id = \
        #     self.pool.get('adm.endereco.tipo').search(cr, uid, [('name', '=', 'Normal')], context=context)[0]
        # _vals_endereco = {
        #     'cep': cep,
        #     'endereco_tipo_id': _endereco_tipo_id,
        #     'complemento': rua_complemento,
        #     'numero': rua_nro,
        #     'pessoa_id': _pessoa_id,
        #     'cidade_id': _localidade_id,
        #     'logradouro_id': _logradouro_id,
        #     'pais_id': _pais_id,
        #     'uf_id': _estado_id,
        #     'bairro_id': _bairro_id,
        # }
        # _end_id = self.pool.get('adm.pessoa.endereco').create(cr, uid, _vals_endereco, context=context)
        # obj_pessoa.write(cr, uid, _pessoa_id, {'endereco_nf_id': _end_id}, context=context)
        return {'status': True, 'pessoa_id': _pessoa_id}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

