# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################


from openerp.osv import osv, fields


class adm_pessoa_relacionamento(osv.Model):
    """
    Tabela de Dependentes ou contatos, se pessoa fisica ou juridica.
    Nesta tabela consta os dependentes da pessoa fisica e/ou
    os contatos da pessoa juridica
    """

    #prefixo = admpre_

    _inherit = 'adm.pessoa.relacionamento'

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        """
        Busca pelo nome da pessoa do relacionamento
        """
        context = context and context.copy() or {}
        ids = []
        if isinstance(name, (str, unicode)):
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit,
                              context=context)
        if not ids:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def name_get(self, cr, uid, ids, context=None):
        """
        Monta uma lista de tuplas que e composta pelo id do objeto e a descricao que sera mostrada
        em tela
        """
        res = []
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.browse(cr, uid, ids, context)
        for record in reads:
            res.append((record['id'], record.dependente_id.nome))
        return res

    def create(self, cr, uid, vals, context=None):
        # Gravar duas vezes o relacionamento normal e a inversao do relacionamento
        context = context and context.copy() or {}
        result = super(adm_pessoa_relacionamento, self).create(cr, uid, vals, context)

        _dep_id = vals.get('dependente_id')
        _pes_id = vals.get('pessoa_id')
        _rel_id = vals.get('relacionamento_tipo_id')
        _set_id = vals.get('admprel_setor_ids')
        brw_relacionamento = self.pool.get('adm.relacionamento.tipo').browse(cr, uid, _rel_id,
                                                                             context=context)
        if brw_relacionamento.relacionamento_inverso_id:
            _rel_id = brw_relacionamento.relacionamento_inverso_id.id
        # invertendo dependente, pessoa e relacionamento
        vals['dependente_id'] = _pes_id
        vals['pessoa_id'] = _dep_id
        vals['relacionamento_tipo_id'] = _rel_id
        vals['admprel_setor_ids'] = _set_id
        # Gravando o relacionamento invertido
        super(adm_pessoa_relacionamento, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        # Gravar duas vezes o relacionamento normal e a inversao do relacionamento
        context = context and context.copy() or {}
        if not isinstance(ids, (list, tuple)):
            ids = [ids]
        for relacionamento_id in ids:
            dados_relacionamento = self.read(cr, uid, relacionamento_id, context=context)
            result = super(adm_pessoa_relacionamento, self).write(cr, uid, relacionamento_id, vals, context)
            brw_relacionamento = self.browse(cr, uid, relacionamento_id, context=context)
            _dep_id = brw_relacionamento.dependente_id.id
            _pes_id = brw_relacionamento.pessoa_id.id
            _rel_id = brw_relacionamento.relacionamento_tipo_id.id
            brw_relacionamento_tipo = self.pool.get('adm.relacionamento.tipo').browse(cr, uid,
                                                                                      _rel_id,
                                                                              context=context)
            if brw_relacionamento_tipo.relacionamento_inverso_id:
                _rel_id = brw_relacionamento_tipo.relacionamento_inverso_id.id
            # invertendo dependente, pessoa e relacionamento
            vals['dependente_id'] = _pes_id
            vals['pessoa_id'] = _dep_id
            vals['relacionamento_tipo_id'] = _rel_id
            #lendo o relacionamento invertido anterior para atualizar
            _args = [('dependente_id', '=', dados_relacionamento.get('pessoa_id')[0]),
                     ('pessoa_id', '=', dados_relacionamento.get('dependente_id')[0])]
            _dep_anterior_ids = self.search(cr, uid, _args, context=context)
            if _dep_anterior_ids:
                # Gravando o relacionamento invertido
                super(adm_pessoa_relacionamento, self).write(cr, uid, _dep_anterior_ids, vals,
                                                             context)
        return True

    def unlink(self, cr, uid, ids, context=None):
        #excluir duas vezes, o relacionamento normal e a inversao do relacionamento
        context = context and context.copy() or {}
        if not isinstance(ids, (list, tuple)):
            ids = [ids]
        _lista_ids = [] # nova lista de ids com os ids a excluir e suas inversoes
        for relacionamento_id in ids:
            _lista_ids.append(relacionamento_id)
            dados_relacionamento = self.read(cr, uid, relacionamento_id, context=context)
            #lendo o relacionamento invertido anterior para atualizar
            _args = [('dependente_id', '=', dados_relacionamento.get('pessoa_id')[0]),
                     ('pessoa_id', '=', dados_relacionamento.get('dependente_id')[0])]
            _dep_anterior_ids = self.search(cr, uid, _args, context=context)
            if _dep_anterior_ids:
                _lista_ids.append(_dep_anterior_ids[0])
        res = super(adm_pessoa_relacionamento, self).unlink(cr, uid, _lista_ids, context=context)
        return res
