# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv
import re
import logging
from openerp.tools.translate import _


class adm_pessoa_telefone(osv.Model):
    _name = 'adm.pessoa.telefone'
    _inherit = 'adm.pessoa.telefone'
    _logger = logging.getLogger(_name)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['numero'], context=context)
        res = []
        for record in reads:
            if 'numero' in record:
                name = record.get('numero')
                res.append((record['id'], name))
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Retorna os dados de um telefone
        o Numero retorna formatado. Formato 0000-0000
        """
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        telefones = super(adm_pessoa_telefone, self).read(cr, uid, ids, fields=fields, context=context, load=load)
        res = []
        for telefone in telefones:
            if telefone.get('numero'):
                _telefone = telefone['numero']
                val = re.sub('[^0-9]', '', _telefone)
                if len(val) == 8:
                    telefone['numero'] = "%s-%s" % (val[:4], val[4:])
                else:
                    telefone['numero'] = val
            res.append(telefone)
        return res

    def create(self, cr, uid, vals, context=None):
        """
        Metodo para insercao do Telefone.
        Antes de inserir, retira todos caracteres nao numericos que foram informados
        no numero, no ddd, e no ddi
        """
        context = context and context.copy() or {}
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        # _usuarioRemoto = False
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            # self._logger.warn('Usuario remoto criando Telefone:' + str(vals))
            return True # Usuario remoto não atualiza mais
        if vals.get('numero') and vals['numero']:
            vals['numero'] = re.sub('[^0-9]', '', vals['numero'])
        if vals.get('ddi') and vals['ddi']:
            vals['ddi'] = re.sub('[^0-9]', '', vals['ddi'])
        if vals.get('ddd') and vals['ddd']:
            vals['ddd'] = re.sub('[^0-9]', '', vals['ddd'])
        _pessoa_id = vals.get('pessoa_id', False)
        # if _usuarioRemoto:
        #     _args = [('pessoa_id', '=', _pessoa_id),
        #              ('telefone_tipo_id', '=', vals.get('telefone_tipo_id')),
        #              ('numero', '=', vals.get('numero'))]
        #     if vals.get('ddd'):
        #         _args.append(('ddd', '=', vals.get('ddd')))
        #     result = self.search(cr, uid, _args)
        #     if result:
        #         return result
        result = super(adm_pessoa_telefone, self).create(cr, uid, vals, context)
        if _pessoa_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoa_id, {}, context=context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        """
        Metodo para alteracao do Telefone.
        Antes de alterar, retira todos caracteres nao numericos que foram informados
        no numero, no ddd, e no ddi
        """
        context = context and context.copy() or {}
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            # _usuarioRemoto = True
            # self._logger.info('===> Usuario remoto criando Telefone:' + str(vals))
            return True # Usuario remoto não atualiza mais
        if vals.get('numero') and vals['numero']:
            vals['numero'] = re.sub('[^0-9]', '', vals['numero'])
        if vals.get('ddi') and vals['ddi']:
            vals['ddi'] = re.sub('[^0-9]', '', vals['ddi'])
        if vals.get('ddd') and vals['ddd']:
            vals['ddd'] = re.sub('[^0-9]', '', vals['ddd'])
        result = super(adm_pessoa_telefone, self).write(cr, uid, ids, vals, context)
        _pessoa_id = vals.get('pessoa_id', False)
        if _pessoa_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoa_id, {}, context=context)
        return result

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        _pessoas_id = []
        for x in self.read(cr, uid, ids, ['pessoa_id'], context=context):
            _pessoas_id.append(x['pessoa_id'][0])
        result = super(adm_pessoa_telefone, self).unlink(cr, uid, ids, context=context)
        if _pessoas_id:
            self.pool.get('adm.pessoa').write(cr, uid, _pessoas_id, {}, context=context)
        return result

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        res = super(adm_pessoa_telefone, self).search(cr, uid, args, offset, limit, order, context, count)
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if brw_res_users and brw_res_users.login and brw_res_users.login == 'remoto':
            self._logger.info(_('===> Usuario remoto pesquisando Telefone:') + str(args))
        return res