# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class adm_pessoa_endereco(osv.Model):
    """
    Tabela de enderecos das pessoas.
    Obs.: o endereco nao pode ser excluido se tiver documentos com ele relacionado, neste caso, deve ser inativado
    """
    # prefixo = admpen_

    from funcoes import _admpen_get_name, _admpen_check_cep, _admpen_get_pais, _admpen_get_uf, _admpen_get_cidade

    _name = 'adm.pessoa.endereco'
    _columns = {
        'name': fields.function(_admpen_get_name, method=True, string='Endereco', type='text', store=False),
        'pessoa_id': fields.many2one("adm.pessoa", 'Pessoa', ondelete="cascade", required=True, select=True),
        'regiao_id': fields.many2one("adm.regiao", 'Regiao', ondelete="restrict", select=True),
        'endereco_tipo_id': fields.many2one("adm.endereco.tipo", 'Tipo de endereco', ondelete="restrict", select=True,
                                            required=True), # id do tipo de endereco
        'pais_id': fields.many2one('adm.pais', "Pais", required=True, ondelete="restrict", select=True), # id do pais
        'uf_id': fields.many2one('adm.uf', "Estado", ondelete="restrict", select=True), # id do estado(UF)
        'cidade_id': fields.many2one('adm.cidade', "Cidade", ondelete="restrict", select=True), # id da cidade
        'bairro_id': fields.many2one('adm.bairro', "Bairro", ondelete="restrict", select=True), # id do bairro
        'logradouro_id': fields.many2one('adm.logradouro', "Logradouro", ondelete="restrict", select=True),
        'numero': fields.char('Numero', size=5), # Numero do imovel
        'complemento': fields.char('Complemento', size=20), # Complemento do endereco. Casa, apto, sala, etc
        'cep': fields.char('CEP', size=10), # Cep do imovel da pessoa
        'observacao': fields.text('Observacao'), # Codigo de Identificacao da pessoa
        'active': fields.boolean('Ativo'), # indica se o endereco esta ativo.
    }
    _order = 'pessoa_id, pais_id, uf_id, cidade_id, bairro_id, logradouro_id'
    _defaults = {
        'active': lambda *a: True,
        'endereco_tipo_id': lambda *a: 1,
        'pais_id': _admpen_get_pais,
        'uf_id': _admpen_get_uf,
        'cidade_id': _admpen_get_cidade,
    }

    _constraints = [(_admpen_check_cep, 'Erro: CEP invalido', ['cep'])]
