# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import re
from openerp import pooler
from SigUtils.cep import Cep

# Funcoes da classe adm_pessoa_categoria_nome
def _admpcn_name_get_fnc(self, cr, uid, ids, prop, unknow_none, context=None):
    """
    Metodo para retornar a estrutura da arvore das categorias
    """
    context = context and context.copy() or {}
    res = self.name_get(cr, uid, ids, context=context)
    return dict(res)


# Funcoes da classe adm_pessoa_documento_complemento
def _admdcc_get_dados(self, cr, uid, ids, name, arg, context=None):
    res = {}
    context = context and context.copy() or {}
    if ids:
        browse_documento_complemento = self.browse(cr, uid, ids)
        for x in browse_documento_complemento:
            documento = x.documento_id
            vals = {
                'admdcc_tipo_id': documento.tipo_id.id,
                'admdcc_numero': documento.numero,
                'admdcc_emitente': documento.emitente,
                'admdcc_data_emissao': documento.data_emissao,
                'admdcc_pessoa_id': documento.pessoa_id.name
            }
            res[x.id] = vals
    return res


# Funcoes da classe adm_pessoa_email
def _admpem_check_email(self, cr, uid, ids):
    for mail in self.browse(cr, uid, ids):
        if not self._valida_email(mail.name):
            return False
    return True


# Funcoes da classe adm_pessoa_endereco
def _admpen_get_name(self, cr, uid, ids, name, arg, context=None):
    """
    Metodo para recuperacao do endereco no seguinte formato:
    Titularidade endereco - numero - bairro - cidade - uf
    Ex.: Rua Margarida Cortona - 684 - Cannan - Sete Lagoas - MG
    """
    context = context and context.copy() or {}
    res = {}
    for endereco in self.browse(cr, uid, ids):
        if endereco.id:
            endereco_con = ''
            try:
                if endereco.logradouro_id:
                    if endereco.logradouro_id.titulo:
                        endereco_con = endereco_con + endereco.logradouro_id.titulo + ' '
                    if endereco.logradouro_id.descricao:
                        endereco_con = endereco_con + endereco.logradouro_id.descricao
            except:
                pass
            try:
                if endereco.numero:
                    endereco_con = endereco_con + ', ' + endereco.numero
            except:
                pass
            try:
                if endereco.bairro_id.name:
                    endereco_con = endereco_con + ' - ' + endereco.bairro_id.name
            except:
                pass
            try:
                if endereco.cidade_id.name:
                    endereco_con = endereco_con + ' - ' + endereco.cidade_id.name
            except:
                pass
            try:
                if endereco.cidade_id.uf_id.sigla:
                    endereco_con = endereco_con + '/' + endereco.cidade_id.uf_id.sigla
            except:
                res[endereco.id] = endereco_con
            res[endereco.id] = endereco_con
    return res


def _admpen_check_cep(self, cr, uid, ids, context=None):
    """
    Metodo que verificar a validade do cep
    """
    context = context and context.copy() or {}
    for individuo in pooler.get_pool(cr.dbname).get('adm.pessoa.endereco').read(cr, uid, ids, ['pessoa_id', 'cep'], context=context):
        nro_cep = individuo['cep']
        if nro_cep:
            nr = Cep(nro_cep)
            return nr.validar()
    return True


def _admpen_get_pais(self, cr, uid, context=None):
    context = context and context.copy() or {}
    if context.get('cadastro_pais'):
        if context.get('pais'):
            sigla = context.get('pais')
            if sigla:
                pais_id = self.pool.get('adm.pais').search(cr, uid, [('sigla', '=', sigla)], context=context)
                if pais_id:
                    return pais_id[0]
    elif context.get('cadastro_estado') or context.get('cadastro_cidade'):
        if context.get('pais'):
            pais = context.get('pais')
            return pais
    return False


def _admpen_get_uf(self, cr, uid, context=None):
    context = context and context.copy() or {}
    if context.get('cadastro_cidade'):
        estado = context.get('estado')
        return estado
    else:
        if context.get('estado'):
            sigla = context.get('estado')
            if sigla:
                uf_id = self.pool.get('adm.uf').search(cr, uid, [('sigla', '=', sigla)], context=context)
                if uf_id:
                    return uf_id[0]
    return False


def _admpen_get_cidade(self, cr, uid, context=None):
    context = context and context.copy() or {}
    if context.get('cadastro_cidade'):
        if context.get('cidade'):
            cidade = context.get('cidade')
            if cidade:
                cidade_id = self.pool.get('adm.cidade').search(cr, uid, [('name', '=', cidade)], context=context)
                if cidade_id:
                    return cidade_id[0]
    return False


# Funcoes da classe adm_logradouro
def _admlgd_nome_rua(self, cursor, user, ids, name, arg, context=None):
    """
    Metodo para retornar a titularidade com o nome da rua
    Ex.: Rua Margarida..., AV Marechal....
    """
    context = context and context.copy() or {}
    res = {}
    for endereco in self.browse(cursor, user, ids):
        if endereco.id:
            res[endereco.id] = endereco.titulo + ' ' + endereco.descricao
        else:
            res[endereco.id] = False
    return res


# Funcoes da classe adm_regiao
def _admreg_verifica_recursao(self, cr, uid, ids):
    """
    Metodo que impede a insercao de regiao com recursividade
    """
    level = 100
    while len(ids):
        cr.execute('select distinct parent_id from adm_regiao where id in (' + ','.join(map(str, ids)) + ')')
        ids = filter(None, map(lambda x: x[0], cr.fetchall()))
        if not level:
            return False
        level -= 1
    return True


# Funcoes da classe adm_pessoa
def _admpes_get_aniversario(self, cr, uid, ids, name, arg, context=None):
    """
    Retorna o mes e o dia do aniversário da pessoa
    """
    context = context and context.copy() or {}
    res = {}
    for _pessoa in self.browse(cr, uid, ids):
        if _pessoa.pessoa_tipo == 'J' or not _pessoa.admpes_fis_data_nascimento:
            res[_pessoa.id] = 0
        else:
            res[_pessoa.id] = int(_pessoa.admpes_fis_data_nascimento[5:7]+_pessoa.admpes_fis_data_nascimento[8:10])
    return res


def admpes_get_cpf_cnpj_obrigatorio(self, cr, uid, context=None):
    context = context and context.copy() or {}
    cpf_cnpj_obrigatorio = False
    obj_parametro = self.pool.get('adm.parametro')
    adm_parametro_id = obj_parametro.search(cr, uid, [], context=context)
    cpf_cnpj_obrigatorio = False
    if adm_parametro_id:
        obj_parametro_cpf = obj_parametro.read(cr, uid, adm_parametro_id[0],
                                               ['admpar_cpf_cnpj_obrigatorio'], context=context)
        cpf_cnpj_obrigatorio = obj_parametro_cpf.get('admpar_cpf_cnpj_obrigatorio', False)
    return cpf_cnpj_obrigatorio

def _admpes_get_cpf_cnpj_obrigatorio(self, cr, uid, ids, name, arg, context=None):
    """
    Retorna o parametro de obrigatoriedade de cpn_cnpj do tabela parametro_financeiro
    """
    context = context and context.copy() or {}
    res = {}
    _cpf_cnpj_obrigatorio = admpes_get_cpf_cnpj_obrigatorio(self, cr, uid, context=context)
    for _pessoa_id in ids:
        res[_pessoa_id] = _cpf_cnpj_obrigatorio
    return res


def _admpes_get_email(self, cursor, user, ids, name, arg, context=None):
    """
    Retorna o email da pessoa
    Pesquisa pelo primeiro email to tipo "comum", caso nao ache,
    retorna qualquer outro que exista
    """
    context = context and context.copy() or {}
    res = {}
    for _id in ids:
        res[_id] = ''
        obj_emails = pooler.get_pool(cursor.dbname).get('adm.pessoa.email')
        emails = obj_emails.search(cursor, user, [('pessoa_id', '=', _id)], context=context)
        for email in obj_emails.browse(cursor, user, emails):
            if email.email_tipo_id.name == 'Comum':
                res[_id] = email.name
                break
            else:
                res[_id] = email.name
    return res


def _admpes_get_telefone(self, cursor, user, ids, name, arg, context=None):
    """
    Retorna o telefone.
    Para pessoa Fisica, retorna o Residencial e Comercial
    caso seja pessoa juridica.
    Nao encontrando o telefone esperado, retorna o primeiro que encontrar.
    """
    context = context and context.copy() or {}
    res = {}
    ssql = """select a.ddd, a.numero
                    from adm_pessoa_telefone a,
                         adm_telefone_tipo b
                    where a.telefone_tipo_id = b.id
                    and   a.pessoa_id =%s
                    order by (CASE b.name WHEN 'Comercial' THEN 1
                          WHEN 'Residencial' THEN 2
                          WHEN 'Celular' THEN 3
                          WHEN 'Fax' THEN 4
                          ELSE 5
                              END)
                    limit 1"""
    for _id in ids:
        res[_id] = ''
        cursor.execute(ssql % str(_id))
        cres = cursor.fetchone()
        if cres:
            _ddd = cres[0] or ''
            _fone = cres[1] or ''
            _ddd = re.sub('[^0-9]', '', _ddd)
            _fone = re.sub('[^0-9]', '', _fone)
            if _ddd:
                _ddd = '(' + str(_ddd) + ')'
            if len(_fone) == 8:
                _fone = "%s-%s" % (_fone[:4], _fone[4:])
            res[_id] = _ddd + _fone
    return res


def _admpes_get_soundex(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    vals = self.read(cr, uid, ids, ['name', 'fantasia_apelido'], context=context)
    for x in vals:
        nome = x['name']
        if x['fantasia_apelido']:
            nome += (' ' + x['fantasia_apelido'])
        res[x['id']] = self.pool.get('adm.utils').get_fonema(nome)
    return res

def _admpes_get_pessoa_ids(self, cr, uid, ids, context=None):
    '''Retorna os ids da classe adm.pessoa que tiveram o campo endereco_ids alterado.'''
    context = context and context.copy() or {}
    return ids

def _admpes_get_pessoa_ids_pelo_endereco(self, cr, uid, ids, context=None):
    '''
    Retorna os ids de 'adm.pessoa' que estao nos enderecos('adm.pessoa.endereco') alterados
    self eh da classe 'adm.pessoa.endereco'
    '''
    context = context and context.copy() or {}
    res = map(lambda x: x.get('pessoa_id'),
              self.read(cr, uid, ids, context=context, load='_classic_write'))

    return res

def _admpes_get_dados_endereco(self, cr, uid, ids, name, arg, context=None):
    """
    Metodo utilizado para invocar o metodo _get_base_endereco retornando todos os elementos 
    do endereco pois nao passou o parametro 'tipo'
    """
    context = context and context.copy() or {}
    res = self._get_base_endereco(cr, uid, ids, name, arg, context=context)
    return res

def _admpes_get_estado(self, cr, uid, ids, name, arg, context=None):
    """
    Metodo utilizado para invocar o metodo _get_base_endereco informando o tipo como "estado"
    """
    context = context and context.copy() or {}
    res = {}
    for id in ids:
        res[id] = ''
        obj_ufs = pooler.get_pool(cr.dbname).get('adm.pessoa.endereco')
        ufs = obj_ufs.search(cr, uid, [('pessoa_id', '=', id)], context=context)
        uf_name = obj_ufs.read(cr, uid, ufs, ['uf_id'], context=context)
        if uf_name:
            b = uf_name[0]['uf_id']
            if not b == False:
                res[id] = uf_name[0]['uf_id'][1]
    res = self._get_base_endereco(cr, uid, ids, res, arg, context=context, tipo='estado')
    return res

def _admpes_check_cpf_cnpj(self, cr, uid, ids, context=None):
    """
    Metodo para validar o CPF/CNPJ
    """
    context = context and context.copy() or {}
    obj_adm_pessoa = self.pool.get('adm.pessoa')
    adm_pessoa_dados = obj_adm_pessoa.read(cr, uid, ids, ['cpf_cnpj', 'pessoa_tipo'], context=context)
    for individuo in adm_pessoa_dados:
        nro_cpf_cnpj = individuo['cpf_cnpj']
        if nro_cpf_cnpj:
            pessoa_tipo = individuo['pessoa_tipo']
            res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, nro_cpf_cnpj, True)
            if res:
                pessoa_ids = obj_adm_pessoa.search(cr, uid, [('cpf_cnpj', '=', res),
                                                             ('id', '!=', individuo.get('id'))], context=context)
                if pessoa_ids:
                    return False
                str_cpf_cnpj = res.replace('/', '').replace('.', '').replace('-', '')
                if len(str_cpf_cnpj) == 14:
                    pessoa_tipo = 'F'
                    str_cpf_cnpj = str_cpf_cnpj[3:]
                else:
                    pessoa_tipo = 'J'
                    str_cpf_cnpj = '000' + str_cpf_cnpj
                res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, str_cpf_cnpj,
                                                                        True)
                if res:
                    pessoa_ids = obj_adm_pessoa.search(cr, uid, [('cpf_cnpj', '=', res),
                                                                 ('id', '!=', individuo.get('id'))], context=context)
                    if pessoa_ids:
                        return False
                return True
            else:
                return False
    return True


# funcoes da classe adm_pessoa_relacionamento
def _admpre_nome_pessoa(self, cursor, user, ids, name, arg, context=None):
    """
    Retorna o nome da pessoa
    """
    context = context and context.copy() or {}
    res = {}
    for pessoa in self.browse(cursor, user, ids):
        if pessoa.id:
            res[pessoa.id] = pessoa.dependente_id.name
        else:
            res[pessoa.id] = False
    return res


# funcoes da classe adm_uf
def _admpuf_get_pais(self, cr, uid, context=None):
    context = context and context.copy() or {}
    if context.get('pais'):
        sigla = context.get('pais')
        if sigla:
            pais_id = self.pool.get('adm.pais').search(cr, uid, [('sigla', '=', sigla)], context=context)
            if pais_id:
                return pais_id[0]
    return False


#funcoes da classe adm_cidade
def _admpci_get_uf(self, cr, uid, context=None):
    context = context and context.copy() or {}
    if context.get('estado'):
        sigla = context.get('estado')
        if sigla:
            uf_id = self.pool.get('adm.uf').search(cr, uid, [('sigla', '=', sigla)], context=context)
            if uf_id:
                return uf_id[0]
    return False


def _admpci_get_pais(self, cr, uid, context=None):
    context = context and context.copy() or {}
    if context.get('pais'):
        pais = context.get('pais')
        return pais
    return False

def _admpes_get_pessoa_tipo(self, cr, uid, context=None):
    res = 'F'
    context = context and context.copy() or {}
    if not 'pessoa_tipo' in context.keys():
        return res
    if context.get('pessoa_tipo') == 'banco':
        res =  'J'
    return res

def _admpes_get_admpes_jur_tipo_empresa(self, cr, uid, context=None):
    res = ''
    context = context and context.copy() or {}
    if not 'pessoa_tipo' in context.keys():
        return res
    if context.get('pessoa_tipo') == 'banco':
        res =  'B'
    return res
