# -*- encoding: utf-8 -*-

##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class adm_pessoa(osv.Model):
    # prefixo = admpes_

    from funcoes import _admpes_get_email, _admpes_get_telefone, _admpes_get_soundex, \
                        _admpes_get_dados_endereco, _admpes_get_pessoa_ids_pelo_endereco, \
                        _admpes_check_cpf_cnpj, _admpes_get_aniversario, \
                        _admpes_get_cpf_cnpj_obrigatorio, admpes_get_cpf_cnpj_obrigatorio, \
                        _admpes_get_pessoa_tipo, _admpes_get_admpes_jur_tipo_empresa, \
                        _admpes_get_pessoa_ids

    _name = 'adm.pessoa'
    _inherits = {
        'res.partner': 'partner_id',
    }

    _columns = {
        'partner_id': fields.many2one("res.partner", 'Partner', ondelete="restrict", required=True, select=True),
        'id': fields.integer(),
        'admpes_id_externo': fields.char("ID externo", size=14, select=True, oldname='admpes_ml_id'),
        'nome': fields.related('partner_id', 'name', type='char', string='Nome', readonly=True, store=True),
        'fantasia_apelido': fields.char('Fantasia/Apelido', size=30),
        'pessoa_tipo': fields.selection([('F', 'Fisica'), ('J', 'Juridica')], 'Tipo Pessoa', required=True, size=1),
        'cpf_cnpj': fields.char('CPF/CNPJ', size=19, select=True), # Numero do Cpf ou CNPJ da pessoa
        'admpes_cpf_cnpj_obrigatorio': fields.function(_admpes_get_cpf_cnpj_obrigatorio,
                                                       method=True, string='Cpf/Cnpj obrigatorio',
                                                       type='boolean', store=False),
        'codigo': fields.integer('Codigo', select=True, readonly=True,
                                 help="Codigo para identificacao da pessoa. \nSe deixar zerado, o sistema atribui um valor para ele. \nEste codigo tambem precisa ser unico!"),
        'active': fields.boolean('Ativo'), # Indica que o cadastro da pessoa esta ativo (True/False)
        'controle': fields.char('controle', size=15), # Desconsiderar
        'admpes_jur_data_fundacao': fields.date('Data fundacao', oldname="jur_data_fundacao"), # Data da fundacao da empresa juridica
        'admpes_jur_tipo_empresa': fields.selection(
            [('I', 'Individual'), ('L', 'Ltda'), ('S', 'Sociedade Anonima'), ('C', 'Cooperativa'),
             ('M', 'Micro empreendedor'),
             ('U', 'Autarquia'), ('P', 'Administracao Publica'), ('A', 'Administradora de cartoes'), ('B', 'Banco')],
            'Tipo empresa', select=True, size=1, oldname="jur_tipo_empresa"),
        'admpes_jur_suframa': fields.char('Suframa', size=9, oldname="jur_suframa"), # codigo da empresa na suframa
        'admpes_jur_regime_tributario': fields.selection(
            [('R', 'Lucro Real'), ('P', 'Lucro Presumido'), ('S', 'Simples Nacional')],
            'Regime tributario', size=1, oldname="jur_regime_tributario"), # Regime Tributario\nR=Lucro Real,\nP=Lucro Presumido,\nS=Simples Nacional
        'admpes_jur_tipo_atividade': fields.selection(
            [('A','Atacadista'),('I', 'Industria ou equiparado'), ('C', 'Comercio'), ('T', 'Transportadora'),
             ('S', 'Prestador de Servico'), ('P', 'Posto de combustivel'), ('M', 'Imobiliaria'), ('F', 'Financeira')],
            'Natureza da atividade', help='Atividade preponderante exercida pela pessoa juridica',
            size=1, oldname="jur_tipo_atividade"),
        'admpes_jur_ie': fields.char('Inscricao Estadual', size=14, oldname="jur_ie"),
        'admpes_jur_cnae_id': fields.many2one("adm.cnae", 'Cnae', ondelete="restrict", oldname="jur_cnae_id",
                                              select=True), # Id do cnae da empresa
        'foto': fields.binary('Foto', filters='*.png,*.gif,*.jpeg'), #Foto da pessoa
        'admpes_fis_sexo': fields.selection([('M', 'Masculino'), ('F', 'Feminino')], 'Sexo', size=1,
                                            oldname="fis_sexo"),
        'admpes_fis_estado_civil': fields.selection([('C', 'Casado'), ('S', 'Solteiro'), ('V', 'Viuvo'),
                                              ('D', 'Divorciado'), ('E', 'Desquitado')],
                                        'Estado civil', size=1, oldname="fis_estado_civil"),
        'admpes_fis_data_nascimento': fields.date('Data de nascimento', oldname="fis_data_nascimento"), # Data de nascimento
        'admpes_fis_aniversario': fields.function(_admpes_get_aniversario, method=True, string='Aniversario',
                                                  type='integer', store=True),
        'admpes_fis_produtor_rural': fields.boolean('Produtor rural', oldname="fis_produtor_rural"), # Identifica se a pessoa e produtor rural
        'admpes_fis_rg': fields.char('RG', size=14, select=True, oldname="fis_rg"),
        'email': fields.function(_admpes_get_email, method=True, string='Email', type='char', size=50, store=True),
        'telefone': fields.function(_admpes_get_telefone, method=True, string='Telefone', type='char', size=25,
                                    store=True), # O Telefone de uma pessoa (Um telefone apenas)
        'email_homepage_ids': fields.one2many('adm.pessoa.homepage', 'pessoa_id', 'Homepages'),
        'email_ids': fields.one2many('adm.pessoa.email', 'pessoa_id', 'Emails'), # Relacao de emails de uma pessoa
        'relacionamento_ids': fields.one2many('adm.pessoa.relacionamento', 'pessoa_id', 'Dependentes/Contatos'),
        'endereco_ids': fields.one2many('adm.pessoa.endereco', 'pessoa_id', 'Enderecos'),
        'telefones_ids': fields.one2many('adm.pessoa.telefone', 'pessoa_id', 'Telefones'),
        'categoria_ids': fields.many2many('adm.pessoa.categoria.nome', 'pessoa_categoria', 'pessoa_id',
                                          'categoria_id', 'Categorias'),
        'documentos_ids': fields.one2many('adm.pessoa.documento.dados', 'pessoa_id', 'Documentos'),
        'user_ids': fields.one2many('res.users', 'pessoa_id', 'Usuario'),
        'soundex': fields.function(_admpes_get_soundex, method=True, string='Soundex', type='char', size=80,
                                   store=True), # O email da pessoa (um email apenas)
        'observacoes': fields.text('Informacoes complementares'), # Informacoes adicionais,
        'lang': fields.char('Lang', size=5), #este campo foi adicionado para que seja tratado o relatorio do tipo rml pois no partner existe o campo lang
        'address': fields.char('Lang', size=50), #este campo foi adicionado para que seja tratado nos modulos do openerp.
        'endereco': fields.function(_admpes_get_dados_endereco, method=True, type="char", size=90,
                                    string='Endereco', multi='dados_endereco',
                                    store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco,
                                                                   ['logradouro_id', 'numero', 'complemento'], 10),
                                           'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}), # Endereco da pessoa
        'bairro': fields.function(_admpes_get_dados_endereco, method=True, type="char", size=30,
                                  string='Bairro', multi='dados_endereco',
                                  store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco, ['bairro_id'], 10),
                                         'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}),
        'cidade': fields.function(_admpes_get_dados_endereco, method=True, type="char", size=30,
                                  string='Cidade', multi='dados_endereco',
                                  store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco, ['cidade_id'], 10),
                                         'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}),
        'uf': fields.function(_admpes_get_dados_endereco, method=True, type="char", size=2,
                              string='Estado', multi='dados_endereco',
                              store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco, ['uf_id'], 10),
                                     'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}),
        'pais': fields.function(_admpes_get_dados_endereco, method=True, type="char", size=30,
                                string='Pais', multi='dados_endereco',
                                store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco, ['pais_id'], 10),
                                       'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}),
        'cep': fields.function(_admpes_get_dados_endereco, method=True, type="char", size=11,
                               string='Cep', multi='dados_endereco',
                               store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco, ['uf_id'], 10),
                                      'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}),

        'codigo_ibge': fields.function(_admpes_get_dados_endereco, method=True, type="integer",
                               string='Codigo IBGE', multi='dados_endereco',
                               store={'adm.pessoa.endereco': (_admpes_get_pessoa_ids_pelo_endereco, ['cidade_id'], 10),
                                      'adm.pessoa': (_admpes_get_pessoa_ids, ['endereco_ids', 'endereco_nf_id'], 10)}),
        'create_date': fields.datetime('Data da Inclusao', readonly=True),
        'admpes_fis_cnh_nro': fields.char('Numero da CNH', size=15, select=True),
        'admpes_fis_cnh_categoria': fields.char('Categoria', size=5),
        'admpes_fis_cnh_validade': fields.date('Validade'),
        'admpes_fis_cnh_orgao_emissor': fields.char('Orgao Emissor', size=10),
        'admpes_fis_rg_orgao_emissor': fields.char('Orgao Emissor', size=10),
        'admpes_fis_rg_data_emisssao': fields.date('Emissao'),
        'admpes_ocorrencia_ids': fields.one2many("adm.ocorrencia", 'admoco_pessoa_id', 'Ocorrencias'),
        'admpes_empresa_ids': fields.one2many("res.company", 'rescon_pessoa_id', 'Empresas'),
        'admpes_inscricao_produtor': fields.char('Inscricao produtor', size=14),
        'admpes_tags_ids': fields.many2many('adm.tags', 'adm_pessoas_tags', 'admpes_pessoa_id',
                                            'admpes_tag_id', 'Marcas'),
    }
    _order = 'nome'
    _defaults = {
        'pessoa_tipo': _admpes_get_pessoa_tipo,
        'lang': 'pt_BR',
        'active': lambda *a: True,
        'admpes_cpf_cnpj_obrigatorio': lambda s, cr, uid, c: s.admpes_get_cpf_cnpj_obrigatorio(cr, uid),
        'admpes_jur_tipo_empresa': _admpes_get_admpes_jur_tipo_empresa,

    }

    _constraints = [(_admpes_check_cpf_cnpj, ' CPF/CNPJ Invalido ou ja existe', ['cpf_cnpj'])]
    _sql_constraints = [
        ('codigo_uniq', 'unique (codigo)', 'O Codigo da pessoa deve ser unico !')
    ]

