# -*- encoding: utf-8 -*-

import openerp.addons.web.http as http
import json
from logging import getLogger
from time import time
import jwt
import base64
import xmlrpclib, socket
from unicodedata import normalize
import estruturas

classes = estruturas.classes

_logger = getLogger("API_SIGERP")


class api_sigerp(http.Controller):
    _cp_path = '/apsg'
    sig_base = ''
    sig_user = ''
    sig_uid = 0
    sig_senha = ''
    sig_server = 'http://'+socket.gethostbyaddr(socket.gethostname())[0]+':8069' # Este servidor
    sig_tempo = '0000'
    sig_token = ''
    sig_refresh_token = ''
    sig_code = ''
    sig_frase = ''
    status_api = estruturas.status_api
    sig_gerar_log = False

    # Métodos da classe
    ## Métodos internos da classe ##########################################

    def __init__(self):
        pass

    def _api_gerar_token(self):
        """
        Método para gerar o token e o refresh token
        :param uid: integer, id do usuario
        :param pwd: char, senha do usuário
        :param base: char, Base de dados para logar
        :param tempo: integer, tempo de validade do token
        :return: lista, contendo:
                    - 'token', Token gerado
                    - 'refresh_token', refresh token gerado
        """
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_gerar_token - Inicio')
        dt_now = int(time())
        header = {
            "alg": 'HS256',
            "typ": 'JWT'
        }
        payload = {
            "sub": self.sig_uid,
            "iss": "SigERP",
            "exp": dt_now + int(self.sig_tempo),
            "base": self.sig_base,
            "pwd": self.sig_senha,
        }
        self.sig_token = jwt.encode(payload, self.sig_frase, algorithm='HS256', headers=header)
        payload['tempo'] = (dt_now + 864000) # agora + 10 dias
        payload['iss'] = 'SigERP_refresh'
        self.sig_refresh_token = jwt.encode(payload, self.sig_frase, algorithm='HS256', headers=header)
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_gerar_token - Saida Token: %s Refrech_token: %s' % (self.sig_token, self.sig_refresh_token))
        return self.sig_token, self.sig_refresh_token

    def _api_acao_teste(self):
        """
        método para teste de acao, não faz nada só retorna a estrutura de retorno
        :param dados:
        :param uid:
        :param pwd:
        :param base:
        :param server:
        :return: dict, contendo:
                - cod: codigo do status
                ' message: mensagem de status qdo tiver
        """
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        return res

    def _api_ler_classe(self, registro, args, fields=[], offset=0, limit=10):
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_ler_classe - Inicio - registro: %s, args: %s, fields: %s, offset: %s limit: %s' % (registro, args, fields, offset, limit))
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        socket.setdefaulttimeout(500)
        sock = xmlrpclib.ServerProxy(self.sig_server + '/xmlrpc/object')
        tab, mod = classes.get(registro.split('.')[1])
        result = self._api_ver_modulo_instalado(mod)
        if not result:
            if self.sig_gerar_log:
                _logger.error('Funcao: _api_ler_classe - Saida cod: 10 message: %s ' % self.status_api.get('10'))
            return {'cod': '10', 'message': self.status_api.get('10') % mod,  'retorno_acao': []}
        try:
            if isinstance(args, int) or (args and isinstance(args, list) and isinstance(args[0], int)):
                if isinstance(args, int):
                    args = [args]
                res['retorno_acao'] = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, tab, 'read', args, fields, {})
            else:
                ids = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, tab, 'search', args, offset, limit)
                if ids:
                    res['retorno_acao'] = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, tab, 'read', ids, fields, {})
                else:
                    if self.sig_gerar_log:
                        _logger.error(
                            'Funcao: _api_ler_classe - Saida cod: 08, message: %s ' % self.status_api.get('08'))
                    res = {'cod': '08', 'message': self.status_api['08'], 'retorno_acao': []}
        except Exception as e:
            mens = e
            if str(type(e)) == "<class 'xmlrpclib.Fault'>":
                mens = normalize('NFKD', e.faultCode).encode('ASCII', 'ignore')
            res = {'cod': '07', 'message': self.status_api['07'] + '\n' + str(mens), 'retorno_acao': []}
        if self.sig_gerar_log:
            _logger.info('Funcao: _api_ler_classe - Saida - %s, ' % str(res))
        return res

    def _api_criar_classe(self, registro, dados):
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_criar_classe - Inicio - registro: %s, dados: %s' % (registro, dados))
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        socket.setdefaulttimeout(500)
        sock = xmlrpclib.ServerProxy(self.sig_server + '/xmlrpc/object')
        tab, mod = classes.get(registro.split('.')[1])
        result = self._api_ver_modulo_instalado(mod)
        if not result:
            if self.sig_gerar_log:
                _logger.error('Funcao: _api_ler_classe - Saida cod: 10 message: %s ' % self.status_api.get('10'))
            return {'cod': '10', 'message': self.status_api.get('10') % mod,  'retorno_acao': []}
        try:
            ret = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, tab, 'create', dados, {})
            if tab == 'estoquepedidos.pedido.item':
                ped_id = dados.get('pedido_id')
                ret_capa = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, 'estoquepedidos.pedido.capa', 'write', [ped_id], {}, {})
            if ret:
                res['retorno_acao'] = {'id': ret}
            else:
                res = {'cod': '12', 'message': self.status_api['12'], 'retorno_acao': []}
        except Exception as e:
            mens = e
            if str(type(e)) == "<class 'xmlrpclib.Fault'>":
                if isinstance(e.faultCode, str):
                    mens = e.faultCode
                else:
                    mens = normalize('NFKD', e.faultCode).encode('ASCII', 'ignore')
            _logger.info('=== Erro ao acessar o SigERP %s' % str(mens))
            res = {'cod': '07', 'message': self.status_api['07'] + '\n' + str(mens), 'retorno_acao': []}
            if self.sig_gerar_log:
                _logger.info('Funcao: _api_criar_classe - Saida - %s, ' % str(res))
            return res
        return res

    def _api_alterar_classe(self, registro, dados):
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_alterar_classe - Inicio - registro: %s, dados: %s' % (registro, dados))
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        socket.setdefaulttimeout(500)
        sock = xmlrpclib.ServerProxy(self.sig_server + '/xmlrpc/object')
        tab, mod = classes.get(registro.split('.')[1])
        result = self._api_ver_modulo_instalado(mod)
        _id = dados.get('id')
        if not result:
            if self.sig_gerar_log:
                _logger.error('Funcao: _api_alterar_classe - Saida cod: 10 message: %s ' % self.status_api.get('10'))
            return {'cod': '10', 'message': self.status_api.get('10') % mod,  'retorno_acao': []}
        try:
            ret = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, tab, _id, 'write', dados, {})
            if ret:
                res['retorno_acao'] = {'id': ret}
            else:
                res = {'cod': '14', 'message': self.status_api['14'], 'retorno_acao': []}
        except Exception as e:
            mens = e
            if str(type(e)) == "<class 'xmlrpclib.Fault'>":
                mens = normalize('NFKD', e.faultCode).encode('ASCII', 'ignore')
            _logger.info('=== Erro ao acessar o SigERP %s' % str(mens))
            res = {'cod': '07', 'message': self.status_api['07'] + '\n' + str(mens), 'retorno_acao': []}
            if self.sig_gerar_log:
                _logger.info('Funcao: _api_alterar_classe - Saida - %s, ' % str(res))
            return res
        return res

    def _api_ver_modulo_instalado(self, modulo):
        socket.setdefaulttimeout(500)
        sock = xmlrpclib.ServerProxy(self.sig_server + '/xmlrpc/object')
        try:
            res = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, 'adm.utils', 'admutl_check_modulo_instalado', modulo, {})
        except Exception as e:
            print e
            res = False
        return res

    def _api_validar_fields(self, tabela, fields):
        """
        Método para verificar se o campo recebido existe na tabela e trocar o nome externo do campo pelo nome interno
        :param tabela:  string, tabela para verificar
        :param fields: list, lista de campos para verificar
        :return: list, lista de campos que constam no dicionario da tabela
        """
        _new_fields = []
        for linha in eval(tabela):
            if not fields: # Incluir todos os campos da tabela
                _new_fields.append(linha[1])
            else: # verifica se existe na tabela para incluir
                for field in fields:
                    if isinstance(field, dict):
                        x = field.keys()[0]
                        if x == linha[0]:
                            tb_filha = 'estruturas.%s' % linha[4].strip()
                            filhos = self._api_validar_fields(tb_filha, field.get(x))
                            for campo in filhos:
                                f = '%s.%s' % (linha[1], campo)
                                _new_fields.append(f)
                            _new_fields.append(linha[1]) # Acrecentar o proprio  ids
                            break
                    if field == linha[0]:
                        _new_fields.append(linha[1])
                        x = linha[1].split('.')
                        if len(x) > 1:
                            _new_fields.append(x[0])
                        break
        return _new_fields

    def _api_ajustar_tipo_campo(self, nome_do_dado, dado, tipo, tamanho, sentido):
        """
        Método para validar o tipo de campo, os campos booleanos precisam ser validados para entrada e saida
        :param nome_do_dado: char, nome do campo
        :param dado: char, conteudo do campo
        :param tipo: tipo de campo
        :param tamanho: int, tamanho do campo
        :param obrigatorio: char, S ou N
        :param sentido: char, 'E=entrando' ou 'S=saindo'
        :return: list, lista contendo o dado a ser retonado e a mensagem que sera vazia se nao houver erro
        """
        res = ''
        mens = ''
        try:
            tamanho = int(tamanho)
        except:
            tamanho = 0
        if tipo == 'string':
            if isinstance(dado, bool):
                res = ''
            elif isinstance(dado, unicode):
                res = normalize('NFKD', dado).encode('ASCII', 'ignore')
                if res > tamanho:
                    res = res[:tamanho]
            elif isinstance(dado, (int, str)):
                res = dado
                if res > tamanho:
                    res = res[:tamanho]
        elif tipo == 'AAAAMMDD':
            if dado:
                if not (len(dado) in (8, 10)):
                    mens = 'Campo: %s invalido' % nome_do_dado
                else:
                    if len(dado) == 10:
                        dado = dado[:4]+dado[5:7]+dado[8:]
                    try:
                        int(dado)
                        res = str(dado)
                    except:
                        mens = 'Campo: %s invalido' % nome_do_dado
            else:
                dado = '00000000'
            if sentido == 'S' and mens:
                dado = '00000000'
                mens = ''
        elif tipo == 'dict':
            res = dado
        elif tipo == 'm2o':
            res = dado
        elif tipo == 'o2m':
            res = dado
        elif tipo == 'm2m':
            res = dado
        elif tipo == 'binary':
            res = dado
        elif tipo == 'float':
            res = dado
        elif tipo == 'integer':
            try:
                res = int(dado)
                res = str(res)
            except:
                res = '0'
        elif isinstance(tipo, (list, tuple)):
            if len(tipo) == 2:
                tipo = (str(tipo[0]), str(tipo[1]))
                if tipo[0] == '0' and tipo[1] == '1': # campo boolean
                    if sentido == 'E': # dados entrando na aplicacao
                        if dado in ('0', '1'):
                            if dado == '0':
                                res = False
                            else:
                                res = True
                        else:
                            mens = 'Campo: %s invalido' % nome_do_dado
                    else: # dados saindo da aplicacao
                        if dado in (True, False):
                            if dado:
                                res = '1'
                            else:
                                res = '0'
                        else:
                            res = '0'
                else: # lista de dois mas nao booleano
                    if not (dado in tipo):
                        mens = 'Campo: %s invalido' % nome_do_dado
                    else:
                        res = dado
                    if sentido == 'S' and mens:
                        res = ''
                        mens = ''
            else:  # lista de mais de dois elementos
                if not (dado in tipo):
                    mens = 'Campo: %s invalido' % nome_do_dado
                else:
                    res = dado
                if sentido == 'S' and mens:
                    res = ''
                    mens = ''

        return res, mens

    def _api_ajustar_retono_campos(self, res, registro, fields, entrando_saindo):
        """
        Metodo para retornar somente os campos pedidos e converter os nomes do campos internos para os externos
        :param res: dict, dicionario contendo o resultado a ser trabalhado
        :param registro: string registro a ser ajustado
        :param fields: list, lista de campos
        :return: dict, dicionario com os campos ajustados
        """
        mensagem_erro = ''
        if res.get('retorno_acao'):
            retorno_acao = []
            for _ret_acao in res.get('retorno_acao'):
                _d = {}
                for linha in eval(registro):
                    for field in fields:
                        if linha[1] == field:
                            if field in _ret_acao.keys():
                                dado = _ret_acao.get(field)
                                dado, mens = self._api_ajustar_tipo_campo(linha[0], dado,linha[3], linha[4], entrando_saindo)
                                if mens:
                                    mensagem_erro += mens +'/n'
                                _d[linha[0]] = dado
                                break
                retorno_acao.append(_d)
            if mensagem_erro:
                res['cod'] = '09'
                res['message'] = self.status_api.get('09')
            res['retorno_acao'] = retorno_acao
        return res

    def _api_acao_post_dados(self, registro, dados):
        """
        Método para Incluir registros
        :param registro: char, contem a tabela a ser trabalhada
        :param dados: dict, dicionario contendo os dados a serem incluidos
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, lista contendo dicionario com os dados da consulta
        """

        if self.sig_gerar_log:
            _logger.error('Funcao: _api_acao_post_dados - Inicio - registro: %s, dados: %s' % (registro, dados))
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        fields = dados.keys()
        _new_dados = {}
        # Montar novo dicionario apenas com os campos válidos
        for field in fields:
            # Retirar campos especiais
            if field in ('id', 'create_date', 'write_date', 'create_uid', 'write_uid'):
                continue
            for linha in eval(registro):
                if linha[5] == 'P': # desconsiderar campos proibidos para alteraçao e inclusao
                    continue
                if linha[0] == field:
                    dado = dados.get(field)
                    dado, x = self._api_ajustar_tipo_campo(linha[0], dado, linha[3], linha[4], 'E')
                    _field_interno = linha[1]
                    _new_dados[_field_interno] = dado
                    if linha[3] == 'm2o' and not isinstance(dados.get(field), int):
                        res_sub = self._api_get_set_m2o(linha[4], dados.get(field))
                        if res_sub.get('cod') == '00' and res_sub.get('retorno_acao'):
                            if res_sub.get('retorno_acao')[0].get('id'):
                                _new_dados[_field_interno] = res_sub.get('retorno_acao')[0].get('id')
                    break
        if _new_dados:
            res = self._api_criar_classe(registro, _new_dados)
        if self.sig_gerar_log:
            _logger.info('Funcao: _api_acao_post_dados - Saida - %s, ' % str(res))
        return res

    def _api_get_set_m2o(self, registro, dados):
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_get_set_m2o - Inicio - registro: %s, dados: %s' % (registro, dados))
        registro = 'estruturas.%s' % registro.strip()
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        if not isinstance(dados, (list, tuple)):
            try:
                d = int(dados)
                res['retorno_acao'] = [{'id': d}]
            except:
                res['cod'] = '12'
                res['message'] = '%s - %s' % (self.status_api.get('12'), 'Dado %s nao é um ID' % dados)
            return res
            # encontrar campos para pesquisa
        _d = {}
        for linha in eval(registro):
            if linha[6] != '0':
                _l = _d.get(linha[6], [])
                _l.append(linha[1])
                _d[linha[6]] = _l
        if _d:
            keys = _d.keys()
            keys.sort()
            for key in keys:
                filters = {}
                for campo in _d.get(key):
                    for linha in eval(registro):
                        if linha[1] == campo:
                            filters[campo] = dados[linha[0]]
                            break
                dados_get = {'filters': filters, 'fields': ['id']}
                res = self._api_acao_get_dados(registro, dados_get)
                if res.get('cod') == '00':
                    for ret_acao in res.get('retorno_acao'):
                        if ret_acao.get('id'):
                            return res
        res = self._api_acao_post_dados(registro, dados)
        if self.sig_gerar_log:
            _logger.info('Funcao: _api_acao_post_dados - Saida - %s, ' % str(res))
        return res

    def _api_acao_put_dados(self, registro, dados):
        """
        Método para alterar registros nas tabelas
        :param registro: char, contem a tabela a ser trabalhada
        :param dados: dict, dicionario contendo os dados a serem alterados
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, lista contendo dicionario com os dados da consulta
        """
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_acao_put_dados - Inicio - registro: %s, dados: %s' % (registro, dados))

        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        fields = dados.keys()
        _new_dados = {}
        _tem_id = False
        for field in fields:
            if field in ('create_date', 'write_date', 'create_uid', 'write_uid'):
                continue
            if field == 'id':
                _tem_id = True
            for linha in eval(registro):
                if linha[5] == 'P': # desconsiderar campos proibidos para alteraçao e inclusao
                    continue
                if linha[0] == field:
                    _field_interno = linha[1]
                    _new_dados[_field_interno] = dados.get(field)
                    break
        if not _tem_id:
            res['cod'] = '13'
            res['message'] = self.status_api.get('13')
            return res
        if _new_dados:
            _new_dados['id'] = dados['id']
            res = self._api_alterar_classe(registro, _new_dados)
        if self.sig_gerar_log:
            _logger.info('Funcao: _api_acao_put_dados - Saida - %s, ' % str(res))
        return res

    def _api_acao_put_dados(self, registro, dados):
        """
        Método para consultar pessoas
        :param dados: dict, dicionario cont:endo os campos para pesquisa.
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, lista contendo dicionario com os dados da consulta
        """

        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        return res

    def _api_acao_delete_dados(self, registro, dados):
        """
        Método para consultar pessoas
        :param dados: dict, dicionario cont:endo os campos para pesquisa.
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, lista contendo dicionario com os dados da consulta
        """

        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        return res

    def _api_acao_get_dados(self, registro, dados):
        """
        Método para consultar pessoas
        :param dados: dict, dicionario cont:endo os campos para pesquisa.
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, lista contendo dicionario com os dados da consulta
        """

        args = []
        _filters = dados.get('filters', False)
        _filters_key = _filters.keys()
        _filters_oper = []
        _filters_code = []
        for x in _filters_key:
            if x[:1] == '_':
                _filters_oper.append(x)
            else:
                _filters_code.append(x)
        if _filters_code:
            for _field in _filters_code:
                _oper_code = '_%s' % _field
                args.append ((_field, dados.get('filters').get(_oper_code, '='), dados.get('filters').get(_field)))
        else:
            args = []
        _limit = dados.get('limit', 10)
        _offset = dados.get('offset', 0)
        fields = dados.get('fields', [])

        _new_fields = self._api_validar_fields(registro, fields)
        res = self._api_ler_classe(registro, args, _new_fields, _offset, _limit)
        if res and res.get('cod') and res.get('cod') == '00':
            _new_ret = []
            lista = []
            _dict_campos = {}
            for field in _new_fields:
                x = field.split('.')
                if len(x) > 1:
                    lista = _dict_campos.get(x[0], [])
                    lista.append(x[1])
                    _dict_campos[x[0]] = lista
            for key in _dict_campos.keys():
                for linha in eval(registro):
                    if key == linha[1]:
                        _new_ret_acao = []
                        for _d in res.get('retorno_acao'):
                            child_ids = _d.get(key)
                            child_res = self._api_ler_classe('estruturas.%s' % linha[4], child_ids, _dict_campos.get(key), _offset, _limit)
                            if child_res.get('cod') == '00':
                                _lista_filho = []
                                for child_linha in child_res.get('retorno_acao'):
                                    _lista_filho.append(child_linha)
                                    _d[key] = _lista_filho
                            _new_ret_acao.append(_d)
                        res['retorno_acao'] = _new_ret_acao
                        break
            res = self._api_ajustar_retono_campos(res, registro, _new_fields, 'S')
        return res

    def _api_classe_campos(self, tabela):
        res = {'cod': '00', 'message': '', 'retorno_acao': []}
        _retorno_acao = []
        tab = eval(tabela)
        for linha in tab:
            _retorno_acao.append({'Campo':linha[0],'Descricao': linha[2], 'tipo de dados': linha[3], 'tamanho': linha[4], 'Obrigatorio na inclusao': linha[5]})
        res['retorno_acao'] = _retorno_acao
        return res

    def _api_acao_get_dados_campos(self, registro):
        """
        Método para consultar dos campos nos registros
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, Lista de dicionario com os dados da consulta
        """
        res = self._api_classe_campos(registro)
        return res

    def _api_valida_token(self, token):
        """
        Método destinado a validar o token
        :param token: char, token a ser validado
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - dados_token dict, dicionario com os dados registrados no token
        """
        if self.sig_gerar_log:
            _logger.error('Funcao: _api_valida_token - Inicio - token: %s' % token)
        res = {'cod': '00', 'message': '', 'dados_token': {}}
        try:
            dados_token = jwt.decode(token, self.sig_frase)
            res['dados_token'] = dados_token
        except jwt.InvalidTokenError:
            res['cod'] = '01'
            res['message'] = 'Token invalido. Tente novamente.'
        except jwt.ExpiredSignatureError:
            res['cod'] = '02'
            res['message'] = 'Token vencido. Tente novamente.'
        if self.sig_gerar_log:
            _logger.info('Funcao: _api_valida_token - Saida - %s, ' % str(res))
        return res

    def _api_get_campos_sigerp(self, tabela):
        res = []
        if tabela == 'sigerp_pedidos':
            tabela = 'fields_pedidos'
            res_itens = ['id', 'quantidade', 'valor_unitario', 'desconto', 'observacoes', 'produto', 'grade']
            res_itens.extend(('local', 'valor_total', 'embalagem'))
            res_parcelas = ['id', 'valor_parcela', 'condicao', 'vencimento']
            res = ['id', 'data_emissao', 'data_entrega', 'numero_pedido', 'observacoes', 'pessoa']
            res.extend(({'itens': res_itens}, {'parcelas': res_parcelas},'state', 'valor', 'tipo'))
            res_condicoes = ['id', 'condicao_forma', 'forma_pgto', {'parcelas': res_parcelas}]
            res.append({'condicoes': res_condicoes})
        return tabela, res

    def _api_executar(self, tabela, dados, retornar_campos):
        """
        Método de uso interno da classe destinado a identificar a ação para chamar o metodo especifico de cada ação
        :param acao: char, nome da ação
        :param dados: dict, contendo informações que são necessárias a cada ação
        :return: dict, contendo:
                - cod: char, codigo do status
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, Lista de dicionario com os dados dde retorno da ação qdo existir.
        """
        dados = json.loads(dados)
        if tabela[:7] == 'sigerp_':
            tabela, fields = self._api_get_campos_sigerp(tabela)
            dados['fields'] = fields
        res =  {'cod': '04', 'message': self.status_api.get('04')}
        tabela = 'estruturas.%s' % tabela.strip()
        if tabela == 'teste':
            res = self._api_acao_teste()
        if self.sig_metodo == 'GET' and retornar_campos:
            res = self._api_acao_get_dados_campos(tabela)
        elif self.sig_metodo == 'GET': # Ler
            res = self._api_acao_get_dados(tabela, dados)
        elif self.sig_metodo == 'POST': # Incluir
            res = self._api_acao_post_dados(tabela, dados)
        elif self.sig_metodo == 'PUT': # alterar
            res = self._api_acao_put_dados(tabela, dados)
        elif self.sig_metodo == 'DELETE': # Excluir
            res = self._api_acao_post_dados(tabela, dados)
        return res



    ## Métodos externos ############################################
    @http.httprequest
    def api_logar(self, req, s_action=None, **kw):
        """
        Metodo para efetuar login no SigERP
        :param req: Dicionario contendo user: nome do usuario, senha: contendo a senha do usuario
        :param s_action:
        :param kw:
        :return: json contendo cod: com o codigo da operacao, message: descricao da mensagem, token: o token calculado,
                 refresh_token, o token para revalidacao
                 cod diferente de '00' eh indicativo de erro
        """
        #_server = str(req.httprequest.host_url)
        res = {'cod': '00', 'message': 'Login bem sucessido!'}
        res = { }
        self.sig_user = req.params.get('usuario')
        self.sig_senha = req.params.get('senha')
        self.sig_code = req.params.get('sg_code')
        self.sig_frase = base64.b64decode(self.sig_code)
        self.sig_base, self.sig_tempo, appid = self.sig_frase.split('*|*')

        try:
            sock_common = xmlrpclib.ServerProxy(self.sig_server + '/xmlrpc/common')
            socket.setdefaulttimeout(3)
            self.sig_uid = sock_common.login(self.sig_base, self.sig_user, self.sig_senha)
        except xmlrpclib.Fault as err:
            _logger.error('A XML-RPC Fault occured: %s' % err.faultString)
            pass
        except socket.error as _mens:
            self.sig_uid = False
            _logger.error('=== Nao consegui conectar! Base:%s user:%s, senha:%s, server:%s, erro:%s' % (
                self.sig_base, self.sig_user, self.sig_senha, self.sig_server, _mens))
            return json.dumps({'cod': '03', 'message': self.status_api.get('03')})
        if not self.sig_uid:
            _logger.info('=== Usuario ou login invalido base:%s user:%s, senha:%s, server:%s' % (
                self.sig_base, self.sig_user, self.sig_senha, self.sig_server))
            return json.dumps({'cod': '03', 'message': self.status_api.get('03')})
        _logger.info('=== Conectado a base %s como usuario %s' % ( self.sig_base, self.sig_user))

        res['err'] = None
        try:
            sock = xmlrpclib.ServerProxy('%s/xmlrpc/object' % self.sig_server)
            args = [('apicon_code', '=', self.sig_code)]
            res_ids = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, 'api.controle', 'search', args)
        except xmlrpclib.Fault as err:
            res_ids = False
            _logger.error('A XML-RPC Fault occured: %s' % err.faultString)
            res['err'] = 'A XML-RPC Fault occured: %s. See logs for details.' % err.faultString
        except socket.error as _mens:
            _logger.error('=== Ocorreu um erro ao validar o sg_code: base:%s, user:%s, sg_code:%s' %(
                self.sig_base, self.sig_user, self.sig_sg_code))
            return json.dumps({'cod': '11', 'message': self.status_api.get('11')})
        if not res_ids:
            _logger.error('=== Nao foi possivel validar o sg_code: base:%s, user:%s, sg_code:%s' %(
                self.sig_base, self.sig_user, self.sig_code))
            return json.dumps({'cod': '11', 'message': self.status_api.get('11'), 'reason': res['err']})
        dados = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, 'api.controle', 'read', res_ids[0], ['apicon_log'])
        self.sig_gerar_log = dados.get('apicon_log')

        res = {'cod': '00', 'message': 'Login bem sucedido!'}
        res['token'], res['refresh_token'] = self._api_gerar_token()
        return json.dumps(res)


    @http.httprequest
    def api_executar(self, req, s_action=None, **kw):
        """
        Metodo a ser chamado pela aplicação externa para execução de alguma ação
        :param req:
        :param s_action:
        :param kw:
        :return: dict, contendo:
                - cod: char, codigo do status - Cod diferente de '00' é indicativo de erro
                - message: char, mensagem de status qdo tiver
                - retorno_acao list, Lista de dicionario com os dados dde retorno da ação qdo existir.
        """
        retorno = {'cod': '00', 'message': ''}
        token = req.httprequest.headers.environ.get('HTTP_AUTHORIZATION')
        res = self._api_valida_token(token)
        retorno['cod'] = res.get('cod')
        retorno['message'] = res.get('message')
        if res.get('cod') != '00':
            return json.dumps(retorno)
        iss = res.get('dados_token').get('iss')
        if iss != 'SigERP':
            return json.dumps({'cod': '06', 'message': self.status_api.get('06')})
        self.sig_uid = res.get('dados_token').get('sub')
        self.sig_senha = res.get('dados_token').get('pwd')
        self.sig_base = res.get('dados_token').get('base')
        self.sig_metodo = req.httprequest.method
        self.sig_metodo_classe = req.params.get('metodo', False)
        _server = str(req.httprequest.host_url)
        sock = xmlrpclib.ServerProxy(self.sig_server + '/xmlrpc/object')
        args = []
        res_ids = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, 'api.controle', 'search', args)
        dados = sock.execute(self.sig_base, self.sig_uid, self.sig_senha, 'api.controle', 'read', res_ids[0], ['apicon_log'])
        self.sig_gerar_log = dados.get('apicon_log')
        retorno = self._api_executar(req.params.get('tabela'), req.params.get('dados_acao'), req.params.get('retornar_campos'))
        res = json.dumps(retorno)
        return res


    @http.httprequest
    def api_revalidar_token(self, req, s_action=None, **kw):
        """
        Metodo para efetuar revalidacao de token vencido
        :param req: Dicionario contendo refresh_token: token a ser utilizado para revalidaca
        :param s_action:
        :param kw:
        :return: json contendo cod: com o codigo da operacao, message: descricao da mensagem, token: o token calculado,
                 refresh_token, o token para revalidacao
                 cod diferente de '00' eh indicativo de erro
        """
        res = {'cod': '00', 'message': '', 'dados_token': {}}
        refresh_token = req.params.get('token')
        try:
            dados_token = jwt.decode(refresh_token, self.sig_frase)
            res['dados_token'] = dados_token
            self.sig_base = dados_token.get('base')
            self.sig_uid = dados_token.get('sub')
            iss = dados_token.get('iss')
            self.sig_senha = dados_token.get('pwd')
            if iss == 'SigERP_refresh':
                token, refresh_token = self._api_gerar_token()
                res['token'] = token
                res['refresh_token'] = refresh_token
            else:
                res['cod'] = '05'
                res['message'] = self.status_api.get('05')
        except jwt.InvalidTokenError:
            res['cod'] = '01'
            res['message'] = self.status_api.get('01')
        except jwt.ExpiredSignatureError:
            res['cod'] = '02'
            res['message'] = self.status_api.get('02')
        return json.dumps(res)

