# -*- encoding: utf-8 -*-

__author__ = 'tiago.santos'

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config


class TestWizardCorrecaoMensalidade(TransactionCase):
    """
    Classe responsavel por testes unitarios dos metodos da classe wizard_correcao_mensalidade
    """

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestWizardCorrecaoMensalidade, self).setUp()
        self.correcao_mensalidade = self.registry('wizard.correcao.mensalidade')
        self.correcao_mensalidade_id = self.cria_correcao_mensalidade()
        self.categoria_id = self.get_categoria_id()
        self.pessoa_id = self.get_pessoa_id()
        self.pessoa_categoria_id = self.create_pessoa_categoria_id()
        self.socio_id = self.get_socio_id()

    def create_pessoa_categoria_id(self):
        sql = """
            SELECT pessoa_id FROM pessoa_categoria WHERE pessoa_id = %s AND categoria_id = %s
        """ % (self.pessoa_id, self.categoria_id)
        self.cr.execute(sql)
        dados = self.cr.dictfetchall()
        if dados:
            return True
        sql = """
            INSERT INTO pessoa_categoria (pessoa_id, categoria_id) VALUES (%s, %s)
        """ % (self.pessoa_id, self.categoria_id)
        self.cr.execute(sql)
        return True

    def cria_correcao_mensalidade(self):
        vals = {'wcomen_valor_mensalidade': 65.00}
        return self.correcao_mensalidade.create(self.cr, self.uid, vals)

    def get_categoria_id(self):
        categoria = self.registry('adm.pessoa.categoria.nome')
        categoria_ids = categoria.search(self.cr, self.uid, [('name', '=', 'TESTE'),
                                                             ('admpcn_tipo_mensalidade', '=', 'C')])
        if categoria_ids:
            categoria_id = categoria_ids[0]
        else:
            vals = {'name': 'TESTE', 'admpcn_tipo_mensalidade': 'C'}
            categoria_id = categoria.create(self.cr, self.uid, vals)
        return categoria_id

    def get_pessoa_id(self):
        pessoa = self.registry('adm.pessoa')
        pessoa_ids = pessoa.search(self.cr, self.uid, [('cpf_cnpj', '=', '98899682070')])
        if pessoa_ids:
            pessoa_id = pessoa_ids[0]
        else:
            vals = dict(pessoa_tipo='F', name='DESTINATARIO DE TESTE ÁÇÃO%°*á',
                        cpf_cnpj='98899682070')
            pessoa_id = pessoa.create(self.cr, self.uid, vals)
        return pessoa_id

    def get_socio_id(self):
        socio = self.registry('associacao.socios')
        socio_ids = socio.search(self.cr, self.uid, [('asssoc_pessoa_id', '=', self.pessoa_id)])
        if socio_ids:
            socio_id = socio_ids[0]
        else:
            vals = {'asssoc_pessoa_id': self.pessoa_id, 'asssoc_dia_vencimento': 10,
                    'asssoc_valor_contribuicao': 50.00}
            socio_id = socio.create(self.cr, self.uid, vals)
        return socio_id

    def test_wcomen_set_valor_mensalidade(self):
        res = self.correcao_mensalidade.wcomen_set_valor_mensalidade(
            self.cr, self.uid, [self.correcao_mensalidade_id], {})
        self.assertTrue(res)
