# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from SigUtils import numero


class automacao_codigo_barra(osv.Model):
    _name = 'automacao.codigo.barra'
    _inherit = 'automacao.codigo.barra'

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        if name.isdigit():
            ids = self.search(cr, uid, [('codigo_barra', '=', name)] + args, limit=limit,
                              context=context)
            if not ids:
                ids = self.search(cr, uid, [('produto_id', '=', name)] + args, limit=limit,
                                  context=context)
        else:
            name = str(name)
            produto_ids = self.pool.get('estoque.produto').search(
                cr, uid, [('name', operator, name)], context=context)
            ids = self.search(cr, uid, [('produto_id', 'in', produto_ids)] + args, limit=limit,
                              context=context)
        return self.name_get(cr, uid, ids)

    def name_get(self, cr, uid, ids, context=None):
        res = []
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['produto_id'], context=context)
        for record in reads:
            res.append((record['id'], record['produto_id'][-1]))
        return res

    def autcba_onchange_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        warning = {}
        if not produto_id:
            vals = {}.fromkeys(['grade_id'], False)
        else:
            vals = {}
            obj_produto_grade = self.pool.get('estoque.produto.grade')
            grade_ids = obj_produto_grade.search(cr, uid, [('produto_id', '=', produto_id)], context=context)
            if len(grade_ids) == 1:
                grade_id = grade_ids[0]
                vals['grade_id'] = grade_id
            else:
                vals['grade_id'] = False

        vals['produto_embalagem_id'] = False
        return {'value': vals, 'warning': warning}

    def autcba_onchange_grade(self, cr, uid, ids, grade_id, produto_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'produto_embalagem_id': False}}
        if not grade_id:
            return res

        obj_emb = self.pool.get('estoque.produto.embalagem')
        emb_ids = obj_emb.search(cr, uid, [('produto_id','=', produto_id)], context=context)
        if not emb_ids:
            return res

        if len(emb_ids) == 1:
            res['produto_embalagem_id'] = emb_ids[0]

        return res

    def autcba_onchange_codigo_barra(self, cr, uid, ids, codigo_barra, usa_digito, context=None):
        """
        Verificacao de codigo de barras - padrao EAN-128
            codigo_barra e composto do codigo numerico + o digito
            antes de chamar a funcao de verificacao, e preciso separar o ultimo digito, e passar apenas a primeira parte.
        """
        context = context and context.copy() or {}
        res = {}
        if codigo_barra:
            try:
                digito = self._autcba_validar_codigo(cr, uid, ids, codigo_barra,
                                                     usa_digito=usa_digito, context=context)
                if digito == True:
                    res['value'] = {}
                else:
                    res['warning'] = {'title': _('Atencao'),
                                      'message': _(
                                          'Codigo de barras invalido. O digito para este codigo e: %s') % digito}
                    res['value'] = {}
            except:
                raise osv.except_osv(_('Atencao'), _('Erro na verificacao do codigo de barras.'))
            context.update({'active_test': False})
            ids_automacao_codigo_barra = self.search(cr, uid, [('codigo_barra', '=', codigo_barra),
                                                               ('autcba_usa_digito','=', usa_digito)], context=context)
            del context['active_test']
            if ids_automacao_codigo_barra:
                automacao_codigo_barra_dados = self.read(cr, uid, ids_automacao_codigo_barra, ['produto_id'], context=context)
                if isinstance(automacao_codigo_barra_dados, list):
                    automacao_codigo_barra_dados = automacao_codigo_barra_dados[0]
                produto_id = automacao_codigo_barra_dados.get('produto_id')
                if isinstance(produto_id, tuple):
                    produto_id = produto_id[0]
                estoque_produto_dados = self.pool.get('estoque.produto').read(cr, uid, [produto_id], ['codigo'], context=context)
                if isinstance(estoque_produto_dados, list):
                    estoque_produto_dados = estoque_produto_dados[0]
                codigo = estoque_produto_dados.get('codigo')
                res['warning'] = {'title': _('Atencao'),
                                  'message': _('Codigo de barra ja utilizado pelo produto de codigo %s') % codigo}
                res['value'] = {}
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('active'):
            brw_automacao_codigo_barra = self.browse(
                cr, uid, ids if isinstance(ids, list) else [ids], context)
            brw_produto_ids = list(set(map(lambda cd: cd.produto_id, brw_automacao_codigo_barra)))
            msg = ''
            if 'produto_id' in vals and vals.get('produto_id'):
                brw_estoque_produto = self.pool.get('estoque.produto').browse(
                    cr, uid, vals.get('produto_id'), context)
                if not brw_estoque_produto.active:
                    msg += '%s - %s \n' % (brw_estoque_produto.codigo,
                                           brw_estoque_produto.name or '')
            else:
                for produto in brw_produto_ids:
                    if produto.active:
                        continue
                    msg += '%s - %s \n' % (produto.codigo, produto.name or '')
            if msg:
                raise osv.except_osv(_('Atencao'),
                                     _('Para ativar o codigo de barras, ative os seguintes '
                                       'produtos: \n%s' % msg))

        if 'produto_id' in vals or 'grade_id' in vals:
            context_preco = context.copy()
            context_preco['sig_force_company'] = True
            obj_preco = self.pool.get('automacao.precos')
            preco_ids = obj_preco.search(cr, uid, [('codigo_barra_id', 'in', ids)],
                                         context=context_preco)
            if preco_ids:
                obj_preco.unlink(cr, uid, preco_ids, context)

        if 'codigo_barra' in vals:
            automacao_codigo_barra_dados = self.read(cr, uid, ids, ['codigo_barra', 'autcba_usa_digito'], context=context)
            if isinstance(automacao_codigo_barra_dados, list):
                automacao_codigo_barra_dados = automacao_codigo_barra_dados[0]
            codigo_barra = automacao_codigo_barra_dados.get('codigo_barra')
            usa_digito_gravado = automacao_codigo_barra_dados.get('autcba_usa_digito')
            if codigo_barra == vals.get('codigo_barra'):
                pass
            else:
                ids_automacao_codigo_barra = self.search(cr, uid, [('codigo_barra', '=', vals.get('codigo_barra'))],
                                                         context=context)
                if ids_automacao_codigo_barra:
                    automacao_codigo_barra_dados = self.read(cr, uid, ids_automacao_codigo_barra, ['produto_id'], context=context)
                    if isinstance(automacao_codigo_barra_dados, list):
                        automacao_codigo_barra_dados = automacao_codigo_barra_dados[0]
                    produto_id = automacao_codigo_barra_dados.get('produto_id')
                    if isinstance(produto_id, tuple):
                        produto_id = produto_id[0]
                    estoque_produto_dados = self.pool.get('estoque.produto').read(cr, uid, [produto_id], ['codigo'], context=context)
                    if isinstance(estoque_produto_dados, list):
                        estoque_produto_dados = estoque_produto_dados[0]
                    codigo = estoque_produto_dados.get('codigo')
                    raise osv.except_osv(_('Atencao'),
                                         _('Codigo de barra ja utilizado pelo produto de codigo %s') % codigo)
            cod = numero.getNumToStr(vals['codigo_barra'], 14, 0, 'N') # formatar para 14 bytes
            if cod == -1:
                raise osv.except_osv(_('Atencao'), _('Codigo de barras invalido.'))
            vals['codigo_barra'] = cod
            usa_digito = True
            if 'autcba_usa_digito' in vals:
                if not vals.get('autcba_usa_digito'):
                    usa_digito = False
            else:
                usa_digito = usa_digito_gravado

            digito = self._autcba_validar_codigo(cr, uid, None, cod, usa_digito=usa_digito, context=context)
            if digito != True:
                raise osv.except_osv(_('Atencao'),
                                     _('Codigo de barras invalido. O digito para este codigo e: %s') % digito)
        res = super(automacao_codigo_barra, self).write(cr, uid, ids, vals, context)
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context and context.get('copy_estoque_produto'):
            # ao duplicar produto nao pode ser criado(duplicado) os codigos de barra
            return 1
        if 'codigo_barra' in vals:
            ids_automacao_codigo_barra = self.search(cr, uid, [('codigo_barra', '=', vals.get('codigo_barra'))], context=context)
            if ids_automacao_codigo_barra:
                if context.get('importacao_nfe'):
                    return ids_automacao_codigo_barra[0]
                raise osv.except_osv(
                    _('Atencao'), _('Codigo de barras ja utilizado pelo produto de codigo %s') %
                    self.browse(cr, uid, ids_automacao_codigo_barra[0], context).produto_id.codigo)

            cod = numero.getNumToStr(vals['codigo_barra'], 14, 0, 'N') # formatar para 14 bytes
            if cod == -1:
                raise osv.except_osv(_('Atencao'), _('Codigo de barras invalido.'))
            vals['codigo_barra'] = cod
        if 'grade_id' in vals:
            if not 'produto_id' in vals:
                obj_grade = self.pool.get('estoque.produto.grade')
                grade = obj_grade.read(cr, uid, [vals['grade_id']], ['produto_id'], context=context)
                if grade[0]['produto_id'][0]:
                    obj_prod = self.pool.get('estoque.produto')
                    produto_id = grade[0]['produto_id'][0]
                    vals['produto_id'] = produto_id
                    if not 'company_id' in vals:
                        empresa = obj_prod.read(cr, uid, [produto_id], ['company_id'], context=context)
                        if empresa[0]['company_id'][0]:
                            company_id = empresa[0]['company_id'][0]
                            vals['company_id'] = company_id
        codigo_barra = vals.get('codigo_barra')
        if codigo_barra:
            usa_digito = True
            if 'autcba_usa_digito' in vals:
                if not vals.get('autcba_usa_digito'):
                    usa_digito = False

            digito = self._autcba_validar_codigo(cr, uid, None, codigo_barra, usa_digito=usa_digito, context=context)
            if digito != True:
                raise osv.except_osv(_('Atencao'),
                                     _('Codigo de barras invalido. O digito para este codigo e: %s') % digito)
        res = super(automacao_codigo_barra, self).create(cr, uid, vals, context)
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        # este contexto eh usado para mostrar os codigos inativos na tree e deixa-los na cor
        # vermelha, vindo do cadastro de produto, grade e codigo de barras, se o search for chamado
        # de alguma funcao do sistema este contexto nao deve ser passado a menos que for para
        # realmente listar os codigos inativos (Felipe)
        if context.get('codigo_barras_inativo'):
            context['active_test'] = False

        for arg in args:
            if 'codigo_barra' in arg and arg[2] == '=':
                if isinstance(arg[2], list):
                    aux = []
                    for i in arg[2]:
                        aux.append(numero.getNumToStr(i, 14, 0, 'N'))

                    cod = aux
                else:
                    cod = numero.getNumToStr(arg[2], 14, 0, 'N')

                args.remove(arg)
                arg = (arg[0], arg[1], cod)
                args.append(arg)
            elif 'codigo_barra' in arg and arg[2]:
                op = arg[1]
                codigo = arg[2]
                args.remove(arg)
                if isinstance(codigo, list):
                    codigos = []
                    for i in codigo:
                        try: # trata erro que vem da importacao de inventario do coletor.
                            codigos.append(str(int(i)).zfill(14))
                        except:
                            continue

                    arg = ('codigo_barra', op, codigos)
                else:
                    arg = ('codigo_barra', op, codigo.zfill(14))

                args.append(arg)

        res = super(automacao_codigo_barra, self).search(cr, uid, args, offset, limit, order,
                                                         context, count)
        return res

    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Atencao'), _('Nao e permitido excluir codigo de barras'))

    def autcba_get_codigo_barra(self, cr, uid, produto_id, grade_id=False,
                                produto_embalagem_id=False, context=None):
        """
        O metodo encontra o codigo de barras para o produto, grade e embalagem informados.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_id: id da tabela estoque_produto, identificador do produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade, identificador da grade do produto.
        :type grade_id: int
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem, identificador da
        embalagem do produto.
        :type produto_embalagem_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: string do codigo de barras.
        :rtype: str
        """
        context = context and context.copy() or {}
        codigo_barra = ''
        if not produto_id and not grade_id:
            return codigo_barra
        context['active_test'] = False
        args = [('produto_id', '=', produto_id)]
        if grade_id:
            args.append(('grade_id', '=', grade_id))
        if produto_embalagem_id:
            args.append(('produto_embalagem_id', '=', produto_embalagem_id))
        codigo_barra_ids = self.search(cr, uid, args, context=context)
        if not codigo_barra_ids:
            return codigo_barra
        return self.browse(cr, uid, codigo_barra_ids[0], context).codigo_barra

    def autcba_get_codigo_barra_formatado_gtin(self, cr, uid, ids, context=None):
        """
        O metodo formata os codigos de barras de acordo com o tipo definido no seu cadastro
        (autcba_tipo_barra).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Dicionario. A chave representa o id e o valor representa o codigo de barras
        formatado.
        :rtype: dict
        """
        res = {}
        for codigo in self.read(cr, uid, ids if isinstance(ids, (list, tuple)) else [ids],
                                ['codigo_barra', 'autcba_tipo_barra'], context):
            tipo = codigo.get('autcba_tipo_barra')
            if tipo == '08':
                res.update({codigo.get('id'): codigo.get('codigo_barra')[-8:]})
            elif tipo == '12':
                res.update({codigo.get('id'): codigo.get('codigo_barra')[-12:]})
            elif tipo == '13':
                res.update({codigo.get('id'): codigo.get('codigo_barra')[-13:]})
            elif tipo == '14':
                res.update({codigo.get('id'): codigo.get('codigo_barra')[-14:]})
            else:
                res.update({codigo.get('id'): 'SEM GTIN'})
        return res
