# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv

# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyAbstractClass,PyPep8Naming
class automacao_precos_promocao(osv.Model):
    _name = 'automacao.precos.promocao'
    _inherit = 'automacao.precos.promocao'

    def create(self, cr, uid, vals, context=None):
        """
        Sobescreve o metodo para criar as promocoes para todas as grades do produto se a grade nao
        for informada
        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param vals: valores utilizados na criacao do registro
        :param context: um dicionario padrao para valores contextuais,
        :return: ID do registro que foi criado
        :rtype: long
        :raise: mensagem de erro de periodo invalido ou se ja existe promocao cadastrada no periodo.
        """

        context = context and context.copy() or {}
        result = 0L

        # Verifica o periodo
        if vals.get('data_termino') < vals.get('data_inicio'):
            raise osv.except_osv(_('Atencao'), _('Data de termino nao pode ser menor que a data '
                                                 'inicial'))

        periodo = vals.get('data_inicio'), vals.get('data_termino')

        # Recupera todas as grades do produto se a grade nao for informada
        if not vals.get('grade_id'):
            grade_ids = self.pool.get('estoque.produto.grade').search(
                cr, uid, [('produto_id', '=', vals.get('produto_id'))], context=context)
        else:
            grade_ids = [vals.get('grade_id')]

        # Verifica se ja existe promocao no periodo

        if self.autppr_verifica_promocao_periodo(
                cr, uid, vals.get('produto_id'), grade_ids, vals.get('company_id'), periodo,
                context):
            raise osv.except_osv(_('Atencao'), _('Promocao ja cadastrada para o produto no '
                                                 'periodo'))

        # Cria as promocoes para as grades do produto
        for grade_id in grade_ids:
            vals = dict(vals, grade_id=grade_id)
            result = super(automacao_precos_promocao, self).create(cr, uid, vals, context)

        return result

    def autppr_onchange_data_inicio(self, cr, uid, ids, grade_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if not grade_id:
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Se a grade nao for informada a promocao sera aplicada '
                                           'para todas as grades do produto')}
        return res

    def autppr_verifica_promocao_periodo(self, cr, uid, produto, grade, empresa, periodo, context):
        """
        Verifica se ja existe promocao para o produto no periodo informado

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param produto: ID do produto
        :param grade: ID da grade
        :param empresa: ID da empresa
        :param periodo: datas do periodo: inicial e final
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type produto: long|list[long]
        :type grade: long|list[long]
        :type empresa: long
        :type periodo: tuple
        :return: lista como o ID dos periodos de promocao
        :rtype: list[long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        args = [('produto_id', 'in' if isinstance(produto, list) else '=', produto),
                ('grade_id', 'in' if isinstance(grade, list) else '=', grade),
                ('company_id', '=', empresa), ('data_inicio', '=', periodo[0]),
                ('data_termino', '=', periodo[1])]

        result = self.search(cr, uid, args, context=context)

        return result

    def unlink(self, cr, uid, ids, context=None):
        produto_ids = []
        grade_ids = []
        obj_empresa_objeto = self.pool.get('adm.empresa.objeto')
        for i in self.read(cr, uid, ids, ['produto_id', 'grade_id'], context, '_classic_write'):
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))

        acb_company_id = obj_empresa_objeto.getEmpresa(cr, uid, 'automacao.codigo.barra',
                                                       context=context)
        args = [('grade_id', 'in', grade_ids), ('produto_id', 'in', produto_ids),
                ('company_id', '=', acb_company_id)]
        context['sig_force_company'] = True
        cb_ids = self.pool.get('automacao.codigo.barra').search(cr, uid, args, context=context)
        if cb_ids:
            obj_ap = self.pool.get('automacao.precos')
            ap_company_id = obj_empresa_objeto.getEmpresa(cr, uid, 'automacao.precos',
                                                          context=context)
            ap_ids = obj_ap.search(cr, uid, [('codigo_barra_id', 'in', cb_ids),
                                             ('company_id', '=', ap_company_id)], context=context)
            if ap_ids:
                obj_ap.unlink(cr, uid, ap_ids, context)

        return super(automacao_precos_promocao, self).unlink(cr, uid, ids, context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: