# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estsug_compra_capa(osv.Model):
    # prefixo: esgccp_
    _inherit = 'estsug.compra.capa'

    def _esgccp_get_dado_minimo_maximo(self, cr, uid, sugestao_id, custo, data_inicial, data_final,
                                       local_id, considera_todas_empresas, company_id, produto_ids,
                                       prazo_media, tipo_curva, listar_estatistica,
                                       codigo_imprimir, considera_pedidos, so_sugestao,
                                       gerar_por_empresa, comprador_id, context=None):
        """
        O metodo realiza a busca dos produtos e suas informacoes de acordo com os parametros
        passados. Para o tipo de sugestao minimo/maximo a busca eh feita basicamente pela tabela
        de grade dos produtos.
        :param cr: cursor no banco de dados
        :param uid: id do usuario
        :param sugestao_id: id da tabela
        :param custo: char de descricao do custo que sera considerado
        :param data_inicial: data inicial do periodo relacionado ao movimento de itens
        :param data_final: data final do periodo relacionado ao movimento de itens
        :param local_id: id da tabela estoque_local
        :param considera_todas_empresas: booleano que identifica se sera usada todas as empresas
        ou somente a que esta relacionada a tabela, para a busca dos dados
        :param company_id: id da tabela res_company
        :param produto_ids: lista de id dos produtos
        :param prazo_media: inteiro que identifica o periodo em meses
        :param tipo_curva: char que identifica a curva, sendo A , B, C ou T (Todas)
        :param listar_estatistica: booleano que identica se as estatisticas de venda de cada
        produto sera exibida
        :param codigo_imprimir: char para definir se sera impresso o codigo do item (I) ou do
        fabricante (F)
        :param gerar_por_empresa: lista os itens e seus detalhes separados por empresa. Boolean.
        :return: Lista com os dados que serao incluidos na tree de itens, tabela estsug_compra_item
        """
        
        context = context and context.copy() or {}
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        res = super(estsug_compra_capa, self)._esgccp_get_dado_minimo_maximo(
            cr, uid, sugestao_id, custo, data_inicial, data_final, local_id,
            considera_todas_empresas, company_id, produto_ids, prazo_media, tipo_curva,
            listar_estatistica, codigo_imprimir, considera_pedidos, so_sugestao, gerar_por_empresa, comprador_id,
            context)
        if codigo_imprimir != 'B':
            return res
        for dado in res:
            dado = dado[2]
            produto_id = dado.get('esgcit_produto_id')
            grade_id = dado.get('esgcit_grade_id')
            dado['esgcit_item_codigo'] = obj_automacao_codigo_barra.autcba_get_codigo_barra(
                cr, uid, produto_id, grade_id, context=context)
        return res

    def _esgccp_get_dado_estatistica(self, cr, uid, sugestao_id, custo, data_inicial, data_final,
                                     local_id, considera_todas_empresas, company_id, produto_ids,
                                     prazo_sugestao, prazo_media, tipo_curva, listar_estatistica,
                                     codigo_imprimir, considera_pedidos, so_sugestao,
                                     gerar_por_empresa, comprador_id, context=None):
        """
        O metodo realiza a busca dos produtos e suas informacoes de acordo com os parametros
        passados. Para o tipo de sugestao estatistica a busca eh feita basicamente pela tabela
        de movimento de itens, mais especificamente no que foi vendido.

        :param cr: cursor no banco de dados
        :param uid: id do usuario
        :param sugestao_id: id da tabela
        :param custo: char de descricao do custo que sera considerado
        :param data_inicial: data inicial do periodo relacionado ao movimento de itens
        :param data_final: data final do periodo relacionado ao movimento de itens
        :param local_id: id da tabela estoque_local
        :param considera_todas_empresas: booleano que identifica se sera usada todas as empresas
        ou somente a que esta relacionada a tabela, para a busca dos dados
        :param company_id: id da tabela res_company
        :param produto_ids: lista de id dos produtos
        :param prazo_sugestao: inteiro que identifica um periodo em dias
        :param prazo_media: inteiro que identifica um periodo de movimentacao, em meses
        :param tipo_curva: char que identifica a curva, sendo A , B, C ou T (Todas)
        :param listar_estatistica: booleano que identica se as estatisticas de venda de cada
        produto sera exibida
        :param codigo_imprimir: char para definir se sera impresso o codigo do item (I) ou do
        fabricante (F)
        :param gerar_por_empresa: lista os itens e seus detalhes separados por empresa. Boolean.
        :return: Lista com os dados que serao incluidos na tree de itens, tabela estsug_compra_item
        """

        context = context and context.copy() or {}
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        res = super(estsug_compra_capa, self)._esgccp_get_dado_estatistica(
            cr, uid, sugestao_id, custo, data_inicial, data_final, local_id,
            considera_todas_empresas, company_id, produto_ids, prazo_sugestao, prazo_media,
            tipo_curva, listar_estatistica, codigo_imprimir, considera_pedidos, so_sugestao,
            gerar_por_empresa, comprador_id, context)
        if codigo_imprimir != 'B':
            return res
        for dado in res:
            dado = dado[2]
            produto_id = dado.get('esgcit_produto_id')
            grade_id = dado.get('esgcit_grade_id')
            dado['esgcit_item_codigo'] = obj_automacao_codigo_barra.autcba_get_codigo_barra(
                cr, uid, produto_id, grade_id, context=context)
        return res
