# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class estoque_movimento_itens(osv.Model):
    """
    Heranca da classe ´estoque_movimento_itens´.
    """

    _inherit = 'estoque.movimento.itens'

    def estmvi_onchange_item_movimento(self, cr, uid, ids, empresa, natureza, nf_tipo, destinatario,
                                       item, local, preco_venda, total, nf_propria, nf_devolucao,
                                       endereco=False, grade_id=False, tributacao=None,
                                       context=None):
        """
        Recupera a grade do produto (item) se existir apenas uma.

        :param cr: curor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento (produto)
        :param empresa: ID da empresa
        :param natureza: ID da natureza da operacao
        :param nf_tipo: tipo da nota fiscal: "E" para entrada/compra ou "S" para saida/venda
        :param destinatario: ID do destinatario: fornecedor se entrada ou cliente se saida
        :param item: ID do produto
        :param local: ID do local do produto
        :param preco_venda: preco de venda do produto
        :param total: valor total do produto
        :param nf_propria: identificador para notas fiscais proprias
        :param nf_devolucao: identificador para notas fiscais de devolucao
        :param endereco: ID do endereco do destinatario
        :param grade_id: o ID da grade do item
        :param tributacao: lista com ID das tributacoes (one2many)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type empresa: long
        :type natureza: long
        :type nf_tipo: str
        :type destinatario: long
        :type item: long
        :type local: long
        :type preco_venda: float
        :type total: float
        :type nf_propria: bool
        :type nf_devolucao: bool
        :type endereco: long
        :type grade_id: long
        :type tributacao: list
        :type context: dict
        :return: valor da grade para ser mostrada no formulario
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(estoque_movimento_itens, self).estmvi_onchange_item_movimento(
            cr, uid, ids, empresa, natureza, nf_tipo, destinatario, item, local, preco_venda, total,
            nf_propria, nf_devolucao, endereco, context=context)
        codigo_barra = self.pool.get('estoque.produto').browse(
            cr, uid, item, context=context).codigo_barra.zfill(14)

        sql = """
        -- noinspection SqlDialectInspection
        SELECT grade_id
        FROM   automacao_codigo_barra
        WHERE  produto_id = %(item)d
          AND  codigo_barra = '%(codigo_barra)s'
        """ % locals()
        cr.execute(sql)
        grade_id = cr.fetchall()

        if grade_id.__len__() == 1:
            res['value']['grade_id'] = grade_id[0]

        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
