# -*- coding: utf-8 -*-

from openerp.osv import osv
from datetime import date


class estoque_preco_venda(osv.Model):
    """
    Heranca da classe ´estoque_preco_venda´.
    """

    _inherit = 'estoque.preco.venda'

    def get_preco_venda_automacao(self, cr, uid, dict_preco, epc_company_id=None,
                        epc_cliente_id=None, epc_grade_ids=[], context=None):
        """
        FUNÇÃO FAZ HERANÇA PARA O ESTOQUE.PRECO.VENDA
        BUSCA OS VALORES E ATUALIZA OS PREÇOS CASO HAJA PROMOÇÕES.
        O metodo retorna em cada dicionario dos produtos, caso a grade tenha sido informada, a
        chave representando o id da grade e seu respectivo valor de venda promocional. Para as
        demais grades do produto (as que nao estao em promocao) tem a chave 'preco_venda' que eh o
        preco de venda geral do produto
        """
        context = context and context.copy() or {}
        hoje = str(date.today())
        epc_produto_ids = dict_preco.keys()
        super(estoque_preco_venda, self).get_preco_venda_automacao(cr, uid, dict_preco, epc_company_id,
                                                               epc_cliente_id, epc_grade_ids, context)
        res = dict_preco.copy()
        if isinstance(epc_produto_ids, (int, long)):
            epc_produto_ids = [epc_produto_ids]
        elif isinstance(epc_produto_ids, tuple):
            epc_produto_ids = list(epc_produto_ids)

        if epc_produto_ids and isinstance(epc_produto_ids, list):
            # CASO A LISTA SEJA CONVERTIDA PARA STRING, O CARACTERE L PERMANECE JUNTO AO NUMERO.
            # SE O ID RETORNADO DA TABELA DE PRODUTOS FOR UM LONG.
            # NAS DUAS LINHAS ABAIXO, CADA ID É TRANSFORMADO EM UMA STRING (QUE RETIRA O L)
            # E DEPOIS TODA A LISTA É TRANSFORMADA EM STRING.
            list_produto_ids = ','.join(map(lambda x: '%s' % x, epc_produto_ids))
            grade_ids = epc_grade_ids
            if grade_ids:
                if len(grade_ids) == 1:
                    grade_ids = str(tuple(grade_ids)).replace(',', '')
                elif len(grade_ids) > 1:
                    grade_ids = str(tuple(grade_ids))
                condicao = '''AND grade_id IN %s ''' % grade_ids
            else:
                condicao = '''AND grade_id IS NULL '''

            # A tabela de promocao (automacao_precos_promocao) pode ter sua classe correlacionada
            # (roteavel = True). Nesse caso, a empresa que deve ser usada eh a definida na
            # correlacao. Senao, devera usar a empresa em que o usuario responsavel pela operacao
            # esta logado.
            promocao_ir_model_ids = self.pool.get('ir.model').search(
                cr, uid, [('model', '=', 'automacao.precos.promocao')], context=context)
            adm_empresa_objeto_ids = self.pool.get('adm.empresa.objeto').search(
                cr, uid, [('ir_model_id', 'in', promocao_ir_model_ids)])
            if adm_empresa_objeto_ids:
                promocao_company_id = epc_company_id
            else:
                promocao_company_id = self.pool.get('res.users').browse(
                    cr, uid, uid, context).company_id.id

            sql_promocao = """
                SELECT produto_id
                     , preco_venda
                     , grade_id
                FROM automacao_precos_promocao
                WHERE data_inicio <= '{data}'
                  AND data_termino >= '{data}'
                  AND produto_id IN ({produto_ids})
                  AND company_id = {company_id}
                  {cond}
            """.format(data=hoje,
                       produto_ids=list_produto_ids,
                       company_id=promocao_company_id,
                       cond=condicao)
            cr.execute(sql_promocao)
            registro_de_promocoes = cr.dictfetchall()
            obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
            obj_estoque_embalagem = self.pool.get('estoque.embalagem')
            produto_ids = list(set(map(lambda i: i.get('produto_id'), registro_de_promocoes)))
            for produto in produto_ids:
                promocao_registros = filter(lambda i: i.get('produto_id') == produto,
                                            registro_de_promocoes)
                preco_produto = False
                for i in promocao_registros:
                    contador = 0
                    produto_id = i.get('produto_id')
                    grade_id = i.get('grade_id')
                    preco_venda = i.get('preco_venda')
                    produto_embalagem_id = obj_estoque_produto_embalagem.search(
                        cr, uid, [('produto_id', '=', produto_id)], context=context)
                    if not res.get(produto_id) or not isinstance(res.get(produto_id), dict):
                        preco_produto =  res[produto_id]
                        res[produto_id] = {'preco_venda': preco_produto}
                    if grade_id:
                        res[produto_id][grade_id] = preco_venda
                    else:
                        res[produto_id] = preco_venda
                    for x in produto_embalagem_id:
                        embalagem_id = obj_estoque_produto_embalagem.read(cr, uid, x,
                                                                          ['embalagem_id'], context=context)
                        if isinstance(embalagem_id, dict):
                            embalagem_id = embalagem_id.get('embalagem_id')
                        if isinstance(embalagem_id, (list, tuple)):
                            embalagem_id = embalagem_id[0]
                        quantidade_na_embalagem = obj_estoque_embalagem.browse(
                            cr, uid, embalagem_id).quantidade_na_embalagem
                        if quantidade_na_embalagem:
                            if contador == 0:
                                preco_venda_minimo = preco_venda
                                quantidade_na_embalagem_minimo = quantidade_na_embalagem
                                contador += 1
                            elif quantidade_na_embalagem < quantidade_na_embalagem_minimo:
                                quantidade_na_embalagem_minimo = quantidade_na_embalagem
                                preco_venda_minimo = preco_venda
                                contador += 1
                            preco = preco_venda_minimo/quantidade_na_embalagem_minimo
                            if grade_id:
                                res[produto_id][grade_id] = preco
                            else:
                                res[produto_id] = preco
                grades_ids = self.pool.get('estoque.produto.grade').search(cr, uid, [('produto_id','=',produto)])
                for g_id in grades_ids:
                    if isinstance(res.get(produto), dict) and not g_id in res.get(produto) and preco_produto:
                        res[produto_id][g_id] = preco_produto
            # quando tiver atualizando o preco da automacao a funcao termina aqui
            if context.get('automacao_atualizar_precos'):
                return res
            # se tiver dados na automacao_precos, vale o preco da automacao_precos
            acb_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'automacao.codigo.barra', context=context)
            ap_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'automacao.precos', context=context)
            produto_embalagem_ids = context.get('produto_embalagem_ids') or []
            if len(produto_embalagem_ids) == 1:
                produto_embalagem_ids = str(tuple(produto_embalagem_ids)).replace(',', '')
            elif len(produto_embalagem_ids) > 1:
                produto_embalagem_ids = str(tuple(produto_embalagem_ids))

            cond_embalagem = ''
            if produto_embalagem_ids:
                cond_embalagem += """AND produto_embalagem_id IN %s """ % produto_embalagem_ids

            sql = '''
                    SELECT acb.produto_id AS produto_id
                         , acb.grade_id   AS grade_id
                         , preco_venda    AS preco_venda
                    FROM automacao_precos               AS ap
                    LEFT JOIN automacao_codigo_barra    AS acb
                        ON acb.id = ap.codigo_barra_id AND ap.company_id = {ap_company_id} AND acb.active IS TRUE
                    LEFT JOIN estoque_produto_embalagem AS epe
                        ON epe.id = acb.produto_embalagem_id
                    LEFT JOIN estoque_embalagem         AS ee
                        ON ee.id = epe.embalagem_id
                    WHERE acb.produto_id IN ({list_produto_ids}) AND acb.company_id = {acb_company_id}
                      {condicao}
                      {cond_embalagem}
                    ORDER BY produto_id, grade_id, ee.quantidade_na_embalagem
                  '''.format(ap_company_id=ap_company_id,
                             list_produto_ids=list_produto_ids,
                             acb_company_id=acb_company_id,
                             condicao=condicao if grade_ids else '',
                             cond_embalagem=cond_embalagem)
            cr.execute(sql)
            dados_automacao = cr.dictfetchall()
            automacao_produto_grade_ids = list()
            # Se nao foi informado os produto_embalagem_ids, trara sempre o preco de venda da
            # embalagem com menor quantidade.
            for i in dados_automacao:
                produto_grade = '%s-%s' % (i.get('produto_id'), i.get('grade_id'))
                if produto_grade in automacao_produto_grade_ids:
                    continue
                if not produto_embalagem_ids:
                    automacao_produto_grade_ids.append(produto_grade)
                if grade_ids:  # se a grade foi passada, o id dela deve entrar em 'res'
                    if not isinstance(res.get(i.get('produto_id')), dict):
                        res[i.get('produto_id')] = {'preco_venda': res.get(i.get('produto_id'))}
                    res[i.get('produto_id')][i.get('grade_id')] = i.get('preco_venda')
                else:
                    res[i.get('produto_id')] = {'preco_venda': i.get('preco_venda')}
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
