# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class automacao_codigo_barra(osv.Model):
    """
    Tabela de codigos de barra
    """
    from funcoes import _autcba_get_dun, _autcba_validar_codigo

    _rec_name = 'produto_id'
    _name = 'automacao.codigo.barra'
    _roteavel = True
    _columns = {
        'write_date': fields.datetime('Data ultima alteracao', select=True),
        'active': fields.boolean('Ativo'),
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="cascade", required=True, select=True),
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', ondelete="cascade",
                                    required=True, select=True,
                                    domain="[('produto_id', '=', produto_id)]"),  # Id da grade
        'produto_embalagem_id': fields.many2one("estoque.produto.embalagem", 'Embalagem',
                                                domain="[('produto_id', '=', produto_id)]",
                                                ondelete="restrict", required=True, select=True), # id da embalagem pedida
        'emite_etiqueta': fields.boolean('Emite etiqueta'), # Flag que indica se e para emitir etiqueta de codigo de barra deste produto.
        'ultima_data_movimentacao': fields.date('Ultima data movimentacao'), # Data da ultima movimentacao deste codigo de barra. Sempre que houver compra ou venda de item utilizando este codigo de barra, esta data devera ser atualizada.
        'codigo_barra': fields.char('Codigo barra', size=14, required=True, select=True), # Codigo de barras do produto
        'peso_embalagem': fields.float('Peso embalagem', digits=(5, 3)), # Peso da embalagem do produto\n
        'precos_ids': fields.one2many('automacao.precos', 'codigo_barra_id', 'Precos'),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # id da empresa\n
        'dun': fields.function(_autcba_get_dun, method=True, string='DUN', type='boolean', store=False,
                               help='Verifica se o codigo de barra e do tipo DUN ( 14 caracteres )'),
        'autcba_modelo_etiqueta': fields.integer('Modelo de etiqueta'),
        'autcba_tipo_barra': fields.selection(
            [('08', 'GTIN-8'), ('12', 'GTIN-12'), ('13', 'GTIN-13'), ('14', 'GTIN-14'),
             ('OT', 'Outro')], 'Tipo', size=2, required=True),
        'autcba_usa_digito': fields.boolean('Usa digito verificador'),
        'autcba_uni_preco': fields.char(_('Unidade p/ Preco'), size=10,
                                        help=_("Informe a unidade para preco por unidade, ex.: "
                                               "metro, quilo, litro, etc.")),
        'autcba_fator_preco': fields.float(_('Fator para Preco'),
                                           help=_("Informe um fator para dividir o preco de "
                                                  "venda")),
    }
    _defaults = {
        'active': lambda *a: True,
        'emite_etiqueta': lambda *a: True,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'autcba_tipo_barra': '13',
        'autcba_usa_digito': True,
    }
    _sql_constraints = [
        ('codigo_barra_uniq', 'unique (codigo_barra, company_id)',
         'Codigo de barra ja utilizado por outro produto na empresa!'.decode('utf-8')),
    ]

    _constraints = [
        (_autcba_validar_codigo, 'Codigo de barras incorreto.', ['codigo_barra'])
    ]
