# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _

class automacao_precos_promocao(osv.Model):
    """
    Tabela de precos promocionais.
    Utilizada em conjunto com a tabela automacao_precos

    # prefixo: autppr_
    """
    from funcoes import _autppr_check_data

    _name = 'automacao.precos.promocao'
    _description = 'Precos promocionais'
    _roteavel = True
    _columns = {
        'write_uid': fields.many2one('res.users', 'Usuario', readonly=True, ondelete="restrict"),
        'write_date': fields.datetime('Data atualizacao', readonly=True),
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="cascade",
                                      required=True, select=True),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade', select=True,
                                    domain="[('produto_id', '=', produto_id)]",
                                    help='Deixe em branco para que a promocao tenha efeito em todas'
                                         ' as grades, ou informe uma grade especifica'),
        'data_inicio': fields.date('Data inicio', required=True), # Data de inicio da promocao
        'data_termino': fields.date('Data termino', required=True), # Data de termino da promocao
        'preco_venda': fields.float('Preco venda', digits=(15, 2), required=True), # Preco de venda promocional
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True),
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }

    _constraints = [(_autppr_check_data, 'Promocao ja cadastrada para o produto no periodo', [])]
    _order = 'data_inicio DESC'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('preco_venda') <= 0:
            raise osv.except_osv(_('Atencao'), _('O preco de venda deve ser maior que zero.'))
        return super(automacao_precos_promocao,self).create(cr, uid, vals, context=context)

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'preco_venda' in vals:
            if vals['preco_venda'] <= 0:
                raise osv.except_osv(_('Atencao'), _('O preco de venda deve ser maior que zero.'))
        return super(automacao_precos_promocao,self).write(cr, uid, ids, vals, context=context)
