# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from datetime import datetime, date
from dateutil.parser import parse
from openerp.tools.translate import _
from openerp.addons.automacao.model.funcoes import _autppr_get_local_default


class automacao_atualizar_precos_osv(osv.TransientModel):
    _name = "automacao.atualizar.precos.osv"
    _columns = {
        'informacao': fields.text(''),
        'produto_id': fields.many2one('estoque.produto', 'Produto'),
        'state': fields.selection([('N', 'Novo'), ('P', 'Pronto')], string="Estado",
                                  readonly=False),
        'local_ids': fields.many2many('estoque.local', 'local_atualiza_preco', 'atualiza_preco_id',
                                      'local_id', 'Locais', required=True),
    }
    _defaults = {
        'state': lambda *a: 'N',
        'local_ids': _autppr_get_local_default,
    }

    def button_atualizar(self, cr, uid, ids, context=None):
        """
        Funcao para autalizar o preco de venda (estoque.preco.venda) do produto para a tabela de preco do automacao (automacao.precos)
        """
        context = context and context.copy() or {}
        context['automacao_atualizar_precos'] = True # este contexto faz com que get_preco_venda busque o preco em apenas
        qtd_produtos_atualizados = 0
        qtd_produtos_criados = 0
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_preco_venda = self.pool.get('estoque.preco.venda')
        obj_automacao_preco = self.pool.get('automacao.precos')
        dados_tela = self.read(cr, uid, ids[0], ['produto_id', 'local_ids'], load='_classic_write',
                               context=context)

        data_hoje = datetime.now().date()
        acb_company_id = obj_adm_empresa_objeto.getEmpresa(
            cr, uid, 'automacao.codigo.barra', context=context)
        app_company_id = obj_adm_empresa_objeto.getEmpresa(
            cr, uid, 'automacao.precos.promocao', context=context)
        ap_company_id = obj_adm_empresa_objeto.getEmpresa(
            cr, uid, 'automacao.precos', context=context)
        locacao_company_id = obj_adm_empresa_objeto.getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context)
        preco_venda_company_id = obj_adm_empresa_objeto.getEmpresa(
            cr, uid, 'estoque.preco.venda', context=context)
        local_ids = ','.join(map(lambda x: str(x), dados_tela.get('local_ids')))
        ep_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'estoque.produto',
                                                          context=context)
        cond_produto = ''
        if dados_tela.get('produto_id'):
            cond_produto = "ep.id = %s AND " % dados_tela.get('produto_id')

        left_join_correlacao = cond_estcorrecao = ""
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'estoquecorrecao',
                                                                    context=context):
            left_join_correlacao  = "LEFT  JOIN estoquecorrecao_produto_cliente AS epc " \
                                    "ON epc.grade_id = acb.grade_id OR epc.produto_id = ep.id"
            cond_estcorrecao = """OR ((epc.grade_id = acb.grade_id
                                    OR epc.produto_id = ep.id) AND epc.cliente_id IS NULL)"""

        sql = """SELECT ep.id                      AS produto_id,
                        ep.codigo                  AS codigo,
                        ep.name                    AS nome_produto,
                        ep.ncm_id                  AS ncm_id,
                        acb.id                     AS codigo_barra_id,
                        acb.grade_id               AS grade_id,
                        ap.id                      AS ap_id,
                        ap.preco_venda             AS preco_venda,
                        ee.quantidade_na_embalagem AS quantidade_na_embalagem
                 FROM automacao_codigo_barra          AS acb
                 LEFT  JOIN estoque_produto           AS ep  ON ep.id = acb.produto_id
                 LEFT  JOIN estoque_preco_venda       AS epv ON epv.produto_id = ep.id
                 LEFT  JOIN automacao_precos          AS ap  ON ap.codigo_barra_id = acb.id AND ap.company_id = {ap_company_id}
                 LEFT  JOIN estoque_produto_embalagem AS epe ON epe.id = acb.produto_embalagem_id
                 LEFT  JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id
                 INNER JOIN estoque_local_locacao     AS ell ON ell.produto_id = ep.id
                 LEFT  JOIN automacao_precos_promocao AS app ON app.grade_id = acb.grade_id AND app.company_id = {app_company_id}
                 {left_join_correlacao}
                 -- o teste de nulo serve para quando o item eh novo e nao tem automacao_preco ainda
                 WHERE {cond_produto} ((epv.create_date > ap.data_atualizacao OR ap.data_atualizacao IS NULL)
                                    OR app.data_inicio <= '{data_hoje}'
                                    {cond_estcorrecao})
                   AND ell.local_id in ({local_ids})
                   AND ell.company_id = {loc_company_id}
                   AND ep.company_id = {ep_company_id}
                   AND acb.company_id = {acb_company_id}
                   AND epv.create_date IS NOT NULL
                GROUP BY ep.id,
                         ep.codigo,
                         ep.name,
                         ep.ncm_id,
                         acb.id,
                         acb.grade_id,
                         ap.id,
                         ap.preco_venda,
                         ee.quantidade_na_embalagem
        """.format(left_join_correlacao=left_join_correlacao,
                   cond_estcorrecao=cond_estcorrecao,
                   cond_produto=cond_produto,
                   data_hoje=data_hoje,
                   local_ids=local_ids,
                   loc_company_id=locacao_company_id,
                   ep_company_id=ep_company_id,
                   acb_company_id=acb_company_id,
                   ap_company_id=ap_company_id,
                   app_company_id=app_company_id)
        sql = self.get_query_produto_dados(cr, uid, sql, context)
        cr.execute(sql)
        dados_produto = cr.dictfetchall()
        if not dados_produto:
            log = _('PRECOS CRIADOS: 0 \nPRECOS ATUALIZADOS: 0')
            self.write(cr, uid, ids, {'informacao': log, 'state': 'P'}, context=context)
            return True

        grade_ids = list(set(map(lambda i: i.get('grade_id'), dados_produto)))
        cod_barra_produto_ids = list(set(map(lambda i: i.get('produto_id'),
                                             dados_produto)))
        dict_produto_preco = obj_preco_venda.get_preco_venda(
            cr, uid, cod_barra_produto_ids, epc_company_id=preco_venda_company_id,
            epc_grade_ids=grade_ids, context=context)
        hoje = datetime.now()
        log_erro = []
        for dado in dados_produto:
            try:
                produto_id = dado.get('produto_id')
            except:
                continue
            grade_id = dado.get('grade_id')
            automacao_preco = dado.get('preco_venda')
            dados_preco_venda = dict_produto_preco.get(produto_id) or {}
            produto_preco = dados_preco_venda.get(grade_id) or dados_preco_venda.get('preco_venda')
            codigo = dado.get('codigo') or ''
            name = dado.get('name') or ''
            codigo_nome_item = '%s - %s' % (codigo, name)
            #Alteracao realizada no chamado 85226 que pede para que o preco seja
            # multiplicado pela quantidade da embalagem cadastrada no codigo de barras
            if not dado.get('quantidade_na_embalagem'):
                log_erro.append('%s%s%s' % (_('O produto '), codigo_nome_item, _(
                    ' possui embalagem sem quantidade.\n')))
                continue

            produto_preco = produto_preco * dado.get('quantidade_na_embalagem')
            # verifica divergencias
            if not dado.get('ncm_id'):
                log_erro.append('%s%s%s' % (_('O produto '), codigo_nome_item, _(
                    ' nao possui NCM cadastrado\n')))
            if not produto_preco > 0:
                log_erro.append('%s%s%s' % (_('O produto '), codigo_nome_item, _(
                    ' esta com o preco invalido. O preco deve ser maior que zero.\n')))

            # havia casos em que o preco na altomacao eh zero, entao tentava criar o preco novamente
            # e dava erro de constrants
            if automacao_preco is False or automacao_preco is None:
                qtd_produtos_criados += 1
                obj_automacao_preco.create(
                    cr, uid, {'codigo_barra_id': dado.get('codigo_barra_id'),
                              'preco_venda': produto_preco,
                              'data_atualizacao': hoje,
                              'company_id': ap_company_id,
                              'autpre_ja_exportou': False}, context)
            else:
                alteracao_preco = round(automacao_preco, 2) != round(produto_preco, 2)

                if alteracao_preco:
                    qtd_produtos_atualizados = qtd_produtos_atualizados + 1 \
                        if alteracao_preco else qtd_produtos_atualizados
                    vals_automacao_preco = {'preco_venda': round(produto_preco, 2),
                                            'data_atualizacao': hoje,
                                            'ja_listou': False,
                                            'autpre_ja_exportou': False,
                                            }
                    obj_automacao_preco.write(cr, uid, dado.get('ap_id') or [],
                                              vals_automacao_preco, context)

        if log_erro:
            log_erro = _('\n Divergencias:\n') + ''.join(log_erro)
        log = _('PRECOS CRIADOS: %d \
              \nPRECOS ATUALIZADOS: %d\n%s') % (
        qtd_produtos_criados, qtd_produtos_atualizados,  log_erro or '')

        self.write(cr, uid, ids, {'informacao': log, 'state': 'P'}, context=context)
        return True

    def get_query_produto_dados(self, cr, uid, sql, context=None):
        """
        O metodo ajusta a query para a busca dos dados dos produtos.
        OBS.: A query e alterada em outros modulos onde a instalacao desses interfere no modo de
        busca dos produtos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param sql: codigo SQL para busca dos produtos.
        :type sql: str
        :param context: dados contextuais.
         :type context: dict
        :return: codigo SQL
        :rtype: str
        """
        context = context and context.copy() or {}
        return sql
