# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
import base64
from SigUtils.numero import getNumeroEditado
import subprocess
import time
from openerp.tools.translate import _
from dateutil.parser import parse


class wizard_impressao_etiqueta(osv.TransientModel):

    def get_estoque_parametro_automacao_impressora(self, cr, uid, context=None):
        context = context and context.copy() or {}
        parametro_dados = self.pool.get('estoque.parametro').estpar_get_fields(
            cr, uid, fields_to_read=['estpar_impressora_termica'], context=context
        )
        if isinstance(parametro_dados, list):
            parametro_dados = parametro_dados[0]
        impressora = parametro_dados.get('estpar_impressora_termica')
        return impressora

    def get_estoque_parametro_automacao_servidor(self, cr, uid, context=None):
        context = context and context.copy() or {}
        parametro_dados = self.pool.get('estoque.parametro').estpar_get_fields(
            cr, uid, fields_to_read=['estpar_impressora_termica_servidor'], context=context
        )
        if isinstance(parametro_dados, list):
            parametro_dados = parametro_dados[0]
        servidor = parametro_dados.get('estpar_impressora_termica_servidor')
        return servidor

    _name = 'wizard.impressao.etiqueta'
    _columns = {
        'modelo_etiqueta_id': fields.many2one('automacao.modelo.etiqueta', 'Modelo etiqueta',
                                              required=True),
        'check_codigo_bar': fields.boolean('Usar codigo de barras'),
        'wizime_codigo_barra': fields.char('Codigo barra', size=14, select=True),
        'data_alteracao': fields.date('Data alteracao'),
        'impressao_etiqueta_itens_ids': fields.one2many('wizard.impressao.etiqueta.itens',
                                                        'wizard_impressao_etiqueta_id', 'Itens'),
        'impressora_termica': fields.char('Impressora', size=64),
        'impressora_termica_servidor': fields.char('Servidor', size=64),
        'reimprimir': fields.selection([('1', _('Todas')),
                                        ('2', _('Impressas')),
                                        ('3', _('Nao Impressas'))], 'Opcoes de Impressao',
                                       required=True),
        'qtd_item': fields.integer('Quantidade'),
        }
    _defaults = {
        'reimprimir': '3',
        'impressora_termica': get_estoque_parametro_automacao_impressora,
        'impressora_termica_servidor': get_estoque_parametro_automacao_servidor,
        'qtd_item': 1,
    }

    def button_check_codigo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        ids = ids[0] if isinstance(ids, list) else ids
        brw_impressao =  self.browse(cr, uid, ids)
        if brw_impressao.check_codigo_bar:
            self.write(cr, uid, ids, {'check_codigo_bar': False}, context=context)
        else:
            self.write(cr, uid, ids, {'check_codigo_bar': True}, context=context)
        return True

    def onchange_wizime_codigo_barra(self, cr, uid, ids, codigo_barra, impressao_etiqueta_itens_ids, context=None):
        context = context and context.copy() or {}
        if not codigo_barra:
            return {}
        res = {}
        codigo_ids = self.pool.get('automacao.codigo.barra').search(cr, uid, [('codigo_barra','=',codigo_barra)],
                                                                    context=context)
        if not codigo_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao existe produto para este codigo de barras!'))
        if isinstance(codigo_ids, list):
            codigo_ids = codigo_ids[0]
        brw_codigo = self.pool.get('automacao.codigo.barra').browse(cr, uid, codigo_ids)
        produto_id = brw_codigo.produto_id.id
        grade_id = brw_codigo.grade_id.id
        imp_etiqueta_id = ids[0] if isinstance(ids, list) else ids
        vals_itens = {'wizard_impressao_etiqueta_id': imp_etiqueta_id,
                      'produto_id': produto_id,
                      'grade_id': grade_id,
                      'qtd_item': 1,
                      'codigo_barra': codigo_barra,
                      'emite_etiqueta': brw_codigo.emite_etiqueta,
        }
        item_ids = self.pool.get('wizard.impressao.etiqueta.itens').search(cr, uid, [('wizard_impressao_etiqueta_id','=',imp_etiqueta_id),
                                                                                     ('produto_id','=',produto_id),
                                                                                     ('grade_id','=',grade_id)],
                                                                           context=context)
        if item_ids:
            item_id = item_ids[0]
            quantidade = self.pool.get('wizard.impressao.etiqueta.itens').browse(cr, uid, item_id, context=context).qtd_item + 1
            self.pool.get('wizard.impressao.etiqueta.itens').write(cr, uid, item_id, {'qtd_item': quantidade},
                                                                   context=context)
        else:
            impressao_etiqueta_itens_ids.append((0, 0, vals_itens))
        res['impressao_etiqueta_itens_ids'] = impressao_etiqueta_itens_ids
        res['wizime_codigo_barra'] = ''
        return {'value': res}

    def onchange_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'data_alteracao': False}}
        if produto_id:
            obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
            grade_ids = obj_estoque_produto_grade.search(cr, uid, [('produto_id', '=', produto_id)], context=context)
            if len(grade_ids) == 1:
                res = {'value': {'data_alteracao': False, 'grade_id': grade_ids[0]}}
            else:
                res = {'value': {'data_alteracao': False, 'grade_id': False}}
        else:
            res = {'value': {'data_alteracao': False, 'grade_id': False}}
        return res

    def onchange_data_alteracao(self, cr, uid, ids, data_alteracao, context=None):
        context = context and context.copy() or {}
        res = {'value': {'produto_id': False, 'grade_id': False}}
        return res

    def button_incluir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_wizard_impressao_etiqueta_itens = self.pool.get('wizard.impressao.etiqueta.itens')
        obj_automacao_precos = self.pool.get('automacao.precos')
        browse_impressao_etiqueta = self.browse(cr, uid, ids[0])
        data_alteracao = browse_impressao_etiqueta.data_alteracao or False
        qtd_item = browse_impressao_etiqueta.qtd_item
        if qtd_item <= 0:
            raise osv.except_osv(_('Atencao'),
                                 _("O valor do campo 'Quantidade' deve ser maior que zero!"))
        if not data_alteracao:
            return True

        reimprimir = browse_impressao_etiqueta.reimprimir
        args = [('data_atualizacao', '>=', data_alteracao + ' 00:00:00'),
                ('data_atualizacao', '<=', data_alteracao + ' 23:59:59')]
        if reimprimir == '2':
            args.append(('autpre_ja_emitiu_etiqueta', '=', True))

        elif reimprimir == '3':
            args.append(('autpre_ja_emitiu_etiqueta', '=', False))

        automacao_precos_ids = obj_automacao_precos.search(cr, uid, args, context=context)
        if not automacao_precos_ids:
            return True

        campos = {'codigo_barra_id': ['produto_id',  'grade_id', 'codigo_barra', 'emite_etiqueta',
                                      'active']}
        dados_aut = self.pool.get('adm.utils').admutl_read_browse(cr, uid, automacao_precos_ids,
                                                              'automacao.precos', campos, context)
        dados_aut = filter(lambda cb: cb.get('codigo_barra_id').get('active'), dados_aut)
        produto_ids =[]
        grade_ids = []
        for i in dados_aut:
            produto_ids.append(i.get('codigo_barra_id').get('produto_id'))
            grade_ids.append(i.get('codigo_barra_id').get('grade_id'))

        dict_preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid,
                                              produto_ids, epc_grade_ids=grade_ids, context=context)
        for x in dados_aut:
            produto_id = x.get('codigo_barra_id').get('produto_id')
            grade_id = x.get('codigo_barra_id').get('grade_id')
            dados_preco_venda = dict_preco_venda.get(produto_id) or {}
            preco_venda = dados_preco_venda.get(grade_id) if grade_id in dados_preco_venda \
                                                        else dados_preco_venda.get('preco_venda')
            # itens sem preco nao geram etiquetas
            if not preco_venda:
                continue

            itens_ids = obj_wizard_impressao_etiqueta_itens.search(cr, uid, [
                ('wizard_impressao_etiqueta_id', '=', ids[0]),
                ('produto_id', '=', produto_id),
                ('grade_id', '=', grade_id),
                ('codigo_barra', '=', x.get('codigo_barra_id').get('codigo_barra'))],
                                                                   context=context)
            if itens_ids:
                continue
            vals = {
                'wizard_impressao_etiqueta_id': ids[0],
                'produto_id': produto_id,
                'grade_id': grade_id,
                'qtd_item': qtd_item,
                'codigo_barra': x.get('codigo_barra_id').get('codigo_barra'),
                'emite_etiqueta': True if x.get('codigo_barra_id').get('emite_etiqueta') else False
                }
            if browse_impressao_etiqueta.wizime_codigo_barra:
                vals['codigo_barra'] = browse_impressao_etiqueta.wizime_codigo_barra

            obj_wizard_impressao_etiqueta_itens.create(cr, uid, vals, context=context)

        self.write(cr, uid, ids, {'produto_id': False, 'grade_id': False, 'qtd_item': 1},
                   context=context)
        return True

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        obj_automacao_precos = self.pool.get('automacao.precos')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        load = '_classic_write'
        # Foi trocado de 'browse' para 'read' para melhorar performance
        dados_imprimir_etiqueta = self.read(cr, uid, ids[0], [], context=context, load=load)
        impressora = dados_imprimir_etiqueta.get('impressora_termica')
        servidor = dados_imprimir_etiqueta.get('impressora_termica_servidor')
        reimprimir = dados_imprimir_etiqueta.get('reimprimir')
        if not impressora:
            raise osv.except_osv(_('Atencao'),
                                 _('Informa uma impressora para realizar a impressao!'))

        modelo_etiqueta_id = dados_imprimir_etiqueta.get('modelo_etiqueta_id')
        arquivo = self.pool.get('automacao.modelo.etiqueta').read(cr, uid, modelo_etiqueta_id,
                                ['autmet_arquivo'], context=context).get('autmet_arquivo') or False
        if not arquivo:
            raise osv.except_osv(_('Atencao!'),
                                 _('Modelo de etiqueta nao possui arquivo cadastrado!'))

        modelo_etiqueta_str = base64.b64decode(arquivo)
        etiqueta_itens_ids = dados_imprimir_etiqueta.get('impressao_etiqueta_itens_ids') or []
        if not etiqueta_itens_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha nada a listar para a selecao informada!'))

        arquivo_diretorio = "/tmp/etiqueta.pbt"
        if modelo_etiqueta_str[0] == 'N' and modelo_etiqueta_str[-3] == 'N':
            arquivo_diretorio = "/tmp/etiqueta.ejf"

        # Foi trocado de 'browse' para 'read' para melhorar performance
        dados_itens = self.pool.get('wizard.impressao.etiqueta.itens').read(cr, uid,
                                                  etiqueta_itens_ids, [],context=context, load=load)
        produto_ids = []
        grade_ids = []
        for i in dados_itens:
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))

        dados_produtos = {}
        campos = ['name', 'preco_venda', 'codigo', 'digito']
        map(lambda x: dados_produtos.update({x.get('id'): x}),
            self.pool.get('estoque.produto').read(cr, uid, produto_ids, campos, context=context))

        campos = ['descricao_resumida', 'descricao', 'grade', 'codigo_no_fabricante', 'name',
                  'produto_codigo_fornecedor_ids']
        dados_grades = {}
        map(lambda x: dados_grades.update({x.get('id'): x}),
            self.pool.get('estoque.produto.grade').read(cr, uid, grade_ids, campos,
                                                        context=context))
        preco_ids = []
        texto = []
        for item in dados_itens:
            dados_produto = dados_produtos.get(item.get('produto_id'))
            dados_grade = dados_grades.get(item.get('grade_id'))
            produto_id = dados_produto.get('id')
            grade_id = dados_grade.get('id')
            qtd_item = item.get('qtd_item')
            if not qtd_item:
                raise osv.except_osv(_('Atencao!'),
                                     _("O produto '%s' nao possui quantidade!") %
                                        dados_produto.get('name'))

            descricao_grade_produto = dados_grade.get('descricao_resumida') or ''
            descricao_grade_produto = descricao_grade_produto.replace('"', '')

            for i in range(0, qtd_item):
                args_automacao_cod_barra = [('produto_id', '=', produto_id),
                                            ('grade_id', '=', grade_id),
                                            ('company_id', '=', company_id)]
                if item.get('codigo_barra'):
                    args_automacao_cod_barra.append(('codigo_barra', '=', item.get('codigo_barra')))
                codigo_barra_ids = obj_automacao_codigo_barra.search(
                    cr, uid, args_automacao_cod_barra, context=context)
                if not codigo_barra_ids:
                    raise osv.except_osv(_('Atencao!'),
                                         _("O produto '%s' nao possui codigo de barras!") %
                                            dados_produto.get('name'))
                brw_automacao_codigo_barra = obj_automacao_codigo_barra.browse(
                    cr, uid, codigo_barra_ids, context=context)

                for cb in brw_automacao_codigo_barra:
                    if not item.get('emite_etiqueta') and not cb.emite_etiqueta:
                        continue

                    # {DESCRICAO_PRODUTO}
                    modelo_etiqueta_str_e = modelo_etiqueta_str.replace('{DESCRICAO_PRODUTO}',
                                                                        descricao_grade_produto)

                    # {EMBALAGEM}
                    embalagem = ''
                    produto_embalagem_id = list()
                    brw_embalagem_id = False
                    if cb.produto_embalagem_id:
                        embalagem_id = cb.produto_embalagem_id.embalagem_id
                        if embalagem_id:
                            embalagem = embalagem_id.name or ''
                            brw_embalagem_id = embalagem_id
                        produto_embalagem_id = [cb.produto_embalagem_id.id]

                    # {CODIGO_DE_BARRA}
                    codigo_barra = cb.codigo_barra or ''
                    if len(codigo_barra) > 12:
                        codigo_barra = codigo_barra[1:13]
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace('{CODIGO_DE_BARRA}',
                                                                          codigo_barra)
                    args = [('codigo_barra_id', '=', cb.id)]
                    preco_id = obj_automacao_precos.search(cr, uid, args,context=context)
                    if not preco_id:
                        raise osv.except_osv(_('Atencao!'),
                                             _("O produto '%s' nao possui preco na automacao!") %
                                             dados_produto.get('name'))
                    if dados_imprimir_etiqueta.get('data_alteracao'):
                        if reimprimir == '2':
                            args.append(('autpre_ja_emitiu_etiqueta', '=', True))

                        elif reimprimir == '3':
                            args.append(('autpre_ja_emitiu_etiqueta', '=', False))

                    preco_id = obj_automacao_precos.search(cr, uid, args,context=context)
                    if not preco_id:
                        continue
                    if i == qtd_item-1:
                        preco_ids.extend(preco_id)

                    # {VALOR_DESCONTO}
                    preco_cadastro = dados_produto.get('preco_venda') or 0.0
                    context['produto_embalagem_ids'] = produto_embalagem_id
                    dict_preco_venda = obj_estoque_preco_venda.get_preco_venda(
                        cr, uid, produto_id, epc_company_id=company_id,
                        epc_grade_ids=grade_id, context=context)
                    dados_preco_venda = dict_preco_venda.get(produto_id) or {}
                    preco_venda = dados_preco_venda.get(grade_id) if \
                        grade_id in dados_preco_venda else dados_preco_venda.get('preco_venda')
                    valor_desconto = preco_cadastro - preco_venda
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{VALOR_DESCONTO}', getNumeroEditado(valor_desconto, 2))

                    # {PRECO_VENDA}
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{PRECO_VENDA}', getNumeroEditado(preco_venda, 2))

                    # {PRECO_CADASTRO}
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{PRECO_CADASTRO}', getNumeroEditado(preco_cadastro, 2))

                    # {UNI}
                    unidade = ''
                    if brw_embalagem_id:
                        unidade_id = brw_embalagem_id.unidade_id
                        if unidade_id:
                            unidade = unidade_id.unidade or ''
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace('{UNI}', unidade)

                    # {PERC_DESC}
                    percentual_desconto = 0.0
                    if preco_cadastro:
                        percentual_desconto = (preco_cadastro - preco_venda) * \
                                              100 / preco_cadastro
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{PERC_DESC}', getNumeroEditado(percentual_desconto, 2))
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace('{EMBALAGEM}',
                                                                          embalagem)

                    # {DESCRICAO_COMPLETA1}
                    descricao_completa = dados_grade.get('name') or ''
                    descricao_completa = descricao_completa.replace('"', '')
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{DESCRICAO_COMPLETA1}', descricao_completa[:20])

                    # {DESCRICAO_COMPLETA2}
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{DESCRICAO_COMPLETA2}', descricao_completa[20:41])

                    # {DESCRICAO_PRODUTO1}
                    descricao_produto = dados_produto.get('name') or ''
                    descricao_produto = descricao_produto.replace('"', '')
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{DESCRICAO_PRODUTO1}', descricao_produto[:20])

                    # {DESCRICAO_PRODUTO2}
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{DESCRICAO_PRODUTO2}', descricao_produto[20:41])

                    # {CODIGO_INTERNO}
                    codigo_produto = dados_produto.get('codigo') or ''
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace('{CODIGO_INTERNO}',
                                                                          codigo_produto)

                    # {GRADE}
                    grade = dados_grade.get('grade') or ''
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace('{GRADE}', grade)

                    # {GRADE_DESCRICAO}
                    grade = dados_grade.get('descricao') or ''
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{GRADE_DESCRICAO}', grade[:40])

                    # {CODIGO_FABRICANTE}
                    codigo_fabricante = ''
                    if dados_grade.get('codigo_no_fabricante'):
                        codigo_fabricante = dados_grade.get('codigo_no_fabricante')
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{CODIGO_FABRICANTE}', codigo_fabricante)

                    # {CODIGO_FORNECEDOR}
                    codigo_fornecedor = ''
                    if dados_grade.get('produto_codigo_fornecedor_ids'):
                        fornecedor_id = dados_grade.get('produto_codigo_fornecedor_ids')[0]
                        codigo_fornecedor = self.pool.get('estoque.produto.codigo.fornecedor').read(
                                            cr, uid, fornecedor_id, ['codigo_fornecedor'],
                                            context=context).get('codigo_fornecedor') or ''

                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{CODIGO_FORNECEDOR}', codigo_fornecedor)
                    #, digito do produto
                    digito = dados_produto.get('digito') or ''
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace('{#}', digito)

                    # {DT_EMISSAO}
                    data_alteracao = time.strftime('%d/%m/%Y')
                    if dados_imprimir_etiqueta.get('data_alteracao'):
                        data_alteracao = parse(
                            dados_imprimir_etiqueta.get('data_alteracao')).strftime('%d/%m/%Y')
                    modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                        '{DT_EMISSAO}', data_alteracao)

                    # {UNIDADE_PRECO} e {PRECO_UNIDADE}
                    if cb.autcba_uni_preco and cb.autcba_fator_preco:
                        modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                            '{UNIDADE_PRECO}', cb.autcba_uni_preco)
                        modelo_etiqueta_str_e = modelo_etiqueta_str_e.replace(
                            '{PRECO_UNIDADE}',
                            getNumeroEditado((preco_venda / cb.autcba_fator_preco), 2))

                    texto.append(modelo_etiqueta_str_e)

        if preco_ids:
            obj_automacao_precos.write(cr, uid, preco_ids, {'autpre_ja_emitiu_etiqueta': True},
                                       context=context)
        p = open(arquivo_diretorio, 'wb')
        p.write(self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid, ''.join(texto)))
        p.close()
        nome_impressora = "-d" + impressora
        host = '-h%s' % servidor if servidor else ''

        try:
            if host:
                subprocess.call(["lp", "-n1", nome_impressora, host, arquivo_diretorio])
            else:
                subprocess.call(["lp", "-n1", nome_impressora, arquivo_diretorio])
        except OSError:
            osv.except_osv(_('Atencao'),
                           _('Impressora nao encontrada, verifique se as configuracoes de '
                             'impressao estao corretas.'))

        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'automacao',
                                                        'wizard_impressao_etiqueta_form')
        view_id = reference and reference[1] or False
        return {
            'name': _('Impressao de Etiquetas'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window'
        }

