# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from SigUtils.formatastring import unicode_to_ascii
f_txt = lambda txt, tam: unicode_to_ascii(txt).ljust(tam)[:tam]
f_num = lambda x, t, d=0: str(long(round(float(x) * 10 ** d, 2))).rjust(t, '0')[-t:] or False


class balanca_util(osv.Model):
    # prefixo: balutl_
    _name = 'balanca.util'
    _auto = False

    def balutl_processa_dado_exportacao_balanca(self, cr, uid, balanca, tipo_exportacao, dados,
                                                context=None):
        """
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param balanca: 'T', 'T6' (MGV6) e 'T7' (MGV7) para Toledo e 'F' para Filizola
        :param tipo_exportacao: 'I' para itens, 'N' para nutricional, 'S' para setor e 'R' para
        receita
        :param dados: Lista de dicionarios com os dados para exportacao
        :return: String formatada no layout da balanca e tipo de exportacao definidos
        """
        context = context and context.copy() or {}
        list_codigo_barra = []

        def _balutl_get_codigo_barra(cb):
            if cb[1:] not in list_codigo_barra:
                list_codigo_barra.append(cb[1:])
                cb = cb[-6:]
            else:
                cb = cb[-7:-1]
            return cb

        s_arq = []
        if balanca == 'F':
            if tipo_exportacao == 'I':
                for x in dados:
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    peso_unidade = x.get('peso_unidade')
                    preco_venda = x.get('preco_venda')
                    validade = x.get('validade')
                    produto_name = x.get('produto_name')

                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(peso_unidade)
                    s_arq.append(f_txt(produto_name, 22))
                    s_arq.append(f_num(preco_venda, 7, 2))
                    s_arq.append(f_num(validade, 3))
                    s_arq.append('\r\n')
            elif tipo_exportacao == 'S':
                for x in dados:
                    setor_nome = x.get('setor_nome')
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    indice = x.get('indice')
                    tecla = x.get('tecla')

                    s_arq.append(f_txt(setor_nome, 12))
                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(f_num(indice, 4))
                    s_arq.append(f_num(tecla, 3))
                    s_arq.append('\r\n')
            elif tipo_exportacao == 'N':
                for x in dados:
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    nome = x.get('nome')
                    valor_calorico = x.get('valor_calorico')
                    per_valor_calorico = x.get('percentual_valor_calorico')
                    carboidratos = x.get('carboidratos')
                    per_caboidratos = x.get('percentual_carboidratos')
                    proteinas = x.get('proteinas')
                    per_proteinas = x.get('percentual_proteinas')
                    gorduras_totais = x.get('gorduras_totais')
                    per_gorduras_totais = x.get('percentual_gorduras_totais')
                    gorduras_saturadas = x.get('gorduras_saturadas')
                    per_gorduras_saturadas = x.get('percentual_gorduras_saturadas')
                    gorduras_trans = x.get('gorduras_trans')
                    per_gorduras_trans = x.get('percentual_gorduras_trans')
                    fibra_alimentar = x.get('fibra_alimentar')
                    per_fibra_alimentar = x.get('percentual_fibra_alimentar')
                    calcio = x.get('calcio')
                    per_calcio = x.get('percentual_calcio')
                    ferro = x.get('ferro')
                    per_ferro = x.get('percentual_ferro')
                    sodio = x.get('sodio')
                    per_sodio = x.get('percentual_sodio')

                    s_arq.append(f_num(long(codigo_barra), 6))
                    s_arq.append(f_txt(nome, 35))
                    s_arq.append(f_num(valor_calorico, 5))
                    s_arq.append(f_num(per_valor_calorico, 4))
                    s_arq.append(f_num(carboidratos, 5, 1))
                    s_arq.append(f_num(per_caboidratos, 4))
                    s_arq.append(f_num(proteinas, 5, 1))
                    s_arq.append(f_num(per_proteinas, 4))
                    s_arq.append(f_num(gorduras_totais, 5, 1))
                    s_arq.append(f_num(per_gorduras_totais, 4))
                    s_arq.append(f_num(gorduras_saturadas, 5, 1))
                    s_arq.append(f_num(per_gorduras_saturadas, 4))
                    s_arq.append(f_num(gorduras_trans, 5, 1))
                    s_arq.append(f_num(per_gorduras_trans, 4))
                    s_arq.append(f_num(fibra_alimentar, 5))
                    s_arq.append(f_num(per_fibra_alimentar, 4))
                    s_arq.append(calcio)
                    s_arq.append(per_calcio)
                    s_arq.append(ferro)
                    s_arq.append(per_ferro)
                    s_arq.append(f_num(sodio, 5, 1))
                    s_arq.append(f_num(per_sodio, 4))
                    s_arq.append('\r\n')
            elif tipo_exportacao == 'R':
                for x in dados:
                    reservado = x.get('reservado')
                    codigo = _balutl_get_codigo_barra(x.get('codigo'))
                    codigo_receita = x.get('codigo_receita')
                    titulo_receita = x.get('titulo_receita')
                    corpo_receita = x.get('corpo_receita')
                    limite_corpo_receita = x.get('limite_corpo_receita')

                    s_arq.append(f_txt(reservado, 12))
                    s_arq.append(f_num(codigo, 6))
                    s_arq.append(f_num(codigo_receita, 6))
                    count = 0
                    diferenca = 0
                    for corpo in corpo_receita:
                        if corpo_receita.index(corpo) == 0:
                            corpo = titulo_receita + '\r\n' + corpo
                        tamanho_corpo = len(corpo)
                        count += tamanho_corpo
                        if count > limite_corpo_receita:
                            tamanho_corpo = limite_corpo_receita - count
                            diferenca = tamanho_corpo
                            corpo = corpo[:tamanho_corpo]
                        if diferenca:
                            s_arq.append(f_txt(corpo, tamanho_corpo))
                            break
                        s_arq.append(f_txt(corpo, tamanho_corpo))
                        s_arq.append('\r\n')
                    s_arq.append('@')
                    s_arq.append('\r\n')
        elif balanca in ('T', 'T6', 'T7'):
            if tipo_exportacao == 'I' and balanca in ('T6', 'T7'):
                for x in dados:
                    setor_id = x.get('setor_id')
                    peso_unidade = x.get('peso_unidade')
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    preco_venda = x.get('preco_venda')
                    validade = x.get('validade')
                    produto_name = x.get('produto_name')

                    s_arq.append(f_num(setor_id, 2))
                    s_arq.append(peso_unidade)
                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(f_num(preco_venda, 6, 2))
                    s_arq.append(f_num(validade, 3))
                    s_arq.append(f_txt(produto_name[:25], 25))
                    s_arq.append(f_txt(produto_name[25:], 25))
                    s_arq.append(f_num(0, 6))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(f_num(validade and 1 or 0, 1))
                    s_arq.append(f_num(0, 1))
                    s_arq.append(f_num(x.get('cod_fornecedor') or 0, 4))
                    s_arq.append(f_num(0, 12))
                    s_arq.append(f_txt('', 11))
                    s_arq.append(f_txt('', 1))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_num(0, 4))
                    s_arq.append(f_txt('', 12))
                    s_arq.append(f_num(0, 6))
                    s_arq.append(f_txt('||', 2))
                    s_arq.append(f_txt(produto_name[50:], 35))
                    s_arq.append(f_txt(produto_name[85:], 35))
                    s_arq.append(f_num(0, 6))
                    s_arq.append(f_num(0, 6))
                    s_arq.append(f_num(0, 6))
                    s_arq.append('\r\n')
            elif tipo_exportacao == 'I':
                for x in dados:
                    setor_id = x.get('setor_id')
                    etiqueta_pic = x.get('etiqueta_pic')
                    peso_unidade = x.get('peso_unidade')
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    preco_venda = x.get('preco_venda')
                    validade = x.get('validade')
                    produto_name = x.get('produto_name')
                    receita_titulo = x.get('receita_titulo')
                    receita_corpo = x.get('receita_corpo')

                    s_arq.append(f_num(setor_id, 2))
                    s_arq.append(etiqueta_pic)
                    s_arq.append(peso_unidade)
                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(f_num(preco_venda, 6, 2))
                    s_arq.append(f_num(validade, 3))
                    s_arq.append(f_txt(produto_name, 25))
                    if context.get('gerar_receita'):
                        s_arq.append(f_txt(receita_titulo, 25))
                        s_arq.append(f_txt(receita_corpo, 250))
                    else:
                        s_arq.append(str().ljust(275))
                    s_arq.append('\r\n')
            elif tipo_exportacao == 'N' and balanca == 'T7':
                for x in dados:
                    reservado = x.get('reservado')
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    reservado1 = x.get('reservado1')
                    quantidade = x.get('quantidade')
                    unidade_porcao = x.get('unidade_porcao')
                    medida_inteiro = x.get('medida_inteiro')
                    medida_decimal = x.get('medida_decimal')
                    medida = x.get('medida')
                    valor_calorico = x.get('valor_calorico')
                    carboidratos = x.get('carboidratos')
                    proteinas = x.get('proteinas')
                    gorduras_totais = x.get('gorduras_totais')
                    gorduras_saturadas = x.get('gorduras_saturadas')
                    gorduras_trans = x.get('gorduras_trans')
                    fibra_alimentar = x.get('fibra_alimentar')
                    acucar_total = x.get('acucar_total')
                    acucar_adicionado = x.get('acucar_adicionado')

                    s_arq.append(reservado)
                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(reservado1)
                    s_arq.append(f_num(quantidade, 3))
                    s_arq.append(f_num(unidade_porcao, 1))
                    s_arq.append(f_num(medida_inteiro, 2))
                    s_arq.append(f_num(medida_decimal, 1))
                    s_arq.append(f_num(medida, 2))
                    s_arq.append(f_num(valor_calorico, 4))
                    s_arq.append(f_num(carboidratos, 4, 1))
                    s_arq.append(f_num(acucar_total, 4, 1))
                    s_arq.append(f_num(acucar_adicionado, 4, 1))
                    s_arq.append(f_num(proteinas, 3, 1))
                    s_arq.append(f_num(gorduras_totais, 3, 1))
                    s_arq.append(f_num(gorduras_saturadas, 3, 1))
                    s_arq.append(f_num(gorduras_trans, 3, 1))
                    s_arq.append(f_num(fibra_alimentar, 3, 1))
                    s_arq.append(f_num(fibra_alimentar, 5, 1))
                    s_arq.append('\r\n')
            elif tipo_exportacao == 'N':
                for x in dados:
                    reservado = x.get('reservado')
                    codigo_barra = _balutl_get_codigo_barra(x.get('codigo_barra'))
                    reservado1 = x.get('reservado1')
                    quantidade = x.get('quantidade')
                    unidade_porcao = x.get('unidade_porcao')
                    medida_inteiro = x.get('medida_inteiro')
                    medida_decimal = x.get('medida_decimal')
                    medida = x.get('medida')
                    valor_calorico = x.get('valor_calorico')
                    carboidratos = x.get('carboidratos')
                    proteinas = x.get('proteinas')
                    gorduras_totais = x.get('gorduras_totais')
                    gorduras_saturadas = x.get('gorduras_saturadas')
                    gorduras_trans = x.get('gorduras_trans')
                    fibra_alimentar = x.get('fibra_alimentar')

                    s_arq.append(reservado)
                    s_arq.append(f_num(codigo_barra, 6))
                    s_arq.append(reservado1)
                    s_arq.append(f_num(quantidade, 3))
                    s_arq.append(f_num(unidade_porcao, 1))
                    s_arq.append(f_num(medida_inteiro, 2))
                    s_arq.append(f_num(medida_decimal, 1))
                    s_arq.append(f_num(medida, 2))
                    s_arq.append(f_num(valor_calorico, 4))
                    s_arq.append(f_num(carboidratos, 4, 1))
                    s_arq.append(f_num(proteinas, 3, 1))
                    s_arq.append(f_num(gorduras_totais, 3, 1))
                    s_arq.append(f_num(gorduras_saturadas, 3, 1))
                    s_arq.append(f_num(gorduras_trans, 3, 1))
                    s_arq.append(f_num(fibra_alimentar, 3, 1))
                    s_arq.append(f_num(fibra_alimentar, 5, 1))
                    s_arq.append('\r\n')
        return ''.join(s_arq)
