# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class balanca_nutricionais(osv.Model):
    """
    tabela de nutricionais para emissao de etiquetas nas balancas

    prefixo: balnut_
    """
    _name = 'balanca.nutricionais'
    _description = 'balanca_nutricionais'
    _roteavel = True
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True),
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="cascade", required=True, select=True),
        'name': fields.char('Descricao', size=45, required=True), # descricao da porcao
        'valor_calorico': fields.float('Valor calorico', digits=(5, 0)), # Valor calorico da porcao em Kcal
        'percentual_valor_calorico': fields.float('Percentual valor calorico', digits=(4, 5)), # percentual de valor calorico da porcao em VD(valores diários)
        'carboidratos': fields.float('Carboidratos', digits=(5, 1)), # Quantidade de carboidratos
        'percentual_carboidratos': fields.float('Percentual carboidratos', digits=(4, 5)), # percentual de carboidratos em VD
        'proteinas': fields.float('Proteinas', digits=(5, 1)), # Quantidade de proteinas
        'percentual_proteinas': fields.float('Percentual proteinas', digits=(4, 0)), # Percentual de proteinas em VD
        'gorduras_totais': fields.float('Gorduras totais', digits=(5, 1)), # Quantidade de gorduras totais
        'percentual_gorduras_totais': fields.float('Percentual gorduras totais', digits=(4, 5)), # Percentual de gorduras totais
        'gorduras_saturadas': fields.float('Gorduras saturadas', digits=(5, 1)), # Quantidade de gorduras saturadas
        'percentual_gorduras_saturadas': fields.float('Percentual gorduras saturadas', digits=(4, 5)), # Percentual de gorduras saturadas
        'colesterol': fields.float('Colesterol', digits=(5, 5)), # Quantidade de colesterol
        'percentual_colesterol': fields.float('Percentual colesterol', digits=(4, 0)), # Percentual de colesterol
        'fibra_alimentar': fields.float('Fibra alimentar', digits=(5, 0)), # Quantidade de fibra alimentar
        'percentual_fibra_alimentar': fields.float('Percentual fibra alimentar', digits=(4, 5)), # Percentual de fibra alimentar em Valores Diarios
        'sodio': fields.float('Sodio', digits=(5, 1)), # Quantidade de sodio
        'percentual_sodio': fields.float('Percentual sodio', digits=(4, 0)), # Percentual de sodio
        'gorduras_trans': fields.float('Gorduras trans', digits=(5, 1)), # Quantidade de gorduras Trans
        'percentual_gorduras_trans': fields.float('Percentual gorduras trans', digits=(4, 0)), # Percentual de gorduras trans
        'quantidade': fields.float('Quantidade', digits=(3, 0)), # quantidade de uma porcao
        'unidade_porcao': fields.selection([('0', 'Gramas'),
                                            ('1', 'Mililitros'),
                                            ('2', 'Unidade')], 'Unidade porcao', size=1),
        'medida': fields.float('Medida', digits=(2, 0)), # Medida caseira, vide norma da anvisa RCD 390
        'medida_inteiro': fields.float('Medida inteiro', digits=(2, 0)), # Parte inteira da medida caseira
        'medida_decimal': fields.float('Medida decimal', digits=(1, 0)), # Parte decimal da medida
        'valor_acucar_total': fields.float('Acucares Totais', digits=(3, 2)),
        'valor_acucar_adicionado': fields.float('Acucares Adicionados', digits=(3, 2)),
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }
    _sql_constraints = [(
        'nutricional_produto_id_uniq',
        'unique(produto_id)',
        'Produto ja cadastrado. As informacoes nutricionais devem ser unicas para o produto.'
    )]

