# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.relativedelta import relativedelta
from datetime import date
import time


class wizard_impressao_etiqueta_interna(osv.TransientModel):
    _name = 'wizard.impressao.etiqueta.interna'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', readonly=True),
        'wimeti_codigo_barra_digito': fields.boolean('Codigo de barras com digito'),
        'wimeti_data_alteracao': fields.date('Data'),
        'wimeti_tipo_dado': fields.selection([('C', 'Cadastro completo'), ('D', u'Data alteração')],
                                             u'Seleção automática de códigos', size=1),
        'wimeti_item_ids': fields.one2many('wizard.impressao.etiqueta.interna.item',
                                           'wimpei_capa_id', 'Itens'),
    }
    _defaults = {
        'wimeti_codigo_barra_digito': lambda *a: False,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'automacao.codigo.barra', context=c),
    }

    def wimeti_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        etiqueta_interna_dados = self.read(cr, uid, ids[0] if isinstance(ids, list) else ids, [],
                                           context)
        item_ids = etiqueta_interna_dados.get('wimeti_item_ids')
        tipo_dado = etiqueta_interna_dados.get('wimeti_tipo_dado')
        data_alteracao = etiqueta_interna_dados.get('wimeti_data_alteracao')
        (item_ids and tipo_dado) and self.pool.get('wizard.impressao.etiqueta.interna.item').unlink(
            cr, uid, item_ids, context)
        obj_cod_bar = self.pool.get('automacao.codigo.barra')
        obj_automacao_precos = self.pool.get('automacao.precos')
        cod_bar_ids = []
        tipo_dado == 'C' and cod_bar_ids.extend(obj_cod_bar.search(cr, uid, [], context=context))
        automacao_precos_ids = []
        tipo_dado == 'D' and automacao_precos_ids.extend(obj_automacao_precos.search(
            cr, uid, [('data_atualizacao', '>=', data_alteracao + ' 00:00:00'),
                      ('data_atualizacao', '<=', data_alteracao + ' 23:59:59')], context=context))
        automacao_precos_dados = automacao_precos_ids and obj_automacao_precos.read(
            cr, uid, automacao_precos_ids, ['codigo_barra_id'], context, '_classic_write')
        automacao_precos_dados and cod_bar_ids.extend(list(set(
            map(lambda f: f.get('codigo_barra_id'), automacao_precos_dados))))

        cod_bar_ids and self.wimeti_processa_dados_codigo_barra(cr, uid, ids, cod_bar_ids, context)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'impressao_etiqueta_interna',
            'name': 'etiqueta_interna',
        }

    def wimeti_processa_dados_codigo_barra(self, cr, uid, ids, cod_bar_ids, context=None):
        context = context and context.copy() or {}
        etq_interna_id = isinstance(ids, list) and ids[0] or ids
        cod_bar_dados = self.pool.get('automacao.codigo.barra').read(
            cr, uid, cod_bar_ids, ['produto_id', 'emite_etiqueta'], context, '_classic_write')
        produto_ids = list(set(map(lambda p: p.get('produto_id'), cod_bar_dados)))
        estoque_produto_dados = self.pool.get('estoque.produto').read(
            cr, uid, produto_ids, ['estpro_validade_interna'], context)
        produto_validade = {}
        map(lambda p: produto_validade.update({p.get('id'): p.get('estpro_validade_interna')}),
            estoque_produto_dados)
        preco_venda_dados = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, produto_ids, context=context)
        data_hoje = time.strftime('%Y-%m-%d %H:%M:%S')
        sql = """
            INSERT INTO wizard_impressao_etiqueta_interna_item 
                (create_date, create_uid, write_date, write_uid, wimpei_capa_id, 
                wimpei_codigo_barra_id, wimpei_preco_venda, wimpei_data_validade)
            VALUES
            
        """
        insert_itens = []
        for bar in cod_bar_dados:
            if not bar.get('emite_etiqueta'):
                continue

            produto_id = bar.get('produto_id')
            preco_venda_produto = preco_venda_dados.get(produto_id) or {}
            preco_venda = preco_venda_produto.get('preco_venda') or 0.0
            validade = produto_validade.get(produto_id)
            validade_produto = validade and (str(date.today() + relativedelta(days=validade))) or \
                'NULL'
            values_insert = "('%s', %s, '%s', %s, %s, %s, %s, '%s')" % (
                data_hoje, uid, data_hoje, uid, etq_interna_id, bar.get('id'), preco_venda,
                validade_produto)
            values_insert = values_insert.replace("'NULL'", 'NULL')
            insert_itens.append(values_insert)
        sql += ',\n'.join(insert_itens)
        cr.execute(sql)
        return True

    def wimeti_onchange_tipo_dado(self, cr, uid, ids, tipo_dado, context=None):
        context = context and context.copy() or {}

        if ids:
            item_ids = self.read(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                 ['wimeti_item_ids'], context).get('wimeti_item_ids')
            item_ids and self.pool.get('wizard.impressao.etiqueta.interna.item').unlink(
                cr, uid, item_ids, context)

        return {'value': {'wimeti_data_alteracao': False, 'wimeti_item_ids': [],
                          'wimeti_codigo_barra_digito': False}}


class wizard_impressao_etiqueta_interna_item(osv.TransientModel):
    _name = 'wizard.impressao.etiqueta.interna.item'
    _columns = {
        'wimpei_capa_id': fields.many2one('wizard.impressao.etiqueta.interna', 'Capa',
                                          ondelete='cascade'),
        'wimpei_codigo_barra_id': fields.many2one('automacao.codigo.barra', 'Codigo de barras',
                                                  ondelete='cascade'),
        'wimpei_codigo_barra': fields.related('wimpei_codigo_barra_id', 'codigo_barra', type='char',
                                              relation='automacao.codigo.barra',
                                              string='Codigo escolhido', readonly=True),
        'wimpei_preco_venda': fields.float('Preco de venda', digits=(14, 5)),
        'wimpei_data_validade': fields.date('Data de validade'),
    }

    def wimpei_onchange_codigo_barra_id(self, cr, uid, ids, codigo_barra_id, codigo_barra_digito,
                                        context=None):
        context = context and context.copy() or {}
        res = {'value': {'wimpei_codigo_barra': ''}}
        if not codigo_barra_id:
            return res
        codigo_barra_dados = self.pool.get('automacao.codigo.barra').read(
            cr, uid, codigo_barra_id, ['codigo_barra', 'produto_id'], context, '_classic_write')
        codigo_barra = codigo_barra_dados.get('codigo_barra')
        produto_id = codigo_barra_dados.get('produto_id')
        validade = self.pool.get('estoque.produto').read(
            cr, uid, produto_id, ['estpro_validade_interna'], context).get(
            'estpro_validade_interna') or 0
        preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, produto_id, context=context).get(produto_id) or {}
        preco_venda = preco_venda.get('preco_venda') or 0.0
        res['value'].update(
            {'wimpei_codigo_barra': codigo_barra,
             'wimpei_preco_venda': preco_venda,
             'wimpei_data_validade': validade and (str(date.today() + relativedelta(days=validade)))
                or ''})
        return res
