# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.tools.translate import _
from openerp.osv import osv
import openerp.addons.web.http as http
import base64

# class sc_contrecf(http.Controller):
#     _cp_path = '/bc/boleto'
#
#     @http.httprequest
#     def pdf(self, req, s_action=None, **kw):
#         _pdf = kw.get('pdf')
#         _pdf1 = base64.b64decode(_pdf)
#         obj = _pdf1
#         headers = [
#             ('Content-type', 'application/pdf'),
#             ('Content-Length', len(obj)),
#         ]
#         return req.make_response(obj, headers)


class integracao(osv.Model):
    _name = 'bf.integracao'
    _auto = False

    def bank_get_saldo(self, cr, uid, codigo_banco_interno, conta_id, data=None, context=None):
        """
        Metodo para buscar saldo no banco.
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param data: data do saldo a ser pesquisada, se não informada sera usada a data do momento da pesquisa
        :type data: str
        :param context: Contexto to Openerp
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: {'saldoDisponivel': saldo disponivel,
                                                              'saldo': saldo total,
                                                              'bloqueado': saldo - saldo bloqueado,
                                                              'limite': limite da conta}
        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Saldo" nao implementado para este banco!')
        }
        return res

    def bank_get_extrato_pdf(self, cr, uid, codigo_banco_interno, conta_id, data_inicial, data_final, context=None):
        """
        Metodo para baixar extrato pdf do banco.
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param data_inicial: Data inicial para consulta do extrato
        :type data_inicial: str
        :param data_final: Data final para consulta do extrato
        :type data_final: str
        :param context: Contexto to Openerp
        :type context: dict
        :return {'status': True/False, 'mensagem':'', pdf: pdf}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Consulta Extrato PDF" nao implementado para este banco!')
        }
        return res


    def bank_get_extrato(self, cr, uid, codigo_banco_interno, conta_id, data_inicial, data_final, context=None):
        """
        Metodo para buscar extrato no banco.
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param data_inicial: Data inicial para consulta do extrato
        :type data_inicial: str
        :param data_final: Data final para consulta do extrato
        :type data_final: str
        :param context: Contexto do Openerp
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: lista contendo:
                                                  credito_debito, valor, data,tipo_transacao, historico, documento
        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Consulta Extrato" nao implementado para este banco!')
        }
        return res

    def pix_rec_enviar(self, cr, uid, codigo_banco_interno, conta_id, valor, pessoa_id=None,
                       vencimento=None, tempoVida=None, multa=None, juros=None, desconto=None, prazoDesconto=None,
                       context=None):
        """
        Metodo para enviar um pix a receber
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param valor: valor do pix a ser enviado.
        :type valor: float
        :param pessoa_id: id de quem vai pagar o pix
        :type pessoa_id: int
        :param vencimento: data de vencimento do pix, qdo nao for agendado.
        :type vencimento: str
        :param tempoVida: Tempo de vida do pix em segundos
        :type tempoVida: int
        :param multa: % de multa pára pix agendado
        :type multa: float
        :param juros: % de juros pára pix agendado
        :type juros: float
        :param desconto: % de desconto pára pix agendado
        :type desconto: float
        :param prazoDesconto: prazo paga calculo da data para o desconto
        :type prazoDesconto: int
        :return {'status': True/False, 'mensagem':'', dados: dados retornados no envio do pix}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo de "Recebimento de PIX" nao implementado para este banco!')
        }
        return res

    def pix_pag_enviar(self, cr, uid, codigo_banco_interno, conta_id, chave, valor, descricao=None, data=None,
                         context=None):
        """
        Metodo para enviar um pix
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param chave: chave para envio do pix.
        :type chave: str
        :param valor: valor do pix a ser enviado.
        :type valor: float
        :param descricao: descricao do pagamento.
        :type descricao: str
        :param data: data para efetivacao do pix.
        :type data: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados no envio do pix}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo de envio de Pix nao implementado para este banco!')
        }
        return res

    def pix_pag_consultar(self, cr, uid, codigo_banco_interno, conta_id, controle=None, endToEnd=None, context=None):
        """
        Metodo para devolver um pix pago
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param controle: codigo de controle do pix enviado.
        :type controle: str
        :param endToEnd: codigo de e2eid do pix enviado.
        :type endToEnd: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados no envio do pix}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Consulta de Pix" nao implementado para este banco!')
        }
        return res

    def pix_devolucao(self, cr, uid, codigo_banco_interno, conta_id, hash=None, controle=None, motivo=None, valor=None,
                      context=None):
        """
        Metodo para devolver um pix
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param hash: codigo de controle do pix enviado.
        :type hash: str
        :param controle: codigo do pix a ser devolvido.
        :type controle: str
        :param motivo: motivo de devolucao do pix
        :type motivo: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados no envio do pix}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Devolucao de Pix" nao implementado para este banco!')
        }
        return res

    def pix_rec_consultar(self, cr, uid, codigo_banco_interno, conta_id, controle=None, hash=None, context=None):
        """
        Metodo para devolver um pix
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param controle: codigo de controle do pix enviado.
        :type controle: str
        :param hash: codigo de hash ou e2eid do pix enviado.
        :type hash: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados no envio do pix}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Consulta de Pix" nao implementado para este banco!')
        }
        return res

    def pag_consultar_boleto(self, cr, uid, codigo_banco_interno, conta_id, codbarra, context=None):
        """
        Metodo para sonsultar se um boleto está registrado
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param codbarra: codigo de barra do boleto
        :type conta_id: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados na inclusao do pagamento}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Consulta de boleto" nao implementado para este banco!')
        }
        return res



    def pag_boleto_incluir(self, cr, uid, codigo_banco_interno, conta_id, codbarra, valor, datapagto, vencimento,
                    descricao=None, context=None):
        """
        Metodo para devolver um pix
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param codbarra: codigo de barra do titulo.
        :type codbarra: str
        :param valor: valor do titulo.
        :type valor: float
        :param datapagto: data do pagamento
        :type datapagto: str
        :param vencimento: vencimento
        :type vencimento: str
        :param descricao: Descricao textual do pagamento
        :type descricao: str
        :param context: Contexto to Openerp.
        :type context: dict
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados na inclusao do pagamento}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo de "Incluir Pagamento" nao implementado para este banco!')
        }
        return res


    def pag_consultar_para_autorizar(self, cr, uid, codigo_banco_interno, conta_id, context=None):
        """
        Metodo para consultar os titulos marcados para pagamento
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :return {'status': True/False, 'mensagem':'', dados: dados retornados na inclusao do pagamento}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Consultar para autorizar" nao implementado para este banco!')
        }
        return res


    def pag_autorizar_pagamento(self, cr, uid, codigo_banco_interno, conta_id, id_titulo, context=None):
        """
        Metodo para autorizar pagamentos / agendamentos
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :paramm id_titulo: id do titulo a ser pago/agendado
        :type id_titulo: int
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem':'', dados: dados retornados na inclusao do pagamento}

        """
        res = {
            'status': False,
            'mensagem': _('Metodo "Autorizar pagamento" nao implementado para este banco!')
        }
        return res

    def cob_boleto_registrar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, hibrido=False, mensagem1=None,
                             mensagem2=None, context=None):
        """
        Metodo para registrar boletos de cobranca
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulo
        :type titulo_id: int
        :param hibrido: indicacao se é boleto hibrido ou tradicional
        :type hibrido: bool
        :param mensagem1: mensagem para o corpo do boleto
        :type mensagem1: str
        :param mensagem2: mensagem para o corpo do boleto
        :type mensagem2: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return: {'status': True/False, 'mensagem': '', dados:{}}
            Quando status é False retorna a mensagtem de erro
            Quando status é True, retorna dados(dict) com:
                controle,
                linhaDigitavel,
                codigoBarras,
                nossoNumero,
                urlPdf,
                urlPagamento,
       """

        res = {
            'status': False,
            'mensagem': _('Metodo "Registrar boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_prorrogar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, dataProrrogacao, context=None):
        """
        Metodo para prorrogar boletos de cobranca
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulo
        :type titulo_id: int
        :param dataProrrogacao: data da prorrogacao
        :type dataProrrogacao: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return: {'status': True/False, 'mensagem': '', dados:{}}
            Quando status é False retorna a mensagtem de erro
            Quando status é True, retorna dados(dict) com:
                controle,
                linhaDigitavel,
                codigoBarras,
                nossoNumero,
                urlPdf,
                urlPagamento,
       """

        res = {
            'status': False,
            'mensagem': _('Metodo "Prorrogar boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_cancelar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        """
        Metodo para cancelar boletos de cobranca no banco
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulos
        :type titulo_id: int
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem': '', dados:{}}
        """

        res = {
            'status': False,
            'mensagem': _('Metodo "Cancelar boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_consultar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        """
        Metodo para Consultar boletos de cobranca no banco
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulos
        :type titulo_id: int
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem': '', dados:{}}
        """

        res = {
            'status': False,
            'mensagem': _('Metodo "Consultar boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_baixar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        """
        Metodo para cancelar boletos de cobranca no banco
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulos
        :type titulo_id: int
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem': '', dados:{}}
        """

        res = {
            'status': False,
            'mensagem': _('Metodo "Baixar boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_protestar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, operacao=None, prazo=None,
                             context=None):
        """
        Metodo para cancelar boletos de cobranca no banco
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulos
        :type titulo_id: int
        :param operacao: valor de 1 a 4
        :type operacao: str
        :param prazo: prazo para negativacao
        :type prazo: int
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem': '', dados:{}}
        """

        res = {
            'status': False,
            'mensagem': _('Metodo "Protestar boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_listar(self, cr, uid, codigo_banco_interno, conta_id, emis_inicial=None, emis_final=None,
                          venc_inicial=None, venc_final=None, movi_inicial=None, movi_final=None, context=None):
        """
        Metodo para listar boletos de cobranca no banco
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param emis_inicial: Data de emissao inicial
        :type emis_inicial: str
        :param emis_final: Data de emissao final
        :type emis_final: str
        :param venc_inicial: Data de vencimento inicial
        :type venc_inicial: str
        :param venc_final: Data de vencimento final
        :type venc_final: str
        :param movi_inicial: Data de movimento inicial
        :type movi_inicial: str
        :param movi_final: Data de movimento final
        :type movi_final: str
        :param context: Contexto to Openerp.
        :type context: dict
        :return: linhas do estrado
        :rtype: list
        """

        res = {
            'status': False,
            'mensagem': _('Metodo "Listagem de boleto" nao implementado para este banco!')
        }
        return res

    def cob_boleto_get_pdf(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        """
        Metodo para buscar o pdf ou o link do pdf do boleto
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param titulo_id: id do titulos
        :type titulo_id: int
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem': '', dados:{}}
        """

        res = {
            'status': False,
            'mensagem': _('Metodo "Para pegar PDF" nao implementado para este banco!')
        }
        return res

    def status_get(self, cr, uid, codigo_banco_interno, conta_id, operacao, context=None):
        """
        Metodo para buscar o pdf ou o link do pdf do boleto
        Deve ser sobrescrito em cada integraçao bancaria
        :param codigo_banco_interno: codigo que identifica o banco a ser integrado.
        :type codigo_banco_interno: str
        :param conta_id: id da conta no sigerp.
        :type conta_id: int
        :param operacao: operação que pode ser bank, pixpag, pixrec, bolrec, bolpag
        :type operacao: string
        :param context: Contexto to Openerp.
        :type context: dict
        :return {'status': True/False, 'mensagem': '', dados:{}}
        """

        res = {
            'status': True, # No caso de não ter implemntação, deve retornar true com dados, para que não dê um falso negativo
            'mensagem': _('Metodo "Para pegar status" nao implementado para este banco!'),
            'dados': {'imagem': ''},
        }
        return res
