# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time

class banfin_pixrec(osv.Model):
    """
        Classe  para recebimento de pix
        prefixo: banfin_pixrec
    """
    from funcoes import _banfin_pixrec_get_dados, _banfin_pixrec_get_state
    _name = 'banfin.pixrec'
    _columns = {
        'id': fields.integer('ID'),
        'create_uid': fields.many2one('res.users', 'Usuario', readonly=True, select=True),
        'create_date': fields.datetime('Data', select=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True),
        'banfin_pixrec_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', ondelete="cascade",
                                               domain="[('banco_caixa', '=', 'B')]", select=True, required=True),
        'banfin_pixrec_valor': fields.float('Valor', digits=(9, 2), required=True),
        'banfin_pixrec_agendado': fields.boolean('Agendado'),
        'banfin_pixrec_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', ondelete='cascade', select=True),
        'banfin_pixrec_vencimento': fields.date('Vencimento'),
        'banfin_pixrec_descricao': fields.text('Descricao', required=True),
        'state': fields.selection([('A', 'A Enviar'), ('P', 'Pendente'), ('R', 'Recebido'), ('D', 'Devolvido'),
                                   ('C', 'Cancelado')], 'Estado', select=True),
        'banfin_pixrec_controle': fields.char('Controle', size=36, select=True),
        'banfin_pixrec_qrcode': fields.text('Codigo QrCode'),
        'banfin_pixrec_hash': fields.char('Hash', size=50),
        'banfin_pixrec_qrcode_image': fields.function(_banfin_pixrec_get_dados, method=True, multi="dados", string='QrCode',
                                              type="binary", store=False),
        'banfin_pixrec_centro_custo_id': fields.many2one("adm.empresa.centrocusto", 'C. de custo',
                                                         ondelete="cascade", select=True,
                                                         domain="[('company_id', '=', company_id)]"),
        'banfin_pixrec_evento_id': fields.many2one('gerencial.evento', 'Evento', ondelete="cascade",
                                                   select=True, domain="[('company_id', '=', company_id)]"),
        'banfin_pixrec_dh_registro': fields.datetime('Horario do registro'),
        'banfin_pixrec_tempo_vida': fields.integer('Tempo de vida (segundos)'),
        'banfin_pixrec_expirado': fields.function(_banfin_pixrec_get_dados, method=True, multi="dados", string='Expirado',
                                              type="boolean", store=False),
        'banfin_pixrec_multa': fields.float('Multa (%)', digits=(1,1),
                                            help='Percentual de multa para PIX agendado'),
        'banfin_pixrec_juros': fields.float('Juros (% ao mes)', digits=(1, 1),
                                            help='Percentual de juros para Pix agendado'),
        'banfin_pixrec_desconto': fields.float('Desconto (%)', digits=(1, 1),
                                               help='Percentual de desconto para Pix agendado pago antecipadamente'),
        'banfin_pixrec_desconto_prazo': fields.integer('Prazo desconto (dias)',
                                            help='Quantidade de dias antes do vencimento para conceder o desconto'),
        'banfin_pixrec_credito_tesouraria_id': fields.many2one('financeiro.tesouraria.movimento', 'Lancamento a credito',
                                                               ondelete="set null", select=True),
        'banfin_pixrec_debito_tesouraria_id': fields.many2one('financeiro.tesouraria.movimento',
                                                               'Lancamento a debito',
                                                               ondelete="set null", select=True),
        'banfin_pixrec_image_status': fields.function(_banfin_pixrec_get_state, method=True, string='Status',
                                               type='binary', store=False),

    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'banfin_pixrec_vencimento': lambda *a: time.strftime('%Y-%m-%d'),
        'state': 'A',
    }
    _order = 'id desc'
