# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wiz_banfin_bol_autorizacoes(osv.TransientModel):
    """
        Classe de Wizard para consulta de autorizacoes de pagamento de boleto
        prefixo: wiz_banfin_bolau
    """
    _name = 'wiz.banfin.bol.autorizacoes'
    _columns = {
        'wiz_banfin_bolau_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', ondelete="cascade",
                                                    domain="[('banco_caixa', '=', 'B')]", select=True, required=True),
        'wiz_banfin_bolau_linha_ids': fields.one2many('wiz.banfin.bol.autorizacoes.linhas',
                                                          'wiz_banfin_bolaul_capa_id',
                                                          'Linhas'),
    }

    def banfin_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if not conta_id:
            return res
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        if not conta_brw.banco_id.admbco_codigo_interno_id:
            return {'warning': {'title': _('Aviso'),
                                'message': _('Conta de tesouraria sem integracao bancaria!')}}
        return res

    def banfin_wiz_autorizacoes_btn_consultar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        exc_ids = [x.id for x in dados_brw.wiz_banfin_bolau_linha_ids]
        self.pool.get('wiz.banfin.bol.autorizacoes.linhas').unlink(cr, uid, exc_ids, context=context)
        ret = self.pool.get('bf.integracao').pag_consultar_para_autorizar(cr, uid,
             dados_brw.wiz_banfin_bolau_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
             dados_brw.wiz_banfin_bolau_conta_id.id,
             context=context)
        if ret.get('status'):
            for _dados_boleto in ret.get('dados').get('message'):
                _codbarra = _dados_boleto.get('linhaDigitavel')
                vals = {
                    'wiz_banfin_bolaul_capa_id': ids[0],
                    'wiz_banfin_bolaul_id_titulo': _dados_boleto.get('id'),
                    'wiz_banfin_bolaul_descricao': _dados_boleto.get('observacao'),
                    'wiz_banfin_bolaul_datavencimento': _dados_boleto.get('dataVencimentoFormatada'),
                    'wiz_banfin_bolaul_data_pagamento': _dados_boleto.get('dataPagamentoFormatada'),
                    'wiz_banfin_bolaul_valor_a_pagar': _dados_boleto.get('valor'),
                }
                _boleto_ids = self.pool.get('financeiro.titulo.boleto').search(cr, uid, [('fintib_codigo_barras', '=', _codbarra)], context=context, limit=1)
                if _boleto_ids:
                    brw_boleto = self.pool.get('financeiro.titulo.boleto').browse(cr, uid, _boleto_ids[0], context=context)
                    vals['wiz_banfin_bolaul_titulo_id'] = brw_boleto.fintib_titulo_id.id
                self.pool.get('wiz.banfin.bol.autorizacoes.linhas').create(cr, uid, vals, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))



class wiz_banfin_bol_autorizacoes_linhas(osv.TransientModel):
    """
        Classe de Wizard das linhas(titulots) consultados para autorizacoes de pagamento de boleto
        prefixo: wiz_banfin_bolau
    """
    _name = 'wiz.banfin.bol.autorizacoes.linhas'
    _columns = {
        'wiz_banfin_bolaul_capa_id': fields.many2one('wiz.banfin.bol.autorizacoes', 'Capa', ondelete="cascade",
                                                    required=True),
        'wiz_banfin_bolaul_id_titulo': fields.char('ID', size=32),
        'wiz_banfin_bolaul_descricao': fields.char('Descricao', size=60),
        'wiz_banfin_bolaul_datavencimento': fields.date('Data Vencimento'),
        'wiz_banfin_bolaul_data_pagamento': fields.date('Data Pagamento'),
        'wiz_banfin_bolaul_valor_a_pagar': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_bolaul_titulo_id': fields.many2one('financeiro.titulo', 'Titulo'),
        'state': fields.selection([('A', 'Aguardando'), ('T', 'Autorizado')], string="Estado",
                                  readonly=False),
    }

    _default = {
        'state': 'A',
    }
    def banfin_wiz_bolau_btn_autorizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').pag_autorizar_pagamento(cr, uid,
             dados_brw.wiz_banfin_bolaul_capa_id.wiz_banfin_bolau_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
             dados_brw.wiz_banfin_bolaul_capa_id.wiz_banfin_bolau_conta_id.id,
             dados_brw.wiz_banfin_bolaul_id_titulo,
             context=context)
        if ret.get('status'):
            # FAzer tratamento com produção, sandbox
                return True

        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))
