# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wiz_banfin_conciliacao(osv.TransientModel):
    """
        Classe de Wizard para conciliacao de contas
        prefixo: wiz_banfin_conc_
    """
    _name = 'wiz.banfin.conciliacao'
    _columns = {
        'wiz_banfin_conc_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', ondelete="cascade",
                                                    domain="[('banco_caixa', '=', 'B')]", select=True, required=True),
        'wiz_banfin_conc_data_inicial': fields.date('Data Inicial'),
        'wiz_banfin_conc_sem_conciliacao': fields.boolean('Sem conciliacao'),
        'wiz_banfin_conc_extrato_ids': fields.one2many('wiz.banfin.conciliacao.extrato', 'wiz_banfin_coex_capa_id',
                                                       'Extrato'),
        'wiz_banfin_conc_tesouraria_ids': fields.one2many('wiz.banfin.conciliacao.tesouraria',
                                                          'wiz_banfin_cote_capa_id',
                                                          'Movimento'),
        'wiz_banfin_conc_saldo_conciliado': fields.float('Saldo conciliado'),
        'wiz_banfin_conc_saldo_final': fields.float('Saldo final'),
    }
    _defaults = {
        'wiz_banfin_conc_sem_conciliacao': True,
    }

    def banfin_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if not conta_id:
            return res
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        res['value']['wiz_banfin_conc_saldo_conciliado'] = conta_brw.fintco_saldo_conciliado
        res['value']['wiz_banfin_conc_saldo_final'] = conta_brw.fintco_saldo_final
        if not conta_brw.banco_id.admbco_codigo_interno_id:
            return {'warning': {'title': _('Aviso'),
                                'message': _('Conta de tesouraria sem integracao bancaria!')}}
        return res

    def banfin_wiz_extratos_btn_consultar(self, cr, uid, ids, context=None):
        def _buscarNoExtrato():
            _args = [('wiz_banfin_coex_capa_id', '=', ids[0]),
                     ('wiz_banfin_coex_conciliado', '=', False),
                     ('wiz_banfin_coex_credito_debito', '=', brw_movTes.debito_credito)]
            _orders = 'wiz_banfin_coex_valor,wiz_banfin_coex_data'
            _extratoIds = self.pool.get('wiz.banfin.conciliacao.extrato').search(cr, uid, _args, order=_orders,
                                                                                 context=context)
            for _brw_extr in self.pool.get('wiz.banfin.conciliacao.extrato').browse(cr, uid, _extratoIds,
                                                                                        context=context):
                if _brw_extr.wiz_banfin_coex_valor == brw_movTes.valor:
                    _vals = {
                        'wiz_banfin_coex_conciliado': True,
                        'wiz_banfin_coex_movimento_id': brw_movTes.id,
                    }
                    self.pool.get('wiz.banfin.conciliacao.extrato').write(cr, uid, _brw_extr.id, _vals, context=context)
                    if _brw_extr.wiz_banfin_coex_data != brw_movTes.data_movimento:
                        _alteracao = True
                    else:
                        _alteracao = False
                    return _brw_extr.wiz_banfin_coex_linha, _brw_extr.id, _alteracao
            return None, None, False

        brw = self.browse(cr, uid, ids[0], context=context)
        vals = {
            'wiz_banfin_conc_saldo_conciliado': brw.wiz_banfin_conc_conta_id.fintco_saldo_conciliado,
            'wiz_banfin_conc_saldo_final': brw.wiz_banfin_conc_conta_id.fintco_saldo_final,
        }
        self.write(cr, uid, ids[0], vals, context=context)
        exc_ids = [x.id for x in brw.wiz_banfin_conc_extrato_ids]
        self.pool.get('wiz.banfin.conciliacao.extrato').unlink(cr, uid, exc_ids, context=context)
        exc_ids = [x.id for x in brw.wiz_banfin_conc_tesouraria_ids]
        self.pool.get('wiz.banfin.conciliacao.tesouraria').unlink(cr, uid, exc_ids, context=context)
        _args = [('banfin_exli_conta_id', '=', brw.wiz_banfin_conc_conta_id.id)]
        if not (brw.wiz_banfin_conc_data_inicial or brw.wiz_banfin_conc_sem_conciliacao):
            raise osv.except_osv(_('Atencao'), _('E necessario selecionar a "Data inicial" ou marcar "Sem conciliacao"!'))
        if brw.wiz_banfin_conc_sem_conciliacao:
            _args.append(('banfin_exli_conciliado', '=', False))
        if brw.wiz_banfin_conc_data_inicial:
            _args.append(('banfin_exli_data', '>=', brw.wiz_banfin_conc_data_inicial))
        _order = 'banfin_exli_data,banfin_exli_valor,banfin_exli_credito_debito'
        # Linhas do extrato
        extrato_ids = self.pool.get('banfin.extrato.linhas').search(cr, uid, _args, context=context, order=_order)
        if extrato_ids:
            _seq = 0
            for brw_extrato in self.pool.get('banfin.extrato.linhas').browse(cr, uid, extrato_ids, context=context):
                _seq += 1
                vals = {
                    'wiz_banfin_coex_capa_id': ids[0],
                    'wiz_banfin_coex_linha': _seq,
                    'wiz_banfin_coex_data': brw_extrato.banfin_exli_data,
                    'wiz_banfin_coex_credito_debito': brw_extrato.banfin_exli_credito_debito,
                    'wiz_banfin_coex_valor': brw_extrato.banfin_exli_valor,
                    'wiz_banfin_coex_historico': brw_extrato.banfin_exli_historico,
                    'wiz_banfin_coex_extrato_id': brw_extrato.id,

                }
                if brw_extrato.banfin_exli_movimento_id:
                    vals['wiz_banfin_coex_movimento_id'] = brw_extrato.banfin_exli_movimento_id.id
                    vals['wiz_banfin_coex_conciliado'] = True
                else:
                    vals['wiz_banfin_coex_conciliado'] = False
                self.pool.get('wiz.banfin.conciliacao.extrato').create(cr, uid, vals, context=context)
        # linhas do movimento da tesouraria
        _args = [('conta_id', '=', brw.wiz_banfin_conc_conta_id.id)]
        if brw.wiz_banfin_conc_sem_conciliacao:
            _args.append(('conciliado', '=', False))
        if brw.wiz_banfin_conc_data_inicial:
            _args.append(('data_movimento', '>=', brw.wiz_banfin_conc_data_inicial))
        _order = 'data_movimento,valor,debito_credito'
        movTes_ids = self.pool.get('financeiro.tesouraria.movimento').search(cr, uid, _args, context=context, order=_order)
        if movTes_ids:
            for brw_movTes in self.pool.get('financeiro.tesouraria.movimento').browse(cr, uid, movTes_ids, context=context):
                _linha, linha_id, _alteracao = _buscarNoExtrato()
                vals = {
                    'wiz_banfin_cote_capa_id': ids[0],
                    'wiz_banfin_cote_linha': _linha,
                    'wiz_banfin_cote_data': brw_movTes.data_movimento,
                    'wiz_banfin_cote_credito_debito': brw_movTes.debito_credito,
                    'wiz_banfin_cote_valor': brw_movTes.valor,
                    'wiz_banfin_cote_historico': brw_movTes.historico,
                    'wiz_banfin_cote_conciliado': True if _linha else False,
                    'wiz_banfin_cote_conciliado_anterior': brw_movTes.conciliado,
                    'wiz_banfin_cote_alteracao': _alteracao,
                    'wiz_banfin_cote_movimento_id': brw_movTes.id,
                    'wiz_banfin_cote_linha_wiz_extrato_id': linha_id,
                }

                self.pool.get('wiz.banfin.conciliacao.tesouraria').create(cr, uid, vals, context=context)

        return True

    def banfin_wiz_extratos_btn_conciliar(self, cr, uid, ids, context=None):
        brw = self.browse(cr, uid, ids[0], context=context)
        for brw_linha in brw.wiz_banfin_conc_tesouraria_ids:
            if brw_linha.wiz_banfin_cote_conciliado:
                # Atualizar o movimento da tesouraria
                saldo_conciliado = brw.wiz_banfin_conc_conta_id.fintco_saldo_conciliado
                vals = {
                    'conciliado': True
                }
                if brw_linha.wiz_banfin_cote_alteracao:
                    vals['data_movimento'] = brw_linha.wiz_banfin_cote_linha_wiz_extrato_id.wiz_banfin_coex_data
                self.pool.get('financeiro.tesouraria.movimento').write(cr, uid,
                                                                       [brw_linha.wiz_banfin_cote_movimento_id.id],
                                                                       vals, context=context)
                if not brw_linha.wiz_banfin_cote_conciliado_anterior:
                    # Atualizar o saldo conciliado
                    if brw_linha.wiz_banfin_cote_credito_debito == 'C':
                        saldo_conciliado += brw_linha.wiz_banfin_cote_valor
                    else:
                        saldo_conciliado -= brw_linha.wiz_banfin_cote_valor
                    self.pool.get('financeiro.tesouraria.contas').write(cr, uid, [brw.wiz_banfin_conc_conta_id.id],
                                                                        {'fintco_saldo_conciliado': saldo_conciliado},
                                                                        context=context)
                # Atualizar o extrato
                bel_id = brw_linha.wiz_banfin_cote_linha_wiz_extrato_id.wiz_banfin_coex_extrato_id.id
                self.pool.get('banfin.extrato.linhas').write(cr, uid, [bel_id],
                                                             {'banfin_exli_movimento_id': brw_linha.wiz_banfin_cote_movimento_id.id},
                                                             context=context)
        return True

class wiz_banfin_conciliacao_extrato(osv.TransientModel):
    """
        Classe de Wizard para as linhas do extrato para conciliacao de contas
        prefixo: wiz_banfin_coex_
    """
    _name = 'wiz.banfin.conciliacao.extrato'
    _columns = {
        'wiz_banfin_coex_capa_id': fields.many2one('wiz.banfin.conciliacao', 'Capa', ondelete="cascade",
                                                    select=True, required=True),
        'wiz_banfin_coex_linha': fields.integer('Lin'),
        'wiz_banfin_coex_data': fields.date('Data'),
        'wiz_banfin_coex_credito_debito': fields.selection([('C', 'C'), ('D', 'D')], 'C/D'),
        'wiz_banfin_coex_valor': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_coex_historico': fields.text('Historico'),
        'wiz_banfin_coex_movimento_id': fields.many2one('financeiro.tesouraria.movimento', 'Movimento',
                                                        ondelete='cascade',
                                                        select=True),
        'wiz_banfin_coex_conciliado': fields.boolean('Conc.'),
        'wiz_banfin_coex_extrato_id': fields.many2one('banfin.extrato.linhas', 'Extrato', ondelete='cascade',
                                                      select=True)
    }

    def wiz_banfin_coex_btn_incluir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not ids:
            return True
        view = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'financeiro', 'view_financeiro_tesouraria_movimento_form')
        linhaBrw = self.browse(cr, uid, ids[0], context=context)
        context['wizardConciliacao'] = {
            'conta_id': linhaBrw.wiz_banfin_coex_capa_id.wiz_banfin_conc_conta_id.id,
            'data_movimento': linhaBrw.wiz_banfin_coex_data,
            'c_d': linhaBrw.wiz_banfin_coex_credito_debito,
            'valor': linhaBrw.wiz_banfin_coex_valor,
            'historico': linhaBrw.wiz_banfin_coex_historico,
        }
        return {
            'name': 'Lancamento tesouraria',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'financeiro.tesouraria.movimento',
            'view_id': [view and view[1] or False],
            'target': 'current',
            'type': 'ir.actions.act_window',
            'context': context,
        }


class wiz_banfin_conciliacao_tesouraria(osv.TransientModel):
    """
        Classe de Wizard para movimento.tesouraria.contas para conciliacao
        prefixo: wiz_banfin_cote_
    """
    _name = 'wiz.banfin.conciliacao.tesouraria'
    _columns = {
        'wiz_banfin_cote_capa_id': fields.many2one('wiz.banfin.conciliacao', 'Capa', ondelete="cascade",
                                                    select=True, required=True),
        'wiz_banfin_cote_linha': fields.integer('Lin'),
        'wiz_banfin_cote_linha_wiz_extrato_id': fields.many2one('wiz.banfin.conciliacao.extrato', 'Linha extrato',
                                                                ondelete='cascade', select=1),
        'wiz_banfin_cote_data': fields.date('Data'),
        'wiz_banfin_cote_credito_debito': fields.selection([('C', 'C'), ('D', 'D')], 'C/D'),
        'wiz_banfin_cote_valor': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_cote_historico': fields.text('Historico'),
        'wiz_banfin_cote_conciliado': fields.boolean('Concilidado'),
        'wiz_banfin_cote_conciliado_anterior': fields.boolean('Concilidado anterior'),
        'wiz_banfin_cote_alteracao': fields.boolean('Alteracao'),
        'wiz_banfin_cote_movimento_id': fields.many2one('financeiro.tesouraria.movimento', 'Movimento',
                                                        ondelete='cascade',
                                                        select=True),
    }
    _order = 'wiz_banfin_cote_capa_id,wiz_banfin_cote_linha'

    def wiz_banfin_cote_manutencao(self, cr, uid, ids, context=None):
        def _get_view_id(_view_name):
            res_id = False
            _obj_ir_model_data = self.pool.get('ir.model.data')
            ids = _obj_ir_model_data.search(cr, uid, [('name', '=', _view_name)], context=context)
            if ids:
                res_id = _obj_ir_model_data.read(cr, uid, ids[0], ['res_id'], context=context)['res_id']
            return res_id

        brw = self.browse(cr, uid, ids[0], context=context)
        context['banfin_wiz_conciliar'] = True
        res = {
            'name': 'Movimento da Tesouraria',
            'view_type': 'tree',
            'view_mode': 'form',
            'res_model': 'financeiro.tesouraria.movimento',
            'context': context,
            'target': 'new',
            'type': 'ir.actions.act_window',
            'res_id': brw.wiz_banfin_cote_movimento_id.id,
            'view_id': _get_view_id('view_financeiro_tesouraria_movimento_form'),
        }
        return res
