# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time
from openerp.tools.translate import _
from SigUtils import datas


class wiz_banfin_extratos(osv.TransientModel):
    """
        Classe de Wizard para consulta de extratos
        prefixo: wiz_banfin_extr_
    """
    _name = 'wiz.banfin.extratos'
    _columns = {
        'wiz_banfin_extr_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', ondelete="cascade",
                                                    domain="[('banco_caixa', '=', 'B')]", select=True, required=True),
        'wiz_banfin_extr_data_inicial': fields.date('Data Inicial', required=True),
        'wiz_banfin_extr_data_final': fields.date('Data Final', required=True),
        'wiz_banfin_extr_linha_ids': fields.one2many('wiz.banfin.extrato.linhas', 'wiz_banfin_exli_capa_id', 'Linhas'),
        'wiz_banfin_extr_saldo_inicial': fields.float('Saldo Inicial', digites=(9, 2)),
        'wiz_banfin_extr_saldo_final': fields.float('Saldo Final', digites=(9, 2)),
    }
    _defaults = {
        'wiz_banfin_extr_data_inicial': lambda *a: time.strftime('%Y-%m-%d'),
        'wiz_banfin_extr_data_final': lambda *a: time.strftime('%Y-%m-%d'),
    }

    def banfin_wiz_limpar_tela(self, cr, uid, ids, context=None):
        vals = {
            'wiz_banfin_extr_saldo_inicial': 0.0,
            'wiz_banfin_extr_saldo_final': 0.0,
        }
        self.write(cr, uid, ids[0], vals, context=context)
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        exc_ids = [x.id for x in dados_brw.wiz_banfin_extr_linha_ids]
        self.pool.get('wiz.banfin.extrato.linhas').unlink(cr, uid, exc_ids, context=context)
        return True

    def banfin_wiz_extratos_btn_consultar(self, cr, uid, ids, context=None):
        def _extrato_incluir_linhas():
            """
            verificar se já existe a linha no extrato(banfin.extrato.linhas), se não existir inclui
            """
            _args = [('banfin_exli_conta_id', '=', conta_id),
                     ('banfin_exli_data', '>=', _data_inicial),
                     ('banfin_exli_data', '<=', _data_final)]
            ex_linha_ids = self.pool.get('banfin.extrato.linhas').search(cr, uid, _args, context=context)
            ex_linhas_brw = self.pool.get('banfin.extrato.linhas').browse(cr, uid, ex_linha_ids, context=context)
            for wiz_linha in ret.get('dados'):
                _data = wiz_linha.get('data')
                _valor = wiz_linha.get('valor')
                _c_d = wiz_linha.get('credito_debito')
                _achou = False
                # Se a linha ja existir no extrato, segue para a proxima
                for ex_linha_brw in ex_linhas_brw:
                    if ex_linha_brw.banfin_exli_data == _data and \
                       ex_linha_brw.banfin_exli_valor == float(_valor) and \
                       ex_linha_brw.banfin_exli_credito_debito == _c_d:
                            ex_linhas_brw.remove(ex_linha_brw)
                            _achou = True
                            break
                if _achou:
                    continue
                # Linha nao existe no extrato, incluir
                vals = {
                    'banfin_exli_conta_id': conta_id,
                    'banfin_exli_data': wiz_linha.get('data'),
                    'banfin_exli_tipo_transacao': wiz_linha.get('tipo_transacao'),
                    'banfin_exli_credito_debito': wiz_linha.get('credito_debito'),
                    'banfin_exli_valor': wiz_linha.get('valor'),
                    'banfin_exli_historico': wiz_linha.get('historico'),
                }
                self.pool.get('banfin.extrato.linhas').create(cr, uid, vals, context=context)

        def _wizard_salvar_linhas(saldo_linha):
            """
            Salva as linhas do extrato no wizard detalhe(wiz.banfin.extrato.linhas)
            """
            for linha in ret.get('dados'):
                vals_w = {
                    'wiz_banfin_exli_capa_id': ids[0],
                    'wiz_banfin_exli_data': linha.get('data'),
                    'wiz_banfin_exli_credito_debito': linha.get('credito_debito'),
                    'wiz_banfin_exli_tipo_transacao': linha.get('tipo_transacao'),
                    'wiz_banfin_exli_valor': linha.get('valor'),
                    'wiz_banfin_exli_saldo': saldo_linha,
                    'wiz_banfin_exli_historico': linha.get('historico'),
                    'wiz_banfin_exli_documento': linha.get('documento'),
                }
                self.pool.get('wiz.banfin.extrato.linhas').create(cr, uid, vals_w, context=context)
                if linha.get('credito_debito') == 'C':
                    saldo_linha -= float(linha.get('valor'))
                else:
                    saldo_linha += float(linha.get('valor'))

            return True
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        exc_ids = [x.id for x in dados_brw.wiz_banfin_extr_linha_ids]
        self.pool.get('wiz.banfin.extrato.linhas').unlink(cr, uid, exc_ids, context=context)
        conta_id = dados_brw.wiz_banfin_extr_conta_id.id
        _data_inicial = dados_brw.wiz_banfin_extr_data_inicial
        _data_final = dados_brw.wiz_banfin_extr_data_final
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        _saldo_inicial = _saldo_final = 0
        ret = self.pool.get('bf.integracao').bank_get_saldo(cr, uid,
                                          conta_brw.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                                          conta_id,
                                          datas.getDataNoFormato(datas.calcVencimento(_data_inicial, -1), 'I', '-'),
                                          context=context)
        if ret.get('status'):
            _saldo_inicial = ret.get('dados').get('saldoDisponivel')
        else:
            raise osv.except_osv(_('Atencao'), ret.get('mensagem'))
        _hoje = time.strftime('%Y-%m-%d')
        ret = self.pool.get('bf.integracao').bank_get_saldo(cr, uid,
                                          conta_brw.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                                          conta_id,
                                          _data_final if _hoje != _data_final else '',
                                          context=context)
        if ret.get('status'):
            _saldo_final = ret.get('dados').get('saldoDisponivel')
        else:
            raise osv.except_osv(_('Atencao'), ret.get('mensagem'))
        ret = self.pool.get('bf.integracao').bank_get_extrato(cr, uid,
                                          conta_brw.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                                          conta_id,
                                          _data_inicial,
                                          _data_final,
                                          context=context)
        if ret.get('status'):
            _wizard_salvar_linhas(_saldo_final)
            _extrato_incluir_linhas()
            vals = {
                'wiz_banfin_extr_saldo_inicial': _saldo_inicial,
                'wiz_banfin_extr_saldo_final': _saldo_final,
            }
            self.write(cr, uid, ids[0], vals, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def banfin_wiz_extratos_btn_consultar_pdf(self, cr, uid, ids, context=None):
        dados_read = self.read(cr, uid, ids[0], [], context=context)
        conta_id = dados_read.get('wiz_banfin_extr_conta_id')[0]
        _data_inicial = dados_read.get('wiz_banfin_extr_data_inicial')
        _data_final = dados_read.get('wiz_banfin_extr_data_final')
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        ret = self.pool.get('bf.integracao').bank_get_extrato_pdf(cr, uid,
                                          conta_brw.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                                          conta_id,
                                          _data_inicial,
                                          _data_final,
                                          context=context)
        if ret.get('status'):
            return {'type': 'ir.actions.report.xml',
                    'report_name': 'relatorios',
                    'datas': {'pdf': {'estrutura': {'pdf': True},
                                      'dados': ret.get('dados').get('pdf')}}}
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))


    def banfin_wiz_extratos_btn_7_days(self, cr, uid, ids, context=None):
        _data_final = datas.getDataDeHoje('I', '-')
        _data_inicial = datas.calcVencimento(_data_final, -6)
        vals = {
            'wiz_banfin_extr_data_final': _data_final,
            'wiz_banfin_extr_data_inicial': _data_inicial
        }
        self.write(cr, uid, ids, vals, context=context)
        self.banfin_wiz_limpar_tela(cr, uid, ids, context=context)
        return True

    def banfin_wiz_extratos_btn_15_days(self, cr, uid, ids, context=None):
        _data_final = datas.getDataDeHoje('I', '-')
        _data_inicial = datas.calcVencimento(_data_final, -14)
        vals = {
            'wiz_banfin_extr_data_final': _data_final,
            'wiz_banfin_extr_data_inicial': _data_inicial
        }
        self.write(cr, uid, ids, vals, context=context)
        self.banfin_wiz_limpar_tela(cr, uid, ids, context=context)
        return True

    def banfin_wiz_extratos_btn_30_days(self, cr, uid, ids, context=None):
        _data_final = datas.getDataDeHoje('I', '-')
        _data_inicial = datas.calcVencimento(_data_final, -29)
        vals = {
            'wiz_banfin_extr_data_final': _data_final,
            'wiz_banfin_extr_data_inicial': _data_inicial
        }
        self.write(cr, uid, ids, vals, context=context)
        self.banfin_wiz_limpar_tela(cr, uid, ids, context=context)
        return True

    def banfin_wiz_extratos_btn_mes_atual(self, cr, uid, ids, context=None):
        _data_final = datas.getDataDeHoje('I', '-')
        _data_inicial = _data_final[:8] + '01'
        vals = {
            'wiz_banfin_extr_data_final': _data_final,
            'wiz_banfin_extr_data_inicial': _data_inicial
        }
        self.write(cr, uid, ids, vals, context=context)
        self.banfin_wiz_limpar_tela(cr, uid, ids, context=context)
        return True

    def banfin_wiz_extratos_btn_mes_anterior(self, cr, uid, ids, context=None):
        _data_inicial = datas.getDataDeHoje('I', '-')
        _data_final = datas.getDataNoFormato(datas.calcVencimento(_data_inicial, - int(_data_inicial[-2:])), 'I', '-')
        _data_inicial = _data_final[:8] + '01'
        vals = {
            'wiz_banfin_extr_data_final': _data_final,
            'wiz_banfin_extr_data_inicial': _data_inicial
        }
        self.write(cr, uid, ids, vals, context=context)
        self.banfin_wiz_limpar_tela(cr, uid, ids, context=context)
        return True

    def banfin_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'wiz_banfin_extr_saldo': 0.0}}
        if not conta_id:
            return res
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        if not conta_brw.banco_id.admbco_codigo_interno_id:
            return {'warning': {'title': _('Aviso'),
                                'message': _('Conta de tesouraria sem integracao bancaria!')}}
        args = [('banfin_exli_conta_id', '=', conta_id)]
        linha_ids = self.pool.get('banfin.extrato.linhas').search(cr, uid, args, limit=1, order='banfin_exli_data DESC',
                                                                  context=context)
        if linha_ids:
            _dataI = self.pool.get('banfin.extrato.linhas').read(cr, uid, linha_ids, ['banfin_exli_data'],
                                                                 context=context)[0].get('banfin_exli_data')
            res['value']['wiz_banfin_extr_data_inicial'] = _dataI
        else:
            res['value']['wiz_banfin_extr_data_inicial'] = datas.getDataDeHoje('I', '-')
        return res


class wiz_banfin_extrato_linhas(osv.TransientModel):
    """
        Classe de Wizard de linhas de um extrato
        prefixo: wiz_banfin_exli_
    """
    _name = 'wiz.banfin.extrato.linhas'
    _columns = {
        'wiz_banfin_exli_capa_id': fields.many2one('wiz.banfin.extratos', 'Extrato', ondelete='cascade', select=1),
        'wiz_banfin_exli_data': fields.date('Data'),
        'wiz_banfin_exli_tipo_transacao': fields.char('Transacao', size=20),
        'wiz_banfin_exli_credito_debito': fields.selection([('C', 'Credito'), ('D', 'Debito')], 'Operacao'),
        'wiz_banfin_exli_valor': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_exli_saldo': fields.float('Saldo', digits=(9, 2)),
        'wiz_banfin_exli_historico': fields.text('Historico'),
        'wiz_banfin_excli_extrato_linha_id': fields.many2one('banfin.extrato.linhas', 'Linha extrato',
                                                             ondelete='cascade', select=True),
        'wiz_banfin_exli_documento': fields.char('Documento', size=32),

    }
