# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wiz_banfin_cob_titulos_lista(osv.TransientModel):
    """
        Classe de Wizard para consulta de titulos em cobrança
        prefixo: wiz_banfin_ticl_
    """
    _name = 'wiz.banfin.cob.titulos.lista'
    _columns = {
        'wiz_banfin_ticl_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', select=True,
                                                    domain="[('banco_caixa','=','B')]", ondelete="cascade",
                                                    required=True),
        'wiz_banfin_ticl_emis_inicial': fields.date('Emissao inicial',
                                                    help='Data de emissao a ser usada para filtro de pesquisa!'),
        'wiz_banfin_ticl_emis_final': fields.date('Emissao final',
                                                  help='Data de emissao a ser usada para filtro de pesquisa!'),
        'wiz_banfin_ticl_venc_inicial': fields.date('Vencimento inicial',
                                                    help='Data de vencimento a ser usada para filtro de pesquisa!'),
        'wiz_banfin_ticl_venc_final': fields.date('Vencimento final',
                                                  help='Data de vencimento a ser usada para filtro de pesquisa!'),
        'wiz_banfin_ticl_movi_inicial': fields.date('Baixa inicial',
                                                    help='Data de baixa a ser usada para filtro de pesquisa!'),
        'wiz_banfin_ticl_movi_final': fields.date('Baixa final',
                                                  help='Data de baixa a ser usada para filtro de pesquisa!'),
        'wiz_banfin_ticl_titulos_ids': fields.one2many('wiz.banfin.titulos.cb.lis.linhas', 'wiz_banfin_tilbli_capa_id',
                                                       'Titulos'),
    }

    def wiz_banfin_ticl_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        if not conta_id:
            return {}
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        if not conta_brw.banco_id.admbco_codigo_interno_id:
            return {'warning': {'title': _('Aviso'),
                                'message': _('Conta de tesouraria sem integracao bancaria!')}}
        return {}

    def wiz_banfin_ticl_btn_consultar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        exc_ids = [x.id for x in dados_brw.wiz_banfin_ticl_titulos_ids]
        self.pool.get('wiz.banfin.titulos.cb.lis.linhas').unlink(cr, uid, exc_ids, context=context)
        _emis_inicial = dados_brw.wiz_banfin_ticl_emis_inicial
        _emis_final = dados_brw.wiz_banfin_ticl_emis_final
        _venc_inicial = dados_brw.wiz_banfin_ticl_venc_inicial
        _venc_final = dados_brw.wiz_banfin_ticl_venc_final
        _movi_inicial = dados_brw.wiz_banfin_ticl_movi_inicial
        _movi_final = dados_brw.wiz_banfin_ticl_movi_final
        ret = self.pool.get('bf.integracao').cob_boleto_listar(cr, uid,
                dados_brw.wiz_banfin_ticl_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                dados_brw.wiz_banfin_ticl_conta_id.id,
                emis_inicial=_emis_inicial,
                emis_final=_emis_final,
                venc_inicial=_venc_inicial,
                venc_final=_venc_final,
                movi_inicial=_movi_inicial,
                movi_final=_movi_final,
                context=context
        )
        if ret.get('status'):
            for x in ret.get('dados'):
                vals = {
                    'wiz_banfin_tilbli_capa_id': ids[0],
                    'wiz_banfin_tilbli_pessoa_id': x.get('pessoa_id'),
                    'wiz_banfin_tilbli_titulo_id': x.get('titulo_id'),
                    'wiz_banfin_tilbli_documento': x.get('documento'),
                    'wiz_banfin_tilbli_parcela': x.get('parcela'),
                    'wiz_banfin_tilbli_emissao': x.get('emissao'),
                    'wiz_banfin_tilbli_vencimento': x.get('vencimento'),
                    'wiz_banfin_tilbli_valor': x.get('valor'),
                    'wiz_banfin_tilbli_situacao': x.get('situacao'),
                    'wiz_banfin_tilbli_controle': x.get('controle'),
                }
                self.pool.get('wiz.banfin.titulos.cb.lis.linhas').create(cr, uid, vals, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))


class wiz_banfin_titulos_cb_lis_linhas(osv.TransientModel):
    """
        Classe de Wizard de titulos
        prefixo: wiz_banfin_tilbli_
    """
    _name = 'wiz.banfin.titulos.cb.lis.linhas'
    _columns = {
        'wiz_banfin_tilbli_capa_id': fields.many2one('wiz.banfin.cob.titulos.lista', 'Capa', ondelete='cascade',
                                                     select=True, required=True),
        'wiz_banfin_tilbli_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', ondelete="cascade", select=True),
        'wiz_banfin_tilbli_titulo_id': fields.many2one('financeiro.titulo', 'Titulo', ondelete="cascade", select=True),
        'wiz_banfin_tilbli_documento': fields.char('Documento', size=30),
        'wiz_banfin_tilbli_parcela': fields.integer('Parcela'),
        'wiz_banfin_tilbli_emissao': fields.date('Emissao'),
        'wiz_banfin_tilbli_vencimento': fields.date('Vencimento'),
        'wiz_banfin_tilbli_valor': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_tilbli_situacao': fields.char('Situacao', size=20),
        'wiz_banfin_tilbli_controle': fields.char('Controle', size=36),
    }

