# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
import time

dict_est_bco = {
    'N': 'Nao enviado',
    'E': 'Enviado',
    'C': 'confirmado',
    'R': 'Rejeitado',
    'B': 'Baixado',
    'A': 'Cancelado'
}


class wiz_banfin_titulos_cobranca(osv.TransientModel):
    """
        Classe de Wizard para cobranca de titulos
        prefixo: wiz_banfin_ticb_
    """
    def wiz_banfin_get_local_cobranca(self, cr, uid, ids, field_name, arg, context=None):
        """
        Metodo de campo funcional. Encontra registros na tabela local de cobranca para o banco selecionado e carteira
        """
        context = context and context.copy() or {}
        res = {}
        carteira_id = self.pool.get('adm.utils').admutl_get_id_nome_externo(cr, uid, 'financeiro',
                                                                            'local_cobranca_carteira')
        for brw_wiz in self.browse(cr, uid, ids, context):
            args = [('finloc_banco_id', '=', brw_wiz.wiz_banfin_ticb_conta_id.banco_id.id)]
            local_ids = self.pool.get('financeiro.local.cobranca').search(cr, uid, args, context=context)
            local_ids.append(carteira_id)
            res[brw_wiz.id] = local_ids
        return res


    _name = 'wiz.banfin.titulos.cobranca'
    _columns = {
        'wiz_banfin_ticb_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', ondelete="cascade", select=True,
                                                     help='Cliente a ser usado para efeito de filtro de pesquisa!'),
        'wiz_banfin_ticb_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', select=True,
                                                    domain="[('banco_caixa','=','B')]", ondelete="cascade",
                                                    required=True),
        'wiz_banfin_ticb_hibrido': fields.boolean('Boleto hibrido', help='boleto Hibrido(Codigo de barras e qrcode),para ser utilizado no registro do titulo'),
        'wiz_banfin_ticb_banco_local_id': fields.many2one('financeiro.local.cobranca', 'Local cobranca', required=True,
                                                          domain="[('finloc_conta_id','=',wiz_banfin_ticb_conta_id)]",
                                                          help='Local a ser utilizado para efeito de registro do titulo',
                                                          ondelete='cascade', select=True),
        'wiz_banfin_ticb_local_natureza_id': fields.many2one('financeiro.local.natureza', 'Natureza', required=True,
                                                             domain="[('finlon_local_cobranca_id','=',wiz_banfin_ticb_banco_local_id)]",
                                                             help='Natureza a ser utilizada para efeito de registro do titulo',
                                                             ondelete='cascade', select=True),
        'wiz_banfin_ticb_venc_inicial': fields.date('Vencimento inicial',
                                                    help='Data a ser usada para evento de filtro de pesquisa!'),
        'wiz_banfin_ticb_venc_final': fields.date('Vencimento final',
                                                  help='Data a ser usada para evento de filtro de pesquisa!'),
        'wiz_banfin_ticb_titulos_ids': fields.one2many('wiz.banfin.titulos.cb.linhas', 'wiz_banfin_tilicb_capa_id',
                                                       'Titulos'),
        'wiz_banfin_ticb_titulos_local_ids': fields.function(wiz_banfin_get_local_cobranca, string='Local cobranca', method=True,
                                                     type='one2many', relation='financeiro.local.cobranca'),
        'wiz_banfin_ticb_mensagem_1': fields.char('Mensagem-1', size=70),
        'wiz_banfin_ticb_mensagem_2': fields.char('Mensagem-2', size=70),
        'wiz_banfin_ticb_negativ_prazo': fields.integer('Prazo negativacao'),
        'wiz_banfin_ticb_negativ_oper': fields.selection([('1', 'Incluir negativacao'), ('2', 'Alterar negativacao'),
                                                          ('3', 'Cancelar negativacao'), ('4', 'Excluir negativacao')],
                                                         'Operacao negativacao', size=1),
        'wiz_banfin_ticb_habil_protesto': fields.boolean('Habilita protesto'),
        'wiz_banfin_ticb_habil_hibrido': fields.boolean('Habilita Hibrido'),
        'wiz_banfin_ticb_habil_baixa': fields.boolean('Habilita baixa'),
        'wiz_banfin_ticb_habil_cancel': fields.boolean('Habilita cancelamento'),
    }
    _defaults = {
        'wiz_banfin_ticb_habil_protesto': True,
        'wiz_banfin_ticb_habil_hibrido': True,
        'wiz_banfin_ticb_habil_baixa': True,
        'wiz_banfin_ticb_habil_cancel': True,
    }

    def banfin_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        if not conta_id:
            return {'value': {'wiz_banfin_ticb_banco_local_id': None}}
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        if not conta_brw.banco_id.admbco_codigo_interno_id:
            return {'warning': {'title': _('Aviso'),
                                'message': _('Conta de tesouraria sem integracao bancaria!')}}
        args = [('finloc_conta_id', '=', conta_id)]
        local_ids = self.pool.get('financeiro.local.cobranca').search(cr, uid, args, context=context, limit=1)
        if local_ids:
            return {'value': {'wiz_banfin_ticb_banco_local_id': local_ids[0]}}
        return {'value': {'wiz_banfin_ticb_banco_local_id': None}}

    def banfin_onchange_local_cobranca(self, cr, uid, ids, local_id, context=None):
        if not local_id:
            return {'value': {'wiz_banfin_ticb_local_natureza_id': None}}
        args = [('finlon_local_cobranca_id', '=', local_id)]
        natureza_ids = self.pool.get('financeiro.local.natureza').search(cr, uid, args, context=context)
        if len(natureza_ids) == 1:
            return {'value': {'wiz_banfin_ticb_local_natureza_id': natureza_ids[0]}}
        return {'value': {'wiz_banfin_ticb_local_natureza_id': None}}

    def banfin_wiz_titulos_cobranca_btn_consultar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        exc_ids = [x.id for x in dados_brw.wiz_banfin_ticb_titulos_ids]
        self.pool.get('wiz.banfin.titulos.cb.linhas').unlink(cr, uid, exc_ids, context=context)
        _pessoa_id = dados_brw.wiz_banfin_ticb_pessoa_id.id
        _data_inicial = dados_brw.wiz_banfin_ticb_venc_inicial
        _data_final = dados_brw.wiz_banfin_ticb_venc_final
        _local_ids = [x.id for x in dados_brw.wiz_banfin_ticb_titulos_local_ids]
        args = [('tipo_titulo', '=', 'R'),
                ('state', '=', 'A'),
                ('local_cobranca_id', 'in', _local_ids)]
        if _pessoa_id:
            args.append(('pessoa_id', '=', _pessoa_id))
        if _data_inicial:
            args.append(('data_prorrogacao', '>=', _data_inicial))
        if _data_final:
            args.append(('data_prorrogacao', '<=', _data_final))
        tit_ids = self.pool.get('financeiro.titulo').search(cr, uid, args, context=context)
        if not tit_ids:
            return True
        _hoje = time.strftime('%Y-%m-%d')
        carteira_id = self.pool.get('adm.utils').admutl_get_id_nome_externo(cr, uid, 'financeiro',
                                                                            'local_cobranca_carteira')
        for tit_brw in self.pool.get('financeiro.titulo').browse(cr, uid, tit_ids, context=context):
            if tit_brw.data_prorrogacao > _hoje:
                _situacao = 'A'
            elif tit_brw.data_prorrogacao < _hoje:
                _situacao = 'V'
            else:
                _situacao = 'H'
            if tit_brw.local_cobranca_id.id == carteira_id:
                _estado = 'C'
            else:
                _estado = 'R'
            _prorrogacao = _prorrogar = False
            if _estado == 'R':
                _prorrogacao = True
                _prorrogar = False
            vals = {
                'wiz_banfin_tilicb_capa_id': ids[0],
                'wiz_banfin_tilicb_pessoa_id': tit_brw.pessoa_id.id,
                'wiz_banfin_tilicb_titulo_id': tit_brw.id,
                'wiz_banfin_tilicb_emissao': tit_brw.data_emissao,
                'wiz_banfin_tilicb_vencimento': tit_brw.data_prorrogacao,
                'wiz_banfin_tilicb_valor': tit_brw.vr_original,
                'wiz_banfin_tilicb_situacao': _situacao,
                'wiz_banfin_tilicb_local_cobranca_id': tit_brw.local_cobranca_id.id,
                'wiz_banfin_tilicb_documento': tit_brw.documento,
                'wiz_banfin_tilicb_parcela': tit_brw.parcela,
                'wiz_banfin_tilicb_estado': _estado,
                'wiz_banfin_tilicb_estado_banco': dict_est_bco.get(tit_brw.state_banco),
                'wiz_banfin_tilicb_habil_btn_prorrogacao': _prorrogacao,
                'wiz_banfin_tilicb_habil_btn_prorrogar': _prorrogar,
                'wiz_banfin_tilicb_habil_protesto': dados_brw.wiz_banfin_ticb_habil_protesto,
                'wiz_banfin_tilicb_habil_baixa': dados_brw.wiz_banfin_ticb_habil_baixa,
                'wiz_banfin_tilicb_habil_cancel': dados_brw.wiz_banfin_ticb_habil_cancel,
            }
            self.pool.get('wiz.banfin.titulos.cb.linhas').create(cr, uid, vals, context=context)
        return True


class wiz_banfin_titulos_cb_linhas(osv.TransientModel):
    """
        Classe de Wizard de titulos
        prefixo: wiz_banfin_tilicb_
    """
    _name = 'wiz.banfin.titulos.cb.linhas'
    _columns = {
        'wiz_banfin_tilicb_capa_id': fields.many2one('wiz.banfin.titulos.cobranca', 'Capa', ondelete='cascade',
                                                     select=True, required=True),
        'wiz_banfin_tilicb_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', ondelete="cascade", select=True),
        'wiz_banfin_tilicb_titulo_id': fields.many2one('financeiro.titulo', 'Titulo', ondelete="cascade", select=True),
        'wiz_banfin_tilicb_documento': fields.char('Documento', size=30),
        'wiz_banfin_tilicb_parcela': fields.integer('Parcela'),
        'wiz_banfin_tilicb_emissao': fields.date('Emissao'),
        'wiz_banfin_tilicb_vencimento': fields.date('Vencimento'),
        'wiz_banfin_tilicb_prorrogacao': fields.date('Prorrogacao'),
        'wiz_banfin_tilicb_habil_btn_prorrogacao': fields.boolean('Habilita btn prorrogacao'),
        'wiz_banfin_tilicb_habil_btn_prorrogar': fields.boolean('Habilita btn prorrogar'),
        'wiz_banfin_tilicb_habil_dig_prorrogacao': fields.boolean('Habilita dig prorrogacao'),
        'wiz_banfin_tilicb_valor': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_tilicb_situacao': fields.char('Situacao', size=1),  # V=Vencido, H=Hoje, A=Avencer
        'wiz_banfin_tilicb_estado': fields.char('Estado', size=1),  # C=Carteira, R=Registrado, P=Protesto, B=Baixado
        'wiz_banfin_tilicb_estado_banco': fields.char('No banco', size=20),  # C=Carteira, R=Rejeitado, P=Protesto, B=Baixado, E=Enviado
        'wiz_banfin_tilicb_local_cobranca_id': fields.many2one('financeiro.local.cobranca', 'Local Cob',
                                                               ondelete="cascade", select=True),
        'wiz_banfin_tilicb_habil_protesto': fields.boolean('Habilita protesto'),
        'wiz_banfin_tilicb_habil_baixa': fields.boolean('Habilita baixa'),
        'wiz_banfin_tilicb_habil_cancel': fields.boolean('Habilita cancelamento'),
    }
    _defaults = {
        'wiz_banfin_tilicb_habil_btn_prorrogacao': True,
        'wiz_banfin_tilicb_habil_btn_prorrogar': False,
        'wiz_banfin_tilicb_habil_dig_prorrogacao': False,
        'wiz_banfin_tilicb_habil_protesto': True,
        'wiz_banfin_tilicb_habil_baixa': True,
        'wiz_banfin_tilicb_habil_cancel': True,
    }

    def banfin_onchange_prorrogacao(self, cr, uid, ids, data_prorrogacao, context=None):
        if not data_prorrogacao:
            vals = {
                'wiz_banfin_tilicb_habil_btn_prorrogacao': True,
                'wiz_banfin_tilicb_habil_btn_prorrogar': False,
                'wiz_banfin_tilicb_habil_dig_prorrogacao': False
            }
            return {'value': vals}
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        if data_prorrogacao <= brw_linha.wiz_banfin_tilicb_vencimento:
            vals = {
                'wiz_banfin_tilicb_prorrogacao': False,
                'wiz_banfin_tilicb_habil_btn_prorrogacao': True,
                'wiz_banfin_tilicb_habil_btn_prorrogar': False,
                'wiz_banfin_tilicb_habil_dig_prorrogacao': False
            }
            return {'value': vals}
        vals = {
            'wiz_banfin_tilicb_habil_btn_prorrogacao': False,
            'wiz_banfin_tilicb_habil_btn_prorrogar': True
        }
        return {'value': vals}


    def wiz_banfin_tilicb_btn_prorrogacao(self, cr, uid, ids, context=None):
        vals = {
            'wiz_banfin_tilicb_habil_dig_prorrogacao': True,
            'wiz_banfin_tilicb_habil_btn_prorrogacao': False
        }
        self.write(cr, uid, ids, vals, context=context)
        return True

    def wiz_banfin_tilicb_btn_prorrogar(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').cob_boleto_prorrogar(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                brw_linha.wiz_banfin_tilicb_prorrogacao,
                context=context
        )
        if ret.get('status'):
            vals = {
                'wiz_banfin_tilicb_prorrogacao': False,
                'wiz_banfin_tilicb_habil_btn_prorrogacao': True,
                'wiz_banfin_tilicb_habil_btn_prorrogar': False,
                'wiz_banfin_tilicb_habil_dig_prorrogacao': False,
                'wiz_banfin_tilicb_vencimento': brw_linha.wiz_banfin_tilicb_prorrogacao
            }
            self.write(cr, uid, ids, vals, context=context)
            vals = {'data_prorrogacao': brw_linha.wiz_banfin_tilicb_prorrogacao}
            self.pool.get('financeiro.titulo').write(cr, uid, brw_linha.wiz_banfin_tilicb_titulo_id.id, vals,
                                                     context=context)
            vals_his = {
                'banfin_bolrec_conta_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                'banfin_bolrec_operacao': 'P',
                'banfin_bolrec_pessoa_id': brw_linha.wiz_banfin_tilicb_pessoa_id.id,
                'banfin_bolrec_titulo_id': brw_linha.wiz_banfin_tilicb_titulo_id.id,
                'banfin_bolrec_emissao': brw_linha.wiz_banfin_tilicb_emissao,
                'banfin_bolrec_vencimento': brw_linha.wiz_banfin_tilicb_vencimento,
                'banfin_bolrec_prorrogacao': brw_linha.wiz_banfin_tilicb_prorrogacao,
                'banfin_bolrec_valor': brw_linha.wiz_banfin_tilicb_valor,
            }
            self.pool.get('banfin.bolrec').create(cr, uid, vals_his, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tilicb_btn_registrar(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').cob_boleto_registrar(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_hibrido,
                mensagem1=brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_mensagem_1,
                mensagem2=brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_mensagem_2,
                context=context)
        if ret.get('status'):
            vals = {
                'wiz_banfin_tilicb_estado': 'R',
                'wiz_banfin_tilicb_estado_banco': dict_est_bco.get('E'),
                'wiz_banfin_tilicb_local_cobranca_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_banco_local_id.id,
            }
            self.write(cr, uid, ids[0], vals, context=context)
            valsBol = {
                'fintib_titulo_id': brw_linha.wiz_banfin_tilicb_titulo_id.id,
                'fintib_codigo_barras': ret.get('dados').get('linhaDigitavel'),
                'fintib_local_cobranca_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_banco_local_id.id,
                'fintib_natureza_cobranca_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_local_natureza_id.id,
                'fintib_nosso_numero': ret.get('dados').get('nossoNumero'),
                'fintib_controle_bco': ret.get('dados').get('controle'),
                'fintib_url_pdf': ret.get('dados').get('urlPdf'),
                'fintib_url_pagamento': ret.get('dados').get('urlPagamento'),
                'fintib_pix_hash': ret.get('dados').get('pixHash'),
                'fintib_pix_qrcode': ret.get('dados').get('pixQrCode'),
            }
            self.pool.get('financeiro.titulo.boleto').create(cr, uid, valsBol, context=context)
            valsTit = {
                'state_banco': 'E',
                'local_cobranca_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_banco_local_id.id,
                'natureza_cobranca_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_local_natureza_id.id,
            }
            self.pool.get('financeiro.titulo').write(cr, uid, brw_linha.wiz_banfin_tilicb_titulo_id.id, valsTit,
                                                     context=context)
            vals_his = {
                'banfin_bolrec_conta_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                'banfin_bolrec_operacao': 'R',
                'banfin_bolrec_pessoa_id': brw_linha.wiz_banfin_tilicb_pessoa_id.id,
                'banfin_bolrec_titulo_id': brw_linha.wiz_banfin_tilicb_titulo_id.id,
                'banfin_bolrec_emissao': brw_linha.wiz_banfin_tilicb_emissao,
                'banfin_bolrec_vencimento': brw_linha.wiz_banfin_tilicb_vencimento,
                'banfin_bolrec_prorrogacao': brw_linha.wiz_banfin_tilicb_prorrogacao,
                'banfin_bolrec_valor': brw_linha.wiz_banfin_tilicb_valor,
            }
            self.pool.get('banfin.bolrec').create(cr, uid, vals_his, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tilicb_btn_baixar(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').cob_boleto_baixar(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                context=context)
        if ret.get('status'):
            carteira_id = self.pool.get('adm.utils').admutl_get_id_nome_externo(cr, uid, 'financeiro',
                                                                                'local_cobranca_carteira')
            bol_ids = self.pool.get('financeiro.titulo.boleto').fintib_get_boleto_ids(cr, uid,
                                              brw_linha.wiz_banfin_tilicb_titulo_id.id,
                                              local_id=brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_banco_local_id.id,
                                              context=context)
            self.pool.get('financeiro.titulo.boleto').write(cr, uid, bol_ids, {'state': 'I'}, context=context)
            vals = {
                'wiz_banfin_tilicb_estado': 'B',
                'wiz_banfin_tilicb_estado_banco': dict_est_bco.get('B'),
                'wiz_banfin_tilicb_local_cobranca_id': carteira_id,
            }
            self.write(cr, uid, ids[0], vals, context=context)
            valsTit = {
                'state_banco': 'B',
                'local_cobranca_id': carteira_id,
                'natureza_cobranca_id': None,
            }
            self.pool.get('financeiro.titulo').write(cr, uid, brw_linha.wiz_banfin_tilicb_titulo_id.id, valsTit,
                                                     context=context)
            vals_his = {
                'banfin_bolrec_conta_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                'banfin_bolrec_operacao': 'B',
                'banfin_bolrec_pessoa_id': brw_linha.wiz_banfin_tilicb_pessoa_id.id,
                'banfin_bolrec_titulo_id': brw_linha.wiz_banfin_tilicb_titulo_id.id,
                'banfin_bolrec_emissao': brw_linha.wiz_banfin_tilicb_emissao,
                'banfin_bolrec_vencimento': brw_linha.wiz_banfin_tilicb_vencimento,
                'banfin_bolrec_prorrogacao': brw_linha.wiz_banfin_tilicb_prorrogacao,
                'banfin_bolrec_valor': brw_linha.wiz_banfin_tilicb_valor,
            }
            self.pool.get('banfin.bolrec').create(cr, uid, vals_his, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tilicb_btn_cancelar(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').cob_boleto_cancelar(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                context=context)
        if ret.get('status'):
            carteira_id = self.pool.get('adm.utils').admutl_get_id_nome_externo(cr, uid, 'financeiro',
                                                                                'local_cobranca_carteira')
            bol_ids = self.pool.get('financeiro.titulo.boleto').fintib_get_boleto_ids(cr, uid,
                                              brw_linha.wiz_banfin_tilicb_titulo_id.id,
                                              local_id=brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_banco_local_id.id,
                                              context=context)
            self.pool.get('financeiro.titulo.boleto').unlink(cr, uid, bol_ids, context=context)
            vals = {
                'wiz_banfin_tilicb_estado': 'C',
                'wiz_banfin_tilicb_estado_banco': dict_est_bco.get('A'),
                'wiz_banfin_tilicb_local_cobranca_id': carteira_id,
            }
            self.write(cr, uid, ids[0], vals, context=context)
            valsTit = {
                'state_banco': 'A',
                'local_cobranca_id': carteira_id,
                'natureza_cobranca_id': None,
            }
            self.pool.get('financeiro.titulo').write(cr, uid, brw_linha.wiz_banfin_tilicb_titulo_id.id, valsTit,
                                                     context=context)
            vals_his = {
                'banfin_bolrec_conta_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                'banfin_bolrec_operacao': 'C',
                'banfin_bolrec_pessoa_id': brw_linha.wiz_banfin_tilicb_pessoa_id.id,
                'banfin_bolrec_titulo_id': brw_linha.wiz_banfin_tilicb_titulo_id.id,
                'banfin_bolrec_emissao': brw_linha.wiz_banfin_tilicb_emissao,
                'banfin_bolrec_vencimento': brw_linha.wiz_banfin_tilicb_vencimento,
                'banfin_bolrec_prorrogacao': brw_linha.wiz_banfin_tilicb_prorrogacao,
                'banfin_bolrec_valor': brw_linha.wiz_banfin_tilicb_valor,
            }
            self.pool.get('banfin.bolrec').create(cr, uid, vals_his, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tilicb_btn_protestar(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        _operacao = brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_negativ_oper
        _prazo = brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_negativ_prazo
        ret = self.pool.get('bf.integracao').cob_boleto_protestar(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                operacao=_operacao,
                prazo=_prazo,
                context=context)
        if ret.get('status'):
            if _operacao in ('1', '2'):
                _oper = 'N'
            elif _operacao == ('3'):
                _oper = 'C'
            else:
                _oper = 'B'
            vals_his = {
                'banfin_bolrec_conta_id': brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                'banfin_bolrec_operacao': _oper,
                'banfin_bolrec_pessoa_id': brw_linha.wiz_banfin_tilicb_pessoa_id.id,
                'banfin_bolrec_titulo_id': brw_linha.wiz_banfin_tilicb_titulo_id.id,
                'banfin_bolrec_emissao': brw_linha.wiz_banfin_tilicb_emissao,
                'banfin_bolrec_vencimento': brw_linha.wiz_banfin_tilicb_vencimento,
                'banfin_bolrec_prorrogacao': brw_linha.wiz_banfin_tilicb_prorrogacao,
                'banfin_bolrec_valor': brw_linha.wiz_banfin_tilicb_valor,
            }
            self.pool.get('banfin.bolrec').create(cr, uid, vals_his, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tilicb_btn_imprimir(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        url = False
        for brw_boleto in brw_linha.wiz_banfin_tilicb_titulo_id.fintit_boleto_ids:
            if brw_boleto.state == 'A':
                if brw_boleto.fintib_local_cobranca_id.id == brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_banco_local_id.id:
                    url = brw_boleto.fintib_url_pdf
                    break
        if url:
            return {
                'type': 'ir.actions.act_url',
                'url': url,
                'target': 'new',
            }
        else:
           res = self.pool.get('bf.integracao').cob_boleto_get_pdf(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                context=context)
           if res.get('status'):
                _pdf = res.get('dados').get('pdf')
                if _pdf:
                    return {'type': 'ir.actions.report.xml',
                           'report_name': 'relatorios',
                           'datas': {'pdf': {'estrutura': {'pdf': True},
                                             'dados': _pdf}}}
        return False

    def wiz_banfin_tilicb_btn_consultar(self, cr, uid, ids, context=None):
        brw_linha = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').cob_boleto_consultar(cr, uid,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                brw_linha.wiz_banfin_tilicb_capa_id.wiz_banfin_ticb_conta_id.id,
                brw_linha.wiz_banfin_tilicb_titulo_id.id,
                context=context)
        if ret.get('status'):
            dados = ret.get('dados')
            vals = {
                'wiz_banfin_ticu_situacao': dados.get('situacao') or None,
                'wiz_banfin_ticu_situacao_codigo': dados.get('situacaoCodigo') or None,
                'wiz_banfin_ticu_emissao': dados.get('emissao') or None,
                'wiz_banfin_ticu_vencimento': dados.get('vencimento') or None,
                'wiz_banfin_ticu_data_registro': dados.get('dataRegistro') or None,
                'wiz_banfin_ticu_linha_digitavel': dados.get('linhaDigitavel') or None,
                'wiz_banfin_ticu_data_limite': dados.get('dataLimite') or None,
                'wiz_banfin_ticu_documento': dados.get('documento') or None,
                'wiz_banfin_ticu_nossoNumero': dados.get('nossoNumero') or None,
                'wiz_banfin_ticu_valor': dados.get('valor') or None,
                'wiz_banfin_ticu_pix_hash': dados.get('pix_hash') or None,
                'wiz_banfin_ticu_sacado': dados.get('sacado') or None,
                'wiz_banfin_ticu_baixa_data': dados.get('baixaData') or None,
                'wiz_banfin_ticu_desconto_data': dados.get('descontoData') or None,
                'wiz_banfin_ticu_desconto': dados.get('desconto') or None,
                'wiz_banfin_ticu_desconto_recebido': dados.get('descontoRecebido') or None,
                'wiz_banfin_ticu_juros': dados.get('juros') or None,
                'wiz_banfin_ticu_juros_recebido': dados.get('jurosRecebido') or None,
                'wiz_banfin_ticu_multa': dados.get('multa') or None,
                'wiz_banfin_ticu_multa_data': dados.get('multaData') or None,
                'wiz_banfin_ticu_valor_pago': dados.get('valorPago') or None,
                'wiz_banfin_ticu_protesto_data': dados.get('protestoData') or None,
                'wiz_banfin_ticu_url_boleto': dados.get('url_boleto') or None,
                'wiz_banfin_ticu_pdf_boleto': dados.get('pdf_boleto') or None,
            }
            wiz_id = self.pool.get('wiz.banfin.cob.titulos.um').create(cr, uid, vals, context=context)
            value = {
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wiz.banfin.cob.titulos.um',
                'type': 'ir.actions.act_window',
                'target': 'new',
                'context': context,
                'res_id': wiz_id
            }
            if context.get('id_messagebox'):
                value.update({'res_id': context['id_messagebox']})
            return value

        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))
