# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time
from openerp.tools.translate import _


class wiz_banfin_titulos_pagar(osv.TransientModel):
    """
        Classe de Wizard para pagamento de titulos
        prefixo: wiz_banfin_tipg_
    """
    _name = 'wiz.banfin.titulos.pagar'
    _columns = {
        'wiz_banfin_tipg_pessoa_id': fields.many2one('adm.pessoa', 'Fornecedor', ondelete="cascade", select=True),
        'wiz_banfin_tipg_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', select=True,
                                                    domain="[('banco_caixa', '=', 'B')]", ondelete="cascade",
                                                    required=True),
        'wiz_banfin_tipg_venc_inicial': fields.date('Vencimento inicial'),
        'wiz_banfin_tipg_venc_final': fields.date('Vencimento final'),
        'wiz_banfin_tipg_titulos_ids': fields.one2many('wiz.banfin.titulos.pg.linhas', 'wiz_banfin_tili_capa_id',
                                                       'Titulos'),
    }

    def banfin_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        if not conta_id:
            return {'value': {}}
        conta_brw = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        if not conta_brw.banco_id.admbco_codigo_interno_id:
            return {'warning': {'title': _('Aviso'),
                                'message': _('Conta de tesouraria sem integracao bancaria!')}}
        return {'value': {}}

    def banfin_wiz_titulos_pagar_btn_consultar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        exc_ids = [x.id for x in dados_brw.wiz_banfin_tipg_titulos_ids]
        self.pool.get('wiz.banfin.titulos.pg.linhas').unlink(cr, uid, exc_ids, context=context)
        _pessoa_id = dados_brw.wiz_banfin_tipg_pessoa_id.id
        _data_inicial = dados_brw.wiz_banfin_tipg_venc_inicial
        _data_final = dados_brw.wiz_banfin_tipg_venc_final
        args = [('tipo_titulo', '=', 'P'), ('state', '=', 'A')]
        if _pessoa_id:
            args.append(('pessoa_id', '=', _pessoa_id))
        if _data_inicial:
            args.append(('data_prorrogacao', '>=', _data_inicial))
        if _data_final:
            args.append(('data_prorrogacao', '<=', _data_final))
        tit_ids = self.pool.get('financeiro.titulo').search(cr, uid, args, context=context)
        if not tit_ids:
            return True
        _hoje = time.strftime('%Y-%m-%d')
        for tit_brw in self.pool.get('financeiro.titulo').browse(cr, uid, tit_ids, context=context):
            if (tit_brw.fintit_pgm_banco_pagador and
                tit_brw.fintit_pgm_banco_pagador != str(dados_brw.wiz_banfin_tipg_conta_id.banco_id.admbco_codigo_bc)):
                continue
            if tit_brw.data_prorrogacao > _hoje:
                _hoje = tit_brw.data_prorrogacao
            vals = {
                'wiz_banfin_tili_capa_id': ids[0],
                'wiz_banfin_tili_pessoa_id': tit_brw.pessoa_id.id,
                'wiz_banfin_tili_titulo_id': tit_brw.id,
                'wiz_banfin_tili_emissao': tit_brw.data_emissao,
                'wiz_banfin_tili_vencimento': tit_brw.data_prorrogacao,
                'wiz_banfin_tili_datapagto': _hoje,
                'wiz_banfin_tili_valor': tit_brw.vr_original,
                'wiz_banfin_tili_valor_a_pagar': tit_brw.vr_original,
                'wiz_banfin_tili_agendado_banco': tit_brw.fintit_pgm_banco_pagador,
                'wiz_banfin_tili_codbarra': tit_brw.fintit_boleto_codigo_barras,
                'wiz_banfin_tili_operacao': tit_brw.fintit_pgm_operacao,
            }
            self.pool.get('wiz.banfin.titulos.pg.linhas').create(cr, uid, vals, context=context)
        return True


class wiz_banfin_titulos_pg_linhas(osv.TransientModel):
    """
        Classe de Wizard de titulos
        prefixo: wiz_banfin_tili_
    """
    _name = 'wiz.banfin.titulos.pg.linhas'
    _columns = {
        'wiz_banfin_tili_capa_id': fields.many2one('wiz.banfin.titulos.pagar', 'Capa', ondelete='cascade', select=True,
                                                   required=True),
        'wiz_banfin_tili_pessoa_id': fields.many2one('adm.pessoa', 'Fornecedor', ondelete="cascade", select=True),
        'wiz_banfin_tili_titulo_id': fields.many2one('financeiro.titulo', 'Titulo', ondelete="cascade", select=True),
        'wiz_banfin_tili_emissao': fields.date('Emissao'),
        'wiz_banfin_tili_vencimento': fields.date('Vencimento'),
        'wiz_banfin_tili_datapagto': fields.date('Data pagamento'),
        'wiz_banfin_tili_valor': fields.float('Valor', digits=(9, 2)),
        'wiz_banfin_tili_valor_a_pagar': fields.float('Valora pagar', digits=(9, 2)),
        'wiz_banfin_tili_descricao': fields.char('Descricao pagto', size=60),
        'wiz_banfin_tili_agendado_banco': fields.char('Banco agendado', size=3),
        'wiz_banfin_tili_codbarra': fields.char('Codigo de barras', size=48),
        'wiz_banfin_tili_breg_beneficiario': fields.char('Beneficiario', size=60),
        'wiz_banfin_tili_breg_beneficiario_cnpj': fields.char('Cnpj', size=18),
        'wiz_banfin_tili_breg_vencimento': fields.date('Vencimento'),
        'wiz_banfin_tili_breg_valor': fields.float('Valor registrado', digits=(9, 2)),
        'wiz_banfin_tili_breg_multa': fields.float('Multa', digits=(9, 2)),
        'wiz_banfin_tili_breg_juros': fields.float('Juros', digits=(9, 2)),
        'wiz_banfin_tili_breg_desconto': fields.float('Desconto', digits=(9, 2)),
        'wiz_banfin_tili_breg_total': fields.float('Total a pagar', digits=(9, 2)),
        'wiz_banfin_tili_operacao': fields.selection([('000', '000-Inclusao de pagamento'),
                                                                    ('517', '517-Alteracao de valor de pagamento'),
                                                                    ('519', '519-Alteracao de data de pagamento'),
                                                                    ('999', '999-Exclusao de pagamento incluido')],
                                                                    'Operacao', size=3),
    }

    def wiz_banfin_tili_btn_consultar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').pag_consultar_boleto(cr, uid,
             dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
             dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.id,
             dados_brw.wiz_banfin_tili_codbarra,
             context=context)
        if ret.get('status'):
            _dados_boleto = ret.get('dados')
            vals = {
                'wiz_banfin_tili_breg_beneficiario': _dados_boleto.get('beneficiarioNome'),
                'wiz_banfin_tili_breg_beneficiario_cnpj': _dados_boleto.get('beneficiarioCnpj'),
                'wiz_banfin_tili_breg_vencimento': _dados_boleto.get('vencimento'),
                'wiz_banfin_tili_breg_valor': _dados_boleto.get('valor'),
                'wiz_banfin_tili_breg_multa': _dados_boleto.get('multa'),
                'wiz_banfin_tili_breg_juros': _dados_boleto.get('juros'),
                'wiz_banfin_tili_breg_desconto': _dados_boleto.get('desconto'),
                'wiz_banfin_tili_breg_total': _dados_boleto.get('total'),
            }
            self.write(cr, uid, ids, vals, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tili_btn_agendar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_brw = self.browse(cr, uid, ids[0], context=context)
        ret = self.pool.get('bf.integracao').pag_boleto_incluir(cr, uid,
                     dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.banco_id.admbco_codigo_interno_id.banfin_cdin_codigo_interno,
                     dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.id,
                     dados_brw.wiz_banfin_tili_codbarra,
                     dados_brw.wiz_banfin_tili_valor_a_pagar,
                     dados_brw.wiz_banfin_tili_datapagto,
                     dados_brw.wiz_banfin_tili_vencimento,
                     dados_brw.wiz_banfin_tili_descricao,
                     context=context)
        if ret.get('status'):
            vals = {
                'banfin_tipg_hs_pessoa_id': dados_brw.wiz_banfin_tili_pessoa_id.id,
                # 'banfin_tipg_hs_titulo_id': dados_brw.wiz_banfin_tili_titulo_id.id,
                'banfin_tipg_hs_conta_id':  dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.id,
                'banfin_tipg_hs_descricao': dados_brw.wiz_banfin_tili_descricao,
                'banfin_tipg_hs_valor': dados_brw.wiz_banfin_tili_valor_a_pagar,
                'banfin_tipg_hs_data': dados_brw.wiz_banfin_tili_datapagto,
            }
            self.pool.get('banfin.titulo.pag.historicos').create(cr, uid, vals, context=context)
            self.write(cr, uid, ids[0], {'wiz_banfin_tili_agendado_banco': dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.banco_id.admbco_codigo_bc},
                       context=context)
            vals = {
                'fintit_pgm_banco_pagador': dados_brw.wiz_banfin_tili_capa_id.wiz_banfin_tipg_conta_id.banco_id.admbco_codigo_bc,
                'fintit_pgm_data_pagamento': dados_brw.wiz_banfin_tili_datapagto,
                'fintit_pgm_valor': dados_brw.wiz_banfin_tili_valor_a_pagar,
                'fintit_pgm_operacao': '000',
            }
            self.pool.get('financeiro.titulo').write(cr, uid, [dados_brw.wiz_banfin_tili_titulo_id.id], vals, context=context)
            return True
        raise osv.except_osv(_('Atencao'), ret.get('mensagem'))

    def wiz_banfin_tili_btn_cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'), _('Funcionalidade nao disponivel neste banco!'))

