# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from ..bancos_financeiro.integracao import integracao
from inter import Inter
from datetime import datetime
from SigUtils.datas import getDataNoFormato, calcVencimento, calcDiferenca
import time
import uuid


class integracao(osv.Model):
    _inherit = 'bf.integracao'

    def bank_get_extrato_pdf(self, cr, uid, codigo_banco_interno, conta_id, data_inicial, data_final, context=None):
        res = super(integracao, self).bank_get_extrato_pdf(cr, uid, codigo_banco_interno, conta_id, data_inicial,
                                                           data_final, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_bank_get_extrato_pdf(data_inicial, data_final)
        return res

    def bank_get_extrato(self, cr, uid, codigo_banco_interno, conta_id, data_inicial, data_final, context=None):
        res = super(integracao, self).bank_get_extrato(cr, uid, codigo_banco_interno, conta_id, data_inicial,
                                                       data_final, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_bank_get_extrato(data_inicial, data_final)
        if res.get('status'):
            linhas = []
            for reg in res.get('dados').get('transacoes'):
                linha = {
                    'credito_debito': reg.get('tipoOperacao'),
                    'valor': reg.get('valor'),
                    'data': getDataNoFormato(reg.get('dataEntrada'), 'I', '-'),
                    'tipo_transacao': reg.get('tipoTransacao'),
                    'historico': '%s %s' % (reg.get('titulo'), reg.get('descricao')),
                    'documento': '',
                }
                linhas.append(linha)
            res['dados'] = linhas
        return res

    def bank_get_saldo(self, cr, uid, codigo_banco_interno, conta_id, data=None, context=None):
        res = super(integracao, self).bank_get_saldo(cr, uid, codigo_banco_interno, conta_id, data=data,
                                                     context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_bank_get_saldo(data=data)
        dados = res.get('dados')
        bloqueadoCheque = dados.get('bloqueadoCheque') or 0.0
        disponivel = dados.get('disponivel')
        bloqueadoJudicialmente = dados.get('bloqueadoJudicialmente') or 0.0
        bloqueadoAdministrativo = dados.get('bloqueadoAdministrativo') or 0.0
        limite = dados.get('limite') or 0.0
        vals = {
            'saldoDisponivel': disponivel,
            'saldo': disponivel - (bloqueadoCheque + bloqueadoJudicialmente + bloqueadoAdministrativo),
            'bloqueado': bloqueadoCheque + bloqueadoJudicialmente + bloqueadoAdministrativo,
            'limite': limite,
        }
        res['dados'] = vals
        return res

    def pix_rec_enviar(self, cr, uid, codigo_banco_interno, conta_id, valor, pessoa_id=None, vencimento=None,
                       tempoVida=None, multa=None, juros=None, desconto=None, prazoDesconto=None, context=None):
        res = super(integracao, self).pix_rec_enviar(cr, uid, codigo_banco_interno, conta_id, valor,
                                                     pessoa_id=pessoa_id, vencimento=vencimento,
                                                     tempoVida=tempoVida, multa=multa, juros=juros,
                                                     desconto=desconto, prazoDesconto=prazoDesconto,
                                                     context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        _hoje = time.strftime('%Y-%m-%d')
        controle = str(uuid.uuid4()).replace('-', '')
        if vencimento == _hoje:
            res = bInter.inter_pix_rec_enviar(controle, valor, tempoVida)
        else:
            valsPessoa = {}
            if pessoa_id:
                brwPessoa = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id, context=context)
                valsPessoa = {
                    'cpfCnpj': brwPessoa.cpf_cnpj,
                    'pessoaTipo': brwPessoa.pessoa_tipo,
                    'nome': brwPessoa.nome,
                    'rua': brwPessoa.endereco,
                    'cidade': brwPessoa.cidade,
                    'uf': brwPessoa.uf,
                    'cep': brwPessoa.cep,
                    'email': brwPessoa.email,
                }
            res = bInter.inter_pix_rec_agendamento(controle, valor, vencimento, dadosPessoa=valsPessoa, multa=multa,
                                                       juros=juros, desconto=desconto, prazoDesconto=prazoDesconto)
        # Trocar dados do inter para dados gerais
        if res.get('status'):
            # Trocar dados do inter para dados gerais
            ret = res.get('dados')
            dados = {
                'hash': '',
                'qrcode': ret.get('pixCopiaECola'),
                'controle': controle,
            }
            res['dados'] = dados
        return res

    def pix_rec_consultar(self, cr, uid, codigo_banco_interno, conta_id, controle=None, hash=None, context=None):
        res = super(integracao, self).pix_rec_consultar(cr, uid, codigo_banco_interno, conta_id, controle=controle,
                                                    hash=hash, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_pix_rec_consultar(controle)
        valorPago = ''
        valor = ''
        situacao = ''
        if res.get('status'):
            # Trocar dados do inter para dados gerais
            ret = res.get('dados')
            if ret.get('status') == 'ATIVA':
                situacao = 'P'
            if ret.get('status') == 'CONCLUIDA':
                if ret.get('pix')[0].get('devolucoes'):
                    situacao = 'D'
                else:
                    situacao = 'R'
                    valorPago = ret.get('pix')[0].get('valor')
                    hash = ret.get('pix')[0].get('endToEndId')
            valor = ret.get('valor')
        else:
            if res.get('status_code') == 404:
                situacao = 'A'
                res['status'] = True
        dados = {
            'situacao': situacao,
            'valor': valor,
            'valorPago': valorPago,
        }
        if situacao == 'R':
            dados['hash'] = hash
        res['dados'] = dados
        return res

    def pix_devolucao(self, cr, uid, codigo_banco_interno, conta_id, hash=None, controle=None, motivo=None, valor=None,
                      context=None):
        res = super(integracao, self).pix_devolucao(cr, uid, codigo_banco_interno, conta_id, hash=hash,
                                                    controle=controle, motivo=motivo, valor=valor, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_pix_rec_devolucao(hash, controle, valor)
        return res

    def pix_pag_consultar(self, cr, uid, codigo_banco_interno, conta_id, controle=None, endToEnd=None, context=None):
        res = super(integracao, self).pix_pag_consultar(cr, uid, codigo_banco_interno, conta_id, controle=controle,
                                                    endToEnd=endToEnd, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_pix_pag_consultar(endToEnd, controle)
        valor = ''
        situacao = ''
        if res.get('status'):
            # Trocar dados do inter para dados gerais
            ret = res.get('dados')
            if not ret:
                return {'status': False, 'mensagem': 'Pix nao encontrado!'}
            if ret.get('status') == 'ATIVA':
                situacao = 'P'
            elif ret.get('status') == 'EFETIVADO':
                situacao = 'R'
            else:
                situacao = 'X'
            valor = ret.get('valor')
            dados = {
                'situacao': situacao,
                'valor': valor,
                'endToEnd': ret.get('endToEnd'),
                'recebedor': ret.get('recebedor').get('nome'),
                'cpfCnpj': ret.get('recebedor').get('cpfCnpj'),
                'dataHoraMovimento': ret.get('dataHoraMovimento'),
            }
            res['dados'] = dados
        return res

    def pix_pag_enviar(self, cr, uid, codigo_banco_interno, conta_id, chave, valor, descricao=None, data=None,
                         context=None):
        res = super(integracao, self).pix_pag_enviar(cr, uid, codigo_banco_interno, conta_id, chave, valor,
                                                     descricao=descricao, data=data, context=context)
        if codigo_banco_interno != 'inter':
            return res
        if data:
            _hoje = datetime.now().strftime('YYYY/MM/DD')
            if data == _hoje:
                data = None
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_pix_pag_incluir(chave, valor, descricao=descricao, data=data)
        if res.get('status'):
            ret = res.get('dados')
            dados = {
                'endToEndId': ret.get('endToEndId'),
                'controle': ret.get('controle'),
                'codigoSolicitacao': ret.get('codigoSolicitacao'),
                'dataPagamento': ret.get('dataPagamento'),
                'dataOperacao': ret.get('dataOperacao'),
                'tipoRetorno': ret.get('tipoRetorno'),
            }
            res['dados'] = dados
        return res

    def pag_boleto_incluir(self, cr, uid, codigo_banco_interno, conta_id, codbarra, valor, datapagto, vencimento,
                    descricao=None, context=None):
        res = super(integracao, self).pag_boleto_incluir(cr, uid, codigo_banco_interno, conta_id, codbarra, valor, datapagto,
                                                  vencimento, descricao=descricao, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_pag_boleto_incluir(codbarra, valor, datapagto, vencimento)
        return res

    def pag_consultar_boleto(self, cr, uid, codigo_banco_interno, conta_id, codigobarra, context=None):
        res = super(integracao, self).pag_consultar_boleto(cr, uid, codigo_banco_interno, conta_id, codigobarra,
                                                           context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_pag_consultar_boleto(codigobarra)
        if res.get('status'):
            ret = res.get('dados')[0]
            vals = {
                'beneficiarioNome': ret.get('nomeBeneficiario'),
                'beneficiarioCnpj': ret.get('cpfCnpjBeneficiario'),
                'vencimento': ret.get('dataVencimentoTitulo'),
                'valor': ret.get('valorNominal'),
                'multa': 0,
                'juros': 0,
                'desconto': 0,
                'total': ret.get('valorPago'),
                'status': ret.get('statusPagamento'),
            }
            res['dados'] = vals
        return res


    def cob_boleto_registrar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, hibrido=False, mensagem1=None,
                             mensagem2=None, context=None):
        res = super(integracao, self).cob_boleto_registrar(cr, uid, codigo_banco_interno, conta_id, titulo_id,
                                                           hibrido=hibrido, mensagem1=mensagem1, mensagem2=mensagem2,
                                                           context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        brwTitulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context=context)
        brwConta = self.pool.get('financeiro.tesouraria.contas').browse(cr, uid, conta_id, context=context)
        endereco = self.pool.get('adm.pessoa').admpes_get_endereco_separado(cr, uid, brwTitulo.pessoa_id.id, context=context)
        if not brwTitulo.data_limite_desconto:
            _diasDesconto = 0
        else:
            _diasDesconto = calcDiferenca(brwTitulo.data_limite_desconto, brwTitulo.data_prorrogacao)
        vals = {
            'tituloId': titulo_id,
            'tituloNumero': brwTitulo.documento,
            'tituloParcela': brwTitulo.parcela,
            'emissao': brwTitulo.data_emissao,
            'valor': brwTitulo.vr_original,
            'vencimento': brwTitulo.data_prorrogacao,
            'descontoValor': brwTitulo.vr_desconto,
            'descontoDias': _diasDesconto,
            'multaValor': brwTitulo.vr_multa,
            'multaData': brwTitulo.multa_apos_data,
            'jurosValor': brwTitulo.vr_juros_dia,
            'jurosData': brwTitulo.multa_apos_data,
            'sacado': {
                'nome': brwTitulo.pessoa_id.nome,
                'cpfCnpjTipo': 'CPF' if brwTitulo.pessoa_id.pessoa_tipo == 'F' else 'CNPJ',
                'cpfCnpj': brwTitulo.pessoa_id.cpf_cnpj,
                'cep': brwTitulo.pessoa_id.cep,
                'enderecoRua': endereco.get('endereco_rua'),
                'enderecoNro': endereco.get('endereco_nro'),
                'enderecoCompl': endereco.get('endereco_compl'),
                'cidade': brwTitulo.pessoa_id.cidade,
                'bairro': brwTitulo.pessoa_id.bairro,
                'uf': brwTitulo.pessoa_id.uf,
                'fone': brwTitulo.pessoa_id.telefone,
                'email': brwTitulo.pessoa_id.email,
            },
            'beneficiario': {
                'nome': brwConta.company_id.rescon_pessoa_id.nome,
                'cpfCnpj': brwConta.company_id.rescon_pessoa_id.cpf_cnpj,
                'tipoCpfCnpj': 'CPF' if brwConta.company_id.rescon_pessoa_id.pessoa_tipo == 'F' else 'CNPJ',
                'cep': brwConta.company_id.rescon_pessoa_id.cep,
                'endereco': brwConta.company_id.rescon_pessoa_id.endereco,
                'bairro': brwConta.company_id.rescon_pessoa_id.bairro,
                'cidade': brwConta.company_id.rescon_pessoa_id.cidade,
                'uf': brwConta.company_id.rescon_pessoa_id.uf,
            },
            'hibrido': hibrido,
            'mensagem1': mensagem1,
            'mensagem2': mensagem2,
        }
        res = bInter.inter_cob_boleto_registrar(**vals)
        # Trocar dados do inter para dados gerais
        if res.get('status'):
            ret = res.get('dados')
            dados = {
                'nossoNumero': ret.get('nossoNumero'),
                'controle': ret.get('controle'),
                'linhaDigitavel': ret.get('linhaDigitavel'),
                'codigoBarras': ret.get('codigoBarras'),
                'urlPdf': '',
                'boletoPdf': ret.get('boletoPdf'),
                'urlPagamento': '',
                'pixHash': '',
                'pixQrCode': '',
            }
            res['dados'] = dados
        return res

    def cob_boleto_get_pdf(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        res = super(integracao, self).cob_boleto_get_pdf(cr, uid, codigo_banco_interno, conta_id, titulo_id,
                                                           context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        brwTitulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context=context)
        controle = False
        for brw_boleto in brwTitulo.fintit_boleto_ids:
            if brw_boleto.state == 'A':
                if brw_boleto.fintib_local_cobranca_id.id == brwTitulo.local_cobranca_id.id:
                    controle = brw_boleto.fintib_controle_bco
                    break
        res = bInter.inter_cob_boleto_get_pdf(controle)
        if res.get('status'):
            # Trocar dados do inter para dados gerais
            ret = res.get('dados')
            dados = {
                'urlPdf': '',
                'pdf':  ret.get('pdf'),
            }
            res['dados'] = dados
        return res

    def cob_boleto_consultar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        res = super(integracao, self).cob_boleto_consultar(cr, uid, codigo_banco_interno, conta_id, titulo_id,
                                                          context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        brwTitulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context=context)
        controle = False
        for brw_boleto in brwTitulo.fintit_boleto_ids:
            if brw_boleto.state == 'A':
                if brw_boleto.fintib_local_cobranca_id.id == brwTitulo.local_cobranca_id.id:
                    controle = brw_boleto.fintib_controle_bco
                    break
        res = bInter.inter_cob_boleto_consultar(controle)
        if res.get('status'):
            # Trocar dados do inter para dados gerais
            ret = res.get('dados')
            _dias = 0
            for x in ret.get('cobranca').get('descontos'):
                _dias = x.get('quantidadeDias')
                break
            _venc = ret.get('cobranca').get('dataVencimento')
            if _dias == 0:
                _dtDesconto = _venc
            else:
                _dtDesconto = getDataNoFormato(calcVencimento(_venc, (_dias * -1)), 'I', '-')
            dados = {
                'situacao': ret.get('cobranca').get('situacao'),
                'nossoNumero': ret.get('boleto').get('nossoNumero'),
                'documento': ret.get('cobranca').get('seuNumero'),
                'sacado': ret.get('cobranca').get('pagador').get('nome'),
                'vencimento': _venc,
                'emissao': ret.get('cobranca').get('dataEmissao'),
                'valor': ret.get('cobranca').get('valorNominal'),
                'linhaDigitavel': ret.get('boleto').get('linhaDigitavel'),
                'descontoData': _dtDesconto,
                'multa': ret.get('cobranca').get('multa').get('valor') if ret.get('cobranca').get('multa') else 0,
                'multaData': _venc,
                'juros': ret.get('cobranca').get('mora').get('valor') if ret.get('cobranca').get('mora') else 0,
                'pdf_boleto': ret.get('pdf_boleto'),
            }
            if ret.get('pis'):
                dados['pixQrCode'] = ret.get('pix').get('pixCopiaECola')
                dados['pix_hash'] = ret.get('pix').get('txid')
            if ret.get('situacao') == 'RECEBIDO':
                dados['baixaData'] = ret.get('cobranca').get('dataSituacao')
                dados['valorPago'] = ret.get('cobranca').get('valorTotalRecebido')
            res['dados'] = dados
        return res

    def cob_boleto_listar(self, cr, uid, codigo_banco_interno, conta_id, emis_inicial=None, emis_final=None,
                          venc_inicial=None, venc_final=None, movi_inicial=None, movi_final=None, context=None):
        res = super(integracao, self).cob_boleto_listar(cr, uid, codigo_banco_interno, conta_id,
                                                        emis_inicial=emis_inicial, emis_final=emis_final,
                                                        venc_inicial=venc_inicial, venc_final=venc_final,
                                                        movi_inicial=movi_inicial, movi_final=movi_final,
                                                        context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        if venc_inicial:
            tipoData = 'V'
            _data_inicial = venc_inicial
            _data_final = venc_final
        elif emis_inicial:
            tipoData = 'E'
            _data_inicial = emis_inicial
            _data_final = emis_final
        else:
            tipoData = 'M'
            _data_inicial = movi_inicial
            _data_final = movi_final
        linhas = []
        if not (_data_inicial and _data_final):
            return {'status': False, 'mensagem': _('Necessario informar uma data inicial e uma final')}
        data_inicial = _data_inicial
        data_final = _data_final
        res = bInter.inter_cob_boleto_listar(data_inicial=data_inicial, data_final=data_final, tipoData=tipoData)
        if not res.get('status'):
            raise osv.except_osv(_('Atencao'), res.get('mensagem'))
        # Trocar dados do inter para dados gerais
        ret = res.get('dados')
        for x in ret.get('cobrancas'):
            _controle = x.get('cobranca').get('seuNumero')
            titulo_id = _controle
            if '-' in _controle:
                titulo_id = _controle.split('-')[0]
            titulo_id = int(titulo_id)
            _controle = x.get('cobranca').get('codigoSolicitacao')
            tit_ids = self.pool.get('financeiro.titulo').search(cr, uid, [('id', '=', titulo_id)], context=context)
            if not tit_ids:
                continue
            brwTitulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context=context)
            dados = {
                'pessoa_id': brwTitulo.pessoa_id.id,
                'titulo_id': titulo_id,
                'documento': brwTitulo.documento,
                'parcela': brwTitulo.parcela,
                'emissao': brwTitulo.data_emissao,
                'vencimento': brwTitulo.data_prorrogacao,
                'valor': brwTitulo.vr_original,
                'situacao': x.get('cobranca').get('situacao') or '',
                'controle': _controle,
            }
            linhas.append(dados)
        res['dados'] = linhas
        return res

    def cob_boleto_cancelar(self, cr, uid, codigo_banco_interno, conta_id, titulo_id, context=None):
        res = super(integracao, self).cob_boleto_cancelar(cr, uid, codigo_banco_interno, conta_id, titulo_id,
                                                          context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        brwTitulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context=context)
        nossoNumero = False
        for brw_boleto in brwTitulo.fintit_boleto_ids:
            if brw_boleto.state == 'A':
                if brw_boleto.fintib_local_cobranca_id.id == brwTitulo.local_cobranca_id.id:
                    nossoNumero = brw_boleto.fintib_controle_bco
                    break
        res = bInter.inter_cob_boleto_cancelar(nossoNumero)
        if res.get('status'):
            # Trocar dados do inter para dados gerais
            ret = res.get('dados')
            if ret:
                dados = {
                    'situacao': ret.get('situacao'),
                    'nossoNumero': ret.get('nossoNumero'),
                    'documento': ret.get('seuNumero'),
                    'sacado': ret.get('pagador').get('nome'),
                    'vencimento': ret.get('dataVencimento'),
                    'emissao': ret.get('dataEmissao'),
                    'valor': ret.get('valorNominal'),
                    'dataLimite': ret.get('dataLimite'),
                    'linhaDigitavel': ret.get('linhaDigitavel'),
                    'descontoData': ret.get('desconto1').get('data'),
                    'multa': ret.get('multa').get('valor'),
                    'multaData': ret.get('multa').get('data'),
                    'juros': ret.get('mora').get('valor'),
                    'pdf_boleto': ret.get('')
                }
                if ret.get('situacao') == 'PAGO':
                    dados['baixaData'] = ret.get('dataHoraSituacao')[:10]
                    dados['valorPago'] = ret.get('valorNominal')
            else:
                dados = {}
            res['dados'] = dados
        return res

    def status_get(self, cr, uid, codigo_banco_interno, conta_id, operacao, context=None):
        res = super(integracao, self).status_get(cr, uid, codigo_banco_interno, conta_id, operacao, context=context)
        if codigo_banco_interno != 'inter':
            return res
        bInter = Inter(cr, uid, conta_id, context=context)
        if not bInter.status:
            return {'status': False, 'mensagem': bInter.mensagem}
        res = bInter.inter_status_get(operacao)
        if res.get('status'):
            if 'dados' in res.keys():
                _status = 'bancos_financeiro/static/src/img/bolaverde_20x20.png'
            else:
                _status = ''
        else:
            _status = 'bancos_financeiro/static/src/img/bolavermelha_20x20.png'
        if _status:
            _imagem = self.pool.get('banfin.util').bfutl_get_state(_status)
        if not res.get('dados'):
            res['dados'] = {}
        res['dados']['imagem'] = _imagem
        return res

