# -*- encoding: utf-8 -*-
# Calcula valor bruto de peças de madeira serrada pelo método de desdobro do quadrado circunscrito

#Dap - diametro a altura do peito
#ht - altura total da árvore
#hi - comprimento da tora = comprimento da peça
#es - espessura da peça
#min_es - menor espessura por tipo de peça
#di_casc - diâmetro com casca
#di_casc2 - diâmetro com casca para calculo do volume da peça
#di- diâmetro sem casca
#l - largura peça
#difl - sobra peça
#v - volume peça
#vr - volume tora
#vt - volume arvore
#n - num.peças
#narv - numero de árvores da amostra
#codpeca-codigo peça

#a, b, c... - quantidade de peças por tipo por arvore,
#va, vb, vc... - volume de peças por tipo por arvore,
#na, nb, nc... - numero de peças por tipo por arvore

#a1, b1, c1... - quantidade de peças por tipo por amostra (todas as árvores),
#va1, vb1, vc1... - volume de peças por tipo por por amostra,
#na1, nb1, nc1... - numero de peças por tipo por amostra.

import math
from openerp.osv import osv

class calcmadeira_utils(osv.Model):
    _name = 'calcmadeira.utils'
    _auto = False

    def torutl_get_diametro(self, cr, uid, altura, altura_arvore, dap):
        di_casc = math.sqrt(1.04975640127245 - 1.48603214408326 * altura / altura_arvore
                            + 0.473173744770541 * (altura / altura_arvore) ** 2) * dap
        return di_casc

    def torutl_quadrado_circunscrito(self, cr, uid, context=None):

        di=di_casc=di_casc2=0

        a1 = b1 = c1 = d1 = e1 = f1 = g1 = h1 =k1= 0
        va1 = vb1 = vc1 = vd1 = ve1 = vf1 = vg1 = vh1 =vk1= 0
        na1=nb1=nc1=nd1=ne1=nf1=ng1=nh1=nk1=0

        vra1=vrb1 = vrc1 = vrd1 = vre1 = vrf1 = vrg1 = vrh1=vrk1 = 0

        vt = 0
        vt1 = 0
        cont=0

        v = 0
        n = 0

        vr = 0

        a = b = c = d = e = f = g = h = k = 0
        va = vb = vc = vd = ve = vf = vg = vh = vk = 0
        na = nb = nc = nd = ne = nf = ng = nh = nk = 0

        vra = vrb = vrc = vrd = vre = vrf = vrg = vrh = vrk = 0

        obj_tora_arvores = self.pool.get('calcmadeira.arvores')
        obj_tora_produtos = self.pool.bet('calcmadeira.produtos')
        obj_tora_parametros = self.pool.bet('calcmadeira.parametros')
        dados_parametros = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'tora', ['cmapar_altura',
                                                                                              'cmapar_altura_corte'],
                                                                            context=context)
        # hi = 3  # todo: Parametro? - Ok
        hi = dados_parametros.get('torpar_altura')
        di_casc = 0
        di = 100
        # hi_atual = 0.3  # todo: Parametro? - Ok
        hi_atual = dados_parametros.get('torpar_altura_corte')
        arvore_ids = obj_tora_arvores.search(cr, uid, [], context=context)
        l_menor = obj_tora_produtos.torpro_get_menor_largura(cr, uid, context=context)
        for dados_arvore  in obj_tora_arvores.read(cr, uid, arvore_ids, [], context=context):
            dap = dados_arvore.get('torarv_diametro')
            ht = dados_arvore.get('torarv_altura')
            vt = math.exp(-10.1891 + 1.9424 * math.log(dap) + 1.0299 * math.log(ht))

            # Calcula peças por seção do tronco até completar altura da árvore
            while hi_atual + hi <= ht:
                difl = 2

                # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
                # di_casc2 = math.sqrt(1.04975640127245 - 1.48603214408326 * ((hi_atual + hi) / ht)
                #                      + 0.473173744770541 * ((hi_atual + hi) / ht) ** 2) * dap
                di_casc2 = self.torutl_get_diametro((hi_atual + hi), ht, dap)

                # calcula di com casca superior (di_casc) na altura atual (hi_atual)
                if hi_atual + hi < ht:
                    # di_casc = math.sqrt(1.04975640127245 - 1.48603214408326 * hi_atual / ht
                    #                     + 0.473173744770541 * (hi_atual / ht) ** 2) * dap
                    di_casc = self.torutl_get_diametro(hi_atual, ht, dap)
                    vr = (math.pi * ((di_casc + di_casc2) / 2) ** 2 / 40000) * hi
                else:
                    vr = (math.pi * ((di_casc + di_casc2) / 2) ** 2 / 40000) * (ht - hi_atual)

                produto_ids = obj_tora_produtos.search(cr, uid, [], context=context)
                # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
                for dados_produto in obj_tora_produtos.read(cr, uid, produto_ids, [], context=context):
                    # di = di_casc2 * .8 # ver se vai se parametro (comprimento sem casca)
                    di = di_casc2 * dados_parametros.get('torpar_comprimento_sem_casca') # ver se vai se parametro (comprimento sem casca)
                    l = di / math.sqrt(2)
                    # while l >= 2: # ver se vai se parametro(buscar menor largura das peças cadastradas)
                    while l >= l_menor:  # ver se vai se parametro(buscar menor largura das peças cadastradas)
                        # if l >= 16: # (Largura da peça)
                        if l >= dados_produto.get('torpro_largura'):  # (Largura da peça)
                            # min_es = 7 # Espessura da peça
                            min_es = dados_produto.get('torpro_espessura_min') # Espessura da peça
                            difl = l

                            # desdobra da maior espessura da peça para a menor
                            for inv_es in range(0, 4): # Variação da espessura da peça incluir passo para variação
                                # es = 10 - inv_es # o 10 é a espessura máxima da peça
                                es = dados_produto.get('torpro_espessura_max') - inv_es # o 10 é a espessura máxima da peça
                                while es >= difl:
                                    es = es - 1 # Passo(variação)

                                    # 1 cm é a menor espessura possível entre todas as peças (Variável)
                                    if difl < 1 or es < min_es:
                                        break
                                if difl < 1 or es < min_es: # Idem
                                    break
                                n = math.trunc(difl / es)
                                v = (n * es / 100) * difl / 100 * hi
                                difl = difl - n * es
                                nk = nk + n
                                vk = vk + v

                            # para executar apenas uma contagem da seção do tronco.
                            if vr > 0:
                                k = k + 1
                                vrk = vrk + vr
                                vr = 0

    def calutl_remover_acentuacao(self, cr, uid, text, cod='utf-8'):
        """
        Funcao Wrapper para remover acentuacao
        :param text:texto onde sera feita a remocao da acentuacao
        :param cod:Tipo de codificacao
        return:string
        """

        from unicodedata import normalize, category

        try:
            unicode_string = str(text)
        except UnicodeEncodeError:
            return normalize('NFKD', text).encode('ASCII', 'ignore')

        return unicode(filter(lambda c: category(c) != 'Mn',
                              normalize('NFKD', unicode_string.decode(cod))))

