# -*-coding: UTF-8-*-

import pandas as pd

pd.set_option('display.max_columns', 10)
from sqlalchemy.dialects.mssql.information_schema import columns

import FFormaEqFormaCubagem
import Formulas
import SubRotinas
import psycopg2
from sqlalchemy import create_engine
import math
import re

import numpy as np

# ------------------------------------------------------------------
# CalculaPecas

# Dap - diametro a altura do peito
# ht - altura total da árvore
# hc - altura comercial da árvore
# hi - comprimento da tora = comprimento da peça
# hi_atual - começa com altura do toco (30 cm)
# di_casc - diâmetro com casca
# di_casc2 - diâmetro com casca para calculo do volume da peça
# di- diâmetro sem casca
# vr - volume tora ou peça roliça
# vt - volume arvore
# cod_tora - numero de toras
# narv - numero de árvores da amostra
# codpeca-codigo peça


def CalculaPecasRolicas_PtaSerrad(prop_casc,cod_arv, talh, parc, dap, ht, hi_atual,
                                   linpeca2,
                                  escolhaEqForma, B0f, B1f, B2f,
                                  codpeca, peca, es_list, l_div_list, n_div_list, cod_tora, vr, v, preco,
                                  di_min_list, di_max_list, hi_list,
                                  vr_list, di_casc2_list, di_casc_list, di_list, ntora_list,
                                  codpeca_list, peca_list, arv_list, talh_list, parc_list, preco_list,
                                  v_list, conic_tora_list, CRV_list, di_min, hi_min): # Alteração 04/08/2023)

    # -----------------------------------------------------------------

    # Para roliça o limite é dado pelo diametro mínimo, não pela hc
    # Com esta regra inclui-se as roliças não classificadas para serrar (hc = 0)
    # Quando hc é diferente de zero significa que a árvore foi classificada para serrar
    # e que a sobra está sendo passada para roliça (hc !=0), com a variável hi_atual informando a altura de partida
    hc = ht

    di_casc = di_casc2 = 0
    arv = 0

    preco = 0.0000

    # declarando variáveis, não necessário no VBA
    di = 100

    if di_min == 0:  # Alteração 04/08/2023
        hi_min_di_min = SubRotinas.parametros('5')

        # diâmetro mínimo (cm)
        di_min = hi_min_di_min[7]

        # comprimento da tora (m)
        hi_min = hi_min_di_min[3]

    hi = hi_min

    # Calcula peças por seção do tronco até completar altura da árvore
    while hi_atual + hi <= hc:

        if escolhaEqForma == '0':
            # -------------------------------------------------
            # Seleciona equação de forma
            di_casc, di_casc2 = Formulas.EquacaoForma(hi_atual, hi, ht, dap, di_casc, di_casc2)
            # -------------------------------------------------

        if escolhaEqForma == '1':
            di_casc2 = Formulas.Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)
            di_casc = Formulas.Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)

            di_casc2 = di_casc2.di_casc2_result()
            di_casc = di_casc.di_casc_result()

        # ALTERAÇÃO 03/12/2023 Calculo derivada di para verificar inconsistência Taper
        di_casc2_deriv = Formulas.Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap, di_casc, di_casc2, di_casc_deriv,
                                            di_casc2_deriv)
        di_casc_deriv = Formulas.Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap, di_casc, di_casc2, di_casc_deriv,
                                           di_casc2_deriv)

        di_casc2_deriv = di_casc2_deriv.di_casc2_deriv_result()
        di_casc_deriv = di_casc_deriv.di_casc_deriv_result()

        ## ALTERAÇÃO 03/12/2023 Se derivada positiva, diametro começa a aumentar
        # Solução: utilizar a taxa com sinal negativo e deduzir do di_casc2
        if di_casc2_deriv <= 0:
            di = di_casc2 * (1 - prop_casc) * 10
        else:
            di = (di_casc2 - di_casc2_deriv) * (1 - prop_casc) * 10
        ##
        if di < di_min or hi_atual + hi_min > hc:
            break

        cod_tora = cod_tora + 1

        conic_tora = 0

        linpeca = linpeca2
        # ----------------------------------------------------------------------------------
        # SelecionaPeca
        hi, cod_tora, \
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, di_min_list, di_max_list, hi_list, vr_list, \
        di_casc2_list, di_casc_list, di_list, ntora_list, arv_list, talh_list, parc_list, preco_list, \
        v_list, conic_tora_list, CRV_list\
            = SubRotinas.SelecionaPeca(dap, ht, hc, hi_atual, hi_min, di_casc, di_casc2, di,
                                       prop_casc, cod_tora, arv, cod_arv, talh, parc, linpeca,
                                       escolhaEqForma, B0f, B1f, B2f, conic_tora,
                                       codpeca_list, peca_list, es_list, l_div_list, n_div_list, di_min_list, di_max_list, hi_list,
                                       vr_list, di_casc2_list, di_casc_list, di_list, ntora_list,
                                       arv_list, talh_list, parc_list, preco_list,
                                       v_list, conic_tora_list, CRV_list) # Alteração 01/08/2023
        # ----------------------------------------------------------------------------------
        linpeca2 = linpeca

        # atualiza hi_atual com o comprimento da tora calculada (hi)
        hi_atual = hi_atual + hi
        hi = hi_min

        # print('hi_atual=', hi_atual)

    return \
        codpeca, peca, es_list, l_div_list, n_div_list, cod_tora, vr, v, preco, \
        di_min_list, di_max_list, hi_list,  vr_list, di_casc2_list, di_casc_list, di_list, ntora_list, \
        codpeca_list, peca_list, arv_list, talh_list, parc_list, preco_list, v_list, conic_tora_list, \
        CRV_list, di_min, hi_min # Alteração 04/08/2023

