#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import math
from openerp.osv import osv
import numpy as np
from numpy.linalg import inv
from openerp.tools.translate import _



class Eqvt():
    def __init__(self, B0, B1, B2,dap,ht):
        self.B0 = B0
        self.B1 = B1
        self.B2 = B2
        self.dap = dap
        self.ht = ht

    def vt_result(self):
        vt=math.exp(self.B0 +self.B1 * math.log(self.dap) + self.B2 * math.log(self.ht))
        return vt


#  Classe para definir altura comercial a partir do menor diametro (que compensa serrar)
class Eqhi():
    def __init__(self, B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite):
        self.B0 = B0f_hi
        self.B1 = B1f_hi
        self.B2 = B2f_hi
        self.hi_atual=hi_atual
        self.hi=hi
        self.ht = ht
        self.dap = dap
        self.di_limite=di_limite

    def hi_result(self):

        if self.di_limite + 0.5 < self.dap:
            # calcula di com casca(di_casc) na altura atual (hi_atual)
            hi = math.sqrt(
                self.B0+self.B1 * (self.di_limite / self.dap) + self.B2 * (self.di_limite / self.dap) ** 2) * self.ht
        else:
            hi = 0

        return hi
# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------


# Formulas
def equacao_volume(dap, ht, b0=0, b1=0, b2=0):
    # equação volume por árvore ILPF GG100 2011
    vt = math.exp(b0 + b1 * math.log(dap) + b2 * math.log(ht))

    # equação volume por árvore ILPF MARAVILHAS 2009
    # vt = math.exp(-10.5312815270227 + 1.7700934601883 * math.log(dap) + 1.29205471199543 * math.log(ht))
    return vt


class Eqdi_casc():
    def __init__(self, B0, B1, B2, hi_atual, hi, ht, dap):
        self.B0 = B0
        self.B1 = B1
        self.B2 = B2
        self.hi_atual=hi_atual
        self.hi=hi
        self.ht = ht
        self.dap = dap

    def di_casc2_result(self):
        # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
        di_casc2 = math.sqrt(self.B0 +self.B1* ((self.hi_atual + self.hi) / self.ht) + self.B2 * (
                    (self.hi_atual + self.hi) / self.ht) ** 2) * self.dap

        return di_casc2

    def di_casc_result(self):

        # calcula di com casca(di_casc) na altura atual (hi_atual)
        di_casc = math.sqrt(
            self.B0+self.B1 * (self.hi_atual / self.ht) + self.B2 * (self.hi_atual / self.ht) ** 2) * self.dap

        return di_casc

    # ALTERAÇÃO 03/12/2023
    def di_casc2_deriv_result(self):
        di_casc2_deriv = (self.dap * (self.B1 * self.ht + 2 * self.B2 * (self.hi_atual + self.hi)))/ \
                         (2 * self.ht * math.sqrt \
                             (self.B0 * self.ht ** 2 + self.B1 * (self.hi_atual + self.hi) * self.ht + self.B2 * (
                                         self.hi_atual + self.hi) ** 2))
        return di_casc2_deriv

    def di_casc_deriv_result(self):
        di_casc_deriv = (self.dap * (self.B1 * self.ht + 2 * self.B2 * self.hi_atual))/ \
                        (2 * self.ht * math.sqrt\
                            (self.B0 * self.ht ** 2 + self.B1 * self.hi_atual * self.ht + self.B2 * self.hi_atual ** 2))
        return di_casc_deriv



# ----------------------------------------------------------------------------------------------------------------

# Equacao de Taper
def equacao_forma(hi_atual, hi, ht, dap, di_casc, di_casc2, b0=0, b1=0, b2=0):
    # if eqUsuario == 1 and eqForma == 1:

    # 'EQUAÇÃO TAPER (KOZAK)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = Sqr(Range("n2") + Range("n3") * ((hi_atual + hi) / ht) + Range("n4") * ((hi_atual + hi) / ht) ^ 2) * Dap

    # 'calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (Range("n2") + Range("n3") * (hi_atual / ht) + Range("n4") * (hi_atual / ht) ^ 2) * Dap
    # Else

    # EQUAÇÃO TAPER (KOZAK) ILPF GG100 2011
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    try:
        di_casc2 = math.sqrt(
            b0 - (b1 * -1) * ((hi_atual + hi) / ht) + b2 * ((hi_atual + hi) / ht) ** 2) * dap

        # calcula di com casca(di_casc) na altura atual (hi_atual)
        di_casc = math.sqrt(
            b0 - (b1 * -1) * (hi_atual / ht) + b2 * (hi_atual / ht) ** 2) * dap
        return di_casc, di_casc2
    except Exception:
        raise osv.except_osv(_('Atencao'),
                             _('Dados de entrada para calculo incorreto'))

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV1 (CITRIODORA GUARITA 2) Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)

    # if hi != 0:
    #     di_casc2 = (1.04317139756795 - 1.15251507113576 * ((hi_atual + hi) / ht) + 0.745608873884745 * math.pow((
    #     (hi_atual + hi) / ht), 2)) * dap
    # else:
    #     #calcula di com casca(di_casc) na altura atual (hi_atual)
    #     di_casc = (1.04317139756795 - 1.15251507113576 * (hi_atual / ht) + 0.745608873884745 * math.pow((hi_atual / ht), 2)) * dap

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV2 (CITRIODORA GUARITA 2) Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = (1.10041792322808 - 1.46182396963 * ((hi_atual + hi) / ht) + 0.77535958077 * ((hi_atual + hi) / ht) ^ 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (1.10041792322808 - 1.46182396963 * (hi_atual / ht) + 0.77535958077 * (hi_atual / ht) ^ 2) * dap

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV3 (CITRIODORA GUARITA 2) s/toco Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = (1.04934898366772 - 1.01123933349793 * ((hi_atual + hi) / ht) - 0.231797825742175 * ((hi_atual + hi) / ht) ^ 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (1.04934898366772 - 1.01123933349793 * (hi_atual / ht) - 0.231797825742175 * (hi_atual / ht) ^ 2) * dap

    # EQUAÇÃO TAPER (KOZAK) MARAVILHAS s/toco Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # if hi!=0:
    # di_casc2 = math.sqrt(1.03698591927493 - 1.45223087205717 * ((hi_atual + hi) / ht) + 0.428220947928028 * ((hi_atual + hi) / ht) ** 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # else:
    # di_casc = math.sqrt(1.03698591927493 - 1.45223087205717 * (hi_atual / ht) + 0.428220947928028 * (hi_atual / ht) ** 2) * dap

# ----------------------------------------------------------------------------------------------------------------



# volume da peça
def calc_v(es_serra, es, l_d, l_e, l_s, l_i, hi):
    if l_s == 0:
        v = ((es - es_serra) / 100) * ((l_d + l_e) / 2) / 100 * hi
    else:
        v = ((es - es_serra) / 100) * ((l_s + l_i) / 2) / 100 * hi
    return v


# SubRotinas
def testa_espessura(es, es_sobra, es_serra, min_es):
    while es > es_sobra:
        es = es - 5
        if es < min_es + es_serra:
            break
    return es


def testa_largura(l_div, l_sobra, l_min, es_serra):
    while l_div > l_sobra:
        l_div = l_div - 5
        if l_div < l_min + es_serra:
            break
    return l_div


def testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra):
    for inv_l in (0, (l_max - l_min) + 1, 5):
        l_div = l_max + es_serra - inv_l

        while l_div > l_sobra:
            l_div = l_div - 5

            if l_div < l_min:
                break

        return l_div


def RelatorioPecaRolica(a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk,
                        codpeca, vr):
    if codpeca == 1:
        a = a + 1
        vra = vra + vr

    elif codpeca == 2:
        b = b + 1
        vrb = vrb + vr

    elif codpeca == 3:
        c = c + 1
        vrc = vrc + vr

    elif codpeca == 4:
        d = d + 1
        vrd = vrd + vr

    elif codpeca == 5:
        e = e + 1
        vre = vre + vr

    elif codpeca == 6:
        f = f + 1
        vrf = vrf + vr

    elif codpeca == 7:
        g = g + 1
        vrg = vrg + vr

    elif codpeca == 8:
        h = h + 1
        vrh = vrh + vr

    elif codpeca == 9:
        k = k + 1
        vrk = vrk + vr

    return a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk


# ----------------------------------------------------------------------------------------------------------------

def RelatorioPeca(a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                  e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk,
                  codpeca, vr, n, n1, v, v1):
    if codpeca == 1:
        na = na + n + n1
        va = va + v + v1

        if vr > 0:
            a = a + 1
            vra = vra + vr
            # relatório por tora
            print('a:{}, vr:{}'.format(a, vr))
            vr = 0

    elif codpeca == 2:
        nb = nb + n + n1
        vb = vb + v + v1

        if vr > 0:
            b = b + 1
            vrb = vrb + vr
            print('b:{}, vr:{}'.format(b, vr))
            vr = 0

    elif codpeca == 3:
        nc = nc + n + n1
        vc = vc + v + v1

        if vr > 0:
            c = c + 1
            vrc = vrc + vr
            print('c:{}, vr:{}'.format(c, vr))
            vr = 0

    elif codpeca == 4:
        nd = nd + n + n1
        vd = vd + v + v1

        if vr > 0:
            d = d + 1
            vrd = vrd + vr
            print('d:{}, vr:{}'.format(d, vr))
            vr = 0

    elif codpeca == 5:
        ne = ne + n + n1
        ve = ve + v + v1

        if vr > 0:
            e = e + 1
            vre = vre + vr
            print('e:{}, vr:{}'.format(e, vr))
            vr = 0

    elif codpeca == 6:
        nf = nf + n + n1
        vf = vf + v + v1

        if vr > 0:
            f = f + 1
            vrf = vrf + vr
            print('f:{}, vr:{}'.format(f, vr))
            vr = 0

    elif codpeca == 7:
        ng = ng + n + n1
        vg = vg + v + v1

        if vr > 0:
            g = g + 1
            vrg = vrg + vr
            print('g:{}, vr:{}'.format(g, vr))
            vr = 0

    elif codpeca == 8:
        nh = nh + n + n1
        vh = vh + v + v1

        if vr > 0:
            h = h + 1
            vrh = vrh + vr
            print('h:{}, vr:{}'.format(h, vrh))
            vr = 0

    elif codpeca == 9:
        nk = nk + n + n1
        vk = vk + v + v1

        if vr > 0:
            k = k + 1
            vrk = vrk + vr
            print('k:{}, vr:{}'.format(k, vr))
            vr = 0

    return a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd, \
           e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk, \
           codpeca, vr


# ----------------------------------------------------------------------------------------------------------------

def RelatorioFinalRolica(a1, vra1, b1, vrb1, c1, vrc1, d1, vrd1, e1, vre1, f1, vrf1, g1, vrg1, h1,
                         vrh1, k1, vrk1, \
                         a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk):
    # transfere as peças contabilizadas da arvore para a variável final,
    # que armazena peças de todas as árvores
    a1 = a1 + a
    vra1 = vra1 + vra
    b1 = b1 + b
    vrb1 = vrb1 + vrb
    c1 = c1 + c
    vrc1 = vrc1 + vrc
    d1 = d1 + d
    vrd1 = vrd1 + vrd
    e1 = e1 + e
    vre1 = vre1 + vre
    f1 = f1 + f
    vrf1 = vrf1 + vrf
    g1 = g1 + g
    vrg1 = vrg1 + vrg
    h1 = h1 + h
    vrh1 = vrh1 + vrh
    k1 = k1 + k
    vrk1 = vrk1 + vrk

    vrt = vra1 + vrb1 + vrc1 + vrd1 + vre1 + vrf1 + vrg1 + vrh1 + vrk1

    return a1, vra1, b1, vrb1, c1, vrc1, d1, vrd1, e1, vre1, f1, vrf1, g1, vrg1, h1, vrh1, k1, vrk1, vrt


# ----------------------------------------------------------------------------------------------------------------

def RelatorioFinal(a1, vra1, na1, va1, b1, vrb1, nb1, vb1, c1, vrc1, nc1, vc1, d1, vrd1, nd1, vd1,
                   e1, vre1, ne1, ve1, f1, vrf1, nf1, vf1, g1, vrg1, ng1, vg1, h1, vrh1, nh1, vh1,
                   k1, vrk1, nk1, vk1,
                   a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                   e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk):
    # transfere peças calculadas da arvore para a variável final, que armazena peças de todas as árvores da amostra
    a1 = a1 + a
    vra1 = vra1 + vra
    na1 = na1 + na
    va1 = va1 + va

    b1 = b1 + b
    vrb1 = vrb1 + vrb
    nb1 = nb1 + nb
    vb1 = vb1 + vb

    c1 = c1 + c
    vrc1 = vrc1 + vrc
    nc1 = nc1 + nc
    vc1 = vc1 + vc

    d1 = d1 + d
    vrd1 = vrd1 + vrd
    nd1 = nd1 + nd
    vd1 = vd1 + vd

    e1 = e1 + e
    vre1 = vre1 + vre
    ne1 = ne1 + ne
    ve1 = ve1 + ve

    f1 = f1 + f
    vrf1 = vrf1 + vrf
    nf1 = nf1 + nf
    vf1 = vf1 + vf

    g1 = g1 + g
    vrg1 = vrg1 + vrg
    ng1 = ng1 + ng
    vg1 = vg1 + vg

    h1 = h1 + h
    vrh1 = vrh1 + vrh
    nh1 = nh1 + nh
    vh1 = vh1 + vh

    k1 = k1 + k
    vrk1 = vrk1 + vrk
    nk1 = nk1 + nk
    vk1 = vk1 + vk

    vrt = vra1 + vrb1 + vrc1 + vrd1 + vre1 + vrf1 + vrg1 + vrh1 + vrk1

    return a1, vra1, na1, va1, b1, vrb1, nb1, vb1, c1, vrc1, nc1, vc1, d1, vrd1, nd1, vd1, \
           e1, vre1, ne1, ve1, f1, vrf1, nf1, vf1, g1, vrg1, ng1, vg1, h1, vrh1, nh1, vh1, k1, vrk1, nk1, vk1, vrt


# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------

def pecamenor_quadcircun(l_min, l_max, codpeca, hi, es, es_serra, es_sobra, l, l_sobra, n_h,
                         pecas_serradas_preco_dados, es_medula, escolha, cont_medula):
    n1 = 0
    v1 = 0.0000

    l_div = l_min+es_serra

    preco = 0.0000

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco2 = 0.0000
    preco_medula = 0.0000

    dados_impressao = []
    while l_sobra >= l_min + es_serra:

        # min_es não pode ser menor do que es_serra!
        if l_div > l_sobra or l_div < l_min + es_serra:
            break

        for inv_l in range(0, ((l_max - l_min)+1), 5):
            l_div = l_max + es_serra - inv_l

            # ---------------------------
            # Testa Largura
            l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
            # ---------------------------

            # min_es não pode ser menor do que es_serra!
            if l_div > l_sobra or l_div < l_min + es_serra:
                break

            # numero de peças na vertical
            n_div = math.trunc(l_sobra / l_div)

            # numero de peças total por laço
            n_div1 = n_h * n_div

            # numero de peças total
            n1 = n1 + n_div1

            # volume de peças na horizontal e vertical
            v = (n_h * (es - es_serra) / 1000.0) * (n_div * (l_div - es_serra) / 1000.0) * hi
            v1 = v1 + v

            es_tmp = es
            l_tmp = l_div
            n_tmp = n_div1
            v_tmp = v
            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
            preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codpeca,
                                    hi, v_tmp)

            preco_menor = preco_menor + preco_peca

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
            #       format(codpeca, es, l_div, n_div1, v, l_sobra))

            if es_sobra > es + es_medula and es_medula > es_serra \
                    and cont_medula == 0:
                n_div_medula = 1
                v_medula = (es_medula - es_serra) / 1000 * (
                            n_div_medula * (l_div - es_serra) / 1000) * hi

                v1 = v1 + v_medula
                n1 = n1 + n_div_medula

                # Relatório por peça na MEDULA
                # print('peca_medul.:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
                #       format(codpeca, es_medula, l_div, 1, v_medula, l_sobra))

                es_tmp = es_medula
                l_tmp = l_div
                n_tmp = n_div_medula
                v_tmp = v_medula
                # -----------------------------------------------------------------
                # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codpeca,
                                        hi, v_tmp)
                # -----------------------------------------------------------------

                preco_medula = preco_medula + preco_peca

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}, preco_peca:{}, preco:{}' \
            #       .format(codpeca, es, l_div, n_div1, v, l_sobra, preco_peca, preco_medula))

            # largura restante (eixo vertical do bloco)
            l_sobra = l_sobra - (l_div * n_div)

            dados_impressao.append({'l_div': l_div, 'n_div': n_div1, 'v': v, 'l_sobra': l_sobra,
                                    'preco': preco_peca})
            # v=0

            # L sobra
            # print('l_sobra:{}'.format(l_sobra))

    if escolha in ('13', '14'):  # '7' - Gira 180 90
        preco2 = preco_menor + preco_medula
    else:
        preco2 = preco_menor + preco_menor2

    return l_sobra, n1, v1, dados_impressao, preco2


# ---------------------------------------------------------------------------------------------

def pecamenor2_14dcostan(n1, v1, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                         pecas_serradas_preco_dados):

    # ---------------------------
    # Testa Largura
    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
    # ---------------------------

    # está embutido na função TestaLargura...
    if l_div > l_sobra or l_div < l_min + es_serra:
        n_div = 0
        v = 0
        preco_menor2 = 0
        dados_impressao = {'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                           'preco': preco_menor2}
        return l_div, l_sobra, n_div, n1, v1, dados_impressao, preco_menor2
        # return l_div == 0

    # numero de peças na vertical
    n_div = math.trunc(l_sobra / l_div)

    # numero de peças total
    n1 += n_div

    # volume da peça dividida em subpeças
    v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
    v1 += v

    # largura restante (eixo vertical)
    l_sobra = l_sobra - (l_div * n_div)

    es_tmp = es
    l_tmp = l_div
    n_tmp = n_div
    v_tmp = v
    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
    preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codpeca,
                            hi, v_tmp)

    preco_menor2 = preco_peca

    # Relatório por peça
    print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}'. \
          format(codpeca, es, l_div, n_div, v, l_sobra))
    dados_impressao = {'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                       'preco': preco_peca}
    v = 0

    return l_div, l_sobra, n_div, n1, v1, dados_impressao, preco_menor2

# -------------------------------------------------------------------------------------------------------


def pecamenor_long_1costaneirabase(l_min, l_max, codpeca, di, hi, es, es_serra,l_sobra, n_h,
                                   flecha0,flecha,
                                   ang_e, ang_d, cord_e, cord_d,r_flecha_base,
                                   pecas_serradas_preco_dados, parametro_dados,
                                   flecha_div_r, ang_base, escolha):
    n1 = 0
    v1 = 0.0000
    cord_e1 = 0.00
    dados_impressao = []

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco_menor2_14D = 0.0000
    es1 = 0

    # aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h + 1):
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div > l_sobra or l_div < l_min + es_serra:
            es1 = es
            break

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codpeca,
                                hi, v_tmp)

        preco_menor = preco_menor + preco_peca

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}, preco_peca:{}, '
              'preco_menor:{}'.format(codpeca, es, l_div, n_div, v, l_sobra, preco_peca,
                                      preco_menor))
        dados_impressao.append({'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                                'preco': preco_peca})
        v = 0

        if l_sobra >= l_min + es_serra:
            l_div2 = l_div
            n_div2 = n_div

            # --------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1, impressao, preco_menor2 \
                = pecamenor2_14dcostan(n1, v1, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                                       pecas_serradas_preco_dados)
            dados_impressao.append(impressao)
            # --------------------------------------------------------
            l_div = l_div2
            n_div = n_div2

            preco_menor2_14D = preco_menor2_14D + preco_menor2


        if x <= n_h:

            # incremento da flecha com espessura da nova peça
            flecha = flecha + es

            if x == n_h:
                es1 = es
                es = es_serra

            if escolha in ('13', '14'):  # '7' - Gira 180 90
                cord_e1 = cord_e

            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(
                flecha, es, es_serra, di, ang_base, r_flecha_base)
            # -----------------------------------------------

            if flecha <= di / 2  or escolha in ('13', '14'):  # '7' - Gira 180 90
                l_sobra = cord_d
            else:
                l_sobra = cord_e

            # 'limite para corte a direita, para giro de 180 graus da tora
            if escolha == '7' and flecha / (di / 2) >= flecha_div_r:
                break

    preco2 = preco_menor + preco_menor2_14D

    es = es1

    return l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, cord_e, cord_d, \
           r_flecha_base, dados_impressao, preco2, cord_e1


# ---------------------------------------------------------------------------------------------------------

def pecamenor_long_2costaneirabase(l_min, l_max, codpeca, di, hi, es, es_serra,l_sobra, n_h,flecha0,flecha,
                        ang_e, ang_d, cord_e, cord_d,r_flecha_base):
    n1=v1=0
    l_div=0.0000
    n_div=0
    #aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h+1):
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div < l_min + es_serra:
            break

        #numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        #numero de peças total
        n1 += n_div

        #volume da peça dividida em subpeças
        v = ((es - es_serra) / 100) * n_div * (l_div - es_serra) / 100 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}'\
              .format(codpeca, es, l_div, n_div,  v, l_sobra))
        v = 0

        if l_sobra - es_serra >= l_min:
            l_div2 = l_div
            n_div2 = n_div

            #--------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1\
            = pecamenor2_14dcostan(n1,v1,l_min, l_max, codpeca, hi, es, es_serra, l_sobra)
            #--------------------------------------------------------
            l_div = l_div2
            n_div = n_div2

        if flecha+es<=di-flecha0:
            #incremento da flecha com espessura da nova peça
            flecha = flecha + es
        else:
            break

        if x <n_h:
            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = formulas14d(flecha, es, di)
            # -----------------------------------------------
            if flecha <= di / 2:
                l_sobra = cord_d
            else:
                l_sobra = cord_e

    return l_div, n_div, l_sobra ,n1, v1,flecha, ang_e, ang_d, cord_e, cord_d, r_flecha_base

def pecamenor_fatia(fatia, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                    pecas_serradas_preco_dados):
    n1 = 0
    v1 = 0.0000
    l_div = 0.0000
    n_div = 0

    preco1 = 0.0000

    dados_impressao = []
    while l_sobra >= l_min + es_serra:
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codpeca,
                                hi, v_tmp)

        preco1 = preco1 + preco_peca

        # Relatório por peça
        # print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, fatia:{}, l_sobra:{}' \
        #       .format(codpeca, es, l_div, n_div, v, fatia, l_sobra))
        # v = 0
        #
        # # aviso de troca de fatia (selecionou peça)
        # peca = 1
        dados_impressao.append({'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                                'fatia': fatia, 'preco': preco_peca})

    return fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco1


def formulas14d( flecha, es, es_serra, di):

    # 'angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # '1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)

    # 'angulos direito e esquerdo com a nova peça (antes do diametro, usa ang_d, após, usa ang_e
    ang_e = 180 - math.acos((es-es_serra + flecha - di / 2) / (di / 2)) * 180 / math.pi
    ang_d = 180 - math.acos((flecha - di / 2) / (di / 2)) * 180 / math.pi

    # 'Cordas esquerda e direita da peça
    cord_e = di * math.sin(ang_e * math.pi / 180)
    cord_d = di * math.sin(ang_d * math.pi / 180)

    return  ang_e, ang_d, cord_e, cord_d


def Formulas_CostaneiraBase(flecha, es, es_serra, di, ang_base, r_flecha_base):

    # 'angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # '1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)

    # 'cos(180-alfa) = (es + flecha - di / 2) / (di / 2)
    # 'acos((es + flecha - di / 2) / (di / 2))=180-alfa
    # 'alfa = 180 - acos((es + flecha - di / 2) / (di / 2))

    # 'angulos direito e esquerdo com a nova peça (antes do diametro, usa ang_d, após, usa ang_e
    ang_e = 180 - math.acos((es -es_serra + flecha - di / 2) / (di / 2)) * 180 / math.pi
    ang_d = 180 - math.acos((flecha - di / 2) / (di / 2)) * 180 / math.pi

     # 'Cordas esquerda e direita da peça
    if ang_d < (90 - ang_base) or ang_d > (90 + ang_base):
        cord_d = di * math.sin(ang_d * math.pi / 180)
    else:
        cord_d = di * math.sin(ang_d * math.pi / 180) / 2 + r_flecha_base

    if ang_e < (90 - ang_base) or ang_e > (90 + ang_base):
        cord_e = di * math.sin(ang_e * math.pi / 180)
    else:
        cord_e = di * math.sin(ang_e * math.pi / 180) / 2 + r_flecha_base

    return ang_e, ang_d, cord_e, cord_d


def calcmadeira_get_totais(self, cr, uid, ids, field_name, arg, context=None):
    """
    Apura o rendimento no calculo das madeiras.
    Para a peca rolica, o rendimento e calculado com o volume total da tora divido pelo volume
    total multiplicado por 100.
    Para as demais pecas, o rendimento e calculado com o volume total da peca divido pelo volume
    total da tora multiplicado por 100.
    :param cr: cursor no banco de dados.
    :type cr: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type uid: int
    :param ids: lista de id da tabela do objeto wizard_calcmadeira_calcula.
    :type ids: list
    :param field_name: descricao do atributo funcional.
    :type field_name: str
    :param arg: lista de argumentos de pesquisa.
    :type arg: list
    :param context: dados contextuais.
    :type context: dict
    :return: dados da identificacao do registro e seu respectivo valor.
    :rtype: dict
    """
    context = context and context.copy()
    res = {}
    for calc in self.browse(cr, uid, ids, context):
        # res.update({calc.id: {'vr_rendimento': 0}})
        # volume_peca_total = sum(map(lambda i: i.volpeca or 0, calc.peca_ids))
        # volume_tora_total = sum(map(lambda i: i.voltora or 0, calc.peca_ids))
        # volume_peca_rolica_total = sum(map(lambda i: i.volpeca or 0, calc.peca_rolica_ids))
        # if context.get('tipo_funcao') == '1' and calc.vt1:
        #     res[calc.id].update({'vr_rendimento': (volume_tora_total / calc.vt1) * 100.0})
        # if context.get('tipo_funcao') != '1' and (calc.vt1 or calc.total_vt_vr) \
        #         and volume_tora_total:
        #     res[calc.id].update({'vr_rendimento': (volume_peca_total / volume_tora_total) * 100.0})
        # if context.get('serrada_e_rolica') and (volume_tora_total + volume_peca_total):
        #     res[calc.id].update({
        #         'vr_rendimento': ((calc.vt1 + volume_peca_rolica_total) /
        #                           (volume_tora_total + volume_peca_rolica_total)) * 100.0})
        # preco_total = sum(map(lambda i: i.preco_item or 0, calc.peca_ids))
        # preco_peca = volume_peca_total and preco_total / volume_peca_total or 0
        # preco_tora = volume_tora_total and preco_total / volume_tora_total or 0
        # if calc.tipo_funcao == '1':  # Rolica
        #     preco_tora = 0
        # preco_arvore = calc.vt1 and preco_total / calc.vt1 or 0
        # quantidade_tora = 0
        # if context.get('funcoes_serrada_tora'):
        #     preco_arvore = 0
        #     quantidade_tora = sum(map(lambda i: i.numtora or 0, calc.peca_ids))
        # arvore_processada_hectare = 0
        # falha_hectare = 0
        # total_volume_hectare = 0
        # if context.get('tipo_funcao') in ('1', '2') and calc.narv:
        #     arvore_processada_hectare = int(
        #         (calc.arvore_processada or 0) * (calc.arvore_por_hectare or 0) / float(calc.narv))
        #     falha_hectare = int((calc.falha or 0) * (calc.arvore_por_hectare or 0) /
        #                         float(calc.narv))
        #     total_volume_hectare = (calc.vt1 or 0) * (calc.arvore_por_hectare or 0) / \
        #         float(calc.narv)
        # valor_residual = 0
        # if context.get('serrada_e_rolica'):
        #     valor_residual = calc.vt1 and (calc.vt1 - volume_tora_total - volume_peca_rolica_total)\
        #         or 0
        # valor_residual_percentual = 0
        # if context.get('calculo_rolica') or context.get('serrada_e_rolica'):
        #     volume_peca = volume_peca_total if context.get('calculo_rolica') \
        #         else volume_peca_rolica_total
        #     valor_residual_percentual = volume_tora_total \
        #         and ((volume_tora_total - volume_peca) / volume_tora_total) or 0
        # res[calc.id].update({
        #     'preco_total': preco_total, 'preco_peca': preco_peca, 'preco_tora': preco_tora,
        #     'preco_arvore': preco_arvore, 'quantidade_tora': quantidade_tora,
        #     'arvore_processada_hectare': arvore_processada_hectare, 'falha_hectare': falha_hectare,
        #     'valor_residual_percentual': valor_residual_percentual,
        #     'valor_residual': valor_residual, 'total_volume_hectare': total_volume_hectare,
        #     'qtd_talhao': len(calc.talhao_ids),
        # })
        res.update({calc.id: {
            'preco_total': 0, 'preco_peca': 0, 'preco_tora': 0,
            'preco_arvore': 0, 'quantidade_tora': 0,
            'arvore_processada_hectare': 0, 'falha_hectare': 0,
            'valor_residual_percentual': 0,
            'valor_residual': 0, 'total_volume_hectare': 0,
            'qtd_talhao': len(calc.talhao_ids),
        }})
    return res


# def calcmadeira_check_calculo_receita(self, cr, uid, ids, field_name, arg, context=None):
#     context = context and context.copy()
#     permite_calculo_receita = True
#     if any([context.get('calculo_rolica'), context.get('calculo_laminada_arvore'),
#             context.get('calculo_laminada_tora'), context.get('calculo_fator_forma'),
#             context.get('funcoes_serrada_arvore')]):
#         permite_calculo_receita = False
#     if context.get('serrada_e_rolica'):
#         # O Calculo de 'Madeira Serrada + Rolica' sempre realiza o calculo da receita.
#         # No entanto, atribui o valor falso para 'permite_calculo_receita' para que a opcao de
#         # selecao nao seja exibida ao usuario
#         permite_calculo_receita = False
#     res = {}.fromkeys(ids, permite_calculo_receita)
#     return res


def forma(forma_dados):

    print('Ajusta Equação de Forma')

    # --------------------
    lintaper, ntaper = forma_dados, len(forma_dados)
    # --------------------

    # numero da árvore
    dado = 0
    dap = 0

    # a = vetor de '1'
    # 'x1= hi/ht
    # 'x2= (hi/ht)**2
    # 'y= (di_casc/dap)**2

    a = []
    x1 = []
    x2 = []
    y = []

    # Acessa arquivo texto, armazena dap e ht por arvore
    taper = 1

    # Executa por seção
    i = 1
    hi = 0
    hi1 = 1

    while taper < ntaper:

        while hi < hi1:
            dados = lintaper[taper - 1]
            hi = float(dados.get('cmatap_hi'))

            if taper == ntaper:
                pass
            else:
                try:
                    dados1 = lintaper[taper]
                    hi1 = float(dados1.get('cmatap_hi'))
                except:
                    print('fim arquivo')
                    hi1 = 0.3
            try:
                arv = int(dados.get('cmatap_arv'))
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))
                ht = float(dados.get('cmatap_ht'))

                print('hi={}, hi1={}/ ht={}'.format(hi, hi1, ht))

            except:
                print('ainda não leu altura total (ht)')

            if hi == 1.3:
                dap = di_casc

            taper = taper + 1

        for c in range(i - 1, taper - 1, 1):
            dados = lintaper[c]
            try:
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))

            except:
                print('arquivo Taper.txt com campos incompletos')

            try:
                x1.append(hi / ht)
                x2.append((hi / ht) ** 2)
                y.append((di_casc / dap) ** 2)

                print('hi/ht:{}'.format(hi / ht))
            except:
                print('falta algum dado no Taper.txt')

            hi = 0
            hi1 = 1
        i = taper

    dado = ntaper
    dado1 = dado

    a = dado1 * [1]
    print('a=', a)
    print('x1f=', x1)
    print('x2f=', x2)
    print('yf=', y)
    # --------------------------
    B, r2 = regressao(dado, a, x1, x2, y)
    # --------------------------

    B0 = B[0]
    B1 = B[1]
    B2 = B[2]

    print('b0={}, b1={}, b2={}'.format(B0, B1, B2))

    return B0, B1, B2, r2


def volume(volume_dados):
    print('Ajusta Equação de Volume')

    # --------------------
    linarv, narv = volume_dados, len(volume_dados)
    # --------------------

    # numero da árvore
    arv = 0
    n_falha = 0
    dado = 0

    # a = vetor de '1'
    # 'x1=ln(dap)
    # 'x2=ln(ht)
    # 'y=ln(vol)

    a = []
    x1 = []
    x2 = []
    y = []

    # Executa por árvore
    while arv < narv:
        try:
            # Acessa arquivo texto, armazena dap e ht por árvore
            dados = linarv[arv]
            dap = float(dados.get('cmacub_diametro'))
            ht = float(dados.get('cmacub_altura'))
            vol = float(dados.get('cmacub_volume'))
        except:
            print('arquivo Arvore.txt com campos incompletos. Vol?')

        if dap == "" or ht == "":
            n_falha = n_falha + 1
        else:
            x1.append(math.log(dap))
            x2.append(math.log(ht))
            y.append(math.log(vol))

        arv = arv + 1

    dado = narv - n_falha

    a = dado * [1]

    # --------------------------
    B, r2 = regressao(dado, a, x1, x2, y)
    # --------------------------

    B0 = B[0]
    B1 = B[1]
    B2 = B[2]

    print('b0={}, b1={}, b2={}'.format(B0, B1, B2))

    # print('x1:{}. x2:{}'.format(x1[1],x2[1]))

    # 'EQUAÇÃO Volume (Schumacher and Hall)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # volest = math.exp(B0 + B1 * x1[1] + B2 * x2[1])
    # print('Volume: {}'.format(volest))

    return B0, B1, B2, r2


def regressao(dado, a, x1, x2, y):
    Y = np.array(y)
    Xt = np.array([a, x1, x2])
    X = Xt.transpose()

    # dimensoes(matriz):
    tam_Xt = (len(Xt), len(Xt[0]))
    print('{} X {}'.format(tam_Xt[0], tam_Xt[1]))

    tam_X = (len(X), len(X[0]))
    print('{} X {}'.format(tam_X[0], tam_X[1]))

    XtX = Xt.dot(X)
    XtXinv = inv(XtX)
    XtY = Xt.dot(Y)
    B = XtXinv.dot(XtY)
    Bt = B.transpose()
    Yt = Y.transpose()
    YtY = Yt.dot(Y)
    BtXtY = Bt.dot(XtY)

    cte = sum(Y) ** 2 / dado

    SQRegres = BtXtY - cte
    SQTotal = YtY - cte
    SQRes = SQTotal - SQRegres
    print(sum(y))

    if SQTotal != 0:
        r2 = SQRegres / SQTotal
        print('r2: {}'.format(r2))
        # print('B0:{},B1:{}, B2:{}'.format(B[0],B[1],B[2]))

    else:
        r2 = 0
        print('r2: {}'.format(r2))

    # B,r2=Regressao(a,Xt,Y)
    # --------------------------

    return B, r2


def Forma_hi(taper_dados):

    print('Ajusta Equação de Forma com arquivo Taper.txt')

    # --------------------
    lintaper,ntaper=taper_dados, len(taper_dados)
    # --------------------

    # numero da árvore
    dado = 0
    dap=0

    # a = vetor de '1'
    # 'x1= di_casc/dap
    # 'x2= (di_casc/dap)**2
    # 'y= (hi/ht)**2

    a = []
    x1 = []
    x2 = []
    y = []

    # Acessa arquivo texto, armazena dap e ht por arvore
    taper = 1

    # Executa por seção
    i=1
    hi=0
    hi1=1

    while taper <= ntaper:

        while hi<hi1:
            dados = lintaper[taper - 1]
            hi = float(dados.get('cmatap_hi'))

            if taper==ntaper:
                pass
            else:
                dados1 = lintaper[taper]
                hi1 = float(dados1.get('cmatap_hi'))

            try:
                arv=int(dados.get('cmatap_arv'))
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))
                ht=float(dados.get('cmatap_ht'))

            except:
                # print('ainda não leu altura total (ht)')
                pass

            if hi == 1.3:
                dap = di_casc

            taper=taper+1


        for c in range(i,taper,1):
            dados = lintaper[c - 1]
            try:
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))

            except:
                print('arquivo Taper.txt com campos incompletos')

            try:
                x1.append(di_casc/dap)
                x2.append((di_casc/dap)**2)
                y.append((hi/ht)**2)

                # print('hi/ht:{}'.format(di_casc/dap))
            except:
                print('falta algum dado no Taper.txt')

            hi=0
            hi1=1
        i = taper

    dado = ntaper

    a = dado * [1]
    # print('a=',a)
    # print('x1=',x1)
    # print('x2=',x2)
    # print('y=',y)
    # --------------------------
    B, r2 =regressao(dado, a, x1, x2, y)
    # --------------------------

    B0f_hi = B[0]
    B1f_hi = B[1]
    B2f_hi = B[2]

    # print('b0f_hi={}, b1f_hi={}, b2f_hi={}'.format(B0f_hi, B1f_hi, B2f_hi))

    return B0f_hi,B1f_hi,B2f_hi


def calcula_altura_comercial(arvore_dados, diametro_minimo, parametro_dados, taper_dados):
    # ------------------------------------------------------------------
    # Calcula Altura comercial a partir da Ht do arquivo Arvore.txt
    altura_comercial_dict = {}
    if not diametro_minimo:
        return altura_comercial_dict
    di_limite = diametro_minimo
    falha = 0

    # -----------------------
    linarv, narv = arvore_dados, len(arvore_dados)
    # -----------------------

    # escolha = '1'
    # -------------------------------------------
    # es_serra, hi_atual, hi, hi_min, prop_casc, l_min_pecas, es_min_pecas, di_min, es_perim = parametros(
    #     escolha)
    hi_atual = parametro_dados.get('cmapar_altura_corte')
    hi = parametro_dados.get('cmapar_altura')
    flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
    es_medula = parametro_dados.get('es_medula')
    # -------------------------------------------

    parc1 = []
    arvore1 = []
    dap1 = []
    ht1 = []
    hc1 = []

    # --------------------------------------
    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
    # --------------------------------------

    for arv in arvore_dados:

        try:
            parc = float(arv.get('cmaarv_linha'))
            arvore = float(arv.get('cmaarv_arvore'))
            dap = float(arv.get('cmaarv_diametro'))
            ht = float(arv.get('cmaarv_altura'))
            hi2 = 0

            if hi <= ht:
                # --------------------------------------
                hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                hi2 = hi1.hi_result()

                if hi2 < hi:
                    hi2 = 0

        except:
            falha += 1

            parc = float(arv.get('cmaarv_linha'))
            arvore = 0
            dap = 0
            ht = 0
            hi2 = 0

        parc1.append(parc)
        arvore1.append(arvore)
        dap1.append(dap)
        ht1.append(ht)
        hc1.append(hi2)

        altura_comercial_dict.update({arv.get('id'): hi2})

        parc = arvore = dap = ht = hi2 = 0

    return altura_comercial_dict


# Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia


def calc_preco_sig(pecas_serradas_preco_dados, codigopeca, hi, v_tmp):
    preco_peca = 0
    w = 0
    if pecas_serradas_preco_dados:
        preco = pecas_serradas_preco_dados[w]
        while codigopeca != int(preco.get('cpspre_codigo_peca')):
            preco = pecas_serradas_preco_dados[w]
            w = w + 1

        es_ref = float(preco.get('cpspre_espessura'))
        l_ref = float(preco.get('cpspre_largura'))
        preco_ref = float(preco.get('cpspre_preco'))

        v_ref = es_ref / 1000 * l_ref / 1000 * hi
        preco_peca = (v_tmp / v_ref) * preco_ref

    return preco_peca



#==============
# # -*-coding: UTF-8-*-
# import math
#
def EquacaoVolume(dap,ht):

    # equação volume por árvore ILPF GG100 2011
    vt = math.exp(-10.1891 + 1.9424 * math.log(dap) + 1.0299 * math.log(ht))

    #equação volume por árvore ILPF MARAVILHAS 2009
    #vt = math.exp(-10.5312815270227 + 1.7700934601883 * math.log(dap) + 1.29205471199543 * math.log(ht))
    return vt

# # ----------------------------------------------------------------------------------------------------------------
# #Equacao de Taper
#
def EquacaoForma(hi_atual,hi,ht,dap, di_casc,di_casc2):

    # if eqUsuario == 1 and eqForma == 1:

    # 'EQUAÇÃO TAPER (KOZAK)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = Sqr(Range("n2") + Range("n3") * ((hi_atual + hi) / ht) + Range("n4") * ((hi_atual + hi) / ht) ^ 2) * Dap


    # 'calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (Range("n2") + Range("n3") * (hi_atual / ht) + Range("n4") * (hi_atual / ht) ^ 2) * Dap
    # Else



    #EQUAÇÃO TAPER (KOZAK) ILPF GG100 2011
    #calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    di_casc2 = math.sqrt(1.04975640127245 - 1.48603214408326 * ((hi_atual + hi) / ht) + 0.473173744770541 * ((hi_atual + hi) / ht) ** 2) * dap

    #calcula di com casca(di_casc) na altura atual (hi_atual)
    di_casc = math.sqrt(1.04975640127245 - 1.48603214408326 * (hi_atual / ht) + 0.473173744770541 * (hi_atual / ht) ** 2) * dap

    #EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV1 (CITRIODORA GUARITA 2) Dados cubados
    #calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)

    # if hi != 0:
    #     di_casc2 = (1.04317139756795 - 1.15251507113576 * ((hi_atual + hi) / ht) + 0.745608873884745 * math.pow((
    #     (hi_atual + hi) / ht), 2)) * dap
    # else:
    #     #calcula di com casca(di_casc) na altura atual (hi_atual)
    #     di_casc = (1.04317139756795 - 1.15251507113576 * (hi_atual / ht) + 0.745608873884745 * math.pow((hi_atual / ht), 2)) * dap

    #EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV2 (CITRIODORA GUARITA 2) Dados cubados
    #calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    #di_casc2 = (1.10041792322808 - 1.46182396963 * ((hi_atual + hi) / ht) + 0.77535958077 * ((hi_atual + hi) / ht) ^ 2) * dap

    #calcula di com casca(di_casc) na altura atual (hi_atual)
    #di_casc = (1.10041792322808 - 1.46182396963 * (hi_atual / ht) + 0.77535958077 * (hi_atual / ht) ^ 2) * dap

    #EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV3 (CITRIODORA GUARITA 2) s/toco Dados cubados
    #calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    #di_casc2 = (1.04934898366772 - 1.01123933349793 * ((hi_atual + hi) / ht) - 0.231797825742175 * ((hi_atual + hi) / ht) ^ 2) * dap

    #calcula di com casca(di_casc) na altura atual (hi_atual)
    #di_casc = (1.04934898366772 - 1.01123933349793 * (hi_atual / ht) - 0.231797825742175 * (hi_atual / ht) ^ 2) * dap

    #EQUAÇÃO TAPER (KOZAK) MARAVILHAS s/toco Dados cubados
    #calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    #if hi!=0:
    # di_casc2 = math.sqrt(1.03698591927493 - 1.45223087205717 * ((hi_atual + hi) / ht) + 0.428220947928028 * ((hi_atual + hi) / ht) ** 2) * dap

    #calcula di com casca(di_casc) na altura atual (hi_atual)
    #else:
    # di_casc = math.sqrt(1.03698591927493 - 1.45223087205717 * (hi_atual / ht) + 0.428220947928028 * (hi_atual / ht) ** 2) * dap

    return di_casc, di_casc2
# ----------------------------------------------------------------------------------------------------------------
#
#
#
def Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s):
    # Formulas metodo Radial

    # angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # 1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)
    # angulos direito, esquerdo (Fatia0), inferior, superior (Fatia1)
    # cordas direita, esquerda (Fatia0), inferior, superior (Fatia1)
    # larguras esquerda, direita, inferior, superior da peça (atualiza com desconto da espessura da peça)

    if fatia == 0:

        ang_e = math.acos(nes1 / (di / 2)) * 180 / math.pi
        ang_d = math.acos((nes1 + es-es_serra) / (di / 2)) * 180 / math.pi

        cord_e = di * math.sin(ang_e * math.pi / 180)
        cord_d = di * math.sin(ang_d * math.pi / 180)

        l_e = (cord_e / 2) - nes2
        l_d = (cord_d / 2) - nes2

    elif fatia == 1:

        ang_i = math.asin(nes2 / (di / 2)) * 180 / math.pi
        ang_s = math.asin((nes2 + es-es_serra) / (di / 2)) * 180 / math.pi

        cord_i = di * math.cos(ang_i * math.pi / 180)
        cord_s = di * math.cos(ang_s * math.pi / 180)

        l_i = (cord_i / 2) - nes1
        l_s = (cord_s / 2) - nes1

    return ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s

#----------------------------------------------------------------------
# informar angulos para costaneiras
def ang_costan():
    # ang_d = float(input("Digite angulo das costaneiras laterais: \n"))
    # ang_base = float(input("Digite angulo da costaneira base: \n"))
    ang_d = 30
    ang_base = 20
    return ang_d, ang_base


#--------------------------------------------------------------------------
def TestaEspessura(es, es_sobra, es_serra, min_es):
    while es > es_sobra:
        es = es - 5

        if es < min_es+es_serra:
            break

    return es


def TestaLargura(l_div, l_sobra, l_min, es_serra):
    while l_div > l_sobra:
        l_div = l_div - 5

        if l_div < l_min + es_serra:
            break

    return l_div


def TestaLargura14D_Costaneiras(l_sobra, l_max, l_min, es_serra):

    for inv_l in (0, (l_max-l_min) + 1, 5):
        l_div = l_max + es_serra - inv_l

        while l_div > l_sobra:
            l_div = l_div - 5

            if l_div < l_min:
                break

        return l_div


# def __getitem__(self, item):
#     return self.RelatorioPeca[item]

# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------

def SelecionaPeca_QuadCircun (es_serra,hi,vr, l, cod_tora,linpeca, linpeca_preco,
    cord_d, es_medula, escolha,
    di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
    ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list):

    # espessura residual (eixo horizontal do bloco)
    if escolha == '7':
        es_sobra = cord_d
    else:
        es_sobra = l

    cont_medula = 0
    v_medula = 0

    es = es_serra
    selec = 0

    # espessura residual (eixo vertical do bloco)
    l_sobra = l
    l1 = l
    # l2 = 0

    preco1 = 0
    preco2 = 0

    # não precisa aumentar em 1 nivel linpeca por causa do cabecalho
    for p in range(1, (len(linpeca))):

        pecas=linpeca[p]

        codpeca = int(pecas[0])
        peca = str(pecas[1])
        min_es = int(pecas[2])
        max_es = int(pecas[3])
        l_min = int(pecas[4])
        l_max = int(pecas[5])
        prior = int(pecas[6])

        selec = 0
        # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
        while (l_sobra-es_serra) >= l_min and (es_sobra-es_serra)>=min_es:

            # min_es não pode ser menor do que es+es_serra!
            if selec == 1 and es < min_es + es_serra:
                break

            selec = 1

            # desdobra da maior espessura da peça para a menor
            for inv_es in range(0, ((max_es-min_es)+1), 5):
                es = max_es - inv_es

                #sobra de espessura não pode ser menor do que flecha0 + menor espessura das peças (1cm) + es_serra
                if(l_sobra - es_serra) < l_min or(es_sobra - es_serra) < min_es:
                    break

                #Considera espessura da serra (fita, corrente motosserra...)
                es = es + es_serra

                #---------------------------------------------
                #Testa es
                es=TestaEspessura( es, es_sobra, es_serra,min_es)
                #---------------------------------------------

                #min_es não pode ser menor do que es+es_serra!
                if (es_sobra - es_serra) < min_es or es < min_es+es_serra:
                    break

                if es_sobra > es + es_medula and es_medula > es_serra and \
                        cont_medula == 0:
                    n_h = math.trunc((es_sobra - es_medula) / es)
                else:
                    n_h = math.trunc(es_sobra / es)

                #Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
                if l_sobra/(l_min+es_serra)>=2:

                    #--------------------------------------------------------------
                    # entra função peças divididas na vertical
                    l_sobra, n1, v1,linpeca_preco, \
                    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,\
                    preco2, hi_list,\
                    vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list, \
                    =PecaMenorQuadCircun(l_min, l_max, codpeca, peca, hi, es, es_serra, es_sobra, l,
                    l_sobra,n_h, linpeca_preco,
                    es_medula, escolha, cont_medula,
                    vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                    vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
                    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list,
                    preco_list)

                    n=0
                    v = 0.0000
                    #--------------------------------------------------------------

                else:
                    n=n_h
                    n1 = 0
                    v1=0.0000

                    #sobra da largura para tentar outra classificação em PecaMenor
                    if l_sobra - es_serra < l_max:
                        l_div = l_max + es_serra
                        # ---------------------------
                        # Testa Largura
                        l_div = TestaLargura(l_div, l_sobra, l_min, es_serra)
                        # ---------------------------
                        l = l_div
                    else:
                        l = l_max + es_serra

                    l_sobra = l_sobra - l

                    #volume de peças sem divisão vertical
                    v = (n_h * (es-es_serra) / 1000) * (l - es_serra) / 1000 * hi

                    # Relatorio por peça
                    print('codpeca:{},es:{}, v:{}, l_sobra:{}'\
                          .format(codpeca, es,  v, l_sobra))

                    es_tmp = es
                    l_tmp = l
                    n_tmp = n
                    v_tmp = v
                    # -----------------------------------------------------------------
                    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                    preco_peca,\
                    ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list\
                    = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                    di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                    ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
                      codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                    # -----------------------------------------------------------------

                    preco1=preco_peca

                    if es_sobra > es + es_medula and es_medula > es_serra \
                            and cont_medula == 0:
                        v_medula = ((es_medula - es_serra) / 1000) * (l - es_serra) / 1000 * hi
                        n_div_medula=1
                        v = v + v_medula
                        n = n + n_div_medula

                        es_tmp = es_medula
                        l_tmp = l_div
                        n_tmp = n_div_medula
                        v_tmp = v_medula
                        # -----------------------------------------------------------------
                        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                        preco_peca, \
                        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                            = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                                         di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                                         ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list,
                                         talh_list, parc_list, conic_tora_list,
                                         codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                        # -----------------------------------------------------------------

                        preco1 = preco1 + preco_peca


                    if l_sobra >= l_min+es_serra:
                        # --------------------------------------------------------------
                        # entra função peças divididas na vertical
                        l_sobra, n1, v1, linpeca_preco, \
                        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                        preco2, hi_list,\
                        vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list, \
                            = PecaMenorQuadCircun(l_min, l_max, codpeca, peca, hi, es, es_serra, es_sobra, l,
                                  l_sobra, n_h, linpeca_preco,
                                  es_medula, escolha, cont_medula,
                                  vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                                  vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
                                  codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                        n = 0
                        v = 0.0000
                        #--------------------------------------------------------------

                    else:
                        break

            # min_es não pode ser menor do que es+es_serra!
            if (es_sobra - es_serra) < min_es or es < min_es + es_serra:
                break

            preco = preco1 + preco2
            preco1 = preco2 = 0

            #espessura restante: eixo horizontal do bloco
            if es_sobra > es + es_medula and es_medula > es_serra \
                    and cont_medula == 0:
                es_sobra = es_sobra - n_h * es - es_medula
            else:
                es_sobra = es_sobra - n_h * es

        l_sobra=l1

        if selec == 1:
            cont_medula = 1

    return vr, cod_tora, codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
           ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list



#------------------------------------------------------------------------------------------------

def SelecionaPeca_Longitudinal_CostaneiraBase (
        l_min_pecas, es_min_pecas, es_serra, hi, vr,
        flecha0, R_flecha_base, l, cod_tora, linpeca,
        linpeca_preco,
        ang_base, escolha,
        flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
        di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list):

    flecha = flecha0
    es = es_serra

    cord_e1 = 0.0000

    #espessura residual(eixo horizontal)
    if escolha == '7':
        es_sobra = di / 2 * flecha_div_r - flecha0
    else:
        es_sobra = di - 2 * flecha0


    preco1=0
    preco2=0
    preco_menor2_14D = 0

    selec=0

    vp1 = 0.0000

    # '-----------------------------------------------
    ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(flecha, es, es_serra, di, ang_base, R_flecha_base)
    # '-----------------------------------------------

    if flecha <= di / 2 or escolha == '7':
        l_sobra = cord_d
    else:
        l_sobra = cord_e

    while selec == 0 and es_sobra>=0:

        # não precisa aumentar em 1 nivel linpeca por causa do cabecalho
        for p in range(1, (len(linpeca))):
            # 08/09/2023
            # pecas = linpeca[p]
            pecas = linpeca[p]
            codpeca = int(pecas[0])
            peca = str(pecas[1])
            min_es = int(pecas[2])
            max_es = int(pecas[3])
            l_min = int(pecas[4])
            l_max = int(pecas[5])
            prior=int(pecas[6])

            selec = 0
            # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
            while (l_sobra - es_serra) >= l_min and (es_sobra - es_serra) >= min_es:

                # min_es não pode ser menor do que es+es_serra! não precisa no VBA, Exit Do no 1o laço
                if selec == 1 and es < min_es + es_serra:
                    break

                selec = 1

                # desdobra da maior espessura da peça para a menor
                for inv_es in range(0,(max_es-min_es)+1, 5):
                    es = max_es - inv_es

                    #sobra de espessura não pode ser menor do que flecha0 + menor espessura das peças (1cm) + es_serra
                    if(es_sobra - es_serra) < min_es:
                        break

                    # Considera espessura da serra (fita, corrente motosserra...)
                    es = es + es_serra

                    # ---------------------------------------------
                    # Testa es
                    es=TestaEspessura(es, es_sobra, es_serra,min_es)
                    # ---------------------------------------------

                    # min_es não pode ser menor do que es+es_serra!
                    if es < min_es + es_serra:
                        break

                    # '-----------------------------------------------
                    ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(flecha, es, es_serra, di, ang_base, R_flecha_base)
                    # '-----------------------------------------------

                    if flecha <= di / 2 or escolha == '7':
                        l_sobra = cord_d
                    else:
                        l_sobra = cord_e

                    n_h = math.trunc(es_sobra / es)

                    #Se l permite divisao no mínimo em 2 grupos de peças selecionadas (p) em l
                    if l_sobra / (l_min + es_serra) >= 2:

                        # --------------------------------------------------------------
                        # entra função peças divididas na vertical
                        l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, cord_e, cord_d,R_flecha_base, \
                        linpeca_preco,\
                        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list,preco_list, \
                        preco2, cord_e1, hi_list,\
                        vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list, \
                                    = PecaMenor_Long_1CostaneiraBase(l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, n_h,
                        flecha0, flecha, ang_e, ang_d, cord_e, cord_d, R_flecha_base, linpeca_preco,
                        flecha_div_r, ang_base,  escolha,
                         vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                         vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,conic_tora_list,
                        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list,preco_list)
                        n = 0
                        v=0.0000
                        # --------------------------------------------------------------

                    else:
                        n=n1=0
                        v=v1=0.0000

                        # sobra da largura para tentar outra classificação em
                        if l_sobra - es_serra < l_max:
                            l_div = l_max + es_serra
                            # ---------------------------
                            # Testa Largura
                            l_div = TestaLargura(l_div, l_sobra, l_min, es_serra)
                            # ---------------------------
                            l = l_div
                        else:
                            l = l_max + es_serra

                        l_sobra = l_sobra - l

                        # volume das peças
                        for x in range(1, n_h + 1):

                            vp = ((es - es_serra) / 1000) * (l - es_serra)/ 1000 * hi

                            es_tmp = es
                            l_tmp = l
                            n_tmp = 1
                            v_tmp = vp
                            # -----------------------------------------------------------------
                            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                            preco_peca, \
                            ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                                = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                                             di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                                             ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list,
                                             talh_list, parc_list, conic_tora_list,
                                             codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list,
                                             preco_list)
                            # -----------------------------------------------------------------

                            preco1 = preco1 + preco_peca

                            # preco = 0

                            # Relatorio por peça
                            print('codpeca:{}, es:{}, l:{}, n:{}, vp:{}, l_sobra:{}, preco_peca:{}, preco1: {}'\
                                  .format(codpeca, es, l, 1, vp, l_sobra, preco_peca, preco1))

                            n = n+1
                            vp1 = vp1 + vp
                            vp = 0

                            if l_sobra >= l_min + es_serra:

                                # --------------------------------------------------------
                                l_div, l_sobra, n_div, n1, v1, \
                                codpeca, peca, es, v, \
                                codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                                preco_menor2, hi_list,\
                                vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                                    = PecaMenor2_14DCostan(n1, v1, l_min, l_max, codpeca, peca, hi, es, es_serra,
                                   l_sobra,
                                   linpeca_preco,
                                   vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                                   vr_list, ntora_list, di_casc2_list, di_casc_list, di_list,
                                   arv_list, talh_list, parc_list, conic_tora_list,
                                   codpeca_list, peca_list, es_list, l_div_list, n_div_list,
                                   v_list, preco_list,
                                   )

                                v = 0.0000
                                # --------------------------------------------------------
                                preco_menor2_14D = preco_menor2_14D + preco_menor2


                            if x <= n_h:
                                flecha=flecha+es

                                if x==n_h:
                                    es1 = es
                                    es = es_serra

                                if escolha == '7':
                                    cord_e1 = cord_e

                                # '-----------------------------------------------
                                ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(flecha, es, es_serra, di, ang_base, R_flecha_base)
                                # '-----------------------------------------------

                                if flecha <= di / 2 or escolha == '7':
                                    l = math.trunc(cord_d/10)*10
                                else:
                                    l = math.trunc(cord_e/10)*10

                                if l > l_max + es_serra:
                                    l = l_max + es_serra

                                if flecha <= di/2 or escolha == '7':
                                    l_sobra = cord_d - l
                                else:
                                    l_sobra = cord_e - l
                        # next

                        es = es1

                        # except novapeca:
                        #     pass

                    # end if


                    preco = preco1 + preco2 + preco_menor2_14D
                    preco1 = preco2 = preco_menor2_14D = 0

                    # espessura restante (eixo horizontal) sobra da peça para novo aproveitamento (cm)
                    es_sobra = es_sobra - n_h * es
                    print(' es_sobra:{}'.format(es_sobra))

                    v = vp1
                    # # -------------------------------------------------------------
                    # # RelatorioPeca
                    # a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd, \
                    # e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk, \
                    # codpeca, peca, vr, \
                    # precoa, precob, precoc, precod, precoe, precof, precog, precoh, precok \
                    # = RelatorioPeca(a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                    # e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk,
                    # codpeca, peca, vr, n, n1, v, v1,
                    # precoa, precob, precoc, precod, precoe, precof, precog, precoh, precok, preco)
                    # # -------------------------------------------------------------
                    vp1 = 0.0000

        if selec == 0:
            flecha = flecha + 0.5
            es = es_min_pecas

            if flecha + es < di - flecha0:
                # '-----------------------------------------------
                ang_e, ang_d, cord_e, cord_d = \
                    Formulas_CostaneiraBase(flecha, es, es_serra, di, ang_base, R_flecha_base)
                # '-----------------------------------------------
                if flecha <= di / 2 or escolha == '7':
                    l_sobra = cord_d
                else:
                    l_sobra = cord_e

                if escolha == '7':
                    es_sobra = di / 2 * flecha_div_r - flecha
                else:
                    es_sobra = di - flecha - flecha0
            else:
                es_sobra = 0

        if es_sobra == 0:
            break


        if escolha == '7' and\
            (selec == 1 or es_sobra < es_min_pecas or flecha / (di / 2) >= flecha_div_r):
            break

    if escolha == '7':
        if s == 0:
            # comprimento da peça, do eixo x até o limite de corte do topo (R-flecha)
            r_flecha_topo = di / 2 * math.cos(ang_d * math.pi / 180)

            cord_d1 = cord_e1
        else:
            r_flecha_base2 = di / 2 * math.cos(ang_d * math.pi / 180)

            cord_d2 = cord_e1

            l = r_flecha_topo + r_flecha_base2

            if cord_d2 >= cord_d1:
                cord_d = cord_d1
            else:
                cord_d = cord_d2


    return  vr, cod_tora, \
            l, cord_d1, cord_d, r_flecha_topo, s, \
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
            ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list

# ----------------------------------------------------------------------------------------------------------------

def SelecionaPeca_f0(
        l_min_pecas, es_min_pecas, linpeca, linpeca_preco,
        ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
        nes2,
        hi, l_e, l_d, l_i, l_s, l_sobra, cod_tora,
        di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,
        preco, fatia):
    es = 0
    selec = 0

    l_sobra = l_d

    preco = 0

    npecas = len(linpeca) - 1

    for p in range(1, (len(linpeca))):
        pecas = linpeca[p]
        codpeca = int(pecas[0])
        peca = str(pecas[1])
        min_es = int(pecas[2])
        max_es = int(pecas[3])
        l_min = int(pecas[4])
        l_max = int(pecas[5])
        prior = int(pecas[6])

        if ang_d >= 45 and l_d >= l_min + es_serra and \
                (nes1 + min_es) <= float(di / 2 * math.cos(45 * math.pi / 180)):

            # Considera espessura da serra (fita, corrente motosserra...)
            es = max_es + es_serra

            # retorna valores quando l_d não atende largura mínima
            while (nes1 + es-es_serra) >= float(di / 2 * math.cos(45 * math.pi / 180)):

                es = es - 5

            selec = 1

            while l_d < l_min + es_serra:
                es = es - 5

                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                = Formulas(fatia, nes1, nes2, es, es_serra,di,ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
                # ---------------------------------------------

                if es < min_es+es_serra:
                    es = 0
                    selec = 0
                    break

            if es < min_es + es_serra:
                continue

            # ---------------------------------------------
            ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
            # ---------------------------------------------
            if l_d < l_min + es_serra:
                selec = 0

            if selec == 0:
                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
                # ---------------------------------------------

                while l_d < l_min + es_serra:
                    es = es - 5
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    if es < min_es + es_serra:
                        es = 0
                        selec = 0
                        break

                if es < min_es + es_serra:
                    continue
            # if es < min_es + es_serra:
            #     continue

            # Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
            if l_d / (l_min + es_serra) >= 2:
                l_sobra = l_d
                # --------------------------------------------------------------
                # entra função peças divididas na vertical
                fatia, l_div, n_div, l_sobra, n1, v1, linpeca_preco, \
                codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                preco1, hi_list,\
                vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                    = PecaMenorFatia(fatia, l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, linpeca_preco,
                                     vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                                     vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                                     conic_tora_list,
                                     codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                n = 0
                v = 0.0000
                # --------------------------------------------------------------
                preco=preco+preco1

            else:
                n1=0
                v1=0.0000
                l_sobra=l_d

                # ---------------------------
                # Testa Largura
                l_div = TestaLargura14D_Costaneiras(l_sobra, l_max, l_min, es_serra)
                # ---------------------------

                # sobra da largura para tentar outra classificação em pecamenor
                l_sobra = l_sobra - l_div

                # ---------------------------------------
                # volume da peça
                #l_s = l_i = 0
                #v = calc_v(es_serra, es, l_d, l_e, l_s, l_i, hi)
                v=((es-es_serra)/1000)*(l_div-es_serra)/1000*hi
                n = 1
                # ---------------------------------------

                es_tmp = es
                l_tmp = l_div
                n_tmp = n
                v_tmp = v
                # -----------------------------------------------------------------
                # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                preco_peca, \
                ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                    = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                                 di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                                 ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                                 conic_tora_list,
                                 codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                # -----------------------------------------------------------------

                preco = preco_peca

                print('codpeca:{}, es:{}, l_div:{}, n:{}, v:{}, fatia:{}, l_sobra:{}, tora:{}, arv:{}, preco_peca:{},preco:{}'\
                      .format(codpeca, es, l_div, n, v, fatia,l_sobra,cod_tora,cod_arv, preco_peca, preco))

                if l_sobra >= l_min + es_serra:
                    # --------------------------------------------------------------
                    # entra função peças divididas na vertical
                    fatia, l_div, n_div, l_sobra, n1, v1, linpeca_preco, \
                    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                    preco1, hi_list,\
                    vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                        = PecaMenorFatia(fatia, l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, linpeca_preco,
                                         vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                                         vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                                         conic_tora_list,
                                         codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                    n = 0
                    v = 0.0000
                    # --------------------------------------------------------------
                    preco=preco+preco1
                    preco1 = 0

             # '------------------------------------------------------
            # RelatorioPeca
            # a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd, \
            # e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk, \
            # codpeca, peca, vr, \
            # precoa, precob, precoc, precod, precoe, precof, precog, precoh, precok \
            # = RelatorioPeca(a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
            # e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk,
            # codpeca, peca, vr, n, n1, v, v1,
            # precoa, precob, precoc, precod, precoe, precof, precog, precoh, precok, preco)
            # '------------------------------------------------------
            selec = 1

            # zera variáveis da seção da árvore
            n=0
            n1=0
            v=0.0000
            v1=0

            preco = 0

        if l_sobra < l_min_pecas + es_serra:
            break

        # Aproveitamento do l_sobra com outro tipo de peça no mesmo es
        elif p < npecas \
            and l_sobra >= l_min_pecas + es_serra \
            and es >= es_min_pecas + es_serra:
            continue

    # define espessura acumulada na Fatia0(das peças selecionadas)
    if es > 0 and selec == 1:
        nes1 = nes1 + es
        es1 = es
        l_s = l_s - es1
    else: l_s = 0

    es_sobra = l_s
    # print(f'es_sobra:{es_sobra}')


    return  linpeca, fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, cod_tora, \
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,\
            ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list


# ---------------------------------------------------------------------
def SelecionaPeca_f1(
        l_min_pecas, es_min_pecas, linpeca, linpeca_preco,
        ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
        nes2,
        hi, l_e, l_d, l_i, l_s, l_sobra, cod_tora,
        di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,
        preco, fatia):
    es = 0
    selec = 0

    l_sobra = l_s

    preco = 0

    npecas = len(linpeca) - 1

    for p in range(1, (len(linpeca))):
        pecas = linpeca[p]
        codpeca = int(pecas[0])
        peca = str(pecas[1])
        min_es = int(pecas[2])
        max_es = int(pecas[3])
        l_min = int(pecas[4])
        l_max = int(pecas[5])
        prior = int(pecas[6])

        if ang_s <= 45 and l_s >= l_min + es_serra and \
                (nes2 + min_es) <= float(di / 2 * math.cos(45 * math.pi / 180)):

            # Considera espessura da serra (fita, corrente motosserra...)
            es = max_es + es_serra

            # retorna valores quando l_d não atende largura mínima
            while (nes2 + es-es_serra) >= float(di / 2 * math.cos(45 * math.pi / 180)):

                es = es - 5

            selec = 1

            while l_s < l_min + es_serra:
                es = es - 5

                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                = Formulas(fatia, nes1, nes2, es, es_serra, di,ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
                # ---------------------------------------------

                if es < min_es+es_serra:
                    es = 0
                    selec = 0
                    break

            if es < min_es+es_serra:
                continue

            # ---------------------------------------------
            ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
            # ---------------------------------------------
            if l_s < l_min + es_serra:
                selec = 0

            if selec == 0:
                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
                # ---------------------------------------------

                while l_s < l_min + es_serra:
                    es = es - 5
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    if es < min_es+es_serra:
                        es = 0
                        selec = 0
                        break

                if es < min_es+es_serra:
                    continue
            # if es < min_es + es_serra:
            #     continue

            # Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
            if l_s / (l_min + es_serra) >= 2:
                l_sobra = l_s
                # --------------------------------------------------------------
                # entra função peças divididas na vertical
                fatia, l_div, n_div, l_sobra, n1, v1, linpeca_preco, \
                codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                preco1, hi_list,\
                vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                    = PecaMenorFatia(fatia, l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, linpeca_preco,
                                     vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                                     vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                                     conic_tora_list,
                                     codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                n = 0
                v = 0.0000
                # --------------------------------------------------------------
                preco=preco+preco1

            else:
                n1=0
                v1=0.0000
                l_sobra = l_s

                # ---------------------------
                # Testa Largura
                l_div = TestaLargura14D_Costaneiras(l_sobra, l_max, l_min, es_serra)
                # ---------------------------

                # sobra da largura para tentar outra classificação em pecamenor
                l_sobra = l_sobra - l_div
                # ---------------------------------------
                # volume da peça
                #l_d = l_e = 0
                #v = calc_v(es_serra, es, l_d, l_e, l_s, l_i, hi)
                v = ((es - es_serra) / 1000) * (l_div-es_serra)/ 1000 * hi
                n = 1
                # ----------------------------------------

                es_tmp = es
                l_tmp = l_div
                n_tmp = n
                v_tmp = v
                # -----------------------------------------------------------------
                # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                preco_peca, \
                ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                    = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                                 di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                                 ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                                 conic_tora_list,
                                 codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                # -----------------------------------------------------------------

                preco = preco_peca

                print('codpeca:{}, es:{}, l_div:{}, n:{}, v:{}, fatia:{}, l_sobra:{}, preco_peca:{},preco:{}'\
                      .format(codpeca, es, l_div, n, v, fatia,l_sobra, preco_peca, preco))

                if l_sobra >= l_min + es_serra:
                    # --------------------------------------------------------------
                    # entra função peças divididas na vertical
                    fatia, l_div, n_div, l_sobra, n1, v1, linpeca_preco, \
                    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                    preco1, hi_list,\
                    vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list\
                    = PecaMenorFatia(fatia, l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, linpeca_preco,
                             vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                             vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
                             codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                    n = 0
                    v = 0.0000
                    # --------------------------------------------------------------
                    preco = preco + preco1
                    preco1 = 0

            # '------------------------------------------------------
            # RelatorioPeca
            # a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd, \
            # e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk, \
            # codpeca, peca, vr, \
            # precoa, precob, precoc, precod, precoe, precof, precog, precoh, precok \
            # = RelatorioPeca(a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
            # e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk,
            # codpeca, peca, vr, n, n1, v, v1,
            # precoa, precob, precoc, precod, precoe, precof, precog, precoh, precok, preco)
            # '------------------------------------------------------
            selec = 1

            # zera variáveis da seção da árvore
            n=0
            n1=0
            v=0.0000
            v1=0.0000

            preco = 0

        if l_sobra < l_min_pecas+es_serra:
            break

        # Aproveitamento do l_sobra com outro tipo de peça no mesmo es
        elif p < npecas \
            and l_sobra >= l_min_pecas + es_serra \
            and es >= es_min_pecas + es_serra:
            continue

    # define espessura acumulada na Fatia1(das peças selecionadas)
    if es > 0 and selec == 1:
        nes2 = nes2 + es
        es2 = es
        l_d = l_d - es2
    else: l_d = 0

    es_sobra = l_d
    # print(f'es_sobra:{es_sobra}')


    return  linpeca, fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, cod_tora,\
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,\
            ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list

# ----------------------------------------------------------------------------------------------------------------

def PecaMenorQuadCircun(l_min, l_max, codpeca, peca, hi, es, es_serra, es_sobra, l,
                        l_sobra,n_h,linpeca_preco,
                        es_medula, escolha, cont_medula,
                        vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                        vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
                        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list):
    n1=0
    v1=0.0000

    l_div = l_min+es_serra

    preco = 0.0000
    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco2 = 0.0000
    preco_medula = 0.0000

    while l_sobra>=l_min+es_serra:

        # min_es não pode ser menor do que es_serra!
        if l_div > l_sobra or l_div < l_min + es_serra:
            break

        for inv_l in range(0, ((l_max - l_min)+1), 5):
            l_div = l_max +es_serra - inv_l

            #---------------------------
            #Testa Largura
            l_div=TestaLargura(l_div,l_sobra,l_min, es_serra)
            #---------------------------

            # min_es não pode ser menor do que es_serra!
            if l_div > l_sobra or l_div < l_min + es_serra:
                break

            #numero de peças na vertical
            n_div = math.trunc(l_sobra / l_div)

            #numero de peças total por laço
            n_div1 = n_h * n_div

            #numero de peças total
            n1 = n1 + n_div1

            #volume de peças na horizontal e vertical
            v = (n_h * (es - es_serra) / 1000) * (n_div * (l_div - es_serra) / 1000) * hi
            v1 = v1 + v

            es_tmp = es
            l_tmp = l_div
            n_tmp = n_div1
            v_tmp = v
            # -----------------------------------------------------------------
            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
            preco_peca, \
            ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                             di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                             ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                             conic_tora_list,
                             codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
            # -----------------------------------------------------------------

            preco_menor = preco_menor + preco_peca

            # Relatorio por peça
            print('codpeca:{}, es:{}, l:{}, n:{}, v:{}, l_sobra:{}, preco_peca:{}, preco_menor:{}' \
                  .format(codpeca, es, l_div, n1, v, l_sobra, preco_peca, preco_menor))

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
            #       format(codpeca, es, l_div, n_div1, v, l_sobra))

            if es_sobra > es + es_medula and es_medula > es_serra \
                    and cont_medula == 0:
                n_div_medula = 1
                v_medula = (es_medula - es_serra) / 1000 * (n_div_medula * (l_div - es_serra) / 1000) * hi

                v1 = v1 + v_medula
                n1 = n1 + n_div_medula

                # Relatório por peça na MEDULA
                print('peca_medul.:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
                      format(codpeca, es_medula, l_div, 1, v_medula, l_sobra))

                es_tmp = es_medula
                l_tmp = l_div
                n_tmp = n_div_medula
                v_tmp = v_medula
                # -----------------------------------------------------------------
                # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
                preco_peca, \
                ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
                    = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                                 di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                                 ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                                 conic_tora_list,
                                 codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
                # -----------------------------------------------------------------

                preco_medula = preco_medula + preco_peca

                # Relatório por peça
                print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}, preco_peca:{}, preco:{}' \
                      .format(codpeca, es, l_div, n_div1, v, l_sobra, preco_peca, preco_medula))

            # largura restante (eixo vertical do bloco)
            l_sobra = l_sobra - (l_div * n_div)

            # L sobra
            print('l_sobra:{}'.format(l_sobra))

    if escolha == '7':
        preco2 = preco_menor + preco_medula
    else:
        preco2 = preco_menor + preco_menor2

    return l_sobra, n1, v1, linpeca_preco,\
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,\
             preco2, hi_list,\
           vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list

# ---------------------------------------------------------------------------------------------

def PecaMenor2_14DCostan(n1, v1, l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra,
            linpeca_preco,
            vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
            vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list):

    #---------------------------
    #Testa Largura
    l_div=TestaLargura14D_Costaneiras(l_sobra, l_max, l_min, es_serra)
    #---------------------------

    #está embutido na função TestaLargura...
    if l_div > l_sobra or l_div < l_min + es_serra:
        n_div = 0
        v = 0
        preco_menor2 = 0

        return l_div, l_sobra, n_div, n1, v1, \
               codpeca, peca, es, v, \
               codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
               preco_menor2
        # return l_div == 0

    #numero de peças na vertical
    n_div = math.trunc(l_sobra / l_div)

    #numero de peças total
    n1 +=  n_div

    #volume da peça dividida em subpeças
    v = ((es - es_serra) / 1000) * n_div * (l_div - es_serra) / 1000 * hi
    v1 +=  v

    # largura restante (eixo vertical)
    l_sobra = l_sobra - (l_div * n_div)

    es_tmp = es
    l_tmp = l_div
    n_tmp = n_div
    v_tmp = v
    # -----------------------------------------------------------------
    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
    codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
    preco_peca, \
    ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
        = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                     di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                     ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                     conic_tora_list,
                     codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
    # -----------------------------------------------------------------

    preco_menor2 = preco_peca

    # Relatório por peça
    print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}, preco_peca:{}, preco_menor2:{}'.\
          format(codpeca, es, l_div, n_div, v, l_sobra, preco_peca, preco_menor2))

    return l_div, l_sobra, n_div, n1, v1, \
            codpeca, peca, es, v, \
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
            preco_menor2, hi_list,\
            vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list

# ------------------------------------------------------------------------------------------------------
def PecaMenor_Long_1CostaneiraBase(l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, n_h,
                        flecha0, flecha, ang_e, ang_d, cord_e, cord_d, R_flecha_base, linpeca_preco,
                        flecha_div_r, ang_base,  escolha,
                         vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                         vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
                        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list,preco_list):
    n1=0
    v1=0.0000
    l_div = 0.0000
    n_div = 0
    cord_e1 = 0.00
    es1 = 0

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco_menor2_14D = 0.0000

    #aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h+1):
        # ---------------------------
        # Testa Largura
        l_div = TestaLargura14D_Costaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div > l_sobra or l_div < l_min + es_serra:
            es1 = es
            break

        #numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        #numero de peças total
        n1 += n_div

        #volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000) * n_div * (l_div - es_serra) / 1000 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # -----------------------------------------------------------------
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
        preco_peca, \
        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
            = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                         di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                         ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                         conic_tora_list,
                         codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
        # -----------------------------------------------------------------

        preco_menor = preco_menor + preco_peca

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}, preco_peca:{}, preco_menor:{}'\
              .format(codpeca, es, l_div, n_div,  v, l_sobra, preco_peca, preco_menor))
        v = 0

        if l_sobra >= l_min + es_serra:
            l_div2 = l_div
            n_div2 = n_div
            #--------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1, \
            codpeca, peca, es, v, \
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
            preco_menor2, hi_list,\
            vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list\
            = PecaMenor2_14DCostan(n1, v1, l_min, l_max, codpeca,peca, hi, es, es_serra, l_sobra,
            linpeca_preco,
            vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
            vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list
            )
            #--------------------------------------------------------
            l_div = l_div2
            n_div = n_div2


            preco_menor2_14D = preco_menor2_14D + preco_menor2


        if x <= n_h:

            # incremento da flecha com espessura da nova peça
            flecha = flecha + es

            if x == n_h:
                es1 = es
                es = es_serra

            if escolha == '7':
                cord_e1 = cord_e
            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(flecha, es, es_serra,di, ang_base, R_flecha_base)
            # -----------------------------------------------

            if flecha <= di / 2 or escolha == '7':
                l_sobra = cord_d
            else:
                l_sobra = cord_e

            # 'limite para corte a direita, para giro de 180 graus da tora
            if escolha =='7' and flecha / (di / 2) >= flecha_div_r:
                break

    preco2 = preco_menor + preco_menor2_14D

    es = es1

    return l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, cord_e, cord_d,R_flecha_base, \
            linpeca_preco,\
            codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
            preco2, cord_e1, hi_list,\
            vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list

# ---------------------------------------------------------------------------------------------------------

def PecaMenorFatia(fatia, l_min, l_max, codpeca, peca, hi, es, es_serra, l_sobra, linpeca_preco,
                   vr, cod_tora, di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora, hi_list,
                   vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                   conic_tora_list,
                   codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list):

    n1 = 0
    v1 = 0.0000
    l_div=0.0000
    n_div=0

    preco1 = 0.0000

    while l_sobra >= l_min + es_serra:
        # ---------------------------
        # Testa Largura
        l_div = TestaLargura14D_Costaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div > l_sobra or l_div < l_min + es_serra:
            break

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000) * n_div * (l_div - es_serra) / 1000 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # -----------------------------------------------------------------
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
        preco_peca, \
        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list \
            = calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
                         di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
                         ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list,
                         conic_tora_list,
                         codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list)
        # -----------------------------------------------------------------

        preco1=preco1+preco_peca

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, fatia:{}, l_sobra:{}, preco_peca:{}, preco1:{}'\
              .format(codpeca, es, l_div, n_div, v, fatia, l_sobra, preco_peca, preco1))

    return fatia, l_div, n_div,  l_sobra, n1, v1, linpeca_preco,\
           codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list, \
           preco1, hi_list,\
           vr_list, ntora_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list


# ---------------------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------------------
def SelecionaPeca(dap, ht, hc,hi_atual, hi_min, di_casc, di_casc2, di,
                  prop_casc, cod_tora, arv, cod_arv, talh, parc, linpeca,
                    escolhaEqForma, B0f, B1f, B2f, conic_tora,
                    codpeca_list, peca_list, es_list, l_div_list, n_div_list, di_min_list, di_max_list, hi_list,
                    vr_list, di_casc2_list, di_casc_list, di_list, ntora_list,
                    arv_list, talh_list, parc_list, preco_list,
                    v_list, conic_tora_list, CRV_list): # Alteração 01/08/2023

    npecas = len(linpeca) - 1

    vr = 0.0000
    v = 0.0000

    # não precisa aumentar em 1 nivel linpeca por causa do cabecalho
    for p in range(1, (len(linpeca))):
        # print('numero peças :{}, p:{}'.format(len(linpeca), p))

        pecas = (linpeca[p])
        codpeca = int(pecas[0])
        peca = pecas[1]
        di_min = int(pecas[2])
        di_max = int(pecas[3])
        hi = float(pecas[4])

        try:
            preco=float(pecas[6])
        except:
            pass

        # Retirei restrição di < di_max. Estava exato, mas pulando peça.
        # Agora a peça, em último caso, enquadra com di > di_min e di_max.
        # Pode acontecer de ser uma peça bem mais grossa para seu comprimento, mas é melhor do que descartar a madeira.

        if di_min <= di and hi_atual+hi <= hc:
            # 08/09/2023
            escolhaEqForma = '1'
            #
            if escolhaEqForma == '0':
                # -------------------------------------------------
                # Seleciona equação de forma
                di_casc, di_casc2 = EquacaoForma(hi_atual, hi, ht, dap, di_casc, di_casc2)
                # -------------------------------------------------

            if escolhaEqForma == '1':
                di_casc2 = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)
                di_casc = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)

                di_casc2 = di_casc2.di_casc2_result()
                di_casc = di_casc.di_casc_result()

            # Conicidade da tora (cm/m)
            conic_tora = (di_casc - di_casc2) / hi

            di = di_casc2*(1-prop_casc)

            # Retirei restrição di < di_max. Estava exato, mas pulando peça.
            # Agora a peça, em último caso, enquadra com di > di_min e di_max.
            # Pode acontecer de ser uma peça bem mais grossa para seu comprimento, mas é melhor do que descartar a madeira.

            if di_min <= di and hi_atual + hi <= hc:

                # volume da tora usando formula de Smalian
                vr = math.pi / 40000 * (di_casc ** 2 + di_casc2 ** 2) / 2 * hi

                # Alteração 17/08/2023
                # if peca != 'NaoRolica':
                #     # volume da peça roliça selecionada (valor fica muito grande por causa do 1o di_max da tabela DimRolica.txt
                #     v = math.pi / 40000 * (di_max ** 2 + di_min ** 2) / 2 * hi
                # else:
                #     v = vr
                v = vr

                codpeca_list.append(codpeca)
                peca_list.append(peca)
                es_list.append(0)
                l_div_list.append(0)
                n_div_list.append(1)
                v_list.append(v)
                ntora_list.append(cod_tora)
                di_min_list.append(di_min)
                di_max_list.append(di_max)
                hi_list.append(hi)
                vr_list.append(vr)
                di_casc2_list.append(di_casc2)
                di_casc_list.append(di_casc)
                di_list.append(di)
                arv_list.append(cod_arv)
                talh_list.append(talh)
                parc_list.append(parc)
                preco_list.append(preco)
                conic_tora_list.append(conic_tora)

                # Alteração 01/08/2023
                # CRV_list.append(v/vr)

                # Alteração 17/08/2023
                CRV_list.append(0)

                # print('codpeca:{},peca:{},\
                # vr:{}, di_casc2:{}, di_casc:{}, di:{}, tora:{},arv:{},  preco:{}, conic_tora:{}, parc: {}'.\
                # format(codpeca,peca, vr, di_casc2, di_casc, di, cod_tora, cod_arv,  preco, conic_tora, parc))

            else:
                if p+1 < len(linpeca):
                    p=p+1
                    # print('numero peças :{}, p:{}'.format(len(linpeca), p))
                else: break

                # 08/09/2023
                pecas = (linpeca[p])
                #
                codpeca = int(pecas[0])
                peca = str(pecas[1])
                di_min = int(pecas[2])
                di_max = int(pecas[3])
                hi = float(pecas[4])
                try:
                    preco = float(pecas[6])
                except:
                    pass


                # Seleciona equação de forma
                if escolhaEqForma == '0':
                    # -------------------------------------------------
                    # Seleciona equação de forma
                    di_casc, di_casc2 = EquacaoForma(hi_atual, hi, ht, dap, di_casc, di_casc2)
                    # -------------------------------------------------

                if escolhaEqForma == '1':
                    di_casc2 = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)
                    di_casc = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)

                    di_casc2 = di_casc2.di_casc2_result()
                    di_casc = di_casc.di_casc_result()
                # -------------------------------------------------

                # Conicidade da tora (cm/m)
                conic_tora = (di_casc - di_casc2) / hi

                # ALTERAÇÃO 04/07/22
                di = di_casc2 * (1 - prop_casc)

                if di_min <= di and hi_atual + hi <= hc:

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    if peca != 'NaoRolica':
                        # volume da peça roliça selecionada
                        v = math.pi / 40000 * (di_max ** 2 + di_min ** 2) / 2 * hi
                    else:
                        v = vr

                    codpeca_list.append(codpeca)
                    peca_list.append(peca)
                    es_list.append(0)
                    l_div_list.append(0)
                    n_div_list.append(1)
                    v_list.append(v)
                    ntora_list.append(cod_tora)
                    di_min_list.append(di_min)
                    di_max_list.append(di_max)
                    hi_list.append(hi)
                    vr_list.append(vr)
                    di_casc2_list.append(di_casc2)
                    di_casc_list.append(di_casc)
                    di_list.append(di)
                    arv_list.append(cod_arv)
                    talh_list.append(talh)
                    parc_list.append(parc)
                    preco_list.append(preco)
                    conic_tora_list.append(conic_tora)

                    # Alteração 01/08/2023
                    # CRV_list.append(v / vr)

                    # Alteração 17/08/2023
                    CRV_list.append(0)

                    # print('codpeca:{},peca:{},\
                    # vr:{}, di_casc2:{}, di_casc:{}, di:{}, tora:{},arv:{},  preco:{}, conic_tora:{}, parc: {}'. \
                    # format(codpeca, peca, vr, di_casc2, di_casc, di, cod_tora, cod_arv, preco, conic_tora, parc))

            if vr != 0:
                break

    # Alteração 01/08/2023
    return hi, cod_tora, \
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, di_min_list, di_max_list, hi_list, vr_list, \
        di_casc2_list, di_casc_list, di_list, ntora_list, arv_list, talh_list, parc_list, preco_list, \
        v_list, conic_tora_list, CRV_list\
# ---------------------------------------------------------------------------------------
# def Volume():
#
#     print('Ajusta Equação de Volume com arquivo Cubagem.txt')
#
#     # --------------------
#     linarv,narv=learq6()
#     # --------------------
#
#     # numero da árvore
#     arv = 0
#     n_falha = 0
#     dado = 0
#
#     # a = vetor de '1'
#     # 'x1=ln(dap)
#     # 'x2=ln(ht)
#     # 'y=ln(vol)
#
#     a = []
#     x1 = []
#     x2 = []
#     y = []
#
#     # Executa por árvore
#     while arv < narv:
#         arv = arv + 1
#
#         try:
#             # Acessa arquivo texto, armazena dap e ht por árvore
#             dados = linarv[arv].split()
#             dap = float(dados[2])
#             ht = float(dados[3])
#             vol = float(dados[5])
#         except:
#             print('arquivo Cubagem.txt com campos incompletos. Vol?')
#
#         if dap == "" or ht == "":
#             n_falha = n_falha + 1
#         else:
#             x1.append(math.log(dap))
#             x2.append(math.log(ht))
#             y.append(math.log(vol))
#
#     dado = narv - n_falha
#
#     a = dado * [1]
#     # print('a=',a)
#     # print('x1v=',x1)
#     # print('x2v=',x2)
#     # print('y=',y)
#
#     # --------------------------
#     B, r2 =regressao(dado, a, x1, x2, y)
#     # --------------------------
#
#     B0v = B[0]
#     B1v = B[1]
#     B2v = B[2]
#
#     # print('b0v={}, b1v={}, b2v={}'.format(B0v, B1v, B2v))
#
#     #print('x1:{}. x2:{}'.format(x1[1],x2[1]))
#
#     # 'EQUAÇÃO Volume (Schumacher and Hall)
#     # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
#     #volest = math.exp(B0 + B1 * x1[1] + B2 * x2[1])
#     #print('Volume: {}'.format(volest))
#
#     return B0v,B1v,B2v
#
# def Forma():
#
#     print('Ajusta Equação de Forma com arquivo Taper.txt para estimar di')
#
#     # --------------------
#     lintaper,ntaper=learq3()
#     # --------------------
#
#     # numero da árvore
#     dado = 0
#     dap=0
#
#     # a = vetor de '1'
#     # 'x1= hi/ht
#     # 'x2= (hi/ht)**2
#     # 'y= (di_casc/dap)**2
#
#     a = []
#     x1 = []
#     x2 = []
#     y = []
#
#     # Acessa arquivo texto, armazena dap e ht por arvore
#     taper = 1
#
#     # Executa por seção
#     i=1
#     hi=0
#     hi1=1
#
#     while taper <= ntaper:
#
#         while hi<hi1:
#             dados = lintaper[taper].split()
#             hi = float(dados[2])
#
#             if taper==ntaper:
#                 pass
#             else:
#                 dados1 = lintaper[taper + 1].split()
#                 hi1 = float(dados1[2])
#
#             try:
#                 arv=int(dados[1])
#                 hi = float(dados[2])
#                 di_casc = float(dados[3])
#                 ht=float(dados[4])
#
#             except:
#                 # print('ainda não leu altura total (ht)')
#                 pass
#
#             if hi == 1.3:
#                 dap = di_casc
#
#             taper=taper+1
#
#
#         for c in range(i,taper,1):
#             dados = lintaper[c].split()
#             try:
#                 hi = float(dados[2])
#                 di_casc = float(dados[3])
#
#             except:
#                 print('arquivo Taper.txt com campos incompletos')
#
#             try:
#                 x1.append(hi/ht)
#                 x2.append((hi/ht)**2)
#                 y.append((di_casc/dap)**2)
#
#                 # print('hi/ht:{}'.format(hi/ht))
#             except:
#                 print('falta algum dado no Taper.txt')
#
#             hi=0
#             hi1=1
#         i = taper
#
#     dado = ntaper
#
#     a = dado * [1]
#     # print('a=',a)
#     # print('x1=',x1)
#     # print('x2=',x2)
#     # print('y=',y)
#     # --------------------------
#     B, r2 =regressao(dado, a, x1, x2, y)
#     # --------------------------
#
#     B0f = B[0]
#     B1f = B[1]
#     B2f = B[2]
#
#     # print('b0f={}, b1f={}, b2f={}'.format(B0f, B1f, B2f))
#
#     return B0f,B1f,B2f
#
# #  -----------------------------------------------------------
# def Forma_hi():
#
#     print('Ajusta Equação de Forma com arquivo Taper.txt para estimar hi')
#
#     # --------------------
#     lintaper,ntaper=learq3()
#     # --------------------
#
#     # numero da árvore
#     dado = 0
#     dap=0
#
#     # a = vetor de '1'
#     # 'x1= di_casc/dap
#     # 'x2= (di_casc/dap)**2
#     # 'y= (hi/ht)**2
#
#     a = []
#     x1 = []
#     x2 = []
#     y = []
#
#     # Acessa arquivo texto, armazena dap e ht por arvore
#     taper = 1
#
#     # Executa por seção
#     i=1
#     hi=0
#     hi1=1
#
#     while taper <= ntaper:
#
#         while hi<hi1:
#             dados = lintaper[taper].split()
#             hi = float(dados[2])
#
#             if taper==ntaper:
#                 pass
#             else:
#                 dados1 = lintaper[taper + 1].split()
#                 hi1 = float(dados1[2])
#
#             try:
#                 arv=int(dados[1])
#                 hi = float(dados[2])
#                 di_casc = float(dados[3])
#                 ht=float(dados[4])
#
#             except:
#                 # print('ainda não leu altura total (ht)')
#                 pass
#
#             if hi == 1.3:
#                 dap = di_casc
#
#             taper=taper+1
#
#
#         for c in range(i,taper,1):
#             dados = lintaper[c].split()
#             try:
#                 hi = float(dados[2])
#                 di_casc = float(dados[3])
#
#             except:
#                 print('arquivo Taper.txt com campos incompletos')
#
#             try:
#                 x1.append(di_casc/dap)
#                 x2.append((di_casc/dap)**2)
#                 y.append((hi/ht)**2)
#
#                 # print('hi/ht:{}'.format(di_casc/dap))
#             except:
#                 print('falta algum dado no Taper.txt')
#
#             hi=0
#             hi1=1
#         i = taper
#
#     dado = ntaper
#
#     a = dado * [1]
#     # print('a=',a)
#     # print('x1=',x1)
#     # print('x2=',x2)
#     # print('y=',y)
#     # --------------------------
#     B, r2 =regressao(dado, a, x1, x2, y)
#     # --------------------------
#
#     B0f_hi = B[0]
#     B1f_hi = B[1]
#     B2f_hi = B[2]
#
#     # print('b0f_hi={}, b1f_hi={}, b2f_hi={}'.format(B0f_hi, B1f_hi, B2f_hi))
#
#     return B0f_hi, B1f_hi, B2f_hi


#  -----------------------------------------------------------
# Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
def calc_preco(linpeca_preco, codpeca, peca, es_tmp, vr, l_tmp, cod_tora, hi, n_tmp, v_tmp,
        di_casc2, di_casc, di, cod_arv, talh, parc, conic_tora,
        ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list,
          codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list):

    w = 1

    # 08/09/2023
    # preco = linpeca_preco[w].split()
    preco = linpeca_preco[w]

    if int(preco[4]) != -1:

        while codpeca != int(preco[0]):
            # 08/09/2023
            # preco = linpeca_preco[w].split()
            preco = linpeca_preco[w]
            #
            w = w + 1

        es_ref = float(preco[2])
        l_ref = float(preco[3])
        preco_ref = float(preco[4])

        v_ref = es_ref / 1000 * l_ref / 1000 * hi
        preco_peca = (v_tmp / v_ref) * preco_ref

        codpeca_list.append(codpeca)
        peca_list.append(peca)
        es_list.append(es_tmp)
        l_div_list.append(l_tmp)
        n_div_list.append(n_tmp)
        v_list.append(v_tmp)
        ntora_list.append(cod_tora)
        hi_list.append(hi)
        vr_list.append(vr)
        di_casc2_list.append(di_casc2)
        di_casc_list.append(di_casc)
        di_list.append(di)
        arv_list.append(cod_arv)
        talh_list.append(talh)
        parc_list.append(parc)
        preco_list.append(preco_peca)
        conic_tora_list.append(conic_tora)

    return codpeca_list, peca_list, es_list, l_div_list, n_div_list, v_list, preco_list,\
             preco_peca, \
            ntora_list, hi_list, vr_list, di_casc2_list, di_casc_list, di_list, arv_list, talh_list, parc_list, conic_tora_list



def CalculaPecasRolicas_PtaSerrad(prop_casc,cod_arv, talh, parc, dap, ht, hi_atual,
                                   linpeca2,
                                  escolhaEqForma, B0f, B1f, B2f,
                                  codpeca, peca, es_list, l_div_list, n_div_list, cod_tora, vr, v, preco,
                                  di_min_list, di_max_list, hi_list,
                                  vr_list, di_casc2_list, di_casc_list, di_list, ntora_list,
                                  codpeca_list, peca_list, arv_list, talh_list, parc_list, preco_list,
                                  v_list, conic_tora_list, CRV_list, di_min, hi_min): # Alteração 04/08/2023)

    # -----------------------------------------------------------------

    # Para roliça o limite é dado pelo diametro mínimo, não pela hc
    # Com esta regra inclui-se as roliças não classificadas para serrar (hc = 0)
    # Quando hc é diferente de zero significa que a árvore foi classificada para serrar
    # e que a sobra está sendo passada para roliça (hc !=0), com a variável hi_atual informando a altura de partida
    hc = ht

    di_casc = di_casc2 = 0
    arv = 0

    preco = 0.0000

    # declarando variáveis, não necessário no VBA
    di = 100

    hi = hi_min

    # Calcula peças por seção do tronco até completar altura da árvore
    while hi_atual + hi <= hc:

        if escolhaEqForma == '0':
            # -------------------------------------------------
            # Seleciona equação de forma
            di_casc, di_casc2 = Formulas.EquacaoForma(hi_atual, hi, ht, dap, di_casc, di_casc2)
            # -------------------------------------------------

        if escolhaEqForma == '1':
            di_casc2 = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)
            di_casc = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)

            di_casc2 = di_casc2.di_casc2_result()
            di_casc = di_casc.di_casc_result()

        # ALTERAÇÃO 03/12/2023 Calculo derivada di para verificar inconsistência Taper
        di_casc2_deriv = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)
        di_casc_deriv = Eqdi_casc(B0f, B1f, B2f, hi_atual, hi, ht, dap)

        di_casc2_deriv = di_casc2_deriv.di_casc2_deriv_result()
        di_casc_deriv = di_casc_deriv.di_casc_deriv_result()

        ## ALTERAÇÃO 03/12/2023 Se derivada positiva, diametro começa a aumentar
        # Solução: utilizar a taxa com sinal negativo e deduzir do di_casc2
        if di_casc2_deriv <= 0:
            di = di_casc2 * (1 - prop_casc) * 10
        else:
            di = (di_casc2 - di_casc2_deriv) * (1 - prop_casc) * 10
        ##
        if di < di_min or hi_atual + hi_min > hc:
            break

        cod_tora = cod_tora + 1

        conic_tora = 0

        linpeca = linpeca2
        # ----------------------------------------------------------------------------------
        # SelecionaPeca
        hi, cod_tora, \
        codpeca_list, peca_list, es_list, l_div_list, n_div_list, di_min_list, di_max_list, hi_list, vr_list, \
        di_casc2_list, di_casc_list, di_list, ntora_list, arv_list, talh_list, parc_list, preco_list, \
        v_list, conic_tora_list, CRV_list\
            = SelecionaPeca(dap, ht, hc, hi_atual, hi_min, di_casc, di_casc2, di,
                                       prop_casc, cod_tora, arv, cod_arv, talh, parc, linpeca,
                                       escolhaEqForma, B0f, B1f, B2f, conic_tora,
                                       codpeca_list, peca_list, es_list, l_div_list, n_div_list, di_min_list, di_max_list, hi_list,
                                       vr_list, di_casc2_list, di_casc_list, di_list, ntora_list,
                                       arv_list, talh_list, parc_list, preco_list,
                                       v_list, conic_tora_list, CRV_list) # Alteração 01/08/2023
        # ----------------------------------------------------------------------------------
        linpeca2 = linpeca

        # atualiza hi_atual com o comprimento da tora calculada (hi)
        hi_atual = hi_atual + hi
        hi = hi_min

        # print('hi_atual=', hi_atual)

    return \
        codpeca, peca, es_list, l_div_list, n_div_list, cod_tora, vr, v, preco, \
        di_min_list, di_max_list, hi_list,  vr_list, di_casc2_list, di_casc_list, di_list, ntora_list, \
        codpeca_list, peca_list, arv_list, talh_list, parc_list, preco_list, v_list, conic_tora_list, \
        CRV_list, di_min, hi_min # Alteração 04/08/2023
