# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from funcoes import forma, volume


class wizard_calcmadeira_calcula_coeficiente(osv.TransientModel):

    _name = 'wizard.calcmadeira.calcula.coeficiente'

    _columns = {
        'company_id': fields.many2one('res.company', _('Empresa'), ondelete='cascade',
                                      required=True, select=True),
        'name': fields.char(_('Descricao'), size=60, required=True),
        'wizcalcoe_calculado': fields.boolean(_('Calculado')),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(
            cr, uid, uid).company_id.id,
        'wizcalcoe_calculado': False,
    }

    def wizcalcoe_gera_coeficiente(self, cr, uid, ids, context=None):
        """
        O metodo processa os dados de taper (forma) ou cubagem (volume) para calculo dos
        coeficientes B0, B1, B2 e r2
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        descricao = self.read(cr, uid, isinstance(ids, list) and ids[0] or ids, ['name'],
                              context).get('name')
        vals_coeficiente_equacao = {'name': descricao}
        if context.get('calcmadeira_forma'):
            obj_calcmadeira_taper = self.pool.get('calcmadeira.taper')
            taper_ids = obj_calcmadeira_taper.search(cr, uid, [], context=context)
            taper_dados = obj_calcmadeira_taper.read(cr, uid, taper_ids, [], context)
            if not taper_ids:
                raise osv.except_osv(_('Atencao'), _('Nao ha dados no cadastro de Taper.'))
            b0, b1, b2, r2 = forma(taper_dados)
            vals_coeficiente_equacao.update({'cmacoe_tipo': 'F', 'cmacoe_b0': b0, 'cmacoe_b1': b1,
                                             'cmacoe_b2': b2, 'cmacoe_r2': r2})
        elif context.get('calcmadeira_volume'):
            obj_calcmadeira_parametro = self.pool.get('calcmadeira.parametro')
            calcmadeira_parametro_id = obj_calcmadeira_parametro.search(cr, uid, [],
                                                                        context=context)
            if not calcmadeira_parametro_id:
                raise osv.except_osv(_('Atencao'), _('Parametros nao cadastrados.'))
            parametro_dados = obj_calcmadeira_parametro.read(
                cr, uid, calcmadeira_parametro_id[0], [], context, '_classic_write')
            context.update({'calculo_fator_forma': True})
            obj_wizard_calcmadeira_calcula = self.pool.get('wizard.calcmadeira.calcula')
            calcmadeira_calcula_id = obj_wizard_calcmadeira_calcula.create(cr, uid, {}, context)
            obj_wizard_calcmadeira_calcula.calcula_peca_fator_forma(
                cr, uid, [calcmadeira_calcula_id], parametro_dados, context)
            del context['calculo_fator_forma']

            obj_calcmadeira_cubagem = self.pool.get('calcmadeira.cubagem')
            cubagem_ids = obj_calcmadeira_cubagem.search(cr, uid, [], context=context)
            cubagem_dados = obj_calcmadeira_cubagem.read(cr, uid, cubagem_ids, [], context)
            if not cubagem_ids:
                raise osv.except_osv(_('Atencao'), _('Nao ha dados no cadastro de Cubagem.'))
            b0, b1, b2, r2 = volume(cubagem_dados)
            vals_coeficiente_equacao.update({'cmacoe_tipo': 'V', 'cmacoe_b0': b0, 'cmacoe_b1': b1,
                                             'cmacoe_b2': b2, 'cmacoe_r2': r2})
        self.pool.get('calcmadeira.coeficiente.equacao').create(cr, uid, vals_coeficiente_equacao,
                                                                context)
        self.write(cr, uid, ids, {'wizcalcoe_calculado': True}, context)
        return True
