#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

__version__ = (0, 1)

import hashlib
import ConfigParser
import os.path
from lib import terminal, openerp
from optparse import OptionParser

# Niveis ( em ´exit()´)
_LEVEL = {'ok': 0, 'alert': 1, 'critical': 2, 'error': 3, 'warning': 4,
          'notice': 5, 'info': 6, 'debug': 7}

_NOECHO = False  # Nao mostra caracteres a serem digitados (terminal)
_GO = True  # Inclui um ´enter´ no final da digitacao de um campo (terminal)

termx = terminal.TelnetEmulation()
xmlrpc = openerp.XmlrpcOpenERP()


class Coletor(object):
    """
    Coletor de dados.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        self.cfg = None
        self.fileconf = None
        self.params()  # Parametros iniciais

        if self.login():
            # Modulos para o coletor
            menu_option = []
            menu_option_module = {}

            args = [('usuario_id', '=', xmlrpc.uid)]
            usuario_ids = xmlrpc.execute('usuario.coletor', 'search', args)

            campos = ['modulo_ids']
            dados_modulo = xmlrpc.execute('usuario.coletor', 'read', usuario_ids, campos)

            if dados_modulo and isinstance(dados_modulo, list):
                dados_modulo = dados_modulo[0]

            if dados_modulo and isinstance(dados_modulo, dict):
                campos = ['name', 'fonte', 'sequence', 'caminho']
                modulo_ids = dados_modulo.get('modulo_ids', [])
                modulos = xmlrpc.execute('modulo.coletor', 'read', modulo_ids, campos)

                if modulos:
                    modulos = sorted(modulos, key=lambda m: m.get('sequence', False))
            else:
                modulos = []

            try:
                for modulo in modulos:
                    menu_option.append(modulo.get('name'))
                    menu_option_module[len(menu_option)] = modulo.get('caminho') + modulo.get('fonte')
            except Exception as err:
                self.log_coletor('error', err)
            self.select_empresa(dados_modulo)
            self.menu_coletor(*menu_option, modules=menu_option_module)

        termx.clear()
        exit(0)

    def select_empresa(self, dados_modulo):
        """
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        campos = ['company_ids']
        dados_res_users = xmlrpc.execute('res.users', 'read', dados_modulo.get('usuario_id'), campos)
        if len(dados_res_users.get('company_ids'))<=1:
            company_id = dados_res_users.get('company_ids')
            if company_id and isinstance(company_id, list):
                company_id=company_id[0]
            return company_id
        __ESC = chr(27)
        termx.clear(2)
        termx.say(2,1,'.------------------.')
        termx.say(3,1,'|Digite o numero da|')
        while True:
            termx.clear(4)
            termx.say(4,1, '|empresa:          |')
            termx.say(5,1, '|                  |')
            termx.say(6,1, '|                  |')
            termx.say(7,1, '|                  |')
            termx.say(8,1, "`------------------'")
            company_id = termx.get(4,11,3)
            if company_id and company_id.isdigit():
                if not int(company_id) in dados_res_users.get('company_ids'):
                    continue
                context={'sig_force_company':True}
                xmlrpc.execute('res.users', 'write', dados_modulo.get('usuario_id'), {'company_id': int(company_id)}, context)
                return company_id
            elif company_id == __ESC:
                termx.clear()
                exit(0)
            else:
                continue

    def params(self):
        """
        Parametros iniciais.

        :return: {object} arquivo de configuracao.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Leitura do arquivo de configuracao
        file_conf = './coletor.conf'

        if args:
            if os.path.isfile(args[0]):
                file_conf = args[0]

        self.cfg = ConfigParser.ConfigParser()
        self.fileconf = self.cfg.read(file_conf)

        # Terminal
        termx.max_col = 20  # 20 colunas por...
        termx.max_row = 10  # ...10 linhas

        return self.cfg

    def login(self):
        """
        Monta a tela de login.

        :return: {boolean} verdadeiro se a conexao com o servidor foi realizada com sucesso.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Cores
        color_title = termx.underline,  # Branco (sublinhado)
        color_header = termx.fg_cyan,  # Azul claro
        color_error = termx.fg_white, termx.bg_red  # Branco/vermelho

        line_error = termx.max_row  # Linha onde aparecerao as mensagens de erro
        __ESC = chr(27)

        # Tela de login
        termx.clear()
        termx.say(1, 1, 'SigERP-Coletor V-%s.%s' % __version__, *color_title)
        termx.say(2, 1, ' - IDENTIFICACAO -  ', *color_header)
        termx.say(3, 1, 'Empresa ', *color_header)
        while True:
            try:
                ident_empresa = termx.get(3, 9, 12)
                if not ident_empresa:
                    continue
                elif ident_empresa == __ESC:
                    termx.clear()
                    return False

                self.cfg.get(ident_empresa, 'hostname')
                break
            except:
                termx.say(line_error-1, 1, 'E: identificacao', *color_error)
                termx.say(line_error, 1, 'da emrpesa incorreta', *color_error)
                termx.get(line_error, 20, 0)
                termx.clear(line_error-2, line_error)

        termx.say(5, 1, 'Usuario __________')
        termx.say(7, 1, '  Senha __________')
        termx.say(9, 1, 'Aguarde', termx.bold, termx.fg_green)
        termx.say(9, 8, '...', termx.blink, termx.bold, termx.fg_green)

        # Conexao com OpenERP
        try:
            host = self.cfg.get(ident_empresa, 'hostname')
            port = self.cfg.get(ident_empresa, 'port')
            db = self.cfg.get(ident_empresa, 'database')
            user = self.cfg.get(ident_empresa, 'username')
            pwd = self.cfg.get(ident_empresa, 'password')

            if not xmlrpc.connect(host, port, user, pwd, db):
                print 'Falha na conexao com OpenERP.'
                print '%s:%s:%s db=%s' % (host, port, user, db)
                basename = os.path.basename(self.fileconf[0])
                print 'DICA: verifique o arquivo %s' % basename
                exit(3)

        except Exception as err:
            self.log_coletor('critical', err)
            return False

        # Login
        while True:
            termx.clear(9)
            user = termx.get(5, 9, 10)  # Usuario (login)
            termx.erase_line(9)

            if not user:
                continue
            elif user == __ESC:
                termx.clear()
                return False

            senha = None

            while not senha:
                senha = termx.get(7, 9, 10, False)

            senha = hashlib.md5(senha).hexdigest()  # Criptografia da senha

            # Valida usuário/senha
            args = [('login_usuario', '=', user), ('senha_coletor', '=', senha)]
            user_ids = xmlrpc.execute('usuario.coletor', 'search', args)

            if not user_ids:
                termx.beep()
                msg = 'ERRO! USUARIO/SENHA.'
                termx.say(line_error, 1, msg, *color_error)
                continue
            else:
                # Recupera dados do usuário do coletor
                campos = ['usuario_id', 'password']
                dados_usuario = xmlrpc.execute('usuario.coletor', 'read', user_ids, campos)

                if dados_usuario and isinstance(dados_usuario, list):
                    dados_usuario = dados_usuario[0]

                if dados_usuario and isinstance(dados_usuario, dict):
                    ids = dados_usuario.get('usuario_id')
                    campos = ['name', 'login', 'password']
                    dados_usuario = xmlrpc.execute('res.users', 'read', ids[0], campos)

                    if dados_usuario and isinstance(dados_usuario, list):
                        dados_usuario = dados_usuario[0]

                    if dados_usuario and isinstance(dados_usuario, dict):
                        xmlrpc.username = dados_usuario.get('login')
                        xmlrpc.password = dados_usuario.get('password')
                        xmlrpc.uid = ids[0]
                        xmlrpc.login_id = ids[0]
                        return xmlrpc.connect(xmlrpc.hostname, xmlrpc.port, xmlrpc.username,
                                              xmlrpc.password, xmlrpc.database)
                    else:
                        termx.beep()
                        msg = 'ERRO! USUARIO/SENHA.'
                        termx.say(line_error, 1, msg, *color_error)
                        continue
                else:
                    termx.beep()
                    msg = 'ERRO! USUARIO/SENHA.'
                    termx.say(line_error, 1, msg, *color_error)
                    continue

        return False

    @staticmethod
    def menu_coletor(*options, **modules):
        """
        Menu do modulo coletor.

        :param options:
        :param modules:
        :return: N/A.
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        while True:
            op = termx.menu(*options)
            if op not in modules.get('modules'):  # Fim!
                break

            # Executa o modulo
            execfile('{0}'.format(modules['modules'][op]))  # Executa o modulo

        termx.quit()
        exit(0)

    @staticmethod
    def log_coletor(level='info', description=None, abend=False):
        """
        ´Log´ do modulo coletor.

        :param level: {int} codigo de retorno,
        :param description: {char} descricao do ´log´ ou erro,
        :param abend: {boolean} verdadeiro finaliza aplicacao,
        :return: {int} nivel do erro.
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if description and __name__ == '__main__':
            print 'Coletor %s: %s' % (level.upper(), description)

        if abend:
            exit(abend)

        return _LEVEL.get(level)

if __name__ == '__main__':
    usage = '\t%prog [config file]'
    parser = OptionParser(usage=usage, version=__version__)
    parser.disable_interspersed_args()

    options, args = parser.parse_args()

    Coletor()  # Chamada da classe

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
