# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

__version__ = '0.1'

from optparse import OptionParser
import xmlrpclib, socket

# Niveis ( em ´exit()´)
_LEVEL = {'ok': 0, 'alert': 1, 'critical': 2, 'error': 3, 'warning': 4, 'notice': 5, 'info': 6,
          'debug': 7}


class XmlrpcOpenERP(object):
    """
    Conexao com OpenERP (xmlrpc)

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        """
        Inicializacao. Definicao de variáveis.

        :return: N/A.
        """

        self.hostname = '127.0.0.1'  # Numero IP do servidor
        self.port = 8069  # Porto de acesso ao ´webservice´. O padrao e 8069
        self.username = 'openerp'
        self.password = 'openerp'
        self.database = 'openerp'  # Banco de dados padrao para acesso no OpenERP
        self.uid = False  # ID do usuario de conexao
        self.login_id = False  # ID do usuario de login
        self.error = None

    def _log_xmlrpc_openerp(self, level='INFO', description=None, abend=False):
        """
        ´Log´.

        :param level: {int} codigo de retorno,
        :param description: {char} descricao do ´log´ ou erro,
        :param abend: {boolean} verdadeiro finaliza aplicacao,
        :return: {int} nivel.
        """

        if description and __name__ == '__main__':
            msg = '\033[1mOpenERP-xmlrpc %s:\033[0m %s.'
            print msg % (level.upper(), description)
            det = '>%s:%s:%s datbase=%s\n'
            print det % (self.hostname, self.port, self.username, self.database)

        if abend:
            exit(abend)

        return _LEVEL.get(level)

    def connect(self, host, port, user, pwd, dbname):
        """
        Realiza conexao via ´webservice´ (xmlrpc) com o OpenERP

        :param host: Nome ou numero IP do servidor webservice,
        :param port: Porto de conexao do webservice,
        :param user: Nome do usuário,
        :param pwd: Senha do usuário,
        :param dbname: Nome do banco de dados para conexao.
        :return: ID do usuário ou falso se a conexao falhar.
        """

        _SOCK_COMMON = 'http://{0:>}:{1:>}/xmlrpc/common'
        self.hostname = host
        self.port = port
        self.database = dbname
        self.username = user
        self.password = pwd

        try:
            sock_common = _SOCK_COMMON.format(host, port)
            socket.setdefaulttimeout(3)
            self.uid = xmlrpclib.ServerProxy(sock_common).login(dbname, user, pwd)
            del sock_common
        except Exception as err:
            self._log_xmlrpc_openerp('error', err)
            self.uid = False

        return self.uid

    def execute(self, model, method, *args):
        """
        Executa um metodo de uma classe/objeto (modelo).

        :param model: Modelo (classe/objeto),
        :param method: Metodo da classe que será executado,
        :param parameters: Argumentos utilizados pelo metodo definido em "method",
        :return: Retorna o valor do pelo metodo que foi utilizado ou falso se a
        conexao falhar.
        """

        try:
            host = self.hostname
            port = self.port
            db = self.database
            uid = self.uid
            pwd = self.password

            sock = xmlrpclib.ServerProxy('http://{0:>}:{1:>}/xmlrpc/object'.format(host, port))
            socket.setdefaulttimeout(300)
            # result = sock.execute(db, uid, pwd, model, method, *args)
            result = sock.execute(db, uid, pwd, 'coletor.util', 'coletor_get_funcao', model,
                                  method, *args)

        except Exception as e:
            self._log_xmlrpc_openerp('error', e)
            result = False

        return result

if __name__ == '__main__':
    usage = '\t%prog [config file]'
    parser = OptionParser(usage=usage, version=__version__)
    parser.disable_interspersed_args()

    options, args = parser.parse_args()

    XmlrpcOpenERP()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
