# -*- coding: utf-8 -*-
#
# OBS: para funcionar eh preciso que tenha a pasta 'static' dentro do modulo

import openerp.addons.web.http as http
from openerp.tools import config as condifERP
import json
import psycopg2
import cups
import xmlrpclib, socket
from datetime import datetime
from base64 import b64decode
import os

import time
import subprocess
from tempfile import NamedTemporaryFile
import ConfigParser

SEM_ERRO = '00'
ERRO_CONEXAO = '01'
USUARIO_SENHA = '02'
ERRO_CNPJ = '03'
ERRO_LICENCA = '04'
ITEM_NAO_CADASTRADO = '05'
ITEM_NAO_CADASTRADO_LOCAL = '06'
LEITURA_DE_PRODUTO = '07'
LOCAIS_NAO_CADASTRADOS = '10'
MODELOS_ETIQUETA_NAO_CADASTRADOS = '15'
IMPRESSORAS_NAO_CADASTRADAS = '20'
ERRO_IMPRESSAO = '21'
SEM_INVENTARIO = '25'
ITEM_NAO_CADASTRADO_INVENTARIO = '26'
MODULO_RESSUPRIMENTO_NAO_INSTALADO = '30'
SEM_REQUISICOES = '40'
REQUISICAO_NAO_CRIADA = '41'
ITEM_REQUISICAO_NAO_CRIADO = '42'
ID_REQUISICAO_INCORRETO = '43'
ERRO_ALTERAR_REQUISICAO = '44'
ERRO_AO_SEPARAR_ITEM = '45'
ITEM_NAO_CADASTRADO_REQUISICAO = '46'
ITEM_NAO_REQUERIDO = '47'
ID_PEDIDO_INCORRETO = '50'
ERRO_ALTERAR_PEDIDO = '51'
ERRO_INTERNO = '99'


class Config(object):
    def __init__(self):
        self.dados = False

    def start(self):
        if self.dados:
            return
        file = condifERP.get('addons_path') + '/coletor_android/app.config'
        if not os.path.exists(file):
            raise Exception('Arquivo de configuração "app.config" não existe no módulo '
                            '"coletor_android"')
        c = ConfigParser.ConfigParser()
        c.read(file)
        self.dados = c


config = Config()


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class MySock(object):
    def __init__(self, params):
        super(MySock, self).__init__()
        self.uid = int(params.get('uid') or 0)
        self.pwd = params.get('pwd') or ''
        url = params.get('url') or ''
        self.host = url
        self.port = params.get('port') or ''
        if not self.port and ':' not in url:
            raise Exception('Porta não informada na URL!')
        elif ':' in url:
            self.host, self.port = url.split(':')

        cnpj = params.get('cnpj')
        config.start()
        self.db = config.dados.get(cnpj, 'database')
        self.sock = xmlrpclib.ServerProxy('http://{0:>}:{1:>}/xmlrpc/object'.format(self.host,
                                                                                    self.port))
        socket.setdefaulttimeout(300)

    def execute(self, model, function, *args):
        return self.sock.execute(self.db, self.uid, self.pwd, 'coletor.util',
                                 'coletor_get_funcao', model, function, *args)


class sc_coletor(http.Controller):
    _cp_path = '/coletor'

    @http.httprequest
    def teste(self, req, s_action=None, **kw):
        ip = req.httprequest.environ.get('REMOTE_ADDR')
        print(ip)
        return 'Resposta Teste: ' + ip

    def _get_cursor(self, params):
        try:
            cnpj = params.get('cnpj')
            config.start()
            try:
                user = config.dados.get(cnpj, 'username')
            except Exception as e:
                if 'No section:' in e.message:
                    raise Exception('O CNPJ "%s" não está cadastrado no arquivo de configuração'
                                    ' "app.config"' % str(cnpj))
                else:
                    raise e
            pwd = config.dados.get(cnpj, 'password')
            if pwd == "''":
                pwd = ''
            host = config.dados.get(cnpj, 'hostname')
            port = config.dados.get(cnpj, 'port')
            db = config.dados.get(cnpj, 'database')
            conn = psycopg2.connect(database=db, password=pwd, user=user, host=host,
                                    port=port, connect_timeout=4)
            cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            return cr, False
        except Exception as e:
            return False, json.dumps({'codigo': ERRO_CONEXAO,
                                      'mensagem': 'Erro: %s' % e,
                                      'retorno': [],
                                      })

    @http.httprequest
    def set_log(self, req, s_action=None, **kw):
        log = '%s: %s\n' % (datetime.now(), str(req.params.get('log') or ''))
        open('/tmp/log.txt', 'a').write(log)
        return json.dumps({'codigo': SEM_ERRO,
                           'mensagem': 'ok',
                           'retorno': []})

    @http.httprequest
    def check_licenca(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            vals = {'serial': req.params.get('serial'),
                    'model': req.params.get('model'),
                    'brand': req.params.get('brand'),
                    'cnpj': req.params.get('cnpj'),
                    }
            if not req.params.get('cnpj'):
                return json.dumps({'codigo': ERRO_CNPJ,
                                   'mensagem': 'CNPJ não informado!',
                                   'retorno': []})

            if not sock.execute('coletor.dispositivo', 'coldis_check_licenca', vals):
                return json.dumps({'codigo': ERRO_LICENCA,
                                   'mensagem': 'Licensa do dispositivo inválida!',
                                   'retorno': []})

            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'ok',
                               'retorno': []})

        except Exception as e:
            if type(e) == xmlrpclib.Fault:
                msg_erro = e.faultString
                if e.faultCode == 'AccessDenied':
                    msg_erro = 'Erro: %s' % "Acesso ao ERP negado para este Usuário e Senha!"
                elif '9999\n' in e.faultCode:
                    msg_erro = e.faultCode.split('9999\n')[1]
                return json.dumps({'codigo': ERRO_INTERNO,
                                   'mensagem': msg_erro,
                                   'retorno': []})

            msg_erro = ''
            if 'message' in dir(e):
                msg_erro = e.message
            else:
                msg_erro == str(e)
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % msg_erro,
                               'retorno': []})

    @http.httprequest
    def logar(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            login = req.params.get('username')
            senha = req.params.get('password')
            cr.execute("""SELECT uc.usuario_id,
                                 ru.password AS pwd,
                                 mc.name     AS nome,
                                 mc.fonte,
                                 mc.sequence AS sequencia,
                                 mc.caminho
                          FROM usuario_coletor     AS uc
                          LEFT JOIN usuario_id     AS us ON us.usuario_id = uc.id  -- tabela many2many 
                          LEFT JOIN modulo_coletor AS mc ON mc.id =  us.modulo_id
                          LEFT JOIN res_users      AS ru ON ru.id = uc.usuario_id
                          WHERE login_usuario = '{login}'
                            AND senha_coletor = '{senha}'
                          ORDER BY mc.sequence""".format(login=login, senha=senha))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'codigo': USUARIO_SENHA,
                                   'mensagem': 'Login ou senha incorretos!',
                                   'retorno': []})

            vals = {}
            for i in dados:
                modulo = {'nome': i.get('nome'),
                          'sequencia': i.get('sequencia'),
                          'arquivo': i.get('fonte')}
                if not vals.get('id'):
                    vals = {'id': i.get('usuario_id'),
                            'company_id': 0,
                            'modulos': [modulo],
                            'pwd': i.get('pwd'),
                            }
                else:
                    vals['modulos'].append(modulo)

            cr.execute("""SELECT rc.id,
                                 rc.name AS nome
                          FROM res_company_users_rel AS rel
                          INNER JOIN res_company     AS rc ON rc.id = rel.cid
                          WHERE user_id = {id}                    
                       """.format(id=vals.get('id')))
            vals['empresas'] = dictfetchall(cr)
            cr.close()
            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_locais(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            company_id = int(req.params.get('loc_company_id') or 0) or int(req.params.get('company_id') or 0)
            sock = MySock(req.params)
            company_id = sock.execute('adm.empresa.objeto', 'getEmpresa', 'estoque.local', False, False, company_id)
            local_ids = sock.execute('estoque.local', 'search', [('company_id', '=', company_id)], 0, 0, '',
                                     {'sig_force_company': True})
            cr.execute("""SELECT id, 
                                 numero,
                                 name AS nome
                          FROM estoque_local
                          WHERE id IN ({ids})
                          ORDER BY numero
                       """.format(ids=','.join(map(lambda x: '%s' % x, local_ids))))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'codigo': LOCAIS_NAO_CADASTRADOS,
                                   'mensagem': 'Nenhum local cadastrado para esta empresa!',
                                   'retorno': []})

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_item(self, req, s_action=None, **kw):
        local_id = int(req.params.get('local_id') or 0)
        codigo_barras = req.params.get('codigo_barra').zfill(14)
        try:
            sock = MySock(req.params)
            args = [('codigo_barra', '=', codigo_barras)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                # cr.close()
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})

            campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
            dados_automacao = sock.execute('automacao.codigo.barra', 'read',
                                           int(automacao_codigo_id[0]), campos)
            if not local_id:
                local_id = sock.execute('estoque.parametro', 'estpar_get_fields',
                                        'local_venda_padrao_id', {})[0].get('local_venda_padrao_id')

            produto_id = dados_automacao.get('produto_id')[0]
            grade_id = int(dados_automacao.get('grade_id')[0])
            args = [('local_id', '=', local_id), ('grade_id', '=', grade_id),
                    ('produto_id', '=', produto_id)]
            estoque_local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)
            if not estoque_local_locacao_id:
                # cr.close()
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_LOCAL,
                                   'mensagem': 'Produto não cadastrado neste local!',
                                   'retorno': []})

            campos = ['name', 'grade']
            dados_grade = sock.execute('estoque.produto.grade', 'read', grade_id, campos)
            prod_embalagem_id = dados_automacao.get('produto_embalagem_id')[0]
            if isinstance(prod_embalagem_id, list):
                prod_embalagem_id = prod_embalagem_id[0]
            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read',
                                                int(prod_embalagem_id),
                                                ['embalagem_id'])
            embalagem_id = int(dados_prod_embalagem.get('embalagem_id')[0])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                               embalagem_id,
                                               ['sigla', 'quantidade_na_embalagem'])
            descricao = dados_grade.get('name')
            eps_grade_id = int(dados_automacao.get('grade_id')[0])
            eps_data = datetime.now().strftime('%Y-%m-%d')
            eps_local_id = local_id
            saldo = sock.execute('estoque.produto.saldo', 'get_saldo_do_dia',
                                 eps_grade_id, eps_data, eps_local_id, False, {'coletor_estoque_saldo': True})
            saldo = saldo.get(str(eps_grade_id)).get('disponivel')
            saldo = float(saldo) / float(dados_emb_embalagem.get('quantidade_na_embalagem'))
            grade_sigla = str(dados_grade.get('grade'))
            preco = sock.execute('estoque.produto', 'read', produto_id, ['preco_venda'])
            if preco and isinstance(preco, list):
                preco = preco[0]
            if preco and isinstance(preco, dict):
                preco = preco.get('preco_venda')
            preco = str(preco)
            preco = preco.replace('.', ',')
            if preco[len(preco) - 2:len(preco) - 1] == ',':
                preco += '0'

            vals = {'id': produto_id,
                    'nome': descricao,
                    'embalagem_id': prod_embalagem_id,
                    'qtd_embalagem': dados_emb_embalagem.get('quantidade_na_embalagem') or 1.0,
                    'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                    'saldo': str(saldo).replace('.', ','),
                    'grade_id': grade_id,
                    'sigla_grade': grade_sigla,
                    'preco': preco}
            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            msg_erro = e
            try:
                if '9999\n' in e.faultCode:
                    msg_erro = e.faultCode.split('9999\n')[1]
            except:
                pass

            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % msg_erro,
                               'retorno': []})

    # -----------------------------  Funcoes de etiqueta de gondola  ---------------------------------------
    @http.httprequest
    def get_modelos(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            modelo_ids = sock.execute('automacao.modelo.etiqueta', 'search', [])
            if not modelo_ids:
                return json.dumps({'codigo': MODELOS_ETIQUETA_NAO_CADASTRADOS,
                                   'mensagem': 'Nenhum modelo de etiqueta cadastrado!',
                                   'retorno': []})

            modelos = sock.execute('automacao.modelo.etiqueta', 'read', modelo_ids)
            dados = []
            cont = 1
            for i in modelos:
                dados.append({'nome': i.get('name'),
                              'numero': cont,
                              'arquivo': b64decode(i.get('autmet_arquivo') or '')
                              })
                cont += 1

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_impressoras(self, req, s_action=None, **kw):
        try:
            impressoras = cups.Connection().getPrinters()
            if not impressoras:
                return json.dumps({'codigo': IMPRESSORAS_NAO_CADASTRADAS,
                                   'mensagem': 'Nenhuma impressora cadastrada!',
                                   'retorno': []})

            # impressoras = dict(zip([i for i in range(len(impressoras))], impressoras))
            return json.dumps({'retorno': impressoras.keys(),
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def imprimir_etiqueta(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            codigo = req.params.get('codigo')
            modelo = req.params.get('modelo')
            impressora = req.params.get('impressora')
            company_id = int(req.params.get('company_id'))
            cb_ids = sock.execute('automacao.codigo.barra', 'search', [('codigo_barra', '=', codigo)])
            cb_id = cb_ids and cb_ids[0]
            item_dados = cb_id and sock.execute(
                'automacao.codigo.barra', 'read', cb_id, ['produto_id', 'grade_id',
                                                          'produto_embalagem_id'])
            produto_id = item_dados and item_dados.get('produto_id')[0]
            produto_dados = produto_id and sock.execute(
                'estoque.produto', 'read', produto_id, ['name', 'preco_venda', 'codigo',
                                                        'digito'], {'coletor': True}) or dict()
            if produto_dados.get('erro'):
                return json.dumps({'codigo': LEITURA_DE_PRODUTO,
                                   'mensagem': produto_dados.get('msg'),
                                   'retorno': []})

            grade_id = item_dados and item_dados.get('grade_id')[0]
            grade_dados = grade_id and sock.execute(
                'estoque.produto.grade', 'read', grade_id, [
                    'grade', 'name', 'descricao', 'codigo_no_fabricante', 'descricao_resumida',
                    'produto_codigo_fornecedor_ids']) or dict()
            fornecedor_ids = grade_dados and grade_dados.get('produto_codigo_fornecedor_ids')
            fornecedor_id = fornecedor_ids and fornecedor_ids[0]
            fornecedor_dados = fornecedor_id and sock.execute(
                'estoque.produto.codigo.fornecedor', 'read', fornecedor_id, ['codigo_fornecedor']) or \
                               dict()
            embalagem_id = item_dados and item_dados.get('produto_embalagem_id')[0]
            embalagem_dados = embalagem_id and sock.execute(
                'estoque.produto.embalagem', 'read', embalagem_id, ['embalagem_id']) or dict()
            embalagem_id = embalagem_dados and embalagem_dados.get('embalagem_id')[0]
            embalagem_dados = embalagem_id and sock.execute(
                'estoque.embalagem', 'read', embalagem_id, ['name', 'unidade_id']) or dict()
            unidade_id = embalagem_dados and embalagem_dados.get('unidade_id')[0]
            unidade_dados = unidade_id and sock.execute(
                'estoque.embalagem.unidade', 'read', unidade_id, ['name', 'unidade']) or dict()
            preco_dados = sock.execute('estoque.preco.venda', 'etiqueta_get_preco_venda', produto_id,
                                         company_id, False, grade_id, True)

            # Valores
            DESCRICAO_PRODUTO = grade_dados and (grade_dados.get('descricao_resumida') or
                                                 grade_dados.get('name'))
            EMBALAGEM = embalagem_dados and embalagem_dados.get('name')
            CODIGO_DE_BARRA = codigo[-13:-1]
            PRECO_CADASTRO = '%.2f' % produto_dados.setdefault('preco_venda', 0)
            preco = preco_dados.setdefault(str(grade_id), {}).setdefault('preco_venda', float(PRECO_CADASTRO))
            if not preco:
                preco = preco_dados.setdefault(str(produto_id), {}).setdefault('preco_venda',
                                                                               float(PRECO_CADASTRO))
            PRECO_VENDA = '%.2f' % preco
            VALOR_DESCONTO = '%.2f' % (float(PRECO_CADASTRO) - float(PRECO_VENDA))
            UNI = unidade_dados.get('unidade') or str()
            PERC_DESC = '%.2f' % ((float(PRECO_CADASTRO) and
                                   float(VALOR_DESCONTO) * 100 / float(PRECO_CADASTRO)) or 0)
            name = produto_dados.get('name')
            DESCRICAO_PRODUTO1 = name[:20]
            DESCRICAO_PRODUTO2 = name[20:40]
            CODIGO_INTERNO = produto_dados.get('codigo') or str()
            GRADE = grade_dados.get('grade') or str()
            GRADE_DESCRICAO = grade_dados.get('descricao') or str()
            CODIGO_FABRICANTE = grade_dados.get('codigo_no_fabricante') or str()
            CODIGO_FORNECEDOR = fornecedor_dados.get('codigo_fornecedor') or str()
            DT_EMISSAO = time.strftime('%d/%m/%Y')

            # Substituicao dos valores no arquivo
            modelo = modelo.format(**locals())

            # Substituicao para o digito (#)
            modelo = modelo.format(**{'#': produto_dados.get('digito') or str()})

            # Impressao
            arquivo = NamedTemporaryFile()
            arquivo.write(modelo)
            arquivo.flush()
            try:
                subprocess.call(["lp", "-n1", "-d" + impressora, arquivo.name])
            except OSError as e:
                return json.dumps({'codigo': ERRO_IMPRESSAO,
                                   'mensagem': 'Erro ao enviar impressao: %s' % e.value,
                                   'retorno': []})
            finally:
                arquivo.close()

            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'Impressão enviada com sucesso!',
                               'retorno': []})

        except Exception as e:
            return json.dumps({'codigo': ERRO_IMPRESSAO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Funcoes do Inventario  ---------------------------------------
    @http.httprequest
    def get_inventarios(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            args = [('atualizado', '=', False)]
            ids = sock.execute('estoque.inventario.capa', 'search', args)
            if not ids:
                return json.dumps({'codigo': SEM_INVENTARIO,
                                   'mensagem': 'Não há inventário disponível!',
                                   'retorno': []})

            campos = ['id', 'name']
            dados = []
            for i in sock.execute('estoque.inventario.capa', 'read', ids, campos):
                dados.append({'id': i.get('id'),
                              'nome': i.get('name')})

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_produto_inventario(self, req, s_action=None, **kw):
        try:
            codigo_barra = req.params.get('codigo') or ''
            inventario_id = int(req.params.get('inventario_id') or 0)
            local_id = int(req.params.get('local_id') or 0)
            incluir_item = int(req.params.get('incluir_item') or 0)  # 0 ou 1
            sock = MySock(req.params)
            args = [('codigo_barra', '=', codigo_barra)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})

            campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
            if isinstance(automacao_codigo_id,list):
                    automacao_codigo_id=automacao_codigo_id[0]
            dados_automacao = sock.execute('automacao.codigo.barra', 'read', int(automacao_codigo_id), campos)
            args = [('inventario_id', '=', inventario_id), ('produto_id', '=', dados_automacao.get('produto_id')[0]),
                    ('grade_id', '=', dados_automacao.get('grade_id')[0]), ('local_id', '=', local_id)]
            contagem_invent_id = sock.execute('estoque.inventario.contagem', 'search', args)
            if not contagem_invent_id and not incluir_item:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_INVENTARIO,
                                   'mensagem': 'Este item não está no inventário!',
                                   'retorno': []})

            campos = ['name']
            dados_grade = sock.execute('estoque.produto.grade', 'read', int(dados_automacao.get('grade_id')[0]), campos)
            prod_embalagem_id = int(dados_automacao.get('produto_embalagem_id')[0])
            if isinstance(prod_embalagem_id, list):
                prod_embalagem_id =prod_embalagem_id[0]

            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read', int(prod_embalagem_id), ['embalagem_id'])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read', int(dados_prod_embalagem.get('embalagem_id')[0]), ['sigla', 'quantidade_na_embalagem', 'unidade_id'])
            emb_unidade_id = dados_emb_embalagem.get('unidade_id')
            qtde_embalagem = dados_emb_embalagem.get('quantidade_na_embalagem', 1)
            if emb_unidade_id and isinstance(emb_unidade_id, list):
                emb_unidade_id=emb_unidade_id[0]

            fracionado = sock.execute('estoque.embalagem.unidade', 'read', int(emb_unidade_id), ['fracao'])
            if isinstance(fracionado, dict):
                fracionado=fracionado.get('fracao')

            produto_id = dados_automacao.get('produto_id')[0]
            grade_id = dados_automacao.get('grade_id')[0]
            descricao = dados_grade.get('name')
            args = [('inventario_id', '=', inventario_id),('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id),('local_id', '=', local_id)]
            contagem_invent_id = sock.execute('estoque.inventario.contagem', 'search', args)
            contagem_invent_id = contagem_invent_id and contagem_invent_id[0]
            usuario_ids = sock.execute('usuario.coletor', 'search', [('usuario_id', '=', sock.uid)])
            usuario_col_dados = sock.execute('usuario.coletor', 'read', usuario_ids[0], ['saldo_produto'])
            if usuario_col_dados and isinstance(usuario_col_dados, list):
                usuario_col_dados=usuario_col_dados[0]

            mostra_saldo_produto = False
            if usuario_col_dados and isinstance(usuario_col_dados, dict):
                mostra_saldo_produto = usuario_col_dados.get('saldo_produto')

            vals = {'nome': str(descricao),
                    'qtd_embalagem': str(int(qtde_embalagem)).replace('.', ','),
                    'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                    'saldo': 0,
                    'mostra_saldo': False,
                    'fracionado': fracionado,
                    'produto_id': produto_id,
                    'grade_id': grade_id,
                    'produto_embalagem_id': prod_embalagem_id}
            if mostra_saldo_produto:
                if contagem_invent_id:
                    saldo_contagem = sock.execute(
                        'estoque.inventario.contagem', 'read', contagem_invent_id,
                        ['saldo_cadastro']) or {}
                    saldo = float(saldo_contagem.get('saldo_cadastro')) / qtde_embalagem
                    vals.update({'mostra_saldo': True,
                                 'saldo': str(saldo).replace('.', ',')})

            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_locacao_inventario(self, req, s_action=None, **kw):
        # verifica se tem locacao, e cria uma locacao se for impormado o parametro para criar
        try:
            sock = MySock(req.params)
            produto_id = int(req.params.get('produto_id'))
            company_id = int(req.params.get('company_id'))
            grade_id = int(req.params.get('grade_id'))
            produto_embalagem_id = int(req.params.get('produto_embalagem_id'))
            local_id = int(req.params.get('local_id') or 0)
            incluir_locacao = int(req.params.get('incluir_locacao') or 0)  # 0 ou 1
            if incluir_locacao:
                dic_local_locacao = {'company_id': company_id,
                                     'local_id': local_id,
                                     'produto_id': produto_id,
                                     'grade_id': grade_id,
                                     'estllc_produto_embalagem_id': produto_embalagem_id}
                sock.execute('estoque.local.locacao', 'create', dic_local_locacao)

            args = [('local_id', '=', local_id), ('grade_id', '=', grade_id), ('produto_id', '=', produto_id)]
            local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)
            if not local_locacao_id and not incluir_locacao:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_LOCAL,
                                   'mensagem': 'Este item não está no Local informado!',
                                   'retorno': []})
            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_contagem_inventario(self, req, s_action=None, **kw):
        try:
            sock = MySock()
            inventario_id = int(req.params.get('inventario_id') or 0)
            produto_id = int(req.params.get('produto_id'))
            grade_id = int(req.params.get('grade_id'))
            local_id = int(req.params.get('local_id') or 0)
            produto_embalagem_id = int(req.params.get('produto_embalagem_id') or 0)
            contagem = float(req.params.get('contagem').replace(',', '.') or 0 )
            if contagem > 999999.99:
                contagem = 999999.99

            args = [('local_id','=',local_id),('grade_id','=',grade_id),('produto_id','=', produto_id)]
            local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)
            pessoa_id  = sock.execute('res.users', 'read', sock.uid, ['pessoa_id','company_id'])
            company_id = ''
            if pessoa_id and isinstance(pessoa_id, list):
                pessoa_id = pessoa_id[0]
            if pessoa_id and isinstance(pessoa_id, dict):
                company_id = pessoa_id.get('company_id')
                pessoa_id = pessoa_id.get('pessoa_id')
                if pessoa_id and isinstance(pessoa_id, list):
                    pessoa_id = pessoa_id[0]
            if company_id and isinstance(company_id, list):
                company_id=company_id[0]
            if isinstance(local_locacao_id, list):
                local_locacao_id=local_locacao_id[0]

            custos = sock.execute('estoque.preco.custo', 'get_custos_do_dia', produto_id, False, False,
                                    {'chamada_pelo_coletor': True})
            args = [('pessoa','=',pessoa_id)]
            funcionario_id = sock.execute('rh.funcionario', 'search', args)
            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read', produto_embalagem_id,
                                                ['embalagem_id'])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                               int(dados_prod_embalagem.get('embalagem_id')[0]),
                                               ['quantidade_na_embalagem'])
            contagem = contagem*int(dados_emb_embalagem.get('quantidade_na_embalagem'))
            args = [('inventario_id', '=', inventario_id), ('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id), ('local_id', '=', local_id)]
            contagem_invent_id = sock.execute('estoque.inventario.contagem', 'search', args)
            contagem_invent_id = contagem_invent_id and contagem_invent_id[0]
            if contagem_invent_id:
                parametro_id = sock.execute('estoque.parametro', 'search',
                                              [('company_id', '=', company_id)])
                tipo_contagem = 'S' # S = substituir, I = incrementar

                if parametro_id:
                    if isinstance(parametro_id, list):
                        parametro_id = parametro_id[0]

                    tipo_contagem = sock.execute('estoque.parametro', 'read',
                                          parametro_id, ['estpar_inventario_tipo_contagem']).get('estpar_inventario_tipo_contagem')


                if tipo_contagem == 'I':
                    saldo_contado = sock.execute('estoque.inventario.contagem', 'read', contagem_invent_id,
                                                 ['saldo_contado']) or {}
                    contagem += saldo_contado.get('saldo_contado') or 0

                sock.execute('estoque.inventario.contagem', 'write', contagem_invent_id,
                               {'saldo_contado':contagem})
            else:
                dic_invent_contagem = {'inventario_id':inventario_id,
                                       'produto_id': produto_id,
                                       'grade_id': grade_id,
                                       'estict_custo_medio_contabil': custos.get('estprc_custo_medio_contabil'),
                                       'estict_ultimo_custo': custos.get('estprc_ultimo_custo'),
                                       'conferido': False,
                                       'saldo_cadastro': 0,
                                       'saldo_contado': contagem,
                                       'requisitado': 0,
                                       'locacao_id': local_locacao_id,
                                       'local_id': local_id,
                                       'estict_item_atualizado': False,}
                if funcionario_id and isinstance(funcionario_id, list):
                    funcionario_id=funcionario_id[0]
                dic_invent_contagem['funcionario_id'] = funcionario_id
                sock.execute('estoque.inventario.contagem', 'create', dic_invent_contagem)

            return json.dumps({'retorno': ['ok'],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Funcoes Digitar Requisicao  -----------------------------------
    @http.httprequest
    def checa_ressuprimento(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            checa_modulo = sock.execute('adm.utils', 'admutl_check_modulo_instalado', 'estoqueressuprimento', {})
            if not checa_modulo:
                return json.dumps({'codigo': MODULO_RESSUPRIMENTO_NAO_INSTALADO,
                                   'mensagem': 'Modulo:"estoqueressuprimento" não está instalado!',
                                   'retorno': []})
            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_state_requisicao(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            state = req.params.get('state')  # 'R'=Esboco, 'P'=Aprovada, 'E'=Em Separacao
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            if not requisicao_id or requisicao_id <= 0:
                return json.dumps({'codigo': ID_REQUISICAO_INCORRETO,
                                   'mensagem': 'Id da Requisição incorreto!',
                                   'retorno': []})

            context = {'ressuprimento_alterar': True}
            res = sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id,
                                          {'state': state}, context)
            if not res:
                return json.dumps({'codigo': ERRO_ALTERAR_REQUISICAO,
                                   'mensagem': 'Erro ao alterar a Requisição!',
                                   'retorno': []})
            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_requisicoes(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            state = req.params.get('state')  # 'R'=Esboco, 'P'=Aprovada, 'E'=Em Separacao
            if len(state) > 1:
                state = eval(state)
            else:
                state = [state]
            args = [('state', 'in', state)]
            requisicao_ids = sock.execute('estoqueressuprimento.requisicao.capa', 'search', args, 0, [], 'data')
            if not requisicao_ids:
                return json.dumps({'codigo': SEM_REQUISICOES,
                                   'mensagem': 'Nenhuma requisição encontrada!',
                                   'retorno': []})
            dados = []
            for requisicao_id in requisicao_ids:
                # Utiliza a requisicao informada
                campos = ['id', 'numero','local_saida_id', 'company_id', 'resrca_empresa_entrada_id',
                          'local_entrada_id', 'state', 'requisicao_itens_ids', 'data']
                requisicao_dados = sock.execute('estoqueressuprimento.requisicao.capa',
                                               'read', int(requisicao_id), campos, {}, '_classic_write')
                if requisicao_dados and isinstance(requisicao_dados, list):
                    requisicao_dados=requisicao_dados[0]

                sai_local_id = int(requisicao_dados.get('local_saida_id'))
                ent_local_id = int(requisicao_dados.get('local_entrada_id'))
                local_saida = sock.execute('estoque.local', 'read', sai_local_id, ['id', 'name','numero'],
                                                  {}, '_classic_write')
                local_saida['nome'] = local_saida.pop('name')
                local_entrada = sock.execute('estoque.local', 'read', ent_local_id, ['id', 'name','numero'],
                                                  {}, '_classic_write')
                local_entrada['nome'] = local_entrada.pop('name')
                empresa_saida_id = int(requisicao_dados.get('company_id'))
                empresa_entrada_id = int(requisicao_dados.get('resrca_empresa_entrada_id'))
                empresa_saida = sock.execute('res.company', 'read', empresa_saida_id, ['id', 'name'], {},
                                             '_classic_write')
                empresa_saida['nome'] = empresa_saida.pop('name')
                empresa_entrada = sock.execute('res.company', 'read', empresa_entrada_id, ['id', 'name'], {},
                                               '_classic_write')
                empresa_entrada['nome'] = empresa_entrada.pop('name')
                data = str(requisicao_dados.get('data')).split('-')
                data.reverse()
                data = '%s/%s/%s' % tuple(data)
                vals = {'id': requisicao_dados.get('id'),
                        'numero': requisicao_dados.get('numero'),
                        'empresa_saida': empresa_saida,
                        'empresa_entrada': empresa_entrada,
                        'local_saida': local_saida,
                        'local_entrada': local_entrada,
                        'state': requisicao_dados.get('state'),
                        'item_ids': requisicao_dados.get('requisicao_itens_ids'),
                        'data': data,
                        }

                dados.append(vals)
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def criar_requisicao(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            entrada_company_id = int(req.params.get('entrada_company_id') or 0)
            entrada_local_id = int(req.params.get('entrada_local_id') or 0)
            saida_local_id = int(req.params.get('saida_local_id') or 0)
            dic_est_requisicao = {'resrca_empresa_entrada_id': entrada_company_id,
                                  'local_entrada_id': entrada_local_id,
                                  'local_saida_id': saida_local_id, }
            requisicao_id = sock.execute('estoqueressuprimento.requisicao.capa', 'create', dic_est_requisicao)
            if not requisicao_id:
                return json.dumps({'codigo': REQUISICAO_NAO_CRIADA,
                                   'mensagem': 'Não foi possível criar a requisição!',
                                   'retorno': []})
            dados_req = sock.execute('estoqueressuprimento.requisicao.capa', 'read', requisicao_id, ['numero'])
            dados = [requisicao_id]
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'Requisição %s criada com sucesso!' % dados_req.get('numero')})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_item_requisicao(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            produto_id = int(req.params.get('produto_id') or 0)
            embalagem_id = int(req.params.get('embalagem_id') or 0)
            grade_id = int(req.params.get('grade_id') or 0)
            quantidade_requisitada = float(req.params.get('quantidade_requisitada') or 0)
            requisicao_item = {'requisicao_id': int(requisicao_id),
                               'produto_id': produto_id,
                               'grade_id': grade_id,
                               'quantidade_requisitada': quantidade_requisitada,
                               'embalagem_id': embalagem_id}

            if not sock.execute('estoqueressuprimento.requisicao.itens', 'create', requisicao_item):
                return json.dumps({'codigo': ITEM_REQUISICAO_NAO_CRIADO,
                                   'mensagem': 'Não foi possível inclior o item na requisição!',
                                   'retorno': []})

            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item incluído na Requisição com sucesso!'})
        
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Separar Requisicao  -----------------------------------
    @http.httprequest
    def set_separar_item_requisicao(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            req_item_id = int(req.params.get('req_item_id') or 0)
            separar_conferir = int(req.params.get('separar_conferir') or 0)
            dict_req_item = {'ja_confirmada': True}
            requisicao_id = False
            if separar_conferir:
                campos = ['quantidade_requisitada', 'requisicao_id']
                dados_item = sock.execute('estoqueressuprimento.requisicao.itens', 'read',
                                          req_item_id, campos, {}, '_classic_write')
                qtd_req = dados_item.get('quantidade_requisitada')
                requisicao_id = dados_item.get('requisicao_id')
                dict_req_item['quantidade_confirmada'] = qtd_req

            if not sock.execute('estoqueressuprimento.requisicao.itens', 'write', req_item_id,
                                dict_req_item):
                return json.dumps({'codigo': ERRO_AO_SEPARAR_ITEM,
                                   'mensagem': 'Não foi possível separar o item na requisição!',
                                   'retorno': []})
            msg = 'Item separado com sucesso!'
            if separar_conferir and requisicao_id:
                msg = 'Item separado e conferido!'
                context = {'ressuprimento_alterar': True}
                sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id, {},
                             context)

            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': msg})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_itens_requisicao(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            context = {'ressuprimento_alterar': True}
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            local_id = int(req.params.get('local_id') or 0)
            company_id = int(req.params.get('company_id') or 0)
            item_ids = eval(req.params.get('item_ids') or '[]')
            args = [('id', 'in', item_ids), ('ja_confirmada', '=', False)]
            item_ids = sock.execute('estoqueressuprimento.requisicao.itens', 'search', args)
            if not item_ids:
                state = {'state': 'S'}
                if sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id, state, context):
                    return json.dumps({'retorno': [],
                                       'codigo': SEM_ERRO,
                                       'mensagem': 'Separação da requisição concluída!'})
                return json.dumps({'codigo': ERRO_ALTERAR_REQUISICAO,
                                   'mensagem': 'Erro ao alterar a Requisição!!',
                                   'retorno': []})

            campos = ['id', 'produto_id', 'grade_id', 'quantidade_requisitada', 'embalagem_id']
            itens_dados = sock.execute('estoqueressuprimento.requisicao.itens', 'read',
                                                     item_ids, campos)

            dados = []
            for item in itens_dados:
                produto_id = int(item.get('produto_id')[0])
                grade_id = int(item.get('grade_id')[0])
                args = [('company_id', '=', company_id),
                        ('produto_id', '=', produto_id),
                        ('grade_id', '=', grade_id),
                        ('local_id', '=', local_id)]
                local_locacao_id = sock.execute('estoque.local.locacao', 'search', args,  0, 0, '',
                                                {'sig_force_company': True})[0]
                campos = ['locacao', 'estllc_produto_embalagem_id']
                locacao_dados = sock.execute('estoque.local.locacao', 'read', local_locacao_id,
                                             campos, {}, '_classic_write')
                locacao = str(locacao_dados.get('locacao') or 'Sem locação')
                prod_embalagem_id = locacao_dados.get('estllc_produto_embalagem_id')
                qtd_requisitada = item.get('quantidade_requisitada')
                campos = ['name']
                grade_dados = sock.execute('estoque.produto.grade', 'read', grade_id, campos)
                dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read',
                                                      prod_embalagem_id, ['embalagem_id'],
                                                    {}, '_classic_write')
                dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                                     dados_prod_embalagem.get('embalagem_id'),
                                                     ['sigla', 'quantidade_na_embalagem'])
                qtd_embalagem = dados_emb_embalagem.get('quantidade_na_embalagem')
                args = [('grade_id', '=', grade_id),
                        ('produto_id', '=', produto_id),
                        ('produto_embalagem_id', '=', prod_embalagem_id)]
                automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)[0]
                campos = ['codigo_barra']
                dados_automacao = sock.execute('automacao.codigo.barra', 'read',
                                               automacao_codigo_id, campos)
                dados.append({'locacao': locacao,
                              'req_item_id': item.get('id'),
                              'codigo': dados_automacao.get('codigo_barra'),
                              'descricao': grade_dados.get('name'),
                              'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                              'qtd_embalagem': str(qtd_embalagem).replace('.',','),
                              'qtd_requisitada': str(qtd_requisitada/qtd_embalagem).replace('.',',')})

            dados = sorted(dados, key=lambda x: x.get('locacao'))
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Conferir Requisicao  -----------------------------------
    @http.httprequest
    def get_item_requisicao_conferencia(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            grade_id = int(req.params.get('grade_id') or 0)
            codigo_barras = req.params.get('codigo_barra').zfill(14)
            args = [('codigo_barra', '=', codigo_barras)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})

            if isinstance(automacao_codigo_id, list):
                automacao_codigo_id = automacao_codigo_id[0]
            campos = ['produto_id', 'grade_id', 'codigo_barra']
            dados_automacao = sock.execute('automacao.codigo.barra', 'read',
                                           int(automacao_codigo_id), campos)
            if dados_automacao and isinstance(dados_automacao, list):
                dados_automacao = dados_automacao[0]

            args = [('requisicao_id', '=', requisicao_id),
                    ('produto_id', '=', dados_automacao.get('produto_id')[0]),
                    ('grade_id', '=', dados_automacao.get('grade_id')[0])]
            item_requisicao_id = sock.execute('estoqueressuprimento.requisicao.itens', 'search',
                                                args)
            if not item_requisicao_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_REQUISICAO,
                                   'mensagem': 'Este produto não está na requisição!',
                                   'retorno': []})
            # verifica se a grade do item digitado eh a mesma do item da requisicao vindo de
            # Separar e Conferir
            if grade_id and grade_id != dados_automacao.get('grade_id')[0]:
                return json.dumps({'codigo': ITEM_NAO_REQUERIDO,
                                   'mensagem': 'Produto diferente do requerido!',
                                   'retorno': []})

            if isinstance(item_requisicao_id, list):
                item_requisicao_id = item_requisicao_id[0]
            grade_id = dados_automacao.get('grade_id')
            if grade_id and isinstance(grade_id, list):
                grade_id = grade_id[0]
            campos = ['name']
            grade_descricao = sock.execute('estoque.produto.grade', 'read', int(grade_id), campos)
            if grade_descricao and isinstance(grade_descricao, dict):
                grade_descricao = grade_descricao.get('name')

            campos = ['embalagem_id', 'quantidade_confirmada', 'quantidade_requisitada']
            dados_req_itens = sock.execute('estoqueressuprimento.requisicao.itens', 'read',
                                             int(item_requisicao_id), campos)
            if isinstance(dados_req_itens, list):
                dados_req_itens = dados_req_itens[0]
            prod_embalagem_id = dados_req_itens.get('embalagem_id')[0]
            if isinstance(prod_embalagem_id, list):
                prod_embalagem_id = prod_embalagem_id[0]
            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read',
                                                  int(prod_embalagem_id), ['embalagem_id'])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                                 int(dados_prod_embalagem.get('embalagem_id')[0]),
                                                 ['sigla', 'quantidade_na_embalagem'])

            qtd_embalagem = dados_emb_embalagem.get('quantidade_na_embalagem')
            qtd_requisitada = dados_req_itens.get('quantidade_requisitada')
            qts_item = 0  # quantidade do item que sera conferida
            qtd_confirmada = dados_req_itens.get('quantidade_confirmada') / qtd_embalagem + qts_item
            vals = {'req_item_id': item_requisicao_id,
                    'codigo': dados_automacao.get('codigo_barra'),
                    'descricao': grade_descricao,
                    'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                    'qtd_embalagem': str(qtd_embalagem).replace('.',','),
                    'qtd_requisitada': str(qtd_requisitada/qtd_embalagem).replace('.',','),
                    'qtd_confirmada': str(qtd_confirmada).replace('.',',')}

            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item separado com sucesso!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_qtd_conferida(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id'))
            req_item_id = int(req.params.get('req_item_id'))
            qtd_confirmada = float(req.params.get('qtd_confirmada') or 0)
            ja_confirmada = eval(req.params.get('ja_confirmada'))
            dict_req_item = {'quantidade_confirmada': qtd_confirmada,
                             'ja_confirmada': ja_confirmada and True or False}
            context = {'ressuprimento_alterar': True}
            sock.execute('estoqueressuprimento.requisicao.itens', 'write', req_item_id,
                         dict_req_item, context)
            sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id, {},context)
            campos = ['resrca_com_pendencia']
            dados_req = sock.execute('estoqueressuprimento.requisicao.capa', 'read',
                                      requisicao_id, campos)
            if not dados_req.get('resrca_com_pendencia'):
                sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id,
                                   {'state': 'F'}, context)
                return json.dumps({'retorno': [{'requisicao_conferida': True}],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Requisicao conferida!'})

            if ja_confirmada:
                return json.dumps({'retorno': [],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Item conferido com sucesso!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Requisicao de compra  -----------------------------------------
    @http.httprequest
    def get_item_requisicao_compra(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            codigo = (req.params.get('codigo_barra') or '').zfill(14)
            args = [('codigo_barra', '=', codigo)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})
            automacao_codigo_id = automacao_codigo_id[0]
            grade_id = sock.execute('automacao.codigo.barra', 'read', automacao_codigo_id,
                                           ['grade_id'], {}, '_classic_write').get('grade_id')
            dados_grade = sock.execute('estoque.produto.grade', 'read', grade_id, ['name'])
            return json.dumps({'retorno': [{'descricao': dados_grade.get('name'),
                                            'codigo_id':  automacao_codigo_id }],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_item_requisicao_compra(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id'))
            automacao_codigo_id = int(req.params.get('codigo_id'))
            qtd = float((req.params.get('quantidade') or '').replace(',', '.'))
            campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
            dados_automacao = sock.execute('automacao.codigo.barra', 'read', automacao_codigo_id,
                                           campos, {}, '_classic_write')

            prod_embalagem_id = dados_automacao.get('produto_embalagem_id')
            numero = ''
            if not requisicao_id:
                requisicao_id = sock.execute('compras.requisicao.capa', 'create',
                                               {'company_id': int(req.params.get('company_id'))})

                numero = sock.execute('compras.requisicao.capa', 'read', requisicao_id, ['numero']
                                      ).get('numero')

            sock.execute('compras.requisicao.itens', 'create',
                           {'requisicao_id': requisicao_id,
                            'produto_id': dados_automacao.get('produto_id'),
                            'grade_id': dados_automacao.get('grade_id'),
                            'comrit_produto_embalagem_id': prod_embalagem_id,
                            'quantidade': qtd})

            return json.dumps({'retorno': [{'requisicao_id': requisicao_id,
                                            'numero': numero}],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item Adicionado!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # --------------------------------  Separar Pedido  --------------------------------------------
    @http.httprequest
    def get_pedidos(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            args = [('state', 'in', ('V','S')), ('tipo', '=', 'V')]
            offset = 0
            limit = 1
            order = 'data_emissao'
            pedido_capa_id = sock.execute('estoquepedidos.pedido.capa', 'search', args, offset,
                                            limit, order)
            if not pedido_capa_id:
                return json.dumps({'retorno': [],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Nenhum pedido para separar!'})

            pedido_capa_id = pedido_capa_id[0]
            campos = ['numero_pedido', 'pedido_item_ids', 'state']
            pedido_capa_dados = sock.execute('estoquepedidos.pedido.capa', 'read',
                                               int(pedido_capa_id), campos)
            dados = []
            for pedido in pedido_capa_dados:
                pedido_itens_ids = pedido.get('pedido_item_ids') or []
                if not pedido_itens_ids:  # pedidos sem itens nao sao enviados
                    continue

                numero_pedido = pedido.get('numero_pedido')
                vals = {'id': pedido.get('id'),
                        'numero': str(numero_pedido),
                        'item_ids': pedido_itens_ids,
                        'state': pedido.get('state')}
                dados.append(vals)

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_state_pedido(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            pedido_id = int(req.params.get('id') or 0)
            state = req.params.get('state')
            if not pedido_id or pedido_id <= 0:
                return json.dumps({'codigo': ID_PEDIDO_INCORRETO,
                                   'mensagem': 'Id do Pedido incorreto!',
                                   'retorno': []})

            context = {'pedcap_write_state': True}
            if not sock.execute('estoquepedidos.pedido.capa', 'write', pedido_id, {'state': state},
                                context):
                return json.dumps({'codigo': ERRO_ALTERAR_PEDIDO,
                                   'mensagem': 'Erro ao alterar a Pedido!',
                                   'retorno': []})

            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_itens_pedido(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            company_id = int(req.params.get('company_id') or 0)
            item_ids = eval(req.params.get('item_ids') or '[]')
            campos = ['produto_id', 'grade_id', 'local_id', 'qtde_atendida', 'qtde_pedida']
            itens_dados = sock.execute('estoquepedidos.pedido.item', 'read', item_ids, campos)
            dados = []
            for item in itens_dados:
                produto_id = item.get('produto_id')[0]
                grade_id = item.get('grade_id')[0]
                local_id = item.get('local_id')[0]
                args = [('company_id', '=', company_id), ('produto_id', '=', produto_id),
                        ('grade_id', '=', grade_id), ('local_id', '=', local_id)]
                local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)[0]
                args = [('grade_id', '=', grade_id),
                        ('produto_id', '=', produto_id)]
                codigo_id = sock.execute('automacao.codigo.barra', 'search', args)[0]
                codigo_barra = sock.execute('automacao.codigo.barra', 'read', codigo_id,
                                        ['codigo_barra']).get('codigo_barra')
                campos = ['locacao']
                local_locacao = sock.execute('estoque.local.locacao', 'read', local_locacao_id,
                                             campos)
                local_locacao = local_locacao.get('locacao') or 'Sem locação'

                codigo_item = \
                sock.execute('estoque.produto', 'read', int(produto_id), ['codigo']).get(
                    'codigo')
                descricao = sock.execute('estoque.produto.grade', 'read', int(grade_id),
                                           ['name']).get('name')
                qtde_atendida = item.get('qtde_atendida')
                qtd_pedida = item.get('qtde_pedida')
                dict_produto = {'id': item.get('id'),
                                'locacao': local_locacao,
                                'codigo': codigo_barra,
                                'codigo_interno': codigo_item,
                                'descricao': descricao,
                                'produto_id': produto_id,
                                'grade_id': grade_id,
                                'local_id': local_id,
                                'qtd_atendida': str(qtde_atendida or '0').replace('.', ','),
                                'qtd_pedida': str(qtd_pedida or '0').replace('.', ','),
                                }
                dados.append(dict_produto)

            # ordena dicionario pelas ordem alfabetica das chaves
            dados = sorted(dados, key=lambda x: x.get('locacao'))
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})


    @http.httprequest
    def set_qtd_item_pedido(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            item_id = int(req.params.get('item_id') or 0)
            qtde_atendida = float((req.params.get('qtd_atendida') or '0').replace(',', '.') or 0)
            sock.execute('estoquepedidos.pedido.item', 'write', item_id,
                         {'qtde_atendida': qtde_atendida})

            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item separado'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})
