# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from coletor import termx, xmlrpc

__version__ = (0, 1)
__lastupdate__ = '2017-04-07'
__mantainer__ = 'Pierre Barbosa'


class ColetorEtiquetaGondola(object):
    """
    Impressao de etiqueta de gondola.

    """

    # Cores utilizadas
    color_input = termx.bold, termx.underline, termx.fg_white  # Branco intenso/sublinhado
    color_title = termx.underline,  # Branco (sublinhado)
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red,  # Branco/vermelho
    color_field = termx.underline, termx.fg_green,  # Verde (sublinhado)
    color_info = termx.reverse, termx.fg_cyan,  # Preto/azul claro
    color_list_os = termx.bold, termx.fg_yellow, termx.underline,  # Amarelo claro (sublinhado)
    color_os = termx.fg_yellow,  # Amarelo
    color_msg_ok = termx.fg_green,  # Verde claro
    color_descricao = termx.fg_black, termx.bg_white

    # Outros parametros
    line_error = termx.max_row  # Linha onde sao mostradas as mensagens de erro
    line_info = line_error  # Linha onde sao mostradas as informacoes
    __ESC = chr(27)

    def __init__(self):
        termx.clear()
        termx.say(1, 1, 'SigERP-Coletor V-%s.%s' % __version__, *self.color_title)

        empresa_dados = xmlrpc.execute('res.users', 'read', xmlrpc.login_id, ['company_id'])
        self.empresa = empresa_dados.get('company_id')[0]
        self.etiqueta_gondola()

    def etiqueta_gondola(self):
        """
        Imprime etiqueta de gondola para um determinado item

        :return:
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        termx.say(1, 1, '- ETIQUETA GONDOLA -', *self.color_header)
        termx.say(2, 1, '-' * 20, *self.color_header)
        termx.say(9, 1, '-' * 20, *self.color_header)
        while True:
            termx.clear(3)
            termx.say(9, 1, '-' * 20, *self.color_header)
            modelo = self._modelo_etiqueta()  # Selecao do modelo de etiqueta
            if not modelo:
                break
            impressora = self._impressora_etiqueta()  # Selecao da impressora
            if not impressora:
                continue
            termx.clear(3)
            termx.say(9, 1, '-' * 20, *self.color_header)
            while True:
                termx.say(3, 1, 'COD.: ______________')
                codigo_barra = termx.get(3, 7, 14, go=True)
                termx.say(5, 1, ' ' * 20)
                if not codigo_barra or codigo_barra == chr(27):
                    break
                codigo_barra = codigo_barra.zfill(14)

                # Recupera a descricao do item
                item_ids = xmlrpc.execute(
                    'automacao.codigo.barra', 'search', [('codigo_barra', '=', codigo_barra)])
                if not item_ids:
                    termx.say(5, 1, ' * NAO ENCONTRADO * ', *self.color_descricao)
                    continue
                item_dados = xmlrpc.execute(
                    'automacao.codigo.barra', 'read', item_ids[0], ['produto_id', 'grade_id'])
                item_id = item_dados.get('produto_id')
                item_id = item_id and item_id[0]
                item_dados = xmlrpc.execute(
                    'estoque.produto', 'read', item_id, ['name'])
                item = item_dados and item_dados.get('name') or ''
                item = xmlrpc.execute('adm.utils', 'admutl_remover_acentuacao', item)
                termx.say(5, 1, item[0:20], *self.color_descricao)
                termx.say(6, 1, item[20:40], *self.color_descricao)
                termx.say(7, 1, item[40:60], *self.color_descricao)

                # Imprimir etiqueta
                termx.say(10, 1, '0 = Nao | 1 = Sim')
                termx.say(8, 1, 'IMPRIMIR? _')
                op = termx.get(8, 11, 0, stand=1)
                if op == chr(27) or not int(op):
                    termx.clear(3)
                    termx.say(9, 1, '-' * 20, *self.color_header)
                    continue
                if not self._imprimir_etiqueta(modelo, impressora, codigo_barra):
                    termx.say(4,  1, '  PROBLEMA        ')
                    termx.say(5,  1, '     NA           ')
                    termx.say(6,  1, '  IMPRESSAO       ')
                    termx.get(7, 12, 0)
                    return False

                termx.clear(3)
                termx.say(9, 1, '-' * 20, *self.color_header)
            if not codigo_barra:
                break

        return True

    def _imprimir_etiqueta(self, modelo, impressora, codigo):
        """ Impirme os dados do produto/servico na etiqueta de gondola. """
        import time
        import subprocess
        from tempfile import NamedTemporaryFile

        cb_ids = xmlrpc.execute('automacao.codigo.barra', 'search', [('codigo_barra', '=', codigo)])
        cb_id = cb_ids and cb_ids[0]
        item_dados = cb_id and xmlrpc.execute(
            'automacao.codigo.barra', 'read', cb_id, ['produto_id', 'grade_id',
                                                      'produto_embalagem_id'])
        produto_id = item_dados and item_dados.get('produto_id')[0]
        produto_dados = produto_id and xmlrpc.execute(
            'estoque.produto', 'read', produto_id, ['name', 'preco_venda', 'codigo',
                                                    'digito'], {'coletor': True}) or dict()
        if produto_dados.get('erro'):
            termx.clear(4)
            termx.say(4, 1, produto_dados.get('msg')[:18])
            termx.say(5, 1, produto_dados.get('msg')[18:36])
            termx.say(6, 1, produto_dados.get('msg')[36:54])
            termx.say(7, 1, produto_dados.get('msg')[54:72])
            termx.get(8, 12, 0)
            return False

        grade_id = item_dados and item_dados.get('grade_id')[0]
        grade_dados = grade_id and xmlrpc.execute(
            'estoque.produto.grade', 'read', grade_id, [
                'grade', 'name', 'descricao', 'codigo_no_fabricante', 'descricao_resumida',
                'produto_codigo_fornecedor_ids']) or dict()
        fornecedor_ids = grade_dados and grade_dados.get('produto_codigo_fornecedor_ids')
        fornecedor_id = fornecedor_ids and fornecedor_ids[0]
        fornecedor_dados = fornecedor_id and xmlrpc.execute(
            'estoque.produto.codigo.fornecedor', 'read', fornecedor_id, ['codigo_fornecedor']) or \
                           dict()
        embalagem_id = item_dados and item_dados.get('produto_embalagem_id')[0]
        embalagem_dados = embalagem_id and xmlrpc.execute(
            'estoque.produto.embalagem', 'read', embalagem_id, ['embalagem_id']) or dict()
        embalagem_id = embalagem_dados and embalagem_dados.get('embalagem_id')[0]
        embalagem_dados = embalagem_id and xmlrpc.execute(
            'estoque.embalagem', 'read', embalagem_id, ['name', 'unidade_id']) or dict()
        unidade_id = embalagem_dados and embalagem_dados.get('unidade_id')[0]
        unidade_dados = unidade_id and xmlrpc.execute(
            'estoque.embalagem.unidade', 'read', unidade_id, ['name', 'unidade']) or dict()
        preco_dados = xmlrpc.execute('estoque.preco.venda', 'etiqueta_get_preco_venda', produto_id,
                                     self.empresa, False, grade_id, True)

        # Valores
        DESCRICAO_PRODUTO = grade_dados and (grade_dados.get('descricao_resumida') or
                                             grade_dados.get('name'))
        EMBALAGEM = embalagem_dados and embalagem_dados.get('name')
        CODIGO_DE_BARRA = codigo[-13:-1]
        PRECO_CADASTRO = '%.2f' % produto_dados.setdefault('preco_venda', 0)
        preco = preco_dados.setdefault(str(grade_id), {}).setdefault('preco_venda', float(PRECO_CADASTRO))
        if not preco:
            preco = preco_dados.setdefault(str(produto_id), {}).setdefault('preco_venda',
                                                                           float(PRECO_CADASTRO))
        PRECO_VENDA = '%.2f' % preco
        VALOR_DESCONTO = '%.2f' % (float(PRECO_CADASTRO) - float(PRECO_VENDA))
        UNI = unidade_dados.get('unidade') or str()
        PERC_DESC = '%.2f' % ((float(PRECO_CADASTRO) and
                              float(VALOR_DESCONTO) * 100 / float(PRECO_CADASTRO)) or 0)
        name =  xmlrpc.execute('adm.utils', 'admutl_remover_acentuacao',
                               produto_dados.get('name') or '')
        DESCRICAO_PRODUTO1 = name[:20]
        DESCRICAO_PRODUTO2 = name[20:40]
        CODIGO_INTERNO = produto_dados.get('codigo') or str()
        GRADE = grade_dados.get('grade') or str()
        GRADE_DESCRICAO = xmlrpc.execute('adm.utils', 'admutl_remover_acentuacao',
                                         grade_dados.get('descricao') or str())
        CODIGO_FABRICANTE = grade_dados.get('codigo_no_fabricante') or str()
        CODIGO_FORNECEDOR = fornecedor_dados.get('codigo_fornecedor') or str()
        DT_EMISSAO = time.strftime('%d/%m/%Y')

        # Substituicao dos valores no arquivo
        modelo = modelo.format(**locals())

        # Substituicao para o digito (#)
        modelo = modelo.format(**{'#': produto_dados.get('digito') or str()})

        # Impressao
        arquivo = NamedTemporaryFile()
        arquivo.write(modelo)
        arquivo.flush()
        try:
            subprocess.call(["lp", "-n1", "-d" + impressora, arquivo.name])
        except OSError:
            return False
        finally:
            arquivo.close()

        return True

    def _modelo_etiqueta(self):
        """ Seleciona o modelo de impressao de etiqueta """
        from base64 import b64decode

        modelo_ids = xmlrpc.execute('automacao.modelo.etiqueta', 'search', [])
        if not modelo_ids:
            termx.say(4,  1, 'NAO FOI ENCONTRADO')
            termx.say(5,  1, 'NENHUM MODELO DE')
            termx.say(6,  1, 'ETIQUETA.')
            termx.get(10, 20, 0)
            return False

        modelos = xmlrpc.execute('automacao.modelo.etiqueta', 'read', modelo_ids)
        modelos = dict(zip([i for i in range(len(modelos))], modelos))
        termx.say(3,  1, '0 - VOLTAR')
        for m in modelos.items()[:5]:
            termx.say(4 + m[0], 1, '%(op)s - %(mod).15s' % dict(op=m[0] + 1, mod=m[1].get('name')))
        termx.clear(10)
        termx.say(10, 1, 'SELECIONE MODELO: _')
        op = termx.get(10, 19, 0, stand=1)
        if op == chr(48) or op == chr(27):
            return False

        return b64decode(modelos[int(op) - 1].get('autmet_arquivo') or '')

    @staticmethod
    def _impressora_etiqueta():
        """ Seleciona a impressora """
        import cups

        impressoras = cups.Connection().getPrinters()
        # key_impressora = filter(lambda im: im == 'ARGOX-OS-214-plus-PPLA-203dpi', impressoras)
        # impressoras = {key_impressora[0]: impressoras.get(key_impressora[0])}
        if not impressoras:
            termx.say(4,  1, 'NAO FOI ENCONTRADA')
            termx.say(5,  1, 'NENHUMA IMPRESSORA')
            termx.get(10, 20, 0)
            return False
        impressoras = dict(zip([i for i in range(len(impressoras))], impressoras))
        termx.say(3,  1, '0 - VOLTAR')
        for i in impressoras.items()[:5]:
            termx.say(4 + i[0], 1, '%(op)s - %(imp).15s' % dict(op=i[0] + 1, imp=i[1]))
        termx.clear(10)
        termx.say(10, 1, 'IMPRESSORA: _')
        op = termx.get(10, 13, 0, stand=1)
        if op == chr(48) or op == chr(27):
            return False

        return impressoras.get(int(op) - 1)

ColetorEtiquetaGondola()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
