# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import re
from openerp.tools.translate import _


class compras_comprador(osv.Model):
    """
    Tabela de Compradores da empresa.
    """
    _name = 'compras.comprador'
    _inherit = 'compras.comprador'

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['pessoa_id'], context=context)
        res = []
        for record in reads:
            name = record['pessoa_id'][1]
            res.append((record['id'], name))
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if args and isinstance(args, list):
            arg = args[0]
            if 'id' in arg:
                args = []
        res = super(compras_comprador, self).search(cr, uid, args, offset, limit, order, context,
                                                    count)
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        vals['pessoa_tipo'] = 'F' # Fixando o tipo = fisica, porque não existe na tela
        pessoa = vals.get('pessoa_id')
        if pessoa:
            pessoa_id = self.pool.get('compras.comprador').search(cr, uid,
                                                                  [('pessoa_id', '=', pessoa),
                                                                   ('active', '=', True)], context=context)
            if pessoa_id:
                self.write(cr, uid, pessoa_id[0], vals, context)
        else:
            pessoa_id = self.pool.get('adm.pessoa').search(cr, uid,
                                                           [('nome', '=', vals.get('name')),
                                                            ('active', '=', True),
                                                            ('cpf_cnpj', '=', vals.get('cpf_cnpj'))], context=context)
            if pessoa_id:
                vals['pessoa_id'] = pessoa_id[0]

        res = super(compras_comprador, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(compras_comprador, self).write(cr, uid, ids, vals, context)
        return res

    def copy(self, cr, uid, id, default={}, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'), _('O comprador ja esta cadastrado'))

    def _comcpr_prepara_dados_comprador(self, cr, uid, pessoa_id, context=None):
        """
        Este médoto prepara os dados da pessoa para serem retornados para o onchange
        o formato do retorno é {'value:{}, 'warning:{}}
        """
        context = context and context.copy() or {}
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_compras_comprador = self.pool.get('compras.comprador')
        comprador_id = obj_compras_comprador.search(cr, uid, [('pessoa_id', '=', pessoa_id),
                                                              ('active', '=', True)], context=context)
        if comprador_id:
            warning = {'title': _('Atencao')}
            warning['message'] = _('Comprador ja cadastrado!')
            return {'warning': warning, 'value': {'cpf_cnpj': '', 'pessoa_id': False}}
        dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_id)
        if isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]
        endereco_ids = []
        for dados_endereco in dados_pessoa.endereco_ids:
            enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                         'active': dados_endereco.active,
                         "cep": dados_endereco.cep,
                         "pais_id": dados_endereco.pais_id.id,
                         "uf_id": dados_endereco.uf_id.id,
                         "cidade_id": dados_endereco.cidade_id.id,
                         "bairro_id": dados_endereco.bairro_id.id,
                         "logradouro_id": dados_endereco.logradouro_id.id,
                         "numero": dados_endereco.numero,
                         "complemento": dados_endereco.complemento,
                         "observacao": dados_endereco.observacao,
            }
            endereco_ids.append((1, dados_endereco.id, enderecos))
        email_ids = []
        for dados_emails in dados_pessoa.email_ids:
            emails = {
                "email_tipo_id": dados_emails.email_tipo_id.id,
                "name": dados_emails.name,
                "observacao": dados_emails.observacao,
            }
            email_ids.append((1, dados_emails.id, emails))
        telefones_ids = []
        for dados_telefones in dados_pessoa.telefones_ids:
            telefones = {
                "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                "ddi": dados_telefones.ddi,
                "ddd": dados_telefones.ddd,
                "numero": dados_telefones.numero,
                "observacoes": dados_telefones.observacoes,
            }
            telefones_ids.append((1, dados_telefones.id, telefones))
        res = {'value':
                   {'cpf_cnpj': dados_pessoa.cpf_cnpj,
                    'name': dados_pessoa.name,
                    'codigo': dados_pessoa.codigo,
                    'fantasia_apelido': dados_pessoa.fantasia_apelido,
                    'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                    'admpes_fis_rg_orgao_emissor': dados_pessoa.admpes_fis_rg_orgao_emissor,
                    'admpes_fis_rg_data_emisssao': dados_pessoa.admpes_fis_rg_data_emisssao,
                    'admpes_fis_cnh_nro': dados_pessoa.admpes_fis_cnh_nro,
                    'admpes_fis_cnh_orgao_emissor': dados_pessoa.admpes_fis_cnh_orgao_emissor,
                    'admpes_fis_cnh_validade': dados_pessoa.admpes_fis_cnh_validade,
                    'admpes_fis_cnh_categoria': dados_pessoa.admpes_fis_cnh_categoria,
                    'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                    'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                    'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                    "foto": dados_pessoa.foto,
                    'endereco_ids': endereco_ids,
                    'email_ids': email_ids,
                    'telefones_ids': telefones_ids,
                    'pessoa': dados_pessoa.id,
                   }
        }
        return res

    def comcpr_onchange_pessoa_id(self, cr, uid, ids, pessoa_id, context=None):
        context = context and context.copy() or {}
        if pessoa_id:
            res = self._comcpr_prepara_dados_comprador(cr, uid, pessoa_id, context)
        else:
            res = {'value': {}, 'warning': {}}
        return res

    def onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        pessoa_ids = obj_pessoa.search(cr, uid, [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar),
                                                 ('pessoa_tipo', '=', pessoa_tipo)], context=context)
        if pessoa_ids:
            res = self._comcpr_prepara_dados_comprador(cr, uid, pessoa_ids[0], context)
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}



