# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


READ_LOAD = '_classic_write'


class compras_solicitacao_capa(osv.Model):
    """
    Tabela de solicitacao de itens do almoxarifado.
    Uma solicitacao será atendida por uma requisicao.
    O requisitante e uma pessoa e deve ser um funcionário ou fornecedor em servico para a empresa.
    """

    _name = 'compras.solicitacao.capa'
    _inherit = 'compras.solicitacao.capa'

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(compras_solicitacao_capa, self).default_get(cr, uid, fields, context=context)
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        usuario_id = uid
        if not isinstance(uid, list):
            usuario_id = [uid]
        res_users_dados = self.pool.get('res.users').read(cr, uid, usuario_id, ['pessoa_id'], context=context)
        for x in res_users_dados:
            pessoa_id = x.get('pessoa_id')
        if isinstance(pessoa_id, tuple):
            pessoa_id = pessoa_id[0]
        funcionario_id = obj_rh_funcionario.search(cr, uid, [('pessoa', '=', pessoa_id)], context=context)
        if not funcionario_id:
            raise osv.except_osv(_('Atencao'), _('Usuario nao cadastrado como funcionario'))
        res['funcionario_id'] = pessoa_id
        return res

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __code = 'solicitacao_capa.numero'
        __name = 'Numero da solicitacao de compra'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 6,
            'prefix': 'SCO-%(year)s-',
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence').search(cr, uid, [('code', '=', __code), ('company_id', '=', company_id)], context=context)
        if not consulta:
            self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def button_analizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        solitacao_dados = self.read(cr, uid, ids, ['solicitacao_item_ids'], context=context, load=READ_LOAD)
        for i in solitacao_dados:
            if not i.get('solicitacao_item_ids'):
                raise osv.except_osv(_('Atencao'), _('Favor inserir itens para a solicitacao!'))
        res = self.write(cr, uid, ids, {'state': 'A'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), _('Solicitacao em analise.'))
        return res

    def button_analizar_itens(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return self.pool.get('wizard.compras.itens.solicitacao').analizar_itens(cr, uid, ids, context)

    def button_aprovar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids, ['company_id', 'solicitacao_item_ids', 'state'], context=context)[0]
        obj_req_capa = self.pool.get('compras.requisicao.capa')
        obj_req_itens = self.pool.get('compras.requisicao.itens')
        req_id = obj_req_capa.create(cr, uid, {
            'company_id': vals['company_id'][0],
            'solicitacao_id': vals['id'],
            'state': 'B'
        }, context)
        #        obj_req_capa. write(cr, uid, ids, {'state': 'D'}, context={'read_only': 'N'})
        # ITENS DA SOLICITACAO
        obj_solicitacao_itens = self.pool.get('compras.solicitacao.item')
        solicitacao_itens = obj_solicitacao_itens.read(cr, uid, vals['solicitacao_item_ids'], context=context)
        for item in solicitacao_itens:
            if item['grade_id']:
                grade_id = item['grade_id'][0]
            else:
                grade_id = False
            if item['comsit_produto_embalagem_id']:
                comrit_produto_embalagem_id = item['comsit_produto_embalagem_id'][0]
            else:
                comrit_produto_embalagem_id = False
            obj_req_itens.create(cr, uid, {
                'produto_id': item['produto_id'][0],
                'quantidade': item['quantidade_atendida'],
                'grade_id': grade_id,
                'comrit_produto_embalagem_id': comrit_produto_embalagem_id,
                'requisicao_id': req_id,
            }, context)
            # Pessoa(usuário) que aprovou a solicitacao
        usuario_id = uid
        if not isinstance(uid, list):
            usuario_id = [uid]
        res_users_dados = self.pool.get('res.users').read(cr, uid, usuario_id, ['pessoa_id'], context=context)
        for x in res_users_dados:
            pessoa_id = x.get('pessoa_id')
        if isinstance(pessoa_id, tuple):
            pessoa_id = pessoa_id[0]
        res = self.write(cr, uid, ids, {'state': 'P', 'requisicao_id': req_id, 'comsol_aprovado_por_id': pessoa_id}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), _('Solicitacao Aprovada'))
        return res


    def button_recusar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {'state': 'R'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), _('Solicitacao Recusada'))
        return res

    def button_rascunho(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {'state': 'E'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), _('Solicitacao em Esboco'))
        return res

    def button_cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {'state': 'C'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), _('Cancelamento Solicitacao'))
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        vals['numero'] = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, 'solicitacao_capa.numero')
        res = super(compras_solicitacao_capa, self).create(cr, uid, vals, context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'),
                             _('Nao e permitido excluir uma solicitacao de compra, apenas cancelar.'))

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        if ids:
            state = self.read(cr, user, ids[0], ['state'], context=context)['state']
            if state != 'E' and 'solicitacao_item_ids' in vals:
                if not 'state' in vals:
                    raise osv.except_osv(_('Atencao'),
                                         _('A alteracao do item e permitida somente quando a solicitacao estiver em Esboco.'))
        return super(compras_solicitacao_capa, self).write(cr, user, ids, vals, context=context)

    def botao_atualizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True

    def abrir_requisicao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj = self.pool.get('compras.solicitacao.capa')
        if isinstance(ids, list):
            ids = ids[0]
        requisicao_id = obj.browse(cr, uid, ids).requisicao_id.id
        if not requisicao_id:
            raise osv.except_osv(_('Atencao'), _('Requisicao nao gerada'))
        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'compras', 'view_compras_requisicao_capa_form')
        view_id = reference and reference[1] or False
        return {
            'name': _('Requisicao Gerada'),
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view_id],
            'res_model': 'compras.requisicao.capa',
            'res_id': requisicao_id,
            'type': 'ir.actions.act_window',
        }

