#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv


class estoque_inventario_contagem(osv.Model):
    _name = 'estoque.inventario.contagem'
    _inherit = 'estoque.inventario.contagem'

    def get_dados_para_contagem(self, cr, uid, local_id, locacao_id, categoria_id, fornecedor_id,
                                 locacao_company_id, context=None, **kwargs):
        """
        Metodo responsavel por retornar as informacoes dos dados dos itens que serao utilizados para
        contagem
        :param cr: Cursor do openerp
        :param uid: Id do usuario que esta fazendo a chamada
        :param local_id: ID de estoque.local para fins de filtro
        :param locacao_id: ID de estoque.local.locacao para fins de filtro
        :param categoria_id: ID de estoque.categoria para fins de filtro
        :param fornecedor_id: ID de estoque.fornecedor para fins de filtro
        :param locacao_company_id: ID da empresa que estoque.local.locacao esta correlacionada
        :param kwargs: Dicionarios contendo demais parametros, e utilizado por classes que
                       sobrescrevem este metodo, poderem receber outros parametros especificos.
                       No caso do modulo compras instalado, este recebe podera receber comprador_id
        :type cr: Curosr
        :type local_id: int
        :type locacao_id: int
        :type categoria_id: int
        :type fornecedor_id: int
        :type locacao_company_id: int
        :type kwargs: dict
        :return: Retorna a lista de dicionarios com os dados dos itens que serao considerados
                 para o inventario
        :rtype: list
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        res = super(estoque_inventario_contagem, self).get_dados_para_contagem(
            cr, uid, local_id, locacao_id, categoria_id, fornecedor_id, locacao_company_id,
            context=None, **kwargs)
        if kwargs.get('comprador_id'):
            args = [('comprador_id', '=', kwargs.get('comprador_id'))]
            produto_ids = self.pool.get('estoque.produto').search(cr, uid, args, context=context)
            res = filter(lambda i: i.get('produto_id') in produto_ids, res)
        return res