# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from SigUtils.numero import getNumeroEditado

def comsit_validar_quantidade(self, cr, uid, ids):
    """
    Valida a quantidade de produtos para o item de solicitacao

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @return: {boolean} verdadeiro se a quantidade for um valor válido
    """
    for item in self.browse(cr, uid, ids):
        if not item.quantidade:
            return False
    return True


# Funcoes da classe compras_cotacoes_capa
def _comccp_get_file_name(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if not len(ids):
        return []
    reads = self.read(cr, uid, ids, ['numero', 'file_cotacao_xls'], context=context)
    res = {}
    for record in reads:
        if not record['file_cotacao_xls']:
            name = 'Nao foi gerado o arquivo!'
        else:
            name = '%s.xls' % record['numero']
        res[record['id']] = name
    return res


def _comccp_get_valor_total(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if not len(ids):
        return []
    obj_item = self.pool.get('compras.cotacoes.itens')
    res = {}
    for id in ids:
        dados_cotacao = self.read(cr, uid, id, ['numero', 'frete_valor', 'despesas_acessorias', 'outras_despesas',
                                                'cotacoes_itens_ids'], context=context)
        soma = dados_cotacao['despesas_acessorias'] + dados_cotacao['outras_despesas'] + dados_cotacao['frete_valor']
        itens_ids = obj_item.search(cr, uid, [('cotacao_id', '=', id)], context=context)
        dados_itens = obj_item.read(cr, uid, itens_ids, ['valor', 'ipi'], context=context)
        for item in dados_itens:
            soma += item['valor'] + item['ipi']
        res[id] = soma
    return res


def _comccp_btn_salvar_visible(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if not len(ids):
        return []
    if not isinstance(context, dict):
        context = {}
    res = {}
    brw_cot_capa = self.browse(cr, uid, ids)
    if context.get('requisicao'):
        for cot_capa in brw_cot_capa:
            res[cot_capa.id] = True
    return res


# Funcoes da classe compras_cotacoes_itens
def _comcit_get_name_cotacao(self, cr, uid, ids, name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    brw_cot_itens = self.browse(cr, uid, ids, context)
    for cot_item in brw_cot_itens:
        _numero = cot_item.cotacao_id.numero
        _fornecedor = cot_item.cotacao_id.fornecedor_id.pessoa_id.nome
        _valor = cot_item.valor
        _valor = getNumeroEditado(_valor, 2, ',')
        cotacao = "%s - %s - R$%s" % (_numero, _fornecedor, _valor)
        res[cot_item.id] = cotacao
    return res


def _comcit_get_numero_cotacao(self, cr, uid, ids, name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    brw_cot_itens = self.browse(cr, uid, ids, context)
    for cot_item in brw_cot_itens:
        _numero = cot_item.cotacao_id.numero
        res[cot_item.id] = _numero
    return res


def _comcit_get_fornecedor_cotacao(self, cr, uid, ids, name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    brw_cot_itens = self.browse(cr, uid, ids, context)
    for cot_item in brw_cot_itens:
        _fornecedor = cot_item.cotacao_id.fornecedor_id.pessoa_id.nome
        res[cot_item.id] = _fornecedor
    return res

def _comcit_get_fornecedor(self, cr, uid, ids, name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    for req_id in self.browse(cr, uid, ids, context):
        brw_pessoa = req_id.cotacao_id.fornecedor_id.pessoa_id
        res[req_id['id']] = brw_pessoa.nome or ''
    return res


def _comcit_get_prazo(self, cr, uid, ids, name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    obj_cota = self.pool.get('compras.cotacoes.capa')
    req_ids = self.read(cr, uid, ids, ['cotacao_id'], context=context)
    for req_id in req_ids:
        prazo = obj_cota.read(cr, uid, req_id['cotacao_id'][0], ['prazo_entrega'], context=context)['prazo_entrega']
        res[req_id['id']] = prazo
    return res


def _comcit_get_vr_total(self, cr, uid, ids, name, args, context=None):
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    res = {}.fromkeys(ids, 0)
    item_ids = self.read(cr, uid, ids, ['vr_unitario', 'quantidade', 'ipi'], context=context)
    for item in item_ids:
        id = item['id']
        vr_unitario = item.get('vr_unitario') or 0
        quantidade = item.get('quantidade') or 0
        aliquota_ipi = item.get('ipi') or 0
        valor_ipi = vr_unitario * aliquota_ipi / 100
        res[id] = (vr_unitario + valor_ipi) * quantidade
    return res


#Funcoes da classe compras_requisicao_capa
def _comrca_name_get(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if not len(ids):
        return []
    res = {}
    for record in self.read(cr, uid, ids, ['numero'], context=context):
        res[record['id']] = record['numero']
    return res


#Funcoes da classe compras_requisicao_itens
def _comrit_get_vr_tot_cot(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if not len(ids):
        return []
    res = {}
    obj_requisicao_itens = self.pool.get('compras.requisicao.itens')
    dados = self.read(cr, uid, ids, ['cotacao_item_id'], context=context)
    obj_compras_cotacoes_itens = self.pool.get('compras.cotacoes.itens')
    result = {}
    for dado in dados:
        valor = 0.00
        cotacao_item_id = dado['cotacao_item_id']
        if cotacao_item_id:
            dados_cotacao = obj_compras_cotacoes_itens.read(cr, uid, cotacao_item_id[0],
                                                            [], context=context)
            valor = dados_cotacao['valor']
            result[dado['id']] = valor
        else:
            args_cotacao_itens = [('requisicao_itens_id', '=', dado['id'])]
            cotacao_item_ids = obj_compras_cotacoes_itens.search(cr, uid, args_cotacao_itens, context=context)

            dados_cot_item = obj_compras_cotacoes_itens.read(cr, uid, cotacao_item_ids, ['valor'], context=context)
            if dados_cot_item:
                try:
                    print dados_cot_item
                    valor = min(filter(lambda x: x > 0, map(lambda i: i['valor'], dados_cot_item)))
                except ValueError:
                    pass
            result[dado['id']] = valor
    for i in ids:
        res[i] = result.get(i)
    return res


def _comrit_get_state_capa(self, cr, uid, ids, field_name, arg, context):
    res = {}
    if ids:
        obj_req = self.pool.get('compras.requisicao.capa')
        for id in ids:
            req_id = self.read(cr, uid, id, ['requisicao_id'], context=context)['requisicao_id'][0]
            state = obj_req.read(cr, uid, req_id, ['state'], context=context)['state']
            res[id] = state
    return res


#Funcoes da classe compras_solicitacao_capa
def _comsol_name_get(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if not len(ids):
        return []
    res = {}
    for record in self.read(cr, uid, ids, ['numero'], context=context):
        res[record['id']] = record['numero']
    return res
