# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time


class compras_solicitacao_capa(osv.Model):
    """
    Tabela de solicitacao de itens do almoxarifado.
    Uma solicitacao será atendida por uma requisicao.
    O requisitante e uma pessoa e deve ser um funcionário ou fornecedor em servico para a empresa.
    """

    from funcoes import _comsol_name_get

    _name = 'compras.solicitacao.capa'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
        'name': fields.function(_comsol_name_get, method=True, string='Name', type='char', size=15, store=False),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", readonly=True, required=True,
                                      select=True), # Id da empresa
        'numero': fields.char('Numero da solicitacao', size=15, select=True), # numero da solicitacao. Deve ser composto da seguinte forma:\nSCO-AAAA-999999
        'data_hora_solicitacao': fields.datetime('Data/Hora solicitacao', select=True),
        'state': fields.selection([('E', 'Esboco'),
                                   ('A', 'Em analise'),
                                   ('P', 'Aprovado'),
                                   ('R', 'Recusado'),
                                   ('C', 'Cancelada')
                                  ], 'Status', required=True, select=True, size=1),
        'funcionario_id': fields.many2one("adm.pessoa", 'Funcionario', readonly=True, ondelete="restrict"),
        'observacao': fields.text('Observacao'), # Observacoes
        'solicitacao_item_ids': fields.one2many('compras.solicitacao.item', 'solicitacao_id', 'Item'),
        'requisicao_id': fields.many2one("compras.requisicao.capa", 'Requisicao', ondelete="set null", select=True),
        'comsol_aprovado_por_id': fields.many2one('adm.pessoa', 'Aprovado por', readonly=True, ondelete="restrict"),
    }
    _order = 'numero desc'
    _defaults = {
        'state': lambda *a: 'E',
        'data_hora_solicitacao': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }

