# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields

# ´´Status´´ do Item de Solicitacao
STATE = [('A', 'Aguardando autorizacao'), ('T', 'Autorizada'),
         ('C', 'Cancelada'), ('N', 'Nao autorizada')]


class compras_solicitacao_item(osv.Model):
    """
    Tabela de itens da solicitacao

    @change: 03.05.2013 - pierre.barbosa
    """

    from funcoes import comsit_validar_quantidade

    _name = 'compras.solicitacao.item'                                              # comsit

    _columns = {
        # ___________________________ Many2One _______________________________
        'solicitacao_id': fields.many2one("compras.solicitacao.capa",               # ID da solicitacao
                                          'Solicitacao', ondelete="cascade",
                                          required=True, select=True),

        'produto_id': fields.many2one("estoque.produto", 'Produto',                 # ID do produto
                                      ondelete="restrict", required=True,
                                      select=True),
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade',               # ID da grade
                                    ondelete="restrict", required=True,
                                    select=True,
                                    domain="[('produto_id', '=', produto_id)]"),
        'comsit_produto_embalagem_id': fields.many2one(
            'estoque.produto.embalagem', 'Embalagem', required=True,
            ondelete="restrict", select=True,
            domain="[('produto_id', '=', produto_id)]"),

        # ___________________________ One2Many _______________________________
        'solicitacao_historico_ids': fields.one2many(
            'compras.solicitacao.historico', 'item_id', 'Historico'),

        # ___________________________ Float __________________________________
        'quantidade': fields.float('Quantidade', digits=(15, 3), required=True),    # Quantidade solicitada
        'quantidade_atendida': fields.float('Quantidade atendida',                  # Quantidade Atendida
                                            digits=(15, 3)),

        # ___________________________ Text ___________________________________
        'aplicacao': fields.text('Aplicacao'),                                      # Finalidade da solicitacao. Onde o item será aplicado
        'observacao': fields.text('Observacao'),                                    # Informacoes complementares

        # ___________________________ Selection ______________________________
        'state': fields.selection(STATE, 'Estado da solicitacao', required=True,    # Estado da solicitacao
                                  size=1),
    }

    _defaults = {
        'quantidade': lambda *args: 1.00,                                           # Minimo = 1
        'state': lambda *args: 'A',                                                 # Aguardando autorizacao
    }

    _constraints = [(comsit_validar_quantidade,
                     'A quantidade do produto deve ser informada ou deve ser maior que zero.',
                     ['Quantidade'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
