# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import socket
import time
from SigUtils.formatastring import formataString
from openerp.tools.translate import _

LISTAGEM_LIMIT = 15
READ_LOAD = '_classic_write'


def _somar_encargos(dados_tributacao, comissoes):
    soma_valor_encargo = 0.0
    for x in dados_tributacao:
        if x.tipo_tributacao != 'T': # So interessa tributacao de saida(Todas/Cons.Final)
            continue
        aliquota = x.tributacao_id.aliquota
        base = x.tributacao_id.base
        soma_valor_encargo += base * aliquota / 100
    soma_valor_encargo += comissoes
    return soma_valor_encargo


class wizard_atualizar_preco_venda_capa(osv.TransientModel):
    _name = 'wizard.atualizar.preco.venda.capa'

    def _get_comprador(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_res_users = self.pool.get('res.users')
        obj_compras_comprador = self.pool.get('compras.comprador')
        res_users_dados = obj_res_users.read(cr, uid, uid, ['pessoa_id'], context=context, load=READ_LOAD)

        if res_users_dados and isinstance(res_users_dados, list):
            res_users_dados = res_users_dados[0]

        pessoa_id = res_users_dados.get('pessoa_id')
        if pessoa_id:
            comprador_ids = obj_compras_comprador.search(cr, uid, [('pessoa_id', '=', pessoa_id)], context=context)
            if comprador_ids:
                return comprador_ids[0]

        return False

    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade"),
        'comprador_id': fields.many2one('compras.comprador', 'Comprador'),
        'itens_ids': fields.one2many('wizard.atualizar.preco.venda.itens', 'capa_id', 'Itens'),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'comprador_id': _get_comprador,
    }

    def wizard_get_produtos(self, cr, produto_ids):
        dados = []
        if produto_ids:
            if len(produto_ids) == 1:
                produto_ids = str(tuple(produto_ids))
                produto_ids = produto_ids.replace(',)', ')')
            else:
                produto_ids = str(tuple(produto_ids))

            sql = """
                SELECT
                    produto.id AS produto_id,
                    produto.codigo,
                    produto.name,
                    produto.comprador_id,

                    (SELECT venda2.create_date  FROM   estoque_preco_venda venda2
                    WHERE  venda2.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as create_date_venda,

                    (SELECT custo2.create_date  FROM   estoque_preco_custo custo2
                    WHERE  custo2.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as create_date_custo,

                    (SELECT preco_venda FROM   estoque_preco_venda venda3
                    WHERE  venda3.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as preco_venda,

                    (SELECT estprc_custo_corrigido  FROM   estoque_preco_custo custo3
                    WHERE  custo3.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as custo_corrigido,

                    (SELECT estprc_custo_reposicao  FROM   estoque_preco_custo custo4
                    WHERE  custo4.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as custo_reposicao,

                    (SELECT estprc_ultimo_custo  FROM   estoque_preco_custo custo5
                    WHERE  custo5.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as ultimo_custo,

                    (SELECT estprc_custo_medio_contabil  FROM   estoque_preco_custo custo6
                    WHERE  custo6.produto_id = produto.id
                    ORDER  BY create_date DESC  LIMIT  1) as custo_medio_contabil,

                    SUM(
                        CASE WHEN trib_encargo.tipo_tributacao = 'T'
                            THEN trib.base * trib.aliquota
                            ELSE 0
                        END
                    ) / 100 as soma_valor_encargo

                FROM   estoque_produto produto
                    LEFT JOIN fiscal_tributacao_encargos trib_encargo ON
                        trib_encargo.produto_id = produto.id
                    LEFT JOIN fiscal_tributacao trib ON
                        trib_encargo.tributacao_id = trib.id

                WHERE  produto.id in %s
                GROUP BY
                       produto.id,
                       produto.codigo,
                       produto.name,
                       produto.comprador_id
                ORDER BY produto.codigo
            """ % produto_ids

            cr.execute(sql)
            dados = cr.dictfetchall()
        return dados

    def flag_atualizar_preco(self, cr, uid, ids, atualizar, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        obj_preco_itens = self.pool.get('wizard.atualizar.preco.venda.itens')
        preco_item_ids = obj_preco_itens.search(
            cr, uid, [('capa_id', '=', ids), ('atualizar_preco', '!=', atualizar)], context=context
        )
        if preco_item_ids:
            res = obj_preco_itens.write(cr, uid, preco_item_ids, {'atualizar_preco': atualizar}, context)
        else:
            res = False
        return res

    def button_marcar_todos(self, cr, uid, ids, context):
        return self.flag_atualizar_preco(cr, uid, ids, True, context)

    def button_desmarcar_todos(self, cr, uid, ids, context):
        return self.flag_atualizar_preco(cr, uid, ids, False, context)

    def calcular_dados_precificacao(self, cr, uid, dados_get_produto=None, produto_ids=None, return_percentuais=False,
                                    margem_atual_menor_margem_minima=False, limit=0, context=None):
        context = context and context.copy() or {}

        context = context and context.copy() or {}
        res = []

        if produto_ids:
            # SE OS PRODUTO_IDS FOREM INFORMADOS A FUNCAO EXECUTA O SQL DENTRO DA FUNCAO wizard_get_produtos
            # CASO PRODUTO_IDS NAO FOR INFORMADO,
            # O PARAMETRO dados_get_produto DEVE CONTER O RESULTADO DA FUNCAO wizard_get_produtos
            if isinstance(produto_ids, (int, long)):
                produto_ids = [produto_ids]
            dados_get_produto = self.wizard_get_produtos(cr, produto_ids)

        if dados_get_produto:
            # OBJETOS
            obj_adm_utils = self.pool.get('adm.utils')
            obj_estoque_utils = self.pool.get('estoque.utils')
            obj_estoque_produto = self.pool.get('estoque.produto')

            # MODULO CONCORRENTE INSTALADO
            concorrencia_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'concorrencia', context)
            if concorrencia_instalado:
                concorrente = True
            else:
                concorrente = False

            # DADOS DO PARAMETRO
            fields_to_read = ['estpar_custo_para_precovenda', 'comissoes', 'custo_fixo', 'estpar_margem_para_preco_venda']
            estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
                cr, uid, tabela='estoque', campos=fields_to_read, context=context
            )
            if not estoque_parametro_dados:
                raise osv.except_osv(_('Atencao'), _('Parametro do estoque nao cadastrado!'))
            # CONFIGURACOES DO PARAMETRO
            custo_para_precovenda = estoque_parametro_dados.get('estpar_custo_para_precovenda') or ''
            comissoes = estoque_parametro_dados.get('comissoes') or 0
            custo_fixo = estoque_parametro_dados.get('custo_fixo') or 0
            estpar_margem_para_precovenda = estoque_parametro_dados.get('estpar_margem_para_preco_venda')

            # FUNCOES E SEREM EXECUTADAS
            funct_get_margens = obj_estoque_produto.estpro_get_margens
            funct_calcular_margem = obj_estoque_utils.estutl_calcular_margem
            funct_calcular_preco_venda = obj_estoque_utils.estutl_calcular_preco_de_venda
            funct_get_custo_hoje = obj_estoque_utils.estutl_get_custo_hoje

            produto_ids = filter(None, map(lambda i: i.get('produto_id'), dados_get_produto))
            dict_produtos = funct_get_margens(cr, uid, produto_ids)

            cont = 0
            for i in dados_get_produto:

                produto_id = i.get('produto_id')
                produto_codigo = i.get('codigo')
                produto_name = i.get('name') or ''

                # CALCULA O PRECO CORRIGIDO PARA O DIA DO HOJE
                args_custo_corrigido = [{'produto_id': produto_id,
                                         'estprc_custo_corrigido': i.get('custo_corrigido')}]
                custo_corrigido = funct_get_custo_hoje(cr, uid, args_custo_corrigido, context)
                custo_corrigido = custo_corrigido[produto_id]['custo_corrigido']
                custo_reposicao = i.get('custo_reposicao') or 0.0
                custo_ultimo = i.get('ultimo_custo') or 0.0

                ultimo_preco_venda = i.get('preco_venda') or 0.0
                comprador_id = i.get('comprador_id') or False

                dict_produto_margens = dict_produtos.get(produto_id)
                del dict_produtos[produto_id]
                produto_margem_ideal = dict_produto_margens.get('margem_ideal') or 0
                produto_margem_minima = dict_produto_margens.get('margem_minima') or 0
                produto_margem_bruta = dict_produto_margens.get('margem_bruta') or 0

                if custo_para_precovenda == 'U':
                    preco_custo = custo_ultimo
                elif custo_para_precovenda == 'M':
                    preco_custo = i.get('custo_medio_contabil') or 0.0
                elif custo_para_precovenda == 'R':
                    preco_custo = i.get('custo_reposicao') or 0.0
                elif custo_para_precovenda == 'C':
                    preco_custo = custo_corrigido
                else:
                    preco_custo = 0

                if estpar_margem_para_precovenda == 'I':
                    percentuais = i.get('soma_valor_encargo')
                    percentuais += comissoes
                    percentuais += custo_fixo
                else:
                    percentuais = 0

                margem_atual = funct_calcular_margem(
                    i.get('preco_venda', 0.0), percentuais, preco_custo)
                novo_preco = funct_calcular_preco_venda(preco_custo, percentuais,
                                                        produto_margem_ideal)

                if margem_atual_menor_margem_minima:
                    if margem_atual > produto_margem_minima:
                        continue

                # CHAVES DO RETORNO
                # NAO DEVEM SER ALTERADAS POIS ESTAS CHAVES SAO USADAS PARA CRIAR A LISTAGEM DOS ITENS DO WIZARD
                vals = {
                    'codigo': produto_codigo,
                    'descricao': produto_name,
                    'margem_ideal': produto_margem_ideal,
                    'margem_minima': produto_margem_minima,
                    'margem_bruta': produto_margem_bruta,
                    'custo_corrigido': custo_corrigido,
                    'custo_reposicao': custo_reposicao,
                    'ultimo_preco_venda': ultimo_preco_venda,
                    'margem_atual': margem_atual,
                    'novo_preco': novo_preco,
                    'concorrente': concorrente,
                    'nova_margem_atual': produto_margem_ideal,
                    'comprador_id': comprador_id
                }
                if return_percentuais:
                    vals.update({
                        'percentuais': percentuais
                    })
                res.append(vals)
                cont += 1
                if limit and cont > limit:
                    break

        return res

    def buscar_itens(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]

        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_wiz_preco_itens = self.pool.get('wizard.atualizar.preco.venda.itens')

        dados = self.browse(cr, uid, ids)
        company_id = dados.company_id.id
        itens_ids = obj_wiz_preco_itens.search(cr, uid, [('capa_id', '=', ids)], context=context)
        if itens_ids:
            obj_wiz_preco_itens.unlink(cr, uid, itens_ids)

        create_ids = [dados.comprador_id.id]
        comprador_dados = dados.comprador_id
        if not comprador_dados:
            raise osv.except_osv(_('Atencao'), _('Nao existe comprador para o usuario logado'))

        comcpr_subordinados_ids = comprador_dados.comcpr_subordinados_ids
        for j in comcpr_subordinados_ids:
            create_ids.append(j.id)
        comcpr_responder_por_id = comprador_dados.comcpr_responder_por_id
        if comcpr_responder_por_id:
            create_ids.append(comcpr_responder_por_id.id)

        args = [('comprador_id', 'in', create_ids), ('company_id', '=', company_id)]

        produto_ids = obj_estoque_produto.search(cr, uid, args, context=context)
        dados = self.wizard_get_produtos(cr, produto_ids)

        if dados:
            new_dados = filter(
                lambda i: i.get('create_date_custo') > i.get('create_date_venda'), dados
            )[:LISTAGEM_LIMIT]
            if new_dados:
                res = self.calcular_dados_precificacao(
                    cr, uid, dados_get_produto=new_dados, context=context)
                for vals in res:
                    vals.update({'capa_id': ids})
                    obj_wiz_preco_itens.create(cr, uid, vals, context)

        return True

    def buscar_itens_abaixo_margem(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]

        if isinstance(ids, list):
            ids = ids[0]

        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_wiz_preco_itens = self.pool.get('wizard.atualizar.preco.venda.itens')

        itens_ids = obj_wiz_preco_itens.search(cr, uid, [('capa_id', '=', ids)], context=context)
        if itens_ids:
            obj_wiz_preco_itens.unlink(cr, uid, itens_ids)

        browse_wiz_capa = self.browse(cr, uid, ids)
        company_id = browse_wiz_capa.company_id.id

        args = [('company_id', '=', company_id)]
        create_ids = [browse_wiz_capa.comprador_id.id]
        record_comprador_id = browse_wiz_capa.comprador_id
        if not record_comprador_id:
            raise osv.except_osv(_('Atencao'), _('Nao existe comprador para o usuario logado'))

        comcpr_subordinados_ids = record_comprador_id.comcpr_subordinados_ids
        for j in comcpr_subordinados_ids:
            create_ids.append(j.id)
        comcpr_responder_por_id = record_comprador_id.comcpr_responder_por_id.id
        if comcpr_responder_por_id:
            create_ids.append(comcpr_responder_por_id)
        args.append(('comprador_id', 'in', list(set(create_ids))))

        produto_ids = obj_estoque_produto.search(cr, uid, args, order='codigo', context=context)
        if produto_ids:
            res = self.calcular_dados_precificacao(
                cr, uid,
                produto_ids=produto_ids,
                margem_atual_menor_margem_minima=True,
                limit=LISTAGEM_LIMIT - 1,
                context=context
            )
            if res:
                for vals in res[:LISTAGEM_LIMIT]:
                    vals.update({
                        'capa_id': ids,
                        'permitir_alterar': True,
                    })
                    obj_wiz_preco_itens.create(cr, uid, vals, context)
        return True

    def alterar_precos_no_gestao(self, admpar_siggestao_servidor, admpar_siggestao_diretorio,
                                 admpar_siggestao_pv_exportar, dados, user):
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        address = ((admpar_siggestao_servidor, 5019))
        try:
            client_socket.connect(address)
        except Exception as e:
            raise osv.except_osv(_('Atencao'), _('Erro na conexao com o servidor:') + str(
                admpar_siggestao_servidor) + '\nMensagem:' + str(e))
        client_socket.send('ESTA AI?')
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), (mensagem))
        client_socket.send('DIR:' + admpar_siggestao_diretorio)
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), _("Diretorio '%s' nao encontrado" % admpar_siggestao_diretorio))
        c = admpar_siggestao_pv_exportar
        _arquivoCMD = c[c.find('{') + 1:c.find('}')].strip()
        if not _arquivoCMD:
            a = str(time.time())
            n = a.find('.')
            _arquivoCMD = 'pv' + a[n - 5:n] + a[n + 1:n + 2] + '.cmd'
        admpar_siggestao_pv_exportar = c[:c.find('{')] + _arquivoCMD + c[c.find('}') + 1:]
        client_socket.send('ARQ NOME:' + admpar_siggestao_diretorio + '/' + _arquivoCMD)
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), (mensagem))
        arquivo = []
        for i in dados.itens_ids:
            if i.novo_preco > 0 and i.atualizar_preco:
                linha = 'PV'
                linha += formataString(i.codigo, 10, '0', 'E')
                linha += '     '
                valor = "%.2f" % i.novo_preco
                linha += formataString(str(valor).replace('.', '').replace(',', ''), 12, '0', 'E')
                linha += str(user).rjust(10, '0')
                arquivo.append(linha)
        for linha in arquivo:
            comando = 'ARQ DADOS:' + linha
            client_socket.send(comando)
            mensagem = client_socket.recv(2048)
            if not mensagem.startswith('OK'):
                raise osv.except_osv(_('Atencao'), (mensagem))
        client_socket.send('ARQ FECHAR:')
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), (mensagem))
        comando = 'EXECUTAR:' + admpar_siggestao_pv_exportar
        client_socket.send(comando)
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            if mensagem.startswith('ERRO!'):
                mensagem = _('Sem permissao para execucao')
            raise osv.except_osv(_('Atencao'), (mensagem))
        return True

    def alterar_precos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_adm_parametro = self.pool.get('adm.parametro')
        obj_wizard_atualizar_preco_venda_itens = self.pool.get('wizard.atualizar.preco.venda.itens')
        adm_parametro_id = obj_adm_parametro.search(
            cr, uid, [('company_id', '=', self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context))]
        )
        dados = self.browse(cr, uid, ids)

        record_item_ids = filter(lambda i: i.atualizar_preco, dados.itens_ids)
        if not record_item_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum item encontra-se marcado para atualizar o preco!'))

        company_id = dados.company_id.id
        fields_to_read = ['admpar_siggestao_servidor', 'admpar_siggestao_diretorio', 'admpar_siggestao_pv_exportar']
        adm_parametros = obj_adm_parametro.read(cr, uid, adm_parametro_id, fields_to_read, context=context)
        for x in adm_parametros:
            admpar_siggestao_servidor = x.get('admpar_siggestao_servidor')
            admpar_siggestao_diretorio = x.get('admpar_siggestao_diretorio')
            admpar_siggestao_pv_exportar = x.get('admpar_siggestao_pv_exportar')
        if admpar_siggestao_pv_exportar:
            self.alterar_precos_no_gestao(admpar_siggestao_servidor,
                                          admpar_siggestao_diretorio,
                                          admpar_siggestao_pv_exportar,
                                          dados, user=uid)
        else:
            for i in dados.itens_ids:
                if i.permitir_alterar and i.novo_preco > 0 and i.atualizar_preco:
                    produto_id = obj_estoque_produto.search(cr, uid, [('codigo', '=', i.codigo)], context=context)
                    if produto_id and isinstance(produto_id, list):
                        produto_id = produto_id[0]
                        obj_estoque_preco_venda.create(
                            cr, uid,
                            {
                                'company_id': company_id,
                                'produto_id': produto_id,
                                'preco_venda': i.novo_preco
                            }, context
                        )
        itens_ids = obj_wizard_atualizar_preco_venda_itens.search(cr, uid, [('capa_id', '=', ids)], context=context)
        obj_wizard_atualizar_preco_venda_itens.unlink(cr, uid, itens_ids)
        return True


class wizard_atualizar_preco_venda_itens(osv.TransientModel):
    _name = 'wizard.atualizar.preco.venda.itens'

    _columns = {
        'capa_id': fields.many2one('wizard.atualizar.preco.venda.capa', 'Capa'),
        'comprador_id': fields.many2one('compras.comprador', 'Comprador'),
        'codigo': fields.char('Codigo', size=20),
        'descricao': fields.char('Descricao', size=60),
        'custo_corrigido': fields.float('Custo Corrigido', digits=(10, 2)),
        'custo_reposicao': fields.float('Custo Reposicao', digits=(10, 2)),
        'ultimo_preco_venda': fields.float('Ultimo Preco Venda', digits=(10, 2)),
        'margem_atual': fields.float('Margem Atual', digits=(10, 2)),
        'margem_ideal': fields.float('Margem Ideal', digits=(10, 2)),
        'margem_minima': fields.float('Margem Minima', digits=(10, 2)),
        'margem_bruta': fields.float('Margem Bruta', digits=(10, 2)),
        'novo_preco': fields.float('Novo Preco', digits=(10, 2)),
        'nova_margem_atual': fields.float('Nova margem atual', digits=(10, 2)),
        'permitir_alterar': fields.boolean('Permitir alterar'),
        'concorrente': fields.boolean('Concorrente'),
        'atualizar_preco': fields.boolean('Atualizar preco'),
    }

    _defaults = {
        'permitir_alterar': True,
        'concorrente': False,
    }

    count = 0
    count1 = 0
    lista = []

    def onchange_nova_margem(self, cr, uid, ids, nova_margem_atual, context=None):
        context = context and context.copy() or {}
        res = {}
        if not nova_margem_atual:
            return res
        self.lista.append(1)
        self.count += 1
        if self.count1:
            if len(self.lista) == 3:
                self.lista = []
                self.count1 = 0
            self.count = 0
            return res

        brw_preco_venda_item = self.browse(cr, uid, ids[0])
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_utils = self.pool.get('estoque.utils')
        produto_id = obj_estoque_produto.search(cr, uid, [('codigo', '=',
                                                           brw_preco_venda_item.codigo)], context=context)
        produto_id = produto_id[0]
        company_id = brw_preco_venda_item.capa_id.company_id.id
        fields_to_read = ['estpar_custo_para_precovenda']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, tabela='estoque', campos=fields_to_read, context=context
        )
        if not estoque_parametro_dados:
            raise osv.except_osv(_('Atencao'), _('Parametro do estoque nao cadastrado!'))
        custo_para_precovenda = estoque_parametro_dados.get('estpar_custo_para_precovenda') or ''
        custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id, company_id)
        custo = custo.get(produto_id)
        if custo_para_precovenda == 'U':
            preco_custo = custo.get('estprc_ultimo_custo') or 0.0
        elif custo_para_precovenda == 'M':
            preco_custo = custo.get('estprc_custo_medio_contabil') or 0.0
        elif custo_para_precovenda == 'R':
            preco_custo = custo.get('estprc_custo_reposicao') or 0.0
        elif custo_para_precovenda == 'C':
            preco_custo = custo.get('estprc_custo_corrigido') or 0.0
        else:
            preco_custo = 0
        percentual_total_encargos = obj_estoque_utils.fisutl_calcular_porcentagem_encargos_produto(
            cr, uid, produto_id, context)
        preco_venda = obj_estoque_utils.estutl_calcular_preco_de_venda(
            preco_custo, percentual_total_encargos, nova_margem_atual
        )
        res['novo_preco'] = preco_venda
        res['atualizar_preco'] = True
        return {'value': res}

    def onchange_novo_preco(self, cr, uid, ids, novo_preco, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        if not novo_preco > 0:
            raise osv.except_osv(_('Atencao'), _('Preco invalido.'))

        val = {}
        warning = {}

        self.count1 += 1
        self.lista.append(1)
        if self.count:
            if len(self.lista) == 3:
                self.lista = []
                self.count = 0
            self.count1 = 0
            return val

        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_ir_model_access = self.pool.get('ir.model.access')
        obj_preco_venda_capa = self.pool.get('wizard.atualizar.preco.venda.capa')

        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, tabela='estoque', campos=['estpar_pvenda_menor_pcusto'], context=context
        )

        preco_venda_menor_preco_custo = estoque_parametro_dados.get('estpar_pvenda_menor_pcusto')  # BOOLEAN

        access_group_pv_menor_margem_minima = obj_ir_model_access.check_groups(
            cr, uid, 'estoque.group_estoque_pv_menor_margem_minima'
        )

        browse_preco_capa = self.browse(cr, uid, ids)
        for i in browse_preco_capa:

            produto_codigo = i.codigo
            produto_ids = obj_estoque_produto.search(cr, uid, [('codigo', '=', produto_codigo)], context=context)
            if produto_ids:
                produto_id = produto_ids[0]
            else:
                produto_id = False

            if produto_id:

                list_vals = obj_preco_venda_capa.calcular_dados_precificacao(
                    cr, uid,
                    produto_ids=produto_id,
                    return_percentuais=True,
                    context=context,
                )

                if list_vals:

                    margem_minima = list_vals[0]['margem_minima']
                    margem_ideal = list_vals[0]['margem_ideal']

                    percentuais = list_vals[0]['percentuais']

                    preco_custo = self.calcula_preco_de_custo(
                        i.novo_preco,
                        percentuais,
                        margem_ideal
                    )

                    margem_atual = self.pool.get('estoque.utils').estutl_calcular_margem(
                        novo_preco,
                        percentuais,
                        preco_custo
                    )

                    if margem_atual < margem_minima and not access_group_pv_menor_margem_minima:
                        warning['title'] = _('Atencao')
                        warning['message'] = _('Margem atual nao pode ser menor que margem minima')
                        val['novo_preco'] = i.novo_preco

                    elif not preco_venda_menor_preco_custo and novo_preco < preco_custo:
                        warning['title'] = _('Atencao')
                        warning['message'] = _('Preco de venda nao pode ser menor que preco custo')
                        val['novo_preco'] = i.novo_preco
                        val['nova_margem_atual'] = i.margem_ideal
                    else:
                        val['novo_preco'] = novo_preco
                        val['nova_margem_atual'] = margem_atual
                        val['atualizar_preco'] = True

        return {'value': val, 'warning': warning}

    def calcula_preco_de_custo(self, pv, perEncargos, mg):
        """
        Calcular Preco de custo

        @param pv: Preco de venda
        @param perEncargos: Percentual de encargos incidentes no preco de venda
        @param mg: Percentual de margem a ser aplicada para calculo do custo

        @return: preco calculado, se nao conseguir calcular retorna 0
        """
        pc = 0
        if (perEncargos + mg) != 100:
            pc = (pv * (100 - perEncargos - mg)) / 100
        return pc

    def mostrar_concorrentes(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_concorrencia_concorrente_item = self.pool.get('concorrencia.concorrente.item')
        if isinstance(ids, list):
            ids = ids[0]
        codigo = self.browse(cr, uid, ids).codigo
        produto_id = obj_estoque_produto.search(cr, uid, [('codigo', '=', codigo)], context=context)
        if not produto_id:
            raise osv.except_osv(_('Atencao'), _('Produto nao encontrado'))
        elif isinstance(produto_id, list):
            produto_id = produto_id[0]
        concorrente_id = obj_concorrencia_concorrente_item.search(cr, uid, [('produto_id', '=', produto_id)],
                                                                  context=context)
        if not concorrente_id:
            raise osv.except_osv(_('Atencao'), _('Nao existe concorrente para este produto'))
        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'concorrencia',
                                                        'view_concorrencia_concorrente_item_tree')
        view_id = reference and reference[1] or False
        return {
            'name': _('Concorrente'),
            'view_type': 'form',
            'view_mode': 'tree',
            'view_id': [view_id],
            'res_model': 'concorrencia.concorrente.item',
            'res_id': False,
            'type': 'ir.actions.act_window',
            'target': 'new',
            'domain': [('produto_id', '=', produto_id)],
        }

    def button_simular_preco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_produto_osv = self.pool.get('estoque.produto.osv')
        obj_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        if isinstance(ids, list):
            ids = ids[0]
        codigo = self.browse(cr, uid, ids).codigo
        produto_id = obj_estoque_produto.search(cr, uid, [('codigo', '=', codigo)], context=context)
        if not produto_id:
            raise osv.except_osv(_('Atencao'), _('Produto nao encontrado'))
        elif isinstance(produto_id, list):
            produto_id = produto_id[0]
        grade_id = obj_grade.search(cr, uid, [('produto_id', '=', produto_id)], context=context)
        if grade_id:
            grade_id = grade_id[0]
        else:
            raise osv.except_osv(_('Atencao'), _('Nao ha grade cadastrada para este produto'))
        produto_embalagem_id = obj_estoque_produto_embalagem.search(cr, uid, [('produto_id', '=', produto_id)],
                                                                    context=context)
        if produto_embalagem_id:
            embalagem_id = produto_embalagem_id[0]
        else:
            raise osv.except_osv(_('Atencao'), _('Nao ha embalagem cadastrada para este produto'))
        company_id = obj_estoque_produto.read(cr, uid, produto_id, ['company_id'], context=context)
        if company_id:
            company_id = company_id[0]['company_id'][0]
            if company_id:
                company_id = company_id

        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, tabela='estoque', campos=['estpar_custo_para_precovenda'], context=context)
        custo_preco_venda = estoque_parametro_dados.get('estpar_custo_para_precovenda')
        val = obj_estoque_produto_osv.dados_simulacao_get(
            cr, uid, produto_id, grade_id, embalagem_id, custo_preco_venda, {'tipo': 'V'})
        vals = {'produto_id': produto_id, 'grade_id': grade_id, 'embalagem_id': embalagem_id,
                'company_id': company_id, 'marg_minima': val.get('marg_minima'),
                'marg_bruta': val.get('marg_bruta'), 'marg_ideal': val.get('marg_ideal'),
                'margem_contrib': val.get('margem_contrib'),
                'margem_praticada': val.get('margem_praticada'), 'saldo': val.get('saldo'),
                'ultimo_custo': val.get('ultimo_custo'), 'custo_medio': val.get('custo_medio'),
                'custo_corrigido': val.get('custo_corrigido'), 'preco_de_vendass': val.get('preco'),
                'juros': val.get('juros'), 'comissoes': val.get('comissoes'),
                'custo_fixo': val.get('custo_fixo'), 'ultima_compra': val.get('ultima_compra'),
                'selecionar_custo': custo_preco_venda}
        produto_osv_id = obj_estoque_produto_osv.create(cr, uid, vals, context)
        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(
            cr, uid, 'fiscal', 'view_estoque_produto_osv_form')
        view_id = reference and reference[1] or False
        result = {'name': _('Simulacao de preco de venda'), 'view_type': 'form', 'target': 'current',
                  'view_mode': 'form', 'view_id': [view_id], 'res_model': 'estoque.produto.osv',
                  'res_id': produto_osv_id, 'type': 'ir.actions.act_window'}

        return result

    def atualizar_novo_preco_por_ultimo_preco_venda(self, cr, uid, atualizar_preco_venda_item_id,
                                                    context=None):
        """
        Atualiza o novo preco do item baseado no ultimo preco de venda
        @param atualizar_preco_venda_item_id: Numero identificador do registro
        @return: True se novo_preco for atualizado, senao False
        """
        context = context and context.copy() or {}
        brw_wizard_atualizar_preco_venda_itens = self.browse(cr, uid,
                                                             atualizar_preco_venda_item_id[0])
        ultimo_preco_venda = brw_wizard_atualizar_preco_venda_itens.ultimo_preco_venda or 0
        vals = {'novo_preco': ultimo_preco_venda, 'atualizar_preco': True}
        res = self.write(cr, uid, atualizar_preco_venda_item_id, vals, context)
        return res


