# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.osv.fields import float_round
from funcoes import wancoi_get_valor_total


class wizard_compras_analisa_cotacao(osv.TransientModel):

    _name = 'wizard.compras.analisa.cotacao'

    _columns = {
        'wancot_requisicao_id': fields.many2one('compras.requisicao.capa', 'Requisicao',
                                                required=True, domain="[('state', '=', 'A')]"),
        'wancot_fornecedor_dados': fields.text('Dados fornecedores'),
        'wancot_item_ids': fields.one2many('wizard.compras.analisa.cotacao.itens', 'wancoi_capa_id',
                                           'Itens'),
        'state': fields.selection([('M', 'Montada'), ('C', 'Concluida')], 'Status', size=1),
        'wancot_fornecedor_id': fields.many2one("financeiro.fornecedor", 'Fornecedor'),
        'wancot_nao_confirmados': fields.boolean('Itens nao confirmados'),
        'wancot_confirmados': fields.boolean('Itens confirmados'),
    }
    _defaults = {
        'wancot_nao_confirmados': False,
        'wancot_confirmados': False
    }

    def wancot_montar_analise(self, cr, uid, ids, context=None):
        """
        O metodo busca os dados dos itens da requisicao selecionada.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: acao com chamada para o mesmo formulario, para atualizacao dos fornecedores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        for analise in self.browse(cr, uid, ids, context):
            cr.execute("""DELETE FROM wizard_compras_analisa_cotacao_itens
                          WHERE wancoi_capa_id = %s""" % analise.id)
            items_ids = list()
            list_fornecedor = list()
            for item in analise.wancot_requisicao_id.requisicao_itens_ids:
                vals = {
                    'wancoi_produto_id': item.produto_id.id,
                    'wancoi_grade_id': item.grade_id.id,
                    'wancoi_embalagem_id': item.comrit_produto_embalagem_id.id,
                    'wancoi_quantidade': item.quantidade or 0,
                    'wancoi_fornecedor_escolhido': item.fornecedor or '',
                    'wancoi_preco_escolhido': float_round((item.vr_tot_cotacao / item.quantidade),
                                                          2),
                    'wancoi_requisicao_item_id': item.id,
                    'wancoi_cotacao_item_id': item.cotacao_item_id.id,
                }
                cotacao_itens = sorted(item.cotacao_itens_ids, key=lambda c: c.vr_unitario)
                # Se nao possuir cotacao autorizada, busca a de menor valor
                if not vals.get('wancoi_fornecedor_escolhido'):
                    vals.update({'wancoi_fornecedor_escolhido': cotacao_itens and cotacao_itens[0].fornecedor or '',
                                 'wancoi_preco_escolhido': cotacao_itens and cotacao_itens[0].vr_unitario or 0,
                                 'wancoi_cotacao_item_id': cotacao_itens and cotacao_itens[0].id})
                list_fornecedor_item = list()
                fornecedores = list()
                for item_cotacao in cotacao_itens:
                    field_fornecedor = item_cotacao.fornecedor.lower().replace(' ', '_')
                    if field_fornecedor in fornecedores:
                        continue
                    fornecedores.append(field_fornecedor)
                    list_fornecedor_item.append({
                        'wancoi_' + field_fornecedor:
                            item_cotacao.vr_unitario or 0,
                        'requisicao_item_id': item.id,
                        'cotacao_item_id': item_cotacao.id,
                        'nome': item_cotacao.fornecedor or ''
                        })
                vals.update({'wancoi_fornecedor_dados': str(list_fornecedor_item)})
                items_ids.append((0, 0, vals))
                list_fornecedor.extend(map(lambda i: i.fornecedor,
                                           cotacao_itens))
            list_fornecedor = list(set(list_fornecedor))
            self.write(
                cr, uid, analise.id,
                {'wancot_item_ids': items_ids, 'wancot_fornecedor_dados': str(list_fornecedor),
                 'state': 'M'},
                context)
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'compras', 'view_wizard_compras_analisa_cotacao_form')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Analisar cotacoes',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.compras.analisa.cotacao',
            'view_id': view_id,
            'target': 'inline',
            'type': 'ir.actions.act_window',
        }

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        res = super(wizard_compras_analisa_cotacao, self).fields_view_get(
            cr, uid, view_id, view_type, context, toolbar, submenu)
        if context.get('active_id') and context.get('active_model') == \
                'wizard.compras.analisa.cotacao':
            fornecedor_dados = eval(self.browse(
                cr, uid, context.get('active_id'), context).wancot_fornecedor_dados)
            str_fornecedores = str()
            for fornecedor in fornecedor_dados:
                nome_campo = 'wancoi_' + fornecedor.lower().replace(' ', '_')
                res['fields']['wancot_item_ids']['views']['tree']['fields'].update(
                    {nome_campo: {'selectable': True, 'type': 'float', 'string': fornecedor,
                                  'digits': (11, 2)}})

                nome_campo_bool = 'wancoi_' + fornecedor.lower().replace(' ', '_') + '_bool'
                res['fields']['wancot_item_ids']['views']['tree']['fields'].update(
                    {nome_campo_bool: {'selectable': True, 'type': 'boolean', 'string': ''}})
                str_fornecedores += """
                    <field name="%s" readonly="True" modifiers="{&quot;readonly&quot;: true}"/>\n
                    <field name="%s" modifiers="{}" context="{'fornecedor': '%s'}"
                           on_change="wancoi_onchange_preco_fornecedor('%s')"/>\n
                """ % (nome_campo, nome_campo_bool, nome_campo, nome_campo)
            if str_fornecedores:
                res['fields']['wancot_item_ids']['views']['tree']['arch'] = res.get('fields').get(
                    'wancot_item_ids').get('views').get('tree').get('arch').replace(
                    """<field name="wancoi_fornecedor_escolhido" readonly="True" modifiers="{&quot;readonly&quot;: true}"/>\n""",
                    """<field name="wancoi_fornecedor_escolhido" readonly="True" modifiers="{&quot;readonly&quot;: true}"/>\n""" +
                    str_fornecedores
                )
        return res

    def wancot_button_concluir_analise(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_compras_cotacoes_itens = self.pool.get('compras.cotacoes.itens')
        obj_analisa_cotacao_itens = self.pool.get('wizard.compras.analisa.cotacao.itens')
        for analise in self.browse(cr, uid, ids, context):
            brw_item_ids = filter(lambda i: not i.wancoi_escolhido, analise.wancot_item_ids)
            for item in brw_item_ids:
                obj_compras_cotacoes_itens.button_vencedor(
                    cr, uid, [item.wancoi_cotacao_item_id.id], context)
            obj_analisa_cotacao_itens.write(cr, uid, map(lambda i: i.id, brw_item_ids),
                                            {'wancoi_escolhido': True}, context)
        self.write(cr, uid, ids, {'state': 'C'}, context)
        return True

    def wancot_onchange_fornecedor_escolhido(self, cr, uid, ids, fornecedor_id, item_ids,
                                             nao_confirmados, context=None):
        context = context and context.copy() or {}
        res = {}
        if not ids:
            return res
        brw_item_ids = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                   context).wancot_item_ids
        if not fornecedor_id:
            if nao_confirmados:
                brw_item_ids = filter(lambda i: not i.wancoi_escolhido, brw_item_ids)
            res['value'] = {'wancot_item_ids': map(lambda i: i.id, brw_item_ids)}
            return res
        fornecedor = self.pool.get('financeiro.fornecedor').browse(
            cr, uid, fornecedor_id, context).pessoa_id.nome
        item_ids = map(lambda i: i[1], item_ids)
        if not item_ids:
            item_ids = map(lambda i: i.id, brw_item_ids)
        args = [('id', 'in', item_ids), ('wancoi_fornecedor_escolhido', '=', fornecedor)]
        if nao_confirmados:
            args.append(('wancoi_escolhido', '=', 'False'))
        item_ids = self.pool.get('wizard.compras.analisa.cotacao.itens').search(
            cr, uid, args, context=context)
        res['value'] = {'wancot_item_ids': item_ids}
        return res

    def wancot_onchange_itens_nao_confirmados(self, cr, uid, ids, nao_confirmados, item_ids,
                                              fornecedor_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if not ids:
            return res
        if nao_confirmados is False:
            brw_item_ids = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                       context).wancot_item_ids
            if fornecedor_id:
                fornecedor = self.pool.get('financeiro.fornecedor').browse(
                    cr, uid, fornecedor_id, context).pessoa_id.nome
                brw_item_ids = filter(lambda i: i.wancoi_fornecedor_escolhido == fornecedor,
                                      brw_item_ids)
            res['value'] = {'wancot_item_ids': map(lambda i: i.id, brw_item_ids)}
            return res
        item_ids = map(lambda i: i[1], item_ids)
        args = [('id', 'in', item_ids), ('wancoi_escolhido', '=', False)]
        if fornecedor_id:
            fornecedor = self.pool.get('financeiro.fornecedor').browse(
                cr, uid, fornecedor_id, context).pessoa_id.nome
            args.append(('wancoi_fornecedor_escolhido', '=', fornecedor))
        item_ids = self.pool.get('wizard.compras.analisa.cotacao.itens').search(
            cr, uid, args, context=context)
        res['value'] = {'wancot_item_ids': item_ids}
        return res

    def wancot_onchange_itens_confirmados(self, cr, uid, ids, confirmados, item_ids,
                                          fornecedor_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if not ids:
            return res
        if confirmados is False:
            brw_item_ids = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                       context).wancot_item_ids
            if fornecedor_id:
                fornecedor = self.pool.get('financeiro.fornecedor').browse(
                    cr, uid, fornecedor_id, context).pessoa_id.nome
                brw_item_ids = filter(lambda i: i.wancoi_fornecedor_escolhido == fornecedor,
                                      brw_item_ids)
            res['value'] = {'wancot_item_ids': map(lambda i: i.id, brw_item_ids)}
            return res
        item_ids = map(lambda i: i[1], item_ids)
        args = [('id', 'in', item_ids), ('wancoi_escolhido', '=', True)]
        if fornecedor_id:
            fornecedor = self.pool.get('financeiro.fornecedor').browse(
                cr, uid, fornecedor_id, context).pessoa_id.nome
            args.append(('wancoi_fornecedor_escolhido', '=', fornecedor))
        item_ids = self.pool.get('wizard.compras.analisa.cotacao.itens').search(
            cr, uid, args, context=context)
        res['value'] = {'wancot_item_ids': item_ids}
        return res

    def wancot_button_confirma_fornecedor(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_compras_cotacoes_itens = self.pool.get('compras.cotacoes.itens')
        obj_analisa_cotacao_itens = self.pool.get('wizard.compras.analisa.cotacao.itens')
        for analise in self.browse(cr, uid, ids, context):
            fornecedor = analise.wancot_fornecedor_id.pessoa_id.nome
            brw_item_ids = filter(lambda i: not i.wancoi_escolhido and
                                  fornecedor == i.wancoi_fornecedor_escolhido,
                                  analise.wancot_item_ids)
            for item in brw_item_ids:
                obj_compras_cotacoes_itens.button_vencedor(
                    cr, uid, [item.wancoi_cotacao_item_id.id], context)
            obj_analisa_cotacao_itens.write(cr, uid, map(lambda i: i.id, brw_item_ids),
                                            {'wancoi_escolhido': True}, context)
        return True


class wizard_compras_analisa_cotacao_itens(osv.TransientModel):

    _name = 'wizard.compras.analisa.cotacao.itens'

    _columns = {
        'wancoi_capa_id': fields.many2one('wizard.compras.analisa.cotacao', 'Capa'),
        'wancoi_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'wancoi_grade_id': fields.many2one('estoque.produto.grade', 'Grade'),
        'wancoi_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem'),
        'wancoi_quantidade': fields.float('Quantidade', digits=(9, 3)),
        'wancoi_fornecedor_escolhido': fields.char('Fornecedor escolhido'),
        'wancoi_preco_escolhido': fields.float('Preco escolhido', digits=(11, 2)),
        'wancoi_requisicao_item_id': fields.many2one('compras.requisicao.itens', 'Item requisicao'),
        'wancoi_cotacao_item_id': fields.many2one('compras.cotacoes.itens', 'Cotacao vencedora'),
        'wancoi_fornecedor_dados': fields.text('Dados fornecedores'),
        'wancoi_escolhido': fields.boolean('Escolhido'),
        'wancoi_valor_total': fields.function(wancoi_get_valor_total, method=True,
                                              string='Vr. Total', type='float', digits=(11, 2)),
    }
    _defaults = {
        'wancoi_escolhido': False,
    }

    def fields_get(self, cr, uid, allfields=None, context=None, write_access=True):
        context = context and context.copy() or {}
        res = super(wizard_compras_analisa_cotacao_itens, self).fields_get(
            cr, uid, allfields, context, write_access)
        if context.get('active_id') and context.get('active_model') == \
                'wizard.compras.analisa.cotacao':
            fornecedor_dados = eval(self.pool.get('wizard.compras.analisa.cotacao').browse(
                cr, uid, context.get('active_id'), context).wancot_fornecedor_dados)
            for fornecedor in fornecedor_dados:
                res.update({'wancoi_' + fornecedor.lower().replace(' ', '_'): {
                    'selectable': True, 'type': 'float', 'string': fornecedor,
                    'digits': (11, 2)}})

                res.update({'wancoi_' + fornecedor.lower().replace(' ', '_') + '_bool': {
                    'selectable': True, 'type': 'boolean', 'string': fornecedor}})
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        if 'wancoi_fornecedor_dados' not in fields:
            fields.append('wancoi_fornecedor_dados')
        res = super(wizard_compras_analisa_cotacao_itens, self).read(cr, uid, ids, fields, context,
                                                                     load)
        for item in res:
            for fornecedor in eval(item.get('wancoi_fornecedor_dados')):
                list_fornecedor = filter(lambda x: x.startswith('wancoi_'), fornecedor)
                for x in list_fornecedor:
                    item.update({x: fornecedor.get(x)})
        return res

    def wancoi_onchange_preco_fornecedor(self, cr, uid, ids, fornecedor, context=None):
        context = context and context.copy() or {}
        res = {}
        brw_item = self.browse(cr, uid, ids[0], context)
        fornecedores_dados = eval(brw_item.wancoi_fornecedor_dados)
        vals = dict()
        for f in fornecedores_dados:
            list_fornecedor = filter(lambda x: x.startswith('wancoi_') and x != fornecedor, f)
            for x in list_fornecedor:
                vals.update({x+'_bool': False})
        fornecedor_dados = filter(lambda i: i.get(fornecedor), fornecedores_dados)
        fornecedor_dados = fornecedor_dados[0] if fornecedor_dados else {}
        if fornecedor_dados:
            vals.update({
                'wancoi_preco_escolhido': fornecedor_dados.get(fornecedor) or 0.0,
                'wancoi_cotacao_item_id': fornecedor_dados.get('cotacao_item_id') or False,
                'wancoi_fornecedor_escolhido': fornecedor_dados.get('nome') or ''})
        else:
            vals.update({fornecedor + '_bool': False})
        res.update({'value': vals})
        return res

    def wancoi_button_concluir_analise_item(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_compras_cotacoes_itens = self.pool.get('compras.cotacoes.itens')
        for item in self.browse(cr, uid, ids, context):
            obj_compras_cotacoes_itens.button_vencedor(
                cr, uid, [item.wancoi_cotacao_item_id.id], context)
        self.write(cr, uid, ids, {'wancoi_escolhido': True}, context)
        return True

    def wancoi_onchange_preco_escolhido(self, cr, uid, ids, produto_embalagem_id, quantidade,
                                        preco_escolhido, context=None):
        context = context and context.copy() or {}
        res = {}
        if not all([produto_embalagem_id, quantidade, preco_escolhido]):
            return res
        quantidade_na_embalagem = self.pool.get('estoque.produto.embalagem').browse(
            cr, uid, produto_embalagem_id, context).embalagem_id.quantidade_na_embalagem or 0
        res.update({'value': {
            'wancoi_valor_total': preco_escolhido * quantidade * quantidade_na_embalagem}})
        return res
