# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from SigUtils.datas import calcVencimentos
import time
from dateutil.parser import parse


class wizard_estsug_compra_gerar_pedido(osv.TransientModel):

    _name = 'wizard.estsug.compra.gerar.pedido'
    _columns = {
        'sugestao_compra_id': fields.many2one('estsug.compra.capa', 'Sugestao compra',
                                              required=True),
        'pessoa_id': fields.many2one('adm.pessoa', 'Fornecedor', required=True),
        'data_entrega': fields.date('Data de entrega'),
        'prazo_tipo': fields.selection([('DD', 'Dias Direto'), ('DFS', 'Dias Fora a Semana'),
                                        ('DFQ', 'Dias fora a quinzena'),
                                        ('DFM', 'Dias Fora o Mes')], 'Tipo de prazo', size=3,
                                       required=True),
        'prazo': fields.char("Prazo", size=60,
                             help="Prazo em dias, separado por VIRGULA(,) ou BARRA(/)",
                             required=True),
        'vr_frete': fields.float('Valor do Frete', digits=(14, 2)),
        'prazo_datas': fields.text('Datas do prazo', help='Datas dos vencimentos das parcelas do '
                                                          'pedido. Devem ser separadas por '
                                                          'VIRGULA(,)'),
    }

    def default_get(self, cr, uid, fields_list, context=None):
        """
        O metodo atribui valor padrao para os atributos da classe.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista com a descricao dos campos.
        :type fields_list: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dados com a descricao dos campos e seu respectivo valor.
        """
        context = context and context.copy() or {}
        res = super(wizard_estsug_compra_gerar_pedido, self).default_get(cr, uid, fields_list,
                                                                         context)
        res['sugestao_compra_id'] = context.get('active_id')
        return res

    def onchange_prazo(self, cr, uid, ids, prazo, prazo_tipo, context=None):
        """
        Metodo chamado ao incluir dados no campo 'prazo'. Calcula as datas de vencimento de acordo
        com o tipo e prazo recebidos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: registro da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de registros da tabela desse objeto.
        :type ids: list
        :param prazo: valores que respresentam o numero de dias entre os vencimentos das parcelas.
        :type prazo: str
        :param prazo_tipo: Valores que representam a forma como os prazos serao calculados.
        :rtype: str
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados da descriçao do campo com seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {'value': {'prazo_datas': ''}}
        if not prazo or not prazo_tipo:
            return res
        prazo = prazo.replace(' ', '').replace('/', ',')
        list_prazos = prazo.split(',')
        list_vencimentos = calcVencimentos(data=time.strftime('%Y-%m-%d'), listaPrazos=list_prazos,
                                           tipoPrazo=prazo_tipo)
        if not list_vencimentos:
            return res
        str_vencimentos = str()
        for vencimento in list_vencimentos:
            str_vencimentos += '%s,' % parse(vencimento).strftime('%d/%m/%Y')
        res['value']['prazo_datas'] = str_vencimentos[:-1]
        return res

    def gerar_pedido(self, cr, uid, ids, context=None):
        """
        O metodo gera os pedidos de compra para os itens que se encontram sugestivos a compra.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dados com acao para abertura de uma janela para informar ao usuario que o processo
        de geracao de pedidos foi finalizado.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_estsug_compra_capa = self.pool.get('estsug.compra.capa')
        pedido_ids = list()
        for i in self.browse(cr, uid, ids, context):
            pedido_ids.extend(obj_estsug_compra_capa.esgccp_gerar_pedido(
                cr, uid, [i.sugestao_compra_id.id], i.pessoa_id.id, i.data_entrega, i.prazo,
                i.prazo_tipo, i.vr_frete, i.prazo_datas, context))
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'estoque', 'view_estsug_compra_capa_alert_form')
        view_id = reference and reference[1] or False
        if not pedido_ids:
            return False
        return {
            'name': 'Atencao',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'estsug.compra.capa',
            'view_id': [view_id],
            'target': 'new',
            'type': 'ir.actions.act_window'}
