# -*- coding: utf-8 -*-

from openerp.osv import osv
import time
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
import logging


class compras_mkp_cron(osv.Model):

    _name = 'compras.mkp.cron'
    _logger = logging.getLogger(_name)

    def commkpcron_ativa_pesquisa(self, cr, uid, context=None):
        """
        Metodo chamado pelo agenddor (cron). Ativador do metodo commkpcron_pesquisar.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadiero.
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            UPDATE 
                ir_cron 
            SET 
                active = True 
            WHERE function = 'commkpcron_pesquisar'
        """
        cr.execute(sql)
        return True

    def commkpcron_pesquisar(self, cr, uid, context=None):
        """
        Metodo chamado pelo agenddor (cron) para pesquisa em sites.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados contextuais.
        :type context: dict
        :return:
        """
        self._logger.info('=====> Compras_mkp cron -> Pesquisando sites!')
        context = context and context.copy() or {}
        obj_compras_mkp_fornecedor = self.pool.get('compras.mkp.fornecedor')
        obj_compras_mkp_produto = self.pool.get('compras.mkp.produto')
        obj_compras_mkp_pesquisa = self.pool.get('compras.mkp.pesquisa')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        obj_estoque_fornecedor = self.pool.get('estoque.fornecedor')
        mkp_fornecedor_ids = obj_compras_mkp_fornecedor.search(cr, uid, [], context=context)

        for brw_mkpfor in obj_compras_mkp_fornecedor.browse(cr, uid, mkp_fornecedor_ids, context):
            context['sig_force_company'] = brw_mkpfor.company_id.id
            financeiro_fornecedor_ids = obj_financeiro_fornecedor.search(
                cr, uid, [('pessoa_id', '=', brw_mkpfor.commkpfor_fornecedor_id.id)],
                context=context)
            if not financeiro_fornecedor_ids:
                continue
            estoque_fornecedor_ids = obj_estoque_fornecedor.search(
                cr, uid, [('fornecedor_id', 'in', financeiro_fornecedor_ids),
                          ('estfor_anuncio', '!=', False)], context=context)
            if not estoque_fornecedor_ids:
                continue
            estoque_fornecedor_dados = obj_estoque_fornecedor.read(
                cr, uid, estoque_fornecedor_ids, ['produto_id', 'fornecedor_id'], context,
                '_classic_write')
            for estfor in estoque_fornecedor_dados:
                if obj_compras_mkp_produto.search(
                        cr, uid, [('commkppro_produto_id', '=', estfor.get('produto_id')),
                                  ('commkppro_fornecedor_id', '=', brw_mkpfor.id)],
                        context=context):
                    continue
                obj_compras_mkp_produto.create(
                    cr, uid, {'commkppro_fornecedor_id': brw_mkpfor.id,
                              'commkppro_produto_id': estfor.get('produto_id')}, context)

        if not obj_compras_mkp_fornecedor.search(cr, uid, [('commkpfor_pesquisando', '=', True)],
                                                 context=context):
            obj_compras_mkp_fornecedor.write(cr, uid, mkp_fornecedor_ids,
                                             {'commkpfor_pesquisando': True}, context)
            mkp_produto_ids = obj_compras_mkp_produto.search(cr, uid, [], context=context)
            obj_compras_mkp_produto.write(cr, uid, mkp_produto_ids, {'commkppro_pesquisado': False},
                                          context)
            cr.commit()
        produtos_a_pesquisar = obj_compras_mkp_produto.search(
            cr, uid, [('commkppro_pesquisado', '=', False)], context=context, count=True)
        qtd_pesquisa = produtos_a_pesquisar / 50
        if qtd_pesquisa < 10:
            qtd_pesquisa = 10
        elif qtd_pesquisa > 30:
            qtd_pesquisa = 30
        mkp_produto_ids = obj_compras_mkp_produto.search(
            cr, uid, [('commkppro_fornecedor_id', 'in', mkp_fornecedor_ids),
                      ('commkppro_pesquisado', '=', False)], context=context, limit=qtd_pesquisa,
            order='commkppro_fornecedor_id')
        brw_mkp_produto = obj_compras_mkp_produto.browse(cr, uid, mkp_produto_ids, context)
        for mkp_produto in brw_mkp_produto:
            dados = obj_compras_mkp_produto.commkppro_processar_pagina(
                cr, uid, mkp_produto.commkppro_fornecedor_id.commkpfor_apelido,
                mkp_produto.commkppro_url, context)
            vals = {}
            if not mkp_produto.commkppro_pesquisa_ids:
                vals = {'commkppes_produto_id': mkp_produto.id,
                        'commkppes_status': dados.get('status'),
                        'commkppes_disponibilidade': dados.get('disponibilidade'),
                        'commkppes_preco': dados.get('preco'),
                        'commkppes_mensagem': dados.get('mensagem'),
                        'commkppes_data_ultima_pesquisa': time.strftime('%Y-%m-%d')}
            else:
                dados_pesquisa_sorted = sorted(mkp_produto.commkppro_pesquisa_ids,
                                               key=lambda p: p.commkppes_data_ultima_pesquisa,
                                               reverse=True)
                ultima_pesquisa = dados_pesquisa_sorted[0]
                if any([ultima_pesquisa.commkppes_status != dados.get('status'),
                        ultima_pesquisa.commkppes_disponibilidade != dados.get('disponibilidade'),
                        ultima_pesquisa.commkppes_preco != dados.get('preco'),
                        ultima_pesquisa.commkppes_mensagem != dados.get('mensagem')]):
                    vals = {'commkppes_produto_id': mkp_produto.id,
                            'commkppes_status': dados.get('status'),
                            'commkppes_disponibilidade': dados.get('disponibilidade'),
                            'commkppes_preco': dados.get('preco'),
                            'commkppes_mensagem': dados.get('mensagem'),
                            'commkppes_data_ultima_pesquisa': time.strftime('%Y-%m-%d')}
            try:
                vals and obj_compras_mkp_pesquisa.create(cr, uid, vals, context)
                obj_compras_mkp_produto.write(cr, uid, mkp_produto.id, {'commkppro_pesquisado': True},
                                          context)
                cr.commit()
            except:
                cr.rollback()
        mkp_produto_ids = obj_compras_mkp_produto.search(
            cr, uid, [('commkppro_pesquisado', '=', False)], context=context)
        if mkp_produto_ids:
            self._logger.info('=====> Compras_mkp cron -> Fim de pesquisa! Saiu False!')
            return False
        obj_compras_mkp_fornecedor.write(cr, uid, mkp_fornecedor_ids,
                                         {'commkpfor_pesquisando': False}, context)
        cr.commit()
        self._logger.info('=====> Compras_mkp cron -> Fim de pesquisa! Saiu True!')
        return True

    def commkpcron_excluir_pesquisa(self, cr, uid, prazo, context=None):
        """
        Metodo chamado pelo agenddor (cron) para exclusao de pesquisas
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param prazo: numero de dias a ser considerado para a busca de pesquisas
        :type prazo: int
        :param context: dados contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        if prazo <= 0:
            return True
        data_limite = (parse(time.strftime('%Y-%m-%d')) - relativedelta(days=prazo)).strftime(
            '%Y-%m-%d')
        obj_compras_mkp_pesquisa = self.pool.get('compras.mkp.pesquisa')
        mkp_pesquisa_ids = obj_compras_mkp_pesquisa.search(
            cr, uid, [('commkppes_data_ultima_pesquisa', '<', data_limite)], context=context)
        obj_compras_mkp_pesquisa.unlink(cr, uid, mkp_pesquisa_ids, context)
        return True
