# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_concorencia_lancamento_lista(osv.TransientModel):
    _name = 'wizard.concorrencia.lancamento.lista'
    _columns = {
        'wconla_data': fields.date('Data', required=True),
        'wconla_produto_id': fields.many2one('estoque.produto', 'Produto', required=True,
                                             ondelete='cascade', select=True),
        'wconla_grade_id': fields.many2one('estoque.produto.grade', 'Grade', ondelete='cascade',
                                           domain="[('produto_id', '=', 'wconli_produto_id')]",
                                           select=True),
        'wconla_item_ids': fields.one2many('wizard.concorrencia.lancamento.lista.item',
                                           'wconli_capa_id', 'Itens'),
    }

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(wizard_concorencia_lancamento_lista, self).default_get(cr, uid, fields_list,
                                                                           context)
        if not context.get('wconla_data'):
            return res
        res['wconla_data'] = context.get('wconla_data')
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = super(wizard_concorencia_lancamento_lista, self).create(cr, uid, vals, context)
        self.wconla_incluir_concorrente_item(cr, uid, res, context)
        return res

    def wconla_incluir_concorrente_item(self, cr, uid, lancamento_lista_id, context):
        """
        O metodo cria novos registros na tabela concorrencia_concorrente_item, baseado nos dados do
        registro desse objeto (lancamento_lista_id).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param lancamento_lista_id: id da tabela desse objeto.
        :type lancamento_lista_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso todos os registros tenham sido criados. Senao, raise com a descri
        cao do problema ocorrido.
        """
        context = context and context.copy() or {}
        obj_concorrencia_concorrente_item = self.pool.get('concorrencia.concorrente.item')
        brw_lancamento_lista = self.browse(cr, uid, lancamento_lista_id, context)
        vals = {'data': brw_lancamento_lista.wconla_data,
                'produto_id': brw_lancamento_lista.wconla_produto_id.id,
                'grade_id': brw_lancamento_lista.wconla_grade_id.id or False}
        itens = brw_lancamento_lista.wconla_item_ids
        if filter(lambda i: i.wconli_preco <= 0, itens):
            raise osv.except_osv(_('Atencao'),
                                 _('Todos os itens devem possuir preco maior que zero'))
        for item in itens:
            vals.update({'concorrente_id': item.wconli_concorrente_id.id,
                         'preco': item.wconli_preco})
            obj_concorrencia_concorrente_item.create(cr, uid, vals, context)
        return True

    def wconla_salvar_e_novo(self, cr, uid, ids, context=None):
        """
        O metodo executa acao para abertura de um novo formulario para esse objeto.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: dados da acao
        :rtype: dict
        """
        context = context and context.copy() or {}
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'concorrencia', 'view_wizard_concorrencia_lancamento_lista_form')
        view_id = reference and reference[1] or False
        context.update({'wconla_data': self.read(
            cr, uid, ids[0] if isinstance(ids, (list, tuple)) else ids, ['wconla_data'],
            context).get('wconla_data')})
        return {
            'name': 'Lancar Lista',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'target': 'inline',
            'type': 'ir.actions.act_window',
            'context': context
        }


class wizard_concorrencia_lancamento_lista_item(osv.TransientModel):
    _name = 'wizard.concorrencia.lancamento.lista.item'
    _columns = {
        'wconli_capa_id': fields.many2one('wizard.concorrencia.lancamento.lista', 'Capa'),
        'wconli_concorrente_id': fields.many2one('concorrencia.concorrente', 'Concorrente',
                                                 ondelete="cascade", required=True, select=True),
        'wconli_preco': fields.float('Preco', digits=(14, 2), required=True),
    }
