# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import time


class wizard_concorrencia_listagem_pesquisa(osv.TransientModel):
    # prefixo: wcopes_

    _name = 'wizard.concorrencia.listagem.pesquisa'
    _columns = {
        'wcopes_codigo': fields.selection([('I', 'Item'), ('F', 'Fabricante')], 'Codigo a listar',
                                          size=1),
        'wcopes_so_marcados': fields.boolean('So marcados', help='Lista apenas os itens que '
                                                                 'estiverem para pesquisa'),
        'wcopes_diaria': fields.boolean('Pesquisa diaria'),
        'wcopes_semanal': fields.boolean('Pesquisa semanal'),
        'wcopes_quinzenal': fields.boolean('Pesquisa quinzenal'),
        'wcopes_mensal': fields.boolean('Pesquisa mensal'),
        'wcopes_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wcopes_comprador_id': fields.many2one('compras.comprador', 'Comprador'),
        'wcopes_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'wcopes_agrupar': fields.boolean('Agrupar', help='Agrupa itens de grade.')
    }
    _defaults = {
        'wcopes_codigo': lambda *a: 'I',
        'wcopes_so_marcados': lambda *a: True,
        'wcopes_diaria': lambda *a: True,
        'wcopes_agrupar': lambda *a: False
    }

    def wcopes_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        pesquisa_id = ids[0] if isinstance(ids, list) else ids
        brw_pesquisa = self.browse(cr, uid, pesquisa_id, context)
        dados = {'produto_dados': self.wcopes_get_produto_dado(cr, uid, ids, context),
                 'data_emissao': time.strftime('%d/%m/%Y'),
                 'codigo': 'Fabricante' if brw_pesquisa.wcopes_codigo == 'F' else 'Item',
                 'so_marcados': 'Sim' if brw_pesquisa.wcopes_so_marcados else 'Nao',
                 'agrupar': 'Sim' if brw_pesquisa.wcopes_agrupar else 'Nao'}
        if brw_pesquisa.wcopes_diaria:
            dados['diaria'] = 'Sim'
        if brw_pesquisa.wcopes_semanal:
            dados['semanal'] = 'Sim'
        if brw_pesquisa.wcopes_quinzenal:
            dados['quinzenal'] = 'Sim'
        if brw_pesquisa.wcopes_mensal:
            dados['mensal'] = 'Sim'
        if brw_pesquisa.wcopes_categoria_id:
            dados['categoria'] = brw_pesquisa.wcopes_categoria_id.name
        if brw_pesquisa.wcopes_comprador_id:
            dados['comprador'] = brw_pesquisa.wcopes_comprador_id.nome
        if brw_pesquisa.wcopes_fabricante_id:
            dados['fabricante'] = brw_pesquisa.wcopes_fabricante_id.nome
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'concorrencia_listagem_pesquisa',
            'datas': dados,
        }

    def wcopes_get_produto_dado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados = []
        obj_estoque_categoria = self.pool.get('estoque.categoria')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.produto', context=context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'local_venda_padrao_id', company_id, context)
        local_id = estoque_parametro_dados.get('local_venda_padrao_id')
        for pesquisa in self.browse(cr, uid, ids, context):
            agrupar = pesquisa.wcopes_agrupar
            codigo = """ep.codigo                                    AS CODIGO"""
            if pesquisa.wcopes_codigo == 'F':
                codigo = """COALESCE(epg.codigo_no_fabricante, '')       AS CODIGO"""
            unidade = """
                    , eeu.unidade                                   AS unidade
            """
            grade = """
                    , epg.grade
                    , epg.descricao                                AS GRADE_DESCRICAO
            """
            if agrupar:
                if local_id:
                    unidade = """
                        ,(SELECT min(unidade) 
                          FROM estoque_embalagem_unidade 
                          WHERE id IN (SELECT unidade_id 
                                       FROM estoque_embalagem 
                                       WHERE id IN (SELECT embalagem_id 
                                                    FROM estoque_produto_embalagem 
                                                    WHERE id IN (SELECT 
                                                                    min(estllc_produto_embalagem_id)
                                                                 FROM estoque_local_locacao
                                                                 WHERE produto_id = ep.id
                                                                       AND local_id = {local_id})))) 
                                                                        AS unidade
                    """.format(local_id=local_id)
                    grade = """
                        , (SELECT grade 
                           FROM estoque_produto_grade 
                           WHERE id IN (SELECT 
                                            min(grade_id)
                                        FROM estoque_local_locacao
                                        WHERE produto_id = ep.id
                                              AND local_id = {local_id}))
                                                                        AS grade
                        , (SELECT descricao 
                           FROM estoque_produto_grade 
                           WHERE id IN (SELECT 
                                            min(grade_id)
                                        FROM estoque_local_locacao
                                        WHERE produto_id = ep.id
                                              AND local_id = {local_id}))
                                                                        AS GRADE_DESCRICAO
                    """.format(local_id=local_id)
                else:
                    unidade = """
                        ,(SELECT min(unidade) 
                          FROM estoque_embalagem_unidade 
                          WHERE id IN (SELECT unidade_id 
                                       FROM estoque_embalagem 
                                       WHERE id IN (SELECT embalagem_id 
                                                    FROM estoque_produto_embalagem 
                                                    WHERE produto_id = ep.id))) 
                                                                        AS unidade
                    """
                    grade = """
                        , (SELECT grade 
                           FROM estoque_produto_grade 
                           WHERE id IN (SELECT min(id) 
                                       FROM estoque_produto_grade 
                                       WHERE produto_id = ep.id))       AS grade
                        , (SELECT descricao 
                           FROM estoque_produto_grade 
                           WHERE id IN (SELECT min(id) 
                                        FROM estoque_produto_grade 
                                        WHERE produto_id = ep.id))      AS GRADE_DESCRICAO
                    """
            sql = """
                SELECT %s
                     , ep.name
                     %s
                     %s
                FROM estoque_produto                                AS ep
            """ % (codigo, grade, unidade)
            if not agrupar:
                sql += """
                INNER JOIN estoque_produto_grade                    AS epg
                        ON ep.id = epg.produto_id
                INNER JOIN estoque_produto_embalagem                AS epe
                        ON ep.id = epe.produto_id
                INNER JOIN estoque_local_locacao ell
                        ON ep.id = ell.produto_id
                       AND epg.id = ell.grade_id
                       AND epe.id = ell.estllc_produto_embalagem_id
                INNER JOIN estoque_embalagem ee
                        ON ee.id = epe.embalagem_id
                INNER JOIN estoque_embalagem_unidade eeu
                        ON eeu.id = ee.unidade_id
            """
            sql += """
                WHERE ep.company_id = %s
            """ % company_id
            if local_id and not agrupar:
                sql += """
                    AND ell.local_id = %s        
                """ % local_id
            if pesquisa.wcopes_comprador_id:
                sql += """
                  AND ep.comprador_id = %s""" % pesquisa.wcopes_comprador_id.id
            if pesquisa.wcopes_fabricante_id:
                sql += """
                  AND ep.fabricante_id = %s""" % pesquisa.wcopes_fabricante_id.id
            if pesquisa.wcopes_categoria_id:
                categoria_ids = obj_estoque_categoria.search(
                    cr, uid, [('parent_id', 'child_of', pesquisa.wcopes_categoria_id.id)],
                    context=context)
                if len(categoria_ids) == 1:
                    categoria_ids = str(tuple(categoria_ids)).replace(',', '')
                else:
                    categoria_ids = str(tuple(categoria_ids))
                sql += """
                  AND ep.categoria_id IN %s""" % categoria_ids
            if pesquisa.wcopes_so_marcados:
                sql += """
                  AND ep.acompanhar_precos_concorrentes IS TRUE"""

            # Determina as pesquisas periodicas
            periodico =(pesquisa.wcopes_diaria and 'ep.pesquisa_diaria IS TRUE',
                        pesquisa.wcopes_semanal and 'ep.pesquisa_semanal IS TRUE',
                        pesquisa.wcopes_quinzenal and 'ep.pesquisa_quinzenal IS TRUE',
                        pesquisa.wcopes_mensal and 'ep.pesquisa_mensal IS TRUE')
            if any(periodico):
                sql += """
                  AND (""" + '\n                   OR '.join(filter(lambda f: f, periodico))
            if any(periodico):
                sql += """)"""

            sql += """
                ORDER BY ep.name"""
            cr.execute(sql)
            dados.extend(cr.dictfetchall())

        return dados

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
