# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import time
from dateutil.parser import parse


class wizard_concorrencia_listagem_resultado(osv.TransientModel):
    # prefixo: wcores_

    _name = 'wizard.concorrencia.listagem.resultado'
    _columns = {
        'wcores_data_inicial': fields.date('Data inicial', required=True),
        'wcores_data_final': fields.date('Data final', required=True),
        'wcores_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'wcores_produto_id': fields.many2one('estoque.produto', 'Item'),
        'wcores_ordem_impressao': fields.selection(
            [('C', 'Codigo'), ('A', 'Alfabetica'), ('F', 'Fabricante')], 'Ordem impressao', size=1,
            required=True),
        'wcores_ordem_resultado': fields.selection([('M', 'Mais barato'), ('C', 'Concorente')],
                                                   'Ordem resultado', size=1),
    }

    def wcores_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        resultado_id = ids[0] if isinstance(ids, list) else ids
        brw_resultado = self.browse(cr, uid, resultado_id, context)

        ordem_impressao = brw_resultado.wcores_ordem_impressao
        if ordem_impressao == 'C':
            ordem_impressao = 'Codigo'
        elif ordem_impressao == 'A':
            ordem_impressao = 'Alfabetica'
        elif ordem_impressao == 'F':
            ordem_impressao = 'Fabricante'

        dados = {'concorrencia_dados': self.wcores_get_produto_dado(
            cr, uid, brw_resultado.wcores_data_inicial, brw_resultado.wcores_data_final,
            brw_resultado.wcores_ordem_impressao, brw_resultado.wcores_fabricante_id.id,
            brw_resultado.wcores_produto_id.id, context),
                 'data_emissao': time.strftime('%d/%m/%Y'),
                 'data_inicial': parse(brw_resultado.wcores_data_inicial).strftime('%d/%m/%Y'),
                 'data_final': parse(brw_resultado.wcores_data_final).strftime('%d/%m/%Y'),
                 'str_ordem_impressao': ordem_impressao,
                 'ordem_impressao': brw_resultado.wcores_ordem_impressao}

        if brw_resultado.wcores_fabricante_id:
            dados['fabricante'] = brw_resultado.wcores_fabricante_id.nome

        if brw_resultado.wcores_produto_id:
            dados['produto'] = brw_resultado.wcores_produto_id.name

        if brw_resultado.wcores_ordem_resultado:
            dados['str_ordem_resultado'] = 'Mais barato' if \
                brw_resultado.wcores_ordem_resultado == 'M' else 'Concorrente'
            dados['ordem_resultado'] = brw_resultado.wcores_ordem_resultado

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'concorrencia_listagem_resultado',
            'datas': dados,
        }

    def wcores_get_produto_dado(self, cr, uid, data_inicial, data_final, ordem_impressao,
                                fabricante_id=False, produto_id=False, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'concorencia.concorrente', context=context)
        if ordem_impressao == 'C':
            order_by = """
            ORDER BY ep.codigo
            """
        elif ordem_impressao == 'A':
            order_by = """
            ORDER BY ep.name
            """
        else:
            order_by = """
            ORDER BY ap2.codigo
            """
        sql = """
            SELECT
                ep.id                         AS produto_id,
                epg.id                        AS grade_id,
                cci.data                      AS data,
                ep.codigo || '/' || ep.digito AS item,
                epg.grade                     AS grade,
                ep.name                       AS descricao_produto,
                epg.descricao                 AS descricao_grade,
                cci.preco || '-' || ap.codigo AS preco_venda_concorrente,
                cci.preco                     AS preco_venda,
                ap.codigo                     AS codigo_concorrente,
                ap.codigo || '-' || ap.nome   AS concorrente
            FROM concorrencia_concorrente_item cci
            INNER JOIN concorrencia_concorrente cc
                ON cc.id = cci.concorrente_id
            INNER JOIN estoque_produto ep
                ON ep.id = cci.produto_id
            INNER JOIN adm_pessoa ap
                ON ap.id = cc.pessoa_id
            LEFT JOIN estoque_produto_grade epg
                ON epg.id = cci.grade_id
            LEFT JOIN adm_pessoa ap2
                ON ap2.id = ep.fabricante_id
            WHERE
                cc.company_id = %s
                AND cci.data BETWEEN '%s' AND '%s'
        """ % (company_id, data_inicial, data_final)

        if fabricante_id:
            sql += """
                AND ep.fabricante_id = %s
            """ % fabricante_id

        if produto_id:
            sql += """
                AND ep.id = %s
            """ % produto_id

        sql += order_by

        cr.execute(sql)
        return cr.dictfetchall()
