# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import re
from SigUtils import calcDigito
from openerp.tools.translate import _


class contabil_conta(osv.Model):
    _name = 'contabil.conta'
    _inherit = 'contabil.conta'

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __code = 'ctb_codigo_reduzido'
        __name = 'Conta reduzida da Contabilidade'
        vals = {
            'name': __name,
            'code': __code,
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        # Gerar sequence para numero do departamento para a empresa
        consulta = self.pool.get('ir.sequence.type').search(cr, uid, [('code', '=', __code)],
                                                            context=context)
        if not consulta:
            self.pool.get('ir.sequence.type').create(cr, uid, {'name': __name, 'code': __code},
                                                     context)
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def contabil_formatar_conta(self, cr, uid, conta, company_id, mascara=None, context=None):
        context = context and context.copy() or {}
        if not mascara:
            mascara = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'adm', [], company_id=company_id, context=context).get(
                'admpar_ctb_mascara_conta') or ''
            if not mascara:
                return conta
        return mascara.replace('#', '{}').format(
            *'%-{}s'.format(len(mascara)) % conta).strip().rstrip(' .')

    def onchange_conta(self, cr, uid, ids, company_id, conta, context=None):
        """
        Metodo invocado na saida do campo de conta da tela.
        formata a conta conforme ctb_mascara do parametro
        """
        context = context and context.copy() or {}
        warning = {}
        _value = {}
        if conta:
            _conta = re.sub('[^0-9]', '', conta)
            _conta = self.contabil_formatar_conta(cr, uid, _conta, company_id, context=context)
            if not _conta:
                warning = {'title': 'Erro na validacao'}
                warning['message'] = 'Conta Invalida!  Digite Novamente!'
            else:
                _grau = _conta.count('.') + 1
                if _grau > 1:
                    _conta_sup = ''
                    for x in _conta.split('.')[:_grau - 1]:
                        _conta_sup += x + '.'
                    _conta_sup = _conta_sup[:-1]
                    _value['parent_id'] = self.search(cr, uid, [('conta', '=', _conta_sup.replace('.', ''))],
                                                      context=context)
                _value['grau'] = _grau
                _value['conta'] = _conta
        return {'warning': warning, 'value': _value}


    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        context = context and context.copy() or {}
        reads = self.read(cr, uid, ids, ['name', 'conta'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['conta']:
                _conta = record['conta']
                name = _conta + ' - ' + record['name']
            res.append((record['id'], name))
        return res

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        try:
            ids = self.search(cr, uid, [('conta_reduzida', '=', str(int(name)))] + args, limit=limit, context=context)
            if ids:
                return self.name_get(cr, uid, ids, context)
        except:
            pass
        _conta = re.sub('[^0-9]', '', name)
        ids = self.search(cr, uid, [('conta', '=', _conta)] + args, limit=limit, context=context)
        if ids:
            return self.name_get(cr, uid, ids, context)
        ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, uid, ids, context)


    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('conta') and vals['conta']:
            _conta = re.sub('[^0-9]', '', vals['conta'])
            vals['conta'] = _conta
        if vals.get('conta_tipo') and vals['conta_tipo'] == 'A':
            vals['digito_conta'] = calcDigito.modulo10(_conta)
            nome = 'ctb_codigo_reduzido' + '%003d' % vals['company_id']
            vals['conta_reduzida'] = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, 'ctb_codigo_reduzido')
            vals['digito_reduzida'] = calcDigito.modulo10(vals['conta_reduzida'])
        if vals.get('ctbctb_rateio_ids') and vals.get('ctbctb_rateio_ids'):
            percentual = 0.0
            ctbctb_rateio_ids = vals.get('ctbctb_rateio_ids')
            for x in ctbctb_rateio_ids:
                for y in x:
                    if isinstance(y, dict):
                        percentual += y.get('ctbccr_percentual') or 0.0
            if percentual > 100.00:
                raise osv.except_osv(_('Atencao'), _('A soma do percentual dos centros de custos ultrapassa os 100%'))
            elif percentual < 100:
                raise osv.except_osv(_('Atencao'), _('A soma do percentual dos centros de custos nao atingiu os 100%'))
        result = super(contabil_conta, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = True
        if vals.get('conta') and vals['conta']:
            _conta = re.sub('[^0-9]', '', vals['conta'])
            vals['conta'] = _conta
        list_rateio = []
        rateio_excluidos = []
        percentual = 0.0
        if vals.get('ctbctb_rateio_ids'):
            ctbctb_rateio_ids = vals.get('ctbctb_rateio_ids')
            for x in ctbctb_rateio_ids:
                for y in x:
                    if isinstance(y, dict):
                        list_rateio.append(x[1])
                        percentual += y.get('ctbccr_percentual') or 0.0
                    if x.index(y) == 0 and y == 2:
                        rateio_excluidos.append(x[1])
            if percentual > 100.00:
                raise osv.except_osv(_('Atencao'),
                                     _('A soma do percentual dos centros de custos ultrapassa os 100%'))
        ctbctb_rateio_ids = self.browse(cr, uid, ids[0]).ctbctb_rateio_ids
        percentual1 = 0.0
        if ctbctb_rateio_ids:
            for x in ctbctb_rateio_ids:
                if x.id not in list_rateio and x.id not in rateio_excluidos:
                    percentual1 += x.ctbccr_percentual or 0.0

            if percentual + percentual1 > 100.00:
                raise osv.except_osv(_('Atencao'), _('A soma do percentual dos centros de custos ultrapassa os 100%'))
            elif percentual + percentual1 < 100:
                raise osv.except_osv(_('Atencao'), _('A soma do percentual dos centros de custos nao atingiu os 100%'))

        result = super(contabil_conta, self).write(cr, uid, ids, vals, context)
        return result

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Le os dados da conta e devolve o numero da conta formatado
        """
        context = context and context.copy() or {}
        if not isinstance(ids, list):
            ids = [ids]
        if fields and not 'company_id' in fields:
            fields.append('company_id')
        res = super(contabil_conta, self).read(cr, uid, ids, fields=fields, context=context, load=load)
        _res = []
        for x in res:
            empresa = x.get('company_id')
            if empresa and isinstance(empresa, tuple):
                empresa = empresa[0]
            if x.get('conta'):
                _conta = self.contabil_formatar_conta(cr, uid, x.get('conta'), empresa, context=context)
                x['conta'] = _conta
            _res.append(x)
        return _res

