#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class pgview_razao_sintetico(osv.Model):

    _name = 'pgview.contabil.razao.sintetico'
    _auto = False
    _columns = {
        'id': fields.integer('ID'),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade"),
        'pgraz_sint_conta_tipo': fields.char('Tipo de conta', size=1),
        'pgraz_sint_conta_numero': fields.char('Conta', size=20),
        'pgraz_sint_conta_name': fields.char('Descricao', size=50),
        'pgraz_sint_conta_grau': fields.integer('Grau'),
        'pgraz_sint_conta_natureza': fields.selection(
            [('A', 'Ativo'), ('P', 'Passivo'), ('R', 'Contas de resultado'), ('L', 'Patrimonio liquido'),
             ('C', 'Compensacao'), ('O', 'Outras')], 'Natureza', size=1
        ),
        'pgraz_sint_conta_parent_id': fields.integer('Parent ID'),
        'pgraz_sint_saldo_mes_ano': fields.char('Mes e Ano do Saldo', size=7),
        'pgraz_sint_saldo_inicial': fields.float('Saldo inicial', digits=(14, 2)),
        'pgraz_sint_saldo_debitos': fields.float('Debitos', digits=(14, 2)),
        'pgraz_sint_saldo_creditos': fields.float('Creditos', digits=(14, 2)),
        'pgraz_sint_saldo_final': fields.float('Saldo Final', digits=(14, 2)),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'pgview_contabil_razao_sintetico')
        cr.execute(
            """
            CREATE VIEW pgview_contabil_razao_sintetico AS (
                SELECT
                    --rank() over(order by conta.id) AS id,
                    conta.id AS id,
                    conta.company_id AS company_id,
                    conta.conta_tipo AS pgraz_sint_conta_tipo,
                    conta.conta AS pgraz_sint_conta_numero,
                    conta.grau AS pgraz_sint_conta_grau,
                    conta.name AS pgraz_sint_conta_name,
                    conta.natureza AS pgraz_sint_conta_natureza,
                    conta.parent_id AS pgraz_sint_conta_parent_id,
                    to_char(saldos.data_saldo, 'MM-YYYY') AS pgraz_sint_saldo_mes_ano,
                    sum(saldos.ctbsal_saldo_inicial) AS pgraz_sint_saldo_inicial,
                    sum(saldos.ctbsal_debitos) AS pgraz_sint_saldo_debitos,
                    sum(saldos.ctbsal_creditos) AS pgraz_sint_saldo_creditos,
                    sum(saldos.ctbsal_saldo_final) AS pgraz_sint_saldo_final
                FROM
                    contabil_conta conta
                LEFT JOIN
                    contabil_saldos saldos ON conta.id = saldos.contabil_conta_id
                GROUP BY
                    conta.id,
                    conta.company_id,
                    conta.conta_tipo,
                    conta.conta,
                    conta.grau,
                    conta.name,
                    conta.natureza,
                    conta.parent_id,
                    saldos.data_saldo
                ORDER BY
                    conta.conta
            );
            """
        )
