#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.pooler import get_pool
from datetime import date
from StringIO import StringIO
from openerp.report.render import render
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from SigUtils.impressao import Impressora
from SigUtils.formatastring import formataString
from openerp.tools.translate import _

NOME_ARQUIVO_DEFAULT = 'CONTABIL_DIFERENCA_DEBITO_CREDITO'
READ_LOAD = '_classic_write'


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_contabil_diferenca_debito_credito(report_int):
    def create(self, cr, uid, ids, data, context=None):
        context = context and context.copy() or {}
        pdf_string = StringIO()
        listagem = Impressora()
        listagem.informaNome(pdf_string, _('Razao Sintetico'))
        listagem.informaTamLinha(160)

        # VARIAVEIS GLOBAIS
        self.traco_linha = 150
        self.space = ' ' * self.traco_linha  # ESPAÇO DE LINHA.
        self.db_name = get_pool(cr.dbname)  # PERMITE ACESSO AS TABELAS.

        self.col_documento = 26
        self.col_data_movimento = 30
        self.col_creidto = 30
        self.col_debito = 30
        self.col_valor_total = 30

        ########################################################3
        WCTB_DIF_COMPANY_ID = data.get('company_id')
        WCTB_DIF_COMPANY_NAME = data.get('company_name') or ''
        WCTB_DIF_MES = data.get('mes')
        WCTB_DIF_ANO = data.get('ano') or date.today().year
        WCTB_DIF_PAGINA = 0

        self.define_cabecalho(listagem, WCTB_DIF_MES, WCTB_DIF_ANO, WCTB_DIF_PAGINA, WCTB_DIF_COMPANY_NAME)
        self.listar_movimentos(listagem, cr, uid, WCTB_DIF_COMPANY_ID, WCTB_DIF_MES, WCTB_DIF_ANO, context)
        ########################################################3

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return self.obj.pdf, 'pdf'

    def define_cabecalho(self, listagem, mes, ano, pagina, company_name):

        MES = {
            '01': 'Janeiro', '02': 'Fevereiro', '03': 'Marco', '04': 'Abril',
            '05': 'Maio', '06': 'Junho', '07': 'Julho', '08': 'Agosto',
            '09': 'Setembro', '10': 'Outubro', '11': 'Novembro', '12': 'Dezembro'
        }

        listagem.setPage(pagina)
        nome_mes = MES[mes]

        contabilidade = _('Contabilidade')
        titulo = _('Diferenca de Debito e Credito - %s de %s') % (nome_mes, str(ano))
        codigo = ''

        col_left = 30
        col_center = 90
        col_right = 30

        str_contabilidade = formataString(contabilidade, col_left, ' ', 'D', 'N', 'S')
        str_titulo = formataString(titulo, col_center, ' ', 'C', 'N', 'S')
        str_ctb330 = formataString(codigo, col_right, ' ', 'E', 'N', 'S')

        str_data_impressao = formataString(date.today().strftime('%d-%m-%Y'), col_left, ' ', 'D', 'N', 'S')
        str_company_name = formataString(company_name, col_center, ' ', 'C', 'N', 'S')
        str_pagina = formataString(_('Pagina: {{ Page , 5 }}'), col_right + 9, ' ', 'E', 'N', 'S')

        str_documento = formataString(_('DOCUMENTO'), self.col_documento, ' ', 'D', 'N', 'S')
        str_data_movimento = formataString(_('DATA MOVIMENTO'), self.col_data_movimento, ' ', 'E', 'N', 'S')
        str_credito = formataString(_('CREDITO'), self.col_creidto, ' ', 'E', 'N', 'S')
        str_debito = formataString(_('DEBITO'), self.col_debito, ' ', 'E', 'N', 'S')
        str_valor_total = formataString(_('VALOR TOTAL'), self.col_valor_total, ' ', 'E', 'N', 'S')

        listagem.informaCab(str_contabilidade + str_titulo + str_ctb330)
        listagem.informaCab(self.space)
        listagem.informaCab(str_data_impressao + str_company_name + str_pagina)
        listagem.informaCab(self.space)
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(
            '|' + str_documento + '|' + str_data_movimento + '|' + str_credito + '|' + str_debito + '|' + str_valor_total + '|')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(self.space)
        listagem.rodapePadrao()

        return True

    def get_movimento(self, cr, uid, company_id, mes, ano, context=None):
        context = context and context.copy() or {}
        obj_pgview_dif_dc = self.db_name.get('pgview.contabil.dif.debito.credito')
        mes_ano = '%s-%s' % (str(mes), str(ano))

        args = [('company_id', '=', company_id), ('pgctb_dif_mes_ano', '=', mes_ano)]
        dif_dc_ids = obj_pgview_dif_dc.search(cr, uid, args, context=context)
        if not dif_dc_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum registro encontrado para a geracao do relatorio'))
        else:
            fields_to_read = [
                'pgctb_dif_documento', 'pgctb_dif_data_movimento', 'pgctb_dif_debito', 'pgctb_dif_credito'
            ]
            res = obj_pgview_dif_dc.read(cr, uid, dif_dc_ids, fields_to_read, context=context, load=READ_LOAD)
            return res

    def listar_movimentos(self, listagem, cr, uid, company_id, mes, ano, context=None):
        context = context and context.copy() or {}
        col_documento = self.col_documento
        col_data_movimento = self.col_data_movimento
        col_credito = self.col_creidto
        col_debito = self.col_debito
        col_valor_total = self.col_valor_total

        # FUNCTIONS
        funct_imprime_linha = listagem.imprimeLinha

        relatorio_dados = self.get_movimento(cr, uid, company_id, mes, ano, context)
        saldo_inicial = 0
        for i in relatorio_dados:
            DIFDC_DOCUMENTO = i.get('pgctb_dif_documento')
            DIFDC_DATA_MOVIMENTO = i.get('pgctb_dif_data_movimento')
            DIFDC_CREDITO = i.get('pgctb_dif_credito')
            DIFDC_DEBITO = i.get('pgctb_dif_debito')

            valor_total = saldo_inicial - DIFDC_CREDITO + DIFDC_DEBITO
            saldo_inicial = valor_total

            str_documento = formataString('%d' % DIFDC_DOCUMENTO, col_documento, ' ', 'D', 'S', 'S')
            str_data_movimento = formataString(DIFDC_DATA_MOVIMENTO, col_data_movimento, ' ', 'E', 'S', 'S')
            str_credito = formataString(getNumeroEditado(DIFDC_CREDITO), col_credito, ' ', 'E', 'S', 'S')
            str_debito = formataString(getNumeroEditado(DIFDC_DEBITO), col_debito, ' ', 'E', 'S', 'S')
            str_valor = formataString(getNumeroEditado(valor_total), col_valor_total, ' ', 'E', 'S', 'S')

            funct_imprime_linha(
                ' ' + str_documento + ' ' + str_data_movimento + ' ' + str_credito + ' ' + str_debito + ' ' + str_valor + ' ')

        return True


report_contabil_diferenca_debito_credito('report.%s' % NOME_ARQUIVO_DEFAULT)
