#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from openerp.osv import osv
from openerp.pooler import get_pool
from datetime import date
from StringIO import StringIO
from openerp.report.render import render
from openerp.report.interface import report_int
from SigUtils.impressao import Impressora
from SigUtils.formatastring import formataString
from openerp.tools.translate import _

NOME_ARQUIVO_DEFAULT = 'CONTABIL_RAZAO_SINTETICO'


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_contabil_razao_sintetico(report_int):
    def create(self, cr, uid, ids, data, context=None):
        context = context and context.copy() or {}
        pdf_string = StringIO()
        listagem = Impressora()
        if context and context.get('balancete'):
            listagem.informaNome(pdf_string, _('Balancete'))
        else:
            listagem.informaNome(pdf_string, _('Razao Sintetico'))
        listagem.informaTamLinha(160)

        # VARIAVEIS GLOBAIS
        self.traco_linha = 150
        self.space = ' ' * self.traco_linha  # ESPAÇO DE LINHA.
        self.db_name = get_pool(cr.dbname)  # PERMITE ACESSO AS TABELAS.

        # COLUNAS DO RELATORIO
        self.col_conta_nome = 60
        self.col_saldo_anterior = 21
        self.col_debitos = 21
        self.cols_creditos = 21
        self.col_saldo_atual = 21

        RAZ_SINT_COMPANY_ID = data.get('company_id')
        RAZ_SINT_COMPANY_NAME = data.get('company_name') or ''
        RAZ_SINT_MES = data.get('mes')
        RAZ_SINT_ANO = data.get('ano') or date.today().year
        RAZ_SINT_NATUREZA = data.get('natureza')
        RAZ_SINT_PAGINA = 0

        self.define_cabecalho(listagem, RAZ_SINT_MES, RAZ_SINT_ANO, RAZ_SINT_PAGINA, RAZ_SINT_COMPANY_NAME, context)
        self.listar_movimentos(
            listagem, cr, uid, RAZ_SINT_COMPANY_ID, RAZ_SINT_MES, RAZ_SINT_ANO, RAZ_SINT_NATUREZA, context
        )
        #########################################################

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return self.obj.pdf, 'pdf'

    def define_cabecalho(self, listagem, mes, ano, pagina, company_name, context=None):
        context = context and context.copy() or {}
        MES = {
            '01': 'Janeiro', '02': 'Fevereiro', '03': 'Marco', '04': 'Abril',
            '05': 'Maio', '06': 'Junho', '07': 'Julho', '08': 'Agosto',
            '09': 'Setembro', '10': 'Outubro', '11': 'Novembro', '12': 'Dezembro'
        }

        listagem.setPage(pagina)
        nome_mes = MES[mes]

        contabilidade = _('Contabilidade')
        if context.get('balancete'):
            ctb330 = 'ctb350'
            titulo = _('Balancete de %s de %s') % (nome_mes, str(ano))
        else:
            ctb330 = 'ctb330'
            titulo = _('Razao Sintetico de %s de %s') % (nome_mes, str(ano))

        col_left = 30
        col_center = 90
        col_right = 30

        str_contabilidade = formataString(contabilidade, col_left, ' ', 'D', 'N', 'S')
        str_titulo = formataString(titulo, col_center, ' ', 'C', 'N', 'S')
        str_ctb330 = formataString(ctb330, col_right, ' ', 'E', 'N', 'S')

        str_data_impressao = formataString(date.today().strftime('%d-%m-%Y'), col_left, ' ', 'D', 'N', 'S')
        str_company_name = formataString(company_name, col_center, ' ', 'C', 'N', 'S')
        str_pagina = formataString(_('Pagina: {{ Page , 5 }}'), col_right + 9, ' ', 'E', 'N', 'S')

        str_conta = formataString(_('  CONTA'), self.col_conta_nome, ' ', 'D', 'N', 'S')
        str_saldo_anterior = formataString(_('SALDO ANTERIOR'), self.col_saldo_anterior, ' ', 'E', 'N', 'S')
        str_debitos = formataString(_('DEBITOS'), self.col_debitos, ' ', 'E', 'N', 'S')
        str_creditos = formataString(_('CREDITOS'), self.cols_creditos, ' ', 'E', 'N', 'S')
        str_saldo_atual = formataString(_('SALDO ATUAL'), self.col_saldo_atual, ' ', 'E', 'N', 'S')

        listagem.informaCab(str_contabilidade + str_titulo + str_ctb330)
        listagem.informaCab(self.space)
        listagem.informaCab(str_data_impressao + str_company_name + str_pagina)
        listagem.informaCab(self.space)
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(
            ' ' + str_conta + ' ' + str_saldo_anterior + ' ' + str_debitos + ' ' + str_creditos + ' ' + str_saldo_atual + ' ')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(self.space)
        listagem.rodapePadrao()

        return True

    def get_movimento_data(self, cr, uid, company_id, mes, ano, natuereza, context=None):
        context = context and context.copy() or {}
        obj_pgview_contabil_razao_sint = self.db_name.get('pgview.contabil.razao.sintetico')
        args = [('company_id', '=', company_id)]
        if natuereza:
            args.append(('pgraz_sint_conta_natureza', '=', natuereza))
        contabil_razao_sint_ids = obj_pgview_contabil_razao_sint.search(cr, uid, args, context=context)

        # contabil_formatar_conta
        fields_to_read = [
            'pgraz_sint_conta_tipo', 'pgraz_sint_conta_numero', 'pgraz_sint_conta_name', 'pgraz_sint_conta_grau',
            'pgraz_sint_conta_parent_id', 'pgraz_sint_saldo_inicial', 'pgraz_sint_saldo_debitos',
            'pgraz_sint_saldo_creditos', 'pgraz_sint_saldo_final', 'pgraz_sint_saldo_mes_ano'
        ]

        if not contabil_razao_sint_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum dado encontrado para a geracao do relatorio'))
        else:
            adm_contabil_mascara = None
            obj_adm_parametro = self.db_name.get('adm.parametro')
            adm_parametro_ids = obj_adm_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
            if not adm_parametro_ids:
                raise osv.except_osv(_('Atencao'), _('Nenhum parametro de administracao encontrado para a empresa'))
            else:
                adm_parametro_dados = obj_adm_parametro.read(cr, uid, adm_parametro_ids, ['admpar_ctb_mascara_conta'],
                                                             context=context)
                for x in adm_parametro_dados:
                    adm_contabil_mascara = x.get('admpar_ctb_mascara_conta')

            funct_contabil_formatar_conta = self.db_name.get('contabil.conta').contabil_formatar_conta
            pgview_dados = obj_pgview_contabil_razao_sint.read(
                cr, uid, contabil_razao_sint_ids, fields_to_read, context=context)

            mes_ano = '%s-%s' % (mes, ano)

            res = filter(
                lambda i: i['pgraz_sint_saldo_mes_ano'] == mes_ano or i['pgraz_sint_conta_tipo'] == 'S', pgview_dados
            )

            for i in res:
                conta = i['pgraz_sint_conta_numero']
                conta_formatada = funct_contabil_formatar_conta(
                    cr, uid,
                    conta=conta,
                    company_id=company_id,
                    mascara=adm_contabil_mascara, context=context
                )
                i['pgraz_sint_conta_numero'] = conta_formatada
            return res

    def totalizar_contas_sinteticas(self, res):
        contas_analiticas = []
        contas_sinteticas = []
        for i in res:
            if i['pgraz_sint_conta_tipo'] == 'A':
                contas_analiticas.append(i)
            else:
                contas_sinteticas.append(i)

        # FUNCAO_____________________________
        def generate_dict(contas):
            d = {}
            for i in contas:
                id = i['id']
                parent_id = i['pgraz_sint_conta_parent_id']
                saldo_inicial = i['pgraz_sint_saldo_inicial']
                debito = i['pgraz_sint_saldo_debitos']
                credito = i['pgraz_sint_saldo_creditos']
                saldo_final = i['pgraz_sint_saldo_final']
                d[id] = {
                    'parent_id': parent_id,
                    'saldo_inicial': saldo_inicial,
                    'debito': debito,
                    'credito': credito,
                    'saldo_final': saldo_final
                }
            return d

        sinteticas = generate_dict(contas_sinteticas)
        analiticas = generate_dict(contas_analiticas)

        for i in analiticas.values():
            parent_id = i['parent_id']
            saldo_inicial = i['saldo_inicial']
            debito = i['debito']
            credito = i['credito']
            saldo_final = i['saldo_final']

            sinteticas[parent_id]['saldo_inicial'] += saldo_inicial
            sinteticas[parent_id]['debito'] += debito
            sinteticas[parent_id]['credito'] += credito
            sinteticas[parent_id]['saldo_final'] += saldo_final

        # FUNCAO_____________________________
        def atualiza_parent(saldo_inicial, debito, credito, saldo_final, parent_id):
            pai = sinteticas[parent_id]['parent_id']
            sinteticas[parent_id]['saldo_inicial'] += saldo_inicial
            sinteticas[parent_id]['debito'] += debito
            sinteticas[parent_id]['credito'] += credito
            sinteticas[parent_id]['saldo_final'] += saldo_final
            if not pai:
                return 0
            else:
                atualiza_parent(saldo_inicial, debito, credito, saldo_final, pai)

        for i in filter(lambda c: c['parent_id'], sinteticas.values()):
            parent_id = i['parent_id']
            saldo_inicial = i['saldo_inicial'] or 0
            debito = i['debito']
            credito = i['credito']
            saldo_final = i['saldo_final']
            atualiza_parent(saldo_inicial, debito, credito, saldo_final, parent_id)

        return sinteticas

    def format_thousand(self, x):
        valor = str(x)
        valor = valor.replace('.', ',')
        res = re_sub(r'(\d{3})(?=\d)', r'\1.', valor[::-1])[::-1]
        real, centavos = res.split(',')
        return '%s,%s' % (real, centavos.ljust(2, '0'))

    def listar_movimentos(self, listagem, cr, uid, company_id, mes, ano, natureza, context=None):
        context = context and context.copy() or {}
        col_conta_nome = self.col_conta_nome
        col_saldo_anterior = self.col_saldo_anterior
        col_debitos = self.col_debitos
        col_creditos = self.cols_creditos
        col_saldo_atual = self.col_saldo_atual

        grau = '  '
        # FUNCTIONS ______
        to_thousand = self.format_thousand
        funct_imprime_linha = listagem.imprimeLinha

        conta_dados = self.get_movimento_data(cr, uid, company_id, mes, ano, natureza, context={})
        sinteticas = self.totalizar_contas_sinteticas(conta_dados)

        for i in conta_dados:

            CONTA_ID = i['id']
            CONTA_TIPO = i['pgraz_sint_conta_tipo']
            CONTA_NUMERO = i['pgraz_sint_conta_numero']
            CONTA_NAME = i['pgraz_sint_conta_name']
            CONTA_GRAU = i['pgraz_sint_conta_grau']
            if CONTA_TIPO == 'A':
                SALDO_INICIAL = i['pgraz_sint_saldo_inicial']
                SALDO_DEBITOS = i['pgraz_sint_saldo_debitos']
                SALDO_CREDITOS = i['pgraz_sint_saldo_creditos']
                SALDO_FINAL = i['pgraz_sint_saldo_final']
            else:
                SALDO_INICIAL = sinteticas[CONTA_ID]['saldo_inicial']
                SALDO_DEBITOS = sinteticas[CONTA_ID]['debito']
                SALDO_CREDITOS = sinteticas[CONTA_ID]['credito']
                SALDO_FINAL = sinteticas[CONTA_ID]['saldo_final']

            conta_numero_nome = '%s  %s' % (CONTA_NUMERO, CONTA_NAME)

            str_conta_numero_nome = formataString(conta_numero_nome, col_conta_nome - len(grau * CONTA_GRAU), ' ', 'D',
                                                  'S', 'S')
            str_saldo_inicial = formataString(to_thousand(SALDO_INICIAL), col_saldo_anterior, ' ', 'E', 'S', 'S')
            str_saldo_debitos = formataString(to_thousand(SALDO_DEBITOS), col_debitos, ' ', 'E', 'S', 'S')
            str_saldo_creditos = formataString(to_thousand(SALDO_CREDITOS), col_creditos, ' ', 'E', 'S', 'S')
            str_saldo_final = formataString(to_thousand(SALDO_FINAL), col_saldo_atual, ' ', 'E', 'S', 'S')

            str_conta_numero_nome = (grau * CONTA_GRAU) + str_conta_numero_nome

            if CONTA_TIPO == 'S':
                funct_imprime_linha(self.space)
            if not context.get('balancete') or CONTA_TIPO == 'S':
                funct_imprime_linha(
                    ' ' + str_conta_numero_nome + ' ' + str_saldo_inicial + ' ' + str_saldo_debitos + ' ' + str_saldo_creditos + ' ' + str_saldo_final + ' ')

        return True


report_contabil_razao_sintetico('report.%s' % NOME_ARQUIVO_DEFAULT)
