# -*- encoding: utf-8 -*-
from openerp.osv import osv


class fiscal_sped(osv.TransientModel):
    _inherit = 'fiscal.sped'
    contabil_conta_ids = list()

    def sped_get_conta_contabil(self, cr, uid, nf_id=False, movimento_item_id=False,
                                natureza_id=False, cfop_id=False, context=None):
        """
        Encontra a conta contabil analitica para a natureza da nota fiscal (nf_id), movimento do
        item no estoque (movimento_item_id), da natureza da emissao (natureza_id) ou do CFOP
        do item movimentado, informados.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_id: int
        :param movimento_item_id: id da tabela estoque_movimento_itens, identificador do item
        movimentado no estoque.
        :type movimento_item_id: int
        :param natureza_id: id da tabela fiscal_cfop_natureza, identificador da natureza da emissao
        da nota fiscal.
        :type natureza_id: int
        :param cfop_id: id da tabela fiscal_cfop, idetificador do CFOP.
        :type cfop_id: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: codigo da conta contabil.
        :rtype: str
        """
        context = context and context.copy() or {}
        if context.get('sped_contribuicoes_agrupado') and self.contabil_conta_ids:
            return self.pool.get('contabil.conta').read(cr, uid, self.contabil_conta_ids[0],
                                                        ['conta'], context).get('conta') or ''
        if not any([nf_id, movimento_item_id, natureza_id, cfop_id]):
            return str()
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        ir_model_ids = self.pool.get('ir.model').search(cr, uid, [('name', '=', 'contabil.conta')],
                                                        context=context)
        empresa_objeto_ids = obj_adm_empresa_objeto.search(
            cr, uid, [('ir_model_id', 'in', ir_model_ids)], context=context)
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'contabil.conta', context=context)
        and_company = str()
        if empresa_objeto_ids:
            and_company = """AND cc.company_id = %s""" % company_id
        join_nf = str()
        and_nf = str()
        if nf_id:
            join_nf = """LEFT JOIN fiscal_nf_capa AS fnc  ON fcn.id = fnc.natureza_id"""
            and_nf = """AND fnc.id = %s""" % nf_id
            if not and_company:
                and_company = """AND cc.company_id = fnc.company_id"""
        join_movimento = str()
        and_movimento = str()
        if movimento_item_id:
            join_movimento = """LEFT JOIN estoque_movimento_itens AS emi ON fcn.id = emi.natureza_id
            """
            and_movimento = """AND emi.id = %s""" % movimento_item_id
            if not and_company:
                and_company = """AND cc.company_id IN (SELECT company_id
                                                       FROM estoque_movimento_capa
                                                       WHERE id = emi.capa_id)"""
        and_natureza = str()
        if natureza_id:
            and_natureza = """AND fcn.id = %s""" % natureza_id
        and_cfop = campo_cfop_id = str()
        if cfop_id:
            if not isinstance(cfop_id, list):
                cfop_id = [cfop_id]
            and_cfop = """AND fcn.cfop_id IN (%s)""" % ','.join(map(lambda x: '%s' % x, cfop_id))
            campo_cfop_id = 'fcn.cfop_id,'
            if not and_company:
                and_company = """AND cc.company_id = %s""" % company_id
        sql = """
            SELECT cc.conta,
                   cc.id,
                   {campo_cfop_id}
                   cc.company_id
            FROM contabil_conta                 AS cc
            INNER JOIN contabil_natureza_fiscal AS cnf ON cc.id = cnf.ctbnfi_conta_debito_id
            INNER JOIN fiscal_cfop_natureza     AS fcn ON cnf.ctbnfi_natureza_id = fcn.id
            {join_nf}
            {join_movimento}
            WHERE cnf.ctbnfi_contabilizar IS TRUE
              {and_cfop}
              {and_nf}
              {and_movimento}
              {and_natureza}
              {and_company}
            LIMIT 1
        """.format(join_nf=join_nf, and_nf=and_nf, join_movimento=join_movimento,
                   and_movimento=and_movimento, and_natureza=and_natureza, and_cfop=and_cfop,
                   and_company=and_company,
                   campo_cfop_id=campo_cfop_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        res = {}
        for i in dados:
            if i.get('id') not in self.contabil_conta_ids:
                self.contabil_conta_ids.append(i.get('id'))

            # este retorno como dict foi criado para melhorar performance no sped, fazendo para
            # mais de 1 cfop por vez
            if not movimento_item_id and cfop_id:
                res = {}.fromkeys(cfop_id, self.pool.get('contabil.conta').contabil_formatar_conta(
                    cr, uid, i.get('conta'), i.get('company_id'), context=context))
            else:
                res = self.pool.get('contabil.conta').contabil_formatar_conta(
                    cr, uid, i.get('conta'), i.get('company_id'), context=context)

        return res or ''
