# -*- encoding: utf-8 -*-
##############################################################################
#   SPED CONTRIBUICOES
##############################################################################
from openerp.osv import osv
from pythonsped.contribuicoes import bloco_0
from dateutil.parser import parse


class fiscal_sped_contribuicoes(osv.TransientModel):
    _inherit = 'fiscal.sped.contribuicoes'
    contabil_conta_ids = list()
    contabil_conta_nf_e_movimento = {}
    contabil_conta_natureza = {}
    contabil_conta_cfop = {}

    def sped_get_conta_contabil(self, cr, uid, nf_id=False, movimento_item_id=False,
                                natureza_id=False, cfop_id=False, context=None):
        """
        Encontra a conta contabil analitica para a natureza da nota fiscal (nf_id), movimento do
        item no estoque (movimento_item_id), da natureza da emissao (natureza_id) ou do CFOP
        do item movimentado, informados.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_id: int
        :param movimento_item_id: id da tabela estoque_movimento_itens, identificador do item
        movimentado no estoque.
        :type movimento_item_id: int
        :param natureza_id: id da tabela fiscal_cfop_natureza, identificador da natureza da emissao
        da nota fiscal.
        :type natureza_id: int
        :param cfop_id: id da tabela fiscal_cfop, idetificador do CFOP.
        :type cfop_id: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: codigo da conta contabil.
        :rtype: str
        """
        context = context and context.copy() or {}
        res = ''
        if context.get('sped_contribuicoes_agrupado') and self.contabil_conta_ids:
            return self.pool.get('contabil.conta').read(cr, uid, self.contabil_conta_ids[0],
                                                        ['conta'], context).get('conta') or ''
        if not any([nf_id, movimento_item_id, natureza_id, cfop_id]):
            return res

        if nf_id:
            res = self.contabil_conta_nf_e_movimento.get('nf').get(nf_id) or ''
        if movimento_item_id:
            res = self.contabil_conta_nf_e_movimento.get('movimento').get(movimento_item_id) or ''
        if natureza_id:
            res = self.contabil_conta_natureza.get(natureza_id) or ''
        if cfop_id:
            res = self.contabil_conta_cfop.get(cfop_id) or ''
        return res

    def sped_get_conta_contabil_nf_e_movimento(self, cr, uid, nf_ids, context=None):
        """
        Encontra a conta contabil analitica para a natureza das notas fiscais (nf_ids) e movimento
        do item no estoque.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_ids: ids da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_ids: list
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        if context.get('sped_contribuicoes_agrupado') and self.contabil_conta_ids:
            return self.pool.get('contabil.conta').read(cr, uid, self.contabil_conta_ids[0],
                                                        ['conta'], context).get('conta') or ''
        if not nf_ids:
            return {}
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        ir_model_ids = self.pool.get('ir.model').search(cr, uid, [('name', '=', 'contabil.conta')],
                                                        context=context)
        empresa_objeto_ids = obj_adm_empresa_objeto.search(
            cr, uid, [('ir_model_id', 'in', ir_model_ids)], context=context)
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'contabil.conta', context=context)
        and_company = str()
        if empresa_objeto_ids:
            and_company = """AND cc.company_id = %s""" % company_id
        # if not and_company:
        #     and_company = """AND cc.company_id = fnc.company_id"""
        sql = """
            SELECT cc.conta,
                   (SELECT conta 
                    FROM contabil_conta 
                    WHERE 
                        id IN (SELECT ctbnfi_conta_debito_id 
                               FROM contabil_natureza_fiscal 
                               WHERE ctbnfi_natureza_id IN (SELECT natureza_id 
                                                            FROM estoque_movimento_itens 
                                                            WHERE id = emi.id) 
                               LIMIT 1
                               )
                    ) AS movimento_conta,
                   cc.id,
                   cc.company_id,
                   fnc.id AS nf_id,
                   emi.id AS movimento_item_id
            FROM contabil_conta                 AS cc
            INNER JOIN contabil_natureza_fiscal AS cnf ON cc.id = cnf.ctbnfi_conta_debito_id
            INNER JOIN fiscal_cfop_natureza     AS fcn ON cnf.ctbnfi_natureza_id = fcn.id
            INNER JOIN fiscal_nf_capa           AS fnc ON fcn.id = fnc.natureza_id
            LEFT JOIN estoque_movimento_itens   AS emi ON fnc.id = emi.nf_id
            WHERE 
                cnf.ctbnfi_contabilizar IS TRUE
                AND fnc.id IN ({nf_ids})
                {and_company}
        """.format(nf_ids=', '.join(map(lambda n: str(n), nf_ids)), and_company=and_company)
        cr.execute(sql)
        dados = cr.dictfetchall()
        obj_contabil_conta = self.pool.get('contabil.conta')
        mascara = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'adm', [], company_id=company_id, context=context).get(
            'admpar_ctb_mascara_conta') or ''
        res_movimento = {}
        res_nf = {}
        for dado in dados:
            if dado.get('id') not in self.contabil_conta_ids:
                self.contabil_conta_ids.append(dado.get('id'))
            conta = dado.get('conta') or ''
            if mascara:
                conta = obj_contabil_conta.contabil_formatar_conta(
                    cr, uid, conta, dado.get('company_id'), mascara, context)
            movimento_conta = dado.get('movimento_conta') or ''
            if mascara:
                movimento_conta = obj_contabil_conta.contabil_formatar_conta(
                    cr, uid, movimento_conta, dado.get('company_id'), mascara, context)
            if dado.get('movimento_item_id'):
                res_movimento.update({dado.get('movimento_item_id'): movimento_conta})
            if dado.get('nf_id'):
                res_nf.update({dado.get('nf_id'): conta})
        self.contabil_conta_nf_e_movimento.update({'nf': res_nf, 'movimento': res_movimento})
        return True

    def sped_get_conta_contabil_natureza(self, cr, uid, context=None):
        """
        Encontra a conta contabil analitica para a natureza da emissao (natureza_id).
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        if context.get('sped_contribuicoes_agrupado') and self.contabil_conta_ids:
            return self.pool.get('contabil.conta').read(cr, uid, self.contabil_conta_ids[0],
                                                        ['conta'], context).get('conta') or ''
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'contabil.conta', context=context)
        sql = """
            SELECT cc.conta,
                   cc.id,
                   fcn.id AS natureza_id,
                   cc.company_id
            FROM contabil_conta                 AS cc
            INNER JOIN contabil_natureza_fiscal AS cnf ON cc.id = cnf.ctbnfi_conta_debito_id
            INNER JOIN fiscal_cfop_natureza     AS fcn ON cnf.ctbnfi_natureza_id = fcn.id
            WHERE cnf.ctbnfi_contabilizar IS TRUE
                AND cc.company_id = {company_id}
        """.format(company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        obj_contabil_conta = self.pool.get('contabil.conta')
        mascara = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'adm', [], company_id=company_id, context=context).get(
            'admpar_ctb_mascara_conta') or ''
        res = {}
        for dado in dados:
            if dado.get('id') not in self.contabil_conta_ids:
                self.contabil_conta_ids.append(dado.get('id'))
            if not dado.get('natureza_id'):
                continue
            conta = dado.get('conta') or ''
            if mascara:
                conta = obj_contabil_conta.contabil_formatar_conta(
                    cr, uid, conta, dado.get('company_id'), mascara, context)
            res.update({dado.get('natureza_id'): conta})
        self.contabil_conta_natureza.update(res)
        return True

    def sped_get_conta_contabil_cfop(self, cr, uid, context=None):
        """
        Encontra a conta contabil analitica para os CFOPs
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        if context.get('sped_contribuicoes_agrupado') and self.contabil_conta_ids:
            return self.pool.get('contabil.conta').read(cr, uid, self.contabil_conta_ids[0],
                                                        ['conta'], context).get('conta') or ''
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'contabil.conta', context=context)
        sql = """
            SELECT cc.conta,
                   cc.id,
                   fc.id AS cfop_id,
                   cc.company_id
            FROM contabil_conta                 AS cc
            INNER JOIN contabil_natureza_fiscal AS cnf ON cc.id = cnf.ctbnfi_conta_debito_id
            INNER JOIN fiscal_cfop_natureza     AS fcn ON cnf.ctbnfi_natureza_id = fcn.id
            INNER JOIN fiscal_cfop              AS fc  ON fc.id = fcn.cfop_id
            WHERE cnf.ctbnfi_contabilizar IS TRUE
              AND cc.company_id = {company_id}
        """.format(company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        obj_contabil_conta = self.pool.get('contabil.conta')
        mascara = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'adm', [], company_id=company_id, context=context).get(
            'admpar_ctb_mascara_conta') or ''
        res = {}
        for dado in dados:
            if dado.get('id') not in self.contabil_conta_ids:
                self.contabil_conta_ids.append(dado.get('id'))
            if not dado.get('cfop_id'):
                continue
            conta = dado.get('conta') or ''
            if mascara:
                conta = obj_contabil_conta.contabil_formatar_conta(
                    cr, uid, conta, dado.get('company_id'), mascara, context)
            res.update({dado.get('cfop_id'): conta})
        self.contabil_conta_cfop.update(res)
        return True

    def sped_gera_registro_0500(self, cr, uid, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Cria os registros 0500 para a apuração do periodo.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        brw_contabil_conta = self.pool.get('contabil.conta').browse(
            cr, uid, self.pool.get('contabil.conta').search(cr, uid, [], context=context), context)
        lista_data = list()
        lista_conta = list()
        if brw_contabil_conta:
            reg_0500 = bloco_0.registro_0500()
        for conta in brw_contabil_conta:
            data_criacao_alteracao = parse(conta.write_date or conta.create_date)
            data_criacao_alteracao = obj_adm_utils.admutl_datetime_to_timezone(
                cr, uid, data_criacao_alteracao, '%d%m%Y')
            if data_criacao_alteracao in lista_data and conta.conta in lista_conta:
                continue
            lista_data.append(data_criacao_alteracao)
            lista_conta.append(conta.conta)
            if conta.natureza == 'A':
                cod_nat_cc = '01'
            elif conta.natureza == 'P':
                cod_nat_cc = '02'
            elif conta.natureza == 'L':
                cod_nat_cc = '03'
            elif conta.natureza == 'R':
                cod_nat_cc = '04'
            elif conta.natureza == 'C':
                cod_nat_cc = '05'
            else:
                cod_nat_cc = '09'
            reg_0500.REG = '0500'
            reg_0500.DT_ALT = data_criacao_alteracao
            reg_0500.COD_NAT_CC = cod_nat_cc
            reg_0500.IND_CTA = conta.conta_tipo
            reg_0500.NIVEL = conta.grau
            reg_0500.COD_CTA = conta.conta
            reg_0500.NOME_CTA = conta.name
            reg_0500.COD_CTA_REF = ''
            reg_0500.CNPJ_EST = ''
            reg_0500.write_line()
        self.contabil_conta_ids = list()

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
