# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta


class contratos_contrato_item(osv.Model):

    """
    Tabela de itens dos contratos
    prefixo: cttite_
    """

    _name = 'contratos.contrato.item'
    _inherit = 'contratos.contrato.item'

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(contratos_contrato_item, self).default_get(cr, uid, fields_list, context)
        if context.get('cttite_state_capa'):
            res['cttite_state_contrato'] = context.get('cttite_state_capa')
        return res

    def cttite_button_tratar_item_contrato(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('cttite_data_cancelamento'):
            capa_id = self.browse(cr, uid, ids[0]).cttite_capa_id.id
            contrato_item_ids = self.search(cr, uid, [('cttite_capa_id', '=', capa_id),
                                                      ('cttite_cancelado', '!=', True)], context=context)
            if len(contrato_item_ids) == 1:
                raise osv.except_osv(_('Atencao'), _('Para cancelamento de todos os itens, cancele '
                                                     'o proprio contrato.'))
        return {'name': 'Novo prazo',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wiz.tratar.item.contrato',
                'view_id': False,
                'target': 'new',
                'type': 'ir.actions.act_window',
                'context': context,
                }

    def cttite_button_incluir_item_contrato(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_contrato_prazo = self.pool.get('contratos.prazos')
        brw_contrato_item = self.browse(cr, uid, ids[0], context)
        data_inicio = brw_contrato_item.cttite_data_inicio
        data_inicio1 = parse(data_inicio, yearfirst=True) + relativedelta(days=+1)
        data_inicio1 = data_inicio1.strftime('%Y-%m-%d')
        prazo_ativo_ids = obj_contrato_prazo.search(
            cr, uid, [('cttprz_data_inicio', '<=', data_inicio1),
                      ('cttprz_data_termino', '>=', data_inicio1),
                      ('cttprz_contrato_id', '=', brw_contrato_item.cttite_capa_id.id),
                      ('cttprz_real_provisao', '=', 'R')], context=context)
        if not prazo_ativo_ids:
            raise osv.except_osv(_('Atencao'), _('Data do item esta fora da faixa do prazo '
                                                 'em vigor!'))
        context['cttite_data_inicio'] = data_inicio
        return self.cttite_button_tratar_item_contrato(cr, uid, ids, context)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        brw_contrato_item = self.browse(cr, uid, ids, context)
        for item in brw_contrato_item:
            produto = item.cttite_produto_id
            codigo = str(produto.codigo or '')
            name = produto.name or '*** Produto sem Descricao ***'
            if context.get('retorno_sem_codigo'):
                codigo = ''
            if context.get('estpro_so_codigo'):
                result = codigo
            else:
                result = "%s%s%s" % (codigo, ' - ' if codigo else '', name)
            res.append((item.id, result))
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if 'cttite_carencia' in vals and vals.get('cttite_carencia') < 0:
            raise osv.except_osv(_('Atencao'), _('Informe um valor valido para a carencia'))
        if vals.get('cttite_carencia'):
            vals['cttite_carencia_considerar'] = vals.get('cttite_carencia')
        res = super(contratos_contrato_item, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'cttite_carencia' in vals and vals.get('cttite_carencia') < 0:
            raise osv.except_osv(_('Atencao'), _('Informe um valor valido para a carencia'))
        if 'cttite_carencia' in vals:
            vals['cttite_carencia_considerar'] = vals.get('cttite_carencia')
        res = super(contratos_contrato_item, self).write(cr, uid, ids, vals, context)
        return res
