# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2016, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


# noinspection PyAbstractClass,PyPep8Naming
class fiscal_nf_capa(osv.Model):
    """
    Heranca da classe fiscal_nf_capa

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _inherit = 'fiscal.nf.capa'

    def gerar_parcelas_nf_propria(self, cr, user, nf, context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento para um nota fiscal propria.
        Altera a data de vencimento das parcelas em funcao da data do contrato.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param nf: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: lista com os dados das parcelas
        :rtype: list[dict]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).gerar_parcelas_nf_propria(cr, user, nf, context)
        nf_capa_dados = self.read(cr, user, nf, ['fisnfc_contrato_id'], context, '_classic_write')

        if nf_capa_dados.get('fisnfc_contrato_id'):
            contrato_dados = self.pool.get('contratos.contrato').read(
                cr, user, nf_capa_dados.get('fisnfc_contrato_id'), ['sabado', 'domingo', 'dia'],
                context)
            vencimento = self.pool.get('contratos.contrato').cttctr_get_data_vencimento(
                contrato_dados.get('dia'), contrato_dados.get('sabado'),
                contrato_dados.get('domingo'))

            # Ajusta o vencimento em funcao da data do contrato
            for parcela in result:
                parcela['vencimento'] = vencimento

        return result

    def fisnfc_verificar_nota_fiscal(self, cr, user, nf, context):
        """
        Realiza a verificacao de uma nota fiscal.
        Atualiza o contexto com o ID do contrato (para compatibilizacao)

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota fiscal foi verificada sem erro ou inconsistencia.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        contrato_id = self.browse(cr, user, nf, context).fisnfc_contrato_id
        contrato_id = contrato_id and self.browse(cr, user, nf, context).fisnfc_contrato_id.id
        context.update({'custifica_contrato': contrato_id or False})

        return super(fiscal_nf_capa, self).fisnfc_verificar_nota_fiscal(cr, user, nf, context)

    def check_condicao_total_parcelas(self, cr, user, dict_condicao, dict_cond_forma,
                                               dict_adm_cond, context=None):
        """
        Nao realiza a verificacao das parcelas se houver contrato.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param condicao: lista com o ID das condicoes de pagamento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type condicao: long
        :type context: dict
        :return: verdadeiro se nao houver contrato (contrato_id)
        :rtype: bool
        """
        context = context and context.copy() or {}

        if not dict_condicao:
            return super(fiscal_nf_capa, self).check_condicao_total_parcelas(
                cr, user, dict_condicao, dict_cond_forma, dict_adm_cond, context)

        # Recupera o ID da nota fiscal das condicoes
        nf_id = dict_condicao.values()[0].get('capa_id')

        contrato_id = self.read(cr, user, nf_id, ['fisnfc_contrato_id'], context).\
                                                                           get('fisnfc_contrato_id')
        if contrato_id:
            return True

        return super(fiscal_nf_capa, self).check_condicao_total_parcelas(
            cr, user, dict_condicao, dict_cond_forma, dict_adm_cond, context)

    def fisnfc_verificar_parcelas(self, cr, user, dados_nf, dict_parcelas, dict_cfop_itens,
                                  context=None):
        """
        Verifica divergencias nas parcelas.
        Nao realiza a verificacao se existir contrato (contrato_id)

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao houver divergencia.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if dados_nf.get('contrato_id'):
            return True

        return super(fiscal_nf_capa, self).fisnfc_verificar_parcelas(cr, user, dados_nf,
                                                            dict_parcelas, dict_cfop_itens, context)

    def fisnfc_verificar_condicao_pagamento(self, cr, user, dados_nf, dict_condicao,
                                            dict_forma_pgto, context=None):
        """
        Verifica divergencias nas condicoes de pagamento.
        Nao realiza a verificacao se existir contrato (contrato_id)

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao houver divergencia.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if self.read(cr, user, dados_nf.get('id'), ['contrato_id'], context,
                     '_classic_write').get('contrato_id'):
            return True

        return super(fiscal_nf_capa, self).fisnfc_verificar_condicao_pagamento(cr, user, dados_nf,
                                                           dict_condicao, dict_forma_pgto, context)

    def fisnfc_cria_titulos_prv_financeiro(self, cr, user, dados_nf, dict_forma_pgto,
                                           dict_cond_forma, dict_adm_cond, fatura, pessoa_id,
                                           dados_condicao=False, dict_condicao={}, dict_parcelas={},
                                           context=None):
        """
        Cria titulo em previsao no financeiro.
        Nao cria o titulo se existir contrato (contrato_id).

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param fatura: ID da fatura
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type fatura: long
        :type context: dict
        :return: o ID do titulo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        nf_id = self.pool.get('financeiro.fatura').read(cr, user, fatura, ['nf_id'], context=context,
                                                load='_classic_write').get('nf_id')
        if self.read(cr, user, nf_id, ['contrato_id'], context, '_classic_write').\
                get('contrato_id'):
            return False

        return super(fiscal_nf_capa, self).fisnfc_cria_titulos_prv_financeiro(cr, user, dados_nf,
                              dict_forma_pgto, dict_cond_forma, dict_adm_cond, fatura, pessoa_id,
                                                                      dados_condicao=dados_condicao,
                                                                      dict_condicao=dict_condicao,
                                                                      dict_parcelas=dict_parcelas,
                                                                      context=context)

    def fisnfc_criar_titulo(self, cr, user, nf_id, fisnfc_vals_titulo, context=None):
        """
        Inclui o ID do contrado no titulo criado.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param fisnfc_vals_titulo: valores do titulo a ser criado
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type fisnfc_vals_titulo: dict
        :type context: dict
        :return: o ID do titulo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        fisnfc_vals_titulo['contrato_id'] = self.read(cr, user, nf_id, ['fisnfc_contrato_id'], context,
                                               '_classic_write').get('fisnfc_contrato_id') or False

        return super(fiscal_nf_capa, self).fisnfc_criar_titulo(cr, user, nf_id, fisnfc_vals_titulo,
                                                               context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
