# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import cttctr_get_flag_gerar_nf
from funcoes import _cttctr_gerar_numero_automatico, _cttctr_get_vencimento,\
    _cttprz_get_valor_contrato, cttctr_get_titulos_faturados
from funcoes import cttctr_gera_centro_custo_evento


class contratos_contrato(osv.Model):
    """
    Tabela de Contratos

    prefixo: cttctr_
    """

    _name = 'contratos.contrato'
    _rec_name = 'numero'
    _roteavel = True
    _inherit = ['ir.needaction_mixin', 'mail.thread']
    _columns = {
        # ____________________________ integer ____________________________
        'id': fields.integer('Id', readonly=True),  # Numero identificador do registro na tabela
        'dia': fields.integer('Dia Vencimento Parcelas'), # Dia do vencimento das parcelas do contrato
        'prazo_medio': fields.integer('Prazo medio'),
        # Se 'atualiza_valor'= True, informar o prazo para calculo da media.\nValor mńimo e 1
        'dia_semana': fields.integer('Dia semana'), # Armazena qual dia deve ocorrer o evento.

        # ____________________________ float ______________________________

        # ____________________________ char _______________________________
        'numero': fields.char('Numero', size=15, required=False, select=True), # Numero do Contrato

        # ____________________________ boolean ____________________________
        'cttctr_controla_nf': fields.boolean('Gerar NF'),  # Indica se o contrato ira ou nao gerar
                                                           # nota fiscal
        'antecipa_feriado': fields.boolean('Antecipa feriado'), # True=Antecipa Feriado, \nFalse=Nao antecipa feriado
        'sabado': fields.boolean('Sabado', help="Marcado: Vencimento nao ocorre aos sabados; Desmarcado: "
                                                "Vencimento pode ocorrer aos sabados."),
        'domingo': fields.boolean('Domingo', help="Marcado: Vencimento nao ocorre aos domingos; Desmarcado: "
                                                  "Vencimento pode ocorrer aos domingos."),
        'atualiza_valor': fields.boolean('Atualiza valor'),
        'cttctr_renova_contrato': fields.boolean('Contrato renovavel'),
        # Este flag controla o fato de Atualizar ou nao o valor no contrato sempre que lancar uma
        # parcela. \nTrue=Sim , \nFalse=Nao
        'cttctr_competencia_por_parcelas': fields.boolean('Competencia por parcelas'),

        #______________________________date____________________________________
        'cttctr_data_limite_atendimento': fields.date('Data limite atendimento'),

        # ____________________________ function _______________________________
        'cttctr_vencimento': fields.function(_cttctr_get_vencimento, method=True,
                                             string='Vencimento', type='date', store=True),
        # Armazena a data do ultimo vencimento dos prazos
        'cttctr_flag_gerar_nf': fields.function(
            cttctr_get_flag_gerar_nf, method=True, type="boolean", string="Flag Button Gerar NF", store=False
        ),
        'cttctr_gerar_numero_automatico': fields.function(
            _cttctr_gerar_numero_automatico, method=True, string='Numero automatico',
            type='boolean', store=False),  # Verifica se o parametro numero_automatico_contrato esta
                                           # marcado no financeiro
                                           # Armazena a data do ultimo vencimento dos prazos
        'cttctr_titulos_faturados': fields.function(cttctr_get_titulos_faturados, method=True, type="boolean", string="Todos os titulos faturados",
                                                store=True
                                           ),

        'cttctr_valor_contrato': fields.function(_cttprz_get_valor_contrato, method=True,
                                                 string='Valor', type='float', digits=(11, 2),
                                                 store=False),
        'cttctr_gera_centro_custo_evento': fields.function(
            cttctr_gera_centro_custo_evento, method=True, string='Gera centro de custos e eventos',
            type='boolean', store=False),
        # ____________________________ selection __________________________

        'state': fields.selection(
            [('R', 'Esboco'), ('A', 'Ativo'), ('F', 'Finalizado'), ('S', 'Suspenso'), ('C', 'Cancelado')], 'Status',
            size=1
        ),
        'tipo': fields.selection(
            [('R', 'Receber'), ('P', 'Pagar')], 'Receber/Pagar', required=True, size=1
        ),
        'periodo': fields.selection(
            [('D', 'Diario'), ('S', 'Semanal'), ('Q', 'Quinzenal'), ('M', 'Mensal')], 'Periodo', size=1
        ),
        'tipo_dia': fields.selection(
            [('D', 'Dia Direto'), ('U', 'Dia Util')], 'Tipo dia', size=1
        ),
        'cttctr_tipo_contrato': fields.selection(
            [('M', 'Mercadoria'), ('S', 'Servico')], 'Servico/Mercadoria', size=1, required=True,
        ),

        # ____________________________ many2one ___________________________
        'pessoa_id': fields.many2one("adm.pessoa", 'Pessoa', ondelete="restrict", required=True, select=True),
        # Pessoa com que foi firmado o contrato cliente/Fornecedor/Transportador, etc...
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True),
        # id da empresa
        'cttctr_servico_id': fields.many2one('estoque.produto', 'Servico para NF',
                                             domain="[('tipo', '=', 'S')]",
                                             help='Se nao for informado este servico, a NF sera '
                                                  'emitida com os servicos dos itens'),
        'cttctr_tipo_id': fields.many2one('contratos.tipo', 'Tipo de contrato'),
        'cttctr_forma_pagamento_id': fields.many2one('adm.forma.pgto', 'Forma Pagamento',
                                                     ondelete="restrict"),

        # ____________________________ one2many ___________________________
        'fornecedor_capa_ids': fields.one2many('contratos.fornecedor.capa', 'contrato_id', 'Capa'),
        'titulos_ids': fields.one2many('financeiro.titulo', 'contrato_id', 'Titulos'),
        'cttctr_nf_ids': fields.one2many('fiscal.nf.capa', 'fisnfc_contrato_id', 'Notas Fiscais'),
        'cttctr_prazos_ids': fields.one2many('contratos.prazos', 'cttprz_contrato_id', 'Prazos'),
        'cttctr_item_ids': fields.one2many('contratos.contrato.item', 'cttite_capa_id', 'Itens'),
    }
    _defaults = {
        'prazo_medio': lambda *a: 1,
        'state': lambda *a: 'R',
        'periodo': lambda *a: 'M',
        'tipo_dia': lambda *a: 'D',
        'antecipa_feriado': lambda *a: False,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                                        s._name, context=c),
        'cttctr_controla_nf': lambda *a: False,
        'cttctr_renova_contrato': lambda *a: False,
    }

    _sql_constraints = [
        ('numero_uniq', 'unique (numero)', 'O numero precisa ser unico!'),
    ]
