# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
import time


# Funcoes da classe contratos_contratos
def cttctr_gera_centro_custo_evento(self, cr, uid, ids, field_name, args, context=None):
    """
    Funcao de campo funcional (function). Verifica nos parametros do financeiro se a empresa usa
    centro de custos e eventos.
    :param cr: cursor no banco de dados
    :param uid: id da tabela res_users, identificador do usuario
    :param ids: lista de id da tabela contratos_contrato
    :param field_name: string com o nome do campo
    :param args: lista de argumentos de pesquisa
    :param context: didcionario com valores contextuais
    :return: dicionario. Para cada chave um id, da lista recebida como parametro (ids), e seu valor
    um booleano com: - True, usa centro de custos e eventos / - False, nao usa centro de custos e
    eventos
    """
    context = context and context.copy() or {}
    res = {}
    financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
        cr, uid, 'financeiro', 'usar_centro_custos_e_eventos', context=context)
    usa_centro_custo_evento = financeiro_parametro_dados.get('usar_centro_custos_e_eventos')
    map(lambda i: res.update({i.id: usa_centro_custo_evento}), self.browse(cr, uid, ids, context=context))
    return res


def cttctr_get_flag_gerar_nf(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    res = {}.fromkeys(ids, False)
    contrato_dados = self.read(cr, uid, ids, ['state'], context=context)
    contratos_ativos_gerar_nf = filter(lambda i: i.get('state') == 'A', contrato_dados)
    if contratos_ativos_gerar_nf:
        contrato_ids = map(lambda i: i['id'], contratos_ativos_gerar_nf)
        res.update({}.fromkeys(contrato_ids, True))
    return res


def cttctr_get_titulos_faturados(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    browse_contratos= self.browse(cr, uid, ids)
    faturado = True
    for x in browse_contratos:
        for t in x.titulos_ids:
            if t.doc_tipo_id.sigla == 'PRV':
                faturado = False
        res[x.id] = faturado

    return res

def _cttctr_gerar_numero_automatico(self, cr, uid, ids, field_name, args, context=None):

    """
    Verifica nos parametros do financeiro (atributo 'numero_automatico_contrato') se o numero do
    contrato eh para ser gerado de forma automatica ou nao.
    @return: Dicionario, a chave representa o id do registro e seu valor sera igual a True ou False
    """

    context = context and context.copy() or {}
    res = {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    obj_adm_utils = self.pool.get('adm.utils')
    financeiro_parametro = obj_adm_utils.admutl_get_parametros(
        cr, uid, 'financeiro', 'numero_automatico_contrato', context=context)
    gera_numero_automatico = financeiro_parametro.get('numero_automatico_contrato')
    brw_contratos_contrato = self.browse(cr, uid, ids)
    for contrato in brw_contratos_contrato:
        res[contrato.id] = gera_numero_automatico
    return res


def _cttctr_get_vencimento(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    obj_contrato_prazo = self.pool.get('contratos.prazos')
    brw_contratos_contrato = self.browse(cr, uid, ids)
    for contrato in brw_contratos_contrato:
        prazo_ids = obj_contrato_prazo.search(
            cr, uid, [('cttprz_contrato_id', '=', contrato.id), ('cttprz_real_provisao', '=', 'R')],
            order='cttprz_data_termino DESC', limit=1, context=context)
        ultima_data_termino = False
        if prazo_ids:
            ultima_data_termino = obj_contrato_prazo.browse(
                cr, uid, prazo_ids[0]).cttprz_data_termino or False
        res[contrato.id] = ultima_data_termino
    return res


def _cttprz_get_valor_contrato(self, cr, uid, ids, field_name, args, context=None):

    """
    A funcao verifica o valor estimado do prazo virgente para data da consulta.
    @return: Dicionario, a chave representa o id do registro e seu valor estimado do prazo virgente.
    """
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]

    res = {}
    obj_contrato_prazo = self.pool.get('contratos.prazos')
    data_hoje = time.strftime('%Y-%m-%d')
    brw_contratos_contrato = self.browse(cr, uid, ids)
    for contrato in brw_contratos_contrato:
        res[contrato.id] = 0.0
        arg_prazo = [('cttprz_data_inicio', '<=', data_hoje),
                     ('cttprz_data_termino', '>=', data_hoje),
                     ('cttprz_contrato_id', '=', contrato.id)]

        prazo_ids = obj_contrato_prazo.search(cr, uid, arg_prazo, context=context)
        if prazo_ids:
            rel = obj_contrato_prazo.read(cr, uid, prazo_ids, ['cttprz_valor', 'cttprz_contrato_id'], context=context)
            if isinstance(rel, list):
                for i in rel:
                    res[contrato.id] = i.get('cttprz_valor') or 0.0
            if isinstance(rel, dict):
                res[contrato.id] = rel.get('cttprz_valor') or 0.0
    return res


#  Funcoes da classe contratos_prazos
def _cttprz_get_valor_total_prazo(self, cr, uid, ids, field_name, args, context=None):

    """
    Metodo de campo function. Ira realizar a soma do valor dos itens do contrato para um determinado
    prazo
    @return: Dicionario, a chave representa o id do registro e seu valor sera igual ao total dos
    itens
    """

    context = context and context.copy() or {}
    res = {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    brw_contrato_prazo = self.browse(cr, uid, ids)
    contrato_tipo = brw_contrato_prazo[0].cttprz_contrato_id.tipo
    for prazo in brw_contrato_prazo:
        cttprz_item_valor_ids = prazo.cttprz_item_valor_ids
        total_item_valor = 0
        for item_valor in cttprz_item_valor_ids:
            total_item_valor += item_valor.cttivl_valor or 0
        if contrato_tipo == 'R' and not total_item_valor:
            erro_contrato = _('Valor Estimado deve ser maior que zero')
            raise osv.except_osv(_('Erro ao criar o contrato'), erro_contrato)
        res[prazo.id] = total_item_valor
    return res


def _cttprz_check_contrato_validado(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    brw_contratos_prazos = self.browse(cr, uid, ids)
    for prazo in brw_contratos_prazos:
        ativo = False
        state = prazo.cttprz_contrato_id.state
        if state == 'A':
            ativo = True
        res[prazo.id] = ativo
    return res


# Funcoes da classe contratos_contrato_item
def verificar_tipo_prazo_previsao(self, cr, uid, ids, field_name, args, context=None):

    """
    A funcao verifica se o prazo em que o item estiver contido for do tipo
    previsao (P), se sim, a data de cancelamento deve estar readonly
    @return: Dicionario, sendo a chave o id do item e o valor da chave igual a True ou False
    """

    context = context and context.copy() or {}
    res = {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    obj_contrato_prazo = self.pool.get('contratos.prazos')
    brw_contrato_item = self.browse(cr, uid, ids, context)
    data_hoje = time.strftime('%Y-%m-%d')
    for item in brw_contrato_item:
        item_id = item.id
        res[item_id] = False
        contrato_id = item.cttite_capa_id.id
        arg_prazo = [('cttprz_contrato_id', '=', contrato_id),
                     ('cttprz_data_inicio', '<=', data_hoje),
                     ('cttprz_data_termino', '>=', data_hoje),
                     ('cttprz_real_provisao', '=', 'P')]
        prazo_ids = obj_contrato_prazo.search(cr, uid, arg_prazo, context=context)
        if prazo_ids:
            item_valor_ids = obj_contrato_prazo.browse(cr, uid, prazo_ids[0]).cttprz_item_valor_ids
            item_ids = map(lambda i: i.cttivl_item_id.id, item_valor_ids)
            if item_id in item_ids:
                res[item_id] = True
    return res


def cttite_get_state_contrato(self, cr, uid, ids, field_name, args, context=None):
    """
    O metodo busca o estado (state) do contrato.
    @return: Dicionario, sendo a chave o id do item e o valor da chave uma string que representa o
    estado
    """
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    res = {}
    brw_contrato_item = self.browse(cr, uid, ids)
    for item in brw_contrato_item:
        res[item.id] = item.cttite_capa_id.state
    return res


def cttite_get_titulo_gerado(self, cr, uid, ids, field_name, args, context=None):
    """
    O metodo identifica se os titulos ja foram gerados para o prazo ativo que possua o item.
    @return: Dicionario, sendo a chave o id do item e o valor da chave um booleano que representa o
    se os titulos foram gerados (True) ou nao (False).
    """
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    obj_contrato_prazo = self.pool.get('contratos.prazos')
    obj_contrato_item_valor = self.pool.get('contratos.contrato.item.valor')
    res = {}
    brw_contrato_item = self.browse(cr, uid, ids)
    for item in brw_contrato_item:
        contrato_id = item.cttite_capa_id.id
        item_id = item.id
        prazo_ids = obj_contrato_prazo.search(cr, uid, [('cttprz_contrato_id', '=', contrato_id),
                                                        ('cttprz_titulos_gerados', '=', True),
                                                        ('cttprz_real_provisao', '=', 'R')], context=context)
        if not prazo_ids:
            res[item_id] = False
        else:
            contrato_item_valor_ids = obj_contrato_item_valor.search(
                cr, uid, [('cttivl_prazo_id', 'in', prazo_ids), ('cttivl_item_id', '=', item_id)], context=context)
            if contrato_item_valor_ids:
                res[item_id] = False
            else:
                res[item_id] = True
    return res
