# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from SigUtils.datas import getDiffDatetime


class wiz_contratos_atualiza_contrato(osv.TransientModel):

    _name = 'wiz.contratos.atualiza.contrato'

    _columns = {
        'data_vencimento': fields.date('Data vencimento', required=True),
        'porcentagem_aumento': fields.float('% de aumento', digits=(3, 4)),
        'atualiza_contrato_item_ids': fields.one2many('wiz.contratos.atualiza.contrato.item',
                                                      'atualiza_contrato_id', 'Contratos'),
        'contrato_atualizado': fields.selection([('S', 'Sim'), ('N', 'Nao')], 'Atualizados',
                                                size=1),
        'pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
    }

    def button_pesquisar_contrato(self, cr, uid, ids, context=None):

        """
        Busca os contratos do tipo “A Receber”, em estado “Ativo”, renovavel e com vencimento até a
        data informada.
        """

        context = context and context.copy() or {}
        obj_contratos_contrato = self.pool.get('contratos.contrato')
        obj_wiz_contratos_atualiza_contrato_item = self.pool.get(
            'wiz.contratos.atualiza.contrato.item')
        atualiza_contrato_item_ids = obj_wiz_contratos_atualiza_contrato_item.search(cr, uid, [], context=context)
        if atualiza_contrato_item_ids:
            obj_wiz_contratos_atualiza_contrato_item.unlink(cr, uid, atualiza_contrato_item_ids)
        brw_wiz_contratos_atualiza_contrato = self.browse(cr, uid, ids)
        for x in brw_wiz_contratos_atualiza_contrato:
            args = [('cttctr_vencimento', '<=', x.data_vencimento), ('tipo', '=', 'R'),
                    ('state', '=', 'A'), ('cttctr_renova_contrato', '=', True)]
            if x.pessoa_id:
                args.append(('pessoa_id', '=', x.pessoa_id.id))
            if context.get('provisao'):
                args.append(('cttctr_tipo_contrato', '=', 'S'),)
            contrato_ids = obj_contratos_contrato.search(cr, uid, args, context=context)
            porcentagem_aumento = x.porcentagem_aumento
            brw_contratos_contrato = obj_contratos_contrato.browse(cr, uid, contrato_ids)
            for contrato in brw_contratos_contrato:
                contrato_id = contrato.id
                vencimento_contrato = contrato.cttctr_vencimento
                valor_contrato = self._get_valor_contrato(cr, uid, vencimento_contrato, contrato_id,
                                                          porcentagem_aumento, True, context)
                vals = {
                    'atualiza_contrato_id': x.id,
                    'contrato_id': contrato_id,
                    'pessoa_id': contrato.pessoa_id.id,
                    'numero': contrato.numero,
                    'vencimento': contrato.cttctr_vencimento,
                    'valor': valor_contrato.get('valor'),
                    'novo_valor': valor_contrato.get('novo_valor'),
                }
                obj_wiz_contratos_atualiza_contrato_item.create(cr, uid, vals, context)
        self.write(cr, uid, ids, {'contrato_atualizado': None}, context)
        return True

    def _get_valor_contrato(self, cr, uid, data_vencimento, contrato_id, porcentagem_aumento, total,
                            context=None):

        """
        O metdodo encontra o ultimo prazo definido para o contrato. Baseado nesse prazo, ele aplica
        o percentual de aumento em cada item e faz a soma deles. Esse resultado é o novo valor.
        @param data_vencimento: Data de vencimento do contrato
        @param contrato_id: Numero identificador do contrato na tabela contratos.contrato
        @param porcentagem_aumento: O percentual que sera aplicado em cada item daquele prazo
        @param total: Se True, retorna no dicionario, na chave 'novo_valor', a soma total dos itens.
        Se False, retorna lista de tuplas, para inserção na tabela contratos_contrato_item_valor.
        @return: Dicionario contendo as chaves 'valor' e 'novo_valor'. Elas armazenam o valor atual
        do contrato e o novo valor ja aplicado o percentual de aumento, respectivamente.
        """

        context = context and context.copy() or {}
        res = {'valor': 0, 'novo_valor': 0}
        obj_contratos_prazos = self.pool.get('contratos.prazos')
        if not data_vencimento:
            return res
        args = [('cttprz_data_termino', '=', data_vencimento),
                ('cttprz_contrato_id', '=', contrato_id)]
        prazo_id = obj_contratos_prazos.search(cr, uid, args, context=context)
        brw_contratos_prazos = obj_contratos_prazos.browse(cr, uid, prazo_id[0])
        res['valor'] = brw_contratos_prazos.cttprz_valor or 0.0
        brw_contratos_contrato_item_valor = brw_contratos_prazos.cttprz_item_valor_ids
        if porcentagem_aumento:
            if total:
                novo_valor = sum(
                    map(
                        lambda i: i.cttivl_valor + (i.cttivl_valor * (porcentagem_aumento / 100)),
                        brw_contratos_contrato_item_valor))
                res['novo_valor'] = novo_valor
            else:
                novo_valor = (map(lambda i: (0, 0, {
                    'cttivl_valor': i.cttivl_valor + (i.cttivl_valor * (porcentagem_aumento / 100)),
                    'cttivl_item_id': i.cttivl_item_id.id}),
                    brw_contratos_contrato_item_valor))
                res['novo_valor'] = novo_valor
        else:
            if total:
                novo_valor = sum(
                    map(
                        lambda i: i.cttivl_valor, brw_contratos_contrato_item_valor))
                res['novo_valor'] = novo_valor
            else:
                novo_valor = (map(lambda i: (0, 0, {
                    'cttivl_valor': i.cttivl_valor,
                    'cttivl_item_id': i.cttivl_item_id.id}),
                    brw_contratos_contrato_item_valor))
                res['novo_valor'] = novo_valor
        return res

    def button_atualizar_contrato(self, cr, uid, ids, context=None):

        """
        O metodo atualiza os contratos pesquisados. Gera novos prazos com os mesmos itens, porem,
        com o valor de cada item atualizado de acordo com o percentual de aumento. Gera novos
        titulos caso o tipo de contrato seja servico
        @return: True se os contratos foram atualizados e False se nenhum foi atualizado
        """

        context = context and context.copy() or {}
        res = False
        obj_contratos_prazos = self.pool.get('contratos.prazos')
        brw_wiz_contratos_atualiza_contrato = self.browse(cr, uid, ids, context)
        gerar_titulo_prazo_ids = []
        for x in brw_wiz_contratos_atualiza_contrato:
            porcentagem_aumento = x.porcentagem_aumento
            brw_wiz_contratos_atualiza_contrato_item = x.atualiza_contrato_item_ids
            for i in brw_wiz_contratos_atualiza_contrato_item:
                valor = i.valor or 0.0
                novo_valor = i.novo_valor or 0.0
                data_inicio = i.vencimento
                meses = 0
                contrato_id = i.contrato_id.id
                prazo_real_ids = obj_contratos_prazos.search(
                    cr, uid, [('cttprz_contrato_id', '=', contrato_id),
                              ('cttprz_real_provisao', '=', 'R')], context=context)
                if prazo_real_ids:
                    brw_contrato_prazo_real = obj_contratos_prazos.browse(
                        cr, uid, prazo_real_ids[0], context)
                    inicio = parse(brw_contrato_prazo_real.cttprz_data_inicio)
                    fim = parse(brw_contrato_prazo_real.cttprz_data_termino)
                    meses = getDiffDatetime(str(inicio), str(fim), 'M') or 1
                data_termino = parse(data_inicio) + relativedelta(months=meses)
                data_termino = data_termino.strftime('%Y-%m-%d')
                tipo_contrato = i.contrato_id.cttctr_tipo_contrato
                valores = self._get_valor_contrato(cr, uid, i.vencimento, contrato_id,
                                                   porcentagem_aumento, False, context)
                item_valor_ids = valores.get('novo_valor')
                if item_valor_ids:
                    prazo_ids = obj_contratos_prazos.search(
                        cr, uid, [('cttprz_contrato_id', '=', contrato_id),
                                  ('cttprz_real_provisao', '=', 'P')], context=context)
                    data_inicio = (parse(data_inicio) + relativedelta(days=1)).strftime('%Y-%m-%d') # Acresce um dia
                    if prazo_ids:
                        brw_contrato_prazo = obj_contratos_prazos.browse(cr, uid, prazo_ids[0])
                        data_inicio = brw_contrato_prazo.cttprz_data_inicio
                        data_termino = brw_contrato_prazo.cttprz_data_termino
                        obj_contratos_prazos.unlink(cr, uid, prazo_ids)
                    if valor != novo_valor and not porcentagem_aumento:
                        diferenca = float(novo_valor - valor) / float(len(item_valor_ids))
                        for v in item_valor_ids:
                            for c in v:
                                if isinstance(c, dict):
                                    c['cttivl_valor'] += diferenca
                    vals = {
                        'cttprz_contrato_id': contrato_id,
                        'cttprz_data_inicio': data_inicio,
                        'cttprz_data_termino': data_termino,
                        'cttprz_item_valor_ids': item_valor_ids,
                    }
                    prazo_id = obj_contratos_prazos.create(cr, uid, vals, context)
                    if prazo_id and tipo_contrato == 'S':
                        gerar_titulo_prazo_ids.append(prazo_id)
                    res = True
        if gerar_titulo_prazo_ids:
            obj_contratos_prazos.cttprz_button_gerar_titulos(cr, uid, gerar_titulo_prazo_ids,
                                                             context)
        if res:
            self.write(cr, uid, ids, {'contrato_atualizado': 'S'}, context)
        else:
            self.write(cr, uid, ids, {'contrato_atualizado': 'N'}, context)
        return res

    def button_print_contrato(self, cr, uid, ids, context=None):

        """
        Metodo responsavel por chamar a classe do relatorio que ira imprimir um arquivo pdf com
        as informacoes dos contratos pesquisados
        @return: Dicionario com acao para a classe do relatorio, os dados referentes e o nome.
        """

        context = context and context.copy() or {}
        brw_wiz_contratos_atualiza_contrato = self.browse(cr, uid, ids[0], context)
        atualiza_contrato_item_ids = brw_wiz_contratos_atualiza_contrato.atualiza_contrato_item_ids
        atualiza_contrato_item_ids = map(lambda i: i.id, atualiza_contrato_item_ids)
        datas = {
            'atualiza_contrato_item_ids': atualiza_contrato_item_ids,
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_contrato_atualizado',
            'datas': datas,
        }


class wiz_contratos_atualiza_contrato_item(osv.TransientModel):

    _name = 'wiz.contratos.atualiza.contrato.item'

    _columns = {
        'atualiza_contrato_id': fields.many2one('wiz.contratos.atualiza.contrato', 'Capa'),
        'contrato_id': fields.many2one('contratos.contrato', 'Contrato'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'numero': fields.char('Numero', size=15),
        'vencimento': fields.date('Vencimento'),
        'valor': fields.float('Valor', digits=(15, 2)),
        'novo_valor': fields.float('Novo valor', digits=(15, 2)),
    }
