# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from openerp.tools.translate import _
import time


class wiz_tratar_item_contrato(osv.TransientModel):

    _name = 'wiz.tratar.item.contrato'

    _columns = {
        'data_inicio': fields.date('Data inicio', required=True),
        'data_fim': fields.date('Data termino', required=True),
        'valor_item': fields.float('Valor item', digits=(11, 2)),
        'tipo': fields.selection([('C', 'Cancelamento'), ('I', 'Inclusao')], 'Tipo', size=1),
        'item_multiplo_contrato_ids': fields.one2many('wiz.tratar.item.multiplo.contrato',
                                                      'wiztim_capa_id', 'Itens'),
        'carencia': fields.integer('Carencia', help='Esta carencia e em meses e sera '
                                                    'considerada na geracao dos titulos'),
    }

    def button_confirmar(self, cr, uid, ids, context=None):

        """
        O metodo gera um novo prazo com um novo valor para o contrato, a partir dos novos itens
        informados.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param ids: lista de id do modelo atual
        @param context: dicionario com valores contextuais
        @return: True
        """

        context = context and context.copy() or {}
        obj_contratos_contrato_item = self.pool.get('contratos.contrato.item')
        obj_contratos_prazos = self.pool.get('contratos.prazos')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        contrato_tipo = False
        contrato_id = False
        brw_wiz_tratar_item_contrato = self.browse(cr, uid, ids)
        carencia = 0
        if context.get('cttctr_inclusao_multipla_item'):
            contrato_id = context.get('active_ids')
            if isinstance(contrato_id, list):
                contrato_id = contrato_id[0]
            contrato_tipo = self.pool.get('contratos.contrato').browse(cr, uid, contrato_id).\
                cttctr_tipo_contrato
            tipo_tratamento = 'I'
            data_tratamento = False
            carencia = brw_wiz_tratar_item_contrato[0].carencia or 0
        else:
            contrato_item_ids = context.get('active_ids') or []
            tipo_tratamento = brw_wiz_tratar_item_contrato[0].tipo
            if tipo_tratamento == 'I':
                valor_item = brw_wiz_tratar_item_contrato[0].valor_item
                if valor_item <= 0:
                    raise osv.except_osv(_('Atencao'), _('Informe um valor valido para o item'))
            brw_contratos_contrato_item = obj_contratos_contrato_item.browse(cr, uid,
                                                                             contrato_item_ids)
            data_tratamento = False
            for item in brw_contratos_contrato_item:
                contrato_id = item.cttite_capa_id
                contrato_tipo = contrato_id.cttctr_tipo_contrato
                if tipo_tratamento == 'C':
                    data_tratamento = item.cttite_data_cancelamento
                    obj_contratos_contrato_item.write(cr, uid, item.id, {'cttite_cancelado': True},
                                                      context)
                elif tipo_tratamento == 'I':
                    data_tratamento = item.cttite_data_inicio
        context['cttctr_tipo_contrato'] = contrato_tipo
        if contrato_id:
            if not isinstance(contrato_id, (int, long)):
                contrato_id = contrato_id.id
            context['contrato_capa_id'] = contrato_id
            prazo_dado = obj_contratos_prazos.default_get(cr, uid, [], context)
            prazo_ids = []
            for x in brw_wiz_tratar_item_contrato:
                data_tratamento = data_tratamento or x.data_inicio
                prazo_dado['cttprz_data_inicio'] = x.data_inicio
                prazo_dado['cttprz_data_termino'] = x.data_fim
                titulo_ids = obj_financeiro_titulo.search(
                    cr, uid, [('data_prorrogacao', '>', x.data_inicio),
                              ('contrato_id', '=', contrato_id)], context=context)
                if titulo_ids:
                    obj_financeiro_titulo.unlink(cr, uid, titulo_ids)
                alterar_prazo_ids = obj_contratos_prazos.search(
                    cr, uid, [('cttprz_data_inicio', '<=', data_tratamento),
                              ('cttprz_data_termino', '>=', data_tratamento),
                              ('cttprz_contrato_id', '=', contrato_id),
                              ('cttprz_real_provisao', '=', 'R')], context=context)
                if alterar_prazo_ids:
                    obj_contratos_prazos.write(
                        cr, uid, alterar_prazo_ids, {'cttprz_data_termino': data_tratamento},
                        context)
                else:
                    raise osv.except_osv(_('Atencao'), _('Data do item esta fora da faixa do '
                                                             'prazo em vigor!'))
                if context.get('cttctr_inclusao_multipla_item'):
                    contrato_item_ids = x.item_multiplo_contrato_ids
                    if not contrato_item_ids:
                        raise osv.except_osv(_('Atencao'), _('Informe o item e o seu valor'))
                    data_inicio = parse(x.data_inicio) + relativedelta(days=+1)
                    data_inicio = data_inicio.strftime('%Y-%m-%d')
                    prazo_dado['cttprz_data_inicio'] = data_inicio
                if tipo_tratamento == 'I' and contrato_item_ids:
                    cttprz_item_valor_ids = []
                    for item in contrato_item_ids:
                        if context.get('cttctr_inclusao_multipla_item'):
                            vals_contrato_item = {'cttite_capa_id': contrato_id,
                                                  'cttite_data_inicio': data_tratamento,
                                                  'cttite_produto_id': item.wiztim_servico_id.id,
                                                  'cttite_observacao': item.wiztim_oberservacao,
                                                  'cttite_carencia': carencia,
                                                  }
                            contrato_item_id = obj_contratos_contrato_item.create(
                                cr, uid, vals_contrato_item, context)
                            vals = {'cttivl_valor': item.wiztim_valor,
                                    'cttivl_item_id': contrato_item_id}
                        else:
                            vals = {'cttivl_valor': x.valor_item, 'cttivl_item_id': item}
                        cttprz_item_valor_ids.append((0, 0, vals))
                    prazo_dado['cttprz_item_valor_ids'].extend(cttprz_item_valor_ids)
                if prazo_dado.get('cttprz_item_valor_ids'):
                    prazo_id = obj_contratos_prazos.create(cr, uid, prazo_dado, context)
                    prazo_ids.append(prazo_id)
            if prazo_ids:
                obj_contratos_prazos.cttprz_button_gerar_titulos(cr, uid, prazo_ids, context)
        return True

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(wiz_tratar_item_contrato, self).default_get(cr, uid, fields_list, context)
        if context.get('cttite_data_cancelamento'):
            data_inicio = context.get('cttite_data_cancelamento')
            data_inicio = parse(data_inicio) + relativedelta(days=+1)
            data_inicio = data_inicio.strftime('%Y-%m-%d')
            res['data_inicio'] = data_inicio
            res['tipo'] = 'C'
        elif context.get('cttite_data_inicio'):
            data = parse(context.get('cttite_data_inicio')) + relativedelta(days=+1)
            data = data.strftime('%Y-%m-%d')
            res['data_inicio'] = data
            res['tipo'] = 'I'
        list_ids = context.get('active_ids') or []
        if context.get('cttctr_inclusao_multipla_item'):
            objeto = self.pool.get('contratos.contrato')
        else:
            objeto = self.pool.get('contratos.contrato.item')
        obj_contratos_prazos = self.pool.get('contratos.prazos')
        brw_objeto = objeto.browse(cr, uid, list_ids,  context)
        for item in brw_objeto:
            if context.get('cttctr_inclusao_multipla_item'):
                contrato_id = item.id
            else:
                contrato_id = item.cttite_capa_id.id
            prazo_ids = obj_contratos_prazos.search(
                cr, uid, [('cttprz_contrato_id', '=', contrato_id),
                          ('cttprz_real_provisao', '=', 'R')], order='cttprz_data_termino DESC',
                limit=1, context=context)
            if prazo_ids:
                res['data_fim'] = obj_contratos_prazos.browse(
                    cr, uid, prazo_ids[0]).cttprz_data_termino
        return res


class wiz_tratar_item_multiplo_contrato(osv.TransientModel):
    # prefixo: wiztim_

    _name = 'wiz.tratar.item.multiplo.contrato'

    _columns = {
        'wiztim_capa_id': fields.many2one('wiz.tratar.item.contrato', 'Capa'),
        'wiztim_servico_id': fields.many2one('estoque.produto', 'Servico',
                                             domain="[('tipo', '=', 'S')]", required=True),
        'wiztim_oberservacao': fields.text('Informacoes complementares'),
        'wiztim_valor': fields.float('Valor', digits=(11, 2), required=True),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('wiztim_valor') <= 0:
            raise osv.except_osv(_('Atencao'), _('Informe um valor valido para o item'))
        res = super(wiz_tratar_item_multiplo_contrato, self).create(cr, uid, vals, context)
        return res
