#!/usr/bin/python
# -*- encoding: utf-8 -*-

import re
from openerp.osv import osv
from SigUtils import inscricao_estadual
from openerp.tools.translate import _


class control_contrecf_pessoa(osv.Model):
    _inherit = 'sig.parceiros'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        pessoa_id = vals.get('pessoa_id')
        if pessoa_id:
            sig_parceiro_ids = self.search(cr, uid, [('pessoa_id', '=', pessoa_id)], context=context)
            if sig_parceiro_ids:
                self.write(cr, uid, sig_parceiro_ids, vals, context)
        else:
            pessoa_id = self.pool.get('adm.pessoa').search(
                cr, uid, [('nome', '=', vals.get('name')), ('active', '=', True)], context=context
            )
            if pessoa_id:
                vals['pessoa_id'] = pessoa_id[0]

        res = super(control_contrecf_pessoa, self).create(cr, uid, vals, context)
        return res

    def sigpar_onchange_ie(self, cr, uid, ids, admpes_jur_ie, context=None):
        context = context and context.copy() or {}
        res = {}
        estado = False
        for partner in self.browse(cr, uid, ids):
            estado = partner.uf.upper()
        if admpes_jur_ie and admpes_jur_ie.isdigit() and estado:
            res = inscricao_estadual.validar(estado, admpes_jur_ie)
            if not res:
                warning = {
                    'title': _('Atencao'),
                    'message': _('Inscricao Estadual invalida para o seguinte estado: %s') % estado
                }
                return {'warning': warning}
        return res

    def sigpar_onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        pessoa_ids = obj_pessoa.search(
            cr, uid, [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar), ('pessoa_tipo', '=', pessoa_tipo)], context=context
        )
        if pessoa_ids:
            cliente_id = self.search(
                cr, uid, [('pessoa_id', 'in', pessoa_ids)], context=context
            )
            if cliente_id:
                warning = {
                    'title': _('Atencao'),
                    'message': _('Parceiro ja cadastrado!')
                }
                return {'warning': warning, 'value': {'cpf_cnpj': ''}}
            dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
            if isinstance(dados_pessoa, list):
                dados_pessoa = dados_pessoa[0]
            endereco_ids = []
            for dados_endereco in dados_pessoa.endereco_ids:
                enderecos = {
                    'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                    'active': dados_endereco.active,
                    'cep': dados_endereco.cep,
                    'pais_id': dados_endereco.pais_id.id,
                    'uf_id': dados_endereco.uf_id.id,
                    'cidade_id': dados_endereco.cidade_id.id,
                    'bairro_id': dados_endereco.bairro_id.id,
                    'logradouro_id': dados_endereco.logradouro_id.id,
                    'numero': dados_endereco.numero,
                    'complemento': dados_endereco.complemento,
                    'observacao': dados_endereco.observacao,
                }
                endereco_ids.append((1, dados_endereco.id, enderecos))
            email_ids = []
            for dados_emails in dados_pessoa.email_ids:
                emails = {
                    "email_tipo_id": dados_emails.email_tipo_id.id,
                    "name": dados_emails.name,
                    "observacao": dados_emails.observacao,
                }
                email_ids.append((1, dados_emails.id, emails))
            telefones_ids = []
            for dados_telefones in dados_pessoa.telefones_ids:
                telefones = {
                    "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                    "ddi": dados_telefones.ddi,
                    "ddd": dados_telefones.ddd,
                    "numero": dados_telefones.numero,
                    "observacoes": dados_telefones.observacoes,
                }
                telefones_ids.append((1, dados_telefones.id, telefones))

            modulo_fiscal_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr,
                                                                                               uid,
                                                                                               'fiscal',
                                                                                               {})
            endereco_nf_id = dados_pessoa.endereco_nf_id.id if modulo_fiscal_instalado else False

            res = {
                'value': {
                    'pessoa_id': dados_pessoa.id,
                    'cpf_cnpj': cpf_cnpj,
                    'admpes_jur_ie': dados_pessoa.admpes_jur_ie,
                    'name': dados_pessoa.name,
                    'codigo': dados_pessoa.codigo,
                    'fantasia_apelido': dados_pessoa.fantasia_apelido,
                    'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                    'admpes_fis_rg_orgao_emissor': dados_pessoa.admpes_fis_rg_orgao_emissor,
                    'admpes_fis_rg_data_emisssao': dados_pessoa.admpes_fis_rg_data_emisssao,
                    'admpes_fis_cnh_nro': dados_pessoa.admpes_fis_cnh_nro,
                    'admpes_fis_cnh_orgao_emissor': dados_pessoa.admpes_fis_cnh_orgao_emissor,
                    'admpes_fis_cnh_validade': dados_pessoa.admpes_fis_cnh_validade,
                    'admpes_fis_cnh_categoria': dados_pessoa.admpes_fis_cnh_categoria,
                    'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                    'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                    'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                    'foto': dados_pessoa.foto,
                    'endereco_ids': endereco_ids,
                    'email_ids': email_ids,
                    'telefones_ids': telefones_ids,
                    'pessoa': dados_pessoa.id,
                    'endereco_nf_id': endereco_nf_id
                }
            }
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}