#!/usr/bin/python
# -*- encoding: utf-8 -*-

import subprocess
from datetime import date
from base64 import encodestring
from openerp.osv import osv, fields
from psycopg2 import connect
from openerp.tools.translate import _
from openerp.addons.contrecf.versionamento.control.Html import Html

AVAILABLE_STATES = [
    ('draft', 'Rascunho'),
    ('open', 'Aberto'),
    ('cancel', 'Cancelado'),
    ('done', 'Fechado'),
    ('pending', 'Pendente'),
    ('parado', 'Parado')
]


class wizard_gerar_documentacao(osv.TransientModel):
    _name = 'wizard.gerar.documentacao'

    _columns = {
        'state': fields.selection(string='Status',
                                  selection=[('A', 'Aguardando gerar documentacao'),
                                             ('E', 'Documentacao gerada')], required=True, size=1),

        'produto_id': fields.many2one('crm_br.produto', 'Produto', required=True, select=True),

        'versao_id': fields.many2one('crm_br.case.category2', 'Versao a liberar',
                                     required=True, select=True),

        'state_case': fields.selection(AVAILABLE_STATES, 'Estado atividades', size=16, required=True,
                                       select=True),

        'stage_case_id': fields.many2one('crm_br.case.stage', 'Estagio atividades', required=True,
                                         select=True),
        'link_id': fields.many2one('versionamento.link', 'Link base', ondelete='cascade'),

        'data_inicial': fields.date('Data Inicial', required=True),
        'data_final': fields.date('Data Final', required=True),
        'gera_doc_cliente': fields.boolean('Gerar documentacao para cliente'),

        'wiz_gerar_documentacao_bibliotecas_ids': fields.one2many('wizard.gerar.documentacao.bibliotecas',
                                                                  'wiz_gerar_documentacao_capa_id',
                                                                  'Bibliotecas'),

        'usa_base_dados_externa': fields.boolean('Buscar registros de base de dados externa'),
        'base_dados_externa': fields.char('Banco de dados', size=30),
        'usuario_base_dados_externa': fields.char('Usuario', size=30),
        'host_base_dados_externa': fields.char('Host', size=30),
        'password_base_dados_externa': fields.char('Senha', size=30),
        'porta_base_dados_externa': fields.char('Porta', size=4),

        'exportar_servidor': fields.boolean('Exportar para servidor'),
        'usuario_servidor': fields.char('Usuario servidor', size=30),
        'host_servidor': fields.char('Host servidor', size=20),
        'diretorio_servidor': fields.char('Diretorio servidor', size=160),

        'documentacao_gerada': fields.boolean('Documentacao gerada'),
        'dados': fields.binary('Clique para abrir o arquivo ', readonly=True),
        'file': fields.char('Nome do arquivo', size=30, readonly=False, required=True),

    }

    _defaults = {'state': 'A',
                 'documentacao_gerada': False}

    # -------------------------------------------------------------------------------------------- #

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(wizard_gerar_documentacao, self).default_get(cr, uid, fields_list, context)
        obj_versionamento_link = self.pool.get('versionamento.link')
        obj_versionamento_biblioteca = self.pool.get('versionamento.biblioteca')
        link_ids = obj_versionamento_link.search(cr, uid, [], limit=1, order='id DESC',
                                                 context=context)
        link = ''
        if link_ids:
            link = obj_versionamento_link.read(cr, uid, link_ids[0], {'name'}, context=context).get(
                'name') or ''
            res['link_id'] = link_ids[0]
        versionamento_biblioteca_ids = self.pool.get('versionamento.biblioteca').search(
            cr, uid, [], context=context)
        bibliotecas_ids = []
        for biblioteca in obj_versionamento_biblioteca.read(
                cr, uid, versionamento_biblioteca_ids, ['name'], context=context):
            descricao_link = ''
            if link:
                if link[-1] == u'/':
                    descricao_link = link + biblioteca.get('name').lower()
                else:
                    descricao_link = link + '/' + biblioteca.get('name').lower()
            bibliotecas_ids.append((0, 0, {'biblioteca_id': biblioteca.get('id'),
                                           'link': descricao_link}))
        if bibliotecas_ids:
            res['wiz_gerar_documentacao_bibliotecas_ids'] = bibliotecas_ids
        return res

    @staticmethod
    def get_connection(database, user, host, password, port):
        connection = connect(database=database, user=user, host=host, password=password, port=port,
                             connect_timeout=3)
        cursor = connection.cursor()
        return cursor

    def get_atendimentos(self, cr, uid, ids, produto, versao, data_inicial, data_final,
                         state_case, stage_case_id, usa_base_dados_externa=False, context=None):

        context = context and context.copy() or {}

        sql = """
            SELECT  cases.id,
                    cases.name                                            AS titulo,
                    categ.name                                            AS categoria,
                    cases.atividade_pai_id                                AS atividade_pai,
                    pessoa.nome                                           AS parceiro

            FROM    crm_br_case cases
                    LEFT  JOIN crm_br_case_categ categ
                            ON cases.categ_id = categ.id
                    INNER JOIN crm_br_case_category2 versao
                            ON cases.versao_liberar_id = versao.id
                    INNER JOIN crm_br_produto produto
                            ON cases.crmcas_produto_id = produto.id
                    LEFT  JOIN adm_pessoa pessoa
                            ON cases.pessoa_id = pessoa.id

            WHERE   versao.name = '%s'
              AND   produto.name = '%s'
              AND   cases.date_closed BETWEEN '%s 00:00'
                                          AND '%s 23:59'
              AND   cases.state = '%s'
              AND   cases.stage_id = %s

            ORDER
               BY   cases.id DESC,
                    versao.id,
                    categ.name,
                    cases.priority ASC

        """ % (versao, produto, data_inicial, data_final, state_case, stage_case_id)

        if state_case != 'done':
            sql = sql.replace('date_closed', 'date')

        cursor = cr
        if usa_base_dados_externa:
            dados = self.read(cr, uid, ids, ['base_dados_externa',
                                             'usuario_base_dados_externa',
                                             'host_base_dados_externa',
                                             'password_base_dados_externa',
                                             'porta_base_dados_externa'], context=context)
            if isinstance(dados, list):
                dados = dados[0]

            cursor = self.get_connection(dados.get('base_dados_externa'),
                                         dados.get('usuario_base_dados_externa'),
                                         dados.get('host_base_dados_externa'),
                                         dados.get('password_base_dados_externa'),
                                         dados.get('porta_base_dados_externa'))

        cursor.execute(sql)
        atendimentos = cursor.fetchall()

        return atendimentos

    def onchange_link_id(self, cr, uid, ids, link_id, biblioteca_ids, context=None):
        context = context and context.copy() or {}
        res = {}
        if not link_id:
            return res
        obj_versionamento_link = self.pool.get('versionamento.link')
        obj_versionamento_biblioteca = self.pool.get('versionamento.biblioteca')
        link = obj_versionamento_link.read(cr, uid, link_id, {'name'}, context=context).get(
            'name') or ''
        versionamento_biblioteca_ids = self.pool.get('versionamento.biblioteca').search(
            cr, uid, [], context=context)
        if ids and biblioteca_ids:
            for i in biblioteca_ids:
                i[0] = 2
        else:
            biblioteca_ids = []
        for biblioteca in obj_versionamento_biblioteca.read(
                cr, uid, versionamento_biblioteca_ids, ['name'], context=context):
            if link[-1] == u'/':
                descricao_link = link + biblioteca.get('name').lower()
            else:
                descricao_link = link + '/' + biblioteca.get('name').lower()
            biblioteca_ids.append((0, 0, {'biblioteca_id': biblioteca.get('id'),
                                           'link': descricao_link}))
        if biblioteca_ids:
            res['value'] = {'wiz_gerar_documentacao_bibliotecas_ids': biblioteca_ids}
        return res

    def onchange_nome_arquivo(self, cr, uid, ids, produto_id, versao_id, gera_doc_cliente=False,
                              context=None):
        context = context and context.copy() or {}
        res = {}

        obj_crm_produto = self.pool.get('crm_br.produto')
        produto_name = obj_crm_produto.read(cr, uid, produto_id, ['name'], context=context).get('name') or None

        obj_crm_case_category2 = self.pool.get('crm_br.case.category2')
        versao_name = obj_crm_case_category2.read(cr, uid, versao_id, ['name'], context=context).get('name') or None

        if produto_name and versao_name:
            file_name = '%s-%s.html' % (produto_name, versao_name)

            if gera_doc_cliente:
                file_name = file_name.replace('.html', '_cliente.html')

            res['file'] = file_name.replace(' ', '_')

        return {'value': res}

    def button_gerar_documentacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        dados = self.read(cr, uid, ids, context=context)

        if isinstance(dados, list):
            dados = dados[0]

        file_name = dados.get('file')
        produto = dados.get('produto_id')[1]
        versao = dados.get('versao_id')[1]
        state_case = dados.get('state_case')
        stage_case_id = dados.get('stage_case_id')[0]
        gera_doc_cliente = dados.get('gera_doc_cliente')
        data_inicial = dados.get('data_inicial')
        data_final = dados.get('data_final')
        usa_base_dados_externa = dados.get('usa_base_dados_externa')

        html = Html()

        diretorio_arquivo = '/tmp/' + file_name.replace('.html', '') + '.html'

        # Abre arquivo para escrita
        arquivo = open(diretorio_arquivo, 'w')
        arquivo.write(html.cabecalho())
        arquivo.write(html.title("%s %s " % (produto, versao)))
        arquivo.write(html.meta())
        arquivo.write(html.css())
        arquivo.write(html.cabecalho_fim())
        arquivo.write(html.corpo())

        arquivo.write(html.divreportheader('%s %s' % (produto, versao),
                                           date.today().strftime("%d/%m/%Y")))

        arquivo.write(html.divereportnotes('Atendimentos liberados no %s %s' % (produto, versao)))

        if gera_doc_cliente:
            arquivo.write('<div id="ReportDetails">')
            arquivo.write(html.details_atendimento_versao_cliente())
        else:
            obj_bibliotecas = self.pool.get('wizard.gerar.documentacao.bibliotecas')
            bibliotecas_ids = dados.get('wiz_gerar_documentacao_bibliotecas_ids')
            brw_documentacao_bibliotecas = obj_bibliotecas.browse(cr, uid, bibliotecas_ids, context)

            if brw_documentacao_bibliotecas:
                arquivo.write(html.detatils(titulo='Bibliotecas'))
                pacotes = {}
                for biblioteca in brw_documentacao_bibliotecas:
                    pacotes[biblioteca.biblioteca_id.name] = [biblioteca.versao, biblioteca.link]
                if pacotes:
                    arquivo.write(html.gerabiblioteca(pacotes))
                    arquivo.write(html.fim_details_biblioteca())

            arquivo.write(html.details_atendimento())

        qtd_atendimentos = 0

        atendimentos = self.get_atendimentos(cr, uid, ids, produto, versao, data_inicial,
                                             data_final, state_case, stage_case_id,
                                             usa_base_dados_externa)

        for atendimento in atendimentos:
            arquivo.write(html.tr())
            chamado = '<td class="ReportTableValueCell"> %s </td>' % atendimento[0]
            titulo = '<td class="ReportTableValueCell"> %s </td>' % atendimento[1]

            if atendimento[2]:
                categoria = '<td class="ReportTableValueCell"> %s </td>' % atendimento[2]
            else:
                categoria = '<td class="ReportTableValueCell"></td>'

            if atendimento[3]:
                atividade_pai = '<td class="ReportTableValueCell"> %s </td>' % atendimento[3]
            else:
                atividade_pai = '<td class="ReportTableValueCell"></td>'

            if atendimento[4]:
                parceiro = '<td class="ReportTableValueCell"> %s </td>' % atendimento[4]
            else:
                parceiro = '<td class="ReportTableValueCell"></td>'

            if gera_doc_cliente:
                arquivo.write(('%s %s %s' % (chamado, titulo, categoria)).encode('utf-8', 'strict'))
            else:
                arquivo.write(('%s %s %s %s %s' % (chamado, titulo,
                                                   categoria,
                                                   atividade_pai,
                                                   parceiro)).encode('utf-8', 'strict'))
            arquivo.write(html.fim_tr())
            qtd_atendimentos += 1

        arquivo.write(html.fim_table())
        arquivo.write(html.fim_br())
        arquivo.write('<p class="ReportTableInfo">%d Atendimentos liberados.</p>' % qtd_atendimentos)
        arquivo.write(html.fim_div())
        arquivo.write(html.reportfooter())
        arquivo.write(html.fim_br())
        arquivo.write(html.fim_body())
        arquivo.write(html.fim_html())

        arquivo.close()

        # Abre arquivo para leitura
        arquivo = open(arquivo.name, 'r')
        conteudo_arquivo = ''
        for line in arquivo.readlines():
            conteudo_arquivo += line
        arquivo.close()

        self.write(cr, uid, ids, {'dados': encodestring(conteudo_arquivo),
                                  'state': 'E', 'documentacao_gerada': True}, context=context)

        # Exportar para servidor
        if dados.get('exportar_servidor'):
            cmd_scp = 'scp %s %s@%s:/%s' % (diretorio_arquivo, dados.get('usuario_servidor'),
                                            dados.get('host_servidor'),
                                            dados.get('diretorio_servidor'))

            exportacao = subprocess.Popen(cmd_scp, shell=True, stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE)

            erro = exportacao.stderr.read()

            if erro:
                raise osv.except_osv(_('Erro'),
                                     _('Falha ao exportar arquivo.\n%s' % str(erro)))

        return True

    def button_limpar_consulta(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.unlink(cr, uid, ids)
        return {'name': 'Gerar documentacao',
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': False,
                'res_model': 'wizard.gerar.documentacao',
                'target': 'inline',
                'type': 'ir.actions.act_window',
                }

