#!/usr/bin/python
# -*- encoding: utf-8 -*-

import os
import subprocess
import compileall
from openerp.tools.translate import _
from openerp.osv import osv, fields


class wizard_gerar_versao(osv.TransientModel):
    _name = 'wizard.gerar.versao'

    _columns = {
        'state': fields.selection(string='Status', selection=[('A', 'Aguardando gerar versao'),
                                                              ('E', 'Versao gerada')],
                                  required=True, size=1),

        'produto_id': fields.many2one('crm_br.produto', 'Produto', required=True, select=True),
        'versao_id': fields.many2one('crm_br.case.category2', 'Versao a liberar', required=True,
                                     select=True),

        'arquitetura': fields.selection(
            [('all', 'Multiplataforma'), ('i386', '32 bits'), ('adm64', '64 bits')],
            'Arquitetura', required=True),

        'maintainer': fields.char('Maintainer', size=60, required=True),
        'description': fields.char('Description', size=160, required=True),

        'section': fields.char('Section', size=30),
        'homepage': fields.char('Homepage', size=60),

        'wiz_gerar_versao_bibliotecas_ids': fields.one2many('wizard.gerar.versao.bibliotecas',
                                                            'wiz_gerar_versao_capa_id',
                                                            'Bibliotecas'),

        'diretorio_destino': fields.char('Diretorio destino p/ o pacote gerado', size=60,
                                         required=True),

        'wiz_compilar_arquivos': fields.boolean('Compilar arquivos'),

        'exportar_servidor': fields.boolean('Exportar para servidor'),
        'usuario_servidor': fields.char('Usuario servidor', size=30),
        'senha_servidor': fields.char('Senha', size=30),
        'host_servidor': fields.char('Host servidor', size=20),
        'diretorio_servidor': fields.char('Diretorio servidor', size=160),

        'versao_gerada': fields.boolean('Versao gerada'),

        'log': fields.text('Log'),
        'file': fields.char('Nome do arquivo', size=30, readonly=False, required=True),
    }

    _defaults = {'state': 'A',
                 'versao_gerada': False,
                 'arquitetura': 'all',
                 'maintainer': 'Sig Informatica <luiz@siginformatica.com.br>',
                 'section': 'SIG',
                 'homepage': 'http://www.sigsolucoes.net.br/',
                 'diretorio_destino': '/tmp'}

    # -------------------------------------------------------------------------------------------- #
    def get_comandos(self, cr, uid, res, produto_id, versao_id, usuario_servidor, context=None):
        context = context and context.copy() or {}
        args = [('comsvn_produto_id','=',produto_id)]
        comando_ids = self.pool.get('comando.svn').search(cr, uid, args, context=context)
        if not comando_ids:
            res.update({'wiz_gerar_versao_bibliotecas_ids': []})
            return False
        lista_comando = []
        for dados_comando in self.pool.get('comando.svn').read(cr, uid, comando_ids,
                                                               ['id', 'comsvn_diretorio_instalacao',
                                                                'comsvn_comando_svn'],
                                                               context=context):
            comando = dados_comando.get('comsvn_comando_svn')
            if usuario_servidor and '{usuario}' in comando:
                if not '{versao}' in comando:
                    comando = comando.format(usuario=usuario_servidor)
                else:
                    comando = comando.format(usuario=usuario_servidor,
                                             versao='{versao}')

            if '{versao}' in comando:
                nome_versao = self.pool.get('crm_br.case.category2').\
                              browse(cr, uid, versao_id, context=context).name
                if not '{usuario}' in comando:
                    comando = comando.format(versao=nome_versao)
                else:
                    comando = comando.format(usuario='{usuario}',
                                             versao=nome_versao)


            lista_comando.append((0, False,
                                  {'wizgvb_comando_id':dados_comando.get('id'),
                                   'diretorio_instalacao': dados_comando.get('comsvn_diretorio_instalacao'),
                                   'comando_svn': comando}
            ))
        res.update({'wiz_gerar_versao_bibliotecas_ids': lista_comando})
        return True

    def funcao_dos_onchanges(self, cr, uid, produto_id, versao_id, usuario_servidor, context=None):
        context = context and context.copy() or {}
        res = {}
        if not produto_id or not versao_id:
            return res

        self.get_comandos(cr, uid, res, produto_id, versao_id, usuario_servidor, context=context)
        obj_crm_produto = self.pool.get('crm_br.produto')
        produto_name = obj_crm_produto.read(cr, uid, produto_id, ['name'], context=context).get('name') or None

        obj_crm_case_category2 = self.pool.get('crm_br.case.category2')
        versao_name = obj_crm_case_category2.read(cr, uid, versao_id, ['name'], context=context).get('name') or None

        if produto_name and versao_name:
            file_name = '%s.%s.deb' % (produto_name, versao_name)
            res['file'] = file_name.replace(' ', '_')

        return {'value': res}

    def onchange_wizgve_produto_id(self, cr, uid, ids, produto_id, versao_id, usuario_servidor,
                                   context=None):
        context = context and context.copy() or {}
        return self.funcao_dos_onchanges(cr, uid, produto_id, versao_id, usuario_servidor,
                                         context=context)

    def onchange_wizgve_versao_id(self, cr, uid, ids, produto_id, versao_id, usuario_servidor,
                                  context=None):
        context = context and context.copy() or {}
        return self.funcao_dos_onchanges(cr, uid, produto_id, versao_id, usuario_servidor,
                                         context=context)

    @staticmethod
    def criar_estrutura_debian(diretorio_destino, produto_id, versao_id, description,
                               arquitetura, maintainer, section, homepage):

        os.mkdir(diretorio_destino)
        os.mkdir('%s/DEBIAN/' % diretorio_destino)

        file_control = open('%s/DEBIAN/control' % diretorio_destino, 'wb')
        file_control.write('Package: %s\n' % produto_id)
        file_control.write('Version: %s\n' % versao_id)
        file_control.write('Description: %s\n' % description)
        file_control.write('Architecture: %s\n' % arquitetura)
        file_control.write('Maintainer: %s\n' % maintainer)
        file_control.write('Section: %s\n' % section)
        file_control.write('Homepage: %s\n' % homepage)
        file_control.close()

        return True

    @staticmethod
    def get_versionamento_log(str_log):
        if os.path.exists('/tmp/versionamento.log'):
            versionamento_log = open('/tmp/versionamento.log', 'r')
            log_lines = versionamento_log.readlines()
            for line in log_lines:
                str_log += '%s\n' % line
            versionamento_log.close()
        return str_log

    @staticmethod
    def realizar_checkout_svn(comando_svn, senha_servidor, diretorio_codigo_fonte):
        # arq = open('/tmp/.sudoaskpass','w')
        # arq.write('#!/bin/sh\nhead -n 1')
        # arq.close()
        # cmd = '%s %s' % (comando_svn, diretorio_codigo_fonte)
        # cmd_aux = """echo {senha} | echo -e '#!/bin/sh\nhead -n 1' > ~/.sudoaskpass && chmod""".\
        #           format(senha=senha_servidor) + \
        #       """ a+x ~/.sudoaskpass && SUDO_ASKPASS=~/.sudoaskpass sudo -A {comando}""".\
        #           format(comando=cmd) + """; rm -f ~/.sudoaskpass """
        cmd = '%s %s' % (comando_svn, diretorio_codigo_fonte)
        # cmd_aux = """echo {senha} | chmod a+x /tmp/.sudoaskpass """.format(senha=senha_servidor)
        #
        # svn_checkout = os.system(cmd_aux)
        cmd_aux = """echo {senha} | sudo -S {comando}""".\
                  format(senha=senha_servidor, comando=cmd)
        svn_checkout = os.system(cmd_aux)
        svn_chmod = os.system(cmd_aux)

        if svn_checkout != 0:
            raise osv.except_osv(_('Erro'), _('Comando svn invalido:\n %s' % cmd))

        os.system("echo {senha} | find {dir} -name '.svn' | xargs sudo -S rm -rf".
                  format(senha=senha_servidor, dir=diretorio_codigo_fonte))

        return True

    @staticmethod
    def compilar_arquivos(diretorio_codigo_fonte):
        compileall.compile_dir(diretorio_codigo_fonte, force=1)
        os.system("find %s -name '*.py' | xargs rm" % diretorio_codigo_fonte)
        return True

    def button_gerar_versao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        dados = self.read(cr, uid, ids, context=context)

        if isinstance(dados, list):
            dados = dados[0]

        diretorio_destino = '%s/%s' % (dados.get('diretorio_destino'),
                                       dados.get('file').replace('.deb', ''))

        if os.path.exists(diretorio_destino):
            os.system('rm -rf %s' % diretorio_destino)

        log = 'Criando estrutura debian ...\n'

        self.criar_estrutura_debian(diretorio_destino, dados.get('produto_id')[1].replace(' ', ''),
                                    dados.get('versao_id')[1], dados.get('description'),
                                    dados.get('arquitetura'), dados.get('maintainer'),
                                    dados.get('section'), dados.get('homepage'))

        obj_bibliotecas = self.pool.get('wizard.gerar.versao.bibliotecas')
        bibliotecas_ids = dados.get('wiz_gerar_versao_bibliotecas_ids')
        dados_bibliotecas = obj_bibliotecas.read(cr, uid, bibliotecas_ids, context=context)

        for biblioteca in dados_bibliotecas:
            comando_svn = biblioteca.get('comando_svn')
            diretorio_instalacao = biblioteca.get('diretorio_instalacao')
            diretorio_codigo_fonte = diretorio_destino
            for diretorio in diretorio_instalacao.split('/'):
                if diretorio:
                    diretorio_codigo_fonte += '/%s' % diretorio
                    if not os.path.exists(diretorio_codigo_fonte):
                        os.mkdir(diretorio_codigo_fonte)

            log += 'Realizando checkout: %s\n' % comando_svn.encode('utf-8', 'strict')

            self.realizar_checkout_svn(comando_svn, dados.get('senha_servidor'),
                                       diretorio_codigo_fonte)

            log += 'Gerado cópia de trabalho!\n'

        # Compilar arquivos python
        if dados.get('wiz_compilar_arquivos'):
            log += 'Compilando arquivos python ...\n'
            self.compilar_arquivos(diretorio_codigo_fonte)

        os.system('rm -rf /tmp/versionamento.log')
        pacote_gerado = os.system('dpkg-deb -b %s %s/%s > /tmp/versionamento.log'
                                  % (diretorio_destino, dados.get('diretorio_destino'),
                                     dados.get('file')))

        log = self.get_versionamento_log(log)

        if pacote_gerado != 0:
            log += 'Erro ao gerar pacote!'
        else:
            log += 'Pacote gerado!\n'
            self.write(cr, uid, ids, {'state': 'E', 'versao_gerada': True,
                                      'log': log}, context=context)

            # Exportar para servidor
            if dados.get('exportar_servidor'):
                log += ("Exportando pacote para '%s' em '%s' ...\n"
                        % (dados.get('diretorio_servidor'),
                           dados.get('host_servidor'))).encode('utf-8', 'strict')

                exportacao = os.system('echo %s | sudo -S scp %s %s@%s:/%s' %
                                       (dados.get('senha_servidor'),
                                        dados.get('diretorio_destino') + '/' + dados.get('file'),
                                        dados.get('usuario_servidor'),
                                        dados.get('host_servidor'),
                                        dados.get('diretorio_servidor')))

                if exportacao == 0:
                    log += 'Pacote exportado!\n'
                else:
                    raise osv.except_osv(_('Erro'),
                                         _('Falha ao exportar pacote. '
                                           'Verifique se o diretorio do servidor esta correto!'))

        self.write(cr, uid, ids, {'log': log}, context=context)

        return True

    def button_limpar_consulta(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.unlink(cr, uid, ids)
        return {'name': 'Gerar versao',
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': False,
                'res_model': 'wizard.gerar.versao',
                'target': 'inline',
                'type': 'ir.actions.act_window',
                }
