# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from funcoes import simple_num_crypt, flag_decrypt
from datetime import datetime
from openerp.addons.adm import messagebox
from openerp.tools.translate import _


class GerarContraSenha(osv.TransientModel):
    _name = 'gerar.contra.senha'

    _columns = {
        'campo1': fields.char('Campo 1', required=True),
        'campo2': fields.char('Campo 2', required=True),
        'campo3': fields.char('Campo 3', required=True),
        'campo4': fields.char('Campo 4', required=True),
        'campo5': fields.char('Campo 5', required=True),
        'campo6': fields.char('Campo 6', required=True),
        'campo7': fields.char('Campo 7', required=True),
        'campo8': fields.char('Campo 8', required=True),
        'campo9': fields.char('Campo 9', required=True),

        'nroserie_id': fields.many2one('contrecf.nroseriesecfs', 'Nro de serie', required=True, ondelete="restrict",
                                       select=True),
        'nroecf': fields.integer('Nro do ecf'),
        'nroloja': fields.integer('Nro da loja'),

        'crz_ecf': fields.integer('CRZ ECF', size=4, help='Contador de Reducao Z da ECF'),
        'crz_db': fields.integer('CRZ Base', size=4, help='Contador de Reducao Z Gravado no Banco do Frente'),
        'cro_ecf': fields.integer('CRO ECF', size=3, help='Contador de Reinicio de Operacoes da ECF'),
        'cro_db': fields.integer('CRO Base', size=3,
                                 help='Contador de Reinicio de Operacoes Gravado no Banco do Frente'),
        'vbd_ecf': fields.float("Venda Bruta ECF (R$)", digits=(12, 2), help="Venda Bruta Diaria na ECF"),
        'vbd_db': fields.float("Venda Bruta Base (R$)", digits=(12, 2), help="Venda Bruta Diaria no Banco do Frente"),

        'contra_senha': fields.char('Contra Senha'),
        'info': fields.text('Informacoes'),
        'state': fields.selection([('P', 'Pendente'), ('V', 'Validado')], 'Status')
    }

    _defaults = {
        'state': 'P',
        'contra_senha': 'Preencha todos os campos'
    }

    def btn_gerar_recomposicao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        info_obtido = self.read(cr, uid, ids[0], ["nroserie", "nroecf", "nroloja", "crz_ecf", "crz_db", "cro_ecf",
                                                  "cro_db", "vbd_ecf", "vbd_db"], context=context)

        contrecf_recomposicao_gt = self.pool.get('contrecf.recomposicao.gt')
        contrecf_nroseriesecfs = self.pool.get('contrecf.nroseriesecfs')

        nro_series_search = contrecf_nroseriesecfs.search(cr, uid, [('nroloja', '=', info_obtido['nroloja']),
                                                                    ('nroecf', '=', info_obtido['nroecf'])],
                                                          context=context)

        if not nro_series_search:
            raise osv.except_osv(_('Erro'), _('Nro. de Serie nao encontrado para: Loja %d, ECF %d.')
                                 % (info_obtido['nroloja'], info_obtido['nroecf']))

        nro_serie = contrecf_nroseriesecfs.read(cr, uid, nro_series_search[0], ['nroserie'], context=context)

        info_obtido.update({
            'nroserie_id': nro_series_search[0],
            'data_alteracao': datetime.today(),
            'nroserie': nro_serie['nroserie']
        })

        contrecf_recomposicao_gt.create(cr, uid, info_obtido, context)

        return messagebox(_('Recomposicao de GT gerada com sucesso!'))

    def onchange_campos(self, cr, uid, ids, cp1, cp2, cp3, cp4, cp5, cp6, cp7, cp8, cp9):

        retval = {}

        ret_dict = {'value': retval}

        # Se todos os campos estiverem prenchidos, gerar a contra senha
        if all([cp1, cp2, cp3, cp4, cp5, cp6, cp7, cp8, cp9]):
            lst_campos = [cp1, cp2, cp3, cp4, cp5, cp6, cp7, cp8, cp9]
            soma = []
            cp_inc = 1
            err = []
            for x in lst_campos:
                if not x.isdigit():
                    err.append('%d' % cp_inc)
                else:
                    soma.append(int(x))
                cp_inc += 1

            if err:
                raise osv.except_osv(_("Erro"),
                                     _("Os seguintes campos devem ter valores numericos: (%s)") % ', '.join(err))

            retval.update({'contra_senha': simple_num_crypt(str(sum(soma)))})

            str_decrypt = simple_num_crypt('-'.join(lst_campos), flag_decrypt)  # Descriptografar os dados
            lst_decrypt = str_decrypt.split('-')

            str_info = ''
            # Loop para remover o primeiro digito que e usado pelo SigFrente para gerar a chave aleatoria
            for d in lst_decrypt[:-2]:
                str_info += d[1:]

            contrecf_nroseriesecfs = self.pool.get('contrecf.nroseriesecfs')

            try:
                info_obtido = {
                    'nroloja': int(str_info[:4]),
                    'nroecf': int(str_info[4:7]),
                    'crz_ecf': int(str_info[7:10]),
                    'crz_db': int(str_info[10:13]),
                    'cro_ecf': int(str_info[13:17]),
                    'cro_db': int(str_info[17:21]),
                    'vbd_ecf': int(lst_decrypt[-1]) / 1e2,
                    'vbd_db': int(lst_decrypt[-2]) / 1e2,
                    'state': False
                }

                nroloja = info_obtido['nroloja']
                nroecf = info_obtido['nroecf']
            except:
                nroloja = 0
                nroecf = 0
                raise osv.except_osv(_("Atencao"),
                                     _("Senha invalida!\n"
                                       "\nConfirme com o operador de caixa se os numeros foram informados corretamente"))

            nro_series_search = contrecf_nroseriesecfs.search(cr, uid, [('nroloja', '=', nroloja),
                                                                        ('nroecf', '=', nroecf)], context=context)

            if not nro_series_search:
                ret_dict.update({'warning': {
                    'title': _('Informacao'),
                    'message': _('Nro. de Serie nao encontrado para: Loja %d, ECF %d.') %
                               (nroloja, nroecf)
                }})
            else:
                info_obtido.update({'nroserie_id': nro_series_search[0], 'state': 'V'})
                ret_dict.update({'warning': {
                    'title': _('Informacao'),
                    'message': _('Contra-Senha gerada com sucesso!\nConfirme com o operador de caixa a operacao '
                                 'e pressione o botao "gerar recomposicao"')
                }})

            retval.update(info_obtido)

        return ret_dict
