# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
import re


class convenio_convenio(osv.Model):
    _name = 'convenio.convenio'
    _inherit = 'convenio.convenio'

    def covcov_onchange_cpf_cnpj(self, cr, uid, ids, cpf_cnpj):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        warning = {}
        pessoa_tipo = False
        obj_pessoa = self.pool.get('adm.pessoa')
        if cpf_cnpj:
            _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
            tamanho_cpf_cnpj = len(_cnpj_cpf_sem_formatar)
            if not pessoa_tipo:
                pessoa_tipo = 'F' if tamanho_cpf_cnpj == 11 else 'J'
            context = {'convenio_pessoa': True}
            vals = obj_pessoa.onchange_cpf_cnpj(cr, uid, ids, pessoa_tipo, cpf_cnpj,
                                                context=context)
            if vals.get('warning'):
                return {'warning': vals['warning']}

            pessoa_ids = obj_pessoa.search(cr, uid,
                                           [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar)], context=context)
            if pessoa_ids:
                convenio_id = self.search(cr, uid, [('covcov_conveniado_id', 'in', pessoa_ids),
                                                    ('active', '=', True)], context=context)
                if convenio_id:
                    warning.update({'title': _('Atencao')})
                    warning.update({'message': _('Ja existe convenio cadastrado para esta pessoa!')})
                    return {'warning': warning, 'value': {'cpf_cnpj': ''}}
                dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
                if isinstance(dados_pessoa, list):
                    dados_pessoa = dados_pessoa[0]
                res = self.pool.get('adm.pessoa').admpes_prepara_onchange_pessoa(cr, uid, dados_pessoa)
                res['value']['covcov_conveniado_id'] = dados_pessoa.id
                res['value'].update({'pessoa_tipo':pessoa_tipo})
                return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj, 'pessoa_tipo':pessoa_tipo}}

    def covcov_onchange_conveniado(self, cr, uid, ids, pessoa_id):
        """
        Metodo invocado na saida do campo covcob_conveniado_id da tela.
        Pesquisa em adm.pessoa, se já existe o cadastro que deseja incluir como convenio
        """
        warning = {}
        if not pessoa_id:
            return {'warning': warning, 'value': {}}
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        dados_pessoa = obj_adm_pessoa.browse(cr, uid, pessoa_id)
        if isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]
        res = self.pool.get('adm.pessoa').admpes_prepara_onchange_pessoa(cr, uid, dados_pessoa)
        res['value']['covcov_conveniado_id'] = dados_pessoa.id
        res['value']['id'] = 1
        return res

    def covcov_onchange_ie(self, cr, uid, ids, admpes_jur_ie):
        res, estado = self.pool.get('adm.pessoa').admpes_validar_ie(cr, uid, ids, admpes_jur_ie)
        if not res:
            return {'warning': {'title': _('Atencao'),
                                'message': _(
                                    'Inscricao Estadual invalida para o seguinte estado: %s') % estado}}
        return res