# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import re
from openerp.tools.translate import _


class convenio_dependente(osv.Model):
    _name = 'convenio.dependente'
    _inherit = 'convenio.dependente'

    def covdep_onchange_cpf_cnpj(self, cr, uid, ids, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar('F', cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_convenio_convenio = self.pool.get('convenio.convenio')
        pessoa_ids = obj_pessoa.search(cr, uid,
                                       [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar), ('pessoa_tipo', '=', 'J')],
                                       context=context)
        if pessoa_ids:
            convenio_id = obj_convenio_convenio.search(cr, uid,
                                                       [('covcov_conveniado_id', 'in', pessoa_ids),
                                                        ('active', '=', True)], context=context)
            if convenio_id:
                warning = {'title': _('Atencao')}
                warning['message'] = _('Ja existe convenio cadastrado para esta pessoa!')
                return {'warning': warning, 'value': {'cpf_cnpj': ''}}
            dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
            if isinstance(dados_pessoa, list):
                dados_pessoa = dados_pessoa[0]
            res = self.pool.get('adm.pessoa').admpes_prepara_onchange_pessoa(cr, uid, dados_pessoa)
            res['value']['covdep_dependente_id'] = dados_pessoa.id
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}

    def covdep_onchange_dependente(self, cr, uid, ids, pessoa_id, context=None):
        """
        Metodo invocado na saida do campo covdep_dependente_id da tela.
        Pesquisa em adm.pessoa, se já existe o cadastro que deseja incluir como dependente
        """
        context = context and context.copy() or {}
        warning = {}
        if not pessoa_id:
            return {'warning': warning, 'value': {}}
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        dados_pessoa = obj_adm_pessoa.browse(cr, uid, pessoa_id)
        if isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]
        res = self.pool.get('adm.pessoa').admpes_prepara_onchange_pessoa(cr, uid, dados_pessoa)
        res['value']['covdep_dependente_id'] = dados_pessoa.id
        res['value']['id'] = 1
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('covgia_usuario_id'):
            vals['covdep_usuario_id'] = context.get('covgia_usuario_id')
        else:
            raise osv.except_osv(_('Atencao'), _('Informe o usuario da guia'))
        res = super(convenio_dependente, self).create(cr, uid, vals, context)
        return res
