# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import time
from openerp.tools.translate import _


class convenio_guia(osv.Model):
    _name = 'convenio.guia'
    _inherit = 'convenio.guia'

    def covgia_botao_ativar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'A'}, context)
        return True

    def covgia_botao_cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = {
            'state': 'C',
            'covgia_cancelado_por_id': uid,
            'covgia_data_cancelamento': time.strftime('%Y-%m-%d')
        }
        self.write(cr, uid, ids, vals, context)
        return True

    def covgia_confirmar_guias(self, cr, uid, guia_ids, context=None):
        """
        Método utilizado para confirmar guias
        @param cr:
        @param uid:
        @param guia_ids: {list} lista de ids das guias a serem confirmadas
        @param context:
        @return: {bool} True / False
        """
        context = context and context.copy() or {}
        self.write(cr, uid, guia_ids, {'covgia_quem_confirmou_id': uid,
                                       'covgia_confirmacao_data': time.strftime('%Y-%m-%d')},
                   context=context)
        return True

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __code = 'convenio.guia'
        __name = 'Sequencia da guia do Convenio'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 6,
            'prefix': '%(year)s-',
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence').search(cr, uid, [('code', '=', __code), ('company_id', '=', company_id)], context=context)
        if not consulta:
            self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        vals['covgia_numero'] = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, 'convenio.guia')
        res = super(convenio_guia, self).create(cr, uid, vals, context)
        return res

    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Atencao'), _('Nao posso duplicar uma guia!'))

    def covgia_receber_guia_a_vista(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_financeiro = self.pool.get('financeiro')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_adm_forma_pgto = self.pool.get('adm.forma.pgto')
        obj_adm_util = self.pool.get('adm.utils')
        obj_convenio_guia = self.pool.get('convenio.guia')

        brw_convenio_guia = self.browse(cr, uid, ids, context)
        list_titulo_ids = []
        data_emissao = time.strftime('%Y-%m-%d')
        for guia in brw_convenio_guia:
            pessoa_id = guia.covgia_usuario_id.covusa_usuario_id.id
            tipo = 'R'
            item_ids = guia.covgia_guia_item_ids
            if not item_ids:
                raise osv.except_osv(_('Atencao'), _('A guia nao posssui itens cadastrados'))
            total_item = 0
            for item in item_ids:
                total_item += item.covite_valor_usuario or 0
            if not total_item or total_item < 0:
                raise osv.except_osv(_('Atencao'), _('Os itens nao possuem valores cadastrados'))
            titulo_dados = [{'vencimento': data_emissao, 'valor': total_item}]
            # Vai criar a fatura e o titulo
            fatura_id = obj_financeiro.financ_incluir_fatura(cr, uid, futl_fat_pessoa_id=pessoa_id,
                                                             futl_fat_data_emissao=data_emissao,
                                                             futl_fat_valor=total_item,
                                                             futl_fat_tipo_fatura=tipo,
                                                             futl_fat_titulo_dados=titulo_dados)
            if not fatura_id:
                msg_erro = _(obj_financeiro.financ_get_erro())
                raise osv.except_osv(_('Atencao'), msg_erro)

            for item in brw_convenio_guia:
                obj_convenio_guia.write(cr, uid, item.id, {
                    'covgia_faturamento_avista_ids': [(4, fatura_id)]
                }, context=context)

            titulo_ids = obj_financeiro_titulo.search(cr, uid, [('fatura_id', '=', fatura_id)], context=context)
            if not titulo_ids:
                raise osv.except_osv(_('Atencao'), _('Nao foi gerado o titulo para realizar a '
                                                     'baixa'))
            list_titulo_ids.append(titulo_ids[0])
        # Vai realizar a baixa do titulo
        forma_pagamento_ids = obj_adm_forma_pgto.search(cr, uid, [('sigla', '=', 'DN')], context=context)
        if not forma_pagamento_ids:
            forma_pagamento_ids = obj_adm_forma_pgto.search(cr, uid, [], context=context)
            if not forma_pagamento_ids:
                raise osv.except_osv(_('Atencao'), _('Nao ha forma de pagamento cadastrada nos '
                                                     'parametros do financeiro'))
        financeiro_parametro = obj_adm_util.admutl_get_parametros(
            cr, uid, 'financeiro', 'finpar_conta_tesouraria_avista_id', context=context)
        tesouraria_conta_id = financeiro_parametro.get('finpar_conta_tesouraria_avista_id')
        if not tesouraria_conta_id:
            raise osv.except_osv(_('Atencao'), _("Cadastre a 'Conta caixa recebimento a vista' "
                                                 "nos parametros do financeiro"))

        titulos = []
        brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, list_titulo_ids, context=context)
        for i in brw_financeiro_titulo:
            titulo_id = i.wbtd_titulo_id.id
            valor_juros = i.wbtd_juros_total
            valor_multa = i.wbtd_multa_total
            valor_desconto = i.wbtd_desconto_total
            valor_pago = i.wbtd_valor_total
            tipo_titulo = i.wbtd_titulo_id.tipo_titulo
            pessoa_id = i.wbtd_titulo_id.pessoa_id.id
            titulos.append({'titulo_id': titulo_id, 'valor_pago': valor_pago,
                            'saldo_baixado': valor_pago,
                            'valor_multa': valor_multa, 'valor_juros': valor_juros,
                            'valor_desconto': valor_desconto, 'tipo_titulo': tipo_titulo,
                            'pessoa_id': pessoa_id})
        baixa_multipla = True if len(titulos) > 1 else False
        obj_financeiro.financ_baixar_titulo(cr, uid, futl_bxti_titulos=titulos,
                                            futl_bxti_data_pagamento=data_emissao,
                                            ftul_bxti_forma_pgto_id=forma_pagamento_ids[0],
                                            futl_bxti_tesouraria_conta_id=tesouraria_conta_id,
                                            futl_bxti_baixa_multipla=baixa_multipla,
                                            )
        return True

    def onchange_covgia_guia_item_ids(self, cr, uid, ids):
        """
        Limpar treeview toda vez que o campo Convenio (covgia_convenio_id) tiver alteracao de valor.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @return: {dict} o valor do covgia_guia_item_ids vazio.

        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {'covgia_guia_item_ids': False}}

        return result
