# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time


class convenio_guia(osv.Model):
    """
    Tabela que contem a utilização do convenio
    prefixo:covgia
    """
    _name = 'convenio.guia'
    _rec_name = 'covgia_usuario_id'
    _columns = {
        'id': fields.integer('id', readonly=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # Codigo de identificacao da empresa
        'covgia_convenio_id': fields.many2one("convenio.convenio", 'Convenio', ondelete="restrict",
                                              required=True, select=True),
        'covgia_usuario_id': fields.many2one("convenio.usuarios", 'Usuario', ondelete="restrict",
                                             required=True, select=True),
        'covgia_dependente_id': fields.many2one("convenio.dependente", 'Dependente',
                                                ondelete="restrict", select=True,
                                                domain="[('covdep_usuario_id','=',covgia_usuario_id)]"),
        'covgia_data': fields.date('Data', required=True),
        'covgia_observacao': fields.text('Observacao'),
        'covgia_numero': fields.char('Guia', size=15, readonly=True),
        'state': fields.selection([('E', 'Esboco'), ('A', 'Ativo'), ('C', 'Cancelado')],
                                  'Estado', size=1, required=True),  # E=Esboco, A=Ativo, C=Cancelado
        'covgia_confirmacao_data': fields.date('Data da confirmacao'),
        'covgia_quem_confirmou_id': fields.many2one("res.users", 'Quem confirmou',
                                                    ondelete="restrict", select=True),
        'covgia_cancelado_por_id': fields.many2one("res.users", 'Quem cancelou',
                                                    ondelete="restrict", select=True),
        'covgia_data_cancelamento': fields.date('Data do cancelamento'),
        'covgia_guia_item_ids': fields.many2many("convenio.item",
                                                   "convenio_guia_m2m_convenio_item",
                                                   "covgia_guia_id", "covgia_item_id",
                                                   'Itens'),
        'covgia_faturamento_usuario_ids': fields.many2many("financeiro.fatura",
                                                   "convenio_guia_usuario_m2m_financeiro_fatura",
                                                   "covfus_guia_id", "covfus_fatura_id",
                                                   'Faturamento usuario'),
        'covgia_faturamento_conveniado_ids': fields.many2many("financeiro.fatura",
                                                   "convenio_guia_conveniado_m2m_financeiro_fatura",
                                                   "covfus_guia_id", "covfus_fatura_id",
                                                   'Faturamento conveniado'),

        'covgia_faturamento_avista_ids': fields.many2many("financeiro.fatura",
                                                   "convenio_guia_avista_m2m_financeiro_fatura",
                                                   "covfus_guia_id", "covfus_fatura_id",
                                                   'Faturamento a vista'),
    }
    _order = 'covgia_numero desc'
    _defaults = {
        'state': lambda *a: 'E',
        'covgia_data': lambda *a: time.strftime('%Y-%m-%d'),
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }
