# -*- coding: utf-8 -*-
from distutils.command.config import config

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import datetime
import base64


class wizard_importa_encomendas(osv.TransientModel):
    'PREFIXO: wizien'
    _name = 'wizard.importa.encomendas'

    def wizien_get_centro_custo_default(self, cr, uid, context):
        context = context and context.copy() or {}
        res = self.pool.get('ir.config_parameter').get_param(cr, uid, 'correios_sfe')
        dado = False
        if res:
            try:
                dado = int(res.split('*|*')[0])
            except:
                dado = False
        return dado

    def wizien_get_evento_default(self, cr, uid, context):
        context = context and context.copy() or {}
        res = self.pool.get('ir.config_parameter').get_param(cr, uid, 'correios_sfe')
        dado = False
        if res:
            dado = int(res.split('*|*')[1])
        return dado

    def wizien_get_pessoa_default(self, cr, uid, context):
        context = context and context.copy() or {}
        res = self.pool.get('ir.config_parameter').get_param(cr, uid, 'correios_sfe')
        dado = False
        if res:
            dado = int(res.split('*|*')[2])
        return dado

    _columns = {
        'wizien_centro_custos_id': fields.many2one('adm.empresa.centrocusto', 'Centro de Custos'),
        'wizien_evento_id': fields.many2one('gerencial.evento', 'Evento'),
        'wizien_arquivo': fields.binary('Nome do arquivo'),
        'wizien_pessoa_id': fields.many2one("adm.pessoa", 'Fornecedor'),
    }

    _defaults = {
        'wizien_centro_custos_id': wizien_get_centro_custo_default,
        'wizien_evento_id': wizien_get_evento_default,
        'wizien_pessoa_id': wizien_get_pessoa_default,
    }

    def wizien_criar_parametro(self, cr, uid, vals, context):
        context = context and context.copy() or {}
        if not all([vals.get('wizien_centro_custos_id'), vals.get('wizien_evento_id'),
                    vals.get('wizien_pessoa_id')]):
            return True
        dados = str(vals.get('wizien_centro_custos_id')) + "*|*" + \
            str(vals.get('wizien_evento_id')) + '*|*' + str(vals.get('wizien_pessoa_id'))
        self.pool.get('ir.config_parameter').set_param(cr, uid, 'correios_sfe', dados)
        return True

    def create(self, cr, user, vals, context=None):
        context = context and context.copy() or {}
        self.wizien_criar_parametro(cr, user, vals, context)
        return super(wizard_importa_encomendas, self).create(cr, user, vals, context)

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        importa_encomenda_dados = self.read(
            cr, user, ids[0] if isinstance(ids, list) else ids,
            ['wizien_centro_custos_id', 'wizien_evento_id', 'wizien_pessoa_id'], context,
            '_classic_write')
        vals_parametro = {
            'wizien_centro_custos_id': vals.get('wizien_centro_custos_id') or
            importa_encomenda_dados.get('wizien_centro_custos_id'),
            'wizien_evento_id': vals.get('wizien_evento_id') or
            importa_encomenda_dados.get('wizien_evento_id'),
            'wizien_pessoa_id': vals.get('wizien_pessoa_id') or
            importa_encomenda_dados.get('wizien_pessoa_id')
        }
        self.wizien_criar_parametro(cr, user, vals_parametro, context)
        return super(wizard_importa_encomendas, self).write(cr, user, ids, vals, context)

    def wizien_button_correios(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {'type': 'ir.actions.act_url',
                'url': 'https://apps.correios.com.br/sfe',
                'target': 'new',
        }

    def wizien_button_importar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_tela = self.browse(cr, uid, ids[0], context)
        if not brw_tela.wizien_arquivo:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum arquivo selecionado!'))
        if not brw_tela.wizien_pessoa_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum fornecedor selecionado!'))

        obj_titulo = self.pool.get('financeiro.titulo')
        arquivo = base64.b64decode(brw_tela.wizien_arquivo).split('\n')
        # O sistema dos correios tem 2 arquivos difrentes, 1 é a previa de uma fatura, o outro é uma fatura, tem que idendificar qual tipo é
        linha1 = arquivo[0]
        lin1col1 = linha1.split(';')[0]
        if lin1col1.startswith(('SFE')):
            _fatura = True
            _posI = 7
        else:
            _fatura = False
            _posI = 3
        for i in arquivo[_posI:]:
            dados = i.split(';')
            try:
                if _fatura:
                    data_postagem = datetime.strptime(dados[2], '%d/%m/%Y').date()  # data da postagem - coluna C
                    etiqueta = dados[9]  # Coluna J
                    valor = float(dados[12].replace('R$ ', '').replace(',', '.'))  # coluna M
                else:
                    data_postagem = datetime.strptime(dados[4], '%d/%m/%Y').date()  # data da postagem
                    etiqueta = dados[12]  # Coluna M
                    valor = float(dados[10].replace('R$ ', '').replace(',', '.'))  # coluna K
            except:
                continue
            titulo_ids = obj_titulo.search(cr, uid, [('documento', '=', etiqueta)], context=context)
            if titulo_ids:
                dados_titulo = obj_titulo.read(cr, uid, titulo_ids[0], ['state', 'vr_original','saldo'],
                                               context)
                # Desconsidera titulos baixados
                if dados_titulo.get('state') == 'B':
                    continue

                elif dados_titulo.get('state') == 'A':
                    dif = valor - (dados_titulo.get('saldo') or 0)
                    # valor do titulo igual ao do correio
                    if -0.01 < dif and dif < 0.01:
                        continue

                    obj_gerencial_mov = self.pool.get('gerencial.movimento')
                    gerencial_ids = obj_gerencial_mov.search(cr, uid,
                                               [('titulo_id', '=', titulo_ids[0])], context=context)
                    if not gerencial_ids:
                        continue

                    obj_gerencial_mov.write(cr, uid, gerencial_ids[0], {'valor': valor}, context)
                    obj_titulo.write(cr, uid, titulo_ids[0], {'vr_original': valor, 'saldo': valor}, context)

            else:
                dia = data_postagem.day
                mes = data_postagem.month
                ano = data_postagem.year
                if dia <= 25 :
                    mes += 1
                    if mes > 12:
                        ano += 1
                        mes = 1
                else:
                    mes += 2
                    if mes > 13:
                        mes = 2
                        ano += 1
                    elif mes > 12:
                        mes = 1
                        ano += 1

                vencimento = datetime(ano, mes, 10).date()
                obj_titulo.create(cr, uid, {'data_vencimento': vencimento,
                                            'vr_original': valor,
                                            'data_emissao': data_postagem,
                                            'tipo_titulo': 'P',
                                            'tipo_lancamento': 'A',
                                            'pessoa_id': brw_tela.wizien_pessoa_id.id,
                                            'fintit_gerencialmov_ids':  [[0, False, {
                                                'evento_id': brw_tela.wizien_evento_id.id,
                                                'centro_custo_id': brw_tela.wizien_centro_custos_id.id,
                                                'valor': valor}]],
                                            'documento': etiqueta}, context=context)

        return True
