# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from openerp.osv import osv, fields
import time
from mx.DateTime import now


class cotacao_cotacoes(osv.Model):
    def _get_sit_itens(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        res={}
        for id in ids:
            #Pendente = ainda nao foi orcado nenhum item
            #Incompleto = alguns itens foram orcados mais ainda faltam outros
            #Completo = todos os itens foram orcados
            _sit_itens = 'Pendente'
            _pendente = False
            for item in self.pool.get('cotacao.cotacoes').browse(cr, uid, id).cotacoes_itens_ids:
                if item.orcado:
                    _sit_itens = 'Completo'
                else:
                    _pendente = True
            if _sit_itens == 'Completo' and _pendente:
                _sit_itens = 'Incompleto'
            res[id]=_sit_itens
        return res    
    _name = 'cotacao.cotacoes'
    _rec_name = 'nro_cotacao'
    _columns = {
        'empresa_id' : fields.many2one('adm.empresa.objeto', 'Empresa', ondelete='cascade', required=True),
        'fornecedor_id' : fields.many2one("financeiro.fornecedor", 'Fornecedor', ondelete="cascade", required=True, select = True),
        'nro_cotacao' : fields.integer('Nro cotacao', required=True, select = True),
        'data_importacao' : fields.date('Data importacao'),
        'data_cotacao' : fields.date('Data cotacao'),
        'estado' : fields.selection([('I','Importado'),('E','Enviado email'), ('C','Cotado'), ('X','Encerrada')], 'Situacao'),
        'situacao_itens' : fields.function(_get_sit_itens, method=True, string='Situacao', type='char', size=10, store=True),
        'prazo_entrega' : fields.date('Prazo entrega'),
        'validade' : fields.integer('Validade'),
        'prazo_pagto1' : fields.integer('Prazo pagto1'),
        'prazo_pagto2' : fields.integer('Prazo pagto2'),
        'prazo_pagto3' : fields.integer('Prazo pagto3'),
        'prazo_pagto4' : fields.integer('Prazo pagto4'),
        'prazo_pagto5' : fields.integer('Prazo pagto5'),
        'prazo_pagto6' : fields.integer('Prazo pagto6'),
        'comprador' : fields.integer('Comprador'),
        'cif_fob' : fields.selection([('C','Cif'), ('F','Fob')], 'Tipo Frete'),
        'transportadora' : fields.char('Transportadora', size=50),
        'algoritimo' : fields.char('Algoritimo', size=9, required=True, select=True),
        'cotacoes_itens_ids' : fields.one2many('cotacao.cotacoes.itens', 'cotacao_id','Itens'),
    }
    _defaults = {
        'data_importacao': lambda *a: time.strftime('%Y-%m-%d'),
        'estado': lambda *a: 'I',
    }
    def read(self, cr, uid, ids, fields=None, context=None,
            load='_classic_read'):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        cotacoes = super(cotacao_cotacoes, self).read(cr, uid, ids, fields=fields,
                context=context, load=load)
        res = cotacoes
        print 'read', ids, context
        if 'client' in context:
            if 'id_fornecedor' in context:
                if cotacoes:
                    if cotacoes[0]['fornecedor_id'][0] != context['id_fornecedor']:
                        res = []
            else:
                res = []
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        ids = super(cotacao_cotacoes, self).search(cr, uid, args, offset, limit, order, context=context, count=count)
        if 'client' in context:
            if 'id_fornecedor' in context:
                id = context['id_fornecedor']
                ids = super(cotacao_cotacoes, self).search(cr, uid, [('fornecedor_id','=',id)], context=context)
        return ids

class cotacao_cotacoes_itens(osv.Model):
    _name = 'cotacao.cotacoes.itens'
    _rec_name = 'grade_id'
    _columns = {
        'cotacao_id' : fields.many2one("cotacao.cotacoes", 'Cotacao', ondelete="cascade", required=True, select = True),
        'grade_id' : fields.many2one("cotacao.produtos.grades", 'Produto', ondelete="cascade", required=True, select = True),
        'valor' : fields.float('Valor', help='Informe o valor do produto/servico'),
        'frete' : fields.float('Frete', help="Informe o valor do frete"),
        'outros' : fields.float('Outros', help='Informe o valor de outras despesas, se houver'),
        'descontos' : fields.float('Descontos', help='Informe o valor dos descontos'),
        'encargos' : fields.float('Encargos', help='Informe dos encargos, se houver'),
        'despesas_acessorias' : fields.float('Despesas acessorias'),
        'icms' : fields.integer('% Icms', help='Informe o percentual do ICMS'),
        'ipi' : fields.integer('% Ipi', help='Informe o percentual do IPI'),
        'iss' : fields.integer('% Iss', help='Informe o percentual do ISS'),
        'garantia_item' : fields.char('Garantia', size=20),
        'orcado':fields.boolean('Orcado'),
        'total':fields.float('Valor Total'),
        'data_hora':fields.datetime('Data/Hora'),
    }
    _defaults = {
        'orcado': lambda *a: 0,
    }
    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        item = self.pool.get('cotacao.cotacoes.itens').read(cr, uid, ids, context=context)[0]
        if vals.get('valor'):
            _valor=vals['valor']
        else:
            _valor=item['valor']
        if vals.get('frete'):
            _frete=vals['frete']
        else:
            _frete=item['frete']
        if vals.get('outros'):
            _outros=vals['outros']
        else:
            _outros=item['outros']
        if vals.get('descontos'):
            _descontos=vals['descontos']
        else:
            _descontos=item['descontos']
        if vals.get('encargos'):
            _encargos=vals['encargos']
        else:
            _encargos=item['encargos']
        if vals.get('ipi'):
            _ipi=vals['ipi']
        else:
            _ipi=item['ipi']
        vals['orcado']=True
        vals['total'] = _valor + _frete + _outros - _descontos + _encargos + (_valor * _ipi / 100)
        vals['data_hora'] = time.strftime('%Y-%m-%d %H:%M:%S')
        result = super(cotacao_cotacoes_itens,self).write(cr, uid, ids, vals, context)
        return result
    def onchange_valor(self, cr, uid, ids, valor,frete,outros,descontos, encargos, ipi): 
        total = valor + frete + outros - descontos + encargos + (valor * ipi / 100)
        return {'value': {'total': total}}
cotacao_cotacoes_itens()

class cotacao_resultado(osv.Model):
    _name = 'cotacao.resultado'
    _rec_name = 'nro_cotacao'
    _columns = {
        'nro_cotacao' : fields.integer('Nro cotacao', required=True, select = True),
        'data_importacao' : fields.date('Data importacao'),
        'item_ids' : fields.one2many("cotacao.resultado.itens", 'resultado_id', "Itens"),
    }
    _order = 'nro_cotacao'
cotacao_resultado()

class cotacao_resultado_itens(osv.Model):
    _name = 'cotacao.resultado.itens'
    _rec_name = 'resultado_id'
    _columns = {
        'resultado_id' : fields.many2one("cotacao.resultado", 'Resultado', ondelete="cascade", required=True, select = True),
        'grade_id' : fields.many2one("cotacao.produtos.grades", 'Produto', ondelete="cascade", required=True, select = True),
        'fornecedor_ids' :fields.one2many("cotacao.resultado.fornecedores", 'resultado_item_id', "Fornecedores"),
    }
    _order = 'resultado_id, grade_id'
cotacao_resultado_itens()

class cotacao_resultado_fornecedores(osv.Model):
    _name = 'cotacao.resultado.fornecedores'
    _rec_name='resultado_item_id'
    _columns = {
        'resultado_item_id' : fields.many2one("cotacao.resultado.itens", 'Item', ondelete="cascade", required=True, select =True),
        'fornecedor_id' : fields.many2one("financeiro.fornecedor", 'Fornecedor', ondelete="cascade", required=True, select = True),
        'cotacao_item_id' : fields.many2one("cotacao.cotacoes.itens", 'Item', ondelete="cascade", required=True, select =True),
        'aprovado' :fields.boolean('Aprovado'),
        'total' : fields.float('total'),
    }
    _order = 'resultado_item_id, total, fornecedor_id'
    _defaults = {
        'aprovado': lambda *a: 0,
    }
    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        result = super(cotacao_resultado_fornecedores,self).write(cr, uid, ids, vals, context)
        for um in ids:
            if vals.get('aprovado'):
                if vals['aprovado']: #para uma aprovacao, todas as outras devem ser desmarcadas
                    oResFor=self.pool.get('cotacao.resultado.fornecedores')
                    a = oResFor.read(cr, uid, um, ['resultado_item_id'], context=context)
                    _resultado_item_id = a['resultado_item_id'][0]
                    sql ='update cotacao_resultado_fornecedores set aprovado = %s where resultado_item_id = %s' % (False, _resultado_item_id)
                    cr.execute(sql)   
                    sql = 'update cotacao_resultado_fornecedores set aprovado = %s where id = %s' % (True, ids[0])
                    cr.execute(sql)  
                    cr.commit()    
        return result
cotacao_resultado_fornecedores()
