# -*- encoding: utf-8 -*-
##############################################################################
#
# OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from datetime import timedelta, datetime
import time
from openerp import tools
from openerp.osv import osv
import mx.DateTime
import base64
from openerp.tools.translate import _
from SigUtils import numero
from dateutil.relativedelta import relativedelta
from dateutil.parser import parse
import logging

_logger = logging.getLogger(__name__)

MAX_LEVEL = 15
READ_LOAD = '_classic_write'


def _links_get(self, cr, uid, context=None):
    context = context and context.copy() or {}
    obj = self.pool.get('res.request.link')
    ids = obj.search(cr, uid, [], context=context)
    res = obj.read(cr, uid, ids, ['object', 'name'], context=context)
    return [(r['object'], r['name']) for r in res]


# noinspection PyUnresolvedReferences,PyUnresolvedReferences
class crm_br_case(osv.Model):
    _name = "crm_br.case"
    _inherit = "crm_br.case"

    def _crmcas_calcula_fim_da_atividade(self, cr, uid, inicio, duracao):
        """
        Calcular o fim de uma atividade, considerando:
        - dia de 8 horas em dois turnos: de 08 as 12 e de 14 as 18
        @param:
           - inicio: data de inicio da atividade do tipo string formato Y-M-D H:M:S
           - duracao: tempo em horas do tipo float
        @return:
            - Data final da atividade do tipo string formato Y-M-D H:M:S
        """
        obj_adm_utils = self.pool.get('adm.utils')
        if isinstance(inicio, datetime):
            inicio = inicio.strftime('%Y-%m-%d %H:%M:%S')
        _fim = _inicio = datetime.strptime(obj_adm_utils.admutl_datetime_to_timezone(cr, uid,
                                                                                     datetime.strptime(inicio[:19],
                                                                                                       "%Y-%m-%d %H:%M:%S")),
                                           "%d-%m-%Y %H:%M:%S")
        _fim = _fim + relativedelta(hours=+duracao)
        _fim_do_dia = _inicio + relativedelta(hour=18, minute=0, second=0)
        if (_inicio < (_inicio + relativedelta(hour=12, minute=0, second=0))
            and _fim > (_inicio + relativedelta(hour=12, minute=0, second=0))):  # Acrescentar duas horas do almoco
            _fim = _fim + relativedelta(hours=+2)
        while _fim > _fim_do_dia:
            _dif = _fim - _fim_do_dia
            while True:
                _fim = _inicio = (_inicio + relativedelta(days=+1, hour=8, minute=0, second=0))
                _dt = str(_fim)[:10]
                res = self.pool.get('adm.calendario.feriados').afmcfe_identificar_data(cr, uid, [], _dt)
                if res == 'U':  # se não for dia util(Exclui feriados e finais de semana)
                    break
            _fim = _fim + _dif
            if (_inicio < (_inicio + relativedelta(hour=12, minute=0, second=0))
                and _fim > (_inicio + relativedelta(hour=12, minute=0, second=0))):  # Acrescentar duas horas do almoco
                _fim = _fim + relativedelta(hours=+2)
            _fim_do_dia = _inicio + relativedelta(hour=18, minute=0, second=0)
        _fim = obj_adm_utils.admutl_converte_tz(cr, uid, _fim, None, 'UTC')
        return _fim.strftime('%Y-%m-%d %H:%M:%S')

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        res = super(crm_br_case, self).fields_view_get(cr, uid, view_id, view_type, context,
                                                       toolbar, submenu)
        # TRATAMENTO FEITO PARA A SEARCH VIEW DO CRM_BR_CASE
        # AO DIGITAR UMA CATEGORIA OCORRIA O ERRO NO DOMINIO DO SECTION_ID.
        # ABAIXO O DOMINIO É CRIADO JA INDICANDO QUAL O ID DO SECTION.
        # COMO A CATEGORIA DEPENDE DE SEÇÃO, ISSO ESTAVA GERANDO UM ERRO NA SEARCH_VIEW
        # ANTERIORMENTE -> [('section_id', '=', section_id]
        # AGORA -> [('section_id', '=', ID] (JA POSSUI O ID DA SEÇÃO DE ACOMPANHAMENTO DE PROBLEMA)
        if context.get('object_id') == 'crm_br.help_desk':
            if res and res.get('fields') and res.get('fields').get('pessoa_id') and 'context' in res.get('fields').get(
                    'pessoa_id'):
                res['fields']['pessoa_id']['context']['cliente'] = True
        secao = context.get('section_id') or ''
        if secao and view_type == 'search':
            categ_id_estrutura = {
                'string': u'Categoria',
                'views': {},
                'relation': 'crm_br.case.categ',
                'context': {},
                'selectable': True,
                'type': 'many2one'
            }
            section_ids = self.pool.get('crm_br.case.section').search(cr, uid,
                                                                      [('code', '=', secao)], context=context)
            if section_ids:
                args_section = [('section_id', '=', section_ids[0])]
                categ_id_estrutura.update({'domain': str(args_section)})
            res['fields']['categ_id'] = categ_id_estrutura
        return res

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(crm_br_case, self).default_get(cr, uid, fields_list, context)
        novo_help_desk = False
        section = context.get('section_id') or context.get('object_id')
        all_help_desk = context.get('all_help_desk')
        if section in ('HD', 'nivel2', 'crm_br.help_desk') and not all_help_desk:
            novo_help_desk = True
        if section in ('HD', 'nivel2', 'crm_br.help_desk'):
            res['crmcas_section_code'] = section
        res['crmcas_novo_helpdesk'] = novo_help_desk
        categoria = context.get('action_categ_id')
        if not categoria:
            return res
        secao = context.get('section_id')
        if not secao:
            return res
        obj_case_categ = self.pool.get('crm_br.case.categ')
        obj_case_section = self.pool.get('crm_br.case.section')
        secao_id = obj_case_section.search(cr, uid, [('code', '=', secao)], context=context)
        if secao_id:
            categoria_id = obj_case_categ.search(cr, uid, [('section_id', '=', secao_id[0]),
                                                           ('name', '=', categoria)], context=context)
            if categoria_id:
                res['categ_id'] = categoria_id[0]
        return res

    def check_section_bugsup(self, cr, uid, section_id, context=None):
        context = context and context.copy() or {}

        args = [('id', '=', section_id), ('code', '=', 'BugSup')]
        obj_crm_section = self.pool.get('crm_br.case.section')
        section_ids = obj_crm_section.search(cr, uid, args, context=context)
        return section_ids

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if isinstance(vals, tuple):
            vals = vals[0]
        if 'create_uid' in vals:
            del vals['create_uid']
        pessoa_id = vals.get('pessoa_id')
        if context.get('object_id') == 'crm_br.help_desk' and \
                (context.get('section_id') in ('HD', 'nivel2') or context.get('all_help_desk')):
            self._crmcas_valida_pessoa_cliente(cr, uid, pessoa_id, context=context)
        res = super(crm_br_case, self).create(cr, uid, vals, context)
        cases = self.browse(cr, uid, [res])
        self.__log(cr, uid, cases, 'draft', context.update({'translated_keyword': _('draft')}))
        self._action(cr, uid, cases, 'draft')
        return res

    def _crmcas_valida_pessoa_cliente(self, cr, uid, pessoa_id, context=None):
        """
        O método identifica se a pessoa informada é ou não um cliente.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param pessoa_id: id da tabela adm_pessoa.
        :param context: dicionário com valores contextuais.
        :return: booleano. True, caso a pessoa seja um cliente. False, caso a pessoa nao tenha sido
        informada ou o módulo financeiro não esteja instalado.
        """
        context = context and context.copy() or {}
        res = False
        if not pessoa_id:
            return res
        obj_adm_util = self.pool.get('adm.utils')
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        financeiro_instalado = obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'financeiro',
                                                                          context)
        if not financeiro_instalado:
            return res
        cliente_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'financeiro.cliente', context=context)
        is_cliente = obj_financeiro_cliente.verificar_incluir_cliente(
            cr, uid, pessoa_id, cliente_company_id, 'verificar', context)
        if not is_cliente:
            nome_pessoa = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id).nome
            raise osv.except_osv(_('Atencao'), _('%s nao e um cliente.' % nome_pessoa))
        res = True
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('painel_helpdesk_12') or context.get('painel_helpdesk_15_fechados'):
            obj_crm_section = self.pool.get('crm_br.case.section')
            args_section = [('code', 'in', ('HD', 'nivel2'))]
            section_ids = obj_crm_section.search(cr, uid, args_section, context=context)
            args.append(('section_id', 'in', section_ids))
            order = 'date desc'
            if context.get('painel_helpdesk_12'):
                limit = 12
            if context.get('painel_helpdesk_15_fechados'):
                limit = 15
        if context.get('painel_helpdesk_nivel1'):
            args.append(('total_tempo_atend', '>', 0.25))
            order = 'date desc'
            limit = 15
            args_section = [('code', '=', 'HD')]
            obj_crm_section = self.pool.get('crm_br.case.section')
            section_ids = obj_crm_section.search(cr, uid, args_section, context=context)
            args.append(('section_id', 'in', section_ids))
        if context.get('painel_helpdesk_atendimento_andamento'):
            order = 'date desc'
            limit = 15
            args_section = [('code', 'in', ('HD', 'nivel2'))]
            obj_crm_section = self.pool.get('crm_br.case.section')
            section_ids = obj_crm_section.search(cr, uid, args_section, context=context)
            args.append(('section_id', 'in', section_ids))
        if context.get('painel_atendimento_aberto'):
            limit = 12

        res = super(crm_br_case, self).search(cr, uid, args, offset, limit, order, context=context, count=count)
        return res

    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not isinstance(default, dict):
            default = {}
        if 'id' in default:
            del default['id']
        default.update({
            'state': 'draft',
            'history_line': [],
            'log_ids': []
        })
        return super(crm_br_case, self).copy(cr, uid, id, default, context)

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        if not fields:
            fields = []

        registros = super(crm_br_case, self).read(cr,uid, ids, fields=fields, context=context,
                                                  load=load)

        return registros

    def crm_onchange_pessoa(self, cr, uid, ids, pessoa_id):
        vals = {}
        vals['crmcas_contato_id'] = ''
        return {'value': vals}

    def onchange_mail(self, cr, uid, ids, contato_id, pessoa_id, context=None):
        """
            Busca o e-mail do contato
        """
        context = context and context.copy() or {}
        vals = {}

        if contato_id:
            bwr_contato = self.pool.get('adm.pessoa').browse(cr, uid, contato_id, context=context)
            email = bwr_contato.email

            if email:
                email_ids = self.pool.get('adm.pessoa.email').search(cr, uid,
                                                                     [('name', 'ilike', email)],
                                                                     context=context)
                if email_ids:
                    vals['email_from_id'] = email_ids[0]
            else:
                vals['email_from_id'] = False
        else:
            vals['email_from_id'] = False

        return {'value': vals}

    def _get_default_pessoa_address(self, cr, uid, context):
        if not context.get('portal', False):
            return False
        return self.pool.get('res.users').browse(cr, uid, uid, context).pessoa_id.id

    def _get_default_pessoa(self, cr, uid, context):
        if not context.get('portal', False):
            return False
        user = self.pool.get('res.users').browse(cr, uid, uid, context)
        if not user.pessoa_id:
            return False
        return user.pessoa_id.id

    def onchange_data_atendimento(self, cr, uid, ids, date, categ_id, context=None):
        context = context and context.copy() or {}
        ret = {'value': {}}
        if not date:
            ret['value']['date_deadline'] = False
        if categ_id:
            tempo = self.pool.get('crm_br.case.categ').read(cr, uid, categ_id, ['tempo'], context=context)['tempo']
            ret['value']['date_deadline'] = False
            if tempo > 0:
                dat_conv = datetime.fromtimestamp(
                    time.mktime(time.strptime(date, "%Y-%m-%d %H:%M:%S")))

                dat = dat_conv
                data_nova = dat + timedelta(hours=tempo)

                d = str(data_nova)
                ret['value']['date_deadline'] = d
        return ret

    def _get_default_email(self, cr, uid, context):
        if not context.get('portal', False):
            return False
        user = self.pool.get('res.users').browse(cr, uid, uid, context)
        if not user.pessoa_id:
            return False
        return user.pessoa_id.email

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for case in self.browse(cr, uid, ids, context):
            if (not case.section_id.allow_unlink) and (case.state <> 'draft'):
                raise osv.except_osv(_('Warning !'),
                                     _(
                                         'You can not delete this case. You should better cancel it.'))

        res = super(crm_br_case, self).unlink(cr, uid, ids, context=context)

        return res

    def _action(self, cr, uid, cases, state_to, scrit=None, context=None):
        context = context and context.copy() or {}
        if not scrit:
            scrit = []
        action_ids = self.pool.get('crm_br.case.rule').search(cr, uid, scrit, context=context)
        level = MAX_LEVEL
        while len(action_ids) and level:
            newactions = []
            actions = self.pool.get('crm_br.case.rule').browse(cr, uid, action_ids, context)
            for case in cases:
                for action in actions:
                    ok = True
                    ok = ok and (not action.trg_state_from or action.trg_state_from == case.state)
                    ok = ok and (not action.trg_state_to or action.trg_state_to == state_to)
                    ok = ok and (
                        not action.trg_section_id or action.trg_section_id.id == case.section_id.id)
                    ok = ok and (
                        not action.trg_stage_id or action.trg_stage_id.id == case.stage_id.id)
                    ok = ok and (
                        not action.trg_categ_id or action.trg_categ_id.id == case.categ_id.id)
                    ok = ok and (
                        not action.trg_user_id.id or action.trg_user_id.id == case.user_id.id)
                    ok = ok and (
                        not action.trg_pessoa_id.id or action.trg_pessoa_id.id == case.pessoa_id.id)
                    ok = ok and (not action.trg_max_history or action.trg_max_history <= (
                        len(case.history_line) + 1))
                    ok = ok and (
                        not action.trg_pessoa_categ_id.id or
                        (
                            case.pessoa_id.id and
                            (action.trg_pessoa_categ_id.id in map(lambda x: x.id,
                                                                  case.pessoa_id.id or []))
                        )
                    )
                    ok = ok and (
                        not action.trg_priority_from or action.trg_priority_from >= case.priority)
                    ok = ok and (
                        not action.trg_priority_to or action.trg_priority_to <= case.priority)
                    if not ok:
                        continue

                    base = False
                    if action.trg_date_type == 'create':
                        base = mx.DateTime.strptime(case.create_date[:19], '%Y-%m-%d %H:%M:%S')
                    elif action.trg_date_type == 'action_last':
                        if case.date_action_last:
                            base = mx.DateTime.strptime(case.date_action_last, '%Y-%m-%d %H:%M:%S')
                        else:
                            base = mx.DateTime.strptime(case.create_date[:19], '%Y-%m-%d %H:%M:%S')
                    elif action.trg_date_type == 'deadline' and case.date_deadline:
                        base = mx.DateTime.strptime(case.date_deadline, '%Y-%m-%d %H:%M:%S')
                    elif action.trg_date_type == 'date' and case.date:
                        base = mx.DateTime.strptime(case.date, '%Y-%m-%d %H:%M:%S')
                    if base:
                        fnct = {
                            'minutes': lambda interval: mx.DateTime.RelativeDateTime(
                                minutes=interval),
                            'day': lambda interval: mx.DateTime.RelativeDateTime(days=interval),
                            'hour': lambda interval: mx.DateTime.RelativeDateTime(hours=interval),
                            'month': lambda interval: mx.DateTime.RelativeDateTime(months=interval),
                        }
                        d = base + fnct[action.trg_date_range_type](action.trg_date_range)
                        dt = d.strftime('%Y-%m-%d %H:%M:%S')
                        ok = (dt <= time.strftime('%Y-%m-%d %H:%M:%S')) and \
                             ((not case.date_action_next) or \
                              (dt >= case.date_action_next and \
                               case.date_action_last < case.date_action_next))
                        if not ok:
                            if not case.date_action_next or dt < case.date_action_next:
                                case.date_action_next = dt
                                self.write(cr, uid, [case.id], {'date_action_next': dt}, context)

                    else:
                        ok = action.trg_date_type == 'none'

                    if ok:
                        write = {}
                        if action.act_state:
                            case.state = action.act_state
                            write['state'] = action.act_state
                        if action.act_section_id:
                            case.section_id = action.act_section_id
                            write['section_id'] = action.act_section_id.id
                        if action.act_user_id:
                            case.user_id = action.act_user_id
                            write['user_id'] = action.act_user_id.id
                        if action.act_priority:
                            case.priority = action.act_priority
                            write['priority'] = action.act_priority
                        if action.act_email_cc:
                            if '@' in (case.email_cc or ''):
                                emails = case.email_cc.split(",")
                                if action.act_email_cc not in emails:  # and '<'+str(action.act_email_cc)+">" not in emails:
                                    write['email_cc'] = case.email_cc + ',' + action.act_email_cc
                            else:
                                write['email_cc'] = action.act_email_cc
                        write['date_action_last'] = time.strftime('%Y-%m-%d %H:%M:%S')
                        self.write(cr, uid, [case.id], write, context)
                        caseobj = self.pool.get('crm_br.case')
                        if action.act_remind_user:
                            caseobj.remind_user(cr, uid, [case.id], context,
                                                attach=action.act_remind_attach)
                        if action.act_remind_pessoa:
                            caseobj.remind_pessoa(cr, uid, [case.id], context,
                                                  attach=action.act_remind_attach)
                        if action.act_method:
                            getattr(caseobj, 'act_method')(cr, uid, [case.id], action, context)
                        emails = []
                        if action.act_mail_to_user:
                            if case.user_id and case.user_id.pessoa_id:
                                emails.append(case.user_id.pessoa_id.email)
                        if action.act_mail_to_pessoa:
                            emails.append(case.email_from)
                        if action.act_mail_to_watchers:
                            emails += (action.act_email_cc or '').split(',')
                        if action.act_mail_to_email:
                            emails += (action.act_mail_to_email or '').split(',')
                        emails = filter(None, emails)
                        if len(emails) and action.act_mail_body:
                            emails = list(set(emails))
                            self.email_send(cr, uid, case, emails, action.act_mail_body)
                        break
            action_ids = newactions
            level -= 1
        return True

    def format_body(self, body):
        return (body or u'').encode('utf8', 'replace')

    def format_mail(self, case, body):
        data = {
            'case_id': case.id,
            'case_subject': case.name,
            'case_date': case.date,
            'case_description': case.description,

            'case_user': (case.user_id and case.user_id.name) or '/',
            'case_user_email': (
                                   case.user_id and case.user_id.pessoa_id and case.user_id.pessoa_id.email) or '/',
            'case_user_phone': (
                                   case.user_id and case.user_id.pessoa_id and case.user_id.pessoa_id.telefone) or '/',

            'email_from_id': case.email_from_id,
            'pessoa': (case.pessoa_id and case.pessoa_id.name) or '/',
            'adm_pessoa_email': (case.pessoa_id and case.pessoa_id.email) or '/',
        }
        return self.format_body(body % data)

    def email_send(self, cr, uid, case, emails, body, context=None):
        context = context and context.copy() or {}
        body = self.format_mail(case, body)
        if case.user_id and case.user_id.pessoa_id and case.user_id.email:
            emailfrom = case.user_id.email
        else:
            emailfrom = case.section_id.reply_to
        name = '[%d] %s' % (case.id, case.name.encode('utf8'))
        reply_to = case.section_id.reply_to or False
        if reply_to: reply_to = reply_to.encode('utf8')
        if not emailfrom:
            raise osv.except_osv(_('Error!'),
                                 _(
                                     "No E-Mail ID Found for your Company address or missing reply address in section!"))
        _logger.info("tools.email_send - From:%s, To:%s, Subject:%s" % (
            emailfrom, emails, unicode(name, 'utf-8')))
        tools.email_send(emailfrom, emails, name, body, reply_to=reply_to,
                         openobject_id=str(case.id))
        return True

    def __log(self, cr, uid, cases, keyword, context=None):
        context = context and context.copy() or {}
        if not self.pool.get('crm_br.event.type').check(cr, uid, 'crm_br_case_' + keyword):
            return False
        for case in cases:
            if case.pessoa_id:
                translated_keyword = keyword
                if context.get('translated_keyword', False):
                    translated_keyword = context['translated_keyword']
                name = _('Case') + ' ' + translated_keyword + ': ' + case.name
                if isinstance(name, str):
                    name = unicode(name, 'utf-8')
                if len(name) > 64:
                    name = name[:61] + '...'
                self.pool.get('crm_br.event').create(cr, uid, {
                    'name': name,
                    'som': (case.som or False) and case.som.id,
                    'description': case.description,
                    'pessoa_id': case.pessoa_id.id,
                    'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'canal_id': (case.canal_id or False) and case.canal_id.id,
                    'user_id': uid,
                    'document': 'crm_br.case,%i' % case.id,
                }, context)
        return True

    def write(self, cr, uid, ids, values, context=None):
        context = context and context.copy() or {}
        pessoa_id = values.get('pessoa_id')
        if context.get('object_id') == 'crm_br.help_desk' and \
                (context.get('section_id') in ('HD', 'nivel2') or context.get('all_help_desk')):
            self._crmcas_valida_pessoa_cliente(cr, uid, pessoa_id, context=context)
        vls_stage_id = values.get('stage_id')
        vls_priority = values.get('priority')
        if vls_stage_id or vls_priority:
            if isinstance(ids, (int, long)):
                ids = [ids]
            cases = self.browse(cr, uid, ids)
            list_dados_email = []
            for case in cases:
                if case.section_id.code == 'BugSup':
                    emailfrom = case.user_id.email or False
                    if not emailfrom:
                        emailfrom = case.user_id.pessoa_id and case.user_id.pessoa_id.email or False
                    if not emailfrom:
                        continue
                    list_email = []
                    email_dono_solicitacao = case.create_uid.email
                    if email_dono_solicitacao and not email_dono_solicitacao in list_email:
                        list_email.append(email_dono_solicitacao)
                    if case.email_cc:
                        list_email.extend(case.email_cc.split(','))
                    atividade = case.id
                    titulo = case.name
                    estagio_anterior = case.stage_id.name
                    novo_estagio = self.pool.get('crm_br.case.stage').browse(cr, uid,
                                                                             vls_stage_id).name
                    assunto = '[%d] %s' % (case.id, case.name.encode('utf8'))
                    _action_ids = self.pool.get('ir.actions.act_window').search(cr, uid, [('name', '=', 'All Bugs')], context=context)
                    if _action_ids:
                        body = _("""
                                Atividade: <a href="http://erp.sigsolucoes.net.br#id=%s&view_type=form&model=crm_br.case&action=%s">%s</a><br />
                                Titulo: %s<br />
                                Estagio anterior: %s<br />
                                Novo estagio: %s
                            """) % (str(atividade), str(_action_ids[0]), str(atividade), titulo, estagio_anterior, novo_estagio)
                    else:
                        body = _("""
                                Atividade: %s<br />
                                Titulo: %s<br />
                                Estagio anterior: %s<br />
                                Novo estagio: %s
                            """) % (str(atividade), titulo, estagio_anterior, novo_estagio)
                    list_dados_email.append({'dados': body, 'emails': list_email,
                                             'email_from': emailfrom, 'assunto': assunto})
            obj_mail_message = self.pool.get('mail.message')
            obj_mail_mail = self.pool.get('mail.mail')
            mail_server_id = False
            mail_server_ids = self.pool.get('ir.mail_server').search(cr, uid, [], context=context)
            if mail_server_ids:
                mail_server_id = mail_server_ids[0]
            for x in list_dados_email:
                body = x.get('dados')
                emails = x.get('emails')
                email_from = x.get('email_from')
                assunto = x.get('assunto')
                for email in emails:
                    mail_message_id = obj_mail_message.create(
                        cr, uid, {'email_from': email_from, 'subject': assunto}, context=context)
                    vals_mail = {
                        'mail_message_id': mail_message_id,
                        'mail_server_id': mail_server_id,
                        'email_from': email_from,
                        'email_to': email,
                        'body_html': body,
                    }
                    obj_mail_mail.create(cr, uid, vals_mail, context=context)
        res = super(crm_br_case, self).write(cr, uid, ids, values, context=context)
        if vls_stage_id or vls_priority:
            if isinstance(ids, (int, long)):
                ids = [ids]
            crm_dados = self.read(cr, uid, ids, ['stage_id', 'section_id', 'crmcal_versao_liberar_id'], context=context,
                                  load=READ_LOAD)
            create_crm_br_case_log = self.pool.get('crm_br.case.log').create
            for i in crm_dados:
                crm_stage_id = i.get('stage_id')
                crm_id = i['id']
                crm_section_id = i.get('section_id')
                vals_crm_log = {
                    'name': _('Historiar'),
                    'crmcal_date': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'crmcal_user_id': uid,
                    'crmcal_case_id': crm_id,
                    'crmcal_stage_id': vls_stage_id or crm_stage_id,
                    'crmcal_section_id': crm_section_id,
                    'crmcal_versao_liberar_id': i.get('versao_liberar_id')
                }
                create_crm_br_case_log(cr, uid, vals_crm_log, context=context)
        return res

    def _crmbr_calcula_tempo_abertura_atual(self, cr, uid, case_id):
        """
        Calcular o tempo de atendimento do caso. Usado quando o caso está em atendimento no momento.
        Não deve ser chamado se o caso não estiver no estado "open"
        @param cr:
        @param uid:
        @param case_id: id do caso
        @return: Tempo de atendimento desde a abertura até o momento
        """
        _sql = 'select crmcal_date from crm_br_case_log where crmcal_case_id = %s order by id desc limit 1' % case_id
        cr.execute(_sql)
        _abriu_em = cr.fetchall()
        _tempo = 0.0
        if _abriu_em:
            _abriu_em = _abriu_em[0][0]
            obj_adm_utils = self.pool.get('adm.utils')
            _abriu_em = datetime.strptime(obj_adm_utils.admutl_datetime_to_timezone(cr, uid,
                                                                                    datetime.strptime(_abriu_em[:19],
                                                                                                      "%Y-%m-%d %H:%M:%S")),
                                          "%d-%m-%Y %H:%M:%S")
            _agora = datetime.strptime(obj_adm_utils.admutl_datetime_to_timezone(cr, uid,
                                                                                 datetime.now()), "%d-%m-%Y %H:%M:%S")
            _tempo = _agora - _abriu_em
            _tempo = _tempo.total_seconds() / 3600
        return _tempo  # tempo em horas

    def crmbrc_calcula_inicio_previsto(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = {}
        _section_id = self.pool.get('crm_br.case.section').search(cr, uid, [('code', '=', 'BugSup')], context=context)
        if not _section_id:
            return True
        _section_id = _section_id[0]
        _stage_id = self.pool.get('crm_br.case.stage').search(cr, uid, [('name', '=', 'Atendimento'),
                                                                        ('section_id', '=', _section_id)],
                                                              context=context)
        if not _stage_id:
            return True
        _stage_id = _stage_id[0]
        _sql = "select distinct(user_id) from crm_br_case where section_id = %s and stage_id = %s and state in ('pending', 'parador', 'open')" % (
        _section_id, _stage_id)
        cr.execute(_sql)
        _user_ids = cr.fetchall()
        for user_id in _user_ids:
            if isinstance(user_id, tuple):
                user_id = user_id[0]
            _args = [('user_id', '=', user_id),
                     ('state', 'in', ('pending', 'open', 'parado')),
                     ('stage_id', '=', _stage_id),
                     ('section_id', '=', _section_id)]
            case_ids = self.search(cr, uid, _args, context=context)
            browse_case_ids = self.browse(cr, uid, case_ids)
            for case in browse_case_ids:
                if browse_case_ids.index(case) == 0:
                    vals['date_deadline'] = datetime.now()
                    self.write(cr, uid, [case.id], vals, context=context)
                else:
                    _brwanterior_case_id = browse_case_ids[browse_case_ids.index(case) - 1]
                    _duracao = _brwanterior_case_id.crmcas_duration or 0.0
                    _sql = ('select sum(crmcal_tempo_atend) from crm_br_case_log '
                            'where crmcal_case_id = %s and crmcal_stage_id = %s') % \
                           (_brwanterior_case_id.id, _stage_id)
                    cr.execute(_sql)
                    _tempo_usado = cr.fetchone()
                    if _tempo_usado:
                        _tempo_usado = _tempo_usado[0]
                        if _brwanterior_case_id.state == 'open':
                            _dif = self._crmbr_calcula_tempo_abertura_atual(cr, uid, _brwanterior_case_id.id)
                            _tempo_usado += _dif
                        if _tempo_usado < _duracao:
                            _duracao -= _tempo_usado
                        else:
                            _duracao = 0.0
                    _fim = vals.get('date_deadline')
                    _fim = self._crmcas_calcula_fim_da_atividade(cr, uid, _fim, _duracao)
                    vals['date_deadline'] = _fim
                    self.write(cr, uid, [case.id], vals, context=context)
        return True

    def remind_pessoa(self, cr, uid, ids, context=None, attach=False):
        context = context and context.copy() or {}
        return self.remind_user(cr, uid, ids, context, attach,
                                destination=False)

    def remind_user(self, cr, uid, ids, context=None, attach=False,
                    destination=True):
        context = context and context.copy() or {}
        for case in self.browse(cr, uid, ids):
            if case.section_id.reply_to and case.email_from:
                src = case.email_from

                if not src:
                    raise osv.except_osv(_('Error!'),
                                         _(
                                             "No E-Mail ID Found for the Responsible pessoa or missing reply address in section!"))

                dest = case.section_id.reply_to
                body = case.email_last or case.description
                if not destination:
                    src, dest = dest, src
                    if case.user_id.signature:
                        body += '\n\n%s' % (case.user_id.signature)
                dest = [dest]

                attach_to_send = None

                if attach:
                    attach_ids = self.pool.get('ir.attachment').search(cr, uid, [
                        ('res_model', '=', 'crm_br.case'),
                        ('res_id', '=', case.id)], context=context)
                    attach_to_send = self.pool.get('ir.attachment').read(cr, uid, attach_ids,
                                                                         ['datas_fname', 'datas'], context=context)
                    attach_to_send = map(
                        lambda x: (x['datas_fname'], base64.decodestring(x['datas'])),
                        attach_to_send)

                # Send an email
                tools.email_send(
                    src,
                    dest,
                    "Reminder: [%s] %s" % (str(case.id), case.name,),
                    self.format_body(body),
                    reply_to=case.section_id.reply_to,
                    openobject_id=str(case.id),
                    attach=attach_to_send
                )

        return True

    def add_reply(self, cursor, user, ids, context=None):
        context = context and context.copy() or {}
        for case in self.browse(cursor, user, ids, context=context):
            if case.history_line:
                description = case.history_line[0].description
                self.write(cursor, user, case.id, {
                    'description': '> ' + description.replace('\n', '\n> '),
                }, context=context)
        return True

    def case_log(self, cr, uid, ids, context=None, email=False, *args):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        vals = {'description': False}

        # pegar e-mail do usuário atual
        brw_user = self.pool.get('res.users').browse(cr, uid, uid)
        if isinstance(brw_user, list):
            brw_user = brw_user[0]
        emails = brw_user.pessoa_id.email_ids
        if any(e for e in emails if e.email_tipo_id.name == 'Comum'):
            email = filter(lambda e: e.email_tipo_id.name == 'Comum', emails)[0].name
        elif emails:
            email = emails[0].name
        else:
            email = False

        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                break
        self._crmcas_history(cr, uid, cases, _('Historiar'), history=True, email=email,
                             context=context)
        return self.write(cr, uid, ids, vals, context=context)

    def case_log_reply(self, cr, uid, ids, context=None, email=False, *args):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        for case in cases:
            if not case.user_id:
                raise osv.except_osv(_('Error!'),
                                     _(
                                         'You must define a responsible user for this case in order to use this action!'))
            if not case.user_id.pessoa_id.email:
                raise osv.except_osv(_('Error!'),
                                     _('You must put a pessoa eMail to use this action!'))
            if not case.description:
                raise osv.except_osv(_('Error!'),
                                     _(
                                         'Can not send mail with empty body,you should have description in the body'))
        # self._crmcas_history(cr, uid, cases, _('Send'), history=True, email=False)
        obj_mail_message = self.pool.get('mail.message')
        obj_mail_mail = self.pool.get('mail.mail')
        obj_case_history = self.pool.get('crm_br.case.history')
        obj_case_log = self.pool.get('crm_br.case.log')
        brw_res_users = self.pool.get('res.users').browse(cr, uid, uid, context)
        mail_server_id = False
        mail_server_ids = self.pool.get('ir.mail_server').search(cr, uid, [], context=context)
        if mail_server_ids:
            mail_server_id = mail_server_ids[0]
        for case in cases:
            if context.get('section_id') == 'BugSup':
                email_to = case.create_uid.email
            else:
                email_to = case.email_from_id.name
            if not email_to:
                continue
            self.write(cr, uid, [case.id], {
                'description': False
            }, context=context)
            emails = [case.user_id.pessoa_id.email]
            if case.create_uid.user_id and case.create_uid.user_id.pessoa_id and case.create_uid.user_id.pessoa_id.email:
                emails.append(case.create_uid.user_id.pessoa_id.email)
            if case.email_cc:
                emails.append((case.email_cc or '').split(','))
            if case.crmcas_contato_id and case.crmcas_contato_id.email:
                emails.append(case.crmcas_contato_id.email)
            emails = filter(None, emails)
            body = case.description or ''
            body = body.replace('\n', '<br />')
            if brw_res_users.signature:
                signature = brw_res_users.signature
                signature = signature.replace('\n', '<br />')
                body += '<br /><br />%s' % signature
            emailfrom = brw_res_users.pessoa_id.email or False
            if not emailfrom:
                raise osv.except_osv(_('Error!'),
                                     _("No E-Mail ID Found for your Company address!"))
            if context.get('section_id') == 'BugSup':
                assunto_email = '[%d] %s' % (case.id, case.name.encode('utf8'))
            else:
                assunto_email = 'Helpdesk e Suporte: Protocolo %s' % case.id
            mail_message_id = obj_mail_message.create(
                cr, uid, {'email_from': emailfrom, 'subject': assunto_email}, context)
            vals_mail = {
                'mail_message_id': mail_message_id,
                'mail_server_id': mail_server_id,
                'email_from': emailfrom,
                'email_to': email_to,
                'body_html': body,
            }
            obj_mail_mail.create(cr, uid, vals_mail, context)
            # Cria log com os dados do email
            data = {
                'name': _('Historiar'),
                'crmcal_som': case.som.id,
                'crmcal_canal_id': case.canal_id.id,
                'crmcal_user_id': uid,
                'crmcal_date': time.strftime('%Y-%m-%d %H:%M:%S'),
                'crmcal_case_id': case.id,
                'crmcal_section_id': case.section_id.id,
                'crmcal_stage_id': case.stage_id.id
            }
            log_id = obj_case_log.create(cr, uid, data, context)
            if case.description:
                data = {
                    'description': 'E-mail enviado. De: %s, para: %s. Descricao: %s' % (
                        emailfrom, email_to, case.description),
                    'email': email or case.user_id.login or (
                        case.user_id and case.user_id.pessoa_id and
                        case.user_id.pessoa_id.email) or False,
                    'log_id': log_id
                }
                obj_case_history.create(cr, uid, data, context)
        self.write(cr, uid, ids, {'description': False}, context)
        return True

    def onchange_categ_id(self, cr, uid, ids, categ, context=None):
        context = context and context.copy() or {}
        return {'value': {}}

    def onchange_data_atendimento(self, cr, uid, ids, date, categ_id, context=None):
        context = context and context.copy() or {}
        ret = {'value': {}}
        if not date:
            ret['value']['date_deadline'] = False
        if categ_id:
            tempo = self.pool.get('crm_br.case.categ').read(cr, uid, categ_id, ['tempo'], context=context)['tempo']
            ret['value']['date_deadline'] = False
            if tempo > 0:
                dat_conv = datetime.fromtimestamp(
                    time.mktime(time.strptime(date, "%Y-%m-%d %H:%M:%S")))

                dat = dat_conv
                data_nova = dat + timedelta(hours=tempo)

                d = str(data_nova)
                ret['value']['date_deadline'] = d
        return ret

    def case_close(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache

        for i in cases:
            if (i.section_id.code == 'HD' or i.section_id.code == 'nivel2') and not i.categ_id:
                raise osv.except_osv(('Atencao'),
                                     ('Para fechar o protocolo, o campo categoria deve estar preenchido!'))
            section_id = i.section_id and i.section_id.id or 0
            section_bugsup_ids = self.check_section_bugsup(cr, uid, section_id)
            if section_bugsup_ids:
                versao_cliente_id = i.category2_id and i.category2_id.id or False
                versao_liberar_id = i.versao_liberar_id and i.versao_liberar_id.id or False

                msg = ''
                if not versao_liberar_id:
                    msg += _('Campo Liberar na Versao nao informado! ')
                if not versao_cliente_id:
                    msg += _('Campo Versao no Cliente nao informado! ')
                if msg:
                    raise osv.except_osv(_('Atencao'), msg)

        self.__log(cr, uid, cases, 'done', context.update({'translated_keyword': _('done')}))
        self._crmcas_history(cr, uid, cases, _('Close'))
        vals = {
            'state': 'done',
            'date_closed': time.strftime('%Y-%m-%d %H:%M:%S')
        }
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                break
        self.write(cr, uid, ids, vals, context=context)

        #
        # We use the cache of cases to keep the old case state
        #
        self._action(cr, uid, cases, 'done')
        return True

    def case_escalate(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        for case in cases:
            data = {'active': True}
            if case.section_id.parent_id:
                data['section_id'] = case.section_id.parent_id.id
                if case.section_id.parent_id.user_id:
                    data['user_id'] = case.section_id.parent_id.user_id.id
            else:
                raise osv.except_osv(_('Error !'),
                                     _(
                                         'You can not escalate this case.\nYou are already at the top level.'))
            self.write(cr, uid, ids, data, context=context)
        cases = self.browse(cr, uid, ids, context=context)
        vals = {'state': 'draft'}
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                break
        self._crmcas_history(cr, uid, cases, _('Escalate'))
        self._action(cr, uid, cases, 'escalate', context=context)
        self.write(cr, uid, ids, vals, context=context)
        return True

    def case_open(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        for i in cases:
            section_id = i.section_id and i.section_id.id or 0
            section_bugsup_ids = self.check_section_bugsup(cr, uid, section_id)
            if section_bugsup_ids:
                versao_cliente_id = i.category2_id and i.category2_id.id or False
                if not versao_cliente_id:
                    raise osv.except_osv(_('Atencao'), _('Campo Versao no Cliente nao informado!'))

        self.__log(cr, uid, cases, 'open', context.update({'translated_keyword': _('open')}))
        self._crmcas_history(cr, uid, cases, _('Open'))
        obj_crm_br_case_stage = self.pool.get('crm_br.case.stage')
        for case in cases:
            vals = {'state': 'open', 'active': True}
            section_code = case.section_id.code
            if section_code == 'BugSup':
                vals['date'] = datetime.now()
                if case.state == 'open':
                    raise osv.except_osv(_('Atencao'),
                                         _('Este problema ja esta aberto!'))

                if case.state == 'draft':
                    simulacao = obj_crm_br_case_stage.search(cr, uid,
                                                             [('name', '=', _('Simulacao'))], context=context)

                    if not simulacao:
                        section = self.pool.get('crm_br.case.section').search(cr, uid, [
                            ('code', '=', 'BugSup')], context=context)
                        if isinstance(section, list):
                            section = section[0]
                        vals = {
                            'name': _('Simulacao'),
                            'section_id': section,
                        }

                        simulacao = obj_crm_br_case_stage.create(cr, uid, vals, context)

                    if isinstance(simulacao, list):
                        simulacao = simulacao[0]

                    vals['stage_id'] = simulacao

            if not case.user_id:
                vals['user_id'] = uid
                # Verificar se o usuário já tem atividade em aberto
            if case.section_id.code in ('BugSup', 'nivel2', 'HD'):
                vals['user_id'] = uid  # Trocar de usuário ao abrir a atividade
                casos_abertos_ids = self.search(cr, uid,
                                                [('user_id', '=', uid), ('state', '=', 'open'),
                                                 ('section_id', '=', case.section_id.id)], context=context)
                if casos_abertos_ids:
                    raise osv.except_osv(_('Atencao'),
                                         _('Voce ja tem outro problema aberto!'))
                    #
            self.write(cr, uid, [case.id], vals, context=context)
        self._action(cr, uid, cases, 'open', context=context)
        return True

    def emails_get(self, cr, uid, id, context=None):
        context = context and context.copy() or {}
        case = self.browse(cr, uid, id)
        return (
            (case.user_id and case.user_id.pessoa_id and case.user_id.pessoa_id.email) or False,
            (case.email_from_id and case.email_from_id.name) or False,
            (
                case.create_uid and case.create_uid.pessoa_id and case.create_uid.pessoa_id.email) or False,
            case.email_cc.split(","),
            case.priority
        )

    def case_cancel(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache
        self.__log(cr, uid, cases, 'cancel', context.update({'translated_keyword': _('cancel')}))
        vals = {'state': 'cancel', 'active': True}
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                break
        self._crmcas_history(cr, uid, cases, _('Cancel'))
        self.write(cr, uid, ids, vals, context=context)
        self._action(cr, uid, cases, 'cancel', context=context)
        return True

    def case_pending(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache
        self.__log(cr, uid, cases, 'pending', context.update({'translated_keyword': _('draft')}))
        vals = {'state': 'pending', 'active': True}
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                break
        self._crmcas_history(cr, uid, cases, _('Pending'))
        self.write(cr, uid, ids, vals, context=context)
        self._action(cr, uid, cases, 'pending', context=context)
        return True

    def case_parado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache
        self.__log(cr, uid, cases, 'parado', context={'translated_keyword': _('parado')})
        vals = {'state': 'parado', 'active': True}
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                break
        self._crmcas_history(cr, uid, cases, _('Parado'))
        self.write(cr, uid, ids, vals, context=context)
        self._action(cr, uid, cases, 'parado', context=context)
        return True

    def case_reset(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache
        self.__log(cr, uid, cases, 'draft', context={'translated_keyword': _('draft')})
        vals = {'state': 'draft', 'active': True}
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                vals['stage_id'] = False
                break
        self._crmcas_history(cr, uid, cases, _('Draft'))
        self.write(cr, uid, ids, vals, context=context)
        self._action(cr, uid, cases, 'draft', context=context)
        return True

    def case_previsao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache
        self.__log(cr, uid, cases, 'pending', context={'translated_keyword': _('pending')})
        vals = {'state': 'pending', 'active': True}
        obj_crm_br_case_stage = self.pool.get('crm_br.case.stage')
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                previsao = obj_crm_br_case_stage.search(cr, uid, [('name', '=', _('Previsao'))], context=context)

                if not previsao:
                    section = self.pool.get('crm_br.case.section').search(cr, uid,
                                                                          [('code', '=', 'BugSup')], context=context)
                    if isinstance(section, list):
                        section = section[0]
                    vals = {
                        'name': _('Previsao'),
                        'section_id': section,
                    }

                    previsao = obj_crm_br_case_stage.create(cr, uid, vals, context)

                if isinstance(previsao, list):
                    previsao = previsao[0]

                vals['stage_id'] = previsao
                vals['stage_simulacao'] = False
                break
        self._crmcas_history(cr, uid, cases, _('Pending'))
        self.write(cr, uid, ids, vals, context=context)
        self._action(cr, uid, cases, 'pending', context=context)
        return True

    def case_aguarda_informacoes_adicionais(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cases = self.browse(cr, uid, ids)
        cases[0].state  # to fill the browse record cache
        self.__log(cr, uid, cases, 'pending', context={'translated_keyword': _('pending')})
        vals = {'state': 'pending', 'active': True}
        obj_crm_br_case_stage = self.pool.get('crm_br.case.stage')
        for case in cases:
            if case.section_id.code == 'BugSup':
                vals['date'] = datetime.now()
                informacoes_adicionais = obj_crm_br_case_stage.search(cr, uid, [
                    ('name', '=', 'Ag.Inf.Adicionais')], context=context)

                if not informacoes_adicionais:
                    section = self.pool.get('crm_br.case.section').search(cr, uid,
                                                                          [('code', '=', 'BugSup')], context=context)
                    if isinstance(section, list):
                        section = section[0]
                    vals = {
                        'name': 'Ag.Inf.Adicionais',
                        'section_id': section,
                    }

                    informacoes_adicionais = obj_crm_br_case_stage.create(cr, uid, vals, context)

                if isinstance(informacoes_adicionais, list):
                    informacoes_adicionais = informacoes_adicionais[0]

                vals['stage_id'] = informacoes_adicionais
                vals['stage_simulacao'] = False
                break
        self._crmcas_history(cr, uid, cases, _('Pending'))
        self.write(cr, uid, ids, vals, context=context)
        self._action(cr, uid, cases, 'pending', context=context)
        return True

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        if name.isdigit():
            new_args = args
            new_args.append(('id', '=', name))
            ids = self.search(cr, uid, new_args, limit=limit, context=context)
        else:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit,
                              context=context)
        res = self.name_get(cr, uid, ids, context)
        return res

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not ids:
            return []
        result = []
        for x in self.browse(cr, uid, ids):
            result.append((x.id, '%s-%s' % (numero.getNumeroEditado(x.id, 0), x.name)))
        return result

    def stage_next(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        ok = False
        sid = self.pool.get('crm_br.case.stage').search(cr, uid, [], context=context)
        s = {}
        previous = {}
        for stage in self.pool.get('crm_br.case.stage').browse(cr, uid, sid, context=context):
            section = stage.section_id.id or False
            s.setdefault(section, {})
            s[section][previous.get(section, False)] = stage.id
            previous[section] = stage.id

        for case in self.browse(cr, uid, ids, context):
            section = (case.section_id.id or False)
            if section in s:
                st = case.stage_id.id or False
                if st in s[section]:
                    self.write(cr, uid, [case.id], {'stage_id': s[section][st]}, context=context)

        return True

    def onchange_case_id(self, cr, uid, ids, case_id, name, pessoa_id, context=None):
        context = context and context.copy() or {}
        if not case_id:
            return {}
        case = self.browse(cr, uid, case_id, context=context)
        value = {}
        if not name:
            value['name'] = case.name
        if (not pessoa_id) and case.pessoa_id:
            value['pessoa_id'] = case.pessoa_id.id
            if case.crmcas_contato_id:
                value['crmcas_contato_id'] = case.crmcas_contato_id.id
            if case.email_from_id:
                value['email_from_id'] = case.email_from_id
        return {'value': value}

    def abrir_conversao_oportunidade(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.pool.get('make.opportunity').selectopportunity(cr, uid, ids, context)
        return {
            'name': _('Converter para Oportunidade'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'make.opportunity',
            'view_id': False,
            'target': 'new',
            'type': 'ir.actions.act_window',
        }

    def crmcas_delete_attachments(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_ir_attachment = self.pool.get('ir.attachment')
        if obj_ir_attachment:
            ir_attachment_ids = obj_ir_attachment.search(cr, uid,
                                                         [('res_model', '=', 'crm_br.case')], context=context)
            if ir_attachment_ids:
                ir_attachment_dados = obj_ir_attachment.read(cr, uid, ir_attachment_ids, ['res_id'], context=context)
                attachment_case_ids = tuple(map(lambda i: i.get('res_id'), ir_attachment_dados))
                data_hoje = time.strftime('%Y-%m-%d')
                data_hoje = parse(data_hoje, yearfirst=True)
                data = data_hoje - relativedelta(months=1)
                data = data.strftime('%Y-%m-%d %H:%S:%M')
                case_ids = self.search(cr, uid, [('state', '=', 'done'),
                                                 ('id', 'in', attachment_case_ids),
                                                 ('date_closed', '<', data),
                                                 ('crmcas_nao_excluir_anexo', '=', False)
                                                 ], context=context)
                if case_ids:
                    ir_attachment_ids = obj_ir_attachment.search(cr, uid,
                                                                 [('res_model', '=', 'crm_br.case'),
                                                                  ('res_id', 'in', case_ids)], context=context)
                    if ir_attachment_ids:
                        for attach in ir_attachment_ids:
                            obj_ir_attachment.unlink(cr, uid, [attach], context=context)
        return True
