# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv

icon_list = {
    'form':'STOCK_NEW',
    'tree':'STOCK_JUSTIFY_FILL',
    'calendar':'STOCK_SELECT_COLOR'
}

class crm_br_case_section(osv.Model):
    _name = "crm_br.case.section"
    _inherit = "crm_br.case.section"

    def menu_create_data(self, cr, uid, data, menu_list, context):
        menus = {}
        menus[0] = data['menu_parent_id']
#        section = self.browse(cr, uid, data['section_id'], context)

        obj_model_data = self.pool.get('ir.model.data')
        tree_reference = obj_model_data.get_object_reference(cr, uid, 'crm_br', 'crm_br_case_tree_view')
        tree_view_id = tree_reference and tree_reference[1] or False
        data['view_tree'] = tree_view_id

        form_reference = obj_model_data.get_object_reference(cr, uid, 'crm_br', 'crm_br_case-view')
        form_view_id = form_reference and form_reference[1] or False
        data['view_form'] = form_view_id

        obj_ir_values = self.pool.get('ir.values')
        obj_ir_ui_menu = self.pool.get('ir.ui.menu')
        obj_ir_actions = self.pool.get('ir.actions.act_window')
        obj_ir_actions_view = self.pool.get('ir.actions.act_window.view')
        for (index, mname, mdomain, latest, view_mode) in menu_list:
            view_mode = data['menu' + str(index) + '_option']
            if view_mode == 'no':
                menus[index] = data['menu_parent_id']
                continue
            icon = icon_list.get(view_mode.split(',')[0], 'STOCK_JUSTIFY_FILL')
            menu_id = obj_ir_ui_menu.create(cr, uid, {
                'name': data['menu' + str(index)],
                'parent_id': menus[latest],
                'icon': icon
            }, context)
            menus[index] = menu_id
            action_id = obj_ir_actions.create(cr, uid, {
                'name': data['menu' + str(index)],
                'res_model': 'crm_br.case',
                'domain': mdomain.replace('SECTION_ID', str(data['section_id'])),
                'view_type': 'form',
                'view_mode': view_mode,
                'context': {'section_id': data['section_code']}
            }, context)
            seq = 0
            for mode in view_mode.split(','):
                obj_ir_actions_view.create(cr, uid, {
                    'sequence': seq,
                    'view_id': data['view_' + mode],
                    'view_mode': mode,
                    'act_window_id': action_id,
                    'multi': True
                }, context)
                seq += 1
            obj_ir_values.create(cr, uid, {
                'name': data['menu' + str(index)],
                'key2': 'tree_but_open',
                'model': 'ir.ui.menu',
                'res_id': menu_id,
                'value': 'ir.actions.act_window,%d' % action_id,
                'object': True
            }, context)
        return True

    #
    # Used when called from .XML file
    #
    def menu_create(self, cr, uid, ids, name, menu_parent_id=False, context=None):
        context = context and context.copy() or {}
        menus = {}
        menus[-1] = menu_parent_id
        for section in self.browse(cr, uid, ids, context):
            for (index, mname, mdomain, latest) in [
                (0, '', "[('section_id','='," + str(section.id) + ")]", -1),
                (1, 'My ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid)]", 0),
                (2, 'My Unclosed ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('state','<>','cancel'), ('state','<>','done')]", 1),
                (5, 'My Open ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('state','=','open')]", 2),
                (6, 'My Pending ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('state','=','pending')]", 2),
                (7, 'My Draft ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('state','=','draft')]", 2),

                (3, 'My Late ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('date_deadline','<=',time.strftime('%Y-%m-%d')), ('state','<>','cancel'), ('state','<>','done')]", 1),
                (4, 'My Canceled ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('state','=','cancel')]", 1),
                (8, 'All ', "[('section_id','='," + str(section.id) + "),]", 0),
                (9, 'Unassigned ', "[('section_id','='," + str(section.id) + "),('user_id','=',False)]", 8),
                (10, 'Late ', "[('section_id','='," + str(section.id) + "),('user_id','=',uid), ('date_deadline','<=',time.strftime('%Y-%m-%d')), ('state','<>','cancel'), ('state','<>','done')]", 8),
                (11, 'Canceled ', "[('section_id','='," + str(section.id) + "),('state','=','cancel')]", 8),
                (12, 'Unclosed ', "[('section_id','='," + str(section.id) + "),('state','<>','cancel'), ('state','<>','done')]", 8),
                (13, 'Open ', "[('section_id','='," + str(section.id) + "),('state','=','open')]", 12),
                (14, 'Pending ', "[('section_id','='," + str(section.id) + "),('state','=','pending')]", 12),
                (15, 'Draft ', "[('section_id','='," + str(section.id) + "),('state','=','draft')]", 12),
                (16, 'Unassigned ', "[('section_id','='," + str(section.id) + "),('user_id','=',False),('state','<>','cancel'),('state','<>','done')]", 12),
            ]:
                view_mode = 'tree,form'
                icon = 'STOCK_JUSTIFY_FILL'
                if index == 0:
                    view_mode = 'form,tree'
                    icon = 'STOCK_NEW'
                menu_id = self.pool.get('ir.ui.menu').create(cr, uid, {
                    'name': mname + name,
                    'parent_id': menus[latest],
                    'icon': icon
                }, context)
                menus[index] = menu_id
                action_id = self.pool.get('ir.actions.act_window').create(cr, uid, {
                    'name': mname + name + ' Cases',
                    'res_model': 'crm_br.case',
                    'domain': mdomain,
                    'view_type': 'form',
                    'view_mode': view_mode,
                }, context)
                self.pool.get('ir.values').create(cr, uid, {
                    'name': 'Open Cases',
                    'key2': 'tree_but_open',
                    'model': 'ir.ui.menu',
                    'res_id': menu_id,
                    'value': 'ir.actions.act_window,%d' % action_id,
                    'object': True
                }, context)
        return True

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['name', 'parent_id', 'code'], context=context)
        res = []
        for record in reads:
            if context.get('painel_helpdesk_atendimento_andamento'):
                codigo = record.get('code')
                if codigo == 'HD':
                    name = '1°'
                elif codigo == 'nivel2':
                    name = '2°'
            else:
                name = record['name']
                if record['parent_id']:
                    name = record['parent_id'][1] + ' / ' + name
            res.append((record['id'], name))
        return res
