# -*- encoding: utf-8 -*-
##############################################################################
#
# OpenERP, Open Source Management Solution
# Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
# $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp.osv import fields, osv


class crm_br_case_atendimentos_capa(osv.Model):
    from funcoes import _media_get, _crmcatc_get_tempo_ocioso

    _name = "crm_br.case.atendimentos.capa"
    _description = "Atendimentos"
    _rec_name = 'crmcatc_data'

    _columns = {
        'id': fields.integer('ID', readonly=True),
        'crmcatc_user_id': fields.many2one('res.users', 'Usuario', select=True, oldname='user_id',
                                           ondelete='cascade'),
        'crmcatc_primeiro': fields.char('Primeiro', size=8, oldname='primeiro'),
        'crmcatc_ultimo': fields.char('Ultimo', size=8, oldname='ultimo'),
        'crmcatc_data': fields.date('Data', select=True, oldname='data'),
        'crmcatc_qtd_trabalhado': fields.integer('Tickets Trabalhados', oldname='qtd_trabalhado'),
        'crmcatc_tempo_utilizado': fields.float('Tempo Utilizado', oldname='tempo_utilizado'),
        'crmcatc_tempo_ocioso': fields.function(_crmcatc_get_tempo_ocioso, method=True,
                                                string='Tempo ocioso',
                                                type='float', store=False),
        'crmcatc_media': fields.function(_media_get, method=True, string='Tempo Medio',
                                         type='float', store=False,
                                         help="Calculado como: Tempo utilizado / Tickets Trabalhados"),
        'crmcatc_qtd_avancado': fields.integer('Tickets avancados', oldname='qtd_avancado'),
        'crmcatc_qtd_retornado': fields.integer('Tickets retornados', oldname='qtd_retornado'),
        'crmcatc_ticket_ids': fields.one2many('crm_br.case.atendimentos.tickets', 'crmcatt_capa_id',
                                              'Atendimentos'),
        'crmcatc_tempo_ids': fields.one2many('crm_br.case.atendimentos.tempo', 'crmcate_capa_id',
                                             'Horarios'),
    }
    _order = 'crmcatc_data desc'


class crm_br_case_atendimentos_tickets(osv.Model):
    """
    Classe contendo os tickets(casos) que foram utilizados para compor a capa
    """
    _name = "crm_br.case.atendimentos.tickets"
    _description = 'Tabela de Tickets dos atendimentos'

    from funcoes import _crmcatt_get_dados

    _columns = {
        'crmcatt_capa_id': fields.many2one('crm_br.case.atendimentos.capa', 'Capa', required=True,
                                           ondelete='cascade', select=True),
        'crmcatt_case_id': fields.many2one('crm_br.case', 'Ticket', required=True, ondelete='cascade', select=True),
        'crmcatt_log_ids': fields.one2many('crm_br.case.atendimentos.logs', 'crmcatl_ticket_id',
                                           'Logs'),
        'crmcatt_state': fields.function(_crmcatt_get_dados, method=True, string="Estado",
                                         multi="dados", type="char", size=14, store=False),
        'crmcatt_stage_id': fields.function(_crmcatt_get_dados, method=True, string="Estagio",
                                            multi="dados", type="many2one",
                                            relation="crm_br.case.stage", store=False),
        'crmcatt_pessoa_id': fields.function(_crmcatt_get_dados, method=True, string="Parceiro",
                                             multi="dados", type="many2one",
                                             relation="adm.pessoa", store=False),
        'crmcatt_versao_liberar_id': fields.function(_crmcatt_get_dados, method=True,
                                                     string="Liberar na versao", multi="dados",
                                                     type="many2one",
                                                     relation="crm_br.case.category2",
                                                     store=False),
        'crmcatt_tempo_trabalhado': fields.function(_crmcatt_get_dados, method=True,
                                                    string="Tempo trabalhado", multi="dados",
                                                    type="float", store=False),
    }


class crm_br_case_atendimentos_logs(osv.Model):
    """
    Classe contendo os logs dos tickets que foram utiliazados para compor a capa
    """
    _name = "crm_br.case.atendimentos.logs"
    _description = 'Tabela de logs dos atendimentos'

    from funcoes import _crmcatl_get_dados

    _columns = {
        "crmcatl_ticket_id": fields.many2one('crm_br.case.atendimentos.tickets', 'Capa',
                                             required=True, ondelete='cascade', select=True),
        "crmcatl_log_id": fields.many2one('crm_br.case.log', 'Log', required=True, ondelete='cascade', select=True),
        'crmcatl_stage_id': fields.function(_crmcatl_get_dados, method=True, string="Estagio",
                                            multi="dados", type="many2one",
                                            relation="crm_br.case.stage", store=False),
        'crmcatl_id_do_log': fields.function(_crmcatl_get_dados, method=True, string="Estagio",
                                             multi="dados", type="integer", store=False),
        'crmcatl_date': fields.function(_crmcatl_get_dados, method=True, string="Data",
                                        multi="dados", type="datetime", store=False),
        'crmcatl_tempo': fields.function(_crmcatl_get_dados, method=True, string="Tempo",
                                         multi="dados", type="float", store=False),
    }
    _order = 'crmcatl_ticket_id,crmcatl_log_id'


class crm_br_case_atendimentos_tempo(osv.Model):
    """
    Classe contendo os tempos de atendimentos dos atendentes
    """
    _name = "crm_br.case.atendimentos.tempo"
    _description = 'Tabela de tempos de atendimentos'

    from funcoes import _crmcate_calcula_tempo, _crmcate_calcula_tempo_ocioso

    _columns = {
        'crmcate_capa_id': fields.many2one('crm_br.case.atendimentos.capa', 'Capa', required=True,
                                           ondelete='cascade', select=True),
        'crmcate_case_id': fields.many2one('crm_br.case', 'Ticket', required=True, ondelete='cascade', select=True),
        'crmcate_inicio': fields.datetime('Inicio'),
        'crmcate_termino': fields.datetime('Fim'),
        'crmcate_tempo_util': fields.function(_crmcate_calcula_tempo, float,
                                              string='Tempo util', store=True),
        'crmcate_tempo_ocioso': fields.function(_crmcate_calcula_tempo_ocioso, float,
                                                string='Tempo ocioso', store=True),
    }
    _order = 'crmcate_capa_id,crmcate_inicio'
