# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


import time
from openerp.osv import fields, osv


AVAILABLE_STATES = [
    ('draft', 'Draft'),
    ('open', 'Open'),
    ('cancel', 'Cancel'),
    ('done', 'Close'),
    ('pending', 'Pending'),
    ('parado', 'Parado')
]

AVAILABLE_PRIORITIES = [
    ('5', 'Lowest'),
    ('4', 'Low'),
    ('3', 'Normal'),
    ('2', 'High'),
    ('1', 'Highest')
]


class crm_br_case(osv.Model):
    from funcoes import _email_last, _calc_tmp_atend, _get_stage_simulacao, _get_section
    from funcoes import _links_get, _get_default_user, _get_chamadas, _crmcas_history
    from funcoes import crmcas_get_section_code, crmcas_get_qtd_dia_criacao
    from funcoes import _crmcas_get_termino_previsto, _crmcas_calcula_tempo_atendimento
    from funcoes import crmcas_get_novo_helpdesk, crmcas_get_tempo_atendimento_grouped

    _name = "crm_br.case"
    _description = "Case"

    _columns = {
        'id': fields.integer('ID', readonly=True),
        'name': fields.char('Description', size=64, required=True),
        'id_atendimento': fields.integer('No.Atendimento', readonly=True),
        # numero do atendimento helpdesk
        'sequence': fields.integer('Ordem'),  # Ordem de atendimento
        'priority': fields.selection(AVAILABLE_PRIORITIES, 'Priority'),
        'active': fields.boolean('Active'),
        'description': fields.text('Your action'),
        'section_id': fields.many2one('crm_br.case.section', 'Section', required=True, select=True),
        'categ_id': fields.many2one('crm_br.case.categ', 'Category'),
        'planned_revenue': fields.float('Planned Revenue'),
        'planned_cost': fields.float('Planned Costs'),
        'probability': fields.float('Probability (%)'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Pessoa', select=1),
        'crmcas_contato_id': fields.many2one('adm.pessoa', 'Contato',
                                        context="{'contatos_de':pessoa_id}", ondelete="restrict"),
        'email_from_id': fields.many2one('adm.pessoa.email', 'Email',
                                         domain="[('pessoa_id','=',pessoa_id)]"),
        'email_cc': fields.char('Watchers Emails', size=252),
        'email_last': fields.function(_email_last, method=True,
                                      string='Latest E-Mail', type='text'),
        'som': fields.many2one('crm_br.som', 'State of Mind'),
        'date': fields.datetime('Date', select=True),
        'create_date': fields.datetime('Created', readonly=True,select=True),
        'write_date': fields.datetime('Ultima atualizacao', readonly=True),
        'date_deadline': fields.datetime('Deadline'),
        'crmcas_termino_previsto': fields.function(_crmcas_get_termino_previsto, type='datetime',
                                               method=True, string='Termino previsto', store=False),
        'date_closed': fields.datetime('Closed', readonly=True),
        'canal_id': fields.many2one('crm_br.canal', 'Channel'),
        'user_id': fields.many2one('res.users', 'Responsible'),
        'history_line': fields.one2many('crm_br.case.history', 'crmcal_case_id', 'Communication',
                                        readonly=1),
        'log_ids': fields.one2many('crm_br.case.log', 'crmcal_case_id', 'Logs History', readonly=1),
        'state': fields.selection(AVAILABLE_STATES, 'Status', size=16, readonly=True),
        'ref': fields.reference('Reference', selection=_links_get, size=128),
        'ref2': fields.reference('Reference 2', selection=_links_get, size=128),

        'date_action_last': fields.datetime('Last Action', readonly=1),
        'date_action_next': fields.datetime('Next Action', readonly=1),
        'total_tempo_atend': fields.function(_calc_tmp_atend, type='float', method=True,
                                             string='Tempo total gasto', store=True,
                                             digits=(3, 2)),
        'crmcas_tempo_atendimento': fields.function(_crmcas_calcula_tempo_atendimento, type='float', method=True,
                                             string='Tempo de Atendimento', store=False, multi="dados",
                                             digits=(3, 2)),
        'crmcas_tempo_simulacao': fields.function(_crmcas_calcula_tempo_atendimento, type='float', method=True,
                                             string='Tempo de Simulacao', store=False, multi="dados",
                                             digits=(3, 2)),
        'crmcas_tempo_homologacao': fields.function(_crmcas_calcula_tempo_atendimento, type='float', method=True,
                                             string='Tempo de Homologacao', store=False, multi="dados",
                                             digits=(3, 2)),
        'date_ultimo_historico': fields.datetime('Data ultimo historico'),
        'create_uid': fields.many2one('res.users', 'Dono da solicitacao', select=1, readonly=True,
                                      ondelete='restrict'),
        'stage_id': fields.many2one('crm_br.case.stage', 'Stage',select=True),
        'category2_id': fields.many2one('crm_br.case.category2', 'Category Name'),
        'versao_liberar_id': fields.many2one('crm_br.case.category2', 'Liberar na versao'),
        'crmcas_duration': fields.float('Duration', oldname='duration', digits=(3, 2)),
        'note': fields.text('Note'),
        'case_id': fields.many2one('crm_br.case', 'Related Case'),
        'pessoa_name': fields.char('Employee Name', size=64),
        'pessoa_name2': fields.char('Contato', size=64),
        'pessoa_phone_ddd': fields.char('DDD', size=4),
        'pessoa_phone': fields.char('Telefone', size=32),
        'pessoa_mobile': fields.char('Celular', size=32),
        'email_from': fields.char('Email', size=64),
        'cidade_id': fields.many2one('adm.cidade', 'Cidade', select=True),
        'atividade_pai_id': fields.many2one('crm_br.case', 'Atividade pai'),
        'atividades_filhas_ids': fields.one2many('crm_br.case', 'atividade_pai_id',
                                                 'Atividades filhas'),
        'crmcas_produto_id': fields.many2one('crm_br.produto', 'Produto'),
        'stage_simulacao': fields.function(_get_stage_simulacao, type='boolean', method=True,
                                           string='Simulacao', store=False),
        'crmcas_faturar': fields.boolean('Faturar',
                                         help="Se marcado, ao concluir a atividade, ela estara disponivel para faturar"),
        'crmcas_nao_excluir_anexo': fields.boolean('Nao excluir anexo',
                                                   help="Se selecionado, o anexo nao sera excluido"),
        'crmcas_tipo': fields.selection([('Manutencao', 'Manutencao'), ('Implementacao', 'Implementacao'),('Outros', 'Outros')], 'Tipo', size=13),
        'crmcas_complexidade': fields.selection([('B', 'Baixa'), ('M', 'Medio'), ('A', 'Alta')], 'Complexidade'),
        'crmcas_criticidade':fields.selection([('B', 'Baixa'), ('M', 'Medio'), ('A', 'Alta')], 'Criticidade'),
        'crmcase_tempo_atendimento_usuario_ids': fields.one2many('crm_br.case.log', 'crmcal_case_id',
                                                                 'Tempo Atendimento Responsavel'),
        'crmcas_tempo_atendimento_usuario_grouped_ids': fields.function(
            crmcas_get_tempo_atendimento_grouped, method=True,
            string='Tempo atendimento responsavel agrupado', type='one2many',
            relation='wizard.crm_br.log.grouped'),
        'crmcas_section_code': fields.function(crmcas_get_section_code, method=True,
                                               string='Codigo da secao', type='char', store=False),
        'crmcas_qtd_dia_criacao': fields.function(crmcas_get_qtd_dia_criacao, method=True,
                                                  string='Dias da criacao', type='integer',
                                                  store=False),
        'crmcas_novo_helpdesk': fields.function(crmcas_get_novo_helpdesk, method=True,
                                                string='Novo helpdesk', type='boolean',
                                                store=False),
    }

    _defaults = {
        'section_id': _get_section,
        'sequence': lambda *a: 100,
        'active': lambda *a: 1,
        'user_id': _get_default_user,
        'canal_id': _get_chamadas,
        'state': lambda *a: 'draft',
        'priority': lambda *a: AVAILABLE_PRIORITIES[2][0],
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'crmcas_nao_excluir_anexo': lambda *a: False,
    }
    _order = 'priority, sequence, date_deadline desc, date desc,id desc'
    _history = _crmcas_history

