# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp.osv import fields, osv


AVAILABLE_STATES = [
    ('draft', 'Draft'),
    ('open', 'Open'),
    ('cancel', 'Cancel'),
    ('done', 'Close'),
    ('pending', 'Pending'),
    ('parado', 'Parado')
]

AVAILABLE_PRIORITIES = [
    ('5', 'Lowest'),
    ('4', 'Low'),
    ('3', 'Normal'),
    ('2', 'High'),
    ('1', 'Highest')
]

class crm_br_case_rule(osv.Model):

    from funcoes import _check_mail

    _name = "crm_br.case.rule"
    _description = "Case Rule"
    _columns = {
        'name': fields.char('Rule Name', size=64, required=True),
        'active': fields.boolean('Active'),
        'sequence': fields.integer('Sequence'),

        'trg_state_from': fields.selection([('', ''), ('escalate', 'Escalate')] + AVAILABLE_STATES, 'Case State', size=16),
        'trg_state_to': fields.selection([('', ''), ('escalate', 'Escalate')] + AVAILABLE_STATES, 'Button Pressed', size=16),

        'trg_date_type':  fields.selection([
            ('none', 'None'),
            ('create', 'Creation Date'),
            ('action_last', 'Last Action Date'),
            ('deadline', 'Deadline'),
            ('date', 'Date'),
            ], 'Trigger Date', size=16),
        'trg_date_range': fields.integer('Delay after trigger date'),
        'trg_date_range_type': fields.selection([('minutes', 'Minutes'), ('hour', 'Hours'), ('day', 'Days'), ('month', 'Months')], 'Delay type'),

        'trg_section_id': fields.many2one('crm_br.case.section', 'Section'),
        'trg_stage_id': fields.many2one('crm_br.case.stage', 'Stage', select=True, domain="[('section_id','=',act_section_id)]"),
        'trg_categ_id':  fields.many2one('crm_br.case.categ', 'Category', domain="[('trg_section_id','=',section_id)]"),
        'trg_user_id':  fields.many2one('res.users', 'Responsible'),

        'trg_pessoa_id': fields.many2one('adm.pessoa', 'Pessoa'),
        'trg_pessoa_categ_id': fields.many2one('adm.pessoa.categoria.nome', 'Pessoa Category'),

        'trg_priority_from': fields.selection([('', '')] + AVAILABLE_PRIORITIES, 'Minimum Priority'),
        'trg_priority_to': fields.selection([('', '')] + AVAILABLE_PRIORITIES, 'Maximim Priority'),
        'trg_max_history': fields.integer('Maximum Communication History'),

        'act_method': fields.char('Call Object Method', size=64),
        'act_state': fields.selection([('', '')] + AVAILABLE_STATES, 'Set state to', size=16),
        'act_section_id': fields.many2one('crm_br.case.section', 'Set section to'),
        'act_user_id': fields.many2one('res.users', 'Set responsible to'),
        'act_priority': fields.selection([('', '')] + AVAILABLE_PRIORITIES, 'Set priority to'),
        'act_email_cc': fields.char('Add watchers (Cc)', size=250, help="These people will receive a copy of the futur communication between pessoa and users by email"),

        'act_remind_pessoa': fields.boolean('Remind pessoa', help="Check this if you want the rule to send a reminder by email to the pessoa."),
        'act_remind_user': fields.boolean('Remind responsible', help="Check this if you want the rule to send a reminder by email to the user."),
        'act_remind_attach': fields.boolean('Remind with attachment', help="Check this if you want that all documents attached to the case be attached to the reminder email sent."),

        'act_mail_to_user': fields.boolean('Mail to responsible'),
        'act_mail_to_pessoa': fields.boolean('Mail to pessoa'),
        'act_mail_to_watchers': fields.boolean('Mail to watchers (Cc)'),
        'act_mail_to_email': fields.char('Mail to these emails', size=128),
        'act_mail_body': fields.text('Mail body')
    }
    _defaults = {
        'active': lambda * a: 1,
        'trg_date_type': lambda * a: 'none',
        'trg_date_range_type': lambda * a: 'day',
        'act_mail_to_user': lambda * a: 0,
        'act_remind_pessoa': lambda * a: 0,
        'act_remind_user': lambda * a: 0,
        'act_mail_to_pessoa': lambda * a: 0,
        'act_mail_to_watchers': lambda * a: 0,
    }
    _order = 'sequence'

    _constraints = [
        (_check_mail, 'Error: The mail is not well formated', ['act_mail_body']),
    ]

