# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields,osv,orm


class crm_br_segmentation(osv.Model):
    '''
        A segmentation is a tool to automatically assign categories on pessoas.
        These assignations are based on criterions.
    '''
    _name = "crm_br.segmentation"
    _description = "Segmentacao de pessoas"
    _columns = {
        'name': fields.char('Name', size=64, required=True, help='The name of the segmentation.'),
        'description': fields.text('Description'),
        'categ_id': fields.many2one('adm.pessoa.categoria.nome', 'Pessoa Category', required=True, help='The pessoa category that will be added to pessoas that match the segmentation criterions after computation.'),
        'exclusif': fields.boolean('Exclusive', help='Check if the category is limited to pessoas that match the segmentation criterions. If checked, remove the category from pessoas that doesn\'t match segmentation criterions'),
        'state': fields.selection([('not running','Not Running'),('running','Running')], 'Execution Status', readonly=True),
        'pessoa_id': fields.integer('Max pessoa ID processed'),
        'segmentation_line': fields.one2many('crm_br.segmentation.line', 'segmentation_id', 'Criteria', required=True),
        'som_interval': fields.integer('Days per Periode', help="A period is the average number of days between two cycle of sale or purchase for this segmentation. It's mainly used to detect if a pessoa has not purchased or buy for a too long time, so we suppose that his state of mind has decreased because he probably bought goods to another supplier. Use this functionality for recurring businesses."),
        'som_interval_max': fields.integer('Max Interval', help="The computation is made on all events that occured during this interval, the past X periods."),
        'som_interval_decrease': fields.float('Decrease (0>1)', help="If the pessoa has not purchased (or bought) during a period, decrease the state of mind by this factor. It\'s a multiplication"),
        'som_interval_default': fields.float('Default (0=None)', help="Default state of mind for period preceeding the 'Max Interval' computation. This is the starting state of mind by default if the pessoa has no event."),
        'sales_purchase_active': fields.boolean('Use The Sales Purchase Rules', help='Check if you want to use this tab as part of the segmentation rule. If not checked, the criteria beneath will be ignored')
    }
    _defaults = {
        'pessoa_id': lambda *a: 0,
        'state': lambda *a: 'not running',
        'som_interval_max': lambda *a: 3,
        'som_interval_decrease': lambda *a: 0.8,
        'som_interval_default': lambda *a: 0.5
    }

