# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import time
from datetime import datetime, timedelta
from openerp.osv import orm
from openerp.tools.translate import _
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta


# Funcoes da classe crm_br_case_atendimentos_tickets
def _crmcatt_get_dados(self, cr, uid, ids, name, arg, context=None):
    res = {}
    context = context and context.copy() or {}
    if ids:
        browse_ticket = self.browse(cr, uid, ids)
        for x in browse_ticket:
            ticket = x.crmcatt_case_id
            _sql = 'select sum(crmcate_tempo_util) from crm_br_case_atendimentos_tempo where crmcate_capa_id = %s and crmcate_case_id = %s' % (
            x.crmcatt_capa_id.id, x.crmcatt_case_id.id)
            cr.execute(_sql)
            _tempo = cr.fetchone()[0] or 0.0
            vals = {
                'crmcatt_state': ticket.state,
                'crmcatt_stage_id': ticket.stage_id.id,
                'crmcatt_pessoa_id': ticket.pessoa_id.id,
                'crmcatt_versao_liberar_id': ticket.versao_liberar_id.id,
                'crmcatt_tempo_trabalhado': _tempo,
            }
            res[x.id] = vals
    return res


# Funcoes da classe crm_br_case_atendimentos_logs
def _crmcatl_get_dados(self, cr, uid, ids, name, arg, context=None):
    res = {}
    context = context and context.copy() or {}
    if ids:
        browse_log = self.browse(cr, uid, ids)
        for x in browse_log:
            log = x.crmcatl_log_id
            vals = {
                'crmcatl_stage_id': log.crmcal_stage_id.id,
                'crmcatl_date': log.crmcal_date,
                'crmcatl_tempo': log.crmcal_tempo_atend,
                'crmcatl_id_do_log': log.id,
            }
            res[x.id] = vals
    return res


# Funcoes da classe crm_br_case_atendimentos_capa
def _media_get(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    vals = self.browse(cr, uid, ids)
    for x in vals:
        res[x.id] = x.crmcatc_tempo_utilizado / x.crmcatc_qtd_trabalhado
    return res


def _crmcatc_get_tempo_ocioso(self, cr, uid, ids, name, arg, context=None):
    """
    Esta função calcula o tempo ocioso do atendente. Tempo ocioso, é o tempo não apontado entre os
    atendimentos
    :param cr:
    :param uid:
    :param ids:
    :param name:
    :param arg:
    :param context:
    :return: float - Tempo ocioso
    """
    context = context and context.copy() or {}
    res = {}
    for _capa in self.browse(cr, uid, ids):
        _tempo = 0
        for x in _capa.crmcatc_tempo_ids:
            _tempo += x.crmcate_tempo_ocioso
        res[_capa.id] = _tempo
    return res


# Funcoes da classe crm_br_case_atendimentos_tempo
def _crmcate_calcula_tempo(self, cr, uid, ids, name, arg, context=None):
    res = {}
    context = context and context.copy() or {}
    if ids:
        for x in self.browse(cr, uid, ids):
            if x.crmcate_termino and x.crmcate_inicio:
                rs = (datetime.fromtimestamp(
                    time.mktime(time.strptime(x.crmcate_termino, "%Y-%m-%d %H:%M:%S")))
                      - datetime.fromtimestamp(
                    time.mktime(time.strptime(x.crmcate_inicio, "%Y-%m-%d %H:%M:%S"))))
                res[x.id] = rs.days * 24 + (float(rs.seconds) / 3600)
    return res


def _crmcate_calcula_tempo_ocioso(self, cr, uid, ids, name, arg, context=None):
    res = {}
    context = context and context.copy() or {}
    if ids:
        browse_log = self.browse(cr, uid, ids)
        for x in browse_log:
            sql = "select crmcate_inicio, crmcate_termino from crm_br_case_atendimentos_tempo where " \
                  "crmcate_capa_id = %s and crmcate_inicio >= '%s' order by crmcate_inicio limit 2" \
                  % (x.crmcate_capa_id.id, x.crmcate_inicio)
            cr.execute(sql)
            _intervalo_ini = _intervalo_fim = 0
            for a in cr.fetchall():
                if _intervalo_ini == 0:
                    _intervalo_ini = a[1]
                else:
                    _intervalo_fim = a[0]
            if _intervalo_fim == 0:
                _ocioso = 0.0
            else:
                if _intervalo_fim and _intervalo_ini:
                    rs = (datetime.fromtimestamp(
                        time.mktime(time.strptime(_intervalo_fim, "%Y-%m-%d %H:%M:%S")))
                          - datetime.fromtimestamp(
                        time.mktime(time.strptime(_intervalo_ini, "%Y-%m-%d %H:%M:%S"))))
                    _ocioso = rs.days * 24 + (float(rs.seconds) / 3600)
                else:
                    _ocioso = 0.0
            res[x.id] = _ocioso
    return res


# Funcoes da classe crm_br_case_categ
def _find_res_model(self, cr, uid, context=None):
    context = context and context.copy() or {}
    model = context and context.get('object_id', False) or False
    return model


def _find_object_id(self, cr, uid, context=None):
    """Finds id for case object
    @param self: The object pointer
    @param cr: the current row, from the database cursor,
    @param uid: the current user’s ID for security checks,
    @param context: A standard dictionary for contextual values
    """
    context = context and context.copy() or {}
    object_id = context and context.get('object_id', False) or False
    ids = self.pool.get('ir.model').search(cr, uid, [('model', '=', object_id)], context=context)
    return ids and ids[0]


# Funcoes da classe crm_br_case


def crmcas_get_tempo_atendimento_grouped(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    if not ids:
        return res
    for _id in ids:
        cr.execute("""DELETE FROM wizard_crm_br_log_grouped WHERE case_id = %s""" % _id)
        sql = """
            SELECT
                crmcal_case_id          AS case_id,
                crmcal_user_id          AS user_id,
                crmcal_stage_id         AS stage_id,
                Sum(crmcal_tempo_atend) AS tempo_atendimento
            FROM
                crm_br_case_log
            WHERE
                crmcal_case_id = %s
            GROUP BY crmcal_case_id, crmcal_user_id, crmcal_stage_id
        """ % _id
        cr.execute(sql)
        dados = cr.dictfetchall()
        log_grouped_ids = list()
        obj_wizard_crm_br_log_grouped = self.pool.get('wizard.crm_br.log.grouped')
        for dado in dados:
            log_grouped_ids.append(obj_wizard_crm_br_log_grouped.create(cr, uid, dado, context))
        res[_id] = log_grouped_ids
    return res


def _crmcas_get_termino_previsto(self, cr, uid, ids, field_name, args, context=None):
    """
    Método para calcular a data e hora prevista de termino do atendimento

    date_deadline é a hora de inicio prevista
    """
    context = context and context.copy() or {}
    res = {}
    for browse_case in self.browse(cr, uid, ids, context=context):
        _fim = False
        if browse_case.date_deadline:
            _duracao = browse_case.crmcas_duration or 0.0
            _gasto = browse_case.crmcas_tempo_atendimento or 0.0
            if _gasto < _duracao:
                _duracao = _duracao - _gasto
            else:
                _duracao = 0
            _fim = browse_case.date_deadline
            _fim = self._crmcas_calcula_fim_da_atividade(cr, uid, _fim, _duracao)
        res[browse_case.id] = _fim
    return res


def _crmcas_calcula_tempo_atendimento(self, cr, uid, ids, field_name, args, context=None):
    """
    Método para calcular o tempo gasto no Estagio de atendimento, simulacao e homologacao
    """
    context = context and context.copy() or {}
    res = {}
    stage_atendimento_id = self.pool.get('crm_br.case.stage').search(cr, uid, [('name', '=', u'Atendimento')],
                                                                     context=context)
    if stage_atendimento_id:
        stage_atendimento_id = stage_atendimento_id[0]
    stage_homologacao_id = self.pool.get('crm_br.case.stage').search(cr, uid, [('name', '=', u'Homologação')],
                                                                     context=context)
    if stage_homologacao_id:
        stage_homologacao_id = stage_homologacao_id[0]
    stage_simulacao_id = self.pool.get('crm_br.case.stage').search(cr, uid, [('name', '=', u'Simulação')],
                                                                   context=context)
    if stage_simulacao_id:
        stage_simulacao_id = stage_simulacao_id[0]
    lista_stage = (stage_atendimento_id, stage_homologacao_id, stage_simulacao_id)
    for browse_case in self.browse(cr, uid, ids, context=context):
        vals = {'crmcas_tempo_atendimento': 0.0,
                'crmcas_tempo_simulacao': 0.0,
                'crmcas_tempo_homologacao': 0.0}
        if browse_case.section_id.code == 'BugSup':
            for stage_id in lista_stage:
                if stage_id:
                    _sql = ('select sum(crmcal_tempo_atend) from crm_br_case_log '
                            'where crmcal_case_id = %s and crmcal_stage_id = %s') % \
                           (browse_case.id, stage_id)
                    cr.execute(_sql)
                    _tempo = cr.fetchone()[0] or 0.0
                else:
                    _tempo = 0.0
                if browse_case.state == 'open' and stage_id == browse_case.stage_id.id:
                    _tempo += self._crmbr_calcula_tempo_abertura_atual(cr, uid, browse_case.id)
                if stage_id == stage_atendimento_id:
                    vals['crmcas_tempo_atendimento'] = _tempo
                if stage_id == stage_homologacao_id:
                    vals['crmcas_tempo_homologacao'] = _tempo
                if stage_id == stage_simulacao_id:
                    vals['crmcas_tempo_simulacao'] = _tempo
        res[browse_case.id] = vals
    return res


def crmcas_get_section_code(self, cr, uid, ids, field_name, args, context=None):
    """
    O metodo recupera o codigo da secao para cada resgistro da tabela crm_br_case, representados
    pelo parametro ids
    """
    context = context and context.copy() or {}
    res = {}
    brw_crm_br_case = self.pool.get('crm_br.case').browse(cr, uid, ids, context)
    for x in brw_crm_br_case:
        res[x.id] = x.section_id.code
    return res


def crmcas_get_qtd_dia_criacao(self, cr, uid, ids, field_name, args, context=None):
    """
    O metodo calcula o numero dias desde a data da abertura do atendimento ate a data de hoje para
    cada resgistro da tabela crm_br_case, representados pelo parametro ids
    """
    context = context and context.copy() or {}
    res = {}
    brw_crm_br_case = self.pool.get('crm_br.case').browse(cr, uid, ids, context)
    data_hoje = datetime.now()
    for x in brw_crm_br_case:
        data_criacao = parse(x.date)
        dif = data_hoje - data_criacao
        res[x.id] = dif.days
    return res


def _get_default_user(self, cr, uid, context):
    if context.get('portal', False):
        return False
    return uid


def _get_chamadas(self, cr, uid, context):
    if context.get('entrada_chamada_tel', True):
        id = self.pool.get('crm_br.canal').search(cr, uid, [('name', '=', 'Telefone')], context=context)
        if id:
            if len(id) > 1:
                return id[1]
            else:
                return id[0]
        else:
            return False
    else:
        return False


def _email_last(self, cursor, user, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for case in self.browse(cursor, user, ids):
        if case.history_line:
            res[case.id] = case.history_line[0].description
        else:
            res[case.id] = False
    return res


def _calc_tmp_atend(self, cr, uid, ids, name, arg, context=None):
    ##ate onde eu vi nos modulos, o widget float_time exibe somente este
    # formato '00:00' entao para o cálculo de tempo total será exibida em horas  e minutos.
    context = context and context.copy() or {}
    res = {}
    cr.execute(
        "SELECT crmcal_case_id, COALESCE(SUM(crmcal_tempo_atend),0) FROM crm_br_case_log WHERE crmcal_case_id IN %s GROUP BY crmcal_case_id",
        (tuple(ids),))
    hours = dict(cr.fetchall())
    for cases in self.browse(cr, uid, ids, context=context):
        res[cases.id] = hours.get(cases.id, 0.0)
    return res


def _get_section(self, cr, uid, context=None):
    context = context and context.copy() or {}
    obj_section = self.pool.get('crm_br.case.section')
    section = context.get('section_id') or context.get('object_id')
    if section in ('HD', 'nivel2', 'crm_br.help_desk'):
        if not isinstance(section, basestring):
            section = str(section)
    elif not section:
        section = 'Phone'
    args = [('code', '=', section)]
    section_ids = obj_section.search(cr, uid, args, context=context)
    if section_ids:
        section_dados = obj_section.read(cr, uid, section_ids[0], ['id'], context=context)
        if isinstance(section_dados, list) and section_dados:
            section_dados = section_dados[0]
        if isinstance(section_dados, dict):
            return section_dados.get('id') or False
    return False


def crmcas_get_novo_helpdesk(self, cr, uid, ids, field_name, arg, context=None):
    """
    O método identifica se a ação está chamando o menu 'Novo Helpdesk' ou 'Novo Atendimento de 2º
    Nível' para que o botão 'Cancelado' não seja exibido.
    :param cr: cursor no banco de dados
    :param uid: identificador do usuário, id da tabela res_users
    :param ids: lista de registros da tabela crm_br_case
    :param field_name: nome do atributo
    :param arg: lista de argumentos para pesquisa
    :param context: dicionário com valores contextuais
    :return: dicionário, cujas chaves representam os ids da tabela crm_br_case e seus valores um
    booleano
    """
    context = context and context.copy() or {}
    res = {}
    novo_help_desk = False
    section = context.get('section_id') or context.get('object_id')
    all_help_desk = context.get('all_help_desk')
    if section in ('HD', 'nivel2', 'crm_br.help_desk') and not all_help_desk:
        novo_help_desk = True
    brw_crm_br_case = self.browse(cr, uid, ids, context)
    for case in brw_crm_br_case:
        res[case.id] = novo_help_desk
    return res


def _get_stage_simulacao(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    dados = self.browse(cr, uid, ids)
    for i in dados:
        if i.stage_id.name == _('Simulacao'):
            res[i.id] = True
        else:
            res[i.id] = False

    return res


def _crmcas_history(self, cr, uid, cases, keyword, history=False, email=False, context=None):
    context = context and context.copy() or {}
    for case in cases:
        data = {
            'name': keyword,
            'crmcal_som': case.som.id,
            'crmcal_canal_id': case.canal_id.id,
            'crmcal_user_id': uid,
            'crmcal_date': time.strftime('%Y-%m-%d %H:%M:%S'),
            'crmcal_case_id': case.id,
            'crmcal_section_id': case.section_id.id,
            'crmcal_stage_id': case.stage_id.id
        }
        obj = self.pool.get('crm_br.case.log')
        if history and case.description:
            obj = self.pool.get('crm_br.case.history')
            data['description'] = case.description
            data['email'] = email or \
                            case.user_id.login or \
                            (case.user_id and case.user_id.pessoa_id and \
                             case.user_id.pessoa_id.email) or False
            data['crmbch_chk_problema'] = True if context.get('crmbch_problema') else False
        obj.create(cr, uid, data, context)
    return True


# Funcoes de varias classes do modulo crm_br
def _links_get(self, cr, uid, context=None):
    context = context and context.copy() or {}
    obj = self.pool.get('res.request.link')
    ids = obj.search(cr, uid, [], context=context)
    res = obj.read(cr, uid, ids, ['object', 'name'], context=context)
    return [(r['object'], r['name']) for r in res]


# Funcoes da classe crm_br_case_log
def _calcula_atendimento(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    vals = self.browse(cr, uid, ids)

    for x in vals:
        caso = x.crmcal_case_id.id
        data_ant = x.crmcal_case_id.create_date
        data_hoje = x.crmcal_date
        estado_atual = x.crmcal_case_id.state
        res[x.id] = 0.0
        ids_2 = self.search(cr, uid,
                            [('crmcal_case_id', '=', caso), ('crmcal_date', '<', data_hoje)], context=context)
        if ids_2:
            vals_ant = self.read(cr, uid, ids_2[0], ['crmcal_date', 'name'], context=context)
            data_ant = vals_ant['crmcal_date']

        # so registrar o tempo se o caso estiver no estado 'open'
        if estado_atual == 'open':
            dat_hj = datetime.fromtimestamp(
                time.mktime(time.strptime(data_hoje, "%Y-%m-%d %H:%M:%S")))
            dt_ant = datetime.fromtimestamp(
                time.mktime(time.strptime(data_ant, "%Y-%m-%d %H:%M:%S")))
            rs = dat_hj - dt_ant
            res[x.id] = rs.days * 24 + (float(rs.seconds) / 3600)
        else:
            res[x.id] = 0.0
    return res


def _crmcal_get_priority(self, cr, uid, ids, name, args, context=None):
    """
    Este método recupera a prioridade do caso
    :param self:
    :param cr:
    :param uid:
    :param ids: ids dos logs
    :param name:
    :param args:
    :param context:
    :return:
    """
    context = context and context.copy() or {}
    if not len(ids):
        return []
    res = {}
    for brw_log in self.browse(cr, uid, ids, context=context):
        _priority = brw_log.crmcal_case_id.priority
        res[brw_log.id] = _priority
    return res




# Funcoes da classe crm_br_case_pipeline
def _pipeline_get(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    vals = self.browse(cr, uid, ids)
    for x in vals:
        res[x.id] = x.campo01 + x.campo02 + x.campo03 + x.campo04 + x.campo05
    return res


# Funcoes da classe crm_br_case_rule
def _check_mail(self, cr, uid, ids, context=None):
    context = context and context.copy() or {}
    caseobj = self.pool.get('crm_br.case')
    emptycase = orm.browse_null()
    for rule in self.browse(cr, uid, ids):
        if rule.act_mail_body:
            try:
                caseobj.format_mail(emptycase, rule.act_mail_body)
            except (ValueError, KeyError, TypeError):
                return False
    return True


# Funcoes da classe crm_br_case_section
def _check_recursion(self, cr, uid, ids):
    level = 100
    while len(ids):
        cr.execute('select distinct parent_id from crm_br_case_section where id in (' + ','.join(
            map(str, ids)) + ')')
        ids = filter(None, map(lambda x: x[0], cr.fetchall()))
        if not level:
            return False
        level -= 1
    return True


# Funcoes da classe report_crm_br_case_section
def _get_data(self, cr, uid, ids, field_name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    state_perc = 0.0
    avg_ans = 0.0

    for case in self.browse(cr, uid, ids, context):
        if field_name != 'avg_answers':
            state = field_name[5:]
            cr.execute("select count(*) from crm_br_case where section_id =%s and state='%s'" % (
                case.section_id.id, state))
            state_cases = cr.fetchone()[0]
            perc_state = (state_cases / float(case.nbr_cases)) * 100

            res[case.id] = perc_state
        else:
            cr.execute('select count(*) from crm_br_case_log l  where l.crmcal_section_id=%s' % (
                case.section_id.id))
            logs = cr.fetchone()[0]

            avg_ans = logs / case.nbr_cases
            res[case.id] = avg_ans

    return res


# Funcoes da classe report_crm_br_case_history
def _crmbch_note_get(self, cursor, user, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    obj_adm_utils = self.pool.get('adm.utils')
    for hist in self.browse(cursor, user, ids, context or {}):
        data = hist.crmcal_date
        data = parse(data, yearfirst=True)
        data = obj_adm_utils.admutl_datetime_to_timezone(cursor, user, data)
        res[hist.id] = (hist.email or '/') + ' (' + str(data) + ')\n'
        res[hist.id] += (hist.description or '')
    return res


def _crmbch_get_state(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    if not isinstance(ids, list):
        ids = [ids]
    for x in ids:
        case_id = self.browse(cr, uid, x).log_id.crmcal_case_id.id
        browse_case = self.pool.get('crm_br.case').browse(cr, uid, case_id, context=context)
        res[x] = browse_case.state
    return res
