# -*- encoding: utf-8 -*-
from openerp.osv import fields, osv
from openerp import tools
from mx.DateTime.DateTime import DateTimeFrom

AVAILABLE_PRIORITIES = [
    ('5', 'Mais baixa'),
    ('4', 'Baixa'),
    ('3', 'Normal'),
    ('2', 'Alta'),
    ('1', 'Maior')
]
AVAILABLE_STATES = [
    ('draft', 'Esboco'),
    ('open', 'Aberto'),
    ('cancel', 'Cancelado'),
    ('done', 'Fechado'),
    ('pending', 'Pendente'),
    ('parado', 'Parado')
]


class report_crm_br_open_category(osv.Model):
    _name = "report.crm_br.open.category"
    _description = "Atendimentos em Aberto por Categoria"
    _auto = False
    _columns = {
        'categ_id': fields.many2one('crm_br.case.categ', 'Categoria', readonly=True),
        'total': fields.integer('Total', readonly=True),
        'parent_id': fields.many2one('crm_br.case.categ', 'Parent Category'),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child Sections'),
        'child_ids': fields.one2many('report.crm_br.open.category', 'parent_id', 'Child Sections'),
    }
    _order = 'total desc'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_open_category, self).read(cr, uid, ids, [], context=context,
                                                                  load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            if 'case_ids' in r:
                if r.get('categ_id'):
                    filtro = '= %d' % r.get('categ_id')[0]
                else:
                    filtro = 'IS NULL'
                cr.execute("""SELECT * FROM crm_br_case
                            WHERE section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                            AND state not in ('cancel', 'done') AND categ_id %s""" % filtro)
                case_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                registros[inc]['case_ids'] = case_ids
        return registros

    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, "report_crm_br_open_category")
        cr.execute("""create view report_crm_br_open_category as (
                        SELECT rank() over(order by C.categ_id) AS id, C.categ_id, count(C.id) as total, T.parent_id FROM crm_br_case C
                        left join crm_br_case_categ T on T.id = C.categ_id
                        WHERE C.section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                        AND C.state not in ('cancel', 'done') group by categ_id, parent_id)"""
        )


report_crm_br_open_category()


class report_crm_br_open_stage(osv.Model):
    _name = "report.crm_br.open.stage"
    _description = "Atendimentos em Aberto por Estagio"
    _auto = False
    _columns = {
        'stage_id': fields.many2one('crm_br.case.stage', 'Estagio', readonly=True),
        'total': fields.integer('Total', readonly=True),
        'seq': fields.integer('Sequencia', readonly=True),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child Sections'),
    }
    _order = 'seq'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_open_stage, self).read(cr, uid, ids, [], context=context,
                                                               load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            if 'case_ids' in r:
                if r.get('stage_id'):
                    filtro = '= %d' % r.get('stage_id')[0]
                else:
                    filtro = 'IS NULL'
                cr.execute("""SELECT * FROM crm_br_case
                            WHERE section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                            AND state not in ('cancel', 'done') AND stage_id %s""" % filtro)
                case_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                registros[inc]['case_ids'] = case_ids
        return registros

    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, "report_crm_br_open_stage")
        cr.execute("""create view report_crm_br_open_stage as (
                        SELECT rank() over(order by C.stage_id) AS id, C.stage_id, count(C.id) as total, S.sequence as seq FROM crm_br_case C
                        left join crm_br_case_stage S on S.id = C.stage_id
                        WHERE C.section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                        AND state not in ('cancel', 'done') group by stage_id, S.sequence
                        )"""
        )


report_crm_br_open_stage()


class report_crm_br_state(osv.Model):
    _name = "report.crm_br.state"
    _description = "Atendimentos por Estado"
    _auto = False
    _columns = {
        'state': fields.selection(AVAILABLE_STATES, 'Estado', size=16, readonly=True),
        'total': fields.integer('Total', readonly=True),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child States'),
    }
    _order = 'total'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_state, self).read(cr, uid, ids, [], context=context,
                                                          load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            if 'case_ids' in r:
                if r.get('state'):
                    filtro = "= '%s'" % r.get('state')
                else:
                    filtro = 'IS NULL'
                cr.execute("""SELECT * FROM crm_br_case
                            WHERE section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                            AND state %s""" % filtro)
                case_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                registros[inc]['case_ids'] = case_ids
        return registros

    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, "report_crm_br_state")
        cr.execute("""create view report_crm_br_state as (
                        SELECT rank() over(order by state) AS id, state, count(id) as total FROM crm_br_case
                        WHERE section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                        group by state)"""
        )


report_crm_br_state()


class report_crm_br_open_user_id(osv.Model):
    _name = "report.crm_br.open.user_id"
    _description = "Atendimentos em Aberto por Responsavel"
    _auto = False
    _columns = {
        'user_id': fields.many2one('res.users', 'Responsavel', readonly=True),
        'total': fields.integer('Total', readonly=True),
        'tempo': fields.integer('T. Prev.', readonly=True),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child Sections')
    }
    _order = 'user_id'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_open_user_id, self).read(cr, uid, ids, [], context=context,
                                                                 load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            if 'case_ids' in r:
                if r.get('user_id'):
                    filtro = '= %d' % r.get('user_id')[0]
                else:
                    filtro = 'IS NULL'
                cr.execute("""SELECT * FROM crm_br_case
                            WHERE section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                            AND stage_id = (SELECT id FROM crm_br_case_stage WHERE name = 'Atendimento')
                            AND state not in ('cancel', 'done') AND user_id %s""" % filtro)
                case_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                registros[inc]['case_ids'] = case_ids
        return registros

    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, "report_crm_br_open_user_id")
        cr.execute("""create view report_crm_br_open_user_id as (
                        SELECT rank() over(order by user_id) AS id, user_id, count(id) as total,
                        sum(crmcas_duration - case when total_tempo_atend < crmcas_duration then total_tempo_atend else crmcas_duration end) as tempo FROM crm_br_case
                        WHERE section_id = (SELECT id FROM crm_br_case_section WHERE code = 'BugSup')
                        AND stage_id = (SELECT id FROM crm_br_case_stage WHERE name = 'Atendimento')
                        AND state not in ('cancel', 'done') group by user_id)"""
        )


report_crm_br_open_user_id()


class report_crm_br_atendimentos_x_user(osv.Model):
    _name = "report.crm_br.atendimentos.x.user"
    _description = "Atendimentos X Responsavel(Desenvolvimento)"
    _auto = False
    _columns = {
        'data_atend': fields.date('Data', readonly=True),
        'data_criacao':fields.datetime('Data Criacao',readonly=True),
        'total_atend': fields.float('Tempo', readonly=True),
        'user_id': fields.many2one('res.users', 'Responsavel', readonly=True),
        'qtd_atend': fields.integer('Atendimentos'),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child Sections')
    }
    _order = 'data_atend desc'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_atendimentos_x_user, self).read(cr, uid, ids, [],
                                                                        context=context, load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            cr.execute("""SELECT DISTINCT(crmcal_case_id) FROM crm_br_case_log
                WHERE DATE(crmcal_date) = %r AND crmcal_user_id = %d;""" % (
                r.get('data_atend'), r.get('user_id')[0]))
            case_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
            registros[inc]['case_ids'] = case_ids
        return registros

    def init(self, cr):

        tools.sql.drop_view_if_exists(cr, "report_crm_br_atendimentos_x_user")
        cr.execute("""create or replace  view report_crm_br_atendimentos_x_user as (
                    select
                    min(id) as id,
                    count(distinct  crm_log.crmcal_case_id) as qtd_atend,
                    crm_log.crmcal_user_id as user_id ,
                    DATE(crm_log.create_date) as data_criacao,
                    date(crm_log.create_date) as data_atend,
                    sum(crm_log.crmcal_tempo_atend) as total_atend
                    from crm_br_case_log  crm_log
                    where
                    date(crm_log.create_date) >= (current_date - interval '5 days')
                    and
                    date(crm_log.create_date)  <= current_date

                    and crm_log.crmcal_section_id  in  (select id from crm_br_case_section where
                    code = 'BugSup')

                    and (date_trunc('second',crm_log.create_date),id) in (
                        select
                        distinct
                        date_trunc('second',create_date),min(id)
                        from crm_br_case_log
                        group by date_trunc('second',create_date)
                        )
                  group by
                    date(crm_log.create_date), crm_log.crmcal_user_id
                        )
                    """)

report_crm_br_atendimentos_x_user()


class report_crm_br_atividades_executadas(osv.Model):
    _name = "report.crm_br.atividades.executadas"
    _description = "Atividades executadas"
    _auto = False
    _columns = {
        'data_atend': fields.date('Data', readonly=True,select=True),
        'total_atend': fields.float('Tempo', readonly=True),
        'user_id': fields.many2one('res.users', 'Responsavel', readonly=True),
        'qtd_atend': fields.integer('Atendimentos'),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child Sections')
    }
    _order = 'data_atend desc'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_atividades_executadas, self).read(cr, uid, ids, [],
                                                                          context=context,
                                                                          load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            sql = """SELECT DISTINCT(crmcal_case_id)
                    FROM crm_br_case_log crm_log
                    LEFT JOIN crm_br_case crm_case on crm_case.id = crm_log.crmcal_case_id
                WHERE DATE(crm_case.date_deadline) = %r AND crmcal_user_id = %d
                AND crm_case.state = 'done';""" % (
                r.get('data_atend'), r.get('user_id')[0])
            cr.execute(sql)
            dados = cr.fetchall()
            qtd = len(dados)
            r['qtd_atend'] = qtd
            case_ids = filter(None, map(lambda x: x[0], dados))
            registros[inc]['case_ids'] = case_ids


        return registros

    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, "report_crm_br_atividades_executadas")
        cr.execute("""create or replace  view report_crm_br_atividades_executadas as (
                    select
                    min(crm_log.id) as id,
                    count(crm_case.id) as qtd_atend,
                    crm_log.crmcal_user_id as user_id ,
                    date(crm_case.date_deadline) as data_atend,
                    sum(crm_log.crmcal_tempo_atend) as total_atend
                    from crm_br_case_log  crm_log
                    left join crm_br_case crm_case on crm_case.id = crm_log.crmcal_case_id
                    where
                    date(crm_case.date_closed) >= (current_date - interval '5 days')
                    and
                    date(crm_case.date_deadline) <= current_date
                    and crm_log.crmcal_section_id  in  (select id from crm_br_case_section where code = 'BugSup')
                    and crm_case.state = 'done'
                    group by
                    date(crm_case.date_deadline),crm_log.crmcal_user_id

                    )
                    """)
report_crm_br_atividades_executadas()


class report_crm_br_open_age(osv.Model):
    _name = "report.crm_br.open.age"
    _description = "Atendimentos em Aberto por Idade"
    _auto = False
    _columns = {
        'data': fields.date('Data Abertura'),
        'user_id': fields.many2one('res.users', 'Responsavel', readonly=True),
        'qtd': fields.integer('Total', readonly=True),
        'priority': fields.selection([('', '')] + AVAILABLE_PRIORITIES, 'Prioridade', readonly=True,
                                     size=1),
        'dias': fields.integer('Idade (dias)', readonly=True),
        'case_ids': fields.one2many('crm_br.case', 'id', 'Child Sections')
    }
    _order = 'priority, data, user_id'

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        registros = super(report_crm_br_open_age, self).read(cr, uid, ids, [], context=context,
                                                             load=load)
        inc = 0
        if isinstance(ids, int):
            registros = [registros]
        for r in registros:
            if 'case_ids' in r:
                f_user = 'IS NULL'
                f_data = 'IS NULL'
                f_priori = 'IS NULL'
                if r.get('user_id'):
                    f_user = '= %s' % r.get('user_id')[0]
                if r.get('data'):
                    f_data = '= %r' % r.get('data')
                if r.get('priority'):
                    f_priori = "= '%s'" % r.get('priority')
                sql = """SELECT id
                         FROM   report_crm_br_open_age_aux
                         WHERE  priority %s
                               AND user_id %s
                               AND Date(data_abertura) %s""" % (f_priori, f_user, f_data)
                cr.execute(sql)
                case_ids = filter(None, map(lambda x: x[0], cr.fetchall()))
                registros[inc]['case_ids'] = case_ids
        return registros

    def init(self, cr):
        view_auxiliar = """
            DROP VIEW IF EXISTS report_crm_br_open_age_aux cascade;
            CREATE VIEW report_crm_br_open_age_aux
            AS
              (SELECT priority,
                      user_id,
                      id,
                      (SELECT Min(crmcal_date)
                       FROM   crm_br_case_log
                       WHERE  crmcal_case_id = caso.id
                              AND name = 'Pendente'
                              AND crmcal_stage_id = (SELECT id
                                              FROM   crm_br_case_stage
                                              WHERE  name = 'Atendimento')) AS data_abertura
               FROM   crm_br_case caso
               WHERE  state IN ( 'open', 'pending', 'parado' )
                      AND stage_id = (SELECT id
                                      FROM   crm_br_case_stage
                                      WHERE  name = 'Atendimento')
                      AND section_id =
                                   (SELECT id
                                   FROM    crm_br_case_section
                                   WHERE   code = 'BugSup'
                                   )
               GROUP  BY priority,
                         user_id,
                         data_abertura,
                         id
               ORDER  BY data_abertura,
                         user_id);"""
        view = """
            DROP VIEW IF EXISTS report_crm_br_open_age;
            CREATE VIEW report_crm_br_open_age
            AS
              (SELECT Rank() OVER(ORDER BY user_id, priority, Date(data_abertura)) AS id,
                       priority,
                       user_id,
                       Count(Date(data_abertura))                                   AS qtd,
                       Date(data_abertura)                                          AS data,
                       CURRENT_DATE - Date(data_abertura)                           AS dias
                FROM   report_crm_br_open_age_aux
                GROUP  BY priority,
                          user_id,
                          data
                ORDER  BY priority,
                          data,
                          user_id
              );"""
        for sql in (view_auxiliar, view):
            cr.execute(sql)


report_crm_br_open_age()
