# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import fields, osv
from openerp.osv import orm

from openerp.tools.translate import _

def _get_answers(cr, uid, ids):
    query = """
    select distinct(answer)
    from profile_question_yes_rel
    where profile in (%s)""" % ','.join([str(i) for i in ids ])

    cr.execute(query)
    ans_yes = [x[0] for x in cr.fetchall()]

    query = """
    select distinct(answer)
    from profile_question_no_rel
    where profile in (%s)""" % ','.join([str(i) for i in ids ])

    cr.execute(query)
    ans_no = [x[0] for x in cr.fetchall()]

    return [ans_yes, ans_no]


def _get_parents(cr, uid, ids):
    ids_to_check = ids
    cr.execute("""
     select distinct(parent_id)
     from crm_br_segmentation
     where parent_id is not null
     and id in (%s)""" % ','.join([str(i) for i in ids ]))

    parent_ids = [x[0] for x in cr.fetchall()]

    trigger = False
    for x in parent_ids:
        if x not in ids_to_check:
            ids_to_check.append(x)
            trigger = True

    if trigger:
        ids_to_check = _get_parents(cr, uid, ids_to_check)

    return ids_to_check


def test_prof(cr, uid, seg_id, pid, answers_ids=[]):
#return True if the pessoa pid fetch the segmentation rule seg_id
    ids_to_check = _get_parents(cr, uid, [seg_id])
    [yes_answers, no_answers] = _get_answers(cr, uid, ids_to_check)
    temp = True
    for y_ans in yes_answers:
        if y_ans not in answers_ids:
            temp = False
            break
    if temp:
        for ans in answers_ids:
            if ans in no_answers:
                temp = False
                break
    if temp:
        return True
    return False


def _recompute_categ(self, cr, uid, pid, answers_ids):
    ok = []
    cr.execute('''
        select r.categoria_id 
        from pessoa_categoria r left join crm_br_segmentation s on (r.categoria_id = s.categ_id) 
        where r.pessoa_id = %s and (s.exclusif = false or s.exclusif is null)
        ''', (pid,))
    for x in cr.fetchall():
        ok.append(x[0])

    query = '''
        select id, categ_id 
        from crm_br_segmentation 
        where profiling_active = true'''
    if ok != []:
        query = query + ''' and categ_id not in(%s)''' % ','.join([str(i) for i in ok ])
    query = query + ''' order by id '''

    cr.execute(query)
    segm_cat_ids = cr.fetchall()

    for (segm_id, cat_id) in segm_cat_ids:
        if test_prof(cr, uid, segm_id, pid, answers_ids):
            ok.append(cat_id)
    return ok



class crm_br_segmentation(osv.Model):
    _name = "crm_br.segmentation"
    _inherit = "crm_br.segmentation"

    def process_continue(self, cr, uid, ids, start=False, context=None):
        context = context and context.copy() or {}
        categs = self.read(cr, uid, ids, ['categ_id', 'exclusif', 'pessoa_id', 'sales_purchase_active', 'profiling_active'], context=context)
        for categ in categs:
            if start:
                if categ['exclusif']:
                    cr.execute('delete from pessoa_categoria where categoria_id=%s', (categ['categ_id'][0],))

            id = categ['id']

            cr.execute('select id from adm_pessoa order by id ')
            pessoas = [x[0] for x in cr.fetchall()]

            if categ['sales_purchase_active']:
                to_remove_list = []
                cr.execute('select id from crm_br_segmentation_line where segmentation_id=%s', (id,))
                line_ids = [x[0] for x in cr.fetchall()]

                for pid in pessoas:
                    if (not self.pool.get('crm_br.segmentation.line').test(cr, uid, line_ids, pid, context=context)):
                        to_remove_list.append(pid)
                for pid in to_remove_list:
                    pessoas.remove(pid)

            if categ['profiling_active']:
                to_remove_list = []
                for pid in pessoas:

                    cr.execute('select distinct(answer) from adm_pessoa_question_rel where pessoa=%s', (pid,))
                    answers_ids = [x[0] for x in cr.fetchall()]

                    if (not test_prof(cr, uid, id, pid, answers_ids)):
                        to_remove_list.append(pid)
                for pid in to_remove_list:
                    pessoas.remove(pid)

            for pessoa_id in pessoas:
                cr.execute('insert into pessoa_categoria (categoria_id,pessoa_id) values (%s,%s)', (categ['categ_id'][0], pessoa_id))
            cr.commit()

            self.write(cr, uid, [id], {'state':'not running', 'pessoa_id':0}, context=context)
            cr.commit()
        return True

