# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv
import time
from openerp.tools.translate import _


class crm_br_case(osv.Model):

    _inherit = "crm_br.case"

    def create(self, cr, uid, vals, context=None):
        """
        O metodo cria um novo registro na tabela crm_br_case.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param vals: dicionario com a descricao dos campos e seus respectivos valores.
        :type vals: dict
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: id do novo registro.
        :rtype: int
        """
        context = context and context.copy() or {}
        res = super(crm_br_case, self).create(cr, uid, vals, context)
        if res:
            if not self.browse(cr, uid, res, context).crmcas_contrato_item_ids:
                self.write(cr, uid, [res], {'crmcas_faturar': True}, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        """
        O metodo atualiza registros na tabela crm_br_case.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param vals: dicionario com a descricao dos campos e seus respectivos valores.
        :type vals: dict
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso consiga atualizar os registros, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = super(crm_br_case, self).write(cr, uid, ids, vals, context)
        case_ids = list()
        if not isinstance(ids, list):
            ids = [ids]
        for case in self.read(cr, uid, ids, ['crmcas_contrato_item_ids', 'crmcas_faturar'],
                              context):
            if not case.get('crmcas_contrato_item_ids') and not case.get('crmcas_faturar'):
                case_ids.append(case.get('id'))
        if case_ids:
            case_ids = map(lambda i: int(i), case_ids)
            if len(case_ids) == 1:
                case_ids = str(tuple(case_ids)).replace(',', '')
            else:
                case_ids = str(tuple(case_ids))
            sql = """UPDATE crm_br_case SET crmcas_faturar = TRUE WHERE id IN %s""" % case_ids
            cr.execute(sql)
        return res
