# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_crmbr_faturamento(osv.TransientModel):
    # prefixo: wcrmfa_

    _name = 'wizard.crmbr.faturamento'

    _columns = {
        'wcrmfa_data_inicial': fields.date('Data inicial', required=True),
        'wcrmfa_data_final': fields.date('Data final', required=True),
        'wcrmfa_item_ids': fields.one2many('wizard.crmbr.faturamento.item', 'wcrmfi_capa_id',
                                           'Itens'),
        'wcrmfa_pesquisado': fields.boolean('Pesquisado'),
        'wcrmfa_faturado': fields.boolean('Faturado'),
    }

    _defaults = {
        'wcrmfa_pesquisado': lambda *a: False,
        'wcrmfa_faturado': lambda *a: False
    }

    def wcrmfa_button_find(self, cr, uid, ids, context=None):
        """
        O metodo encontra todos os atendimentos de helpdesk (crm_br_case, section_id = 'HD') que se
        encontram dentro do periodo informado e que sao a faturar.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso encontre algum helpdesk, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False

        preco_venda = self.wcrmfa_get_produto_dados(cr, uid, context).get('preco_venda')

        cr.execute('DELETE FROM wizard_crmbr_faturamento_item')
        for x in self.browse(cr, uid, ids, context):
            sql = """
                SELECT
                    crm.id                                                    AS wcrmfi_crm_id,
                    crm.pessoa_id                                             AS wcrmfi_pessoa_id,
                    crm.date                                                  AS wcrmfi_data,
                    crm.user_id                                               AS wcrmfi_user_id,
                    round(crm.total_tempo_atend * %s, 5)                      AS wcrmfi_valor
                FROM crm_br_case crm
                INNER JOIN crm_br_case_section sec
                    ON sec.id = crm.section_id
                WHERE
                    crm.state = 'done'
                    AND sec.code = 'HD'
                    AND crm.date >= '%s 00:00:00'
                    AND crm.date <= '%s 23:59:59'
                    AND crm.crmcas_faturar = TRUE
                    AND (crm.crmcas_pedido_item_id IS NULL OR crm.crmcas_pedido_item_id IN (
                        SELECT id FROM estoquepedidos_pedido_item WHERE pedido_id IN (
                            SELECT id FROM estoquepedidos_pedido_capa WHERE state = 'C')))
                GROUP BY
                    crm.id, crm.pessoa_id, crm.date, crm.user_id
            """ % (preco_venda, x.wcrmfa_data_inicial, x.wcrmfa_data_final)
            cr.execute(sql)
            crm_dados = cr.dictfetchall()
            if not crm_dados:
                continue
            vals = []
            map(lambda i: vals.append((0, 0, i)), crm_dados)
            res = self.write(cr, uid, x.id, {'wcrmfa_item_ids': vals, 'wcrmfa_pesquisado': True},
                             context)
        return res

    def wcrmfa_button_faturar(self, cr, uid, ids, context=None):
        """
        O metodo vai gerar os pedidos de venda por cliente para todos os atendimentos helpdesk
        encontrados.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso gere os pedidos de venda, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}

        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        obj_crm_br_case = self.pool.get('crm_br.case')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')

        produto_dados = self.wcrmfa_get_produto_dados(cr, uid, context)
        local_id = produto_dados.get('local_id')
        produto_id = produto_dados.get('produto_id')
        preco_venda = produto_dados.get('preco_venda')
        grade_id = produto_dados.get('grade_id')
        produto_embalagem_id = produto_dados.get('produto_embalagem_id')

        res = False
        for x in self.browse(cr, uid, ids, context):
            pessoa_ids = list(set(map(lambda i: i.wcrmfi_pessoa_id.id, x.wcrmfa_item_ids)))
            for pessoa_id in pessoa_ids:
                crm_faturamento_itens = filter(lambda i: i.wcrmfi_pessoa_id.id == pessoa_id,
                                               x.wcrmfa_item_ids)
                endereco_id = False
                pessoa_endereco_ids = obj_adm_pessoa_endereco.search(
                    cr, uid, [('pessoa_id', '=', pessoa_id)], context=context)
                if pessoa_endereco_ids:
                    endereco_id = pessoa_endereco_ids[0]
                vals_pedido = {'pessoa_id': pessoa_id, 'pedcap_endereco_id': endereco_id,
                               'tipo': 'V'}
                pedido_id = obj_estoquepedidos_pedido_capa.create(cr, uid, vals_pedido, context)
                obs = ''
                for item in crm_faturamento_itens:
                    if not item.wcrmfi_crm_id.crmcas_faturar:
                        continue

                    obs += '%s, ' % item.wcrmfi_crm_id.id

                    vendedor_ids = obj_vendas_vendedor.search(
                        cr, uid, [('pessoa_id', '=', item.wcrmfi_user_id.pessoa_id.id)])
                    if not vendedor_ids:
                        vendedor_id = obj_vendas_vendedor.create(
                            cr, uid, {'pessoa_id': item.wcrmfi_user_id.pessoa_id.id}, context)
                    else:
                        vendedor_id = vendedor_ids[0]

                    quantidade = item.wcrmfi_crm_id.total_tempo_atend

                    vals_pedido_item = {
                        'pedido_id': pedido_id,
                        'qtde_pedida': quantidade,
                        'vr_unitario': preco_venda,
                        'produto_id': produto_id,
                        'grade_id': grade_id,
                        'local_id': local_id,
                        'produto_embalagem_id': produto_embalagem_id,
                        'vendedor_id': vendedor_id}
                    pedido_item_id = obj_estoquepedidos_pedido_item.create(
                        cr, uid, vals_pedido_item, context)
                    obj_crm_br_case.write(
                        cr, uid, item.wcrmfi_crm_id.id,
                        {'crmcas_pedido_item_id': pedido_item_id}, context)

                if obs:
                    obj_estoquepedidos_pedido_capa.write(
                        cr, uid, pedido_id,
                        {'observacao': 'Numero dos atendimentos helpdesk: %s.' % obs[:-2]}, context)

        self.write(cr, uid, ids, {'wcrmfa_faturado': True}, context)
        return res

    def wcrmfa_get_produto_dados(self, cr, uid, context=None):
        """
        O metodo encontra o produto para helpdesk, nos parametros do estoque, como tambem demais
        dados referentes a ele.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com os dados do id e preco de venda do produto.
        :rtype: dict
        """
        context = context and context.copy() or {}

        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        campos = ['local_venda_padrao_id', 'estpar_helpdesk_produto_id',
                  'estpar_embalagem_servico_id']
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', campos, context=context)

        embalagem_id = estoque_parametro_dados.get('estpar_embalagem_servico_id')
        if not embalagem_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre a "Embalagem para servicos" nos '
                                                 'parametros do estoque'))

        helpdesk_produto_id = estoque_parametro_dados.get('estpar_helpdesk_produto_id')
        if not helpdesk_produto_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre o "Produto para helpdesk" nos '
                                                 'parametros do estoque.'))

        preco_venda_dados = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, epc_produto_ids=helpdesk_produto_id, context=context)
        preco_venda = preco_venda_dados.get(helpdesk_produto_id) or 0
        if preco_venda:
            preco_venda = preco_venda.get('preco_venda') or 0

        local_id = estoque_parametro_dados.get('local_venda_padrao_id')
        if not local_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre o "Local para saidas" nos parametros do '
                                                 'estoque.'))

        grade_ids = obj_estoque_produto_grade.search(
            cr, uid, [('produto_id', '=', helpdesk_produto_id)], context=context)

        produto_embalagem_ids = obj_estoque_produto_embalagem.search(
            cr, uid, [('produto_id', '=', helpdesk_produto_id)], context=context)

        if produto_embalagem_ids:
            produto_embalagem_id = produto_embalagem_ids[0]
        else:
            produto_embalagem_id = obj_estoque_produto_embalagem.create(
                cr, uid, {'produto_id': helpdesk_produto_id, 'embalagem_id': embalagem_id}, context)
        return {'produto_id': helpdesk_produto_id, 'preco_venda': preco_venda, 'local_id': local_id,
                'produto_embalagem_id': produto_embalagem_id, 'grade_id': grade_ids[0]}


class wizard_crmbr_faturamento_item(osv.TransientModel):
    # prefixo: wcrmfi_

    _name = 'wizard.crmbr.faturamento.item'

    _columns = {
        'wcrmfi_capa_id': fields.many2one('wizard.crmbr.faturamento', 'Capa'),
        'wcrmfi_crm_id': fields.many2one('crm_br.case', 'Protocolo'),
        'wcrmfi_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wcrmfi_data': fields.datetime('Data'),
        'wcrmfi_user_id': fields.many2one('res.users', 'Responsavel'),
        'wcrmfi_valor': fields.float('Valor', digits=(15, 5)),
    }