# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from dateutil.parser import parse
import time


class delivery_acertos(osv.Model):
    # prefixo: dvyace_

    _inherit = 'delivery.acertos'

    def name_get(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        res = []
        for i in self.read(cr, user, ids, ['dvyace_data', 'dvyace_entregador_id'], context=context):
            name = '%s - %s' % (time.strftime('%d/%m/%Y', time.strptime(i.get('dvyace_data'),
                                                                        '%Y-%m-%d')),
                                i.get('dvyace_entregador_id')[1].split(' - ')[1])
            res.append((i.get('id'), name))

        return res

    def dvyace_button_efetivar(self, cr, uid, ids, context=None):
        """
        O metodo altera o status para 'F' (efetivado) desde que consiga movimentar na tesouraria o
        valor recebido das entregas.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificar do usuario
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais
        :return: booleano True
        """
        context = context and context.copy() or {}
        self.dvyace_movimenta_tesouraria(cr, uid, ids, context=context)
        self.write(cr, uid, ids, {'state': 'F'}, context)
        obj_delivery_entregas = self.pool.get('delivery.entregas')
        obj_adm_utils = self.pool.get('adm.utils')
        entrega_ids = []
        delivery_acertos_dados = self.read(cr, uid, ids, ['dvyace_entrega_ids'], context)
        map(lambda i: entrega_ids.extend(i.get('dvyace_entrega_ids')), delivery_acertos_dados)
        brw_delivery_entregas = obj_delivery_entregas.browse(cr, uid, entrega_ids, context)
        msg = ''
        for entrega in brw_delivery_entregas:
            if not entrega.dvyeta_acerto_id:
                continue
            numero_entrega = entrega.dvyeta_numero
            entregador = entrega.dvyeta_acerto_id.dvyace_entregador_id.dvyeto_pessoa_id.nome \
                or ''
            create_date = entrega.dvyeta_acerto_id.create_date
            create_date = obj_adm_utils.admutl_datetime_to_timezone(
                cr, uid, parse(create_date), '%d/%m/%Y %H:%M:%S')
            msg += '%s - %s / %s\n' % (numero_entrega, entregador, create_date)
        if msg:
            raise osv.except_osv(_('Atencao'), _('As seguintes entregas ja estao cadastradas para '
                                                 'os respectivos acertos:\n %s' % msg))
        for _id in ids:
            obj_delivery_entregas.write(cr, uid, entrega_ids, {'dvyeta_acerto_id': _id}, context)
        return True

    def dvyace_custificar_acerto_entregador(self, cr, uid, acerto, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                    tabela='financeiro.parametros', context=context)
        campos = ['finpar_c_custo_acerto_entregador_id','finpar_evento_acerto_entregador_id',
                  'usar_centro_custos_e_eventos']
        parametros = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                            'financeiro.parametros', campos, company_id=company_id, context=context)
        if not parametros.get('usar_centro_custos_e_eventos') or \
                not acerto.get('dvyace_valor_recebido'):
            return False

        evento_id = parametros.get('finpar_evento_acerto_entregador_id')
        ccusto_id = parametros.get('finpar_c_custo_acerto_entregador_id')
        msg = []
        if not ccusto_id:
            msg.append(_("Centro de custo para 'Acerto Entregador' nao cadastrado nos parametros do"
                         " financeiro."))
        if not evento_id:
            msg.append(_("Evento para 'Acerto Entregador' nao cadastrado nos parametros do "
                         "financeiro."))

        if msg:
            raise osv.except_osv(_('Atencao!'), '\n'.join(msg))

        vals = dict(valor=acerto.get('dvyace_valor_recebido'),
                    evento_id=evento_id,
                    centro_custo_id=ccusto_id,
                    company_id=company_id,
                    pessoa_id=acerto.get('dvyace_entregador_id')[0],
                    fingmo_operacao='E',
                    mes_competencia=acerto.get('dvyace_data')[5:7],
                    ano_competencia=acerto.get('dvyace_data')[:4],
                    fingmo_data=acerto.get('dvyace_data'),
                    fingmo_acerto_id=acerto.get('id'))

        return vals

    def dvyace_button_cancelar(self, cr, uid, ids, context=None):
        """
        O metodo altera o status para 'C' (cancelado) desde que consiga movimentar na tesouraria o
        valor estornado das entregas.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificar do usuario
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais
        :return: booleano True
        """
        context = context and context.copy() or {}
        self.dvyace_movimenta_tesouraria(cr, uid, ids, estorno=True, context=context)
        self.write(cr, uid, ids, {'state': 'C'}, context)
        entrega_ids = []
        delivery_acertos_dados = self.read(cr, uid, ids, ['dvyace_entrega_ids'], context)
        map(lambda i: entrega_ids.extend(i.get('dvyace_entrega_ids')), delivery_acertos_dados)
        self.pool.get('delivery.entregas').write(cr, uid, entrega_ids, {'dvyeta_acerto_id': False},
                                                 context)
        return True

    def dvyace_movimenta_tesouraria(self, cr, uid, ids, estorno=False, context=None):
        """
        O metodo lanca na tesouraria os valores referentes as entregas, no momento do acerto do
        entregador com a empresa. Caso o acerto seja efetivado, e realizado um credito. Caso o
        acerto seja cancelado, e realizado um debito (estorno).
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param ids: lista de id da tabela desse objeto
        :param estorno: booleano. Caso seja True, indica que sera realizado o debito na tesouraria
        :param context: dicionario com valores contextuais
        :return: booleano True
        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        obj_gerencial_mov = self.pool.get('gerencial.movimento')
        financeiro_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'financeiro', 'finpar_conta_tesouraria_avista_id', context=context)
        conta_id = financeiro_parametro_dados.get('finpar_conta_tesouraria_avista_id')
        if not conta_id:
            raise osv.except_osv(_('Atencao'), _('Conta caixa para recebimento a vista nao '
                                                 'cadastrada nos parametros do financeiro'))
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        campos = ['dvyace_entregador_id', 'dvyace_valor_recebido', 'dvyace_data']
        unlink_gerencial_ids = []
        for acerto in self.read(cr, uid, ids, campos, context=context):
            valor_recebido = acerto.get('dvyace_valor_recebido') or 0.0
            entregador = acerto.get('dvyace_entregador_id')[1].split(' - ')[1] or ''
            historico = 'Recebimento de entregas. Entregador %s' % entregador
            debito_credito = 'C'
            vals_gerencial_mov = {}
            if estorno:
                context['fintem_nao_verificar_custificacao'] = True
                historico = 'Estorno de recebimento de entregas. Entregador %s' % entregador
                debito_credito = 'D'
                unlink_gerencial_ids.extend(obj_gerencial_mov.search(cr, uid,
                                    [('fingmo_acerto_id', '=', acerto.get('id'))], context=context))
            else:
                vals_gerencial_mov = self.dvyace_custificar_acerto_entregador(cr, uid, acerto,
                                                                              context=context)

            data_movimento = acerto.get('dvyace_data')
            mes_competencia = data_movimento[5:7]
            ano_competencia = data_movimento[:4]
            vals_movimento = {
                'conta_id': conta_id,
                'data_movimento': data_movimento,
                'mes_competencia': mes_competencia,
                'ano_competencia': ano_competencia,
                'valor': valor_recebido,
                'historico': historico,
                'debito_credito': debito_credito,
            }
            if vals_gerencial_mov:
                vals_movimento['gerencial_movimento_ids'] = [[0,0, vals_gerencial_mov]]

            obj_financeiro_tesouraria_movimento.create(cr, uid, vals_movimento, context)

        if unlink_gerencial_ids:
            obj_gerencial_mov.unlink(cr, uid, unlink_gerencial_ids, context=context)

        return True
