# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

# ___________________________ delivery.entregas ______________________
def dvyeta_get_valor_total_pedidos(self, cr, user, ids, fild_name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos como tipo funcao.
    Calcula o valor total dos pedidos de entrega.

    :param cr: a linha atual do cursor do banco de dados,
    :param user: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param fild_name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :type cr: cursor
    :type user: int
    :type ids: long|list[long]
    :type arg: dict
    :type context: dict
    :return: valores ajustados dos campos.
    :rtype: dict

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}
    result = {}.fromkeys(ids, 0.0)

    for brw_entrega in self.browse(cr, user, ids if isinstance(ids, list) else [ids]):
        for brw_pedido in brw_entrega.dvyeta_pedido_ids:
            result[brw_entrega.id] = brw_pedido.valor_total

    return result

def dvyeta_get_quantidade_pedidos(self, cr, user, ids, fild_name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos como tipo funcao.
    Retorna a quantidade de pedidos de uma entrega.

    :param cr: a linha atual do cursor do banco de dados,
    :param user: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param fild_name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :type cr: cursor
    :type user: int
    :type ids: long|list[long]
    :type arg: dict
    :type context: dict
    :return: valores ajustados dos campos.
    :rtype: dict

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}
    result = {}.fromkeys(ids, 0)

    for brw_entrega in self.browse(cr, user, ids if isinstance(ids, list) else [ids]):
        result[brw_entrega.id] = len(brw_entrega.dvyeta_pedido_ids)

    return result


# ______________________ delivery.acertos _______________________
def _dvyace_get_valores(self, cr, uid, ids, field_name, args, context=None):
    """
    O metodo encontra o valor total das entregas realizadas pelo entregador, como tambem calcula a
    diferenca desse valor total com o valor recebido por ele.
    :param cr: cursor no banco de dados
    :param uid: id da tabela res_users, identificador do usuario
    :param ids: lista de id da tabela delivery_acertos
    :param field_name: char representado o nome do campo
    :param args: lista de argumentos
    :param context: dicionario com valores contextuais
    :return: dicionario, com chave e valor representando o campo e valor do total das entregas e
    diferenca
    """
    context = context and context.copy() or {}
    res = {}
    brw_delivery_acertos = self.browse(cr, uid, ids, context)
    for acerto in brw_delivery_acertos:
        valor_recebido = acerto.dvyace_valor_recebido or 0.0
        total_entregas = sum(map(lambda e: e.dvyeta_valor_total_pedidos, acerto.dvyace_entrega_ids))
        diferenca = valor_recebido - total_entregas
        res[acerto.id] = {'dvyace_total_entregas': total_entregas, 'dvyace_diferenca': diferenca}
    return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4

