# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report.render import render
from openerp.report.interface import report_int
import StringIO
import locale
from SigUtils.impressao import Impressora
# noinspection PyProtectedMember
from openerp.tools.translate import _
import time
from openerp import pooler
from datetime import datetime
from dateutil.parser import parse


# noinspection PyPep8Naming
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class RegDelivery(object):
    """
    Constroi a classe de registro de entrega. Comissao de entregas.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, **entries):
        self.entrega_numero = None
        self.entrega_create_date = None
        self.entrega_data_saida = None
        self.entrega_data_retorno = None
        self.pedido_quantidade = 0
        self.pedido_valor_total = 0.0
        self.entregador_nome = None
        self.entregador_horario_entrada = False
        self.entregador_horario_saida = False
        self.entregador_comissao_valor = 0.0
        self.__dict__.update(entries)


# Nome do relatorio
_report_name = 'report.relatorio.comissao.entrega'

# Converte ´float´ em horas e minutos (h:mm)
float2time = lambda t: '%d:%02d' % (int(t) if t < 23 else 0, (t - int(t)) * 60)
datetime_fmt = lambda dt: datetime.strptime(dt[:19], '%Y-%m-%d %H:%M:%S').strftime('%x %X')

# noinspection PyPep8Naming
class report_comissao_entrega(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        datetime_to_timezone = pooler.get_pool(cr.dbname).get(
            'adm.utils').admutl_datetime_to_timezone

        # Dados informados
        periodo = datas.get('periodo')
        entregador_id = datas.get('entregador')[0] if datas.get('entregador') else False

        # Cria o relatorio
        titulo = _('Comissao de entregadores')
        pdf_string = StringIO.StringIO()
        report = Impressora()
        report.defineFormato(orientacao='0')
        report.informaNome(pdf_string, titulo)
        report.informaTamLinha(160)
        report.defineMargem(0.5, 0.5, 0.5, 0.5)

        # Define a fomatacao de data e numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))
        date_format = locale.nl_langinfo(locale.D_FMT)

        # Cabecalho do relatorio
        try:
            company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'relatorio.comissao.entrega', context=context)
            empresa = pooler.get_pool(cr.dbname).get('res.company').browse(
                cr, uid, company_id).name
        except AttributeError:
            empresa = None

        det_format = '|{:^19}|{:^25}|{:^25}|{:^25}|{:>14}|{:>17}|{:>17}|'.decode('utf-8')
        det_tot_format = '|{:>97}|{:>14}|{:>17}|{:>17}|'.decode('utf-8')
        cab_format = ' {:<11} {:<50} {:>20} {:^5} {:>20} {:^5} {:>25} {:<3}%'.decode('utf-8')
        report.informaCab('{:^150}'.decode('utf-8').format(empresa))
        report.informaCab('')
        report.informaCab(' {:<10} {:^126} {:>10} '.decode('utf-8').format(
            time.strftime(date_format), titulo[:126], _('Pagina: ')[:10] + '{{ Page }}'))
        report.informaCab('')
        report.informaCab(cab_format.format(
            _('Entregador:')[:11], '', _('Horario de entrada:')[:20], '0:00',
            _('Horario de saida:'), '0:00', _('Comissao:')[:10], 0))
        report.informaCab('*' + '-' * 148 + '*')
        report.informaCab(det_format.format(
            _('Numero da entrega')[:19], _('Data da entrega')[:25], _('Data de saida')[:25],
            _('Data de retorno')[:25], _('Qtde. pedidos')[:14], _('Valor da entrega')[:17],
            _('Comissao')[:17]))
        report.informaCab('*' + '-' * 148 + '*')

        # Rodape
        report.rodapePadrao()

        # Registros seleciondos do banco
        registros = self._gera_consulta(cr, uid, periodo, entregador_id, context=context)

        # Inicializa subtotais e totais
        pedido_quantidade_total = pedido_quantidade = 0
        entrega_valor_total = entrega_valor_subtotal = 0.0
        comissao_valor_total = comissao_valor_subtotal = 0.0
        entregador_nome = None
        resumo = list()  # Guarda os subtotais dos entregadores

        for registro in registros:
            if entregador_id:
                if entregador_id != registro.dvyeta_entregador_id.id:
                    # Imprime subtotais
                    report.imprimeLinha('*' + '-' * 148 + '*')
                    report.imprimeLinha(det_tot_format.format(
                        _('TOTAL'), pedido_quantidade,
                        locale.format('%.2f', entrega_valor_subtotal, grouping=True),
                        locale.format('%.2f', comissao_valor_subtotal, grouping=True)))
                    report.imprimeLinha('*' + '-' * 148 + '*')

                    # Registro de resumo
                    reg_resumo = {
                        'entregador_nome': entregador_nome,
                        'pedido_quantidade': pedido_quantidade,
                        'pedido_valor_total': entrega_valor_subtotal,
                        'entregador_comissao_valor': comissao_valor_subtotal
                    }
                    resumo.append(RegDelivery(**reg_resumo))

                    # Nova pagina
                    entregador_id = registro.dvyeta_entregador_id.id
                    entregador_nome = registro.dvyeta_entregador_id.dvyeto_pessoa_id.nome[:50]
                    entregador_comissao = registro.dvyeta_entregador_id.dvyeto_comissao
                    report.cabecalhos[4] = cab_format.format(
                        _('Entregador:')[:11], entregador_nome, _('Horario de entrada:')[:20],
                        float2time(registro.dvyeta_entregador_id.dvyeto_horario_entrada),
                        _('Horario de saida:'),
                        float2time(registro.dvyeta_entregador_id.dvyeto_horario_saida),
                        _('Comissao:')[:10], locale.format('%.2f', entregador_comissao))
                    report.imprimeLinha('{{ Page }}')
                    pedido_quantidade = 0
                    entrega_valor_subtotal = comissao_valor_subtotal = 0.0
                else:
                    entregador_nome = registro.dvyeta_entregador_id.dvyeto_pessoa_id.nome[:50]
                    entregador_comissao = registro.dvyeta_entregador_id.dvyeto_comissao
            else:
                entregador_id = registro.dvyeta_entregador_id.id
                entregador_nome = registro.dvyeta_entregador_id.dvyeto_pessoa_id.nome[:50]
                entregador_comissao = registro.dvyeta_entregador_id.dvyeto_comissao

            # Ajusta os dados do entregador no cabecalho
            report.cabecalhos[4] = cab_format.format(
                _('Entregador:')[:11], entregador_nome, _('Horario de entrada:')[:20],
                float2time(registro.dvyeta_entregador_id.dvyeto_horario_entrada),
                _('Horario de saida:'),
                float2time(registro.dvyeta_entregador_id.dvyeto_horario_saida), _('Comissao:')[:10],
                locale.format('%.2f', entregador_comissao))

            # Imprime a linha detalhe
            pedido_valor_total = registro.dvyeta_valor_total_pedidos or 0.0
            comissao_valor = round(pedido_valor_total * entregador_comissao / 100.0, 2)
            report.imprimeLinha(det_format.format(
                registro.dvyeta_numero,
                datetime_fmt(datetime_to_timezone(
                    cr, uid, parse(registro.create_date), '%Y-%m-%d %H:%M:%S')),
                datetime_fmt(datetime_to_timezone(
                    cr, uid, parse(registro.dvyeta_entrega_saida),
                    '%Y-%m-%d %H:%M:%S')) if registro.dvyeta_entrega_saida else '',
                datetime_fmt(datetime_to_timezone(
                    cr, uid, parse(registro.dvyeta_entrega_retorno),
                    '%Y-%m-%d %H:%M:%S')) if registro.dvyeta_entrega_retorno else '',
                registro.dvyeta_quantidade_pedidos or 0,
                locale.format('%.2f', pedido_valor_total, grouping=True),
                locale.format('%.2f', comissao_valor, grouping=True)))

            # Subtotais e totais
            pedido_quantidade += registro.dvyeta_quantidade_pedidos or 0
            pedido_quantidade_total += registro.dvyeta_quantidade_pedidos or 0
            entrega_valor_subtotal += pedido_valor_total
            entrega_valor_total += pedido_valor_total
            comissao_valor_subtotal += comissao_valor
            comissao_valor_total += comissao_valor

        # Subtotal do ultimo entregador
        report.imprimeLinha('*' + '-' * 148 + '*')
        report.imprimeLinha(det_tot_format.format(
            _('TOTAL'), pedido_quantidade,
            locale.format('%.2f', entrega_valor_subtotal, grouping=True),
            locale.format('%.2f', comissao_valor_subtotal, grouping=True)))
        report.imprimeLinha('*' + '-' * 148 + '*')

        # Resumo somente se mais de um entregador
        if not datas.get('entregador'):
            # Registro de resumo do ultimo entregador
            reg_resumo = {
                'entregador_nome': entregador_nome,
                'pedido_quantidade': pedido_quantidade,
                'pedido_valor_total': entrega_valor_subtotal,
                'entregador_comissao_valor': comissao_valor_subtotal
            }
            resumo.append(RegDelivery(**reg_resumo))

            report.cabecalhos[4] = '{:^146}'.encode('utf-8').format(_('RESUMO')[:146])
            det_format = '|{:<97}|{:>14}|{:>17}|{:>17}|'.decode('utf-8')
            report.cabecalhos[6] = det_format.format(
                _('Entregador')[:97], _('Qtde. pedidos')[:14],
                _('Valor da entrega')[:17], _('Comissao'))
            report.imprimeLinha('{{ Page }}')  # Nova pagina
            pedido_quantidade_total = 0
            entrega_valor_total = comissao_valor_total = 0.0

            for item in resumo:
                report.imprimeLinha(det_format.format(
                    item.entregador_nome, item.pedido_quantidade,
                    locale.format('%.2f', item.pedido_valor_total),
                    locale.format('%.2f', item.entregador_comissao_valor)))
                pedido_quantidade_total += item.pedido_quantidade
                entrega_valor_total += item.pedido_valor_total
                comissao_valor_total += item.entregador_comissao_valor

            # Totais do resumo
            report.imprimeLinha('*' + '-' * 148 + '*')
            report.imprimeLinha(det_tot_format.format(
                _('TOTAL'), pedido_quantidade_total,
                locale.format('%.2f', entrega_valor_total, grouping=True),
                locale.format('%.2f', comissao_valor_total, grouping=True)))
            report.imprimeLinha('*' + '-' * 148 + '*')

        # Finaliza a impressao
        report.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    @staticmethod
    def _gera_consulta(cr, uid, periodo, entregador, context=None):
        """
        Realiza consulta SQL.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param periodo: data inicial e final que compreende o periodo da consulta
        :param entregador: ID do entregador (id -> adm.pessoa)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type periodo: tuple
        :type entregador: long
        :type context: dict
        :return: registros da consulta SQL
        :rtype: br

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects ________________________________
        obj_delivery_entregas = pooler.get_pool(cr.dbname).get('delivery.entregas')

        # Argumentos da consulta
        args = [('create_date', '>=', periodo[0]), ('create_date', '<=', periodo[1]),
                ('state', '=', 'ET')]

        if entregador:  # Apenas um entregador ou todos
            args.append(('dvyeta_entregador_id', '=', entregador))

        delivery_entrega_ids = obj_delivery_entregas.search(
            cr, uid, args, order= 'dvyeta_entregador_id, dvyeta_numero desc', context=context)

        return obj_delivery_entregas.browse(cr, uid, delivery_entrega_ids, context)

report_comissao_entrega(_report_name)
