# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


def _links_get(self, cr, uid, context=None):
    """
    Retorna o link para o tipo de objeto selecionado
    """
    context = context and context.copy() or {}
    obj = self.pool.get('res.request.link')
    ids = obj.search(cr, uid, [], context=context)
    res = obj.read(cr, uid, ids, ['object', 'name'], context=context)
    return [(r['object'], r['name']) for r in res]


class documento_documento_tipo(osv.Model):
    """
    Tabela de Tipos de Documentos.
    Os tipos a seguir sao pre-definidos enao podem ser excluido, para isto o atributo bloqueio deve estar com True
    Siglas pre-definidas:
    BOL-Boleto bancario
    CHQ-Cheque
    CON-Contrato
    COT-Cotacao
    CUP-Cupom Fiscal
    DUP-Duplicata
    FAT-Fatura
    GUI-Guia
    NF-Nota Fiscal
    ORC-Orcamento
    ORP-Ordem de Producao
    PEC-Pedido de Compra
    PEV-Pedido de Venda
    RQA-Requisicao de almoxarifado
    RQC-Requisicao de compras
    SLA- Solicitacao de itens do almoxarifado
    """
    _name = 'documento.documento.tipo'
    _inherit = 'documento.documento.tipo'


    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        """
        Pesquisa tipo de documentos pelo nome ou pela sigla
        """
        context = context and context.copy() or {}
        ids = []
        if len(name) in (1, 2, 3):
            ids = self.search(cr, user, [('sigla', '=', name.upper())] + args, limit=limit, context=context)
        if not ids:
            ids = self.search(cr, user, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    def unlink(self, cr, uid, ids, context=None):
        """
        Metodo para excluir tipo de documento.
        Nao permite excluir tipo de documento cujo atibuto bloqueio seja True
        """
        context = context and context.copy() or {}
        for documento_tipo in self.browse(cr, uid, ids):
            if documento_tipo.bloqueio == True:
                raise osv.except_osv(_('Mensagem:'), _('Este tipo de documento nao pode ser excluido !'))
        return super(documento_documento_tipo, self).unlink(cr, uid, ids)

    def create(self, cr, uid, vals, context=None):
        """
        Metodo para inserir novos tipos de documento
        Converte a sigla para maiuscula
        """
        context = context and context.copy() or {}
        if 'sigla' in vals and vals.get('sigla'):
            vals['sigla'] = vals['sigla'].upper()
        result = super(documento_documento_tipo, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        """
        Metodo para atualizar tabela de tipos de documentos.
        Nao permite alterar tipo de documentos que tenham atributo bloqueio = True
        """
        context = context and context.copy() or {}
        if vals.get('sigla'):
            vals['sigla'] = vals['sigla'].upper()
        for documento_tipo in self.browse(cr, uid, ids):
            if documento_tipo.bloqueio:
                if 'docdot_centro_custo_id' in vals or 'docdot_evento_id' in vals and vals.__len__() in (1, 2):
                    continue
                else:
                    raise osv.except_osv(_('Mensagem:'), _('Este tipo de documento nao pode ser alterado !'))
        ''' Gravando a sigla e descricao no campo name '''
        result = super(documento_documento_tipo, self).write(cr, uid, ids, vals, context)
        return result

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """
        Metodo para pesquisar tipo de documentos com opcao para usar a sigla como dominio de pesquisa
        """
        context = context and context.copy() or {}
        if context.get('sigla'):
            args.append(('sigla', '=', context['sigla']))
        return super(documento_documento_tipo, self).search(cr, uid, args, offset, limit,
                                                            order, context=context, count=count)

