# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv, fields


class documento_documento_tipo(osv.Model):
    """
    Tabela de Tipos de Documentos.
    Os tipos a seguir sao pre-definidos enao podem ser excluido, para isto o atributo bloqueio deve estar com True
    Siglas pre-definidas:
    BOL-Boleto bancario
    CHQ-Cheque
    CON-Contrato
    COT-Cotacao
    CUP-Cupom Fiscal
    DUP-Duplicata
    FAT-Fatura
    GUI-Guia
    NF-Nota Fiscal
    ORC-Orcamento
    ORP-Ordem de Producao
    PEC-Pedido de Compra
    PEV-Pedido de Venda
    RQA-Requisicao de almoxarifado
    RQC-Requisicao de compras
    SLA- Solicitacao de itens do almoxarifado
    """

    from funcoes import _sigla_descricao

    _name = 'documento.documento.tipo'
    _columns = {
        'descricao': fields.char('Descricao', size=30), # Descricao do tipo de documento
        'name': fields.function(_sigla_descricao, method=True, string='Name', type='text', store=True),
        # Descricao do tipo de documento
        'bloqueio': fields.boolean('Bloqueio'),
        # True=Bloqueio, nao pode ser excluido. False=Nao bloqueia, pode ser excluido\nEste campo nao deve aparecer na tela de manutencao e seu valor default deve ser False\n
        'sigla': fields.char('Sigla', size=3, select=True), # Sigla para identificaco do documento
        'liquidavel': fields.boolean('Liquidavel'), # Informa se o documento e liquidavel ou nao
    }

    _defaults = {
        'bloqueio': lambda *a: False,
        'liquidavel': lambda *a: True,
    }
    _sql_constraints = [
        ('sigla_uniq', 'unique (sigla)', 'A sigla precisa ser unica!'),
    ]
