# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from datetime import date
from openerp.addons.adm import print_log
from openerp.tools.translate import _
from openerp import pooler
from datetime import datetime, timedelta
import re
import threading
from SigUtils import datas


def synchronized_method(method):
    """
    Responsavel por sincronizar os metodos, de forma
    que entre apenas uma requisicao por vez em determindado metodo.
    E utilizado para reduzir problemas de concorrencia.

    Para utilizacao e necessario decorar o metodo que deseja sincronizar.
    Antes de utilizar avaliar se metodos que chamam o metodo a ser sincronizado nao
    fazem alteracao em registros de tabelas que o metodo sincronizado vai ler ou alterar, pois
    isso podera causar deadlock

    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    outer_lock = threading.Lock()
    lock_name = "__" + method.__name__ + "_lock" + "__"

    def sync_method(self, *args, **kwargs):
        with outer_lock:
            if not hasattr(self, lock_name) or not getattr(self, lock_name).locked():
                setattr(self, lock_name, threading.Lock())
                lock = getattr(self, lock_name)
                with lock:
                    return method(self, *args, **kwargs)
    return sync_method


class ecf_cupons_capa(osv.Model):
    _name = 'ecf.cupons.capa'
    _inherit = 'ecf.cupons.capa'

    def _update_store(self, cr, obj, campo):
        """
        Este metodo computa os campos funcionais storiados novos ou que tiveram sua
        estrutura alterada. Caso nao queira que o novo campo funcioal seja atualizado
        ao levantar o modulo, e necessario adicionar o atributo funcional na
        lista _fields_no_store
        :param cr: a linha atual do cursor do banco de dados
        :param obj: Objeto da classe functions
        :param campo: Campo funcional a ser computado
        :type cr: object
        :type obj: object
        :type campo: str
        """
        if campo not in self._fields_no_store:
            super(ecf_cupons_capa, self)._update_store(cr, obj, campo)

    def copy(self, cr, uid, id, default={}, context=None):
        context = context and context.copy() or {}
        if isinstance(default, dict):
            default.update({'nf_id': False})
        return super(ecf_cupons_capa, self).copy(cr, uid, id, default, context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        if all([context.get('selecao_tipo_cupom'), context.get('selecao_state')]):
            args.extend([('tipo_cupom', '=', 'V'), ('state', '=', 'V')])
        res = super(ecf_cupons_capa, self).search(cr, uid, args, offset, limit, order, context,
                                                  count)
        return res

    def button_gerar_nf(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return self.pool.get('fiscal.nf.capa').gerar_nf_cupom(cr, uid, ids, context=context)

    def get_financeiro_parametro(self, cr, uid, company_id, identificador=None, context=None):
        context = context and context.copy() or {}
        obj_financeiro_parametro = self.pool.get('financeiro.parametros')
        financeiro_parametro = obj_financeiro_parametro.search(cr, uid,
                                                 [('company_id', '=', company_id)], context=context)
        if not financeiro_parametro:
            pooler.get_pool(cr.dbname).get('ecf.importar.cupom.osv').cria_log(
                cr, uid, None, None, None, None, _('Parametro nao econtrado no Financeiro para a '
                                                   'empresa.'), context=context)
        else:
            financ_parametro_dados = obj_financeiro_parametro.read(
                cr, uid, financeiro_parametro[0], ['moeda_id', 'local_cobranca_id'],
                context=context)
            if isinstance(financ_parametro_dados, list):
                financ_parametro_dados = financ_parametro_dados[0]
            if isinstance(financ_parametro_dados, dict):
                try:
                    financ_parametro_dados['local_cobranca_id'] = financ_parametro_dados['local_cobranca_id'][0]
                except:
                    financ_parametro_dados['local_cobranca_id'] = []
                    pooler.get_pool(cr.dbname).get('ecf.importar.cupom.osv').cria_log(
                        cr, uid, None, None, None, None, _('Excecao: Nao foi possivel acessar o '
                                                           'campo local de cobranca no parametro '
                                                           'do Financeiro.'), context=context)

                # Os identificadores ´CF´ e ´CV´ utilizam a funcao. Somente o identificador ´CV´
                # necessita de moeda.
                if identificador == 'CV':
                    try:
                        financ_parametro_dados['moeda_id'] = financ_parametro_dados['moeda_id'][0]
                    except:
                        financ_parametro_dados['moeda_id'] = []
                        pooler.get_pool(cr.dbname).get('ecf.importar.cupom.osv').cria_log(
                            cr, uid, None, None, None, None, _('Excecao: Nao foi possivel acessar '
                                                               'o campo moeda no parametro do '
                                                               'Financeiro.'), context=context)
                return financ_parametro_dados
        return {}

    def get_cupom_dados(self, cr, uid, cupom_id, context=None):
        context = context and context.copy() or {}
        args_dados = ['cliente_id', 'total_cupom', 'date_cupom',
                      'ecf_id', 'tipo_cupom', 'cupom_financeiro_fatura_ids']
        dados_cupom = self.read(cr, uid, cupom_id, args_dados, context=context)
        if isinstance(dados_cupom, list):
            dados_cupom = dados_cupom[0]
        if isinstance(dados_cupom, dict):
            if dados_cupom.get('cliente_id'):
                try:
                    dados_cupom['cliente_id'] = dados_cupom['cliente_id'][0]
                except:
                    dados_cupom['cliente_id'] = []
            if dados_cupom.get('ecf_id'):
                try:
                    dados_cupom['ecf_id'] = dados_cupom['ecf_id'][0]
                except:
                    pooler.get_pool(cr.dbname).get('ecf.importar.cupom.osv').cria_log(
                        cr, uid, None, None, None, None,
                        _('Excecao: Nao foi possivel obter os dados da ECF cadastrada no cupom.'),
                        context=context)
                    dados_cupom['ecf_id'] = []
            return dados_cupom
        else:
            return {}

    def check_cupom(self, cr, uid, iLoja, iCaixa, iNroCupom, dData, tipo_registro, company_id,
                    arquivo=None, linha=None, context=None):
        context = context and context.copy() or {}

        sql = """
            SELECT
                id
            FROM ecf_ecf
            WHERE
                loja = %s
                AND caixa = %s
                AND company_id = %s
        """ % (iLoja, iCaixa, company_id)
        cr.execute(sql)
        ecf_dados = cr.dictfetchall()

        if not ecf_dados:
            pooler.get_pool(cr.dbname).get('ecf.importar.cupom.osv').cria_log(
                cr, uid, None, None, None, None,
                _("Nao foi encontrado nenhum ECF cadastrado no sistema para o "
                  "CAIXA: %s e LOJA: %s.") % (str(iCaixa), str(iLoja)), context=context)
        else:
            if isinstance(ecf_dados, list):
                ecf_dados = ecf_dados[0]
            sql = """
                SELECT
                    id
                FROM
                    ecf_cupons_capa
                WHERE
                    coo = %s AND date_cupom = '%s' AND ecf_id = %s
            """ % (iNroCupom, dData, ecf_dados.get('id'))
            cr.execute(sql)
            cupom_id = cr.dictfetchone()
            if not cupom_id:
                registro = ''
                if tipo_registro == 'CI':
                    registro = 'ITENS'
                elif tipo_registro == 'CF':
                    registro = 'FINALIZADORAS'
                elif tipo_registro == 'CV':
                    registro = 'VENCIMENTOS'
                elif tipo_registro == 'CZ':
                    registro = 'REDUCAO Z ou X'
                pooler.get_pool(cr.dbname).get('ecf.importar.cupom.osv').cria_log(
                    cr, uid, None, None, None, None,
                    _("Os campos de CAIXA: %d, LOJA: %d, ou DATA: %s - do registro de %s, "
                      "encontram-se diferentes de seu cupom de origem") % (iLoja, iCaixa,
                                                                           dData, registro),
                    context=context)
            else:
                return cupom_id.get('id')

        return False

    list_dados_log = []

    def ecfccp_armazena_dados_log(self, cr, uid, nrocupom, linha, coluna, col_fim, erro,
                                  cupom_id=None):
        dict_dados = {'nrocupom': nrocupom, 'linha': linha, 'coluna': coluna, 'col_fim': col_fim,
                      'erro': erro, 'cupom_id': cupom_id}
        self.list_dados_log.append(dict_dados)
        return self.list_dados_log

    def verifica_cupons(self, cr, uid, ids, context):
        cr.commit()
        for cupom_id in ids:
            self.verifica_cupom(cr, uid, cupom_id, context)

    @synchronized_method
    def verifica_cupom(self, cr, uid, ids, context):
        context = context and context.copy() or {}
        # este contexto nao deve ser retirado, para nao ocorrer o erro de pegar a empresa de onde
        # o admin logou por ultimo
        context.update({'sig_force_company': True})
        cupons_capa = self.browse(cr, uid, ids, context=context)
        if cupons_capa.state not in ('C', 'D', 'A'):
            return

        log = self.ecfccp_armazena_dados_log
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_estoque_movimento_capa = self.pool.get('estoque.movimento.capa')
        obj_mapa_resumo_capa = self.pool.get('ecf.mapa.resumo.capa')
        obj_mapa_resumo_itens = self.pool.get('ecf.mapa.resumo.itens')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        obj_estoque_embalagem = self.pool.get('estoque.embalagem')
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_cupons_xz = self.pool.get('ecf.cupons.xz')
        obj_cupons_totalizadores_xz = self.pool.get('ecf.cupons.xztotalizadores')
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        obj_ecf_cupons_finalizacao = self.pool.get('ecf.cupons.finalizacao')
        obj_adm_condicao = self.pool.get('adm.condicao')
        obj_adm_forma_pgto = self.pool.get('adm.forma.pgto')
        obj_adm_condicao_forma = self.pool.get('adm.condicao.forma')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        condicoes_ativas_ids = obj_adm_condicao.search(cr, uid, [('active', '=', True)],
                                                       context=context)
        forma_ativas_ids = obj_adm_forma_pgto.search(cr, uid, [('active', '=', True)],
                                                     context=context)
        self.list_dados_log = []
        movimento_itens_ids = [] # ids do movimento de itens para gerar custificacao
        erro = False
        if not cupons_capa.valor_alteracao or not float(cupons_capa.valor_alteracao):
            cupons_capa.valor_alteracao = 0
        print_log(_('- Verificando cupom %s...') % cupons_capa.coo)
        funcionario_id = False

        # PARA CUPONS DE CANCELAMENTO, VERIFICA SE O SEU CUPOM DE VENDA JA FOI IMPORTADO
        # SE O CUPOM DE VENDA NAO FOI IMPORTADO, O DE CANCELMAENTO FICA NO STATE
        # 'AGUARDANDO CUPOM DE VENDA'
        vendedor_id = cupons_capa.ecc_vendedor_id.id or False
        if cupons_capa.tipo_cupom == 'C':
            coo_cupom_venda = cupons_capa.coo - 1
            ecf_id = cupons_capa.ecf_id.id
            datas = "'%s', '%s'" % (datetime.strptime(cupons_capa.date_cupom, '%Y-%m-%d').date()
                                    - timedelta(days=1),
                                    cupons_capa.date_cupom)
            sql = '''
                    SELECT id
                    FROM ecf_cupons_capa
                    WHERE coo = {coo_cupom_venda}
                      AND date_cupom in ({data_cupom})
                      AND ecf_id = {ecf_id}
                      AND state = 'V'
                  '''.format(coo_cupom_venda=coo_cupom_venda,
                             data_cupom=datas,
                             ecf_id=ecf_id)
            cr.execute(sql)
            if not cr.dictfetchall():
                self.write(cr, uid, cupons_capa.id, {'state': 'A'}, context=context)
                cr.commit()
                return True

        if not cupons_capa.operador:
            codigo_operador = cupons_capa.ecfccp_operador_no_supervisor
            args = [('ecf_operador', '=', codigo_operador)]
            funcionario_ids = obj_rh_funcionario.search(cr, uid, args, context=context)
            if funcionario_ids:
                funcionario_id = funcionario_ids
                if isinstance(funcionario_ids, list):
                    funcionario_id = funcionario_ids[0]
            else:
                erro = True
                msg = _('\t Cupom divergente. Nao ha funcionario cadastrado com o '
                        '"Operador do ECF" de numero %s.') % codigo_operador
                log(cr, uid, cupons_capa.coo, 1, 1, 50, msg, cupons_capa.id)
        if cupons_capa.tipo_cupom == 'X' or cupons_capa.tipo_cupom == 'Z':  # Reducao X ou Z
            cupons_xz_id = obj_cupons_xz.search(
                cr, uid, [('cupom_id', '=', cupons_capa.id)], context=context)
            totalizadores_xz_id = obj_cupons_totalizadores_xz.search(
                cr, uid, [('cupom_id', '=', cupons_capa.id)], context=context)
            ecf_mapa_resumo_capa_id = None
            if not (cupons_xz_id and totalizadores_xz_id):
                erro = True
                msg = _('\t Cupom divergente. Nao existe registro de Reducao X ou Z para o cupom.')
                log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                if context.get('manutencao_cupom'):
                    raise osv.except_osv(_('Atencao!'), msg)
            elif cupons_capa.tipo_cupom == 'Z':
                if cupons_xz_id:
                    vals = {}
                    dados_ecf_cupons_xz = obj_cupons_xz.read(
                        cr, uid, cupons_xz_id, ['create_date', 'cupom_id', 'crz',
                                                'cupom_inicial', 'cupom_final', 'gt_inicial',
                                                'gt_final', 'cancelamentos', 'descontos',
                                                'acrescimos', 'cro'], context=context)
                    if isinstance(dados_ecf_cupons_xz, list):
                        dados_ecf_cupons_xz = dados_ecf_cupons_xz[0]
                    cupom_id = dados_ecf_cupons_xz.get('cupom_id')
                    if isinstance(cupom_id, tuple):
                        cupom_id = cupom_id[0]
                    dados_ecf_cupons_capa = self.read(cr, uid, cupom_id, ['ecf_id', 'coo'],
                                                      context=context)
                    ecf_id = dados_ecf_cupons_capa.get('ecf_id')
                    if isinstance(ecf_id, tuple):
                        ecf_id = ecf_id[0]
                    dados_ecf_ecf = self.pool.get('ecf.ecf').read(
                        cr, uid, [ecf_id], ['caixa'], context=context)
                    if isinstance(dados_ecf_ecf, list):
                        dados_ecf_ecf = dados_ecf_ecf[0]
                    caixa = dados_ecf_ecf.get('caixa')
                    if isinstance(dados_ecf_cupons_capa, list):
                        dados_ecf_cupons_capa = dados_ecf_cupons_capa[0]

                    vals = {
                        'data': cupons_capa.date_cupom,
                        'ecf': ecf_id,
                        'numero_z': dados_ecf_cupons_xz.get('crz'),
                        'cupom_inicial': dados_ecf_cupons_xz.get('cupom_inicial'),
                        'cupom_final': dados_ecf_cupons_xz.get('cupom_final'),
                        'gt_inicial': dados_ecf_cupons_xz.get('gt_inicial'),
                        'gt_final': dados_ecf_cupons_xz.get('gt_final'),
                        'cancelamentos': dados_ecf_cupons_xz.get('cancelamentos'),
                        'descontos': dados_ecf_cupons_xz.get('descontos'),
                        'acrescimos': dados_ecf_cupons_xz.get('acrescimos'),
                        'coo': dados_ecf_cupons_capa.get('coo'),
                        'cro': dados_ecf_cupons_xz.get('cro'),
                        'ecfmrc_cupom_id': cupons_capa.id,
                    }

                    mapa_resumo_existe = obj_mapa_resumo_capa.search(
                        cr, uid, [('data', '=', cupons_capa.date_cupom),
                                  ('ecf', '=', ecf_id)], context=context)
                    if not mapa_resumo_existe:
                        context['importacao_validacao_cupom'] = True
                        try:
                            ecf_mapa_resumo_capa_id = obj_mapa_resumo_capa.create(
                                cr, uid, vals, context)
                        except Exception as e:
                            erro = True
                            msg = _(
                                '\t Cupom divergente. Data/Ecf devem ser unicos p/ o mapa resumo.')
                            log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                            if context.get('manutencao_cupom'):
                                raise osv.except_osv(_('Atencao!'), msg)

                    else:
                        erro = True
                        msg = _('\t Cupom divergente. Data/Ecf devem ser unicos p/ o mapa resumo.')
                        log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                        if context.get('manutencao_cupom'):
                            raise osv.except_osv(_('Atencao!'), msg)

                if totalizadores_xz_id and ecf_mapa_resumo_capa_id:
                    vals = {}
                    dados_ecf_cupons_xztotalizadores = self.pool.get(
                        'ecf.cupons.xztotalizadores').read(
                        cr, uid, totalizadores_xz_id, ['tipo_totalizador', 'imposto',
                                                       'aliquota', 'base_imposto',
                                                       'valor_imposto',
                                                       'base_reduzida_concede_desconto'],
                        context=context)
                    for totalizador in dados_ecf_cupons_xztotalizadores:
                        tipo_totalizador = totalizador.get('tipo_totalizador')
                        if tipo_totalizador == 'U':
                            tipo_totalizador = 'F'
                        if not tipo_totalizador:
                            descricao_totalizador = totalizador.get('imposto') or ''
                            if descricao_totalizador == 'C':
                                descricao_totalizador = 'ICMS'
                            elif descricao_totalizador == 'S':
                                descricao_totalizador = 'ISS'
                            erro = True
                            msg = ('\t Cupom divergente. Tipo de totalizador nao informado para '
                                   'o imposto %s %s.') % (descricao_totalizador,
                                                         totalizador.get('aliquota'))
                            log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                            continue
                        vals = {
                            'capa_id': ecf_mapa_resumo_capa_id,
                            'tipo': tipo_totalizador,
                            'imposto': totalizador.get('imposto') or '0',
                            'aliquota': totalizador.get('aliquota'),
                            'base_valor': totalizador.get('base_imposto'),
                            'imposto_valor': totalizador.get('valor_imposto'),
                            'base_reduzida': totalizador.get('base_reduzida_concede_desconto')
                        }
                        obj_mapa_resumo_itens.create(cr, uid, vals, context)
        # SUPRIMENTO
        elif cupons_capa.tipo_cupom == 'U':
            if cupons_capa.cupons_finalizacao_ids:
                total_finalizacao = sum(
                    filter(None, map(lambda x: x.valor, cupons_capa.cupons_finalizacao_ids)))

                if round(cupons_capa.total_cupom, 2) != round(total_finalizacao, 2):
                    erro = True
                    msg = _('\t Cupom divergente. O valor registrado na finalizadora '
                            'difere do valor total do cupom. Total: R$%0.2f, Finalizadora: '
                            'R$%0.2f.') % (cupons_capa.total_cupom, total_finalizacao)
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    if context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'), msg)
            else:
                erro = True
                msg = _('\t Cupom divergente. Nao existe registro de Finalizadora para o cupom.')
                log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                if context.get('manutencao_cupom'):
                    raise osv.except_osv(_('Atencao!'), msg)
        # Venda ou Cancelamento
        elif cupons_capa.tipo_cupom == 'V' or cupons_capa.tipo_cupom == 'C':
            total_itens = 0  # total bruto dos itens
            desconto_total_itens = 0 # desconto dos itens mais o desconto rateado da capa
            acrescimo_total_itens = 0 # acrescimo dos itens mais o acrescimo rateado da capa
            mov_capa_id = False
            if not cupons_capa.ecf_cupons_item_ids:
                erro = True
                msg = _('\t Cupom divergente. Nao existe registro de itens para o cupom.')
                log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                if context.get('manutencao_cupom'):
                    raise osv.except_osv(_('Atencao!'), msg)
            else:
                total_cupom = cupons_capa.total_cupom # total liquido do cupom
                # filtra os itens do tipo (V) se o cupom for de venda,
                # ou itens do tipo (C) se o cupom for de cancelamento
                itens_vendidos = filter(lambda i: i.venda_cancelamento == 'V'
                                                  if cupons_capa.tipo_cupom == 'V' else i.venda_cancelamento == 'C',
                                                  cupons_capa.ecf_cupons_item_ids or [])

                cupom_company_id = cupons_capa.company_id.id
                acb_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid,
                                                                   tabela='automacao.codigo.barra',
                                                                   context=context,
                                                                   user_company_id=cupom_company_id)
                # Verifica se os itens possue todos os encargos obrigatorios
                itens_erro = []
                erro_tributacao = []
                context_aut = context.copy()
                context_aut['active_test'] = False # preciso de todos ids da automacao
                for item_vendido in itens_vendidos:
                    automacao_ids = self.pool.get('automacao.codigo.barra').search(
                        cr, uid, [('codigo_barra', '=', item_vendido.item),
                                  ('company_id', '=', acb_company_id)], context=context_aut)
                    if not automacao_ids:
                        erro = True
                        msg = (('\t Cupom divergente. O item "%s" nao esta cadastrado em '
                               'Estoque->Automacao->Codigo de barras!') % item_vendido.item)
                        log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                        continue

                    brw_produto = automacao_ids and self.pool.get('automacao.codigo.barra').browse(
                        cr, uid, automacao_ids[0], context).produto_id
                    if not self.pool.get('estoque.produto').estpro_verificar_encargo_obrigatorio(
                            cr, uid, brw_produto.id, context):
                        itens_erro.append(item_vendido.item)

                    tem_icms_saida = False
                    if brw_produto.tributacao_ids:
                        for i in brw_produto.tributacao_ids:
                            if i.tipo_tributacao == 'T' and i.encargo_id.sigla.upper() == 'ICMS':
                                tem_icms_saida = True
                                break

                    if not tem_icms_saida:
                        erro_tributacao.append(item_vendido.item)


                if erro_tributacao:
                    erro = True
                    msg = _('\t Cupom divergente. Os itens a seguir nao possuem Tributacao de Saida'
                            ' para ICMS:\n') + '\n'.join(erro_tributacao)
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    if context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'), msg)

                if itens_erro:
                    erro = True
                    msg = _('\t Cupom divergente. Os itens a seguir nao possuem todos os encargos '
                            'obrigatorios: \n') + '\n'.join(itens_erro)
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    if context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'), msg)

                msg_erros = []
                for item_vendido in itens_vendidos:
                    total_itens += item_vendido.preco_total
                    desconto_total_itens += item_vendido.desconto
                    acrescimo_total_itens += item_vendido.acrescimo
                    if item_vendido.preco_total == item_vendido.desconto:
                        msg_erros.append(item_vendido.item)

                if msg_erros:
                    erro = True
                    msg = ('\t Cupom divergente. Os itens a seguir possuem desconto igual ao '
                           'preco total: ') + ','.join(msg_erros)
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    if context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'), msg)

                total_itens += acrescimo_total_itens - desconto_total_itens
                if round(total_cupom, 2) != round(total_itens, 2) and cupons_capa.tipo_cupom in ('V', 'C'):
                    erro = True
                    msg = _('\t Cupom divergente. O preco total da venda/cancelamento difere do '
                            'valor total do cupom. Total liquido do cupom: R$%0.2f Total liquido '
                            'dos Itens: R$%0.2f.') % (total_cupom, total_itens)
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    if context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'), msg)
                if not cupons_capa.cupons_finalizacao_ids:
                    erro = True
                    msg = _('\t Cupom divergente. Nao existe registro de Finalizadora para o '
                            'cupom.')
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    if context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'), msg)
                else:
                    total_finalizacao = 0
                    for cupons_finalizacao_id in cupons_capa.cupons_finalizacao_ids:
                        # finalizadora padrao
                        if cupons_finalizacao_id.finalizadora_id.finalizadora_na_ecf == 9999:
                            msg = _('\t Cupom divergente. Finalizadora padrao, cadastre a '
                                    'finalizadora em que a venda foi realizada.')
                            log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                            if context.get('manutencao_cupom'):
                                raise osv.except_osv(_('Atencao!'), msg)

                        valor = cupons_finalizacao_id.valor or 0.0
                        troco = cupons_finalizacao_id.troco or 0.0
                        total_finalizacao += valor - troco
                        if valor <= troco:
                            erro = True
                            msg = _('\t Cupom divergente. O valor registrado na finalizadora '
                                    'deve ser maior que o valor do troco. Valor: R$%0.2f, '
                                    'Troco: R$%0.2f.') % (valor, troco)
                            log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                            if context.get('manutencao_cupom'):
                                raise osv.except_osv(_('Atencao!'), msg)

                        finalizadora_na_ecf = cupons_finalizacao_id.ecffin_finalizadora_na_ecf
                        args_condicao = [('adm_condicao_id', 'in', condicoes_ativas_ids),
                                         ('forma_pagamento_id', 'in', forma_ativas_ids)]
                        if not finalizadora_na_ecf and cupons_finalizacao_id.finalizadora_id:
                            finalizadora_na_ecf = cupons_finalizacao_id.finalizadora_id.finalizadora_na_ecf

                        if finalizadora_na_ecf:
                            args_condicao.append(('finalizadora_na_ecf', '=', finalizadora_na_ecf))

                        finalizadora_ids = obj_adm_condicao_forma.search(
                            cr, uid, args_condicao, context=context)
                        if not finalizadora_ids:
                            erro = True
                            msg = _('\t Cupom divergente. A finalizadora para o "Codigo na ECF": '
                                    '%s nao esta cadastrada.') % finalizadora_na_ecf
                            log(cr, uid, cupons_capa.coo, None, None, None, msg,
                                cupons_capa.id)
                        else:
                            obj_ecf_cupons_finalizacao.write(
                                cr, uid, [cupons_finalizacao_id.id],
                                {'finalizadora_id': finalizadora_ids[0]}, context)
                            brw_finalizadora = obj_adm_condicao_forma.browse(
                                    cr, uid, finalizadora_ids[0], context=context)
                            cupons_finalizacao_id.finalizadora_id = brw_finalizadora

                        condicao_tipo = False
                        if cupons_finalizacao_id.finalizadora_id.adm_condicao_id:
                            condicao_tipo = cupons_finalizacao_id.finalizadora_id.\
                                adm_condicao_id.tipo
                        elif finalizadora_ids:
                            condicao_tipo = obj_adm_condicao_forma.browse(
                                cr, uid, finalizadora_ids[0]).adm_condicao_id.tipo
                        if condicao_tipo == 'C':
                            vencimentos = cupons_finalizacao_id.ecf_cupons_vencimento_ids
                            if vencimentos:
                                total_vencimentos = sum(filter(None, map(lambda x: x.valor, vencimentos)))
                                if round(total_vencimentos, 2) != round(valor, 2):
                                    erro = True
                                    msg = _('\t Cupom divergente. O valor total das parcelas da '
                                            'finalizadora difere do valor total dos vencimentos. '
                                            'Total: R$ %0.2f, Vencimentos: R$ %0.2f') % \
                                          (valor, total_vencimentos)
                                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                                    if context.get('manutencao_cupom'):
                                        raise osv.except_osv(_('Atencao!'), msg)
                                elif not vencimentos and cupons_capa.tipo_cupom != 'C':
                                    erro = True
                                    msg = _('\t Cupom divergente. Finalizadora a prazo nao possui '
                                            'vencimentos.')
                                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                                    if context.get('manutencao_cupom'):
                                        raise osv.except_osv(_('Atencao!'), msg)
                            else:
                                erro = True
                                msg = _('\t Cupom divergente. Finalizadora a prazo nao possui '
                                        'vencimentos.')
                                log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                                if context.get('manutencao_cupom'):
                                    raise osv.except_osv(_('Atencao!'), msg)

                    if round(cupons_capa.total_cupom, 2) != round(total_finalizacao, 2):
                        erro = True
                        msg = _('\t Cupom divergente. O valor registrado na finalizadora difere '
                                'do valor total do cupom. Total: R$%0.2f. Finalizadora: '
                                'R$%0.2f".') % (cupons_capa.total_cupom, total_finalizacao)
                        log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                        if context.get('manutencao_cupom'):
                            raise osv.except_osv(_('Atencao!'), msg)

                if not erro:
                    est_par = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque.parametro',
                                    ['estpar_ecf_operacao_importacao_id'],
                                              company_id=cupom_company_id, context=context)
                    ecm_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid,
                                                                   tabela='estoque.movimento.capa',
                                                  context=context, user_company_id=cupom_company_id)

                    epc_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid,
                                                                       tabela='estoque.preco.custo',
                                                   context=context,user_company_id=cupom_company_id)
                    operacao_id = est_par.get('estpar_ecf_operacao_importacao_id')
                    if not operacao_id:
                        raise osv.except_osv(_('Atencao!'),
                                             _('Cadastre a "Operacao para '
                                               'importacao" nos parametros '
                                               'do Estoque.'))

                    local_id = cupons_capa.ecf_id and cupons_capa.ecf_id.local_id.id or False
                    if not local_id:
                        erro = True
                        msg = _('\tCupom divergente. Local nao cadastrado para a ecf '
                                'Caixa: {c} Loja: {l}. Este LOCAL recebera a baixa de estoque da '
                                'importacao dos cupons.'). \
                                format(c=cupons_capa.ecf_id.caixa, l=cupons_capa.ecf_id.loja)
                        log(cr, uid, cupons_capa.coo, None, None, None, msg,
                            cupons_capa.id)

                        if context.get('manutencao_cupom'):
                            raise osv.except_osv(_('Atencao!'), msg)

                    cbs = map(lambda x: x.item.strip(), cupons_capa.ecf_cupons_item_ids)
                    obj_codigo_barra = self.pool.get('automacao.codigo.barra')
                    cb_ids = obj_codigo_barra.search(cr, uid,
                                                     [('codigo_barra', 'in', cbs),
                                                      ('company_id','=',acb_company_id)],
                                                     context=context_aut)
                    dados_cb = obj_codigo_barra.read(cr, uid, cb_ids, ['grade_id', 'produto_id',
                                                     'produto_embalagem_id', 'codigo_barra'],
                                                     context=context)
                    dict_cb_dados = {}
                    grade_embalagem = {}
                    embalagem_ids = []
                    for i in dados_cb:
                        grade_id = i.get('grade_id')[0]
                        emb_id = i.get('produto_embalagem_id')[0]
                        embalagem_ids.append(emb_id)
                        if not grade_embalagem.get(emb_id):
                            grade_embalagem[emb_id] = [grade_id]
                        else:
                            grade_embalagem[emb_id].append(grade_id)

                        dict_cb_dados[i.get('codigo_barra')] = i

                    qtd_embalagens = {}
                    for i in obj_produto_embalagem.browse(cr, uid, embalagem_ids, context=context):
                        for j in grade_embalagem.get(i.id):
                            if qtd_embalagens.get(j):
                                qtd_embalagens[j].update(
                                    {i.id: i.embalagem_id.quantidade_na_embalagem or 1})
                            else:
                                qtd_embalagens[j] = {i.id: i.embalagem_id.quantidade_na_embalagem or 1}

                    for cupons_item_id in cupons_capa.ecf_cupons_item_ids:
                        vals_movimento = {}
                        cod_barra = cupons_item_id.item
                        sequencia = cupons_item_id.sequencia
                        # a empresa do codigo de barras deve ser olhada na correlacao por isso o
                        # uso da funcao
                        if not cupom_company_id:
                            ecf_name = self.browse(cr, uid, cupons_capa.id).ecf_id.nro_serie
                            erro = True
                            msg = _('\tCupom divergente. Nao foi encontrada nenhuma empresa para '
                                    'o ECF cadastrado no sistema com o codigo %s.') % ecf_name
                            log(cr, uid, cupons_capa.coo, None, None, None, msg,
                                cupons_capa.id)
                            if context.get('manutencao_cupom'):
                                raise osv.except_osv(_('Atencao!'), msg)
                        else:
                            empresa_nome = self.pool.get('res.company').read(
                                cr, uid, acb_company_id, ['name'], context=context)
                            if isinstance(empresa_nome, list):
                                empresa_nome = empresa_nome[0]
                            if isinstance(empresa_nome, dict):
                                empresa_nome = empresa_nome.get('name') or ''
                            context.update({'active_test': False})
                            dados_codigo_barra = dict_cb_dados.get((cod_barra.strip()).zfill(14))
                            if not dados_codigo_barra:
                                erro = True
                                msg = _('\tCupom divergente. Nao foi encontrado nenhum CODIGO DE '
                                        'BARRA cadastrado no sistema com o codigo: %s para '
                                        'a empresa %s.') % (cod_barra, empresa_nome)
                                log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                                if context.get('manutencao_cupom'):
                                    raise osv.except_osv(_('Atencao!'), msg)

                            if dados_codigo_barra and cupons_capa.id:
                                if isinstance(dados_codigo_barra, list):
                                    dados_codigo_barra = dados_codigo_barra[0]
                                if isinstance(dados_codigo_barra, dict):
                                    try:
                                        produto_id = dados_codigo_barra['produto_id'][0]
                                        vals_movimento['produto_id'] = produto_id
                                    except:
                                        erro = True
                                        log(cr, uid, cupons_capa.coo, None, None, None,
                                            _('Registro CI: Excecao - Na importacao dos itens, '
                                              'nao foi possivel acessar o produto'), cupons_capa.id)
                                    try:
                                        produto_nome = dados_codigo_barra['produto_id'][1]
                                    except:
                                        produto_nome = ''
                                    obj_produto_grade = self.pool.get('estoque.produto.grade')
                                    grade_id = dados_codigo_barra.get('grade_id')
                                    if grade_id and isinstance(grade_id, tuple):
                                        grade_id = grade_id[0]
                                    if not grade_id:
                                        erro = True
                                        msg = _('\t Cupom divergente. O produto %s nao pode ser '
                                                'importado porque nao possui grade cadastrada.' ) \
                                              % produto_nome
                                        log(cr, uid, cupons_capa.coo, None, None, None, msg,
                                            cupons_capa.id)
                                        if context.get('manutencao_cupom'):
                                            raise osv.except_osv(_('Atencao!'), msg)
                                    else:
                                        vals_movimento['grade_id'] = grade_id
                                        grade_codigo_barra = obj_produto_grade.read(
                                            cr, uid, vals_movimento['grade_id'],
                                            ['codigo_barra_ids'], context=context)
                                        grade_codigo_barra = obj_produto_grade.read(
                                            cr, uid, vals_movimento['grade_id'],
                                            ['codigo_barra_ids'], context=context)
                                        if isinstance(grade_codigo_barra, list):
                                            grade_codigo_barra = grade_codigo_barra[0]
                                        if isinstance(grade_codigo_barra, dict):
                                            codigo_barra_ids = obj_codigo_barra.search(
                                                cr, uid,
                                                [('codigo_barra', '=', cod_barra.strip()),
                                                 ('company_id', '=', acb_company_id),
                                                 ('produto_id', '=', vals_movimento['produto_id'])],
                                                context=context_aut)
                                            if not codigo_barra_ids:
                                                raise osv.except_osv(
                                                    _('Atencao!'),
                                                    _('No cadastro da grade do '
                                                      'Produto: %s, nao foi encontrado nenhum '
                                                      'codigo de barra: %s') % (produto_nome,
                                                                                cod_barra))
                                            elif len(codigo_barra_ids) > 1:
                                                raise osv.except_osv(_('Atencao!'), _(
                                                    'No cadastro da grade do Produto: %s, o '
                                                    'codigo de barra: %s encontra-se cadastrado'
                                                    ' mais de uma vez!') % (produto_nome,
                                                                            cod_barra))
                                            codigo_barra = obj_codigo_barra.read(
                                                cr, uid, codigo_barra_ids[0],
                                                ['produto_embalagem_id'], context=context)
                                            if isinstance(object, list):
                                                codigo_barra = codigo_barra[0]
                                            if isinstance(codigo_barra, dict):
                                                try:
                                                    vals_movimento['produto_embalagem_id'] = \
                                                    codigo_barra['produto_embalagem_id'][0]
                                                except:
                                                    vals_movimento['produto_embalagem_id'] = []
                                            if vals_movimento['produto_embalagem_id']:
                                                produto_embalagem = obj_produto_embalagem.read(
                                                    cr, uid,
                                                    vals_movimento['produto_embalagem_id'],
                                                    ['embalagem_id'], context=context)
                                                embalagem_id = []
                                                if isinstance(produto_embalagem, list):
                                                    produto_embalagem = produto_embalagem[0]
                                                if isinstance(produto_embalagem, dict):
                                                    try:
                                                        embalagem_id = produto_embalagem['embalagem_id'][0]
                                                    except:
                                                        embalagem_id = []
                                                        erro = True
                                                        log(cr, uid, cupons_capa.coo,
                                                            None, None, None,
                                                            _('Registro CI: Excecao - '
                                                              'Na importacao dos itens, nao foi '
                                                              'possivel acessar a embalagem '
                                                              'utilizada para o produto.'),
                                                            cupons_capa.id)
                                                if embalagem_id:
                                                    embalagem_dados = obj_estoque_embalagem.read(
                                                        cr, uid, embalagem_id,
                                                        ['quantidade_na_embalagem'], context=context)
                                                    if isinstance(embalagem_dados, list):
                                                        embalagem_dados = embalagem_dados[0]

                                    if est_par:
                                        mov_capa_id = obj_estoque_movimento_capa.search(cr, uid, [
                                            ('company_id', '=', ecm_company_id),
                                            ('data', '=', cupons_capa.date_cupom),
                                            ('funcionario_id', '=', cupons_capa.operador.id),
                                            ('cupons_capa_id', '=', cupons_capa.id)
                                        ], context=context)
                                        if not mov_capa_id:
                                            mov_capa_id = obj_estoque_movimento_capa.create(cr, uid, {
                                                'company_id': ecm_company_id,
                                                'data': cupons_capa.date_cupom,
                                                'estmvc_operacao_id': operacao_id,
                                                'funcionario_id': cupons_capa.operador.id,
                                                'cupons_capa_id': cupons_capa.id,
                                                'state': 'F'}, context)

                                        if isinstance(mov_capa_id, list):
                                            mov_capa_id = mov_capa_id[0]

                                        vals_movimento['capa_id'] = mov_capa_id

                                        tributacao_id = obj_fiscal_tributacao.search(
                                            cr, uid, [('tributacao_ecf', '=', cupons_item_id.tributacao_retag)],
                                            context=context)

                                        produto_id = vals_movimento.get('produto_id')
                                        grade_id = vals_movimento.get('grade_id')
                                        movimento_itens_id = False
                                        if not tributacao_id:
                                            erro = True
                                            log(cr, uid, cupons_item_id.capa_id.coo,
                                                None, None, None,
                                                _("A TRIBUTACAO RETAGUARDA %s nao esta "
                                                  "cadastrada.") % cupons_item_id.tributacao_retag,
                                                cupons_capa.id)
                                        else:
                                            # Cria movimentação se não houver movimentação p/ o item
                                            if isinstance(tributacao_id, list):
                                                tributacao_id = tributacao_id[0]
                                            args_mov_item = [('cupom_id', '=', cupons_capa.id),
                                                             ('grade_id', '=', grade_id),
                                                             ('produto_id', '=', produto_id),
                                                             ('produto_embalagem_id', '=',
                                                              vals_movimento.get('produto_embalagem_id')),
                                                             ('item_cancelado', '=',
                                                             (cupons_item_id.venda_cancelamento == 'C'))]
                                            if cupons_item_id.ecfcit_cod_vendedor or vendedor_id:
                                                args_mov_item.append(
                                                    ('vendedor_id', '=',
                                                     cupons_item_id.ecfcit_cod_vendedor or vendedor_id))
                                            mov_item_id = obj_estoque_movimento_itens.search(
                                                cr, uid, args_mov_item, context=context)
                                            qtd_embalagem = qtd_embalagens.get(grade_id).get(
                                                vals_movimento.get('produto_embalagem_id')) or 1
                                            if mov_item_id and isinstance(mov_item_id, list):
                                                movimento_itens_id = mov_item_id[0]
                                                brw_cupom_itens = cupons_item_id
                                                brw_mov_itens = obj_estoque_movimento_itens.browse(cr, uid,
                                                                                                   movimento_itens_id)
                                                custos = self.pool.get('estoque.preco.custo').get_custos_do_dia(
                                                    cr, uid, vals_movimento.get('produto_id'), epc_company_id,
                                                    cupons_item_id.capa_id.date_cupom, context)
                                                custos = custos.get(vals_movimento.get('produto_id'))
                                                quantidade = cupons_item_id.quantidade
                                                vr_total_item_rateado = cupons_item_id.preco_total + cupons_item_id.acrescimo - cupons_item_id.desconto
                                                vr_total_item_rateado += brw_mov_itens.vr_total_item_rateado
                                                custo_corrigido = ultimo_custo = custo_reposicao = custo_medio_contabil = 0
                                                if custos:
                                                    custo_corrigido = (custos.get(
                                                        'estprc_custo_corrigido') or 0) * quantidade * qtd_embalagem + brw_mov_itens.estmvi_saidas_custo_corrigido_total
                                                    ultimo_custo = (custos.get(
                                                        'estprc_ultimo_custo') or 0) * quantidade * qtd_embalagem + brw_mov_itens.estmvi_saidas_ultimo_custo_total
                                                    custo_reposicao = (custos.get(
                                                        'estprc_custo_reposicao') or 0) * quantidade * qtd_embalagem + brw_mov_itens.estmvi_saidas_custo_reposicao_total
                                                    custo_medio_contabil = (custos.get(
                                                        'estprc_custo_medio_contabil') or 0) * quantidade * qtd_embalagem + brw_mov_itens.estmvi_saidas_custo_medio_contabil_total

                                                vr_acrescimos = brw_mov_itens.vr_acrescimos + brw_cupom_itens.acrescimo
                                                vr_desconto = brw_mov_itens.vr_desconto + brw_cupom_itens.desconto
                                                vr_despesas_rateado = brw_mov_itens.vr_despesas_rateado
                                                vr_desconto_rateado = brw_mov_itens.vr_desconto_rateado

                                                vals_item = {'quantidade': brw_mov_itens.quantidade + quantidade,
                                                             'vr_total_item': brw_mov_itens.vr_total_item + cupons_item_id.preco_total,
                                                             'estmvi_saidas_custo_corrigido_total': custo_corrigido,
                                                             'estmvi_saidas_ultimo_custo_total': ultimo_custo,
                                                             'estmvi_saidas_custo_reposicao_total': custo_reposicao,
                                                             'estmvi_saidas_custo_medio_contabil_total': custo_medio_contabil,
                                                             'vr_total_item_rateado': vr_total_item_rateado,
                                                             'vr_acrescimos': vr_acrescimos,
                                                             'vr_desconto': vr_desconto,
                                                             'vr_despesas_rateado': vr_despesas_rateado,
                                                             'vr_desconto_rateado': vr_desconto_rateado
                                                             }
                                                cod_vendedor = cupons_item_id.ecfcit_cod_vendedor
                                                # calcula comissao caso haja vendedor
                                                if cod_vendedor:
                                                    vededor_ids = self.pool.get('vendas.vendedor').search(cr, uid,
                                                                                                          [('id','=',cod_vendedor)],
                                                                                                          context=context)
                                                    if (vededor_ids or vendedor_id) and cupons_item_id.venda_cancelamento != 'C':
                                                        comissao_dados = self.ecfccp_get_comissao(cr, uid, brw_mov_itens.produto_id,
                                                                                                  cupons_item_id.preco_total,
                                                                                                  cod_vendedor or vendedor_id)
                                                        vr_comissao = brw_mov_itens.estmvi_vr_comissao \
                                                                      + comissao_dados.get('estmvi_vr_comissao')
                                                        if comissao_dados:
                                                            vals_item.update({'estmvi_vr_comissao': vr_comissao})
                                                    if not vendedor_id and vededor_ids:
                                                        vendedor_id = cod_vendedor

                                                obj_estoque_movimento_itens.write(cr, uid, movimento_itens_id,
                                                                                  vals_item, context=context)
                                            else:
                                                self.item_cancelado = False
                                                vals_movimento['local_id'] = local_id
                                                movimento_itens_id, erro = self.cria_movimentacao_itens(
                                                    cr, uid, vals_movimento, cupons_item_id,
                                                    tributacao_id, qtd_embalagem, context)
                                                # nao coloca o id do item cancelado na lista para nao calcular CMV
                                                if not self.item_cancelado:
                                                    movimento_itens_ids.append(movimento_itens_id)

                                        condicoes_validas = [
                                            context.get('inconsistencia'),
                                            context.get('valida_cupom'),
                                            context.get('importacao_automatica')
                                        ]
                                        if tributacao_id and movimento_itens_id \
                                                and (cupons_capa.state in ('C', 'A') or any(
                                                    condicoes_validas)):

                                            # Grava triburação dos itens do cupom
                                            res_trib = self.atualiza_fiscal_item_tributacao(cr, uid,
                                                          produto_id, tributacao_id, cupons_item_id,
                                                          movimento_itens_id, context)
                                            if isinstance(res_trib, (str, unicode)) and res_trib.startswith('1'):
                                                erro = True
                                                log(cr, uid, cupons_capa.coo, None,
                                                    None, None,
                                                    _("O item %s possui o encargo da tributacao"
                                                      " diferente do encargo do cadastro."
                                                      ) % cod_barra, cupons_capa.id)
                                            elif isinstance(res_trib, (str, unicode)) and res_trib.startswith('2-'):
                                                erro = True
                                                log(cr, uid, cupons_capa.coo, None, None, None,
                                                    _("Item %s, %s") % (cod_barra,
                                                                        res_trib.split('-')[1]),
                                                    cupons_capa.id)

            if cupons_capa.tipo_cupom == 'V' and not erro and mov_capa_id:
                self.ecfccp_trata_expedicao_cupom_venda(cr, uid, cupons_capa, context)
                self.ecfccp_trata_fidelidade_cupom_venda(cr, uid, cupons_capa, mov_capa_id,
                                                         context)
            elif cupons_capa.tipo_cupom == 'C' and not erro and mov_capa_id:
                self.ecfccp_trata_expedicao_cupom_cancelamento(cr, uid, cupons_capa, context)
                self.ecfccp_trata_fidelidade_cupom_cancelamento(cr, uid, cupons_capa,
                                                                mov_capa_id, context)
        res = self.atualiza_cupom(cr, uid, cupons_capa, movimento_itens_ids, erro,
                                  funcionario_id, vendedor_id, context)
        return res

    def atualiza_cupom(self, cr, uid, cupons_capa, movimento_itens_ids, erro=False,
                       funcionario_id=False, vendedor_id=False, context=None):
        context = context and context.copy() or {}
        context['verificado'] = True
        res_estoque = False
        res_financeiro = False
        # Se o cupom nao contem erro, atualizar estoque e financeiro
        if not erro and cupons_capa.tipo_cupom in ('V', 'C'):
            res_estoque = self.atualiza_estoque_cupom(cr, uid, cupons_capa, context=context)
            if not res_estoque:
                erro = True
            else:
                res_financeiro = self.atualiza_financeiro_cupom(cr, uid, cupons_capa,
                                                                movimento_itens_ids, vendedor_id, context)
                if not res_financeiro:
                    erro = True

        if erro:
            vals = {'state': 'D'}
            # quando 'res_estoque' ou 'res_financeiro' for False as transacoes
            # anteriores devem ser desfeitas. Pois se não o cupom pode ficar divergente e haver movimentacao, etc...
            if not (res_estoque and res_financeiro): # ^ verificar a alteracao desta condicao caso precise modifica-la
                cr.rollback()
        else:
            vals = {'state': 'V'}
            if funcionario_id:
                vals['operador'] = funcionario_id

            if cupons_capa.ecfccp_numero_pedido:
                # apenas cupons de venda e cancelamento terao 'ecfccp_numero_pedido'
                self.ecfccp_fatura_pedido(cr, uid, cupons_capa, context=context)

        res = self.write(cr, uid, cupons_capa.id, vals, context)
        log_divergencia = self.pool.get('ecf.importar.cupom.osv').cria_log
        obj_ecf_cupom_log = self.pool.get('ecf.cupom.log')
        ecf_cupom_log_ids = obj_ecf_cupom_log.search(cr, uid,
                                     ['|',('cupom_id', '=', cupons_capa.id),
                                          ('cupom_id', '=', False)], context=context)
        obj_ecf_cupom_log.unlink(cr, uid, ecf_cupom_log_ids)
        for log in self.list_dados_log:
            nro_cupom = log.get('nrocupom')
            linha = log.get('linha')
            coluna = log.get('coluna')
            col_fim = log.get('col_fim')
            erro = log.get('erro')
            cupom_id = log.get('cupom_id')
            log_divergencia(cr, uid, nro_cupom, linha, coluna, col_fim, erro, cupom_id, context=context)
        if res:
            cr.commit()

        return res

    def cria_movimentacao_itens(self, cr, uid, vals_movimento, brw_cupons_item, tributacao_id,
                                qtd_embalagem, context=None):
        context = context and context.copy() or {}
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        obj_fiscal_cfop = self.pool.get('fiscal.cfop')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        log = self.ecfccp_armazena_dados_log

        if vals_movimento.get('produto_id'):
            produto_id = vals_movimento.get('produto_id')
            icms_substituicao = False
            brw_fiscal_tributacao = obj_fiscal_tributacao.browse(cr, uid, tributacao_id, context)
            encargo = brw_fiscal_tributacao.encargo_id.sigla
            tipo_cst = brw_fiscal_tributacao.cst_id.tipo

            if encargo == 'ICMS' and tipo_cst == 'U':
                icms_substituicao = True

            browse_produto = obj_estoque_produto.browse(cr, uid, produto_id)
            tipo_produto = browse_produto.tipo

            if tipo_produto == 'S':
                cfop_id = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', '5933')], context=context)
                if cfop_id:
                    vals_movimento['cfop_id'] = cfop_id[0]
            elif icms_substituicao:
                cfop_id = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', '5405')], context=context)
                if cfop_id:
                    vals_movimento['cfop_id'] = cfop_id[0]
            else:
                cfop_id = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', '5102')], context=context)
                if cfop_id:
                    vals_movimento['cfop_id'] = cfop_id[0]

        quantidade = brw_cupons_item.quantidade
        vr_total_item = brw_cupons_item.preco_total + brw_cupons_item.acrescimo - brw_cupons_item.desconto
        vals_movimento['quantidade'] = quantidade
        vals_movimento['sequencia'] = brw_cupons_item.sequencia
        vals_movimento['vr_unitario'] = brw_cupons_item.preco_total / quantidade
        vals_movimento['vr_preco_venda'] = brw_cupons_item.preco_total / quantidade
        vals_movimento['vr_total_item'] = vr_total_item
        vals_movimento['vr_acrescimos'] = brw_cupons_item.acrescimo
        vals_movimento['vr_desconto'] = brw_cupons_item.desconto
        vals_movimento['vr_total_item_rateado'] = vr_total_item
        vals_movimento['vr_despesas_rateado'] = 0  # o acrescimo da capa ja esta contemplando no do item
        vals_movimento['vr_desconto_rateado'] = 0  # o desconto da capa ja esta contemplando no do item
        vals_movimento['cupom_id'] = brw_cupons_item.capa_id.id
        vals_movimento['cupom_item_id'] = brw_cupons_item.id

        if brw_cupons_item.venda_cancelamento != 'C':
            vals_movimento['item_cancelado'] = False
        else:
            vals_movimento['item_cancelado'] = True

        custos = obj_estoque_preco_custo.get_custos_do_dia(
            cr, uid, vals_movimento.get('produto_id'), brw_cupons_item.capa_id.company_id.id,
            brw_cupons_item.capa_id.date_cupom, context)

        custos = custos.get(vals_movimento.get('produto_id'))
        if custos:
            vals_movimento['estmvi_saidas_custo_corrigido_total'] = (custos.get('estprc_custo_corrigido') or 0) * quantidade * qtd_embalagem
            vals_movimento['estmvi_saidas_ultimo_custo_total'] = (custos.get('estprc_ultimo_custo') or 0) * quantidade * qtd_embalagem
            vals_movimento['estmvi_saidas_custo_reposicao_total'] = (custos.get('estprc_custo_reposicao') or 0) * quantidade * qtd_embalagem
            vals_movimento['estmvi_saidas_custo_medio_contabil_total'] = (custos.get('estprc_custo_medio_contabil') or 0) * quantidade * qtd_embalagem

            cupom_date = brw_cupons_item.capa_id.hora_cupom_fim
            if not cupom_date:
                cupom_date = brw_cupons_item.capa_id.hora_cupom_inicio

        erro = False
        cod_vendedor = brw_cupons_item.ecfcit_cod_vendedor
        if cod_vendedor:
            vededor_ids = self.pool.get('vendas.vendedor').search(cr, uid,
                                                                  [('id','=',cod_vendedor)],
                                                                  context=context)
            if not vededor_ids:
                erro = True
                log(cr, uid, brw_cupons_item.capa_id.coo, None,
                    None, None,
                    _("O vendedor %s(%s) nao esta registrado em Vendas->Cadastros->Vendedores."
                      ) % (brw_cupons_item.ecfcit_nome_vendedor, 'Cod.:%s' % cod_vendedor),
                    brw_cupons_item.capa_id.id)
                return False, erro

            vals_movimento['vendedor_id'] = cod_vendedor
            if brw_cupons_item.venda_cancelamento != 'C':
                comissao_dados = self.ecfccp_get_comissao(cr, uid, vals_movimento.get('produto_id'),
                                                          vr_total_item,
                                                          cod_vendedor)
                if comissao_dados:
                    vals_movimento.update(comissao_dados)

        res = obj_estoque_movimento_itens.create(cr, uid, vals_movimento, context)
        # caso o item seja cancelado, nao gera CMV
        if vals_movimento['item_cancelado']:
            self.item_cancelado = True

        return res, erro

    def atualiza_fiscal_item_tributacao(self, cr, uid, produto_id, tributacao_id, brw_cupons_item,
                                        movimento_itens_id, context=None):
        """
        Cria tributação iss/icms e pis/cofins para os itens do cupom fiscal.
        """

        context = context and context.copy() or {}
        create_date = write_date = str(datetime.now())
        create_uid = write_uid = uid

        # -------------------------------ecf_cupons_item_tributacao--------------------------------
        def atualiza_ecf_item_tributacao(vals):
            """
            Popula a tabela ecf_cupons_item_tributacao para visualizar
            a tributação dos itens na Manutenção de Cupons
            """
            list_sql = []
            cst_ids = map(lambda x: "%s" % x.get('ecfcit_cst_id'), vals)
            sql_cst = '''
                    SELECT id, codigo, tipo
                    FROM fiscal_cst
                    WHERE id IN (%s)
                  ''' % ','.join(cst_ids)
            cr.execute(sql_cst)
            dados_cst = {}
            map(lambda x: dados_cst.update({x.get('id'): x}), cr.dictfetchall())
            for trib in vals:
                if trib.get('ecfcit_cst_id'):
                    cst_codigo = dados_cst.get(trib.get('ecfcit_cst_id')).get('codigo')
                    cst_tipo = dados_cst.get(trib.get('ecfcit_cst_id')).get('tipo')
                    tipo_substituicao = False
                    if cst_tipo in ['U', 'TU']:
                        if cst_codigo in ('10', '30', '70'):
                            tipo_substituicao = True

                sql = '''({create_uid}, '{create_date}', '{write_date}', {write_uid},
                                {encargo_id}, {cst_id}, {tributacao_id}, {aliquota},
                                {vr_base_calculo}, {vr_encargo}, {vr_reducao_base},
                                {itens_id}, {cupom_item_id}, {tipo_substituicao} )'''.\
                          format(create_uid=create_uid,
                                 create_date=create_date,
                                 write_date=write_date,
                                 write_uid=write_uid,
                                 encargo_id=trib.get('ecfcit_encargo_id'),
                                 cst_id=trib.get('ecfcit_cst_id'),
                                 tributacao_id=trib.get('ecfcit_tributacao_id'),
                                 aliquota=trib.get('ecfcit_aliquota'),
                                 vr_base_calculo=trib.get('ecfcit_vr_base_calculo'),
                                 vr_encargo=trib.get('ecfcit_vr_encargo'),
                                 vr_reducao_base=trib.get('ecfcit_vr_reducao_base'),
                                 itens_id=movimento_itens_id,
                                 cupom_item_id=brw_cupons_item.id,
                                 tipo_substituicao=tipo_substituicao)
                list_sql.append(sql)

            ecf_item_tributacao_ids = []
            if list_sql:
                sql_insert = """
                            INSERT INTO ecf_cupons_item_tributacao (
                                        create_uid, create_date, write_date, write_uid,
                                        ecfcit_encargo_id, ecfcit_cst_id, ecfcit_tributacao_id, ecfcit_aliquota,
                                        ecfcit_vr_base_calculo, ecfcit_vr_encargo, ecfcit_vr_reducao_base,
                                        ecfcit_movimento_itens_id, ecfcit_cupom_item_id, ecfcit_tipo_substituicao)
                            VALUES %s
                            returning id""" % ',\n'.join(list_sql)
                cr.execute(sql_insert)
                dados = cr.dictfetchall()
                ecf_item_tributacao_ids.extend(map(lambda x: x.get('id'), dados))

            if not (vals or ecf_item_tributacao_ids):
                return '1'
            elif len(vals) != len(ecf_item_tributacao_ids):
                return '1'

            return '0'

        # -----------------------------------------------------------------------------------------
        obj_cupons_item_tributacao = self.pool.get('ecf.cupons.item.tributacao')

        tributacao_icms_iss = obj_cupons_item_tributacao.ecfcit_calcular_tributacao_icms_iss(
            cr, uid, [tributacao_id], brw_cupons_item, context=context)
        preco_total = brw_cupons_item.preco_total - brw_cupons_item.desconto + brw_cupons_item.acrescimo
        tributacao_pis_cofins = obj_cupons_item_tributacao.ecfcit_calcular_tributacao_pis_cofins(
            cr, uid, produto_id, preco_total, brw_cupons_item, context=context)

        if isinstance(tributacao_pis_cofins, (str, unicode)):
            return '2-%s' % tributacao_pis_cofins

        fiscal_icms_iss = tributacao_icms_iss[0]
        ecf_ecms_iss = tributacao_icms_iss[1]

        fiscal_pis_cofins = tributacao_pis_cofins[0]
        ecf_pis_cofins = tributacao_pis_cofins[1]

        fiscal_item_tributacao_dados = fiscal_icms_iss
        ecf_item_tributacao_dados = ecf_ecms_iss

        for f in fiscal_pis_cofins:
            fiscal_item_tributacao_dados.append(f)

        for e in ecf_pis_cofins:
            ecf_item_tributacao_dados.append(e)

        fiscal_item_tributacao_ids = []
        encargo_ids = map(lambda x: x.get('encargo_id'),fiscal_item_tributacao_dados)
        naturezas_rec_pis_cofins = self.ecfccp_get_natureza_receita_pis_cofins(cr, uid, produto_id,
                                                                               encargo_ids)
        sql = '''
                SELECT id,
                       vr_base_calculo,
                       vr_encargo,
                       vr_reducao_base,
                       encargo_id,
                       movimento_itens_id
                FROM fiscal_nf_item_tributacao
                WHERE encargo_id IN (%s)
                  AND movimento_itens_id = %s
              ''' % (','.join(map(lambda x: '%s' % x, encargo_ids)), movimento_itens_id)
        cr.execute(sql)
        dados_nf_itens_tribs = cr.dictfetchall()
        encargo_item_trib = {}
        map(lambda x: encargo_item_trib.update(
            {'%s-%s' %(x.get('encargo_id'), x.get('movimento_itens_id')): x}), dados_nf_itens_tribs)
        list_sql = []
        for trib in fiscal_item_tributacao_dados:
            chave = '%s-%s' % (trib.get('encargo_id'),movimento_itens_id)
            dados_nf_itens_trib = encargo_item_trib.get(chave)
            if dados_nf_itens_trib:
                if isinstance(dados_nf_itens_trib, list):
                    dados_nf_itens_trib = dados_nf_itens_trib[0]
                item_tributacao_id = dados_nf_itens_trib.get('id')
                vr_base_calculo = dados_nf_itens_trib.get('vr_base_calculo') + trib.get('vr_base_calculo')
                vr_encargo = dados_nf_itens_trib.get('vr_encargo') + trib.get('vr_encargo')
                vr_reducao_base = dados_nf_itens_trib.get('vr_reducao_base') + trib.get('vr_reducao_base')
                if item_tributacao_id not in fiscal_item_tributacao_ids:
                    fiscal_item_tributacao_ids.append(item_tributacao_id)

                sql = '''
                    UPDATE fiscal_nf_item_tributacao
                    SET write_date = '%s', write_uid = %s,
                        vr_base_calculo = %s,
                        vr_encargo = %s,
                        vr_reducao_base = %s
                    WHERE id = %s
                      ''' % (write_date, write_uid, vr_base_calculo, vr_encargo, vr_reducao_base, item_tributacao_id)
                cr.execute(sql)
            else:

                tipo_substituicao = False
                if trib.get('cst_id'):
                    sql_cst = '''
                                SELECT codigo, tipo
                                FROM fiscal_cst
                                WHERE id = %s
                              ''' % (trib.get('cst_id'))
                    cr.execute(sql_cst)
                    dados_cst = cr.dictfetchall()[0]
                    cst_codigo = dados_cst.get('codigo')
                    cst_tipo = dados_cst.get('tipo')
                    if cst_tipo in ['U', 'TU']:
                        if cst_codigo in ('10', '30', '70'):
                            tipo_substituicao = True

                nat_pis_cofins = re.sub('[^0-9]', '', (naturezas_rec_pis_cofins.get(
                                                                     trib.get('encargo_id')) or ''))
                sql = '''({create_uid}, '{create_date}', '{write_date}', {write_uid},
                          {encargo_id}, {cst_id}, {tributacao_id}, {aliquota},
                          {vr_base_calculo}, {vr_encargo}, {vr_reducao_base}, {movimento_itens_id},
                          '{natureza_rec_pis_cofins}', {tipo_subst})'''.\
                          format(create_uid=create_uid,
                                 create_date=create_date,
                                 write_date=write_date,
                                 write_uid=write_uid,
                                 encargo_id=trib.get('encargo_id'),
                                 cst_id=trib.get('cst_id'),
                                 tributacao_id=trib.get('tributacao_id'),
                                 aliquota=trib.get('aliquota'),
                                 vr_base_calculo=trib.get('vr_base_calculo'),
                                 vr_encargo=trib.get('vr_encargo'),
                                 vr_reducao_base=trib.get('vr_reducao_base'),
                                 movimento_itens_id=movimento_itens_id,
                                 natureza_rec_pis_cofins=nat_pis_cofins,
                                 tipo_subst=tipo_substituicao)
                list_sql.append(sql)

        if list_sql:
            sql_insert = """
                        INSERT INTO fiscal_nf_item_tributacao (create_uid, create_date, write_date, write_uid,
                                    encargo_id, cst_id, tributacao_id, aliquota, vr_base_calculo,
                                    vr_encargo, vr_reducao_base, movimento_itens_id,
                                    fisnit_nat_rec_pis_cofins, tipo_substituicao)
                        VALUES %s
                        returning id""" % ',\n'.join(list_sql)
            cr.execute(sql_insert)
            dados = cr.dictfetchall()
            fiscal_item_tributacao_ids.extend(map(lambda x: x.get('id'), dados))


        if not (fiscal_item_tributacao_dados or fiscal_item_tributacao_ids) \
                or not atualiza_ecf_item_tributacao(ecf_item_tributacao_dados) \
                or len(fiscal_item_tributacao_dados) != len(fiscal_item_tributacao_ids):
            return '1'

        # quando tiver o contexto 'funcao_006' faz o write em massa fora desta funcao
        if not context.get('funcao_006'):
            # apenas recalcula os metodos funcionais
            context_trib = context.copy()
            context_trib['fisnit_super_write'] = True
            self.pool.get('fiscal.nf.item.tributacao').write(cr, uid, fiscal_item_tributacao_ids,
                                                             {}, context=context_trib)

        return fiscal_item_tributacao_ids

    def atualiza_financeiro_cupom(self, cr, uid, cupons_capa, movimento_itens_ids, vendedor_id, context=None):
        context = context and context.copy() or {}
        if not cupons_capa:
            return False

        cupom_titulo_id = False
        context['fintit_manter_prv'] = True
        context['cumpom_de_cancelamento'] = True

        log = self.ecfccp_armazena_dados_log
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')
        obj_ecf_ecf = self.pool.get('ecf.ecf')
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_fiscal_parametro = self.pool.get('fiscal.parametro')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_cancelamento_titulo = self.pool.get('cancelamento.titulo.osv')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_empresa = self.pool.get('adm.empresa.objeto')
        obj_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')

        calc_titulo_vr_original = obj_financeiro_titulo._float_persist
        financeiro_parametro = obj_adm_utils.admutl_get_parametros(cr, uid, 'financeiro',
                                                                   ['usar_centro_custos_e_eventos',
                                                                    'finpar_conta_tesouraria_avista_id'],
                                                                   company_id=cupons_capa.company_id.id)

        # se o cupom nao for de venda ou cancelamento e a capa nao possuir finalizacoes,
        # nao precisa atualizar financeiro
        if not (cupons_capa.tipo_cupom in ('V', 'C') and cupons_capa.cupons_finalizacao_ids):
            return True

        possui_finalizadora_a_prazo = False
        cartao_credito_debito = False
        forma_pgto_cartao = map(lambda i: i.finalizadora_id.forma_pagamento_id.cartao_credito_debito,
                                cupons_capa.cupons_finalizacao_ids)
        if any(forma_pgto_cartao):
            cartao_credito_debito = True
        tipos_finalizadora = map(lambda i: i.finalizadora_id.adm_condicao_id.tipo,
                                 cupons_capa.cupons_finalizacao_ids)
        if 'C' in tipos_finalizadora:
            possui_finalizadora_a_prazo = True
        ecf_id = cupons_capa.ecf_id.id
        company_id = cupons_capa.company_id.id
        if not company_id:
            # verificar tratamento qndo tiver empresa
            log(cr, uid, None, None, None,
                None, _("Nao foi possivel localizar a rmpresa logada."), cupons_capa.id)
            return False
        pessoa_id = None
        cupom_pessoa_id = None
        if cupons_capa.cliente_id.id:
            pessoa_id = cupons_capa.cliente_id.id
            cupom_pessoa_id = pessoa_id
        elif not pessoa_id and possui_finalizadora_a_prazo and not cartao_credito_debito:
            log(cr, uid, cupons_capa.coo, None, None, None,
                _("Nao e possivel gerar a fatura e o titulo do financeiro. O cupom nao "
                  "possui cliente cadastrado."), cupons_capa.id)
            return False

        # se o cupom for de cancelamento, deve cancelar os titulos que tenham
        # a mesma fatura do cupom
        titulo_cartao_ids = []
        obj_gerencial_movimento = self.pool.get('gerencial.movimento')
        if cupons_capa.tipo_cupom == 'C':
            coo_cupom_anterior = cupons_capa.coo - 1
            ecf_id = cupons_capa.ecf_id.id
            cupom_anterior_ids = self.search(cr, uid, [('coo', '=', coo_cupom_anterior),
                                                       ('ecf_id', '=', ecf_id),
                                                       ('date_cupom', '=', cupons_capa.date_cupom),
                                                       ('company_id', '=', company_id)], context=context)
            brw_cupom_anterior = self.browse(cr, uid, cupom_anterior_ids, context=context)
            if isinstance(brw_cupom_anterior, list):
                brw_cupom_anterior = brw_cupom_anterior[0]

            # Identifica se houve movimento da tesouraria para o cupom, quando pago em
            # dinheiro
            tesouraria_movimento_ids = obj_financeiro_tesouraria_movimento.search(
                cr, uid, [('fintem_cupom_id', '=', brw_cupom_anterior.id)], context=context)
            brw_faturas = brw_cupom_anterior.cupom_financeiro_fatura_ids
            brw_titulos = []
            brw_titulos_baixa = []
            fatura_ids = []
            titulo_ids = []
            # Identifica se houve movimento da tesouraria referente a baixa de titulo
            # para o cupom
            titulo_baixa_ids = []
            if brw_faturas:
                map(lambda i: (brw_titulos.extend(i.titulo_ids), fatura_ids.append(i.id)), brw_faturas)
                map(lambda i: (brw_titulos_baixa.extend(i.fintit_baixa_ids), titulo_ids.append(i.id)),
                    brw_titulos)

            if not tesouraria_movimento_ids:
                    map(lambda i:(tesouraria_movimento_ids.append(i.tesouraria_movimento_id.id),
                                  titulo_baixa_ids.append(i.id)), brw_titulos_baixa)

            # Se houve a baixa de titulo e/ou movimento da tesouraria, esses devem ser
            # excluidos
            if titulo_baixa_ids:
                obj_financeiro_titulo_baixa.unlink(cr, uid, titulo_baixa_ids, context=context)

            if tesouraria_movimento_ids:
                # A exclusao do movimento da tesouraria utilizando o metodo unlink,
                # recalcula o saldo das contas envolvidas
                context['excluir_movimento'] = True
                obj_financeiro_tesouraria_movimento.unlink(
                    cr, uid, tesouraria_movimento_ids, context=context)

            if titulo_ids:
                obj_financeiro_titulo.unlink(cr, uid, titulo_ids, context)

            if fatura_ids:
                obj_financeiro_fatura.unlink(cr, uid, fatura_ids, context=context)
            cupom_titulo_id = True

            # deleta o movimento gerencial do cupom de venda.
            gerencial_movimento_id = obj_gerencial_movimento.search(cr, uid,
                                         [('fingmo_cupom_id','=', cupom_anterior_ids[0])], context=context)
            if gerencial_movimento_id:
                obj_gerencial_movimento.unlink(cr, uid, gerencial_movimento_id, context=context)

        elif cupons_capa.tipo_cupom == 'V':
            if not ecf_id:
                log(cr, uid, None, None, None, None,
                    _("Nao foi possivel gerar a fatura do financeiro, pois a ECF do cupom nao "
                      "foi localizada."),
                    cupons_capa.id)
                return False

            cupom_titulo_id = False
            finalizacao_ids = cupons_capa.cupons_finalizacao_ids
            if not finalizacao_ids:
                log(cr, uid, None, None, None, None,
                    _("Nao foi encontrada nenhuma finalizadora com a mesma sequencia do "
                      "identificador CV (Vencimentos) para o cupom."),
                    cupons_capa.id)
                return False

            # CRIAR AS FATURAS P/ AS FINALIZADORAR A PRAZO
            obj_doc_tipo = self.pool.get('documento.documento.tipo')
            vencimento_financ_parametro = self.get_financeiro_parametro(
                cr, uid, company_id, 'CV', context=context)
            cupom_fatura_ids = []
            loja = str(cupons_capa.ecf_id.loja).rjust(4, '0')
            caixa = str(cupons_capa.ecf_id.caixa).rjust(3, '0')
            cupom = str(cupons_capa.coo).rjust(6, '0')
            documento = '%s-%s-%s' % (loja, caixa, cupom)

            for final in finalizacao_ids:
                pessoa_id = cupom_pessoa_id
                dict_titulo_pgto = {} # chave: id do titulo a vista, id da finalizadora
                vencimentos = final.ecf_cupons_vencimento_ids
                list_valor_vencimento = map(lambda i: i.valor, vencimentos)
                if not all(list_valor_vencimento):
                    log(cr, uid, cupons_capa.coo, None, None, None,
                        _("\tFinalizadora deve possuir todos os vencimentos"
                          " com valores validos."), cupons_capa.id)
                    return False

                ecf_dado = obj_ecf_ecf.read(cr, uid, ecf_id, ['ecf_conta_id'], context=context,
                                            load='_classic_write')
                if isinstance(ecf_dado, list):
                    ecf_dado = ecf_dado[0]
                ecf_conta = ecf_dado.get('ecf_conta_id')
                if final.finalizadora_id.adm_condicao_id.tipo == 'C' or \
                                final.finalizadora_id.forma_pagamento_id.sigla not \
                                in ('DN', 'VT') or pessoa_id:
                    tipo_pagamento_cartao = final.finalizadora_id.forma_pagamento_id.cartao_credito_debito
                    if tipo_pagamento_cartao:
                        pessoa_id = final.finalizadora_id.forma_pagamento_id.admfpg_cartao_pessoa_id.id
                        if not pessoa_id:
                            log(cr, uid, cupons_capa.coo, None, None, None,
                                _("Nao e possivel gerar a fatura e o titulo do financeiro. A "
                                  "forma de pagamento da finalizadora nao possui operadora "
                                  "cadastrada."),
                                cupons_capa.id)
                            return False

                    forma_pgto_id = final.finalizadora_id.forma_pagamento_id.id
                    if final.tipo_finalizadora == 'H':
                        doc_tipo_id = self.ecfccp_get_documento_id(
                            cr, uid, forma_pgto_id, 'CHQ', context=context)
                        if doc_tipo_id:
                            doc_tipo_id = doc_tipo_id[0]
                            if not doc_tipo_id:
                                log(cr, uid, None, None, None, None,
                                    _("\tNao foi encontrado nenhum documento cadastrado no "
                                      "sistema com a sigla CHQ."), cupons_capa.id)
                                return False
                    else:
                        doc_tipo_id = self.ecfccp_get_documento_id(
                            cr, uid, forma_pgto_id, 'CUP', context=context)
                        if doc_tipo_id:
                            doc_tipo_id = doc_tipo_id[0]
                            if not doc_tipo_id:
                                log(cr, uid, None, None, None, None,
                                    _("\tNao foi encontrado nenhum documento cadastrado no "
                                      "sistema com a sigla CUP."), cupons_capa.id)
                                return False

                    # se tiver pessoa_id, tem que gerar fatura, nao importa tipo de pagamento,
                    if not pessoa_id and \
                        (final.avista_aprazo in ('V', 'S') or tipo_pagamento_cartao):
                        continue

                    # VALORES P/ CRIAÇÃO DA CAPA DA FATURA
                    vals_fatura = {
                        'pessoa_id': pessoa_id,
                        'company_id': company_id,
                        'valor': final.valor - final.troco,
                        'tipo_fatura': 'R',
                        'data_emissao': cupons_capa.date_cupom,
                        'cupom_id': cupons_capa.id,
                        'qtd_parcelas': 0,
                        'documento': documento
                    }
                    finalizadora_forma_pgto_id = final.finalizadora_id.forma_pagamento_id.id
                    context.update({'importacao_ecf': True})  # USADO NO CREATE DA FATURA

                    # deve gerar fatura apenas se houver cliente ou se a finalizadora for cartao
                    # no caso de cartao, a pessoa_id eh o id da operadora de cartao
                    # uma fatura para todas as formas de pagamento
                    if not cupom_fatura_ids:
                        fatura_id = obj_financeiro_fatura.create(cr, uid, vals_fatura, context)
                    else:
                        fatura_id = cupom_fatura_ids[0]

                    if not fatura_id:
                        log(cr, uid, None, None, None, None,
                            _("\tNao foi possivel criar a fatura para a finalizadora %s" %
                              final.finalizadora_id.name), cupons_capa.id)
                        return False

                    cupom_fatura_ids.append(fatura_id)

                    # CRIAR AS PARCELAS DA FATURA
                    if not vencimentos:
                        vencimentos = [{'data_vencimento': cupons_capa.date_cupom,
                                        'valor': final.valor - final.troco,
                                        }]

                    context.update({'ctx_fisnfc_titulo_create_sem_custificacao': True})
                    taxa = final.finalizadora_id.taxa or 0
                    for vcto in vencimentos:
                        if datas.sigDatas(vcto['data_vencimento']).codErro == 1:
                            log(cr, uid, None, None, None, None,
                            _("\tData de vencimento invalida."),
                                cupons_capa.id)
                            return False
                        if vcto['data_vencimento'] < cupons_capa.date_cupom:
                            log(cr, uid, None, None, None, None,
                            _("\tData de vencimento menor do que a data de emissao"),
                                cupons_capa.id)
                            return False

                        vals = {
                            'data_vencimento': vcto['data_vencimento'],
                            'data_prorrogacao': vcto['data_vencimento'],
                            'tipo_titulo': 'R',
                            'local_cobranca_id': vencimento_financ_parametro.get('local_cobranca_id'),
                            'adm_moeda_id': vencimento_financ_parametro.get('moeda_id'),
                            'valor': vcto['valor'],
                            'pessoa_id': pessoa_id,
                            'doc_tipo_id': doc_tipo_id,
                            'company_id': company_id,
                            'fatura_id': fatura_id,
                            'vr_original': vcto['valor'],
                            'data_emissao': cupons_capa.date_cupom,
                            'tipo_sacado': 'C',
                            'documento': documento,
                            'state': "A"
                        }
                        if final.tipo_finalizadora == 'H':
                            context.update({'lancamento_cheque': True})
                            vals['documento'] = final.numero_cheque
                            vals['fintit_chq_emitente'] = final.cnpj or ''
                            vals['fintit_chq_banco'] = final.numero_banco or ''
                            vals['fintit_chq_agencia'] = final.agencia or ''
                            vals['fintit_chq_conta'] = final.conta or ''
                        elif 'lancamento_cheque' in context:
                            del context['lancamento_cheque']

                        vals['tipo_lancamento'] = 'A' # lancamento Automatico
                        if vendedor_id:
                            vals['vendedor_id'] = vendedor_id

                        try:
                            cupom_titulo_id = obj_financeiro_titulo.create(
                                cr, uid, vals, context=context)
                        except Exception as e:
                            log(cr, uid, None, None, None, None, '%s' % e, cupons_capa.id)
                            return False

                        if cupom_titulo_id:
                            obj_financeiro_titulo.write(
                                cr, uid, cupom_titulo_id, {
                                    'titulo_parent_id': cupom_titulo_id},
                                context=context)
                        # baixa apenas os titulos que a finalizadora seja a vista e a data de
                        # vencimento seja menor ou igual a data de hoje e nao seja cheque
                        if final.finalizadora_id.adm_condicao_id.tipo == 'V' \
                                        and final.finalizadora_id.tipo_finalizadora != 'H' \
                                        and vcto['data_vencimento'] <= str(datetime.now().date()):
                            dict_titulo_pgto[cupom_titulo_id] = finalizadora_forma_pgto_id
                            if ecf_conta:
                                context['ecf_conta_id'] = ecf_conta

                        if tipo_pagamento_cartao and taxa > 0:
                            vr_titulo_taxa = vcto['valor'] * (taxa / 100)
                            vr_titulo_taxa = float(calc_titulo_vr_original(
                                'vr_original', vr_titulo_taxa))
                            # Nao criar titulo da taxa de cartao se o valor for
                            # menor que 1 centavo
                            if vr_titulo_taxa < 0.01:
                                continue

                            vals_fatura_taxa = vals_fatura
                            vals_fatura_taxa['tipo_fatura'] = 'P'
                            taxa_fatura_id = obj_financeiro_fatura.create(
                                cr, uid, vals_fatura, context)

                            if context.get('lancamento_cheque'):
                                del context['lancamento_cheque']

                            vals['vr_original'] = vr_titulo_taxa
                            vals['tipo_titulo'] = 'P'
                            vals['fatura_id'] = taxa_fatura_id
                            vals['tipo_lancamento'] = 'A' # lancamento Automatico
                            cupom_titulo_id = obj_financeiro_titulo.create(
                                cr, uid, vals, context=context)
                            if not cupom_titulo_id:
                                log(cr, uid, None, None, None, None,
                                    _("\tNao foi possivel criar o titulo a pagar para a "
                                      "finalizadora %s." % final.finalizadora_id.name),
                                    cupons_capa.id)
                                return False

                            titulo_cartao_ids.append(cupom_titulo_id)

                else:
                    fiscal_parametro_id = obj_fiscal_parametro.search(
                        cr, uid, [('company_id', '=', company_id)], context=context)
                    if not fiscal_parametro_id:
                        raise osv.except_osv(_('Atencao!'),
                                             _('As informacoes dos parametros do Fiscal devem '
                                               'ser cadastradas.'))

                    finpar_conta_tesouraria_avista_id = financeiro_parametro.get(
                        'finpar_conta_tesouraria_avista_id')
                    if ecf_conta:
                        conta_id = ecf_conta
                    elif final.finalizadora_id.forma_pagamento_id.admfpg_conta_tesouraria_id:
                        conta_id = final.finalizadora_id.forma_pagamento_id.\
                            admfpg_conta_tesouraria_id.id
                    elif finpar_conta_tesouraria_avista_id:
                        conta_id = finpar_conta_tesouraria_avista_id
                    else:
                        log(cr, uid, None, None, None, None,
                            _("\tA conta caixa para recebimento a vista deve ser cadastrada nos "
                              "parametros do Financeiro."), cupons_capa.id)
                        return False
                    hist = 'Recebimento' if cupons_capa.tipo_cupom == 'V' else 'Cancelamento'
                    historico = '%s do cupom %s. ' % (hist, cupons_capa.coo)
                    historico += 'Loja %s e caixa %s. %s' % (
                        cupons_capa.ecf_id.loja, cupons_capa.ecf_id.caixa,
                        final.finalizadora_id.name)
                    vals_tesouraria = {
                        'state': 'A',
                        'conta_id': conta_id,
                        'fintem_cupom_id': cupons_capa.id,
                        'historico': historico,
                        'debito_credito': 'C' if cupons_capa.tipo_cupom == 'V' else 'D',
                        'data_movimento': cupons_capa.date_cupom,
                        'ano_competencia': cupons_capa.date_cupom[0:4],
                        'mes_competencia': cupons_capa.date_cupom[5:7],
                        'valor': (final.valor or 0.0) - (final.troco or 0.0)
                    }
                    context['validacao_cupom'] = True
                    obj_financeiro_tesouraria_movimento.create(
                        cr, uid, vals_tesouraria, context=context)

                if final.tipo_finalizadora != 'H':
                    for i in dict_titulo_pgto.keys():
                        # REALIZA A BAIXA PARA OS TITULOS DA FATURA, APENAS PARA
                        # OS TITULOS QUE O VENCIMENTO SEJA HOJE E
                        # O TIPO DE PAGAMENTO SEJA DO TIPO A VISTA.
                        context.update({'ecfccp_cupom_titulo': cupons_capa.coo})
                        obj_nf_capa.fisnfc_baixar_titulo(
                            cr, uid, titulo_id=i, vr_pago=0, forma_pgto_id=dict_titulo_pgto[i],
                            local_cobranca_id=vencimento_financ_parametro.get('local_cobranca_id'),
                            cheque_id=False, cheque_numero=documento, emitente_id=False,
                            agencia_id=False, nf_terceiro=False, company_id=company_id,
                            context=context)

            # Classificação de vendas
            usa_centro_custo_evento = financeiro_parametro.get('usar_centro_custos_e_eventos')
            if usa_centro_custo_evento and (titulo_cartao_ids or movimento_itens_ids):
                try:
                    obj_gerencial_movimento.custificar_vendas_cupom(cr, uid, movimento_itens_ids,
                                             cupons_capa.id, titulo_cartao_ids, company_id, context)
                except Exception as e:
                    # msg = _('\t Nao existe centro de custo e evento para venda cadastrados nos '
                    #         'parametros do Financeiro.')
                    msg = e
                    if isinstance(msg, osv.except_osv):
                        msg = msg[1]
                    log(cr, uid, cupons_capa.coo, None, None, None, msg, cupons_capa.id)
                    return False

        return True

    def atualiza_estoque_cupom(self, cr, uid, cupons_capa, context=None):
        context = context and context.copy() or {}
        if not cupons_capa:
            return False

        res = False
        erro = False

        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        log = self.ecfccp_armazena_dados_log
        local_id = cupons_capa.ecf_id and cupons_capa.ecf_id.local_id.id or False
        if not local_id:
            erro = True
            msg = _('\tCupom divergente. Local nao cadastrado para a ecf '
                    'Caixa: {c} Loja: {l}. Este LOCAL recebera a baixa de estoque da '
                    'importacao dos cupons.'). \
                format(c=cupons_capa.ecf_id.caixa, l=cupons_capa.ecf_id.loja)
            log(cr, uid, cupons_capa.coo, None, None, None, msg,
                cupons_capa.id)

        movimentacao = 'S' if cupons_capa.tipo_cupom == 'V' else 'E'

        if movimentacao == 'E':
            record_cupom_item_ids = obj_estoque_movimento_itens.search(
                cr, uid, [('cupom_id', '=', cupons_capa.id), ('item_cancelado', '=', True)], context=context)
        else:
            record_cupom_item_ids = obj_estoque_movimento_itens.search(
                cr, uid, [('cupom_id', '=', cupons_capa.id), ('item_cancelado', '!=', True)], context=context)

        record_cupom_item_ids = obj_estoque_movimento_itens.browse(
            cr, uid, record_cupom_item_ids)

        grade_qtd = {}
        for i in record_cupom_item_ids:
            embalagem_quantidade = i.produto_embalagem_id.embalagem_id.quantidade_na_embalagem
            if not embalagem_quantidade:
                erro = True
                log(cr, uid, None, None, None, None,
                    _('Registro CI: Excecao - Na importacao dos itens, nao foi possivel acessar'
                      ' a embalagem utilizada para o produto.'), cupons_capa.id)
            else:
                qtd_movimentada = i.quantidade * embalagem_quantidade
                if i.grade_id.id in grade_qtd:
                    grade_qtd[i.grade_id.id] += qtd_movimentada
                else:
                    grade_qtd[i.grade_id.id] = qtd_movimentada

        if grade_qtd:
            company_id = cupons_capa.company_id.id
            self.dia = obj_estoque_produto_saldo.atualizar_saldo_do_dia(
                cr, uid, company_id, grade_qtd, cupons_capa.date_cupom, movimentacao, local_id,
                context=context)
        if not erro:
            res = True
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = None
        if vals.get('coo') <= 0:
            raise osv.except_osv(_('Atencao!'), _('Coo deve ser maior que zero.'))
        if vals.get('tipo_cupom') == 'Z' and not context.get('importacao_cupom'):
            cupons_xz_ids = vals.get('cupons_xz_ids')
            if cupons_xz_ids:
                del vals['cupons_xz_ids']
            else:
                cupons_xz_ids = [(0, 0, {})]
            cupons_xztotalizadores_ids = vals.get('cupons_xztotalizadores_ids')
            if cupons_xztotalizadores_ids:
                del vals['cupons_xztotalizadores_ids']
            res = super(ecf_cupons_capa, self).create(cr, uid, vals, context)
            if cupons_xztotalizadores_ids:
                for x in cupons_xztotalizadores_ids:
                    for y in x:
                        if isinstance(y, dict):
                            y['cupom_id'] = res
                            self.pool.get('ecf.cupons.xztotalizadores').create(cr, uid, y, context)
            if cupons_xz_ids:
                for x in cupons_xz_ids:
                    for y in x:
                        if isinstance(y, dict):
                            y['cupom_id'] = res
                            self.pool.get('ecf.cupons.xz').create(cr, uid, y, context)
        if not res:
            res = super(ecf_cupons_capa, self).create(cr, uid, vals, context)

        return res

    def button_esboco_cupom(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        log_ids = self.read(cr, uid, ids, ['ecf_cupom_log_ids'], load='_classic_write',
                            context=context)[0].get('ecf_cupom_log_ids')
        self.write(cr, uid, ids, {'state': 'C'}, context=context)
        if log_ids:
            self.pool.get('ecf.cupom.log').write(cr, uid, log_ids, {'cupom_id': False},
                                                 context=context)

        return True

    def button_validar_cupom(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context.update({'valida_cupom': True})
        self.verifica_cupons(cr, uid, ids, context=context)
        return True

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('nota_fiscal_entrada'):
            res = self.write(cr, uid, ids, vals={'fisnfc_nf_referencia_id': False}, context=context)
            return res
        if context.get('referencia_nf_cupom'):
            res = self.write(cr, uid, ids, vals={'fisnfc_nf_referencia_id': False,
                                                 'nf_id': False},
                             context=context)
            return res
        self.ecfccp_estorna_movimento_cupom(cr, uid, ids, context)
        res = super(ecf_cupons_capa, self).unlink(cr, uid, ids, context)
        return res

    def ecfccp_estorna_movimento_cupom(self, cr, uid, ids, context=None):
        """
        O metodo realiza o estorno referente ao movimento financeiro e de estoque, dos cupons de
        venda validados. Alem disso, ao excluir cupom de reduçao Z, exclui tambem os dados do mapa
        de ECF (ecf_mapa_resumo_capa) referente
        :param cr: cursor no banco de dados
        :param uid: id do regsitro da tabela res_users, identificador do usuario
        :param ids: lista de ids dos registros a excluir da tabela ecf_cupons_capa
        :param context: dicionario com valores contextuais
        :return: booleano. Para False, nao estornou nenhum movimento e para True, estornou os
        movimentos
        """
        context = context and context.copy() or {}
        res = False
        # Leitura dos dados necessarios dos cupons a serem excluidos
        cupom_dados = self.read(
            cr, uid, ids,
            ['state', 'coo', 'total_cupom', 'tipo_cupom', 'ecf_id', 'company_id', 'date_cupom'],
            context=context, load='_classic_write')
        # Filtra os cupons de reduçao Z que estao validados
        cupom_z_dados = filter(lambda i: i.get('state') == 'V' and i.get('tipo_cupom') == 'Z',
                               cupom_dados)
        # Filtra os cupons de venda que estao validados
        cupom_dados = filter(lambda i: i.get('state') == 'V' and i.get('tipo_cupom') == 'V',
                             cupom_dados)
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')
        obj_ecf_mapa_resumo_capa = self.pool.get('ecf.mapa.resumo.capa')
        # Caso tenha cupom de reduçao Z validado, na lista de ids passada, deve excluir os dados do
        # mapa de ECF referente ao cupom
        for cupom_z in cupom_z_dados:
            coo_z = cupom_z.get('coo')
            ecf_id = cupom_z.get('ecf_id')
            mapa_resumo_capa_ids = obj_ecf_mapa_resumo_capa.search(
                cr, uid, [('coo', '=', coo_z), ('ecf', '=', ecf_id)], context=context)
            obj_ecf_mapa_resumo_capa.unlink(cr, uid, mapa_resumo_capa_ids,
                                            {'ecfccp_exclui_reducao_z': True})
        # Caso nao tenha cupom de venda validado, na lista de ids passada, retorna False, pois
        # somente esses possuem movimentos de estoque e financeiro
        if not cupom_dados:
            return res

        for cupom in cupom_dados:
            company_id = cupom.get('company_id')
            tesouraria_movimento_de_baixa = False
            coo = cupom.get('coo')
            res = True
            # Identifica se houve movimento da tesouraria para o cupom, quando pago em dinheiro
            tesouraria_movimento_ids = obj_financeiro_tesouraria_movimento.search(
                cr, uid, [('fintem_cupom_id', '=', cupom.get('id'))], context=context)
            # Identifica se houve movimento da tesouraria para o cupom, quando nao pago em dinheiro
            if not tesouraria_movimento_ids:
                fatura_ids = obj_financeiro_fatura.search(cr, uid,
                                                          [('cupom_id', '=', cupom.get('id'))], context=context)
                if fatura_ids:
                    fatura_dados = obj_financeiro_fatura.read(cr, uid, fatura_ids, ['titulo_ids'], context=context)
                    titulo_ids = []
                    map(lambda i: titulo_ids.extend(i.get('titulo_ids')), fatura_dados)
                    titulo_baixa_ids = obj_financeiro_titulo_baixa.search(
                        cr, uid, [('titulo_id', 'in', titulo_ids)], context=context)
                    titulo_baixa_dados = obj_financeiro_titulo_baixa.read(
                        cr, uid, titulo_baixa_ids, ['tesouraria_movimento_id'], context=context,
                        load='_classic_write')
                    map(lambda i: tesouraria_movimento_ids.append(i.get('tesouraria_movimento_id')),
                        titulo_baixa_dados)
                    tesouraria_movimento_de_baixa = True
            if tesouraria_movimento_ids:
                if not tesouraria_movimento_de_baixa:
                    tesouraria_movimento_ids = [tesouraria_movimento_ids[0]]
                # Identifica se houve movimento da tesouraria referente a baixa de titulo
                # para o cupom
                titulo_baixa_ids = obj_financeiro_titulo_baixa.search(
                    cr, uid, [('tesouraria_movimento_id', 'in', tesouraria_movimento_ids)], context=context)
                # Se houve a baixa de titulo e/ou movimento da tesouraria, esses devem ser excluidos
                if titulo_baixa_ids:
                    obj_financeiro_titulo_baixa.unlink(cr, uid, titulo_baixa_ids, context)
                # A exclusao do movimento da tesouraria utilizando o metodo unlink, recalcula o
                # saldo das contas envolvidas
                context['excluir_movimento'] = True
                obj_financeiro_tesouraria_movimento.unlink(cr, uid, tesouraria_movimento_ids,
                                                           context)
            # Identifica o movimento do estoque para os itens vendidos no cupom
            movimento_ids = obj_estoque_movimento_itens.search(
                cr, uid, [('cupom_id', '=', cupom.get('id'))], context=context)
            # Leitura dos dados necessarios para a atualizacao de saldo dos produtos
            movimento_dados = obj_estoque_movimento_itens.read(
                cr, uid, movimento_ids,
                ['produto_id', 'grade_id', 'local_id', 'quantidade', 'produto_embalagem_id'], context=context,
                load='_classic_write')
            # Atualiza o saldos dos produtos acrescentando de volta ao estoque a quantidade que
            # havia movimentado
            grade_qtd = {}
            for movimento in movimento_dados:
                quantidade = movimento.get('quantidade')
                local_id = movimento.get('local_id')
                produto_embalagem_id = movimento.get('produto_embalagem_id')
                if produto_embalagem_id:
                    embalagem_id = obj_estoque_produto_embalagem.browse(
                        cr, uid, produto_embalagem_id).embalagem_id
                    if embalagem_id:
                        quantidade_embalagem = embalagem_id.quantidade_na_embalagem or 1.0
                        quantidade *= quantidade_embalagem

                if movimento.get('grade_id') in grade_qtd:
                    grade_qtd[movimento.get('grade_id')] += quantidade
                else:
                    grade_qtd[movimento.get('grade_id')] = quantidade

            if grade_qtd:
                obj_estoque_produto_saldo.atualizar_saldo_do_dia(cr, uid,
                                                                 eps_company_id=company_id,
                                                                 grade_qtd=grade_qtd,
                                                                 eps_data=cupom.get('date_cupom'),
                                                                 eps_entrada_saida='E',
                                                                 eps_local_id=local_id,
                                                                 context=context
                )
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'coo' in vals and vals.get('coo') <= 0:
            raise osv.except_osv(_('Atencao!'), _('Coo deve ser maior que zero.'))
        obj_ecf_cupons_xztotalizadores = self.pool.get('ecf.cupons.xztotalizadores')
        base_imposto = False
        cupons_xz_ids = None
        cupons_xztotalizadores_ids = None
        tipo_cupom = vals.get('tipo_cupom') or ''
        if not isinstance(ids, list):
            ids = [ids]
        cupons_capa_dados = self.read(cr, uid, ids, ['state', 'tipo_cupom'], context=context)
        for x in cupons_capa_dados:
            tipo_cupom = x.get('tipo_cupom')
            state = x.get('state')
            if vals:
                if ('nf_id' not in vals) and ('flag_status_nf' not in vals) and not context.get(
                        'importacao_automatica'):
                    if state == 'V' and context.get('manutencao_cupom'):
                        raise osv.except_osv(_('Atencao!'),
                                             _('Cupom validado nao pode ser editado.'))
        xz_totalizadores_criado = False
        if tipo_cupom == 'Z':
            cupons_xz_ids = vals.get('cupons_xz_ids')
            cupons_xztotalizadores_ids = vals.get('cupons_xztotalizadores_ids')
            if cupons_xztotalizadores_ids:
                for x in cupons_xztotalizadores_ids:
                    for y in x:
                        if isinstance(y, dict):
                            if isinstance(ids, list):
                                ids = ids[0]
                            if 'base_imposto' in y:
                                base_imposto = True
                            y['cupom_id'] = ids
                            if x[1]:
                                if vals.get('cupons_xztotalizadores_ids'):
                                    del vals['cupons_xztotalizadores_ids']
                                obj_ecf_cupons_xztotalizadores.write(cr, uid, x[1], y, context)
                            else:
                                obj_ecf_cupons_xztotalizadores.create(cr, uid, y, context)
                                xz_totalizadores_criado = True
            if cupons_xz_ids or base_imposto:
                if base_imposto and not cupons_xz_ids:
                    ids_cupons_xz = self.pool.get('ecf.cupons.xz').search(cr, uid, [('cupom_id', '=', ids)],
                                                                          context=context)
                    if ids_cupons_xz:
                        for x in ids_cupons_xz:
                            self.pool.get('ecf.cupons.xz').write(cr, uid, x, {'cupom_id': ids}, context=context)
                    else:
                        id_cupons_xz = self.pool.get('ecf.cupons.xz').create(cr, uid, {'cupom_id': ids},
                                                                             context=context)
                else:
                    for x in cupons_xz_ids:
                        for y in x:
                            if isinstance(y, dict):
                                if isinstance(ids, list):
                                    ids = ids[0]
                                if x[1]:
                                    if vals.get('cupons_xz_ids'):
                                        del vals['cupons_xz_ids']
                                    y['cupom_id'] = ids
                                    self.pool.get('ecf.cupons.xz').write(cr, uid, x[1], y, context=context)
        if 'ecf_cupons_item_ids' in vals:
            vals.update({'state': u'D'})
        if xz_totalizadores_criado:
            del vals['cupons_xztotalizadores_ids']
        res = super(ecf_cupons_capa, self).write(cr, uid, ids, vals, context)
        if not context.get('importacao_cupom') and res and not context.get('verificado'):
            if cupons_xztotalizadores_ids:
                for x in cupons_xztotalizadores_ids:
                    for y in x:
                        if isinstance(y, dict):
                            if isinstance(ids, list):
                                ids = ids[0]
                            if 'base_imposto' in y:
                                base_imposto = True
                            if x[1]:
                                if vals.get('cupons_xztotalizadores_ids'):
                                    del vals['cupons_xztotalizadores_ids']
                                y['cupom_id'] = ids
                                self.pool.get('ecf.cupons.xztotalizadores').write(cr, uid, x[1], y, context=context)

            if cupons_xz_ids or base_imposto:
                if base_imposto and not cupons_xz_ids:
                    ids_cupons_xz = self.pool.get('ecf.cupons.xz').search(cr, uid, [('cupom_id', '=', ids)],
                                                                          context=context)
                    if ids_cupons_xz:
                        for x in ids_cupons_xz:
                            self.pool.get('ecf.cupons.xz').write(cr, uid, x, {'cupom_id': ids}, context=context)
                    else:
                        id_cupons_xz = self.pool.get('ecf.cupons.xz').create(cr, uid, {'cupom_id': ids},
                                                                             context=context)
                else:
                    for x in cupons_xz_ids:
                        for y in x:
                            if isinstance(y, dict):
                                if isinstance(ids, list):
                                    ids = ids[0]
                                if x[1]:
                                    if vals.get('cupons_xz_ids'):
                                        del vals['cupons_xz_ids']
                                    y['cupom_id'] = ids
                                    self.pool.get('ecf.cupons.xz').write(cr, uid, x[1], y, context=context)
        return res

    def cupom_abrir_nf(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        mod_obj = self.pool.get('ir.model.data')
        result = mod_obj.get_object_reference(cr, uid, 'fiscal', 'view_fiscal_nf_capa_form')
        view_id = result and result[1] or False
        nf_id = self.browse(cr, uid, ids).nf_id.id
        context['proprio'] = True
        if not nf_id:
            raise osv.except_osv(_('Atencao!'), _('Nota Fiscal nao foi gerada.'))
        return {
            'res_id': nf_id,
            'name': 'Nota Fiscal de Saida',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'fiscal.nf.capa',
            'view_id': view_id,
            'type': 'ir.actions.act_window',
        }

    def ecfccp_verificar_z(self, cr, uid, context=None):

        """
        Para cada ecf cadastrada o metodo deve procurar pelos ultimos 5 Zs e verificar:
        - se o total das finalizadoras no Z, bate com os cupons de venda e cancelamento
        - se o total das tributacoes no Z bate com o total das de venda e cancelamento
        O retorno do método deve ser uma lista que deve conter:
        - o numero da loja
        - o numero do caixa
        - data no formato invertido (amd)
        - true ou false(Tá ok ou tem erro)
        """
        context = context and context.copy() or {}
        res = []
        obj_ecf_ecf = self.pool.get('ecf.ecf')
        ecf_ids = obj_ecf_ecf.search(cr, uid, [('ativo', '=', True)], context=context)
        if ecf_ids:
            dados_ecf = obj_ecf_ecf.read(cr, uid, ecf_ids, ['caixa', 'loja'], context=context)
            for dado in dados_ecf:
                ecf_id = dado.get('id')
                caixa = dado.get('caixa')
                loja = dado.get('loja')
                total_finalizadoras = 0
                total_cupons = 0
                total_imposto_z = 0
                total_imposto_cupons = 0
                cupom_z_ids = self.search(cr, uid, [('ecf_id', '=', ecf_id),
                                                    ('tipo_cupom', '=', 'Z'),
                                                    ('state', '=', 'V')
                                                    ], limit=5, order='date_cupom DESC', context=context)
                if cupom_z_ids:
                    browse_cupons = self.browse(cr, uid, cupom_z_ids)
                    for cupom in browse_cupons:
                        date_cupom = cupom.date_cupom
                        lista = "loja: %s, Caixa: %s, Data: %s," % (loja, caixa, date_cupom)
                        cupons_finalizacao_ids = cupom.cupons_finalizacao_ids
                        total_finalizadoras = sum(map(lambda i: i.valor or 0,
                                                      cupons_finalizacao_ids))
                        cupons_xz_totalizadores_ids = cupom.cupons_xztotalizadores_ids
                        total_imposto_z = sum(map(lambda i: i.valor_imposto or 0,
                                                  cupons_xz_totalizadores_ids))
                        cupons_xz_ids = cupom.cupons_xz_ids
                        if cupons_xz_ids:
                            cupom_inicial = cupons_xz_ids[0].cupom_inicial
                            cupom_final = cupons_xz_ids[0].cupom_final
                            cupom_ids = self.search(cr, uid, [('state', '=', 'V'),
                                                              ('coo', '>=', cupom_inicial),
                                                              ('coo', '<=', cupom_final),
                                                              ('tipo_cupom', 'in', ('C', 'V'))], context=context)
                            if cupom_ids:
                                dados_cupom = self.read(cr, uid, cupom_ids, ['total_cupom',
                                                                             'tipo_cupom'], context=context)
                                total_cupons = sum(map(lambda i: i.get('total_cupom') * -1
                                if i.get('tipo_cupom') == 'C'
                                else i.get('total_cupom'), dados_cupom))

                                total_imposto_cupons = self.ecfccp_get_total_imposto_cupons(
                                    cr, uid, cupom_ids, context)
                        if total_cupons != total_finalizadoras or \
                                        total_imposto_z != total_imposto_cupons:
                            lista += ' Com erro'
                            res.append(lista)
        return res

    def ecfccp_get_total_imposto_cupons(self, cr, uid, cupom_ids, context=None):

        """
        Calcula e retorna o valor total do imposto de todos os itens dos cupons.
        """
        context = context and context.copy() or {}
        total_imposto_cupons = 0
        total_imposto_cupons_cancelamento = 0
        total_imposto_cupons_venda = 0
        obj_ecf_cupons_item = self.pool.get('ecf.cupons.item')
        # Encontra os cupons de cancelamento
        cupons_cancelamento_ids = self.search(cr, uid, [('id', 'in', cupom_ids),
                                                        ('tipo_cupom', '=', 'C')], context=context)
        # Encontra os cupons de venda
        cupons_venda_ids = self.search(cr, uid, [('id', 'in', cupom_ids),
                                                 ('tipo_cupom', '=', 'V')], context=context)
        if cupons_cancelamento_ids:
            cupons_item_ids = obj_ecf_cupons_item.search(cr, uid, [('capa_id', 'in',
                                                                    cupons_cancelamento_ids)], context=context)
            # Encontra o valor total do imposto dos itens de cancelamento
            if cupons_item_ids:
                browse_cupons_item = obj_ecf_cupons_item.browse(cr, uid, cupons_item_ids)
                total_imposto_cupons_cancelamento = sum(map(lambda i: (i.aliquota / (i.base or 1)) *
                                                                      i.preco_total, browse_cupons_item))
        if cupons_venda_ids:
            cupons_item_ids = obj_ecf_cupons_item.search(cr, uid, [('capa_id', 'in',
                                                                    cupons_venda_ids)], context=context)
            # Encontra o valor total do imposto dos itens de venda
            if cupons_item_ids:
                browse_cupons_item = obj_ecf_cupons_item.browse(cr, uid, cupons_item_ids)
                total_imposto_cupons_venda = sum(map(lambda i: (i.aliquota / (i.base or 1)) *
                                                               i.preco_total, browse_cupons_item))
        total_imposto_cupons = total_imposto_cupons_venda - total_imposto_cupons_cancelamento
        return total_imposto_cupons

    def ecfccp_button_salvar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {}, context)
        return res

    def ecfccp_get_natureza_receita_pis_cofins(self, cr, uid, produto_id, encargo_ids, context=None):
        context = context and context.copy() or {}
        natureza_receita = {}.fromkeys(encargo_ids, '')
        if not produto_id or not encargo_ids:
            return natureza_receita

        dados_produto = self.pool.get('estoque.produto').read(cr, uid, produto_id,
                                   ['estpro_pis_nat_rec', 'estpro_cofins_nat_rec'], context=context)
        obj_fiscal_encargo = self.pool.get('fiscal.encargo')
        for encargo in obj_fiscal_encargo.read(cr, uid, encargo_ids, ['sigla'],context=context):
            if encargo.get('sigla') not in ('PIS', 'COFINS'):
                continue

            natureza_receita[encargo.get('id')] = dados_produto.get('estpro_cofins_nat_rec') or ''
            if encargo.get('sigla') == 'PIS':
                natureza_receita[encargo.get('id')] = dados_produto.get('estpro_pis_nat_rec') or ''

        return natureza_receita

    def ecfccp_gerar_pis_cofins(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_cupom = self.browse(cr, uid, ids)
        if isinstance(brw_cupom, list):
            brw_cupom = brw_cupom[0]
        cupons_capa_id = brw_cupom.id
        args = [('cupom_id', '=', cupons_capa_id)]
        movimento_itens_ids = self.pool.get('estoque.movimento.itens').search(cr, uid, args, context=context)
        brw_movimento_itens = self.pool.get('estoque.movimento.capa').browse(cr, uid, movimento_itens_ids)
        if brw_cupom.state == 'V':
            for brw_item_mov in brw_movimento_itens:
                item_tributacao_ids = self.pool.get('fiscal.nf.item.tributacao').search(cr, uid, [
                    ('movimento_itens_id', '=', brw_item_mov.id)], context=context)
                brw_item_tributacao = self.pool.get('fiscal.nf.item.tributacao').browse(cr, uid, item_tributacao_ids)
                for item_trib in brw_item_tributacao:
                    if item_trib.encargo_id.sigla == 'PIS' or item_trib.encargo_id.sigla == 'COFINS':
                        self.pool.get('fiscal.nf.item.tributacao').unlink(cr, uid, item_trib.id, context)
        bwr_cupom_itens = brw_cupom.ecf_cupons_item_ids
        obj_ecf_cupons_item_tributacao = self.pool.get('ecf.cupons.item.tributacao')
        for brw_cupom_item in bwr_cupom_itens:
            cod_bar = brw_cupom_item.item
            context['active_test'] = False
            automacao_ids = self.pool.get('automacao.codigo.barra').search(
                cr, uid, [('codigo_barra', '=', cod_bar)], context=context)
            del context['active_test']
            if not automacao_ids:
                continue

            cupons_item_tributacao_ids = map(
                lambda i: i.id,
                filter(lambda x: x.ecfcit_encargo_id.sigla in ('PIS', 'COFINS'),
                       brw_cupom_item.tributacao_item_ids))

            brw_automacao = self.pool.get('automacao.codigo.barra').browse(cr, uid, automacao_ids)
            brw_produto = brw_automacao[0].produto_id
            tributacao_dados = self.pool.get('ecf.cupons.item.tributacao').\
                ecfcit_calcular_tributacao_pis_cofins(cr, uid, brw_produto.id,
                                                      brw_produto.preco_venda, brw_cupom_item,
                                                      context)
            if isinstance(tributacao_dados, list):
                map(lambda i: obj_ecf_cupons_item_tributacao.unlink(cr, uid, i, context),
                    cupons_item_tributacao_ids)

                dict_nf_item = {}
                item_mov_id = brw_cupom_item.tributacao_item_ids[0].ecfcit_movimento_itens_id.id
                dict_nf_item['movimento_itens_id'] = item_mov_id
                dict_nf_item['aliquota'] = tributacao_dados[0][0].get('aliquota') or 0
                dict_nf_item['vr_base_calculo'] = tributacao_dados[0][0].get('vr_base_calculo') or 0
                dict_nf_item['cst_id'] = tributacao_dados[0][0].get('cst_id') or ''
                dict_nf_item['vr_encargo'] = tributacao_dados[0][0].get('vr_encargo') or 0
                dict_nf_item['tributacao_id'] = tributacao_dados[0][0].get('tributacao_id') or ''
                dict_nf_item['encargo_id'] = tributacao_dados[0][0].get('encargo_id') or ''
                context['manutencao_cupom'] = True
                res = self.pool.get('fiscal.nf.item.tributacao').create(cr, uid, dict_nf_item,
                                                                        context)

                dict_ecf_item_tributacao = {
                    'ecfcit_nf_item_tributacao_id': res,
                    'ecfcit_movimento_itens_id': item_mov_id,
                    'ecfcit_cupom_item_id': brw_cupom_item.id,
                    'ecfcit_encargo_id': tributacao_dados[0][0].get('encargo_id') or '',
                    'ecfcit_aliquota': tributacao_dados[0][0].get('aliquota') or 0,
                    'ecfcit_vr_base_calculo': tributacao_dados[0][0].get('vr_base_calculo') or 0,
                    'ecfcit_cst_id': tributacao_dados[0][0].get('cst_id') or '',
                    'ecfcit_vr_encargo': tributacao_dados[0][0].get('vr_encargo') or 0,
                    'ecfcit_tributacao_id': tributacao_dados[0][0].get('tributacao_id') or '',
                }
                obj_ecf_cupons_item_tributacao.create(cr, uid, dict_ecf_item_tributacao, context)

            if isinstance(tributacao_dados, list) and len(tributacao_dados) >= 1 and \
                    len(tributacao_dados[0]) > 1:
                dict_nf_item = {}
                item_mov_id = brw_cupom_item.tributacao_item_ids[0].ecfcit_movimento_itens_id.id
                dict_nf_item['movimento_itens_id'] = item_mov_id
                dict_nf_item['aliquota'] = tributacao_dados[0][1].get('aliquota') or 0
                dict_nf_item['vr_base_calculo'] = tributacao_dados[0][1].get('vr_base_calculo') or 0
                dict_nf_item['cst_id'] = tributacao_dados[0][1].get('cst_id') or ''
                dict_nf_item['vr_encargo'] = tributacao_dados[0][1].get('vr_encargo') or 0
                dict_nf_item['tributacao_id'] = tributacao_dados[0][1].get('tributacao_id') or ''
                dict_nf_item['encargo_id'] = tributacao_dados[0][1].get('encargo_id') or ''
                context['manutencao_cupom'] = True
                res = self.pool.get('fiscal.nf.item.tributacao').create(cr, uid, dict_nf_item,
                                                                        context)

                dict_ecf_item_tributacao = {
                    'ecfcit_nf_item_tributacao_id': res,
                    'ecfcit_movimento_itens_id': item_mov_id,
                    'ecfcit_cupom_item_id': brw_cupom_item.id,
                    'ecfcit_encargo_id': tributacao_dados[0][1].get('encargo_id') or '',
                    'ecfcit_aliquota': tributacao_dados[0][1].get('aliquota') or 0,
                    'ecfcit_vr_base_calculo': tributacao_dados[0][1].get('vr_base_calculo') or 0,
                    'ecfcit_cst_id': tributacao_dados[0][1].get('cst_id') or '',
                    'ecfcit_vr_encargo': tributacao_dados[0][1].get('vr_encargo') or 0,
                    'ecfcit_tributacao_id': tributacao_dados[0][1].get('tributacao_id') or '',
                }
                obj_ecf_cupons_item_tributacao.create(cr, uid, dict_ecf_item_tributacao, context)

        return True

    def get_msg_contribuinte(self, cr, uid, cupons_ids, context=None):
        """
        @:param: cr - cursor do openerp
        @:uid: uid - usuario do openerp
        @:cupons_ids: lista de id dos cupons
        @:type:list
        @:return: dicionario contendo ecf_id como chave, e o numero dos
                  cupons como valor da chave
        @rtype:dict
        """
        context = context and context.copy() or {}
        values = {}
        msg = ''
        for i in self.browse(cr, uid, cupons_ids, context=context):
            coo = i.coo
            modelo = i.ecf_id.modelo
            nserie = i.ecf_id.nro_serie
            marca = i.ecf_id.fabricante_id.nome
            if i.ecf_id.id not in values:
                msg = "Marca: %s Mod.: %s N.Fab.: %s - Cupons: %s " % (marca, modelo, nserie, coo)
            else:
                msg = values.get(i.ecf_id.id)
                msg += ", %s" % coo
            values[i.ecf_id.id] = msg
        return values

    def ecfccp_get_documento_id(self, cr, uid, adm_forma_pgto_id, sigla_alternativa=False, context=None):
        context = context and context.copy() or {}
        obj_doc_tip = self.pool.get('documento.documento.tipo')
        obj_adm_form_pgto = self.pool.get('adm.forma.pgto')
        res = False
        doc_tip = obj_adm_form_pgto.read(cr, uid, adm_forma_pgto_id, ['admfpg_documento_tipo_id'], context=context,
                                         load='_classic_write')
        if doc_tip and doc_tip.get('admfpg_documento_tipo_id'):
            res = [doc_tip.get('admfpg_documento_tipo_id')]
        else:
            if sigla_alternativa:
                args = [('sigla', '=', sigla_alternativa)]
                res = obj_doc_tip.search(cr, uid, args, context=context)
        return res

    def ecfccp_fatura_pedido(self, cr, uid, bwr_capa, context=None):
        '''Atualiza o status do pedido para faturado/validado dos cupons que vieram de DAV's
           apenas os cupom com supcap_tipo = ('V', 'C')
        '''
        context = context and context.copy() or {}
        if bwr_capa.tipo_cupom not in ('V', 'C'):
            return True

        obj_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_titulos = self.pool.get('financeiro.titulo')
        nro_pedido = 'PVE-%010d' % int(bwr_capa.ecfccp_numero_pedido)
        pedido_ids = obj_pedido_capa.search(cr, uid, [('numero_pedido','=',nro_pedido),
                                                      ('company_id', '=', bwr_capa.company_id.id)],
                                                      context=context)
        context_aux = context.copy()
        context_aux['pedcap_write_state'] = True
        context_aux['fisnfc_tratar_exclusao'] = True
        # cupons de venda
        vals = {'state': 'F',
                'loja': bwr_capa.ecf_id.loja,
                'caixa':bwr_capa.ecf_id.caixa,
                'coo': bwr_capa.coo,
                'exportado_frente': True}
        if bwr_capa.tipo_cupom == 'C': # cupons cancelados
            vals = {'state': 'R',
                    'loja': 0,
                    'caixa': 0,
                    'coo': 0,
                    'exportado_frente': False}
            context['nao_verificar_state_pedido'] = True
            obj_pedido_capa.set_rascunho_pedido(cr, uid, pedido_ids, context)
            self.pool.get('supervisor.dav.capa').write(cr, uid, pedido_ids, {'supdva_nro_cupom': 0},
                                                       context=context)

        obj_pedido_capa.write(cr, uid, pedido_ids, vals, context=context_aux)

        # o pedido pode ter mais de um titulo, deletar todos eles, pois estao em previsao.
        titulo_ids = obj_titulos.search(cr, uid, [('fintit_pedido_id','in',pedido_ids)],
                                        context=context)
        obj_titulos.unlink(cr, uid, titulo_ids, context=context_aux)
        return True

    def ecfccp_trata_expedicao_cupom_venda(self, cr, uid, brw_cupom, context=None):
        """
        Metodo realmente usado no modulo expedicao_ecf, onde se encontra heranca para ele.
        Ele se encontra aqui pois quando o expedicao_ecf estiver instalado, e preciso que seja
        realizado o controle dele antes que se execute o commit, para que tudo referente ao cupom
        esteja num mesmo processo.
        """
        return True

    def ecfccp_trata_expedicao_cupom_cancelamento(self, cr, uid, cupons_capa, context=None):
        """
        Metodo realmente usado no modulo expedicao_ecf, onde se encontra heranca para ele.
        Ele se encontra aqui pois quando o expedicao_ecf estiver instalado, e preciso que seja
        realizado o controle dele antes que se execute o commit, para que tudo referente ao cupom
        esteja num mesmo processo.
        """
        return True

    def ecfccp_trata_fidelidade_cupom_venda(self, cr, uid, cupons_capa, mov_id, context=None):
        """
        Iplmentado por heranca no modulo fidelidade cupom, gera os pontos de fidelidade de acordo
        com o valor do cupom
        """
        return True

    def ecfccp_trata_fidelidade_cupom_cancelamento(self, cr, uid, cupons_capa, mov_id,
                                                   context=None):
        """
        Iplmentado por heranca no modulo fidelidade cupom, deduz os pontos de fidelidade de acordo
        com o valor do cupom
        """
        return True

    def ecfccp_get_comissao(self, cr, uid, produto_id, vr_total_item, vendedor_id):
        """
        O metodo calcula a porcentagem e o valor da comissao para insercao na tabela
        estoque_movimento_itens. A porcentagem da comissao eh encontrada no cadastro do produto,
        senão na categoria do mesmo, senão no cadastro do vendedor ou senão no parametro do estoque.
        O valor base para calculo da comissao eh referente ao total vendido de cada item.

        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param produto_id: id da tabela estoque_produto
        :param vr_total_item: float do total do item rateado
        :param vendedor_id: id da tabela vendas_vendedor

        :return: Dicionario com os valores do percentual e total da comissao
        """
        res = {}

        if not produto_id or not vr_total_item or not vendedor_id:
            return res

        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_adm_util = self.pool.get('adm.utils')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')

        brw_estoque_produto = obj_estoque_produto.browse(cr, uid, produto_id)

        comissao = brw_estoque_produto.estpro_percentual_comissao or 0.0

        if not comissao:
            categoria_id = brw_estoque_produto.categoria_id
            while categoria_id:
                comissao = categoria_id.estcat_percentual_comissao or 0.0
                if comissao:
                    break
                else:
                    categoria_id = categoria_id.parent_id

        if not comissao and vendedor_id:
            comissao = obj_vendas_vendedor.browse(cr, uid, vendedor_id).percentual or 0.0

        if not comissao:
            estoque_parametro_dados = obj_adm_util.admutl_get_parametros(cr, uid, 'estoque',
                                                                         'comissoes')
            comissao = estoque_parametro_dados.get('comissoes') or 0.0

        vr_comissao = round((comissao / 100) * vr_total_item, 6)

        res.update({'estmvi_percentual_comissao': comissao,
                    'estmvi_vr_comissao': vr_comissao})

        return res

    def ecfccp_exclui_estorna_movimento_cupom_periodo(self, cr, uid, company_id, data_inicial,
                                                      data_final, ecf=[], context=None):
        """
        O metodo realiza os processos de exclusao e estorno dos cupons em um determinado periodo.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param data_inicial: data inicial do periodo de exclusao/estorno.
        :type data_inicial: date
        :param data_final: data final do periodo de exclusao/estorno.
        :type data_final: date
        :param ecf: lista das ECFs envolvidas no processo.
        :type ecf: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso encontre cupons de acordo com os parametros, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            SELECT 
                ecc.id AS id,
                ecc.coo AS coo,
                ecf.caixa AS caixa,
                ecf.loja AS loja
            FROM
                ecf_cupons_capa ecc
            LEFT JOIN
                ecf_ecf ecf ON ecf.id = ecc.ecf_id
            WHERE
                ecc.company_id = {company_id}
                AND ecc.date_cupom BETWEEN '{data_inicial}' AND '{data_final}'
                {caixa}
        """.format(company_id=company_id, data_inicial=data_inicial, data_final=data_final,
                   caixa='AND ecf.caixa IN %s' % str(ecf).replace('[', '(').replace(']', ')')
                   if ecf else '')
        cr.execute(sql)
        cupom_dados = cr.dictfetchall()
        if not cupom_dados:
            return False
        for cupom in cupom_dados:
            try:
                self.unlink(cr, uid, [cupom.get('id')], context)
                cr.commit()
                print_log(_('- COO: %s LOJA: %s CAIXA: %s excluido.') % (
                    cupom.get('coo'), cupom.get('loja'), cupom.get('caixa')))
            except:
                print_log(_('- Nao foi possivel excluir COO: %s LOJA: %s CAIXA: %s.') % (
                    cupom.get('coo'), cupom.get('loja'), cupom.get('caixa')))
        return True
