#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.osv import fields

class ecf_cupons_item_tributacao(osv.Model):
    _name = 'ecf.cupons.item.tributacao'
    _inherit = 'ecf.cupons.item.tributacao'

    def onchange_tributacao(self, cr, uid, ids, ecfcit_encargo_id, ecfcit_tributacao_id,
                            ecfcit_vr_total_item, context=None):
        context = context and context.copy() or {}
        vals = {}
        if ecfcit_tributacao_id:
            vals = self.ecfcit_calcula_tributacao_item(cr, uid, ecfcit_tributacao_id,
                                                       ecfcit_vr_total_item)
        return {'value': vals}

    def onchange_encargo(self, cr, uid, ids, ecfcit_encargo_id, context=None):
        context = context and context.copy() or {}
        vals = {}
        if ecfcit_encargo_id:
            vals.update({'ecfcit_encargo_id': ecfcit_encargo_id, 'ecfcit_tributacao_id': False})
        return {'value': vals}

    def onchange_valida(self, cr, uid, ids, ecfcit_percentual_mva, ecfcit_aliquota_venda,
                        ecfcit_modbcst, ecfcit_vr_base_calculo, ecfcit_aliquota, context=None):
        context = context and context.copy() or {}

        vals = {'ecfcit_percentual_mva': ecfcit_percentual_mva,
                'ecfcit_aliquota_venda': ecfcit_aliquota_venda}
        valor_icms_st = self.ecfcit_calcular_vr_icms_st_item(ecfcit_modbcst, ecfcit_vr_base_calculo,
                                                             ecfcit_aliquota,
                                                             ecfcit_aliquota_venda,
                                                             ecfcit_percentual_mva)
        if valor_icms_st:
            vals['ecfcit_vr_icms_substituicao'] = valor_icms_st
        return {'value': vals}

    def ecfcit_calcula_tributacao_item(self, cr, uid, ecfcit_tributacao_id,
                                       ecfcit_valor_item, context=None):
        context = context and context.copy() or {}
        if ecfcit_tributacao_id:
            if isinstance(ecfcit_tributacao_id, (int, long)):
                obj_tributacao = self.pool.get('fiscal.tributacao')
                dados_tributacao = obj_tributacao.read(cr, uid, ecfcit_tributacao_id,
                                                       ['base', 'aliquota', 'cst_id', 'encargo_id'], context=context)
                if isinstance(dados_tributacao, list):
                    dados_tributacao = dados_tributacao[0]
                if isinstance(dados_tributacao, dict):
                    valor_base = ecfcit_valor_item * dados_tributacao['base'] / 100
                    valor_encargo = valor_base * dados_tributacao['aliquota'] / 100
                    valor_base_reduzida = ecfcit_valor_item - valor_base
                    if isinstance(dados_tributacao['cst_id'], (tuple, list)):
                        cst_id = dados_tributacao['cst_id'][0]
                    else:
                        cst_id = dados_tributacao['cst_id']
                    tipo_substituicao = False
                    if cst_id:
                        browse_fiscal_cst = self.pool.get('fiscal.cst').browse(cr, uid, cst_id)
                        if browse_fiscal_cst.tipo == 'U' and browse_fiscal_cst.codigo in (
                                '10', '30', '70'):
                            tipo_substituicao = True
                    if isinstance(dados_tributacao['encargo_id'], (tuple, list)):
                        encargo_id = dados_tributacao['encargo_id'][0]
                    else:
                        encargo_id = dados_tributacao['encargo_id']
                    vals = {'ecfcit_tributacao_id': ecfcit_tributacao_id,
                            'ecfcit_aliquota': dados_tributacao['aliquota'],
                            'ecfcit_vr_base_calculo': valor_base,
                            'ecfcit_vr_encargo': valor_encargo,
                            'ecfcit_vr_reducao_base': valor_base_reduzida,
                            'ecfcit_cst_id': cst_id,
                            'ecfcit_encargo_id': encargo_id,
                            'ecfcit_tipo_substituicao': tipo_substituicao}

                    if not tipo_substituicao:
                        vals.update({'ecfcit_percentual_mva': 0,
                                     'ecfcit_vr_icms_substituicao': 0,
                                     'ecfcit_aliquota_venda': 0,
                                     'ecfcit_modbcst': ''})
                    return vals

    def ecfcit_calcular_vr_icms_st_item(self, ecfcit_modbcst, ecfcit_vr_base_calculo,
                                        ecfcit_aliquota_compra,
                                        ecfcit_aliquota_venda, ecfcit_percentual_mva):

        """
        Calcula o valor do ICMS ST do item para modalidade margem valor agregado.
        :param ecfcit_modbcst:
        :param ecfcit_vr_base_calculo:
        :param ecfcit_aliquota_compra:
        :param ecfcit_aliquota_venda:
        :param ecfcit_percentual_mva:
        return : retorna o total do ICMS ST para o item
        """

        vr_icms_substituicao = 0
        if ecfcit_modbcst == '4':
            icms_st_compra_item = ecfcit_vr_base_calculo * (float(ecfcit_aliquota_compra) / 100)
            icms_st_venda_item = (ecfcit_vr_base_calculo + (
                ecfcit_vr_base_calculo * float(ecfcit_percentual_mva) / 100)) * (
                                     float(ecfcit_aliquota_venda) / 100)
            total_icms_st_item = round(icms_st_venda_item, 2) - round(icms_st_compra_item, 2)
            valor_icms_st = total_icms_st_item * -1 if total_icms_st_item < 0 else total_icms_st_item
            vr_icms_substituicao = valor_icms_st
        return vr_icms_substituicao

    def ecfcit_calcular_tributacao_icms_iss(self, cr, uid, tributacao_id,
                                            brw_cupom_item, context=None):

        """
        O metodo calcula os valores e gera os dados, a partir da tributacao e dos dados dos itens
        do cupom, necessarios para uma insercao nas tabelas
        'fiscal_nf_item_tributacao' e 'ecf_cupons_item_tributacao'
        @return: Retorna uma Lista com Listas de Dicionario com os dados a serem inseridos na tabela
        """
        context = context and context.copy() or {}
        vals = []
        valores_nf_item = []
        valores_coo_item = []
        obj_fisnit = self.pool.get('fiscal.nf.item.tributacao')
        tributacao_id = tributacao_id[0]
        sql = '''
                SELECT encargo_id, cst_id, aliquota, base
                FROM fiscal_tributacao
                WHERE id = %s
              ''' % (tributacao_id)
        cr.execute(sql)
        dados_trib = cr.dictfetchall()[0]
        encargo_id = dados_trib.get('encargo_id')
        cst_id = dados_trib.get('cst_id')
        qt_emb = 1
        aliquota = dados_trib.get('aliquota')
        base = dados_trib.get('base')
        acrescimo = brw_cupom_item.acrescimo or 0
        desconto = brw_cupom_item.desconto or 0
        float_round = fields.float_round
        total_base = float(float_round(
            obj_fisnit.fisnit_calcula_vr_base_calculo(brw_cupom_item.preco_total,
                                                      percentual_base=base), 2))
        tot_base_reduzida = (brw_cupom_item.preco_total - total_base) * brw_cupom_item.quantidade * qt_emb
        total_base += acrescimo - desconto
        total_imposto = obj_fisnit.fisnit_calcula_vr_encargo(total_base, aliquota)
        valores_nf_item.append({'encargo_id': encargo_id,
                                'cst_id': cst_id,
                                'tributacao_id': tributacao_id,
                                'aliquota': aliquota,
                                'vr_base_calculo': total_base,
                                'vr_encargo': total_imposto,
                                'vr_reducao_base': tot_base_reduzida})

        valores_coo_item.append({'ecfcit_encargo_id': encargo_id,
                                 'ecfcit_cst_id': cst_id,
                                 'ecfcit_tributacao_id': tributacao_id,
                                 'ecfcit_aliquota': aliquota,
                                 'ecfcit_vr_base_calculo': total_base,
                                 'ecfcit_vr_encargo': total_imposto,
                                 'ecfcit_vr_reducao_base': tot_base_reduzida})

        vals.append(valores_nf_item)
        vals.append(valores_coo_item)

        return vals

    def ecfcit_calcular_tributacao_pis_cofins(self, cr, uid, produto_id, vr_produto,
                                              brw_cupom_item, context=None):

        """
        O metodo calcula os valores e gera os dados, a partir da tributacao e dos dados dos itens
        do cupom, necessarios para uma insercao nas tabelas
        'fiscal_nf_item_tributacao' e 'ecf_cupons_item_tributacao'
        @return: Retorna uma Lista com Listas de Dicionario com os dados a serem inseridos na tabela
        """
        context = context and context.copy() or {}
        vals = []
        valores_nf_item = []
        valores_coo_item = []
        obj_fiscal_utils = self.pool.get('fiscal.utils')
        context['proprio'] = True
        try:
            calc = obj_fiscal_utils.calcular_imposto(cr, uid, produto_id, vr_produto=vr_produto,
                                                 context=context)
        except Exception as e:
            calc = []
            if 'Cupom CSON' in e.message:
                return e.message.split('-')[1]


        encargo_ids = ','.join(map(lambda x: '%s' % x.get('encargo_id'), calc))
        if not encargo_ids:
            return [[],[]]

        sql = '''SELECT id,
                        sigla
                 FROM fiscal_encargo
                 WHERE id IN (%s)
              ''' % encargo_ids
        cr.execute(sql)
        dados = cr.dictfetchall()

        id_encargo_sigla = {}
        map(lambda x: id_encargo_sigla.update({x.get('id'): x.get('sigla')}), dados)
        for imp in calc:
            encargo_sigla = id_encargo_sigla.get(imp.get('encargo_id')) or ''
            if encargo_sigla in ('PIS', 'COFINS'):
                tot_base_reduzida = vr_produto - imp['valor_base']
                valores_nf_item.append({'encargo_id': imp['encargo_id'],
                                        'cst_id': imp['cst_id'],
                                        'tributacao_id': imp['tributacao_id'],
                                        'aliquota': imp['aliquota'],
                                        'vr_base_calculo': imp.get('valor_base'),
                                        'vr_encargo': imp.get('valor_imposto'),
                                        'vr_reducao_base': tot_base_reduzida})

                valores_coo_item.append({'ecfcit_encargo_id': imp['encargo_id'],
                                         'ecfcit_cst_id': imp['cst_id'],
                                         'ecfcit_tributacao_id': imp['tributacao_id'],
                                         'ecfcit_aliquota': imp['aliquota'],
                                         'ecfcit_vr_base_calculo': imp.get('valor_base'),
                                         'ecfcit_vr_encargo': imp.get('valor_imposto'),
                                         'ecfcit_vr_reducao_base': tot_base_reduzida})

        vals.append(valores_nf_item)
        vals.append(valores_coo_item)

        return vals

    def ecfcit_verifica_ecf_item_tributacao(self, cr, uid, ids, vals,
                                            add_nf_item_trib_id=False, context=None):

        """
        O metodo verifica os dados da ecf_cupons_item_tributacao, e retorna os dados necessarios
        para uma insercao na tabela 'fiscal_nf_item_tributacao'
        @return: Dicionario com os dados a serem inseridos na tabela
        """
        context = context and context.copy() or {}
        res = {}
        read_ecf_item_trib = self.read(cr, uid, ids, context=context)
        if read_ecf_item_trib:
            movimento_itens = read_ecf_item_trib[0]['ecfcit_movimento_itens_id']
            if movimento_itens:
                movimento_itens_id = movimento_itens[0]
            else:
                movimento_itens_id = vals['ecfcit_movimento_itens_id']
            if movimento_itens_id:
                args_ecf_item_trib = ('ecfcit_movimento_itens_id', '=', movimento_itens_id)
                ecf_item_trib_ids = self.search(cr, uid, [args_ecf_item_trib], context=context)
                read_ecf_item_trib = self.read(cr, uid, ecf_item_trib_ids, context=context)
                vals_nf_item_trib = []
                for i in read_ecf_item_trib:
                    nf_item_trib = {
                        'vr_base_icms_substituicao': i['ecfcit_vr_base_icms_substituicao'],
                        'modbcst': i['ecfcit_modbcst'],
                        'aliquota': i['ecfcit_aliquota'],
                        'vr_base_calculo': i['ecfcit_vr_base_calculo'],
                        'percentual_mva': i['ecfcit_percentual_mva'],
                        'valor_pauta': i['ecfcit_valor_pauta'],
                        'retido': i['ecfcit_retido'],
                        'tipo_substituicao': i['ecfcit_tipo_substituicao'],
                        'vr_icms_substituicao_guia': i['ecfcit_vr_icms_substituicao_guia'],
                        'vr_encargo': i['ecfcit_vr_encargo'],
                        'aliquota_venda': i['ecfcit_aliquota_venda'],
                        'fisnit_nat_rec_pis_cofins': i['ecfcit_nat_rec_pis_cofins'],
                        'vr_reducao_base': i['ecfcit_vr_reducao_base'],
                        'vr_icms_substituicao': i['ecfcit_vr_icms_substituicao']
                    }
                    if i['ecfcit_movimento_itens_id']:
                        nf_item_trib['movimento_itens_id'] = i['ecfcit_movimento_itens_id'][0]
                    if i['ecfcit_cst_id']:
                        nf_item_trib['cst_id'] = i['ecfcit_cst_id'][0]
                    if i['ecfcit_encargo_id']:
                        nf_item_trib['encargo_id'] = i['ecfcit_encargo_id'][0]
                    if i['ecfcit_tributacao_id']:
                        nf_item_trib['tributacao_id'] = i['ecfcit_tributacao_id'][0]
                    if i['ecfcit_csosn_id']:
                        nf_item_trib['csosn_id'] = i['ecfcit_csosn_id'][0]
                    if add_nf_item_trib_id:
                        nf_item_trib['nf_item_trib_id'] = i['ecfcit_nf_item_tributacao_id']
                    vals_nf_item_trib.append(nf_item_trib)
                res = vals_nf_item_trib
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            id = ids[0]
        else:
            id = ids
        res = {}
        if ids:
            brw_cupons_item_tributacao = self.browse(cr, uid, id)
            obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
            obj_movimento_itens = self.pool.get('estoque.movimento.itens')

            if brw_cupons_item_tributacao:
                cupom_item_id = brw_cupons_item_tributacao.ecfcit_cupom_item_id
                movimento_itens = brw_cupons_item_tributacao.ecfcit_movimento_itens_id
                encargo_id = brw_cupons_item_tributacao.ecfcit_encargo_id
                tributacao_id = brw_cupons_item_tributacao.ecfcit_tributacao_id
                if movimento_itens:
                    movimento_itens_id = movimento_itens.id
                else:
                    movimento_itens_id = obj_movimento_itens.search(
                        cr, uid, [('cupom_item_id', '=', cupom_item_id.id)], context=context)
                if movimento_itens_id and encargo_id and tributacao_id:
                    res = super(ecf_cupons_item_tributacao, self).unlink(cr, uid, [id], context)
                    args_mov_item = ('movimento_itens_id', '=', movimento_itens_id)
                    args_encargo = ('encargo_id', '=', encargo_id.id)
                    args_trib = ('tributacao_id', '=', tributacao_id.id)

                    fiscal_nf_item_trib_id = obj_fiscal_nf_item_tributacao.search(cr, uid,
                                                                                  [args_mov_item,
                                                                                   args_encargo,
                                                                                   args_trib], context=context)

                    # Excluindo os dados na fiscal_nf_item_tributacao
                    obj_fiscal_nf_item_tributacao.unlink(cr, uid, fiscal_nf_item_trib_id,
                                                         context=context)

        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = {}
        if vals:
            obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
            obj_movimento_itens = self.pool.get('estoque.movimento.itens')
            cupom_item_id = vals['ecfcit_cupom_item_id']
            encargo_id = vals['ecfcit_encargo_id']
            tributacao_id = vals['ecfcit_tributacao_id']
            if 'ecfcit_movimento_itens_id' in vals:
                movimento_itens_id = vals['ecfcit_movimento_itens_id']
            else:
                movimento_itens_id = obj_movimento_itens.search(cr, uid, [
                    ('cupom_item_id', '=', cupom_item_id)], context=context)
            if movimento_itens_id:
                if isinstance(movimento_itens_id, list):
                    movimento_itens_id = movimento_itens_id[0]
                vals.update({'ecfcit_movimento_itens_id': movimento_itens_id})
                args_mov_item = ('movimento_itens_id', '=', movimento_itens_id)
                args_encargo = ('encargo_id', '=', encargo_id)
                args_trib = ('tributacao_id', '=', tributacao_id)
                fiscal_nf_item_trib_id = obj_fiscal_nf_item_tributacao.search(cr, uid,
                                                                              [args_mov_item,
                                                                               args_encargo,
                                                                               args_trib], context=context)
                if not 'ecfcit_nf_item_tributacao_id' in vals:
                    if fiscal_nf_item_trib_id:
                        if isinstance(fiscal_nf_item_trib_id, list):
                            fiscal_nf_item_trib_id = fiscal_nf_item_trib_id[0]
                        vals.update({'ecfcit_nf_item_tributacao_id': fiscal_nf_item_trib_id})

                arg_encargos_item = [
                    ('ecfcit_encargo_id', '=', encargo_id),
                    ('ecfcit_movimento_itens_id', '=', movimento_itens_id),
                    ('ecfcit_nf_item_tributacao_id', '=', vals.get('ecfcit_nf_item_tributacao_id'))
                ]
                encargos_item = self.search(cr, uid, arg_encargos_item, context=context)
                if encargos_item:
                    msg = 'Nao e permitido incluir o mesmo encargo em mais de uma tributacao'
                    obj_ecf_cupons_item = self.pool.get('ecf.cupons.item')
                    brw_cupons_item = obj_ecf_cupons_item.browse(cr, uid, cupom_item_id)
                    if brw_cupons_item:
                        cod_produto = brw_cupons_item.item
                        msg += '\n\nCodigo do produto: %s' % cod_produto if cod_produto else ''
                    raise osv.except_osv(_('Atencao!'), _(msg))

                res = super(ecf_cupons_item_tributacao, self).create(cr, uid, vals, context)
                id = res
                ids = [id]
                if id and context.get('manutencao_cupom'):
                    if not movimento_itens_id and encargo_id and tributacao_id:
                        res = {}
                    else:
                        vals_nf_item_trib = self.ecfcit_verifica_ecf_item_tributacao(cr, uid, ids,
                                                                                     vals, False,
                                                                                     context)
                        for vals_nf in vals_nf_item_trib:
                            if vals_nf['movimento_itens_id'] == movimento_itens_id and \
                                    vals_nf['encargo_id'] == encargo_id and \
                                    vals_nf['tributacao_id'] == tributacao_id:

                                    # Inserindo os dados na fiscal_nf_item_tributacao
                                    id_create = obj_fiscal_nf_item_tributacao.create(cr, uid,
                                                                                     vals_nf,
                                                                                     context)

                                    # Inserindo o id da fiscal_nf_item_tributacao na coluna
                                    # ecfcit_nf_item_tributacao_id da ecf_cupons_item_tributacao
                                    vals_create = {'ecfcit_nf_item_tributacao_id': id_create}
                                    super(ecf_cupons_item_tributacao, self).write(cr, uid, ids,
                                                                                  vals_create,
                                                                                  context)

        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            id = ids[0]
        else:
            id = ids
        res = {}
        if vals:
            res = super(ecf_cupons_item_tributacao, self).write(cr, uid, ids, vals, context)
            obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
            brw_cupons_item_tributacao = self.browse(cr, uid, id)
            movimento_itens_id = brw_cupons_item_tributacao.ecfcit_movimento_itens_id
            encargo_id = brw_cupons_item_tributacao.ecfcit_encargo_id
            tributacao_id = brw_cupons_item_tributacao.ecfcit_tributacao_id

            if 'ecfcit_encargo_id' in vals:
                arg_encargos_item = [
                    ('ecfcit_encargo_id', '=', encargo_id.id),
                    ('ecfcit_movimento_itens_id', '=', movimento_itens_id.id)
                ]
                encargos_item = self.search(cr, uid, arg_encargos_item, context=context)
                if encargos_item:
                    msg = 'Nao e permitido incluir o mesmo encargo em mais de uma tributacao'
                    obj_ecf_cupons_item = self.pool.get('ecf.cupons.item')
                    cupom_item_id = brw_cupons_item_tributacao.ecfcit_cupom_item_id
                    if cupom_item_id:
                        brw_cupons_item = obj_ecf_cupons_item.browse(cr, uid, cupom_item_id.id)
                        if brw_cupons_item:
                            cod_produto = brw_cupons_item.item
                            msg += '\n\nCodigo do produto: %s' % cod_produto if cod_produto else ''
                        raise osv.except_osv(_('Atencao!'), _(msg))

            if not movimento_itens_id and encargo_id and tributacao_id:
                res = {}
            else:
                vals_nf_item_trib = self.ecfcit_verifica_ecf_item_tributacao(cr, uid, ids, vals,
                                                                             True, context)
                for vals_nf in vals_nf_item_trib:
                    if vals_nf['movimento_itens_id'] == movimento_itens_id.id and \
                            vals_nf['encargo_id'] == encargo_id.id and \
                            vals_nf['tributacao_id'] == tributacao_id.id:

                        if 'nf_item_trib_id' in vals_nf:
                            nf_item_trib_id = vals_nf['nf_item_trib_id']
                            if isinstance(nf_item_trib_id, (list, tuple)) and nf_item_trib_id:
                                nf_item_trib_id = nf_item_trib_id[0]
                            del vals_nf['nf_item_trib_id']

                            # Alterando os dados na fiscal_nf_item_tributacao
                            obj_fiscal_nf_item_tributacao.write(cr, uid, nf_item_trib_id,
                                                                vals_nf, context=context)

        return res
