#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.osv import osv
from datetime import date, datetime
import MySQLdb as mysql
import psycopg2
import psycopg2.extras
from openerp.tools.translate import _


class ecf_exportar_saldo_supervisor(osv.Model):
    _inherit = 'ecf.exportar.saldo.super'

    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
        for i in args:
            # completa o codido de barras com zero a esquerda
            if 'ecfess_codigo_barra' in i:
                i[2] = i[2].zfill(14)
                break

        return super(ecf_exportar_saldo_supervisor, self).search(cr, user, args, offset=offset,
                                           limit=limit, order=order, context=context, count=count)

    def ecfess_get_parametro_id(self, cr, uid, obj_estoque_parametro=None, context=None):
        context = context and context.copy() or {}
        parametro_id = False
        estoque_parametro = 'estoque.paramtro'
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, estoque_parametro, context=context)
        if company_id:
            if not obj_estoque_parametro:
                obj_estoque_parametro = self.pool.get(estoque_parametro)
            parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
        return parametro_id

    def ecfess_parametro_read_fields(self, cr, uid, fields_to_read, context=None):
        context = context and context.copy() or {}
        parametro_dados = {}
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        parametro_id = self.ecfess_get_parametro_id(cr, uid, obj_estoque_parametro, context)
        if parametro_id and isinstance(fields_to_read, list):
            parametro_dados = obj_estoque_parametro.read(
                cr, uid, parametro_id, fields_to_read, context=context
            )
            if parametro_dados and isinstance(parametro_dados, list):
                parametro_dados = parametro_dados[0]
        return parametro_dados

    def ecfess_atualizar_registros(self, cr, uid, company_id, local_id, grades_saldo, context=None):
        context = context and context.copy() or {}
        if not all([company_id, local_id, grades_saldo]):
            return False

        excluir_consulta = self.pool.get('estoque.local').browse(cr, uid, local_id,
                                                                 context).excluir_consultas
        if excluir_consulta:
            return False

        fields_to_read = ['estpar_super_local_id', 'estpar_super_loja']

        # -- BUSCA DO ESTOQUE.PARAMETRO DO ID DO LOCAL E A LOJA DO SUPERVISOR --
        # -- O FRENTE TRABALHA APENAS COM UM LOCAL, QUE É INDICADO NO PARAMETRO --
        parametro_dados = self.ecfess_parametro_read_fields(cr, uid, fields_to_read, context)
        if not parametro_dados:
            return False

        supervisor_local_ids = parametro_dados.get('estpar_super_local_id') or []
        supervisor_loja = parametro_dados.get('estpar_super_loja')

        # -- SE O LOCAL QUE ESTA SOFRENDO ALTERAÇÃO É IGUAL
        # AO LOCAL DO PARAMETRO PARA EXPORTAR PARA O SUPERVISOR --
        if local_id not in supervisor_local_ids or not supervisor_loja:
            return False

        grade_ids = ','.join(map(lambda x: '%s' % x, grades_saldo.keys()))

        acb_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'automacao.codigo.barra', context=context, user_company_id=company_id)
        sql = """SELECT acb.grade_id,
                        acb.codigo_barra,
                        COALESCE (ee.quantidade_na_embalagem, 1) AS qtd_embalagem,
                        eeu.fracao                               AS fracao
                 FROM automacao_codigo_barra         AS acb
                 LEFT JOIN estoque_produto_embalagem AS epe ON epe.id = acb.produto_embalagem_id
                 LEFT JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id
                 LEFT JOIN estoque_embalagem_unidade AS eeu ON eeu.id = ee.unidade_id
                 WHERE acb.grade_id IN ({grade_ids})
                   AND acb.company_id = {company_id}
              """.format(grade_ids=grade_ids,
                         company_id=acb_company_id)
        cr.execute(sql)
        dados_itens = cr.dictfetchall()

        sql_insert = """INSERT INTO ecf_exportar_saldo_super
                          (ecfess_loja, ecfess_saldo, ecfess_exporta_saldo_inicial,
                           ecfess_codigo_barra, company_id)
                        VALUES {vals}
                     """
        list_export = []
        codigos_barras = []
        for item in dados_itens:
            grade_id = item.get('grade_id')
            saldo = int(grades_saldo.get(grade_id) / item.get('qtd_embalagem'))
            if item.get('fracao'):
                saldo = grades_saldo.get(grade_id) / item.get('qtd_embalagem')

            if saldo < 0:
                saldo = 0

            valores = """({loja}, {saldo}, {saldo_inicial}, '{codigo_barra}', {company_id})
                               """.format(loja=supervisor_loja,
                                          saldo=saldo,
                                          saldo_inicial=context.get('exporta_saldo_inicial') or False,
                                          codigo_barra=item.get('codigo_barra'),
                                          company_id=company_id)
            codigos_barras.append(item.get('codigo_barra'))
            list_export.append(valores)

        if not list_export:
            return False

        # para exportacao Total(manual), deletar a tabela ecf_exportar_saldo_super
        if context.get('ecfexp_exp_total_saldo'):
            cr.execute("DELETE FROM ecf_exportar_saldo_super WHERE company_id = %s" % company_id)

        sql_select = """
            SELECT id
            FROM ecf_exportar_saldo_super
            WHERE ecfess_loja = {loja}
              AND ecfess_codigo_barra IN ({itens})
              AND company_id = {company_id}
        """.format(loja=supervisor_loja,
                   itens="'%s'" % "','".join(codigos_barras),
                   company_id=company_id)
        cr.execute(sql_select)
        saldo_ids = map(lambda  x: x.get('id'), cr.dictfetchall())
        if saldo_ids: # caso o registro ja existe, atualiza e nao insere
            cr.execute("DELETE FROM ecf_exportar_saldo_super WHERE id IN (%s) AND company_id = %s"
                       % (','.join(map(lambda x: '%s' % x,  saldo_ids)), company_id))

        sql = sql_insert.format(vals=','.join(list_export))
        cr.execute(sql)
        return True

    def ecfess_cron_exportar_saldo_supervisor(self, cr, uid, context=None):
        context = context and context.copy() or {}
        create_date = write_date = datetime.utcnow()

        sql = """
            SELECT
                company_id,
                estpar_super_host,
                estpar_super_db,
                estpar_super_user,
                estpar_super_password,
                estpar_tipo_exportacao,
                estpar_super_loja,
                estpar_usar_master
            FROM
                estoque_parametro
            WHERE estpar_super_host IS NOT NULL
              AND estpar_super_db IS NOT NULL
              AND estpar_super_user IS NOT NULL
        """
        cr.execute(sql)
        estoque_parametro = cr.dictfetchall()

        usar_master = filter(lambda x: x.get('estpar_usar_master'), estoque_parametro) and True or \
                      False
        for parametro in estoque_parametro:
            loja = parametro.get('estpar_super_loja')
            host = parametro.get('estpar_super_host')
            user = parametro.get('estpar_super_user')
            password = parametro.get('estpar_super_password') or ''
            db = parametro.get('estpar_super_db')
            tipo_exportacao = parametro.get('estpar_tipo_exportacao')
            company_id = parametro.get('company_id')
            if tipo_exportacao not in ['M', 'P']:
                continue

            if not all([host, user, db, tipo_exportacao]):
                continue

            context_saldo = context.copy()
            context_saldo['sig_force_company'] = True
            exportacao_saldo_supervisor_ids = self.search(cr, uid, [('ecfess_loja', '=', loja),
                                                                    ('company_id','=',company_id)],
                                                          context=context_saldo)

            if not exportacao_saldo_supervisor_ids:
                continue

            today = date.today().strftime('%Y-%m-%d')

            exportacao_dados = self.read(cr, uid, exportacao_saldo_supervisor_ids,
                                         ['ecfess_loja', 'ecfess_codigo_barra', 'ecfess_saldo',
                                          'ecfess_exporta_saldo_inicial'], context=context)
            connection = False
            _cr = None

            if tipo_exportacao == 'M':
                try:
                    connection = mysql.connect(host=host, user=user, passwd=password, db=db)
                    _cr = connection.cursor()
                except Exception as e:
                    raise osv.except_osv(_('Atencao!'), _('Nao foi possivel conectar ao supervisor. '
                                                          'Verifique as informacoes nos parametros do '
                                                          'estoque.'))
            elif tipo_exportacao == 'P':
                try:
                    # CONEXAO COM O BANCO DE DADOS
                    connection = psycopg2.connect(database=db, user=user, password=password, host=host,
                                                  connect_timeout=3)
                    _cr = connection.cursor(cursor_factory=psycopg2.extras.DictCursor)
                except Exception as err:
                    if connection:
                        connection.close()
                    raise osv.except_osv(_('Atencao!'), _('Nao foi possivel conectar ao supervisor. '
                                                          'Verifique as informacoes nos parametros do '
                                                          'estoque.'))

            str_codigo_barra = "'" + "','".join(map(lambda x: x.get('ecfess_codigo_barra'),
                                                    exportacao_dados)) + "'"
            cond_emp = camp_emp = valor_emp = ''
            if usar_master:
                cond_emp = """AND supsal_nro_empresa = %s""" % company_id
                camp_emp = ', supsal_nro_empresa'
                valor_emp = ', %s' % company_id

            sql_select = """
                SELECT id,
                       supsal_item,
                       supsal_loja AS sup_loja
                FROM supervisor_mov_saldo
                WHERE supsal_item IN ({item})
                  AND supsal_loja = {loja}
                  {cond_emp}
            """.format(item=str_codigo_barra, loja=loja, cond_emp=cond_emp)
            _cr.execute("SET statement_timeout = '20s'")
            _cr.execute(sql_select)
            codbarra_loja = {}
            codbarra_id = {}
            dados_mov_saldo = self.pool.get('adm.utils').admutl_converts_fetchall_to_dictfetchall(_cr)
            map(lambda x: codbarra_loja.update({x.get('supsal_item'): x.get('sup_loja')}),
                dados_mov_saldo)
            map(lambda x: codbarra_id.update({x.get('supsal_item'): x.get('id')}),
                dados_mov_saldo)
            list_ids = []
            sql_insert = """
                INSERT INTO supervisor_mov_saldo (create_uid, create_date, write_date,
                                                  write_uid, supsal_loja, supsal_item,
                                                  supsal_saldo, supsal_saldo_inicial,
                                                  supsal_data %s)
                VALUES {vals}
            """ % camp_emp
            list_vals = []
            for i in exportacao_dados:
                loja = i.get('ecfess_loja')
                codbarra = str(i.get('ecfess_codigo_barra') or '')
                if codbarra_loja.get(codbarra) and loja != codbarra_loja.get(codbarra):
                    continue

                if codbarra_id.get(codbarra):
                    list_ids.append(str(codbarra_id.get(codbarra)))

                saldo = i.get('ecfess_saldo') or 0
                saldo = saldo if saldo >= 0 else 0

                if tipo_exportacao == 'M':
                    sql_insert = """
                        INSERT INTO TblMov_Saldo (Loja, Item, Saldo, Data)
                        VALUES ( {loja}, '{item}', {saldo},'{data}') on DUPLICATE KEY
                        UPDATE Saldo = {saldo};
                    """
                    sql = sql_insert

                elif tipo_exportacao == 'P':
                    vals = """({create_uid}, '{create_date}', '{write_date}', {write_uid},
                               {loja}, '{item}', {saldo}, {saldo_inicial}, '{data}' {valor_emp})
                """.format(create_date=create_date,
                           create_uid=uid,
                           write_date=write_date,
                           write_uid=uid,
                           loja=loja,
                           item=codbarra,
                           saldo=saldo,
                           saldo_inicial=saldo,
                           data=today,
                           valor_emp=valor_emp)
                    list_vals.append(vals)

            if not list_vals:
                connection.commit()
                connection.close()
                continue

            sentenca = sql_insert.format(vals=','.join(list_vals))
            if list_ids:
                _cr.execute("DELETE FROM supervisor_mov_saldo WHERE id IN (%s) %s" %
                            (','.join(list_ids), cond_emp))

            _cr.execute(sentenca)

            str_ids = ','.join(map(lambda x: str(x), exportacao_saldo_supervisor_ids))
            sql_delete = """DELETE FROM ecf_exportar_saldo_super 
                            WHERE id IN (%s) 
                            AND company_id = %s""" % (str_ids, company_id)
            try:
                cr.execute(sql_delete)
            except: # pode ser que o registro ja foi deletado pela exportacao, entao nao faz nada
                pass

            connection.commit()
            connection.close()

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
