# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _


class ecf_mapa_resumo_capa(osv.Model):
    _name = 'ecf.mapa.resumo.capa'
    _inherit = 'ecf.mapa.resumo.capa'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = super(ecf_mapa_resumo_capa, self).create(cr, uid, vals, context)
        if not context.get('importacao_validacao_cupom'):
            self.ecfmrc_valida_mapa(cr, uid, [res], context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(ecf_mapa_resumo_capa, self).write(cr, uid, ids, vals, context)
        self.ecfmrc_valida_mapa(cr, uid, ids, context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_mapa_resumo = self.browse(cr, uid, ids, context)
        cupom_ids = map(lambda i: i.ecfmrc_cupom_id.id, brw_mapa_resumo)
        if any(cupom_ids) and not context.get('ecfccp_exclui_reducao_z'):
            raise osv.except_osv(_('Atencao'), _('Este mapa somente pode ser excluido atraves da '
                                                 'manutencao de cupons'))
        res = super(ecf_mapa_resumo_capa, self).unlink(cr, uid, ids, context)
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        args_ecf = [('company_id', '=', company_id)]
        for i in args:
            if not i[0] == 'ecf':
                continue
            if isinstance(i[2], int):
                args_ecf.append(('id', i[1], i[2]))
            elif isinstance(i[2], str):
                if i[2].isdigit():
                    args_ecf.append(('caixa', i[1], i[2]))
                else:
                    args_ecf.append(('modelo', i[1], i[2]))
        ecf_ids = self.pool.get('ecf.ecf').search(cr, uid, args_ecf,
                                                  context=context)
        args = filter(lambda i: 'ecf' not in i, args)
        args.append(('ecf', 'in', ecf_ids))
        res = super(ecf_mapa_resumo_capa, self).search(cr, uid, args, offset, limit, order,
                                                       context, count)
        return res

    def ecfmrc_valida_mapa(self, cr, uid, ids, context=None):
        """
        O metodo realiza a validacao dos registros do mapa fiscal para garantir que os valores
        informados estejam fechando, ou seja, GT_FINAL - GT_INICIAL - CANCELAMENTOS - DESCONTOS -
        TOTAL DA BASE seja igual a zero.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificdor do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :return: booleano True caso nao possua divergencia nos valores, senao, raise para impedir a
        persistencia dos dados e informar o usuario quais mapas estao com divergencia.
        """
        context = context and context.copy() or {}
        res = True
        obj_ecf_mapa_resumo_itens = self.pool.get('ecf.mapa.resumo.itens')
        lista = ['coo', 'gt_final', 'gt_inicial', 'cancelamentos', 'descontos', 'item_ids']
        ecf_mapa_resumo_capa_dados = self.read(cr, uid, ids, lista, context=context)
        msg = ''
        for mapa in ecf_mapa_resumo_capa_dados:
            coo = mapa.get('coo')
            gt_final = mapa.get('gt_final') or 0
            gt_inicial = mapa.get('gt_inicial') or 0
            cancelamentos = mapa.get('cancelamentos') or 0
            descontos = mapa.get('descontos')
            item_ids = mapa.get('item_ids')
            ecf_mapa_resumo_itens_dados = obj_ecf_mapa_resumo_itens.read(
                cr, uid, item_ids, ['base_valor'], context=context)
            total_base_valor = sum(map(lambda i: i.get('base_valor') or 0,
                                       ecf_mapa_resumo_itens_dados))
            diferenca = round((gt_final - gt_inicial - cancelamentos - descontos -
                               total_base_valor), 2)
            if diferenca:
                msg += '\n %s' % coo
        if msg:
            raise osv.except_osv(_('Atencao'), _('Ha divergencia nos valores das reducoes '
                                                 'abaixo:\n %s' % msg))
        return res

