#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv


class wizard_listagem_participacao_vendas(osv.TransientModel):
    _name = 'wizard.listagem.participacao.vendas'
    _inherit = 'wizard.listagem.participacao.vendas'

    def get_sql(self, cr, uid, company_ids, categoria_id, opcao, data_inicial, data_final,
                agrupar_sem_grade, context=None):
        """
        Retorna o sql para movimentacao de cupons.
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param categoria_id: id da categoria a filtrar
        :param opcao: Opcao a listar ('I', 'Itens'), ('C', 'Categorias'), ('F', 'Fabricante'),
                      ('V', 'Vendedores')
        :param data_inicial: Data inicial para filtro
        :param data_final: Data final para filtro
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type company_ids: list
        :type categoria_id: int, long
        :type opcao: str
        :type data_inicial: str
        :type data_final: str
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        sql = super(wizard_listagem_participacao_vendas, self).get_sql(cr, uid, company_ids,
                                                                       categoria_id, opcao,
                                                                       data_inicial, data_final,
                                                                       agrupar_sem_grade,
                                                                       context=None)
        campos = ''
        categoria_ids = []
        context = context and context.copy() or {}
        if company_ids and categoria_id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)
        elif categoria_id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)

        company_ids = ''' AND emc.company_id IN %s''' % str(company_ids).replace('[',
                                                                                 '(').replace(
            ']',
            ')') \
            if company_ids else ''' '''
        where_categoria = ''' '''
        join_categoria = ''' '''
        if categoria_ids:
            join_categoria = ''' INNER JOIN estoque_categoria      AS ec  ON ec.id = ep.categoria_id'''
            where_categoria = '''AND ec.id IN %s''' % str(categoria_ids).replace('[',
                                                                                 '(').replace(
                ']', ')')

        join = ''' '''
        if opcao == 'I':
            # Listar por Itens
            campos = '''
                           ep.codigo                      AS codigo,
                           ep.name                        AS produto_name'''
            if not agrupar_sem_grade:
                campos += ''', epg.grade                      AS grade
                             , epg.id                         AS grade_id'''
            join = '''INNER JOIN estoque_produto_grade  AS epg ON epg.id = emi.grade_id'''
        elif opcao == 'C':
            # Listar por Categoria
            campos = '''
                           ep.categoria_id                AS categoria_id,
                           (SELECT conta
                            FROM estoque_categoria
                            WHERE id = ep.categoria_id
                            LIMIT 1)                      AS conta,
                           (SELECT name
                            FROM estoque_categoria
                            WHERE id = ep.categoria_id
                            LIMIT 1)                      AS categoria_name,
                           'C'                            AS opcao '''

        elif opcao == 'F':
            # Listar por Fabricante
            campos = '''
                           ep.fabricante_id               AS fabricante_id,
                           (SELECT codigo
                            FROM adm_pessoa
                            WHERE id = ep.fabricante_id
                            LIMIT 1)                      AS fabricante_codigo,
                           (SELECT nome
                            FROM adm_pessoa
                            WHERE id = ep.fabricante_id
                            LIMIT 1)                      AS fabricante_name,
                           'F'                            AS opcao '''

        elif opcao == 'V':
            # Listar por  Vendedor
            campos = '''
                           adp.nome                       AS vendedor,
                           'V'                            AS opcao '''
            join = '''LEFT  JOIN vendas_vendedor        AS vv  ON vv.id = emi.vendedor_id
                          LEFT  JOIN adm_pessoa             AS adp ON adp.id = vv.pessoa_id'''

        sql += '''
                            UNION ALL
                            SELECT (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.vr_total_item_rateado
                                         ELSE  emi.vr_total_item_rateado
                                    END) AS vr_total_item_rateado,
                                   (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.vr_total_item
                                         ELSE  emi.vr_total_item
                                    END) AS vr_total_item,
                                   (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.quantidade
                                         ELSE  emi.quantidade
                                    END) AS quantidade,
                                   (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.estmvi_saidas_custo_reposicao_total
                                         ELSE  emi.estmvi_saidas_custo_reposicao_total
                                    END) AS custo_reposicao,
                                   (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.estmvi_saidas_ultimo_custo_total
                                         ELSE  emi.estmvi_saidas_ultimo_custo_total
                                    END) AS ultimo_custo,
                                   (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.estmvi_saidas_custo_corrigido_total
                                         ELSE  emi.estmvi_saidas_custo_corrigido_total
                                    END) AS custo_corrigido,
                                   (CASE WHEN ecc.tipo_cupom = 'C'
                                         THEN -emi.estmvi_saidas_custo_medio_contabil_total
                                         ELSE  emi.estmvi_saidas_custo_medio_contabil_total
                                    END) AS custo_medio,
                                   %s
                            FROM estoque_movimento_itens      AS emi
                            INNER JOIN estoque_movimento_capa AS emc ON emc.id = emi.capa_id
                            INNER JOIN fiscal_cfop            AS fc  ON fc.id  = emi.cfop_id
                            INNER JOIN ecf_cupons_capa        AS ecc ON ecc.id = emi.cupom_id
                            LEFT  JOIN estoque_produto        AS ep  ON ep.id  = emi.produto_id
                            %s
                            %s
                            WHERE emc.data >= '%s'
                              AND emc.data <= '%s'
                              AND fc.movimento_tipo = 'C'
                              AND fc.tipo = 'S'
                              AND emi.cupom_id IS NOT NULL
                              AND not (emi.item_cancelado is True AND  ecc.tipo_cupom = 'V')
                              AND emc.state = 'F'
                              %s
                              %s
                      ''' % (campos, join, join_categoria, data_inicial, data_final, company_ids,
                             where_categoria)
        return sql
