# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


# noinspection PyUnresolvedReferences,PyPep8Naming
class wizard_listagem_totais_por_categoria_osv(osv.TransientModel):
    _name = 'wizard.listagem.totais.por.categoria.osv'
    _inherit = 'wizard.listagem.totais.por.categoria.osv'

    def sql_report(self, dt_inicio, dt_fim, company, local):
        """
        Acrescenta na sentenca SQL de consulta do relatorio os movimentos de cupom.

        :param dt_inicio: data inicial da consulta
        :param dt_fim: data final utilizada na consulta
        :param company: lista com o ID das empresas
        :param local: lista com o ID dos locais (locacao)
        :type dt_inicio: str
        :type dt_fim: str
        :type company: list[long]
        :type local: list[long]
        :return: a sentenca SQL utilizada na consulta do relatorio.
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        sql = super(wizard_listagem_totais_por_categoria_osv, self).sql_report(
            dt_inicio, dt_fim, company, local)
        sql += """
               UNION
               """
        args = dict(dt_inicio=dt_inicio, dt_fim=dt_fim,
                    company=', '.join([str(_id)for _id in company]))
        cond_local = ''

        if local:
            cond_local = """
                  AND emi.local_id IN (%s) """ % ', '.join([str(_id) for _id in local])

        args = dict(args, local=cond_local)
        sql += """
              SELECT  'S'                             AS estmpg_tipo_movimento
                    , emi.produto_id                  AS estmpg_produto_id
                    , ep.categoria_id                 AS estmpg_categoria_id
                    , emi.quantidade                  AS estmpg_quantidade
                    , CASE WHEN cc.tipo_cupom = 'C' THEN
                            -emi.vr_total_item_rateado
                      ELSE
                            emi.vr_total_item_rateado
                      END                             AS estmpg_vr_total_item
                    , 'U'                             AS estmpg_cfop_movimento
                    , emi.id                          AS estmpg_movimento_id
                    , CASE WHEN cc.tipo_cupom = 'C' THEN
                           -emi.estmvi_saidas_custo_corrigido_total
                      ELSE
                           emi.estmvi_saidas_custo_corrigido_total
                      END                             AS estmpg_custo_corrigido
                    , CASE WHEN cc.tipo_cupom = 'C' THEN
                           -emi.estmvi_saidas_custo_medio_contabil_total
                      ELSE
                           emi.estmvi_saidas_custo_medio_contabil_total
                      END                             AS estmpg_custo_medio_contabil
                    , CASE WHEN cc.tipo_cupom = 'C' THEN
                           -emi.estmvi_saidas_custo_reposicao_total
                      ELSE
                           emi.estmvi_saidas_custo_reposicao_total
                      END                             AS estmpg_custo_reposicao
                    , CASE WHEN cc.tipo_cupom = 'C' THEN
                           -emi.estmvi_saidas_ultimo_custo_total
                      ELSE
                           emi.estmvi_saidas_ultimo_custo_total
                      END                             AS estmpg_ultimo_custo
                    , (SELECT SUM (fnit.vr_encargo)
                       FROM fiscal_nf_item_tributacao            AS fnit
                       INNER JOIN fiscal_encargo                 AS fe   ON fe.id = fnit.encargo_id
                       WHERE fnit.movimento_itens_id = emi.id
                         AND fe.sigla IN ('PIS','COFINS', 'ICMS')) AS estmp_valor_tributos
                    , (SELECT SUM (fnit.vr_icms_substituicao)
                       FROM fiscal_nf_item_tributacao            AS fnit
                       INNER JOIN fiscal_encargo                 AS fe   ON fe.id = fnit.encargo_id
                       WHERE fnit.movimento_itens_id = emi.id
                         AND fe.sigla IN ('PIS','COFINS', 'ICMS')) AS estmp_icms_substituicao
             FROM   estoque_movimento_itens           AS emi
                INNER JOIN ecf_cupons_capa            AS cc  ON cc.id = emi.cupom_id
                INNER JOIN estoque_produto            AS ep  ON ep.id = emi.produto_id
                INNER JOIN estoque_movimento_capa     AS emc ON emc.id = emi.capa_id
              WHERE  cc.state = 'V'
                AND  cc.date_cupom           BETWEEN '%(dt_inicio)s' AND '%(dt_fim)s'
                AND  cc.company_id           IN  (%(company)s)
                AND  NOT (emi.item_cancelado IS TRUE AND cc.tipo_cupom = 'V')
                AND  emc.state = 'F'
                AND  cc.tipo_cupom           IN ('V', 'C')
                %(local)s
                """ % args

        return sql

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
