# -*- encoding: utf-8 -*-
##############################################################################
#                                                                            #
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca        #
#                    www.siginformatica.com.br luiz@siginformatica.com.br    #
#                                                                            #
##############################################################################

from openerp.osv import osv
from openerp.osv.fields import float_round


class gerencial_movimento(osv.Model):
    """
    Classe gerencial_movimento
    prefixo:fingmo_
    """

    _inherit = 'gerencial.movimento'

    def fingmo_get_valor_classificacao_origem(self, cr, uid, ids, context=None):
        """
        O metodo apura os valores das contas envolvidas no DRE. A apuracao e realizada no regime de
        fato gerador com classificacao de origem.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: valor total apurado das contas envolvidas no DRE.
        """
        context = context and context.copy() or {}
        vr_total = super(gerencial_movimento, self).fingmo_get_valor_classificacao_origem(
            cr, uid, ids, context)
        brw_gerencial_movimento = self.browse(cr, uid, ids, context)
        for movimento in brw_gerencial_movimento:
            titulo_ids = movimento.tesouraria_movimento_id.fintem_titulo_ids
            if not titulo_ids:
                continue
            titulo_ids = filter(lambda i: i.fatura_id, titulo_ids)
            titulo_ids = filter(lambda i: i.fatura_id.cupom_id, titulo_ids)
            for titulo in titulo_ids:
                cupom_id = titulo.fatura_id.cupom_id.id
                vr_titulo = titulo.vr_original or 0.0
                vr_cupom = titulo.fatura_id.cupom_id.total_cupom or 0.0
                if round(vr_titulo, 2) != round(vr_cupom, 2):
                    proporcao = vr_titulo / vr_cupom
                    movimento_ids = self.search(cr, uid, [('fingmo_cupom_id', '=', cupom_id)],
                                                context=context)
                    sum_mov = sum(map(lambda i: i.valor * proporcao,
                                      self.browse(cr, uid, movimento_ids, context)))
                    vr_total += float(float_round(sum_mov, 2))
                else:
                    vr_total += vr_cupom

        return vr_total

    def fingmo_get_gerencial_movimento_classificacao_origem_nivel_acima(self, cr, uid, ids,
                                                                        context=None):
        """
        O metodo busca o gerencial movimento dos titulos originados de cupons.
        E usado para o regime de caixa com classificacao de origem.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: registros da tabela desse objeto.
        :rtype: list
        """
        context = context and context.copy() or {}
        gerencial_movimento_ids, outra_conta_gerencial_movimento_ids = super(
            gerencial_movimento, self).\
            fingmo_get_gerencial_movimento_classificacao_origem_nivel_acima(cr, uid, ids, context)
        brw_gerencial_movimento = self.browse(cr, uid, ids, context)
        for movimento in brw_gerencial_movimento:
            titulo_ids = movimento.tesouraria_movimento_id.fintem_titulo_ids
            titulo_ids = filter(lambda i: i.fatura_id, titulo_ids)
            titulo_ids = filter(lambda i: i.fatura_id.cupom_id, titulo_ids)
            if titulo_ids:
                gerencial_movimento_ids.append(movimento.id)
        return list(set(gerencial_movimento_ids)), outra_conta_gerencial_movimento_ids
