# -*- coding: utf-8 -*-

from openerp.osv import osv


class participacao_clientes_fornecedores_osv(osv.TransientModel):

    _inherit = 'participacao.clientes.fornecedores.osv'

    def get_query_dados(self, cr, uid, company_ids, data_inicial, data_final, tipo_titulo,
                        context=None):
        """
        O metodo forma a query sql para a busca dos dados dos titulos. Essa sobreescricao considera
        os titulos originadados de cupons e notas fiscais.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param company_ids: lista de id da tabela res_company, identificadores das empresas.
        :type company_ids: list
        :param data_inicial: descricao da data inicial do periodo de pesquisa.
        :type data_inicial: str
        :param data_final: descricao da data final do periodo de pesquisa.
        :type data_final: str
        :param tipo_titulo: caracter de identificação do tipo do titulo. 'R' a receber e 'P' a
        pagar.
        :type tipo_titulo: str
        :param context: dados com valores conxtuais.
        :type context: dict
        :return: query sql
        :rtype: str
        """
        context = context and context.copy() or {}
        super(participacao_clientes_fornecedores_osv, self).get_query_dados(
            cr, uid, company_ids, data_inicial, data_final, tipo_titulo, context)
        sql = """
            SELECT
                  DISTINCT
                      CASE
                          WHEN ff.nf_id IS NOT NULL AND nf.destinatario_id IS NOT NULL
                              THEN nf.destinatario_id
                          WHEN ff.nf_id IS NOT NULL AND nf.destinatario_id IS NULL
                              THEN NULL
                          WHEN ff.cupom_id IS NOT NULL AND ecf.cliente_id IS NOT NULL
                              THEN ecf.cliente_id
                          WHEN ff.cupom_id IS NOT NULL AND ecf.cliente_id IS NULL
                              THEN NULL
                          ELSE f.pessoa_id
                      END            AS p_id,
                  SUM(f.vr_original) AS valor
            FROM  financeiro_titulo f
            LEFT JOIN financeiro_fatura ff
                ON ff.id = f.fatura_id
            LEFT JOIN fiscal_nf_capa nf
                ON nf.id = ff.nf_id
            LEFT JOIN fiscal_cfop_natureza fcn
                ON fcn.id = nf.natureza_id
            LEFT JOIN fiscal_cfop cfop
                ON fcn.cfop_id = cfop.id
            LEFT JOIN ecf_cupons_capa ecf
                ON ecf.id = ff.cupom_id
            WHERE f.data_emissao BETWEEN DATE('{data_ini}') AND   DATE('{data_fim}')
              AND f.tipo_titulo = '{tipo_titulo}'
              AND f.company_id in ({company_id})
              AND CASE
                      WHEN ff.nf_id IS NOT NULL AND 'R' = '{tipo_titulo}'
                          THEN nf.state = '3' AND fcn.tipo = 'S' AND cfop.tipo = 'S'
                               AND cfop.movimento_tipo = 'C' AND cfop.codigo NOT IN ('5929', '6929')
                      WHEN ff.nf_id IS NOT NULL AND 'P' = '{tipo_titulo}'
                          THEN nf.state = '3' AND fcn.tipo = 'E' AND cfop.tipo = 'E'
                               AND cfop.movimento_tipo = 'C'
                      ELSE TRUE
                  END
            GROUP BY p_id
            ORDER BY valor desc
        """.format(company_id=','.join(map(str, company_ids)),
                   data_ini=data_inicial,
                   data_fim=data_final,
                   tipo_titulo=tipo_titulo)
        return sql
