#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from functools import wraps
from openerp.osv.fields import float_round
from datetime import datetime


class fiscal_nf_capa(osv.Model):
    _name = 'fiscal.nf.capa'
    _inherit = 'fiscal.nf.capa'

    def write(self, cr, uid, ids, vals, context=None):
        if vals.get('fisnfc_cupons_referenciados_ids'):
            context['referencia_nf_cupom'] = True
        res = super(fiscal_nf_capa, self).write(cr, uid, ids, vals, context)
        return res

    def button_rascunho(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).button_rascunho(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def button_validar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).button_validar(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def button_faturado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).button_faturado(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def button_cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).button_cancelar(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)
        obj_ecf_cupons_capa = self.pool.get('ecf.cupons.capa')
        cupom_ids = obj_ecf_cupons_capa.search(cr, uid, [('nf_id', 'in', ids)], context=context)
        if cupom_ids:
            obj_ecf_cupons_capa.write(cr, uid, cupom_ids, {'nf_id': False}, context)

        return result

    def button_validado_com_erro(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).button_validado_com_erro(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def fisnfc_nfe_denegada(self, cr, uid, ids, tipo_movimento, modelo_nf_series_id, nfe_status,
                            nfe_chave_acesso, nfe_xml_proc, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).fisnfc_nfe_denegada(
            cr, uid, ids, tipo_movimento, modelo_nf_series_id, nfe_status, nfe_chave_acesso,
            nfe_xml_proc, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def inutilizado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).inutilizado(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def aguarda_inutilizacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).aguarda_inutilizacao(cr, uid, ids, context)
        self.atualiza_status_ecf(cr, uid, ids, context)

        return result

    def atualiza_status_ecf(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        state = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context).state

        if context.get('cancelar'):
            state = 'C'

        dados_itens_ids = self.read(
            cr, uid, ids[0] if isinstance(ids, list) else ids, ['estoque_movimento_itens_ids'],
            context=context)

        if isinstance(dados_itens_ids, (list, tuple)):
            dados_itens_ids = dados_itens_ids[0]

        if isinstance(dados_itens_ids, dict):
            dados_itens_ids = dados_itens_ids['estoque_movimento_itens_ids']

        obj_ecf_capa = self.pool.get('ecf.cupons.capa')
        brw_movimento_itens = self.pool.get('estoque.movimento.itens').browse(
            cr, uid, dados_itens_ids, context)

        for x in brw_movimento_itens:
            if x.cupom_id:
                vals = {'flag_status_nf': state}
                obj_ecf_capa.write(cr, uid, x.cupom_id.id, vals, context)

        return True

    def get_observacao_titulo(self, cr, uid, parcela_id, context=None):
        """
        Retorna o numero da autorizacao do cartao na observacao do titulo.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type parcela_id: long
        :type context: dict
        :return: o numero da autorizacao do cartao.
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).get_observacao_titulo(cr, uid, parcela_id, context)

        brw_fiscal_nf_parcela = self.pool.get('fiscal.nf.parcelas').browse(
            cr, uid, parcela_id, context)

        if brw_fiscal_nf_parcela.nf_condicao_id:
            brw_condicao_forma = brw_fiscal_nf_parcela.nf_condicao_id.condicao_forma_id

            if brw_condicao_forma.tipo_finalizadora == 'C':
                result = _('No. da autorizacao do cartao: %s') % \
                         brw_fiscal_nf_parcela.nf_condicao_id.fiscpg_nro_autorizacao_cartao

        return result

    def gera_parcelas_nf_cupom(self, cr, user, nf, context=None):
        """
        Gera parcelas para cada condicao de pagamento

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf: Id da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: Formato O2m para alteracao da condicao incluindo as parcelas
        :rtype: list(tuple(dict)))

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_adm_condicao = self.pool.get('adm.condicao')
        parcelas = []
        nf_condicao_ids = self.browse(cr, user, nf, context=context).nf_condicao_ids
        for condicao in nf_condicao_ids:
            adm_condicao_id = condicao.condicao_forma_id.adm_condicao_id.id
            vals = {}
            for parcela in obj_adm_condicao.gerar_parcelas(cr, user, adm_condicao_id,
                                                           condicao.valor,
                                                           context=context):
                vals['vencimento'] = parcela.get('vencimento')
                vals['valor'] = parcela.get('valor')
                vals['nf_condicao_id'] = condicao.id
                parcelas.append((1, condicao.id, {'nf_parcela_ids': [(0, 0, vals)]}))

        return parcelas

    def gerar_nf_cupom(self, cr, uid, cupons, context=None):
        """
        Este metodo e responsavel por gerar nota fiscal de cupons
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param cupons: lista de ids dos cupons a serem gerados na nota
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type cupons: list
        :type context: dict
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        #  Todo: Verificar a possibilidade de remover os contextos
        context['gerar_nf_cupom'] = True
        context['proprio'] = True  # Utilizado para definir nota fiscal propria
        context['nf_from_cupom'] = True
        context['nota_fiscal_saida'] = True
        load = '_classic_write'
        obj_ecf = self.pool.get('ecf.cupons.capa')
        cupons_dados = obj_ecf.read(
            cr, uid, cupons, ['cliente_id', 'ecf_cupons_item_ids', 'cupom_fiscal_parcelas_ids',
                              'date_cupom', 'tipo_cupom', 'ecf_id', 'company_id', 'coo',
                              'total_cupom'],
            context=context, load=load)
        self._check_cupons_venda(cupons_dados)
        self._check_itens_cupons(cupons_dados)
        # Recupera o cliente para a nota
        clientes = self._get_clientes_cupons(cupons_dados)
        pessoa_id = False
        if isinstance(clientes, list) and len(clientes) == 1:
            pessoa_id = clientes[0]
        fiscal_parametros = self._get_parametros(
            cr, uid, modulo='fiscal', campos=['cupom_nota_natureza_id',
                                              'cupom_nota_uf_fora_natureza_id',
                                              'cupom_nota_condicao_id'],
            context=context)
        natureza_id = self._get_natureza_nf(cr, uid, fiscal_parametros, pessoa_id,
                                            company_id=self._get_company_id(cupons_dados),
                                            context=context)
        self._check_encargos_natureza(cr, uid, natureza_id, context=context)
        condicao_id = self._get_condicao_nf(fiscal_parametros)
        nf_condicao_ids = self._get_condicoes(cr, uid, condicao_id, cupons_dados, context=context)
        # Gera os itens da nota fiscal
        itens = self._get_dados_itens_nf_cupom(cr, uid, cupons, context=context)

        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        vals_itens = []
        if isinstance(cupons_dados, list):
            cupom_id = cupons_dados[0].get('id')
        else:
            cupom_id = cupons_dados.get('id')

        produto_ids = []
        grade_ids = []
        for i in itens:
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))

        dict_precos =  self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid, produto_ids,
                                                           epc_grade_ids=grade_ids, context=context)
        for item in itens:
            produto_id=item.get('produto_id')
            local_id=item.get('local_id')
            grade_id=item.get('grade_id')
            produto_embalagem_id=item.get('produto_embalagem_id')
            quantidade=item.get('quantidade')
            vr_desconto=item.get('vr_desconto', 0.0)
            vr_unitario=item.get('vr_unitario', 0.0)
            vr_total_item=item.get('vr_total_item_rateado', 0.0)
            context['fiscal'] = True
            vals_item = {
                'produto_id': produto_id,
                'local_id': local_id,
                'quantidade': quantidade,
                'vr_desconto': vr_desconto,
                'produto_embalagem_id': produto_embalagem_id,
                'grade_id': grade_id,
                'cupom_id': cupom_id,
                'tributacoes_ids': item.get('tributacoes_ids')
            }
            #Calculando o valor unitario pela embalagem do produto
            if not vr_unitario:
                dados_preco_venda = dict_precos.get(produto_id) or {}
                vr_unitario = dados_preco_venda.get(grade_id) or \
                              dados_preco_venda.get('preco_venda')
                if vr_unitario and produto_embalagem_id:
                    embalagem_dados = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                                                                                   produto_embalagem_id)
                    embalagem = embalagem_dados.embalagem_id
                    qt_emb = 1.0
                    if embalagem:
                        qt_emb = embalagem.quantidade_na_embalagem or 1.0

                    vr_unitario *= qt_emb

            vals_item['vr_unitario'] = vr_unitario

            if not vr_total_item:
                vr_total_item = float(float_round((quantidade * vr_unitario - vr_desconto), 2))

            vals_item['vr_total_item'] = vr_total_item
            if vals_item.get('estmvi_volume'):
                vals_item['vr_total_item'] *= vals_item['estmvi_volume']


            #grade
            if not grade_id:
                grade_id = obj_estoque_produto_grade.search(cr, uid, [('produto_id', '=', produto_id)],
                                                            context=context)
                if grade_id:
                    vals_item['grade_id'] = grade_id[0]

            vals_itens.append([0, 0, vals_item])

        # Gera a capa da nota fiscal
        nf_id = self._gerar_capa_nf(cr, uid, company_id=self._get_company_id(cupons_dados),
                                    destinatario_id=pessoa_id, natureza_id=natureza_id,
                                    nf_condicao_ids=nf_condicao_ids, tipo='S',
                                    estoque_movimento_itens_ids=vals_itens,context=context)

        # Atualiza nf com as mensagens ao contribuinte
        info_contrib = []
        info_contribuintes = obj_ecf.get_msg_contribuinte(cr, uid, cupons, context=context)
        for i in info_contribuintes:
            info_contrib.append(info_contribuintes.get(i))

        text = '\n'.join(info_contrib)
        info_contribuinte = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid, text)
        vals = {'fisnfc_infcomp_contribuinte': info_contribuinte}
        context['super_write_fiscal_nf_capa'] = True
        self.write(cr, uid, nf_id, vals, context=context)
        # Atualiza referencia da nf nos cupons
        return obj_ecf.write(cr, uid, cupons, {'nf_id': nf_id,
                                               'flag_status_nf': '',
                                               'fisnfc_nf_referencia_id': nf_id}, context=context)

    @staticmethod
    def _check_cupons_venda(cupons):
        """
        Este metodo e responsavel por checar se todos os cupons sao de venda
        caso nao seja gera um raise
        :param cupons: Lista de dicionarios contendo os dados dos cupons
        :type cupons: list
        :raise: Se existe cupom diferente de Venda
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if filter(lambda i: i.get('tipo_cupom') != 'V', cupons):
            raise osv.except_osv(_('Atencao'),
                                 _('Somente Cupons do tipo VENDA podem gerar Notas Fiscais!'))

    @staticmethod
    def _check_itens_cupons(cupons):
        """
        Este metodo e responsavel por checar se existe item nos cupons, caso nao exista
        gera um raise mostrando a informacao
        :param cupons: Lista de dicionarios contendo os dados dos cupons
        :type cupons: list
        :raise: Se existe cupom sem item
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if filter(lambda i: i.get('ecf_cupons_item_ids') is None, cupons):
            raise osv.except_osv(_('Atencao'), _('O cupom nao possui NENHUM ITEM cadastrado!'))

    @staticmethod
    def _get_clientes_cupons(cupons):
        """
        Este metodo gera uma lista com os clientes para os cupons informados
        :param cupons: Lista de dicionarios contendo os dados dos cupons
        :type cupons: list
        :rtype: list
        :return: Retorna uma lista contendo os clientes dos cupons informados
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return list(set(map(lambda i: i.get('cliente_id'), cupons)))

    def _check_parametro(method):
        """
        Metodo responsavel por checar se nao existe o modulo retornado pelo metodo _get_parametros
        Caso nao exista gera um raise informando. Se existir, apenas retorna o retorno do metodo
        _get_parametros
        :raise: Se nao existir o parametro informado
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        @wraps(method)
        def wrapper(*args, **kwargs):
            r = method(*args, **kwargs)
            if not r:
                modulo = kwargs.get('mudulo')
                raise osv.except_osv(_('Atencao'), _('Parametro do {modulo} nao cadastrado!'
                                                     ''.format(modulo=modulo)))
            return r
        return wrapper

    @_check_parametro
    def _get_parametros(self, cr, uid, modulo=None, campos=None, context=None):
        """
        Metodo responsavel por retornar os parametros do modulo informado
        :param cr: Cursor do open
        :param uid: Id do usuario do open
        :param modulo: modulo a retornar as parametros
        :param campos: lista de campos a serem retornados
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type modulo: str
        :type campos: list
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo os valores dos campos informados no parametro
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return self.pool.get('adm.utils').admutl_get_parametros(cr, uid, modulo, campos,
                                                                context=context)

    def _get_natureza_nf(self, cr, uid, fiscal_parametro_dados, pessoa_id, company_id,
                         context=None):
        """
        O metodo encontra a natureza para a nota fiscal, se para dentro do estado ou fora, de acordo
        com as UF do destinatario (pessoa_id) e da empresa (company_id).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fiscal_parametro_dados: dicionario com a descricao e os respectivos valores dos
        parametros do fiscal.
        :type fiscal_parametro_dados: dict
        :param pessoa_id: id da tabela adm_pessoa.
        :type pessoa_id: int
        :param company_id: id da tabela res_company.
        :type company_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: id da tabela fiscal_cfop_natureza
        :rtype: int
        """
        natureza_id = fiscal_parametro_dados.get('cupom_nota_natureza_id')
        if not pessoa_id:
            return natureza_id

        context['super_read_adm_pessoa'] = True
        uf_pessoa = self.pool.get('adm.pessoa').read(cr, uid, pessoa_id, ['uf'], context).get('uf')
        emp_pessoa_id = self.pool.get('res.company').read(cr, uid, company_id, ['rescon_pessoa_id'],
                                             context, load='_classic_write').get('rescon_pessoa_id')
        uf_empresa = self.pool.get('adm.pessoa').read(cr, uid, emp_pessoa_id, ['uf'],
                                                      context).get('uf')
        apd_ids = self.pool.get('adm.pessoa.endereco').search(cr, uid,
                                                   [('pessoa_id', '=', pessoa_id)], context=context)
        uf_id = self.pool.get('adm.pessoa.endereco').read(cr, uid, apd_ids[0], ['uf_id'], context,
                                                          load='_classic_write').get('uf_id')
        uf_pessoa = self.pool.get('adm.uf').read(cr, uid, uf_id, ['sigla'], context).get('sigla')


        if uf_empresa != uf_pessoa:
            natureza_id = fiscal_parametro_dados.get(
                'cupom_nota_uf_fora_natureza_id')
        return natureza_id

    @staticmethod
    def _get_condicao_nf(parametro):
        """
        Metodo responsavel por retornar a condicao padrao para nf de cupom de acordo com o parametro
        do fiscal.
        :param parametro: Contem os parametros do modulo fiscal retornados do metodo _get_parametros
        :raise: Se nao existir condicao cadastrada para emissao de nf de cupom
        :rtype: int
        :return: Retorna o ID da condicao de pagamento padrao para emissao de nota fiscal de cupom
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if not parametro.get('cupom_nota_condicao_id'):
            raise osv.except_osv(_('Atencao'), _(
                'CONDICAO DE PAGAMENTO utilizada pelo Cupom para a geracao de NF nao esta '
                'cadastrada no Parametro do Fiscal'))
        return parametro.get('cupom_nota_condicao_id')

    def _get_condicoes(self, cr, uid, condicao_id, cupons, context=None):
        """
        Metodo responsavel por gerar as condicoes de pagamento da nota fiscal de cupom
        :param cr: Cursor do openerp
        :param uid: ID do usuario do openerp
        :param condicao_id: ID da condicao de pagamento de acordo com o parametro do fiscal
        :param cupons: Lista de dicionarios contendo os dados dos cupons
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type condicao_id: int
        :type cupons: list
        :type context: dict
        :rtype: list
        :return Retorna uma lista no formato O2M contendo as condicoes de pagamento da nf de cupom
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_adm_condicao_forma = self.pool.get('adm.condicao.forma')
        browse_condicao_forma = obj_adm_condicao_forma.browse(cr, uid, condicao_id, context=context)
        valor_condicao = self._get_total_cupons(cupons)
        data_emissao = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
            cr, uid, datetime.now(), 'UTC', context=context), '%Y-%m-%d')
        parcelas = [[0, 0, p] for p in self.pool.get('adm.condicao').gerar_parcelas(
                    cr, uid, browse_condicao_forma.adm_condicao_id.id, valor_condicao, data_emissao,
                    context)]
        vals = dict(forma_pagto_id=browse_condicao_forma.forma_pagamento_id.id,
                    condicao_forma_id=condicao_id, valor=valor_condicao, nf_parcela_ids=parcelas)
        res = [(0, 0, vals)]
        return res

    @staticmethod
    def _get_total_cupons(cupons):
        """
        Metodo responsavel por retornar o somatorio dos cupons informados para a nota fiscal
        :param cupons: Lista de dicionarios contendo os dados dos cupons
        :type cupons: list
        :rtype: float
        :return: Retorna o somatorio dos cupons informados para a nota fiscal
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return round(sum(map(lambda i: i.get('total_cupom') or 0.0, cupons)), 2)

    def _check_encargos_natureza(self, cr, uid, natureza, context=None):
        """
        Metodo responsavel por verificar se a natureza cadastrada no parametro do fiscal, para
        notas de cupons, possui Encargos cadastrados
        :param cr: Cursor do openerp
        :param uid: ID do usuario do openerp
        :param natureza: ID da natureza para gerar a nfe de cupom
        :param context: Dicionario contendo os valores contextuais
        :type cr: Cursor
        :type uid: int
        :type natureza: int
        :type context: dict
        :raise: Se nao existe encargos cadastrados para a natureza de nf de cupom
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not self.pool.get('fiscal.cfop.natureza.encargos').search(
                cr, uid, [('natureza_id', '=', natureza)], context=context):
            raise osv.except_osv(_('Atencao'),
                                 _('A Natureza da Operacao, nao possui encargos cadastrados'))

    def _gerar_capa_nf(self, cr, uid, **kwargs):
        """
        Metodo responsavel por gerar a capa da nata fiscal de cupom
        :rtype: int
        :return: Retorna o ID da nota fiscal criada
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return self.pool.get('fiscal.utils').gerar_nf(cr, uid, **kwargs)

    def _gerar_itens_nf(self, cr, uid, nf_id, itens, context=None):
        """
        Metodo responsavel por gerar os itens da nota fiscal de cupom
        :param cr: Cursor do openerp
        :param uid: ID do usuario do openerp
        :param nf_id: ID da nota fiscal
        :param itens: Dicionario contendo os dados dos itens a serem gerados na nota
        :param context: Dicionario contendo os valores contextuais
        :type cr: Cursor
        :type uid: int
        :type nf_id: int | long
        :type itens: list
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fiscal_utils = self.pool.get('fiscal.utils')
        for item in itens:
            obj_fiscal_utils.gerar_nf_itens(
                cr, uid, nf_id=nf_id, produto_id=item.get('produto_id'), tipo='P',
                local_id=item.get('local_id'), grade_id=item.get('grade_id'),
                produto_embalagem_id=item.get('produto_embalagem_id'),
                quantidade=item.get('quantidade'), vr_desconto=item.get('vr_desconto', 0.0),
                vr_unitario=item.get('vr_unitario', 0.0), sequencia=itens.index(item) + 1,
                vr_total_item=item.get('vr_total_item_rateado', 0.0), context=context)

    def _get_dados_itens_nf_cupom(self, cr, uid, cupom_ids, context=None):
        """
        Metodo responsavel por buscar as informacoes dos itens que devem ser gerados na nota fiscal
        de cupom
        :param cr: Cursor do openerp
        :param uid: ID do usuario do banco
        :param cupom_ids: IDs dos cupons informados para a nota fiscal
        :param context: Dicionario contendo valores conextuais
        :type cr: Cursor
        :type uid: int
        :type cupom_ids: list
        :type context: dict
        :rtype: list
        :return: Retorna uma lista contendo os dicionarios com os valores dos itens a serem gerados
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        itens_ids = self.pool.get('estoque.movimento.itens').search(
            cr, uid, [('cupom_id', 'in', cupom_ids), ('item_cancelado', '!=', True),
                      ('cupom_item_id', '!=', False)], context=context)
        itens_ids = str(itens_ids).replace('[', '(').replace(']', ')')
        sql = """
                 SELECT  emi.produto_id,
                         emi.produto_embalagem_id,
                         emi.grade_id,
                         emi.local_id,
                         Sum(emi.quantidade)            AS quantidade,
                         Sum(emi.vr_desconto)           AS desconto,
                         emi.vr_unitario                AS vr_unitario,
                         Sum(emi.vr_acrescimos)         AS vr_acrescimos,
                         Sum(emi.vr_total_item_rateado) AS vr_total_item_rateado,
                         trib.encargo_id,
                         trib.cst_id,
                         trib.tributacao_id,
                         trib.aliquota,
                         trib.retido,
                         trib.percentual_mva,
                         trib.modbcst,
                         trib.aliquota_venda,
                         trib.tipo_substituicao,
                         trib.fisnit_csosn_id,
                         trib.fisnit_nat_rec_pis_cofins,
                         trib.fisnit_tipo_movimento_cst,
                         trib.fisnit_tipo_cst,
                         trib.fisnit_cfop_id,
                         SUM(trib.vr_base_calculo)            AS vr_base_calculo,
                         SUM(trib.vr_encargo)                 AS vr_encargo,
                         SUM(trib.vr_reducao_base)            AS vr_reducao_base,
                         SUM(trib.valor_pauta)                AS valor_pauta,
                         SUM(trib.vr_base_icms_substituicao)  AS vr_base_icms_substituicao,
                         SUM(trib.vr_icms_substituicao)       AS vr_icms_substituicao,
                         SUM(trib.vr_icms_substituicao_guia)  AS vr_icms_substituicao_guia,
                         SUM(trib.fisnit_vr_contabil)         AS fisnit_vr_contabil,
                         SUM(trib.fisnit_vr_tributado)        AS fisnit_vr_tributado,
                         SUM(trib.fisnit_vr_isento)           AS fisnit_vr_isento,
                         SUM(trib.fisnit_vr_outros)           AS fisnit_vr_outros,
                         SUM(trib.fisnit_vr_diferido)         AS fisnit_vr_diferido,
                         SUM(trib.fisnit_vr_substituicao)     AS fisnit_vr_substituicao,
                         SUM(trib.fisnit_vr_servico)          AS fisnit_vr_servico,
                         SUM(trib.fisnit_vr_difal_uf_origem)  AS fisnit_vr_difal_uf_origem,
                         SUM(trib.fisnit_vr_difal_uf_destino) AS fisnit_vr_difal_uf_destino,
                         SUM(trib.fisnit_vr_fcp)              AS fisnit_vr_fcp,
                         SUM(trib.fisnit_vr_fcp_st)           AS fisnit_vr_fcp_st,
                         SUM(trib.fisnit_vr_icms_desonerado)  AS fisnit_vr_icms_desonerado
                                      
                            FROM     estoque_movimento_itens     AS emi
                            INNER JOIN fiscal_nf_item_tributacao AS trib ON trib.movimento_itens_id = emi.id
                            WHERE    emi.id IN {itens_ids}
                            group BY emi.produto_id,
                                     emi.produto_embalagem_id,
                                     emi.grade_id,
                                     emi.local_id,
                                     emi.vr_unitario,
                                     
                                     trib.encargo_id,
                                     trib.cst_id,
                                     trib.tributacao_id,
                                     trib.aliquota,
                                     trib.retido,
                                     trib.percentual_mva,
                                     trib.modbcst,
                                     trib.aliquota_venda,
                                     trib.tipo_substituicao,
                                     trib.fisnit_csosn_id,
                                     trib.fisnit_nat_rec_pis_cofins,
                                     trib.fisnit_tipo_movimento_cst,
                                     trib.fisnit_tipo_cst,
                                     trib.fisnit_cfop_id

        """.format(itens_ids=itens_ids)
        cr.execute(sql)
        dados = cr.dictfetchall()
        aux = {}
        for i in dados:
            chave = '%s-%s-%s-%s-%s-%s'  % (i.get('produto_id'),
                                            i.get('produto_embalagem_id'),
                                            i.get('grade_id'),
                                            i.get('local_id'),
                                            i.get('vr_unitario'),
                                            i.get('fisnit_tipo_movimento_cst'))
            vals_trib = {'encargo_id': i.get('encargo_id'),
                         'cst_id': i.get('cst_id'),
                         'tributacao_id': i.get('tributacao_id'),
                         'aliquota': i.get('aliquota'),
                         'retido': i.get('retido'),
                         'percentual_mva': i.get('percentual_mva'),
                         'modbcst': i.get('modbcst'),
                         'aliquota_venda': i.get('aliquota_venda'),
                         'tipo_substituicao': i.get('tipo_substituicao'),
                         'fisnit_csosn_id': i.get('fisnit_csosn_id'),
                         'fisnit_nat_rec_pis_cofins': i.get('fisnit_nat_rec_pis_cofins'),
                         'fisnit_tipo_movimento_cst': i.get('fisnit_tipo_movimento_cst'),
                         'fisnit_tipo_cst': i.get('fisnit_tipo_cst'),
                         'fisnit_cfop_id': i.get('fisnit_cfop_id'),
                         'vr_base_calculo': i.get('vr_base_calculo'),
                         'vr_encargo': i.get('vr_encargo'),
                         'vr_reducao_base': i.get('vr_reducao_base'),
                         'valor_pauta': i.get('valor_pauta'),
                         'vr_base_icms_substituicao': i.get('vr_base_icms_substituicao'),
                         'vr_icms_substituicao': i.get('vr_icms_substituicao'),
                         'vr_icms_substituicao_guia': i.get('vr_icms_substituicao_guia'),
                         'fisnit_vr_contabil': i.get('fisnit_vr_contabil'),
                         'fisnit_vr_tributado': i.get('fisnit_vr_tributado'),
                         'fisnit_vr_isento': i.get('fisnit_vr_isento'),
                         'fisnit_vr_outros': i.get('fisnit_vr_outros'),
                         'fisnit_vr_diferido': i.get('fisnit_vr_diferido'),
                         'fisnit_vr_substituicao': i.get('fisnit_vr_substituicao'),
                         'fisnit_vr_servico': i.get('fisnit_vr_servico'),
                         'fisnit_vr_difal_uf_origem': i.get('fisnit_vr_difal_uf_origem'),
                         'fisnit_vr_difal_uf_destino': i.get('fisnit_vr_difal_uf_destino'),
                         'fisnit_vr_fcp': i.get('fisnit_vr_fcp'),
                         'fisnit_vr_fcp_st': i.get('fisnit_vr_fcp_st'),
                         'fisnit_vr_icms_desonerado': i.get('fisnit_vr_icms_desonerado')}
            if not aux.get(chave):
                aux[chave] = {'produto_id': i.get('produto_id'),
                              'produto_embalagem_id': i.get('produto_embalagem_id'),
                              'grade_id': i.get('grade_id'),
                              'local_id': i.get('local_id'),
                              'quantidade': i.get('quantidade'),
                              'desconto': i.get('desconto'),
                              'vr_unitario': i.get('vr_unitario'),
                              'vr_acrescimos': i.get('vr_acrescimos'),
                              'vr_total_item_rateado': i.get('vr_total_item_rateado'),
                              'tributacoes_ids': [[0, 0, vals_trib]]
                              }
            else:
                aux[chave]['tributacoes_ids'].append([0, 0, vals_trib])

        return aux.values()

    def _get_msg_contribuinte(self, cr, uid, cupons, context=None):
        """
        Metodo responvel por gerar as mensagens dos contribuintes na nota fiscal de cupom
        :param cr: Cursor do openerp
        :param uid: ID do usuario do banco
        :param cupons: IDs dos cupons informados para a nota fiscal
        :param context: Dicionario contendo valores conextuais
        :type cr: Cursor
        :type uid: int
        :type cupons: list
        :type context: dict
        :rtype: str
        :return: Retorna a mensagem a ser gravada na aba informacoes complementares
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        msg = ''.join(self.pool.get('ecf.cupons.capa').get_msg_contribuinte(
            cr, uid, cupons, context=context).values())
        return msg

    @staticmethod
    def _get_company_id(cupons):
        """
        Metodo responsavel por retornar a company_id dos cupons a serem gerados para a nota fiscal
        :param cupons: Lista de dicionarios contendo os dados do cupons informados para a nota
        :rtype: int
        :return: Retorna a company_id a ser gerada para a nota fiscal
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        company_id = map(lambda i: i.get('company_id'), cupons)
        return company_id[0] if company_id and isinstance(company_id, list) else company_id

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
