# -*- encoding: utf-8 -*-

from openerp.osv import osv


class fiscal_nf_item_tributacao(osv.Model):
    _inherit = 'fiscal.nf.item.tributacao'

    def ecf_fisnit_calcula_metodos_funcionais(self, cr, uid, context=None):
        super(fiscal_nf_item_tributacao, self).ecf_fisnit_calcula_metodos_funcionais(cr, uid, context=context)
        sql = '''SELECT fnit.id,
                        vr_base_calculo                AS base_calculo,
                        vr_reducao_base                AS base_reducao,
                        cst.tipo                       AS cst_tipo,
                        emi.cfop_id,
                        emi.vr_total_item_rateado      AS valor_contabil,
                        'E'                            AS nf_tipo
                 FROM fiscal_nf_item_tributacao     AS fnit
                 INNER JOIN fiscal_cst              AS cst  ON cst.id = fnit.cst_id
                 INNER JOIN estoque_movimento_itens AS emi  ON emi.id = fnit.movimento_itens_id
                 WHERE emi.nf_id IS NULL
                   AND emi.cupom_id IS NOT NULL
                 ORDER BY id
              '''
        cr.execute(sql)
        return cr.dictfetchall()

    def fisnit_get_vr_fcp(self, cr, uid, dados_fisnit, context=None):
        """
        Este metodo retorna o valor do FCP de caso o destinatario seja consumidor final
        :return: Retorna o valor do FCP caso o destinatario seja um consumidor final
        :rtype: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if dados_fisnit.get('fisnit_capa_id'):
            return super(fiscal_nf_item_tributacao, self).fisnit_get_vr_fcp(
                cr, uid, dados_fisnit, context=context)

        aliquota_fcp = dados_fisnit.get('aliquota_fcp')
        base_fcp = dados_fisnit.get('vr_base_calculo')
        return self.fisnit_calcula_vr_fcp(base_fcp, aliquota_fcp)

    def fisnit_get_vr_fcp_st(self, cr, user, dados_fisnit, context=None):
        """
        Calcula o valor do FCP/ST para consumidor final
        
        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao do perfil
        :param tributacao: o ID da tributacao (fiscal.nf.item.tributacao)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type tributacao: long 
        :type context: dict
        :return: o valor do FCP/ST (consumidor final)
        :rtype: float
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        if dados_fisnit.get('fisnit_capa_id'):
            return super(fiscal_nf_item_tributacao, self).fisnit_get_vr_fcp_st(
                cr, user, dados_fisnit, context)

        base = dados_fisnit.get('vr_base_icms_substituicao')
        aliquota = dados_fisnit.get('fistri_aliquota_fcp')

        return base * aliquota / 100.

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
