# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class wizard_gerar_dapi(osv.TransientModel):
    # prefixo: wizdapi_
    _inherit = 'wizard.gerar.dapi'

    def wizdapi_get_movimento_dados(self, cr, uid, periodo_inicial, periodo_final, context=None):
        """
        O metodo busca o movimento de itens do estoque para os cupons fiscais de venda e
        cancelamento validos. Esses dizem respeito somente ao ICMS.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param periodo_inicial: data inicial para a busca do movimento.
        :type: datetime
        :param periodo_final: data final para a busca do movimento.
        :type: datetime
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dados da movimentacao de acordo com o periodo definido.
        :rtype: list[dict]
        """
        context = context and context.copy() or {}
        res = super(wizard_gerar_dapi, self).wizdapi_get_movimento_dados(
            cr, uid, periodo_inicial, periodo_final, context)
        sql = """
            SELECT
                cfop.fiscfop_dapi                         AS dapi,
                cfop.tipo                                 AS cfop_tipo,
                CASE
                    WHEN cfop.tipo = 'E' AND cfop.movimento_tipo = 'C'
                        THEN 'Compras'
                    WHEN cfop.tipo = 'S' AND cfop.movimento_tipo = 'C'
                        THEN 'Vendas'
                    WHEN cfop.movimento_tipo = 'D'
                        THEN 'Devolucao'
                    WHEN cfop.movimento_tipo = 'T'
                        THEN 'Transferencia'
                    WHEN cfop.movimento_tipo = 'E'
                        THEN 'Remessa'
                    WHEN cfop.movimento_tipo = 'R'
                        THEN 'Retorno'
                    WHEN cfop.movimento_tipo = 'U'
                        THEN 'Outras'
                END                                       AS cfop_movimento,
                CASE
                    WHEN cfop.produto_servico_tipo = 'P'
                        THEN 'Mercadoria / Produto'
                    WHEN cfop.produto_servico_tipo = 'F'
                        THEN 'Transporte'
                    WHEN cfop.produto_servico_tipo = 'M'
                        THEN 'Comunicacao'
                    WHEN cfop.produto_servico_tipo = 'S'
                        THEN 'Servico'
                    WHEN cfop.produto_servico_tipo = 'E'
                        THEN 'Energia Eletrica'
                    WHEN cfop.produto_servico_tipo = 'L'
                        THEN 'Combustivel / Lubrificante'
                    WHEN cfop.produto_servico_tipo = 'O'
                        THEN 'Outros'
                END				          AS cfop_transacao_des,
                cfop.produto_servico_tipo                 AS cfop_transacao,
                Sum(
                    CASE
                        WHEN movimento.item_cancelado IS TRUE
                            THEN movimento.vr_total_item_rateado * -1
                        ELSE movimento.vr_total_item_rateado
                    END
                )                                         AS vr_contabil,
                Sum(
                    CASE
                        WHEN movimento.item_cancelado IS TRUE
                            THEN item_tributacao.vr_base_calculo * -1
                        ELSE item_tributacao.vr_base_calculo
                    END
                )                                         AS vr_base_calculo,
                Sum(
                    CASE
                        WHEN movimento.item_cancelado IS TRUE
                            THEN item_tributacao.vr_encargo * -1
                        ELSE item_tributacao.vr_encargo
                    END
                )                                         AS vr_encargo,
                0			                  AS vr_imposto_sem_ap_credito,
                Sum(
                    CASE
                        WHEN cst.tipo = 'I' AND movimento.item_cancelado IS TRUE
                            THEN movimento.vr_total_item_rateado * -1
                        WHEN cst.tipo = 'I' AND movimento.item_cancelado IS NOT TRUE
                            THEN movimento.vr_total_item_rateado
                        ELSE 0
                    END
                )                                         AS vr_isento_nao_tributado,
                Sum(
                    CASE
                        WHEN movimento.item_cancelado IS TRUE
                            THEN item_tributacao.vr_reducao_base *-1
                        ELSE item_tributacao.vr_reducao_base
                    END
                )			                  AS vr_base_reduzida,
                Sum(
                    CASE
                        WHEN cst.tipo = 'D' AND movimento.item_cancelado IS TRUE
                            THEN movimento.vr_total_item_rateado * -1
                        WHEN cst.tipo = 'D' AND movimento.item_cancelado IS NOT TRUE
                            THEN movimento.vr_total_item_rateado
                        ELSE 0
                    END
                )                                         AS vr_diferido,
                0                                         AS vr_suspenso,
                Sum(
                    CASE
                        WHEN movimento.item_cancelado IS TRUE
                            THEN item_tributacao.vr_icms_substituicao * -1
                        ELSE item_tributacao.vr_icms_substituicao
                    END
                )                                         AS vr_icms_substituicao,
                Sum(
                    CASE
                        WHEN cst.tipo = 'O' AND movimento.item_cancelado IS TRUE
                            THEN movimento.vr_total_item_rateado * -1
                        WHEN cst.tipo = 'O' AND movimento.item_cancelado IS NOT TRUE
                            THEN movimento.vr_total_item_rateado
                        ELSE 0
                    END
                )                                         AS vr_outros,
                Sum(COALESCE(item_tributacao.fisnit_vr_fcp, 0))        AS vr_fcp
            FROM
                estoque_movimento_itens movimento
            INNER JOIN ecf_cupons_capa cup
                ON cup.id = movimento.cupom_id
            INNER JOIN fiscal_nf_item_tributacao item_tributacao
                ON movimento.id = item_tributacao.movimento_itens_id
            INNER JOIN fiscal_cfop cfop
                ON cfop.id = movimento.cfop_id
            INNER JOIN fiscal_encargo encargo
                ON encargo.id = item_tributacao.encargo_id
            INNER JOIN fiscal_cst cst
                ON cst.id = item_tributacao.cst_id
            WHERE
                cup.date_cupom BETWEEN '{periodo_inicial}' AND '{periodo_final}'
                AND cup.state = 'V'
                AND cup.tipo_cupom IN ('V', 'C')
                AND encargo.sigla = 'ICMS'
                AND cfop.fiscfop_dapi IS NOT NULL
                AND cfop.codigo NOT IN ('5929', '6929')
                AND NOT (cup.tipo_cupom = 'V' AND movimento.item_cancelado IS TRUE)
            GROUP BY
                cfop.fiscfop_dapi,
                cfop.tipo,
                cfop.produto_servico_tipo,
                cfop.movimento_tipo
        """.format(periodo_inicial=periodo_inicial, periodo_final=periodo_final)
        cr.execute(sql)
        dados = cr.dictfetchall()
        res.extend(dados)
        return res
