# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
import psycopg2
import psycopg2.extras
from ....adm import print_log
from datetime import datetime
from decimal import Decimal
import time
TWOPLACES = Decimal(10) ** -2
mostrar_logs = True


class wizard_importar_nfce(osv.TransientModel):
    '''
    Classe para importar NFC-e emitida pelo Frente de loja
    Prefixo: wizimn
    '''
    _name = 'wizard.importar.nfce'

    def cron_importar_nfce(self, cr, uid, context=None):
        _tinicio = time.time()
        print_log(_('Cron Importar nfce-Inicio .......'))
        context = context and context.copy() or {}
        sql = """
            SELECT ep.company_id,
                   ep.estpar_super_host,
                   ep.estpar_super_db,
                   ep.estpar_super_user,
                   ep.estpar_super_password,
                   ep.estpar_usar_master,
                   ep.dir_arq_import,
                   ep.estpar_tipo_exportacao,
                   fp.fispar_nfce_natureza_id,
                   fp.id                      AS par_id,
                   fcn.fisnat_operacao_id
            FROM estoque_parametro          AS ep
            LEFT JOIN fiscal_parametro      AS fp  ON fp.company_id = ep.company_id
            LEFT JOIN fiscal_cfop_natureza  AS fcn ON fcn.id = fp.fispar_nfce_natureza_id
        """
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            print_log(_("Sem Parametros de ecf cadastrados no Estoque"))
            return True

        try:
            for i in dados:
                if i.get('estpar_tipo_exportacao') == 'A':  # por arquivo
                    self.wizimn_importar_nfce_arquivo(cr, uid, i, context)
                else:  # por banco de dados
                    self.wizimn_importar_nfce_supervisor(cr, uid, i, context)
        except Exception as e:
            try:
                print_log(_('Cron Importar nfce-Fim - Com erro: %s' % e))
            except:
                print_log(_('Cron Importar nfce-Fim - Com erro'))
        _tfim = time.time()
        print_log(_('Cron Importar nfce-Fim ....... Tempo: %s') % (_tfim - _tinicio))
        return True

    def wizimn_importar_nfce_arquivo(self, cr, uid, parametros, context=None):
        context = context and context.copy() or {}
        return True

    def wizimn_importar_nfce_supervisor(self, cr, uid, parametros, context=None):
        context = context and context.copy() or {}
        company_id = parametros.get('company_id')
        natureza_id = parametros.get('fispar_nfce_natureza_id')
        operacao_id = parametros.get('fisnat_operacao_id')
        host = parametros.get('estpar_super_host')
        if not host:
            return True
        db = parametros.get('estpar_super_db')
        user = parametros.get('estpar_super_user')
        password = parametros.get('estpar_super_password')
        usar_master = parametros.get('estpar_usar_master')
        conn = False
        if mostrar_logs:
            print_log(_('----Conectar supervisor empresa: %s host: %s') % (company_id, host))
            _tinicio = time.time()
        try:
            # CONEXAO COM O BANCO DE DADOS
            conn = psycopg2.connect(database=db, user=user, password=password, host=host, connect_timeout=3)
            _cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
        except:
            if conn:
                conn.close()
            print_log(_('Nao foi possivel conectar ao supervisor com host: %s e banco de '
                        'dados: %s. Verifique as informacoes nos parametros do Estoque' %
                        (host, db)))
            return True

        if mostrar_logs:
            print_log(_('----Conectado - %s') % (time.time() - _tinicio))
            _tinicio = time.time()
        _cr.execute("SET statement_timeout = '10s'")  # setando timeout para as querys do supervisor
        try:
            dados_nfce = self.wizimn_get_dados_capa(cr, _cr, usar_master, company_id, context)
        except Exception as e:
            try:
                erro = str(e)
            except:
                erro = e.message

            if mostrar_logs:
                _dif = time.time() - _tinicio
                print_log(_('----Erro ao pegar NFCes: Tempo:%s - Erro:%s') % (_dif, erro))
            erro = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid, erro).replace("'", '')
            cr.execute("""INSERT INTO fiscal_log_nfce
                                      (company_id, fislon_nfce_numero, fislon_nfce_caixa, 
                                       fislon_nfce_erro, fislon_tipo, create_date)
                          VALUES ({company_id}, {numero}, {caixa}, '{erro}', '{tipo}', '{create_date}');
                          COMMIT;
                       """.format(company_id=company_id,
                                  numero=999999,
                                  caixa=999,
                                  erro=erro,
                                  tipo='N',
                                  create_date=datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                       )
            dados_nfce = None
        if not dados_nfce:
            if conn:
                conn.close()
            if mostrar_logs:
                print_log(_('----Nenhuma nota a importar: Tempo:%s') % (time.time() - _tinicio))
            return True
        if mostrar_logs:
            print_log(_('----Identificado as NFC-Es - %s') % (time.time() - _tinicio))
            _tinicio = time.time()

        context_force = context.copy()
        context_force['sig_force_company'] = True
        tz_admin = self.pool.get('res.users').read(cr, uid, uid, ['tz'], context).get('tz')
        uid = self.pool.get('res.users').search(cr, uid, [('login','=','ecf_remoto')],
                                                context=context_force)[0]
        tz_remoto = self.pool.get('res.users').read(cr, uid, uid, ['tz'], context).get('tz')
        vals_remoto = {'company_id': company_id}
        if not tz_remoto or tz_admin != tz_remoto:
            vals_remoto['tz'] = tz_admin

        self.pool.get('res.users').write(cr, uid, uid, vals_remoto, context=context_force)
        if mostrar_logs:
            print_log(_('----Atualizado o usuario - %s') % (time.time() - _tinicio))
            _tinicio = time.time()
        dados_user = self.pool.get('res.users').read(cr, uid, uid, ['login', 'company_id'], context,
                                                     load='_classic_write')
        if company_id != dados_user.get('company_id'):
            print_log(_('- Erro ao importar Notas: Usuario "%s" nao possui permissao cadastrada para '
                        'empresa %s\n\n' % (dados_user.get('login'), company_id)))
            if conn:
                conn.close()
            return True

        obj_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_log_nfce = self.pool.get('fiscal.log.nfce')
        nfce_ids = map(lambda x: '%s' % x.get('id'), dados_nfce)
        _tInicio = datetime.now()
        print_log(_('- Importando NFC-e do supervisor(%s)......... Qtds: %s' % (db, len(nfce_ids))))
        _tinicio = time.time()
        dict_corpos = self.wizimn_get_dados_corpos(cr, uid, _cr, nfce_ids, company_id,  context)
        if mostrar_logs:
            print_log(_('----Peguei dados do corpo - %s') % (time.time() - _tinicio))
            _tinicio = time.time()
        dict_finalizacao = self.wizimn_get_dados_finalizacao(_cr, nfce_ids, context)
        if mostrar_logs:
            print_log(_('----Peguei dados da finalizacao - %s') % (time.time() - _tinicio))
            _tinicio = time.time()
        _qtd = 0
        for i in dados_nfce:
            try:
                if mostrar_logs:
                    _qtd += 1
                    print_log(_('--Importando %s-serie/nota %s-%s  Tempo:%s') %
                              (_qtd, i.get('supcap_caixa'), i.get('supcap_nro_cupom'), (time.time() - _tinicio)))
                    _tinicio_nfce = _tinicio = time.time()
                if i.get('supcap_state') in ('2E', '4', '6'):
                    i['supcap_state'] = '3'

                elif i.get('supcap_state') == '7':
                    i['supcap_state'] = 'C'

                sql_begin = """BEGIN; SAVEPOINT importar_nfce;"""
                cr.execute(sql_begin)  # erp
                _cr.execute(sql_begin)  # supervisor
                serie = int(i.get('supcap_caixa'))
                ambiente = i.get('supcap_ambiente') == 'P' and 1 or 2
                obj_serie = self.pool.get('fiscal.modelo.nf.series')
                obj_modelo = self.pool.get('fiscal.modelo.nf')
                codigo_modelo = '65'
                if i.get('supcap_tipo') == 'D':
                    codigo_modelo = '91'

                modelo_id = obj_modelo.search(cr, uid, [('fismnf_codigo', '=', codigo_modelo)],
                                              context=context)
                if not modelo_id:
                    raise Exception('Modelo %s nao foi esta cadastrado!' % codigo_modelo)

                modelo_id = modelo_id[0]
                serie_id = obj_serie.search(cr, uid, [('modelo_nf_id', '=', modelo_id),
                                                      ('serie_nf', '=', '%03d' % serie),
                                                      ('ambiente', '=', ambiente),
                                                      ('state', '=', 'A'),
                                                      ('parametro_id', '=',
                                                       parametros.get('par_id'))],
                                            context=context)
                if not serie_id:
                    nome_ambiente = ambiente == 1 and _('Producao') or _('Homologacao')
                    raise Exception(_('A Serie %03d Ambiente %s Status "Ativo" '
                                      'na Empresa %s nao esta cadastrada') %
                                    (serie, nome_ambiente, company_id))

                    serie_id = obj_serie.create(cr, uid, vals, context)

                serie_id = serie_id[0]
                nf_faturada_id = False
                # nota que foi importada antes de ser faturada ou cacelada no supervisor
                if i.get('supcap_state') in ('3', 'C'):
                    args = [('numero_nf', '=', i.get('supcap_nro_cupom')),
                            ('serie_nf', '=', serie),
                            ('modelo_nf_id', '=', modelo_id),
                            ('modelo_nf_series_id', '=', serie_id),
                            ('company_id', '=', company_id)]
                    nf_faturada_id = obj_nf_capa.search(cr, uid, args, context=context)

                i['company_id'] = company_id
                capa_id = i.get('id')
                nova_capa_id = False
                if not nf_faturada_id:
                    if mostrar_logs:
                        print_log(_('----Vou inserir nova nota - %s') % (time.time() - _tinicio))
                        _tinicio = time.time()
                    corpos = dict_corpos.get(capa_id)
                    nova_capa_id = self.wizimn_create_nfce(cr, uid, i, company_id, natureza_id,
                                                           modelo_id, serie_id, context)
                    if mostrar_logs:
                        print_log(_('----Capa da nfce criada! - %s') % (time.time() - _tinicio))
                        _tinicio = time.time()
                    # caso a nota ja tenha sido importada, nao importa novamente.
                    if not nova_capa_id:
                        sql = """UPDATE supervisor_mov_cupom_servidor
                                                 SET supcse_enviado = True
                                                 WHERE supcse_capa_id = %s""" % capa_id
                        _cr.execute(sql)
                        sql_commit = """RELEASE SAVEPOINT importar_nfce;
                                                        COMMIT;"""
                        cr.execute("""DELETE FROM fiscal_log_nfce
                                                      WHERE fislon_nfce_numero = %s
                                                        AND fislon_nfce_caixa = %s
                                                        AND company_id = %s""" % (
                                                                        i.get('supcap_nro_cupom'),
                                                                        i.get('supcap_caixa'),
                                                                        company_id))
                        cr.execute(sql_commit)
                        _cr.execute(sql_commit)
                        continue

                    mov_capa_id = False
                    if i.get('supcap_state') == '3':
                        mov_capa_id = self.wizimn_create_mov_capa(cr, uid, i, nova_capa_id,
                                                                  company_id, operacao_id, context)
                    if mostrar_logs:
                        print_log(_('----Capa do estoque criada! - %s') % (time.time() - _tinicio))
                        _tinicio = time.time()

                    local_id = obj_serie.read(cr, uid, serie_id, ['fismns_local_id'], context, '_classic_write').get('fismns_local_id')
                    self.wizimn_create_itens(_cr, cr, uid, natureza_id, local_id, company_id,
                                             nova_capa_id, mov_capa_id, corpos, i, context)
                    if mostrar_logs:
                        print_log(_('----Itens do estoque criados! - %s') % (time.time() - _tinicio))
                        _tinicio = time.time()
                    finalizacoes = dict_finalizacao.get(capa_id)
                    self.wizimn_create_condicao(cr, uid, nova_capa_id, finalizacoes, i, context)
                    if mostrar_logs:
                        print_log(_('----Condicoes da nfce criada! - %s') % (time.time() - _tinicio))
                        _tinicio = time.time()
                    context['super_write_fiscal_nf_capa'] = True
                    context['fisnfc_nao_faturar_sefaz'] = True
                    context['fisnfc_importacao_nfce_do_supervisor'] = True
                    context['nota_fiscal_consumidor'] = True
                    obj_nf_capa.write(cr, uid, nova_capa_id, {}, context)
                    if mostrar_logs:
                        print_log(_('----Vou faturar a nfce! - %s') % (time.time() - _tinicio))
                        _tinicio = time.time()
                    if i.get('supcap_state') == '3':
                        obj_nf_capa.fisnfc_funcao_button_faturar(cr, uid, [nova_capa_id], context)
                        if mostrar_logs:
                            print_log(_('----nfce faturada! - %s') % (time.time() - _tinicio))
                            _tinicio = time.time()

                elif i.get('supcap_state') == '3':
                    xml_proc = i.get('supcap_xml_proc') or ''
                    if "\'" in xml_proc:
                        xml_proc = xml_proc.replace('\'', "''")

                    xml_gerado = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid,
                                                                                      i.get('supcap_xml_gerado') or '')
                    status = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid, (i.get('supcap_status') or '')[:254])
                    vals = {'nfe_lote': i.get('supcap_lote'),
                            'nfe_xml_auto': xml_proc or None,
                            'nfe_protocolo': i.get('supcap_protocolo'),
                            'nfe_xml_gerado': xml_gerado,
                            'state': '3',
                            'nfe_status': (status or '').replace("'", '').replace('"', '')}

                    context_aux = context.copy()
                    context_aux['super_write_fiscal_nf_capa'] = True
                    obj_nf_capa.write(cr, uid, nf_faturada_id,  vals, context_aux)

                elif i.get('supcap_state') == 'C':
                    # cancela a nota quando a mesma foi faturada e deu baixa no financeiro
                    context_cancelar = context.copy()
                    context_cancelar['cancelamento_nf'] = True
                    obj_nf_capa.fisnfc_funcao_button_cancelar(cr, uid, nf_faturada_id, context_cancelar)
                    campos = ['nfe_lote', 'nfe_protocolo', 'nfe_xml_gerado','nfe_xml_auto']
                    dados_cancelada = obj_nf_capa.read(cr, uid, nf_faturada_id, campos, context)[0]
                    vals = {'nfe_protocolo_cancelamento': i.get('supcap_protocolo_cancelamento'),
                            'nfe_xml_cancelamento': i.get('supcap_xml_cancelamento'),
                            'nfe_status': (i.get('supcap_status') or '')[:254]}
                    if not dados_cancelada.get('nfe_lote'):
                        vals['nfe_lote'] = i.get('supcap_lote')
                    if not dados_cancelada.get('nfe_protocolo'):
                        vals['nfe_protocolo'] = i.get('supcap_protocolo')
                    if not dados_cancelada.get('nfe_xml_gerado'):
                        vals['nfe_xml_gerado'] = i.get('supcap_xml_gerado')
                    if not dados_cancelada.get('nfe_xml_auto'):
                        vals['nfe_xml_auto'] = i.get('supcap_xml_auto')

                    context_aux = context.copy()
                    context_aux['super_write_fiscal_nf_capa'] = True
                    obj_nf_capa.write(cr, uid, nf_faturada_id,  vals, context_aux)

                if i.get('supcap_numero_pedido') and nova_capa_id:
                    self.wizimn_fatura_pedido(cr, uid, i, company_id, nova_capa_id, context)

                sql = """UPDATE supervisor_mov_cupom_servidor
                         SET supcse_enviado = True
                         WHERE supcse_capa_id = %s""" % capa_id
                _cr.execute(sql)
                sql_commit = """RELEASE SAVEPOINT importar_nfce;
                                COMMIT;"""
                cr.execute("""DELETE FROM fiscal_log_nfce
                              WHERE fislon_nfce_numero = %s
                                AND fislon_nfce_caixa = %s
                                AND company_id = %s""" % (i.get('supcap_nro_cupom'),
                                                          i.get('supcap_caixa'),
                                                          company_id))
                cr.execute(sql_commit)
                _cr.execute(sql_commit)

            except Exception as e:
                sql_rollback = """ROLLBACK TO SAVEPOINT importar_nfce;
                                  COMMIT;"""
                cr.execute(sql_rollback)
                _cr.execute(sql_rollback)
                try:
                    erro = str(e)
                except:
                    erro = e.message

                if e.__class__ == osv.except_osv:
                    erro = e[0] + '\n' + e[1]

                erro = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid, erro).\
                                                                                    replace("'", '')
                log_ids = obj_log_nfce.search(cr, uid, [('fislon_nfce_numero', '=', i.get('supcap_nro_cupom')),
                                                        ('fislon_nfce_caixa', '=', i.get('supcap_caixa')),
                                                        ('fislon_nfce_erro', '=', erro)], context=context)
                if log_ids:
                    continue

                tipo = 'D' if i.get('supcap_tipo') == 'D' else 'N'
                cr.execute("""INSERT INTO fiscal_log_nfce
                                          (company_id, fislon_nfce_numero, fislon_nfce_caixa, 
                                           fislon_nfce_erro, fislon_tipo)
                              VALUES ({company_id}, {numero}, {caixa}, '{erro}', '{tipo}');
                              COMMIT;
                           """.format(company_id=company_id,
                                      numero=i.get('supcap_nro_cupom'),
                                      caixa=i.get('supcap_caixa'),
                                      erro=erro,
                                      tipo=tipo)
                           )
            print_log(_('------Fim da NFce - %s') % (time.time() - _tinicio_nfce))
        if conn:
            conn.close()
        _tFim = datetime.now()
        print_log(_('- Importando NFC-e do supervisor-Fim......... Tempo: %s' % str(_tFim - _tInicio)))
        return True

    def wizimn_get_dados_capa(self, cr, _cr, usar_master, company_id, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        cond_empresa = ''
        if usar_master:
            cond_empresa = 'AND smc.supcap_nro_empresa = %s' % company_id

        cr.execute("""SELECT fislon_nfce_numero AS nro,
                              fislon_nfce_caixa  AS caixa,  
                              fislon_tipo        AS tipo,  
                              company_id          
                       FROM fiscal_log_nfce
                       WHERE company_id = %s""" % company_id)
        cond_exclui = []
        for i in obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(cr):
            cond_exclui.append('(supcap_nro_cupom = %s AND supcap_caixa = %s)' %
                               (i.get('nro'), i.get('caixa')))
        if cond_exclui:
            _cr.execute("""SELECT id
                           FROM supervisor_mov_capa 
                           WHERE %s
                           AND supcap_state IN ('3', '2E', '4', '6', '7', 'C', 'I', 'D') 
                        """ % '\n OR '.join(cond_exclui))
            cond_exclui = []
            ids = []
            for i in obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr):
                ids.append(str(i.get('id')))
            if ids:
                cond_exclui.append('AND smc.id NOT IN (%s)' % ','.join(ids))

        if cond_exclui:
            cond_exclui = '\n                   '.join(cond_exclui)

        sql = """SELECT smc.id,
                        smc.supcap_nro_cupom,
                        smc.supcap_loja,
                        smc.supcap_caixa,
                        smc.supcap_tipo,
                        smc.supcap_data_cupom,
                        smc.supcap_data_inicio,
                        smc.supcap_cliente,
                        smc.supcap_alteracao,
                        smc.supcap_valor_desc_acres,
                        smc.supcap_total_cupom,
                        smc.supcap_operador_id,
                        smc.supcap_cod_vendedor,
                        scv.name                 AS nome_vendedor,
                        smc.supcap_lote,
                        smc.supcap_status,
                        smc.supcap_xml_proc,
                        smc.supcap_protocolo,
                        smc.supcap_protocolo_cancelamento,
                        smc.supcap_xml_gerado,
                        smc.supcap_xml_cancelamento,
                        smc.supcap_chave_acesso,
                        smc.supcap_chave_cte,
                        smc.supcap_state,
                        smc.supcap_ambiente,
                        smc.supcap_numero_pedido,
                        sco.supope_codigo
                 FROM supervisor_mov_capa                 AS smc
                 INNER JOIN supervisor_mov_cupom_servidor AS smcs ON smcs.supcse_capa_id = smc.id
                 INNER JOIN supervisor_cad_operadores     AS sco  ON sco.id = smc.supcap_operador_id
                 LEFT  JOIN supervisor_cad_vendedor       AS scv  ON scv.supven_codigo = smc.supcap_cod_vendedor
                 WHERE smc.supcap_state IN ('3', '2E', '4', '6', '7', 'C', 'I', 'D')
                   AND smcs.supcse_enviado = FALSE 
                   {cond_exclui}
                   {cond_empresa}
                 ORDER BY smc.supcap_data_cupom
                 LIMIT {limit}
                """
        _cr.execute(sql.format(cond_empresa=cond_empresa, cond_exclui=cond_exclui or '', limit=50))
        dados = obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr)
        dados_log = dados
        qtd_restante = 50 - len(dados)
        if qtd_restante > 0:
            cond_exclui = []
            ids = []
            for i in dados:
                ids.append(str(i.get('id')))
                if ids:
                    cond_exclui.append('AND smc.id NOT IN (%s)' % ','.join(ids))


            if cond_exclui:
                cond_exclui = '\n                   '.join(cond_exclui)

            _cr.execute(sql.format(cond_empresa=cond_empresa, cond_exclui=cond_exclui or '', limit=qtd_restante))
            dados = obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr)
            dados.extend(dados_log[0:qtd_restante])

        return dados

    def wizimn_get_dados_corpos(self, cr, uid, _cr, capa_ids, company_id, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT smc.supcor_capa_id,
                   smc.supcor_item,
                   smc.supcor_preco_unitario,
                   smc.supcor_quantidade,
                   smc.supcor_preco_total,
                   smc.supcor_venda_cancelamento,
                   smc.supcor_tributacao_ecf,
                   smc.supcor_tributacao_retag,
                   smc.supcor_sequencia,
                   smc.supcor_desconto_item,
                   smc.supcor_desconto_item_sem_rat,
                   smc.supcor_acrescimo_item,
                   smc.supcor_acrescimo_item_sem_rat,
                   smc.supcor_aliquota,
                   smc.supcor_base,
                   smc.supcor_cod_vendedor,
                   smc.supcor_nome_vendedor,
                   smc.supcor_vr_fcp_st,
                   smc.supcor_vr_icms,
                   smc.supcor_base_icms,
                   smc.supcor_codigo_trib_icms,
                   smc.supcor_vr_pis,
                   smc.supcor_base_pis,
                   smc.supcor_codigo_trib_pis,
                   smc.supcor_vr_cofins,
                   smc.supcor_base_cofins,
                   smc.supcor_codigo_trib_cofins,
                   smc.supcor_codigo_cfop,
                   sci.supite_codigo_anp        AS codigo_anp,
                   sci.supite_producao          AS producao,
                   sci.id                       AS item_id,
                   icms.suptnf_tipo_tributacao  AS tipo_trib,
                   icms.suptnf_base             AS base_icms,
                   icms.id                      AS icms_id,
                   smca.supcap_data_cupom       AS data_nota
                   
            FROM   supervisor_mov_corpos             AS smc
            LEFT JOIN supervisor_cad_itens           AS sci  ON sci.supite_codigo_item = smc.supcor_item
            INNER JOIN supervisor_mov_capa           AS smca ON smca.id = smc.supcor_capa_id 
            LEFT JOIN supervisor_cad_tributacao_nfce AS icms ON icms.suptnf_codigo = sci.supite_tributacao_icms_id 
            WHERE smc.supcor_capa_id IN (%s)
            ORDER BY smc.supcor_sequencia
        """ % ','.join(capa_ids)
        _cr.execute(sql)
        dict_corpos = {}
        obj_adm_utils = self.pool.get('adm.utils')
        cbs = []  # lista de codigo de barras
        for i in obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr):
            cbs.append('%r' % str(i.get('supcor_item')))
            capa_id = i.get('supcor_capa_id')
            if not dict_corpos.get(capa_id):
                dict_corpos[capa_id] = [i]
            else:
                dict_corpos[capa_id].append(i)

        if not dict_corpos:
            return {}.fromkeys(capa_ids, [])

        dict_cb = {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                       tabela='automacao.codigo.barra', context=context, user_company_id=company_id)
        if cbs:
            cr.execute("""SELECT acb.codigo_barra AS supcor_item,
                                 acb.grade_id,
                                 acb.produto_id,
                                 acb.produto_embalagem_id,
                                 ee.quantidade_na_embalagem AS qtd_embalagem
                          FROM automacao_codigo_barra         AS acb
                          LEFT JOIN estoque_produto_embalagem AS epe ON epe.id = acb.produto_embalagem_id
                          LEFT JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id
                          WHERE acb.codigo_barra IN ({cbs})
                            AND acb.company_id = {company_id}""".format(cbs=','.join(cbs),
                                                                        company_id=company_id)
                       )
            map(lambda x: dict_cb.update({x.get('supcor_item'): x}), cr.dictfetchall())

        for i in dict_corpos.values():
            for j in i:
                dados = dict_cb.get(j.get('supcor_item')) or {}
                j.update(dados)

        return dict_corpos

    def wizimn_get_dados_finalizacao(self, _cr, capa_ids, context=None):
        context = context and context.copy() or {}
        dict_fin = {}
        sql = """
            SELECT smf.id AS fechamento_id,
                   smf.supfec_capa_id,
                   smf.supfec_finalizadora,
                   smf.supfec_nome_finalizadora,
                   smf.supfec_valor,
                   smf.supfec_sequencia, 
                   smf.supfec_tipo_finalizadora, 
                   smf.supfec_troco,
                   smf.supfec_nro_banco, 
                   smf.supfec_agencia, 
                   smf.supfec_nro_cheque,
                   smf.supfec_conta, 
                   smf.supfec_juros, 
                   smf.supfec_cnpj, 
                   smf.supfec_aut_cartao,
                   smv.supfve_valor_parcela,
                   smv.supfve_vencimento,
                   smca.supcap_data_cupom,
                   smca.supcap_loja, 
                   smca.supcap_caixa,
                   smca.supcap_cliente
            FROM supervisor_mov_fechamentos      AS smf
            LEFT JOIN supervisor_mov_vencimentos AS smv  ON smv.supfve_fechamentos_id = smf.id
            INNER JOIN supervisor_mov_capa       AS smca ON smf.supfec_capa_id = smca.id
            WHERE smf.supfec_capa_id IN (%s)
            ORDER BY smf.supfec_capa_id, smf.supfec_sequencia
        """ % ','.join(capa_ids)
        _cr.execute(sql)
        obj_adm_utils = self.pool.get('adm.utils')
        for i in obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr):
            capa_id = i.get('supfec_capa_id')
            if not dict_fin.get(capa_id):
                dict_fin[capa_id] = [i]
            else:
                dict_fin[capa_id].append(i)

        return dict_fin

    def wizimn_create_nfce(self, cr, uid, dados, company_id, natureza_id, modelo_id, serie_id,
                           context=None):
        context = context and context.copy() or {}
        serie = int(dados.get('supcap_caixa'))
        args = [('numero_nf', '=', dados.get('supcap_nro_cupom')),
                ('serie_nf', '=', serie),
                ('modelo_nf_id', '=', modelo_id),
                ('modelo_nf_series_id', '=', serie_id),
                ('company_id', '=', company_id)]
        context_aux = context.copy()
        context_aux['sig_force_company'] = True
        # caso a nota ja exista, nao importa ela novamente pois duplica o movimento
        if self.pool.get('fiscal.nf.capa').search(cr, uid, args, context=context_aux):
            return False

        obj_rh_func = self.pool.get('rh.funcionario')
        obj_vendedor = self.pool.get('vendas.vendedor')
        cpf_cnpj = dados.get('supcap_cliente')
        destinatario_id = 'null'
        if cpf_cnpj:
            pessoa_ids = self.pool.get('adm.pessoa').search(cr, uid,
                                          ['|', ('cpf_cnpj', '=', cpf_cnpj[-11:]),
                                                ('cpf_cnpj', '=', cpf_cnpj[-14:])], context=context)
            if pessoa_ids:
                destinatario_id = pessoa_ids[0]

            if not destinatario_id:
                raise Exception('Cliente nao encontrado com o CPF/CNPJ %s' % cpf_cnpj)

        vendedor_id = 'null'
        if dados.get('supcap_cod_vendedor'):
            ved_ids = obj_vendedor.search(cr, uid, [('id', '=', dados.get('supcap_cod_vendedor'))],
                                          context=context)
            if not ved_ids:
                raise Exception('O vendedor %s-%s nao esta cadastrado no vendas!' %
                                (dados.get('supcap_cod_vendedor'), dados.get('nome_vendedor')))

            vendedor_id = ved_ids[0]

        ambiente = dados.get('supcap_ambiente') == 'P' and 1 or 2
        desconto = 0
        acrescimo = 0
        if dados.get('supcap_alteracao') == 'D' and dados.get('supcap_state') == '3':
            desconto = dados.get('supcap_valor_desc_acres')
        elif dados.get('supcap_alteracao') == 'A' and dados.get('supcap_state') == '3':
            acrescimo = dados.get('supcap_valor_desc_acres')

        text_prod = ''
        text_homo = _('\n**** HOMOLOGACAO ****\n')
        if ambiente == 1:
            text_prod = _('\n**** PRODUCAO ****\n')
            text_homo = ''

        data = datetime.now()
        xml_gerado = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid,
                                                               dados.get('supcap_xml_gerado') or '')
        status = self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid,
                                                                      (dados.get('supcap_status') or '')[:254])
        if not dados.get('supcap_xml_proc') and dados.get('supcap_tipo') not in ('V', 'D', 'I', 'C'):
            raise Exception('Nota %s-%s sem XML assinado!' % (dados.get('supcap_nro_cupom'), serie))

        if dados.get('supcap_tipo') == 'C' and not dados.get('supcap_xml_cancelamento'):
            raise Exception('Nota %s-%s sem XML de cancelamento!' % (dados.get('supcap_nro_cupom'), serie))

        xml_proc = dados.get('supcap_xml_proc') or 'null'
        if "\'" in xml_proc:
            xml_proc = xml_proc.replace('\'', "''")

        if dados.get('supcap_state') not in ('3', 'C'):
            xml_gerado = 'null'
            xml_proc = 'null'

        fisnfc_consumidor = True
        gerar_xml = True
        if dados.get('supcap_tipo') == 'D':
            fisnfc_consumidor = False
            gerar_xml = False

        sql = """INSERT INTO fiscal_nf_capa
                             (create_date, create_uid, write_date, write_uid,
                              fisnfc_tipo_servico, modelo_55, frete, nfe_forma_impressao_danfe,
                              numero_nf, serie_nf, vr_desconto, vr_icms_substituicao,
                              vr_despesas_acessorias, vr_base_icms_substituicao, nf_propria, 
                              data_emissao, hora_emissao, data_saida_entrada, tipo, state, 
                              vr_total_nf, company_id, natureza_id, modelo_nf_series_id, 
                              modelo_nf_id, destinatario_id, fisnfc_consumidor,
                              gerar_xml, finalidade_emissao, fisnfc_nfe_indicador_presenca,
                              vendedor_id,
                              nfe_lote, nfe_status, nfe_xml_auto, nfe_protocolo,
                              nfe_xml_gerado, nfe_chave_acesso, nfe_xml_cancelamento,
                              nfe_processo_emissao, nfe_protocolo_cancelamento,
                              fisnfc_nfe_status_configuracao_producao, 
                              fisnfc_nfe_status_configuracao_homologacao)
                 VALUES ('{create_date}', {create_uid}, '{write_date}', {write_uid},
                          False, False, '9', '4',
                          {numero_nf}, '{serie_nf}', {vr_desconto}, {vr_icms_substituicao},
                          {vr_despesas_acessorias}, {vr_base_icms_substituicao}, {nf_propria}, 
                          '{data_emissao}', '{hora_emissao}', '{data_saida_entrada}', '{tipo}', 
                          '{state}', 
                          {vr_total_nf}, {company_id}, {natureza_id}, {modelo_nf_series_id}, 
                          {modelo_nf_id}, {destinatario_id}, {fisnfc_consumidor},
                          {gerar_xml}, '{finalidade_emissao}', '{fisnfc_nfe_indicador_presenca}',
                          {vendedor_id},
                         '{nfe_lote}', '{nfe_status}', '{nfe_xml_auto}', '{nfe_protocolo}',
                         '{nfe_xml_gerado}', '{nfe_chave_acesso}', '{nfe_xml_cancelamento}',
                         '{nfe_processo_emissao}', '{nfe_protocolo_cancelamento}',
                         '{status_configuracao_prod}', '{status_configuracao_homo}')
                 RETURNING id;
              """.format(create_date=data,
                         create_uid=uid,
                         write_date=data,
                         write_uid=uid,
                         numero_nf=dados.get('supcap_nro_cupom'),
                         serie_nf=serie,
                         vr_desconto=desconto,
                         vr_icms_substituicao=0,
                         vr_despesas_acessorias=acrescimo,
                         vr_base_icms_substituicao=0,
                         nf_propria=True,
                         data_emissao=dados.get('supcap_data_inicio')[:19],
                         hora_emissao=dados.get('supcap_data_inicio')[:19],
                         data_saida_entrada=dados.get('supcap_data_inicio')[:19],
                         tipo='S',
                         state=dados.get('supcap_state'),
                         vr_total_nf=dados.get('supcap_total_cupom') or 0,
                         company_id=company_id,
                         natureza_id=natureza_id or 'null',
                         modelo_nf_series_id=serie_id,
                         modelo_nf_id=modelo_id,
                         destinatario_id=destinatario_id,
                         fisnfc_consumidor=fisnfc_consumidor,
                         gerar_xml=gerar_xml,
                         finalidade_emissao='1',
                         fisnfc_nfe_indicador_presenca='1',
                         vendedor_id=vendedor_id,
                         nfe_lote=dados.get('supcap_lote') or 'null',
                         nfe_xml_auto=xml_proc,
                         nfe_protocolo=dados.get('supcap_protocolo') or 'null',
                         nfe_xml_gerado=xml_gerado or 'null',
                         nfe_chave_acesso=dados.get('supcap_chave_acesso') or 'null',
                         nfe_xml_cancelamento=dados.get('supcap_xml_cancelamento') or 'null',
                         nfe_processo_emissao='0',
                         nfe_protocolo_cancelamento=dados.get('supcap_protocolo_cancelamento') or 'null',
                         nfe_status=(status or '').replace("'", '').replace('"', ''),
                         status_configuracao_prod=text_prod,
                         status_configuracao_homo=text_homo)

        cr.execute(sql.replace("'null'", 'null'))
        capa_id = cr.dictfetchall()
        return capa_id[0].get('id')

    def wizimn_create_mov_capa(self, cr, uid, dados, nf_id, company_id, operacao_id, context=None):
        context = context and context.copy() or {}
        if dados.get('supcap_state') == '3':
            state = 'F'
        elif dados.get('supcap_state') in ('C','D', 'I'):
            state = 'C'

        vals = {'company_id': company_id,
                'nf_capa_id': nf_id,
                'funcionario_id': dados.get('funcionario_id'),
                'estmvc_operacao_id': operacao_id,
                'data': dados.get('supcap_data_cupom'),
                'state': state,
        }
        context['sig_force_company'] = True
        mov_capa_id = self.pool.get('estoque.movimento.capa').create(cr, uid, vals, context=context)
        return mov_capa_id

    def wizimn_create_itens(self, _cr, cr, uid, natureza_id, local_id, company_id,  nf_id,
                            mov_capa_id, corpos, dados_nfce, context=None):
        context = context and context.copy() or {}
        if not corpos or dados_nfce.get('supcap_state') in ('I', 'C') or not mov_capa_id:
            if dados_nfce.get('supcap_state') == '3':
                raise Exception('Nota sem itens')

            return True

        if not local_id:
            raise Exception('Sem local cadastrado no modelo do parametro do fiscal')

        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')
        data = datetime.now()
        sql = """INSERT INTO estoque_movimento_itens
                        (create_date, create_uid, write_date, write_uid,
                         nf_id, sequencia, vr_unitario, vr_desconto, vr_desconto_rateado, 
                         vr_total_item, vr_preco_venda, cfop_id, natureza_id, vr_despesas_rateado,
                         estmvi_vr_despesas_extras_rateado, vr_total_item_rateado,
                         capa_id, produto_id, grade_id, local_id, produto_embalagem_id,
                         quantidade, 
                         estmvi_saidas_custo_corrigido_total,
                         estmvi_saidas_custo_medio_contabil_total,
                         estmvi_saidas_ultimo_custo_total,
                         estmvi_saidas_custo_reposicao_total,
                         estmvi_percentual_comissao,
                         estmvi_vr_comissao,
                         vendedor_id
                        )
                 VALUES %s
                 RETURNING id, sequencia;
              """
        produto_ids = []
        for i in corpos:
            if i.get('supcor_venda_cancelamento') == 'C' or not i.get('produto_id'):
                continue

            produto_ids.append(i.get('produto_id'))

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                       tabela='estoque.preco.custo', context=context, user_company_id=company_id)
        vals_custos = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_ids, company_id,
                                                                i.get('data_nota'), context)
        itens = []
        msg_itens = []
        msg_acb = []
        msg_icms = []
        for i in corpos:
            if i.get('supcor_venda_cancelamento') == 'C':
                continue

            if not i.get('item_id'):
                msg_itens.append(i.get('supcor_item'))
                continue

            if not i.get('icms_id'):
                msg_icms.append(i.get('supcor_item'))
                continue

            if not i.get('produto_id') or not i.get('grade_id'):
                msg_acb.append(i.get('supcor_item'))

            desconto = (i.get('supcor_desconto_item') or 0) - (i.get('supcor_desconto_item_sem_rat') or 0)
            acres = (i.get('supcor_acrescimo_item') or 0) - (i.get('supcor_acrescimo_item_sem_rat') or 0)
            total_item_rat = i.get('supcor_preco_total') - (i.get('supcor_desconto_item') or 0) + \
                         (i.get('supcor_acrescimo_item') or 0)
            total_item = i.get('supcor_preco_total') - (i.get('supcor_desconto_item_sem_rat') or 0) + \
                         (i.get('supcor_acrescimo_item_sem_rat') or 0)
            cfop = i.get('supcor_codigo_cfop')
            cfop_id = self.pool.get('fiscal.cfop').search(cr, uid, [('codigo', '=', cfop)],
                                                          context=context)
            if not cfop_id:
                raise Exception('Sem CFOP no ERP para o item %s-%s' % (i.get('supcor_sequencia'),
                                                                       i.get('supcor_item')))

            cfop_id = cfop_id[0]
            produto_id = i.get('produto_id')
            custos = vals_custos.get(produto_id) or {}
            qtd_embalagem = i.get('qtd_embalagem') or 1
            quantidade = i.get('supcor_quantidade') * qtd_embalagem
            custo_corrigido = (custos.get('estprc_custo_corrigido') or 0) * quantidade
            ultimo_custo = (custos.get('estprc_ultimo_custo') or 0) * quantidade
            custo_reposicao = (custos.get('estprc_custo_reposicao') or 0) * quantidade
            custo_medio_contabil = (custos.get('estprc_custo_medio_contabil') or 0) * quantidade

            dados_comissao = {}
            vendedor_id = 'null'
            if i.get('supcor_cod_vendedor'):
                vendedor_ids = obj_vendas_vendedor.search(
                    cr, uid, [('id', '=', i.get('supcor_cod_vendedor'))], context=context)
                if not vendedor_ids:
                    raise Exception('O vendedor %s-%s nao esta cadastrado no vendas!' %
                                    (i.get('supcor_cod_vendedor'), i.get('supcor_nome_vendedor')))

                vendedor_id = vendedor_ids[0]
                dados_comissao = obj_estoque_movimento_itens.estmvi_get_comissao(
                    cr, uid, produto_id, total_item_rat, vendedor_id, context=context)

            itens.append("""('{create_date}', {create_uid}, '{write_date}', {write_uid},
                              {nf_id}, {sequencia}, {vr_unitario}, {vr_desconto}, 
                              {vr_desconto_rateado}, {vr_total_item}, {vr_preco_venda}, {cfop_id}, 
                              {natureza_id}, {vr_despesas_rateado},
                              {estmvi_vr_despesas_extras_rateado}, {vr_total_item_rateado},
                              {capa_id}, {produto_id}, {grade_id}, {local_id}, 
                              {produto_embalagem_id}, {quantidade}, 
                              {estmvi_saidas_custo_corrigido_total},
                              {estmvi_saidas_custo_medio_contabil_total},
                              {estmvi_saidas_ultimo_custo_total},
                              {estmvi_saidas_custo_reposicao_total},
                              {estmvi_percentual_comissao},
                              {estmvi_vr_comissao},
                              {vendedor_id})            
            """.format(create_date=data,
                       create_uid=uid,
                       write_date=data,
                       write_uid=uid,
                       nf_id=nf_id,
                       sequencia=i.get('supcor_sequencia'),
                       vr_unitario=i.get('supcor_preco_unitario'),
                       vr_desconto=i.get('supcor_desconto_item_sem_rat'),
                       vr_desconto_rateado=desconto,
                       vr_total_item=total_item,
                       vr_preco_venda=total_item,
                       cfop_id=cfop_id,
                       natureza_id=natureza_id,
                       vr_despesas_rateado=i.get('supcor_acrescimo_item_sem_rat'),
                       estmvi_vr_despesas_extras_rateado=acres,
                       vr_total_item_rateado=total_item_rat,
                       capa_id=mov_capa_id,
                       produto_id=produto_id,
                       grade_id=i.get('grade_id'),
                       local_id=local_id,
                       produto_embalagem_id=i.get('produto_embalagem_id'),
                       quantidade=quantidade,
                       estmvi_saidas_custo_corrigido_total=custo_corrigido,
                       estmvi_saidas_custo_medio_contabil_total=custo_medio_contabil,
                       estmvi_saidas_ultimo_custo_total=ultimo_custo,
                       estmvi_saidas_custo_reposicao_total=custo_reposicao,
                       estmvi_percentual_comissao=dados_comissao.get(
                           'estmvi_percentual_comissao') or 0,
                       estmvi_vr_comissao=dados_comissao.get('estmvi_vr_comissao') or 0,
                       vendedor_id=vendedor_id
                       ).replace("'None'", "''").replace('None', 'null')
                         )

        if msg_itens:
            raise Exception('Os itens a seguir nao estao no supervisor:\n%s' % '\n'.join(msg_itens))

        if msg_acb:
            raise Exception('Os itens a seguir nao estao na automacao:\n%s' % '\n'.join(msg_acb))

        if msg_icms:
            raise Exception('Os itens a seguir nao possuem tributacao de ICMS no supervisor:\n%s' %
                            '\n'.join(msg_icms))
        if not itens:
            raise Exception('Nota de venda apenas com itens cancelados!')

        _cr.execute('SELECT * FROM supervisor_cad_tributacao_nfce')
        dict_tribs_supervisor = {}
        map(lambda x: dict_tribs_supervisor.update({x.get('suptnf_codigo'): x}),
            self.pool.get('adm.utils').admutl_converts_fetchall_to_dictfetchall(_cr))
        cr.execute('SELECT id, sigla FROM fiscal_encargo')
        dict_encargos = {}
        map(lambda x: dict_encargos.update({x.get('sigla'): x.get('id')}), cr.dictfetchall())
        cr.execute('SELECT id, codigo FROM fiscal_csosn')
        dict_csosn = {}
        map(lambda x: dict_csosn.update({x.get('codigo'): x.get('id')}), cr.dictfetchall())
        cr.execute('SELECT id, tipo, codigo, encargo_id FROM fiscal_cst')
        dict_cst = {}
        for i in cr.dictfetchall():
            chave = '%s-%s-%s' % (i.get('tipo'), i.get('codigo'), i.get('encargo_id'))
            dict_cst.update({chave: i.get('id')})

        cr.execute(sql % ',\n'.join(itens))
        dict_novos_dados = {}
        map(lambda x: dict_novos_dados.update({x.get('sequencia'): x.get('id')}), cr.dictfetchall())
        itens_ids = []
        tribs = []
        for i in corpos:
            if i.get('supcor_venda_cancelamento') == 'C':
                continue

            item_id = dict_novos_dados.get(i.get('supcor_sequencia'))
            itens_ids.append(item_id)
            i['novo_id'] = item_id
            icms = dict_tribs_supervisor.get(i.get('supcor_codigo_trib_icms'))
            pis = dict_tribs_supervisor.get(i.get('supcor_codigo_trib_pis'))
            cofins = dict_tribs_supervisor.get(i.get('supcor_codigo_trib_cofins'))
            for trib in [icms, pis, cofins]:
                if not trib:
                    codigo = (not icms and i.get('supcor_codigo_trib_icms')) or \
                             (not pis and i.get('supcor_codigo_trib_pis')) or\
                             (not cofins and i.get('supcor_codigo_trib_cofins'))
                    raise Exception(_('Tributacao nao encontrada no "Supervisor" com'
                                      ' "Codigo Tributacao": %s') % codigo)

                tributacao_id = trib.get('suptnf_codigo')
                aliquota = trib.get('suptnf_aliquota')
                encargo = trib.get('suptnf_encargo')
                codigo_csosn = trib.get('suptnf_codigo_csosn')
                csosn_id = dict_csosn.get(codigo_csosn) or 'null'
                tipo = trib.get('suptnf_tipo_tributacao')
                codigo_cst = trib.get('suptnf_codigo_cst')
                sigla = {'I': 'ICMS',
                         'S': 'ISS',
                         'P': 'PIS',
                         'C': 'COFINS'}.get(encargo)
                encargo_id = dict_encargos.get(sigla)
                chave = '%s-%s-%s' % (tipo, codigo_cst, encargo_id)
                cst_id = dict_cst.get(chave)
                if not cst_id:
                    raise Exception(_('Tributacao no ERP diferente do SUPERVISOR: \nItem: %s\nEncargo: %s') %
                                    (i.get('supcor_item'), sigla))

                nat_pis_cofins = ''
                vr_reducao_base = 0
                if sigla in ('ICMS', 'ISS'):
                    vr_encargo = i.get('supcor_vr_icms')
                    vr_base_calculo = Decimal(i.get('supcor_base_icms'))
                    vr_item_ratedo = Decimal(Decimal(i.get('supcor_preco_total')) - \
                                             Decimal(i.get('supcor_desconto_item') or 0) + \
                                             Decimal(i.get('supcor_acrescimo_item') or 0)).quantize(TWOPLACES)
                    vr_reducao_base = Decimal(vr_item_ratedo - vr_base_calculo).quantize(TWOPLACES) or Decimal(0)
                elif sigla == 'PIS':
                    vr_encargo = i.get('supcor_vr_pis')
                    vr_base_calculo = i.get('supcor_base_pis')
                    nat_pis_cofins = self.pool.get('estoque.produto').read(
                        cr, uid, i.get('produto_id'), ['estpro_pis_nat_rec'], context).get(
                        'estpro_pis_nat_rec') or ''
                elif sigla == 'COFINS':
                    vr_encargo = i.get('supcor_vr_cofins')
                    vr_base_calculo = i.get('supcor_base_cofins')
                    nat_pis_cofins = self.pool.get('estoque.produto').read(
                        cr, uid, i.get('produto_id'), ['estpro_cofins_nat_rec'], context).get(
                        'estpro_cofins_nat_rec') or ''
                else:
                    vr_encargo = vr_base_calculo = 0

                vr_base_calculo = Decimal(vr_base_calculo).quantize(TWOPLACES)
                vr_encargo = Decimal(vr_encargo).quantize(TWOPLACES)
                tribs.append("""('{create_date}', {create_uid}, '{write_date}', {write_uid},
                                 {movimento_itens_id}, {encargo_id}, {fisnit_capa_id},
                                 {fisnit_vr_fcp_st}, {aliquota}, {vr_encargo}, {vr_base_calculo},
                                 {vr_reducao_base},
                                 '{fisnit_nat_rec_pis_cofins}', {fisnit_csosn_id}, {cst_id}, 
                                 {tributacao_id})
                             """.format(create_date=data,
                                        create_uid=uid,
                                        write_date=data,
                                        write_uid=uid,
                                        movimento_itens_id=item_id,
                                        encargo_id=encargo_id,
                                        fisnit_capa_id=nf_id,
                                        fisnit_vr_fcp_st=i.get('supcor_vr_fcp_st') or 0,
                                        aliquota=aliquota,
                                        vr_encargo=vr_encargo,
                                        vr_base_calculo=vr_base_calculo,
                                        vr_reducao_base=vr_reducao_base,
                                        fisnit_csosn_id=csosn_id,
                                        cst_id=cst_id,
                                        fisnit_nat_rec_pis_cofins=nat_pis_cofins,
                                        tributacao_id=tributacao_id)
                             )

        cr.execute("""INSERT INTO fiscal_nf_item_tributacao
                            (create_date, create_uid, write_date, write_uid,
                             movimento_itens_id, encargo_id, fisnit_capa_id,
                             fisnit_vr_fcp_st, aliquota, vr_encargo, vr_base_calculo, 
                             vr_reducao_base,
                             fisnit_nat_rec_pis_cofins, fisnit_csosn_id, cst_id, tributacao_id) 
                      VALUES %s
                      RETURNING id;
                   """ % ',\n'.join(tribs))
        tributacao_ids = map(lambda x: x.get('id'), cr.dictfetchall())
        context['super_write_estoque_movimento_itens'] = True
        context['fisnit_super_write'] = True
        self.pool.get('estoque.movimento.itens').write(cr, uid, itens_ids, {}, context)
        self.pool.get('fiscal.nf.item.tributacao').write(cr, uid, tributacao_ids, {}, context)
        return True

    def wizimn_create_condicao(self, cr, uid, nf_id, dados, dados_nfce, context):
        context = context and context.copy() or {}
        if not dados or dados_nfce.get('supcap_state') != '3':
            if dados_nfce.get('supcap_state') == '3':
                raise Exception('Nota sem fechamento')

            return True

        data_nf = dados_nfce.get('supcap_data_cupom')
        obj_cond_forma = self.pool.get('adm.condicao.forma')
        fechamento_id = False
        for i in dados:
            if fechamento_id != i.get('fechamento_id'):
                fechamento_id = i.get('fechamento_id')
                cond_forma_ids = obj_cond_forma.search(cr, uid,
                      [('finalizadora_na_ecf', '=', i.get('supfec_finalizadora'))], context=context)
                if not cond_forma_ids:
                    raise Exception('Sem condicao de pagamento correlacionada com a finalizadora:\n'
                                    '%s - %s' %(i.get('supfec_nome_finalizadora'), i.get('supfec_finalizadora')))

                cond_forma_id = cond_forma_ids[0]
                dados_cond_forma = obj_cond_forma.read(cr, uid, cond_forma_id,
                                                  ['forma_pagamento_id'], context, '_classic_write')
                forma_pagamento_id = dados_cond_forma.get('forma_pagamento_id')
                vals = {'capa_id': nf_id,
                        'valor': i.get('supfec_valor') - i.get('supfec_troco'),
                        'condicao_forma_id': cond_forma_id,
                        'forma_pagto_id': forma_pagamento_id,
                        'fiscpg_nro_autorizacao_cartao': i.get('supfec_aut_cartao') or '',
                        'fiscpg_chq_numero': i.get('supfec_nro_cheque') or '',
                        'fiscpg_chq_banco': i.get('supfec_nro_banco') or 0,
                        'fiscpg_chq_agencia': i.get('supfec_agencia') or 0,
                        'fiscpg_chq_emitente': i.get('supfec_cnpj') or '',
                        'fiscpg_chq_conta': i.get('supfec_conta') or ''}
                cond_pgto_id = self.pool.get('fiscal.condicao.pagamento').create(cr, uid, vals,
                                                                                 context)

            if not i.get('supfve_valor_parcela'):
                vals = {'nf_condicao_id': cond_pgto_id,
                        'valor': i.get('supfec_valor') - i.get('supfec_troco'),
                        'vencimento': data_nf}
            else:
                vals = {'nf_condicao_id': cond_pgto_id,
                        'valor': i.get('supfve_valor_parcela'),
                        'vencimento': i.get('supfve_vencimento')}

            self.pool.get('fiscal.nf.parcelas').create(cr, uid, vals, context)

        return True

    def wizimn_fatura_pedido(self, cr, uid, dados_capa, company_id, nova_capa_id, context=None):
        '''Atualiza o status do pedido para faturado/validado das NFC-es que vieram de DAV's
           apenas as NFC-es com supcap_tipo = ('V', 'C')
        '''
        context = context and context.copy() or {}
        if dados_capa.get('supcap_tipo') not in ('V', 'C', 'D'):
            return True

        obj_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_titulos = self.pool.get('financeiro.titulo')
        nro_pedido = 'PVE-%010d' % int(dados_capa.get('supcap_numero_pedido'))
        pedido_ids = obj_pedido_capa.search(cr, uid, [('numero_pedido','=',nro_pedido),
                                                      ('company_id', '=', company_id)],
                                                      context=context)
        self.pool.get('fiscal.nf.capa').write(cr, uid, nova_capa_id,
                                              {'fisnfc_pedido_ids': [(6, 0, pedido_ids)]}, context)
        context_aux = context.copy()
        context_aux['pedcap_write_state'] = True
        context_aux['fisnfc_tratar_exclusao'] = True
        vals = {'state': 'F',
                'loja': dados_capa.get('supcap_loja'),
                'caixa':dados_capa.get('supcap_caixa'),
                'coo': dados_capa.get('supcap_nro_cupom'),
                'exportado_frente': True}
        if dados_capa.get('supcap_tipo') == 'C': # NFC-e canceladas
            vals = {'state': 'R',
                    'loja': 0,
                    'caixa': 0,
                    'coo': 0,
                    'exportado_frente': False}
            context['nao_verificar_state_pedido'] = True
            obj_pedido_capa.set_rascunho_pedido(cr, uid, pedido_ids, context)
            self.pool.get('supervisor.dav.capa').write(cr, uid, pedido_ids, {'supdva_nro_cupom': 0},
                                                       context=context)

        obj_pedido_capa.write(cr, uid, pedido_ids, vals, context=context_aux)

        # o pedido pode ter mais de um titulo, deletar todos eles, pois estao em previsao.
        titulo_ids = obj_titulos.search(cr, uid, [('fintit_pedido_id','in',pedido_ids)],
                                        context=context)
        obj_titulos.unlink(cr, uid, titulo_ids, context=context_aux)
        return True
