# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class wizard_listagem_item_tributacao(osv.TransientModel):
    _inherit = 'wizard.listagem.item.tributacao'  # wizlit

    def wizlit_get_dados(self, cr, data, context=None):
        """
        Retorna os dados da consulta SQL em funcao dos parametros do formulario (Listagem de itens 
        e tributacao). -- Cupom fiscal --
        
        :param cr: o cursor do banco de dados
        :param data: dodos do formulario (browser)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type data: object.browser
        :type context: dict
        :return: lista de dicionarios com o resultado da consulta
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        dados = super(wizard_listagem_item_tributacao, self).wizlit_get_dados(cr, data, context)
        # Mostra cupons somente se nao for informado o tipo da nota fiscal ou tipo = Cupom
        if data.tipo_nf in ['E', 'S']:
            return dados

        sql = """
        -- noinspection SqlDialectInspectionForFile
        SELECT      'COO-' || ecc.id              AS documento_id,
                    ap.nome                       AS pessoa_nome,
                    fcn.name                      AS natureza_nome,
                    fc.movimento_tipo             AS cfop_movimento_tipo,
                    fc.codigo                     AS cfop_codigo,
                    'COO' || ecc.coo              AS documento_numero,
                    ecf.loja || '/' || ecf.caixa  AS documento_comp,
                    ecc.date_cupom                AS documento_data,
                    ecc.tipo_cupom                AS documento_tipo,
                    ep.codigo                     AS produto_codigo,
                    ep.name                       AS produto_nome,
                    epg.descricao                 AS grade_descricao,
                    fe.sigla                      AS encargo_sigla,
                    fct.codigo                    AS cst_codigo,
                    emi.vr_total_item_rateado     AS item_vr_total,
                    emi.quantidade                AS quantidade,
                    fnit.vr_base_calculo          AS vr_base_calculo,
                    fnit.aliquota                 AS aliquota,
                    fnit.vr_encargo               AS vr_encargo,
                    emi.id                        AS emi_id,
                    fnit.vr_base_icms_substituicao
                                              AS vr_base_icms_st,
                    fnit.vr_icms_substituicao AS vr_icms_st
        FROM        estoque_movimento_itens       AS emi
        INNER JOIN  ecf_cupons_capa               AS ecc  ON  ecc.id = emi.cupom_id
        INNER JOIN  ecf_ecf                       AS ecf  ON  ecf.id = ecc.ecf_id
        INNER JOIN  fiscal_nf_item_tributacao     AS fnit ON  fnit.movimento_itens_id = emi.id
        INNER JOIN  fiscal_cfop                   AS fc   ON  fc.id = emi.cfop_id
        INNER JOIN  fiscal_cst                    AS fct  ON  fct.id = fnit.cst_id
        INNER JOIN  fiscal_encargo                AS fe   ON  fe.id = fnit.encargo_id
        INNER JOIN  estoque_produto               AS ep   ON  ep.id = emi.produto_id
        INNER JOIN  estoque_produto_grade         AS epg  ON  epg.id = emi.grade_id
        LEFT  JOIN  fiscal_cfop_natureza          AS fcn  ON  fcn.id = emi.natureza_id
        LEFT  JOIN  adm_pessoa                    AS ap   ON  ap.id = ecc.cliente_id
        WHERE       ecc.date_cupom BETWEEN '%s' AND '%s'
        """ % (data.data_inicial, data.data_final)
        if data.pessoa_id:
            sql += """
          AND       ecc.cliente_id = %s
        """ % data.pessoa_id.id
        if data.tipo_movimento:
            sql += """
          AND       fc.movimento_tipo = '%s'
        """ % data.tipo_movimento
        if data.company_id:
            sql += """
          AND       ecc.company_id = %s
        """ % data.company_id.id
        if data.produto_id:
            sql += """
          AND       emi.produto_id = %s
        """ % data.produto_id.id
        if data.grade_id:
            sql += """
          AND       emi.grade_id = %s
        """ % data.grade_id.id
        if data.encargo_id:
            if not data.listar_todos_encargos:
                sql += """
                          AND       fnit.encargo_id = %s
                """ % data.encargo_id.id
        if data.nf_id:
            sql += """
          AND       emi.nf_id = %s
        """ % data.nf_id.id
        if data.wizlit_cst_id:
            if not data.listar_todos_encargos:
                sql += """
                  AND       fnit.cst_id = %s
                """ % data.wizlit_cst_id.id
            else:
                sql += """
                  AND       CASE 
                                WHEN fnit.encargo_id = {encargo_id} THEN fnit.cst_id = {cst_id} 
                                ELSE fnit.encargo_id != {encargo_id} AND 
                                    (SELECT TRUE 
                                     FROM fiscal_nf_item_tributacao 
                                     WHERE 
                                        movimento_itens_id = fnit.movimento_itens_id 
                                        AND encargo_id = {encargo_id} 
                                        AND cst_id = {cst_id})
                            END
                """.format(encargo_id=data.encargo_id.id, cst_id=data.wizlit_cst_id.id)
        if data.wizlit_cfop_ids:
            sql += """
          AND       fc.id IN (%s)
        """ % ', '.join(map(lambda cfop: str(cfop.id), data.wizlit_cfop_ids))
        sql += """
          AND ecc.ecfccp_cupom_cancelamento_id IS NULL
          AND emi.item_cancelado IS NOT TRUE
        ORDER BY    ecc.coo ASC, emi.id,  fnit.encargo_id
        """
        cr.execute(sql)
        result = cr.dictfetchall()

        return dados + result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
