#!/usr/bin/python
# -*- coding: utf-8 -*-
from openerp.osv import osv, fields
import time


class wizard_selecao_nf(osv.Model):
    _name = 'wizard.selecao.nf'
    _inherit = 'wizard.selecao.nf'
    _columns = {
        'cp_ids': fields.many2many('ecf.cupons.capa', 'fiscal_cp_selecao', 'selecao_id', 'cp_id',
                                   "Cupons Fiscais"),
    }

    def confirmar_selecao(self, cr, uid, ids, context=None):
        """
        Cria os itens nota baseados nos itens das cupons selecionadas.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :param context: dict
        :return: sempre verdadeiro.
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        obj_ecf_cupons_capa = self.pool.get('ecf.cupons.capa')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        # Instanciando objetos
        super(wizard_selecao_nf, self).confirmar_selecao(cr, uid, ids, context=context)
        dados = self.read(cr, uid, ids, ['cp_ids'], context=context)
        if dados and isinstance(dados, list):
            dados = dados[0]
        brw_ecf_cupons_capa = obj_ecf_cupons_capa.browse(cr, uid, dados.get('cp_ids'), context=context)
        str_informacao_complementar = str()
        for cupom in brw_ecf_cupons_capa:
            numero = cupom.coo
            data_emissao = time.strftime('%d-%m-%Y', time.strptime(cupom.date_cupom,
                                                                   '%Y-%m-%d'))
            str_informacao_complementar += 'Numero: %s. Data de emissao: %s.\n' % \
                                           (numero, data_emissao)
        if str_informacao_complementar:
            nf_capa_dados = obj_fiscal_nf_capa.read(
                cr, uid, context.get('active_id'), ['fisnfc_infcomp_contribuinte'], context=context)
            if isinstance(nf_capa_dados, list):
                nf_capa_dados = nf_capa_dados[0]
            nf_informacao_complementar = nf_capa_dados.get('fisnfc_infcomp_contribuinte') or ''
            informacao_complementar = 'Cupons Referenciados: \n' + \
                                      str_informacao_complementar
            if nf_informacao_complementar:
                informacao_complementar += '\n' + nf_informacao_complementar
            obj_fiscal_nf_capa.write(cr, uid, context.get('active_id'),
                                     {'fisnfc_infcomp_contribuinte': informacao_complementar}, context=context)
        cp_ids = dados.get('cp_ids')
        if cp_ids:
            self._wizsnf_atualiza_referencia_nf_cp(cr, uid, context.get('active_id'), cp_ids,
                                                   context=context)
            # Se opcao wizselnf_importar_itens desmarcada, nao devera importar os itens dos cupons
            selecao_id = ids[0] if ids and isinstance(ids, (list, tuple)) else ids
            if self.browse(cr, uid, selecao_id, context).wizselnf_importar_itens:
                self._wizsnf_criar_itens_do_cp(cr, uid, context.get('active_id'), cp_ids,
                                               context=context)
        return True

    def _wizsnf_atualiza_referencia_nf_cp(self, cr, uid, nf_id, cp_ids, context):
        """
        Gera os cupons fiscais referenciados para devolucao

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nova nota fiscal
        :param cp_ids: lista com os IDs dos cupons fiscais selecionadas
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type nf_id: long
        :type cp_ids: list[long]
        :param context: dict
        :rtype: boolean
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not cp_ids:
            return
        obj_ecf_cupons_capa = self.pool.get('ecf.cupons.capa')
        res = obj_ecf_cupons_capa.write(cr, uid, cp_ids, {'fisnfc_nf_referencia_id': nf_id},
                                        context)
        return res

    def _wizsnf_criar_itens_do_cp(self, cr, uid, nf_id, cp_ids, context):
        """
        Cria os itens na nota fiscal

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nova nota fiscal
        :param cp_ids: lista com os IDs dos cupons fiscais selecionadas
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type nf_id: long
        :type cp_ids: list[long]
        :param context: dict
        :rtype: boolean
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        # Instanciando objetos
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        nf_itens = []  # Novos itens
        capa_ids = ()
        for i in cp_ids:
            capa_ids += (i,)
        mov_itens_ids = self.pool.get('estoque.movimento.itens').search(cr, uid, [('cupom_id', 'in', capa_ids),
                                                                                  ('nf_id', '=', False)],
                                                                        context=context)
        brw_mov_itens = self.pool.get('estoque.movimento.itens').browse(cr, uid, mov_itens_ids, context)
        for brw_item in brw_mov_itens:
            brw_nf_capa_criada = obj_fiscal_nf_capa.browse(cr, uid, nf_id)
            codigo_cfop = brw_nf_capa_criada.natureza_id.cfop_id.codigo or False
            brw_tributacoes = brw_item.tributacoes_ids
            vr_acrescimos = brw_item.vr_acrescimos or 0
            vr_desconto = brw_item.vr_desconto or 0
            for trib in brw_tributacoes:
                if trib.encargo_id.sigla == 'ICMS' and trib.cst_id.tipo == 'U':  # U = Subs. tributaria
                    if codigo_cfop:
                        if codigo_cfop[1:] == '201':
                            codigo_cfop = codigo_cfop[:1] + '410'
                        elif codigo_cfop[1:] == '202':
                            codigo_cfop = codigo_cfop[:1] + '411'
                        break
            cfop_id = self.pool.get('fiscal.cfop').search(cr, uid, [('codigo', '=', codigo_cfop)], context=context)
            if cfop_id:
                cfop_id = cfop_id[0]

            # Acrescenta as tributacoes do item
            tributacao_ids = self._wizsnfc_get_itens_tributacao_cp(cr, uid, brw_item.id, context)
            quantidade = brw_item.quantidade * brw_item.produto_embalagem_id.embalagem_id.\
                quantidade_na_embalagem
            nf_itens.append(
                [0, 0, dict(local_id=brw_item.local_id.id, produto_id=brw_item.produto_id.id,
                            grade_id=brw_item.grade_id.id, vr_desconto=brw_item.vr_desconto,
                            vr_total_item=brw_item.vr_total_item,
                            quantidade=quantidade,
                            tributacoes_ids=tributacao_ids,
                            produto_embalagem_id=brw_item.produto_embalagem_id.id, cfop_id=cfop_id,
                            vr_unitario=brw_item.vr_unitario,
                            vr_total_item_rateado=brw_item.vr_total_item_rateado,
                            vendedor_id=brw_item.vendedor_id and brw_item.vendedor_id.id or False,
                            estmvi_vr_comissao=brw_item.estmvi_vr_comissao or 0,
                            )])

        # Cria os novos itens da nova nota fiscal
        result = obj_fiscal_nf_capa.write(
            cr, uid, nf_id, {'estoque_movimento_itens_ids': nf_itens}, context)

        return result

    def _wizsnfc_get_itens_tributacao_cp(self, cr, uid, mov_item_id, context):
        """
        Gera tributacao para os itens importados
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param mov_item_id: ID do movimento do item referente ao cupom
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type mov_item_id: int
        :param context: dict
        :rtype: list[dict]
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        # Instanciando objetos
        obj_fisnit = self.pool.get('fiscal.nf.item.tributacao')
        obj_fis_utl = self.pool.get('fiscal.utils')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')

        fisnit_ids = obj_fisnit.search(cr, uid, [('movimento_itens_id', '=', mov_item_id)], context=context)
        fields_to_read = ['vr_base_icms_substituicao', 'aliquota', 'vr_base_calculo',
                          'percentual_mva', 'cst_id', 'vr_encargo', 'tributacao_id',
                          'vr_icms_substituicao', 'modbcst', 'vr_reducao_base', 'tipo_substituicao',
                          'aliquota_venda', 'fisnit_csosn', 'encargo_id']
        fisnit_dados = obj_fisnit.read(cr, uid, fisnit_ids, fields_to_read, context=context,
                                       load='_classic_write')
        nf_item_tributacao_dados = []

        for vals in fisnit_dados:
            base = obj_fiscal_tributacao.browse(cr, uid, vals.get('tributacao_id'),
                                                context).base or 0.0
            del vals['id']
            cst_id, tributacao_id = obj_fis_utl.get_cst_oposto(cr, uid,
                                                               vals.get('cst_id'),
                                                               vals.get('tributacao_id'),
                                                               vals.get('encargo_id'),
                                                               base,
                                                               vals.get('aliquota'),
                                                               context=context)
            vals['cst_id'] = cst_id
            vals['tributacao_id'] = tributacao_id
            nf_item_tributacao_dados.append((0, 0, vals))
        return nf_item_tributacao_dados
