# -*- coding: utf-8 -*-

from openerp.osv import osv


class wizard_verifica_sped(osv.TransientModel):

    _inherit = 'wizard.verifica.sped'

    def get_query_inherit_dados(self, cr, uid, sql, context=None):
        sql = super(wizard_verifica_sped, self).get_query_inherit_dados(cr, uid, sql, context)
        sql = sql.replace('emi.id          AS movimento',
                          "emi.id          AS movimento,"
                          "ecc.coo         AS numero_cupom,"
                          "ecf.loja || '-' || ecf.caixa AS loja_caixa")
        sql = sql.replace('LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id',
                          'LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id '
                          'LEFT JOIN ecf_cupons_capa ecc ON ecc.id = emi.cupom_id '
                          'LEFT JOIN ecf_ecf ecf ON ecf.id = ecc.ecf_id')
        sql = sql.replace('GROUP BY numero_nf, tipo_nf, movimento, produto, grade',
                          'GROUP BY numero_nf, tipo_nf, movimento, numero_cupom, loja_caixa, '
                          '         produto, grade')
        sql = sql.replace('AND emi.nf_id IS NOT NULL',
                          'AND (emi.nf_id IS NOT NULL OR emi.cupom_id IS NOT NULL)')
        return sql

    def get_dados_inconsistencia_sped(self, cr, uid, data_inicial, data_final, context=None):
        context = context and context.copy() or {}

        dados = super(wizard_verifica_sped, self).get_dados_inconsistencia_sped(
            cr, uid, data_inicial, data_final, context)

        # Cupom importado sem CST
        query_cupom_importado_sem_cst = self.get_query_cupom_importado_sem_cst(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_cupom_importado_sem_cst)
        dados_cupom_importado_sem_cst = cr.dictfetchall()

        # Diferenca da base de calculo entre reducao Z e movimento do estoque
        query_diferenca_base_calculo_reducaoz_estoque = self.\
            get_query_diferenca_base_calculo_reducaoz_estoque(cr, uid, data_inicial, data_final,
                                                              context)
        cr.execute(query_diferenca_base_calculo_reducaoz_estoque)
        dados_diferenca_base_calculo_reducaoz_estoque = cr.dictfetchall()

        # Reduções Z faltantes.
        dados_reducao_z_faltante = self.get_dados_reducao_z_faltante(
            cr, uid, data_inicial, data_final, context)

        dados_ecf = {
            'cupom_importado_sem_cst': dados_cupom_importado_sem_cst,
            'diferenca_base_calculo_reducaoz_estoque':
                dados_diferenca_base_calculo_reducaoz_estoque,
            'reducao_z_faltante': dados_reducao_z_faltante
        }
        dados.update(dados_ecf)
        return dados

    def get_query_cupom_importado_sem_cst(self, cr, uid, data_inicial, data_final, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                ecc.coo                                        AS numero_cupom,
                ecf.loja || '-' || ecf.caixa                   AS loja_caixa,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                                               AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                                               AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN ecf_cupons_capa ecc
                    ON ecc.id = emi.cupom_id
                INNER JOIN ecf_ecf ecf ON ecf.id = ecc.ecf_id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emc.data BETWEEN '%s' AND '%s'
                AND emi.cupom_id IS NOT NULL
                AND fnit.cst_id IS NULL
            GROUP BY numero_cupom, loja_caixa, produto, grade
        """ % (company_id, data_inicial, data_final)
        return sql

    def get_query_diferenca_base_calculo_reducaoz_estoque(self, cr, uid, data_inicial, data_final,
                                                          context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'ecf.ecf', context=context)
        sql = """
            SELECT *
            FROM (
                SELECT
                    data,
                    loja,
                    caixa,
                    sum(valor) AS valor
                FROM (
                         SELECT
                             mrc.data                        AS data,
                             ecf.loja                        AS loja,
                             ecf.caixa                       AS caixa,
                             sum(mrc.cancelamentos + (SELECT sum(base_valor)
                                                      FROM ecf_mapa_resumo_itens
                                                      WHERE capa_id = mrc.id)) 
                                                             AS valor
                         FROM ecf_mapa_resumo_capa mrc
                             INNER JOIN ecf_ecf ecf
                                 ON ecf.id = mrc.ecf
                         WHERE
                             ecf.company_id = {company_id}
                             AND mrc.data BETWEEN '{data_inicial}' AND '{data_final}'
                         GROUP BY data, loja, caixa
                         UNION ALL
                         SELECT
                             date_cupom                      AS data,
                             ecf.loja                        AS loja,
                             ecf.caixa                       AS caixa,
                             sum((
                                    CASE 
                                        WHEN tipo_cupom = 'V' THEN vr_total_item_rateado 
                                        WHEN tipo_cupom = 'C' THEN vr_desconto 
                                        ELSE 0 
                                    END) * -1)
                                                             AS valor
                         FROM estoque_movimento_itens emi
                             INNER JOIN ecf_cupons_capa ecc
                                 ON ecc.id = emi.cupom_id
                             INNER JOIN ecf_ecf ecf
                                 ON ecf.id = ecc.ecf_id
                         WHERE
                             ecf.company_id = {company_id}
                             AND date_cupom BETWEEN '{data_inicial}' AND '{data_final}'
                             AND tipo_cupom IN ('V', 'C')
                         GROUP BY data, loja, caixa
                         UNION ALL
                         SELECT
                             date_cupom                      AS data,
                             ecf.loja                        AS loja,
                             ecf.caixa                       AS caixa,
                             sum(eci.preco_total * -1)       AS valor
                         FROM ecf_cupons_capa ecc
                             INNER JOIN ecf_ecf ecf
                                 ON ecf.id = ecc.ecf_id
                             INNER JOIN ecf_cupons_item eci
                                 ON ecc.id = eci.capa_id
                         WHERE
                             ecf.company_id = {company_id}
                             AND date_cupom BETWEEN '{data_inicial}' AND '{data_final}'
                             AND tipo_cupom = 'N'
                         GROUP BY data, loja, caixa
                     ) AS tmp
                GROUP BY data, loja, caixa
                ORDER BY data, loja, caixa
            ) AS tmp
            WHERE
                valor != 0
        """.format(company_id=company_id, data_inicial=data_inicial, data_final=data_final)
        return sql

    def get_dados_reducao_z_faltante(self, cr, uid, data_inicial, data_final, context=None):
        context = context and context.copy() or {}
        dict_data_sem_z = dict()
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'ecf.ecf', context=context)
        sql = """
            SELECT emr.data, emr.ecf AS ecf_id, ecf.caixa
            FROM ecf_mapa_resumo_capa emr
            INNER JOIN ecf_ecf ecf
                ON ecf.id = emr.ecf
            WHERE
                emr.data >= '%s'
                AND emr.data <= '%s'
                AND ecf.company_id = %s
        """ % (data_inicial, data_final, company_id)
        cr.execute(sql)
        movimento_mapa_z = cr.dictfetchall()
        sql = """
            SELECT
                ecc.date_cupom, ecc.tipo_cupom, ecc.ecf_id, ecf.caixa
            FROM
                ecf_cupons_capa ecc
            INNER JOIN
                ecf_ecf ecf ON ecf.id = ecc.ecf_id
            WHERE
                ecc.date_cupom BETWEEN '%s' AND '%s'
                AND ecc.tipo_cupom IN ('V', 'Z')
                AND ecc.company_id = %s
            GROUP BY ecc.date_cupom, ecc.tipo_cupom, ecc.ecf_id, ecf.caixa
            ORDER BY ecc.date_cupom, ecc.tipo_cupom, ecc.ecf_id, ecf.caixa
        """ % (data_inicial, data_final, company_id)
        cr.execute(sql)
        movimento_cupom_z = cr.dictfetchall()
        datas_list = list(set(map(lambda i: i.get('date_cupom'), movimento_cupom_z)))
        for data in datas_list:
            data_movimento_cupom_z = filter(lambda z: z.get('date_cupom') == data,
                                            movimento_cupom_z)
            caixas = []
            caixa_list = map(lambda ec: ec.get('caixa'), data_movimento_cupom_z)
            for e in caixa_list:
                if caixa_list.count(e) == 1:
                    caixas.append(str(e))
            if (len(data_movimento_cupom_z) % 2) or not filter(
                    lambda z: z.get('tipo_cupom') == 'Z' and z.get('date_cupom') == data,
                    movimento_cupom_z):
                dict_data_sem_z.update({data: ', '.join(caixas)})
                continue
            data_movimento_mapa_z = filter(lambda z: z.get('data') == data, movimento_mapa_z)
            ecf_ids = list(set(map(lambda e: e.get('ecf_id'), data_movimento_mapa_z)))
            caixas = []
            caixa_list = map(lambda ec: ec.get('caixa'), data_movimento_mapa_z)
            for e in caixa_list:
                if caixa_list.count(e) == 1:
                    caixas.append(str(e))
            if len(data_movimento_mapa_z) != len(ecf_ids):
                dict_data_sem_z.update({data: ', '.join(caixas)})
                continue
        return dict_data_sem_z
