# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class ecf_cupons_capa(osv.Model):
    #prefixo = ecfccp_

    from funcoes import ecfccp_get_dados_cupom, ecfccp_get_cupom_venda_id

    """
    Tabela de cupons fiscais
    """

    _name = 'ecf.cupons.capa'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _rec_name = 'coo'
    _limit = 50
    _order = 'hora_cupom_inicio DESC'
    _columns = {
        'coo': fields.integer('Coo', required=True, select=True), # Numero do cupom
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True),
        'ecf_id': fields.many2one("ecf.ecf", 'Ecf', ondelete="restrict", required=True,select=True), # id do ecf
        'tipo_cupom': fields.selection([
                                           ('A', 'Sangria'),
                                           ('b', 'Correspondente bancario'),
                                           ('B', 'Recibo do correspondente bancario'),
                                           ('C', 'Cancelamento'),
                                           ('c', 'Cancelamento do correspondente bancario'),
                                           ('F', 'Fechamento de turno'),
                                           ('G', 'Relatorio Gerencial'),
                                           ('M', 'Memoria Fiscal'),
                                           ('N', 'cupom anulado'),
                                           ('O', 'Contra vale'),
                                           ('P', 'Promissoria'),
                                           ('R', 'Recarga de Celular'),
                                           ('T', 'cupom Tef'),
                                           ('U', 'Suprimento'),
                                           ('V', 'Venda'),
                                           ('X', 'Reducao X'),
                                           ('W', 'Recebimentos diversos'),
                                           ('w', 'Recibo dos recebimentos diversos'),
                                           ('Z', 'Rreducao Z')], 'Tipo cupom', required=True, size=1),
        'date_cupom': fields.date('Data cupom', required=True, select=True ), # Data de emissao do cupom
        'hora_cupom_inicio': fields.datetime('Hora abertura do cupom', required=True), #trocado de time para date.time
        'hora_cupom_fim': fields.datetime('Hora fechamento do cupom'), #trocado de time para date.time
        'cliente_id': fields.many2one("adm.pessoa", 'Cliente', ondelete="restrict"), # Pessoa destinatária.
        'alteracao': fields.selection([('A', 'Acrescimo'), ('D', 'Desconto')], 'Acrescimo/Desconto', size=1),
        'valor_alteracao': fields.float('Valor Acrescimo/Desconto', digits=(9, 2)),
        'total_cupom': fields.float('Total Liquido do Cupom', digits=(9, 2)),
        'cliente_contra_vale': fields.char('Cliente Contra Vale', size=5),
        'valor_contra_vale': fields.float('Valor Contra Vale', digits=(9, 2)),
        'cupons_finalizacao_ids': fields.one2many('ecf.cupons.finalizacao', 'cupom_id', 'Finalizacao'),
        'ecfccp_finan_tes_mov': fields.one2many('financeiro.tesouraria.movimento', 'fintem_cupom_id', 'Movimentos da Tesouraria'),
        'cupons_xz_ids': fields.one2many('ecf.cupons.xz', 'cupom_id', 'Xz'),
        'cupons_xztotalizadores_ids': fields.one2many('ecf.cupons.xztotalizadores', 'cupom_id', 'Xztotalizadores'),
        'state': fields.selection([('C', 'Criado'),
                                   ('A', 'Aguardando cupom de venda'),
                                   ('D', 'Divergente'),
                                   ('V', 'Validado')], "Estado", required=True,
                                  size=1),
        'operador': fields.many2one("rh.funcionario", 'Funcionario', ondelete="restrict", required=False,
                                    domain="[('company_id', '=', company_id)]"),
        'nf_id': fields.many2one('fiscal.nf.capa', 'Nro da Nota Fiscal',select=True),
        'ecf_cupons_item_ids': fields.one2many('ecf.cupons.item', 'capa_id', 'itens do Cupom'),
        'ecf_cupom_log_ids': fields.one2many('ecf.cupom.log', 'cupom_id', 'Divergencias'),
        'cupom_fiscal_parcelas_ids': fields.one2many('fiscal.nf.parcelas', 'cupom_id', "Parcelas do Cupom"),
        'cupom_financeiro_fatura_ids': fields.one2many('financeiro.fatura', 'cupom_id', "Faturas do Cupom"),
        'flag_faturado': fields.function(ecfccp_get_dados_cupom, method=True, string='Faturado', type='boolean',
                                         store=False, multi="dados_cupons"),
        'flag_status_nf': fields.char('Status NF', size=3),
        'ecc_vendedor_id': fields.many2one('vendas.vendedor', 'Vendedor'),
        'fisnfc_nf_referencia_id': fields.many2one('fiscal.nf.capa', 'Nota referenciada',
                                                   help='Nota de devolucao'),
        'ecfccp_operador_no_supervisor': fields.integer('Operador no supervisor'),
        'ecfccp_numero_pedido': fields.char('Numero do Pedido', size=20),
        'ecfccp_cupom_cancelamento_id': fields.function(ecfccp_get_dados_cupom,
                                                        string="Cupom de cancelamento",
                                                        method=True,
                                                        store={'ecf.cupons.capa': (ecfccp_get_cupom_venda_id, [], 10)},
                                                        type="many2one",
                                                        relation="ecf.cupons.capa",
                                                        multi="dados_cupons"),

    }
    _defaults = {
        'state': lambda *a: 'C',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }
    _sql_constraints = [
        ('coo_ecf_id_date', 'unique (coo, ecf_id, date_cupom)', 'O COO deve ser unico para cada ECF!'),
    ]

    _fields_no_store = ['ecfccp_cupom_cancelamento_id']

    _order = 'date_cupom desc, hora_cupom_inicio desc, ecf_id desc, coo desc'

