# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class ecf_cupons_finalizacao(osv.Model):
    """
    Tabela de finalizacao dos cupons
    Prefixo: ecffin_
    """
    TIPO_FINALIZADORA = (('D', 'Dinheiro'), ('H', 'Cheque'), ('T', 'Ticket'), ('C', 'Cartao'), ('O', 'Outros'))

    _name = 'ecf.cupons.finalizacao'
    _columns = {
        'cupom_id': fields.many2one("ecf.cupons.capa", 'Cupom', ondelete="cascade", required=True, select=True),
        # id do cupom
        'coo': fields.char('Numero cupom', size=6),
        'finalizadora_id': fields.many2one("adm.condicao.forma", 'Finalizadora',
                                           ondelete="restrict", required=False,
                                           select=True),  # id da finalizadora
        'data': fields.date('Data', help='Data do movimento no formato YYYYMMDD'),
        'loja': fields.char('Loja', size=4, help='Numero da loja'),
        'caixa': fields.char('Caixa', size=3, help='Numero do caixa'),
        'valor': fields.float("Valor", digits=(11, 2), required=True), # Valor da finalizacao
        'sequencia': fields.integer("Sequencia", help="Sequencia da finalizadora no cupom"),
        'tipo_finalizadora': fields.selection(TIPO_FINALIZADORA, "Tipo da Finalizadora"),
        'valor_finalizadora': fields.float('Valor Finalizadora', digits=(11, 2)),
        'avista_aprazo': fields.selection([('V', 'A vista'), ('P', 'A prazo')], "Condicao Finalizadora", size=1),
        'troco': fields.float("Troco", digits=(11, 2)),
        'numero_banco': fields.char('Numero do banco', size=3, help='Numero do banco no caso de cheques'),
        'agencia': fields.char('Numero agencia', size=5, help='Numero da agencia no caso de cheques'),
        'numero_cheque': fields.char('Numero do cheque', size=10),
        'conta': fields.char('Conta', size=7, help='Numero da conta no caso de cheques'),
        'juros': fields.float('Juros', digits=(11, 2), help='Valor dos juros cobrados quando venda a prazo'),
        'cnpj': fields.char('CNPJ', size=15, help='CNPJ/CPF do emitente do cheque'),
        'valor_taxa_adm': fields.float('Valor Taxa Administracao', digits=(11, 2),
                                       help='Valor da Taxa de Administracao (Para Tickets e Cartoes'),
        'cliente_adm': fields.char('Codigo do Cliente', size=5,
                                   help='Codigo do cliente (Administradora dos Cartoes/Tickets)'),
        'desconto': fields.float('Descontos', digits=(11, 2)),
        'acrescimo': fields.float('Acrescimo', digits=(11, 2)),
        'ecf_cupons_vencimento_ids': fields.one2many('ecf.cupons.vencimento', 'finalizacao_id',
                                                     'Vencimentos do Fechamento'),
        'ecffin_finalizadora_na_ecf': fields.integer('Finalizadora na ECF'),
    }



