#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class sintegra_registro_60d_view(osv.Model):

    _name = 'sintegra.registro.60d.view'
    _auto = False
    _columns = {
        'id': fields.integer('ID'),
        'sint_reg60d_data_cupom': fields.char('Data Cupom', size=8, select=True),
        'sint_reg60d_numero_serie': fields.char('Numero de Serie', size=20),
        'sint_reg60d_item': fields.char('Item', size=20,select=True),
        'sint_reg60d_quantidade': fields.float('Quantidade', digits=(9, 3)),
        'sint_reg60d_valor_total': fields.float('Valor Liquido', digits=(13, 2)),
        'sint_reg60d_base_icms': fields.float('Base ICMS', digits=(13, 2)),
        'sint_reg60d_tributacao_retag': fields.char('Tributacao retaguarda', size=3),
        'sint_reg60d_tributacao_ecf': fields.char('Situacao Tributaria/Aliquota', size=3),
        'sint_reg60d_aliquota': fields.float('Aliquota', digits=(2, 2)),
        'sint_reg60d_valor_icms': fields.float('Valor ICMS', digits=(13, 2)),
        'sint_reg60d_blank': fields.char('Blank', size=13),
        'sint_reg60d_ecf_id': fields.many2one('ecf.ecf', 'Ecf',select=True),
        'sint_reg60d_company_id': fields.many2one('res.company', 'Empresa'),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'sintegra_registro_60d_view')
        sql = """
            create or replace view sintegra_registro_60d_view as (
                    SELECT Rank()
                             OVER (
                               ORDER BY cupom_item.id)                AS id,
                           To_char(cupom_capa.date_cupom, 'YYYYMMDD') AS sint_reg60d_data_cupom,
                           cupom_capa.ecf_id                          AS sint_reg60d_ecf_id,
                           ecf.nro_serie                              AS sint_reg60d_numero_serie,
                           cupom_item.item                            AS sint_reg60d_item,
                           CASE
                             WHEN cupom_item.venda_cancelamento = 'V'
                                  AND cupom_capa.tipo_cupom = 'V' THEN cupom_item.quantidade
                             WHEN cupom_capa.tipo_cupom = 'C' THEN -cupom_item.quantidade
                             ELSE 0
                           END                                        AS sint_reg60d_quantidade,
                           CASE
                             WHEN cupom_item.venda_cancelamento = 'V'
                                  AND cupom_capa.tipo_cupom = 'V'
                                      THEN cupom_item.preco_total - cupom_item.desconto
                             WHEN cupom_capa.tipo_cupom = 'C' 
                                      THEN (cupom_item.preco_total - cupom_item.desconto) * -1
                             ELSE 0
                           END                                        AS sint_reg60d_valor_total,
                           CASE
                             WHEN cupom_item.venda_cancelamento = 'V'
                                  AND cupom_capa.tipo_cupom = 'V' THEN
                             (
                             cupom_item.preco_total * cupom_item.base * cupom_item.aliquota / 10000
                             )
                             WHEN cupom_capa.tipo_cupom = 'C' THEN -(
                             cupom_item.preco_total *
                             cupom_item.base * cupom_item.aliquota / 10000 )
                             ELSE 0
                           END                                        AS sint_reg60d_valor_icms,
                           cupom_item.base                            AS sint_reg60d_base_icms,
                           cupom_item.tributacao_ecf                  AS sint_reg60d_tributacao_ecf,
                           cupom_item.tributacao_retag                AS
                           sint_reg60d_tributacao_retag,
                           cupom_item.aliquota                        AS sint_reg60d_aliquota,
                           Cast ('' AS TEXT)                          AS sint_reg60d_blank,
                           cupom_capa.company_id                      AS sint_reg60d_company_id
                    FROM   ecf_cupons_capa cupom_capa
                           INNER JOIN ecf_ecf ecf
                                   ON ecf.id = cupom_capa.ecf_id
                           INNER JOIN ecf_cupons_item cupom_item
                                   ON cupom_capa.id = cupom_item.capa_id
                    WHERE  cupom_capa.state = 'V'
            );
        """
        cr.execute(sql)


class sintegra_registro_60i_view(osv.Model):
    _name = 'sintegra.registro.60i.view'
    _auto = False
    _columns = {
        'id': fields.integer('ID'),
        'sint_reg60i_data_cupom': fields.char('Data Cupom', size=8,select=True),
        'sint_reg60i_numero_serie': fields.char('Numero de Serie', size=14),
        'sint_reg60i_modelo': fields.char('Modelo', size=2),  # ??
        'sint_reg60i_coo': fields.integer('COO'),
        'sint_reg60i_sequencia': fields.integer('Sequencia'),
        'sint_reg60i_item': fields.char('Codigo Produto', size=14,select=True),
        'sint_reg60i_quantidade': fields.float('Quantidade', digits=(10, 3)),
        'sint_reg60i_valor_total': fields.float('Valor Total', digits=(11, 2)),
        'sint_reg60i_base': fields.float('Base ICMS', digits=(10, 2)),
        'sint_reg60i_tributacao_ecf': fields.char('Tributacao ECF', size=4),
        'sint_reg60i_tributacao_retag': fields.char('Tributacao Retag', size=4),
        'sint_reg60i_aliquota': fields.float('Aliquota', digits=(2, 2)),
        'sint_reg60i_valor_icms': fields.float('Valor ICMS', digits=(10, 2)),
        'sint_reg60i_blank': fields.char('Blank', size=16),
        'sint_reg60i_ecf_id': fields.many2one('ecf.ecf', 'Ecf',select=True),
        'sint_reg60i_company_id': fields.many2one('res.company', 'Empresa'),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'sintegra_registro_60i_view')
        sql = """
            create or replace view sintegra_registro_60i_view as (
                select
                rank() over (order by cupom_item.id) as id,
                to_char(cupom_capa.date_cupom, 'YYYYMMDD') as sint_reg60i_data_cupom,
                cupom_capa.ecf_id as sint_reg60i_ecf_id,
                ecf.nro_serie as sint_reg60i_numero_serie,
                '2D' as sint_reg60i_modelo, -- nao existe tratamento no sistema. Foi fixado como 2D
                cupom_capa.coo as sint_reg60i_coo,
                cupom_item.sequencia as sint_reg60i_sequencia,
                cupom_item.item as sint_reg60i_item,
                cupom_item.quantidade as sint_reg60i_quantidade,
                cupom_item.preco_total as sint_reg60i_valor_total,
                cupom_item.base as sint_reg60i_base,
                cupom_item.tributacao_ecf as sint_reg60i_tributacao_ecf,
                cupom_item.tributacao_retag as sint_reg60i_tributacao_retag,
                cupom_item.aliquota as sint_reg60i_aliquota,
                cupom_item.preco_total * cupom_item.base * cupom_item.aliquota / 10000 as sint_reg60i_valor_icms,
                '' as sint_reg60i_blank,
                cupom_capa.company_id AS sint_reg60i_company_id
                from
                    ecf_cupons_capa cupom_capa
                inner join ecf_ecf ecf on ecf.id = cupom_capa.ecf_id
                inner join ecf_cupons_item cupom_item on cupom_capa.id = cupom_item.capa_id
                and cupom_item.venda_cancelamento = 'V'
                order by sint_reg60i_data_cupom, modelo, nro_serie, coo, cupom_item.sequencia
            );
        """
        cr.execute(sql)
