# -*- encoding: utf-8 -*-

from openerp import pooler
from openerp.osv import osv, fields
from openerp.report.interface import report_int
from openerp.report.render import render
from SigUtils.impressao import Impressora
import StringIO
import time


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_fechamento_ecf(report_int):
    def _get_ecf_fechamento(self, cr, uid, ecf_id, data, resumo=False, context=None):
        """
            Funcao cria 5 dicionário de listas com as estrutura para impressao :
            Vendas  ( Total de vendas do dia da Reducao Z )
            Entrada (Suprimentos),
            Saida ( Sangrias ) ,
            Totais  ( Por Tipo de Finalizadora ) [ D - Dinheiro , H - Cheque, C - Cartao, T - Ticket, O - Outros ]
            Resumo  ( Dicionário com as finalizadoras da Reducao Z )
            @param ecf_id: Id da tabela ecf.ecf;
            @param data: data para validacao dos dados
            @param resumo:
            @return: O retorno e uma estutura de dicionários {'entrada':[valor1,valor2,valor3...],
            'saida':[valor1,valor2,valor3...],'totais:{Finalizadoa1:valor1,Finalizadoa2:valor2,Finalizadoa3:valor3...]'}

        """
        #
        context = context and context.copy() or {}
        res = {'vendas': 0,
               'saida': {'D': 0.0, 'C': 0.0, 'H': 0.0, 'T': 0.0, 'O': 0.0},
               'entrada': {'D': 0.0, 'C': 0.0, 'H': 0.0, 'T': 0.0, 'O': 0.0},
               'totais': {'D': 0.0, 'C': 0.0, 'H': 0.0, 'T': 0.0, 'O': 0.0},
               'resumo': {'totais': {'D': 0.0, 'C': 0.0, 'H': 0.0, 'T': 0.0, 'O': 0.0}}}
        obj_ecf_tesouraria_capa = self.pool.get("ecf.tesouraria.capa")
        obj_ecf_tesouraria_movimento = self.pool.get("ecf.tesouraria.movimento")
        obj_ecf_cupons_capa = self.pool.get("ecf.cupons.capa")
        obj_ecf_cupons_finalizacao = self.pool.get("ecf.cupons.finalizacao")

        total_vendas = 0
        # Buscando Vendas do Cupons Capa para o total de vendas e tambem separando por finalizadora
        cupom_ids = obj_ecf_cupons_capa.search(cr, uid, [('ecf_id', '=', ecf_id), ('date_cupom', '=', data),
                                                         ('tipo_cupom', '=', 'Z')], context=context)
        if cupom_ids:
            # read_ecf_cupons_capa = obj_ecf_cupons_capa.read(cr,uid,cupom_ids,['total_cupom'])

            cupom_fin_ids = obj_ecf_cupons_finalizacao.search(cr, uid, [('cupom_id', 'in', cupom_ids)], context=context)
            browse_cupons_finalizacao = obj_ecf_cupons_finalizacao.browse(cr, uid, cupom_fin_ids)
            total_vendas = sum(map(lambda x: x.valor, browse_cupons_finalizacao)) or 0
            res['vendas'] = total_vendas
            if resumo:
                for finalizadora in browse_cupons_finalizacao:
                    res['resumo']['totais'][finalizadora.tipo_finalizadora] += finalizadora.valor or 0
                    total_vendas += finalizadora.valor

        # Separando Entrada  (Vendas + Suprimento) e Saida ( Sangria)
        capa_ids = obj_ecf_tesouraria_capa.search(cr, uid, [('ecf_id', '=', ecf_id), ('data_movimento', '=', data)],
                                                  context=context)
        browse_tesouraria_movimento = []
        if capa_ids:
            mov_ids = obj_ecf_tesouraria_movimento.search(cr, uid, [('capa_id', 'in', capa_ids)], context=context)
            browse_tesouraria_movimento = obj_ecf_tesouraria_movimento.browse(cr, uid, mov_ids)
        for mov in browse_tesouraria_movimento:
            #
            if mov.finalizadora_id.tipo_finalizadora:
                if mov.tipo == 'A':  # Sangria
                    res['saida'][mov.finalizadora_id.tipo_finalizadora] += mov.valor or 0
                else:  # Suprimento
                    res['entrada'][mov.finalizadora_id.tipo_finalizadora] += mov.valor or 0

                a = mov.valor
                res['totais'][mov.finalizadora_id.tipo_finalizadora] += a
        return res

    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        self.pool = pooler.get_pool(cr.dbname)
        pdf_file = StringIO.StringIO()
        obj_ecf = self.pool.get("ecf.ecf")
        # DADOS DO CONTEXTO
        empresa = datas['company_id']
        nome_empresa = datas.get('nome_empresa')
        data_ini = datas['data_ini']
        resumo = datas['resumo']

        if not datas['ecf_id']:
            ecf_ids = obj_ecf.search(cr, uid, [('company_id', '=', empresa)], context=context)
        else:
            ecf_ids = [datas['ecf_id'][0]]
        data_ecf = obj_ecf.read(cr, uid, ecf_ids, ['name'], context=context)



        # CABEÇARIO RELATORIO
        listagem = Impressora()

        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_file, 'Registro de entrada')
        listagem.informaTamLinha(80)
        listagem.defineMargem(1, 1, 1, 1)

        listagem.informaCab('*---------------------------------------------------------------------------------*')
        listagem.informaCab(
            '|                      Fechamento ECF                                         {{ Page,2 }}  |')
        listagem.informaCab(u'|Empresa: %20s                  Data: %s |' % (nome_empresa, data_ini))
        listagem.informaCab('|---------------------------------------------------------------------------------|')
        listagem.informaCab('|                                                                                 |')
        contador = 0
        for ecf in data_ecf:
            # Dados da ECF
            listagem.imprimeLinha(
                '| ECF: %10s                                                              |' % ecf.get('name'))
            listagem.imprimeLinha('|-- Entradas -------*-- Saidas ------*-------------------Totais-------------------|')
            linha = '| %20s   |  %20s  | %10s  %10s        |'

            # Get data ECF.Fechamento
            result = self._get_ecf_fechamento(cr, uid, ecf.get('id'), data_ini, resumo, context)
            imprime = True
            total_entrada = 0
            total_saida = 0
            finalizadora = ['Dinheiro :', 'D', 'Cartao :', 'C', 'Cheque :', 'H', 'Ticket :', 'T', 'Outros :', 'O']
            finalizadora_entrada = ['Dinheiro :', 'D', 'Cartao :', 'C', 'Cheque :', 'H', 'Ticket :', 'T', 'Outros :',
                                    'O']
            finalizadora_saida = ['Dinheiro :', 'D', 'Cartao :', 'C', 'Cheque :', 'H', 'Ticket :', 'T', 'Outros :', 'O']
            while imprime:
                imprime = False
                if result['entrada']:
                    if finalizadora_entrada:
                        imprime = True
                        sg_finaliza_entrada = finalizadora_entrada.pop()
                        ds_finaliza_entrada = finalizadora_entrada.pop()
                        if result['entrada'].get(sg_finaliza_entrada, 0.0) > 0:
                            entrada = ds_finaliza_entrada + '%10.2f' % result['entrada'].get(sg_finaliza_entrada, 0.0)
                            total_entrada += result['entrada'].get(sg_finaliza_entrada, 0.0)
                        else:
                            entrada = ''
                            imprime = False
                else:
                    entrada = ''
                if result['saida']:
                    if finalizadora_saida:
                        imprime = True
                        sg_finaliza_saida = finalizadora_saida.pop()
                        ds_finaliza_saida = finalizadora_saida.pop()
                        if result['saida'].get(sg_finaliza_saida, 0.0) > 0:
                            saida = ds_finaliza_saida + '%10.2f' % result['saida'].get(sg_finaliza_saida, 0.0)
                            total_saida += result['saida'].get(sg_finaliza_saida, 0.0)
                        else:
                            saida = ''
                            imprime = False
                else:
                    saida = ''
                if finalizadora:
                    imprime = True
                    sg_finaliza = finalizadora.pop()
                    ds_finaliza = finalizadora.pop()
                    finaliz = '%10.2f' % result['totais'].get(sg_finaliza, 0.0)
                else:
                    ds_finaliza = ''
                    finaliz = ''
                if imprime:
                    listagem.imprimeLinha(linha % (entrada, saida, ds_finaliza, finaliz))
            entrada = '%15.2f' % result['vendas']
            total_entrada += result['vendas']
            listagem.imprimeLinha('| Vendas                 *                        *                               |')
            listagem.imprimeLinha(linha % ('', '', '', ''))
            listagem.imprimeLinha(linha % (entrada, '', '', ''))
            listagem.imprimeLinha('|----- Total Entrada-----*---Total Saida----------*----Diferenca------------------|')

            listagem.imprimeLinha('|      %15.2f   |          %12.2f  | %11.2f  %15s  |' % (
            total_entrada, total_saida, total_entrada - total_saida, ''))
            if resumo:
                finalizadora = ['Dinheiro :', 'D', 'Cartao :', 'C', 'Cheque :', 'H', 'Ticket :', 'T', 'Outros :', 'O']
                listagem.imprimeLinha(
                    '|---Vendas p/ finalizadora -------------------------------------------------------|')
                while finalizadora:
                    sg_finaliza = finalizadora.pop()
                    ds_finaliza = finalizadora.pop()
                    listagem.imprimeLinha('|   %10s %15.2f                                                    |' % (
                        ds_finaliza, result['resumo']['totais'].get(sg_finaliza, 0.0)))
            listagem.imprimeLinha('*---------------------------------------------------------------------------------*')
            listagem.imprimeLinha('')
            listagem.imprimeLinha('')
            contador += 1
            if contador == 3:
                for i in range(1, 14):
                    listagem.imprimeLinha('')
                contador = 1
                # ****************************************************************************************************************************************

        listagem.finalizaImpressao()

        self.obj = external_pdf(pdf_file.getvalue())
        self.obj.render()
        pdf_file.close()
        return (self.obj.pdf, 'pdf')


report_fechamento_ecf('report.fechamento.ecf')


class report_fechamento_ecf_osv(osv.TransientModel):
    _name = 'fechamento.ecf.osv'
    _description = 'Fechamento Ecf'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', required=True),
        'data_ini': fields.date('Data', help='Data inicial geracao relatorio', required=True),
        'ecf_id': fields.many2one("ecf.ecf", 'Ecf', ondelete="cascade"),  # ID DO ECF
        'resumo': fields.boolean('Resumo Vendas p/ Finalizadora')
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id,
        'data_ini': lambda *a: time.strftime('%Y-%m-%d'),
        'resumo': lambda *a: True,
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        fechamento_ecf_osv_dados = self.read(cr, uid, ids, [], context=context)
        for x in fechamento_ecf_osv_dados:
            company_id = x.get('company_id')
            data_ini = x.get('data_ini')
            ecf_id = x.get('ecf_id')
            resumo = x.get('resumo')
        if isinstance(company_id, tuple):
            company_id = company_id[0]
        res_company_dados = self.pool.get("res.company").read(cr, uid, [company_id], ['name'], context=context)
        for x in res_company_dados:
            nome_empresa = x.get('name')
        datas = {
            'company_id': company_id,
            'nome_empresa': nome_empresa,
            'data_ini': data_ini,
            'ecf_id': ecf_id,
            'resumo': resumo,
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'fechamento.ecf',
            'datas': datas,
        }


report_fechamento_ecf_osv()
