# -*- encoding: utf-8 -*-
from os import path, listdir, remove, rename
from time import strptime
from fnmatch import fnmatch
from dateutil.parser import parse
from datetime import date
from openerp.osv import osv, fields
from openerp.addons.adm import print_log
from openerp.tools.translate import _
import psycopg2
import psycopg2.extras
from decimal import Decimal, ROUND_DOWN
from datetime import datetime
import logging

_logger = logging.getLogger(__name__)

class ecf_importar_cupom_osv(osv.TransientModel):
    """
    Wizard de importacao para ECF.
    """

    cliente_id = None
    _company_id = False

    def convert_data_loja_caixa_nro(self, data, loja, caixa, nro_cupom):
        self.erro = ''
        dData = iLoja = iCaixa = iNroCupom = None

        try:
            dData = parse(data, yearfirst=True).date().strftime("%d-%m-%Y")
        except:
            self.erro += _("\nCaractere Invalido na coluna DATA - %s;") % data

        try:
            iLoja = int(loja)
        except:
            self.erro += _("\nCaractere Invalido na coluna LOJA - %s;") % loja
        try:
            iCaixa = int(caixa)
        except:
            self.erro += _("\nCaractere Invalido na coluna CAIXA - %s;") % caixa
        try:
            iNroCupom = int(nro_cupom)
        except:
            self.erro += _("\nCaractere Invalido na coluna Nro CUPOM - %s;") % nro_cupom

        if self.erro:
            return False
        else:
            return dict(data=str(dData), loja=iLoja, caixa=iCaixa, nro_cupom=iNroCupom)

    def recupera_linha_erro(self, arquivo_linha, posicao_inicial, posicao_final):
        txt_log = ''
        if arquivo_linha and posicao_inicial and posicao_final:
            tamanho_campo = posicao_final - posicao_inicial
            tamanho_fragmento_linha = round((80 - tamanho_campo) / 2) - 1
            posicao_inicial_linha = int(
                posicao_inicial - tamanho_fragmento_linha if posicao_inicial - tamanho_fragmento_linha > 0 else 0)
            txt_log = arquivo_linha[posicao_inicial_linha:posicao_inicial_linha + 80]
        return txt_log

    def cria_log(self, cr, uid, nrocupom, linha, coluna, col_fim, erro, cupom_id=None, context=None):
        # nao remover este tratamento, pois ele e usado na validacao de cupons xz quando lancados manualmente. Tiago
        context = context and context.copy() or {}
        try:
            line = self.line
        except:
            line = ''
        try:
            arquivo = self.arquivo
        except:
            arquivo = ''
        try:
            log_erro = self.recupera_linha_erro(line, coluna, col_fim)
            res = self.pool.get('ecf.cupom.log').create(cr, uid,
                                                        {'data_criacao': date.today(), 'nome_arquivo_log': arquivo,
                                                         'numero_cupom': nrocupom, 'numero_linha_log': linha,
                                                         'numero_coluna_log': coluna, 'txt_log': log_erro,
                                                         'txt_erro': erro, 'cupom_id': cupom_id}, context)
        except:
            res = False

        return res

    def CupomCC(self, cr, uid, ids, context):
        res = False
        erro = False
        dData = ''
        iLoja = ''
        iCaixa = ''
        iNroCupom = ''
        sClienteContraVale = ''
        fValorContraVale = False
        sTipo_Capa = ''
        fValorDescAcres = False
        sHoraInicio = False
        sHoraFim = False
        fTotalCupom = False
        sAlteracao = ''
        operador = ''
        vendedor = ''
        self.mov_capa_id = False
        self.cupom_capa_lancado = None
        conversao = self.convert_data_loja_caixa_nro(self.line[2:10], self.line[10:14], self.line[14:17],
                                                     self.line[17:23])
        if conversao:
            dData = conversao.get('data') or ''
            iLoja = conversao.get('loja') or ''
            iCaixa = conversao.get('caixa') or ''
            iNroCupom = conversao.get('nro_cupom') or ''
        try:
            pessoa_id = False
            if 'None' in self.line[33:48] or 'False' in self.line[33:48]:
                sCpf_Cnpj = ''
            else:
                sCpf_Cnpj = int(self.line[33:48])
                if sCpf_Cnpj == 0:
                    sCpf_Cnpj = ''
                else:
                    if len(str(sCpf_Cnpj)) > 11:
                        sCpf_Cnpj = '%014.0f' % sCpf_Cnpj
                    else:
                        sCpf_Cnpj = '%011.0f' % sCpf_Cnpj
                    obj_pessoa = self.pool.get('adm.pessoa')
                    pessoa_id = obj_pessoa.search(cr, uid, [('cpf_cnpj', '=', sCpf_Cnpj)], context=context)
                    if not pessoa_id:
                        pessoa_id = None
                    else:
                        pessoa_id = pessoa_id[0]
                        self.cliente_id = pessoa_id
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 33, 48,
                          _("Caracter Invalido na coluna CPF/CNPJ - %s") % self.line[33:48]
                          )
        try:
            strptime(self.line[48:50] + ':' + self.line[50:52], '%H:%M')
            sHoraInicio = dData + ' ' + self.line[48:50] + ':' + self.line[50:52]
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 50, 52,
                          _("Caractere Invalido na coluna HORA INICIO - %s") % self.line[48:50] + ':' + self.line[50:52]
                          )
        try:
            strptime(self.line[52:54] + ':' + self.line[54:56], '%H:%M')
            sHoraFim = dData + ' ' + self.line[52:54] + ':' + self.line[54:56]
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 52, 56,
                          _("Caractere Invalido na coluna HORA FIM - %s") % self.line[52:54] + ':' + self.line[54:56]
                          )

        sTipo_Capa = self.line[56:57]
        if (sTipo_Capa and "AbBCcFGMNPRTUVXWwZ".count(sTipo_Capa)):
            self.cupom_venda_cancel = sTipo_Capa
        else:
            erro = True
            sTipo_Capa = ''
            self.cria_log(cr, uid, iNroCupom, self.linha, 56, 57,
                          _("Caractere Invalido na coluna TIPO - %s") % self.line[56:57]
                          )
        if self.line[57:58] in ['A', 'D', ' ']:
            sAlteracao = self.line[57:58]
        else:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 57, 58,
                          _("Caractere Invalido na coluna ALTERACAO - %s") % self.line[57:58]
                          )
        try:
            fValorDescAcres = round(float(self.line[58:70]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 58, 70,
                          _("Caractere Invalido na coluna VALOR DESCONTO/ACRESCIMO - %s") % self.line[58:70]
                          )
        try:
            fTotalCupom = round(float(self.line[70:82]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 70, 82,
                          _("Caractere Invalido na coluna TOTAL CUPOM - %s") % self.line[70:82]
                          )
        sClienteContraVale = self.line[82:87]
        try:
            fValorContraVale = round(float(self.line[117:129]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 117, 129,
                          _("Caractere Invalido na coluna VALOR CONTRA VALE - %s") % self.line[117:129]
                          )
        operador = self.line[24:33]
        operador_id = 0
        if not operador.isdigit():
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 24, 33,
                          _("Caractere Invalido na coluna OPERADOR - %s") % self.line[24:33]
                          )
        else:
            operador = int(operador)
            operador_id = self.pool.get('rh.funcionario').search(
                cr, uid, [('ecf_operador', '=', operador)], context=context) or False
            if operador_id and isinstance(operador_id, list):
                operador_id = operador_id[0]

        vendedor = self.line[129:135].replace('\r\n', '')
        vendedor_id = False
        if not vendedor:
            self.cria_log(cr, uid, iNroCupom, self.linha, 130, 135,
                          _("O codigo do VENDEDOR nao esta contido no cupom"))
        elif not vendedor.isdigit():
            self.cria_log(cr, uid, iNroCupom, self.linha, 130, 135,
                          _("Caractere Invalido na coluna VENDEDOR - %s") % self.line[129:135])
        elif vendedor == '000000':
            vendedor_id = False
        else:
            vendedor = int(vendedor)
            if not vendedor:
                self.cria_log(cr, uid, iNroCupom, self.linha, 130, 135,
                              _("Nao ha VENDEDOR registrado para o cupom"))

            else:
                vendedor_id = self.pool.get('vendas.vendedor').search(cr, uid, [('id', '=', vendedor)], context=context)
                if vendedor_id and isinstance(vendedor_id, list):
                    vendedor_id = vendedor_id[0]
                if not vendedor_id:
                    self.cria_log(cr, uid, iNroCupom, self.linha, 130, 135,
                                  _("VENDEDOR nao cadastrado."))

        sql = """
            SELECT
                id
            FROM ecf_ecf
            WHERE
                loja = %s
                AND caixa = %s
                AND company_id = %s
        """ % (iLoja, iCaixa, self._company_id)
        cr.execute(sql)
        ecf_dados = cr.dictfetchall()
        if not ecf_dados:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, None, None,
                          _("Nao foi encontrado nenhum ECF cadastrado no sistema para o CAIXA: %s"
                            "e LOJA: %s") % (str(iCaixa), str(iLoja)))

        if ecf_dados and isinstance(ecf_dados, list):
            ecf_dados = ecf_dados[0]

        if not erro:
            vals = {'coo': iNroCupom,
                    'ecf_id': ecf_dados.get('id'),
                    'cliente_contra_vale': sClienteContraVale,
                    'valor_contra_vale': fValorContraVale,
                    'tipo_cupom': sTipo_Capa,
                    'valor_alteracao': fValorDescAcres,
                    'date_cupom': dData,
                    'hora_cupom_inicio': sHoraInicio,
                    'hora_cupom_fim': sHoraFim,
                    'operador': operador_id,
                    'cliente_id': pessoa_id,
                    'company_id': self._company_id,
                    'total_cupom': fTotalCupom,
                    'ecc_vendedor_id': vendedor_id,
                    'ecfccp_operador_no_supervisor': operador,
                    }
            if sAlteracao != ' ':
                vals['alteracao'] = sAlteracao
            context['sig_force_company'] = True
            cupom_lancado = self.pool.get('ecf.cupons.capa').search(
                cr, uid, [('date_cupom', '=', dData), ('ecf_id', '=', ecf_dados.get('id')),
                          ('coo', '=', iNroCupom),
                          ('company_id', '=', self._company_id)], context=context)
            del context['sig_force_company']
            context['importacao_cupom'] = True
            if not cupom_lancado:
                context['sig_force_company'] = True
                res = self.pool.get('ecf.cupons.capa').create(cr, uid, vals, context)
                del context['sig_force_company']
            else:
                cupons_capa_dados = self.pool.get('ecf.cupons.capa').read(cr, uid, cupom_lancado, ['state'],
                                                                          context=context)
                for x in cupons_capa_dados:
                    cupom_validado = x.get('state')
                if cupom_validado != 'V':
                    self.pool.get('ecf.cupons.capa').unlink(cr, uid, cupom_lancado)
                    cr.commit()
                    context['sig_force_company'] = True
                    res = self.pool.get('ecf.cupons.capa').create(cr, uid, vals, context)
                    del context['sig_force_company']
                else:
                    res = cupom_lancado[0]
        return res

    def CupomCI(self, cr, uid, ids, context):
        erro = False
        res = False
        dData = ''
        iLoja = None
        iCaixa = None
        iNroCupom = None
        iSequencia = None
        sItem = ''
        fQuantidade = None
        fPrecoTotal = None
        fDesconto = None
        fAcrescimo = None
        bVendaCancelamento = ''
        iTributacaoRetag = ''
        res_fiscal_tributacao = None
        sAliquota = ''
        sBase = ''
        conversao = self.convert_data_loja_caixa_nro(self.line[2:10], self.line[10:14], self.line[14:17],
                                                     self.line[17:23])
        if conversao:
            dData = conversao.get('data') or ''
            iLoja = conversao.get('loja') or ''
            iCaixa = conversao.get('caixa') or ''
            iNroCupom = conversao.get('nro_cupom') or ''
        try:
            iSequencia = int(self.line[23:26])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 23, 26,
                          _("Caractere Invalido na coluna SEQUENCIA - %s") % self.line[23:26]
                          )
        try:
            int(self.line[26:40])
            sItem = self.line[26:40]
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 26, 40,
                          _("Caractere Invalido na coluna CODIGO DO ITEM - %s") % self.line[26:40]
                          )
        try:
            fQuantidade = float(self.line[42:53]) / 1000
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 42, 53,
                          _("Caractere Invalido na coluna QUANTIDADE - %s") % self.line[42:53]
                          )
        try:
            fPrecoTotal = round(float(self.line[53:65]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 53, 65,
                          _("Caractere Invalido na coluna PRECO TOTAL - %s") % self.line[53:65]
                          )
        try:
            fDesconto = round(float(self.line[65:77].replace('-', '0')) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 65, 77,
                          _("Caractere Invalido na coluna DESCONTO - %s") % self.line[65:77]
                          )
        try:
            fAcrescimo = round(float(self.line[77:89]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 77, 89,
                          _("Caractere Invalido na coluna ACRESCIMO - %s") % self.line[77:89]
                          )
        try:
            bVendaCancelamento = self.line[89:90]
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 89, 90,
                          _("Caractere Invalido na coluna VENDA CANCELAMENTO (DEVE HAVER OS VALORES: V - VENDA OU C - CANCELAMENTO)\
                ") % self.line[89:90])
        try:
            sTributacaoEcf = self.line[90:93]
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 90, 93,
                          _("Caractere Invalido na coluna TRIBUTACAO NA ECF - %s") % self.line[90:93])

        try:
            iTributacaoRetag = int(self.line[93:95])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 93, 95,
                          _("Caractere Invalido na coluna TRIBUTACAO NO RETAGUARDA - %s") % self.line[93:95]
                          )
        if not iTributacaoRetag:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 93, 95,
                          _("Caractere Invalido na coluna TRIBUTACAO NO RETAGUARDA - %s") % self.line[93:95]
                          )
        try:
            sAliquota = self.line[95:99]
            sAliquota = float(sAliquota) / 100
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 95, 99,
                          _("Caractere Invalido na coluna ALIQUOTA - %s") % self.line[95:99])
        try:
            sBase = self.line[99:104]
            sBase = float(sBase) / 100
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 99, 104,
                          _("Caractere Invalido na coluna BASE - %s") % self.line[99:104])

        obj_ecf_cupons_item = self.pool.get('ecf.cupons.item')
        if not erro:
            cupom_id = self.pool.get('ecf.cupons.capa').check_cupom(
                cr, uid, iLoja, iCaixa, iNroCupom, dData, 'CI', self._company_id, context=context)
            if cupom_id:
                cupom_lancado = self.pool.get('ecf.cupons.item').search(
                    cr, uid, [('capa_id', '=', cupom_id), ('sequencia', '=', iSequencia),
                              ('item', '=', sItem)], context=context)
                if not cupom_lancado:
                    vals_item = {
                        'capa_id': cupom_id,
                        'item': sItem,
                        'quantidade': fQuantidade,
                        'preco_total': fPrecoTotal,
                        'desconto': fDesconto,
                        'acrescimo': fAcrescimo,
                        'venda_cancelamento': bVendaCancelamento,
                        'tributacao_ecf': sTributacaoEcf,
                        'tributacao_retag': iTributacaoRetag,
                        'aliquota': sAliquota,
                        'base': (sBase / 100.0) * fPrecoTotal,
                        'sequencia': iSequencia
                    }
                    obj_ecf_cupons_item.create(cr, uid, vals_item, context=context)
            else:
                erro = True
        if erro:
            return False
        else:
            return 'lancado'

    def CupomCF(self, cr, uid, ids, context):
        erro = False
        res = False
        dData_Finalizadora = ''
        iLoja_Finalizadora = None
        iCaixa_Finalizadora = None
        iNroCupom_Finalizadora = None
        iSequencia_Finalizadora = False
        iFinalizadora = False
        sTipo_Finalizadora = False
        fValor_Finalizadora = False
        sTroco = False
        sNroBanco = False
        sAgencia = False
        sNroCheque = False
        sConta = False
        sJuros = False
        sCNPJ = False
        sAvista_Aprazo_Finalizadora = False
        sValorTaxaAdm = False
        sClienteAdm = False
        sDescontos = False
        sAcrescimo = False
        conversao = self.convert_data_loja_caixa_nro(self.line[2:10], self.line[10:14], self.line[14:17],
                                                     self.line[17:23])
        if not conversao:
            erro = True
        else:
            dData_Finalizadora = conversao.get('data') or ''
            iLoja_Finalizadora = conversao.get('loja') or ''
            iCaixa_Finalizadora = conversao.get('caixa') or ''
            iNroCupom_Finalizadora = conversao.get('nro_cupom') or ''
        try:
            iSequencia_Finalizadora = int(self.line[23:26])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 23, 26,
                          _("Caractere Invalido na coluna SEQUENCIA - %s") % self.line[23:26]
                          )
        try:
            iFinalizadora = int(self.line[26:29])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 26, 29,
                          _("Caractere Invalido na coluna FINALIZADORA - %s") % self.line[26:29]
                          )

        if self.line[29:30] in ['D', 'H', 'T', 'C', 'O']:
            sTipo_Finalizadora = self.line[29:30]
        else:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 29, 30,
                          _("Caractere Invalido na coluna TIPO - %s") % self.line[29:30]
                          )
        try:
            fValor_Finalizadora = round(float(self.line[30:42]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 30, 42,
                          _("Caractere Invalido na coluna VALOR - %s") % self.line[30:42]
                          )
        try:
            sTroco = round(float(self.line[42:54]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 42, 54,
                          _("Caractere Invalido na coluna TROCO - %s") % self.line[42:54]
                          )
        try:
            sNroBanco = int(self.line[54:57])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 54, 57,
                          _("Caracter Invalido na coluna NUMERO BANCO - %s") % self.line[54:57]
                          )
        try:
            sAgencia = int(self.line[57:62])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 57, 62,
                          _("Caractere Invalido na coluna AGENCIA - %s") % self.line[57:62]
                          )
        try:
            sNroCheque = int(self.line[62:72])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 62, 72,
                          _("Caractere Invalido na coluna NUMERO CHEQUE - %s") % self.line[62:72]
                          )
        try:
            sConta = int(self.line[72:79])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 72, 79,
                          _("Caractere Invalido na coluna CONTA - %s") % self.line[72:79]
                          )
        try:
            sJuros = round(float(self.line[79:91]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 79, 91,
                          _("Caractere Invalido na coluna JUROS - %s") % self.line[79:91]
                          )
        try:
            sCNPJ = int(self.line[91:106])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 91, 106,
                          _("Caractere Invalido na coluna CNPJ - %s") % self.line[91:106]
                          )
        if self.line[106:107] in ['V', 'P']:
            sAvista_Aprazo_Finalizadora = self.line[106:107]
        else:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 106, 107,
                          _(
                              "Caractere Invalido na coluna VENDA CANCELAMENTO (DEVE HAVER OS VALORES: V - a VISTA OU P - PRAZO) - %s") % self.line[
                                                                                                                                          106:107]
                          )
        try:
            sValorTaxaAdm = round(float(self.line[107:119]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 107, 119,
                          _("Caractere Invalido na coluna VALOR TAXA ADM - %s") % self.line[107:119]
                          )
        try:
            sClienteAdm = int(self.line[119:124])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 119, 124,
                          _("Caractere Invalido na coluna CLIENTE ADM - %s") % self.line[119:124]
                          )
        try:
            sDescontos = round(float(self.line[124:136]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 124, 136,
                          _("Caractere Invalido na coluna DESCONTOS - %s") % self.line[124:136]
                          )
        try:
            sAcrescimo = round(float(self.line[137:148]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, 137, 148,
                          _("Caractere Invalido na coluna ACRESCIMOS - %s") % self.line[137:148]
                          )
        cupom_id = self.pool.get('ecf.cupons.capa').check_cupom(
            cr, uid, iLoja_Finalizadora, iCaixa_Finalizadora, iNroCupom_Finalizadora,
            dData_Finalizadora, 'CF', self._company_id, context=context)
        if cupom_id:
            finalizadora_cupom_dados = self.pool.get('ecf.cupons.capa').get_cupom_dados(cr, uid, cupom_id, context)
            if finalizadora_cupom_dados:
                obj_adm_condicao = self.pool.get('adm.condicao')
                obj_adm_forma_pgto = self.pool.get('adm.forma.pgto')
                forma_ativas_ids = obj_adm_forma_pgto.search(cr, uid, [('active', '=', True)], context=context)
                condicoes_ativas_ids = obj_adm_condicao.search(cr, uid, [('active', '=', True)], context=context)
                finalizadora_ids = self.pool.get('adm.condicao.forma').search(cr, uid,
                                                                              [('finalizadora_na_ecf', '=',
                                                                                iFinalizadora),
                                                                               ('forma_pagamento_id', 'in',
                                                                                forma_ativas_ids),
                                                                               ('adm_condicao_id', 'in',
                                                                                condicoes_ativas_ids)], context=context)

                if not finalizadora_ids:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, None, None,
                                  _(
                                      "Nao foi encontrado nenhuma FINALIZADORA ativa cadastrada no sistema com o codigo: %s") % (
                                  str(iFinalizadora))
                                  )

                else:
                    if sTipo_Finalizadora == 'C' and not sClienteAdm:
                        context_aux = context.copy()
                        context_aux['super_read_adm_pessoa'] = True
                        campos = {'forma_pagamento_id': {'admfpg_cartao_pessoa_id':['codigo']}}
                        dados_fin = self.pool.get('adm.utils').admutl_read_browse(
                            cr, uid, finalizadora_ids, 'adm.condicao.forma', campos,
                            context=context_aux)[0]
                        codigo_cliente_adm = str()
                        if dados_fin.get('forma_pagamento_id').get('admfpg_cartao_pessoa_id'):
                            codigo_cliente_adm = dados_fin.get('forma_pagamento_id').get(
                                'admfpg_cartao_pessoa_id').get('codigo')
                        sClienteAdm = codigo_cliente_adm

                    # GRAVA A FINALIZADORA DO CUPOM
                    if not erro:
                        cupom_lancado = self.pool.get('ecf.cupons.finalizacao').search(cr, uid,
                                                   [('cupom_id', '=', cupom_id),
                                                    ('sequencia', '=',iSequencia_Finalizadora),
                                                    ('finalizadora_id', '=',finalizadora_ids[0])],
                                                   context=context)
                        if not cupom_lancado:
                            res = self.pool.get('ecf.cupons.finalizacao').create(cr, uid,
                                                     {'cupom_id': cupom_id,
                                                      'coo': iNroCupom_Finalizadora,
                                                      'finalizadora_id': finalizadora_ids[0],
                                                      'ecffin_finalizadora_na_ecf': iFinalizadora,
                                                      'data': dData_Finalizadora,
                                                      'loja': iLoja_Finalizadora,
                                                      'caixa': iCaixa_Finalizadora,
                                                      'valor': fValor_Finalizadora,
                                                      'sequencia': iSequencia_Finalizadora,
                                                      'tipo_finalizadora': sTipo_Finalizadora,
                                                      'avista_aprazo': sAvista_Aprazo_Finalizadora,
                                                      'troco': sTroco,
                                                      'numero_banco': sNroBanco,
                                                      'agencia': sAgencia,
                                                      'numero_cheque': sNroCheque,
                                                      'conta': sConta, 'juros': sJuros,
                                                      'cnpj': sCNPJ,
                                                      'valor_taxa_adm': sValorTaxaAdm,
                                                      'cliente_adm': sClienteAdm,
                                                      'desconto': sDescontos,
                                                      'acrescimo': sAcrescimo
                                                      }, context)
                        else:
                            res = 'lancado'
                    if finalizadora_cupom_dados.get('ecf_id'):
                        if not self._company_id:
                            erro = True
                            self.cria_log(cr, uid, iNroCupom_Finalizadora, self.linha, None, None,
                                          _("Nao foi possivel localizar a Empresa logada.")
                                          )

        if erro or not res:
            return False
        else:
            return res

    def CupomCV(self, cr, uid, ids, context):
        res = False
        erro = False
        iSequencia = None
        sSequenciaVencim = None
        fValorParcela = None
        dVencimento = None
        conversao = self.convert_data_loja_caixa_nro(self.line[2:10], self.line[10:14], self.line[14:17],
                                                     self.line[17:23])
        if not conversao:
            erro = True
        else:
            dData = conversao.get('data') or ''
            iLoja = conversao.get('loja') or ''
            iCaixa = conversao.get('caixa') or ''
            iNroCupom = conversao.get('nro_cupom') or ''
        try:
            iSequencia = int(self.line[23:26])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 23, 26,
                          _("Caractere Invalido na coluna SEQUENCIA - %s") % self.line[23:26]
                          )
        try:
            sSequenciaVencim = int(self.line[26:28])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 26, 28,
                          _("Caractere Invalido na coluna SEQUENCIA VENCIMENTO - %s") % self.line[26:28]
                          )
        try:
            fValorParcela = round(float(self.line[28:40]) / 100, 2)
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 28, 40,
                          _("Caractere Invalido na coluna VALOR PARCELA - %s") % self.line[28:40]
                          )
        try:
            dVencimento = parse(self.line[40:48], yearfirst=True).date().strftime("%d-%m-%Y")
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 40, 48,
                          _("Caractere Invalido na coluna VENCIMENTO - %s") % self.line[40:48]
                          )
        cupom_id = self.pool.get('ecf.cupons.capa').check_cupom(
            cr, uid, iLoja, iCaixa, iNroCupom, dData, 'CV', self._company_id, context=context)
        if cupom_id:  # a funcao check_cupom ja adiciona mensagem caso nao seja encontrado os dados
            vencimento_cupom_dados = self.pool.get('ecf.cupons.capa').get_cupom_dados(cr, uid, cupom_id, context)
            if vencimento_cupom_dados:
                if not self._company_id:
                    self.cria_log(cr, uid, iNroCupom, self.linha, None, None,
                                  _("Nao foi possivel localizar a Empresa logada.")
                                  )
                obj_cupom_finalizacao = self.pool.get('ecf.cupons.finalizacao')
                cf_id = obj_cupom_finalizacao.search(cr, uid,
                                                     [('loja', '=', iLoja), ('caixa', '=', iCaixa),
                                                      ('coo', '=', iNroCupom),
                                                      ('sequencia', '=', iSequencia)], context=context)

                if cf_id:
                    if not erro:
                        vals_vencimentos = {'data_vencimento': dVencimento,
                                            'valor': fValorParcela,
                                            'finalizacao_id': cf_id[0]
                                            }
                        res = self.pool.get('ecf.cupons.vencimento').create(
                            cr, uid, vals_vencimentos, context)
                else:
                    erro = True
                    self.cria_log(
                        cr, uid, iNroCupom, self.linha, None, None,
                        _("Nao foi encontrada nenhuma FINALIZADORA cadastrada no sistema"))
        if erro or not res:
            return False
        else:
            return res

    def CupomCZ(self, cr, uid, ids, context):
        erro = False
        res = False
        sTipo = None
        iCRZ = None
        iCupomInicial = None
        iCupomFinal = None
        fGTInicial = None
        fGTFinal = None
        fCancelamentos = None
        fDescontos = None
        fAcrescimos = None
        iCRO = None
        sTributacaonaECF = None
        iTributacaonaRetag = None
        fBaseImposto = 0
        sTipoFinalizadora = None
        fValorFinalizadora = None
        fValorBruto = None
        fTroco = None
        iFinalizadora = None

        conversao = self.convert_data_loja_caixa_nro(self.line[2:10], self.line[10:14], self.line[14:17],
                                                     self.line[17:23])
        if not conversao:
            erro = True
        else:
            dData = conversao.get('data') or ''
            iLoja = conversao.get('loja') or ''
            iCaixa = conversao.get('caixa') or ''
            iNroCupom = conversao.get('nro_cupom') or ''
        try:
            sTipo = int(self.line[23:24])
        except:
            erro = True
            self.cria_log(cr, uid, iNroCupom, self.linha, 23, 24,
                          _("Caractere Invalido na coluna TIPO - %s") % self.line[23:24]
                          )
        cupom_id = self.pool.get('ecf.cupons.capa').check_cupom(
            cr, uid, iLoja, iCaixa, iNroCupom, dData, 'CZ', self._company_id, self.arquivo,
            self.linha, context=context)
        if cupom_id:  # funcao ja adiciona mensagem caso ocorra erro
            obj_ecf_xz = self.pool.get('ecf.cupons.xz')
            obj_ecf_cupons_xz_totalizadores = self.pool.get('ecf.cupons.xztotalizadores')
            obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
            cupom_dados = self.pool.get('ecf.cupons.capa').read(cr, uid, cupom_id, ['ecf_id'], context=context)
            if sTipo == 1:
                try:
                    iCRZ = int(self.line[24:34])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 24, 34,
                                  _("Caractere Invalido na coluna CRZ - %s") % self.line[24:34]
                                  )
                try:
                    iCupomInicial = int(self.line[34:40])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 34, 40,
                                  _("Caractere Invalido na coluna CUPOM INICIAL - %s") % self.line[34:40]
                                  )
                try:
                    iCupomFinal = int(self.line[40:46])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 40, 46,
                                  _("Caractere Invalido na coluna CUPOM FINAL - %s") % self.line[40:46]
                                  )
                try:
                    fGTInicial = round(float(self.line[46:64]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 46, 64,
                                  _("Caractere Invalido na coluna GT INICIAL - %s") % self.line[46:64]
                                  )
                try:
                    fGTFinal = round(float(self.line[64:82]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 64, 82,
                                  _("Caractere Invalido na coluna GT FINAL - %s") % self.line[64:82]
                                  )
                try:
                    fCancelamentos = round(float(self.line[82:94]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 82, 94,
                                  _("Caractere Invalido na coluna CANCELAMENTOS - %s") % self.line[82:94]
                                  )
                try:
                    fDescontos = round(float(self.line[94:106]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 94, 106,
                                  _("Caractere Invalido na coluna DESCONTOS - %s") % self.line[94:106]
                                  )
                try:
                    fAcrescimos = round(float(self.line[106:118]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 106, 118,
                                  _("Caractere Invalido na coluna ACRESCIMOS - %s") % self.line[106:118]
                                  )
                try:
                    iCRO = int(self.line[118:122])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 118, 122,
                                  _("Caractere Invalido na coluna CRO - %s") % self.line[118:122]
                                  )
                if cupom_dados.get('ecf_id', False):
                    cupom_lancado = self.pool.get('ecf.cupons.xz').search(cr, uid, [('cupom_id', '=', cupom_id)],
                                                                          context=context)
                    if not cupom_lancado:
                        context['importacao_xz'] = True
                        res = obj_ecf_xz.create(cr, uid, {'cupom_final': iCupomFinal,
                                                          'acrescimos': fAcrescimos,
                                                          'crz': iCRZ,
                                                          'gt_final': fGTFinal,
                                                          'cancelamentos': fCancelamentos,
                                                          'gt_inicial': fGTInicial,
                                                          'cupom_id': cupom_id,
                                                          'cupom_inicial': iCupomInicial,
                                                          'descontos': fDescontos,
                                                          'cro': iCRO
                                                          }, context)
                    else:
                        res = 'lancado'

            elif sTipo == 2:

                try:
                    sTributacaonaECF = int(self.line[26:29])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 26, 29,
                                  _("Caractere Invalido na coluna TRIBUTACAO ECF - %s") % self.line[26:29]
                                  )

                try:
                    iTributacaonaRetag = int(self.line[29:31])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 29, 31,
                                  _("Caractere Invalido na coluna TRIBUTACAO RETAGUARDA - %s") % self.line[29:31]
                                  )
                try:
                    fBaseImposto = round(float(self.line[31:43]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 31, 43,
                                  _("Caractere Invalido na coluna VALOR - %s") % self.line[31:43]
                                  )

                cupom_lancado = obj_ecf_cupons_xz_totalizadores.search(
                    cr, uid, [('cupom_id', '=', cupom_id),
                              ('tributacao_retag_id', '=', iTributacaonaRetag),
                              ('tributacao_na_ecf', '=', sTributacaonaECF)], context=context)
                aliquota = 0.0
                if sTributacaonaECF:
                    sTributacaonaECF1 = '0%s' % sTributacaonaECF
                    tributacao_id = obj_fiscal_tributacao.search(
                        cr, uid, [('tributacao_ecf', 'in', (str(sTributacaonaECF),
                                                            sTributacaonaECF1))], context=context)
                    if tributacao_id:
                        browse_fiscal_tributacao = obj_fiscal_tributacao.browse(cr, uid,
                                                                                tributacao_id[0])
                        aliquota = browse_fiscal_tributacao.aliquota
                        imposto = browse_fiscal_tributacao.encargo_id.sigla
                        tipo_totalizador = browse_fiscal_tributacao.cst_id.tipo
                        if tipo_totalizador not in ('I', 'O', 'U', 'D', 'T'):
                            tipo_totalizador = False
                        if imposto == 'ICMS':
                            imposto = 'C'
                        elif imposto == 'ISS':
                            imposto = 'S'
                        else:
                            imposto = False
                        valor_imposto = fBaseImposto * (aliquota / 100)
                        valor_imposto = Decimal(str(valor_imposto)).quantize(Decimal('.01'),
                                                                             rounding=ROUND_DOWN)
                        if not cupom_lancado:
                            if imposto:
                                res = obj_ecf_cupons_xz_totalizadores.create(cr, uid, {
                                    'base_imposto': fBaseImposto,
                                    'valor_imposto': valor_imposto,
                                    'cupom_id': cupom_id,
                                    'tributacao_retag_id': iTributacaonaRetag,
                                    'tributacao_na_ecf': sTributacaonaECF,
                                    'aliquota': aliquota,
                                    'imposto': imposto,
                                    'tipo_totalizador': tipo_totalizador,
                                }, context)
                        else:
                            dados_xztotalizadores = obj_ecf_cupons_xz_totalizadores.read(
                                cr, uid, cupom_lancado, ['valor_imposto'], context=context)
                            if isinstance(dados_xztotalizadores, list):
                                dados_xztotalizadores = dados_xztotalizadores[0]
                            novo_fvalor = valor_imposto + dados_xztotalizadores.get('valor_imposto')
                            obj_ecf_cupons_xz_totalizadores.write(cr, uid, cupom_lancado,
                                                                  {'valor_imposto': novo_fvalor})
                            res = 'lancado'
                    else:
                        erro = True
                        self.cria_log(cr, uid, iNroCupom, self.linha, None, None,
                                      _("Nao foi encontrado nenhuma TRIBUTACAO NA ECF cadastrada no"
                                        " sistema com o codigo: %s") % (self.line[26:29])
                                      )
            elif sTipo == 3:
                try:
                    sTipoFinalizadora = self.line[27:28]
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 27, 28,
                                  _("Caractere Invalido na coluna TIPO FINALIZADORA - %s") % self.line[27:28]
                                  )
                try:
                    fValorFinalizadora = round(float(self.line[29:41]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 29, 41,
                                  _("Caractere Invalido na coluna VALOR FINALIZADORA - %s") % self.line[29:41]
                                  )
                try:
                    fValorBruto = round(float(self.line[103:115]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 103, 115,
                                  _("Caractere Invalido na coluna VALOR BRUTO - %s") % self.line[103:115]
                                  )
                try:
                    fDescontos = round(float(self.line[115:127]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 115, 127,
                                  _("Caractere Invalido na coluna DESCONTOS - %s") % self.line[115:127]
                                  )
                try:
                    fAcrescimos = round(float(self.line[142:154]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 142, 154,
                                  _("Caractere Invalido na coluna ACRESCIMOS - %s") % self.line[142:154]
                                  )
                try:
                    fTroco = round(float(self.line[170:181]) / 100, 2)
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 169, 181,
                                  _("Caractere Invalido na coluna TROCO - %s") % self.line[169:181]
                                  )
                try:
                    iFinalizadora = int(self.line[24:27])
                except:
                    erro = True
                    self.cria_log(cr, uid, iNroCupom, self.linha, 24, 27,
                                  _("Caractere Invalido na coluna FINALIZADORA - %s") % self.line[24:27]
                                  )
                obj_adm_condicao = self.pool.get('adm.condicao')
                condicoes_ativas_ids = obj_adm_condicao.search(cr, uid, [('active', '=', True)], context=context)
                obj_adm_forma_pgto = self.pool.get('adm.forma.pgto')
                forma_ativas_ids = obj_adm_forma_pgto.search(cr, uid, [('active', '=', True)], context=context)
                finalizadora_id = self.pool.get('adm.condicao.forma').search(cr, uid,
                                                                             [('finalizadora_na_ecf', '=',
                                                                               iFinalizadora,),
                                                                              ('forma_pagamento_id', 'in',
                                                                               forma_ativas_ids),
                                                                              ('adm_condicao_id', 'in',
                                                                               condicoes_ativas_ids)], context=context)
                if not finalizadora_id:
                    self.cria_log(cr, uid, iNroCupom, self.linha, None, None,
                                  _(
                                      "Nao foi encontrado nenhuma FINALIZADORA ativa cadastrada no sistema com o codigo: %s") % (
                                  self.line[24:27])
                                  )
                else:
                    res = self.pool.get('ecf.cupons.finalizacao').create(cr, uid, {
                        'cupom_id': cupom_id,
                        'tipo_finalizadora': sTipoFinalizadora,
                        'valor_finalizadora': fValorFinalizadora,
                        'desconto': fDescontos,
                        'acrescimo': fAcrescimos,
                        'troco': fTroco,
                        'finalizadora_id': finalizadora_id[0],
                        'valor': fValorBruto
                    }, context)
                    res = 'lancado'
                    # me foi passado que nao serao realizados as importacoes dos tipo 4 e 9. Luis Paulo
            elif sTipo in [4, 9]:
                res = 'lancado'
            else:
                erro = True
                self.cria_log(cr, uid, iNroCupom, self.linha, 23, 24,
                              _("Caractere Invalido na coluna TIPO - %s") % self.line[23:24]
                              )
        if erro or not res:
            return False
        else:
            return res

    # Variavel para identificar se a rotina de validacao da inconsistencia de cupons esta ativa.
    # Implementacao para evitar o traceback de concorrencia de atualizacao.
    executando = False

    def button_importar(self, cr, uid, ids, context=None, file_erro=False):
        context = context and context.copy() or {}
        opts = {'CC': self.CupomCC, 'CI': self.CupomCI, 'CF': self.CupomCF, 'CV': self.CupomCV, 'CZ': self.CupomCZ}
        self.log = ''
        self.erro = ''
        self.line = ''
        self.arquivo = ''
        self.cupom_venda_cancel = ''
        self.linha = 0
        cupons_retorno = []
        arquivos_nao_deletados = ''
        dir_arq_import = ''

        sql = """
            SELECT
                company_id,
                dir_arq_import,
                estpar_tipo_exportacao
            FROM
                estoque_parametro
            WHERE
                dir_arq_import IS NOT NULL
                AND estpar_tipo_exportacao = 'A'
        """
        cr.execute(sql)
        estoque_parametro_dados = cr.dictfetchall()

        obj_ecf_cupom_capa = self.pool.get('ecf.cupons.capa')
        print_log(_("- Importando cupons (lg's) ........."))

        if not estoque_parametro_dados and not context.get('inconsistencia'):
            self.cria_log(cr, uid, None, None, None, None,_('Configure o diretorio de importacao '
                                                            'nos Parametros de Estoque/Local do '
                                                            'Arquivo de Importacao'))
            return True
        cupom_identificadores = []
        capa_id = []
        for x in estoque_parametro_dados:
            dir_arq_import = x.get('dir_arq_import')
            self._company_id = x.get('company_id')
            files = ''
            try:
                files = tuple(listdir(dir_arq_import))
            except:
                self.cria_log(
                    cr, uid, None, None, None, None, _('Local do arquivo de importacao nao '
                                                       'encontrado: %s' % dir_arq_import))
            print_log(_('- DIRETORIO: %s') % dir_arq_import)
            files_import = []
            for i in files:
                if fnmatch(i, 'lg??????.???') or fnmatch(i, '?????????.txt') or (
                            file_erro and fnmatch(i, 'lg??????.???.ERRO')):
                    files_import.append(i)

            if not files_import:
                print_log(_('- NENHUM CUPOM ENCONTRADO NO DIRETORIO'))
                self.log = _('\nNenhum arquivo de importacao encontrado no diretorio: %s') % (
                    dir_arq_import)
            else:
                for i in files_import:
                    self.arquivo = i
                    try:
                        fp = open(path.join(dir_arq_import, i))
                    except:
                        fp = False
                    if fp:
                        self.linha = 1
                        for line in iter(fp.readline, ''):
                            identificador = line[0:2]
                            self.line = line
                            if "CI;CF;CV;CZ;CC".count(identificador):
                                if identificador == 'CC':
                                    try:
                                        self.cupom_venda_cancel = line[56:57]
                                    except:
                                        self.cupom_venda_cancel = ''
                                    try:
                                        COO = int(line[17:23])
                                    except:
                                        COO = False
                                        self.cria_log(cr, uid, None, self.linha, 17, 23,
                                                      _('Caracter Invalido na coluna COO - %s') % line[17:23]
                                                      )
                                if identificador == 'CZ':
                                    try:
                                        stipo = line[23:24]
                                    except:
                                        stipo = ''
                                create_cupom = opts[identificador]
                                retorno = create_cupom(cr, uid, ids, context)
                                if retorno:
                                    if identificador == 'CC':
                                        if retorno == 'lancado':
                                            if isinstance(self.cupom_capa_lancado, list):
                                                self.cupom_capa_lancado = \
                                                    self.cupom_capa_lancado[0]
                                        else:
                                            if isinstance(retorno, list):
                                                capa_id = retorno[0]
                                            capa_id = retorno
                                cupom_identificadores.append(retorno)
                                cupons_retorno.append(retorno)
                            self.linha += 1
                        fp.close()
                        if all(cupom_identificadores):  # so entra se todas as linhas do arquivo estiverem sem erro
                            cr.commit()
                            try:
                                remove(path.join(dir_arq_import, i))
                            except:
                                arquivos_nao_deletados += '%s; ' % i
                        elif not '.ERRO' in i:
                            try:
                                rename(path.join(dir_arq_import, i),
                                       path.join(dir_arq_import, i) + '.ERRO')
                            except Exception as err:
                                raise osv.except_osv(_('Atencao'), str(err))
                        cupom_identificadores = []

            if x.get('estpar_tipo_exportacao') == 'A':
                if dir_arq_import:
                    msg = "Diretorio: %s \n*********** " \
                          "Todos Arquivos Importados com Sucesso ***********" % dir_arq_import
                    self.log = _(msg)
                else:
                    self.log = _(
                        "Diretorio: %s \n*********** Arquivo nao Importado ***********%s") % (
                        dir_arq_import, self.log)

        self.arquivo = ''

        # Ajuste referente a atividade 94567, onde diz que a funcao de validar inconsistencia deve
        # buscar apenas cupons divergentes.
        cupom_states = ['A', 'C', 'D']
        if context.get('inconsistencia'):
            cupom_states = ['A', 'D']

        cupom_ids = obj_ecf_cupom_capa.search(cr, uid, [('state', 'in', cupom_states)],
                                  order='date_cupom, hora_cupom_inicio, ecf_id, coo, state desc',
                                  context=context)
        log = self.pool.get('ecf.importar.cupom.osv').cria_log
        if self.executando:
            log(cr, uid, None, None, None, None,
                _("*********** A VALIDACAO JA ESTA SENDO EXECUTADA ***********"), False)
            cupom_ids = []
        # Foi alterado para executar de um em um para ter controle dos erros de concorrencia
        # e assim reimportar os cupons caso aconteca
        verifica_cupom = False
        for cupom_id in cupom_ids:
            try:
                verifica_cupom = obj_ecf_cupom_capa.verifica_cupons(cr, uid, [cupom_id],
                                                                    context=context)
            except psycopg2.IntegrityError:
                # Se ocorrer um erro de integridade tenta reimportar novamente
                verifica_cupom = obj_ecf_cupom_capa.verifica_cupons(cr, uid, [cupom_id],
                                                                    context=context)
            if verifica_cupom:
                self.executando = True
        if verifica_cupom:
            msg = "Diretorio: %s \n*********** " \
                  "Arquivos Importados com Divergencia. ***********\n" \
                  ">>> Favor executar a validacao dos cupoms p/ " \
                  "verificar as divergencias." % dir_arq_import
            self.log = _(msg)
        if arquivos_nao_deletados:
            self.log += '%s: %s'.decode('utf-8') % (
            _('\n\nNao foi possivel excluir o arquivo(s)'), arquivos_nao_deletados)
        if not context.get('inconsistencia'):
            self.write(cr, uid, ids, {'state': 'C', 'informacao': self.log}, context=context)
        if cupom_ids:
            self.executando = False
        return True

    _name = "ecf.importar.cupom.osv"
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', required=True),
        'state': fields.selection([('R', 'Esboco'), ('C', 'Concluido'), ('E', 'Erro')], 'Estado'),
        'informacao': fields.text('Informacao'),
    }

    _defaults = {
        'state': lambda *a: 'R',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),

    }

    def importa_cupons_supervisor(self, cr, uid, context=None):

        """
        Importa os cupons do Supervisor via banco de dados Postgres
        """

        context = context and context.copy() or {}
        context['importacao_supervisor_erp'] = True

        sql = """
            SELECT
                company_id,
                estpar_super_host,
                estpar_super_db,
                estpar_super_user,
                estpar_super_password,
                estpar_usar_master
            FROM
                estoque_parametro
            WHERE estpar_super_host IS NOT NULL
              AND estpar_super_db IS NOT NULL
              AND estpar_super_user IS NOT NULL
        """
        cr.execute(sql)
        estoque_parametro = cr.dictfetchall()
        for parametro in estoque_parametro:
            host = parametro.get('estpar_super_host')
            user = parametro.get('estpar_super_user')
            password = parametro.get('estpar_super_password') or ''
            db = parametro.get('estpar_super_db')
            if not all([host, user, password, db]):
                continue
            usar_master = parametro.get('estpar_usar_master')
            company_id = parametro.get('company_id')
            conn = False
            try:
                # CONEXAO COM O BANCO DE DADOS
                conn = psycopg2.connect(database=db, user=user, password=password, host=host,
                                        connect_timeout=3)
                _cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            except:
                if conn:
                    conn.close()
                print_log(_('Nao foi possivel conectar ao supervisor com host: %s e banco de '
                            'dados: %s. Verifique as informacoes nos parametros do Estoque' %
                            (host, db)))
                continue

            _cr.execute("SET statement_timeout = '3s'")
            list_cupom = []
            supervisor_cupom_dados = self._get_cupons_supervisor(_cr, company_id,
                                                                 usar_master=usar_master)
            print_log(_('- Importando cupons do supervisor(%s).........' % db))
            # A as operacoes abaixo devem ser tratadas e comitadas uma a uma para evitar que
            # um cupom trave o processo e impeça que os outros nao sejam importados.
            for supervisor_cupom in supervisor_cupom_dados:
                try:
                    sql_begin = """BEGIN;
                                   SAVEPOINT importa_cupom;"""
                    cr.execute(sql_begin) # erp
                    _cr.execute(sql_begin) # supervisor
                    if not supervisor_cupom.get('supcap_nro_serie'):
                        self.update_ecf_cupom_servidor(_cr, supervisor_cupom.get('id'), context)
                        sql_commit = """RELEASE SAVEPOINT importa_cupom;
                                        COMMIT;"""
                        cr.execute(sql_commit)
                        _cr.execute(sql_commit)
                        continue

                    cupom_id = self.create_ecf_cupom_capa(cr, uid, supervisor_cupom, _cr, context)
                    if cupom_id:
                        sup_mov_capa_id = supervisor_cupom.get('id')
                        self.create_ecf_cupom_itens(cr, uid, _cr, cupom_id, sup_mov_capa_id)
                        self.create_ecf_cupom_xz(cr, uid, _cr, cupom_id, sup_mov_capa_id)
                        self.create_ecf_cupom_xz_totalizadores(cr, uid, _cr, cupom_id,
                                                               sup_mov_capa_id)
                        self.create_ecf_cupom_finalizacao(cr, uid, _cr, cupom_id, sup_mov_capa_id,
                                                          context)
                        list_cupom.append(cupom_id)
                        self.update_ecf_cupom_servidor(_cr, sup_mov_capa_id, context)

                    sql_commit = """RELEASE SAVEPOINT importa_cupom;
                                    COMMIT;"""
                    cr.execute(sql_commit)
                    _cr.execute(sql_commit)
                except Exception as e:
                    print_log(_('Erro ao importar o cupom %s: %s' %
                                (supervisor_cupom.get('supcap_nro_cupom'), str(e)) ))
                    sql_rollback = """ROLLBACK TO SAVEPOINT importa_cupom;
                                      COMMIT;"""
                    cr.execute(sql_rollback)
                    _cr.execute(sql_rollback)

            qtd_cupons_importados = 0
            print_log(_('- %s cupons importados do supervisor(%s).........' %
                        (qtd_cupons_importados, db)))
            context.update({'sig_force_company': True})
            # deve buscar apenas os cupons com state 'A'(aguardando cupom de venda), 'C'(criado)
            cupom_ids = self.pool.get('ecf.cupons.capa').search(
                cr, uid, [('state', 'in', ['A', 'C'])],
                order='date_cupom, hora_cupom_inicio, ecf_id, coo, state desc',
                context=context)
            list_cupom.extend(cupom_ids)
            list_cupom = list(set(list_cupom))
            self.pool.get('ecf.cupons.capa').verifica_cupons(cr, uid, list_cupom, context=context)
            if conn:
                conn.close()
        return True

    def update_ecf_cupom_servidor(self, _cr, capa_id, context=None):
        """
        Atualiza a tabela 'supervisor_mov_cupom_servidor' para que o cupom fique como enviado.
        """
        context = context and context.copy() or {}
        sql = """
            UPDATE supervisor_mov_cupom_servidor
            SET supcse_enviado = True
            WHERE supcse_capa_id = %s
        """ % capa_id
        _cr.execute(sql)
        return True

    def _get_cupons_supervisor(self, _cr, company_id, usar_master=False):

        """
        Realiza a busca de todos os cupons do supervisor que ainda nao foram enviados
        """
        cond_empresa = ''
        if usar_master:
            cond_empresa = 'AND smc.supcap_nro_empresa = %s' % company_id

        sql = """
            SELECT smc.id,
                   smc.supcap_nro_cupom,
                   smc.supcap_loja,
                   smc.supcap_caixa,
                   smc.supcap_tipo,
                   smc.supcap_data_cupom,
                   smc.supcap_data_inicio,
                   smc.supcap_cliente,
                   smc.supcap_alteracao,
                   smc.supcap_valor_desc_acres,
                   smc.supcap_total_cupom,
                   smc.supcap_operador_id,
                   smc.supcap_cod_vendedor,
                   smc.supcap_data_inicio,
                   smc.supcap_data_fim,
                   smc.supcap_nro_serie,
                   smc.supcap_numero_pedido,
                   sco.supope_codigo
            FROM supervisor_mov_capa                 AS smc
            INNER JOIN supervisor_mov_cupom_servidor AS smcs ON smcs.supcse_capa_id = smc.id
            INNER JOIN supervisor_cad_operadores     AS sco  ON sco.id = smc.supcap_operador_id
            WHERE smcs.supcse_enviado = FALSE 
              AND smc.supcap_state IS NULL
              {cond}
            ORDER BY smc.supcap_nro_cupom,
                     smc.supcap_caixa,
                     smc.supcap_loja
        """.format(cond=cond_empresa)
        _cr.execute(sql)
        obj_adm_utils = self.pool.get('adm.utils')
        supervisor_cupom_dados = obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr)
        return supervisor_cupom_dados

    def create_ecf_cupom_capa(self, cr, uid, supervisor_cupom, _cr, context=None):

        """
        Realiza a insercao da capa dos cupons na tabela ecf_cupons_capa
        """

        context = context and context.copy() or {}
        obj_ecf_ecf = self.pool.get('ecf.ecf')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')
        obj_ecf_cupons_capa = self.pool.get('ecf.cupons.capa')
        obj_supervisor_cad_operadores = self.pool.get('supervisor.cad.operadores')
        obj_rh_funcionario = self.pool.get('rh.funcionario')

        company_id = False
        cupom_id = False
        ecf_cupom_id = False
        loja = supervisor_cupom.get('supcap_loja')
        caixa = supervisor_cupom.get('supcap_caixa')
        ecf_nro_serie = supervisor_cupom.get('supcap_nro_serie')
        sql = """
            SELECT id
            FROM ecf_ecf
            WHERE
                loja = %s
                AND caixa = %s
                AND nro_serie = '%s'
        """ % (loja, caixa, ecf_nro_serie)
        cr.execute(sql)
        ecf_id = cr.dictfetchall()
        ecf_id = map(lambda i: i.get('id'), ecf_id)
        cupom = supervisor_cupom.get('supcap_nro_cupom')
        data_cupom = supervisor_cupom.get('supcap_data_cupom')
        if not ecf_id:
            raise Exception(_('ECF nao encontrada. LOJA: %s, CAIXA: %s, NRO.SERIE: %s') %
                              (loja, caixa, ecf_nro_serie))

        ecf_id = ecf_id[0]
        company_id = obj_ecf_ecf.browse(cr, uid, ecf_id, context).company_id.id
        context['sig_force_company'] = True
        ecf_cupom_id = obj_ecf_cupons_capa.search(cr, uid, [('coo', '=', cupom),
                                                            ('ecf_id', '=', ecf_id),
                                                            ('date_cupom', '=', data_cupom),
                                                            ('company_id', '=', company_id)],
                                                  context=context)

        # Se o cupom já existe em ecf_cupons_capa,
        # atualizar supervisor_mov_cupom_servidor para supcse_enviado = True
        if ecf_cupom_id:
            sql_update_mov_cupom_servisor = """
                                UPDATE supervisor_mov_cupom_servidor
                                SET supcse_enviado = True
                                WHERE supcse_capa_id = %s
                            """ % supervisor_cupom.get('id')
            _cr.execute(sql_update_mov_cupom_servisor)
            return False # Retorna false para nao criar os itens do cupom

        codigo_operador = supervisor_cupom.get('supope_codigo')
        numero_pedido = supervisor_cupom.get('supcap_numero_pedido') or 'null'
        if codigo_operador and isinstance(codigo_operador, list):
            codigo_operador = codigo_operador[0]
        if not codigo_operador:
            codigo_operador = 'null'
        args = [('ecf_operador', '=', codigo_operador), ('company_id', '=', company_id)]
        funcionario_ids = obj_rh_funcionario.search(cr, uid, args, context=context)
        state = 'C'
        if funcionario_ids:
            if isinstance(funcionario_ids, list):
                funcionario_ids = funcionario_ids[0]
        else:
            funcionario_ids = 'null'
        tipo_cupom = supervisor_cupom.get('supcap_tipo')
        hora_inicio = supervisor_cupom.get('supcap_data_inicio')
        cliente = supervisor_cupom.get('supcap_cliente')
        pessoa_id = 'null'
        if cliente:
            pessoa_id = self.pool.get('adm.pessoa').search(
                    cr, uid, ['|',('cpf_cnpj','=', cliente[1:]),
                                  ('cpf_cnpj','=', cliente[4:])], context=context)

            if pessoa_id:
                pessoa_id=pessoa_id[0]
            else:
                pessoa_id = 'null'
                print_log('Nao foi encontrado no ERP a pessoa com o cpf/cnpj: %s' % cliente)

        fields_required = [data_cupom, cupom, company_id, tipo_cupom, hora_inicio]
        vendedor_id = obj_vendas_vendedor.search(
            cr, uid, [('id', '=', supervisor_cupom.get('supcap_cod_vendedor'))], context=context)
        if vendedor_id:
            vendedor_id = vendedor_id[0]
        else:
            vendedor_id = 'null'
        hora_cupom_fim = supervisor_cupom.get('supcap_data_fim') or 'null'
        alteracao = supervisor_cupom.get('supcap_alteracao') or 'null'
        valor_alteracao = supervisor_cupom.get('supcap_valor_desc_acres') or 'null'
        total_cupom = supervisor_cupom.get('supcap_total_cupom') or 'null'
        create_date = write_date = str(datetime.now())
        create_uid = write_uid = uid
        # cria cupons em ecf_cupons_capa
        if all(fields_required):
            sql_insert_cupom_capa = """
                INSERT INTO ecf_cupons_capa
                    (create_uid, create_date, write_date, write_uid,
                    coo, company_id, ecf_id, tipo_cupom, date_cupom, hora_cupom_inicio,
                    hora_cupom_fim, cliente_id, alteracao, valor_alteracao, total_cupom,
                    state, operador, ecfccp_operador_no_supervisor, ecc_vendedor_id,
                    ecfccp_numero_pedido)
                VALUES ({create_uid}, '{create_date}', '{write_date}', {write_uid},
                         {cupom}, {company_id}, {ecf_id}, '{tipo_cupom}', '{data_cupom}', '{hora_inicio}',
                         '{hora_cupom_fim}', {cliente_id}, '{alteracao}', {valor_alteracao}, {total_cupom},
                         '{state}', {funcionario_ids}, {codigo_operador}, {vendedor_id},
                         {numero_pedido})
                returning id;
                """.format(create_uid=create_uid,
                           create_date=create_date,
                           write_date=write_date,
                           write_uid=write_uid,
                           cupom=cupom,
                           company_id=company_id,
                           ecf_id=ecf_id,
                           tipo_cupom=tipo_cupom,
                           data_cupom=data_cupom,
                           hora_inicio=hora_inicio,
                           hora_cupom_fim=hora_cupom_fim,
                           cliente_id=pessoa_id,
                           alteracao=alteracao,
                           valor_alteracao=valor_alteracao,
                           total_cupom=total_cupom,
                           state=state,
                           funcionario_ids=funcionario_ids,
                           codigo_operador=codigo_operador,
                           vendedor_id=vendedor_id,
                           numero_pedido=numero_pedido)
            sql_insert_cupom_capa = sql_insert_cupom_capa.replace("'null'", 'null')
            try:
                cr.execute(sql_insert_cupom_capa)
                cupom_id = cr.dictfetchall()[0].get('id')
            except Exception as e:
                _logger.error(_("Erro ao na importacao do cupom '%s'. \nERRO: %s" % (cupom, str(e))))

        return cupom_id

    def create_ecf_cupom_itens(self, cr, uid, _cr, cupom_id, mov_capa_id):

        """
        Realiza a insercao dos itens dos cupons na tabela ecf_cupons_item
        """

        res = False
        sql_select_supervisor_cupom_itens = """
            SELECT smc.supcor_capa_id,
                   smc.supcor_item,
                   smc.supcor_quantidade,
                   smc.supcor_preco_total,
                   smc.supcor_venda_cancelamento,
                   smc.supcor_tributacao_ecf,
                   smc.supcor_tributacao_retag,
                   smc.supcor_item,
                   smc.supcor_sequencia,
                   smc.supcor_desconto_item,
                   smc.supcor_acrescimo_item,
                   smc.supcor_aliquota,
                   smc.supcor_base,
                   smc.supcor_cod_vendedor,
                   smc.supcor_nome_vendedor
            FROM   supervisor_mov_corpos smc
                   INNER JOIN supervisor_mov_capa smca
                           ON smc.supcor_capa_id = smca.id
            WHERE  smc.supcor_capa_id = %s
        """ % mov_capa_id
        _cr.execute(sql_select_supervisor_cupom_itens)
        obj_adm_utils = self.pool.get('adm.utils')
        supervisor_corpos_dados = obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr)
        if supervisor_corpos_dados:
            for corpo in supervisor_corpos_dados:
                capa_id = corpo.get('supcor_capa_id')
                item = corpo.get('supcor_item')
                quantidade = corpo.get('supcor_quantidade')
                preco_total = corpo.get('supcor_preco_total')
                venda_cancelamento = corpo.get('supcor_venda_cancelamento')
                tributacao_ecf = corpo.get('supcor_tributacao_ecf')
                tributacao_retag = corpo.get('supcor_tributacao_retag')
                sequencia = corpo.get('supcor_sequencia') or 'null'
                desconto_item = corpo.get('supcor_desconto_item') or 0
                acrescimo_item = corpo.get('supcor_acrescimo_item') or 0
                aliquota = corpo.get('supcor_aliquota') or 0
                base = round(((corpo.get('supcor_base') or 0) / 100.0) * preco_total, 3)
                cod_vendedor = corpo.get('supcor_cod_vendedor') or 0
                nome_vendedor = corpo.get('supcor_nome_vendedor') or 'null'
                fields_itens = [capa_id, item, quantidade, preco_total,
                                venda_cancelamento,
                                tributacao_ecf, tributacao_retag]
                if all(fields_itens):
                    valores = (cupom_id, item, sequencia, quantidade, preco_total, desconto_item,
                               acrescimo_item, venda_cancelamento, tributacao_ecf, tributacao_retag,
                               aliquota, base, cod_vendedor, nome_vendedor)
                    valores = " %s, '%s', %s, %s, %s, %s, %s, '%s', '%s', '%s', %s, %s, %s, '%s' " \
                              % valores
                    sql_insert_cupom_item = """
                        INSERT INTO  ecf_cupons_item
                                    (capa_id,
                                     item,
                                     sequencia,
                                     quantidade,
                                     preco_total,
                                     desconto,
                                     acrescimo,
                                     venda_cancelamento,
                                     tributacao_ecf,
                                     tributacao_retag,
                                     aliquota,
                                     base,
                                     ecfcit_cod_vendedor,
                                     ecfcit_nome_vendedor)
                        VALUES      (%s)
                    """ % valores
                    sql_insert_cupom_item = sql_insert_cupom_item.replace("'null'", 'null')
                    cr.execute(sql_insert_cupom_item)
                    res = True
        return res

    def create_ecf_cupom_xz(self, cr, uid, _cr, cupom_id, mov_capa_id):

        """
        Realiza a insercao dos dados X e Z dos cupons na tabela ecf_cupons_xz
        """

        res = False
        sql_select_supervisor_cupom_xz = """
            SELECT smcx.supcxz_cupom_inicial, smcx.supcxz_cupom_final, smcx.supcxz_gt_anterior,
                   smcx.supcxz_gt_atual, smcx.supcxz_cancelamentos, smcx.supcxz_descontos,
                   smcx.supcxz_acrescimos, smcx.supcxz_cro, smca.supcap_crz
            FROM supervisor_mov_corpo_xz smcx
            INNER JOIN supervisor_mov_capa smca ON smcx.supcxz_capa_id = smca.id
            WHERE smcx.supcxz_capa_id = %s
        """ % mov_capa_id
        _cr.execute(sql_select_supervisor_cupom_xz)
        obj_adm_utils = self.pool.get('adm.utils')
        supervisor_corpos_xz_dados = obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(_cr)
        if supervisor_corpos_xz_dados:
            for xz in supervisor_corpos_xz_dados:
                cupom_inicial = xz.get('supcxz_cupom_inicial') or 'null'
                cupom_final = xz.get('supcxz_cupom_final') or 'null'
                gt_inicial = xz.get('supcxz_gt_anterior') or 'null'
                gt_final = xz.get('supcxz_gt_atual') or 'null'
                cancelamentos = xz.get('supcxz_cancelamentos') or 'null'
                descontos = xz.get('supcxz_descontos') or 'null'
                acrescimos = xz.get('supcxz_acrescimos') or 'null'
                cro = xz.get('supcxz_cro') or 'null'
                crz = xz.get('supcap_crz') or 'null'

                valores = (cupom_id, cupom_inicial, cupom_final, gt_inicial, gt_final,
                           cancelamentos, descontos, acrescimos, cro, crz)
                valores = "%s, %s, %s, %s, %s, %s, %s, %s, %s, %s" % valores
                sql_insert_supervisor_cupom_xz = """
                    INSERT INTO ecf_cupons_xz
                        (cupom_id, cupom_inicial, cupom_final, gt_inicial, gt_final, cancelamentos,
                        descontos, acrescimos, cro, crz)
                    VALUES (%s)
                """ % valores
                sql_insert_supervisor_cupom_xz = sql_insert_supervisor_cupom_xz.replace("'null'",
                                                                                        'null')
                cr.execute(sql_insert_supervisor_cupom_xz)
                res = True
        return res

    def create_ecf_cupom_xz_totalizadores(self, cr, uid, _cr, cupom_id, mov_capa_id):

        """
        Realiza a insercao dos dados dos totalizadores X e Z dos cupons na tabela
        ecf_cupons_xz_totalizadores
        """

        res = False
        sql_select_supervisor_cupom_xztotalizadores = """
            SELECT smtxz.suptxz_tipo_tributacao, smtxz.suptxz_codigo_trib_ecf,
                   smtxz.suptxz_tipo_imposto, smtxz.suptxz_aliquota, smtxz.suptxz_base,
                   smtxz.suptxz_valor
            FROM supervisor_mov_totalizadores_xz smtxz
            WHERE smtxz.suptxz_capa_id = %s
        """ % mov_capa_id
        _cr.execute(sql_select_supervisor_cupom_xztotalizadores)
        obj_adm_utils = self.pool.get('adm.utils')
        supervisor_corpos_xz_totalizadores_dados = obj_adm_utils. \
            admutl_converts_fetchall_to_dictfetchall(_cr)
        if supervisor_corpos_xz_totalizadores_dados:
            for totalizador in supervisor_corpos_xz_totalizadores_dados:
                tipo_totalizador = totalizador.get('suptxz_tipo_tributacao') or 'null'
                if tipo_totalizador == 'F':
                    tipo_totalizador = 'U'
                elif tipo_totalizador == 'N':
                    tipo_totalizador = 'O'
                try:
                    tributacao_na_ecf = int(totalizador.get('suptxz_codigo_trib_ecf'))
                except:
                    tributacao_na_ecf = 'null'
                imposto = totalizador.get('suptxz_tipo_imposto') or 0
                aliquota = totalizador.get('suptxz_aliquota') or 0
                base_imposto = totalizador.get('suptxz_valor') or 0
                valor_imposto = (float(aliquota) / 100) * float(base_imposto)
                valor_imposto = Decimal(str(valor_imposto)).quantize(Decimal('.01'),
                                                                     rounding=ROUND_DOWN)
                valores = (cupom_id, tipo_totalizador, imposto, aliquota, base_imposto,
                           valor_imposto, tributacao_na_ecf)
                valores = "%s, '%s', '%s', %s, %s, %s, %s" % valores
                sql_insert_ecf_cupom_xztotalizadores = """
                    INSERT INTO ecf_cupons_xztotalizadores
                        (cupom_id, tipo_totalizador, imposto, aliquota, base_imposto,
                        valor_imposto, tributacao_na_ecf)
                    VALUES (%s)
                """ % valores
                sql_insert_ecf_cupom_xztotalizadores = sql_insert_ecf_cupom_xztotalizadores. \
                    replace("'null'", 'null')
                cr.execute(sql_insert_ecf_cupom_xztotalizadores)
                res = True
        return res

    def create_ecf_cupom_finalizacao(self, cr, uid, _cr, cupom_id, mov_capa_id, context=None):

        """
        Realiza a insercao das finalizadoras dos cupons na tabela ecf_cupons_finalizacao
        """

        context = context and context.copy() or {}
        res = False
        obj_adm_condicao_forma = self.pool.get('adm.condicao.forma')
        obj_adm_utils = self.pool.get('adm.utils')
        sql_select_supervisor_cupom_finalizacao = """
            SELECT smf.id as fechamento_id, smf.supfec_finalizadora, smf.supfec_valor,
                   smf.supfec_sequencia, smf.supfec_tipo_finalizadora, smf.supfec_troco,
                   smf.supfec_nro_banco, smf.supfec_agencia, smf.supfec_nro_cheque,
                   smf.supfec_conta, smf.supfec_juros, smf.supfec_cnpj, smca.supcap_data_cupom,
                   smca.supcap_loja, smca.supcap_caixa
            FROM supervisor_mov_fechamentos smf
            INNER JOIN supervisor_mov_capa smca ON smf.supfec_capa_id = smca.id
            WHERE smf.supfec_capa_id = %s
        """ % mov_capa_id
        _cr.execute(sql_select_supervisor_cupom_finalizacao)
        supervisor_mov_fechamentos_dados = obj_adm_utils. \
            admutl_converts_fetchall_to_dictfetchall(_cr)
        if supervisor_mov_fechamentos_dados:
            obj_adm_condicao = self.pool.get('adm.condicao')
            obj_adm_forma_pgto = self.pool.get('adm.forma.pgto')
            for fechamento in supervisor_mov_fechamentos_dados:
                condicoes_ativas_ids = obj_adm_condicao.search(cr, uid, [('active', '=', True)], context=context)
                forma_ativas_ids = obj_adm_forma_pgto.search(cr, uid, [('active', '=', True)], context=context)
                finalizadora_ids = obj_adm_condicao_forma.search(
                    cr, uid, [('finalizadora_na_ecf', '=', fechamento.get('supfec_finalizadora')),
                              ('adm_condicao_id', 'in', condicoes_ativas_ids),
                              ('forma_pagamento_id', 'in', forma_ativas_ids)], context=context)
                loja = fechamento.get('supcap_loja')
                caixa = fechamento.get('supcap_caixa')
                data = fechamento.get('supcap_data_cupom')
                valor = fechamento.get('supfec_valor')
                sequencia = fechamento.get('supfec_sequencia') or 'null'
                tipo_finalizadora = fechamento.get('supfec_tipo_finalizadora') or 'null'
                troco = fechamento.get('supfec_troco') or 'null'
                numero_banco = fechamento.get('supfec_nro_banco') or 'null'
                agencia = fechamento.get('supfec_agencia') or 'null'
                numero_cheque = fechamento.get('supfec_nro_cheque') or 'null'
                conta = fechamento.get('supfec_conta') or 'null'
                juros = fechamento.get('supfec_juros') or 'null'
                cnpj = fechamento.get('supfec_cnpj') or 'null'
                finalizadora = fechamento.get('supfec_finalizadora') or 0
                fields_fechamento = [finalizadora, valor]
                if all(fields_fechamento):
                    if finalizadora_ids:
                        finalizadora_id = finalizadora_ids[0]
                        avista_aprazo = obj_adm_condicao_forma.\
                                        browse(cr, uid, finalizadora_ids[0],
                                                      context=context).adm_condicao_id.tipo
                    else:
                        finalizadora_id = 'null'
                        avista_aprazo = 'null'

                    sql_insert_ecf_cupom_finalizacao = """
                        INSERT INTO ecf_cupons_finalizacao
                            (cupom_id, data, loja, caixa, finalizadora_id, valor, sequencia,
                             tipo_finalizadora, troco, numero_banco, agencia, numero_cheque,
                             conta, juros, cnpj, ecffin_finalizadora_na_ecf, avista_aprazo)
                        VALUES ({cupom_id}, '{data}', '{loja}', '{caixa}', {finalizadora_id},
                                {valor}, {sequencia}, '{tipo_finalizadora}', {troco},
                                '{numero_banco}', '{agencia}', '{numero_cheque}', '{conta}',
                                {juros}, '{cnpj}', {ecffin_finalizadora_na_ecf}, '{avista_aprazo}')
                    """.format(cupom_id=cupom_id,
                               data=data,
                               loja=loja, caixa=caixa,
                               finalizadora_id=finalizadora_id,
                               valor=valor,
                               sequencia=sequencia,
                               tipo_finalizadora=tipo_finalizadora,
                               troco=troco,
                               numero_banco=numero_banco,
                               agencia=agencia,
                               numero_cheque=numero_cheque,
                               conta=conta,
                               juros=juros,
                               cnpj=cnpj,
                               ecffin_finalizadora_na_ecf=finalizadora,
                               avista_aprazo=avista_aprazo)
                    sql_insert_ecf_cupom_finalizacao = sql_insert_ecf_cupom_finalizacao.replace(
                        "'null'", 'null')
                    cr.execute(sql_insert_ecf_cupom_finalizacao)
                    sql_select_id_finalizacao = """
                        SELECT id
                        FROM ecf_cupons_finalizacao
                        WHERE
                            cupom_id = %s AND
                            ecffin_finalizadora_na_ecf = %s AND
                            valor = %s
                    """ % (cupom_id, finalizadora, valor)
                    cr.execute(sql_select_id_finalizacao)
                    finalizacao_id = obj_adm_utils.admutl_converts_fetchall_to_dictfetchall(cr)
                    if finalizacao_id:
                        if isinstance(finalizacao_id, list):
                            finalizacao_id = finalizacao_id[0]
                        finalizacao_id = finalizacao_id.get('id')
                        # vencimentos
                        fechamento_id = fechamento.get('fechamento_id')
                        self.create_ecf_cupom_vencimento(cr, uid, _cr, finalizacao_id,
                                                         fechamento_id)
                        res = True
        return res

    def create_ecf_cupom_vencimento(self, cr, uid, _cr, finalizacao_id, supervisor_fechamento_id):

        """
        Realiza a insercao dos vencimentos das finalizadoreas dos cupons na tabela
        ecf_cupons_vencimento
        """

        res = False
        sql_select_supervisor_mov_vencimentos = """
            SELECT supfve_vencimento, supfve_valor_parcela
            FROM supervisor_mov_vencimentos
            WHERE supfve_fechamentos_id = %s
        """ % supervisor_fechamento_id
        _cr.execute(sql_select_supervisor_mov_vencimentos)
        obj_adm_utils = self.pool.get('adm.utils')
        supervisor_mov_vencimentos_dados = obj_adm_utils. \
            admutl_converts_fetchall_to_dictfetchall(_cr)
        if supervisor_mov_vencimentos_dados:
            for vencimento in supervisor_mov_vencimentos_dados:
                data_vencimento = vencimento.get('supfve_vencimento')
                if data_vencimento:
                    valor = vencimento.get('supfve_valor_parcela') or 0

                    valores = (finalizacao_id, data_vencimento, valor)
                    valores = "%s, '%s', %s" % valores
                    sql_insert_ecf_cupom_vencimento = """
                        INSERT INTO ecf_cupons_vencimento
                            (finalizacao_id, data_vencimento, valor)
                        VALUES (%s)
                    """ % valores
                    sql_insert_ecf_cupom_vencimento = sql_insert_ecf_cupom_vencimento.replace(
                        "'null'", 'null')
                    cr.execute(sql_insert_ecf_cupom_vencimento)
                    res = True
        return res
