# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _

class energia_projetos_instalacao(osv.Model):
    """
    Tabela de locais de instalacao
    Prefixo enepji_
    """
    _inherit = 'energia.projetos.instalacao'

    def _enepji_get_painel(self, cr, uid, instalacao_id, context=None):
        """
        Método para identificar o id do painel a ser utilizado
        @param cr:
        @param uid:
        @param id: id da instalacao
        @param context:
        @return: id do painel ou None se não houver
        """
        context = context and context.copy() or {}
        _painel_id = None
        if instalacao_id:
            _dados_instalacao = self.read(cr, uid, instalacao_id, [], context=context)
            _painel = _dados_instalacao.get('enepji_painel_id', False)
            if isinstance(_painel, (tuple, list)):
                _painel_id = _painel[0]
            else:
                _painel_id = _painel
        return _painel_id

    def _enepji_calcular_producao(self, cr, uid, ids, qtd_paineis, context=None):
        """
        Método utilizado para calcular a producao pela quantidade de paineis
        @param cr:
        @param uid:
        @param ids: ids dos projetos(energia.projetos)
        @param qtd_paineis e 2: Quantidade de paineis mppt1 e 2
        @param context:
        @return: dicionario com id do projeto e producao Ex: {5:1.2}
        """
        context = context and context.copy() or {}
        res = {'status': False, 'mensagem': _('Sem instalacao cadastrada!'), 'producao': {}}
        for _instalacao in self.browse(cr, uid, ids, context=context):
            _producao = 0
            _painel_id = self._enepji_get_painel(cr, uid, _instalacao.id, context=context)
            # _inversor_id = self._enepji_get_inversor(cr, uid, _instalacao.id, context=context)
            if _painel_id:
                _mens = ''
                if not _instalacao.enepji_geradora_id:
                    return {'status': False, 'mensagem': _('O local de instalacao "%s", nao tem unidade geradora cadastrada!' % _instalacao.enepji_descricao)}
                if not _instalacao.enepji_geradora_id.enepco_temp_max:
                    _mens += _('A unidade geradora "%s", nao tem temperatura maxima cadastrada!\n' % _instalacao.enepji_geradora_id.name)
                if not _instalacao.enepji_geradora_id.enepco_tipo_estrutura:
                    _mens += _('A unidade geradora "%s", nao tem tipo de estrutura cadastrada!' % _instalacao.enepji_geradora_id.name)
                if _mens:
                    return {'status': False, 'mensagem': _mens, 'producao': {}}
                _dados_painel = self.pool.get('energia.painel.fotovoltaico').enepfv_get_caracteristicas(cr, uid,
                                                                                                        [_painel_id],
                                                                                                        _instalacao.enepji_geradora_id.enepco_temp_max,
                                                                                                        _instalacao.enepji_geradora_id.enepco_tipo_estrutura)
                _qtd_paineis = qtd_paineis or _instalacao.enepji_quantidade_paineis
                _potencia_painel = _dados_painel.get(_painel_id).get('enepfv_pmax')
                _producao = _qtd_paineis * (_potencia_painel / 1000)
                _producao = _producao * 98.8 / 100 # Eficiencie do inversor, esta fixo porque nao esta identificado cada inversor
                _perdas = (100 - _instalacao.enepji_projeto_id.eneprj_percentual_perdas) / 100
                _producao = _producao * _perdas

                # if _inversor_id:
                #     _dados_inversor = self.pool.get('energia.inversores').read(cr, uid, _inversor_id, [], context=context)
                #     if _dados_inversor and _dados_inversor.get('eneinv_fator_eficiencia'):
                #         _producao = _producao * _dados_inversor.get('eneinv_fator_eficiencia') / 100
            else:
                return {'status': False, 'mensagem': _('O local de instalacao "%s", nao tem paineis cadastrados!' % _instalacao.enepji_descricao), 'producao': {}}
            res['producao'][_instalacao.id] = _producao
            res['status'] = True
        return res

    def _enepji_calcular_paineis_por_area(self, cr, uid, instalacao_id, context=None):
        """
        Calcular capacidade de paineis na area a instalar, só considera informações mecanicas, não as eletricas
        @param cr:
        @param uid:
        @param instalacao_id: id da classe energia instalacao (integer)
        @param context:
        @return: dicionario contendo 'qtd' que é quantidade de paineis e 'posicao', que deve ser R=Retrato ou P=Paisagem
        """
        res = {'qtd_paisagem': 0, 'qtd_retrato': 0, 'posicao': 'R'}
        if instalacao_id:
            _painel_id = self._enepji_get_painel(cr, uid, instalacao_id, context)
            if _painel_id:
                _dados_painel = self.pool.get('energia.painel.fotovoltaico').read(cr, uid, _painel_id, [],
                                                                                  context=context)
                _comprimento = _dados_painel.get('enepfv_comprimento', False)
                _largura = _dados_painel.get('enepfv_largura', False)
                _dados_instalacao = self.read(cr, uid, instalacao_id, [], context=context)
                _instalacao_comprimento = _dados_instalacao.get('enepji_comprimento', False)
                _instalacao_largura = _dados_instalacao.get('enepji_largura', False)
                if _comprimento and _largura and _instalacao_comprimento and _instalacao_largura:
                    # retirando margem de seguranca
                    _instalacao_comprimento = (_instalacao_comprimento * 1000) - 40
                    _instalacao_largura = (_instalacao_largura * 1000) - 40
                    # Calcular quantidade no formato retrato
                    _qtd_comprimento_retr = int(_instalacao_comprimento / _largura)
                    _qtd_largura_retr = int(_instalacao_largura / _comprimento)
                    _qtd_retrato = _qtd_comprimento_retr * _qtd_largura_retr
                    # Calcular quantidade no formato paisagem
                    _qtd_comprimento_pai = int(_instalacao_comprimento / _comprimento)
                    _qtd_largura_pai = int(_instalacao_largura / _largura)
                    _qtd_paisagem = _qtd_comprimento_pai * _qtd_largura_pai
                    res = {'qtd_paisagem': _qtd_paisagem,
                           'qtd_largura_paisagem': _qtd_largura_pai,
                           'qtd_comprimento_paisagem': _qtd_comprimento_pai,
                           'qtd_retrato': _qtd_retrato,
                           'qtd_largura_retrato': _qtd_largura_retr,
                           'qtd_comprimento_retrato': _qtd_comprimento_retr
                           }
        return res

    def _enepji_calcular_paineis(self, cr, uid, ids, context=None):
        """
        Método utilizado para calcular a quantidade de paineis necessários
        @param cr:
        @param uid:
        @param ids: ids da classe energia.projetos
        @param context:
        @return: dicionario cuja chave é o id do projeto contendo um int Quantidade de paineis Ex {5:6}
        """
        res = {'status': False, 'mensagem': _('Nao existe locais de instalacao cadastrados!'), 'paineis': {}}
        for _instalacao in self.browse(cr, uid, ids, context=context):
            _painel_id = self._enepji_get_painel(cr, uid, _instalacao.id, context=context)
            _qtd_paineis = 0
            if _painel_id:
                if _instalacao.enepji_quantidade_paineis > 0:
                    _qtd_paineis = _instalacao.enepji_quantidade_paineis
                else:
                    obj_energia_painel_fotovoltaico = self.pool.get('energia.painel.fotovoltaico')
                    if not _instalacao.enepji_geradora_id:
                        return {'status': False, 'mensagem': _('O local de instalacao "%s", nao tem unidade geradora cadastrada!') % _instalacao.enepji_descricao}
                    if not _instalacao.enepji_geradora_id.enepco_temp_max:
                        return {'status': False, 'mensagem': _('Na unidade geradora "%s", nao tem temperatura maxima cadastrada!') % _instalacao.enepji_geradora_id.name}
                    if not _instalacao.enepji_geradora_id.enepco_tipo_estrutura:
                        return {'status': False, 'mensagem': _('Na unidade geradora "%s", nao tem tipo de estrutura cadastrado!') % _instalacao.enepji_geradora_id.name}
                    _dados_painel = obj_energia_painel_fotovoltaico.enepfv_get_caracteristicas(cr, uid, [_painel_id],
                                                                                               _instalacao.enepji_geradora_id.enepco_temp_max,
                                                                                               _instalacao.enepji_geradora_id.enepco_tipo_estrutura)
                    _potencia_painel = _dados_painel.get(_painel_id).get('enepfv_pmax')
                    _hsp = _instalacao.enepji_irradiancia_id.eneirr_media or 5
                    _consumo_hora = _instalacao.enepji_projeto_id.eneprj_media / 30 / _hsp
                    _qtd_paineis = int(round(_consumo_hora / (_potencia_painel / 1000), 0))
            else:
                return {'status': False, 'mensagem': _('O local de instalacao "%s", nao tem paineis cadastrados!' % _instalacao.enepji_descricao)}
            res['paineis'][_instalacao.id] = (_qtd_paineis)
            res['status'] = True
        return res

    # def enepji_onchange_inversor(self, cr, uid, ids, inversor_id, context=None):
    #     context = context and context.copy() or {}
    #     obj_inversores = self.pool.get('energia.inversores')
    #     dados_inversor = obj_inversores.read(cr, uid, inversor_id, ['eneinv_nrostring'], context=context)
    #     _qtd = dados_inversor.get('eneinv_nrostring') or 1
    #     return {'value': {'enepji_inversor_qtd_mppt':_qtd}}

    def enepji_button_calcular(self, cr, uid, ids, context=None):
        def __enepji_get_gtd_paineis_ja_calculado():
            # Achar a quantidade de paineis ja calculado em todas as instalacoes da opcao exceto da instalacao atual
            _qtd = 0
            for _brw_instalacao_do_projeto in _brw_instalacao.enepji_projeto_id.eneprj_instalacao_ids:
                # if _brw_instalacao_do_projeto.enepji_opcao == _brw_instalacao.enepji_opcao:  # So instalacoes da opcao atual
                    if _brw_instalacao_do_projeto.id != _brw_instalacao.id:  # Nao considerar da instalacao atual
                        _qtd += _brw_instalacao_do_projeto.enepji_quantidade_paineis
            return _qtd

        def __enepji_get_capacidade(_cap_pos, _qtd_paineis):
            _qtd_max = _cap_pos.get('qtd_paisagem') if _cap_pos.get('qtd_paisagem') > _cap_pos.get(
                'qtd_retrato') else _cap_pos.get('qtd_retrato')
            if _qtd_max == 0:
                return 0,0,0
            vals['enepji_capacidade_qtd'] = _qtd_max
            # Calcular quantidade necessária de paineis
            if _qtd_paineis != _brw_instalacao.enepji_quantidade_paineis:  # Alterou a quantidade de paineis
                if _qtd_ja_calculado < _qtd_paineis:
                    _qtd_paineis = _qtd_paineis - _qtd_ja_calculado
            # if _qtd_paineis > _qtd_max:
            #     _qtd_paineis = _qtd_max
            #     mens = 'A area da instalacao e insuficiente para quantidade de paineis!'
            # else:
            #     mens = ''
            if _qtd_paineis > _cap_pos.get('qtd_retrato'):
                vals['enepji_capacidade_posicao'] = 'P'
                _qtd_largura = _cap_pos.get('qtd_largura_paisagem')
                _qtd_comprimento = _cap_pos.get('qtd_comprimento_paisagem')
            else:
                vals['enepji_capacidade_posicao'] = 'R'
                _qtd_largura = _cap_pos.get('qtd_largura_retrato')
                _qtd_comprimento = _cap_pos.get('qtd_comprimento_retrato')
            return _qtd_largura, _qtd_comprimento, _qtd_paineis
            # return _qtd_largura, _qtd_comprimento, _qtd_paineis, mens

        context = context and context.copy() or {}
        for _brw_instalacao in self.browse(cr, uid, ids, context=context):
            _qtd_ja_calculado = __enepji_get_gtd_paineis_ja_calculado()
            _res_paineis = self._enepji_calcular_paineis(cr, uid, [_brw_instalacao.id], context)
            if _res_paineis.get('status') == False:
                raise osv.except_osv(_('Atencao'), _res_paineis.get('mensagem'))
            _dados_dict = _res_paineis.get('paineis')
            _painel_id = self._enepji_get_painel(cr, uid, _brw_instalacao.id, context=context)
            vals = {}
            # Calcular qtos paineis cabem no local de instalacao
            _cap_pos = self._enepji_calcular_paineis_por_area(cr, uid, _brw_instalacao.id, context=None)
            _qtd_largura, _qtd_comprimento, _qtd_paineis = __enepji_get_capacidade(_cap_pos, _dados_dict.get(_brw_instalacao.id))
            if _brw_instalacao.enepji_quantidade_paineis == 0:
                vals['enepji_quantidade_paineis'] = _qtd_paineis
            else:
                _qtd_paineis = _brw_instalacao.enepji_quantidade_paineis
            _err = _mens = ''
            if _painel_id:
                _dados_painel = self.pool.get('energia.painel.fotovoltaico').read(cr, uid, _painel_id, context=context)
            else:
                _dados_painel = False
            # Calcular a potencia instalada(Producao)
            _res_prod = self._enepji_calcular_producao(cr, uid, [_brw_instalacao.id], _qtd_paineis, context)
            if _res_prod.get('status') == False:
                raise osv.except_osv(_('Atencao'), _res_prod.get('mensagem'))
            _producao = _res_prod.get('producao')
            # if _producao and _brw_inversor and _brw_inversor.eneinv_pmax_dc:
            #     _prod = int(_producao.get(_brw_instalacao.id) * 1000)
            #     _pmax_dc = _brw_inversor.eneinv_pmax_dc
            #     if _prod > _pmax_dc:
            #         _err = '%s producao %sW e maior do que a potencia maxima de entrada do inversor(MPPT1) %sW' % (_err,
            #                                                                                                 _prod,
            #                                                                                               _pmax_dc)
            # vals['enepji_mensagem_1'] += _err
            vals['enepji_potencia_1'] = _producao[_brw_instalacao.id]
            vals['enepji_potencia_pico_1'] = 1.0 * _qtd_paineis * _dados_painel.get('enepfv_pmax') / 1000
            res = self.write(cr, uid, _brw_instalacao.id, vals, context)
        return True
