# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import enepji_get_dados


class energia_projetos_instalacao(osv.Model):
    """
    Tabela de locais de instalacao
    Prefixo enepji_
    """
    _name = 'energia.projetos.instalacao'
    _description = 'Tabela de locais de instalacao'
    _inherits = {
        'energia.projetos': 'enepji_projeto_id',
    }
    _columns = {
        'enepji_opcao':  fields.selection([('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6'), ('7', '7'), ('8', '8'), ('9', '9')],
                          'Opcao de instalacao',
                          help='Identifica as opcoes de instalacao, Uma opcao tem que produzir a quantidade de que o cliente necessita'),
        'enepji_projeto_id': fields.many2one("energia.projetos", "Projeto", required=True, select=True,
                                             ondelete='cascade'),
        'enepji_sequencia': fields.integer('Seq.'),
        'enepji_descricao': fields.char("Descricao", size=40, required=True),
        'enepji_desvio': fields.integer("Orientacao (Gr)", select=True, help='Desvio azimutal'),
        'enepji_inclinacao': fields.integer("Inclinacao(Gr) ", select=True, help='Inclinacao dos paineis'),
        'enepji_irradiancia_id': fields.many2one('energia.irradiancia', 'Irradiancia', ondelete='restrict'),
        'enepji_potencia_1': fields.float('Potencia a produzir (Kwh)', digits=(3, 3)),
        'enepji_potencia': fields.function(enepji_get_dados, method=True, multi='dados', type='float',
                                                    digits=(3, 3), string='Potencia total a produzir(Kwp)'),
        'enepji_potencia_pico_1': fields.float('Potencia (Kwp)', digits=(3, 3), readonly=True),
        'enepji_potencia_pico': fields.function(enepji_get_dados, method=True, multi='dados', type='float',
                                                    digits=(3, 3), string='Potencia pico total(Kwp)'),
        'enepji_painel_id': fields.many2one('energia.painel.fotovoltaico', 'Painel', ondelete="restrict",
                                            help="Se não informar o painel, sera usado o que foi definido na aba 'Dados preliminares'"),
        'enepji_quantidade_paineis': fields.integer("Quantidade de paineis",
                                                    help="Quantidde de paineis para geracao de energia. Deixando a quantidade zerada, o sistema ira calcular a quantidade baseada no comsumo."),
        'enepji_comprimento': fields.float('Comprimento (m)', digits=(4, 1),
                                           help='Comprimento em metros do local de instalacao. No telhado, meca no sentido subindo(Vertical)'),
        'enepji_largura': fields.float('Largura (m)', digits=(4, 1),
                                       help='Largura em metros do local de instalacao. No telhado, meca no sentido da largura(Horizontal)'),
        'enepji_capacidade_qtd': fields.integer('Capacidade de instalacao'),
        'enepji_capacidade_posicao': fields.selection([('P', 'Paisagem'), ('R', 'Retrato')], 'Posicao dos paineis',
                                                      size=1),
        'enepji_informacoes': fields.text('Informacoes complementares'),
        'enepji_fotos_ids': fields.one2many('energia.projetos.locpan.fotos', 'enepjif_local_id', 'Fotos'),
        'enepji_projeto_inversor_ids': fields.many2many('energia.projetos.inversores', 'energia_m2m_instacao_inversor', 'instalacao_id', 'inversor_id','Inversores'),
        'enepji_geradora_id': fields.many2one('energia.projeto.consumos', 'UG', select=True, ondelete='set null',
                                              help='Selecione a Unidade geradora de creditos'),
        'enepji_active': fields.boolean('Ativo', hel='Considerar nos calculos')
    }
    _order = 'enepji_projeto_id,enepji_opcao,enepji_sequencia'
    _defaults = {
        'enepji_opcao': 1,
        'enepji_sequencia': 1,
        'enepji_active': True,
    }
