# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import eneprj_get_dados


class energia_projetos(osv.Model):
    """
    Tabela de projetos
    Prefixo eneprj_
    """
    _name = 'energia.projetos'
    _description = 'Tabela de projetos'
    _columns = {
        'name': fields.char('Descricao', size=40),
        'eneprj_pessoa_id': fields.many2one("adm.pessoa", "Cliente", required=True, select=True, ondelete='restrict'),
        'eneprj_endereco_id': fields.many2one("adm.pessoa.endereco", "Endereco", required=True, ondelete='restrict',
                                              domain="[('pessoa_id','=', eneprj_pessoa_id)]"),
        'create_date': fields.datetime('Criacao', readonly=True),
        'create_uid': fields.many2one('res.users', 'Usuario', readonly=True),
        'eneprj_distribuidora_id': fields.many2one("adm.pessoa", "Distribuidora", ondelete='set null',
                                                   help='Distribuidora de energia eletrica'),
        'eneprj_distruidora_cliente_codigo': fields.char('Codigo do cliente na distribuidora', size=15),
        'active': fields.boolean('Ativo'),
        'eneprj_padrao_id': fields.many2one("energia.padrao", "Tipo de padrao", ondelete='restrict'),
        'eneprj_mes_01': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Janeiro', store=True),
        'eneprj_mes_02': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Fevereiro', store=True),
        'eneprj_mes_03': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Marco', store=True),
        'eneprj_mes_04': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Abril', store=True),
        'eneprj_mes_05': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Maio', store=True),
        'eneprj_mes_06': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Junho', store=True),
        'eneprj_mes_07': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Julho', store=True),
        'eneprj_mes_08': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Agosto', store=True),
        'eneprj_mes_09': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Setembro', store=True),
        'eneprj_mes_10': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Outubro', store=True),
        'eneprj_mes_11': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Novembro', store=True),
        'eneprj_mes_12': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Dezembro', store=True),
        'eneprj_anual': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Cons.anual', store=True),
        'eneprj_media': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Consumo medio', store=True),
        'eneprj_potencia_sugerida': fields.function(eneprj_get_dados, method=True, multi="dados", type='float', digits=(6, 2),
                                        string='Potencia pico sugerida', store=True),
        'eneprj_potencia_inversores': fields.function(eneprj_get_dados, method=True, multi="dados", type='float', digits=(3, 3),
                                        string='Potencia inversores(Kw)', store=True),
        'eneprj_qtd_mppts': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Qtd.Mppts', store=True),
        'eneprj_qtd_strings': fields.function(eneprj_get_dados, method=True, multi="dados", type='integer',
                                        string='Qtd.Strings', store=True),
        'eneprj_latitude': fields.float("Latitude", digits=(2, 6), select=True, help='Latitude da instalacao'),
        'eneprj_longitude': fields.float("Longitude", digits=(2, 6), select=True, help='Latitude da instalacao'),
        'eneprj_percentual_perdas': fields.float("Percentual de perdas", digits=(2, 1), help='Percentual de perdas '),
        'eneprj_itens_ids': fields.one2many('energia.projetos.itens', 'enepri_projeto_id', 'Itens'),
        'eneprj_inversores_ids': fields.one2many('energia.projetos.inversores', 'enepiv_projeto_id', 'Inversores'),
        'eneprj_instalacao_ids': fields.one2many('energia.projetos.instalacao', 'enepji_projeto_id', 'Instalacoes'),
        'eneprj_orcamento_ids': fields.one2many('energia.orcamentos', 'eneorc_projeto_id', 'Orcamentos'),
        'eneprj_potencia_prevista': fields.function(eneprj_get_dados, method=True, multi='dados', type='float',
                                                    digits=(3, 3), string='Potencia prevista(Kwh)', store=True),
        'eneprj_potencia_pico': fields.function(eneprj_get_dados, method=True, multi='dados', type='float',
                                                    digits=(3, 3), string='Potencia pico(Kwp)', store=True),
        'eneprj_quantidade_paineis': fields.function(eneprj_get_dados, method=True, multi='dados', type='integer',
                                                    string='Quantidade de paineis', store=True),
        'eneprj_saldo_creditos': fields.integer('Creditos anuais',
                                                help='Saldo em creditos energeticos previsto por ano'),
        'eneprj_valor_proposto': fields.float("Valor proposto", digits=(9, 2), select=True, help='Valor proposto ao cliente'),
        'eneprj_proposta_emissao': fields.date('Data da emissao'),
        'eneprj_proposta_validade': fields.date('Validade da proposta'),
        'eneprj_proposta_vl_kwp': fields.function(eneprj_get_dados, method=True, multi="dados", type='float',
                                                  digits=(6, 2), string='Valor p/Kwp', store=True),
        'eneprj_inv_distancia': fields.integer('Distancia dos paineis', help='informe a distancia que o inversor se encontra da instalacao dos paineis'),
        'eneprj_inv_largura': fields.float('Largura', help='Largura do local de instalacao do inversor', digits=(2,2)),
        'eneprj_inv_altura': fields.float('Altura', help='Altura do local de instalacao do inversor', digits=(2, 2)),
        'eneprj_locinv_ids': fields.one2many('energia.projetos.locinv.fotos', 'enelfi_projeto_id', 'Local inversores'),
        'eneprj_locpad_ids': fields.one2many('energia.projetos.locpad.fotos', 'enelfp_projeto_id', 'Local do padrao'),
        'eneprj_loccax_ids': fields.one2many('energia.projetos.loccax.fotos', 'enecax_projeto_id', 'Local da caixa de distribuicao'),
        'eneprj_cax_distancia': fields.integer('Distancia do inversor',
                                               help='informe a distancia que a caixa de distriuicao se encontra do inversor!'),
        'eneprj_detalhe_linhas': fields.text('Detalhe das linhas'),
        'eneprj_detalhe_strings': fields.text('Detalhe das strings'),
        'state': fields.selection([('1E', 'Esboco'), ('2P', 'Em projeto'), ('3O', 'Aguadando orcamento'),
                                   ('4F', 'Fazer proposta'), ('5A', 'Aguardando aprovacao'), ('6X', 'Executando'),
                                   ('7C', 'Concluido'), ('8U', 'Futuro'), ('9D', 'Perdido')], 'Estado'),

        'eneprj_consumo_ids': fields.one2many('energia.projeto.consumos', 'enepco_projeto_id',
                                              'Consumos'),
        'eneprj_observacoes': fields.text('Informacoes complementares'),
        'sequence': fields.integer('Ordem'),
        'eneprj_converter_tarifas': fields.boolean('Fazer convercao de tarifas'),

    }
    _order = 'create_date desc'
    _defaults = {
        'active': True,
        'state': '1E',
        'eneprj_percentual_perdas': 17,
        'eneprj_converter_tarifas': False,
        'sequence': 10,
    }
