# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from SigUtils.formatastring import unicode_to_ascii
import base64
from openerp.tools.translate import _

f_txt = lambda txt, tam: unicode_to_ascii(txt).ljust(tam)[:tam]
f_num = lambda x, t, d=0: str(long(round(float(x) * 10 ** d, 2))).rjust(t, '0')[-t:] or False


class balanca_exportar_cadastro_osv(osv.TransientModel):
    _name = 'balanca.exportar.cadastro.osv'

    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True),
        'balexc_balanca': fields.selection([('T', 'Toledo'), ('F', 'Filizola')], required=True),
        'setor_id': fields.many2one('balanca.setor', 'Setor', ondelete='cascade'),
        'balexc_gerar_nutricional': fields.boolean('Gerar informacao nutricional'),
        'balexc_gerar_receita': fields.boolean('Gerar arquivo de receitas'),
        'state': fields.selection([('R', 'Esboco'), ('C', 'Concluido'), ('E', 'Erro')], 'Estado'),
        'balexc_informacao': fields.text(''),
        'dados_itens': fields.binary('Itens ', readonly=True),
        'dados_setor': fields.binary('Setor ', readonly=True),
        'dados_nutricionais': fields.binary('Nutricionais', readonly=True),
        'balexc_dado_receita': fields.binary('Receita', readonly=True),
        'file_itens': fields.char('Nome do arquivo', size=50, readonly=False),
        'file_setor': fields.char('Nome do arquivo', size=50, readonly=False),
        'file_nutricionais': fields.char('Nome do arquivo', size=50, readonly=False),
        'balexc_file_receita': fields.char('Nome do arquivo', size=50),
    }

    _defaults = {
        'balexc_gerar_nutricional': lambda *a: False,
        'state': lambda *a: 'R',
        'file_itens': 'Itens.txt',
        'file_setor': 'Setor.txt',
        'file_nutricionais': 'Nutricionais.txt',
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id
    }

    def balexc_button_exportar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_adm_util = self.pool.get('adm.utils')
        adm_parametro = obj_adm_util.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_balanca_exportar_digito_codigo_barra')
        # contexto para identificar se o codigo de barra do item sera exportado com o digito
        # verificador ou nao
        context['balexc_exportar_digito'] = adm_parametro.get(
            'estpar_balanca_exportar_digito_codigo_barra')
        fields_read = ['balexc_balanca', 'balexc_gerar_nutricional', 'balexc_gerar_receita']
        vals = self.read(cr, uid, ids[0], fields_read, context=context)
        if vals.get('balexc_balanca', False) == 'F': # Layout Filizola
            self._balexcgerar_arq_itens_filizola(cr, uid, ids, context=context)
            self.balexc_gerar_arq_setor_filizola(cr, uid, ids, context=context)
            if vals.get('balexc_gerar_nutricional', False):
                self.balexc_gerar_arq_nut_filizola(cr, uid, ids, context=context)
            if vals.get('balexc_gerar_receita'):
                self._balexc_gerar_arquivo_receita_filizola(cr, uid, ids, context)
        elif vals.get('balexc_balanca', False) == 'T':  # Layout Toledo
            if vals.get('balexc_gerar_receita'):
                context['gerar_receita'] = True
            self._balexcgerar_arq_itens_toledo(cr, uid, ids, context=context)
            if vals.get('balexc_gerar_nutricional', False):
                self.balexc_gerar_arq_nut_toledo(cr, uid, ids, context=context)
        else:
            raise osv.except_osv(_('Atencao'), _('Selecione a balanca desejada.'))
        return self.write(cr, uid, ids, {'state': 'C'}, context=context)

    def _balexc_get_parametro(self, cr, uid, nome_parametro, context=None):
        context = context and context.copy() or {}
        estoque_parametro_ids = self.pool.get('estoque.parametro').search(cr, uid, [], context=context)
        if not estoque_parametro_ids:
            raise osv.except_osv(_('Atencao'), _('Nao ha parametros de configuracao para esta empresa.'))
        parametro = self.pool.get('estoque.parametro').read(cr, uid, estoque_parametro_ids[0], context=context)[nome_parametro]
        if parametro:
            return parametro
        else:
            return False

    def _balexc_salvar_arquivo(self, data, caminho):
        if len(data) > 0 and caminho:
            try:
                arq = open(caminho, 'w')
                arq.write(data)
                arq.close()
                return True
            except IOError as err:
                raise osv.except_osv(_('Atencao'), _('%s' % str(err).split(']')[1]))
        elif len(data) > 0:
            return True
        else:
            raise osv.except_osv(_('Atencao'), _('Nao ha registros para exportar'))

    def _balexcgerar_arq_itens_filizola(self, cr, uid, ids, context=None):
        log = ''
        """
        Gera arquivo de itens no layout da Filizola:
        BAL2-ITEM PIC 9(06)
        BAL2-TIPO PIC X
        BAL2-NOME PIC X(22)
        BAL2-PRECO PIC 9(5)V99
        BAL2-VALIDADE PIC 999
        BAL2-CRLF PIC XX.
            ITEM: sao os 6 ultimos digitos do codigo de barra
            TIPO: e o atributo peso_unidade do estoque_produto

        Onde:
        X = codigo alfanumerico (letras e/ou numeros)
        9 - apenas numeros
        v9 - casas decimais (virgula implicita, remover se necessario)
        (n) sequencia de n caracteres. ex: 9(5) = 99999
        """
        context = context and context.copy() or {}
        obj_balanca_util = self.pool.get('balanca.util')
        obj_balanca_setor = self.pool.get('balanca.setor')
        obj_automacao_preco = self.pool.get('automacao.precos')
        obj_automacao_cod_barra = self.pool.get('automacao.codigo.barra')
        obj_estoque_produto = self.pool.get('estoque.produto')
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], context=context)
        setor_id = vals.get('setor_id', False)
        if setor_id:
            if isinstance(setor_id, tuple):
                setor_id = setor_id[0]
            setor_ids = [setor_id]
        else:
            company_id = vals.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            produto_ids = obj_estoque_produto.search(cr, uid, [('company_id', '=', company_id)], context=context)
            rec_produtos = obj_estoque_produto.read(cr, uid, produto_ids, ['setor_id'], context=context)
            dict_setores = {}
            for rec_produto in rec_produtos:
                if rec_produto.get('setor_id', False):
                    dict_setores[rec_produto['setor_id'][0]] = True
            setor_ids = dict_setores.keys()
        if not setor_ids:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha registros a serem exportados. Nao ha nenhum produto '
                                   'associado a um setor de balanca, ou nao ha setores de balanca '
                                   'cadastrados.'))
        arquivo_de_itens = self._balexc_get_parametro(cr, uid, 'arquivo_de_itens', context=context)
        context['retorno_sem_codigo'] = True
        list_dados = []
        for rec_setor in obj_balanca_setor.browse(cr, uid, setor_ids, context=context):
            for rec_produto in rec_setor.estoque_produto_ids:
                for rec_grade in rec_produto.produto_grade_ids:
                    for rec_codigo_barra in rec_grade.codigo_barra_ids:
                        if rec_produto.preco_venda_ids:
                            codigo_barra = rec_codigo_barra.codigo_barra
                            if not context.get('balexc_exportar_digito'):
                                codigo_barra = codigo_barra[:-1]
                            peso_unidade = rec_produto.peso_unidade or 'U'
                            preco_venda = 0
                            codigo_barra_id = obj_automacao_cod_barra.search(
                                cr, uid, [('codigo_barra', '=', rec_codigo_barra.codigo_barra)],
                                context=context)
                            if codigo_barra_id:
                                preco_venda_id = obj_automacao_preco.search(
                                    cr, uid, [('codigo_barra_id', '=', codigo_barra_id[0])], context=context)
                                read_preco_venda = obj_automacao_preco.read(cr, uid, preco_venda_id,
                                                                            ['preco_venda'], context=context)
                                if read_preco_venda:
                                    if isinstance(read_preco_venda, list):
                                        read_preco_venda = read_preco_venda[0]
                                        preco_venda = read_preco_venda.get('preco_venda')
                            if not preco_venda:
                                ultimo_preco_data = rec_produto.preco_venda_ids[0].create_date
                                preco_venda = rec_produto.preco_venda_ids[0].preco_venda
                                for preco in rec_produto.preco_venda_ids:
                                    if preco.create_date > ultimo_preco_data:
                                        ultimo_preco_data = preco.create_date
                                        preco_venda = preco.preco_venda
                            validade = rec_produto.validade
                            produto_name = rec_grade.descricao_resumida or ''
                            if not produto_name:
                                produto_name = rec_produto.name
                            produto_name = produto_name.encode('utf-8').replace('°', '').\
                                replace('º', '').replace('ª', '').replace('´', '')
                            vals = {'codigo_barra': codigo_barra, 'peso_unidade': peso_unidade,
                                    'produto_name': produto_name, 'preco_venda': preco_venda,
                                    'validade': validade}
                            list_dados.append(vals)
        s_arq = obj_balanca_util.balutl_processa_dado_exportacao_balanca(cr, uid, 'F', 'I',
                                                                         list_dados, context)
        log += _(' \n''Produtos Gravados Filizola:\n%s') % s_arq
        values = {'balexc_informacao': log}
        if not arquivo_de_itens:
            values['dados_itens'] = base64.encodestring(s_arq.encode('utf-8', 'strict'))
        self.write(cr, uid, ids, values, context=context)
        return self._balexc_salvar_arquivo(s_arq, arquivo_de_itens)

    def _balexcgerar_arq_itens_toledo(self, cr, uid, ids, context=None):
        log = ''
        """
        Gera arquivo de itens de balanca no layout da Toledo:
        BAL-DEPARTAMENTO PIC 99
        BAL-ETIQUETA PIC 99         *SEMPRE 01
            TIPO 1=PRODUTO 2=PESO
        BAL-TIPO PIC X
        BAL-ITEM PIC 9(06)
        BAL-PRECO PIC 9(4)V99
        BAL-VALIDADE PIC 999
        BAL-NOME1 PIC X(25)
        BAL-NOME2 PIC X(25)         *BRANCO
        BAL-RECEITA1 PIC X(50)      *BRANCO
        BAL-RECEITA2 PIC X(50)      *BRANCO
        BAL-RECEITA3 PIC X(50)      *BRANCO
        BAL-RECEITA4 PIC X(50)      *BRANCO
        BAL-RECEITA5 PIC X(50)      *BRANCO
        BAL-CRLF PIC XX

        Onde:
        X = codigo alfanumerico (letras e/ou numeros)
        9 - apenas numeros
        v9 - casas decimais (virgula implicita, remover se necessario)
        (n) sequencia de n caracteres. ex: 9(5) = 99999
        """
        context = context and context.copy() or {}
        obj_balanca_util = self.pool.get('balanca.util')
        obj_balanca_setor = self.pool.get('balanca.setor')
        obj_automacao_preco = self.pool.get('automacao.precos')
        obj_automacao_cod_barra = self.pool.get('automacao.codigo.barra')
        obj_estoque_produto = self.pool.get('estoque.produto')
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], context=context)
        setor_id = vals.get('setor_id', False)
        if setor_id:
            if isinstance(setor_id, tuple):
                setor_id = setor_id[0]
            setor_ids = [setor_id]
        else:
            company_id = vals.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            produto_ids = obj_estoque_produto.search(cr, uid, [('company_id', '=', company_id)],
                                                                  context=context)
            rec_produtos = obj_estoque_produto.read(cr, uid, produto_ids, ['setor_id'], context=context)
            dict_setores = {}
            for rec_produto in rec_produtos:
                if rec_produto.get('setor_id', False):
                    dict_setores[rec_produto['setor_id'][0]] = True
            setor_ids = dict_setores.keys()
        if not setor_ids:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha registros a serem exportados. Nao ha nenhum produto associado a um setor de balanca, ou nao ha setores de balanca cadastrados.'))
        arquivo_de_itens = self._balexc_get_parametro(cr, uid, 'arquivo_de_itens', context=context)
        context['retorno_sem_codigo'] = True
        list_dados = []
        for rec_setor in obj_balanca_setor.browse(cr, uid, setor_ids, context=context):
            for rec_produto in rec_setor.estoque_produto_ids:
                for rec_grade in rec_produto.produto_grade_ids:
                    for rec_codigo_barra in rec_grade.codigo_barra_ids:
                        if rec_produto.preco_venda_ids:
                            peso_unidade = rec_produto.peso_unidade or False
                            if peso_unidade == 'U':
                                peso_unidade = '1'
                            elif peso_unidade == 'P':
                                peso_unidade = '0'
                            else:
                                peso_unidade = '2'
                            produto_name = rec_grade.descricao_resumida or ''
                            if not produto_name:
                                produto_name = rec_produto.name
                            produto_name = produto_name.encode('utf-8').replace('°', '').replace('º', '').replace('ª', '').replace('´', '')
                            codigo_barra = rec_codigo_barra.codigo_barra
                            if not context.get('balexc_exportar_digito'):
                                codigo_barra = codigo_barra[:-1]

                            receita_titulo = rec_produto.estpro_receita_titulo or ''
                            receita_corpo = (rec_produto.estpro_receita_corpo or '').replace('\n',
                                                                                             ' ')

                            obj_adm_utils = self.pool.get('adm.utils')
                            receita_titulo = obj_adm_utils.admutl_remove_caracter(receita_titulo)
                            receita_titulo = obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                                                                receita_titulo)
                            receita_corpo = obj_adm_utils.admutl_remove_caracter(receita_corpo)
                            receita_corpo = obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                                                                receita_corpo)
                            setor_id = rec_setor.id
                            etiqueta_pic = '01'
                            preco_venda = 0
                            validade = rec_produto.validade
                            codigo_barra_id = obj_automacao_cod_barra.search(
                                cr, uid, [('codigo_barra', '=', rec_codigo_barra.codigo_barra)],
                                context=context)
                            if codigo_barra_id:
                                preco_venda_id = obj_automacao_preco.search(
                                    cr, uid, [('codigo_barra_id', '=', codigo_barra_id[0])], context=context)
                                read_preco_venda = obj_automacao_preco.read(cr, uid, preco_venda_id,
                                                                            ['preco_venda'], context=context)
                                if read_preco_venda:
                                    if isinstance(read_preco_venda, list):
                                        read_preco_venda = read_preco_venda[0]
                                        preco_venda = read_preco_venda.get('preco_venda')
                            if not preco_venda:
                                ultimo_preco_data = rec_produto.preco_venda_ids[0].create_date
                                preco_venda = rec_produto.preco_venda_ids[0].preco_venda
                                for preco in rec_produto.preco_venda_ids:
                                    if preco.create_date > ultimo_preco_data:
                                        ultimo_preco_data = preco.create_date
                                        preco_venda = preco.preco_venda

                            vals = {'setor_id': setor_id, 'etiqueta_pic': etiqueta_pic,
                                    'peso_unidade': peso_unidade, 'codigo_barra': codigo_barra,
                                    'preco_venda': preco_venda, 'validade': validade,
                                    'produto_name': produto_name, 'receita_titulo': receita_titulo,
                                    'receita_corpo': receita_corpo}
                            list_dados.append(vals)
        s_arq = obj_balanca_util.balutl_processa_dado_exportacao_balanca(cr, uid, 'T', 'I',
                                                                         list_dados, context)
        log += _(' \n''Produtos Gravados Toledo:\n%s') % s_arq
        values = {'balexc_informacao': log}
        if not arquivo_de_itens:
            values['dados_itens'] = base64.encodestring(s_arq.encode('utf-8', 'strict'))
        self.write(cr, uid, ids, values, context=context)
        return self._balexc_salvar_arquivo(s_arq, arquivo_de_itens)

    def balexc_gerar_arq_setor_filizola(self, cr, uid, ids, context=None):
        """
        Gera arquivo de setores para balancas Filizola.
        Layout:
        SET2-SETOR PIC X(12)
        SET2-ITEM PIC 9(06)
        SET2-INDICE PIC 9(04)
        SET2-TECLA PIC 999
        SET2-CRLF PIC XX
        Onde:
        X = codigo alfanumerico (letras e/ou numeros)
        9 - apenas numeros
        v9 - casas decimais (virgula implicita, remover se necessário)
        (n) sequencia de n caracteres. ex: 9(5) = 99999
        """
        obj_balanca_util = self.pool.get('balanca.util')
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], context=context)
        setor_id = vals.get('setor_id', False)
        if setor_id:
            if isinstance(setor_id, tuple):
                setor_id = setor_id[0]
            setor_ids = [setor_id]
        else:
            company_id = vals.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            produto_ids = self.pool.get('estoque.produto').search(cr, uid, [('company_id', '=', company_id)],
                                                                  context=context)
            rec_produtos = self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['setor_id'], context=context)
            dict_setores = {}
            for rec_produto in rec_produtos:
                if rec_produto.get('setor_id', False):
                    dict_setores[rec_produto['setor_id'][0]] = True
            setor_ids = dict_setores.keys()
        if not setor_ids:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha registros a serem exportados. Nao ha nenhum produto associado a um setor de balanca, ou nao ha setores de balanca cadastrados.'))
        arquivo_de_setor = self._balexc_get_parametro(cr, uid, 'arquivo_de_setor', context=context)
        indice = 1  # sequencia numerica exigida no layout, nao e preciso persistir
        list_dados = []
        for rec_setor in self.pool.get('balanca.setor').browse(cr, uid, setor_ids, context=context):
            for rec_produto in rec_setor.estoque_produto_ids:
                for rec_grade in rec_produto.produto_grade_ids:
                    for rec_codigo_barra in rec_grade.codigo_barra_ids:
                        codigo_barra = rec_codigo_barra.codigo_barra
                        if not context.get('balexc_exportar_digito'):
                            codigo_barra = codigo_barra[:-1]
                        setor_nome = rec_setor.name
                        tecla = rec_produto.tecla

                        indice += 1
                        vals = {'setor_nome': setor_nome, 'codigo_barra': codigo_barra,
                                'indice': indice, 'tecla': tecla}
                        list_dados.append(vals)
        s_arq = obj_balanca_util.balutl_processa_dado_exportacao_balanca(cr, uid, 'F', 'S',
                                                                         list_dados, context)
        if not arquivo_de_setor:
            self.write(cr, uid, ids, {'dados_setor': base64.encodestring(s_arq.encode('utf-8',
                                                                                      'strict'))}, context=context)
        return self._balexc_salvar_arquivo(s_arq, arquivo_de_setor)

    def balexc_gerar_arq_nut_filizola(self, cr, uid, ids, context=None):
        """
        Gera arquivo de informacao nutricional para o layout Filizola.

        NUT-ITEM PIC 9(6)
        NUT-PORCAO PIC X(35)
        NUT-VALOR-CALORICO PIC 9(5)
        NUT-PERC-VALOR-CALORICO PIC 9(4)
        NUT-CARBOIDRATOS PIC 9(4)v9
        NUT-PERC-CARBOIDRATOS PIC 9(4)
        NUT-PROTEINAS PIC 9(4)v9
        NUT-PERC-PROTEINAS PIC 9(4)
        NUT-GORDURAS-TOTAIS PIC 9(4)v9
        NUT-PERC-GORDURAS-TOTAIS PIC 9(4)
        NUT-GORDURAS-SATURADAS PIC 9(4)v9
        NUT-PERC-GORDURAS-SATURADAS PIC 9(4)
        NUT-GORDURAS-TRANS PIC 9(4)v9
        NUT-PERC-GORDURAS-TRANS PIC 9(4)
        NUT-FIBRA-ALIMENTAR PIC 9(5)
        NUT-PERC-FIBRA-ALIMENTAR PIC 9(4)
        NUT-CALCIO PIC X(5)
        NUT-PERC-CALCIO PIC X(4)
        NUT-FERRO PIC X(5)
        NUT-PERC-FERRO PIC X(4)
        NUT-SODIO PIC 9(4)v9
        NUT-PERC-SODIO PIC 9(4)
        NUT-CRLF PIC XX

        Onde:
        X = codigo alfanumerico (letras e/ou numeros)
        9 - apenas numeros
        v9 - casas decimais (virgula implicita, remover se necessário)
        (n) sequencia de n caracteres. ex: 9(5) = 99999
        """
        obj_balanca_util = self.pool.get('balanca.util')
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], context=context)
        setor_id = vals.get('setor_id', False)

        if setor_id:
            if isinstance(setor_id, tuple):
                setor_id = setor_id[0]
            setor_ids = [setor_id]
        else:
            company_id = vals.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            produto_ids = self.pool.get('estoque.produto').search(cr, uid, [('company_id', '=', company_id)],
                                                                  context=context)
            rec_produtos = self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['setor_id'], context=context)
            dict_setores = {}
            for rec_produto in rec_produtos:
                if rec_produto.get('setor_id', False):
                    dict_setores[rec_produto['setor_id'][0]] = True
            setor_ids = dict_setores.keys()
        if not setor_ids:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha registros a serem exportados. Nao ha nenhum produto associado a um setor de balanca, ou nao ha setores de balanca cadastrados.'))
        arquivo_de_nutricionais = self._balexc_get_parametro(cr, uid, 'arquivo_de_nutricionais', context=context)
        list_dados = []
        for rec_setor in self.pool.get('balanca.setor').browse(cr, uid, setor_ids, context=context):
            for rec_produto in rec_setor.estoque_produto_ids:
                for rec_grade in rec_produto.produto_grade_ids:
                    for rec_codigo in rec_grade.codigo_barra_ids:
                        for rec_nutricional in rec_produto.nutricionais_ids:
                            codigo_barra = rec_codigo.codigo_barra
                            if not context.get('balexc_exportar_digito'):
                                codigo_barra = codigo_barra[:-1]
                            nome = rec_nutricional.name
                            valor_calorico = rec_nutricional.valor_calorico
                            per_valor_calorico = rec_nutricional.percentual_valor_calorico
                            carboidratos = rec_nutricional.carboidratos
                            per_caboidratos = rec_nutricional.percentual_carboidratos
                            proteinas = rec_nutricional.proteinas
                            per_proteinas = rec_nutricional.percentual_proteinas
                            gorduras_totais = rec_nutricional.gorduras_totais
                            per_gorduras_totais = rec_nutricional.percentual_gorduras_totais
                            gorduras_saturadas = rec_nutricional.gorduras_saturadas
                            per_gorduras_saturadas = rec_nutricional.percentual_gorduras_saturadas
                            gorduras_trans = rec_nutricional.gorduras_trans
                            per_gorduras_trans = rec_nutricional.percentual_gorduras_trans
                            fibra_alimentar = rec_nutricional.fibra_alimentar
                            per_fibra_alimentar = rec_nutricional.percentual_fibra_alimentar
                            calcio = '00000'
                            per_calcio = '0000'
                            ferro = '00000'
                            per_ferro = '0000'
                            sodio = rec_nutricional.sodio
                            per_sodio = rec_nutricional.percentual_sodio

                            vals = {'codigo_barra': codigo_barra, 'nome': nome,
                                    'valor_calorico': valor_calorico,
                                    'percentual_valor_calorico': per_valor_calorico,
                                    'carboidratos': carboidratos,
                                    'percentual_carboidratos': per_caboidratos,
                                    'proteinas': proteinas,
                                    'percentual_proteinas': per_proteinas,
                                    'gorduras_totais': gorduras_totais,
                                    'percentual_gorduras_totais': per_gorduras_totais,
                                    'gorduras_saturadas': gorduras_saturadas,
                                    'percentual_gorduras_saturadas': per_gorduras_saturadas,
                                    'gorduras_trans': gorduras_trans,
                                    'percentual_gorduras_trans': per_gorduras_trans,
                                    'fibra_alimentar': fibra_alimentar,
                                    'percentual_fibra_alimentar': per_fibra_alimentar,
                                    'calcio': calcio, 'percentual_calcio': per_calcio,
                                    'ferro': ferro, 'percentual_ferro': per_ferro,
                                    'sodio': sodio, 'percentual_sodio': per_sodio}
                            list_dados.append(vals)
        s_arq = obj_balanca_util.balutl_processa_dado_exportacao_balanca(cr, uid, 'F', 'N',
                                                                         list_dados, context)
        if not arquivo_de_nutricionais:
            self.write(cr, uid, ids, {'dados_nutricionais': base64.encodestring(s_arq.encode(
                'utf-8', 'strict'))}, context=context)
        return self._balexc_salvar_arquivo(s_arq, arquivo_de_nutricionais)

    def balexc_gerar_arq_nut_toledo(self, cr, uid, ids, context=None):
        """
        Gera arquivo de exportacao de nutricionais para balanca Toledo.
        Layout:
        NUT-NOVO PIC X
        NUT-ITEM PIC 9(6)
        NUT-RESERVADO PIC X
        NUT-QUANTIDADE PIC 999
        NUT-UNIDADE PIC 9
        NUT-MEDIDA-INTEIRA PIC 99
        NUT-MEDIDA-decimal PIC 9
        NUT-MEDIDA PIC 99
        NUT-VALOR-CALORICO PIC 9(4)
        NUT-CARBOIDRATOS PIC 9(3)v9
        NUT-PROTEINAS PIC 9(2)v9
        NUT-GORDURAS-TOTAIS PIC 9(2)v9
        NUT-GORDURAS-satur PIC 9(2)v9
        NUT-GORDURAS-TRANS PIC 9(2)v9
        NUT-FIBRA-ALIMENTAR PIC 9(2)v9
        NUT-SODIO PIC 9(4)v9
        NUT-CRLF PIC XX

        Onde:
        X = codigo alfanumerico (letras e/ou numeros)
        9 - apenas numeros
        v9 - casas decimais (virgula implicita, remover se necessário)
        (n) sequencia de n caracteres. ex: 9(5) = 99999
        """
        obj_balanca_util = self.pool.get('balanca.util')
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], context=context)
        setor_id = vals.get('setor_id', False)
        if setor_id:
            if isinstance(setor_id, tuple):
                setor_id = setor_id[0]
            setor_ids = [setor_id]
        else:
            company_id = vals.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            produto_ids = self.pool.get('estoque.produto').search(cr, uid, [('company_id', '=', company_id)],
                                                                  context=context)
            rec_produtos = self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['setor_id'], context=context)
            dict_setores = {}
            for rec_produto in rec_produtos:
                if rec_produto.get('setor_id', False):
                    dict_setores[rec_produto['setor_id'][0]] = True
            setor_ids = dict_setores.keys()
        if not setor_ids:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha registros a serem exportados. Nao ha nenhum produto associado a um setor de balanca, ou nao ha setores de balanca cadastrados.'))
        arquivo_de_nutricionais = self._balexc_get_parametro(cr, uid, 'arquivo_de_nutricionais', context=context)
        list_dados = []
        for rec_setor in self.pool.get('balanca.setor').browse(cr, uid, setor_ids, context=context):
            for rec_produto in rec_setor.estoque_produto_ids:
                for rec_grade in rec_produto.produto_grade_ids:
                    for rec_codigo_barra in rec_grade.codigo_barra_ids:
                        for rec_nutricional in rec_produto.nutricionais_ids:
                            if rec_codigo_barra.precos_ids:
                                codigo_barra = rec_codigo_barra.codigo_barra
                                if not context.get('balexc_exportar_digito'):
                                    codigo_barra = codigo_barra[:-1]
                                reservado = ' '
                                reservado1 = ' '
                                quantidade = rec_nutricional.quantidade
                                unidade_porcao = rec_nutricional.unidade_porcao
                                medida_inteiro = rec_nutricional.medida_inteiro
                                medida_decimal = rec_nutricional.medida_decimal
                                medida = rec_nutricional.medida
                                valor_calorico = rec_nutricional.valor_calorico
                                carboidratos = rec_nutricional.carboidratos
                                proteinas = rec_nutricional.proteinas
                                gorduras_totais = rec_nutricional.gorduras_totais
                                gorduras_saturadas = rec_nutricional.gorduras_saturadas
                                gorduras_trans = rec_nutricional.gorduras_trans
                                fibra_alimentar = rec_nutricional.fibra_alimentar

                                vals = {'reservado': reservado, 'codigo_barra': codigo_barra,
                                        'reservado1': reservado1, 'quantidade': quantidade,
                                        'unidade_porcao': unidade_porcao,
                                        'medida_inteiro': medida_inteiro,
                                        'medida_decimal': medida_decimal, 'medida': medida,
                                        'valor_calorico': valor_calorico,
                                        'carboidratos': carboidratos, 'proteinas': proteinas,
                                        'gorduras_totais': gorduras_totais,
                                        'gorduras_saturadas': gorduras_saturadas,
                                        'gorduras_trans': gorduras_trans,
                                        'fibra_alimentar': fibra_alimentar}
                                list_dados.append(vals)
        s_arq = obj_balanca_util.balutl_processa_dado_exportacao_balanca(cr, uid, 'T', 'N',
                                                                         list_dados, context)
        if not arquivo_de_nutricionais:
            self.write(cr, uid, ids, {'dados_nutricionais': base64.encodestring(s_arq.encode('utf-8', 'strict'))}, context=context)
        return self._balexc_salvar_arquivo(s_arq, arquivo_de_nutricionais)

    def _balexc_gerar_arquivo_receita_toledo(self, cr, uid, ids, context=None):

        """
        Gera arquivo de exportacao de receita para a balanca toledo
        """

        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        parametro_estoque = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_diretorio_arquivo_de_receita', context=context)
        diretorio_arquivo_receita = parametro_estoque.get('estpar_diretorio_arquivo_de_receita')
        dados_estoque_produto = self._balexc_get_dado_receita(cr, uid, ids, context=context)
        s_arq = str()
        for produto in dados_estoque_produto:
            codigo = produto.get('codigo_barra')
            if not context.get('balexc_exportar_digito'):
                codigo = codigo[:-1]

            titulo_receita = produto.get('estpro_receita_titulo') or ''
            corpo_receita = (produto.get('estpro_receita_corpo') or '').replace('\n', ' ')
            titulo_receita = obj_adm_utils.admutl_remove_caracter(titulo_receita)
            titulo_receita = obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                                                titulo_receita)
            corpo_receita = obj_adm_utils.admutl_remove_caracter(corpo_receita)
            corpo_receita = obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                                                corpo_receita)
            limite_corpo_receita = 15 * 56
            if len(corpo_receita) > limite_corpo_receita:
                corpo_receita = corpo_receita[:limite_corpo_receita]
            corpo_receita = corpo_receita.split('\n')
            s_arq += f_num(codigo, 6)
            s_arq += f_txt(titulo_receita, 100)
            count = 0
            diferenca = 0
            for corpo in corpo_receita:
                tamanho_corpo = len(corpo)
                count += tamanho_corpo
                if count > limite_corpo_receita:
                    tamanho_corpo = limite_corpo_receita - count
                    diferenca = tamanho_corpo
                    corpo = corpo[:tamanho_corpo]
                elif tamanho_corpo < 56:
                    corpo = f_txt(corpo, 56)
                    tamanho_corpo = 56
                if diferenca:
                    s_arq += f_txt(corpo, tamanho_corpo)
                    break
                s_arq += f_txt(corpo, tamanho_corpo)
            s_arq += '\r\n'
        if not diretorio_arquivo_receita:
            vals = {'balexc_dado_receita': base64.encodestring(s_arq.encode('utf-8', 'strict')),
                    'balexc_file_receita': 'txtinfo.txt'}
            self.write(cr, uid, ids, vals, context)
        else:
            diretorio_arquivo_receita += '/txtinfo.txt'
        return self._balexc_salvar_arquivo(s_arq, diretorio_arquivo_receita)

    def _balexc_gerar_arquivo_receita_filizola(self, cr, uid, ids, context=None):

        """
        Gera arquivo de exportacao de receita para a balanca filizola
        """

        obj_balanca_util = self.pool.get('balanca.util')
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        parametro_estoque = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_diretorio_arquivo_de_receita', context=context)
        diretorio_arquivo_receita = parametro_estoque.get('estpar_diretorio_arquivo_de_receita')
        dados_estoque_produto = self._balexc_get_dado_receita(cr, uid, ids, context=context)
        list_dados = []
        for produto in dados_estoque_produto:
            reservado = ''
            codigo = produto.get('codigo_barra')
            if not context.get('balexc_exportar_digito'):
                codigo = codigo[:-1]
            codigo_receita = codigo
            titulo_receita = produto.get('estpro_receita_titulo') or ''
            corpo_receita = (produto.get('estpro_receita_corpo') or '').replace('\n', ' ')
            titulo_receita = obj_adm_utils.admutl_remove_caracter(titulo_receita)
            titulo_receita = obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                                                titulo_receita)
            corpo_receita = obj_adm_utils.admutl_remove_caracter(corpo_receita)
            corpo_receita = obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                                                corpo_receita)
            limite_corpo_receita = 4760
            if len(corpo_receita) > limite_corpo_receita:
                corpo_receita = corpo_receita[:limite_corpo_receita]
            corpo_receita = corpo_receita.split('\n')
            vals = {'reservado': reservado, 'codigo': codigo, 'codigo_receita': codigo_receita,
                    'titulo_receita': titulo_receita, 'corpo_receita': corpo_receita,
                    'limite_corpo_receita': limite_corpo_receita}
            list_dados.append(vals)
        s_arq = obj_balanca_util.balutl_processa_dado_exportacao_balanca(cr, uid, 'F', 'R',
                                                                         list_dados, context)
        if not diretorio_arquivo_receita:
            vals = {'balexc_dado_receita': base64.encodestring(s_arq.encode('utf-8', 'strict')),
                    'balexc_file_receita': 'rec_ass.txt'}
            self.write(cr, uid, ids, vals, context)
        return self._balexc_salvar_arquivo(s_arq, diretorio_arquivo_receita)

    def _balexc_get_dado_receita(self, cr, uid, ids, context=None):

        """
        Busca das tabelas estoque_produto e automacao_codigo_barra os dados referentes a receita
        @return: Lista de dicionarios contendo os dados
        """

        context = context and context.copy() or {}
        setor_id = self.browse(cr, uid, ids[0], context).setor_id.id
        sql = """
            SELECT
                ep.estpro_receita_titulo, ep.estpro_receita_corpo, acb.codigo_barra
            FROM
                estoque_produto ep
            INNER JOIN automacao_codigo_barra acb ON ep.id = acb.produto_id
            WHERE
                ep.produto_de_balanca = True AND
                ep.estpro_receita_titulo is not null AND
                ep.estpro_receita_corpo is not null AND
        """
        if setor_id:
            sql += """
                ep.setor_id = %d
            """ % setor_id
        else:
            sql += """
                ep.setor_id is not null
            """
        cr.execute(sql)
        dados_produto_receita = cr.dictfetchall()
        if not dados_produto_receita:
            raise osv.except_osv(_('Atencao'), _('Nao ha produto de balanca e setor ou titulo e '
                                                 'receita nao estao cadastrados'))
        return dados_produto_receita
