# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time


class entrega_montar_carga_osv(osv.TransientModel):
    def _get_carga(self, cr, uid, ids, name, arg, context=None):
        """
        Metodo Calcular a carga dos itens identificados para a viagem
        Deve buscar o peso e o volume de cada item identificado na viagem para
        apurar a carga total
        """
        context = context and context.copy() or {}
        res = {}
        for _id in ids:
            res[_id] = {}
            for brw_wizard in self.browse(cr, uid, ids):
                _volume = _peso = 0.00
                for brw_wiz_entregas in brw_wizard.entrega_ids:
                    for brw_wizard_item in brw_wiz_entregas.item_ids:
                        _volume += (brw_wizard_item.quantidade *
                                    brw_wizard_item.item_id.entvei_embalagem_id.estpre_cubagem)
                        _peso += (brw_wizard_item.quantidade *
                                  brw_wizard_item.item_id.entvei_embalagem_id.peso_bruto)
                res[_id].update({'carga_peso': _peso})
                res[_id].update({'carga_volume': _volume})
        return res

    _name = 'entrega.montar.carga.osv'
    _columns = {
        'viagem_id': fields.many2one('entrega.viagem', 'Codigo da viagem', ondelete='cascade'),
        'data': fields.date('Data prevista', required=True),
        'veiculo_volume': fields.float('Volume(Veic)', readonly=True),
        'veiculo_peso': fields.float('Peso(Veic)', readonly=True),
        'veiculo_id': fields.many2one("veiculos.veiculo", 'Veiculo', ondelete="cascade",
                                      required=True, select=True),
        'motorista_id': fields.many2one("veiculos.motorista", 'Motorista',
                                        ondelete="cascade", required=True, select=True),
        'ajudante_ids': fields.many2many("rh.funcionario", "entrega_montar_carga_ajudantes_osv",
                                         "viagem_id",
                                         "funcionario_id", 'Ajudantes'),
        'state': fields.selection([('E', 'Esboco'), ('A', 'Ativo')],
                                  'Situacao', size=1, required=True),
        'carregamento_inicio': fields.datetime('Inicio do carregamento'),
        'carregamento_termino': fields.datetime('Termino do carregamento'),
        'data_hora_saida': fields.datetime('Data/Hora da saida'),
        'data_hora_retorno': fields.datetime('Data/Hora do retorno'),
        'obs': fields.text('Informacoes complementares'),
        'liberou_user_id': fields.many2one('res.users', 'Usuario que liberou',
                                           ondelete='restrict'),
        'liberou_data_hora': fields.datetime('Data/Hora da liberacao'),
        'carga_volume': fields.function(_get_carga, multi='volume', method=True,
                                        string='Volume(Carga)', type='float', store=True),
        'carga_peso': fields.function(_get_carga, multi='peso', method=True,
                                      string='Peso(Carga)', type='float', store=True),
        'entrega_ids': fields.one2many('entrega.montar.carga.entregas.osv', 'capa_id', 'Entregas'),
        'item_ids': fields.one2many('entrega.montar.carga.item.osv', 'capa_id', 'Itens'),

    }
    _defaults = {
        'state': lambda *a: 'E',
        'data': lambda *a: time.strftime('%Y-%m-%d'),
    }

    def button_salvar(self, cr, uid, ids, context=None):
        return True

    def button_ativar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for brw_carga in self.browse(cr, uid, ids):
            vals = {
                'entvig_data': brw_carga.data,
                'entvig_veiculo_volume': brw_carga.veiculo_volume,
                'entvig_veiculo_peso': brw_carga.veiculo_peso,
                'entvig_veiculo_id': brw_carga.veiculo_id.id,
                'entvig_motorista_id': brw_carga.motorista_id.id,
                'state': 'M',
                'entvig_obs': brw_carga.obs,
                'entvig_item_ids': [],
                'entvig_ajudante_ids': [],
            }
            for _item in brw_carga.item_ids:
                _vals_itens = {
                    'entvit_item_id': _item.item_id.id,
                    'entvit_quantidade': _item.quantidade,
                    'entvit_venda_id': _item.venda_id.id,
                    'entvit_embalagem_id': _item.item_id.entvei_embalagem_id.id
                }
                vals['entvig_item_ids'].append((0, 0, _vals_itens))
            for _ajudante in brw_carga.ajudante_ids:
                vals['entvig_ajudante_ids'].append((4, _ajudante.id))
            _viagem_id = self.pool.get('entrega.viagem').create(cr, uid, vals, context=context)
        self.write(cr, uid, ids, {'state': 'A', 'viagem_id': _viagem_id}, context=context)
        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = super(entrega_montar_carga_osv, self).create(cr, uid, vals, context)
        return res


    def wiz_onchange_veiculo(self, cr, uid, ids, veiculo_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if not veiculo_id:
            return res
        _campos = ['carga_maxima_peso', 'carga_altura', 'carga_largura', 'carga_comprimento',
                   'motorista_id']
        dados_veiculos_veiculo = self.pool.get('veiculos.veiculo').read(cr, uid, veiculo_id,
                                                                        _campos, context=context)
        _vol = ((dados_veiculos_veiculo.get('carga_altura') or 0) *
                (dados_veiculos_veiculo.get('carga_largura') or 0) *
                (dados_veiculos_veiculo.get('carga_comprimento') or 0))
        vals = {
            'veiculo_volume': _vol,
            'veiculo_peso': dados_veiculos_veiculo.get('carga_maxima_peso') or 0,
            'motorista_id': dados_veiculos_veiculo.get('motorista_id')
        }
        res['value'] = vals
        return res


class entrega_montar_carga_entregas_osv(osv.TransientModel):
    def _get_name(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        res = {}
        for i in self.browse(cr, uid, ids):
            res.update({i.id: i.entrega_id.name})
        return res

    _name = 'entrega.montar.carga.entregas.osv'
    _columns = {
        'name': fields.function(_get_name, method=True, string='Descricao',
                                type='text', store=True),
        'capa_id': fields.many2one('entrega.montar.carga.osv', 'Capa', required=True,
                                   ondelete='cascade'),
        'entrega_id': fields.many2one('entrega.vendas.capa', 'Entrega', required=True,
                                      ondelete="cascade", domain="[('state', 'in', ('PE','PA'))]"),
        'pedido_id': fields.many2one("estoquepedidos.pedido.capa", 'Pedido', ondelete="cascade"),
        'nf_id': fields.many2one('fiscal.nf.capa', 'Nota', ondelete='cascade'),
        'data_prevista': fields.date('Data prevista'),
        'state': fields.selection([('PE', 'Pendente'), ('PA', 'Parcial'), ('SP', 'Separado'),
                                   ('CA', 'Cancelada')], 'Situacao', size=2),
        'item_ids': fields.one2many("entrega.montar.carga.item.osv", "capa_entrega_id",
                                    "Itens da carga"),

    }

    def wiz_onchange_entrega(self, cr, uid, ids, entrega_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if not entrega_id:
            return res
        brw_entrega_vendas_capa = self.pool.get('entrega.vendas.capa').browse(cr, uid, entrega_id)
        vals = {
            'pedido_id': brw_entrega_vendas_capa.entvec_pedido_id.id or None,
            'nf_id': brw_entrega_vendas_capa.entvec_nf_id.id or 0,
            'data_prevista': brw_entrega_vendas_capa.entvec_data_prevista,
            'state': brw_entrega_vendas_capa.state,
            'item_ids': []
        }
        for brw_entrega_vendas_item in brw_entrega_vendas_capa.entvec_vendas_item_ids:
            _qtd = brw_entrega_vendas_item.entvei_qtd_pedida - brw_entrega_vendas_item.entvei_qtd_separada
            if _qtd == 0:
                continue
            _vals_itens = {
                'name': brw_entrega_vendas_item.name,
                'item_id': brw_entrega_vendas_item.id,
                'venda_id': brw_entrega_vendas_item.entvei_capa_id.id,
                'quantidade': _qtd,
                'embalagem_id': brw_entrega_vendas_item.entvei_embalagem_id.id
            }
            vals['item_ids'].append((0, 0, _vals_itens))
        res['value'] = vals
        return res


class entrega_montar_carga_item_osv(osv.TransientModel):
    def _get_name(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        res = {}
        for i in self.browse(cr, uid, ids):
            res.update({i.id: i.item_id.entvei_produto_id.name})
        return res

    _name = 'entrega.montar.carga.item.osv'
    _columns = {
        'name': fields.function(_get_name, method=True, string='Item', type='text', store=True),
        'capa_id': fields.many2one('entrega.montar.carga.osv', 'Capa', required=True,
                                   ondelete='cascade'),
        'capa_entrega_id': fields.many2one('entrega.montar.carga.entregas.osv', 'Entrega',
                                           required=True, ondelete='cascade'),
        'venda_id': fields.many2one('entrega.vendas.capa', 'Capa', ondelete="cascade", required=True),
        'item_id': fields.many2one('entrega.vendas.item', 'Item', ondelete="cascade", required=True,
                                   select=True),
        'quantidade': fields.float('Quantidade', required=True),
        'embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                               ondelete='restrict', required=True),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        _entrega_id = vals.get('capa_entrega_id')
        _capa_id = self.pool.get('entrega.montar.carga.entregas.osv').browse(cr, uid,
                                                                             _entrega_id).capa_id.id
        vals['capa_id'] = _capa_id
        res = super(entrega_montar_carga_item_osv, self).create(cr, uid, vals, context)
        return res
