# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import time


class estoque_produto_descricao_anterior(osv.Model):

    _inherit = 'estoque.produto.descricao.anterior'

    def esteda_verifica_inclusao(self, cr, uid, produto_ids, descricao, context=None):
        """
        O metodo faz inclusao de registro na tabela dessa classe baseada na alterecao da descricao
        do produto (name). Caso ja possua um registro para um mesmo produto e data fim, nao sera
        criado um novo.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_ids: conjunto de id da tabela estoque_produto.
        :type produto_ids: list
        :param descricao: novo nome para os produtos.
        :type descricao: str
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, para a inclusao de um ou mais registros. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False
        estoque_produto_dados = self.pool.get('estoque.produto').read(
            cr, uid, produto_ids, ['name'], context=context)
        for produto in estoque_produto_dados:
            if self.search(cr, uid, [('esteda_produto_id', '=', produto.get('id')),
                                     ('esteda_data_fim', '=', time.strftime('%Y-%m-%d'))],
                           context=context):
                continue
            descricao_anterior_ids = self.search(
                cr, uid, [('esteda_produto_id', '=', produto.get('id')),
                          ('esteda_data_fim', '<', time.strftime('%Y-%m-%d'))], limit=1,
                order='esteda_data_fim DESC', context=context)
            if descricao_anterior_ids:
                data_inicio = self.browse(cr, uid, descricao_anterior_ids[0],
                                          context).esteda_data_inicio or ''
            else:
                sql = """
                    SELECT
                        min(emc.data) as data_inicio
                    FROM estoque_movimento_itens emi
                    INNER JOIN estoque_movimento_capa emc
                        ON emc.id = emi.capa_id
                    WHERE emi.produto_id = %s
                """ % produto.get('id')
                cr.execute(sql)
                data_inicio = cr.fetchone()
                if data_inicio:
                    data_inicio = data_inicio[0]
            vals = {'esteda_descricao_anterior': produto.get('name'),
                    'esteda_data_fim': time.strftime('%Y-%m-%d'),
                    'esteda_produto_id': produto.get('id'),
                    'esteda_data_inicio': data_inicio or time.strftime('%Y-%m-%d')}
            self.create(cr, uid, vals, context)
            res = True
        return res
