# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from datetime import datetime
import calendar
from openerp.tools.translate import _


class estoque_inventario_contagem(osv.Model):
    _name = 'estoque.inventario.contagem'
    _inherit = 'estoque.inventario.contagem'

    def read(self, cr, uid, ids, fields=[], context=None, load='_classic_read'):
        fields = list(set(fields))
        res = super(estoque_inventario_contagem, self).read(cr, uid, ids, fields, context, load)
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        grade_id = vals.get('grade_id')
        if isinstance(grade_id, list):
            grade_id = grade_id[0]

        if not vals['produto_id']:
            obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
            vals['produto_id'] = obj_estoque_produto_grade.read(cr, uid, grade_id, ['produto_id'],
                                           context=context, load='_classic_write').get('produto_id')

        res = super(estoque_inventario_contagem, self).create(cr, uid, vals, context=context)
        return res

    def gerar_contagem(self, cr, uid, inventario_id, data_inventario, efeito_de_inventario,
                       zerar_saldo, local_id=False, locacao_id=False, categoria_id=False,
                       fornecedor_id=False, zerar_saldo_negativo=False, context=None, **kwargs):
        """
        Este metodo tem a funcionalidade de gerar os dados na estoque
        :param cr: Cursor do openerp
        :param uid: Id do usuario requisita
        :param data_inventario: Data do inventario
        :param efeito_de_inventario: Se o inventario e apenas para efeito de inventario
        :param zerar_saldo: Se e pra zerar o saldo dos itens
        :param local_id: Id do local
        :param locacao_id: Id da locacao
        :param categoria_id: Id da categoria
        :param fornecedor_id: Id do fornecedor
        :param zerar_saldo_negativo: Boolean que identifica se itens com saldo negativo sera zerado
        :param context: Dicionario contendo os valores contextuais
        :type cr: Curosr
        :type uid: int
        :type inventario_id: int
        :type data_inventario: str
        :type efeito_de_inventario: bool
        :type zerar_saldo: bool
        :type fornecedor_id: int
        :type zerar_saldo_negativo: bool
        :type context: dict
        :return: Retorna uma lista dos registros criados
        :rtype: list
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = []
        locacao_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context)
        dados = self.get_dados_para_contagem(cr, uid, local_id, locacao_id, categoria_id,
                                             fornecedor_id, locacao_company_id, context=context,
                                             **kwargs)
        if dados:
            produtos_por_local = self._agrupar_por_local(dados)
            dados_para_contagem, produto_ids, grades_ids = self._get_saldo_para_inventario(
                cr, uid, data_inventario, efeito_de_inventario, produtos_por_local, context=context)
            custos = self._get_custo_itens_para_inventario(
                cr, uid, efeito_de_inventario, data_inventario, produto_ids, context)
            funcionario_id = self.pool.get('rh.funcionario').get_id_func(cr, uid)
            if not funcionario_id:
                raise osv.except_osv(_('Atencao'), _('Usuario nao esta cadastrado como '
                                                     'funcionario.'))
            res = self.gerar_contagem_inicial(
                cr, uid, grades_ids, dados_para_contagem, custos, zerar_saldo, funcionario_id,
                inventario_id, zerar_saldo_negativo, context=context)
        return res

    def get_dados_para_contagem(self, cr, uid, local_id, locacao_id, categoria_id, fornecedor_id,
                                locacao_company_id, context=None, **kwargs):
        """
        Metodo responsavel por retornar as informacoes dos dados dos itens que serao utilizados para
        contagem
        :param cr: Cursor do openerp
        :param uid: Id do usuario que esta fazendo a chamada
        :param local_id: ID de estoque.local para fins de filtro
        :param locacao_id: ID de estoque.local.locacao para fins de filtro
        :param categoria_id: ID de estoque.categoria para fins de filtro
        :param fornecedor_id: ID de estoque.fornecedor para fins de filtro
        :param locacao_company_id: ID da empresa que estoque.local.locacao esta correlacionada
        :param kwargs: Dicionarios contendo demais parametros, e utilizado por classes que
                       sobrescrevem este metodo, poderem receber outros parametros especificos
        :type cr: Curosr
        :type local_id: int
        :type locacao_id: int
        :type categoria_id: int
        :type fornecedor_id: int
        :type locacao_company_id: int
        :type kwargs: dict
        :return: Retorna a lista de dicionarios com os dados dos itens que serao considerados
                 para o inventario
        :rtype: list
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        vals_to_sql = {
            'args_company_id': locacao_company_id
        }
        sql = """
            select
                locacao.id               as locacao_id,
                locacao.local_id         as local_id,
                locacao.produto_id       as produto_id,
                locacao.grade_id         as grade_id,
                fornecedor.fornecedor_id as fornecedor_id
            from
                estoque_local_locacao locacao
                left join estoque_produto produto
                    on produto.id = locacao.produto_id
                left join estoque_produto_grade grade
                    on grade.id = locacao.grade_id
                left join estoque_local estlocal
                    on estlocal.id = locacao.local_id
                left join estoque_fornecedor fornecedor
                    on produto.id = fornecedor.produto_id
            where
                produto.active IS TRUE
                AND grade.active IS TRUE
                AND locacao.company_id = '%(args_company_id)s'
                %(args_local_id)s
                %(args_categoria)s
                %(args_locacao_id)s
                %(args_fornecedor_id)s
            order by
                estlocal.name, locacao.locacao, produto.name
        """
        if local_id:
            args_local = "AND estlocal.id = %d AND locacao.active IS TRUE" % local_id
        else:
            args_local = ""
        if locacao_id:
            args_locacao = "and locacao.id = %d" % locacao_id
        else:
            args_locacao = ""
        if categoria_id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)
            if len(categoria_ids) == 1:
                categoria_ids = str(tuple(categoria_ids)).replace(',', '')
            elif len(categoria_ids) > 1:
                categoria_ids = str(tuple(categoria_ids))
            args_categoria = "and produto.categoria_id IN %s" % categoria_ids
        else:
            args_categoria = ""
        if fornecedor_id:
            args_fornecedor = "and fornecedor.fornecedor_id = %d" % fornecedor_id
        else:
            args_fornecedor = ""

        vals_to_sql.update({'args_local_id': args_local, 'args_categoria': args_categoria,
                            'args_locacao_id': args_locacao,
                            'args_fornecedor_id': args_fornecedor, })

        sql_formatted = sql % vals_to_sql

        cr.execute(sql_formatted)
        return cr.dictfetchall()

    @staticmethod
    def _agrupar_por_local(dados_para_contagem):
        """
        Este metodo agrupa todos os itens por local
        EX: LOCAL 1: PRODUTO_IDS - 1,2,3,4 GRADE_IDS - 1,2,3,4
        EX: LOCAL 2: PRODUTO_IDS - 1,2,3,5,6 GRADE_IDS - 1,2,3,5,6
        :param dados_para_contagem: Lista de dicionarios contendo os valores recuperados
                                    do metodo get_dados_para_contagem
        :type dados_para_contagem: list
        :return: Dicionarios contendo os produtos agrupados pelos locais
        :rtype: dict:
        author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        dict_local_produtos = {}
        for i in dados_para_contagem:
            produto_id = i.get('produto_id')
            grade_id = i.get('grade_id')
            local_id = i.get('local_id')

            if local_id in dict_local_produtos:
                dict_local_produtos[local_id]['produto_ids'].append(produto_id)
                dict_local_produtos[local_id]['grade_ids'].append(grade_id)
            else:
                dict_local_produtos[local_id] = {
                    'produto_ids': [produto_id],
                    'grade_ids': [grade_id],
                    }
        return dict_local_produtos

    def _get_saldo_para_inventario(self, cr, uid, data_inventario, efeito_de_inventario,
                                   itens_agrupados_por_local, context=None):
        """
        Este metodo tem a funcionalidade de buscar o saldo dos itens para o inventario.
        Se o efeito_de_inventario for Verdadeiro busca o saldo do ultimo dia do mes do inventario,
        senao busca data data corrente.
        :param cr: Cursor do openerp
        :param uid: Id do usuario responsavel pela chamada da funcao
        :param efeito_de_inventario: Se e apenas para efeito de inventario
        :param itens_agrupados_por_local: dados dos itens agrupados pelo seu local
        :param context: Dicionario contendo os valores contextuais
        :type cr: Curosr
        :type uid: int
        :type data_inventario: str
        :type efeito_de_inventario: bool
        :type itens_agrupados_por_local: dict
        :type context: dict
        :returns: Retorna a lista do saldo dos itens alem dos ids da grade e dos produtos
        :rtype: list, list, list
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        list_produto_ids = []
        list_grades_ids = []
        res = []
        data_movimento = 'current_date'
        if efeito_de_inventario:
            ano = datetime.strptime(data_inventario, " '%Y-%m-%d' ").year
            mes = datetime.strptime(data_inventario, " '%Y-%m-%d' ").month
            ultimo_dia_mes_ano = self._get_ultimo_dia_mes_ano(ano, mes)
            data_movimento = " '%s' " % ultimo_dia_mes_ano
        for local_id in itens_agrupados_por_local:
            produto_ids = tuple(set(itens_agrupados_por_local[local_id]['produto_ids']))
            grade_ids = tuple(set(itens_agrupados_por_local[local_id]['grade_ids']))
            list_produto_ids.extend(produto_ids)
            list_grades_ids.extend(grade_ids)
            res.extend(
                self._get_saldo_local_para_inventario(
                    cr, uid, data_movimento, local_id, produto_ids, grade_ids, context))
        return res, list_produto_ids, list_grades_ids

    def _get_saldo_local_para_inventario(self, cr, uid, data_movimento, local_id, produto_ids,
                                         grade_ids, context=None):
        """
        Este metodo tem a funcionalidade de buscar o saldo dos itens para o local informado.
        :param cr: Cursor do openerp
        :param uid: Id do usuario requisita
        :param data_movimento: Data a ser considerada na pesquisa
        :param produto_ids: Ids dos produtos a pesquisar
        :param grade_ids: Ids das grades a pesquisar
        :param context: Dicionario contendo os valores contextuais
        :type cr: Curosr
        :type uid: int
        :type data_movimento str
        :type local_id: int
        :type produto_ids: tuple
        :type grade_ids: tuple
        :type context: dict
        :return: Retorna um dicionario contendo o dados referente ao saldo do produto
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        locacao_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context)
        saldo_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.produto.saldo', context=context)
        str_produto_fields = '%s, ' * produto_ids.__len__()
        str_grade_fields = '%s, ' * grade_ids.__len__()

        str_produto_values = str_produto_fields % produto_ids
        str_grade_values = str_grade_fields % grade_ids

        str_produto_values = ("(%s)" % str_produto_values).replace(', )', ')')
        str_grade_values = ("(%s)" % str_grade_values).replace(', )', ')')
        sql = """
                SELECT row_number() over (ORDER BY locacao.id) AS id,
                       locacao.local_id   AS pgsaldo_local_id,
                       locacao.id         AS pgsaldo_locacao_id,
                       locacao.produto_id AS pgsaldo_produto_id,
                       locacao.grade_id   AS pgsaldo_grade_id,
                       locacao.company_id AS pgsaldo_company_id,
                       (SELECT SUM(saldo.saldo) AS saldo
                        FROM estoque_produto_saldo AS saldo
                        WHERE saldo.data_movimento = (SELECT Max(data_movimento)
                                                      FROM estoque_produto_saldo AS eps1
                                                      WHERE eps1.company_id = saldo.company_id
                                                        AND eps1.produto_id = saldo.produto_id
                                                        AND eps1.grade_id = saldo.grade_id
                                                        AND eps1.local_id = saldo.local_id
                                                        AND eps1.data_movimento <= {data_movimento}
                                                        AND eps1.company_id = {saldo_company_id}
                                                      )
                          AND saldo.produto_id = locacao.produto_id
                          AND saldo.grade_id = locacao.grade_id
                          AND saldo.local_id = locacao.local_id
                          AND saldo.company_id = {saldo_company_id}
                       ) AS pgsaldo_saldo

                FROM estoque_local_locacao AS locacao
                WHERE locacao.company_id = {locacao_company_id}
                  AND locacao.produto_id IN {produto_ids}
                  AND locacao.grade_id IN {grade_ids}
                  AND locacao.local_id = {local_id}

                ORDER BY locacao.local_id,
                         locacao.produto_id
                """.format(data_movimento=data_movimento, saldo_company_id=saldo_company_id,
                           locacao_company_id=locacao_company_id, produto_ids=str_produto_values,
                           grade_ids=str_grade_values, local_id=local_id)
        cr.execute(sql)
        return cr.dictfetchall()

    def atualiza_vals_com_trib(self, vals, produto_id, dict_trib, context=None):
        'Implementado por Heranca no modulo fiscal'
        context = context and context.copy() or {}

    def get_dict_trib(self, cr, uid, produto_ids, dict_trib, context=None):
        'Implementado por Heranca no modulo fiscal'
        context = context and context.copy() or {}

    def gerar_contagem_inicial(self, cr, uid, grade_ids, dados_para_contagem, custos,
                               zerar_saldo, funcionario_id, inventario_capa_id,
                               zerar_saldo_negativo, context=None):
        """
        Este metodo tem a funcionalidade de gerar os dados na estoque
        :param cr: Cursor do openerp
        :param uid: Id do usuario requisita
        :param grade_ids: Lista contendo os ids de estoque.produto.grade
        :param dados_para_contagem: Lista de dicionarios contendo os dados retornados do metodo
                                    get_dados_para_contagem
        :param custos:  Custo dos produtos retornados do metodo _get_custo_itens_para_inventario
        :param zerar_saldo: Boolean informando se eh pra zerar saldo ou nao
        :param funcionario_id: Id do funcionario que esta gerando o inventario
        :param inventario_capa_id: Id do inventario criado
        :param zerar_saldo_negativo: Boolean que identifica se itens com saldo negativo sera zerado
        :param context: Dicionario contendo os valores contextuais
        :type cr: Curosr
        :type uid: int
        :type grade_ids: list
        :type dados_para_contagem: list
        :type custos: dict
        :type zerar_saldo: bool
        :type funcionario_id: int
        :type inventario_capa_id: int
        :type zerar_saldo_negativo: bool
        :type context: dict
        :return: Retorna uma lista dos registros criados
        :rtype: list
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        grade = self.pool.get('estoque.produto.grade')
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_inventario_tipo_contagem', context=context)
        tipo_contagem = estoque_parametro_dados.get('estpar_inventario_tipo_contagem')
        qtd_reservada = grade.estgrd_get_requisitado(cr, uid, list(set(grade_ids)), context=context)

        args_contagem = [('inventario_id', '=', inventario_capa_id)]
        contagem_ids = self.search(cr, uid, args_contagem, context=context)
        dict_contagem = {}
        if contagem_ids:
            for i in self.read(cr, uid, contagem_ids, ['produto_id', 'grade_id', 'local_id'],
                               context=context, load='_classic_write'):
                chave = '-'.join([i.get('produto_id'), i.get('grade_id'), i.get('local_id')])
                dict_contagem[chave] = i.get('id')

        dict_trib = {}
        produto_ids = map(lambda x: x.get('produto_id'), dados_para_contagem)
        self.get_dict_trib(cr, uid, produto_ids, dict_trib, context=context)
        ids = []
        dados_contagem_insert = []
        for dado in dados_para_contagem:
            requisitado = 0.0
            grade_id = dado.get('pgsaldo_grade_id')
            local_id = dado.get('pgsaldo_local_id')
            locacao_id = dado.get('pgsaldo_locacao_id')
            produto_id = dado.get('pgsaldo_produto_id')
            if qtd_reservada.get(grade_id):
                requisitado = qtd_reservada.get(grade_id).get(local_id) or 0.0
            saldo = (dado.get('pgsaldo_saldo') or 0) - requisitado
            custo_medio = None
            ultimo_custo = None
            if custos.get(produto_id):
                custo_medio = custos.get(produto_id).get('estprc_custo_medio_contabil')
                ultimo_custo = custos.get(produto_id).get('estprc_ultimo_custo')

            saldo_contado = saldo
            if zerar_saldo:
                saldo_contado = 0.00
            elif zerar_saldo_negativo and saldo < 0:
                saldo_contado = 0.00

            conferido = False if not zerar_saldo else True
            vals = dict(inventario_id=inventario_capa_id,
                        produto_id=produto_id,
                        grade_id=grade_id,
                        local_id=local_id,
                        locacao_id=locacao_id,
                        estict_custo_medio_contabil=custo_medio or 0.0,
                        estict_ultimo_custo=ultimo_custo or 0.0,
                        saldo_cadastro=saldo,
                        saldo_contado=saldo_contado,
                        conferido=conferido,
                        funcionario_id=funcionario_id,
                        requisitado=requisitado)
            self.atualiza_vals_com_trib(vals, produto_id, dict_trib, context=context)
            chave = '%s-%s-%s' % (produto_id, grade_id, local_id)
            contagem_id = dict_contagem.get(chave)
            res = self.salvar_contagem(cr, uid, inventario_capa_id, produto_id, grade_id, local_id,
                                       vals, tipo_contagem, contagem_id, context=context)
            # Implementacao para otimizacao da geracao da contagem. Sera feito o insert em
            # massa no lugar do create item a item.
            if context.get('gerar_contagem_inventario'):
                dados_contagem_insert.append(res)
                continue
            ids.append(res)

        # Implementacao para otimizacao da geracao da contagem. Sera feito o insert em
        # massa no lugar do create item a item.
        if dados_contagem_insert:
            data = str(datetime.now())
            sql = """
                INSERT INTO estoque_inventario_contagem 
                    (create_uid, write_uid, create_date, write_date, locacao_id, saldo_contado, 
                    inventario_id, requisitado, estict_custo_medio_contabil, estict_ultimo_custo,
                    grade_id, conferido, local_id, funcionario_id, produto_id, saldo_cadastro)
                VALUES
                    %s
                RETURNING ID  
            """ % ', '.join(map(lambda ic: str(tuple([uid, uid, data, data] + ic.values())),
                                dados_contagem_insert))
            cr.execute(sql)
            ret_ids = cr.fetchall()
            if ret_ids:
                ids = map(lambda it: it[0], ret_ids)

        return ids

    def _get_custo_itens_para_inventario(self, cr, uid, efeito_de_inventario, data_inventario,
                                         produto_ids, context=None):
        """
        Este metodo busca o custo do produto de acordo com a data do inventario
        :param cr: Cursor do banco
        :param uid: Usuario que esta executando a operacao
        :param efeito_de_inventario: Se e apenas para efeito de inventario
        :param data_inventario: Data do inventario
        :param produto_ids: Lista contendo os ids dos produtos
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type efeito_de_inventario: bool
        :type data_inventario: str
        :type produto_ids: list
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        data_custo = data_inventario
        if efeito_de_inventario:
            ano = datetime.strptime(data_inventario, " '%Y-%m-%d' ").year
            mes = datetime.strptime(data_inventario, " '%Y-%m-%d' ").month
            data_custo = self._get_ultimo_dia_mes_ano(ano, mes)
        return self.pool.get('estoque.preco.custo').get_custos_do_dia(
            cr, uid, list(set(produto_ids)), epc_data=data_custo, context=context)

    @staticmethod
    def _get_ultimo_dia_mes_ano(ano, mes):
        """
        Retorna a data do ultimo dia do mes e ano informado no parametro
        :param ano: Ano informado para o inventario
        :param mes: Mes informado para o inventario
        :type ano: int
        :type mes: int
        :return: A string da data contendo o ultimo dia do mes, no formato "%Y-%m-%d"
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return '%s-%s-%s 23:59:59' % (ano, mes, calendar.monthrange(ano, mes)[1])

    def salvar_contagem(self, cr, uid, inventario_id, produto_id, grade_id, local_id, vals,
                        tipo_contagem, contagem_id, grade_ids_sem_contagem=[], context=None):
        """
        Este metodo salva a contagem gerada
        Esta funcao eh usada ao gerar um inventario quanto para a importacao de contagem do cloetor
        :param cr: Cursor do banco
        :param uid: Usuario que esta executando a operacao:
        :param inventario_id: Id do inventario gerado
        :param produto_id: Id do produto
        :param grade_id: Id da grade
        :param local_id: Id do local
        :param context: Dicionario contendo os valores contextuais
        :type cr: Cursor
        :type uid: int
        :type inventario_id: int
        :type produto_id: int
        :type grade_id: int
        :type local_id: int
        :type context: dict
        :return: Retorna o id do registro que foi criado ou alterado
        :rtype: long
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = contagem_id
        if contagem_id:
            if tipo_contagem == 'I':
                novo_saldo_contado = self.read(cr, uid, contagem_id, ['saldo_contado'], context
                                               ).get('saldo_contado')
                novo_saldo_contado += vals.get('saldo_contado')
                vals['saldo_contado'] = novo_saldo_contado
            self.write(cr, uid, contagem_id, vals, context=context)
        else: # os produtos que nao tiverem contagem nao devem ser criadas contagens.
            # observa o contexto para nao criar uma nova contagem quando estiver importando do
            # coletor
            if context.get('inventario_importado_do_coletor'):
                grade_ids_sem_contagem.append(grade_id)
            else:
                vals['inventario_id'] = int(inventario_id)

                # Implementacao para otimizacao da geracao da contagem. Sera feito o insert em
                # massa no lugar do create item a item.
                if context.get('gerar_contagem_inventario'):
                    return vals

                res = self.create(cr, uid, vals, context=context)

        return res
