# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class estoque_local(osv.Model):
    """
    Tabela de locais de armazenagem.
    """

    _name = 'estoque.local'
    _inherit = 'estoque.local'

    def get_permite_saldo_negativo(self, cr, uid, local_ids, company_id=False, context=None):
        context = context and context.copy() or {}
        dict_locais = {}
        if not company_id:
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        if isinstance(local_ids, (int, long)):
            local_ids = [local_ids]
        if isinstance(local_ids, (list, tuple)):
            local_ids = list(set(local_ids))
            dict_locais = {}.fromkeys(local_ids, False)
            local_company_ids = self.search(cr, uid, [('id', 'in', local_ids), ('company_id', '=', company_id)], context=context)
            if not local_company_ids and local_ids:
                empresa_name = self.pool.get('res.company').browse(cr, uid, company_id).name
                raise osv.except_osv(_('Atencao'), _('Nenhum local foi encontrado para a empresa: %s' % empresa_name))
            else:
                local_dados = self.read(cr, uid, local_ids, ['aceita_saldo_negativo'], context=context)
                if isinstance(local_dados, list):
                    try:
                        dict_locais.update(dict(map(lambda i: (i['id'], i['aceita_saldo_negativo']), local_dados)))
                    except:
                        pass
        return dict_locais

    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return {}
        context = context and context.copy() or {}
        reads = self.read(cr, uid, ids, ['numero', 'name'], context=context)
        res = []
        for record in reads:
            name = '%03d - %s' % (record['numero'], record['name'])
            if context.get('estlcl_so_codigo'):
                name = '%03d' % record['numero']
            res.append((record['id'], name))
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        # Bloco inserido na versao 3G1, para evitar erro na insercao de dados do arquivo local.xml
        # em bases que ja tem local cadastrado.
        # TODO: Pode ser removido apos a versao  4A1
        if 'install_mode' in context and context['install_mode']:
            args = [('company_id', '=', 1), ('numero', '=', 001)]
            _id = self.search(cr, uid, args, context=context)
            if _id:
                return _id[0]
            # Fim do Bloco #
        if vals.get('numero') <= 0:
            raise osv.except_osv(_('Atencao'), _('O numero do local deve ser maior que zero'))
        result = super(estoque_local, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'numero' in vals and vals.get('numero') < 1:
            raise osv.except_osv(_('Atencao'), _('O numero do local deve ser maior que zero'))
        result = super(estoque_local, self).write(cr, uid, ids, vals, context)
        return result

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        ids = []
        try:
            name = int(name)
        except:
            pass
        if isinstance(name, int):
            ids = self.search(cr, uid, [('numero', '=', name)] + args, limit=limit, context=context)
        if not ids:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

