# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time
from openerp.tools.translate import _
READ_LOAD = '_classic_write'


class estoque_local_locacao(osv.Model):
    """
    Tabela de locacoes
    Prefixo dos campos "estllc"
    """

    _name = 'estoque.local.locacao'
    _inherit = 'estoque.local.locacao'

    def onchange_local_id(self, cr, uid, ids, company_id, local_id, locacao, context=None):
        context = context and context.copy() or {}
        ini = time.time()
        if locacao:
            locacao = locacao.upper()
        fields = ['altura', 'largura', 'profundidade', 'data_validade',
                  'active', 'flag', 'produto_id', 'grade_id']
        fields_false = [False for x in range(len(fields))]
        fields_dict = dict(zip(fields, fields_false))
        flag = False
        if not ids:
            locacao_id = self.search(cr, uid, [('company_id', '=', company_id),
                                               ('local_id', '=', local_id),
                                               ('locacao', '=', locacao)], context=context)
            if locacao_id:
                vals = self.read(cr, uid, locacao_id, fields, context=context)[0]
                del vals['id']
                vals['flag'] = flag
                vals['locacao'] = locacao
                return {
                    'value': vals,
                    'warning': {
                        'title': _('Atencao'),
                        'message': _('Foi encontrada uma locacao com os mesmo valores digitados e '
                                     'ela foi retornada para a tela atual')
                    }
                }
            else:
                fields_dict['flag'] = flag
                return {'value': fields_dict}
        return {'value': {'flag': flag}}

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if 'locacao' in vals:
            if vals['locacao']:
                vals['locacao'] = vals['locacao'].upper()
        if vals.get('active') and not vals.get('active'):
            pass
        else:
            locacao_id = self.search(cr, uid, [('company_id', '=', vals.get('company_id')),
                                               ('local_id', '=', vals.get('local_id')),
                                               ('produto_id', '=', vals.get('produto_id')),
                                               ('grade_id', '=', vals.get('grade_id')),
                                               ('active', '=', True)], context=context
            )
            if locacao_id:
                raise osv.except_osv(_('Atencao'), _('Empresa, local, produto e grade precisam ser '
                                                     'unicos na locacao'))
        res = super(estoque_local_locacao, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if not ids:
            return True
        if 'locacao' in vals:
            if vals['locacao']:
                vals['locacao'] = vals['locacao'].upper()
        dados_locacao = self.read(cr, uid, ids[0], ['produto_id', 'company_id', 'local_id',
                                                    'grade_id'], context=context,
                                  load='_classic_write')
        if 'active' in vals and not vals.get('active'):
            pass
        else:
            if vals.get('produto_id'):
                produto_id = vals.get('produto_id')
            else:
                produto_id = dados_locacao.get('produto_id')
            if vals.get('company_id'):
                company_id = vals.get('company_id')
            else:
                company_id = dados_locacao.get('company_id')
            if vals.get('local_id'):
                local_id = vals.get('local_id')
            else:
                local_id = dados_locacao.get('local_id')
            if vals.get('grade_id'):
                grade_id = vals.get('grade_id')
            else:
                grade_id = dados_locacao.get('grade_id')
            locacao_id = self.search(cr, uid, [('company_id', '=', company_id),
                                               ('local_id', '=', local_id),
                                               ('produto_id', '=', produto_id),
                                               ('grade_id', '=', grade_id),
                                               ('active', '=', True)], context=context
                                     )
            if locacao_id:
                if locacao_id[0] != ids[0]:
                    raise osv.except_osv(_('Atencao'), _('Empresa, local, produto e grade precisam '
                                                         'ser unicos na locacao'))
        result = super(estoque_local_locacao, self).write(cr, uid, ids, vals, context)
        return result

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        ini = time.time()
        context = context and context.copy() or {}

        if args is None:
            args = []
        args.append(('locacao', operator, name))
        ids = self.search(cr, user, args, limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return {}
        browse_local_locacao = self.browse(cr, uid, ids, context=context)
        res = []
        for record in browse_local_locacao:
            name = '%03d - %s - %s' % (
                record.local_id.numero, record.grade_id.descricao or '', record.produto_id.name
                or '')
            res.append((record.id, name))
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'), _('Nao e permitido excluir registros da locacao'))

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        context.update({'active_test': False})
        res = super(estoque_local_locacao, self).search(cr, uid, args, offset, limit, order,
                                                        context, count)
        return res

    def estllc_onchange_active(self, cr, uid, ids, active, saldo, context=None):
        context = context and context.copy() or {}
        res = {}
        if not active and saldo:
            res.update({'value': {'active': True},
                        'warning': {'title': _('Atencao'),
                                    'message': _('Nao e possivel inativar locacao que possua '
                                                 'saldo.')}})
        return res

    def estllc_cadastra_locacao_demais_empresa(self, cr, uid, ids, context=None):
        """
        O metodo realiza o cadastro de locacao para todas as empresas registradas, baseado nos
        dados das locacoes recebidas (ids).
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: conjunto de ids da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_ir_model = self.pool.get('ir.model')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_estoque_local = self.pool.get('estoque.local')
        obj_adm_empresa_centrocusto = self.pool.get('adm.empresa.centrocusto')

        # Identifica as empresas que estao correlacionando a classe estoque_local_locacao
        locacao_model_ids = obj_ir_model.search(
            cr, uid, [('model', '=', 'estoque.local.locacao')], context=context)
        empresa_objeto_ids = obj_adm_empresa_objeto.search(
            cr, uid, [('ir_model_id', 'in', locacao_model_ids)], context=context)
        adm_empresa_objeto_dados = obj_adm_empresa_objeto.read(
            cr, uid, empresa_objeto_ids, ['empresa_escolhida_id'], context, '_classic_write')
        nao_cadastra_locacao_company_ids = map(lambda c: c.get('empresa_escolhida_id'),
                                               adm_empresa_objeto_dados)

        # Identifica as empresas que estao correlacionando a classe estoque_local
        local_model_ids = obj_ir_model.search(
            cr, uid, [('model', '=', 'estoque.local')], context=context)
        empresa_objeto_ids = obj_adm_empresa_objeto.search(
            cr, uid, [('ir_model_id', 'in', local_model_ids)], context=context)
        adm_empresa_objeto_dados = obj_adm_empresa_objeto.read(
            cr, uid, empresa_objeto_ids, ['empresa_escolhida_id'], context, '_classic_write')
        nao_cadastra_local_company_ids = map(lambda c: c.get('empresa_escolhida_id'),
                                             adm_empresa_objeto_dados)

        # Identifica as empresas que estao correlacionando a classe adm_empresa_centrocusto
        centrocusto_model_ids = obj_ir_model.search(
            cr, uid, [('model', '=', 'adm.empresa.centrocusto')], context=context)
        empresa_objeto_ids = obj_adm_empresa_objeto.search(
            cr, uid, [('ir_model_id', 'in', centrocusto_model_ids)], context=context)
        adm_empresa_objeto_dados = obj_adm_empresa_objeto.read(
            cr, uid, empresa_objeto_ids, ['empresa_escolhida_id'], context, '_classic_write')
        nao_cadastra_centrocusto_company_ids = map(lambda c: c.get('empresa_escolhida_id'),
                                                   adm_empresa_objeto_dados)

        context.update({'sig_force_company': True})
        company_ids = self.pool.get('res.company').search(cr, uid, [], context=context)
        company_ids.sort()
        fields_list = ['company_id', 'local_id', 'produto_id', 'grade_id',
                       'estllc_produto_embalagem_id']
        for locacao in self.read(cr, uid, ids, fields_list, context, '_classic_write'):
            for company_id in company_ids:
                if (company_id == locacao.get('company_id')) or \
                        (company_id in nao_cadastra_locacao_company_ids):
                    continue
                # Identifica se o local_id e correlacionado a outra empresa. Se nao for e ele nao
                # estiver cadastrado para a empresa, e criado
                local_id = locacao.get('local_id')
                if company_id not in nao_cadastra_local_company_ids:
                    fields_list = ['name', 'excluir_consultas', 'aceita_saldo_negativo', 'active',
                                   'numero', 'centrocusto_id']
                    estoque_local_dados = obj_estoque_local.read(cr, uid, local_id, fields_list,
                                                                 context, '_classic_write')
                    local_ids = obj_estoque_local.search(
                        cr, uid, [('numero', '=', estoque_local_dados.get('numero')),
                                  ('company_id', '=', company_id)], context=context)
                    if local_ids:
                        local_id = local_ids[0]
                    else:
                        # Identifica se o centrocusto_id e correlacionado a outra empresa.
                        # Se nao for e ele nao estiver cadastrado para a empresa, e criado
                        local_centrocusto_id = estoque_local_dados.get('centrocusto_id')
                        if company_id not in nao_cadastra_centrocusto_company_ids:
                            fields_list = ['name', 'sigla', 'active']
                            centrocusto_dados = obj_adm_empresa_centrocusto.read(
                                cr, uid, local_centrocusto_id, fields_list, context,
                                '_classic_write')
                            centrocusto_ids = obj_adm_empresa_centrocusto.search(
                                cr, uid, [('sigla', '=', centrocusto_dados.get('sigla')),
                                          ('company_id', '=', company_id)], context=context)
                            if centrocusto_ids:
                                local_centrocusto_id = centrocusto_ids[0]
                            else:
                                vals_centrocusto = {
                                    'company_id': company_id,
                                    'name': centrocusto_dados.get('name'),
                                    'active': centrocusto_dados.get('active'),
                                    'sigla': centrocusto_dados.get('sigla')
                                }
                                local_centrocusto_id = obj_adm_empresa_centrocusto.create(
                                    cr, uid, vals_centrocusto, context)

                        vals_local = {
                            'company_id': company_id,
                            'name': estoque_local_dados.get('name'),
                            'excluir_consultas': estoque_local_dados.get('excluir_consultas'),
                            'aceita_saldo_negativo': estoque_local_dados.get(
                                'aceita_saldo_negativo'),
                            'active': estoque_local_dados.get('active'),
                            'numero': estoque_local_dados.get('numero'),
                            'centrocusto_id': local_centrocusto_id
                        }
                        local_id = obj_estoque_local.create(cr, uid, vals_local, context)
                if self.search(
                        cr, uid,
                        [('company_id', '=', company_id),
                         ('local_id', '=', local_id),
                         ('produto_id', '=', locacao.get('produto_id')),
                         ('grade_id', '=', locacao.get('grade_id')),
                         ('estllc_produto_embalagem_id', '=', locacao.get(
                             'estllc_produto_embalagem_id'))], context=context):
                    continue
                vals = {'company_id': company_id,
                        'local_id': local_id,
                        'produto_id': locacao.get('produto_id'),
                        'grade_id': locacao.get('grade_id'),
                        'estllc_produto_embalagem_id': locacao.get('estllc_produto_embalagem_id')}
                self.create(cr, uid, vals, context)
        return True

    def estllc_button_cadastra_locacao_empresa(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Cadastrar locacao outras empresas'
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: conjunto de ids da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        self.estllc_cadastra_locacao_demais_empresa(cr, uid, ids, context)
        return True
