# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import time
from openerp.osv import osv
from openerp.tools.translate import _

READ_LOAD = '_classic_write'


class estoque_movimento_capa(osv.Model):
    """
    Tabela de movimento do estoque(Capa)
    """
    _name = 'estoque.movimento.capa'
    _inherit = 'estoque.movimento.capa'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}

        ctx_para_entrada_producao = context.get('entrada_producao')
        ctx_para_perda_producao = context.get('ctx_pcp_perda_producao')

        # LANCAMENTOS DE ENTRADA DE PRODUCAO OU PERDA DE PRODUCAO
        if ctx_para_entrada_producao or ctx_para_perda_producao:

            if ctx_para_entrada_producao:
                field_operacao_id = 'estpar_entradaproducao_operacao_id'
                msg_erro = 'Entrada de Producao'
            else:
                field_operacao_id = 'estpar_operacao_pcp_perda_id'
                msg_erro = 'Perda de Producao'

            estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, tabela='estoque', campos=[field_operacao_id], context=context
            )
            if not estoque_parametro_dados:
                raise osv.except_osv('Atencao', 'Parametro do Estoque nao cadastrado!')

            operacao_id = estoque_parametro_dados.get(field_operacao_id) or False
            if not operacao_id:
                error = 'Operacao p/ %s nao cadastrada no Parametro do Estoque!' % msg_erro
                raise osv.except_osv('Atencao', error)

            vals['estmvc_operacao_id'] = operacao_id
        if not vals.get('funcionario_id'):
            pessoa_id = self.pool.get('res.users').read(cr, uid, uid, ['pessoa_id'], context,
                                                        '_classic_write').get('pessoa_id')
            rh_funcionario_id = self.pool.get('rh.funcionario').search(cr, uid, [('pessoa', '=', pessoa_id)],
                                                                       context=context)
            if rh_funcionario_id:
                vals['funcionario_id'] = rh_funcionario_id[0]
            else:
                raise osv.except_osv(_('Atencao'), _('O usuario nao esta cadastrado como funcionario'))
        res = super(estoque_movimento_capa, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, user, ids, vals, context=None):
        return super(estoque_movimento_capa, self).write(cr, user, ids, vals, context)

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('entrada_producao'):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel excluir uma entrada de producao!'))
        res = super(estoque_movimento_capa, self).unlink(cr, uid, ids, context)
        return res

    def estmvc_button_esboco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        # atualiza saldo dos itens, decrescendo o que foi acrescentado.
        movimento_itens_ids = self.browse(cr, uid, ids[0]).movimento_itens_ids
        if movimento_itens_ids:
            local_grade_qtd = {}
            for i in movimento_itens_ids:
                if i.local_id.id in local_grade_qtd:
                    if i.grade_id.id in local_grade_qtd[i.local_id.id].keys():
                        local_grade_qtd[i.local_id.id][i.grade_id.id] += i.quantidade
                    else:
                        local_grade_qtd[i.local_id.id][i.grade_id.id] = i.quantidade
                else:
                    local_grade_qtd[i.local_id.id] = {}
                    if i.grade_id.id in local_grade_qtd[i.local_id.id].keys():
                        local_grade_qtd[i.local_id.id][i.grade_id.id] += i.quantidade
                    else:
                        local_grade_qtd[i.local_id.id][i.grade_id.id] = i.quantidade
            obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
            for local in local_grade_qtd:
                obj_estoque_produto_saldo.atualizar_saldo_do_dia(
                    cr, uid, company_id, local_grade_qtd.get(local), time.strftime("%Y-%m-%d"), 'S',
                    local, context=context)
        res = self.write(cr, uid, ids, {'state': 'E'}, context)
        return res

    def estmvc_button_efetivado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        # atualiza saldo dos itens, acrescentando.
        today = time.strftime("%Y-%m-%d")
        load = '_classic_write'
        dados_capa = self.read(cr, uid, ids[0], ['estmvc_operacao_id', 'movimento_itens_ids'],
                                    context, load)
        campos = ['estnto_tipo_operacao', 'estnto_atualizar_saldo']
        dados_operacao = self.pool.get('estoque.natureza.operacao').read(cr, uid,
                                              dados_capa.get('estmvc_operacao_id'), campos, context)
        item_ids = dados_capa.get('movimento_itens_ids')
        if not item_ids:
            raise osv.except_osv(_('Atencao'), _('Nao ha itens cadastrados'))

        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        dict_produto_embalagem = {}
        obj_saldo = self.pool.get('estoque.produto.saldo')
        campos = ['local_id', 'produto_id', 'grade_id', 'quantidade', 'produto_embalagem_id']
        dados_itens = self.pool.get('estoque.movimento.itens').read(cr, uid, item_ids, campos,
                                                                    context, load)
        prod_emb_ids = []
        produto_ids = []
        grade_ids = []
        local_ids = []
        for i in dados_itens:
            prod_emb_ids.append(i.get('produto_embalagem_id'))
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))
            local_ids.append(i.get('local_id'))

        args = [('produto_id', 'in', produto_ids),
                ('grade_id', 'in', grade_ids),
                ('estllc_produto_embalagem_id', 'in', prod_emb_ids),
                ('local_id', 'in', local_ids)]
        locacao_ids = self.pool.get('estoque.local.locacao').search(cr, uid, args, context=context)
        dict_locacao = {}
        campos = ['produto_id', 'grade_id', 'estllc_produto_embalagem_id', 'local_id']
        for i in self.pool.get('estoque.local.locacao').read(cr, uid, locacao_ids, campos, context,
                                                             load):
            chave = '%s-%s-%s' % (i.get('produto_id'),
                                  i.get('grade_id'),
                                  i.get('estllc_produto_embalagem_id'))
            if not dict_locacao.get(chave):
                dict_locacao[chave] = [i.get('local_id')]
            else:
                dict_locacao[chave].append(i.get('local_id'))

        dict_prod_emb = {}
        embalagem_ids = []
        for i in obj_produto_embalagem.read(cr, uid, prod_emb_ids, ['embalagem_id'], context, load):
            dict_prod_emb[i.get('id')] = i
            embalagem_ids.append(i.get('embalagem_id'))

        dict_embalagem = {}
        map(lambda x: dict_embalagem.update({x.get('id'): x}),
            self.pool.get('estoque.embalagem').read(cr, uid, embalagem_ids,
                                                    ['quantidade_na_embalagem'], context))

        for item in dados_itens:
            local_id = item.get('local_id')
            produto_id = item.get('produto_id')
            grade_id = item.get('grade_id')
            prod_emb_id = item.get('produto_embalagem_id')
            chave = '%s-%s-%s' % (produto_id, grade_id, prod_emb_id)
            if local_id not in (dict_locacao.get(chave) or []):
                nome_produto = self.pool.get('estoque.produto.grade').read(cr, uid, grade_id,
                                                                      ['name'], context).get('name')
                dados_local = self.pool.get('estoque.local').read(cr, uid, local_id,
                                                                  ['name', 'numero'], context)
                nome_local = dados_local.get('name')
                numero_local = dados_local.get('numero')
                raise osv.except_osv(_('Atencao'),
                                     _('O produto:\n"%s"\nnao esta cadastrado no local:\n"%s-%s"') %
                                     (nome_produto, numero_local, nome_local))

            quantidade = item.get('quantidade')
            if quantidade == 0:
                raise osv.except_osv(_('Atencao'), _('Existem produtos sem quantidade.'))

            prod_emb = dict_prod_emb.get(prod_emb_id)
            embalagem = dict_embalagem.get(prod_emb.get('embalagem_id'))

            embalagem_quantidade = embalagem.get('quantidade_na_embalagem') or 1
            quantidade *= embalagem_quantidade

            if produto_id in dict_produto_embalagem:
                dict_produto_embalagem[produto_id].append(embalagem.get('id'))
            else:
                dict_produto_embalagem[produto_id] = [embalagem.get('id')]

            if dados_operacao.get('estnto_atualizar_saldo'):
                obj_saldo.atualizar_saldo_do_dia(cr, uid,
                    eps_company_id=company_id,
                    grade_qtd={grade_id: quantidade},
                    eps_data=today,
                    eps_entrada_saida=dados_operacao.get('estnto_tipo_operacao'),
                    eps_local_id=local_id,
                    context=context)

        # ATUALIZA AS EMBALAGENS UTILIZADAS PELO PRODUTO, CASO O USUARIO ESCOLHA UMA QUE NAO ESTEJA
        # CADASTRADA.
        for key_produto_id in dict_produto_embalagem:
            args = [('produto_id', '=', key_produto_id)]
            embalagem_ids = dict_produto_embalagem[key_produto_id]
            for value_embalagem_id in embalagem_ids:
                args2 = [('embalagem_id', '=', value_embalagem_id)]

                produto_embalagem_ids = obj_produto_embalagem.search(cr, uid, args + args2,
                                                                     context=context)
                if not produto_embalagem_ids:
                    vals = {
                        'produto_id': key_produto_id,
                        'embalagem_id': value_embalagem_id,
                        'active': True,
                    }
                    obj_produto_embalagem.create(cr, uid, vals, context)

        res = self.write(cr, uid, ids, {'state': 'F'}, context)
        return res

    def estmvc_button_cancelado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        state = self.browse(cr, uid, ids[0]).state
        if state == 'F':
            # atualiza saldo dos itens, decrescendo o que foi acrescentado, caso esteja
            # efetivada ('F').
            movimento_itens_ids = self.read(
                cr, uid, ids[0], ['movimento_itens_ids'], context=context,
                load='_classic_write').get('movimento_itens_ids')
            if movimento_itens_ids:
                dados_itens = self.pool.get('estoque.movimento.itens').read(
                    cr, uid, movimento_itens_ids, ['grade_id', 'local_id', 'quantidade'],
                    context=context, load='_classic_write')
                for item in dados_itens:
                    obj_estoque_produto_saldo.atualizar_saldo_do_dia(
                        cr, uid, company_id, {item.get('grade_id'): item.get('quantidade')},
                        time.strftime("%Y-%m-%d"), 'S', item.get('local_id'), context=context)
        res = self.write(cr, uid, ids, {'state': 'C'}, context)
        return res
