# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from datetime import datetime, timedelta
import time
from openerp.tools.translate import _


class estoque_preco_custo(osv.Model):
    _name = 'estoque.preco.custo'
    _inherit = 'estoque.preco.custo'

    def get_custos_do_dia(self, cr, uid, epc_produto_ids, epc_company_id=None, epc_data=None,
                          context=None):
        """
        Metodo para recuperar os custos de um ou varios itens em um dia
        """
        context = context and context.copy() or {}
        obj_estoque_utils = self.pool.get('estoque.utils')
        if not epc_company_id:
            epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                            context=context)
        else:
            epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, self._name, context=context, user_company_id=epc_company_id)
        if not isinstance(epc_company_id, list):
            epc_company_id = [epc_company_id]
        if not epc_data:
            epc_data = time.strftime('%Y-%m-%d %H:%M:%S')
        if len(epc_data) <= 10:
            epc_data += ' 23:59:59'

        if not isinstance(epc_produto_ids, list):
            epc_produto_ids = [epc_produto_ids]

        res = {}
        for produto_id in epc_produto_ids:
            res[produto_id] = {'estprc_custo_corrigido':0.0,
                               'estprc_custo_medio_contabil': 0.0,
                               'estprc_ultimo_custo': 0.0,
                               'estprc_custo_reposicao': 0.0}

        if epc_produto_ids:
            if len(epc_produto_ids) == 1:
                epc_produto_ids = [int(epc_produto_ids[0])]
                epc_produto_ids = str(tuple(epc_produto_ids))
                epc_produto_ids = epc_produto_ids.replace(',', '')
            else:
                epc_produto_ids = map(lambda i: int(i), epc_produto_ids)
                epc_produto_ids = str(tuple(epc_produto_ids))
            if len(epc_company_id) == 1:
                epc_company_id = str(tuple(epc_company_id))
                epc_company_id = epc_company_id.replace(',', '')
            else:
                epc_company_id = str(tuple(epc_company_id))
            sql = """
            -- noinspection SqlDialectInspection
            SELECT DISTINCT epc.produto_id,
                   (SELECT  pc.estprc_ultimo_custo
                    FROM    estoque_preco_custo             AS pc
                    WHERE   pc.company_id IN %(epc_company_id)s
                    AND     pc.estprc_data <= '%(epc_data)s'
                    AND     pc.produto_id = epc.produto_id
                    AND     active = TRUE
                    ORDER   BY pc.estprc_data DESC, id DESC LIMIT 1) AS estprc_ultimo_custo,
                   (SELECT  pc.estprc_custo_corrigido
                    FROM    estoque_preco_custo             AS pc
                    WHERE   pc.company_id IN %(epc_company_id)s
                    AND     pc.estprc_data <= '%(epc_data)s'
                    AND     pc.produto_id = epc.produto_id
                    AND     active = TRUE
                    ORDER   BY pc.estprc_data DESC, id DESC LIMIT 1) AS estprc_custo_corrigido,
                   (SELECT  pc.estprc_custo_medio_contabil
                    FROM    estoque_preco_custo             AS pc
                    WHERE   pc.company_id IN %(epc_company_id)s
                    AND     pc.estprc_data <= '%(epc_data)s'
                    AND     pc.produto_id = epc.produto_id
                    AND     active = TRUE
                    ORDER   BY pc.estprc_data DESC, id DESC LIMIT 1) AS estprc_custo_medio_contabil,
                   (SELECT  pc.estprc_custo_reposicao
                    FROM    estoque_preco_custo             AS pc
                    WHERE   pc.company_id IN %(epc_company_id)s
                    AND     pc.estprc_data <= '%(epc_data)s'
                    AND     pc.produto_id = epc.produto_id
                    AND     active = TRUE
                    ORDER   BY pc.estprc_data DESC, id DESC LIMIT 1) AS estprc_custo_reposicao
            FROM    estoque_preco_custo epc
            WHERE   epc.company_id IN %(epc_company_id)s
            AND     epc.estprc_data <= '%(epc_data)s'
            AND     epc.produto_id IN %(epc_produto_ids)s
            AND     epc.active = True
            GROUP   BY epc.produto_id
            """ % locals()
            cr.execute(sql)
            list_produtos = cr.dictfetchall()

            if list_produtos:
                res = obj_estoque_utils.estutl_get_custo_hoje(cr, uid, list_produtos, context)

        # o valor das chaves do dicionario devem ser strings para o coletor
        if context.get('chamada_pelo_coletor'):
            # no caso do coletor, pega sempre o custo de 1 produto.
            aux = {}
            for i in res:
                return res.get(i)

        return res

    def estprc_get_custos_do_dia_anterior(self, cr, uid, epcda_produto_ids, epcda_company_id=None,
                                          epcda_data=None, epcda_posicao=1, context=None):
        """
        Metodo para recuperar os custos de um ou varios itens em um dia anterior a data (epcda_data)
        e de acordo com a posicao (epcda_posicao) informados. Ex.: Se a posicao for 1, retornara o
        primeiro conjusto dos custos anterior a data, se posicao for 2, retornara o segundo conjunto
        dos custos anterior a data e assim, sucessivamente.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param epcda_produto_ids:  lista de id da tabela estoque_produto.
        :param epcda_company_id: id da tabela res_company, identificador da empresa.
        :param epcda_data: string de um datetime, representa a data inicio para busca dos custos.
        :param epcda_posicao: inteiro que representa a posicao de selecao dos custos.
        :param context: dicionario com valores contextuais.
        :return: lista de dicionarios. Cada chave do dicionario representa o id do produto e seu
        respectivo valor e um outro dicionario que contem a descricao dos custos e seus valores.
        """
        context = context and context.copy() or {}
        if not epcda_company_id:
            epcda_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                              context=context)
        if not isinstance(epcda_company_id, list):
            epcda_company_id = [epcda_company_id]
        if not epcda_data:
            epcda_data = time.strftime('%Y-%m-%d %H:%M:%S')

        res = {}
        if not isinstance(epcda_produto_ids, list):
            epcda_produto_ids = [str(epcda_produto_ids)]
        if not epcda_produto_ids:
            return res
        if len(epcda_produto_ids) == 1:
            epcda_produto_ids = [int(epcda_produto_ids[0])]
            epcda_produto_ids = str(tuple(epcda_produto_ids))
            epcda_produto_ids = epcda_produto_ids.replace(',', '')
        else:
            epcda_produto_ids = map(lambda i: int(i), epcda_produto_ids)
            epcda_produto_ids = str(tuple(epcda_produto_ids))
        if len(epcda_company_id) == 1:
            epcda_company_id = str(tuple(epcda_company_id))
            epcda_company_id = epcda_company_id.replace(',', '')
        else:
            epcda_company_id = str(tuple(epcda_company_id))

        sql = """
            SELECT
                produto_id,
                estprc_ultimo_custo,
                estprc_custo_corrigido,
                estprc_custo_medio_contabil,
                estprc_custo_reposicao
            FROM
                (SELECT
                    produto_id,
                    estprc_ultimo_custo,
                    estprc_custo_corrigido,
                    estprc_custo_medio_contabil,
                    estprc_custo_reposicao,
                    rank() OVER (PARTITION BY produto_id ORDER BY estprc_data DESC) AS pos
                FROM
                    estoque_preco_custo epc
                WHERE
                    epc.company_id in %s
                    AND epc.estprc_data <= '%s'
                    AND epc.produto_id in %s
                    AND epc.active IS TRUE
                ORDER BY produto_id, estprc_data desc) AS ss
            WHERE pos > %d
            ORDER BY produto_id
        """ % (epcda_company_id, epcda_data, epcda_produto_ids, epcda_posicao)
        cr.execute(sql)
        custo_dados = cr.dictfetchall()
        produto_ids = []
        for custo in custo_dados:
            produto_id = custo.get('produto_id')
            if produto_id in produto_ids:
                continue
            produto_ids.append(produto_id)
            res[produto_id] = custo
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('estprc_custo_corrigido'):
            obj_estoque_parametro = self.pool.get('estoque.parametro')
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
            estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)],
                                                                context=context)
            if estoque_parametro_id:
                sigla_moeda = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[0]).indexador_id.sigla
            else:
                sigla_moeda = 'IIP'
            custo_corrigido = vals.get('estprc_custo_corrigido') or 0.0
            obj_adm_moeda_valor = self.pool.get('adm.moeda.valor')
            ano = time.strftime('%Y')
            mes = time.strftime('%m')
            dia = time.strftime('%d')
            valor_indexador = obj_adm_moeda_valor.achar_valor_moeda(cr, uid, sigla_moeda, ano, mes, dia, "<=", '1.00',
                                                                    context=context)
            if valor_indexador.get('mensagem'):
                raise osv.except_osv(_('Atencao'), '%s' % valor_indexador.get('mensagem'))
            if valor_indexador and valor_indexador.get('valor'):
                valor_indexador = valor_indexador.get('valor') or 1.0
                if not isinstance(valor_indexador, float):
                    valor_indexador = float(valor_indexador)
                vals['estprc_custo_corrigido'] = custo_corrigido / valor_indexador

        if vals.get('create_date'):
            vals.pop('create_date')
        res = super(estoque_preco_custo, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('estprc_custo_corrigido'):
            obj_estoque_parametro = self.pool.get('estoque.parametro')
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
            estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)],
                                                                context=context)
            if estoque_parametro_id:
                sigla_moeda = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[0]).indexador_id.sigla
            else:
                sigla_moeda = 'IIP'
            custo_corrigido = vals.get('estprc_custo_corrigido') or 0.0
            obj_adm_moeda_valor = self.pool.get('adm.moeda.valor')
            ano = time.strftime('%Y')
            mes = time.strftime('%m')
            dia = time.strftime('%d')
            valor_indexador = obj_adm_moeda_valor.achar_valor_moeda(cr, uid, sigla_moeda, ano, mes, dia, "<=", '1.00',
                                                                    context=context)
            if valor_indexador.get('mensagem'):
                raise osv.except_osv(_('Atencao'), '%s' % valor_indexador.get('mensagem'))
            if valor_indexador and valor_indexador.get('valor'):
                valor_indexador = valor_indexador.get('valor') or 1.0
                if not isinstance(valor_indexador, float):
                    valor_indexador = float(valor_indexador)
                vals['estprc_custo_corrigido'] = custo_corrigido / valor_indexador

        if vals.get('create_date'):
            vals.pop('create_date')
        res = super(estoque_preco_custo, self).write(cr, uid, ids, vals, context)
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        if not fields:
            fields = []
        if fields and not 'estprc_custo_corrigido' in fields:
            fields.append('estprc_custo_corrigido')
        res = super(estoque_preco_custo, self).read(cr, uid, ids, fields, context=context, load=load)
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)],
                                                            context=context)
        if estoque_parametro_id:
            sigla_moeda = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[0]).indexador_id.sigla
        else:
            sigla_moeda = 'IIP'
        if isinstance(res, list):
            for custo in res:
                if custo.get('estprc_custo_corrigido'):
                    custo_corrigido = custo.get('estprc_custo_corrigido') or 0.0
                    obj_adm_moeda_valor = self.pool.get('adm.moeda.valor')
                    ano = time.strftime('%Y')
                    mes = time.strftime('%m')
                    dia = time.strftime('%d')
                    valor_indexador = obj_adm_moeda_valor.achar_valor_moeda(cr, uid, sigla_moeda, ano, mes, dia, "<=",
                                                                            '1.00', context=context)
                    if valor_indexador and valor_indexador.get('valor'):
                        valor_indexador = valor_indexador.get('valor') or 1.0
                        if not isinstance(valor_indexador, float):
                            valor_indexador = float(valor_indexador)
                        custo['estprc_custo_corrigido'] = custo_corrigido * valor_indexador
        else:
            if res.get('estprc_custo_corrigido'):
                custo_corrigido = res.get('estprc_custo_corrigido') or 0.0
                obj_adm_moeda_valor = self.pool.get('adm.moeda.valor')
                ano = time.strftime('%Y')
                mes = time.strftime('%m')
                dia = time.strftime('%d')
                valor_indexador = obj_adm_moeda_valor.achar_valor_moeda(cr, uid, sigla_moeda, ano, mes, dia, "<=",
                                                                        '1.00', context=context)
                if valor_indexador and valor_indexador.get('valor'):
                    valor_indexador = valor_indexador.get('valor') or 1.0
                    if not isinstance(valor_indexador, float):
                        valor_indexador = float(valor_indexador)
                    res['estprc_custo_corrigido'] = custo_corrigido * valor_indexador
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('intervalo_data'):
            data_inicial = args[0][2]
            data_final = datetime.strftime(datetime.strptime(data_inicial, "%Y-%m-%d %H:%M:%S") + timedelta(days=1),
                                           "%Y-%m-%d %H:%M:%S")
            args.append(('estprc_data', '<=', data_final))
        if context.get('cadastro_produto'):
            limit = 4

        if order:
            order += ',id desc'
        else:
            order = 'id desc'

        res = super(estoque_preco_custo, self).search(cr, uid, args, offset, limit, order, context, count)
        return res
