# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.tools import config as openerp_config
from ...fiscal.control.utils import log_debug
from datetime import datetime, timedelta
from openerp.tools.translate import _
from openerp.osv import osv
import logging
import time


class estoque_preco_venda(osv.Model):
    _name = 'estoque.preco.venda'
    _inherit = 'estoque.preco.venda'
    _logger = logging.getLogger(_name)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    def get_preco_venda_automacao(self, cr, uid, dict_preco, epc_company_id=None,
                        epc_cliente_id=None, epc_grade_ids=[], context=None):
        '''
            implementado por heranca no modulo 'automacao'
        '''
        pass

    def get_preco_venda_estoquecorrecao(self, cr, uid, dict_preco, epc_company_id=None,
                        epc_cliente_id=None, epc_grade_ids=[], context=None):
        '''
            implementado por heranca no modulo 'estoquecorrecao'
        '''
        pass

    def get_preco_venda(self, cr, uid, epc_produto_ids, epc_company_id=None, epc_cliente_id=None,
                        epc_grade_ids=[], pv_so_do_cadastro=False ,context=None):
        """
        Metodo para recuperar o preco de venda do item
        @return: Dicionario tendo epc_produto_ids como chaves. Para cada id em epc_produto_ids e
        uma chave (um numero inteiro) no dicionario que retorna. Cada uma dessas chaves possui um
        novo dicionario com a chave 'preco_venda' e o respectivo valor. Isso para que caso o modulo
        automacao esteja instalado e a grade possua um valor promocional, o valor do preco de venda
        sera identificado pela grade, caso a mesma tenha sido informada.
        OBS: se houver a o modulo de automacao, valera o preco da automacao se houver registro. Se houver o modulo
        estoquecorrecao verificar se tem correcao do preco de venda, se houver, vale a correcao.
        :param pv_so_do_cadastro: quando este parametro for True, retorna o preco do cadastro do produto
        """
        context = context and context.copy() or {}
        assert type(epc_produto_ids) not in [type(int), type(long),
                                             type(tuple)], "epc_produto_ids tipo invalido: %s" % type(epc_produto_ids)

        if not epc_company_id:
            epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        else:
            epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                            args=epc_company_id, context=context)
        if isinstance(epc_produto_ids, (int, long)):
            epc_produto_ids = [epc_produto_ids]
        elif isinstance(epc_produto_ids, tuple):
            epc_produto_ids = list(epc_produto_ids)
        if epc_grade_ids and not isinstance(epc_grade_ids, list):
            epc_grade_ids = [epc_grade_ids]

        # Verificando se modulo 'kit' esta instalado.
        # Se tiver instalado e o produto for kit e seu preco for a soma dos itens.
        dict_preco_kit = {}
        modulo_kit_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'kit', context)
        if modulo_kit_instalado:
            #and context.get('preco_kit'):
            obj_kit_produto = self.pool.get('kit.produto')
            context_prod = context.copy()
            context_prod['super_estoque_produto_search'] = True
            list_produtos_kit = self.pool.get('estoque.produto').search(cr, uid,
                                                    [('id', 'in', epc_produto_ids),
                                                     ('preco_kit','=','S')], context=context_prod)
            for x in list_produtos_kit:
                preco_venda_kit = obj_kit_produto.kitpro_get_preco_venda(
                    cr, uid, x, context=context)
                dict_preco_kit[x] = preco_venda_kit

        # Nao retirar o fromkeys, pois qualquer id que for passado em epc_produto_ids gerara uma chave.
        # No modulo de automacao essa chave e usada para verificar o produto, assim como os outros modulos que utilizam esta funcao.
        dict_preco = {}.fromkeys(epc_produto_ids, 0.0)
        if epc_produto_ids and isinstance(epc_produto_ids, list):
            # O ID RETORNADO DA TABELA DE PRODUTOS TRAZ UM LONG DENTRO DE UMA LISTA.
            # CASO A LISTA SEJA CONVERTIDA PARA STRING, O CARACTERE L PERMANECE JUNTO AO NUMERO.
            # NAS DUAS LINHAS ABAIXO, CADA ID E TRANSFORMADO EM UMA STRING (QUE RETIRA O L).
            epc_produto_ids = map(str, epc_produto_ids)
            epc_produto_ids = str(epc_produto_ids)
            epc_produto_ids = epc_produto_ids.replace('[', '(')
            epc_produto_ids = epc_produto_ids.replace(']', ')')
            sql = """
                SELECT produto_id, preco_venda
                FROM estoque_preco_venda preco
                WHERE produto_id IN %s
                  AND company_id = %s
                  AND id IN (SELECT preco2.id
                             FROM estoque_preco_venda preco2
                             WHERE preco2.produto_id = preco.produto_id
                               AND preco2.company_id = preco.company_id
                             ORDER BY preco2.create_date DESC, preco2.id DESC
                             LIMIT 1)
                  """ % (epc_produto_ids, epc_company_id)
            cr.execute(sql)
            registro = cr.fetchall()
            if registro:
                dict_preco.update(dict(registro))
        if dict_preco_kit:
            dict_preco.update(dict_preco_kit)

        # usado pelo kit para calcular o preco total do itens do kit
        if context.get('preco_apenas_da_classe_preco_venda'):
            for x in dict_preco:
                if not isinstance(dict_preco.get(x), dict):
                    dict_preco[x] = {'preco_venda': dict_preco.get(x)}
            return dict_preco

        preco_automacao = {}
        if not pv_so_do_cadastro:
            preco_automacao = self.get_preco_venda_automacao(cr, uid, dict_preco, epc_company_id,
                            epc_cliente_id, epc_grade_ids, context)

        # se houver automacao e preco na tabela, vale este preco
        if preco_automacao:
            dict_preco = preco_automacao

        for x in dict_preco:
            if not isinstance(dict_preco.get(x), dict):
                dict_preco[x] = {'preco_venda': dict_preco.get(x)}

        # nao executa quando estiver atualizando os precos em automacao->atualizar precos
        if not context.get('automacao_atualizar_precos') and not pv_so_do_cadastro:
            if preco_automacao:
                preco_correcao = self.get_preco_venda_estoquecorrecao(cr, uid, preco_automacao, epc_company_id,
                            epc_cliente_id, epc_grade_ids, context)
            else:
                preco_correcao = self.get_preco_venda_estoquecorrecao(cr, uid, dict_preco, epc_company_id,
                            epc_cliente_id, epc_grade_ids, context)


            # se houver estoquecorrecao e correcao na tabela, vale este preco corrigido
            if preco_correcao:
                dict_preco = preco_correcao

        for x in dict_preco:
            if not isinstance(dict_preco.get(x), dict):
                dict_preco[x] = {'preco_venda': dict_preco.get(x)}
        return dict_preco

    @log_debug
    def estpve_verificar_custo_maior_pvenda(self, cr, uid, preco_venda, produto_id, tributacao_ids=[], context=None):
        context = context and context.copy() or {}
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_preco_custo = self.pool.get('estoque.preco.custo')

        # preco_venda e produto_id devem ser obrigatorios
        if produto_id and preco_venda:
            valor_encargos = 0.0
            valor_custo = 0.0
            #tributacao_ids e uma lista das tributacoes incidentes no item e e opcional, se nao for informado,
            #devera usar a tributacao que esta cadastrada para o produto.
            #calculando os encargos do produto
            if tributacao_ids:
                fiscal_tributacao_encargos_ids = obj_fiscal_tributacao_encargos.search(cr, uid,
                                                                                       [('produto_id', '=', produto_id),
                                                                                        ('tributacao_id', 'in',
                                                                                         tributacao_ids),
                                                                                       ])
                browse_fiscal_tributacao_encargos = obj_fiscal_tributacao_encargos.browse(cr, uid,
                                                                                          fiscal_tributacao_encargos_ids)
                for x in browse_fiscal_tributacao_encargos:
                    if x.tipo_tributacao == 'T':
                        valor_base = (preco_venda * x.tributacao_id.base or 0.0) / 100
                        valor_encargos += (valor_base * x.tributacao_id.aliquota or 0.0) / 100
            else:
                tributacao_ids = obj_estoque_produto.read(cr, uid, produto_id, ['tributacao_ids'],
                                                          context).get('tributacao_ids')
                if tributacao_ids:
                    for x in self.pool.get('fiscal.tributacao.encargos').browse(cr, uid,
                                                                           tributacao_ids, context):
                        if x.tipo_tributacao == 'T':
                            valor_base = (preco_venda * x.tributacao_id.base or 0.0) / 100
                            valor_encargos += (valor_base * x.tributacao_id.aliquota or 0.0) / 100
                #calcular o custo, usando o parametro 'custo para preco de venda'
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
            estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)])
            if estoque_parametro_id:
                parametro_dados = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[0])
                custo_preco_venda = parametro_dados.estpar_custo_para_precovenda
                comissoes = parametro_dados.comissoes * preco_venda / 100
                custo_fixo = parametro_dados.custo_fixo * preco_venda / 100
                if custo_preco_venda == 'C':
                    custo_preco_venda = 'estprc_custo_corrigido'
                elif custo_preco_venda == 'U':
                    custo_preco_venda = 'estprc_ultimo_custo'
                elif custo_preco_venda == 'M':
                    custo_preco_venda = 'estprc_custo_medio_contabil'
                elif custo_preco_venda == 'R':
                    custo_preco_venda = 'estprc_custo_reposicao'

                if custo_preco_venda:
                    custo = obj_preco_custo.get_custos_do_dia(cr, uid, produto_id, company_id,
                                                              time.strftime('%Y-%m-%d'), context)
                    if custo.get(produto_id):
                        custo = custo.get(produto_id)
                    else:
                        custo = {}
                    custo = custo.get(custo_preco_venda, 0.0)
                    #qdo digo custo e custo mais encargos
                    valor_custo = custo + valor_encargos + custo_fixo + comissoes
                #o retorno devera ser True ou False
            #True, o preco de custo e maior do que o de venda
            #False o preco de custo e igual ou menor ao de venda
            if valor_custo > preco_venda:
                return True
            else:
                return False
        else:
            return False


    """
    O METODO CREATE TEM A FUNCAO DE ATUALIZAR O CAMPO preco_venda DO ESTOQUE PRODUTO,
    FAZENDO O WRITE PARA ACIONAR O CAMPO FUNCTION
    """

    def valida_preco_margem(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        preco_venda = vals.get('preco_venda')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_ir_model_access = self.pool.get('ir.model.access')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        produto_id = vals.get('produto_id')
        margens = obj_estoque_produto.estpro_get_margens(cr, uid, produto_id)
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)])
        if isinstance(estoque_parametro_id, list):
            estoque_parametro_id = estoque_parametro_id[0]
        custo_para_precovenda = ''
        pvenda_menor_pcusto = ''
        custo_fixo = 0
        if estoque_parametro_id:
            browse_estoque_parametro = obj_estoque_parametro.read(cr, uid, estoque_parametro_id,
                                                                  ['estpar_custo_para_precovenda',
                                                                   'estpar_pvenda_menor_pcusto',
                                                                   'custo_fixo',
                                                                   'comissoes'], context=context)
            custo_para_precovenda = browse_estoque_parametro.get('estpar_custo_para_precovenda')
            pvenda_menor_pcusto = browse_estoque_parametro.get('estpar_pvenda_menor_pcusto')
            custo_fixo = browse_estoque_parametro.get('custo_fixo') or 0
            comissoes = browse_estoque_parametro.get('comissoes') or 0
        res = obj_ir_model_access.check_groups(cr, uid, 'estoque.group_estoque_pv_menor_margem_minima')
        lucro = obj_estoque_produto.estpro_calcula_lucro(cr, uid, produto_id, preco_venda,
                                                         custo_para_precovenda,
                                                         encargos=False,
                                                         custo_fixo=custo_fixo,
                                                         comissoes=False, context=None)
        if lucro < margens.get(produto_id).get('margem_minima') and not res:
            raise osv.except_osv(_('Atencao'), _('Preco de venda nao pode ser menor que margem minima'))
        if lucro < 0 and not pvenda_menor_pcusto:
            raise osv.except_osv(_('Atencao'), _('Para este preco de venda, a margem de lucro esta negativa!'))

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('copy_estoque_produto'):
            # ao duplicar produto nao pode ser criado(duplicado) o preco de venda
            return 1
        if vals.get('create_date'):
            del vals['create_date']
        if not vals.get('preco_venda') or vals.get('preco_venda') < 0:
            raise osv.except_osv(_('Atencao'), _('Informe um preco de venda valido'))
        self.valida_preco_margem(cr, uid, vals, context=context)

        res = super(estoque_preco_venda, self).create(cr, uid, vals, context)
        if vals.get('produto_id'):
            # wirte somente para o campo function preco_venda ser recalculado pela funcao.
            self.pool.get('estoque.produto').write(cr, uid, vals['produto_id'], {}, context)
        return res

    def write(self, cr, uid, ids, values, context=None):
        context = context and context.copy() or {}
        preco_venda = values.get('preco_venda')
        if preco_venda:
            raise osv.except_osv(
                _('Atencao'),
                _('O preco de venda nao pode ser alterado. Para a alteracao deve-se criar um novo preco de venda.')
            )

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'), _('O preco de venda nao pode ser excluido.'))

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('intervalo_data'):
            data_inicial = args[0][2]
            data_final = datetime.strftime(
                datetime.strptime(data_inicial, "%Y-%m-%d %H:%M:%S") + timedelta(days=1), "%Y-%m-%d %H:%M:%S"
            )
            args.append(('create_date', '<=', data_final))
        if context.get('cadastro_produto') and not limit:
            limit = 4
        res = super(estoque_preco_venda, self).search(cr, uid, args, offset, limit, order, context, count)
        return res

    def etiqueta_get_preco_venda(self, cr, user, item, empresa=False, cliente=False, grade=False,
                                 cadastro=False, context=None):
        """ Metodo utilizado pelo modulo de impressao de etiqueta de gondola do Coletor. """
        context = context and context.copy() or dict()
        grade = list() if not grade else grade
        result = self.get_preco_venda(cr, user, item, empresa, cliente, grade, cadastro, context)

        # Os valores das chaves dos dicionarios sao convertidas em ´string´
        result = dict(map(lambda m: (str(m[0]), m[1]), [r for r in result.items()]))

        return result

    def onchange_estoque_preco_venda(self, cr, uid, ids, preco_venda, context=None):
        context = context and context.copy() or {}
        return {'value': {'create_date': str(datetime.now())}}


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
