# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import time
from time import strftime
from SigUtils import calcDigito
from SigUtils.formatastring import formataString
from openerp.tools.translate import _
from openerp import pooler, tools
from datetime import date
import threading
from collections import OrderedDict
import re


READ_LOAD = '_classic_write'


def synchronized_method(method):
    """
    Responsavel por sincronizar os metodos, de forma
    que entre apenas uma requisicao por vez em determindado metodo.
    E utilizado para reduzir problemas de concorrencia.

    Para utilizacao e necessario decorar o metodo que deseja sincronizar.
    Antes de utilizar avaliar se metodos que chamam o metodo a ser sincronizado nao
    fazem alteracao em registros de tabelas que o metodo sincronizado vai ler ou alterar, pois
    isso podera causar deadlock

    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    outer_lock = threading.Lock()
    lock_name = "__" + method.__name__ + "_lock" + "__"

    def sync_method(self, *args, **kwargs):
        with outer_lock:
            if not hasattr(self, lock_name) or not getattr(self, lock_name).locked():
                setattr(self, lock_name, threading.Lock())
                lock = getattr(self, lock_name)
                with lock:
                    return method(self, *args, **kwargs)
    return sync_method


class estoque_produto(osv.Model):
    """
    Tabela de Produtos/Servicos/Materia Prima/Insumos/Mercadorias

                       Quando o valor digitado e Nro. a pesquisa e feita por codigo de barras, Nro fabricante e Nro fornecedor.
                       Flags cb=99999 --> Pesquisa pelo codigo de barraaldo_do
                             fa=99999 --> Pesquisa pelo Fabricante_id
                             fo=99999 --> Pesquisa pelo Fornecedor_id
    """
    _name = 'estoque.produto'
    _inherit = 'estoque.produto'
    codigo_barra = ''

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)

        estpar_dados = self.pool.get('estoque.parametro').estpar_get_fields(
            cr, uid, fields_to_read=['estpar_tamanho_codigo'], context=context
        )
        if isinstance(estpar_dados, list):
            estpar_dados = estpar_dados[0]

        tamanho_codigo_produto = estpar_dados['estpar_tamanho_codigo']

        __code = 'estoque.produto'
        __name = 'Codigo de Produto/Servico'
        vals = {
            'name': __name,
            'code': __code,
            'company_id': company_id,
            'padding': tamanho_codigo_produto,
            'implementation': 'no_gap'
        }
        # Gerar sequence para Produto/Servico para a empresa
        consulta = self.pool.get('ir.sequence.type').search(cr, uid, [('code', '=', __code)],
                                                            context=context)
        if not consulta:
            self.pool.get('ir.sequence.type').create(cr, uid, {'name': __name, 'code': __code},
                                                     context)
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def _get_parametro_usa_digito(self, cr, uid, context=None):
        res = {}
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        estoque_parametro_id = obj_estoque_parametro.search(cr, uid,
                                                            [('company_id', '=', company_id)], context=context)
        if estoque_parametro_id:
            res = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[0]).usa_digito
        else:
            res = False
        return res

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        def _estpro_fields_view_get_compras(res, context):
            if res['type'] == 'tree' and context.get('estoque_servico'):
                res['arch'] = res['arch'].replace("""<field name="comprador_id" modifiers="{}"/>""", '')
            return res

        context = context and context.copy() or {}
        # verificacao simples nacional/ Lucro Real
        simples_nacional = self.pool.get('adm.pessoa').verificar_empresa_simples_nacional(cr, uid)
        if simples_nacional:
            context['empresa_simples_nacional'] = True
        else:
            context['empresa_normal'] = True

        res = super(estoque_produto, self).fields_view_get(cr, uid, view_id, view_type, context,
                                                           toolbar=toolbar,
                                                           submenu=submenu)
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'compras', context):
            res = _estpro_fields_view_get_compras(res, context)
        if context.get('tipo_servico', False):
            if res['type'] == 'tree' and context['tipo_servico'] == 'S':
                res['arch'] = res['arch'].replace('string="Produtos">', 'string="Servicos">')
        """
        SOLUCAO ABAIXO FOI FEITA PARA NAO PERMITIR QUE O USUARIO CADASTRE, ALTERE OU ORDENE UMA GRADE A NAO SER QUE ESTEJE NO
        CADASTRO DE PRODUTO. ISTO FOI FEITO POIS A INTERFACE WEB APRESENTA ERRO QUANDO O RECURSO DO PRODUTO E ABERTO E TENTA-SE
        MANIPULAR A GRADE.
        """
        if isinstance(context, dict):
            if not context.get('cadastro_produto'):
                res['arch'] = res['arch'].replace(
                    """<page string="Grade" attrs="{'invisible': [('tipo','=','S')]}">""",
                    """<page string="Grade" invisible="1">""")

        if self.fields_get(cr, uid, ['tipo_kit']).get('tipo_kit'):
            res['arch'] = res['arch'].replace("""<page string="Precos">""",
                                              """<page string="Precos"  attrs="{'invisible': ['|',('tipo_kit', '=', 'C'),('preco_kit','=','S')]}">""")

        if res.get('name') == 'estoque.produto.tree':
            usa_digito = self._get_parametro_usa_digito(cr, uid, context)
            if not usa_digito:
                res['arch'] = res['arch'].replace("""<field name="digito" modifiers="{}"/>""",
                                                  """<field name="digito" modifiers="{&quot;tree_invisible&quot;:true}"/>""")
        return res

    def cardLuhnChecksumIsValid(self, card_number):
        """
           checks to make sure that the card passes a luhn mod-10 checksum
        """
        sum = 0
        num_digits = len(card_number)
        oddeven = num_digits & 1

        for count in range(0, num_digits):
            digit = int(card_number[count])

            if not ((count & 1) ^ oddeven):
                digit = digit * 2
            if digit > 9:
                digit = digit - 9

            sum = sum + digit

        return ((sum % 10))

    def fields_get(self, cr, user, fields=None, context=None):
        context = context and context.copy() or {}
        res = super(estoque_produto, self).fields_get(cr, user, fields, context)
        res['par_usa_aplicacao'] = {'selectable': True, 'type': 'boolean',
                                    'string': 'par usa aplicacao'}
        res['par_usa_similar'] = {'selectable': True, 'type': 'boolean',
                                  'string': 'par usa similar'}
        res['par_usa_digito'] = {'selectable': True, 'type': 'boolean', 'string': 'par usa digito'}
        res['estpar_cadastrar_custo'] = {'selectable': True, 'type': 'boolean',
                                         'string': 'cadastrar custo'}
        return res

    def search_produto_in_locacao(self, cr, uid, local_id, context=None):
        context = context and context.copy() or {}
        res = []
        obj_local = self.pool.get('estoque.local.locacao')
        ids = obj_local.search(cr, uid, [('local_id', '=', local_id)], context=context)
        produto_ids = obj_local.read(cr, uid, ids, ['produto_id'], context=context)
        for x in produto_ids:
            if x['produto_id'][0]:
                if not x['produto_id'][0] in res:
                    res.append(x['produto_id'][0])
        return res

    def _get_parametro(self, cr, uid, company_id, context=None):
        context = context and context.copy() or {}
        usa_digito = False
        usar_aplicacao = False
        usar_similaridade = False
        cadastrar_custo = False
        codigo_numerico = False
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        par_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
        vals = False
        if par_id:
            vals = obj_estoque_parametro.browse(cr, uid, par_id[0])
            usa_digito = vals.usa_digito or False
            usar_aplicacao = vals.usar_aplicacao or False
            usar_similaridade = vals.usar_similaridade or False
            codigo_numerico = vals.codigo_numerico or False
            cadastrar_custo = vals.estpar_cadastrar_custo or False
            comercializa_combustivel_lubrificante = vals.estpar_comercializa_combustivel_lubrificante or False

        res = {
            'estpar_cadastrar_custo': cadastrar_custo,
            'usa_digito': usa_digito,
            'usar_aplicacao': usar_aplicacao,
            'usar_similaridade': usar_similaridade,
            'codigo_numerico': codigo_numerico,
            'estpar_comercializa_combustivel_lubrificante': comercializa_combustivel_lubrificante,
        }

        obj_adm_utils = pooler.get_pool(cr.dbname).get('adm.utils')
        if obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'balanca', context={}):
            usar_tara = vals.estpar_balanca_usar_tara or False
            if usar_tara:
                res['estpar_balanca_usar_tara'] = usar_tara

        return res

    # funcao codigo - descricao  do produto
    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        # O browse foi retirado por causa de performance em varios lugares do ERP, para corrigir a
        # questao da ordenacao que o read nao respeita, o 'for' eh feito nos ids que continuam
        # ordenados corretamente.
        id_dados_produto = {} # chave=id do produto, valor=read do produto
        map(lambda x: id_dados_produto.update({x.get('id'):x}),
            self.read(cr, uid, ids, ['codigo', 'name', 'digito'], context=context))
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['usa_digito'])
        usa_digito = estoque_parametro_dados.get('usa_digito')
        for id in ids:
            produto = id_dados_produto.get(id)
            codigo = str(produto.get('codigo') or '')
            name = produto.get('name') or '*** Produto sem Descricao ***'
            if context.get('retorno_sem_codigo'):
                codigo = ''

            if context.get('estpro_so_codigo'):
                result = codigo

            elif context.get('estpro_codigo_e_digito'):
                if usa_digito:
                    digito = produto.get('digito')
                    result = "%s/%s" % (codigo, digito)
                else:
                    result = codigo

            else:
                result = "%s%s%s" % (codigo, ' - ' if codigo else '', name)

            res.append((produto.get('id'), result))

        return res

    def get_preco_unitario(self, cr, uid, company_id, produto_id, grade_id,
                           data=strftime('%Y-%m-%d')):
        preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, produto_id, epc_company_id=company_id, epc_grade_ids=grade_id)
        preco_unitario = preco_venda.get(produto_id)
        return preco_unitario

    def validar_name(self, name, context=None):
        """O proposito desta funcao eh verificar se o nome do produto possui pelo menos uma letra,
          ao vender um produto, a ECF nao aceita descricao com apenas numeros"""
        context = context and context.copy() or {}
        if not name:
            return True

        if not re.sub('[^a-zA-Z]', '', name):
            return False

        return True

    def onchange_name_produto(self, cr, uid, ids, name, context=None):
        context = context and context.copy() or {}
        if not self.validar_name(name, context):
            return {'warning': {'title': _('Atencao'),
                                'message': _('O nome do produto deve conter letras')
                                }
                    }
        return {}

    def onchange_codigo(self, cr, uid, ids, codigo, context=None):
        context = context and context.copy() or {}
        if codigo:
            produto_ids = self.search(cr, uid, [('codigo', '=', codigo)], context=context)
            if produto_ids:
                return {'warning': {
                    'title': _('Atencao'),
                    'message': _('Codigo do Produto/Servico ja cadastrado para a Empresa!')
                }}
        return {}

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(estoque_produto, self).default_get(cr, uid, fields, context)
        if res.get('company_id'):
            vals = self._get_parametro(cr, uid, res['company_id'], context)
            res['estpro_visibilidade_codigo_anp'] = vals.get(
                'estpar_comercializa_combustivel_lubrificante')
            res['par_usa_aplicacao'] = vals.get('usar_aplicacao')
            res['par_usa_similar'] = vals.get('usar_similaridade')
            res['codigo_numerico'] = vals.get('codigo_numerico')
            res['par_usa_digito'] = vals.get('usa_digito')
            res['estpar_cadastrar_custo'] = vals.get('estpar_cadastrar_custo')
            if not context.get('pcp'):
                if context.get('estoque_servico'):
                    res['tipo_item'] = '09'
                    res['tipo'] = 'S'
                else:
                    res['tipo_item'] = '00'
                    res['tipo'] = 'P'
            if res.get('tipo') == 'S':
                res['tempo_previsto'] = 1.0
        if context.get('produto_kit'):
            res['kit_de_produtos'] = True
            res['estpro_cadastro_kit'] = True
        return res

    def estoque_produto_in_fornecedor(self, cr, uid, company_id, fornecedor_id, produto_id,
                                      produto_embalagem_id, context=None):
        ## verifica se a pessoa informada no pedido e um fornecedor, caso nao seja, e adicionada
        ## verifica se o produto e diponibilizado pela pessoa(fornecedor) do pedido, se nao for, faz a associacao

        context = context and context.copy() or {}
        financeiro_fornecedor_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'financeiro.fornecedor')
        obj_financeiro_parametro = self.pool.get('financeiro.parametros')
        parametro_financeiro_id = obj_financeiro_parametro. \
            search(cr, uid, [('company_id', '=', company_id)])
        if not parametro_financeiro_id:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao a parametros cadastrado no financeiro p/ essa empresa'))
        else:
            cadastro_automatico_fornecedor = obj_financeiro_parametro. \
                browse(cr, uid, parametro_financeiro_id[0]).cadastro_automatico_fornecedor
            if cadastro_automatico_fornecedor:
                obj_fornecedor = self.pool.get('financeiro.fornecedor')
                fornecedor_ids = obj_fornecedor.search(
                    cr, uid, [('pessoa_id', '=', fornecedor_id),
                              ('company_id', '=', financeiro_fornecedor_company_id)],
                    context=context)
                if not fornecedor_ids:
                    fornecedor_id = obj_fornecedor.create(cr, uid, {'pessoa_id': fornecedor_id,
                                                                    'company_id': financeiro_fornecedor_company_id}, context)
                else:
                    fornecedor_id = fornecedor_ids[0]
                if produto_embalagem_id:
                    embalagem_id = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                                                                                     produto_embalagem_id).embalagem_id.id or 0
                else:
                    embalagem_id = False
                obj_estoque_fornecedor = self.pool.get('estoque.fornecedor')
                fornecedor_produto_ids = obj_estoque_fornecedor.search(cr, uid, [
                    ('fornecedor_id', '=', fornecedor_id),
                    ('produto_id', '=', produto_id)], context=context)
                if not fornecedor_produto_ids:
                    obj_estoque_fornecedor.create(cr, uid,
                                                  {'fornecedor_id': fornecedor_id, 'produto_id': produto_id,
                                                   'embalagem_id': embalagem_id})
        return True

    def estoque_check_produto_in_local(self, cr, uid, company_id=0, produto_id=0, local_id=0, context=None):
        context = context and context.copy() or {}
        local_locacao_ids = False
        if all(map(lambda i: isinstance(i, (int, long)), [company_id, produto_id, local_id])):
            produto_tipo = ''
            produto_dados = self.read(cr, uid, produto_id, ['tipo'], context=context)
            if isinstance(produto_dados, list) and produto_dados:
                produto_dados = produto_dados[0]
            if isinstance(produto_dados, dict):
                produto_tipo = produto_dados.get('tipo')
            if produto_tipo == 'P':
                local_locacao_ids = self.pool.get('estoque.local.locacao').search(cr, uid,
                                                                                  [('produto_id',
                                                                                    '=',
                                                                                    produto_id),
                                                                                   ('local_id', '=',
                                                                                    local_id),
                                                                                   ('company_id',
                                                                                    '=',
                                                                                    company_id)], context=context)
            elif produto_tipo == 'S':
                return True
        return True if local_locacao_ids else False

    def valida_codigo_numerico(self, cr, uid, vals, context=None):
        """
        Verifica se no parametros se o codigo do produto deve ser numerico,
        Se o codigo for numerico e for deixado em branco, retorna o proximo codigo da sequencia;

        """
        if 'par_usa_aplicacao' in vals:
            del vals['par_usa_aplicacao']
        if 'par_usa_similar' in vals:
            del vals['par_usa_similar']
            # Busca o parametro da empresa
        context = context and context.copy() or {}
        obj_par = self.pool.get('estoque.parametro')
        par_id = obj_par.search(cr, uid, [], context=context)
        parametro = obj_par.read(cr, uid, par_id, ['codigo_numerico', 'usa_digito'], context=context)

        if parametro:
            # Verself._columns[f]._symbol_set[1](value)ifica se o parametro e para somente codigo numerico
            if parametro[0]['codigo_numerico']:
                codigo = 0
                if 'codigo' in vals:
                    try:
                        codigo = int(vals['codigo'])
                    except:
                        raise osv.except_osv(_('Atencao'),
                                             _('O Codigo do Produto deve ser Numerico!'))
                if (codigo == 0):
                    # verifica se o codigo esta vazio, para inserir o codigo do sequence
                    try:
                        codigo = self.pool.get('adm.utils').get_sequence(cr, uid, self._name,
                                                                         'estoque.produto')
                        vals['codigo'] = codigo
                    except:
                        codigo = 1
                if parametro[0]['usa_digito']:
                    # Tarefa 18324
                    digito = calcDigito.ean(codigo)
                    vals['digito'] = str(digito)
            else:
                if not vals.get('codigo'):
                    raise osv.except_osv(_('Atencao'), _('Preencher codigo para o produto!'))
        return vals

    def estpro_calcula_lucro(self, cr, uid, produto_id, preco_venda, custo_a_utilizar=False,
                             encargos=False, custo_fixo=0, comissoes=0, context=None):
        context = context and context.copy() or {}
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        if not encargos:
            encargos = []
        if not produto_id:
            return False
        custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id)
        if custo.get(produto_id):
            custo = custo.get(produto_id)
        else:
            custo = {}

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        estoque_parametro_id = obj_estoque_parametro.search(cr, uid,
                                                            [('company_id', '=', company_id)], context=context)
        if not estoque_parametro_id:
            return False
        browse_estoque_parametro = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[0])
        if not custo_a_utilizar:
            custo_a_utilizar = browse_estoque_parametro.estpar_custo_para_precovenda
        custo_utilizar = False
        if custo_a_utilizar == 'U':
            custo_utilizar = custo.get('estprc_ultimo_custo') or 0.0
        elif custo_a_utilizar == 'M':
            custo_utilizar = custo.get('estprc_custo_medio_contabil') or 0.0
        elif custo_a_utilizar == 'C':
            custo_utilizar = custo.get('estprc_custo_corrigido') or 0, 0
        elif custo_a_utilizar == 'R':
            custo_utilizar = custo.get('estprc_custo_reposicao') or 0.0
        elif isinstance(custo_a_utilizar, (int, float)):
            custo_utilizar = custo_a_utilizar

        if isinstance(custo_utilizar, (list, tuple)):
            custo_utilizar = custo_utilizar[0]

        if comissoes and isinstance(comissoes, (int, float)):
            soma_vr_encargos = comissoes
        else:
            porcentagem_comissao = browse_estoque_parametro.comissoes or 0.0
            soma_vr_encargos = porcentagem_comissao
            # se houver encargos no parametro, sera --> ex: [['icms',1.76],['pis',1.2]...]
        if encargos:
            if isinstance(encargos, (int, float)):
                soma_vr_encargos += encargos
            else:
                for x in encargos:
                    soma_vr_encargos += x[1]
        # Todo: levar o que esta nesta condicao para o modulo fiscal, pois quando o mesmo nao esta
        # Todo: instalado ocorre erro ao criar um preco para um produto.
        elif self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context):
            tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(cr, uid, [
                ('produto_id', '=', produto_id)], context=context)
            if tributacao_encargo_ids:
                browse_tributacao_encargo = obj_fiscal_tributacao_encargos.browse(cr, uid,
                                                                                  tributacao_encargo_ids)
                for x in browse_tributacao_encargo:
                    if x.tipo_tributacao == 'T':
                        aliquota = x.tributacao_id.aliquota or 0.0
                        base = x.tributacao_id.base or 0.0
                        encargo = (aliquota * base) / 100
                        soma_vr_encargos += encargo

        if not custo_fixo:
            custo_fixo = browse_estoque_parametro.custo_fixo or 0
        soma_vr_encargos += custo_fixo
        lucro = self.pool.get('estoque.utils').estutl_calcular_margem(preco_venda,
                                                                      soma_vr_encargos,
                                                                      custo_utilizar)
        return lucro

    def estpro_calcular_preco_venda(self, cr, uid, produto_id, embalagem_id=0, prazo=0,
                                    custo_a_utilizar=False,
                                    encargos=False, juros=0, margem_a_utilizar=False, custo_fixo=0,
                                    comissoes=0,
                                    context=None):
        context = context and context.copy() or {}
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_estoque_utils = self.pool.get('estoque.utils')
        if not encargos:
            encargos = []
        if produto_id:
            margens = self.estpro_get_margens(cr, uid, produto_id)
            margens = margens.get(produto_id)
            margem_bruta = margens.get('margem_bruta') or 0.0
            margem_ideal = margens.get('margem_ideal') or 0.0
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)

            custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id, company_id)
            if custo.get(produto_id):
                custo = custo.get(produto_id)
            else:
                custo = {}

            taxa_juros = 0.0
            if prazo and juros:
                taxa_juros = (juros * 30) / prazo

            if embalagem_id:
                qtd_embalagem = obj_estoque_produto_embalagem.browse(cr, uid,
                                                                     embalagem_id).embalagem_id.quantidade_na_embalagem or 1.0
            else:
                qtd_embalagem = 1.0
                embalagem_ids = obj_estoque_produto_embalagem.search(cr, uid, [
                    ('produto_id', '=', produto_id)], context=context)
                if embalagem_ids:
                    browse_produto_embalagem = obj_estoque_produto_embalagem.browse(cr, uid,
                                                                                    embalagem_ids)
                    qtd_embalagem = []
                    for x in browse_produto_embalagem:
                        qtd_embalagem.append(x.embalagem_id.quantidade_na_embalagem or 1.0)
                if qtd_embalagem and isinstance(qtd_embalagem, list):
                    qtd_embalagem.sort()
                    qtd_embalagem = qtd_embalagem[0]

            estoque_parametro_id = obj_estoque_parametro.search(cr, uid,
                                                                [('company_id', '=', company_id)], context=context)
            if estoque_parametro_id:
                browse_estoque_parametro = obj_estoque_parametro.browse(cr, uid,
                                                                        estoque_parametro_id[0])
                if not custo_fixo:
                    custo_fixo = browse_estoque_parametro.custo_fixo or 0.0

                margem = 0.0
                if margem_a_utilizar:
                    if margem_a_utilizar == 'B':
                        margem_para_preco_venda = 'B'
                        margem = margem_bruta
                    elif margem_a_utilizar == 'I':
                        margem_para_preco_venda = 'I'
                        margem = margem_ideal
                    elif isinstance(margem_a_utilizar, (float, int)):
                        margem_para_preco_venda = 'I'
                        margem = margem_a_utilizar
                else:
                    margem_para_preco_venda = browse_estoque_parametro.estpar_margem_para_preco_venda
                    if margem_para_preco_venda == 'B':
                        margem = margem_bruta
                    elif margem_para_preco_venda == 'I':
                        margem = margem_ideal
                if not margem_para_preco_venda:
                    return False
                if custo_a_utilizar:
                    custo_utilizar = False
                    if custo_a_utilizar == 'uc':
                        custo_utilizar = custo.get('estprc_ultimo_custo') or 0.0
                    elif custo_a_utilizar == 'mc':
                        custo_utilizar = custo.get('estprc_custo_medio_contabil') or 0.0
                    elif custo_a_utilizar == 'cc':
                        custo_utilizar = custo.get('estprc_custo_corrigido') or 0, 0
                    elif custo_a_utilizar == 'cr':
                        custo_utilizar = custo.get('estprc_custo_reposicao') or 0.0
                    elif isinstance(custo_a_utilizar, (int, float)):
                        custo_utilizar = custo_a_utilizar

                    if isinstance(custo_utilizar, (list, tuple)):
                        custo_utilizar = custo_utilizar[0]

                    if margem_para_preco_venda == 'B':
                        preco_venda = custo_utilizar * 100 / (100 - margem)
                        preco_venda *= qtd_embalagem
                    else:
                        if comissoes and isinstance(comissoes, (int, float)):
                            soma_vr_encargos = comissoes
                        else:
                            porcentagem_comissao = browse_estoque_parametro.comissoes or 0.0
                            soma_vr_encargos = porcentagem_comissao
                            # se houver encargos no parametro, sera --> ex: [['icms',1.76],['pis',1.2]...]
                        if encargos:
                            if isinstance(encargos, (int, float)):
                                soma_vr_encargos += encargos
                            else:
                                for x in encargos:
                                    soma_vr_encargos += x[1]
                        else:
                            tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(cr, uid,
                                                                                           [('produto_id', '=', produto_id)],
                                                                                           context=context)
                            if tributacao_encargo_ids:
                                browse_tributacao_encargo = obj_fiscal_tributacao_encargos.browse(
                                    cr, uid,
                                    tributacao_encargo_ids)
                                for x in browse_tributacao_encargo:
                                    if x.tipo_tributacao == 'T':
                                        aliquota = x.tributacao_id.aliquota or 0.0
                                        base = x.tributacao_id.base or 0.0
                                        encargo = (aliquota * base) / 100
                                        soma_vr_encargos += encargo

                        # (I)Ideal -> sugestao = (custo / (100 - encargos - margem-ideal )) * 100
                        if soma_vr_encargos + margem + taxa_juros + custo_fixo != 100:
                            soma_vr_encargos += taxa_juros + custo_fixo
                            preco_venda = obj_estoque_utils.estutl_calcular_preco_de_venda(
                                custo_utilizar,
                                soma_vr_encargos, margem)
                        else:
                            preco_venda = 0
                        preco_venda *= qtd_embalagem

                else:
                    if margem_para_preco_venda == 'B':
                        custo_reposicao = custo.get('estprc_custo_reposicao') or 0.0
                        # (B)Bruta -> sugestao = reposicao * 100 / (100 - margem-bruta(do cadastro de estoque))
                        preco_venda = custo_reposicao * 100 / (100 - margem)
                        preco_venda *= qtd_embalagem
                    elif margem_para_preco_venda == 'I':
                        if comissoes and isinstance(comissoes, (int, float)):
                            soma_vr_encargos = comissoes
                        else:
                            porcentagem_comissao = browse_estoque_parametro.comissoes or 0.0
                            soma_vr_encargos = porcentagem_comissao
                            # se houver encargos no parametro, sera --> ex: [['icms',1.76],['pis',1.2]...]
                        if encargos:
                            if isinstance(encargos, (int, float)):
                                soma_vr_encargos += encargos
                            else:
                                for x in encargos:
                                    soma_vr_encargos += x[1]
                        else:
                            tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(cr, uid,
                                                                                           [
                                                                                               (
                                                                                                   'produto_id',
                                                                                                   '=',
                                                                                                   produto_id)],
                                                                                           context=context)
                            if tributacao_encargo_ids:
                                browse_tributacao_encargo = obj_fiscal_tributacao_encargos.browse(
                                    cr, uid,
                                    tributacao_encargo_ids)
                                for x in browse_tributacao_encargo:
                                    if x.tipo_tributacao == 'T':
                                        aliquota = x.tributacao_id.aliquota or 0.0
                                        base = x.tributacao_id.base or 0.0
                                        encargo = (aliquota * base) / 100
                                        soma_vr_encargos += encargo

                        custo_para_precovenda = browse_estoque_parametro.estpar_custo_para_precovenda
                        if custo_para_precovenda == 'U':
                            custo_para_precovenda = 'estprc_ultimo_custo'
                        elif custo_para_precovenda == 'M':
                            custo_para_precovenda = 'estprc_custo_medio_contabil'
                        elif custo_para_precovenda == 'R':
                            custo_para_precovenda = 'estprc_custo_reposicao'
                        elif custo_para_precovenda == 'C':
                            custo_para_precovenda = 'estprc_custo_corrigido'
                        if custo_para_precovenda:
                            custo_para_precovenda = custo.get(custo_para_precovenda) or 0.0
                        else:
                            return False

                        # (I)Ideal -> sugestao = (custo / (100 - encargos - margem-ideal )) * 100
                        if soma_vr_encargos + margem + taxa_juros + custo_fixo != 100:
                            soma_vr_encargos += taxa_juros + custo_fixo
                            preco_venda = obj_estoque_utils.estutl_calcular_preco_de_venda(
                                custo_para_precovenda,
                                soma_vr_encargos, margem)
                        else:
                            preco_venda = 0
                        preco_venda *= qtd_embalagem
                    else:
                        return False
            else:
                return False
        else:
            return False
        return preco_venda

    def estpro_atualiza_produto(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {}, context)
        return res

    def estpro_get_margens(self, cr, uid, produto_id):
        res = {}
        obj_estoque_produto = self.pool.get('estoque.produto')
        if isinstance(produto_id, (int, long)):
            produto_id = [produto_id]
        browse_produto = obj_estoque_produto.browse(cr, uid, produto_id)
        for i in browse_produto:
            margens_dict = {'margem_ideal': 0.0, 'margem_minima': 0.0, 'margem_bruta': 0.0}
            margem_ideal = i.margem_ideal
            margem_minima = i.margem_minima
            margem_bruta = i.margem_bruta
            if margem_ideal or margem_minima or margem_bruta:
                margens_dict['margem_ideal'] = margem_ideal
                margens_dict['margem_minima'] = margem_minima
                margens_dict['margem_bruta'] = margem_bruta
            else:
                categoria_id = i.categoria_id
                while categoria_id:
                    margem_ideal = categoria_id.margem_ideal
                    margem_minima = categoria_id.margem_minima
                    margem_bruta = categoria_id.margem_bruta
                    if margem_ideal or margem_minima or margem_bruta:
                        margens_dict['margem_ideal'] = margem_ideal
                        margens_dict['margem_minima'] = margem_minima
                        margens_dict['margem_bruta'] = margem_bruta
                        break
                    else:
                        categoria_id = categoria_id.parent_id

            res[i.id] = margens_dict
        return res

    def create(self, cr, uid, vals, context=None):
        def _estpro_create_fiscal(vals):
            if vals.get('estpro_fci'):
                vals['estpro_fci'] = vals['estpro_fci'].upper()
            return vals

        context = context and context.copy() or {}
        if not self.validar_name(vals.get('name'), context):
            raise osv.except_osv(_('Atencao!'),
                                 _('O nome do produto deve conter letras'))
        if 'estpro_foto' in vals:
            vals['estpro_foto'] = tools.image_resize_image(vals.get('estpro_foto'), (150, 150))

        if vals.get('name'):
            estoque_ids = self.search(cr, uid, [('name', '=', vals['name'])], context=context)
            if estoque_ids:
                brw_estoque = self.browse(cr, uid, estoque_ids[0])
                raise osv.except_osv(_('Atencao'), _('Produto ja cadastrado com essa descricao: ' +
                                                     brw_estoque.codigo + ' - ' + brw_estoque.name))

        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context):
            vals = _estpro_create_fiscal(vals)
        vals = self.valida_codigo_numerico(cr, uid, vals, context)
        if context.get('estoque_servico'):  # Se for um servico, seu tipo sera '09' - (Servicos)
            vals['tipo'] = 'S'
            vals['tipo_item'] = '09'
            if context.get('oservico'):
                estoque_parametro = self.pool.get('adm.utils').admutl_get_parametros(
                    cr, uid, 'estoque', 'estpar_embalagem_servico_id', context=context)
                embalagem_id = estoque_parametro.get('estpar_embalagem_servico_id')
                if not embalagem_id:
                    raise osv.except_osv(_('Atencao'), _('Nao existe embalagem para servicos '
                                                         'cadastrada no parametro do estoque'))
                else:
                    vals['produto_embalagem_ids'] = [(0, 0, {'embalagem_id': embalagem_id})]
            if 'tempo_previsto' in vals and vals.get('tempo_previsto') <= 0:
                raise osv.except_osv(_('Atencao'), _('Quantidade invalida para "Tempo previsto"'))
        else:
            vals['tipo'] = 'P'
            # Grava uma grade com descricao vazia

        if 'preco_venda_ids' in vals and context.get('cadastro_produto'):
            vals['preco_venda_ids'] = sorted(vals.get('preco_venda_ids'),
                                             key=lambda p: p[2].get('create_date') if p[2] else '')
            vals['preco_venda_ids'] = filter(
                lambda i: i[2].get('preco_venda') if i[2] else not i[2],
                vals.get('preco_venda_ids'))

        res = super(estoque_produto, self).create(cr, uid, vals, context)
        if not vals.get('grade_usar', False):
            self.pool.get('estoque.produto.grade').create(cr, uid,
                                                          {'produto_id': res, 'grade': 'UNICA',
                                                           'descricao': 'UNICA'}, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        def _estpro_write_fiscal(vals):
            if vals.get('estpro_fci'):
                vals['estpro_fci'] = vals['estpro_fci'].upper()
            return vals

        if 'estpro_foto' in vals:
            vals['estpro_foto'] = tools.image_resize_image(vals.get('estpro_foto'), (150, 150))

        context = context and context.copy() or {}
        if not self.validar_name(vals.get('name'), context):
            raise osv.except_osv(_('Atencao!'),
                                 _('O nome do produto deve conter letras'))

        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context):
            vals = _estpro_write_fiscal(vals)
            # VERIFICACÂO DO DIGITO CONFORM PARAMETROS
            # removendo os campos field_view_get
        if 'tributacao_ids' in vals:
            if vals['tributacao_ids']:
                for i in range(0, len(vals['tributacao_ids'])):
                    if vals['tributacao_ids'][i][2]:  # Verifica se houve alguma mudanca
                        if 'encargo_cidade' in vals['tributacao_ids'][i][2]:
                            del vals['tributacao_ids'][i][2]['encargo_cidade']
                        if 'encargo_uf' in vals['tributacao_ids'][i][2]:
                            del vals['tributacao_ids'][i][2]['encargo_uf']
                        if 'tipo_cst' in vals['tributacao_ids'][i][2]:
                            del vals['tributacao_ids'][i][2]['tipo_cst']
        if vals.get('name'):
            estoque_ids = self.search(cr, uid, [('name', '=', vals['name'])], context=context)
            if estoque_ids:
                brw_estoque = self.browse(cr, uid, estoque_ids[0])
                raise osv.except_osv(_('Atencao'), _('Produto ja cadastrado com essa descricao: ' +
                                                     brw_estoque.codigo + ' - ' + brw_estoque.name))
            self.pool.get('estoque.produto.descricao.anterior').esteda_verifica_inclusao(
                cr, uid, ids, vals.get('name'), context)

        o_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        if 'active' in vals:
            # Quando desativar uma produto desativar tambem as grades deste produto.
            # Quando for ativa-lo, se for grade unica, ativa-la tambem.
            id = ids
            if isinstance(ids, list):
                id = ids[0]

            ativo = vals.get('active')
            if ativo:
                self.estpro_check_produto_active_name(cr, uid, ids, context)
                args = [('produto_id', '=', id), ('active', '=', False)]
            else:
                produto_ativo = self.browse(cr, uid, [id])[0].active
                # Caso o produto ja esteja inativo, retira o {active: False} do vals
                if not produto_ativo:
                    self.estpro_check_produto_active_name(cr, uid, ids, context)
                _grades_ids = o_estoque_produto_grade.search(cr, uid, [('produto_id', '=', id)], context=context)
                dict_saldo = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                    cr, uid, _grades_ids, time.strftime('%Y-%m-%d'), context=context)
                saldo = 0.0
                for i in dict_saldo.keys():
                    saldo += dict_saldo.get(i).get('saldo', 0.0)
                args = [('produto_id', '=', id)]
                if saldo > 0:
                    raise osv.except_osv(_('Saldo: ' + str(saldo)),
                                         _('Produto/Servico com saldo positivo.'
                                           '\nNao pode ser inativado!'))
                # Inativar tambem as locacoes
                for brw_estoque in self.browse(cr, uid, ids, context=context):
                    locacao_ids = brw_estoque.locacao_ids \
                        and map(lambda loc: loc.id, brw_estoque.locacao_ids) or []
                    self.pool.get('estoque.local.locacao').write(
                        cr, uid, locacao_ids, {'active': False}, context=context)


            produto_grade_ids = o_estoque_produto_grade.search(cr, uid, args, context=context)
            if (len(produto_grade_ids) == 1 and ativo) or not ativo:
                for grade_id in produto_grade_ids:
                    o_estoque_produto_grade.write(cr, uid, grade_id,
                                                  {'active': ativo}, context=context)
        if 'tempo_previsto' in vals and vals.get('tempo_previsto') <= 0:
            raise osv.except_osv(_('Atencao'), _('Quantidade invalida para "Tempo previsto"'))

        if 'preco_venda_ids' in vals and context.get('cadastro_produto'):
            vals['preco_venda_ids'] = sorted(vals.get('preco_venda_ids'),
                                             key=lambda p: p[2].get('create_date') if p[2] else '')
            vals['preco_venda_ids'] = filter(
                lambda i: i[2].get('preco_venda') if i[2] else not i[2],
                vals.get('preco_venda_ids'))

        result = super(estoque_produto, self).write(cr, uid, ids, vals, context)

        # se houver alteracao no nome do produto, deve ser feito um write vazio nas grades do mesmo para recalcular o
        # campo 'name' da grade que eh um funcional com stored=True
        if 'name' in vals:
            produto_ids = ids if isinstance(ids, list) else [ids]
            for id in produto_ids:
                grade_ids = o_estoque_produto_grade.search(cr, uid, [('produto_id', '=', id)], context=context)
                o_estoque_produto_grade.write(cr, uid, grade_ids, {}, context=context)
        return result

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        if not fields:
            fields = []
        else:
            if not 'codigo' in fields:
                fields.append('codigo')

        if not ids:
            return []

        try:
            res = super(estoque_produto, self).read(cr, uid, ids, fields=fields, context=context,
                                                load=load)
        except Exception as e:
            if context.get('coletor'):
                msg = e.name + '! ' + 'Classe: ' + e.value.split('Modelo de documento: ')[1][:-1]
                return {'erro': True, 'msg': msg}
            raise e

        if isinstance(res, list) and isinstance(fields, list):
            if 'company_id' in fields:
                for i in res:
                    if i.get('company_id'):
                        try:
                            company_id = i['company_id'][0]
                        except:
                            company_id = []
                        if company_id:
                            vals_par = self._get_parametro(cr, uid, company_id, context)
                            i['par_usa_aplicacao'] = vals_par.get('usar_aplicacao') and context.get(
                                'view_name') == 'estoque_produto'
                            i['par_usa_similar'] = vals_par.get(
                                'usar_similaridade') and context.get(
                                'view_name') == 'estoque_produto'
                            i['codigo_numerico'] = vals_par.get('codigo_numerico')
                            i['par_usa_digito'] = vals_par.get('usa_digito')
                            i['estpar_cadastrar_custo'] = vals_par.get('estpar_cadastrar_custo')
                            i['estpro_peso_tara_flag'] = vals_par.get('estpar_balanca_usar_tara')
                    break  # BREAK ESTA SENDO UTILIZADO PARA BUSCAR SOMENTE UM REGISTRO POIS E MOSTRADO SOMENTE NO FORM VIEW (QUE EXIBE SOMENTE UM REGISTRO POR VEZ)
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):

        context = context and context.copy() or {}
        # este contexto deve ser removido quando for refatorar o metodo search
        # ele impede que a aplicacao entre em loop ao entrar na tela de produtos (Felipe)
        if context.get('super_estoque_produto_search'):
            return super(estoque_produto, self).search(cr, uid, args, offset, limit, order, context,
                                                  count)
        list_produto = []
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        new_args = []
        sem_fonema_produto_ids = []
        name_search = False
        descricao = ''
        if args:
            for x in args:
                if x[0] == 'categoria_id':
                    categoria_ids = x[2]
                    if not isinstance(categoria_ids, list):
                        if isinstance(categoria_ids, int) or categoria_ids.isdigit():
                            categoria_ids = [categoria_ids]
                        elif isinstance(categoria_ids, (str, unicode)):
                            categoria_ids = self.pool.get('estoque.categoria').search(cr, uid,
                                                [('name', x[1], x[2])], context=context)
                        else:
                            categoria_ids = []
                        if categoria_ids:
                            categoria_ids.extend(self.pool.get('estoque.categoria').search(cr, uid,
                                       [('parent_id', 'child_of', categoria_ids)], context=context))
                    else:
                        categoria_ids.extend(self.pool.get('estoque.categoria').search(cr, uid,
                                                       [('parent_id', 'child_of', x[2])], context=context))
                    new_args.append(('categoria_id', 'in', categoria_ids))

                elif (context.get('name_search') or context.get('busca_field')) and x[0] == 'id' \
                        and x[1] == 'in' and x[2]:
                    list_produto = x[2]
                    name_search = True
                elif x[0] == 'name' and x[1] == 'ilike' and not order:
                    descricao = x[2]
                    sem_fonema_produto_ids = self.pool.get(
                        'adm.utils').admutl_get_objeto_ids_sem_fonema(
                        cr, uid, descricao=descricao, nome_tabela='estoque_produto',
                        nome_coluna='name', company_id=company_id, limite=limit,
                        context=context, offset=offset)
                    new_args.append(x)
                elif x[0] == 'estpro_codigo_fornecedor' and x[1] == 'ilike':
                    x[2] = '%%%s%%' % x[2]
                    new_args.append(x)
                else:
                    new_args.append(x)
        args = new_args

        def _estpro_search_estoqueserie(args, context):
            args = filter(lambda i: isinstance(i, (tuple, list)) or i == '|', args)
            if context.get('ctx_wizserie_produto_controla_serie'):
                new_args = filter(lambda i: 'estpro_nserie_tipo' not in i, args)
                args_serie = [('estpro_nserie_tipo', 'in', ('NS', 'NC'))]
                new_args.extend(args_serie)
                args = new_args
            return args

        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'estoqueserie', context):
            args = _estpro_search_estoqueserie(args, context)
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        obj_estoque_fornecedor = self.pool.get('estoque.fornecedor')
        estpar_dados = obj_estoque_parametro.estpar_get_fields(
            cr, uid, fields_to_read=['codigo_numerico', 'estpar_tamanho_codigo'], context=context
        )
        if isinstance(estpar_dados, list):
            estpar_dados = estpar_dados[0]

        if context.get('local_id', False):
            local_id = context['local_id']
            if args:
                args = filter(lambda i: i[0] != 'id', args)

            cond_limit = campo_name = inner_produto = cond_prod = order_by= ''
            if context.get('name_search_sem_name') and limit:
                cond_limit = "LIMIT %s" % limit
                campo_name = ', ep.name'
                inner_produto = 'INNER JOIN estoque_produto AS ep ON ep.id = ell.produto_id'
                cond_prod = 'AND ep.active = True'
                order_by = 'ORDER BY name'


            # TODO: separa o sql e verificar o tipo se eh produto, servico ou os dois
            sql = """
                SELECT distinct ell.produto_id
                       {name}
                FROM estoque_local_locacao AS ell
                {inner_produto}
                WHERE ell.local_id = {local}
                  AND ell.active = True
                  {cond_prod}
                -- Acrescenta servicos que nao tem locais
                UNION

                SELECT id AS produto_id
                       {name}
                FROM estoque_produto AS ep
                WHERE tipo = 'S'
                  AND active = True
                {order_by}
                {cond_limit}
            """.format(name=campo_name,
                       inner_produto=inner_produto,
                       cond_prod=cond_prod,
                       local=local_id,
                       order_by=order_by,
                       cond_limit=cond_limit)
            cr.execute(sql)
            dados = cr.dictfetchall()
            if dados:
                produto_ids = map(lambda i: i.get('produto_id'), dados)
                args.append(('id', 'in', produto_ids))

        args1 = []
        tamanho_codigo_produto = estpar_dados['estpar_tamanho_codigo']
        for x in args:
            if 'fornecedor_ids' in x:
                if isinstance(x[2], (str, unicode)):
                    estoque_fornecedor_ids = []
                    name = '%' + x[2] + '%'
                    sql = """
                        SELECT id
                        FROM adm_pessoa
                        WHERE
                            nome ILIKE '%s'
                            AND active = True
                    """ % name
                    cr.execute(sql)
                    dados_pessoa = cr.dictfetchall()
                    pessoa_ids = map(lambda i: i.get('id'), dados_pessoa)
                    if pessoa_ids:
                        financeiro_fornecedor_ids = obj_financeiro_fornecedor.search(
                            cr, uid, [('pessoa_id', 'in', pessoa_ids)])
                        if financeiro_fornecedor_ids:
                            estoque_fornecedor_ids = obj_estoque_fornecedor.search(
                                cr, uid, [('fornecedor_id', 'in', financeiro_fornecedor_ids)])
                            if x[1] == '=':
                                x[1] = 'in'
                            elif x[1] == '!=':
                                x[1] = 'not in'
                            x[2] = estoque_fornecedor_ids
                    else:
                        x[1] = 'in'
                        x[2] = estoque_fornecedor_ids
            if 'codigo' in x:
                try:
                    if not isinstance(x[-1], list):
                        if estpar_dados['codigo_numerico']:
                            _codigo = formataString(x[-1].lstrip('0').rstrip(), tamanho_codigo_produto,
                                                    '0', 'E')
                        else:
                            _codigo = x[-1]
                        args1.append(('codigo', '=', _codigo))
                    else:
                        list_codigo = []
                        if estpar_dados['codigo_numerico']:
                            for cod in x[-1]:
                                _codigo = formataString(cod.lstrip('0').rstrip(),
                                                        tamanho_codigo_produto, '0', 'E')
                                list_codigo.append(_codigo)
                        else:
                            _codigo = x[-1]
                        args1.append(('codigo', 'in', list_codigo))
                    context['active_test'] = False
                except:
                    pass
            else:
                if 'automacao_ids' in x:
                    try:
                        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
                        codigo_barra_ids = obj_automacao_codigo_barra.search(cr, uid, [('codigo_barra', '=', x[-1])],
                                                                             context=context)
                        if codigo_barra_ids:
                            brw_automacao_codigo_barra = obj_automacao_codigo_barra.browse(cr, uid, codigo_barra_ids[0])
                            produto_id = brw_automacao_codigo_barra.produto_id.id
                            args1.append(('id', '=', produto_id))
                        else:
                            args1.append(('codigo', '=', x[-1]))
                        context['active_test'] = False
                    except:
                        pass
                else:
                    args1.append(x)
        args = args1

        if (context.get('name_search') or context.get('busca_field')) and name_search:
            return list_produto

        res = super(estoque_produto, self).search(cr, uid, args, offset, limit, order, context,
                                                  count)
        if not count: # Count = True, nao esta lendo, quer apenas saber quantos sao
            if sem_fonema_produto_ids:
                sem_fonema_produto_ids.extend(res)
                ids = []
                if context.get('busca_field') or context.get('cadastro_produto'):
                    ids = sem_fonema_produto_ids
                else:
                    [ids.append(_id) for _id in sem_fonema_produto_ids if res.count(_id)]
                res = list(OrderedDict.fromkeys(ids))
                res = res[:limit] if limit else res

        return res

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        context['name_search'] = True
        ids = []
        obj_adm_util = self.pool.get('adm.utils')
        if not context.get('view_name') == 'estoque_produto':
            _xName = name.strip()
            if args is None:
                args = []
            if _xName:
                sbusca = ['ci', 'cb', 'fo', 'fa', 'name']
                if _xName[:3] == 'cb=':
                    sbusca = ['cb']
                    cod = _xName[3:]
                elif _xName[:3] == 'fa=':
                    sbusca = ['fa']
                    cod = _xName[3:]
                elif _xName[:3] == 'fo=':
                    sbusca = ['fo']
                    cod = _xName[3:]
                elif _xName[:3] == 'ci=':
                    sbusca = ['ci']
                    cod = _xName[3:]
                elif _xName.isdigit():
                    cod = _xName
                else:
                    sbusca = ['ci', 'fa', 'fo', 'name']
                    cod = _xName
                if cod.isdigit():
                    if 'name' in sbusca:
                        sbusca.remove('name')
                for busca in sbusca:
                    if busca == 'cb':  # Codigo de barras
                        barra_ids = self.pool.get('automacao.codigo.barra').search(cr, uid, [
                            ('codigo_barra', '=', cod)],
                                                                                   limit=limit,
                                                                                   context=context)
                        if barra_ids:
                            regBarra = self.pool.get('automacao.codigo.barra').read(cr, uid, barra_ids,
                                                                                    ['produto_id'], context=context)
                            if isinstance(regBarra, list):
                                ids = map(lambda i: i['produto_id'][0], regBarra)
                            if context.get('nao_pesquisa_kit'):
                                ids = self.search(cr, uid, [('id', '=', ids[0])], context=context)

                            if context.get('nf_propria'):
                                self.codigo_barra = cod
                            else:
                                self.codigo_barra = ''
                            break # encontrou codigo de barras, parar laco
                    if busca == 'fa': # Codigo de fabricante
                        grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid,
                                                                                  [(
                                                                                      'codigo_no_fabricante',
                                                                                      '=', cod)],
                                                                                  limit=limit,
                                                                                  context=context)
                        if grade_ids:
                            reggrade = self.pool.get('estoque.produto.grade').read(cr, uid, grade_ids,
                                                                                   ['produto_id'], context=context)
                            if isinstance(reggrade, list):
                                ids = map(lambda i: i['produto_id'][0], reggrade)
                            break  # encontrou codigo de fabricante, parar laco
                    if busca == 'fo':  # Codigo de fornecedor
                        args_for = [('codigo_fornecedor', '=', cod)]
                        if context.get('fornecedor_id', False):
                            fornecedor_ids = self.pool.get('financeiro.fornecedor').search(
                                cr, uid, [('pessoa_id', '=', context['fornecedor_id'])],
                                context=context)
                            args_for.append(('fornecedor_id', 'in', fornecedor_ids))
                        codigo_fornecedor_ids = self.pool.get('estoque.produto.codigo.fornecedor').\
                            search(cr, uid, args_for, limit=limit, context=context)
                        if codigo_fornecedor_ids:
                            reggrade = self.pool.get('estoque.produto.codigo.fornecedor').read(
                                cr, uid, codigo_fornecedor_ids, ['produto_id'], context=context)
                            if isinstance(reggrade, list):
                                ids = map(lambda i: i['produto_id'][0], reggrade)
                            break  # encontrou codigo de fabricante, parar laco
                    if busca == 'ci':  # Codigo interno
                        dados_par_estoque = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                                                                                             'estoque',
                                                                                             ['codigo_numerico',
                                                                                              'estpar_tamanho_codigo'],
                                                                                             context=context)
                        cod_interno = cod
                        if dados_par_estoque['codigo_numerico']:
                            cod_interno = formataString(cod.lstrip('0').rstrip(),
                                                        dados_par_estoque['estpar_tamanho_codigo'], '0',
                                                        'E')
                            if cod_interno.isdigit() and int(cod_interno) <= 0:
                                continue

                        args_novo = [('codigo', '=', str(cod_interno))]
                        ids = self.search(cr, uid, args + args_novo, limit=limit, context=context)
                        if ids:
                            break
                    if busca == 'name':  # Pesquisa pelo nome
                        args.append(['name', operator, name])
                        ids = self.search(cr, uid, args, limit=80 if limit else None,
                                          context=context)
                        if ids:
                            break
            else:
                # indica que na consulta de produto nao foi passado nenhum argumento pelo usuario
                context['name_search_sem_name'] = True

        if not ids:
            if name:
                args = [('name', operator, name)] + args

            ids = self.search(cr, uid, args, limit=limit,context=context)

        res = self.name_get(cr, uid, ids, context)
        return res

    @synchronized_method
    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not default:
            default = {}
        codigo = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, 'estoque.produto')
        produto = self.browse(cr, uid, id, context)
        produto_name = produto.name + '*' or False
        default.update({'codigo': codigo, 'name': produto_name, 'locacao_ids': [], 'local_ids': [],
                        'preco_custo_ids': [], 'preco_venda_ids': [], 'fornecedor_ids': [],
                        'produto_grade_ids': [], 'produto_similaridade_ids': []})
        context['copy_estoque_produto'] = True  ## contexto de copia
        res = super(estoque_produto, self).copy(cr, uid, id, default, context)
        return res

    def estpro_get_estoque_parametro_codigo(self, cr, uid, context=None):
        context = context and context.copy() or {}
        res = True
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto', context=context)
        if not company_id:
            raise osv.except_osv(_('Atencao'),
                                 _('Nenhuma empresa foi encontrada para o usuario logado!'))
        else:
            obj_estoque_parametro = self.pool.get('estoque.parametro')
            estoque_parametro_ids = obj_estoque_parametro.search(cr, uid, [], context=context)
            if not estoque_parametro_ids:
                company_name = self.pool.get('res.company').browse(cr, uid, company_id).name or ''
                raise osv.except_osv(_('Atencao'), _(
                    'Nenhum parametro cadastrado para a Empresa: %s' % company_name))
            else:
                estoque_parametro_dados = obj_estoque_parametro.read(cr, uid,
                                                                     estoque_parametro_ids[0],
                                                                     ['codigo_numerico'], context=context)
                if estoque_parametro_dados and isinstance(estoque_parametro_dados, list):
                    estoque_parametro_dados = estoque_parametro_dados[0]
                if isinstance(estoque_parametro_dados, dict):
                    res = estoque_parametro_dados.get('codigo_numerico')
        return res

    def estpro_check_produto_active_name(self, cr, uid, ids, context):
        res = True
        if not isinstance(ids, list):
            ids = [ids]
        for brw_produto in self.browse(cr, uid, ids):
            name = brw_produto.name
            args = [('name', '=', name), ('id', '!=', brw_produto.id), ('active', '=', True)]
            produto_ja_cadastrado_id = self.search(cr, uid, args, context=context)
            if produto_ja_cadastrado_id:
                codigo = self.browse(cr, uid, produto_ja_cadastrado_id[0]).codigo
                name = self.browse(cr, uid, produto_ja_cadastrado_id[0]).name
                msg = _("Ja existe produto ativo com esta descricao, nao e possivel realizar "
                        "alteracao: " + codigo + " - " + name)
                raise osv.except_osv(_('Atencao'), _(msg))
        return res

    def estpro_atualiza_tributacao_entrada(self, cr, user, produto, fornecedor, encargo, cst,
                                           tributacao, context=None):
            """
            Atualiza a tributacao de entrada (encargo) para um produto com determinado fornecedor.

            :param cr: cursor do banco de dados
            :param user: ID do usuario utilizado para a verificacao de perfil
            :param produto ID do produto
            :param fornecedor: ID do fornecedor
            :param encargo: ID do encargo a ser atualizado
            :param cst: ID do CST a ser substituido
            :param tributacao: ID da tributacao a ser substituida
            :param context: um dicionario padrao para valores contextuais
            :type cr: object
            :type user: int
            :type produto: long
            :type fornecedor: long
            :type encargo: long
            :type cst: long
            :type tributacao: long
            :type context: dict
            :return: verdadeiro se as tributacoes de entrada foram atualizadas.
            :rtype: bool

            :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
            """

            context = context and context.copy() or {}
            fornecedor_ids = self.pool.get('financeiro.fornecedor').search(
                cr, user, [('pessoa_id', '=', fornecedor)], context=context)

            # Recupera a tributacao de entrada
            tributacao_entrada_ids = self.pool.get('fiscal.tributacao.entrada').search(
                cr, user, [('fornecedor_id', '=', fornecedor_ids and fornecedor_ids[0]),
                           ('produto_id', '=', produto)], context=context)

            # Recupera os dados da tributacao de entrada (encargos)
            brw_tributacao_entrada = self.pool.get('fiscal.tributacao.entrada').browse(
                cr, user, tributacao_entrada_ids and tributacao_entrada_ids[0], context)

            # Realiza a troca da tributacao e CST
            brw_tributacao_entrada_encargo = filter(lambda f: f.encargo_id.id == encargo,
                                                    brw_tributacao_entrada and
                                                    brw_tributacao_entrada.tributacao_entrada_ids)
            return self.pool.get('fiscal.tributacao.entrada.encargos').write(
                cr, user, brw_tributacao_entrada_encargo and brw_tributacao_entrada_encargo[0].id,
                dict(cst_id=cst, tributacao_id=tributacao))

    def estpro_get_saldo(self, cr, uid, ids, pesquisar="S", local=None, data=None, context=None):
        """
        Funcao para calcular o saldo, requisitados, ou disponivel de uma lista de produtos
        @param self:
        @param cr:
        @param uid:
        @param ids: lista de ids dos produtos
        @param pesquisar: Identifica o que deseja pesquisar: S=Saldo, R=Requisitado, D=Disponivel
        @param local: local especifico a pesquisar ou None para todos
        @param data: data especifica para busca do saldo
        @param context:
        @return: dicionario de produtos com o resultado da pesquisa(saldo, requisitado ou
        disponivel)
        """
        context = context and context.copy() or {}
        res = {}
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        tipo_pesquisa = {'S': 'saldo', 'D': 'disponivel', 'R': 'qtd_reservada'}[pesquisar]
        obj_est_prod_grade = self.pool.get('estoque.produto.grade')
        grade_ids = obj_est_prod_grade.search(cr, uid, [('produto_id','in', ids)], context=context)
        saldos = obj_estoque_produto_saldo.get_saldo_do_dia(
            cr, uid, grade_ids, data or date.today(), local, context=context)
        for grade in obj_est_prod_grade.read(
                cr, uid, grade_ids, ['produto_id'], load='_classic_write', context=context):
            if not res.get(grade.get('produto_id')):
                res[grade.get('produto_id')] = saldos.get(grade.get('id')).get(tipo_pesquisa, 0.0)
            else:
                res[grade.get('produto_id')] += saldos.get(grade.get('id')).get(tipo_pesquisa, 0.0)

        # Evita erro na consulta de produtos sem grade, pois o id dele era passado e nao era
        # retornado
        map(lambda y: res.update({y:0.0}), filter(lambda x: x not in res, ids))

        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
