# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class estoque_produto_embalagem(osv.Model):

    _name = 'estoque.produto.embalagem'
    _inherit = 'estoque.produto.embalagem'

    def quantidade_na_embalagem_get(self, cr, uid, ids, context=None):
        """
        Retorna a quantidade de unidades da embalagem

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} lista de dicionarios com o ID da embalagem e sua respectiva quantidade.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = []                                                                                 # Lista com as quantidades nas embalagens

        try:
            for brw_embalagem in self.browse(cr, uid, ids, context):
                quantidade_na_embalagem = brw_embalagem.embalagem_id.quantidade_na_embalagem
                result.append({'embalagem_id': brw_embalagem.id,
                               'quantidade_na_embalagem': quantidade_na_embalagem})
        except:
            pass

        return result

    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        context = context and context.copy() or {}
        reads = self.browse(cr, uid, ids, context=context)
        res = []
        for record in reads:
            _nome = '%s-%s' % (record.embalagem_id.sigla, record.embalagem_id.name)
            if context.get('estpre_so_codigo'):
                _nome = record.embalagem_id.sigla
            res.append((record['id'], _nome))
        return res

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        res = []
        ids = self.search(cr, uid, args, context=context)
        if ids:
            res = self.name_get(cr, uid, ids, context)
        return res

    def check_embalagem_ja_cadastrada(self, cr, uid, produto_id, embalagem_id, context=None):
        context = context and context.copy() or {}

        res = []
        if produto_id and embalagem_id:
            args = [('produto_id', '=', produto_id), ('embalagem_id', '=', embalagem_id), ('active', '=', True)]
            res = self.search(cr, uid, args, context=context)
        return res

    def raise_embalagem_cadastrada(self, cr, uid, produto_id, embalagem_id, context=None):
        context = context and context.copy() or {}
        # NOME DO PRODUTO
        produto_name = ''
        if produto_id:
            dict_produto_name = self.pool.get('estoque.utils').estutl_dict_produto_name(
                cr, uid, [produto_id], context
            )
            produto_name = dict_produto_name.get(produto_id)
        if not produto_name:
            produto_name = _('** Descricao do produto nao encontrada **')
            # NOME DA EMBALAGEM
        embalagem_name = ''
        if embalagem_id:
            name_get_embalagem = self.pool.get('estoque.embalagem').name_get(cr, uid, [embalagem_id], context)
            if name_get_embalagem:
                embalagem_name = dict(name_get_embalagem).get(embalagem_id)
        if not embalagem_name:
            embalagem_name = _('** Descricao da embalagem nao encontrada **')

        raise osv.except_osv(
            _('Atencao'), _('O produto %s ja possui a embalagem %s cadastrada!') % (produto_name, embalagem_name)
        )

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        if context.get('produto_id'):
            produto_id = context.get('produto_id')
        else:
            produto_id = False
        res = super(estoque_produto_embalagem, self).default_get(cr, uid, fields_list, context)
        res['produto_id'] = produto_id
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('produto_id'):
            vals['produto_id'] = context.get('produto_id')

        produto_id = vals.get('produto_id')
        embalagem_id = vals.get('embalagem_id')

        # VERIFICA SE A EMBALAGEM JA ENCONTRA-SE CADASTRADA PARA O PRODUTO.
        # SO DEVE HAVER UMA EMBALAGEM ATIVA CADASTRADA PARA O PRODUTO.
        produto_embalagem_ids = self.check_embalagem_ja_cadastrada(cr, uid, produto_id, embalagem_id, context)
        if produto_embalagem_ids:
            self.raise_embalagem_cadastrada(cr, uid, produto_id, embalagem_id)

        res = super(estoque_produto_embalagem, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'produto_id' in vals and 'embalagem_id' in vals:
            produto_id = vals['produto_id']
            embalagem_id = vals['embalagem_id']

            # VERIFICA SE A EMBALAGEM JA ENCONTRA-SE CADASTRADA PARA O PRODUTO.
            # SO DEVE HAVER UMA EMBALAGEM ATIVA CADASTRADA PARA O PRODUTO.
            produto_embalagem_ids = self.check_embalagem_ja_cadastrada(cr, uid, produto_id, embalagem_id, context)
            if produto_embalagem_ids:
                self.raise_embalagem_cadastrada(cr, uid, produto_id, embalagem_id)

        res = super(estoque_produto_embalagem, self).write(cr, uid, ids, vals, context)
        return res

