# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from dateutil.relativedelta import relativedelta
from datetime import datetime
import time


class estoque_produto_grade(osv.Model):
    """
    Tabela de Grades. Item
    Mesmo que o produto nao utilize grade, sempre tera um registro nesta tabela, pois o nome do produto esta aqui. Neste caso a descricao da grade sera \
    """

    _name = 'estoque.produto.grade'
    _inherit = 'estoque.produto.grade'

    def estpgr_get_saldo(self, cr, uid, ids, pesquisar="S", local=None, context=None):
        """
        Funcao para calcular o saldo, requisitados, ou disponivel de uma lista de grades
        @param self:
        @param cr:
        @param uid:
        @param ids: lista de ids das grades
        @param pesquisar: Identifica o que deseja pesquisar: S=Saldo, R=Requisitado, D=Disponivel
        @param local: local especifico a pesquisar ou None para todos
        @param context:
        @return: dicionario de grades com o resultado da pesquisa(saldo, requisitado ou disponivel)
        """
        context = context and context.copy() or {}
        res = {}
        o_saldo = self.pool.get('estoque.produto.saldo')
        if local:
            res_saldo = o_saldo.get_saldo_do_dia(cr, uid, ids, time.strftime('%Y-%m-%d'), eps_local_id=local,
                                                 context=context)
        else:
            res_saldo = o_saldo.get_saldo_do_dia(cr, uid, ids, time.strftime('%Y-%m-%d'), context=context)
        for _grade_id in res_saldo.keys():
            if pesquisar == 'D':
                resultado = res_saldo.get(_grade_id).get('disponivel', 0.0)
            elif pesquisar == 'R':
                resultado = res_saldo.get(_grade_id).get('qtd_reservada', 0.0)
            else:
                resultado = res_saldo.get(_grade_id).get('saldo', 0.0)
            res[_grade_id] = resultado
        return res


    def _estgrd_requisitado_acumular_valores(self, cr, uid, sql, res):
        """
        Acumulados valores de SQL para classes que interferem em itens requisitados
        Este metodo e chamado pelo estgrd_get_requisitado das diversas classes que interferem em itens requisitados
        @param cr:
        @param uid:
        @param sql: sql que deve ser executado
        @param dados_iniciais: a mesma estrutura do return
        @return: dicionario de grades contendo dicionario de locais com as quantidades requisitadas ex.:
                 {77:{1:0,2:0,'T':0}, ...}
                 onde 77 e o id da grade, 1 e 2 sao os ids dos locais, T e a soma de todos os locais da grade e 0 e a quantidade
        """
        cr.execute(sql)
        for x in cr.dictfetchall():
            _grade_id = x.get('grade_id')
            _local_id = x.get('local_id')
            res.setdefault(_grade_id, {}).setdefault(_local_id, 0.0)
            res.setdefault(_grade_id, {}).setdefault('T', 0.0)
            res[_grade_id][_local_id] += x.get('quantidade', 0.0)
            res[_grade_id]['T'] += x.get('quantidade', 0.0)
        return res

    def estgrd_get_requisitado(self, cr, uid, ids, context=None, company_id=False):
        """
        Metodo para retornar a quantidade requisitada de um ou varios itens(grades)
        :param cr:
        :param uid:
        :param ids: ids das grades para retornar as quantidades requisitadas
        :param context:
        :return: dicionario de grades contendo dicionario de locais com as quantidades requisitadas ex.:
                 {77:{1:0,2:0,'T':0}, ...}
                 onde 77 e o id da grade, 1 e 2 sao os ids dos locais, T e a soma de todos os locais da grade e 0 e a quantidade
        """

        context = context and context.copy() or {}
        dados = {}
        map(lambda x: dados.update({x:{}.copy()}), ids)
        return dados

    def estgrd_get_embalagem_default(self, cr, uid, produto_id, local_id=False, grade_id=False, context=None):
        context = context and context.copy() or {}
        produto_embalagem_id = False
        if not (local_id and produto_id and grade_id):
            return False

        obj_local_locacao = self.pool.get('estoque.local.locacao')
        args = [('local_id', '=', local_id),
                ('produto_id', '=', produto_id),
                ('grade_id', '=', grade_id)]
        locacao_ids = obj_local_locacao.search(cr, uid, args, context=context)
        if locacao_ids:
            ell = obj_local_locacao.read(cr, uid, locacao_ids[0],
                        ['estllc_produto_embalagem_id'], context=context, load='_classic_write')
            produto_embalagem_id = ell.get('estllc_produto_embalagem_id')

        if not produto_embalagem_id:
            produto_embalagem_id = self.pool.get('estoque.produto.embalagem').search(cr, uid,
                                                 [('produto_id', '=', produto_id)], context=context)
            # caso tenha mais de uma embalagem para o produto, retorna a primeira
            if produto_embalagem_id:
                produto_embalagem_id = produto_embalagem_id[0]

        return produto_embalagem_id

    def estgrd_consulta_saldo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        obj_wizard_consulta_saldos = self.pool.get('wizard.consulta.saldos')
        context['consulta_saldo_produto_grade'] = True
        res = obj_wizard_consulta_saldos.button_pesquisar(cr, uid, ids, context)
        return {
            'res_id': res,
            'name': 'Consulta de Saldos',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.consulta.saldos',
            'view_id': False,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }

    def ler(self, cr, uid, args, context=None):
        context = context and context.copy() or {}
        if not args:
            return []
        try:
            ids = self.search(cr, uid, args, context=context)
            vals = self.read(cr, uid, ids, [], context=context)
            return vals
        except:
            return []

    def create(self, cr, uid, vals, context=None):
        # Quanto e criada uma grade nova para o produto na importacao de nota fiscal eletronica, o produto nao
        # e passado para a grade. Este produto e passado dentro do contexto (usar_codigo_barra_na_grade_produto_id).
        context = context and context.copy() or {}
        if context.get('usar_codigo_barra_na_grade_produto_id'):
            vals['produto_id'] = context['usar_codigo_barra_na_grade_produto_id']

        grade = self.pool.get('adm.utils').admutl_remove_caracter(vals.get('grade'))
        if grade != vals.get('grade'):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel incluir a grade com caracter especial'))
        # ao duplicar produto nao pode ser criada(duplicada) a grade
        if context.get('copy_estoque_produto'):
            return 1

        # pesquisa produto.
        id = self.search(cr, uid, [('produto_id', '=', vals.get('produto_id'))], context=context)
        obj_prod = self.pool.get('estoque.produto')
        # verificando se o produto eh obrigado a usar grade
        if vals.get('produto_id'):
            vals_prod = obj_prod.read(cr, uid, vals['produto_id'], ['grade_usar'], context=context)
            if isinstance(vals_prod, list):
                vals_prod = vals_prod[0]
                # verifica se eh para usar grade
            if not vals_prod['grade_usar']:
                # verifica se o produto ja possui grade cadastrada
                if len(id) > 0:
                    raise osv.except_osv(_('Atencao'), _('Produto deve possuir apenas uma grade cadastrada.'))
            else:
                if not vals['grade']:
                    raise osv.except_osv(_('Atencao'), _('Campo grade deve ser preenchido.'))

                produto_grade_ids = self.search(cr, uid, [('produto_id', '=', vals.get('produto_id')),
                                                          ('grade', '=', vals.get('grade').upper())], context=context)
                if produto_grade_ids:
                    return produto_grade_ids[0]

        if 'grade' in vals:
            vals['grade'] = vals['grade'].upper()
            # preparando a locacao para gravar
        if 'local_locacao_ids' in vals:
            if vals['local_locacao_ids']:
                company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto', context=context)
                # gravando os codigo em listas de locacoes
                for loc_id in range(0, len(vals['local_locacao_ids'])):
                    vals['local_locacao_ids'][loc_id][2]['produto_id'] = vals['produto_id']
                    vals['local_locacao_ids'][loc_id][2]['company_id'] = company_id

        res = super(estoque_produto_grade, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        def _estpgr_write_automacao(cr, user, ids, vals, context):
            if vals.get('active', None) == False:
                # Quando desativar uma grade desativar tambem os codigos de barra da grade
                produtos_grade_dados = self.read(cr, user, ids, ['codigo_barra_ids'], context=context)
                if not isinstance(produtos_grade_dados, list):
                    produtos_grade_dados = [produtos_grade_dados]
                for produto_grade_dados in produtos_grade_dados:
                    # GERA UMA LISTA COM OS IDS DOS CODIGOS DE BARRAS, PODE HAVER NONE NOS ELEMENTOS DA LISTA
                    cod_barras = produto_grade_dados['codigo_barra_ids']
                    # FILTRA SOMENTE ELEMENTOS QUE TENHAM VALOR
                    cod_barra_ids = filter(lambda i: i, cod_barras)
                    if cod_barra_ids:
                        self.pool.get('automacao.codigo.barra').write(cr, user, cod_barra_ids, {'active': False},
                                                                      context=context)
            return True

        context = context and context.copy() or {}
        if 'grade' in vals and vals.get('grade'):
            grade = self.pool.get('adm.utils').admutl_remove_caracter(vals.get('grade'))
            if grade != vals.get('grade'):
                raise osv.except_osv(_('Atencao'), _('Nao e possivel incluir a grade com caracter especial'))
        # teste implementado para agilizar o write por causa da integracao
        if 'active' in vals:
            if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'automacao', context):
                _estpgr_write_automacao(cr, uid, ids, vals, context)
        if 'grade' in vals:
            if vals['grade']:
                vals['grade'] = vals['grade'].upper()
                # preparando a locacao para gravar
        if 'local_locacao_ids' in vals:
            if vals['local_locacao_ids']:
                produto = self.pool.get('estoque.produto.grade').read(cr, uid, ids, ['produto_id'], context=context)
                company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.local.locacao',
                                                                            context=context)

                # gravando os codigo em listas de locacoes
                for loc_id in range(0, len(vals['local_locacao_ids'])):
                    if isinstance(vals['local_locacao_ids'][loc_id][2], dict):
                        vals['local_locacao_ids'][loc_id][2]['produto_id'] = produto[0]['produto_id'][0]
                        vals['local_locacao_ids'][loc_id][2]['company_id'] = company_id
                        vals['local_locacao_ids'][loc_id][2]['grade_id'] = ids[0]

        # TODO: Verificar se o erro ainda acontece se o contexto nao for ´´{}´´
        # @change: 31.05.2013 - pierre.barbosa
        res = super(estoque_produto_grade, self).write(cr, uid, ids, vals,
                                                       context)  # PASSAR O CONTEXT={} PARA QUE NO CADASTRO DE LOCACAO NA GRADE NAO HAJA CONCORRENCIA ENTRE OS FILHOS, QUE NAO DEIXA ELES SEREM GRAVADOS SIMULTANEAMENTE
        return res

    def unlink(self, cr, uid, ids, context=None):
        """
        Exclui a grade do produto.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {super} Produto excluido.

        @change: 12.11.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')

        # Verifica se existe alguma movimentacao com a grade
        args = [('grade_id', 'in', ids)]
        if obj_estoque_movimento_itens.search(cr, uid, args, context=context):
            titulo = 'Atencao!'
            msg = 'Acao cancelada. Existem movimentacoes para a grade.'

            raise osv.except_osv(titulo, msg)

        result = super(estoque_produto_grade, self).unlink(
            cr, uid, ids, context=context)

        return result

    def estpgr_onchange_grade_descricao(self, cr, uid, ids, grade, descricao, produto_id, context=None):
        res = {}
        context = context and context.copy() or {}
        if not ids and grade and descricao and produto_id:
            grade_usar = self.pool.get('estoque.produto').browse(cr, uid, produto_id).grade_usar
            if grade_usar:
                vals = {'grade': grade, 'descricao': descricao, 'produto_id': produto_id}
                context['grade_criada'] = True
        return res

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(estoque_produto_grade, self).default_get(cr, uid, fields, context=context)
        if context.get('produto_id'):
            produto_id = context.get('produto_id')
            obj_estoque_produto = self.pool.get('estoque.produto')
            read_estoque_produto = obj_estoque_produto.read(cr, uid, [produto_id],
                                                            ['codigo', 'digito'], context=context)
            for dado in read_estoque_produto:
                res['codigo'] = dado.get('codigo')
                res['digito'] = dado.get('digito')
            res['produto_id'] = context.get('produto_id')
        return res

    def name_get(self, cr, uid, ids, context=None):
        res = []
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_adm_utils = self.pool.get('adm.utils')
        estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [], context=context)
        estoque_parametro_dados = obj_estoque_parametro.read(cr, uid, estoque_parametro_id, ['estpar_descricao_grade'],
                                                             context=context)
        descricao_grade = None
        for x in estoque_parametro_dados:
            if x.get('estpar_descricao_grade') == 'P':
                descricao_grade = True
        produto_grade_dados = self.read(cr, uid, ids, ['grade', 'descricao', 'produto_id'],
                                        context=context, load='_classic_write')
        for record in produto_grade_dados:
            nome_produto = ''
            id = record.get('id')
            descricao = obj_adm_utils.admutl_remover_acentuacao(cr, uid, record.get(
                'descricao')) or '*** Grade sem Descricao ***'
            grade = record.get('grade') or ''
            if descricao_grade:
                produto_id = record.get('produto_id')
                if produto_id and isinstance(produto_id, tuple):
                    produto_id = produto_id[0]
                if produto_id:
                    estoque_produto_dados = obj_estoque_produto.read(cr, uid, produto_id, ['name'], context=context)
                    nome_produto = obj_adm_utils.admutl_remover_acentuacao(
                        cr, uid, str(estoque_produto_dados.get('name').encode('utf-8')))
                    if grade == 'UN':
                        descricao = ''
                    descricao = '%s%s%s' % (nome_produto, ' ' if nome_produto else '', descricao)
            result = '%s%s%s' % (_(grade), ' - ' if grade else '', _(descricao))
            if context.get('retorno_sem_codigo'):
                result = descricao
            if context.get('estpgr_so_codigo'):
                result = grade
            if len(result) > 63:
                result = result[0:63]
            res.append((id, result))
        return res

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        ids = self.search(cr, uid, [('grade', '=', name.upper())] + args, limit=limit, context=context)
        if not ids:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, uid, ids, context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        def _estpro_search_estoqueressuprimento(cr, uid, context):
            # ('ressupri_grade_search')FLAG PARA ACIONAR A PESQUISA
            grade_ids = []
            if context.get('ressupri_grade_search') and context.get('local_saida_id') and context.get(
                    'company_saida_id') and \
                    context.get('produto_id') and context.get('local_entrada_id') and context.get('company_entrada_id'):
                # PESQUISA PERMITE QUE SEJAM SOMENTE EXIBIDAS A GRADES QUE POSSUEM O LOCAL DE SAIDA INFORMADO
                obj_local_locacao = self.pool.get('estoque.local.locacao')
                entrada_local_grade_ids = obj_local_locacao.search(cr, uid, [('produto_id', '=', context['produto_id']),
                                                                             ('local_id', '=',
                                                                              context['local_saida_id']),
                                                                             ('company_id', '=',
                                                                              context['company_saida_id'])],
                                                                   context=context)
                saida_local_grade_ids = obj_local_locacao.search(cr, uid, [('produto_id', '=', context['produto_id']),
                                                                           ('local_id', '=',
                                                                            context['local_entrada_id']),
                                                                           ('company_id', '=',
                                                                            context['company_entrada_id'])],
                                                                 context=context)
                if not entrada_local_grade_ids or not saida_local_grade_ids:
                    return []
                locacao_ids = entrada_local_grade_ids + saida_local_grade_ids
                local_grade = obj_local_locacao.read(cr, uid, locacao_ids, ['grade_id'], context=context)
                if isinstance(local_grade, dict):
                    local_grade = [local_grade]
                for i in local_grade:
                    try:
                        grade_ids.append(i['grade_id'][0])
                    except:
                        grade_ids = []
                        pass
                grade_ids = list(set(grade_ids))
            return grade_ids

        context = context and context.copy() or {}
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'estoqueressuprimento', context):
            resuprimento_ids = _estpro_search_estoqueressuprimento(cr, uid, context)
            if resuprimento_ids:
                return resuprimento_ids
        if context.get('cadastro_produto'):
            context['active_test'] = False
        return super(estoque_produto_grade, self).search(cr, uid, args, offset=offset, limit=limit, order=order,
                                                         context=context, count=count)

    def estpgr_onchange_codigo(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        vals = {}
        if 'produto_id' in context:
            vals['produto_id'] = context.get('produto_id')
        obj_estoque_produto = self.pool.get('estoque.produto')
        if produto_id:
            brw_estoque_produto = obj_estoque_produto.browse(cr, uid, produto_id)
            codigo_produto = brw_estoque_produto.codigo
            digito_produto = brw_estoque_produto.digito
            vals['codigo'] = codigo_produto
            vals['digito'] = digito_produto
            vals['grade'] = ''
        return {'value': vals}

    def onchange_grade(self, cr, uid, ids, produto_id, grade, context=None):
        context = context and context.copy() or {}
        if not grade:
            return {}
        vals = {}
        if 'produto_grade_ids' in context:  # vem da tela do produto
            for id in context.get('produto_grade_ids'):
                grade_sigla = id[1] or id[2].get('grade')
                if isinstance(grade_sigla, int):
                    grade_sigla = self.browse(cr, uid, grade_sigla).grade
                if grade == grade_sigla:
                    warning = {}
                    vals['grade'] = ''
                    warning['message'] = _('Essa grade ja existe para este produto')
                    return {'warning': warning, 'value': vals}
        else:  # vem da tela da grade
            args = [('produto_id', '=', produto_id)]
            grades_ids = self.search(cr, uid, args, context=context)
            brw_grades = self.browse(cr, uid, grades_ids)
            for brw_grade in brw_grades:
                if brw_grade.grade == grade:
                    warning = {}
                    vals['grade'] = ''
                    warning['message'] = _('Essa grade ja existe para este produto')
                    return {'warning': warning, 'value': vals}
        return {'value': vals}

    def estpgr_calcula_curvaabc(self, cr, uid, tipo_curva, dados, context=None):
        """
        O metodo classifica a importancia de cada um dos produtos no estoque, baseado no total geral
        vendido e no total de cada item ou tambem no seu saldo. As classificacoes A, B e C  sao
        baseadas por valores parametrizados.
        @param cr: cursor do banco de dados
        @param uid: id do usuario
        @param tipo_curva: char com uma das opcoes A, B, C ou T (todas)
        @param dados: lista de dicionarios com pelo menos as informacoes de id do produto e seu
        total vendido num periodo
        @return: dicionario com a chave representando o id do produto e o valor da mesma a
        classificacao da curva ou True caso o metodo seja chamado pelo cron. Nesse ultimo, o
        cadastro de grades eh atualizado de acordo com a classificacao pertinente
        """
        context = context and context.copy() or {}
        obj_adm_util = self.pool.get('adm.utils')
        obj_estoque_util = self.pool.get('estoque.utils')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_produto = self.pool.get('estoque.produto')
        campos = ['estpar_tipo_curva', 'estpar_periodo', 'estpar_primeiro_percentual',
                  'estpar_segundo_percentual', 'estpar_tipo_analise', 'estpar_forma_execucao_curva',
                  'estpar_custo_para_precovenda', 'estpar_margem_para_preco_venda']
        estoque_parametro_dados = obj_adm_util.admutl_get_parametros(cr, uid, 'estoque', campos,
                                                                     context=context)
        primeiro_percentual = estoque_parametro_dados.get('estpar_primeiro_percentual')
        segundo_percentual = estoque_parametro_dados.get('estpar_segundo_percentual')
        if not dados:
            forma_execucao = estoque_parametro_dados.get('estpar_forma_execucao_curva')
            if not forma_execucao or forma_execucao == 'M':
                return True
            curva = estoque_parametro_dados.get('estpar_tipo_curva')
            if not curva:
                return True
            tipo_analise = estoque_parametro_dados.get('estpar_tipo_analise')
            if not tipo_analise:
                return True
            if curva == 'V':
                movimento_item_company_id = obj_adm_empresa_objeto.getEmpresa(
                    cr, uid, 'estoque.movimento.itens', context=context)
                periodo = estoque_parametro_dados.get('estpar_periodo')
                data_final = datetime.now()
                data_inicial = data_final - relativedelta(days=periodo)
                data_final = data_final.strftime('%Y-%m-%d')
                data_inicial = data_inicial.strftime('%Y-%m-%d')
                sql = """
                    SELECT
                        emi.produto_id AS produto_id,
                        emi.grade_id AS grade_id,
                        sum(emi.vr_total_item) as vr_total_item
                    FROM estoque_movimento_itens emi
                    INNER JOIN estoque_produto ep ON emi.produto_id = ep.id
                    WHERE
                        emi.capa_id IN (
                            SELECT id
                            FROM estoque_movimento_capa
                            WHERE company_id = %s
                                AND data >= '%s'
                                AND data <= '%s'
                                AND estmvc_operacao_id IN (
                                            SELECT id
                                            FROM estoque_natureza_operacao
                                            WHERE
                                                estnto_atualizar_saldo = True
                                                AND estnto_tipo_operacao = 'S'
                                                AND active = True
                                                AND estnto_gerar_estatistica = True
                                            )
                        )
                        AND emi.cfop_id IN (
                                    SELECT id
                                    FROM fiscal_cfop
                                    WHERE
                                        tipo = 'S'
                                        AND movimento_tipo = 'C'
                                    )
                        AND ep.active = True
                        AND ep.estpro_sazonal is not True
                    GROUP BY emi.grade_id, emi.produto_id
                """ % (movimento_item_company_id, data_inicial, data_final)
                cr.execute(sql)
                dados = cr.dictfetchall()
                if tipo_analise == 'L':
                    produto_ids = list(set(map(lambda i: i.get('produto_id'), dados)))
                    custos = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_ids)
                    custo_para_precovenda = estoque_parametro_dados.get(
                        'estpar_custo_para_precovenda')
                    custo = ''
                    if custo_para_precovenda == 'U':
                        custo = 'estprc_ultimo_custo'
                    elif custo_para_precovenda == 'M':
                        custo = 'estprc_custo_medio_contabil'
                    elif custo_para_precovenda == 'R':
                        custo = 'estprc_custo_reposicao'
                    elif custo_para_precovenda == 'C':
                        custo = 'estprc_custo_corrigido'
                    margem_preco_venda = estoque_parametro_dados.get(
                        'estpar_margem_para_preco_venda')
                    for x in dados:
                        produto_id = x.get('produto_id')
                        margem = obj_estoque_produto.estpro_get_margens(cr, uid, produto_id)
                        percentual_margem = margem.get(produto_id).get(margem_preco_venda) or 0.0
                        custo_produto = custos.get(produto_id) or {}
                        preco_custo = custo_produto.get(custo) or 0.0
                        percentual_encargos = obj_estoque_util. \
                            estutl_calcular_porcentagem_encargos_produto(cr, uid, produto_id,
                                                                         context)
                        preco_venda = obj_estoque_util.estutl_calcular_preco_de_venda(
                            preco_custo, percentual_encargos, percentual_margem)
                        x['vr_total_item'] = preco_venda
            elif curva == 'E':
                sql = """
                    SELECT
                        epg.id AS grade_id,
                        epg.produto_id
                    FROM estoque_produto_grade epg
                    INNER JOIN estoque_produto ep ON epg.produto_id = ep.id
                    WHERE
                        epg.active = True
                        AND ep.active = True
                        AND ep.estpro_sazonal is not True
                    ORDER BY produto_id
                """
                cr.execute(sql)
                dados = cr.dictfetchall()
                grade_ids = map(lambda i: i.get('grade_id'), dados)
                data_hoje = datetime.now().strftime('%Y-%m-%d')
                dados = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, grade_ids, data_hoje, context=context)
        dict_curva = {}
        total_valor_geral = sum(map(lambda i: i.get('vr_total_item') or
                                              (i.get('disponivel') if i.get('disponivel') > 0 else 0), dados))
        list_produto_vr_total = sorted(dados,
                                       key=lambda i: i.get('vr_total_item') or i.get('disponivel'),
                                       reverse=True)

        acumulado = 0
        for x in list_produto_vr_total:
            total_item = x.get('vr_total_item') or (x.get('disponivel') if x.get('disponivel') > 0 else 0)
            acumulado += total_item
            porcentagem = 0
            if total_valor_geral:
                porcentagem = (acumulado / total_valor_geral) * 100
            if porcentagem <= primeiro_percentual:
                if context.get('estpgr_curva_abc_cron'):
                    dict_curva[x.get('grade_id')] = 'A'
                else:
                    if tipo_curva in ('T', 'A', False):
                        dict_curva[x.get('produto_id')] = 'A'
            elif primeiro_percentual < porcentagem <= segundo_percentual:
                if context.get('estpgr_curva_abc_cron'):
                    dict_curva[x.get('grade_id')] = 'B'
                else:
                    if tipo_curva in ('T', 'B', False):
                        dict_curva[x.get('produto_id')] = 'B'
            elif porcentagem > segundo_percentual:
                if context.get('estpgr_curva_abc_cron'):
                    dict_curva[x.get('grade_id')] = 'C'
                else:
                    if tipo_curva in ('T', 'C', False):
                        dict_curva[x.get('produto_id')] = 'C'
        if context.get('estpgr_curva_abc_cron') and dict_curva:
            dict_grade_ids = {}
            curva_a = filter(lambda i: dict_curva.get(i) == 'A', dict_curva)
            if curva_a:
                dict_grade_ids['A'] = tuple(curva_a)
            curva_b = filter(lambda i: dict_curva.get(i) == 'B', dict_curva)
            if curva_b:
                dict_grade_ids['B'] = tuple(curva_b)
            curva_c = filter(lambda i: dict_curva.get(i) == 'C', dict_curva)
            if curva_c:
                dict_grade_ids['C'] = tuple(curva_c)
            for x in dict_grade_ids:
                sql = """
                    UPDATE
                        estoque_produto_grade
                    SET estpgr_classificacao_curva = '%s'
                    WHERE
                        id in %s
                """ % (x, dict_grade_ids.get(x))
                cr.execute(sql)
            return True
        else:
            return dict_curva

    def estpgr_get_produto_id_grade_unica(self, cr, uid, produto_id, context=None):
        """
        O metodo tem o objetivo de encontrar o id da grade para um produto especifico, caso a mesma
        seja unica.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param produto_id: id da tabela estoque_produto.
        :param context: dicionario com valores contextuais.
        :return: id da grade caso o produto possua somente ela. Booleano False caso nao seja
        informado o produto ou o produto possua mais de uma grade.
        """
        context = context and context.copy() or {}
        res = False
        if not produto_id:
            return res
        produto_grade_ids = self.search(cr, uid, [('produto_id', '=', produto_id)], context=context)
        if len(produto_grade_ids) == 1:
            res = produto_grade_ids[0]
        return res

    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not default:
            default = {}
        default.update({
            'produto_grade_ids': [],
            'local_locacao_ids': [],
            'produto_saldo_ids': [],
            'inventario_contagem_ids': [],
            'inventario_oficial_ids': [],
            'produto_codigo_fornecedor_ids':[],
            'movimento_itens_ids':[],
        })
        res = super(estoque_produto_grade, self).copy(cr, uid, id, default, context)
        return res
