# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp import SUPERUSER_ID
from datetime import datetime

class estoque_produto_saldo(osv.Model):
    _name = 'estoque.produto.saldo'
    _inherit = 'estoque.produto.saldo'

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        res = super(estoque_produto_saldo, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=count)
        return res

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        res = super(estoque_produto_saldo, self).name_search(
            cr, uid, name=name, args=args, operator=operator, context=context, limit=limit)
        return res

    def _auto_init(self, cr, context=None):
        context = context and context.copy() or {}
        super(estoque_produto_saldo, self)._auto_init(cr, context)
        cr.execute("SELECT 1 FROM pg_indexes WHERE indexname='estoque_produto_saldo_grade_data_idx'")
        if not cr.fetchone():
            cr.execute(
                'CREATE INDEX estoque_produto_saldo_grade_data_idx ON estoque_produto_saldo (grade_id,data_movimento)')
        cr.execute("SELECT 1 FROM pg_indexes WHERE indexname='estoque_produto_saldo_completo_data_idx'")
        if not cr.fetchone():
            cr.execute(
                'CREATE INDEX estoque_produto_saldo_completo_data_idx ON estoque_produto_saldo (grade_id,company_id,data_movimento,local_id)')

    def _set_local_locacao(self, cr, uid, local_id, dict_grade_produto, context=None):
        """
        Verifica se existe locacao para as grades no local especifico,
        para as grades que nao existir, cria a locacao
        :param cr:
        :param uid:
        :param local_id: id do local
        :param dict_grade_produto: id da grade eh a chave e o id do produto eh o valor
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        obj_pod_emb = self.pool.get('estoque.produto.embalagem')
        cr.execute("""SELECT produto_id,
                             grade_id
                      FROM estoque_local_locacao
                      WHERE grade_id IN ({grade_ids})
                        AND local_id = {local_id}
                   """.format(grade_ids=','.join(map(lambda x: '%s' % x, dict_grade_produto.keys())),
                              local_id=local_id)
                   )
        dados_locacao = cr.dictfetchall()
        # retira o contexto 'sig_force_company' para nao criar locacao na empresa errada
        if 'sig_force_company' in context:
            del context['sig_force_company']

        chaves = map(lambda x: '%s-%s' % (x.get('produto_id'), x.get('grade_id')), dados_locacao)
        for grade_id, produto_id in dict_grade_produto.items():
            chave = '%s-%s' % (produto_id, grade_id)
            if chave not in chaves:
                vals = {'produto_id': produto_id,
                        'grade_id': grade_id,
                        'local_id': local_id}
                prod_emb_ids = obj_pod_emb.search(cr, uid, [('produto_id', '=', produto_id)],
                                                  context=context)
                emb_id = False
                if prod_emb_ids:
                    emb_id = obj_pod_emb.read(cr, uid, prod_emb_ids[0], ['embalagem_id'], context,
                                              '_classic_write').get('embalagem_id')
                if emb_id:
                    vals['estllc_produto_embalagem_id'] = emb_id

                self.pool.get('estoque.local.locacao').create(cr, uid, vals, context)

    def atualizar_saldo_do_dia(self, cr, uid, eps_company_id, grade_qtd, eps_data,
                               eps_entrada_saida, eps_local_id, context=None):
        '''
        Atualiza o saldo do dia passado em 'eps_data' e os saldos posteriores a este dia se houver
        :param cr:
        :param uid:
        :param eps_company_id: id da empresa que sera alterado o saldo
        :param grade_qtd: dicionario onde a chave eh o id da grade e o valor eh a quantidade
        :param eps_data: data do dia a ser atualizado
        :param eps_entrada_saida: 'E'=entrada, 'S'=Saida tipo do movimento
        :param eps_local_id: id do local que o saldo sera alterado
        :param eps_quantidade: valor a ser somado ou subtraido do saldo
        :param context:
        :return:
        '''

        context = context and context.copy() or {}
        grade_ids = grade_qtd.keys()
        saldos = self.get_saldo_do_dia(cr, uid, grade_ids, eps_data,
                              eps_local_id=eps_local_id, company_id=eps_company_id, context=context)

        str_grade_ids = ','.join(map(lambda x: '%s' % x, grade_ids))
        # FILTRA OS PRODUTOS RETIRANDO OS SERVICOS SE HOUVER (P)
        cr.execute("""SELECT ep.id  AS produto_id,
                             epg.id AS grade_id
                      FROM estoque_produto             AS ep
                      INNER JOIN estoque_produto_grade AS epg ON epg.produto_id = ep.id
                      WHERE epg.id IN ({grade_ids})
                        AND ep.tipo = 'P' """.format(grade_ids=str_grade_ids)
                  )
        dados_produto = cr.dictfetchall()
        if not dados_produto: # nao tem produto, somente servico
            return True

        dict_grade_produto = {}
        grade_ids = []
        for i in dados_produto:
            dict_grade_produto[i.get('grade_id')] = i.get('produto_id')
            grade_ids.append('%s' % i.get('grade_id'))

        self._set_local_locacao(cr, uid, eps_local_id, dict_grade_produto, context)
        # VERIFICA SE EXISTE SALDO P/ grade_ids e eps_data
        sql = '''SELECT id AS saldo_id,
                        grade_id,
                        produto_id,
                        create_date,
                        create_uid,
                        saldo,
                        data_movimento
                 FROM estoque_produto_saldo
                 WHERE grade_id IN ({grade_ids})
                   AND company_id = {company_id}
                   AND data_movimento = '{data}'
                   AND local_id = {local_id}
                   '''.format(data=eps_data,
                              grade_ids=','.join(grade_ids),
                              local_id=eps_local_id,
                              company_id=eps_company_id)
        cr.execute(sql)
        pgrade_saldo = {} # chave composta por ids do produto e da grade e dict com dados do saldo
        for dado in cr.dictfetchall():
            chave = '%s-%s' % (dado.get('produto_id'), dado.get('grade_id'))
            pgrade_saldo[chave] = {'id': dado.get('saldo_id'),
                                   'create_date': dado.get('create_date'),
                                   'create_uid': dado.get('create_uid') or 1}

        grade_saldo = {}
        list_excluir_saldos = []
        list_vals_saldos = []
        write_date = datetime.now()
        for dado in dados_produto:
            produto_id = dado.get('produto_id')
            grade_id = dado.get('grade_id')
            chave = '%s-%s' % (produto_id, grade_id)
            registro_saldo_id = pgrade_saldo.get(chave) and pgrade_saldo.get(chave).get('id')
            create_date = datetime.now()
            create_uid = uid
            if registro_saldo_id:
                create_date = pgrade_saldo.get(chave).get('create_date')
                create_uid = pgrade_saldo.get(chave).get('create_uid')
            saldo = saldos.get(grade_id).get('saldo', 0)
            if eps_entrada_saida == 'E':
                saldo = float(saldo) + float(grade_qtd.get(grade_id))
            else:
                saldo = float(saldo) - float(grade_qtd.get(grade_id))
            if -0.0001 < saldo and saldo < 0.0001:
                saldo = 0
            list_vals_saldos.append("""('{create_date}', {create_uid}, '{write_date}', {write_uid},
                {saldo}, {company_id}, {produto_id}, {local_id}, {grade_id}, '{data_movimento}')
                    """.format(saldo=saldo, company_id=eps_company_id, produto_id=produto_id,
                               local_id=eps_local_id, grade_id=grade_id, data_movimento=eps_data,
                               create_date=create_date, write_uid=uid, create_uid=create_uid,
                               write_date=write_date))
            grade_saldo[grade_id] = saldo
            if registro_saldo_id:
                list_excluir_saldos.append(registro_saldo_id)

        # Alterar o saldo de todos os registros com data posterior a data que esta entrando
        sql = sql.replace("data_movimento = '%s'" % eps_data,
                          "data_movimento > '%s'" % eps_data)
        cr.execute(sql)
        for dado in cr.dictfetchall():
            valor_saldo = dado.get('saldo') or 0
            grade_id = dado.get('grade_id')
            if eps_entrada_saida == 'E':
                valor_saldo += float(grade_qtd.get(grade_id))
            else:
                valor_saldo -= float(grade_qtd.get(grade_id))
            list_vals_saldos.append("""('{create_date}', {create_uid}, '{write_date}', {write_uid},
                {saldo}, {company_id}, {produto_id}, {local_id}, {grade_id}, '{data_movimento}')
                    """.format(saldo=valor_saldo, company_id=eps_company_id,
                               produto_id=dado.get('produto_id'), local_id=eps_local_id,
                               grade_id=grade_id, data_movimento=dado.get('data_movimento'),
                               create_date=dado.get('create_date'), write_date=write_date,
                               create_uid=dado.get('create_uid'), write_uid=uid))
            list_excluir_saldos.append(dado.get('saldo_id'))

        # Detela os saldos exitentes para depous incluir em massa para ganhar performance
        if list_excluir_saldos:
            sql_delete = """
                DELETE FROM estoque_produto_saldo WHERE id IN (%s)
            """ % ','.join(map(lambda x: '%s' % x,list_excluir_saldos))
            cr.execute(sql_delete)

        tam_saldos = len(list_vals_saldos)
        _max = 30000  # Maximo de registros inseridos em massa para evitar estouro de memoria
        partes = tam_saldos / _max
        resto = tam_saldos % _max
        ultimo_intervalo = 0
        sql_insert = """
            INSERT INTO estoque_produto_saldo
                  (create_date, create_uid, write_date, write_uid, saldo, company_id,
                   produto_id, local_id, grade_id, data_movimento)
            VALUES %s
        """
        for i in range(0, partes):
            cr.execute(sql_insert % ',\n'.join(list_vals_saldos[i * _max:(i + 1) * _max]))
            ultimo_intervalo = (i + 1) * _max
        if resto:  # Insere a ultima parte
            cr.execute(sql_insert % ',\n'.join(list_vals_saldos[ultimo_intervalo:]))

        # Atualiza o saldo no supervisor quando o modulo ecf estiver instalado
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(
                cr, uid, 'ecf', context) and grade_saldo:
            self.pool.get('ecf.exportar.saldo.super').ecfess_atualizar_registros(
                cr, uid, eps_company_id, eps_local_id, grade_saldo, context=context)

        return True

    def get_saldo_do_dia(self, cr, uid, eps_grade_ids, eps_data, eps_local_id=False,
                         company_id=False, context=None):
        """
        Metodo que retorna o saldo de um ou de uma lista de produtos para um dia
        @param cr:
        @param uid:
        @param eps_grade_ids: id ou lista de ids das grades
        @param eps_data: data a ser considerada para verificacao do saldo
        @param eps_local_id: local especifico para pesquisa do saldo
        @param context:
        @return: Dict grades, com dicionario contendo saldo, reerva e disponivel
                 ex.: {111:{'saldo': 1.0, 'qtd_reservada': 0.0, 'disponivel': 0.0} ...}
                 onde 111 e o id da grade
        """
        context = context and context.copy() or {}
        if not eps_grade_ids:
            return {}
        if eps_grade_ids and not isinstance(eps_grade_ids, list):
            eps_grade_ids = [eps_grade_ids]

        if None in eps_grade_ids:
            eps_grade_ids = list(set(eps_grade_ids))
            eps_grade_ids.remove(None)

        resultado = {}
        cr.execute("""SELECT epg.id AS grade_id,
                             epe.id AS produto_emb_id
                      FROM estoque_produto_grade           AS epg
                      INNER JOIN estoque_produto_embalagem AS epe ON epg.produto_id = epe.produto_id
                      WHERE epg.id IN (%s)
                   """ % ','.join(map(lambda x: '%s' % x, eps_grade_ids)))
        grade_emb = {}
        for i in cr.dictfetchall():
            if not grade_emb.get(i.get('grade_id')):
                grade_emb[i.get('grade_id')] = [i.get('produto_emb_id')]
            else:
                grade_emb[i.get('grade_id')].append(i.get('produto_emb_id'))

        for grade_id in eps_grade_ids:
            produto_emb_ids = grade_emb.get(grade_id)
            default_emb = {}
            if produto_emb_ids:
                for i in produto_emb_ids:
                    default_emb[i] = {'saldo': 0.0,
                                      'qtd_reservada': 0.0,
                                      'disponivel': 0.0}
            resultado.update(
                {grade_id: {'saldo': 0.0, 'qtd_reservada': 0.0, 'disponivel': 0.0,
                            'saldo_na_embalagem': default_emb
                            }
                })

        eps_company_id = context.get('company_id') or company_id or False
        if not eps_company_id:
            eps_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'estoque.produto.saldo', context=context)
        # Identifica se a empresa que esta sendo consultado o saldo, e fiscal
        consultada_empresa_fiscal = self.pool.get('res.company').browse(cr, uid, eps_company_id,
                                                                        context).rescon_fiscal

        # Identifica se a empresa em que o usuario esta 'logado', e fiscal
        logada_empresa_fiscal = self.pool.get('res.users').browse(
            cr, uid, uid, context).company_id.rescon_fiscal

        if eps_local_id:
            eps_local_ids = eps_local_id
            if not isinstance(eps_local_ids, (list, tuple)):
                eps_local_ids = [eps_local_ids]
        else:
            el_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                    cr, uid, 'estoque.local', context=context)
            _sql = '''SELECT id
                      FROM estoque_local
                      WHERE company_id = %s
                        AND excluir_consultas = False
                      ORDER BY id''' %  el_company_id
            cr.execute(_sql)
            res = cr.fetchall()
            eps_local_ids = [x[0] for x in res]

        str_local_ids = '('+','.join(str(e) for e in eps_local_ids)+')'
        str_grade_ids = '('+','.join(str(e) for e in eps_grade_ids)+')'
        cond_local = ''
        if eps_local_ids:
            cond_local += " AND eps.local_id in " + str_local_ids
        """ 
        criado indice composto estoque_produto_saldo_completo_data_idx para (grade, company, data_movimento e local) 
            para melhorar a performance.
        Se alterar a clausula where deste sql, eh melhor ajustar o indice tambem
        """
        sql = '''SELECT SUM(eps.saldo) AS saldo,
                        grade_id
                  FROM estoque_produto_saldo AS eps
                  INNER JOIN res_company rc
                    ON rc.id = eps.company_id
                  WHERE eps.grade_id IN {grade_ids}
                    AND rc.id = {company_id}
                    AND eps.data_movimento = (SELECT MAX(data_movimento)
                                              FROM estoque_produto_saldo eps1
                                              WHERE eps1.grade_id = eps.grade_id
                                                AND eps1.company_id = eps.company_id
                                                AND eps1.data_movimento <= '{data}'
                                                AND eps1.local_id = eps.local_id
                                                )
                    {cond_local}
                    {cond_empresa_fiscal}
                  GROUP BY grade_id'''.format(
            grade_ids=str_grade_ids,
            data=str(eps_data),
            cond_local=cond_local,
            company_id=eps_company_id,
            cond_empresa_fiscal='AND rc.rescon_fiscal IS FALSE' if not logada_empresa_fiscal
            else '')
        cr.execute(sql)
        res = cr.dictfetchall()
        grade_qtd_na_embalagem = self.pool.get('estoque.utils').get_grade_qtd_embalagem(cr,
                                                                     eps_grade_ids, context=context)
        for x in res:
            resultado[x.get('grade_id')]['saldo'] = x.get('saldo', 0.0)
            if grade_emb.get(x.get('grade_id')):
                for produto_emb_id in grade_emb.get(x.get('grade_id')):
                    chave = '%s-%s' % (x.get('grade_id'), produto_emb_id)
                    qtd_na_embalagem = grade_qtd_na_embalagem.get(chave).get('qtd') or 1
                    resultado[x.get('grade_id')]['saldo_na_embalagem'][produto_emb_id]['saldo'] = \
                                                              x.get('saldo', 0.0) / qtd_na_embalagem

        #---pegar qtd reservada
        dados_requisitados = self.pool.get('estoque.produto.grade').estgrd_get_requisitado(cr, uid,
                                          eps_grade_ids, context=context, company_id=eps_company_id)
        if not logada_empresa_fiscal and consultada_empresa_fiscal:
            dados_requisitados = {}
        if dados_requisitados:
            _idxs = dados_requisitados.keys()
            for _grade_id in _idxs:
                _res_grade = dados_requisitados.get(_grade_id)
                if eps_local_id:
                    _qtd_res = _res_grade.get(eps_local_id, 0.0)
                else:
                    _qtd_res = _res_grade.get('T', 0.0)

                disponivel = float(resultado.get(_grade_id).get('saldo')) - float(_qtd_res)
                resultado[_grade_id]['qtd_reservada'] = _qtd_res
                resultado[_grade_id]['disponivel'] = disponivel
                if grade_emb.get(_grade_id):
                    for produto_emb_id in grade_emb.get(_grade_id):
                        chave = '%s-%s' % (_grade_id, produto_emb_id)
                        qtd_na_embalagem = grade_qtd_na_embalagem.get(chave).get('qtd') or 1
                        resultado[_grade_id]['saldo_na_embalagem'][produto_emb_id]['qtd_reservada'] = \
                                                                             _qtd_res / qtd_na_embalagem
                        resultado[_grade_id]['saldo_na_embalagem'][produto_emb_id]['disponivel'] = \
                                                                           disponivel / qtd_na_embalagem

        modulo_kit_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'kit', context)
        if modulo_kit_instalado:
            _sql = """SELECT epg.id                         AS kit_grade_id,
                             kp.kitpro_grade_id             AS item_kit_grade_id,
                             sum(kp.kitpro_quantidade)      AS qtd,
                             kp.kitpro_produto_embalagem_id AS prod_emb_id,
                             kp.kitpro_item_produto_id      AS item_produto_id,
                             kp.kitpro_produto_id           AS produto_id
                      FROM kit_produto                 AS kp
                      INNER JOIN estoque_produto       AS ep  ON ep.id = kp.kitpro_produto_id
                      INNER JOIN estoque_produto_grade AS epg ON epg.produto_id = ep.id
                      WHERE epg.id IN %s 
                        AND ep.kit_de_produtos IS TRUE
                      GROUP BY epg.id, kp.kitpro_grade_id, kp.kitpro_produto_embalagem_id, 
                               kp.kitpro_item_produto_id, kp.kitpro_produto_id
                      ORDER BY epg.id, qtd DESC
                   """ % str_grade_ids
            cr.execute(_sql)
            res = cr.dictfetchall()
            if res:
                produto_ids = set()
                item_produto_ids = set()
                itens_do_kit_grade_ids = []
                for i in res:
                    produto_ids.update({i.get('produto_id')})
                    item_produto_ids.update({i.get('item_produto_id')})
                    itens_do_kit_grade_ids.append(i.get('item_kit_grade_id'))

                msg_erro = []
                erro_produto_ids = list(produto_ids.intersection(item_produto_ids))
                # verifica kits motadados com itens que sao kits
                if erro_produto_ids:
                    read_produto_ids = []
                    for i in res:
                        if i.get('item_produto_id') in erro_produto_ids:
                            read_produto_ids.append(i.get('produto_id'))

                    for i in self.pool.get('estoque.produto').read(cr, uid, read_produto_ids,
                                                                   ['name', 'codigo'], context):
                        msg_erro.append('%s - %s' % (i.get('codigo'), i.get('name')))

                    if msg_erro:
                        raise osv.except_osv(_('Atencao!'),
                                             _('Os kits a seguir contem itens que sao kits\n %s') %
                                               '\n'.join(msg_erro))

                saldo_itens_do_kit = self.get_saldo_do_dia(cr, uid, itens_do_kit_grade_ids, eps_data,
                                      eps_local_id=eps_local_id, company_id=company_id, context=context)
                # limpa o saldo dos kits
                for i in res:
                    resultado[i.get('kit_grade_id')] = {'saldo': 0,
                                                        'qtd_reservada': 0,
                                                        'disponivel': 0,
                                                        'saldo_na_embalagem': {'saldo': 0,
                                                                               'qtd_reservada': 0,
                                                                               'disponivel': 0}}
                kit_grade_ids = []
                for i in res:
                    posicao_zero = any([res.index(i) == 0,
                                        i.get('kit_grade_id') not in kit_grade_ids])
                    item_grade_id = i.get('item_kit_grade_id')
                    if context.get('coletor_estoque_saldo'):
                        item_grade_id = str(item_grade_id)
                    if i.get('kit_grade_id') not in kit_grade_ids:
                        kit_grade_ids.append(i.get('kit_grade_id'))
                    if saldo_itens_do_kit.get(item_grade_id).get('saldo_na_embalagem') and \
                        saldo_itens_do_kit.get(item_grade_id).get('saldo_na_embalagem'
                                                                  ).get(i.get('prod_emb_id')):
                        saldos = saldo_itens_do_kit.get(item_grade_id).get('saldo_na_embalagem'
                                                                         ).get(i.get('prod_emb_id'))
                    else:
                        saldos = saldo_itens_do_kit.get(item_grade_id)

                    qtd = i.get('qtd') or 1
                    disponivel = int(saldos.get('disponivel') / qtd)
                    qtd_reservada = int(saldos.get('qtd_reservada') / qtd)
                    saldo = int(saldos.get('saldo') / qtd)
                    if resultado.get(i.get('kit_grade_id')).get('saldo') > saldo or posicao_zero:
                        resultado[i.get('kit_grade_id')]['saldo'] = saldo

                    if resultado.get(i.get('kit_grade_id')).get('qtd_reservada') > qtd_reservada or\
                            posicao_zero:
                        resultado[i.get('kit_grade_id')]['qtd_reservada'] = qtd_reservada

                    if resultado.get(i.get('kit_grade_id')).get('disponivel') > disponivel or \
                            posicao_zero:
                        resultado[i.get('kit_grade_id')]['disponivel'] = disponivel

        # o coletor nao aceita dicionario com chave integer, aceita apenas string
        if context.get('coletor_estoque_saldo'):
            aux = {}
            for chave in resultado:
                aux['%s' % chave] = resultado.get(chave)
                aux_emb = {}
                for chave_emb in resultado.get(chave).get('saldo_na_embalagem'):
                    aux_emb['%s' % chave_emb] = resultado.get(chave).get('saldo_na_embalagem').get(
                                                                                          chave_emb)
                aux['%s' % chave]['saldo_na_embalagem'] = aux_emb.copy()
            resultado = aux.copy()
        return resultado
