# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
import time
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta


class estoque_utils(osv.Model):
    _name = 'estoque.utils'
    _auto = False

    def estutl_dict_produto_name(self, cr, uid, produto_ids, context=None):
        context = context and context.copy() or {}
        produto_name = self.pool.get('estoque.produto').name_get(cr, uid, produto_ids, context)
        if produto_name:
            return dict(produto_name)
        else:
            return {}

    # noinspection PyBroadException
    @staticmethod
    def estutl_calcular_margem(preco_venda, encargos, preco_custo):
        """
        Calcular margem de lucro

        @param preco_venda: {float} o preco de venda do produto,
        @param encargos: {float} o somatorio do percentual de encargos incidentes no preco de venda,
        @param preco_custo: {float} o preco de custo do produto.

        @return: {float} margem calculada, se nao conseguir calcular retorna zero.
        """

        try:
            encargos = encargos * preco_venda / 100
            margem_lucro = ((preco_venda - encargos - preco_custo) / preco_venda) * 100
        except:
            margem_lucro = 0.0

        result = round(margem_lucro, 2)

        return result

    @staticmethod
    def estutl_calcular_preco_de_venda(preco_custo, encargos, margem_lucro):
        """
        Calcula o preco de venda do produto em funcao do custo encargos e margem de lucro.

        @param preco_custo: {float} o preco de custo do produto,
        @param encargos: {float} Percentual de encargos incidentes no preco de venda
        @param margem_lucro: {float} Percentual de margem a ser aplicada para calculo do custo

        @return: {float} preco de venda do produto ou zero se a soma de encargos e margem de lucro
        superar 100%.
        """
        try:
            if (encargos + margem_lucro) < 100:
                    preco_venda = (100 * preco_custo) / (100 - encargos - margem_lucro)
            else:
                preco_venda = 0.0
        except:
            preco_venda = 0.0

        result = round(preco_venda, 2)

        return result

    def get_grade_qtd_embalagem(self, cr, grade_ids, context=None):
        '''
        Esta funcao retona um dict onde a chave eh o id da grade e da embalagem e o valor eh outro
         dict com chaves 'qtd' e 'unidade', contendo os seus valores
        embalagem para aquela grade no local passado.
        :param cr:
        :param grade_id:
        :param local_id:
        :param context:
        :return: dict
        Autor: Felipe
        '''
        res = {}
        if not grade_ids:
            return res

        # Foi feito em sql para nao ficar lento ao criar notas pelo sistem
        sql = """
            SELECT epg.id                                  AS grade_id,
                   COALESCE(ee.quantidade_na_embalagem, 1) AS qtd_na_embalagem,
                   COALESCE(epemb.unidade, '')             AS unidade,
                   epemb.fracao                            AS fracao,
                   epe.id                                  AS produto_embalagem_id
            FROM estoque_embalagem                AS ee
            INNER JOIN estoque_produto_embalagem  AS epe   ON epe.embalagem_id = ee.id
            INNER JOIN estoque_embalagem_unidade  AS epemb ON epemb.id = ee.unidade_id
            INNER JOIN estoque_produto_grade      AS epg   ON epg.produto_id = epe.produto_id
            WHERE epg.id IN ({grade_ids})
        """.format(grade_ids=','.join(map(lambda x: '%s' % x, grade_ids)))
        cr.execute(sql)
        map(lambda x: res.update({'%s-%s' %
                                 (x.get('grade_id'),
                                  x.get('produto_embalagem_id')): {'qtd': x.get('qtd_na_embalagem'),
                                                      'unidade': x.get('unidade') or '',
                                                      'fracao': x.get('fracao')}
                                 }), cr.dictfetchall())
        return res

    def estutl_get_custo_hoje(self, cr, uid, custo_corrigido, context=None):
        """
        Parametro: custo_corrigido e uma lista de dicionarios que deve conter
        produto_id e o custo_corrigido
        Sera retornado um dicionario de dicionarios onde a chave sera o produto_id
        """
        context = context and context.copy() or {}
        parametro_indexador = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque',
                    'indexador_id', context=context)
        indexador_id = parametro_indexador.get('indexador_id')
        res = {}
        if indexador_id:
            sigla = self.pool.get('adm.moeda').browse(cr, uid, indexador_id).sigla
            dia_atual = time.strftime('%d')
            mes_atual = time.strftime('%m')
            ano_atual = time.strftime('%Y')
            valor_moeda_indexador = self.pool.get('adm.moeda.valor').achar_valor_moeda(cr, uid, sigla, ano_atual,
                mes_atual, dia_atual, '<=', context=context)
            valor_moeda_indexador = valor_moeda_indexador.get('valor')
            for i in custo_corrigido:
                if valor_moeda_indexador and isinstance(valor_moeda_indexador, float):
                    _custo_corrigido = i.get('estprc_custo_corrigido') * valor_moeda_indexador
                else:
                    _custo_corrigido = i.get('estprc_custo_corrigido')
                i['estprc_custo_corrigido'] = _custo_corrigido
                res[i.get('produto_id')] = i
        else:
            for i in custo_corrigido:
                res[i.get('produto_id')] = i
        return res

    def estutl_calcular_porcentagem_encargos_produto(self, cr, uid, produto_id, context=None):

        """
        Calcula a porcentagem total de encargos para um determinado produto, considerando suas
        tributacoes, comissao e custo fixo.
        @produto_id: Numero do registro do produto (tabela estoque_produto)
        @return: Valor percentual (float)
        """

        context = context and context.copy() or {}
        res = 0.0
        if not produto_id:
            return res
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_produto = self.pool.get('estoque.produto')
        # DADOS DO PARAMETRO
        fields_to_read = ['comissoes', 'custo_fixo']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, tabela='estoque', campos=fields_to_read, context=context
        )
        if not estoque_parametro_dados:
            raise osv.except_osv(_('Atencao'), _('Parametro do estoque nao cadastrado!'))
        # CONFIGURACOES DO PARAMETRO
        comissoes = estoque_parametro_dados.get('comissoes') or 0
        custo_fixo = estoque_parametro_dados.get('custo_fixo') or 0
        # PORCENTAGEM DAS TRIBUTACOES
        valor_encargos = 0.0
        fiscal_instalado = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'fiscal', context)
        if fiscal_instalado:
            tributacao_ids = obj_estoque_produto.browse(cr, uid, produto_id).tributacao_ids
            if tributacao_ids:
                for x in tributacao_ids:
                    if x.tipo_tributacao == 'T':
                        base = x.tributacao_id.base or 0.0
                        aliquota = x.tributacao_id.aliquota or 0.0
                        valor_encargos += (base * aliquota) / 100
        # TOTAL DE ENCARGOS
        res = comissoes + custo_fixo + valor_encargos
        return res

    def estutl_get_estatistica_movimento(self, cr, uid, company_id, produto_id, grade_id,
                                         context=None):
        """
        O metodo encontra os dados totais (quantidade) de movimento (entrada e saida) por data.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param company_id: id da tabela res_users, identificador da empresa.
        :type company_id: int
        :param produto_id: id da tabela estoque_produto, identificador do produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade, identificador da grade do produto.
        :type grade_id: int
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados com a totalizacao das entradas e saidas, por data.
        :rtype: list[dict]
        """
        context = context and context.copy() or {}
        # Busca empresas filhas
        company_ids = self.pool.get('res.company').search(
            cr, uid, [('parent_id', 'child_of', company_id)], context=context)
        obj_adm_util = self.pool.get('adm.utils')
        madeireira_instalado = obj_adm_util.admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        ecf_instalado = obj_adm_util.admutl_check_modulo_instalado(
            cr, uid, 'ecf', context)
        sql = """
            SELECT emc.data AS data_movimento,
                SUM(
        """

        if madeireira_instalado:
            if ecf_instalado:
                sql += """
                CASE WHEN eno.estnto_tipo_operacao = 'S' and emi.estmvi_volume > 0 THEN
                   CASE WHEN ecc.tipo_cupom = 'C' THEN
                       -emi.estmvi_volume
                   ELSE
                       emi.estmvi_volume
                   END
                ELSE
                   CASE WHEN ecc.tipo_cupom = 'C' THEN
                       -emi.quantidade
                   ELSE
                       emi.quantidade
                   END
                """
            else:
                sql += """
                CASE WHEN eno.estnto_tipo_operacao = 'S' and emi.estmvi_volume > 0 THEN
                   emi.estmvi_volume
                ELSE
                   emi.quantidade
                """
        else:
            if ecf_instalado:
                sql += """
                CASE WHEN eno.estnto_tipo_operacao = 'S' THEN
                   CASE WHEN ecc.tipo_cupom = 'C' THEN
                       -emi.quantidade
                   ELSE
                       emi.quantidade
                   END
                ELSE
                   0
                """
            else:
                sql += """
                CASE WHEN eno.estnto_tipo_operacao = 'S' THEN
                   emi.quantidade
                ELSE
                   0
                """

        sql += """
               END)                               AS total_saida,
               SUM(CASE WHEN eno.estnto_tipo_operacao = 'E' THEN
                       emi.quantidade
                   ELSE
                       0
                   END)                           AS total_entrada
            FROM   estoque_movimento_itens        AS emi
            INNER  JOIN estoque_movimento_capa    AS emc ON emc.id = emi.capa_id
            INNER  JOIN estoque_natureza_operacao AS eno ON emc.estmvc_operacao_id = eno.id
            """

        if ecf_instalado:
            sql += """
            LEFT   JOIN ecf_cupons_capa           AS ecc ON ecc.id = emi.cupom_id
            """

        sql += """
            WHERE  emc.company_id IN (%s)
              AND  emc.estmvc_operacao_id IN (
                    SELECT id
                    FROM   estoque_natureza_operacao
                    WHERE  estnto_gerar_estatistica = True
                         AND  active = True
              )
                AND  emi.cfop_id IN (
                    SELECT id
                    FROM   fiscal_cfop
                    WHERE  tipo in ('E','S')
                )
                AND emi.produto_id = %s
                AND emi.grade_id = %s
                AND emc.state = 'F'
        """ % (', '.join(map(lambda c: str(c), company_ids)), produto_id, grade_id)
        if ecf_instalado:
            sql += """
                AND not (emi.item_cancelado is True AND ecc.tipo_cupom = 'V')
            """
        sql += """
            GROUP BY emc.data
        """
        cr.execute(sql)
        estatistica_dados = cr.dictfetchall()
        return estatistica_dados

    def estutl_get_estatistica_movimento_saida_diario(self, cr, uid, company_id, produto_id,
                                                      grade_id, context=None):
        """
        O metodo encontra os dados totais (quantidade) de movimento de saida por dia.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param company_id: id da tabela res_users, identificador da empresa.
        :type company_id: int
        :param produto_id: id da tabela estoque_produto, identificador do produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade, identificador da grade do produto.
        :type grade_id: int
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados com o numero do dia, descricao do dia da semana e totalizacao das saidas.
        :rtype: dict[list]
        """
        context = context and context.copy() or {}
        dict_dia_semana = {0: 'Segunda-feira', 1: _('Terca-feira'), 2: 'Quarta-feira',
                           3: 'Quinta-feira', 4: 'Sexta-feira', 5: _('Sabado'), 6: 'Domingo'}
        estatistica_diaria_dados = dict()
        estatistica_dados = self.estutl_get_estatistica_movimento(
            cr, uid, company_id, produto_id, grade_id, context)
        list_ano_mes = []
        map(lambda ma: list_ano_mes.append(
            '%s-%s' % (ma.get('data_movimento')[:4], ma.get('data_movimento')[5:7])),
            estatistica_dados)
        list_ano_mes = list(set(list_ano_mes))
        for ano_mes in list_ano_mes:
            ano_mes_dados = filter(lambda ma: ma.get('data_movimento')[:7] == ano_mes,
                                   estatistica_dados)
            estatistica_diaria_dados[ano_mes] = list()
            ultimo_dia_ano_mes = ((parse(ano_mes+'-01') + relativedelta(months=1)) -
                                  relativedelta(days=1)).day
            for dia in range(1, ultimo_dia_ano_mes+1):
                if dia < 10:
                    dia = '0'+str(dia)
                else:
                    dia = str(dia)
                ano_mes_dia_dados = filter(lambda am: am.get('data_movimento')[8:] == dia,
                                           ano_mes_dados)
                if ano_mes_dia_dados:
                    data = ano_mes_dia_dados[0].get('data_movimento')
                else:
                    data = ano_mes+'-%s' % dia
                estatistica_diaria_dados[ano_mes].append({
                    'dia': dia,
                    'dia_semana': dict_dia_semana.get(parse(data).weekday()),
                    'quantidade_entrada': sum(
                        map(lambda s: s.get('total_entrada') or 0, ano_mes_dia_dados)),
                    'quantidade_saida': sum(
                        map(lambda s: s.get('total_saida') or 0, ano_mes_dia_dados))})
        return estatistica_diaria_dados
