# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estoque_categoria(osv.Model):
    """
    Tabela de Categoria de produtos (Segmentacao)
    """
    _name = 'estoque.categoria'
    _parent_store = True

    from funcoes import _estcat_get_codigo, _estcat_tem_filho, _estcat_nome_completo

    _columns = {
        'parent_id': fields.many2one("estoque.categoria", 'Categoria Pai', ondelete="cascade", select=True),
        'name': fields.char('Nome Categoria', size=50, required=True), # Descricao, nome da categoria
        'parent_left': fields.integer('Parent left', select=True), # Categoria ascendente
        'parent_right': fields.integer('Parent right', select=True), # Categoria descendente
        'child_ids': fields.one2many('estoque.categoria', 'parent_id', 'Categorias filhas'),
        'produto_ids': fields.one2many('estoque.produto', 'categoria_id', 'Produtos'),
        'conta': fields.function(_estcat_get_codigo, method=True, string='Nivel', type='char', store=True, size=20,
                                 select=True),
        'controle': fields.integer('Codigo de Controle'), # Controla a criacao dos codigos das contas ex: 2.2.4
        'verifica_filho': fields.function(_estcat_tem_filho, method=True, string='Contem filho', type='boolean'),
        'quantidade': fields.float('Quantidade', digits=(9, 3), required=False),
        'estcat_centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de Custos', ondelete='cascade',
                                                  help="'Informe a conta para classificar o centro de custos nas emissoes de nfs/cupons."
                                                       "\nEste centro de custo ira sobrepor ao de vendas cadastrado no parametro do estoque."
                                                       "\nTambem sera sobreposto se houver centro de custo cadastrado na natureza da operacao fiscal."),
        'margem_bruta': fields.float('Margem Bruta', digits=(7, 3)), # Nao busque este valor diretamente, use o metodo estpro_get_margens
        'margem_ideal': fields.float('Margem Ideal', digits=(7, 3)), # Nao busque este valor diretamente, use o metodo estpro_get_margens
        'margem_minima': fields.float('Margem Minima', digits=(7, 3)), # Nao busque este valor diretamente, use o metodo estpro_get_margens
        'nome_completo': fields.function(_estcat_nome_completo, method=True, type="char", string='Descricao completa'),
        'estcat_percentual_comissao': fields.float('% Comissao', digits=(5, 3), help="Percentual de comissao a ser aplicado na venda do produto/servico."),
    }

    _order = 'conta, parent_id'
