# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import esgccp_get_dias_mes_atual


class estsug_compra_capa(osv.Model):
    # prefixo: esgccp_
    _name = 'estsug.compra.capa'
    _gerar_logs_logs = 'N'

    def esgccp_get_selection_codigo_imprimir(self, cr, uid, context=None):
        """
        O metodo disponibiliza as opcoes possiveis para impressao do codigo que representa o produto
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: lista com os valores de cada tipo de impressao.
        :rtype: list
        """
        context = context and context.copy() or {}
        res = [('I', 'Item'), ('F', 'Fabricante')]
        return res

    _columns = {
        # ------------ INTEGER ------------
        'id': fields.integer('Id', readonly=True),
        'esgccp_prazo_sugestao': fields.integer('Prazo estoque para sugestao', required=True,
                                                help='Prazo em dias'),
        'esgccp_prazo_media': fields.integer('Prazo para calculo em media', required=True,
                                             help='Prazo em meses'),
        'esgccp_dias_mes_atual': fields.function(esgccp_get_dias_mes_atual, method=True,
                                                 string='Dias do mes atual', type='integer'),

        # ------------ CHAR ---------------
        'name': fields.char('Descricao', size=60, required=True),
        'esgccp_name_arquivo_csv': fields.char('Nome arquivo', size=19),

        # ------------ MANY2ONE ------------
        'esgccp_usuario_id': fields.many2one('res.users', 'Usuario', ondelete='cascade',
                                             readonly=True),
        'esgccp_fornecedor_id': fields.many2one('financeiro.fornecedor', 'Fornecedor'),
        'esgccp_representante_id': fields.many2one('financeiro.representante', 'Representante'),
        'esgccp_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'esgccp_local_id': fields.many2one('estoque.local', 'Local'),
        'esgccp_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),

        # ------------ DATETIME ------------
        'write_date': fields.datetime('Data sugestao compra', readonly=True),

        # ------------ ONE2MANY ------------
        'esgccp_item_ids': fields.one2many('estsug.compra.item', 'esgcit_capa_id', 'Itens'),

        # ------------ SELECTION ------------
        'esgccp_ordem': fields.selection([('O', 'Codigo'), ('D', 'Descricao'), ('U', 'Curva')],
                                         'Ordem a listar', size=1),
        'esgccp_sugestao': fields.selection([('M', 'Minimo/Maximo'), ('E', 'Estatistica')],
                                            'Sugestao', size=1, required=True),
        'esgccp_codigo_imprimir': fields.selection(esgccp_get_selection_codigo_imprimir,
                                                   'Codigo para impressao', size=1, required=True),
        'esgccp_curva': fields.selection([('A', 'Classe A'), ('B', 'Classe B'), ('C', 'Classe C'),
                                          ('T', 'Todas')], 'Curva', size=1),
        'esgccp_custo': fields.selection([('C', 'Custo corrigido'), ('M', 'Custo medio contabil'),
                                          ('R', 'Custo de reposicao'), ('U', 'Ultimo custo')],
                                         'Custo', size=1),

        # ------------ BOOLEAN ------------
        'esgccp_mes_atual': fields.boolean('Considerar mes atual'),
        'esgccp_lista_estatistica': fields.boolean('Listar estatisticas'),
        'esgccp_considera_empresa': fields.boolean('Considerar todas empresas'),
        'esgccp_gerar_por_empresa': fields.boolean('Gerar por empresa'),
        'esgccp_considera_pedidos': fields.boolean('Considerar pedidos compra',
                                                   help='Deduz os pedidos ja realizados na sugestao'),
        'esgccp_so_sugestao': fields.boolean('Considerar so sugestao'),
        'esgccp_ajusta_media': fields.boolean(
            'Ajustar media', help='Se marcado, considera como media somente o periodo em que o item'
                                  ' tinha saldo.'),

        # ------------ BINARY -------------
        'esgccp_arquivo_csv': fields.binary('Arquivo CSV', readonly=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True,
                                      select=True), # id da empresa\n
    }

    _defaults = {
        'esgccp_usuario_id': lambda s, cr, uid, c: uid,
        'esgccp_ordem': lambda * a: 'O',
        'esgccp_sugestao': lambda * a: 'E',
        'esgccp_codigo_imprimir': lambda * a: 'I',
        'esgccp_curva': lambda * a: 'T',
        'esgccp_considera_pedidos': lambda * a: True,
        'esgccp_so_sugestao': lambda * a: True,
        'esgccp_name_arquivo_csv': 'sugestao_compra.csv',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                                        s._name,
                                                                                        context=c),
    }

    _order = 'id DESC'
