# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estoque_movimento_itens(osv.Model):
    from funcoes import _estmvi_get_local_venda_padrao
    from funcoes import _estmvi_get_dados_stored_true, _estmvi_get_dados_stored_false

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.__dict__.update(entries)

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or {}
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in ['financeiro', 'fiscal', 'estoque', 'adm',
                                                         'rh']])
        return self.ParamToObject(**parametros)

    _name = 'estoque.movimento.itens'
    _columns = {
        'id': fields.integer('Id', readonly=True),
        'capa_id': fields.many2one("estoque.movimento.capa", 'Capa', ondelete="cascade", required=False, select=True),
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="restrict", required=True, select=True),
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', ondelete="restrict", required=False, select=True,
                                    domain="[('produto_id', '=', produto_id)]"), # Id do produto a ser pedido\n
        'local_id': fields.many2one("estoque.local", 'Local', ondelete="restrict", required=True, select=True),
        'produto_embalagem_id': fields.many2one("estoque.produto.embalagem", 'Embalagem', ondelete="restrict",
                                                required=True, select=True,
                                                domain="[('produto_id', '=', produto_id)]"), # id da embalagem pedida
        'quantidade': fields.float('Quantidade', digits=(12, 3), required=True), # Quantidade movimentada
        'name': fields.function(_estmvi_get_dados_stored_true, method=True, string='name',
                                multi='dados', type='text', store=True),
        'create_date': fields.datetime('Data Criacao'),
        'write_date': fields.datetime('Data Compra'),
        'locacao_id': fields.many2one('estoque.local.locacao', 'Locacao', required=False, ondelete='restrict', select=True,
                                      domain="[('produto_id','=',produto_id),('grade_id','=',grade_id),('embalagem_id','=',produto_embalagem_id)]"),
        'estmvi_saidas_custo_corrigido_total': fields.float('Custo Corrigido', digits=(15, 2), oldname="custo_corrigido"), # Custo corrigido do produto(medio)
        'estmvi_saidas_custo_medio_contabil_total': fields.float('Custo Medio Contabil', digits=(15, 2), oldname="custo_medio_contabil"), # Custo medio contabil
        'estmvi_saidas_ultimo_custo_total': fields.float('ultimo Custo', digits=(15, 2), oldname="ultimo_custo"), # Ultimo custo. Custo da ultima nota
        'estmvi_saidas_custo_reposicao_total': fields.float('Custo Reposicao', digits=(15, 2), oldname="custo_reposicao"), # Custo de reposicao
        'estmvi_entrada_preco_custo_id': fields.many2one("estoque.preco.custo", 'Preco de custo', oldname="estmvi_preco_custo_id"),
        'estmvi_saldo': fields.function(_estmvi_get_dados_stored_false, method=True, string='Saldo',
                                        type='float', digits=(15, 3), store=False, multi='dados_store_false'),
        'estmvi_reservada': fields.function(_estmvi_get_dados_stored_false, method=True,
                                            string='Qtd. reservada', type='float', digits=(15, 3),
                                            store=False, multi='dados_store_false'),
        'estmvi_percentual_comissao': fields.float('% Comissao', digits=(5, 3),help="Percentual de comissao a ser aplicado no momento da venda na hierarquia Item/categoria/vendedor/parametro do estoque."),
        'estmvi_vr_comissao': fields.float('Valor da Comissao', digits=(5, 6),help="Valor calculado para comissao armazenado nas vendas obedecendo a hierarquia de % no Item/Categoria/Vendedor/Parametro Estoque."),
        'estmvi_operacao_custo_corrigido_unitario': fields.function(
            _estmvi_get_dados_stored_true, 'Custo Corrigido', type='float', digits=(15, 2),
            store=True, help='Custo corrigido unitario da operacao', multi='dados'),
        'estmvi_operacao_custo_medio_contabil_unitario': fields.function(
            _estmvi_get_dados_stored_true, 'Custo Medio', type='float', digits=(15, 2), store=True,
            multi='dados', help='Custo medio contabil unitario da operacao'),
        'estmvi_operacao_ultimo_custo_unitario': fields.function(
            _estmvi_get_dados_stored_true, 'Ultimo custo', type='float', digits=(15, 2), store=True,
            multi='dados', help='Ultimo custo unitario da operacao'),
        'estmvi_operacao_custo_reposicao_unitario': fields.function(
            _estmvi_get_dados_stored_true, 'Custo Reposicao', type='float', digits=(15, 2),
            store=True, multi='dados', help='Custo de reposicao unitario da operacao'),
    }
    _defaults = {
        'local_id': _estmvi_get_local_venda_padrao,
        'quantidade': 1
    }

    _fields_no_store = [
        'estmvi_operacao_custo_corrigido_unitario', 'estmvi_operacao_custo_medio_contabil_unitario',
        'estmvi_operacao_ultimo_custo_unitario', 'estmvi_operacao_custo_reposicao_unitario', 'name'
        ]  # Nao serao computados ao subir a base

    def _update_store(self, cr, f, k):
        """
        Este metodo computa os campos funcionais storiados novos ou que tiveram sua
        estrutura alterada. Caso nao queira que o novo campo funcioal seja atualizado
        ao levantar o modulo, e necessario adicionar o atributo funcional na
        lista _fields_no_store
        :param cr: a linha atual do cursor do banco de dados
        :param f: Objeto da classe functions
        :param k: Campo funcional a ser computado
        :type cr: object
        :type f: object
        :type k: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if k not in self._fields_no_store:
            super(estoque_movimento_itens, self)._update_store(cr, f, k)
