# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estoque_parametro(osv.Model):
    """
    Tabela de parametros de produtos
    """
    _name = 'estoque.parametro'
    _rec_name = 'company_id'
    _description = 'Tabela de parametros do estoque'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # Id da empresa
        'usa_digito': fields.boolean('Usa digito'),
        'codigo_numerico': fields.boolean('Codigo Numerico'),
        'juros': fields.float('Taxa de Deflacao', digits=(5, 3),
                              help='% Juros para deflacionar os produtos quando adquiridos a prazo.'),
        'indexador_id': fields.many2one("adm.moeda", 'Indexador', ondelete="restrict", required=True, select=True,
                                        help='Indexador para deflacionar custos'),
        'comissoes': fields.float('% Comissoes', digits=(5, 3)),
        'custo_fixo': fields.float('% Custo Fixo', digits=(5, 3)),
        'usar_aplicacao': fields.boolean('Usa Aplicacao',
                                         help="Habilita o campo Aplicacao no cadastro de itens do estoque"),
        'usar_similaridade': fields.boolean('Usa Similaridade'),
        'custo_inventario': fields.selection([('U', 'Ultimo Custo'), ('C', 'Custo Medio Contabil')],
                                             'Custo para Inventario', required=True, size=1),
        'local_venda_padrao_id': fields.many2one("estoque.local", 'Local para saidas', ondelete="restrict"),
        'estpar_descricao_grade': fields.selection([('P', 'Produto + Grade'), ('G', 'Apenas grade')],
                                                   'Descricao da grade'),
        'estpar_local_compra_padrao_id': fields.many2one('estoque.local', 'Local para entradas', ondelete='restrict'),
        'estpar_entradaproducao_operacao_id': fields.many2one('estoque.natureza.operacao',
                                                              'Operacao para entrada de producao'),
        'estpar_inventario_operacao_entrada_id': fields.many2one('estoque.natureza.operacao',
                                                                 'Operacao de entrada para inventario',
                                                                 oldname='estpar_inventario_operacao_id'),
        'estpar_inventario_operacao_saida_id': fields.many2one('estoque.natureza.operacao',
                                                               'Operacao de saida para inventario'),
        'estpar_incluir_frete_no_custoreposicao': fields.boolean('Incluir frete no custo reposicao'),
        'estpar_incluir_ipi_no_custoreposica': fields.boolean('Incluir Ipi no custo reposicao'),
        'estpar_custo_para_precovenda': fields.selection(
            [('U', 'Ultimo Custo'), ('M', 'Medio Contabil'), ('R', 'Reposicao'), ('C', 'Corrigido')],
            'Custoparaprecodevenda', size=1), # U=Ultimo custo, M=Medio Contabil, R=Reposicao, C=Corrigido
        'estpar_cadastrar_custo': fields.boolean('Permite cadastrar custo'),
        'estpar_custo_entrada_producao': fields.selection(
            [('U', 'Ultimo custo'), ('C', 'Custo medio contabil'), ('O', 'Custo corrigido'),
             ('R', 'Custo de reposicao')], 'Custo entrada producao', size=1),
        'estpar_custificar_por_produto': fields.boolean('Custificar por produto',
                                                        help='Se marcado na custificacao do financeiro o produto sera identificado'),
        'estpar_margem_para_preco_venda': fields.selection([('I', 'Ideal'), ('B', 'Bruta')],
                                                           'Margem para preco de venda', size=1, required=True),
        'estpar_comercializa_combustivel_lubrificante': fields.boolean('Comercializa Combustivel e Lubrificantes',
                                                                       help='Marque este parametro caso a empresa va comercializar Combustiveis e Lubrificantes'),
        'estpar_unidade_servico_id': fields.many2one('estoque.embalagem.unidade', 'Unidade para servico'),
        'estpar_embalagem_servico_id': fields.many2one('estoque.embalagem', 'Embalagem para Servicos'),
        'estpar_pvenda_menor_pcusto': fields.boolean('Preco venda menor preco custo',
                                                     help='Se marcado, permite usar preco de venda inferior ao preco de custo!'),
        'estpar_tamanho_codigo': fields.integer('Tamanho Codigo Produto', required=True),
        'estpar_custo_cmv':fields.selection([(1, 'Ultimo Custo'),
                                             (2, 'Custo Medio'),
                                             (3, 'Custo Corrigido')], 'Custo para CMV', size=1),
        'estpar_forma_execucao_curva': fields.selection(
            [('M', 'Manual'), ('A', 'Automatica')], size=1, string='Curva ABC',
            help='Determina a forma de execucao da curva, manual/automatica'),
        'estpar_tipo_curva': fields.selection(
            [('V', 'Venda'), ('E', 'Estoque')], size=1, string='Curva de venda ou estoque',
            help='Define o local de busca dos dados para a classificacao da curva ABC'),
        'estpar_periodo': fields.integer('Periodo (dias)', help='Periodo para analise das vendas'),
        'estpar_primeiro_percentual': fields.integer('Primeiro percentual (A/B)',
                                                     help='Primeiro percentual de separacao faixa '
                                                          'A -> B'),
        'estpar_segundo_percentual': fields.integer('Segundo percentual (B/C)',
                                                    help='Segundo percentual de separacao faixa '
                                                         'B -> C'),
        'estpar_tipo_analise': fields.selection(
            [('L', 'Lucro'), ('V', 'Venda')], size=1, string='Analisar curva/venda',
            help='Analise da curva por contribuicao da margem de lucro ou quantidade vendida'),
        'estpar_inventario_tipo_contagem': fields.selection(
            [('I', 'Incrementar'), ('S', 'Substituir')], 'Tipo de contagem', size=1,
            help='Identifica se as quantidades lancadas na digitacao do inventario vao incrementar '
                 'ou substituir as quantidades ja digitadas.'),
    }
    _defaults = {
        'codigo_numerico': False,
        'estpar_tamanho_codigo': 6,
        'usar_aplicacao': False,
        'usar_similaridade': False,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'estpar_descricao_grade': 'P',
        'estpar_margem_para_preco_venda': 'I',
        'estpar_custo_cmv': 3,
        'estpar_forma_execucao_curva': lambda *a: 'A',
        'estpar_tipo_curva': lambda *a: 'V',
        'estpar_periodo': lambda *a: 90,
        'estpar_primeiro_percentual': lambda *a: 80,
        'estpar_segundo_percentual': lambda *a: 95,
        'estpar_tipo_analise': lambda *a: 'V',
    }

    _sql_constraints = [
        ('company_id_uniq', 'unique (company_id)', 'A empresa precisa ser unica!'),
    ]
