# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class estoque_produto(osv.Model):
    _name = 'estoque.produto'
    _description = 'Cadastro de produtos.'

    from funcoes import _estpro_preco_venda_produto, _estpro_saldo_produto, estpro_function_get_estparametro
    from funcoes import _estpro_get_parametro_comercializa_combustivel_lubrificante
    from funcoes import _estpro_verifica_pcp_instalado, _estpro_get_unidade_para_inventario
    from funcoes import _estpro_check_cadastro_produto, _estpro_get_codigo_fornecedor
    from funcoes import _estpro_search_codigo_fornecedor
    from funcoes import estpro_get_regime_tributario

    _roteavel = True
    _columns = {
        'id': fields.integer(),
        'create_uid': fields.integer(),
        'create_date': fields.datetime('Data de criacao'),
        'write_date': fields.datetime('Data ultima alteracao', select=True),
        'tipo': fields.selection([('P', 'Produto'), ('S', 'Servico')], select=True),
        'name': fields.char('Descricao', size=60, required=True, oldname="descricao"), # Descricao do produto
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # id da empresa\n
        'fabricante_id': fields.many2one("adm.pessoa", 'Fabricante', select=True, ondelete="restrict"),
        'codigo_fabricante': fields.related('produto_grade_ids', 'codigo_no_fabricante', type="char",
                                            relation="estoque.produto.grade", store=False),
        'margem_ideal': fields.float('Margem Ideal', digits=(7, 3)),  # Nao busque este valor diretamente, use o metodo estpro_get_margens
        'margem_minima': fields.float('Margem Minima', digits=(7, 3)),  # Nao busque este valor diretamente, use o metodo estpro_get_margens
        'margem_bruta': fields.float('Margem Bruta', digits=(7, 3)),  # Nao busque este valor diretamente, use o metodo estpro_get_margens
        'tempo_reposicao': fields.integer('Tempo Reposicao'),
        'codigo': fields.char('Codigo', size=20, select=True, required=False),
        'grade_usar': fields.boolean('Usa Grade'),
        'estpro_grade_descricao': fields.char('Descricao da grade', size=20),
        'digito': fields.char('Digito', size=1), # Digito de controle do codigo do produto
        'produto_grade_ids': fields.one2many('estoque.produto.grade', 'produto_id', 'Grade'),
        'preco_custo_ids': fields.one2many('estoque.preco.custo', 'produto_id', 'Custo'),
        'preco_venda_ids': fields.one2many('estoque.preco.venda', 'produto_id', 'Venda'),
        'fornecedor_ids': fields.one2many('estoque.fornecedor', 'produto_id', 'Fornecedor'),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria', select=True, ondelete="restrict"),
        'aplicacao': fields.text('Aplicacao do Produto'),
        'produto_embalagem_ids': fields.one2many('estoque.produto.embalagem', 'produto_id', 'Embalagem'),
        'produto_similaridade_ids': fields.one2many('estoque.produto.similaridade', 'produto_original_id',
                                                    'Similaridades'),
        'active': fields.boolean('Ativo'),
        'tipo_item': fields.selection([('00', '00-Mercadoria para Revenda'), ('01', '01-Materia-Prima'),
                                       ('02', '02-Embalagem'), ('03', '03-Produto em Processo'),
                                       ('04', '04-Produto Acabado'), ('05', '05-Subproduto'),
                                       ('06', '06-Produto Intermediario'),
                                       ('07', '07-Material de Uso e Consumo'),
                                       ('08', '08-Ativo Imobilizado'),
                                       ('09', '09-Servicos'),
                                       ('10', '10-Outros insumos'), ('99', '99-Outras')],
                                      'Tipo do item', required=True),
        'validade': fields.integer('Validade', help='Validade do produto em dias'),
        'tempo_previsto': fields.float("Tempo Previsto", help="Tempo previsto para execucao do servico!"),
        'preco_venda': fields.function(_estpro_preco_venda_produto, method=True, string='Preco Venda'),  # store=False devido a possibilidade de ser multiempresa, usando o mesmo cadastro para as empresas e saldo diferente para elas
        'saldo': fields.function(_estpro_saldo_produto, method=True, string='Saldo'),  # store=False devido a possibilidade de ser multiempresa, usando o mesmo cadastro para as empresas e saldo diferente para elas
        'locacao_ids': fields.one2many('estoque.local.locacao', 'produto_id', 'Locacoes'),
        'local_ids': fields.related('locacao_ids', 'local_id', type="one2many", relation="estoque.local"),

        'producao_propria': fields.boolean('Producao Propria'),
        'estpro_parametro_codigo': fields.function(estpro_function_get_estparametro, method=True,
                                                   string="Cod. Parametro Numerico", type="boolean", store=False),
        'estpro_visibilidade_codigo_anp': fields.function(_estpro_get_parametro_comercializa_combustivel_lubrificante,
                                                          method=True, string="Parametro", type="boolean", store=False),
        'estpro_foto': fields.binary('Foto', filters='*.png,*.gif,*.jpeg'),
        'estpro_pcp_instalado': fields.function(_estpro_verifica_pcp_instalado, method=True, string="PCP instalado",
                                                type="boolean", store=False),
        'estpro_unidade_inventario': fields.function(_estpro_get_unidade_para_inventario,
                                                     method=True, string='Unidade inventario',
                                                     type='char', store=True),
        'estpro_percentual_comissao': fields.float('% Comissao', digits=(5, 3),
                                                   help='Percentual de comissao a ser aplicado na venda '
                                                        'do produto/servico.'),
        'estpro_habilidade_id': fields.many2one('rh.habilidade', 'Habilidade', ondelete='restrict',
                                                select=True, help='Habilidade necessaria para '
                                                                  'executar este servico'),
        'estpro_sazonal': fields.boolean('Sazonal', help='Indica se o produto e temporario'),
        'estpro_codigo_fornecedor': fields.function(_estpro_get_codigo_fornecedor, method=True,
                                                    string='Codigo do item no fornecedor', type='char',
                                                    fnct_search=_estpro_search_codigo_fornecedor,
                                                    help='Codigo do produto para um fornecedor '
                                                         'proprio.'),
        'estpro_empresa_regime_tributario': fields.function(
            estpro_get_regime_tributario, method=True, string='Regime tributario', type='char'),
        'estpro_tags_ids': fields.many2many('adm.tags', 'estoque_produto_tags', 'pro_produto_id',
                                            'pro_tag_id', 'Marcas'),
    }
    _defaults = {
        'margem_bruta': lambda *a: 0,
        'active': lambda *a: True,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'estpro_parametro_codigo': lambda s, cr, uid, c: s.estpro_get_estoque_parametro_codigo(cr, uid, None),
    }
    _sql_constraints = [
        ('codigo_uniq', 'unique (codigo, company_id)',
         'O Codigo do Produto precisa ser unico para empresa!'.decode('utf-8')),
    ]

    _constraints = [(_estpro_check_cadastro_produto,
                     _('Produto ja cadastrado para a empresa com essa descricao.'),
                     ['active', 'name', 'company_id'])]

    _order = 'name'

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
