# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estoque_produto_grade(osv.Model):
    """
    Tabela de Grades. Item
    Mesmo que o produto nao utilize grade, sempre tera um registro nesta tabela, pois o nome do produto esta aqui. Neste caso a descricao da grade sera \
    """

    from funcoes import _estpgr_get_dados, _estpgr_get_name

    _name = 'estoque.produto.grade'
    _inherits = {
        'estoque.produto': 'produto_id',
    }

    _columns = {
        'id': fields.integer(),
        'write_date': fields.datetime('Ultima atualizacao', readonly=True),
        'write_uid': fields.many2one('res.users', 'Usuario que atualizou', readonly=True),
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="cascade", select=True, required=True),
        'grade': fields.char('Grade', size=2, required=True), # Grade  do produto
        'active': fields.boolean('Ativo'), # Flag que indica se o produto esta ativo
        'descricao': fields.char('Descricao', size=40, required=True),
        'name': fields.function(_estpgr_get_name, method=True, string='Nome', type='char', size=63, store=True), #Deixar o store=True pois se for passado para False os itens das notas fiscais nao sao exibidos na tela e tambem gera outros erros de leitura de dados.
        'minimo': fields.integer('Minimo'), # Quantidade minima de estoque. Usado para fazer pedido de compras
        'maximo': fields.integer('Maximo'), # Quantidade maxima de estoque. Usado para controlar ate quanto comprar.
        'create_date': fields.datetime('Data Inclusao', required=True), # Data da inclusao do produto
        'descricao_complementar': fields.text('Descricao Complementar', colspan="4"),
        'produto_para_baixa_id': fields.many2one("estoque.produto.grade", 'Produto para Baixa', ondelete="cascade"),
        'produto_grade_ids': fields.one2many('estoque.produto.grade', 'produto_para_baixa_id', 'Grade'),
        'local_locacao_ids': fields.one2many('estoque.local.locacao', 'grade_id', 'Locacao'),
        'local_ids': fields.related('local_locacao_ids', 'local_id', type="many2one", relation="estoque.local",
                                    store=False),
        'produto_saldo_ids': fields.one2many('estoque.produto.saldo', 'grade_id', 'Saldo'),
        'inventario_contagem_ids': fields.one2many('estoque.inventario.contagem', 'grade_id', 'Contagem'),
        'inventario_oficial_ids': fields.one2many('estoque.inventario.oficial', 'grade_id', 'Oficial'),
        'produto_codigo_fornecedor_ids': fields.one2many('estoque.produto.codigo.fornecedor', 'grade_id', 'Fornecedor'),
        'movimento_itens_ids': fields.one2many('estoque.movimento.itens', 'grade_id', 'Itens'),
        'saldo': fields.function(_estpgr_get_dados, multi="saldo", method=True, string='Saldo', type='float',
                                 digits=(16, 3), store=False),
        'codigo_no_fabricante': fields.char('Codigo no fabricante', size=20, select=True),
        'estpgr_qtd_reservada': fields.function(_estpgr_get_dados, multi="saldo", method=True,
                                                string='Quantidade reservada', type='float',
                                                digits=(16, 3), store=False),
        'estpgr_classificacao_curva': fields.selection(
            [('A', 'A'), ('B', 'B'), ('C', 'C'), ('N', 'N')], size=1, string='Classificacao ABC',
            help='Indica em qual curva a grade do produto esta classificada'),
        'estpgr_forma_curva': fields.function(_estpgr_get_dados, multi="saldo", method=True,
                                             string='Tipo curva', type='char', store=False),
    }
    _defaults = {
        'active': lambda *a: True,
        'estpgr_classificacao_curva': lambda *a: 'N',
    }
    _sql_constraints = [
        ('produto_grade_uniq', 'unique (produto_id, grade)', 'A grade precisa ser unica para o produto!'),
    ]


