# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class estoque_estatisticas_mensal_pgview(osv.Model):

    _name = "estoque.estatisticas.mensal.pgview"
    _auto = False
    _columns = {
        'estemp_company_id': fields.many2one('res.company', 'Empresa'),
        'estemp_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'estemp_local_id': fields.many2one('estoque.local', 'Local'),
        'estemp_quantidade_embalagem': fields.float('Quantidade embalagem', digits=(8, 3)),
        'estemp_unidade': fields.char('Unidade', size=3),
        'estemp_grade_id': fields.many2one('estoque.produto.grade', 'Grade'),
        'estemp_total_entrada': fields.float('Quantidade entrada', digits=(12, 3)),
        'estemp_total_saida': fields.float('Quantidade saida', digits=(12, 3)),
        'estemp_quantidade': fields.float('Quantidade', digits=(12, 3)),
        'estemp_tipo_operacao': fields.char('Tipo operacao', size=1),
        'estemp_data_movimento': fields.date('Data movimento', select=True),
        'estemp_volume': fields.float('Volume', digits=(15, 4)),
    }

    def init(self, cr):
        madeireira_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, 1, 'madeireira', {})
        sql = """
                CREATE OR replace VIEW estoque_estatisticas_mensal_pgview
                AS
                  (SELECT Row_number()
                            over (
                              ORDER BY mov_item.produto_id)         AS id,
                          mov_capa.company_id                       AS estemp_company_id,
                          mov_capa.data                             AS estemp_data_movimento,
                          mov_item.local_id                         AS estemp_local_id,
                          mov_item.produto_id                       AS estemp_produto_id,
                          mov_item.grade_id                         AS estemp_grade_id,
                          mov_item.quantidade                       AS estemp_quantidade,
        """
        if madeireira_instalado:
            sql += """
                          mov_item.estmvi_volume                    AS estemp_volume,
            """
        sql += """
                          natureza_operacao.estnto_tipo_operacao    AS estemp_tipo_operacao,
                          embalagem.quantidade_na_embalagem         AS estemp_quantidade_embalagem,
                          unidade.unidade                           AS estemp_unidade,
                          SUM(CASE
                                WHEN natureza_operacao.estnto_tipo_operacao = 'S' THEN
                                mov_item.quantidade
                                ELSE 0
                              END)                                  AS estemp_total_saida,
                          SUM(CASE
                                WHEN natureza_operacao.estnto_tipo_operacao = 'E' THEN
                                mov_item.quantidade
                                ELSE 0
                              END)                                  AS estemp_total_entrada
                   FROM   estoque_movimento_itens mov_item
                          inner join estoque_movimento_capa mov_capa
                                  ON mov_item.capa_id = mov_capa.id
                          inner join estoque_natureza_operacao natureza_operacao
                                  ON mov_capa.estmvc_operacao_id = natureza_operacao.id
                          left join estoque_produto_embalagem produto_embalagem
                                 ON mov_item.produto_embalagem_id = produto_embalagem.id
                          left join estoque_embalagem embalagem
                                 ON produto_embalagem.embalagem_id = embalagem.id
                          left join estoque_embalagem_unidade unidade
                                 ON embalagem.unidade_id = unidade.id
                   WHERE  natureza_operacao.estnto_gerar_estatistica = TRUE
                   GROUP  BY mov_capa.data,
                             mov_capa.company_id,
                             mov_item.local_id,
                             mov_item.produto_id,
                             mov_item.grade_id,
                             mov_item.quantidade,
        """
        if madeireira_instalado:
            sql += """
                             mov_item.estmvi_volume,
            """
        sql += """
                             embalagem.quantidade_na_embalagem,
                             natureza_operacao.estnto_tipo_operacao,
                             unidade.unidade)
        """
        drop_view_if_exists(cr, 'estoque_estatisticas_mensal_pgview')
        cr.execute(sql)
