#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.report.interface import report_int
from openerp.report.render import render
import time
import locale
from openerp import pooler
import StringIO
from openerp.tools.translate import _
from SigUtils.impressao import Impressora


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        wizlip_local_ids = False
        wizlip_listar_grades = False
        wizlip_agrupar_por_categoria = False
        wizlip_percentual = False
        wizlip_imprimir_saldo = False
        wizlip_saldo = False
        wizlip_ordenar = False
        filtros = datas.get('filtros')
        for filtro in filtros:
            wizlip_local_ids = filtro.get('wizlip_local_ids')
            wizlip_listar_grades = filtro.get('wizlip_listar_grades')
            wizlip_agrupar_por_categoria = filtro.get('wizlip_agrupar_por_categoria')
            wizlip_alterar_preco = filtro.get('wizlip_alterar_preco')
            wizlip_percentual = filtro.get('wizlip_percentual')
            wizlip_imprimir_saldo = filtro.get('wizlip_imprimir_saldo')
            wizlip_saldo = filtro.get('wizlip_saldo')
            wizlip_ordenar = filtro.get('wizlip_ordenar')

        obj_categoria = pooler.get_pool(cr.dbname).get('estoque.categoria')
        obj_saldo = pooler.get_pool(cr.dbname).get('estoque.produto.saldo')
        obj_est_preco_venda = pooler.get_pool(cr.dbname).get('estoque.preco.venda')
        title = _('Listagem do livro de precos')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()

        listagem.defineFormato(orientacao='1')
        listagem.informaNome(pdf_string, title)
        listagem.informaTamLinha(160)
        listagem.defineFonte(tam=8.6)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'wizard.listagem.livro.preco')
        nome_empresa = pooler.get_pool(cr.dbname).get('res.company').browse(cr, uid,
                                                                            company_id).name
        data = time.strftime("%d/%m/%Y")
        format_categ = '|{:<80}{:<74}|'
        format_fornec = '|{:<80}{:<74}|'
        if wizlip_agrupar_por_categoria:
            det_format = '|{:^16}|{:<50}|{:^4}|{:^3}|{:^35}|{:>20}|{:>20}|'
            det_format_cabecalho = '|{:^16}|{:^50}|{:^4}|{:^3}|{:^35}|{:^20}|{:^20}|'
        else:
            det_format = '|{:^16}|{:<40}|{:^4}|{:^3}|{:^25}|{:<35}|{:>10}|{:>15}|'
            det_format_cabecalho = '|{:^16}|{:^40}|{:^4}|{:^3}|{:^25}|{:^35}|{:^10}|{:^15}|'

        det_format = det_format.decode('utf-8')
        det_format_cabecalho = det_format_cabecalho.decode('utf-8')
        listagem.informaCab(' {:^153} '.decode('utf-8').format(title[:153]))
        listagem.informaCab('')
        listagem.informaCab(' {:<10} {:^132} {:>10} '.decode('utf-8').format(
            data, nome_empresa, _('Pagina: {{ Page }}')))
        listagem.informaCab('')
        listagem.informaCab('*' + '-' * 155 + '*')
        if wizlip_agrupar_por_categoria:
            listagem.informaCab(det_format_cabecalho.format(
                _('COD./DV')[:16], _('DESCRICAO')[:50], _('GR')[:4],
                _('UND')[:3], _('LOCACAO')[:35],
                _('SALDO')[:20], _('PR.VENDA')[:20]))
        else:
            listagem.informaCab(det_format_cabecalho.format(
                _('COD./DV')[:16], _('DESCRICAO')[:40], _('GR')[:4],
                _('UND')[:3], _('LOCACAO')[:25], _('CATEGORIA')[:35],
                _('SALDO')[:10], _('PR.VENDA')[:15]))

        listagem.informaCab('*' + '-' * 155 + '*')
        listagem.rodapePadrao()
        produto_ids = list(set(map(lambda i: i.get('produto_id'), datas.get('dados'))))
        if wizlip_listar_grades:
            grade_ids = list(set(map(lambda i: i.get('grade_id'), datas.get('dados'))))
        else:
            grade_ids = None

        # Informar este context para nao verificar precos especiais para clientes em get_preco_venda
        context.update({'automacao_atualizar_precos': True})
        preco_venda_itens = obj_est_preco_venda.get_preco_venda(cr, uid, produto_ids,
                                                                epc_company_id=None,
                                                                epc_cliente_id=None,
                                                                epc_grade_ids=grade_ids,
                                                                context=context)
        categoria_controle = ''
        fornecedor_controle = ''
        dict_saldo = {}
        if wizlip_imprimir_saldo:
            if wizlip_local_ids:
                for local_id in wizlip_local_ids:
                    dict_saldo[local_id] = obj_saldo.get_saldo_do_dia(cr, uid, grade_ids,
                                                                      eps_local_id=local_id,
                                                                      eps_data=data,
                                                                      context=context)
            else:
                dict_saldo[False] = obj_saldo.get_saldo_do_dia(cr, uid, grade_ids, eps_data=data, context=context)
        produtos_imprecos = []
        for dado in datas.get('dados'):
            produto_id = dado.get('produto_id')
            grade_id = dado.get('grade_id') or False
            # quando nao passar local nos filtro o produto deve ser impresso apenas uma vez
            if not wizlip_local_ids and grade_id in produtos_imprecos:
                continue

            local_id = dado.get('local_id') or False
            if wizlip_local_ids and local_id not in wizlip_local_ids:
                continue

            produtos_imprecos.append(grade_id)
            categoria = dado.get('categoria_nome')
            if dado.get('categoria_id') and wizlip_agrupar_por_categoria:
                categoria = obj_categoria.name_get(cr, uid, [dado.get('categoria_id')],
                                                   context=context)[0][1]
                if categoria_controle != categoria:
                    listagem.imprimeLinha(format_categ.format(' ', ' '))
                    listagem.imprimeLinha(format_categ.format('CATEGORIA: ' +
                                                              categoria.strip()[:80].encode(
                                                                  'utf-8'), ' '))
                    listagem.imprimeLinha(format_categ.format(' ', ' '))
                    categoria_controle = categoria

            if wizlip_ordenar == 'F':
                fornecedor = dado.get('fornecedor') or 'SEM FORNECEDOR'
                if fornecedor != fornecedor_controle:
                    listagem.imprimeLinha(format_fornec.format(' ', ' '))
                    listagem.imprimeLinha(format_fornec.format(
                        'Fornecedor: ' + fornecedor.strip()[:80].encode('utf-8'), ' '))
                    listagem.imprimeLinha(format_fornec.format(' ', ' '))
                    fornecedor_controle = fornecedor

            preco_venda = preco_venda_itens.get(produto_id).get('preco_venda') or 0
            if wizlip_alterar_preco in ('A', 'D'):
                preco_venda = self.alterar_preco_venda(wizlip_alterar_preco, preco_venda,
                                                       wizlip_percentual)
            preco_venda = locale.format('%.2f', preco_venda, grouping=True)[:15]
            saldo_item = ''
            if wizlip_imprimir_saldo:
                if wizlip_local_ids:
                    saldo_item = (dict_saldo.get(local_id).get(grade_id) or {}).get(
                        'disponivel', 0.0)
                else:
                    saldo_item = dict_saldo.get(False).get(grade_id).get('disponivel', 0.0)

                if wizlip_saldo == 'S' and saldo_item != 0:
                    continue
                elif wizlip_saldo == 'C' and saldo_item <= 0:
                    continue
                elif wizlip_saldo == 'N' and saldo_item >= 0:
                    continue
                else:
                    saldo_item = locale.format('%.2f', saldo_item, grouping=True)[:10]

            if not wizlip_agrupar_por_categoria:
                linha = det_format.format(
                    dado.get('produto_codigo')[:16],
                    dado.get('produto_nome')[:40],
                    dado.get('grade_grade')[:4] if wizlip_listar_grades else '',
                    dado.get('unidade')[:3] if dado.get('unidade') else '',
                    dado.get('local_locacao')[:25] if dado.get('local_locacao') else '',
                    categoria[:35],
                    saldo_item,
                    preco_venda)
            else:
                linha = det_format.format(
                    dado.get('produto_codigo')[:16],
                    dado.get('produto_nome')[:50],
                    dado.get('grade_grade')[:4] if wizlip_listar_grades else '',
                    dado.get('unidade')[:3] if dado.get('unidade') else '',
                    dado.get('local_locacao')[:35] if dado.get('local_locacao') else '',
                    saldo_item,
                    preco_venda)
            listagem.imprimeLinha(linha)

        listagem.imprimeLinha('*' + '-' * 155 + '*')
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    @staticmethod
    def alterar_preco_venda(operacao, preco_venda, percentual):
        """
        Metodo para alterar o valor do preco de venda na listagem. O valor alterado nao e
        persistido no banco.

        :param operacao: String informado A para aumentar ou D para diminuir
        :param preco_venda: Preco de venda atual calculado pelo metodo get_preco_venda
        :param percentual: Percentual de alteracao informado pelo usuario no filtro do relatorio
        :type operacao: str
        :type preco_venda: float
        :type percentual: float
        :rtype: float

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        valor_percentual = (preco_venda * percentual) / 100
        if operacao == 'A':
            preco_venda += valor_percentual
        else:
            preco_venda -= valor_percentual
        return preco_venda


report_custom('report.listagem.livro.preco')
