# -*- coding: utf-8 -*-

from SigUtils.impressao import Impressora
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
import locale
import time
from openerp import pooler
# noinspection PyProtectedMember
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_empresa_objeto = pooler.get_pool(cr.dbname).get('adm.empresa.objeto')
        obj_adm_utils = pooler.get_pool(cr.dbname).get('adm.utils')

        title = _('Movimento Sintetico')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()

        listagem.defineFormato(orientacao='1')
        listagem.informaNome(pdf_string, title)
        listagem.informaTamLinha(160)
        listagem.defineFonte(tam=8.6)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        data_inicial = datas.get('data_inicial')
        data_final = datas.get('data_final')
        totaliza_item_igual = datas.get('totaliza_item_igual')

        params = ['estpar_custo_para_precovenda', 'comissoes', 'indexador_id', 'custo_fixo']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', params, context=context)
        custo_parametro = estoque_parametro_dados.get('estpar_custo_para_precovenda')

        comissao = estoque_parametro_dados.get('comissoes') or 0.0
        custo_name = ''

        if custo_parametro == 'C':
            custo_parametro = 'custo_corrigido'
            custo_name = _('CUSTO CORRIGIDO')
        elif custo_parametro == 'U':
            custo_parametro = 'ultimo_custo'
            custo_name = _('ULTIMO CUSTO')
        elif custo_parametro == 'M':
            custo_parametro = 'custo_medio'
            custo_name = _('CUSTO MEDIO CONTABIL')
        elif custo_parametro == 'R':
            custo_parametro = 'custo_reposicao'
            custo_name = _('CUSTO REPOSICAO')

        dados = datas.get('dados')
        pcp_instalado = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'pcp', context)
        madeireira_instalado = obj_adm_utils.admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        list_dados = []
        list_produtos_cm = []
        list_diff = []

        for movimento in dados:
            produto_id = movimento.get('produto_id')
            grade = movimento.get('grade')
            grade_id = movimento.get('grade_id')
            cm = self.get_cm(movimento.get('tipo_movimento'), movimento.get('origem'),
                             movimento.get('cfop_movimento'))

            movimento.update({'cm': cm})

            if not (produto_id, grade, cm) in list_diff:
                list_diff.append((produto_id, grade, cm))
                list_produtos_cm.append(
                    {'produto': produto_id, 'grade_id': grade_id, 'grade': grade, 'cm': cm})

        codigo_produto = grade = name_produto = ''

        for produto in list_produtos_cm:
            entrou = False
            icms = pis = cofins = total_item = quantidade = parametro_custo = custo_para_p_venda = 0
            filt_dados = filter(
                lambda i: produto.get('produto') == i.get('produto_id') and produto.get(
                    'grade') == i.get('grade') and produto.get('cm') == i.get('cm'), dados)

            for movimento in filt_dados:
                entrou = True
                fator_can = 1

                # Se o cupom for de cancelamento 'fator_can' = -1
                if movimento.get('tipo_cupom') == 'C':
                    fator_can = -1

                icms += (movimento.get('vr_icms') or 0.0) * fator_can
                pis += (movimento.get('vr_pis') or 0.0) * fator_can
                cofins += (movimento.get('vr_cofins') or 0.0) * fator_can
                total_item += (movimento.get('vr_total_item') or 0.0) + (movimento.get(
                    'vr_icms_st') or 0.0) + (movimento.get('vr_ipi') or 0.0)
                tipo_movimento = movimento.get('tipo_movimento')

                if madeireira_instalado and tipo_movimento == 'S' and movimento.get('volume'):
                    quantidade += movimento.get('volume') or 0.0
                else:
                    quantidade += movimento.get('quantidade') or 0.0

                custo = movimento.get(custo_parametro) or 0.0
                parametro_custo += round(custo, 4) * fator_can
                custo_para_p_venda += (movimento.get(custo_parametro) or 0.0) * fator_can
                custo_para_p_venda = round(custo_para_p_venda, 4)
                codigo_produto = movimento.get('produto_codigo') or ''
                grade = movimento.get('grade') or ''
                name_produto = movimento.get('produto_name') or ''

            if entrou:
                comissao_item = total_item * (comissao / 100.0)
                custo_fixo = estoque_parametro_dados.get('custo_fixo') or 0.0
                custo_fixo = total_item * (custo_fixo / 100.0)
                tributos = pis + cofins + icms
                custo_mais_encargos = custo_para_p_venda + custo_fixo + comissao_item + tributos
                total_custos = parametro_custo + tributos + comissao_item
                porcentagem_margem = 0.0

                # Se foi infomado apenas um local, a quantidade devera ser mostrada na unidade
                # embalagem do local informado. Se nenhum local informado ou mais de um local for
                # informado,a quantidade devera ser mostrada em unidades simples.
                produto_id = movimento.get('produto_id')
                grade_id = movimento.get('grade_id')

                if datas.get('local_ids').__len__() == 1:
                    unidade, qtde_na_embalagem = self._get_embalagem_qtde_embalagem_local(
                        cr, produto_id, grade_id, datas.get('local_ids')[0])
                else:
                    unidade, qtde_na_embalagem = self._get_embalagem_menor_qtde_embalagem(
                        cr, produto_id, grade_id)

                if qtde_na_embalagem:
                    quantidade /= qtde_na_embalagem
                if total_item:
                    porcentagem_margem = (total_item - total_custos) * 100 / total_item

                vals = {'cm': produto.get('cm'),
                        'codigo_produto': codigo_produto,
                        'grade': grade,
                        'name_produto': name_produto,
                        'quantidade': quantidade or 0.0,
                        'unidade': unidade or '',
                        'custo': parametro_custo or 0.0,
                        'custo_mais_encargos': custo_mais_encargos or 0.0,
                        'total': total_item or 0.0,
                        'margem': porcentagem_margem or 0.0,
                        'categoria_id': movimento.get('categoria_id')
                        }
                list_dados.append(vals)

        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, context=context)
        nome_empresa = pooler.get_pool(cr.dbname).get('res.company').browse(
            cr, uid, company_id, context).name
        local_ids = datas.get('local_ids')
        locais = ', '.join(map(
            lambda m: m.name, pooler.get_pool(cr.dbname).get('estoque.local').browse(
                cr, uid, local_ids)))

        data = time.strftime("%d/%m/%Y")
        data_inicial = time.strftime('%d/%m/%Y', time.strptime(data_inicial, '%Y-%m-%d'))
        data_final = time.strftime('%d/%m/%Y', time.strptime(data_final, '%Y-%m-%d'))
        periodo = _('Periodo: %s a %s') % (data_inicial, data_final)
        det_format = '|{:^4}|{:^16}|{:<40}|{:^7}|{:>12}|{:^8}|{:>20}|{:>17}|{:>12}|{:>10}|'
        det_format = det_format.decode()
        listagem.informaCab(' {:^153} '.decode().format(title[:153]))
        listagem.informaCab('')
        listagem.informaCab(' {:<10} {:^132} {:>10} '.decode().format(
            data, nome_empresa, _('Pagina: {{ Page }}')))
        listagem.informaCab(' {:<153} '.decode().format(periodo))
        listagem.informaCab(' {:<6} {:<146} '.decode().format(
            _('Local:')[:6], locais or _('Todos')))
        listagem.informaCab('')
        listagem.informaCab('*' + '-' * 155 + '*')
        listagem.informaCab(det_format.format(
            _('CM')[:4], _('CODIGO DO ITEM')[:16], _('DESCRICAO')[:40], _('GRADE')[:7],
            _('QUANTIDADE')[:12], _('UNIDADE')[:8], custo_name[:20], _('CUSTO + ENCARGOS')[:17],
            _('VALOR TOTAL')[:12], _('% MARGEM')[:10]))
        listagem.informaCab('*' + '-' * 155 + '*')
        listagem.rodapePadrao()

        # Quantidade de digitos a ser mostrada
        if madeireira_instalado:
            digitos = '%.2f'
        else:
            digitos = '%.4f'

        list_dados = sorted(
            list_dados, key=lambda x: (x.get('name_produto'), x.get('grade'), x.get('cm')))

        # Totais
        total_sd = total_st = total_sn = total_si = total_sr = total_ss = qtd_sd = qtd_st = 0
        qtd_sn = qtd_si = qtd_sr = qtd_ss = total_ed = total_et = total_en = total_ei = total_er = 0
        total_es = total_ep = qtd_ed = qtd_et = qtd_en = qtd_ei = qtd_er = qtd_es = qtd_ep = 0
        custo_ed = custo_et = custo_ei = custo_ep = custo_er = custo_es = custo_en = 0
        custo_sd = custo_st = custo_si = custo_sr = custo_ss = custo_sn = 0

        codigo_produto = ''
        qtd_entrada = 0.0
        qtd_saida = 0.0
        vr_entrada = 0.0
        vr_saida = 0.0
        det_total_format = '|{:<71}{:>12}|{:>60}|{:>10}|'
        det_total_format = det_total_format.decode()
        categria_ids = []
        for i in dados:
            if i.get('categoria_id'):
                categria_ids.append(i.get('categoria_id'))

        dados_catg = {}
        if categria_ids:
            map(lambda x: dados_catg.update({x.get('id'): x.get('nome_completo')}),
                pooler.get_pool(cr.dbname).get('estoque.categoria').read(cr, uid, categria_ids,
                                                                 ['nome_completo'], context))
        categoria_id = False
        for dado in list_dados:
            if dado.get('quantidade') == 0:
                continue

            if codigo_produto != dado.get('codigo_produto') and list_dados.index(dado) != 0 and \
                    totaliza_item_igual:
                if qtd_entrada or vr_entrada:
                    listagem.imprimeLinha(det_total_format.format(
                        'T O T A L  D E  E N T R A D A',
                        locale.format(digitos, qtd_entrada, grouping=True),
                        locale.format('%.2f', vr_entrada, grouping=True),
                        '')
                    )
                if qtd_saida or vr_saida:
                    listagem.imprimeLinha(det_total_format.format(
                        'T O T A L  D E  S A I D A',
                        locale.format(digitos, qtd_saida, grouping=True),
                        locale.format('%.2f', vr_saida, grouping=True),
                        '')
                    )
                listagem.imprimeLinha('|' + ' ' * 155 + '|')
                qtd_entrada = 0.0
                qtd_saida = 0.0
                vr_entrada = 0.0
                vr_saida = 0.0

            if dado.get('categoria_id') and categoria_id != dado.get('categoria_id'):
                if categoria_id:
                    listagem.imprimeLinha('|' + ''.ljust(155) + '|')

                categoria_id = dado.get('categoria_id')
                nome_categ = "SEM CATEGORIA"
                if categoria_id:
                    nome_categ = dados_catg.get(categoria_id) or nome_categ

                linha = ('|' + nome_categ).ljust(156) + '|'
                listagem.imprimeLinha(linha)

            linha = det_format.format(
                dado.get('cm')[:4], dado.get('codigo_produto')[:16], dado.get('name_produto')[:40],
                dado.get('grade')[:7],
                locale.format(digitos, dado.get('quantidade'), grouping=True)[-12:],
                dado.get('unidade')[:8],
                locale.format('%.2f', dado.get('custo'), grouping=True)[-20:],
                locale.format('%.2f', dado.get('custo_mais_encargos'), grouping=True)[-17:],
                locale.format('%.2f', dado.get('total'), grouping=True)[-12:],
                locale.format('%.2f', dado.get('margem') if dado.get('cm') == 'SN' else 0.0,
                              grouping=True)[-10:])

            listagem.imprimeLinha(linha)

            if dado.get('cm')[0] == 'E':
                qtd_entrada += (dado.get('quantidade') or 0.0)
                vr_entrada += (dado.get('total') or 0.0)
            elif dado.get('cm')[0] == 'S':
                qtd_saida += (dado.get('quantidade') or 0.0)
                vr_saida += (dado.get('total') or 0.0)

            if list_dados.index(dado) == (len(list_dados) - 1) and totaliza_item_igual:
                if qtd_entrada or vr_entrada:
                    listagem.imprimeLinha(det_total_format.format(
                        'T O T A L  D E  E N T R A D A',
                        locale.format(digitos, qtd_entrada, grouping=True),
                        locale.format('%.2f', vr_entrada, grouping=True),
                        '')
                    )
                if qtd_saida or vr_saida:
                    listagem.imprimeLinha(det_total_format.format(
                        'T O T A L  D E  S A I D A',
                        locale.format(digitos, qtd_saida, grouping=True),
                        locale.format('%.2f', vr_saida, grouping=True),
                        '')
                    )
                qtd_entrada = 0.0
                qtd_saida = 0.0
                vr_entrada = 0.0
                vr_saida = 0.0

            codigo_produto = dado.get('codigo_produto')

            # Calcula os totais
            total_ed += dado.get('total') if dado.get('cm') == 'ED' else 0
            qtd_ed += dado.get('quantidade') if dado.get('cm') == 'ED' else 0
            custo_ed += dado.get('custo') if dado.get('cm') == 'ED' else 0

            total_et += dado.get('total') if dado.get('cm') == 'ET' else 0
            qtd_et += dado.get('quantidade') if dado.get('cm') == 'ET' else 0
            custo_et += dado.get('custo') if dado.get('cm') == 'ET' else 0

            total_en += dado.get('total') if dado.get('cm') in ['EN', 'EC'] else 0
            qtd_en += dado.get('quantidade') if dado.get('cm') in ['EN', 'EC'] else 0
            custo_en += dado.get('custo') if dado.get('cm') in ['EN', 'EC'] else 0

            total_ei += dado.get('total') if dado.get('cm') == 'EI' else 0
            qtd_ei += dado.get('quantidade') if dado.get('cm') == 'EI' else 0
            custo_ei += dado.get('custo') if dado.get('cm') == 'EI' else 0

            total_ep += dado.get('total') if dado.get('cm') == 'EP' else 0
            qtd_ep += dado.get('quantidade') if dado.get('cm') == 'EP' else 0
            custo_ep += dado.get('custo') if dado.get('cm') == 'EP' else 0

            total_er += dado.get('total') if dado.get('cm') == 'ER' else 0
            qtd_er += dado.get('quantidade') if dado.get('cm') == 'ER' else 0
            custo_er += dado.get('custo') if dado.get('cm') == 'ER' else 0

            total_es += dado.get('total') if dado.get('cm') == 'ES' else 0
            qtd_es += dado.get('quantidade') if dado.get('cm') == 'ES' else 0
            custo_es += dado.get('custo') if dado.get('cm') == 'ES' else 0

            total_sd += dado.get('total') if dado.get('cm') == 'SD' else 0
            qtd_sd += dado.get('quantidade') if dado.get('cm') == 'SD' else 0
            custo_sd += dado.get('custo') if dado.get('cm') == 'SD' else 0

            total_st += dado.get('total') if dado.get('cm') == 'ST' else 0
            qtd_st += dado.get('quantidade') if dado.get('cm') == 'ST' else 0
            custo_st += dado.get('custo') if dado.get('cm') == 'ST' else 0

            total_sn += dado.get('total') if dado.get('cm') == 'SN' else 0
            qtd_sn += dado.get('quantidade') if dado.get('cm') == 'SN' else 0
            custo_sn += dado.get('custo') if dado.get('cm') == 'SN' else 0

            total_si += dado.get('total') if dado.get('cm') == 'SI' else 0
            qtd_si += dado.get('quantidade') if dado.get('cm') == 'SI' else 0
            custo_si += dado.get('custo') if dado.get('cm') == 'SI' else 0

            total_sr += dado.get('total') if dado.get('cm') == 'SR' else 0
            qtd_sr += dado.get('quantidade') if dado.get('cm') == 'SR' else 0
            custo_sr += dado.get('custo') if dado.get('cm') == 'SR' else 0

            total_ss += dado.get('total') if dado.get('cm') == 'SS' else 0
            qtd_ss += dado.get('quantidade') if dado.get('cm') == 'SS' else 0
            custo_ss += dado.get('custo') if dado.get('cm') == 'SS' else 0

        listagem.imprimeLinha('*' + '-' * 155 + '*')

        total_format = '| {:<70}{:>12}|{:>29}|{:>30}|          |'.decode()

        listagem.imprimeLinha(total_format.format(
            _('Total de entradas por devolucao')[:70],
            locale.format('%.2f', qtd_ed, grouping=True),
            locale.format('%.2f', custo_ed, grouping=True),
            locale.format('%.2f', total_ed, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de entradas por transferencia')[:70],
            locale.format('%.2f', qtd_et, grouping=True),
            locale.format('%.2f', custo_et, grouping=True),
            locale.format('%.2f', total_et, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de entradas por inventario')[:70],
            locale.format('%.2f', qtd_ei, grouping=True),
            locale.format('%.2f', custo_ei, grouping=True),
            locale.format('%.2f', total_ei, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de entradas por requisicao almoxarifado')[:70],
            locale.format('%.2f', qtd_er, grouping=True),
            locale.format('%.2f', custo_er, grouping=True),
            locale.format('%.2f', total_er, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de entradas por requisicao ressuprimento')[:70],
            locale.format('%.2f', qtd_es, grouping=True),
            locale.format('%.2f', custo_es, grouping=True),
            locale.format('%.2f', total_es, grouping=True)))

        if pcp_instalado:
            listagem.imprimeLinha(total_format.format(
                _('Total de entradas de producao')[:70],
                locale.format(digitos, qtd_ep, grouping=True),
                locale.format('%.2f', custo_ep, grouping=True),
                locale.format('%.2f', total_ep, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total das compras')[:70],
            locale.format('%.2f', qtd_en, grouping=True),
            locale.format('%.2f', custo_en, grouping=True),
            locale.format('%.2f', total_en, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de saidas por devolucao')[:70],
            locale.format('%.2f', qtd_sd, grouping=True),
            locale.format('%.2f', custo_sd, grouping=True),
            locale.format('%.2f', total_sd, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de saidas por transferencia')[:70],
            locale.format('%.2f', qtd_st, grouping=True),
            locale.format('%.2f', custo_st, grouping=True),
            locale.format('%.2f', total_st, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de saidas por inventario')[:70],
            locale.format('%.2f', qtd_si, grouping=True),
            locale.format('%.2f', custo_si, grouping=True),
            locale.format('%.2f', total_si, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de saidas por requisicao almoxarifado')[:70],
            locale.format('%.2f', qtd_sr, grouping=True),
            locale.format('%.2f', custo_sr, grouping=True),
            locale.format('%.2f', total_sr, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total de saidas por requisicao ressuprimento')[:70],
            locale.format('%.2f',  qtd_ss, grouping=True),
            locale.format('%.2f', custo_ss, grouping=True),
            locale.format('%.2f', total_ss, grouping=True)))

        listagem.imprimeLinha(total_format.format(
            _('Total das vendas'),
            locale.format(digitos, qtd_sn, grouping=True),
            locale.format('%.2f', custo_sn, grouping=True),
            locale.format('%.2f', total_sn, grouping=True)))

        listagem.imprimeLinha('*' + '-' * 155 + '*')

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    @staticmethod
    def get_cm(tipo_movimento, origem, cfop_movimento):
        cm = tipo_movimento or ''
        if cfop_movimento:
            cm += cfop_movimento or '' if cfop_movimento not in 'UC' else 'N'
        cm += origem or '' if len(cm) < 2 and origem != 'U' else 'N' if len(cm) < 2 else ''

        return cm

    @staticmethod
    def _get_embalagem_qtde_embalagem_local(cr, produto, grade, local):
        """
        Recupera a sigla da embalagem e a quantidade na embalagem de um produto em um determinado
        local (local/locacao).

        :param cr: a linha atual do cursor do banco de dados
        :param produto: ID do produto
        :param grade: ID da grade do produto
        :param local: ID do local (local/locacao)
        :type cr: cursor
        :type produto: long
        :type grade: long
        :type local: long
        :return: tupla com a sigla da embalagem e a quantidade de unidades nesta embalagem
        :rtype: tuple(unicode, float)

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # noinspection SqlDialectInspection
        _sql = """
          SELECT      ee.sigla, ee.quantidade_na_embalagem
          FROM        estoque_local_locacao     AS ell
          INNER JOIN  estoque_produto_embalagem AS epe  ON epe.id = ell.estllc_produto_embalagem_id
          INNER JOIN  estoque_embalagem         AS ee   ON ee.id = epe.embalagem_id
          WHERE       ell.produto_id = %d
            AND       ell.grade_id = %d
            AND       ell.local_id = %d
        """ % (produto, grade, local)
        cr.execute(_sql)
        result = cr.fetchone()

        if not result:
            result = None, 0.0

        return result

    @staticmethod
    def _get_embalagem_menor_qtde_embalagem(cr, produto, grade):
        """
        Recupera a sigla da embalagem e a menor quantidade na embalagem de um produto.

        :param cr: a linha atual do cursor do banco de dados
        :param produto: ID do produto
        :param grade: ID da grade do produto
        :type cr: cursor
        :type produto: long
        :type grade: long
        :return: tupla com a sigla da embalagem e a quantidade de unidades nesta embalagem
        :rtype: tuple(unicode, float)

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # noinspection SqlDialectInspection
        _sql = """
          SELECT      ee.sigla, ee.quantidade_na_embalagem
          FROM        estoque_local_locacao     AS ell
          INNER JOIN  estoque_produto_embalagem AS epe  ON epe.id = ell.estllc_produto_embalagem_id
          INNER JOIN  estoque_embalagem         AS ee   ON ee.id = epe.embalagem_id
          WHERE       ell.produto_id = %d
            AND       ell.grade_id = %d
          ORDER BY    ee.quantidade_na_embalagem
        """ % (produto, grade)
        cr.execute(_sql)
        result = cr.fetchone()  # Pela ordem crescente dos registro recupera a menor unidade

        if not result:
            result = None, 0.0

        return result

report_custom('report.listagem_movimento_sintetico')
