#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from random import randint
from datetime import date, datetime


# noinspection PyUnresolvedReferences
class TestEstoqueInventarioContagem(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestEstoqueInventarioContagem, self).setUp()
        self.funcionario_id = self.registry('rh.funcionario').get_id_func(self.cr, self.uid)
        self.saldo = 10
        self.company_id = 1
        self.local_id = self.registry('estoque.local').create(
            self.cr, self.uid, self.get_vals_local())
        self.produto_id = self.registry('estoque.produto').create(
            self.cr, self.uid, self.get_vals_produto())
        self.grade_id = self.registry('estoque.produto.grade').create(
            self.cr, self.uid, self.get_vals_grade(self.produto_id))
        self.locacao_id = self.registry('estoque.local.locacao').create(
            self.cr, self.uid, self.get_vals_locacao(self.grade_id, self.local_id, self.produto_id))
        self.invcap = self.registry('estoque.inventario.capa')
        self.invcont = self.registry('estoque.inventario.contagem')
        wizgerarcontagem = self.registry('estoque.gerar.contagem.osv')
        descricao = 'teste' + str(randint(1, 1000))
        self.ano = str(date.today().year)
        self.mes = str(date.today().month).zfill(2)
        vals_inventario = dict(name=descricao, descricao=descricao,
                               local=self.get_local_id('LOJA')[0], mes=self.mes, ano=self.ano,
                               company_id=1)
        wizcontagem_id = wizgerarcontagem.create(self.cr, self.uid, vals_inventario, context={})
        wizgerarcontagem._gerar_inventario_capa(self.cr, self.uid, wizcontagem_id, context={})
        self.data_inventario = wizgerarcontagem._get_data_movimento(
            False, self.ano, self.mes)
        self.inventario_id = self.invcap.search(
            self.cr, self.uid, [('descricao', '=', descricao), ('ano', '=', self.ano),
                                ('mes', '=', self.mes)])[0]
        grade_qtd = {self.grade_id, self.saldo}
        self.registry('estoque.produto.saldo').atualizar_saldo_do_dia(
            self.cr, self.uid, self.company_id, grade_qtd, self.data_inventario, 'E', self.local_id,
            context=None)

    def test_gerar_contagem(self):
        dados = self.invcont.gerar_contagem(
            self.cr, self.uid, self.inventario_id, self.data_inventario, False, False,
            local_id=self.get_local_id('LOJA')[0])
        self.assertIsInstance(dados, list)

    def test_gerar_contagem_inicial(self):
        dados_para_contagem = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, self.local_id, locacao_id=False, categoria_id=False,
            fornecedor_id=False,
            locacao_company_id=self.company_id)
        itens_agrupados_por_local = self.invcont._agrupar_por_local(dados_para_contagem)
        str_data_inventario = " '%s' " % self.data_inventario
        res, produto_ids, grade_ids = self.invcont._get_saldo_para_inventario(
            self.cr, self.uid, str_data_inventario, True, itens_agrupados_por_local, context=None)
        custos = self.invcont._get_custo_itens_para_inventario(
            self.cr, self.uid, False, self.data_inventario, produto_ids, context=None)
        dados = self.invcont.gerar_contagem_inicial(
            self.cr, self.uid, grade_ids, res, custos, False, self.funcionario_id,
            self.inventario_id, False)
        self.assertIsInstance(dados, list)

    def test_get_dados_para_contagem(self):
        locacao = self.registry('estoque.local.locacao')
        local_id = self.get_local_id('LOJA')[0]
        locacao_ids = locacao.search(self.cr, self.uid, [('local_id', '=', local_id)])
        dados = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, local_id, locacao_id=False, categoria_id=False, fornecedor_id=False,
            locacao_company_id=self.company_id)
        self.assertIsInstance(dados, list)
        for dado in dados:
            self.assertTrue(dado.get('locacao_id') in locacao_ids)

    def test_agrupar_por_local(self):
        local_id = self.get_local_id('LOJA')[0]
        dados_para_contagem = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, local_id, locacao_id=False, categoria_id=False, fornecedor_id=False,
            locacao_company_id=self.company_id)
        dado = self.invcont._agrupar_por_local(dados_para_contagem)
        self.assertIsInstance(dado, dict)

    def test_get_saldo_inventario(self):
        local_id = self.local_id
        dados_para_contagem = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, local_id, locacao_id=False, categoria_id=False, fornecedor_id=False,
            locacao_company_id=self.company_id)
        itens_agrupados_por_local = self.invcont._agrupar_por_local(dados_para_contagem)
        str_data_inventario = " '%s' " % self.data_inventario
        res, produto_ids, grade_ids = self.invcont._get_saldo_para_inventario(
            self.cr, self.uid, str_data_inventario, False, itens_agrupados_por_local, context=None)
        self.assertIsInstance(res, list)
        self.assertIsInstance(grade_ids, list)
        self.assertIsInstance(produto_ids, list)
        self.assertEqual(produto_ids[0], self.produto_id)
        self.assertEqual(grade_ids[0], self.grade_id)
        self.assertEqual(res[0].get('pgsaldo_saldo'), self.saldo)

    def test_get_saldo_inventario_efeito_inventario(self):
        dados_para_contagem = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, self.local_id, locacao_id=False, categoria_id=False,
            fornecedor_id=False,
            locacao_company_id=self.company_id)
        str_data_inventario = " '%s' " % self.data_inventario
        itens_agrupados_por_local = self.invcont._agrupar_por_local(dados_para_contagem)
        res, produto_ids, grade_ids = self.invcont._get_saldo_para_inventario(
            self.cr, self.uid, str_data_inventario, True, itens_agrupados_por_local, context=None)
        self.assertIsInstance(res, list)
        self.assertIsInstance(grade_ids, list)
        self.assertIsInstance(produto_ids, list)
        self.assertEqual(produto_ids[0], self.produto_id)
        self.assertEqual(grade_ids[0], self.grade_id)
        self.assertEqual(res[0].get('pgsaldo_saldo'), self.saldo)

    def test_get_saldo_local_para_inventario(self):
        ano = datetime.strptime(self.data_inventario, '%Y-%m-%d').year
        mes = datetime.strptime(self.data_inventario, '%Y-%m-%d').month
        ultimo_dia_mes_ano = self.invcont._get_ultimo_dia_mes_ano(ano, mes)
        data_movimento = " '%s' " % ultimo_dia_mes_ano
        str_data_inventario = " '%s' " % self.data_inventario
        dados_para_contagem = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, self.local_id, locacao_id=False, categoria_id=False,
            fornecedor_id=False,
            locacao_company_id=self.company_id)
        itens_agrupados_por_local = self.invcont._agrupar_por_local(dados_para_contagem)
        res, produto_ids, grade_ids = self.invcont._get_saldo_para_inventario(
            self.cr, self.uid, str_data_inventario, True, itens_agrupados_por_local, context=None)
        dados = self.invcont._get_saldo_local_para_inventario(
            self.cr, self.uid, data_movimento, self.local_id, tuple(produto_ids),
            tuple(grade_ids), context=None)
        self.assertEqual(dados[0].get('pgsaldo_saldo'), 10)

    def test_get_custo_itens_para_inventario(self):
        dados_para_contagem = self.invcont.get_dados_para_contagem(
            self.cr, self.uid, self.local_id, locacao_id=False, categoria_id=False,
            fornecedor_id=False,
            locacao_company_id=self.company_id)
        str_data_inventario = " '%s' " % self.data_inventario
        itens_agrupados_por_local = self.invcont._agrupar_por_local(dados_para_contagem)
        res, produto_ids, grade_ids = self.invcont._get_saldo_para_inventario(
            self.cr, self.uid, str_data_inventario, True, itens_agrupados_por_local, context=None)
        dados = self.invcont._get_custo_itens_para_inventario(
            self.cr, self.uid, False, self.data_inventario, produto_ids, context=None)
        self.assertIsInstance(dados, dict)

    def get_local_id(self, nome_local):
        return self.registry('estoque.local').search(self.cr, self.uid, [('name', '=', nome_local)])

    def get_vals_produto(self, grade_unica=True):
        nome_prod = 'teste' + str(randint(1, 1000))
        return dict(company_id=self.company_id, name=nome_prod, origem_mercadoria='0',
                    tipo_item='00', grade_usar=grade_unica)

    def get_vals_grade(self, produto_id, grade='UN'):
        nome_grade = 'teste' + str(randint(1, 1000))
        return dict(descricao=nome_grade, grade=grade, produto_id=produto_id)

    def get_vals_local(self):
        local = self.registry('estoque.local')
        nome_local = 'teste' + str(randint(1, 1000))
        centro_custo_id = self.registry('adm.empresa.centrocusto').search(
            self.cr, self.uid, [], limit=1)[0]
        local_maior_numero_id = local.search(self.cr, self.uid, [], order='numero desc', limit=1)[0]
        local_numero = local.browse(self.cr, self.uid, local_maior_numero_id).numero + 1

        return dict(centrocusto_id=centro_custo_id, company_id=self.company_id,
                    name=nome_local, numero=local_numero)

    def get_vals_locacao(self, grade_id, local_id, produto_id):
        return dict(company_id=self.company_id, grade_id=grade_id, local_id=local_id,
                    produto_id=produto_id)

    def test_salvar_contagem(self):
        vals = {'grade_id': self.grade_id, 'produto_id': self.produto_id, 'local_id': self.local_id,
                'locacao_id': self.locacao_id, 'saldo_contado': 15.0, 'saldo_cadastro': 5.0}
        res = self.registry('estoque.inventario.contagem').salvar_contagem(
            self.cr, self.uid, self.inventario_id, self.produto_id, self.grade_id, self.local_id,
            vals, context={})
        self.assertTrue(type(res) in (list, long))
