#-*- encoding: utf-8 -*-

from datetime import date
from openerp.osv import osv, fields
from openerp import pooler
from openerp.tools.translate import _
from funcoes import get_state_capa


class digitacao_inventario_capa_osv(osv.Model):
    def _get_funcionario(self, cr, user, context=None):
        context = context and context.copy() or {}
        res_user = self.pool.get('res.users').read(cr, user, user, ['pessoa_id'], context=context)
        pessoa_id = res_user.get('pessoa_id', [False])[0]
        rh_func = self.pool.get('rh.funcionario').search(cr, user, [('pessoa', '=', pessoa_id)], context=context)
        if rh_func:
            return rh_func[0]
        else:
            raise osv.except_osv(_('Atencao'), _('Usuario nao cadastrado como funcionario'))

    _name = 'digitacao.inventario.capa.osv'
    _description = 'Digitacao inventario Capa'
    _columns = {
        'local_id': fields.many2one('estoque.local', 'Local', required=True), # Id do local
        'inventario_id': fields.many2one('estoque.inventario.capa', ondelete='cascade'),
        'funcionario_id': fields.many2one('rh.funcionario', 'Funcionario', required=True), # Id do funcionario
        'state_id': fields.many2one('digitacao.inventario.capa.state', 'Status',
                                    ondelete='set null'),
    }
    _defaults = {
        'funcionario_id': lambda s, cr, uid, c: s._get_funcionario(cr, uid, c),
    }

    def finaliza_digitacao(self, cr, uid, id, context=None):
        context = context and context.copy() or {}
        return {}


class digitacao_inventario_osv(osv.Model):
    _name = 'digitacao.inventario.osv'
    _description = 'Digitacao inventario'
    _columns = {
        'digitacao_inventario_id': fields.many2one('digitacao.inventario.capa.osv',
                                                   'Digitacao Capa',ondelete="cascade"),
        'produto_id': fields.many2one("estoque.produto", 'Produto'),  # Id do Produto
        'local_id': fields.many2one('estoque.local', 'Local'),  # Id do local
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade',
                                    domain="[('produto_id', '=', produto_id)]"),  # Id do
        # produto a ser pedido
        'produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                                required=True),  # Id da embalagem do produto
        'saldo_contado': fields.char('Saldo contado', size=15, required=True),  # saldo contado
        'tipo_state': fields.function(get_state_capa, method=True, string='Status', type='char',
                                      size=1),
    }
    _defaults = {
        'tipo_state': lambda *a: 'I',
    }

    def default_get(self, cr, uid, fields_list, context=None):
        res = super(digitacao_inventario_osv, self).default_get(cr, uid, fields_list, context)
        if context.get('state_id'):
            res['tipo_state'] = self.pool.get('digitacao.inventario.capa.state').browse(
                cr, uid, context.get('state_id'), context).state
        return res

    def verifica_produto_pertence_inventario(self, cr, uid, inventario_id, produto_id, local_id,
                                             context=None):
        """
        O metodo verifica se o produto (produto_id) informado esta entre os produtos da contagem
        desse inventario (inventario_id).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param inventario_id: id da tabela estoque_inventario_capa.
        :type inventario_id: int
        :param produto_id:  id da tabela estoque_produto.
        :type produto_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Falso, se o produto nao pertencer a contagem do inventario, senao, Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = True
        if not inventario_id or not produto_id:
            return res

        args = [('produto_id','=',produto_id),
                ('inventario_id','=',inventario_id),
                ('local_id','=',local_id)]
        return self.pool.get('estoque.inventario.contagem').search(cr, uid, args, context=context)

    def onchange_produto_id(self, cr, uid, ids, produto_id, inventario_id, tipo_state, local_id,
                            context=None):
        if tipo_state == 'B':
            return {}

        context = context and context.copy() or {}
        val = {'value': {'produto_embalagem_id': False, 'grade_id': False,'saldo_contado':0}}
        if not produto_id:
            return val

        grade_id = self.pool.get('estoque.produto.grade').search(cr, uid,
                                                 [('produto_id', '=', produto_id)], context=context)
        if len(grade_id) == 1:
            val['value']['grade_id'] = grade_id[0]

        produto_embalagem_ids = self.pool.get('estoque.produto.embalagem').search(
            cr, uid, [('produto_id', '=', produto_id)], context=context)
        if len(produto_embalagem_ids) == 1:
            val['value']['produto_embalagem_id'] = produto_embalagem_ids[0]

        if not inventario_id:
            return val

        produto_do_inventario = self.verifica_produto_pertence_inventario(
            cr, uid, inventario_id, produto_id, local_id, context)

        if not produto_do_inventario:
            val = {'value': {'produto_id': False, 'produto_embalagem_id': False, 'grade_id': False}}
            val['warning'] = {'title': _('Atencao'),
                              'message': _('O produto %s nao pertence ao inventario.') %
                                           self.pool.get('estoque.produto').read(
                                               cr, uid, produto_id, ['name'], context).get('name')}
        return val

    def onchange_grade_id(self, cr, uid, ids, local_id, grade_id, tipo_state, context=None):
        """
        O metodo encontra em estoque_local_locacao a embalagem (estllc_produto_embalagem_id) para o
        local_id e grade_id recebidos. Metodo chamado na saida do atributo grade_id.
        :param cr: cursor no banco dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param local_id: id da tabela estoque_local.
        :type local_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com os atributos e seus respectivos valores que serao exibidos na tela.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if tipo_state == 'B':
            return {}

        res = {'value': {'produto_embalagem_id': False}}
        if not local_id or not grade_id:
            return res

        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        locacao_ids = obj_estoque_local_locacao.search(
            cr, uid, [('grade_id', '=', grade_id), ('local_id', '=', local_id)], context=context)
        if not locacao_ids:
            return res

        produto_embalagem_id = obj_estoque_local_locacao.read(cr, uid, locacao_ids[0],
                      ['estllc_produto_embalagem_id'], context=context, load='_classic_write').get(
                      'estllc_produto_embalagem_id') or False

        res['value']['produto_embalagem_id'] = produto_embalagem_id
        return res

    def tratar_inventario(self, cr, uid, capa_id, inventario_id, produto_id, grade_id, saldo_contado, context=None):
        '''Funcao para insercao e edicao da contagem do Inventario'''
        context = context and context.copy() or {}
        obj_estoque_inventario_contagem = self.pool.get('estoque.inventario.contagem')
        obj_capa = self.pool.get('digitacao.inventario.capa.osv').read(cr, uid, capa_id, ['local_id', 'funcionario_id'], context=context)

        local_id = obj_capa.get('local_id', False)

        if local_id:
            if isinstance(local_id, tuple):
                local_id = local_id[0]
        else:
            raise osv.except_osv(_('Atencao'), _('Local nao informado.'))
        args = [('local_id', '=', local_id), ('grade_id', '=', grade_id),
                ('produto_id', '=', produto_id)]
        locacao_id = self.pool.get('estoque.local.locacao').search(cr, uid, args, context=context)
        if locacao_id:
            locacao_id = locacao_id[0]
        else:
            locacao_id = False

        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_inventario_tipo_contagem', context=context)
        tipo_contagem = estoque_parametro_dados.get('estpar_inventario_tipo_contagem')
        args_contagem = [('inventario_id', '=', inventario_id),
                         ('produto_id', '=', produto_id),
                         ('grade_id', '=', grade_id),
                         ('local_id', '=', local_id)]
        contagem_ids = obj_estoque_inventario_contagem.search(cr, uid, args_contagem,
                                                              context=context)
        contagem_id = contagem_ids and contagem_ids[0] or False
        vals_contagem = dict(grade_id=grade_id, produto_id=produto_id, local_id=local_id,
                             locacao_id=locacao_id, saldo_contado=saldo_contado)
        dict_trib = {}
        obj_estoque_inventario_contagem.get_dict_trib(cr, uid, [produto_id], dict_trib, context=context)
        obj_estoque_inventario_contagem.atualiza_vals_com_trib(vals_contagem, produto_id, dict_trib,
                                                               context=context)
        obj_estoque_inventario_contagem.salvar_contagem(cr, uid, inventario_id, produto_id,
                     grade_id, local_id, vals_contagem, tipo_contagem, contagem_id, context=context)
        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = []
        try:
            val_contado = float(vals['saldo_contado'].replace(',', '.'))
        except ValueError:
            raise osv.except_osv(_('Atencao'), _('Valor do Saldo Contado invalido.'))
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        quantidade_na_embalagem = obj_estoque_produto_embalagem.browse(
            cr, uid, vals.get('produto_embalagem_id'), context).embalagem_id.\
            quantidade_na_embalagem or 1
        val_contado *= quantidade_na_embalagem
        digitacao_inventario_id = vals.get('digitacao_inventario_id')
        digitacao_inventario_capa_dados = self.pool.get('digitacao.inventario.capa.osv').read(cr, uid,
                                                                                              [digitacao_inventario_id],
                                                                                              ['inventario_id',
                                                                                               'local_id'], context=context)
        for x in digitacao_inventario_capa_dados:
            inventario_id = x.get('inventario_id')
            local_id = x.get('local_id')
            if inventario_id:
                if isinstance(inventario_id, tuple):
                    inventario_id = inventario_id[0]
            if local_id:
                if isinstance(local_id, tuple):
                    local_id = local_id[0]
                vals['local_id'] = local_id
            self.tratar_inventario(cr, uid, vals['digitacao_inventario_id'], inventario_id, vals['produto_id'],
                                   vals['grade_id'], val_contado)
        res = super(digitacao_inventario_osv, self).create(cr, uid, vals, context=context)
        return res

    def write(self, cr, uid, ids, vals, context=None):

        context = context and context.copy() or {}
        if isinstance(ids,list):
            id = ids[0]
        else:
            id = ids
        brw_digitacao_inventario_osv = self.browse(cr,uid,id)
        try:
            if 'saldo_contado' in vals:
                val_contado = float(vals['saldo_contado'].replace(',', '.'))
            else:
                val_contado = self.browse(cr,uid,id).saldo_contado
        except ValueError:
            raise osv.except_osv(_('Atencao'), _('Valor do Saldo Contado invalido.'))
        if 'produto_embalagem_id' in vals:
            produto_embalagem_id = vals['produto_embalagem_id']
        else:
            produto_embalagem_id = brw_digitacao_inventario_osv.produto_embalagem_id.id

        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        quantidade_na_embalagem = obj_estoque_produto_embalagem.browse(
            cr, uid, produto_embalagem_id, context).embalagem_id.\
            quantidade_na_embalagem or 1
        val_contado *= quantidade_na_embalagem
        digitacao_inventario_id = vals.get('digitacao_inventario_id')
        if not digitacao_inventario_id:
            digitacao_inventario_id = brw_digitacao_inventario_osv.digitacao_inventario_id.id
        produto_id = vals.get('produto_id')
        if not produto_id:
            produto_id = brw_digitacao_inventario_osv.produto_id.id
        grade_id = vals.get('grade_id')
        if not grade_id:
            grade_id = brw_digitacao_inventario_osv.grade_id.id
        inventario_id = self.pool.get('digitacao.inventario.capa.osv').browse(
            cr, uid, digitacao_inventario_id).inventario_id.id
        self.tratar_inventario(cr, uid, digitacao_inventario_id, inventario_id, produto_id,
                               grade_id, val_contado)
        return super(digitacao_inventario_osv, self).write(cr, uid, ids, vals, context=context)

    def onchange_saldo_contado(self, cr, uid, ids, produto_embalagem_id, saldo_contado, context=None):
        context = context and context.copy() or {}
        res = {}
        if not produto_embalagem_id or not saldo_contado:
            return res
        try:
            saldo_contado = float(saldo_contado.replace(',', '.'))
        except:
            return res
        fracao = self.pool.get('estoque.produto.embalagem').browse(
            cr, uid, produto_embalagem_id, context).embalagem_id.unidade_id.fracao
        if not fracao and saldo_contado != int(saldo_contado):
            res['value'] = {'saldo_contado': ''}
            res['warning'] = {'title': _('Atencao'),
                              'message': 'Embalagem nao aceita fracionamento.'}
        return res


class digitacao_inventario_capa_osv_inherit(osv.Model):
    _name = 'digitacao.inventario.capa.osv'
    _inherit = 'digitacao.inventario.capa.osv'
    _description = 'Digitacao inventario Capa'
    _columns = {
        'digitacao_inventario_ids': fields.one2many('digitacao.inventario.osv', 'digitacao_inventario_id',
                                                    'Digitacao Inventario')
    }


class wizard_nova_digitacao(osv.TransientModel):
    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(wizard_nova_digitacao, self).default_get(cr, uid, fields, context)
        if res.get('local_id'):
            res['flag'] = True
        return res

    def _get_local_inventario_capa(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        local_id = False
        if vals.get('active_id'):
            active_id = vals.get('active_id')
            local_id = self.pool.get('estoque.inventario.capa').browse(cr, uid, active_id).local_id.id or False
        return local_id

    _name = 'wizard.nova.digitacao'
    _description = 'Digitacao inventario Capa'
    _columns = {
        'local_id': fields.many2one('estoque.local', 'Local', required=True),
        'flag': fields.boolean('Flag'),
    }
    _defaults = {
        'local_id': _get_local_inventario_capa,
    }

    def abrir_digitacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        data = self.read(cr, uid, ids[0], context=context)
        data['id'] = context['active_id']
        return self.pool.get('estoque.inventario.capa').button_digitar(cr, uid, data, context=context)





