# -*- coding: utf-8 -*-
from datetime import date
import os
from time import strftime
import calendar
from openerp.osv import osv, fields
from openerp.tools.translate import _


READ_LOAD = '_classic_write'


# noinspection PyProtectedMember,PyAbstractClass
class estoque_gerar_contagem_osv(osv.TransientModel):
    _name = 'estoque.gerar.contagem.osv'

    def _get_tipo_contagem(self, cr, uid, res={}, ids=[], context=None):
        context = context and context.copy() or {}
        tipo_contagem = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                                                         'estoque.parametro',
                                                         ['estpar_inventario_tipo_contagem'],
                                                         context=context).\
                                                         get('estpar_inventario_tipo_contagem')
        for id in ids:
            res[id] = tipo_contagem

        if not ids:
            return tipo_contagem or ''

    def _estict_tipo_contagem(self, cr, uid, ids, name, args, context=None):
        context = context and context.copy() or {}
        res = {}
        self._get_tipo_contagem(cr, uid, res, ids, context=context)
        return res

    def get_default_tipo_contagem(self, cr, uid, context=None):
        context = context and context.copy() or {}
        return self._get_tipo_contagem(cr, uid, context=context)

    def get_default_zerar_saldo(self, cr, uid, context=None):
        tipo_contagem = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                                                         'estoque.parametro',
                                                         ['estpar_inventario_tipo_contagem'],
                                                         context=context).\
                                                         get('estpar_inventario_tipo_contagem')
        if tipo_contagem == 'I':
            return True

        return False

    def _get_img(self, cr, uid, context=None):
        context = context and context.copy() or {}
        root_path = os.path.dirname(os.path.abspath(__file__))
        image_file = os.path.join(root_path, 'pixmaps', 'icon.png')
        return open(image_file, 'rb').read().encode('base64')

    _columns = {
        'name': fields.char('Descricao', size=30, required=True),
        'company_id': fields.many2one("res.company", 'Empresa', required=True),
        'mes': fields.selection([
                                    ('01', 'Janeiro'), ('02', 'Fevereiro'), ('03', 'Marco'),
                                    ('04', 'Abril'), ('05', 'Maio'), ('06', 'Junho'),
                                    ('07', 'Julho'), ('08', 'Agosto'), ('09', 'Setembro'),
                                    ('10', 'Outubro'), ('11', 'Novembro'), ('12', 'Dezembro')],
                                'Mes', required=True),
        'ano': fields.char('Ano', size=4, required=True),
        'imagem': fields.binary('Photo', readonly=True),
        'status': fields.boolean('Estdo'),
        'local_id': fields.many2one('estoque.local', 'Local'),
        'mensagem': fields.char('Mensagem', size=30),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'fornecedor_id': fields.many2one('financeiro.fornecedor', 'Fornecedor'),
        'locacao_id': fields.many2one('estoque.local.locacao', 'Locacao',
                                      domain="[('local_id','=',local_id)]"),
        'zerar_saldo_contado': fields.boolean(
            'Zerar contagem', help="Ao gerar o inventario, por padrao, se voce nao digitar nenhum "
                                   "item na contagem de inventario, nao havera diferenca de "
                                   "contagem.\nMarcando esta opcao, o valor do saldo contado sera "
                                   "zero, portanto, nao fazendo nenhuma digitacao de contagem, "
                                   "todo o estoque sera zerado!"),
        'efeito_de_inventario': fields.boolean('Apenas p/ efeito de inventario',
                                               help='Se marcado, vai gerar o inventario com o saldo'
                                                    ' e o custo do ultimo dia do mes informado, e '
                                                    'marcado com atualizado, nao permitindo alterar'
                                                    ' o saldo do estoque'),
        'estict_par_tipo_contagem': fields.function(_estict_tipo_contagem, method=True,
                                                    string='Tipo de Contagem', type='char',
                                                    size=1, store=False),
        'zerar_saldo_negativo': fields.boolean('Zerar saldo negativo',
                                               help='Zera o saldo contado para todos os itens que '
                                                    'estiverem com saldo negativo'),
    }

    _defaults = {
    }

    _defaults = {
        'mes': lambda *a: '%02d' % date.today().month,
        'ano': lambda *a: str(date.today().year),
        'imagem': _get_img,
        'status': False,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                                        s._name,
                                                                                        context=c),
        'zerar_saldo_contado': get_default_zerar_saldo,
        'estict_par_tipo_contagem': get_default_tipo_contagem,
    }

    def onchange_local(self, cr, uid, ids, local_id, context=None):
        res = {}
        context = context and context.copy() or {}
        if not local_id:
            res['value'] = {'locacao_id': None}
        return res

    def gerar_inventario(self, cr, uid, ids, context=None, **kwargs):
        """
        Este metodo tem a funcionalidade de preparar os dadados informados para a geracao do
        inventario e fazer as chamadas dos metodos que criam  os registros estoque.inventario.capa e
        estoque.inventario.contagem
        :param cr: Cursor do openerp
        :param uid: Id do usuario requisita
        :param ids: Lista de ids gerados pela wizard
        :type cr: Curosr
        :type uid: int
        :type ids: int | long
        :type context: dict
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        context.update({'gerar_contagem_inventario': True})
        if isinstance(ids, (int, long)):
            ids = [ids]
        fields_to_read = [
            'local_id', 'categoria_id', 'fornecedor_id', 'locacao_id', 'name', 'mes', 'ano',
            'zerar_saldo_contado', 'efeito_de_inventario', 'zerar_saldo_negativo'
        ]
        wiz_contagem_dados = self.read(cr, uid, ids, fields_to_read, context=context,
                                       load=READ_LOAD)
        capa_ids = []
        for i in wiz_contagem_dados:
            cont_ano = i.get('ano')
            cont_mes = i.get('mes')
            cont_local_id = i.get('local_id') or 0
            cont_locacao_id = i.get('locacao_id') or 0
            cont_categoria_id = i.get('categoria_id') or 0
            cont_fornecedor_id = i.get('fornecedor_id') or 0
            cont_zerar_saldo_contado = i.get('zerar_saldo_contado') or False
            efeito_de_inventario = i.get('efeito_de_inventario') or False
            inventario_capa_id = self._gerar_inventario_capa(cr, uid, i.get('id'), context)
            capa_ids.append(inventario_capa_id)
            zerar_saldo_negativo = i.get('zerar_saldo_negativo')
            data_movimento = self._get_data_movimento(efeito_de_inventario, cont_ano, cont_mes)
            itens_para_contagem = self._gerar_estoque_inventario_contagem(
                cr, uid, inventario_capa_id, data_movimento, efeito_de_inventario,
                cont_zerar_saldo_contado, cont_local_id, cont_locacao_id, cont_categoria_id,
                cont_fornecedor_id, zerar_saldo_negativo, context=context, **kwargs)
            if not itens_para_contagem:
                raise osv.except_osv(_('Atencao'), _('Nao ha dados para Gerar a Contagem'))
            self.write(cr, uid, ids,
                       {'status': True, 'mensagem': _('Inventario gerado com sucesso!')},
                       context=context)

        self.pool.get('estoque.inventario.capa').write(cr, uid, capa_ids, {}, context=context)
        return True

    def _gerar_inventario_capa(self, cr, uid, wiz_gerar_contagem_id, context=None):
        """
        Metodo responsavel por tratar os valores informados na wizard para criar o inventario
        :param cr: Cursor do openerp
        :param uid: Id do usuario que esta chamando esta funcionalidade
        :param wiz_gerar_contagem_id: Id da wizard criada para geracao do inventario
        :param context: Dicionario contendo os valores contextuais
        :type cr: Curosr
        :type uid: int
        :type wiz_gerar_contagem_id: int | long
        :type context: dict
        :return: Retorna o id do inventario criado pela classe estoque.inventario.capa
        :rtype: int | long
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.inventario.capa', context=context)
        brw_gerar_inv = self.browse(cr, uid, wiz_gerar_contagem_id, context=context)
        vals = dict(descricao=brw_gerar_inv.name, company_id=company_id,
                    ano=brw_gerar_inv.ano, mes=brw_gerar_inv.mes)
        if brw_gerar_inv.local_id:
            vals['local_id'] = brw_gerar_inv.local_id.id
        if brw_gerar_inv.efeito_de_inventario:
            vals['atualizado'] = True

        return self.pool.get('estoque.inventario.capa').create(cr, uid, vals, context)

    def _get_data_movimento(self, efeito_de_inventario, ano, mes):
        """
        Metodo responsavel gerar a data para o inventario de acordo com o parametro
        efeito_de_inventario. Se efeito_de_inventario e verdadeiro retorna a data do movimento
        com o ultimo dia referente ao dia e o mes que foram informados para gerar o inventario.
        Se nao retorna a data do movimento com a data do dia da geracao do inventario.
        :param efeito_de_inventario: Se o inventario e apenas para efeito de inventario
        :param ano: Ano informado para a geracao do inventario
        :param mes: Mes informado para a geracao do inventario
        :type efeito_de_inventario: bool
        :type ano: str
        :type mes: str
        :return: Retorna a data para movimento do inventario de acordo com os parametros informados
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        data_movimento = date.today().strftime('%Y-%m-%d')
        if efeito_de_inventario:
            ultimo_dia_mes_ano = self._get_ultimo_dia_mes_ano(ano, mes)
            data_movimento = " '%s' " % ultimo_dia_mes_ano
        return data_movimento

    @staticmethod
    def _get_ultimo_dia_mes_ano(ano, mes):
        """
        Metodo responsavel por retornar o ultimo dia referente ao mes e o ano passados como
        parametro
        :param ano: Ano que deseja verificar
        :param mes: Mes que deseja verificar
        :type ano: str
        :type mes: str
        :return: Retorna a data contendo o ultimo dia do mes de acordo com os parametros passados
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return '%s-%s-%s' % (ano, mes, calendar.monthrange(int(ano), int(mes))[1])

    def _gerar_estoque_inventario_contagem(self, cr, uid, inventario_id, data_movimento,
                                           efeito_de_inventario, zerar_saldo, local_id=False,
                                           locacao_id=False, categoria_id=False,
                                           fornecedor_id=False, zerar_saldo_negativo=False,
                                           context=None, **kwargs):
        """
        Este metodo tem a funcionalidade de fazer a chamadas dos mestos responsaveis por gerar
        os valores iniciais para estoque.inventario.contagem
        :param cr: Cursor do openerp
        :param uid: Id do usuario requisita
        :param inventario_id: ID do inventario criado
        :param data_movimento: Data para movimento do inventario
        :param efeito_de_inventario: Se o inventario e apenas para efeito de inventario
        :param zerar_saldo: Se e pra zerar o saldo dos itens
        :param local_id: Id do local
        :param locacao_id: Id da locacao
        :param categoria_id: Id da categoria
        :param fornecedor_id: Id do fornecedor
        :param context: Dicionario contendo os valores contextuais
        :type cr: Curosr
        :type uid: int
        :type inventario_id: int
        :type data_movimento: str
        :type efeito_de_inventario: bool
        :type zerar_saldo: bool
        :type fornecedor_id: int
        :type context: dict
        :return: Retorna uma lista dos registros criados
        :rtype: list
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        return self.pool.get('estoque.inventario.contagem').gerar_contagem(
            cr, uid, inventario_id, data_movimento, efeito_de_inventario, zerar_saldo,
            local_id=local_id, locacao_id=locacao_id, categoria_id=categoria_id,
            fornecedor_id=fornecedor_id, zerar_saldo_negativo=zerar_saldo_negativo, context=context,
            **kwargs)
