# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from datetime import date
from openerp.tools.translate import _


class wizard_estoque_local_saldo(osv.TransientModel):
    _name = 'wizard.estoque.local.saldo'
    _order = 'local_id'
    _columns = {
        'local_id': fields.many2one('estoque.local', 'Local'),
        'saldo': fields.float('Saldo', digits=(14, 3)),
        'saldo_total': fields.float('Saldo total', digits=(14, 3)),
        'saldo_disponivel': fields.float('Saldo Disponivel', digits=(14, 3)),
        'qtd_reservada_wizard': fields.float('Quantidade reservada', digits=(14, 3)),
        'embalagem_id': fields.many2one('estoque.embalagem', 'Embalagem'),
        'preco_venda': fields.float('Preco Venda'),
        'consulta_saldo_id': fields.many2one('wizard.consulta.saldos', 'Consulta_id', ondelete="cascade")
    }


class wizard_estoque_consulta_saldos(osv.TransientModel):
    _name = 'wizard.consulta.saldos'

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(wizard_estoque_consulta_saldos, self).default_get(cr, uid, fields, context)
        produto_id = False
        if context.get('produto_id'):
            produto_id = context.get('produto_id')
        elif context.get('active_model') and context.get('active_id') and isinstance(context.get('active_id'), int):
            if context.get('active_model') == 'estoque.produto':
                produto_id = self.pool.get(context.get('active_model')).browse(cr, uid, context.get('active_id')).id
            else:
                produto_id = self.pool.get(context.get('active_model')).browse(cr, uid,
                                                                               context.get('active_id')).produto_id.id
        if context.get('active_model') == 'estoque.produto.grade' and context.get('active_id'):
            res['grade_id'] = context.get('active_id')
        if produto_id:
            res['produto_id'] = produto_id
        if produto_id and not res.get('grade_id'):
            ids_grade = self.pool.get('estoque.produto.grade').search(cr, uid, [('produto_id', '=', produto_id)],
                                                                      context=context)
            if ids_grade:
                if len(ids_grade) == 1:
                    res['grade_id'] = ids_grade[0]
        return res

    def _get_total_locais(self, cr, uid, ids, fields, args, context=None):
        context = context and context.copy() or {}
        res = {}
        for x in ids:
            total_itens = 0.0
            local_saldo_ids = self.browse(cr, uid, x).local_saldo_ids
            if local_saldo_ids:
                for y in local_saldo_ids:
                    if y.saldo_total > 0:
                        total_itens += y.saldo_total or 0.0
            res[x] = total_itens
        return res

    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa'),
        'produto_id': fields.many2one(
            'estoque.produto', "Produto",
            domain="[('tipo', '=', 'P'), ('company_id', '=', company_id)]"),
        'grade_id': fields.many2one('estoque.produto.grade', "Grade",
                                    domain="[('produto_id', '=', produto_id)]"),
        'local_saldo_ids': fields.one2many('wizard.estoque.local.saldo', 'consulta_saldo_id',
                                           'Local - Saldos'),
        'total_itens': fields.function(_get_total_locais, method=True, string="TOTAL", type="float",
                                       store=False),
        'reserva_venda_ids': fields.one2many('wizard.consulta.reserva.venda',
                                             'wcsrv_consulta_saldo_id', 'Reservas'),
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        context.update({'sig_force_company': True})
        res = super(wizard_estoque_consulta_saldos, self).create(cr, uid, vals, context)
        return res

    def onchange_company(self, cr, uid, ids, company_id, context=None):
        context = context and context.copy() or {}
        res = []
        empresa_estoque_produto_grade = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto.grade',
                                                                                       context=context)
        if company_id != empresa_estoque_produto_grade:
            res = {}.fromkeys(['produto_id', 'grade_id'], False)
        return {'value': res}

    def onchange_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        if context.get('active_model') == 'estoque.produto.grade' and context.get('active_id'):
            return {}
        res = {'grade_id': False}
        if produto_id:
            grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid, [('produto_id', '=', produto_id)],
                                                                      context=context)
            if len(grade_ids) == 1:
                res.update({'grade_id': grade_ids[0]})
        return {'value': res}

    def button_limpar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('current'):
            local_saldo_dados = self.read(cr, uid, ids, ['local_saldo_ids'], context=context)
            if isinstance(local_saldo_dados, list):
                local_saldo_dados = local_saldo_dados[0]
            if isinstance(local_saldo_dados, dict):
                if local_saldo_dados.get('local_saldo_ids'):
                    self.pool.get('wizard.estoque.local.saldo').unlink(cr, uid, local_saldo_dados['local_saldo_ids'],
                                                                       context=context)
            return self.write(cr, uid, ids, {'company_id': False, 'produto_id': False, 'grade_id': False},
                              context=context)
        # Para que a wizard nao feche ao clicar nos botoes, foi implementado a solucao abaixo, cuja funcao e retornar a
        # a mesma tela.
        # Essa implementacao deve ser desfeita caso este problema com as wizards seja resolvido.
        else:
            return {
                'res_id': False,
                'name': 'Consulta da Saldos',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wizard.consulta.saldos',
                'view_id': False,
                'target': 'new',
                'type': 'ir.actions.act_window'}

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        cr.execute('DELETE FROM wizard_consulta_reserva_venda')
        return self.funcao_pesquisar(cr, uid, ids, context=context)

    def funcao_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]

        consulta_saldo_dados = self.read(cr, uid, ids, ['company_id', 'produto_id', 'grade_id', 'local_saldo_ids'],
                                         context=context)
        if isinstance(consulta_saldo_dados, list):
            consulta_saldo_dados = consulta_saldo_dados[0]
        if isinstance(consulta_saldo_dados, dict):
            company_id = consulta_saldo_dados.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            produto_id = consulta_saldo_dados.get('produto_id')
            if isinstance(produto_id, tuple):
                produto_id = produto_id[0]
            grade_id = consulta_saldo_dados.get('grade_id')
            if isinstance(grade_id, tuple):
                grade_id = grade_id[0]
            obj_wizard_local_saldo = self.pool.get('wizard.estoque.local.saldo')
            if consulta_saldo_dados.get('local_saldo_ids'):
                obj_wizard_local_saldo.unlink(cr, uid, consulta_saldo_dados['local_saldo_ids'], context=context)
            obj_local_locacao = self.pool.get('estoque.local.locacao')
            local_locacao_ids = obj_local_locacao.search(cr, uid, [('company_id', '=', company_id),
                                                                   ('produto_id', '=', produto_id),
                                                                   ('grade_id', '=', grade_id)],
                                                         context=context)

            if not local_locacao_ids:
                raise osv.except_osv(_('Atencao'),
                                     _('Nenhum local foi encontrado para os criterios solicitados acima!'))
            else:
                local_locacao_dados = obj_local_locacao.read(cr, uid, local_locacao_ids,
                                                             ['local_id', 'estllc_produto_embalagem_id'],
                                                             context=context)
                try:
                    local_ids = tuple(map(lambda i: i['local_id'][0], local_locacao_dados))
                except:
                    local_ids = []
                try:
                    ids_produto_embalagem = tuple(
                        map(lambda x: x['estllc_produto_embalagem_id'][0], local_locacao_dados))
                except:
                    ids_produto_embalagem = []
                    for x in local_locacao_dados:
                        produto_embalagem_id = x.get('estllc_produto_embalagem_id')
                        if isinstance(produto_embalagem_id, (list, tuple)):
                            produto_embalagem_id = produto_embalagem_id[0]
                        ids_produto_embalagem.append(produto_embalagem_id)
                if ids_produto_embalagem:
                    ids_embalagem = []
                    for x in ids_produto_embalagem:
                        if x:
                            produto_embalagem_dados = self.pool.get('estoque.produto.embalagem').read(cr, uid, x,
                                                                                                      ['embalagem_id'],
                                                                                                      context=context)
                            ids_embalagem.append(produto_embalagem_dados.get('embalagem_id')[0])
                        else:
                            ids_embalagem.append(None)
                obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
                hoje = date.today()
                obj_estoque_embalagem = self.pool.get('estoque.embalagem')
                obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
                context.update({'company_id': company_id})
                for i in local_ids:
                    context.update({'sig_force_company': True})
                    saldo_produto = obj_estoque_produto_saldo.get_saldo_do_dia(
                        cr, uid,
                        eps_grade_ids=grade_id,
                        eps_data=hoje,
                        eps_local_id=i,
                        context=context)
                    # o contexto 'sig_force_company' deve ser passado apenas no saldo que nao eh roteavel
                    context.pop('sig_force_company')
                    saldo = saldo_produto.get(grade_id).get('saldo')
                    saldo_disponivel = saldo_produto.get(grade_id).get('disponivel')
                    qtd_reservada_wizard = saldo_produto.get(grade_id).get('qtd_reservada')
                    try:
                        embalagem_id = ids_embalagem[local_ids.index(i)]
                    except:
                        embalagem_id = None
                    preco = None
                    quantidade = 1
                    if embalagem_id:
                        estoque_embalagem_dados = obj_estoque_embalagem.read(cr, uid, [embalagem_id],
                                                                             ['quantidade_na_embalagem'],
                                                                             context=context)
                        if isinstance(estoque_embalagem_dados, list):
                            estoque_embalagem_dados = estoque_embalagem_dados[0]
                        quantidade = estoque_embalagem_dados.get('quantidade_na_embalagem') or 1
                    preco_venda = obj_estoque_preco_venda.get_preco_venda(cr, uid, produto_id,
                                                                          epc_company_id=company_id,
                                                                          epc_grade_ids=grade_id,
                                                                          context=context)
                    dados_preco_venda = preco_venda.get(produto_id) or {}
                    preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                        'preco_venda')
                    if preco_venda:
                        preco = preco_venda * quantidade

                    if saldo:
                        sal = saldo / quantidade
                        saldo_total = saldo
                        saldo_disponivel = saldo_disponivel / quantidade
                    else:
                        sal = saldo
                        saldo_total = saldo
                    obj_wizard_local_saldo.create(cr, uid, {'local_id': i, 'embalagem_id': embalagem_id,
                                                            'preco_venda': preco, 'saldo': sal,
                                                            'saldo_total': saldo_total, 'consulta_saldo_id': ids,
                                                            'qtd_reservada_wizard': qtd_reservada_wizard,
                                                            'saldo_disponivel': saldo_disponivel},
                                                  context=context)
                    # Para que a wizard nao feche ao clicar nos botoes, foi implementado a solucao abaixo, cuja funcao e retornar a
        # a mesma tela. Nessa situacao, e passado o id do registro criado para que, ao abrir a tela novamente, ja venha com
        # os dados pesquisados.
        # Essa implementacao deve ser desfeita caso este problema com as wizards seja resolvido.
        if not context.get('current'):
            return {
                'res_id': ids,
                'name': _('Consulta da Saldos'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wizard.consulta.saldos',
                'view_id': False,
                'target': 'new',
                'type': 'ir.actions.act_window'}
        return True
