# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_estoque_consulta_movimento(osv.TransientModel):

    _name = "wizard.estoque.consulta.movimento"
    _columns = {
        'res_id': fields.many2one('res.partner', 'RES ID'),
        'company_id': fields.many2one('res.company', 'Empresa'),
        'data_inicio': fields.date('Data Inicio'),
        'data_fim': fields.date('Data Fim'),
        'tipo_movimento': fields.selection([('E', 'Entrada'), ('S', 'Saida')], 'Tipo', size=1),
        'operacao_id': fields.many2one('estoque.natureza.operacao', 'Operacao'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Pessoa'),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade', domain="[('produto_id','=',produto_id)]"),
        'local_id': fields.many2one('estoque.local', 'Local'),
        'produto_id': fields.many2one('estoque.produto', 'Produto'),
        'categoria_id': fields.many2one('estoque.categoria',' Categoria'),
        'origem': fields.char('Origem', readonly=True),
        'origem_id': fields.many2one('estoque.movimento.origem', 'Origem'),
        'inventario_id': fields.many2one('estoque.inventario.capa', 'Inventario', ondelete='cascade'),
        'itens_ids': fields.one2many('wizard.estoque.consulta.movimento.itens', 'capa_id', 'Itens'),
        'possui_movimento_acima_limite': fields.boolean('Possui movimento acima do limite'),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id,
    }

    def onchange_tipo_movimento(self, cr, uid, ids, tipo_movimento, context=None):
        context = context and context.copy() or {}
        res = {}
        if tipo_movimento:
            res['operacao_id'] = False
        return {'value': res}

    def onchange_operacao_id(self, cr, uid, ids, operacao_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if operacao_id:
            res['tipo_movimento'] = ''
        return {'value': res}

    def onchange_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        grade_id = False
        if produto_id:
            obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
            grade_ids = obj_estoque_produto_grade.search(cr, uid, [('produto_id', '=', produto_id)], context=context)
            if grade_ids.__len__() == 1:
                grade_id = grade_ids[0]
        return {'value': {'grade_id': grade_id}}

    def onchange_origem(self, cr, uid, ids, origem_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'origem': ""}}
        obj_est_mov_origem = self.pool.get('estoque.movimento.origem')
        origens = obj_est_mov_origem.get_movimento_origem(cr, uid, origem_id, context=context)
        for origem in origens:
            res = {'value': {'origem': origem[0]}}
        return res

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids[0], {'possui_movimento_acima_limite': False}, context)
        obj_wizard_estoque_consulta_movimento_itens = self.pool.get('wizard.estoque.consulta.movimento.itens')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_adm_util = self.pool.get('adm.utils')
        obj_estoque_categoria = self.pool.get('estoque.categoria')
        cr.execute('''DELETE FROM wizard_estoque_consulta_movimento_itens''')
        filtro_dados = self.browse(cr, uid, ids[0])
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'estoque.movimento.itens', context=context)
        data_inicio = filtro_dados.data_inicio
        data_fim = filtro_dados.data_fim
        operacao_id = filtro_dados.operacao_id.id
        pessoa_id = filtro_dados.pessoa_id.id
        grade_id = filtro_dados.grade_id.id
        local_id = filtro_dados.local_id.id
        produto_id = filtro_dados.produto_id.id
        categoria_id = filtro_dados.categoria_id.id
        origem_id = filtro_dados.origem_id.id
        origem = filtro_dados.origem or (filtro_dados.origem_id.estmog_codigo if origem_id else '')
        tipo_movimento = filtro_dados.tipo_movimento
        nf_id = False
        cupom_id = False
        inventario_id = filtro_dados.inventario_id.id
        requisicao_alm_id = False
        requisicao_res_id = False
        cfop_ids = False
        list_documento = [origem, inventario_id, produto_id, pessoa_id, categoria_id]
        if obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'fiscal', context):
            nf_id = filtro_dados.nf_id.id
            list_documento.append(nf_id)
            cfop_ids = map(lambda i: i.id, filtro_dados.cfop_ids)
            if len(cfop_ids) == 1:
                cfop_ids = str(tuple(cfop_ids)).replace(',', '')
            elif len(cfop_ids) > 1:
                cfop_ids = str(tuple(cfop_ids))
        ecf_instalado = obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'ecf', context)
        if ecf_instalado:
            cupom_id = filtro_dados.cupom_id.id
            list_documento.append(cupom_id)
        if obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'estoquealmoxarifado', context):
            requisicao_alm_id = filtro_dados.requisicao_almoxarifado_id.id
            list_documento.append(requisicao_alm_id)
        if obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'estoqueressuprimento', context):
            requisicao_res_id = filtro_dados.requisicao_ressuprimento_id.id
            list_documento.append(requisicao_res_id)
        if not any(list_documento):
            raise osv.except_osv(_('Atencao'), _('Deve ser informado no minimo um dos seguintes '
                                                 'filtros: "Categoria", "Produto", "Pessoa" ou '
                                                 '"Origem"'))
        madereira_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'madeireira', context)
        if madereira_instalado:
            volume = 'mov_item.estmvi_volume,'
        else:
            volume = ''
        if ecf_instalado:
            cupom = 'mov_item.cupom_id, mov_item.item_cancelado,'
        else:
            cupom = ''

        sql = """
            SELECT
                mov_item.id as movimento_id,
                %s
                %s
                mov_item.capa_id as movimento_capa_id,
                (select numero_documento from estoque_movimento_capa where id = mov_item.capa_id)
                AS numero_documento,
                (select company_id from estoque_movimento_capa where id = mov_item.capa_id)
                AS company_id,
                (select pessoa_id from estoque_movimento_capa where id = mov_item.capa_id)
                AS pessoa_id,
                (select data from estoque_movimento_capa where id = mov_item.capa_id) AS data,
                (select estmvc_operacao_id from estoque_movimento_capa where id = mov_item.capa_id)
                AS operacao_id,
                (select estnto_tipo_operacao from estoque_natureza_operacao where id =
                    (select estmvc_operacao_id from estoque_movimento_capa where id =
                     mov_item.capa_id)) AS tipo_movimento,
                (select funcionario_id from estoque_movimento_capa where id = mov_item.capa_id)
                AS funcionario_id,
                mov_item.local_id AS local_id,
                mov_item.produto_id AS produto_id,
                (select codigo from estoque_produto where id = mov_item.produto_id)
                AS produto_codigo,
                (select categoria_id from estoque_produto where id = mov_item.produto_id)
                AS categoria_id,
                mov_item.grade_id AS grade_id,
                mov_item.produto_embalagem_id AS produto_embalagem_id,
                mov_item.quantidade AS quantidade,
                mov_item.estmvi_entrada_preco_custo_id AS preco_custo_id,
                -- COLUNAS DO FISCAL
                mov_item.vr_total_item_rateado AS vr_total_item,
                (select codigo from fiscal_cfop where id = mov_item.cfop_id) AS cfop_codigo,
                (select movimento_tipo from fiscal_cfop where id = mov_item.cfop_id)
                AS cfop_movimento,

                CASE WHEN mov_item.quantidade > 0
                    then mov_item.vr_total_item / quantidade
                    else 0
                END
                AS vr_unitario_item,

                CASE WHEN mov_item.estmvi_saidas_custo_medio_contabil_total is not null
                    then mov_item.vr_total_item - mov_item.estmvi_saidas_custo_medio_contabil_total
                    else mov_item.vr_total_item
                END
                AS vr_agregado_custo_medio,

                CASE WHEN mov_item.estmvi_saidas_custo_corrigido_total is not null
                    then mov_item.vr_total_item - mov_item.estmvi_saidas_custo_corrigido_total
                    else mov_item.vr_total_item
                END
                AS vr_agregado_custo_corrigido,

                CASE WHEN mov_item.estmvi_saidas_custo_reposicao_total is not null
                    then mov_item.vr_total_item - mov_item.estmvi_saidas_custo_reposicao_total
                    else mov_item.vr_total_item
                END
                AS vr_agregado_custo_reposicao,

                CASE WHEN mov_item.estmvi_saidas_ultimo_custo_total is not null
                    then mov_item.vr_total_item - mov_item.estmvi_saidas_ultimo_custo_total
                    else mov_item.vr_total_item
                END
                AS vr_agregado_ultimo_custo,

                CASE
                    WHEN mov_item.vr_total_item = 0 then 0
                    WHEN mov_item.estmvi_saidas_custo_medio_contabil_total is null then 100
                    else 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_custo_medio_contabil_total) /
                                mov_item.vr_total_item
                END
                AS porc_venda_custo_medio,

                CASE
                    WHEN mov_item.vr_total_item = 0 then 0
                    WHEN mov_item.estmvi_saidas_custo_corrigido_total is null then 100
                    else 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_custo_corrigido_total) /
                                mov_item.vr_total_item
                END
                AS porc_venda_custo_corrigido,

                CASE
                    WHEN mov_item.vr_total_item = 0 then 0
                    WHEN mov_item.estmvi_saidas_custo_reposicao_total is null then 100
                    else 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_custo_reposicao_total) /
                                mov_item.vr_total_item
                END
                AS porc_venda_custo_reposicao,

                CASE
                    WHEN mov_item.vr_total_item = 0 then 0
                    WHEN mov_item.estmvi_saidas_ultimo_custo_total is null then 100
                    else 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_ultimo_custo_total) /
                                mov_item.vr_total_item
                END
                AS porc_venda_ultimo_custo,

                CASE
                    WHEN mov_item.estmvi_saidas_custo_medio_contabil_total = 0 or mov_item.estmvi_saidas_custo_medio_contabil_total is null
                        THEN 0
                    ELSE 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_custo_medio_contabil_total) /
                                mov_item.estmvi_saidas_custo_medio_contabil_total
                END
                AS porc_custo_custo_medio,

                CASE
                    WHEN mov_item.estmvi_saidas_custo_corrigido_total = 0 or mov_item.estmvi_saidas_custo_corrigido_total is null
                        THEN 0
                    ELSE 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_custo_corrigido_total) /
                                mov_item.estmvi_saidas_custo_corrigido_total
                END
                AS porc_custo_custo_corrigido,

                CASE
                    WHEN mov_item.estmvi_saidas_custo_reposicao_total = 0 or mov_item.estmvi_saidas_custo_reposicao_total is null
                        THEN 0
                    ELSE 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_custo_reposicao_total) /
                                mov_item.estmvi_saidas_custo_reposicao_total
                END
                AS porc_custo_custo_reposicao,

                CASE
                    WHEN mov_item.estmvi_saidas_ultimo_custo_total = 0 or mov_item.estmvi_saidas_ultimo_custo_total is null
                        THEN 0
                    ELSE 100 * (mov_item.vr_total_item - mov_item.estmvi_saidas_ultimo_custo_total) /
                                mov_item.estmvi_saidas_ultimo_custo_total
                END
                AS porc_custo_ultimo_custo,

                -- ORIGEM
                (SELECT estmog_codigo
                   FROM estoque_movimento_origem
                   WHERE id = (SELECT estnto_movimento_origem_id
                               FROM estoque_natureza_operacao
                               WHERE id = (SELECT estmvc_operacao_id
                                           FROM estoque_movimento_capa
                                           WHERE id = mov_item.capa_id))) AS origem,
                -- CUSTO CORRIGIDO
                mov_item.estmvi_operacao_custo_corrigido_unitario AS custo_corrigido,

                -- CUSTO MEDIO CONTABIL
                mov_item.estmvi_operacao_custo_medio_contabil_unitario AS custo_medio_contabil,

                -- CUSTO REPOSICAO
                mov_item.estmvi_operacao_custo_reposicao_unitario AS custo_reposicao,

                -- ULTIMO CUSTO
                mov_item.estmvi_operacao_ultimo_custo_unitario AS ultimo_custo

            FROM estoque_movimento_itens mov_item
            WHERE mov_item.capa_id IN (SELECT id
                                       FROM estoque_movimento_capa
                                       WHERE data >= '%s'
                                             AND data <= '%s'
                                             AND company_id = %s
                                             AND state = 'F'
        """ % (volume, cupom, data_inicio, data_fim, company_id)
        sql_where = ''
        if any([operacao_id, pessoa_id, origem, tipo_movimento, nf_id, cupom_id, inventario_id,
                requisicao_res_id, requisicao_alm_id]):
            if tipo_movimento and not operacao_id:
                natureza_operacao_ids = self.pool.get('estoque.natureza.operacao').search(
                    cr, uid, [('estnto_tipo_operacao', '=', tipo_movimento)], context=context)
                if natureza_operacao_ids:
                    if len(natureza_operacao_ids) == 1:
                        natureza_operacao_ids = str(tuple(natureza_operacao_ids)).replace(',', '')
                    else:
                        natureza_operacao_ids = str(tuple(natureza_operacao_ids))
                    sql_where += '''AND estmvc_operacao_id in %s ''' % natureza_operacao_ids
            if nf_id:
                sql_where += '''AND nf_capa_id = %s ''' % nf_id
            if cupom_id:
                sql_where += '''AND cupons_capa_id = %s ''' % cupom_id
            if inventario_id:
                sql_where += '''AND inventario_id = %s ''' % inventario_id
            if requisicao_alm_id:
                sql_where += '''AND requisicao_id = %s ''' % requisicao_alm_id
            if requisicao_res_id:
                sql_where += '''AND requisicaoressuprimento_id = %s ''' % requisicao_res_id
            if operacao_id and not tipo_movimento:
                sql_where += '''AND estmvc_operacao_id = %s ''' % operacao_id
            if pessoa_id:
                sql_where += '''AND pessoa_id = %s ''' % pessoa_id
            if origem:
                sql_where += '''
                AND estmvc_operacao_id IN (SELECT id
                                                            FROM estoque_natureza_operacao
                                                            WHERE estnto_movimento_origem_id = %s)
                ''' % origem_id
        sql_where += ''') '''
        if local_id:
            sql_where += '''AND mov_item.local_id = %s ''' % local_id
        if categoria_id:
            categoria_ids = obj_estoque_categoria.search(
                cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)
            estoque_categoria_dados = obj_estoque_categoria.read(cr, uid, categoria_ids,
                                                                 ['produto_ids'], context=context)
            produto_ids = list()
            map(lambda p: produto_ids.extend(p.get('produto_ids')), estoque_categoria_dados)
            produto_ids = list(set(produto_ids))
            if produto_ids:
                if len(produto_ids) == 1:
                    produto_ids = str(tuple(produto_ids)).replace(',', '')
                else:
                    produto_ids = str(tuple(produto_ids))
                sql_where += '''AND mov_item.produto_id in %s ''' % produto_ids
            else:
                sql_where += '''AND mov_item.produto_id IS NULL '''
        if produto_id:
            if sql_where:
                sql_where += '''AND mov_item.produto_id = %s ''' % produto_id
            else:
                sql_where += '''mov_item.produto_id = %s ''' % produto_id
        if grade_id:
            sql_where += '''AND mov_item.grade_id = %s ''' % grade_id
        if cfop_ids:
            sql_where += ''' AND mov_item.cfop_id IN %s ''' % cfop_ids
        sql += sql_where
        dados = self.executar_sql_consulta_movimento(cr, sql)

        if dados:
            if len(dados) > 2000:
                self.write(cr, uid, ids[0], {'possui_movimento_acima_limite': True}, context)
            cont = 0
            if ecf_instalado:
                dados_sem_cupom = filter(lambda i: not i.get('cupom_id'), dados)
                dados_cupom_sem_cancelamento = filter(lambda i: i.get('cupom_id') and
                                                      not i.get('item_cancelado'), dados)
                dados_cupom_cancelamento = filter(lambda i: i.get('cupom_id') and
                                                            i.get('item_cancelado'), dados)
                cupom_cancelado_ids = self.pool.get('ecf.cupons.capa').search(
                    cr, uid, [('date_cupom', '>=', data_inicio), ('date_cupom', '<=', data_fim),
                              ('tipo_cupom', '=', 'C')], context=context)
                dados_cupom_cancelamento = filter(lambda i:
                                                  i.get('cupom_id') in cupom_cancelado_ids,
                                                  dados_cupom_cancelamento)
                for x in dados_cupom_cancelamento:
                    x['quantidade'] *= -1
                    x['vr_total_item'] *= -1
                dados_sem_cupom.extend(dados_cupom_cancelamento)
                dados_sem_cupom.extend(dados_cupom_sem_cancelamento)
                dados = dados_sem_cupom
            for i in dados:
                cont += 1
                if cont > 2000:
                    break
                i['capa_id'] = ids[0]
                del i['movimento_capa_id']
                del i['tipo_movimento']
                if ecf_instalado:
                    del i['cupom_id']
                    del i['item_cancelado']
                brw_estoque_embalage = obj_estoque_produto_embalagem.browse(
                    cr, uid, i.get('produto_embalagem_id')).embalagem_id
                if brw_estoque_embalage:  # Converte a quantidade para o quantidade informada no
                                          # na embalagem.
                    if brw_estoque_embalage.quantidade_na_embalagem > 0:
                        i['quantidade'] /= brw_estoque_embalage.quantidade_na_embalagem
                obj_wizard_estoque_consulta_movimento_itens.create(cr, uid, i, context)

        return True

    # noinspection PyMethodMayBeStatic
    def executar_sql_consulta_movimento(self, cr, sql):
        """
        Executa sentenca SQL e retorna os dados da consulta.

        :param cr: cursor do banco de dados,
        :param sql: a sentenca sql a ser executada
        :type cr: cursor
        :type sql: str
        :return: lista de dicionario com os dados da consulta
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        cr.execute(sql)

        return cr.dictfetchall()


class wizard_estoque_consulta_movimento_itens(osv.TransientModel):

    from funcoes import get_movimento_item_tributacao
    from funcoes import get_sigla_embalagem
    from funcoes import get_numero_local
    from funcoes import get_name_origem

    _name = "wizard.estoque.consulta.movimento.itens"

    _columns = {
        'capa_id': fields.many2one('wizard.estoque.consulta.movimento', 'Capa'),
        'company_id': fields.many2one('res.company','Empresa'),
        'data': fields.date('Data'),
        'operacao_id': fields.many2one('estoque.natureza.operacao','Operacao'),
        'funcionario_id': fields.many2one('rh.funcionario','Funcionario'),
        'origem': fields.function(get_name_origem, string='Origem', type='char', readonly=True,
                                  store=False, method=True),
        'pessoa_id': fields.many2one('adm.pessoa', 'Pessoa'),
        'cfop_movimento': fields.selection([('C','Compra/Venda'),
                                            ('D', 'Devolucao'),
                                            ('T', 'Transferencia'),
                                            ('E', 'Remessa'),
                                            ('R','Retorno'),
                                            ('U','Outras')], 'Movimento tipo', size=1),
        'cfop_codigo': fields.char('Codigo CFOP', size=4),
        'custo_corrigido': fields.float('Custo Corrigido', digits=(15,2)),
        'custo_medio_contabil': fields.float('Custo Medio Contabil', digits=(15,2)),
        'custo_reposicao': fields.float('Custo Reposicao', digits=(15,2)),
        'grade_id': fields.many2one('estoque.produto.grade','Grade', domain="[('produto_id', '=', esgcit_produto_id)]"),
        'local_id': fields.many2one('estoque.local','Local'),
        'produto_embalagem_id': fields.many2one('estoque.produto.embalagem','Embalagem'),
        'produto_id': fields.many2one('estoque.produto','Produto'),
        'produto_codigo': fields.char('Codigo Produto', size=20),
        'quantidade': fields.float('Quantidade', digits=(12,3)),
        'ultimo_custo': fields.float('Ultimo Custo', digits=(15,2)),
        'categoria_id': fields.many2one('estoque.categoria','Categoria'),
        'vr_total_item': fields.float('Valor Total Item', digits=(15,2)),
        'vr_unitario_item': fields.float('Valor Unitario Item', digits=(15,2)),
        'movimento_id': fields.many2one('estoque.movimento.itens','Movimento'),
        'vr_agregado_custo_medio': fields.float('Valor Agregado Custo Medio', digits=(15,2)),
        'vr_agregado_custo_corrigido': fields.float('Valor Agregado Custo Corrigido', digits=(15,2)),
        'vr_agregado_custo_reposicao': fields.float('Valor Agregado Custo Reposicao', digits=(15,2)),
        'vr_agregado_ultimo_custo': fields.float('Valor Agregado Ultimo Custo', digits=(15,2)),
        'porc_venda_custo_medio': fields.float('% Venda Custo Medio', digits=(15,2)),
        'porc_venda_custo_corrigido': fields.float('% Venda Custo Corrigido', digits=(15,2)),
        'porc_venda_custo_reposicao': fields.float('% Venda Custo Reposicao', digits=(15,2)),
        'porc_venda_ultimo_custo': fields.float('% Venda Ultimo Custo', digits=(15,2)),
        'porc_custo_custo_medio': fields.float('% Compra Custo Medio', digits=(15,2)),
        'porc_custo_custo_corrigido': fields.float('% Compra Custo Corrigido', digits=(15,2)),
        'porc_custo_custo_reposicao': fields.float('% Compra Custo Reposicao', digits=(15,2)),
        'porc_custo_ultimo_custo': fields.float('% Compra Ultimo Custo', digits=(15,2)),
        'numero_documento': fields.char('Nro Documento'),
        'item_tributacao_ids': fields.function(get_movimento_item_tributacao, method=True, string="Tributacao",
            store=False, type="one2many", relation="wizard.estoque.consulta.movimento.tributacao"),
        'sigla_embalagem': fields.function(get_sigla_embalagem, method=True, string='Embalagem',
                                           type='char', store=False),
        'numero_local': fields.function(get_numero_local, method=True, string='Local',
                                        type='integer', store=False),
        }


class wizard_estoque_consulta_movimento_tributacao(osv.TransientModel):

    _name = "wizard.estoque.consulta.movimento.tributacao"
    _columns = {
        'consulta_movimento_item_id': fields.many2one('wizard.estoque.consulta.movimento.itens', 'Movimento item'),
        'encargo' : fields.char('Encargo', size=20),
        'cst' : fields.char('Cst', size=50),
        'tributacao' : fields.char('Tributacao', size=50),
        'aliquota' : fields.float('Aliquota', digits=(6,3)),
        'vr_base_calculo' : fields.float('Vr base calculo', digits=(15,2)),
        'vr_encargo' : fields.float('Vr encargo', digits=(15,2)),
    }
