# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from random import randint
import base64
import time
from SigUtils.formatastring import formataString
from SigUtils.numero import getNumeroEditado
from openerp.tools.translate import _


class wizard_inventario_exportar_coletor(osv.TransientModel):

    _name = 'wizard.inventario.exportar.coletor'
    _columns = {
        'local_id': fields.many2one('estoque.local', 'Local', required=True),
        'locacao': fields.char('Locacao', size=15),
        'fornecedor_id': fields.many2one('financeiro.fornecedor', 'Fornecedor'),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'arquivo': fields.binary('Arquivo', readonly=True),
        'arquivo_nome': fields.char('Nome arquivo', size=20),
    }
    _defaults = {
        'arquivo_nome': 'produtos_coletor.txt',
    }

    def button_processar(self, cr, uid, ids, context=None):
        """
        O metodo encontra todos os dados dos produtos que serao exportados para o coletor.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: dados de valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso seja encontrado algum produto. Senao, falso.
        :rtype: bool
        """
        context = context and context.copy() or {}

        if not self.pool.get('adm.utils').admutl_check_modulo_instalado(
                cr, uid, 'automacao', context):
            raise osv.except_osv(_('Atencao'), _("Modulo 'automacao' deve estar instalado para "
                                                 "realizar esta operacao."))

        exportar_coletor_id = ids[0] if isinstance(ids, list) else ids
        brw_wizard_inventario_exportar_coletor = self.browse(cr, uid, exportar_coletor_id, context)
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context)
        sql = """
            SELECT
                ep.id                       AS produto_id,
                epg.id                      AS grade_id,
                acb.codigo_barra::bigint    AS codigo_barra,
                epg.descricao               AS descricao
            FROM
                estoque_local_locacao ell
            INNER JOIN estoque_produto ep
                ON ep.id = ell.produto_id
            INNER JOIN estoque_produto_grade epg
                ON epg.id = ell.grade_id
            INNER JOIN automacao_codigo_barra acb
                ON ep.id = acb.produto_id AND epg.id = acb.grade_id
            WHERE
                ell.company_id = %s
                AND ell.local_id = %s
        """ % (company_id, brw_wizard_inventario_exportar_coletor.local_id.id)
        if brw_wizard_inventario_exportar_coletor.locacao:
            sql += """
                AND ell.locacao = '%s'
            """ % brw_wizard_inventario_exportar_coletor.locacao
        if brw_wizard_inventario_exportar_coletor.fornecedor_id:
            sql += """
                AND ep.id IN (SELECT produto_id FROM estoque_fornecedor WHERE fornecedor_id = %s)
            """ % brw_wizard_inventario_exportar_coletor.fornecedor_id.id
        if brw_wizard_inventario_exportar_coletor.categoria_id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of',
                           brw_wizard_inventario_exportar_coletor.categoria_id.id)],
                context=context)
            if len(categoria_ids) == 1:
                categoria_ids = str(tuple(categoria_ids)).replace(',', '')
            else:
                categoria_ids = str(tuple(categoria_ids))
            sql += """
                AND ep.id IN (SELECT id FROM estoque_produto WHERE categoria_id IN %s)
            """ % categoria_ids
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            raise osv.except_osv(_('Atencao'), _('Nenhum produto encontrado para os filtros '
                                                 'selecionados.'))
        arquivo = self.gerar_arquivo_exportacao(
            cr, uid, brw_wizard_inventario_exportar_coletor.local_id.id, dados, context)
        vals = {'arquivo': False}
        if arquivo:
            vals = {'arquivo': base64.encodestring(arquivo)}
        self.write(cr, uid, exportar_coletor_id, vals, context)
        return any(dados)

    def gerar_arquivo_exportacao(self, cr, uid, local_id, dados, context=None):
        """
        O metodo inclui e formata num arquivo txt os dados dos produtos que serao exportados para o
        coletor. Esse arquivo estara disponivel para download.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param local_id: id da tabela estoque_local. Local de onde sera buscado o saldo dos
        produtos.
        :type local_id: int
        :param dados: dados dos produtos que serao exportados.
        :type dados: list
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados do arquivo gerado.
        :rtype: str
        """
        context = context and context.copy() or {}

        if not dados:
            return ''

        dados = filter(lambda d: len(str(d.get('codigo_barra'))) <= 13, dados)
        if not dados:
            return ''

        arquivo_diretorio = '/tmp/inventario_exportar_coletor-%s.txt' % randint(1, 1000)
        arquivo = open(arquivo_diretorio, 'wb')
        line_break = '\r\n'

        produto_ids = list(set(map(lambda i: i.get('produto_id'), dados)))
        grade_ids = list(set(map(lambda i: i.get('grade_id'), dados)))

        dados_preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, epc_produto_ids=produto_ids, epc_grade_ids=grade_ids, context=context)
        dados_saldo = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
            cr, uid, grade_ids, time.strftime('%Y-%m-%d'), local_id, context=context)

        for dado in dados:
            produto_id = dado.get('produto_id')
            grade_id = dado.get('grade_id')

            preco_venda = dados_preco_venda.get(produto_id) or 0.0
            if preco_venda:
                preco_venda = preco_venda.get(grade_id) or preco_venda.get('preco_venda')

            saldo = dados_saldo.get(grade_id) or 0.0
            if saldo:
                saldo = saldo.get('saldo')

            linha = str(dado.get('codigo_barra')).zfill(13)
            linha += formataString(dado.get('descricao'), 20, ' ', 'D', 'N', 'S')
            linha += formataString(getNumeroEditado(preco_venda), 10, ' ', 'E', 'N', 'S')
            linha += formataString(getNumeroEditado(saldo, 3), 10, ' ', 'E', 'N', 'S')
            linha += line_break
            arquivo.write(linha)

        arquivo.close()
        return open(arquivo_diretorio, 'r').read()

