# -*- coding: utf-8 -*-
from openerp.osv import osv, fields
from base64 import decodestring
from openerp.tools.translate import _


class wizard_inventario_importar_coletor(osv.TransientModel):
    _name = 'wizard.inventario.importar.coletor'
    _description = 'Wizard Importar Coletor'
    _columns = {
        'local_id': fields.many2one("estoque.local", 'Local', required=True),
        'arquivo': fields.binary('Arquivo', required=True),
        'somar_contagem': fields.boolean('Somar contagem de codigos repetidos',
                                         help='Para usar esta opcao, o inventario tem que ter sido '
                                              'gerado com a opcao "Zerar contagem" marcada.'),
        'state': fields.selection([('N', 'Aguardando executar importacao'),
                                   ('S','Importacao executada com sucesso')],
                                  'Importado', required=True, size=1, readonly=True),
        'log': fields.text('Log', readonly=True)
    }

    _defaults = {
        'state': 'N'
    }

    def button_importar_arquivo_coletor(self, cr, uid, ids, context=None):
        from collections import defaultdict

        context = context and context.copy() or {}
        log_msg = [_('>>> CONTAGEM CONCLUIDA (Clique em "Fechar" para sair desta tela)')]
        inventario_id = context.get('active_id')
        data_self = self.read(cr, uid, ids[0], ['arquivo', 'local_id'], context=context,
                              load='_classic_write')

        # ___________________________ Objects ________________________________
        obj_contagem = self.pool.get('estoque.inventario.contagem')
        obj_codigo_barra = self.pool.get('automacao.codigo.barra')

        # Registros do arquivo de inventario
        str_registros_nao_importados = str()
        registros = list()
        for registro in filter(lambda f: f.startswith('INVE'),
                               decodestring(data_self.get('arquivo')).splitlines()):
            try:
                registros.append((registro[12:25], float(registro[-8:])/1000.))
            except:
                str_registros_nao_importados += registro + '\n'

        if str_registros_nao_importados:
            log_msg.append(_('>>> Os registros abaixo nao foram importados: \n %s' %
                             str_registros_nao_importados))

        inventario = defaultdict(list)  # Dados do inventario (codigo de barras e quantidade)

        # Lista com as respectivas contagens de cada codigo de barras
        for chave, valor in registros:
            inventario[chave].append(valor)

        # De acordo com o parametro ´somar_contagem´, totaliza as quantidades inventariadas de cada
        # produto (codigo de barras)
        if self.read(cr, uid, ids[0], ['somar_contagem'], context).get('somar_contagem'):
            inventario = [{chave: sum(inventario[chave])} for chave in sorted(inventario)]
        else:
            inventario = [{chave: inventario[chave][-1]} for chave in sorted(inventario)]

        local_id = data_self.get('local_id')
        codigos_barras = map(lambda x: x.keys()[0], inventario)
        codigo_barra_ids = obj_codigo_barra.search(cr, uid,
                                          [('codigo_barra', 'in', codigos_barras)], context=context)
        # chave=codigo de barras, valor=Dict(chaves= 'produto_id' e 'grade_id', valores = ids dos
        #  produtos e grades)
        dict_codigo_dados = {}
        produto_ids = []
        grade_ids = []
        for cb in obj_codigo_barra.read(cr, uid, codigo_barra_ids, ['codigo_barra', 'produto_id',
                       'grade_id', 'produto_embalagem_id'], context=context, load='_classic_write'):
            dict_codigo_dados.update({cb.get('codigo_barra'): {'produto_id': cb.get('produto_id'),
                                                               'grade_id': cb.get('grade_id'),
                                             'produto_embalagem_id': cb.get('produto_embalagem_id')}
                                    })
            produto_ids.append(cb.get('produto_id'))
            grade_ids.append(cb.get('grade_id'))

        obj_local_loc = self.pool.get('estoque.local.locacao')
        args = [('produto_id', 'in', produto_ids),
                ('grade_id', 'in', grade_ids),
                ('local_id', '=', local_id)]
        local_loc_ids = obj_local_loc.search(cr, uid, args, context=context)
        # chave= produto_id-grade_id, valor = local_locacao_id
        dict_local_locacao = {}
        map(lambda x: dict_local_locacao.update(
            {'%s-%s' % (x.get('produto_id'), x.get('grade_id')): x.get('id')}),
            obj_local_loc.read(cr, uid, local_loc_ids, ['produto_id', 'grade_id'], context=context,
                               load='_classic_write'))
        grade_qtd_embalagem = self.pool.get('estoque.utils').get_grade_qtd_embalagem(cr, grade_ids,
                                                                                    context=context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_inventario_tipo_contagem', context=context)
        tipo_contagem = estoque_parametro_dados.get('estpar_inventario_tipo_contagem')
        args_contagem = [('inventario_id', '=', inventario_id)]
        contagem_ids = obj_contagem.search(cr, uid, args_contagem, context=context)
        dict_contagem = {}
        if contagem_ids:
            for i in obj_contagem.read(cr, uid, contagem_ids, ['produto_id', 'grade_id',
                                            'local_id'], context=context, load='_classic_write'):
                chave = '%s-%s-%s' % (i.get('produto_id'), i.get('grade_id'), i.get('local_id'))
                dict_contagem[chave] = i.get('id')

        produto_local_diferente = []
        grade_ids_sem_contagem = []
        context_aux = context.copy()
        context_aux['inventario_importado_do_coletor'] = True
        produto_ids = []
        for item in inventario:
            codigo_barras, quantidade = item.keys()[0], item.values()[0]
            codigo_barras = codigo_barras.zfill(14)
            if dict_codigo_dados.get(codigo_barras):
                produto_id = dict_codigo_dados.get(codigo_barras).get('produto_id')
                produto_ids.append(produto_id)

        dict_trib = {}
        obj_contagem.get_dict_trib(cr, uid, produto_ids, dict_trib, context=context)
        for item in inventario:
            codigo_barras, quantidade = item.keys()[0], item.values()[0]
            codigo_barras = codigo_barras.zfill(14)
            if dict_codigo_dados.get(codigo_barras):
                produto_id = dict_codigo_dados.get(codigo_barras).get('produto_id')
                grade_id = dict_codigo_dados.get(codigo_barras).get('grade_id')
                produto_emb_id = dict_codigo_dados.get(codigo_barras).get('produto_embalagem_id')
                chave = '%s-%s' % (produto_id, grade_id)
                locacao_id = dict_local_locacao.get(chave)
                if not locacao_id:
                    try:
                        produto_local_diferente.append(
                            _('O produto %(codigo_barras)s nao esta cadastrado no local selecionado')
                            % locals())
                    except:
                        pass

                    continue

                if quantidade > 999999.99:
                    quantidade = 999999.99

                chave = '%s-%s' % (grade_id, produto_emb_id)
                quantidade *= grade_qtd_embalagem.get(chave).get('qtd') or 1
                # Gera/atualiza a contagem
                chave = '%s-%s-%s' % (produto_id, grade_id, local_id)
                contagem_id = dict_contagem.get(chave)
                vals = {'saldo_contado': quantidade}
                obj_contagem.atualiza_vals_com_trib(vals, produto_id, dict_trib, context=context)
                obj_contagem.salvar_contagem(cr, uid, inventario_id, produto_id, grade_id, local_id,
                    vals=vals, tipo_contagem=tipo_contagem,
                    contagem_id=contagem_id,
                    grade_ids_sem_contagem=grade_ids_sem_contagem, context=context_aux)
            else:
                try:
                    log_msg.append(_('Codigo de barras %(codigo_barras)s nao encontrado.') %
                                   locals())
                except:
                    pass

        if grade_ids_sem_contagem:
            cb_ids = obj_codigo_barra.search(cr, uid, [('grade_id', 'in', grade_ids_sem_contagem)],
                                             context=context)
            for i in obj_codigo_barra.read(cr, uid, cb_ids, ['codigo_barra'] ,context=context):
                log_msg.append(_('O produto %s nao esta no inventario') % i.get('codigo_barra'))

        log_msg.extend(produto_local_diferente)
        # Grava o log e muda o status para atualizar a tela do wizard
        self.write(cr, uid, ids, {'log': '\n'.join(log_msg), 'state': 'S'}, context=context)


        return {
            'name': "Importar do coletor",
            'view_type': 'form',
            'view_mode': 'form',
            'res_id': ids[0],
            'res_model': 'wizard.inventario.importar.coletor',
            'type': 'ir.actions.act_window',
            'context': context,
            'ref': 'action_wizard_inventario_importar_coletor',
            'target': 'new',
        }


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
