# -*- coding: utf-8 -*-
from openerp.osv import osv, fields
from openerp.report.render import render
from openerp.report.interface import report_int
import StringIO
from SigUtils.impressao import Impressora
from openerp.tools.translate import _
import time
from openerp import pooler
from datetime import datetime
from SigUtils.formatastring import formataString
from SigUtils.numero import getNumeroEditado

CONST_REPORT_WIDTH = 182


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):

        def centralize(left, center, right, length=CONST_REPORT_WIDTH):
            left_padding = (length - len(center)) / 2
            right_padding = length - left_padding - len(center)
            title = left
            title += (' ' * (left_padding - len(left)))
            title += center
            title += (' ' * (right_padding - len(right)))
            title += right
            return title

        def get_model(modulo):
            return pooler.get_pool(cr.dbname).get(modulo)

        def is_modulo_instalado(modulo):
            return get_model('adm.utils').admutl_check_modulo_instalado(cr, uid, modulo, context=context)

        context = context and context.copy() or {}
        brw_wizard = get_model('wizard.listagem.geral').browse(cr, uid, ids[0], context=context)
        item_ids = get_model('wizard.listagem.geral.item').search(cr, uid, [('listagem_id', '=', ids[0])],
                                                                  context=context)
        res_items = get_model('wizard.listagem.geral.item').read(cr, uid, item_ids, context=context)

        pdf_string = StringIO.StringIO()
        impressora = Impressora()

        impressora.informaTamLinha(CONST_REPORT_WIDTH)
        impressora.defineMargem(0.5, 0.5, 0.5, 0.5)
        impressora.defineFonte(tam=7.4)
        impressora.informaNome(pdf_string, _('Listagem geral de cadastro de itens'))

        nome_empresa = get_model('res.company').browse(cr, uid, brw_wizard.company_id.id).name or ''
        titulo = _('Data: ') + time.strftime('%d/%m/%Y')
        titulo += _('Listagem geral de itens').center(CONST_REPORT_WIDTH - 32, ' ')
        titulo += 8 * ' '
        titulo += _('Pag. {{ Page , 3}}')
        impressora.informaCab(titulo)
        # 'Data: dd/mm/YYYYPag. 999'
        impressora.informaCab(_('Empresa: ') + nome_empresa)
        impressora.informaCab(_('Parametros:'))

        # impressao dos parâmetros
        if is_modulo_instalado('automacao'):
            label_codigo = {'P': _('Codigo do produto'), 'B': _('Codigo de barras'), 'F': _('Codigo no Fabricante')}
            impressora.informaCab('    ' + _('Codigo a listar: ') + label_codigo[brw_wizard.codigo_a_listar])

        labels_custo = {'C': _('C. corrigido'), 'M': _('Medio ctb.'), 'R': _('C. reposicao'),
                        'U': _('U. custo')}
        impressora.informaCab('    ' + _('Tipo de custo: ') + labels_custo[brw_wizard.tipo_custo])
        if brw_wizard.local_id:
            brw_local = get_model('estoque.local').browse(cr, uid, brw_wizard.local_id.id)
            if isinstance(brw_local, list):
                brw_local = brw_local[0]
            impressora.informaCab('    ' + _('Local: ') + brw_local.name)
        if brw_wizard.fornecedor_id:
            brw_fornecedor = get_model('financeiro.fornecedor').browse(cr, uid, brw_wizard.fornecedor_id.id)
            if isinstance(brw_fornecedor, list):
                brw_fornecedor = brw_fornecedor[0]
            cabecalho_fornecedor = '    Fornecedor: ' + brw_fornecedor.pessoa_id.nome
            if brw_wizard.fornecedor_id.pessoa_id.telefone:
                cabecalho_fornecedor += '    Fone: ' + brw_wizard.fornecedor_id.pessoa_id.telefone
            if brw_wizard.fornecedor_id.pessoa_id.relacionamento_ids:
                cabecalho_fornecedor += '    Contato: ' + brw_wizard.fornecedor_id.pessoa_id.\
                    relacionamento_ids[0].dependente_id.nome or ''
                if brw_wizard.fornecedor_id.pessoa_id.relacionamento_ids[0].dependente_id.telefone:
                    cabecalho_fornecedor += '    Fone: ' + brw_wizard.fornecedor_id.pessoa_id.\
                        relacionamento_ids[0].dependente_id.telefone or ''
            impressora.informaCab(cabecalho_fornecedor)

        if brw_wizard.fornecedor_id and brw_wizard.representante_id:
            brw_representante = get_model('financeiro.representante').browse(cr, uid, brw_wizard.representante_id.id)
            if isinstance(brw_representante, list):
                brw_representante = brw_representante[0]
            impressora.informaCab('    ' + _('Representante: ') + brw_representante.pessoa_id.nome)

        if brw_wizard.fabricante_id:
            cabecalho_fabricante = '    ' + _('Fabricante: ') + brw_wizard.fabricante_id.nome
            if brw_wizard.fabricante_id.telefone:
                cabecalho_fabricante += '    Fone: ' + brw_wizard.fabricante_id.telefone
            if brw_wizard.fabricante_id.relacionamento_ids:
                cabecalho_fabricante += '    Contato: ' + brw_wizard.fabricante_id.\
                    relacionamento_ids[0].dependente_id.nome or ''
                if brw_wizard.fabricante_id.relacionamento_ids[0].dependente_id.telefone:
                    cabecalho_fabricante += '    Fone: ' + brw_wizard.fabricante_id.\
                        relacionamento_ids[0].dependente_id.telefone or ''
            impressora.informaCab(cabecalho_fabricante)

        if is_modulo_instalado('compras') and brw_wizard.comprador_id:
            brw_comprador = get_model('compras.comprador').browse(cr, uid, brw_wizard.comprador_id.id)
            impressora.informaCab('    ' + _('Comprador: ') + brw_comprador.pessoa_id.nome)
        labels_ordem = {'P': _('Codigo'), 'A': _('Alfabetica'), 'C': _('Categoria'), 'F': _('Fornecedor')}
        impressora.informaCab('    ' + _('Ordem: ') + labels_ordem[brw_wizard.ordem])
        labels_filtro = {'T': _('Todos'), 'S': _('Sem saldo'), 'C': _('Com saldo'), 'N': _('Negativos'),
                         'A': _('Ativos'), 'I': _('Inativos')}
        impressora.informaCab('    ' + _('Listar itens: ') + labels_filtro[brw_wizard.filtro])

        impressora.informaCab('-' * CONST_REPORT_WIDTH)

        cabecalho = ''
        if is_modulo_instalado('automacao') and brw_wizard.codigo_a_listar == 'B':
            cabecalho += formataString(_('Cod. bar.'), 14, ' ', 'D', 'N', 'S')
        elif brw_wizard.codigo_a_listar == 'P':
            cabecalho += formataString(_('Cod. prod '), 14, ' ', 'D', 'N', 'S')
        else:
            cabecalho += formataString(_('Cod. fabr.'), 14, ' ', 'D', 'N', 'S')
        cabecalho += formataString(_('Gr'), 3, ' ', 'D', 'N', 'S')
        cabecalho += formataString(_('Descricao'), 55, ' ', 'D', 'N', 'S')
        cabecalho += formataString(_('Categoria'), 11, ' ', 'D', 'N', 'S')
        cabecalho += formataString('Emb', 4, ' ', 'E', 'N', 'S') + ' '
        cabecalho += formataString(_('UND'), 4, ' ', 'D', 'N', 'S')
        if is_modulo_instalado('fiscal'):
            cabecalho += formataString(_('Trb'), 8, ' ', 'D', 'N', 'S')
        else:
            cabecalho += ''.ljust(8)

        cabecalho += 'U. Entrada '
        cabecalho += '  U. Saida '
        cabecalho += '     Margem   '
        cabecalho += formataString(_(labels_custo[brw_wizard.tipo_custo]), 11, ' ', 'E', 'N', 'S')
        cabecalho += formataString('P. venda', 9, ' ', 'E', 'N', 'S')
        cabecalho += formataString(_('Minimo'), 8, ' ', 'E', 'N', 'S')
        cabecalho += formataString(_('Maximo'), 8, ' ', 'E', 'N', 'S')
        cabecalho += formataString(_('*SALDO-*'), 10, ' ', 'E', 'N', 'S')
        impressora.informaCab(cabecalho)
        # cabecalho para as colunas de margem
        cabecalho = 'Ideal Bruta Minima'.rjust(136)
        impressora.informaCab(cabecalho)
        impressora.informaCab('-' * CONST_REPORT_WIDTH)

        impressora.informaCab('')

        # impressao dos itens

        group_items = [(0, '')]
        dict_categorias = {}
        if brw_wizard.ordem == 'P':
            res_items = sorted(res_items, key=lambda x: x['codigo_item'])
        elif brw_wizard.ordem == 'A':
            res_items = sorted(res_items, key=lambda x: x['descricao'])
        elif brw_wizard.ordem == 'C':
            categoria_ids = list(set(map(lambda x: x['categoria_id'][0],
                                         filter(lambda c: c.get('categoria_id'), res_items))))
            brw_categoria = get_model('estoque.categoria').browse(cr, uid, categoria_ids)
            lista_categs = []
            for i in brw_categoria:
                lista_categs.append((i.id, i.conta))
                chave = '%s-%s' % (i.id, i.conta)
                dict_categorias[chave] = i.nome_completo

            group_items = lista_categs
            group_items = sorted(group_items, key=lambda x: x[1])
            if filter(lambda c: not c.get('categoria_id'), res_items):
                group_items.append((False, 'Sem categoria'))
        elif brw_wizard.ordem == 'F':
            fabricante_ids = list(set(map(lambda x: x['fabricante_id'][0],
                                          filter(lambda f: f.get('fabricante_id'), res_items))))
            itens_fabricante = list()
            group_items = get_model('adm.pessoa').browse(cr, uid, fabricante_ids)
            for x in group_items:
                descricao_fabricante = x.nome
                if x.telefone:
                    descricao_fabricante += '    Fone: ' + x.telefone
                if x.relacionamento_ids:
                    descricao_fabricante += '    Contato: ' + x.relacionamento_ids[0].\
                        dependente_id.nome or ''
                    if x.relacionamento_ids[0].dependente_id.telefone:
                        descricao_fabricante += '    Fone: ' + x.relacionamento_ids[0].\
                            dependente_id.telefone or ''

                itens_fabricante.append((x.id, descricao_fabricante))
            group_items = sorted(itens_fabricante, key=lambda x: x[1])

        total_saldo = 0.0
        for (i, (group_id, group_name)) in enumerate(group_items):
            items_by_group = res_items
            if brw_wizard.ordem in ('C', 'F'):
                group_labels = {'C': _('Categoria: '), 'F': _('Fabricante: ')}
                group_fields = {'C': 'categoria_id', 'F': 'fabricante_id'}
                if i > 0:
                    impressora.imprimeLinha('-' * CONST_REPORT_WIDTH)
                if brw_wizard.ordem == 'C':
                    chave = '%s-%s' % (group_id, group_name)
                    nome_categ = dict_categorias.get(chave) or ''
                    impressora.imprimeLinha(centralize('', group_labels[brw_wizard.ordem] + group_name + ' - ' +nome_categ, ''))
                else:
                    impressora.imprimeLinha(centralize('', group_labels[brw_wizard.ordem] + group_name, ''))
                impressora.imprimeLinha('-' * CONST_REPORT_WIDTH)
                if group_id:
                    items_by_group = [x for x in res_items if x[group_fields[brw_wizard.ordem]] and
                                      x[group_fields[brw_wizard.ordem]][0] == group_id]
                else:
                    items_by_group = [x for x in res_items if x[group_fields[brw_wizard.ordem]] ==
                                      group_id]
            if not brw_wizard.ordem:
                items_by_group = sorted(items_by_group, key=lambda ci: ci.get('codigo_item'))
            elif brw_wizard.ordem == 'C':
                items_by_group = sorted(items_by_group, key=lambda ci: ci.get('descricao'))
            for item in items_by_group:
                total_saldo += (item.get('saldo') or 0.0)
                linha = []
                linha.append(formataString(item.get('codigo_item', ''), 14, ' ', 'D', 'N', 'S'))
                linha.append(formataString(item['grade'], 3, ' ', 'D', 'N', 'S'))
                linha.append(formataString(item['descricao'], 54, ' ', 'D', 'N', 'S') + ' ')
                linha.append(formataString(item['categoria'], 10, ' ', 'D', 'N', 'S') + ' ')
                linha.append(formataString(item['quantidade_na_embalagem'], 4, ' ', 'E', 'N', 'S') + ' ')
                linha.append(formataString(item['unidade'], 4, ' ', 'D', 'N', 'S'))
                if is_modulo_instalado('fiscal'):
                    linha.append(formataString(item['trb'], 8, ' ', 'D', 'N', 'S'))
                else:
                    linha.append(''.ljust(8))

                dt_entrada= item.get('dt_entrada') and time.strftime('%d/%m/%Y',
                                            time.strptime(item.get('dt_entrada'), '%Y-%m-%d')) or ''
                dt_saida= item.get('dt_saida') and time.strftime('%d/%m/%Y',
                                              time.strptime(item.get('dt_saida'), '%Y-%m-%d')) or ''
                linha.append(formataString(dt_entrada, 11, ' ', 'E', 'N', 'S'))
                linha.append(formataString(dt_saida, 11, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item.get('margem_ideal') or 0.0, 0), 5, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item.get('margem_bruta') or 0.0, 0), 5, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item.get('margem_minima') or 0.0, 0), 5, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item['custo']), 10, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item['preco_venda']), 9, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item.get('minimo') or 0.0), 8, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item.get('maximo') or 0.0), 8, ' ', 'E', 'N', 'S'))
                linha.append(formataString(getNumeroEditado(item['saldo']), 10, ' ', 'E', 'N', 'S'))
                impressora.imprimeLinha(''.join(linha))

        # Saldo total:
        impressora.imprimeLinha('-' * CONST_REPORT_WIDTH)
        str_total = formataString(_('Saldo Total: '), CONST_REPORT_WIDTH - 10, ' ', 'E', 'N' 'S')
        str_total += formataString(getNumeroEditado(total_saldo), 10, ' ', 'E', 'N', 'S')
        impressora.imprimeLinha(str_total)

        impressora.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return (self.obj.pdf, 'pdf')


report_custom('report.listagem_geral')


class wizard_listagem_geral(osv.TransientModel):
    def _get_pode_imprimir(self, cr, uid, ids, field_name, args, context=None):
        context = context and context.copy() or {}
        res = {}
        for id in ids:
            if self.browse(cr, uid, id).listagem_item_ids:
                res[id] = True
            else:
                res[id] = False
        return res

    _name = 'wizard.listagem.geral'
    _columns = {
        'id': fields.integer('Id', readonly=True),
        'company_id': fields.many2one('res.company', 'Empresa', required=True),
        'codigo_a_listar': fields.selection([
            ('P', 'Codigo do produto'),
            ('F', 'Codigo no fabricante')
        ], 'Codigo a listar', required=True),
        'tipo_custo': fields.selection([
            ('C', 'Custo corrigido'),
            ('M', 'Custo medio contabil'),
            ('R', 'Custo de reposicao'),
            ('U', 'Ultimo custo')
        ], 'Tipo de custo', required=True),
        'local_id': fields.many2one('estoque.local', 'Local'),
        'fornecedor_id': fields.many2one('financeiro.fornecedor', 'Fornecedor'),
        'fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'representante_id': fields.many2one('financeiro.representante', 'Representante'),
        'ordem': fields.selection([
            ('P', 'Codigo'),  # Código do produto
            ('A', 'Alfabetica'),
            ('C', 'Categoria'),
            ('F', 'Fabricante')

        ], 'Ordenar por', required=True),
        'filtro': fields.selection([
            ('T', 'Todos'),
            ('S', 'Sem saldo'),
            ('C', 'Com saldo'),
            ('N', 'Negativos'),
            ('A', 'Ativos'),
            ('I', 'Inativos')

        ], 'Listar itens', required=True),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'listagem_item_ids': fields.one2many('wizard.listagem.geral.item', 'listagem_id'),
        'pode_imprimir': fields.function(_get_pode_imprimir, method=True, string='Imprimir', type='boolean',
                                         store=False),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'tipo_custo': lambda *args: 'U',
        'ordem': lambda *args: 'P',
        'filtro': lambda *args: 'T'
    }

    def onchange_fornecedor(self, cr, uid, ids, fornecedor_id, context=None):
        context = context and context.copy() or {}
        val = {'representante_id': ''}
        if fornecedor_id:
            representante_id = self.pool.get('financeiro.representante') \
                .search(cr, uid, [('fornecedor_id', '=', fornecedor_id)], context=context)
            if len(representante_id) == 1:
                val['representante_id'] = representante_id[0]
        return {'value': val}

    def search_and_create_items(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        vals = self.browse(cr, uid, ids[0])
        sql = '''
                DELETE FROM wizard_listagem_geral_item where listagem_id = %s
              ''' % (ids[0])
        cr.execute(sql)
        parametros = {
            'company_id': self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                         context=context),
            'codigo_a_listar': vals.codigo_a_listar,
            'tipo_custo': vals.tipo_custo,
            'local_id': vals.local_id and vals.local_id.id,
            'fornecedor_id': vals.fornecedor_id and vals.fornecedor_id.id,
            'fabricante_id': vals.fabricante_id and vals.fabricante_id.id,
            'representante_id': vals.representante_id and vals.representante_id.id,
            'ordem': vals.ordem,
            'filtro': vals.filtro,
            'categoria': vals.categoria_id and vals.categoria_id.conta,
            'str_categoria_ids': '',
            'locacao_company_id': self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'estoque.local.locacao', context=context)
        }
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'compras', context):
            parametros['comprador_id'] = vals.comprador_id and vals.comprador_id.id

        if vals.categoria_id.id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', vals.categoria_id.id)], context=context)
            if len(categoria_ids) == 1:
                str_categoria_ids = str(tuple(categoria_ids)).replace(',', '')
            else:
                str_categoria_ids = str(tuple(categoria_ids))
            parametros['str_categoria_ids'] = str_categoria_ids

        itens = self.pool.get('wizard.listagem.geral.item').get_items(cr, uid, ids, parametros, context)
        sql = """
            -- noinspection SqlDialectInspectionForFile
            INSERT INTO wizard_listagem_geral_item(create_uid, create_date, write_date, write_uid,
                categoria, categoria_id, codigo_item, custo, descricao, grade, listagem_id, 
                margem_bruta, margem_ideal, margem_minima, maximo, minimo, preco_venda, 
                quantidade_na_embalagem, saldo, trb, unidade, fornecedor_id, dt_entrada, dt_saida, 
                fabricante_id )
            VALUES %s
              """
        sql_itens = []
        count = 0
        local_produto = list()
        for item in itens:
            key_local_produto = '%s/%s/%s' % (item.get('produto_id'), item.get('grade_id'),
                                              item.get('local_id'))
            if key_local_produto in local_produto:
                continue
            local_produto.append(key_local_produto)
            categoria = item.get('categoria') or ''
            categoria_id = item.get('categoria_id')
            codigo_item = item.get('codigo_item').replace("'", '') if item.get('codigo_item') else ''
            custo = item.get('custo') or 0.0
            descricao = obj_adm_utils.admutl_remover_acentuacao(cr, uid, item.get('descricao') or '')
            descricao = descricao.replace("'", '')
            grade = item.get('grade')
            listagem_id = item.get('listagem_id')
            if not listagem_id:
                listagem_id = ids[0] if isinstance(ids, list) else ids
            margem_bruta = item.get('margem_bruta') or 0
            margem_ideal = item.get('margem_ideal') or 0
            margem_minima = item.get('margem_minima') or 0
            maximo = item.get('maximo') or 0
            minimo = item.get('minimo') or 0
            preco_venda = item.get('preco_venda') or 0
            quantidade_na_embalagem = item.get('quantidade_na_embalagem')
            saldo = item.get('saldo') or 0
            trb = item.get('trb') or ''
            dt_entrada = item.get('dt_entrada') and ("'%s'" % item.get('dt_entrada')) or 'null'
            dt_saida = item.get('dt_saida') and ("'%s'" % item.get('dt_saida')) or 'null'
            unidade = item.get('unidade')
            create_uid = write_uid = uid
            create_date = write_date = str(datetime.now())
            fabricante_id = item.get('fabricante_id') or 'NULL'
            sql_item = '''
                ({create_uid}, '{create_date}','{write_date}', {write_uid},
                '{categoria}', {categoria_id}, '{codigo_item}', {custo}, '{descricao}', '{grade}',
                 {listagem_id}, {margem_bruta}, {margem_ideal}, {margem_minima}, {maximo},
                 {minimo}, {preco_venda}, {quantidade_na_embalagem}, {saldo}, '{trb}', '{unidade}',
                 {fornecedor_id}, {dt_entrada}, {dt_saida}, {fabricante_id})
                  '''.format(create_uid=create_uid,
                             create_date=create_date,
                             write_date=write_date,
                             write_uid=write_uid,
                             categoria=categoria,
                             categoria_id=categoria_id or 'NULL',
                             codigo_item=codigo_item,
                             custo=custo,
                             descricao=descricao,
                             grade=grade,
                             listagem_id=listagem_id or 'NULL',
                             margem_bruta=margem_bruta,
                             margem_ideal=margem_ideal,
                             margem_minima=margem_minima,
                             maximo=maximo,
                             minimo=minimo,
                             preco_venda=preco_venda,
                             quantidade_na_embalagem=quantidade_na_embalagem or 1,
                             saldo=saldo,
                             trb=trb,
                             unidade=unidade or '',
                             fornecedor_id=item.get('fornecedor_id') or 'NULL',
                             dt_entrada=dt_entrada,
                             dt_saida=dt_saida,
                             fabricante_id=fabricante_id)
            sql_itens.append(sql_item)
            count += 1
            # quebra o sql para nao inserir todos itens de uma vez, podendo estourar a memoria
            # do postgres
            if count >= 2000:
                cr.execute(sql % ','.join(sql_itens))
                sql_itens = []
                count = 0

        if sql_itens:
            cr.execute(sql % ','.join(sql_itens))

        self.write(cr, uid, ids[0], {}, context)
        return True

    def button_find(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.search_and_create_items(cr, uid, ids, context=context)
        return True

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_geral',
            'datas': {'ids': ids}
        }


class wizard_listagem_geral_item(osv.TransientModel):
    _name = 'wizard.listagem.geral.item'

    _columns = {
        'listagem_id': fields.many2one('wizard.listagem.geral', select=True),
        'codigo_item': fields.char('Codigo', 20),
        'grade': fields.char('Grade', 2),
        'descricao': fields.char('Descricao', 63),
        'categoria': fields.char('Categoria', 50),
        'quantidade_na_embalagem': fields.float('Embalagem', digits=(8, 3)),
        'unidade': fields.char('Unidade', 3),
        'margem_ideal': fields.float('Margem ideal', digits=(8, 3)),
        'margem_bruta': fields.float('Margem bruta', digits=(8, 3)),
        'margem_minima': fields.float('Margem minima', digits=(8, 3)),
        'custo': fields.float('Custo', digits=(15, 2)),
        'preco_venda': fields.float('Preco venda', digits=(15, 2)),
        'minimo': fields.integer('Minimo'),
        'maximo': fields.integer('Maximo'),
        'saldo': fields.float('Saldo', digits=(14, 5)),
        'categoria_id': fields.many2one('estoque.categoria', select=True),
        'fornecedor_id': fields.many2one('financeiro.fornecedor', select=True),
        'trb': fields.char('Trb', 8),
        'dt_entrada': fields.date('Ultima entrada'),
        'dt_saida': fields.date('Ultima saida'),
        'fabricante_id': fields.many2one("adm.pessoa", 'Fabricante', select=True),
    }

    def build_query(self, params):
        # Define o campo ´codigo´ a ser mostrado: P=codigo do produto, F=codigo do fornecedor
        if params.get('codigo_a_listar') == 'P':
            campo_codigo = 'ep.codigo                     AS codigo_item,'
        elif params.get('codigo_a_listar') == 'F':
            campo_codigo = 'epg.codigo_no_fabricante      AS codigo_item,'
        else:
            campo_codigo = ''
        local_id = params.get('local_id') or params.get('local_padrao_id')

        # Condicoes da consulta -- WHERE
        cond_comprador = ''
        if params.get('comprador_id', False):
            cond_comprador = params.get('comprador_id')
            cond_comprador = cond_comprador and ('AND ep.comprador_id = %s' % cond_comprador) or ''
        cond_local = local_id and ('AND ell.local_id = %s' % local_id) or ''
        cond_fornecedor = params.get('fornecedor_id') or ''
        left_fornecedor = ''
        if cond_fornecedor:
            cond_fornecedor = cond_fornecedor and ('AND ef.fornecedor_id = %s' % cond_fornecedor) or ''
            left_fornecedor = 'LEFT JOIN estoque_fornecedor         AS ef  ON ef.produto_id = ep.id'

        cond_representante = params.get('representante_id')
        cond_representante = cond_representante and ('AND ef.representante_id = %s' %
                                                     cond_representante) or ''
        cond_fabricante = params.get('fabricante_id')
        cond_fabricante = cond_fabricante and ('AND ep.fabricante_id = %s' % cond_fabricante) or ''
        cond_categoria = ''
        if params.get('str_categoria_ids'):
            cond_categoria = """ AND ec.id IN %s """ % params.get('str_categoria_ids')

        company_id = params.get('company_id')  # Uso na momtagem do SQL
        if params.get('filtro') == 'A':
            cond_filtro = 'AND epg.active'
        elif params.get('filtro') == 'I':
            cond_filtro = 'AND epg.active IS FALSE'
        else:
            cond_filtro = ''

        left_pessoa = ''
        agrupamento_fabricante = ''
        select_fabricante = ''
        # Ordem da consulta -- ORDER BY
        if params.get('ordem') == 'P':
            order_by = 'ep.codigo'
        elif params.get('ordem') == 'A':
            order_by = 'epg.name'
        elif params.get('ordem') == 'C':
            order_by = 'ec.conta, descricao'
        else:  # Nome do fabricante
            agrupamento_fabricante = ', ap.id'
            order_by = 'ap.nome'
            left_pessoa = 'LEFT JOIN adm_pessoa              AS ap  ON ap.id = ep.fabricante_id'
            select_fabricante = ', ap.id AS fabricante_id'

        descricao_local = str()
        if local_id:
            local_id = """
               el.id                         AS local_id,
            """
            descricao_local = """
               lpad(el.numero::text, 3, '0') AS local,
            """

        # Empresa para locacao (locacao e uma classe correlacionavel)
        locacao_company_id = params.get('locacao_company_id')

        sql = """
        -- noinspection SqlDialectInspectionForFile
        SELECT ep.id                         AS produto_id,
               epg.id                        AS grade_id,
               ec.id                         AS categoria_id,
               %(local_id)s
               %(descricao_local)s
               ec.conta                      AS categoria,
               epg.grade,
               CASE WHEN epg.name IS NOT NULL
                    THEN ltrim(epg.name, ' ')
                    ELSE ' '
               END                           AS descricao,
               ee.quantidade_na_embalagem,
               epe.estpre_unidade_produto AS unidade,
               ep.margem_ideal,
               ep.margem_bruta,
               ep.margem_minima,
               CASE WHEN epg.minimo IS NULL THEN 0 ELSE epg.minimo END AS minimo,
               cst.tipo as trb,
               ft.aliquota,
               {codigo_heranca}
               %(campo_codigo)s
               epg.maximo
               %(select_fabricante)s
        FROM estoque_produto_grade           AS epg
        LEFT JOIN estoque_produto            AS ep  ON ep.id = epg.produto_id
        LEFT JOIN estoque_categoria          AS ec  ON ec.id = ep.categoria_id
        LEFT JOIN estoque_local_locacao      AS ell ON ell.grade_id = epg.id
        LEFT JOIN estoque_local              AS el  ON el.id = ell.local_id
        LEFT JOIN estoque_produto_embalagem  AS epe ON epe.id = ell.estllc_produto_embalagem_id
        LEFT JOIN estoque_embalagem          AS ee  ON ee.id = epe.embalagem_id
        %(left_fornecedor)s
        %(left_pessoa)s
        LEFT JOIN fiscal_tributacao_encargos AS fte ON fte.produto_id = ep.id
        LEFT JOIN fiscal_encargo             AS fe  ON fe.id = fte.encargo_id
        LEFT JOIN fiscal_tributacao          AS ft  ON ft.id = fte.tributacao_id
        LEFT JOIN fiscal_cst                 AS cst ON cst.id = fte.cst_id
        WHERE fe.sigla = 'ICMS' 
          %(cond_fabricante)s
          %(cond_fornecedor)s
          %(cond_representante)s
          %(cond_categoria)s
          %(cond_local)s
          %(cond_filtro)s
          %(cond_comprador)s
          AND ep.company_id = %(company_id)s
          AND ell.company_id = %(locacao_company_id)s

        group by ep.id, epg.id, ec.id, el.id, lpad(el.numero::text, 3, '0'), ec.conta, epg.grade,
               epg.name, ee.quantidade_na_embalagem, epe.estpre_unidade_produto, ep.margem_ideal, 
               ep.margem_bruta, ep.margem_minima, epg.minimo, cst.tipo, ft.aliquota, 
               {codigo_heranca_group} ep.codigo, epg.maximo %(agrupamento_fabricante)s
        ORDER BY %(order_by)s
        """ % locals()
        if params.get('codigo_a_listar') != 'B':
            sql = sql.format(codigo_heranca='', codigo_heranca_group='')
        return sql

    def get_datas_movimento(self, cr, grade_ids, context=None):
        context = context and context.copy() or {}
        sql = """
        SELECT max(data) AS data, 
               grade_id,
               tipo_movimento
        FROM (SELECT max(emc.data) AS data, 
                     emi.grade_id  AS grade_id,
                     CASE WHEN fnc.tipo is not null  and fnc.tipo = 'E'
                          THEN fnc.tipo
                          ELSE 'S'
                     END           AS tipo_movimento
            FROM estoque_movimento_capa        AS emc
            INNER JOIN estoque_movimento_itens AS emi ON emi.capa_id = emc.id
            LEFT  JOIN fiscal_nf_capa          AS fnc ON fnc.id = emi.nf_id
            WHERE emi.grade_id IN (%s)
              AND fnc.state = '3'
            GROUP BY emi.grade_id, tipo ) AS tmp
        GROUP BY grade_id, tipo_movimento """ % ','.join(map(lambda x: "%s" % x, list(set(grade_ids))))
        cr.execute(sql)
        res = {}
        for i in cr.dictfetchall():
            grade_id = i.get('grade_id')
            tipo_movimento = i.get('tipo_movimento')
            vals = {'dt_entrada': i.get('data') or ''}
            if tipo_movimento == 'S':
                vals = {'dt_saida': i.get('data') or ''}

            if not res.get(grade_id):
                res[grade_id] = vals
            else:
                res[grade_id].update(vals)

        return res

    def get_items(self, cr, uid, ids, params, context=None):
        context = context and context.copy() or {}
        cr.execute('SELECT local_venda_padrao_id FROM estoque_parametro WHERE company_id=%s' % (params['company_id']))
        local_padrao_id = cr.dictfetchall()
        if local_padrao_id:
            params['local_padrao_id'] = local_padrao_id[0].get('local_venda_padrao_id')

        sql = self.build_query(params)
        cr.execute(sql)
        items = cr.dictfetchall()
        if not items:
            return items

        grade_ids = list(set(map(lambda x: x.get('grade_id'), items)))
        saldo_local = dict()  # Saldo por local
        estoque_saldo_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.produto.saldo', context=context)
        for local in filter(lambda c: c, list(set(map(lambda i: i.get('local_id'), items)))):
            saldo_local[local] = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                cr, uid, grade_ids, datetime.now().date(), company_id=estoque_saldo_company_id,
                eps_local_id=local, context=context)
        saldos = dict()
        if not saldo_local:
            saldos = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                cr, uid, grade_ids, datetime.now().date(), company_id=params['company_id'],
                context=context)
        itens = []
        custos = dict(C='estprc_custo_corrigido', M='estprc_custo_medio_contabil',
                      R='estprc_custo_reposicao', U='estprc_ultimo_custo')
        tipo_custo = custos[params.get('tipo_custo')] or 'estprc_ultimo_custo'
        produto_ids = []
        grade_ids = []
        local_produto = []
        for item in items:  # Recupera o saldo e o custo para cada item
            produto_id = item.get('produto_id')
            grade_id = item.get('grade_id')
            local_id = item.get('local_id')
            key_local_produto = '%s/%s/%s' % (produto_id, grade_id, local_id)
            if key_local_produto in local_produto:
                continue

            local_produto.append(key_local_produto)
            if saldo_local:
                item['saldo'] = saldo_local[local_id].get(grade_id).get(
                    'disponivel')
            else:
                item['saldo'] = saldos.get(grade_id).get('disponivel')

            if params.get('filtro') == 'S' and item['saldo'] != 0:
                # Remove o item com saldo diferente de zero
                continue
            elif params.get('filtro') == 'C' and item['saldo'] <= 0:
                # Remove o item com saldo menor ou igual a zero
                continue
            elif params.get('filtro') == 'N' and item['saldo'] >= 0:
                # Remove o item com saldo maior ou itual a zero
                continue

            itens.append(item)
            produto_ids.append(produto_id)
            grade_ids.append(grade_id)

        del saldo_local  # otimizando memoria RAM
        del saldos  # otimizando memoria RAM
        produto_ids = list(set(map(lambda i: i['produto_id'], items)))
        res_custos = self.pool.get('estoque.preco.custo').get_custos_do_dia(
            cr, uid, epc_produto_ids=produto_ids, context=context)

        res_precos = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, produto_ids, epc_company_id=params['company_id'], context=context)
        vals_datas = self.get_datas_movimento(cr, grade_ids, context=context)
        for item in itens:
            produto_id = item.get('produto_id')
            grade_id = item.get('grade_id')
            if vals_datas.get(grade_id):
                item.update(vals_datas.get(grade_id))

            item['custo'] = res_custos.get(produto_id, {tipo_custo: 0})[tipo_custo]
            item['preco_venda'] = res_precos.get(produto_id, {'preco_venda': 0})['preco_venda']
            item['listagem_id'] = ids[0]
            if item.get('trb') == 'T':
                item['trb'] = ('%s%s' %(item.get('trb'), ' {:.2%}'.format(item['aliquota'] / 100.))
                               ).replace('100.00', '100.0')

        return itens
