# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from dateutil.parser import parse


class wizard_listagem_item_parado(osv.TransientModel):
    # prefixo: lisipa_

    _name = 'wizard.listagem.item.parado'

    _columns = {
        'lisipa_data': fields.date('Itens parados desde', required=True),
        'lisipa_considera_entrada': fields.boolean('Considerar entradas'),
        'lisipa_local_id': fields.many2one('estoque.local', 'Local'),
        'lisipa_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'lisipa_fornecedor_id': fields.many2one('financeiro.fornecedor', 'Fornecedor'),
        'lisipa_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'lisipa_custo': fields.selection([('U', 'Ultimo Custo'), ('M', 'Medio Contabil'),
                                          ('R', 'Reposicao'), ('C', 'Corrigido')], 'Custo a listar',
                                         size=1, required=True),
        'lisipa_somente_com_saldo': fields.boolean('Somente com saldo'),
    }

    _defaults = {
        'lisipa_somente_com_saldo': False,
    }

    def lisipa_button_print(self, cr, uid, item_parado_id, context=None):
        """
        O metodo reune os dados necessarios para a criacao do relatorio em PDF. Esses dados sao
        enviados para a classe relatorio, onde serao processados.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com acao para a chamada do relatorio.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')

        if isinstance(item_parado_id, (list, tuple)):
            item_parado_id = item_parado_id[0]

        brw_listagem_item_parado = self.browse(cr, uid, item_parado_id, context)

        data = brw_listagem_item_parado.lisipa_data
        considera_entrada = brw_listagem_item_parado.lisipa_considera_entrada
        local_id = brw_listagem_item_parado.lisipa_local_id.id
        fabricante_id = brw_listagem_item_parado.lisipa_fabricante_id.id
        fornecedor_id = brw_listagem_item_parado.lisipa_fornecedor_id.id
        categoria_id = brw_listagem_item_parado.lisipa_categoria_id.id
        custo = brw_listagem_item_parado.lisipa_custo
        codigo_custo = custo

        if custo == 'U':
            desc_custo = 'Ultimo custo'
            custo = 'estprc_ultimo_custo'
        elif custo == 'M':
            desc_custo = 'Medio contabil'
            custo = 'estprc_custo_medio_contabil'
        elif custo == 'R':
            desc_custo = 'Reposicao'
            custo = 'estprc_custo_reposicao'
        else:
            desc_custo = 'Corrigido'
            custo = 'estprc_custo_corrigido'

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto')
        args = [('company_id', '=', company_id)]

        categoria = ''
        if categoria_id:
            categoria = self.pool.get('estoque.categoria').browse(cr, uid, categoria_id,
                                                                  context).nome_completo
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)
            args.append(('categoria_id', 'in', categoria_ids))

        fornecedor = ''
        if fornecedor_id:
            fornecedor_ids = self.pool.get('estoque.fornecedor').search(
                cr, uid, [('fornecedor_id', '=', fornecedor_id)], context=context)
            args.append(('fornecedor_ids', 'in', fornecedor_ids))
            fornecedor = self.pool.get('financeiro.fornecedor').browse(
                cr, uid, fornecedor_id, context).pessoa_id.name

        local = ''
        local_default = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque',
                                  ['local_venda_padrao_id'], company_id=company_id,
                                  context=context).get('local_venda_padrao_id')
        if local_id or local_default:
            args.append(('local_ids', '=', local_id or local_default))
            local = self.pool.get('estoque.local').read(cr, uid, local_id or local_default,
                                                        ['name'], context).get('name') or ''

        fabricante = ''
        if fabricante_id:
            args.append(('fabricante_id', '=', fabricante_id))
            fabricante = self.pool.get('adm.pessoa').browse(cr, uid, fabricante_id, context).name

        operacao = "('S')"
        if considera_entrada:
            operacao = "('E', 'S')"

        produto_ids = self.pool.get('estoque.produto').search(cr, uid, args, context=context)
        list_produto_ids = produto_ids
        if not produto_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum produto encontrado para os filtros '
                                                 'selecionados.'))
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))

        sql = """
            SELECT distinct emi.produto_id AS produto_id
            FROM estoque_movimento_itens         AS emi
            INNER JOIN estoque_movimento_capa    AS emc ON emc.id = emi.capa_id
            INNER JOIN estoque_natureza_operacao AS eno ON eno.id = emc.estmvc_operacao_id
            WHERE emi.produto_id IN {produto_ids}
              AND emc.data >= '{data}'
              AND emc.state = 'F'
              AND eno.estnto_tipo_operacao IN {operacao}
        """.format(data=data, produto_ids=produto_ids, operacao=operacao)
        cr.execute(sql)
        movimentado_produto_ids = map(lambda i: i.get('produto_id'), cr.dictfetchall())
        produto_ids = filter(lambda i: i not in movimentado_produto_ids, list_produto_ids)
        if not produto_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum produto encontrado para os filtros '
                                                 'selecionados.'))
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))

        sql = """
            SELECT ep.id                      AS produto_id,
                   epg.id                     AS grade_id,
                   ep.codigo                  AS codigo,
                   epg.grade                  AS grade,
                   ep.name                    AS descricao,
                   eeu.unidade                AS unidade,
                   ee.quantidade_na_embalagem AS quantidade_embalagem,
                   (SELECT max(data)
                    FROM estoque_movimento_capa AS c
                    INNER JOIN estoque_movimento_itens AS i ON c.id = i.capa_id
                    INNER JOIN estoque_natureza_operacao AS f ON f.id = c.estmvc_operacao_id
                    WHERE i.produto_id = ep.id
                      AND i.grade_id = epg.id
                      AND c.data <= '{data}'
                      AND c.state = 'F'
                      AND f.estnto_tipo_operacao = 'E'
                   )                          AS ultima_entrada,
                   (SELECT max(data)
                    FROM estoque_movimento_capa AS c
                    INNER JOIN estoque_movimento_itens AS i ON c.id = i.capa_id
                    INNER JOIN estoque_natureza_operacao AS f ON f.id = c.estmvc_operacao_id
                    WHERE i.produto_id = ep.id
                      AND i.grade_id = epg.id
                      AND c.data <= '{data}'
                      AND c.state = 'F'
                      AND f.estnto_tipo_operacao = 'S'
                   )                          AS ultima_saida
            FROM estoque_produto                 AS ep
            INNER JOIN estoque_produto_grade     AS epg ON epg.produto_id = ep.id
            INNER JOIN estoque_local_locacao     AS ell ON ell.grade_id = epg.id
            INNER JOIN estoque_produto_embalagem AS epe ON epe.id = ell.estllc_produto_embalagem_id  
            INNER JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id 
            INNER JOIN estoque_embalagem_unidade AS eeu ON eeu.id = ee.unidade_id  
            WHERE ep.id IN {produto_ids}
              AND ell.local_id = {local_id}
            ORDER BY ep.codigo
        """.format(data=data, produto_ids=produto_ids, operacao=operacao, local_id=(local_id or
                                                                                    local_default))
        cr.execute(sql)
        itens = cr.dictfetchall()
        dados = {'data': parse(data).strftime('%d/%m/%Y'),
                 'considera_entrada': 'Sim' if considera_entrada else 'Nao',
                 'desc_local': local,
                 'local_id': local_id,
                 'fabricante': fabricante,
                 'fornecedor': fornecedor,
                 'categoria': categoria,
                 'custo': custo,
                 'desc_custo': desc_custo,
                 'codigo_custo': codigo_custo,
                 'itens': itens,
                 'somente_com_saldo': brw_listagem_item_parado.lisipa_somente_com_saldo}
        return {'type': 'ir.actions.report.xml',
                'report_name': 'item_parado',
                'datas': dados,
                'name': 'Itens parados'}
