#!/usr/bin/python
# -*- encoding: utf-8 -*-
from openerp.osv import osv, fields

CODIGO_LISTAR = [('I', 'Item'), ('F', 'Fornecedor'), ('B', 'Barras')]
ALTERACAO_PRECO = [('N', 'Nao mudar'), ('A', 'Aumentar'), ('D', 'Diminuir')]
SALDO_A_LISTAR = [('S', 'Sem Saldo'), ('C', 'Com saldo'), ('N', 'Negativo')]
ESTADO_DO_ITEM = [('A', 'Ativo'), ('I', 'Inativo')]
ORDENAR = [('C', 'Codigo do produto'), ('D', 'Descricao do produto'), ('F', 'Fornecedor'),
           ('L', 'Locacao')]


class wizard_listagem_livro_preco(osv.TransientModel):
    _name = 'wizard.listagem.livro.preco'
    _columns = {
        'wizlip_codigo_listar': fields.selection(CODIGO_LISTAR, string='Codigo a listar',
                                                 help='Codigo a ser impresso na listagem',
                                                 required=True),
        'wizlip_alterar_preco': fields.selection(ALTERACAO_PRECO, string='Alterar preco',
                                                 help='Almenta ou diminui o preco do item '
                                                      '(altera apenas o preco na listagem)'),
        'wizlip_percentual': fields.float(string='Percentual', digits=(4, 2),
                                          help='Percentual a aumentar ou diminuir do preco do item'),
        'wizlip_listar_grades': fields.boolean(string='Listar todas as grades',
                                               help='Se marcado, mostra todas as grades do item'),
        'wizlip_imprimir_saldo': fields.boolean(string='Imprimir saldo',
                                                help='Se marcado, mostra o saldo do item na '
                                                     'listagem'),
        'wizlip_local_ids': fields.many2many('estoque.local',
                                             'wizard_listagem_livro_preco_m2m_estoque_local',
                                             'wizlip_listagem_livro_preco_id', 'estloc_local_id',
                                             string='Locais'),
        'wizlip_fornecedor_ids': fields.many2many('financeiro.fornecedor',
                                                  'wizard_listagem_livro_preco_m2m_financeiro_fornecedor',
                                                  'wizlip_listagem_livro_preco_fornecedor_id',
                                                  'finfor_fornecedor_id', string='Fornecedores'),
        'wizlip_saldo': fields.selection(SALDO_A_LISTAR, string='Saldo a listar'),
        'wizlip_estado_item': fields.selection(ESTADO_DO_ITEM, string='Estado do item'),
        'wizlip_categoria_ids': fields.many2many('estoque.categoria',
                                                 'wizard_listagem_livro_preco_m2m_estoque_categoria',
                                                 'wizlip_listagem_livro_preco_categoria_id',
                                                 'estcat_categoria_id', string='Categorias'),
        'wizlip_agrupar_por_categoria': fields.boolean(string='Agrupar por categoria'),
        'wizlip_ordenar': fields.selection(ORDENAR, string='Ordenar por',
                                           help='Forma de ordenacao do relatorio')
    }

    _defaults = {
        'wizlip_listar_grades': True,
        'wizlip_codigo_listar': 'I',
        'wizlip_agrupar_por_categoria': False,
        'wizlip_estado_item': 'A',
        'wizlip_imprimir_saldo': True
    }

    def button_print(self, cr, uid, ids, context=None):
        """
        Este botao ira montar os dados necessarios para a geracao do
        relatorio de Livro de precos.
        O relatorio de livro de precos consiste em demonstrar os precos
        de venda dos itens cadastrados para a empresa. Este relatorio e
        impresso e utilizado caso ocorra alguma falta de energia na
        empresa por exemplo.
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das notas fiscais
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        wizlip_local_ids = False
        wizlip_categoria_ids = False
        wizlip_listar_grades = False
        wizlip_agrupar_por_categoria = False
        wizlip_codigo_listar = False
        wizlip_estado_item = False
        wizlip_fornecedor_ids = False
        wizlip_ordenar = False
        obj_categoria = self.pool.get('estoque.categoria')
        dados = self.read(cr, uid, ids, context=context)
        listar_fornecedor = False
        for dado in dados:
            wizlip_local_ids = dado.get('wizlip_local_ids')
            wizlip_listar_grades = dado.get('wizlip_listar_grades')
            wizlip_categoria_ids = dado.get('wizlip_categoria_ids')
            wizlip_categoria_ids.extend(
                self.get_categoria_filha_ids(cr, uid, obj_categoria, wizlip_categoria_ids, [],
                                             context))
            wizlip_agrupar_por_categoria = dado.get('wizlip_agrupar_por_categoria')
            wizlip_codigo_listar = dado.get('wizlip_codigo_listar')
            wizlip_estado_item = dado.get('wizlip_estado_item')
            wizlip_fornecedor_ids = dado.get('wizlip_fornecedor_ids')
            wizlip_ordenar = dado.get('wizlip_ordenar')
            if wizlip_ordenar == 'F':
                listar_fornecedor = True
        function = """
                    CREATE OR replace FUNCTION text_if_null(VARCHAR,
                                                            VARCHAR) returns text
                    AS
                      $$
                      DECLARE
                        valor alias FOR $1;
                        texto alias FOR $2;
                        result VARCHAR;
                      BEGIN
                        IF valor IS NULL THEN
                          result:=texto;
                        ELSE
                          result:=valor;
                        END IF;
                        RETURN result;
                      END;
                      $$ LANGUAGE plpgsql
        """
        cr.execute(function)
        function = """
                    CREATE OR replace FUNCTION categ_completo(INTEGER) returns VARCHAR
                    AS
                      $$
                      DECLARE
                        c_id               INTEGER;
                        p_id               INTEGER;
                        categoria          VARCHAR;
                        categoria_completa VARCHAR;
                        controle           INTEGER;
                      BEGIN
                        c_id:=$1;
                        SELECT c.parent_id
                        INTO   p_id
                        FROM   PUBLIC.estoque_categoria c
                        WHERE  c.id = c_id;
                        controle:=1;
                        categoria_completa:='';
                        WHILE controle = 1
                        LOOP
                          IF p_id IS NULL THEN
                            controle:=0;
                          END IF;
                          SELECT c.name
                          INTO   categoria
                          FROM   PUBLIC.estoque_categoria c
                          WHERE  c.id = c_id;

                          SELECT c.parent_id
                          INTO   c_id
                          FROM   PUBLIC.estoque_categoria c
                          WHERE  c.id = c_id;

                          SELECT c.parent_id
                          INTO   p_id
                          FROM   PUBLIC.estoque_categoria c
                          WHERE  c.id = c_id;

                          IF categoria_completa = '' THEN
                            categoria_completa:=categoria;
                          ELSE
                            categoria_completa:=categoria
                            ||'/'
                            || categoria_completa;
                          END IF;
                        END LOOP;
                        RETURN categoria_completa;
                      END;
                      $$language plpgsql;
        """
        cr.execute(function)
        sql = """
                SELECT ep.id            AS produto_id,
                       {codigo_listar}
                       ep.name          AS produto_nome,
                       {grade}
                       ell.locacao      AS locacao_locacao,
                       ec.id            AS categoria_id,
                       text_if_null(ec.NAME, 'SEM CATEGORIA')
                                        AS categoria_nome,
                       CASE WHEN ec.id IS NOT NULL 
                            THEN (SELECT categ_completo(ec.id))
                       END AS categoria_completa,
                       el.id            AS local_id,
                       {unidade}
                       {fornecedor}
                       0                AS preco_venda
                FROM   estoque_produto_grade        AS epg
                LEFT JOIN estoque_produto           AS ep  ON ep.id = epg.produto_id
                LEFT JOIN estoque_local_locacao     AS ell ON ell.grade_id = epg.id
                LEFT JOIN estoque_local             AS el  ON el.id = ell.local_id
                LEFT JOIN estoque_categoria         AS ec  ON ec.id = ep.categoria_id
                LEFT JOIN estoque_produto_embalagem AS epe ON epe.id = ell.estllc_produto_embalagem_id
                LEFT JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id
                LEFT JOIN estoque_embalagem_unidade AS eeu ON eeu.id = ee.unidade_id
                LEFT JOIN automacao_codigo_barra    AS acb ON acb.grade_id = epg.id
                LEFT JOIN estoque_fornecedor        AS ef  ON ef.produto_id = ep.id
                LEFT JOIN financeiro_fornecedor     AS ff  ON ff.id = ef.fornecedor_id
                LEFT JOIN adm_pessoa                AS ap  ON ap.id = ff.pessoa_id
                WHERE {query_active}
                {query_produto}
                {query_locais}
                {query_categorias}
                {query_fornecedores}
                GROUP BY  {group_grade}
                          {group_fornecedor}
                          unidade,
                          ep.id,
                          produto_codigo,
                          ep.name,
                          ell.locacao,
                          el.id,
                          ec.id,
                          ec.name
                ORDER  BY {ordenar_categoria}
                          {ordenar}
        """.format(codigo_listar=self.monta_campo_codigo_produto(wizlip_codigo_listar),
                   query_produto=self.query_produto(cr, uid, context),
                   query_locais=self.monta_filtro_de_locais(wizlip_local_ids),
                   unidade=self.monta_campo_unidade(wizlip_local_ids),
                   fornecedor=self.monta_campo_fornecedor(listar_fornecedor),
                   grade=self.monta_campo_grade(wizlip_listar_grades),
                   query_categorias=self.monta_filtro_de_categirias(wizlip_categoria_ids),
                   query_fornecedores=self.monta_filtro_de_fornecedor(wizlip_fornecedor_ids),
                   query_active=self.filtro_item_ativo(wizlip_estado_item),
                   group_grade=self.monta_group_grade(wizlip_listar_grades),
                   group_fornecedor=self.agrupar_por_fornecedor(wizlip_ordenar),
                   ordenar_categoria=self.ordernar_por_categoria(wizlip_agrupar_por_categoria,
                                                                 wizlip_ordenar),
                   ordenar=self.ordenar(wizlip_agrupar_por_categoria, wizlip_ordenar))
        cr.execute(sql)
        results = cr.dictfetchall()
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem.livro.preco',
            'datas': {'dados': results, 'filtros': dados}
        }

    @staticmethod
    def monta_filtro_de_locais(local_ids):
        filtro = ''
        if local_ids:
            filtro = 'AND el.id in %s' % str(local_ids).replace('[', '(').replace(']', ')')
        return filtro

    @staticmethod
    def monta_campo_unidade(local_ids):
        if len(local_ids) == 1:
            campo = 'eeu.unidade                  AS unidade,'
        else:
            campo = 'ep.estpro_unidade_inventario   AS unidade,'
        return campo

    @staticmethod
    def monta_campo_fornecedor(listar_fornecedor=False):
        campo = ''
        if listar_fornecedor:
            campo = 'ap.nome                        AS fornecedor,'
        return campo

    @staticmethod
    def monta_campo_grade(listar_grades=True):
        campo = ''
        if listar_grades:
            campo = """
                    epg.id                        AS grade_id,
                    epg.grade                     AS grade_grade,
                    """
        return campo

    @staticmethod
    def monta_group_grade(listar_grade=True):
        group = ''
        if listar_grade:
            group = 'epg.id, epg.grade,'
        return group

    @staticmethod
    def monta_filtro_de_categirias(categoria_ids):
        filtro = ''
        if categoria_ids:
            filtro = 'AND ec.id in %s' % str(categoria_ids).replace('[', '(').replace(']', ')')
        return filtro

    @staticmethod
    def monta_filtro_de_fornecedor(fornecedor_ids):
        filtro = ''
        if fornecedor_ids:
            filtro = 'AND ff.id in %s' % str(fornecedor_ids).replace('[', '(').replace(']', ')')
        return filtro

    @staticmethod
    def ordernar_por_categoria(agrupar_por_categoria, ordenar):
        order = ''
        if agrupar_por_categoria:
            order = 'categoria_completa,' if ordenar else 'categoria_completa'
        return order

    @staticmethod
    def agrupar_por_fornecedor(ordenar):
        group = ''
        if ordenar == 'F':
            group = 'ap.nome,'
        return group

    @staticmethod
    def ordenar(agrupar_por_categoria, ordenar):
        ordens = {'P': 'produto_codigo', 'D': 'produto_nome', 'F': 'ap.nome',
                  'L': 'locacao_locacao'}
        ordem = ordens.get(ordenar, 'produto_codigo')
        if agrupar_por_categoria and not ordenar:
            ordem = ',' + ordem
        return ordem

    @staticmethod
    def monta_campo_codigo_produto(codigo_listar):
        if codigo_listar == 'I':
            campo = """
            text_if_null(ep.codigo || '/' || ep.digito, 'SEM CODIGO')     AS produto_codigo,
            """
        elif codigo_listar == 'B':
            campo = """
            text_if_null(acb.codigo_barra::varchar, 'SEM COD/BAR') AS produto_codigo,
            """
        else:
            campo = """
                text_if_null(ap.codigo::varchar, 'SEM COD/FOR') AS produto_codigo,
            """
        return campo

    @staticmethod
    def filtro_item_ativo(opcao):
        if not opcao:
            return 'ep.active IN (True, False)'
        else:
            return 'ep.active = {active}'.format(active=True if opcao == 'A' else False)

    def query_produto(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_utilizar = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.produto', context=context
        )
        filtro = 'AND ep.company_id = {company_utilizar}'.format(
            company_utilizar=company_utilizar)
        return filtro

    def get_categoria_filha_ids(self, cr, uid, obj_categoria, parent_ids, categorias, context=None):
        """
        Metodo recursivo para buscar todas as categorias filhas de uma determinada categoria

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param obj_categoria: Objeto da classe estoque.categoria
        :param parent_ids: Lista contendo os parent_ids referentes a classe estoque.categoria
        :categorias: Lista onde sera adicionado todos os ids das categorias que devem ser
                     filtrados na consulta
        :type cr: cursor
        :type uid: int
        :type obj_categoria: object
        :type parent_ids: list
        :type categorias: list
        :type context: dict
        :rtype: list

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if not parent_ids:
            return categorias
        parent_ids = obj_categoria.search(cr, uid, [('parent_id', 'in', parent_ids)])
        categorias.extend(parent_ids)
        categorias = list(set(categorias))
        return self.get_categoria_filha_ids(cr, uid, obj_categoria, parent_ids, categorias, context)
