# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from datetime import date
# noinspection PyProtectedMember
from openerp.tools.translate import _
from openerp import pooler
from openerp import SUPERUSER_ID
import time
from SigUtils.numero import getNumeroEditado
from dateutil.parser import parse


# noinspection PyAbstractClass,PyPep8Naming
class wizard_listagem_movimento_sintetico_movimento(osv.AbstractModel):
    """
    Cria o modelo abstrato.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.listagem.movimento.sintetico.movimento'
    _auto = False
    _rec_name = 'movimento'
    _columns = {
        'id': fields.integer('id'),
        'movimento': fields.char('Movimento', size=10)
    }

    _order = 'movimento'

    def _auto_init(self, cr, context=None):
        result = super(wizard_listagem_movimento_sintetico_movimento, self)._auto_init(cr, context)
        self._execute_sql(cr)

        return result

    def _execute_sql(self, cr):
        params = dict(
            normal=_('Normal'), transferencias=_('Transferencia'), devolucoes=_('Devolucao'))
        # noinspection SqlDialectInspection
        _sql = """
        CREATE OR REPLACE VIEW wizard_listagem_movimento_sintetico_movimento AS
            SELECT 1                    AS id
                 , '%(normal)s'         AS movimento
            UNION
            SELECT 2                    AS id
                 , '%(transferencias)s' AS movimento
            UNION
            SELECT 3                    AS id
                 , '%(devolucoes)s'     AS movimento
        """ % params
        cr.execute(_sql)
        self._records = dict([(params.keys().index(i) + 1, params[i]) for i in params])


# noinspection PyAbstractClass,PyPep8Naming
class wizard_listagem_movimento_sintetico(osv.TransientModel):
    def _get_companies(self, cr, uid, context=None):
        context = context and context.copy() or {}
        return [self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, context=context)]

    _name = 'wizard.listagem.movimento.sintetico'
    _columns = {
        'data_inicial': fields.date('Periodo inicial', required=True),
        'data_final': fields.date('Periodo final', required=True),
        'company_ids': fields.many2many('res.company', 'res_company_movimento_sintetico', 'id',
                                        'company_id', 'Empresas'),
        'local_ids': fields.many2many('estoque.local', 'estoque_local_movimento_sintetico', 'id',
                                      'local_id', 'Locais'),
        'entrada': fields.boolean('Entrada'),
        'saida': fields.boolean(_('Saida')),
        'operacao_ids': fields.many2many('estoque.natureza.operacao',
                                         'estnat_operacao_listagem_movimento_sintetico',
                                         'id', 'operacao_id', _('Operacao')),
        'movimento': fields.many2many(
            'wizard.listagem.movimento.sintetico.movimento', 'movimento_origem_m2m_view_movimento',
            'id', 'movimento_id', 'Movimento'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Cliente/Fornecedor'),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'origem': fields.many2many(
            'estoque.movimento.origem', 'movimento_origem_m2m_movimento_sintetico',
            'id', 'origem_id', 'Origem'),
        'produto_id': fields.many2one('estoque.produto', 'Produto'),
        'grade_id': fields.many2one(
            'estoque.produto.grade', 'Grade', domain="[('produto_id','=',produto_id)]"),
        'fabricante_id': fields.many2one('adm.pessoa', 'Fabricante',
                                         domain="[('pessoa_tipo', '=', 'J')]"),
        'totaliza_item_igual': fields.boolean('Totalizar item', help='Totaliza os lancamentos para '
                                                                     'mesmos itens.'),
        'agrupar': fields.selection([('1', 'Categoria')], 'Agrupar por'),
        'grau_categoria': fields.integer('Grau a listar'),
        'tipo_ordenacao': fields.selection([('C', u'Código'), ('D', u'Descricao')],
                                           'Ordem a listar', size=1),
    }

    _defaults = {
        'data_inicial': date(date.today().year, date.today().month, 1).strftime('%Y-%m-%d'),
        'data_final': date.today().strftime('%Y-%m-%d'),
        'origem': lambda self, cr, user, context: self.pool.get(
            'estoque.movimento.origem').search(cr, user, [], context=context),
        'entrada': True,
        'saida': True,
        'movimento': lambda self, cr, user, context: self.pool.get(
            'wizard.listagem.movimento.sintetico.movimento').search(cr, user, [], context=context),
        'company_ids': _get_companies,
    }

    def onchange_agrupar(self, cr, uid, ids, agrupar, context=None):
        context = context and context.copy() or {}
        res = {}
        if agrupar == '1':
            return res

        res['values'] = {'grau_categoria': 0}
        return res

    def onchange_wizlms_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        args = [('produto_id', '=', produto_id)]
        grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid, args, context=context)
        if len(grade_ids) == 1:
            res['value'] = {'grade_id': grade_ids}
        else:
            res['value'] = {'grade_id': ''}
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        res = super(wizard_listagem_movimento_sintetico, self).read(cr, SUPERUSER_ID, ids, fields,
                                                                    context, load)
        return res

    def write(self, cr, user, ids, vals, context=None):
        res = super(wizard_listagem_movimento_sintetico, self).write(
            cr, SUPERUSER_ID, ids, vals, context)
        return res

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        estrutura, dados = self.get_dados_rel(cr, uid, ids, context)
        context['estrutura'] = estrutura
        context['dados'] = dados
        return self.pool.get('wizard.relatorios.impressao').get_tela_relatorios(cr, uid, context)

    def get_dados_rel(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        dados_movimento_sintetico = self.get_dados_movimento_sintetico(cr, uid, ids, context)
        pagina = {
            'relpag_nome_rel': _('Movimento sintetico'),
            'relpag_tam_linha': 155,
            'relpag_fonte_tamanho': 8.6,
            'relpag_margem_esquerda': 0.5,
            'relpag_margem_direita': 0.5,
            'relpag_margem_superior': 0.5,
            'relpag_margem_inferior': 0.5,
            'relpag_cabecalhos': [{'relcab_name': 'cab1',
                                   'relcab_separa_coluna': '|',
                                   'relcab_colunas': [{'relcol_name': 'titulo',
                                                       'relcol_dados_tam': 153,
                                                       'relcol_dados_alinhamento': 'C'}]},
                                  {'relcab_name': 'cab2',
                                   'relcab_separa_coluna': '|',
                                   'relcab_colunas': [{'relcol_name': 'data',
                                                       'relcol_dados_tam': 55,
                                                       'relcol_dados_alinhamento': 'D'},
                                                      {'relcol_name': 'empresa',
                                                       'relcol_dados_tam': 55,
                                                       'relcol_dados_alinhamento': 'C',
                                                       'relcol_nao_separa_coluna': True},
                                                      {'relcol_name': 'pagina',
                                                       'relcol_dados_tam': 50,
                                                       'relcol_dados_alinhamento': 'E',
                                                       'relcol_nao_separa_coluna': True}]
                                   },
                                  {'relcab_name': 'cab3',
                                   'relcab_separa_coluna': '|',
                                   'relcab_colunas': [{'relcol_name': 'periodo',
                                                       'relcol_dados_tam': 153,
                                                       'relcol_dados_alinhamento': 'D'}]},
                                  {'relcab_name': 'cab4',
                                   'relcab_separa_coluna': '|',
                                   'relcab_divide_linha': True,
                                   'relcab_colunas': [{'relcol_name': 'locais',
                                                       'relcol_dados_tam': 153,
                                                       'relcol_dados_alinhamento': 'D'}]
                                  },
                                  {'relcab_name': 'cabcol1',
                                   'relcab_separa_coluna': '|',
                                   'relcab_divide_linha': True,
                                   'relcab_cor': 'Blue',
                                   'relcab_colunas': [
                                       {'relcol_name': 'col1',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 4},
                                       {'relcol_name': 'col2',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 16},
                                       {'relcol_name': 'col3',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 38},
                                       {'relcol_name': 'col4',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 7},
                                       {'relcol_name': 'col5',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 12},
                                       {'relcol_name': 'col6',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 8},
                                       {'relcol_name': 'col7',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 20},
                                       {'relcol_name': 'col8',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 17},
                                       {'relcol_name': 'col9',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 12},
                                       {'relcol_name': 'col10',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 10}]
                                   }
                                  ],
            'relpag_linhas': [{'rellin_name': 'linha1',
                               'rellin_separa_coluna': '|',
                               'rellin_colunas': [{'relcol_name': 'categoria',
                                                   'relcol_dados_alinhamento': 'D',
                                                   'relcol_dados_tam': 153}]},
                              {'rellin_name': 'linha2',
                               'rellin_separa_coluna': '|',
                               'rellin_colunas': [
                                       {'relcol_name': 'cm',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 4},
                                       {'relcol_name': 'codigo_item',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 16},
                                       {'relcol_name': 'descricao',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 38},
                                       {'relcol_name': 'grade',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 7},
                                       {'relcol_name': 'quantidade',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 12},
                                       {'relcol_name': 'unidade',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_dados_tam': 8},
                                       {'relcol_name': 'custo',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 20},
                                       {'relcol_name': 'custo_encargo',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 17},
                                       {'relcol_name': 'valor_total',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 12},
                                       {'relcol_name': 'margem',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_dados_tam': 10}]},
                              {'rellin_name': 'linha3',
                               'rellin_colunas': [
                                   {'relcol_name': 'cm',
                                    'relcol_dados_alinhamento': 'D',
                                    'relcol_dados_tam': 5},
                                   {'relcol_name': 'codigo_item',
                                    'relcol_dados_alinhamento': 'D',
                                    'relcol_dados_tam': 17},
                                   {'relcol_name': 'descricao',
                                    'relcol_dados_alinhamento': 'D',
                                    'relcol_dados_tam': 39},
                                   {'relcol_name': 'grade',
                                    'relcol_dados_alinhamento': 'D',
                                    'relcol_dados_tam': 8},
                                   {'relcol_name': 'quantidade',
                                    'relcol_dados_alinhamento': 'E',
                                    'relcol_dados_tam': 13},
                                   {'relcol_name': 'unidade',
                                    'relcol_dados_alinhamento': 'D',
                                    'relcol_dados_tam': 9},
                                   {'relcol_name': 'custo',
                                    'relcol_dados_alinhamento': 'E',
                                    'relcol_dados_tam': 21},
                                   {'relcol_name': 'custo_encargo',
                                    'relcol_dados_alinhamento': 'E',
                                    'relcol_dados_tam': 18},
                                   {'relcol_name': 'valor_total',
                                    'relcol_dados_alinhamento': 'E',
                                    'relcol_dados_tam': 13},
                                   {'relcol_name': 'margem',
                                    'relcol_dados_alinhamento': 'E',
                                    'relcol_dados_tam': 10}]},
                              {'rellin_name': 'linha_total',
                               'rellin_separa_coluna': '|',
                               'rellin_divide_linha_anterior': True,
                               'rellin_colunas': [
                                       {'relcol_name': 'descricao',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 68},
                                       {'relcol_name': 'quantidade',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 12},
                                       {'relcol_name': 'unidade',
                                        'relcol_dados_alinhamento': 'D',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 8},
                                       {'relcol_name': 'custo',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 20},
                                       {'relcol_name': 'custo_encargo',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 17},
                                       {'relcol_name': 'valor_total',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 12},
                                       {'relcol_name': 'margem',
                                        'relcol_dados_alinhamento': 'E',
                                        'relcol_negrito': True,
                                        'relcol_dados_tam': 10}]}
                              ]
        }

        data_inicial = dados_movimento_sintetico.get('data_inicial')
        data_final = dados_movimento_sintetico.get('data_final')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        nome_empresa = self.pool.get('res.company').browse(cr, uid, company_id, context).name
        local_ids = dados_movimento_sintetico.get('local_ids')
        locais = ', '.join(map(
            lambda m: m.name, self.pool.get('estoque.local').browse(cr, uid, local_ids)))

        data = time.strftime("%d/%m/%Y")
        data_inicial = time.strftime('%d/%m/%Y', time.strptime(data_inicial, '%Y-%m-%d'))
        data_final = time.strftime('%d/%m/%Y', time.strptime(data_final, '%Y-%m-%d'))
        periodo = _('Periodo: %s a %s') % (data_inicial, data_final)

        params = ['estpar_custo_para_precovenda', 'comissoes', 'indexador_id', 'custo_fixo']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', params, context=context)
        custo_parametro = estoque_parametro_dados.get('estpar_custo_para_precovenda')
        custo_name = ''
        if custo_parametro == 'C':
            custo_name = _('CUSTO CORRIGIDO')
        elif custo_parametro == 'U':
            custo_name = _('ULTIMO CUSTO')
        elif custo_parametro == 'M':
            custo_name = _('CUSTO MEDIO CONTABIL')
        elif custo_parametro == 'R':
            custo_name = _('CUSTO REPOSICAO')

        dados = {'cab1': {'titulo': 'Movimento sintetico'},
                 'cab2': {'data': data,
                          'empresa': nome_empresa,
                          'pagina': '|pag|'},
                 'cab3': {'periodo': 'Periodo: ' + periodo},
                 'cab4': {'locais': 'Local: ' + (locais or 'Todos')},
                 'cabcol1': {'col1': 'CM',
                             'col2': 'CODIGO DO ITEM',
                             'col3': 'DESCRICAO',
                             'col4': 'GRADE',
                             'col5': 'QUANTIDADE',
                             'col6': 'UNIDADE',
                             'col7': custo_name,
                             'col8': 'CUSTO + ENCARGOS',
                             'col9': 'VALOR TOTAL',
                             'col10': '% MARGEM'}
                 }
        dados['linhas'] = self.get_linha_dados_sintetico(cr, uid, ids, dados_movimento_sintetico,
                                                         context) or []
        return pagina, dados

    def get_linha_dados_sintetico(self, cr, uid, ids, dados_movimento_sintetico, context=None):
        context = context and context.copy() or {}
        brw_movimento_sintetico = self.browse(
            cr, uid, ids[0] if isinstance(ids, (list, tuple)) else ids, context)
        ordenacao = brw_movimento_sintetico.tipo_ordenacao
        data_final_str = parse(brw_movimento_sintetico.data_final).strftime('%d/%m/%Y')
        linhas = []
        # ___________________________ Objects ________________________________
        obj_adm_utils = pooler.get_pool(cr.dbname).get('adm.utils')

        totaliza_item_igual = dados_movimento_sintetico.get('totaliza_item_igual')

        params = ['estpar_custo_para_precovenda', 'comissoes', 'indexador_id', 'custo_fixo']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', params, context=context)
        custo_parametro = estoque_parametro_dados.get('estpar_custo_para_precovenda')

        if custo_parametro == 'C':
            custo_parametro = 'custo_corrigido'
        elif custo_parametro == 'U':
            custo_parametro = 'ultimo_custo'
        elif custo_parametro == 'M':
            custo_parametro = 'custo_medio'
        elif custo_parametro == 'R':
            custo_parametro = 'custo_reposicao'

        comissao = estoque_parametro_dados.get('comissoes') or 0.0

        dados = dados_movimento_sintetico.get('dados')
        pcp_instalado = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'pcp', context)
        madeireira_instalado = obj_adm_utils.admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        list_dados = []
        list_produtos_cm = []
        list_diff = []
        grade_ids = list(set(map(lambda emi: emi.get('grade_id'), dados)))
        if not brw_movimento_sintetico.local_ids:
            estoque_saldo_dados = [self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                cr, uid, grade_ids, brw_movimento_sintetico.data_final, context=context)]
        else:
            estoque_saldo_dados = []
            for ms_local_id in brw_movimento_sintetico.local_ids:
                estoque_saldo_dados.append(self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                    cr, uid, grade_ids, brw_movimento_sintetico.data_final, ms_local_id.id,
                    context=context)
                )

        for movimento in dados:
            produto_id = movimento.get('produto_id')
            grade = movimento.get('grade')
            grade_id = movimento.get('grade_id')
            cm = self.get_cm(movimento.get('tipo_movimento'), movimento.get('origem'),
                             movimento.get('cfop_movimento'))

            movimento.update({'cm': cm})

            if not (produto_id, grade, cm) in list_diff:
                list_diff.append((produto_id, grade, cm))
                list_produtos_cm.append(
                    {'produto': produto_id, 'grade_id': grade_id, 'grade': grade, 'cm': cm})

        codigo_produto = grade = name_produto = ''

        for produto in list_produtos_cm:
            entrou = False
            icms = pis = cofins = total_item = quantidade = parametro_custo = custo_para_p_venda = 0
            filt_dados = filter(
                lambda i: produto.get('produto') == i.get('produto_id') and produto.get(
                    'grade') == i.get('grade') and produto.get('cm') == i.get('cm'), dados)

            for movimento in filt_dados:
                entrou = True
                fator_can = 1

                # Se o cupom for de cancelamento 'fator_can' = -1
                if movimento.get('tipo_cupom') == 'C':
                    fator_can = -1

                icms += (movimento.get('vr_icms') or 0.0) * fator_can
                pis += (movimento.get('vr_pis') or 0.0) * fator_can
                cofins += (movimento.get('vr_cofins') or 0.0) * fator_can
                total_item += (movimento.get('vr_total_item') or 0.0) + (movimento.get(
                    'vr_icms_st') or 0.0) + (movimento.get('vr_ipi') or 0.0)
                tipo_movimento = movimento.get('tipo_movimento')

                if madeireira_instalado and tipo_movimento == 'S' and movimento.get('volume'):
                    quantidade += movimento.get('volume') or 0.0
                else:
                    quantidade += movimento.get('quantidade') or 0.0

                custo = movimento.get(custo_parametro) or 0.0
                parametro_custo += round(custo, 4) * fator_can
                custo_para_p_venda += (movimento.get(custo_parametro) or 0.0) * fator_can
                custo_para_p_venda = round(custo_para_p_venda, 4)
                codigo_produto = movimento.get('produto_codigo') or ''
                grade = movimento.get('grade') or ''
                name_produto = movimento.get('produto_name') or ''

            if entrou:
                comissao_item = total_item * (comissao / 100.0)
                custo_fixo = estoque_parametro_dados.get('custo_fixo') or 0.0
                custo_fixo = total_item * (custo_fixo / 100.0)
                tributos = pis + cofins + icms
                custo_mais_encargos = custo_para_p_venda + custo_fixo + comissao_item + tributos
                total_custos = parametro_custo + tributos + comissao_item
                porcentagem_margem = 0.0

                # Se foi infomado apenas um local, a quantidade devera ser mostrada na unidade
                # embalagem do local informado. Se nenhum local informado ou mais de um local for
                # informado,a quantidade devera ser mostrada em unidades simples.
                produto_id = movimento.get('produto_id')
                grade_id = movimento.get('grade_id')

                if dados_movimento_sintetico.get('local_ids').__len__() == 1:
                    unidade, qtde_na_embalagem = self._get_embalagem_qtde_embalagem_local(
                        cr, produto_id, grade_id, dados_movimento_sintetico.get('local_ids')[0])
                else:
                    unidade, qtde_na_embalagem = self._get_embalagem_menor_qtde_embalagem(
                        cr, produto_id, grade_id)

                if qtde_na_embalagem:
                    quantidade /= qtde_na_embalagem
                if total_item:
                    porcentagem_margem = (total_item - total_custos) * 100 / total_item

                vals = {'cm': produto.get('cm'),
                        'codigo_produto': codigo_produto,
                        'grade': grade,
                        'name_produto': name_produto,
                        'quantidade': quantidade or 0.0,
                        'unidade': unidade or '',
                        'custo': parametro_custo or 0.0,
                        'custo_mais_encargos': custo_mais_encargos or 0.0,
                        'total': total_item or 0.0,
                        'margem': porcentagem_margem or 0.0,
                        'categoria_id': movimento.get('categoria_id'),
                        'grade_id': movimento.get('grade_id'),
                        }
                list_dados.append(vals)

        # Quantidade de digitos a ser mostrada
        if madeireira_instalado:
            digitos = 2
        else:
            digitos = 4

        if not ordenacao:
            list_dados = sorted(
                list_dados, key=lambda x: (x.get('name_produto'), x.get('grade'), x.get('cm')))

        # Totais
        total_sd = total_st = total_sn = total_si = total_sr = total_ss = qtd_sd = qtd_st = 0
        qtd_sn = qtd_si = qtd_sr = qtd_ss = total_ed = total_et = total_en = total_ei = total_er = 0
        total_es = total_ep = qtd_ed = qtd_et = qtd_en = qtd_ei = qtd_er = qtd_es = qtd_ep = 0
        custo_ed = custo_et = custo_ei = custo_ep = custo_er = custo_es = custo_en = 0
        custo_sd = custo_st = custo_si = custo_sr = custo_ss = custo_sn = 0

        codigo_produto = ''
        qtd_entrada = 0.0
        qtd_saida = 0.0
        vr_entrada = 0.0
        vr_saida = 0.0
        det_total_format = '|{:<71}{:>12}|{:>60}|{:>10}|'
        categria_ids = []
        for i in dados:
            if i.get('categoria_id'):
                categria_ids.append(i.get('categoria_id'))

        dados_catg = {}
        if categria_ids:
            map(lambda x: dados_catg.update({x.get('id'): x.get('nome_completo')}),
                pooler.get_pool(cr.dbname).get('estoque.categoria').read(cr, uid, categria_ids,
                                                                         ['nome_completo'],
                                                                         context))
        categoria_id = False
        ms_grade_id = False
        # produto_saldo_dados = self.pool.get('estoque.produto').estpro_get_saldo(
        #    cr, uid, list(set(map(lambda p: p.get('produto_id'), list_dados))),
        #    data=brw_movimento_sintetico.data_final, context=context)
        for dado in list_dados:
            if dado.get('quantidade') == 0:
                continue

            if codigo_produto != dado.get('codigo_produto') and list_dados.index(dado) != 0 and \
                    totaliza_item_igual:
                if qtd_entrada or vr_entrada:
                    linhas.append({'linha3': {'descricao': 'T O T A L  D E  E N T R A D A',
                                              'quantidade': getNumeroEditado(qtd_entrada, digitos),
                                              'valor_total': getNumeroEditado(vr_entrada)}})
                if qtd_saida or vr_saida:
                    linhas.append({'linha3': {'descricao': 'T O T A L  D E  S A I D A',
                                              'quantidade': getNumeroEditado(qtd_saida, digitos),
                                              'valor_total': getNumeroEditado(vr_saida)}})
                qtd_entrada = 0.0
                qtd_saida = 0.0
                vr_entrada = 0.0
                vr_saida = 0.0

            if ms_grade_id != dado.get('grade_id') and list_dados.index(dado) != 0 and \
                    totaliza_item_igual:
                linhas.append({'linha3': {
                    'descricao': 'Saldo em: %s' % data_final_str,
                    'quantidade': sum(map(lambda esd: esd.get(
                        ms_grade_id or dado.get('grade_id'), {}).get('saldo') or 0,
                                          estoque_saldo_dados)),
                }})

            if codigo_produto != dado.get('codigo_produto') and list_dados.index(dado) != 0 and \
                    totaliza_item_igual:
                linhas.append({'linha3': {}})

            if dado.get('categoria_id') and categoria_id != dado.get('categoria_id'):
                if categoria_id:
                    linhas.append({'linha1': {}})

                categoria_id = dado.get('categoria_id')
                nome_categ = "SEM CATEGORIA"
                if categoria_id:
                    nome_categ = dados_catg.get(categoria_id) or nome_categ

                linhas.append({'linha1': {'categoria': nome_categ}})

            linhas.append({'linha2': {
                'cm': dado.get('cm'),
                'codigo_item': dado.get('codigo_produto'),
                'descricao': dado.get('name_produto'),
                'grade': dado.get('grade'),
                'quantidade': getNumeroEditado(dado.get('quantidade'), digitos),
                'unidade': dado.get('unidade'),
                'custo': getNumeroEditado(dado.get('custo') or 0),
                'custo_encargo': getNumeroEditado(dado.get('custo_mais_encargos')),
                'valor_total': getNumeroEditado(dado.get('total')),
                'margem': getNumeroEditado(dado.get('margem') if dado.get('cm') == 'SN' else 0.0)}})

            if dado.get('cm')[0] == 'E':
                qtd_entrada += (dado.get('quantidade') or 0.0)
                vr_entrada += (dado.get('total') or 0.0)
            elif dado.get('cm')[0] == 'S':
                qtd_saida += (dado.get('quantidade') or 0.0)
                vr_saida += (dado.get('total') or 0.0)

            if list_dados.index(dado) == (len(list_dados) - 1) and totaliza_item_igual:
                if qtd_entrada or vr_entrada:
                    linhas.append({'linha3': {'descricao': 'T O T A L  D E  E N T R A D A',
                                              'quantidade': getNumeroEditado(qtd_entrada, digitos),
                                              'valor_total': getNumeroEditado(vr_entrada)}})
                if qtd_saida or vr_saida:
                    linhas.append({'linha3': {'descricao': 'T O T A L  D E  S A I D A',
                                              'quantidade': getNumeroEditado(qtd_saida, digitos),
                                              'valor_total': getNumeroEditado(vr_saida)}})
                linhas.append({'linha3': {
                    'descricao': 'Saldo em: %s' % data_final_str,
                    'quantidade': sum(map(lambda esd: esd.get(
                        ms_grade_id or dado.get('grade_id'), {}).get('saldo') or 0,
                                          estoque_saldo_dados)),
                }})
                qtd_entrada = 0.0
                qtd_saida = 0.0
                vr_entrada = 0.0
                vr_saida = 0.0

            codigo_produto = dado.get('codigo_produto')

            # Calcula os totais
            total_ed += dado.get('total') if dado.get('cm') == 'ED' else 0
            qtd_ed += dado.get('quantidade') if dado.get('cm') == 'ED' else 0
            custo_ed += dado.get('custo') if dado.get('cm') == 'ED' else 0

            total_et += dado.get('total') if dado.get('cm') == 'ET' else 0
            qtd_et += dado.get('quantidade') if dado.get('cm') == 'ET' else 0
            custo_et += dado.get('custo') if dado.get('cm') == 'ET' else 0

            total_en += dado.get('total') if dado.get('cm') in ['EN', 'EC'] else 0
            qtd_en += dado.get('quantidade') if dado.get('cm') in ['EN', 'EC'] else 0
            custo_en += dado.get('custo') if dado.get('cm') in ['EN', 'EC'] else 0

            total_ei += dado.get('total') if dado.get('cm') == 'EI' else 0
            qtd_ei += dado.get('quantidade') if dado.get('cm') == 'EI' else 0
            custo_ei += dado.get('custo') if dado.get('cm') == 'EI' else 0

            total_ep += dado.get('total') if dado.get('cm') == 'EP' else 0
            qtd_ep += dado.get('quantidade') if dado.get('cm') == 'EP' else 0
            custo_ep += dado.get('custo') if dado.get('cm') == 'EP' else 0

            total_er += dado.get('total') if dado.get('cm') == 'ER' else 0
            qtd_er += dado.get('quantidade') if dado.get('cm') == 'ER' else 0
            custo_er += dado.get('custo') if dado.get('cm') == 'ER' else 0

            total_es += dado.get('total') if dado.get('cm') == 'ES' else 0
            qtd_es += dado.get('quantidade') if dado.get('cm') == 'ES' else 0
            custo_es += dado.get('custo') if dado.get('cm') == 'ES' else 0

            total_sd += dado.get('total') if dado.get('cm') == 'SD' else 0
            qtd_sd += dado.get('quantidade') if dado.get('cm') == 'SD' else 0
            custo_sd += dado.get('custo') if dado.get('cm') == 'SD' else 0

            total_st += dado.get('total') if dado.get('cm') == 'ST' else 0
            qtd_st += dado.get('quantidade') if dado.get('cm') == 'ST' else 0
            custo_st += dado.get('custo') if dado.get('cm') == 'ST' else 0

            total_sn += dado.get('total') if dado.get('cm') == 'SN' else 0
            qtd_sn += dado.get('quantidade') if dado.get('cm') == 'SN' else 0
            custo_sn += dado.get('custo') if dado.get('cm') == 'SN' else 0

            total_si += dado.get('total') if dado.get('cm') == 'SI' else 0
            qtd_si += dado.get('quantidade') if dado.get('cm') == 'SI' else 0
            custo_si += dado.get('custo') if dado.get('cm') == 'SI' else 0

            total_sr += dado.get('total') if dado.get('cm') == 'SR' else 0
            qtd_sr += dado.get('quantidade') if dado.get('cm') == 'SR' else 0
            custo_sr += dado.get('custo') if dado.get('cm') == 'SR' else 0

            total_ss += dado.get('total') if dado.get('cm') == 'SS' else 0
            qtd_ss += dado.get('quantidade') if dado.get('cm') == 'SS' else 0
            custo_ss += dado.get('custo') if dado.get('cm') == 'SS' else 0

            ms_grade_id = dado.get('grade_id')

        linhas.append({'linha_total': {
            'descricao': _('Total de entradas por devolucao'),
            'quantidade': getNumeroEditado(qtd_ed),
            'custo': getNumeroEditado(custo_ed),
            'valor_total': getNumeroEditado(total_ed),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de entradas por transferencia'),
            'quantidade': getNumeroEditado(qtd_et),
            'custo': getNumeroEditado(custo_et),
            'valor_total': getNumeroEditado(total_et),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de entradas por inventario'),
            'quantidade': getNumeroEditado(qtd_ei),
            'custo': getNumeroEditado(custo_ei),
            'valor_total': getNumeroEditado(total_ei),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de entradas por requisicao almoxarifado'),
            'quantidade': getNumeroEditado(qtd_er),
            'custo': getNumeroEditado(custo_er),
            'valor_total': getNumeroEditado(total_er),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de entradas por requisicao ressuprimento'),
            'quantidade': getNumeroEditado(qtd_es),
            'custo': getNumeroEditado(custo_es),
            'valor_total': getNumeroEditado(total_es),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        if pcp_instalado:
            linhas.append({'linha_total': {
                'descricao': _('Total de entradas de producao'),
                'quantidade': getNumeroEditado(qtd_ep),
                'custo': getNumeroEditado(custo_ep),
                'valor_total': getNumeroEditado(total_ep),
                'unidade': '',
                'custo_encargo': '',
                'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total das compras'),
            'quantidade': getNumeroEditado(qtd_en),
            'custo': getNumeroEditado(custo_en),
            'valor_total': getNumeroEditado(total_en),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de saidas por devolucao'),
            'quantidade': getNumeroEditado(qtd_sd),
            'custo': getNumeroEditado(custo_sd),
            'valor_total': getNumeroEditado(total_sd),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de saidas por transferencia'),
            'quantidade': getNumeroEditado(qtd_st),
            'custo': getNumeroEditado(custo_st),
            'valor_total': getNumeroEditado(total_st),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de saidas por inventario'),
            'quantidade': getNumeroEditado(qtd_si),
            'custo': getNumeroEditado(custo_si),
            'valor_total': getNumeroEditado(total_si),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de saidas por requisicao almoxarifado'),
            'quantidade': getNumeroEditado(qtd_sr),
            'custo': getNumeroEditado(custo_sr),
            'valor_total': getNumeroEditado(total_sr),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total de saidas por requisicao ressuprimento'),
            'quantidade': getNumeroEditado(qtd_ss),
            'custo': getNumeroEditado(custo_ss),
            'valor_total': getNumeroEditado(total_ss),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        linhas.append({'linha_total': {
            'descricao': _('Total das vendas'),
            'quantidade': getNumeroEditado(qtd_sn),
            'custo': getNumeroEditado(custo_sn),
            'valor_total': getNumeroEditado(total_sn),
            'unidade': '',
            'custo_encargo': '',
            'margem': ''}})

        return linhas

    @staticmethod
    def get_cm(tipo_movimento, origem, cfop_movimento):
        cm = tipo_movimento or ''
        if cfop_movimento:
            cm += cfop_movimento or '' if cfop_movimento not in 'UC' else 'N'
        cm += origem or '' if len(cm) < 2 and origem != 'U' else 'N' if len(cm) < 2 else ''

        return cm

    @staticmethod
    def _get_embalagem_qtde_embalagem_local(cr, produto, grade, local):
        """
        Recupera a sigla da embalagem e a quantidade na embalagem de um produto em um determinado
        local (local/locacao).

        :param cr: a linha atual do cursor do banco de dados
        :param produto: ID do produto
        :param grade: ID da grade do produto
        :param local: ID do local (local/locacao)
        :type cr: cursor
        :type produto: long
        :type grade: long
        :type local: long
        :return: tupla com a sigla da embalagem e a quantidade de unidades nesta embalagem
        :rtype: tuple(unicode, float)

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # noinspection SqlDialectInspection
        _sql = """
              SELECT      ee.sigla, ee.quantidade_na_embalagem
              FROM        estoque_local_locacao     AS ell
              INNER JOIN  estoque_produto_embalagem AS epe  ON epe.id = ell.estllc_produto_embalagem_id
              INNER JOIN  estoque_embalagem         AS ee   ON ee.id = epe.embalagem_id
              WHERE       ell.produto_id = %d
                AND       ell.grade_id = %d
                AND       ell.local_id = %d
            """ % (produto, grade, local)
        cr.execute(_sql)
        result = cr.fetchone()

        if not result:
            result = None, 0.0

        return result

    @staticmethod
    def _get_embalagem_menor_qtde_embalagem(cr, produto, grade):
        """
        Recupera a sigla da embalagem e a menor quantidade na embalagem de um produto.

        :param cr: a linha atual do cursor do banco de dados
        :param produto: ID do produto
        :param grade: ID da grade do produto
        :type cr: cursor
        :type produto: long
        :type grade: long
        :return: tupla com a sigla da embalagem e a quantidade de unidades nesta embalagem
        :rtype: tuple(unicode, float)

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # noinspection SqlDialectInspection
        _sql = """
              SELECT      ee.sigla, ee.quantidade_na_embalagem
              FROM        estoque_local_locacao     AS ell
              INNER JOIN  estoque_produto_embalagem AS epe  ON epe.id = ell.estllc_produto_embalagem_id
              INNER JOIN  estoque_embalagem         AS ee   ON ee.id = epe.embalagem_id
              WHERE       ell.produto_id = %d
                AND       ell.grade_id = %d
              ORDER BY    ee.quantidade_na_embalagem
            """ % (produto, grade)
        cr.execute(_sql)
        result = cr.fetchone()  # Pela ordem crescente dos registro recupera a menor unidade

        if not result:
            result = None, 0.0

        return result

    def get_dados_movimento_sintetico(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_res_company = pooler.get_pool(cr.dbname).get('res.company')
        brw_movimento = self.browse(cr, uid, ids[0])
        ordenacao = brw_movimento.tipo_ordenacao
        company_ids = map(lambda i: i.id, brw_movimento.company_ids)
        local_ids = map(lambda i: i.id, brw_movimento.local_ids)
        if not company_ids:
            company_ids = obj_res_company.search(
                cr, uid, [], context=context)
        else:
            child_company_ids = []
            for c in company_ids:
                child_company_ids += obj_res_company.search(cr, uid, [('parent_id', 'child_of', c)],
                                                            context=context)
            company_ids = list(set(child_company_ids))
        str_company_ids = ', '.join(map(lambda c: str(c), company_ids))

        produto_id = ''
        if brw_movimento.produto_id:
            produto_id = '''AND emi.produto_id = '%s' ''' % brw_movimento.produto_id.id

        grade_id = ''
        if brw_movimento.grade_id:
            grade_id = '''AND emi.grade_id = '%s' ''' % brw_movimento.grade_id.id

        if brw_movimento.fabricante_id:
            fabricante_produto_ids = obj_estoque_produto.search(
                cr, uid, [('fabricante_id', '=', brw_movimento.fabricante_id.id)],
                context=context)
            if fabricante_produto_ids:
                if len(fabricante_produto_ids) == 1:
                    str_fabricante_produto_ids = str(tuple(
                        fabricante_produto_ids)).replace(',', '')
                else:
                    str_fabricante_produto_ids = str(tuple(fabricante_produto_ids))
                produto_id = '''AND emi.produto_id in %s ''' % str_fabricante_produto_ids
            else:
                produto_id = '''AND emi.produto_id IS NULL '''
        if brw_movimento.produto_id and brw_movimento.fabricante_id:
            if brw_movimento.produto_id.fabricante_id != brw_movimento.fabricante_id:
                produto_id = '''AND emi.produto_id IS NULL '''
        obj_adm_util = self.pool.get('adm.utils')
        fiscal_instalado = obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'fiscal', context)
        ecf_instalado = obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'ecf', context)
        modelo_nota = ' '
        filtro_cfop_ids = []
        if fiscal_instalado:
            if brw_movimento.modelo_nf_id:
                modelo_nota = '''AND emc.nf_capa_id in (SELECT id
                                            FROM fiscal_nf_capa
                                            WHERE modelo_nf_id = %s)
                                            ''' % brw_movimento.modelo_nf_id.id
            filtro_cfop_ids = map(lambda i: i.id, brw_movimento.cfop_ids)
        if brw_movimento.data_inicial > brw_movimento.data_final:
            raise osv.except_osv(_('Atencao'), _('Data inicial nao pode ser maior que a data '
                                                 'final'))
        madeireira_instalado = obj_adm_util.admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        if madeireira_instalado:
            volume = 'emi.estmvi_volume AS volume,'
        else:
            volume = ''
        if ecf_instalado:
            tipo_cupom = '''
                            ,
                ecc.tipo_cupom
                         '''
            left_join_ecf = '''
            LEFT JOIN ecf_cupons_capa AS ecc ON ecc.id = emi.cupom_id
                            '''
            query_quantidade = """
                Sum(
                    CASE
                        WHEN ecc.tipo_cupom = 'C'
                            THEN emi.quantidade * -1
                        ELSE emi.quantidade
                    END
                ) AS quantidade,
            """
            query_total_item = """
                Sum(
                    CASE
                        WHEN ecc.tipo_cupom = 'C'
                            THEN emi.vr_total_item_rateado * -1
                        ELSE emi.vr_total_item_rateado
                    END
                ) AS vr_total_item,
            """
        else:
            tipo_cupom = '''
                         '''
            left_join_ecf = '''
                            '''
            query_quantidade = """
                sum(emi.quantidade) AS quantidade,
            """
            query_total_item = """
                sum(emi.vr_total_item_rateado) AS vr_total_item,
            """
        # noinspection SqlDialectInspection
        sql = """
            SELECT
                %s
                %s
                emi.produto_id AS produto_id,
                emi.grade_id   AS grade_id,
                %s
                sum(emi.estmvi_saidas_custo_corrigido_total) AS custo_corrigido,
                sum(emi.estmvi_saidas_custo_reposicao_total) AS custo_reposicao,
                sum(emi.estmvi_saidas_custo_medio_contabil_total) AS custo_medio,
                sum(emi.estmvi_saidas_ultimo_custo_total) AS ultimo_custo,
                (SELECT codigo FROM estoque_produto WHERE id = emi.produto_id)
                            AS produto_codigo,
                (SELECT name FROM estoque_produto WHERE id = emi.produto_id)
                        AS produto_name,
                (SELECT grade FROM estoque_produto_grade WHERE id = emi.grade_id)
                        AS grade,
                %s
            """ % (query_quantidade, volume, query_total_item,
                   self.get_tipo_movimento(cr, uid, context))
        if fiscal_instalado:
            sql += """
                {origem_movimento}
                sum((SELECT vr_encargo FROM fiscal_nf_item_tributacao WHERE movimento_itens_id =
                    emi.id AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS'))
                    ) AS vr_icms,
                sum((SELECT vr_icms_substituicao FROM fiscal_nf_item_tributacao WHERE
                    movimento_itens_id = emi.id AND encargo_id IN (SELECT id FROM fiscal_encargo
                                                                   WHERE sigla = 'ICMS')
                    )) AS vr_icms_st,
                sum((SELECT vr_encargo FROM fiscal_nf_item_tributacao WHERE
                    movimento_itens_id = emi.id AND encargo_id IN (SELECT id FROM fiscal_encargo
                                                                   WHERE sigla = 'IPI')
                    )) AS vr_ipi,
                sum((SELECT vr_encargo FROM fiscal_nf_item_tributacao WHERE movimento_itens_id =
                    emi.id AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'PIS'))
                    ) AS vr_pis,
                sum((SELECT vr_encargo FROM fiscal_nf_item_tributacao WHERE movimento_itens_id =
                    emi.id AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'COFINS'))
                    ) AS vr_cofins,
            """.format(origem_movimento=self.get_origem_movimento(cr, uid, context))
            left_join_fiscal = """
            LEFT JOIN fiscal_cfop fc
                ON fc.id = emi.cfop_id
            """
        else:
            left_join_fiscal = """ """

        campo_conta = ''
        campo_categ = ''
        order_categ = ''
        left_categ = ''
        group_categ = ''
        if brw_movimento.agrupar == '1':
            group_categ = ', ec.conta, ec.id'
            order_categ = 'ec.conta,'
            campo_conta = 'ec.conta AS conta,'
            campo_categ = 'ec.id AS categoria_id,'
            left_categ = """INNER JOIN estoque_produto AS ep ON ep.id = emi.produto_id
            LEFT JOIN estoque_categoria AS ec ON ec.id = ep.categoria_id"""

        sql += """
            %s
            %s
            emo.estmog_codigo AS origem
                %s
            FROM estoque_movimento_itens emi
            INNER JOIN estoque_movimento_capa emc
                ON emc.id = emi.capa_id
            INNER JOIN estoque_natureza_operacao eno
                ON eno.id = emc.estmvc_operacao_id
            LEFT JOIN estoque_movimento_origem emo
                ON emo.id = eno.estnto_movimento_origem_id
            %s
            %s
            %s
            WHERE
                emc.data >= '%s'
                AND emc.data <= '%s'
                AND emc.state = 'F'
                AND emc.company_id in (%s)
                %s
        """ % (campo_conta, campo_categ, tipo_cupom, left_categ, left_join_fiscal, left_join_ecf,
               brw_movimento.data_inicial, brw_movimento.data_final,
               str_company_ids, modelo_nota)
        args_natureza_operacao = [('estnto_atualizar_saldo', '=', True)]
        if not brw_movimento.operacao_ids:
            tipo_operacao = []
            if brw_movimento.entrada:
                tipo_operacao.append('E')
            if brw_movimento.saida:
                tipo_operacao.append('S')

            if tipo_operacao:
                args_natureza_operacao.append(('estnto_tipo_operacao', 'in', tipo_operacao))

            natureza_operacao_ids = map(lambda x: '%s' % x,
                                        self.pool.get('estoque.natureza.operacao').search(cr, uid,
                                        args_natureza_operacao, context=context))
        else:
            natureza_operacao_ids = map(lambda x: '%s' % x.id, brw_movimento.operacao_ids)

        if natureza_operacao_ids:
            sql += """ AND estmvc_operacao_id in (%s)""" % ','.join(natureza_operacao_ids)

        if brw_movimento.pessoa_id:
            sql += """ AND emc.pessoa_id = %s""" % brw_movimento.pessoa_id.id

        origem_e_cfop_list = []
        cfop_movimento_tipo = list(set(''.join(filter(lambda f: f in 'CUTD', str(map(
            lambda m: {1: ['C', 'U'], 2: 'T', 3: 'D'}.get(m.id),
            brw_movimento.movimento))))))

        origem = map(lambda m: str(m.estmog_codigo), brw_movimento.origem)

        if origem:
            origem_cfop = []
            if cfop_movimento_tipo:
                origem_cfop = filter(lambda o: o in ['N', 'C'], origem)
                origem = filter(lambda o: o not in ['N', 'C'], origem)
            if not origem:
                origem = 'FALSE'
            elif len(origem) == 1:
                origem = str(tuple(origem)).replace(',', '')
            else:
                origem = str(tuple(origem))
            if origem_cfop:
                origem_e_cfop_list.append(origem == 'FALSE' and origem or (
                        'emo.estmog_codigo in %s' % origem))
                origem_e_cfop_list.append("emo.estmog_codigo in ('%s')" % "', '".join(
                    origem_cfop))
            else:
                sql += """ AND emo.estmog_codigo in %s""" % origem

        if local_ids:
            sql += """ AND emi.local_id in (%s)""" % ', '.join([str(_id) for _id in local_ids])

        if brw_movimento.categoria_id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', brw_movimento.categoria_id.id)],
                context=context)

            if categoria_ids:
                if len(categoria_ids) == 1:
                    categoria_ids = str(tuple(categoria_ids)).replace(',', '')
                else:
                    categoria_ids = str(tuple(categoria_ids))
                sql += """ AND emi.produto_id IN (
                    SELECT id FROM estoque_produto WHERE categoria_id IN %s)""" % categoria_ids
            else:
                sql += '''AND emi.produto_id IS NULL '''

        if cfop_movimento_tipo:
            sql_cfop = """ AND fc.movimento_tipo IN ('%s') """ % "', '".join(cfop_movimento_tipo)
            if origem_e_cfop_list:
                origem_e_cfop_list.append(sql_cfop)
                sql += """ AND (%s OR (%s AND %s)) """ % (
                    origem_e_cfop_list[0], origem_e_cfop_list[1], origem_e_cfop_list[2].replace(
                        'AND', ''))
            else:
                sql += sql_cfop

        if filtro_cfop_ids:
            if len(filtro_cfop_ids) == 1:
                cfop_ids = str(tuple(filtro_cfop_ids)).replace(',', '')
            elif len(filtro_cfop_ids) > 1:
                cfop_ids = str(tuple(filtro_cfop_ids))

            # noinspection PyUnboundLocalVariable
            sql += """ AND emi.cfop_id IN %s """ % cfop_ids

        if brw_movimento.vendedor_id:
                sql += """ AND emi.vendedor_id = %s""" % brw_movimento.vendedor_id.id

        if ecf_instalado:
            sql += """
                      AND not (emi.item_cancelado is True AND  ecc.tipo_cupom = 'V')
                   """

        sql += """ %s
                   %s """ % (produto_id, grade_id)
        # O 'tipo_movimento' usado no group by abaixo eh incluido por meio do metodo
        # 'get_tipo_movimento'. Ele possui heranca. Atentar para alteracao ou remocao.
        sql += """
            GROUP BY emi.id,
                     emi.produto_id,
                     emi.grade_id,
                     tipo_movimento,
                     cfop_movimento,
                     emo.estmog_codigo %s
        """ % group_categ
        if madeireira_instalado:
            sql += """
                , emi.estmvi_volume
            """
        if ecf_instalado:
            sql += """
                , ecc.tipo_cupom
            """

        ordem_padrao = """ %s emi.produto_id, emi.grade_id""" % order_categ
        sql += """
            ORDER BY
                {ordem}
        """.format(ordem=ordenacao == 'C' and 'produto_codigo' or ordenacao == 'D'
                   and 'produto_name' or ordem_padrao)

        estoque_parametro_dados = obj_adm_util.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_custo_para_precovenda', context=context)
        custo_parametro = estoque_parametro_dados.get('estpar_custo_para_precovenda')
        if not custo_parametro:
            raise osv.except_osv(_('Atencao'), _(
                'Custo para formacao do preco nao esta configurado no parametro.'))

        cr.execute(sql)
        dados = cr.dictfetchall()

        # mostra as categorias do nivel informado e suas filhas
        if dados and brw_movimento.grau_categoria > 0:
            dados = filter(
                lambda c: (len((c.get('conta') or '').split('.')) if c.get('conta') else -1) >=
                          brw_movimento.grau_categoria, dados)

        if not dados:
            raise osv.except_osv(_('Atencao'),
                                 _('Com os filtros selecionados nao ha dados a serem impressos'))

        datas = {'data_inicial': brw_movimento.data_inicial,
                 'data_final': brw_movimento.data_final,
                 'company_ids': company_ids,
                 'local_ids': local_ids,
                 'dados': dados,
                 'totaliza_item_igual': brw_movimento.totaliza_item_igual,
                 }

        return datas

    def get_origem_movimento(self, cr, uid, context=None):
        """
        O metodo identifica a origem do item movimentado no estoque.
        Se nota ou cupom fiscal, eh identificado pelo tipo de movimento do CFOP.
        Se inventario, eh identificado pelo id desse (inventario_id) na capa do movimento.
        Metodo sobreescrito nos modulos estoquealmoxarifado e estoqueressuprimento.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: query para busca dos dados no movimento
        :rtype: str
        """
        context = context and context.copy() or {}
        sql = """
            CASE
                WHEN emc.inventario_id IS NOT NULL THEN 'I'
                ELSE fc.movimento_tipo
            END   AS cfop_movimento,
        """
        return sql

    def get_tipo_movimento(self, cr, uid, context=None):
        """
        O metodo identifica o tipo da natureza da operacao do movimento.
        Metodo sobreescrito no modulo estoquealmoxarifado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: query para busca dos dados no movimento
        :rtype: str
        """
        context = context and context.copy() or {}
        sql = """
                eno.estnto_tipo_operacao AS tipo_movimento,
        """
        return sql

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
