# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from dateutil.parser import parse
from SigUtils.numero import getNumeroEditado
import time


class wizard_listagem_participacao_vendas(osv.TransientModel):
    _name = 'wizard.listagem.participacao.vendas'
    _columns = {
        'data_inicial': fields.date('Periodo inicial', required=True),
        'data_final': fields.date('Periodo final', required=True),
        'opcao': fields.selection(
            [('I', 'Itens'), ('C', 'Categorias'), ('F', 'Fabricante'), ('V', 'Vendedores')],
            'Opcoes a listar', size=1, required=True),
        'porcentagem_listar': fields.integer('% a listar'),
        'ordem': fields.selection([('F', 'Faturamento'), ('V', 'Valor agregado')], 'Ordem a listar',
                                  size=1, required=True),
        'custo': fields.selection(
            [('UC', 'Ultimo custo'), ('CC', 'Custo corrigido'), ('RE', 'Reposicao'), ('CM', 'Custo medio')],
            'Custo a considerar', size=2, required=True),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'agrupar_sem_grade': fields.boolean('Agrupar sem grade'),
        'categoria_grau': fields.integer('Grau a listar'),
    }
    _defaults = {
        'opcao': 'I',
        'porcentagem_listar': 100,
        'ordem': 'F',
        'custo': 'CC',
        'categoria_grau': 0,
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        estrutura, dados = self.get_dados_rel(cr, uid, ids, context)
        context['estrutura'] = estrutura
        context['dados'] = dados
        return self.pool.get('wizard.relatorios.impressao').get_tela_relatorios(cr, uid, context)

    def get_dados_rel(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_filtro = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.itens', context=context)
        list_company = self.pool.get('res.company')._get_company_children(cr, uid, company_id)

        if brw_filtro.data_inicial > brw_filtro.data_final:
            raise osv.except_osv(_('Atencao'), _('Data inicial nao pode ser maior que a data '
                                                 'final'))
        sql = self.get_sql(cr, uid, list_company, brw_filtro.categoria_id.id, brw_filtro.opcao,
                           brw_filtro.data_inicial, brw_filtro.data_final,
                           brw_filtro.agrupar_sem_grade, context)
        sql += ''') as tmp
                        %s ''' % self.get_group_by(brw_filtro.opcao, brw_filtro.agrupar_sem_grade)
        sql = sql.format(quantidade=self.sql_quantidade())
        cr.execute(sql)
        itens_dados = cr.dictfetchall()
        if brw_filtro.opcao == 'C' and brw_filtro.categoria_grau > 0:
            itens_dados = filter(
                lambda c: (len((c.get('conta') or '').split('.')) if c.get('conta') else -1) ==
                brw_filtro.categoria_grau, itens_dados)

        pagina = {'relpag_nome_rel': _('Participacao nas Vendas'),
                  'relpag_tam_linha': 160,
                  'relpag_fonte_tamanho': 8.4,
                  'relpag_margem_esquerda': 0.5,
                  'relpag_margem_direita': 0.5,
                  'relpag_margem_superior': 0.5,
                  'relpag_margem_inferior': 0.5,
                  'relpag_cabecalhos': [{'relcab_name': 'cab1',
                                         'relcab_colunas': [{'relcol_name': 'titulo',
                                                             'relcol_dados_tam': 155,
                                                             'relcol_dados_alinhamento': 'C'}]},
                                        {'relcab_name': 'cab2',
                                         'relcab_divide_linha': True,
                                         'relcab_colunas': [{'relcol_name': 'periodo',
                                                             'relcol_dados_tam': 35},
                                                            {'relcol_name': 'empresa',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 100},
                                                            {'relcol_name': 'pagina',
                                                             'relcol_dados_tam': 23}], },
                                        {'relcab_name': 'cab_col',
                                         'relcab_separa_coluna': '|',
                                         'relcab_cor': 'Blue',
                                         'relcab_colunas': [{'relcol_name': 'col1',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 5},
                                                            {'relcol_name': 'col2',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 15},
                                                            {'relcol_name': 'col3',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 5},
                                                            {'relcol_name': 'col4',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 37},
                                                            {'relcol_name': 'col5',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 8},
                                                            {'relcol_name': 'col6',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 11},
                                                            {'relcol_name': 'col7',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 11},
                                                            {'relcol_name': 'col8',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 12},
                                                            {'relcol_name': 'col9',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 8},
                                                            {'relcol_name': 'col10',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 9},
                                                            {'relcol_name': 'col11',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 12},
                                                            {'relcol_name': 'col12',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 14}
                                                            ], },
                                        {'relcab_name': 'cab_col1',
                                         'relcab_separa_coluna': ' ',
                                         'relcab_divide_linha': True,
                                         'relcab_cor': 'Blue',
                                         'relcab_colunas': [{'relcol_name': 'col1',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 5},
                                                            {'relcol_name': 'col2',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 15},
                                                            {'relcol_name': 'col3',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 5},
                                                            {'relcol_name': 'col4',
                                                             'relcol_dados_alinhamento': 'D',
                                                             'relcol_dados_tam': 37},
                                                            {'relcol_name': 'col5',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 8},
                                                            {'relcol_name': 'col6',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 11},
                                                            {'relcol_name': 'col7',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 11},
                                                            {'relcol_name': 'col8',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 12},
                                                            {'relcol_name': 'col9',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 8},
                                                            {'relcol_name': 'col10',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 9},
                                                            {'relcol_name': 'col11',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 12},
                                                            {'relcol_name': 'col12',
                                                             'relcol_dados_alinhamento': 'E',
                                                             'relcol_dados_tam': 14}
                                                            ], },
                                        ],

                  'relpag_linhas': [{'rellin_name': 'linha1',
                                     'rellin_separa_coluna': '|',
                                     'rellin_colunas': [{'relcol_name': 'sequencia',
                                                         'relcol_dados_alinhamento': 'D',
                                                         'relcol_dados_tam': 5},
                                                        {'relcol_name': 'codigo',
                                                         'relcol_dados_alinhamento': 'D',
                                                         'relcol_dados_tam': 15},
                                                        {'relcol_name': 'grade',
                                                         'relcol_dados_alinhamento': 'D',
                                                         'relcol_dados_tam': 5},
                                                        {'relcol_name': 'descricao',
                                                         'relcol_dados_alinhamento': 'D',
                                                         'relcol_dados_tam': 37},
                                                        {'relcol_name': 'saldo',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 8},
                                                        {'relcol_name': 'quantidade',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 11},
                                                        {'relcol_name': 'vl_total',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 11},
                                                        {'relcol_name': 'vl_acumulado',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 12},
                                                        {'relcol_name': 'percentual_individual',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 8},
                                                        {'relcol_name': 'percentual_acumulado',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 9},
                                                        {'relcol_name': 'vl_agregado',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 12},
                                                        {'relcol_name': 'percentual_agregado_venda',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 14}]
                                     }, ],
                  }

        pagina['relpag_linhas'] += [pagina['relpag_linhas'][0].copy()]
        pagina['relpag_linhas'][-1].update({'rellin_name': 'linha_total'})

        periodo_inicial = brw_filtro.data_inicial
        periodo_final = brw_filtro.data_final
        custo = brw_filtro.custo
        ordem = brw_filtro.ordem
        opcao = brw_filtro.opcao
        if opcao == 'I':
            tipo_item = 'ITEM'
        elif opcao == 'C':
            tipo_item = 'CATEGORIA'
        elif opcao == 'F':
            tipo_item = 'FABRICANTE'
        else:
            tipo_item = 'VENDEDOR'
        porcentagem_listar = brw_filtro.porcentagem_listar
        valor_total = sum(map(lambda d: d.get('vr_total_item_rateado') or 0, itens_dados))

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        nome_empresa = self.pool.get("res.company").read(cr, uid, company_id, ['name'],
                                                         context).get('name')
        dados = {'cab1': {'titulo': _('Participacao nas Vendas')},
                 'cab2': {'periodo': 'Periodo: %s a %s' % (
                     parse(periodo_inicial).strftime("%d/%m/%Y"),
                     parse(periodo_final).strftime("%d/%m/%Y")),
                          'empresa': nome_empresa,
                          'pagina': '|pag|'},
                 'cab_col': {'col1': _('SEQ'),
                             'col2': _('COD ' + tipo_item),
                             'col3': _('GR'),
                             'col4': _('DESCRICAO'),
                             'col5': 'SALDO',
                             'col6': _('QUANT'),
                             'col7': _('VL TOTAL'),
                             'col8': _('VL ACUMUL'),
                             'col9': _('% INDIV'),
                             'col10': _('% ACUMUL'),
                             'col11': _('VL AGREGADO'),
                             'col12': _('% VL AGREGADO')},
                 'cab_col1': {'col1': '',
                              'col2': '',
                              'col3': '',
                              'col4': '',
                              'col5': '',
                              'col6': '',
                              'col7': '',
                              'col8': '',
                              'col9': '',
                              'col10': '',
                              'col11': '',
                              'col12': _('S/ VENDAS')}
                 }
        linhas = []

        i = 1
        porcentagem_acumulada = 0.00
        total_agregado = 0.00
        total_acumulado = 0
        for movimento in itens_dados:
            if movimento.get('vr_total_item_rateado') == 0:
                movimento['valor_agregado'] = 0
                continue

            total_item = movimento.get('vr_total_item_rateado') or 0
            if custo == 'CC':
                preco_custo = movimento.get('custo_corrigido') or 0.0
            elif custo == 'UC':
                preco_custo = movimento.get('ultimo_custo') or 0.0
            elif custo == 'CM':
                preco_custo = movimento.get('custo_medio') or 0.0
            else:
                preco_custo = movimento.get('custo_reposicao') or 0.0

            valor_agregado = round(total_item - preco_custo, 2)
            movimento['valor_agregado'] = valor_agregado
        if ordem == 'F':
            itens_dados = sorted(itens_dados, key=lambda v: v.get('vr_total_item_rateado'),
                                 reverse=True)
        elif ordem == 'V':
            itens_dados = sorted(itens_dados, key=lambda v: (v.get('valor_agregado'),
                                                             v.get('vr_total_item_rateado')),
                                 reverse=True)

        # Saldo do estoque. Para quando a opcao a listar seja 'Itens'.
        estoque_saldo = {}
        if opcao == 'I' and not brw_filtro.agrupar_sem_grade:
            grade_ids = list(set(map(lambda g: g.get('grade_id'), itens_dados)))
            estoque_saldo = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                cr, uid, grade_ids, time.strftime('%Y-%m-%d'), context=context)

        for movimento in itens_dados:
            if movimento.get('vr_total_item_rateado') == 0:
                continue
            quantidade = movimento.get('quantidade') or 0
            total_item = movimento.get('vr_total_item_rateado') or 0
            total_acumulado += total_item
            grade = ''
            codigo_item = ''
            item_name = ''
            if opcao == 'I':
                codigo_item = movimento.get('codigo')
                grade = movimento.get('grade') or ''
                item_name = movimento.get('produto_name') or ''
            elif opcao == 'C':
                codigo_item = movimento.get('conta') or ''
                item_name = movimento.get('categoria_name') or 'SEM CATEGORIA'
            elif opcao == 'F':
                codigo_item = movimento.get('fabricante_codigo') or ''
                item_name = movimento.get('fabricante_name') or 'SEM FABRICANTE'
            elif opcao == 'V':
                codigo_item = movimento.get('vendedor_id') or ''
                item_name = movimento.get('nome') or 'SEM VENDEDOR'

            valor_agregado = movimento.get('valor_agregado')
            porcentagem_individual = (total_item / valor_total) * 100 if valor_total else 0
            porcentagem_valor_agregado_venda = (valor_agregado / total_item) * 100 if total_item \
                else 0

            if int(porcentagem_acumulada) >= porcentagem_listar:
                break
            porcentagem_acumulada += porcentagem_individual
            if int(porcentagem_acumulada) > porcentagem_listar:
                break
            total_agregado += valor_agregado

            linhas.append({'linha1': {
                'sequencia': i,
                'codigo': codigo_item,
                'grade': grade,
                'descricao': item_name,
                'saldo': estoque_saldo.get(movimento.get('grade_id')).get(
                    'saldo') if estoque_saldo.get(movimento.get('grade_id')) else 0,
                'quantidade': getNumeroEditado(quantidade, 3),
                'vl_total': getNumeroEditado(total_item),
                'vl_acumulado': getNumeroEditado(total_acumulado),
                'percentual_individual': getNumeroEditado(porcentagem_individual, 3),
                'percentual_acumulado': getNumeroEditado(porcentagem_acumulada, 3),
                'vl_agregado': getNumeroEditado(valor_agregado),
                'percentual_agregado_venda': getNumeroEditado(porcentagem_valor_agregado_venda, 3)
                }})
            i += 1

        if opcao == 'I':
            linhas.append({'linha_total': {
                'vl_acumulado': getNumeroEditado(total_acumulado),
                'vl_agregado': getNumeroEditado(total_agregado),
                'percentual_agregado_venda': getNumeroEditado(((total_agregado / total_acumulado)
                                                               * 100 if total_acumulado else 0))}})
        else:
            linhas.append({'linha_total': {
                'vl_agregado': getNumeroEditado(total_agregado)}})

        dados['linhas'] = linhas
        return pagina, dados

    def get_sql(self, cr, uid, company_ids, categoria_id, opcao, data_inicial, data_final,
                agrupar_sem_grade, context=None):
        """
        Retorna o sql para movimentacao de notas.
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param categoria_id: id da categoria a filtrar
        :param opcao: Opcao a listar ('I', 'Itens'), ('C', 'Categorias'), ('F', 'Fabricante'),
                      ('V', 'Vendedores')
        :param data_inicial: Data inicial para filtro
        :param data_final: Data final para filtro
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type company_ids: list
        :type categoria_id: int, long
        :type opcao: str
        :type data_inicial: str
        :type data_final: str
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        campos_agrupados = ''
        campos = ''
        categoria_ids = []
        context = context and context.copy() or {}
        if company_ids and categoria_id or categoria_id:
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)

        company_ids = ''' AND emc.company_id IN %s''' % str(company_ids).replace('[',
                                                                                 '(').replace(
            ']',
            ')') \
            if company_ids else ''' '''
        where_categoria = ''' '''
        join_categoria = ''' '''
        if categoria_ids:
            join_categoria = ''' INNER JOIN estoque_categoria      AS ec  ON ec.id = ep.categoria_id'''
            where_categoria = '''AND ec.id IN %s''' % str(categoria_ids).replace('[',
                                                                                 '(').replace(
                ']', ')')

        join = ''' '''
        if opcao == 'I':
            # Listar por Itens
            campos = '''
                           ep.codigo                      AS codigo,
                           ep.name                        AS produto_name '''
            if not agrupar_sem_grade:
                campos += ''', epg.grade                      AS grade
                             , epg.id                         AS grade_id'''
            join = '''INNER JOIN estoque_produto_grade  AS epg ON epg.id = emi.grade_id'''
            campos_agrupados = '''codigo, produto_name'''
            if not agrupar_sem_grade:
                campos_agrupados += ', grade, grade_id '''

        elif opcao == 'C':
            # Listar por Categoria
            campos = '''
                           ep.categoria_id                AS categoria_id,
                           (SELECT conta
                            FROM estoque_categoria
                            WHERE id = ep.categoria_id
                            LIMIT 1)                      AS conta,
                           (SELECT name
                            FROM estoque_categoria
                            WHERE id = ep.categoria_id
                            LIMIT 1)                      AS categoria_name,
                           'C'                            AS opcao '''

            campos_agrupados = '''categoria_id, conta, categoria_name'''
        elif opcao == 'F':
            # Listar por Fabricante
            campos = '''
                           ep.fabricante_id               AS fabricante_id,
                           (SELECT codigo
                            FROM adm_pessoa
                            WHERE id = ep.fabricante_id
                            LIMIT 1)                      AS fabricante_codigo,
                           (SELECT nome
                            FROM adm_pessoa
                            WHERE id = ep.fabricante_id
                            LIMIT 1)                      AS fabricante_name,
                           'F'                            AS opcao '''

            campos_agrupados = '''fabricante_id, fabricante_codigo, fabricante_name'''
        elif opcao == 'V':
            # Listar por  Vendedor
            campos = '''
                           adp.nome                       AS vendedor,
                           'V'                            AS opcao '''
            join = '''LEFT  JOIN vendas_vendedor        AS vv  ON vv.id = emi.vendedor_id
                          LEFT  JOIN adm_pessoa             AS adp ON adp.id = vv.pessoa_id'''

            campos_agrupados = '''vendedor '''
        sql = '''
                    select sum(vr_total_item_rateado) as vr_total_item_rateado,
                           sum(vr_total_item) as vr_total_item,
                           sum(quantidade) as quantidade,
                           sum(custo_reposicao) as custo_reposicao,
                           sum(ultimo_custo) as ultimo_custo,
                           sum(custo_corrigido) as custo_corrigido,
                           sum(custo_medio) as custo_medio,
                           %s
                    from (
                            SELECT emi.vr_total_item_rateado AS vr_total_item_rateado,
                                   emi.vr_total_item         AS vr_total_item,
                                   {quantidade}
                                   emi.estmvi_saidas_custo_reposicao_total       AS custo_reposicao,
                                   emi.estmvi_saidas_ultimo_custo_total          AS ultimo_custo,
                                   emi.estmvi_saidas_custo_corrigido_total       AS custo_corrigido,
                                   emi.estmvi_saidas_custo_medio_contabil_total  AS custo_medio,
                                   %s
                            FROM estoque_movimento_itens      AS emi
                            INNER JOIN estoque_movimento_capa AS emc ON emc.id = emi.capa_id
                            INNER JOIN fiscal_cfop            AS fc  ON fc.id  = emi.cfop_id
                            LEFT  JOIN estoque_produto        AS ep  ON ep.id  = emi.produto_id
                            %s
                            %s
                            WHERE emc.data >= '%s'
                              AND emc.data <= '%s'
                              AND fc.movimento_tipo = 'C'
                              AND fc.tipo = 'S'
                              AND emi.nf_id IS NOT NULL
                              AND emc.state = 'F'
                              %s
                              %s
                    ''' % (
            campos_agrupados, campos, join, join_categoria, data_inicial, data_final,
            company_ids,
            where_categoria)
        return sql

    @staticmethod
    def get_group_by(opcao, agrupar_sem_grade):
        """
        Retorna o sql para agrupar de acordo com a opcao do relatorio.
        :param opcao: Opcao a listar ('I', 'Itens'), ('C', 'Categorias'), ('F', 'Fabricante'),
                      ('V', 'Vendedores')
        :type opcao: str
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if opcao == 'I':
            group_by = 'GROUP BY  codigo, produto_name'
            if not agrupar_sem_grade:
                group_by += ', grade, grade_id'
        elif opcao == 'C':
            group_by = 'GROUP BY categoria_id, conta, categoria_name'
        elif opcao == 'F':
            group_by = 'GROUP BY fabricante_id, fabricante_codigo, fabricante_name'
        elif opcao == 'V':
            group_by = 'GROUP BY vendedor'
        else:
            group_by = ''
        return group_by

    @staticmethod
    def sql_quantidade():
        """
        Retorna a campo de quantidade para ser utilizado na query
        Este metodo e sobrescrevido no madeireira para utilizar o volume.
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return "emi.quantidade as quantidade,"

