# -*- coding: utf-8 -#-

from openerp.osv import osv
from openerp.tools.translate import _


class estoque_composicao_capa(osv.Model):
    _inherit = 'estoque.composicao.capa'

    def estcoc_button_efetivar(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Efetivar'. Para que a composicao seja efetivada, deve possuir itens na
        origem e destino, os totais dos custos serem iguais, os itens devem ser movimentados no
        estoque e atualizado os custos dos itens no destino.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso o processe de efetivacao seja realizado. Senao, mensagem ao
        usuario (raise) que informa o motivo na nao efetivacao.
        :rtype: bool
        """
        context = context and context.copy() or {}
        msg = ''
        obj_estoque_item_composicao_origem = self.pool.get('estoque.composicao.item.origem')
        obj_estoque_item_composicao_destino = self.pool.get('estoque.composicao.item.destino')
        for composicao in self.browse(cr, uid, ids, context):
            if not composicao.estcoc_item_origem_ids:
                msg += 'A composicao nao possui nenhum item na origem.\n'
            if not composicao.estcoc_item_destino_ids:
                msg += 'A composicao nao possui nenhum item no destino.\n'
            if filter(lambda i: i.estcio_quantidade <= 0, composicao.estcoc_item_origem_ids):
                msg += 'Itens da origem possuem quantidade menor ou igual a zero.\n'
            if filter(lambda i: i.estcid_quantidade <= 0, composicao.estcoc_item_destino_ids):
                msg += 'Itens do destino possuem quantidade menor ou igual a zero.\n'
            medio_contabil_origem = round(sum(map(lambda i: i.estcio_custo_medio_contabil,
                                                  composicao.estcoc_item_origem_ids)), 5)
            medio_contabil_destino = round(sum(map(lambda i: i.estcid_custo_medio_contabil,
                                                   composicao.estcoc_item_destino_ids)), 5)
            if medio_contabil_origem != medio_contabil_destino:
                msg += 'O total do custo medio contabil na origem deve ser igual ao destino.\n'
            corrigido_origem = round(sum(map(lambda i: i.estcio_custo_corrigido,
                                             composicao.estcoc_item_origem_ids)), 5)
            corrigido_destino = round(sum(map(lambda i: i.estcid_custo_corrigido,
                                              composicao.estcoc_item_destino_ids)), 5)
            if corrigido_origem != corrigido_destino:
                msg += 'O total do custo corrigido na origem deve ser igual ao destino.'
            if msg:
                raise osv.except_osv(_('Atencao'), msg)
            obj_estoque_item_composicao_origem.estcio_gera_movimento_itens_saida(
                cr, uid, map(lambda i: i.id, composicao.estcoc_item_origem_ids), context)
            item_destino_ids = map(lambda i: i.id, composicao.estcoc_item_destino_ids)
            obj_estoque_item_composicao_destino.estcid_gera_movimento_itens_entrada(
                cr, uid, item_destino_ids, context)
            obj_estoque_item_composicao_destino.estcid_atualiza_custos_entrada(
                cr, uid, item_destino_ids, context)
        return self.write(cr, uid, ids, {'state': 'F'}, context)

    def estcoc_button_cancelar(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Cancelar'. Atualiza as composicoes para o estado de 'Cancelado'.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso seja atualizado o estado. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        return self.write(cr, uid, ids, {'state': 'C'}, context)
