# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
import time


class estoque_composicao_item_destino(osv.Model):
    _inherit = 'estoque.composicao.item.destino'

    def estcid_gera_movimento_itens_entrada(self, cr, uid, ids, context=None):
        """
        O metodo gera a movimentacao dos itens do estoque do destino. Operacao de entrada.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        brw_composicao_item_destino = self.browse(cr, uid, ids, context)
        company_id = brw_composicao_item_destino[0].estcid_capa_id.company_id.id
        composicao_capa_id = brw_composicao_item_destino[0].estcid_capa_id.id
        obj_estoque_composicao_capa = self.pool.get('estoque.composicao.capa')
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')

        # Cria a capa do movimento do estoque
        funcionario_id = self.pool.get('rh.funcionario').get_id_func(cr, uid, context)
        if not funcionario_id:
            raise osv.except_osv(_('Atencao!'), _('Usuario nao esta cadastrado como funcionario.'))
        operacao_id = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_composicao_nat_ent', context=context).get(
            'estpar_composicao_nat_ent')
        if not operacao_id:
            raise osv.except_osv(_('Atencao'),
                                 _('"Operacao de entrada" nao cadastrada nos parametros do estoque.'
                                   ' Aba "Composicao".'))
        vals_movimento_capa = {
            'company_id': company_id,
            'funcionario_id': funcionario_id,
            'estmvc_operacao_id': operacao_id,
            'state': 'F'
        }
        context.update({'composicao_capa_id': composicao_capa_id, 'composicao_movimento_tipo': 'E'})
        movimento_capa_id = self.pool.get('estoque.movimento.capa').create(
            cr, uid, vals_movimento_capa, context)
        obj_estoque_composicao_capa.write(
            cr, uid, composicao_capa_id, {'estcoc_destino_movimento_capa_id': movimento_capa_id},
            context)

        # Cria o movimento de itens do estoque
        data = time.strftime('%Y-%m-%d')
        for item in brw_composicao_item_destino:
            brw_produto_embalagem = item.estcid_produto_embalagem_id
            grade_id = item.estcid_grade_id.id
            quantidade = item.estcid_quantidade * \
                (brw_produto_embalagem.quantidade_na_embalagem_get()[0].get(
                    'quantidade_na_embalagem') or 1)
            local_id = item.estcid_local_id.id
            vals_movimento_itens = {'capa_id': movimento_capa_id,
                                    'produto_id': item.estcid_produto_id.id,
                                    'grade_id': grade_id,
                                    'local_id': local_id,
                                    'produto_embalagem_id': brw_produto_embalagem.id,
                                    'quantidade': quantidade,
                                    'vr_total_item': item.estcid_custo_medio_contabil,
                                    'vr_total_item_rateado': item.estcid_custo_medio_contabil,
                                    'vr_unitario': item.estcid_custo_medio_contabil / quantidade
                                    }
            movimento_item_id = obj_estoque_movimento_itens.create(cr, uid, vals_movimento_itens,
                                                                   context)
            self.write(cr, uid, item.id, {'estcid_movimento_item_id': movimento_item_id}, context)

            # Atualiza o saldo do item movimentado
            obj_estoque_produto_saldo.atualizar_saldo_do_dia(
                cr, uid, eps_company_id=company_id, grade_qtd={grade_id: quantidade},
                eps_data=data, eps_entrada_saida='E', eps_local_id=local_id,
                context=context)
        return True

    def estcid_onchange_produto(self, cr, uid, ids, produto_id, company_id, context=None):
        context = context and context.copy() or {}
        res = {}
        vals = {'estcid_grade_id': False, 'estcid_produto_embalagem_id': False}
        if not produto_id or not company_id:
            res = {'value': vals}
            return res
        grade_ids = self.pool.get('estoque.produto.grade').search(
            cr, uid, [('produto_id', '=', produto_id)], context=context)
        if len(grade_ids) == 1:
            vals.update({'estcid_grade_id': grade_ids[0]})
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        produto_embalagem_ids = obj_estoque_produto_embalagem.search(
            cr, uid, [('produto_id', '=', produto_id)], context=context)
        if len(produto_embalagem_ids) == 1:
            vals.update({'estcid_produto_embalagem_id': produto_embalagem_ids[0]})
        if vals:
            res.update({'value': vals})
        return res

    def estcid_atualiza_custos_entrada(self, cr, uid, ids, context=None):
        """
        O metodo atualiza os custos dos itens que deram entrada no estoque.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_composicao_item_destino = self.browse(cr, uid, ids, context)
        obj_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque',
            ['estpar_composicao_alter_ult_custo', 'estpar_composicao_alter_c_repos'],
            context=context)
        altera_ultimo_custo = estoque_parametro_dados.get('estpar_composicao_alter_ult_custo')
        altera_custo_reposicao = estoque_parametro_dados.get('estpar_composicao_alter_c_repos')

        epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.preco.custo', context=context)
        produto_ids = []
        grade_ids = []
        for item in brw_composicao_item_destino:
            produto_ids.append(item.estcid_produto_id.id)
            grade_ids.append(item.estcid_grade_id.id)
        qtd_embalagens = self.pool.get('estoque.utils').get_grade_qtd_embalagem(cr, grade_ids,
                                                                                context)
        saldos = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
            cr, uid, grade_ids, time.strftime('%Y-%m-%d'), context=context)
        custos = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_ids, context=context)
        for item in brw_composicao_item_destino:
            produto_id = item.estcid_produto_id.id
            grade_id = item.estcid_grade_id.id
            movimento_item = item.estcid_movimento_item_id
            item_embalagem_id = item.estcid_produto_embalagem_id.id
            item_quantidade = item.estcid_quantidade

            preco_custo_ids = obj_estoque_preco_custo.search(
                cr, uid, [('company_id', '=', epc_company_id), ('produto_id', '=', produto_id),
                          ('estprc_data', '<=', time.strftime('%Y-%m-%d %H:%M:%S'))],
                order='estprc_data desc', limit=1, context=context)

            if not preco_custo_ids:
                preco_custo_dados = {}.fromkeys(
                    ['estprc_custo_corrigido', 'estprc_custo_medio_contabil', 'estprc_ultimo_custo',
                     'estprc_custo_reposicao'], 0)
            else:
                preco_custo_dados = obj_estoque_preco_custo.read(
                    cr, uid, preco_custo_ids[0],
                    ['estprc_custo_corrigido', 'estprc_custo_medio_contabil', 'estprc_ultimo_custo',
                     'estprc_custo_reposicao'], context=context)

            emb_qtd = 1
            if item_embalagem_id:
                chave = '%s-%s' % (grade_id, item_embalagem_id)
                emb_qtd = qtd_embalagens.get(chave) and qtd_embalagens.get(chave).get('qtd') or 1

            quantidade_total_item = emb_qtd * item_quantidade

            # Como neste ponto os itens da composicao já foram somados ao saldo, é necessário tirar
            # a quantidade da composicao do saldo dos produtos para não calcular os custos medios
            # errados
            saldo = saldos.get(grade_id).get('saldo') - quantidade_total_item
            if saldo < 0:
                saldo = 0

            item_destino_custo = custos.get(produto_id) or {}
            custo_medio_contabil = item_destino_custo.get('estprc_custo_medio_contabil') or 0
            custo_corrigido = item_destino_custo.get('estprc_custo_corrigido') or 0

            vr_ultimo_custo = movimento_item.vr_total_item_rateado / quantidade_total_item
            vr_medio_contabil = (
                    ((vr_ultimo_custo * item_quantidade) +
                     (custo_medio_contabil * saldo)) / (quantidade_total_item + saldo))
            vr_custo_reposicao = movimento_item.vr_total_item / quantidade_total_item
            vr_custo_corrigido = (
                    ((vr_ultimo_custo * item_quantidade) +
                     (custo_corrigido * saldo)) / (quantidade_total_item + saldo))
            custo_id = obj_estoque_preco_custo.create(
                cr, uid, {'company_id': epc_company_id,
                          'produto_id': produto_id,
                          'estprc_custo_corrigido': vr_custo_corrigido,
                          'estprc_custo_medio_contabil': vr_medio_contabil,
                          'estprc_ultimo_custo': vr_ultimo_custo if altera_ultimo_custo else
                          (preco_custo_dados.get('estprc_ultimo_custo') or 0.0),
                          'estprc_custo_reposicao': vr_custo_reposicao if altera_custo_reposicao
                          else (preco_custo_dados.get('estprc_custo_reposicao') or 0.0),
                          'estprc_data': time.strftime('%Y-%m-%d %H:%M:%S')
                          }, context)
            # Atualiza o ID do custo na tabela de movimentacao de itens
            obj_movimento_itens.write(cr, uid, movimento_item.id,
                                      {'estmvi_entrada_preco_custo_id': custo_id}, context)
        return True
