# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_estoque_conferencia_entrada(osv.TransientModel):
    # prefixo: westcon_
    _name = 'wizard.estoque.conferencia.entrada'

    _columns = {
        'westcon_local_id': fields.many2one('estoque.local', 'Local', required=True),
        'westcon_item_ids': fields.one2many('wizard.estoque.conferencia.entrada.item',
                                            'westcoi_capa_id', 'Itens'),
        'westcon_conferido': fields.boolean('Conferido'),
        'westcon_impresso': fields.boolean('Impresso'),
    }

    def westcon_button_busca_produto(self, cr, uid, ids, context=None):
        """
        O metodo busca os produtos, e demais dados, do movimento do estoque que nao foram
        conferidos. Operacao de entrada.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        cr.execute("""DELETE FROM wizard_estoque_conferencia_entrada_item WHERE create_uid = %s"""
                   % uid)
        context = context and context.copy() or {}
        local_id = self.read(cr, uid, isinstance(ids, list) and ids[0] or ids, ['westcon_local_id'],
                             context, '_classic_write').get('westcon_local_id')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        movimento_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'estoque.movimento.capa',
                                                                 context=context)
        locacao_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'estoque.local.locacao',
                                                               context=context)
        sql = """
            SELECT
                emi.id          AS movimento_id,
                emi.produto_id  AS produto_id,
                ep.codigo       AS codigo,
                emi.grade_id    AS grade_id,
                ep.name         AS descricao,
                epg.grade       AS grade,
                emi.quantidade  AS quantidade,
                ell.id          AS locacao_id,
                ell.locacao     AS locacao
            FROM
                estoque_movimento_itens emi
            INNER JOIN
                estoque_movimento_capa emc ON emc.id = emi.capa_id
            INNER JOIN
                estoque_produto ep ON ep.id = emi.produto_id
            INNER JOIN
                estoque_produto_grade epg ON epg.id = emi.grade_id
            INNER JOIN
                estoque_local_locacao ell ON emi.local_id = ell.local_id 
                    AND emi.produto_id = ell.produto_id AND emi.grade_id = ell.grade_id
                    AND emi.produto_embalagem_id = ell.estllc_produto_embalagem_id
            INNER JOIN 
                estoque_natureza_operacao eno ON eno.id = emc.estmvc_operacao_id
            WHERE
                emc.company_id = {movimento_company_id}
                AND emc.state = 'F'
                AND emi.local_id = {local_id}
                AND ell.company_id = {locacao_company_id}
                AND emi.estmvi_conferencia IS NOT TRUE
                AND eno.estnto_tipo_operacao = 'E'
                AND eno.estnto_atualizar_saldo IS TRUE
        """.format(movimento_company_id=movimento_company_id, local_id=local_id,
                   locacao_company_id=locacao_company_id, )
        cr.execute(sql)
        dados = cr.dictfetchall()
        dados_agrupados = {}
        for dado in dados:
            movimento_id = dado.get('movimento_id')
            locacao_id = dado.get('locacao_id')
            locacao = dado.get('locacao') or ''
            produto_id = dado.get('produto_id')
            grade_id = dado.get('grade_id')
            codigo = dado.get('codigo')
            descricao = dado.get('descricao')
            grade = dado.get('grade')
            quantidade = dado.get('quantidade')
            locacao_produto_grade = str(locacao_id) + str(produto_id) + str(grade_id)
            if locacao_produto_grade not in dados_agrupados:
                dados_agrupados[locacao_produto_grade] = {
                    'westcoi_locacao': locacao, 'westcoi_produto_codigo': codigo,
                    'westcoi_grade_descricao': descricao, 'westcoi_grade': grade,
                    'westcoi_quantidade': quantidade, 'westcoi_movimento': [movimento_id]}
            else:
                dados_agrupados[locacao_produto_grade]['westcoi_quantidade'] += quantidade
                dados_agrupados[locacao_produto_grade]['westcoi_movimento'].append(movimento_id)
        item_ids = []
        map(lambda i: item_ids.append((0, 0, i)), dados_agrupados.values())
        return self.write(cr, uid, ids, {
            'westcon_item_ids': item_ids,
            'westcon_conferido': False,
            'westcon_impresso': False
        }, context)

    def westcon_button_confere(self, cr, uid, ids, context=None):
        """
        O metodo reune todos os ids de movimento envolvidos no itens encontrados e atualiza os
        registros para conferido.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_estoque_conferencia_entrada = self.browse(
            cr, uid, isinstance(ids, list) and ids[0] or ids, context)
        movimento_ids = []
        if not brw_estoque_conferencia_entrada.westcon_impresso:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao posso marcar como conferido antes de imprimir!'))
        for item in brw_estoque_conferencia_entrada.westcon_item_ids:
            movimento_ids += eval(item.westcoi_movimento)
        # sql = """
        #     UPDATE
        #         estoque_movimento_itens
        #     SET
        #         estmvi_conferencia = TRUE
        #     WHERE
        #         id IN ({movimento_ids})
        # """.format(movimento_ids=', '.join(map(lambda i: str(i), movimento_ids)))
        # cr.execute(sql)
        # mov_ids = map(lambda i: str(i), movimento_ids)
        self.pool.get('estoque.movimento.itens').write(cr, uid, movimento_ids, {'estmvi_conferencia': True}, context=context)
        return self.write(cr, uid, ids, {'westcon_conferido': True}, context)

    def westcon_button_print(self, cr, uid, ids, context=None):
        """
        Metodo para impressao dos produtos encontrados
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Dados de acao para a classe do relatorio
        :rtype: dict
        """
        context = context and context.copy() or {}
        estoque_conferencia_entrada_dados = self.read(
            cr, uid, isinstance(ids, list) and ids[0] or ids, [], context, '_classic_write')
        estoque_local_dados = self.pool.get('estoque.local').read(
            cr, uid, estoque_conferencia_entrada_dados.get('westcon_local_id'), ['numero', 'name'],
            context)
        itens = self.pool.get('wizard.estoque.conferencia.entrada.item').read(
            cr, uid, estoque_conferencia_entrada_dados.get('westcon_item_ids'), [], context)
        self.write(cr, uid, ids, {'westcon_impresso': True}, context)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'estoque_conferencia_entrada',
            'name': 'estoque_conferencia_entrada',
            'datas': {'dados': itens, 'local': estoque_local_dados}
        }


class wizard_estoque_conferencia_entrada_item(osv.TransientModel):
    # prefixo: westcoi_
    _name = 'wizard.estoque.conferencia.entrada.item'

    _columns = {
        'westcoi_capa_id': fields.many2one('wizard.estoque.conferencia.entrada', 'Capa'),
        'westcoi_locacao': fields.char('Locacao', size=15),
        'westcoi_produto_codigo': fields.char('Codigo', size=20),
        'westcoi_grade_descricao': fields.char('Descricao', size=40),
        'westcoi_grade': fields.char('Grade', size=2),
        'westcoi_quantidade': fields.float('Quantidade', digits=(12, 3)),
        'westcoi_movimento': fields.text('Ids dos movimentos'),
    }

    _order = 'westcoi_locacao'
