# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.osv.fields import float_round


class fiscal_nf_capa(osv.Model):

    _inherit = 'fiscal.nf.capa'

    def button_verificar_nf(self, cr, uid, nf, context=None):
        """
        Verifica e valida uma nota fiscal. Sobrescricao para tratar a decomposicao de um produto
        na entrada da Nota Fiscal.
        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota foi verificada e validada.
        :rtype: bool
        """
        context = context and context.copy() or {}
        nf = nf[0] if isinstance(nf, list) else nf
        obj_estoque_decomposicao_capa = self.pool.get('estoque.decomposicao.capa')
        brw_nf = self.browse(cr, uid, nf, context)
        # Funcionalidade exclusiva para Nota Fiscal de terceiros.
        if brw_nf.nf_propria is True:
            return super(fiscal_nf_capa, self).button_verificar_nf(cr, uid, nf, context)
        msg = ''
        movimento_itens = []
        for item in brw_nf.estoque_movimento_itens_ids:
            list_movimento_itens = []
            produto_id = item.produto_id.id
            grade_id = item.grade_id.id
            if not produto_id or not grade_id:
                continue
            decomposicao_capa_ids = obj_estoque_decomposicao_capa.search(
                cr, uid, [('estdcc_produto_id', '=', produto_id),
                          ('estdcc_grade_id', '=', grade_id)], context=context)
            if not decomposicao_capa_ids:
                continue
            if obj_estoque_decomposicao_capa.estdcc_get_porcentagem_participacao(
                    cr, uid, decomposicao_capa_ids[0], context) != 100.:
                msg += '%s - %s\n' % (item.produto_id.codigo, item.produto_id.name)
                continue
            quantidade_total_partes = valor_total_partes = valor_total_rateado_partes = 0
            vr_fcp_st_total = vr_icms_substituicao_total = vr_base_calculo_total = 0
            list_partes = sorted(obj_estoque_decomposicao_capa.browse(
                    cr, uid, decomposicao_capa_ids[0], context).estdcc_itens_ids,
                                 key=lambda p: p.estdep_percentual, reverse=True)
            for parte in list_partes:
                # Define o percentual de participacao
                participacao = lambda v, p: float_round(v * parte.estdep_percentual / 100., p)

                list_tributacoes = []
                qtde_na_embalagem = parte.estdep_produto_embalagem_id.\
                    quantidade_na_embalagem_get()
                qtde_na_embalagem = qtde_na_embalagem[0].get('quantidade_na_embalagem') or 1
                quantidade = participacao(item.quantidade / qtde_na_embalagem, 3)
                quantidade_total_partes += quantidade
                vr_total_item = participacao(item.vr_total_item, 2)
                valor_total_partes += vr_total_item
                vr_total_item_rateado = participacao(item.vr_total_item_rateado, 2)
                valor_total_rateado_partes += vr_total_item_rateado
                vals_movimento_itens = {
                    'capa_id': item.capa_id.id,
                    'local_id': item.local_id.id, 'produto_id': parte.estdep_produto_id.id,
                    'grade_id': parte.estdep_grade_id.id,
                    'produto_embalagem_id': parte.estdep_produto_embalagem_id.id,
                    'quantidade': quantidade,
                    'vr_unitario': float_round(vr_total_item / quantidade, 6),
                    'vr_desconto': participacao(item.vr_desconto, 2),
                    'vr_despesas_rateado': participacao(item.vr_despesas_rateado, 2),
                    'vr_total_item': vr_total_item, 'cfop_id': item.cfop_id.id,
                    'vr_desconto_rateado': participacao(item.vr_desconto_rateado, 2),
                    'vr_frete_rateado': participacao(item.vr_frete_rateado, 2),
                    'vr_seguro_rateado': participacao(item.vr_seguro_rateado, 2),
                    'vr_total_item_rateado': vr_total_item_rateado
                }
                vr_fcp_st_total = vr_icms_substituicao_total = vr_base_calculo_total = 0
                for tributacao in item.tributacoes_ids:
                    vr_fcp_st_total += tributacao.fisnit_vr_fcp_st or 0
                    vr_icms_substituicao_total += tributacao.vr_icms_substituicao
                    vr_base_calculo_total = tributacao.vr_base_calculo
                    vals_tributacoes = {
                        'encargo_id': tributacao.encargo_id.id,
                        'cst_id': tributacao.cst_id.id,
                        'tributacao_id': tributacao.tributacao_id.id,
                        'aliquota': tributacao.aliquota,
                        'vr_base_calculo': participacao(tributacao.vr_base_calculo, 2),
                        'vr_encargo': participacao(tributacao.vr_encargo, 2),
                        'vr_reducao_base': participacao(tributacao.vr_reducao_base, 2),
                        'fisnit_vr_difal_uf_origem': participacao(
                            tributacao.fisnit_vr_difal_uf_origem, 2),
                        'fisnit_vr_difal_uf_destino': participacao(
                            tributacao.fisnit_vr_difal_uf_origem, 2),
                        'vr_icms_substituicao': participacao(tributacao.vr_icms_substituicao, 2),
                        'fisnit_vr_fcp_st': participacao(tributacao.fisnit_vr_fcp_st, 2),
                    }
                    list_tributacoes.append([0, False, vals_tributacoes])
                vals_movimento_itens['tributacoes_ids'] = list_tributacoes
                vals_movimento_itens['sequencia'] = \
                    list_partes.index(parte) + 1
                list_movimento_itens.append([0, False, vals_movimento_itens])
            if list_partes:  # Ajusta as diferencas
                # Quantidade
                diferenca = float_round(item.quantidade - quantidade_total_partes, 3)
                list_movimento_itens[0][2]['quantidade'] += diferenca

                # Valor total do item
                diferenca = float_round(item.vr_total_item - valor_total_partes, 2)
                list_movimento_itens[0][2]['vr_total_item'] += diferenca

                # Valor total do item rateado
                diferenca = float_round(item.vr_total_item_rateado - valor_total_rateado_partes, 2)
                list_movimento_itens[0][2]['vr_total_item_rateado'] += diferenca

                tributacoes_validas = filter(lambda f: f[0] != 2, map(lambda m: m,
                                                                      list_movimento_itens))

                # Valor do FCP/ST
                diferenca = float_round(vr_fcp_st_total - sum(map(
                    lambda p: p[0][2]['fisnit_vr_fcp_st'], map(lambda n: n[2]['tributacoes_ids'],
                                                               tributacoes_validas))), 2)
                list_movimento_itens[0][2]['tributacoes_ids'][0][2]['fisnit_vr_fcp_st'] += diferenca

                # Valor do ICMS/ST
                diferenca = float_round(vr_icms_substituicao_total - sum(map(
                    lambda p: p[0][2]['vr_icms_substituicao'], map(
                        lambda n: n[2]['tributacoes_ids'], tributacoes_validas))), 2)
                list_movimento_itens[0][2]['tributacoes_ids'][0][2][
                    'vr_icms_substituicao'] += diferenca

                # Valor da base de calculo
                diferenca = float_round(vr_base_calculo_total - sum(map(
                    lambda p: p[0][2]['vr_base_calculo'], map(
                        lambda n: n[2]['tributacoes_ids'], tributacoes_validas))), 2)
                map(lambda t: t[2].update(dict(
                    vr_base_calculo=t[2]['vr_base_calculo'] + diferenca)),
                    list_movimento_itens[0][2]['tributacoes_ids'])
            list_movimento_itens.append([2, item.id, False])
            movimento_itens += list_movimento_itens
        if msg:
            raise osv.except_osv(
                _('Atencao'),
                _('Os itens a seguir sao de decomposicao e estao com percentual de participacao '
                  'diferente de 100.00:\n') + msg)
        context.update({'super_write_fiscal_nf_capa': True})
        self.write(cr, uid, nf, {'estoque_movimento_itens_ids': movimento_itens}, context)
        del context['super_write_fiscal_nf_capa']
        return super(fiscal_nf_capa, self).button_verificar_nf(cr, uid, nf, context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
