# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class estoque_fiscal_capa(osv.Model):
    # prefixo: estfca_

    _inherit = 'estoque.fiscal.capa'

    def estfca_get_status(self, cr, uid, ids, context=None):
        """
        O metodo define o state dos resultados das importacoes baseado no state dos seus itens.
        Se todos itens estiverem em esboco 'E', a capa esta em esboco.
        Se todos estiverem atualizados 'A', a capa esta atualizada.
        Se todos estiverem nao atualizar 'N', a capa esta em esboco.
        Se possuir algum atualizado 'A', a capa esta atualizada parcialmente 'P'.
        Se estiverem divididos entre esboco e nao atualizar, a capa esta em esboco.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: dados com a identificacao da importacao e seu respectivo status.
        :rtype: dict
        """
        context = context and context.copy() or {}
        for capa in self.browse(cr, uid, ids, context):
            states = list(set(map(lambda c: c.state, capa.estfca_item_ids)))
            if len(states) == 1 and (states[0] == 'E' or states[0] == 'N'):
                state = 'E'
            elif len(states) == 1 and states[0] == 'A':
                state = 'A'
            elif 'A' in states:
                state = 'P'
            else:
                state = 'E'
            self.write(cr, uid, capa.id, {'state': state}, context)
        return True

    def estfca_button_atualizar(self, cr, uid, ids, context=None):
        """
        Metodo responsavel por realizar o processo de atualizacao de todos os itens.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso todos os itens tenham sido atualizados. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False
        obj_estoque_fiscal_itens = self.pool.get('estoque.fiscal.itens')
        for capa in self.browse(cr, uid, ids, context):
            res = obj_estoque_fiscal_itens.estfit_button_atualizar(
                cr, uid, map(lambda i: i.id, capa.estfca_item_ids), context)
        self.estfca_get_status(cr, uid, ids, context)
        return res
