# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from time import strftime
from openerp.tools.translate import _
from datetime import datetime
from openerp.report import report_sxw
from ...adm import messagebox
import base64


class almoxarifado_requisicao_capa(osv.Model):
    _name = 'almoxarifado.requisicao.capa'
    _inherit = 'almoxarifado.requisicao.capa'

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __code = 'numero.requisicao'
        __name = 'Sequencia da Requisicao'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 6,
            'company_id': company_id,
            'prefix': 'RAL-%(year)s-',
            'implementation': 'no_gap',
        }
        # Gerar sequence para numero do departamento para a empresa
        consulta = self.pool.get('ir.sequence.type').search(cr, uid, [('code', '=', __code)],
                                                            context=context)
        if not consulta:
            self.pool.get('ir.sequence.type').create(cr, uid, {'name': __name, 'code': __code},
                                                     context)
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def check_itens_requisicao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        requisicao_itens = self.read(cr, uid, ids, ['requisicao_item_ids'], context=context)
        if isinstance(requisicao_itens, list):
            requisicao_itens = requisicao_itens[0]
        if isinstance(requisicao_itens, dict):
            if not requisicao_itens.get('requisicao_item_ids'):
                raise osv.except_osv(_('Atencao'), _('Nenhum produto encontra-se cadastrado para a requisicao!'))
            else:
                return True
        return False

    def _verifica_saldo_negativo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        requisicao_dados = self.browse(cr, uid, ids)
        aceita_saldo_negativo = requisicao_dados.local_id.aceita_saldo_negativo
        local_id = requisicao_dados.local_id.id
        if not aceita_saldo_negativo:
            requisicao_itens = requisicao_dados.requisicao_item_ids
            obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
            for i in requisicao_itens:
                produto_id = i.produto_id.id
                grade_id = i.grade_id.id
                saldo = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, grade_id,
                                           datetime.today(), eps_local_id=local_id, context=context)
                _saldo_disponivel = saldo.get(grade_id).get('disponivel')
                if _saldo_disponivel <= 0 or _saldo_disponivel < i.quantidade_requisitada:
                    raise osv.except_osv(_('Atencao'), _('Produto %s possui saldo insuficiente') % i.produto_id.name)

    def changeStatus(self, cr, uid, state, context=None):
        context = context and context.copy() or {}
        try:
            log = (self.pool.get('res.users').read(cr, uid, uid, ['name'], context=context)['name'])
            log += " - " + (strftime('%d-%m-%Y %H:%M:%S'))
            log += " - State: " + state
            return log
        except:
            raise osv.except_osv(_('Atencao'), _('Erro: Nao foi encontrado o nome do Usuario para a criacao do log.'))

    def button_esboco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {'state': 'R'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), self.changeStatus(cr, uid, _('Esboco')))
        return res

    def button_requerer(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        itens = self.check_itens_requisicao(cr, uid, ids, context)
        if itens:
            devolucao = self.browse(cr, uid, ids[0]).almrqc_devolucao
            if not devolucao:
                self._verifica_saldo_negativo(cr, uid, ids)
            self.write(cr, uid, ids, {'state': 'D'}, context=context)
            self.message_post(cr, uid, ids[0], _('Alteracao de Status'), self.changeStatus(cr, uid, _('Requerido')))
        return True

    def button_aprovar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'A'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), self.changeStatus(cr, uid, _('Aprovado')))
        return True

    def button_separar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'S'}, context=context)
        self.message_post(cr, uid, ids[0], _('Alteracao de Status'), self.changeStatus(cr, uid, _('Em separacao')))
        return True

    def button_entregar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        ids_1 = None
        self.write(cr, uid, ids, {'state': 'E'}, context=context.update({'requisicao_entrege': True}))
        if isinstance(ids, list) and ids:
            ids_1 = ids[0]
        self.message_post(cr, uid, ids_1, _('Alteracao de Status'), self.changeStatus(cr, uid, _('Entregue')))
        if not isinstance(ids, list):
            ids = [ids]
        almrqc_devolucao = False
        for x in  self.read(cr, uid, ids,['data_emissao', 'centrocusto_id', 'company_id', 'numero',
                                        'requisicao_item_ids', 'almrqc_devolucao'],context=context):
            data = x.get('data_emissao')
            ano_competencia = data[0:4]
            centro_custo_id = x.get('centrocusto_id')
            if isinstance(centro_custo_id, tuple):
                centro_custo_id = centro_custo_id[0]
            company_id = x.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            historico = _("Requisicao de almoxarifado %s") % x.get('numero')
            mes_competencia = data[5:7]
            requisicao_item_ids = x.get('requisicao_item_ids')
            almrqc_devolucao = x.get('almrqc_devolucao')
        if centro_custo_id:
            produto_evento_ids = []
            for x in self.pool.get('estoquealmoxarifado.requisicao.item').read(cr, uid,
                                        requisicao_item_ids,['produto_id', 'evento_id', 'grade_id',
                                 'quantidade_requisitada'], context=context, load='_classic_write'):
                produto_evento_ids.append((x.get('produto_id'),
                                           x.get('evento_id'),
                                           x.get('grade_id'),
                                           x.get('quantidade_requisitada')))
            parametro_ids = self.pool.get('estoque.parametro').search(cr, uid, [('company_id', '=', company_id)],
                                                                      context=context)
            if parametro_ids:
                estoque_parametro_dados = self.pool.get('estoque.parametro').read(cr, uid, parametro_ids,
                                                                                  ['custo_a_utilizar'], context=context)
                for x in estoque_parametro_dados:
                    custo_a_utilizar = x.get('custo_a_utilizar')
            tipo_custo = ''
            if custo_a_utilizar == '1':
                tipo_custo = 'estprc_ultimo_custo'
            elif custo_a_utilizar == '2':
                tipo_custo = 'estprc_custo_reposicao'
            elif custo_a_utilizar == '3':
                tipo_custo = 'estprc_custo_medio_contabil'
            elif custo_a_utilizar == '4':
                tipo_custo = 'preco_venda'
            obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
            obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
            for x in produto_evento_ids:
                if tipo_custo == 'preco_venda':
                    dict_valor = obj_estoque_preco_venda.get_preco_venda(
                        cr, uid, x[0], epc_grade_ids=x[2], epc_company_id=company_id)
                    dados_preco_venda = dict_valor.get(x[0]) or {}
                    valor = dados_preco_venda.get(x[2]) or dados_preco_venda.get(
                        'preco_venda')
                else:
                    dict_valor = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, x[0], company_id, data)
                    if dict_valor.get(x[0]):
                        dict_valor = dict_valor.get(x[0])
                    else:
                        dict_valor = {}
                    valor = dict_valor.get(tipo_custo) or 0.0
                if almrqc_devolucao:
                    valor *= -1
                vals = {
                    'ano_competencia': ano_competencia,
                    'centro_custo_id': centro_custo_id,
                    'company_id': company_id,
                    'fingmo_data': data[:10],
                    'evento_id': x[1],
                    'fingmo_historico': historico,
                    'mes_competencia': mes_competencia,
                    'produto_id': x[0],
                    'valor': valor * x[3]
                }
                if vals.get('evento_id'):
                    self.pool.get('gerencial.movimento').create(cr, uid, vals, context)
        return True

    def button_cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for requisicao_dados in  self.browse(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state': 'C'}, context=context)
            mov_capa_ids = self.pool.get('estoque.movimento.capa').search(cr, uid, [('requisicao_id', '=', ids)],
                                                                          context=context)
            self.pool.get('estoque.movimento.capa').write(cr, uid, mov_capa_ids, {'state': 'C'}, context=context)
            if isinstance(ids, list) and ids:
                ids = ids[0]
            self.message_post(cr, uid, ids, _('Alteracao de Status'), self.changeStatus(cr, uid, _('Cancelada')))
            # Se for de devolucao, deixa a requisicao original visivel novamente para devolucao
            fields_devolucao = ['almrqc_requisicao_origem_id', 'almrqc_devolucao']
            requisicao_devolvida = self.read(cr, uid, ids, fields_devolucao, context=context)
            if all(requisicao_devolvida.values()):
                almrqc_requisicao_origem_id = requisicao_devolvida['almrqc_requisicao_origem_id'][0]
                vals = {'almrqc_flag_exibir_requisicao': True}
                self.write(cr, uid, [almrqc_requisicao_origem_id], vals, context)
        return True

    def button_imprimir(self, cr, uid, ids, context=None):
        """
            Imprimi requisicao do almoxarifado
        """
        context = context and context.copy() or {}
        hoje = datetime.now()
        documento = 'report.requisicao.' + str(hoje)
        obj_ir_act_report_xml = self.pool.get('ir.actions.report.xml')
        report_id = obj_ir_act_report_xml.search(cr, uid,
                                                 [('report_name', '=', 'almoxarifado.requisicao.capa')],
                                                 context=context)
        report_xml = obj_ir_act_report_xml.browse(cr, uid, report_id[0])
        report_xml._data.get(report_id[0])['name'] = _('Requisicao')
        report_pdf = report_sxw.report_sxw(documento, 'almoxarifado.requisicao.capa',
                                           'estoquealmoxarifado/report/rel_requisicao_almoxarifado.rml').create_single_pdf(
            cr, uid, ids,
            {'report_type': 'pdf'},
            report_xml)
        obj_messagebox = self.pool.get('adm.messagebox')
        # criando arquivo em messagebox para efetuar download
        id_messagebox = obj_messagebox.create(cr, uid,
                                              {'dados': base64.encodestring(report_pdf[0]),
                                               'filename': 'Impressao Requisicao Almoxarifado.pdf'}, context)
        context.update({'id_messagebox': id_messagebox, 'exibir_download': True, 'mensagem': 'Requisicao'})

        return messagebox('teste', context=context)

    def onchange_centro_custo(self, cr, uid, ids, funcionario_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if funcionario_id:
            if not isinstance(funcionario_id, list):
                funcionario_id = [funcionario_id]
            rh_funcionario_dados = self.pool.get('rh.funcionario').read(cr, uid, funcionario_id, ['centro_custo'],
                                                                        context=context)
            for x in rh_funcionario_dados:
                centro_custo_id = x.get('centro_custo')
            if isinstance(centro_custo_id, tuple):
                centro_custo_id = centro_custo_id[0]
            res = {'value': {'centrocusto_id': centro_custo_id}}
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = {}
        if 'observacao' in vals:
            observacao = vals.get('observacao')
            contador = 0
            if observacao and len(observacao) > 130:
                for x in range(len(observacao)):
                    if observacao[x] == ' ':
                        contador = 0
                    if contador == 130:
                        observacao = observacao[:x] + '- ' + observacao[x:]
                        contador = 0
                    contador += 1
            vals['observacao'] = observacao
        if 'requisicao_item_ids' in vals:
            requisicoes = vals['requisicao_item_ids']
            for requisicao_item in requisicoes:
                if requisicao_item[2]['quantidade_requisitada'] == 0:
                    raise osv.except_osv(_('Atencao'), _('Nao pode ter item com quantidade requisitada nula.'))
        vals['numero'] = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, 'numero.requisicao')
        res = super(almoxarifado_requisicao_capa, self).create(cr, uid, vals, context)
        self.pool.get('adm.transacao.capa').admtrc_criar_transacao(cr, uid, 'almx', res, context=None)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = {}
        if 'observacao' in vals:
            observacao = vals.get('observacao')
            contador = 0
            if len(observacao) > 130:
                for x in range(len(observacao)):
                    if observacao[x] == ' ':
                        contador = 0
                    if contador == 130:
                        observacao = observacao[:x] + '- ' + observacao[x:]
                        contador = 0
                    contador += 1
            vals['observacao'] = observacao
        if vals.get('requisicao_item_ids'):
            requisicoes = vals['requisicao_item_ids']
            for requisicao_item in requisicoes:
                if isinstance(requisicao_item[2], dict) and requisicao_item[2]:
                    if 'quantidade_requisitada' in requisicao_item[2]:
                        if requisicao_item[2]['quantidade_requisitada'] == 0:
                            raise osv.except_osv(_('Atencao'), _('Nao pode ter item com quantidade requisitada nula.'))
                        else:
                            if requisicao_item[2].get('produto_embalagem_id', False):
                                produto_embalagem_id = requisicao_item[2].get('produto_embalagem_id')
                                embalagem = self.pool.get('estoque.produto.embalagem').read(cr, uid,
                                                                                            [produto_embalagem_id],
                                                                                            ['embalagem_id'],
                                                                                            context=context)
                                if isinstance(embalagem, list):
                                    embalagem = embalagem[0]
                                embalagem_id = embalagem.get('embalagem_id')
                                if isinstance(embalagem_id, tuple):
                                    embalagem_id = embalagem_id[0]
                                quantidade_na_embalagem = self.pool.get('estoque.embalagem').read(cr, uid,
                                                                                                  [embalagem_id], [
                                                                                                      'quantidade_na_embalagem'],
                                                                                                  context=context)
                                if isinstance(quantidade_na_embalagem, list):
                                    quantidade_na_embalagem = quantidade_na_embalagem[0]
                                quantidade_na_embalagem = quantidade_na_embalagem.get('quantidade_na_embalagem')
                                quantidade_requisitada = requisicao_item[2].get(
                                    'quantidade_requisitada') * quantidade_na_embalagem
                                requisicao_item[2]['quantidade_requisitada'] = quantidade_requisitada
        res = super(almoxarifado_requisicao_capa, self).write(cr, uid, ids, vals, context=context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'), _('Nao e permitido excluir requisicao.'))

    def copy(self, cr, uid, id, default={}, context=None):
        context = context and context.copy() or {}
        if isinstance(default, dict):
            default.update({'recebeu_id': False, 'data_hora_recebimento': None})
        return super(almoxarifado_requisicao_capa, self).copy(cr, uid, id, default, context)

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        obj_financeiro_parametros = self.pool.get('financeiro.parametros')
        res = super(almoxarifado_requisicao_capa, self).read(cr, uid, ids, fields=fields, context=context, load=load)
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        financeiro_parametros_ids = obj_financeiro_parametros.search(cr, uid, [('company_id', '=', company_id)],
                                                                     context=context)
        if financeiro_parametros_ids:
            financeiro_parametros_dados = obj_financeiro_parametros.read(cr, uid, financeiro_parametros_ids,
                                                                         ['usar_centro_custos_e_eventos'],
                                                                         context=context)
            for x in financeiro_parametros_dados:
                usar_centro_custos_e_eventos = x.get('usar_centro_custos_e_eventos')
            if usar_centro_custos_e_eventos:
                if isinstance(res, dict):
                    res['flag'] = True
                else:
                    for i in res:
                        if 'flag' in i:
                            i['flag'] = True
        return res

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        if not args:
            args = []
        context = context and context.copy() or {}
        # verifica requisições que já  foram devolvidas para que nao sejam exibidas na tela de devolucao
        if context.get('requisicao_devolucao'):
            args1 = [('almrqc_requisicao_origem_id', '!=', False), ('state', '!=', 'C')]
            requisicao_ids = self.search(cr, uid, args1, limit=limit, context=context)
            if requisicao_ids:
                brw_almoxarifado_requisicao_capa = self.browse(cr, uid, requisicao_ids)
                requisicoes_devolvidas_ids = [x.almrqc_requisicao_origem_id.id for x in
                                              brw_almoxarifado_requisicao_capa]
                args.extend([['id', 'not in', requisicoes_devolvidas_ids]])
        if name:
            ids = self.search(cr, uid, [('numero', operator, name)] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(almoxarifado_requisicao_capa, self).default_get(cr, uid, fields, context=context)
        obj_financeiro_parametros = self.pool.get('financeiro.parametros')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        financeiro_parametros_ids = obj_financeiro_parametros.search(cr, uid, [('company_id', '=', company_id)],
                                                                     context=context)
        if financeiro_parametros_ids:
            financeiro_parametros_dados = obj_financeiro_parametros.read(cr, uid, financeiro_parametros_ids,
                                                                         ['usar_centro_custos_e_eventos'],
                                                                         context=context)
            for x in financeiro_parametros_dados:
                usar_centro_custos_e_eventos = x.get('usar_centro_custos_e_eventos')
            if usar_centro_custos_e_eventos:
                res['flag'] = True
        return res

    def botao_atualizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True
