# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class wizard_estoque_consulta_saldos(osv.TransientModel):
    _inherit = 'wizard.consulta.saldos'

    def button_limpar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = super(wizard_estoque_consulta_saldos, self).button_limpar(cr, uid, ids, context)
        obj_wiz_saldo_reserva_venda = self.pool.get('wizard.consulta.reserva.venda')
        reserva_venda_dados = self.read(cr, uid, ids, ['reserva_venda_ids'], context=context)
        for reserva in reserva_venda_dados:
            obj_wiz_saldo_reserva_venda.unlink(cr, uid, reserva.get('reserva_venda_ids'),
                                               context=context)

        return res

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]

        res = super(wizard_estoque_consulta_saldos, self).button_pesquisar(cr, uid, ids, context)

        obj_wiz_saldo_reserva_venda = self.pool.get('wizard.consulta.reserva.venda')

        consulta_saldo_dados = self.read(cr, uid, ids, ['company_id', 'produto_id'],
                                         context=context)
        if not consulta_saldo_dados:
            return res

        produto_id = consulta_saldo_dados.get('produto_id')[0]
        company_id = consulta_saldo_dados.get('company_id')[0]

        reservas_almoxariado = self.get_reservas_almoxarifado(cr, uid, produto_id, company_id,
                                                              context=context)

        for reserva in reservas_almoxariado:
            reserva['wcsrv_consulta_saldo_id'] = ids
            obj_wiz_saldo_reserva_venda.create(cr, uid, reserva, context=context)

        return res

    def get_reservas_almoxarifado(self, cr, uid, produto_id, company_id, context=None):
        """
        Método que retorna reservas de venda pendentes com o produto informado no parâmetro
        'produto_id'.

        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param produto_id: id do produto
        :param company_id: id da empresa
        :param context: dicionario padrao para valores contextuais

        :type cr: object
        :type uid: int
        :type produto_id: int
        :type company_id: int
        :type context: dict
        :rtype: list

        :return: lista de requisicoes de almoxarifado que estão pendentes, com o produto
                 informado no parâmetro 'produto_id'.
        """
        context = context and context.copy() or {}

        sql = """
            SELECT
                rc.id 				            AS wcsrv_requisicao_almoxarifado_id,
                rc.numero			            AS wcsrv_documento,
                rc.data_emissao 	            AS wcsrv_data_emissao,
                sum(ri.quantidade_requisitada)	AS wcsrv_quantidade
            FROM almoxarifado_requisicao_capa rc
            INNER JOIN estoquealmoxarifado_requisicao_item ri
                ON rc.id = ri.requisicao_id
            WHERE 
                rc.state IN ('D', 'A', 'S')
                AND rc.company_id = %s
                AND ri.produto_id = %s
            GROUP BY rc.id
        """ % (company_id, produto_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados
