# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from time import strftime


class almoxarifado_requisicao_capa(osv.Model):
    """
    Tabela de requisicao de itens ao almoxarifado(capa)
    """

    _name = 'almoxarifado.requisicao.capa'
    _rec_name = 'numero'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
        'local_id': fields.many2one('estoque.local', 'Local', ondelete="restrict", required=True,
                                    domain="[('company_id', '=', company_id)]"), # Id do local requisitado
        'company_id': fields.many2one('res.company', 'Empresa', ondelete="cascade", required=True),
        'numero': fields.char('Numero', size=20, required=False), # Numero da requisicao. Usar formato:\nRAL-AAAA-999999
        'data_emissao': fields.datetime('Emissao'),
        'state': fields.selection([
                                      ('R', 'Esboco'),
                                      ('D', 'Requerido'),
                                      ('A', 'Aprovado'),
                                      ('S', 'Em Separacao'),
                                      ('E', 'Entregue'),
                                      ('C', 'Cancelada')],
                                  string='Estado', required=True, size=1, select=1),
        # Estado da requisicao:\nA=Rascunho\nE=Entregue\nC=Cancelada
        'motivo_cancelamento': fields.char('Motivo cancelamento', size=70,
                                           help="Descricao do motivo para o cancelamento da requisicao."),
        'observacao': fields.text('Observacao'),
        'funcionario_id': fields.many2one('rh.funcionario', 'Requisitante', ondelete="restrict", required=True),
        # id do funcionario requisitante
        'recebeu_id': fields.many2one('rh.funcionario', 'Recebeu', domain="[('company_id', '=', company_id)]",
                                      ondelete="restrict"), # Id do funcionario que recebeu a mercadoria
        'data_hora_recebimento': fields.datetime('Data hora recebimento'), # Data e hora do recebimento\n
        'centrocusto_id': fields.many2one('adm.empresa.centrocusto', 'Centrocusto', ondelete="restrict",
                                          domain="[('company_id', '=', company_id)]"), # id do centro de custo
        'requisicao_item_ids': fields.one2many('estoquealmoxarifado.requisicao.item', 'requisicao_id', 'Item'),
        'estoque_movimento_capa_ids': fields.one2many('estoque.movimento.capa', 'requisicao_id', 'Capa'),
        'flag': fields.boolean('Flag'),
        # Flag para verificacao do parâmetro do financeiro 'usar_centro_custos_e_eventos'
        'almrqc_devolucao': fields.boolean('Devolucao'),
        'almrqc_requisicao_origem_id': fields.many2one('almoxarifado.requisicao.capa', 'Requisicao original'),
        'almrqc_flag_exibir_requisicao': fields.boolean('Exibir requisicao',
                                                        help='Flag para determinar se a requisicao deve aparecer na lista de requisicoes na tela de devolucao')
    }
    _defaults = {
        'state': lambda *a: 'R',
        'data_emissao': lambda *a: strftime('%Y-%m-%d %H:%M:%S'),
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'flag': lambda *a: False,
        'almrqc_devolucao': False,
        'almrqc_flag_exibir_requisicao': True,
    }
    _order = 'data_emissao desc, state'
    _sql_constraints = [
        ('numero_uniq', 'unique (numero)', 'O numero precisa ser unico!'),
    ]

