# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
import time
from openerp.tools.translate import _


class estoquealmoxarifado_requisicao_devolucao_osv(osv.TransientModel):

    domain_requisicao_original_id = "['&',('state','=','E'), " \
                                    "('almrqc_flag_exibir_requisicao','=',True)," \
                                    "('almrqc_devolucao', '=', False),]"

    _name = 'estoquealmoxarifado.requisicao.devolucao.osv'

    _columns = {
        'requisicao_original_id': fields.many2one('almoxarifado.requisicao.capa',
                                                  'Requisicao de origem', required=True,
                                                  domain=domain_requisicao_original_id),
        'data_requisicao_original': fields.datetime('Data requisicao original'),
        'requisitante_original_id': fields.many2one('rh.funcionario', 'Nome requisitante da requisicao original'),
    }

    def onchange_requisicao(self, cr, uid, ids, requisicao_id, context=None):
        context = context and context.copy() or {}
        res = {}
        obj_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        if requisicao_id:
            browse_requisicao = obj_requisicao_capa.browse(cr, uid, requisicao_id)
            res['requisitante_original_id'] = browse_requisicao.funcionario_id.id
            res['data_requisicao_original'] = browse_requisicao.data_emissao
        return {'value': res}

    def button_requisicao_devolucao(self, cr, uid, ids, context=None):
        """
        Realiza a devolucao de uma requisicao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {int} ID da requisicao de devolucao.

        @change: 07.03.2014 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_model_data = self.pool.get('ir.model.data')
        obj_rh_funcionario = self.pool.get('rh.funcionario')

        # Verfica se usuario eh um funcionario
        if not obj_rh_funcionario.get_id_func(cr, uid):
            raise osv.except_osv(_('Atencao!'), _('O usuario deve ser um funcionario.'))

        # Cria a requisicao de devolucao baseada nos dados da requisicao original
        self.wizard_criar_requisicao_devolucao(cr, uid, ids, context)

        reference = obj_model_data.get_object_reference(
            cr, uid, 'estoquealmoxarifado', 'estoquealmoxarifado_requisicao_devolucao_form')
        view_id = reference and reference[1] or False
        result = {'name': _('Devolucao de requisicao'), 'view_type': 'form', 'view_mode': 'form',
                  'res_model': 'estoquealmoxarifado.requisicao.devolucao.osv', 'view_id': [view_id],
                  'target': 'inline', 'type': 'ir.actions.act_window'}

        return result

    def wizard_criar_requisicao_devolucao(self, cr, uid, ids, context):
        """
        Cria a requisicao de devolucao baseada na requisicao original.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {ind} ID da requisicao de devolucao.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoquealmoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        obj_estoquealmoxarifado_requisicao_item = self.pool.get('estoquealmoxarifado.requisicao.item')
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        obj_rh_funcionario = self.pool.get('rh.funcionario')

        requisicao_capa_id = False
        values = {}  # Valores utilizados na criacao da nova requisicao
        campos = ['requisicao_original_id']
        dados_requisicao_original = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_requisicao_original and isinstance(dados_requisicao_original, list):
            dados_requisicao_original = dados_requisicao_original[0]

        if dados_requisicao_original and isinstance(dados_requisicao_original, dict):
            requisicao_original_id = dados_requisicao_original.get('requisicao_original_id', False)

            obj_estoquealmoxarifado_requisicao_capa.write(
                cr, uid, requisicao_original_id, {'almrqc_flag_exibir_requisicao': False}, context=context)
            campos = ['local_id', 'company_id', 'motivo_cancelamento', 'observacao', 'recebeu_id',
                      'data_hora_recebimento', 'centrocusto_id', 'requisicao_item_ids',
                      'estoque_movimento_capa_ids', 'flag']
            dados_requisicao_capa = obj_almoxarifado_requisicao_capa.read(
                cr, uid, requisicao_original_id, campos, context=context, load='_classic_write')

            if dados_requisicao_capa and isinstance(dados_requisicao_capa, list):
                dados_requisicao_capa = dados_requisicao_capa[0]

            if dados_requisicao_capa and isinstance(dados_requisicao_capa, dict):
                local_id = dados_requisicao_capa.get('local_id')
                company_id = dados_requisicao_capa.get('company_id')
                recebeu_id = dados_requisicao_capa.get('recebeu_id')
                centrocusto_id = dados_requisicao_capa.get('centrocusto_id')
                values['motivo_cancelamento'] = dados_requisicao_capa.get('motivo_cancelamento')
                values['observacao'] = dados_requisicao_capa.get('observacao')
                values['data_hora_recebimento'] = dados_requisicao_capa.get('data_hora_recebimento')
                values['flag'] = dados_requisicao_capa.get('flag', False)
                values['centrocusto_id'] = centrocusto_id
                values['recebeu_id'] = recebeu_id
                values['company_id'] = company_id
                values['local_id'] = local_id
                values['funcionario_id'] = obj_rh_funcionario.get_id_func(cr, uid)
                values['data_emissao'] = time.strftime('%Y-%m-%d %H:%M:%S')
                values['state'] = 'R'  # Esboco (rascunho)
                values['almrqc_devolucao'] = True  # Registro de requisicao para devolucao
                values['almrqc_requisicao_origem_id'] = requisicao_original_id

                # Cria a requisicao de devolucao
                requisicao_capa_id = obj_estoquealmoxarifado_requisicao_capa.create(
                    cr, uid, values, context)
                requisicao_item_ids = dados_requisicao_capa.get('requisicao_item_ids', [])

                # Cria os itens da nova requisicao
                if requisicao_item_ids:
                    campos = ['quantidade_requisitada', 'produto_id', 'grade_id', 'local_id',
                              'produto_embalagem_id', 'evento_id', 'aplicacao']
                    dados_requisicao_item = obj_estoquealmoxarifado_requisicao_item.read(
                        cr, uid, requisicao_item_ids, campos, context=context, load='_classic_write')

                    if dados_requisicao_item and isinstance(dados_requisicao_item, dict):
                        dados_requisicao_item = [dados_requisicao_item]

                    if dados_requisicao_item and isinstance(dados_requisicao_item, list):
                        for item in dados_requisicao_item:
                            # Ajusta o numero da requisicao
                            item['requisicao_id'] = requisicao_capa_id
                            obj_estoquealmoxarifado_requisicao_item.create(cr, uid, item, context=context)

        return requisicao_capa_id

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
