#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from decimal import Decimal, ROUND_UP
import threading


def synchronized_method(method):
    """
    Responsavel por sincronizar os metodos, de forma
    que entre apenas uma requisicao por vez em determindado metodo.
    E utilizado para reduzir problemas de concorrencia.

    Para utilizacao e necessario decorar o metodo que deseja sincronizar.
    Antes de utilizar avaliar se metodos que chamam o metodo a ser sincronizado nao
    fazem alteracao em registros de tabelas que o metodo sincronizado vai ler ou alterar, pois
    isso podera causar deadlock

    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    outer_lock = threading.Lock()
    lock_name = "__" + method.__name__ + "_lock" + "__"

    def sync_method(self, *args, **kwargs):
        with outer_lock:
            if not hasattr(self, lock_name) or not getattr(self, lock_name).locked():
                setattr(self, lock_name, threading.Lock())
                lock = getattr(self, lock_name)
                with lock:
                    return method(self, *args, **kwargs)
    return sync_method


class pedido_utils(osv.Model):
    _name = 'pedido.utils'
    _auto = False

    _tem_erro = u''
    _mensagem_erro = u''

    def pedutl_get_erro(self):
        return self._mensagem_erro

    def pedutl_get_pedido_names(self, cr, uid, pedido_ids, context=None):
        # NAME DOS ORCAMENTO_IDS
        context = context and context.copy() or {}
        res_name_get = self.pool.get('estoquepedidos.pedido.capa').name_get(cr, uid, pedido_ids, context) or [('', '')]
        return dict(res_name_get)

    def pedutl_validar_condicoes_pgto_valor_prazo(self, cr, uid, pedido_ids, context=None):
        context = context and context.copy() or {}
        self._tem_erro = u''
        self._mensagem_erro = u''

        # ___________________________ Objects ________________________________
        obj_venda_utils = self.pool.get('vendas.utils')
        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')

        if isinstance(pedido_ids, (int, long)):
            pedido_ids = [pedido_ids]

        erro_prazo_maior_valor_total = u''
        dict_pedido_names = self.pedutl_get_pedido_names(cr, uid, pedido_ids, context=context)
        pedido_dados = obj_estoquepedidos_pedido_capa.read(
            cr, uid, pedido_ids, ['pessoa_id', 'valor_total'], context=context)

        for i in pedido_dados:
            id = i['id']
            pessoa_id = i.get('pessoa_id') and i['pessoa_id'][0] or False
            pedido_valor_total = i.get('valor_total') or 0
            pedido_name = (dict_pedido_names.get(i['id']) or '').encode('utf-8') or \
                          _('** Pedido nao encontrado **')

            total_a_prazo = obj_venda_utils.venutl_get_pedido_total_a_prazo(
                cr, uid, venutl_pessoa_ids=pessoa_id, venutl_pedido_id=id, context=context)

            total_a_prazo = total_a_prazo.get(pessoa_id) or 0
            dif = total_a_prazo - pedido_valor_total
            if dif > 0.01:
                erro_prazo_maior_valor_total += u'%s, ' % pedido_name

        if erro_prazo_maior_valor_total:
            self._tem_erro = True
            self._mensagem_erro += \
                _(u'* Valor a prazo maior que o valor total nos pedidos: %s') % \
                erro_prazo_maior_valor_total

        if self._tem_erro:
            return False

        return True

    def pedutl_venda_liberacao_desconto(self, cr, uid, pedido_ids, context=None):
        context = context and context.copy() or {}
        if isinstance(pedido_ids, (int, long)):
            pedido_ids = [pedido_ids]

        self._mensagem_erro = u''
        self._tem_erro = False

        # OBJETOS
        obj_pedido = self.pool.get('estoquepedidos.pedido.capa')
        obj_liberar_desconto = self.pool.get('vendas.liberar.desconto')

        # NAME DOS PEDIDO_IDS
        dict_pedido_names = self.pedutl_get_pedido_names(cr, uid, pedido_ids, context=context)

        # CONTROLES
        erros_pedido = u''
        info_ped_liberacao_desconto_criada = u''
        erro_ped_liberacao_desconto_nao_autorizada = u''

        pedido_dados = obj_pedido.read(cr, uid, pedido_ids, ['valor_total', 'pedido_condicao_ids'],
                                       context=context)
        for i in pedido_dados:

            ped_id = i['id']
            ped_valor_total = i.get('valor_total') or 0

            pedido_name = (dict_pedido_names.get(ped_id) or '').encode('utf-8') or _('** Nro Pedido nao encontrado **')

            # VALIDAÇÃO DE DESCONTOS DADOS NOS ITENS E NA CAPA.
            desconto_liberado = self.pedutl_validar_desconto_itens(
                cr, uid, pedido_ids=ped_id, context=context
            )
            if not desconto_liberado and not i.get('pedido_condicao_ids'):
                continue

            # *** Liberacao de Desconto ***
            liberacacao_desconto_ped_ids = obj_liberar_desconto.search(
                cr, uid, [('venauv_pedido_id', '=', ped_id)], context=context
            )

            if desconto_liberado:
                # SE NÃO HOUVER NENHUM PROBLEMA REFERENTE A DESCONTO E EXISTIR ALGUM REGISTRO DE LIBERACAO DE
                # DESCONTO, O MESMO SERÁ EXCLUIDO, POIS NÃO SERÁ MAIS NECESSARIO POR NÃO HAVER MAIS DESCONTO.
                if liberacacao_desconto_ped_ids:
                    obj_liberar_desconto.unlink(cr, uid, liberacacao_desconto_ped_ids, context)
                else:
                    continue
            else:
                if not liberacacao_desconto_ped_ids:

                    erros_pedido += self.pedutl_get_erro()

                    # CASO O DESCONTO EXCEDA O LIMITE NOS ITENS, O SISTEMA IRÁ CRIAR AUTOMATICAMENTE UMA LIBERAÇÃO
                    # DE DESCONTO NÃO AUTORIZADA, PARA QUE UM SUPERVISOR AUTORIZE.
                    vals_liberar_desconto = {
                        'venauv_pedido_id': ped_id,
                        'venauv_autorizado': False,
                    }
                    obj_liberar_desconto.create(cr, uid, vals_liberar_desconto, context)
                    # NÃO RETIRAR O COMMIT, POIS HOUVE ERRO E SERÁ FEITO UM RAISE. POREM, É PRECISO
                    # CRIAR A LIBERAÇÃO DE DESCONTO.
                    cr.commit()
                    info_ped_liberacao_desconto_criada += u'%s, ' % pedido_name

                else:
                    args_liberacao_autorizada = [
                        ('venauv_autorizado', '=', True),
                        ('venauv_valor_liquido', '=', round(ped_valor_total, 2)),
                        ('id', 'in', liberacacao_desconto_ped_ids)
                    ]
                    desconto_liberado_valor_liquido_ids = obj_liberar_desconto.search(
                        cr, uid, args_liberacao_autorizada, context=context
                    )
                    if desconto_liberado_valor_liquido_ids:
                        continue
                    else:
                        erros_pedido += self.pedutl_get_erro()

                        erro_ped_liberacao_desconto_nao_autorizada += u'%s, ' % pedido_name
                        values = {
                            'venauv_autorizado': False
                        }
                        obj_liberar_desconto.write(cr, uid, liberacacao_desconto_ped_ids, values, context)
                        cr.commit()
        if info_ped_liberacao_desconto_criada:
            erros_pedido += _('** [CRIADA] - Liberacao de Desconto criada para os pedidos: %s') % \
                            info_ped_liberacao_desconto_criada

        if erro_ped_liberacao_desconto_nao_autorizada:
            erros_pedido += _('* [DESCONTO NAO AUTORIZADO] Pedidos: %s') % erro_ped_liberacao_desconto_nao_autorizada

        if erros_pedido:
            self._tem_erro = True
            self._mensagem_erro = erros_pedido
            return False
        else:
            return True

    def pedutl_validar_desconto_itens(self, cr, uid, pedido_ids, context=None):
        """
        Funcao Valida:
            * Se o desconto dado aos itens e maior que o permitido.
        Retorna:
            * True - Descontos dentro do permitido. Pedido OK
            * False - Descontos acima do limite.
        """
        context = context and context.copy() or {}
        self._mensagem_erro = u''
        self._tem_erro = False

        if isinstance(pedido_ids, (int, long)):
            pedido_ids = [pedido_ids]

        # OBJETOS
        obj_adm_utils = self.pool.get('adm.utils')
        obj_pedido = self.pool.get('estoquepedidos.pedido.capa')
        obj_estoque_produto = self.pool.get('estoque.produto')

        # NAME DOS PEDIDO_IDS
        dict_pedido_names = self.pedutl_get_pedido_names(cr, uid, pedido_ids, context)

        # CONTROLE
        erro_pedido_item_desconto = u''

        erro_pedido_condicao_desconto = u''

        # CONTROLE CONDICAO PAGAMENTO SEM PERMISSAO DE DESCONTO
        nao_permite_desconto_condicao_ids = list()

        browse_pedido = obj_pedido.browse(cr, uid, pedido_ids, context)

        msg = []
        for p in browse_pedido:

            # CONTROLE ITEM
            erro_item_desconto = u''
            erro_condicao_desconto = u''

            # DADOS DO PEDIDO
            PED_ID = p.id

            pedido_name = (dict_pedido_names.get(PED_ID) or '').encode('utf-8') or _('** Nro Pedido nao encontrado **')

            if not p.pedido_condicao_ids:
                msg.append(pedido_name)
                continue

            RECORD_PEDIDO_ITENS = p.pedido_item_ids

            total_desconto_itens = sum(map(lambda i: i.vr_desconto or 0, RECORD_PEDIDO_ITENS))
            PED_DESCONTO = p.vr_desconto or 0

            if not RECORD_PEDIDO_ITENS:
                continue
            else:
                rateio_capa_itens = {}
                if PED_DESCONTO > 0:
                    # SE HOUVER DESCONTO NA CAPA DO ORCAMENTO, RATEA-LO PARA O ITENS.
                    elemetos = map(lambda i: [i.id, i.valor_total], RECORD_PEDIDO_ITENS)
                    elementos_rateados = map(tuple, obj_adm_utils.efetuar_rateio(PED_DESCONTO, elemetos))
                    rateio_capa_itens = dict(elementos_rateados)

                # ITENS DO PEDIDO
                for i in RECORD_PEDIDO_ITENS:
                    ITEM_PED_PRODUTO_ID = i.produto_id.id
                    percentual_desconto_item = obj_estoque_produto.estpro_get_desconto_maximo(cr, uid,
                                                                                              ITEM_PED_PRODUTO_ID, context).get(ITEM_PED_PRODUTO_ID)
                    if percentual_desconto_item:
                        ITEM_PED_ID = i.id
                        ITEM_PED_VALOR_LIQUIDO = i.valor_total or 0
                        ITEM_PED_DESCONTO = i.vr_desconto or 0

                        valor_bruto = ITEM_PED_VALOR_LIQUIDO + ITEM_PED_DESCONTO
                        capa_desconto_rateado = rateio_capa_itens.get(ITEM_PED_ID) or 0
                        # DESCONTO TOTAL DADO NO ITEM (CAPA + DETALHE)
                        desconto_total_item = float(Decimal(str(
                            ITEM_PED_DESCONTO + capa_desconto_rateado)).quantize(
                            Decimal('.01'), rounding=ROUND_UP))

                        desconto_maximo_permitido = float(Decimal(str(
                            valor_bruto * percentual_desconto_item / 100)).quantize(
                            Decimal('.01'), rounding=ROUND_UP))
                        if desconto_total_item:
                            if desconto_total_item > desconto_maximo_permitido:
                                item_name_get = obj_estoque_produto.name_get(
                                    cr, uid, [ITEM_PED_PRODUTO_ID], context) or [('', '')]
                                dict_item_name = dict(item_name_get)
                                produto_name = (dict_item_name.get(ITEM_PED_PRODUTO_ID) or '').encode('utf-8') \
                                    or _('** Produto nao encontrado **')
                                erro_item_desconto += u'%s, ' % produto_name.decode('utf-8')

            if erro_item_desconto:
                erro_pedido_item_desconto += _('%s - Itens: %s') % (pedido_name, erro_item_desconto)

            if PED_DESCONTO or total_desconto_itens:
                nao_permite_desconto_condicao_ids = filter(
                    lambda i: not i.condicao_forma_id.admcof_aceita_desconto,
                    p.pedido_condicao_ids)
                for condicao in nao_permite_desconto_condicao_ids:
                    erro_condicao_desconto += u'%s, ' % condicao.condicao_forma_id.name.decode(
                        'utf-8')
            if erro_condicao_desconto:
                erro_pedido_condicao_desconto += _('%s - Condicoes: %s') % (pedido_name,
                                                                            erro_condicao_desconto)

        if msg:
            self._tem_erro = True
            self._mensagem_erro += _('Pedidos sem condicao de pagamento:\n%s\n\n') % '\n'.join(msg)

        if erro_pedido_item_desconto:
            self._tem_erro = True
            self._mensagem_erro += _('* [DESCONTO ACIMA DO LIMITE]: %s ') % erro_pedido_item_desconto

        if nao_permite_desconto_condicao_ids:
            self._tem_erro = True
            self._mensagem_erro += _('* [CONDICAO DE PAGAMENTO NAO PERMITE DESCONTO]: %s ') % \
                erro_pedido_condicao_desconto

        if self._tem_erro:
            return False
        else:
            return True

    def pedutl_restricao_venda(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self._tem_erro = False
        self._mensagem_erro = u''
        if isinstance(ids, (int, long)):
            ids = [ids]
        pedido_dados = self.pool.get('estoquepedidos.pedido.capa').read(
            cr, uid, ids, ['pessoa_id', 'state'], context=context)
        for i in pedido_dados:
            id = i['id']
            pessoa_id = i.get('pessoa_id') and i['pessoa_id'][0] or False
            state = i.get('state')
            venda_liberada = self.pool.get('vendas.utils').venutl_documento_restricao_venda(
                cr, uid, id, pessoa_id, state, 'PED', context)
            if not venda_liberada:
                erro_venda = self.pool.get('vendas.utils').venutl_get_erro()
                self._tem_erro = True
                self._mensagem_erro += erro_venda
        if self._tem_erro:
            return False

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

