#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields


class wizard_estoque_consulta_saldos(osv.TransientModel):
    _name = 'wizard.consulta.saldos'
    _inherit = 'wizard.consulta.saldos'
    _columns = {
        'pedido_compra_ids': fields.one2many("wizard.consulta.saldo.pedido.compra",
                                             'wcspc_consulta_saldo_id',
                                             'Pedidos de compra'),
    }

    def button_limpar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_wiz_saldo_pedido_compra = self.pool.get('wizard.consulta.saldo.pedido.compra')
        pedido_compra_dados = self.read(cr, uid, ids, ['pedido_compra_ids'], context=context)
        if isinstance(pedido_compra_dados, list):
            for pedido in pedido_compra_dados:
                obj_wiz_saldo_pedido_compra.unlink(cr, uid, pedido.get('pedido_compra_ids'),
                                                   context=context)

        obj_wiz_saldo_reserva_venda = self.pool.get('wizard.consulta.reserva.venda')
        reserva_venda_dados = self.read(cr, uid, ids, ['reserva_venda_ids'], context=context)
        if isinstance(reserva_venda_dados, list):
            for reserva in reserva_venda_dados:
                obj_wiz_saldo_reserva_venda.unlink(cr, uid, reserva.get('reserva_venda_ids'),
                                                   context=context)

        return True

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]

        res = super(wizard_estoque_consulta_saldos, self).button_pesquisar(cr, uid, ids, context)

        obj_wiz_saldo_pedido_compra = self.pool.get('wizard.consulta.saldo.pedido.compra')
        obj_wiz_saldo_reserva_venda = self.pool.get('wizard.consulta.reserva.venda')

        consulta_saldo_dados = self.read(cr, uid, ids, ['company_id', 'produto_id'],
                                         context=context)
        if not consulta_saldo_dados:
            return res

        produto_id = consulta_saldo_dados.get('produto_id')[0]
        company_id = consulta_saldo_dados.get('company_id')[0]

        pedidos_compra = self.get_pedidos(cr, uid, produto_id, company_id, tipo='C', context=context)
        reservas_venda = self.get_reservas_venda(cr, uid, produto_id, company_id, context=context)

        for pedido in pedidos_compra:
            pedido['wcspc_consulta_saldo_id'] = ids
            obj_wiz_saldo_pedido_compra.create(cr, uid, pedido, context=context)

        for reserva in reservas_venda:
            reserva['wcsrv_consulta_saldo_id'] = ids
            obj_wiz_saldo_reserva_venda.create(cr, uid, reserva, context=context)

        return res

    def get_reservas_venda(self, cr, uid, produto_id, company_id, context=None):
        """
        Método que retorna reservas de venda pendentes com o produto informado no parâmetro
        'produto_id'.

        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param produto_id: id do produto
        :param company_id: id da empresa
        :param context: dicionario padrao para valores contextuais

        :type cr: object
        :type uid: int
        :type produto_id: int
        :type company_id: int
        :type context: dict
        :rtype: list

        :return: lista de pedidos de venda e nf de saída que estão pendentes, com o produto
                 informado no parâmetro 'produto_id'.
        """
        context = context and context.copy() or {}

        res = self.get_pedidos(cr, uid, produto_id, company_id, tipo='V', context=context)

        if not self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context):
            return res

        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_estoque_mov_itens = self.pool.get('estoque.movimento.itens')

        nf_capa_ids = obj_fiscal_nf_capa.search(cr, uid, [('state', 'not in', ['1', '3', 'C']),
                                                          ('tipo', '=', 'S'),
                                                          ('company_id', '=', company_id)],
                                                context=context)

        mov_itens_ids = obj_estoque_mov_itens.search(cr, uid, [('nf_id', 'in', nf_capa_ids),
                                                               ('produto_id', '=', produto_id)],
                                                     context=context)

        brw_mov_itens = obj_estoque_mov_itens.browse(cr, uid, mov_itens_ids, context=context)

        for item in brw_mov_itens:
            nf_id = item.nf_id.id
            data_emissao = item.nf_id.data_emissao
            qtde_pendente = item.quantidade or 0.0

            res.append({
                'wcsrv_nf_id': nf_id,
                'wcsrv_data_emissao': data_emissao,
                'wcsrv_quantidade': qtde_pendente,
                'wcsrv_documento': 'NF-%s' % item.nf_id.name,
            })

        return res

    def get_pedidos(self, cr, uid, produto_id, company_id, tipo='C', context=None):
        """
        Método que retorna pedidos pendentes com o produto informado no parâmetro 'produto_id'.

        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param produto_id: id do produto
        :param company_id: id da empresa
        :param tipo: 'C' compra | 'V' venda
        :param context: dicionario padrao para valores contextuais

        :type cr: object
        :type uid: int
        :type produto_id: int
        :type company_id: int
        :type tipo: str
        :type context: dict
        :rtype: list

        :return: lista de pedidos que estão pendentes com o produto informado no parâmetro
                 'produto_id'.
        """
        context = context and context.copy() or {}
        cr.execute('DELETE FROM wizard_consulta_saldo_pedido_compra')

        pedidos = []

        obj_estoque_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_estoque_pedido_itens = self.pool.get('estoquepedidos.pedido.item')

        pedido_capa_ids = obj_estoque_pedido_capa.search(cr, uid, [('state', '=', 'V'),
                                                                   ('tipo', '=', tipo),
                                                                   ('company_id', '=', company_id)],
                                                         context=context)

        pedido_itens_ids = obj_estoque_pedido_itens.search(cr, uid,
                                                           [('pedido_id', 'in', pedido_capa_ids),
                                                            ('produto_id', '=', produto_id)],
                                                           context=context)

        brw_pedido_itens = obj_estoque_pedido_itens.browse(cr, uid, pedido_itens_ids,
                                                           context=context)

        for item in brw_pedido_itens:
            pedido_id = item.pedido_id.id
            data_emissao = item.pedido_id.data_emissao
            data_entrega = item.pedido_id.data_entrega
            qtde_pendente = (item.qtde_pedida or 0.0) - (item.qtde_atendida or 0.0)

            vals = {}

            if tipo == 'C':
                vals = {
                    'wcspc_pedido_id': pedido_id,
                    'wcspc_data_emissao': data_emissao,
                    'wcspc_data_entrega': data_entrega,
                    'wcspc_qtde_pendente': qtde_pendente,
                }

            elif tipo == 'V':
                vals = {
                    'wcsrv_pedido_id': pedido_id,
                    'wcsrv_data_emissao': data_emissao,
                    'wcsrv_data_entrega': data_entrega,
                    'wcsrv_quantidade': qtde_pendente,
                    'wcsrv_documento': item.pedido_id.numero_pedido,
                }

            pedidos.append(vals)

        return pedidos
