# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from datetime import datetime
from openerp.tools.translate import _


class estoquepedidos_pedido_capa(osv.Model):
    """
    Tabela de pedidos(Compra e venda)
    """

    from funcoes import pedcap_get_ocorrencias, _pedcap_get_field_stored
    from funcoes import _pedcap_get_divergencia_qtd, _pedcap_get_premite_alterar, pedcap_get_foto_cliente
    from funcoes import pedcap_simp_get_pedido_state

    def pedcap_fields_selection_state(self, cr, uid, context=None):
        context = context and context.copy() or {}
        res = [('R', 'Esboco'), ('V', 'Validado'), ('S', 'Em separacao'), ('D', 'Separado'),
               ('F', 'Faturado'), ('C', 'Cancelado')]
        return res

    _name = 'estoquepedidos.pedido.capa'
    _rec_name = 'numero_pedido'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
        ## _________________________________ char ______________________________________________
        # 'name': fields.char('Numero do pedido', size=20),
        'numero_pedido': fields.char('Numero do pedido', size=20, select=True),  # Numero do pedido.
        'observacao': fields.text('Observacao'),
        'pedcap_endereco_eletronico': fields.char('Endereco eletronico', size=255),
        'pedcap_nro_pedido_externo': fields.char('Nro externo do pedido', size=40, select=True,
                                                 help='Numero do pedido no cliente/fornecedor',
                                                 oldname='pedcap_ml_nro_pedido'),
        'pedcap_placa': fields.char('Placa', size=10),

        ## _________________________________ datetime __________________________________________
        'data_emissao': fields.date('Data de Emissao', required=True),  # Data de emissao do pedido
        'create_date': fields.datetime('Create date', readonly=True),  # Data de registro do pedido no sistema
        'write_date': fields.datetime('Ultima alteracao', readonly=True),  # Data de registro do pedido no sistema
        'data_entrega': fields.date('Data de Entrega', select=True),  # Data da previsao a ser entregue o pedido.
        'pedcap_data_faturamento': fields.date('Data de faturamento', select=True, readonly=True),

        ## _________________________________ function __________________________________________
        'valor_total': fields.function(_pedcap_get_field_stored, method=True, string='Total pedido', type='float',
                                       digits=(15, 2), store=True, multi='dados'),  # Valor total do Pedido
        'pedcap_pj_pf': fields.function(_pedcap_get_field_stored, method=True, string='PJ PF', type='char', size=2,
                                        store=True, multi='dados'),  # Flag que identifica se o cliente é PJ ou PF
        'pedcap_ocorrencia_ids': fields.function(pedcap_get_ocorrencias, method=True, store=False, string='Ocorrencias',
                                                 type="one2many", relation="adm.ocorrencia"),
        'pedcap_divergencia_qtd': fields.function(_pedcap_get_divergencia_qtd, method=True,
                                                  string='Divergencia na quantidade',
                                                  type='boolean', store=False),
        'pedcap_permite_alterar_condicoes_pgto': fields.function(_pedcap_get_premite_alterar,
                                                                 'Permitir alterar condicoes de pagamento',
                                                                 method=True, type='boolean', store=False),

        ## _________________________________ boolean ___________________________________________
        'exportado_frente': fields.boolean('Exportado p/ frente', readonly=True, select=True,
                                           help="Indica se o frente importou o pedido"),
        'pedcap_ja_impresso': fields.boolean('Pedido impresso'),

        ## _________________________________ float _____________________________________________
        'vr_frete': fields.float('Valor do Frete', digits=(14, 2)),  # Valor do frete
        'vr_seguro': fields.float('Valor do seguro', digits=(15, 2), help="Valor do Seguro"),  # Valor do seguro
        'vr_desconto': fields.float('Desconto', digits=(15, 2), help="Valor do Desconto"),
        # Desconto em valor, isto e, a bonificacao.
        'vr_acrescimo': fields.float(string='Acrescimo', digits=(15, 2), help="Valor do acrescimo"),
        'vr_despesas_acessorias': fields.float('Despesas Acessorias', digits=(15, 2),
                                               help="Valor das despesas Acessorias"),  # Valor das despesas acessorias
        'pedcap_percentual_desconto': fields.float('% Desconto', digits=(3, 2),
                                                   help="Percentual de desconto no Pedido."),

        ## _________________________________ selection _____________________________________________
        'state': fields.selection(pedcap_fields_selection_state, 'Estado do Pedido', required=True, size=1, select=True),
        'tipo': fields.selection([('C', 'Compra'), ('V', 'Venda')], 'Tipo', required=True, size=1, select=True),
        # Tipo de pedido: \nC=Compra, \nV=Venda
        'frete': fields.selection([('C', 'CIF'), ('F', 'FOB')], 'Frete', size=1),

        ## _________________________________ many2one _____________________________________________
        'pessoa_id': fields.many2one("adm.pessoa", 'Pessoa', ondelete="restrict", required=False,
                                     select=True, help="Informe o nome do cliente, CPF ou CNPJ, "
                                                       "código interno ou use:\n'f=' seguido do "
                                                       "telefone, \n'a=' seguido do apelido, \n"
                                                       "'d=' seguido da data de nascimento, \n'"
                                                       "e=' seguido do email ou \n'c=' seguido do "
                                                       "cep"),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True),
        'pedcap_contato_id': fields.many2one('adm.pessoa', 'Contato', context="{'contatos_de': pessoa_id}"),
        # Id do contato
        'transportador_id': fields.many2one("adm.transportador", 'Transportador', ondelete="restrict", select=True),
        # id do transportador
        'pedcap_responsavel_id': fields.many2one('rh.funcionario', 'Responsavel pela producao',
                                                 help='Funcionario que realizara a separacao da '
                                                      'mercadoria para pedido'),
        'pedcap_endereco_id': fields.many2one("adm.pessoa.endereco", 'Endereco ',
                                              ondelete="restrict",
                                              domain="[('pessoa_id', '=', pessoa_id)]"),
        'pedcap_state_id': fields.many2one('estoquepedidos.pedido.capa.state', 'Status',
                                           ondelete='set null'),

        ## _________________________________ one2one _____________________________________________
        'pedido_item_ids': fields.one2many('estoquepedidos.pedido.item', 'pedido_id', 'Item'),
        'pedido_parcela_ids': fields.one2many('estoquepedidos.pedido.parcela', 'pedido_id', 'Parcela'),
        'pedido_condicao_ids': fields.one2many('estoquepedidos.pedido.condicao', 'capa_id', 'Condicoes'),
        'pedcap_pessoa_clientes_id': fields.function(_pedcap_get_field_stored, method=True, string='ID Cliente',
                                                     type='many2one', relation='financeiro.cliente', store=True,
                                                     multi='dados'),
        'pedcap_foto_cliente': fields.function(pedcap_get_foto_cliente, type='binary', method=True, string='Foto Cliente', store=False),

        # Atributos especificos para o pedido de venda simplificado
        'pedcap_simp_pessoa_nome': fields.char('Nome', size=128),
        'pedcap_simp_cpf_cnpj': fields.char('CPF/CNPJ', size=19),
        'pedcap_simp_uf': fields.char("Estado", size=20),
        'pedcap_simp_cidade': fields.char("Cidade", size=50),
        'pedcap_simp_bairro': fields.char("Bairro", size=30),
        'pedcap_simp_logradouro': fields.char("Logradouro", size=50),
        'pedcap_simp_numero': fields.char('Numero', size=5),
        'pedcap_simp_cep': fields.char('CEP', size=10),
        'pedcap_simp_email': fields.char('E-mail', size=75),
        'pedcap_simp_telefone_numero': fields.char('Telefone', size=25),
        'pedcap_simp_quantidade': fields.float('Quantidade', digits=(15, 3)),
        'pedcap_simp_state': fields.function(pedcap_simp_get_pedido_state, method=True,
                                             string='Status', type='char', size=1),
        'pedcap_simp_produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="restrict",
                                                  domain="[('tipo', '=', 'P')]"),
        'pedcap_simp_grade_id': fields.many2one(
            "estoque.produto.grade", 'Grade', ondelete="restrict",
            domain="[('produto_id', '=', pedcap_simp_produto_id)]"),
        # Id do produto a ser pedido\n
        'pedcap_tags_ids': fields.many2many('adm.tags', 'estoquepedidos_tags', 'pedtag_pedido_id',
                                            'ped_tag_id', 'Marcas'),

    }
    _defaults = {
        'exportado_frente': lambda *a: False,
        'vr_desconto': lambda *a: 0,
        'data_emissao': lambda *a: datetime.strftime(a[0].pool.get('adm.utils').
                                                      admutl_converte_tz(a[1], a[2], datetime.now(),
                                                      'UTC', context=a[3]), '%Y-%m-%d'),
        'state': lambda *a: 'R',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'pedcap_ja_impresso': lambda *a: False,
        'pedcap_simp_quantidade': 1.0,
    }
    _order = 'numero_pedido desc'
