# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields


class estoquepedidos_pedido_item(osv.Model):
    """
    PREFIXO PARA OS CAMPOS: pedite_
    """

    from funcoes import pedite_get_item_valor_total, pedite_get_ipi_valor
    from funcoes import pedite_get_local_venda_padrao, _pedite_get_foto, pedite_get_state
    from funcoes import _pedite_get_saldo_produto, _pedite_get_prod_similares
    from funcoes import _pedite_get_par_similaridade, default_get_pedite_par_similaridade
    from funcoes import _pedite_get_qtd_pedida_compra, pedite_check_permissao_altera_preco
    from funcoes import _pedite_get_item_locacao, _pedite_func_get_saldo_empresas
    from funcoes import pedite_get_state_capa

    _rec_name = 'pedido_id'
    _name = 'estoquepedidos.pedido.item'
    _inherits = {'estoquepedidos.pedido.capa': 'pedido_id'}
    _columns = {
        'create_date': fields.datetime('Data criacao'),
        'pedido_id': fields.many2one("estoquepedidos.pedido.capa", 'Pedido', ondelete="cascade", required=True,
                                     select=True), # id do pedido\n
        'qtde_pedida': fields.float('Qtd. Pedida', digits=(15, 3), required=True), # Quantidade pedida
        'qtde_atendida': fields.float('Qtd. Atendida', digits=(15, 3)), # Quantidade ja entregue
        'vr_unitario': fields.float('Valor Unit.', digits=(15, 3), required=True), # Valor unitario do item
        'vr_desconto': fields.float('Valor Desc.', digits=(15, 2)), # Valor do desconto
        'pedite_despesas': fields.float('Despesas', digits=(15, 2),
                                        help="Valor das despesas do item"),
        'observacao': fields.text('Observacao'), # informacoes complementares
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="restrict", required=True, select=True,
                                      domain="[('tipo', '=', 'P')]"), # Produto
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', ondelete="restrict", select=True, required=True,
                                    domain="[('produto_id', '=', produto_id)]"), # Id do produto a ser pedido\n
        'local_id': fields.many2one("estoque.local", 'Local', ondelete="restrict", select=True, required=True),
        # Id do local a ser movimentado
        'produto_embalagem_id': fields.many2one("estoque.produto.embalagem", 'Embalagem', ondelete="restrict",
                                                select=True, required=True, domain="[('produto_id', '=', produto_id)]"),
        # id da embalagem pedida
        'valor_total': fields.function(pedite_get_item_valor_total, method=True, string='Valor Total', type='float',
                                       digits=(15, 2), store=True),
        'pei_preco_venda': fields.float('Preco Venda', digits=(15, 2)),
        'pedite_ipi_aliquota_entrada': fields.float('% IPI', digits=(5, 2)),
        'pedite_percentual_desconto': fields.float('% Desconto', digits=(3, 2),
                                                   help="Percentual do Desconto do Item."),
        'pedite_ipi_valor_entrada': fields.function(pedite_get_ipi_valor, method=True, string="Valor IPI", type="float",
                                                    store=False),
        'pedite_vr_bonificacao': fields.float('Valor Bonificacao', digits=(15, 2)),
        'pedite_foto': fields.function(_pedite_get_foto, method=True, string='Foto', type="binary", store=False),
        'pedite_state': fields.function(pedite_get_state, method=True, string='State Pedido',
                                        type="char", store=False),
        'pedite_saldo': fields.function(_pedite_get_saldo_produto, method=True, string='Saldo Disponivel',
                                        type='float', digits=(15, 3), store=False, multi='saldo'),
        'pedite_reservada': fields.function(_pedite_get_saldo_produto, method=True,
                                            string='Qtd. reservada', type='float', digits=(15, 3),
                                            store=False, multi='saldo'),
        'pedite_prod_similares': fields.function(_pedite_get_prod_similares, method=True,
                                                string='Produtos Similares', relation='estoque.produto.similaridade',type='one2many', store=False),
        'pedite_par_similaridade': fields.function(_pedite_get_par_similaridade, method=True,
                                                string='Similaridade', type='boolean', store=False),
        'pedite_qtd_pedida_compra': fields.function(_pedite_get_qtd_pedida_compra, method=True,
                                                    string='Qtd. pedida p/ compra', type='float',
                                                    digits=(15, 3), store=False),
        'pedite_pedido_cliente_nro': fields.char('Numero do pedido do cliente', size=15),
        'pedite_pedido_cliente_linha': fields.integer('Linha do item do pedido'),
        'pedite_altera_preco_venda': fields.function(
            pedite_check_permissao_altera_preco, method=True, string='Permite alterar preco venda',
            type='boolean'),
        'pedite_pedido_item_locacao': fields.function(_pedite_get_item_locacao, method=True,
                                                      type='char', size=15, string='Locacao',
                                                      store=False),

        'pedite_saldo_empresas': fields.function(_pedite_func_get_saldo_empresas, method=True,
                                                 type='one2many', multi="saldos",
                                                 relation='wizard.estoque.produto.saldo',
                                                 string='Saldos'),
        'pedite_tipo_state': fields.function(pedite_get_state_capa, method=True, string='Status',
                                             type='char', size=1),
    }
    _defaults = {
        'vr_desconto': lambda *a: 0,
        'local_id': pedite_get_local_venda_padrao,
        'pedite_par_similaridade': default_get_pedite_par_similaridade,
        'pedite_tipo_state': lambda *a: 'I',
    }
