# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class wizard_importa_pedido_empresa_atual(osv.TransientModel):

    _name = 'wizard.importa.pedido.empresa.atual'
    _columns = {
        'id': fields.integer('ID', readonly=True),
        'wimpea_item_ids': fields.one2many('wizard.importa.pedido.empresa.atual.item',
                                           'wimpei_capa_id', 'Itens'),
    }

    def default_get(self, cr, uid, fields_list, context=None):
        """
        O metodo atribui valor padrao para os campos recebidos em 'fields_list'.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista da descricao com atributos (campos).
        :type fields_list: list
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados da descricao dos atributos com seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(wizard_importa_pedido_empresa_atual, self).default_get(
            cr, uid, fields_list, context)
        if not context.get('active_ids'):
            return res
        items_ids = list()
        for pedido in self.pool.get('estoquepedidos.pedido.capa').browse(
                cr, uid, context.get('active_ids'), context):
            vals = {'wimpei_pedido_id': pedido.id,
                    'company_id': pedido.company_id.id,
                    'wimpei_numero_pedido': pedido.numero_pedido,
                    'wimpei_pessoa_id': pedido.pessoa_id.id,
                    'wimpei_data_emissao': pedido.data_emissao,
                    'wimpei_valor_total': pedido.valor_total,
                    'wimpei_state': pedido.state,
                    }
            items_ids.append((0, 0, vals))
        if items_ids:
            res['wimpea_item_ids'] = items_ids
        return res

    def wimpea_button_confirma_operacao(self, cr, uid, ids, context=None):
        """
        O metodo realiza a importacao (duplica) dos pedidos de venda selecionados de uma outra
        empresa para a que esta logada.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: dados de valores contextuais.
        :type context: dict
        :return: acao para chamada do formulario desse registro com os dados atualizados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        logada_company_id = self.pool.get('res.users').browse(cr, uid, uid, context).company_id.id
        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_wizard_importa_pedido_item = self.pool.get('wizard.importa.pedido.empresa.atual.item')
        default = {'company_id': logada_company_id}
        context.update({'estoquepedidos_importacao': True})
        for capa in self.browse(cr, uid, ids, context):
            for item in capa.wimpea_item_ids:
                new_pedido_id = obj_estoquepedidos_pedido_capa.copy(
                    cr, uid, item.wimpei_pedido_id.id, default, context)
                novo_numero_pedido = obj_estoquepedidos_pedido_capa.browse(
                    cr, uid, new_pedido_id, context).numero_pedido
                obj_wizard_importa_pedido_item.write(
                    cr, uid, item.id, {'wimpei_novo_numero_pedido': novo_numero_pedido}, context)
        return {
            'res_id': ids[0],
            'name': "Importar ped. na empresa atual",
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': False,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }


class wizard_importa_pedido_empresa_atual_item(osv.TransientModel):

    def wimpei_fields_selection_state(self, cr, uid, context=None):
        """
        O metodo encontra os estados disponiveis para um pedido.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados com o identificador e descricao de cada estado.
        :rtype: list
        """
        context = context and context.copy() or {}
        res = self.pool.get('estoquepedidos.pedido.capa').pedcap_fields_selection_state(cr, uid,
                                                                                        context)
        return res

    _name = 'wizard.importa.pedido.empresa.atual.item'
    _columns = {
        'wimpei_capa_id': fields.many2one('wizard.importa.pedido.empresa.atual', 'Capa'),
        'wimpei_pedido_id': fields.many2one('estoquepedidos.pedido.capa', 'Pedido'),
        'company_id': fields.many2one('res.company', 'Empresa'),
        'wimpei_numero_pedido': fields.char('Numero do pedido', size=20, select=True),
        'wimpei_pessoa_id': fields.many2one('adm.pessoa', 'Pessoa'),
        'wimpei_data_emissao': fields.date('Data de emissao'),
        'wimpei_valor_total': fields.float('Total pedido', digits=(15, 2)),
        'wimpei_state': fields.selection(wimpei_fields_selection_state, 'Estado do Pedido'),
        'wimpei_novo_numero_pedido': fields.char('Numero novo pedido', size=20, select=True),
    }
