# -*- coding: utf-8 -*-

from openerp.osv import osv
import time
from openerp.tools.translate import _


class estoqueressuprimento_lista_capa(osv.Model):

    _inherit = 'estoqueressuprimento.lista.capa'

    def reslca_button_pesquisa(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Pesquisar'. Acao para busca dos produtos na empresa de saida.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        cr.execute("""DELETE FROM estoqueressuprimento_lista_itens WHERE reslit_capa_id = %s""" % (
                isinstance(ids, list) and ids[0] or ids))
        lista_capa_dados = self.read(cr, uid, isinstance(ids, list) and ids[0] or ids, [], context,
                                     '_classic_write')
        empresa_saida_id = lista_capa_dados.get('reslca_empresa_saida_id')
        local_saida_id = lista_capa_dados.get('reslca_local_saida_id')
        company_id = lista_capa_dados.get('company_id')
        local_entrada_id = lista_capa_dados.get('reslca_local_entrada_id')
        data_inicial = lista_capa_dados.get('reslca_data_inicial')
        data_final = lista_capa_dados.get('reslca_data_final')
        produto_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.produto', context=context, user_company_id=empresa_saida_id)
        sql = """
            SELECT
                DISTINCT ep.codigo, ell.grade_id, ell.produto_id, 
                ell.estllc_produto_embalagem_id AS produto_embalagem_id
            FROM
                estoque_local_locacao ell
            LEFT JOIN
                estoque_produto ep ON ep.id = ell.produto_id
            WHERE
                ell.local_id = {local_saida_id}
                AND ep.company_id = {produto_company_id}
                AND ell.active IS TRUE
                AND ep.active IS TRUE
            ORDER BY
                ep.codigo
        """.format(local_saida_id=local_saida_id, produto_company_id=produto_company_id)
        cr.execute(sql)
        locacao_dados = cr.dictfetchall()
        grade_produto_dict = {}
        map(lambda g: grade_produto_dict.update(
            {g.get('grade_id'): {'produto_id': g.get('produto_id'),
                                 'produto_embalagem_id': g.get('produto_embalagem_id')}}),
            locacao_dados)
        grade_ids = grade_produto_dict.keys()
        empresa_saida_saldos = obj_estoque_produto_saldo.get_saldo_do_dia(
            cr, uid, grade_ids, time.strftime('%Y-%m-%d'), local_saida_id, empresa_saida_id,
            context)
        saldo_grade_ids = filter(lambda s: empresa_saida_saldos.get(s).get('disponivel') > 0,
                                 empresa_saida_saldos)
        if not saldo_grade_ids:
            return True
        empresa_entrada_saldos = obj_estoque_produto_saldo.get_saldo_do_dia(
            cr, uid, saldo_grade_ids, time.strftime('%Y-%m-%d'), local_entrada_id, company_id,
            context)

        empresa_entrada_vendas_dict = self.reslca_get_quantidade_vendida_empresa_entrada(
            cr, uid, company_id, saldo_grade_ids, local_entrada_id, data_inicial, data_final,
            context)

        itens = []
        for grade_id in saldo_grade_ids:
            vals = {'reslit_produto_id': grade_produto_dict.get(grade_id).get('produto_id'),
                    'reslit_grade_id': grade_id,
                    'reslit_saldo_saida': empresa_saida_saldos.get(grade_id) and
                    empresa_saida_saldos.get(grade_id).get('disponivel') or 0,
                    'reslit_saldo_entrada': empresa_entrada_saldos.get(grade_id) and
                    empresa_entrada_saldos.get(grade_id).get('disponivel') or 0,
                    'reslit_qtd_vendida': empresa_entrada_vendas_dict.get(grade_id) or 0,
                    'reslit_produto_embalagem_id': grade_produto_dict.get(grade_id).get(
                        'produto_embalagem_id')}
            itens.append((0, 0, vals))
        itens and self.write(cr, uid, ids, {'reslca_itens_ids': itens}, context)
        return True

    def reslca_get_quantidade_vendida_empresa_entrada(self, cr, uid, company_id, grade_ids,
                                                      local_entrada_id, data_inicial, data_final,
                                                      context=None):
        """
        O metodo calcula a quantidade vendida de cada produto (grade_ids) na empresa de entrada
        (company_id), considerando um local e um periodo.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param company_id: id da tabela res_company, identificador da empresa
        :type company_id: int
        :param grade_ids: lista de id da tabela estoque_produto_grade, identificadores dos produtos
        :type grade_ids: list
        :param local_entrada_id: id da tabela estoque_local, identificador do local do produto
        :type local_entrada_id: int
        :param data_inicial: descricao da data inicial do periodo
        :type data_inicial: str
        :param data_final: descricao da data final do periodo
        :type data_final: str
        :param context: dados contextuais
        :type context: dict
        :return: dados da identificacao das grades e suas respectivas quantidades vendidas
        :rtype: dict
        """
        context = context and context.copy() or {}
        sql = """
            SELECT
                emi.grade_id AS grade_id,
                sum(emi.quantidade) AS quantidade
            FROM
                estoque_movimento_itens emi
            INNER JOIN
                estoque_movimento_capa emc ON emc.id = emi.capa_id
            LEFT JOIN
                estoque_natureza_operacao eno ON eno.id = emc.estmvc_operacao_id
            LEFT JOIN
                fiscal_nf_capa nf ON nf.id = emi.nf_id
            LEFT JOIN
                fiscal_cfop fc ON fc.id = emi.cfop_id
            WHERE
                emc.company_id = {company_id}
                AND emi.grade_id IN ({grade_ids})
                AND emi.local_id = {local_entrada_id}
                AND fc.movimento_tipo = 'C'
                AND fc.tipo = 'S'
                AND nf.state = '3'
                AND nf.data_emissao BETWEEN '{data_inicial} 00:00:00' AND '{data_final} 23:59:59'
                AND eno.estnto_tipo_operacao = 'S'
            GROUP BY
                emi.grade_id 
        """.format(company_id=company_id,
                   grade_ids=', '.join(map(lambda g: str(g), grade_ids)),
                   local_entrada_id=local_entrada_id, data_inicial=data_inicial,
                   data_final=data_final)
        cr.execute(sql)
        empresa_entrada_vendas = cr.dictfetchall()
        empresa_entrada_vendas_dict = {}
        map(lambda v: empresa_entrada_vendas_dict.update({v.get('grade_id'): v.get('quantidade')}),
            empresa_entrada_vendas)
        return empresa_entrada_vendas_dict

    def reslca_onchange_empresa_saida(self, cr, uid, ids, empresa_saida_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'reslca_local_empresa_saida_id': False, 'reslca_local_saida_id': False}}
        if not empresa_saida_id:
            return res
        local_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local', context=context, user_company_id=empresa_saida_id)
        res['value'].update({'reslca_local_empresa_saida_id': local_company_id})
        return res

    def reslca_button_gera_requisicao(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Gerar requisicao'. Cria uma requisicao de ressuprimento com os itens que
        possuem quantidade a transferir.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_lista_capa = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids, context)
        itens = filter(lambda i: i.reslit_qtd_transferir > 0, brw_lista_capa.reslca_itens_ids)
        if not itens:
            raise osv.except_osv(_('Atencao'), _('Quantidade a transferir invalida para os itens!'))
        vals = {
            'company_id': brw_lista_capa.reslca_empresa_saida_id.id,
            'resrca_empresa_entrada_id': brw_lista_capa.company_id.id,
            'local_entrada_id': brw_lista_capa.reslca_local_entrada_id.id,
            'local_saida_id': brw_lista_capa.reslca_local_saida_id.id,
        }
        requisicao_item_ids = []
        obj_estoque_locacao = self.pool.get('estoque.local.locacao')
        context.update({'sig_force_company': True})
        itens_entrada = []
        for item in itens:
            list_args = [('produto_id', '=', item.reslit_produto_id.id),
                         ('grade_id', '=', item.reslit_grade_id.id),
                         ('company_id', '=', brw_lista_capa.company_id.id),
                         ('local_id', '=', brw_lista_capa.reslca_local_entrada_id.id),
                         ('estllc_produto_embalagem_id', '=', item.reslit_produto_embalagem_id.id)
                         ]
            if not obj_estoque_locacao.search(cr, uid, list_args, context=context):
                itens_entrada.append(
                    item.reslit_produto_id.codigo + ' - ' + item.reslit_produto_id.name + ' - ' +
                    item.reslit_grade_id.grade)
                continue
            vals_item = {
                'produto_id': item.reslit_produto_id.id,
                'grade_id': item.reslit_grade_id.id,
                'quantidade_requisitada': item.reslit_qtd_transferir,
                'embalagem_id': item.reslit_produto_embalagem_id.id,
            }
            requisicao_item_ids.append((0, 0, vals_item))

        if itens_entrada:
            raise osv.except_osv(_('Atencao'), _('O produto e a grade selecionados, \
                nao encontram-se cadastrados para o local de entrada:\n %s') % ', \n'.join(
                itens_entrada))

        vals.update({'requisicao_itens_ids': requisicao_item_ids})
        requisicao_id = self.pool.get('estoqueressuprimento.requisicao.capa').create(
            cr, uid, vals, context)
        del context['sig_force_company']
        self.write(cr, uid, ids, {'reslca_requisicao_id': requisicao_id}, context)
        return True

    def reslca_button_imprime_lista(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Imprimir'. Acao para chamada da classe de impressao da lista.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: dados de acao para a classe do relatorio
        :rtype: dict
        """
        context = context and context.copy() or {}
        lista_capa_id = isinstance(ids, list) and ids[0] or ids
        lista_capa_dados = self.read(
            cr, uid, lista_capa_id, ['reslca_empresa_saida_id', 'reslca_local_saida_id'], context,
            '_classic_write')
        empresa_saida_id = lista_capa_dados.get('reslca_empresa_saida_id')
        local_saida_id = lista_capa_dados.get('reslca_local_saida_id')
        sql = """
            SELECT
                rc.id || ' - ' || rc.name AS empresa_saida,
                el1.numero || ' - ' || el1.name AS local_saida,
                el2.numero || ' - ' || el2.name AS local_entrada,
                elc.reslca_data_inicial AS data_inicial,
                elc.reslca_data_final AS data_final,
                elc.name AS descricao,
                ap.nome AS usuario
            FROM
                estoqueressuprimento_lista_capa elc
            LEFT JOIN
                res_company rc ON rc.id = elc.reslca_empresa_saida_id
            LEFT JOIN
                estoque_local el1 ON el1.id = elc.reslca_local_saida_id
            LEFT JOIN
                estoque_local el2 ON el2.id = elc.reslca_local_entrada_id
            LEFT JOIN
                res_users ru ON ru.id = elc.create_uid
            LEFT JOIN
                adm_pessoa ap ON ap.id = ru.pessoa_id
            WHERE
                elc.id = {lista_id} 
        """.format(lista_id=lista_capa_id)
        cr.execute(sql)
        dados_capa = cr.dictfetchall()[0]

        sql = """
            SELECT
                (SELECT locacao 
                 FROM estoque_local_locacao
                 WHERE
                    company_id = {empresa_saida_id}
                    AND local_id = {local_saida_id}
                    AND produto_id = eli.reslit_produto_id
                    AND grade_id = eli.reslit_grade_id
                    AND estllc_produto_embalagem_id = eli.reslit_produto_embalagem_id
                ) AS locacao,
                ep.codigo AS produto_codigo,
                ep.name AS produto_name,
                epg.grade AS grade_grade,
                eli.reslit_saldo_saida AS saldo_saida,
                eli.reslit_saldo_entrada AS saldo_entrada,
                eli.reslit_qtd_vendida AS qtd_vendida,
                eli.reslit_qtd_transferir AS qtd_transferir
            FROM
                estoqueressuprimento_lista_itens eli
            LEFT JOIN
                estoque_produto ep ON ep.id = eli.reslit_produto_id
            LEFT JOIN
                estoque_produto_grade epg ON epg.id = eli.reslit_grade_id
            WHERE
                eli.reslit_capa_id = {lista_capa_id}
                AND eli.reslit_qtd_transferir > 0
            ORDER BY
                locacao
        """.format(empresa_saida_id=empresa_saida_id, local_saida_id=local_saida_id,
                   lista_capa_id=lista_capa_id)
        cr.execute(sql)
        dados_itens = cr.dictfetchall()
        if not dados_itens:
            raise osv.except_osv(_('Atencao'), _('Nao ha itens com quantidade a transferir.'))
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'lista_para_ressuprimento',
            'name': 'lista_para_ressuprimento',
            'datas': {'capa': dados_capa, 'itens': dados_itens}
        }
