# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from time import strftime
from openerp.tools.translate import _


class estoqueressuprimento_requisicao_capa(osv.Model):
    _name = 'estoqueressuprimento.requisicao.capa'
    _inherit = 'estoqueressuprimento.requisicao.capa'

    def copy(self, cr, uid, requisicao_id, default=None, context=None):
        context = context and context.copy() or {}
        if not default:
            default = {}
        default.update({
            'nf_id': False,
        })
        res = super(estoqueressuprimento_requisicao_capa, self).copy(cr, uid, requisicao_id,
                                                                     default, context)
        return res

    def default_get(self, cr, uid, fields_list, context=None):
        res = super(estoqueressuprimento_requisicao_capa, self).default_get(cr, uid, fields_list,
                                                                            context)
        if res.get('company_id'):
            a_usar_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'estoque.local', context=context,
                user_company_id=res.get('company_id'))
            res['resrca_a_usar_company_id'] = a_usar_company_id
        return res

    def read(self, cr, user, ids, fields=None, context=None, load='_classic_read'):
        res = super(estoqueressuprimento_requisicao_capa, self).read(cr, user, ids, fields,
                                                                     context, load)
        return res

    def resrca_button_atualizar(self, cr, uid, ids, context=None):
        """
        O Metodo forca a criacao/atualizacao de um ou vários registros. Isso porque metodo de
        botoes do tipo 'object' sempre executa o create ou write.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        return True

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __code = 'numero_requisicao.ressuprimento'
        __name = 'Numero Requisicao Ressuprimento'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 6,
            'company_id': company_id,
            'prefix': 'RQ-%(year)s-',
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence.type').search(cr, uid, [('code', '=', __code)],
                                                            context=context)
        if not consulta:
            self.pool.get('ir.sequence.type').create(cr, uid, {'name': __name, 'code': __code},
                                                     context)
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def button_esboco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        return self.write(cr, uid, ids, {'state': 'R'}, context=context)

    def button_cancelado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        mov_capa_ids = self.pool.get('estoque.movimento.capa').search(
            cr, uid, [('requisicaoressuprimento_id','=',ids)], context=context)
        self.pool.get('estoque.movimento.capa').write(cr, uid, mov_capa_ids, {'state': 'C'}, context=context)
        return self.write(cr, uid, ids, {'state': 'C'}, context=context)

    def button_aprovado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        itens_lancados = self.read(cr, uid, ids, ['requisicao_itens_ids'], context=context)
        if isinstance(itens_lancados, list):
            itens_lancados = itens_lancados[0]
        if isinstance(itens_lancados, dict):
            if not itens_lancados.get('requisicao_itens_ids'):
                raise osv.except_osv(_('Atencao'), _('É necessario que a requisicao tenha itens para ser aprovada!'))
        return self.write(cr, uid, ids, {'state': 'P'}, context=context)

    def button_separacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        return self.write(cr, uid, ids, {'state': 'E'}, context=context)

    def button_separado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        return self.write(cr, uid, ids, {'state': 'S'}, context=context)

    def button_conferencia(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        return self.write(cr, uid, ids, {'state': 'M'}, context=context)

    def button_voltar_conferencia(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return self.button_conferencia(cr, uid, ids, context=context)

    ## BOTÃO É UTILIZADO NA CONFERENCIA DA REQUISIÇÃO. ESTA PRESENTE TBM NO WORKFLOW (Luis)
    def button_conferido(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        return self.write(cr, uid, ids, {'state': 'F'}, context=context)

    def button_entregue(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['ressuprimento_alterar'] = True
        mov_capa_ids = self.pool.get('estoque.movimento.capa').search(
            cr, uid, [('requisicaoressuprimento_id','=',ids)], context=context)
        self.pool.get('estoque.movimento.capa').write(cr, uid, mov_capa_ids, {'state': 'F'}, context=context)
        return self.write(cr, uid, ids, {'state': 'O'}, context=context)

    def _get_produto_grade_ids(self, dados_itens_req, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        produto_ids = []
        grade_ids = []
        for i in dados_itens_req:
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))

        return  list(set(produto_ids)), list(set(grade_ids))

    def _get_campos_itens(self, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        return ['produto_id', 'grade_id', 'quantidade_confirmada', 'embalagem_id']

    def _get_dados_req_itens(self, cr, uid, itens_ids, context=None, load=''):
        context = context and context.copy() or {}
        campos = self._get_campos_itens()
        dados_itens_req = self.pool.get('estoqueressuprimento.requisicao.itens').read(cr, uid,
                                                     itens_ids,  campos, context=context, load=load)
        produto_ids, grade_ids = self._get_produto_grade_ids(dados_itens_req, context)
        return dados_itens_req, produto_ids, grade_ids

    def _get_dict_embalagesns(self, cr, uid, produto_ids, context=None, load=''):
        context = context and context.copy() or {}
        res = {}
        if not produto_ids:
            return res

        ojb_emb = self.pool.get('estoque.produto.embalagem')
        emb_ids = ojb_emb.search(cr, uid, [('produto_id', 'in', produto_ids)], context=context)
        map(lambda x: res.update({x.get('produto_id'): x.get('id')}),
            ojb_emb.read(cr, uid, emb_ids, ['produto_id'], context=context, load=load))
        return res

    def _get_embalagem(self, tipo_movimento, item, dict_emb, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        return item.get('embalagem_id')

    def _get_produto_grade_id(self, tipo_movimento, item, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        return item.get('produto_id'), item.get('grade_id')

    def _get_qtd_item(self, tipo_movimento, item, qtd_emb, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        return item.get('quantidade_confirmada') * qtd_emb

    def button_gerar_nota_ressuprimento(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        # Validar requisicao
        erros = self._validar_requisicao(cr, uid, ids[0], context)
        if erros:
            raise osv.except_osv(_('Atencao!'),
                                 ';\n'.join(map(lambda m: '%s. %s' % ((erros.index(m) + 1), m),
                                                erros)) + '.')

        # Se a empresas forem diferentes gera a nota fiscal. Senao, somente atualiza o saldo dos
        # locais
        load = '_classic_write'
        campos = ['local_entrada_id', 'local_saida_id', 'resrca_empresa_entrada_id', 'company_id',
                  'requisicao_itens_ids']
        dados_ressuprimento = self.read(cr, uid, ids[0], campos, context=context, load=load)
        if dados_ressuprimento.get('company_id') != dados_ressuprimento.get('resrca_empresa_entrada_id'):
            vals = {'nf_id':self._gerar_nota_fiscal(cr, uid, ids[0], context)}
            self.write(cr, uid, ids, vals, context)
            return self.button_entregue( cr, uid, ids, context)
        else:
            # TODO: Transformar em metodo
            obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
            obj_estoqueressuprimento_requisicao_itens = self.pool.get(
                'estoqueressuprimento.requisicao.itens')
            dict_empresa_local = dict(
                E=dict(empresa_id=dados_ressuprimento.get('resrca_empresa_entrada_id'),
                       local_id=dados_ressuprimento.get('local_entrada_id')),
                S=dict(empresa_id=dados_ressuprimento.get('company_id'),
                       local_id=dados_ressuprimento.get('local_saida_id')))
            obj_estoque_movimento_capa = self.pool.get('estoque.movimento.capa')
            obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
            obj_estoque_parametro = self.pool.get('estoque.parametro')
            obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, self._name, context=context)
            estoque_parametro_id = obj_estoque_parametro.search(
                cr, uid, [('company_id', '=', company_id)], context=context)
            if estoque_parametro_id:
                estpar_ressuprimento_operacao_entrada_id = obj_estoque_parametro.browse(
                    cr, uid,
                    estoque_parametro_id[0]).estpar_ressuprimento_operacao_entrada_id.id
                if not estpar_ressuprimento_operacao_entrada_id:
                    raise osv.except_osv(_('Atencao'),
                                         _('A "Operacao para ressuprimento(Entrada)" nao '
                                           'esta cadastrada nos parametros do estoque'))
                estpar_ressuprimento_operacao_saida_id = obj_estoque_parametro.browse(
                    cr, uid,
                    estoque_parametro_id[0]).estpar_ressuprimento_operacao_saida_id.id
                if not estpar_ressuprimento_operacao_saida_id:
                    raise osv.except_osv(
                        _('Atencao'),
                        _('A "Operacao para ressuprimento(Saida)" nao esta cadastrada nos '
                          'parametros do estoque'))
            else:
                raise osv.except_osv(_('Atencao'),
                                     _('Os parametros do estoque nao estao cadastrados'))

            dados_itens_req , produto_ids, grade_ids = self._get_dados_req_itens(cr, uid,
                                     dados_ressuprimento.get('requisicao_itens_ids'), context, load)
            dict_embalagens = self._get_dict_embalagesns(cr, uid, produto_ids, context, load)
            qtd_embalagens = self.pool.get('estoque.utils').get_grade_qtd_embalagem(cr, grade_ids,
                                                                                    context)
            sql_insert = """
                INSERT INTO estoque_movimento_itens
                    (create_date, create_uid, capa_id, produto_id, grade_id, quantidade,
                     local_id, produto_embalagem_id, vr_unitario)
                VALUES
            """
            values_insert = list()

            # Gera movimento
            for i in dict_empresa_local.keys():
                if i == 'E':
                    mov_capa_id = obj_estoque_movimento_capa.create(
                        cr, uid, {
                            'company_id': dict_empresa_local[i]['empresa_id'],
                            'data': strftime('%Y-%m-%d'),
                            'estmvc_operacao_id': estpar_ressuprimento_operacao_entrada_id,
                            'funcionario_id': self.pool.get('rh.funcionario').get_id_func(cr, uid),
                            'requisicaoressuprimento_id': ids[0]
                        }, context)
                elif i == 'S':
                    mov_capa_id = obj_estoque_movimento_capa.create(
                        cr, uid, {
                            'company_id': dict_empresa_local[i]['empresa_id'],
                            'data': strftime('%Y-%m-%d'),
                            'estmvc_operacao_id': estpar_ressuprimento_operacao_saida_id,
                            'funcionario_id': self.pool.get('rh.funcionario').get_id_func(cr, uid),
                            'requisicaoressuprimento_id': ids[0]
                        }, context)
                self.write(cr, uid, ids, {'movimento_capa_id': mov_capa_id}, context=context)

                dict_preco_venda = obj_estoque_preco_venda.get_preco_venda(
                    cr, uid, produto_ids, epc_grade_ids=grade_ids,
                    epc_company_id=dict_empresa_local[i]['empresa_id'], context=context)

                grade_qtd = {}
                for item in filter(lambda q: q.get('quantidade_confirmada') > 0, dados_itens_req):
                    embalagem_id = self._get_embalagem(i, item, dict_embalagens, context)
                    produto_id, grade_id = self._get_produto_grade_id(i, item, context)

                    dados_preco_venda = dict_preco_venda.get(produto_id) or {}
                    if not dados_preco_venda:
                        msg = 'Nao foi localizado preco de venda para o produto %s'
                        raise osv.except_osv(_('Atencao!'), (_(msg) % produto_id))

                    vr_unitario = dados_preco_venda.get(grade_id) or \
                        dados_preco_venda.get('preco_venda')

                    chave = '%s-%s' % (grade_id, embalagem_id)
                    qtd_na_embalagem = qtd_embalagens.get(chave).get('qtd') if qtd_embalagens.get(
                        chave) else 1
                    qtd = self._get_qtd_item(i, item, qtd_na_embalagem, context)
                    values_insert.append("('%s', %s, %s, %s, %s, %s, %s, %s, %s)" %
                                         (strftime('%Y-%m-%d %H:%M:%S.%s'), uid,
                                          mov_capa_id, produto_id, grade_id,
                                          qtd,
                                          dict_empresa_local[i]['local_id'],
                                          embalagem_id, vr_unitario))

                    if grade_id in grade_qtd:
                        grade_qtd[grade_id] += qtd
                    else:
                        grade_qtd[grade_id] = qtd

                if grade_qtd: # executa uma vez a funcao para atualizar o saldo fazendo em massa
                    obj_estoque_produto_saldo.atualizar_saldo_do_dia(
                        cr, uid, dict_empresa_local[i]['empresa_id'],
                        grade_qtd,
                        strftime('%Y-%m-%d'), i,
                        # Entrada ou Saida
                        dict_empresa_local[i]['local_id'],
                        context=context)

            if values_insert:
                sql_insert += ','.join(values_insert)
                cr.execute(sql_insert)

            return self.button_entregue(cr, uid, ids, context=context)

        return False

    def _gerar_nota_fiscal(self, cr, user, requisicao, context=None):
        """
        Cria a nota fiscal de transferencia.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param requisicao: ID da requisicao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type requisicao: long
        :type context: dict
        :return: o ID da nota fiscal de tranferencia.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        dados_requisicao = self.read(cr, user, requisicao, [], context, load)
        context.update({'sig_force_company': True})
        parametro_fiscal = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'fiscal', [], context=context, company_id=dados_requisicao.get('company_id'))

        if not parametro_fiscal.get('modelo_nf_series_id'):
            raise osv.except_osv(_("Atencao!"),
                                 _("Parametro do fiscal 'Modelo padrao' nao esta cadastrado!"))

        if not parametro_fiscal.get('fispar_natureza_transferencia_saida_id'):
            raise osv.except_osv(_("Atencao!"),
                                 _("Parametro do fiscal 'Natureza para saida por transferencia' nao esta cadastrado!"))

        if not parametro_fiscal.get('condicao_pgto_transferencia_id'):
            raise osv.except_osv(_("Atencao!"),
                                 _("Parametro do fiscal 'Condicao de Pgto. p/ Transferencias de NF' nao esta cadastrado!"))

        # Para compatibilidade. Excluir se nao utilizado
        context['diff_company_id'] = dados_requisicao.get('company_id')

        # Parceiro (pessoa_id)
        company_partner = self.pool.get('res.company').read(cr, user,
                            dados_requisicao.get('resrca_empresa_entrada_id'), ['rescon_pessoa_id'], context, load)
        campos = ['produto_id', 'grade_id', 'quantidade_confirmada', 'embalagem_id']
        dados_itens = self.pool.get('estoqueressuprimento.requisicao.itens').read(cr, user,
                                dados_requisicao.get('requisicao_itens_ids'), campos, context, load)
        produto_ids = map(lambda x: x.get('produto_id'), dados_itens)
        campos = ['tipo', 'origem_mercadoria']
        dict_produtos = {}
        map(lambda x: dict_produtos.update({x.get('id'): x}),
            self.pool.get('estoque.produto').read(cr, user, produto_ids, campos, context))
        obj_pessoa = self.pool.get('adm.pessoa')
        campos = ['cpf_cnpj', 'endereco_ids', 'admpes_jur_tipo_atividade']
        dados_destinatario = obj_pessoa.read(cr, user, company_partner.get('rescon_pessoa_id'),
                                             campos, context)
        campos = ['cfop_id', 'natureza_encargos_ids', 'tipo']
        dados_natureza = self.pool.get('fiscal.cfop.natureza').read(cr, user,
                         parametro_fiscal.get('fispar_natureza_transferencia_saida_id'), campos,
                                                                    context, load)
        campos = ['movimento_tipo', 'codigo', 'sigla']
        dados_cfop_nat = self.pool.get('fiscal.cfop').read(cr, user, dados_natureza.get( 'cfop_id'),
                                                           campos, context)
        campos = ['encargo_id', 'sobrepor_tributacao_id', 'fiscne_sobrepoe_so_tributados']
        dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(cr, user,
                                                  dados_natureza.get('natureza_encargos_ids'),
                                                  campos, context, load)
        encargo_ids = map(lambda x: x.get('encargo_id'), dados_nat_encargo)
        dict_encargos_nat = {}
        map(lambda x: dict_encargos_nat.update({x.get('id'): x}),
            self.pool.get('fiscal.encargo').read(cr, user, encargo_ids, ['sigla'], context))

        # Itens da nota fiscal
        context['proprio'] = True  # Utilizado no calculo das tributacoes
        nf_itens = []
        vr_total_itens = 0
        dados_itens = filter(lambda di: di.get('quantidade_confirmada'), dados_itens)
        for item in dados_itens:
            vr_unitario = self._get_valor_unitario(cr, user, item.get('produto_id'), dados_requisicao.get('company_id'), context) or 0.01
            vr_total = vr_unitario * item.get('quantidade_confirmada')
            tributacoes = self.pool.get('estoque.movimento.itens').calcular_tributo_itens(cr, user,
                                   dict_produtos.get(item.get('produto_id')), dados_natureza, dados_nat_encargo, dados_cfop_nat,
                                   encargo_ids, dados_destinatario, 'T', vr_total, context=context)
            vals = {'local_id': dados_requisicao.get('local_saida_id'),
                    'produto_id': item.get('produto_id'),
                    'grade_id': item.get('grade_id'),
                    'cfop_id': dados_natureza.get('cfop_id'),
                    'produto_embalagem_id': item.get('embalagem_id'),
                    'quantidade': item.get('quantidade_confirmada'),
                    'tipo': 'P',
                    'desconto': 0,
                    'vr_unitario': vr_unitario,
                    'vr_total_item': vr_total,
                    'tributacoes_ids': tributacoes}
            nf_itens.append([0 , 0, vals])
            vr_total_itens += vr_total

        # Condicao de pagamento utilizada para transferencia (apenas uma condicao)
        condicao_transferencia_id = parametro_fiscal.get('condicao_pgto_transferencia_id')
        brw_condicao_forma = self.pool.get('adm.condicao.forma').browse(
            cr, user, condicao_transferencia_id, context)
        nf_condicao_ids = [[0, 0, dict(
            forma_pagto_id=brw_condicao_forma.forma_pagamento_id.id,
            condicao_forma_id=condicao_transferencia_id, valor=vr_total_itens)]]

        # Parcelas da condicao
        nf_condicao_ids[0][2]['nf_parcela_ids'] = [
            [0, 0, p] for p in self.pool.get('adm.condicao').gerar_parcelas(
                cr, user, brw_condicao_forma.adm_condicao_id.id, vr_total_itens,
                strftime('%Y-%m-%d'), context)]

        # Gera nota fiscal de transferencia (saida)
        brw_modelo = self.pool.get('fiscal.modelo.nf.series').browse(
            cr, user, parametro_fiscal.get('modelo_nf_series_id'), context)
        destinatario_id = company_partner.get('rescon_pessoa_id')
        destinatario_endereco_ids = self.pool.get('adm.pessoa.endereco').search(
            cr, user, [('pessoa_id', '=', destinatario_id)], context=context)
        endereco_id = destinatario_endereco_ids and destinatario_endereco_ids[0]
        forma_emissao = '1' if not parametro_fiscal.get('fispar_nfe_contingencia') else '2'
        context['nota_fiscal_saida'] = True
        result = self.pool.get('fiscal.nf.capa').create(
            cr, user, dict(modelo_nf_series_id=brw_modelo.id, tipo='S', nf_propria=True,
                           modelo_55=brw_modelo.modelo_nf_id.fismnf_tipo_nfe,
                           nfe_forma_emissao=forma_emissao, finalidade_emissao='1',
                           company_id=dados_requisicao.get('company_id'),
                           data_emissao=strftime('%Y-%m-%d %H:%M:%S'),
                           nfe_forma_impressao_danfe='1',
                           natureza_id=parametro_fiscal.get(
                               'fispar_natureza_transferencia_saida_id'),
                           destinatario_id=destinatario_id, destinatario_endereco_id=endereco_id,
                           nf_condicao_ids=nf_condicao_ids, estoque_movimento_itens_ids=nf_itens),
            context)

        return result

    def onchange_empresa_saida_id(self, cr, uid, ids, company_id, context=None):
        """
        Ao trocar a empresa de saida, limpa o local de saida
        """
        context = context and context.copy() or {}
        res = {'value': {'local_saida_id': False}}
        if company_id:
            a_usar_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'estoque.local', context=context, user_company_id=company_id)
            res['value'].update({'resrca_a_usar_company_id': a_usar_company_id})
        return res

    def onchange_local_saida(self, cr, uid, ids, local_saida_id, context=None):
        """
        Ao trocar o local de saida, limpa o local de entrada
        """
        context = context and context.copy() or {}
        return dict(value=dict(local_entrada_id=False))

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('troca_mercadoria', False):
            vals['numero'] = self.pool.get('adm.utils').get_sequence(
                cr, uid, self._name, 'numero_troca_mercadoria.troca')
        else:
            vals['numero'] = self.pool.get('adm.utils').get_sequence(
                cr, uid, self._name, 'numero_requisicao.ressuprimento')

        # este contexto esta sendo passado na action do ressuprimento, nao passei 'sig_force_company'
        # diretamente pois o mesmo seria passado para todas as outras funcoes.
        if context.get('usar_sig_force'):
            context['sig_force_company'] = True

        return super(estoqueressuprimento_requisicao_capa, self).create(cr, uid, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        # Nao permitir excluir requisicoes
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao!'), _('A requisicao nao pode ser excluida!'))

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if context.get('usar_sig_force'):
            context['sig_force_company'] = True
        if context.get('ressuprimento_alterar'):
            super(estoqueressuprimento_requisicao_capa, self).write(cr, uid, ids, vals, context=context)
            return True
        # Nao permitir alterar a requisicao se nao estiver no ´status´ "R" ou nao possuir o contexto
        # 'ressuprimento_alterar'
        if isinstance(context, dict):
            state = ''
            state_requisicao = self.read(cr, uid, ids, ['state'], context=context)
            if isinstance(state_requisicao, list):
                state_requisicao = state_requisicao[0]
            if isinstance(state_requisicao, dict):
                state = state_requisicao.get('state') or ''
            if context.get('ressuprimento_alterar') or state == 'R' or not context:
                res = super(estoqueressuprimento_requisicao_capa, self).write(cr, uid, ids, vals, context=context)
            else:
                if not context.get('atualizar_capa'):
                    raise osv.except_osv(_('Atencao'),
                                         _('A requisicao nao pode ser alterada, pois nao encontra-se '
                                           'em esboco!'))
        return res

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        if args is None:
            args = []
        ids = self.search(cr, user, args, limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if not args:
            args = []
        if isinstance(context, dict):
            # Permite pesquisar em mais de um dos status da search view. (Luis)
            if context.get('ressupri_filter', False):
                index_remove = []
                states = []
                for i in args:
                    if 'state' in i:
                        try:
                            states.append(i[2])
                        except:
                            pass
                        index_remove.append(args.index(i))
                index_remove.reverse()  # NÃO RETIRAR O REVERSE, SEM ELE PODE GERAR IndexError. (Luis)
                for r in index_remove:
                    args.pop(r)
                args.append(('state', 'in', states))
        context.update({'sig_force_company': True})
        return super(estoqueressuprimento_requisicao_capa, self).search(
            cr, uid, args, offset, limit, order, context, count)

    def _validar_requisicao(self, cr, user, requisicao, context=None):
        """
        Valida os dados da requisicao.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil,
        :param requisicao: o ID da requisicao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type requisicao: long
        :type context: dict
        :return: lista com os erros encontrados na validacao da requisicao.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        brw_requisicao = self.browse(cr, user, requisicao, context)
        if brw_requisicao.company_id.id == brw_requisicao.resrca_empresa_entrada_id.id:
            requisicao_entre_lojas = False
        else:
            requisicao_entre_lojas = True
        parametro_fiscal = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'fiscal', [], context=context)
        parametro_estoque = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'estoque', [], context=context)
        result = list()  # Lista de erros

        # Verifica se o usuario logado esta cadastrado como funcionario
        if not self.pool.get('rh.funcionario').get_id_func(cr, user):
            result.append(_('Usuario nao esta cadastrado como funcionario'))

        # Verifica a condicao de pagamento utilizada para transferencia
        if not parametro_fiscal.get('condicao_pgto_transferencia_id') and requisicao_entre_lojas:
            result.append(_('A condicao de pagamento para transferencia nao esta cadastrada no '
                            'parametro do Fiscal'))

        # Verifica a natureza da operacao utilizada para transferencia
        if not parametro_fiscal.get('fispar_natureza_transferencia_saida_id') and \
                requisicao_entre_lojas:
            result.append(_('A natureza para transferencia nao esta cadastrada no parametro do '
                            'Fiscal; '))

        # Verifica se o parceiro da empresa de saida foi selecionado
        if not self.pool.get('res.company').read(
                cr, user, brw_requisicao.company_id.id, ['rescon_pessoa_id'], context=context):
            result.append(_('Nao foi localizado nenhum parceiro cadastrado para a empresa %s') %
                          brw_requisicao.company_id.name)

        # Verifica se a emrpesa possui endereco cadastrado para nota fiscal
        adm_pessoa_endereco_nf = self.pool.get('adm.pessoa').read(
            cr, user, brw_requisicao.company_id.rescon_pessoa_id.id, ['endereco_nf_id'],
            context=context)
        if not adm_pessoa_endereco_nf.get('endereco_nf_id'):
            result.append(_('A empresa %s nao possui endereco cadastrado para nota fiscal') %
                          brw_requisicao.company_id.name)

        # Recupera o valor unitario em funcao do parametro ´estpar_preco_para_transferencia´
        if not parametro_estoque.get('estpar_preco_para_transferencia') and requisicao_entre_lojas:
            result.append(_('Preco para tranferencia nao foi definido no parametro do Estoque'))

        return result

    def _get_valor_unitario(self, cr, user, item, empresa, context=None):
        """
        Recupera o valor unitario em funcao do parametro ´estpar_preco_para_transferencia´
        """
        context = context and context.copy() or {}
        parametro_estoque = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'estoque', [], context=context, company_id=empresa)

        # Se for preco de venda recupera o valor de ´get_preco_venda´...
        preco_para_transferencia = parametro_estoque.get('estpar_preco_para_transferencia')
        if preco_para_transferencia == 'V':
            result = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, user, item, empresa, context=context).get(item).get('preco_venda')
        else:  # ...se nao, recupera o preco de ´get_custos_do_dia´
            result = self.pool.get('estoque.preco.custo').get_custos_do_dia(
                cr, user, item, empresa, context=context).get(item).get(
                dict(U='estprc_ultimo_custo', M='estprc_custo_medio_contabil',
                     R='estprc_custo_reposicao',
                     C='estprc_custo_corrigido').get(preco_para_transferencia)) or 0

        return result

    def resrca_button_tipo_codigo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('resrca_interno'):
            vals = {'resrca_tipo_codigo': 'I'}
        else:
            vals = {'resrca_tipo_codigo': 'B'}
        self.write(cr, uid, ids, vals, context)
        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
