# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from time import strftime
from openerp.tools.translate import _


class estoqueressuprimento_requisicao_itens(osv.Model):
    _name = 'estoqueressuprimento.requisicao.itens'
    _inherit = 'estoqueressuprimento.requisicao.itens'

    def default_get(self, cr, uid, fields_list, context=None):
        res = super(estoqueressuprimento_requisicao_itens, self).default_get(cr, uid, fields_list,
                                                                             context)
        if context.get('tipo_codigo'):
            res['tipo_codigo'] = context.get('tipo_codigo')
        return res

    def fields_get(self, cr, user, fields=None, context=None):
        """
        CRIA UM CAMPO VIRTUAL APENAS PEGAR O SALDO CALCULO
        FOI CRIADO PARA NÃO SER PRECISO RECALCULAR O SALDO QUANDO FOR GRAVADO,
        POIS O CAMPO É READONLY E NÃO É PASSADO NO VALUES
        """
        context = context and context.copy() or {}
        res = super(estoqueressuprimento_requisicao_itens, self).fields_get(cr, user, fields,
                                                                            context)
        res['saldo_temp'] = {'digits': (9, 3), 'selectable': True, 'type': 'float',
                             'string': u'Saldo Memory',
                             'required': False}
        return res

    def get_local_dados(self, cr, uid, local_id, quantidade_requisitada, produto_id, grade_id,
                        company_id, context=None):
        context = context and context.copy() or {}
        res = {'saldo_atual': 0}
        if isinstance(local_id, (list, tuple)):
            local_id = local_id[0]
        local_dados = self.pool.get('estoque.local').read(cr, uid, local_id,
                                                          ['aceita_saldo_negativo', 'active'], context=context)
        if isinstance(local_dados, list):
            local_dados = local_dados[0]
        if isinstance(local_dados, dict):
            ativo = local_dados.get('active') or False
            if not ativo:
                msg = 'O local de saida informado nao encontra-se ativo! \
                       Para ativar o local, acesse Cadastros > Estoque > Locacoes e Embalagens > Locais'
                return {'warning': {'title': _('Atencao'), 'message': _(msg)}}
            else:
                saldo_negativo = local_dados.get('aceita_saldo_negativo') or False
                if quantidade_requisitada:
                    if not isinstance(quantidade_requisitada, (int, long, float)):
                        msg = 'Informe somente numeros para a quantidade requisitada!'
                        return {'warning': {'title': _('Atencao'), 'message': _(msg)}}

                obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
                context['company_id'] = company_id
                get_saldo_atual = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, grade_id, strftime('%Y-%m-%d'),
                                                                             local_id, context=context)
                saldo_atual = get_saldo_atual.get(grade_id).get('saldo')
                res['saldo_atual'] = saldo_atual

                # SE O LOCAL ACEITAR SALDO NEGATIVO....
                if not saldo_negativo:
                    # VERIFICAR SE O SALDO ATUAL É MENOR QUE ZERO. SE FOR NÃO PODE SER REQUISITADO
                    if saldo_atual < 0:
                        msg = 'O saldo do produto selecionado encontra-se negativo. \
                        O local de saida informado nao aceita saldo negativo!'
                        return {'warning': {'title': _('Atencao'), 'message': _(msg)}}
                    elif quantidade_requisitada:
                        # SE A QUANTIDADE REQUISITADA FOR MAIOR QUE O SALDO DISPONIVEL,
                        # NÃO PODE SER REQUISITADO
                        if round(saldo_atual < float(quantidade_requisitada), 3):
                            msg = 'O local de saida nao aceita saldo negativo. \
                            Quantidade requisitada %0.3f, superior ao saldo disponivel %0.3f!'
                            return {'warning': {'title': _('Atencao'), 'message': _(msg) % (
                                quantidade_requisitada, saldo_atual)}}
        return res

    def get_embalagem_unidade(self, cr, uid, produto_embalagem_id, context=None):
        """
        BUSCA A QUANTIDADE NA EMBALAGEM E A UNIDADE UTILIZADA
        RETORNA A QUANTIDADE NA EMBALAGEM E SE A UNIDADE ACEITA FRACIONAMENTO
        """
        context = context and context.copy() or {}
        unidade_fracao = False
        quantidade_embalagem = 1
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        produto_embalagem_dados = obj_estoque_produto_embalagem.read(cr, uid, produto_embalagem_id,
                                                                     ['embalagem_id'], context=context)
        if isinstance(produto_embalagem_dados, list):
            produto_embalagem_dados = produto_embalagem_dados[0]
        if isinstance(produto_embalagem_dados, dict):
            try:
                embalagem_id = produto_embalagem_dados['embalagem_id'][0]
            except:
                embalagem_id = []
            if embalagem_id:
                obj_estoque_embalagem = self.pool.get('estoque.embalagem')
                embalagem_dados = obj_estoque_embalagem.read(cr, uid, embalagem_id,
                                                             ['quantidade_na_embalagem',
                                                              'unidade_id'], context=context)
                if isinstance(embalagem_dados, list):
                    embalagem_dados = embalagem_dados[0]
                if isinstance(embalagem_dados, dict):
                    # PEGA A QUANTIDADE NA EMBALAGEM
                    quantidade_embalagem = embalagem_dados.get('quantidade_na_embalagem') or 1
                    try:
                        unidade_id = embalagem_dados['unidade_id'][0]
                    except:
                        unidade_id = []

                    if unidade_id:
                        obj_estoque_embalagem_unidade = self.pool.get('estoque.embalagem.unidade')
                        unidade_dados = obj_estoque_embalagem_unidade.read(cr, uid, unidade_id,
                                                                           ['fracao'], context=context)
                        if isinstance(unidade_dados, list):
                            unidade_dados = unidade_dados[0]
                        if isinstance(unidade_dados, dict):
                            # BUSCA SE A UNIDADE CADASTRADA ACEITA FRAÇÃO
                            unidade_fracao = unidade_dados.get('fracao') or False

        return {'quantidade_na_embalagem': quantidade_embalagem, 'fracao': unidade_fracao}

    def get_produto_in_locais(self, cr, uid, produto_id, grade_id, entrada_local,
                              entrada_company_id, saida_local,
                              saida_company_id, context=None):
        """
        VERIFICA SE O PRODUTO E A GRADE SELECIONADA ESTAO PRESENTES NO LOCAL DE ENTRADA E DE SAIDA
        """
        context = context and context.copy() or {}
        context['sig_force_company'] = True
        if isinstance(produto_id, (list, tuple)):
            produto_id = produto_id[0]
        if isinstance(grade_id, (list, tuple)):
            grade_id = grade_id[0]
        if isinstance(entrada_local, (list, tuple)):
            entrada_local = entrada_local[0]
        if isinstance(entrada_company_id, (list, tuple)):
            entrada_company_id = entrada_company_id[0]
        if isinstance(saida_local, (list, tuple)):
            saida_local = saida_local[0]
        if isinstance(saida_company_id, (list, tuple)):
            saida_company_id = saida_company_id[0]
        obj_local_locacao = self.pool.get('estoque.local.locacao')
        saida_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context, user_company_id=saida_company_id)
        entrada_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context, user_company_id=entrada_company_id)
        entrada = obj_local_locacao.search(cr, uid, [('produto_id', '=', produto_id),
                                                     ('grade_id', '=', grade_id),
                                                     ('company_id', '=', entrada_company_id),
                                                     ('local_id', '=', entrada_local)], context=context)

        saida = obj_local_locacao.search(cr, uid, [('produto_id', '=', produto_id),
                                                   ('grade_id', '=', grade_id),
                                                   ('company_id', '=', saida_company_id),
                                                   ('local_id', '=', saida_local)], context=context)
        msg = ''
        if entrada and saida:
            return True
        elif not entrada and not saida:
            msg = ("Saida e Entrada")
        else:
            msg = _("Entrada") if not entrada else _("Saida")
        if msg:
            raise osv.except_osv(_('Atencao'), _('O produto e a grade selecionados, \
            nao encontram-se cadastrados para o local de %s!') % msg)
        return False

    def get_saldo_local(self, cr, uid, ids, empresa_saida_id, local_saida_id, produto_id, grade_id,
                        produto_embalagem_id, quantidade_requisitada=0, context=None):
        """
        VERIFICA SE O PRODUTO, GRADE, EMPRESA E LOCAL DE SAIDA ESTÃO CADASTRADOS.
        VERIFICA SE O LOCAL PERMITE SALDO NEGATIVO E SE ESTA ATIVO
        RETORNA O SALDO DIVIDO PELA QUANTIDADE DA EMBALAGEM
        """
        context = context and context.copy() or {}
        values = {}
        if empresa_saida_id and local_saida_id and produto_id:
            args_local = [('company_id', '=', empresa_saida_id), ('local_id', '=', local_saida_id),
                          ('produto_id', '=', produto_id)]
            if grade_id:
                args_local.append(('grade_id', '=', grade_id))
            obj_local_locacao = self.pool.get('estoque.local.locacao')
            # verifica se existe local.locacao cadastrado para o produto, local, embalagem, grade
            # e company selecionado.
            local_locacao_saida_ids = obj_local_locacao.search(cr, uid, args_local, context=context)
            if not local_locacao_saida_ids:
                msg = 'O PRODUTO selecionado nao encontram-se cadastrados para o LOCAL DE SAIDA escolhido!'
                return {'warning': {'title': _('Atencao'), 'message': _(msg)}}
            elif produto_embalagem_id:
                local_dados = self.get_local_dados(cr, uid, local_saida_id, quantidade_requisitada,
                                                   produto_id,
                                                   grade_id, empresa_saida_id, context={})
                if isinstance(local_dados, dict):
                    if local_dados.get('warning'):
                        return local_dados
                    else:
                        saldo_atual = local_dados.get('saldo_atual') or 0
                        values['saldo'] = 0
                        values['saldo_temp'] = 0
                        embalagem = self.get_embalagem_unidade(cr, uid, produto_embalagem_id,
                                                               context={})
                        if isinstance(embalagem, dict):
                            # VERIFICA SE A QUANITDADE PODE SER FRACIONADA
                            if not embalagem.get('fracao'):
                                quantidade_float = quantidade_requisitada % 1
                                if quantidade_float:
                                    msg = 'A embalagem selecionada nao aceita uma quantidade fracionada!'
                                    return {'warning': {'title': _('Atencao'), 'message': _(msg)}}
                            # TRATA A QUANTIDADE NA EMBALAGEM
                            saldo_atual /= embalagem.get('quantidade_na_embalagem') or 1
                            values['saldo'] = saldo_atual
                            values['saldo_temp'] = saldo_atual
        return values

    def onchange_produto_id(self, cr, uid, ids, empresa_saida_id, local_saida_id, produto_id,
                            grade_id, produto_embalagem_id, tipo_codigo, quantidade_requisitada=0,
                            context=None):
        context = context and context.copy() or {}
        empresa_saida_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context, user_company_id=empresa_saida_id)
        context['sig_force_company'] = True
        local_grade_id = []
        local_embalagem_id = []
        val_fields = {'quantidade_requisitada': 0, 'saldo': 0, 'saldo_temp': 0}
        if produto_id and local_saida_id and tipo_codigo == 'I':
            val_fields.update({'grade_id': False, 'embalagem_id': False})
            obj_local_locacao = self.pool.get('estoque.local.locacao')
            args_local_produto = [('company_id', '=', empresa_saida_id),
                                  ('produto_id', '=', produto_id),
                                  ('local_id', '=', local_saida_id)]
            search_grades_ids = obj_local_locacao.search(cr, uid, args_local_produto, context=context)

            if len(search_grades_ids) == 1:
                local_grade = obj_local_locacao.read(cr, uid, search_grades_ids[0], ['grade_id'], context=context)

                if isinstance(local_grade, list):
                    local_grade = local_grade[0]
                if isinstance(local_grade, dict):
                    try:
                        local_grade_id = local_grade['grade_id'][0]
                    except:
                        pass
                    val_fields['grade_id'] = local_grade_id
                if local_grade_id:
                    args_local_produto.append(('grade_id', '=', local_grade_id))
                    search_embalagem_ids = obj_local_locacao.search(cr, uid, args_local_produto, context=context)
                    if search_embalagem_ids:
                        local_embalagem = obj_local_locacao.read(cr, uid, search_embalagem_ids[0],
                                                                 ['estllc_produto_embalagem_id'], context=context)
                        if isinstance(local_embalagem, list):
                            local_embalagem = local_embalagem[0]
                        if isinstance(local_embalagem, dict):
                            try:
                                local_embalagem_id = local_embalagem['estllc_produto_embalagem_id'][
                                    0]
                            except:
                                pass
                            val_fields['embalagem_id'] = local_embalagem_id
                # CASO O LOCAL NÃO POSSUA UMA EMBALAGEM CADASTRADA
                # É PESQUISADO QUAIS O PRODUTO POSSUI.
                if not local_embalagem_id:
                    obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
                    estoq_produto_embalagem_ids = obj_estoque_produto_embalagem.search(cr, uid, [
                        ('produto_id', '=', produto_id)], context=context)
                    if estoq_produto_embalagem_ids:
                        local_embalagem_id = estoq_produto_embalagem_ids[0]
                        val_fields['embalagem_id'] = estoq_produto_embalagem_ids[0]
        elif tipo_codigo == 'I':
            return {'value': {'grade_id': False, 'embalagem_id': False, 'quantidade_requisitada': 0,
                              'saldo': 0,
                              'saldo_temp': 0}}
        saldo = self.get_saldo_local(cr, uid, ids, empresa_saida_id, local_saida_id, produto_id,
                                     local_grade_id or grade_id,
                                     local_embalagem_id or produto_embalagem_id,
                                     quantidade_requisitada, context=context)
        if saldo and isinstance(saldo, dict):
            if saldo.get('warning'):
                return saldo
            else:
                val_fields['saldo'] = saldo.get('saldo') or 0
                val_fields['saldo_temp'] = saldo.get('saldo_temp') or 0
        return {'value': val_fields}

    def onchange_calcular_saldo(self, cr, uid, ids, entrada_company_id, empresa_saida_id,
                                local_entrada_id, local_saida_id, produto_id, grade_id,
                                produto_embalagem_id, quantidade_requisitada, tipo_codigo,
                                context=None):
        """
        AO SELECIONAR UM PRODUTO PREENCHE A GRADE, EMBALAGEM DO PRODUTO
        VERIFICA SE O PRODUTO, GRADE E EMBALAGEM ESTÃO NO LOCAL DE SAIDA.
        """
        context = context and context.copy() or {}
        values = {}
        empresa_saida_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context, user_company_id=empresa_saida_id)
        entrada_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao', context=context, user_company_id=entrada_company_id)
        context['sig_force_company'] = True
        if self.get_produto_in_locais(cr, uid, produto_id, grade_id, local_entrada_id,
                                      entrada_company_id,
                                      local_saida_id, empresa_saida_id, context):
            if produto_id and grade_id and produto_embalagem_id:
                saldo_local = self.get_saldo_local(cr, uid, ids, empresa_saida_id, local_saida_id,
                                                   produto_id, grade_id,
                                                   produto_embalagem_id, quantidade_requisitada,
                                                   context)
                if saldo_local and isinstance(saldo_local, dict):
                    if saldo_local.get('warning'):
                        return saldo_local
                    else:
                        values.update(saldo_local)
                        return {'value': values}
            values.update({'saldo': 0, 'saldo_temp': 0})
        if (produto_id and grade_id and tipo_codigo == 'I') and not produto_embalagem_id:
            obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
            args_local_produto = [('company_id', '=', empresa_saida_id),
                                  ('produto_id', '=', produto_id),
                                  ('local_id', '=', local_saida_id),
                                  ('grade_id', '=', grade_id)]
            locacao_ids = obj_estoque_local_locacao.search(cr, uid, args_local_produto,
                                                           context=context)
            if locacao_ids:
                values.update({'embalagem_id': obj_estoque_local_locacao.browse(
                    cr, uid, locacao_ids[0], context).estllc_produto_embalagem_id.id})
        return {'value': values}

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        # este contexto eh passado na requisicao total, pois as verificacoes feitas a baixo ja
        # foram feitas, e o write na capa para calcular os metodos funcionais seram feitos no final
        # do fluxo da resuisicao total
        if context.get('super_estoqueressuprimento_requisicao_itens'):
            return super(estoqueressuprimento_requisicao_itens, self).create(cr, uid, vals, context)

        obj_estoque_requisicao_capa = self.pool.get('estoqueressuprimento.requisicao.capa')
        # VERIFICA SE A QUANTIDADE REQUISITADA QUE FOI DIGITADA É MAIOR QUE ZERO
        if vals.get('quantidade_requisitada') == 0:
            raise osv.except_osv(_('Atencao'), _('A quantidade lancada deve ser maior que zero!'))

        atualizou = False
        # BUSCA A EMPRESA E LOCAL (DE SAIDA E ENTRADA) DA CAPA
        capa_dados = obj_estoque_requisicao_capa.read(cr, uid, vals.get('requisicao_id'),
                                                      ['local_entrada_id',
                                                       'company_id',
                                                       'local_saida_id',
                                                       'resrca_empresa_entrada_id'], context=context)
        if isinstance(capa_dados, list):
            capa_dados = capa_dados[0]
        if isinstance(capa_dados, dict):
            try:
                entrada_local_id = capa_dados['local_entrada_id'][0]
                entrada_company_id = capa_dados['resrca_empresa_entrada_id'][0]
                saida_local_id = capa_dados['local_saida_id']
                saida_empresa_id = capa_dados['company_id'][0]
            except:
                entrada_local_id = []
                entrada_company_id = []
                saida_empresa_id = []
                saida_local_id = []
            # VERIFICAR SE O PRODUTO ESTA NO LOCAL DE ENTRADA E SAIDA
            if self.get_produto_in_locais(cr, uid, vals.get('produto_id'), vals.get('grade_id'),
                                          entrada_local_id,
                                          entrada_company_id, saida_local_id, saida_empresa_id, context):
                if vals.get('quantidade_requisitada'):
                    # CHAMA FUNÇÃO QUE RETORNA A QUANTIDADE NA EMBALAGEM E A UNIDADE UTILIZADA
                    embalagem = self.get_embalagem_unidade(cr, uid, vals.get('embalagem_id') or [],
                                                           context={})
                    if isinstance(embalagem, dict):
                        if not embalagem.get('fracao'):
                            # SE A EMBALAGEM NÃO PERMITIR FRACIONAMENTO E O USUÁRIO DIGITAR
                            # UMA QUANTIDADE FRACIONADA O RAISE É LEVANTADO
                            quantidade_float = vals['quantidade_requisitada'] % 1
                            if quantidade_float:
                                dados_produto = self.pool.get('estoque.produto').read(cr, uid,
                                        vals.get('produto_id'), ['codigo', 'name'], context=context)
                                msg = 'A embalagem selecionada do produto %s-%s' % \
                                      (dados_produto.get('codigo'), dados_produto.get('name')) + \
                                      ' nao aceita uma quantidade fracionada!'
                                raise osv.except_osv(_('Atencao'), _(msg))

                    # VERIFICA SE O LOCAL PERMITE LOCAL NEGATIVO, SE ENCONTRA-SE ATIVO
                    # E PERMITE FRACIONAMENTO.
                    # COM ESSES DADOS VERIFICA O SALDO, SE A QUANTIDADE É FRACIONADA
                    # E SE NÃO EXCEDE A QUANTIDADE DO LOCAL - CASO NÃO PERMITA.
                    local_dados = self.get_local_dados(cr, uid, saida_local_id,
                                                       vals['quantidade_requisitada'],
                                                       vals.get('produto_id'), vals.get('grade_id'),
                                                       saida_empresa_id,
                                                       context)
                    if isinstance(local_dados, dict):
                        if local_dados.get('warning'):
                            local_dados = local_dados['warning']
                            raise osv.except_osv((local_dados.get('title') or ''), (local_dados.get(
                                'message') or _(
                                '(RESSSUPRIMENTO) - Nao foi possivel inserir o produto no grid')))

                # PASSA O VALOR DO CAMPO saldo_temp PARA O CAMPO saldo.
                # ISTO É FEITO PORQUE O CAMPO SALDO É READONLY
                # E NO CREATE O CAMPO NÃO ESTA PRESENTE DENTRO DO VALS.
                if vals.get('saldo_temp'):
                    vals['saldo'] = vals['saldo_temp']
                    del vals['saldo_temp']

                # AO CRIAR UMA REQUISIÇÃO A QUANTIDADE CONFIRMADA DEVE SER ZERO.
                # É DECLARADO AQUI POIS QUANDO OCORRE A DUPLICAÇÃO,
                # NÃO DUPLICAR A QUANTIDADE CONFIRMADA
                vals['quantidade_confirmada'] = 0
                if vals.get('requisicao_id'):
                    atualizou = True
                    obj_estoque_requisicao_capa.write(cr, uid, vals.get('requisicao_id'), {}, context=context)
                return super(estoqueressuprimento_requisicao_itens, self).create(cr, uid, vals,
                                                                                 context)
        if vals.get('requisicao_id') and not atualizou:
            obj_estoque_requisicao_capa.write(cr, uid, vals.get('requisicao_id'), {}, context=context)

        return 0

    def write(self, cr, uid, ids, values, context=None):
        # SE A QUANTIDADE CONFIRMADA (TELA DE CONFERENCIA) FOR MENOR OU
        # IGUAL A ZERO, LEVANTA O RAISE
        context = context and context.copy() or {}
        if 'quantidade_requisitada' in values and values.get('quantidade_requisitada')<=0:
            raise osv.except_osv(_('Atencao'),
                                 _('A quantidade requisitada deve ser maior que zero!'))
        if 'quantidade_confirmada' in values and values.get('quantidade_confirmada')<=0:
            raise osv.except_osv(_('Atencao'),
                                 _('A quantidade confirmada deve ser maior que zero!'))
        # BUSCA A REQUISICAO_ID, PRODUTO_ID E GRADE_ID DO REGISTRO ATUAL
        obj_estoque_requisicao_capa = self.pool.get('estoqueressuprimento.requisicao.capa')
        requisicao_capa = self.read(cr, uid, ids, ['requisicao_id', 'produto_id', 'grade_id'], context=context)
        if isinstance(requisicao_capa, list):
            requisicao_capa = requisicao_capa[0]
        atualizou = False
        requisicao_id = None
        if isinstance(requisicao_capa, dict):
            try:
                requisicao_id = requisicao_capa['requisicao_id'][0]
            except:
                requisicao_id = []
            # SE DENTRO DE VALUES NÃO TIVER PRODUTO_ID, BUSCA O ID DO PRODUTO
            if not values.get('produto_id'):
                try:
                    produto_id = requisicao_capa['produto_id'][0]
                except:
                    produto_id = []
                values['produto_id'] = produto_id
            # SE DENTRO DE VALUES NÃO TIVER GRADE_ID, BUSCA O ID DO GRADE
            if not values.get('grade_id'):
                try:
                    grade_id = requisicao_capa['grade_id'][0]
                except:
                    grade_id = []
                values['grade_id'] = grade_id

            # BUSCA A EMPRESA E LOCAL (DE SAIDA E ENTRADA) DA CAPA
            capa_dados = obj_estoque_requisicao_capa.read(cr, uid, requisicao_id,
                                                          ['local_entrada_id',
                                                           'company_id',
                                                           'local_saida_id',
                                                           'resrca_empresa_entrada_id'], context=context)
            if isinstance(capa_dados, list):
                capa_dados = capa_dados[0]
            if isinstance(capa_dados, dict):
                try:
                    entrada_local_id = capa_dados['local_entrada_id'][0]
                    entrada_company_id = capa_dados['resrca_empresa_entrada_id'][0]
                    saida_local_id = capa_dados['local_saida_id'][0]
                    saida_empresa_id = capa_dados['company_id'][0]
                except:
                    entrada_local_id = []
                    entrada_company_id = []
                    saida_empresa_id = []
                    saida_local_id = []
                # VERIFICAR SE O PRODUTO ESTA NO LOCAL DE ENTRADA E SAIDA
                if self.get_produto_in_locais(cr, uid, values.get('produto_id') or 0,
                                              values.get('grade_id') or 0,
                                              entrada_local_id, entrada_company_id, saida_local_id,
                                              saida_empresa_id, context):

                    if values.get('quantidade_requisitada'):
                        # SE NÃO TIVER A EMBALAGEM_ID DENTRO DE VALUES, É BUSCADA,
                        # POIS DEVE SER PASSADA NA FUNÇÃO get_embalagem_quantidade
                        if not values.get('embalagem_id'):
                            produto_embalagem = self.read(cr, uid, ids, ['embalagem_id'], context=context)
                            if isinstance(produto_embalagem, list):
                                produto_embalagem = produto_embalagem[0]
                            if isinstance(produto_embalagem, dict):
                                try:
                                    values['embalagem_id'] = produto_embalagem['embalagem_id'][0]
                                except:
                                    values['embalagem_id'] = []

                        # VERIFICA SE O LOCAL PERMITE LOCAL NEGATIVO, SE ENCONTRA-SE ATIVO
                        # E PERMITE FRACIONAMENTO.
                        # COM ESSES DADOS VERIFICA O SALDO, SE A QUANTIDADE É FRACIONADA
                        # E SE NÃO EXCEDE A QUANTIDADE DO LOCAL - CASO NÃO PERMITA.
                        local_dados = self.get_local_dados(cr, uid, saida_local_id,
                                                           values['quantidade_requisitada'],
                                                           values.get('produto_id'),
                                                           values.get('grade_id'),
                                                           saida_empresa_id, context=context)
                        if isinstance(local_dados, dict):
                            if local_dados.get('warning'):
                                local_dados = local_dados['warning']
                                raise osv.except_osv((local_dados.get('title') or ''),
                                                     (local_dados.get(
                                                         'message') or _(
                                                         '(RESSSUPRIMENTO) - Nao foi possivel inserir o produto no grid')))

                        # FUNÇÃO QUE RETORNA A QUANTIDADE NA EMBALAGEM E A UNIDADE UTILIZADA
                        embalagem = self.get_embalagem_unidade(cr, uid,
                                                               values.get('embalagem_id') or [],
                                                               context={})
                        if isinstance(embalagem, dict):
                            if not embalagem.get('fracao'):
                                # SE A EMBALAGEM NÃO PERMITIR FRACIONAMENTO E O USUÁRIO DIGITAR UMA
                                # QUANTIDADE FRACIONADA O RAISE É LEVANTADO
                                quantidade_float = values['quantidade_requisitada'] % 1
                                if quantidade_float:
                                    msg = 'A embalagem selecionada nao aceita uma quantidade fracionada!'
                                    raise osv.except_osv(_('Atencao'), _(msg))

                    # PASSA O VALOR DO CAMPO saldo_temp PARA O CAMPO saldo.
                    # ISTO É FEITO PORQUE O CAMPO SALDO É READONLY E NO CREATE O CAMPO
                    # NÃO ESTA PRESENTE DENTRO DO VALS.
                    if values.get('saldo_temp'):
                        values['saldo'] = values['saldo_temp']
                        del values['saldo_temp']
                    if requisicao_id:
                        atualizou = True
                        obj_estoque_requisicao_capa.write(cr, uid, requisicao_id, {}, context=context)
                    return super(estoqueressuprimento_requisicao_itens, self).write(cr, uid, ids,
                                                                                    values,
                                                                                    context=context)
        if requisicao_id and not atualizou:
            obj_estoque_requisicao_capa.write(cr, uid, values.get('requisicao_id'), {}, context=context)
        return False

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        res = super(estoqueressuprimento_requisicao_itens, self).search(cr, uid, args, offset,
                                                                        limit, order, context=context)
        if context.get('ressuprimento_com_pendencia'):
            itens_com_pendencia_ids = []
            if res:
                browse_requisicao_itens = self.browse(cr, uid, res)
                if browse_requisicao_itens:
                    for x in browse_requisicao_itens:
                        requisitado = x.quantidade_requisitada
                        confirmado = x.quantidade_confirmada
                        if requisitado != confirmado:
                            itens_com_pendencia_ids.append(x.id)
            res = itens_com_pendencia_ids
        return res

    def onchange_codigo_barra_id(self, cr, uid, ids, codigo_barra_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'produto_id': False, 'grade_id': False, 'produto_embalagem_id': False}}
        if not codigo_barra_id:
            return res
        vals = self.pool.get('automacao.codigo.barra').read(
            cr, uid, codigo_barra_id, ['produto_id', 'grade_id', 'produto_embalagem_id'],
            context, '_classic_write')
        del vals['id']
        produto_embalagem_id = vals.get('produto_embalagem_id')
        del vals['produto_embalagem_id']
        vals['embalagem_id'] = produto_embalagem_id
        res = {'value': vals}
        return res
