# -*- encoding: utf-8 -*-

from dateutil.parser import parse
from datetime import date
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from openerp.report.interface import report_int
from openerp.report.render import render
from openerp.osv import osv
import StringIO
from openerp import pooler
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_ressuprimento(report_int):
    def create(self, cr, uid, ids, data, context=None):
        context = context and context.copy() or {}
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.informaNome(pdf_string, _('Listagem de Requiscao - Ressuprimento'))
        listagem.informaTamLinha(160)
        listagem.setPage(0)
        self.db_name = pooler.get_pool(cr.dbname)

        obj_ressuprimento_capa = self.db_name.get('estoqueressuprimento.requisicao.capa')
        ressuprimento = obj_ressuprimento_capa.read(cr, uid, ids, context=context)

        if isinstance(ressuprimento, list):
            self.traco_linha = 155
            ## Divisao das colunas do grid onde sao exibidos os produtos
            self.col_item = 16
            self.col_grade = 9
            self.col_descricao = 80
            self.col_qtde = 12
            self.col_unidade = 6
            self.col_embalagem = 12
            self.col_preco = 14
            # Espaco no relatorio
            self.space = ' ' * self.traco_linha

            self.define_cabecalho(cr, uid, listagem)
            cont = 1
            listagem.rodapePadrao()
            total_requisicoes = len(ressuprimento)
            req = None
            for ressuprimento_capa in ressuprimento:
                obj_ressprimento_view = self.db_name.get('estoqueressuprimento.report.view')
                args = [('id', '=', ressuprimento_capa.get('id') or 0)]
                report_view_id = obj_ressprimento_view.search(cr, uid, args, context=context)
                if report_view_id:
                    self.print_itens(cr, uid, listagem, ressuprimento_capa or [])
                    if cont != total_requisicoes:
                        listagem.imprimeLinha('{{ Page }}')
                    cont += 1
                else:
                    req = '\n' + str(ressuprimento_capa.get('numero'))

            listagem.finalizaImpressao()
            self.obj = external_pdf(pdf_string.getvalue())
            self.obj.render()
            pdf_string.close()
            res = (self.obj.pdf, 'pdf')
            if req:
                msg = 'Nao foi encontrado nenhum registro para a requisicao nro: %s' % req or ''
                raise osv.except_osv(_('Atencao'), _(msg))
            return res

    def define_cabecalho(self, cr, uid, listagem, context=None):
        context = context and context.copy() or {}
        str_estoque = _('Contr.Estoque')
        str_requisicao = formataString(_('Listagem de Requisicao'),
                                       self.traco_linha - len(str_estoque), ' ', 'C', 'N', 'S')

        str_hoje = date.today().strftime('%d/%m/%Y')

        company_id = str_company = self.db_name.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        str_company = self.db_name.get('res.company').browse(cr, uid, company_id).name or ''
        str_company_name = formataString(str_company, self.traco_linha - len(str_hoje) - 9, ' ',
                                         'C', 'N', 'S')

        str_item = formataString(_('ITEM/DIGITO'), self.col_item, ' ', 'E', 'N', 'S')
        str_grade = formataString(_('GRADE'), self.col_grade, ' ', 'E', 'N', 'S')
        str_descricao = formataString(_('DESCRICAO'), self.col_descricao, ' ', 'C', 'N', 'S')
        str_qtde = formataString(_('QTDE'), self.col_qtde, ' ', 'E', 'N', 'S')
        str_unidade = formataString(_('UN'), self.col_unidade, ' ', 'E', 'N', 'S')
        str_embalagem = formataString(_('EMBALAGEM'), self.col_embalagem, ' ', 'E', 'N', 'S')
        str_preco = formataString(_('PC. VENDA'), self.col_preco, ' ', 'E', 'N', 'S')

        listagem.informaCab(' ' + str_estoque + str_requisicao + ' ')
        listagem.informaCab(' ' + self.space + ' ')
        listagem.informaCab(' ' + str_hoje + str_company_name + _('Pag.: {{ Page }}') + ' ')
        listagem.informaCab(' ' + self.space + ' ')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(
            ' ' + str_item + ' ' + str_grade + ' ' + str_descricao + ' ' + str_qtde + ' '
            + str_unidade + ' ' + str_embalagem + ' ' + str_preco + ' ')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')

    def print_itens(self, cr, uid, listagem, ressuprimento_capa, context=None):
        context = context and context.copy() or {}
        try:
            str_company_in = ressuprimento_capa['resrca_empresa_entrada_id'][1]
        except:
            str_company_in = ''

        try:
            str_local_in = ressuprimento_capa['local_entrada_id'][1]
        except:
            str_local_in = ''

        try:
            str_company_out = ressuprimento_capa['company_id'][1]
        except:
            str_company_out = ''

        try:
            str_local_out = ressuprimento_capa['local_saida_id'][1]
        except:
            str_local_out = ''

        try:
            str_data_req = parse(ressuprimento_capa['data']).date().strftime('%d/%m/%Y')
        except:
            str_data_req = ''

        str_numero_req = ressuprimento_capa.get('numero') or ''

        str_numero_req = formataString(_('Requisicao Nro.: ') + str_numero_req,
                                       self.traco_linha * 25 / 100, ' ', 'D', 'N', 'S')

        str_data_req = formataString(_('Data da Requisicao: ') + str_data_req,
                                     self.traco_linha * 25 / 100, ' ', 'D', 'N', 'S')

        str_company_out = formataString(_('Empresa Saida: ') + str_company_out,
                                        self.traco_linha / 2, ' ', 'D', 'N', 'S')

        str_local_out = formataString(_('Local Saida: ') + str_local_out, self.traco_linha / 2, ' ',
                                      'D', 'N', 'S')

        str_company_in = formataString(_('Empresa Entrada: ') + str_company_in,
                                       self.traco_linha / 2, ' ', 'D', 'N', 'S')

        str_local_in = formataString(_('Local Entrada: ') + str_local_in, self.traco_linha / 2, ' ',
                                     'D', 'N', 'S')

        # Se a linha for impar acrescenta um espaco a mais
        linha_impar_par = '' if float(self.traco_linha) / 2 % 1 == 0 else ' '

        listagem.imprimeLinha(' ' + self.space + ' ')
        listagem.imprimeLinha(' ' + str_numero_req + linha_impar_par + str_data_req)
        listagem.imprimeLinha(' ' + str_company_out + linha_impar_par + str_company_in + ' ')
        listagem.imprimeLinha(' ' + str_local_out + linha_impar_par + str_local_in + ' ')
        listagem.imprimeLinha(' ' + self.space + ' ')

        obj_ressprimento_view = self.db_name.get('estoqueressuprimento.report.view')
        args = [('id', '=', ressuprimento_capa.get('id') or 0)]
        report_view_id = obj_ressprimento_view.search(cr, uid, args, context=context)
        if not report_view_id:
            msg = 'Nao foi encontrado nenhum registro para a requisicao ' \
                  'nro: %s' % ressuprimento_capa.get('numero') or ''
            raise osv.except_osv(_('teste'), _(msg))
        else:
            report_view_dados = obj_ressprimento_view.read(cr, uid, report_view_id, context=context)
            produto_ids = list(set(map(lambda i: i.get('produto_id') or 0, report_view_dados)))
            grade_ids = list(set(map(lambda i: i.get('grade_id') or 0, report_view_dados)))
            obj_estoque_preco_venda = self.db_name.get('estoque.preco.venda')
            dict_produto_preco = obj_estoque_preco_venda.get_preco_venda(cr, uid, produto_ids,
                                                                         epc_grade_ids=grade_ids)
            cont = False
            for i in report_view_dados:
                locacao = i.get('locacao') or ''
                if cont != locacao:
                    if cont:
                        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
                    listagem.imprimeLinha(' ' + self.space + ' ')
                    str_locacao = formataString(_('Locacao: ') + locacao,
                                                self.traco_linha * 50 / 100, ' ', 'D', 'N', 'S')
                    listagem.imprimeLinha(' ' + str_locacao + ' ')
                    # listagem.imprimeLinha(' ' + self.space + ' ')
                    cont = i.get('locacao') or ''
                if cont == locacao:
                    str_codigo = formataString(
                        (i.get('codigo') or '') + '/' + (i.get('digito') or ''), self.col_item,
                        ' ', 'E', 'N', 'S')

                    str_grade_name = formataString(i.get('grade') or '', self.col_grade, ' ', 'E',
                                                   'N', 'S')

                    str_produto_name = formataString(' ' + i.get('descricao') or '',
                                                     self.col_descricao, ' ', 'D', 'N', 'S')

                    str_qtde = formataString('%.3f' % i.get('quantidade_requisitada') or 0.0,
                                             self.col_qtde, ' ', 'E', 'N', 'S')

                    str_unidade = formataString(i.get('unidade') or '', self.col_unidade, ' ', 'E',
                                                'N', 'S')

                    str_qtde_embalagem = formataString('%.3f' % i.get('qtde_embalagem') or 0.0,
                                                       self.col_embalagem, ' ', 'E', 'N', 'S')

                    dados_preco_venda = dict_produto_preco.get(i.get('produto_id')) or {}
                    valor_preco_venda = dados_preco_venda.get(i.get('grade_id')) or \
                        dados_preco_venda.get('preco_venda')
                    str_preco = formataString('%.2f' % valor_preco_venda, self.col_preco, ' ',
                                              'E', 'N', 'S')

                    listagem.imprimeLinha(
                        ' ' + str_codigo + ' ' + str_grade_name + ' ' + str_produto_name + ' '
                        + str_qtde + ' ' + str_unidade + ' ' + str_qtde_embalagem + ' '
                        + str_preco + ' ')

            listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')


report_ressuprimento('report.requisicao_ressuprimento')
