# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp import pooler


class wizard_conferencia_total(osv.Model):
    _name = 'wizard.conferencia.total'

    def onchange_data_requisicao(self, cr, uid, ids, data_requisicao):
        status = ('state', '=', 'M')
        return dict(domain=dict(
            requisicao_id=data_requisicao and [('data', '=', data_requisicao), status] or [status]))

    _columns = {
        'data_requisicao': fields.date('Data Requisicao'),
        'requisicao_id': fields.many2one('estoqueressuprimento.requisicao.capa', 'Requisicao',
                                         required=True, domain="[('state', '=', 'M')]"),
        'wizcnf_req_numero': fields.char('Numero'),
        'wizcnf_req_nf_id': fields.many2one('fiscal.nf.capa', "Nro da Nota Fiscal", readonly=True),
        'wizcnf_req_empresa_saida_id': fields.many2one("res.company", 'Empresa de Saida',
                                                       readonly="True"),
        'wizcnf_req_data': fields.date('Data', readonly=True),
        'wizcnf_req_tipo': fields.selection([('TR', 'TROCA MERCADORIA'),
                                             ('QR', 'REQUISICAO')], 'Tipo', size=2),
        'wizcnf_req_local_saida_id': fields.many2one("estoque.local", 'Local saida', readonly=True),
        'wizcnf_req_local_entrada_id': fields.many2one("estoque.local", 'Local entrada',
                                                       readonly=True),
        'wizcnf_req_requisicao_itens_ids': fields.one2many('wizard.conferencia.requisicao.itens',
                                                           'wizcri_requisicao_id', 'Itens',
                                                           ondelete='cascade'),
    }

    def wizcnf_consultar_requisicao(self, cr, user, ids, context=None):
        """
        Recupera e mostra os valores da requisicao selecionada.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com ID do formulario de consulta
        :param context:
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return:  verdadeiro se as informacoes da requisicao foram mostrados
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        wizcnf_conferencia_dados = self.read(cr, user, ids[0], ['requisicao_id'], context,
                                             '_classic_write')
        requisicao_dados = self.pool.get('estoqueressuprimento.requisicao.capa').read(
            cr, user, wizcnf_conferencia_dados.get('requisicao_id'),
            ['wizcnf_req_nf_id', 'cpmany_id', 'data', 'tipo', 'local_saida_id', 'numero',
             'local_entrada_id', 'requisicao_itens_ids'], context, '_classic_write')
        values = dict(wizcnf_req_numero=requisicao_dados.get('numero'),
                      wizcnf_req_nf_id=requisicao_dados.get('nf_id'),
                      wizcnf_req_empresa_saida_id=requisicao_dados.get('company_id'),
                      wizcnf_req_data=requisicao_dados.get('data'),
                      wizcnf_req_tipo=requisicao_dados.get('tipo'),
                      wizcnf_req_local_saida_id=requisicao_dados.get('local_saida_id'),
                      wizcnf_req_local_entrada_id=requisicao_dados.get('local_entrada_id'),
                      wizcnf_req_requisicao_itens_ids=[
                          [0, False, dict([('wizcri_%s' % k, d[k]) for k in filter(
                              lambda f: f != 'id', d)])] for d in
                          self.pool.get('estoqueressuprimento.requisicao.itens').read(
                              cr, user, requisicao_dados.get('requisicao_itens_ids'),
                              ['produto_id', 'grade_id', 'embalagem_id', 'quantidade_requisitada',
                               'quantidade_confirmada', 'saldo', ], context, '_classic_write')])
        self.pool.get('wizard.conferencia.requisicao.itens').unlink(
            cr, user, self.pool.get('wizard.conferencia.requisicao.itens').search(
                cr, user, [('wizcri_requisicao_id', '=', ids[0])], context=context), context)
        self.write(cr, user, ids[0], values, context)

        return True

    def confirmar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        conf_total = self.read(cr, uid, ids, context=context)
        requisicao_id = conf_total[0]['requisicao_id'][0]
        req_capa_obj = pooler.get_pool(cr.dbname).get('estoqueressuprimento.requisicao.capa')
        context['ressuprimento_alterar'] = True
        req_capa_obj.write(cr, uid, requisicao_id, {'state': 'F'}, context=context)
        req_itens_obj = pooler.get_pool(cr.dbname).get('estoqueressuprimento.requisicao.itens')
        itens_ids = req_itens_obj.search(cr, uid, [('requisicao_id', '=', requisicao_id)],
                                         context=context)
        requisicao_itens_dados = req_itens_obj.read(cr, uid, itens_ids, ['quantidade_requisitada'],
                                                    context=context)
        for item in requisicao_itens_dados:
            sql = """
            -- noinspection SqlDialectInspectionForFile
            UPDATE estoqueressuprimento_requisicao_itens SET quantidade_confirmada = %s
            WHERE id = %s
            """ % (item.get('quantidade_requisitada'), item.get('id'))
            cr.execute(sql)
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'estoqueressuprimento', 'wizard_conferencia_total_form')
        view_id = reference and reference[1] or False

        return dict(view_type='form', view_mode='form', view_id=[view_id], target='inline',
                    res_model='wizard.conferencia.total', type='ir.actions.act_window')


class wizcnf_requisicao_itens(osv.Model):
    _name = 'wizard.conferencia.requisicao.itens'
    # _rec_name = "wizcri_requisicao_id"
    _columns = {
        'wizcri_requisicao_id': fields.many2one("wizard.conferencia.total", 'Requisicao'),
        'wizcri_produto_id': fields.many2one("estoque.produto", 'Produto',
                                             domain="[('tipo', '=', 'P')]"),
        'wizcri_grade_id': fields.many2one("estoque.produto.grade", 'Grade',
                                           domain="[('produto_id', '=', produto_id)]"),
        'wizcri_quantidade_requisitada': fields.float('Quantidade requisitada', digits=(9, 3)),
        'wizcri_quantidade_confirmada': fields.float('Quantidade confirmada', digits=(9, 3)),
        'wizcri_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                               domain="[('produto_id', '=', produto_id)]"),
        'wizcri_saldo': fields.float('Saldo Atual', digits=(9, 3)),
        'wizcri_ja_confirmada': fields.boolean(string='Requisicao de item confirmada')
    }
    _defaults = {
        'wizcri_quantidade_confirmada': 0,
        'wizcri_ja_confirmada': False
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

