# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
import time
from datetime import datetime
from openerp.tools.translate import _


class wizard_ressuprimento_automatico_osv(osv.TransientModel):
    _description = 'Ressuprimento automatico'
    _name = 'wizard.ressuprimento.automatico.osv'
    _columns = {
        'wizrau_saida_company_id': fields.many2one('res.company', 'Empresa de saida', required=True,
                                      help="Empresa de onde saira os itens da requisicao", oldname='company_id'),
        'local_saida': fields.many2one('estoque.local', 'Local saida', required=True,
                                       help="Local de onde saira os itens da requisicao"),
        'wizrau_entrada_company_id': fields.many2one('res.company', 'Empresa de entrada', required=True,
                                            help="Empresa de onde vai entrar os itens da requisicao"),
        'local_entrada': fields.many2one('estoque.local', 'Local entrada', required=True,
                                         help="Local de entrda dos itens na empresa atual"),
        'locacao': fields.char('Locacao(Entrada)', size=15, help='Locacao a ser ressuprida no local de entrada'),
        'log': fields.char('Log', size=50),
    }
    _defaults = {
        'wizrau_saida_company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'wizrau_entrada_company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'log': 'Ressuprimento automatico concluido com sucesso!'
    }

    def onchange_wizrau_saida(self, cr, uid, ids, saida_company_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if not saida_company_id:
            res['value']['local_saida'] = ''
        else:
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.local',
                                                  context=context, user_company_id=saida_company_id)
            context['sig_force_company'] = True
            local_ids = self.pool.get('estoque.local').search(cr, uid,
                                                 [('company_id', '=', company_id)], context=context)
            res['value']['local_saida'] = ''
            if len(local_ids) == 1:
                res['value']['local_saida'] = local_ids[0]

        return res

    def onchange_wizrau_entrada(self, cr, uid, ids, entrada_company_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if not entrada_company_id:
            res['value']['local_entrada'] = ''
        else:
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.local',
                                                context=context, user_company_id=entrada_company_id)
            context['sig_force_company'] = True
            local_ids = self.pool.get('estoque.local').search(cr, uid,
                                                              [('company_id', '=', company_id)],
                                                              context=context)
            res['value']['local_entrada'] = ''
            if len(local_ids) == 1:
                res['value']['local_entrada'] = local_ids[0]

        return res

    def gerar_ressuprimento(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        warning = {}
        ressuprimento_automatico_dados = self.read(cr, uid, ids, [], context=context, load='_classic_write')
        for x in ressuprimento_automatico_dados:
            empresa_saida = x.get('wizrau_saida_company_id')
            empresa_entrada = x.get('wizrau_entrada_company_id')
            if isinstance(empresa_saida, tuple):
                empresa_saida = empresa_saida[0]
            local_saida = x.get('local_saida')
            if isinstance(local_saida, tuple):
                local_saida = local_saida[0]
            local_entrada = x.get('local_entrada')
            if isinstance(local_entrada, tuple):
                local_entrada = local_entrada[0]
            locacao = x.get('locacao')

        obj_local_locacao = self.pool.get('estoque.local.locacao')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_ressuprimento = self.pool.get('estoqueressuprimento.requisicao.capa')
        obj_ressuprimento_itens = self.pool.get('estoqueressuprimento.requisicao.itens')
        obj_produto = self.pool.get('estoque.produto')
        context_sig_force = context.copy()
        context_sig_force['sig_force_company'] = True
        saldo_inferior_produto_ids = []
        produtos_sem_embalagem = ''
        produtos_saldo_insuficiente_embalagem = ''
        capacidade_locacao_cadastrada = False
        vals = []
        if locacao:
            locacao = locacao.upper()
            local_locacao_ids = obj_local_locacao.search(cr, uid, [('locacao', 'ilike', '%s%s' % (locacao, '%')),
                                                                   ('local_id', '=', local_entrada),
                                                                   ('company_id', '=', empresa_entrada)], context=context_sig_force)
        else:
            # PESQUISA OS LOCAIS NO CAMPO RELATED (local_ids) DE PRODUTO
            company_localcao = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.local.locacao',
                                                  context=context, user_company_id=empresa_entrada)
            local_locacao_ids = obj_local_locacao.search(cr, uid, [('local_id', '=', local_entrada),
                                  ('company_id', '=', company_localcao)], context=context_sig_force)
        if locacao and not local_locacao_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum produto encontra-se cadastrado para a locacao informada!'))
        if not locacao and not local_locacao_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum produto encontra-se cadastrado para o local de entrada!'))
        if local_locacao_ids:
            local_locacao_dados = obj_local_locacao.read(cr, uid, local_locacao_ids,
                                                         ['produto_id', 'estllc_lastro', 'estllc_percentual_lastro',
                                                          'grade_id'], context=context, load='_classic_write')
            grade_ids = map(lambda x: x.get('grade_id'), local_locacao_dados)
            saldos_entrada = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, grade_ids,
                                          time.strftime('%Y-%m-%d'), local_entrada, context=context)
            for y in local_locacao_dados:
                produto_id = y.get('produto_id')
                grade_id = y.get('grade_id')
                saldo_entrada = saldos_entrada.get(grade_id).get('disponivel')
                if saldo_entrada < 0:  # saldo negativo deve ser considerado como zero
                    saldo_entrada = 0
                capacidade_locacao = y.get('estllc_lastro')
                percent_capac_locacao = y.get('estllc_percentual_lastro')
                if capacidade_locacao:
                    capacidade_locacao_cadastrada = True
                    if percent_capac_locacao:
                        if percent_capac_locacao == 0 or percent_capac_locacao == 100:
                            percent_capac_locacao = 100
                        percent_capac_locacao = float(percent_capac_locacao / 100.0)
                    else:  # se nao houver percentual percent_capac_locacao=1 nao faz diferenca na conta
                        percent_capac_locacao = 1
                    if saldo_entrada < capacidade_locacao * percent_capac_locacao:
                        qtd_ressuprimento = capacidade_locacao - saldo_entrada
                        saldo_inferior_produto_ids.append(
                            {'produto_id': produto_id, 'quantidade_requisitada': qtd_ressuprimento,
                             'grade_id': grade_id})
            if not capacidade_locacao_cadastrada:
                raise osv.except_osv(_('Atencao'),
                                     _('As locacoes para o local de entrada nao possuem capacidade cadastrada'))

        if saldo_inferior_produto_ids:
            # BUSCA NO ESTOQUE PARAMETRO A QUANTIDADE DE ITENS PARA UM RESSUPRIMENTO
            estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', empresa_entrada)],
                                                                context=context)
            if not estoque_parametro_id:
                raise osv.except_osv(_('Atencao'),
                                     _('Nao foi encontrado nenhum parametro de estoque criado para a empresa logada!'))
            else:
                ressuprimento_item_qtde_max = obj_estoque_parametro.browse(cr, uid, estoque_parametro_id[
                    0]).ressuprimento_item_qtde_max or 0

            saida_grades_ids = map(lambda x: x.get('grade_id'), saldo_inferior_produto_ids)
            saldos_saida = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, saida_grades_ids,
                                                                   time.strftime('%Y-%m-%d'), local_saida,
                                                                   company_id=empresa_saida, context=context)

            for x in saldo_inferior_produto_ids:
                saldo_saida = saldos_saida.get(x.get('grade_id')).get('disponivel')
                if saldo_saida > 0:
                    produto_embalagem_id = False
                    # BUSCA A EMBALAGEM DEFINIDA PARA O LOCAL DE SAÍDA
                    local_embalagem_ids = obj_local_locacao.search(cr, uid, [('produto_id', '=', x.get('produto_id')),
                                                                             ('grade_id', '=', x.get('grade_id')),
                                                                             ('local_id', '=', local_saida),
                                                                             ('company_id', '=', empresa_saida)],
                                                                   context=context_sig_force)
                    if local_embalagem_ids:
                        local_embalagem = obj_local_locacao.read(cr, uid, local_embalagem_ids,
                                                                 ['estllc_produto_embalagem_id'], context=context)
                        for y in local_embalagem:
                            produto_embalagem_id = y.get('estllc_produto_embalagem_id')
                            if isinstance(produto_embalagem_id, tuple):
                                produto_embalagem_id = produto_embalagem_id[0]
                                # SE O LOCAL NÃO TIVER EMBALAGEM CADASTRADA BUSCA NA TABELA DE ESTOQUE.PRODUTO.EMBALAGEM
                    if not produto_embalagem_id:
                        produto_embalagem_ids = obj_estoque_produto_embalagem.search(cr, uid, [
                            ('produto_id', '=', x.get('produto_id'))], context=context)
                        if len(produto_embalagem_ids) == 1:
                            produto_embalagem_id = produto_embalagem_ids[0]
                    if not produto_embalagem_id:
                        try:
                            produtos_sem_embalagem += '* %s, ' % obj_produto.browse(cr, uid, x.get('produto_id')).name
                        except:
                            continue
                    else:
                        qtd_emb_saida = obj_ressuprimento_itens.get_embalagem_unidade(cr, uid, produto_embalagem_id)
                        qtd_emb_saida = qtd_emb_saida.get('quantidade_na_embalagem') or 1
                        quantidade_requisitada = x.get('quantidade_requisitada')
                        quantidade_requisitada = int(quantidade_requisitada/qtd_emb_saida)
                        saldo_saida = int(saldo_saida/qtd_emb_saida)
                        if saldo_saida <= 0:
                            continue

                        if not quantidade_requisitada and not int(x.get('quantidade_requisitada')):
                            produtos_saldo_insuficiente_embalagem += ' %s, ' % obj_produto.browse(cr, uid, x.get(
                                'produto_id')).name
                            continue
                        elif not quantidade_requisitada:
                            continue

                        if saldo_saida < quantidade_requisitada:
                            x['quantidade_requisitada'] = saldo_saida
                            qtd_nao_atendida = quantidade_requisitada - saldo_saida
                            grade = self.pool.get('estoque.produto.grade').browse(cr, uid, x.get('grade_id'))
                            if isinstance(grade, list):
                                grade = grade[0]
                            grade = grade.grade
                            vals_local = {'company_id': empresa_entrada,
                                          'resnat_data': datetime.now().date(),
                                          'resnat_saida_empresa_id': empresa_saida,
                                          'resnat_saida_local_id': local_saida,
                                          'resnat_entrada_local_id': local_entrada,
                                          'resnat_produto_id': x.get('produto_id'),
                                          'resnat_grade': grade,
                                          'resnat_qtd_pedido': quantidade_requisitada,
                                          'resnat_qtd_atendida': saldo_saida,
                                          'resnat_qtd_diferenca': qtd_nao_atendida,
                                          }
                            self.pool.get('estoqueressuprimento.nao.atendido.itens').create(cr, uid, vals_local,
                                                                                            context)
                        if saldo_saida >= quantidade_requisitada:
                            x['quantidade_requisitada'] = quantidade_requisitada

                        x['saldo'] = saldo_saida
                        x['embalagem_id'] = produto_embalagem_id
                    vals.append(x)

            if produtos_sem_embalagem:
                raise osv.except_osv(_('Atencao'), _(
                    'Os produtos a seguir nao possuem nenhuma embalagem cadastrada: %s'.decode(
                        'utf-8')) % produtos_sem_embalagem)
            if produtos_saldo_insuficiente_embalagem:
                raise osv.except_osv(_('Atencao'),
                                     _(
                                         'Os produtos a seguir nao possuem saldo suficiente para formar a embalagem: %s') % produtos_saldo_insuficiente_embalagem)

        if vals:
            total_capas = 1
            if ressuprimento_item_qtde_max < len(vals) and ressuprimento_item_qtde_max:
                total_capas = len(vals) / ressuprimento_item_qtde_max
            if ressuprimento_item_qtde_max > len(vals):
                ressuprimento_item_qtde_max = len(vals)
            for i in xrange(0, total_capas):
                # CRIA O RESSUPRIMENTO CAPA
                requisicao_id = obj_ressuprimento.create(cr, uid,
                                                         {'resrca_empresa_entrada_id': empresa_entrada,
                                                          'local_saida_id': local_saida,
                                                          'company_id': empresa_saida,
                                                          'local_entrada_id': local_entrada,
                                                          'tipo': 'QR'}, context_sig_force)
                if requisicao_id:
                    cont = 0
                    index_to_remove = []
                    for j in vals:
                        j['requisicao_id'] = requisicao_id
                        # CRIA OS ITENS DO RESSUPRIMENTO.
                        obj_ressuprimento_itens.create(cr, uid, j, context)
                        # OS PRODUTOS JA GRAVADOS DEVEM SER REMOVIDOS. A LISTA CONTEM O INDICE DESTES PRODUTOS.
                        index_to_remove.append(vals.index(j))
                        cont += 1
                        # SE A QUANTIDADE DE PRODUTOS INSERIDOS FOR IGUAL AO LIMITE.
                        if cont == ressuprimento_item_qtde_max:
                            index_to_remove.reverse()
                            # EXCLUI OS ITENS DE DENTRO DO VALS.
                            for r in index_to_remove:
                                vals.pop(r)
                                # SAI DO LAÇO E VOLTA PARA A CAPA
                            break
        else:
            raise osv.except_osv(_('Atencao'), _('Nao ha produtos para ressuprimento'))

        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'estoqueressuprimento',
                                                        'wizard_ressuprimento_automatico_form_log')
        view_id = reference and reference[1] or False
        return {'name': _('Ressuprimento Automatico'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wizard.ressuprimento.automatico.osv',
                'view_id': [view_id],
                'target': 'new',
                'type': 'ir.actions.act_window',
                'action_id': 'action_wizard_ressuprimento_automatico_log'}
