# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _


class fiscal_nf_capa(osv.Model):
    """
    Heranca da classe ´fiscal_nf_capa´.
    """

    _inherit = 'fiscal.nf.capa'

    def estoqueserie_raise_msg_produtos(self, cr, uid, produto_ids, message, context=None):
        context = context and context.copy() or {}
        message = unicode(message)
        obj_estoque_produto = self.pool.get('estoque.produto')
        res_name_get = obj_estoque_produto.name_get(cr, uid, produto_ids, context)

        if res_name_get:
            msg_produtos = u''
            dict_produto_name = dict(res_name_get)
            for i in produto_ids:
                msg_produtos += u'%s, ' % dict_produto_name.get(i) or ''
            if msg_produtos:
                raise osv.except_osv(_('Atencao!'), u'%s: %s' % (message, msg_produtos))

        return True

    def button_cancelar(self, cr, uid, ids, context=None):
        """
        Cancela uma nota fiscal.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se a nota fiscal foi cancelada
        :rtype: bool
        """

        context = context and context.copy() or {}

        if self.pool.get('estoque.produto').estpro_get_parametro_serie(cr, uid, context=context):
            brw_nf_capa = self.browse(cr, uid, ids[0], context)
            nf_tipo = brw_nf_capa.tipo

            # Somente depois que a nota de entrada eh faturada, a tabela de movimentacao de
            # serie eh preenchida. Por isso o cancelamento ocorre somente se estiver faturada.
            if nf_tipo == 'E':
                nf_state = brw_nf_capa.state

                if nf_state == '3':  # 3 - Faturada
                    nf_tipo = brw_nf_capa.tipo

                    if nf_tipo == 'E':
                        # Ao cancelar uma nota fiscal que esteja faturada, sera feita a
                        # exclusao dos numeros de serie
                        serie_ids = []

                        for mov_item in brw_nf_capa.estoque_movimento_itens_ids:
                            serie_ids.extend(
                                map(lambda serie_mov: serie_mov.esemov_serie_id.id,
                                    mov_item.estmvi_movserie_ids))

                        self.pool.get('estoqueserie.produto').unlink(cr, uid, serie_ids, context)
            else:
                # Ja na nota de saida, ao indicar o numero de serie do produto, a tabela de
                # movimentacao, se serie eh preenchida. por isso, ao cancelar uma nota de saida
                # nao eh preciso verificar o status.
                serie_mov_ids = []

                for mov_item in brw_nf_capa.estoque_movimento_itens_ids:
                    serie_mov_ids.extend(map(lambda i: i.id, mov_item.estmvi_movserie_ids))

                self.pool.get('estoqueserie.movimento').unlink(cr, uid, serie_mov_ids, context)

        return super(fiscal_nf_capa, self).button_cancelar(cr, uid, ids, context)

    def button_faturar(self, cr, uid, ids, context=None):
        """
        Fatura uma nota fiscal. Cadastra o numero de serie nas grades dos produtos.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: verdadeiro se a nota fiscal foi faturada.
        :rtype: bool
        """

        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).button_faturar(cr, uid, ids, context)

        if not self.pool.get('estoque.produto').estpro_get_parametro_serie(
                cr, uid, context=context) or result.get('state') != '3':
            return result

        brw_nf_capa = self.browse(cr, uid, ids[0], context)

        if brw_nf_capa.tipo != 'E':
            return result

        obj_serie_utils = self.pool.get('serie.utils')
        serie_entrada_invalida = {}

        # Somente produtos que possuem atribuicao de controle de serie
        for item in filter(lambda f: f.produto_id.estpro_nserie_tipo in ('NS', 'NC'),
                           brw_nf_capa.estoque_movimento_itens_ids):
            mov_item_produto_id = item.produto_id.id
            record_mov_item_serie_entrada = item.estmvi_serie_entrada_ids or []
            list_series = filter(None, map(
                lambda s: s.eseent_numero_serie, record_mov_item_serie_entrada))
            series_inseridas = obj_serie_utils.serutl_generate_range_serie(
                list_serie=list_series)

            if not isinstance(series_inseridas, list):
                erro_serie = obj_serie_utils.serutl_get_erro()
                raise osv.except_osv(_('Erro no numero de serie'), erro_serie)

            # Verifica se o numero de serie jah encontra-se cadastrado para a grade.
            serie_entrada_cadastrada = obj_serie_utils.serutl_restricao_entrada(
                cr, uid, series_inseridas, item.grade_id.id, context)

            if serie_entrada_cadastrada:
                if mov_item_produto_id in serie_entrada_invalida:
                    serie_entrada_invalida[mov_item_produto_id] += serie_entrada_cadastrada
                else:
                    serie_entrada_invalida[mov_item_produto_id] = serie_entrada_cadastrada

            for serie in series_inseridas:
                vals_serie = {
                    'esepro_nro_serie': serie,
                    'esepro_produto_id': mov_item_produto_id,
                    'esepro_grade_id': item.grade_id.id,
                }
                serie_produto_id = self.pool.get('estoqueserie.produto').create(
                    cr, uid, vals_serie, context=context)
                vals_movimento = {
                    'esemov_serie_id': serie_produto_id,
                    'esemov_operacao': brw_nf_capa.tipo,
                    'esemov_movimento_itens_id': item.id
                }
                self.pool.get('estoqueserie.movimento').create(
                    cr, uid, vals_movimento, context=context)

        if serie_entrada_invalida:
            produto_ids = serie_entrada_invalida.keys()
            res_name = self.pool.get('estoque.produto').name_get(cr, uid, produto_ids, context)
            dict_produto_names = res_name and dict(res_name) or {}
            msg = u''

            for item in serie_entrada_invalida:
                produto_name = dict_produto_names.get(item) or _('** Produto nao encontrado **')
                series = serie_entrada_invalida[item]
                msg += u'%s: %s' % (produto_name, series)

            # Quando for nota fiscal eletronica de entrada propria o erro deve ser tratado de
            # outra forma
            # raise osv.except_osv(_('Numeros de serie ja cadastrados'), msg)

        return result

    def fisnfc_verificar_nota_fiscal(self, cr, uid, nf, context):
        """
        Verifica se a quantide de numeros de serie inseridos eh igual a quantidade de produtos.
        """

        context = context and context.copy() or {}
        result = super(fiscal_nf_capa, self).fisnfc_verificar_nota_fiscal(cr, uid, nf, context)

        if not self.pool.get('estoque.produto').estpro_get_parametro_serie(
                cr, uid, context=context):
            return result

        brw_nf_capa = self.browse(cr, uid, nf, context)
        nf_tipo = brw_nf_capa.tipo
        serie_divergente_produto_ids = []
        nao_possui_serie_produto_ids = []
        serie_nao_lancada_produto_ids = []
        msg_serie_erro = {}

        for item in filter(lambda f: f.produto_id.estpro_nserie_tipo in ('NS', 'NC'),
                           brw_nf_capa.estoque_movimento_itens_ids):
            produto_id = item.produto_id.id
            item_quantidade = item.quantidade or 0
            series_inseridas = []

            # Nota de entrada
            if nf_tipo == 'E':
                record_item_serie = item.estmvi_serie_entrada_ids or []

                if not record_item_serie:
                    serie_nao_lancada_produto_ids.append(produto_id)
                else:
                    list_series = filter(
                        None, map(lambda s: s.eseent_numero_serie, record_item_serie))
                    series_inseridas = self.pool.get('serie.utils').serutl_generate_range_serie(
                        list_serie=list_series)

                    if not series_inseridas:
                        serie_erro = self.pool.get('serie.utils').serutl_get_erro()

                        if isinstance(serie_erro, basestring):
                            key = hash(serie_erro)

                            if msg_serie_erro.get(key):
                                msg_serie_erro[key]['produto_ids'].append(produto_id)
                            else:
                                msg_serie_erro[key] = {
                                    'produto_ids': [produto_id],
                                    'msg': serie_erro
                                }
            else:  # Nota de saida
                record_item_serie = item.estmvi_movserie_ids or []

                if not record_item_serie:
                    serie_nao_lancada_produto_ids.append(produto_id)
                else:
                    series_inseridas = record_item_serie

            if series_inseridas:
                if isinstance(series_inseridas, list):
                    total_series_lancadas = len(series_inseridas)
                else:
                    total_series_lancadas = 0

                if int(item_quantidade) != total_series_lancadas:
                    serie_divergente_produto_ids.append(produto_id)

        if serie_nao_lancada_produto_ids:
            msg = _('Nenhum numero de serie foi informado para os produtos a seguir')
            self.estoqueserie_raise_msg_produtos(
                cr, uid, produto_ids=serie_nao_lancada_produto_ids, message=msg,
                context=context)

        if msg_serie_erro:
            msg = self.pool.get('serie.utils').serutl_generate_msg_serie_produtos(
                cr, uid, dict_msg=msg_serie_erro, context=context)
            raise osv.except_osv(_('Numero de serie invalido'), msg)

        if serie_divergente_produto_ids:
            msg = _('Os produtos a seguir possuem os numeros de serie diferente da '
                    'quantidade inserida no item')
            self.estoqueserie_raise_msg_produtos(
                cr, uid, produto_ids=serie_divergente_produto_ids, message=msg,
                context=context)

        if nao_possui_serie_produto_ids:
            msg = _('Os produtos a seguir nao possuem controle de numero de serie')
            self.estoqueserie_raise_msg_produtos(
                cr, uid, produto_ids=nao_possui_serie_produto_ids, message=msg,
                context=context)

        return result

    def nfe_infnfe_det_infadprod(self, cr, user, item, context=None):
        """
        Acrescenta os numeros de serie aas informacoes adicionais do item.
        
        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type context: dict
        :return: lista com as informacoes adicionais do item.
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # //NFe.infNFe.det.infAdProd - Informacoes adicionais do produto
        result = super(fiscal_nf_capa, self).nfe_infnfe_det_infadprod(cr, user, item, context)
        brw_item = self.pool.get('estoque.movimento.itens').browse(cr, user, item, context)
        if brw_item.estmvi_movserie_ids:
            result.append('Numeros de serie: %s' % ', '.join(
                map(lambda m: m.esepro_nro_serie, brw_item.estmvi_movserie_ids)))

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
