# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from datetime import datetime
from openerp.tools.translate import _


class expedicao_entregas_capa(osv.Model):
    """
    Tabela que contem todas as entregas já realizadas
    """
    _name = 'expedicao.entregas.capa'
    _inherit = 'expedicao.entregas.capa'

    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if not order:
            order = 'expetc_data_entrega desc'
        for arg in args:
            if 'expetc_data_entrega' not in arg or count:
                continue
            if ('expetc_data_entrega' in arg and '<=' in arg) or ('expetc_data_entrega' in arg and
                                                                  '>=' in arg):
                continue
            arg[1] = '>='
            arg[2] = arg[2][:10] + ' 00:00:00'
            args.append(['expetc_data_entrega', '<=', arg[2][:10] + ' 23:59:59'])
            break
        res = super(expedicao_entregas_capa, self).search(cr, user, args, offset, limit, order,
                                                          context, count)
        return res

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        brw_entrega = self.browse(cr, uid, ids, context=context)
        for entrega in brw_entrega:
            nome = (entrega.id, entrega.expetc_vendas_capa_id.expvec_documento)
            res.append(nome)
        return res

    def expetc_button_cancelar_entrega(self, cr, uid, ids, context=None):
        """
        Este metodo serve para abrir uma wizard que pedira o motivo e a confirmacao
        do cancelamento da expedicao de entrega

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das entregas
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        context.update({'expedicao_entrega_id': ids})
        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'expedicao',
                                                        'wizard_cancelamento_entrega_form')
        view_id = reference and reference[1] or False
        return {
            'name': 'Cancelamento de entrega',
            'view_type': 'form',
            'view_mode': 'form',
            'res_id': False,
            'res_model': 'wizard.cancelamento.entrega',
            'view_id': view_id,
            'context': context,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }

    def expetc_cancelar_entregas(self, cr, uid, ids, motivo, context=None):
        """
        Este metodo serve cancelar uma expedicao de entrega

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das entregas
        :param motivo: Text informando o motivo do cancelamento da entrega
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list
        :type motivo: str
        :type context: dict
        :rtype: bool

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        vals = {
            'state': 'C',
            'expetc_cancel_user_id': uid,
            'expetc_cancel_motivo': motivo,
            'expetc_cancel_data': datetime.now(),
            }
        res = super(expedicao_entregas_capa, self).write(cr, uid, ids, vals, context)
        user_name, data_impressao = self.expetc_get_dados_log(cr, uid, context)
        self.message_post(cr, uid, ids, 'por %s em %s' % (user_name, data_impressao),
                          'Entrega cancelada')
        return res

    def expetc_button_print(self, cr, uid, ids, context=None):
        """
        O metodo realizada a chamada da impressao do arquivo pdf.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais
        :return: action do relatorio
        """
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'expetc_impresso': True}, context)
        entrega_id = ids
        if isinstance(entrega_id, (list, tuple)):
            entrega_id = entrega_id[0]

        dados_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_formato_impressao_expedicao',
                                 'estpar_expedicao_retrato_paisagem'])
        formato = dados_parametro.get('estpar_formato_impressao_expedicao')
        modo = dados_parametro.get('estpar_expedicao_retrato_paisagem')
        path = '/expedicao/report/entrega.rml'
        self.pool.get('estoque.parametro').estpar_converte_formato_impressao(cr, uid, formato, path,
                                                                             'p', modo)

        # Contexto verificado no metodo '__init__' da classe 'rel_entrega'. Seu objetivo e nao
        # deixar gerar a opcao de download do arquivo e impressao direta dele na impressora.
        context['expetc_nao_enviar_impressora'] = True
        user_name, data_impressao = self.expetc_get_dados_log(cr, uid, context)
        self.message_post(cr, uid, ids, 'por %s em %s' % (user_name, data_impressao),
                          'Entrega impressa')
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'expedicao.entregas',
            'name': 'CE-%s' % entrega_id,
            'context': context
        }

    def expetc_get_dados_log(self, cr, uid, context=None):
        """
        Metodo responsavel pela busca do nome do usuario responsavel pela transacao e data do
        momento, formatada. Dados para inclusao do log de mudanca de status.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param context: dados contextuais
        :type context: dict
        :return: Nome do usuario e data do momento formatada
        :rtype: str
        """
        context = context and context.copy() or {}
        user_name = self.pool.get('res.users').browse(cr, uid, uid, context).pessoa_id.nome
        data_impressao = self.pool.get('adm.utils').admutl_datetime_to_timezone(
            cr, uid, datetime.now(), '%d-%m-%Y %H:%M:%S.%f')
        return user_name, data_impressao

    def expetc_button_separar(self, cr, uid, ids, context=None):
        """
        O metodo altera o status da entrega para 'Separada' ('S').
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso o status seja alterado, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        if any(map(lambda e: e.state != 'P', self.browse(cr, uid, ids, context))):
            raise osv.except_osv(_('Atencao'), _('Atualize sua tela, este controle de entrega ja '
                                                 'foi alterado.'))
        self.write(cr, uid, ids, {'state': 'S'}, context)
        user_name, data_impressao = self.expetc_get_dados_log(cr, uid, context)
        self.message_post(cr, uid, ids, 'por %s em %s' % (user_name, data_impressao),
                          'Entrega separada')
        return self.expetc_print_ordem_separacao(cr, uid, ids, context)

    def expetc_print_ordem_separacao(self, cr, uid, ids, context=None):
        """
        O metodo realizada a chamada da impressao do arquivo pdf.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais
        :return: action do relatorio
        """
        context = context and context.copy() or {}
        entrega_id = ids
        if isinstance(entrega_id, (list, tuple)):
            entrega_id = entrega_id[0]

        dados_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_formato_impressao_expedicao',
                                 'estpar_expedicao_retrato_paisagem'])
        formato = dados_parametro.get('estpar_formato_impressao_expedicao')
        modo = dados_parametro.get('estpar_expedicao_retrato_paisagem')
        path = '/expedicao/report/entrega.rml'
        self.pool.get('estoque.parametro').estpar_converte_formato_impressao(cr, uid, formato, path,
                                                                             'p', modo)
        context.update({'expetc_nao_enviar_impressora': True})
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'expedicao.entregas',
            'name': 'Separacao-%s' % entrega_id,
            'context': context
        }

    def expetc_button_entregar(self, cr, uid, ids, context=None):
        """
        O metodo exibi uma nova janela para que o usuario informe os dados necessarios para realizar
        a entrega.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Dados para a chamada de uma view em uma nova janela.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if any(map(lambda i: i.expetc_impresso, self.browse(cr, uid, ids, context))):
            raise osv.except_osv(_('Atencao'),
                                 _('Entrega ja realizada. Atualize a tela pressionando F5.'))
        return {
            'name': _('Confirmacao do destinatario'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.confirmar.entrega',
            'view_id': False,
            'context': context,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }
