# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv


class expedicao_vendas_itens(osv.Model):

    _name = 'expedicao.vendas.itens'
    _inherit = 'expedicao.vendas.itens'

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_itens = self.read(cr, uid, ids, ['expvei_grade_id'], context, '_classic_write')
        grade_ids = map(lambda x: x.get('expvei_grade_id'), dados_itens)
        dict_grades = {}
        map(lambda x: dict_grades.update({x.get('id'): x}),
            self.pool.get('estoque.produto.grade').read(cr, uid, grade_ids, ['name'], context))
        res = map(lambda x: (x.get('id'), dict_grades.get(x.get('expvei_grade_id')).get('name')),
                  dados_itens)
        return res
    
    def expvei_deduzir_quantidade_entregue(self, cr, uid, ids, quantidade, context=None):
        """
        Este metodo serve para deduzir a quantidade entregue, quando eh feito um cancelamento
        da entrega (class: expedicao.entregas.capa)
        O status do item sera alterado de acordo com a quantidade entregue
        onde 'A' = A entregar, 'P' = Entregue parcialmente e 'E' = Entregue

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID dos itens vendidos
        :param quantidade: float contendo a quantidade a ser deduzida
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids:list
        :type quantidade: float
        :type context: dict
        :rtype: bool

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = False
        campos = ['expvei_quantidade_entregue', 'expvei_quantidade_vendida']
        for item in self.read(cr, uid, ids, campos, context):
            qtd_entregue = item.get('expvei_quantidade_entregue') - quantidade
            if qtd_entregue <= 0:
                novo_status = 'A'
            elif qtd_entregue == item.get('expvei_quantidade_vendida'):
                novo_status = 'E'
            else:
                novo_status = 'P'

            vals = {'expvei_quantidade_entregue': qtd_entregue,
                    'status': novo_status}
            res = super(expedicao_vendas_itens, self).write(cr, uid, ids, vals, context)

        return res

    def espvei_valida_quantidade_entregue(self, cr, uid, expvei_id, qtd_entregar, context=None):
        """
        Este metodo verifica se a quantidade informada para entregar mais a quantidade
        que foi entregue e maior do que a quantidade vendida.

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param expvei_id: ID do item 'expedicao.vendas.itens'
        :param qtd_entregar: float com a quantidade informada para entregar
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type expvei_id: int, long
        :type qtd_entregar: float
        :type context: dict
        :rtype: bool
        :return: Retorna False se a quantidade vendida for menor que a quantidade entregue +
                 a quantidade informada para entrega, senao retorna True
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        brw_expvei = self.browse(cr, uid, expvei_id, context=context)
        qtd_vendida = brw_expvei.expvei_quantidade_vendida or 0.0
        qtd_entregue = sum(map(
            lambda i: i.expeti_quantidade if i.expeti_capa_id.state != 'C' else 0,
            brw_expvei.entregas_itens_ids))
        if round(qtd_entregar, 3) > round(qtd_vendida - qtd_entregue, 3):
            return False
        return True
