# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

# ___________________________ wizard.vendas __________________________
def wizvei_get_status(self, cr, uid, ids, field, args, context=None):
    """
    Atualiza o ´status´ da capa da expedicao de venda em funcao dos itens

    :param cr: a linha atual do cursor do banco de dados
    :param uid: o ID do usuario utilizado para a verificacao de perfil
    :param ids: IDs dos itens de expedicao
    :param field: lista com o nome dos campos
    :param args: argumentos para consulta
    :param context: um dicionario padrao para valores contextuais
    :type cr: cursor
    :type uid: int
    :type id: long|list[long]
    :type field: list
    :type args: tuple
    :return: dicionario com os valores do campo ´status´ (da capa) ajustado
    :rtype: dict
    """

    context = context and context.copy() or {}
    res = {}

    for capa in self.browse(cr, uid, ids, context=context):
        a_entregar = entregue = True

        for item in capa.wizvec_vendas_itens_ids:
            if item.status == 'A':  # Item a ser entregue
                entregue = False
            elif item.status == 'E':  # Item jah entregue
                a_entregar = False
            else:  # Item parcialmente entregue (P)
                a_entregar = entregue = False
                break

        res[capa.id] = 'A' if a_entregar else 'E' if entregue else 'P'

    return res

def wizvei_get_status_qtde(self, cr, uid, ids, field, args, context=None):
    """
    Informa se um item jah foi entregue.

    :param cr: a linha atual do cursor do banco de dados
    :param uid: o ID do usuario utilizado para a verificacao de perfil
    :param ids: IDs dos itens de expedicao
    :param field: lista com o nome dos campos
    :param args: argumentos para consulta
    :param context: um dicionario padrao para valores contextuais
    :type cr: cursor
    :type uid: int
    :type ids: list[long]
    :type field: list
    :type args: tuple
    :return: dicionario com os valores do campo ´status´ (da capa) ajustado
    :rtype: dict
    """

    context = context and context.copy() or {}
    res = {}

    for _id in ids:
        sql = """
            -- noinspection SqlDialectInspection
            SELECT wizvei_quantidade_vendida
                 , wizvei_quantidade_entregue
            FROM   wizard_vendas_itens
            WHERE  id = %s""" % _id
        cr.execute(sql)
        dados = cr.dictfetchall()

        if dados and isinstance(dados, list):
            dados = dados[0]
            qtd_vendida = dados.get('wizvei_quantidade_vendida')
            qtd_entrege = dados.get('wizvei_quantidade_entregue')

            if qtd_vendida == qtd_entrege:
                res[_id] = True
            else:
                res[_id] = False

    return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
