# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from datetime import datetime
from openerp.report import report_sxw
import random
import string
import subprocess
from dateutil.parser import parse
from functools import wraps
from openerp.tools.translate import _
import logging


def random_name():
    random.seed()
    d = [random.choice(string.ascii_letters) for x in xrange(6)]
    name = "".join(d)
    return name


# noinspection PyPep8Naming,PyAbstractClass
class wizard_confirmar_entrega(osv.TransientModel):

    def get_data(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return str(datetime.now())

    _name = 'wizard.confirmar.entrega'
    _logger = logging.getLogger(_name)
    _columns = {
        'wizcoe_data_entrega': fields.datetime('Data de entrega', required=True),
        'wizcoe_quem_recebeu': fields.char('Quem recebeu', required=True, size=45),
        'wizcoe_quem_entregou_id': fields.many2one("adm.pessoa", 'Quem entregou', ondelete="restrict", required=True, select=True),
    }
    _defaults = {
        'wizcoe_data_entrega': get_data,
    }

    def check_status(metodo):
        # noinspection PyCallingNonCallable
        @wraps(metodo)
        def wrapper(self, cr, uid, ids, context):

            res = metodo(self, cr, uid, ids, context)
            return res
        return wrapper

    # noinspection PyArgumentList
    @check_status
    def button_wizcoe_confirmar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_expedicao_entregas_capa = self.pool.get('expedicao.entregas.capa')

        if obj_expedicao_entregas_capa.browse(cr, uid, context.get('active_id'),
                                              context).state == 'E':
            raise osv.except_osv(_('Atencao'),
                                 _('Entrega ja confirmada. Atualize a tela pressionando F5.'))

        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_impressora_expedicao', 'estpar_pedido_tipo_codigo'],
            context=context)
        impressora_expedicao = estoque_parametro_dados.get('estpar_impressora_expedicao')
        tipo_codigo = estoque_parametro_dados.get('estpar_pedido_tipo_codigo')

        id = ids[0] if ids and isinstance(ids, list) else ids
        brw_wiz_confimrar = self.browse(cr, uid, id)
        vals_entrega = {'expetc_data_entrega': brw_wiz_confimrar.wizcoe_data_entrega,
                        'expetc_quem_recebeu': brw_wiz_confimrar.wizcoe_quem_recebeu,
                        'expetc_quem_entregou_id': brw_wiz_confimrar.wizcoe_quem_entregou_id.id,
                        'state': 'E'
                       }
        if impressora_expedicao:
            vals_entrega.update({'expetc_impresso': True})
        obj_expedicao_entregas_capa.write(cr, uid, context.get('active_id'), vals_entrega, context)
        user_name, data_impressao = obj_expedicao_entregas_capa.expetc_get_dados_log(
            cr, uid, context)
        obj_expedicao_entregas_capa.message_post(
            cr, uid, [context.get('active_id')], 'por %s em %s' % (user_name, data_impressao),
            'Entrega realizada')
        brw_expedicao_entregas_capa = obj_expedicao_entregas_capa.browse(
            cr, uid, context.get('active_id'), context)
        entregas_itens_ids = map(lambda i: i.id, brw_expedicao_entregas_capa.entregas_itens_ids)
        if len(entregas_itens_ids) == 1:
            entregas_itens_ids = str(tuple(entregas_itens_ids)).replace(',', '')
        elif len(entregas_itens_ids) > 1:
            entregas_itens_ids = str(tuple(entregas_itens_ids))
        sql_itens_ids = '''
                            SELECT evi.id                         AS vendas_itens_id,
                                   evi.expvei_quantidade_vendida  AS vendida,
                                   evi.expvei_quantidade_entregue AS entregue,
                                   eei.expeti_quantidade          AS entregar
                            FROM expedicao_entregas_itens eei
                            INNER JOIN expedicao_vendas_itens evi
                                ON eei.expeti_vendas_itens_id = evi.id
                            WHERE
                                eei.id IN %s
                        ''' % entregas_itens_ids
        cr.execute(sql_itens_ids)
        dados_itens = cr.dictfetchall()
        for item in dados_itens:
            qtd_vendida = item.get('vendida') or 0.0
            qtd_entregue = item.get('entregue') or 0.0
            qtd_entregar = item.get('entregar') or 0.0
            status = 'P'
            if qtd_vendida == qtd_entregue + qtd_entregar:
                status = 'E'
            sql_venda_item = '''
                                UPDATE expedicao_vendas_itens
                                SET status = '%s',
                                    expvei_quantidade_entregue = %s
                                WHERE id = %s
                             ''' % (status, qtd_entregue+qtd_entregar,
                                    item.get('vendas_itens_id'))
            cr.execute(sql_venda_item)

        # Buscar id da entrega
        if impressora_expedicao:
            self.wizcoe_imprimi_expedicao_entrega(cr, uid, context.get('active_id'),
                                                  impressora_expedicao, tipo_codigo, context)

        return {'res_id': context.get('active_id'),
                'view_type': 'form',
                'view_mode': 'tree,form',
                'res_model': 'expedicao.vendas.capa',
                'view_id': False,
                'type': 'ir.actions.act_window'}

    def wizcoe_imprimi_expedicao_entrega(self, cr, uid, entrega_id, impressora_expedicao,
                                         tipo_codigo, context=None):
        """
        O metodo manda para a impressora definida em 'impressora_expedicao' o arquivo pdf gerado a
        partir da entrega da expedicao
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param impressora_expedicao: char com a descricao do nome da impressora
        :param tipo_codigo: char que representa se o codigo impresso sera o iterno ou do fabricante
        :param context: dicionario com valores contextuais
        :return: booleano True
        """
        context = context and context.copy() or {}

        def converte_datetime(data, formato):
            try:
                if not isinstance(data, datetime):
                    data = parse(data)
                data = self.pool.get('adm.utils').admutl_datetime_to_timezone(
                    cr, uid, data, formato)
            except:
                data = ''
            return data

        def get_vendedor(brw_vendas_capa):
            vendedor = ''
            if brw_vendas_capa.expvec_nf_id.id:
                if brw_vendas_capa.expvec_nf_id.vendedor_id:
                    vendedor = brw_vendas_capa.expvec_nf_id.vendedor_id.pessoa_id.nome
            else:
                if brw_vendas_capa.expvec_cupom_id.ecc_vendedor_id:
                    vendedor = brw_vendas_capa.expvec_cupom_id.ecc_vendedor_id.pessoa_id.nome
            return vendedor

        context.update({
            'tipo_codigo': tipo_codigo,
            'converte_datetime': converte_datetime,
            'get_vendedor': get_vendedor,
            'datetime': datetime
        })
        self.context = context

        dados_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_formato_impressao_expedicao',
                                 'estpar_expedicao_retrato_paisagem'])
        formato = dados_parametro.get('estpar_formato_impressao_expedicao')
        modo = dados_parametro.get('estpar_expedicao_retrato_paisagem')
        path = '/expedicao/report/entrega.rml'
        self.pool.get('estoque.parametro').estpar_converte_formato_impressao(cr, uid, formato, path,
                                                                             'p', modo)

        documento = 'report.' + random_name()
        obj_ir_act_report_xml = self.pool.get('ir.actions.report.xml')
        report_id = obj_ir_act_report_xml.search(
            cr, uid, [('report_name', '=', 'expedicao.entregas.capa')], context=context)
        report_xml = obj_ir_act_report_xml.browse(cr, uid, report_id[0])
        report_pdf = False
        if not context.get('impressao_expedicao_entrega'):
            context.update({'impressao_expedicao_entrega': True})
            report_pdf = report_sxw.report_sxw(
                documento, 'expedicao.entregas.capa',
                'addons/expedicao/report/entrega.rml').create_single_pdf(
                cr, uid, [entrega_id], {'report_type': 'pdf'}, report_xml,
                context)
        if report_pdf and impressora_expedicao:
            file_name = 'CE-%s.pdf' % entrega_id
            p = open('/tmp/%s' % file_name, 'wb')
            p.write(report_pdf[0])
            p.close()
            impressora = "-d" + impressora_expedicao
            subprocess.call(["lp", "-n1", impressora, "/tmp/%s" % file_name])
            subprocess.call(['rm', "/tmp/%s" % file_name])

            user_name = self.pool.get('res.users').browse(cr, uid, uid, context).pessoa_id.nome
            data_impressao = self.pool.get('adm.utils').admutl_datetime_to_timezone(
                cr, uid, datetime.now(), '%d-%m-%Y %H:%M:%S.%f')
            self.pool.get('expedicao.entregas.capa').message_post(
                cr, uid, [entrega_id], 'por %s em %s' % (user_name, data_impressao),
                'Entrega impressa')
            self._logger.info('Entrega impressa por %s em %s.' % (user_name, data_impressao))
        return True
