# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class wiz_expedicao_demonstrativo_entrega(osv.TransientModel):
    # prefixo: wizede_
    _name = 'wiz.expedicao.demonstrativo.entrega'
    _columns = {
        'wizede_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wizede_venda_id': fields.many2one('expedicao.vendas.capa', 'Venda'),
        'wizede_data_inicial': fields.datetime('Data inicial'),
        'wizede_data_final': fields.datetime('Data final'),
    }

    def wizede_button_print(self, cr, uid, ids, context=None):
        """
        O metodo reune em um dicionario os dados necessarios para gerar o relatorio. Esses dados sao
        recebidos e processados na classe do relatorio.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais
        :return: dicionario com uma acao definida para chamar a classe do relatorio
        """
        context = context and context.copy() or {}
        obj_entregas_capa = self.pool.get('expedicao.entregas.capa')
        obj_vendas_capa = self.pool.get('expedicao.vendas.capa')
        brw_demonstrativo_entrega = self.browse(cr, uid, ids, context)
        datas = {}
        for demonstrativo in brw_demonstrativo_entrega:
            args = [('state', '=', 'E')]
            args_vendas = []
            venda_ids = []
            pessoa_id = demonstrativo.wizede_pessoa_id.id
            venda_id = demonstrativo.wizede_venda_id.id
            data_inicial = demonstrativo.wizede_data_inicial
            data_final = demonstrativo.wizede_data_final
            if pessoa_id:
                args_vendas.append(('expvec_pessoa_id', '=', pessoa_id))
            if venda_id:
                args.append(('expetc_vendas_capa_id', '=', venda_id))
            if data_inicial:
                args.append(('expetc_data_entrega', '>=', data_inicial))
            if data_final:
                args.append(('expetc_data_entrega', '<=', data_final))

            entrega_ids = obj_entregas_capa.search(cr, uid, args, context=context)
            for i in obj_entregas_capa.read(cr, uid, entrega_ids, ['expetc_vendas_capa_id'],
                                            context, '_classic_write'):
                venda_ids.append(i.get('expetc_vendas_capa_id'))

            if args_vendas and venda_ids:
                args_vendas.append(('id', 'in', venda_ids))
                venda_ids = obj_vendas_capa.search(cr, uid, args_vendas, context=context)

            datas.update({'venda_ids': venda_ids,
                          'totais': self.wizede_get_total_documento(cr, uid, venda_ids, context),
                          'cliente': demonstrativo.wizede_pessoa_id.nome or '',
                          'venda': demonstrativo.wizede_venda_id.expvec_documento or '',
                          'data_inicial': demonstrativo.wizede_data_inicial or '',
                          'data_final': demonstrativo.wizede_data_final or ''})
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_demonstrativo_entrega',
            'datas': datas,
        }

    def wizede_get_total_documento(self, cr, uid, venda_ids, context=None):
        """
        O metodo busca o valor da nota fiscal de cada uma das vendas identificadas em 'venda_ids'
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param venda_ids: lista de id da tabela 'expedicao_vendas_capa'
        :param context: dicionario com valores contextuais
        :return: dicionario. Cada chave representa um id da tabela 'expedicao_vendas_capa' e seu
        valor representa o total da nota fiscal
        """
        context = context and context.copy() or {}
        res = {}
        if not venda_ids:
            return res
        brw_expedicao_venda_capa = self.pool.get('expedicao.vendas.capa').browse(cr, uid, venda_ids,
                                                                                 context)
        for venda in brw_expedicao_venda_capa:
            if venda.expvec_nf_id:
                res[venda.id] = venda.expvec_nf_id.vr_total_nf or 0.0
        return res
