# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import wizvei_get_status, wizvei_get_status_qtde
from openerp.tools.translate import _

STATUS = [('A', 'A entregar'),
          ('P', 'Entregue parcialmente'),
          ('E', 'Entregue')]


class wizard_vendas_capa(osv.TransientModel):

    _name = 'wizard.vendas.capa'
    _rec_name = 'wizvec_documento'

    _columns = {
        'wizvec_documento': fields.char('Documento', size=20, required=True),
        'wizvec_data': fields.datetime('Data'),
        'wizvec_pessoa_id': fields.many2one("adm.pessoa", 'Cliente', required=True, select=True),
        'wizvec_nf_id': fields.many2one("fiscal.nf.capa", 'Nota fiscal', select=True),
        'wizvec_cupom_id': fields.many2one("ecf.cupons.capa", 'Cupom', select=True),
        'wizvec_vendas_itens_ids': fields.one2many('wizard.vendas.itens', 'wizvei_capa_id',
                                                   'Itens'),
        'status': fields.function(wizvei_get_status, string="Status", selection=STATUS,
                                  type='selection', method=True),
    }

    def acao_botao(self, cr, uid, ids, context=None):
        """
        Acao para os botoes de marcar e desmarcar todos os itens
        """

        context = context and context.copy() or {}

        if not ids:
            return False

        sql_itens_ids = """
            -- noinspection SqlDialectInspection
            SELECT id,
                   wizvei_quantidade_vendida AS vendida,
                   wizvei_quantidade_entregue AS entregue
            FROM wizard_vendas_itens
            WHERE wizvei_capa_id = %s""" % (ids[0] if isinstance(ids, list) else ids)
        cr.execute(sql_itens_ids)
        dados_itens = cr.dictfetchall()
        qtd_entregue = 0

        for dado in dados_itens:
            if context.get('acao') == 'M':
                vendida = dado.get('vendida')
                entregue = dado.get('entregue')
                qtd = float(vendida) - float(entregue)
            else:
                qtd = 0.0

            sql = """
                 -- noinspection SqlDialectInspection
                UPDATE wizard_vendas_itens
                 SET wizvei_quantidade_a_entregar = %s
                 WHERE id = %s""" % (qtd, dado.get('id'))
            cr.execute(sql)
            qtd_entregue += qtd

        return {
            'name': _('Confirmacao'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_id': ids[0] if isinstance(ids, list) else ids,
            'res_model': 'wizard.vendas.capa',
            'view_id': False,
            'context': context,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }

    def wizvei_button_marcar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['acao'] = 'M'
        return self.acao_botao(cr, uid, ids, context)

    def wizvei_button_desmarcar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context['acao'] = 'D'
        return self.acao_botao(cr, uid, ids, context)

    def wizvei_entregar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        ids = ids[0] if isinstance(ids, list) else ids
        qtd_entregue = 0

        sql = """
            -- noinspection SqlDialectInspection
            SELECT SUM (wizvei_quantidade_a_entregar) AS qtd
            FROM   wizard_vendas_itens
            WHERE  wizvei_capa_id = %s""" % ids
        cr.execute(sql)
        dados_itens = cr.dictfetchall()

        if dados_itens and isinstance(dados_itens, list):
            dados_itens = dados_itens[0]
            qtd_entregue = dados_itens.get('qtd')

        if not qtd_entregue:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha quantidade a entregar em nenhum item.'))

        return self.wizvei_criar_entrega(cr, uid, ids, context)

    def wizvei_criar_entrega(self, cr, uid, ids, context=None):
        """
        O metodo realiza a criacao de um novo registro da entrega e seus itens, tabelas
        expedicao_entregas_capa e expedicao_entregas_itens, respectivamente.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Dados de uma action para exibir novamente a treeview das vendas.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_expvei = self.pool.get('expedicao.vendas.itens')

        venda_id = context.get('venda_id')
        entrega_id = self.pool.get('expedicao.entregas.capa').create(
            cr, uid, {'expetc_vendas_capa_id': venda_id}, context)
        sql_itens_ids = '''
                    SELECT wvi.wizvei_vendas_itens_id,
                           wvi.wizvei_quantidade_vendida    AS vendida,
                           wvi.wizvei_quantidade_entregue   AS entregue,
                           wvi.wizvei_quantidade_a_entregar AS entregar
                    FROM wizard_vendas_itens           AS wvi
                    WHERE wvi.wizvei_capa_id = %s
                      AND wvi.wizvei_quantidade_a_entregar <> 0
                        ''' % ids
        cr.execute(sql_itens_ids)
        dados_itens = cr.dictfetchall()
        exp_item_ids = map(lambda x: '%s' % x.get('wizvei_vendas_itens_id'), dados_itens)
        cr.execute("""
                    SELECT evi.id,
                           eec.state
                    FROM expedicao_vendas_itens        AS evi 
                    LEFT JOIN expedicao_entregas_itens AS eei ON eei.expeti_vendas_itens_id = evi.id
                    LEFT JOIN expedicao_entregas_capa  AS eec ON eec.id = eei.expeti_capa_id
                    WHERE evi.id in (%s)""" % ','.join(exp_item_ids))
        dict_venda_itens = {}
        qtd_invalida_item_ids = []
        map(lambda x: dict_venda_itens.update({x.get('id'): x}), cr.dictfetchall())
        for item in dados_itens:
            qtd_entregar = item.get('entregar') or 0.0
            expvei_id = item.get('wizvei_vendas_itens_id')
            if not obj_expvei.espvei_valida_quantidade_entregue(cr, uid, expvei_id,
                                                                qtd_entregar, context):
                qtd_invalida_item_ids.append(expvei_id)
                continue
            sql_item_entrega = '''
                                  INSERT INTO expedicao_entregas_itens(
                                          expeti_capa_id, expeti_vendas_itens_id, expeti_quantidade)
                                  VALUES (%s, %s, %s)
                               ''' % (entrega_id, item.get('wizvei_vendas_itens_id'), qtd_entregar)
            cr.execute(sql_item_entrega)

        if qtd_invalida_item_ids:
            expedicao_vendas_itens_name = self.pool.get('expedicao.vendas.itens').name_get(
            cr, uid, qtd_invalida_item_ids, context)
            raise osv.except_osv(
                _('Atencao'),
                _('Quantidade entregue + quantidade informada para entrega maior que a quantidade '
                  'vendida:\n' + ' \n'.join(map(lambda i: i[1], expedicao_vendas_itens_name))))

        return {'res_id': context.get('active_id'),
                'view_type': 'form',
                'view_mode': 'tree,form',
                'res_model': 'expedicao.vendas.capa',
                'view_id': False,
                'type': 'ir.actions.act_window'}


class wizard_vendas_itens(osv.TransientModel):

    _name = 'wizard.vendas.itens'
    _columns = {
        'wizvei_capa_id': fields.many2one("wizard.vendas.capa", 'Capa', ondelete="cascade",
                                          required=True, select=True),
        'wizvei_vendas_itens_id': fields.many2one("expedicao.vendas.itens", 'Item',
                                                  ondelete="cascade", required=True, select=True),
        'wizvei_quantidade_vendida': fields.float('Quantidade vendida', digits=(8, 3),
                                                  required=True),
        'wizvei_quantidade_entregue': fields.float('Quantidade entregue', digits=(8, 3)),
        'wizvei_quantidade_a_entregar': fields.float('Quantidade a entregar', digits=(8, 3)),
        'wizvei_vendida_igual_entrega': fields.function(wizvei_get_status_qtde, method=True,
                                                        type='boolean'),
        'status': fields.selection(STATUS, 'Estado', size=1, required=True),
    }

    def onchange_wizvei_qtd_entregar(self, cr, uid, ids, item_id, qtd_venda, qtd_entregar,
                                     qtd_entregue, context=None):
        context = context and context.copy() or {}

        if not qtd_entregar:
            return {}

        res = {}
        warning = {}

        dif = qtd_venda - (qtd_entregar + qtd_entregue)
        # não se pode comparar 2 floats, deve comparar a diferenca entre eles
        if dif < -0.001 and 0.001 < dif or not qtd_entregar > 0:
            res['wizvei_quantidade_a_entregar'] = 0
            warning['title'] = 'Atencao'
            warning['message'] = _('Quantidade informada nao e valida.')

        return {'value': res, 'warning': warning}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
