# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from SigUtils.formatastring import formataString
from openerp.addons.adm import print_log
from openerp.tools.translate import _


class ecf_cupons_capa(osv.Model):
    _name = 'ecf.cupons.capa'
    _inherit = 'ecf.cupons.capa'

    def ecfccp_trata_expedicao_cupom_venda(self, cr, uid, brw_cupom, context=None):
        """
        O metodo gera a expedicao para cupons de venda.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param cupom_id: id da tabela do objeto atual.
        :param context: dicionario com valores contextuais.
        :return: booleano True
        """
        context = context and context.copy() or {}
        res = super(ecf_cupons_capa, self).ecfccp_trata_expedicao_cupom_venda(cr, uid, brw_cupom,
                                                                              context)
        expedicao_vendas_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                  'expedicao.vendas.capa', context=context, user_company_id=brw_cupom.company_id.id)
        obj_expedicao_vendas_capa = self.pool.get('expedicao.vendas.capa')
        obj_expedicao_vendas_itens = self.pool.get('expedicao.vendas.itens')
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        context_exp = context.copy()
        context_exp['sig_force_company'] = True
        exp_ven_capa_ids = self.pool.get('expedicao.vendas.capa').search(cr, uid,
                                      [('expvec_cupom_id', '=', brw_cupom.id)], context=context_exp)
        if exp_ven_capa_ids:
            return res

        if not brw_cupom.cliente_id:  # cria expedicao apenas para os cupons com cliente
            return res

        brw_ecf = brw_cupom.ecf_id
        loja = str(brw_ecf.loja)
        caixa = str(brw_ecf.caixa)
        coo = str(brw_cupom.coo)
        documento = loja + '-' + caixa + '-' + coo
        vals_capa = {'company_id': expedicao_vendas_company_id,
                     'expvec_cupom_id': brw_cupom.id,
                     'expvec_documento': documento,
                     'expvec_data': brw_cupom.hora_cupom_fim,
                     'expvec_pessoa_id': brw_cupom.cliente_id.id}
        exp_ven_capa_id = obj_expedicao_vendas_capa.create(cr, uid, vals_capa,
                                                           context_exp)
        print_log(_('- Criando expedicao cupom %s') % coo)
        barra_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                 'automacao.codigo.barra', context=context, user_company_id=brw_cupom.company_id.id)
        context_barra = context.copy()
        context_barra['sig_force_company'] = True
        for item in brw_cupom.ecf_cupons_item_ids:
            if item.venda_cancelamento == 'V':
                codigo_barra = item.item
                auto_cod_bar_ids = obj_automacao_codigo_barra.search(cr, uid,
                             [('codigo_barra', '=', codigo_barra),
                              ('company_id', '=', barra_company_id)], context=context_barra)
                brw_codigo_barra = obj_automacao_codigo_barra.browse(cr, uid, auto_cod_bar_ids,
                                                                     context)
                if brw_codigo_barra:
                    if isinstance(brw_codigo_barra, list):
                        brw_codigo_barra = brw_codigo_barra[0]
                    vals_item = {
                        'expvei_capa_id': exp_ven_capa_id,
                        'expvei_grade_id': brw_codigo_barra.grade_id.id,
                        'expvei_quantidade_vendida': item.quantidade,
                        'status': 'A',
                        'expvei_embalagem_id': brw_codigo_barra.produto_embalagem_id.id}
                    obj_expedicao_vendas_itens.create(cr, uid, vals_item, context)

        return res

    def ecfccp_trata_expedicao_cupom_cancelamento(self, cr, uid, brw_cupom, context=None):
        """
        O metodo exclui da expedicao os cupons de venda que foram cancelados.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param brw_cupom: browse da tabela 'ecf_cupons_capa'
        :param context: dicionario com velores contextuais.
        :return: booleano True
        """
        context = context and context.copy() or {}
        res = super(ecf_cupons_capa, self).ecfccp_trata_expedicao_cupom_cancelamento(
            cr, uid, brw_cupom, context)
        obj_expedicao_vendas_capa = self.pool.get('expedicao.vendas.capa')
        brw_ecf = brw_cupom.ecf_id
        coo = int(brw_cupom.coo) - 1  # coo do cupom de venda
        args = [('coo', '=', coo), ('ecf_id', '=', brw_ecf.id)]
        cupom_ids = self.search(cr, uid, args, context=context)
        if cupom_ids:
            exp_ven_capa_ids = obj_expedicao_vendas_capa.search(
                cr, uid, [('expvec_cupom_id', 'in', cupom_ids)], context=context)
            if exp_ven_capa_ids:
                obj_expedicao_vendas_capa.unlink(cr, uid, exp_ven_capa_ids, context)
                print_log(_('- Excluindo expedicao cupom %s cancelado') % coo)

        return res
