# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.parser import parse
import time
from openerp.tools.translate import _


class wizard_fidelidade_extrato(osv.TransientModel):
    # prefixo: wfidex_

    _name = 'wizard.fidelidade.extrato'
    _columns = {
        'wfidex_data_inicial': fields.date('Data inicial', required=True),
        'wfidex_data_final': fields.date('Data final', required=True),
        'wfidex_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
    }

    def wfidex_button_print(self, cr, uid, ids, context=None):
        """
        O metodo imprime o relatorio com o extrato do movimento fidelidade.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario cujos dados representam uma acao para a chamada da classe do relatorio.
        :rtype: dict
        """
        context = context and context.copy() or {}
        dados_movimento = list()
        obj_fidelidade_movimento = self.pool.get('fidelidade.movimento')
        dados = dict()
        for extrato in self.browse(cr, uid, ids, context):
            args = [('data', '>=', extrato.wfidex_data_inicial + ' 00:00:00'),
                    ('data', '<=', extrato.wfidex_data_final + ' 23:59:59')]
            if extrato.wfidex_pessoa_id:
                args.append(('pessoa_id', '=', extrato.wfidex_pessoa_id.id))
            fidelidade_movimento_ids = obj_fidelidade_movimento.search(cr, uid, args,
                                                                       context=context)
            if not fidelidade_movimento_ids:
                continue
            for movimento in obj_fidelidade_movimento.browse(cr, uid, fidelidade_movimento_ids,
                                                             context):
                vals = {'codigo': movimento.pessoa_id.codigo,
                        'nome': movimento.pessoa_id.nome,
                        'data': movimento.data, 'pontos': movimento.pontos or 0,
                        'descricao': movimento.name}
                vals.update(obj_fidelidade_movimento.fidmov_get_dados_documento(
                    cr, uid, movimento.id, context))
                dados_movimento.append(vals)
            dados = {'data_impressao': time.strftime('%d/%m/%Y'),
                     'data_inicial': parse(extrato.wfidex_data_inicial).strftime('%d/%m/%Y'),
                     'data_final': parse(extrato.wfidex_data_final).strftime('%d/%m/%Y'),
                     'dados_movimento': dados_movimento}
        if not dados_movimento:
            raise osv.except_osv(_('Atencao'), _('Para os filtros selecionados, nao ha dados a '
                                                 'serem impressos'))
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'fidelidade_extrato',
            'datas': dados
        }
