# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from SigUtils.formatastring import formataString
from openerp.addons.adm import print_log
from openerp.tools.translate import _


class ecf_cupons_capa(osv.Model):
    _name = 'ecf.cupons.capa'
    _inherit = 'ecf.cupons.capa'

    def _auto_init(self, cr, context=None):
        res = super(ecf_cupons_capa, self)._auto_init(cr, context)
        cr.execute("SELECT id FROM res_users WHERE login = 'ecf_remoto' ")
        dados = cr.dictfetchall()
        if not dados:
            return res

        uid = dados[0].get('id')
        cr.execute("SELECT id FROM res_groups WHERE name = 'Fidelidade / Manager' ")
        dados = cr.dictfetchall()
        if not dados:
            return res

        gid = dados[0].get('id')
        cr.execute('SELECT * FROM res_groups_users_rel WHERE uid = %s AND gid = %s' % (uid, gid))
        if cr.dictfetchall():
            return res

        try:
            cr.execute('INSERT INTO res_groups_users_rel (uid, gid) VALUES(%s, %s)' % (uid, gid))
        except Exception as e:
            print_log(e, 'E')

        return res

    def get_vals_fidelidade(self, cr, uid, brw_cupom, mov_id, venda_cancelamento ,context=None):
        '''
        :param venda_cancelamento: True para cupom de venda, False para cupom de cancelamento
        '''
        context = context and context.copy() or {}
        if not brw_cupom.cliente_id:
            return {}

        try:
            context.pop('sig_force_company')
        except:
            pass

        valor_pontos = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                                                'fidelidade.parametros', 'valor_pontos',
                                                company_id=brw_cupom.company_id.id,
                                                context=context).get('valor_pontos') or 0
        fm_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                                    cr, uid, 'fidelidade.movimento', context=context,
                                    user_company_id=brw_cupom.company_id.id)
        venda_cancel = venda_cancelamento and 'Compra' or 'Cancelamento'
        name = '%s de Cupom fiscal-%s-%s-%s' % (venda_cancel, brw_cupom.ecf_id.loja,
                                                brw_cupom.ecf_id.caixa,brw_cupom.coo)

        data_movimento = brw_cupom.date_cupom
        estmvi_ids = self.pool.get('estoque.movimento.itens').search(cr, uid,
                                             [('cupom_id','=',brw_cupom.id),
                                              ('item_cancelado', '=', not venda_cancelamento)],
                                                        context=context)
        pontos_extras = self.pool.get('fidelidade.produtos').get_pontos_extras(cr, uid, estmvi_ids,
                                                                    data_movimento, context=context)
        # os pontos sao truncados, ou seja se vender 1.5 e o valor for 1, sera 1 ponto e nao 1,5
        pontos = int(brw_cupom.total_cupom / valor_pontos) + pontos_extras
        vals = {'company_id': fm_company_id,
                'movimento_id': mov_id,
                'pessoa_id': brw_cupom.cliente_id.id,
                'data': brw_cupom.hora_cupom_inicio,
                'name': name,
                'pontos': pontos,
        }
        return vals

    def ecfccp_trata_fidelidade_cupom_venda(self, cr, uid, brw_cupom, mov_id, context=None):
        """
        O metodo gera a expedicao para cupons de venda.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param cupom_id: id da tabela do objeto atual.
        :param mov_id: id da tabela 'estoque.movimento.capa'
        :param context: dicionario com valores contextuais.
        :return: booleano True
        """
        context = context and context.copy() or {}
        res = super(ecf_cupons_capa, self).ecfccp_trata_fidelidade_cupom_venda(cr, uid, brw_cupom,
                                                                            mov_id, context=context)
        vals = self.get_vals_fidelidade(cr, uid, brw_cupom, mov_id, True, context=context)
        if vals.get('pontos') > 0:
            self.pool.get('fidelidade.movimento').create(cr, uid, vals, context=context)

        return res

    def ecfccp_trata_fidelidade_cupom_cancelamento(self, cr, uid, brw_cupom, mov_id, context=None):
        """
        O metodo exclui da expedicao os cupons de venda que foram cancelados.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param brw_cupom: browse da tabela 'ecf_cupons_capa'
        :param mov_id: id da tabela 'estoque.movimento.capa'
        :param context: dicionario com velores contextuais.
        :return: booleano True
        """
        context = context and context.copy() or {}
        res = super(ecf_cupons_capa, self).ecfccp_trata_fidelidade_cupom_cancelamento(
            cr, uid, brw_cupom, mov_id, context=context)

        vals = self.get_vals_fidelidade(cr, uid, brw_cupom, mov_id, False, context=context)
        if vals.get('pontos') > 0:
            vals['pontos'] = vals.get('pontos') * -1 # estorna os pontos do cupom de venda
            self.pool.get('fidelidade.movimento').create(cr, uid, vals, context=context)

        return res
