# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv


class fiscal_nf_capa(osv.Model):
    _name = 'fiscal.nf.capa'
    _inherit = 'fiscal.nf.capa'

    def get_vals_fidelidade(self, cr, uid, brw_nf, venda_cancelamento ,context=None):
        '''
        :param venda_cancelamento: True para cupom de venda, False para cupom de cancelamento
        '''
        context = context and context.copy() or {}
        # gera os pontos somente para nota de saida, onde o tipo da natureza seja saida, o tipo
        # do movimento seja compra/venda e a transacao seja com comercializacao
        if brw_nf.tipo != 'S' or brw_nf.natureza_id.cfop_id.tipo != 'S' or \
                brw_nf.natureza_id.cfop_id.movimento_tipo != 'C' or \
                brw_nf.natureza_id.cfop_id.transacao_tipo != 'C':
            return {}

        try:
            context.pop('sig_force_company')
        except:
            pass

        valor_pontos = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                                                'fidelidade.parametros', 'valor_pontos',
                                                company_id=brw_nf.company_id.id,
                                                context=context).get('valor_pontos') or 0
        fm_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                                    cr, uid, 'fidelidade.movimento', context=context,
                                    user_company_id=brw_nf.company_id.id)
        name = 'Compra por nota fiscal %s-%s' % (brw_nf.numero_nf, brw_nf.serie_nf)
        mov_ids = self.pool.get('estoque.movimento.capa').search(
            cr, uid, [('nf_capa_id', '=', brw_nf.id)], context=context)
        if not mov_ids:
            return {}

        data_movimento = brw_nf.data_emissao
        estmvi_ids = self.pool.get('estoque.movimento.itens').search(cr, uid,
                                                         [('nf_id','=',brw_nf.id)], context=context)
        pontos_extras = self.pool.get('fidelidade.produtos').get_pontos_extras(cr, uid, estmvi_ids,
                                                                    data_movimento, context=context)
        # os pontos sao truncados, ou seja se vender 1.5 e o valor for 1, sera 1 ponto e nao 1,5
        pontos = 0
        if valor_pontos:
            pontos = int(brw_nf.vr_total_nf / valor_pontos) + pontos_extras

        vals = {'company_id': fm_company_id,
                'movimento_id': mov_ids[0],
                'pessoa_id': brw_nf.destinatario_id.id,
                'data': data_movimento,
                'name': name,
                'pontos': pontos,
        }
        return vals

    def fisnfc_faturar_nf_erp(self, cr, uid, ids, context=None):
        """
        Gera a expedicao para cupons de venda.

        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista com o ID da nota fiscal
        :param context: dicionario com valores contextuais.
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: dicionario com dados da nota fiscal
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(fiscal_nf_capa, self).fisnfc_faturar_nf_erp(cr, uid, ids, context=context)
        for brw_nf in self.browse(cr, uid, ids, context):
            # Pula notas nao faturadas ou sem cliente
            if res.get('state') != '3' or not brw_nf.destinatario_id.id:
                continue

            vals = self.get_vals_fidelidade(cr, uid, brw_nf, True, context=context)
            if vals.get('pontos') > 0:
                self.pool.get('fidelidade.movimento').create(cr, uid, vals, context=context)

        return res

    def fisnfc_funcao_button_cancelar(self, cr, uid, ids, context=None):
        """
        O metodo exclui os pontos de fidelidade referente a(s) nota(s) cancelada(s)
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param nf_ids: ids da tabela fiscal_nf_capa
        :param context: dicionario com velores contextuais.
        :return: booleano True
        """
        context = context and context.copy() or {}
        res = super(fiscal_nf_capa, self).fisnfc_funcao_button_cancelar(cr, uid, ids, context=context)
        if not isinstance(ids, list):
            nf_ids = [ids]
        else:
            nf_ids = ids

        mov_ids = self.pool.get('estoque.movimento.capa').search(cr, uid,
                                                   [('nf_capa_id','in', nf_ids)], context=context)
        if not mov_ids:
            return res

        fidelidade_id = self.pool.get('fidelidade.movimento').search(cr, uid,
                                                   [('movimento_id','in',mov_ids)], context=context)
        if fidelidade_id:
            self.pool.get('fidelidade.movimento').unlink(cr, uid, fidelidade_id,context=context)

        return res
