# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import re
from SigUtils import inscricao_estadual
from openerp.tools.translate import _


class financeiro_cliente(osv.Model):
    _name = 'financeiro.cliente'
    _inherit = 'financeiro.cliente'

    def verificar_incluir_cliente(self, cr, uid, pessoa_id, company_id, acao='verificar', context=None):
        """
        Este metodo e usado para verificar se a Pessoa ja está cadastrada como Cliente na Empresa informada.
        Efetua cadastro automatico de Cliente dependendo do Parametro do Financeiro
        :param cr: database cursor
        :param uid: usuário atual
        :param pessoa_id: id da pessoa
        :param company_id: id da empresa
        :param acao: 'verificar'/'incluir' - se incluir cadastra a pessoa_id como cliente
        :param context: argumentos do contexto
        :rtype: boolean
        """
        # Veririca se o objeto <type 'str'> acao, e igual a 'verificar'/'incluir'
        # senao, gerar um AssertionError.
        context = context and context.copy() or {}
        try:
            assert acao in ('verificar', 'incluir')
        except AssertionError:
            raise Exception(_("AssertionError: valor para a acao deve ser 'verificar' ou 'incluir'"))

        context = context and context.copy() or {}

        # verificar se a pessoa e cliente

        cliente_ids = self.search(cr, uid, [('pessoa_id', '=', pessoa_id),
                                            ('company_id', '=', company_id)], context=context)
        if cliente_ids:
            return True

        inativos = self.search(cr, uid, [('pessoa_id', '=', pessoa_id),
                                         ('company_id', '=', company_id),
                                         ('active','=',False)],
                                         context=context)
        if inativos:
            raise osv.except_osv(_('Atencao'), _('Cliente inativo.'
                                                 'Para utilizar este cliente e necessario '
                                                 'ativa-lo'))
        # objeto financeiro.parametros
        obj_par_financ = self.pool.get('financeiro.parametros')
        par_financ_id = obj_par_financ.search(cr, uid, [], context=context)
        # possui parametros cadastrados
        if not par_financ_id:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao a parametros cadastrado no financeiro p/ essa empresa'))

        res = False
        cadastro_automatico_cliente = obj_par_financ.read(cr, uid, par_financ_id[0],
                        ['cadastro_automatico_cliente'], context).get('cadastro_automatico_cliente')
        if (cadastro_automatico_cliente and acao == 'incluir') or (
                acao == 'incluir' and context.get('pedcap_simplificado')):
            res = True
            # cadastra a pessoa como cliente
            self.create(cr, uid, {'pessoa_id': pessoa_id, 'company_id': company_id}, context)

        return res

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        if name.isdigit():
            pessoa_ids = obj_adm_pessoa.search(cr, uid, [('codigo', '=', name)], context=context)
        else:
            pessoa_ids = obj_adm_pessoa.search(cr, uid, [('nome', operator, name)], context=context)
        cliente_ids = self.search(cr, uid, [('pessoa_id', 'in', pessoa_ids)], context=context)
        return self.name_get(cr, uid, cliente_ids, context)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        brw_cliente = self.browse(cr, uid, ids, context=context)
        for cliente in brw_cliente:
            codigo = cliente.pessoa_id.codigo or ''
            name = cliente.pessoa_id.name
            if context.get('fincli_retorno_sem_codigo'):
                codigo = ''
            result = "%s%s%s" % (codigo, ' - ' if codigo else '', name)
            res.append((cliente.id, result))
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(financeiro_cliente, self).write(cr, uid, ids, vals, context)
        return res

    def onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        obj_adm_empresa = self.pool.get('adm.empresa.objeto')
        company_id = obj_adm_empresa.getEmpresa(cr, uid, 'financeiro.cliente', args=None,
                                                context=context)
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        if not pessoa_tipo:
            warning['title'] = _('Atencao')
            warning['message'] = _('Tipo de pessoa nao informado!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        pessoa_ids = obj_pessoa.search(cr, uid,
                                       [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar), ('pessoa_tipo', '=', pessoa_tipo)],
                                       context=context)
        if pessoa_ids:
            cliente_id = obj_financeiro_cliente.search(cr, uid,
                                                       [('pessoa_id', 'in', pessoa_ids), ('active', '=', True)],
                                                       context=context)
            if cliente_id:
                warning = {'title': 'Atencao'}
                warning['message'] = 'Cliente ja cadastrado!'
                return {'warning': warning, 'value': {'cpf_cnpj': ''}}
            dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
            for pessoa_id in pessoa_ids:
                self.verificar_incluir_cliente(cr, uid, pessoa_id, company_id, 'verificar',
                                               context=context)
            if isinstance(dados_pessoa, list):
                dados_pessoa = dados_pessoa[0]
            endereco_ids = []
            for dados_endereco in dados_pessoa.endereco_ids:
                enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                             'active': dados_endereco.active,
                             "cep": dados_endereco.cep,
                             "pais_id": dados_endereco.pais_id.id,
                             "uf_id": dados_endereco.uf_id.id,
                             "cidade_id": dados_endereco.cidade_id.id,
                             "bairro_id": dados_endereco.bairro_id.id,
                             "logradouro_id": dados_endereco.logradouro_id.id,
                             "numero": dados_endereco.numero,
                             "complemento": dados_endereco.complemento,
                             "observacao": dados_endereco.observacao,
                }
                endereco_ids.append((1, dados_endereco.id, enderecos))
            email_ids = []
            for dados_emails in dados_pessoa.email_ids:
                emails = {
                    "email_tipo_id": dados_emails.email_tipo_id.id,
                    "name": dados_emails.name,
                    "observacao": dados_emails.observacao,
                }
                email_ids.append((1, dados_emails.id, emails))
            telefones_ids = []
            for dados_telefones in dados_pessoa.telefones_ids:
                telefones = {
                    "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                    "ddi": dados_telefones.ddi,
                    "ddd": dados_telefones.ddd,
                    "numero": dados_telefones.numero,
                    "observacoes": dados_telefones.observacoes,
                }
                telefones_ids.append((1, dados_telefones.id, telefones))

            modulo_fiscal_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context=None)
            endereco_nf_id = dados_pessoa.endereco_nf_id.id if modulo_fiscal_instalado else False

            res = {'value': {'cpf_cnpj': cpf_cnpj,
                             'name': dados_pessoa.name,
                             'codigo': dados_pessoa.codigo,
                             'fantasia_apelido': dados_pessoa.fantasia_apelido,
                             'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                             'admpes_fis_rg_orgao_emissor': dados_pessoa.admpes_fis_rg_orgao_emissor,
                             'admpes_fis_rg_data_emisssao': dados_pessoa.admpes_fis_rg_data_emisssao,
                             'admpes_fis_cnh_nro': dados_pessoa.admpes_fis_cnh_nro,
                             'admpes_fis_cnh_orgao_emissor': dados_pessoa.admpes_fis_cnh_orgao_emissor,
                             'admpes_fis_cnh_validade': dados_pessoa.admpes_fis_cnh_validade,
                             'admpes_fis_cnh_categoria': dados_pessoa.admpes_fis_cnh_categoria,
                             'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                             'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                             'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                             'admpes_jur_data_fundacao': dados_pessoa.admpes_jur_data_fundacao,
                             'admpes_jur_tipo_empresa': dados_pessoa.admpes_jur_tipo_empresa,
                             'admpes_jur_suframa': dados_pessoa.admpes_jur_suframa,
                             'admpes_jur_regime_tributario': dados_pessoa.admpes_jur_regime_tributario,
                             'admpes_jur_tipo_atividade': dados_pessoa.admpes_jur_tipo_atividade,
                             'admpes_jur_ie': dados_pessoa.admpes_jur_ie,
                             'admpes_jur_cnae_id': dados_pessoa.admpes_jur_cnae_id.id,
                             "foto": dados_pessoa.foto,
                             'endereco_ids': endereco_ids,
                             'email_ids': email_ids,
                             'telefones_ids': telefones_ids,
                             'pessoa_id': dados_pessoa.id,
                             'endereco_nf_id': endereco_nf_id
            }
            }
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}

    def onchange_ie(self, cr, uid, ids, admpes_jur_ie):
        res = {}
        estado = False
        for partner in self.browse(cr, uid, ids):
            estado = partner.uf.upper()
        if admpes_jur_ie and admpes_jur_ie.isdigit() and estado:
            res = inscricao_estadual.validar(estado, admpes_jur_ie)
            if not res:
                return {'warning': {'title': _('Atencao'),
                                    'message': _('Inscricao Estadual invalida para o seguinte estado: %s') % estado}}
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        context.update({'sig_search_sem_filhos': True})
        if not args:
            args = []
        for x in args:
            if 'cep' in x:
                try:
                    cep = x[2]
                    cep = str(cep)
                    cep = '%s.%s-%s' % (cep[0:2], cep[2:5], cep[5:])
                    x[2] = cep
                except:
                    pass
        res = super(financeiro_cliente, self).search(cr, uid, args, offset, limit, order, context,
                                                     count)
        return res

    def cliente_ativo(self, cr, uid, ids, context=None):
        """
        Verifica se o cliente esta ou nao ativo.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se o cliente estiver ativo.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        context['active_test'] = False  # Permite recuperar os registros inativos
        ids = [ids] if not isinstance(ids, (list, tuple)) else ids
        args = [('pessoa_id', 'in', ids)]
        pessoa_ids = self.search(cr, uid, args, context=context)
        dados_pessoa = self.read(cr, uid, pessoa_ids or 0, ['active'], context)

        if dados_pessoa and isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]

        # Se nao foi encontrado, nao ha como estar inativo (retorna verdadeiro)
        return dados_pessoa.get('active', True) if dados_pessoa else True

    def fincli_onchange_tipo_atividade(self, cr, uid, ids, pessoa_tipo, tipo_atividade,
                                       context=None):
        """
        Metodo chamado na saida nos atributos pessoa_tipo e admpes_jur_tipo_atividade. Valida a
        natureza da atividade para pessoas fisicas e juridica.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param pessoa_tipo: string que representa a pessoa fisica ('F') ou juridica ('J').
        :type pessoa_tipo: str
        :param tipo_atividade: string que representa a natureza da atividade exercida pela empresa.
        :type tipo_atividade: str
        :param context: dados de valores contextuais.
        :type context: dict
        :return: dados dos atributos com seus respectivos valores e/ou mensagem de alerta ao
        usuario.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not tipo_atividade:
            return res
        if pessoa_tipo == 'F' and tipo_atividade != 'T':
            res['value'] = {'admpes_jur_tipo_atividade': ''}
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Natureza da atividade invalida para pessoa fisica')}
        return res

    def fincli_get_compra_dados(self, cr, uid, pessoa_id, context=None):
        """
        Metodo realiza busca de estatisticas no faturamento de notas fiscais a respeito de compras
        realizadas pelos clientes.
        Metodo sbrescrito no modulo fiscal.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: dados com valores e datas das compras realizadas.
        :rtype: dict
        """
        return dict()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

