# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp import pooler


class financeiro_fatura(osv.Model):
    _name = 'financeiro.fatura'
    _inherit = 'financeiro.fatura'

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(financeiro_fatura, self).default_get(cr, uid, fields, context=context)
        if context.get('lancamento_fatura_r'):
            res['tipo_fatura'] = 'R'
        if context.get('lancamento_fatura_p'):
            res['tipo_fatura'] = 'P'
        financeiro_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', ['usar_centro_custos_e_eventos'], context=context)
        res['finfat_gera_centro_custo_evento'] = financeiro_parametro.get(
            'usar_centro_custos_e_eventos')
        return res

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __code = 'cod_fatura_nf'
        __name = 'Codigo de Fatura'
        vals = {
            'name': __name,
            'code': __code,
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence.type').search(cr, uid, [('code', '=', __code)],
                                                            context=context)
        if not consulta:
            self.pool.get('ir.sequence.type').create(cr, uid, {'name': __name, 'code': __code},
                                                     context)
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        res = super(financeiro_fatura, self).fields_view_get(cr, uid, view_id, view_type, context,
                                                             toolbar, submenu)

        if context.get('lancamento_fatura_p', False) == True:
            """
                Para mostrar onde for faturas a pagar a string do pessoa_id = Fornecedor
            """
            res['fields']['pessoa_id']['string'] = res['fields']['pessoa_id']['string'].replace(
                'Cliente', 'Fornecedor')

        return res

    def onchange_parametro_empresa(self, cr, uid, ids, company_id):
        '''Busca  em financeiro.parametros da empresa, preenche os campos e libera permissoes de acordo com os parametros'''
        id_parametro = self.pool.get('financeiro.parametros').search(cr, uid, [])
        if not id_parametro:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao foi cadastrado parametros para esta empresa.'))

    def finfat_atualiza_competencia_gerencial_movimento(self, cr, uid, ids,
                                                        competencia_por_parcelas, context):
        """
        O metodo reune os dados para que os registros de competencia (mes / ano) da tabela
        gerencial_movimento sejam atualizados. Se competencia_por_parcelas for True, os registros
        serao atualizados baseado na data de vencimento dos titulos, senao, na data de emissao.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param ids: lista de id da tabela desse objeto
        :param competencia_por_parcelas: booleano
        :param context: dicionario com valores contextais
        :return: booleano, com True caso algum registro da tabela gerencial_movimento foi
        atualizado e False caso nenhum foi atualizado
        """
        context = context and context.copy() or {}
        if not ids:
            return False
        financeiro_fatura_dados = self.read(cr, uid, ids, ['titulo_ids'], context)
        titulo_ids = []
        map(lambda i: titulo_ids.extend(i.get('titulo_ids')), financeiro_fatura_dados)
        return self.pool.get('gerencial.movimento').fingmo_atualiza_competencia(
            cr, uid, titulo_ids, competencia_por_parcelas, context)

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('futil_super'):
            del context['futil_super']
            return super(financeiro_fatura, self).create(cr, uid, vals, context)

        obj_financeiro_utils = self.pool.get('financeiro')
        obj_documento_documento_tipo = self.pool.get('documento.documento.tipo')
        obj_adm_utils = self.pool.get('adm.utils')
        finfat_pessoa_id = vals.get('pessoa_id')
        finfat_data_emissao = vals.get('data_emissao')
        finfat_tipo_fatura = vals.get('tipo_fatura')
        finfat_valor = vals.get('valor') or 0
        finfat_nf_id = vals.get('nf_id') or False
        finfat_documento = vals.get('documento') or False
        # SE HOUVER NOTA FISCAL, NÃO SERÁ NECESSARIO PASSAR O CUPOM_ID (Segundo Luiz)
        finfat_cupom_id = False if finfat_nf_id else vals.get('cupom_id')
        evento_id = False
        centro_custo_id = False
        finfat_documento_tipo_id = vals.get('finfat_documento_tipo_id') or False
        if finfat_documento_tipo_id:
            browse_documento_tipo = obj_documento_documento_tipo.browse(cr, uid,
                                                                        finfat_documento_tipo_id)
            centro_custo_id = browse_documento_tipo.docdot_centro_custo_id.id
            evento_id = browse_documento_tipo.docdot_evento_id.id
        if (not centro_custo_id or not evento_id) and finfat_tipo_fatura == 'R':
            campos = ['finpar_centro_custo_entr_tit_rec_id', 'finpar_evento_entr_tit_rec_id']
            financeiro_parametros_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'financeiro',
                                                                              campos,
                                                                              context=context)
            centro_custo_id = financeiro_parametros_dados.get('finpar_centro_custo_entr_tit_rec_id')
            evento_id = financeiro_parametros_dados.get('finpar_evento_entr_tit_rec_id')
        gerencial_mov = False
        if evento_id and centro_custo_id:
            gerencial_mov = {
                'centro_custo_id': centro_custo_id,
                'evento_id': evento_id,
                'mes_competencia': str(vals.get('data_emissao')[5:7]),
                'ano_competencia': str(vals.get('data_emissao')[0:4]),
            }
        elif not vals.get('finfat_gera_centro_custo_evento'):
            gerencial_mov = False
            context['ctx_fisnfc_titulo_create_sem_custificacao'] = True

        if vals.get('titulo_ids'):
            # CRIA UMA LISTA COM DICIONARIOS CONTENDO A DATA DE VENCIMENTO E O VALOR DO TITULO PARA SER PASSADO PARA O FUNÇÃO
            # O APPEND NA LISTA É FEITO NA ORDER CRESCENTE DOS VENCIMENTOS.
            dict_titulo_venc_valor = {}
            for i in vals['titulo_ids']:
                data_vencimento = i[2]['data_prorrogacao']
                valor = i[2]['vr_original']
                gerencial_movimento = gerencial_mov
                if gerencial_mov:
                    gerencial_mov.update({'valor': valor})
                    gerencial_movimento = [(0, 0, gerencial_mov)]
                elif finfat_tipo_fatura == 'P' and i[2].get('fintit_gerencialmov_ids'):
                    gerencial_movimento = i[2].get('fintit_gerencialmov_ids')
                parcela = i[2]['parcela']
                if dict_titulo_venc_valor.get(data_vencimento):
                    dict_titulo_venc_valor[data_vencimento].append({'valor': valor,
                                                                    'custificacao': gerencial_movimento,
                                                                    'data_vencimento': data_vencimento,
                                                                    'parcela': parcela})
                else:
                    dict_titulo_venc_valor[data_vencimento] = [
                        {'valor': valor, 'custificacao': gerencial_movimento,
                         'data_vencimento': data_vencimento,
                         'parcela': parcela}]

            keys_sorted = dict_titulo_venc_valor.keys()
            keys_sorted.sort()
            fatura_titulos = []
            for i in keys_sorted:
                for t in dict_titulo_venc_valor[i]:
                    titulo_gerencial = t.get('custificacao')
                    titulo_dados = {'vencimento': t.get('data_vencimento'),
                                    'valor': t.get('valor') or 0,
                                    'parcela': t.get('parcela', 0),
                                    'doc_tipo_id': finfat_documento_tipo_id}
                    if titulo_gerencial:
                        titulo_dados.update({'fintit_gerencialmov_ids': titulo_gerencial})
                    fatura_titulos.append(titulo_dados)

        else:
            fatura_titulos = False
        res_id = obj_financeiro_utils.financ_incluir_fatura(
            cr, uid,
            # CAMPOS ABAIXO OBRIGATORIOS
            futl_fat_pessoa_id=finfat_pessoa_id,
            futl_fat_data_emissao=finfat_data_emissao,
            futl_fat_valor=finfat_valor,
            futl_fat_tipo_fatura=finfat_tipo_fatura,
            # CAMPOS ABAIXO POSSUEM DEFAULT(Nao sao obrigatorios)
            futl_fat_titulo_dados=fatura_titulos,
            # LISTA COM DICIONARIOS (CRIA OS TITULOS)-
            # {KEY='vencimento': VALUE=data, KEY='valor': VALUE=valor}
            futl_fat_obs=False,  # OBS PARA O TITULO
            futl_fat_moeda=False,
            # BUSCA A MOEDA A SER UTILIZADA DENTRO DA FUNÇÃO SE NÃO FOR INFORMADO
            futl_fat_doc_tipo_id=finfat_documento_tipo_id,
            # ALTERA O TIPO DE DOCUMENTO PARA DUPLICATA DENTRO DA FUNÇÃO SE NÃO FOR INFORMADO
            futl_fat_documento=finfat_documento,
            # BUSCA O Nro DOCUMENTO DENTRO DA FUNÇÃO SE NÃO FOR INFORMADO
            futl_fat_tipo_sacado=False,
            # VALIDADO DENTRO DA FUNÇÃO POR _financ_validar_tipo_sacado
            futl_fat_vendedor_id=False,
            # VENDEDOR A SER UTILIZADO NO TITULO
            futl_fat_vendedor_externo_id=False,
            # VENDEDOR EXTERNO A SER UTILIZADO NO TITULO
            futl_fat_tipo_lancamento=False,
            # TIPO DE LANÇAMENTO TITULO (M - MANUAL, A - AUTOMATICO)
            futl_fat_cupom_id=finfat_cupom_id,  #
            futl_fat_nf_id=finfat_nf_id,
            # NF DA FATURA
            futl_fat_company_id=vals.get(
                'company_id'),
            futl_competencia_parcela=vals.get('finfat_competencia_por_parcelas'),
            context=context)
        if not res_id:
            msg = obj_financeiro_utils.financ_get_erro()
            if not context.get('importacao_supervisor_erp'):
                raise osv.except_osv(_('Atencao'), msg)
        else:
            titulo_ids = self.browse(cr, uid, res_id).titulo_ids
            if titulo_ids:
                titulo_ids = map(lambda i: i.id, titulo_ids)
                if titulo_ids:
                    obj_gerencial_movimento = self.pool.get('gerencial.movimento')
                    gerencial_movimento_ids = obj_gerencial_movimento.search(cr, uid,
                                                                             [('titulo_id', 'in',
                                                                               titulo_ids)], context=context)
                    if gerencial_movimento_ids:
                        obj_gerencial_movimento.write(cr, uid, gerencial_movimento_ids, {}, context=context)
            self.finfat_atualiza_competencia_gerencial_movimento(
                cr, uid, [res_id], vals.get('finfat_competencia_por_parcelas'), context)
            return res_id

    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Atencao!'), _('Fatura nao pode ser duplicada!'))

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'pessoa_id' in vals:
            raise osv.except_osv(_('Atencao'), _('O cliente nao pode ser alterado'))
        if not context.get('cancelamento_nf') and not context.get('lancamento_fatura_r'):
            _args = [('fatura_id', 'in', ids)] if isinstance(ids, (list, tuple)) else [('fatura_id', '=', ids)]
            _args.append(('state', '=', 'B'))
            if self.pool.get('financeiro.titulo').search(cr, uid, _args, context=context):
                raise osv.except_osv(_('Atencao'),
                                     _(
                                         'Voce nao pode alterar uma fatura cujo um Titulo ja foi baixado.'))
        if vals.get('cancelada') == True:
            _args = [('fatura_id', 'in', ids)] if isinstance(ids, (list, tuple)) else [('fatura_id', '=', ids)]
            titulos_id = self.pool.get('financeiro.titulo').search(cr, uid, _args, context=context)
            if titulos_id:
                context['active_ids'] = titulos_id
                pooler.get_pool(cr.dbname).get('cancelamento.titulo.osv').salvar(cr, uid,
                                                                                 titulos_id,
                                                                                 context)
        if vals.get('cancelada') == False:
            _args = [('fatura_id', 'in', ids)] if isinstance(ids, (list, tuple)) else [('fatura_id', '=', ids)]
            titulos_id = self.pool.get('financeiro.titulo').search(cr, uid, _args, context=context)
            if titulos_id:
                self.pool.get('financeiro.titulo').write(cr, uid, titulos_id, {'state': 'A'}, context=context)
        doc_tipo_id = self.pool.get('documento.documento.tipo').search(cr, uid,
                                                                       [('sigla', '=', 'DP')], context=context)
        if doc_tipo_id:
            doc_tipo_id = doc_tipo_id[0]
        if not isinstance(ids, list):
            ids = [ids]
        financeiro_fatura_dados = self.read(
            cr, uid, ids, ['company_id', 'pessoa_id', 'documento', 'data_emissao', 'tipo_fatura',
                           'titulo_ids'], context)
        titulo_ids = []
        map(lambda i: titulo_ids.extend(i.get('titulo_ids')), financeiro_fatura_dados)
        if vals.get('tipo_fatura') and not 'titulo_ids' in vals:
            self.pool.get('financeiro.titulo').write(
                cr, uid, titulo_ids, {'tipo_titulo': vals.get('tipo_fatura')}, context)
        if 'finfat_competencia_por_parcelas' in vals:
            self.finfat_atualiza_competencia_gerencial_movimento(
                cr, uid, ids, vals.get('finfat_competencia_por_parcelas'), context)
        for titulo in financeiro_fatura_dados:
            if 'id' in titulo:
                del titulo['id']
            if 'company_id' in vals:
                titulo['company_id'] = vals['company_id']
            else:
                titulo['company_id'] = titulo['company_id'][0]

            if 'pessoa_id' in vals:
                titulo['pessoa_id'] = vals['pessoa_id']
            else:
                titulo['pessoa_id'] = titulo['pessoa_id'][0]

            if 'documento' in vals:
                titulo['documento'] = vals['documento']
            if 'data_emissao' in vals:
                titulo['data_emissao'] = vals['data_emissao']

            if 'tipo_fatura' in vals:
                titulo['tipo_fatura'] = vals['tipo_fatura']

            parametro_id = self.pool.get('financeiro.parametros').search(cr, uid, [], context=context)
            if parametro_id:
                parametros = self.pool.get('financeiro.parametros').read(cr, uid, parametro_id,
                                                                         ['moeda_id',
                                                                          'local_cobranca_id'])
                for x in parametros:
                    local_cobranca_id = x.get('local_cobranca_id')
                if isinstance(local_cobranca_id, tuple):
                    local_cobranca_id = local_cobranca_id[0]
                vals['local_cobranca_id'] = local_cobranca_id
                if 'moeda_id' in parametros:
                    adm_moeda_id = parametros['moeda_id'][0]
                juros = self.pool.get('financeiro.parametros').read(cr, uid, parametro_id,
                                                                    ['juros_atraso'])
                if juros:
                    for x in juros:
                        juros = x.get('juros_atraso')
                else:
                    juros = 0
                multa = self.pool.get('financeiro.parametros').read(cr, uid, parametro_id,
                                                                    ['multa_atraso'])
                if multa:
                    for x in multa:
                        multa = x.get('multa_atraso')
                else:
                    multa = 0

                if 'titulo_ids' in vals:
                    if not vals.get('titulo_ids', False):
                        vals2 = vals
                        del vals2['titulo_ids']
                        financeiro_titulos = self.pool.get('financeiro.titulo').search(cr, uid,
                                                                                       [(
                                                                                            'fatura_id',
                                                                                            '=',
                                                                                            ids)], context=context)
                        for i in financeiro_titulos:
                            self.pool.get('financeiro.titulo').write(cr, uid, i, vals2, context=context)
                    else:
                        titulos = []
                        for i in vals['titulo_ids']:
                            if i[0] in [0, 1]:
                                if i[2].get('vr_original') and i[2].get('data_prorrogacao') and \
                                        i[2].get('parcela'):
                                    vr_juros = (i[2]['vr_original'] * juros / 100) / 30
                                    vr_multa = i[2]['vr_original'] * multa / 100
                                    vals_titulos = {
                                        'tipo_titulo': titulo['tipo_fatura'],
                                        'pessoa_id': titulo['pessoa_id'],
                                        'documento': titulo.get('documento'),
                                        'company_id': titulo['company_id'],
                                        'local_cobranca_id': local_cobranca_id,
                                        'data_emissao': titulo['data_emissao'],
                                        'data_prorrogacao': i[2]['data_prorrogacao'],
                                        'data_vencimento': i[2]['data_prorrogacao'],
                                        'vr_original': i[2]['vr_original'],
                                        'saldo': i[2]['vr_original'],
                                        'parcela': i[2]['parcela'],
                                        'vr_juros_dia': vr_juros,
                                        'vr_multa': vr_multa,
                                        'fintit_gerencialmov_ids': i[2][
                                            'fintit_gerencialmov_ids'],
                                    }
                                    titulos.append((i[0], i[1], vals_titulos))
                            if i[0] == 2:
                                titulos.append((i[0], i[1], False))
                            if titulos:
                                vals['titulo_ids'] = titulos
                return super(financeiro_fatura, self).write(cr, uid, ids, vals, context=context)
        return False

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        company = self.pool.get('res.users')._get_company(cr, uid)
        ids_empresa = self.pool.get('res.company')._get_company_children(cr, uid, company)
        args += [('company_id', 'in', ids_empresa)]
        res = super(financeiro_fatura, self).search(cr, uid, args, offset, limit, order, context,
                                                    count)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        titulo_ids = []
        financeiro_fatura_dados = self.read(cr, uid, ids, ['titulo_ids'], context)
        map(lambda i: titulo_ids.extend(i.get('titulo_ids')), financeiro_fatura_dados)
        # O cupom de cancelamento deve deletar a fatura do seu cupom de venda
        if not context.get('cumpom_de_cancelamento'):
            ids_financeiro_titulo = self.pool.get('financeiro.titulo').search(
                cr, uid, [('fatura_id', 'in', ids), ('state', '=', 'B')], context=context)
            financeiro_titulo_baixa_ids = self.pool.get('financeiro.titulo.baixa').search(
                cr, uid, [('titulo_id', 'in', titulo_ids)], context=context)
            ids_financeiro_titulo.extend(financeiro_titulo_baixa_ids)
            if ids_financeiro_titulo:
                raise osv.except_osv(_('Atencao'), _('Fatura que possui titulos baixados nao pode ser '
                                                     'excluida'))
        res = super(financeiro_fatura, self).unlink(cr, uid, ids, context=context)
        return res

    def button_gerar_parcelas(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True

    def button_atualizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True

