# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
from ..model.funcoes import get_conta_agencia


class financeiro_local_natureza(osv.Model):
    _name = "financeiro.local.natureza"
    _inherit = "financeiro.local.natureza"

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        ids = []
        obj_financeiro_local_cobranca = self.pool.get('financeiro.local.cobranca')
        obj_financeiro_natureza_cobranca = self.pool.get('financeiro.natureza.cobranca')
        local_cobranca_ids = obj_financeiro_local_cobranca.search(cr, uid, [('descricao', '=',
                                                                             name)], context=context)
        natureza_cobranca_ids = obj_financeiro_natureza_cobranca.search(cr, uid, [('name', '=',
                                                                                   name)], context=context)
        if local_cobranca_ids:
            operator = 'in'
            ids = self.search(
                cr, uid, [('finlon_local_cobranca_id', operator, local_cobranca_ids)] + args,
                limit=limit, context=context)
        elif natureza_cobranca_ids:
            operator = 'in'
            ids = self.search(
                cr, uid, [('finlon_natureza_cobranca_id', operator, natureza_cobranca_ids)] + args,
                limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def name_get(self, cr, uid, ids, context=None):
        res = []
        context = context and context.copy() or {}
        if not len(ids):
            return []
        brw_financeiro_local_natureza = self.browse(cr, uid, ids, context)
        for x in brw_financeiro_local_natureza:
            local_cobranca = x.finlon_local_cobranca_id.descricao or 'Sem descricao'
            natureza_cobranca = x.finlon_natureza_cobranca_id.name or 'Sem descricao'
            descricao = '%s - %s' % (local_cobranca, natureza_cobranca)
            res.append((x['id'], descricao), )
        return res

    def onchange_finlon_local_cobranca(self, cr, uid, ids, finlon_local_cobranca, context=None):
        if not finlon_local_cobranca:
            return {}
        context = context and context.copy() or {}
        res = {'value': {}}
        brw_local_cobranca = self.pool.get('financeiro.local.cobranca').browse(cr, uid, finlon_local_cobranca)
        if isinstance(brw_local_cobranca, list):
            brw_local_cobranca=brw_local_cobranca[0]
        tipo = brw_local_cobranca.tipo
        if tipo != 'B':       # diferente de Banco
            res['value'] = {'finlon_natureza_cobranca_id': '',
                            'finlon_nosso_numero': '',
                            'finlon_agencia': '',
                            'finlon_agencia_dv': '',
                            'finlon_conta': '',
                            'finlon_conta_dv': '',
                            'finlon_carteira': '',
                            'finlon_convenio': '',
                            'finlon_local_tipo': False
                            }
        else:
            context['finlon_local_cobranca'] = True
            dados = get_conta_agencia(self, cr, uid, finlon_local_cobranca, context)
            res['value'] = {
                            'finlon_local_tipo': True,
                            'finlon_agencia': dados.get('finlon_agencia') or '',
                            'finlon_agencia_dv': dados.get('finlon_agencia_dv') or '',
                            'finlon_conta': dados.get('finlon_conta') or '',
                            'finlon_conta_dv': dados.get('finlon_conta_dv') or '',
                            }
        return res

    def finlon_onchange_carteira(self, cr, uid, ids, carteira, context=None):
        """
        O metodo valida o tipo de dado informado para o campo 'Carteira' (finlon_carteira). Deve
        permitir somente inteiros. E chamado apos os dados serem inseridos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param carteira: dados do numero da carteira.
        :type carteira: str
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com a descricao do campo (chave) e seu novo valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not carteira:
            return res
        if not carteira.isdigit():
            res['value'] = {'finlon_carteira': ''}
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Informe numeros inteiros para a Carteira.')}
        return res

    def finlon_get_set_nosso_numero(self, cr, uid, titulo_id, local_natureza_id, context=None):
        """
        Método para retornar e incrementar 1 em nosso_numero e atualizar a classe
        @param cr:
        @param uid:
        @titulo_id:
        @param local_natureza_id:
        @param context:
        @return: nosso_numero ou False caso não consiga
        """
        def _finlon_verifica_se_existe():
            args_bol = [('fintib_nosso_numero', '=', nosso_numero),
                        ('fintib_local_cobranca_id', '=', _local_cobranca_id)]
            _boleto_ids = obj_boleto.search(cr, uid, args_bol, limit=1, context=context)
            if _boleto_ids: #ja existe nosso numero
                return True
            else:
                return False

        context = context and context.copy() or {}
        dados_read = self.read(cr, uid, local_natureza_id, ['finlon_nosso_numero', 'finlon_local_cobranca_id'],
                               context=context, load='_classic_write')
        if not dados_read:
            return False
        obj_boleto = self.pool.get('financeiro.titulo.boleto')
        _local_cobranca_id = dados_read.get('finlon_local_cobranca_id')
        args_bol = [('fintib_titulo_id', '=', titulo_id), ('fintib_local_cobranca_id', '=', _local_cobranca_id)]
        boleto_ids = obj_boleto.search(cr, uid, args_bol, context=context)
        if boleto_ids:
            dados_boleto = obj_boleto.read(cr, uid, boleto_ids[0], [], context=context)
            nosso_numero = dados_boleto.get('fintib_nosso_numero')
            if nosso_numero:
                return nosso_numero
        try:
            nosso_numero = int(dados_read.get('finlon_nosso_numero', 0)) + 1
        except:
            nosso_numero = 0
        while _finlon_verifica_se_existe():
            nosso_numero += 1
        self.write(cr, uid, local_natureza_id, {'finlon_nosso_numero': nosso_numero}, context=context)
        return nosso_numero
