# -*- encoding: utf-8 -*-


from openerp.osv import osv
from openerp.tools.translate import _
from SigUtils.calcDigito import codigo_barra_bancario
from datetime import datetime
from dateutil.relativedelta import relativedelta


class financeiro_titulo_boleto(osv.Model):
    # prefixo: fintib_

    _inherit = 'financeiro.titulo.boleto'

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(financeiro_titulo_boleto, self).default_get(cr, uid, fields_list, context)
        res.update({'fintib_tipo_titulo': context.get('tipo_titulo')})
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('fintib_codigo_barras'):
            vals['fintib_codigo_barras'] = vals.get('fintib_codigo_barras').replace('.', '').\
                replace(' ', '')
            res0, x, tit_codigo_barras = codigo_barra_bancario(vals.get('fintib_codigo_barras'))
            if not res0:
                raise osv.except_osv(_('Atencao'), _('Codigo de barras invalido'))
        res = super(financeiro_titulo_boleto, self).create(cr, uid, vals, context)
        if not self.fintib_verificar_boleto(cr, uid, [res], context):
            raise osv.except_osv(_('Atencao'),
                                 _('O codigo de barras ou o nosso numero e local de cobranca ja se '
                                   'encontram cadastrados em outro titulo.'))
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('fintib_codigo_barras'):
            vals['fintib_codigo_barras'] = vals.get('fintib_codigo_barras').replace('.', '').\
                replace(' ', '')
            res0, x, tit_codigo_barras = codigo_barra_bancario(vals.get('fintib_codigo_barras'))
            if not res0:
                raise osv.except_osv(_('Atencao'), _('Codigo de barras invalido'))
        res = super(financeiro_titulo_boleto, self).write(cr, uid, ids, vals, context)
        if not self.fintib_verificar_boleto(cr, uid, ids, context):
            raise osv.except_osv(_('Atencao'),
                                 _('O codigo de barras ou o nosso numero e local de cobranca ja se '
                                   'encontram cadastrados em outro titulo.'))
        return res

    def fintib_verificar_boleto(self, cr, uid, ids, context=None):
        """
        O metodo verifica se ha codigo de barras e nosso numero, respectivamente, ja cadastrados.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: False se codigo de barras ou nosso numero ja se encontra cadastrado, senao, True.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = True
        titulo_ids = self.pool.get('financeiro.titulo').search(cr, uid, [('state', '=', 'A')],
                                                               context=context)
        brw_titulo_boleto = self.browse(cr, uid, ids, context)
        for boleto in brw_titulo_boleto:
            titulo_boleto_ids = []
            if boleto.fintib_codigo_barras:
                args = [('fintib_codigo_barras', '=', boleto.fintib_codigo_barras),
                        ('fintib_titulo_id', 'in', titulo_ids)]
                titulo_boleto_ids = self.search(cr, uid, args, context=context)
            if not titulo_boleto_ids and boleto.fintib_nosso_numero:
                args = [('fintib_nosso_numero', '=', boleto.fintib_nosso_numero),
                        ('fintib_local_cobranca_id', '=', boleto.fintib_local_cobranca_id.id)]
                titulo_boleto_ids = self.search(cr, uid, args, context=context)
            if len(titulo_boleto_ids) > 1:
                res = False
        return res

    def fintib_onchange_cod_barras(self, cr, uid, ids, codigo_barras, valor, vencimento,
                                   context=None):
        """
        Este eh chamado ao sair do campo fintib_codigo_barras apos uma alteracao e serve
        para chamar os metodos que validam os digitos do codigo de barras, valor e vencimento.
        Caso estes nao sejam validados eh mostrado ao usuario final uma mensagem informando o erro
        e o campo fintit_codigo_barras eh limpado.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param codigo_barras: codigo de barras convertido para o padrao digitavel
        :param valor: valor informado no titulo
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type codigo_barras: str
        :type valor: float
        :type context: dict
        :return: Retorna ao usuario um aviso do erro e limpa o campo fintit_codigo_barras caso
        o codigo de barras nao seja valido
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if codigo_barras:
            if vencimento:
                vencimento = datetime.strptime(vencimento, '%Y-%m-%d')
            res, x, codigo_barras = codigo_barra_bancario(codigo_barras)
            value = {'fintib_codigo_barras': ''}
            if res:
                if not self.fintib_valida_venc_cod_barras(codigo_barras, vencimento):
                    msg = 'Vencimento do titulo difere do vencimento informado no codigo de barras!'
                    return dict(warning=dict(title=_('Aviso.'), message=msg), value=value) if msg \
                        else dict()
                if not self.fintib_valida_valor_cod_barras(codigo_barras, valor):
                    msg = 'Valor do titulo difere do valor informado no codigo de barras!'
                    return dict(warning=dict(title=_('Aviso.'), message=msg), value=value) if msg \
                        else dict()
            else:
                msg = 'Codigo de barras invalido'
                return dict(warning=dict(title=_('Aviso.'), message=msg), value=value) if msg else \
                    dict()
        return True

    @staticmethod
    def fintib_valida_venc_cod_barras(codigo_barras, vencimento):
        """
        (CH-85021)
        Este metodo valida o vencimento informado no codigo de barras pelo vencimento informado
        no titulo, para validacao do codigo de barras eh necessario verificar se o codigo de barras
        eh referente a convenio(Codigos de barra iniciados com '8') ou boleto.
        Caso o codigo de barras seja de convenio, nao existira informacoes sobre fator
        de vencimento no codigo de barras, entao este metodo retorna apenas True para esta situacao.
        Caso o codigo de barras seja boleto, e necessario encontrar no codigo de barras o fator de
        vencimento e somar a data 07-10-1997, para encontrar o vencimento do boleto

        :param codigo_barras: codigo de barras convertido para o padrao digitavel
        :param vencimento: vencimento do titulo
        :type codigo_barras: str
        :type vencimento: datetime
        :return: booleano setado como True para vencimento valido, ou False para vencimento invalido
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        data = datetime(year=1997, day=7, month=10)
        res = False
        if codigo_barras[0] == '8':
            res = True
        else:
            fator_vencimento = int(codigo_barras[33:37])
            if fator_vencimento == 0:
                return True
            data = data + relativedelta(days=fator_vencimento)
            if vencimento == data:
                res = True
        return res

    @staticmethod
    def fintib_valida_valor_cod_barras(codigo_barras, valor):
        """
        Este metodo valida o valor informado no titulo pelo valor descrito no codigo de barras

        :param codigo_barras: codigo de barras convertido para o padrao digitavel
        :param valor: valor informado no titulo
        :type codigo_barras: str
        :type valor: float
        :return: booleano setado como True para valor valido, ou False para valor invalido
        :rtype: bool
        """
        if codigo_barras[0] == '8':
            valor_cod_barra = float(codigo_barras[4:11] + codigo_barras[12:16]) / 100
        else:
            valor_cod_barra = float(codigo_barras[37:47]) / 100
        if valor == valor_cod_barra or valor_cod_barra == 0:
            return True
        else:
            return False

    def fintib_get_boleto_ids(self, cr, uid, titulo_id, codBarra=None, local_id=None, ativo=True, context=None):
        """
        Metodo utilizado para retornar os ids dos boletos que satisfaçam aos parametros
        :param titulo_id: id do titulo que deseja os boletos
        :param codBarra: codigo de barras que deseja encontrar
        :param local_id: local de cobrança a ser considerado para localizar o codigo de barras
        :param ativo: Identifica se são somente codigos ativos ou nao
        :type titulo_id: integer
        :type codBarra: char
        :type local_id: integer
        :type ativo: boolean
        :return: ids dos boletos
        :rtype: bool
        """
        args = [('fintib_titulo_id', '=', titulo_id)]
        if ativo:
            args.append(('state', '=', 'A'))
        if codBarra:
            args.append(('fintib_codigo_barras', '=', codBarra))
        elif local_id:
            args.append(('fintib_local_cobranca_id', '=', local_id))
        return self.search(cr, uid, args, context=context)

