# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from SigUtils import inscricao_estadual
import re
from openerp.tools.translate import _


class financeiro_fornecedor(osv.Model):
    _name = 'financeiro.fornecedor'
    _inherit = 'financeiro.fornecedor'

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        context.update({'sig_search_sem_filhos': True})
        res = super(financeiro_fornecedor, self).search(cr, uid, args, offset, limit, order,
                                                        context, count)
        return res

    def verificar_incluir_fornecedor(self, cr, uid, pessoa_id, company_id, acao='verificar',
                                     context=None):
        """
        Efetua cadastro automatico de Fornecedor dependendo do Parametro do Financeiro

        :param cr: database cursor
        :param uid: usuário atual
        :param pessoa_id: id da pessoa
        :param company_id: id da empresa
        :param acao: 'verificar'/'incluir' - se incluir cadastra a pessoa_id como fornecedor
        :param context: argumentos do contexto
        :rtype: boolean

        Este metodo e usado para verificar se a Pessoa ja está cadastrada como Fornecedor na Empresa informada.
        """

        # Veririca se o objeto <type 'str'> acao, e igual a 'verificar'/'incluir'
        # senao, gerar um AssertionError.
        context = context and context.copy() or {}
        try:
            assert acao in ('verificar', 'incluir')
        except AssertionError:
            raise Exception(
                _("AssertionError: valor para a acao deve ser 'verificar' ou 'incluir'"))

        context = context and context.copy() or {}
        res = False

        # verificar se a pessoa e fornecedor
        ativos = self.search(cr, uid, [('pessoa_id', '=', pessoa_id),
                                        ('company_id', '=', company_id)], context=context)
        is_fornecedor = ativos
        if not ativos:
            inativos = self.search(cr, uid, [('pessoa_id', '=', pessoa_id),
                                  ('company_id', '=', company_id),('active','=',False)], context=context)
            if inativos:
                raise osv.except_osv(_('Atencao'),
                                     _('Fornecedor inativo. '
                                       'Ative o mesmo para que possa utiliza-lo.'))
        # e fornecedor
        if is_fornecedor:
            res = True
        # nao e fornecedor
        else:
            # objeto financeiro.parametros
            obj_par_financ = self.pool.get('financeiro.parametros')
            par_financ_id = obj_par_financ.search(cr, uid, [('company_id', '=', company_id)], context=context)
            # possui parametros cadastrados
            if par_financ_id:
                cadastro_automatico_fornecedor = \
                    obj_par_financ.read(cr, uid, par_financ_id[0],
                                        ['cadastro_automatico_fornecedor'], context)[
                        'cadastro_automatico_fornecedor']
                if cadastro_automatico_fornecedor:
                    if acao == 'incluir':
                        # cadastra a pessoa como fornecedor
                        res = self.create(cr, uid, {
                            'pessoa_id': pessoa_id,
                            'company_id': company_id,
                        }, context)
            else:
                raise osv.except_osv(_('Atencao'),
                                     _('Nao a parametros cadastrado no financeiro p/ essa empresa'))
        return res


    def onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        if not pessoa_tipo:
            warning['title'] = _('Atencao')
            warning['message'] = _('Tipo de pessoa nao informado!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        pessoa_ids = obj_pessoa.search(cr, uid,
                                       [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar),
                                        ('pessoa_tipo', '=', pessoa_tipo)], context=context)
        if pessoa_ids:
            fornecedor_id = obj_financeiro_fornecedor.search(cr, uid,
                                                             [('pessoa_id', 'in', pessoa_ids),
                                                              ('active', '=', True)], context=context)
            if fornecedor_id:
                warning = {'title': _('Atencao')}
                warning['message'] = _('Fornecedor ja cadastrado!')
                return {'warning': warning, 'value': {'cpf_cnpj': ''}}
            dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
            if isinstance(dados_pessoa, list):
                dados_pessoa = dados_pessoa[0]
            endereco_ids = []
            for dados_endereco in dados_pessoa.endereco_ids:
                enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                             'active': dados_endereco.active,
                             "cep": dados_endereco.cep,
                             "pais_id": dados_endereco.pais_id.id,
                             "uf_id": dados_endereco.uf_id.id,
                             "cidade_id": dados_endereco.cidade_id.id,
                             "bairro_id": dados_endereco.bairro_id.id,
                             "logradouro_id": dados_endereco.logradouro_id.id,
                             "numero": dados_endereco.numero,
                             "complemento": dados_endereco.complemento,
                             "observacao": dados_endereco.observacao,
                }
                endereco_ids.append((1, dados_endereco.id, enderecos))
            email_ids = []
            for dados_emails in dados_pessoa.email_ids:
                emails = {
                    "email_tipo_id": dados_emails.email_tipo_id.id,
                    "name": dados_emails.name,
                    "observacao": dados_emails.observacao,
                }
                email_ids.append((1, dados_emails.id, emails))
            telefones_ids = []
            for dados_telefones in dados_pessoa.telefones_ids:
                telefones = {
                    "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                    "ddi": dados_telefones.ddi,
                    "ddd": dados_telefones.ddd,
                    "numero": dados_telefones.numero,
                    "observacoes": dados_telefones.observacoes,
                }
                telefones_ids.append((1, dados_telefones.id, telefones))
            res = {'value': {'cpf_cnpj': cpf_cnpj,
                             'pessoa_id': pessoa_ids[0],
                             'name': dados_pessoa.name,
                             'codigo': dados_pessoa.codigo,
                             'fantasia_apelido': dados_pessoa.fantasia_apelido,
                             'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                             'admpes_fis_rg_orgao_emissor': dados_pessoa.admpes_fis_rg_orgao_emissor,
                             'admpes_fis_rg_data_emisssao': dados_pessoa.admpes_fis_rg_data_emisssao,
                             'admpes_fis_cnh_nro': dados_pessoa.admpes_fis_cnh_nro,
                             'admpes_fis_cnh_orgao_emissor': dados_pessoa.admpes_fis_cnh_orgao_emissor,
                             'admpes_fis_cnh_validade': dados_pessoa.admpes_fis_cnh_validade,
                             'admpes_fis_cnh_categoria': dados_pessoa.admpes_fis_cnh_categoria,
                             'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                             'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                             'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                             'admpes_jur_data_fundacao': dados_pessoa.admpes_jur_data_fundacao,
                             'admpes_jur_tipo_empresa': dados_pessoa.admpes_jur_tipo_empresa,
                             'admpes_jur_suframa': dados_pessoa.admpes_jur_suframa,
                             'admpes_jur_regime_tributario': dados_pessoa.admpes_jur_regime_tributario,
                             'admpes_jur_tipo_atividade': dados_pessoa.admpes_jur_tipo_atividade,
                             'admpes_jur_ie': dados_pessoa.admpes_jur_ie,
                             'admpes_jur_cnae_id': dados_pessoa.admpes_jur_cnae_id.id,
                             "foto": dados_pessoa.foto,
                             'endereco_ids': endereco_ids,
                             'email_ids': email_ids,
                             'telefones_ids': telefones_ids,
                             'pessoa_id': dados_pessoa.id,
            }
            }
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}


    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not isinstance(default, dict):
            default = {}
        raise osv.except_osv(_('Atencao'), _('Nao e permitido duplicar Fornecedor'))

    def onchange_ie(self, cr, uid, ids, admpes_jur_ie):
        res = {}
        estado = False
        for partner in self.browse(cr, uid, ids):
            estado = partner.uf.upper()
        if admpes_jur_ie and admpes_jur_ie.isdigit() and estado:
            res = inscricao_estadual.validar(estado, admpes_jur_ie)
            if not res:
                return {'warning': {'title': _('Atencao'),
                                    'message': _('Inscricao Estadual invalida para o seguinte estado: %s') % estado}}
        return res

    def fornecedor_ativo(self, cr, uid, ids, context=None):
        """
        Verifica se o fornecedor esta ou nao ativo.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se o fornecedor estiver ativo.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        context['active_test'] = False  # Permite recuperar os registros inativos
        ids = [ids] if not isinstance(ids, (list, tuple)) else ids
        args = [('pessoa_id', 'in', ids)]
        pessoa_ids = self.search(cr, uid, args, context=context)
        dados_pessoa = self.read(cr, uid, pessoa_ids or 0, ['active'], context)

        if dados_pessoa and isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]

        # Se nao foi encontrado, nao ha como estar inativo (retorna verdadeiro)
        return dados_pessoa.get('active', True) if dados_pessoa else True

    def name_get(self, cr, uid, ids, context={}):
        res = []
        brw_fornecedor = self.browse(cr, uid, ids, context=context)
        for fornecedor in brw_fornecedor:
            codigo = fornecedor.pessoa_id.codigo or ''
            name = fornecedor.pessoa_id.name
            if context.get('finfor_retorno_sem_codigo'):
                codigo = ''
            result = "%s%s%s" % (codigo, ' - ' if codigo else '', name)
            res.append((fornecedor.id, result))
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
