# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import time


class financeiro_titulo_baixa(osv.Model):
    _name = 'financeiro.titulo.baixa'
    _inherit = 'financeiro.titulo.baixa'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if 'conta_id' in vals:
            del vals['conta_id']
        if 'baixar_titulo' in vals:
            del vals['baixar_titulo']
        if 'forma_pgto_id' in vals:
            del vals['forma_pgto_id']
        if 'hist_mov_tesouraria' in vals:
            del vals['hist_mov_tesouraria']
        mov = None
        tipo = self.pool.get('financeiro.titulo').read(cr, uid, vals['titulo_id'], ['tipo_titulo'],
                                                       context=context)['tipo_titulo']
        if tipo == 'R':
            dc = 'C'
        elif tipo == 'P':
            dc = 'D'
        if 'baixar_titulo' in vals:
            if 'hist_mov_tesouraria' in vals:
                hist = vals['hist_mov_tesouraria']
            else:
                hist = 'Baixa'
            data = time.strftime('%Y-%m-%d')
            val = {'conta_id': vals['conta_id'],
                   'data_movimento': data,
                   'mes_competencia': str(data[5:7]),
                   'ano_competencia': str(data[0:4]),
                   'historico': hist,
                   'debito_credito': dc,
                   'valor': vals.get('valor_pago', 0)
            }
            mov = self.pool.get('financeiro.tesouraria.movimento').create(cr, uid, val, context)
            vals['tesouraria_movimento_id'] = mov
            res = super(financeiro_titulo_baixa, self).create(cr, uid, vals, context=context)
            #Se forma de pagamento for cheque (tesouraria/wizard/baixar_titulos.forma_pgto_id), nao gerar movimento
            if not context.get('pgto_cheque', False):
                mov = self.pool.get('gerencial.movimento').search(cr, uid, [('titulo_id', '=', vals['titulo_id'])], context=context)
                if mov:
                    for a in mov:
                        self.pool.get('gerencial.movimento').write(cr, uid, a, {
                            'tesouraria_movimento_id': vals['tesouraria_movimento_id']}, context=context)
        else:
            res = super(financeiro_titulo_baixa, self).create(cr, uid, vals, context=context)
        if mov:
            #implementado desta forma para a atualizacao do campo funcao (valor) em tesouraria movimento
            aux = self.pool.get('financeiro.tesouraria.movimento')._get_valor_b(cr, uid, [mov], 'valor', None, context)
            self.pool.get('financeiro.tesouraria.movimento').write(cr, uid, [mov], {'valor': aux[mov]}, context)
        if 'conta_id' in vals:
            del vals['conta_id']
        if 'baixar_titulo' in vals:
            del vals['baixar_titulo']
        return res

    def fintib_efetua_cancelamento(self, cr, uid, ids, context=None):
        """
        O metodo realiza o processo de cancelamento de baixa de titulos. Os titulos voltam para o
        estado de 'Aberto' e soma-se ao saldo o valor baixado. Os historicos referentes a baixa sao
        excluidos. As baixas sao setadas como inativas para posteriormente serem excluidas junto com
        o movimento da tesouraria.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso possua baixas a serem inativadas. Senao, Falso.
        :rtype: bool
        """

        context = context and context.copy() or {}

        obj_financeiro_titulo_historico = self.pool.get('financeiro.titulo.historico')

        baixa_ids = list()

        for baixa in self.browse(cr, uid, ids, context):
            if baixa.fintib_baixa_ids:
                baixa_ids.extend(map(lambda i: i.id, baixa.fintib_baixa_ids))
                for b in baixa.fintib_baixa_ids:
                    saldo = (b.titulo_id.saldo or 0) + (b.valor_pago or 0)
                    sql = """
                        UPDATE financeiro_titulo SET state = 'A', saldo = %s WHERE id = %s
                    """ % (saldo, b.titulo_id.id)
                    cr.execute(sql)
            else:
                saldo = (baixa.titulo_id.saldo or 0) + (baixa.valor_pago or 0)
                sql = """
                    UPDATE financeiro_titulo SET state = 'A', saldo = %s WHERE id = %s
                """ % (saldo, baixa.titulo_id.id)
                cr.execute(sql)
                baixa_ids.append(baixa.id)

            titulo_historico_ids = obj_financeiro_titulo_historico.search(
                cr, uid, [('titulo_id', '=', baixa.titulo_id.id),
                          ('operacao', 'in', ('B', 'BP', 'BR'))], context=context)
            if not titulo_historico_ids:
                continue
            obj_financeiro_titulo_historico.unlink(cr, uid, titulo_historico_ids, context)

        return self.write(cr, uid, baixa_ids, {'active': False}, context)

    def fintib_estorna_baixa(self, cr, uid, ids, context=None):
        """
        O metodo cria o movimento da tesouraria inverso, como também o movimento gerencial, caso
        possua, para cada baixa previamente realizada.
        E excluido os registros de baixa aqui recebidos, uma vez que, foram estornadas.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso os registros de baixa sejam deletados. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_gerencial_movimento = self.pool.get('gerencial.movimento')
        obj_financeiro_utils = self.pool.get('financeiro')
        for baixa in self.browse(cr, uid, ids, context):
            tesouraria = baixa.tesouraria_movimento_id
            valor_pago = baixa.valor_pago
            vals_tesouraria = {
                'conta_id': tesouraria.conta_id.id,
                'data_movimento': time.strftime('%Y-%m-%d'),
                'mes_competencia': tesouraria.mes_competencia,
                'ano_competencia': tesouraria.ano_competencia,
                'valor': valor_pago,
                'historico': 'Estorno da baixa do titulo: ' + baixa.titulo_id.documento,
                'debito_credito': 'D' if baixa.titulo_id.tipo_titulo == 'R' else 'C',
                'state': 'A',
            }
            context.update({'financ_estorno_baixa': True})
            movimento_id = obj_financeiro_tesouraria_movimento.create(cr, uid, vals_tesouraria,
                                                                      context)
            sql = """
                INSERT INTO financeiro_m2m_titulo_tesouraria (id, titulo_id)
                VALUES (%s, %s)
            """ % (movimento_id, baixa.titulo_id.id)
            cr.execute(sql)
            if not tesouraria.gerencial_movimento_ids:
                continue
            brw_gerencial_movimento = tesouraria.gerencial_movimento_ids[0]
            titulo_custificacao_dados = [{
                'titulo_id': baixa.titulo_id.id,
                'tipo_titulo': 'R' if baixa.titulo_id.tipo_titulo == 'P' else 'P',
                'valor_juros': baixa.valor_juros or 0.0, 'valor_multa': baixa.valor_multa or 0.0,
                'valor_desconto': baixa.valor_desconto or 0.0, 'valor_pago': valor_pago,
                'valor_acrescimo': baixa.fintib_vr_acrescimo or 0.0
            }]
            custificacao_dados = obj_financeiro_utils.financ_get_custificacao_baixa(
                cr, uid, titulo_custificacao_dados, context)
            vals_gerencial = {
                'company_id': brw_gerencial_movimento.company_id.id,
                'tesouraria_movimento_id': movimento_id,
                'fingmo_data': time.strftime('%Y-%m-%d'),
                'mes_competencia': brw_gerencial_movimento.mes_competencia,
                'ano_competencia': brw_gerencial_movimento.ano_competencia,
                'pessoa_id': brw_gerencial_movimento.pessoa_id.id or baixa.titulo_id.pessoa_id.id
            }
            for gerencial in custificacao_dados:
                vals_gerencial.update({
                    'valor': gerencial.get('valor') or 0.0,
                    'evento_id': gerencial.get('evento_id'),
                    'centro_custo_id': gerencial.get('centro_custo_id'),
                    'fingmo_operacao': 'E' if gerencial.get('tipo_titulo') == 'R' else 'S',
                    'fingmo_historico': 'Estorno da baixa. ' + gerencial.get('historico')
                })
                obj_gerencial_movimento.create(cr, uid, vals_gerencial, context)
        return self.unlink(cr, uid, ids, context)
