# -*- coding: utf-8 -*-

from openerp.osv import osv
from SigUtils import datas
from dateutil.parser import parse
from time import strftime
from decimal import Decimal, ROUND_DOWN, ROUND_UP
from datetime import date, datetime
from dateutil.relativedelta import relativedelta
from openerp.tools.translate import _
from random import randint

TWO_PLACES = Decimal('10') ** -2


class financeiro(osv.Model):
    _name = 'financeiro'
    _auto = False

    _mensagem_erro = u''
    _tem_erro = False
    _context_financeiro_util = {
        'futil_super': True
    }

    def financ_get_erro(self):
        return self._mensagem_erro

    def _financ_validar_pessoa(self, pessoa_id):
        if not pessoa_id:
            self._tem_erro = True
            self._mensagem_erro += _('Nao foi informado um cliente!')

    def _financ_validar_data(self, data, tipo):
        if not data:
            self._tem_erro = True
            self._mensagem_erro += _('Nao foi informado uma data de %s!\n') % tipo
        else:
            if not datas.sigDatas(data).getDataOk():
                self._tem_erro = True
                self._mensagem_erro += _('Data de %s invalida!\n') % tipo

    def _financ_validar_venc_maior_emissao(self, emissao, vencimento):
        if not self._tem_erro:
            emissao = datas.sigDatas(emissao).getDataNoFormato('I', 'S', '-')
            vencimento = datas.sigDatas(vencimento).getDataNoFormato('I', 'S', '-')
            if vencimento < emissao:
                self._tem_erro = True
                self._mensagem_erro += _('Data de vencimento do titulo menor do que a data de emissao!\n')
        return emissao, vencimento

    def _financ_validar_tipo_rec_pag(self, tipo, rec_pag):
        if not tipo:
            self._tem_erro = True
            self._mensagem_erro += _('Tipo de %s nao informado(R=Receber, P=Pagar)!\n') % rec_pag
        else:
            if tipo not in ('R', 'P'):
                self._tem_erro = True
                self._mensagem_erro += _('Tipo de %s invalido!(R=Receber, P=Pagar)!\n') % rec_pag

    def _financ_validar_valor(self, valor):
        if not valor:
            self._tem_erro = True
            self._mensagem_erro += _('Valor do Titulo nao informado!\n')
        else:
            if not valor > 0:
                self._tem_erro = True
                self._mensagem_erro += _('Valor do titulo invalido!\n')

    def _financ_validar_tipo_sacado(self, tipo_fatura, tipo_sacado):
        if not tipo_sacado:
            if tipo_fatura == 'R':
                tipo_sacado = 'C'
            else:
                tipo_sacado = 'F'
        if tipo_sacado not in ('C', 'F', 'O'):
            self._tem_erro = True
            self._mensagem_erro += _('Tipo de sacado invalido!\n')
        return tipo_sacado

    # ###################################################################################################################
    # CRIAÇÃO DE FATURA
    ####################################################################################################################
    def financ_incluir_fatura(self, cr, uid, futl_fat_pessoa_id, futl_fat_data_emissao,
                              futl_fat_valor, futl_fat_tipo_fatura,
                              futl_fat_titulo_dados=False, futl_fat_obs=False, futl_fat_moeda=False,
                              futl_fat_doc_tipo_id=False,
                              futl_fat_documento=False, futl_fat_tipo_sacado=False,
                              futl_fat_vendedor_id=False, futl_fat_cupom_id=False,
                              futl_fat_vendedor_externo_id=False, futl_fat_tipo_lancamento=False,
                              futl_fat_nf_id=False, futl_fat_company_id=False,
                              futl_competencia_parcela=False, context=None):
        """
        - Este medodo faz o seguinte:
          - Inclui a fatura no financeiro_fatura,
          - Chama o metodo financ_titulo_entradapara fazer a inclusao de titulos,
          - Atualiza o sistema de controle de transacoes,
          - Inclui a pessoa como cliente/fornecedor se estiver marcado para cadastrar automaticamente.
        - Parcelas, contem lista de dicionario com vencimento e valor. Nao seja informado o campo parcelas, nao sera feita a inclusao de titulos
        - Os Campos com valores Default especificados, nao sao obrigatorios e nao devem ser informados se nao houver a informacao
        - Retorna: False se houver algum problema ou
                   o id da fatura se tudo der certo
        - Havendo erro na execucao do metodo(return False), recuperar o erro atraves do metodo financ_get_erro()
        """
        context = context and context.copy() or {}
        context.update(self._context_financeiro_util)

        self._mensagem_erro = u''
        self._tem_erro = False
        obj_adm_empresa = self.pool.get('adm.empresa.objeto')
        fields_to_read = ['moeda_id', 'cadastro_automatico_cliente',
                          'cadastro_automatico_fornecedor']
        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', fields_to_read, context=context)
        finpar_moeda_id = financeiro_parametro_dados.get('moeda_id')
        finpar_cad_cliente_auto = financeiro_parametro_dados.get('cadastro_automatico_cliente')
        finpar_cad_fornec_auto = financeiro_parametro_dados.get('cadastro_automatico_fornecedor')

        self._financ_validar_data(futl_fat_data_emissao, 'emissao')
        self._financ_validar_pessoa(futl_fat_pessoa_id)
        self._financ_validar_tipo_rec_pag(futl_fat_tipo_fatura, u'fatura')

        # se o numero do documento nao for informado e o tipo da fatura for 'R'
        if not futl_fat_documento and futl_fat_tipo_fatura == 'R':
            futl_fat_documento = '%009d' % int(self.pool.get('adm.utils').get_sequence(
                cr, uid, 'financeiro.fatura', 'cod_fatura_nf')
            )
        doc_tipo = True
        # se tiver parcelas, vai lancar os titulos, portanto tem que validar os demais campos
        if futl_fat_titulo_dados:
            if not futl_fat_documento:
                self._tem_erro = True
                self._mensagem_erro += _('Nao foi informado o numero do documento!\n')
            if not futl_fat_moeda:
                futl_fat_moeda = finpar_moeda_id
            if not futl_fat_doc_tipo_id:
                doc_tipo = False
                futl_fat_doc_tipo_id = self.pool.get('documento.documento.tipo').search(cr, uid,
                                                                                        [(
                                                                                             'sigla',
                                                                                             '=',
                                                                                             'DP')],
                                                                                        context=context)[0]
            futl_fat_tipo_sacado = self._financ_validar_tipo_sacado(futl_fat_tipo_fatura,
                                                                    futl_fat_tipo_sacado)
            seq = 0
            total_titulos = 0
            vals_titulo = []
            for p in futl_fat_titulo_dados:
                seq += 1
                vencimento = p['vencimento']
                valor_titulo = p['valor']
                titulo_custificacao = p.get('fintit_gerencialmov_ids')
                self._financ_validar_data(vencimento, 'vencimento')
                futl_fat_data_emissao, vencimento = self._financ_validar_venc_maior_emissao(
                    futl_fat_data_emissao, vencimento)
                self._financ_validar_valor(valor_titulo)
                titulo_dados = {
                    'parcela': p.get('parcela') or seq,
                    'vencimento': vencimento,
                    'valor': valor_titulo
                }
                if titulo_custificacao:
                    titulo_dados.update({'fintit_gerencialmov_ids': titulo_custificacao})
                vals_titulo.append(titulo_dados)
                total_titulos += valor_titulo
        if self._tem_erro:
            return False
        finfat_company_id = futl_fat_company_id
        if not finfat_company_id:
            finfat_company_id = obj_adm_empresa.getEmpresa(cr, uid, 'financeiro.fatura',
                                                           context=context)
        vals_fatura = {
            'company_id': finfat_company_id,
            'data_emissao': futl_fat_data_emissao,
            'documento': futl_fat_documento,
            'pessoa_id': futl_fat_pessoa_id,
            'tipo_fatura': futl_fat_tipo_fatura,
            'finfat_documento_tipo_id': futl_fat_doc_tipo_id if doc_tipo else False,
            'finfat_competencia_por_parcelas': futl_competencia_parcela or False
        }
        if futl_fat_cupom_id:
            vals_fatura['cupom_id'] = futl_fat_cupom_id
        if futl_fat_nf_id:
            vals_fatura['nf_id'] = futl_fat_nf_id
        if futl_fat_titulo_dados:
            vals_fatura['qtd_parcelas'] = seq

        # incluir titulos
        if futl_fat_titulo_dados:
            list_val_titulos = []
            for i in vals_titulo:
                # [CREATE] ________________________ CRIA TITULOS __________________________________
                vals_titulo = {
                    'data_vencimento': i['vencimento'],
                    'documento': futl_fat_documento,
                    'vr_original': i['valor'],
                    'fintit_gerencialmov_ids': i.get('fintit_gerencialmov_ids'),
                    'pessoa_id': futl_fat_pessoa_id,
                    'data_emissao': futl_fat_data_emissao,
                    'parcela': i.get('parcela'),
                    'tipo_lancamento': futl_fat_tipo_lancamento,
                    'tipo_titulo': futl_fat_tipo_fatura,
                    'observacoes': futl_fat_obs or '',
                    'adm_moeda_id': futl_fat_moeda,
                    'doc_tipo_id': futl_fat_doc_tipo_id,
                    'tipo_sacado': futl_fat_tipo_sacado,
                }
                seq += 1
                list_val_titulos.append([0, False, vals_titulo])
            if list_val_titulos:
                vals_fatura.update({'titulo_ids': list_val_titulos})
        else:
            if futl_fat_nf_id:
                try:
                    brw_nota = self.pool.get('fiscal.nf.capa').browse(cr, uid, futl_fat_nf_id)
                    movimento_tipo = brw_nota.natureza_id.cfop_id.movimento_tipo
                except:
                    movimento_tipo = False
                if movimento_tipo != 'D':
                    # Incluir cliente/fornecedor. Esta no else, porque se tem titulo ja fez isto no titulo.#
                    if futl_fat_tipo_fatura == 'R' and finpar_cad_cliente_auto:
                        context_inativo = context.copy()
                        context_inativo['active_test'] = False
                        cliente_id = self.pool.get('financeiro.cliente').search(cr, uid, [
                            ('pessoa_id', '=', futl_fat_pessoa_id)], context=context_inativo)
                        if not cliente_id:
                            self.pool.get('financeiro.cliente').create(cr, uid, {
                                'pessoa_id': futl_fat_pessoa_id}, context)
                    if futl_fat_tipo_fatura == 'P' and finpar_cad_fornec_auto:
                        fornecedor_id = self.pool.get('financeiro.fornecedor').search(cr, uid, [
                            ('pessoa_id', '=', futl_fat_pessoa_id)], context=context)
                        if not fornecedor_id:
                            self.pool.get('financeiro.fornecedor').create(cr, uid, {
                                'pessoa_id': futl_fat_pessoa_id}, context)
            else:
                # Incluir cliente/fornecedor. Esta no else, porque se tem titulo ja fez isto no titulo.#
                if futl_fat_tipo_fatura == 'R' and finpar_cad_cliente_auto:
                    cliente_id = self.pool.get('financeiro.cliente').search(cr, uid, [
                        ('pessoa_id', '=', futl_fat_pessoa_id)], context=context)
                    if not cliente_id:
                        self.pool.get('financeiro.cliente').create(cr, uid, {
                            'pessoa_id': futl_fat_pessoa_id}, context)
                if futl_fat_tipo_fatura == 'P' and finpar_cad_fornec_auto:
                    fornecedor_id = self.pool.get('financeiro.fornecedor').search(cr, uid, [
                        ('pessoa_id', '=', futl_fat_pessoa_id)], context=context)
                    if not fornecedor_id:
                        self.pool.get('financeiro.fornecedor').create(cr, uid, {
                            'pessoa_id': futl_fat_pessoa_id}, context)

        # [CREATE] ________________________ CRIA A FATURA __________________________________
        # OBRIGATORIO O USO DO CONTEXTO
        futl_fat_fatura_id = self.pool.get('financeiro.fatura').create(cr, uid, vals_fatura,
                                                                       context=context)
        self.pool.get('adm.transacao.capa').admtrc_criar_transacao(cr, uid, 'fatu',
                                                                   futl_fat_fatura_id,
                                                                   context=None)

        #
        return futl_fat_fatura_id

    ####################################################################################################################
    # CRIAÇÃO DE TITULO
    ####################################################################################################################
    def financ_titulo_entrada(self, cr, uid, fult_tit_pessoa_id, fult_tit_emissao,
                              fult_tit_vencimento, fult_tit_documento, fult_tit_parcela,
                              fult_tit_valor,
                              fult_tit_tipo_lancamento, fult_tit_tipo_titulo, fult_tit_obs=False,
                              fult_tit_fatura_id=False, fult_tit_moeda=False,
                              fult_tit_doc_tipo_id=False,
                              fult_tit_previsao_id=False, fult_tit_tipo_sacado=False,
                              fult_tit_vendedor_id=False, fult_tit_vendedor_externo_id=False,
                              fult_tit_chq_banco_id=False, fult_tit_chq_agencia_id=False,
                              fult_tit_chq_conta=False, fult_tit_chq_emitente_id=False,
                              fult_tit_custificacao_dados=False, fult_tit_operacao_id=False,
                              fult_tit_codigo_barras=False,
                              filt_tit_local_cobranca_id=False,
                              filt_tit_natureza_cobranca_id=False,
                              context=None,
                              **campos_adicionais):
        """
        - Este metodo tem a seguinte funcao:
          - Incluir Titulos no financeiro_titulo,
          - Inclui a pessoa como cliente/fornecedor se estiver marcado para cadastrar automaticamente.
          - Atualiza o sistema de controle de transacoes,
          - Inclui historico na classe financeiro_titulo_historico.
        - Os Campos com valores Default especificados, nao sao obrigatorios e nao devem ser informados se nao houver a informacao
        - Retorna: False se houver algum problema ou
                 o id do titulo se tudo der certo
        - Havendo erro na execucao do metodo, recuperar o erro atraves do metodo financ_get_erro()
        - campos_adicionais é um dicionario com outros campos que não estão na assinatura e são necessários para gravar o titulo
        """
        context = context and context.copy() or {}
        # UPDATE UTILIZADO NO CONTEXTO PARA EVITAR RECURSIVIDADE NO CREATE
        context.update(self._context_financeiro_util)

        # ___________________________ Objects ________________________________
        obj_documento_tipo = self.pool.get('documento.documento.tipo')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_adm_empresa = self.pool.get('adm.empresa.objeto')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        obj_financeiro_titulo_boleto = self.pool.get('financeiro.titulo.boleto')

        self._mensagem_erro = u''
        self._tem_erro = False
        self._financ_validar_pessoa(fult_tit_pessoa_id)
        self._financ_validar_data(fult_tit_emissao, u'emissao')
        self._financ_validar_data(fult_tit_vencimento, u'vencimento')

        brw_adm_pessoa = obj_adm_pessoa.browse(cr, uid, fult_tit_pessoa_id, context)
        fult_tit_emissao, fult_tit_vencimento = self._financ_validar_venc_maior_emissao(
            fult_tit_emissao, fult_tit_vencimento)
        doc_tip_prv_id = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)[0]

        if not fult_tit_documento:
            if not doc_tip_prv_id or doc_tip_prv_id != fult_tit_doc_tipo_id:
                self._tem_erro = True
                self._mensagem_erro += _('Nao foi informado o numero do documento!\n')

        self._financ_validar_tipo_rec_pag(fult_tit_tipo_titulo, u'titulo')
        self._financ_validar_valor(fult_tit_valor)

        fields_to_read = [
            'moeda_id', 'local_cobranca_id', 'cadastro_automatico_cliente', 'juros_atraso',
            'multa_atraso', 'usar_centro_custos_e_eventos', 'cadastro_automatico_fornecedor']
        parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', fields_to_read, context=context)
        finpar_local_cobranca_id = parametro_dados.get('local_cobranca_id')
        finpar_moeda_id = parametro_dados.get('moeda_id')
        finpar_cad_cliente_auto = parametro_dados.get('cadastro_automatico_cliente')
        finpar_cad_fornec_auto = parametro_dados.get('cadastro_automatico_fornecedor')
        finpar_controle_custificacao = parametro_dados.get('usar_centro_custos_e_eventos')

        # CONTEXTO - ctx_fisnfc_titulo_create_sem_custificacao
        # LOCALIZA-SE - na fatura da nf
        # UTILIZADO PARA - criar um titulo sem que seja necessario passar os dados da
        # custificacao (fult_ldict_custifacacao)
        # MOTIVO - evitar redundancia de ter que utilizar a funcao que busca os dados para cada
        # titulo, pois a custificacao e a mesma para todos os titulos criados para a nota. Apos a
        # criacao do titulo, a funcao para criar custificacoes e acionada na nota fiscal
        if not context.get('ctx_fisnfc_titulo_create_sem_custificacao') and not context.get(
                'fintit_titulo_sem_custificacao'):
            if finpar_controle_custificacao:
                # no refinanciamento nao eh obrigatorio a custificacao
                if not fult_tit_custificacao_dados and not context.get('refinanciamento_titulo'):
                    self._tem_erro = True
                    self._mensagem_erro += _(
                        'Custificacao do Titulo nao informada. (Centro de custo e Evento)\n')

        if not fult_tit_moeda:
            fult_tit_moeda = finpar_moeda_id
        company_id = obj_adm_empresa.getEmpresa(cr, uid, 'financeiro.titulo', context=context)
        if not fult_tit_doc_tipo_id:
            fult_tit_doc_tipo_id = obj_documento_tipo.search(
                cr, uid, [('sigla', '=', 'DP')], context=context)[0]
        local_cobranca_id = filt_tit_local_cobranca_id or finpar_local_cobranca_id
        natureza_cobranca_id = filt_tit_natureza_cobranca_id
        fult_tit_tipo_sacado = self._financ_validar_tipo_sacado(fult_tit_tipo_titulo,
                                                                fult_tit_tipo_sacado)
        if self._tem_erro:
            return False

        # Atributos obrigatorios (requeridos) #
        vals_titulo = {
            'adm_moeda_id': fult_tit_moeda,
            'company_id': company_id,
            'data_emissao': fult_tit_emissao,
            'data_prorrogacao': fult_tit_vencimento,
            'data_vencimento': fult_tit_vencimento,
            'doc_tipo_id': fult_tit_doc_tipo_id,
            'documento': fult_tit_documento,
            'local_cobranca_id': local_cobranca_id,
            'natureza_cobranca_id': natureza_cobranca_id,
            'parcela': fult_tit_parcela,
            'pessoa_id': fult_tit_pessoa_id,
            'saldo': fult_tit_valor,
            'state': 'A',
            'tipo_sacado': fult_tit_tipo_sacado,
            'tipo_titulo': fult_tit_tipo_titulo,
            'vr_original': fult_tit_valor,
            'fintit_operacao_id': fult_tit_operacao_id,
        }

        if context.get('custifica_contrato') or context.get('contrato_capa_id'):
            vals_titulo.update({'contrato_id': context.get('custifica_contrato') or context.get(
                'contrato_capa_id')})

        if 'vr_juros_dia' in campos_adicionais:
            vals_titulo['vr_juros_dia'] = campos_adicionais.get('vr_juros_dia')
            del campos_adicionais['vr_juros_dia']

        if 'vr_multa' in campos_adicionais:
            vals_titulo['vr_multa'] = campos_adicionais.get('vr_multa')
            del campos_adicionais['vr_multa']

        doc_tipo_dados = obj_documento_tipo.read(
            cr, uid, fult_tit_doc_tipo_id, ['sigla'], context=context)

        if doc_tipo_dados and isinstance(doc_tipo_dados, list):
            doc_tipo_dados = doc_tipo_dados[0]

        if isinstance(doc_tipo_dados, dict):
            doc_tipo_sigla = doc_tipo_dados.get('sigla')

            if doc_tipo_sigla == 'CHQ':
                if fult_tit_chq_agencia_id:
                    vals_titulo['fintit_chq_agencia'] = fult_tit_chq_agencia_id

                if fult_tit_chq_banco_id:
                    vals_titulo['fintit_chq_banco'] = fult_tit_chq_banco_id

                if fult_tit_chq_conta:
                    vals_titulo['fintit_chq_conta'] = fult_tit_chq_conta

                if fult_tit_chq_emitente_id:
                    vals_titulo['fintit_chq_emitente'] = fult_tit_chq_emitente_id
                else:
                    cpf_cnpj = brw_adm_pessoa.cpf_cnpj or ''
                    vals_titulo['fintit_chq_emitente'] = cpf_cnpj

        if fult_tit_fatura_id:
            vals_titulo['fatura_id'] = fult_tit_fatura_id

        # Incluir cliente/fornecedor
        fornecedor_ativo = True
        fornecedor_ids = obj_financeiro_fornecedor.search(
            cr, uid, [('pessoa_id', '=', fult_tit_pessoa_id)], context=context)

        if not fornecedor_ids:
            fornecedor_ids = obj_financeiro_fornecedor.search(
                cr, uid, [('pessoa_id', '=', fult_tit_pessoa_id), ('active', '=', False)],
                context=context)

            if fornecedor_ids:
                fornecedor_ativo = False

        if fult_tit_tipo_titulo == 'R' and finpar_cad_cliente_auto:
            cliente_ids = self.pool.get('financeiro.cliente').search(
                cr, uid, [('pessoa_id', '=', fult_tit_pessoa_id)], context=context)

            if not cliente_ids:
                context.update({'active_test': False})
                cliente_inativo_ids = self.pool.get('financeiro.cliente').search(
                    cr, uid, [('pessoa_id', '=', fult_tit_pessoa_id)], context=context)
                del context['active_test']
                if cliente_inativo_ids:
                    raise osv.except_osv(
                        _('Atencao'), _('O cliente "%s - %s" se encontra inativo.' % (
                            brw_adm_pessoa.codigo, brw_adm_pessoa.name)))

                self.pool.get('financeiro.cliente').create(
                    cr, uid, {'pessoa_id': fult_tit_pessoa_id}, context=context)

        elif fult_tit_tipo_titulo == 'P' and finpar_cad_fornec_auto:
            if fornecedor_ids and not fornecedor_ativo:
                obj_financeiro_fornecedor.write(
                    cr, uid, fornecedor_ids, {'active': True}, context)
            if not fornecedor_ids:
                obj_financeiro_fornecedor.create(
                    cr, uid, {'pessoa_id': fult_tit_pessoa_id}, context)
        elif fult_tit_tipo_titulo == 'P':
            if not finpar_cad_fornec_auto and not fornecedor_ativo:
                raise osv.except_osv(_('Atencao'),
                                     _('Fornecedor inativo, ativa-lo antes de lancar o titulo'))

        # Multa e juros devem ser calculados para o titulo a receber.
        if fult_tit_tipo_titulo == 'R':
            dado_multa_juros = self.pool.get('financeiro.titulo').calcula_multa_juros(
                cr, uid, fult_tit_valor, context=context)
            if dado_multa_juros:
                vals_titulo['vr_juros_dia'] = dado_multa_juros['juros']
                vals_titulo['vr_multa'] = dado_multa_juros['multa']
                if vals_titulo.get('vr_multa'):
                    vals_titulo['multa_apos_data'] = fult_tit_vencimento



        if fult_tit_obs:
            vals_titulo['observacoes'] = fult_tit_obs

        if fult_tit_previsao_id:
            vals_titulo['previsao_id'] = fult_tit_previsao_id

        if fult_tit_tipo_lancamento:
            vals_titulo['tipo_lancamento'] = fult_tit_tipo_lancamento

        if fult_tit_vendedor_id:
            vals_titulo['vendedor_id'] = fult_tit_vendedor_id

        if fult_tit_vendedor_externo_id:
            vals_titulo['vendedor_externo_id'] = fult_tit_vendedor_externo_id

        obj_financeiro_titulo = self.pool.get('financeiro.titulo')

        if campos_adicionais:
            vals_titulo.update(campos_adicionais)

        # Cria o titulo
        titulo_id = obj_financeiro_titulo.create(cr, uid, vals_titulo, context=context)

        # Atualiza a transacao
        self.pool.get('adm.transacao.capa').admtrc_criar_transacao(cr, uid, 'titu', titulo_id)

        # Exclui as previsoes financeiras para o cliente na data de vencimento informada
        doc_tipo_previsao_id = obj_documento_tipo.search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)[0]
        previsao_ids = obj_financeiro_titulo.search(cr, uid, [
            ('data_vencimento', '=', fult_tit_vencimento),
            ('pessoa_id', '=', fult_tit_pessoa_id), ('doc_tipo_id', '=', doc_tipo_previsao_id),
            ('id', '!=', titulo_id), ('tipo_titulo', '=', fult_tit_tipo_titulo),
            ('fatura_id', '=', False)], context=context)

        if previsao_ids:
            if context.get('custifica_contrato'):
                obj_gerencial_movimento = self.pool.get('gerencial.movimento')
                gerencial_movimento_ids = obj_gerencial_movimento.search(cr, uid, [
                    ('titulo_id', '=', previsao_ids[0])], context=context)
                obj_gerencial_movimento.write(cr, uid, gerencial_movimento_ids[0],
                                              {'titulo_id': titulo_id}, context=context)
            if not context.get('fintit_manter_prv'):
                obj_financeiro_titulo.unlink(cr, uid, previsao_ids, context)

        # Cria custificacao do titulo
        if fult_tit_custificacao_dados:
            obj_gerencial_movimento = self.pool.get('gerencial.movimento')
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'gerencial.movimento', context=context)
            operacao = 'E' if fult_tit_tipo_titulo == 'R' else 'S'

            for i in fult_tit_custificacao_dados:
                brw_financeiro_titulo = obj_financeiro_titulo.browse(
                    cr, uid, titulo_id, context)
                evento_id = i.get('custf_evento_id') or i.get('evento_id')
                centro_custo_id = i.get('custf_centro_custo_id') or i.get('centro_custo_id')
                valor = i.get('custf_valor') or i.get('valor')
                historico = i.get('custf_historico') or False
                vals = {
                    'company_id': company_id,
                    'valor': valor,
                    'titulo_id': titulo_id,
                    'evento_id': evento_id,
                    'centro_custo_id': centro_custo_id,
                    'pessoa_id': fult_tit_pessoa_id,
                    'fingmo_operacao': operacao,
                    'fingmo_data': brw_financeiro_titulo.data_emissao
                }
                if context.get('custifica_contrato') or context.get('contrato_capa_id'):
                    vals['fingmo_data'] = brw_financeiro_titulo.data_prorrogacao[:8] + '01'
                if historico:
                    vals['fingmo_historico'] = historico
                mov_id = obj_gerencial_movimento.create(cr, uid, vals, context)

        vals_historico = {
            'data_prorrogacao': fult_tit_vencimento,
            'local_cobranca_id': local_cobranca_id,
            'natureza_cobranca_id': natureza_cobranca_id,
            'operacao': 'I',
            'titulo_id': titulo_id,
        }

        # Cria historico do titulo
        self.pool.get('financeiro.titulo.historico').create(cr, uid, vals_historico, context)

        if fult_tit_codigo_barras:
            titulo_boleto_ids = obj_financeiro_titulo_boleto.search(
                cr, uid, [('fintib_titulo_id', '=', titulo_id),
                          ('fintib_local_cobranca_id', '=', local_cobranca_id),
                          ('fintib_codigo_barras', '=', fult_tit_codigo_barras)], context=context)
            if not titulo_boleto_ids:
                vals_boleto = {
                    'fintib_titulo_id': titulo_id,
                    'fintib_local_cobranca_id': local_cobranca_id,
                    'fintib_natureza_cobranca_id': natureza_cobranca_id,
                    'fintib_codigo_barras': fult_tit_codigo_barras
                }
                obj_financeiro_titulo_boleto.create(cr, uid, vals_boleto, context)

        return titulo_id

    def financ_get_parametro_id(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                    'financeiro.parametros', context=context)
        financeiro_parametro_ids = self.pool.get('financeiro.parametros').search(cr, uid, [
            ('company_id', '=', company_id)], context=context)
        if not financeiro_parametro_ids:
            raise osv.except_osv(_('Atencao'), _(
                'Nenhum registro encontrado no parametro do financeiro para a empresa logada!'))
        return financeiro_parametro_ids[0]

    def financ_get_parametro_field(self, cr, uid, field_to_read, context=None):
        """
        Funcao busca no parametro do financeiro se esta marcado para alterar juros, desconto e multa.
        """
        context = context and context.copy() or {}
        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', [field_to_read], context=context)
        res = financeiro_parametro_dados.get(field_to_read)
        return res

    def financ_convert_str_date(self, str_data, str_data_name, context=None):  #PASSADO PARA O UTILS
        """
        Funcao converte:
        * Formato a ser passado - YYYY-mm-dd
        * String p/ date
        """
        context = context and context.copy() or {}
        str_data_name = unicode(str_data_name)
        try:
            data = parse(str_data, yearfirst=True).date()
            return data
        except Exception:
            self._mensagem_erro += _('Data de %s nao e uma data valida! ') % str_data_name
            self._tem_erro = True
            return False

    def financ_validate_dates(self, str_data_inicio, str_data_final, str_data_inicio_name,
                              str_data_fim_name, brw_titulo=None, context=None):
        """
        Funcao verifica:
        * Data_inicial e maior que a data final
        """
        context = context and context.copy() or {}
        str_data_inicio_name = unicode(str_data_inicio_name)
        str_data_fim_name = unicode(str_data_fim_name)
        date_inicio = self.financ_convert_str_date(str_data=str_data_inicio,
                                                   str_data_name=str_data_inicio_name,
                                                   context=context)
        date_final = self.financ_convert_str_date(str_data=str_data_final,
                                                  str_data_name=str_data_fim_name, context=context)
        if date_inicio > date_final:
            self._mensagem_erro += _('A data de %s e maior que a data de %s ') % (
                str_data_inicio_name, str_data_fim_name)
            if brw_titulo:
                self._mensagem_erro += _(
                    '. \nDocumento: %s \nParcela: %s \nCliente: %s \nEmissao: %s') % (
                    brw_titulo.documento,
                    brw_titulo.parcela,
                    '%s - %s' % (brw_titulo.pessoa_id.codigo, brw_titulo.pessoa_id.nome),
                    parse(brw_titulo.data_emissao).strftime('%d/%m/%Y'))
            self._tem_erro = True
        return True

    def financ_forma_pgto_tipo_cheque(self, cr, uid, forma_pagamento_id, context=None):
        """
        Funcao verifica:
        * Se a sigla do pagamento e cheque.
        True - Tipo Cheque
        False - Nao e cheque
        """
        context = context and context.copy() or {}
        cheque = False
        if forma_pagamento_id:
            forma_pgto_dados = self.pool.get('adm.forma.pgto').read(
                cr, uid, forma_pagamento_id, ['sigla', 'condicao_forma_ids'], context=context)
            if forma_pgto_dados and isinstance(forma_pgto_dados, list):
                forma_pgto_dados = forma_pgto_dados[0]
            if isinstance(forma_pgto_dados, dict):
                cheque = forma_pgto_dados.get('sigla') == 'CH'
            if not cheque:
                condicao_forma_ids = forma_pgto_dados.get('condicao_forma_ids')
                if condicao_forma_ids:
                    brw_condicao_forma = self.pool.get('adm.condicao.forma').browse(
                        cr, uid, condicao_forma_ids, context=context)
                    cheque = list(set(map(lambda i: i.tipo_finalizadora == 'H',
                                          brw_condicao_forma)))[0]
        return cheque

    def financ_convert_to_decimal(self, value, places=2, arredondamento=None):
        """
        @param value: {int, long, float}
        @param places: {2 - Duas casas Decimais, 3 - Três casas decimais}
        @param arredondamento: {'DOWN' - Down, 'UP' - Up}
        @return:
        """

        casas_decimais = Decimal('10') ** (places * -1)

        if arredondamento == 'DOWN':
            arred = ROUND_DOWN
        elif arredondamento == 'UP':
            arred = ROUND_UP
        else:
            arred = None

        value = value or 0
        res = value
        if isinstance(value, (int, long, float)):
            res = Decimal(str(value)).quantize(exp=casas_decimais, rounding=arred)
        elif isinstance(value, Decimal):
            res = value.quantize(exp=casas_decimais, rounding=arred)
        return res

    ####################################################################################################################
    # BAIXA DE TITULO
    ####################################################################################################################
    def financ_baixar_titulo(self, cr, uid, futl_bxti_titulos, futl_bxti_data_pagamento,
                             ftul_bxti_forma_pgto_id, futl_bxti_tesouraria_conta_id,
                             futl_bxti_baixa_multipla=False, futl_bxti_cheque_conta='',
                             futl_bxti_cheque_banco_id=False, futl_bxti_cheque_numero='',
                             futl_bxti_cheque_emitente_id=False, futl_bxti_agencia_id=False,
                             futl_bxti_data_credito_debito=False, futl_bxti_cheque_bom_para=False,
                             futl_bxti_pessoa_id=False,
                             futl_bxti_pagamento_dados=[], futl_bxti_company_id=False,
                             context=None):
        """
        ------------------------------------------------------------
        [PARAMETROS TIPOS]:
        INTEGER - futl_bxti_data_pagamento, futl_bxti_tesouraria_conta_id, futl_bxti_cheque_banco_id, futl_bxti_cheque_emitente_id, futl_bxti_agencia_id

        CHAR - futl_bxti_data_pagamento, futl_bxti_cheque_conta, futl_bxti_cheque_numero

        FLOAT - futl_bxti_valor_pago, futl_bxti_multa, futl_bxti_juros_dia, futl_bxti_desconto

        BOOLEAN - futl_bxti_baixa_multipla

        LIST WITH DICTIONARY - futl_bxti_titulos - [
                                        {'titulo_id', 'valor_pago', 'valor_multa', 'valor_juros',
                                        'valor_desconto'}
                               ],
        DICTONARY - context

        ------------------------------------------------------------
        [FUNÇÕES DO METODO]
        Atualiza:
            [WRITE] ATUALIZA O REGISTRO NO FINANCEIRO.TITULO
            [WRITE] ATUALIZA O REGISTRO NO FINANCEIRO.TITULO.BAIXA

        Cria:
            [CREATE] CRIA UM REGISTRO NO FINANCEIRO.TITULO.HISTORICO
            [CREATE] CRIA UM REGISTRO NO FINANCEIRO.TITULO.BAIXA
            [CREATE] CRIA UM REGISTRO NO FINANCEIRO.TITULO (CHEQUE)
            [CREATE] CRIA UM REGISTRO NO TESOURARIA.MOVIMENTO
            [CREATE] CRIA UM REGISTRO NO TESOURARIA.BAIXA.FORMA.PGTO
            [CREATE] CRIA UM REGISTRO NO GERENCIAL.MOVIMENTO (CUSTIFICACAO INFORMADA NO TITULO)
            [CREATE] CRIA UM REGISTRO NO GERENCIAL.MOVIMENTO (SALDO RESTANTE)
            [CREATE] CRIA UM REGISTRO NO GERENCIAL.MOVIMENTO (DESCONTOS)
            [CREATE] CRIA UM REGISTRO NO GERENCIAL.MOVIMENTO (JUROS)
            [CREATE] CRIA UM REGISTRO NO GERENCIAL.MOVIMENTO (MULTA)

        [MENSAGENS]
        As mensagens terminas em ':' servem para mostrar o titulo para o usuario.
        A string 'ctx_util_baixa_lt' da mensagem de erro server como um flag no
        WIZARD DE BAIXA para mostrar o documento/parcela do titulo nas mensagens.
        """
        context = context and context.copy() or {}
        # UPDATE UTILIZADO NO CONTEXTO PARA EVITAR RECURSIVIDADE NOS CREATES.
        context.update(self._context_financeiro_util)
        context.update({'financ_baixar_titulo': True})

        self._mensagem_erro = ''
        self._tem_erro = False

        if not futl_bxti_data_credito_debito:
            futl_bxti_data_credito_debito = futl_bxti_data_pagamento

        # ___________________________ Objects ________________________________
        obj_gerencial_mov = self.pool.get('gerencial.movimento')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_tesouraria_baixa_pgto = self.pool.get('financeiro.tesouraria.baixa.forma.pgto')
        obj_documento_tipo = self.pool.get('documento.documento.tipo')

        convert_2_decimal = self.financ_convert_to_decimal
        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', ['prazo_perdao',
                    'finpar_evento_refiananciamento_rec_id', 'finpar_centro_custo_refinanciamento_rec_id',
                    'finpar_evento_refinanciamento_pag_id', 'finpar_centro_custo_refinanciamento_pag_id'])
        prazo_perdao = financeiro_parametro_dados.get('prazo_perdao') or 0.0

        # parametros de centro de custo e evento para titulos renegociados a receber e pagar
        reneg_rec_evento_id = financeiro_parametro_dados.get('finpar_evento_refiananciamento_rec_id')
        reneg_rec_centro_id = financeiro_parametro_dados.get('finpar_centro_custo_refinanciamento_rec_id')
        reneg_pag_evento_id = financeiro_parametro_dados.get('finpar_evento_refinanciamento_pag_id')
        reneg_pag_centro_id = financeiro_parametro_dados.get('finpar_centro_custo_refinanciamento_pag_id')

        titulo_refinanciado = False

        if not futl_bxti_titulos:
            self._mensagem_erro += _('Nenhum titulo informado para ser baixado')
            self._tem_erro = True

        # Parametro do Financeiro
        parametro_controla_custificacao = self.financ_get_parametro_field(
            cr, uid, field_to_read='usar_centro_custos_e_eventos', context=context)
        date_baixa_pagamento = self.financ_convert_str_date(
            str_data=futl_bxti_data_pagamento, str_data_name=u'data pagamento')

        #################### CRIAÇÃO DA TESOURARIA #######################
        #SE O TITULO FOR RECEBIMENTO, NA TESOURARIA SERÁ CRÉDITO, SENÃO DEBITO.
        """
        Atividade: 79126
        O histórico deve ser montado da seguinte maneira:
            - Agrupar por cliente/fornecedor e por tipo dos titulos (receber/pagar)
        """
        pessoa_ids = list(set(map(lambda i: i.get('pessoa_id'), futl_bxti_titulos)))
        titulo_ids = map(lambda i: i.get('titulo_id'), futl_bxti_titulos)
        titulo_receber = filter(lambda i: i.get('tipo_titulo') == 'R', futl_bxti_titulos)
        total_titulo_receber = sum(map(lambda i: i.get('valor_pago'), titulo_receber))
        titulo_pagar = filter(lambda i: i.get('tipo_titulo') == 'P', futl_bxti_titulos)
        total_titulo_pagar = sum(map(lambda i: i.get('valor_pago'), titulo_pagar))
        valor_pago = total_titulo_receber - total_titulo_pagar
        tesouraria_cred_ou_deb = 'C' if valor_pago > 0 else 'D'

        titulo_historico = ''
        pessoa_dados = self.pool.get('adm.pessoa').read(cr, uid, pessoa_ids, ['nome'], context=context)
        for pessoa in pessoa_dados:
            pessoa_id = pessoa.get('id')
            pessoa_nome = pessoa.get('nome') or ''
            pessoa_titulos = filter(lambda i: i.get('pessoa_id') == pessoa_id, futl_bxti_titulos)
            pessoa_titulo_receber = filter(lambda i: i.get('tipo_titulo') == 'R', pessoa_titulos)
            pessoa_titulo_pagar = filter(lambda i: i.get('tipo_titulo') == 'P', pessoa_titulos)
            if pessoa_titulo_receber:
                pessoa_titulo_ids = map(lambda i: i.get('titulo_id'), pessoa_titulo_receber)
                list_documentos = list((d['documento'] for d in obj_financeiro_titulo.read(
                    cr, uid, pessoa_titulo_ids, ['documento'], context=context)))
                str_operacao = _('Recebimento')
                if len(pessoa_titulo_receber) > 1:
                    str_documento = '%s e %s' % (', '.join(list_documentos[:-1]), list_documentos[-1])
                    titulo_historico += _('\n%s do montante da baixa dos titulos %s de %s') % (
                        str_operacao, str_documento, pessoa_nome)
                else:
                    if context.get('ecfccp_cupom_titulo'):
                        str_tipo_documento = 'cupom'
                        list_documentos = [context.get('ecfccp_cupom_titulo')]
                    else:
                        str_tipo_documento = 'titulo'
                    titulo_historico += _('\n%s do %s %s de %s') % (
                        str_operacao, str_tipo_documento, list_documentos[0], pessoa_nome)
            if pessoa_titulo_pagar:
                pessoa_titulo_ids = map(lambda i: i.get('titulo_id'), pessoa_titulo_pagar)
                list_documentos = list((d['documento'] for d in obj_financeiro_titulo.read(
                    cr, uid, pessoa_titulo_ids, ['documento'], context=context)))
                str_operacao = _('Pagamento')
                if len(pessoa_titulo_pagar) > 1:
                    str_documento = '%s e %s' % (', '.join(list_documentos[:-1]), list_documentos[-1])
                    titulo_historico += _('\n%s do montante da baixa dos titulos %s de %s') % (
                        str_operacao, str_documento, pessoa_nome)
                else:
                    titulo_historico += _('\n%s do titulo %s de %s') % (
                        str_operacao, list_documentos[0], pessoa_nome)

        if valor_pago < 0:
            valor_pago *= -1

        tesouraria_mov_id = False
        pagamento_dados = [{'tesouraria_conta_id': futl_bxti_tesouraria_conta_id,
                            'valor': valor_pago}]

        # caso tenha este parametro, e houver condicoes a prazo, estaremos refinanciando o titulo
        if futl_bxti_pagamento_dados:
            pagamento_dados = futl_bxti_pagamento_dados
            # Filtra os pagamentos realizados com condicao a prazo, se tiver, o titulo esta sendo
            # refinanciado.
            if filter(lambda i: i.get('condicao_tipo') == 'C', futl_bxti_pagamento_dados):
                titulo_refinanciado = True

        brw_titulos = obj_financeiro_titulo.browse(cr, uid, titulo_ids, context)
        titulo_cheque_ids = filter(lambda i: i.fintit_cheque, brw_titulos)
        if titulo_cheque_ids:
            historico_cheque = '. Cheque(s) numero(s): '
            for titulo_cheque in titulo_cheque_ids:
                historico_cheque += titulo_cheque.documento
                if titulo_cheque != titulo_cheque_ids[-1]:
                    historico_cheque += ', '
                else:
                    historico_cheque += '.'
            titulo_historico += historico_cheque

        # guarda os ids das pessoas para criar os novos titulos e faturas
        novos_titulos_pessoa_ids = []
        nf_id = False
        for i in brw_titulos:
            if i.fatura_id.id and i.fatura_id.nf_id.id:
                # caso tenha titulos de notas diferentes, nao grava o id da nota no movimento da tesouraria
                if nf_id and nf_id != i.fatura_id.nf_id.id:
                    nf_id = False
                    break

                nf_id = i.fatura_id.nf_id.id

        for pagamento in pagamento_dados:
            # para titulos refinanciados nao cria movimento na tesouraria
            if titulo_refinanciado:
                novos_titulos_pessoa_ids.append(
                            pagamento.get('administradora_cartao_pessoa_id') or futl_bxti_pessoa_id
                            or futl_bxti_titulos[0].get('pessoa_id'))
                continue
            if titulo_historico[:1] == '\n':
                titulo_historico = titulo_historico[1:]
            vals_tesouraria = {
                'state': 'A',
                'conta_id': pagamento.get('tesouraria_conta_id'),
                'historico': titulo_historico,
                'debito_credito': tesouraria_cred_ou_deb,
                'data_movimento': futl_bxti_data_credito_debito,
                'ano_competencia': date_baixa_pagamento.year,
                'mes_competencia': date_baixa_pagamento.month,
                'valor': pagamento.get('valor'),
            }
            if nf_id:
                vals_tesouraria['fintem_nf_id'] = nf_id

            # Contexto utilizado no create da tesouraria.movimento
            context.update({'baixar_titulo': True})
            tesouraria_mov_id = obj_tesouraria_movimento.create(
                cr, uid, vals_tesouraria, context=context)

            if not tesouraria_mov_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('Os dados da tesouraria movimento nao foram criados na base '
                                       'de dados!'))

            for x in titulo_ids:
                sql = """
                    INSERT INTO financeiro_m2m_titulo_tesouraria (id, titulo_id)
                    VALUES (%s, %s)
                """ % (tesouraria_mov_id, x)
                cr.execute(sql)

        # retira ids repetidos
        novos_titulos_pessoa_ids = list(set(novos_titulos_pessoa_ids))
        baixa_titulos_refinanciados_a_vista = []
        # cria faturas novas para o refinanceamento de acordo com as formas de pagamento e seus
        # respectivos titulos
        refinanciada_fatura_ids = []
        for fatura_pessoa_id in novos_titulos_pessoa_ids:
            tipo_fatura = 'R' if tesouraria_cred_ou_deb == 'C' else 'P'
            documento_fatura = False

            if tipo_fatura == 'P':
                documento_fatura = futl_bxti_data_pagamento.replace('-', '') + str(randint(1, 1000))

            fatura_id = self.financ_incluir_fatura(
                cr, uid,
                futl_fat_pessoa_id=fatura_pessoa_id,
                futl_fat_data_emissao=futl_bxti_data_pagamento,
                futl_fat_valor=False,
                futl_fat_tipo_fatura=tipo_fatura,
                futl_fat_documento=documento_fatura,
                context=context)

            # Em cada titulo selecionado para baixa, informe o id desta nova fatura no campo
            # fintit_renegociacao_fatura_ids
            refinanciada_fatura_ids.append(fatura_id)
            # Para cada linha a prazo ou cartao da tree, crie novas duplicatas ligadas a esta nova
            # fatura, nos prazos da forma de pagamento e condicao escolhida
            pagamento_data_vencimento = futl_bxti_data_pagamento

            # Documento tipo CHEQUE
            documento_tipo_ids = obj_documento_tipo.search(cr, uid, [('sigla', '=', 'CHQ')],
                                                           context=context)
            documento_tipo_cheque_id = documento_tipo_ids[0] if documento_tipo_ids else False
            # para refinanciamento de titulos gera novos titulos de acordo com os pagamentos
            for pagamento in pagamento_dados:
                # cria os titulos apenas para a fatrua que o titulo corresponde
                if not pagamento.get('administradora_cartao_pessoa_id') and \
                       pagamento.get('pessoa_id') != fatura_pessoa_id or \
                      (pagamento.get('administradora_cartao_pessoa_id') and \
                       pagamento.get('administradora_cartao_pessoa_id') != fatura_pessoa_id):
                    continue

                # titulo a vista nao tem parcelas mas precisa ser criado
                qtd_parcelas = pagamento.get('qtd_parcelas') or 0
                a_vista = False
                if not qtd_parcelas:
                    qtd_parcelas = 1
                    a_vista = True

                prazo = 0
                valor_parcela = round(pagamento.get('valor') / qtd_parcelas, 2)
                list_valor_parcela = list()
                map(lambda i: list_valor_parcela.append(valor_parcela), range(0, qtd_parcelas))
                total_parcelas = valor_parcela * qtd_parcelas
                if round(total_parcelas, 2) != round(pagamento.get('valor'), 2):
                    diferenca = total_parcelas - pagamento.get('valor')
                    list_valor_parcela[0] += diferenca * -1

                datas_pgto = []
                # este argumento vem da baixa parcial de titulos
                if pagamento.get('parcelas'):
                    list_valor_parcela = []
                    for i in sorted(pagamento.get('parcelas'), key=lambda x: x.get('data')):
                        list_valor_parcela.append(i.get('valor'))
                        datas_pgto.append(i.get('data'))

                for parcela in range(1, qtd_parcelas + 1):
                    if not prazo: # prazo da primeira parcela
                        prazo = pagamento.get('dias_primeira_parcela') or 0
                    else: # prazo das demais parcelas
                        prazo = pagamento.get('dias_incremento') or 0

                    pagamento_data_vencimento = datas.calcVencimento(pagamento_data_vencimento, prazo)
                    if datas_pgto:
                        pagamento_data_vencimento = datas_pgto[parcela-1]

                    documento_titulo = datetime.now().strftime('%y%m%d%H%M%s')
                    if all([pagamento.get('agencia'), pagamento.get('banco'),
                            pagamento.get('bom_para')]):
                        documento_tipo_id = documento_tipo_cheque_id
                    else:
                        documento_tipo_id = self.pool.get('adm.forma.pgto').read(
                            cr, uid, pagamento.get('forma_pagamento_id'), ['admfpg_documento_tipo_id'],
                            context, '_classic_write').get('admfpg_documento_tipo_id')

                    dados_custificacao = False
                    if parametro_controla_custificacao:
                        if tipo_fatura == 'R'  and reneg_rec_evento_id  and reneg_rec_centro_id:
                            dados_custificacao = {'custf_evento_id': reneg_rec_evento_id,
                                                  'custf_centro_custo_id': reneg_rec_centro_id}
                        elif reneg_pag_evento_id and reneg_pag_centro_id:
                            dados_custificacao = {'custf_evento_id': reneg_pag_evento_id,
                                                  'custf_centro_custo_id': reneg_pag_centro_id}
                        if dados_custificacao:
                            dados_custificacao.update({'custf_valor': pagamento.get('valor')/
                                                                      qtd_parcelas,
                                                       'custf_historico': 'Refinanciamento'})
                            dados_custificacao = [dados_custificacao]

                    context_ref = context.copy()
                    context_ref['refinanciamento_titulo'] = True
                    tit_ref_id = self.financ_titulo_entrada(
                        cr, uid,
                        fult_tit_pessoa_id=fatura_pessoa_id,
                        fult_tit_emissao=futl_bxti_data_pagamento,
                        fult_tit_vencimento=pagamento.get('bom_para') or pagamento_data_vencimento,
                        fult_tit_documento=pagamento.get('numero') or documento_titulo,
                        fult_tit_parcela=parcela,
                        fult_tit_valor=list_valor_parcela[parcela - 1],
                        fult_tit_tipo_lancamento='A',
                        fult_tit_tipo_titulo=tipo_fatura,
                        fult_tit_fatura_id=fatura_id,
                        fult_tit_doc_tipo_id=documento_tipo_id,
                        fult_tit_chq_banco_id=pagamento.get('banco'),
                        fult_tit_chq_agencia_id=pagamento.get('agencia'),
                        fult_tit_chq_conta=pagamento.get('conta'),
                        fult_tit_chq_emitente_id=futl_bxti_pessoa_id,
                        fult_tit_custificacao_dados=dados_custificacao,
                        context=context_ref)

                    #guarda
                    if a_vista:
                        baixa_titulos_refinanciados_a_vista.append(dict(
                            titulos=[{'titulo_id': tit_ref_id,
                                      'pessoa_id': fatura_pessoa_id,
                                      'valor_pago': pagamento.get('valor'),
                                      'valor_juros': 0,
                                      'valor_multa': 0,
                                      'saldo_baixado': pagamento.get('valor'),
                                      'valor_desconto': 0,
                                      'valor_acrescimo': 0,
                                      'tipo_titulo': tipo_fatura}],
                            data_pagamento=futl_bxti_data_pagamento,
                            forma_pgto_id=ftul_bxti_forma_pgto_id,
                            tesouraria_conta_id=futl_bxti_tesouraria_conta_id,
                            pessoa_id=fatura_pessoa_id,
                            pagamento_dados=[pagamento],
                            context=context)
                        )

                    # Gerar a fatura e o titulo referente as taxas do cartao
                    if pagamento.get('taxa') <= 0 or tipo_fatura == 'P':
                        continue

                    documento_fatura = futl_bxti_data_pagamento.replace('-', '') + str(randint(1, 1000))
                    valor_taxa = list_valor_parcela[parcela - 1] * (pagamento.get('taxa') / 100.0)
                    taxa_fatura_id = self.financ_incluir_fatura(
                        cr, uid,
                        futl_fat_pessoa_id=pagamento.get('administradora_cartao_pessoa_id') or
                        futl_bxti_pessoa_id,
                        futl_fat_data_emissao=futl_bxti_data_pagamento,
                        futl_fat_valor=valor_taxa,
                        futl_fat_tipo_fatura='P',
                        futl_fat_documento=documento_fatura,
                        context=context)
                    refinanciada_fatura_ids.append(taxa_fatura_id)
                    self.financ_titulo_entrada(
                        cr, uid,
                        fult_tit_pessoa_id=pagamento.get('administradora_cartao_pessoa_id') or
                        futl_bxti_pessoa_id,
                        fult_tit_emissao=futl_bxti_data_pagamento,
                        fult_tit_vencimento=pagamento_data_vencimento,
                        fult_tit_documento=documento_titulo,
                        fult_tit_parcela=parcela,
                        fult_tit_valor=valor_taxa,
                        fult_tit_tipo_lancamento='A',
                        fult_tit_tipo_titulo='P',
                        fult_tit_fatura_id=taxa_fatura_id,
                        context=context_ref, **dict(pagto_cartao=True))
            if fatura_id and not obj_financeiro_titulo.search(
                    cr, uid, [('fatura_id', '=', fatura_id)], limit=1, context=context):
                raise osv.except_osv(_('Atencao'), _('Fatura de refinanciamento sem titulo '
                                                     'informado.'))

        if refinanciada_fatura_ids:
            obj_financeiro_titulo.write(
                cr, uid, titulo_ids,
                {'fintit_renegociacao_fatura_ids': [(6, False, refinanciada_fatura_ids)]},
                context=context)

        # Custificaçao do titulo. Titulos refinanciados nao geram custos
        custificacao_dados = list()
        if parametro_controla_custificacao and tesouraria_mov_id:
            pessoa_ids = list(set(map(lambda i: i.get('pessoa_id'), futl_bxti_titulos)))
            pessoa_id = False
            pessoa_nome = ''

            if len(pessoa_ids) == 1:
                pessoa_id = pessoa_ids[0]
                pessoa_nome = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id).nome or ''

            custificao_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'gerencial.movimento', context=context)
            vals_custo_evento = {
                'pessoa_id': pessoa_id,
                'company_id': futl_bxti_company_id or custificao_company_id,
                'tesouraria_movimento_id': tesouraria_mov_id
            }

            # Cria as custificacoes para o titulo. Porem, estas possuem o ID da tesouraria,
            # indicando que nao estao em previsao.
            custificacao_dados = self.financ_get_custificacao_baixa(
                cr, uid, futl_bxti_titulos, context)

            for g in custificacao_dados:
                ger_mov_operacao = 'E' if g.get('tipo_titulo') == 'R' else 'S'
                custificacao_evento_id = g.get('evento_id') or False
                custificacao_centro_custo_id = g.get('centro_custo_id') or False
                custificacao_valor = g.get('valor') or 0
                historico = g.get('historico')
                data_gerencial = g.get('data')
                if pessoa_id and len(futl_bxti_titulos) == 1:
                    historico += ' de %s' % pessoa_nome

                vals_custo_evento.update({
                    'valor': custificacao_valor,
                    'evento_id': custificacao_evento_id,
                    'centro_custo_id': custificacao_centro_custo_id,
                    'fingmo_operacao': ger_mov_operacao,
                    'fingmo_historico': historico,
                    'fingmo_data': futl_bxti_data_credito_debito,
                })
                obj_gerencial_mov.create(cr, uid, vals_custo_evento, context=context)

        # Dados do titulo
        for titulo in futl_bxti_titulos:
            brw_titulo = obj_financeiro_titulo.browse(cr, uid, titulo.get('titulo_id'))
            titulo_id = titulo.get('titulo_id')
            valor_pago = titulo.get('valor_pago')
            valor_multa = titulo.get('valor_multa')
            valor_juros = titulo.get('valor_juros')
            valor_desconto = titulo.get('valor_desconto')
            saldo_baixado = titulo.get('saldo_baixado') or 0
            valor_acrescimo = titulo.get('valor_acrescimo')
            titulo_tipo = brw_titulo.tipo_titulo
            titulo_saldo = brw_titulo.saldo or 0
            titulo_parcela = brw_titulo.parcela or ''
            titulo_documento = brw_titulo.documento or ''
            titulo_data_emissao = brw_titulo.data_emissao
            titulo_data_prorrogacao = brw_titulo.data_prorrogacao
            titulo_pessoa_nome = brw_titulo.pessoa_id and brw_titulo.pessoa_id.name or ''
            titulo_natureza_id = brw_titulo.natureza_cobranca_id and brw_titulo.natureza_cobranca_id.id or False
            titulo_doc_tipo_sigla = brw_titulo.doc_tipo_id and brw_titulo.doc_tipo_id.sigla or ''
            titulo_local_cobranca_id = brw_titulo.local_cobranca_id and brw_titulo.local_cobranca_id.id or False

            date_titulo_prorrogacao = self.financ_convert_str_date(
                str_data=titulo_data_prorrogacao, str_data_name=u'prorrogacao', context=context
            )
            nro_semana = date.weekday(date_titulo_prorrogacao)
            dias_atraso = (date_baixa_pagamento - date_titulo_prorrogacao).days
            if (dias_atraso == 1 and nro_semana == 6) or (dias_atraso == 2 and nro_semana == 5):
                if nro_semana == 5:  # Sabado
                    date_titulo_prorrogacao = (parse(date_titulo_prorrogacao.strftime('%Y-%m-%d'),
                                                     yearfirst=True) + relativedelta(
                        days=+2)).date()
                elif nro_semana == 6:  # Domingo
                    date_titulo_prorrogacao = (parse(date_titulo_prorrogacao.strftime('%Y-%m-%d'),
                                                     yearfirst=True) + relativedelta(
                        days=+1)).date()
            if prazo_perdao > 0 and titulo_tipo == 'R':
                data_aux = date_titulo_prorrogacao + relativedelta(days=prazo_perdao)

            # [VALIDAÇÃO]
            if not ftul_bxti_forma_pgto_id:
                self._mensagem_erro += _(
                    '* Forma de pagamento nao informada para a baixa de titulo, ')
                self._tem_erro = True

            # Se o documento do titulo for previsao financeira, nao pode ser baixado.
            if titulo_doc_tipo_sigla == 'PRV':
                self._mensagem_erro += _('* Os titulos a seguir nao podem ser baixados, pois o '
                                         'tipo de documento e "Previsao financeira": '
                                         'ctx_util_baixa_lt')
                self._tem_erro = True

            # Se o parametro do financeiro estiver marcado para controlar custificação e o titulo
            # nao possuir nenhuma eh verificado se o tipo do titulo eh diferente de cheque e não
            # possui custificação
            if parametro_controla_custificacao and titulo_doc_tipo_sigla != 'CHQ' and not \
                    titulo_refinanciado:
                if not custificacao_dados and not context.get('context_nao_custificar_titulos'):
                    self._mensagem_erro += _('* Os titulos a seguir nao possuem custificacao: '
                                             'ctx_util_baixa_lt')
                    self._tem_erro = True

            # Datas (pagamento e emissao).
            self.financ_validate_dates(
                str_data_inicio_name=u'emissao', str_data_inicio=titulo_data_emissao,
                str_data_fim_name=u'pagamento', str_data_final=futl_bxti_data_pagamento,
                brw_titulo=brw_titulo, context=context
            )

            # Saldo restante do titulo
            if futl_bxti_baixa_multipla:
                # Em baixa multipla nao pode haver saldo restante, pois nao eh permitido pagar parte
                # do titulo, somente o valor integral
                saldo_restante = 0
            else:
                saldo_restante = convert_2_decimal(titulo_saldo - saldo_baixado)

            vals_titulo_update = {
                'saldo': saldo_restante,
            }

            vals_titulo_historico = {
                'titulo_id': titulo_id,
                'data_prorrogacao': titulo_data_prorrogacao,
                'local_cobranca_id': titulo_local_cobranca_id,
                'natureza_cobranca_id': titulo_natureza_id
            }

            if not saldo_restante:
                # Titulo baixado
                vals_titulo_update.update({'state': 'B'})
                vals_titulo_historico.update({'operacao': 'B'})
            elif saldo_restante < 0:  # saldo restante negativo
                self._mensagem_erro += _('* Titulo possui saldo restante negativo, ')
                self._tem_erro = True
            else:  # Baixa parcial (historico)
                vals_titulo_historico.update({'operacao': 'BP'})

            if self._tem_erro:
                if not futl_bxti_baixa_multipla:
                    # Remove a string CTX_UTIL_BAIXA_LT da mensagem de erro. esta string serve como
                    # um flag no wizard de baixa para mostrar o documento/parcela do titulo nas
                    # mensagens
                    self._mensagem_erro = self._mensagem_erro.replace('ctx_util_baixa_lt', '')
                return False
            else:
                if titulo_refinanciado:
                    context.update({'baixa_renegociacao': True})

                # Registro da baixa
                obj_financeiro_titulo.write(cr, uid, [titulo_id], vals_titulo_update, context)
                titulo_documento_parcela = u'%s/%s' % (titulo_documento, titulo_parcela)
                titulo_historico = _('%s Titulo %s de %s') % \
                                   (_('Recebimento') if titulo_tipo == 'R' else _('Pagamento'),
                                    titulo_documento_parcela,
                                    titulo_pessoa_nome)
                vals_titulo_baixa = {
                    'conta_id': futl_bxti_tesouraria_conta_id,
                    'descricao': 'Baixa',
                    'data_pgto': futl_bxti_data_pagamento,
                    'titulo_id': titulo_id,
                    'data_baixa': strftime('%Y-%m-%d'),
                    'valor_pago': valor_pago,
                    'valor_multa': valor_multa,
                    'valor_juros': valor_juros,
                    'forma_pgto_id': ftul_bxti_forma_pgto_id,
                    'valor_desconto': valor_desconto,
                    'fintib_vr_acrescimo': valor_acrescimo,
                    'hist_mov_tesouraria': titulo_historico,
                    'tesouraria_movimento_id': tesouraria_mov_id
                }

                # Cria um titulo baixa
                obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
                finan_titulo_baixa_id = obj_financeiro_titulo_baixa.create(
                    cr, uid, vals_titulo_baixa, context=context)
                obj_financeiro_titulo.write(cr, uid, [titulo_id], {}, context)

                if not finan_titulo_baixa_id:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Os dados do Financeiro Baixa nao foram criados na '
                                           'base de dados.'))

                if tesouraria_mov_id:  # Baixa na tesouraria somente pagamentos ´a vista´
                    vals_tesouraria_baixa = {
                        'forma_pgto_id': ftul_bxti_forma_pgto_id,
                        'tesouraria_baixa_id': finan_titulo_baixa_id,
                        'tesouraria_movimento_id': tesouraria_mov_id
                    }
                    vals_tesouraria_baixa.update({'valor': valor_pago})

                    # Cria um registro no tesouraria.baixa.forma.pgto
                    tesouraria_baixa_pgto_id = obj_tesouraria_baixa_pgto.create(
                        cr, uid, vals_tesouraria_baixa, context=context
                    )
                    if not tesouraria_baixa_pgto_id:
                        raise osv.except_osv(_('Atencao!'),
                                             _('As informacoes para a geracao de dados na '
                                               'Tesouraria Baixa, referentes as formas de '
                                               'pagamento nao foram criadas.'))

        # os titulos do gerados pelo refinanceamento a vista sao baixados
        for i in baixa_titulos_refinanciados_a_vista:
            self.financ_baixar_titulo(
                        cr, uid,
                        futl_bxti_titulos=i.get('titulos'),
                        futl_bxti_data_pagamento=i.get('data_pagamento'),
                        ftul_bxti_forma_pgto_id=i.get('forma_pgto_id'),
                        futl_bxti_tesouraria_conta_id=i.get('tesouraria_conta_id'),
                        futl_bxti_pessoa_id=i.get('pessoa_id'),
                        futl_bxti_pagamento_dados=i.get('pagamento'),
                        context=context)


        return True

    def financ_get_custificacao_baixa(self, cr, uid, titulos, context=None):
        context = context and context.copy() or {}
        custificacao_dados = []

        # Classificacao de contas
        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')

        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro',
            ['finpar_centro_custo_recebimentos_id', 'finpar_evento_recebimentos_id',
             'finpar_centro_custo_pagamentos_id', 'finpar_evento_pagamentos_id',
             'centro_custo_desconto_recebido_id', 'evento_desconto_recebido_id',
             'centro_custo_desconto_pago_id', 'evento_desconto_pago_id',
             'centro_custo_juros_recebidos_id', 'evento_juros_recebidos_id',
             'centro_custo_juros_pagos_id', 'evento_juros_pagos_id',
             'centro_custo_multa_recebida_id', 'evento_multa_recebida_id',
             'centro_custo_multa_paga_id', 'evento_multa_paga_id',
             'finpar_centro_custo_acrescimo_pago_id', 'finpar_centro_custo_acrescimo_recebido_id',
             'finpar_evento_acrescimo_pago_id', 'finpar_evento_acrescimo_recebido_id'])

        if not all(financeiro_parametro_dados.values()):
            raise osv.except_osv(_('Atencao!'),
                                 _('Valores de centro de custos nao estao corretos no parametro do '
                                   'Financeiro.'))

        dict_custificacao = {}
        for titulo in titulos:
            titulo_id = titulo.get('titulo_id')
            tipo_titulo = titulo.get('tipo_titulo')
            valor_juros = titulo.get('valor_juros')
            valor_multa = titulo.get('valor_multa')
            valor_desconto = titulo.get('valor_desconto')
            valor_acrescimo = titulo.get('valor_acrescimo') or 0
            valor_pago = titulo.get('valor_pago') - valor_juros - valor_multa + valor_desconto - \
                valor_acrescimo

            brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, titulo_id, context)
            data_emissao = brw_financeiro_titulo.data_emissao
            centro_custo_id = False
            evento_id = False
            local_cobranca_id = brw_financeiro_titulo.local_cobranca_id.id
            documento = brw_financeiro_titulo.documento

            if local_cobranca_id and tipo_titulo == 'R':
                args_financeiro_contas = [('finlon_local_cobranca_id', '=', local_cobranca_id)]
                financeiro_contas_ids = obj_financeiro_local_natureza.search(
                    cr, uid, args_financeiro_contas, context=context)
                if financeiro_contas_ids:
                    brw_financeiro_local_natureza = obj_financeiro_local_natureza. \
                        browse(cr, uid, financeiro_contas_ids[0], context)
                    centro_custo_id = brw_financeiro_local_natureza. \
                        finlon_centro_custo_id.id if brw_financeiro_local_natureza. \
                        finlon_centro_custo_id else False
                    evento_id = brw_financeiro_local_natureza. \
                        finlon_evento_id.id if brw_financeiro_local_natureza. \
                        finlon_evento_id else False

            #  Caso não exista na classificação de contas centro de custo ou evento
            # então será buscado  no parâmetro
            if tipo_titulo == 'R':
                if not centro_custo_id and financeiro_parametro_dados:
                    centro_custo_id = financeiro_parametro_dados.get(
                        'finpar_centro_custo_recebimentos_id')
                if not evento_id and financeiro_parametro_dados:
                    evento_id = financeiro_parametro_dados.get('finpar_evento_recebimentos_id')
            else:
                if not centro_custo_id and financeiro_parametro_dados:
                    centro_custo_id = financeiro_parametro_dados.get(
                        'finpar_centro_custo_pagamentos_id')
                if not evento_id and financeiro_parametro_dados:
                    evento_id = financeiro_parametro_dados.get('finpar_evento_pagamentos_id')
            if centro_custo_id and evento_id:
                centro_evento = '%s%s%s%s' % (centro_custo_id, evento_id, tipo_titulo, 'P')
                if not centro_evento in dict_custificacao:
                    dict_custificacao[centro_evento] = {'centro_custo_id': centro_custo_id,
                                                        'evento_id': evento_id,
                                                        'valor': valor_pago,
                                                        'tipo_titulo': tipo_titulo,
                                                        'tipo': 'P',
                                                        'documentos': [documento],
                                                        'data': data_emissao,
                                                        'titulo_id': titulo_id}
                else:
                    dict_custificacao[centro_evento]['valor'] += valor_pago
                    dict_custificacao[centro_evento]['documentos'].append(documento)
            if valor_desconto > 0:
                if tipo_titulo == 'R':
                    centro_custo_id = financeiro_parametro_dados.get(
                        'centro_custo_desconto_recebido_id') or False
                    evento_id = financeiro_parametro_dados.get(
                        'evento_desconto_recebido_id') or False
                    tipo_titulo_desc = 'P'
                else:
                    centro_custo_id = financeiro_parametro_dados.get(
                        'centro_custo_desconto_pago_id') or False
                    evento_id = financeiro_parametro_dados.get('evento_desconto_pago_id') or False
                    tipo_titulo_desc = 'R'
                if centro_custo_id and evento_id:
                    centro_evento = '%s%s%s%s' % (centro_custo_id, evento_id, tipo_titulo, 'D')
                    if not centro_evento in dict_custificacao:
                        dict_custificacao[centro_evento] = {'centro_custo_id': centro_custo_id,
                                                            'evento_id': evento_id,
                                                            'valor': valor_desconto,
                                                            'tipo_titulo': tipo_titulo_desc,
                                                            'tipo': 'D',
                                                            'documentos': [documento],
                                                            'data': data_emissao,
                                                            'titulo_id': titulo_id}
                    else:
                        dict_custificacao[centro_evento]['valor'] += valor_desconto
                        dict_custificacao[centro_evento]['documentos'].append(documento)

            if valor_acrescimo > 0:
                if tipo_titulo == 'R':
                    centro_custo_id = financeiro_parametro_dados.get(
                        'finpar_centro_custo_acrescimo_recebido_id') or False
                    evento_id = financeiro_parametro_dados.get(
                        'finpar_evento_acrescimo_recebido_id') or False
                else:
                    centro_custo_id = financeiro_parametro_dados.get(
                        'finpar_centro_custo_acrescimo_pago_id') or False
                    evento_id = financeiro_parametro_dados.get('finpar_evento_acrescimo_pago_id') \
                        or False
                if centro_custo_id and evento_id:
                    centro_evento = '%s%s%s%s' % (centro_custo_id, evento_id, tipo_titulo, 'A')
                    if not centro_evento in dict_custificacao:
                        dict_custificacao[centro_evento] = {'centro_custo_id': centro_custo_id,
                                                            'evento_id': evento_id,
                                                            'valor': valor_acrescimo,
                                                            'tipo_titulo': tipo_titulo,
                                                            'tipo': 'A',
                                                            'documentos': [documento],
                                                            'data': data_emissao,
                                                            'titulo_id': titulo_id}
                    else:
                        dict_custificacao[centro_evento]['valor'] += valor_acrescimo
                        dict_custificacao[centro_evento]['documentos'].append(documento)

            if valor_juros > 0:
                if tipo_titulo == 'R':
                    centro_custo_id = financeiro_parametro_dados.get(
                        'centro_custo_juros_recebidos_id') or False
                    evento_id = financeiro_parametro_dados.get('evento_juros_recebidos_id') or False
                else:
                    centro_custo_id = financeiro_parametro_dados.get(
                        'centro_custo_juros_pagos_id') or False
                    evento_id = financeiro_parametro_dados.get('evento_juros_pagos_id') or False
                if centro_custo_id and evento_id:
                    centro_evento = '%s%s%s%s' % (centro_custo_id, evento_id, tipo_titulo, 'J')
                    if not centro_evento in dict_custificacao:
                        dict_custificacao[centro_evento] = {'centro_custo_id': centro_custo_id,
                                                            'evento_id': evento_id,
                                                            'valor': valor_juros,
                                                            'tipo_titulo': tipo_titulo,
                                                            'tipo': 'J',
                                                            'documentos': [documento],
                                                            'data': data_emissao,
                                                            'titulo_id': titulo_id}
                    else:
                        dict_custificacao[centro_evento]['valor'] += valor_juros
                        dict_custificacao[centro_evento]['documentos'].append(documento)

            if valor_multa > 0:
                if tipo_titulo == 'R':
                    centro_custo_id = financeiro_parametro_dados.get(
                        'centro_custo_multa_recebida_id') or False
                    evento_id = financeiro_parametro_dados.get('evento_multa_recebida_id') or False
                else:
                    centro_custo_id = financeiro_parametro_dados.get(
                        'centro_custo_multa_paga_id') or False
                    evento_id = financeiro_parametro_dados.get('evento_multa_paga_id') or False
                if centro_custo_id and evento_id:
                    centro_evento = '%s%s%s%s' % (centro_custo_id, evento_id, tipo_titulo, 'M')
                    if not centro_evento in dict_custificacao:
                        dict_custificacao[centro_evento] = {'centro_custo_id': centro_custo_id,
                                                            'evento_id': evento_id,
                                                            'valor': valor_multa,
                                                            'tipo_titulo': tipo_titulo,
                                                            'tipo': 'M',
                                                            'documentos': [documento],
                                                            'data': data_emissao,
                                                            'titulo_id': titulo_id}
                    else:
                        dict_custificacao[centro_evento]['valor'] += valor_multa
                        dict_custificacao[centro_evento]['documentos'].append(documento)

        for x in dict_custificacao:
            tipo = dict_custificacao.get(x).get('tipo')
            tipo_titulo = dict_custificacao.get(x).get('tipo_titulo')
            isPlural = len(dict_custificacao.get(x).get('documentos')) > 1

            if tipo == 'P':
                if tipo_titulo == 'R':
                    if isPlural:
                        _historico = _('Recebimento do montante da baixa dos titulos %s')
                    else:
                        _historico = _('Recebimento do titulo %s')
                else:
                    if isPlural:
                        _historico = _('Pagamento do montante da baixa dos titulos %s')
                    else:
                        _historico = _('Pagamento do titulo %s')
            elif tipo == 'D':
                if tipo_titulo == 'P':
                    if isPlural:
                        _historico = _('Desconto concedido no montante da baixa dos titulos %s')
                    else:
                        _historico = _('Desconto concedido no titulo %s')
                else:
                    if isPlural:
                        _historico = _('Desconto obtido no montante da baixa dos titulos %s')
                    else:
                        _historico = _('Desconto obtido no titulo %s')
            elif tipo == 'A':
                if tipo_titulo == 'R':
                    if isPlural:
                        _historico = _('Acrescimo recebido no montante da baixa dos titulos %s')
                    else:
                        _historico = _('Acrescimo recebido no titulo %s')
                else:
                    if isPlural:
                        _historico = _('Acrescimo pago no montante da baixa dos titulos %s')
                    else:
                        _historico = _('Acrescimo pago no titulo %s')
            elif tipo == 'J':
                if tipo_titulo == 'R':
                    if isPlural:
                        _historico = _('Juros recebidos do montante da baixa dos titulos %s')
                    else:
                        _historico = _('Juros recebidos do titulo %s')
                else:
                    if isPlural:
                        _historico = _('Juros pagos do montante da baixa dos titulos %s')
                    else:
                        _historico = _('Juros pagos do titulo %s')
            elif tipo == 'M':
                if tipo_titulo == 'R':
                    if isPlural:
                        _historico = _('Multa recebida do montante da baixa dos titulos %s')
                    else:
                        _historico = _('Multa recebida do titulo %s')
                else:
                    if isPlural:
                        _historico = _('Multa paga do montante da baixa dos titulos %s')
                    else:
                        _historico = _('Multa paga do titulo %s')

            if isPlural:
                _historico %= ', '.join(dict_custificacao[x]['documentos'])
            else:
                _historico %= dict_custificacao[x]['documentos'][0]

            del dict_custificacao[x]['tipo']
            del dict_custificacao[x]['documentos']
            dict_custificacao[x]['historico'] = _historico
            custificacao_dados.append(dict_custificacao.get(x))

        return custificacao_dados

    def finutl_get_cliente_ids(self, cr, uid, venutl_pessoa_id, context=None):
        """
        BUSCA O CLIENTE_ID REFERENTE A PESSOA INFORMADA
        RETORNA UMA LISTA DOS IDS DOS CLIENTES OU LISTA VAZIA
        """
        context = context and context.copy() or {}
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        if not venutl_pessoa_id:
            return []

        if not isinstance(venutl_pessoa_id, list):
            venutl_pessoa_id = [venutl_pessoa_id]

        cliente_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                            'financeiro.cliente', context=context)
        cliente_company_ids = obj_adm_empresa_objeto.admemp_get_empresa_escolhida_ids(
            cr, uid, 'financeiro.cliente', cliente_company_id, context)
        context['sig_force_company'] = True
        return self.pool.get('financeiro.cliente').search(
            cr, uid, [('pessoa_id', 'in', venutl_pessoa_id),
                      ('company_id', 'in', cliente_company_ids)], context=context)

    def finutl_get_valor_em_atraso(self, cr, uid, venutl_pessoa_ids, context=None):
        """
        FUNCAO RETORNA DICIONARIO, SENDO A CHAVE O ID DA PESSOA E O VALOR DA CHAVE O TOTAL DOS
        TITULOS EM ATRASO
        """
        context = context and context.copy() or {}
        dict_total_valor_atraso = {}
        if isinstance(venutl_pessoa_ids, (int, long)):
            venutl_pessoa_ids = [venutl_pessoa_ids]
        if not venutl_pessoa_ids:
            return dict_total_valor_atraso

        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_adm_util = self.pool.get('adm.utils')
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        # Titulos do tipo previsao nao serao considerados
        doc_tipo_ids = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)

        data_prorrogacao = date.today()
        financeiro_parametro_dados = obj_adm_util.admutl_get_parametros(
            cr, uid, 'financeiro', ['finpar_carencia_restricao'])
        carencia_restricao = financeiro_parametro_dados.get('finpar_carencia_restricao') or 0
        carencia_restricao = 99999 if carencia_restricao > 99999 else carencia_restricao
        data_prorrogacao -= relativedelta(days=+carencia_restricao)
        args_financeiro = [
            ('pessoa_id', 'in', venutl_pessoa_ids), ('data_prorrogacao', '<', data_prorrogacao),
            ('state', '=', 'A'), ('tipo_titulo', '=', 'R'),
            ('doc_tipo_id', 'not in', doc_tipo_ids)
        ]

        cliente_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'financeiro.cliente',
                                                               context=context)
        company_ids = obj_adm_empresa_objeto.admemp_get_empresa_escolhida_ids(
            cr, uid, 'financeiro.cliente', cliente_company_id, context)

        # Identifica se a empresa em que o usuario esta 'logado', e fiscal
        logada_empresa_fiscal = self.pool.get('res.users').browse(
            cr, uid, uid, context).company_id.rescon_fiscal
        if not logada_empresa_fiscal:
            for c in self.pool.get('res.company').read(cr, uid, company_ids, ['rescon_fiscal'],
                                                       context):
                if c.get('rescon_fiscal'):
                    company_ids.remove(c.get('id'))

        args_financeiro.append(('company_id', 'in', company_ids))
        context['sig_force_company'] = True

        titulo_ids = obj_financeiro_titulo.search(cr, uid, args_financeiro, context=context)
        if not titulo_ids:
            return dict_total_valor_atraso

        titulo_dados = obj_financeiro_titulo.read(
            cr, uid, titulo_ids, ['saldo', 'pessoa_id'], context=context, load='_classic_write')
        for titulo in titulo_dados:
            pessoa_id = titulo.get('pessoa_id') or 0
            if not pessoa_id:
                continue

            valor_titulo = titulo.get('saldo') or 0
            if not dict_total_valor_atraso.get(pessoa_id):
                dict_total_valor_atraso[pessoa_id] = valor_titulo
            else:
                dict_total_valor_atraso[pessoa_id] += valor_titulo

        return dict_total_valor_atraso

    def finutl_get_valor_a_vencer(self, cr, uid, pessoa_ids, context=None):
        """
        O metodo calcula o valor total de titulos a vencer, por pessoa (pessoa_ids).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param pessoa_ids: lista de id da tabela adm_pessoa.
         :type pessoa_ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: DICIONARIO, SENDO A CHAVE O ID DA PESSOA E O VALOR DA CHAVE O TOTAL DOS
        TITULOS A VENCER
        """
        context = context and context.copy() or {}
        dict_total_valor_a_vencer = {}
        if isinstance(pessoa_ids, (int, long)):
            pessoa_ids = [pessoa_ids]
        if not pessoa_ids:
            return dict_total_valor_a_vencer

        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        # Titulos do tipo previsao nao serao considerados
        doc_tipo_ids = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)

        data_prorrogacao = date.today()
        args_financeiro = [
            ('pessoa_id', 'in', pessoa_ids), ('data_prorrogacao', '>=', data_prorrogacao),
            ('state', '=', 'A'), ('tipo_titulo', '=', 'R'),
            ('doc_tipo_id', 'not in', doc_tipo_ids)
        ]

        cliente_ids = list()
        cliente_ids.extend(self.finutl_get_cliente_ids(cr, uid, pessoa_ids, context))
        company_ids = list()
        map(lambda i: company_ids.append(i.get('company_id')),
            obj_financeiro_cliente.read(cr, uid, cliente_ids, ['company_id'], context=context,
                                        load='_classic_write'))
        company_ids = list(set(company_ids))
        args_financeiro.append(('company_id', 'in', company_ids))
        context['sig_force_company'] = True

        titulo_ids = obj_financeiro_titulo.search(cr, uid, args_financeiro, context=context)
        if not titulo_ids:
            return dict_total_valor_a_vencer

        titulo_dados = obj_financeiro_titulo.read(
            cr, uid, titulo_ids, ['saldo', 'pessoa_id'], context=context, load='_classic_write')
        for titulo in titulo_dados:
            pessoa_id = titulo.get('pessoa_id') or 0
            if not pessoa_id:
                continue

            valor_titulo = titulo.get('saldo') or 0
            if not dict_total_valor_a_vencer.get(pessoa_id):
                dict_total_valor_a_vencer[pessoa_id] = valor_titulo
            else:
                dict_total_valor_a_vencer[pessoa_id] += valor_titulo

        return dict_total_valor_a_vencer

    def finutl_verificar_restricao_cliente(self, cr, uid, finutl_pessoa_id, context=None):
        context = context and context.copy() or {}
        if not finutl_pessoa_id:
            return False
        else:
            obj_vendas_utils = self.pool.get('vendas.utils')
            #TITULOS EM ATRASO
            titulo_em_atraso = self.finutl_get_valor_em_atraso(
                cr, uid, venutl_pessoa_ids=finutl_pessoa_id, context=context
            ).get(finutl_pessoa_id) or 0

            #BLOQUEADO POR MENSAGEM
            bloqueado_por_mensagem = obj_vendas_utils.venutl_cliente_bloqueado_por_msg(
                cr, uid, venutl_pessoa_ids=finutl_pessoa_id, context=context
            ).get('pessoa_ids').get(finutl_pessoa_id)

            #LIMITE DE CREDITO
            cliente_limite_credito = obj_vendas_utils.venutl_get_limite_credito_cliente(
                cr, uid, pessoa_ids=finutl_pessoa_id, context=context
            ).get('pessoa_ids').get(finutl_pessoa_id)

            #BLOQUEADO POR LIMITE DE CREDITO
            bloqueado_por_credito = False

            if cliente_limite_credito:
                # DEBITO ANTEIOR
                cliente_debito = obj_vendas_utils.venutl_get_total_debito_cliente(
                    cr, uid,
                    venutl_pessoa_ids=finutl_pessoa_id,
                    context=context
                )
                cliente_debito = cliente_debito.get(finutl_pessoa_id) or 0

                if cliente_debito > cliente_limite_credito:
                    bloqueado_por_credito = True

            cliente_com_restricoes = any([titulo_em_atraso, bloqueado_por_mensagem,
                                          bloqueado_por_credito])
            return cliente_com_restricoes


    def finutl_observacao_cliente(self, cr, uid, pessoa_ids, context=None):

        '''Retorna as mensagens existentes para o cliente informado'''

        context = context and context.copy() or {}
        if isinstance(pessoa_ids, (int, long)):
            pessoa_ids = [pessoa_ids]
        res = {}.fromkeys(pessoa_ids, '')
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_financeiro_clientes_mensagem = self.pool.get('financeiro.clientes.mensagem')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        brw_pessoa = obj_adm_pessoa.browse(cr, uid, pessoa_ids)
        for i in brw_pessoa:
            record_pessoa = i
            if record_pessoa:
                pessoa_obs = record_pessoa.observacoes or ''
                pessoa_id = record_pessoa.id
                cliente_id = obj_financeiro_cliente.search(cr, uid,
                                                           [('pessoa_id', '=', pessoa_id)], context=context)
                if cliente_id:
                    cliente_mensagem_id = obj_financeiro_clientes_mensagem.search(cr, uid,
                                                                                  [(
                                                                                       'cliente_id',
                                                                                       'in',
                                                                                       cliente_id)], context=context)
                    if cliente_mensagem_id:
                        clientes_mensagem_dados = obj_financeiro_clientes_mensagem.browse(cr,
                                                                                          uid,
                                                                                          cliente_mensagem_id)
                        for j in clientes_mensagem_dados:
                            if j.modo_exibicao in ('V', 'A'):
                                mensagem_id = j.mensagens_id
                                if mensagem_id:
                                    pessoa_obs += '\n' + mensagem_id.name
            else:
                pessoa_obs = ''
            res[i.id] = pessoa_obs
        return res

    def finutl_formata_observacao(self, limit, observacao):
        """
        Metodo para quebrar em itens da lista quando estourar o limit maximo de caracters da
        observacao.
        :param limit: numero com a quantidade maxima de caracteres por linha.
        :type: integer
        :param observacao: descricao completa da observacao.
        :type: string
        :return: descricao da observacao repartida em uma lista.
        :rtype: list
        """
        cont = 0
        lista_obs = []
        obs = ""
        observacao = observacao.split('\n')
        observacao_ok = map(lambda i: len(i) <= 67, observacao)
        if all(observacao_ok):
            return observacao
        for i in observacao:
            cont_total = len(obs) + len(i)
            if len(i) < limit < cont_total:
                cont = cont_total
            for j in i:
                cont += 1
                if cont <= limit:
                    obs += j
                else:
                    lista_obs.append(obs)
                    obs = j
                    cont = 0
            obs = obs.ljust(len(obs) + 1)
            cont += 1
        if len(obs):
            lista_obs.append(obs)
        return lista_obs

    def finutl_formata_observacao(self, limit, observacao):
        """
        Metodo para quebrar em itens da lista quando estourar o limit maximo de caracters da
        observacao.
        :param limit: numero com a quantidade maxima de caracteres por linha.
        :type: integer
        :param observacao: descricao completa da observacao.
        :type: string
        :return: descricao da observacao repartida em uma lista.
        :rtype: list
        """
        cont = 0
        lista_obs = []
        obs = ""
        observacao = observacao.split('\n')
        observacao_ok = map(lambda i: len(i) <= 67, observacao)
        if all(observacao_ok):
            return observacao
        for i in observacao:
            cont_total = len(obs) + len(i)
            if len(i) < limit < cont_total:
                cont = cont_total
            for j in i:
                cont += 1
                if cont <= limit:
                    obs += j
                else:
                    lista_obs.append(obs)
                    obs = j
                    cont = 0
            obs = obs.ljust(len(obs) + 1)
            cont += 1
        if len(obs):
            lista_obs.append(obs)
        return lista_obs
