#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.tools import config as openerp_config
from ....fiscal.control.utils import log_debug
from openerp.tools.translate import _
from openerp.osv import osv
import logging

class groupby(dict):
    def __init__(self, seq, key=lambda x: x):
        for value in seq:
            k = key(value)
            self.setdefault(k, []).append(value)

    __iter__ = dict.iteritems


class gerencial_movimento(osv.Model):
    _name = 'gerencial.movimento'
    _inherit = 'gerencial.movimento'
    _logger = logging.getLogger(_name)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    def fingmo_atualiza_competencia(self, cr, uid, titulo_ids, competencia_por_parcelas,
                                    context=None):
        """
        O metodo atualiza os registros de competencia (mes / ano) da tabela gerencial_movimento.
        Se competencia_por_parcelas for True, os registros
        serao atualizados baseado na data de vencimento dos titulos, senao, na data de emissao.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param titulo_ids: lista de id da tabela financeiro_titulo
        :param competencia_por_parcelas: booleano
        :param context: dicionario com valores contextais
        :return: booleano, com True caso algum registro da tabela gerencial_movimento foi
        atualizado e False caso nenhum foi atualizado
        """
        context = context and context.copy() or {}
        res = False
        if not titulo_ids:
            return res
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        gerencial_movimento_ids = self.search(cr, uid, [('company_id', '=', company_id),
                                                        ('titulo_id', 'in', titulo_ids)],
                                              context=context)
        brw_gerencial_movimento = self.browse(cr, uid, gerencial_movimento_ids, context)
        for movimento in brw_gerencial_movimento:
            vals = {}
            if competencia_por_parcelas:
                data = movimento.titulo_id.data_prorrogacao
                if context.get('wizard_gerar_parcelas'):
                    vals.update({'fingmo_data': data})
            else:
                data = movimento.titulo_id.data_emissao
            mes_competencia = data[5:7]
            ano_competencia = data[0:4]
            vals.update({'mes_competencia': mes_competencia, 'ano_competencia': ano_competencia})
            res = self.write(cr, uid, movimento.id, vals, context)
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not isinstance(args, list):
            args = []
        context = context and context.copy() or {}
        titulos = filter(lambda i: i[0] == 'titulo_id', args)
        list_titulo_ids = []
        for i in titulos:
            operador = i[1]
            titulo_ids = i[2]
            if operador == '=':
                list_titulo_ids.append(titulo_ids)
            elif operador == 'in':
                list_titulo_ids.extend(titulo_ids)
        res = super(gerencial_movimento, self).search(cr, uid, args, offset=offset, limit=limit,
                                                      order=order,
                                                      context=context, count=count)
        if not args or count:
            return res
        gerencial_movimento_ids = []
        if len(res) == 1:
            gerencial_movimento_ids = str(tuple(res)).replace(',', '')
        elif len(res) > 1:
            gerencial_movimento_ids = str(tuple(res))
        if context.get('group_by') == 'centro_custo_id' and gerencial_movimento_ids:
            sql = """
                SELECT gm.id
                FROM gerencial_movimento gm
                INNER JOIN adm_empresa_centrocusto cc
                    ON cc.id = gm.centro_custo_id
                WHERE
                    gm.id in %s
                ORDER BY
                    cc.codigo
            """ % gerencial_movimento_ids
            cr.execute(sql)
            gerencial_movimento_ids = cr.dictfetchall()
            gerencial_movimento_ids = map(lambda i: i.get('id'), gerencial_movimento_ids)
            return gerencial_movimento_ids
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if not vals.get('fingmo_data') and vals.get('tesouraria_movimento_id'):
            vals['fingmo_data'] = self.pool.get('financeiro.tesouraria.movimento').browse(
                cr, uid, vals.get('tesouraria_movimento_id'), context).data_movimento
        res = super(gerencial_movimento, self).create(cr, uid, vals, context)
        if not 'mes_competencia' in vals:
            vals['mes_competencia'] = self.fingmo_get_mes_ano(cr, uid, [res], 'mes_competencia',
                                                              context)
        if not 'ano_competencia' in vals:
            vals['ano_competencia'] = self.fingmo_get_mes_ano(cr, uid, [res], 'ano_competencia',
                                                              context)
        self.write(cr, uid, res, vals)
        return res

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(gerencial_movimento, self).write(cr, user, ids, vals, context)
        return res

    def fingmo_get_mes_ano(self, cr, uid, ids, name, context=None):
        context = context and context.copy() or {}
        id = ids[0]
        res = ''
        brw_dados = self.browse(cr, uid, id, context=context)
        tesouraria_movimento_id = brw_dados.tesouraria_movimento_id
        titulo_id = brw_dados.titulo_id
        ecf_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'ecf',
                                                                                 context)
        contrato_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'contratos', context)

        if context.get('classificar_vendas_data'):
            data = context['classificar_vendas_data']
            if name == 'mes_competencia':
                res = str(data[5:7])
            elif name == 'ano_competencia':
                res = str(data[0:4])
        elif tesouraria_movimento_id:
            if name == 'mes_competencia':
                res = str(tesouraria_movimento_id.mes_competencia)
            elif name == 'ano_competencia':
                res = str(tesouraria_movimento_id.ano_competencia)
        elif titulo_id:
            fatura_id = titulo_id.fatura_id
            if fatura_id.nf_id:
                data = fatura_id.nf_id.data_emissao
            elif contrato_instalado and titulo_id.contrato_id:
                data = titulo_id.data_prorrogacao
            else:
                data = titulo_id.data_emissao
            if name == 'mes_competencia':
                res = str(data[5:7])
            elif name == 'ano_competencia':
                res = str(data[0:4])
        elif ecf_instalado:
            data = brw_dados.fingmo_cupom_id.date_cupom
            if data:
                if name == 'mes_competencia':
                    res = str(data[5:7])
                elif name == 'ano_competencia':
                    res = str(data[0:4])
        elif brw_dados.fingmo_nf_id:
            data = brw_dados.fingmo_nf_id.data_emissao
            if name == 'mes_competencia':
                res = str(data[5:7])
            elif name == 'ano_competencia':
                res = str(data[0:4])

        return res.zfill(2) if res else ''

    def agrupar_gerencial_centro_custo(self, cr, uid, centro_custo_evento, documento, context=None):
        """
        Agrupa centro de custo e eventos.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param centro_custo_evento: dados do centro de custo e evento a serem agrupados
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type centro_custo_evento: list(dict)
        :type context: dict
        :return: lista com os valores do movimento no Gerencial
        :rtype: list
        """
        context = context and context.copy() or {}
        if not centro_custo_evento:
            return list()

        res = []
        # Grupos de centro de custo e evento
        for centro_custo_evento, vals in groupby(
                centro_custo_evento, key=lambda r: (r.get('centro_custo_id', 0),
                                                    r.get('evento_id', 0),
                                                    r.get('fingmo_historico'))):
            valor = sum(map(lambda i: i.get('valor', 0), vals))
            vals[0]['valor'] = valor
            if not valor:
                continue

            # Cria movimento
            res.append(vals[0])
            # para nota cria o movimento apenas depois de faturada, por causa da comunicacao com o
            # SEFAZ (Felipe)
            if documento.get('tipo_documento') == 'N':
                continue

            self.create(cr, uid, vals[0], context=context)

        return res

    def classificar_valor_fiscal(self, company_id, centro_custo_venda_id, documento, item,
                                 historico, vals, parametros, context=None):
        context = context and context.copy() or {}
        operacao = documento.get('tipo_nf') == 'E' and 'S' or 'E'
        values = {'centro_custo_id': centro_custo_venda_id, 'company_id': company_id,
                  'pessoa_id': documento.get('pessoa_id'), 'fingmo_operacao': operacao,
                  'ano_competencia': documento.get('data')[:4],
                  'mes_competencia': documento.get('data')[5:7],
                  'fingmo_data': documento.get('data')}
        if documento.get('tipo_documento') == 'C':
            values['fingmo_cupom_id'] = documento.get('id')
        else:
            values['fingmo_nf_id'] = documento.get('id')

        doc_tipo = documento.get('tipo_nf')
        # Seguro
        compra_venda = doc_tipo == 'E' and 'compra' or 'venda'
        seguro_evento_id = doc_tipo == 'E' and \
                           parametros.get('finpar_evento_compra_seguro_id') or \
                           parametros.get('finpar_evento_venda_seguro_id')
        if item.get('vr_seguro_rateado'):
            if not seguro_evento_id:
                raise osv.except_osv(_('Atencao!'),
                                   _('Sem evento para Seguro na %s no parametro do financeiro') %
                                     compra_venda)

            vals.append(dict(values,
                             valor=item.get('vr_seguro_rateado'),
                             evento_id=seguro_evento_id,
                             fingmo_historico='%s - Seguro' % historico))
        # Despesas acessorias
        despesas_evento_id = doc_tipo == 'E' and \
                             parametros.get('finpar_evento_compra_desp_acessoria_id') or \
                             parametros.get('finpar_evento_venda_desp_acessoria_id')
        if item.get('vr_despesas_rateado') or item.get('vr_acrescimos'):
            if not despesas_evento_id:
                raise osv.except_osv(_('Atencao!'),
                                   _('Sem evento para Despesas na %s no parametro do financeiro') %
                                     compra_venda)

            vals.append(dict(values,
                             valor=(item.get('vr_despesas_rateado') or 0) +
                                   (item.get('vr_acrescimos') or 0),
                             evento_id=despesas_evento_id,
                             fingmo_historico='%s - Despesa Acessoria' % historico))
        # Desconto
        desconto_evento_id = doc_tipo == 'E' and \
                             parametros.get('finpar_desconto_compra_evento_id') or \
                             parametros.get('finpar_evento_venda_desconto_id')
        if item.get('vr_desconto_rateado') or item.get('vr_desconto'):
            if not desconto_evento_id:
                raise osv.except_osv(_('Atencao!'),
                                   _('Sem evento para Desconto na %s no parametro do financeiro') %
                                     compra_venda)

            vals.append(dict(values,
                             valor=(item.get('vr_desconto') or 0) +
                                   (item.get('vr_desconto_rateado') or 0),
                             fingmo_operacao=documento.get('tipo_nf') == 'E' and 'E' or 'S',
                             evento_id=desconto_evento_id,
                             fingmo_historico='%s - Desconto' % historico))
        # Frete
        frete_evento_id = doc_tipo == 'E' and \
                             parametros.get('finpar_evento_compra_frete_id') or \
                             parametros.get('finpar_evento_venda_frete_id')
        # tipo de frete deve ser diferente de por fora('F')
        if item.get('vr_frete_rateado') and documento.get('tipo_frete') != 'F':
            if not frete_evento_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('Sem evento para Frete na %s no parametro do financeiro') %
                                       compra_venda)

            vals.append(dict(values,
                             valor=item.get('vr_frete_rateado'),
                             evento_id=frete_evento_id,
                             fingmo_historico='%s - Frete' % historico))
        return True

    def get_centro_evento_categoria(self, cr, uid, vals, categoria_ids, context=None):
        """
        Pega o centro de custo e envento da categoria, se nao tiver, verifica se tem na
        categoria pai e assim por diante de forma recursiva.
        :param cr:
        :param uid:
        :param categoria_ids: ids da categoria
        :param context:
        :return: dict
        """
        context = context and context.copy() or {}
        dados_categorias = self.pool.get('estoque.categoria').read(cr, uid, categoria_ids,
                      ['parent_id', 'estcat_evento_cmv_id', 'estcat_centro_custo_id'],
                                                             context=context, load='_classic_write')
        for categoria in dados_categorias:
            categ_id = categoria.get('id')
            centro_id = categoria.get('estcat_centro_custo_id')
            evento_id = categoria.get('estcat_evento_cmv_id')
            parent_id = categoria.get('parent_id')
            if not vals.get(categ_id):
                vals[categ_id] = {'centro_custo_id': centro_id,
                                  'evento_id': evento_id,
                                  'parent_id': parent_id}
                if centro_id and evento_id or not parent_id:
                    continue

            if not (vals.get(categ_id).get('centro_custo_id') and
                    vals.get(categ_id).get('evento_id')) and parent_id:
                self.get_centro_evento_categoria(cr, uid, vals, [parent_id], context)

            if not vals.get(categ_id).get('centro_custo_id') and parent_id:
                vals[categ_id]['centro_custo_id'] = vals[parent_id]['centro_custo_id']

            if not vals.get(categ_id).get('evento_id') and parent_id:
                vals[categ_id]['evento_id'] = vals[parent_id]['evento_id']

        return vals

    def custificar_documento(self, cr, uid, documento, par_centro_custo_id,
                             nat_centro_custo_id, vals, evento_id,
                             company_id, historico, vals_gerencial_movimento, dict_categoria,
                             dict_local_cc, context=None):
        """
        Custifica a venda do cupom
        :param cr: 
        :param uid: 
        :param context: 
        :return: 
        """
        context = context and context.copy() or {}
        valor_total = documento.get('vr_documento')
        if not valor_total:
            return True

        if not evento_id:
            msg = _('Evento nao cadastrado na natureza da nota fiscal, ou no parametro!')
            if documento.get('tipo_documento') == 'C': # mensagem para cupom
                msg = _('Evento nao cadastrado na natureza de venda no parametro do fiscal!')

            raise osv.except_osv(_('Atencao!'), msg)

        operacao = documento.get('tipo_nf') == 'E' and 'S' or 'E'
        vals_gerencial_movimento.update(
            {'valor': valor_total, 'evento_id': evento_id,
             'centro_custo_id': nat_centro_custo_id or par_centro_custo_id,
             'company_id': company_id, 'pessoa_id': documento.get('pessoa_id'),
             'fingmo_historico': historico, 'fingmo_operacao': operacao,
             'mes_competencia': documento.get('data')[5:7],
             'ano_competencia': documento.get('data')[:4], 'fingmo_data': documento.get('data')})
        if documento.get('tipo_documento') == 'C':
            vals_gerencial_movimento['fingmo_cupom_id'] = documento.get('id')
        else:
            vals_gerencial_movimento['fingmo_nf_id'] = documento.get('id')

        if nat_centro_custo_id:
            vals.append(vals_gerencial_movimento)
        elif dict_categoria:
            for i in dict_categoria.values():
                vals_gerencial = vals_gerencial_movimento.copy()
                vals_gerencial.update({'valor': i.get('valor'),
                                       'centro_custo_id': i.get('centro_custo_id')})
                vals.append(vals_gerencial.copy())
        elif dict_local_cc:
            for centro_custo_id, valor in dict_local_cc.items():
                vals_gerencial = vals_gerencial_movimento.copy()
                vals_gerencial.update({'valor': valor,
                                       'centro_custo_id': centro_custo_id})
                vals.append(vals_gerencial.copy())
        else:
            vals.append(vals_gerencial_movimento)

    def custificar_frete_por_fora(self, cr, uid, documento, centro_custo_id, vals, parametros,
                                  company_id, context=None):
        """
        Custifica a nota de frete, quando o frete for por Fora
        :param cr: 
        :param uid: 
        :param context: 
        :return: 
        """
        context = context and context.copy() or {}
        valor_total = documento.get('vr_documento')
        if not valor_total or documento.get('tipo_frete') != 'F' or \
                not documento.get('nf_frete_ids'):
            return True

        evento_id = parametros.get('finpar_evento_compra_frete_id')
        if not evento_id:
            msg = _('Evento de Frete nao cadastrado no parametro do financeiro para compra!')
            raise osv.except_osv(_('Atencao!'), msg)

        operacao = documento.get('tipo_nf') == 'E' and 'S' or 'E'
        campos = ['destinatario_id', 'serie_nf', 'numero_nf', 'vr_total_nf', 'data_emissao']
        for nf_frete in self.pool.get('fiscal.nf.capa').read(cr, uid, documento.get('nf_frete_ids'),
                                                    campos, context=context, load='_classic_write'):
            data = nf_frete.get('data_emissao')[:10]
            historico = 'Nota Fiscal de Frete %s-%s' % (nf_frete.get('serie_nf'), 
                                                        nf_frete.get('numero_nf'))
            vals_gerencial_movimento_frete = {
                'valor': nf_frete.get('vr_total_nf') or 0,
                'evento_id': evento_id,
                'centro_custo_id': centro_custo_id,
                'company_id': company_id,
                'pessoa_id': nf_frete.get('destinatario_id') or False,
                'fingmo_historico': historico,
                'fingmo_operacao': operacao,
                'mes_competencia': data[5:7],
                'ano_competencia': data[:4], 'fingmo_data': data,
                'fingmo_nf_id': nf_frete.get('id')}
            vals.append(vals_gerencial_movimento_frete)

    def custificar_guias(self, cr, uid, vals_gerencial_movimento, documento, vals, parametros,
                         context=None):
        """Custifica as guias das notas de entrada
        """
        context = context and context.copy() or {}
        if not documento.get('guia_ids'):
            return

        dados_guia = self.pool.get('fiscal.guia').read(cr, uid, documento.get('guia_ids'),
                                                  ['valor'], context=context, load='_classic_write')
        total_guias = sum(map(lambda x: x.get('valor') or 0, dados_guia))
        if not total_guias:
            return

        if not parametros.get('finpar_gnre_evento_id'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Evento para GNRE em compras nao cadastrado no parametro do '
                                   'financeiro'))
        vals_gia = vals_gerencial_movimento.copy()
        vals_gia['evento_id'] = parametros.get('finpar_gnre_evento_id')
        vals_gia['valor'] = total_guias
        vals_gia['fingmo_operacao'] = 'S'
        vals.append(vals_gia)

    def custificar_taxa_cartao(self, cr, uid, titulo_cartao, vals_gerencial_movimento, documento,
                                     vals, parametros, historico, context=None):
        context = context and context.copy() or {}

        # Cria custificacao para taxa do cartao
        if not titulo_cartao or documento.get('tipo_nf') == 'E':
            return True

        evento_cartao_id = parametros.get('finpar_evento_venda_taxa_adm_id')
        if not evento_cartao_id:
            raise osv.except_osv(_('Atencao!'),
                  _('Evento para taxa de administracao nao cadastrado no parametro do financeiro!'))

        dados_titulo = self.pool.get('financeiro.titulo').read(cr, uid, titulo_cartao,
                                                             context=context, load='_classic_write')

        for titulo in dados_titulo:
            vals_gerencial_movimento_cartao = vals_gerencial_movimento.copy()
            vals_gerencial_movimento_cartao.update(
                {'valor': titulo.get('vr_original'), 'evento_id': evento_cartao_id,
                 'fingmo_operacao': 'S', 'pessoa_id': titulo.get('pessoa_id'),
                 'fingmo_historico': '%s - Taxa Cartao' % historico,
                 'fingmo_data': titulo.get('data_emissao')})
            vals.append(vals_gerencial_movimento_cartao)

    def custificar_venda_tributacao(self, cr, uid, item, documento, vals_impostos, parametros,
                                    centro_custo_id, gm_company_id, historico, context=None):
        context = context and context.copy() or {}
        # para notas de entrada nao custifica a tributacao
        if documento.get('tipo_nf') == 'E':
            return

        for trib in item.get('tributacoes_ids'):
            if not trib.get('vr_encargo') or trib.get('cst_id').get('tipo') not in ['U','TU','T']:
                continue

            msg = (_('Nao existe Evento para tributacao cadastrado no encargo %s!')
                                     % trib.get('encargo_id').get('name'))
            valor  = trib.get('vr_encargo')
            evento_id = trib.get('encargo_id').get('fisenc_evento_id')
            if trib.get('cst_id').get('tipo') in ['U', 'TU']:
                valor  = trib.get('vr_icms_substituicao')
                evento_id = parametros.get('finpar_subst_evento_id')
                msg = _('Nao existe Evento Icms Subst. Tributaria em vendas no parametro do'
                        'financeiro!')

            if not valor:
                continue

            if not evento_id: # Não é obrigatoria a custificação dos encargos
                # raise osv.except_osv(_('Atencao!'), msg)
                continue

            vals = {'valor': valor, 'evento_id': evento_id, 'centro_custo_id': centro_custo_id,
                    'company_id': gm_company_id, 'pessoa_id': documento.get('pessoa_id'),
                    'fingmo_historico': '%s - Impostos: %s' % (historico,
                                                             trib.get('encargo_id').get('name')),
                    'fingmo_operacao': 'S', 'ano_competencia': documento.get('data')[:4],
                    'mes_competencia': documento.get('data')[5:7],
                    'fingmo_data': documento.get('data')}
            if documento.get('tipo_documento') == 'C':
                vals['fingmo_cupom_id'] = documento.get('id')
            else:
                vals['fingmo_nf_id'] = documento.get('id')

            vals_impostos.append(vals)

    def custificar_venda_cmv(self, cr, uid, item, documento, vals_cmv, custos_cmv,
                             centro_custo_id, gm_company_id, tipo_custo_cmv_parametro, historico,
                             dict_categoria, context=None):
        context = context and context.copy() or {}
        # CMV por nota/cupom apenas para venda
        if documento.get('tipo_nf') == 'E' or context.get('fisnfc_nao_gerar_cmv'):
            return

        # CMV - Deve gerar CMV apenas se tiver cadastrado evento na categoria
        valor_custo = 0
        produto_id = item.get('produto_id').get('id')
        custo_cmv = custos_cmv.get(produto_id)
        if not custo_cmv:
            return

        if tipo_custo_cmv_parametro == 1:
            valor_custo = custo_cmv.get('estprc_ultimo_custo')
        elif tipo_custo_cmv_parametro == 2:
            valor_custo = custo_cmv.get('estprc_custo_medio_contabil')
        elif tipo_custo_cmv_parametro == 3:
            valor_custo = custo_cmv.get('estprc_custo_corrigido')


        # gera CMV apenas se houver custo
        if not valor_custo:
            return

        categoria_id = item.get('produto_id').get('categoria_id')
        if not categoria_id:
            raise osv.except_osv(_('Atencao!'),
                      _('Nao existe Categoria cadastrada no produto %s-%s para custificar o CMV') %
                                     (item.get('produto_id').get('codigo'),
                                      item.get('produto_id').get('name')))

        evento_id = dict_categoria.get(categoria_id).get('evento_id')
        if not evento_id:
            raise osv.except_osv(
        _('Atencao!'), _('Nao existe Evento CMV cadastrado na categoria do produto %s-%s')
                           % (item.get('produto_id').get('codigo'),
                              item.get('produto_id').get('name')))

        vals = {'produto_id': produto_id, 'valor': item.get('quantidade') * valor_custo,
                'evento_id': evento_id, 'centro_custo_id': centro_custo_id,
                'company_id': gm_company_id, 'pessoa_id': documento.get('pessoa_id'),
                'fingmo_historico': '%s - Classificacao CMV' % historico, 'fingmo_operacao': 'S',
                'ano_competencia': documento.get('data')[:4],
                'mes_competencia': documento.get('data')[5:7], 'fingmo_data': documento.get('data')}
        if documento.get('tipo_documento') == 'C':
            vals['fingmo_cupom_id'] = documento.get('id')
        else:
            vals['fingmo_nf_id'] = documento.get('id')

        vals_cmv.append(vals)

    def custificar_venda_comissao(self, cr, uid, item, documento, vals_vendedor, centro_custo_id,
                                  gm_company_id, historico, context=None):
        context = context and context.copy() or {}
        # Comissao vendedor
        if not item.get('estmvi_vr_comissao') or not item.get('vendedor_id') or \
                not item.get('produto_id'):
            return

        evento_vendedor_id = item.get('vendedor_id').get('venven_evento_id') or False
        if not evento_vendedor_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao existe Evento cadastrado para o vendedor %s ') %
                                     item.get('vendedor_id').get('pessoa_id').get('nome'))

        vr_comissao = item.get('estmvi_vr_comissao')
        vals = {'valor': vr_comissao, 'evento_id': evento_vendedor_id,
                'centro_custo_id': centro_custo_id, 'company_id': gm_company_id,
                'pessoa_id': item.get('vendedor_id').get('pessoa_id').get('id'),
                'fingmo_historico': '%s - Comissao Venda Vendedor' % historico,
                'fingmo_operacao': 'S', 'ano_competencia': documento.get('data')[:4],
                'mes_competencia': documento.get('data')[5:7], 'fingmo_data': documento.get('data')}
        if documento.get('tipo_documento') == 'C':
            vals['fingmo_cupom_id'] = documento.get('id')
        else:
            vals['fingmo_nf_id'] = documento.get('id')

        vals_vendedor.append(vals)


    @log_debug
    def fingmo_valida_usar_centro_custo_evento(self, cr, uid, context=None):
        """
        Verifica e valida o centro de custo e eventos
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                           'financeiro.parametros', context=context)
        parametros = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                                          'financeiro.parametros', ['usar_centro_custos_e_eventos'],
                                                             company_id=company_id, context=context)
        if not parametros:
            raise osv.except_osv(_('Atencao!'),
                        _('Nenhum registro encontrado no parametro do financeiro para a empresa!')
            )

        return parametros.get('usar_centro_custos_e_eventos')

    def custificar_nota_fiscal(self, cr, uid, brw_nf, context=None):
        """
        Realiza a custificacao da Nota de entrada ou saida.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param nf_id: browse da nota
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type nf_id: browse object
        :type context: dict
        :return: dados da custificacao da venda efetuada.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        # cupom(cfop=5929/6929) e tipo de movimento do cfop igual a Transferencia,
        # Remessa ou Retorno nao gera custificacao
        if not(self.fingmo_valida_usar_centro_custo_evento(cr, uid, context) and \
           brw_nf.natureza_id.cfop_id.movimento_tipo not in ['E', 'R'] and \
           brw_nf.natureza_id.cfop_id.codigo not in ['5929','6929']):
            return []

        # Notas de frete (nao contem itens) nao geram custificacao
        # FIX: Nao permitir custificar vendas se houver contrato (heranca do modulo contratos)
        if not brw_nf.estoque_movimento_itens_ids and brw_nf.modelo_nf_id.fismnf_codigo != u'57':
            return []

        context.update({'classificar_vendas_data': brw_nf.data_emissao})
        company_id = brw_nf.company_id.id
        # Nota Fiscal de saida de devolucao/transferencia nao deve gerar gerencial referente
        # ao CMV.
        if brw_nf.natureza_id.cfop_id.movimento_tipo in ['D', 'T']:
            context.update({'fisnfc_nao_gerar_cmv': True})

        context['sig_search_sem_filhos'] = True
        fatura_ids = self.pool.get('financeiro.fatura').search(cr, uid, [('nf_id', '=', brw_nf.id)],
                                                               context=context)
        # Titulos da taxa de administracao
        pagar_titulos_ids = self.pool.get('financeiro.titulo').search(cr, uid,
                      [('fatura_id', 'in', fatura_ids), ('tipo_titulo', '=', 'P')], context=context)
        if context.get('gerencial_nao_custificar'):
            return []
        return self.custificar(cr, uid, [], False, brw_nf.id, pagar_titulos_ids, company_id,
                               context=context)

    def custificar_vendas_cupom(self, cr, uid, itens, cupom_id, titulo_cartao,  empresa,
                                context=None):
        context = context and context.copy() or {}
        self.custificar(cr, uid, itens, cupom_id, False, titulo_cartao,  empresa, context=context)
        return True

    def custificar(self, cr, uid, itens, cupom_id, nf_id, titulo_cartao,  empresa, context=None):
        """ **********************************
        Realiza a custificacao da venda/compra de nota/cupom de acordo com o chamado 56085. 
            ********************************** """
        context = context and context.copy() or {}
        context['sig_search_sem_filhos'] = True
        obj_adm_utils = self.pool.get('adm.utils')
        user_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context,
                                                                         user_company_id=empresa)
        list_parametros = ['finpar_centro_custo_venda_id', 'finpar_evento_venda_id',
                           'finpar_evento_venda_taxa_adm_id', 'finpar_evento_venda_seguro_id',
                           'finpar_evento_venda_desp_acessoria_id', 'finpar_evento_venda_frete_id',
                           'finpar_evento_venda_desconto_id', 'finpar_subst_evento_id',
                           # parametros de compra
                           'finpar_evento_compra_frete_id','finpar_evento_compra_desp_acessoria_id',
                           'finpar_evento_compra_contrato_id', 'finpar_desconto_compra_evento_id',
                           'finpar_evento_compra_seguro_id', 'finpar_evento_venda_contrato_id',
                           'finpar_gnre_evento_id', 'finpar_desconto_compra_evento_id']
        parametros = obj_adm_utils.admutl_get_parametros(cr, uid,'financeiro.parametros',
                                       list_parametros, company_id=user_company_id, context=context)
        if not parametros:
            raise osv.except_osv(_('Atencao!'), _('Parametro do Financeiro nao cadastrado.'))


        parametro_fiscal = obj_adm_utils.admutl_get_parametros(cr, uid, 'fiscal.parametro',
                                                             ['fispar_natureza_uf_dentro_venda_id'],
                                                        company_id=user_company_id, context=context)
        if not parametro_fiscal and cupom_id:
            raise osv.except_osv(_('Atencao!'), _('Parametro do Fiscal nao cadastrado.'))

        natureza_id = parametro_fiscal.get('fispar_natureza_uf_dentro_venda_id')
        if not natureza_id and cupom_id:
            raise osv.except_osv(_('Atencao!'),
               _('Natureza de vendas dentro do estado nao esta cadastrada no parametro do fiscal!'))


        dados_cupom = {}
        if cupom_id:
            campos = ['coo', 'cliente_id','date_cupom', 'ecc_vendedor_id', 'ecf_cupons_item_ids']
            dados_cupom = self.pool.get('ecf.cupons.capa').read(cr, uid, cupom_id, campos,
                                                             context=context, load='_classic_write')
            total_bruto = 0
            for i in self.pool.get('ecf.cupons.item').read(cr, uid,
                      dados_cupom.get('ecf_cupons_item_ids'), ['preco_total', 'venda_cancelamento'],
                                                           context):
                if i.get('venda_cancelamento') == 'C':
                    continue

                total_bruto += i.get('preco_total')

            # valor de desconto e acrescimo nao devem entrar na custificacao do cupom, ha uma
            # custificacao separada para cada um
            dados_cupom['total_cupom'] = total_bruto

        historico = 'Cupom fiscal %s' % dados_cupom.get('coo')
        dados_nf = {}
        if nf_id:
            campos = ['estoque_movimento_itens_ids', 'serie_nf', 'numero_nf', 'data_emissao',
                      'destinatario_id', 'natureza_id', 'tipo_frete', 'tipo', 'guia_ids',
                      'data_saida_entrada',  'fisnfc_nf_frete_ids', 'vr_total_nf']
            dados_nf = self.pool.get('fiscal.nf.capa').read(cr, uid, nf_id, campos, context=context,
                                                            load='_classic_write')
            # valor de desconto, frete, seguro e despesas acessorias  nao devem entrar na
            # custificacao da nota, ha uma custificacao separada para cada um
            total_bruto = 0
            context_it = context.copy()
            context_it['super_read_estoque_movimento_itens'] = False
            for i in self.pool.get('estoque.movimento.itens').read(cr, uid,
                                           dados_nf.get('estoque_movimento_itens_ids'),
                                                      ['vr_total_item', 'vr_desconto'], context_it):
                total_bruto += i.get('vr_total_item') + (i.get('vr_desconto') or 0)

            # calcula o valor pelos itens, no caso de CTE nao calcula pois nao tem itens
            if total_bruto:
                dados_nf['vr_total_nf'] = total_bruto

            natureza_id = dados_nf.get('natureza_id')
            itens = dados_nf.get('estoque_movimento_itens_ids')
            historico = 'Nota Fiscal %s-%s' % (dados_nf.get('serie_nf'), dados_nf.get('numero_nf'))

        cfop_natureza = self.pool.get('fiscal.cfop.natureza').read(cr, uid, natureza_id,
                                            ['fna_evento_id', 'fisnat_centro_custo_id', 'tipo'],
                                                         context=context, load='_classic_write')
        # Parametro do centro de custo da venda
        nat_centro_custo_id = cfop_natureza.get('fisnat_centro_custo_id')
        par_centro_custo_id = parametros.get('finpar_centro_custo_venda_id')

        classe = 'estoque.movimento.itens'
        campos = {'item_cancelado': [],
                  'quantidade': [],
                  'vr_desconto': [],
                  'vr_seguro_rateado': [],
                  'vr_despesas_rateado': [],
                  'vr_desconto_rateado': [],
                  'vr_total_item_rateado': [],
                  'vr_frete_rateado': [],
                  'estmvi_vr_comissao': [],
                  'local_id': ['centrocusto_id'],
                  'produto_id': ['categoria_id', 'codigo', 'name'],
                  'vendedor_id': {'venven_evento_id': [],
                                  'pessoa_id': ['nome']},
                  'tributacoes_ids': {'vr_encargo': [],
                                      'cst_id': ['tipo'],
                                      'encargo_id': ['fisenc_evento_id', 'name', 'sigla'],
                                      'vr_icms_substituicao': [],
                                      },
                  }
        dados_itens = obj_adm_utils.admutl_read_browse(cr, uid, itens, classe, campos, context)
        produto_ids = []
        dict_categoria = {}
        dict_local_cc = {}  # centro de custo dos locais
        for i in dados_itens:
            produto_ids.append(i.get('produto_id').get('id'))
            categoria = i.get('produto_id').get('categoria_id')
            if categoria:
                if categoria not in dict_categoria:
                    dict_categoria[categoria] = {'valor': i.get('vr_total_item_rateado') or 0}
                else:
                    dict_categoria[categoria]['valor'] += i.get('vr_total_item_rateado') or 0

            local_cc_id = i.get('local_id').get('centrocusto_id')
            if local_cc_id:
                if local_cc_id not in dict_local_cc:
                    dict_local_cc[local_cc_id] = i.get('vr_total_item_rateado') or 0
                else:
                    dict_local_cc[local_cc_id] += i.get('vr_total_item_rateado') or 0

        centro_evento_categ = self.get_centro_evento_categoria(cr, uid, {}, dict_categoria.keys(),
                                                               context)
        tem_centro_categoria = True
        for i in dict_categoria:
            dict_categoria[i].update(centro_evento_categ.get(i).copy())
            if not centro_evento_categ.get(i).get('centro_custo_id'):
                tem_centro_categoria = False

        del centro_evento_categ
        tem_centro_local = True
        for i in dict_local_cc:
            if not dict_local_cc.get(i):
                tem_centro_local = False
                break

        if not tem_centro_local:
            dict_local_cc = {}

        if not (par_centro_custo_id or nat_centro_custo_id):
            raise osv.except_osv(_('Atencao!'),
                     _('Centro de custo para venda nao cadastrado no parametro do financeiro/Natureza!'))

        if not (tem_centro_categoria or tem_centro_local):
            raise osv.except_osv(_('Atencao!'),
                     _('Centro de custo para venda nao cadastrado na categoria/Local!'))
        vals_gerencial_movimento = {}
        documento = {'id': cupom_id or nf_id,
                     'tipo_documento': cupom_id and 'C' or 'N', # Cupom ou nota ('C' ou 'N')
                     'tipo_nf': dados_nf and dados_nf.get('tipo') or False,
                     'tipo_frete': dados_nf and dados_nf.get('tipo_frete') or False,
                     'guia_ids': dados_nf and dados_nf.get('guia_ids') or False,
                     'pessoa_id': dados_cupom.get('cliente_id') or dados_nf.get('destinatario_id')
                                  or False,
                     'vendedor_id': dados_cupom.get('ecc_vendedor_id') or
                                    dados_nf.get('vendedor_id'),
                     'data': dados_cupom.get('date_cupom') or dados_nf.get('data_emissao')[:10],
                     'nf_frete_ids': dados_nf.get('fisnfc_nf_frete_ids') or [],
                     'vr_documento': dados_cupom.get('total_cupom') or dados_nf.get('vr_total_nf')
                                     or 0,}
        if dados_nf and dados_nf.get('tipo') == 'E':
            documento['data'] = dados_nf.get('data_saida_entrada')[:10]
        vals = []
        self.custificar_documento(cr, uid, documento, par_centro_custo_id, nat_centro_custo_id,
                                  vals,
                                  cfop_natureza.get('fna_evento_id') or parametros.get('finpar_evento_venda_id'),
                                  empresa, historico,
                                  vals_gerencial_movimento,
                                  tem_centro_categoria and dict_categoria or {},
                                  dict_local_cc,
                                  context=context)
        self.custificar_frete_por_fora(cr, uid, documento,
                                       nat_centro_custo_id or par_centro_custo_id, vals,
                                       parametros, empresa, context=context)
        self.custificar_guias(cr, uid, vals_gerencial_movimento, documento, vals, parametros,
                              context=context)

        # Custifica taxa de cartao se houver pagamento com cartao contendo taxa de administracao
        self.custificar_taxa_cartao(cr, uid, titulo_cartao, vals_gerencial_movimento, documento,
                                    vals, parametros, historico, context=context)

        # Classificação de impostos tributados, CMV e comissão de vendedor
        parametro_estoque = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque.parametro', ['estpar_custo_cmv'], company_id=user_company_id,
            context=context)
        if not parametro_estoque:
            raise osv.except_osv(_('Atencao!'), _('Parametro do Estoque nao cadastrado.'))

        tipo_custo_cmv_parametro = parametro_estoque.get('estpar_custo_cmv')
        epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.preco.custo', context=context, user_company_id=empresa)

        gm_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'gerencial.movimento', context=context, user_company_id=empresa)
        custos_cmv = self.pool.get('estoque.preco.custo').get_custos_do_dia(cr, uid, produto_ids,
                                             epc_company_id=epc_company_id,
                                             epc_data='%s 23:59:59' % documento.get('data'),
                                             context=context)
        capa_vendedor = False
        if documento.get('vendedor_id'):
            campos = {'venven_evento_id': [],
                      'pessoa_id': ['nome']}
            capa_vendedor = obj_adm_utils.admutl_read_browse(cr, uid, documento.get('vendedor_id'),
                                                         'vendas.vendedor', campos, context=context)

        centro_custo_id = nat_centro_custo_id or par_centro_custo_id
        for item in dados_itens:
            # Somente Tributados
            if item.get('item_cancelado'):
                continue

            self.custificar_venda_tributacao(cr, uid, item, documento, vals, parametros,
                                         centro_custo_id, gm_company_id, historico, context=context)
            self.custificar_venda_cmv(cr, uid, item, documento, vals, custos_cmv,
                                      centro_custo_id,gm_company_id, tipo_custo_cmv_parametro,
                                      historico, dict_categoria, context=context)
            # passa o vendedor da capa para o item caso o item nao possua vendedor
            if not item.get('vendedor_id') and capa_vendedor:
                item['vendedor_id'] = capa_vendedor

            self.custificar_venda_comissao(cr, uid, item, documento, vals,
                                         centro_custo_id, gm_company_id, historico, context=context)

            self.classificar_valor_fiscal(gm_company_id, centro_custo_id, documento, item,
                                          historico, vals, parametros, context)

        # Agrupa e insere em gerencial movimento
        return self.agrupar_gerencial_centro_custo(cr, uid, vals, documento, context)

    def fingmo_get_valor_classificacao_origem(self, cr, uid, ids, context=None):
        """
        O metodo apura os valores das contas envolvidas no DRE. A apuracao e realizada no regime de
        caixa com classificacao de origem. Esse metodo e sobrescrito nos modulos fiscal e ecf onde
        realmente sao usados.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: valor total apurado das contas envolvidas no DRE.
        """
        pass
        
    def fingmo_get_gerencial_movimento_classificacao_origem_nivel_acima(self, cr, uid, ids,
                                                                        context=None):
        """
        O metodo busca o gerencial movimento dos titulos originados de notas fiscais e cupons.
        E usado para o regime de caixa com classificacao de origem. Esse metodo e sobrescrito nos
        modulos fiscal e ecf onde realmente sao usados.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: valor total apurado das contas envolvidas no DRE.
        """
        return []

    def fingmo_get_gerencial_movimento_classificacao_origem(self, cr, uid, ids, context=None):
        """
        O metodo busca o gerencial movimento original e os dos titulos de origem. E usado para o
        regime de fato gerador com classificacao de origem.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: registros da tabela desse objeto.
        :rtype: list
        """
        context = context and context.copy() or {}
        brw_gerencial_movimento = self.browse(cr, uid, ids, context)
        gerencial_movimento_ids = list()
        outra_conta_gerencial_movimento_ids = []
        for movimento in brw_gerencial_movimento:
            if not movimento.tesouraria_movimento_id:
                continue
            titulo_ids = movimento.tesouraria_movimento_id.fintem_titulo_ids
            if not titulo_ids:
                gerencial_movimento_ids.append(movimento.id)
                continue

            for titulo in titulo_ids:
                if titulo.fintit_gerencialmov_ids:
                    gerencialmov_ids = filter(
                        lambda i: i.evento_id.id == movimento.evento_id.id and
                                  i.centro_custo_id.id == movimento.centro_custo_id.id,
                        titulo.fintit_gerencialmov_ids)
                    gerencial_movimento_ids.extend(map(lambda i: i.id, gerencialmov_ids))

                    outra_conta_gerencialmov_ids = filter(
                        lambda i: i.evento_id.id != movimento.evento_id.id or
                                  i.centro_custo_id.id != movimento.centro_custo_id.id,
                        titulo.fintit_gerencialmov_ids)
                    outra_conta_gerencial_movimento_ids.extend(map(lambda i: i.id,
                                                                   outra_conta_gerencialmov_ids))
                else:
                    ger_mov_ids, outra_conta_ger_mov_ids = self.\
                        fingmo_get_gerencial_movimento_classificacao_origem_nivel_acima(
                            cr, uid, [movimento.id], context)
                    gerencial_movimento_ids.extend(ger_mov_ids)
                    outra_conta_gerencial_movimento_ids.extend(outra_conta_ger_mov_ids)
        return list(set(gerencial_movimento_ids)), outra_conta_gerencial_movimento_ids
