# -*- encoding: utf-8 -*-

__author__ = 'joao.almeida'

import time
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
import StringIO
from openerp import pooler
from openerp.report.interface import report_int
from openerp.report.render import render
from openerp.osv import fields, osv
from openerp.tools.translate import _
from SigUtils.formatastring import unicode_to_ascii as u2ascii


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class wizard_gerencial_analitico(osv.TransientModel):
    _name = 'wizard.gerencial.analitico'
    _description = 'Relatorio gerencial analitico.'

    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', required=True),
        'data_ini': fields.date('Data', required=True),
        'data_fim': fields.date('Data', required=True),
        'centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de Custo'),
        'evento_id': fields.many2one('gerencial.evento', 'Evento'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Pessoa'),
        'ordem': fields.selection([('C', 'Centro de Custo'), ('E', 'Evento')], 'Ordem', required=True),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'ordem': lambda *a: 'C'
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dict_wizard = self.read(cr, uid, ids, context=context)
        if isinstance(dict_wizard, list):
            dict_wizard = dict_wizard[0]

        params = {
            'company_id': dict_wizard['company_id'][0],
            'data_ini': dict_wizard['data_ini'],
            'data_fim': dict_wizard['data_fim'],
            'centro_custo_id': dict_wizard.get('centro_custo_id') and dict_wizard['centro_custo_id'][0],
            'evento_id': dict_wizard.get('evento_id') and dict_wizard['evento_id'][0],
            'pessoa_id': dict_wizard.get('pessoa_id') and dict_wizard['pessoa_id'][0],
            'produto_id': dict_wizard.get('produto_id') and dict_wizard['produto_id'][0],
            'ordem': dict_wizard['ordem']
        }

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'relatorio.gerencial.analitico',
            'datas': params
        }


class report_gerencial_analitico(report_int):

    def _get_conta_descricao_completa(self, cr, uid, evento_ids, centro_custo_ids, context=None):
        """
        O metodo encontra as contas pai das contas recebidas em evento_ids e centro_custo_ids. Monta
        um dicionario tendo como chave o id da conta e o valor da chave a descricao da conta
        incluindo a descricao das contas pai.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param evento_ids: lista de id da tabela gerencial_evento.
        :type evento_ids: list
        :param centro_custo_ids: lista de id da tabela adm_empresa_centrocusto.
        :type centro_custo_ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario tendo como chave o id da conta e o valor da chave a descricao da conta.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = dict()
        if evento_ids:
            brw_gerencial_evento = pooler.get_pool(cr.dbname).get('gerencial.evento').browse(
                cr, uid, evento_ids, context)
            res['evento'] = dict()
            for g in brw_gerencial_evento:
                name = '%s' % g.name
                conta_pai = g.parent_id
                while conta_pai:
                    name += ' / %s %s' % (conta_pai.codigo, conta_pai.name)
                    conta_pai = conta_pai.parent_id
                res['evento'][g.id] = name
        if centro_custo_ids:
            brw_adm_empresa_centro_custo = pooler.get_pool(cr.dbname).get(
                'adm.empresa.centrocusto').browse(cr, uid, centro_custo_ids, context)
            res['centro_custo'] = dict()
            for c in brw_adm_empresa_centro_custo:
                name = '%s' % c.name
                conta_pai = c.parent_id
                while conta_pai:
                    name += ' / %s %s' % (conta_pai.codigo, conta_pai.name)
                    conta_pai = conta_pai.parent_id
                res['centro_custo'][c.id] = name
        return res

    def create(self, cr, uid, ids, datas, context=None):
        def get_model(modulo):
            return pooler.get_pool(cr.dbname).get(modulo)

        def is_modulo_instalado(modulo):
            return get_model('adm.utils').admutl_check_modulo_instalado(cr, uid, modulo, context)

        context = context and context.copy() or {}
        obj_gerencial_movimento = pooler.get_pool(cr.dbname).get('gerencial.movimento')
        obj_adm_utils = pooler.get_pool(cr.dbname).get('adm.utils')
        pdf_file = StringIO.StringIO()
        company_id = datas['company_id']
        res_company_dados = pooler.get_pool(cr.dbname).get('res.company').read(
            cr, uid, company_id, ['name', 'parent_id'], context=context, load='_classic_write')
        company_name = res_company_dados.get('name') or ''

        printer = Impressora()
        printer.informaTamLinha(80)
        printer.defineMargem(0.5, 0.5, 0.5, 0.5)
        printer.defineFonte(tam=11.8)
        printer.informaNome(pdf_file, _('Gerencial Analitico'))
        title = _('Gerencial Analitico').center(80)[:-8] + _('Pag. ') + '{{ Page, 3 }}'
        printer.informaCab(title)
        printer.informaCab(' ')
        printer.informaCab(time.strftime('%d-%m-%Y') + (' ' * 13) + company_name)
        printer.informaCab(' ')
        printer.informaCab('--')
        table_header = ''
        table_header += _('Data').center(12)
        table_header += _('Competencia').center(11)
        table_header += _('Historico').center(42)
        table_header += _('Valor').center(15)
        printer.informaCab(table_header)
        printer.informaCab('--')
        printer.rodapePadrao()

        args = [('company_id', '=', datas.get('company_id')),
                ('fingmo_data', '>=', datas.get('data_ini')),
                ('fingmo_data', '<=', datas.get('data_fim'))]
        if datas.get('centro_custo_id'):
            cc_ids = pooler.get_pool(cr.dbname).get('adm.empresa.centrocusto').search(cr, uid,
                       [('parent_id', 'child_of', [datas.get('centro_custo_id')])], context=context)
            args.append(('centro_custo_id', 'in', cc_ids))

        if datas.get('evento_id'):
            ev_ids = pooler.get_pool(cr.dbname).get('gerencial.evento').search(cr, uid,
                             [('parent_id', 'child_of', [datas.get('evento_id')])], context=context)
            args.append(('evento_id', 'in', ev_ids))

        if datas.get('pessoa_id'):
            args.append(('pessoa_id', '=', datas.get('pessoa_id')))
        if datas.get('produto_id'):
            args.append(('produto_id', '=', datas.get('produto_id')))

        if res_company_dados.get('parent_id'):
            context.update({'sig_force_company': True})
        gerencial_movimento_ids = obj_gerencial_movimento.search(
            cr, uid, args, context=context)
        if 'sig_force_company' in context:
            del context['sig_force_company']
        res_data = obj_gerencial_movimento.read(
            cr, uid, gerencial_movimento_ids, ['company_id', 'centro_custo_id', 'evento_id',
                                               'fingmo_data', 'mes_competencia', 'ano_competencia',
                                               'fingmo_historico', 'valor'],
            context=context, load='_classic_write')

        if res_data:
            res_centro_custo = get_model('adm.empresa.centrocusto').read(cr, uid, list(set(map(lambda x: x['centro_custo_id'], res_data))), ['id', 'codigo', 'name'], context=context)
            res_evento = get_model('gerencial.evento').read(
                cr, uid, list(set(map(lambda x: x['evento_id'], res_data))),
                ['id', 'codigo', 'name'], context=context)

            conta_descricao_completa_dados = self._get_conta_descricao_completa(
                cr, uid, map(lambda i: i.get('id'), res_evento),
                map(lambda i: i.get('id'), res_centro_custo), context)

            if datas['ordem'] == 'C':
                groups = [res_centro_custo, res_evento]
                group_fields = ['centro_custo_id', 'evento_id']
                group_labels = [_('Centro custo'), _('Evento')]
            else:
                groups = [res_evento, res_centro_custo]
                group_fields = ['evento_id', 'centro_custo_id']
                group_labels = [_('Evento'), _('Centro de custo')]

            centro_custo_descricao_dados = conta_descricao_completa_dados.get('centro_custo')
            evento_descricao_dados = conta_descricao_completa_dados.get('evento')

            for g1 in sorted(groups[0], key=lambda x: x['codigo']):
                if datas.get('ordem') == 'C':
                    descricao = centro_custo_descricao_dados.get(g1.get('id'))
                else:
                    descricao = evento_descricao_dados.get(g1.get('id'))

                descricao = pooler.get_pool(cr.dbname).get('adm.utils').admutl_remover_acentuacao(
                                                                                 cr, uid, descricao)
                printer.imprimeLinha('{}: {} {}'.format(
                    group_labels[0].ljust(13),
                    u2ascii(g1['codigo']),
                    u2ascii(descricao[:34] or g1['name'])).ljust(55) + ('Total: ' + getNumeroEditado(
                        sum([r['valor'] for r in res_data if r[group_fields[0]] == g1['id']]))).
                                     rjust(25))
                if len(descricao) > 34:
                    descricao = descricao[34:]
                    while descricao:
                        printer.imprimeLinha('{}  {}'.format(
                            ' '.ljust(13),
                            u2ascii(descricao[:34])).ljust(55))
                        descricao = descricao[34:]

                intervalo = sorted([g for g in groups[1] if (g1['id'], g['id']) in map(lambda m: (m[group_fields[0]], m[group_fields[1]]), res_data)], key=lambda x: x['codigo'])
                for g2 in intervalo:
                    if datas.get('ordem') == 'E':
                        descricao = centro_custo_descricao_dados.get(g2.get('id'))
                    else:
                        descricao = evento_descricao_dados.get(g2.get('id'))
                    printer.imprimeLinha('{}: {} {}'.format(
                        group_labels[1].ljust(13),
                        u2ascii(g2['codigo']),
                        u2ascii(obj_adm_utils.admutl_remover_acentuacao(cr, uid,
                                       (descricao[:34] or g2['name']))
                                )).ljust(55)
                                         + ('Total: ' + getNumeroEditado(
                            sum([r['valor'] for r in res_data if r[group_fields[0]] == g1['id'] and
                                 r[group_fields[1]] == g2['id']]))).rjust(25))
                    if len(descricao) > 34:
                        descricao = descricao[34:]
                        while descricao:
                            printer.imprimeLinha('{}  {}'.format(
                                ' '.ljust(13),
                                u2ascii(descricao[:34])).ljust(55))
                            descricao = descricao[34:]
                    for r in sorted([m1 for m1 in res_data if (m1[group_fields[0]], m1[group_fields[1]]) == (g1['id'], g2['id'])], key=lambda m2: m2['fingmo_data']):
                        historico = (r.get('fingmo_historico') or 'Sem Historico')[:42]
                        printer.imprimeLinha(time.strftime('%d/%m/%Y', time.strptime(r['fingmo_data'], '%Y-%m-%d')).ljust(12) +
                                                '{}/{}'.format(r['mes_competencia'], r['ano_competencia']).ljust(11) +
                                                historico.ljust(42) +
                                                getNumeroEditado(r['valor'])[:14].rjust(15))

                    if g2 != intervalo[-1]:
                        printer.imprimeLinha(''.ljust(80, '-'))

                printer.imprimeLinha(''.ljust(80, '='))

        else:
            printer.imprimeLinha(_('Nenhum registro encontrado.').center(80))

        printer.finalizaImpressao()
        self.obj = external_pdf(pdf_file.getvalue())
        self.obj.render()
        pdf_file.close()
        return (self.obj.pdf, 'pdf')


report_gerencial_analitico('report.relatorio.gerencial.analitico')
