# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields

# Tipos de documento
TESOURARIA = 1
TITULO = 2


class wizard_gerencial_documento_sem_classificacao(osv.TransientModel):
    # prefixo: wsemcla_

    _name = 'wizard.gerencial.documento.sem.classificacao'
    _columns = {
        'wsemcla_documento_tipo': fields.selection([(TESOURARIA, 'Documentos de tesouraria'),
                                                    (TITULO, 'Titulos')], 'Documento',
                                                   size=1),
        'wsemcla_conta_tesouraria_id': fields.many2one('financeiro.tesouraria.contas',
                                                       'Conta da tesouraria'),
        'wsemcla_tesouraria_ids': fields.one2many('wizard.gerencial.documento.tesouraria',
                                                  'wgdtes_id', 'Documento de tesouraria'),
        'wsemcla_titulo_documento_id': fields.many2one('documento.documento.tipo',
                                                       'Tipo de documento do titulo'),
        'wsemcla_titulo_tipo': fields.selection([('P', 'A pagar'), ('R', 'A Receber')],
                                                'Tipo do titulo'),
        'wsemcla_titulo_ids': fields.one2many('wizard.gerencial.documento.titulo', 'wgdtit_id',
                                              'Titulo'),
        'wsemcla_periodo_inicio': fields.date('Inicio do periodo', required=True),
        'wsemcla_periodo_fim': fields.date('Fim do periodo', required=True),
        'wsemcla_consulta': fields.boolean('Consultar')  # Apresentacao do botao de consulta
    }
    _defaults = {
        'wsemcla_consulta': True
    }

    def wsemcla_button_print(self, cr, user, ids, context=None):
        """
        O metodo gera o arquivo PDF do relatorio de documentos sem classificacao.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: o arquivo PDF com os dados de documentos sem classificacao
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        dados=dict()
        brw_documentos = self.browse(cr, user, ids[0], context)
        tipo = brw_documentos.wsemcla_documento_tipo
        if not tipo or tipo == TESOURARIA:
            dados['tesouraria'] = self.wsemcla_get_dados_tesouraria(cr, user, ids, context)
        if not tipo or tipo == TITULO:
            dados['titulo'] = self.wsemcla_get_dados_titulo(cr, user, ids, context)

        return dict(type='ir.actions.report.xml', report_name='documento_sem_classificacao',
                    datas=dados)

    def wsemcla_button_consultar(self, cr, user, ids, context=None):
        """
        Mostra na tela os dados reunidos de documentos que nao tiveram classificacao.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a consulta ao dados do formulario de documentos sem classificacao
        foi efetuada com sucesso.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        brw_documentos = self.browse(cr, user, ids[0], context)
        tipo = brw_documentos.wsemcla_documento_tipo
        result = dict()
        if not tipo or tipo == TESOURARIA:
            result['wsemcla_tesouraria_ids'] = [
                [0, False, record] for record in self.wsemcla_get_dados_tesouraria(
                    cr, user, ids, context)]
        if not tipo or tipo == TITULO:
            result['wsemcla_titulo_ids'] = [
                [0, False, record] for record in self.wsemcla_get_dados_titulo(
                    cr, user, ids, context)]
        result['wsemcla_consulta'] = False

        return self.write(cr, user, ids, result)

    def wsemcla_get_dados_tesouraria(self, cr, user, ids, context=None):
        """
        Dados de contas de tesouraria sem classificacao.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: dados de contas de tesouraria sem classificacao.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        parametros = self.read(cr, user, ids[0], [], context, '_classic_write')
        sql = """
        SELECT ftc.apelido                      AS WGDTES_CONTA
             , ftm.data_movimento               AS WGDTES_DATA
             , ftm.debito_credito               AS WGDTES_DEBITO_CREDITO
             , ftm.valor                        AS WGDTES_VALOR
             , BTRIM(ftm.historico, '\n')       AS WGDTES_HISTORICO
        FROM financeiro_tesouraria_movimento    AS ftm
        INNER JOIN financeiro_tesouraria_contas AS ftc
                ON ftc.id = ftm.conta_id
        WHERE ftm.id NOT IN (
              SELECT tesouraria_movimento_id
              FROM gerencial_movimento
              WHERE tesouraria_movimento_id IS NOT NULL)"""
        if parametros.get('wsemcla_conta_tesouraria_id'):
            sql += """
          AND ftm.conta_id = %(wsemcla_conta_tesouraria_id)s"""
        else:
            sql += """
          AND ftm.conta_destino_id IS NULL"""
        sql += """
          AND ftm.data_movimento BETWEEN '%(wsemcla_periodo_inicio)s' AND '%(wsemcla_periodo_fim)s 23:59:59'
        ORDER BY ftm.data_movimento DESC"""
        cr.execute(sql % parametros)

        return cr.dictfetchall()

    def wsemcla_get_dados_titulo(self, cr, user, ids, context=None):
        """
        Dados de titulos sem classificacao.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: dados de titulos sem classificacao
        valores.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        parametros = self.read(cr, user, ids[0], [], context, '_classic_write')
        parametros = dict(parametros, complemento=str())
        fiscal_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, user, 'fiscal', context)
        if fiscal_instalado:
            parametros['complemento'] += """
          AND ff.nf_id IS NULL"""
        ecf_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, user, 'ecf', context)
        if ecf_instalado:
            parametros['complemento'] += """
          AND ff.cupom_id IS NULL"""
        sql = """
        SELECT ap.codigo || '-' || ap.nome  AS WGDTIT_CLIENTE_FORNECEDOR
             , ft.tipo_titulo               AS WGDTIT_TIPO
             , ddt.name                     AS WGDTIT_DOCUMENTO
             , ft.documento                 AS WGDTIT_DOCUMENTO_NUMERO
             , ft.parcela                   AS WGDTIT_PARCELA
             , ft.data_emissao              AS WGDTIT_DATA_EMISSAO
             , ft.data_prorrogacao          AS WGDTIT_DATA_VENCIMENTO
             , ft.saldo                     AS WGDTIT_VALOR
        FROM financeiro_titulo              AS ft
        INNER JOIN adm_pessoa               AS ap
            ON ap.id = ft.pessoa_id
        LEFT JOIN documento_documento_tipo  AS ddt
            ON ddt.id = ft.doc_tipo_id
        LEFT JOIN financeiro_fatura         AS ff
            ON ff.id = ft.fatura_id
        WHERE ft.id NOT IN (
              SELECT titulo_id
              FROM   gerencial_movimento
              WHERE  titulo_id IS NOT NULL)"""
        if parametros.get('wsemcla_titulo_documento_id'):
            sql += """
          AND ft.doc_tipo_id = %(wsemcla_titulo_documento_id)s"""
        if parametros.get('wsemcla_titulo_tipo'):
            sql += """
          AND ft.tipo_titulo = '%(wsemcla_titulo_tipo)s'"""
        sql += """
          AND ft.state != 'C'
          AND ddt.sigla != 'PRV'
          AND ft.fatura_id IS NULL
          AND ft.data_emissao BETWEEN '%(wsemcla_periodo_inicio)s' AND '%(wsemcla_periodo_fim)s 23:59:59'
          %(complemento)s
        ORDER BY ft.data_emissao DESC"""
        cr.execute(sql % parametros)

        return cr.dictfetchall()

    def wsemcla_onchange_documento_tipo(self, cr, user, ids, tesouraria, titulo, context=None):
        """
        Remove os valores de documentos de tesouraria e de titulos e prepara para nova consulta.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com o ID dos documentos de tesouraria
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria e de titulos foram removidos
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        tesouraria_ids = map(lambda m: m[1], tesouraria)
        titulo_ids = map(lambda m: m[1], titulo)
        return dict(value=dict(wsemcla_tesouraria_ids=self._wsemcla_set_documento_tesouraria(
            cr, user, ids, tesouraria_ids, context), wsemcla_titulo_ids=self._wsemcla_set_titulo(
            cr, user, ids, titulo_ids, context), wsemcla_conta_tesouraria_id=False,
            wsemcla_titulo_documento_id=False, wsemcla_consulta=True))

    def wsemcla_onchange_conta_tesouraria(self, cr, user, ids, tesouraria, titulo, context=None):
        """
        Remove os valores de documentos de tesouraria e de titulos e prepara para nova consulta.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com o ID dos documentos de tesouraria
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria e de titulos foram removidos
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        tesouraria_ids = map(lambda m: m[1], tesouraria)
        titulo_ids = map(lambda m: m[1], titulo)
        return dict(value=dict(wsemcla_tesouraria_ids=self._wsemcla_set_documento_tesouraria(
            cr, user, ids, tesouraria_ids, context), wsemcla_titulo_ids=self._wsemcla_set_titulo(
            cr, user, ids, titulo_ids, context), wsemcla_titulo_documento_id=False,
            wsemcla_consulta=True))

    def wsemcla_onchange_titulo_documento(self, cr, user, ids, tesouraria, titulo, context=None):
        """
        Remove os valores de documentos de tesouraria e de titulos e prepara para nova consulta.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com o ID dos documentos de tesouraria
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria e de titulos foram removidos
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        tesouraria_ids = map(lambda m: m[1], tesouraria)
        titulo_ids = map(lambda m: m[1], titulo)
        return dict(value=dict(wsemcla_tesouraria_ids=self._wsemcla_set_documento_tesouraria(
            cr, user, ids, tesouraria_ids, context), wsemcla_titulo_ids=self._wsemcla_set_titulo(
            cr, user, ids, titulo_ids, context), wsemcla_titulo_tipo=False, wsemcla_consulta=True))

    def wsemcla_onchange_titulo_tipo(self, cr, user, ids, tesouraria, titulo, context=None):
        """
        Remove os valores de documentos de tesouraria e de titulos e prepara para nova consulta.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com o ID dos documentos de tesouraria
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria e de titulos foram removidos
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        tesouraria_ids = map(lambda m: m[1], tesouraria)
        titulo_ids = map(lambda m: m[1], titulo)
        return dict(value=dict(wsemcla_tesouraria_ids=self._wsemcla_set_documento_tesouraria(
            cr, user, ids, tesouraria_ids, context), wsemcla_titulo_ids=self._wsemcla_set_titulo(
            cr, user, ids, titulo_ids, context), wsemcla_consulta=True))

    def wsemcla_onchange_periodo_inicio(self, cr, user, ids, tesouraria, titulo, context=None):
        """
        Remove os valores de documentos de tesouraria e de titulos e prepara para nova consulta.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com o ID dos documentos de tesouraria
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria e de titulos foram removidos
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        tesouraria_ids = map(lambda m: m[1], tesouraria)
        titulo_ids = map(lambda m: m[1], titulo)
        return dict(value=dict(wsemcla_tesouraria_ids=self._wsemcla_set_documento_tesouraria(
            cr, user, ids, tesouraria_ids, context), wsemcla_titulo_ids=self._wsemcla_set_titulo(
            cr, user, ids, titulo_ids, context), wsemcla_consulta=True))

    def wsemcla_onchange_periodo_fim(self, cr, user, ids, tesouraria, titulo, context=None):
        """
        Remove os valores de documentos de tesouraria e de titulos e prepara para nova consulta.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com o ID dos documentos de tesouraria
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria e de titulos foram removidos
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        tesouraria_ids = map(lambda m: m[1], tesouraria)
        titulo_ids = map(lambda m: m[1], titulo)
        return dict(value=dict(wsemcla_tesouraria_ids=self._wsemcla_set_documento_tesouraria(
            cr, user, ids, tesouraria_ids, context), wsemcla_titulo_ids=self._wsemcla_set_titulo(
            cr, user, ids, titulo_ids, context), wsemcla_consulta=True))

    def _wsemcla_set_documento_tesouraria(self, cr, user, ids, tesouraria, context=None):
        """
        Remove os valores de documentos de tesouraria.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param tesouraria: lista com ID dos documentos de tesouraria
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tesouraria: list
        :type context: dict
        :return: verdadeiro se os valores de documentos de tesouraria foram removidos com sucesso
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()

        return [[2, _id, False] for _id in tesouraria]

    def _wsemcla_set_titulo(self, cr, user, ids, titulo, context=None):
        """
        Remove os valores de titulos.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param titulo: lista com o ID dos titulos
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type titulo: list
        :type context: dict
        :return: verdadeiro se os valores de titulos foram removidos com sucessso
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()

        return [[2, _id, False] for _id in titulo]


class wizard_gerencial_documento_tesouraria(osv.TransientModel):
    # prefixo: wgdtes_

    _name = 'wizard.gerencial.documento.tesouraria'
    _columns = {
        'wgdtes_id': fields.many2one('wizard.gerencial.documento.sem.classificacao', 'Tesouraria'),
        'wgdtes_conta': fields.char('Conta'),
        'wgdtes_data': fields.date('Data do movimento'),
        'wgdtes_debito_credito': fields.selection([('D', 'Debito'), ('C', 'Credito')],
                                                  'Debito/Credito'),
        'wgdtes_valor': fields.float('Valor', digts=(13, 2)),
        'wgdtes_historico': fields.char('Historico')
    }


class wizard_gerencial_documento_titulo(osv.TransientModel):
    # prefixo: wgdtit_

    _name = 'wizard.gerencial.documento.titulo'
    _columns = {
        'wgdtit_id': fields.many2one('wizard.gerencial.documento.sem.classificacao', 'Titulo'),
        'wgdtit_cliente_fornecedor': fields.char('Cliente/Fornecedor'),
        'wgdtit_tipo': fields.selection([('P', 'A pagar'), ('R', 'A receber')], 'Pagar/Receber'),
        'wgdtit_documento': fields.char('Documento'),
        'wgdtit_documento_numero': fields.char('Numero'),
        'wgdtit_parcela': fields.integer('Parcela'),
        'wgdtit_data_emissao': fields.date('Data de emissao'),
        'wgdtit_data_vencimento': fields.date('Data de vencimento'),
        'wgdtit_valor': fields.float('Saldo', digits=(13, 2))
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
