# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.relativedelta import relativedelta
from dateutil.parser import parse
from SigUtils.numero import getNumeroEditado
from calendar import isleap
import time


class wizard_gerencial_evolucao_conta(osv.TransientModel):
    # prefixo: wgerec_

    _name = 'wizard.gerencial.evolucao.conta'
    _columns = {
        'wgerec_mes': fields.char('Mes', size=2, required=True),
        'wgerec_ano': fields.char('Ano', size=4, required=True),
        'wgerec_tipo_conta': fields.selection([('C', 'Centro de custos'), ('E', 'Eventos')],
                                              'Tipo de conta', size=1, required=True),
        'wgerec_tipo_classificacao': fields.selection([('C', 'Competencia'), ('F', 'Fato gerador')],
                                                      'Tipo de classificacao', size=1,
                                                      required=True),
        'wgerec_item_ids': fields.one2many('wizard.gerencial.evolucao.conta.item', 'wgerei_capa_id',
                                           'Itens'),
        'wgerec_alteracao_filtro': fields.boolean('Alteracao filtro'),
        'wgerec_centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de custo',
                                                  ondelete='cascade'),
    }

    def default_get(self, cr, uid, fields_list, context=None):
        """
        O metodo carrega valores padrao para os campos recebidos.
        :param cr: curspr no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista com a descricao com campos.
        :type fields_list: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com a descricao dos campos e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(wizard_gerencial_evolucao_conta, self).default_get(cr, uid, fields_list,
                                                                       context)
        mes_anterior = str(int(time.strftime('%m')) - 1)
        ano = time.strftime('%Y')
        if mes_anterior == '0':
            mes_anterior = 12
            ano = str(int(ano) - 1)
        res.update({'wgerec_mes': mes_anterior, 'wgerec_ano': ano, 'wgerec_tipo_conta': 'E'})
        return res

    def wgerec_onchange_filtro(self, cr, uid, ids, mes, ano, tipo_conta, tipo_classificacao,
                               context=None):
        """
        O metodo atribui o valor True para o campo 'wgerec_alteracao_filtro' caso algum dos campos
        da tela (parametros) seja alterado.
        :param cr: curspr no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param mes: carater com o numero do mes.
        :type mes: str
        :param ano: caracter com o numero do ano.
        :type ano: str
        :param tipo_conta: 'C' para centro de custos e 'E' para eventos.
        :type tipo_conta: str
        :param tipo_classificacao: 'C' para competencia e 'F' para fato gerador
        :type tipo_classificacao: str
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario cuja chave 'value' recebe um outro dicionario com o campo
        'wgerec_alteracao_filtro' e seu respectivo valor
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        vals = dict()
        if any([mes, ano, tipo_conta, tipo_classificacao]):
            vals.update({'wgerec_alteracao_filtro': True})
        res.update({'value': vals})
        return res

    def _wgerec_get_sequencia_mes_ano(self, cr, uid, mes, ano, context=None):
        """
        O metodo reune em um dicionario doze datas (mensal) de forma crescente ate atingir mes e ano
        informadados.
        :param cr: curspr no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param mes: carater com o numero do mes.
        :type mes: str
        :param ano: caracter com o numero do ano.
        :type ano: str
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario, com chave inteira sequencial de 1 a 12 e as respectivas datas como seu
        valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        list_range = range(1, 12)
        list_range.sort(reverse=True)
        data = ano + '-' + mes + '-' + '01'
        res[12] = parse(data).strftime('%Y-%m-%d')
        for i in list_range:
            data = parse(data) - relativedelta(months=1)
            data = data.strftime('%Y-%m-%d')
            res[i] = data
        return res

    def wgerec_button_find(self, cr, uid, ids, context=None):
        """
        O metodo encontra na tabela 'gerencial_movimento' os dados de acordo com os atributos
        definidos nesse objeto.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso encontre algum registro em 'gerencial_movimento', senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_adm_empresa_centrocusto = self.pool.get('adm.empresa.centrocusto')
        obj_gerencial_evento = self.pool.get('gerencial.evento')
        brw_evolucao_conta = self.browse(cr, uid, ids, context)
        cr.execute("""DELETE FROM wizard_gerencial_evolucao_conta_item""")
        gerencial_movimento_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, context=context)
        company_ids = self.pool.get('res.company').search(
            cr, uid, [('parent_id', 'child_of', gerencial_movimento_company_id)], context=context)
        for conta in brw_evolucao_conta:
            itens = []
            mes = conta.wgerec_mes
            ano = conta.wgerec_ano
            dict_sequencia_mes_ano = self._wgerec_get_sequencia_mes_ano(cr, uid, mes, ano, context)
            tipo_conta = conta.wgerec_tipo_conta
            tipo_classificacao = conta.wgerec_tipo_classificacao
            centro_custo_id = conta.wgerec_centro_custo_id.id
            if tipo_classificacao == 'C':
                dados = self.wgerec_get_dado_competencia(
                    cr, uid, mes, ano, tipo_conta, company_ids, centro_custo_id,
                    context)
            else:
                dados = self.wgerec_get_dado_fato_gerador(
                    cr, uid, mes, ano, tipo_conta, company_ids, centro_custo_id,
                    context)
            if not dados:
                continue

            if tipo_conta == 'C':
                if centro_custo_id:
                    conta_ids = obj_adm_empresa_centrocusto.search(cr, uid,
                                     [('parent_id', 'child_of',[centro_custo_id])], context=context)
                else:
                    conta_ids = obj_adm_empresa_centrocusto.search(cr, uid, [], context=context)
                dados_conta = obj_adm_empresa_centrocusto.read(
                    cr, uid, conta_ids, ['codigo', 'name'], context=context)
            else:
                conta_ids = obj_gerencial_evento.search(cr, uid, [], context=context)
                dados_conta = obj_gerencial_evento.read(cr, uid, conta_ids, ['codigo', 'name'],
                                                        context=context)

            for d_conta in dados_conta:
                descricao_conta = '%s-%s' % (d_conta.get('codigo'), d_conta.get('name'))
                vr_total_conta = 0.0

                if tipo_conta == 'C':
                    conta_filha_ids = obj_adm_empresa_centrocusto.search(
                        cr, uid, [('parent_id', 'child_of', d_conta.get('id'))], context=context)
                    conta_filha_ids = filter(lambda i: i != d_conta.get('id'), conta_filha_ids)
                else:
                    conta_filha_ids = obj_gerencial_evento.search(
                        cr, uid, [('parent_id', 'child_of', d_conta.get('id'))], context=context)
                    conta_filha_ids = filter(lambda i: i != d_conta.get('id'), conta_filha_ids)

                filter_dados = filter(lambda i: i.get('conta_id') == d_conta.get('id'), dados)

                dado_mes1 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(1), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes1.get('vr_total')

                dado_mes2 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(2), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes2.get('vr_total')

                dado_mes3 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(3), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes3.get('vr_total')

                dado_mes4 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(4), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes4.get('vr_total')

                dado_mes5 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(5), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes5.get('vr_total')

                dado_mes6 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(6), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes6.get('vr_total')

                dado_mes7 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(7), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes7.get('vr_total')

                dado_mes8 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(8), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes8.get('vr_total')

                dado_mes9 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(9), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes9.get('vr_total')

                dado_mes10 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(10), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes10.get('vr_total')

                dado_mes11 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(11), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes11.get('vr_total')

                dado_mes12 = self._wgerec_get_descricao_valor_mes(
                    cr, uid, dict_sequencia_mes_ano.get(12), filter_dados, conta_filha_ids, dados,
                    context)
                vr_total_conta += dado_mes12.get('vr_total')

                vals = {
                    'wgerei_capa_id': conta.id,
                    'wgerei_cod_conta': d_conta.get('codigo'),
                    'wgerei_descricao_conta': descricao_conta[:30],
                    'wgerei_mes1': dado_mes1.get('descricao'),
                    'wgerei_mes1_valor': dado_mes1.get('vr_total'),
                    'wgerei_mes2': dado_mes2.get('descricao'),
                    'wgerei_mes2_valor': dado_mes2.get('vr_total'),
                    'wgerei_mes3': dado_mes3.get('descricao'),
                    'wgerei_mes3_valor': dado_mes3.get('vr_total'),
                    'wgerei_mes4': dado_mes4.get('descricao'),
                    'wgerei_mes4_valor': dado_mes4.get('vr_total'),
                    'wgerei_mes5': dado_mes5.get('descricao'),
                    'wgerei_mes5_valor': dado_mes5.get('vr_total'),
                    'wgerei_mes6': dado_mes6.get('descricao'),
                    'wgerei_mes6_valor': dado_mes6.get('vr_total'),
                    'wgerei_mes7': dado_mes7.get('descricao'),
                    'wgerei_mes7_valor': dado_mes7.get('vr_total'),
                    'wgerei_mes8': dado_mes8.get('descricao'),
                    'wgerei_mes8_valor': dado_mes8.get('vr_total'),
                    'wgerei_mes9': dado_mes9.get('descricao'),
                    'wgerei_mes9_valor': dado_mes9.get('vr_total'),
                    'wgerei_mes10': dado_mes10.get('descricao'),
                    'wgerei_mes10_valor': dado_mes10.get('vr_total'),
                    'wgerei_mes11': dado_mes11.get('descricao'),
                    'wgerei_mes11_valor': dado_mes11.get('vr_total'),
                    'wgerei_mes12': dado_mes12.get('descricao'),
                    'wgerei_mes12_valor': dado_mes12.get('vr_total'),
                    'wgerei_vr_total': vr_total_conta,
                }
                itens.append((0, 0, vals))
            self.write(cr, uid, conta.id,
                       {'wgerec_item_ids': itens, 'wgerec_alteracao_filtro': False}, context)
        return True

    def _wgerec_get_descricao_valor_mes(self, cr, uid, data, dados, conta_filha_ids, dados_geral,
                                        context=None):
        """
        O metodo calcula o valor da conta para a data informada, como tambem, concatena o mes e ano
        dessa data com o valor encontrado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param data: string com a data do calculo da conta.
        :type data: str
        :param dados: lista de dicionarios com os dados da conta.
        :type dados: list
        :param conta_filha_ids: lista de id da tabela adm_empresa_controcusto. Contas filhas da
        conta em calculo.
        :type conta_filha_ids: list
        :param dados_geral: lista de dicionarios com os dados de todas as contas.
        :type dados_geral: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com dados da descricao da conta e seu respectivo valor para essa data.
        :rtype: dict
        """
        context = context and context.copy() or {}
        mes = filter(lambda i: i.get('data') == data, dados)
        dict_mes = {'descricao': '%s/%s:\n0,00' % (data[5:7], data[:4]), 'vr_total': 0.0}
        if mes:
            dict_mes['vr_total'] = mes[0].get('vr_total') + sum(
                    map(lambda i: i.get('vr_total'),
                        filter(lambda x: x.get('conta_id') in conta_filha_ids and
                               x.get('data') == data, dados_geral)))
            dict_mes['descricao'] = '%s/%s:\n%s' % (mes[0].get('data')[5:7], mes[0].get('data')[:4],
                                                    getNumeroEditado(dict_mes.get('vr_total')))
        elif filter(lambda x: x.get('conta_id') in conta_filha_ids and x.get('data') == data,
                    dados_geral):
            dict_mes['vr_total'] = sum(map(lambda i: i.get('vr_total'),
                                           filter(lambda x: x.get('conta_id') in conta_filha_ids and
                                                            x.get('data') == data, dados_geral)))
            dict_mes['descricao'] = '%s/%s:\n%s' % (data[5:7], data[:4],
                                                    getNumeroEditado(dict_mes.get('vr_total')))
        return dict_mes

    def wgerec_get_dado_competencia(self, cr, uid, mes, ano, tipo_conta, company_ids,
                                    centro_custo_id=False, context=None):
        """
        O metodo encontra os dados na tabela 'gerencial_movimento' baseado no mes e ano de
        competencia, como tambem na conta, se preco de custo ou evento.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param mes: carater com o numero do mes.
        :type mes: str
        :param ano: caracter com o numero do ano.
        :type ano: str
        :param tipo_conta: 'C' para centro de custos e 'E' para eventos.
        :type tipo_conta: str
        :param company_ids: ids da tabela adm_empresa_objeto, identificadores da
        empresa para a tabela gerencial_movimento
        :type company_ids: list
        :param centro_custo_id: identificador do centro de custos.
        :type centro_custo_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        tabela_conta = 'adm_empresa_centrocusto' if tipo_conta == 'C' else 'gerencial_evento'
        centro_custo = ''
        if centro_custo_id:

            conta_ids = self.pool.get('adm.empresa.centrocusto').search(cr, uid,
                                    [('parent_id', 'child_of', [centro_custo_id])], context=context)
            centro_custo = """
                AND gm.centro_custo_id = %s
            """ % ','.join(map(lambda x: '%s' % x, conta_ids))
        sql = """
            SELECT
                c.codigo                   AS codigo,
                c.id                       AS conta_id,
                c.codigo || '-' || c.name  AS descricao_conta,
                to_date(gm.ano_competencia || '-' || gm.mes_competencia || '-' || 01, 'YYYY-MM-DD')
                                           AS data,
                sum(gm.valor)              AS vr_total
            FROM
                %s c
            INNER JOIN
                gerencial_movimento gm ON c.id = gm.%s
            WHERE
                gm.company_id IN (%s)
                AND gm.ano_competencia IN ('%s', '%s')
                %s
            GROUP BY
                c.id, descricao_conta, data
            ORDER BY c.codigo
        """ % (tabela_conta, 'centro_custo_id' if tipo_conta == 'C' else 'evento_id',
               ', '.join(map(lambda c: str(c), company_ids)), ano, str(int(ano) - 1), centro_custo)
        cr.execute(sql)
        gerencial_dados = cr.dictfetchall()
        data = ano + '-' + mes + '-' + '01'
        gerencial_dados = filter(lambda i: i.get('data') <= data, gerencial_dados)
        return gerencial_dados

    def wgerec_get_dado_fato_gerador(self, cr, uid, mes, ano, tipo_conta, company_ids,
                                     centro_custo_id, context=None):
        """
        O metodo encontra os dados na tabela 'gerencial_movimento' baseado na data de movimento,
        como tambem na conta, se preco de custo ou evento.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param mes: carater com o numero do mes.
        :type mes: str
        :param ano: caracter com o numero do ano.
        :type ano: str
        :param tipo_conta: 'C' para centro de custos e 'E' para eventos.
        :type tipo_conta: str
        :param company_ids: ids da tabela adm_empresa_objeto, identificadores da
        empresa para a tabela gerencial_movimento
        :type company_ids: list
        :param centro_custo_id: identificador do centro de custos.
        :type centro_custo_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        data = ano + '-' + mes + '-' + self.wgerec_get_ultimo_dia_mes_ano(cr, uid, str(int(mes)),
                                                                          ano, context)
        tabela_conta = 'adm_empresa_centrocusto' if tipo_conta == 'C' else 'gerencial_evento'
        centro_custo = ''
        if centro_custo_id:
            centro_custo = """
                AND gm.centro_custo_id = %s
            """ % centro_custo_id
        sql = """
            SELECT
                c.codigo,
                c.id                       AS conta_id,
                c.codigo || '-' || c.name  AS descricao_conta,
                to_date(date_part('year', gm.fingmo_data) || '-' ||
                        date_part('month', gm.fingmo_data) || '-' || 01, 'YYYY-MM-DD')
                                           AS data,
                sum(gm.valor)              AS vr_total
            FROM
                %s c
            INNER JOIN
                gerencial_movimento gm ON c.id = gm.%s
            WHERE
                gm.company_id IN (%s)
                AND gm.fingmo_data <= '%s'
                %s
            GROUP BY
                c.id, descricao_conta, data
            ORDER BY c.codigo
        """ % (tabela_conta, 'centro_custo_id' if tipo_conta == 'C' else 'evento_id',
               ', '.join(map(lambda c: str(c), company_ids)), data, centro_custo)
        cr.execute(sql)
        gerencial_dados = cr.dictfetchall()
        data = ano + '-' + mes + '-' + '01'
        gerencial_dados = filter(lambda i: i.get('data') <= data, gerencial_dados)
        return gerencial_dados

    def wgerec_get_ultimo_dia_mes_ano(self, cr, uid, mes, ano, context=None):
        """
        O metodo recebe strings que representam algum mes e ano para encontrar o ultimo dia
        referente a eles. Identifica, inclusive, se o ano e bissexto.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param mes: caracter que representa um dos meses do ano.
        :type: char
        :param ano: caracter que respresenta um ano.
        :type: char
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: caracter que representa o ultimo dia do mes e ano recebidos.
        :rtype: char
        """
        context = context and context.copy() or {}
        if mes in ('1', '3', '5', '7', '8', '10', '12'):
            ultimo_dia = '31'
        elif mes != '2':
            ultimo_dia = '30'
        else:
            ultimo_dia = '28'
            if isleap(int(ano)):
                ultimo_dia = '29'
        return ultimo_dia

    def wgerec_button_print(self, cr, uid, ids, context=None):
        """
        O metodo reune os dados necessarios para a criacao do relatorio em PDF.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com acao para a chamada do relatorio.
        :rtype: dict
        """
        if isinstance(ids, list):
            ids = ids[0]
        context = context and context.copy() or {}
        evolucao_conta_dados = self.read(cr, uid, ids, [], context)
        dados = {
            'mes': evolucao_conta_dados.get('wgerec_mes'),
            'ano': evolucao_conta_dados.get('wgerec_ano'),
            'tipo_conta': evolucao_conta_dados.get('wgerec_tipo_conta'),
            'tipo_classificacao': evolucao_conta_dados.get('wgerec_tipo_classificacao'),
            'item_ids': self.pool.get('wizard.gerencial.evolucao.conta.item').read(
                cr, uid, evolucao_conta_dados.get('wgerec_item_ids'), [], context)
        }
        return {'type': 'ir.actions.report.xml',
                'report_name': 'gerencial_evolucao_conta',
                'datas': dados,
                'name': 'Evolucao de contas'
                }


class wizard_gerencial_evolucao_conta_item(osv.TransientModel):
    # prefixo: wgerei_

    _name = 'wizard.gerencial.evolucao.conta.item'
    _columns = {
        'wgerei_capa_id': fields.many2one('wizard.gerencial.evolucao.conta', 'Capa'),
        'wgerei_cod_conta': fields.char('Codigo', size=30),
        'wgerei_descricao_conta': fields.char('Conta', size=30),
        'wgerei_mes1': fields.char('1', size=30),
        'wgerei_mes1_valor': fields.float('valor1', digits=(15, 2)),
        'wgerei_mes2': fields.char('2', size=30),
        'wgerei_mes2_valor': fields.float('valor2', digits=(15, 2)),
        'wgerei_mes3': fields.char('3', size=30),
        'wgerei_mes3_valor': fields.float('valor3', digits=(15, 2)),
        'wgerei_mes4': fields.char('4', size=30),
        'wgerei_mes4_valor': fields.float('valor4', digits=(15, 2)),
        'wgerei_mes5': fields.char('5', size=30),
        'wgerei_mes5_valor': fields.float('valor5', digits=(15, 2)),
        'wgerei_mes6': fields.char('6', size=30),
        'wgerei_mes6_valor': fields.float('valor6', digits=(15, 2)),
        'wgerei_mes7': fields.char('7', size=30),
        'wgerei_mes7_valor': fields.float('valor7', digits=(15, 2)),
        'wgerei_mes8': fields.char('8', size=30),
        'wgerei_mes8_valor': fields.float('valor8', digits=(15, 2)),
        'wgerei_mes9': fields.char('9', size=30),
        'wgerei_mes9_valor': fields.float('valor9', digits=(15, 2)),
        'wgerei_mes10': fields.char('10', size=30),
        'wgerei_mes10_valor': fields.float('valor10', digits=(15, 2)),
        'wgerei_mes11': fields.char('11', size=30),
        'wgerei_mes11_valor': fields.float('valor11', digits=(15, 2)),
        'wgerei_mes12': fields.char('12', size=30),
        'wgerei_mes12_valor': fields.float('valor12', digits=(15, 2)),
        'wgerei_vr_total': fields.float('Total', digits=(15, 2))
    }
    _order = 'wgerei_cod_conta'
