# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta


class wizard_gerencial_listagem_dre(osv.Model):
    # prefixo: wizgld_
    _name = 'wizard.gerencial.listagem.dre'
    _columns = {
        'wizgld_data_inicial': fields.date('Data inicial', required=True),
        'wizgld_data_final': fields.date('Data final', required=True),
        'wizgld_dre_id': fields.many2one('financeiro.gerencial.dre', 'DRE', required=True),
        'wizgld_regime': fields.selection([('A', 'Fato gerador'), ('O', 'Competencia')], 'Regime',
                                          size=1, required=True),
        'wizgld_classificacao_origem': fields.boolean('Classificacao da origem'),
        'wizgld_centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de custos',
                                                  ondelete='cascade'),
        'wizgld_tesouraria': fields.boolean('So lancamento da tesouraria'),
    }

    _defaults = {
        'wizgld_tesouraria': lambda *a: False,
    }

    def wizgld_button_print(self, cr, uid, ids, context=None):
        """
        O método realiza o agrupamento dos dados necessários para quem sejam mandados e procesados
        numa classe ralatório.
        :param cr: cursor do banco de dados
        :param uid: identificador do usuário, id da tabela res_users
        :param ids: lista de registros da tabela do objeto
        :param context: dicionário com valores contextuais
        :return: Action, com a chamada da classe do relatório juntamente com os dados a serem
        impressos
        """
        context = context and context.copy() or {}
        brw_gerencial_listagem_dre = self.browse(cr, uid, ids, context)
        dados = {}
        obj_gerencial_evento = self.pool.get('gerencial.evento')
        obj_centro_custo = self.pool.get('adm.empresa.centrocusto')
        for listagem_dre in brw_gerencial_listagem_dre:
            data_inicial = listagem_dre.wizgld_data_inicial
            data_final = listagem_dre.wizgld_data_final
            regime = listagem_dre.wizgld_regime
            classificacao_origem = listagem_dre.wizgld_classificacao_origem
            dre_name = listagem_dre.wizgld_dre_id.name or ''
            linha_ids = listagem_dre.wizgld_dre_id.fingerdre_linha_ids
            so_lancamento_tesouraria = listagem_dre.wizgld_tesouraria
            filtro_centro_custo_id = listagem_dre.wizgld_centro_custo_id.id
            if filtro_centro_custo_id:
                linha_cc_ids = filter(lambda c: c.fingerdrl_centro_custo_id.id ==
                                      filtro_centro_custo_id or
                                      not c.fingerdrl_centro_custo_id.id, linha_ids)
                if linha_cc_ids:
                    linha_cc_ids += filter(lambda c: c.fingerdrl_conta_formula == 'F', linha_ids)
                linha_ids = list(set(linha_cc_ids))
            linha_ids = sorted(linha_ids, key=lambda i: i.fingerdrl_conta_formula)
            dict_sequencia_valor = {}
            linhas_dre = []
            outra_conta_gerencial_movimento_ids = []
            for linha in linha_ids:
                nome = linha.name
                sequencia = linha.fingerdrl_linha
                conta_formula = linha.fingerdrl_conta_formula
                formula_original = linha.fingerdrl_formula or ''
                natureza_id = linha.fingerdrl_natureza_id.id or False
                evento_id = linha.fingerdrl_evento_id.id or False
                centro_custo_id = filtro_centro_custo_id or linha.fingerdrl_centro_custo_id.id or False
                if conta_formula == 'C':
                    valor, gerencial_movimento_ids = self._wizgld_get_valor_conta(
                        cr, uid, natureza_id, evento_id, centro_custo_id, data_inicial, data_final,
                        regime, classificacao_origem, so_lancamento_tesouraria, context)
                    dict_sequencia_valor[sequencia] = valor
                    outra_conta_gerencial_movimento_ids.extend(gerencial_movimento_ids)
                else:
                    valor = 0.0
                    if ('L' or 'l') in formula_original:
                        valor = self._wizgld_get_valor_formula(
                            cr, uid, formula_original, dict_sequencia_valor, context)
                        dict_sequencia_valor[sequencia] = valor
                        linhas_dre.append({'name': nome, 'valor': valor, 'sequencia': sequencia})
                        continue
                    conta_valor_dict = {}
                    if ('E' or 'e') in formula_original:
                        formula_e = formula_original.replace('E', '')
                        list_formula = filter(
                            lambda e: 'C' not in e, self._wizgld_remove_operator(cr, uid, formula_e,
                                                                                 context))
                        evento_ids = obj_gerencial_evento.search(
                            cr, uid, [('codigo', 'in', list_formula)], context=context)
                        gerencial_evento_dados = obj_gerencial_evento.read(
                            cr, uid, evento_ids, ['codigo'], context)
                        for evento in gerencial_evento_dados:
                            valor, gerencial_movimento_ids = self._wizgld_get_valor_conta(
                                cr, uid, natureza_id, evento.get('id'), False, data_inicial,
                                data_final, regime, classificacao_origem, so_lancamento_tesouraria,
                                context)
                            conta_valor_dict.update({'E' + evento.get('codigo'): valor})
                    if ('C' or 'c') in formula_original:
                        formula_c = formula_original.replace('C', '')
                        list_formula = filter(
                            lambda c: 'E' not in c, self._wizgld_remove_operator(cr, uid, formula_c,
                                                                                 context))
                        centro_custo_ids = obj_centro_custo.search(
                            cr, uid, [('codigo', 'in', list_formula)], context=context)
                        centro_custo_dados = obj_centro_custo.read(
                            cr, uid, centro_custo_ids, ['codigo'], context)
                        for custo in centro_custo_dados:
                            valor, gerencial_movimento_ids = self._wizgld_get_valor_conta(
                                cr, uid, natureza_id, custo.get('id'), False, data_inicial,
                                data_final, regime, classificacao_origem, so_lancamento_tesouraria,
                                context)
                            conta_valor_dict.update({'C' + custo.get('codigo'): valor})
                    for c in conta_valor_dict:
                        formula_original = formula_original.replace(c, str(conta_valor_dict.get(c)))
                    try:
                        valor = eval(formula_original)
                    except:
                        valor = 0.0
                    dict_sequencia_valor[sequencia] = valor
                linhas_dre.append({'name': nome, 'valor': valor, 'sequencia': sequencia})
            if outra_conta_gerencial_movimento_ids:
                linhas_dre = self._wizgld_trata_linhas_dre_classificacao_origem(
                    cr, uid, list(set(outra_conta_gerencial_movimento_ids)), linha_ids, linhas_dre,
                    context)
            linhas_dre = sorted(linhas_dre, key=lambda i: i.get('sequencia'))
            dados['linhas_dre'] = linhas_dre
            dados['dre_name'] = dre_name
            dados['data_inicial'] = parse(data_inicial).strftime('%d/%m/%Y')
            dados['data_final'] = parse(data_final).strftime('%d/%m/%Y')
            dados['classificacao_origem'] = True if classificacao_origem else False
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_dre',
            'datas': dados,
        }

    def _wizgld_remove_operator(self, cr, uid, formula, context=None):
        context = context and context.copy() or {}
        if '+' in formula:
            formula = formula.replace('+', '?')
        if '-' in formula:
            formula = formula.replace('-', '?')
        if '/' in formula:
            formula = formula.replace('/', '?')
        if '*' in formula:
            formula = formula.replace('*', '?')
        formula = formula.split('?')
        return formula

    def _wizgld_trata_linhas_dre_classificacao_origem(self, cr, uid, gerencial_movimento_ids,
                                                      linha_ids, linhas_dre, context=None):
        """
        O metodo soma valor a linha existente ou acrescenta uma nova linha as linhas do DRE. Isso e
        devido aos valores encontrados referente a classificacao de origem para as contas que estao
        envolvidas no DRE.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param gerencial_movimento_ids: lista de registros da tabela gerencial_movimento,
        encontrados na classificacao de origem.
        :param linha_ids: lista de objetos da classe financeiro_gerencial_dre_linhas
        :type linha_ids: list
        :param linhas_dre: lista de dicionarios com os dados das contas do DRE.
        :type linhas_dre: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: lista de dicionarios com os dados atualizados das contas do DRE.
        :rtype: list
        """
        context = context and context.copy() or {}
        brw_gerencial_movimento = self.pool.get('gerencial.movimento').browse(
            cr, uid, gerencial_movimento_ids, context)
        for linha in linha_ids:
            evento_id = linha.fingerdrl_evento_id.id or False
            centro_custo_id = linha.fingerdrl_centro_custo_id.id or False
            if not evento_id or not centro_custo_id:
                continue
            for gerencial in brw_gerencial_movimento:
                if gerencial.centro_custo_id.id != centro_custo_id or \
                                gerencial.evento_id.id != evento_id:
                    continue
                dre_linhas = filter(lambda i: i.get('name') == linha.name and
                                              i.get('sequencia') == linha.fingerdrl_linha,
                                    linhas_dre)
                if dre_linhas:
                    dre_linha = dre_linhas[0]
                    dre_linha['valor'] += gerencial.valor
                    linhas_dre.remove(dre_linhas[0])
                    linhas_dre.append(dre_linha)
                else:
                    linhas_dre.append({'name': linha.name, 'valor': gerencial.valor,
                                       'sequencia': linha.fingerdrl_linha})
        return linhas_dre

    def _wizgld_get_valor_formula(self, cr, uid, formula, dict_sequencia_valor, context=None):
        """
        O método realiza o cálculo da fórmula, retonando o valor total.
        :param cr: cursor do banco de dados
        :param uid: identificador do usuário, id da tabela res_users
        :param formula: string que representa a operação que será executada
        :param dict_sequencia_valor: dicionário. A chave e valor representam a sequência e o valor
        das contas, respectivamente, das linhas do DRE
        :param context: dicionário com valores contextuais
        :return: float
        """
        context = context and context.copy() or {}
        linhas = []
        linha_formula = ''
        iniciou = False
        formula = formula.split()
        formula = ''.join(formula)
        contador = 0
        for f in formula:
            contador += 1
            if f.isalpha() or (iniciou and f.isdigit()):
                iniciou = True
                linha_formula += f
            else:
                if iniciou and linha_formula:
                    linhas.append(linha_formula)
                iniciou = False
                linha_formula = ''
            if (f == formula[-1] and iniciou) and contador == len(formula):
                linhas.append(linha_formula)
        nova_formula = formula
        operadores = filter(lambda op: op in '+*-/', nova_formula)
        contador = 0
        nova_formula = ''
        for op in operadores:
            nova_formula += str(dict_sequencia_valor.get(int(linhas[contador][1:]), 0)) + op
            contador += 1
        nova_formula += str(dict_sequencia_valor.get(int(linhas[-1][1:]), 0))
        valor = round(eval(nova_formula), 2)
        return valor

    def _wizgld_get_valor_conta(self, cr, uid, natureza_id, evento_id, centro_custo_id,
                                data_inicial, data_final, regime, classificacao_origem,
                                so_lancamento_tesouraria, context=None):
        """
        O método apura os valores das contas envolvidas dentro do período especificado,
        retonando o valor total.
        :param cr: cursor do banco de dados.
        :type: object
        :param uid: identificador do usuário, id da tabela res_users.
        :type: integer
        :param natureza_id: natureza dos eventos financeiros, id da tabela
        gerencial_evento_natureza.
        :type: integer
        :param centro_custo_id: centro de custo da empresa, id da tabela adm.empresa.centrocusto.
        :type: integer
        :param evento_id: evento financeiro de receitas e despesas, id da tabela gerencial_evento.
        :type: integer
        :param regime: caracter que identifica se e regime de caixa ('A') ou de competencia ('O').
        :type: char
        :param so_lancamento_tesouraria: verdadeiro, para movimentos que possuem tesouraria, senao,
        falso.
        :type so_lancamento_tesouraria: bool
        :param context: dicionário com valores contextuais.
        :type: dict
        :return: soma do total das contas.
        :rtype: float
        """
        context = context and context.copy() or {}
        obj_gerencial_movimento = self.pool.get('gerencial.movimento')
        obj_gerencial_evento = self.pool.get('gerencial.evento')
        args_gerencial_movimento = []
        valor = 0.00
        if not any([natureza_id, evento_id, centro_custo_id]):
            return valor, []
        evento_ids = []
        if natureza_id:
            evento_ids = obj_gerencial_evento.search(cr, uid, [('evento_natureza_id', '=',
                                                                natureza_id)])
        if evento_id:
            evento_ids.extend(obj_gerencial_evento.search(
                cr, uid, [('parent_id', 'child_of', evento_id)], context=context))
        if evento_ids:
            args_gerencial_movimento.append(('evento_id', 'in', evento_ids))
        if centro_custo_id:
            args_gerencial_movimento.append(('centro_custo_id', '=', centro_custo_id))
        if not args_gerencial_movimento:
            return valor, []
        if regime == 'O':
            g_movimento_ids = []
            d_inicial = parse(data_inicial)
            d_final = parse(data_final)
            while d_inicial <= d_final:
                mes_competencia = str(d_inicial.month)
                ano_competencia = str(d_inicial.year)
                g_movimento_ids.extend(obj_gerencial_movimento.search(
                    cr, uid, [('mes_competencia', '=', mes_competencia if len(mes_competencia) == 2
                              else '0'+mes_competencia),
                              ('ano_competencia', '=', ano_competencia)], context=context))
                d_inicial += relativedelta(months=1)
            args_gerencial_movimento.append(('id', 'in', g_movimento_ids))
        else:
            args_gerencial_movimento.extend([('fingmo_data', '>=', data_inicial),
                                             ('fingmo_data', '<=', data_final)])
        if so_lancamento_tesouraria:
            args_gerencial_movimento.append(('tesouraria_movimento_id', '!=', None))
        gerencial_movimento_ids = obj_gerencial_movimento.search(cr, uid, args_gerencial_movimento,
                                                                 context=context)

        outra_conta_gerencial_movimento_ids = []
        if classificacao_origem:
            gerencial_movimento_ids, outra_conta_gerencial_movimento_ids = obj_gerencial_movimento.\
                fingmo_get_gerencial_movimento_classificacao_origem(
                    cr, uid, gerencial_movimento_ids, context)
            valor = obj_gerencial_movimento.fingmo_get_valor_classificacao_origem(
                cr, uid, gerencial_movimento_ids, context) or 0.0
        brw_gerencial_movimento = obj_gerencial_movimento.browse(cr, uid, gerencial_movimento_ids,
                                                                 context)
        if not classificacao_origem or not valor:
            valor = round(sum(map(lambda i: i.valor or 0.00, brw_gerencial_movimento)), 2)
        return valor, outra_conta_gerencial_movimento_ids

    def wizgld_onchange_lancamento_tesouraria(self, cr, uid, ids, tesouraria, context=None):
        """
        O metodo altera o valor do campo 'wizgld_classificacao_origem' para False caso
        'So lancamento da tesouraria' (tesouraria) seja False, pois somente movimento da
        tesouraria sera classificado pela origem.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param tesouraria: Verdadeiro, para somente lancamentos da tesouraria. Senao, Falso.
        :type tesouraria: bool
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario com a descricao do campo como chave e seu respectivo valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not tesouraria:
            res['value'] = {'wizgld_classificacao_origem': False}
        return res



