# -*- encoding: utf-8 -*-

import time
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
import StringIO
from openerp import pooler
from openerp.report.interface import report_int
from openerp.report.render import render
from openerp.osv import fields, osv
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class wizard_gerencial_sintetico(osv.TransientModel):
    _name = "wizard.gerencial.sintetico"
    _description = "Wizard de relatorio gerencial sintetico"
    _columns = {
        'wgs_company_id': fields.many2one('res.company', 'Empresa', required=True),
        'wgs_evento_id': fields.many2one('gerencial.evento', 'Evento'),
        'wgs_centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de Custo'),
        'wgs_data_ini': fields.date('Data', required=True),
        'wgs_data_fim': fields.date('Data', required=True),
        'wgs_ordem': fields.selection([('C', 'Centro de Custo'), ('E', 'Evento')], 'Ordem'),

    }
    _defaults = {
        'wgs_ordem': lambda *a: 'C',
        'wgs_company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }

    def button_imprimir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        wizard_gerencial_sintetico = self.read(cr, uid, ids, [], context=context)
        datas = {}
        company = False
        nome_empresa = ""
        for x in wizard_gerencial_sintetico:
            company_id = x.get('wgs_company_id')
            evento_id = x.get('wgs_evento_id')
            ordem = x.get('wgs_ordem')
            centro_custo_id = x.get('wgs_centro_custo_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
                company = company_id
            if isinstance(evento_id, tuple):
                evento_id = evento_id[0]
            if isinstance(centro_custo_id, tuple):
                centro_custo_id = centro_custo_id[0]
            if not isinstance(company_id, list):
                company_id = [company_id]
            res_company_dados = self.pool.get("res.company").read(cr, uid, company_id, ['name'], context=context)
            for i in res_company_dados:
                nome_empresa = i.get('name')
            datas = {
                'company_id': company,
                'nome_empresa': nome_empresa,
                'centro_custo_id': centro_custo_id,
                'data_ini': x.get('wgs_data_ini'),
                'data_fim': x.get('wgs_data_fim'),
                'evento_id': evento_id,
                'ordem': ordem,

            }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'relatorio.gerencial.sintetico',
            'datas': datas,
        }


class report_relatorio_gerencial_sintetico(report_int):

    def _get_conta_descricao_completa(self, cr, uid, evento_ids, centro_custo_ids, context=None):
        """
        O metodo encontra as contas pai das contas recebidas em evento_ids e centro_custo_ids. Monta
        um dicionario tendo como chave o id da conta e o valor da chave a descricao da conta
        incluindo a descricao das contas pai.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param evento_ids: lista de id da tabela gerencial_evento.
        :type evento_ids: list
        :param centro_custo_ids: lista de id da tabela adm_empresa_centrocusto.
        :type centro_custo_ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario tendo como chave o id da conta e o valor da chave a descricao da conta.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = dict()
        if evento_ids:
            brw_gerencial_evento = pooler.get_pool(cr.dbname).get('gerencial.evento').browse(
                cr, uid, evento_ids, context)
            res['evento'] = dict()
            for g in brw_gerencial_evento:
                conta_pai = g.parent_id
                res['evento'][g.id] = list()
                while conta_pai:
                    res['evento'][g.id].append({'id': conta_pai.id, 'nome': conta_pai.name,
                                                'codigo': conta_pai.codigo, 'valor': 0})
                    res['evento'][g.id] = sorted(res['evento'][g.id], key=lambda i: i.get('codigo'))
                    conta_pai = conta_pai.parent_id

        if centro_custo_ids:
            brw_adm_empresa_centro_custo = pooler.get_pool(cr.dbname).get(
                'adm.empresa.centrocusto').browse(cr, uid, centro_custo_ids, context)
            res['centro_custo'] = dict()
            for c in brw_adm_empresa_centro_custo:
                conta_pai = c.parent_id
                res['centro_custo'][c.id] = list()
                while conta_pai:
                    res['centro_custo'][c.id].append({'id': conta_pai.id, 'nome': conta_pai.name,
                                                      'codigo': conta_pai.codigo, 'valor': 0})
                    res['centro_custo'][c.id] = sorted(res['centro_custo'][c.id],
                                                       key=lambda i: i.get('codigo'))
                    conta_pai = conta_pai.parent_id
        return res

    def create(self, cr, uid, ids, datas, context=None):
    #
        context = context and context.copy() or {}
        pdf_file = StringIO.StringIO()
        empresa = datas['company_id']
        data_ini = datas['data_ini']
        data_fim = datas['data_fim']
        evento = datas['evento_id']
        centro_custo = datas['centro_custo_id']
        ordem = datas['ordem']
        obj_emp = pooler.get_pool(cr.dbname).get("res.company")
        nome_empresa = obj_emp.read(cr, uid, empresa, ['name'], context=context)['name']
        arg = [('company_id', '=', empresa)]
        if not data_ini:
            arg.append(('data', '>=', data_ini))
        if not data_fim:
            arg.append(('data', '<=', data_fim))
        if not evento:
            evento = '''
                        SELECT evento_id
                        FROM gerencial_movimento
                        WHERE fingmo_data >='%s' AND fingmo_data <= '%s'
                        AND company_id = %d
                        ''' % (data_ini, data_fim, empresa)
        else:
            ev_ids = pooler.get_pool(cr.dbname).get('gerencial.evento').search(cr, uid,
                                       [('parent_id', 'child_of', [evento])], context=context)
            evento = ','.join(map(lambda x: '%s' % x, ev_ids))

        if not centro_custo:
            centro_custo = '''
                            SELECT centro_custo_id
                            FROM gerencial_movimento
                            WHERE fingmo_data >='%s' AND fingmo_data <= '%s'
                            AND company_id = %d
                            ''' % (data_ini, data_fim, empresa)
        else:
            cc_ids = pooler.get_pool(cr.dbname).get('adm.empresa.centrocusto').search(cr, uid,
                                        [('parent_id','child_of', [centro_custo])], context=context)
            centro_custo = ','.join(map(lambda x: '%s' % x, cc_ids))

        listagem = Impressora()
        listagem.informaTamLinha(80)
        listagem.informaNome(pdf_file, _('Gerencial Sintetico'))
        cab = _('                              Gerencial Sintetico                            Pag.' + '{{ Page }}')

        listagem.informaCab(cab)
        listagem.informaCab(' ')
        data_emi = time.strftime('%d-%m-%Y')
        listagem.informaCab(data_emi + '             ' + nome_empresa)
        listagem.informaCab(' ')
        listagem.informaCab('--')
        cab = _('     ')
        cab += formataString(_('Codigo'), 15, ' ', 'D', 'S', 'S')
        cab += formataString(_('Descricao'), 23, '  ', 'D', 'N', 'S')
        cab += formataString(_('Valor'), 22, ' ', 'E', 'N', 'S')
        listagem.informaCab(cab)
        listagem.informaCab('--')
        listagem.informaCab(' ')
        listagem.rodapePadrao()
        aux = []
        data = []
        if ordem == 'C':

            _sql = """
                    SELECT  centrocusto.id as centro_custo_id,
                            evento.id      as evento_id,
                            centrocusto.codigo as centro_custo_codigo,
                            centrocusto.name as centro_custo_nome,
                            evento_natureza.id evento_natureza_id,
                            evento_natureza.name evento_natureza_nome,
                            evento.codigo as evento_codigo,
                            evento.name as evento_nome,
                            SUM(gerencial.valor) as valor
                    FROM gerencial_movimento gerencial
                    INNER JOIN gerencial_evento evento
                          ON evento.id = gerencial.evento_id
                    INNER JOIN adm_empresa_centrocusto centrocusto
                          ON gerencial.centro_custo_id = centrocusto.id
                    INNER JOIN gerencial_evento_natureza evento_natureza
                          ON evento.evento_natureza_id = evento_natureza.id
                    WHERE fingmo_data >= '%s'
                    AND fingmo_data <= '%s'
                    AND gerencial.centro_custo_id IN(%s)
                    AND gerencial.evento_id IN(%s)
                    AND gerencial.company_id = %d
                    GROUP BY    evento.id,
                                gerencial.centro_custo_id,
                                centrocusto.codigo,
                                centrocusto.name,
                                centrocusto.id,
                                evento_natureza.id,
                                evento_natureza.name
                    ORDER BY    centrocusto.codigo ASC,
                                evento.codigo

                    """ % (data_ini, data_fim, centro_custo, evento, empresa)
        else:

            _sql = """
                    SELECT      centrocusto.id as centro_custo_id,
                                evento.id      as evento_id,
                                evento.codigo as evento_codigo,
                                evento.name as evento_nome,
                                centrocusto.codigo as centro_custo_codigo,
                                centrocusto.name as centro_custo_nome,

                                SUM(gerencial.valor) as valor
                        FROM gerencial_movimento gerencial
                        INNER JOIN gerencial_evento evento
                              ON evento.id = gerencial.evento_id
                        INNER JOIN adm_empresa_centrocusto centrocusto
                              ON gerencial.centro_custo_id = centrocusto.id
                        INNER JOIN gerencial_evento_natureza evento_natureza
                              ON evento.evento_natureza_id = evento_natureza.id
                        WHERE fingmo_data >= '%s'
                        AND fingmo_data <= '%s'
                        AND gerencial.centro_custo_id IN(%s)
                        AND gerencial.evento_id IN(%s)
                        AND gerencial.company_id = %d
                        GROUP BY    evento.id,
                                    gerencial.centro_custo_id,
                                    centrocusto.codigo,
                                    centrocusto.name,
                                    centrocusto.id
                        ORDER BY evento.codigo ASC,
                                 centrocusto.codigo ASC

                    """ % (data_ini, data_fim, centro_custo, evento, empresa)
        cr.execute(_sql)
        gerencial_dados = cr.dictfetchall()

        conta_descricao_completa_dados = self._get_conta_descricao_completa(
            cr, uid, list(set(map(lambda i: i.get('evento_id'), gerencial_dados))),
            list(set(map(lambda i: i.get('centro_custo_id'), gerencial_dados))), context)

        centro_custo_descricao_dados = conta_descricao_completa_dados.get('centro_custo')
        evento_descricao_dados = conta_descricao_completa_dados.get('evento')

        for dados in gerencial_dados:
            aux.append(dados.get('centro_custo_id'))
            if ordem == 'C':
                aux.append(dados.get('centro_custo_codigo'))
                aux.append(dados.get('centro_custo_nome'))
                aux.append(dados.get('evento_nome'))
                aux.append(dados.get('evento_codigo'))
                aux.append(dados.get('valor'))
                aux.append(dados.get('evento_natureza_id'))
                aux.append(dados.get('evento_natureza_nome'))
                aux.append(dados.get('evento_id'))
            else:
                aux.append(dados.get('evento_codigo'))
                aux.append(dados.get('evento_nome'))
                aux.append(dados.get('centro_custo_codigo'))
                aux.append(dados.get('centro_custo_nome'))
                aux.append(dados.get('valor'))
                aux.append(dados.get('evento_id'))
            a = tuple(aux)
            data.append(a)
            aux = []
        if data:
            cod_centro_custo = data[0][1]
            cod_evento = data[0][1]
            natureza_total = {}
            natureza_resumo = {}
            if ordem == 'C':
                if centro_custo_descricao_dados.get(data[0][0]):
                    for parent in centro_custo_descricao_dados.get(data[0][0]):
                        listagem.imprimeLinha(_('Centro de Custo: ' + parent.get('codigo') + ' ' +
                                                parent.get('nome')))
                listagem.imprimeLinha(_('Centro de Custo: ' + data[0][1] + ' ' + data[0][2]))
                listagem.imprimeLinha(_('Evento: '))
            else:
                if evento_descricao_dados.get(data[0][6]):
                    for parent in evento_descricao_dados.get(data[0][6]):
                        listagem.imprimeLinha(_('Evento: ' + parent.get('codigo') + ' ' +
                                                parent.get('nome')))
                listagem.imprimeLinha(_('Evento: ') + data[0][1] + ' ' + data[0][2])
                listagem.imprimeLinha(_('Centro de Custo:'))
            total_evento = 0.00
            fim = len(data)
            ini = 0
            list_evento = list()
            for g in data:
                ini += 1
                if ordem == 'C':

                    if g[1] != cod_centro_custo:
                        linha_res = resumo = ''
                        for i in natureza_resumo.items():
                            resumo += i[0] + ': %s' % getNumeroEditado(i[1]) + ' '
                        linha_res = formataString(getNumeroEditado(resumo), 65, ' ', 'D', 'N', 'N')
                        listagem.imprimeLinha(linha_res)
                        linha2 = _('     ')
                        listagem.imprimeLinha(linha2)
                        linha2 = _('    ')
                        listagem.imprimeLinha(linha2)
                        natureza_resumo = {}
                        if centro_custo_descricao_dados.get(g[0]):
                            for parent in centro_custo_descricao_dados.get(g[0]):
                                if parent.get('id') in list_evento:
                                    continue
                                listagem.imprimeLinha(_('Centro de Custo: ' + parent.get('codigo') +
                                                        ' ' + parent.get('nome')))
                                list_evento.append(parent.get('id'))
                        listagem.imprimeLinha(_('Centro de Custo: ' + g[1] + ' ' + g[2]))
                        listagem.imprimeLinha(_('Evento: '))
                        cod_centro_custo = g[1]
                        list_evento = list()

                    if evento_descricao_dados.get(g[8]):
                        for parent in evento_descricao_dados.get(g[8]):
                            if parent.get('id') in list_evento:
                                continue
                            linha = _('     ')
                            linha += formataString(parent.get('codigo'), 15, ' ', 'D', 'N', 'S')
                            linha += ' '
                            linha += formataString(parent.get('nome'), 35, ' ', 'D', 'N', 'S')
                            linha += ' '
                            linha += formataString('', 12, ' ', 'E', 'N', 'S')
                            linha += ' '
                            listagem.imprimeLinha(linha)
                            list_evento.append(parent.get('id'))
                    linha = _('     ')
                    linha += formataString(g[4], 15, ' ', 'D', 'N', 'S')
                    linha += ' '
                    linha += formataString(g[3], 35, ' ', 'D', 'N', 'S')
                    linha += ' '
                    linha += formataString(getNumeroEditado(g[5]), 12, ' ', 'E', 'N', 'S')
                    linha += ' '
                    list_evento.append(g[8])
                    if g[7] in natureza_resumo.keys():
                        atualiza_resumo = {g[7]: g[5] + natureza_resumo.get(g[7])}
                        natureza_resumo.update(atualiza_resumo)
                    else:
                        atualiza_resumo = {g[7]: g[5]}
                        natureza_resumo.update(atualiza_resumo)
                    if g[7] in natureza_total.keys():
                        atualiza_total = {g[7]: g[5] + natureza_total.get(g[7])}
                        natureza_total.update(atualiza_total)
                    else:
                        atualiza_total = {g[7]: g[5]}
                        natureza_total.update(atualiza_total)
                    listagem.imprimeLinha(_(linha))
                    if fim == ini:
                        linha2 = _('     ')
                        listagem.imprimeLinha(_(linha2))
                        for i in natureza_resumo.items():
                            resumo = i[0] + ': %s' % getNumeroEditado(i[1])
                            listagem.imprimeLinha(_(getNumeroEditado(resumo)))
                        listagem.imprimeLinha(_('   '))
                        listagem.imprimeLinha('--')
                        listagem.imprimeLinha(_('Resumo total: '))
                        resumo = ''
                        for x in natureza_total.items():
                            resumo += x[0] + ' : %s' % getNumeroEditado(x[1]) + ' '
                        linha = formataString(getNumeroEditado(resumo), 65, ' ', 'D', 'N', 'N')
                        listagem.imprimeLinha(_(linha))
                else:
                    if g[1] != cod_evento:
                        linha2 = _('     ')
                        listagem.imprimeLinha(linha2)
                        linha2 = _('     ')
                        listagem.imprimeLinha(linha2)
                        linha2 = formataString(('Total: %s' % getNumeroEditado(total_evento)), 69,
                                               ' ', 'E', 'N', 'S')
                        listagem.imprimeLinha(linha2)
                        total_evento = 0.00
                        linha2 = _('    ')
                        listagem.imprimeLinha(linha2)
                        if evento_descricao_dados.get(g[6]):
                            for parent in evento_descricao_dados.get(g[6]):
                                if parent.get('id') in list_evento:
                                    continue
                                listagem.imprimeLinha(
                                    _('Evento: ' + parent.get('codigo') + ' ' + parent.get('nome')))
                                list_evento.append(parent.get('id'))
                        listagem.imprimeLinha(_('Evento: ' + g[1] + ' ' + g[2]))
                        listagem.imprimeLinha(_('Centro de Custo: '))
                        cod_evento = g[1]
                        list_evento = list()

                    if centro_custo_descricao_dados.get(g[0]):
                        for parent in centro_custo_descricao_dados.get(g[0]):
                            if parent.get('id') in list_evento:
                                continue
                            linha = _('     ')
                            linha += formataString(parent.get('codigo'), 15, ' ', 'D', 'N', 'S')
                            linha += ' '
                            linha += formataString(parent.get('nome'), 35, ' ', 'D', 'N', 'S')
                            linha += ' '
                            linha += formataString('', 12, ' ', 'E', 'N', 'S')
                            linha += ' '
                            listagem.imprimeLinha(linha)
                            list_evento.append(parent.get('id'))

                    linha = _('     ')
                    linha += formataString(g[3], 15, ' ', 'D', 'N', 'S')
                    linha += ' '
                    linha += formataString(g[4], 35, ' ', 'D', 'N', 'S')
                    linha += ' '
                    linha += formataString(getNumeroEditado(g[5]), 12, ' ', 'E', 'N', 'S')
                    linha += ' '
                    list_evento.append(g[0])

                    listagem.imprimeLinha(linha)
                    listagem.imprimeLinha(' ')
                    total_evento += g[5]
                    if fim == ini:
                        linha2 = formataString(('Total: %s' % getNumeroEditado(total_evento)), 69,
                                               ' ', 'E', 'N', 'S')
                        listagem.imprimeLinha(_(linha2))
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_file.getvalue())
        self.obj.render()
        pdf_file.close()
        return (self.obj.pdf, 'pdf')


report_relatorio_gerencial_sintetico('report.relatorio.gerencial.sintetico')
