# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp import pooler
import time


class financeiro_cliente(osv.Model):
    """
    ===> Classe financeiro.cliente <===
        Especializacao da classe adm.pessoa
        Uma pessoa pode ser cliente em varias empresas
    """

    from funcoes import _fincli_verificar_status_cliente
    from funcoes import _fincli_get_dados_adicionais_cliente
    from funcoes import _fincli_get_clientes_estatisticas_compras
    from funcoes import _fincli_get_total_em_aberto
    from funcoes import _fincli_verifica_duplicidade
    from funcoes import fincli_get_titulo

    _name = 'financeiro.cliente'
    _description = 'Cadastro de clientes'
    _inherits = {
        'adm.pessoa': 'pessoa_id',
    }
    _rec_name = 'pessoa_id'
    _roteavel = True
    _columns = {
        'id': fields.integer('Id'),
        'pessoa_id': fields.many2one("adm.pessoa", 'Pessoa', ondelete="cascade", select=True, required=True),
        # id da pessoa
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # id da empresa onde e cliente
        'data_inclusao': fields.date('Data inclusao', required=True), # Data de inclusao do cliente
        'limite_credito': fields.float('Limite credito', digits=(12, 2)), # Limite de credito do cliente
        'fincli_total_em_aberto': fields.function(_fincli_get_total_em_aberto, method=True,
                                                  string="Total em aberto", store=False,
                                                  type="float"),  # Total de debito em aberto
        'fincli_forma_pgto_id': fields.many2one("adm.forma.pgto", 'Forma de Pgto.', ondelete="set null", select=True),
        'fincli_condicao_forma_id': fields.many2one("adm.condicao.forma", 'Condicao de Pgto.', ondelete="set null", select=True,
                                            domain="[('forma_pagamento_id', '=', fincli_forma_pgto_id)]"),
        'active': fields.boolean('Ativo'), # Indica se o cliente está ativo
        'mensagem_ids': fields.one2many('financeiro.clientes.mensagem', 'cliente_id', 'Mensagens'),
        'create_date': fields.datetime('Data criacao', readonly=True),
        'write_date': fields.datetime('Data edicao', readonly=True),
        'fincli_status_cliente': fields.function(_fincli_verificar_status_cliente, string='Status Cliente', type="char",
                                                 size=64, store=False),
        'fincli_primeira_compra_data': fields.function(_fincli_get_dados_adicionais_cliente, string="Data",
                                                       multi="dados_adicionais", store=False, type="date"),
        'fincli_primeira_compra_valor': fields.function(_fincli_get_dados_adicionais_cliente, string="Valor",
                                                        multi="dados_adicionais", store=False, type="float"),
        'fincli_ultima_compra_data': fields.function(_fincli_get_dados_adicionais_cliente, string="Data",
                                                     multi="dados_adicionais", store=False, type="date"),
        'fincli_ultima_compra_valor': fields.function(_fincli_get_dados_adicionais_cliente, string="Valor",
                                                      multi="dados_adicionais", store=False, type="float"),
        'fincli_maior_compra_data': fields.function(_fincli_get_dados_adicionais_cliente, string="Data",
                                                    multi="dados_adicionais", store=False, type="date"),
        'fincli_maior_compra_valor': fields.function(_fincli_get_dados_adicionais_cliente, string="Valor",
                                                     multi="dados_adicionais", store=False, type="float"),
        'fincli_media_compra_valor': fields.function(_fincli_get_dados_adicionais_cliente, string="Valor comprado",
                                                     multi="dados_adicionais", store=False, type="float"),
        'fincli_media_compra_atraso': fields.function(_fincli_get_dados_adicionais_cliente, string="Atrasos",
                                                      multi="dados_adicionais", store=False, type="float"),
        'fincli_maior_compra_data_atraso': fields.function(_fincli_get_dados_adicionais_cliente, string="Data",
                                                           multi="dados_adicionais", store=False, type="date"),
        'fincli_maior_compra_dias_atraso': fields.function(_fincli_get_dados_adicionais_cliente, string="Dias",
                                                           multi="dados_adicionais", store=False, type="integer"),
        'fincli_media_compra_prazo': fields.function(_fincli_get_dados_adicionais_cliente, string="Compras",
                                                     multi="dados_adicionais", store=False, type="float"),
        'fincli_media_compra_prorrogacao': fields.function(_fincli_get_dados_adicionais_cliente, string="Prorrogacao",
                                                           multi="dados_adicionais", store=False, type="float"),
        'fincli_cliente_estatisticas_compras_ids': fields.function(_fincli_get_clientes_estatisticas_compras,
                                                                   method=True, string="Estatisticas", type="one2many",
                                                                   relation="cliente.estatisticas.compras.osv",
                                                                   store=False),
        'fincli_titulo_cliente_ids': fields.function(fincli_get_titulo, method=True,
                                                     string='Titulos', type='one2many',
                                                     relation='financeiro.titulo', store=False),
    }
    _order = 'pessoa_id'
    _defaults = {
        'active': lambda *a: 1,
        'data_inclusao': lambda *a: time.strftime('%Y-%m-%d'),
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }
    _constraints = [(_fincli_verifica_duplicidade, 'CPF/CNPJ Ja cadastrado como cliente',
                    ['cpf_cnpj'])]


class cliente_estatisticas_compras_osv(osv.TransientModel):
    _name = 'cliente.estatisticas.compras.osv'
    _columns = {
        'cliente_id': fields.many2one('financeiro.cliente', 'Cliente', required=True, ondelete='cascade'),
        'data_estatistica': fields.char('Meses', size=12, required=True),
        'total_mes': fields.float('Valor da compra', digits=(16, 2), required=True),
        'data': fields.date('Data'),
    }
    _order = 'data DESC'
