# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class financeiro_local_natureza(osv.Model):
    _name = "financeiro.local.natureza"
    from funcoes import _finlon_get_default_par_centro_custo, _finlon_get_par_centro_custo
    from funcoes import _finlon_get_conta_agencia, _finlon_get_local_tipo, _finlon_get_default_local_tipo
    from funcoes import _finlon_verificar_carteira

    _columns = {
        'finlon_local_cobranca_id': fields.many2one('financeiro.local.cobranca',
                                                    'Local de cobranca', required=True),

        'finlon_natureza_cobranca_id': fields.many2one('financeiro.natureza.cobranca',
                                                       'Natureza de cobranca', required=False),
        'finlon_nosso_numero': fields.char('Nosso numero', size=25, help="Numero que identifica o boleto no banco."),
        'finlon_conta': fields.function(_finlon_get_conta_agencia, method=True, string='Conta',
                                        multi="banco_agencia", type='char', size=10, store=False, readonly=True),
        'finlon_conta_dv': fields.function(_finlon_get_conta_agencia, method=True, string='Digito',
                                        multi="banco_agencia", type='char', size=1, store=False, readonly=True),
        'finlon_agencia': fields.function(_finlon_get_conta_agencia, method=True, string='Agencia',
                                        multi="banco_agencia", type='char', size=5, store=False, readonly=True),
        'finlon_agencia_dv': fields.function(_finlon_get_conta_agencia, method=True, string='Digito',
                                        multi="banco_agencia", type='char', size=1, store=False, readonly=True),
        'finlon_carteira': fields.char('Carteira', size=4, required=False,
                                        help="Código que identifica a carteira conforme modalidade de cobranca. Deve ser informado pelo banco."),
        'finlon_convenio': fields.char('Convenio', size=10,
                                       help="Numero do contrato firmado entre empresa e banco para cobranca atraves de boleto bancario. Deve ser informado pelo banco. Para Sicoob, o convenio é o numero do cliente"),
        'finlon_prazo_liberacao': fields.integer('Prazo p/ liberacao',
                                                 help='Quantos dias o banco demora para creditar o recebimento na conta corrente.'),
        'finlon_centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de custo',
                                                  help='Centro de custo a ser utilizado nas baixas '
                                                       'de titulos do contas a receber neste '
                                                       'local/natureza de cobranca. Este centro de '
                                                       'custo vai sobrepor ao cadastrado no '
                                                       'parametro do financeiro para recebimento '
                                                       'de titulos.'),
        # Este evento vai sobrepor ao cadastrado no parametro do financeiro para recebimento de
        # titulos
        'finlon_evento_id': fields.many2one('gerencial.evento', 'Evento',
                                            help='Evento a ser utilizado nas baixas de titulos do '
                                                 'contas a receber neste local/natureza de '
                                                 'cobranca. Este evento vai sobrepor ao cadastrado '
                                                 'no parametro do financeiro para recebimento de '
                                                 'titulos.'),
        'finlon_boleto_emissao': fields.selection([('1', 'Banco emite'), ('2', 'Cedente emite')], 'Emissao do boleto'),
        'finlon_boleto_distribuicao': fields.selection([('1', 'Banco distribui'), ('2', 'Cedente distribui')], 'Distribuicao do boleto'),
        'finlon_padrao_cnab': fields.selection([('000', 'Nao definido'), ('240', '240'), ('400', '400')], 'Modelo CNAB'),
        'finlon_registro_banco': fields.selection([('1', 'Com registro'), ('2', 'Sem registro')], 'Registro no banco'),
        'finlon_usar_centro_custos_e_eventos': fields.function(_finlon_get_par_centro_custo, method=True, string='CCE', type='boolean', store=False),
        'finlon_local_tipo': fields.function(_finlon_get_local_tipo, method=True, string='Tipo', type='boolean', store=False),
    }

    _defaults = {
        'finlon_usar_centro_custos_e_eventos': _finlon_get_default_par_centro_custo,
        'finlon_local_tipo': _finlon_get_default_local_tipo,
        'finlon_padrao_cnab': '000',
    }
    _sql_constraints = [
        ('finlon_local_natureza_uniq',
         'unique (finlon_local_cobranca_id, finlon_natureza_cobranca_id)',
         'O Local e Natureza de cobranca precisam ser unicos'),
    ]
    _constraints = [(_finlon_verificar_carteira, 'Carteira ja cadastrada para esta conta bancaria!',
                    ['finlon_conta', 'finlon_carteira'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4