# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class financeiro_parametros(osv.Model):
    """
    Tabela de parametros do financeiro
    """
    _name = 'financeiro.parametros'
    _description = 'Tabela de parametros do financeiro'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # id da empresa
        'juros_atraso': fields.float('Juros mensal', digits=(4, 2),
                                     help='Taxa de juros mensal a ser aplicada sobre o valor do titulo para cada dia de atraso no pagamento'),
        # Juros mensal a ser cobrado no caso de atraso do pagamento do titulo
        'multa_atraso': fields.float('Multa por atraso', digits=(4, 2),
                                     help="Percentual a ser cobrado sobre o valor do titulo no caso de atraso de pagamento, a titulo de multa"),
        # Multa a ser cobrada no caso de atraso no pagamento do titulo
        'prazo_perdao': fields.integer('Prazo perdao'),
        # Quantidade de dias a ser considera como perdao no caso de atraso no pagamento de titulos
        'local_cobranca_id': fields.many2one("financeiro.local.cobranca", 'Local de Cobranca', ondelete="restrict",
                                             required=True, domain="[('company_id', '=', company_id)]",
                                             help="Local de cobranca a ser utilizado na entrada de titulos, qdo nao se tem um local expecificado explicitamente."),
        'alterar_multa_juros': fields.boolean('Alterar multa e juros'), #permite ou nao alteracao no juros e multa
        'moeda_id': fields.many2one("adm.moeda", 'Moeda', ondelete="restrict", required=True, select=True),
        'altera_movimento': fields.boolean('Permite alterar movimento'),
        'exclusao_movimento': fields.boolean('Permite exclusao de movimento'),
        'usar_centro_custos_e_eventos': fields.boolean('Usar centro de custos e eventos'),
        'evento_juros_recebidos_id': fields.many2one("gerencial.evento", 'Evento Juros Recebidos', ondelete="cascade"),
        'finpar_evento_venda_taxa_adm_id': fields.many2one("gerencial.evento",
                                                           'Evento para taxa administrativa',
                                                           ondelete="cascade",
                                                           help="Evento para classificacao dos "
                                                                "valores referentes a taxa "
                                                                "administrativa nas operacoes de venda"),
        'centro_custo_juros_recebidos_id': fields.many2one("adm.empresa.centrocusto",
                                                           'Centro de custos p/Juros Recebidos', ondelete="cascade"),
        'evento_juros_pagos_id': fields.many2one("gerencial.evento", 'Evento Juros Pagos.', ondelete="cascade"),
        'centro_custo_juros_pagos_id': fields.many2one("adm.empresa.centrocusto", 'Centro de custos p/Juros Pagos',
                                                       ondelete="cascade"),
        'evento_multa_paga_id': fields.many2one("gerencial.evento", 'Evento Multa Pagos.', ondelete="cascade"),
        'centro_custo_multa_paga_id': fields.many2one("adm.empresa.centrocusto", 'Centro de custos p/Multa Paga',
                                                      ondelete="cascade"),
        'evento_multa_recebida_id': fields.many2one("gerencial.evento", 'Evento Multa Recebida', ondelete="cascade"),
        'centro_custo_multa_recebida_id': fields.many2one("adm.empresa.centrocusto",
                                                          'Centro de custos p/Multa Recebida', ondelete="cascade"),
        'evento_desconto_pago_id': fields.many2one("gerencial.evento", 'Evento Desconto Pago', ondelete="cascade"),
        'centro_custo_desconto_pago_id': fields.many2one("adm.empresa.centrocusto", 'Centro de custos p/Desconto Pago',
                                                         ondelete="cascade"),
        'evento_desconto_recebido_id': fields.many2one("gerencial.evento", 'Evento Desconto Recebido',
                                                       ondelete="cascade"),
        'centro_custo_desconto_recebido_id': fields.many2one("adm.empresa.centrocusto",
                                                             'Centro de custos p/Desconto Recebido',
                                                             ondelete="cascade"),
        'cadastro_automatico_cliente': fields.boolean('Cadastro automatico de cliente'),
        'cadastro_automatico_fornecedor': fields.boolean('Cadastro automatico de fornecedor'),
        'finpar_centro_custo_pagamentos_id': fields.many2one('adm.empresa.centrocusto',
                                                               'Centro Custo p/pagamentos',
                                                               help='Centro de custo a ser utilizado'
                                                                    ' no pagamento dos titulos'),

        'finpar_evento_pagamentos_id': fields.many2one("gerencial.evento", 'Evento para pagamentos',
                                                       ondelete="cascade"),
        'finpar_esconde_forma_pgto': fields.boolean('Esconder forma de pagamento'),
        'finpar_prazo_media_atraso': fields.integer('Prazo p/ media de atraso',
                                                    help='Prazo em meses para calculo da media de atraso do cliente no'
                                                         ' pagamento de titulos'),
        'finpar_centro_custo_entr_tit_rec_id': fields.many2one('adm.empresa.centrocusto',
                                                               'Centro Custo entrada titulos(a receber)',
                                                               help='Centro de custo a ser utilizado no lancamento de '
                                                                    'entradas de titulos a receber. Este campo sera '
                                                                    'sobreposto se ao dar entrada no titulo, usar um '
                                                                    'tipo de documento que tenha  outro centro de '
                                                                    'custo cadastrado'),
        'finpar_evento_entr_tit_rec_id': fields.many2one('gerencial.evento', 'Evento entrada titulos(a receber)',
                                                         help='Evento a ser utilizado no lancamento de entradas de '
                                                              'titulos a receber. Este campo sera sobreposto se ao dar '
                                                              'entrada no titulo, usar um tipo de documento que tenha '
                                                              'outro evento cadastrado'),
        'finpar_centro_custo_recebimentos_id': fields.many2one('adm.empresa.centrocusto',
                                                               'Centro Custo p/recebimento',
                                                               help='Centro de custo a ser utilizado'
                                                                    ' no recebimento dos titulos'),
        'finpar_evento_recebimentos_id': fields.many2one("gerencial.evento", 'Evento para recebimentos',
                                                         ondelete="cascade"),
        'finpar_conta_tesouraria_avista_id': fields.many2one(
            'financeiro.tesouraria.contas', 'Conta caixa recebimento/pagamento a vista',
            help='Esta conta da tesouraria pode ser usada nos modulos Fiscal/ECF/Financeiro/'
                 'Convenio'),
        'finpar_carencia_restricao': fields.integer('Carencia de restricao',
                                                    help='Prazo a ser considerado para carencia de '
                                                         'restricao de venda.'),
        'finpar_mensagem_boleto': fields.text('Observacao no boleto',
                                              help="Observacoes que serao levadas para o campo 'Observacoes' ao gerar um boleto"),
        'finpar_imprimir_informacoes_titulo_boleto': fields.boolean('Imprimir informacoes do titulo '
                                                                    'no boleto',
                                                                    help='Se marcado considera para'
                                                                         'boleto as informacoes do'
                                                                         'titulo'),
        'finpar_evento_venda_frete_id': fields.many2one('gerencial.evento', 'Evento p/Frete',
                                                        help="Evento para classificacao dos valores"
                                                             " referentes a frete nas operacoes de "
                                                             "venda"),
        'finpar_evento_compra_frete_id': fields.many2one('gerencial.evento', 'Evento p/Frete',
                                                         help="Evento para classificacao dos "
                                                              "valores referentes a frete nas "
                                                              "operacoes de compra"),
        'finpar_evento_venda_id': fields.many2one('gerencial.evento', 'Evento Venda',
                                                  help="Evento para classificacao de operacoes de "
                                                       "venda"),
        'finpar_evento_venda_seguro_id': fields.many2one('gerencial.evento', 'Evento p/Seguro',
                                                         help="Evento para classificacao dos "
                                                              "valores referentes a seguro nas "
                                                              "operacoes de venda"),
        'finpar_evento_compra_seguro_id': fields.many2one('gerencial.evento', 'Evento p/Seguro',
                                                          help="Evento para classificacao dos "
                                                               "valores referentes a seguro nas "
                                                               "operacoes de compra"),
        'finpar_evento_venda_desp_acessoria_id': fields.many2one('gerencial.evento',
                                                                 'Evento p/Despesas acessorias',
                                                                 help="Evento para classificacao "
                                                                      "dos valores referentes a "
                                                                      "despesas acessorias nas "
                                                                      "operacoes de venda"),
        'finpar_evento_compra_desp_acessoria_id': fields.many2one('gerencial.evento',
                                                                  'Evento p/Despesas acessorias',
                                                                  help="Evento para classificacao "
                                                                       "dos valores referentes a "
                                                                       "despesas acessorias nas "
                                                                       "operacoes de compra"),
        'finpar_evento_venda_desconto_id': fields.many2one('gerencial.evento',
                                                           'Evento p/Desconto concedido',
                                                           help="Evento para classificacao dos "
                                                                "valores referentes a desconto "
                                                                "concedido nas operacoes de venda"),
        'finpar_subst_evento_id': fields.many2one('gerencial.evento',
                                                  'Evento p/Icms Subst.Tribut.'),
        'finpar_bonificacao_evento_id': fields.many2one('gerencial.evento',
                                                        'Evento p/ Bonificacao'),
        'finpar_desconto_compra_evento_id': fields.many2one('gerencial.evento',
                                                            'Evento p/ Desconto na Compra'),
        'finpar_gnre_evento_id': fields.many2one('gerencial.evento', 'Evento p/GNRE'),
        'finpar_evento_compra_contrato_id': fields.many2one("gerencial.evento", 'Evento para contrato fornecedor',
                                                            ondelete="cascade",
                                                            help="Evento a ser utilizado no lancamento de contratos fornecedor. Este campo nao sera 'sobreposto'. ",
                                                            ),
        'finpar_centro_custo_acrescimo_pago_id': fields.many2one(
            'adm.empresa.centrocusto', 'Centro de custos p/ Acrescimo Pago', ondelete='cascade'),
        'finpar_centro_custo_acrescimo_recebido_id': fields.many2one(
            'adm.empresa.centrocusto', 'Centro de custos p/ Acrescimo Recebido',
            ondelete='cascade'),
        'finpar_evento_acrescimo_pago_id': fields.many2one(
            'gerencial.evento', 'Evento Acrescimo Pago', ondelete='cascade'),
        'finpar_evento_acrescimo_recebido_id': fields.many2one(
            'gerencial.evento', 'Evento Acrescimo Recebido', ondelete='cascade'),
        'finpar_evento_refiananciamento_rec_id': fields.many2one('gerencial.evento',
                                                               'Evento para Refinanciamento de titulos'),
        'finpar_centro_custo_refinanciamento_rec_id': fields.many2one('adm.empresa.centrocusto',
                                                               'Centro Custo para Refinanciamento de titulos',
                                                               help='Centro de custo a ser utilizado'
                                                                    ' no refinanciamento dos titulos'),
        'finpar_evento_refinanciamento_pag_id': fields.many2one('gerencial.evento',
                                                                'Evento para Refinanciamento de titulos'),
        'finpar_centro_custo_refinanciamento_pag_id': fields.many2one('adm.empresa.centrocusto',
                                                               'Centro Custo para Refinanciamento de titulos',
                                                               help='Centro de custo a ser utilizado'
                                                                    ' no refinanciamento dos titulos'),
        'finpar_recibo_baixa_laser_matricial': fields.selection(
            [('L', 'Laser'), ('M', 'Matricial')], 'Tipo impressao recibo', size=1),
        'finpar_recibo_impressora': fields.char('Impressora para recibo', size=128),
        'finpar_email_server_id': fields.many2one('ir.mail_server',
                                                  'Servidor para envio dos e-mails'),
        'finpar_conta_para_baixa_titulos': fields.many2one('financeiro.tesouraria.contas',
                                   'Conta recebto/pagto de titulos',
                                   help='Conta da tesouraria para  baixa de titulos para evitar ter'
                                        ' que informar todas as vezes'),
        'finpar_forma_pagto_baixa_titulos': fields.many2one('adm.forma.pgto',
                                   'Forma de pagto p/recebto/pagto de titulos',
                                   help='Forma de pagamento para baixa de titulos para evitar ter '
                                        'que informar todas as vezes'),
        'finpar_condicao_forma_baixa_titulos': fields.many2one(
            'adm.condicao.forma', 'Condicao de pagto p/recebto/pagto de titulos',
            help='Condicao de pagamento para baixa de titulos para evitar ter que informar todas '
                 'as vezes',
            domain="[('forma_pagamento_id', '=', finpar_forma_pagto_baixa_titulos)]"),
        'finpar_data_conciliacao': fields.date(_('Data de conciliacao')),
    }
    _defaults = {
        'alterar_multa_juros': lambda *a: True,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'exclusao_movimento': lambda *a: True,
        'cadastro_automatico_cliente': lambda *a: True,
        'cadastro_automatico_fornecedor': lambda *a: True,
        'finpar_esconde_forma_pgto': lambda *a: True,
        'usar_centro_custos_e_eventos': lambda *a: False,
        'finpar_mensagem_boleto': '',
        'finpar_recibo_baixa_laser_matricial': 'L',
    }
    _sql_constraints = [
        ('company_id_uniq', 'unique (company_id)', 'Ja existe um parametro cadastrado para esta empresa!'),
    ]
