# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import fields
from openerp.osv import osv


class financeiro_titulo_baixa(osv.Model):
    """
    Tabela que contem os recebimentos e pagamentos de titulos do Contas a Receber/Pagar.
    """
    """
    PREFIXO PARA NOVOS CAMPOS - fintib_
    """

    from funcoes import _fintib_get_forma_pagamento_baixa

    _name = 'financeiro.titulo.baixa'
    _rec_name = 'descricao'
    _columns = {
        'active': fields.boolean('Ativo'),
        'create_uid': fields.many2one('res.users', 'Usuario', readonly=True),
        'data_baixa': fields.date('Data baixa', required=True, select=1, help='Data da baixa na empresa'),  # Data da baixa na empresa.
        'data_pgto': fields.date('Data Pagamento', required=True, select=1, help='Data do pagamento efetuado'),  # Data do pagamento efetuado
        # pelo sacado.
        'descricao': fields.char('Descricao', size=80),  # Descricao da baixa
        'titulo_id': fields.many2one("financeiro.titulo", 'Titulo', ondelete="cascade",
                                     required=True, select=True),  # Id do Titulo baixado
        'valor_pago': fields.float('Valor Pago', digits=(15, 2)),
        'valor_juros': fields.float('Valor Juros', digits=(15, 2)),
        'valor_multa': fields.float('Valor Multa', digits=(15, 2)),
        'valor_desconto': fields.float('Valor Desconto', digits=(15, 2)),
        'fintib_vr_acrescimo': fields.float('Vr acrescimo', digits=(15, 2)),
        'tesouraria_movimento_id': fields.many2one("financeiro.tesouraria.movimento", 'Movimento',
                                                   ondelete="restrict", select=True),  # Historico
                                                   # de movimentacao de um titulo
        'baixa_forma_pgto_ids': fields.one2many('financeiro.tesouraria.baixa.forma.pgto',
                                                'tesouraria_baixa_id', 'Pgto'),  # Relacao das
                                                # formas de pagamento utilizadas em uma baixa
        'fintib_forma_pagamento_baixa': fields.function(_fintib_get_forma_pagamento_baixa,
                                                        method=True, string='Forma de pagamento',
                                                        type='char', size=30,
                                                        store=False, multi='dados_titulo'),
        'fintib_chq_banco': fields.function(_fintib_get_forma_pagamento_baixa,
                                            method=True, string='Banco',
                                            type='integer', store=False, multi='dados_titulo'),
        'fintib_chq_agencia': fields.function(_fintib_get_forma_pagamento_baixa,
                                              method=True, string='Agencia',
                                              type='integer', store=False, multi='dados_titulo'),
        'fintib_chq_conta': fields.function(_fintib_get_forma_pagamento_baixa,
                                            method=True, string='Conta',
                                            type='char', store=False, multi='dados_titulo'),
        'fintib_chq_emitente': fields.function(_fintib_get_forma_pagamento_baixa,
                                               method=True, string='CPF/CNPJ Emitente',
                                               type='char', store=False, multi='dados_titulo'),
        'fintib_documento': fields.function(_fintib_get_forma_pagamento_baixa,
                                            method=True, string='Numero do cheque',
                                            type='char', store=False, multi='dados_titulo'),
        'fintib_is_cheque': fields.function(_fintib_get_forma_pagamento_baixa,
                                            method=True, string='Numero do cheque',
                                            type='boolean', store=False, multi='dados_titulo'),
        'fintib_baixa_ids': fields.function(_fintib_get_forma_pagamento_baixa,
                                            method=True, string='Titulos da baixa',
                                            type='one2many', store=False, multi='dados_titulo',
                                            relation='financeiro.titulo.baixa'),
        'fintib_tesouraria_movimento_valor': fields.related(
            'tesouraria_movimento_id', 'valor', type='float',
            relation='financeiro.tesouraria.movimento', string='Valor total'),
        'fintib_pessoa_id': fields.related(
            'titulo_id', 'pessoa_id', type='many2one',
            relation='adm.pessoa', string='Cliente/Fornecedor'),
        'fintib_parcela': fields.related(
            'titulo_id', 'parcela', type='integer',
            relation='financeiro.titulo', string='Parcela'),
    }

    _defaults = {
        'active': lambda *a: True,
    }


