# -*- encoding: utf-8 -*-
##############################################################################
#                                                                            #
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca        #
#                    www.siginformatica.com.br luiz@siginformatica.com.br    #
#                                                                            #
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.sql import drop_view_if_exists


class financeiro_titulo_sintetico_view(osv.Model):
    _name = 'financeiro.titulo.sintetico.view'
    _description = "Financeiro - Posicao Diaria"
    _auto = False
    _order = 'data_vencimento asc'
    _columns = {
        'id': fields.integer(),
        'company_id': fields.many2one("res.company", 'Empresa'),
        'tipo_titulo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo titulo', readonly=True),
        'data_vencimento': fields.date('Vencimento'),
        'vr_receber': fields.float('Valor receber', digits=(15, 2)),
        'vr_pagar': fields.float('Valor pagar', digits=(15, 2)),
        'saldo': fields.float('saldo', digits=(15, 2)),
    }

    def init(self, cr):
        drop_view_if_exists(cr, "financeiro_titulo_sintetico_view")
        cr.execute("""
            CREATE OR REPLACE VIEW financeiro_titulo_sintetico_view AS
             SELECT trunc(EXTRACT(EPOCH FROM financeiro_titulo.data_vencimento)/10000) as id,
                financeiro_titulo.company_id, financeiro_titulo.data_vencimento, financeiro_titulo.tipo_titulo, 0.0 AS saldo,
                SUM(CASE financeiro_titulo.tipo_titulo
                WHEN 'P' THEN 0
                ELSE financeiro_titulo.vr_original
                END )AS vr_receber,
                SUM(CASE financeiro_titulo.tipo_titulo
                WHEN 'P' THEN financeiro_titulo.vr_original
                ELSE 0
                END )AS vr_pagar
                FROM financeiro_titulo
                WHERE financeiro_titulo.state = 'A'
                GROUP BY financeiro_titulo.company_id,financeiro_titulo.data_vencimento,financeiro_titulo.tipo_titulo
                ORDER BY financeiro_titulo.data_vencimento;
        """)

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        sobreescricao do metodo na heranca dessa classe no modulo Tesouraria
        """
        context = context and context.copy() or {}
        contas_ids = self.pool.get("financeiro.tesouraria.contas").search(cr, uid, [('integra_fluxo', '=', True)],
                                                                          context=context)
        res = super(financeiro_titulo_sintetico_view, self).read(cr, uid, ids, fields, context=context, load=load)
        saldo = 0.0
        for x in contas_ids:
            saldo += self.pool.get("financeiro.tesouraria.saldos").get_saldo_do_dia(cr, uid, x,
                                                                                    res[0]["data_vencimento"], context=context)
        inc = 0
        for x in res:
            saldo += x['vr_receber'] - x['vr_pagar']
            res[inc]['saldo'] = saldo
            inc += 1
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        company = self.pool.get('res.users')._get_company(cr, uid)
        ids_empresa = self.pool.get('res.company')._get_company_children(cr, uid, company)
        args += [('company_id', 'in', ids_empresa)]
        res = super(financeiro_titulo_sintetico_view, self).search(cr, uid, args, offset, limit, order, context, count)
        return res
