# -*- encoding: utf-8 -*-
##############################################################################
#                                                                            #
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca        #
#                    www.siginformatica.com.br luiz@siginformatica.com.br    #
#                                                                            #
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.sql import drop_view_if_exists


class financeiro_titulo_view(osv.Model):
    _name = 'financeiro.titulo.view'
    _description = "Financeiro - Posicao Diaria"
    _auto = False
    _order = 'data_vencimento asc'
    _columns = {
        'id': fields.integer(),
        'tipo_titulo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo titulo', readonly=True),
        'company_id': fields.many2one("res.company", 'Empresa'),
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente/Fornecedor'),
        'data_vencimento': fields.date('Vencimento'),
        'documento': fields.char('Documento', size=30),
        'vr_original': fields.float('Valor', digits=(15, 2)),
        'saldo': fields.float('saldo', digits=(15, 2)),
    }

    def init(self, cr):
        drop_view_if_exists(cr, "financeiro_titulo_view")
        cr.execute("""
            CREATE OR REPLACE VIEW financeiro_titulo_view AS
             SELECT financeiro_titulo.id, financeiro_titulo.data_vencimento,
                    financeiro_titulo.company_id, financeiro_titulo.pessoa_id,
                    financeiro_titulo.tipo_titulo,
                    financeiro_titulo.documento, 0.0 AS saldo,
                    CASE financeiro_titulo.tipo_titulo
                        WHEN 'P'::text THEN financeiro_titulo.vr_original * (-1)::numeric
                        ELSE financeiro_titulo.vr_original
                    END AS vr_original
               FROM financeiro_titulo
              WHERE financeiro_titulo.state::text = 'A'::text
              ORDER BY financeiro_titulo.data_vencimento;
        """)

    def read(self, cr, user, ids, fields=None, context=None, load='_classic_read'):
        """
        sobreescricao do metodo na heranca dessa classe no modulo Tesouraria
        """
        context = context and context.copy() or {}
        res = super(financeiro_titulo_view, self).read(cr, user, ids, fields, context=context, load=load)
        saldo = 0.0
        inc = 0
        for x in res:
            saldo += x['vr_original']
            res[inc]['saldo'] = saldo
            inc += 1
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        company = self.pool.get('res.users')._get_company(cr, uid)
        ids_empresa = self.pool.get('res.company')._get_company_children(cr, uid, company)
        args += [('company_id', 'in', ids_empresa)]
        res = super(financeiro_titulo_view, self).search(cr, uid, args, offset, limit, order, context, count)
        return res