# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp import tools


class financeiro_tesouraria_contas(osv.Model):
    _name = 'financeiro.tesouraria.contas'
    _inherit = 'financeiro.tesouraria.contas'


    def _get_contas_filhas(self, cr, uid=None, conta_id=None, context=None):
        """
        Retorna as contas que são filhas da conta que está sendo pesquisada
        @param cr:
        @param uid:
        @param conta_id:
        @return: lista com ids das contas que são filhas, netas, etc da conta informada e o id da propria conta
        """
        if not conta_id:
            return []

        ids = self.search(cr, uid, [('parent_id', 'child_of', [conta_id])], context=context)
        return ids

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        for arg in args:
            if 'Atualizar' in arg:
                self.pool.get('financeiro.tesouraria.saldos').fintsa_recalcular_saldos(cr, uid)
        company = self.pool.get('res.users')._get_company(cr, uid)
        ids_empresa = self.pool.get('res.company')._get_company_children(cr, uid, company)
        args += [('company_id', 'in', ids_empresa)]
        res = super(financeiro_tesouraria_contas, self).search(cr, uid, args, offset=offset, limit=limit, order=order,
                                                               context=context, count=count)
        return res

    def _on_create_write(self, cr, uid, id, context=None):
        context = context and context.copy() or {}
        ml = self.browse(cr, uid, id, context)
        return map(lambda x: x.id, ml.move_id.line_id)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.browse(cr, uid, ids, context=context)
        res = []
        for record in reads:
            banco_caixa = 'Tesouraria'
            if record.banco_caixa == 'C':
                banco_caixa = 'Caixa'
            elif record.banco_caixa == 'B':
                banco_caixa = 'Banco'
            _id = record.apelido + ' - ' + banco_caixa
            res.append((record['id'], _id))
        return res

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        if context.get('wbt_tipo_conta'):
            tipo_conta = context.get('wbt_tipo_conta')
            if isinstance(tipo_conta, (str, unicode)):
                tipo_conta = tipo_conta.split(',')

            args_search = [('banco_caixa', 'in', tipo_conta)]
            if name:
                args_search.append(('apelido', operator, name))

            ids = self.search(cr, uid, args_search + args, limit=limit, context=context)
        elif name:
            ids = self.search(cr, uid, [('apelido', operator, name)] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def fintco_atualizar_saldo_conciliado(self, cr, uid, tc_conta_id, tc_valor, tc_debito_credito, context=None):
        """
        Metodo atualizar_saldo_conciliado
        Permite a atualizacao do atributo "saldo_conciliado".
            - Ler a conta para "tc_conta_id"
            - Verificar se o atributo "tipo" = 'B', do contrario, retornar "-1"
            - se "tc_debito_credito" = "C" somar "tc_valor" ao atributo "saldo_conciliado"
            - se "tc_debito_credito" = "D" subtrair "tc_valor" do atributo "saldo_conciliado"
            - chamar o metodo "write" da classe "financeiro.tesouraria.contas"
            - Qualquer erro retornar "-2"
        """
        context = context and context.copy() or {}
        browse_tesouraria_conta = self.browse(cr, uid, tc_conta_id)
        tipo_conta = browse_tesouraria_conta.banco_caixa
        result = -1
        # VERIFICA SE A CONTA É DO TIPO BANCO
        if tipo_conta == 'B':
            if tc_debito_credito == 'C':
                valor = browse_tesouraria_conta.fintco_saldo_conciliado + tc_valor
            elif tc_debito_credito == 'D':
                valor = browse_tesouraria_conta.fintco_saldo_conciliado - tc_valor
            if tc_debito_credito in ['C', 'D']:
                result = self.write(cr, uid, tc_conta_id, {'fintco_saldo_conciliado': valor}, context)
        return result

    def _verifica_recursao(self, cr, uid, ids):
        level = 100
        while len(ids):
            cr.execute("select distinct parent_id from financeiro_tesouraria_contas where id in (" + ','.join(
                map(str, ids)) + ')')
            ids = filter(None, map(lambda x: x[0], cr.fetchall()))
            if not level:
                return False
            level -= 1
        return True

    _constraints = [
        (_verifica_recursao, 'Erro! Voce nao pode criar uma conta recursiva!', ['parent_id'])
    ]

    def fields_view_get(self, cr, user, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        res = super(financeiro_tesouraria_contas, self).fields_view_get(cr, user, view_id=view_id, view_type=view_type,
                                                                        context=context, toolbar=toolbar, submenu=submenu)
        return res

