# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.osv.orm import except_orm
from openerp.tools.translate import _
from openerp import SUPERUSER_ID


class financeiro_tesouraria_movimento(osv.Model):

    _inherit = 'financeiro.tesouraria.movimento'  # fintem

    def name_get(self, cr, uid, ids, context=None):
        res = []
        context = context and context.copy() or {}
        if not len(ids):
            return []
        dados = self.read(cr, uid, ids, ['name'], context=context)
        for x in dados:
            if context.get('fintem_so_codigo'):
                res.append((x['id'], str(x['id'])))
            else:
                res.append((x['id'], x['name']))
        return res


    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(financeiro_tesouraria_movimento, self).default_get(cr, uid, fields, context)
        financeiro_parametro_id = self.pool.get('financeiro.parametros').search(cr, uid, [], context=context)
        parametro_centro_custo_evento = self.pool.get('financeiro.parametros').read(cr, uid, financeiro_parametro_id,
                                                                                    ['usar_centro_custos_e_eventos'], context=context)
        if isinstance(parametro_centro_custo_evento, list):
            parametro_centro_custo_evento = parametro_centro_custo_evento[0]
        parametro_centro_custo_evento = parametro_centro_custo_evento.get('usar_centro_custos_e_eventos')
        if 'parametro_centro_custo_evento' in res:
            res['parametro_centro_custo_evento'] = parametro_centro_custo_evento

        parametro_esconde_forma_pgto = self.pool.get('financeiro.parametros').read(cr, uid, financeiro_parametro_id,
                                                                                   ['finpar_esconde_forma_pgto'], context=context)
        if isinstance(parametro_esconde_forma_pgto, list):
            parametro_esconde_forma_pgto = parametro_esconde_forma_pgto[0]
        parametro_esconde_forma_pgto = parametro_esconde_forma_pgto.get('finpar_esconde_forma_pgto')
        if 'parametro_esconde_forma_pgto' in res:
            res['parametro_esconde_forma_pgto'] = parametro_esconde_forma_pgto

        return res

    def onchange_data_movimento(self, cr, user, ids, data_movimento, mes_competencia, ano_competencia):
        if mes_competencia and int(mes_competencia) <= 0 or not mes_competencia:
            mes_competencia = str(data_movimento[5:7])
        if ano_competencia and int(ano_competencia) <= 0 or not ano_competencia:
            ano_competencia = str(data_movimento[0:4])
        return {
            'value': {
                'mes_competencia': mes_competencia.zfill(2),
                'ano_competencia': ano_competencia.zfill(4)
            }
        }

    def _perm_write(self, cr, uid, conta_id, context=None):
        # ID DA EMPRESA
        context = context and context.copy() or {}
        pool = self.pool.get('financeiro.parametros')
        # PARAMETRO ALTERAR MOVIMENTO DA EMPRESA
        par_id = pool.search(cr, uid, [], context=context)
        perm = pool.read(cr, uid, par_id, ['altera_movimento'], context=context)
        if perm:
            res = perm[0]['altera_movimento']
        else:
            res = True
        return res

    def _get_valor_b(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        res = {}
        for id in ids:
            vlr = 0
            o_baixa = self.pool.get('financeiro.tesouraria.baixa.forma.pgto')
            baixas = o_baixa.search(cr, uid, [('tesouraria_movimento_id', '=', id)], context=context)
            for baixa in baixas:
                vlr += o_baixa.read(cr, uid, baixa, ['valor'], context=context)['valor']
            res[id] = vlr
        return res

    def _perm_unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        msg = _('Este movimento nao pode ser excluido devido restricao de permissao!')
        if len(ids) > 1:
            msg = _('Alguns dos movimentos nao podem ser excluidoss devido restricoes de permissao!')

        pool = self.pool.get('financeiro.parametros')
        for id in self.browse(cr, uid, ids):
            company_id = id.conta_id.company_id.id
            if pool.search(cr, uid, [('company_id', '=', company_id), ('exclusao_movimento', '=', False)],
                           context=context):
                raise osv.except_osv(_('Atencao'), (msg))
        return True

    def create(self, cr, uid, vals, context=None):
        """
        Criar novo registro de movimento
            - Executar metodo "financeiro_tesouraria_saldos.atualizar_saldo" informando os dados vindos do
              dicionario da tela "vals". "valor" deve ser o valor do lancamento
            - executar o metodo create da classe financeiro_tesouraria_movimento
        """
        if vals.get('valor') == 0:
            raise osv.except_osv(_('Atencao'), _('Informe um valor para o movimento'))
        if vals.get('valor') < 0 and 'baixar_titulo' not in context:
            raise osv.except_osv(_('Atencao'), _('Valor invalido para o movimento'))
        self.fintem_verifica_data_conciliacao_create(cr, uid, vals.get('data_movimento'), context)
        context = context and context.copy() or {}
        vals['state'] = 'A'
        res = super(financeiro_tesouraria_movimento, self).create(cr, uid, vals, context)
        self.pool.get('adm.transacao.capa').admtrc_criar_transacao(cr, uid, 'teso', res, context=None)
        self._fintem_incluir_historico_gerencial(cr, uid, res, context)
        obj_financeiro_tesouraria_saldos = self.pool.get('financeiro.tesouraria.saldos')
        obj_financeiro_tesouraria_saldos.atualizar_saldo(
            cr, uid, vals['conta_id'], vals['data_movimento'], vals['debito_credito'],
            vals['valor'], context)
        ################   Criar Registro para a Conta Destino    #####
        vals_destino = vals.get('conta_destino_id', False)
        vals_origem = vals['conta_id']
        vals_tipo = vals['debito_credito']
        if vals_destino:
            vals_novo = vals.copy()
            #        Inverte o Tipo Credito / Debito
            vals['debito_credito'] = 'D' if vals_tipo == 'C' else 'C'
            #        Inverte as Contas Origem / Destino
            vals['conta_destino_id'] = vals_origem
            vals['conta_id'] = vals_destino
            vals['movimento_destino_id'] = res
            movimento_destino_id = super(financeiro_tesouraria_movimento, self).create(
                cr, uid, vals, context)
            self.pool.get('financeiro.tesouraria.saldos').atualizar_saldo(
                cr, uid, vals['conta_id'], vals['data_movimento'], vals['debito_credito'],
                vals['valor'])
            super(financeiro_tesouraria_movimento, self).write(
                cr, uid, [res], {'movimento_destino_id': movimento_destino_id}, context)
        if not any([context.get('financ_baixar_titulo'), context.get('ecf_tesouraria_capa'),
                    context.get('validacao_cupom'), context.get('cancelar_nota_fiscal'),
                    context.get('financ_estorno_baixa')]):
            self._fintem_verificar_custificacao(cr, uid, res, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        """
        Atualizar o registro de movimento
            1 - Se foi alterado a conta_destino_id e/ou a data e/ou o valor: (dicionario "vals")
                1.2 - Ler lancamento original
                1.3 - executar o metodo "financeiro_tesouraria_saldos.atualizar_saldo" informando:
                    (para deduzir o valor ja registrado)
                    1.3.1 - "ts_conta_destino_id" = "conta_destino_id" lida
                    1.3.2 - "ts_data" = "data_movimento" lido
                    1.3.3 - "ts_valor" = "valor" lido
                    1.3.4 - para o atributo "conciliado"  lido = True:
                        1.3.4.1 - para "debito_credito" lido:
                            1.3.4.1.1 - igual a 'D' chamar metodo "financeiro.tesouraria.contas.atualizar_saldo_conciliado" informando:
                                       'conta_destino_id' lida, "valor" lido, "C"
                            1.3.4.1.2 - igual a 'C' chamar o metodo "financeiro.tesouraria.contas.atualizar_saldo_conciliado" informando:
                                       'conta_destino_id' lida, 'valor' lido, "D"
                    1.3.5 - se "debito_credito" lido:
                        1.3.5.1 - igual a 'C', "ts_debito_credito" = "D"
                        1.3.5.2 - igual a 'D', "ts_debito_credito" = "C"
                1.4 - executar o metodo "financeiro_tesouraria_saldos.atualizar_saldo" informando:
                      (para registrar o novo valor ou nova conta/data)
                      Caso algum dos campos do registro nao tenham sido alterados
                      (nao estao no dicionario "vals"), considerar o valor original do lancamento
                    1.4.1 - "ts_conta_destino_id" = "conta_destino_id" digitada(do dicionario vals)
                            se existir, do contrario usar "conta_destino_id" lido
                    1.4.2 - "ts_data" = "data_movimento" digitado se existir, do contrario
                            usar "data_movimento" lido
                    1.4.3 - "ts_valor" = "valor" digitado se existir, do contrario
                            usar "valor" lido
                    1.4.4 - para o atributo "conciliado" digitado se existir, (do contrario
                            considerar "conciliado" lido), igual a true:
                        1.4.4.1 - chamar o metodo "financeiro.tesouraria.contas.atualizar_saldo_conciliado" informando:
                                  "conta_destino_id" digitada, "valor" digitado, "debito_credito" digitado
                    1.4.5 - "ts_debito_credito" = "debito_credito" digitado se existir,
                            do contrario usar "debito_credito" lido:
            2 - executar o metodo write da classe financeiro_tesouraria_movimento
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        write_force = context.get('write_force')
        obj_financeiro_par = self.pool.get('financeiro.parametros')
        obj_financeiro_tesouraria_saldos = self.pool.get('financeiro.tesouraria.saldos')
        parametro = obj_financeiro_par.search(cr, uid, [], context=context)
        if not parametro:
            raise osv.except_osv(_('Atencao'), _('Nao existe parametro cadastrado para o Financeiro.'))
        else:
            altera_movimento = obj_financeiro_par.browse(cr, uid, parametro[0]).altera_movimento

        view_name = context.get('view_name')
        if view_name == 'financeiro.tesouraria.movimento' and not altera_movimento:
            if not write_force:
                raise osv.except_osv(_('Atencao'), _('Nao e permitido a alteracao do movimento.'))

        baixa_forma_pgto_ids, gerencial_movimento_ids = [], []
        if vals.get('baixa_forma_pgto_ids'):
            baixa_forma_pgto_ids = vals.get('baixa_forma_pgto_ids')
        if vals.get('gerencial_movimento_ids'):
            gerencial_movimento_ids = vals.get('gerencial_movimento_ids')
            #
        # VALORES ATUAIS
        #
        vals_atual = self.read(
            cr, uid, ids, ['company_id', 'conta_id', 'conta_destino_id', 'data_movimento',
                           'debito_credito', 'conciliado', 'valor', 'historico',
                           'movimento_destino_id'], context=context)
        self.fintem_verifica_data_conciliacao_write(
            cr, uid, vals.get('data_movimento') or vals_atual[0].get('data_movimento'), context)
        conta_id_atual = vals_atual[0]['conta_id'][0]
        conta_destino_id_atual = vals_atual[0]['conta_destino_id'] and vals_atual[0]['conta_destino_id'][0]
        data_atual = vals_atual[0]['data_movimento']
        dc_atual = vals_atual[0]['debito_credito']
        conc_atual = vals_atual[0]['conciliado']
        vlr_atual = vals_atual[0]['valor']
        dc_inv = 'D' if dc_atual == 'C' else 'C'
        #        Busca contra-conta
        movimento_destino_id = vals_atual[0].get('movimento_destino_id')
        if not movimento_destino_id:
            movimento_destino_ids = self.search(
                cr, uid, [('conta_id', '=', conta_destino_id_atual),
                          ('conta_destino_id', '=', conta_id_atual),
                          ('data_movimento', '=', data_atual), ('debito_credito', '=', dc_inv),
                          ('valor', '=', vlr_atual)], )
            if movimento_destino_ids:
                movimento_destino_id = movimento_destino_ids[0]
        #
        # VALORES NOVOS
        #
        result = super(financeiro_tesouraria_movimento, self).write(cr, uid, ids, vals, context)
        vals_novo = self.read(cr, uid, ids,
                              ['company_id', 'conta_id', 'conta_destino_id', 'data_movimento',
                               'debito_credito', 'conciliado', 'valor', 'historico',
                               'mes_competencia', 'ano_competencia', 'movimento_destino_id'], context=context)
        if not altera_movimento and not write_force:
            if vals_atual != vals_novo:
                raise osv.except_osv(_('Atencao'), _('Nao e permitido a alteracao do movimento.'))
            if baixa_forma_pgto_ids:
                if isinstance(baixa_forma_pgto_ids[0], tuple):
                    if baixa_forma_pgto_ids[0][0] == 1:
                        raise osv.except_osv(_('Atencao'), _('Nao e permitido a alteracao do movimento.'))
            if gerencial_movimento_ids:
                if isinstance(gerencial_movimento_ids[0], tuple):
                    if gerencial_movimento_ids[0][0] == 1:
                        raise osv.except_osv(_('Atencao'), _('Nao e permitido a alteracao do movimento.'))

        vlr_novo = vals_novo[0]['valor']
        conta_id_novo = vals_novo[0]['conta_id'][0]
        conta_destino_id_novo = vals_novo[0]['conta_destino_id'] and vals_novo[0]['conta_destino_id'][0]
        data_novo = vals_novo[0]['data_movimento']
        dc_novo = vals_novo[0]['debito_credito']
        if isinstance(movimento_destino_id, (list, tuple)):
            movimento_destino_id = movimento_destino_id[0]
        if not conta_destino_id_atual and conta_destino_id_novo:
            vals_movimento_contra_conta = vals_novo[0].copy()
            #        Inverte o Tipo Credito / Debito
            vals_movimento_contra_conta['debito_credito'] = 'D' if dc_novo == 'C' else 'C'
            #        Inverte as Contas Origem / Destino
            vals_movimento_contra_conta['conta_destino_id'] = conta_id_novo
            vals_movimento_contra_conta['conta_id'] = conta_destino_id_novo
            vals_movimento_contra_conta['movimento_destino_id'] = ids[0]
            del vals_movimento_contra_conta['id']
            movimento_destino_id = super(financeiro_tesouraria_movimento, self).create(
                cr, uid, vals_movimento_contra_conta, context)
            self.pool.get('financeiro.tesouraria.saldos').atualizar_saldo(
                cr, uid, vals_movimento_contra_conta['conta_id'],
                vals_movimento_contra_conta['data_movimento'],
                vals_movimento_contra_conta['debito_credito'],
                vals_movimento_contra_conta['valor'])
            self.write(cr, uid, ids, {'movimento_destino_id': movimento_destino_id}, context)
            self.write(cr, uid, [movimento_destino_id], {'movimento_destino_id': ids[0]}, context)
        elif conta_destino_id_atual and not conta_destino_id_novo:
            self.unlink(cr, uid, movimento_destino_id, context)
            movimento_destino_id = False
        elif conta_destino_id_atual and conta_destino_id_novo and conta_destino_id_atual != \
                conta_destino_id_novo:
            vals_movimento_contra_conta = vals_novo[0].copy()
            #        Inverte o Tipo Credito / Debito
            vals_movimento_contra_conta['debito_credito'] = 'D' if dc_novo == 'C' else 'C'
            if dc_atual != dc_novo:
                vals_movimento_contra_conta['debito_credito'] = 'D' if dc_novo == 'C' else 'C'
            vals_movimento_contra_conta['conta_id'] = conta_destino_id_novo
            vals_movimento_contra_conta['conta_destino_id'] = conta_id_atual
            del vals_movimento_contra_conta['movimento_destino_id']
            vals_movimento_contra_conta['company_id'] = company_id
            del vals_movimento_contra_conta['id']
            self.write(cr, uid, [movimento_destino_id], vals_movimento_contra_conta, context)
        #
        # ATUALIZAR SALDO
        #
        if (conta_id_atual, data_atual, vlr_atual) != (conta_id_novo, data_novo, vlr_novo):
            dc = 'C' if dc_atual == 'D' else 'D'
            valor = vlr_atual if vlr_atual == vlr_novo else vlr_atual - vlr_novo
            obj_financeiro_tesouraria_saldos.atualizar_saldo(cr, uid, conta_id_atual, data_atual, dc,
                                                                          valor)
            conta_id = conta_id_atual if conta_id_atual == conta_id_novo else conta_id_novo
            data = data_atual if data_atual == data_novo else data_novo
            dc = dc_atual if dc_atual == dc_novo else dc_novo
            valor = vlr_atual if vlr_atual == vlr_novo else vlr_novo
            if conta_id_atual != conta_id_novo:
                if dc_atual == 'D':
                    dc = 'D'
                else:
                    dc = 'C'
                obj_financeiro_tesouraria_saldos.atualizar_saldo(cr, uid, conta_id_atual, data_atual, dc,
                                                                              vlr_atual)
            if conc_atual:
                valor = vlr_atual if vlr_atual == vlr_novo else vlr_novo - vlr_atual
                self.pool.get('financeiro.tesouraria.contas').fintco_atualizar_saldo_conciliado(cr, uid, conta_id_atual, valor,
                                                                                         dc)
            obj_financeiro_tesouraria_saldos.atualizar_saldo(cr, uid, conta_id, data, dc, valor)
        if ((conta_id_atual, data_atual, vlr_atual) == (conta_id_novo, data_novo, vlr_novo)) and (dc_atual != dc_novo):
            dc = dc_atual if dc_atual == dc_novo else dc_novo
            obj_financeiro_tesouraria_saldos.atualizar_saldo(cr, uid, conta_id_atual, data_atual, dc,
                                                                          vlr_atual * 2)
        if context.get('update', False):
            self.pool.get('financeiro.tesouraria.contas').fintco_atualizar_saldo_conciliado(cr, uid, conta_id_atual, vlr_atual,
                                                                                     dc_atual)
            #        Atualiza Contra-Conta
        if movimento_destino_id and not context.get('contra_conta', False) and not context.get('conciliado', False):
            vals['valor'] = vlr_novo
            vals['conta_destino_id'] = conta_id_novo
            vals['conta_id'] = conta_destino_id_novo
            vals['data_movimento'] = data_novo
            vals['debito_credito'] = 'D' if dc_novo == 'C' else 'C'
            context['contra_conta'] = True
            super(financeiro_tesouraria_movimento, self).write(cr, uid, [movimento_destino_id],
                                                               vals, context)
        if not context.get('conciliado'):
            self._fintem_verificar_custificacao(cr, uid, ids, context)
            self._fintem_incluir_historico_gerencial(cr, uid, ids, context)
        return result

    def unlink(self, cr, uid, ids, context=None):
        """Excluir o movimento selecionado e atualizar o saldo
             1 - para cada registro a ser excluido:
             1.1 - executar o metodo read da classe financeiro_tesouraria_movimento
             1.2 - executar o metodo financeiro_tesouraria_saldos.atualizar_saldo informando:
                   1.2.1 - "ts_conta_destino_id" = "conta_destino_id"
                   1.2.2 - "ts_data" = "data_movimento"
                   1.2.3 - para atributo "debito_credito":
                           1.2.3.1 - = 'C' "ts_debito_credito="D"
                           1.2.3.2 - = "D" "ts_debito_credito="C"
                   1.2.4 - "ts_valor" = "valor"
             1.3 - para atributo "conciliado" igual a True:
                   1.3.1 - Para atributo "debito_credito"
                   1.3.1.1 - = 'C' chamar o metodo "financeiro.tesouraria.contas.atualizar_saldo_conciliado" informando:
                               ' "conta_destino_id", "valor", "D"
                   1.3.1.2 - = 'D' chamar o metodo "financeiro.tesouraria.contas.atualizar_saldo_conciliado" informando:
                               ' "conta_destino_id", "valor", "C"
                   1.3.2 - Atualizar a classe "financeiro.tesouraria.saldos"
             2 - executar o metodo unlink da classe financeiro_tesouraria_movimento

             OBS. Se precisar excluir um movimento e nao for permitido do parametro passe por
                  contexto {'exluir_movimento: True}"""
        context = context and context.copy() or {}
        if not ids:
            return True

        if isinstance(ids, (int, long)):
            ids = [ids]
        if not self.search(cr, uid, [('id', 'in', ids)], context=context):
            raise osv.except_osv(_('Atencao'), _('Movimento ja excluido!'))
        movimentos_conciliacao = self.fintem_verifica_conciliacao(cr, uid, ids, context)
        if any(movimentos_conciliacao.values()):
            movimento_conciliado = filter(lambda mc: movimentos_conciliacao.get(mc),
                                          movimentos_conciliacao)
            movimento_conciliado.sort()
            raise osv.except_osv(
                _('Atencao'),
                _('Os movimentos a seguir estao conciliados. Realize a desconciliacao desses para '
                  'exclui-los:\n' + ', \n'.join(map(lambda ftm: str(ftm), movimento_conciliado))))
        financeiro_parametros = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', 'exclusao_movimento', context=context)
        if financeiro_parametros:
            if not financeiro_parametros.get('exclusao_movimento') and not context.get(
                    'excluir_movimento'):
                raise osv.except_osv(_('Atencao'), _('Nao e permitido a exclusao do movimento.'))
        else:
            raise osv.except_osv(_('Atencao'), _('Nao existe parametro cadastrado para o Financeiro.'))

        obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
        obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
        obj_financeiro_tesouraria_saldos = self.pool.get('financeiro.tesouraria.saldos')
        financeiro_tesouraria_movimento_dados = self.read(cr, uid, ids,
                                                          ['conta_id', 'conta_destino_id', 'data_movimento', 'valor',
                                                           'debito_credito', 'conciliado'], context=context)
        self.fintem_verifica_data_conciliacao_unlink(
            cr, uid, financeiro_tesouraria_movimento_dados[0].get('data_movimento'), context)
        # Procura o registro da conta destino para atualizar o saldo
        conta_destino_mov_ids = self.search(
            cr, uid, [('movimento_destino_id', 'in', ids)], context=context)

        res = super(financeiro_tesouraria_movimento, self).unlink(cr, uid, ids, context)
        if isinstance(financeiro_tesouraria_movimento_dados, list):
            for i in financeiro_tesouraria_movimento_dados:
                # VERIFICA SE HA TITULOS PARA O MOVIMENTO
                finan_titulo_baixa_id = obj_financeiro_titulo_baixa.search(cr, uid, [
                    ('tesouraria_movimento_id', '=', i.get('id') or 0)], context=context)
                if finan_titulo_baixa_id:
                    if len(finan_titulo_baixa_id) > 1:
                        msg = _('Movimento (id = %d) nao foi excluido. Ha %d Titulos relacionados a ele.') % (
                            i.get('id') or 0, len(finan_titulo_baixa_id))
                    else:
                        msg = _('Movimento (id = %d) nao foi excluido. Ha %d Titulo relacionado a ele.') % (
                            i.get('id') or 0, len(finan_titulo_baixa_id))
                    raise except_orm(_('Atencao'), msg)
                try:
                    conta_destino_id = i['conta_destino_id'][0]
                except:
                    conta_destino_id = 0
                try:
                    conta_id = i['conta_id'][0]
                except:
                    conta_id = 0
                valor = i.get('valor') or 0
                debito_credito = i.get('debito_credito')
                conciliado = i.get('conciliado')
                data_movimento = i.get('data_movimento') or ''
                debito_credito_old = 'D' if debito_credito == 'C' else 'C'
                # ATUALIZA O SALDO
                obj_financeiro_tesouraria_saldos.atualizar_saldo(cr, uid, conta_id, data_movimento, debito_credito_old,
                                                                 valor, context=context)
                # SE O MOVIMENTO ESTIVER CONCILIADO E O TIPO DA CONTA FOR CAIXA(tipo da conta e verificado na funcao), ATUALIZA O SALDO CONCILIADO
                if conciliado:
                    obj_financeiro_tesouraria_contas.fintco_atualizar_saldo_conciliado(cr, uid, conta_id, valor,
                                                                                debito_credito_old, context=context)

            if conta_destino_mov_ids:
                financeiro_tesouraria_movimento_destino_dados = self.read(cr, uid, conta_destino_mov_ids,
                                                                          ['conta_id', 'conta_destino_id',
                                                                           'data_movimento', 'valor', 'debito_credito',
                                                                           'conciliado'], context=context)
                if isinstance(financeiro_tesouraria_movimento_destino_dados, list):
                    for i in financeiro_tesouraria_movimento_destino_dados:
                        try:
                            conta_id = i['conta_id'][0]
                        except:
                            conta_id = 0
                        valor = i.get('valor') or 0
                        debito_credito = i.get('debito_credito')
                        conciliado = i.get('conciliado')
                        data_movimento = i.get('data_movimento') or ''
                        debito_credito_old = 'D' if debito_credito == 'C' else 'C'
                        # ATUALIZA O SALDO
                        obj_financeiro_tesouraria_saldos.atualizar_saldo(cr, uid, conta_id, data_movimento,
                                                                         debito_credito_old, valor, context=context)
                        # SE O MOVIMENTO ESTIVER CONCILIADO E O TIPO DA CONTA FOR CAIXA(tipo da conta e verificado na funcao), ATUALIZA O SALDO CONCILIADO
                        if conciliado:
                            obj_financeiro_tesouraria_contas.fintco_atualizar_saldo_conciliado(cr, uid, conta_id, valor,
                                                                                        debito_credito, context=context)
                    ids += conta_destino_mov_ids
                self.unlink(cr, uid, conta_destino_mov_ids, context)
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        if not self.search(cr, uid, [('id', 'in', ids if isinstance(ids, list) else [ids])],
                           context=context) and context.get('fintem_views'):
            if isinstance(ids, list):
                return []
            if isinstance(ids, (int, long)):
                return {}
        return super(financeiro_tesouraria_movimento, self).read(cr, uid, ids, fields, context,
                                                                 load)

    def button_atualizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        if uid == SUPERUSER_ID: # para o usuario administrador não pode observar a company
            conta_model_id = self.pool.get('ir.model').search(
                cr, uid, [('model', '=', 'financeiro.tesouraria.contas')])
            if not self.pool.get('adm.empresa.objeto').search(
                    cr, uid, [('ir_model_id', 'in', conta_model_id)]):
                context['sig_force_company'] = True
        conta_id_args = any(map(lambda i: 'conta_id' in i, args))
        if not conta_id_args:
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                        context=context)
            conta_ids = self.pool.get('financeiro.tesouraria.contas').search(
                cr, uid, [('company_id', '=', company_id)], context=context)
            args.append(('conta_id', 'in', conta_ids))
        res = super(financeiro_tesouraria_movimento, self).search(cr, uid, args, offset, limit, order, context, count)
        return res

    def fintem_onchange_operacoes(self, cr, uid, ids, fintem_operacoes_id, parametro_centro_custo_evento, valor,
                                  context=None):
        context = context and context.copy() or {}
        res = {}
        obj_financeiro_operacoes = self.pool.get('financeiro.operacoes')
        obj_gerencial_movimento = self.pool.get('gerencial.movimento')
        if ids:
            gerencial_movimento_ids = self.browse(cr, uid, ids[0]).gerencial_movimento_ids
            if gerencial_movimento_ids:
                gerencial_movimento_ids = map(lambda i: i.id, gerencial_movimento_ids)
                if gerencial_movimento_ids:
                    obj_gerencial_movimento.unlink(cr, uid, gerencial_movimento_ids)
        if fintem_operacoes_id and parametro_centro_custo_evento:
            browse_financeiro_operacoes = obj_financeiro_operacoes.browse(cr, uid, fintem_operacoes_id)
            centro_custo_id = browse_financeiro_operacoes.finope_centro_custo_id.id or False
            evento_id = browse_financeiro_operacoes.finope_evento_id.id or False
            valor = valor or 0
            if evento_id and centro_custo_id:
                vals = {
                    'centro_custo_id': centro_custo_id,
                    'evento_id': evento_id,
                    'valor': valor
                }
                res['value'] = {'gerencial_movimento_ids': [(0, 0, vals)]}
        return res

    def _fintem_verificar_custificacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        #Todo: verificar onde contextos estao sendo testados para nao verificar a custificacao e
        #Todo: substuir por este contexto
        if context.get('fintem_nao_verificar_custificacao'):
            return True

        obj_adm_utils = self.pool.get('adm.utils')
        parametro_financeiro = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'financeiro', 'usar_centro_custos_e_eventos', context=context)
        _usar_centro_custos = parametro_financeiro.get('usar_centro_custos_e_eventos')
        if not _usar_centro_custos:
            return True
        if not isinstance(ids, list):
            ids = [ids]
        for brw_tesouraria_movimento in self.browse(cr, uid, ids, context=context):
            _valor_movimento = brw_tesouraria_movimento.valor or 0
            _valor_custos = 0
            for brw_gerencial_movimento in brw_tesouraria_movimento.gerencial_movimento_ids:
                _valor_custos += brw_gerencial_movimento.valor
            # Qdo houver conta de destino e os custos forem zero, não tem erro.
            if brw_tesouraria_movimento.conta_destino_id:
                if _valor_custos == 0:
                    return True
                else:
                    raise osv.except_osv(_('Atencao'), _('Lancamento com conta de destino, nao pode ter custos!'))
            if round(_valor_movimento, 2) != round(_valor_custos, 2):
                raise osv.except_osv(_('Atencao'), _('Soma dos custos nao confere com o valor do lancamento!'))
        return True

    def _fintem_incluir_historico_gerencial(self, cr, uid, ids, context):
        if not isinstance(ids, list):
            ids = [ids]
        for brw_tesouraria_movimento in self.browse(cr, uid, ids, context=context):
            _historico_tesouraria = brw_tesouraria_movimento.historico or False
            vals = {'fingmo_historico': _historico_tesouraria}
            _gerencial_movimento_ids = []
            for brw_gerencial_movimento in brw_tesouraria_movimento.gerencial_movimento_ids:
                _gerencial_movimento_ids.append(brw_gerencial_movimento.id)
            if _gerencial_movimento_ids:
                self.pool.get('gerencial.movimento').write(cr, uid, _gerencial_movimento_ids, vals, context=context)
        return True

    def fintem_verifica_conciliacao(self, cr, uid, ids, context=None):
        """
        O metodo identifica se os movimentos da tesouraria, ou movimento destino desses, esta
        conciliado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Dados da identificao do movimento e o respectivo valor, sendo verdadeiro, caso o
        movimento esteja conciliado, senao, falso.
        :rtype: bool
        """
        context = context.copy() or {}
        brw_tesouraria_movimento = self.browse(cr, uid, ids, context)
        res = {}
        for movimento in brw_tesouraria_movimento:
            movimento_id = movimento.id
            conciliado = movimento.conciliado
            if movimento.movimento_destino_id:
                conciliado = movimento.movimento_destino_id.conciliado
                movimento_id = movimento.movimento_destino_id.id
            res.update({movimento_id: conciliado})
        return res

    def fintem_verifica_data_conciliacao_create(self, cr, uid, data_movimento, context=None):
        """
        Metodo invocado na criacao do movimento da tesouraria. Baseado na data de movimento, e
        verificado se está dentro do prazo para conciliacao.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param data_movimento: data de movimento da tesouraria
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :rtype: bool
        """
        context = context and context.copy() or {}
        if not self.fintem_verifica_data_conciliacao(cr, uid, data_movimento, context):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel lancar na tesouraria nesta data. '
                                                 'Este dia ja esta conciliado.'))

    def fintem_verifica_data_conciliacao_write(self, cr, uid, data_movimento, context=None):
        """
        Metodo invocado na atualizacao do movimento da tesouraria. Baseado na data de movimento, e
        verificado se está dentro do prazo para conciliacao.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param data_movimento: data de movimento da tesouraria
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :rtype: bool
        """
        context = context and context.copy() or {}
        if not self.fintem_verifica_data_conciliacao(cr, uid, data_movimento, context):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel alterar lancamentos na tesouraria '
                                                 'nesta data. Este dia ja esta conciliado.'))

    def fintem_verifica_data_conciliacao_unlink(self, cr, uid, data_movimento, context=None):
        """
        Metodo invocado na exclusao do movimento da tesouraria. Baseado na data de movimento, e
        verificado se está dentro do prazo para conciliacao.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param data_movimento: data de movimento da tesouraria
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :rtype: bool
        """
        context = context and context.copy() or {}
        if not self.fintem_verifica_data_conciliacao(cr, uid, data_movimento, context):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel excluir lancamentos na tesouraria '
                                                 'nesta data. Este dia ja esta conciliado.'))

    def fintem_verifica_data_conciliacao(self, cr, uid, data, context=None):
        """
        O metodo basea-se na data de movimento da tesouraria para verificar se está dentro do prazo
        para conciliacao.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param data: data de movimento da tesouraria
        :type data: str
        :param context: dados contextuais
        :type context: dict
        :rtype: bool
        """
        context = context and context.copy() or {}
        data_conciliacao = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', 'finpar_data_conciliacao', context=context).get(
            'finpar_data_conciliacao')
        if not data_conciliacao or not data:
            return True
        if data <= data_conciliacao:
            return False
        return True
