# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp import SUPERUSER_ID
import time
from ..model.funcoes import fintsa_get_saldo_final

class financeiro_tesouraria_saldos(osv.Model):
    """
    Tabela que armazena os saldos diarios das contas
        Obs.: nao recuperar o saldo direto do metodo read,
              usar o metodo get_saldo_do_dia
    """
    _name = 'financeiro.tesouraria.saldos'
    _inherit = 'financeiro.tesouraria.saldos'

    def atualizar_saldo(self, cr, uid, ts_conta_id, ts_data, ts_debito_credito, ts_valor=0.00, context=None):
        """
        Este metodo tem a funcao de promover a atualizacao dos saldos.
            - Recuperar o "saldo" para a "ts_conta_id" e "ts_data" informados
            - Caso nao exista registro:
                - recuperar o "saldo" para "ts_conta_id" na data imediatamente anterior a "ts_data".
                    - se nao existir tambem:
                        - criar saldo para "ts_conta_id" e "ts_data", com o "saldo" igual a 0.00
                    - se existir
                        - criar saldo para "ts_conta_id" e "ts_data", com o "saldo" igual "saldo"
                          lidos para a data imediatamente anterior a "ts_data"
                          (que foi lida no passo anterior)
            - Adicionar "ts_valor" ao "saldo" caso "ts_debito_credito" seja = "C"
            - Subtrair "ts_valor" do "saldo" caso  "ts_debito_credito" seja = "D"
            - executar o metodo "write" da classe "financeiro.tesouraria.saldos"
            - Recuperar o "saldo" para a conta informada ("ts_conta_id") em todas as datas
              posteriores a "ts_data"
            - para cada registro recuperado
                - Adicionar "ts_valor" ao "saldo" caso "ts_debito_credito" seja = "C"
                - Subtrair "ts_valor" do "saldo"  caso "ts_debito_credito" seja = "D"
                - executar o metodo "write" da classe "financeiro.tesouraria.saldos"
            - Retornar True, caso nao tenha acontecido nenhum erro
            - Retornar False, caso aconteca qualquer erro.
            - Obs.: Caso a conta informada possua contas filhas, deve ser feita a mesma
                    operacao que foi feita na conta original em todas as contas filhas.
        ------------------------------------------------------------------------
        """
        context = context and context.copy() or {}
        ts_id = self.search(cr, uid, [('conta_id', '=', ts_conta_id),
                                      ('data_movimento', '=', ts_data)], context=context)
        # Verifica se existe conta para a data passada
        if ts_id:
            self.write(cr, uid, ts_id, {}, context)
            ts_saldo = self.read(cr, uid, ts_id[0], ['fintsa_saldo_final'], context=context)[
                'fintsa_saldo_final']
        else:
            ts_id = self.search(cr, uid, [('conta_id', '=', ts_conta_id),
                                          ('data_movimento', '<', ts_data)],
                                order='data_movimento desc', limit=1, context=context)
            if ts_id:
                # Criar novo com o saldo da data anterior + ts_valor
                ts_saldo = self.read(cr, uid, ts_id[-1], ['fintsa_saldo_final'],
                                     context=context).get('fintsa_saldo_final') or 0.00
                saldo_inicial = ts_saldo
                if ts_debito_credito == 'C':
                    ts_saldo = float(ts_saldo) + float(ts_valor)
                elif ts_debito_credito == 'D':
                    ts_saldo = float(ts_saldo) - float(ts_valor)
                self.create(cr, uid, {'conta_id': ts_conta_id, 'data_movimento': ts_data,
                                      'fintsa_saldo_inicial': saldo_inicial}, context)
            else:
                # Criar novo com saldo passado (ts_valor)
                if ts_debito_credito == 'C':
                    ts_saldo = 0.00 + float(ts_valor)
                elif ts_debito_credito == 'D':
                    ts_saldo = 0.00 - float(ts_valor)
                ts_saldo = "%.2f" % ts_saldo
                self.create(
                    cr, uid, {'conta_id': ts_conta_id, 'data_movimento': ts_data,
                              'fintsa_saldo_inicial': 0}, context)

        # Atualiza saldos posteriores
        ts_ids = self.search(
            cr, uid, [('conta_id', '=', ts_conta_id), ('data_movimento', '>', ts_data)],
            order='data_movimento', context=context)

        # Verifica se ha saldos posteriores
        if ts_ids:
            if context.get('validacao_cupom'):
                if len(ts_ids) == 1:
                    tesouraria_saldo_ids = str(tuple(ts_ids)).replace(',', '')
                else:
                    tesouraria_saldo_ids = str(tuple(ts_ids))
                var = '-'
                if ts_debito_credito == 'C':
                    var = '+'
                sql = """
                    UPDATE
                        financeiro_tesouraria_saldos
                    SET
                        fintsa_saldo_inicial = coalesce(fintsa_saldo_inicial, 0) %s %f,
                        fintsa_saldo_final = coalesce(fintsa_saldo_final, 0) %s %f
                    WHERE
                        id in %s
                """ % (var, ts_valor, var, ts_valor, tesouraria_saldo_ids)
                cr.execute(sql)
            else:
                for var_ts_id in ts_ids:
                    saldo_inicial = ts_saldo
                    self.write(
                        cr, uid, var_ts_id, {'fintsa_saldo_inicial': saldo_inicial, }, context)
                    ts_saldo = self.read(cr, uid, var_ts_id, ['fintsa_saldo_final'], context).get(
                        'fintsa_saldo_final') or 0.0
        contas_filhas = self.pool.get('financeiro.tesouraria.contas').search(
            cr, uid, [('parent_id', '=', ts_conta_id)], context=context)

        # Se houver contas filhas, atualiza
        for id in contas_filhas:
            self.atualizar_saldo(cr, uid, id, ts_data, ts_debito_credito, ts_valor)

        return True

    def fintsa_recalcular_saldos(self, cr, uid, context=None):
        """
        RECOMPONHE O SALDO FUTURO PARA QUANDO HOUVER DELECAO NO MOVIMENTO.
        Não usar esta função em processos normais, porque é muito lenta. Ela só é chamada quando
        na tela de conta usa-se a palavra 'Atualizar' na pesquisa.
        """
        context = context and context.copy() or {}
        context['sig_force_company'] = True # para não trocar a company, porque no create do movimento vai executar o método funcional e quem executa os metodos funcionais é o usuario 1 = SUPERUSER_ID
        sql = "delete from financeiro_tesouraria_saldos"
        cr.execute(sql)
        sql = """select conta_id, data_movimento,
                     sum(case when debito_credito = 'C' then valor else valor * -1 end) as saldo,
                     sum(case when conciliado is True then
                          (case when debito_credito = 'C' then valor else valor * -1 end)
                        else 0.00 end) as conciliado
                     from financeiro_tesouraria_movimento
                     group by conta_id, data_movimento
                     order by conta_id, data_movimento"""
        cr.execute(sql)
        _conta_id = 0
        _saldos_ids = []
        _contas_ids = {}
        for _registro in cr.fetchall():
            if _conta_id != _registro[0]:
                _valor_inicial = 0
            _conta_id = _registro[0]
            if not _conta_id in _contas_ids.keys():
                _contas_ids[_conta_id] = 0.00
            vals = {
                'conta_id': _registro[0],
                'data_movimento': _registro[1],
                'fintsa_saldo_inicial': _valor_inicial,
            }
            _saldo_id = self.create(cr, uid, vals, context=context)
            _saldos_ids.append(_saldo_id)
            _valor_inicial += _registro[2]
            _contas_ids[_conta_id] += _registro[3]
        for _conta_id in _contas_ids.keys():
            self.pool.get('financeiro.tesouraria.contas').write(cr, uid, _conta_id,
                                                       {'fintco_saldo_conciliado': _contas_ids[_conta_id]},
                                                       context=context)
        return True

    def get_saldo_do_dia(self, cr, uid, ts_conta_id, ts_data, context=None):
        """
        Metodo para recuperar o saldo de um dia
            - Selecionar saldos para conta = a ts_conta_id e data igual ou imediatamente anterior a ts_data
            - retornar o saldo
        """
        context = context and context.copy() or {}
        ids = self.search(cr, uid, ['&', ('conta_id', '=', ts_conta_id), ('data_movimento', '<=', ts_data)],
                          context=context, order='data_movimento desc', limit=1)
        saldo = 0.00
        for id in ids:
            res = self.read(cr, uid, [id], ['fintsa_saldo_final'], context=context)
            saldo = res[0]['fintsa_saldo_final']
        return saldo
