# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


HELP_DATA_CONCILIACAO = """
    Informando esta data, o sistema nao permitira fazer lancamentos nesta conta com data igual ou anterior a data
    informada
"""


class financeiro_tesouraria_contas(osv.Model):
    """
    Tabela de contas financeiras, como banco e caixas.
    Esta tabela pode tambem ter uma conta que representa um cliente, fornecedor, funcionario, etc.
    Nestes casos ocorre quando de um adiantamento, por exemplo.
    Para contas do tipo "C"(Caixa), desabilitar os campos banco_id, agencia, nro_conta
        gerente_pessoa_id, limite_especial.
    saldo_conciliado, nao deve ser mostrado no formulario de cadastramento de contas
    Para contas do tipo "B"(Banco), desabilitar o campo "conta_pai_id"
    Prefixo: fintco
    """
    from funcoes import fintco_get_saldo

    _name = 'financeiro.tesouraria.contas'
    _rec_name = "apelido"
    _roteavel = True
    _columns = {
        'banco_caixa': fields.selection([('B', 'Banco'), ('C', 'Caixa'), ('E', 'Caixa - ECF')], 'Banco/Caixa',
                                        required=True, size=1), # Tipo de Conta \nB=Banco, \nC=Caixa
        'banco_id': fields.many2one("adm.banco", 'Banco', ondelete="restrict", select=True), # Id do banco
        'gerente_pessoa_id': fields.many2one("adm.pessoa", 'Gerente', ondelete="restrict", select=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # id da empresa
        'apelido': fields.char('Apelido', size=30, required=True, select=True), # Apelido, identificacao da conta.
        'agencia_id': fields.many2one("adm.banco.agencia", 'Agencia', ondelete="restrict", select=True,
                                      domain="[('admbag_banco_id','=',banco_id)]"), # Id da agencia
        'nro_conta': fields.char('Nro conta', size=30), # Numero da conta na agencia
        'conta_dv': fields.char('Conta DV', size=1, help='Digito verificador da conta'),
        'limite_especial': fields.float('Limite especial', digits=(14, 2)),
        'integra_fluxo': fields.boolean('Integra fluxo'),
        'active': fields.boolean('Ativo'), # Informa se a conta esta ativa
        'parent_id': fields.many2one("financeiro.tesouraria.contas", 'Conta Pai', ondelete="restrict", select=True),
        'fintco_saldo_conciliado': fields.float('Saldo conciliado', digits=(15, 2), oldname="saldo_conciliado"), # Valor do saldo conciliado ate o momento.
        'contas_ids': fields.one2many('financeiro.tesouraria.contas', 'parent_id', 'Contas'),
        'movimento_ori_ids': fields.one2many('financeiro.tesouraria.movimento', 'conta_id', 'Movimento Origem'),
        'movimento_des_ids': fields.one2many('financeiro.tesouraria.movimento', 'conta_destino_id',
                                             'Movimento Destino'), # Movimentos que tem uma conta como destino
        'saldos_ids': fields.one2many('financeiro.tesouraria.saldos', 'conta_id', 'Saldos'),
        'fintco_saldo_final': fields.function(fintco_get_saldo, method=True, multi="saldo", type="float",
                                              string='Saldo', store=False),
        'fintco_data_conciliacao': fields.date('Conciliado ate')
    }
    _defaults = {
        'active': lambda *a: True,
        'integra_fluxo': lambda *a: True,
        'banco_caixa': lambda *a: 'B',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }
    _sql_constraints = [
        ('apelido_uniq', 'unique (company_id, apelido)', 'O apelido precisa ser unico!'),
    ]

