# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import time
from openerp.osv import osv, fields


class financeiro_tesouraria_movimento(osv.Model):
    """
    Tabela com os lancamentos de recebimentos, pagamentos e outros movimentos da Tesouraria.
    Quando se tratar de recebimento ou pagamento, ela e a capa do lote dos documentos baixados.
    prefixo: fintem

    """
    from funcoes import _fintem_get_parametro_centro_custo_evento, _fintem_get_parametro_esconde_forma_pgto, \
        tesmov_check_criacao_movimento_conta
    from funcoes import _fintem_get_dados_tesouraria

    _name = 'financeiro.tesouraria.movimento'
    _columns = {
        'id': fields.integer('Controle', readonly=True),
        'name': fields.function(_fintem_get_dados_tesouraria, method=True, string='Nome',
                                type='char', store=False, size=50, multi='dados'),
        'conta_id': fields.many2one("financeiro.tesouraria.contas", 'Conta', ondelete="restrict", required=True),
        'conta_destino_id': fields.many2one("financeiro.tesouraria.contas", 'Conta destino', ondelete="restrict"),
        'movimento_destino_id': fields.many2one('financeiro.tesouraria.movimento',
                                                'Movimento destino'),
        'data_movimento': fields.date('Data movimento', required=True), # Data do movimento
        'mes_competencia': fields.char('Mes competencia', size=2, required=True), # Mes de competencia
        'ano_competencia': fields.char('Ano competencia', size=4, required=True), # Ano de competencia
        'valor': fields.float('Valor', digits=(15, 2), required=True),
        'historico': fields.text('Historico', required=True), # Historico do lancamento
        'conciliado': fields.boolean('Conciliado'), # Flag que indica que o lancamento foi conciliado
        'debito_credito': fields.selection([('D', 'Debito'), ('C', 'Credito')], 'Debito/Credito', required=True,
                                           size=1),
        'baixa_ids': fields.one2many('financeiro.titulo.baixa', 'tesouraria_movimento_id', 'Baixa'),
        'baixa_forma_pgto_ids': fields.one2many('financeiro.tesouraria.baixa.forma.pgto', 'tesouraria_movimento_id',
                                                'Pgto'), #Lista das formas de pagamento usadas em um movimento
        'fintem_titulo_ids': fields.many2many(
            'financeiro.titulo', 'financeiro_m2m_titulo_tesouraria', 'id', 'titulo_id'),
        'state': fields.selection([('A', 'Ativo'), ('E', 'Estornado')], 'Status', size=1),
        'gerencial_movimento_ids': fields.one2many('gerencial.movimento', 'tesouraria_movimento_id'),
        'parametro_centro_custo_evento': fields.function(_fintem_get_parametro_centro_custo_evento, method=True,
                                                         type="boolean", string="Parametro Centro Custo e Evento",
                                                         store=False),
        'parametro_esconde_forma_pgto': fields.function(_fintem_get_parametro_esconde_forma_pgto, method=True,
                                                        type="boolean", string="Parametro Esconde Forma de Pagamento",
                                                        store=False),
        'fintem_possui_titulo': fields.function(_fintem_get_dados_tesouraria, string='Possui titutlos',
                                                method=True, type='boolean', store=False, multi='dados'),
        'fintem_operacoes_id': fields.many2one('financeiro.operacoes', 'Operacao', ondelete="restrict"),
        'fintem_documento': fields.char('Documento', size=20, select=1),
    }
    _defaults = {
        'conciliado': lambda *a: False,
        'data_movimento': lambda *a: time.strftime('%Y-%m-%d'),
        'parametro_centro_custo_evento': lambda *a: False,
        'parametro_esconde_forma_pgto': lambda *a: False,
    }
    _order = 'conta_id, data_movimento desc,conciliado,debito_credito'

    _constraints = [(
                        tesmov_check_criacao_movimento_conta,
                        '\nLANCAMENTO NAO REALIZADO.'
                        '\nA DATA DO MOVIMENTO E MENOR QUE A DATA DE CONCILIACAO CADASTRADA NA CONTA UTILIZADA',
                        ['data_movimento']
                    )]


