# -*- encoding: utf-8 -*-

import time
from openerp import pooler
import StringIO
from openerp.osv import osv, fields
from datetime import date, timedelta
from openerp.report.render import render
from dateutil.parser import parse
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from SigUtils.impressao import Impressora
from SigUtils.formatastring import formataString
from openerp.tools.translate import _
import re

MESES = [
    _('JANEIRO'), _('FEVEREIRO'), _('MARCO'), _('ABRIL'), _('MAIO'), _('JUNHO'),
    _('JULHO'), _('AGOSTO'), _('SETEMBRO'), _('OUTUBRO'), _('NOVEMBRO'), _('DEZEMBRO')
]


# noinspection PyPep8Naming
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):
    def create(self, cr, uid, ids, data, context=None):
        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        db_name = pooler.get_pool(cr.dbname)
        obj_tesouraria_livro_caixa = db_name.get('tesouraria.livro.caixa')
        obj_tesouraria_livro_caixa_movimentos = db_name.get('tesouraria.livro.caixa.movimentos')
        obj_financeiro_tesouraria_contas = db_name.get('financeiro.tesouraria.contas')
        obj_res_company = db_name.get('res.company')
        obj_financeiro_tesouraria_saldos = db_name.get('financeiro.tesouraria.saldos')
        obj_adm_paginas = db_name.get('adm.paginas')

        caixa_movimento_ids = obj_tesouraria_livro_caixa.read(cr, uid, ids, ['movimentos_ids'], context=context)

        if caixa_movimento_ids:
            try:
                caixa_movimento_ids = caixa_movimento_ids[0]['movimentos_ids']
            except:
                caixa_movimento_ids = []

            if caixa_movimento_ids:
                movimento_dados = obj_tesouraria_livro_caixa_movimentos.read(
                    cr, uid, caixa_movimento_ids, context=context)

                # ID da conta incluindo as filhas se houver
                # Apelido conta
                data['conta_apelido'] = obj_financeiro_tesouraria_contas.read(
                    cr, uid, data['form']['conta_id'][0], ['apelido'], context=context)['apelido']
                tesouraia_company_id = obj_financeiro_tesouraria_contas.read(
                    cr, uid, data['form']['conta_id'][0], ['company_id'], context=context)['company_id'][0]
                company_name = obj_res_company.browse(cr, uid, tesouraia_company_id).name
                periodo_incial = data['form'].get('data_ini')
                periodo_final = data['form'].get('data_fim')

                try:
                    periodo_incial = parse(periodo_incial, yearfirst=True).strftime('%d/%m/%Y')
                    periodo_final = parse(periodo_final, yearfirst=True).strftime('%d/%m/%Y')
                except:
                    periodo_incial = '//'
                    periodo_final = '//'

                data['nome_empresa'] = company_name

                # Saldo do dia anterior
                y1, m1, d1 = (int(x) for x in data['form']['data_ini'].split('-'))
                data_ini = date(y1, m1, d1) - timedelta(days=1)
                data['saldo_dia'] = obj_financeiro_tesouraria_saldos.get_saldo_do_dia(
                    cr, uid, data['form']['conta_id'][0], data_ini)

                # Pagina
                args = [('data', '=', data_ini), ('company_id', '=', data['form']['company_id'][0])]
                adm_pagina_ids = obj_adm_paginas.search(cr, uid, args, context=context)

                if adm_pagina_ids:
                    pag_ini = obj_adm_paginas.read(
                        cr, uid, adm_pagina_ids, ['inicial'], context=context)[0]['inicial']
                else:
                    pag_ini = 1

                data['pagina'] = pag_ini

                def center_title(sleft, scenter, sright, swidth=100):
                    title_c = (swidth - len(scenter)) / 2
                    title = sleft

                    for i in range(title_c - len(sleft)):
                        title += ' '
                    title += scenter

                    for i in range(title_c - len(sright)):
                        title += ' '

                    title += sright

                    return title

                def det_resumo(sum_pag_e, sum_pag_s, saldo_ant, saldo_atual, sum_tot_e, sum_tot_s):
                    listagem.imprimeLinha('*' + '='*105 + '*')
                    listagem.imprimeLinha(_('*             Detalhamento do saldo   |               Totais da pagina...: %s | %s *') % (formataString(getNumeroEditado(sum_pag_e), 15, ' ', 'E', 'N', 'S'), formataString(getNumeroEditado(sum_pag_s), 12, ' ', 'E', 'N', 'S')))
                    listagem.imprimeLinha('*' + '='*105 + '*')
                    listagem.imprimeLinha(_('*                                     |               Saldo anterior.....: %s | ************ *') % formataString(getNumeroEditado(saldo_ant), 15, ' ', 'E', 'N', 'S'))
                    listagem.imprimeLinha('*                                     |===================================================================*')
                    listagem.imprimeLinha(_('* ----------------------------------- |               Saldo atual........: *************** | %s *') % formataString(getNumeroEditado(saldo_atual), 12, ' ', 'E', 'N', 'S'))
                    listagem.imprimeLinha(_('*      Assinatura do responsavel      |===================================================================*'))
                    listagem.imprimeLinha(_('*                                     |               TOTAL GERAL........: %s | %s *') % (formataString(getNumeroEditado(sum_tot_e), 15, ' ', 'E', 'N', 'S'), formataString(getNumeroEditado(sum_tot_s), 12, ' ', 'E', 'N', 'S')))
                    listagem.imprimeLinha('*' + '='*105 + '*')

                pdf_string = StringIO.StringIO()
                listagem = Impressora()
                listagem.defineFormato()
                listagem.defineFonte('Courier', 9)
                listagem.informaNome(pdf_string, _('Livro Caixa'))
                listagem.defineMargem(0.3, 0.3, 0.3, 0.3)
                listagem.informaRodape('*' + '-' * 105 + '*')
                listagem.informaRodape(' ' * 79 + _('**  Sig Informatica Ltda  **'))

                # Variaveis de controle
                page = data['form']['paginas']
                inc_x = 0
                [sum_pag_e, sum_pag_s] = [0.0, 0.0]
                saldo_ant = data['saldo_dia']
                dt = data['form']['data_ini']
                dia = ''

                listagem.cabecalhos = []
                listagem.informaCab(center_title(_('Controle de Caixa'), _('Caixa Diario '), str(data['conta_apelido']), 107))
                listagem.informaCab(center_title(time.strftime('%d/%m/%Y'), data['nome_empresa'] + ' ' * 9, _('Folha: {{ Page , 5 }}'), 117))
                listagem.informaCab(center_title('', _('Periodo: %s a %s') % (periodo_incial, periodo_final), '', 107))
                listagem.informaCab('*' + '=' * 105 + '*')
                listagem.informaCab('*       ' + _('H I S T O R I C O') + ' ' * 49 + _('* E N T R A D A *   S A I D A   *'))
                listagem.informaCab('*' + '=' * 105 + '*')

                movimento_dados = sorted(movimento_dados, key=lambda dm: dm.get('data_movimento'))
                data_movimento_list = list()
                for x in movimento_dados:
                    if x.get('data_movimento') not in data_movimento_list:
                        data_movimento_list.append(x.get('data_movimento'))
                        listagem.imprimeLinha('')
                        listagem.imprimeLinha(_(' CAIXA DO DIA: %s de %s de %s') % (
                            x['data_movimento'][8:], MESES[int(x['data_movimento'][5:7]) - 1],
                            x['data_movimento'][:-6]))

                    #Separando os campos em variáveis, para facilitar o entendimento
                    historico = x['historico']

                    if not x['valor_ent']:
                        valor_ent = ''
                    else:
                        valor_ent = getNumeroEditado(x['valor_ent'])

                    if not x['valor_sai']:
                        valor_sai = ''
                    else:
                        valor_sai = getNumeroEditado(x['valor_sai'])

                    if len(historico) <= 75:
                        ln = formataString(historico.replace('\n', ''), 75, ' ', 'D', 'N', 'S')
                        ln += '   '

                        ln += formataString(valor_ent or '', 12, ' ', 'E', 'N', 'S')
                        ln += '   '
                        ln += formataString(valor_sai, 12, ' ', 'E', 'N', 'S')

                        listagem.imprimeLinha(ln)
                    else:  # implementar quebra de linha

                        split_text = historico.replace('\n', '').split(' ')
                        lines = [split_text.pop(0)]
                        indent = 0
                        while len(split_text) > 0:
                            if len(lines[-1] + split_text[0]) > (75 - indent):
                                lines[-1] = ' ' * indent + \
                                        formataString(lines[-1], (75 - indent), ' ', 'D', 'N', 'S')
                                listagem.imprimeLinha(lines[-1])
                                indent = 4
                                lines.append(split_text.pop(0))
                            else:
                                lines[-1] += ' ' + split_text.pop(0)
                        else:  # imprime a última linha
                            lines[-1] = ' ' * indent + \
                                        formataString(lines[-1], (75 - indent), ' ', 'D', 'N', 'S')
                            lines[-1] += '   '
                            lines[-1] += formataString(valor_ent, 12, ' ', 'E', 'N', 'S')
                            lines[-1] += '   '
                            lines[-1] += formataString(valor_sai, 12, ' ', 'E', 'N', 'S')
                            listagem.imprimeLinha(lines[-1])


                    inc_x += 1
                    sum_pag_e += x['valor_ent']
                    sum_pag_s += x['valor_sai']
                    dia = x['data_movimento']

                    if inc_x == 72:
                        saldo_atual = saldo_ant + sum_pag_e - sum_pag_s
                        sum_tot_e = sum_pag_e + saldo_ant
                        sum_tot_s = sum_pag_s + saldo_atual
                        det_resumo(sum_pag_e, sum_pag_s, saldo_ant, saldo_atual, sum_tot_e,
                                   sum_tot_s)
                        listagem.imprimeLinha('{{ Page }}')
                        page += 1
                        inc_x = 0
                        [sum_pag_e, sum_pag_s] = [0.0, 0.0]
                        saldo_ant = saldo_atual

                    dt = x['data_movimento']

                saldo_atual = saldo_ant + sum_pag_e - sum_pag_s
                sum_tot_e = sum_pag_e + saldo_ant
                sum_tot_s = sum_pag_s + saldo_atual
                det_resumo(sum_pag_e, sum_pag_s, saldo_ant, saldo_atual, sum_tot_e, sum_tot_s)
                page += 1

                listagem.finalizaImpressao()

                # Atualiza o adm paginas
                company_id = data['form']['company_id'][0]
                args = [('data', '=', data['form']['data_fim']), ('company_id', '=', company_id)]
                adm_pagina_ids = obj_adm_paginas.search(cr, uid, args, context=context)

                if not adm_pagina_ids:
                    obj_adm_paginas.create(cr, uid, {
                        'company_id': company_id,
                        'tipo': 'LCA',
                        'data': data['form']['data_fim'],
                        'inicial': pag_ini,
                        'final': page
                    }, context)
                else:
                    obj_adm_paginas.write(
                        cr, uid, adm_pagina_ids, dict(inicial=pag_ini, final=page), context)

        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'


report_custom('report.livro_caixa')


class tesouraria_livro_caixa_movimento_1(osv.TransientModel):
    _name = 'tesouraria.livro.caixa.movimentos'
    _columns = {
        'cod': fields.integer("Codigo"),
        'mov_cod_id': fields.many2one('tesouraria.livro.caixa', "Codigo"),
        'data_movimento': fields.date('Data do movimento'),
        'historico': fields.text('Historico'),
        'valor': fields.float('Valor', digits=(15, 2)),
        'valor_ent': fields.float('Entrada', digits=(15, 2)),
        'valor_sai': fields.float('Saida', digits=(15, 2)),
    }
    _defaults = {
        'cod': lambda *a: '1',
    }


tesouraria_livro_caixa_movimento_1()


class tesouraria_livro_caixa(osv.TransientModel):
    # LIMPAR DADOS DA TELA

    movimentos = []

    def button_clear(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        self.unlink(cr, uid, ids)
        obj_tesouraria_livro_caixa_movimentos = self.pool.get('tesouraria.livro.caixa.movimentos')
        id_s = obj_tesouraria_livro_caixa_movimentos.search(cr, uid, [], context=context)
        obj_tesouraria_livro_caixa_movimentos.unlink(cr, uid, id_s)

        return {
            'name': 'Livro Caixa',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'tesouraria.livro.caixa',
            'view_id': False,
            'target': 'inline',
            'type': 'ir.actions.act_window',
        }

    # BUSCAR DADOS
    def button_find(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        db_name = pooler.get_pool(cr.dbname)
        data = {}
        data['form'] = self.read(cr, uid, ids, context=context)[0]
        if data['form']['data_ini'] > data['form']['data_fim']:
            raise osv.except_osv(_('Atencao'), _('A Data inicial nao poder ser maior que a Data final.'))
            # ID DA CONTA INCLUINDO AS FILHAS SE HOUVER
        obj_tesouraria = db_name.get('financeiro.tesouraria.contas')
        contas_id = obj_tesouraria.search(cr, uid, [('parent_id', '=', data['form']['conta_id'][0])], context=context)
        contas_id.append(data['form']['conta_id'][0])
        # APELIDO CONTA
        data['conta_apelido'] = obj_tesouraria.read(cr, uid, data['form']['conta_id'][0], ['apelido'], context=context)['apelido']
        data['nome_empresa'] = obj_tesouraria.read(cr, uid, data['form']['conta_id'][0], ['company_id'], context=context)['company_id'][
            1]
        # BUSCAR MOVIMENTOS
        obj_tesouraria = db_name.get('financeiro.tesouraria.movimento')
        args_search = [
            ('conta_id', 'in', contas_id),
            ('data_movimento', '>=', data['form']['data_ini']),
            ('data_movimento', '<=', data['form']['data_fim']),
        ]
        movimentos = []
        movimento_ids = obj_tesouraria.search(cr, uid, args_search, order='data_movimento,id',
                                              context=context)
        if not movimento_ids:
            raise osv.except_osv(_('Atencao'), ('Nao existe movimento para os filtros informados'))
        cont = 0
        for x in obj_tesouraria.browse(cr, uid, movimento_ids):
            if x.debito_credito == 'D':
                valor_ent = 0
                valor_sai = x.valor
            else:
                valor_ent = x.valor
                valor_sai = 0
            cont += 1
            movimentos.append({'valor_ent': valor_ent, 'valor_sai': valor_sai, 'mov_cod_id': ids[0],
                               'data_movimento': x.data_movimento, 'historico': x.historico})

        obj = self.pool.get('tesouraria.livro.caixa.movimentos')
        id_s = obj.search(cr, uid, [], context=context)
        obj.unlink(cr, uid, id_s)

        for x in movimentos:
            obj.create(cr, uid, x, context)
        self.write(cr, uid, ids, {'state': '1'}, context)

        return True
    # IMPRIMIR
    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        data = {}
        campos = ['company_id', 'conta_id', 'data_ini', 'data_fim', 'paginas', 'state', 'flag']
        data['form'] = self.read(cr, uid, ids, campos, context=context)[0]
        data['ids'] = ids

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'livro_caixa',
            'datas': data,
        }

    def onchange_movimento(self, cr, uid, ids, movimentos_ids):
        return {'value': {'conta_id': False}}

    _name = 'tesouraria.livro.caixa'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True),
        'conta_id': fields.many2one("financeiro.tesouraria.contas", 'Conta', ondelete="cascade", required=True,
                                    select=True, domain="[('banco_caixa','=','C'), ('company_id','=',company_id)]"),
        'data_ini': fields.date("Data Inicial", required=True),
        'data_fim': fields.date("Data Final", required=True),
        'paginas': fields.integer("Paginas"),

        'movimentos_ids': fields.one2many('tesouraria.livro.caixa.movimentos', 'mov_cod_id', 'Movimentos'),
        'state': fields.char('Status', size=1),
        'flag': fields.integer('Status')
    }
    _defaults = {
        'state': lambda *a: '0',
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id
    }


tesouraria_livro_caixa()
