# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from funcoes import wizfdi_is_movimento_conciliado
from funcoes import wizftd_get_saldo_conciliado


class wizard_financeiro_tesouraria_desconciliacao(osv.TransientModel):
    # prefixo: wizftd_

    _name = 'wizard.financeiro.tesouraria.desconciliacao'
    _columns = {
        'wizftd_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta',
                                           required=True),
        'wizftd_saldo_conciliado': fields.function(wizftd_get_saldo_conciliado, method=True,
                                                   string='Saldo conciliado', digits=(14, 2)),
        'wizftd_item_ids': fields.one2many('wizard.financeiro.tesouraria.desconciliacao.item',
                                           'wizfdi_capa_id', 'Itens'),
    }

    def wizftd_onchange_item_ids(self, cr, uid, ids, item_ids, context=None):
        context = context and context.copy() or {}
        res = {}
        conta_id = context.get('conta_id')
        if not conta_id or not item_ids:
            return res
        obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
        self.wizftd_trata_conciliacao(cr, uid, conta_id, item_ids, context)
        res['value'] = {'wizftd_saldo_conciliado': obj_financeiro_tesouraria_contas.browse(
            cr, uid, conta_id, context).fintco_saldo_conciliado or 0.0}
        return res

    def wizftd_trata_conciliacao(self, cr, uid, conta_id, item_ids, context=None):
        """
        O metodo trata a conciliacao/desconciliacao dos movimentos da tesouraria e acerta o saldo
        conciliado na tela e no cadastro de contas.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, indentificador do usuario.
        :param conta_id: id da tabela financeiro_tesouraria_contas.
        :param item_ids: lista com os dados da tabela
        wizard_financeiro_tesouraria_desconciliacao_item
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
        desconciliado_item_ids = filter(lambda i: 'wizfdi_desconciliar' in i[2], item_ids)
        item_ids = list()
        map(lambda i: item_ids.append(i[2]), desconciliado_item_ids)
        tesouraria_movimento_dict = dict()
        movimento_ids = map(lambda i: i.get('wizfdi_tesouraria_movimento_id'), item_ids)
        brw_tesouraria_movimento = obj_financeiro_tesouraria_movimento.browse(
            cr, uid, movimento_ids, context)
        map(lambda i: tesouraria_movimento_dict.update({i.id: i.conciliado}),
            brw_tesouraria_movimento)
        for item in item_ids:
            tesouraria_movimento_conciliado = tesouraria_movimento_dict.get(item.get(
                'wizfdi_tesouraria_movimento_id'))
            debito_credito = item.get('wizfdi_debito_credito')
            if tesouraria_movimento_conciliado == (not item.get('wizfdi_desconciliar')):
                continue
            valor = item.get('wizfdi_valor') or 0
            if item.get('wizfdi_desconciliar'):
                debito_credito = 'D' if debito_credito == 'C' else 'C'
            obj_financeiro_tesouraria_contas.fintco_atualizar_saldo_conciliado(
                cr, uid, conta_id, valor, debito_credito, context)
            context.update({'conciliado': True})
            obj_financeiro_tesouraria_movimento.write(
                cr, uid, [item.get('wizfdi_tesouraria_movimento_id')],
                {'conciliado': False if item.get('wizfdi_desconciliar') else True}, context)
        return True

    def wizftd_onchange_conta_id(self, cr, uid, ids, conta_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'wizftd_saldo_conciliado': 0}}
        if not conta_id:
            return res
        obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
        saldo_conciliado = obj_financeiro_tesouraria_contas.browse(
            cr, uid, conta_id, context).fintco_saldo_conciliado or 0.0
        res['value'] = {'wizftd_saldo_conciliado': saldo_conciliado}
        return res

    def wizftd_get_itens(self, cr, uid, ids, context=None):
        """
        O metodo procura os movimentos da tesouraria que estao conciliados e os exibem em ordem
        decrescente de data. Exibe tambem o saldo conciliado.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, indentificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        context = context and context.copy() or {}
        res = {}
        cr.execute('DELETE FROM wizard_financeiro_tesouraria_desconciliacao_item')
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        for desconciliacao in self.browse(cr, uid, ids, context):
            conta_id = desconciliacao.wizftd_conta_id.id
            tesouraria_movimento_ids = obj_financeiro_tesouraria_movimento.search(
                cr, uid, [('conta_id', '=', conta_id), ('conciliado', '=', True)],
                order='data_movimento DESC', context=context)
            if not tesouraria_movimento_ids:
                return res
            brw_tesouraria_movimento = obj_financeiro_tesouraria_movimento.browse(
                cr, uid, tesouraria_movimento_ids, context)
            desconciliacao_item_ids = list()
            for movimento in brw_tesouraria_movimento:
                vals = {'wizfdi_codigo': movimento.id,
                        'wizfdi_data': movimento.data_movimento,
                        'wizfdi_historico': movimento.historico or '',
                        'wizfdi_debito_credito': movimento.debito_credito,
                        'wizfdi_valor': movimento.valor,
                        'wizfdi_tesouraria_movimento_id': movimento.id}
                desconciliacao_item_ids.append((0, 0, vals))
            res.update({'wizftd_item_ids': desconciliacao_item_ids})
            self.write(cr, uid, desconciliacao.id, res, context)
        return True

    # def wizftd_button_desconciliar(self, cr, uid, ids, context=None):
    #     """
    #     O metodo desconcilia os movimentos da tesouraria e acerta o saldo conciliado no cadastro de
    #     contas.
    #     :param cr: cursor no banco de dados.
    #     :param uid: id da tabela res_users, indentificador do usuario.
    #     :param ids: lista de id da tabela desse objeto.
    #     :param context: dicionario com valores contextuais.
    #     :return: booleano True.
    #     """
    #     context = context and context.copy() or {}
    #     brw_tesouraria_desconciliacao = self.browse(cr, uid, ids, context)
    #     obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
    #     obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
    #     for desconciliacao in brw_tesouraria_desconciliacao:
    #         conta_id = desconciliacao.wizftd_conta_id.id
    #         brw_tesouraria_movimento = map(lambda i: i.wizfdi_tesouraria_movimento_id,
    #                                        desconciliacao.wizftd_item_ids)
    #         for movimento in brw_tesouraria_movimento:
    #             debito_credito = 'D' if movimento.debito_credito == 'C' else 'C'
    #             valor = movimento.valor or 0
    #             obj_financeiro_tesouraria_contas.fintco_atualizar_saldo_conciliado(
    #                 cr, uid, conta_id, valor, debito_credito, context)
    #         tesouraria_movimento_ids = map(lambda i: i.id, brw_tesouraria_movimento)
    #         obj_financeiro_tesouraria_movimento.write(cr, uid, tesouraria_movimento_ids,
    #                                                   {'conciliado': False}, context)
    #     return True


class wizard_financeiro_tesouraria_desconciliacao_item(osv.TransientModel):
    # prefixo: wizfdi

    _name = 'wizard.financeiro.tesouraria.desconciliacao.item'
    _columns = {
        'wizfdi_capa_id': fields.many2one('wizard.financeiro.tesouraria.desconciliacao',
                                          'Capa'),
        'wizfdi_codigo': fields.integer('ID', readonly=True),
        'wizfdi_data': fields.date('Data do movimento', readonly=True),
        'wizfdi_historico': fields.text('Historico', readonly=True),
        'wizfdi_debito_credito': fields.selection([('D', 'Debito'), ('C', 'Credito')],
                                                  'Debito/credito', size=1, readonly=True),
        'wizfdi_valor': fields.float('Valor', digits=(15, 2), readonly=True),
        'wizfdi_tesouraria_movimento_id': fields.many2one('financeiro.tesouraria.movimento',
                                                          'Movimento'),
        'wizfdi_movimento_conciliado': fields.function(wizfdi_is_movimento_conciliado, method=True,
                                                       string='Conciliado', type='boolean'),
        'wizfdi_desconciliar': fields.boolean('Desconciliar'),
    }

    _defaults = {
        'wizfdi_desconciliar': lambda *a: False,
    }

    def wizfdi_trata_conciliacao(self, cr, uid, ids, context=None):
        """
        O metodo trata a conciliacao/desconciliacao dos movimentos da tesouraria e acerta o saldo
        conciliado no cadastro de contas.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, indentificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        context = context and context.copy() or {}
        # obj_tesouraria_desconciliacao = self.pool.get(
        #     'wizard.financeiro.tesouraria.desconciliacao')
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
        brw_tesouraria_desconciliacao_item = self.browse(cr, uid, ids, context)
        for item in brw_tesouraria_desconciliacao_item:
            conta_id = item.wizfdi_capa_id.wizftd_conta_id.id
            debito_credito = item.wizfdi_tesouraria_movimento_id.debito_credito
            conciliado = False if item.wizfdi_movimento_conciliado else True
            valor = item.wizfdi_tesouraria_movimento_id.valor or 0
            if item.wizfdi_movimento_conciliado:
                debito_credito = 'D' if debito_credito == 'C' else 'C'
            obj_financeiro_tesouraria_contas.fintco_atualizar_saldo_conciliado(
                cr, uid, conta_id, valor, debito_credito, context)
            context.update({'conciliado': True})
            obj_financeiro_tesouraria_movimento.write(
                cr, uid, [item.wizfdi_tesouraria_movimento_id.id], {'conciliado': conciliado},
                context)
        return True
