# -*- coding: utf-8 -*-

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
import time
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from random import randint


class TestFinanceiro(TransactionCase):

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestFinanceiro, self).setUp()
        self.valor_multa = 5.67
        self.valor_juros = 3.28
        self.valor_desconto = 1.00
        self.valor_acrescimo = 0.95
        self.obj_financeiro_titulo = self.registry('financeiro.titulo')
        self.obj_financeiro = self.registry('financeiro')
        self.pessoa_id = self.get_pessoa_id()
        self.cria_cliente(self.pessoa_id)
        self.titulo_dados = self._get_titulo_dados()
        valor_pago = self.titulo_dados.get('valor_pago') + self.valor_multa + self.valor_juros - \
            self.valor_desconto
        self.forma_pagamento_id = self.registry('adm.forma.pgto').search(
            self.cr, self.uid, [('name', 'ilike', 'dinheiro')])[0]
        self.tesouraria_conta_id = self.registry('financeiro.tesouraria.contas').search(
            self.cr, self.uid, [('banco_caixa', '=', 'C')], limit=1)[0]
        self.list_pagamentos = [{'tesouraria_conta_id': self.tesouraria_conta_id,
                                 'forma_pagamento_id': self.forma_pagamento_id,
                                 'condicao_tipo': 'V',
                                 'valor': valor_pago}]
        self.titulo_dados = [self.titulo_dados]

    def _get_titulo_dados(self):
        data_emissao_vencimento = time.strftime('%Y-%m-%d')
        vals = {'documento': 'TESTE' + str(randint(1, 100)), 'tipo_titulo': 'R',
                'data_emissao': data_emissao_vencimento, 'data_vencimento': data_emissao_vencimento,
                'pessoa_id': self.pessoa_id, 'vr_original': 100.00}
        titulo_id = self.obj_financeiro_titulo.create(self.cr, self.uid, vals)
        return {'titulo_id': titulo_id,
                'valor_pago': vals.get('vr_original'),
                'valor_multa': self.valor_multa,
                'valor_juros': self.valor_juros,
                'valor_desconto': self.valor_desconto,
                'valor_acrescimo': self.valor_acrescimo,
                'tipo_titulo': vals.get('tipo_titulo'),
                'pessoa_id': self.pessoa_id}

    def get_pessoa_id(self):
        pessoa = self.registry('adm.pessoa')
        pessoa_ids = pessoa.search(self.cr, self.uid, [('cpf_cnpj', '=', '98899682070')])
        if pessoa_ids:
            pessoa_id = pessoa_ids[0]
        else:
            vals = dict(pessoa_tipo='F', name='DESTINATARIO DE TESTE ACAO%°*a',
                        cpf_cnpj='98899682070')
            pessoa_id = pessoa.create(self.cr, self.uid, vals)
        return pessoa_id

    def cria_cliente(self, pessoa_id):
        cliente = self.registry('financeiro.cliente')
        cliente_ids = cliente.search(self.cr, self.uid, [('pessoa_id', '=', pessoa_id)])
        if cliente_ids:
            cliente_id = cliente_ids[0]
        else:
            cliente_id = cliente.create(self.cr, self.uid, {'pessoa_id': pessoa_id,
                                                            'limite_credito': 200.0})
        return cliente_id

    def test_financ_baixar_titulo(self):
        res = self.obj_financeiro.financ_baixar_titulo(
            self.cr, self.uid,
            futl_bxti_titulos=self.titulo_dados,
            futl_bxti_data_pagamento=time.strftime('%Y-%m-%d'),
            ftul_bxti_forma_pgto_id=self.forma_pagamento_id,
            futl_bxti_tesouraria_conta_id=self.tesouraria_conta_id,
            futl_bxti_pessoa_id=self.pessoa_id,
            futl_bxti_pagamento_dados=self.list_pagamentos)
        self.assertTrue(res)

    def test_financ_get_custificacao_baixa(self):
        res = self.obj_financeiro.financ_get_custificacao_baixa(
            self.cr, self.uid, self.titulo_dados, {})
        self.assertEqual(type(res), list)

    def test_financ_titulo_entrada(self):
        emissao = time.strftime('%Y-%m-%d')
        vencimento = parse(emissao) + relativedelta(days=30)
        cliente_pessoa_id = self.get_pessoa_id()
        self.cria_cliente(cliente_pessoa_id)
        res = self.obj_financeiro.financ_titulo_entrada(
            self.cr, self.uid, cliente_pessoa_id, emissao, vencimento.strftime('%Y-%m-%d'),
            'teste_teste', 1, 129.99, 'M', 'P', context={'fintit_titulo_sem_custificacao': True})
        self.assertEqual(type(res), long)
