#!/usr/bin/python
# -*- encoding: utf-8 -*-

__author__ = 'tiago.santos'

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
import time


class TestFinanceiroCliente(TransactionCase):
    """
    Classe responsavel por testes unitarios dos metodos da classe financeiro_cliente
    :author: Tiago Henrique Prates dos Santos <tiago.santos[at]siginformatica.com.br>
    """

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestFinanceiroCliente, self).setUp()
        self.financeiro_cliente = self.registry('financeiro.cliente')
        self.financeiro_titulo = self.registry('financeiro.titulo')
        self.cliente_estatisticas_compras_osv = self.registry('cliente.estatisticas.compras.osv')
        self.pessoa_id = self.cria_pessoa()
        self.cliente_id = self.cria_cliente(self.pessoa_id)
        self.local_cobranca_id = self.registry('financeiro.local.cobranca').search(
            self.cr, self.uid, [], limit=1)[0]

    def test_fincli_get_clientes_estatisticas_compras(self):
        self.cria_titulo()
        res = self.financeiro_cliente._fincli_get_clientes_estatisticas_compras(
            self.cr, self.uid, [self.cliente_id], None, None)
        brw_cliente_estatisticas_compras = self.cliente_estatisticas_compras_osv.browse(
            self.cr, self.uid, res.get(self.cliente_id))
        total_compras = sum(map(lambda i: i.total_mes, brw_cliente_estatisticas_compras))
        self.assertEqual(total_compras, 60.0)

    def cria_titulo(self):
        list_valor = [15.00, 20.00, 25.00]
        for i in range(0, 3):
            data_emissao = self.get_data_emissao(i)
            vals = {'vr_original': list_valor[i], 'documento': str(i),
                    'data_emissao': data_emissao, 'data_vencimento': data_emissao,
                    'data_prorrogacao': data_emissao, 'tipo_titulo': 'R',
                    'pessoa_id': self.pessoa_id, 'local_cobranca_id': self.local_cobranca_id}
            self.financeiro_titulo.create(self.cr, self.uid, vals)

    def get_data_emissao(self, qtd_dias):
        data_emissao = parse(time.strftime('%Y-%m-%d')) - relativedelta(days=(qtd_dias or 1))
        data_emissao = data_emissao.strftime('%Y-%m-%d')
        return data_emissao

    def cria_pessoa(self):
        pessoa = self.registry('adm.pessoa')
        vals = dict(pessoa_tipo='F', name='DESTINATARIO DE TESTE ACO%°*a', cpf_cnpj='47078736770')
        return pessoa.create(self.cr, self.uid, vals)

    def cria_cliente(self, pessoa_id):
        return self.financeiro_cliente.create(self.cr, self.uid, {'pessoa_id': pessoa_id,
                                                                  'limite_credito': 60.0})
