#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from datetime import date
from random import randint

__author__ = 'tiago.santos'


class TestFinanceiroTitulo(TransactionCase):
    """
    Classe responsavel por testes unitarios dos metodos da classe financeiro_titulo
    :author: Tiago Henrique Prates dos Santos <tiago.santos[at]siginformatica.com.br>
    """

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestFinanceiroTitulo, self).setUp()
        self.financeiro_titulo = self.registry('financeiro.titulo')
        self.pessoa_id = self.cria_pessoa()
        self.cria_cliente(self.pessoa_id)

    def cria_pessoa(self):
        pessoa = self.registry('adm.pessoa')
        vals = dict(pessoa_tipo='F', name='DESTINATARIO DE TESTE ACAO%°*a', cpf_cnpj='47078736770')
        return pessoa.create(self.cr, self.uid, vals)

    def cria_cliente(self, pessoa_id):
        return self.registry('financeiro.cliente').create(
            self.cr, self.uid, {'pessoa_id': pessoa_id, 'limite_credito': 200.0})

    def test_fintit_check_condicao_pessoa(self):
        res = self.financeiro_titulo.fintit_check_condicao_pessoa(self.cr, self.uid, self.pessoa_id,
                                                                  200.0)
        self.assertTrue(res)

    def test_fintit_get_total_em_aberto_00(self):
        self.assertEqual(self.financeiro_titulo.fintit_get_total_em_aberto(
            self.cr, self.uid, [self.pessoa_id], 'R').get(self.pessoa_id), 0)

    def test_fintit_get_total_em_aberto_01(self):
        data_vencimento = date.today().strftime('%Y-%m-%d')
        vals_titulo = dict(documento='TESTE' + str(randint(1, 100)), vr_original=100.01,
                           tipo_titulo='R', tipo_lancamento='M', state='A',
                           data_vencimento=data_vencimento, data_emissao=data_vencimento,
                           pessoa_id=self.pessoa_id)
        self.financeiro_titulo.create(self.cr, self.uid, vals_titulo)
        self.assertEqual(self.financeiro_titulo.fintit_get_total_em_aberto(
            self.cr, self.uid, [self.pessoa_id], 'R').get(self.pessoa_id), 100.01)