#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
import time

__author__ = 'tiago.santos'
__lat_update__ = '2017-02-06'
__mantainer__ = 'Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>'


class TestWizardBaixaTitulo(TransactionCase):
    """
    Classe responsavel por testes unitarios dos metodos da classe wizard_baixa_titulo
    :author: Tiago Henrique Prates dos Santos <tiago.santos[at]siginformatica.com.br>
    """

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestWizardBaixaTitulo, self).setUp()

        self.financeiro_titulo = self.registry('financeiro.titulo')
        self.wizard_baixa_titulo = self.registry('wizard.baixa.titulo')
        self.pessoa_id = self.cria_pessoa()
        self.local_cobranca_id = self.registry('financeiro.local.cobranca').search(
            self.cr, self.uid, [], limit=1)[0]
        self.titulo_id = self.cria_unico_titulo()
        self.wizard_baixa_titulo_id = self.cria_baixa_titulo()
        self.cria_pagamentos(self.wizard_baixa_titulo_id)
        baixa_titulo_detalhe_ids = self.wizard_baixa_titulo.read(
            self.cr, self.uid, self.wizard_baixa_titulo_id, ['wbt_titulo_detalhe_ids']).get(
            'wbt_titulo_detalhe_ids')
        self.registry('wizard.baixa.titulo.detalhe').write(
            self.cr, self.uid, baixa_titulo_detalhe_ids, {'wbtd_flag_selecionado': True})

    def cria_pessoa(self):
        vals = dict(pessoa_tipo='F', name='DESTINATARIO DE TESTE ACAO%°*a', cpf_cnpj='47078736770')
        pessoa_id = self.registry('adm.pessoa').create(self.cr, self.uid, vals)

        # Registra como cliente com limite de credito igual a R$ 250,00
        self.registry('financeiro.cliente').create(self.cr, self.uid, dict(
            pessoa_id=pessoa_id, limite_credito=250.0))

        return pessoa_id

    def cria_unico_titulo(self):
        data_emissao = time.strftime('%Y-%m-%d')
        vals = {'vr_original': 250.00, 'documento': time.strftime('%Y%m%d') + '1',
                'data_emissao': data_emissao, 'data_vencimento': data_emissao,
                'data_prorrogacao': data_emissao, 'tipo_titulo': 'R',
                'pessoa_id': self.pessoa_id, 'local_cobranca_id': self.local_cobranca_id}
        return self.financeiro_titulo.create(self.cr, self.uid, vals)

    def cria_baixa_titulo(self):
        vals = {
            'wbt_valor_pago': 250.00, 'wbt_cliente_id': self.pessoa_id,
            'wbt_data_prorrogacao_ini': time.strftime('%Y-%m-%d'),
            'wbt_local_cobranca_id': self.local_cobranca_id,
            'wbt_titulo_detalhe_ids': [[0, 0, dict(wbtd_titulo_id=self.titulo_id,
                                                   wbtd_pessoa_id=self.pessoa_id,
                                                   wbtd_saldo=250.0, wbtd_titulo_tipo='R')]]
        }
        return self.wizard_baixa_titulo.create(self.cr, self.uid, vals)

    def cria_pagamentos(self, baixa_titulo_id):
        brw_wizard_baixa_titulo = self.wizard_baixa_titulo.browse(self.cr, self.uid,
                                                                  baixa_titulo_id)
        conta_id = brw_wizard_baixa_titulo.wbt_conta_id.id
        forma_pagamento_id = self.registry('adm.forma.pgto').search(
            self.cr, self.uid, [('sigla', '=', 'DN'), ('name', '=', 'Dinheiro')])[0]
        condicao_forma_id = self.registry('adm.condicao.forma').search(
            self.cr, self.uid, [('forma_pagamento_id', '=', forma_pagamento_id)])[0]
        vals_pagamentos = {'wizbtp_baixa_id': baixa_titulo_id, 'wizbtp_conta_id': conta_id,
                           'wizbtp_forma_pgto_id': forma_pagamento_id,
                           'wizbtp_forma_pgto_tipo': 'outros', 'wizbtp_condicao_pgto_tipo': 'V',
                           'wizbtp_condicao_id': condicao_forma_id, 'wizbtp_valor': 250.00}
        return self.wizard_baixa_titulo.write(self.cr, self.uid, [baixa_titulo_id],
                                              {'wbt_pagamento_ids': [(0, 0, vals_pagamentos)]})

    def test_wbt_button_baixar_titulo(self):
        res = self.wizard_baixa_titulo.wbt_button_baixar_titulo(self.cr, self.uid,
                                                                [self.wizard_baixa_titulo_id])
        self.assertTrue(res)

    def test_wbt_button_listar_titulos(self):
        res = self.wizard_baixa_titulo.wbt_button_listar_titulos(self.cr, self.uid,
                                                                 [self.wizard_baixa_titulo_id])
        self.assertTrue(res)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
