#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.parser import parse
from openerp.tools.translate import _
from time import strftime
READ_LOAD = '_classic_write'


class wizard_baixa_cheque(osv.TransientModel):

    _name = 'wizard.baixa.cheque'
    _columns = {

        # _________________date______________________________
        'wbt_data_bom_para': fields.date('Bom para', required=True),
        'wbt_data_deposito': fields.date('Data Deposito', required=True),


        # _________________many2one______________________________
        'wbt_conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta', required=True),
        'wbt_chq_banco': fields.many2one('adm.banco','Banco'),
        'wbt_pessoa_id': fields.many2one('adm.pessoa','Cliente'),

        # _________________selection______________________________
        'state': fields.selection(
            [('TL', 'Cheque Listado'), ('BA', 'Cheque Baixado')], 'Status', size=2
        ),

        # _________________char______________________________
        'wbt_cheque_emitente': fields.char('CPF/CNPJ Emitente', size=19),
        'wbt_cheque_numero': fields.char('Numero cheque', size=30),

        # _________________one2many______________________________
        'wbt_titulo_detalhe_ids': fields.one2many('wizard.baixa.cheque.detalhe', 'wbtd_baixa_id', 'Titulos', readonly=True),

    }

    _defaults = {
        'wbt_data_deposito': lambda *a: strftime('%Y-%m-%d'),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = super(wizard_baixa_cheque, self).create(cr, uid, vals, context)
        return res

    def wbt_button_selecao(self, cr, uid, ids, booleano, context=None):
        """
        Funcao altera: campo - wbtd_flag_selecionado, na tabela - tabela wizard.baixa.cheque.detalhe
        * Indica se o titulo esta selecionado ou nao
        """
        context = context and context.copy() or {}
        if isinstance(ids, list):
            baixa_cheque_id = ids[0]
        else:
            baixa_cheque_id = ids
        obj_baixa_cheque_detalhe = self.pool.get('wizard.baixa.cheque.detalhe')
        titulo_detalhe_ids = obj_baixa_cheque_detalhe.search(
            cr, uid, [('wbtd_baixa_id', '=', baixa_cheque_id)], context=context
        )
        if titulo_detalhe_ids:
            obj_baixa_cheque_detalhe.write(
                cr, uid, titulo_detalhe_ids, {'wbtd_flag_selecionado': booleano}, context=context
            )
        return True

    def wbt_button_marcar_todos(self, cr, uid, ids, context=None):
        """
        Marca todos os titulos listados no grid.
        """
        context = context and context.copy() or {}
        res = self.wbt_button_selecao(cr, uid, ids, booleano=True, context=context)
        return res

    def wbt_button_desmarcar_todos(self, cr, uid, ids, context=None):
        """
        Desmarca todos os titulos listados no grid.
        """
        context = context and context.copy() or {}
        res = self.wbt_button_selecao(cr, uid, ids, booleano=False, context=context)
        return res

    def wbt_raise_msg_titulos(self, cr, uid, titulo_ids, message, context=None):
        context = context and context.copy() or {}
        message = unicode(message)
        res_name_get = self.pool.get('financeiro.titulo').name_get(cr, uid, titulo_ids, context)
        if res_name_get:
            msg_titulos = u''
            dict_titulo_name = dict(res_name_get)
            for i in titulo_ids:
                msg_titulos += u'%s, ' % dict_titulo_name.get(i) or ''
            if msg_titulos:
                raise osv.except_osv(_('Atencao'), u'%s: %s' % (message, msg_titulos))
        return True

    def wbt_get_titulo_selecionados_saldo(self, cr, uid, titulo_selecionado_ids, context=None):
        """
        Funcao soma:
        * Valor total de todos os titulos indicados.
        """
        context = context and context.copy() or {}
        if titulo_selecionado_ids:

            wiz_baixa_detalhe_dados = self.pool.get('wizard.baixa.cheque.detalhe').read(
                cr, uid, titulo_selecionado_ids, ['wbtd_valor_total'], context=context
            )

            somatorio_titulos = sum(map(lambda i: i.get('wbtd_valor_total') or 0, wiz_baixa_detalhe_dados))
        else:
            somatorio_titulos = 0
        return somatorio_titulos

    def wbt_validar_valor_pago_titulos(self, cr, uid, valor_pago, titulo_selecionado_ids, context=None):
        """
        Funcao verifica:
        * Se valor pago e menor ou igual a zero
        * Se foi selecionado mais do que um titulo, o valor pago deve ser igual ao somatorio desses titulos
        * Se o valor pago e maior que o valor dos titulos
        """
        context = context and context.copy() or {}
        convert_2_decimal = self.pool.get('financeiro').financ_convert_to_decimal

        total_titulos_selecionados = len(titulo_selecionado_ids or [])
        somatorio_titulos = self.wbt_get_titulo_selecionados_saldo(
            cr, uid, titulo_selecionado_ids=titulo_selecionado_ids, context=context
        )

        deci_somatorio_titulos = convert_2_decimal(somatorio_titulos)
        deci_valor_pago = convert_2_decimal(valor_pago)

        if total_titulos_selecionados == 1:
            if deci_valor_pago <= 0:
                raise osv.except_osv(_('Atencao'), _('Informe um valor pago maior que zero!'))
            if deci_valor_pago > deci_somatorio_titulos:
                raise osv.except_osv(
                    _('Atencao'),
                    _('O valor pago: R$ %.3f, e maior que o valor do titulo: R$ %.3f. Informe um valor menor ou igual ao valor do titulo!') % (deci_valor_pago, deci_somatorio_titulos)
                )
        return True

    def wbt_button_listar_cheques(self, cr, uid, ids, context=None):
        """
        Funcao cria:
        * Lista todos os titulos em aberto para o cliente selecianado.
        """
        if isinstance(ids, list):
            baixa_cheque_id = ids[0]
        else:
            baixa_cheque_id = ids
        context = context and context.copy() or {}

        args_financeiro = []
        fields_to_read = ['wbt_data_deposito', 'wbt_valor_pago', 'wbt_documento_tipo_id',
                          'wbt_data_bom_para', 'wbt_data_deposito', 'wbt_chq_banco',
                          'wbt_pessoa_id']
        baixa_cheque_dados = self.read(cr, uid, baixa_cheque_id, fields_to_read, context=context, load=READ_LOAD)
        if baixa_cheque_dados and isinstance(baixa_cheque_dados, list):
            baixa_cheque_dados = baixa_cheque_dados[0]
        if isinstance(baixa_cheque_dados, dict):
            baixa_data_pagamento = baixa_cheque_dados.get('wbt_data_deposito') or ''
            baixa_filtro_data_bom_para = baixa_cheque_dados.get('wbt_data_bom_para') or ''
            baixa_filtro_cheque_banco = baixa_cheque_dados.get('wbt_chq_banco') or ''
            baixa_filtro_pessoa_id = baixa_cheque_dados.get('wbt_pessoa_id') or False



            # CONVERTE A DATA DE PAGAMENTO PARA OBJETO DO TIPO DATE (VERIFICA SE É UMA DATA VÁLIDA)
            date_baixa_pagamento = self.pool.get('financeiro').financ_convert_str_date(
                str_data=baixa_data_pagamento, str_data_name='pagamento', context=context
            )

            obj_financeiro = self.pool.get('financeiro.titulo')

            # PROCURA POR TODOS OS TITULOS DO TIPO CHEQUE QUE ESTAO EM ABERTO PARA O CLIENTE INDICADO
            args_financeiro.append(('state', '=', 'A'))  # A - Aberto.
            args_financeiro.append(('doc_tipo_id', '=', 'CHQ'))
            args_financeiro.append(('fintit_cheque', '=', True))
            args_financeiro.append(('data_prorrogacao', '=', baixa_filtro_data_bom_para))
            if baixa_filtro_cheque_banco:
                args_financeiro.append(('fintit_chq_banco', '=', baixa_filtro_cheque_banco))

            if baixa_filtro_pessoa_id:
                args_financeiro.append(('pessoa_id', '=', baixa_filtro_pessoa_id))

            financeiro_ids = obj_financeiro.search(cr, uid, args_financeiro, context=context)
            if not financeiro_ids:
                msg = _('Nenhum cheque encontrado!')
                raise osv.except_osv(_('Atencao'), msg)
            else:
                # PARA CADA TITULO ENCONTRADO SERÁ CRIADO UM REGISTRO NO WIZARD (DETALHE)
                browse_titulo = obj_financeiro.browse(cr, uid, financeiro_ids)

                # EXCLUI DA TREEVIEW OS TITULOS JA LISTADOS.
                obj_baixa_cheque_detalhe = self.pool.get('wizard.baixa.cheque.detalhe')
                baixa_titulo_detalhe_ids = obj_baixa_cheque_detalhe.search(
                    cr, uid, [('wbtd_baixa_id', '=', baixa_cheque_id)], context=context
                )
                if baixa_titulo_detalhe_ids:
                    obj_baixa_cheque_detalhe.unlink(cr, uid, baixa_titulo_detalhe_ids, context)

                for i in browse_titulo:
                    titulo_id = i.id
                    titulo_pessoa_id = i.pessoa_id and i.pessoa_id.id or False
                    titulo_data_emissao = i.data_emissao
                    titulo_data_prorrogacao = i.data_prorrogacao
                    titulo_valor_total = i.vr_original


                    vals_detalhe = {
                        'wbtd_baixa_id': baixa_cheque_id,
                        'wbtd_titulo_id': titulo_id,
                        'wbtd_pessoa_id': titulo_pessoa_id,
                        'wbtd_data_emissao': titulo_data_emissao,
                        'wbtd_data_prorrogacao': titulo_data_prorrogacao,
                        'wbtd_valor_total': titulo_valor_total,
                        'wbtd_titulo_tipo': 'R',
                    }
                    # _________________________[CREATE] WIZARD BAIXA TITULO DETALHE__________________________________
                    obj_baixa_cheque_detalhe.create(cr, uid, vals_detalhe, context)
                self.write(cr, uid, ids, {'state': 'TL', 'wbt_valor_saldo_restante': 0}, context)
        return True

    def wbt_get_forma_pagamento_dinheiro(self, cr, uid, context=None):
        context = context and context.copy() or {}
        forma_pagamento_id = self.pool.get('adm.forma.pgto').search(
            cr, uid, [('name', 'ilike', 'dinheiro')] , context=context
        )
        if forma_pagamento_id:
            return forma_pagamento_id[0]
        else:
            return False

    def wbt_button_baixar_cheque(self, cr, uid, ids, context=None):
        """
        Funcao realiza a baixa de cheques
        """
        context = context and context.copy() or {}
        if isinstance(ids, list):
            baixa_cheque_id = ids[0]
        else:
            baixa_cheque_id = ids
        browse_baixa_cheque = self.browse(cr, uid, baixa_cheque_id)

        # WIZARD BAIXA TITULO(VARIAVEIS - NÃO DEVEM RECEBER NENHUMA ATRIBUICAO)
        BT_DATA_PAGAMENTO = browse_baixa_cheque.wbt_data_deposito
        BT_FORMA_PGTO_ID = self.wbt_get_forma_pagamento_dinheiro(cr, uid, context)
        BT_CONTA_ID = browse_baixa_cheque.wbt_conta_id.id


        RECORD_BT_TITULO_DETALHE_IDS = browse_baixa_cheque.wbt_titulo_detalhe_ids

        # FILTRA APENAS OS TITULOS MARCADOS PARA SEREM BAIXADOS.
        record_baixa_titulo_detalhe_ids = filter(lambda i: i.wbtd_flag_selecionado is True, RECORD_BT_TITULO_DETALHE_IDS)
        if not record_baixa_titulo_detalhe_ids:
            raise osv.except_osv(_('Atencao'), _('Para realizar a baixar, e necessario marcar ao menos um cheque!'))

        baixa_titulo_detalhe_ids = map(lambda i: i.id, record_baixa_titulo_detalhe_ids)

        # OBJETOS
        obj_financeiro_utils = self.pool.get('financeiro')
        obj_finaceiro_titulo = self.pool.get('financeiro.titulo')

        total_titulos_a_ser_baixados = len(baixa_titulo_detalhe_ids or [])
        baixa_multipla = True if total_titulos_a_ser_baixados > 1 else False

        msg_erro = u''
        msg_erro_many_titulos = {}


        obj_financeiro_parametro = self.pool.get('financeiro.parametros')
        parametro_ids = obj_financeiro_parametro.search(cr, uid, [], context=context)
        _centro_custo_param_id = False
        _evento_param_id = False
        if parametro_ids:
            brw_financeiro_parametro = obj_financeiro_parametro.browse(cr, uid, parametro_ids[0],
                                                                       context)
            _centro_custo_param_id = brw_financeiro_parametro.\
                finpar_centro_custo_recebimentos_id.id if brw_financeiro_parametro.\
                finpar_centro_custo_recebimentos_id else False
            _evento_param_id = brw_financeiro_parametro. \
                finpar_evento_recebimentos_id.id if brw_financeiro_parametro. \
                finpar_evento_recebimentos_id else False
        titulos = []
        titulo_ids = []
        for i in record_baixa_titulo_detalhe_ids:
            # WIZARD BAIXA TITULO DETALHE(VARIAVEIS - NÃO DEVEM RECEBER NENHUMA ATRIBUICAO)
            BTD_TITULO_ID = i.wbtd_titulo_id.id
            BT_VALOR_PAGO = i.wbtd_valor_total
            BTD_TIPO_TITULO = i.wbtd_titulo_id.tipo_titulo
            BTD_PESSOA_ID = i.wbtd_titulo_id.pessoa_id.id
            titulos.append({'titulo_id': BTD_TITULO_ID,
                            'valor_pago': BT_VALOR_PAGO,
                            'saldo_baixado': BT_VALOR_PAGO,
                            'valor_multa': 0, 'valor_juros': 0,
                            'valor_desconto': 0, 'tipo_titulo': BTD_TIPO_TITULO,
                            'pessoa_id': BTD_PESSOA_ID})
            titulo_ids.append(BTD_TITULO_ID)

        res_baixa = obj_financeiro_utils.financ_baixar_titulo(
            cr, uid,
            futl_bxti_titulos=titulos,
            futl_bxti_data_pagamento=BT_DATA_PAGAMENTO,
            ftul_bxti_forma_pgto_id=BT_FORMA_PGTO_ID,
            futl_bxti_tesouraria_conta_id=BT_CONTA_ID,
            futl_bxti_baixa_multipla=baixa_multipla,
            context=context
        )
        if not res_baixa:
            baixa_erro = obj_financeiro_utils.financ_get_erro()
            if isinstance(baixa_erro, basestring):
                if 'ctx_util_baixa_lt' in baixa_erro:
                    baixa_erro = baixa_erro.replace('ctx_util_baixa_lt', '')
                    key = hash(baixa_erro)
                    if key in msg_erro_many_titulos:
                        msg_erro_many_titulos[key]['titulo_ids'].extend(titulo_ids)
                    else:
                        msg_erro_many_titulos[key] = {'titulo_ids': titulo_ids, 'msg': baixa_erro}
                else:
                    if baixa_erro.strip().endswith(':'):
                        res_name_get = obj_finaceiro_titulo.name_get(
                            cr, uid, titulo_ids, context) or [('', '')]
                        dict_name_get = dict(res_name_get)
                        titulo_name = ''
                        for x in dict_name_get:
                            titulo_name += '%s, ' % \
                                           dict_name_get.get(x) or _('** Titulo nao encontrado **')
                    else:
                        titulo_name = ''
                    msg_erro += '%s %s' % (baixa_erro, titulo_name)

        if msg_erro_many_titulos:
            new_msg = self.wbt_generate_msg_titulos(cr, uid, dict_msg=msg_erro_many_titulos, context=context)
            msg_erro += new_msg

        if msg_erro:
            raise osv.except_osv(_('Atencao'), msg_erro)
        else:
            self.write(cr, uid, ids, {'state': 'BA', 'wbt_valor_pago': 0}, context=context)
            return True

    def wbt_generate_msg_titulos(self, cr, uid, dict_msg, context=None):
        context = context and context.copy() or {}
        msg = u''
        obj_finaceiro_titulo = self.pool.get('financeiro.titulo')
        for i in dict_msg:
            titulo_ids = dict_msg[i]['titulo_ids']
            titulo_msg = dict_msg[i]['msg']

            res_name_get = obj_finaceiro_titulo.name_get(cr, uid, titulo_ids, context) or [('', '')]
            dict_name_get = dict(res_name_get)
            titulo_names = ''
            for t in titulo_ids:
                titulo_names += '%s, ' % (dict_name_get.get(t) or _('** Titulo nao encontrado **'))
            msg += '%s%s' % (titulo_msg, titulo_names)
        return msg
