#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
READ_LOAD = '_classic_write'


class wizard_baixa_titulo_detalhe(osv.TransientModel):


    def wbtd_function_valor_total(self, cr, uid, ids, name, args, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        res = {}.fromkeys(ids, 0)
        fields_to_read = [
            'wbtd_dias_atraso', 'wbtd_juros_por_dia', 'wbtd_multa_total', 'wbtd_desconto_total', 'wbtd_saldo'
        ]
        convert_2_decimal = self.pool.get('financeiro').financ_convert_to_decimal
        titulho_detalhe_dados = self.read(cr, uid, ids, fields_to_read, context=context)
        for i in titulho_detalhe_dados:
            id = i['id']
            multa = convert_2_decimal(i.get('wbtd_multa_total') or 0, places=5)
            juros = convert_2_decimal(i.get('wbtd_juros_por_dia') or 0, places=5)
            desconto = convert_2_decimal(i.get('wbtd_desconto_total') or 0, places=5)
            saldo = convert_2_decimal(i.get('wbtd_saldo') or 0, places=5)
            dias_atraso = i.get('wbtd_dias_atraso') or 0

            total = saldo + juros * dias_atraso + multa - desconto

            res[id] = total
        return res

    _name = 'wizard.baixa.cheque.detalhe'
    _rec_name = 'wbtd_titulo_id'
    _columns = {
        'wbtd_baixa_id': fields.many2one('wizard.baixa.cheque', 'Baixa ID', required=True, ondelete='cascade'),
        'wbtd_titulo_id': fields.many2one('financeiro.titulo', 'Titulo/Parcela', required=True, readonly=True),
        'wbtd_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', required=True, readonly=True),
        'wbtd_data_emissao': fields.date('Data Emissao', readonly=True),
        'wbtd_data_prorrogacao': fields.date('Data Prorrogacao', readonly=True),
        'wbtd_valor_total': fields.float('Valor Total', digits=(15, 2)),
        'wbtd_flag_selecionado': fields.boolean('Titulo Selecionado'),
        'wbtd_titulo_tipo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo', size=1, required=True, readonly=True),


    }
    _defaults = {
        'wbtd_flag_selecionado': False,
    }
    _order = 'wbtd_titulo_id'

    def wbtd_get_baixa_state(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            titulo_detalhe_id = ids[0]
        else:
            titulo_detalhe_id = ids
        baixa_state = ''
        titulo_detalhe_dados = self.read(cr, uid, titulo_detalhe_id, ['wbtd_baixa_id'], context=context, load=READ_LOAD)
        if isinstance(titulo_detalhe_dados, list):
            titulo_detalhe_dados = titulo_detalhe_dados[0]
        if isinstance(titulo_detalhe_dados, dict):
            baixa_id = titulo_detalhe_dados.get('wbtd_baixa_id')
            if baixa_id:
                baixa_dados = self.pool.get('wizard.baixa.cheque').read(cr, uid, baixa_id, ['state'], context=context)
                if isinstance(baixa_dados, list):
                    baixa_dados = baixa_dados[0]
                if isinstance(baixa_dados, dict):
                    baixa_state = baixa_dados.get('state')
        return baixa_state

    def wbtd_check_selecionar_desmarcar_titulo(self, cr, uid, ids, context=None):
        """
        Funcao verificar o state da baixa, caso os valores tenham sido atualizados,
        nao permite que os titulos sejam marcados ou desmarcados.
        """
        context = context and context.copy() or {}
        baixa_state = self.wbtd_get_baixa_state(cr, uid, ids, context)
        if baixa_state != 'TL':
            raise osv.except_osv(
                _('Erro - Titulo nao pode ser alterado'),
                _('Apos os valores terem sido atualizados, o titulo nao pode ser marcado/desmarcado para ser baixado.')
            )
        return True

    def wbtd_button_selecionar_titulo(self, cr, uid, ids, context=None):
        """
        Funcao seleciona um titulo.
        """
        context = context and context.copy() or {}
        self.wbtd_check_selecionar_desmarcar_titulo(cr, uid, ids, context=context)
        res = self.write(cr, uid, ids, {'wbtd_flag_selecionado': True}, context=context)
        return res

    def wbtd_button_desmarcar_titulo(self, cr, uid, ids, context=None):
        """
        Funcao desmarca um titulo.
        """
        self.wbtd_check_selecionar_desmarcar_titulo(cr, uid, ids, context=context)
        res = self.write(cr, uid, ids, {'wbtd_flag_selecionado': False}, context=context)
        return res
