#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from dateutil.parser import parse
from datetime import date
READ_LOAD = '_classic_write'


class wizard_baixa_titulo_detalhe(osv.TransientModel):

    def wbtd_default_get_altera_valores(self, cr, uid, context=None):
        """
        Funcao utilizada no campo function - wbtd_flag_altera_valores,
        para quando o osv_memory for aberto buscar o parametro.
        """
        context = context and context.copy() or {}
        res = self.pool.get('financeiro').financ_get_parametro_field(
            cr, uid, field_to_read='alterar_multa_juros', context=context)
        return res

    def wbtd_get_valores(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        res = {}
        parametros = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,
                'financeiro.parametros', ['alterar_multa_juros', 'usar_centro_custos_e_eventos'],
                context=context)
        altera_juros_multa = parametros.get('alterar_multa_juros')
        usar_custificacao = parametros.get('usar_centro_custos_e_eventos')
        for titulo_baixa in self.read(cr, uid, ids, ['wbtd_flag_selecionado', 'wbtd_baixa_id',
                                'wbtd_data_prorrogacao'], load='_classic_write', context=context):
            dados_baixa = self.pool.get('wizard.baixa.titulo').read(cr, uid,
                 titulo_baixa.get('wbtd_baixa_id'), ['state'], context=context)

            if titulo_baixa.get('wbtd_flag_selecionado'):
                altera_juros_multa = False

            res[titulo_baixa.get('id')] = {'wbtd_flag_altera_valores': altera_juros_multa,
                                           'wbtd_flag_custificacao': usar_custificacao,
                                           'wbtd_state_baixa': dados_baixa.get('state')}


        return res

    def wbtd_default_get_utiliza_custificacao(self, cr, uid, context=None):
        """
        Funcao utilizada no campo function - wbtd_flag_custificacao,
        para quando o osv_memory for aberto buscar o parametro.
        """
        context = context and context.copy() or {}
        res = self.pool.get('financeiro').financ_get_parametro_field(cr, uid, field_to_read='usar_centro_custos_e_eventos', context=context)
        return res

    def wbtd_function_get_stored_values(self, cr, uid, ids, name, args, context=None):
        context = context and context.copy() or {}
        res = {}
        convert_2_decimal = self.pool.get('financeiro').financ_convert_to_decimal
        fields_to_read = ['wbtd_dias_atraso', 'wbtd_juros_por_dia', 'wbtd_multa_total',
                          'wbtd_desconto_total', 'wbtd_vr_acrescimo', 'wbtd_saldo']
        for dados_baixa in self.read(cr, uid, ids, fields_to_read, context=context):
            juros_por_dia = dados_baixa.get('wbtd_juros_por_dia') or 0
            dias_atraso = dados_baixa.get('wbtd_dias_atraso') or 0
            vr_juros = convert_2_decimal(juros_por_dia * dias_atraso, places=5)
            multa = convert_2_decimal(dados_baixa.get('wbtd_multa_total') or 0, places=5)
            desconto = convert_2_decimal(dados_baixa.get('wbtd_desconto_total') or 0, places=5)
            acrescimo = convert_2_decimal(dados_baixa.get('wbtd_vr_acrescimo') or 0, places=5)
            saldo = convert_2_decimal(dados_baixa.get('wbtd_saldo') or 0, places=5)
            total = saldo + vr_juros + multa - desconto + acrescimo
            res[dados_baixa.get('id')] = {'wbtd_juros_total': vr_juros,
                                          'wbtd_valor_total': float(self._float_persist(
                                                                    'wbtd_valor_total', total))
                                          }

        return res


    _name = 'wizard.baixa.titulo.detalhe'
    _rec_name = 'wbtd_titulo_id'
    _columns = {
        'wbtd_baixa_id': fields.many2one('wizard.baixa.titulo', 'Baixa ID', required=True,
                                         ondelete='cascade'),
        'wbtd_titulo_id': fields.many2one('financeiro.titulo', 'Titulo/Parcela', required=True,
                                          readonly=True),
        'wbtd_pessoa_id': fields.many2one('adm.pessoa', 'Cliente/Fornecedor', required=True,
                                          readonly=True),
        'wbtd_data_emissao': fields.date('Data Emissao', readonly=True),
        'wbtd_data_prorrogacao': fields.date('Prorrogacao', readonly=True),
        'wbtd_multa_apos_data': fields.date('Multa apos data', readonly=True),
        'wbtd_dias_atraso': fields.function(wbtd_get_valores, string='Dias em Atraso',
                                            multi="wbtd_valores",
                                            method=True, type='char', store=False),
        'wbtd_multa_apos_data': fields.date('Multa apos data', readonly=True),
        'wbtd_dias_atraso': fields.integer('Dias em Atraso', readonly=True),
        'wbtd_juros_por_dia': fields.float('Juros p/ Dia', digits=(15, 5)),
        'wbtd_juros_total': fields.function(wbtd_function_get_stored_values, string="Vr. Juros",
                                            method=True, type="float", digits=(15, 2),
                                            multi="wbtd_stored_values", store=True),
        'wbtd_multa_total': fields.float('Vr. Multa', digits=(15, 2)),
        'wbtd_desconto_total': fields.float('Vr. Desconto', digits=(15, 2)),
        'wbtd_vr_acrescimo': fields.float('Vr acrescimo', digits=(15, 2)),
        'wbtd_saldo': fields.float('Saldo do Titulo', digits=(15, 2), required=True, readonly=True),
        'wbtd_valor_total': fields.function(wbtd_function_get_stored_values, method=True,
                                            string='Valor Total', type="float", digits=(15, 2),
                                            multi="wbtd_stored_values", store=True),
        'wbtd_flag_selecionado': fields.boolean('Titulo Selecionado'),
        'wbtd_titulo_tipo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo', size=1,
                                             required=True, readonly=True),
        'wbtd_flag_altera_valores': fields.function(wbtd_get_valores, multi="wbtd_valores",
                                                    string='Alterar Juros, Multa e Desconto',
                                                    method=True, type='boolean', store=False),
        'wbtd_flag_custificacao': fields.function(wbtd_get_valores, multi="wbtd_valores",
                                                  string='Utiliza Custificacao', method=True,
                                                  type='boolean', store=False),
        'wbtd_state_baixa': fields.function(wbtd_get_valores, string='Estado baixa',
                                            multi="wbtd_valores",
                                            method=True, type='char', store=False),
    }
    _defaults = {
        'wbtd_flag_selecionado': False,
        'wbtd_flag_altera_valores': wbtd_default_get_altera_valores,
        'wbtd_flag_custificacao': wbtd_default_get_utiliza_custificacao,
    }
    def get_valor_total(self, cr, uid, ids, vr_juros=0, multa=0, desconto=0, acrescimo=0, saldo=0,
                        context=None):
        context = context and context.copy() or {}
        convert_2_decimal = self.pool.get('financeiro').financ_convert_to_decimal
        res = {}
        for id in ids:
            vr_juros = convert_2_decimal(vr_juros, places=5)
            multa = convert_2_decimal(multa, places=5)
            desconto = convert_2_decimal(desconto, places=5)
            acrescimo = convert_2_decimal(acrescimo, places=5)
            saldo = convert_2_decimal(saldo, places=5)
            total = saldo + vr_juros + multa - desconto + acrescimo
            res[id] = float(self._float_persist('wbtd_valor_total', total))

        return res

    def wbtd_onchange_juros(self, cr, uid, ids, juros, dias_atraso, context=None):
        context = context and context.copy() or {}
        return {'value':  {'wbtd_juros_total': juros*dias_atraso}}

    def wbtd_onchange_juros_total(self, cr, uid, ids, vr_juros, multa, desconto, acrescimo, saldo=0,
                            context=None):
        context = context and context.copy() or {}
        valores = self.get_valor_total(cr, uid, ids, vr_juros, multa, desconto, acrescimo,
                                            saldo, context=context)
        res = {}
        for id in valores:
            res['value'] = {'wbtd_valor_total': valores.get(id)}

        return res

    def wbtd_onchange_multa(self, cr, uid, ids, vr_juros, multa, desconto, acrescimo, saldo=0,
                            context=None):
        context = context and context.copy() or {}
        valores = self.get_valor_total(cr, uid, ids, vr_juros, multa, desconto, acrescimo,
                                            saldo, context=context)
        res = {}
        for id in valores:
            res['value'] = {'wbtd_valor_total': valores.get(id)}

        return res

    def wbtd_onchange_desconto(self, cr, uid, ids, vr_juros, multa, desconto, acrescimo, saldo=0,
                               context=None):
        context = context and context.copy() or {}
        valores = self.get_valor_total(cr, uid, ids, vr_juros, multa, desconto, acrescimo,
                                            saldo, context=context)
        res = {}
        for id in valores:
            res['value'] = {'wbtd_valor_total': valores.get(id)}

        return res

    def wbtd_onchange_vr_acrescimo(self, cr, uid, ids, vr_juros, multa, desconto, acrescimo,
                                   saldo=0, context=None):
        context = context and context.copy() or {}
        valores = self.get_valor_total(cr, uid, ids, vr_juros, multa, desconto, acrescimo,
                                            saldo, context=context)
        res = {}
        for id in valores:
            res['value'] = {'wbtd_valor_total': valores.get(id)}

        return res

    def wbtd_get_baixa_state(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            titulo_detalhe_id = ids[0]
        else:
            titulo_detalhe_id = ids
        baixa_state = ''
        titulo_detalhe_dados = self.read(cr, uid, titulo_detalhe_id, ['wbtd_baixa_id'], context=context, load=READ_LOAD)
        if isinstance(titulo_detalhe_dados, list):
            titulo_detalhe_dados = titulo_detalhe_dados[0]
        if isinstance(titulo_detalhe_dados, dict):
            baixa_id = titulo_detalhe_dados.get('wbtd_baixa_id')
            if baixa_id:
                baixa_dados = self.pool.get('wizard.baixa.titulo').read(cr, uid, baixa_id, ['state'], context=context)
                if isinstance(baixa_dados, list):
                    baixa_dados = baixa_dados[0]
                if isinstance(baixa_dados, dict):
                    baixa_state = baixa_dados.get('state')
        return baixa_state

    def wbtd_check_selecionar_desmarcar_titulo(self, cr, uid, ids, context=None):
        """
        Funcao verificar o state da baixa, caso os valores tenham sido atualizados,
        nao permite que os titulos sejam marcados ou desmarcados.
        """
        context = context and context.copy() or {}
        baixa_state = self.wbtd_get_baixa_state(cr, uid, ids, context)
        if baixa_state != 'TL':
            raise osv.except_osv(
                _('Erro - Titulo nao pode ser alterado'),
                _('Apos os valores terem sido atualizados, o titulo nao pode ser marcado/desmarcado para ser baixado.')
            )
        return True

    def wbtd_button_selecionar_titulo(self, cr, uid, ids, context=None):
        """
        Funcao seleciona um titulo.
        """
        context = context and context.copy() or {}
        self.wbtd_check_selecionar_desmarcar_titulo(cr, uid, ids, context=context)
        self.write(cr, uid, ids, {'wbtd_flag_selecionado': True, 'teste':True}, context=context)
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'financeiro', 'view_wizard_baixa_titulo_form')
        view_id = reference and reference[1] or False
        baixa_id = self.read(cr, uid, ids, ['wbtd_baixa_id'], load='_classic_write',
                             context=context)[0].get('wbtd_baixa_id')
        self.pool.get('wizard.baixa.titulo').write(cr, uid, [baixa_id], {}, context)

        return {
            'name': 'Baixa de titulos',
            'res_id': baixa_id,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.baixa.titulo',
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'inline'}

    def wbtd_button_desmarcar_titulo(self, cr, uid, ids, context=None):
        """
        Funcao desmarca um titulo.
        """
        context = context and context.copy() or {}
        self.wbtd_check_selecionar_desmarcar_titulo(cr, uid, ids, context=context)
        self.write(cr, uid, ids, {'wbtd_flag_selecionado': False}, context=context)
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'financeiro', 'view_wizard_baixa_titulo_form')
        view_id = reference and reference[1] or False
        baixa_id = self.read(cr, uid, ids, ['wbtd_baixa_id'], load='_classic_write',
                             context=context)[0].get('wbtd_baixa_id')
        self.pool.get('wizard.baixa.titulo').write(cr, uid, [baixa_id], {}, context)
        return {
            'name': 'Baixa de titulos',
            'res_id': baixa_id,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.baixa.titulo',
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'inline'}

    def wbtd_onchange_valores(self, cr, uid, ids, dias_atraso, juros, multa, desconto, saldo, context=None):
        """
        Ao alterar juros, multa ou desconto, e refeito o calculo para o valor total.
        """
        context = context and context.copy() or {}
        res = {'value': {}}
        valor_juros = juros * dias_atraso
        valor_total = valor_juros + multa + saldo - desconto
        res['value'].update({'wbtd_valor_total': float(self._float_persist(
            'wbtd_valor_total', valor_total))})
        return res
