# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
import time
from dateutil.parser import parse
from funcoes import wizgei_get_email
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
from openerp import pooler


class wizard_gera_emails_cobranca(osv.TransientModel):
    # prefixo: wizgec_

    _name = 'wizard.gera.emails.cobranca'
    _columns = {
        'wizgec_mensagem_id': fields.many2one('financeiro.emails.cadastro', 'Texto', required=True),
        'wizgec_tipo_cobranca': fields.selection([('E', 'E-mail'), ('C', 'Carta')],
                                                 'Tipo de cobranca', size=1, required=True),
        'wizgec_data_inicial': fields.date('Data inicial', required=True),
        'wizgec_data_final': fields.date('Data final', required=True),
        'wizgec_setor_id': fields.many2one('adm.setor', 'Setor'),
        'wizgec_local_cobranca_id': fields.many2one('financeiro.local.cobranca',
                                                    'Local de cobranca'),
        'wizgec_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wizgec_titulo_ids': fields.one2many('wizard.gera.emails.cobranca.item', 'wizgei_capa_id',
                                             'Titulos'),
        'wizgec_email_enviado': fields.boolean('Email enviado'),
        'wizgec_carta_ids': fields.one2many('wizard.gera.emails.cobranca.carta', 'wizgca_capa_id',
                                            'Cartas'),
    }

    _defaults = {
        'wizgec_email_enviado': lambda *a: False,
    }

    def wizgec_button_find(self, cr, uid, ids, context=None):
        """
        O metodo pesquisa por titulos a receber e que se encontram em aberto, obedecendo, tambem,
        aos demais filtros da tela.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        context = context and context.copy() or {}
        cr.execute('DELETE FROM wizard_gera_emails_cobranca_item')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        brw_gera_emails_cobranca = self.browse(cr, uid, ids, context)
        for cobranca in brw_gera_emails_cobranca:
            vals = []
            data_incicial = cobranca.wizgec_data_inicial
            data_final = cobranca.wizgec_data_final
            pessoa_id = cobranca.wizgec_pessoa_id.id
            local_cobranca_id = cobranca.wizgec_local_cobranca_id.id
            setor_id = cobranca.wizgec_setor_id.id
            args_titulo = [('data_prorrogacao', '>=', data_incicial),
                           ('data_prorrogacao', '<=', data_final),
                           ('state', '=', 'A'), ('tipo_titulo', '=', 'R')]
            if pessoa_id:
                args_titulo.append(('pessoa_id', '=', pessoa_id))
            if local_cobranca_id:
                args_titulo.append(('local_cobranca_id', '=', local_cobranca_id))
            # if setor_id:
            #     sql = """
            #         SELECT pessoa_id
            #         FROM adm_pessoa_email
            #         WHERE id in (SELECT email_id
            #                      FROM   adm_m2m_email_setor
            #                      WHERE  setor_id  = %s
            #                      )
            #     """ % setor_id
            #     if pessoa_id:
            #         sql += """
            #               AND pessoa_id = %s
            #         """ % pessoa_id
            #     cr.execute(sql)
            #     pessoa_ids = list(set(map(lambda i: i.get('pessoa_id'), cr.dictfetchall())))
            #     if pessoa_id:
            #         args_titulo.pop(4)
            #     args_titulo.append(('pessoa_id', 'in', pessoa_ids))
            titulo_ids = obj_financeiro_titulo.search(
                cr, uid, args_titulo, order='pessoa_id, documento', context=context)
            brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, titulo_ids, context)
            map(lambda i: vals.append((0, 0, {'wizgei_capa_id': cobranca.id,
                                              'wizgei_pessoa_id': i.pessoa_id.id,
                                              'wizgei_titulo_id': i.id,
                                              'wizgei_documento': i.documento,
                                              'wizgei_data_emissao': i.data_emissao,
                                              'wizgei_vr_original': i.vr_original,
                                              'wizgei_data_prorrogacao': i.data_prorrogacao})),
                brw_financeiro_titulo)
            self.write(cr, uid, [cobranca.id], {'wizgec_titulo_ids': vals,
                                                'wizgec_email_enviado': False}, context)
        return True

    def wizgec_button_send(self, cr, uid, ids, context=None):
        """
        O metodo reune e organiza todos os dados necessarios para a criacao dos emails.
        :param cr: cursor do baco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        context = context and context.copy() or {}
        brw_gera_emails_cobranca = self.browse(cr, uid, ids, context)
        obj_financeiro_emails_mensagens = self.pool.get('financeiro.emails.mensagens')
        obj_adm_util = self.pool.get('adm.utils')
        obj_adm_ocorrencia = self.pool.get('adm.ocorrencia')

        financeiro_parametro_dados = obj_adm_util.admutl_get_parametros(
            cr, uid, 'financeiro', 'finpar_email_server_id', context=context)
        mail_server_id = financeiro_parametro_dados.get('finpar_email_server_id') or False
        if not mail_server_id:
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
            mail_server_id = self.pool.get('res.company').browse(cr, uid, company_id, cotnext=context).servidor_smtp_id.id
        email_enviado = False
        for cobranca in brw_gera_emails_cobranca:
            assunto_email = cobranca.wizgec_mensagem_id.fineca_assunto
            texto_email = cobranca.wizgec_mensagem_id.fineca_corpo
            titulo_ids = filter(lambda i: i.wizgei_email_ids != [], cobranca.wizgec_titulo_ids)
            pessoa_ids = list(set(map(lambda i: i.wizgei_pessoa_id.id, titulo_ids)))
            for pessoa_id in pessoa_ids:
                pessoa_titulo_ids = filter(lambda i: i.wizgei_pessoa_id.id == pessoa_id, titulo_ids)
                titulo_dados = """
                    <table>
                        <tr>
                            <td><b>CLIENTE</b></td>
                            <td><b>CPF/CNPJ</b></td>
                            <td><b>DOCUMENTO</b></td>
                            <td><b>EMISSAO</b></td>
                            <td><b>VALOR</b></td>
                            <td><b>VENCIMENTO</b></td>
                        </tr>
                """
                email_ids = []
                for titulo in pessoa_titulo_ids:
                    documento = titulo.wizgei_documento
                    data_emissao = parse(titulo.wizgei_data_emissao).strftime('%d/%m/%Y')
                    valor = 'R$' + getNumeroEditado(titulo.wizgei_vr_original)
                    vencimento = parse(titulo.wizgei_data_prorrogacao).strftime('%d/%m/%Y')
                    titulo_dados += """
                        <tr>
                            <td>%s</td>
                            <td>%s</td>
                            <td>%s</td>
                            <td>%s</td>
                            <td>%s</td>
                            <td>%s</td>
                        </tr>
                    """ % (titulo.wizgei_pessoa_id.nome, titulo.wizgei_pessoa_id.cpf_cnpj,
                           documento, data_emissao, valor, vencimento)
                    email = titulo.wizgei_email_ids[0].name
                    email_ids.extend(titulo.wizgei_email_ids[1:])
                titulo_dados += """
                    </table>
                """
                email_ids = list(set(email_ids))
                destinararios = ''
                for mail in email_ids:
                    destinararios += mail.name + (', ' if mail != email_ids[-1] else '')
                mail_id = obj_adm_util.admutl_send_email(
                    cr, uid, email=email, assunto=assunto_email, corpo=texto_email,
                    destinatarios=destinararios, titulo_dados=titulo_dados,
                    mail_server_id=mail_server_id, context=context)
                vals = {'create_date': time.strftime('%Y-%m-%d %H:%M:%S'),
                        'fineme_texto_id': cobranca.wizgec_mensagem_id.id,
                        'create_uid': uid,
                        'fineme_pessoa_id': pessoa_id,
                        'fineme_mail_id': mail_id,
                        'fineme_email_boleto': 'E'
                        }
                email_mensagem_id = obj_financeiro_emails_mensagens.create(cr, uid, vals, context)
                if email_mensagem_id:
                    email_enviado = True
                for titulo in pessoa_titulo_ids:
                    sql = """
                        INSERT INTO financeiro_mensagens_m2m_titulos (mensagem_id, titulo_id)
                        VALUES (%s, %s)
                    """ % (email_mensagem_id, titulo.wizgei_titulo_id.id)
                    cr.execute(sql)

                    obj_adm_ocorrencia.create(
                        cr, uid, {'admoco_pessoa_id': pessoa_id,
                                  'admoco_descricao': _('E-mail de cobranca enviado'),
                                  'admoco_titulo_id': titulo.wizgei_titulo_id.id,
                                  'admoco_email_id': email_mensagem_id}, context)
        self.write(cr, uid, ids, {'wizgec_email_enviado': email_enviado}, context)
        return True

    def wizgec_onchange_tipo_cobranca(self, cr, uid, ids, tipo_cobranca, context=None):
        """
        O metodo atribui False ao campo wizgec_setor_id caso o tipo_cobranca seja carta (C), pois
        somente cobranca do tipo e-mail (E) tera setor.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param tipo_cobranca: caracter que representa o tipo de cobranca, com 'E' para e-mail e 'C'
        para carta
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario com o campo e seu respectivo valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if tipo_cobranca == 'C':
            res['value'] = {'wizgec_setor_id': False}
        return res

    def wizgec_button_print_letter(self, cr, uid, ids, context=None):
        """
        O metodo cria as cartas de cobranca em formato pdf e os disponibiliza para download.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True
        :rtype: bool
        """
        context = context and context.copy() or {}
        dados = self.wizgec_get_letter_data(cr, uid, ids, context)
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        vals_list = list()
        cr.execute('DELETE FROM wizard_gera_emails_cobranca_carta WHERE create_uid = %s' % uid)
        for dado in dados:
            report_pdf = report_custom._services.get('report.listagem_carta_cobranca').create(
                cr, uid, ids, dado, context)
            pessoa_nome = obj_adm_pessoa.browse(cr, uid, dado.get('pessoa_id'), context).name
            vals_list.append((0, 0, {'wizgca_pessoa_nome': pessoa_nome+'.pdf',
                                     'wizgca_arquivo': report_pdf.encode('base64')}))
        self.write(cr, uid, ids, {'wizgec_carta_ids': vals_list}, context)
        return True

    def wizgec_get_letter_data(self, cr, uid, ids, context=None):
        """
        O metodo reune e organiza todos os dados necessarios para a criacao das cartas.
        :param cr: cursor do baco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        context = context and context.copy() or {}
        brw_gera_emails_cobranca = self.browse(cr, uid, ids, context)
        obj_financeiro_emails_mensagens = self.pool.get('financeiro.emails.mensagens')
        dados_carta = list()
        for cobranca in brw_gera_emails_cobranca:
            texto = cobranca.wizgec_mensagem_id.fineca_corpo
            list_espacamento = list()
            soma = 0
            contador = -1
            for t in texto:
                contador += 1
                if t != '\n':
                    continue
                soma += 1
                try:
                    if texto[contador + 1] == '\n':
                        continue
                    else:
                        list_espacamento.append(soma)
                        soma = 0
                except:
                    list_espacamento.append(soma)
                    soma = 0
            list_texto = cobranca.wizgec_mensagem_id.fineca_corpo.split('\n')
            pessoa_ids = list(set(map(lambda i: i.wizgei_pessoa_id.id, cobranca.wizgec_titulo_ids)))
            for pessoa_id in pessoa_ids:
                pessoa_titulo_ids = filter(lambda i: i.wizgei_pessoa_id.id == pessoa_id,
                                           cobranca.wizgec_titulo_ids)

                vals = {'create_date': time.strftime('%Y-%m-%d %H:%M:%S'),
                        'fineme_texto_id': cobranca.wizgec_mensagem_id.id,
                        'create_uid': uid,
                        'fineme_pessoa_id': pessoa_id,
                        'fineme_email_carta': 'C',
                        }
                email_mensagem_id = obj_financeiro_emails_mensagens.create(cr, uid, vals, context)
                for titulo in pessoa_titulo_ids:
                    sql = """
                        INSERT INTO financeiro_mensagens_m2m_titulos (mensagem_id, titulo_id)
                        VALUES (%s, %s)
                    """ % (email_mensagem_id, titulo.wizgei_titulo_id.id)
                    cr.execute(sql)
                dados_carta.append({'list_texto': list_texto, 'list_espacamento': list_espacamento,
                                    'titulo_ids': map(lambda i: i.wizgei_titulo_id.id,
                                                      pessoa_titulo_ids),
                                    'pessoa_id': pessoa_id})
        return dados_carta


class wizard_gera_emails_cobranca_item(osv.TransientModel):
    # prefixo: wizgei_

    _name = 'wizard.gera.emails.cobranca.item'
    _columns = {
        'wizgei_capa_id': fields.many2one('wizard.gera.emails.cobranca', 'Capa'),
        'wizgei_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wizgei_documento': fields.char('Documento', size=30),
        'wizgei_data_emissao': fields.date('Emissao'),
        'wizgei_vr_original': fields.float('Valor', digits=(15, 2)),
        'wizgei_data_prorrogacao': fields.date('Vencimento'),
        'wizgei_email_ids': fields.function(wizgei_get_email, method=True, string='Emails',
                                            type='one2many', relation='adm.pessoa.email',
                                            multi='email'),
        'wizgei_possui_email': fields.function(wizgei_get_email, method=True,
                                               string='Possui e-mail', type='boolean',
                                               multi='email'),
        'wizgei_titulo_id': fields.many2one('financeiro.titulo', 'Titulo'),
    }


class wizard_gera_emails_cobranca_carta(osv.TransientModel):
    # prefixo: wizgca_

    _name = 'wizard.gera.emails.cobranca.carta'
    _columns = {
        'wizgca_capa_id': fields.many2one('wizard.gera.emails.cobranca', 'Capa'),
        'wizgca_pessoa_nome': fields.char('Nome', size=100),
        'wizgca_arquivo': fields.binary('Arquivo'),
    }


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        def center_title(e, c, d, t=160):
            title_c = (t - len(c)) / 2
            titulo = e
            for i in range(title_c - len(e)):
                titulo += ' '
            titulo += c
            for i in range(title_c - len(d)):
                titulo += ' '
            titulo += d
            return titulo

        context = context and context.copy() or {}
        obj_financeiro_titulo = pooler.get_pool(cr.dbname).get('financeiro.titulo')
        obj_adm_pessoa = pooler.get_pool(cr.dbname).get('adm.pessoa')
        brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, datas.get('titulo_ids'),
                                                             context)
        brw_adm_pessoa = obj_adm_pessoa.browse(cr, uid, datas.get('pessoa_id'), context)

        list_texto = filter(lambda t: t != '', datas.get('list_texto'))

        # title = center_title('', _('Carta de Cobranca'), '')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato()
        listagem.informaNome(pdf_string, _('carta_cobranca'))
        listagem.defineFormato()
        listagem.defineFonte(tam=10)

        data = time.strftime("%d/%m/%Y")
        docs = formataString(data, 46, ' ', 'D', 'N', 'S')
        docs += formataString(_('Pagina: {{ Page }} '), 58, ' ', 'E', 'N', 'S')

        # listagem.informaCab(''+title)
        listagem.informaCab('')
        listagem.informaCab(''+docs)
        listagem.informaCab(brw_adm_pessoa.name + ' / ' + brw_adm_pessoa.cpf_cnpj)
        listagem.informaCab(brw_adm_pessoa.endereco)
        listagem.informaCab('%s    %s    %s' % (brw_adm_pessoa.cep, brw_adm_pessoa.cidade,
                                                brw_adm_pessoa.uf))
        listagem.informaCab('')
        listagem.informaCab('')
        listagem.informaCab('')
        listagem.informaCab('')
        listagem.informaCab('')
        contador = -1
        for texto in list_texto:
            while texto:
                listagem.informaCab(texto[:93])
                texto = texto[93:]
            contador += 1
            try:
                for espaco in range(0, datas.get('list_espacamento')[contador]):
                    listagem.informaCab('')
            except:
                continue
        listagem.informaCab('')
        listagem.informaCab('')
        listagem.informaCab(_('DOCUMENTO                      EMISSAO               VALOR '
                              'VENCIMENTO '))
        listagem.informaCab('')
        listagem.rodapePadrao()

        for titulo in brw_financeiro_titulo:
            linha = formataString(titulo.documento, 30, ' ', 'D', 'N', 'S')
            linha += ' '
            linha += formataString(parse(titulo.data_emissao).strftime('%d/%m/%Y'), 10, ' ', 'D',
                                   'N', 'S')
            linha += ' '
            linha += formataString(getNumeroEditado(titulo.vr_original), 16, ' ', 'E', 'N', 'S')
            linha += ' '
            linha += formataString(parse(titulo.data_prorrogacao).strftime('%d/%m/%Y'), 10, ' ',
                                   'D', 'N', 'S')
            linha += ' '
            listagem.imprimeLinha(linha)

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return self.obj.pdf

report_custom('report.listagem_carta_cobranca')
