# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from SigUtils.boleto import boleto
import base64
import time
from SigUtils.numero import getNumeroEditado
from reportlab.lib.utils import ImageReader
from PIL import Image
import StringIO
from funcoes import wizgbi_get_email
from openerp.tools.translate import _
from dateutil.parser import parse
from SigUtils import cnab as cnabUtils

class wizard_gerar_boleto(osv.TransientModel):
    """
    Wizard para gerar boletos de titulos
    prefixo: wizgbo_
    """

    def _wizgbo_get_tipo_carteira(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = False
        obj_financeiro_local_cobranca = self.pool.get('financeiro.local.cobranca')
        local_cobranca_ids = obj_financeiro_local_cobranca.search(cr, uid, [('tipo', '=', 'C')], context=context)
        if local_cobranca_ids:
            res = local_cobranca_ids[0]
        return res
    def _wizgbo_get_par_observacoes(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obs = self.pool.get('adm.utils').admutl_get_parametros(cr, uid,'financeiro.parametros', 'finpar_mensagem_boleto')
        if not obs:
            obs = ''
        if isinstance(obs, dict):
            obs = obs.get('finpar_mensagem_boleto')
        return obs

    _name = 'wizard.gerar.boleto'

    _columns = {
        'wizgbo_data_emissao_inicial': fields.date('Data emissao inicial'),
        'wizgbo_data_emissao_final': fields.date('Data emissao final'),
        'wizgbo_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wizgbo_local_cobranca_id': fields.many2one('financeiro.local.cobranca',
                                                    'Local de cobranca'),
        'wizgbo_natureza_cobranca_id': fields.many2one('financeiro.natureza.cobranca',
                                                       'Natureza de cobranca'),
        'wizgbo_documento_tipo_id': fields.many2one('documento.documento.tipo',
                                                    'Tipo de documento'),
        'wizgbo_setor_id': fields.many2one('adm.setor', 'Setor'),
        'wizgbo_mensagem_id': fields.many2one('financeiro.emails.cadastro', 'Texto'),
        'wizgbo_data_vencimento_inicial': fields.date('Data vencimento inicial'),
        'wizgbo_data_vencimento_final': fields.date('Data vencimento final'),
        'wizgbo_bol_banco_id': fields.many2one('financeiro.local.cobranca', 'Banco p/emissao',
                                               required=True, domain="[('tipo', '=', 'B')]"),
        'wizgbo_bol_natureza_cobranca_id': fields.many2one('financeiro.natureza.cobranca',
                                                           'Natureza a utilizar', required=True),
        'wizgbo_via_impressora': fields.boolean('Arquivo em impressora'),
        'wizgbo_impressora': fields.char('Impressora', size=15),
        'wizgbo_nosso_numero': fields.char('Ultimo nosso numero', required=True, size=20),
        'wizgbo_gerar_separado': fields.boolean('Gerar boleto separado',
                                                help='Se marcado: em cada arquivo sera gerado um '
                                                     'boleto. Se desmarcado: sera gerado um '
                                                     'arquivo com todos os boletos'),
        'wizgbo_enviar_email': fields.boolean('Enviar por e-mail'),
        'wizgbo_modelo': fields.selection([('B', 'Boleto'), ('C', 'Carne')], 'Modelo', size=1,
                                          required=True),
        'wizgbo_item_ids': fields.one2many('wizard.gerar.boleto.item', 'wizgbi_gerar_boleto_id',
                                           'Titulos'),
        'wizgbo_arquivo_ids': fields.one2many('wizard.gerar.boleto.arquivo',
                                              'wizgba_gerar_boleto_id', 'Arquivos'),
        'state': fields.selection([('I', 'Inicial'), ('G', 'Gerado')], 'Estado', size=1),
        'wizgbo_observacoes': fields.text('Observacoes', help='As observacoes descritas aqui serao '
                                                             'impressas no boleto')
    }

    _defaults = {
        'wizgbo_local_cobranca_id': _wizgbo_get_tipo_carteira,
        'wizgbo_modelo': 'B',
        'state': 'I',
        'wizgbo_observacoes': _wizgbo_get_par_observacoes,
        'wizgbo_enviar_email': lambda *a: False,
    }

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(wizard_gerar_boleto, self).write(cr, uid, ids, vals, context)
        return res

    def onchange_wizgbo_modelo(self, cr, uid, ids, wizgbo_modelo, context=None):
        context = context and context.copy() or {}
        if not wizgbo_modelo:
            return {}
        res = {}
        if wizgbo_modelo == 'C':
            res['wizgbo_gerar_separado'] = False
        return {'value': res}

    def wizgbo_button_pesquisar(self, cr, uid, ids, context=None):

        """
        Alimenta a tabela wizard.gerar.boleto.item. Cria os itens (titulos) a partir dos parametros
        escolhidos.
        @return: True
        """

        context = context and context.copy() or {}
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_wizard_gerar_boleto_item = self.pool.get('wizard.gerar.boleto.item')
        brw_wizard_gerar_boleto = self.browse(cr, uid, ids[0], context)
        documento_previsao_ids = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)
        args = [('doc_tipo_id', 'not in', documento_previsao_ids)]
        data_emissao_inicial = brw_wizard_gerar_boleto.wizgbo_data_emissao_inicial
        if data_emissao_inicial:
            args.append(('data_emissao', '>=', data_emissao_inicial), )
        data_emissao_final = brw_wizard_gerar_boleto.wizgbo_data_emissao_final
        if data_emissao_final:
            args.append(('data_emissao', '<=', data_emissao_final), )
        pessoa_id = brw_wizard_gerar_boleto.wizgbo_pessoa_id.id
        if pessoa_id:
            args.append(('pessoa_id', '=', pessoa_id))
        local_cobranca_id = brw_wizard_gerar_boleto.wizgbo_local_cobranca_id.id
        if local_cobranca_id:
            args.append(('local_cobranca_id', '=', local_cobranca_id), )
        natureza_cobranca_id = brw_wizard_gerar_boleto.wizgbo_natureza_cobranca_id.id
        if natureza_cobranca_id:
            args.append(('natureza_cobranca_id', '=', natureza_cobranca_id))
        documento_tipo_id = brw_wizard_gerar_boleto.wizgbo_documento_tipo_id.id
        if documento_tipo_id:
            args.append(('doc_tipo_id', '=', documento_tipo_id), )
        data_vencimento_inicial = brw_wizard_gerar_boleto.wizgbo_data_vencimento_inicial
        if data_vencimento_inicial:
            args.append(('data_prorrogacao', '>=', data_vencimento_inicial), )
        data_vencimento_final = brw_wizard_gerar_boleto.wizgbo_data_vencimento_final
        if data_vencimento_final:
            args.append(('data_prorrogacao', '<=', data_vencimento_final), )
        args.extend([('state', '=', 'A'), ('tipo_titulo', '=', 'R')])
        titulo_ids = obj_financeiro_titulo.search(cr, uid, args, context=context)
        if not titulo_ids:
            return True
        brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, titulo_ids, context)
        item_ids = obj_wizard_gerar_boleto_item.search(cr, uid, [], context=context)
        if item_ids:
            obj_wizard_gerar_boleto_item.unlink(cr, uid, item_ids, context)
        for titulo in brw_financeiro_titulo:
            vals = {
                'wizgbi_gerar_boleto_id': ids[0],
                'wizgbi_pessoa_id': titulo.pessoa_id.id,
                'wizgbi_documento': titulo.documento,
                'wizgbi_parcela': titulo.parcela,
                'wizgbi_data_emissao': titulo.data_emissao,
                'wizgbi_vr_original': titulo.vr_original,
                'wizgbi_data_prorrogacao': titulo.data_prorrogacao,
                'wizgbi_titulo_id': titulo.id,
            }
            obj_wizard_gerar_boleto_item.create(cr, uid, vals, context)
        return True

    def wizgbo_button_imprimir(self, cr, uid, ids, context=None):

        """
        Realiza a impressao dos boletos ou carnes, seja em arquivo ou impressora
        @return: True
        """

        context = context and context.copy() or {}
        obj_financeiro_util = self.pool.get('financeiro')
        obj_wizard_gerar_boleto_item = self.pool.get('wizard.gerar.boleto.item')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_financeiro_titulo_boleto = self.pool.get('financeiro.titulo.boleto')
        obj_local_natureza = self.pool.get('financeiro.local.natureza')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'financeiro.titulo', context=context)
        self.write(cr, uid, ids, vals={}, context=context)
        brw_wizard_gerar_boleto = self.browse(cr, uid, ids, context=context)
        boleto_gerado = False
        for capa in brw_wizard_gerar_boleto:
            local_cobranca_id = capa.wizgbo_bol_banco_id.id
            enviar_email = capa.wizgbo_enviar_email
            if enviar_email and not all(map(lambda i: i.wizgbi_possui_email,
                                            capa.wizgbo_item_ids)):
                raise osv.except_osv(_('Atencao'), _('Para imprimir, todos os clientes devem estar '
                                                     'com e-mail cadastrado.'))
            # nosso_numero_informado = long(capa.wizgbo_nosso_numero) or 0
            # if nosso_numero_informado <= 0:
            #     nosso_numero_informado = 0
            # Titulos
            item_ids = capa.wizgbo_item_ids

            via_impressora = capa.wizgbo_via_impressora
            impressora = capa.wizgbo_impressora
            gerar_separado = capa.wizgbo_gerar_separado
            modelo = capa.wizgbo_modelo
            # Se carne (C), gerar todos os carnes num so arquivo
            if modelo == 'C':
                gerar_separado = False
            imprimir = 'arquivo'
            if via_impressora and impressora:
                imprimir = impressora
            # Instancia de boleto
            arq1_boleto = arq_boleto = boleto.Boleto()
            for item in item_ids:
                args = [('finlon_local_cobranca_id', '=', capa.wizgbo_bol_banco_id.id),
                        ('finlon_natureza_cobranca_id', '=', capa.wizgbo_bol_natureza_cobranca_id.id)]
                local_natureza_ids = obj_local_natureza.search(cr, uid, args, context=context)
                if not local_natureza_ids:
                    raise osv.except_osv(_('Atencao'), _("Nao ha Local e Natureza cadastrado para o local de cobranca "
                                       "e natureza de cobranca deste titulo!"))
                local_natureza_id = local_natureza_ids[0]
                nosso_numero_informado = obj_local_natureza.finlon_get_set_nosso_numero(cr, uid, item.wizgbi_titulo_id.id,
                                                                          local_natureza_id, context=context)
                # Trabalhando com a observacao do boleto
                lista_observacoes = []
                _multa = item.wizgbi_titulo_id.vr_multa
                _juros = item.wizgbi_titulo_id.vr_juros_dia
                _desconto = item.wizgbi_titulo_id.vr_desconto
                if _multa:
                    _multa_apos_data = item.wizgbi_titulo_id.multa_apos_data
                    _multa = getNumeroEditado(_multa, decimals=2, dec_point=',')
                    if all([_multa, _multa_apos_data]):
                        _multa_apos_data = time.strptime(_multa_apos_data, '%Y-%m-%d')
                        _multa_apos_data = time.strftime('%d/%m/%Y', _multa_apos_data)
                        _obs = "MULTA DE R$%s   APOS: %s." % (_multa, _multa_apos_data)
                    else:
                        _obs = "MULTA DE R$%s   ATE VENCIMENTO." % _multa
                    lista_observacoes.append(_obs)
                if _juros:
                    _juros = getNumeroEditado(_juros, decimals=2, dec_point=',')
                    _obs = "JUROS DE R$%s   AO DIA." % _juros
                    lista_observacoes.append(_obs)
                if _desconto:
                    _desconto = getNumeroEditado(_desconto, decimals=2, dec_point=',')
                    _data_limite_desconto = item.wizgbi_titulo_id.data_limite_desconto
                    if all([_desconto, _data_limite_desconto]):
                        _data_limite_desconto = time.strptime(_data_limite_desconto, '%Y-%m-%d')
                        _data_limite_desconto = time.strftime('%d/%m/%Y', _data_limite_desconto)
                        _obs = "DESCONTO DE R$%s    ATE %s." % (_desconto, _data_limite_desconto)
                    else:
                        _obs = "DESCONTO DE R$%s    ATE VENCIMENTO." % _desconto
                    lista_observacoes.append(_obs)
                    lista_observacoes.append(" ")
                if capa.wizgbo_observacoes:
                    _obs = obj_financeiro_util.finutl_formata_observacao(
                        67, capa.wizgbo_observacoes)
                    lista_observacoes += map(None, _obs)
                imprimir_obs_tit = self.get_imprimir_inf_titulo(cr, uid)
                observacoes_titulo = item.wizgbi_titulo_id.observacoes
                if observacoes_titulo:
                    if imprimir_obs_tit:
                        _obs = obj_financeiro_util.finutl_formata_observacao(67, observacoes_titulo)
                        lista_observacoes += map(None, _obs)
                # Instancia com os dados do boleto
                arq1_boleto = arq_boleto = self._wizgbo_gerar_boleto(
                    cr, uid, arq_boleto, item, imprimir, company_id, gerar_separado,
                    nosso_numero_informado, lista_observacoes, context)
                titulo_id = item.wizgbi_titulo_id.id
                banco_id = capa.wizgbo_bol_banco_id.finloc_banco_id.id
                nat_id = capa.wizgbo_bol_natureza_cobranca_id.id
                if capa.wizgbo_bol_banco_id.finloc_conta_id.agencia_id:
                    agencia_id = capa.wizgbo_bol_banco_id.finloc_conta_id.agencia_id.id or False
                else:
                    agencia_id = False
                # Gera o boleto separado (1 por arquivo)
                if gerar_separado:
                    arq_boleto.gerarBoleto()
                    boleto_gerado = True
                    self._wizgbo_inclui_titulo_boleto(
                        cr, uid, [titulo_id], nosso_numero_informado, local_cobranca_id,
                        arq_boleto.titulo.codigoBarras or '', gerar_separado, context)
                if gerar_separado and imprimir == 'arquivo':
                    # inserir na tabela wizard_gerar_boleto_arquivo se impressao for do tipo
                    # arquivo
                    impressao = arq_boleto.cedente.impressora
                    self._wizgbo_criar_arquivo_download(cr, uid, capa.id, impressao, context)
                    boleto_gerado = True

                # Registrar titulo para o banco informado
                #nosso_numero_completo = arq_boleto.nossoNumeroCompleto
                #Testado para a CEF, não registra nosso numero completo, porque no retorno não é ele que vem
                self._wizgbo_registrar_titulo(cr, uid, titulo_id, local_cobranca_id, nat_id,
                                              nosso_numero_informado, context)

                # Cria arquivo pdf para ser anexo ao e-mail que sera enviado
                if not enviar_email:
                    continue
                arq2_boleto = boleto.Boleto()
                arq2_boleto = self._wizgbo_gerar_boleto(
                    cr, uid, arq2_boleto, item, imprimir, company_id, True,
                    nosso_numero_informado, lista_observacoes, context)
                arq2_boleto.cedente.impressora = 'arquivo/tmp/boleto_email_%s.pdf' % time.strftime(
                    '%H:%M:%S')
                arq2_boleto.gerarBoleto()
                impressao = arq2_boleto.cedente.impressora
                local_arquivo = impressao[7:]
                arquivo = open(local_arquivo, 'r')
                read_arquivo = arquivo.read()
                boleto_binary = base64.encodestring(read_arquivo)
                obj_wizard_gerar_boleto_item.wizgbi_gera_email(cr, uid, item.id, boleto_binary,
                                                               context)

            # Gera os boletos (todos em um arquivo)
            if not gerar_separado and arq1_boleto:
                arq1_boleto.gerarBoleto()
                boleto_gerado = True
                for bol in arq1_boleto._Boleto__titulos:
                    titulo_boleto_ids = obj_financeiro_titulo_boleto.search(
                        cr, uid, [('fintib_nosso_numero', '=', bol.nossoNroSeq)], context=context)
                    titulo_ids = map(lambda i: i.fintib_titulo_id.id,
                                     obj_financeiro_titulo_boleto.browse(cr, uid, titulo_boleto_ids,
                                                                         context))
                    args = [('documento', '=', bol.documento),
                            ('data_emissao', '=', parse(bol.emissao).strftime('%Y-%m-%d'))]
                    if titulo_ids:
                        args.append(('id', 'in', titulo_ids))
                    titulo_ids = obj_financeiro_titulo.search(cr, uid, args, context=context)
                    if titulo_ids:
                        self._wizgbo_inclui_titulo_boleto(cr, uid, titulo_ids, bol.nossoNroSeq,
                                                          local_cobranca_id, bol.codigoBarras or '',
                                                          gerar_separado, context)

            # inserir na tabela wizard_gerar_boleto_arquivo se impressao for do tipo arquivo
            if imprimir == 'arquivo' and not gerar_separado:
                impressao = arq_boleto.cedente.impressora
                self._wizgbo_criar_arquivo_download(cr, uid, capa.id, impressao, context)
                boleto_gerado = True
            # _id = obj_local_natureza.search(cr, uid, [
            #     ('finlon_local_cobranca_id', '=', capa.wizgbo_bol_banco_id.id),
            #     ('finlon_natureza_cobranca_id', '=', nat_id)], context=context)
            # if _id:
            #     finlon_nosso_numero = obj_local_natureza.browse(
            #         cr, uid, _id[0], context).finlon_nosso_numero or 0
            #     if int(finlon_nosso_numero) > int(nosso_numero_informado):
            #         # _id = []
            # self.pool.get('financeiro.local.natureza').write(cr, uid, _id,
            #                                         {'finlon_nosso_numero': nosso_numero_informado}, context)
        if boleto_gerado:
            self.write(cr, uid, ids, {'state': 'G'}, context)
        return True

    def _wizgbo_inclui_titulo_boleto(self, cr, uid, titulo_ids, nosso_numero, local_cobranca_id,
                                     codigo_barras, gerar_separado, context=None):
        """
        O metodo inclui dados na tabela financeiro_titulo_boleto de acordo com os parametros
        recebidos
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param titulo_ids: lista de id da tabela financeiro_titulo
        :type: list
        :param nosso_numero: sequencia de caracteres que identifica o documento no banco.
        :type: char
        :param local_cobranca_id: id da tabela financeiro_local_cobranca.
        :type: integer
        :param codigo_barras: sequencia de caracteres que identifica o documento.
        :type: char
        :param gerar_separado: booleano que identifica se os boletos serao gerados num unico arquivo
        ou em arquivos separados.
        :type: bool
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_financeiro_titulo_boleto = self.pool.get('financeiro.titulo.boleto')
        args = [('fintib_titulo_id', 'in', titulo_ids),
                ('fintib_nosso_numero', '=', nosso_numero),
                ('fintib_local_cobranca_id', '=', local_cobranca_id)]
        # if gerar_separado:
        #     args.append(('fintib_codigo_barras', '=', codigo_barras))
        titulo_boleto_ids = obj_financeiro_titulo_boleto.search(cr, uid, args, context=context)
        if gerar_separado and not titulo_boleto_ids:
            obj_financeiro_titulo_boleto.create(
                cr, uid, {'fintib_titulo_id': titulo_ids[0],
                          'fintib_nosso_numero': nosso_numero,
                          'fintib_boleto_impresso': True,
                          'fintib_local_cobranca_id': local_cobranca_id,
                          'fintib_codigo_barras': codigo_barras}, context)
        elif titulo_boleto_ids:
            obj_financeiro_titulo_boleto.write(
                cr, uid, titulo_boleto_ids, {'fintib_codigo_barras': codigo_barras, 'fintib_boleto_impresso': True},
                context)
        return True

    def _wizgbo_registrar_titulo(self, cr, uid, titulo_id, local_id, nat_id,
                                 nosso_numero, context=None):

        """
        Realiza o registro de titulos do financeiro de acordo com o banco selecionado
        @return: True: o titulo foi registrado, False: o titulo nao foi registrado
        """

        context = context and context.copy() or {}
        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')
        args = [('finlon_local_cobranca_id', '=', local_id), ('finlon_natureza_cobranca_id', '=', nat_id)]
        local_natureza_ids = obj_financeiro_local_natureza.search(cr, uid, args, context=context)
        brw_local_natureza = self.pool.get('financeiro.local.natureza').browse(cr, uid, local_natureza_ids[0])
        obj_financeiro_titulo_boleto = self.pool.get('financeiro.titulo.boleto')
        brw_titulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context=context)
        res, digitavel, scaneavel = cnabUtils.cnab_get_codigo_barra(brw_local_natureza, brw_titulo, nosso_numero)
        args = [('fintib_titulo_id', '=', titulo_id), ('fintib_nosso_numero', '=', nosso_numero),
                ('fintib_local_cobranca_id', '=', local_id)]
        titulo_boleto_ids = obj_financeiro_titulo_boleto.search(cr, uid, args, context=context)
        if not titulo_boleto_ids:
            vals = {
                'fintib_local_cobranca_id': local_id,
                'fintib_natureza_cobranca_id': nat_id,
                'fintib_titulo_id': titulo_id,
                'fintib_nosso_numero': nosso_numero,
                'fintib_codigo_barras': digitavel,
            }
            titulo_boleto_ids = obj_financeiro_titulo_boleto.create(cr, uid, vals, context)
        vals = {'local_cobranca_id': local_id, 'natureza_cobranca_id': nat_id}
        self.pool.get('financeiro.titulo').write(cr, uid, titulo_id, vals, context=context)
        return True

    def _wizgbo_criar_arquivo_download(self, cr, uid, gerar_boleto_id, impressao, context=None):

        """
        Alimenta a tabela wizard.gerar.boleto.arquivo. Cria os arquivos de boleto ou carne para
        download, caso nao seja selecionado arquivo em impressora
        @return: True
        """

        context = context and context.copy() or {}
        obj_wizard_gerar_boleto_arquivo = self.pool.get('wizard.gerar.boleto.arquivo')
        nome_arquivo = impressao[12:]
        local_arquivo = impressao[7:]
        arquivo = open(local_arquivo, 'r')
        arquivos = arquivo.read()
        arquivo = base64.encodestring(arquivos)
        vals = {'wizgba_gerar_boleto_id': gerar_boleto_id,
                'wizgba_dado_boleto': arquivo,
                'wizgba_arquivo_boleto': nome_arquivo,
        }
        if 'wizard_gerar_boleto_titulo' in context:
            vals['wizgba_gerar_boleto_titulo_id'] = context.get('wizard_gerar_boleto_titulo')
        res = obj_wizard_gerar_boleto_arquivo.create(cr, uid, vals, context)
        return True

    def _wizgbo_get_dado_cedente(self, cr, uid, company_id, local_cobranca_id, natureza_cobranca_id,
                                 context=None):

        """
        Busca todos os dados pertencente ao cedente
        @company_id: id da tabela res_company
        @local_cobranca_id: id da tabela financeiro_local_cobranca
        @natureza_cobranca_id: id da tabela financeiro_natureza_cobranca
        @return: Dicionario
        """

        context = context and context.copy() or {}
        obj_res_company = self.pool.get('res.company')
        brw_res_company = obj_res_company.browse(cr, uid, company_id, context)
        dado_empresa = brw_res_company.rescon_pessoa_id
        cedente_nome = dado_empresa.nome or ''
        cedente_cpf_cnpj = dado_empresa.cpf_cnpj or ''
        cedente_telefone = dado_empresa.telefone or ''

        # Busca logomarca da empresa
        logo = brw_res_company.logo or ''
        if logo:
            logo = base64.decodestring(logo)
            logo = StringIO.StringIO(logo)
            logo.seek(0)
            max_size = 1920
            im = Image.open(logo)
            im = im.convert('RGB')
            nova_logo = StringIO.StringIO()
            im.thumbnail([max_size, max_size], Image.BILINEAR)
            im.save(nova_logo, format='JPEG')
            nova_logo.seek(0)
            logo = ImageReader(nova_logo)

        res = {'convenio': '', 'agencia': '', 'conta': '', 'carteira': '',
               'cedente_nome': cedente_nome, 'cedente_cpf_cnpj': cedente_cpf_cnpj,
               'cedente_telefone': cedente_telefone, 'cedente_logo': logo}
        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')
        args = [('finlon_local_cobranca_id', '=', local_cobranca_id),
                ('finlon_natureza_cobranca_id', '=', natureza_cobranca_id)]
        local_natureza_ids = obj_financeiro_local_natureza.search(cr, uid, args, context=context)
        if local_natureza_ids:
            brw_financeiro_local_natureza = obj_financeiro_local_natureza.browse(
                cr, uid, local_natureza_ids[0])
            res = {
                'convenio': brw_financeiro_local_natureza.finlon_convenio or '',
                'agencia': brw_financeiro_local_natureza.finlon_agencia or '',
                'agencia_dv': brw_financeiro_local_natureza.finlon_agencia_dv or '',
                'conta': brw_financeiro_local_natureza.finlon_conta or '',
                'conta_dv': brw_financeiro_local_natureza.finlon_conta_dv or '',
                'carteira': brw_financeiro_local_natureza.finlon_carteira or '',
                'cedente_nome': cedente_nome,
                'cedente_cpf_cnpj': cedente_cpf_cnpj,
                'cedente_telefone': cedente_telefone,
                'cedente_logo': logo,
                'modalidade': brw_financeiro_local_natureza.finlon_registro_banco or ''
            }
        return res

    def _wizgbo_get_dado_pessoa(self, cr, uid, brw_adm_pessoa, context=None):

        """
        Busca todos os dados pertencente a pessoa (sacado)
        @brw_adm_pessoa: browse da tabela adm_pessoa
        @return: Dicionario
        """

        context = context and context.copy() or {}
        res = {
            'endereco_rua': '',
            'endereco_numero': '',
            'endereco_bairro': '',
            'endereco_cidade': '',
            'endereco_estado': '',
            'endereco_cep': '',
            'nome': brw_adm_pessoa.nome or '',
            'codigo': brw_adm_pessoa.codigo or 0,
            'cpf_cnpj': brw_adm_pessoa.cpf_cnpj or ''
        }
        endereco_id = self.pool.get('adm.pessoa').admpes_get_cobranca_endereco_id(
            cr, uid, brw_adm_pessoa.id, context)
        dado_pessoa_endereco = filter(lambda end: end.id == endereco_id,
                                      brw_adm_pessoa.endereco_ids)
        if dado_pessoa_endereco:
            dado_pessoa_endereco = dado_pessoa_endereco[0]
            logradouro_id = dado_pessoa_endereco.logradouro_id
            if logradouro_id:
                res['endereco_rua'] = '%s %s' % (logradouro_id.titulo, logradouro_id.descricao)
            res['endereco_numero'] = dado_pessoa_endereco.numero or ''
            bairro_id = dado_pessoa_endereco.bairro_id
            if bairro_id:
                res['endereco_bairro'] = bairro_id.name or ''
            cidade_id = dado_pessoa_endereco.cidade_id
            if cidade_id:
                res['endereco_cidade'] = cidade_id.name or ''
            uf_id = dado_pessoa_endereco.uf_id
            if uf_id:
                res['endereco_estado'] = uf_id.sigla or ''
            res['endereco_cep'] = dado_pessoa_endereco.cep or ''
        return res

    def _wizgbo_gerar_boleto(self, cr, uid, arq_boleto, brw_gerar_boleto_item, imprimir, company_id,
                             gerar_boleto_separado, nosso_numero_informado, observacoes=None,
                             context=None):

        """
        Gera os arquivos de boleto e carne
        @arq_boleto: objeto da classe boleto
        @brw_gerar_boleto_item: browse da tabela wizard_gerar_boleto_item
        @imprimir: string com as informacoes de impressao
        @company_id: id da tabela res_company
        @gerar_boleto_separado: booleano. True ira gerar o boleto separado, um por arquivo. False
        ira gerar todos os boletos ou carnes num so arquivo
        @nosso_numero_informado: inteiro para sequencia da informacao 'nosso numero'.
        @return: objeto da classe boleto com os atributos preenchidos
        """

        context = context and context.copy() or {}
        capa_id = brw_gerar_boleto_item.wizgbi_gerar_boleto_id
        local_cobranca_id = capa_id.wizgbo_bol_banco_id.id
        natureza_cobranca_id = capa_id.wizgbo_bol_natureza_cobranca_id.id
        dado_cedente = self._wizgbo_get_dado_cedente(cr, uid, company_id, local_cobranca_id,
                                                     natureza_cobranca_id, context)
        nosso_numero = nosso_numero_informado

        dado_pessoa = self._wizgbo_get_dado_pessoa(cr, uid, brw_gerar_boleto_item.wizgbi_pessoa_id,
                                                   context)

        dado_titulo = brw_gerar_boleto_item.wizgbi_titulo_id
        data_emissao = dado_titulo.data_emissao or ''
        if data_emissao:
            data_emissao = time.strftime('%d/%m/%Y', time.strptime(data_emissao, '%Y-%m-%d'))
        data_vencimento = dado_titulo.data_prorrogacao or ''
        if data_vencimento:
            data_vencimento = time.strftime('%d/%m/%Y', time.strptime(data_vencimento, '%Y-%m-%d'))

        if gerar_boleto_separado:
            # Instancia de boleto
            arq_boleto = boleto.Boleto()
        if not isinstance(observacoes, list):
            observacoes = list(observacoes)
        if dado_titulo.fatura_id and dado_titulo.fatura_id.nf_id:
            brw_nota = dado_titulo.fatura_id.nf_id
            nf_no_titulo = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'fiscal.parametro', ['fispar_numero_nf_no_titulo'], context=context)
            if nf_no_titulo and isinstance(nf_no_titulo, list):
                nf_no_titulo = nf_no_titulo[0]
            if nf_no_titulo and isinstance(nf_no_titulo, dict):
                nf_no_titulo = nf_no_titulo.get('fispar_numero_nf_no_titulo')
            if nf_no_titulo:
                obs = 'Referente a nota fiscal ' + str(brw_nota.numero_nf or '') + ' serie ' + str(brw_nota.serie_nf) +\
                      ' de ' + brw_nota.data_emissao[8:10] +'/'+ brw_nota.data_emissao[5:7] +'/'+ brw_nota.data_emissao[:4]
                observacoes.append(obs)
        # Dados para formacao do boleto
        # Banco
        arq_boleto.banco.formato = capa_id.wizgbo_modelo
        arq_boleto.banco.numeroBacen = str(
            capa_id.wizgbo_bol_banco_id.finloc_banco_id.admbco_codigo_bc)
        arq_boleto.banco.convenio = dado_cedente.get('convenio')
        arq_boleto.banco.agencia = dado_cedente.get('agencia')
        arq_boleto.banco.agencia_dv = dado_cedente.get('agencia_dv')
        arq_boleto.banco.conta = dado_cedente.get('conta')
        arq_boleto.banco.conta_dv = dado_cedente.get('conta_dv')
        arq_boleto.banco.carteira = dado_cedente.get('carteira')
        if dado_cedente.get('modalidade') == '1':
            arq_boleto.banco.modalidade = '01'
        elif dado_cedente.get('modalidade') == '2':
            arq_boleto.banco.modalidade = '02'

        # Cedente
        arq_boleto.cedente.nome = dado_cedente.get('cedente_nome')
        arq_boleto.cedente.CpfCnpj = dado_cedente.get('cedente_cpf_cnpj')
        arq_boleto.cedente.telefone = dado_cedente.get('cedente_telefone')
        arq_boleto.cedente.cedente_logo = dado_cedente.get('cedente_logo')

        # Titutlo
        arq_boleto.titulo.sacadoNome = dado_pessoa.get('nome')
        arq_boleto.titulo.sacadoCodigo = dado_pessoa.get('codigo')
        arq_boleto.titulo.nossoNroSeq = nosso_numero   # nosso numero deve ser no titulo e não no cedende, para o numero nao sair repetido na impressao
        arq_boleto.titulo.sacadoCpfCnpj = dado_pessoa.get('cpf_cnpj')
        arq_boleto.titulo.enderecoRua = dado_pessoa.get('endereco_rua')
        arq_boleto.titulo.enderecoNumero = dado_pessoa.get('endereco_numero')
        arq_boleto.titulo.enderecoBairro = dado_pessoa.get('endereco_bairro')
        arq_boleto.titulo.enderecoCidade = dado_pessoa.get('endereco_cidade')
        arq_boleto.titulo.enderecoEstado = dado_pessoa.get('endereco_estado')
        arq_boleto.titulo.enderecoCep = dado_pessoa.get('endereco_cep')
        arq_boleto.titulo.documento = dado_titulo.documento or ''
        arq_boleto.titulo.emissao = data_emissao
        valor = getNumeroEditado(dado_titulo.vr_original or 0, 2)
        data_venci = data_vencimento
        if context.get('titulo_vencido'):
            valor = getNumeroEditado(brw_gerar_boleto_item.wizgbi_vr_original or 0, 2)
            data_venci = brw_gerar_boleto_item.wizgbi_data_prorrogacao[8:10]+'/'+brw_gerar_boleto_item.wizgbi_data_prorrogacao[5:7]+'/'+brw_gerar_boleto_item.wizgbi_data_prorrogacao[0:4]
        arq_boleto.titulo.valor = valor
        arq_boleto.titulo.vencimento = data_venci
        arq_boleto.titulo.parcela = dado_titulo.parcela or 1
        arq_boleto.titulo.observacoes = observacoes

        if imprimir == 'arquivo':
            if gerar_boleto_separado:
                arq_boleto.cedente.impressora = imprimir + '/tmp/%s-%s.pdf' % (str(
                    dado_pessoa.get('codigo') or 0).rjust(6, '0'), arq_boleto.titulo.documento)
            else:
                arq_boleto.cedente.impressora = imprimir + '/tmp/boletos.pdf'
        else:
            arq_boleto.cedente.impressora = imprimir

        # Processa titulo
        arq_boleto.processaTitulo()
        return arq_boleto

    def wizgbo_onchange_local_natureza(self, cr, uid, ids, local_cobranca_id, natureza_cobranca_id,
                                       context=None):

        """
        Realiza a busca na tabela financeiro_local_cobranca pelo proximo 'nosso numero', a partir
        do local e natureza informados
        @ids: id ta tabela wizard_gerar_boleto
        @local_cobranca_id: id da tabela financeiro_local_cobranca
        @natureza_cobranca_id: id da tabela financeiro_natureza_cobranca
        @return: Dicionario, com a informacao do proximo 'nosso numero', caso tenha encontrado
        """

        context = context and context.copy() or {}
        res = dict()
        res['value'] = {'wizgbo_nosso_numero': 0}
        if not local_cobranca_id or not natureza_cobranca_id:
            return res
        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')
        args = [('finlon_local_cobranca_id', '=', local_cobranca_id),
                ('finlon_natureza_cobranca_id', '=', natureza_cobranca_id)]
        local_natureza_ids = obj_financeiro_local_natureza.search(cr, uid, args, context=context)
        if local_natureza_ids:
            try:
                nosso_numero = long(obj_financeiro_local_natureza.browse(
                    cr, uid, local_natureza_ids[0]).finlon_nosso_numero)
            except:
                nosso_numero = 0
            if nosso_numero:
                res['value'] = {'wizgbo_nosso_numero': str(nosso_numero)}
        return res

    def get_imprimir_inf_titulo(self, cr, uid):
        obj_adm_utils = self.pool.get('adm.utils')
        field_parametro_imprimir = ['finpar_imprimir_informacoes_titulo_boleto']
        imprimir_inf_tit = obj_adm_utils.admutl_get_parametros(cr, uid, 'financeiro',
                                                               field_parametro_imprimir)
        if imprimir_inf_tit:
            if imprimir_inf_tit.get('finpar_imprimir_informacoes_titulo_boleto'):
                return True
        return False


class wizard_gerar_boleto_item(osv.TransientModel):
    """
    Wizard para exibicao dos itens (titulos)
    prefixo: wizgbi_
    """

    _name = 'wizard.gerar.boleto.item'

    _columns = {
        'wizgbi_gerar_boleto_id': fields.many2one('wizard.gerar.boleto', 'Capa'),
        'wizgbi_pessoa_id': fields.many2one("adm.pessoa", 'Cliente'),
        'wizgbi_documento': fields.char('Documento', size=30),
        'wizgbi_parcela': fields.integer('Parcela'),
        'wizgbi_data_emissao': fields.date('Emissao'),
        'wizgbi_vr_original': fields.float('Valor', digits=(15, 2)),
        'wizgbi_data_prorrogacao': fields.date('Vencimento'),
        'wizgbi_titulo_id': fields.many2one('financeiro.titulo', 'Titulo'),
        'wizgbi_possui_email': fields.function(wizgbi_get_email, method=True,
                                               string='Possui e-mail', type='boolean'),
    }

    def wizgbi_gera_email(self, cr, uid, item_id, boleto_binary,  context=None):
        """
        O metodo reune e organiza todos os dados necessarios para a criacao dos emails.
        :param cr: cursor no baco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param item_id: id da tabela desse objeto.
        :type: integer
        :param boleto_binary: arquivo do boleto.
        :type: binary
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = True

        brw_wizard_gerar_boleto_item = self.browse(cr, uid, item_id, context)
        obj_financeiro_emails_mensagens = self.pool.get('financeiro.emails.mensagens')
        obj_adm_pessoa_email = self.pool.get('adm.pessoa.email')
        obj_adm_util = self.pool.get('adm.utils')

        financeiro_parametro_dados = obj_adm_util.admutl_get_parametros(
            cr, uid, 'financeiro', 'finpar_email_server_id', context=context)
        mail_server_id = financeiro_parametro_dados.get('finpar_email_server_id') or False
        if not mail_server_id:
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
            mail_server_id = self.pool.get('res.company').browse(cr, uid, company_id, context=context).servidor_smtp_id.id

        gerar_boleto_id = brw_wizard_gerar_boleto_item.wizgbi_gerar_boleto_id
        pessoa_id = brw_wizard_gerar_boleto_item.wizgbi_pessoa_id.id
        assunto_email = gerar_boleto_id.wizgbo_mensagem_id.fineca_assunto
        texto_email = gerar_boleto_id.wizgbo_mensagem_id.fineca_corpo
        pessoa_email_ids = obj_adm_pessoa_email.search(
            cr, uid, [('pessoa_id', '=', pessoa_id),
                      ('admpem_setor_ids', '=', gerar_boleto_id.wizgbo_setor_id.id)])
        brw_adm_pessoa_email = obj_adm_pessoa_email.browse(cr, uid, pessoa_email_ids, context)
        email = brw_adm_pessoa_email[0].name
        destinararios = ''
        for mail in brw_adm_pessoa_email[1:]:
            destinararios += mail.name + (', ' if mail != brw_adm_pessoa_email[-1] else '')
        mail_id = obj_adm_util.admutl_send_email(
            cr, uid, email=email, assunto=assunto_email, corpo=texto_email,
            destinatarios=destinararios, anexo=boleto_binary, nome_anexo='Boleto',
            mail_server_id=mail_server_id, context=context)

        vals = {'create_date': time.strftime('%Y-%m-%d %H:%M:%S'),
                'fineme_texto_id': gerar_boleto_id.wizgbo_mensagem_id.id,
                'create_uid': uid,
                'fineme_pessoa_id': pessoa_id,
                'fineme_mail_id': mail_id,
                }
        email_mensagem_id = obj_financeiro_emails_mensagens.create(cr, uid, vals, context)

        sql = """
            INSERT INTO financeiro_mensagens_m2m_titulos (mensagem_id, titulo_id)
            VALUES (%s, %s)
        """ % (email_mensagem_id, brw_wizard_gerar_boleto_item.wizgbi_titulo_id.id)
        cr.execute(sql)

        self.pool.get('adm.ocorrencia').create(
            cr, uid, {'admoco_pessoa_id': pessoa_id,
                      'admoco_descricao': 'Boleto enviado por e-mail',
                      'admoco_titulo_id': brw_wizard_gerar_boleto_item.wizgbi_titulo_id.id,
                      'admoco_email_id': email_mensagem_id}, context)
        return res


class wizard_gerar_boleto_arquivo(osv.TransientModel):
    """
    Wizard para exibicao e download dos arquivos de boleto, caso a impressao seja em arquivo
    prefixo: wizgba_
    """

    _name = 'wizard.gerar.boleto.arquivo'

    _columns = {
        'wizgba_gerar_boleto_id': fields.many2one('wizard.gerar.boleto', 'Capa'),
        'wizgba_gerar_boleto_titulo_id': fields.many2one('wizard.gerar.boleto.titulo', 'Capa titulo'),
        'wizgba_dado_boleto': fields.binary('Boleto', readonly=True),
        'wizgba_arquivo_boleto': fields.char('Nome do arquivo', size=50),
    }
