# -*- coding: utf-8 -*-

from openerp.osv import fields, osv
from datetime import date, timedelta, datetime
from SigUtils import datas
from openerp.tools.translate import _


class wizard_gerar_parcelas(osv.TransientModel):
    def _fintit_get_documento(self, cr, uid, context=None):
        context = context and context.copy() or {}
        return context.get('numero_documento')
    """
    Data de alteração: 19/02/2015
    Responsável: joao.almeida
    Observação: evitei adicionar mais um campo ao contexto,
    mas se houver impacto na performance reconsiderar o context para
    evitar ir ao Postgre sem necessidade.
    """
    def _get_dia_vencimento_fixo(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')
        fatura_id = context.get('active_id', False)
        if not fatura_id:
            return 0
        else:
            fatura = obj_financeiro_fatura.browse(cr, uid, fatura_id)
            return int(fatura.data_emissao[-2:])
    #end _get_dia_vencimento_fixo

    _name = 'wizard.gerar.parcelas'
    _description = 'Gerar Parcelas'

    _columns = {
        'documento': fields.char('Documento', size=30, required=True, select=True),
        'qtde_parcelas': fields.integer('Quantidade Parcelas', required=True),
        'prazo': fields.integer('Prazo', required=True),
        'valor': fields.float('Valor', digits=(9, 2), required=True),
        'wizgerparmov_vencimento_fixo': fields.boolean('Vencimento fixo',
                                                       help="Se marcado gera as "
                                                            "parcelas com dia fixo "
                                                            "obedecendo a data de "
                                                            "emissao"),
        'fintit_gerencialmov_ids': fields.one2many('wizard.gerar.parcelas.movimentos', 'titulo_id',
                                                   'Movimento'),
        'wizgerparmov_vencimento_fixo_dia': fields.integer('Dia de vencimento',
                                                           help="Selecionando vencimento fixo, "
                                                                "todas as parcelas vencerão "
                                                                "nesse dia do mês")
    }

    _defaults = {
        'documento': _fintit_get_documento,
        'wizgerparmov_vencimento_fixo': False,
        'wizgerparmov_vencimento_fixo_dia': _get_dia_vencimento_fixo
    }

    def gerar_parcelas(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')
        fatura_id = context.get('active_id', False)
        fatura = obj_financeiro_fatura.browse(cr, uid, fatura_id, context)
        competencia_por_parcelas = fatura.finfat_competencia_por_parcelas
        pessoa_id = fatura.pessoa_id.id
        dados = self.browse(cr, uid, ids)
        vencimento_fixo = dados[0].wizgerparmov_vencimento_fixo
        data_emissao = fatura.data_emissao
        mes_emissao = data_emissao[5:7]
        emi_ultimo_dia_fevereiro = False
        dia_fixo = '%02d' % dados[0].wizgerparmov_vencimento_fixo_dia
        if mes_emissao == '02':
            ultm_dia_fev = datas.sigDatas(data_emissao).getUltimoDiaDoMes()
            if dia_fixo == str(ultm_dia_fev):
                emi_ultimo_dia_fevereiro = True
        soma_centro_custo = 0
        vals_gerencial_mov = []
        context.update({'wizard_gerar_parcelas': True})
        for i in dados:
            if i.fintit_gerencialmov_ids:
                for mov in i.fintit_gerencialmov_ids:
                    soma_centro_custo += mov.valor
                    vals_gerencial_mov.append([0, False, {'evento_id': mov.evento_id.id,
                                                          'centro_custo_id': mov.centro_custo_id.id,
                                                          'valor': mov.valor}])
                if round(soma_centro_custo, 2) != round(i.valor, 2):
                    raise osv.except_osv(_('Atencao'), _(
                        'A soma dos valores dos centro de custo deve ser igual ao valor do titulo.'))
            data = data_emissao
            for x in range(i.qtde_parcelas):
                dt = datas.sigDatas(data).getCalculaPrazo(i.prazo, 'DD', 'N')
                if all([vencimento_fixo, dia_fixo]):
                    dt = self.get_data_fixa(data, dia_fixo, emi_ultimo_dia_fevereiro)
                vals = {
                    'vr_original': i.valor,
                    'documento': i.documento,
                    'data_emissao': competencia_por_parcelas and dt or data_emissao,
                    'data_vencimento': dt,
                    'tipo_titulo': fatura.tipo_fatura,
                    'tipo_lancamento': 'A',
                    'pessoa_id': pessoa_id,
                    'fatura_id': fatura_id,
                    'parcela': x + 1,
                    'fintit_gerencialmov_ids': vals_gerencial_mov
                }
                context.update({'futil_super': False, 'fintit_manter_prv': True})
                titulo_id = obj_financeiro_titulo.create(cr, uid, vals, context)
                data = dt
        obj_financeiro_fatura.finfat_atualiza_competencia_gerencial_movimento(
            cr, uid, [fatura_id], competencia_por_parcelas, context)
        return {'type': 'ir.actions.act_window_close'}

    def wizgerpar_onchange_vencimento_fixo(self, cr, uid, ids):
        return {'value': {'wizgerparmov_vencimento_fixo': False}}

    @staticmethod
    def get_data_fixa(data, dia_fixo, emi_ultimo_dia_fevereiro=False):
        """
        Retorno   : Renotorna uma string com a  data do mes seguinte com o dia fixo para vencimento
        Parametros: @data --> data a ser acrecentado o mes
                    @dia_fixo --> dia fixo para o vencimento da parcela
                    @emi_ultimo_dia_fevereiro --> Utilizando quando o dia de emissao e o ultimo dia
                    de fevereiro.
        Funcao    : Se o dia fixo eh ultimo dia do mes, sera considerado para dia fixo o dia 30.
                    Caso a parcela seja de fevereiro, sera considerado o ultimo dia de feveiro
                    Se o dia nao for o ultimo dia do mes, este sera o dia fixo
        """
        data = datetime.strptime(data, '%Y-%m-%d')
        mes = data.month
        if mes == 12:
            proximo_mes = '01'
            ano = data.year + 1
            ano = str(ano)
        else:
            proximo_mes = mes + 1
            proximo_mes = str(proximo_mes).zfill(2)
            ano = str(data.year)
        if int(dia_fixo) > 30 or emi_ultimo_dia_fevereiro:
            dia_fixo = 30
        if proximo_mes == '02':
            data_calc = ano+'-'+proximo_mes+'-'+'01'
            ultimo_dia = datas.sigDatas(data_calc).\
                getUltimoDiaDoMes(proximo_mes)
            if int(dia_fixo) >= ultimo_dia:
                dia_fixo = ultimo_dia
        dt = str(ano) + '-' + str(proximo_mes) + '-' + str(dia_fixo)
        return dt


class wizard_gerar_parcelas_movimentos(osv.TransientModel):
    _name = 'wizard.gerar.parcelas.movimentos'
    _columns = {
        'titulo_id': fields.many2one('wizard.gerar.parcelas'),
        'valor': fields.float('Valor', digits=(15, 2), required=True), # Valor do rateio para o centro de custo.
        'evento_id': fields.many2one("gerencial.evento", 'Evento', required=True), # Id do Evento
        'centro_custo_id': fields.many2one("adm.empresa.centrocusto", 'Centro de custo',
                                           required=True),
    }
