# -*- encoding: utf-8 -*-
from openerp.osv import osv, fields


class wizard_movimento_titulo_historico(osv.TransientModel):
    _name = "wizard.financeiro.movimento.historico"
    _rec_name = 'titulo_id'
    _columns = {
        'company_id': fields.many2one("res.company", "Empresa"),
        'movimento_titulo_id': fields.many2one("wizard.financeiro.movimento.titulo", "Movimento Titulo"),
        'pessoa_id': fields.many2one('adm.pessoa', "Cliente/Fornecedor"),
        'user_id': fields.many2one('res.users', 'Usuario'), # Usuario que alterou o titulo
        'data_criacao': fields.datetime('Data historico'), #Data que ocorreu o historico
        'titulo_id': fields.many2one("financeiro.titulo", 'Titulo'),
        'tipo_titulo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo', size=1),
        'local_cobranca_id': fields.many2one("financeiro.local.cobranca", 'Local cobranca',
                                             domain="[('company_id', '=', company_id)]"),
        'natureza_cobranca_id': fields.many2one("financeiro.natureza.cobranca", 'Natureza cobranca'),
        'data_prorrogacao': fields.date('Prorrogacao'), #Data de prorrogacao do titulo
        'operacao': fields.selection(
            [('I', 'Entrada'), ('A', 'Alteracao'), ('R', 'Registro'), ('B', 'Baixa'), ('BP', 'Baixa Parcial'),
             ('PR', 'Prorrogacao'), ('C', 'Cancelamento'), ('BR', 'Baixa por Renegociacao'),
             ('EB', 'Estorno de Baixa')], 'Operacao', required=True),
        'valor': fields.float("Valor", digits=(15, 2)),
        'vencido': fields.boolean('Vencido', store=False),
    }



class wizard_movimento_titulos(osv.TransientModel):
    _name = "wizard.financeiro.movimento.titulo"
    _rec_name = 'titulo_id'
    _columns = {
        'pessoa_id': fields.many2one('adm.pessoa', "Cliente/Fornecedor"),
        'data_emissao': fields.date("Data de Emissao"),
        'data_inicio_mov': fields.datetime("Data Inicio"),
        'data_final_mov': fields.datetime("Data Final"),
        'data_inicio_vencimento': fields.date('Data Inicio Vencimento'),
        'data_final_vencimento': fields.date('Data Final Vencimento'),
        'titulo_id': fields.many2one('financeiro.titulo', "Titulo",
                                     domain="[('tipo_titulo','=', context.get('tipo'))]"),
        'tipo_titulo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo', size=1),
        'operacao': fields.selection(
            [('', ' '), ('I', 'Entrada'), ('A', 'Alteracao'), ('R', 'Registro'), ('B', 'Baixa'),
             ('BP', 'Baixa Parcial'), ('PR', 'Prorrogacao'), ('C', 'Cancelamento')], 'Operacao'),
        'doc_tipo_id': fields.many2one("documento.documento.tipo", 'Tipo de documento'), # id do Tipo de documento
        'movimento_historico_ids': fields.one2many('wizard.financeiro.movimento.historico', 'movimento_titulo_id',
                                                   "Historico"),
        'possui_titulos': fields.boolean('Possui titulos'),
    }

    def find_history(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_wizard_historico = self.pool.get('wizard.financeiro.movimento.historico')
        obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
        historico_antigo_ids = obj_wizard_historico.search(cr, uid, [('movimento_titulo_id', 'in', ids)],
                                                           context=context)
        if historico_antigo_ids:
            obj_wizard_historico.unlink(cr, uid, historico_antigo_ids)
        dados_pesquisa = self.read(cr, uid, ids[0], context=context)
        args_historico = []
        args = []
        if dados_pesquisa.get('tipo_titulo'):
            args.append(('tipo_titulo', '=', dados_pesquisa['tipo_titulo']))
        if dados_pesquisa.get('pessoa_id'):
            args.append(('pessoa_id', '=', dados_pesquisa['pessoa_id'][0]))
        if dados_pesquisa.get('data_inicio_vencimento'):
            args.append(('data_prorrogacao', '>=', dados_pesquisa['data_inicio_vencimento']))
        if dados_pesquisa.get('data_final_vencimento'):
            args.append(('data_prorrogacao', '<=', dados_pesquisa['data_final_vencimento']))
        if dados_pesquisa.get('doc_tipo_id'):
            args.append(('doc_tipo_id', '=', dados_pesquisa['doc_tipo_id'][0]))
        if dados_pesquisa.get('data_emissao'):
            args.append(('data_emissao', '=', dados_pesquisa['data_emissao']))
        if dados_pesquisa.get('titulo_id'):
            args.append(('id', '=', dados_pesquisa['titulo_id'][0]))
        if dados_pesquisa.get('data_inicio_mov'):
            args_historico.append(('create_date', '>=', dados_pesquisa['data_inicio_mov']))
        if dados_pesquisa.get('data_final_mov'):
            args_historico.append(('create_date', '<=', dados_pesquisa['data_final_mov']))
        obj_titulo = self.pool.get('financeiro.titulo')
        obj_titulo_historico = self.pool.get('financeiro.titulo.historico')

        financeiro_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'financeiro.titulo', context=context)
        company_ids = self.pool.get('res.company')._get_company_children(cr, uid, financeiro_company_id)
        args += [('company_id', 'in', company_ids)]

        list_titulos = []
        titulo_ids = obj_titulo.search(cr, uid, args, context=context)
        possui_titulos = False
        if titulo_ids:
            titulo_dados = obj_titulo.read(cr, uid, titulo_ids,
                                           ['titulo_historico_ids', 'pessoa_id', 'company_id', 'id', 'tipo_titulo',
                                            'vr_original', 'data_prorrogacao', 'data_pagamento'], context=context)
            if isinstance(titulo_dados, list):
                for i in titulo_dados:
                    try:
                        company_id = i['company_id'][0]
                    except:
                        company_id = False
                    try:
                        pessoa_id = i['pessoa_id'][0]
                    except:
                        pessoa_id = False
                    args_historico.append(('titulo_id', '=', i['id']))
                    historico_ids = obj_titulo_historico.search(cr, uid, args_historico,
                                                                context=context.update({'pesquisa_movimento': True}))
                    historico_dados = obj_titulo_historico.read(cr, uid, historico_ids, context=context)
                    args_historico.remove(args_historico[-1])
                    for j in historico_dados:
                        if dados_pesquisa.get('operacao'):
                            if dados_pesquisa.get('operacao') != j['operacao']:
                                continue
                        try:
                            usuario = j['create_uid'][0]
                        except:
                            usuario = ''
                        try:
                            titulo_id = j['titulo_id'][0]
                        except:
                            titulo_id = ''
                        try:
                            local_cobranca_id = j['local_cobranca_id'][0]
                        except:
                            local_cobranca_id = False
                        try:
                            natureza_cobranca_id = j['natureza_cobranca_id'][0]
                        except:
                            natureza_cobranca_id = False

                        if titulo_id:
                            if j.get('operacao') == 'BP':
                                titulo_baixa_id = obj_financeiro_titulo_baixa.search(cr, uid,
                                                                                     [('titulo_id', '=', titulo_id)],
                                                                                     context=context)
                                if titulo_baixa_id:
                                    if titulo_id in list_titulos:
                                        valor = obj_financeiro_titulo_baixa.browse(cr, uid, titulo_baixa_id[
                                            list_titulos.count(titulo_id)]).valor_pago
                                    else:
                                        valor = obj_financeiro_titulo_baixa.browse(cr, uid,
                                                                                   titulo_baixa_id[0]).valor_pago
                                else:
                                    valor = 0.0
                                list_titulos.append(titulo_id)
                            else:
                                valor = i.get('vr_original')
                        else:
                            valor = 0.0

                        if i.get('data_pagamento') > i.get('data_prorrogacao'):
                            vencido = True
                        else:
                            vencido = False
                        obj_wizard_historico.create(cr, uid, {'company_id': company_id,
                                                              'pessoa_id': pessoa_id,
                                                              'movimento_titulo_id': ids[0],
                                                              'user_id': usuario,
                                                              'data_criacao': j.get('create_date') or '',
                                                              'titulo_id': titulo_id,
                                                              'local_cobranca_id': local_cobranca_id,
                                                              'natureza_cobranca_id': natureza_cobranca_id,
                                                              'data_prorrogacao': j.get('data_prorrogacao') or '',
                                                              'operacao': j.get('operacao'),
                                                              'valor': valor,
                                                              'tipo_titulo': i.get('tipo_titulo'),
                                                              'vencido': vencido
                        }, context=context)
                        possui_titulos = True
                if possui_titulos:
                    self.write(cr, uid, ids[0], {'possui_titulos': True}, context)
                return True
        return False

    def button_impressao_movimento_titulo(self, cr, uid, ids, context):
        obj_financeiro_movimento_historico = self.pool.get('wizard.financeiro.movimento.historico')
        movimento_historico_ids = obj_financeiro_movimento_historico.search(
            cr, uid, [('movimento_titulo_id', 'in', ids)], context=context)
        vals = {'historico_ids': movimento_historico_ids}
        self.write(cr, uid, ids[0], {'possui_titulos': False}, context)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_movimento_titulo',
            'datas': vals,
        }
