# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import date
from datetime import timedelta
import time
from dateutil.relativedelta import relativedelta
from dateutil.parser import parse


class wizard_movimentacoes_diarias_sintetico(osv.TransientModel):

    def _get_dados(self, cr, uid, ids, name, arg, context=None):
        """
        Solucao para tratar a ordenacao na treeview, porque somente campos funcao nao sao
        ordenados
        """
        res = {}
        context = context and context.copy() or {}
        for x in self.browse(cr, uid, ids):
            vals = {
                'f_data_prorrogacao': x.data_prorrogacao,
                'f_valor_receber': x.valor_receber,
                'f_valor_pagar': x.valor_pagar,
                'f_vr_previsao_receber': x.vr_previsao_receber,
                'f_vr_previsao_pagar': x.vr_previsao_pagar,
                'f_saldo': x.saldo,
                'f_saldo_acumulado': x.saldo_acumulado,
                'f_saldo_com_previsao': x.saldo_com_previsao,
            }
            res[x.id] = vals
        return res

    _name = 'wizard.movimentacao.diaria.movimento'
    _order = 'data_prorrogacao'
    _columns = {
        'data_prorrogacao': fields.date('Data Vencimento'),
        'f_data_prorrogacao': fields.function(_get_dados, method=True, string='Data Vencimento',
                                           type='date',  multi="dados",store=False),
        'valor_receber': fields.float('Valor a Receber', digits=(15, 2)),
        'f_valor_receber': fields.function(_get_dados, method=True, string='Valor a Receber',
                                            type='float', digits=(15, 2),multi="dados", store=False),
        'valor_pagar': fields.float('Valor a Pagar', digits=(15, 2)),
        'f_valor_pagar': fields.function(_get_dados, method=True, string='Valor a Pagar',
                                           type='float', digits=(15, 2),multi="dados", store=False),
        'vr_previsao_receber': fields.float('Valor Prev. a Receber', digits=(15, 2)),
        'f_vr_previsao_receber': fields.function(_get_dados, method=True, string='Valor Prev. a Receber',
                                         type='float', digits=(15, 2), multi="dados",store=False),
        'vr_previsao_pagar': fields.float('Valor Prev. a Pagar', digits=(15, 2)),
        'f_vr_previsao_pagar': fields.function(_get_dados, method=True, string='Valor Prev. a Pagar',
                                                 type='float', digits=(15, 2),multi="dados", store=False),
        'f_vr_previsao_pagar': fields.function(_get_dados, method=True, string='Valor Prev. a Pagar',
                                               type='float', digits=(15, 2),multi="dados", store=False),
        'saldo': fields.float('Saldo', digits=(15, 2)),
        'f_saldo': fields.function(_get_dados, method=True, string='Saldo',
                                   type='float', digits=(15, 2),multi="dados", store=False),
        'saldo_acumulado':fields.float('Acumulado', digits=(15, 2)),
        'f_saldo_acumulado': fields.function(_get_dados, method=True, string='Acumulado',
                                   type='float', digits=(15, 2),multi="dados", store=False),
        'saldo_com_previsao':fields.float('Saldo com Previsao', digits=(15, 2)),
        'f_saldo_com_previsao': fields.function(_get_dados, method=True, string='Saldo com Previsao',
                                             type='float', digits=(15, 2),multi="dados", store=False),
        'company_id': fields.many2one('res.company', 'Empresa'),
        'diaria_capa': fields.many2one('wizard.movimentacao.diaria.capa', "Movimentacao Diaria Capa"),
        'positivo_negativo': fields.selection([('P', 'Positivo'), ('N', 'Negativo')], 'Positivo/Negativo', size=1),
        'sequencia': fields.integer()
    }

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        if context.get('read') or not context.get('bin_size'):
            res = super(wizard_movimentacoes_diarias_sintetico, self).read(cr, uid, ids, fields, context=context)
        else:
            res = []
        return res


class wizard_movimentacoes_diarias_capa(osv.TransientModel):
    _name = 'wizard.movimentacao.diaria.capa'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa'),
        'data_inicial': fields.date("Data Inicial"),
        'data_final': fields.date("Data Final"),
        'saldo_inicial': fields.float("Saldo Inicial", digits=(15, 2)),
        'movimento_diario_ids': fields.one2many('wizard.movimentacao.diaria.movimento', 'diaria_capa', "Movimentacoes"),
        'considerar_disponibilidade': fields.boolean('Considerar Disponibilidade'),
        'considerar_previsao': fields.boolean('Considerar Previsao'),
        'vr_disponivel': fields.float('Valor Disponivel', digits=(15, 2)),
        'titulo_vencido_ids': fields.one2many(
            'wiz.financeiro.titulo.vencido', 'financeiro_movimentacao_diaria_id',
            'Titulos vencidos'),
        'agrupar_por_mes': fields.boolean('Agrupar por mes'),
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id,
        'considerar_previsao':True,
        'considerar_disponibilidade':True,
        'data_inicial': time.strftime('%Y-%m-%d'),
        'data_final': time.strftime('%Y-%m-%d'),
        'agrupar_por_mes': lambda *a: False,
    }



    def onchange_calcula_disponibilidade(self, cr, uid, ids, considerar_disponibilidade,
                                         data_inicial, context=None):
        '''Calcula a disponibilidade de acordo com data infomada'''

        context = context and context.copy() or {}
        vr_disponivel = 0
        warning = {}
        data = (parse(data_inicial) - relativedelta(days=1)).strftime('%Y-%m-%d')
        if considerar_disponibilidade:
            vr_disponivel = self._verifica_disp(cr, uid, data)
        else:
            considerar_disponibilidade = False

        return {'value': {'vr_disponivel': vr_disponivel,
                          'considerar_disponibilidade': considerar_disponibilidade}, 'warning': warning}

    def _verifica_disp(self, cr, uid, data, context=None):

        '''
                Verifica Saldo disponivel de acorodo com a data informada
              '''
        context = context and context.copy() or {}
        obj_conta = self.pool.get("financeiro.tesouraria.contas")
        company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id.id
        id_conta = obj_conta.search(cr, uid, [('company_id', '=', company_id),('integra_fluxo','=',True)],
                                    context=context)
        # Localizando as contas filhas
        contas = []
        for i in id_conta:
            contas += [i]
            a = obj_conta.search(cr, uid, [('parent_id', '=', i)], context=context)
            if not a == []:
                contas += a
                # Subtraindo data inicial para o dia anterior
        y, m, d = (int(x) for x in data.split('-'))
        data_ant = date(y, m, d)
        # Calculando o saldo do dia
        obj_financeiro_tesouraria_saldos = self.pool.get('financeiro.tesouraria.saldos')
        saldo_dia = 0
        for i in contas:
            saldo_dia += obj_financeiro_tesouraria_saldos.get_saldo_do_dia(cr, uid, i, str(data_ant), context)
        return saldo_dia


    def button_limpar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        movimentos = self.read(cr, uid, ids, ['movimento_diario_ids'], context=context)
        if isinstance(movimentos, list):
            movimentos = movimentos[0]
        if isinstance(movimentos, dict):
            if movimentos.get('movimento_diario_ids'):
                self.pool.get('wizard.movimentacao.diaria.movimento').unlink(cr, uid,
                                                                             movimentos['movimento_diario_ids'],
                                                                             context)
        return self.write(cr, uid, ids, {'data_inicial': False,
                                         'data_final': False,
                                         'considerar_previsao': True,
                                         'considerar_disponibilidade':False,
                                         'vr_disponivel': 0, }, context)

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        capa = self.read(cr, uid, ids, context=context)
        if isinstance(capa, list):
            capa = capa[0]
        if not isinstance(capa, dict):
            return False

        # Agrupamento mensal
        agrupar_por_mes = capa.get('agrupar_por_mes')
        dados_agrupamento = list()

        # Datas originais do periodo, serao usadas para limitar a insercao de itens (titulos)
        data_ini_o = capa.get('data_inicial')
        data_fim_o = capa.get('data_final')
        data_atual = str(date.today())
        if capa.get('data_inicial') < data_atual:
            raise osv.except_osv(_('Atencao'), _('Data inicial anterior ao do dia da emissao'))
        if capa.get('data_inicial') > capa.get('data_final'):
            raise osv.except_osv(_('Atencao'), _('Data inicial maior que a data final'))

        ## apaga o resultado de pesquisas anteriores. É feito porque o wizard esta ficando em
        # memoria
        obj_wizard_diaria_movimento = self.pool.get('wizard.movimentacao.diaria.movimento')
        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')
        obj_wiz_financeiro_titulo_vencido = self.pool.get('wiz.financeiro.titulo.vencido')
        wizard_diaria_mov_ids = obj_wizard_diaria_movimento.search(cr, uid, [], context=context)
        if wizard_diaria_mov_ids:
            obj_wizard_diaria_movimento.unlink(cr, uid, wizard_diaria_mov_ids)
        if capa.get('vr_disponivel'):
            saldo_acumulado = capa['vr_disponivel']
        else:
            saldo_acumulado = 0
        data_ini = capa.get('data_inicial')
        data_fim = capa.get('data_final')

        # Buscar prazo para liberacao que tenha o maior valor
        local_natureza_ids = obj_financeiro_local_natureza.search(
            cr, uid, [('finlon_prazo_liberacao', '>', 0)], limit=1,
            order='finlon_prazo_liberacao DESC', context=context)
        if local_natureza_ids:
            prazo_liberacao = obj_financeiro_local_natureza.browse(
                cr, uid, local_natureza_ids[0]).finlon_prazo_liberacao
            if prazo_liberacao:
                # Aplica o prazo para liberacao nos periodos inicial e final
                data_ini = parse(data_ini, yearfirst=True)
                data_ini -= relativedelta(days=prazo_liberacao)
                data_ini = data_ini.strftime('%Y-%m-%d')

                data_fim = parse(data_fim, yearfirst=True)
                data_fim += relativedelta(days=prazo_liberacao)
                data_fim = data_fim.strftime('%Y-%m-%d')

        and_company_id = ''' '''
        and_natureza_id = ''' '''
        company_id = False
        if capa.get('company_id'):
            company_id = capa.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            and_company_id = '''
                          AND ft.company_id = %s
                             ''' % (company_id)

        # Naturezas de cobranca sem recebimento antecipado
        natureza_cobranca_ids = self.pool.get('financeiro.natureza.cobranca').search(
            cr, uid, [('recebimento_antecipado', '=', True)], context=context)
        if natureza_cobranca_ids:
            aux = ()
            for j in natureza_cobranca_ids:
                aux += (j,)
            natureza_cobranca_ids = aux
            and_natureza_id = '''
                          AND (ft.natureza_cobranca_id not in %s or ft.natureza_cobranca_id isnull)
                              ''' % (str(natureza_cobranca_ids))
            and_natureza_id = and_natureza_id.replace(',)', ')')
        sql_titulo = '''
                        SELECT ft.tipo_titulo,
                               ft.data_prorrogacao,
                               ft.saldo AS vr_original,
                               ft.local_cobranca_id,
                               ft.company_id,
                               ft.doc_tipo_id,
                               ft.natureza_cobranca_id
                        FROM financeiro_titulo AS ft
                        LEFT JOIN financeiro_local_cobranca AS flc ON flc.id = local_cobranca_id
                        LEFT JOIN financeiro_tesouraria_contas AS ftc ON ftc.id = conta_id
                        WHERE ft.data_prorrogacao >= '%s'
                          AND ft.data_prorrogacao <= '%s'
                          AND ft.state = 'A'
                          %s
                          %s
                        ORDER BY ft.data_prorrogacao
                     '''
        _sql = sql_titulo % (data_ini, data_fim, and_company_id, and_natureza_id)
        cr.execute(_sql)
        dados_titulos = cr.dictfetchall()

        sql_tesouraria_movimento = """
            SELECT
                conta.company_id,
                null as doc_tipo_id,
                CASE
                    WHEN movimento.debito_credito = 'C' THEN 'R'
                    ELSE 'P'
                END as tipo_titulo,
                movimento.valor as vr_original,
                movimento.data_movimento as data_prorrogacao,
                null as natureza_cobranca_id,
                null as local_cobranca_id
            FROM
                financeiro_tesouraria_movimento movimento
            INNER JOIN
                financeiro_tesouraria_contas conta ON conta.id = movimento.conta_id
            WHERE
                state = 'A'
                AND conta.integra_fluxo = True
                {periodo_tesouraria}
                {company_id}
            ORDER BY
                data_prorrogacao
        """
        if data_ini < time.strftime('%Y-%m-%d') > data_fim:
            sql_tesouraria_movimento = ''
        elif data_ini < time.strftime('%Y-%m-%d'):
            data_ini = time.strftime('%Y-%m-%d')
        if sql_tesouraria_movimento:
            periodo_tesouraria = "AND movimento.data_movimento BETWEEN '%s' AND '%s'" % (data_ini,
                                                                                         data_fim)
            company = ''
            if company_id:
                company = 'AND conta.company_id = %s' % company_id
            sql_tesouraria_movimento = sql_tesouraria_movimento.format(
                periodo_tesouraria=periodo_tesouraria, company_id=company)
            cr.execute(sql_tesouraria_movimento)
            dados_tesouraria = cr.dictfetchall()
            dados_titulos.extend(dados_tesouraria)

        if not dados_titulos:
            raise osv.except_osv(_('Atencao'), _('Nenhum titulo em aberto encontrado para o periodo'
                                                 ' informado!'))
        financeiro_titulos = dados_titulos
        # Ignorando os documentos excluir

        fluxo_caixa_instalado = self.pool.get('adm.utils'). \
            admutl_check_modulo_instalado(cr, uid, 'fluxocaixa', context)
        list_doc_tipo_ids = []
        if fluxo_caixa_instalado:
            obj_fluxocaixa_excluir = self.pool.get('fluxocaixa.excluir')
            fluxocaixa_excluir_ids = obj_fluxocaixa_excluir.\
                search(cr, uid, [('company_id', '=', company_id)])
            if fluxocaixa_excluir_ids:
                browse_fluxocaixa_excluir = obj_fluxocaixa_excluir.\
                    browse(cr, uid, fluxocaixa_excluir_ids)
                for x in browse_fluxocaixa_excluir:
                    list_doc_tipo_ids.append(x.doc_tipo_id.id)

        obj_documento_tipo = self.pool.get('documento.documento.tipo')
        tipo_previsao_id = obj_documento_tipo.search(cr, uid, [('sigla', '=', 'PRV')], context=context)
        if tipo_previsao_id:
            tipo_previsao_id = tipo_previsao_id[0]
        obj_wizard_movimentacoes = self.pool.get('wizard.movimentacao.diaria.movimento')
        obj_financeiro_cobranca = self.pool.get('financeiro.local.cobranca')
        obj_financeiro_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')

        # Lista com as naturezas de cobranca que nao estao com recebimento antecipado selecionado
        list_natureza_cobranca = map(lambda x: x, natureza_cobranca_ids)

        # Filtra os titulos que possuem natureza de cobranca
        financeiro_titulos_1 = filter(lambda x: x.get('natureza_cobranca_id'), financeiro_titulos)

        # Filtra os titulos que nao possuem natureza de cobranca
        financeiro_titulos_2 = filter(lambda x: not x.get('natureza_cobranca_id'),
                                      financeiro_titulos)

        # Filtra os titulos que possuem natureza de cobranca e que nao estao com recebimento
        # antecipado selecionado
        financeiro_titulos_1 = filter(
            lambda x: x.get('natureza_cobranca_id') in list_natureza_cobranca,
            financeiro_titulos_1)

        financeiro_titulos_1.extend(financeiro_titulos_2)
        financeiro_titulos = financeiro_titulos_1
        sequencia = 1
        for i in financeiro_titulos:
            gerar_listagem = True
            try:
                empresa_id = i.get('company_id')
            except:
                empresa_id = []
            valor_receber = 0
            valor_pagar = 0
            vr_previsao_pagar = 0
            vr_previsao_receber = 0
            saldo = 0
            if i.get('doc_tipo_id') != tipo_previsao_id and \
                            i.get('doc_tipo_id') not in list_doc_tipo_ids:
                if i.get('tipo_titulo') == 'P':
                    valor_pagar = i.get('vr_original')
                else:
                    valor_receber = i.get('vr_original')
            if i.get('doc_tipo_id') == tipo_previsao_id and \
                            i.get('doc_tipo_id') not in list_doc_tipo_ids\
                            and capa.get('considerar_previsao'):
                vr_previsao_pagar = i.get('vr_original') if i.get('tipo_titulo') == 'P' else 0
                vr_previsao_receber = i.get('vr_original') if i.get('tipo_titulo') == 'R' else 0

            saldo += valor_receber - valor_pagar
            total_previsao = (vr_previsao_receber - vr_previsao_pagar) if capa.get(
                'considerar_previsao') else 0
            if i.get('local_cobranca_id'):
                try:
                    local_cobranca_id = i['local_cobranca_id'][0]
                except:
                    local_cobranca_id = []
                if local_cobranca_id:
                    ## NECESSARIO O TRATAMENTO POIS O CAMPO É CRIADO NO
                    # FINANCEIRO POR MEIO DE HERANÇA DO TESOURARIA;
                    if obj_financeiro_cobranca.fields_get(cr, uid, [
                        'finloc_conta_id']):
                        tesouraria_conta = obj_financeiro_cobranca.read(cr, uid,
                                                                        local_cobranca_id,
                                                                        ['finloc_conta_id'], context=context)
                        if isinstance(tesouraria_conta, list):
                            tesouraria_conta = tesouraria_conta[0]
                        if isinstance(tesouraria_conta, dict):
                            if tesouraria_conta.get('conta_id'):
                                try:
                                    conta_id = tesouraria_conta['conta_id'][0]
                                except:
                                    conta_id = []
                                if conta_id:
                                    integra_fluxo = obj_financeiro_tesouraria_contas.read(
                                        cr, uid, conta_id,
                                        ['integra_fluxo'], context=context)
                                    if isinstance(integra_fluxo, list):
                                        integra_fluxo = integra_fluxo[0]
                                    if isinstance(integra_fluxo, dict):
                                        if not integra_fluxo['integra_fluxo']:
                                            gerar_listagem = False
            data_vencimento = i.get('data_prorrogacao')
            natureza_cobranca_id = i.get('natureza_cobranca_id')
            if isinstance(natureza_cobranca_id, (list, tuple)):
                natureza_cobranca_id = natureza_cobranca_id[0]
            local_cobranca_id = i.get('local_cobranca_id')
            if isinstance(local_cobranca_id, (list, tuple)):
                local_cobranca_id = local_cobranca_id[0]
            if natureza_cobranca_id and local_cobranca_id and data_vencimento:
                prazo_liberacao = 0
                local_natureza_ids = obj_financeiro_local_natureza.search(
                    cr, uid, [('finlon_local_cobranca_id', '=', local_cobranca_id),
                              ('finlon_natureza_cobranca_id', '=', natureza_cobranca_id)], context=context)
                if local_natureza_ids:
                    prazo_liberacao = obj_financeiro_local_natureza.browse(
                        cr, uid, local_natureza_ids[0]).finlon_prazo_liberacao
                data_vencimento = parse(data_vencimento, yearfirst=True)
                data_vencimento += relativedelta(days=prazo_liberacao)
                data_vencimento = data_vencimento.strftime('%Y-%m-%d')
            if gerar_listagem:
                positivo_negativo = 'P'
                movimento_id = obj_wizard_movimentacoes.search(cr, uid, [
                    ('data_prorrogacao', '=', data_vencimento)])
                if movimento_id:
                    read_mov = obj_wizard_movimentacoes.read(cr, uid, movimento_id[0],
                                                             ['valor_receber', 'valor_pagar',
                                                              'saldo','vr_previsao_receber',
                                                              'vr_previsao_pagar', 'saldo'
                                                              ],
                                                             context=context.update({'read': True}))
                    if read_mov and isinstance(read_mov, list):
                        read_mov = read_mov[0]
                    if read_mov and isinstance(read_mov, dict):

                        if (read_mov['valor_receber'] + read_mov['valor_pagar'] < 0) and read_mov[
                            'valor_pagar'] < 0:
                            positivo_negativo = 'N'
                        saldo_com_previsao = read_mov['saldo'] + saldo \
                                             + ((read_mov['vr_previsao_receber']+ vr_previsao_receber)\
                                             - (read_mov['vr_previsao_pagar'] + vr_previsao_pagar))
                        obj_wizard_movimentacoes.write(cr, uid, movimento_id, {
                            'valor_receber': read_mov['valor_receber'] + valor_receber,
                            'valor_pagar': read_mov['valor_pagar'] + valor_pagar,
                            'vr_previsao_receber':read_mov['vr_previsao_receber']+ vr_previsao_receber,
                            'vr_previsao_pagar':read_mov['vr_previsao_pagar'] + vr_previsao_pagar,
                            'saldo': read_mov['saldo'] + saldo,
                            'saldo_com_previsao':saldo_com_previsao,
                            'positivo_negativo': positivo_negativo,
                        }, context)
                else:
                    #acumulado
                    if valor_receber + valor_pagar < 0 and valor_receber > 0:
                        positivo_negativo = 'N'
                    if data_ini_o <= data_vencimento <= data_fim_o:
                        saldo_com_previsao = saldo + (vr_previsao_receber - vr_previsao_pagar)
                        vals_movimentacao = {'company_id': empresa_id,
                                             'data_prorrogacao': data_vencimento,
                                             'vr_previsao_receber': vr_previsao_receber,
                                             'vr_previsao_pagar': vr_previsao_pagar,
                                             'valor_receber': valor_receber,
                                             'valor_pagar': valor_pagar,
                                             'saldo': saldo,
                                             'saldo_com_previsao': saldo_com_previsao,
                                             'diaria_capa': ids[0],
                                             'positivo_negativo': positivo_negativo,
                                             'sequencia': sequencia}
                        if agrupar_por_mes:
                            dados_agrupamento.append(vals_movimentacao)
                        else:
                            obj_wizard_movimentacoes.create(cr, uid, vals_movimentacao, context)
            sequencia += 1

        if dados_agrupamento:
            self.gera_movimentacao_mensal(cr, uid, dados_agrupamento, context)

        movimento_ids = obj_wizard_movimentacoes.search(
            cr, uid, [('diaria_capa', 'in', ids)], order='data_prorrogacao', context=context)
        brw_wizard_movimento = obj_wizard_movimentacoes.browse(cr, uid, movimento_ids)
        for movimento in brw_wizard_movimento:
            movimento_saldo_com_previsao = movimento.saldo_com_previsao or 0.0
            saldo_acumulado += movimento_saldo_com_previsao
            obj_wizard_movimentacoes.write(cr, uid, movimento.id,
                                           {'saldo_acumulado': saldo_acumulado}, context)

        wiz_titulo_vencido_ids = obj_wiz_financeiro_titulo_vencido.search(
            cr, uid, [('financeiro_movimentacao_diaria_id', 'in', ids)], context=context)
        obj_wiz_financeiro_titulo_vencido.unlink(cr, uid, wiz_titulo_vencido_ids, context)

        titulo_dados = self.get_titulo_receber_vencido(cr, uid, context)

        self.write(cr, uid, ids, {'titulo_vencido_ids': titulo_dados}, context)

        return True

    def gera_movimentacao_mensal(self, cr, uid, dados_agrupamento, context=None):
        context = context and context.copy() or {}
        obj_wizard_movimentacoes = self.pool.get('wizard.movimentacao.diaria.movimento')
        lista_datas = list(set(map(lambda i: i.get('data_prorrogacao'), dados_agrupamento)))
        lista_datas.sort()
        lista_ano_mes = list()
        sequencia = 0
        for data in lista_datas:
            ano_mes = data[:7]
            ultimo_dia = ((parse(ano_mes+'-01') + relativedelta(months=1)) -
                          relativedelta(days=1)).strftime('%Y-%m-%d')
            if ano_mes in lista_ano_mes:
                continue
            sequencia += 1
            lista_ano_mes.append(ano_mes)
            dados_agrupamento_ano_mes = filter(lambda i: i.get('data_prorrogacao')[:7] == ano_mes,
                                               dados_agrupamento)
            positivo_negativo = 'P'
            valor_receber = sum(map(lambda a: a.get('valor_receber') or 0,
                                    dados_agrupamento_ano_mes))
            valor_pagar = sum(map(lambda a: a.get('valor_pagar') or 0, dados_agrupamento_ano_mes))
            if ((valor_receber + valor_pagar) < 0) and valor_receber > 0:
                positivo_negativo = 'N'
            vals_movimentacao = {
                'company_id': dados_agrupamento_ano_mes[0].get('company_id'),
                'data_prorrogacao': ultimo_dia,
                'vr_previsao_receber': sum(map(lambda a: a.get('vr_previsao_receber') or 0,
                                               dados_agrupamento_ano_mes)),
                'vr_previsao_pagar': sum(map(lambda a: a.get('vr_previsao_pagar') or 0,
                                             dados_agrupamento_ano_mes)),
                'valor_receber': valor_receber,
                'valor_pagar': valor_pagar,
                'saldo': sum(map(lambda a: a.get('saldo') or 0, dados_agrupamento_ano_mes)),
                'saldo_com_previsao': sum(map(lambda a: a.get('saldo_com_previsao') or 0,
                                              dados_agrupamento_ano_mes)),
                'diaria_capa': dados_agrupamento_ano_mes[0].get('diaria_capa'),
                'positivo_negativo': positivo_negativo,
                'sequencia': sequencia
            }
            obj_wizard_movimentacoes.create(cr, uid, vals_movimentacao, context)
        return True

    def get_titulo_receber_vencido(self, cr, uid, context=None):
        """
        O metodo encontra os titulos a receber que estao vencidos. Nao considera previsao
        financeira.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: lista de dicionarios com os dados dos titulos.
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'financeiro.titulo',
                                                                    context=context)
        sql_titulo_vencido = """
            SELECT
                titulo.parcela          AS parcela,
                titulo.pessoa_id        AS pessoa_id,
                titulo.saldo            AS saldo,
                titulo.data_prorrogacao AS data_prorrogacao,
                titulo.documento        AS documento
            FROM
                financeiro_titulo titulo
            WHERE
                state = 'A'
                AND titulo.tipo_titulo = 'R'
                AND titulo.data_prorrogacao < CURRENT_DATE
                AND titulo.company_id = {company_id}
                AND titulo.doc_tipo_id NOT IN (
                    SELECT id FROM documento_documento_tipo WHERE sigla = 'PRV')
        """.format(company_id=company_id)
        cr.execute(sql_titulo_vencido)
        titulo_dados = list()
        map(lambda i: titulo_dados.append((0, 0, i)), cr.dictfetchall())
        return titulo_dados

    def button_titulo_receber_atraso(self, cr, uid, ids, context=None):
        """
        O metodo executa acao para abertura de uma nova janela. Nessa eh possivel selecionar os
        titulos a receber que estao vencidos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com dados da acao
        :rtype: dict
        """
        context = context and context.copy() or {}
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'financeiro', 'view_wizard_posicao_diaria_sintetico_titulo_vencido_form')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Titulos a receber em atraso',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'new'
        }

    def button_tratar_titulo_receber_atraso(self, cr, uid, ids, context=None):
        """
        O metodo trata se todos os titulos serao ou nao selecionados.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Dicionario com dados para execucao de uma acao que abrira novamente a mesma janela
        ja com os registros selecionados ou nao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_wiz_financeiro_titulo_vencido = self.pool.get('wiz.financeiro.titulo.vencido')
        for x in self.browse(cr, uid, ids, context):
            titulo_vencido_ids = map(lambda i: i.id, x.titulo_vencido_ids)
            selecionado = False
            if context.get('marcar_todos'):
                selecionado = True
            obj_wiz_financeiro_titulo_vencido.write(cr, uid, titulo_vencido_ids,
                                                    {'selecionado': selecionado}, context)
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'financeiro', 'view_wizard_posicao_diaria_sintetico_titulo_vencido_form')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Titulos a receber em atraso',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'new'
        }

    def button_confirma_titulo_receber_atraso(self, cr, uid, ids, context=None):
        """
        O metodo soma o valor a receber do ultimo dia do periodo com o total dos titulos
        selecionados.
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Dicionario com dados para execucao de uma acao que abrira novamente o mesmo
        formulario ja com o valor da linha atualizado.
        :rtype: dict
        """
        context = context and context.copy() or {}
        for x in self.browse(cr, uid, ids, context):
            if not x.titulo_vencido_ids:
                continue

            titulo_vencido_selecionado_ids = filter(lambda i: i.selecionado, x.titulo_vencido_ids)
            if not titulo_vencido_selecionado_ids or not x.movimento_diario_ids:
                continue
            vr_total_titulo_vencido = sum(map(lambda i: i.saldo, titulo_vencido_selecionado_ids))
            dado = {
                'valor_receber': vr_total_titulo_vencido + x.movimento_diario_ids[-1].valor_receber,
                'saldo': vr_total_titulo_vencido + x.movimento_diario_ids[-1].saldo,
                'saldo_com_previsao': vr_total_titulo_vencido +
                x.movimento_diario_ids[-1].saldo_com_previsao,
                'saldo_acumulado': vr_total_titulo_vencido +
                x.movimento_diario_ids[-1].saldo_acumulado
            }
            self.write(cr, uid, x.id,
                       {'movimento_diario_ids': [(1, x.movimento_diario_ids[-1].id, dado)]},
                       context)

        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'financeiro', 'view_wizard_posicao_diaria_sintetico_form')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Sintetico',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'inline'
        }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if ids:
            datas = self.read(cr, uid, ids[0], context=context)
            return {
                'type': 'ir.actions.report.xml',
                'report_name': 'posicao_diario_sintetico_osv',
                'datas': datas,
                }

