# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.parser import parse


class wizard_titulo_movimentado(osv.TransientModel):
    # prefixo: wiztmo_
    _name = 'wizard.titulo.movimentado'
    _columns = {
        'wiztmo_data_inicial': fields.date('Data inicial'),
        'wiztmo_data_final': fields.date('Data final'),
        'wiztmo_tipo_periodo': fields.selection([('E', 'Emissao/Baixa'), ('L', 'Lancamento')],
                                                'Data do periodo', size=1),
        'wiztmo_movimento': fields.selection(
            [('I', 'Entrada'), ('A', 'Alteracao'), ('R', 'Registro'), ('B', 'Baixa'),
             ('BP', 'Baixa Parcial'), ('PR', 'Prorrogacao'), ('C', 'Cancelamento'),
             ('TR', 'Transformacao'), ('BR', 'Baixa por Renegociacao'), ('EB', 'Extorno de Baixa')],
            'Movimento', size=2),
        'wiztmo_doc_tipo_id': fields.many2one('documento.documento.tipo', 'Tipo de documento'),
        'wiztmo_ordem': fields.selection(
            [('C', 'Cliente/Fornecedor'), ('D', 'Data')], 'Ordem', size=1),
        'wiztmo_tipo_titulo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo do titulo',
                                               size=1),
        'wiztmo_pessoa_id': fields.many2one('adm.pessoa', 'Cliente/Fornecedor'),
    }
    _defaults = {
        'wiztmo_ordem': lambda *a: 'D',
    }

    def wiztmo_button_print(self, cr, uid, ids, context=None):
        """
        O metodo reune os dados necessarios para gerar o relatorio em arquivo pdf. Seu retorno e uma
        acao (action) com a chamada do relatorio.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identidicador do usuario.
        :param ids: lista de id da tabela desse objeto
        :param context: dicionario com valores contextuais
        :return: dicionario com acao para chamada do relatorio
        """
        context = context and context.copy() or {}
        brw_titulo_movimentado = self.browse(cr, uid, ids, context)
        for mov in brw_titulo_movimentado:
            data_ini = mov.wiztmo_data_inicial or ''
            if data_ini:
                data_ini = parse(data_ini).strftime('%d/%m/%Y')
            data_fim = mov.wiztmo_data_final or ''
            if data_fim:
                data_fim = parse(data_fim).strftime('%d/%m/%Y')
            str_tipo_periodo = 'Lancamento'
            tipo_periodo = 'L'
            if mov.wiztmo_tipo_periodo == 'E':
                str_tipo_periodo = 'Emissao/Baixa'
                tipo_periodo = 'E'
            movimento = mov.wiztmo_movimento or False
            str_doc_tipo = 'Todos'
            doc_tipo_id = False
            doc_tipo = mov.wiztmo_doc_tipo_id
            if doc_tipo:
                str_doc_tipo = doc_tipo.name or ''
                doc_tipo_id = doc_tipo.id
            ordem = mov.wiztmo_ordem or False
            str_ordem = ''
            if ordem == 'D':
                str_ordem = 'Data'
            elif ordem == 'C':
                str_ordem = 'Cliente/Fornecedor'
            tipo_titulo = mov.wiztmo_tipo_titulo or False
            str_tipo_titulo = 'Todos'
            if tipo_titulo == 'R':
                str_tipo_titulo = 'Receber'
            elif tipo_titulo == 'P':
                str_tipo_titulo = 'Pagar'
            str_movimento = self.wiztmo_get_descricao_movimento(cr, uid, movimento, context)
            vendedor_id = False
            vendedor_externo_id = False
            if context.get('titulo_movimentado_vendedor'):
                vendedor_id = mov.wiztmo_vendedor_id.id or False
                vendedor_externo_id = mov.wiztmo_vendedor_externo_id.id or False
            dados = {
                'data_inicial': data_ini,
                'data_final': data_fim,
                'tipo_periodo': str_tipo_periodo,
                'movimento': str_movimento,
                'doc_tipo': str_doc_tipo,
                'ordem': str_ordem,
                'tipo_titulo': str_tipo_titulo,
                'pessoa': mov.wiztmo_pessoa_id.nome or ''
            }
            dict_dado = {
                'data_inicial': data_ini,
                'data_final': data_fim,
                'tipo_periodo': tipo_periodo,
                'movimento': movimento,
                'doc_tipo_id': doc_tipo_id,
                'ordem': ordem,
                'tipo_titulo': tipo_titulo,
                'vendedor_id': vendedor_id,
                'vendedor_externo_id': vendedor_externo_id,
                'pessoa_id': mov.wiztmo_pessoa_id.id or False
            }
            dados.update({'dado_movimento': self.wiztmo_get_dado_movimento(
                cr, uid, dict_dado, context)})
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'titulo_movimentado',
            'datas': dados,
        }

    def wiztmo_get_dado_movimento(self, cr, uid, dict_dado, context=None):
        """
        O metodo reune os dados do movimento de titulos baseado nos parametros recebidos em
        dict_dado.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param dict_dado: dicionario com parametros de pesquisa ao movimento de titulos.
        :param context: dicionario com valores conextuais.
        :return: lista de dicionarios. Cada dicionario contem todos os dados necessarios do titulo
        movimentado.
        """
        context = context and context.copy() or {}
        company_ids = self.pool.get('res.company').search(
            cr, uid, [('parent_id', 'child_of', self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, self._name, context=context))], context=context)
        sql = """
            SELECT
                ap.codigo                                   AS pessoa_codigo,
                ap.nome                                     AS pessoa_nome,
                ap.endereco                                 AS pessoa_endereco,
                ap.cidade                                   AS pessoa_cidade,
                ap.uf                                       AS pessoa_uf,
                ft.documento                                AS titulo_documento,
                ft.data_emissao                             AS titulo_emissao,
                ft.vr_original                              AS titulo_valor,
                ft.data_prorrogacao                         AS titulo_vencimento,
                ft.tipo_titulo                              AS titulo_tipo,
                date(date_trunc('day', fth.create_date))    AS movimento_lancamento,
                flc.id                                      AS movimento_bco,
                ftb.valor_pago                              AS movimento_valor_pago,
                fth.operacao                                AS operacao,
                ftb.valor_juros                             AS movimento_valor_juros,
                ftb.valor_desconto                          AS movimento_valor_desconto,
                ft.observacoes                              AS titulo_observacao
            FROM financeiro_titulo_historico fth
            INNER JOIN financeiro_titulo ft
                ON ft.id = fth.titulo_id
            LEFT JOIN financeiro_titulo_baixa ftb
                ON ft.id = ftb.titulo_id
            INNER JOIN financeiro_local_cobranca flc
                ON flc.id = ft.local_cobranca_id
            INNER JOIN documento_documento_tipo dt
                ON dt.id = ft.doc_tipo_id
            INNER JOIN adm_pessoa ap
                ON ap.id = ft.pessoa_id
            WHERE
                ft.company_id IN (%s)
        """ % ', '.join(map(lambda c: str(c), company_ids))
        if not any(dict_dado.values()):
            cr.execute(sql)
            dado_movimento = cr.dictfetchall()
            return dado_movimento
        if dict_dado.get('data_inicial') and dict_dado.get('tipo_periodo') != 'E':
            sql += """
                AND fth.create_date >= '%s 00:00:00'
            """ % dict_dado.get('data_inicial')
        if dict_dado.get('data_final') and dict_dado.get('tipo_periodo') != 'E':
            sql += """
                AND fth.create_date <= '%s 23:59:59'
            """ % dict_dado.get('data_final')
        if dict_dado.get('movimento'):
            sql += """
                AND fth.operacao = '%s'
            """ % dict_dado.get('movimento')
        if dict_dado.get('doc_tipo_id'):
            sql += """
                AND dt.id = %s
            """ % dict_dado.get('doc_tipo_id')
        if dict_dado.get('tipo_titulo'):
            sql += """
                AND ft.tipo_titulo = '%s'
            """ % dict_dado.get('tipo_titulo')
        if dict_dado.get('vendedor_id'):
            sql += """
                AND ft.vendedor_id = %s
            """ % dict_dado.get('vendedor_id')
        if dict_dado.get('vendedor_externo_id'):
            sql += """
                AND ft.vendedor_externo_id = %s
            """ % dict_dado.get('vendedor_externo_id')
        if dict_dado.get('pessoa_id'):
            sql += """
                AND ap.id = %s
            """ % dict_dado.get('pessoa_id')
        if dict_dado.get('tipo_periodo') == 'E' and dict_dado.get('data_inicial'):
            sql += """
                AND CASE
                        WHEN ft.state = 'B' THEN ftb.data_baixa >= '{data_inicial}'
                        ELSE ft.data_emissao >= '{data_inicial}'
                    END
            """.format(data_inicial=dict_dado.get('data_inicial'))
        if dict_dado.get('tipo_periodo') == 'E' and dict_dado.get('data_final'):
            sql += """
                AND CASE
                        WHEN ft.state = 'B' THEN ftb.data_baixa <= '{data_final}'
                        ELSE ft.data_emissao <= '{data_final}'
                    END
            """.format(data_final=dict_dado.get('data_final'))
        if dict_dado.get('ordem'):
            if dict_dado.get('ordem') == 'C':
                sql += """
            ORDER BY ap.nome
                """
            else:
                sql += """
            ORDER BY fth.create_date, ap.nome
                """
        cr.execute(sql)
        dado_movimento = cr.dictfetchall()
        return dado_movimento

    def wiztmo_get_descricao_movimento(self, cr, uid, movimento, context=None):
        """
        O metodo recebe do parametro 'movimento' um char de tamanho 1 que identifica o tipo de
        movimento do titulo.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param movimento: char que representa o tipo de movimento.
        :param context: dicionario com valores contextuais.
        :return: string com a descricao do tipo de movimento
        """
        context = context and context.copy() or {}
        str_movimento = 'Todos'
        if not movimento:
            return str_movimento
        if movimento == 'I':
            str_movimento = 'Entrada'
        elif movimento == 'A':
            str_movimento = 'Alteracao'
        elif movimento == 'R':
            str_movimento = 'Registro'
        elif movimento == 'B':
            str_movimento = 'Baixa'
        elif movimento == 'BP':
            str_movimento = 'Baixa parcial'
        elif movimento == 'PR':
            str_movimento = 'Prorrogacao'
        elif movimento == 'C':
            str_movimento = 'Cancelamento'
        elif movimento == 'TR':
            str_movimento = 'Transformacao'
        elif movimento == 'BR':
            str_movimento = 'Baixa por renegociacao'
        elif movimento == 'EB':
            str_movimento = 'Extorno de Baixa'
        return str_movimento
