# -*- coding: utf-8 -*_

from openerp.osv import osv, fields
from dateutil.parser import parse
from SigUtils.numero import getNumeroEditado
import base64
from openerp.tools.translate import _


class wizard_exporta_baixa_dominio(osv.TransientModel):
    _name = 'wizard.exporta.baixa.dominio'

    def wizdom_get_dados_sem_conta_contabil(self, cr, uid, ids, field_name, args, context=None):
        context = context and context.copy() or {}
        res = {}
        obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
        for dominio in self.browse(cr, uid, ids, context):
            res[dominio.id] = {'wizdom_pessoa_ids': [], 'wizdom_conta_ids': []}
            baixa_ids = obj_financeiro_titulo_baixa.search(
                cr, uid,
                [
                    ('data_baixa', '>=', dominio.wizdom_data_inicio),
                    ('data_baixa', '<=', dominio.wizdom_data_fim),
                ],
                context=context
            )
            if not baixa_ids:
                continue
            brw_financeiro_titulo_baixa = obj_financeiro_titulo_baixa.browse(cr, uid, baixa_ids,
                                                                             context)
            pessoa_ids = list(set(map(lambda ftb: ftb.fintib_pessoa_id.id,
                                      brw_financeiro_titulo_baixa)))

            obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
            fornecedor_ids = obj_financeiro_fornecedor.search(
                cr, uid, [
                    ('pessoa_id', 'in', pessoa_ids),
                    ('finfor_dom_conta_contabil', '=', False)
                ], context=context)
            fornecedor_pessoa_ids = map(
                lambda ff: ff.pessoa_id.id, obj_financeiro_fornecedor.browse(
                    cr, uid, fornecedor_ids, context))

            obj_financeiro_cliente = self.pool.get('financeiro.cliente')
            cliente_ids = obj_financeiro_cliente.search(
                cr, uid, [
                    ('pessoa_id', 'in', pessoa_ids),
                    ('fincli_dom_conta_contabil', '=', False)
                ], context=context)
            cliente_pessoa_ids = map(lambda fc: fc.pessoa_id.id, obj_financeiro_cliente.browse(
                cr, uid, cliente_ids, context))

            pessoa_ids = list(set(fornecedor_pessoa_ids + cliente_pessoa_ids))
            conta_ids = map(lambda ftb: ftb.tesouraria_movimento_id.conta_id,
                            brw_financeiro_titulo_baixa)
            conta_ids = list(set(map(
                lambda c: c.id, filter(lambda ftc: not ftc.fintco_dom_conta_contabil, conta_ids))))
            res[dominio.id] = {'wizdom_pessoa_ids': pessoa_ids, 'wizdom_conta_ids': conta_ids}
        return res

    _columns = {
        'wizdom_data_inicio': fields.date('Data inicial', required=True),
        'wizdom_data_fim': fields.date('Data final', required=True),
        'wizdom_pessoa_ids': fields.function(
            wizdom_get_dados_sem_conta_contabil, method=True, sting='Clientes/Fornecedores',
            relation='adm.pessoa', type='one2many', multi='sem_conta_contabil'),
        'wizdom_conta_ids': fields.function(
            wizdom_get_dados_sem_conta_contabil, method=True, sting='Contas da Tesouraria',
            relation='financeiro.tesouraria.contas', type='one2many', multi='sem_conta_contabil'),
        'wizdom_arquivo': fields.binary('Arquivo', readonly=True),
        'wizdom_arquivo_nome': fields.char('Nome Arquivo', size=64),
    }
    _defaults = {
        'wizdom_arquivo_nome': 'integracao_dominio.txt',
    }

    def wizdom_button_export(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Exportar', responsavel pela chamada do metodo do metodo que tratara os
        dados da baixa.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabelas res_users, identificador do usuario na transacao
        :type uid: int
        :param ids: lista de identificadores da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        if not self.wizdom_trata_exportacao_dados_baixa(cr, uid, ids, context):
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para o periodo.'))
        return True

    def wizdom_trata_exportacao_dados_baixa(self, cr, uid, ids, context=None):
        """
        Metodo responsavel pelo tratamento dos dados da baixa para geracao do arquivo para download.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabelas res_users, identificador do usuario na transacao
        :type uid: int
        :param ids: lista de identificadores da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro, se houve dados para geracao do arquivo, senao, falso
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False
        brw_exporta_baixa_dominio = self.browse(
            cr, uid, isinstance(ids, (list, tuple)) and ids[0] or ids, context)
        obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        remove_acentuacao = self.pool.get('adm.utils').admutl_remover_acentuacao
        baixa_ids = obj_financeiro_titulo_baixa.search(
            cr, uid,
            [
                ('data_baixa', '>=', brw_exporta_baixa_dominio.wizdom_data_inicio),
                ('data_baixa', '<=', brw_exporta_baixa_dominio.wizdom_data_fim),
            ],
            context=context
        )
        if not baixa_ids:
            return res
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'financeiro.titulo.baixa', args=None, context=context)
        cpf_cnpj = self.pool.get('res.company').browse(
            cr, uid, company_id, context).rescon_pessoa_id.cpf_cnpj.replace('.', '').\
            replace('-', '').replace('/', '')
        file_directory = '/tmp/dominio_integration_file_data.txt'
        file_data = open(file_directory, 'wb')
        registro_0000 = '|{id_registro}|{cpf_cnpj}|\r\n'.format(
            id_registro='0000',
            cpf_cnpj=cpf_cnpj
        )
        file_data.write(registro_0000)
        registro_6000 = '|{id_registro}|{tipo_lancamento}|{cod_lancamento_padrao}|{localizador}' \
                        '|{rtt_fcont}|\r\n'.\
            format(
                id_registro='6000',
                tipo_lancamento='X',
                cod_lancamento_padrao='',
                localizador='',
                rtt_fcont='',
            )
        finpar_dom_codigo = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', ['finpar_dom_codigo'], context=context).get(
            'finpar_dom_codigo') or ''
        brw_financeiro_titulo_baixa = obj_financeiro_titulo_baixa.browse(cr, uid, baixa_ids,
                                                                         context)
        for baixa in brw_financeiro_titulo_baixa:
            file_data.write(registro_6000)
            if baixa.titulo_id.tipo_titulo == 'R':
                conta_contabil_debito = (
                    baixa.tesouraria_movimento_id
                    and baixa.tesouraria_movimento_id.conta_id.nro_conta) \
                    or 'Não cadastrado'  # banco ou caixa
                conta_cliente = 'Não cadastrado'
                cliente_ids = obj_financeiro_cliente.search(
                    cr, uid, [('pessoa_id', '=', baixa.titulo_id.pessoa_id.id)], context=context)
                if cliente_ids:
                    conta_cliente = obj_financeiro_cliente.read(
                        cr, uid, cliente_ids[0], ['fincli_dom_conta_contabil'], context=context).\
                        get('fincli_dom_conta_contabil') or conta_cliente
                conta_contabil_credito = conta_cliente  # conta do cliente
                historico = 'RECEBIMENTO DUPL. %s - %s' % (baixa.titulo_id.documento,
                                                           baixa.titulo_id.pessoa_id.nome)
            else:
                conta_fornecedor = 'Não cadastrado'
                fornecedor_ids = obj_financeiro_fornecedor.search(
                    cr, uid, [('pessoa_id', '=', baixa.titulo_id.pessoa_id.id)], context=context)
                if fornecedor_ids:
                    conta_fornecedor = obj_financeiro_fornecedor.read(
                        cr, uid, fornecedor_ids[0], ['finfor_dom_conta_contabil'],
                        context=context).get('finfor_dom_conta_contabil') or conta_fornecedor
                conta_contabil_debito = conta_fornecedor  # conta do fornecedor
                conta_contabil_credito = (
                    baixa.tesouraria_movimento_id
                    and baixa.tesouraria_movimento_id.conta_id.nro_conta) \
                    or 'Não cadastrado'  # banco ou caixa
                historico = 'PAGAMENTO DUPL. %s - %s' % (baixa.titulo_id.documento,
                                                         baixa.titulo_id.pessoa_id.nome)
            registro_6001 = '|{id_registro}|{data_lancamento}|{conta_contabil_debito}' \
                            '|{conta_contabil_credito}|{valor_lancamento}' \
                            '|{cod_historico_contabil}|{desc_historico_contabil}|{usuario}' \
                            '|{cod_filial}|{lanc_scp}|\r\n'.\
                format(
                    id_registro='6001',
                    data_lancamento=parse(baixa.data_baixa).strftime('%d/%m/%Y'),
                    conta_contabil_debito=conta_contabil_debito,
                    conta_contabil_credito=conta_contabil_credito,
                    valor_lancamento=getNumeroEditado(baixa.valor_pago),
                    cod_historico_contabil='',
                    desc_historico_contabil=remove_acentuacao(cr, uid, historico),
                    usuario='',
                    cod_filial=finpar_dom_codigo,
                    lanc_scp='',
                )
            file_data.write(registro_6001)
        file_data.close()
        file_data = open(file_directory, 'r')
        data = file_data.read()
        data = base64.encodestring(data)
        self.write(cr, uid, ids, {'wizdom_arquivo': data}, context)
        return True

