# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from SigUtils.cnab.bancos import itau, hsbc, cef, sicoob, bb, bradesco, inter, sicredi
from SigUtils.cnab.tipos import ArquivoCob
import os
import codecs
import base64
from openerp.tools.translate import _
from openerp import SUPERUSER_ID
import time
from random import randint
from SigUtils.cnab.bancos.inter import versao_layout_inter
from datetime import datetime


class wizard_cobranca_magnetica(osv.TransientModel):
    _name = "wizard.cobranca.magnetica"

    _columns = {
        'id': fields.integer('Id'),
        'local_cobranca_id': fields.many2one('financeiro.local.cobranca', 'Local cobranca', required=True),
        'banco_bc': fields.integer('Banco'),
        'local_natureza_id': fields.many2one('financeiro.local.natureza', 'Natureza',
                                             required=True, domain="[('finlon_local_cobranca_id', "
                                                                   "'=', local_cobranca_id)]"),
        'titulo_ids': fields.one2many('wizard.cobranca.magnetica.titulos', 'cobranca_magnetica_id', 'Titulos'),
        'titulo_selecionado_ids': fields.one2many('wizard.cobranca.magnetica.titulos.selecionados',
                                                  'cobranca_magnetica_id', 'Titulos selecionados'),
        'operacao': fields.selection([('E', 'Entrada de titulo'), ('B', 'Baixa de titulo')],
                                     'Operacao', size=1, required=True),
        'state_banco': fields.selection(
            [('N', 'Nao enviado'), ('E', 'Enviado'), ('C', 'confirmado'), ('R', 'Rejeitado')],
            'Situacao no banco', size=1),
        'cliente_id': fields.many2one('adm.pessoa', 'Cliente'),
        'vencimento_inicial': fields.date('Vencimento inicial'),
        'vencimento_final': fields.date('Vencimento final'),
        'emissao_inicial': fields.date('Emissao inicial'),
        'emissao_final': fields.date('Emissao final'),
        'doc_tipo_id': fields.many2one('documento.documento.tipo', 'Tipo de documento'),
        'valor_minimo': fields.float('Valor minimo', digits=(10, 2)),
        'remessa_gerada': fields.boolean('Remessa gerada'),
        'titulos_selecionados': fields.boolean('Titulos gerados'),
        'log': fields.char('Log', size=50),
        'file_name': fields.char('Arquivo', size=30, readonly=False),
        'anexo': fields.binary('Anexo'),
        'codigo_protesto': fields.selection([('0', 'Sem instrucao'), ('1', 'Protestar(dias corridos'),
                                             ('2', 'Protestar(dias uteis)'), ('3', 'Nao protestar'),
                                             ('7', 'Negativar(dias corridos'), ('8', 'Nao negativar')],
                                            'Codigo protesto', size=1, required=True,
                                            help='Informa se o titulo deve ser protestado ou nao'),
        'dias_protesto': fields.integer('Dias para protesto'),
        'codigo_baixa': fields.selection([('0', 'Sem instrucao'), ('1', 'Baixar apos x dias vencido')],
                                         'Codigo para baixa', size=1, required=1,
                                         help='Informa se apos vencido o titulo devera ser baixado'),
        'dias_baixa': fields.integer('Dias para baixa'),
        'filtro_local_cobranca_id': fields.many2one('financeiro.local.cobranca', 'Local cobranca'),
        'mensagem1': fields.char('Mensagem 1', size=40, help='Mensagem para todos os titulos'),
        'mensagem2': fields.char('Mensagem 2', size=40, help='Mensagem para todos os titulos'),
        'especie': fields.many2one('financeiro.febraban.especie','Especie', required=True,
                                   domain="[('finffe_banco_bc', '=', banco_bc)]"),
    }
    _defaults = {
        'remessa_gerada': False,
        'titulos_selecionados': False,
        'log': 'REMESSA GERADA COM SUCESSO',
        'codigo_protesto': '0',
        'dias_protesto': 0,
        'codigo_baixa': '0',
        'dias_baixa': 0,
        'operacao': 'E',
        'state_banco': 'N',
    }

    def button_busca_titulos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_wizard_cobranca_magnetica_titulos = self.pool.get('wizard.cobranca.magnetica.titulos')
        args_titulo = []
        ids_titulos = obj_wizard_cobranca_magnetica_titulos.search(
            cr, uid, [('cobranca_magnetica_id', '=', ids[0])], context=context)
        if ids_titulos:
            obj_wizard_cobranca_magnetica_titulos.unlink(cr, uid, ids_titulos)
        brw_cobranca_magnetica = self.browse(cr, uid, ids[0])
        cliente_id = brw_cobranca_magnetica.cliente_id.id or False
        vencimento_inicial = brw_cobranca_magnetica.vencimento_inicial or ''
        vencimento_final = brw_cobranca_magnetica.vencimento_final or ''
        emissao_inicial = brw_cobranca_magnetica.emissao_inicial or False
        emissao_final = brw_cobranca_magnetica.emissao_final or False
        doc_tipo_id = brw_cobranca_magnetica.doc_tipo_id.id or False
        valor_minimo = brw_cobranca_magnetica.valor_minimo or 0.0
        filtro_local_cobranca_id = brw_cobranca_magnetica.filtro_local_cobranca_id or False
        state_banco = brw_cobranca_magnetica.state_banco or False
        if cliente_id:
            args_titulo.append(('pessoa_id', '=', cliente_id), )
        if vencimento_inicial and vencimento_final:
            if vencimento_inicial > vencimento_final:
                raise osv.except_osv(_('Atencao'), _('Informe um periodo de vencimento valido'))
        if vencimento_inicial:
            args_titulo.append(('data_prorrogacao', '>=', vencimento_inicial), )
        if vencimento_final:
            args_titulo.append(('data_prorrogacao', '<=', vencimento_final), )
        if emissao_inicial:
            args_titulo.append(('data_emissao', '>=', emissao_inicial), )
        if emissao_final:
            args_titulo.append(('data_emissao', '<=', emissao_final), )
        if doc_tipo_id:
            args_titulo.append(('doc_tipo_id', '=', doc_tipo_id), )
        if valor_minimo:
            args_titulo.append(('vr_original', '>=', valor_minimo), )
        if filtro_local_cobranca_id:
            args_titulo.append(('local_cobranca_id', '=', filtro_local_cobranca_id.id))
        if state_banco:
            args_titulo.append(('state_banco', '=', state_banco))
        args_titulo.extend([('state', '=', 'A'), ('tipo_titulo', '=', 'R')])
        titulo_ids = obj_financeiro_titulo.search(cr, uid, args_titulo, context=context)
        if titulo_ids:
            browse_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, titulo_ids)
            for titulo in browse_financeiro_titulo:
                vals = {
                    'cobranca_magnetica_id': ids[0],
                    'pessoa_id': titulo.pessoa_id.id,
                    'documento': titulo.documento or '',
                    'parcela': titulo.parcela or False,
                    'data_emissao': titulo.data_emissao or False,
                    'vr_original': titulo.vr_original or 0.0,
                    'data_prorrogacao': titulo.data_prorrogacao or False,
                    'titulo_id': titulo.id,
                }
                obj_wizard_cobranca_magnetica_titulos.create(cr, uid, vals, context)
        else:
            raise osv.except_osv(_('Atencao'), _('Nao ha titulos para os filtros selecionados'))
        return True

    def button_adiciona_titulos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_wizard_cobranca_magnetica_titulos_selecionados = self.pool.get(
            'wizard.cobranca.magnetica.titulos.selecionados')
        brw_cobranca_magnetica = self.browse(cr, uid, ids[0])
        brw_cobranca_magnetica_titulos = brw_cobranca_magnetica.titulo_ids
        if brw_cobranca_magnetica_titulos:
            for x in brw_cobranca_magnetica_titulos:
                titulo_id = x.titulo_id.id
                titulos_selecionados_id = obj_wizard_cobranca_magnetica_titulos_selecionados.search(
                    cr, uid, [('titulo_id', '=', titulo_id), ('cobranca_magnetica_id', 'in', ids)], context=context)
                if not titulos_selecionados_id:
                    vals = {
                        'cobranca_magnetica_id': ids[0],
                        'pessoa_id': x.pessoa_id.id,
                        'documento': x.documento or '',
                        'operacao': x.cobranca_magnetica_id.operacao or '',
                        'vr_original': x.vr_original or 0.0,
                        'data_prorrogacao': x.data_prorrogacao or False,
                        'titulo_id': titulo_id,
                    }
                    obj_wizard_cobranca_magnetica_titulos_selecionados.create(cr, uid, vals, context)
        else:
            raise osv.except_osv(_('Atencao'), _('Nao ha titulos a selecionar'))
        self.write(cr, uid, ids, {'titulos_selecionados': True}, context)
        return True

    def button_gerar_remessa(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_cnab_cnab = self.pool.get('cnab.cnab')
        brw_cobranca_magnetica = self.browse(cr, uid, ids[0])
        local_natureza_id = brw_cobranca_magnetica.local_natureza_id.id
        banco = brw_cobranca_magnetica.local_natureza_id.finlon_local_cobranca_id.\
            finloc_banco_id.admbco_codigo_bc or 0
        padrao_cnab = brw_cobranca_magnetica.local_natureza_id.finlon_padrao_cnab
        if banco:
            banco = str(banco)
        if banco == '341':
            _obj_banco = itau
        elif banco == '399':
            _obj_banco = hsbc
        elif banco == '104':
            _obj_banco = cef
        elif banco == '1':
            _obj_banco = bb
        elif banco == '756':
            _obj_banco = sicoob
        elif banco == '237':
            _obj_banco = bradesco
        elif banco == '77':
            _obj_banco = inter
        elif banco == '748':
            _obj_banco = sicredi
        else:
            raise osv.except_osv(_('Atencao'), _('Layout nao implementado para este banco!'))
        _nome = _obj_banco.__name__.split('.')[-1]
        dados = obj_cnab_cnab.cnbcnb_get_data_from_dict(cr, uid, ids[0], local_natureza_id, context)
        tam_reg_cnab = 240
        nome = '%s.rem' % _nome
        if banco == '237':
            nome = 'CB' + time.strftime('%d%m') + str(randint(10, 99)) + '.REM'
        if banco == '77':
            tam_reg_cnab = 400
            numero_sequencial_inter = randint(1, 9999999)
            nome = 'CI400_' + versao_layout_inter + '_' + str(numero_sequencial_inter).zfill(7) + \
                '.rem'
            dados['arquivo']['arquivo_sequencia'] = numero_sequencial_inter
        if banco == '748':
            sicredi_mes = datetime.now().month
            if sicredi_mes == 10:
                sicredi_mes = 'O'
            elif sicredi_mes == 11:
                sicredi_mes = 'N'
            elif sicredi_mes == 12:
                sicredi_mes = 'D'
            sicredi_mes = sicredi_mes.__str__()
            nome = brw_cobranca_magnetica.local_natureza_id.finlon_conta + sicredi_mes \
                + datetime.now().day.__str__() + '.' + randint(1, 999).__str__().zfill(3)
        if banco == '341' and padrao_cnab == '400':
            tam_reg_cnab = 400
        arquivo = ArquivoCob(_obj_banco, tam_reg_cnab, **dados)
        return_file_path = os.path.join('/tmp/', nome)
        ret_file = codecs.open(return_file_path, 'w', encoding='ascii')
        ret_file.seek(0)
        arquivo.incluir_cobranca(ret_file, dados['cobranca'])
        cobranca = open(return_file_path, 'r')
        dados_cobranca = ''.join(map(lambda x: x, cobranca))
        self._grade_ids = []
        self.write(cr, uid, ids, {'remessa_gerada': True,
                                  'file_name': nome,
                                  'anexo': base64.encodestring(dados_cobranca)}, context)
        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'financeiro_febraban',
                                                        'wizard_cobranca_magnetica_form_log')
        view_id = reference and reference[1] or False
        obj_wizard_cobranca_magnetica_titulos = self.pool.get('wizard.cobranca.magnetica.titulos')
        titulo_ids = obj_wizard_cobranca_magnetica_titulos.search(cr, uid, [], context=context)
        obj_wizard_cobranca_magnetica_titulos.unlink(cr, uid, titulo_ids)
        obj_wizard_cobranca_magnetica_titulos_selecionados = self.pool.get(
            'wizard.cobranca.magnetica.titulos.selecionados')
        titulo_selecionado_ids = obj_wizard_cobranca_magnetica_titulos_selecionados.search(cr, uid, [], context=context)
        obj_wizard_cobranca_magnetica_titulos_selecionados.unlink(cr, uid, titulo_selecionado_ids)
        return {'name': 'Cobranca magnetica',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wizard.cobranca.magnetica',
                'view_id': [view_id],
                'target': 'new',
                'type': 'ir.actions.act_window'
                }

    def onchange_local_cobranca(self, cr, uid, ids, local_id, context=None):
        if not local_id:
            return True
        obj_local_cobranca = self.pool.get('financeiro.local.cobranca')
        brw = obj_local_cobranca.browse(cr, uid, local_id, context=context)
        vals = {}
        if len(brw.finloc_local_natureza_ids) == 1:
            vals['local_natureza_id'] = brw.finloc_local_natureza_ids[0].id
        else:
            vals['local_natureza_id'] = 0
        vals['banco_bc'] = 0
        try:
            if brw.finloc_conta_id.banco_id.admbco_codigo_bc:
                vals['banco_bc'] = brw.finloc_conta_id.banco_id.admbco_codigo_bc
        except:
            pass
        return {'value': vals}

    def onchange_local_natureza(self, cr, uid, ids, local_cobranca_id, local_natureza_id, context=None):
        vals = {}
        especie, codigo_protesto, dias_protesto, codigo_baixa, dias_baixa, mensagem1, mensagem2 = self._buscar_parametros(cr, uid, local_cobranca_id, local_natureza_id,
                                                                context=context)
        vals['especie'] = especie
        vals['codigo_protesto'] = codigo_protesto
        vals['dias_protesto'] = dias_protesto
        vals['codigo_baixa'] = codigo_baixa
        vals['dias_baixa'] = dias_baixa
        vals['mensagem1'] = mensagem1
        vals['mensagem2'] = mensagem2
        return {'value': vals}

    @staticmethod
    def __get_parametro_chave(local_id, local_natureza_id):
        # a chave é o texto'Cobranca magnetica', o id do local, o id de local_natureza
        return '%s-%s-%s' % ('Cobranca magnetica', local_id, local_natureza_id)

    def _salvar_parametros(self, cr, uid, ids, context=None):
        for brw in self.browse(cr, uid, ids, context=context):
            if brw.operacao != 'E':
                continue
            chave = self.__get_parametro_chave(brw.local_cobranca_id.id, brw.local_natureza_id.id)
            #parametro especie, mensagem1 e mensagem2
            values = '%s*|*%s*|*%s*|*%s*|*%s*|*%s*|*%s' % (
                brw.especie.id,
                brw.codigo_protesto,
                brw.dias_protesto,
                brw.codigo_baixa,
                brw.dias_baixa,
                brw.mensagem1 or '',
                brw.mensagem2 or ''
            )
            self.pool.get('ir.config_parameter').set_param(cr, SUPERUSER_ID, chave, values, context)
        return True


    def _buscar_parametros(self, cr, uid, local_id, local_natureza_id, context=None):
        chave = self.__get_parametro_chave(local_id, local_natureza_id)
        values = self.pool.get('ir.config_parameter').get_param(cr, SUPERUSER_ID, chave, False)
        try:
            especie, codigo_protesto, dias_protesto, codigo_baixa, dias_baixa, mensagem1, mensagem2 = values.split('*|*')
            especie = int(especie)
            dias_protesto = int(dias_protesto)
            dias_baixa = int(dias_baixa)
        except:
            especie = mensagem1 = mensagem2 = ''
            codigo_protesto = codigo_baixa = '0'
            dias_protesto = dias_baixa = 0
        return especie, codigo_protesto, dias_protesto, codigo_baixa, dias_baixa, mensagem1, mensagem2

    def create(self, cr, uid, vals, context=None):
        res = super(wizard_cobranca_magnetica, self).create(cr, uid, vals, context=context)
        self._salvar_parametros(cr, uid, [res], context=None)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        super(wizard_cobranca_magnetica, self).write(cr,uid, ids, vals, context=context)
        self._salvar_parametros(cr, uid, ids, context=None)
        return True

class wizard_cobranca_magnetica_titulos(osv.TransientModel):
    _name = "wizard.cobranca.magnetica.titulos"
    _columns = {
        'cobranca_magnetica_id': fields.many2one('wizard.cobranca.magnetica', 'Cobranca magnetica'),
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente'),
        'documento': fields.char('Documento', size=30),
        'parcela': fields.integer('Parcela'),
        'data_emissao': fields.date('Emissao'),
        'vr_original': fields.float('Valor', digits=(15, 2)),
        'data_prorrogacao': fields.date('Vencimento'),
        'titulo_id': fields.many2one('financeiro.titulo', 'Titulo'),
    }


class wizard_cobranca_magnetica_titulos_selecionados(osv.TransientModel):
    _name = "wizard.cobranca.magnetica.titulos.selecionados"
    _columns = {
        'cobranca_magnetica_id': fields.many2one('wizard.cobranca.magnetica', 'Cobranca magnetica'),
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente/Fornecedor'),
        'documento': fields.char('Documento', size=30),
        'vr_original': fields.float('Valor', digits=(15, 2)),
        'data_prorrogacao': fields.date('Vencimento'),
        'titulo_id': fields.many2one('financeiro.titulo', 'Titulo'),
        'operacao': fields.selection([('E', 'Entrada de titulo'), ('B', 'Baixa de titulo')], 'Operacao', size=1),
    }


