# -*- encoding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
# #############################################################################

import base64
import time
import sys

from openerp.osv import osv, fields
from openerp.tools.translate import _
import openerp

from SigUtils.cnab.tipos import ArquivoCob
from SigUtils.cnab.bancos import itau, hsbc, cef, sicoob, bb, inter
from SigUtils import datas
from SigUtils.formatastring import formataString


class wizard_cobranca_magnetica_retorno(osv.TransientModel):
    _name = "wizard.cobranca.magnetica.retorno"
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade'),
        'arquivo_retorno': fields.binary('Arquivo retorno', required=True),
        'baixa_realizada': fields.boolean('Baixa realizada'),
        'titulo_retorno_ids': fields.one2many('wizard.cobranca.magnetica.retorno.titulos',
                                              'cobranca_magnetica_retorno_id', 'Titulos'),
        'state': fields.selection([('I', 'Inicio'), ('C', 'Carregado'), ('P', 'Processado')],
                                  'Estado', size=1),
        'conta_corrente_empresa': fields.char('Conta corrente', size=12),
        'conta_id': fields.many2one('financeiro.tesouraria.contas', 'Conta da tesouraria', ondelete='cascade'),
        'banco_bc': fields.integer('Banco no BC'),
        'numero_retorno': fields.char('Numero retorno', size=8),
        'data_retorno': fields.date('Data retorno'),
        'logo_banco': fields.binary('Logomarca', readonly=True),
        'forma_pagamento_id': fields.many2one('adm.forma.pgto', 'Forma de pagamento',
                                              required=True, ondelete='cascade'),
    }
    _defaults = {
        'baixa_realizada': False,
        'state': 'I',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'forma_pagamento_id': lambda  s, cr, uid, c: s.pool.get('adm.forma.pgto').search(
            cr, uid, [('sigla', '=', 'DN')], context=c)[0]
    }

    def _carregar_layout(self, cr, uid, nroBancoBC, tamanhoRegistro, context=None):
        """
        carregar o layout do bandco
        Args:
            cr:
            uid:
            nroBancoBC: Nro do banco no BC
            tamanhoRegistro: tamanho do registro
            context:

        Returns:

        """
        try:
            if nroBancoBC == 1:
                instance_arquivo = ArquivoCob(bb, tamanhoRegistro, **{})
            elif nroBancoBC == 104:
                instance_arquivo = ArquivoCob(cef, tamanhoRegistro, **{})
            elif nroBancoBC == 341:
                instance_arquivo = ArquivoCob(itau, tamanhoRegistro, **{})
            elif nroBancoBC == 399:
                instance_arquivo = ArquivoCob(hsbc, tamanhoRegistro, **{})
            elif nroBancoBC == 756:
                instance_arquivo = ArquivoCob(sicoob, tamanhoRegistro, **{})
            elif nroBancoBC == 77:
                instance_arquivo = ArquivoCob(inter, tamanhoRegistro, **{})
            else:
                return False
        except:
            return False
        return instance_arquivo


    def _localizar_titulo(self, cr, uid, nosso_numero, documento, nroBancoBC, data_vencimento,
                          context=None):
        """
        Método utilizado para identificar o titulo dento do contas a receber
        @param cr:
        @param uid:
        @param nosso_numero: numero que o banco deu para o titulo
        @param documento: Numero do titulo(documento)
        @param nroBancoBC: numero do banco no BC
        @return: browser do financeiro.titulo ou False
        """
        def __localizar_por_nosso_numero():
            args = [('fintib_nosso_numero', '=', nosso_numero)]
            if local_cobranca_ids:
                args.append(('fintib_local_cobranca_id', 'in', local_cobranca_ids))
            titulo_boleto_ids = obj_financeiro_titulo_boleto.search(cr, uid, args, context=context)
            titulo_ids = map(lambda x: x.fintib_titulo_id.id, obj_financeiro_titulo_boleto.browse(
                cr, uid, titulo_boleto_ids, context))
            if not titulo_ids:
                return False
            args = [('tipo_titulo', '=', 'R'), ('state', '=', 'A'), ('doc_tipo_id', '!=', doc_prv_id)]
            args.append(('id', 'in', titulo_ids))
            titulo_ids = obj_financeiro_titulo.search(cr, uid, args, context=context)
            if len(titulo_ids) == 1:  # Achou o titulo
                return titulo_ids[0]
            return  False

        def __localizar_por_documento():
            args = [('tipo_titulo', '=', 'R'), ('state', '=', 'A'),
                    ('doc_tipo_id', '!=', doc_prv_id), ('documento', '=', documento),
                    ('data_prorrogacao', '=', data_vencimento)]
            titulo_ids = obj_financeiro_titulo.search(cr, uid, args, context=context)
            if len(titulo_ids) == 1:  # Achou o titulo
                return titulo_ids[0]
            elif not titulo_ids:
                return False
            if local_cobranca_ids:
                args.append(('id', 'in', titulo_ids))
                args.append(('local_cobranca_id', 'in', local_cobranca_ids))
                titulo_ids = obj_financeiro_titulo.search(cr, uid, args, context=context)
                if len(titulo_ids) == 1:  # Achou o titulo
                    return titulo_ids[0]
            return False

        context = context and context.copy() or {}
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_financeiro_titulo_boleto = self.pool.get('financeiro.titulo.boleto')
        obj_local_cobranca = self.pool.get('financeiro.local.cobranca')
        local_cobranca_ids = obj_local_cobranca.finloc_get_locais_cobranca_por_banco(cr, uid, nroBancoBC,
                                                                                     context=context)

        doc_prv_id = self.pool.get('documento.documento.tipo').search(cr, uid, [('sigla', '=', 'PRV')], context=context)
        if isinstance(doc_prv_id, (list, tuple)):
            doc_prv_id = doc_prv_id[0]
        titulo_id = __localizar_por_nosso_numero()
        if not titulo_id:
            titulo_id = __localizar_por_documento()
        if titulo_id:
            return obj_financeiro_titulo.browse(cr, uid, titulo_id, context=context)
        else:
            return False

    def button_processar_retorno(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        _carteira_id = self.pool.get('financeiro.local.cobranca').search(cr, uid, [('descricao', '=', 'Carteira')], context=context)
        if _carteira_id and isinstance(_carteira_id, (list, tuple)):
            _carteira_id = _carteira_id[0]
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        obj_financeiro_utils = self.pool.get('financeiro')
        obj_adm_utils = self.pool.get('adm.utils')
        dados_capa = self.read(cr, uid, ids[0], [], context=context, load='_classic_write')
        dados_arquivo = base64.b64decode(dados_capa.get('arquivo_retorno'))
        try:
            dados_arquivo = obj_adm_utils.admutl_remover_acentuacao(cr, uid, dados_arquivo)
        except:
            dados_arquivo = obj_adm_utils.admutl_remove_caracter(
                dados_arquivo, outros=['\r\n', '\n'], substituto=' ')
        if '\r\n' in dados_arquivo:
            arquivo = dados_arquivo.split('\r\n')
        else:
            arquivo = dados_arquivo.split('\n')
        _tamanhoRegistro = len(arquivo[0])
        identificador_banco = arquivo[0][:3]
        try:
            identificador_banco = int(identificador_banco)
        except:
            identificador_banco = 0
        instance_arquivo = self._carregar_layout(cr, uid, identificador_banco, _tamanhoRegistro, context=None)
        msg_erro = u''

        titulos_retorno = instance_arquivo.carregar_retorno(arquivo)
        forma_pagamento_id = dados_capa.get('forma_pagamento_id')
        list_identificacao_titulo = []
        titulo_ids = []
        for x in titulos_retorno:
            _nosso_numero = x.get('nosso_numero')
            if identificador_banco == 756 and len(
                    _nosso_numero) == 15:  # Ajustar porque o nosso numero enviado é acrescido do digito, pardela, carteira, etc
                try:
                    _nosso_numero = str(int(_nosso_numero[:9]))  # se não conseguir converter deixa como estava
                except:
                    pass
            data_vencimento = time.strftime('%Y-%m-%d', time.strptime(x.get('vencimento_titulo'),
                                                                      '%d%m%Y'))
            brw_titulo = self._localizar_titulo(cr, uid, _nosso_numero, x.get('numero_documento'),
                                                identificador_banco, data_vencimento,
                                                context=context)
            if not brw_titulo:
                continue
            titulo_ids.append(brw_titulo.id)
            x['titulo_id'] = brw_titulo.id
            x['saldo'] = brw_titulo.saldo
            if float(x.get('valor_titulo')) < brw_titulo.saldo:
                x['titulo_desconto'] = float(x.get('titulo_desconto')) + \
                (brw_titulo.saldo - float(x.get('valor_titulo')))
            elif brw_titulo.saldo < float(x.get('valor_titulo')):
                x['titulo_acrescimos'] = float(x.get('titulo_acrescimos')) + \
                (float(x.get('valor_titulo')) - brw_titulo.saldo)
            list_identificacao_titulo.append(x)
        if not list_identificacao_titulo:
            raise osv.except_osv(_('Atencao'),
                                 _('Os titulos de retorno nao existem na base de dados'))
        # Gera o movimento na tesouraria e custifica o valor das tarifas
        tesouraria_e_custo = self._custifica_tarifa(
            cr, uid, dados_capa.get('id'), list(set(titulo_ids)), dados_capa.get('conta_id'),
            context)
        if not tesouraria_e_custo:
            raise osv.except_osv(_('Atencao'), _('Cadastre o centro de custo e evento para '
                                                 'tarifa de cobranca no centro de custos '
                                                 'a receber do parametro financeiro'))
        msg_erro_many_titulos = {}
        titulos = []
        titulo_ids = []
        datas_pagamento = []
        datas_credito = []
        for titulo in list_identificacao_titulo:
            titulo_id = titulo.get('titulo_id', False)
            if titulo.get('codigo_movimento') == 6: #Liquidação do titulo
                brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, titulo_id,context)
                _dt_pgto = datas.sigDatas(titulo.get('data_pagamento')).getDataNoFormato('I', 'S',
                                                                                         '-')
                data_credito = ''
                if int(titulo.get('data_credito') or '0'):
                    data_credito = datas.sigDatas(titulo.get('data_credito')).getDataNoFormato(
                        'I', 'S', '-')
                valor_pago = titulo.get('titulo_pago') or 0.0

                saldo_baixado = titulo.get('valor_titulo') or 0.0

                # Tratamento especial para o Banco Itau. O valor pago no arquivo de retorno ja se
                # encontra deduzido o valor da tarifa (valor cobrado pelo boleto).
                if identificador_banco == 341:
                    valor_pago += (titulo.get('valor_tarifas') or 0)
                # elif identificador_banco == 756:
                #     saldo_baixado -= (titulo.get('titulo_acrescimos') or 0)

                titulos.append({'titulo_id': titulo_id,
                                'valor_pago': valor_pago,
                                'saldo_baixado': saldo_baixado,
                                'valor_multa': 0,
                                'valor_juros': titulo.get('titulo_acrescimos') or 0.0,
                                'valor_desconto': titulo.get('titulo_desconto') or 0.0,
                                'tipo_titulo': brw_financeiro_titulo.tipo_titulo,
                                'pessoa_id': brw_financeiro_titulo.pessoa_id.id})
                titulo_ids.append(titulo_id)
                datas_pagamento.append(_dt_pgto)
                datas_credito.append(data_credito)
                context['financ_baixar_titulo'] = True
            elif titulo.get('codigo_movimento') == 9: # Baixa do título
                _nosso_numero = titulo.get('nosso_numero')
                if identificador_banco == 756 and len(
                        _nosso_numero) == 15:  # Ajustar porque o nosso numero enviado é acrescido do digito, pardela, carteira, etc
                    try:
                        _nosso_numero = str(int(_nosso_numero[:9]))  # se não conseguir converter deixa como estava
                    except:
                        pass
                fin_tit_bol_id = self.pool.get('financeiro.titulo.boleto').search(cr, uid, [
                    ('fintib_titulo_id', '=', titulo_id),
                    ('fintib_nosso_numero', '=', _nosso_numero)
                ])
                vals = {'local_cobranca_id': _carteira_id, 'natureza_cobranca_id': '', 'state_banco': 'N'}
                if fin_tit_bol_id:
                    vals['fintit_boleto_ids'] = [(1, fin_tit_bol_id[0], {'state': 'I'})]
                obj_financeiro_titulo.write(cr, uid, titulo_id, vals, context=context)
            elif titulo.get('codigo_movimento') == 2: #Entrada confirmada
                obj_financeiro_titulo.write(cr, uid, titulo_id, {'state_banco':'C'}, context=context)
            elif titulo.get('codigo_movimento') == 3: #Entrada recusada
                obj_financeiro_titulo.write(cr, uid, titulo_id, {'state_banco':'R'}, context=context)
        if titulos: # somente as baixas
            datas_pagamento.sort()
            datas_credito.sort()
            res_baixa = obj_financeiro_utils.financ_baixar_titulo(
                cr, uid,
                futl_bxti_titulos=titulos,
                futl_bxti_data_pagamento=datas_pagamento[0],
                ftul_bxti_forma_pgto_id=forma_pagamento_id,
                futl_bxti_tesouraria_conta_id=dados_capa.get('conta_id'),
                futl_bxti_baixa_multipla=True if len(titulos) > 1 else False,
                futl_bxti_data_credito_debito=datas_credito[0],
                context=context
            )
            if not res_baixa:
                baixa_erro = obj_financeiro_utils.financ_get_erro()
                if isinstance(baixa_erro, basestring):
                    if 'ctx_util_baixa_lt' in baixa_erro:
                        baixa_erro = baixa_erro.replace('ctx_util_baixa_lt', '')
                        key = hash(baixa_erro)
                        if key in msg_erro_many_titulos:
                            msg_erro_many_titulos[key]['titulo_ids'].extend(titulo_ids)
                        else:
                            msg_erro_many_titulos[key] = {'titulo_ids': titulo_ids, 'msg': baixa_erro}
                    else:
                        if baixa_erro.strip().endswith(':'):
                            res_name_get = obj_financeiro_titulo.name_get(
                                cr, uid, titulo_ids, context) or [('', '')]
                            dict_name_get = dict(res_name_get)
                            titulo_name = ''
                            for x in dict_name_get:
                                titulo_name += '%s, ' % dict_name_get.get(x) or _('** Titulo nao encontrado **')
                        else:
                            titulo_name = ''
                        msg_erro += '%s %s' % (baixa_erro, titulo_name)
            else:
                self.write(cr, uid, ids[0], {'baixa_realizada': True}, context)
                obj_financeiro_titulo.write(cr, uid, map(lambda t: t.get('titulo_id'), titulos),
                                            {'fintit_baixa_arquivo_retorno': True}, context)
        if msg_erro_many_titulos:
            new_msg = self.generate_msg_erro_titulos(cr, uid,
                                                     dict_msg=msg_erro_many_titulos,
                                                     context=context)
            msg_erro += new_msg
        if msg_erro:
            raise osv.except_osv(_('Atencao'), msg_erro)
        self.write(cr, uid, ids, {'state': 'P'}, context)
        return True

    def _get_conta_corrente(self, cr, uid, convenio, nroBancoBC, context=None):
        """
        Identifica a conta corrente através do convenio e se necessário do banco
        Args:
            cr:
            uid:
            convenio: Numero do convenio
            nroBancoBC: Numero do banco na camara de compensação
            context:

        Returns: retorna duas informacoes: False,False  se não conseguiu ou (numero da conta corrente, banco_id)

        """
        context = context and context.copy() or {}
        obj_adm_banco = self.pool.get('adm.banco')
        obj_local_natureza = self.pool.get('financeiro.local.natureza')
        obj_local_cobranca = self.pool.get('financeiro.local.cobranca')
        banco_ids = obj_adm_banco.search(cr, uid, [('admbco_codigo_bc', '=', nroBancoBC)], context=context)
        local_cobranca_ids = obj_local_cobranca.search(cr, uid, [('finloc_banco_id', 'in', banco_ids)], context=context)
        local_natureza_ids = obj_local_natureza.search(cr, uid, [('finlon_convenio', '=', convenio),
                                                                 ('finlon_local_cobranca_id', 'in', local_cobranca_ids)],
                                                       context=context)
        if len(local_natureza_ids) > 0:
            brw_local_natureza = obj_local_natureza.browse(cr, uid, local_natureza_ids[0], context=context)
            return brw_local_natureza.finlon_local_cobranca_id.finloc_conta_id.nro_conta, \
                   brw_local_natureza.finlon_local_cobranca_id.finloc_conta_id.id
        return False, False

    def _custifica_tarifa(self, cr, uid, cobranca_magnetica_retorno_id, titulo_ids, conta_id,
                          context=None):
        """
        O metodo gera movimento na tesouraria e no gerencial (custifica) com o valor da tarifa
        cobrada pelo banco para o servico de cobranca magnetica. O valor da tarifa e destacado no
        arquivo de retorno
        @param cr: cursor no banco da dados
        @param uid: id da tabela res_users, identificador do usuario
        @param cobranca_magnetica_retorno_id: id da tabela transient
        wizard_cobranca_magnetica_retorno
        @param titulo_ids: lista de ids da tabela financeiro_titulo, identificadores dos titulos
        @param context: dicionario com valores contextuais
        @return: True, se os lancamentos na tesouraria e gerencial forem efetuados ou se nao houver
        lancamentos para realizar. False, ha lancamentos na tesouraria e gerencial para ser
        realizado, porem centro de custo e evento nao estao cadastrados
        """
        context = context and context.copy() or {}
        if not self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'financeiro.parametros',
                                 ['usar_centro_custos_e_eventos'], context=context).\
                                 get('usar_centro_custos_e_eventos'):
            return True

        obj_cobranca_magnetica_retorno_titulo = self.pool.get(
            'wizard.cobranca.magnetica.retorno.titulos')
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        obj_gerencial_movimento = self.pool.get('gerencial.movimento')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', ['finpar_centro_custo_tarifa_retorno_id',
                                    'finpar_evento_tarifa_retorno_id',
                                    'finpar_conta_tesouraria_avista_id'])
        centro_custo_id = financeiro_parametro_dados.get('finpar_centro_custo_tarifa_retorno_id')
        evento_id = financeiro_parametro_dados.get('finpar_evento_tarifa_retorno_id')
        retorno_titulo_ids = obj_cobranca_magnetica_retorno_titulo.search(
            cr, uid, [('cobranca_magnetica_retorno_id', '=', cobranca_magnetica_retorno_id),
                      ('titulo_id', 'in', titulo_ids),
                      ('valor_debito', '>', 0)], context=context)
        if not centro_custo_id or not evento_id and retorno_titulo_ids:
            return False
        retorno_titulo_dados = obj_cobranca_magnetica_retorno_titulo.read(
            cr, uid, retorno_titulo_ids,
            ['valor_debito', 'titulo_id', 'data_debito', 'codigo_movimento', 'documento'],
            context=context, load='_classic_write')
        # 6=Liquidado, 9=Baixado
        retorno_titulo_dados = filter(lambda r: r.get('codigo_movimento') in (6, 9),
                                      retorno_titulo_dados)
        gerencial_company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'gerencial.movimento', context=context)
        historico = 'Tarifa de cobranca na baixa de titulos'
        tarifa_movimento_unico = self.pool.get('financeiro.tesouraria.contas').read(
            cr, uid, conta_id, ['fintco_tarifa_movimento_unico'], context).get(
            'fintco_tarifa_movimento_unico')
        vals_tesouraria = {
            'state': 'A',
            'conta_id': conta_id,
            'historico': historico,
            'debito_credito': 'D',
        }
        context.update({'financ_baixar_titulo': True})
        if tarifa_movimento_unico and retorno_titulo_dados:
            data_debito = retorno_titulo_dados[0].get('data_debito')
            vals_tesouraria.update(
                {'data_movimento': data_debito, 'ano_competencia': data_debito[0:4],
                 'mes_competencia': data_debito[5:7],
                 'valor': sum(map(lambda r: r.get('valor_debito'), retorno_titulo_dados))})
            vals_tesouraria['historico'] += ': ' + ', '.join(map(lambda r: r.get('documento'),
                                                                 retorno_titulo_dados))
            tesouraria_movimento_id = obj_financeiro_tesouraria_movimento.create(
                cr, uid, vals_tesouraria, context)

        vals_gerencial = {
            'company_id': gerencial_company_id,
            'evento_id': evento_id,
            'centro_custo_id': centro_custo_id,
            'fingmo_historico': historico,
            'fingmo_operacao': 'S',
        }
        for x in retorno_titulo_dados:
            if not tarifa_movimento_unico:
                _data_debito = x.get('data_debito')
                vals_tesouraria.update({'valor': x.get('valor_debito'),
                                        'data_movimento': _data_debito,
                                        'ano_competencia': _data_debito[0:4],
                                        'mes_competencia': _data_debito[5:7]
                                        })
                tesouraria_movimento_id = obj_financeiro_tesouraria_movimento.create(
                    cr, uid, vals_tesouraria, context)
            titulo_id = x.get('titulo_id')
            brw_financeiro_titulo = obj_financeiro_titulo.browse(cr, uid, titulo_id)
            pessoa_id = brw_financeiro_titulo.pessoa_id.id or False
            data_emissao = brw_financeiro_titulo.data_emissao or False
            vals_gerencial.update({'valor': x.get('valor_debito'),
                                   'pessoa_id': pessoa_id,
                                   'tesouraria_movimento_id': tesouraria_movimento_id,
                                   'fingmo_data': data_emissao})
            obj_gerencial_movimento.create(cr, uid, vals_gerencial, context)
        return True

    def generate_msg_erro_titulos(self, cr, uid, dict_msg, context=None):
        context = context and context.copy() or {}
        msg = u''
        obj_finaceiro_titulo = self.pool.get('financeiro.titulo')
        for i in dict_msg:
            titulo_ids = dict_msg[i]['titulo_ids']
            titulo_msg = dict_msg[i]['msg']

            res_name_get = obj_finaceiro_titulo.name_get(cr, uid, titulo_ids, context) or [('', '')]
            dict_name_get = dict(res_name_get)
            titulo_names = ''
            for t in titulo_ids:
                titulo_names += '%s, ' % (dict_name_get.get(t) or _('** Titulo nao encontrado **'))
            msg += '%s%s' % (titulo_msg, titulo_names)
        return msg

    def button_carregar_retorno(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_wizard_cobranca_magnetica_retorno_titulos = self.pool.get('wizard.cobranca.magnetica.retorno.titulos')
        obj_local_natureza = self.pool.get('financeiro.local.natureza')
        obj_tesouraria_contas = self.pool.get('financeiro.tesouraria.contas')
        obj_adm_utils = self.pool.get('adm.utils')
        dados_capa = self.read(cr, uid, ids[0], [], context=context)
        dados_arquivo = base64.b64decode(dados_capa.get('arquivo_retorno'))
        try:
            dados_arquivo = obj_adm_utils.admutl_remover_acentuacao(cr, uid, dados_arquivo)
        except:
            dados_arquivo = obj_adm_utils.admutl_remove_caracter(
                dados_arquivo, outros=['\r\n', '\n'], substituto=' ')
        if '\r\n' in dados_arquivo:
            arquivo = dados_arquivo.split('\r\n')
        else:
            arquivo = dados_arquivo.split('\n')
        _tamanhoRegistro = len(arquivo[0])
        if _tamanhoRegistro not in (240, 400):
            raise osv.except_osv(_('Atencao'),
                                 _('Layout desconhecido!  tamanho: %s' % _tamanhoRegistro))

        identificador_banco = arquivo[0][:3]
        try:
            if identificador_banco.isdigit():
                identificador_banco = int(identificador_banco)
            else:
                identificador_banco = int(arquivo[0][76:79])
        except:
            raise osv.except_osv(_('Atencao'), _(
                'Nao consegui identificar o banco no arquivo!\nDado lido: %s' % str(
                    identificador_banco)))

        # Busca o diretorio do addons no computador
        openerp.tools.config.parse_config(sys.argv[1:])
        config = openerp.tools.config
        addons_path = config['addons_path']

        instance_arquivo = self._carregar_layout(cr, uid, identificador_banco, _tamanhoRegistro, context=context)
        if not instance_arquivo:
            raise osv.except_osv(_('Atencao'), _('Layout desconhecido!\n Banco: %s' % str(identificador_banco)))
        titulos_arquivo_retorno = instance_arquivo.carregar_retorno(arquivo)
        try:
            convenio = str(instance_arquivo._lotes[0].header.codigo_cedente)
        except:
            convenio = False
        try:
            conta_corrente_empresa = str(instance_arquivo._lotes[0].header.cedente_conta)
            args = [('nro_conta', '=', conta_corrente_empresa)]
            conta_ids = obj_tesouraria_contas.search(cr, uid, args, context=context)
            if conta_ids:
                conta_id = conta_ids[0]
            else:
                raise osv.except_osv(_('Atencao'),
                    _('Nao encontrei uma conta de tesouraria para este conta!\n Conta: %s' % conta_corrente_empresa))
        except:
            conta_corrente_empresa = False
        if not conta_corrente_empresa and convenio:
               conta_corrente_empresa, conta_id = self._get_conta_corrente(cr, uid, convenio, identificador_banco,
                                                                           context=context)
        if not conta_corrente_empresa:
            raise osv.except_osv(_('Atencao'),
                 _('Nao encontrei uma conta de tesouraria para este banco/convenio!\n Banco: %s Convenio: %s' %
                   (str(identificador_banco), convenio)))

        numero_retorno = str(
            instance_arquivo._lotes[0].header.controlecob_numero).rjust(6, '0')
        data_retorno = time.strftime('%Y-%m-%d', time.strptime(
            formataString(str(instance_arquivo._lotes[0].header.controlecob_data_gravacao),
                          8, '0', 'E'), '%d%m%Y'))
        ids_titulos = obj_wizard_cobranca_magnetica_retorno_titulos.search(cr, uid, [
            ('cobranca_magnetica_retorno_id', '=', ids[0])
        ], context=context)
        if ids_titulos:
            obj_wizard_cobranca_magnetica_retorno_titulos.unlink(cr, uid, ids_titulos)
        if titulos_arquivo_retorno:
            segmento_t = instance_arquivo.detailT
            segmento_u = instance_arquivo.detailU
            for titulo in titulos_arquivo_retorno:
                _nosso_numero = titulo.get('nosso_numero')
                if identificador_banco == 756 and len(_nosso_numero) == 15: # Ajustar porque o nosso numero enviado é acrescido do digito, pardela, carteira, etc
                    try:
                        _nosso_numero = str(int(_nosso_numero[:9])) # se não conseguir converter deixa como estava
                    except:
                        pass
                data_vencimento = time.strftime(
                    '%Y-%m-%d', time.strptime(titulo.get('vencimento_titulo'), '%d%m%Y'))
                browse_financeiro_titulo = self._localizar_titulo(
                    cr, uid, _nosso_numero, titulo.get('numero_documento'), identificador_banco,
                    data_vencimento, context=None)
                if browse_financeiro_titulo:
                    vals = {
                        'pessoa_id': browse_financeiro_titulo.pessoa_id.id,
                        'documento': browse_financeiro_titulo.documento or '',
                        'parcela': browse_financeiro_titulo.parcela or False,
                        'data_emissao': browse_financeiro_titulo.data_emissao or False,
                        'vr_original': browse_financeiro_titulo.vr_original or 0.0,
                        'data_prorrogacao': browse_financeiro_titulo.data_prorrogacao or False,
                        'titulo_id': browse_financeiro_titulo.id,
                        'tipo_doc': browse_financeiro_titulo.doc_tipo_id.sigla
                    }
                    args = [('finlon_local_cobranca_id', '=', browse_financeiro_titulo.local_cobranca_id.id),
                            ('finlon_natureza_cobranca_id', '=', browse_financeiro_titulo.natureza_cobranca_id.id)]
                    local_natureza_ids = obj_local_natureza.search(cr, uid, args, context=context)
                    if local_natureza_ids:
                        vals['local_natureza_id'] = local_natureza_ids[0]
                        brw_local_natureza = obj_local_natureza.browse(cr, uid, local_natureza_ids[0], context=context)
                        vals['local_cobranca_id'] = brw_local_natureza.finlon_local_cobranca_id.id
                        vals['natureza_id'] = brw_local_natureza.finlon_natureza_cobranca_id.id
                        vals['prazo_liberacao'] = brw_local_natureza.finlon_prazo_liberacao
                else:
                    vencimento = titulo.get('vencimento_titulo') or False
                    if vencimento == '00000000':
                        vencimento = str(time.localtime().tm_mday) + str(time.localtime().tm_mon) + str(time.localtime().tm_year)
                    if vencimento:
                        vencimento = '%s-%s-%s' % (
                            vencimento[4:8], vencimento[2:4], vencimento[:2])
                    vals = {
                        'documento': titulo.get('numero_documento') or '',
                        'vr_original': titulo.get('valor_titulo') or 0.0,
                        'data_prorrogacao': vencimento,
                    }
                vals['codigo_movimento'] = titulo.get('codigo_movimento')
                vals['cobranca_magnetica_retorno_id'] = ids[0]
                _pagto = titulo.get('data_pagamento')
                if int(_pagto):
                    _pagto = '%s-%s-%s' % (
                        _pagto[4:8], _pagto[2:4], _pagto[:2])
                    vals['data_pagamento'] = _pagto
                var_t = segmento_t[titulos_arquivo_retorno.index(titulo)]
                var_u = segmento_u[titulos_arquivo_retorno.index(titulo)]
                try:
                    nosso_numero_modalidade = str(var_t.nosso_numero_modalidade) + '/'
                except:
                    nosso_numero_modalidade = ''
                nosso_numero_banco = nosso_numero_modalidade + str(
                    var_t.nosso_numero).rjust(15, '0')
                try:
                    banco_cobrador = str(var_t.banco_cobrador)
                except:
                    banco_cobrador = '0'
                try:
                    cedente_agencia_dv = '-' + str(var_t.cedente_agencia_dv or 0)
                except:
                    cedente_agencia_dv = ''
                if var_u.data_credito:
                    data_credito = time.strftime('%Y-%m-%d', time.strptime(
                        formataString(str(var_u.data_credito), 8, '0', 'E'), '%d%m%Y'))
                    vals['data_credito'] = data_credito
                try:
                    data_debito_ocorrencia = var_u.debito_data
                except:
                    data_debito_ocorrencia = var_u.data_ocorrencia
                if data_debito_ocorrencia:
                    data_debito_ocorrencia = time.strftime('%Y-%m-%d', time.strptime(
                        formataString(str(data_debito_ocorrencia), 8, '0', 'E'), '%d%m%Y'))
                    vals['data_debito'] = data_debito_ocorrencia
                vals_espelho_retorno = {
                    'numero_banco': banco_cobrador.rjust(3, '0'),
                    'numero_agencia': str(var_t.agencia_cobradora).rjust(5, '0') +
                    cedente_agencia_dv,
                    'nosso_numero_banco': nosso_numero_banco,
                    'data_ocorrencia': time.strftime('%Y-%m-%d', time.strptime(
                        formataString(str(var_u.data_ocorrencia), 8, '0', 'E'), '%d%m%Y')),
                    'valor_pagamento': float(var_u.titulo_pago),
                    'valor_credito': float(var_u.titulo_liquido),
                    'valor_debito': float(var_t.valor_tarifas),
                }
                vals.update(vals_espelho_retorno)

                obj_wizard_cobranca_magnetica_retorno_titulos.create(cr, uid, vals, context)
        else:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao ha titulos para o a arquivo selecionado'))
        addons_path += '/financeiro_febraban/report/logos/%s.jpg' % str(identificador_banco)
        try:
            logo = open(addons_path, 'r').read()
        except:
            raise osv.except_osv(_('Atencao'),
                                 _('Arquivo nao encontrado %s' % addons_path))
        logo = base64.encodestring(logo)
        vals_cobranca_magnetica_retorno = {
            'state': 'C',
            'conta_corrente_empresa': conta_corrente_empresa,
            'banco_bc': identificador_banco,
            'conta_id': conta_id,
            'numero_retorno': numero_retorno,
            'data_retorno': data_retorno,
            'logo_banco': logo
        }
        self.write(cr, uid, ids, vals_cobranca_magnetica_retorno, context)
        return True

    def button_print_espelho_retorno(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {'type': 'ir.actions.report.xml',
                'report_name': 'febraban_espelho_retorno',
                'datas': {'ids': ids},
                }


class wizard_cobranca_magnetica_retorno_titulos(osv.TransientModel):
    _name = "wizard.cobranca.magnetica.retorno.titulos"
    _columns = {
        'cobranca_magnetica_retorno_id': fields.many2one('wizard.cobranca.magnetica.retorno',
                                                         'Cobranca magnetica retorno'),
        'local_natureza_id': fields.many2one('financeiro.local.natureza', 'Tipo cobranca'),
        'local_cobranca_id': fields.many2one('financeiro.local.cobranca', 'Local'),
        'natureza_id': fields.many2one('financeiro.natureza.cobranca', 'Natureza'),
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente'),
        'documento': fields.char('Documento', size=30),
        'parcela': fields.integer('Parc.'),
        'data_emissao': fields.date('Emissao'),
        'vr_original': fields.float('Valor', digits=(15, 2)),
        'data_prorrogacao': fields.date('Vencimento'),
        'data_pagamento': fields.date('Dt.Pagto'),
        'titulo_id': fields.many2one('financeiro.titulo', 'Titulo'),
        'codigo_movimento': fields.selection(
            [(2, '02-Entrada confirmada'), (3, '03-Entrada recusada'),
             (4, '04-Transferencia de Carteira/Entrada'), (6, '06-Liquidado'),
             (9, '09-Baixa'), (29, '29-Ocorrencias do sacado'),
             (48, '48-Confirm. instr. transf. carteira/modalidade cobr.')], 'Situacao', size=2),
        'numero_banco': fields.char('Banco', size=3),
        'numero_agencia': fields.char('Agencia', size=7),
        'nosso_numero_banco': fields.char('Nosso numero', size=20),
        'data_ocorrencia': fields.date('Data ocorrencia'),
        'valor_pagamento': fields.float('Valor pagamento', digits=(13, 2)),
        'data_credito': fields.date('Data credito'),
        'valor_credito': fields.float('Valor credito', digits=(13, 2)),
        'data_debito': fields.date('Data debito'),
        'prazo_liberacao': fields.integer('Prazo liberacao'),
        'valor_debito': fields.float('Valor debito', digits=(13, 2)),
        'tipo_doc': fields.char('Tp.Doc.', size=4),
    }
