# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import openerp
import sys
import base64
from openerp.tools.translate import _
from SigUtils.cnab.tipos import ArquivoDDA
from SigUtils.cnab.bancos import itau
import time


class wizard_debito_direto_autorizado(osv.TransientModel):

    _name = 'wizard.debito.direto.autorizado'
    _columns = {
        'wizdda_arquivo': fields.binary('Arquivo DDA', required=True),
        'wizdda_logo': fields.binary('Logomarca', readonly=True),
        'wizdda_importado': fields.boolean('Arquivo importado'),
    }
    _defaults = {
        'wizdda_importado': lambda *a: False,
    }

    def wizdda_button_carregar_retorno(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Carregar retorno'. Realiza a leitura do arquivo do DDA e faz a inclusao dos
        dados na tabela financeiro_febraban_dda_titulo.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id databela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_res_company = self.pool.get('res.company')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_financeiro_febraban_dda_titulo = self.pool.get('financeiro.febraban.dda.titulo')
        dda_dados = self.read(cr, uid, ids[0] if isinstance(ids, (list, tuple)) else ids, [],
                              context=context)
        dados_arquivo = base64.b64decode(dda_dados.get('wizdda_arquivo'))
        try:
            dados_arquivo = obj_adm_utils.admutl_remover_acentuacao(cr, uid, dados_arquivo)
        except:
            dados_arquivo = obj_adm_utils.admutl_remove_caracter(
                dados_arquivo, outros=['\r\n', '\n'], substituto=' ')
        if '\r\n' in dados_arquivo:
            arquivo = dados_arquivo.split('\r\n')
        else:
            arquivo = dados_arquivo.split('\n')
        _tamanhoRegistro = len(arquivo[0])
        if _tamanhoRegistro != 240:
            raise osv.except_osv(_('Atencao'),
                                 _('Layout desconhecido!  tamanho: %s' % _tamanhoRegistro))
        identificador_banco = int(arquivo[0][:3])

        instance_arquivo = self.wizdda_carregar_layout(
            cr, uid, identificador_banco, _tamanhoRegistro, context=context)
        if not instance_arquivo:
            raise osv.except_osv(_('Atencao'), _('Layout desconhecido!\n Banco: %s' % str(
                identificador_banco)))
        titulos_arquivo_retorno = instance_arquivo.carregar_retorno(arquivo)
        dados_header = titulos_arquivo_retorno.get('header')
        dados_segmentos = titulos_arquivo_retorno.get('segmentos')
        context.update({'sig_force_company': True})
        dda_titulo_ids = []
        for dado in dados_segmentos:
            if len(str(dado)) > 11:
                cpf_cnpj = str(dado).zfill(14)
            else:
                cpf_cnpj = str(dado).zfill(11)
            pessoa_ids = obj_adm_pessoa.search(cr, uid, [('cpf_cnpj', '=', cpf_cnpj)],
                                               context=context)
            company_ids = obj_res_company.search(cr, uid, [('rescon_pessoa_id', 'in', pessoa_ids)],
                                                 context=context)
            if not company_ids:
                raise osv.except_osv(_('Atencao'),
                                     _('Não ha empresa cadastrada para o CPF/CNPJ: %s' % cpf_cnpj))
            for segmento in dados_segmentos.get(dado).get('G'):
                if len(str(dado)) > 11:
                    cpf_cnpj_fornecedor = str(segmento.get('inscricao_numero') or '').zfill(14)
                else:
                    cpf_cnpj_fornecedor = str(segmento.get('inscricao_numero') or '').zfill(11)
                codigo_barras = str(segmento.get('codigo_banco')).zfill(3) + \
                    str(segmento.get('codigo_moeda')) + \
                    str(segmento.get('dv_codigo_barras')) + \
                    str(segmento.get('fator_vencimento')).zfill(4) + \
                    str(segmento.get('valor_codigo_barras')).replace('.', '').zfill(10) + \
                    str(segmento.get('codigo_barras')).zfill(25)
                # ex. linha digitavel correta = '34194690100000514001098893875851426143606000'
                vals = {
                    'company_id': company_ids[0],
                    'state': 'P',
                    'fifdda_nome_fornecedor': segmento.get('cedente_nome') or '',
                    'fifdda_emissao': time.strftime('%Y-%m-%d', time.strptime(str(segmento.get(
                        'emissao_titulo')), '%d%m%Y')),
                    'fifdda_vencimento': time.strftime('%Y-%m-%d', time.strptime(str(segmento.get(
                        'data_vencimento')), '%d%m%Y')),
                    'fifdda_codigo_barra': codigo_barras,
                    'fifdda_titulo_id': False,
                    'fifdda_valor_titulo': float(segmento.get('valor_titulo')) or 0.0,
                    'fifdda_cpf_cnpj_fornecedor': cpf_cnpj_fornecedor,
                    'fifdda_numero_documento': segmento.get('numero_documento') or '',
                    'fifdda_codigo_banco': int(segmento.get('codigo_banco')),
                    'fifdda_nome_banco': dados_header.get('nome_do_banco') or '',
                }
                dda_titulo_ids.append(obj_financeiro_febraban_dda_titulo.create(cr, uid, vals,
                                                                                context))
        del context['sig_force_company']
        if dda_titulo_ids:
            obj_financeiro_febraban_dda_titulo.fifdda_atualiza_titulo(cr, uid, dda_titulo_ids,
                                                                      context)
            self.write(cr, uid, ids, {'wizdda_importado': True}, context)
        return True

    def wizdda_onchange_arquivo(self, cr, uid, ids, arquivo, context=None):
        context = context and context.copy() or {}
        res = {'value': {'wizdda_logo': False}}
        if not arquivo:
            return res
        dados_arquivo = base64.b64decode(arquivo)
        identificador_banco = dados_arquivo[:3]
        try:
            identificador_banco = int(identificador_banco)
        except:
            res['value'].update({'wizdda_arquivo': False})
            res['warning'] = {
                'title': _('Atencao'),
                'message': _('Nao foi possivel identificar o banco no arquivo!\nDado lido: %s') %
                str(identificador_banco)}
            return res
        # Busca o diretorio do addons no computador
        openerp.tools.config.parse_config(sys.argv[1:])
        config = openerp.tools.config
        addons_path = config['addons_path']
        addons_path += '/financeiro_febraban/report/logos/%s.jpg' % str(identificador_banco)
        try:
            logo = open(addons_path, 'r').read()
        except:
            res['warning'] = {
                'title': _('Atencao'),
                'message': _('Arquivo nao encontrado: %s') % addons_path}
            return res
        logo = base64.encodestring(logo)
        res['value']['wizdda_logo'] = logo
        return res

    def wizdda_carregar_layout(self, cr, uid, nroBancoBC, tamanhoRegistro, context=None):
        try:
            if nroBancoBC == 341:
                instance_arquivo = ArquivoDDA(itau, tamanhoRegistro, **{})
            else:
                return False
        except:
            return False
        return instance_arquivo
