# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class fiscal_cest(osv.Model):
    """
    Tabela CEST-Codigo Especificador da Substituicao Tributaria
    prefixo: fiscest_
    """
    _name = 'fiscal.cest'
    _inherit = 'fiscal.cest'

    def create(self, cr, user, vals, context=None):
        context = context and context.copy() or {}
        self.fiscest_valida_codigo(vals)
        return super(fiscal_cest, self).create(cr, user, vals, context)

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        self.fiscest_valida_codigo(vals)
        return super(fiscal_cest, self).write(cr, user, ids, vals, context)

    def fiscest_valida_codigo(self, vals):
        if 'fiscest_codigo' in vals and not vals.get('fiscest_codigo').isdigit():
            raise osv.except_osv(_('Atencao!'),
                                 _('Codigo do CEST deve conter apenas numeros sem espaco!'))

    def fiscest_gerar_codigo(self, cr, uid, context=None):
        """
        Método chamado pelo executor de funcoes para gerar o codigo cest no cadastro de ncm
        :param cr:
        :param uid:
        :param ids:
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        cest_ids = self.search(cr, uid, [], context=context)
        for cest_browse in self.browse(cr, uid, cest_ids, context=context):
            if not cest_browse.fiscest_ncms:
                continue
            ncms = str(cest_browse.fiscest_ncms).split(',')
            if not isinstance(ncms, (list or tuple)):
                ncms = list(ncms)
            if len(ncms) == 0:
                continue
            elif len(ncms) == 1:
                sql = "select id from fiscal_ncm where codigo = '%s'" % ncms[0]
            else:
                sql = "select id from fiscal_ncm where codigo in %s" % str(tuple(ncms))
            cr.execute(sql)
            ncm_ids = map(lambda x: x[0], cr.fetchall())
            vals = {'fisnc_cest_id': cest_browse.id}
            self.pool.get('fiscal.ncm').write(cr, uid, ncm_ids, vals, context=context)
        return True

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        for brw_cest in self.browse(cr, uid, ids, context=context):
            codigo = str(brw_cest.fiscest_codigo or '')
            name = brw_cest.name or '*** cest sem Descricao ***'
            if context.get('retorno_sem_codigo'):
                codigo = ''
            if context.get('fiscest_so_codigo'):
                result = codigo
            else:
                result = "%s%s%s" % (codigo, ' - ' if codigo else '', name)
            res.append((brw_cest.id, result))
        return res

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        """
        Sobreescricao da classe BaseModel. O metodo busca os registros dessa tabela baseado na
        descricao e operador recebidos em 'name' e 'operator', respectivamente.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param name: string com a descricao da pesquisa.
        :type name: str
        :param args: argumentos de pesquisa.
        :type args: list
        :param operator: descricao do operador de pesquisa.
        :type operator: str
        :param context: dicionario com valores contextuais.
        :type context: dict
        :param limit: quantidade maxima de registros que devem ser encontrados.
        :type limit: int
        :return: lista de tuplas dos registros encontrados.
        :rtype: list
        """
        context = context and context.copy() or {}
        args = []
        if name:
            if name.isdigit():
                args.append(('fiscest_codigo', operator, name))
            else:
                args.append(('name', operator, name))
        ids = self.search(cr, uid, args, context=context, limit=limit)
        return self.name_get(cr, uid, ids, context)

