# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_cfop(osv.Model):
    """
    Tabela de CFOPs
    """

    _name = 'fiscal.cfop'
    _inherit = 'fiscal.cfop'

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        """
        Metodo utilizado para localizar o nome do CFOP pelo codigo
        se houver.
        """
        context = context and context.copy() or {}
        ids = []
        try:
            name = int(name)
        except:
            pass
        if isinstance(name, int):
            ids = self.search(cr, uid, [('codigo', '=', str(name))] + args, limit=limit, context=context)
        if not ids:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if limit == 160: # ha um limit de 160 sendo passado em algum lugar nao identificado
            limit = None
        if context.get('produto_id'):
            if self.pool.get('estoque.produto').browse(cr, uid, context['produto_id']).tipo == 'S':
                args.append(('produto_servico_tipo', 'in', ('S', 'F', 'M')))
        if context.get('nf_servico'):
            args.append(('produto_servico_tipo', '=', 'S'))
        res = super(fiscal_cfop, self).search(cr, uid, args, offset, limit, order, context, count)

        return res


    def create(self, cr, uid, vals, context=None):

        context = context and context.copy() or {}
            # Bloco inserido na versao 3G1, para evitar erro na insercao de dados do arquivo local.xml
        # em bases que já tem local cadastrado.
        # TODO: Pode ser removido apos a versao  4A1
        if context.get('install_mode') and context['install_mode']:
            args = [('codigo', '=', vals['codigo'])]
            _id = self.search(cr, uid, args, context=context)
            if _id:
                return _id[0]
            # Fim do Bloco #
        result = super(fiscal_cfop, self).create(cr, uid, vals, context)
        return result

    def correlacao_cfop(self, cr, user, cfop, natureza, context=None):
        """
        Sugere a troca de CFOP.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param cfop: o codigo do CFOP
        :param natureza: o ID da natureza da operacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type cfop: str
        :type natureza: long
        :return: o ID do CFOP da correlacao entrada/saida.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        # Retorna o CFOP da natureza se CFOP nao for de bonificacao
        if cfop[-3:] != '910':
            cfop = self.pool.get('fiscal.cfop.natureza').browse(
                cr, user, natureza, context).cfop_id.codigo

        # Ajusta o CFOP do item entrada/saida
        if cfop.startswith('5'):  # Entrada dentro do estado/UF
            cfop = cfop.replace('5', '1', 1)
        elif cfop.startswith('6'):  # Entrada fora do estado/UF
            cfop = cfop.replace('6', '2', 1)

        return cfop

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
