# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.addons.fiscal.model.funcoes import fiscal_codigo_natureza_credito


class fiscal_cfop_natureza(osv.Model):
    """
    Tabela de Naturezas do CFOP

    """
    _name = 'fiscal.cfop.natureza'
    _inherit = 'fiscal.cfop.natureza'

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        ids = []
        try:
            name = int(name)
        except:
            pass
        args_cfop = []
        if isinstance(name, int):
            args_cfop.append(('codigo', 'ilike', str(name)))
        if args_cfop:
            cfop_ids = self.pool.get('fiscal.cfop').search(cr, uid, args_cfop, limit=limit, context=context)
            if cfop_ids:
                ids = self.search(cr, uid, [('cfop_id', 'in', cfop_ids)] + args, limit=limit, context=context)
        if not ids:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        fields_to_read = ['name']
        if not context.get('cfop_natureza_sem_codigo'):
            fields_to_read.append('cfop_id')
        cfop_natureza_dados = self.read(cr, uid, ids, fields_to_read, context=context)
        for cfop_natureza in cfop_natureza_dados:
            id = cfop_natureza.get('id')
            cfop = (cfop_natureza.get('cfop_id') and cfop_natureza['cfop_id'][1][:4]) or ''
            natureza = cfop_natureza.get('name') or _('*** Natureza sem Descricao ***')
            result = "%s%s%s" % (cfop, ' - ' if cfop else '', natureza)
            res.append((id, result))
        return res

    def onchange_cfop(self, cr, uid, ids, cfop_id, context=None):
        context = context and context.copy() or {}
        vals = {}
        tipo = False
        fisnat_codigo_natureza_credito = False
        if cfop_id:
            dados_fiscal_cfop = self.pool.get('fiscal.cfop').read(cr, uid, cfop_id, ['tipo','codigo'], context=context)
            tipo = dados_fiscal_cfop['tipo']
            if tipo == 'E':
                #inserindo codigo da natureza de crédito
                codigo_cfop = dados_fiscal_cfop['codigo']
                for x in fiscal_codigo_natureza_credito:
                    if codigo_cfop in fiscal_codigo_natureza_credito[x]:
                        fisnat_codigo_natureza_credito = x

        vals.update({'cfop_id': cfop_id, 'tipo': tipo,
                         'fisnat_codigo_natureza_credito':fisnat_codigo_natureza_credito})
        return {'value':vals }

    def onchange_name(self, cr, uid, ids, name, context=None):
        context = context and context.copy() or {}
        vals = {}
        if not ids and name:
            vals.update({'fisnat_natureza_fiscal':name})
        return {'value': vals}

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}

        tipo = self.pool.get('fiscal.cfop').read(cr, uid, vals['cfop_id'], ['tipo'], context=context)['tipo']
        vals['tipo'] = tipo
        if not (context.get('install_mode') and context['install_mode']):
            if vals.get('active'):
                encargos_novos = []
                if vals.get('natureza_encargos_ids'):
                    for i in vals['natureza_encargos_ids']:
                        if isinstance(i[2], dict):
                            encargos_novos.append(i[2]['encargo_id'])
                encargos_faltantes = self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(cr, uid,
                                                                                                         encargos_novos, context=context)
                msg = ''
                for m in encargos_faltantes:
                    msg += '* ' + m[1] + '\n'
                if msg:
                    raise osv.except_osv(_('Atencao'),
                                         _('Os seguintes encargos obrigatorios para esta empresa devem ser inseridos:\n') + msg)


                operacao_id = vals.get('fisnat_operacao_id')
                if operacao_id:
                    obj_estoque_natureza_operacao = self.pool.get('estoque.natureza.operacao')
                    tipo_operacao = obj_estoque_natureza_operacao.browse(cr,uid,operacao_id).estnto_tipo_operacao
                    if tipo != tipo_operacao:
                        raise osv.except_osv(_('Atencao'), _('O tipo da operacao difere do tipo da natureza,'
                                                     'corrija por favor.'))

        res = super(fiscal_cfop_natureza, self).create(cr, uid, vals, context=context)
        self.fisnat_verifica_pis_cofins_mesmo_cst(cr, uid, [res], context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}

        if 'cfop_id' in vals:
            tipo = self.pool.get('fiscal.cfop').read(cr, uid, vals['cfop_id'], ['tipo'], context=context)['tipo']
            vals['tipo'] = tipo
            #verificacao de encargo obrigatorio de acordo com o encargo emprea
        if 'active' in vals:
            ativo = vals.get('active')
        else:
            ativo = self.browse(cr, uid, ids[0]).active
        if ativo:
            encargos_novos = []
            if 'natureza_encargos_ids' in vals:
                obj_cfop_natureza_encargos = self.pool.get('fiscal.cfop.natureza.encargos')
                for x in vals.get('natureza_encargos_ids'):
                    if isinstance(x[2], dict):
                        if x[2].get('encargo_id'):
                            encargos_novos.append(x[2]['encargo_id'])
                    elif isinstance(x[1], (int, long)) and not x[0] == 2:#verifica editados e excluidos
                        encargo_id = obj_cfop_natureza_encargos.browse(cr, uid, x[1]).encargo_id.id
                        encargos_novos.append(encargo_id)

            if 'fisnat_operacao_id' in vals:
                if 'tipo' in vals:
                    tipo = vals['tipo']
                else:
                    tipo = self.browse(cr,uid,ids[0]).tipo
                operacao_id = vals.get('fisnat_operacao_id')
                if operacao_id:
                        obj_estoque_natureza_operacao = self.pool.get('estoque.natureza.operacao')
                        tipo_operacao = obj_estoque_natureza_operacao.browse(cr,uid,operacao_id).estnto_tipo_operacao
                        if tipo != tipo_operacao:
                            raise osv.except_osv(_('Atencao'), _('O tipo da operacao difere do tipo da natureza,'
                                                     'corrija por favor.'))

            encargos_ids = self.browse(cr, uid, ids[0]).natureza_encargos_ids
            for i in encargos_ids:
                encargos_novos.append(i.encargo_id.id)

            encargos_faltantes = self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(cr, uid,
                                                                                                     encargos_novos, context=context)
            msg = ''
            for m in encargos_faltantes:
                msg += '* ' + m[1] + '\n'
            if msg:
                raise osv.except_osv(_('Atencao'),
                                     _('Os seguintes encargos obrigatorios para esta empresa devem ser inseridos:\n') + msg)

        res = super(fiscal_cfop_natureza, self).write(cr, uid, ids, vals, context=context)
        self.fisnat_verifica_pis_cofins_mesmo_cst(cr, uid, ids, context)
        return res

    def fisnat_verifica_pis_cofins_mesmo_cst(self, cr, uid, ids, context=None):
        """
        O método verifica se o cst das tributações de PIS e COFINS, de sobreposição, são
        compatíveis. Caso não sejam, deve abortar o processo de criação ou atualização da natureza,
        de modo a informar o usuário a situação de impecílio.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuário
        :param ids: lista de ids da tabela fiscal_cfop_natureza
        :param context: dicionário com valores contextuais
        :return: booleano, com True caso os cst's sejam compatíveis, senão, um raise com a
        respectiva mensagem
        """
        context = context and context.copy() or {}
        brw_cfop_natureza = self.browse(cr, uid, ids, context)
        for cfop_natureza in brw_cfop_natureza:
            natureza_encargos_ids = cfop_natureza.natureza_encargos_ids
            pis_cofins_natureza_encargos_ids = filter(
                lambda i: i.encargo_id.sigla in ('PIS', 'COFINS'), natureza_encargos_ids)
            sobrepor_tributacao_ids = map(lambda i: i.sobrepor_tributacao_id or False,
                                          pis_cofins_natureza_encargos_ids)
            if len(sobrepor_tributacao_ids) > 1 and all(sobrepor_tributacao_ids):
                codigos_cst = list(set(map(lambda i: i.cst_id.codigo, sobrepor_tributacao_ids)))
                if len(codigos_cst) > 1:
                    raise osv.except_osv(_('Atencao'),
                                         _('A sobreposicao de tributacao possui PIS/COFINS '
                                         'cadastrados com CST diferentes'))
        return True

    # noinspection PyMethodMayBeStatic
    def fisnat_onchange_icms_desonerado_motivo(self, cr, user, ids, motivo, context):
        """
        Ajusta estado do ICMS desonerado.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param motivo: o motivo da desoneracao (se houver)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type motivo: str
        :type context: dict
        :return: valor ajustado do estado do ICMS desonerado
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        result = {'value': {'fisnat_icms_desonerado': bool(motivo)}}
        if not ids:
            return {}
        elif motivo:
            # Desoneracao de ICMS. Nao aplicada se empresa for Simples Nacional
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, user, context=context)
            pessoa_id = self.pool.get('res.company').read(
                cr, user, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                'rescon_pessoa_id')
            if self.pool.get('adm.pessoa').read(
                    cr, user, pessoa_id, ['admpes_jur_regime_tributario'], context).get(
                'admpes_jur_regime_tributario') == 'S':
                result.update({'warning': {
                    'title': _('Aviso.'),
                    'message': _('Desoneracao de ICMS nao pode ocorrer para empresa cadastrada no '
                                 'Simples Nacional.')},
                    'value': {'fisnat_icms_desonerado_motivo': '',
                              'fisnat_codigo_ajuste_desonerado': '',
                              'fisnat_icms_desonerado': False}
                     }
                )

        return result

    # noinspection PyMethodMayBeStatic
    def fisnat_onchange_icms_desonerado(self, cr, user, ids, desonerado, context):
        """
        Ajusta estado do ICMS desonerado.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param desonerado: ICMS desonerado ou nao
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type desonerado: bool
        :type context: dict
        :return: valor ajustado do motivo do ICMS desonerado
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        result = {'value': {}}
        if not ids:
            return {}
        elif desonerado:
            # Desoneracao de ICMS. Nao aplicada se empresa for Simples Nacional
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, user, context=context)
            pessoa_id = self.pool.get('res.company').read(
                cr, user, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                'rescon_pessoa_id')
            if self.pool.get('adm.pessoa').read(
                    cr, user, pessoa_id, ['admpes_jur_regime_tributario'], context).get(
                'admpes_jur_regime_tributario') == 'S':
                result['value']['fisnat_icms_desonerado'] = False
                result.update({'warning': {
                    'title': _('Aviso.'),
                    'message': _('Desoneracao de ICMS nao pode ocorrer para empresa cadastrada no '
                                 'Simples Nacional.')}
                })

        return result

    def copy(self, cr, uid, cfop_natureza_id, default=None, context=None):
        """
        Sobrescricao do metodo copy da BaseModel.
        @param cr: cursor no banco de dados
        @type cr: object
        @param uid: id da tabela res_users, identificador do usuario
        @type uid: int
        @param cfop_natureza_id: id do registro dessa tabela a ser copiado
        @type cfop_natureza_id: int
        @param default: dados dos atributos previmante definidos para a copia
        @type default: None
        @param context: dados contextuais
        @type context: None
        @return: id do novo registro criado
        @rtype: int
        """
        context = context and context.copy() or {}
        if not default:
            default = {}
        default.update({'estoque_movimento_itens_ids': []})
        res = super(fiscal_cfop_natureza, self).copy(cr, uid, cfop_natureza_id, default, context)
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
