# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class fiscal_cst(osv.Model):
    """
    Tabela de CSTs
    """

    _name = 'fiscal.cst'
    _inherit = 'fiscal.cst'

    def name_get(self, cr, uid, ids, context=None):
        res = []
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['name', 'codigo'], context=context)

        for record in reads:
            if context.get('fiscst_so_codigo'):
                res.append((record['id'], record['codigo']))
            else:
                res.append((record['id'], record['codigo'] + ' - ' + record['name']))
        return res

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        ids = []

        context = context and context.copy() or {}
        if name:
            if len(name) == 2:
                try:
                    int(name)
                    ids = self.search(cr, uid, [('codigo', '=', name)] + args, limit=limit, context=context)
                except ValueError:
                    pass
            if not ids:
                ids = self.search(cr, uid, [('name', 'ilike', name)] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)

        return self.name_get(cr, uid, ids)

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'), _('Desative o CST ao inves de exclui-lo!'))

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('install_mode') and context['install_mode']:
            args = [('encargo_id', '=', vals['encargo_id']),
                    ('codigo', '=', vals['codigo'])
            ]
            _id = self.search(cr, uid, args, context=context)
            if _id:
                return _id[0]
            # Fim do Bloco #
        res = super(fiscal_cst, self).create(cr, uid, vals, context)
        return res

    def fiscst_get_tipo_movimento(self, cr, uid, codigo_cst, encargo_id, context=None):
        """
        Recupera o tipo de movimento em funcao do CST e do encargo.

        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param codigo_cst: String contendo o codigo do cst
        :param encargo_id: Id do encargo a ser verificado
        :param context: dicionario com valores contextuais
        :type cr: object
        :type uid: int
        :type codigo_cst: str
        :type encargo_id: int
        :type context: dict
        :return o valor total da bonificacao da nota fiscal.
        :rtype: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        cst_ids = self.search(cr, uid, [('codigo', '=', codigo_cst),
                                        ('encargo_id', '=', encargo_id)], context=context)

        return cst_ids and self.browse(cr, uid, cst_ids[0], context).fiscst_tipo_movimento

    def fiscst_check_tipo_movimento(self, cr, user, cst, encargo, movimento, context=None):
        """
        Verifica se existe CST para o tipo de movimento (Entrada/Saida/Ambos) informado.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param cst: o codigo do CST
        :param encargo: o ID do encargo
        :param movimento: tipos de movimento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type cst: str
        :type encargo: long
        :type movimento list|tuple
        :type context: dict
        :return: verdadeiro se o codigo
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        if not isinstance(movimento, (list, tuple)):
            movimento = [movimento]
        return self.pool.get('fiscal.cst').search(
            cr, user, [('encargo_id', '=', encargo), ('fiscst_tipo_movimento', 'in', movimento),
                       ('codigo', '=', cst)], context=context) and True or False

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
