# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2013 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_cst_correlacao(osv.Model):
    _name = 'fiscal.cst.correlacao'
    _inherit = 'fiscal.cst.correlacao'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('install_mode') and context['install_mode']:
            args = [('encargo_id', '=', vals['encargo_id']),
                    ('cst_saida_id', '=', vals['cst_saida_id'])
            ]
            _id = self.search(cr, uid, args, context=context)
            if _id:
                return _id[0]
            # Fim do Bloco #
        res = super(fiscal_cst_correlacao, self).create(cr, uid, vals, context)
        return res

    def correlacao_cst(self, cr, uid, encargo_id, codigo, movimento, tipo, context=None):
        """
        Sugere a troca de CST na importacao dos itens da NFe de acordo com o cadastro de correlacao.

        :param cr: cursor do banco de dados
        :param uid: id do usuario responsavel pela transacao
        :param encargo_id: ID do encargo
        :param codigo: codigo do CST
        :param movimento: tipo de movimento (CST): compra/venda, devolucao, transferencia, etc
        :param tipo (CST): entrada ou saida (E/S)
        :param context: dicionario com valores contextuais
        :type cr: object
        :type uid: int
        :type codigo: str
        :type encargo_id: long
        :type movimento: str
        :type tipo: str
        :type context: dict
        :return: o ID do CFOP da correlacao entrada/saida para itens de produtos.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_fiscal_cst = self.pool.get('fiscal.cst')

        # Ajusta o CFOP do item entrada/saida
        args = [('encargo_id', '=', encargo_id), ('codigo', '=', codigo)]
        cst_ids = obj_fiscal_cst.search(cr, uid, args, context=context)

        if not cst_ids:
            return codigo

        cst_id = cst_ids[0]

        if movimento == 'D':  # Notas de devolucao
            # Se a nota fiscal for de devolucao, busca pelo CST oposto
            brw_cst = obj_fiscal_cst.browse(cr, uid, cst_id, context=context)
            cst = brw_cst.fiscst_cst_oposto_id.codigo
        else:
            args.pop()
            # 'tipo' eh o tipo do documento que sera criado
            # se tipo do movimento for entrada, tem que achar qual cst de entrada corresponde ao que
            # tenho de saida
            if tipo == 'E':  # Entrada
                args += [('cst_saida_id', '=', cst_id)]

            # se tipo do movimento for saida, tem que achar qual cst de saida corresponde ao que
            # tenho de entrada
            else:  # Saida
                args += [('cst_entrada_id', '=', cst_id)]

            correlacao_ids = self.search(cr, uid, args, context=context)

            # Faz a correlacao
            if correlacao_ids:
                correlacao_id = correlacao_ids[0]
                brw_correlacao = self.browse(cr, uid, correlacao_id, context=context)

                if tipo == 'E':
                    cst = brw_correlacao.cst_entrada_id.codigo
                else:
                    cst = brw_correlacao.cst_saida_id.codigo
            else:
                cst = codigo

        return cst

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
