# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2016, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import osv
from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400
from openerp.tools.translate import _
import base64
import re
import xml.etree.ElementTree as ET
from datetime import date, datetime

# noinspection PyAbstractClass,PyPep8Naming
class fiscal_nfe_consumo(osv.Model):
    """
    Manifestacao destinatario.
    """
    _inherit = 'fiscal.nfe.consumo'

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.financeiro = None
            self.fiscal = None
            self.estoque = None
            self.adm = None
            self.rh = None
            self.__dict__.update(entries)

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de pedidos ainda nao concluidos.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :return: dominio utilizado para contar pedidos pendentes.

        :see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        :copyright: OpenERP SA.
        """
        context = context and context.copy() or {}
        return [('state', 'not in', ('IM','OR','DO','CO','BA'))]

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or dict()
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in ['financeiro', 'fiscal', 'estoque', 'adm', 'rh']])
        return self.ParamToObject(**parametros)

    def confirma_evento(self, cr, uid, ids, tipo_evento, context=None, motivo=None):
        context = context and context.copy() or {}
        parametro = self._parametros(cr, uid)
        brw_pessoa = self.pool.get('res.company').browse(
            cr, uid, self.pool.get('res.users').browse(
                cr, uid, uid, context).company_id.id, context)
        enderecos = self.pool.get('adm.pessoa').read(
            cr, uid, brw_pessoa and brw_pessoa.rescon_pessoa_id.id, ['endereco_ids'],
            context=context).get('endereco_ids')
        cnpj_emissor = brw_pessoa and re.sub('[^0-9]', '', self.pool.get('adm.pessoa').browse(
            cr, uid, brw_pessoa.rescon_pessoa_id.id, context).cpf_cnpj)
        estado = False
        for x in self.pool.get('adm.pessoa.endereco').browse(cr, uid, enderecos, context):
            if x.endereco_tipo_id.name == 'Comercial':
                estado = x.uf_id.sigla
                break
            if x.endereco_tipo_id.name == 'Normal':
                estado = x.uf_id.sigla
                break
        if not estado:
            raise osv.except_osv(_('Atencao!'),
                                 _('Cadastre um endereco valido para a empresa.'))
        if not self.valida_data_certif(parametro.fiscal.data_validade):
            raise osv.except_osv(_('Atencao!'),
                                 _(' Informacoes do certificado digital nao existem ou sao '
                                   'invalidas.'))
        nfe = nfe_400
        nfe = nfe(certify=parametro.fiscal.certificado_conteudo,
                  key=parametro.fiscal.chave_conteudo,
                  environment=parametro.fiscal.fispar_nfe_ambiente,
                  state=estado,
                  contingency='')
        result = nfe.efetuar_manifesto(cnpj_emissor, tipo_evento, self.browse(
            cr, uid, ids[0], context).chave_nfe, motivo)

        return result

    def confirmar_operacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        processo, __ = self.confirma_evento(cr, uid, ids, '210200', context)
        if processo.resposta.cStat.valor == u'128':  # Lote de evento processado
            if processo.resposta.infEvento.cStat.valor in (u'573', u'135'):
                self.write(cr, uid, ids, dict(state='CO', fisnfco_confirmou_operacao=True), context)
            else:
                raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.infEvento.cStat.valor,
                                                              processo.resposta.infEvento.xMotivo.valor))
        else:
            if not processo.resposta.cStat.valor:
                processo.resposta.cStat.valor = unicode(processo.resposta.status)
                processo.resposta.xMotivo.valor = processo.resposta.reason
            raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.status, processo.resposta.reason))
        return True

    def efetuar_ciencia_operacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        remover_acentuacao = self.pool.get('adm.utils').admutl_remover_acentuacao
        data_da_ciencia = date.today()
        processo, __ = self.confirma_evento(cr, uid, ids, '210210', context)
        if processo.resposta.cStat.valor == u'128':  # Lote de evento processado
            if processo.resposta.infEvento.cStat.valor in (u'573', u'135'):
                self.write(cr, uid, ids, dict(state='CI', fisnfco_data_ciencia_oper=data_da_ciencia), context)
            else:
                if not processo.resposta.infEvento.cStat.valor:
                    processo.resposta.infEvento.cStat.valor = unicode(processo.resposta.status)
                    processo.resposta.infEvento.xMotivo.valor = processo.resposta.reason
                raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.infEvento.cStat.valor,
                                                              processo.resposta.infEvento.xMotivo.valor))
        else:
            if not processo.resposta.cStat.valor:
                processo.resposta.cStat.valor = unicode(processo.resposta.status)
                processo.resposta.xMotivo.valor = remover_acentuacao(cr, uid, processo.resposta.reason)
            raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.cStat.valor,
                                                          processo.resposta.xMotivo.valor))

        return True

    def desconhecimento_operacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        processo, __ = self.confirma_evento(cr, uid, ids, '210220', context)
        if processo.resposta.cStat.valor == u'128':  # Lote de evento processado
            if processo.resposta.infEvento.cStat.valor in (u'573', u'135'):
                self.write(cr, uid, ids, dict(state='DO', fisnfco_desconheceu=True), context)
            else:
                raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.infEvento.cStat.valor,
                                                              processo.resposta.infEvento.xMotivo.valor))
        else:
            raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.cStat.valor,
                                                          processo.resposta.xMotivo.valor))

        return True

    def operacao_nao_realizada(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        id_info = self.pool.get('wizard.nfe.operacao.nao.realizada').create(
            cr, uid, {'wizonr_nfe_consumo_id': ids[0]}, context)
        return {
            'name': _('Manifestacao destinatario - Operacao nao realizada'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_id': id_info,
            'res_model': 'wizard.nfe.operacao.nao.realizada',
            'view_id': False,
            'context': context,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }

    @staticmethod
    def remove_accents(text):
        from unicodedata import normalize, category
        unicode_string = unicode(text, 'utf-8') if type(text) == str else text
        text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

        return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

    def download_xml(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        parametro = self._parametros(cr, uid)
        ambiente = parametro.fiscal.fispar_nfe_ambiente
        empresa_corrente_id = self.pool.get('res.users').browse(
            cr, uid, uid, context).company_id.id
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_pessoa_end = self.pool.get('adm.pessoa.endereco')
        pessoa_id = self.pool.get('res.company').browse(
            cr, uid, empresa_corrente_id, context).rescon_pessoa_id.id
        if not pessoa_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao existe uma pessoa associada a empresa cadastrada.'))
        enderecos = obj_pessoa.read(
            cr, uid, pessoa_id, ['endereco_ids'], context)['endereco_ids']
        cnpj_emissor = ''.join(filter(lambda f: f in '01234567890', obj_pessoa.browse(
            cr, uid, pessoa_id, context).cpf_cnpj))
        estado = False
        if enderecos:
            for x in obj_pessoa_end.browse(cr, uid, enderecos, context):
                if x.endereco_tipo_id.name == 'Comercial':
                    estado = x.uf_id.sigla
                    break
                if x.endereco_tipo_id.name == 'Normal':
                    estado = x.uf_id.sigla
                    break
            if not estado:
                raise osv.except_osv(_('Atencao!'),
                                     _('Cadastre um endereco valido para a empresa.'))
        chave_acesso = self.browse(cr, uid, ids[0], context).chave_nfe
        danfe = str()
        nfe = nfe_400(certify=parametro.fiscal.certificado_conteudo,
                      key=parametro.fiscal.chave_conteudo,
                      environment=parametro.fiscal.fispar_nfe_ambiente, state=estado,
                      contingency='')
        processo, __ = nfe.download_xml(cnpj_emissor, chave_acesso)
        if processo.resposta.cStat.valor == u'138':  # Documento localizado para o destinatario
            if processo.resposta.loteDistDFeInt:
                if processo.resposta.loteDistDFeInt[0].schema.valor.startswith('procNFe'):
                    xml = self._decodifica_nfe(processo.resposta.loteDistDFeInt[0].docZip.valor)
                    xml = self.remove_accents(xml)
                    try:
                        # O sistema estah implementado apenas DANFE formato retrato
                        if xml.find('<tpImp>1</tpImp>') != -1:
                            danfe = nfe.gerar_danfe(xml)
                    except Exception as erro:
                        raise osv.except_osv(_('Atencao!'),
                                             _('Erro ao tentar gerar o DANFE desta nota: %s' %
                                               erro))
                    self.write(cr, uid, ids, dict(
                        status_sefaz='%s - %s' % (processo.resposta.cStat.valor, processo.resposta.xMotivo.valor),
                        proc_resposta=xml,
                        state='BA',
                        xml_file=base64.encodestring(xml.encode('utf-8', 'strict')),
                        fisnfco_danfe_nfe=danfe and base64.encodestring(danfe),
                        file=chave_acesso + '.xml',
                        fisnfco_nome_danfe=danfe and chave_acesso + '.pdf'), context)
            else:
                if not processo.resposta.cStat.valor:
                    processo.resposta.cStat.valor = unicode(processo.resposta.status)
                    processo.resposta.xMotivo.valor = unicode(processo.resposta.reason)
                raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.cStat.valor,
                                                              processo.resposta.xMotivo.valor))
        else:
            if processo.resposta.reason == 'OK':
                raise osv.except_osv(_('Erro!'), '%s - %s' % (processo.resposta.cStat.valor,
                                                              processo.resposta.xMotivo.valor))
            else:
                raise osv.except_osv(_('Erro de conexao com o servidor da SEFAZ'), _('%s - %s' % (
                    processo.resposta.status, self.remove_accents(processo.resposta.reason))))
        return ids

    @staticmethod
    def _decodifica_nfe(binario):
        import os
        import gzip
        nfe = binario.decode('base64')
        tmp_file = os.tempnam('/tmp/', binario)
        arquivo = open(tmp_file, 'wb')
        arquivo.write(nfe)
        arquivo.close()
        nfe = gzip.open(tmp_file)
        xml = nfe.read()
        nfe.close()
        os.remove(tmp_file)

        return xml

    def button_gerar_nf_para_importacao(self, cr, uid, ids, context):
        context = context and context.copy() or {}
        obj_fiscal_import_nfe = self.pool.get('fiscal.import.nfe.osv')
        dados = self.read(cr, uid, ids, ['xml_file', 'fisnfco_natureza_id', 'fisnfco_local_id'],
                          load='_classic_write')
        xml_file = False
        res = False
        natureza_id = False
        local_id = False
        for dado in dados:
            if not dado.get('fisnfco_natureza_id'):
                raise osv.except_osv(_('Atencao!'),
                                     _('E necessario informar a natureza para importar a nota.'))
            xml_file = dado.get('xml_file')
            natureza_id = dado.get('fisnfco_natureza_id')
            local_id = dado.get('fisnfco_local_id')
        dados_xml = ET.fromstring(xml_file.decode('base64'))
        dados_xml = filter(lambda i: i.tag.split('}')[-1] == 'nfeProc', dados_xml.iter())
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'fiscal.nfe.consumo')
        dados_parametro_estoque = self.pool.get('adm.utils'). \
            admutl_get_parametros(cr, uid, 'estoque.parametro', 'estpar_local_compra_padrao_id',
                                  context=None)
        if not local_id and not dados_parametro_estoque.get('estpar_local_compra_padrao_id'):
            osv.except_osv(_('Atencao!'),
                           _('Defina um local para entradas no parametro do estoque.'))
        novo_xml = ""
        vals = {
            'company_id': company_id,
            'natureza_id': natureza_id,
            'local_id': local_id or dados_parametro_estoque.get('estpar_local_compra_padrao_id')
        }
        for i in dados_xml:
            novo_xml += ET.tostring(i)
        if novo_xml:
            novo_xml = novo_xml.encode('base64')
            vals['xml_file'] = novo_xml
            res = obj_fiscal_import_nfe.create(cr, uid, vals, context=context)
        return res

    def button_abrir_nf(self, cr, uid, ids, context):
        context = context and context.copy() or {}
        obj_fisnfc = self.pool.get('fiscal.nf.capa')
        obj_fisinfe = self.pool.get('fiscal.import.nfe.osv')
        obj_model_data = self.pool.get('ir.model.data')
        nf_id = False
        importada = False
        for i in self.browse(cr, uid, ids):
            chave_nfe = i.chave_nfe
            if chave_nfe:
                args_fiscal_nf_capa = [('state', '!=', 'C'), ('tipo', '=', 'E'),
                                       ('nfe_chave_acesso', '=', chave_nfe)]
                nf_id = obj_fisnfc.search(cr, uid, args_fiscal_nf_capa, context=context)
                if nf_id:
                    importada = True
                else:
                    args_import_nfe = [('ide_id', 'like', chave_nfe)]
                    nf_id = obj_fisinfe.search(cr, uid, args_import_nfe, context=context)
        if nf_id and importada:
            if isinstance(nf_id, list):
                nf_id = nf_id[0]
            reference = obj_model_data.get_object_reference(cr, uid, 'fiscal',
                                                            'view_fiscal_nf_capa_terceiro_form')
            view_id = reference and reference[1] or False
            res = {
                'name': 'Nota fiscal de entrada',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'fiscal.nf.capa',
                'view_id': [view_id],
                'res_id': nf_id,
                'context': {'proprio': False, 'terceiro': True},
                'type': 'ir.actions.act_window',
            }
        elif nf_id and not importada:
            if isinstance(nf_id, list):
                nf_id = nf_id[0]
            reference = obj_model_data.get_object_reference(cr, uid, 'fiscal',
                                                            'fiscal_import_nfe_osv_view')
            view_id = reference and reference[1] or False
            res = {
                'name': 'Importacao de Nota fiscal de entrada',
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'fiscal.import.nfe.osv',
                'view_id': [view_id],
                'res_id': nf_id,
                'context': {'proprio': False, 'terceiro': True},
                'type': 'ir.actions.act_window',
            }
        else:
            res = {}

        return res

    @staticmethod
    def valida_data_certif(data_certificado):
        """
        Este metodo verifica se a data do certificado digital passado como parametro
        eh menor que a data atual
        :param data_certificado: String contendo a data de vencimento do certificado
        :type data_certificado: str
        :rtype: boolean
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        data_certificado = datetime.strptime(data_certificado, '%Y-%m-%d %H:%M:%S')
        if data_certificado < datetime.now():
            return False
        return True

    def fisnfce_button_active(self, cr, user, ids, context):
        context = context and context.copy() or {}
        nfe_consumo_dados = self.read(cr, user, ids[0], ['active'], context)

        return self.write(cr, user, ids, {'active': not nfe_consumo_dados['active']}, context)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

