#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class fiscal_sugestao_preco_capa(osv.Model):
    """
    Classe reponsavel pela regra de negocio da sugestao de precos
    Esta classe era uma Transient Model que tinha como funcionalidade apenas gerar relatorio.
    Foi alterada para uma model e tem funcionalidade de alterar o preco de venda dos itens
    na tabela estoque.preco.venda alem aumentar o percentual da sugestao.
    A implemntacao feita nao alterou a regra de calculo que existiam no relatorio
    Implementacao no CHAMADO 91304.
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    _name = 'fiscal.sugestao.preco'
    _inherit = 'fiscal.sugestao.preco'

    _custos = {'U': ['ultimo_custo', 'ULTIMO'], 'M': ['custo_medio_contabil', 'CONTABIL'],
               'R': ['custo_reposicao', 'REPOSICAO'], 'C': ['custo_corrigido', 'CORRIGIDO']}
    _margem_preco_venda = {'I': 'margem_ideal', 'B': 'margem_bruta'}

    def fissup_get_parametros(self, cr, uid, context=None):
        """
        Metodo responsavel por retornar parametros que serao utilizados para elaboracao
        da sugestao de precos, como custos e margens a serem utilizadas
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type context: dict
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy()
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'fiscal.sugestao.preco.item', context=context)
        obj_est_par = self.pool.get('estoque.parametro')
        est_par_id = obj_est_par.search(cr, uid, [('company_id', '=', company_id)], context=context)
        vals = {}
        custo_fixo = 0.0
        comissao = 0.00
        custo_para_preco_venda = ''
        string_custo = ''
        margem_parametro = ''
        if est_par_id:
            brw_est_par = obj_est_par.browse(cr, uid, est_par_id[0])
            if brw_est_par.custo_fixo:
                custo_fixo = brw_est_par.custo_fixo
            if brw_est_par.comissoes:
                comissao = brw_est_par.comissoes
            if brw_est_par.estpar_custo_para_precovenda:
                custo_para_preco_venda = \
                    self._custos.get(brw_est_par.estpar_custo_para_precovenda)[0]
                string_custo = self._custos.get(brw_est_par.estpar_custo_para_precovenda)[1]
                margem_parametro = self._margem_preco_venda.get(
                    brw_est_par.estpar_margem_para_preco_venda)
        vals['fissup_custo_fixo'] = custo_fixo
        vals['fissup_custo_para_preco_venda'] = custo_para_preco_venda
        vals['fissup_comissao'] = comissao
        vals['fissup_string_custo'] = string_custo
        vals['fissup_margem_parametro'] = margem_parametro
        return vals

    def pesquisar(self, cr, uid, ids, context=None):
        """
        Metodo realizar a pesquisa dos itens que se enquadrao nos parametros de sugestao
        de precos, para isso utiliza-se do metodo fisspi_gera_dados_itens da classe
        'fiscal.sugestao.preco.item'
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param ids: Lista de ids da sugestao de preco
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'local_venda_padrao_id', context=context)
        local_venda_padrao_id = estoque_parametro_dados.get('local_venda_padrao_id')
        if not local_venda_padrao_id:
            raise osv.except_osv(_('Atencao'), _("Cadastre o 'local para saidas' nos parametros do "
                                                 "estoque"))
        for sugestap_capa in self.browse(cr, uid, ids, context=context):
            self.pool.get('fiscal.sugestao.preco.item').fisspi_gera_dados_itens(
                cr, uid, sugestap_capa, local_venda_padrao_id, context=context)
        return True

    def imprimir(self, cr, uid, ids, context=None):
        """
        Metodo responsavel por fazer a chamada da classe de impressao do relatorio de sugestao de
        precos
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param ids: Lista de ids da sugestao de preco
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        sugestao_itens_ids = self.pool.get('fiscal.sugestao.preco.item').search(cr, uid, [
            ('fisspi_capa_id', 'in', ids)])
        datas = dict(sugestao=self.read(cr, uid, ids[0], [], context),
                     sugestao_itens=self.pool.get('fiscal.sugestao.preco.item').read(
                         cr, uid, sugestao_itens_ids, [], context=context, load='_classic_write'))
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'sugestao_precos',
            'datas': datas,
        }

    def fissup_atualizar_precos(self, cr, uid, ids, context=None):
        """
        Metodo responsavel atualizar para atualizar todos os precos de venda
        dos itens contidos na listagem, para isso utiliza-se do metodo fisspi_atualiza_preco_item
        da classe 'fiscal.sugestao.preco.item'
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param ids: Lista de ids da sugestao de preco
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_est_preco_venda = self.pool.get('estoque.preco.venda')
        desc_itens = str()
        for capa in self.browse(cr, uid, ids, context=context):
            for item in capa.fissup_sugestao_preco_itens_ids:
                vals = {'produto_id': item.fisspi_produto_id.id,
                        'preco_venda': item.fisspi_sugestao}
                try:
                    obj_est_preco_venda.valida_preco_margem(cr, uid, vals, context)
                except:
                    desc_itens += '%s - %s\n' % (item.fisspi_produto_id.codigo,
                                                 item.fisspi_produto_id.name)
                    continue
                item.fisspi_atualiza_preco_item(sugestao_item=item, context=context)
        if desc_itens:
            raise osv.except_osv(_('Atencao'),
                                 _('Os precos de venda dos itens abaixo nao podem ser menores que '
                                   'a margem minima ou as margens de lucro estao negativas:\n %s' %
                                   desc_itens))
        return True

    def fissup_onchange_item(self, cr, user, ids, item, context=None):
        """
        Filtra os itens da sugestao em funcao de um item.
        
        :param cr: 
        :param user: 
        :param ids: 
        :param item: o ID do item utilizdo no filtro
        :param context: 
        :type cr: object
        :type user: int
        :type ids: list
        :type item: long
        :type context: dict
        :return: 
        :rtype: dict
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        item_ids = self._fissup_filtrar_sugestao_por_item(cr, user, ids, item, context)

        return dict(value=dict(fissup_sugestao_preco_itens_ids=item_ids))

    def _fissup_filtrar_sugestao_por_item(self, cr, user, sugestao, item, context=None):
        """
        Filtra os itens da sugestao em funcao de um item.
        
        :param cr: 
        :param user: 
        :param sugestao: lista com os ID das sugestoes de preco
        :param item: o ID do item utilizdo no filtro
        :param context: 
        :type cr: object
        :type user: int
        :type sugestao: list
        :type item: long
        :type context: dict
        :return: lista com o ID dos itens de satisfazem as condicoes do filtro.
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if item:  # Filtro por um item especifico
            item_ids = self.pool.get('fiscal.sugestao.preco.item').search(
                cr, user, [('fisspi_produto_id', '=', item), ('fisspi_capa_id', 'in', sugestao)],
                context=context)
        else:  # Mostra todos os itens
            item_ids = self.pool.get('fiscal.sugestao.preco.item').search(
                cr, user, [('fisspi_capa_id', 'in', sugestao)], context=context)

        return item_ids

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
