#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
import time
from openerp.tools.translate import _


class fiscal_sugestao_preco_item(osv.Model):
    """
    Classe reponsavel pela regra de negocio do itens da sugestao de precos
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """

    _name = 'fiscal.sugestao.preco.item'
    _inherit = 'fiscal.sugestao.preco.item'

    def fisspi_get_preco_custo_ficha_tecnica(self, cr, uid, grade_ids, context=None):
        """
        O metodo define os custos para os produtos (grade_ids) recebidos baseando-se nos custos
        totais da ficha tecnica de cada um. Metodo sobreescrito no modulo PCP.
        :param cr: cursor no banco de dados.
        :type cr: int
        :param uid: id da tabela res_users, identificador do usuario na transacao.
        :type uid: int
        :param grade_ids: lista de id da tabela estoque_produto_grade, identificadores dos produtos.
        :type grade_ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: dados da identificacao das grades com seus respectivos custos.
        :rtype: dict
        """
        context = context and context.copy() or {}
        return {}

    def fisspi_gera_dados_itens(self, cr, uid, sugestao_capa, local_venda_padrao_id, context=None):
        """
        Este metodo e reponsavel por fazer a chamada dos metodos que
        irao gerar os registros do item da sugestao de precos. Para fazer
        isto e analisado qual a opcao selecionada para a sugestao.
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param sugestao_capa: Brwoser da capa da sugestao
        :param local_venda_padrao_id: id da tabela estoque_local
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type sugestao_capa: browse
        :type local_venda_padrao_id: int
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        sugestao_itens_ids = map(lambda i: i.id, sugestao_capa.fissup_sugestao_preco_itens_ids)
        self.unlink(cr, uid, sugestao_itens_ids, context)
        self.est_par_margem_preco_venda = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque_parametro', 'estpar_margem_para_preco_venda')
        if self.est_par_margem_preco_venda and isinstance(self.est_par_margem_preco_venda, list):
            self.est_par_margem_preco_venda = self.est_par_margem_preco_venda[0]
        if self.est_par_margem_preco_venda and isinstance(self.est_par_margem_preco_venda, dict):
            self.est_par_margem_preco_venda = self.est_par_margem_preco_venda.get(
                'estpar_margem_para_preco_venda')

        if sugestao_capa.fissup_opcoes == 'E':
            self.fisspi_dados_itens_entradas(cr, uid, sugestao_capa, local_venda_padrao_id,
                                             context=context)
        else:
            self.fisspi_gera_dados_itens_totais_sugetao(cr, uid, sugestao_capa, context=context)

    def _fisspi_get_dados_menor_embalagem(self, cr, uid, produto_id, context=None):
        """
        O metodo encontra a quantidade na embalagem e a unidade para a menor embalagem do produto.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com os valores na menor embalagem.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        res = {'quantidade_na_embalagem': 1.0}
        produto_embalagem_ids = obj_estoque_produto_embalagem.search(
            cr, uid, [('produto_id', '=', produto_id)], context=context)
        if not produto_embalagem_ids:
            return res
        brw_produto_embalagem = obj_estoque_produto_embalagem.browse(
            cr, uid, produto_embalagem_ids, context)
        embalagem_ids = map(lambda i: i.embalagem_id.quantidade_na_embalagem,
                            brw_produto_embalagem)
        embalagem_ids.sort()
        res['quantidade_na_embalagem'] = embalagem_ids[0]
        res['unidade_id'] = sorted(map(lambda i: i.embalagem_id, brw_produto_embalagem),
                                   key=lambda i: i.quantidade_na_embalagem)[0].unidade_id.id
        return res

    def fisspi_dados_itens_entradas(self, cr, uid, sugestao_capa, local_venda_padrao_id,
                                    context=None):
        """
        Este metodo e reponsavel por gerar os dados da sugestao olhando a entrada dos itens
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param sugestao_capa: Brwoser da capa da sugestao
        :param local_venda_padrao_id: id da tabela estoque_local
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type sugestao_capa: browse
        :type local_venda_padrao_id: int
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        obj_fis_enc = self.pool.get('fiscal.encargo')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        vals = {}
        dados_nota = self.fisspi_get_dados_entrada(cr, sugestao_capa, company_id, context)
        if not dados_nota:
            raise osv.except_osv(_('Atencao'), _('Nao ha notas fiscais para a data de entrada.'))
        ids_produto = list(set(map(lambda i: i['produto_id'], dados_nota)))
        ids_grade = list(set(map(lambda i: i.get('grade_id'), dados_nota)))
        custo_ficha_tecnica_dados = self.fisspi_get_preco_custo_ficha_tecnica(cr, uid, ids_grade,
                                                                              context)
        tributos = self._fisspi_get_tributos(cr, ids_produto)
        # custos
        custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, ids_produto, company_id,
                                                          context=context)
        # custos anterior
        custo_anterior = obj_estoque_preco_custo.estprc_get_custos_do_dia_anterior(
            cr, uid, ids_produto, company_id, context=context)
        # preco venda
        preco_venda = obj_estoque_preco_venda.get_preco_venda(
            cr, uid, ids_produto, epc_grade_ids=ids_grade, epc_company_id=company_id,
            pv_so_do_cadastro=True, context=context)
        # saldo
        saldos = obj_estoque_produto_saldo.get_saldo_do_dia(
            cr, uid, ids_grade, time.strftime('%Y-%m-%d'), False, context=context)
        # margens
        margens = obj_estoque_produto.estpro_get_margens(cr, uid, ids_produto)
        custo_parametro = 'estprc_%s' % sugestao_capa.fissup_custo_para_preco_venda
        produto_ids = list()
        for dado in dados_nota:
            produto_id = dado.get('produto_id')
            grade_id = dado.get('grade_id')
            if produto_id in produto_ids:
                continue
            produto_ids.append(produto_id)
            saldo = (saldos.get(grade_id) or {}).get('saldo') or 0.0
            if not sugestao_capa.fissup_listar_saldo_zero and not saldo:
                continue
            val_custo_par = 0.00
            val_custo_ant_par = 0.00
            val_custo_sel = 0.00
            dados_menor_embalagem = self._fisspi_get_dados_menor_embalagem(cr, uid, produto_id,
                                                                           context)
            qt_embalagem = dados_menor_embalagem.get('quantidade_na_embalagem')
            icms_id = obj_fis_enc.search(cr, uid, [('sigla', '=', 'ICMS')], context=context)[0]
            args_trib_enc = [('produto_id', '=', produto_id),
                             ('encargo_id', '=', icms_id),
                             ('tipo_tributacao', '=', 'T')]
            tributacao_encargo_id = obj_fiscal_tributacao_encargos.search(cr, uid,
                                                                          args_trib_enc,
                                                                          context=context)
            brw_trib_encar = obj_fiscal_tributacao_encargos.browse(cr, uid,
                                                                   tributacao_encargo_id)
            custo_listar = self._fisspi_get_field_custo(sugestao_capa.fissup_custo)
            soma_encargo = tributos.get(produto_id) or 0
            if soma_encargo:
                soma_encargo = soma_encargo.get('aliq')
            if custo_anterior.get(produto_id):
                val_custo_ant_par = (custo_anterior.get(produto_id).get(
                    custo_parametro) or 0.0) * qt_embalagem
            if custo.get(produto_id):
                val_custo_par = \
                    (custo.get(produto_id).get(custo_parametro) or 0.0) * qt_embalagem
                val_custo_sel = \
                    (custo.get(produto_id).get(custo_listar) or 0.0) * qt_embalagem
            dados_preco_venda = preco_venda.get(produto_id) or {}
            valor_preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')
            preco_venda_item = valor_preco_venda * qt_embalagem
            margem_atual = self._calcula_margem_atual(
                preco_venda_item, val_custo_par, soma_encargo, sugestao_capa.fissup_custo_fixo,
                sugestao_capa.fissup_comissao)
            margem_ideal = margens[produto_id]['margem_ideal']
            margem_minima = margens[produto_id]['margem_minima']
            margem_bruta = margens[produto_id]['margem_bruta']
            total_encargos = soma_encargo + sugestao_capa.fissup_comissao + sugestao_capa. \
                fissup_custo_fixo
            if sugestao_capa.fissup_usar_margem_bruta_calculada:
                if preco_venda_item:
                    custo_reposicao = custo[produto_id]['estprc_custo_reposicao']
                    margem_bruta = self._calcula_margem_bruta(preco_venda_item, custo_reposicao)
                total_encargos = 0.0
            else:
                margem_bruta = margens[produto_id]['margem_bruta']
            margem_para_preco_venda = margens[produto_id][sugestao_capa.fissup_margem_parametro]

            icms = False
            aliquota = False
            qtd = 0
            if brw_trib_encar:
                icms = brw_trib_encar[0].cst_id.tipo
                aliquota = brw_trib_encar[0].tributacao_id.aliquota

            if sugestao_capa.fissup_opcoes == 'E':  # So entradas
                qtd = dado.get('quantidade') or 0

            preco_sugestao = self._fisspi_calcula_sugestao(
                    self.est_par_margem_preco_venda, val_custo_par, total_encargos,
                    margem_para_preco_venda, margem_bruta)

            vals['fisspi_capa_id'] = sugestao_capa.id
            vals['fisspi_produto_id'] = dado.get('produto_id')
            vals['fisspi_grade_id'] = dado.get('grade_id')
            vals['fisspi_estoque_embalagem_unidade_id'] = dados_menor_embalagem.get(
                'unidade_id') or dado.get('fisspi_estoque_embalagem_unidade_id')
            vals['fisspi_estoque_embalagem_id'] = dado.get('embalagem_id')
            vals['fisspi_quantidade_embalagem'] = qt_embalagem
            vals['fisspi_nf_id'] = dado.get('nf_id')
            vals['fisspi_preco_custo_parametro'] = custo_ficha_tecnica_dados.get(grade_id) or \
                val_custo_par
            vals['fisspi_preco_custo_anterior'] = val_custo_ant_par
            vals['fisspi_preco_custo_selecionado'] = val_custo_sel
            vals['fisspi_preco_venda'] = preco_venda_item
            vals['fisspi_tipo_icms'] = icms
            vals['fisspi_aliquota'] = aliquota
            vals['fisspi_qtd'] = qtd
            vals['fisspi_margem_atual'] = margem_atual
            vals['fisspi_margem_ideal'] = margem_ideal
            vals['fisspi_margem_minima'] = margem_minima
            vals['fisspi_margem_bruta'] = margem_bruta
            vals['fisspi_sugestao'] = preco_sugestao
            vals['fisspi_produto_kit'] = dado.get('kit_de_produtos') or False
            self.create(cr, uid, vals, context=context)

    def fisspi_gera_dados_itens_totais_sugetao(self, cr, uid, sugestao_capa, context=None):
        """
        Este metodo e reponsavel por gerar os dados de acordo com os
        filtros totais ou so sugestao do campo opcao informado na capa
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param uid: ID do usuario logado no sistema
        :param sugestao_capa: Brwoser da capa da sugestao
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type sugestao_capa: browse
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fis_enc = self.pool.get('fiscal.encargo')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        dados = self.fisspi_get_total_sugestao(cr, uid, sugestao_capa, context)
        vals = {}
        produto_ids = list(set(map(lambda i: i['produto_id'], dados)))
        grade_ids = list(set(map(lambda i: i.get('grade_id'), dados)))
        custo_ficha_tecnica_dados = self.fisspi_get_preco_custo_ficha_tecnica(cr, uid, grade_ids,
                                                                              context)
        tributos = self._fisspi_get_tributos(cr, produto_ids)
        icms_id = obj_fis_enc.search(cr, uid, [('sigla', '=', 'ICMS')], context=context)[0]
        tipo_tributo = self._fisspi_get_tipo_encargo(cr, produto_ids, icms_id)
        custo_parametro = 'estprc_%s' % sugestao_capa.fissup_custo_para_preco_venda
        if not produto_ids:
            return True
        # custos
        custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_ids, company_id,
                                                          context=context)
        # custos anterior
        custo_anterior = obj_estoque_preco_custo.estprc_get_custos_do_dia_anterior(
            cr, uid, produto_ids, company_id, context=context)
        # preco venda
        preco_venda = obj_estoque_preco_venda.get_preco_venda(cr, uid, produto_ids,
                                                              epc_company_id=company_id,
                                                              epc_grade_ids=grade_ids,
                                                              pv_so_do_cadastro=True,
                                                              context=context)
        # saldo
        saldos = obj_estoque_produto_saldo.get_saldo_do_dia(
            cr, uid, grade_ids, time.strftime('%Y-%m-%d'), False, context=context)

        custo_listar = self._fisspi_get_field_custo(sugestao_capa.fissup_custo)
        list_produto = list()
        for dado in dados:
            produto_id = dado.get('produto_id')
            grade_id = dado.get('grade_id')
            if produto_id in list_produto:
                continue
            list_produto.append(produto_id)
            if not sugestao_capa.fissup_listar_saldo_zero:
                saldo = (saldos.get(grade_id) or {}).get('saldo') or 0.0
                if not saldo:
                    continue
            vals['fisspi_capa_id'] = sugestao_capa.id
            vals['fisspi_produto_id'] = dado.get('produto_id')
            vals['fisspi_grade_id'] = dado.get('grade_id')
            vals['fisspi_quantidade_embalagem'] = vals.get('embalagem') or 1.0
            if produto_id in tipo_tributo:
                vals['fisspi_tipo_icms'] = tipo_tributo[produto_id]['icms']
                vals['fisspi_aliquota'] = tipo_tributo[produto_id]['aliq']
                soma_encargo = tributos[produto_id]['aliq']
                vals['fisspi_encargo_venda'] = True
            else:
                vals['fisspi_encargo_venda'] = False
                soma_encargo = 0.00
            custo_anterior_parametro = 0.00
            if custo_anterior.get(produto_id):
                custo_anterior_parametro = (custo_anterior.get(produto_id).get(
                    custo_parametro) or 0.0) * vals.get('fisspi_quantidade_embalagem')
            vals['fisspi_margem_minima'] = 0.00
            vals['fisspi_margem_ideal'] = 0.00
            vals['fisspi_margem_atual'] = 0.00
            vals['fisspi_preco_custo_parametro'] = custo_ficha_tecnica_dados.get(grade_id) or 0.00
            vals['fisspi_preco_custo_anterior'] = 0.00
            vals['fisspi_preco_custo_selecionado'] = 0.00
            dados_preco_venda = preco_venda.get(produto_id) or {}
            valor_preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')
            vals['fisspi_preco_venda'] = valor_preco_venda * vals['fisspi_quantidade_embalagem']
            if produto_id in custo:
                vals['fisspi_preco_custo_parametro'] = custo_ficha_tecnica_dados.get(grade_id) or\
                    custo[produto_id][custo_parametro] * vals.get('fisspi_quantidade_embalagem')
                vals['fisspi_preco_custo_anterior'] = custo_anterior_parametro
                vals['fisspi_preco_custo_selecionado'] = \
                    custo[produto_id][custo_listar] * vals['fisspi_quantidade_embalagem']
                if sugestao_capa.fissup_opcoes == 'T':
                    vals['fisspi_margem_atual'] = self._calcula_margem_atual(
                        vals['fisspi_preco_venda'], vals['fisspi_preco_custo_parametro'],
                        soma_encargo, sugestao_capa.fissup_custo_fixo,
                        sugestao_capa.fissup_comissao)
                else:
                    vals['fisspi_margem_atual'] = 0.00
                vals['fisspi_margem_ideal'] = dado['margem_ideal']
                vals['fisspi_margem_minima'] = dado['margem_minima']
            vals['fisspi_margem_bruta'] = 0.00
            if sugestao_capa.fissup_usar_margem_bruta_calculada:
                if vals.get('fisspi_preco_venda'):
                    vals['fisspi_margem_bruta'] = self._calcula_margem_bruta(
                        vals.get('fisspi_preco_venda'),
                        vals.get('fisspi_preco_custo_parametro'))
            else:
                vals['fisspi_margem_bruta'] = dado['margem_bruta']
            total_encargos = soma_encargo + sugestao_capa.fissup_comissao + sugestao_capa. \
                fissup_custo_fixo
            vals['fisspi_sugestao'] = self._fisspi_calcula_sugestao(
                self.est_par_margem_preco_venda, vals.get('fisspi_preco_custo_parametro'),
                total_encargos, vals['fisspi_%s' % sugestao_capa.fissup_margem_parametro],
                vals.get('fisspi_margem_bruta')
            )
            if sugestao_capa.fissup_opcoes == 'T' and sugestao_capa.fissup_percentual_correcao > 0:
                vals['fisspi_sugestao'] = \
                    self.fisspi_get_valor_correcao(vals['fisspi_preco_venda'],
                                                   sugestao_capa.fissup_percentual_correcao)
            vals['fisspi_produto_kit'] = dado.get('kit_de_produtos') or False
            self.create(cr, uid, vals, context=context)

    @staticmethod
    def _fisspi_get_tributos(cr, produto_ids):
        """
        Metodo responsavel por retornar os tributos dos produtos,
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param produto_ids: IDs dos produtos
        :type cr: Cursor
        :type produto_ids: list
        :rtype: dict
        """
        if produto_ids:
            if isinstance(produto_ids, list):
                produto_ids = str(produto_ids).replace('[', '(').replace(']', ')')

            sql = """
                    SELECT produto_id AS produto_id,
                           Sum(aliq)  aliq
                    FROM   (SELECT produto_id                  AS produto_id,
                                   cst_id                      AS cst_id,
                                   encargo_id                  AS encargo_id,
                                   (SELECT tipo
                                    FROM   fiscal_cst
                                    WHERE  id = cst_id)        AS icms,
                                   (SELECT sigla
                                    FROM   fiscal_encargo
                                    WHERE  id = encargo_id)    AS sigla,
                                   (SELECT (aliquota * base) / 100
                                    FROM   fiscal_tributacao
                                    WHERE  id = tributacao_id) AS aliq
                            FROM   fiscal_tributacao_encargos
                            WHERE  produto_id IN %s
                                   AND tipo_tributacao = 'T') AS tmp
                    GROUP  BY produto_id

            """ % produto_ids
            cr.execute(sql)
            dados = cr.dictfetchall()
            vals = {}
            for i in dados:
                vals[i['produto_id']] = i
        else:
            vals = {}
        return vals

    @staticmethod
    def _fisspi_get_tipo_encargo(cr, produto_ids, encargo_id):
        """
        Metodo responsavel por retornar o tipo de encargo dos produtos
        :param cr: Cursor do banco de dados utilizado pelo openerp
        :param produto_ids: IDs dos produtos a serem consultados
        :param encargo_id: ID do do encargo a serem consultado
        :type cr: Cursor
        :type produto_ids: list
        :rtype: dict
        """
        if produto_ids:
            if isinstance(produto_ids, list):
                produto_ids = str(produto_ids).replace('[', '(').replace(']', ')')
            sql = """
                    SELECT produto_id                   AS produto_id,
                           (SELECT tipo
                            FROM   fiscal_cst
                            WHERE  id = cst_id)         AS icms,
                           (SELECT aliquota
                            FROM   fiscal_tributacao
                            WHERE  id = tributacao_id)  AS aliq
                    FROM   fiscal_tributacao_encargos
                    WHERE  encargo_id = %d
                    AND produto_id in %s
                    AND tipo_tributacao = 'T'
            """ % (encargo_id, produto_ids)
            cr.execute(sql)
            dados = cr.dictfetchall()
            vals = {}
            for dado in dados:
                vals[dado['produto_id']] = dado
        else:
            vals = {}
        return vals

    def _calcula_margem_atual(self, preco_venda, preco_custo, encargos, custo_fixo, comissao):
        """
        Margem atual:
        x = Preço de venda – preço de custo
        z = x / preço de venda = y * 100
        margem atual = z – (custo fixo + encargos + comissão (se destacar))
        :param preco_venda: Preco de venda do item
        :param preco_custo: Preco de custo do item
        :param encargos: Total dos encargos do item
        :param custo_fixo: Custo fixo cadastrado no parametro do estoque
        :param comissao: Comissao cadastrada no parametro do estoque
        :type preco_venda: float
        :type preco_custo: float
        :type encargos: float
        :type custo_fixo: float
        :type comissao: float
        :rtype: float
        """
        encargos = round(encargos, 2)
        if preco_venda == 0:
            return 0

        res = self._calcula_margem_bruta(preco_venda, preco_custo) # Margem Bruta
        if self.est_par_margem_preco_venda == 'I': # Margem Ideal
            res = res - (custo_fixo + encargos + comissao)

        return round(res, 2)

    @staticmethod
    def _calcula_margem_bruta(preco_venda, custo):
        """
        Margem bruta = ((preco-venda - reposicao)/ preco) * 100
        Chamar quando usar_margem_bruta_calculada' = True
        :param preco_venda: Preco de venda do produto
        :param custo: Preco de custo do produto
        :type preco_venda: float
        :type custo: float
        :rtype: float
        """
        margem_bruta = ((preco_venda - custo) / preco_venda) * 100
        return round(margem_bruta, 2)

    def _fisspi_calcula_sugestao(self, tipo_margem_preco_venda, val_custo_par, total_encargos,
                                 margem_para_preco_venda=0.0, margem_bruta=0.0):

        """
        Metodo responsavel por calcular o valor da sugestao de preco
        :param tipo_margem_preco_venda: Tipo da margem de lucro para o preco de venda
               (I=IDEAL, B=BRUTA)
        :param val_custo_par: Valor do custo de acordo com o custo selecionado no parametro
               do estoque
        :param total_encargos: Valor total dos encargos
        :param margem_para_preco_venda: Percentua da margem para o preco de venda
        :param margem_bruta: Percentual da margem bruta
        :type tipo_margem_preco_venda: str
        :type val_custo_par: float
        :type total_encargos: float
        :type margem_para_preco_venda: float
        :type margem_bruta: float
        :rtype: float
        """
        obj_estoque_utils = self.pool.get('estoque.utils')
        if tipo_margem_preco_venda == 'I':
            preco_sugestao = obj_estoque_utils.estutl_calcular_preco_de_venda(
                val_custo_par, total_encargos, margem_para_preco_venda)
        else:
            preco_sugestao = val_custo_par * 100 / ((100 - margem_bruta) or 100.0)
        return preco_sugestao

    def _fisspi_get_sql_dados_kit(self, capa):
        '''Implementado por henranca no modulo Kit'''
        return '', '', ''

    def fisspi_get_dados_entrada(self, cr, sugestao_capa, company_id, context=None):
        """
        Metodo responsavel por executar o sql de consulta dos dados referente a entrada na sugestao
        :param cr: Cursor do banco de dados
        :param sugestao_capa: Browser de fiscal.sugestao.preco
        :param company_id: ID da empresa que esta executando o processo
        :param context: dados contextuais
        :type cr: Cursor
        :type sugestao_capa: Browse
        :type company_id: int
        :type context: dict
        :rtype: dict
        """
        context = context and context.copy() or {}
        if sugestao_capa.fissup_data_entrada:
            query_data_receb = """nf.data_saida_entrada BETWEEN '%s 00:00:00' AND '%s 23:59:59' 
                          AND"""  % (sugestao_capa.fissup_data_entrada,
                                     sugestao_capa.fissup_data_entrada)
        else:
            query_data_receb = ""
        if sugestao_capa.fissup_nf_id:
            query_nf = sugestao_capa.fissup_nf_id.id
        else:
            nf_ids = []
            cr.execute("""
                        SELECT nf.id
                        FROM fiscal_nf_capa nf
                        INNER JOIN fiscal_cfop_natureza natureza ON natureza.id = nf.natureza_id
                        INNER JOIN fiscal_cfop cfop ON cfop.id = natureza.cfop_id 
                        WHERE {query_data_receb} nf.nf_propria = false
                          AND nf.tipo = 'E'
                          AND nf.state = '3'
                          AND nf.company_id = {company_id}
                          AND cfop.movimento_tipo = 'C'
                     """.format(query_data_receb=query_data_receb, company_id=company_id))
            dados = cr.dictfetchall()
            if dados:
                nf_ids = map(lambda x: '%s' % x.get('id'), dados)

            query_nf = ','.join(nf_ids or ['-1'])

        sql = """
                SELECT emi.nf_id                  AS nf_id,
                       emi.quantidade             AS quantidade, 
                       fnc.numero_nf              AS numero_nf,
                       emi.produto_id             AS produto_id,
                       ep.codigo                  AS codigo,
                       epg.codigo_no_fabricante   AS cod_fabr,
                       emi.grade_id               AS grade_id,
                       epg.grade                  AS grade,
                       ep.name                    AS descricao,
                       eeu.id                     AS fisspi_estoque_embalagem_unidade_id,
                       ee.id                      AS embalagem_id,
                       ee.quantidade_na_embalagem AS embalagem
                FROM   estoque_movimento_itens       AS emi
                INNER JOIN fiscal_nf_capa            AS fnc ON fnc.id = emi.nf_id
                INNER JOIN estoque_produto           AS ep  ON ep.id = emi.produto_id
                INNER JOIN estoque_produto_grade     AS epg ON epg.id = emi.grade_id
                INNER JOIN estoque_produto_embalagem AS epe ON epe.id = emi.produto_embalagem_id
                INNER JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id
                INNER JOIN estoque_embalagem_unidade AS eeu ON eeu.id = ee.unidade_id
                WHERE  nf_id IN ({query_nf})
                ORDER  BY numero_nf, emi.id ASC;
                  """.format(query_nf=query_nf)
        cr.execute(sql)
        return cr.dictfetchall()

    def fisspi_get_total_sugestao(self, cr, uid, sugestao_capa, context):
        """
        Metodo responsavel por executar o sql de consulta dos dados referente ao filtro
        sugestao ou total
        :param cr: Cursor do banco de dados
        :param uid: id da tabela res_users, identificador do usuario.
        :param sugestao_capa: Browser de fiscal.sugestao.preco
        :param context: dados contextuais
        :type cr: Cursor
        :type uid: int
        :type sugestao_capa: Browse
        :type context: dict
        :rtype: dict
        """
        context = context and context.copy() or {}
        locacao_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.local.locacao')

        query_categoria = ""
        if sugestao_capa.fissup_categoria_id:
            query_categoria = """AND ep.categoria_id = %d""" % sugestao_capa.fissup_categoria_id.id

        query = """AND ep.active = TRUE"""
        if sugestao_capa.fissup_pessoa_id:
            query = """AND ep.fabricante_id = %s""" % sugestao_capa.fissup_pessoa_id.id

        select_local = """0                            AS embalagem_id,
                       ep.estpro_unidade_inventario AS unidade,
                       1.0                          AS qt_na_embalagem"""
        inner_embalagem = query_local = ""
        group_by = 'ep.estpro_unidade_inventario, embalagem_id, unidade'
        if sugestao_capa.fissup_local_id:
            inner_embalagem = """
                INNER JOIN estoque_produto_embalagem AS epe ON epe.id = ell.estllc_produto_embalagem_id
                INNER JOIN estoque_embalagem         AS ee  ON ee.id = epe.embalagem_id
                INNER JOIN estoque_embalagem_unidade AS eeu ON eeu.id = ee.unidade_id"""
            select_local = """
                       epe.embalagem_id           AS embalagem_id,
                       eeu.unidade                AS unidade,
                       ee.quantidade_na_embalagem AS qt_na_embalagem"""
            query_local = "AND ell.local_id = %s" % sugestao_capa.fissup_local_id.id
            group_by = 'eeu.unidade, ee.quantidade_na_embalagem, epe.embalagem_id'

        query_fornecedor = inner_fornecedor = ''
        if sugestao_capa.fissup_fornecedor_id:
            inner_fornecedor = 'INNER JOIN estoque_fornecedor    AS ef  ON ef.produto_id = ' \
                               'ell.produto_id'
            query_fornecedor = 'AND ef.fornecedor_id = %s' % sugestao_capa.fissup_fornecedor_id.id

        campo_kit, cond_kit, group = self._fisspi_get_sql_dados_kit(sugestao_capa)
        group_by += group
        sql = """
                SELECT ell.produto_id               AS produto_id,
                       ell.grade_id                 AS grade_id,
                       epg.grade                    AS grade,
                       ep.codigo                    AS codigo,
                       ep.margem_minima             AS margem_minima,
                       ep.margem_ideal              AS margem_ideal,
                       ep.margem_bruta              AS margem_bruta,
                       epg.codigo_no_fabricante     AS cod_fabr,
                       ep.name                      AS descricao,
                       {campo_kit}
                       {select_local}
                FROM  estoque_local_locacao      AS ell
                INNER JOIN estoque_produto       AS ep  ON ep.id = ell.produto_id
                INNER JOIN estoque_produto_grade AS epg ON epg.id = ell.grade_id
                {inner_fornecedor}
                {inner_embalagem}
                WHERE ell.company_id = {company_id}
                  {cond_kit}
                  {query}
                  {query_local}
                  {query_fornecedor}
                  {query_categoria}
                  AND epg.active = TRUE 
                GROUP  BY ell.produto_id,
                          ell.grade_id,
                          ep.codigo,
                          epg.grade,
                          ep.name,
                          epg.codigo_no_fabricante,
                          ep.margem_minima,
                          ep.margem_ideal,
                          ep.margem_bruta,
                          {group_by}
                ORDER  BY ep.codigo
        """.format(select_local=select_local,
                   query=query,
                   query_local=query_local,
                   query_categoria=query_categoria,
                   query_fornecedor=query_fornecedor,
                   campo_kit=campo_kit, cond_kit=cond_kit,
                   company_id=locacao_company_id,
                   inner_fornecedor=inner_fornecedor,
                   inner_embalagem=inner_embalagem,
                   group_by=group_by)
        cr.execute(sql)
        return cr.dictfetchall()

    def fisspi_copia_preco_venda(self, cr, uid, ids, context=None):
        """
        Metodo responsavel por copiar o preco de venda contido no registro e copia-lo
        para o campo fisspi_sugestao
        :param cr: Cursor do openerp
        :param uid: ID do usuario que esta executando esta funcionalidade
        :param ids: IDs dos registros
        :param context: Dicionario dos contextos
        :type cr: Cursor
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        for item in self.browse(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'fisspi_sugestao': item.fisspi_preco_venda}, context=context)
        return True

    def fisspi_atualiza_preco_item(self, cr, uid, ids, sugestao_item, context=None):
        """
        Metodo responsavel atualizar o preco de venda na classe 'estoque.preco.venda' do item,
        de acordo com a sugestao informado no registro
        :param cr: Cursor do openerp
        :param uid: ID do usuario que esta executando esta funcionalidade
        :param ids: IDs dos registros
        :param sugestao_item: Browse do registro
        :param context: Dicionario dos contextos
        :type cr: Cursor
        :type uid: int
        :type ids: list
        :type sugestao_item: Browse
        :type context: dict
        :rtype: int
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = False
        obj_est_preco_venda = self.pool.get('estoque.preco.venda')
        vals = dict(produto_id=sugestao_item.fisspi_produto_id.id,
                    preco_venda=sugestao_item.fisspi_sugestao)
        if sugestao_item.fisspi_sugestao > 0:
            res = obj_est_preco_venda.create(cr, uid, vals, context)
        return res

    @staticmethod
    def fisspi_get_valor_correcao(preco_venda, percentual):
        """
        Metodo responsavel por acrecentar um percentual no valor da sugestao do item
        de acordo com o perecentual inforado na capa da listagem
        :param preco_venda: Preco de venda do item
        :param percentual: Percentual a corrigir
        :type preco_venda: float
        :type percentual: float
        :rtype: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return preco_venda + (preco_venda * percentual / 100)

    @staticmethod
    def _fisspi_get_field_custo(tipo_custo):
        """
        Metodo responsavel por retornar o campo do custo que sera utilizado
        para consultas na classe estoque.preco.custo
        :param tipo_custo: Tipo do custo ('R' - Reposicao, 'C' - Corrigido)
        :type tipo_custo: str
        :rtype: str
        """
        custo = dict(R='estprc_custo_reposicao', C='estprc_custo_corrigido')
        return custo.get(tipo_custo)
