# -*- coding: utf-8 -*-


from openerp.osv import osv, fields


class fiscal_tributacao_entrada_encargos(osv.Model):
    _inherit = 'fiscal.tributacao.entrada.encargos'

    def onchange_encargo(self, cr, uid, ids, encargo_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'tributacao_id': False, 'cst_id': False}}
        return res

    def onchange_cst(self, cr, uid, ids, context=None):
        """
        Este metodo limpa o valor do atributo tributacao_id quando o cst e trocado
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: Lista de ids com os registros
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo o valor dp campo tributacao_id = False
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = {'value': {'tributacao_id': False}}
        return res

    def fteenc_get_cst_tributacao(self, cr, user, encargo, tributacao, context=None):
        """
        Retorna o CST e a tributacao em funcao do encargo (PIS, COFINS, ICMS, IPI) e da tributacao de
        entrada.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param encargo: o ID do encargo
        :param tributacao: o ID da tributacao de entrada (fiscal_tributacao_entrada)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type encargo: long
        :type tributacao: long
        :type context: dict
        :return: o ID do CST e o ID da tributacao.
        :rtype: tuple

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        tributacao_entrada_encargo_ids = encargo and tributacao and self.search(
            cr, user, [('encargo_id', '=', encargo), ('tributacao_entrada_id', '=', tributacao)],
            context=context)
        if tributacao_entrada_encargo_ids:
            dados_tributacao = self.read(
                cr, user, tributacao_entrada_encargo_ids[0], ['cst_id', 'tributacao_id'], context,
                '_classic_write')
            cst_id, tributacao_id = \
                dados_tributacao.get('cst_id'), dados_tributacao.get('tributacao_id')
        else:
            cst_id = tributacao_id = False

        return cst_id, tributacao_id

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
