#!/usr/bin/python
# -*- coding: utf-8 -*-
from openerp.osv import osv


class fiscal_tributacao_interestadual(osv.Model):
    """
    Cadastro de tributacoes interestaduais

    :prefixo: fistin
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    _name = 'fiscal.tributacao.interestadual'
    _inherit = 'fiscal.tributacao.interestadual'

    def fistin_get_tributacao_interestadual(self, cr, uid, uf_id, encargo_id, produto_nacional,
                                            context=None):
        """
        Este metodo retorna a aliquota, base de calculo e browse's de cst e tributacao
        cadastradas na tabela de tributacao interestadual de acordo com a uf_id e a origem do
        produto que pode ser importada ou nacional
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param uf_id: Id do estado a consultar
        :param encargo_id: Id do encargo a consultar
        :param produto_nacional: informa se o produto e nacional ou importado
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type uf_id: int
        :type encargo_id: id
        :type produto_nacional: bool
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo a base e aliquota cadastradas para calculo
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if produto_nacional:
            return self.fistin_get_tributacao_interestadual_nacional(
                cr, uid, uf_id, encargo_id, context)
        else:
            return self.fistin_get_tributacao_interestadual_importado(
                cr, uid, uf_id, encargo_id, context)

    def fistin_get_tributacao_interestadual_nacional(self, cr, uid, uf_id, encargo_id,
                                                     context=None):
        """
        Este metodo retorna a aliquota, base de calculo e browse's de cst e tributacao
        cadastradas na tabela de tributacao interestadual nacional de acordo com a uf_id
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param uf_id: Id do estado a consultar
        :param encargo_id: Id do encargo a consultar
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type uf_id: int
        :type encargo_id: id
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo a base e aliquota cadastradas para calculo
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        args = [('fistin_uf', '=', uf_id), ('fistin_encargo', '=', encargo_id)]
        ids = self.search(cr, uid, args, context=context)
        vals = {}

        if not ids:
            return vals

        campos = ['fistin_cod_tribut_interestadual_nac']
        dados = self.read(cr, uid, ids, campos, context, load)
        tributacao_ids = map(lambda x: x.get('fistin_cod_tribut_interestadual_nac') or 0, dados)
        campos = ['base', 'aliquota', 'cst_id', 'fistri_aliquota_fcp', 'fistri_icms_desonerado']
        dict_tributacao = {}
        map(lambda x: dict_tributacao.update({x.get('id'): x}),
            self.pool.get('fiscal.tributacao').read(cr, uid, tributacao_ids, campos, context, load))
        for i in filter(lambda f: f.get('fistin_cod_tribut_interestadual_nac'), dados):
            trib = dict_tributacao.get(i.get('fistin_cod_tribut_interestadual_nac')) or {}
            base = trib.get('base')
            aliquota = trib.get('aliquota')
            aliquota_fcp = trib.get('fistri_aliquota_fcp')
            cst = trib.get('cst_id')
            tributacao = trib.get('id')
            vals.update({'base': base, 'aliquota': aliquota, 'cst': cst, 'tributacao': tributacao,
                         'aliquota_fcp': aliquota_fcp,
                         'fistri_icms_desonerado': trib.get('fistri_icms_desonerado') or 0})

        return vals

    def fistin_get_tributacao_interestadual_importado(self, cr, uid, uf_id, encargo_id,
                                                      context=None):
        """
        Este metodo retorna a aliquota, base de calculo e browse's de cst e tributacao
        cadastradas na tabela de tributacao interestadual importada de acordo com a uf_id
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param uf_id: Id do estado a consultar
        :param encargo_id: Id do encargo a consultar
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type uf_id: int
        :type encargo_id: id
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo a base e aliquota cadastradas para calculo
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        args = [('fistin_uf', '=', uf_id), ('fistin_encargo', '=', encargo_id)]
        ids = self.search(cr, uid, args, context=context)
        vals = {}
        if not ids:
            return vals

        campos = ['fistin_cod_tribut_interestadual_imp']
        dados = self.read(cr, uid, ids, campos, context, load)
        dados = filter(lambda x: x.get('fistin_cod_tribut_interestadual_imp'), dados)
        tributacao_ids = map(lambda x: x.get('fistin_cod_tribut_interestadual_imp') or 0, dados)
        if not tributacao_ids:
            return vals
        campos = ['base', 'aliquota', 'cst_id']
        dict_tributacao = {}
        map(lambda x: dict_tributacao.update({x.get('id'): x}),
            self.pool.get('fiscal.tributacao').read(cr, uid, tributacao_ids, campos, context, load))
        for i in filter(lambda f: f.get('fistin_cod_tribut_interestadual_imp'), dados):
            trib = dict_tributacao.get(i.get('fistin_cod_tribut_interestadual_imp'))
            base = trib.get('base')
            aliquota = trib.get('aliquota')
            cst = trib.get('cst_id')
            tributacao = trib.get('id')
            vals.update({'base': base, 'aliquota': aliquota, 'cst': cst, 'tributacao': tributacao})

        return vals

    def fistin_get_tributacao_interna(self, cr, uid, uf_id, encargo_id, produto_nacional, context=None):
        """
        Este metodo retorna a aliquota, base de calculo e browse's de cst e tributacao
        cadastradas na tabela de tributacao interna de acordo com a uf_id
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param uf_id: Id do estado a consultar
        :param encargo_id: Id do encargo a consultar
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type uf_id: int
        :type encargo_id: id
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo a base e aliquota cadastradas para calculo
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        args = [('fistin_uf', '=', uf_id), ('fistin_encargo', '=', encargo_id)]
        ids = self.search(cr, uid, args, context=context)
        context = context and context.copy() or {}
        vals = {}
        if not ids:
            return vals
        for brw in filter(lambda f: f.fistin_cod_tribut_interno, self.browse(cr, uid, ids, context=context)):
            base = brw.fistin_cod_tribut_interno.base
            aliquota = brw.fistin_cod_tribut_interno.aliquota
            if produto_nacional:
                aliquota_inter = brw.fistin_cod_tribut_interestadual_nac.aliquota
            else:
                aliquota_inter = brw.fistin_cod_tribut_interestadual_imp.aliquota
            aliquota_fcp = brw.fistin_cod_tribut_interno.fistri_aliquota_fcp
            cst = brw.fistin_cod_tribut_interno.cst_id.id
            tributacao = brw.fistin_cod_tribut_interno.id
            base_dupla = brw.fistin_difal_base_dupla
            vals.update({'base': base, 'aliquota': aliquota, 'cst': cst, 'tributacao': tributacao,
                         'aliquota_fcp': aliquota_fcp, 'base_dupla': base_dupla, 'aliquota_inter': aliquota_inter})

        return vals

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
