# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _


class fiscal_tributacao_simples_nacional(osv.Model):
    _inherit = 'fiscal.tributacao.simples.nacional'

    def fistsn_calcula_aliquota_imposto(self, cr, uid, produto_ids, context=None):
        """
        O metodo calcula o total da aliquota dos impostos a ser aplicada sobre o valor da venda,
        para cada produto recebido.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_ids: lista de ids da tabela estoque_produto, identificadores dos produtos.
        :type produto_ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: dados da identificacao do produto em conjunto a aliquota a ser aplicada.
        :rtype: dict
        """
        res = {}
        if not produto_ids:
            return res
        receita_anual = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', 'fispar_receita_anual', context=context) or {}
        receita_anual = receita_anual and receita_anual.get('fispar_receita_anual') or 0.0
        if receita_anual <= 0:
            raise osv.except_osv(_('Atencao'), _('Cadastre o valor da receita anual nos parametros '
                                                 'do fiscal.'))
        simples_nacional_ids = self.search(
            cr, uid, [('fistsn_valor_faturamento', '>=', receita_anual)],
            order='fistsn_valor_faturamento', limit=1, context=context)
        if not simples_nacional_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao existe tributacao cadastrada para "Simples '
                                   'Nacional" que se esteja na faixa de receita anual da '
                                   'empresa. Verifique o parametro do Fiscal.'))
        sql = """
            SELECT
                produto.id				                AS produto_id,
                COALESCE((
                    SELECT sum(
                        CASE
                            WHEN cst.tipo = 'T' AND encargo.sigla = 'ICMS'
                                THEN COALESCE(simples.fistsn_aliq_icms, 0)
                            ELSE 0
                        END +
                        CASE
                            WHEN cst.tipo = 'T' AND encargo.sigla = 'PIS'
                                THEN COALESCE(simples.fistsn_aliq_pis, 0)
                            ELSE 0
                        END +
                        CASE
                            WHEN cst.tipo = 'T' AND encargo.sigla = 'COFINS'
                                THEN COALESCE(simples.fistsn_aliq_cofins, 0)
                            ELSE 0
                        END)
                    FROM
                        fiscal_tributacao_simples_nacional simples1
                    LEFT JOIN
                        fiscal_tributacao_encargos tributacao ON produto.id = tributacao.produto_id
                    LEFT JOIN
                        fiscal_cst cst ON cst.id = tributacao.cst_id
                    LEFT JOIN
                        fiscal_encargo encargo ON encargo.id = tributacao.encargo_id
                    WHERE
                        simples1.id = simples.id
                        AND tributacao.tipo_tributacao != 'E'
                ), 0) +
                COALESCE(simples.fistsn_aliq_irpj, 0) +
                COALESCE(simples.fistsn_aliq_csll, 0) + 
                COALESCE(simples.fistsn_aliq_inss, 0)	AS vr_aliquota
            FROM
                estoque_produto produto
            LEFT JOIN
                fiscal_tributacao_simples_nacional simples ON simples.id = {simples_id}
            WHERE
                produto.id IN ({produto_ids})
            ORDER BY produto.id
        """.format(simples_id=simples_nacional_ids[0], produto_ids=', '.join(map(
            lambda i: str(i), produto_ids)))
        cr.execute(sql)
        dados = cr.dictfetchall()
        for dado in dados:
            res.update({dado.get('produto_id'): dado.get('vr_aliquota') or 0})
        return res
