# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe import __version__ as VersionNfe
from pynfe.nfe.manual_600.nfe_400 import NFe, NFRef, Dup, AutXML, Vol, Det, DetPag, DI, Rastro
from pynfe.nfe.manual_500.nfe_310 import Adi
from pynfe.xml_sped.__init__ import UF_CODIGO
from SigUtils.numero import getNumeroEditado
from pynfe.nfce.manual_600.nfce_p import Det as NFCeDet
# noinspection PyProtectedMember
from openerp.tools.translate import _
from unicodedata import normalize
from pynfe.nfce.manual_600.nfce_p import NFCe
from datetime import datetime
from openerp.osv import osv
from decimal import Decimal
import itertools
import hashlib
import locale
import re
TWOPLACES = Decimal(10) ** -2
TREEPLACES = Decimal(10) ** -3


# noinspection PyPep8Naming
class fiscal_nf_capa(osv.Model):
    def perm_write(self, cr, user, ids, fields, context=None):
        pass  # Este metodo nao existe mais

    _inherit = 'fiscal.nf.capa'

    def __init__(self, *args):
        super(fiscal_nf_capa, self).__init__(*args)
        self.total_aproveitamento_credito = 0

    def gerar_nfe_xml(self, cr, user, nf, vals, context=None):
        """
        Gera o XML da nota fiscal eletronica (saida)
        
        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: o XML com os valores da NF-e
        :rtype: basestring
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        load = '_classic_write'
        campos = ['company_id', 'natureza_id', 'modelo_nf_series_id', 'serie_nf', 'numero_nf',
                  'data_emissao', 'data_saida_entrada', 'tipo', 'nfe_forma_impressao_danfe',
                  'finalidade_emissao', 'destinatario_id', 'fisnfc_nf_referenciada_ids',
                  'fisnfc_infcomp_fisco', 'fisnfc_infcomp_contribuinte', 'vr_seguro', 'vr_frete',
                  'destinatario_endereco_id', 'nfe_chave_acesso', 'serie_nf', 'nf_condicao_ids',
                  'fisnfc_uf_saida_pais', 'fisnfc_local_exportacao', 'fisnfc_local_despacho',
                  'frete', 'transportador_id', 'fisnfc_placa_frete', 'quantidade_volume', 'especie',
                  'marca', 'numero_volume', 'peso_bruto', 'peso_liquido', 'vr_total_nf',
                  'vr_desconto', 'fisnfc_nf_referencia_ids', 'estoque_movimento_itens_ids',
                  'nf_parcelas_ids', 'fisnfc_numero_nota_empenho', 'fisnfc_nfe_indicador_presenca',
                  'fisnfc_cupons_referenciados_ids', 'fisnfc_consumidor', 'vr_seguro',
                  'modelo_nf_id', 'fisnfc_di_ids', 'nf_propria', 'fisnfc_nfe_indintermed',
                  'fisnfc_nfe_intermediador_cnpj', 'fisnfc_nfe_intermediador_identificador', 'nfe_forma_emissao']
        dados_nf = self.read(cr, user, nf, campos, context, load)
        company_pessoa_id = self.pool.get('res.company').read(
            cr, user, dados_nf.get('company_id'), ['rescon_pessoa_id'], context,
            load).get('rescon_pessoa_id')
        context['super_read_adm_pessoa'] = True
        campos = ['cpf_cnpj', 'endereco_ids']
        dados_company_pessoa = self.pool.get('adm.pessoa').read(
            cr, user, company_pessoa_id, campos, context)
        dict_end_pessoa_company = {}
        end_tipo_ids = []
        uf_ids = []
        campos = ['endereco_tipo_id', 'uf_id']
        for endereco in self.pool.get('adm.pessoa.endereco').read(
                cr, user, dados_company_pessoa.get('endereco_ids'), campos, context, load):
            dict_end_pessoa_company.update({endereco.get('id'): endereco})
            end_tipo_ids.append(endereco.get('endereco_tipo_id'))
            uf_ids.append(endereco.get('uf_id'))
        dict_tipo_edereco = {}
        map(lambda x: dict_tipo_edereco.update({x.get('id'): x}),
            self.pool.get('adm.endereco.tipo').read(cr, user, end_tipo_ids, ['name'], context))
        dict_uf = {}
        map(lambda x: dict_uf.update({x.get('id'): x}),
            self.pool.get('adm.uf').read(cr, user, uf_ids, ['sigla'], context))
        if dados_nf.get('finalidade_emissao') == '2':  # Nota complementar
            vals['fisnfc_nfe_indicador_presenca'] = '0'

        # NF-e/NFC-e
        nfe = NFCe() if context.get('nota_fiscal_consumidor') else NFe()
        nfe.infNFe.Id.valor = 'NFe%s' % (dados_nf.get('nfe_chave_acesso'))
        nfe.infNFe.infRespTec.hashCSRT.valor = parametro.fiscal.fispar_nfe_csrt and hashlib.sha1(
            parametro.fiscal.fispar_nfe_csrt + dados_nf.get('nfe_chave_acesso')).digest().encode(
            'base64').replace('\n', u'') or u''
        nfe.Signature.URI = nfe.infNFe.Id.valor

        # Motivo da desoneracao de ICMS (natureza da operacao)
        self._motivo_desoneracao = \
            dados_nf.get('natureza_id') and self.pool.get('fiscal.cfop.natureza').read(
                cr, user, dados_nf.get('natureza_id'), ['fisnat_icms_desonerado_motivo'],
                context).get('fisnat_icms_desonerado_motivo') or False
        nfe.infNFe.compra.xNEmp.valor = dados_nf.get('fisnfc_numero_nota_empenho') or u''
        nfe.infNFe.compra.xPed.valor = u''  # Pedido
        nfe.infNFe.compra.xCont.valor = u''  # Contrato de compra

        self._fisnfc_set_nfe(cr, user, dados_nf, nfe, context)
        if nfe.infNFe.ide.mod.valor == 55 and dados_nf.get('data_saida_entrada'):
            if dados_nf.get('data_saida_entrada') < dados_nf.get('data_emissao'):
                vals['data_saida_entrada'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        else:
            vals['data_saida_entrada'] = False  # Nao consta como atributo de NFC-e
        return normalize('NFKD', nfe.xml).encode('ASCII', 'ignore')

    def _fisnfc_set_nfe(self, cr, user, dados_nf, nfe, context=None):
        context = context and context.copy() or {}
        self._fisnfc_set_nfe_infnfe(cr, user, dados_nf, nfe, context)  # Nota fiscal eletronica

    def _fisnfc_set_nfe_infnfe(self, cr, user, dados_nf, nfe, context=None):
        context = context and context.copy() or {}
        load = '_classic_write'
        obj_pessoa = self.pool.get('adm.pessoa')
        campos = ['rescon_pessoa_id']
        dados_company = self.pool.get('res.company').read(
            cr, user, dados_nf.get('company_id'), campos, context, load)
        dados_natureza = self.pool.get('fiscal.cfop.natureza').read(
            cr, user, dados_nf.get('natureza_id'), ['name', 'cfop_id', 'fisnat_ir_retencao',
                                                    'fisnat_ir_aliquota'], context, load)
        dados_cfop_nat = self.pool.get('fiscal.cfop').read(
            cr, user, dados_natureza.get('cfop_id'), ['codigo', 'movimento_tipo'], context)
        context['super_read_adm_pessoa'] = True
        campos = ['endereco_nf_id', 'endereco_ids', 'admpes_jur_regime_tributario', 'telefone',
                  'documentos_ids', 'fantasia_apelido', 'admpes_jur_ie', 'name', 'pessoa_tipo',
                  'admpes_jur_cnae_id', 'admpes_jur_tipo_atividade', 'cpf_cnpj']
        dados_pessoa_company = obj_pessoa.read(
            cr, user, dados_company.get('rescon_pessoa_id'), campos, context, load)

        dict_condicoes = {}
        obj_adm_condicao_forma = self.pool.get('adm.condicao.forma')
        obj_condicao_pag = self.pool.get('fiscal.condicao.pagamento')
        condicao_forma_ids = []
        forma_pgto_ids = []
        campos = ['condicao_forma_id', 'fiscpg_nro_autorizacao_cartao', 'forma_pagto_id', 'valor',
                  'fiscpg_nro_autorizacao_cartao']
        for i in obj_condicao_pag.read(
                cr, user, dados_nf.get('nf_condicao_ids'), campos, context, load):
            dict_condicoes[i.get('id')] = i
            i.get('condicao_forma_id') and condicao_forma_ids.append(i.get('condicao_forma_id'))
            i.get('forma_pagto_id') and forma_pgto_ids.append(i.get('forma_pagto_id'))

        dict_forma_pgto = {}
        campos = ['admfpg_tipo', 'admfpg_cartao_pessoa_id', 'name']
        cartao_pessoa_ids = []
        for i in self.pool.get('adm.forma.pgto').read(
                cr, user, forma_pgto_ids, campos, context, load):
            dict_forma_pgto[i.get('id')] = i
            if i.get('admfpg_cartao_pessoa_id'):
                cartao_pessoa_ids.append(i.get('admfpg_cartao_pessoa_id'))
        dict_pessoas_cartao = {}
        if cartao_pessoa_ids:
            map(lambda x: dict_pessoas_cartao.update({x.get('id'): x}),
                obj_pessoa.read(cr, user, cartao_pessoa_ids, ['cpf_cnpj'], context))
        condicao_ids = []
        dict_cond_forma = {}
        campos = ['forma_pagamento_id', 'tipo_finalizadora', 'adm_condicao_id']
        for i in obj_adm_condicao_forma.read(cr, user, condicao_forma_ids, campos, context, load):
            dict_cond_forma[i.get('id')] = i
            condicao_ids.append(i.get('adm_condicao_id'))
        campos = ['tipo', 'cartao_credito_debito']
        tipo_condicoes = []
        for i in self.pool.get('adm.condicao').read(cr, user, condicao_ids, campos, context):
            tipo_condicoes.append(i.get('tipo'))
        endereco_id = dados_pessoa_company.get('endereco_nf_id') or \
                      dados_pessoa_company.get('endereco_ids')[0]
        dados_endereco = {}
        dados_uf = {}
        dados_cidade = {}
        if endereco_id:
            campos = ['uf_id', 'cidade_id', 'logradouro_id', 'numero', 'complemento', 'bairro_id',
                      'cep', 'pais_id']
            dados_endereco = self.pool.get('adm.pessoa.endereco').read(
                cr, user, endereco_id, campos, context, load)
            if dados_endereco.get('uf_id'):
                dados_uf = self.pool.get('adm.uf').read(
                    cr, user, dados_endereco.get('uf_id'), ['sigla'], context)
            campos = ['codigo_ibge', 'name', 'pais_id']
            if dados_endereco.get('cidade_id'):
                dados_cidade = self.pool.get('adm.cidade').read(
                    cr, user, dados_endereco.get('cidade_id'), campos, context, load)
        telefones = dados_nf.get('destinatario_id') and obj_pessoa.get_telefones(
            cr, user, [dados_nf.get('destinatario_id')], context) or {}
        campos = ['cidade_id', 'logradouro_id', 'numero', 'bairro_id', 'uf_id', 'complemento',
                  'cep', 'pais_id']
        end_dest = \
            dados_nf.get('destinatario_endereco_id') and self.pool.get('adm.pessoa.endereco').read(
                cr, user, dados_nf.get('destinatario_endereco_id'), campos, context, load) or {}
        if end_dest:
            end_dest.update({'telefone': telefones.get(dados_nf.get('destinatario_id')) or {}})

        # Identificacao da nota
        self._fisnfc_set_nfe_infnfe_ide(
            cr, user, dados_nf, dados_natureza, dados_cfop_nat, dados_cidade, dados_uf, nfe,
            context)

        # Dados do emitente
        self._fisnfc_set_nfe_infnfe_emit(
            cr, user, dados_pessoa_company, dados_endereco, dados_cidade, dados_uf, nfe, end_dest, context)

        # Dados do destinatario
        self._fisnfc_set_nfe_infnfe_dest(cr, user, dados_nf, end_dest, dados_cfop_nat, nfe, context)

        # Autorizados download XML
        self._fisnfc_set_nfe_infnfe_autxml(cr, user, nfe, context)

        # Dados de pagamento
        self._fisnfc_set_nfe_infnfe_pag(
            cr, user, dados_nf, dict_condicoes, dict_forma_pgto, dict_pessoas_cartao,
            dict_cond_forma, nfe, context)

        # NF de exportacao
        self._fisnfc_set_nfe_infnfe_exporta(cr, user, dados_nf, dados_cfop_nat, nfe, context)

        # Dados da transportadora
        self._fisnfc_set_nfe_infnfe_transp(cr, user, dados_nf, nfe, context)

        # Faturamento
        if not context.get('nota_fiscal_consumidor'):
            self._fisnfc_set_nfe_infnfe_cobr_fat(
                cr, user, dados_nf, dict_condicoes, dict_forma_pgto, nfe, context)

        # Itens da NF
        dados_nf['estoque_movimento_itens_ids'] = sorted(dados_nf.get('estoque_movimento_itens_ids'))
        self._fisnfc_set_nfe_infnfe_prod(
            cr, user, dados_nf, end_dest, dados_pessoa_company, dados_cfop_nat, dict_condicoes,
            dict_cond_forma, nfe, context)

        # Total produtos
        self._fisnfc_set_nfe_infnfe_total_icmstot(cr, user, dados_nf, nfe, dados_natureza, context)

        nfe.infNFe.infAdic.infCpl.valor = nfe.infNFe.infAdic.infCpl.valor

    def _fisnfc_set_nfe_infnfe_ide(self, cr, user, dados_nf, dados_natureza, dados_cfop_nat,
                                   dados_cidade, dados_uf, nfe, context=None):
        # //NFe.infNFe.Ide - Identificacao da NF-e
        load = '_classic_write'
        campos = ['modelo_nf_id', 'tipo_contingencia', 'motivo_scan', 'data_scan', 'ambiente']
        dados_modelo_serie = self.pool.get('fiscal.modelo.nf.series').read(
            cr, user, dados_nf.get('modelo_nf_series_id') or 0, campos, context, load) or {}
        modelo_nf_id = dados_modelo_serie.get('modelo_nf_id') or dados_nf.get('modelo_nf_id') or 0
        campos = ['fismnf_codigo']
        dados_modelo = self.pool.get('fiscal.modelo.nf').read(
            cr, user, modelo_nf_id, campos, context)
        uf_sigla = dados_uf.get('sigla') or ''
        codigo_ibge = dados_cidade.get('codigo_ibge')
        nfe.infNFe.ide.cUF.valor = int(nfe.chave[:2] or UF_CODIGO.get(uf_sigla) or False)
        nfe.infNFe.ide.cNF.valor = nfe.chave[-9:-1] or str(dados_nf.get('id'))[:8].zfill(8)
        nfe.infNFe.ide.natOp.valor = dados_natureza.get('name')[:60]
        nfe.infNFe.ide.mod.valor = int(nfe.chave[20:22] or dados_modelo.get('fismnf_codigo'))
        nfe.infNFe.ide.serie.valor = int(nfe.chave[22:25] or dados_nf.get('serie_nf'))
        nfe.infNFe.ide.nNF.valor = int(nfe.chave[25:34] or dados_nf.get('numero_nf'))
        nfe.infNFe.ide.procEmi.valor = 0  # 0=Emissao de NF-e com aplicativo do contribuinte;
        nfe.infNFe.ide.dhEmi.valor = datetime.strptime(
            self.pool.get('adm.utils').admutl_datetime_to_timezone(
                cr, user, datetime.strptime(dados_nf.get('data_emissao'), '%Y-%m-%d %H:%M:%S'),
                '%d-%m-%Y %T'), '%d-%m-%Y %H:%M:%S')
        if dados_nf.get('fisnfc_nfe_indicador_presenca'):
            if dados_nf.get('finalidade_emissao') != '2':
                nfe.infNFe.ide.indPres.valor = dados_nf.get('fisnfc_nfe_indicador_presenca') or '0'
                if str(nfe.infNFe.ide.indPres.valor) in '2349':
                    nfe.infNFe.ide.indIntermed.valor = dados_nf.get('fisnfc_nfe_indintermed')
            else:
                nfe.infNFe.ide.indPres.valor = '0'
                nfe.infNFe.ide.indIntermed.valor = '0'
        else:
            if context.get('nota_fiscal_consumidor'):
                nfe.infNFe.ide.indPres.valor = '1'
        if not dados_nf.get('data_saida_entrada') or \
                        dados_nf.get('data_saida_entrada') < dados_nf.get('data_emissao'):
            dados_nf['data_saida_entrada'] = dados_nf.get('data_emissao')
        if nfe.infNFe.ide.mod.valor == 55 and dados_nf.get('data_saida_entrada'):
            nfe.infNFe.ide.dhSaiEnt.valor = datetime.strptime(
                self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, user, datetime.strptime(
                    dados_nf.get('data_saida_entrada')[:19], '%Y-%m-%d %H:%M:%S'), '%d-%m-%Y %T'),
                '%d-%m-%Y %H:%M:%S')
        nfe.infNFe.ide.tpNF.valor = 0 if dados_nf.get('tipo') == 'E' else 1
        nfe.infNFe.ide.idDest.valor = \
            {'5': 1, '1': 1, '6': 2, '2': 2, '7': 3, '3': 3}.get(dados_cfop_nat.get('codigo')[0])
        self._operacao_interestadual = nfe.infNFe.ide.idDest.valor == 2  # Operacao interestadual?
        nfe.infNFe.ide.cMunFG.valor = codigo_ibge
        nfe.infNFe.ide.tpImp.valor = int(dados_nf.get('nfe_forma_impressao_danfe'))
        # nfe.infNFe.ide.verProc.valor = nfe.arquivo_esquema.split('.')[0]
        nfe.infNFe.ide.verProc.valor = 'PyNFe %s' % VersionNfe
        if  nfe.infNFe.ide.mod.valor == 55 and dados_modelo_serie.get('tipo_contingencia'):
            nfe.infNFe.ide.tpEmis.valor = {
                'SVC-AN': 6, 'SVC-RS': 7}.get(dados_modelo_serie.get('tipo_contingencia')) or 1
            nfe.infNFe.ide.xJust.valor = dados_modelo_serie.get('motivo_scan')
            nfe.infNFe.ide.dhCont.valor = \
                datetime.strptime(self.pool.get('adm.utils').admutl_datetime_to_timezone(
                    cr, user, datetime.strptime(
                        dados_modelo_serie.get('data_scan'), '%Y-%m-%d %H:%M:%S'), '%d-%m-%Y %T'),
                    '%d-%m-%Y %H:%M:%S')
        if dados_nf.get('nfe_forma_emissao') == '2' and nfe.infNFe.ide.mod.valor == 65:  # NFC-e contingencia offline
            nfe.infNFe.ide.xJust.valor = 'Sem conexao com a SEFAZ'
            nfe.infNFe.ide.dhCont.valor = nfe.infNFe.ide.dhEmi.valor
            nfe.infNFe.ide.tpEmis.valor = 9

        nfe.infNFe.ide.cDV.valor = int(nfe.infNFe.Id.valor[-1])
        nfe.infNFe.ide.tpAmb.valor = dados_modelo_serie.get('ambiente')
        nfe.infNFe.ide.indFinal.valor = u'1' if self.pool.get(
            'adm.pessoa').admpes_get_tipo_consumidor(
            cr, user, dados_nf.get('destinatario_id'), context=None) == 'F' else u'0'
        self._consumidor_final = nfe.infNFe.ide.indFinal.valor == u'1'  # Consumidor final
        nfe.infNFe.ide.finNFe.valor = int(dados_nf.get('finalidade_emissao'))
        self._fisnfc_set_nfe_infnfe_ide_nref(cr, user, dados_nf, nfe, context)
        self._fisnfc_set_nfe_infnfe_infintermed(cr, user, dados_nf, nfe, context)

    def _fisnfc_set_nfe_infnfe_infintermed(self, cr, user, dados_nf, nfe, context=None):
        # NFe.infNFe.infIntermed
        if nfe.infNFe.ide.indIntermed.valor == '1':
            nfe.infNFe.infIntermed.CNPJ.valor = re.sub(
                    '[^0-9]', u'', dados_nf.get('fisnfc_nfe_intermediador_cnpj') or u'')
            nfe.infNFe.infIntermed.idCadIntTran.valor = \
                dados_nf.get('fisnfc_nfe_intermediador_identificador')

    def _fisnfc_set_nfe_infnfe_ide_nref(self, cr, user, dados_nf, nfe, context=None):
        # //NFe.infNFe.Ide.NFref - Notas fiscais referenciadas
        load = '_classic_write'
        campos = ['modelo_55', 'nfe_chave_acesso', 'company_id', 'data_emissao', 'tipo',
                  'modelo_nf_id', 'modelo_nf_series_id', 'serie_nf', 'numero_nf',]
        if dados_nf.get('fisnfc_nf_referenciada_ids'):
            for nota in self.read(cr, user, dados_nf.get('fisnfc_nf_referenciada_ids'), campos, context, load):
                campos = ['rescon_pessoa_id']
                dados_company = self.pool.get('res.company').read(
                    cr, user, dados_nf.get('company_id'), campos, context, load)
                context['super_read_adm_pessoa'] = True
                campos = ['endereco_ids', 'cpf_cnpj']
                dados_pessoa_company = self.pool.get('adm.pessoa').read(
                    cr, user, dados_company.get('rescon_pessoa_id'), campos, context, load)
                nf_ref = NFRef()
                if nota.get('nfe_chave_acesso'):  # Identifica uma NF-e
                    nf_ref.refNFe.valor = nota.get('nfe_chave_acesso')
                else:
                    data_emissao = nota.get('data_emissao')
                    sigla = ''
                    if dados_pessoa_company.get('endereco_ids'):
                        dados_endereco = self.pool.get('adm.pessoa.endereco').read(
                            cr, user, dados_pessoa_company.get('endereco_ids')[0], ['uf_id'],
                            context, load)
                    if dados_endereco.get('uf_id'):
                        dados_uf = self.pool.get('adm.uf').read(
                            cr, user, dados_endereco.get('uf_id'), ['sigla'], context)
                        sigla = dados_uf.get('sigla') or ''
                    uf_emit_ref = sigla
                    nf_ref.refNF.cUF.valor = uf_emit_ref and UF_CODIGO[uf_emit_ref] or u''
                    nf_ref.refNF.AAMM.valor = data_emissao[2:4] + data_emissao[5:7]
                    nf_ref.refNF.CNPJ.valor = \
                        re.sub('[^0-9]', u'', dados_pessoa_company.get('cpf_cnpj'))
                    nf_ref.refNF.mod.valor = u'01'
                    nf_ref.refNF.serie.valor = nota.get('serie_nf') or u''
                    nf_ref.refNF.nNF.valor = nota.get('numero_nf') or u''
                nfe.infNFe.ide.NFref.append(nf_ref)
        if dados_nf.get('fisnfc_infcomp_contribuinte'):
            nfe.infNFe.infAdic.infCpl.valor += dados_nf.get('fisnfc_infcomp_contribuinte')
        nfe.infNFe.infAdic.infAdFisco.valor = dados_nf.get('fisnfc_infcomp_fisco') or u''

    def _fisnfc_set_nfe_infnfe_emit(self, cr, user, dados_emit, dados_endereco, dados_cidade,
                                    dados_uf, nfe, end_dest, context=None):
        # //NFe.infNFe.emit - Dados do emitente
        if dados_emit.get('pessoa_tipo') == u'F':
            nfe.infNFe.emit.CPF.valor = re.sub('[^0-9]', u'', dados_emit.get('cpf_cnpj') or u'')
        else:
            nfe.infNFe.emit.CNPJ.valor = re.sub('[^0-9]', u'', dados_emit.get('cpf_cnpj') or u'')
        xNome = dados_emit.get('name')
        xNome = xNome or dados_emit.get('fantasia_apelido')
        xNome = xNome.replace(u'\u2013', '-')
        nfe.infNFe.emit.xNome.valor = xNome[:60]
        nfe.infNFe.emit.IE.valor = re.sub('[^0-9A-Z]', u'', dados_emit.get('admpes_jur_ie') or u'')
        nfe.infNFe.emit.xFant.valor = dados_emit.get('fantasia_apelido') or u''
        dados_uf_dest = end_dest.get('uf_id') and self.pool.get('adm.uf').read(
            cr, user, end_dest.get('uf_id'), ['sigla'], context) or {}
        uf_dest = dados_uf_dest.get('sigla') or u''
        for documento in dados_emit.get('documentos_ids'):
            documento_dados = self.pool.get('adm.pessoa.documento.dados').read(
                cr, user, documento, ['tipo_id', 'numero'], context, '_classic_write')
            tipo_id = documento_dados.get('tipo_id')
            sigla = tipo_id and self.pool.get('adm.pessoa.documento.tipo').read(
                cr, user, tipo_id, ['sigla'], context).get('sigla')
            if sigla == 'IM':
                nfe.infNFe.emit.IM.valor = documento_dados.get('numero') or u''
            if sigla == 'IEST' and documento_dados.get('emitente').upper() == uf_dest:
                nfe.infNFe.emit.IEST.valor = documento_dados.get('numero') or 'u'
        nfe.infNFe.emit.enderEmit.fone.valor = re.sub('[^0-9]', '', str(dados_emit.get('telefone')) or '')

        fiscal_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'fiscal.parametro', ['fispar_receita_anual', 'fispar_sublimite_estadual'], context=context)

        if dados_emit.get('admpes_jur_regime_tributario') == 'S':
            if fiscal_parametro_dados.get('fispar_receita_anual') < fiscal_parametro_dados.get('fispar_sublimite_estadual'):
                nfe.infNFe.emit.CRT.valor = 1
            else:
                nfe.infNFe.emit.CRT.valor = 2
        else:
            nfe.infNFe.emit.CRT.valor = 3
        # nfe.infNFe.emit.CRT.valor = dados_emit.get('admpes_jur_regime_tributario') == 'S' and 1 or 3
        if dados_emit.get('admpes_jur_cnae_id'):
            cnae_codigo = self.pool.get('adm.cnae').read(
                cr, user, dados_emit.get('admpes_jur_cnae_id'), ['admcna_codigo'], context) or u''
            cnae_codigo = cnae_codigo and cnae_codigo[0].get('admcna_codigo') or u''
            nfe.infNFe.emit.CNAE.valor = nfe.infNFe.emit.CRT.valor == 3 and re.sub(
                '[^0-9]', u'', cnae_codigo) or u''
        self._fisnfc_set_nfe_infnfe_emit_enderemit(
            cr, user, dados_endereco, dados_cidade, dados_uf, nfe, context)

    def _fisnfc_set_nfe_infnfe_emit_enderemit(self, cr, user, dados_end, dados_cidade, dados_uf,
                                              nfe, context=None):
        # //NFe.infNFe.emit.enderEmit - Endereco do emitente
        if not dados_end:
            return
        campos = ['titulo', 'descricao']
        logradouro = dados_end.get('logradouro_id') and self.pool.get('adm.logradouro').read(
            cr, user, dados_end.get('logradouro_id'), campos, context) or {}
        cidade = dados_cidade
        bairro = dados_end.get('bairro_id') and self.pool.get('adm.bairro').read(
            cr, user, dados_end.get('bairro_id'), ['name'], context, '_classic_write') or {}
        campos = ['codigo_bc', 'name']
        pais = cidade.get('pais_id') and self.pool.get('adm.pais').read(
            cr, user, cidade.get('pais_id')[0], campos, context) or {}
        nfe.infNFe.emit.enderEmit.xLgr.valor = logradouro and \
            (logradouro.get('titulo') + u' ' + logradouro.get('descricao')).strip() or u'Logradouro'
        nfe.infNFe.emit.enderEmit.xLgr.valor = nfe.infNFe.emit.enderEmit.xLgr.valor
        nfe.infNFe.emit.enderEmit.nro.valor = dados_end.get('numero') or u'S/N'
        nfe.infNFe.emit.enderEmit.xCpl.valor = dados_end.get('complemento') or u''
        nfe.infNFe.emit.enderEmit.xBairro.valor = bairro.get('name') or u'Bairro'
        nfe.infNFe.emit.enderEmit.xMun.valor = cidade.get('name') or u'Cidade'
        nfe.infNFe.emit.enderEmit.cMun.valor = cidade.get('codigo_ibge') or 0
        nfe.infNFe.emit.enderEmit.UF.valor = dados_uf.get('sigla') or u''
        nfe.infNFe.emit.enderEmit.cPais.valor = int(pais.get('codigo_bc')[-4:]) or 0
        nfe.infNFe.emit.enderEmit.xPais.valor = pais.get('name') or u'Brasil'
        nfe.infNFe.emit.enderEmit.CEP.valor = re.sub('[^0-9]', u'', dados_end.get('cep') or u'')

    def _fisnfc_set_nfe_infnfe_dest(self, cr, user, dados_nf, end_dest, dados_cfop_nat, nfe,
                                    context=None):
        # //NFe.infNFe.dest - Dados do destinatário
        context = context and context.copy() or {}
        if end_dest:
            self._fisnfc_set_nfe_infnfe_dest_enderdest(
                cr, user, end_dest, dados_cfop_nat, nfe, context)
        campos = ['observacoes', 'name', 'admpes_jur_ie', 'pessoa_tipo', 'cpf_cnpj', 'email',
                  'admpes_inscricao_produtor', 'admpes_jur_suframa', 'telefone',
                  'admpes_fis_produtor_rural']
        dados_dest = dados_nf.get('destinatario_id') and self.pool.get('adm.pessoa').read(
            cr, user, dados_nf.get('destinatario_id'), campos, context) or {}
        exterior = dados_cfop_nat.get('codigo')[0] in (u'3', u'7')
        if nfe.infNFe.ide.tpAmb.valor == 2:
            nfe.infNFe.dest.xNome.valor = _(u'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR '
                                            u'FISCAL')
        else:
            # Se nao houver destinatario (NFC-e) assume 'CONSUMIDOR NAO IDENTIFICADO'
            nome_consumidor = dados_dest.get('name') and dados_dest.get('name')[:60] or \
                              'CONSUMIDOR NAO IDENTIFICADO'
            nfe.infNFe.dest.xNome.valor = nome_consumidor
        nfe.infNFe.dest.email.valor = dados_dest.get('email')
        uf_regra_E16a_30 = ('AM', 'BA', 'CE', 'GO', 'MG', 'MS', 'MT', 'PA', 'PE', 'RN', 'SE', 'SP', 'EX')
        if dados_dest.get('pessoa_tipo') == u'J':
            nfe.infNFe.dest.CNPJ.valor = re.sub(
                '[^0-9]', u'', dados_dest.get('cpf_cnpj') if not exterior else u'')

            # Regra de validacao SEFAZ "E16a-30"
            if not dados_dest.get('admpes_jur_ie') or \
                            dados_dest.get('admpes_jur_ie').upper() in (u'ISENTA', u'ISENTO'):
                if nfe.infNFe.dest.enderDest.UF.valor not in uf_regra_E16a_30:
                    nfe.infNFe.dest.indIEDest.valor = u'2'
                    self._consumidor_final = False
                else:
                    nfe.infNFe.dest.indIEDest.valor = u'9'
                    self._consumidor_final = True
            else:
                nfe.infNFe.dest.indIEDest.valor = u'1'
                nfe.infNFe.dest.IE.valor = \
                    re.sub('[^0-9A-Z]', u'', dados_dest.get('admpes_jur_ie') or u'ISENTO')
                self._consumidor_final = False
        else:
            nfe.infNFe.dest.CPF.valor = dados_dest and re.sub(
                '[^0-9]', u'', dados_dest.get('cpf_cnpj') if not exterior else u'') or u''
            nfe.infNFe.dest.indIEDest.valor = u'9'
            if dados_dest.get('admpes_fis_produtor_rural'):
                nfe.infNFe.dest.IE.valor = \
                    re.sub('[^0-9]', u'', dados_dest.get('admpes_inscricao_produtor') or u'')
                if dados_dest.get('admpes_inscricao_produtor'):
                    nfe.infNFe.dest.indIEDest.valor = u'1'
                    self._consumidor_final = False

        nfe.infNFe.dest.ISUF.valor = dados_dest.get('admpes_jur_suframa') or u''

    def _fisnfc_set_nfe_infnfe_dest_enderdest(self, cr, user, end_dest, dados_cfop_nat, nfe,
                                              context=None):
        # //NFe.infNFe.dest.enderDest - Endereco do destinatario
        context = context and context.copy() or {}
        load = '_classic_write'
        exterior = dados_cfop_nat.get('codigo')[0] in (u'7', u'3')
        if end_dest or exterior:
            bairro = end_dest.get('bairro_id') and self.pool.get('adm.bairro').read(
                cr, user, end_dest.get('bairro_id'), ['name'], context)or {}
            campos = ['codigo_ibge', 'name', 'pais_id', 'codigo_bc']
            cidade = end_dest.get('cidade_id') and self.pool.get('adm.cidade').read(
                cr, user, end_dest.get('cidade_id'), campos, context, load) or {}
            dados_uf = end_dest.get('uf_id') and self.pool.get('adm.uf').read(
                cr, user, end_dest.get('uf_id'), ['sigla'], context) or {}
            pais = cidade.get('pais_id') and self.pool.get('adm.pais').read(
                cr, user, cidade.get('pais_id')[0], campos, context) or {}
            logradouro_id = end_dest.get('logradouro_id')
            logradouro = logradouro_id and self.pool.get('adm.logradouro').read(
                cr, user, logradouro_id, ['titulo', 'descricao'], context) or {}
            nfe.infNFe.dest.idEstrangeiro.valor = u'EXTERIOR' if exterior else u''
            nfe.infNFe.dest.enderDest.xLgr.valor = logradouro and \
                (logradouro.get('titulo') + u' ' + logradouro.get('descricao')).strip() or \
                                                   u'Logradouro'
            # nfe.infNFe.dest.enderDest.xLgr.valor = nfe.infNFe.dest.enderDest.xLgr.valor
            nfe.infNFe.dest.enderDest.nro.valor = str(end_dest.get('numero') or u'S/N')
            nfe.infNFe.dest.enderDest.xBairro.valor = bairro and bairro.get('name') or u'Bairro'
            nfe.infNFe.dest.enderDest.cMun.valor = \
                9999999 if exterior else cidade.get('codigo_ibge') or 0
            nfe.infNFe.dest.enderDest.xMun.valor = u'EXTERIOR' if exterior else cidade and cidade.get('name') or u'Cidade'
            nfe.infNFe.dest.enderDest.UF.valor = u'EX' if exterior else dados_uf.get('sigla') or u''
            nfe.infNFe.dest.enderDest.xCpl.valor = end_dest.get('complemento') or u''
            nfe.infNFe.dest.enderDest.CEP.valor = re.sub('[^0-9]', u'', end_dest.get('cep') or u'')

            telefones = end_dest.get('telefone').values()
            telefones = telefones[0] if telefones else []
            nfe.infNFe.dest.enderDest.fone.valor = re.sub('[^0-9]', u'', telefones[0] if telefones else u'')

            if end_dest.get('pais_id'):
                pais = self.pool.get('adm.pais').browse(cr, user, end_dest.get('pais_id'), context)
                nfe.infNFe.dest.enderDest.cPais.valor = int(pais.codigo_bc[-4:] or 0)
                nfe.infNFe.dest.enderDest.xPais.valor = pais.name or u''

    def _fisnfc_set_nfe_infnfe_autxml(self, cr, user, nfe, context=None):
        # //NFe.infNFe.autXML - Pessoas autorizadas para o download do XML da NF-e
        context = context and context.copy() or {}
        if nfe.infNFe.emit.enderEmit.UF.valor == 'BA':  # Contemplado apenas pela Bahia
            if (nfe.infNFe.dest.CPF.valor or nfe.infNFe.dest.CNPJ.valor) != u'13937073000156':
                autxml = AutXML()
                autxml.CNPJ.valor = '13937073000156'  # CNPJ da SEFAZ Bahia
                nfe.infNFe.autXML.append(autxml)

        # Inclui o CNPJ da SIG INFORMATICA
        if nfe.infNFe.emit.CNPJ.valor != '01526558000178' != nfe.infNFe.dest.CNPJ.valor:
            autxml = AutXML()
            autxml.CNPJ.valor = '01526558000178'
            nfe.infNFe.autXML.append(autxml)

        adm_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'adm.parametro', ['contador_id'], context=context)
        if adm_parametro_dados.get('contador_id'):
            adm_pessoa_dados = self.pool.get('adm.pessoa').read(
                cr, user, adm_parametro_dados.get('contador_id'), ['cpf_cnpj', 'pessoa_tipo'],
                context)
            cpf_cnpj = adm_pessoa_dados.get('cpf_cnpj')
            if cpf_cnpj and cpf_cnpj != (nfe.infNFe.dest.CNPJ.valor or nfe.infNFe.dest.CPF.valor):
                autxml = AutXML()
                if adm_pessoa_dados.get('pessoa_tipo') == 'F':
                    autxml.CPF.valor = adm_pessoa_dados.get('cpf_cnpj')
                else:
                    autxml.CNPJ.valor = adm_pessoa_dados.get('cpf_cnpj')
                nfe.infNFe.autXML.append(autxml)

    def _fisnfc_set_nfe_infnfe_pag(self, cr, user, dados_nf, dict_condicoes, dict_forma_pgto,
                                   dict_pessoas_cartao, dict_cond_forma, nfe, context=None):
        # //NFe.infNFe.pag - Dados de pagamento
        context = context and context.copy() or {}
        avista_ids = self.pool.get('adm.condicao').search(cr, user, [('tipo', '=', 'V')],
                                                          context=context)
        for condicao in dict_condicoes.values():
            detpag = DetPag()
            adm_condicao_id = dict_cond_forma.get(dict_condicoes.get(condicao.get('id')).get(
                'condicao_forma_id')).get('adm_condicao_id')
            detpag.indPag.valor = str(int(adm_condicao_id not in avista_ids))
            forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
            if dados_nf.get('finalidade_emissao') in u'34':
                detpag.tPag.valor = u'90'
                detpag.vPag.valor = u'0.00'
            else:
                detpag.tPag.valor = forma_pgto.get('admfpg_tipo')
                detpag.vPag.valor = \
                    '%.2f' % condicao.get('valor') if detpag.tPag.valor != u'90' else u'0.00'
            if forma_pgto.get('admfpg_tipo') == '99':
                detpag.tPag.valor = '99'
                detpag.xPag.valor = (forma_pgto.get('name') or '')[:60]

            if detpag.tPag.valor in (u'03', u'04'):  # Cartao credito/debito
                detpag.card.tpIntegra.valor = 2  # ERP nao integra sistema de automacao
            nfe.infNFe.pag.detPag.append(detpag)

            if nfe.infNFe.ide.indIntermed.valor == '1':
                detpag.card.CNPJ.valor = nfe.infNFe.infIntermed.CNPJ.valor

    def _fisnfc_set_nfe_infnfe_exporta(self, cr, user, dados_nf, dados_cfop_nat, nfe, context=None):
        # //NFe.infNFe.exporta - Nota fiscal de exportacao
        context = context and context.copy() or {}
        if dados_cfop_nat.get('codigo')[0] == u'7':
            uf_saida_pais_id = dados_nf.get('fisnfc_uf_saida_pais') or []
            uf = self.pool.get('adm.uf').read(cr, user, uf_saida_pais_id, ['sigla'], context)
            nfe.infNFe.exporta.UFSaidaPais.valor = uf and uf.get('sigla') or u''
            nfe.infNFe.exporta.xLocExporta.valor = dados_nf.get('fisnfc_local_exportacao') or u''
            nfe.infNFe.exporta.xLocDespacho.valor = dados_nf.get('fisnfc_local_despacho') or u''

    def _fisnfc_set_nfe_infnfe_transp(self, cr, user, dados_nf, nfe, context=None):
        # //NFe.infNfE.transp - Dados da transportadora
        context = context and context.copy() or {}
        load = '_classic_write'

        # O DANFE da versao 4.0 ainda nao contempla todas as opcoes de frete
        frete = {'0': '0', '1': '1', '2': '2', '3': '0', '4': '1'}.get(dados_nf.get('frete'))
        nfe.infNFe.transp.modFrete.valor = frete or '9'

        if dados_nf.get('transportador_id') and dados_nf.get('frete') != u'9':
            campos = ['name', 'pessoa_id', 'admtra_annt']
            dados_transp = self.pool.get('adm.transportador').read(
                cr, user, dados_nf.get('transportador_id'), campos, context, load)
            campos = ['endereco_nf_id', 'pessoa_tipo', 'cpf_cnpj', 'admpes_jur_ie', 'endereco_ids']
            pessoa_transp = self.pool.get('adm.pessoa').read(
                cr, user, dados_transp.get('pessoa_id'), campos, context, load)
            nfe.infNFe.transp.transporta.xNome.valor = (dados_transp.get('name') or u'')[:45]
            if pessoa_transp.get('pessoa_tipo') == 'J':
                nfe.infNFe.transp.transporta.CNPJ.valor = re.sub(
                    '[^0-9]', u'', pessoa_transp.get('cpf_cnpj') or u'')
                nfe.infNFe.transp.transporta.IE.valor = re.sub(
                    '[^0-9A-Z]', u'', pessoa_transp.get('admpes_jur_ie') or '').upper()
            else:
                nfe.infNFe.transp.transporta.CPF.valor = re.sub(
                    '[^0-9A-z]', u'', pessoa_transp.get('cpf_cnpj') or u'')
            if pessoa_transp:
                obj_endereco = self.pool.get('adm.pessoa.endereco')
                campos = ['logradouro_id', 'cidade_id']
                ender_transp = pessoa_transp.get('endereco_nf_id') and obj_endereco.read(
                    cr, user, pessoa_transp.get('endereco_nf_id'), campos, context, load) or {}
                if pessoa_transp.get('endereco_ids'):
                    ender_transp = ender_transp or obj_endereco.read(
                        cr, user, pessoa_transp.get('endereco_ids')[0], campos, context, load)
                    campos = ['titulo', 'descricao']
                    logradouro = ender_transp.get('logradouro_id') and self.pool.get('adm.logradouro').read(
                        cr, user, ender_transp.get('logradouro_id'), campos, context) or {}
                    campos = ['uf_id', 'name']
                    cidade = ender_transp.get('cidade_id') and self.pool.get('adm.cidade').read(
                        cr, user, ender_transp.get('cidade_id'), campos, context, load) or {}
                    uf = cidade.get('uf_id') and self.pool.get('adm.uf').read(
                        cr, user, cidade.get('uf_id'), ['sigla'], context) or {}
                    nfe.infNFe.transp.transporta.xEnder.valor = \
                        logradouro and ((logradouro.get('titulo') or u'') + u' ' +
                                        (logradouro.get('descricao') or u'')).strip() or u''
                    nfe.infNFe.transp.transporta.xMun.valor = cidade.get('name') or u'Cidade'
                    nfe.infNFe.transp.transporta.UF.valor = uf.get('sigla') or u''

                # Em Minas Gerais a NFe v4.0 nao contempla dados de transporte (frete) no estado.
                # Ver Nota Tecnica 2016.002
                if nfe.infNFe.ide.idDest.valor == 1 and nfe.infNFe.emit.enderEmit.UF.valor != u'MG':
                    nfe.infNFe.transp.veicTransp.placa.valor = re.sub(
                        '[^a-zA-Z0-9]', u'', dados_nf.get('fisnfc_placa_frete') or u'')
                    nfe.infNFe.transp.veicTransp.UF.valor = uf and uf.get('sigla') or u''
                    if dados_transp.get('admtra_annt') and dados_nf.get('fisnfc_placa_frete'):
                        nfe.infNFe.transp.veicTransp.RNTC.valor = dados_transp.get('admtra_annt')

        self._fisnfc_set_nfe_infnfe_transp_vol(cr, user, dados_nf, nfe, context)

    def _fisnfc_set_nfe_infnfe_transp_vol(self, cr, user, dados_nf, nfe, context=None):
        # //NFe.infNFe.transp.vol - Dados dos volumes
        context = context and context.copy() or {}
        vol = Vol()
        vol.qVol.valor = int(dados_nf.get('quantidade_volume') or 0)
        vol.esp.valor = dados_nf.get('especie') or u''
        vol.marca.valor = dados_nf.get('marca') or u''
        vol.nVol.valor = dados_nf.get('numero_volume') or u''
        vol.pesoB.valor = '%.3f' % dados_nf.get('peso_bruto') or 0
        vol.pesoL.valor = '%.3f' % dados_nf.get('peso_liquido') or 0
        nfe.infNFe.transp.vol.append(vol)

    def _fisnfc_set_nfe_infnfe_cobr_fat(self, cr, user, dados_nf, dict_condicoes, dict_forma_pgto,
                                        nfe, context=None):
        # //NFe.infNFe.cobr.fat - Faturamento (nao envolve 03 - Cartao de credito
        #                                                  04 - Cartao de debito
        #                                                  14 - Duplicata Mercantil
        #                                                  90 - Sem pagamento)
        context = context and context.copy() or {}
        tot_fatura = 0
        for i in dict_condicoes.values():
            if dict_forma_pgto.get(i.get('forma_pagto_id')).get('admfpg_tipo') \
                    in (u'03',u'04',u'14',u'90'):
                continue

            tot_fatura += i.get('valor')

        if tot_fatura:
            fatura_id = self.pool.get('financeiro.fatura').search(
                cr, user, [('nf_id', '=', dados_nf.get('id'))], context=context)
            if fatura_id:
                nfe.infNFe.cobr.fat.nFat.valor = \
                    dados_nf.get('numero_nf') and str(dados_nf.get('numero_nf')).zfill(9) or u''
                nfe.infNFe.cobr.fat.vOrig.valor = '%.2f' % tot_fatura
                nfe.infNFe.cobr.fat.vLiq.valor = '%.2f' % tot_fatura
                if filter(lambda f: f.indPag.valor, nfe.infNFe.pag.detPag):  # A prazo
                    self._fisnfc_set_nfe_infnfe_cobr_dup(cr, user, dados_nf, dict_condicoes,
                                                         dict_forma_pgto, nfe, context)

    def _fisnfc_set_nfe_infnfe_cobr_dup(self, cr, user, dados_nf, dict_condicoes, dict_forma_pgto,
                                        nfe, context=None):
        # //NFe.infNFe.cobr.dup - Duplicatas
        context = context and context.copy() or {}
        # sem fatura nao gera suplicata
        if not nfe.infNFe.cobr.fat.nFat.valor:
            return

        cont = 1
        campos = ['nf_condicao_id', 'vencimento', 'valor']
        for parcela in self.pool.get('fiscal.nf.parcelas').read(
                cr, user, dados_nf.get('nf_parcelas_ids'), campos, context, '_classic_write'):
            condicao = dict_condicoes.get(parcela.get('nf_condicao_id'))
            tipo_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id')).get('admfpg_tipo')
            if parcela.get('nf_condicao_id') not in (dados_nf.get('nf_condicao_ids') or []) \
                    or tipo_pgto in (u'03',u'04',u'14',u'90'):
                continue

            dup = Dup()
            dup.nDup.valor = str(cont).zfill(3)
            dup.dVenc.valor = datetime.strptime(parcela.get('vencimento'), '%Y-%m-%d')
            dup.vDup.valor = '%.2f' % parcela.get('valor') or 0
            nfe.infNFe.cobr.dup.append(dup)
            cont += 1

    # Documento de importacao
    def _fisnfc_set_nfe_infnfe_prod_di(self, cr, user, di_id, dados_item, prod, nfe, context=None):
        context = context and context.copy() or {}
        brw_di = self.pool.get('fiscal.declaracao.importacao').browse(
            cr, user, di_id, context)
        di = DI()
        di.nDI.valor = brw_di.fisdim_numero
        di.dDI.valor = brw_di.fisdim_data
        di.xLocDesemb.valor = brw_di.fisdim_local_desembaraco
        di.UFDesemb.valor = brw_di.fisdim_uf_id.sigla
        di.UFTerceiro.valor = nfe.infNFe.emit.enderEmit.UF.valor
        di.CNPJ.valor = nfe.infNFe.emit.CNPJ.valor
        di.dDesemb.valor = brw_di.fisdim_data_desembaraco
        di.tpViaTransp.valor = brw_di.fisdim_via_transporte
        di.vAFRMM.valor = '%.2f' % (brw_di.fisdim_vr_afrmm * dados_item.get('percentual_rateio'))
        di.tpIntermedio.valor = brw_di.fisdim_forma_importacao
        di.cExportador.valor = brw_di.fisdim_exportador
        for adicao in  brw_di.fisdim_adicao_ids:
            adi = Adi()
            adi.nAdicao.valor = adicao.fisdia_numero
            adi.nSeqAdic.valor = adicao.sequence
            adi.cFabricante.valor = adicao.fisdia_fabricante
            adi.vDescDI.valor = 0
            di.adi.append(adi)
        prod.DI.append(di)

    def _fisnfc_set_nfe_infnfe_prod(self, cr, user, dados_nf, end_dest, dados_pessoa_company,
                                    dados_cfop_nat, dict_condicoes, dict_cond_forma, nfe, context):
        # //NFe.infNFe.prod - Itens da nota fiscal
        context = context and context.copy() or {}
        locale.setlocale(locale.LC_ALL, 'pt_BR.utf8')
        tot_ncm_vr_tributo_fed = tot_ncm_vr_tributo_est = tot_ncm_vr_tributo_mun = 0
        cfop_tipo_venda = True and dados_cfop_nat.get('movimento_tipo') == 'C'
        cfop_importacao = True and dados_cfop_nat.get('codigo')[0] == '3'  # Documento de importacao
        ncm_chave = u''
        load = '_classic_write'

        # Inicia os valores totais
        self.total_aproveitamento_credito = 0

        campos = ['produto_id', 'grade_id', 'estmvi_comprimento', 'estmvi_largura', 'vr_unitario',
                  'estmvi_espessura', 'quantidade', 'produto_embalagem_id', 'vr_total_item',
                  'estmvi_volume', 'vr_desconto', 'estmvi_pedido_cliente_nro', 'vr_seguro_rateado',
                  'estmvi_pedido_cliente_linha', 'sequencia', 'cfop_id', 'vr_desconto_rateado',
                  'vr_frete_rateado',  'tributacoes_ids', 'vr_despesas_rateado', 'natureza_id',
                  'vr_total_item_rateado']
        dados_itens = self.pool.get('estoque.movimento.itens').read(
            cr, user, dados_nf.get('estoque_movimento_itens_ids'), campos, context, load)
        if not dados_itens:
            return True

        dados_itens = sorted(dados_itens, key=lambda x: x.get('id'))
        map(lambda m: m.update({'sequencia': dados_itens.index(m) + 1}), dados_itens)
        produto_ids = []
        grade_ids = []
        prod_emb_ids = []
        cfop_ids = []
        tributacoes_ids = []
        ii_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'II')], context=context)
        ii_id = ii_ids and ii_ids[0]
        valor_total = sum([item.get('vr_total_item') for item in dados_itens])
        for i in dados_itens:
            i.get('produto_id') and produto_ids.append(i.get('produto_id'))
            i.get('grade_id') and grade_ids.append(i.get('grade_id'))
            i.get('produto_embalagem_id') and prod_emb_ids.append(i.get('produto_embalagem_id'))
            i.get('cfop_id') and cfop_ids.append(i.get('cfop_id'))
            i.get('tributacoes_ids') and tributacoes_ids.extend(i.get('tributacoes_ids'))
            i['percentual_rateio'] = ii_id and valor_total and i.get('vr_total_item')/valor_total
        dict_prod_emb = {}
        embalagem_ids = []
        for i in self.pool.get('estoque.produto.embalagem').read(
                cr, user, prod_emb_ids, ['embalagem_id'], context, load):
            dict_prod_emb[i.get('id')] = i
            embalagem_ids.append(i.get('embalagem_id'))
        dict_embalagem = {}
        unidade_ids = []
        campos = ['unidade_id', 'quantidade_na_embalagem']
        for i in self.pool.get('estoque.embalagem').read(
                cr, user, embalagem_ids, campos, context, load):
            dict_embalagem[i.get('id')] = i
            unidade_ids.append(i.get('unidade_id'))
        dict_unidade = {}
        campos = ['unidade', 'name']
        map(lambda x: dict_unidade.update({x.get('id'): x}),
            self.pool.get('estoque.embalagem.unidade').read(cr, user, unidade_ids, campos, context))
        dict_trib = {}
        cst_ids = []
        tributacao_ids = []
        campos = ['vr_icms_substituicao', 'tributacao_id', 'cst_id', 'vr_base_calculo', 'aliquota',
                  'vr_encargo', 'fisnit_vr_fcp', 'modbcst', 'percentual_mva', 'vr_base_icms_substituicao',
                  'fisnit_vr_fcp_st', 'vr_reducao_base', 'fisnit_vr_icms_desonerado', 'encargo_id',
                  'fisnit_vr_difal_uf_destino', 'fisnit_vr_difal_uf_origem', 'fisnit_vr_difal_base']
        for i in self.pool.get('fiscal.nf.item.tributacao').read(
                cr, user, tributacoes_ids, campos, context, load):
            dict_trib[i.get('id')] = i
            i.get('cst_id') and cst_ids.append(i.get('cst_id'))
            i.get('tributacao_id') and tributacao_ids.append(i.get('tributacao_id'))
        dict_tributacao = {}
        encargo_ids = []
        campos = ['fistri_aliquota_fcp', 'cst_id', 'encargo_id', 'fistri_codigo_beneficio']
        for i in self.pool.get('fiscal.tributacao').read(
                cr, user, tributacao_ids, campos, context, load):
            dict_tributacao[i.get('id')] = i
            i.get('cst_id') and cst_ids.append(i.get('cst_id'))
            i.get('encargo_id') and encargo_ids.append(i.get('encargo_id'))
        dict_encargo = {}
        map(lambda x: dict_encargo.update({x.get('id'): x}),
            self.pool.get('fiscal.encargo').read(cr, user, encargo_ids, ['sigla'], context))
        dict_cst = {}
        csosn_ids = []
        campos = ['codigo', 'fiscst_csosn_nao_contribuinte_id', 'fiscst_csosn_contribuinte_id',
                  'tipo']
        for i in self.pool.get('fiscal.cst').read(cr, user, cst_ids, campos, context, load):
            dict_cst[i.get('id')] = i
            if i.get('fiscst_csosn_nao_contribuinte_id'):
                csosn_ids.append(i.get('fiscst_csosn_nao_contribuinte_id'))
            if i.get('fiscst_csosn_contribuinte_id'):
                csosn_ids.append(i.get('fiscst_csosn_contribuinte_id'))
        dict_csosn = {}
        map(lambda x: dict_csosn.update({x.get('id'): x}),
            self.pool.get('fiscal.csosn').read(cr, user, csosn_ids, ['codigo'], context))
        dict_produtos = {}
        automacao_ids = []
        ncm_ids = []
        anp_ids = []
        tab_servico_ids = []
        campos = ['grade_usar', 'codigo', 'automacao_ids', 'ncm_id', 'origem_mercadoria',
                  'estpro_codigo_anp_id', 'estpro_fci', 'codigo_tab_servicos_id', 'estpro_med_codigo_anvisa',
                  'estpro_med_motivo_isencao','estpro_med_preco_maximo', 'estpro_nlote', 'estpro_qlote',
                  'estpro_dfab', 'estpro_dval', 'estpro_cagreg',
        ]
        for i in self.pool.get('estoque.produto').read(
                cr, user, produto_ids, campos, context, load):
            dict_produtos[i.get('id')] = i
            dict_produtos[i.get('id')]['natureza_id'] = filter(
                lambda f: f.get('produto_id') == i.get('id'), dados_itens)[0].get('natureza_id')
            i.get('automacao_ids') and automacao_ids.extend(i.get('automacao_ids'))
            i.get('ncm_id') and ncm_ids.append(i.get('ncm_id'))
            i.get('estpro_codigo_anp_id') and anp_ids.append(i.get('estpro_codigo_anp_id'))
            if i.get('codigo_tab_servicos_id'):
                tab_servico_ids.append(i.get('codigo_tab_servicos_id'))
        cest_produtos = self.pool.get('estoque.produto').estpro_get_cest(
            cr, user, produto_ids, context)
        cest_ids = list(set(cest_produtos.values()))
        None in cest_ids and cest_ids.remove(None)
        False in cest_ids and cest_ids.remove(False)
        dict_cest = {}
        map(lambda x: dict_cest.update({x.get('id'): x}),
            self.pool.get('fiscal.cest').read(cr, user, cest_ids, ['fiscest_codigo'], context))
        dict_tab_serv = {}
        if tab_servico_ids:
            map(lambda x: dict_tab_serv.update({x.get('id'): x}),
                self.pool.get('fiscal.tabela.servico').read(
                    cr, user, tab_servico_ids, ['codigo'], context))
        dict_ncm = {}
        campos = ['codigo', 'fisnc_aliq_ibpt_fed_nac', 'fisnc_aliq_ibpt_fed_imp',
                  'fisnc_aliq_ibpt_est', 'fisnc_aliq_ibpt_mun', 'fisnc_ibpt_chave']
        map(lambda x: dict_ncm.update({x.get('id'): x}),
            self.pool.get('fiscal.ncm').read(cr, user, ncm_ids, campos, context))
        dict_anp = {}
        map(lambda x: dict_anp.update({x.get('id'): x}),
            self.pool.get('fiscal.cadastro.anp').read(
                cr, user, anp_ids, ['fisanp_codigo', 'name', 'fisanp_pglp', 'fisanp_pgnn',
                                    'fisanp_pgni', 'fisanp_partida', 'fisanp_adrem'], context))
        dict_automacao = {}
        map(lambda x: dict_automacao.update({x.get('id'): x}),
            self.pool.get('automacao.codigo.barra').read(
                cr, user, automacao_ids, ['grade_id'], context, load))
        dict_grades = {}
        campos = ['grade', 'name', 'descricao', 'descricao_complementar']
        map(lambda x: dict_grades.update({x.get('id'): x}),
            self.pool.get('estoque.produto.grade').read(cr, user, grade_ids, campos, context))
        dict_cfop = {}
        map(lambda x: dict_cfop.update({x.get('id'): x}),
            self.pool.get('fiscal.cfop').read(cr, user, cfop_ids, ['codigo'], context))
        obj_par_estoque = self.pool.get('estoque.parametro')
        exibir_desc_grade = False
        par_estoque_ids = obj_par_estoque.search(cr, user, [], context=context)
        if par_estoque_ids:
            exibir_desc_grade = obj_par_estoque.read(cr, user, par_estoque_ids[0],
                                                  ['estpar_desc_complementar_grade_nota'], context)\
                                                         .get('estpar_desc_complementar_grade_nota')
        for item in dados_itens:
            produto = dict_produtos.get(item.get('produto_id'))
            grade = dict_grades.get(item.get('grade_id'))
            if produto.get('grade_usar'):
                p_cprod = produto.get('codigo') + u'-' + grade.get('grade')
            else:
                p_cprod = produto.get('codigo')
            if produto.get('automacao_ids'):  # FIX: Deve ser heranca do modulo Automacao
                p_cean = \
                    filter(lambda f: dict_automacao.get(f).get('grade_id') == item.get('grade_id'),
                           produto.get('automacao_ids'))
                if p_cean:
                    codigo_barra = self.pool.get(
                        'automacao.codigo.barra').autcba_get_codigo_barra_formatado_gtin(
                        cr, user, p_cean[0], context)

                    # Na nota 4.0 o primeiro digito do codigo de barras nao pode ser 0 quando tiver
                    # 14 digitos, o primeiro digito deve ser de 1 a 9 que indicara a quantidade na
                    # embalagem
                    if len(codigo_barra.get(p_cean[0])) == 14:
                        if codigo_barra.get(p_cean[0])[0] == '0':
                             codigo_barra[p_cean[0]] = codigo_barra.get(p_cean[0])[1:]
                    p_cean = codigo_barra.get(p_cean[0])
                else:
                    p_cean = u'SEM GTIN'
            else:
                p_cean = u'SEM GTIN'
                
            nome_produto = grade.get('name') or grade.get('descricao') or u''
            if self.pool.get('adm.utils').admutl_check_modulo_instalado(
                    cr, user, 'madeireira', context):
                comprimento = u' ' + str(item.get('estmvi_comprimento')) \
                    if item.get('estmvi_comprimento') else u''
                largura = u' X ' + str(item.get('estmvi_largura')) \
                    if item.get('estmvi_largura') else u''
                espessura = u' X ' + str(item.get('estmvi_espessura')) \
                    if item.get('estmvi_espessura') else u''
                medidas = (comprimento + largura + espessura) or u''
            else:
                medidas = u''
            p_xprod = nome_produto + medidas
            if exibir_desc_grade and grade.get('descricao_complementar'):
                p_xprod = (p_xprod + ' ' + grade.get('descricao_complementar'))[:120] # max permitido 120

            ncm = dict_ncm.get(produto.get('ncm_id'))
            ncm_vr_tributo_fed = 0
            if produto.get('origem_mercadoria') in u'0345':  # Nacional
                if ncm.get('fisnc_aliq_ibpt_fed_nac') > 0:
                    ncm_vr_tributo_fed = item.get('vr_total_item_rateado')
                    ncm_vr_tributo_fed *= ncm.get('fisnc_aliq_ibpt_fed_nac') or 0
                    ncm_vr_tributo_fed /= 100.
            elif produto.get('origem_mercadoria') in u'1267':  # Importado
                if ncm.get('fisnc_aliq_ibpt_fed_imp') > 0:
                    ncm_vr_tributo_fed = item.get('vr_total_item_rateado')
                    ncm_vr_tributo_fed *= ncm.get('fisnc_aliq_ibpt_fed_imp') or 0
                    ncm_vr_tributo_fed /= 100.
            ncm_vr_tributo_est = item.get('vr_total_item_rateado')
            ncm_vr_tributo_est *= ncm.get('fisnc_aliq_ibpt_est') or 0
            ncm_vr_tributo_est /= 100.
            ncm_vr_tributo_mun = item.get('vr_total_item_rateado')
            ncm_vr_tributo_mun *= ncm.get('fisnc_aliq_ibpt_mun') or 0
            ncm_vr_tributo_mun /= 100.
            if not ncm_chave:
                ncm_chave = ncm.get('fisnc_ibpt_chave') or u''
            tot_ncm_vr_tributo_fed += ncm_vr_tributo_fed
            tot_ncm_vr_tributo_est += ncm_vr_tributo_est
            tot_ncm_vr_tributo_mun += ncm_vr_tributo_mun
            prod_embalagem = dict_prod_emb.get(item.get('produto_embalagem_id'))
            embalagem = dict_embalagem.get(prod_embalagem.get('embalagem_id'))
            unidade = dict_unidade.get(embalagem.get('unidade_id'))
            p_ucom = unidade.get('unidade')
            if self._parametros(cr, user).fiscal.fispar_imprimir_qtde_embalagem_uni_danfe:
                p_ucom += str(int(embalagem.get('quantidade_na_embalagem')))[:6]

            unidade_tributavel_dados = \
                self.pool.get('estoque.produto').estpro_get_dados_unidade_tributavel(
                    cr, user, item.get('produto_id'), context)
            p_qcom = '%.3f' % item.get('quantidade') or 0
            if self.pool.get('adm.utils').admutl_check_modulo_instalado(
                    cr, user, 'madeireira', context):
                p_qcom = '%.3f' % (item.get('estmvi_volume') or item.get('quantidade'))
            p_vuncom = '%f' % item.get('vr_unitario')
            p_vuncom = p_vuncom.rstrip('0')  # Desconsidera zeros aa direita
            _valor = (item.get('vr_total_item') or 0) + (item.get('vr_desconto') or 0)
            p_vprod = '%.2f' % _valor or 0
            p_ceantrib = p_cean
            p_indtot = 1   # Valor do item entra no valor total da nota, 1-sim, 0-nao
            det = Det() if nfe.infNFe.ide.mod.valor == 55 else NFCeDet()
            det.prod.xPed.valor = item.get('estmvi_pedido_cliente_nro') or u''
            det.prod.nItemPed.valor = u'%s' % (item.get('estmvi_pedido_cliente_linha') or u'')
            det.nItem.valor = item.get('sequencia')
            det.prod.cProd.valor = p_cprod or u''
            det.prod.cEAN.valor = p_cean or u''

            # Se NFC-e em ambiente de homologacao, a descricao do primeiro item deve ser
            # obrigatoriamente NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL
            det.prod.xProd.valor = p_xprod or u'Item %s' % dados_itens.index(item) + 1
            if nfe.infNFe.ide.mod.valor == 65:
                if nfe.infNFe.ide.tpAmb.valor == 2 and dados_itens.index(item) < 1:  # Homologacao
                    det.prod.xProd.valor = u'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM '\
                                           u'VALOR FISCAL'

            # Tratamento para combustiveis. Verifica se o produto tem CFOP para combustiveis e
            # faz a insercao das tags
            cfop = dict_cfop.get(item.get('cfop_id'))
            anp = False
            if cfop.get('codigo')[1:3] in ('65', '66'):
                if produto.get('estpro_codigo_anp_id'):
                    anp = dict_anp.get(produto.get('estpro_codigo_anp_id'))
                    det.prod.comb.cProdANP.valor = anp.get('fisanp_codigo') or u''
                    det.prod.comb.descANP.valor = anp.get('name') or u''

                    # Gas de cozinha (GLP) tem tratamento diferente
                    if det.prod.comb.cProdANP.valor == 210203001:
                        det.prod.comb.descANP.valor = u'GLP'
                        det.prod.comb.pGLP.valor = '%.4f' % anp.get('fisanp_pglp') or 0
                        det.prod.comb.pGNn.valor = '%.4f' % anp.get('fisanp_pgnn') or 0
                        det.prod.comb.pGNi.valor = '%.4f' % anp.get('fisanp_pgni') or 0
                        det.prod.comb.vPart.valor = '%.2f' % anp.get('fisanp_partida') or 0

                det.prod.comb.UFCons.valor = nfe.infNFe.dest.enderDest.UF.valor or nfe.infNFe.emit.enderEmit.UF.valor or ''

            det.prod.NCM.valor = re.sub('[^0-9]', u'', ncm.get('codigo') or '') or u''
            if det.prod.NCM.valor[:4] in ('3001', '3002', '3003', '3004', '3005', '3006'):
                if nfe.infNFe.ide.mod.valor == 55:  # nfce nao precisa informar
                    rastro = Rastro()
                    rastro.nLote.valor = produto.get('estpro_nlote') or ''
                    rastro.qLote.valor = Decimal(produto.get('estpro_qlote')).quantize(TREEPLACES) or 0
                    rastro.dFab.valor = produto.get('estpro_dfab')
                    rastro.dVal.valor = produto.get('estpro_dval')
                    rastro.cAgreg.valor = produto.get('estpro_cagreg') or ''
                    det.prod.rastro.append(rastro)

                det.prod.med.cProdANVISA.valor = produto.get('estpro_med_codigo_anvisa') or ''
                det.prod.med.xMotivoIsencao.valor = produto.get('estpro_med_motivo_isencao') or ''
                det.prod.med.vPMC.valor = Decimal(produto.get('estpro_med_preco_maximo')).quantize(TWOPLACES) or 0

            det.prod.nFCI.valor = produto.get('estpro_fci') or u''
            det.prod.CFOP.valor = int(cfop.get('codigo'))
            det.prod.uCom.valor = p_ucom or u''
            det.prod.qCom.valor = p_qcom or 0
            det.prod.vUnCom.valor = p_vuncom or 0
            det.prod.vDesc.valor = \
                '%.2f' % ((item.get('vr_desconto_rateado') or 0) + (item.get('vr_desconto') or 0))
            det.prod.vFrete.valor = '%.2f' % item.get('vr_frete_rateado') or 0
            det.prod.vSeg.valor = '%.2f' % item.get('vr_seguro_rateado') or 0
            substituicao_rateado = 0
            for trib_id in item.get('tributacoes_ids'):
                trib = dict_trib.get(trib_id)
                if trib.get('vr_icms_substituicao') > 0:
                    substituicao_rateado = trib.get('vr_icms_substituicao')
                    break

            # Adiciona-se o ICMS/ST quando nao eh industria
            if dados_pessoa_company.get('admpes_jur_tipo_atividade') != u'I':
                if dados_nf.get('tipo') == u'E' and dados_cfop_nat.get('movimento_tipo') != 'D':
                    det.prod.vOutro.valor = '%.2f' % ((item.get('vr_despesas_rateado') or 0) +
                                                      substituicao_rateado) or 0
                else:
                    det.prod.vOutro.valor = '%.2f' % item.get('vr_despesas_rateado') or 0
            else:
                det.prod.vOutro.valor = '%.2f' % item.get('vr_despesas_rateado') or 0

            det.prod.vProd.valor = p_vprod or 0
            det.prod.cEANTrib.valor = p_ceantrib or u''

            fator = unidade_tributavel_dados.get('estpro_fator_conversao_unidade_tributavel')

            # Se a unidade tributavel do item tiver fator de conversao, realiza os calculos e
            # atribui valores...
            if fator:
                det.prod.uTrib.valor = unidade_tributavel_dados.get('unidade')
                det.prod.qTrib.valor = '%.3f' % (float(p_qcom) * fator)
                det.prod.vUnTrib.valor = '%f' % (float(p_vuncom) / fator)
            else:  # ...senao, iguala as unidades e valores
                det.prod.uTrib.valor = p_ucom or u''
                det.prod.qTrib.valor = p_qcom or 0
                det.prod.vUnTrib.valor = p_vuncom or 0

            det.prod.indTot.valor = p_indtot or 0

            # //NFe.infNfe.det.infAdProd - Informacoes adicionais (Maximo de 500 caracteres)
            det.infAdProd.valor = ' | '.join(self.nfe_infnfe_det_infadprod(
                cr, user, item.get('id'), context))[:500]

            # //NFe.infNFe.imposto - Encargos e tributacoes
            det.imposto.vTotTrib.valor = \
                '%.2f' % (ncm_vr_tributo_fed + ncm_vr_tributo_est + ncm_vr_tributo_mun)
            if nfe.infNFe.emit.CRT.valor == 1:
                self.set_regime_tributario_simples_nacional(
                    cr, user, dados_nf, end_dest, item, produto, dict_trib, dict_tributacao,
                    dict_encargo, dict_cfop, dict_cst, dict_csosn, cest_produtos, dict_cest, det,
                    nfe, context=context)
            else:
                self.set_regime_tributario_outros(
                    cr, user, dados_nf, end_dest, item, produto, dict_trib, dict_tributacao,
                    dict_encargo, dict_cst, dict_cfop, dict_tab_serv, cest_produtos, dict_cest, det,
                    nfe, anp, context=context)

            if cfop_importacao:
                self._fisnfc_set_nfe_infnfe_prod_di(
                    cr, user, dados_nf.get('fisnfc_di_ids')[0], item, det.prod, nfe, context)

            nfe.infNFe.det.append(det)

        # Aproveitamento de ICMS para Simples Nacional
        if self.pool.get('adm.utils').admutl_get_parametros(
                cr, user, 'fiscal', [], context).get('fispar_receita_anual') > 0:
            if nfe.infNFe.emit.CRT.valor == 1:
                nfe.infNFe.infAdic.infCpl.valor += \
                    _(u'\n Documento emitido por ME ou EPP optante pelo Simples Nacional. - NAO '
                      u'GERA DIREITO DE CREDITO FISCAL DE IPI. - Permite aproveitamento '
                      u'de credito ICMS no valor de R$ %.2f correspondente a aliq. %.2f%% nos '
                      u'termos ART.23 LC123/06') % (
                        self.total_aproveitamento_credito or 0, self._fisnfc_get_aliquota_sn(
                            cr, user, context))

        if not dados_nf.get('fisnfc_nf_referencia_ids') and dados_nf.get('tipo') == 'S' \
                and cfop_tipo_venda:
            if dados_nf.get('vr_total_nf'):
                valor_percent_aprox_fed = tot_ncm_vr_tributo_fed / dados_nf.get('vr_total_nf') * 100
                valor_percent_aprox_est = tot_ncm_vr_tributo_est / dados_nf.get('vr_total_nf') * 100
                valor_percent_aprox_mun = tot_ncm_vr_tributo_mun / dados_nf.get('vr_total_nf') * 100
            else:
                valor_percent_aprox_fed = valor_percent_aprox_est = valor_percent_aprox_mun = 0
            nfe.infNFe.infAdic.infCpl.valor += \
                _(u' \n Trib.Aprox.: %s (%s%%) federal %s (%s%%) estadual %s (%s%%) municipal. '
                  u'Fonte IBPT \n%s') % (
                    locale.currency(tot_ncm_vr_tributo_fed, grouping=True),
                    locale.currency(valor_percent_aprox_fed, symbol=False),
                    locale.currency(tot_ncm_vr_tributo_est, grouping=True),
                    locale.currency(valor_percent_aprox_est, symbol=False),
                    locale.currency(tot_ncm_vr_tributo_mun, grouping=True),
                    locale.currency(valor_percent_aprox_mun, symbol=False), ncm_chave)
        valor_cfop_por_aliquota = \
            self.pool.get('fiscal.nf.item.tributacao').fisnit_get_cfop_por_aliquota(
                cr, user, list(itertools.chain.from_iterable(
                    [item.get('tributacoes_ids') for item in dados_itens])), context)
        str_resumo = u''
        for vcpa in valor_cfop_por_aliquota:
            str_resumo += '[%s - CFOP: %s ' % (vcpa.get('encargo'), vcpa.get('cfop'))
            if vcpa.get('encargo') == 'ICMS' and vcpa.get('vr_icms_desonerado'):
                str_resumo += 'Isentos: %s ' % getNumeroEditado(
                        vcpa.get('vr_base_calculo') - vcpa.get('vr_icms_desonerado'))
                str_resumo += 'Base: 0,00'
            else:
                if vcpa.get('vr_icms_substituicao'):
                    str_resumo += _('Substituicao') + ': %s]' % getNumeroEditado(
                        vcpa.get('vr_icms_substituicao'))
                    continue
                if vcpa.get('vr_isento'):
                    str_resumo += 'Isentos: %s ' % getNumeroEditado(vcpa.get('vr_isento'))
                if vcpa.get('aliquota'):
                    str_resumo += 'Trib.a: %s%% ' % getNumeroEditado(vcpa.get('aliquota'), 3)
                if vcpa.get('vr_base_calculo'):
                    str_resumo += 'Base: %s ' % getNumeroEditado(vcpa.get('vr_base_calculo'))
                if vcpa.get('vr_encargo'):
                    str_resumo += 'Imp.: %s' % getNumeroEditado(vcpa.get('vr_encargo'))
            str_resumo += ']'
        if str_resumo:
            nfe.infNFe.infAdic.infCpl.valor += u'\n Resumo: ' + str_resumo
        total_icms_uf_dest = sum(map(lambda m: float(m.imposto.ICMSUFDest.vICMSUFDest.valor),
                                     nfe.infNFe.det))
        total_fcp = sum(map(lambda m: float(m.imposto.ICMS.vFCP.valor), nfe.infNFe.det))
        if total_icms_uf_dest:
            nfe.infNFe.infAdic.infCpl.valor += _(u' \n Parcela do diferencial de aliquota devido '
                                                 u'ao estado destino: R$%.2f\n Valor do fundo de '
                                                 u'combate a pobreza: R$%.2f') % (
                                                   total_icms_uf_dest, total_fcp)
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, user, 'ecf', context):
            for cond in dict_condicoes.values():
                cond_forma = dict_cond_forma.get(cond.get('condicao_forma_id'))
                if cond_forma.get('tipo_finalizadora') == u'C':
                    nfe.infNFe.infAdic.infCpl.valor += \
                        _(u'\n Autorizacao do cartao: %s') % cond.get('fiscpg_nro_autorizacao_cartao')

    def _fisnfc_set_nfe_infnfe_total_icmstot(self, cr, user, dados_nf, nfe, dados_natureza,
                                             context=None):
        # NFe.infNFe.total.ICMSTot - Total do grupo ICMS - Produtos
        context = context and context.copy() or {}
        nfe.infNFe.total.ICMSTot.vBC.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vBC.valor), nfe.infNFe.det))
        if not context.get('nota_fiscal_consumidor'):
            nfe.infNFe.total.ICMSTot.vICMS.valor = '%.2f' % (sum(map(
                lambda m: float((not m.imposto.ICMS.motDesICMS.valor or
                                 m.imposto.ICMS.CST.valor == '20') and m.imposto.ICMS.vICMS.valor),
                nfe.infNFe.det)) or 0)
        else:
            nfe.infNFe.total.ICMSTot.vICMS.valor = '%.2f' % sum(
                map(lambda m: float(m.imposto.ICMS.vICMS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSDeson.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSDeson.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.qBCMonoRet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.qBCMonoRet.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSMonoRet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSMonoRet.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFCP.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vFCP.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vBCST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vBCST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFCPST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vFCPST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vProd.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vProd.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFrete.valor = '%.2f' % dados_nf.get('vr_frete') or 0
        nfe.infNFe.total.ICMSTot.vSeg.valor = '%.2f' % dados_nf.get('vr_seguro') or 0
        nfe.infNFe.total.ICMSTot.vDesc.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vDesc.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vII.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.II.vII.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vIPI.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.IPI.vIPI.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vIPIDevol.valor = '%.2f' % sum(map(
                lambda m: float(m.impostoDevol.IPI.vIPIDevol.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vPIS.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.PIS.vPIS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vCOFINS.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.COFINS.vCOFINS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vOutro.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vOutro.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vNF.valor = '%.2f' % dados_nf.get('vr_total_nf')
        nfe.infNFe.total.ICMSTot.vTotTrib.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.vTotTrib.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSUFDest.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMSUFDest.vICMSUFDest.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSUFRemet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMSUFDest.vICMSUFRemet.valor), nfe.infNFe.det))
        if dados_natureza.get('fisnat_ir_retencao'):
            valor_ir = Decimal(dados_nf.get('vr_total_nf')) * \
                       Decimal(dados_natureza.get('fisnat_ir_aliquota') or 0) / Decimal(100.0)
            valor_ir = valor_ir.quantize(TWOPLACES)
            nfe.infNFe.total.retTrib.vBCIRRF.valor = str(dados_nf.get('vr_total_nf'))
            nfe.infNFe.total.retTrib.vIRRF.valor = valor_ir
            self.write(cr, user, dados_nf.get('id'), {'fisnfc_retencao_ir_valor': valor_ir},
                       context)
            info_txt = _(u'\nRetencao de I.R. base de calculo: %s, valor retido: %s conforme a '
                         u'Instrucao Normativa RFB 2.145/2023, do Anexo I aliquota correspondente a %s%%') % \
                        (str(nfe.infNFe.total.retTrib.vBCIRRF.valor).replace('.', ','),
                         str(valor_ir).replace('.', ','),
                         str(dados_natureza.get('fisnat_ir_aliquota')).replace('.', ','))

            nfe.infNFe.infAdic.infCpl.valor += info_txt

    def gerar_schema_ICMSUFDest(self, cr, uid, det, dados_nf, end_dest, trib, produto_nacional,
                                context=None):
        """
        Este metodo gera o bloco ICMSUFDest em operacoes interestaduais onde deve-se apurar
        o valor do ICMS Difal.
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param det: Instancia de objeto do bloco det
        :param produto_nacional: Booleando sendo verdadeiro para produto nacional e falso para
        produtos importados
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type det: det
        :type produto_nacional: bool
        :type context: dict
        """
        context = context and context.copy() or {}
        trib_inter_uf_destino = \
            self.pool.get('fiscal.tributacao.interestadual').fistin_get_tributacao_interna(
                cr, uid, end_dest.get('uf_id'), trib.get('encargo_id'), context)
        trib_interestadual = \
            self.pool.get('fiscal.tributacao.interestadual').fistin_get_tributacao_interestadual(
                cr, uid, end_dest.get('uf_id'), trib.get('encargo_id'), produto_nacional, context)
        ano_nf = datetime.strptime(dados_nf.get('data_emissao'), '%Y-%m-%d %H:%M:%S').year
        percentual_difal = \
            self.pool.get('fiscal.nf.item.tributacao').fisnit_get_percentual_difal(ano_nf)
        aliquota_interna = trib_inter_uf_destino.get('aliquota') or 0
        aliquota_interestadual = trib_interestadual.get('aliquota') or 0
        aliquota_interestadual -= trib_interestadual.get('aliquota_fcp') or 0
        if trib.get('fisnit_vr_difal_uf_destino') and self._consumidor_final:
            det.imposto.ICMSUFDest.vBCUFDest.valor = \
                '%.2f' % trib.get('fisnit_vr_difal_base') or 0
            det.imposto.ICMSUFDest.pFCPUFDest.valor = '%.2f' % 0
            det.imposto.ICMSUFDest.pICMSUFDest.valor = '%.4f' % aliquota_interna or 0
            det.imposto.ICMSUFDest.pICMSInter.valor = '%.2f' % aliquota_interestadual or 0
            det.imposto.ICMSUFDest.pICMSInterPart.valor = \
                '%.4f' % percentual_difal.get('destino') or 0
            det.imposto.ICMSUFDest.vFCPUFDest.valor = '%.2f' % 0
            det.imposto.ICMSUFDest.vICMSUFDest.valor = \
                '%.2f' % trib.get('fisnit_vr_difal_uf_destino') or 0
            det.imposto.ICMSUFDest.vICMSUFRemet.valor = \
                '%.2f' % trib.get('fisnit_vr_difal_uf_origem') or 0

    def set_regime_tributario_outros_icms(self, cr, user, dados_nf, end_dest, produto, trib,
                                          tributacao, dict_cst, cfop, cest_produtos, dict_cest, det,
                                          anp, context=None):
        context = context and context.copy() or {}

        # Grupo Repasse (CST 41 ou 60)
        if not dados_nf.get('fisnfc_consumidor'):
            det.imposto.ICMS.repasse = det.prod.comb.cProdANP.valor == 210203001

        codigo_cst = dict_cst.get(trib.get('cst_id')).get('codigo')
        det.imposto.ICMS.CST.valor = codigo_cst
        det.imposto.ICMS.orig.valor = int(produto.get('origem_mercadoria'))
        vr_encargo = trib.get('vr_base_calculo') * trib.get('aliquota') / 100.
        det.imposto.ICMS.pICMS.valor = '%.2f' % trib.get('aliquota') or 0
        det.imposto.ICMS.vICMS.valor = '%.2f' % vr_encargo or 0

        # Em operacoes interestaduais (idDest=2) para consumidor final (indFinal=1) nao
        # contribuinte (indIEDest=9), o valor do FCP deve ser informado no campo vFCPUFDest
        if self._operacao_interestadual and self._consumidor_final:
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
        else:
            det.imposto.ICMS.pFCP.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp')
            det.imposto.ICMS.vFCP.valor = '%.2f' % trib.get('fisnit_vr_fcp') or 0

        if codigo_cst == '00':  # Tributado integralmente
            det.imposto.ICMS.modBC.valor = u'3'
            det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
        elif codigo_cst == '10':  # Tributado e com cobranca por substituicao tributaria
            modbc, modbcst = self._get_modbc(trib.get('modbcst'))
            det.imposto.ICMS.modBC.valor = modbc
            det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.ICMS.modBCST.valor = modbcst
            det.imposto.ICMS.vBCST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pICMSST.valor = '%.2f' % trib.get('aliquota')
            det.imposto.ICMS.vICMSST.valor = '%.2f' % trib.get('vr_icms_substituicao')
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pFCPST.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp')
            det.imposto.ICMS.vFCPST.valor = '%.2f' % trib.get('fisnit_vr_fcp_st')
            det.prod.CEST.valor = dict_cest.get(cest_produtos.get(produto.get('id'))).get(
                'fiscest_codigo') if dict_cest.get(cest_produtos.get(produto.get('id'))) else ''
        elif codigo_cst == '20':  # Com reducao de base de calculo
            det.imposto.ICMS.modBC.valor = u'3'
            det.imposto.ICMS.pRedBC.valor = '%.4f' % (trib.get('vr_reducao_base') * 100. / (
                trib.get('vr_base_calculo') + trib.get('vr_reducao_base')))
            det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % trib.get('vr_base_calculo')

            det.prod.cBenef.valor = tributacao.get('fistri_codigo_beneficio')
        elif codigo_cst == '30':  # Isenta ou nao tributado com substituicao tributaria
            det.imposto.ICMS.pICMS.valor = '%.2f' % 0
            det.imposto.ICMS.vICMS.valor = '%.2f' % 0
            det.imposto.ICMSUFDest.pFCPUFDest.valor = '%.4f' % 0
            det.imposto.ICMSUFDest.vFCPUFDest.valor = '%.2f' % 0
            det.imposto.ICMS.modBCST.valor = trib.get('modbcst') or '4'
            det.imposto.ICMS.vBCST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pICMSST.valor = '%.2f' % trib.get('aliquota')
            det.imposto.ICMS.vICMSST.valor = '%.2f' % vr_encargo
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pFCPST.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp')
            det.imposto.ICMS.vFCPST.valor = '%.2f' % trib.get('fisnit_vr_fcp_st')
            det.prod.CEST.valor = dict_cest.get(cest_produtos.get(produto.get('id'))).get(
                'fiscest_codigo') if dict_cest.get(cest_produtos.get(produto.get('id'))) else ''

            det.prod.cBenef.valor = tributacao.get('fistri_codigo_beneficio')
        elif codigo_cst == '40':  # Isento
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
            if self._motivo_desoneracao:
                det.imposto.ICMS.vICMSDeson.valor = '%.2f' % trib.get('fisnit_vr_icms_desonerado')
                det.imposto.ICMS.motDesICMS.valor = int(self._motivo_desoneracao)

            det.prod.cBenef.valor = tributacao.get('fistri_codigo_beneficio')
        elif codigo_cst in ('41', '50'):  # Nao tributado / com suspensao
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
        elif codigo_cst == '51':  # Com diferimento
            det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % trib.get('vr_base_calculo')
        elif codigo_cst == '60':  # Cobrado anteriormente com substituicao tributaria
            det.prod.CEST.valor = dict_cest.get(cest_produtos.get(produto.get('id'))).get(
                'fiscest_codigo') if dict_cest.get(cest_produtos.get(produto.get('id'))) else ''
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % trib.get('vr_base_calculo')

            # Gas de cozinha (GLP) tem tratamento diferente - obriga o grupo de repasse
            if det.prod.comb.cProdANP.valor == 210203001:
                det.imposto.ICMS.vBCSTDest.valor = '%.2f' % trib.get('vr_base_calculo')
                det.imposto.ICMS.vICMSSTRet.valor = '%.2f' % trib.get('vr_icms_substituicao')
                det.imposto.ICMS.vICMSSTDest.valor = '%.2f' % trib.get('vr_icms_substituicao')
            else:
                det.imposto.ICMS.pFCP.valor = '%.4f' % 0
                det.imposto.ICMS.vFCP.valor = '%.2f' % 0
                det.imposto.ICMS.vICMSST.valor = '%.2f' % trib.get('vr_icms_substituicao')
        elif codigo_cst == '61':  # Monofásico combustiveis
            det.prod.CEST.valor = dict_cest.get(cest_produtos.get(produto.get('id'))).get(
                'fiscest_codigo') if dict_cest.get(cest_produtos.get(produto.get('id'))) else ''
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % trib.get('vr_base_calculo')
            if det.prod.comb.cProdANP.valor == 210203001:
                det.imposto.ICMS.qBCMonoRet.valor = '%.4f' % float(det.prod.qTrib.valor)
                det.imposto.ICMS.adRemICMSRet.valor = '%.4f' % anp.get('fisanp_adrem') or 0
                vICMSMonoRet = anp.get('fisanp_adrem') * float(det.prod.qTrib.valor)
                det.imposto.ICMS.vICMSMonoRet.valor = '%.2f' % vICMSMonoRet
        elif codigo_cst == '70':  # Com reducao da base de calculo e por substituicao tributaria
            modbc, modbcst = self._get_modbc(trib.get('modbcst'))
            det.imposto.ICMS.modBC.valor = modbc
            det.imposto.ICMS.pRedBC.valor = '%.4f' % trib.get('aliquota')
            det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pFCPST.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp')
            det.imposto.ICMS.vFCPST.valor = '%.2f' % trib.get('fisnit_vr_fcp_st')
            det.imposto.ICMS.modBCST.valor = modbcst
            det.imposto.ICMS.vBCST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.vICMSST.valor = '%.2f' % trib.get('vr_icms_substituicao')
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % trib.get('vr_base_calculo')
            det.prod.CEST.valor = dict_cest.get(cest_produtos.get(produto.get('id'))).get(
                'fiscest_codigo') if dict_cest.get(cest_produtos.get(produto.get('id'))) else ''

            det.prod.cBenef.valor = tributacao.get('fistri_codigo_beneficio')
        elif codigo_cst == '90':  # Outras tributacoes
            modbc, modbcst = self._get_modbc(trib.get('modbcst'))
            det.imposto.ICMS.modBC.valor = modbc
            det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % trib.get('vr_base_calculo')
            det.prod.CEST.valor = dict_cest.get(cest_produtos.get(produto.get('id'))).get(
                'fiscest_codigo') if dict_cest.get(cest_produtos.get(produto.get('id'))) else ''

        if det.imposto.ICMS.modBCST.valor == 4:
            det.imposto.ICMS.pMVAST.valor = str(trib.get('percentual_mva') or 0)

        # Isentos
        if dict_cst.get(tributacao.get('cst_id')).get('tipo') == u'I':
            det.imposto.ICMS.vBC.valor = '%.2f' % 0

        if cfop.get('codigo')[0] == '6':  # Operacoes interestaduais (calcular DIFAL)
            produto = self.pool.get('estoque.produto').estpro_produto_nacional(
                produto.get('origem_mercadoria'))

            # Verifica se a natureza da operacao permite calcular DIFAL
            if codigo_cst not in ('40', '41') or self.pool.get('fiscal.cfop.natureza').search(
                    cr, user, [('id', '=', dados_nf.get('natureza_id')),
                               ('fisnat_calcula_difal', '=', True)], context=context):
                self.gerar_schema_ICMSUFDest(
                    cr, user, det, dados_nf, end_dest, trib, produto, context)

    def set_regime_tributario_outros_pis(self, cr, user, item, trib, cst, det, context=None):
        context = context and context.copy() or {}
        codigo_cst = cst.get('codigo')
        codigo_cst = '99' if codigo_cst == '05' else codigo_cst
        det.imposto.PIS.CST.valor = codigo_cst
        if codigo_cst in ('01', '02'):
            det.imposto.PIS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.PIS.pPIS.valor = '%.4f' % trib.get('aliquota')
            det.imposto.PIS.vPIS.valor = '%.2f' % trib.get('vr_encargo')
        elif codigo_cst == '03':
            det.imposto.PIS.qBCProd.valor = '%.4f' % item.get('quantidade')
            det.imposto.PIS.vAliqProd.valor = \
                '%.4f' % (trib.get('vr_base_calculo') * (trib.get('aliquota') / 100.))
            det.imposto.PIS.vPIS.valor = '%.2f' % trib.get('vr_encargo')
        elif codigo_cst in ('04', '06', '07', '08', '09'):
            pass
        elif codigo_cst in ('49', '50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63',
                            '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99'):
            det.imposto.PIS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.PIS.pPIS.valor = '%.4f' % trib.get('aliquota')
            det.imposto.PIS.vPIS.valor = '%.2f' % trib.get('vr_encargo')
            det.imposto.PIS.qBCProd.valor = '%.4f' % item.get('quantidade')
            det.imposto.PIS.vAliqProd.valor = \
                '%.4f' % (trib.get('vr_base_calculo') * (trib.get('aliquota') / 100.))
            det.imposto.PIS.vPIS.valor = '%.2f' % trib.get('vr_encargo')

    def set_regime_tributario_outros_cofins(self, cr, user, item, trib, cst, det, context=None):
        context = context and context.copy() or {}
        codigo_cst = cst.get('codigo')
        codigo_cst = '99' if codigo_cst == '05' else codigo_cst
        det.imposto.COFINS.CST.valor = codigo_cst
        if codigo_cst in ('01', '02'):
            det.imposto.COFINS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.COFINS.pCOFINS.valor = '%.4f' % trib.get('aliquota')
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % trib.get('vr_encargo')
        elif codigo_cst == '03':
            det.imposto.COFINS.qBCProd.valor = '%.4f' % item.get('quantidade')
            det.imposto.COFINS.vAliqProd.valor = '%.4f' % (
                trib.get('vr_base_calculo') * (trib.get('aliquota') / 100.))
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % trib.get('vr_encargo')
        elif codigo_cst in ('04', '06', '07', '08', '09'):
            pass
        elif codigo_cst in ('49', '50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63',
                            '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99'):
            det.imposto.COFINS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
            det.imposto.COFINS.pCOFINS.valor = '%.4f' % trib.get('aliquota')
            det.imposto.COFINS.qBCProd.valor = '%.2f' % item.get('quantidade')
            det.imposto.COFINS.vAliqProd.valor = \
                '%.4f' % (trib.get('vr_base_calculo') * (trib.get('aliquota') / 100.))
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % trib.get('vr_encargo')

    def set_regime_tributario_outros_ipi(self, cr, user, trib, cst, det, cfop, context=None):
        context = context and context.copy() or {}
        devolucao = cfop and self.pool.get('fiscal.cfop').read(
            cr, user, cfop.get('id'), ['movimento_tipo'], context).get('movimento_tipo') == 'D' \
            or False
        codigo_cst = cst.get('codigo')
        det.imposto.IPI.CST.valor = codigo_cst
        if codigo_cst in ('00', '49', '50', '99'):
            if devolucao:
                det.impostoDevol.pDevol.valor = '%.2f' % 100
                det.impostoDevol.IPI.vIPIDevol.valor = '%.2f' % trib.get('vr_encargo')
            else:
                det.imposto.IPI.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
                det.imposto.IPI.pIPI.valor = '%.4f' % trib.get('aliquota')
                det.imposto.IPI.vIPI.valor = '%.2f' % trib.get('vr_encargo')

        if codigo_cst in ('01', '02', '03', '04', '51', '52', '53', '54', '55'):
            pass

    def set_regime_tributario_outros_ii(self, cr, user, trib, det, context=None):
        context = context and context.copy() or {}
        det.imposto.II.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
        det.imposto.II.vDespAdu.valor = '%.2f' % trib.get('vr_despesa_aduaneira')
        det.imposto.II.vII.valor = '%.2f' % trib.get('vr_encargo')
        det.imposto.II.vIOF.valor = '%.2f' % 0

    def set_regime_tributario_outros_iss(self, cr, user, produto, trib, dict_tab_serv, det,
                                         context=None):
        context = context and context.copy() or {}
        det.imposto.ISSQN.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
        det.imposto.ISSQN.vAliq.valor = '%.4f' % trib.get('aliquota')
        det.imposto.ISSQN.vISSQN.valor = '%.2f' % trib.get('vr_encargo')
        codigo_servico = re.sub('[^0-9]', u'',
                                (dict_tab_serv.get(produto.get('codigo_tab_servicos_id') or {})
                                 ).get('codigo') or u'')
        if int(codigo_servico) <= 4001 and len(codigo_servico) in (3, 4):
                det.imposto.ISSQN.cListServ.valor = codigo_servico
        det.imposto.ISSQN.cSitTrib.valor = u'N'

    def set_regime_tributario_outros(self, cr, uid, dados_nf, end_dest, item, produto, dict_trib,
                                     dict_tributacao, dict_encargo, dict_cst, dict_cfop,
                                     dict_tab_serv, cest_produtos, dict_cest, det, nfe, anp, context=None):
        context = context and context.copy() or {}
        for trib_id in item.get('tributacoes_ids'):
            trib = dict_trib.get(trib_id)
            tributacao = dict_tributacao.get(trib.get('tributacao_id'))
            encargo = dict_encargo.get(tributacao.get('encargo_id'))
            cst = dict_cst.get(trib.get('cst_id'))
            cfop = dict_cfop.get(item.get('cfop_id'))
            if encargo.get('sigla') == 'ICMS':
                self.set_regime_tributario_outros_icms(
                    cr, uid, dados_nf, end_dest, produto, trib, tributacao, dict_cst, cfop,
                    cest_produtos, dict_cest, det, anp, context)
            elif encargo.get('sigla') == 'PIS':
                self.set_regime_tributario_outros_pis(cr, uid, item, trib, cst, det, context)
            elif encargo.get('sigla') == 'COFINS':
                self.set_regime_tributario_outros_cofins(cr, uid, item, trib, cst, det, context)
            elif encargo.get('sigla') == 'IPI':
                self.set_regime_tributario_outros_ipi(cr, uid, trib, cst, det, cfop, context)
            elif encargo.get('sigla') == 'II':
                itens = self.pool.get('estoque.movimento.itens').browse(
                    cr, uid, dados_nf.get('estoque_movimento_itens_ids'), context)
                valor_total = sum([i.vr_total_item for i in itens])
                despesa_aduaneira = \
                    dados_nf.get('fisnfc_di_ids') and self.pool.get(
                        'fiscal.declaracao.importacao').browse(
                        cr, uid, dados_nf['fisnfc_di_ids'], context)[0].fisdim_vr_despesa or 0
                percentual_rateio = valor_total and item.get('vr_total_item') / valor_total
                trib['vr_despesa_aduaneira'] = despesa_aduaneira * percentual_rateio
                self.set_regime_tributario_outros_ii(cr, uid, trib, det, context)
            elif encargo.get('sigla') == 'ISS':
                self.set_regime_tributario_outros_iss(
                    cr, uid, produto, trib, dict_tab_serv, det, context)
                det.imposto.ISSQN.cMunFG.valor = nfe.infNFe.emit.enderDest.cMun.valor
            else:  # Nao implementado
                pass

    @staticmethod
    def _get_modbc(modbcst):
        """
        Retorna a modalidade da base de calculo do ICMS/ST.

        :param modbcst: codigo da modalidade
        :type modbcst: str
        :return: modalidade da BC do ICMS/ST
        :rtype: list
        """
        modbc = {'4': '0', '0': '2', '5': '1'}.get(modbcst or '4') or '0'

        return modbc, modbcst or '0'

    def set_regime_tributario_simples_nacional_icms(self, cr, user, dados_nf, end_dest, produto,
                                                    det, trib, tributacao, cfop, cst, csosn,
                                                    cest_produtos, dict_cest, item, context=None):
        """
        Atribui os valores de ICMS para regime tributario Simples Nacional.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param det: item de movimentacao
        :param trib: dados do trib
        :param item: dados do item
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type det: Det (objeto ´nfe.infNFe.det´)
        :type encargo: dict
        :type item: dict
        :type context: dict
        :return: n/a
        :rtype: n/a
        """
        cfop_ids = self.pool.get('fiscal.cfop').search(
            cr, user, [('codigo', '=', cfop.get('codigo'))], context=context)
        cfop_id = cfop_ids and cfop_ids[0]
        devolucao = cfop_id and self.pool.get('fiscal.cfop').read(
            cr, user, cfop_id, ['movimento_tipo'], context).get('movimento_tipo') == 'D'
        saida = dados_nf.get('tipo') == 'S'

        # Regime tributario do cliente/fornecedor para avaliar o destaque de valores de ICMS
        destinatario_id = self.pool.get('fiscal.nf.capa').read(
            cr, user, dados_nf['id'], ['destinatario_id'], context, '_classic_write').get(
            'destinatario_id') or 0
        crt = destinatario_id and self.pool.get('adm.pessoa').read(
            cr, user, destinatario_id, ['admpes_jur_regime_tributario'], context).get(
            'admpes_jur_regime_tributario') or False
        debito_credito = crt and crt != 'S'  # Lucro real ou lucro presumido

        if self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                cr, user, dados_nf.get('destinatario_id'), context) != 'C':
            csosn = cst.get('fiscst_csosn_nao_contribuinte_id') and \
                    csosn.get(cst.get('fiscst_csosn_nao_contribuinte_id')).get('codigo') or u''
        else:
            csosn = cst.get('fiscst_csosn_contribuinte_id') and \
                    csosn.get(cst.get('fiscst_csosn_contribuinte_id')).get('codigo') or u''

        if not csosn:  # Como o CSOSN nao eh obrigatorio evita-se erro ao validar o arquivo .xsd
            produto_nome = self.pool.get('estoque.produto').read(
                cr, user, produto['id'], ['name'], context).get('name')
            sequencia = self.pool.get('estoque.movimento.itens').read(
                cr, user, item['id'], ['sequencia'], context).get('sequencia')
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum CSOSN cadastrado para ICMS/%s no item %s-%s.') % (
                                     cst.get('codigo'), sequencia, produto_nome))

        # Os valores de ICMS serao destacados ou nao a depender das condicoes do cliente/fornecedor
        csosn = u'900' if devolucao and saida else csosn

        # Troca o CSOSN para 900 se for entrada propria e nota fiscal de importacao
        if dados_nf.get('tipo') == 'E' and dados_nf.get('nf_propria'):
            if self.pool.get('fiscal.cfop.natureza').browse(cr
                    , user, dados_nf.get('natureza_id'), context).cfop_id.codigo[0] == '3':
                csosn = u'900'

        det.imposto.ICMS.orig.valor = produto.get('origem_mercadoria')
        det.imposto.ICMS.CSOSN.valor = csosn
        vr_cred_icms_sn = trib.get('vr_base_calculo') * self._fisnfc_get_aliquota_sn(
            cr, user, context) / 100.
        if csosn == '101':
            det.imposto.ICMS.pCredSN.valor = '%.2f' % self._fisnfc_get_aliquota_sn(
                cr, user, context)
            det.imposto.ICMS.vCredICMSSN.valor = '%.2f' % vr_cred_icms_sn
        elif csosn == '201':
            det.imposto.ICMS.modBCST.valor = trib.get('modbcst')
            det.imposto.ICMS.vBCST.valor = \
                '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pICMSST.valor = '%.2f' % trib.get('aliquota')
            det.imposto.ICMS.vICMSST.valor = '%.2f' % trib.get('vr_icms_substituicao')
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pFCPST.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp') or 0
            det.imposto.ICMS.vFCPST.valor = '%.2f' % trib.get('fisnit_vr_fcp_st')
            det.imposto.ICMS.pCredSN.valor = '%.2f' % self._fisnfc_get_aliquota_sn(
                cr, user, context)
            det.imposto.ICMS.vCredICMSSN.valor = '%.2f' % vr_cred_icms_sn
            det.prod.CEST.valor = cest_produtos.get(produto.get('id')) and \
                                  dict_cest.get(cest_produtos.get(produto.get('id'))).get('fiscest_codigo') or ''
        elif csosn in ('202', '203'):
            det.imposto.ICMS.modBCST.valor = trib.get('modbcst')
            det.imposto.ICMS.vBCST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pICMSST.valor = '%.2f' % trib.get('aliquota')
            det.imposto.ICMS.vICMSST.valor = '%.2f' % trib.get('vr_icms_substituicao')
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pFCPST.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp') or 0
            det.imposto.ICMS.vFCPST.valor = '%.2f' % trib.get('fisnit_vr_fcp_st')
            det.prod.CEST.valor = cest_produtos.get(produto.get('id')) and \
                                  dict_cest.get(cest_produtos.get(produto.get('id'))).get('fiscest_codigo') or ''
        elif csosn == '500':
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % trib.get('vr_base_calculo')
            det.prod.CEST.valor = cest_produtos.get(produto.get('id')) and \
                                  dict_cest.get(cest_produtos.get(produto.get('id'))).get('fiscest_codigo') or ''
        elif csosn == '900':
            modbc, modbcst = self._get_modbc(trib.get('modbcst'))
            det.imposto.ICMS.modBC.valor = modbc
            if cst.get('codigo') == '60': # Para st não destacar a base
                det.imposto.ICMS.vBC.valor = '%.2f' % 0
                det.imposto.ICMS.pICMS.valor = '%.2f' % 0
                det.imposto.ICMS.vICMS.valor = '%.2f' % 0
            else:
                det.imposto.ICMS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
                det.imposto.ICMS.pICMS.valor = '%.2f' % trib.get('aliquota')
                det.imposto.ICMS.vICMS.valor = '%.2f' % trib.get('vr_encargo')
            det.imposto.ICMS.modBCST.valor = modbcst
            det.imposto.ICMS.vBCST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pICMSST.valor = '%.2f' % trib.get('aliquota')
            det.imposto.ICMS.vICMSST.valor = '%.2f' % trib.get('vr_icms_substituicao')
            det.imposto.ICMS.pCredSN.valor = '%.2f' % self._fisnfc_get_aliquota_sn(
                cr, user, context)
            det.imposto.ICMS.vCredICMSSN.valor = '%.2f' % vr_cred_icms_sn
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % trib.get('vr_base_icms_substituicao')
            det.imposto.ICMS.pFCPST.valor = '%.4f' % tributacao.get('fistri_aliquota_fcp') or 0
            det.imposto.ICMS.vFCPST.valor = '%.2f' % trib.get('fisnit_vr_fcp_st')
            det.prod.CEST.valor = cest_produtos.get(produto.get('id')) and \
                                  dict_cest.get(cest_produtos.get(produto.get('id'))).get('fiscest_codigo') or ''

        if det.imposto.ICMS.modBCST.valor == 4:
            det.imposto.ICMS.pMVAST.valor = str(trib.get('percentual_mva') or 0)

        if cfop.get('codigo')[0] == u'6':
            produto = self.pool.get('estoque.produto').estpro_produto_nacional(
                produto.get('origem_mercadoria'))

            # Verifica se a natureza da operacao permite calcular DIFAL
            if csosn not in ('103', '300', '400') or self.pool.get('fiscal.cfop.natureza').search(
                    cr, user, [('id', '=', dados_nf.get('natureza_id')),
                               ('fisnat_calcula_difal', '=', True)], context=context):
                self.gerar_schema_ICMSUFDest(
                    cr, user, det, dados_nf, end_dest, trib, produto, context)

        # Para tipo tributado soma o aproveitamento de credito
        if cst.get('tipo') == u'T':
            # Aproveitamento de credito
            self.total_aproveitamento_credito += \
                (trib.get('vr_base_calculo') * self._fisnfc_get_aliquota_sn(
                    cr, user, context) / 100.)

    def set_regime_tributario_simples_nacional(self, cr, user, dados_nf, end_dest, item, produto,
                                               dict_trib, dict_tributacao, dict_encargo, dict_cfop,
                                               dict_cst, dict_csosn, cest_produtos, dict_cest, det,
                                               nfe, context=None):
        """
        Atribui valores para o regime Simples Nacional
        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item: o item da nota fiscal
        :param det: o item (detalhe)  da nota fiscal
        :param nfe: dados da nota fiscal eletronica
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type det: Det (objeto 'det')
        :type nfe: NFe (objeto ´nfe´)
        :type context: dict
        :return: n/a
        :rtype: n/a
        """
        context = context and context.copy() or {}
        for trib_id in item.get('tributacoes_ids'):
            trib = dict_trib.get(trib_id)
            tributacao = dict_tributacao.get(trib.get('tributacao_id'))
            encargo = dict_encargo.get(tributacao.get('encargo_id'))
            cst = dict_cst.get(trib.get('cst_id'))
            cfop = dict_cfop.get(item.get('cfop_id'))
            cfop_ids = self.pool.get('fiscal.cfop').search(
                cr, user, [('codigo', '=', cfop.get('codigo'))], context=context)
            cfop_id = cfop_ids and cfop_ids[0]
            devolucao = cfop_id and self.pool.get('fiscal.cfop').read(
                cr, user, cfop_id, ['movimento_tipo'], context).get('movimento_tipo') == 'D'
            if encargo.get('sigla').upper() == 'ICMS':
                det.imposto.ICMS.regime_tributario = 1  # Simples Nacional
                self.set_regime_tributario_simples_nacional_icms(
                    cr, user, dados_nf, end_dest, produto, det, trib, tributacao, cfop, cst,
                    dict_csosn, cest_produtos, dict_cest, item, context)
                if nfe.infNFe.ide.finNFe.valor == 2:  # Nota complementar
                    if det.imposto.ICMS.CSOSN.valor == '500':
                        det.imposto.ICMS.vICMSSTRet.valor = \
                            '%.2f' % trib.get('vr_icms_substituicao')
            if encargo.get('sigla').upper() == 'IPI':
                det.imposto.IPI.CST.valor = u'99'  # Outras saidas
                if devolucao:
                    det.impostoDevol.pDevol.valor = '%.2f' % 100
                    det.impostoDevol.IPI.vIPIDevol.valor = '%.2f' % trib.get('vr_encargo')
                    det.imposto.IPI.pIPI.valor = '%.4f' % 0
                    det.imposto.IPI.vIPI.valor = '%.2f' % 0

            if encargo.get('sigla').upper() == 'PIS':
                det.imposto.PIS.CST.valor = u'99'  # Outras operacoes
                if devolucao:
                    det.imposto.PIS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
                    det.imposto.PIS.pPIS.valor = '%.4f' % trib.get('aliquota')
                    det.imposto.PIS.vPIS.valor = '%.2f' % trib.get('vr_encargo')
            if encargo.get('sigla').upper() == 'COFINS':
                det.imposto.COFINS.CST.valor = u'99'  # Outras operacoes
                if devolucao:
                    det.imposto.COFINS.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
                    det.imposto.COFINS.pCOFINS.valor = '%.4f' % trib.get('aliquota')
                    det.imposto.COFINS.vCOFINS.valor = '%.2f' % trib.get('vr_encargo')
            if encargo.get('sigla').upper() == 'II':
                itens = self.pool.get('estoque.movimento.itens').browse(
                    cr, user, dados_nf.get('estoque_movimento_itens_ids'), context)
                valor_total = sum([i.vr_total_item for i in itens])
                despesa_aduaneira = \
                    dados_nf.get('fisnfc_di_ids') and self.pool.get(
                        'fiscal.declaracao.importacao').browse(
                        cr, user, dados_nf['fisnfc_di_ids'], context)[0].fisdim_vr_despesa or 0
                percentual_rateio = valor_total and item.get('vr_total_item') / valor_total
                trib['vr_despesa_aduaneira'] = despesa_aduaneira * percentual_rateio
                det.imposto.II.vBC.valor = '%.2f' % trib.get('vr_base_calculo')
                det.imposto.II.vDespAdu.valor = '%.2f' % trib.get('vr_despesa_aduaneira')
                det.imposto.II.vII.valor = '%.2f' % trib.get('vr_encargo')
                det.imposto.II.vIOF.valor = '%.2f' % 0

    def _fisnfc_get_aliquota_sn(self, cr, user, context=None):
        """
        Recupera o valor da aliquota do Simples Nacional.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type context: dict
        :return: n/a
        :rtype: n/a
        """
        context = context and context.copy() or {}
        simples_ids = self.pool.get('fiscal.tributacao.simples.nacional').search(cr, user, [],
                                                  order='fistsn_valor_faturamento', context=context)
        aliquota_sn = 0
        faturamento_anual = self.pool.get('adm.utils').admutl_get_parametros(cr, user, 'fiscal', [],
                                                        context=context).get('fispar_receita_anual')
        for r in self.pool.get('fiscal.tributacao.simples.nacional').read(cr, user, simples_ids,
                                 ['fistsn_valor_faturamento', 'fistsn_aliq_icms'], context=context):
            if faturamento_anual <= r.get('fistsn_valor_faturamento'):
                aliquota_sn = r.get('fistsn_aliq_icms') or 0
                break

        return aliquota_sn

    def nfe_infnfe_det_infadprod(self, cr, user, item, context=None):
        """
        Acrescenta informacoes adicionais aos dados do item.
        
        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type context: dict
        :return: lista com as informacoes adicionais do item.
        :rtype: list
        """
        # //NFe.infNFe.det.infAdProd - Informacoes adicionais do produto
        informacoes = []
        brw_item = self.pool.get('estoque.movimento.itens').browse(cr, user, item, context)
        if brw_item.produto_embalagem_id.embalagem_id.quantidade_na_embalagem > 1:
            if self._parametros(cr, user).fiscal.fispar_imprimir_qtde_embalagem_danfe:
                informacoes.append('%s c/%.0f' % (
                    brw_item.produto_embalagem_id.embalagem_id.unidade_id.name,
                    brw_item.produto_embalagem_id.embalagem_id.quantidade_na_embalagem))
        regime_tributario = brw_item.nf_id.company_id.rescon_pessoa_id.admpes_jur_regime_tributario
        for itemTributacaoRcs in brw_item.tributacoes_ids:
            if itemTributacaoRcs.encargo_id.sigla == 'ICMS':
                if itemTributacaoRcs.cst_id.codigo == '61':
                    informacoes.append(_('ICMS monofasico sobre combustiveis cobrado anteriormente conforme Convenio ICMS 199/2022;\n'))
                    break

        return informacoes

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
