# -*- encoding: utf-8 -*-

__author__ = 'marcilene.ribeiro'
from openerp.osv import osv
from openerp.tools.translate import _
from pynfe.nfse.nfse_202 import EnviarLoteRpsSincronoEnvio, tcDeclaracaoPrestacaoServico
from datetime import datetime
from decimal import Decimal


# noinspection PyPep8Naming
class fiscal_nf_capa(osv.Model):
    _inherit = 'fiscal.nf.capa'

    @staticmethod
    def remove_accents(text):
        from unicodedata import normalize, category
        unicode_string = unicode(text, 'utf-8') if type(text) == str else text
        text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

        return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

    def verifica_cod_servico(self, cr, uid, dados_itens, dados_produtos, dados_cod_servico,
                             context=None):
        '''Verifica se os itens de uma nota possuem codigos diferente. Todos os itens da mesma 
          nota devem ter o mesmo codigo'''
        context = context and context.copy() or {}
        dict_nf = {}
        nf_ids = []
        nf_ids_sem_cod = []
        for i in dados_itens.values():
            produto = dados_produtos.get(i.get('produto_id'))
            cod_servico = dados_cod_servico.get(produto.get('codigo_tab_servicos_id')) and \
                          dados_cod_servico.get(produto.get('codigo_tab_servicos_id')).get('codigo') \
                          or False
            if not cod_servico:
                nf_ids_sem_cod.append(i.get('nf_id'))
                continue

            if dict_nf.get(i.get('nf_id')) and cod_servico in dict_nf.get(i.get('nf_id')):
                nf_ids.append(i.get('nf_id'))

            dict_nf[i.get('nf_id')] = cod_servico

        if not nf_ids and not nf_ids_sem_cod:
            return True

        msg = []
        if nf_ids_sem_cod:
            for i in self.read(cr, uid, nf_ids_sem_cod, ['fisnfc_rps_numero', 'serie_nf'], context):
                msg.append(_('Numero: %s Serie: %s') % (i.get('fisnfc_rps_numero'), i.get('serie_nf')))

            raise osv.except_osv(_('Atencao!'),
                                 _('As notas a seguir nao possuem codigos nos servicos:\n%s') %
                                 '\n'.join(msg))

        for i in self.read(cr, uid, nf_ids, ['fisnfc_rps_numero', 'serie_nf'], context):
            msg.append(_('Numero: %s Serie: %s') % (i.get('fisnfc_rps_numero'), i.get('serie_nf')))

        raise osv.except_osv(_('Atencao!'),
                             _('As notas a seguir possuem servicos com codigos diferente:\n%s') %
                             '\n'.join(msg))

    def gerar_nfse_xml(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        load = '_classic_write'
        dados_nf = self.read(cr, uid, ids, [], context=context, load=load)
        destinatario_ids = []
        item_ids = []
        for i in dados_nf:
            destinatario_ids.append(i.get('destinatario_id'))
            item_ids.extend(i.get('estoque_movimento_itens_ids'))

        obj_res_company = self.pool.get('res.company')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_adm_utl = self.pool.get('adm.utils')
        obj_doc = self.pool.get('adm.pessoa.documento.dados')
        obj_par = self.pool.get('fiscal.parametro')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        mensagem = ''  # Mensagens de erro

        nf_company_id = dados_nf[0].get('company_id')
        empresa_nome = self.pool.get('res.company').browse(cr, uid, nf_company_id).name
        emitente_id = obj_res_company.read(cr, uid, nf_company_id, ['rescon_pessoa_id'],
                                           context=context, load=load).get('rescon_pessoa_id')
        context['super_read_adm_pessoa'] = True
        dados_emitente = obj_adm_pessoa.read(cr, uid, emitente_id, ['admpes_jur_cnae_id',
                                            'cpf_cnpj'], context=context, load=load)
        campos_dest = ['name', 'cpf_cnpj', 'email', 'telefone', 'pessoa_tipo']
        dados_destinatarios = obj_adm_utl.admutl_get_dados_classe(cr, uid, 'adm.pessoa',
                                                       destinatario_ids, campos_dest, context, load)
        campos_itens = ['produto_id', 'nf_id']
        dados_itens = obj_adm_utl.admutl_get_dados_classe(cr, uid, 'estoque.movimento.itens',
                                                          item_ids, campos_itens, context, load)
        produto_ids = map(lambda x: x.get('produto_id'), dados_itens.values())
        campos_prod = ['codigo_tab_servicos_id', 'name']
        dados_produtos = obj_adm_utl.admutl_get_dados_classe(cr, uid, 'estoque.produto',
                                                            produto_ids, campos_prod, context, load)
        cod_servico_ids = list(set(map(lambda x: x.get('codigo_tab_servicos_id'),
                                       dados_produtos.values())))
        if False in cod_servico_ids:
            cod_servico_ids.remove(False)

        dados_cod_servico = obj_adm_utl.admutl_get_dados_classe(cr, uid, 'fiscal.tabela.servico',
                                                                cod_servico_ids, [], context, load)
        self.verifica_cod_servico(cr, uid, dados_itens, dados_produtos, dados_cod_servico, context)
        id_par = obj_par.search(cr, uid, [('company_id', '=', nf_company_id)], context=context)
        envisoRps = EnviarLoteRpsSincronoEnvio()

        enderecos = obj_adm_pessoa.admpes_endereco_fiscal(cr, uid, destinatario_ids, context)

        documento_ids = obj_doc.search(cr, uid, [('pessoa_id', '=', emitente_id)], context=context)
        p_inscricao_municipal = ''
        if documento_ids:
            browse_documento = obj_doc.browse(cr, uid, documento_ids)
            for i in browse_documento:
                sigla = i.tipo_id.sigla

                if sigla == 'IM':  # Nao obrigatorio
                    p_inscricao_municipal = '%s' % (i.numero or '')

        documento_ids = obj_doc.search(cr, uid, [('pessoa_id', 'in', destinatario_ids)],
                                       context=context)
        inscricoes_municipal = {}
        if documento_ids:
            dados_documento = obj_doc.read(cr, uid, documento_ids, ['pessoa_id', 'tipo_id'], context, load)
            dict_tipos = obj_adm_utl.admutl_get_dados_classe(cr, uid, 'adm.pessoa.documento.dados',
                                                             ['sigla'], context)
            for i in dados_documento:
                sigla = dict_tipos.get(i.get('tipo_id').get('sigla'))
                if sigla == 'IM':  # Nao obrigatorio
                    inscricoes_municipal[i.get('pessoa_id')] = '%s' % (i.get('numero') or '')
        # Info Rps
        lista_rps = []
        nf_id_rps = {}
        for i in dados_nf:
            endereco = enderecos.get(i.get('destinatario_id'))
            destinatario = dados_destinatarios.get(i.get('destinatario_id'))
            fisnfc_rps_numero = self.pool.get('adm.utils').get_sequence(cr, uid, self._name,
                                                                        'numero_rps.fiscal')
            id = '%s_%s' % (fisnfc_rps_numero, i.get('serie_nf'))
            nf_id_rps[i.get('id')] = fisnfc_rps_numero
            t_cpf = ''
            t_cnpj = ''
            t_inscricao_municipal = ''
            if destinatario.get('pessoa_tipo') == 'J':
                t_cnpj = destinatario.get('cpf_cnpj')
                t_inscricao_municipal = inscricoes_municipal.get(i.get('destinatario_id')) or ''
            else:
                t_cpf = destinatario.get('cpf_cnpj')

            item = dados_itens.get(i.get('estoque_movimento_itens_ids')[0])
            produto = dados_produtos.get(item.get('produto_id'))
            cod_servico = dados_cod_servico.get(produto.get('codigo_tab_servicos_id'))
            item_lista_servico = cod_servico.get('codigo')
            discriminacao = [cod_servico.get('name')]
            for item_id in i.get('estoque_movimento_itens_ids'):
                dados_prod = dados_produtos.get(dados_itens.get(item_id).get('produto_id'))
                discriminacao.append(dados_prod.get('name'))

            discriminacao = self.remove_accents((', '.join(discriminacao))[:2000])

            declaracao = tcDeclaracaoPrestacaoServico('Rps', raiz='%s%s/' %
                                           (envisoRps.LoteRps.raiz_tag, envisoRps.LoteRps.nome_tag))
            infRps = declaracao.InfDeclaracaoPrestacaoServico
            infRps.Id.valor = 'InfDec_Rps_%s' % id
            # declaracao.Signature.URI = infRps.Id.valor
            # declaracao.Signature.Id = 'Nfse_Ass_Rps_%s' % id
            infRps.Competencia.valor = datetime.strptime(i.get('data_emissao'), '%Y-%m-%d %H:%M:%S')
            infRps.ConstrucaoCivil.valor = ''
            infRps.IncentivoFiscal.valor = 2   # 1=Sim, 2=Nao

            # Nao popular intermediario
            # infRps.Intermediario.IdentificacaoIntermediario.CpfCnpj.Cnpj.valor= ''
            # infRps.Intermediario.IdentificacaoIntermediario.InscricaoMunicipal.valor = ''
            # infRps.Intermediario.RazaoSocial.valor = ''
            rps = infRps.Rps
            rps.Id.valor = 'Rps_%s' % id
            rps.IdentificacaoRps.Numero.valor = fisnfc_rps_numero
            rps.IdentificacaoRps.Serie.valor = i.get('serie_nf')
            rps.IdentificacaoRps.Tipo.valor = 1 # 1=RPS; 2=Nota Fiscal Conjugada (Mista); 3=Cupom
            rps.DataEmissao.valor = datetime.strptime(i.get('data_emissao'), '%Y-%m-%d %H:%M:%S')
            rps.Status.valor = 1  # 1=Normal; 2=Cancelado
            # rps.RpsSubstituido.Numero.valor = 0
            # rps.RpsSubstituido.Serie.valor = ''
            # rps.RpsSubstituido.Tipo.valor = 1

            servico = infRps.Servico
            servico.Valores.ValorServicos.valor = Decimal(i.get('vr_total_nf'))
            servico.Valores.ValorDeducoes.valor = 0
            servico.Valores.ValorPis.valor = 0
            servico.Valores.ValorCofins.valor = 0
            servico.Valores.ValorInss.valor = 0
            servico.Valores.ValorIr.valor = 0
            servico.Valores.ValorCsll.valor = 0
            servico.Valores.OutrasRetencoes.valor = 0
            servico.Valores.ValorIss.valor = 0
            servico.Valores.Aliquota.valor = 0
            servico.Valores.DescontoIncondicionado.valor = 0
            servico.Valores.DescontoCondicionado.valor = 0
            servico.IssRetido.valor = 1  # Sim
            servico.ItemListaServico.valor = item_lista_servico
            servico.CodigoCnae.valor = 0
            servico.CodigoTributacaoMunicipio.valor = ''
            servico.Discriminacao.valor = discriminacao
            servico.CodigoMunicipio.valor = 0
            servico.CodigoPais.valor = ''
            servico.ExigibilidadeISS.valor = 1  # Exigivel;
            servico.MunicipioIncidencia.valor = 0

            infRps.Prestador.CpfCnpj.Cnpj.valor = dados_emitente.get('cpf_cnpj')
            infRps.Prestador.InscricaoMunicipal.valor = p_inscricao_municipal

            tomador = infRps.Tomador
            tomador.IdentificacaoTomador.CpfCnpj.Cpf.valor = t_cpf
            tomador.IdentificacaoTomador.CpfCnpj.Cnpj.valor = t_cnpj
            tomador.IdentificacaoTomador.InscricaoMunicipal.valor = t_inscricao_municipal
            tomador.RazaoSocial.valor = empresa_nome
            tomador.Endereco.Endereco.valor = endereco.get('logradouro') or ''
            tomador.Endereco.Numero.valor = endereco.get('numero') or ''
            tomador.Endereco.Complemento.valor = endereco.get('complemento') or ''
            tomador.Endereco.Bairro.valor = endereco.get('bairro') or ''
            tomador.Endereco.CodigoMunicipio.valor = endereco.get('codigo_ibge') or ''
            tomador.Endereco.Uf.valor = endereco.get('uf_sigla') or ''
            tomador.Endereco.CodigoPais.valor = endereco.get('codigo_pais') or ''
            tomador.Endereco.Cep.valor = endereco.get('cep') or ''
            tomador.Contato.Telefone.valor = destinatario.get('telefone') or ''
            tomador.Contato.Email.valor = destinatario.get('email') or ''

            infRps.RegimeEspecialTributacao.valor = 1  # Microempresa Municipal;
            infRps.OptanteSimplesNacional.valor = 1  # Sim
            infRps.IncentivoFiscal.valor = 2  # Nao
            lista_rps.append(declaracao)

        # info LoteRps
        envisoRps.LoteRps.Id.valor = u'lote'
        envisoRps.LoteRps.CpfCnpj.Cnpj.valor = dados_emitente.get('cpf_cnpj')
        envisoRps.LoteRps.InscricaoMunicipal.valor = p_inscricao_municipal
        envisoRps.LoteRps.NumeroLote.valor = 1
        envisoRps.LoteRps.QuantidadeRps.valor = len(ids)
        envisoRps.LoteRps.Rps.extend(lista_rps)

        # Assinatura
        envisoRps.Signature.URI = envisoRps.LoteRps.Id.valor
        envisoRps.Signature.Id = 'Nfse_Ass_lote'
        try:
            envisoRps.validar()
        except Exception as e:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro de estrutura do xml:\n%s' % e))

        if not id_par:
            raise osv.except_osv(_('Atencao'), _('Parametro do Fiscal nao cadastrado.'))

        id_estoque_parametro = obj_estoque_parametro.search(cr, uid,
                                              [('company_id', '=', nf_company_id)], context=context)

        if not id_estoque_parametro:
            raise osv.except_osv(_('Atencao'), _('Parametro do Estoque nao cadastrado.'))

        # uf = endereco_cliente.get('uf_sigla') or ''
        for i in dados_destinatarios:
            if not dados_destinatarios.get(i).get('cpf_cnpj'):
                mensagem += _('* CPF/CNPJ do cliente nao cadastrado: %s\n' %
                              dados_destinatarios.get(i).get('name'))

        if mensagem:
            raise osv.except_osv(_('Atencao'), _(mensagem))

        for i in ids:
            self.write(cr, uid, i, {'fisnfc_rps_numero': nf_id_rps.get(i),
                                    'nfe_xml_auto': envisoRps.xml}, context=context)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
