# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class fiscal_modelo_nf_series(osv.Model):
    _name = 'fiscal.modelo.nf.series'
    _inherit = 'fiscal.modelo.nf.series'  # fismns

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.browse(cr, uid, ids, context=context)
        res = []
        for record in reads:
            _nome = '%s - %s' % (record.modelo_nf_id.fismnf_codigo, record.modelo_nf_id.name)
            if record.modelo_nf_id.fismnf_codigo in ('55', '65'):
                if record.ambiente == 1:
                    ambiente = _('Producao')
                else:
                    ambiente = _('Homologacao')
                _nome += ' v-%s - %s' % (record.versao_xml or '', ambiente)
                if record.tipo_contingencia:
                    operacao = 'Contingencia - ' + record.tipo_contingencia
                    if record.modelo_nf_id.fismnf_codigo == '65':  # NFC-e
                        if self.pool.get('adm.utils').admutl_get_parametros(
                                cr, uid, 'fiscal', []).get('fispar_nfce_contingencia_offline'):
                            operacao = 'Contingencia OFF-LINE'
                else:
                    operacao = 'SEFAZ'
                _nome += _(' - %s - Serie %s') % (operacao, record.serie_nf)
            res.append((record['id'], _nome))
        return res

    def onchange_modelo(self, cr, uid, ids, modelo_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'flag_modelo_nfe': False}}
        if not modelo_id:
            return res

        dados_modelo = self.pool.get('fiscal.modelo.nf').read(cr, uid, modelo_id,
                               ['fismnf_tipo_nfe', 'fismnf_tipo_nfe_servico'], context=context)
        if dados_modelo:
            if dados_modelo.get('fismnf_tipo_nfe'):
                res['value']['flag_modelo_nfe'] = True
                res['value']['tipo_modelo'] = '1'

            elif dados_modelo.get('fismnf_tipo_nfe_servico'):
                res['value']['tipo_modelo'] = '2'

            else:
                res['value']['tipo_modelo'] = ''

        return res

    def button_ativo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'A'}, context=context)
        self.log(cr, uid, ids[0], _('Modelo NF ativo'))
        return True

    def button_esboco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'E'}, context)
        self.log(cr, uid, ids[0], _('Modelo NF em esboco'))
        return True

    def button_desabilitado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        modelo_serie = self.read(cr, uid, ids, ['id'], context=context)
        if modelo_serie:
            if isinstance(modelo_serie, list):
                modelo_serie = modelo_serie[0]
            obj_fiscal_parametro = self.pool.get('fiscal.parametro')
            parametro_id = obj_fiscal_parametro.search(cr, uid,
                                                       [('modelo_nf_series_id', '=', modelo_serie.get('id', 0))],
                                                       context=context)
            if parametro_id:
                try:
                    parametro_id = parametro_id[0]
                except:
                    parametro_id = []
                obj_fiscal_parametro.write(cr, uid, parametro_id, {'modelo_nf_series_id': []}, context)
        self.write(cr, uid, ids, {'state': 'D'}, context)
        self.log(cr, uid, ids[0], _('Modelo NF desabilitado'))
        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if not (context.get('install_mode') and context['install_mode']):
            if not vals.get('parametro_id', False):
                raise osv.except_osv(_('Atencao'),
                                     _('Para Cadastrar modelo de nota fiscal, utilize Cadastro/Fiscal/Modelo NF.'))

        mod = self.search(cr, uid, [('modelo_nf_id', '=', vals.get('modelo_nf_id', False)),
                                    ('serie_nf', '=', vals.get('serie_nf', False)), ('state', '!=', 'D'),
                                    ('parametro_id', '=', vals.get('parametro_id', False))], context=context)
        if mod:
            if context.get('install_mode') and context['install_mode']:
                return mod[0]
            raise osv.except_osv(_('Atencao'),
                                 _('Nao e permitido inserir modelo e serie iguais.'))

        res = super(fiscal_modelo_nf_series, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(fiscal_modelo_nf_series, self).write(cr, uid, ids, vals, context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'),
                             _('Nao e permitido excluir o modelo, somente desabilitar.'))

    def fismnfs_atualiza_modelo_numero_nf(self, cr, uid, ids, context):
        """
        Este metodo atualiza o numero da ultima nota emitida para o modelo passado como parametro
        e retorna um dicionario contendo o ultimo numero de nota e a serie da nota antes de
        atualizar or numero

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: id do modelo a ser atualizado
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: int
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not ids:
            return {}

        if isinstance(ids, int):
            ids = [ids]
        serie_nf = '0'
        numero_nf = 0
        res = {}
        for i in self.browse(cr, uid, ids, context=context):
            serie_nf = i.serie_nf or '0'
            numero_nf = i.numero_nf or 0
            res = {'numero_nf': numero_nf, 'serie_nf': serie_nf}
        self.write(cr, uid, ids, {'serie_nf': serie_nf, 'numero_nf': numero_nf + 1}, context)
        # Commit feito pois pode acontecer de duas notas pegarem o mesmo numero ao mesmo tempo
        cr.execute("""COMMIT;""")
        return res

    def fismnfs_onchange_nfe_contingencia(self, cr, user, ids, contingencia, context=None):
        """
        Verifica se a contigencia pode ser utilizada pela UF.
        
        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs do formulario
        :param contingencia: tipo de contingencia: SVC-AN ou SVC-RS
        :param context: um dicionario padrao para valores contextuais,
        :type cr: 
        :type user: 
        :type ids: 
        :type contingencia: 
        :type context: 
        :return: mensagem de aviso 
        :rtype: 
 
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br> 
        """
        context = context and context.copy() or dict()
        result = dict()
        if not contingencia:
            result = dict(value=dict(motivo_scan=False, data_scan=False))

        return result

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        # Nao deve ser aplicado quando a selecao eh feita no parametro do Fiscal
        if not context.get('fiscal_parametro'):
            arg = ['modelo_nf_id', 'not in', self.pool.get('fiscal.modelo.nf').search(
                cr, user, [('fismnf_codigo', '=', '65')])]
            if not context.get('fisnfc_acesso_modelo_nf'):
                arg[1] = 'in'
                arg[2] += self.pool.get('fiscal.modelo.nf').search(
                    cr, user, [('fismnf_codigo', '>=', '90')])
            args += [arg]
        return super(fiscal_modelo_nf_series, self).name_search(
            cr, user, name, args, operator, context, limit)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
