# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2011 All Rights Reserved. Larissa Franca Borges
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
import requests
from requests.exceptions import ProxyError
import json
from openerp.osv.orm import except_orm
from openerp import tools
from logging import getLogger


_logger_atualizacao_ibpt = getLogger("[ATUALIZANDO TRIBUTOS IBPT]")


class fiscal_ncm(osv.Model):
    _name = 'fiscal.ncm'
    _inherit = 'fiscal.ncm'

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        for r in self.read(cr, uid, ids, ['codigo', 'descricao'], context=context):
            res.append((r['id'], '%s - %s' % (r['codigo'], r['descricao'])))
        return res

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        name = name.replace(".", "")
        sql = " SELECT id FROM fiscal_ncm WHERE codigo LIKE" + "('%"+name+"%')" \
            + " AND LENGTH(codigo) >= 8"
        cr.execute(sql)
        dados = cr.dictfetchall()
        cod_ncm = [i.get('id') for i in dados]
        if cod_ncm:
            args.append(('id', 'in', cod_ncm))
        if limit > 10:
            limit = None
        ids = []
        if name:
            if name.isdigit():
                ids = self.search(cr, uid, [('codigo', 'like', name)] + args, limit=limit, context=context)
            if not ids:
                ids = self.search(cr, uid, [('descricao', operator, name)] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids)

    @staticmethod
    def formata_ncm(codigo):
        """
        Metodo para formatacao do NCM. Formato 0000.00.00
        """
        codigo = codigo[:8].ljust(8, '0')

        return '%s.%s.%s' % (codigo[0:4], codigo[4:6], codigo[6:8])

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Le o numero do NCM e o devolve formatado
        """
        if not fields:
            fields = []
        context = context and context.copy() or {}
        if not isinstance(ids, list):
            ids = [ids]
        res = super(fiscal_ncm, self).read(cr, uid, ids, fields=fields, context=context, load=load)
        if 'codigo' in fields:
            for x in res:
                if x.get('codigo', False):
                    x['codigo'] = self.formata_ncm(x['codigo'])
        return res

    def ncm_tributo(self, cr, uid, ncm_id, context=None):
        context = context and context.copy() or {}
        obj_fiscal_param = self.pool.get('fiscal.parametro')
        parametro_id = obj_fiscal_param.search(cr, uid, [], context=context)
        tributos_ids = obj_fiscal_param.read(cr, uid, parametro_id, ['tributos_ids'], context=context)
        if tributos_ids:
            tributos_ids = tributos_ids[0]['tributos_ids']
            obj_ncm_tributacao = self.pool.get('fiscal.tributacao.encargos')
            ncm_tributos = self.pool.get('fiscal.tributacao.encargos').read(cr, uid, tributos_ids, context=context)
            for x in ncm_tributos:
                obj_ncm_tributacao.create(cr, uid, {'encargo_id': x['encargo_id'] and x['encargo_id'][0], #M2O
                                                    'cst_id': x['cst_id'] and x['cst_id'][0], #M2O
                                                    'tributacao_id': x['tributacao_id'] and x['tributacao_id'][0], #M2O
                                                    'mva': x['mva'],
                                                    'valor_pauta': x['valor_pauta'],
                                                    'modbcst': x['modbcst'],
                                                    'ncm_id': ncm_id,
                }, context)
        return True

    def button_tributos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.ncm_tributo(cr, uid, ids[0], context=context)
        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        install_mode = context.get('install_mode')
        ncm_id = self.search(cr, uid, [('codigo', '=', vals.get('codigo'))], context=context)
        if ncm_id:
            if install_mode:
                return ncm_id[0]
            else:
                raise osv.except_osv(
                    _('Atencao'),
                    _('Codigo do NCM: %s, ja cadastrado.') % (vals.get('codigo'))
                )
        codigo = vals['codigo']
        if '.' in codigo:
            codigo = codigo.replace('.', '')
        codigo = codigo.replace('\n', '')
        vals['codigo'] = codigo
        res = super(fiscal_ncm, self).create(cr, uid, vals, context=context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('codigo'):
            codigo = vals['codigo']
            if '.' in codigo:
                codigo = codigo.replace('.', '')
            codigo = codigo.replace('\n', '')
            vals['codigo'] = codigo
        res = super(fiscal_ncm, self).write(cr, uid, ids, vals, context=context)
        return res

    def atualiza_tributos_ibpt(self, cr, uid, context=None):
        """
        Este metodo tem o objetivo atualizar os valores dos tributos fisnc_aliq_ibpt_fed_nac,
        fisnc_aliq_ibpt_fed_imp, fisnc_aliq_ibpt_est de todos os NCMs cadastrados de acordo com a
        o webservice http://iws.ibpt.org.br/Help que e chamado pelo metodo atualiza_tributo_ibpt_ws
        :param cr: Cursor do openerp
        :param uid: Usuario solicitante da chamada
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        _logger_atualizacao_ibpt.info('INCIANDO ATUALIZACAO...')
        context = context and context.copy() or {}
        fields = ['fispar_ibpt_token', 'fispar_ibpt_cnpj', 'fispar_ibpt_uf']
        parametro = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'fiscal', fields,
                                                                     context=context)
        token = parametro.get('fispar_ibpt_token')
        cnpj = parametro.get('fispar_ibpt_cnpj')
        uf = parametro.get('fispar_ibpt_uf')
        if not all([token, cnpj, uf]):
            raise except_orm(_('Atencao'), _('As configuracoes para consulta no webservice do IBPT'
                                             'sao invalidas!'
                                             'Verifique se a aba IBPT do parametro do fiscal esta'
                                             'configurado corretamente. '))
        proxy = tools.config.get('http_proxy') or False
        proxies = {'http': 'http://{proxy}'.format(proxy=proxy)}
        for ncm_id in self.search(cr, uid, [], context=context):
            self.atualiza_tributo_ibpt_ws(cr, uid, ncm_id, token, cnpj, uf, proxies=proxies)
        return True

    def atualiza_tributo_ibpt_ws(self, cr, uid, ncm_id, token, cnpj, uf, context=None, **kwargs):
        """
        Este metodo tem o objetivo atualizar os valores dos tributos fisnc_aliq_ibpt_fed_nac,
        fisnc_aliq_ibpt_fed_imp, fisnc_aliq_ibpt_est de acordo com a busca realizada
        atraves do webservice DeOlhoNoImposto - Produtos com instrucoes disponibilizadas em
        http://iws.ibpt.org.br/Help
        :param cr: Cursor do openerp
        :param uid: Usuario solicitante da chamada
        :param ncm_id: Id do ncm a ser atualizado
        :param token: Token da empresa - Item adquirido no site De Olho no Imposto
                      (http://deolhonoimposto.ibpt.org.br)
        :param cnpj: CNPJ da empresa que esta fazendo a consulta
        :param uf: Estado a verificar
        :param context: Dicionario contendo valores contextuais
        :param kwargs: Dicionario contendo parametros adicionais como proxy por exemplo
        :type cr: Cursor
        :type uid: int
        :type ncm_id: int
        :type token: str
        :type cnpj: str
        :type uf: str
        :type context: dict
        :type kwargs: dict
        :return: Retorna um booleano informando se o registro foi atualizado ou nao
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        vals = {}
        context = context and context.copy() or {}
        ncm = self.browse(cr, uid, ncm_id, context=context)
        tributo_ibpt = self.get_tributo_ibpt(
            token=token, cnpj=cnpj, uf=uf, ncm_codigo=ncm.codigo, **kwargs)
        if tributo_ibpt:
            vals['fisnc_aliq_ibpt_fed_nac'] = tributo_ibpt.get('Nacional') or 0.0
            vals['fisnc_aliq_ibpt_fed_imp'] = tributo_ibpt.get('Importado') or 0.0
            vals['fisnc_aliq_ibpt_est'] = tributo_ibpt.get('Estadual') or 0.0
        return self.write(cr, uid, ncm_id, vals=vals, context=context)

    @staticmethod
    def get_tributo_ibpt(token, cnpj, ncm_codigo, uf, ex=0, codigo_interno='', descricao='',
                         unidade_medida='', valor=0.0, gtin='', **kwargs):
        """
        Este metodo tem o objetivo de buscar os tributos de acordo com o
        IBPT (INSTITUTO BRASILEIRO DE PLANEJAMENTO E TRIBUTACAO), atraves do webservice
        DeOlhoNoImposto - Produtos com instrucoes disponibilizadas em http://iws.ibpt.org.br/Help

        :param token: Token da empresa - Item adquirido no site De Olho no Imposto
                      (http://deolhonoimposto.ibpt.org.br)
        :param cnpj: CNPJ da empresa que esta fazendo a consulta
        :param ncm_codigo:  Codigo NCM
        :param uf: Estado a verificar
        :param ex: Codigo da excecao a regra aplicada na NCM. Caso nao haja, informe 0 (zero)
        :param codigo_interno: Codigo interno do produto no sistema
        :param descricao: Descricao do produto
        :param unidade_medida: Unidade de medida utilizado para a venda do produto
        :param valor: valor unitario do produto
        :param gtin: Codigo GTIN do produto
        :param kwargs: Dicionario contendo parametros adicionais como proxy por exemplo
        :type token: str
        :type cnpj: str
        :type ncm_codigo: str
        :type uf: str
        :type ex: int
        :type codigo_interno: str
        :type descricao: str
        :type unidade_medida: str
        :type valor: float
        :type gtin: str
        :type kwargs: dict
        :return: Retorna um dicionario contendo os valores encontrados no webservice ou vazio
                 se nao encontrou dados para o ncm
        :rtype: dict
        :raises: Pode retornar erro de configuracao de proxy ou token invalido ou expirado
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        try:
            res = {}
            proxies = kwargs.get('proxies')
            url = """http://iws.ibpt.org.br/api/Produtos?token={token}&cnpj={cnpj}&codigo={codigo}
            &uf={uf}&ex={ex}&codigoInterno={codigo_interno}&descricao={descricao}
            &unidadeMedida={unidade_medida}
            &valor={valor}&gtin={gtin}""".format(
                token=token, cnpj=cnpj, codigo=ncm_codigo, uf=uf, ex=ex,
                codigo_interno=codigo_interno, descricao=descricao, unidade_medida=unidade_medida,
                valor=valor, gtin=gtin)
            resposta = requests.get(url, proxies=proxies)
            if resposta.ok:
                res = json.loads(resposta.content)
                _logger_atualizacao_ibpt.info('ATUALIZANDO NCM: ' + ncm_codigo + ' OK!')
            elif resposta.status_code == 403:
                _logger_atualizacao_ibpt.warning(
                    'NCM NAO ATUALIZADO' + ncm_codigo + json.loads(resposta.content).get('Message'))
            return res
        except ProxyError:
            raise except_orm(_('Atencao'), _('Ocorreu um erro na configuracao do proxy!'))


