# -*- coding: utf-8 -*-

from openerp.osv import osv
from logging import getLogger
from openerp.tools.translate import _
from openerp.osv.fields import float_round
from datetime import datetime


class fiscal_nf_item_tributacao(osv.Model):
    _inherit = 'fiscal.nf.item.tributacao'

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.financeiro = None
            self.fiscal = None
            self.estoque = None
            self.adm = None
            self.rh = None
            self.__dict__.update(entries)

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or dict()
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in ['financeiro', 'fiscal', 'estoque', 'adm',
                                                         'rh']])
        return self.ParamToObject(**parametros)

    def _update_store(self, cr, obj, campo):
        """
        Este metodo computa os campos funcionais storiados novos ou que tiveram sua
        estrutura alterada. Caso nao queira que o novo campo funcioal seja atualizado
        ao levantar o modulo, e necessario adicionar o atributo funcional na
        lista _fields_no_store
        :param cr: a linha atual do cursor do banco de dados
        :param obj: Objeto da classe functions
        :param campo: Campo funcional a ser computado
        :type cr: object
        :type obj: object
        :type campo: str
        :author: Felipe Franca <felipe.franca[at]siginformatica.com.br>
        """
        if campo not in self._fields_no_store:
            super(fiscal_nf_item_tributacao, self)._update_store(cr, obj, campo)

    def fisnit_get_cfop_por_aliquota(self, cr, uid, ids, context=None):
        """
        O metodo apura os valores fiscais por CFOP e aliquota.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Lista de dicionarios. Cada dicionario possui os os dados do CFOP, aliquota e os
        valores referente a isento, base de calculo, encargo e substituicao.
        :rtype: list
        """
        context = context and context.copy() or {}
        res = list()
        if not ids:
            return res
        if len(ids) == 1:
            item_tributacao_ids = str(tuple(ids)).replace(',', '')
        else:
            item_tributacao_ids = str(tuple(ids))
        sql = """
            SELECT fe.sigla                                         AS encargo
                 , fc.codigo                                        AS cfop
                 , fnit.aliquota                                    AS aliquota
                 , SUM(COALESCE(fnit.fisnit_vr_isento, 0))          AS vr_isento
                 , SUM(COALESCE(fnit.vr_base_calculo, 0))           AS vr_base_calculo
                 , SUM(COALESCE(fnit.vr_encargo, 0))                AS vr_encargo
                 , SUM(COALESCE(fnit.vr_icms_substituicao, 0))      AS vr_icms_substituicao
                 , SUM(COALESCE(fnit.fisnit_vr_icms_desonerado, 0)) AS vr_icms_desonerado
            FROM  fiscal_nf_item_tributacao                         AS fnit
            INNER JOIN fiscal_cfop                                  AS fc
               ON fc.id = fnit.fisnit_cfop_id
            INNER JOIN fiscal_encargo                               AS fe 
               ON fe.id = fnit.encargo_id             
            WHERE fnit.id IN %s
            GROUP BY fe.sigla, fc.codigo, fnit.aliquota
        """ % item_tributacao_ids
        cr.execute(sql)
        res = cr.dictfetchall()
        return res

    def calcula_tributacao_item(self, cr, user, item_id, tributacao_id, base_calculo, context=None,
                                natureza_id=False, movimento_id=False):
        """
        Realiza o calculo das tributacoes baseado no valor total do item (produto).

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao do perfil
        :param item_id: o ID do item (produto/servico)
        :param tributacao_id: o ID da tributacao
        :param base_calculo: o valor da base utilizado no calculo das tributacoes
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza para calculo do valor da desoneracao de ICMS se exigir
        :param movimento_id: o ID do item de movimento (estoque_movimento_itens). Para recuperar os
        valores rateados que devem somar na base de calculo
        :type cr: object
        :type user: int
        :type item_id: long
        :type tributacao_id: long
        :type base_calculo: float
        :type context: dict
        :type natureza_id: long
        :type movimento_id: long
        :return: dicionario com os valores das tributacoes.
        :rtype: dict
        """
        context = context and context.copy() or {}
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]
        tributacao_dados = tributacao_id and self.pool.get('fiscal.tributacao').read(
            cr, user, tributacao_id, ['base', 'aliquota', 'cst_id'], context,
            '_classic_write') or {}
        aliquota = tributacao_dados.get('aliquota') or 0
        base = tributacao_dados.get('base') or 0

        # Soma os valores rateados ao valor da base de calculo
        valores_rateado = movimento_id and self.pool.get('fiscal.nf.capa').resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [movimento_id],
            ['vr_desconto_rateado', 'vr_despesas_rateado', 'vr_frete_rateado', 'vr_seguro_rateado'],
            context)
        if valores_rateado:
            base_calculo += sum(map(lambda m: (m[0] in [
                'vr_desconto_rateado', 'vr_despesas_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado'] and m[1]) or 0, valores_rateado[0].items())) or 0

        vr_base = base * base_calculo / 100.
        vr_encargo = vr_base * aliquota / 100.
        vr_base_reduzida = base_calculo - vr_base
        if vr_base_reduzida < 0:
            vr_base_reduzida = 0
        cst_id = tributacao_dados.get('cst_id')
        cst_dados = cst_id and self.pool.get('fiscal.cst').read(
            cr, user, cst_id, ['tipo', 'encargo_id', 'codigo'], context, '_classic_write') or {}
        values = {'vr_base_calculo': vr_base, 'aliquota': aliquota, 'vr_encargo': vr_encargo,
                  'vr_reducao_base': vr_base_reduzida}

        # Valores para ICMS
        encargo_id = cst_dados.get('encargo_id')
        if encargo_id == icms_id:
            tipo_substituicao = True if cst_dados.get('tipo') in ('U', 'TU') else False
            if tipo_substituicao:
                tributacao_encargo_ids = self.pool.get('fiscal.tributacao.encargos').search(
                    cr, user, [('produto_id', '=', item_id), ('encargo_id', '=', encargo_id),
                               ('cst_id', '=', cst_id)], context=context)
                tributacao_encargo_id = tributacao_encargo_ids and tributacao_encargo_ids[0]
                tributacao_encargo_dados = \
                    tributacao_encargo_id and self.pool.get('fiscal.tributacao.encargos').read(
                        cr, user, tributacao_encargo_id,
                        ['modbcst', 'fisten_aliquota_venda', 'mva'], context,
                        '_classic_write') or {}
                mva = tributacao_encargo_dados.get('mva') or 0
                aliquota_venda = tributacao_encargo_dados.get('fisten_aliquota_venda') or 0
                modbcst = tributacao_encargo_dados.get('modbcst') or ''
                # calcula apenas para '4 = MVA' e notas de saida
                if modbcst not in ['4', ''] or context.get('entrada_terceiros'):
                    return values

                icms_substituicao = self.fisnit_calcular_vr_icms_st_item(
                    base_calculo, aliquota, aliquota_venda, mva)
                vr_icms_st = icms_substituicao.get('vr_icms_substituicao') or 0
                vr_base_icms_st = icms_substituicao.get('vr_base_icms_substituicao') or 0
                values.update(tipo_substituicao=tipo_substituicao, percentual_mva=mva,
                              vr_icms_substituicao=vr_icms_st, aliquota_venda=aliquota_venda,
                              vr_base_icms_substituicao=vr_base_icms_st, modbcst=modbcst)
            else:
                if not context.get('entrada_terceiros'):
                    values.update(tipo_substituicao=tipo_substituicao, percentual_mva=0, modbcst='',
                                  vr_icms_substituicao=0, aliquota_venda=0,
                                  vr_base_icms_substituicao=0)

            # Desoneracao de ICMS. Se houver, ajusta-se o CST e a tributacao, e o valor sera o
            # mesmo valor do encargo
            natureza_dados = natureza_id and self.pool.get('fiscal.cfop.natureza').read(
                    cr, user, natureza_id, ['fisnat_icms_desonerado'], context) or {}
            if natureza_dados.get('fisnat_icms_desonerado'):  # Troca do CST e tributacao
                cst40_ids = self.pool.get('fiscal.cst').search(
                    cr, user, [('codigo', '=', '40')], context=context)
                cst40_id = cst40_ids and cst40_ids[0]
                trib40_ids = self.pool.get('fiscal.tributacao').search(
                    cr, user, [('encargo_id', '=', icms_id), ('cst_id', '=', cst40_id)],
                    context=context)
                trib40_id = trib40_ids.__len__() == 1 and trib40_ids[0] or False
                values.update(fisnit_vr_icms_desonerado=vr_encargo, cst_id=cst40_id,
                              tributacao_id=trib40_id)
            # elif cst_dados.get('codigo') in ['20', '30', '40', '70', '90']:
            #     Verifica desoneracao para CST 20, 30, 40, 70 e 90
            #     values['fisnit_vr_icms_desonerado'] = values.get('vr_base_calculo') * \
            #                                           values.get('aliquota') - \
            #                                           values.get('vr_encargo')
            else:
                values['fisnit_vr_icms_desonerado'] = 0

        return values

    def onchange_tributacao(self, cr, uid, ids, item_id, tributacao_id, base_calculo, context=None,
                            natureza_id=False):
        context = context and context.copy() or {}
        res = self.calcula_tributacao_item(
            cr, uid, item_id, tributacao_id, base_calculo, context, natureza_id)
        if 'cst_id' in res and context.get('fisnit_nf_propria'):
            del res['cst_id']
        return {'value': res}

    def fisnit_onchange_tributacao_servico(self, cr, user, ids, item_id, tributacao_id,
                                           base_calculo, context=None, natureza_id=False):
        """
        Calcula os valores dos encargos para o item em funcao da tributacao.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param item_id: o ID do item (produto/servico)
        :param tributacao_id: o ID da tributacao
        :param base_calculo: o valor da base de calculo do item
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacao_id: long
        :type base_calculo: float
        :type context: dict
        :type natureza_id: long
        :return: os valores atribuidos ao item em funcao da tributacao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        movimento_id = ids and ids[0]
        result = self.calcula_tributacao_item(
            cr, user, item_id, tributacao_id, base_calculo, context, natureza_id, movimento_id)
        if 'cst_id' in result:
            del result['cst_id']

        return {'value': result}

    def fisnit_onchange_tributacao_propria(self, cr, user, ids, tributacao_id, context=None,
                                           vr_item=0, vr_despesa=0, vr_frete=0, vr_seguro=0,
                                           vr_desconto=0, incide_despesa=True, natureza_id=None,
                                           aliquota_venda=0, mva=0):
        """
        Recupera o valor da aliquota em funcao tributacao.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param tributacao_id: o ID da tributacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tributacao_id: long
        :type context: dict
        :return: o valor dos encargos do item.
        :rtype: dict
        """
        context = context and context.copy() or {}
        vr_base_calculo = vr_item + vr_despesa + vr_frete + vr_seguro - vr_desconto
        values = self.fisnit_get_tributacao_valor(
            cr, user, tributacao_id, vr_base_calculo, context, vr_despesa, incide_despesa,
            natureza_id)
        brw_tributacao = tributacao_id and self.pool.get('fiscal.tributacao').browse(
            cr, user, tributacao_id, context)
        if tributacao_id:
            values.update(self.fisnit_calcular_vr_icms_st_item(
                vr_base_calculo, values.get('aliquota'), aliquota_venda, mva))
        if brw_tributacao and brw_tributacao.cst_id:
            if brw_tributacao.cst_id.encargo_id:
                substituicao = brw_tributacao.cst_id.tipo in ('U', 'TU')
                icms = brw_tributacao.encargo_id.sigla.upper() == 'ICMS'
                cst60 = brw_tributacao.cst_id.codigo == '60'

                # Se nao ICMS/ST ou ICMS/ST com CST 60
                if icms and cst60 or not substituicao:
                    values.update({'vr_icms_substituicao': 0, 'vr_base_icms_substituicao': 0,
                                   'modbcst': False, 'percentual_mva': 0, 'aliquota_venda': 0})
            values['fisnit_percentual_base_zero'] = not brw_tributacao.base
        else:
            values.update({'vr_icms_substituicao': 0, 'vr_base_icms_substituicao': 0})
            values['fisnit_percentual_base_zero'] = False

        return {'value': values}

    def fisnit_onchange_frete_tributacao(self, cr, user, ids, tributacao, context=None):
        """
        Ajusta o CST do encargo em funcao da tributacao.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param tributacao: o ID da tributacao
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type tributacao: long
        :type context: dict
        :return: o valor ajustado do encargo
        :rtype: float
        """
        context = context and context.copy() or {}
        dados_cst = tributacao and self.pool.get('fiscal.tributacao').read(
            cr, user, tributacao, ['cst_id'], context, '_classic_write')

        return {'value': {'cst_id': dados_cst and dados_cst.get('cst_id')}}

    def fisnit_onchange_base_calculo_terceiro(self, cr, user, ids, base, aliquota, total,
                                              context=None):
        """
        Ajusta o valor do encargo em funcao do valor da base de calculo.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param aliquota: o valor da aliquota
        :param base: o valor da base de calculo do item (vr_total_item_rateado)
        :param total: o valor total do item
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type aliquota: float
        :type base: float
        :type total: float
        :type context: dict
        :return: o valor ajustado do encargo
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'vr_encargo': aliquota * base / 100.,
                          'vr_reducao_base': total > base and total - base}}

    def fisnit_onchange_aliquota_terceiro(self, cr, user, ids, base, aliquota, context=None):
        """
        Ajusta o valor do encargo em funcao do valor da aliquota.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param aliquota: o valor da aliquota
        :param base: o valor da base de calculo do item (vr_total_item_rateado)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type aliquota: float
        :type base: float
        :type context: dict
        :return: o valor ajustado do encargo
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'vr_encargo': aliquota * base / 100.}}

    def fisnit_onchange_tributacao_terceiro(self, cr, user, ids, item_id, tributacao_id,
                                            base_calculo, context=None, natureza_id=False):
        """
        Calcula a tributacao do item. Nao informa a tributacao se o CST
        nao foi informado.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param item_id: o ID do item (produto/servico)
        :param tributacao_id: o ID da tributacao
        :param base_calculo: o valor da base de calculo do item
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacao_id: long
        :type base_calculo: float
        :type context: dict
        :type natureza_id: long
        :return: valores ajustados da tributacao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        movimento_id = ids and ids[0]
        values = self.calcula_tributacao_item(
            cr, user, item_id, tributacao_id, base_calculo, context, natureza_id, movimento_id)

        return {'value': values}

    def onchange_cst(self, cr, uid, ids, cst_id,encargo_id, context=None):
        context = context and context.copy() or {}
        tributacao_id = False
        if cst_id and encargo_id:
            obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
            tributacao_ids = obj_fiscal_tributacao.search(
                cr, uid,[('encargo_id', '=', encargo_id), ('cst_id', '=', cst_id)], context=context)
            if tributacao_ids and len(tributacao_ids) == 1:
                tributacao_id = tributacao_ids[0]
        return {'value': {'cst_id': cst_id, 'tributacao_id':tributacao_id}}

    def fisnit_onchange_cst_propria(self, cr, user, ids, encargo_id, cst_id, context=None):
        """
        Atribui o valor da tributacao baseado no encargo e no CST.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param encargo_id: o ID do encargo
        :param cst_id: o ID do CST
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type encargo_id: long
        :type cst_id: long
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de tributacao ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}

        # Se houver apenas uma tributacao, esta sera informada
        values = {'tributacao_id': False}
        tributacao_ids = self.pool.get('fiscal.tributacao').search(
            cr, user, [('encargo_id', '=', encargo_id), ('cst_id', '=', cst_id)], context=context)
        if tributacao_ids and len(tributacao_ids) == 1:
            values['tributacao_id'] = tributacao_ids[0]

        # ICMS/ST - Identifica CST com substituicao tributaria
        tipo_cst = cst_id and self.pool.get('fiscal.cst').read(cr, user, cst_id, ['tipo'],
                                                               context).get('tipo') or ''
        values['tipo_substituicao'] =  tipo_cst in ('U', 'TU') or False
        values['fisnit_tipo_cst'] =  tipo_cst
        return {'value': values}

    def fisnit_onchange_cst(self, cr, user, ids, cst, encargo, context=None, item=False, total=0):
        """
        Atribui o valor da tributacao baseado no encargo e no CST. Nao informa a tributacao se o CST
        nao foi informado.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param cst: o ID do CST
        :param encargo: o ID do encargo
        :param context: um dicionario padrao para valores contextuais,
        :param item: o ID do item de movimentacao
        :param total: o valor total do item
        :type cr: object
        :type user: int
        :type ids: list
        :type cst: long
        :type encargo: long
        :type context: dict
        :type item: long
        :type total: float
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        tributacao_ids = self.pool.get('fiscal.tributacao').search(
            cr, user, [('encargo_id', '=', encargo), ('cst_id', '=', cst)], context=context)

        # Se houver apenas uma tributacao, esta sera informada
        if tributacao_ids and len(tributacao_ids) == 1:
            aliquota = self.pool.get('fiscal.tributacao').read(
                cr, user, tributacao_ids[0], ['aliquota'], context).get('aliquota') or 0
            tributacao_encargo_ids = self.pool.get('fiscal.tributacao.encargos').search(
                cr, user, [('produto_id', '=', item), ('encargo_id', '=', encargo),
                           ('cst_id', '=', cst)], context=context)
            tributacao_encargo_dados = self.pool.get('fiscal.tributacao.encargos').read(
                cr, user, tributacao_encargo_ids and tributacao_encargo_ids[0],
                ['modbcst', 'fisten_aliquota_venda', 'mva'], context, '_classic_write')
            percentual_mva = tributacao_encargo_dados and tributacao_encargo_dados.get('mva') or 0
            aliquota_venda = tributacao_encargo_dados and \
                             tributacao_encargo_dados.get('fisten_aliquota_venda') or 0
            modbcst = tributacao_encargo_dados and tributacao_encargo_dados.get('modbcst') or str()
            value = dict(tributacao_id=tributacao_ids[0], modbcst=modbcst,
                         percentual_mva=percentual_mva, aliquota_venda=aliquota_venda)
            if self.pool.get('fiscal.cst').search(
                    cr, user, [('id', '=', cst), ('tipo', 'in', ('U', 'TU'))], context=context):
                value.update(self.fisnit_calcular_vr_icms_st_item(
                    total, aliquota, aliquota_venda, percentual_mva))
            else:
                value.update(vr_icms_substituicao=0, vr_base_icms_substituicao=0, modbcst=str(),
                             percentual_mva=0, aliquota_venda=0)

        else:
            value = dict(vr_icms_substituicao=0, vr_base_icms_substituicao=0, modbcst=str(),
                         percentual_mva=0, aliquota_venda=0, tributacao_id=False)

        return dict(value=value)

    @staticmethod
    def fisnit_calcular_vr_icms_st_item(vr_base_calculo, aliquota_compra, aliquota_venda,
                                        percentual_mva):
        """
        Calcula os valores do ICMS ST do item para modalidade margem valor agregado.

        :param vr_base_calculo: valor do item para base de calculo
        :param aliquota_compra: percentual da aliquota de compra
        :param aliquota_venda: percentual da aliquota de venda
        :param percentual_mva: percentual da margem de valor agregado
        :type vr_base_calculo: float
        :type aliquota_compra: float
        :type aliquota_venda: float
        :return: retorna o total do ICMS ST para o item
        :rtype: dict
        """
        vr_base_icms_substituicao = vr_base_calculo + vr_base_calculo * percentual_mva / 100.
        vr_encargo = vr_base_calculo * aliquota_compra / 100.
        vr_icms_substituicao = \
            aliquota_venda and vr_base_icms_substituicao * aliquota_venda / 100. - vr_encargo
        return {'vr_icms_substituicao': vr_icms_substituicao or 0,
                'vr_base_icms_substituicao': vr_base_icms_substituicao}

    def onchange_modcst(self, cr, uid, ids, modcst, context=None):

        """
        Atribui valor zero para os campos quando alterada a modalidade base calculo do ICMS/ST
        """
        context = context and context.copy() or {}
        value = {'aliquota_venda': 0, 'percentual_mva': 0, 'vr_icms_substituicao':0}
        return {'value': value}

    def fisnit_onchange_encargo_propria(self, cr, user, ids, encargo_id, context=None):
        """
        Atribui o valor do CST em funcao do encargo.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param encargo_id: o ID do encargo
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type encargo_id: long
        :type context: dict
        :return: o ID da tributacao se houver apenas uma relacionada ao encargo
        :rtype: float
        """
        context = context and context.copy() or {}
        cst_ids = self.pool.get('fiscal.cst').search(
            cr, user, [('encargo_id', '=', encargo_id)], context=context)
        cst_id = cst_ids[0] if len(cst_ids) == 1 else False

        return {'value': {'cst_id': cst_id}}

    def onchange_encargo(self, cr, user, ids, encargo, context=None):
        """
        Atribui o valor do CST em funcao do encargo.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param encargo: o ID do encargo
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type encargo: long
        :type context: dict
        :return: o ID da tributacao se houver apenas uma relacionada ao encargo
        :rtype: float
        """
        context = context and context.copy() or {}
        cst_ids = self.pool.get('fiscal.cst').search(
            cr, user, [('encargo_id', '=', encargo)], context=context)
        cst_id = cst_ids[0] if len(cst_ids) == 1 else False

        return dict(value=dict(cst_id=cst_id))

    def fisnit_onchange_encargo_manutencao(self, cr, user, ids, encargo, context=None):
        """
        Atribui o valor do CST em funcao do encargo.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param encargo: o ID do encargo
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type encargo: long
        :type context: dict
        :return: o ID da tributacao se houver apenas uma relacionada ao encargo
        :rtype: dict
        """
        context = context and context.copy() or {}
        cst_ids = self.pool.get('fiscal.cst').search(
            cr, user, [('encargo_id', '=', encargo)], context=context)
        cst_id = cst_ids[0] if len(cst_ids) == 1 else False

        return {'value': {'cst_id': cst_id}}

    def fisnit_onchange_cst_manutencao(self, cr, user, ids, encargo, cst, context=None):
        """
        Atribui o valor da tributacao baseado no encargo e no CST. 

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param encargo: o ID do encargo
        :param cst: o ID do CST
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type encargo: long
        :type cst: long
        :type context: dict
        :return: 
        :rtype: dict
        """
        context = context and context.copy() or {}
        tributacao_ids = self.pool.get('fiscal.tributacao').search(
            cr, user, [('encargo_id', '=', encargo), ('cst_id', '=', cst)], context=context)
        tributacao_id = tributacao_ids[0] if len(tributacao_ids) == 1 else False

        return {'value': {'tributacao_id': tributacao_id}}

    def fisnit_onchange_tributacao_manutencao(self, cr, user, ids, item_id, tributacao_id,
                                              base_calculo, context=None, natureza_id=False):
        """
        Calcula os valores dos encargos para o item em funcao da tributacao.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param item_id: o ID do item de movimentacao (produto/servico)
        :param tributacao_id: o ID da tributacao
        :param base_calculo: o valor da base de calculo do item
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacao_id: long
        :type base_calculo: float
        :type context: dict
        :type natureza_id: long
        :return: os valores atribuidos ao item em funcao da tributacao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        values = self.calcula_tributacao_item(
            cr, user, item_id, tributacao_id, base_calculo, context, natureza_id)

        return {'value': values}

    def fisnit_onchange_encargo_terceiro(self, cr, user, ids, encargo, context):
        """
        Na troca do encargo, elimina o CST.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param encargo: o ID do encargo
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type encargo: long
        :type context: dict
        :return: CST (cst_id) sem valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        cst_ids = self.pool.get('fiscal.cst').search(
            cr, user, [('encargo_id', '=', encargo)], context=context)
        cst_id = cst_ids[0] if len(cst_ids) == 1 else False

        return {'value': {'cst_id': cst_id}}

    def fisnit_onchange_mva(self, cr, user, ids, cst, aliquota, base, mva, aliquota_venda,
                            context=None):
        """
        Realiza o calculo dos valores de ICMS/ST.
        
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param cst: 
        :param aliquota: 
        :param base: 
        :param mva: 
        :param aliquota_venda: 
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type cst: long
        :type aliquota: float 
        :type base: float
        :type mva: float
        :type aliquota_venda: 
        :type context: dict
        :return: os valores relativos ao ICMS/ST em funcao do valor percentual do MVA
        :rtype: dict
        """
        context = context and context.copy() or {}

        # Realiza o calculo somente para ICMS/ST
        if cst and self.pool.get('fiscal.cst').read(
                cr, user, cst, ['tipo'], context).get('tipo') in ('U', 'TU'):
            value = self.fisnit_calcular_vr_icms_st_item(base, aliquota, aliquota_venda, mva)
        else:
            value = {'modbcst': '', 'vr_icms_substituicao': 0, 'percentual_mva': 0,
                     'aliquota_venda': 0}

        return {'value': value}

    def fisnit_onchange_aliquota_venda(self, cr, user, ids, cst, aliquota, base, mva,
                                       aliquota_venda, context=None):
        """
        Realiza o calculo dos valores de ICMS/ST em funcao do valor da aliquota de venda.
        
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param cst: o ID do CST
        :param aliquota: o percentual da aliquota da tributacao
        :param base: o valor da base de calculo
        :param mva: o valor percentual do MVA
        :param aliquota_venda: o valor percentual da aliquota de venda (ICMS/ST)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type cst: long
        :type aliquota: float 
        :type base: float
        :type mva: float
        :type aliquota_venda: 
        :type context: dict
        :return: os valores relativos ao ICMS/ST em funcao do valor da aliquota de venda.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if not cst:
            return {'value': {'modbcst': '', 'vr_icms_substituicao': 0, 'percentual_mva': 0,
                              'aliquota_venda': 0}}
        
        # Realiza o calculo somente para ICMS/ST
        if self.pool.get('fiscal.cst').read(
                cr, user, cst, ['tipo'], context).get('tipo') in ('U', 'TU'):
            value = self.fisnit_calcular_vr_icms_st_item(base, aliquota, aliquota_venda, mva)
        else:
            value = {'modbcst': '', 'vr_icms_substituicao': 0, 'percentual_mva': 0,
                     'aliquota_venda': 0}

        return {'value': value}

    def onchange_valida(self, cr, uid, ids, percentual_mva, aliquota_venda, modbcst,
                        vr_base_calculo, aliquota, context=None):
        context = context and context.copy() or {}
        value = self.fisnit_calcular_vr_icms_st_item(
            vr_base_calculo, aliquota, aliquota_venda, percentual_mva)
        value.update({'aliquota': aliquota, 'vr_base_calculo': vr_base_calculo})

        return {'value': value}

    def create(self, cr, uid, vals, context=None):
        # testo se é inteiro ou longo, pelo fato a nf ja estar gravada ou nao, se já está gravada,
        # vem int ou longo, se não, vem string no formato 'one2many_v_d_999'
        # e ai neste caso não tem o que browsear(browse_estoque_movimento_itens)
        context = context and context.copy() or {}
        if (vals.get('movimento_itens_id') and not vals.get('fisnit_capa_id')
            and isinstance(vals.get('movimento_itens_id'), (int, long))):
            arg_itens = [('encargo_id', '=', vals['encargo_id']),
                         ('movimento_itens_id', '=', vals['movimento_itens_id'])]
            if context.get('tributos_excluidos'):
                arg_itens.append(('id', 'not in', context['tributos_excluidos']))
            nf_itens_trib_ids = self.search(cr, uid, arg_itens, context=context)
            if not context.get('manutencao_cupom'):
                if nf_itens_trib_ids:
                    return False
            if not vals.get('fisnit_capa_id'):
                vals['fisnit_capa_id'] = self.pool.get('estoque.movimento.itens').read(
                    cr, uid, vals.get('movimento_itens_id'), ['nf_id'], context,
                    '_classic_write').get('nf_id') or False
            res = super(fiscal_nf_item_tributacao, self).create(cr, uid, vals, context)
            if not nf_itens_trib_ids or context.get('manutencao_cupom'):
                return res
        elif vals.get('fisnit_capa_id') and not vals.get('movimento_itens_id'):
            nf_frete_ids = self.search(
                cr, uid, [('encargo_id', '=', vals['encargo_id']),
                          ('fisnit_capa_id', '=', vals['fisnit_capa_id'])], context=context)
            if not nf_frete_ids:
                return super(fiscal_nf_item_tributacao, self).create(cr, uid, vals, context)
        else:
            return super(fiscal_nf_item_tributacao, self).create(cr, uid, vals, context)

    def ecf_fisnit_calcula_metodos_funcionais(self, cr, uid, context=None):
        '''
        Metodo implementado por heranca no modulo ecf
        '''
        return []

    def fisnit_calcula_metodos_funcionais(self, cr, uid, context=None):
        '''
        Apenas recalcula os metodos funcionais da classe, eh chamado pelo cron 'cronjob_calcular_fiscal_nf_item_trib',
        devido ao proble que acontecia quando dava um write na classe e o proprio open calcula, depois de um certo
        tempo o open parava o servico, por este problema e por performace foi a funcao foi feita em sql
        '''
        context = context and context.copy() or {}
        getLogger(self._name).info(
            _('Calculando metodos funcionais da classe "fiscal_nf_item_tributacao"...'))
        # notas com itens
        sql = '''SELECT fnit.id,
                        fnit.vr_icms_substituicao,
                        fnit.vr_encargo,
                        fnit.vr_base_calculo        AS base_calculo,
                        fnit.vr_reducao_base        AS base_reducao,
                        cst.tipo                    AS cst_tipo,
                        emi.cfop_id,
                        emi.vr_total_item_rateado   AS valor_contabil,
                        fnc.tipo                    AS nf_tipo
                 FROM fiscal_nf_item_tributacao     AS fnit
                 INNER JOIN fiscal_cst              AS cst  
                         ON cst.id = fnit.cst_id
                 INNER JOIN estoque_movimento_itens AS emi  
                         ON emi.id = fnit.movimento_itens_id
                 INNER JOIN fiscal_nf_capa          AS fnc  
                         ON fnc.id = emi.nf_id
                 WHERE movimento_itens_id IS NOT NULL
                 ORDER BY id
              '''
        cr.execute(sql)
        dados_trib = cr.dictfetchall()
        # notas sem itens
        sql = '''SELECT fnit.id,
                        fnit.vr_icms_substituicao,
                        fnit.vr_encargo,
                        fnit.vr_base_calculo     AS base_calculo,
                        fnit.vr_reducao_base     AS base_reducao,
                        cst.tipo                 AS cst_tipo,
                        fcn.cfop_id,
                        fnc.vr_total_nf          AS valor_contabil,
                        fnc.tipo                 AS nf_tipo,
                        fe.sigla                 AS sigla_encargo
                 FROM fiscal_nf_item_tributacao  AS fnit
                 INNER JOIN fiscal_cst           AS cst  
                         ON cst.id = fnit.cst_id
                 INNER JOIN fiscal_nf_capa       AS fnc  
                         ON fnc.id = fnit.fisnit_capa_id
                 INNER JOIN fiscal_cfop_natureza AS fcn  
                         ON fcn.id = fnc.natureza_id
                 INNER JOIN fiscal_encargo       AS fe   
                         ON fe.id = fnit.encargo_id
                 WHERE movimento_itens_id IS NULL
                 ORDER BY id
              '''

        cr.execute(sql)
        dados_trib.extend(cr.dictfetchall())
        # cupons
        dados_trib.extend(self.ecf_fisnit_calcula_metodos_funcionais(cr, uid, context=context))
        for trib in dados_trib:
            total = (trib.get('base_calculo') or 0) + (trib.get('base_reducao') or 0)
            tipo_cst = trib.get('cst_tipo') or ''
            tributado = outros = substituicao = isento = diferido = servico = 0
            valor_contabil = trib.get('valor_contabil') or 0
            cfop_id = trib.get('cfop_id') or False
            nf_tipo = trib.get('nf_tipo') or ''
            if tipo_cst == 'T' or tipo_cst == 'TU': # Tributado ou Tributado com substituicao
                tributado = trib.get('base_calculo') or 0
                outros = trib.get('base_reducao') or 0
            elif tipo_cst == 'U': # Substituicao
                substituicao = trib.get('vr_icms_substituicao') or 0
                outros = total
            elif tipo_cst == 'I': # Isento
                isento = total
            elif tipo_cst == 'O': # Outros
                outros = total
            elif tipo_cst == 'D': # Diferido
                diferido = total
            elif tipo_cst == 'S': # Servico
                servico = total
            if trib.get('sigla_encargo') == 'IPI':
                outros += trib.get('vr_encargo') or 0
            sql = '''UPDATE fiscal_nf_item_tributacao
                     SET  fisnit_tipo_movimento_cst = '{nf_tipo}',
                          fisnit_tipo_cst = '{tipo_cst}',
                          fisnit_vr_tributado = {tributado},
                          fisnit_vr_isento = {isento},
                          fisnit_vr_outros = {outros},
                          fisnit_vr_diferido = {diferido},
                          fisnit_vr_substituicao = {substituicao},
                          fisnit_vr_servico = {servico},
                          fisnit_vr_contabil = {valor_contabil},
                          fisnit_cfop_id = {cfop_id}
                     WHERE id = {id}
                  '''.format(id=trib.get('id'), outros=outros, isento=isento, cfop_id=cfop_id,
                             servico=servico, nf_tipo=nf_tipo, diferido=diferido, tipo_cst=tipo_cst,
                             tributado=tributado, substituicao=substituicao,
                             valor_contabil=valor_contabil)
            cr.execute(sql)

        getLogger(self._name).info(
            _("Fim do calculo dos metodos funcionais da classe 'fiscal_nf_item_tributacao'." ))

        return True

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}

        # Contexto usado para apenas recalcular metodos funcinais da tributacao pelo cupom
        if context.get('fisnit_super_write'):
            return super(fiscal_nf_item_tributacao, self).write(cr, uid, ids, vals, context=context)

        if isinstance(ids, (list, tuple)):
            id = ids[0]
        else:
            id = ids
        obj_nf_capa = self.pool.get('fiscal.nf.capa')
        campos = ['movimento_itens_id', 'encargo_id']
        dados_trib = self.read(cr, uid, id, campos, context=context, load='_classic_write')
        if not dados_trib.get('movimento_itens_id'):
            return super(fiscal_nf_item_tributacao, self).write(cr, uid, ids, vals, context=context)
        context_aux = context.copy()
        context_aux['super_read_estoque_movimento_itens'] = True
        campos = ['nf_id', 'produto_id', 'vr_total_item_rateado']
        dados_item = self.pool.get('estoque.movimento.itens').read(
            cr, uid, dados_trib.get('movimento_itens_id'), campos, context, '_classic_write')
        nf_id = dados_item.get('nf_id') or []
        brw_nf = obj_nf_capa.browse(cr, uid, nf_id, context)
        produto_id = dados_item.get('produto_id')
        if 'encargo_id' in vals:
            if produto_id:
                encargo_id = vals.get('encargo_id') or dados_trib.get('encargo_id') or False
                if encargo_id:
                    if isinstance(encargo_id, (tuple, list)):
                        encargo_id = encargo_id[0]
                    sigla_encargo = self.pool.get('fiscal.encargo').read(
                        cr, uid, encargo_id, ['sigla'], context).get('sigla')
                    dados_produto = self.pool.get('estoque.produto').read(
                        cr, uid, produto_id,['estpro_pis_nat_rec', 'estpro_cofins_nat_rec'],
                        context)
                    if sigla_encargo.upper() == 'PIS' and not vals.get('fisnit_nat_rec_pis_cofins'):
                        receita_pis = dados_produto.get('estpro_pis_nat_rec')
                        vals['fisnit_nat_rec_pis_cofins'] = receita_pis
                    elif sigla_encargo.upper() == 'COFINS' and not vals.get('fisnit_nat_rec_pis_cofins'):
                        receita_cofins = dados_produto.get('estpro_cofins_nat_rec')
                        vals['fisnit_nat_rec_pis_cofins'] = receita_cofins
            if brw_nf.natureza_id:
                encargo_ids = obj_nf_capa.verificar_encargos_natureza(
                    cr, uid, brw_nf.natureza_id.id, context)
                if brw_nf.nf_propria and brw_nf.natureza_id.cfop_id.movimento_tipo != 'D':
                    if not context.get('importacao_nfe'):
                        encargo_dados = self.pool.get('fiscal.encargo').read(
                            cr, uid, vals.get('encargo_id'), ['sigla'], context)
                        sigla = encargo_dados and encargo_dados.get('sigla') or '????'
                        if vals.get('encargo_id') not in encargo_ids:
                            raise osv.except_osv(_('Atencao!'),
                                                 _('Nao e permitido incluir o encargo %s para a '
                                                   'natureza de operacao informada.') % sigla)
        # Preenche ID na NF referente
        if nf_id:
            vals['fisnit_capa_id'] = nf_id
            if not all(['fisnit_vr_difal_uf_origem' in vals,
                        'fisnit_vr_difal_uf_destino' in vals]):
                # Calcula icms difal
                field_list = ['estoque_movimento_itens_ids', 'destinatario_endereco_id', 'data_emissao', 'company_id']
                dados_nf = obj_nf_capa.read(cr, uid, nf_id, field_list, context, '_classic_write')
                self.fisnit_gerar_difal(cr, uid, dados_nf, context)

        return super(fiscal_nf_item_tributacao, self).write(cr, uid, ids, vals, context=context)

    def fisnit_valida_cst_pis_cofins(self, cr, uid, dados_item, dict_trib, dict_encargos, dict_cst,
                                     context=None):
        context = context and context.copy() or {}
        dict_cst_pis_cofins = {}
        for trib_id in dados_item.get('tributacoes_ids'):
            trib = dict_trib.get(trib_id)
            encargo = dict_encargos.get(trib.get('encargo_id'))
            cst = dict_cst.get(trib.get('cst_id'))
            if encargo.get('sigla') in ('PIS', 'COFINS'):
                dict_cst_pis_cofins[encargo.get('sigla')] = cst.get('codigo')
        if dict_cst_pis_cofins.get('PIS') != dict_cst_pis_cofins.get('COFINS'):
            return False

        return True

    def fisnit_onchange_aliquota_propria(self, cr, user, ids, vr_item, vr_despesa, incide_despesa,
                                         vr_frete, vr_seguro, vr_desconto, tributacao_id,
                                         context=None, natureza_id=None):
        context = context and context.copy() or dict()
        vr_base_calculo = vr_item + vr_despesa + vr_frete + vr_seguro - vr_desconto
        values = self.fisnit_get_tributacao_valor(
            cr, user, tributacao_id, vr_base_calculo, context, vr_despesa, incide_despesa,
            natureza_id)

        return {'value': values}

    def fisnit_get_tributacao_valor(self, cr, user, tributacao_id, vr_base_calculo, context=None,
                                    vr_despesa=0, incide_despesa=True, natureza_id=None):
        context = context and context.copy() or dict()
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]
        tributacao_dados = tributacao_id and self.pool.get('fiscal.tributacao').read(
            cr, user, tributacao_id, ['encargo_id', 'aliquota', 'base', 'cst_id'], context,
            '_classic_write') or {}
        encargo_id = tributacao_dados.get('encargo_id')
        aliquota = tributacao_dados.get('aliquota') or 0
        base = tributacao_dados.get('base') or 0
        vr_base = 0 < base < 100 and base * .01 * vr_base_calculo or vr_base_calculo
        vr_desoneracao = 0
        vr_reducao = vr_base_calculo - vr_base if 0 < base < 100 else 0
        # Remove o valor da despesa da base de calculo e acrescenta na reducao se o valor da
        # despesa nao incidir na base
        vr_reducao += vr_despesa if not incide_despesa else 0
        vr_base_calculo -= vr_reducao
        if vr_base_calculo < 0:
            vr_base_calculo = 0
        vr_encargo = aliquota / 100. * vr_base_calculo

        # ICMS
        if encargo_id == icms_id:
            # Desoneracao de ICMS. Nao aplicada se empresa for Simples Nacional
            desoneracao = False
            if natureza_id:
                dados_nat = self.pool.get('fiscal.cfop.natureza').read(cr, user, natureza_id,
                                    ['fisnat_complemento_arroz' ,'fisnat_icms_desonerado'], context)
                desoneracao = dados_nat.get('fisnat_icms_desonerado')
                if dados_nat.get('fisnat_complemento_arroz'):
                    vr_base = 0
                    vr_encargo = 0
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, user, context=context)
            pessoa_id = self.pool.get('res.company').read(
                cr, user, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                'rescon_pessoa_id')
            simples = self.pool.get('adm.pessoa').read(
                cr, user, pessoa_id, ['admpes_jur_regime_tributario'], context).get(
                'admpes_jur_regime_tributario') == 'S'
            vr_desoneracao = desoneracao and not simples and vr_encargo or 0

        return {'vr_base_calculo': vr_base, 'vr_encargo': vr_encargo, 'aliquota': aliquota,
                'vr_reducao_base': vr_reducao, 'fisnit_vr_icms_desonerado': vr_desoneracao}

    def fisnit_onchange_vr_base_calculo(self, cr, uid, ids, aliquota, vr_base_calculo,
                                        vr_reducao_base, context=None):
        context = context and context.copy() or {}
        vr_encargo = aliquota and (vr_base_calculo * aliquota / 100.) or 0
        vals = {'vr_encargo': vr_encargo}
        return {'value': vals}

    @staticmethod
    def fisnit_calcula_vr_base_calculo(vr_item, vr_frete=0, vr_despesas=0, vr_seguro=0,
                                       vr_desconto=0, vr_acrescimo=0, percentual_base=100,
                                       incide_despesa=True):
        """
        Este metodo e utilizado para realizar o calculo do valor da base de calculo do encargo
        :param vr_item: Valor do produtos/servico
        :param vr_frete: Valor do frete rateado para o produto
        :param vr_despesas: Valor da despesa acessoria referente ao item
        :param vr_seguro: Valor do seguro referente ao item
        :param vr_desconto: Valor do desconto referente ao item
        :param vr_acrescimo: Valor do acrescimo do item
        :param percentual_base: Percentual para calculo da base, reduzida ou integral
        :param incide_despesa: incide ou nao o valor da despesa na base de calculo
        :type vr_item: float
        :type vr_frete: float
        :type vr_despesas: float
        :type vr_seguro: float
        :type vr_desconto: float
        :type vr_acrescimo: float
        :type percentual_base: float
        :type incide_despesa: bool
        :return: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        incide_despesa = True if incide_despesa is None else incide_despesa
        base_calculo = vr_item + vr_frete + vr_seguro + vr_acrescimo
        base_calculo += vr_despesas if incide_despesa else 0
        base_calculo -= vr_desconto
        base_calculo = float(float_round((base_calculo * percentual_base) / 100., 2))

        return base_calculo

    @staticmethod
    def fisnit_calcula_vr_reducao_base(vr_item, vr_frete, vr_despesas, vr_seguro, vr_desconto,
                                       vr_acrescimo, base=100, incide_despesa=True):
        """
        Calcula o valor de reducao da base de calculo do encargo.
        
        :param vr_item: valor do item (produtos/servico)
        :param vr_frete: valor do frete rateado para o item
        :param vr_despesas: valor da despesa rateada referente ao item
        :param vr_seguro: valor do seguro rateado referente ao item
        :param vr_desconto: valor do desconto rateado referente ao item
        :param vr_acrescimo: valor do acrescimo do item
        :param base: percentual da base para calculo da reducao
        :param incide_despesa: incide ou nao o valor da despesa na reducao
        :type vr_item: float
        :type vr_frete: float
        :type vr_despesas: float
        :type vr_seguro: float
        :type vr_desconto: float
        :type vr_acrescimo: float
        :type base: float
        :type incide_despesa: bool
        :return: valor da reducao
        :rtype: float
        """
        incide_despesa = True if incide_despesa is None else incide_despesa  # Para NF de entrada

        # Nao ha reducao se o % da base nao for menor que 100
        if not base < 100:
            return vr_despesas if not incide_despesa else 0  # Inside ou nao na reducao

        vr_liquido = vr_item + vr_despesas + vr_frete + vr_seguro + vr_acrescimo - vr_desconto
        vr_reducao = float(float_round((vr_liquido * base) / 100., 2))
        vr_liquido += vr_despesas if not incide_despesa else 0  # Inside ou nao na reducao

        return vr_liquido - vr_reducao

    @staticmethod
    def fisnit_calcula_vr_encargo(base, aliquota):
        """
        Este metodo e utilizado para realizar o calculo do valor do encargo

        :param base: Valor da base de calculo
        :param aliquota: aliquota definida para o encargo
        :type base: float
        :type aliquota: float
        :return: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        return float(float_round(((base or 0) * (aliquota or 0)) / 100., 2))

    def fisnit_soma_ipi_base_pis_cofins(self, cr, uid, tributacoes, vr_total_item_rateado,
                                        context=None):
        """
        Este metodo soma o valor do ipi na base de calculo pis e cofins para empresas nao sao
        industrias conforme solicitacao do chamado 93158
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param tributacoes: lista de dicionarios contendo as tributacoes do item movimentado
        :param vr_total_item_rateado: valor total do item deduzido de descontos e acrescentado
        de seguro e outras despesas
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type tributacoes: list(dict)
        :type vr_total_item_rateado: float
        :type context: dict
        :rtype: list(dict)

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        vr_ipi = self.fisnit_get_total_ipi(cr, uid, tributacoes, context)
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name)
        tipo_atividade = self.pool.get('res.company').browse(
            cr, uid, company_id, context).rescon_pessoa_id.admpes_jur_tipo_atividade or False
        encargo_pis_cofins_ids = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', 'in', ['PIS', 'COFINS'])], context=context)
        if vr_ipi and tipo_atividade != 'I':
            for tributo in tributacoes:
                if tributo.get('encargo_id') in encargo_pis_cofins_ids:
                    vr_base_calculo = vr_total_item_rateado + vr_ipi
                    vr_encargo = self.fisnit_calcula_vr_encargo(vr_base_calculo, tributo.get(
                        'aliquota'))
                    tributo.update({'vr_base_calculo': vr_base_calculo,
                                    'vr_encargo': vr_encargo})
        return tributacoes

    def fisnit_get_total_ipi(self, cr, uid, tributacoes, context=None):
        """
        Este metodo filtra o encargo IPI nos tributos do item movimentado e retorna o valor
        Este valor sera utilizado para somar a base de pis/cofins conforme solicitacao do chamado
        93158
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param tributacoes: lista de dicionarios contendo as tributacoes do item movimentado
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type tributacoes: list(dict)
        :type context: dict
        :rtype: list(dict)

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        encargo_ipi_id = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', 'ilike', 'IPI')], context=context)
        total_ipi = 0
        for tributo in tributacoes:
            if tributo.get('encargo_id') in encargo_ipi_id:
                total_ipi += tributo.get('vr_encargo')

        return total_ipi

    def fisnit_get_tipo_consumidor_final(self, cr, uid, nf_id, context=None):
        """
        Este metodo verifica se o destinatario da nota e um contribuinte ou consumidor final
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: id da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf_id: int
        :type context: dict
        :rtype: boolean
        :return: Retorna True se contribuinte ou False se consumidor final
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        destinatario_id = self.pool.get('fiscal.nf.capa').read(
            cr, uid, nf_id, ['destinatario_id'], context, '_classic_write').get('destinatario_id')
        if not destinatario_id:
            return True
        context['super_read_adm_pessoa'] = True # nao faz tratamento no read

        return self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
            cr, uid, destinatario_id, context=context).get(destinatario_id) == 'F'

    def fisnit_calcula_icms_estado_destino(self, cr, uid, uf_id, encargo_id, vr_item, produto_nacional, context=None):
        """
        Este metodo calcula o icms do do estado destino
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param uf_id: id do destino da venda
        :param encargo_id: id do encargo para realizar o calculo
        :param vr_item: total liquido do item, este valor ja deve ter todos os seus valores extras
                        embutidos como icms/st desconto, ipi, despesas acessorias etc.
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type uf_id: int
        :type encargo_id: int
        :type vr_item: float
        :type context: dict
        :rtype: float
        :return: Retorna o valor do icms calculado pela aliquota interna
        """
        trib_interna = \
            self.pool.get('fiscal.tributacao.interestadual').fistin_get_tributacao_interna(
                cr, uid, uf_id, encargo_id, produto_nacional, context)
        base_interna = trib_interna.get('base') or 0
        aliquota_interna = trib_interna.get('aliquota') or 0
        aliquota_interestadual = trib_interna.get('aliquota_inter') or 0
        if trib_interna.get('base_dupla'):
            vr_base_sem_icms = vr_item - (vr_item * aliquota_interestadual / 100.)
            base_difal = vr_base_sem_icms / ((100. - aliquota_interna)/100)
            icms_destino = base_difal * base_interna / 100. * aliquota_interna / 100.
        else:
            base_difal = vr_item
            icms_destino = vr_item * base_interna / 100. * aliquota_interna / 100.
        return base_difal, icms_destino

    def fisnit_calcula_icms_estado_origem(self, cr, uid, uf_id, encargo_id, vr_item,
                                          produto_nacional, context=None):
        """
        Este metodo calcula o icms do do estado origem
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param uf_id: id do destino da venda
        :param encargo_id: id do encargo para realizar o calculo
        :param vr_item: total liquido do item, este valor ja deve ter todos os seus valores extras
                        embutidos como icms/st desconto, ipi, despesas acessorias etc.
        :param produto_nacional: informa se a origem do item eh nacional
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type uf_id: int
        :type encargo_id: int
        :type vr_item: float
        :type produto_nacional: bool
        :type context: dict
        :rtype: float
        :return: Retorna o valor do icms calculado pela aliquota interna
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        obj_fistrib_inter = self.pool.get('fiscal.tributacao.interestadual')
        trib_interestadual = obj_fistrib_inter.fistin_get_tributacao_interestadual(
            cr, uid, uf_id, encargo_id, produto_nacional, context)
        base_interestadual = trib_interestadual.get('base') or 0
        aliq_interestadual = trib_interestadual.get('aliquota') or 0
        icms_origem = vr_item * base_interestadual / 100. * aliq_interestadual / 100.

        return icms_origem

    def fisnit_calcula_icms_difal(self, cr, uid, uf_id, encargo_id, vr_item, ano_emissao_nf,
                                  produto_nacional, context=None):
        """
        Este metodo calcula o icms difal para o estado origem e para o estado destino de acordo
        com a tributacao interestadual cadastrada em 'fiscal.tributacao.interestadual'.
        Este calculo sera realizado observando o ano de emissao da nota fiscal, tendo como
        percentual para calculo de origem e destino os valores estipulados internamente no codigo
        atraves do metodo fisnit_get_percentual_difal.

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de ids de fiscal.nf.item.tributacao
        :param dados_nf: dados da nota fiscal ('fiscal.nf.capa')
        :param uf_id: do destino da venda
        :param encargo_id: id do encargo para realizar o calculo
        :param vr_item: total liquido do item, este valor ja deve ter todos os seus valores extras
        embutidos como icms/st desconto, ipi, despesas acessorias etc.
        :param ano_emissao_nf: Ano da emissao da nota fiscal
        :param produto_nacional: informa se a origem do item eh nacional
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type uf_id: int
        :type encargo_id: int
        :type vr_item: float
        :type ano_emissao_nf: int
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo os valores de difal para a origem e destino.
                 As chaves fisnit_vr_difal_uf_origem e fisnit_vr_difal_uf_destino possuem o mesmo
                 nome dos atributos que seram persistido em fiscal.nf.item.tributacao
        """
        context = context and context.copy() or {}
        base_difal, icms_destino = self.fisnit_calcula_icms_estado_destino(
            cr, uid, uf_id, encargo_id, vr_item, produto_nacional, context)
        icms_origem = icms_destino and self.fisnit_calcula_icms_estado_origem(
            cr, uid, uf_id, encargo_id, vr_item, produto_nacional, context) or 0
        icms_difal = icms_destino - icms_origem
        percentual_difal = self.fisnit_get_percentual_difal(ano_emissao_nf)
        icms_difal_destino = (icms_difal * percentual_difal.get('destino')) / 100
        icms_difal_destino = \
            float(self._float_persist('fisnit_vr_difal_uf_destino', icms_difal_destino))
        icms_difal_origem = (icms_difal * percentual_difal.get('origem')) / 100
        icms_difal_origem = \
            float(self._float_persist('fisnit_vr_difal_uf_origem', icms_difal_origem))

        return {'fisnit_vr_difal_uf_origem': icms_difal_origem,
                'fisnit_vr_difal_uf_destino': icms_difal_destino,
                'fisnit_vr_difal_base': base_difal}

    def fisnit_apurar_difal(self, cr, uid, dados_nf, context=None):
        """
        Este metodo verifica se para a movimentacao o difal deve ser apurado ou nao
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param brw_nf: browse da nota fiscal (fiscal.nf.capa)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type dados_nf: dados_nf
        :type context: dict
        :rtype: bool
        :return: Retorna True se deve apurar o difal para a movimentacao, se nao False
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not dados_nf.get('destinatario_id'):
            return False
        if self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                cr, uid, dados_nf.get('destinatario_id'), context) == 'F':

            # Os dados da nota vem por parametro para otimizar o processo para notas muito grandes
            cfop_id = self.pool.get('fiscal.cfop.natureza').read(
                cr, uid, dados_nf.get('natureza_id'), ['cfop_id'], context,
                '_classic_write').get('cfop_id')
            dados_cfop = self.pool.get('fiscal.cfop').read(
                cr, uid, cfop_id, ['codigo', 'movimento_tipo'], context)
            cfop = dados_cfop.get('codigo')
            movimento_tipo = dados_cfop.get('movimento_tipo')
            if cfop != '6929' and cfop[0] == '6' and movimento_tipo == 'C':
                return True

        return False

    @staticmethod
    def fisnit_get_percentual_difal(ano):
        """
        Este metodo retorna os percentuais que deveram ser utilizados para calculo do difal de
        acordo com o ano da emissao da nota. Estes valores sao fixos.
        :param ano: Ano da emissao da nota fiscal
        :rtype: dict
        :return: Retorna um dicionario informando o percentual para calculo da origem e o percentual
                 para calculo do destino de acordo com o ano da emissao
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if ano < 2016:
            return {'origem': 0, 'destino': 0}
        elif ano > 2018:
            return {'origem': 0, 'destino': 100}

        return {2016: {'origem': 60, 'destino': 40}, 2017: {'origem': 40, 'destino': 60},
                2018: {'origem': 20, 'destino': 80}}.get(ano)

    def fisnit_get_total_item_com_tributos(self, cr, uid, vr_item, tributacoes, dict_encargo,
                                           context=None):
        """
        Este metodo calcula o total do item acrescentando os tributos e demais depesas e descontos
        no total do item devem ser acrescentado os valores de despesas, seguro, frete icms/st e ipi
        e decrescido os valores de desconto
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type context: dict
        :rtype: float
        :return: Retorna o valor calculado do item
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        for tributo in tributacoes:
            if dict_encargo.get(tributo[2].get('encargo_id')) == 'IPI':
                vr_item += tributo[2].get('vr_encargo')
            elif dict_encargo.get(tributo[2].get('encargo_id')) == 'ICMS':
                vr_item += tributo[2].get('vr_icms_substituicao') or 0

        return vr_item

    def fisnit_gerar_difal(self, cr, uid, dados_nf, context=None):
        """
        Este metodo e responsavel por gerar os valores que alimentam os campos
        fisnit_vr_difal_uf_origem, fisnit_vr_difal_uf_destino
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type context: dict
        :rtype: dict
        :return: Retorna um dicionario contendo os valores dos atributos fisnit_vr_difal_uf_destino
        e fisnit_vr_difal_uf_origem
        """
        from utils import adjust_m2o
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]
        context = context and context.copy() or {}
        obj_produto = self.pool.get('estoque.produto')
        if dados_nf.get('destinatario_endereco_id'):
            uf_id, uf_destinatario = self.pool.get(
            'adm.pessoa.endereco').read(
            cr, uid, dados_nf.get('destinatario_endereco_id'), ['uf_id'], context).get('uf_id')
        else:
            uf_id = False
            uf_destinatario = ''
        brw_company = self.pool.get('res.company').browse(cr, uid, dados_nf.get('company_id'), context=context)
        uf_origem = brw_company.rescon_pessoa_id.uf
        produto_ids = []
        encargo_ids = []
        context['super_read_estoque_movimento_itens'] = True
        dados_nf['estoque_movimento_itens_ids'] = [
            [0 if not i[0] else 1 if i[0] in [1, 4] else i[0], i[1], adjust_m2o(
                self.pool.get('fiscal.nf.capa').resolve_2many_commands(
                    cr, uid, 'estoque_movimento_itens_ids', [i],
                    ['produto_id', 'tributacoes_ids', 'vr_total_item_rateado'], context)[0])]
            for i in [j if isinstance(j, (list, tuple)) else [4, j, False]
                      for j in dados_nf.get('estoque_movimento_itens_ids')]]
        for i in dados_nf['estoque_movimento_itens_ids']:
            i = adjust_m2o(i[2])
            produto_ids.append(i.get('produto_id'))
            tributacoes = self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                cr, uid, 'tributacoes_ids', i.get('tributacoes_ids'), ['encargo_id'])
            if not tributacoes:
                continue
            for j in tributacoes:
                j = adjust_m2o(j)
                if j.get('encargo_id'):
                    encargo_ids.append(j.get('encargo_id'))
        dict_produtos = {}
        map(lambda x: dict_produtos.update({x.get('id'): x.get('origem_mercadoria')}),
            obj_produto.read(cr, uid, produto_ids, ['origem_mercadoria'], context))
        dict_encargo = {}
        map(lambda x: dict_encargo.update({x.get('id'): x.get('sigla')}),
            self.pool.get('fiscal.encargo').read(cr, uid, encargo_ids, ['sigla'], context))
        ano_emissao = datetime.strptime(dados_nf.get('data_emissao'), '%Y-%m-%d %H:%M:%S').year
        for i in dados_nf['estoque_movimento_itens_ids']:
            item = adjust_m2o(i[2])
            origem_mercadoria = dict_produtos.get(item.get('produto_id'))
            produto_nacional = obj_produto.estpro_produto_nacional(origem_mercadoria)
            item['tributacoes_ids'] = [
                [0 if not t[0] else 1 if t[0] in [1, 4] else t[0], t[1], adjust_m2o(
                    self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                        cr, uid, 'tributacoes_ids', [t], [])[0])]
                for t in item.get('tributacoes_ids')]
            if not item.get('tributacoes_ids'):
                continue
            vr_item = self.fisnit_get_total_item_com_tributos(
                cr, uid, item.get('vr_total_item_rateado') or 0, item.get('tributacoes_ids'),
                dict_encargo, context)
            for j in map(lambda m: m[2], item.get('tributacoes_ids')):
                encargo_id = j.get('encargo_id')
                tributado = self.pool.get('fiscal.cst').read(
                    cr, uid, j.get('cst_id'), ['tipo']).get('tipo') in ['T', 'TU']
                tributado = tributado or context.get('fisnat_calcula_difal')  # Forca o calculo!
                icms_difal = {'fisnit_vr_difal_uf_origem': 0,
                              'fisnit_vr_difal_uf_destino': 0,
                              'fisnit_vr_difal_base': 0}

                # Somente para NAO contribuinte
                if not self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                        cr, uid, dados_nf.get('destinatario_id'), context=None) == 'C':
                    if encargo_id == icms_id and tributado and uf_destinatario != uf_origem:
                        icms_difal = uf_id and self.fisnit_calcula_icms_difal(
                            cr, uid, uf_id, encargo_id, vr_item - j.get('vr_reducao_base', 0),
                            ano_emissao, produto_nacional, context) or {}
                j.update(icms_difal)

        return True

    def fisnit_calcula_tributos_interestaduais(self, cr, uid, item_id, uf_id, encargo_id, vr_item,
                                               produto_nacional, context=None, natureza_id=False):
        """
        Este metodo e responsavel por gerar os valroes dos tributos na nota fiscal de saida
        quando a movimentacao for interestadual. Quando existir tributos cadastrados para
        o estado de destino da nota em 'fiscal.tributacao.interestadual', este ira substituir os
        tributos cadastrados no item pela aliquota interestadual, quando o item for tributado.
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param item_id: o ID do item de movimentacao (produto/servico)
        :param uf_id: Id do estado de destino da nota
        :param encargo_id: Id do encargo em fiscal.encargo
        :param vr_item: Valor liquido do item
        :param produto_nacional: informa se a origem do produto eh nacional
        :param context: Dicionario dos contextos
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type uid: int
        :type item_id: long
        :type uf_id: int
        :type encargo_id: int
        :type vr_item: float
        :type context: dict
        :type natureza_id: long
        :rtype: dict
        :return: Retorna um dicionario contendo os valores dos tributos calculados de acordo
        com a aliquota interestadual
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        tributos = {}
        obj_fistrib_inter = self.pool.get('fiscal.tributacao.interestadual')
        trib_interestadual = obj_fistrib_inter.fistin_get_tributacao_interestadual(
            cr, uid, uf_id, encargo_id, produto_nacional, context)
        if trib_interestadual:
            tributacao_id = trib_interestadual.get('tributacao')
            tributos = self.calcula_tributacao_item(
                cr, uid, item_id, tributacao_id, vr_item, context, natureza_id)
            tributos['cst_id'] = trib_interestadual.get('cst')
            tributos['base'] = trib_interestadual.get('base') or 0.0
            tributos['tributacao_id'] = tributacao_id

        return tributos

    def fisnit_get_vr_fcp(self, cr, uid, dados_fisnit, context=None):
        """
        Este metodo retorna o valor do FCP de caso o destinatario seja consumidor final
        :return: Retorna o valor do FCP caso o destinatario seja um consumidor final
        :rtype: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        tipo_consumidor = dados_fisnit.get('tipo_consumidor')
        if not tipo_consumidor or tipo_consumidor != 'F':
            return 0
        aliquota_fcp = dados_fisnit.get('aliquota_fcp')
        base_fcp = dados_fisnit.get('vr_base_calculo')

        return self.fisnit_calcula_vr_fcp(base_fcp, aliquota_fcp)

    @staticmethod
    def fisnit_calcula_vr_fcp(base, aliquota):
        """
        Este metodo calcula o valor do FCP
        :param base: Valor da base de calculo de icms para calculo do FCP,
                     o valor da base de calculo
                     formula(vr_item + vr_despesas + vr_frete + vr_seguro + vr_acrescimo -
                     vr_desconto), o mesmo valor recuperado do metodo fisnit_calcula_vr_base_calculo
        :type base: float
        :type aliquota: Valor da aliquota do FCP
        :type aliquota: float
        :return: Retorna o valor do FCP caso o parametro base e aliquota sejam maior 0
        :rtype: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """

        return float(float_round(base * aliquota / 100, 2)) if aliquota and base else 0.0

    def fisnit_get_vr_fcp_st(self, cr, user, dados_fisnit, context=None):
        """
        Calcula o valor do FCP/ST.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao do perfil
        :param tributacao: o ID da tributacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type tributacao: long
        :type context: dict
        :return: o valor do FCP/ST
        :rtype: float
        """
        context = context and context.copy() or {}
        tipo_consumidor = dados_fisnit.get('tipo_consumidor')
        if not tipo_consumidor or tipo_consumidor != 'F':
            return 0
        base = dados_fisnit.get('vr_base_icms_substituicao') or 0
        aliquota = dados_fisnit.get('fistri_aliquota_fcp')

        return base * aliquota / 100.

    def fisnit_onchange_vr_base_icms_substituicao(self, cr, user, ids, tributacao, base,
                                                  context=None):
        """
        Calcula o valor do FCP/ST em funcao do valor da base de ICMS/ST.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao do perfil
        :param tributacao: o ID da tributacao do item
        :param base: valor da base de calculo do ICMS/ST
        :param context: um dicionario padrao para valores contextuais
        :param ids: lista com o ID da tributacao do item
        :type cr: object
        :type user: int
        :type ids: list
        :type tributacao: long
        :type base: float
        :type context: dict
        :return: o valor do FCP/ST
        :rtype: float
        """
        context = context and context.copy() or {}
        if not tributacao:
            return {}
        dados_trib = self.pool.get('fiscal.tributacao').read(
            cr, user, tributacao, ['fistri_aliquota_fcp'], context)
        dados_trib['vr_base_icms_substituicao'] = base

        return {'value': {'fisnit_vr_fcp_st': self.fisnit_get_vr_fcp_st(
            cr, user, dados_trib, context)}}

    def fisnit_exclui_valor_icms_da_base_calculo_pis_cofins(
            self, cr, uid, estoque_movimento_itens_ids, context=None):
        """
        Metodo responsavel por tratar a exclusao do valor do ICMS nas bases de cálculo do PIS e
        COFINS
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: usuario ativo na transacao, id da tabela res_users
        :type uid: int
        :param estoque_movimento_itens_ids: conjunto de identificadores do movimento do estoque,
        ids da tabela estoque_movimento_itens
        :type estoque_movimento_itens_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Falso, se nenhuma base de calculo for atualizada. Senao, verdadeiro.
        """
        context = context and context.copy() or {}
        res = False
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        # Se o parametro nao esta definido, nao ha o que deduzir
        if not self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'fiscal', 'fispar_deduz_icms_do_pis_cofins', context=context).get(
                'fispar_deduz_icms_do_pis_cofins'):
            return res

        if not isinstance(estoque_movimento_itens_ids, (list, tuple)):
            estoque_movimento_itens_ids = [estoque_movimento_itens_ids]

        # Busca pelas tributacoes dos movimentos recebidos, oriundos de notas fiscais
        nf_item_tributacao_ids = self.search(
            cr, uid, [
                ('movimento_itens_id', 'in', estoque_movimento_itens_ids),
                ('fisnit_capa_id', '!=', False)
            ]
        )
        # Se nao ha tributacoes para esses movimentos do estoque oriundos de notas fiscais, nao ha
        # o que deduzir
        if not nf_item_tributacao_ids:
            return res

        # Encontra as notas fiscais eletronicas previstas pela regra (55, 57 e 65), dentre as
        # tributacoes encontradas
        nf_ids = list(set(map(lambda fnit: fnit.fisnit_capa_id.id, filter(
            lambda it: it.fisnit_capa_id.modelo_nf_id.fismnf_codigo in (
                '55', '57', '65'), self.browse(cr, uid, nf_item_tributacao_ids, context)))))

        # Encontra os movimentos do estoque baseados nos movimentos recebidos e nas notas fiscais
        # eletronicas (55, 57, 65) encontradas nas tributacoes
        estoque_movimento_itens_ids = obj_estoque_movimento_itens.search(
            cr, uid, [('id', 'in', estoque_movimento_itens_ids), ('nf_id', 'in', nf_ids)])
        for movimento_item in obj_estoque_movimento_itens.browse(
                cr, uid, estoque_movimento_itens_ids, context):
            if not movimento_item.vr_total_item_rateado:
                continue
            vr_base_calculo_pis_cofins = movimento_item.vr_total_item_rateado
            valor_icms = 0
            brw_tributacao_pis = None
            brw_tributacao_cofins = None
            for tributacao in movimento_item.tributacoes_ids:
                if tributacao.encargo_id.sigla == 'ICMS' and not tributacao.vr_encargo:
                    continue
                if tributacao.encargo_id.sigla in ('PIS', 'COFINS') \
                        and not tributacao.vr_base_calculo:
                    continue
                if tributacao.encargo_id.sigla == 'ICMS':
                    valor_icms = tributacao.vr_encargo
                elif tributacao.encargo_id.sigla == 'PIS':
                    brw_tributacao_pis = tributacao
                elif tributacao.encargo_id.sigla == 'COFINS':
                    brw_tributacao_cofins = tributacao
            (valor_icms and brw_tributacao_pis) and brw_tributacao_pis.write(
                {'vr_base_calculo': vr_base_calculo_pis_cofins - valor_icms,
                 'vr_encargo': round((vr_base_calculo_pis_cofins - valor_icms) *
                                     (brw_tributacao_pis.aliquota / 100.0), 2)
                 }
            )
            (valor_icms and brw_tributacao_cofins) and brw_tributacao_cofins.write(
                {'vr_base_calculo': vr_base_calculo_pis_cofins - valor_icms,
                 'vr_encargo': round((vr_base_calculo_pis_cofins - valor_icms) *
                                     (brw_tributacao_cofins.aliquota / 100.0), 2)
                 }
            )
            res = True
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
