# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import osv
from SigUtils.datas import calcDiferenca


class nf_terceiros(osv.Model):
    """
    Nota fiscal de terceiros (somente entrada).
    """
    _inherit = 'fiscal.nf.capa'

    def fisnfc_onchange_parcelas_terceiros(self, cr, user, ids, parcelas, context=None):
        """
        Atualiza o prazo em funcao dos valores das parcelas.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param parcelas: lista com os dados das parcelas (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type parcelas: list
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        from datetime import datetime
        data_emissao = ids and self.browse(cr, user, ids[0], context).data_emissao
        data_emissao = data_emissao and data_emissao[:10] or datetime.now().strftime('%Y-%m-%d')
        _parcelas = self.resolve_2many_commands(
            cr, user, 'fisnfc_parcelas_terceiros_ids', parcelas, ['vencimento'])
        prazo = ', '.join([str(p) for p in sorted(list(set(map(
            lambda m: abs(calcDiferenca(m['vencimento'], data_emissao)), _parcelas))))]) or '0'

        return dict(value=dict(prazo=prazo))

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
