# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.nfce.manual_600.webservices_flag import WS_CONSULTA_QRCODE, WS_CONSULTA_CHAVE
from openerp.modules.module import load_information_from_description_file as info
from ..report.promissoria import rel_fiscal_promissoria
from pynfe.xml_sped.certificado import Certificado
from pynfe.xml_sped.__init__ import CODIGO_UF
from openerp.tools import config as openerp_config
from pynfe.nfce.manual_600 import danfe_nfce_bobina as bobina
from pynfe.nfe.manual_600.cancnfe_evento import EnvEvento, RetEnvEvento, ProcEventoNFe
from pynfe.nfe.manual_600.processador import ProcessoNFe
from pynfe.xml_sped.__init__ import UF_CODIGO
from pynfe.nfce.manual_600.webservices import ESTADO_WS
from pynfe.nfce.manual_600.consrecinfce import ProcNFCe
from openerp.report import report_sxw
from openerp.tools.translate import _
from pynfe.nfce.manual_600.NFCe import NFCe as nfc_e
from SigUtils import calcDigito
from pynfe.nfce.manual_600.nfce_p import NFCe
from datetime import datetime
from pynfe.nfce.manual_600 import qr_code
from openerp.osv import osv
from pynfe import __version__
from PIL import Image
import StringIO
import subprocess
import tempfile
import hashlib
import logging
import socket
import base64
import pytz
import time
import re

# Ambientes para a nota fiscal
HOMOLOGACAO = 2
PRODUCAO = 1


class fiscal_nfce(osv.Model):
    """
    Nota fiscal de consumidor eletronica.
    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    _inherit = 'fiscal.nf.capa'
    _logger = logging.getLogger(_inherit)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    def _auto_init(self, cr, context=None):
        context = context and context.copy() or {}
        super(fiscal_nfce, self)._auto_init(cr, context)
        cr.execute("SELECT 1 FROM pg_indexes WHERE indexname='fiscal_nf_capa_contingencia_offline_index'")
        if not cr.fetchone():
            cr.execute(
                'CREATE INDEX fiscal_nf_capa_contingencia_offline_index ON fiscal_nf_capa (fisnfc_consumidor, '
                                                              'fisnfc_nfce_contingencia_offline, company_id, state)')

    def dados_onchange_modelo(self, cr, user, modelo, propria, context=None):
        """
        Ajusta os valores para nota fiscal de consumidor.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param modelo: o ID do modelo de nota
        :param propria: Indicacao de nota fiscal propria ou de terceiro
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type modelo: long
        :type propria: bool
        :type context: dict
        :return: dados ajustados para tipo de nota fiscal
        :rtype:
        """
        context = context and context.copy() or {}
        result = super(fiscal_nfce, self).dados_onchange_modelo(cr, user, modelo, propria, context)
        if context.get('nota_fiscal_consumidor') and not result.get('modelo_nf_id'):
            raise osv.except_osv(_('Atencao!'), _('O modelo para nota fiscal de consumidor nao foi '
                                                  'definido.'))
        elif result.get('fisnfc_modelo_serie', '')[:2] == '65':
            result = dict(result, fisnfc_consumidor=True, nfe_forma_impressao_danfe='4',
                          modelo_55=False, data_saida_entrada=False, finalidade_emissao='1',
                          nfe_forma_emissao='1')

        return result

    def onchange_modelo_nf(self, cr, user, ids, modelo, tipo, propria, context=None):
        context = context and context.copy() or {}
        result = super(fiscal_nfce, self).onchange_modelo_nf(
            cr, user, ids, modelo, tipo, propria, context)
        if context.get('nota_fiscal_consumidor') and not context.get('fisnfc_nfe_de_nfce'):
            if 'value' in result and result['value'].get('fisnfc_modelo_serie')[:2] != '65':
                result['value']['modelo_nf_series_id'] = False
                result['warning'] = dict(
                    title=_('Atencao!'), message=_('O modelo de nota fiscal informado nao e valido '
                                                   'para NFC-e.'))
        if propria and tipo == 'S' and self.pool.get('fiscal.modelo.nf.series').browse(
            cr, user, modelo, context).modelo_nf_id.fismnf_tipo_nfe:
            validade_certificado = self.fisnfc_validade_certificado_digital(cr, user, context)
            if validade_certificado.days < 0:
                result['warning'] = {
                    'title': _('Aviso.'),
                    'message': _('Certificado digital com data expirada.')
                }
            elif validade_certificado.days < 15:
                result['warning'] = {
                    'title': _('Aviso.'),
                    'message': _('Certificado digital proximo de expirar. %d dia%s para '
                                 'o vencimento.' % (
                        validade_certificado.days, 's' if validade_certificado.days > 1 else ''))
                }
        return result

    def onchange_natureza(self, cr, user, ids, natureza, empresa, endereco, tipo, itens,
                          consumidor=False, destinatario=False, context=None):
        """
        Verifica natureza da operacao da nota fiscal de consumidor.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param natureza: o ID da natureza da operacao
        :param empresa: o ID da empresa
        :param tipo: tipo da nota fiscal
        :param endereco: o ID do endereco do destinatario da nota fiscal
        :param itens: lista com o ID dos itens da nota fiscal
        :param consumidor: indicativo de nota fiscal de consumidor
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type natureza: long
        :type empresa: long
        :type tipo: str
        :type endereco: long
        :type itens: list
        :type consumidor: bool
        :type context: dict
        :return: valores ajustados na nota fiscal.
        :rtype: dict
        """
        context = context and context.copy() or {}
        result = super(fiscal_nfce, self).onchange_natureza(
            cr, user, ids, natureza, empresa, endereco, tipo, itens, consumidor, destinatario, context)
        if consumidor and natureza:
            # CFOP validos para NFC-e
            validos = ['5101', '5102', '5103', '5104', '5115', '5405', '5656', '5667', '5933']

            brw_cfop_natureza =self.pool.get('fiscal.cfop.natureza').browse(
                cr, user, natureza, context)
            if brw_cfop_natureza.cfop_id.movimento_tipo != 'C':
                # Natureza somente para compra/venda
                msg = _('Nota fiscal de consumidor nao pode ter CFOP com tipo de movimento '
                        'diferente de compra/venda')
                result['warning'] = dict(title=_('Aviso.'), message=msg)
            elif brw_cfop_natureza.cfop_id.codigo not in validos:
                msg =_('o CFOP %s nao e valido para nota fiscal de consumidor.')
                msg %= brw_cfop_natureza.cfop_id.codigo
                result['warning'] = dict(title=_('Aviso.'), message=msg)
        if result.get('warning'):
            result['value']['natureza_id'] = False

        return result

    def fisnfc_onchange_vr_frete(self, cr, user, ids, tipo, modelo, itens, vr_frete, tipo_frete,
                                 nat_id, context=None):
        """
        Na NFC-e havendo valor de frete o indicador de presenca do comprador deve ser ajustado para
        ´´4 - NFC-e em operacao com entrega em domicilio´´.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param tipo_nf: tipo da nota fiscal: ´E´= Entrada ou ´S´= Saida
        :param modelo_nf_id: o ID do modelo da nota
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param vr_frete: o valor do frete na capa da nota fiscal
        :param tipo_frete: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type tipo_nf: str
        :type modelo_nf_id: long
        :type itens: list
        :type vr_frete: float
        :type tipo_frete: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        result = super(fiscal_nfce, self).fisnfc_onchange_vr_frete(
            cr, user, ids, tipo, modelo, itens, vr_frete, tipo_frete, nat_id, context)
        if context.get('nota_fiscal_consumidor') and vr_frete:
            if 'value' in result:
                result['value']['fisnfc_nfe_indicador_presenca'] = '4'
            else:
                result['value'] = dict(fisnfc_nfe_indicador_presenca='4')

        return result

    def fisnfc_button_gerar_danfe_nfce(self, cr, user, ids, context=None):
        """
        # Gera DANFE da Nota Fiscal de Consumidor Eletronica
        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal (NFC-e)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user:  int
        :type ids: list
        :type context: dict
        :return: DANFE/NFC-e
        :rtype: object
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        brw_nfce = self.browse(cr, user, ids[0], context)
        logo = self._fisnfc_get_logo_danfe_nfce(
            self.pool.get('res.company').browse(cr, user, brw_nfce.company_id.id, context).logo)
        context['active_id'] = ids[0]
        context['gerar_danfe'] = True
        if not parametro.fiscal.fispar_nfce_impressora_danfe or \
                parametro.fiscal.fispar_nfce_impressora_formato != 'BO':
            res_id = self._fisnfc_gerar_danfe_nfce_pdf(cr, user, ids[0], logo, context)
            return {'name': _('Impressao de DANFE'), 'view_type': 'form', 'view_mode': 'form',
                    'res_id': res_id, 'res_model': 'fiscal.export.nfe.osv', 'view_id': False,
                    'target': 'new', 'type': 'ir.actions.act_window'}
        else:
            danfe_file = tempfile.NamedTemporaryFile(mode='w+')
            danfe_file.write(
                self._fisnfc_gerar_danfe_nfce_bobina(cr, user, ids[0], context))
            danfe_file.flush()
            printer = parametro.fiscal.fispar_nfce_impressora_danfe
            subprocess.call(["lp", "-o raw", "-n1", '-d%s' % printer, danfe_file.name])
            danfe_file.close()
            raise osv.except_osv(
                _('Atencao!'), _('Impressao do DANFE diretamente para a impressora %s.' % printer))

    def _fisnfc_gerar_danfe_nfce_pdf(self, cr, user, nfce_id, logo, context=None):
        context = context and context.copy() or {}
        brw_nfce = self.browse(cr, user, nfce_id, context)
        # ajusta o xml de cancelamento caso esteja errado
        if brw_nfce.state == 'C' and (not brw_nfce.nfe_xml_cancelamento or
                                      ('procEventoNFe' not in brw_nfce.nfe_xml_cancelamento or \
            brw_nfce.nfe_xml_cancelamento and brw_nfce.nfe_xml_cancelamento.count('retCancNFe') == 1)):
            cr.execute("""SELECT * 
                          FROM fiscal_parametro
                          WHERE company_id = %s""" % brw_nfce.company_id.id)
            dados_par = cr.dictfetchall()[0]
            if dados_par.get('fispar_nfe_ambiente') == 1 and not brw_nfce.fisnfc_nfe_status_configuracao_producao or \
                    dados_par.get('fispar_nfe_ambiente') != 1 and brw_nfce.fisnfc_nfe_status_configuracao_producao:
                raise osv.except_osv(_('Atencao!'), _('Nota com xml de cancelamento invalido e ambiente da nota'
                                                      ' diferente do parametro do fiscal!'))
            self.fisnfc_ajustar_protocolo_autorizacao_nfce(cr, user, nfce_id, dados_par, context)
            brw_nfce = self.browse(cr, user, nfce_id, context)

        return self.pool.get('fiscal.export.nfe.osv').create(
            cr, user, {'xml_export': base64.encodestring(nfc_e().danfe_nfce(
                brw_nfce.nfe_xml_proc, brw_nfce.nfe_xml_cancelamento, logo))}, context)

    def _fisnfc_gerar_danfe_nfce_bobina(self, cr, user, nfce_id, context=None):
        context = context and context.copy() or {}
        brw_nfce = self.browse(cr, user, nfce_id, context)
        parametro = self._parametros(cr, user)
        marca_id = parametro.fiscal.fispar_nfce_impressora_marca
        brw_imp = self.pool.get('fiscal.nfce.impressora').browse(cr, user, marca_id, context)

        # Define a impressora do QrCode
        printer_qrcode = self.pool.get('fiscal.nfce.impressora').read(
            cr, user, parametro.fiscal.fispar_nfce_impressora_marca, ['fisimp_qrcode'], context)

        nfce = ProcNFCe()
        if not brw_nfce.nfe_xml_proc:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro ao montar xml da nota! Sem xml gerado ou sem xml de autorizacao!'))

        nfce.xml = str(brw_nfce.nfe_xml_proc)
        from pynfe.nfce.manual_600.report import Style
        style = Style()
        style.reset = None, brw_imp.fisimp_espacamento_linha.decode('string_escape')
        style.bold = brw_imp.fisimp_negrito_desliga.decode('string_escape'), \
                     brw_imp.fisimp_negrito_liga.decode('string_escape')
        style.underline = brw_imp.fisimp_sublinhado_desliga.decode('string_escape'), \
                          brw_imp.fisimp_sublinhado_liga.decode('string_escape')
        style.expanded = brw_imp.fisimp_expande_desliga.decode('string_escape'), \
                         brw_imp.fisimp_expande_liga.decode('string_escape')
        style.condensed = brw_imp.fisimp_condensado_desliga.decode('string_escape'), \
                          brw_imp.fisimp_condensado_liga.decode('string_escape')
        style.paper_cut = None, brw_imp.fisimp_corta_papel.decode('string_escape')
        bobina.Line._styles = style
        report = '\n'.join([bobina._header(nfce.NFe.infNFe.emit),
                            bobina._detail(nfce.NFe.infNFe.det),
                            bobina._sumary(nfce.NFe.infNFe),
                            bobina._access(nfce),
                            bobina._client(nfce.NFe.infNFe.dest),
                            bobina._identify(nfce, str(printer_qrcode.get('fisimp_qrcode'))),
                            bobina._complement(nfce.NFe.infNFe.infAdic.infCpl.valor),
                            bobina._footer()])
        return report

    def _fisnfc_gerar_promissoria_bobina(self, cr, user, nfce_id, context=None):
        import locale
        from textwrap import wrap
        context = context and context.copy() or {}
        brw_nfce = self.browse(cr, user, nfce_id, context)
        parametro = self._parametros(cr, user)
        marca_id = parametro.fiscal.fispar_nfce_impressora_marca
        brw_imp = self.pool.get('fiscal.nfce.impressora').browse(cr, user, marca_id, context)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        def remove_accents(text):
            from unicodedata import normalize, category
            unicode_string = unicode(text, 'utf-8') if type(text) == str else text
            text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

            return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

        reset = u'\x1b\x40' + brw_imp.fisimp_espacamento_linha.decode('string_escape')
        cut_paper = brw_imp.fisimp_corta_papel.decode('string_escape')
        bold_on = brw_imp.fisimp_negrito_liga.decode('string_escape')
        bold_off = brw_imp.fisimp_negrito_desliga.decode('string_escape')
        report = [reset, bold_on, u'{:^42}\n'.format(
            _('NOTA PROMISSORIA Nr. %s') % brw_nfce.numero_nf), bold_off, u'\n',
                  u'{:<9.9}: {:{dfmt} {tfmt}}\n'.format(
                      _('Data/hora'), datetime.now(), dfmt='%d/%m/%Y', tfmt='%Hh%M')]
        endereco = u'{:} - {:}, {:}/{:} - {:}\n'.format(
            u'{:<8.8}: {:}'.format(_('Endereco'),
                                  brw_nfce.company_id.rescon_pessoa_id.endereco.strip()),
            brw_nfce.company_id.rescon_pessoa_id.bairro.strip(),
            brw_nfce.company_id.rescon_pessoa_id.cidade.strip(),
            brw_nfce.company_id.rescon_pessoa_id.uf.strip(),
            brw_nfce.company_id.rescon_pessoa_id.cep)
        endereco = [w + u'\n' for w in wrap(endereco, 42)]
        report.extend(endereco)
        label = u'CPF' if brw_nfce.company_id.rescon_pessoa_id.pessoa_tipo == 'F' else u'CNPJ'
        report.append(u'{:<4.4}: {:}\n'.format(
            label, brw_nfce.company_id.rescon_pessoa_id.cpf_cnpj))
        report.append(u'{:<8.8}: {:}\n'.format(
            _('Telefone'), brw_nfce.company_id.rescon_pessoa_id.telefone))
        report.append(u'-' * 42 + '\n')
        label = _(u'Reconheco a exatidao desta NOTA PROMISSORIA que pagarei em seus vencimentos a '
                  u'%s, CNPJ: %s ou a sua ordem nos vencimentos indicados.')
        label %= (brw_nfce.company_id.name, brw_nfce.company_id.rescon_pessoa_id.cpf_cnpj)
        report.extend([w + u'\n' for w in wrap(label, 40)])
        report.append(u'-' * 42 + '\n')
        parcelas = filter(lambda f: f.nf_condicao_id.forma_pagto_id.admfpg_promissoria,
                          brw_nfce.nf_parcelas_ids)
        total = sum(map(lambda m: m.valor, parcelas))
        report.append(bold_on)
        report.append(u'{:<11.11}: {:}\n'.format(_('Valor total'), locale.currency(
            total, grouping=True)))
        report.append(bold_off + u'\n')
        report.append(u'{:}: {:<30.30}\n'.format(_('Emitente'), brw_nfce.destinatario_id.name))
        endereco = u'{:} - {:}, {:}/{:} - {:}\n'.format(
            u'{:<8.8}: {:}'.format(_('Endereco'), brw_nfce.destinatario_id.endereco.strip()),
            brw_nfce.destinatario_id.bairro.strip(), brw_nfce.destinatario_id.cidade.strip(),
            brw_nfce.destinatario_id.uf.strip(), brw_nfce.destinatario_id.cep)
        endereco = [w + u'\n' for w in wrap(endereco, 42)]
        report.extend(endereco)
        label = u'CPF' if brw_nfce.destinatario_id.pessoa_tipo == 'F' else u'CNPJ'
        report.append(u'{:<4.4}: {:}\n'.format(label, brw_nfce.destinatario_id.cpf_cnpj))
        report.append(u'{:<8.8}: {:}\n'.format(_('Telefone'), brw_nfce.destinatario_id.telefone))
        report.append(u'\n')
        report.append(u'{:<10.10} {:^12.12} {:>17.17}\n'.format(
            _('Parcela'), _('Vencimento'), _('Valor')))
        report.append(u'{:<10.10} {:^12.12} {:>17.17}\n'.format(
            u'=' * 10, u'=' * 12, u'=' * 17))
        for p in parcelas:
            report.append(u'{:<10.10} {:^12.12} {:>17.17}\n'.format(
                str(parcelas.index(p) + 1), datetime.strptime(
                    p.vencimento, '%Y-%m-%d').strftime('%x'), locale.currency(
                    p.valor, grouping=True, symbol=False)))
        report.append(u'\n')
        report.append(u'{:}, {:} de {:} de {:}.\n'.format(
            brw_nfce.company_id.rescon_pessoa_id.cidade.strip(), datetime.now().strftime('%d'),
            datetime.now().strftime('%B'), datetime.now().strftime('%Y')))
        report.append(u'\n')
        report.append(u'\n')
        report.append(u'{:^42}\n'.format('_'*35))
        report.append(u'{:^42.35}'.format(brw_nfce.destinatario_id.name.strip()))
        report.append(u'\n')
        report.append(cut_paper)  # Corta o papel
        report = ''.join([remove_accents(l) for l in report])

        return report

    @staticmethod
    def _fisnfc_get_logo_danfe_nfce(logo):
        """Converte uma imagem em logomarca na impressao do DANFE/NFC-e
        :param logo: imagem a ser convertida logomarga
        :type logo: basestring
        :return: Imagem (logo)
        :rtype: basestring
        """
        logo = base64.decodestring(logo)
        logo = StringIO.StringIO(logo)
        logo.seek(0)
        max_size = 1920, 1920
        im = Image.open(logo)
        im = im.convert('P', palette=Image.WEB).convert('RGB')
        nova_logo = StringIO.StringIO()
        im.resize(max_size, Image.BILINEAR)
        im.save(nova_logo, format='JPEG')
        nova_logo.seek(0)

        return nova_logo

    def fisnfc_button_verificar_status_nfce(self, cr, user, ids, context=None):
        """
        Verifica a situacao da nota fiscal eletronica na SEFAZ. Mostra o resultado da consulta no
        ´log´ (historico de mensagens)

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: sempre verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        ambiente = self._parametros(cr, user).fiscal.fispar_nfe_ambiente
        brw_nfe = self.browse(cr, user, ids[0], context=context)
        chave = brw_nfe.nfe_chave_acesso
        result = self.fisnfc_verificar_status_nfce(cr, user, chave, ambiente, context)
        agora = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
            cr, user, datetime.strptime(
                datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '%Y-%m-%d %H:%M:%S'), 'UTC',
            context=context), '%Y-%m-%d %H:%M:%S')
        body = 'Consulta realizada em %s<br/>' % agora
        body += '%s - %s' % (result.resposta.cStat.valor, result.resposta.xMotivo.valor)
        self.message_post(cr, user, ids, body=body, subject='Verificar NFC-e %s [%s]' % (
            chave, {1: _('Producao'), 2: _('Homologacao')}.get(result.envio.tpAmb.valor)))
        if brw_nfe.state != '3':
            if result.resposta.cStat.valor == u'100':
                # forçar faturar alterar data de emissao
                context['ctx_fiscal_nao_alterar_emissao'] = True
                self.pool.get('fiscal.nf.capa').fisnfc_faturar_nf_erp(cr, user, ids, context=context)
                vals = {
                    'nfe_protocolo': result.resposta.protNFe.infProt.nProt.valor,
                    'nfe_status': '%s - %s.' % (result.resposta.cStat.valor,
                                                result.resposta.xMotivo.valor),
                    'nfe_xml_auto': result.resposta.protNFe.infProt.xml}
                self.pool.get('fiscal.nf.capa').write(cr, user, ids, vals, context)
        return True

    def fisnfc_verificar_status_nfce(self, cr, user, chave, ambiente, context=None):
        """
        Verifica a situacao da nota fiscal eletronica na SEFAZ

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param chave: a chave da da nota fiscal (44 caracteres)
        :param ambiente: 1=Producao ou 2=Homologacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type chave: char
        :type ambiente: int
        :type context: dict
        :return: sempre verdadeiro.
        :rtype: nfce.ProcessoNFe
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        if chave[20:22] != u'65':
           return self.verifica_status_nfe_sefaz(cr, user, int(chave[35:43]), context)[0]
        estado = CODIGO_UF[int(chave[:2])]
        proc_nfce = nfc_e(parametro.fiscal.certificado_conteudo,
                          parametro.fiscal.chave_conteudo,
                          ambiente,
                          estado)
        proc_nfce.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
        proc_nfce.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
        processo = proc_nfce.process.consultar_nfce(chave)

        return processo

    def verifica_status_nfe_sefaz(self, cr, user, nf_id, context=None):
        """
        Verifica e retorna o estado da nota fiscal eletronica na SEFAZ.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf_id: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type context: dict
        :return: tupla com dados do processamento e os arquivos.
        :rtype: tuple
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        brw_nf = self.browse(cr, user, nf_id, context)
        if brw_nf.fisnfc_consumidor:
            processo =  self.fisnfc_verificar_status_nfce(
                cr, user, brw_nf.nfe_chave_acesso, parametro.fiscal.fispar_nfe_ambiente, context)
            return processo, None

        return super(fiscal_nfce, self).verifica_status_nfe_sefaz(cr, user, nf_id, context)

    def button_rascunho(self, cr, user, ids, context=None):
        """
        Retorna o estado da nota fiscal para esboco.

        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list[long]
        :type context: dict
        :return: verdadeiro se o estado foi retornado para esboco.
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, ids[0], context)
        if not brw_nf.fisnfc_consumidor:
            return super(fiscal_nfce, self).button_rascunho(cr, user, ids, context=None)

        if brw_nf.nfe_forma_emissao == '2':  # Contingencia
            raise osv.except_osv(_('Atencao!'),
                                 _('Nota em contingencia, a alteracao deve ser feita via menu "Fiscal"'))
        # Verifica o estado da nota na SEFAZ se for eletronica
        if brw_nf.nfe_chave_acesso:
            processo = self.verifica_status_nfce_sefaz(cr, user, brw_nf.id, context)
            msg = _('O estado da nota fiscal nao pode ser alterado para esboco, pois se encontra '
                    '%s na SEFAZ.')
            if processo.resposta.cStat.valor == u'100':  # Autorizada
                raise osv.except_osv(_('Atencao!'), msg % _('autorizada'))
            elif processo.resposta.cStat.valor == u'150':  # Autorizada fora do prazo
                raise osv.except_osv(_('Atencao!'), msg % _('autorizada (fora do prazo)'))
            elif processo.resposta.cStat.valor == u'101':  # Cancelada
                raise osv.except_osv(_('Atencao!'), msg % _('cancelada'))
            elif processo.resposta.cStat.valor == u'102':  # Inutilizada
                raise osv.except_osv(_('Atencao!'), msg % _('inutilizada'))
            elif processo.resposta.cStat.valor == u'999':  # Erro na comunicacao SEFAZ
                raise osv.except_osv(_('Atencao!'), msg % _('indisponivel (problemas de conexao)'))
            else:  # Provavelmente a nota nao se encontra na SEFAZ (retorno ´237´)
                pass

        context['super_write_fiscal_nf_capa'] = True
        values = {'state': '1', 'fisnfc_infcomp_fisco': ''}
        self.write(cr, user, ids, values, context=context)
        self.fisnfc_deleta_fatura_prv_financeiro(cr, user, ids, context)

        return True

    def _fisnfc_set_nfe(self, cr, user, nf, nfe, context=None):
        """
        Informacoes complementares da nota fiscal de consumidor eletronica (tag infNFeSupl):
         - qrCode: Texto com o QR-Code impresso no DANFE NFC-e.
         - urlChave: Texto com a URL de consulta por chave de acesso a ser impressa no DANFE NFC-e.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param nfe: objeto nota fiscal eletronica - NFe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type nfe: NFCe
        :type context: dict
        :return: N/A
        :rtype: N/A
        """
        # noinspection PyProtectedMember
        super(fiscal_nfce, self)._fisnfc_set_nfe(cr, user, nf, nfe, context)
        if nfe.infNFe.ide.mod.valor == 65:
            self._fisnfc_set_nfe_infnfesupl(cr, user, nf, nfe, context)

    def _fisnfc_set_nfe_infnfe_ide(self, cr, user, nf, natureza, cfop, cidade, uf, nfe,
                                   context=None):
        """
        Ajusta dados de identificacao da NFC-e
        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param natureza:
        :param cfop: dados do CFOP (Natureza da operacao)
        :param cidade: dados da cidade (emitente)
        :param uf: uf do emitente da NFC-e
        :param nfe: objeto nota fiscal eletronica - NFCe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type natureza: dict
        :type cfop: dict
        :type cidade: dict
        :type uf: dict
        :type nfe: NFCe
        :type context: dict
        :return: N/A
        :rtype: N/A
        """
        super(fiscal_nfce, self)._fisnfc_set_nfe_infnfe_ide(
            cr, user, nf, natureza, cfop, cidade, uf, nfe, context)
        if nfe.infNFe.ide.mod.valor == 65:
            nfe.infNFe.ide.indFinal.valor = u'1'  # Sempre consumidor final

    def _fisnfc_set_nfe_infnfe_dest(self, cr, user, nf, endereco, cfop, nfe, context=None):
        """
        Ajusta os dados do destinatario para NFC-e.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param endereco: dados do endereco do destinatario
        :param cfop: dados do CFOP (Natureza da operacao)
        :param nfe: objeto nota fiscal eletronica - NFCe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type endereco: dict
        :type cfop: dict
        :type nfe: NFCe
        :type context: dict
        :return: N/A
        :rtype: N/A
        """
        super(fiscal_nfce, self)._fisnfc_set_nfe_infnfe_dest(
            cr, user, nf, endereco, cfop, nfe, context)
        if nfe.infNFe.ide.mod.valor == 65:
            nfe.infNFe.dest.indIEDest.valor = '9'  # Sempre nao contribuinte
            self._consumidor_final = True

    def _fisnfc_set_nfe_infnfe_pag(self, cr, user, nf, condicoes, forma_pgto, pessoas_cartao,
                                   cond_forma, nfe, context=None):
        """
        Ajusta o detalhamento do pagamento para NFC-e.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param condicoes: condicoes de pagamento
        :param forma_pgto: formas de pagamento
        :param pessoas_cartao: Operadoras de cartao (CNPJ e nome da operadora)
        :param cond_forma: a condicao, a forma de pagamento e a finalizadora -- Heranca do Ecf?
        :param nfe: objeto nota fiscal de consumidor eletronica - NFCe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type condicoes: dict
        :type forma_pgto: dict
        :type pessoas_cartao: dict
        :type cond_forma: dict
        :type nfe: NFCe
        :type context: dict
        :return: N/A
        :rtype: N/A
        """
        super(fiscal_nfce, self)._fisnfc_set_nfe_infnfe_pag(
            cr, user, nf, condicoes, forma_pgto, pessoas_cartao, cond_forma, nfe, context)
        if nfe.infNFe.ide.mod.valor != 65:
            return

        # NFC-e nao permite tipo de pagamento ´tPag´ igual a 90
        for detpag in nfe.infNFe.pag.detPag:
            detpag.tPag.valor = u'99' if detpag.tPag.valor == u'90' else detpag.tPag.valor

    def _fisnfc_set_nfe_infnfe_cobr_fat(self, cr, user, nf, condicoes, forma_pgto, nfe,
                                        context=None):
        """
        Nota fiscal de consumidor eletronica nao apresenta dados de cobranca.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param condicoes: condicoes de pagamento
        :param forma_pgto: formas de pagamento
        :param pessoas_cartao: Operadoras de cartao (CNPJ e nome da operadora)
        :param cond_forma: a condicao, a forma de pagamento e a finalizadora -- Heranca do Ecf?
        :param nfe: objeto nota fiscal de consumidor eletronica - NFCe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type condicoes: dict
        :type forma_pgto: dict
        :type nfe: NFCe
        :type context: dict
        :return: N/A
        :rtype: N/A
        """
        context = context and context.copy() or {}
        if nfe.infNFe.ide.mod.valor != 65:  # NFC-e nao tem cobranca
            return super(fiscal_nfce, self)._fisnfc_set_nfe_infnfe_cobr_fat(
                cr, user, nf, condicoes, forma_pgto, nfe, context)

    def _fisnfc_set_nfe_infnfesupl(self, cr, user, nf, nfe, context=None):
        """
        Atribui os valores do QR-Code e da URL de consulta por chave da NFC-e.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param nfe: objeto nota fiscal eletronica - NFe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type nfe: NFe
        :type context: dict
        :return: N/A
        :rtype: N/A
        """
        # //NFe.infNFeSupl - Informacoes complementares da NFC-e
        parametro = self._parametros(cr, user)
        ambiente = parametro.fiscal.fispar_nfe_ambiente
        nfe.chave or nfe.gera_nova_chave()
        uf = CODIGO_UF[int(nfe.chave[:2])]
        versao = parametro.fiscal.fispar_nfe_versao_xml
        if parametro.fiscal.fispar_nfce_versao_qrcode == 1:
            nfe.infNFeSupl.qrCode.valor = self._qrcode(cr, user, nfe, context)
        else:
            nfe.infNFeSupl.qrCode.valor = self._qrcode_v2(cr, user, nf, nfe, context)
        nfe.infNFeSupl.urlChave.valor = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_CHAVE]['url']

    def _qrcode(self, cr, user, nfce, context=None):
        """
        Constroi texto do QR-Code da nota fiscal de consumidor.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nfce: objeto nota fiscal de consumidor eletronica - NFCe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nfce: NFCe
        :type context: dict
        :return: texto com o QR-Code a ser impresso no DANFE NFC-e.
        :rtype: str
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        ambiente = parametro.fiscal.fispar_nfe_ambiente
        nfce.chave or nfce.gera_nova_chave()
        uf = CODIGO_UF[int(nfce.chave[:2])]
        versao = parametro.fiscal.fispar_nfe_versao_xml
        url = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_QRCODE]['url'] or u''
        chave = nfce.chave
        dt_emi = nfce.infNFe.ide.dhEmi.txt
        vr_nf = nfce.infNFe.total.ICMSTot.vNF.valor
        vr_icms = nfce.infNFe.total.ICMSTot.vTotTrib.valor
        csc = parametro.fiscal.fispar_nfce_csc1 or parametro.fiscal.fispar_nfce_csc2
        csc_id = 1 if parametro.fiscal.fispar_nfce_csc1 else 2
        if ambiente == HOMOLOGACAO:
            csc = parametro.fiscal.fispar_nfce_csc_homologacao or csc
            csc_id = 1
        csc = [csc_id, csc]
        reg_id = nfce.infNFe.dest.CNPJ.valor or nfce.infNFe.dest.CPF.valor or u''

        # Calcula o DigestValue
        certificado = Certificado()
        certificado.certificado = parametro.fiscal.certificado_conteudo
        certificado.chave = parametro.fiscal.chave_conteudo
        certificado.assina_xmlnfe(nfce)
        dig_val = nfce.Signature.DigestValue

        return qr_code.qrcode(ambiente, url, chave, dig_val, csc, reg_id, dt_emi, vr_nf, vr_icms)

    def _qrcode_v2(self, cr, user, nf, nfce, context=None):
        """
        Constroi texto do QR-Code versao 2.0 da nota fiscal de consumidor.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param nfce: objeto nota fiscal eletronica - NFe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type nfce: NFCe
        :type context: dict
        :return: texto com o QR-Code a ser impresso no DANFE NFC-e.
        :rtype: str
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        ambiente = parametro.fiscal.fispar_nfe_ambiente
        nfce.chave or nfce.gera_nova_chave()
        uf = CODIGO_UF[int(nfce.chave[:2])]
        versao = parametro.fiscal.fispar_nfe_versao_xml
        url_qrcode = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_QRCODE]['url'] or u''
        csc = parametro.fiscal.fispar_nfce_csc1 or parametro.fiscal.fispar_nfce_csc2 or u''
        csc_id = 1 if parametro.fiscal.fispar_nfce_csc1 else 2
        if ambiente == HOMOLOGACAO:
            csc = parametro.fiscal.fispar_nfce_csc_homologacao or csc
            csc_id = 1
        csc = [csc_id, csc]

        # DigestValue em funcao da contingencia OFFLINE
        if nfce.chave[34] == '9':
            # Assina a NFC-e para recuperar o DigestValue
            certificado = Certificado()
            certificado.certificado = parametro.fiscal.certificado_conteudo
            certificado.chave = parametro.fiscal.chave_conteudo
            certificado.assina_xmlnfe(nfce)

            result = qr_code.qrcode_v2_offline(
                ambiente, url_qrcode, nfce.chave, nfce.Signature.DigestValue, csc,
                int(nf.get('data_emissao')[8:10]), nf.get('vr_total_nf'))
        else:
            result = qr_code.qrcode_v2_online(ambiente, url_qrcode, nfce.chave, csc)

        return result

    def default_get(self, cr, user, fields, context=None):
        """Valores padrao para nota fiscal de consumidor NFC-e.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param fields: lista com os campos da classe
        :param context: um dicionario padrao para valores contextuais
        :return: um dicionario com os valores padrao para Nota Fiscal de Consumidor Eletronica
        :type cr: object
        :type user: int
        :type fields: list
        :type context: dict
        :rtype: dict
        """
        context = context and context.copy() or {}
        result = super(fiscal_nfce, self).default_get(cr, user, fields, context)

        # Modelo da nota (Modelo 65)
        if context.get('nota_fiscal_consumidor'):
            result['modelo_nf_series_id'] = \
                self._parametros(cr, user).fiscal.fispar_nfce_modelo_series_id
            result['fisnfc_consumidor'] = True

        return result

    def gerar_nfce(self, cr, user, nfce_id, context=None):
        """
        Gera a NFC-e - Nota Fiscal de Consumidor Eletronica

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nfce_id: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nfce_id: long
        :type context: dict
        :return: o objeto NFCe()
        :rtype: NFCe
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        load = '_classic_write'
        nfce_dados = self.read(cr, user, nfce_id, [], context, load)
        nfce = NFCe()
        nfce.infNFe.Id.valor = 'NFe%s' % nfce_dados.get('nfe_chave_acesso')
        nfce.chave = nfce_dados.get('nfe_chave_acesso') or u''
        nfce.Signature.URI = nfce.infNFe.Id.valor
        nfce.infNFe.infRespTec.hashCSRT.valor = parametro.fiscal.fispar_nfe_csrt and hashlib.sha1(
            parametro.fiscal.fispar_nfe_csrt + nfce.chave).digest().encode(
            'base64').replace('\n', u'') or u''
        self._motivo_desoneracao = int(
            nfce_dados.get('natureza_id') and self.pool.get('fiscal.cfop.natureza').read(
                cr, user, nfce_dados.get('natureza_id'), ['fisnat_icms_desonerado_motivo'],
                context).get('fisnat_icms_desonerado_motivo'))
        self._fisnfc_set_nfe(cr, user, nfce_dados, nfce, context)

        return nfce

    def fisnfc_monta_chave_acesso_nfe(self, cr, user, nf, context=None):
        """
        Monta a chave de acesso da nota fiscal eletronica.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: chave de acesso da nota fiscal eletronica.
        :rtype: str
        """
        context = context and context.copy() or {}
        chave_acesso = super(fiscal_nfce, self).fisnfc_monta_chave_acesso_nfe(
            cr, user, nf, context)
        nf_dados = self.read(cr, user, nf, ['fisnfc_consumidor', 'nfe_forma_emissao'], context)
        if not nf_dados.get('fisnfc_consumidor'):
            return chave_acesso
        chave_acesso = bytearray(chave_acesso, 'utf-8')

        # Troca a forma de emissao da nota utilizada na NFC-e
        if nf_dados.get('nfe_forma_emissao') == '1':
            chave_acesso[34] = '1'
        else:
            chave_acesso[34] = '9'

        # Ajusta o digito verificador em funcao da troca
        chave_acesso = str(chave_acesso)[:-1]
        dv = calcDigito.baseModulo(chave_acesso, 98765432)
        dv = '0' if dv.__len__() > 1 else dv
        chave_acesso += dv

        return chave_acesso

    def fisnfc_processar_nota_fiscal_eletronica(self, cr, user, nfce_id, context=None):
        """
        Realiza o processamento de uma nota fiscal eletronica.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nfce_id: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user int
        :type nfce_id: long
        :type context: dict
        :return: valores ajustados da nota fiscal.
        :rtype: dict
        """
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True  # Para compatibilidade. Remover se nao usado
        brw_nfce = self.browse(cr, user, nfce_id, context)
        if not brw_nfce.fisnfc_consumidor:
            return super(fiscal_nfce, self).fisnfc_processar_nota_fiscal_eletronica(
                cr, user, nfce_id, context)
        parametro = self._parametros(cr, user)

        # Gera NFC-e
        try:
            _nfce = self.gerar_nfce(cr, user, nfce_id, context)
            _nfce.validar()  # Valida a NFC-e no esquema XML
            xml_gerado = _nfce.xml
        except Exception as erro:
            raise osv.except_osv(
                _('Atencao!'),
                _('Nao foi possivel gerar XML da NFC-e para a versao %s.\nErro: %s') % (
                    parametro.fiscal.fispar_nfe_versao_xml, str(erro)))

        estado = False
        for brw_endereco in brw_nfce.company_id.rescon_pessoa_id.endereco_ids:
            if brw_endereco.endereco_tipo_id.name == 'Comercial':
                estado = brw_endereco.uf_id.sigla
                break
            if brw_endereco.endereco_tipo_id.name == 'Normal':
                estado = brw_endereco.uf_id.sigla
                break
        if not parametro.fiscal.certificado_conteudo or \
                not parametro.fiscal.chave_conteudo:
            raise osv.except_osv(_('Atencao!'),
                                 _('Certificado nao importado no parametro do Fiscal.'))

        # Envio da NFC-e para a SEFAZ
        proc_nfce = nfc_e(
            parametro.fiscal.certificado_conteudo,
            parametro.fiscal.chave_conteudo,
            brw_nfce.modelo_nf_series_id.ambiente,
            estado,
            False)
        proc_nfce.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
        proc_nfce.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
        imprimir_danfe_nfce = bool(re.sub(
            r'\s+', '', parametro.fiscal.fispar_nfce_impressora_danfe or ''))
        values = {'nfe_observacao': ''}
        chave_acesso = _nfce.chave
        try:
            agora = datetime.now().strftime('%Y%m%d%H%M%S')
            inicio_contingencia = parametro.fiscal.fispar_nfce_contingencia_inicio or ''
            inicio_contingencia = re.sub('[^0-9]', '', inicio_contingencia)
            fim_contingencia = parametro.fiscal.fispar_nfce_contingencia_fim or ''
            fim_contingencia = re.sub('[^0-9]', '', fim_contingencia)
            if inicio_contingencia < agora < fim_contingencia and brw_nfce.state in '23' and not \
                    brw_nfce.fisnfc_nfce_contingencia_offline:
                # Provoca erro para emissao em contingencia automatica apenas para NFC-e validada
                context['contingencia_automatica'] = True
                raise Exception(_(u'** CONTINGENCIA AUTOMATICA **'))
            else:
                # Consulta pela chave da NFC-e
                processo = proc_nfce.process.consultar_nfce(_nfce.chave)
                if processo.resposta.cStat.valor in (u'100', u'150'):  # Autorizado o uso da NF-e
                    procnfce = ProcNFCe()
                    procnfce.NFe.xml = str(_nfce.xml)
                    procnfce.protNFe.xml = processo.resposta.protNFe.xml
                    values.update({'nfe_protocolo': processo.resposta.protNFe.infProt.nProt.valor,
                                   'nfe_status': '%s - %s.' % (processo.resposta.cStat.valor,
                                                               processo.resposta.xMotivo.valor),
                                   'nfe_xml_proc': procnfce.xml, 'state': '3'})
                    return values
                elif processo.resposta.cStat.valor == u'217':  # Nao consta na base
                    proc_nfce.process.nova_versao = True  # envio em duas etapas para alvar o lote.
                    processo = proc_nfce.process.processar_lote(_nfce)
                    lote = processo.resposta.infRec.nRec.valor
                    if lote:
                        values['nfe_lote'] = lote

                    if processo.resposta.cStat.valor == u'103':  # Lote recebido com sucesso
                        processo = proc_nfce.process.consultar_recibo(lote)
        except Exception as erro:
            values['nfe_status'] = '%s - %s.' % (u'999', erro)
            if 'ERROR:SCHEMA' in str(erro):
                values['nfe_observacao'] = _('Problema ocasionado por arquivo de esquema '
                                             'desatualizado ou o XML incompativel com os padroes '
                                             'estabelecidos pela SEFAZ.')
            if brw_nfce.nfe_forma_emissao == '1':
                self.fisnfc_nfce_tratar_contingencia_offline(cr, user, brw_nfce.id, values, xml_gerado, context)
            elif brw_nfce.nfe_lote:  # Consulta pelo lote
                processo = proc_nfce.process.consultar_recibo(brw_nfce.nfe_lote)
                if processo.resposta.cStat.valor in (u'100', u'150'):
                    values.update({'nfe_protocolo': processo.resposta.protNFe.infProt.nProt.valor,
                                   'nfe_status': '%s - %s.' % (processo.resposta.cStat.valor,
                                                               processo.resposta.xMotivo.valor),
                                   'nfe_xml_auto': processo.resposta.protNFe[0].xml,
                                   'state': '3'})
            else:  # Consulta pela chave da NFC-e
                processo = proc_nfce.process.consultar_nfce(_nfce.chave)
                if processo.resposta.cStat.valor in (u'100', u'150'):
                    values.update({'nfe_protocolo': processo.resposta.protNFe.infProt.nProt.valor,
                                   'nfe_status': '%s - %s.' % (processo.resposta.cStat.valor,
                                                               processo.resposta.xMotivo.valor),
                                   'state': '3'})
                else:
                    values['fisnfc_nfce_contingencia_offline'] = True
            if not context.get('contingencia_automatica') and \
                    not values.get('fisnfc_nfce_contingencia_offline'):
                values['fisnfc_nfce_contingencia_offline'] = False
        else:
            msg_erro = _('Erro de conexao com a SEFAZ. Tente novamente.')
            if processo.resposta.cStat.valor == u'104':  # Lote processado
                protocolo = processo.resposta.protNFe
                if protocolo.infProt.cStat.valor in (u'100', u'150'):  # autorizada na SEFAZ
                    values.update({'nfe_lote': processo.envio.idLote.valor,
                                   'nfe_protocolo': protocolo.infProt.nProt.valor,
                                   'nfe_status': '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                               protocolo.infProt.xMotivo.valor),
                                   'nfe_xml_auto': protocolo.xml,
                                   # 'nfe_chave_acesso': protocolo.infProt.chNFe.valor,
                                   'state': '3'})
                    values['fisnfc_nfce_contingencia_offline'] = False
                elif protocolo.infProt.cStat.valor == u'204':  # Consultar o lote
                    # 204 = Duplicidade de NF-e [nRec:999999999999999]
                    lote = re.sub('[^0-9]', '', protocolo.infProt.xMotivo.valor or '')
                    processo = proc_nfce.process.consultar_recibo(lote)
                    protocolo = processo.resposta.protNFe[0]  # Somente uma nota foi enviada
                    if not protocolo:
                        if brw_nfce.nfe_forma_emissao == '1':
                            self.fisnfc_nfce_tratar_contingencia_offline(
                                cr, user, brw_nfce.id, values, xml_gerado, context)
                    else:
                        values.update({
                            'nfe_xml_auto': protocolo.xml, 'nfe_lote': lote,
                            # 'nfe_chave_acesso': brw_nfce.nfe_chave_acesso,
                            'state': protocolo.infProt.cStat.valor in u'100-150' and '3' or 'P',
                            'nfe_protocolo': protocolo.infProt.nProt.valor,
                            'nfe_status': '%s - %s' % (protocolo.infProt.cStat.valor,
                                                       protocolo.infProt.xMotivo.valor)})
                        values['fisnfc_nfce_contingencia_offline'] = False
                elif protocolo.infProt.cStat.valor in u'110-205-301-302-303':  # Nota denegada
                    # 110 = Uso Denegado
                    # 205 = NF-e esta denegada na base de dados da SEFAZ [nRec:999999999999999]
                    # 301 = Uso Denegado: Irregularidade fiscal do emitente
                    # 302 = Rejeiçao: Irregularidade fiscal do destinatario
                    # 303 = Uso Denegado: Destinatario nao habilitado a operar na UF
                    values.update({'nfe_status': '%s - %s.' % (
                        protocolo.infProt.cStat.valor, protocolo.infProt.xMotivo.valor),
                                   'state': 'D'})
                    imprimir_danfe_nfce = False  # Nao realiza a impressao do DANFE/NFC-e
                elif protocolo.infProt.cStat.valor in '206-218':  # Inutilizada ou cancelada
                    # 206 = Rejeicao: NF-e ja esta inutilizada na Base de dados da SEFAZ
                    # 218 = NF-e ja esta cancelada na base de dados da SEFAZ [nRec:999999999999999]
                    values.update({'nfe_status': '%s - %s.' % (
                        protocolo.infProt.cStat.valor, protocolo.infProt.xMotivo.valor),
                                   'state': '2E' if brw_nfce.nfe_forma_emissao == '1' else 'P'})
                    imprimir_danfe_nfce = False  # Nao realiza a impressao do DANFE/NFC-e
                else:  # Lote processado, demais rejeicoes
                    values['nfe_status'] = '%s - %s.' % (
                        protocolo.infProt.cStat.valor, protocolo.infProt.xMotivo.valor)
                    values['nfe_lote'] = processo.envio.idLote.valor
                    if brw_nfce.nfe_forma_emissao == '1':
                        if brw_nfce.state == 'P':  # Ja existe contingencia para esta nota fiscal
                            pass  # Permanece na lista de pendentes
                        else:
                            values['state'] = '2E'
                            self.fisnfc_nfce_tratar_contingencia_offline(
                                cr, user, brw_nfce.id, values, xml_gerado, context)
                            values['state'] = '2E'  # mantem "2E" pois ela sera faturada no ERP, mas nao na sefaz
                    else:
                        values['state'] = '2E'
                        imprimir_danfe_nfce = False  # Nao realiza a impressao do DANFE/NFC-e
            else:
                values['nfe_status'] = '%s - %s.' % (
                    processo.resposta.cStat.valor or u'999',
                    processo.resposta.xMotivo.valor or msg_erro)
                if brw_nfce.nfe_forma_emissao == '1':
                    self.fisnfc_nfce_tratar_contingencia_offline(
                        cr, user, brw_nfce.id, values, xml_gerado, context)
                else:
                    values['state'] = '2E'
                    imprimir_danfe_nfce = False  # Nao realiza a impressao do DANFE/NFC-e
        finally:
            self._logger.info('>> NFC-e %s ---> %s' % (_nfce.chave, values.get('nfe_status')))
            values['nfe_xml_gerado'] = _nfce.xml
            self.write(cr, user, nfce_id, values, context)

            # Realiza ou nao a impressao automatica do DANFE/NFC-e
            if imprimir_danfe_nfce:
                if parametro.fiscal.fispar_nfce_impressora_formato == 'BO' and \
                        parametro.fiscal.fispar_nfce_impressora_marca:
                    printer = parametro.fiscal.fispar_nfce_impressora_danfe
                    danfe = tempfile.NamedTemporaryFile(mode='w+')
                    danfe.write(
                        self._fisnfc_gerar_danfe_nfce_bobina(cr, user, nfce_id, context))
                    danfe.flush()
                    subprocess.call(["lp", "-o raw", "-n1", '-d%s' % printer, danfe.name])
                    danfe.close()

                    # Promissoria
                    if brw_nfce.fisnfc_imprimi_promissoria:
                        promissoria = tempfile.NamedTemporaryFile(mode='w+')
                        promissoria.write(
                            self._fisnfc_gerar_promissoria_bobina(cr, user, nfce_id, context))
                        promissoria.flush()
                        subprocess.call(["lp", "-o raw", "-n1", '-d%s' % printer, promissoria.name])
                        promissoria.close()
                else:
                    logo = self._fisnfc_get_logo_danfe_nfce(self.pool.get('res.company').browse(
                        cr, user, brw_nfce.company_id.id, context).logo)
                    danfe_file = tempfile.NamedTemporaryFile()  # Default = 'w+b'
                    danfe_file.write(nfc_e().danfe_nfce(
                                     self.gerar_nfce(cr, user, nfce_id, context).xml, None, logo))
                    danfe_file.flush()
                    files = danfe_file.name

                    # Promissoria
                    if brw_nfce.fisnfc_imprimi_promissoria:
                        promissoria_file = tempfile.NamedTemporaryFile()
                        promissoria = report_sxw.report_sxw(
                            'report.%s' % time.strftime('%Y%m%d%H%M%S'), 'fiscal.nf.capa',
                            'addons/fiscal/report/promissoria.rml', parser=rel_fiscal_promissoria,
                            header=False)
                        promissoria = promissoria.create(
                            cr, user, [brw_nfce.id], {'report_type': 'txt'}, context)
                        promissoria_file.write(promissoria[0])
                        promissoria_file.flush()
                        files += ' ' + promissoria_file.name

                    printer = "-d" + parametro.fiscal.fispar_nfce_impressora_danfe
                    media = "-omedia='%s'" % parametro.fiscal.fispar_nfce_impressora_formato
                    try:
                        subprocess.call(["lp", "-n1 -ofit-to-page", media, printer, files])
                    except OSError:
                        osv.except_osv(_('Atencao'),
                                       _('Nao foi possivel imprimir o DANFE/NFC-e. Verifique se as '
                                         'configuracoes de impressao estao corretas em:\n Fiscal/'
                                         'Cadastros/Parametros/Parametros NF-e/NFC-e/Nome da '
                                         'impressora para o DANFE/NFC-e.'))
                    finally:
                        danfe_file.close()

        return values

    def fisnfc_nfce_tratar_contingencia_offline(self, cr, user, nfce_id, values, xml_antigo, context=None):
        """
        Tratamento de NFC-e para contingencia offline
        :return: N/A
        :rtype: N/A
        """
        context = context and context.copy() or {}
        # desfaz financeiro, expedicao, fidelidade, etc...
        cr.execute("""ROLLBACK TO SAVEPOINT fisnfe_faturar_nota;
                      COMMIT;""")
        obj_itens = self.pool.get('estoque.movimento.itens')
        # Duplica a NFC-e para tratamento de contingencia sem os itens da nota somente se nao for
        # contingencia automatica
        dados_nf = self.read(cr, user, nfce_id, [
            'modelo_nf_series_id', 'natureza_id', 'data_emissao','numero_nf', 'fisnfc_consumidor',
            'serie_nf', 'nfe_chave_acesso', 'tipo', 'fisnfc_nfe_status_configuracao_homologacao',
            'fisnfc_nfe_status_configuracao_producao', 'nfe_forma_emissao', 'finalidade_emissao',
            'nfe_versao_processo_emissao', 'nfe_forma_impressao_danfe', 'state'], context,
                             '_classic_write')

        # Altera a chave da nota para numeracao de contingencia
        chave_acesso = bytearray(dados_nf.get('nfe_chave_acesso'), 'utf-8')
        if not context.get('contingencia_automatica'):
            new_nfce_id = self.copy(cr, user, nfce_id, {}, context)
            self.write(cr, user, new_nfce_id, {
                'state': 'P',
                'fisnfc_nfce_contingencia_offline': True,
                'nfe_xml_gerado': xml_antigo,
                'nfe_chave_acesso': dados_nf.get('nfe_chave_acesso'),
                'numero_nf': dados_nf.get('numero_nf'),
                'serie_nf': dados_nf.get('serie_nf')},
                       context={'super_write_fiscal_nf_capa': True})

            # Altera o id do movimento da capa nos novos itens que ficaram com a capa antiga feito
            # na funcao 'copy' da nota
            capa_id = self.pool.get('estoque.movimento.capa').search(cr, user,
                                             [('nf_capa_id', '=', new_nfce_id)], context=context)[0]
            item_ids = obj_itens.search(cr, user, [('nf_id', '=', new_nfce_id)], context=context)
            obj_itens.write(cr, user, item_ids, {'capa_id': capa_id}, context=context)

            # Atualiza numeracao da nota e do modelo da nota para o envio em contingencia
            values.update(self.pool.get('fiscal.modelo.nf.series').fismnfs_atualiza_modelo_numero_nf(
                cr, user, dados_nf.get('modelo_nf_series_id'), context))

            chave_acesso[22:25] = str(values.get('serie_nf')).zfill(3)
            chave_acesso[25:34] = str(values.get('numero_nf')).zfill(9)
        chave_acesso[34] = '9'  # Contingencia - Forma de emissao = ´2´
        chave_acesso = str(chave_acesso)[:-1]
        dv = calcDigito.baseModulo(chave_acesso, 98765432)
        dv = '0' if dv.__len__() > 1 else dv
        chave_acesso += dv  # Ajusta o digito verificador da chave

        # NFC-e em contingencia sempre sera faturada no sistema...
        values['nfe_chave_acesso'] = chave_acesso  # Atribui a nova chave de acesso
        values['nfe_forma_emissao'] = '2'  # Muda para emissao em contingencia (na chave = '9')
        values['fisnfc_nfce_contingencia_offline'] = True  # ...e aguardarao envio para a SEFAZ
        self.write(cr, user, nfce_id, values, context)
        # as NFCe's em contingencia agora serao faturadas no ERP para dar baixa nos estoque, financeiro, etc...
        values.update(self.fisnfc_faturar_nf_erp(cr, user, [nfce_id], context))

    def fisnfc_faturar_nfce_contingencia_offline(self, cr, user, ids=None, context=None):
        """
        [CRON]
        Fatura as NFC-e que foram validadas, enviadas aa SEFAZ mas que retornaram com rejeicao ou
        por motivos tecnicos nao puderam ser enviadas e jah foram emitidas.
        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das notas fiscais
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :rtype: bool
        :return: verdadeiro se sucesso
        """
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True

        user_ids = self.pool.get('res.users').search(
            cr, user, [('login', '=', 'cron_nfce_painel')]) or [user]
        user_id = user_ids and user_ids[0]

        # Todas as empresas
        for company_id in self.pool.get('res.company').search(cr, user, []):
            # log
            self._logger.info(
                '>> [Fiscal v%s::python-nfe %s]Empresa %d' % (
                    info('fiscal').get('version'), __version__, company_id))

            # atribui o usuario do CRON na execucao
            self.pool.get('res.users').write(cr, user, user_ids, {
                'company_id': company_id})
            self._fisnfc_faturar_nfce_contingencia_offline(
                cr, user_id, None, company_id, context)

        return True

    def _fisnfc_faturar_nfce_contingencia_offline(self, cr, user, ids, company_id, context):
        parametro = self._parametros(cr, user)
        context.update({'sig_force_company': True})

        # NFC-e pendentes (forma de emissao em contingencia)
        ids = ids or self.search(
            cr, user, [('fisnfc_consumidor', '=', True),
                       ('fisnfc_nfce_contingencia_offline', '=', True),
                       ('company_id', '=', company_id),
                       ('state', 'in', ['3', 'P'])], context=context)
        ids = ids if isinstance(ids, (list, tuple)) else [ids]
        for nf_id in sorted(ids):
            values = {}
            dados_nf = self.read(
                cr, user, nf_id, ['numero_nf', 'serie_nf', 'nfe_chave_acesso', 'nfe_protocolo',
                                  'state', 'nfe_forma_emissao', 'nfe_status'], context,
                '_classic_write')
            status = dados_nf.get('nfe_status')
            if status and (status.startswith('501') or status.startswith('100')):
                if status.startswith('100'):
                    self.write(cr, user, nf_id, {'fisnfc_nfce_contingencia_offline': False})
                continue

            ambiente = parametro.fiscal.fispar_nfe_ambiente
            try:
                processo = self.fisnfc_verificar_status_nfce(cr, user, dados_nf.get('nfe_chave_acesso'), ambiente)
                # se houver problema na sefaz nao tenta fazer as outras notas, espera a proxima iteracao do cron
                if processo.resposta.cStat.valor == u'999':
                    values['nfe_status'] = '%s - %s' % (
                                        processo.resposta.retEvento.infEvento.cStat.valor,
                                        processo.resposta.retEvento.infEvento.xMotivo.valor)
                    self.write(cr, user, nf_id, values, context)
                    break

                if processo.resposta.cStat.valor in (u'100', u'101', u'150'):
                    nr_protocolo = processo.resposta.protNFe.infProt.nProt.valor
                    processo.resposta.protNFe.Signature.URI = ''  # tira a assinatura do xml de autorizacao
                    xml_auto = processo.resposta.protNFe.infProt.xml
                    values.update({'nfe_xml_auto': xml_auto,
                                   'nfe_protocolo': nr_protocolo})

                if dados_nf.get('nfe_forma_emissao') == '1':  # Deve ser cancelada/inutilizada
                    evento = 'Retorno da SEFAZ para consulta da NF por Chave de Acesso'
                    if processo.resposta.cStat.valor == u'999':
                        continue
                    if processo.resposta.cStat.valor == u'217':  # nao encontrada sera inutilizada
                        self.fisnfc_inutilizar_nota_fiscal(
                            cr, user, nf_id, _('Nota fiscal inutilizada por substituicao de NFC-e '
                                               'em contingencia offline.'), context)
                        continue
                    if processo.resposta.cStat.valor == u'562': # NF existe com chave diferente
                        # 562 = Codigo numerico informado difere do codigo numerico da chave

                        # ajusta para a chave correta
                        pos_chave = processo.resposta.xMotivo.valor.find('NF-e') + 4
                        nova_chave = processo.resposta.xMotivo.valor[pos_chave:]
                        if nova_chave[25:34] == dados_nf.get('numero_nf') and \
                            nova_chave[22:25] == dados_nf.get('serie_nf'):
                            values['nfe_chave_acesso'] = nova_chave

                            # refaz a consulta com chave correta
                            processo = self.fisnfc_verificar_status_nfce(
                                cr, user, values['nfe_chave_acesso'], ambiente)

                            if processo.resposta.cStat.valor == u'999':
                                continue

                    if processo.resposta.cStat.valor == u'101':  # Ja se encontra cancelada
                        # 101 = Cancelamento de NF-e homologado
                        protocolo = processo.resposta.protNFe.infProt.nProt.valor
                        values['nfe_xml_cancelamento'] = self.fisnit_gerar_xml_cancelamento(cr, user, processo,
                                                    dados_nf.get('nfe_chave_acesso'), parametro.fiscal.__dict__,
                                                                                            protocolo, context)
                        values['nfe_protocolo_cancelamento'] = \
                            processo.resposta.retCancNFe.infCanc.nProt.valor
                        values['state'] = 'C'
                        values['nfe_status'] = '%s - %s' % (
                            processo.resposta.retCancNFe.infCanc.cStat.valor,
                            processo.resposta.retCancNFe.infCanc.xMotivo.valor)
                        self.fisnfc_funcao_button_cancelar(cr, user, [nf_id], context)
                    elif processo.resposta.cStat.valor == u'501':  # Habilita faturamento da NFC-e
                        # 501 = Prazo de cancelamento superior ao previsto na legislacao
                        pass  # Habilita o botao de faturar / aguardar cancelamento extemporaneo
                    elif processo.resposta.cStat.valor in (u'100', u'150'):  # Sera cancelada
                        # 100 = Autorizado o uso da NF-e
                        # 150 = Autorizado o uso da NF-e, autorizacao fora de prazo
                        proc_nfce = nfc_e(
                            parametro.fiscal.certificado_conteudo,  parametro.fiscal.chave_conteudo,
                            ambiente, CODIGO_UF[int(dados_nf.get('nfe_chave_acesso')[:2])])
                        proc_nfce.process.tentativas = \
                            parametro.fiscal.fispar_nfce_tentativa_envio or 5
                        proc_nfce.process.timeout = \
                            parametro.fiscal.fispar_nfce_conexao_timeout or 10
                        justificativa = _('Nota fiscal cancelada por substituicao de NFC-e em '
                                          'contingencia offline.')
                        chave_acesso = processo.resposta.protNFe.infProt.chNFe.valor
                        nr_protocolo = processo.resposta.protNFe.infProt.nProt.valor
                        processo = proc_nfce.process.cancelar_nfce(
                            chave_acesso, nr_protocolo, justificativa)
                        if processo.resposta.cStat.valor == u'999':
                            continue
                        if processo.resposta.cStat.valor == u'128':
                            if processo.resposta.retEvento.infEvento.cStat.valor in u'135-151-155':
                                # 135 = Evento registrado e vinculado a NF-e
                                # 151 = Cancelamento de NF-e homologado fora de prazo
                                # 155 = Cancelamento homologado fora de prazo (isso mesmo: de novo!)
                                values.update({'nfe_xml_cancelamento': processo.processo_cancelamento_nfe.xml,
                                               'nfe_protocolo_cancelamento': processo.resposta.retEvento.infEvento.nProt.valor,
                                               'state': 'C'})
                                self.fisnfc_funcao_button_cancelar(cr, user, [nf_id], context)
                            elif processo.resposta.retEvento.infEvento.cStat.valor == u'501':
                                # 501 - Prazo de cancelamento superior ao previsto na Legislacao
                                # O Cancelamento deve ser realizado por substituicao em ate 7 dias.
                                chave_ref = bytearray(chave_acesso, 'utf-8')

                                # Troca o numero e a forma de emissao da NFC-e e ajusta o digito
                                # verificador em funcao das trocas
                                chave_ref[34] = '9'  #
                                chave_ref[25:34] = str(int(chave_ref[25:34]) + 1).zfill(9)
                                chave_ref = str(chave_ref)[:-1]
                                dv = calcDigito.baseModulo(chave_ref, 98765432)
                                dv = '0' if dv.__len__() > 1 else dv
                                chave_ref += dv

                                # consulta da chave de referencia
                                processo = self.fisnfc_verificar_status_nfce(cr, user, chave_ref, ambiente)
                                if processo.resposta.cStat.valor == u'999':
                                    continue
                                if processo.resposta.cStat.valor == u'562':  # NF existe com chave diferente
                                    # 562 = Codigo numerico informado difere do codigo numerico da chave

                                    # ajusta para a chave correta
                                    pos_chave = processo.resposta.xMotivo.valor.find('NF-e') + 4
                                    chave_ref = processo.resposta.xMotivo.valor[pos_chave:]

                                processo = proc_nfce.process.cancelar_nfce_substituicao(chave_acesso, nr_protocolo,
                                                                                        justificativa, chave_ref)
                                if processo.resposta.cStat.valor == u'999':
                                    continue

                                # Efetiva o cancelamento por substituicao
                                if processo.resposta.retEvento.infEvento.cStat.valor in (u'135', u'151', u'155'):
                                    # 135 = Evento registrado e vinculado a NF-e
                                    # 151 = Cancelamento de NF-e homologado fora de prazo
                                    # 155 = Cancelamento homologado fora de prazo (de novo!)
                                    values['nfe_xml_cancelamento'] = processo.processo_cancelamento_nfe.xml
                                    values['nfe_protocolo_cancelamento'] = processo.resposta.retEvento.infEvento.nProt.valor
                                    values['state'] = 'C'
                                    self.fisnfc_funcao_button_cancelar(cr, user, [nf_id], context)
                            values['nfe_status'] = '%s - %s' % (
                                processo.resposta.retEvento.infEvento.cStat.valor,
                                processo.resposta.retEvento.infEvento.xMotivo.valor)
                        else:  # Pode ter havido problema de comunicacao com a SEFAZ
                            self._logger.warn(
                                '<< [Fiscal %s::python-nfe %s]NFC-e %s (Empresa %d): %s\n%s' % (
                                    info('fiscal').get('version'), __version__,
                                    dados_nf.get('nfe_chave_acesso'), parametro.fiscal.company_id,
                                    evento, processo.resposta.xml))
                            continue  # Nao atualiza ´´status´´ da nota
                    elif processo.resposta.cStat.valor == u'999':
                        pass
                    else:
                        values['state'] = '2E'
                else:
                    if processo.resposta.cStat.valor in (u'100', u'150'):
                        values['fisnfc_nfce_contingencia_offline'] = False  # remove do painel
                    elif processo.resposta.cStat.valor == u'217':  # Não consta na base
                        values.update(self.fisnfc_processar_nota_fiscal_eletronica(
                            cr, user, nf_id, context))
                    else:
                        values['nfe_status'] = '%s - %s' % (processo.resposta.retEvento.infEvento.cStat.valor,
                                                            processo.resposta.retEvento.infEvento.xMotivo.valor)
            except Exception as e:
                self._logger.info(
                    '>> [Fiscal v%s::python-nfe %s]NFC-e Id %d@%s (Empresa %d): %s' % (
                        info('fiscal').get('version'), __version__, dados_nf.get('id'),
                        dados_nf.get('nfe_chave_acesso'), parametro.fiscal.company_id, e))
            self.write(cr, user, nf_id, values, context)

        return True

    def verifica_status_nfce_sefaz(self, cr, user, nf_id, context=None):
        """
        Verifica e retorna o estado da NFC-e na SEFAZ.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf_id: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type context: dict
        :return: dados do processamento com o estado da nota fiscal
        :rtype: nfce.process
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        brw_nf = self.browse(cr, user, nf_id, context)
        estado = brw_nf.nfe_chave_acesso and CODIGO_UF[int(brw_nf.nfe_chave_acesso[:2])]
        try:
            proc_nfce = nfc_e(
                parametro.fiscal.certificado_conteudo,  parametro.fiscal.chave_conteudo,
                brw_nf.modelo_nf_series_id.ambiente, estado,
                brw_nf.modelo_nf_series_id.tipo_contingencia)
            proc_nfce.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
            proc_nfce.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
            processo = proc_nfce.process.consultar_nfce(brw_nf.nfe_chave_acesso)
        except socket.gaierror:
            raise osv.except_osv(_('Atencao!'),
                                 _('Falha ao comunicar com o servidor da SEFAZ. Verifique se '
                                   'existe conexao com a Internet.'))
        except Exception as erro:
            raise osv.except_osv(_('Erro!'), erro.message) or erro
        except IOError:
            raise osv.except_osv(_('Erro!'), _('Falha ao comunicar com o servidor da SEFAZ. Tente '
                                               'novamente mais tarde.'))
        return processo

    def button_cancelar(self, cr, user, ids, context=None):
        """
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a nota fiscal foi cancelada
        :rtype: bool
        """
        context = context and context.copy() or {}
        super(fiscal_nfce, self).button_cancelar(cr, user, ids, context)
        dados_nf = self.read(
            cr, user, ids[0], ['fisnfc_modelo_serie', 'fisnfc_consumidor', 'state', 'nf_propria',
                               'nfe_chave_acesso'], context, '_classic_write')
        if not dados_nf.get('nf_propria') or \
                not dados_nf.get('nfe_chave_acesso') or \
                not dados_nf.get('fisnfc_consumidor'):
            return True
        ambiente = self._parametros(cr, user).fiscal.fispar_nfe_ambiente

        # Cancelamento de tesouraria modelo 2D
        if dados_nf.get('fisnfc_consumidor'):
            if dados_nf.get('state') == '3':  # Somente NF faturada realizou movimento financeiro
                self.fisnfc_estorna_movimento_tesouraria_mod2D(cr, user, dados_nf, context)

        # Realiza a inutilizacao da nota fiscal se nao houver correspondencia na SEFAZ
        processo = self.fisnfc_verificar_status_nfce(
            cr, user, dados_nf.get('nfe_chave_acesso'), ambiente)
        if processo.resposta.cStat.valor == u'217':  # Nota nao encontrada deve ser inutilizada
            self.fisnfc_inutilizar_nota_fiscal(
                cr, user, ids[0], 'Nota fiscal inutilizada por desistencia de compra.', context)
        elif processo.resposta.cStat.valor in (u'100', u'150'):  # autorizada deve ser faturada
            raise osv.except_osv(_('Erro!'),
                                 _('A nota fiscal nao pode ser cancelada pois, se encontra faturada'
                                   'na SEFAZ. A nota deve ser faturada primeiro.'))
        elif processo.resposta.cStat.valor == u'101':  # Nota cancelada na SEFAZ. OK!
            return True
        else:  # Outros problemas
            if not processo.resposta.cStat.valor:
                raise osv.except_osv(_('Atencao!'),
                                     _('Falha no servidor da SEFAZ. Tente novamente mais tarde.'))
            else:
                raise osv.except_osv(_('Atencao!.'),
                                     'Nao foi possivel cancelar a nota fiscal.\n%s - %s' % (
                                         processo.resposta.cStat.valor,
                                         processo.resposta.xMotivo.valor))

        return True

    def fisnfc_inutilizar_nota_fiscal(self, cr, user, nf, motivo, context=None):
        """
        Inutiliza NFC-e.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param nf: o ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type nf: long
        :return: verdadeiro se a nota fiscal foi inutilizada
        :rtype: bool
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)  # Parametros do sistema
        motivo = motivo and motivo.strip() or _('Nota fiscal inutilizada por desistencia de compra')
        if len(motivo) < 15:
            raise osv.except_osv(_('Atencao!'),
                                 _('A justificativa deve ter no minimo 15 caracteres.'))
        context['inutilizacao'] = True
        company_id = self.pool.get('res.users').browse(
            cr, user, user, context=context).company_id.id
        pessoa_id = self.pool.get('res.company').read(
            cr, user, company_id, ['rescon_pessoa_id'], context=context)['rescon_pessoa_id'][0]
        enderecos = self.pool.get('adm.pessoa').admpes_endereco_fiscal(cr, user, pessoa_id, context)
        endereco = enderecos.get(pessoa_id)
        if not endereco.get('uf_sigla'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao foi possivel inutilizar a nota fiscal. Cadastre um '
                                   'endereco valido para a empresa.'))
        cnpj = pessoa_id and self.pool.get('adm.pessoa').read(
            cr, user, pessoa_id, ['cpf_cnpj'], context)['cpf_cnpj'] or ''
        if not cnpj:
            raise osv.except_osv(_('Atencao!'), _('Nao foi possivel inutilizar a nota fiscal. Nao '
                                                  'existe CNPJ cadastro para a empresa emitente.'))
        cnpj = re.sub('[^0-9]', '', cnpj)
        brw_nf = self.browse(cr, user, nf, context)
        ano = datetime.now().strftime(u'%y')
        try:
            if brw_nf.fisnfc_consumidor:
                proc_nfce = nfc_e(
                    parametro.fiscal.certificado_conteudo,  parametro.fiscal.chave_conteudo,
                    brw_nf.modelo_nf_series_id.ambiente, endereco.get('uf_sigla'), False)
                proc_nfce.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
                proc_nfce.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
                processo = proc_nfce.process.inutilizar_nfce(brw_nf.nfe_chave_acesso, motivo)
            else:
                from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400
                nfe = nfe_400
                nfe = nfe(certify=parametro.fiscal.certificado_conteudo,
                          key=parametro.fiscal.chave_conteudo,
                          environment=parametro.fiscal.fispar_nfe_ambiente,
                          state=endereco.get('uf_sigla'),
                          contingency=parametro.fiscal.fispar_nfe_contingencia)
                nfe.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
                nfe.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
                processo = nfe.inutilizar_faixa_numeracao(
                    ano, cnpj, int(brw_nf.serie_nf), brw_nf.numero_nf, brw_nf.numero_nf, motivo)
        except Exception as err:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro de conexao com servidor. \n%s') % err)

        values = {'nfe_status': '%s - %s' % (processo.resposta.infInut.cStat.valor,
                                             processo.resposta.infInut.xMotivo.valor),
                  'nfe_observacao': ''}
        if processo.resposta.infInut.cStat.valor in u'102-206-563':  # Efetiva a inutilizacao
            # 102 = Inutilizacao de numero homologado
            # 206 = NF-e ja esta inutilizada na base de dados da SEFAZ
            # 563 = Ja existe pedido de inutilizacao com a mesma faixa de inutilizacao

            # Elimina o movimento da NFC-e
            movimento_itens = [[2, item.id, False] for item in brw_nf.estoque_movimento_itens_ids]
            values.update({
                'nfe_protocolo': re.sub('[^0-9]', '', processo.resposta.infInut.xMotivo.valor),
                'state': 'I', 'estoque_movimento_itens_ids': movimento_itens,
                'fisnfc_nfce_contingencia_offline': False})
        elif processo.resposta.infInut.cStat.valor == u'241':
            # 241 = Um numero da faixa ja foi utilizado (nota autorizada, cancelada ou denegada)
            if brw_nf.modelo_55 or (brw_nf.fisnfc_consumidor and brw_nf.nfe_forma_emissao != '1'):
                values.update({'fisnfc_nfce_contingencia_offline': False, 'state': '2E',
                               'nfe_observacao': _('Nao pode-se inutilizar numeracoes de notas '
                                                   'quando ja foram autorizadas, canceladas ou '
                                                   'denegadas. Verifique a sequencia de numeracao '
                                                   'de notas.')})
        else:  # Nao foi possivel inutilizar a nota fiscal
            values.update({'fisnfc_nfce_contingencia_offline': False})

        return self.write(cr, user, nf, values, context=context)

    def enviar_email(self, cr, user, ids, context=None):
        """
        Envia email com XML e DANFE da nota fiscal de consumidor eletronica.
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com o ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se o email foi enviado
        :rtype: bool
        """
        from base import BODY_HTML_NFCE
        import locale

        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, ids[0] if isinstance(ids, list) else ids, context)
        mail_server_id = brw_nf.company_id.servidor_smtp_id.id or None
        if not brw_nf.fisnfc_consumidor:
            return super(fiscal_nfce, self).enviar_email(cr, user, ids, context)
        emails = ''

        for x in brw_nf.destinatario_id.email_ids:
            emails += x.name + ','
        emails = emails[:-1]
        locale.setlocale(locale.LC_ALL, 'pt_BR.utf8')
        v_data = datetime.strptime(brw_nf.data_emissao[:10], "%Y-%m-%d")
        v_data = v_data.strftime("%d/%m/%Y")
        brw_user = self.pool.get('res.users').browse(cr, user, user, context)
        if not brw_user.email:
            raise osv.except_osv(_('Email requerido'),
                                 _('O usuario atual deve ter um endereco de e-mail configuradas em '
                                   '"Preferencias do usuario" para ser capaz de enviar e-mails.'))
        elif not emails:
            raise osv.except_osv(_('Email requerido'),
                                 _('O destinatario da nota fiscal deve possuir um e-mail '
                                   'cadastrado.'))
        subject = 'NFC-e %s' % brw_nf.nfe_chave_acesso
        body = BODY_HTML_NFCE % (
            brw_nf.destinatario_id.name, brw_nf.numero_nf, brw_nf.serie_nf, brw_nf.nfe_chave_acesso,
            brw_nf.numero_nf, brw_nf.serie_nf, locale.currency(brw_nf.vr_total_nf, grouping=True),
            v_data, brw_nf.company_id.name)
        mail_id = self.pool.get('mail.mail').create(
            cr, user, {'email_from': brw_user.email, 'email_to': emails, 'subject': subject,
                       'body_html': body, 'mail_server_id':mail_server_id}, context=context)
        attachment_ids = []
        datas = [('Arquivo XML', base64.encodestring(brw_nf.nfe_xml_proc), 'xml'),
                 ('DANFE', self.exportar_xml_danfe(
                     cr, user, ids, context={'gerar_danfe_pdf': True}, show_wizard=False), 'pdf')]
        for data in datas:
            file_name, file_data, file_ext = data
            attachment_data = {
                'name': file_name,
                'datas_fname': '%s.%s' % (brw_nf.nfe_chave_acesso, file_ext),
                'datas': file_data,
                'res_model': self.pool.get('mail.mail')._name,
                'res_id': mail_id}
            attachment_ids.append(self.pool.get('ir.attachment').create(
                cr, user, attachment_data, context=context))
        if attachment_ids:
            self.pool.get('mail.mail').write(
                cr, user, mail_id, {'attachment_ids': [(6, 0, attachment_ids)]}, context=context)
        self.message_post(
            cr, user, ids, subject=subject, body='E-mail criado com sucesso.', context=context)

        return True

    def fields_view_get(self, cr, user, view_id=None, view_type=False, context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        res = super(fiscal_nfce, self).fields_view_get(
            cr, user, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,
            submenu=submenu)
        if not context.get('nota_fiscal_consumidor') and 'toolbar' in res:
            res['toolbar']['action'] = [a for a in res['toolbar']['action'] if
                                        a['name'] != 'Gerar NF-e']

        return res

    def fisnfc_gerar_nfe_nfce(self, cr, user, ids, context=None):
        """
        Gera nota fiscal de notas fiscais de consumidor eletronica.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das notas fiscais
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: o ID da NF-e que foi criada.
        :rtype: long
        """
        context = context and context.copy() or {}
        context['nota_fiscal_consumidor'] = True
        context['nota_fiscal_saida'] = True
        context['proprio'] = True
        context['ctx_nf_propria'] = True
        parametro = self._parametros(cr, user)  # Parametros do sistema
        brw_nfce = self.browse(cr, user, ids, context)
        clientes = set([nfce.destinatario_id.id for nfce in brw_nfce if nfce.destinatario_id.id])
        if len(clientes) > 1:
            raise osv.except_osv(_('Atencao!'),
                                 _('Na criacao de NF-e, somente sao aceitas NFC-e de um mesmo '
                                   'cliente ou sem identificacao do cliente.'))
        cliente_id = clientes and clientes.pop() or False
        # notas denegadas, canceladas e de entrada nao sao consideradas
        if filter(lambda f: f.state != '3', brw_nfce):
            raise osv.except_osv(_('Atencao!'),
                                 _('Na criacao de NF-e, e permitida apenas NFC-e fatudara'))
        cr.execute("""
SELECT emi.produto_id,
       emi.produto_embalagem_id,
       emi.grade_id,
       emi.local_id,
       SUM(emi.quantidade / COALESCE(ee.quantidade_na_embalagem, 1)) AS quantidade,
       SUM(COALESCE(emi.vr_desconto, 0))        AS vr_desconto,
       ROUND(AVG(emi.vr_unitario * COALESCE(ee.quantidade_na_embalagem, 1))::NUMERIC, 6)  AS vr_unitario,
       SUM(COALESCE(emi.vr_acrescimos, 0))      AS vr_acrescimos,
       SUM(emi.vr_total_item)                   AS vr_total_item,
       SUM(emi.vr_frete_rateado)                AS vr_frete_rateado,
       SUM(emi.vr_total_item_rateado)           AS vr_total_item_rateado
FROM   estoque_movimento_itens              AS emi
INNER JOIN estoque_produto_embalagem        AS epe ON epe.id = emi.produto_embalagem_id
INNER JOIN estoque_embalagem                AS ee  ON ee.id = epe.embalagem_id
WHERE  emi.id IN ({item_ids})
GROUP  BY emi.produto_id, emi.produto_embalagem_id, emi.grade_id, emi.local_id, emi.vr_unitario""".format(**dict(item_ids=', '.join(
            [item for itens in map(lambda m: map(
                lambda m: str(m.id), m.estoque_movimento_itens_ids), brw_nfce)
             for item in itens]))))
        itens = [[0, False, i] for i in cr.dictfetchall()]
        vr_total_nf, vr_desconto, vr_seguro, vr_despesas_acessorias, vr_frete = [
            sum(s) for s in zip(*[[nf.vr_total_nf, nf.vr_desconto, nf.vr_seguro,
                                   nf.vr_despesas_acessorias, nf.vr_frete] for nf in brw_nfce])]
        hoje = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
            cr, user, datetime.now(), 'UTC', context=context), '%Y-%m-%d')

        # A condicao deve obedecer o parametro do fiscal para notas de cupom. Se nao existir a
        # condicao padrao sera "dinheiro" - a vista.
        avista_ids = self.pool.get('adm.forma.pgto').search(
            cr, user, [('admfpg_tipo', '=', '01')], context=context)
        dinheiro_ids = avista_ids and self.pool.get('adm.condicao.forma').search(
            cr, user, [('forma_pagamento_id', '=', avista_ids[0])], context=context)
        dinheiro_id = dinheiro_ids and dinheiro_ids[0]
        condicao_id = parametro.fiscal.cupom_nota_condicao_id or dinheiro_id

        condicoes = [[0, False, dict(forma_pagto_id=c.forma_pagamento_id.id, condicao_forma_id=c.id,
                                     valor=vr_total_nf, nf_parcela_ids=[[
                0, False, p] for p in self.pool.get('adm.condicao').gerar_parcelas(
                cr, user, c.adm_condicao_id.id, vr_total_nf, hoje)])]
                     for c in condicao_id and self.pool.get('adm.condicao.forma').browse(
                cr, user, [condicao_id], context) or []]
        context['estmvi_nao_calcula_vr_total'] = True
        context['fisnfc_nfe_de_nfce'] = True
        nf_id = self.pool.get('fiscal.utils').gerar_nf(
            cr, user, company_id=brw_nfce[0].company_id.id, destinatario_id=cliente_id,
            natureza_id=parametro.fiscal.cupom_nota_natureza_id, tipo='S', vr_desconto=vr_desconto,
            vr_seguro=vr_seguro, vr_despesas_acessorias=vr_despesas_acessorias, vr_frete=vr_frete,
            tipo_frete=vr_frete and 'D' or 'F', nf_condicao_ids=condicoes,
            estoque_movimento_itens_ids=itens, context=context)
        inf = ['Notas fiscais referenciadas: ']
        for i in brw_nfce:
            inf.append('Numero: %s. Serie: %s. Emissao: %s. ' % (i.numero_nf, i.serie_nf, i.data_emissao))

        self.write(cr, user, nf_id, {'fisnfc_infcomp_contribuinte': '\n'.join(inf)}, context)
        # Referenciar as NFC-e
        cr.execute("""INSERT INTO fiscal_nf_referencia
                                  (id, nf_ref_id)
                           VALUES %s
                   """ % ',\n               '.join(map(lambda x: '(%s, %s)' % (nf_id, x), ids)))

        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, user, 'fiscal', 'view_fiscal_nf_capa_form')
        view_id = reference and reference[1] or False
        return {'name': _('NF-e'),
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view_id],
                'res_model': 'fiscal.nf.capa',
                'res_id': nf_id,
                'type': 'ir.actions.act_window'}

    def fisnfc_ajustar_protocolo_autorizacao_nfce(self, cr, user, nf_id, parametro, context=None):
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, nf_id, context)
        company_id = parametro.get('company_id')
        pessoa_id = self.pool.get('res.company').read(
            cr, user, company_id, ['rescon_pessoa_id'], context=context)['rescon_pessoa_id'][0]
        enderecos = self.pool.get('adm.pessoa').admpes_endereco_fiscal(cr, user, pessoa_id, context)
        endereco = enderecos.get(pessoa_id)
        try:
            nf = nfc_e(parametro.get('certificado_conteudo'),
                       parametro.get('chave_conteudo'),
                       brw_nf.modelo_nf_series_id.ambiente,
                       endereco.get('uf_sigla'),
                       False)
            nf.process.tentativas = parametro.get('fispar_nfce_tentativa_envio') or 5
            nf.process.timeout = parametro.get('fispar_nfce_conexao_timeout') or 10
            if brw_nf.nfe_lote and brw_nf.state == '3':
                processo = nf.process.consultar_recibo(brw_nf.nfe_lote)
                try:
                    numero = int(processo.resposta.protNFe[0].infProt.chNFe.valor[25:34])
                except:
                    numero = False

                if processo.resposta.cStat.valor != u'104' or numero != brw_nf.numero_nf:  # Lote nao encontrado
                    processo, __ = nf.consultar_nota(brw_nf.nfe_chave_acesso)
                    if processo.resposta.cStat.valor in (u'100', u'150'):
                        self.write(
                            cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe.xml},
                            context)
                else:  # Lote processado
                    self.write(
                        cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe[0].xml},
                        context)
            elif brw_nf.nfe_chave_acesso:
                processo, __ = nf.process.consultar_nota(brw_nf.nfe_chave_acesso)
                if processo.resposta.cStat.valor in (u'100', u'150'):
                    self.write(cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe.xml}, context)
                elif processo.resposta.cStat.valor == u'101':
                    protocolo = processo.resposta.protNFe.infProt.nProt.valor
                    xml_canc = self.fisnit_gerar_xml_cancelamento(cr, user, processo, brw_nf.nfe_chave_acesso,
                                                                  parametro, protocolo, context)
                    prot_cancelamento = processo.resposta.retCancNFe.infCanc.nProt.valor
                    processo.resposta.protNFe.Signature.URI = ''  # tira a assinatura do xml de autorizacao
                    self.write(cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe.xml,
                                                 'nfe_xml_cancelamento': xml_canc,
                                                 'state': 'C',
                                                 'nfe_status': '%s - %s' % (processo.resposta.retCancNFe.infCanc.cStat.valor,
                                                                            processo.resposta.retCancNFe.infCanc.xMotivo.valor),
                                                 'nfe_protocolo_cancelamento': prot_cancelamento}, context)
                elif processo.resposta.cStat.valor in (u'137', u'217') and brw_nf.state != '3':
                    self.write(cr, user, nf_id, {'state': 'I',
                                                 'nfe_status': '102 - Inutilizacao de numero homologado'}, context)
        except Exception as err:
            self._logger.error('CRON - Ajusta XML %s >> %s' % (brw_nf.nfe_chave_acesso, err))

    def fisnit_gerar_xml_cancelamento(self, cr, user, consulta, nfe_chave_acesso, parametros, protocolo, context=None):
        """Refaz o xml de cancelamento"""
        context = context and context.copy() or {}
        ambiente = parametros.get('fispar_nfe_ambiente') or 2
        company_id = parametros.get('company_id')
        pessoa_id = self.pool.get('res.company').read(
            cr, user, company_id, ['rescon_pessoa_id'], context=context)['rescon_pessoa_id'][0]
        enderecos = self.pool.get('adm.pessoa').admpes_endereco_fiscal(cr, user, pessoa_id, context)
        endereco = enderecos.get(pessoa_id)
        proc_nfce = nfc_e(parametros.get('certificado_conteudo'),
                          parametros.get('chave_conteudo'),
                          ambiente,
                          endereco.get('uf_sigla'))
        chave = nfe_chave_acesso
        envio = EnvEvento()
        resposta = RetEnvEvento()
        processo = ProcessoNFe(2, envio, resposta)
        envio.infEvento.tpAmb.valor = ambiente
        envio.infEvento.chNFe.valor = chave
        envio.infEvento.nProt.valor = protocolo
        envio.infEvento.xJust.valor = 'Nota fiscal emitida em contingencia'
        envio.infEvento.cOrgao.valor = UF_CODIGO[endereco.get('uf_sigla')]
        dt = datetime.utcnow()
        dt = dt.replace(tzinfo=pytz.utc)
        data_hora_tz = datetime.astimezone(dt, pytz.timezone("Brazil/East"))
        data_evento = datetime.strftime(data_hora_tz, u'%Y-%m-%dT%H:%M:%S') + str(data_hora_tz)[-6:]
        envio.infEvento.dhEvento.valor = data_evento
        envio.infEvento.CNPJ.valor = chave[6:20]
        proc_nfce.process.certificado.assina_xmlnfe(envio)
        envio.validar()
        # seta os dados que nao tem
        resposta.retEvento.infEvento.cStat.valor = u'135'
        resposta.retEvento.infEvento.tpAmb.valor = ambiente
        resposta.retEvento.infEvento.verAplic.valor = consulta.resposta.protNFe.infProt.verAplic.valor
        resposta.retEvento.infEvento.cOrgao.valor = consulta.resposta.retCancNFe.infCanc.cOrgao.valor
        resposta.retEvento.infEvento.xMotivo.valor = consulta.resposta.retCancNFe.infCanc.xMotivo.valor
        resposta.retEvento.infEvento.dhRegEvento.valor = str(consulta.resposta.retCancNFe.infCanc.dhRegEvento.valor).replace(' ', 'T')
        protocolo_canc = consulta.resposta.retCancNFe.infCanc.nProt.valor
        resposta.retEvento.infEvento.nProt.valor = protocolo_canc
        resposta.retEvento.infEvento.chNFe.valor = chave
        resposta.retEvento.infEvento.Id.valor = 'ID%s' % protocolo_canc
        processo_cancelamento_nfe = ProcEventoNFe()
        processo_cancelamento_nfe.envEvento = envio
        processo_cancelamento_nfe.retEnvEvento = resposta
        processo_cancelamento_nfe.validar()
        processo.processo_cancelamento_nfe = processo_cancelamento_nfe
        return processo.processo_cancelamento_nfe.xml

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
