# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.nfe.manual_500.carta_correcao import ProcEventoNFe
from pynfe.xml_sped.__init__ import CODIGO_UF, UF_CODIGO
from pynfe.nfe.manual_600.consrecinfe import ProtNFe
from ..report.carta_correcao import CartaCorrecaoReport
from pynfe.xml_sped.certificado import Certificado
from openerp.tools import config as openerp_config
from pynfe.nfe.manual_600.nfe_400 import NFe
from geraldo.generators.pdf import PDFGenerator
from xml.dom.minidom import parseString
from SigUtils import numero, calcDigito
from openerp.tools.translate import _
from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400
from pynfe.nfe.manual_500.nf_e import NFe as nfe_validacao
from datetime import datetime
from utils import log_debug
from openerp.osv import osv
from pynfe.nfce.manual_600 import nfce_p  # Nao deveria estar aqui! Separar a emissao do DANFE-NFCe
from base import BODY_HTML
from PIL import Image
import subprocess
import StringIO
import logging
import socket
import base64
import pytz
import time
import re


# noinspection PyPep8Naming,PyAbstractClass,PyAbstractClass
class fiscal_nf_capa(osv.Model):
    """
    Nota fiscal eletronica
    """
    _inherit = 'fiscal.nf.capa'
    _logger = logging.getLogger(_inherit)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    def fisnfc_processar_nota_fiscal_eletronica(self, cr, user, nf, context=None):
        """
        Realiza o processamento de uma nota fiscal eletronica.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user int
        :type nf: long
        :type context: dict
        :return: valores ajustados da nota fiscal.
        :rtype: dict
        """
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True  # Para compatibilidade. Remover se nao usado
        brw_nf = self.browse(cr, user, nf, context)
        parametro = self._parametros(cr, user)

        # Ajusta vencimento de parcelas "a vista" e data de emissao
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        parcelas = [[1, p.id, {'vencimento': agora[:10]}]
                    for p in brw_nf.nf_parcelas_ids if p.vencimento == brw_nf.data_emissao[:10]]
        values = {}
        if brw_nf.tipo == 'S':
            if brw_nf.data_saida_entrada and brw_nf.data_saida_entrada < agora:
                data_saida_entrada = agora
            else:
                data_saida_entrada = brw_nf.data_saida_entrada
            values = {'data_emissao': agora, 'data_saida_entrada': data_saida_entrada,
                      'nf_parcelas_ids': parcelas}
            self.write(cr, user, nf, values, context)
        elif brw_nf.nf_propria:
            if brw_nf.data_emissao < agora:
                values = {'data_emissao': agora, 'data_saida_entrada': agora,
                          'nf_parcelas_ids': parcelas}
                self.write(cr, user, nf, values, context)

        # Notas de terceiros com valor igual a zero nao geram financeiro
        if brw_nf.vr_total_nf == 0 and not brw_nf.nf_propria:
            return {'state': '3'}  # Nota fiscal faturada!

        # Envio da NF-e para a SEFAZ
        if brw_nf.nf_propria and brw_nf.modelo_nf_series_id:
            item_old = self._fisnfc_ajusta_tributacao_icms_desonerado(cr, user, brw_nf, context)
            xml_nfe = self.gerar_nfe_xml(cr, user, nf, values, context) or False
            if not xml_nfe:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao e possivel gerar XML para a versao %s.' %
                                       parametro.fiscal.fispar_nfe_versao_xml))
            estado = False
            forma_emissao = '2' if brw_nf.modelo_nf_series_id.tipo_contingencia else '1'
            for brw_endereco in brw_nf.company_id.rescon_pessoa_id.endereco_ids:
                if brw_endereco.endereco_tipo_id.name == 'Comercial':
                    estado = brw_endereco.uf_id.sigla
                    break

                if brw_endereco.endereco_tipo_id.name == 'Normal':
                    estado = brw_endereco.uf_id.sigla
                    break
            if not parametro.fiscal.certificado_conteudo or \
                    not parametro.fiscal.chave_conteudo or not xml_nfe:
                raise osv.except_osv(_('Atencao!'),
                                     _('Certificado nao importado no parametro do Fiscal.'))

            # Processa a NF-e.
            try:
                nfe = nfe_400(certify=parametro.fiscal.certificado_conteudo,
                              key=parametro.fiscal.chave_conteudo,
                              environment=brw_nf.modelo_nf_series_id.ambiente, state=estado,
                              contingency=brw_nf.modelo_nf_series_id.tipo_contingencia)

                nfe.process.nova_versao = True
                processo, arquivos = nfe.processar_notas([xml_nfe])
                lote = processo.resposta.infRec.nRec.valor
                if lote:
                    values['nfe_lote'] = lote

                if processo.resposta.cStat.valor == u'103':
                    processo, n_arquivos = nfe.process.consultar_recibo(lote)
                    arquivos.extend(n_arquivos)
            except socket.gaierror:
                raise osv.except_osv(_('Atencao!'),
                                     _('Falha no servidor da SEFAZ. Verifique se existe conexao '
                                       'com a Internet.'))
            except IOError:
                raise osv.except_osv(_('Atencao!'),
                                     _('Falha no servidor da SEFAZ. Tente novamente mais tarde.'))
            except Exception as erro:
                if str(erro) == "'RetConsStatServ' object has no attribute 'infRec'":
                    erro = _('Nao houve resposta da SEFAZ no tempo previsto!')
                try:
                    if unicode(erro).find('certificate expired') >= 1:
                        erro = _('Certificado digital com validade vencida.')
                except:
                    if str(erro).find('certificate expired') >= 1:
                        erro = _('Certificado digital com validade vencida.')

                raise osv.except_osv(_('Atencao!'),
                                     self.pool.get('adm.utils').admutl_remover_acentuacao(
                                         cr, user, unicode(erro)))
            else:
                if processo.resposta.cStat.valor == u'105':  # Lote em processamento
                    values.update({'state': 'EP',
                                   'nfe_lote': processo.envio.nRec.valor})
                    self.write(cr, user, nf, item_old, context)
                elif processo.resposta.cStat.valor == u'104':  # Lote processado
                    dhrecbto = processo.resposta.protNFe
                    dhrecbto = dhrecbto and processo.resposta.protNFe[0].infProt.dhRecbto.valor
                    dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(
                        cr, user, datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
                        'America/Sao_Paulo', 'UTC')
                    self.pool.get('fiscal.evento').create(cr, user, {
                        'fiseve_xml_env': processo.envio.xml,
                        'fiseve_xml_ret': processo.resposta.xml,
                        'fiseve_tipo': '00999102',  # Envio do lote
                        'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                        'fiseve_status': processo.resposta.cStat.valor,
                        'fiseve_motivo': processo.resposta.xMotivo.valor,
                        'fiseve_lote': processo.envio.nRec.valor,
                        'fiseve_protocolo': False,
                        'fiseve_nf_id': brw_nf.id,
                    })
                    if processo.resposta.protNFe:
                        protocolo = processo.resposta.protNFe[0]  # Somente uma nota enviada
                    else:
                        protocolo = ProtNFe()
                        protocolo.infProt.cStat.valor = u'999'
                        protocolo.infProt.xMotivo.valor = u'Erro de comunicacao com a SEFAZ'
                    dhrecbto = processo.resposta.protNFe
                    dhrecbto = dhrecbto and processo.resposta.protNFe[0].infProt.dhRecbto.valor
                    dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(
                        cr, user, datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
                        'America/Sao_Paulo', 'UTC')
                    self.pool.get('fiscal.evento').create(cr, user, {
                        'fiseve_xml_env': processo.envio.xml,
                        'fiseve_xml_ret': processo.resposta.xml,
                        'fiseve_tipo': '00110100',  # Autorizacao da NF-e
                        'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                        'fiseve_status': protocolo.infProt.cStat.valor,
                        'fiseve_motivo': protocolo.infProt.xMotivo.valor,
                        'fiseve_lote': processo.envio.nRec.valor,
                        'fiseve_protocolo': protocolo.infProt.nProt.valor,
                        'fiseve_nf_id': brw_nf.id,
                    })
                    xml_file = protocolo.infProt.chNFe.valor + '-nfe-cert.xml'
                    values.update({
                        # 'nfe_chave_acesso': protocolo.infProt.chNFe.valor,
                                   'nfe_forma_emissao': forma_emissao})
                    if protocolo.infProt.cStat.valor == u'100':
                        # Nota fiscal faturada!
                        values.update({'nfe_lote': processo.envio.nRec.valor,
                                       'nfe_protocolo': protocolo.infProt.nProt.valor,
                                       'nfe_status': '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                                   protocolo.infProt.xMotivo.valor),
                                       'nfe_xml_gerado': dict(arquivos).get(xml_file),
                                       'nfe_xml_auto': protocolo.xml, 'state': '3'})

                        # Se o protocolo nao existir (a SEFAZ nao retornou) refaz o XML consultando
                        # a chave e recuperando o protocolo
                        if not values.get('nfe_protocolo'):
                            self._fisnfc_recupera_protocolo(cr, user, nf, values, context)
                    elif protocolo.infProt.cStat.valor == u'204':  # Duplicidade de nota fiscal
                        # Consultar o lote
                        lote = re.sub('[^0-9]', '', protocolo.infProt.xMotivo.valor or '')
                        processo, __ = nfe.consultar_recibo(
                            lote, parametro.fiscal.certificado_conteudo,
                            parametro.fiscal.chave_conteudo, brw_nf.modelo_nf_series_id.ambiente,
                            estado, brw_nf.modelo_nf_series_id.tipo_contingencia)
                        protocolo = processo.resposta.protNFe[0]  # Somente uma nota foi enviada
                        xml_file = protocolo.infProt.chNFe.valor + '-nfe-cert.xml'
                        values.update(
                            {'nfe_lote': lote, 'nfe_chave_acesso': protocolo.infProt.chNFe.valor,
                             'nfe_forma_emissao': forma_emissao,
                             'nfe_protocolo': protocolo.infProt.nProt.valor,
                             'nfe_xml_auto': protocolo.xml,
                             'nfe_xml_gerado': dict(arquivos).get(xml_file),
                             'state': '3' if protocolo.infProt.cStat.valor == u'100' else '2E',
                             'nfe_status': '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                         protocolo.infProt.xMotivo.valor)})
                    elif protocolo.infProt.cStat.valor == u'228':  # Data de emissao atrasada
                        values = {'nfe_chave_acesso': False,
                                  'state': '1', 'nfe_lote': processo.envio.nRec.valor,
                                  'nfe_status': '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                              protocolo.infProt.xMotivo.valor)}
                        self.write(cr, user, nf, item_old, context)
                    elif protocolo.infProt.cStat.valor in (u'110', u'205', u'301', u'302', u'303'):
                        # 110 = Uso denegado
                        # 205 = NF-e esta denegada na base de dados da SEFAZ
                        # 301 = Uso denegado: irregularidade fiscal do emitente
                        # 302 = Uso denegado: irregularidade fiscal do destinatario
                        # 303 = Destinatario nao habilitado a operar na UF
                        values.update({'nfe_status': '%s - %s.' % (
                            protocolo.infProt.cStat.valor, protocolo.infProt.xMotivo.valor),
                                       'state': 'D'})
                        self.write(cr, user, nf, item_old, context)
                    else:  # Lote processado mas, houve erro
                        values['state'] = '2E'
                        values['nfe_status'] = '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                             protocolo.infProt.xMotivo.valor)
                        self.write(cr, user, nf, item_old, context)
                else:
                    if hasattr(processo.resposta, 'protNFe'):
                        dhrecbto = processo.resposta.protNFe and processo.resposta.protNFe[0].dh
                        dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(
                            cr, user, datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
                            'America/Sao_Paulo', 'UTC')
                        self.pool.get('fiscal.evento').create(cr, user, {
                            'fiseve_xml_env': processo.envio.xml,
                            'fiseve_xml_ret': processo.resposta.xml,
                            'fiseve_tipo': '00999102',  # Envio do lote
                            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                            'fiseve_status': processo.resposta.cStat.valor,
                            'fiseve_motivo': processo.resposta.xMotivo.valor,
                            'fiseve_lote': processo.envio.nRec.valor,
                            'fiseve_protocolo': False,
                            'fiseve_nf_id': brw_nf.id,
                        })
                    values['state'] = '2E'
                    values['nfe_xml_gerado'] = xml_nfe
                    if not processo.resposta.cStat.valor:
                        values['nfe_status'] = '%s-%s\nErro interno na SEFAZ. Tente novamente.' % (processo.resposta.status,
                                                                                                   processo.resposta.reason)
                    else:
                        values['nfe_status'] = '%s - %s.' % (processo.resposta.cStat.valor,
                                                             processo.resposta.xMotivo.valor)
                    self.write(cr, user, nf, item_old, context)
        else:  # Nota fiscal de terceiro
            values['state'] = '3'  # Nota fiscal faturada!

        return values

    def _fisnfc_ajusta_tributacao_icms_desonerado(self, cr, user, nf, context=None):
        # Ajusta as tributacoes em funcao da desoneracao do ICMS (se houver)
        result = {}
        if not context.get('nota_fiscal_consumidor'):
            if nf.natureza_id.fisnat_icms_desonerado:
                icms_ids = self.pool.get('fiscal.encargo').search(
                    cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
                icms_id = icms_ids and icms_ids[0]
                cst40_ids = self.pool.get('fiscal.cst').search(
                    cr, user, [('codigo', '=', '40')], context=context)
                cst40_id = cst40_ids and cst40_ids[0]
                tributacao_ids = self.pool.get('fiscal.tributacao').search(
                    cr, user, [('encargo_id', '=', icms_id), ('cst_id', '=', cst40_id)],
                    context=context)
                tributacao_id = tributacao_ids and tributacao_ids[0]

                # Guarda os valores antigos do item e suas respectivas tributacoes. Se a nota
                # nao for autorizada os valores deverao ser restaurados
                result = {
                    'estoque_movimento_itens_ids': [[1, i.id, {
                        'vr_total_item_rateado': i.vr_total_item_rateado,
                        'tributacoes_ids': [[1, t.id, {
                            'cst_id': t.cst_id.id,
                            'tributacao_id': t.tributacao_id.id,
                            'aliquota': t.aliquota,
                            'vr_base_calculo': t.vr_base_calculo,
                            'vr_encargo': t.vr_encargo,
                            'vr_reducao_base': t.vr_reducao_base
                        }] for t in i.tributacoes_ids]
                    }] for i in nf.estoque_movimento_itens_ids]
                }

                # Recalcula a base de calculo em funcao do valor do ICMS desonerado e
                # consequentemente o valor do encargo
                item_updt = {
                    'estoque_movimento_itens_ids': [[1, i.id, {
                        'vr_total_item_rateado': i.vr_total_item_rateado - filter(
                            lambda f: f.encargo_id.id == icms_id,
                            i.tributacoes_ids)[0].fisnit_vr_icms_desonerado,
                        'tributacoes_ids': [[1, t.id, {
                            'cst_id': cst40_id if t.encargo_id.id == icms_id else t.cst_id.id,
                            'tributacao_id': tributacao_id if t.encargo_id.id == icms_id else
                            t.tributacao_id.id,
                            'aliquota': 0 if t.encargo_id.id == icms_id else t.aliquota,
                            'vr_base_calculo': t.vr_base_calculo - filter(
                                lambda f: f.encargo_id.id == icms_id,
                                i.tributacoes_ids)[0].fisnit_vr_icms_desonerado,
                            'vr_encargo': 0 if t.encargo_id.id == icms_id else
                            (t.vr_base_calculo - filter(
                                lambda f: f.encargo_id.id == icms_id,
                                i.tributacoes_ids)[0].fisnit_vr_icms_desonerado) *
                            t.aliquota / 100.,
                            'vr_reducao_base': 0
                        }] for t in i.tributacoes_ids]
                    }] for i in nf.estoque_movimento_itens_ids]
                }
                self.write(cr, user, nf.id, item_updt, context)

        return result

    def fisnfc_button_recuperar_protocolo(self, cr, user, ids, context=None):
        """Recupera o protocolo da nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: dados do protocolo da nota fiscal
        :rtype: verdadeiro se o protocolo foi recuperado
        """
        context = context and context.copy() or {}
        values = self.read(cr, user, ids[0], ['nfe_xml_auto', 'nfe_xml_cancelamento'], context)
        del values['id']
        self._fisnfc_recupera_protocolo(cr, user, ids[0], values, context)

        return self.write(cr, user, ids, values, context)

    def _fisnfc_recupera_protocolo(self, cr, user, nf, values, context=None):
        """Recupera o protocolo da nota fiscal. Refaz o XML em funcao do protocolo recuperado.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param values: valores da nota incluindo o XML da nota (valores retornados por referencia)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type values: dict
        :type context: dict
        :return: dados do protocolo da nota fiscal
        :rtype: dict
        """
        context = context and context.copy() or {}
        try:
            if context.get('nota_fiscal_consumidor'):
                processo = self.verifica_status_nfce_sefaz(cr, user, nf, context)
            else:
                processo, __ = self.verifica_status_nfe_sefaz(cr, user, nf, context)

            processo.resposta.protNFe.Signature.URI = ''  # tira a assinatura do xml de autorizacao
            values['nfe_xml_auto'] = processo.resposta.protNFe.xml
            values['nfe_protocolo'] = processo.resposta.protNFe.infProt.nProt.valor
            values['nfe_status'] = '%s - %s' % (processo.resposta.protNFe.infProt.cStat.valor,
                                                processo.resposta.protNFe.infProt.xMotivo.valor)
        except Exception as e:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao foi possivel recuperar o numero de protocolo da nota '
                                   'fiscal. Tente novamente.\n----\n%s' %
                                   processo.resposta.xMotivo.valor))

    def fisnfc_button_processar_lote(self, cr, user, ids, context=None):
        """
        Nota fiscal eletronica com ´status´ em "105 - Lote em processamento" sera verificada para
        conhecimento do termino do processo de faturamento em lote. Ao final do processo, se o
        ´status´ mudar para "100 - Autorizado o uso da NF-e", o da nota fiscal no sitema tambem ira
        mudar para "3 - Faturada". Caso contrario, sera alterado para "2E - Validada com erro".

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se o lote foi processado.
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True  # Remover se nao usado
        parametro = self._parametros(cr, user)

        # Dados da nota fiscal
        brw_nf = self.browse(cr, user, ids[0], context=context)

        if brw_nf.state == '3':
            return True  # A nota fiscal jah se encontra faturada
        if not brw_nf.nfe_lote:  # Se a nota nao contem lote, sequer foi enviada para SEFAZ...
            self.write(cr, user, ids, {'state': '2'}, context)  # Retorna para "validada"
            return self.button_faturar(cr, user, ids, context)  # Fatura a nota fiscal
        values = {}
        if not parametro.fiscal.certificado_conteudo or not parametro.fiscal.chave_conteudo:
            raise osv.except_osv(_('Atencao!'),
                                 _('Certificado nao importado no parametro do Fiscal.'))
        estado = False
        for brw_endereco in brw_nf.company_id.rescon_pessoa_id.endereco_ids:
            if brw_endereco.endereco_tipo_id.name == 'Comercial':
                estado = brw_endereco.uf_id.sigla
                break
            if brw_endereco.endereco_tipo_id.name == 'Normal':
                estado = brw_endereco.uf_id.sigla
                continue

        item_old = self._fisnfc_ajusta_tributacao_icms_desonerado(cr, user, brw_nf, context)
        # Consulta do lote na SEFAZ
        nfe = nfe_400
        nfe = nfe(certify=parametro.fiscal.certificado_conteudo,
                  key=parametro.fiscal.chave_conteudo,
                  environment=brw_nf.modelo_nf_series_id.ambiente, state=estado,
                  contingency=brw_nf.modelo_nf_series_id.tipo_contingencia)
        processo, __ = nfe.consultar_recibo(brw_nf.nfe_lote)

        if processo.resposta.cStat.valor == u'104':  # O lote foi processado
            if processo.resposta.protNFe:
                protocolo = processo.resposta.protNFe[0]  # Normalmente somente uma nota eh enviada
            else:
                protocolo = ProtNFe()
                protocolo.infProt.cStat.valor = u'999'
                protocolo.infProt.xMotivo.valor = u'Erro de comunicacao com a SEFAZ'
            processo.resposta.cStat.valor = protocolo.infProt.cStat.valor
            processo.resposta.xMotivo.valor = protocolo.infProt.xMotivo.valor
            if protocolo.infProt.cStat.valor == u'100':  # Nota fiscal esta autorizada (SEFAZ)
                # Gera o XML e assina
                nfe = NFe()
                nfe.xml = self.gerar_nfe_xml(cr, user, brw_nf.id, {}, context)
                certificado = Certificado()
                certificado.certificado = parametro.fiscal.certificado_conteudo
                certificado.chave = parametro.fiscal.chave_conteudo
                certificado.assina_xmlnfe(nfe)
                values['nfe_xml_gerado'] = nfe.xml
                values['nfe_xml_auto'] = protocolo.xml
                values['nfe_protocolo'] = protocolo.infProt.nProt.valor
                values['state'] = '3'
            elif protocolo.infProt.cStat.valor in (u'110', u'205', u'301', u'302', u'303'):
                # 110 = Uso denegado
                # 205 = NF-e esta denegada na base de dados da SEFAZ
                # 301 = Uso denegado: irregularidade fiscal do emitente
                # 302 = Uso denegado: irregularidade fiscal do destinatario
                # 303 = Destinatario nao habilitado a operar na UF
                values['state'] = 'D'
                self.write(cr, user, brw_nf.id, item_old, context)
            else:
                values['state'] = '2E'
                self.write(cr, user, brw_nf.id, item_old, context)
            if values.get('state') == '3':
                movimento = self.pool.get('gerencial.movimento').custificar_nota_fiscal(
                    cr, user, brw_nf, context)
                values.update(self._fisnfc_faturar_nota_fiscal(cr, user, brw_nf.id, values,
                                                               movimento, context))
        elif processo.resposta.cStat.valor == u'105':  # O lote continua em processamento
            pass
        else:  # O lote nao foi processado. Ocorreu algum erro
            values['state'] = '2E'
            self.write(cr, user, brw_nf.id, item_old, context)
        values['nfe_status'] = '%s - %s' % (processo.resposta.cStat.valor,
                                            processo.resposta.xMotivo.valor)
        result = self.write(cr, user, ids, values, context)
        self.message_post(cr, user, ids[0], values.get('nfe_status'))

        return result

    def fisnfc_faturar_nota_fiscal_eletronica(self, cr, user, nf_id, context=None):
        """
        Fatura a nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type context: dict
        :return: valores ajustados da nota fiscal.
        :rtype: dict
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)

        # Dados da nota fiscal
        dados_nf = self.read(
            cr, user, nf_id, ['numero_nf', 'serie_nf', 'modelo_nf_series_id', 'state',
                              'nfe_chave_acesso', 'fisnfc_consumidor', 'nfe_forma_emissao'], context, '_classic_write')

        # este contexto esta sendo passado na importacao de NFC-E do supervisor, pois a mesma ja
        # esta faturada na SEFAZ.
        if context.get('fisnfc_nao_faturar_sefaz'):
            return dados_nf

        # para nota de consumidor, altera data de emissao devido a diferenca maxima de 5 min que
        # a SEFAZ aceita entre a hora de envio com da emissao apenas quando nao for contingencia
        if dados_nf.get('fisnfc_consumidor') and dados_nf.get('nfe_forma_emissao') != '2':
            self.write(cr, user, nf_id, {'data_emissao': datetime.strptime(
                                datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '%Y-%m-%d %H:%M:%S')})

        # Processa NF-e. Gera o arquivo xml da nota e envia para a SEFAZ
        values = self.fisnfc_processar_nota_fiscal_eletronica(cr, user, nf_id, context)
        context['nfe_xml_dhRecbto'] = values.get('data_emissao') or time.strftime('%Y-%m-%d')
        agora = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
            cr, user, datetime.strptime(
                datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '%Y-%m-%d %H:%M:%S'), 'UTC',
            context=context), '%Y-%m-%d %H:%M:%S')
        ambiente = {1: _('Producao'), 2: _('Homologacao')}.get(parametro.fiscal.fispar_nfe_ambiente)
        self.message_post(cr, user, nf_id, values.get('nfe_status') or '' + ' em %s' % agora,
                          '%s %s-%s - %s [%s]' % (
                              'NFC-e' if context.get('nota_fiscal_consumidor') else 'NF-e',
                              dados_nf.get('numero_nf'), dados_nf.get('serie_nf'),
                              values.get('nfe_chave_acesso'), ambiente))
        if values.get('state') == 'D':  # Nota fiscal denegada
            # Exclui fatura em previsao no financeiro
            cr.execute("""ROLLBACK TO SAVEPOINT fisnfe_faturar_nota;
                          COMMIT;""")
            self.fisnfc_deleta_fatura_prv_financeiro(cr, user, nf_id, context)

        return values

    def verificar_situacao_destinatario(self, cr, uid, ids, context=None):
        """
        Verifica a situacao cadastral (IE) do cliente/destinatario nas bases de dados da SEFAZ.

        :param cr: o cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: o indicativo da sistuacao do cadastro IE: regular ou irregular.
        :rtype: object
        """
        context = context and context.copy() or {}
        vals = {}
        parametros = self._parametros(cr, uid)
        brw_nf = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        mod_nf = brw_nf.modelo_nf_series_id and brw_nf.modelo_nf_series_id
        if mod_nf:
            if mod_nf.tipo_contingencia:
                raise osv.except_osv(_('Atencao!'),
                                     _('Consulta de situacao cadastral do destinatario nao '
                                       'disponivel para emissao em contingencia.'))
        else:
            raise osv.except_osv(_('Atencao'), _('Modelo de NF-e nao informado.'))
        if not brw_nf.destinatario_id:
            raise osv.except_osv(_('Atencao!'), _('Destinatario nao informado'))
        vals['observacao_pessoa'] = brw_nf.destinatario_id.observacoes or ''
        cpf_cnpj = re.sub('[^0-9]', '', brw_nf.destinatario_id.cpf_cnpj or '')
        inscricao_estadual = re.sub('[^0-9]', '', brw_nf.destinatario_id.admpes_jur_ie or '')
        if brw_nf.destinatario_id.endereco_nf_id:
            estado = brw_nf.destinatario_id.endereco_nf_id.uf_id
            estado = estado and estado.sigla or ''
        else:
            raise osv.except_osv(_('Atencao!'),
                                 _('O endereco para nota fiscal do destinatario nao foi '
                                   'informado.'))
        if not cpf_cnpj:
            raise osv.except_osv(_('Atencao!'), _('Destinatario sem informacao de CPF/CNPJ.'))
        if inscricao_estadual.upper() in ('ISENTO', 'ISENTA'):
            inscricao_estadual = ''
        if not all((parametros.fiscal.certificado_conteudo, parametros.fiscal.chave_conteudo)):
            raise osv.except_osv(_('Atencao!'),
                                 _('Informacoes do certificado digital nao existem ou sao '
                                   'invalidas.'))
        try:
            nfe = nfe_400
            nfe = nfe(certify=parametros.fiscal.certificado_conteudo,
                      key=parametros.fiscal.chave_conteudo,
                      environment=brw_nf.modelo_nf_series_id.ambiente, state=estado,
                      contingency=mod_nf.tipo_contingencia)
            processo, __ = nfe.consultar_cadastro_contribuinte(cpf_cnpj, inscricao_estadual)
        except Exception as err:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro %s. Nao foi possivel consultar o servidor. Tente '
                                   'novamente mais tarde.') % err.message)
        else:
            situacao_xml = parseString(processo.resposta.xml.encode('utf-8')).toprettyxml()
            situacao_motivo = '%s - %s' % (
                processo.resposta.infCons.cStat.valor,
                processo.resposta.infCons.xMotivo.valor)
            if processo.resposta.infCons.cStat.valor in (u'111', u'112'):
                # 111 - consulta cadastro com uma ocorrência;
                # 112 - consulta cadastro com mais de uma ocorrência, existe mais de um estabelecimento
                situacao = processo.resposta.infCons.infCad[0].cSit.valor == 1 and 'R' or 'I'
                self.write(cr, uid, ids, {'fisnfc_nfe_situacao_cliente': situacao,
                                          'fisnfc_nfe_situacao_motivo': situacao_motivo,
                                          'fisnfc_nfe_xml_situacao_cliente': situacao_xml}, context)
                self.message_post(
                    cr, uid, ids, _('%s Consulta situacao %s para IE: %s') % (
                        datetime.now().strftime('%d/%m/%Y %H:%M:%S'),
                        {'R': 'REGULAR', 'I': 'IRREGULAR'}[situacao], inscricao_estadual),
                    _('Consulta destinatario/%s %s - %s' % (
                        estado, cpf_cnpj, brw_nf.destinatario_id.name)))
                dhrecbto = processo.resposta.infCons.dhCons.valor
                dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(
                    cr, uid, datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
                    'America/Sao_Paulo', 'UTC')
                self.pool.get('fiscal.evento').create(cr, uid, {
                    'fiseve_xml_env': processo.envio.xml,
                    'fiseve_xml_ret': processo.resposta.xml,
                    'fiseve_tipo': '00999200',  # Consultar situacao contribuinte
                    'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                    'fiseve_status': processo.resposta.infCons.cStat.valor,
                    'fiseve_motivo': processo.resposta.infCons.xMotivo.valor,
                    'fiseve_lote': False,
                    'fiseve_protocolo': False,
                    'fiseve_nf_id': ids[0],
                })
            else:
                motivo = processo.resposta.infCons.xMotivo.valor
                if not motivo:
                    motivo = _('O servico de consulta de cadastro nao esta disponivel para a '
                               'UF. (%s).' % estado)
                raise osv.except_osv(_('Atencao!'), motivo)

        return vals

    def verifica_status_nfe_sefaz(self, cr, user, nf_id, context=None):
        """
        Verifica e retorna o estado da nota fiscal eletronica na SEFAZ.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param nf_id: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type context: dict
        :return: tupla com dados do processamento e os arquivos.
        :rtype: tuple
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)
        brw_nf = self.browse(cr, user, nf_id, context)
        if not brw_nf.nfe_chave_acesso:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nota sem Chave de acesso.'))
        chave_acesso = brw_nf.nfe_chave_acesso.strip()
        if len(chave_acesso) != 44:
            raise osv.except_osv(_('Atencao!'),
                                 _('Chave de acesso invalida.'))
        estado = CODIGO_UF[int(chave_acesso[:2])]
        try:
            nfe = nfe_400
            nfe = nfe(certify=parametro.fiscal.certificado_conteudo,
                      key=parametro.fiscal.chave_conteudo,
                      environment=parametro.fiscal.fispar_nfe_ambiente, state=estado,
                      contingency=brw_nf.modelo_nf_series_id.tipo_contingencia)
            nfe.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
            nfe.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
            result, arquivos = nfe.consultar_nota(chave_acesso)
        except socket.gaierror:
            raise osv.except_osv(_('Atencao!'),
                                 _('Falha ao comunicar com o servidor da SEFAZ. Verifique se '
                                   'existe conexao com a Internet.'))
        except Exception as erro:
            raise osv.except_osv(_('Erro!'), erro.message) or erro
        except IOError:
            raise osv.except_osv(_('Erro!'), _('Falha ao comunicar com o servidor da SEFAZ. Tente '
                                               'novamente mais tarde.'))

        if result.resposta.cStat.valor == u'562':  # Nota com diferenca na chave de acesso
            # Pode acontecer quanto a numeracao da nota foi alterada (para numeracoes anteriores)
            chave_acesso = re.sub('[^0-9]', '', result.resposta.xMotivo.valor)
            if chave_acesso:
                result, arquivos = nfe.consultar_nota(chave_acesso)

        return result, arquivos

    def fisnfc_button_verificar_status_nfe(self, cr, uid, ids, context=None):
        """
        Verifica a situacao da nota fiscal eletronica na SEFAZ. Mostra o resultado da consulta no
        ´log´ (historico de mensagens)

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: sempre verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        if self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'fiscal', [],
                        context=context).get('data_validade') < datetime.now().strftime('%Y-%m-%d'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Certificado digital com data expirada.'))

        processo, __ = self.verifica_status_nfe_sefaz(cr, uid, ids[0], context)
        if processo.resposta.status == 'Erro':
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao foi possivel conectar com o servidor da SEFAZ.\n- %s') % \
                                 self.pool.get('adm.utils').admutl_remover_acentuacao(cr, uid,
                                                                          processo.resposta.reason.message))
        if not processo.resposta.cStat.valor:
            processo.resposta.cStat.valor = unicode(processo.resposta.status)
            processo.resposta.xMotivo.valor = processo.resposta.reason

        if not processo.resposta.protNFe:
            raise osv.except_osv(_('Atencao!'),
                                 _('%s - %s') % (processo.resposta.cStat.valor,
                                                 processo.resposta.xMotivo.valor))

        dhrecbto = processo.resposta.protNFe.infProt.dhRecbto.valor
        dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(
            cr, uid, datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
            'America/Sao_Paulo', 'UTC')
        self.pool.get('fiscal.evento').create(cr, uid, {
            'fiseve_xml_env': processo.envio.xml,
            'fiseve_xml_ret': processo.resposta.xml,
            'fiseve_tipo': '00999101',  # Consultar NF-e por chave de acesso
            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'fiseve_status': processo.resposta.cStat.valor,
            'fiseve_motivo': processo.resposta.xMotivo.valor,
            'fiseve_lote': False,
            'fiseve_protocolo': processo.resposta.protNFe.infProt.nProt.valor,
            'fiseve_nf_id': ids[0],
        })

        return True

    def fisnfc_monta_chave_acesso_nfe(self, cr, uid, nf, context=None):
        """
        Monta a chave de acesso da nota fiscal eletronica.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict
        :return: chave de acesso da nota fiscal eletronica.
        :rtype: str
        """
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, uid, nf, context)

        # Codigo numerico da UF do emissor
        uf = filter(lambda f: f.endereco_tipo_id.name == 'Normal',
                    brw_nf.company_id.rescon_pessoa_id.endereco_ids)
        if not uf:
            raise osv.except_osv(_("Atencao!"),
                                 _("Pessoa ligada a empresa logada sem UF cadastrado!"))
        uf = str(UF_CODIGO[uf[0].uf_id.sigla])

        nf_emi = datetime.strptime(self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid,
                                   datetime.strptime(brw_nf.data_emissao, '%Y-%m-%d %H:%M:%S'), '%d-%m-%Y %T'),
                                   '%d-%m-%Y %H:%M:%S').strftime('%y%m')  # Formato AAMM
        cnpj = re.sub('[^0-9]', '', brw_nf.company_id.rescon_pessoa_id.cpf_cnpj or '')
        modelo = brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_codigo
        numero =  str(brw_nf.numero_nf or brw_nf.modelo_nf_series_id.numero_nf or 1).zfill(9)
        serie = brw_nf.serie_nf.zfill(3) or brw_nf.modelo_nf_series_id.serie_nf.zfill(3)

        # Forma de emissao da nota
        contingencia = self._parametros(cr, uid).fiscal.fispar_nfe_contingencia
        emis = '6' if contingencia == 'SVC-AN' else '7' if contingencia == 'SVC-RS' else '1'

        # Monta a chave de acesso
        chave_acesso = uf + nf_emi + cnpj + modelo + serie + numero + emis + str(brw_nf.id).zfill(8)
        dv = calcDigito.baseModulo(chave_acesso, 98765432)
        dv = '0' if dv.__len__() > 1 else dv
        chave_acesso += dv

        return chave_acesso

    def fisnfc_nfe_denegada(self, cr, uid, ids, tipo_movimento, modelo_nf_series_id, nfe_status,
                            nfe_chave_acesso, nfe_xml_auto, context=None):
        """
        Este metodo serve para alterar o stauts da nota fiscal para denegada quando o status da nfe
        retornado pela sefaz estiver contido em '110', '205', '233', '234', '301', '302'

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das notas fiscais
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type context: dict
        :rtype: boolean
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        self.fisnfc_deleta_fatura_prv_financeiro(cr, uid, ids, context)
        obj_fiscal_modelo_nf_series = self.pool.get('fiscal.modelo.nf.series')
        brw_modelo_nf_series = obj_fiscal_modelo_nf_series.browse(
            cr, uid, modelo_nf_series_id, context)
        serie_nf = brw_modelo_nf_series.serie_nf or '0'
        numero_nf = brw_modelo_nf_series.numero_nf or 0
        obj_fiscal_modelo_nf_series.write(
            cr, uid, modelo_nf_series_id, {
                'serie_nf': serie_nf,
                'numero_nf': numero_nf + 1
            }, context)
        self.write(cr, uid, ids, {'state': 'D', 'nfe_status': nfe_status,
                                  'nfe_chave_acesso': nfe_chave_acesso,
                                  'nfe_xml_auto': nfe_xml_auto}, context)
        return True

    def exportar_danfe(self, cr, uid, ids, context=None):
        # foi necessário separar a funcao para o DANFE devido ao erro  no contexto na tree view
        context = context and context.copy() or {}
        context['gerar_danfe_pdf'] = True

        return self.exportar_xml_danfe(cr, uid, ids, context)

    def exportar_xml_danfe(self, cr, uid, ids, context=None, show_wizard=True):
        """
        Realiza a exportacao do XML e realiza a geracao do DANFE

        :type cr: object
        :param uid:
        :param ids:
        :param context:
        :param show_wizard: Usado para exibir ou nao o wizard com os dados gerados
        :return:
        """
        import xml.etree.ElementTree as ET
        parametro = self._parametros(cr, uid)
        if isinstance(ids, list):
            ids = ids[0]
        nf_id = ids
        obj_adm_utils = self.pool.get('adm.utils')
        dados_nf_capa = self.read(
            cr, uid, nf_id, ['nfe_xml_proc', 'state', 'nfe_forma_impressao_danfe', 'nfe_xml_lote',
                             'nfe_xml_cancelamento', 'nfe_protocolo', 'data_emissao', 'nfe_status',
                             'nfe_chave_acesso', 'nfe_versao_processo_emissao'], context=context)
        if isinstance(dados_nf_capa, list):
            dados_nf_capa = dados_nf_capa[0]
        xml_canc = dados_nf_capa.get('nfe_xml_proc') or ''
        if dados_nf_capa.get('state') == 'C':
            if not context.get('gerar_danfe') and not context.get('gerar_danfe_pdf'):
                xml_proc = base64.encodestring(xml_canc.encode('utf-8', 'strict'))
                context['active_id'] = nf_id
                id_export = self.pool.get('fiscal.export.nfe.osv').create(
                    cr, uid, {'xml_export': xml_proc}, context)
                retval = xml_proc
                if show_wizard:
                    retval = {
                        'name': 'Exportacao NF-e',
                        'view_type': 'form',
                        'view_mode': 'form',
                        'res_id': id_export,
                        'res_model': 'fiscal.export.nfe.osv',
                        'view_id': False,
                        'target': 'new',
                        'type': 'ir.actions.act_window'}
                return retval
            else:
                xml_canc = dados_nf_capa.get('nfe_xml_cancelamento') or ''
        else:
            xml_canc = dados_nf_capa.get('nfe_xml_cancelamento') or ''

        xml_proc = dados_nf_capa.get('nfe_xml_proc')
        versao = dados_nf_capa.get('nfe_versao_processo_emissao') or '4.00'
        if dados_nf_capa.get('state') in '12' or not xml_proc:
            xml_proc = self.gerar_nfe_xml(cr, uid, nf_id, {}, context)
        if 'nfeProc' not in xml_proc:
            ini_tag = xml_proc.find('<?xml')
            fim_tag = xml_proc.find('?>')
            xml_proc = xml_proc.replace(xml_proc[ini_tag:fim_tag + 2], u'')
            protocolo = dados_nf_capa.get('nfe_protocolo')
            nfe_status = ''
            if len(dados_nf_capa.get('nfe_status')) > 3:
                nfe_status = dados_nf_capa.get('nfe_status')[0:3]
            motivo = dados_nf_capa.get('nfe_status')
            dt = datetime.strptime(dados_nf_capa.get('data_emissao'), '%Y-%m-%d %H:%M:%S')
            dt = dt.replace(tzinfo=pytz.utc)
            data_hora_tz = datetime.astimezone(dt, pytz.timezone("America/Sao_Paulo"))
            dt_recebimento = datetime.strftime(data_hora_tz, u'%Y-%m-%dT%H:%M:%S') + str(
                data_hora_tz)[-6:]
            root = ET.fromstring(xml_proc)
            dig_val = root.find(".//{http://www.w3.org/2000/09/xmldsig#}DigestValue")
            dig_val = dig_val.text if dig_val is not None else ''
            ambiente = root.find(".//{http://www.portalfiscal.inf.br/nfe}tpAmb")
            ambiente = ambiente.text if ambiente is not None else ''
            xml_proc = u"""
            <?xml version="1.0" encoding="UTF-8"?>
                <nfeProc versao="{versao}" xmlns="http://www.portalfiscal.inf.br/nfe">
                {xml_proc}
                    <protNFe versao="{versao}">
                        <infProt>
                            <tpAmb>{ambiente}</tpAmb>
                            <verAplic>SigERP 1.0</verAplic>
                            <chNFe>{chave}</chNFe>
                            <dhRecbto>{dt_recebimento}</dhRecbto>
                            <nProt>{prot_nfe}</nProt>
                            <digVal>{digval}</digVal>
                            <cStat>{nfe_status}</cStat>
                            <xMotivo>{motivo}</xMotivo>
                        </infProt>
                    </protNFe>
                </nfeProc>
            """.format(xml_proc=xml_proc, prot_nfe=protocolo, nfe_status=nfe_status,
                       chave=dados_nf_capa.get('nfe_chave_acesso'), digval=dig_val,
                       dt_recebimento=dt_recebimento, ambiente=ambiente, motivo=motivo,
                       versao=versao).strip()
        if context.get('gerar_so_xml'):
            return xml_proc
        if not context.get('gerar_danfe') and not context.get('gerar_danfe_pdf'):
            xml_proc = base64.encodestring(xml_proc.encode('utf-8', 'strict'))
            context['active_id'] = nf_id
            id_export = self.pool.get('fiscal.export.nfe.osv').create(
                cr, uid, {'xml_export': xml_proc}, context)
            retval = xml_proc
            if show_wizard:
                retval = {
                    'name': 'Exportacao NF-e',
                    'view_type': 'form',
                    'view_mode': 'form',
                    'res_id': id_export,
                    'res_model': 'fiscal.export.nfe.osv',
                    'view_id': False,
                    'target': 'new',
                    'type': 'ir.actions.act_window'}
            return retval
        else:
            # Gera DANFE
            brw_nf = self.browse(cr, uid, nf_id, context)
            company_id = brw_nf.company_id.id
            if brw_nf.nf_propria:
                logo = self.pool.get('res.company').browse(cr, uid, company_id, context).logo
            else:
                logo = False
            nova_logo = u''
            xml_proc = obj_adm_utils.admutl_remover_acentuacao(cr, uid, xml_proc).replace('&#x27;', " ")
            if ET.fromstring(xml_proc).find(".//{http://www.portalfiscal.inf.br/nfe}tpImp").text == '2':
                raise osv.except_osv(_('Atencao'),
                                     _('DANFE em formato paisagem nao implementado.'))
            if logo:
                # Trata a imagem (logomarca)
                logo = base64.decodestring(logo)
                logo = StringIO.StringIO(logo)
                logo.seek(0)
                max_size = 1920, 1920
                im = Image.open(logo)
                im = im.convert('P', palette=Image.WEB).convert('RGB')
                nova_logo = StringIO.StringIO()
                im.resize(max_size, Image.BILINEAR)
                im.save(nova_logo, format='JPEG')
                nova_logo.seek(0)
            xml_proc = filter(lambda f: f not in '\n\t', xml_proc)
            xml_canc = filter(lambda f: f not in '\n\t', xml_canc)
            danfe = None
            try:
                if brw_nf.nf_propria:
                    modelo_eletronico = brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_codigo
                else:
                    modelo_eletronico = brw_nf.modelo_nf_id.fismnf_codigo
                if modelo_eletronico == u'55':  # NF-e
                    danfe = nfe_400().gerar_danfe(str(xml_proc), str(xml_canc), logo=nova_logo)
                elif modelo_eletronico == u'65':  # NFC-e
                    danfe = nfce_p().danfe_nfce(xml_proc, xml_canc, nova_logo)

            except ValueError:
                raise osv.except_osv(_('Atencao!'),
                                     _('Erro na renderizacao do danfe!'))

            if modelo_eletronico not in (u'55', u'65'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Este Modelo de nota não gera DANFE na entrada de terceiros!'))
            # Verifica se existe configuracao para impressao do DANFE no parametro, se existir
            # envia direto para impressora informada.
            if context.get('gerar_danfe'):
                if parametro.fiscal.impressora_danfe and parametro.fiscal.quantidade_vias_danfe > 0:
                    p = open('/tmp/danfe.pdf', 'wb')
                    p.write(danfe)
                    p.close()
                    vias = "-n" + str(parametro.fiscal.quantidade_vias_danfe)
                    formato = '-omedia=' + parametro.fiscal.fispar_tamanho_danfe or '-omedia=A4'
                    fit = '-ofit-to-page'
                    nome_impressora = "-d" + parametro.fiscal.impressora_danfe
                    try:
                        return subprocess.call(["lp", vias, formato, fit, nome_impressora, "/tmp/danfe.pdf"])
                    except OSError:
                        osv.except_osv(_('Atencao'),
                                       _('Impressora nao encontrada, verifique se as configuracoes '
                                         'de impressao estao corretas.'))
            else:
                danfe = base64.encodestring(danfe)
                context['active_id'] = nf_id
                context['gerar_danfe'] = True
                id_danfe = self.pool.get('fiscal.export.nfe.osv').create(
                    cr, uid, {'xml_export': danfe}, context)
                retval = danfe
                if show_wizard:
                    retval = {
                        'name': _('Impressao de DANFE'),
                        'view_type': 'form',
                        'view_mode': 'form',
                        'res_id': id_danfe,
                        'res_model': 'fiscal.export.nfe.osv',
                        'view_id': False,
                        'target': 'new',
                        'type': 'ir.actions.act_window'}

                return retval

    def exportar_xml_cancelada(self, cr, uid, ids, context=None, show_wizard=True):
        """
        Realiza a exportacao do XML de notas canceladas

        :type cr: object
        :param uid:
        :param ids:
        :param context:
        :param show_wizard: Usado para exibir ou nao o wizard com os dados gerados
        :return:
        """
        if isinstance(ids, list):
            ids = ids[0]
        nf_id = ids
        dados_nf_capa = self.read(
            cr, uid, nf_id, ['nfe_xml_cancelamento'], context=context)
        if isinstance(dados_nf_capa, list):
            dados_nf_capa = dados_nf_capa[0]
        xml_canc = dados_nf_capa.get('nfe_xml_cancelamento') or ''
        xml_canc = base64.encodestring(xml_canc.encode('utf-8', 'strict'))
        context['active_id'] = nf_id
        context['tipo'] = 'Cancelamento'
        id_export = self.pool.get('fiscal.export.nfe.osv').create(
            cr, uid, {'xml_export': xml_canc}, context)
        retval = xml_canc
        if show_wizard:
            retval = {
                'name': 'Exportacao NF-e',
                'view_type': 'form',
                'view_mode': 'form',
                'res_id': id_export,
                'res_model': 'fiscal.export.nfe.osv',
                'view_id': False,
                'target': 'new',
                'type': 'ir.actions.act_window'}
        return retval

    def abrir_carta_correcao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context.update({'carta_correcao': True})

        return {
            'name': _('Carta de Correcao'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.carta.correcao',
            'view_id': False,
            'context': context,
            'target': 'new',
            'type': 'ir.actions.act_window'}

    def enviar_email(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        nf_capa_data = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        mail_server_id = nf_capa_data.company_id.servidor_smtp_id or None
        v_chave_acesso = nf_capa_data.nfe_chave_acesso
        v_cliente = nf_capa_data.destinatario_id.name
        emails = ''
        for x in nf_capa_data.destinatario_id.email_ids:
            emails += x.name + ','
        emails = emails[:-1]
        v_num_nf = nf_capa_data.numero_nf
        v_total_nf = numero.getNumeroEditado(nf_capa_data.vr_total_nf, 2)
        v_data = datetime.strptime(nf_capa_data.data_emissao[:10], "%Y-%m-%d")
        v_data = v_data.strftime("%d/%m/%Y")
        v_name_company = nf_capa_data.company_id.name
        xml_carta_correcao = nf_capa_data.fisnfc_nfe_xml_correcao
        mail_mail = self.pool.get('mail.mail')
        ir_attachment = self.pool.get('ir.attachment')
        user = self.pool.get('res.users').browse(cr, uid, uid, context)
        if not user.email:
            raise osv.except_osv(_('Email requerido'),
                                 _('O usuario atual deve ter um endereco de e-mail configuradas em '
                                   '"Preferencias do usuario" para ser capaz de enviar e-mails.'))
        elif not emails:
            raise osv.except_osv(_('Email requerido'),
                                 _('O destinatario da nota fiscal deve possuir um e-mail '
                                   'cadastrado.'))
        subject = 'NF-e Numero: %d' % v_num_nf
        body = BODY_HTML % (v_cliente, v_num_nf, v_chave_acesso, v_num_nf, v_total_nf, v_data,
                            v_name_company)
        mail_id = mail_mail.create(
            cr, uid, {'email_from': user.email, 'email_to': emails, 'subject': subject,
                      'body_html': body, 'mail_server_id':mail_server_id}, context=context)
        attachment_ids = []
        datas = [('danfe.pdf', self.exportar_xml_danfe(
            cr, uid, ids, context={'gerar_danfe_pdf': True}, show_wizard=False)),
                 ('nfe.xml', self.exportar_xml_danfe(
                     cr, uid, ids, context=context, show_wizard=False))]
        if xml_carta_correcao:
            datas.append(('carta_correcao.xml', base64.encodestring(
                xml_carta_correcao.encode('utf-8'))))
        for data in datas:
            name, file_data = data
            attachment_data = {
                'name': name,
                'datas_fname': '%s-%s' % (v_chave_acesso, name),
                'datas': file_data,
                'res_model': mail_mail._name,
                'res_id': mail_id}
            attachment_ids.append(ir_attachment.create(cr, uid, attachment_data, context=context))
        if attachment_ids:
            mail_mail.write(
                cr, uid, mail_id, {'attachment_ids': [(6, 0, attachment_ids)]}, context=context)
        self.message_post(
            cr, uid, ids, subject=subject, body='E-mail criado com sucesso.', context=context)

        return True

    def button_rascunho(self, cr, user, ids, context=None):
        """
        Retorna o estado da nota fiscal para esboco.

        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list[long]
        :type context: dict
        :return: verdadeiro se o estado foi retornado para esboco.
        :rtype: bool
        """
        context = context and context.copy() or {}
        super(fiscal_nf_capa, self).button_rascunho(cr, user, ids, context=None)
        brw_nf = self.browse(cr, user, ids[0], context)

        # Verifica o estado da nota na SEFAZ se for eletronica
        if brw_nf.modelo_nf_series_id and brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe:
            processo, __ = self.verifica_status_nfe_sefaz(cr, user, brw_nf.id, context)
            msg = _('O estado da nota fiscal nao pode ser alterado para esboco, pois se encontra '
                    '%s na SEFAZ.')
            if processo.resposta.cStat.valor == u'100':  # Faturada
                raise osv.except_osv(_('Atencao!'), msg % _('faturada'))
            elif processo.resposta.cStat.valor == u'101':  # Cancelada
                raise osv.except_osv(_('Atencao!'), msg % _('cancelada'))
            elif processo.resposta.cStat.valor == u'102':  # Inutilizada
                raise osv.except_osv(_('Atencao!'), msg % _('inutilizada'))
            else:  # Pode ter havido algum problema. Verificar ´´cStat´´ para tratar
                pass

        return True

    def button_enviar_email(self, cr, uid, ids, context=None):
        """Envia email com as informacoes de acesso para consulta na Secretaria de Fazenda sobre
        os dados da NF-e."""
        context = context and context.copy() or {}
        return self.enviar_email(cr, uid, ids, context=context)

    @log_debug
    def fisnfc_verifica_dados_endereco(self, brw_nf, context=None):
        """
        Verifica os dados obrigatorios do endereco
        :param brw_nf:
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        msg = []
        if context.get('nota_fiscal_consumidor'):
            return True
        if not brw_nf.fisnfc_consumidor or brw_nf.destinatario_endereco_id:
            if not brw_nf.destinatario_endereco_id.logradouro_id.id:
                msg.append(_('logradouro'))
            if not brw_nf.destinatario_endereco_id.numero:
                msg.append(_('numero'))
            if not brw_nf.destinatario_endereco_id.bairro_id.id:
                msg.append(_('bairro'))
            if not brw_nf.destinatario_endereco_id.uf_id.id:
                msg.append(_('UF'))
            if not brw_nf.destinatario_endereco_id.cep:
                msg.append(_('cep'))
            if not brw_nf.destinatario_endereco_id.cidade_id.id:
                msg.append(_('cidade'))
        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('Endereco sem %s cadastrado.') % ', '.join(msg))
        return True

    @log_debug
    def fisnfc_verificar_nota_fiscal(self, cr, uid, nf, context=None):
        """
        Realiza a verificacao de uma nota fiscal.

        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota fiscal foi verificada sem erro ou inconsistencia.
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, uid, nf, context)
        res = super(fiscal_nf_capa, self).fisnfc_verificar_nota_fiscal(cr, uid, nf, context)
        parametro = self._parametros(cr, uid)
        if not brw_nf.modelo_nf_id.fismnf_tipo_nfe:
            return res

        # Verifica chave de acesso para nota fiscal eletronica
        if brw_nf.nfe_chave_acesso and brw_nf.numero_nf:
            # Trado desta forma para notas fiscais serie U por exemplo, estas vao para a sefaz
            # com a serie 0
            # ex: no chamado 98895
            if brw_nf.serie_nf and not brw_nf.serie_nf.isdigit():
                raise osv.except_osv(_('Atencao'),
                                     _('Serie invalida para este modelo de nota fiscal.'))

        # Verifica certificado digital para nota fiscal eletronica (propria)
        if brw_nf.nf_propria:
            if not parametro.fiscal.certificado_conteudo or not parametro.fiscal.chave_conteudo:
                raise osv.except_osv(_('Atencao!'),
                                     _('Certificado nao importado no parametro do Fiscal.'))

        # Verifica itens sem NCM
        if brw_nf.nf_propria:
            itens = []
            ncm_codigo = []
            itens_cest = []
            for i in brw_nf.estoque_movimento_itens_ids:
                if not i.produto_id.ncm_id.id:
                    itens.append(i)

                elif not i.produto_id.ncm_id.codigo:
                    ncm_codigo.append(i)

                for trib in i.tributacoes_ids:
                    if trib.encargo_id.sigla == 'ICMS' and trib.cst_id.tipo in ['U', 'TU']:
                        if not i.produto_id.estpro_cest_id:
                            itens_cest.append(i)

            if itens:
                raise osv.except_osv(_('Atencao!'),
                                     _('Os itens a seguir nao possuem NCM:\n%s') %
                                     '\n'.join(map(lambda m: '%s - %s' % (
                                         m.sequencia, m.produto_id.name), itens)))

            if ncm_codigo:
                raise osv.except_osv(_('Atencao!'),
                                     _('O NCM dos itens a seguir nao possui codigo:\n%s') %
                                     '\n'.join(map(lambda m: '%s - %s' % (
                                         m.sequencia, m.produto_id.name), ncm_codigo)))

            if itens_cest and not self.pool.get('adm.pessoa').verificar_empresa_simples_nacional(
                    cr, uid):
                raise osv.except_osv(_('Atencao!'),
                                     _('Os itens a seguir tem ICMS por substituicao e nao possuem CEST:\n%s') %
                                     '\n'.join(map(lambda m: '%s - %s' % (
                                         m.sequencia, m.produto_id.name), itens_cest)))

        if brw_nf.natureza_id and brw_nf.natureza_id.cfop_id.codigo[0] not in '37':
            self.fisnfc_verifica_dados_endereco(brw_nf, context)
        if brw_nf.modelo_nf_id.id and brw_nf.modelo_nf_id.fismnf_tipo_nfe and not context.get('terceiro') and \
            self.fisnfc_validade_certificado_digital(cr, uid, context).total_seconds() < 0:
            if brw_nf.nf_propria or not self._parametros(cr, uid).fiscal.data_validade:
                raise osv.except_osv(_('Atencao!'), _('Certificado digital com data expirada.'))

        return res

    def fisnfc_validade_certificado_digital(self, cr, user, context):
        context = context and context.copy() or {}
        certificado_validade = datetime.strptime(
            self._parametros(cr, user).fiscal.data_validade, '%Y-%m-%d %H:%M:%S')
        return certificado_validade - datetime.now()

    def fisnfc_onchange_nfe_chave_acesso(self, cr, uid, ids, chave, processo_emissao, context=None):
        """
        Metodo invocado na saida do atributo 'nfe_chave_acesso'. Executa o metodo de validacao da
        chave de acesso a cada alteracao dessa, recebida no parametro chave.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param chave: conjunto de caracteres que repesentam a chave de acesso.
        :type chave: str
        :param processo_emissao: caracter que representa o tipo de emissao da Nota Fiscal
        eletronica.
        :type processo_emissao: str
        :param context: dados contextuais.
        :type context: dict
        :return: Caso a chave seja invalida, retorna dicionario com as chaves 'warning' e 'value'.
        A primeira possui como valor um outro dicionario com dados de alerta para o usuario.
        A segunda possui como valor um outro dicionario com descricao e valor dos campos.
        Caso a chave seja valida, retorna dicionario vazio.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not ids or not chave:
            return res
        nf = ids[0] if isinstance(ids, (list, tuple)) else ids
        nfe_valida = self.fisnfc_valida_chave_nfe(cr, uid, nf, chave, processo_emissao, context)
        if not nfe_valida.get('valida'):
            res['warning'] = {'title': _('Atencao!'),
                              'message': nfe_valida.get('msg').decode('utf-8', 'ignore')}

        return res

    def fisnfc_valida_chave_nfe(self, cr, uid, nf, chave='', processo_emissao='', context=None):
        """
        O metodo realiza a validacao do conteudo da chave de acesso da nota fiscal eletronica.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param nf: ID da tabela desse objeto.
        :param chave: conjunto de caracteres que repesentam a chave de acesso.
        :param processo_emissao: caracter que representa o tipo de emissao da Nota Fiscal
        eletronica.
        :param context: dados contextuais.
        :type cr: object
        :type uid: int
        :type nf: long
        :type chave: str
        :type processo_emissao: str
        :type context: dict
        :return: dicionario com as chaves 'valida' e 'msg'. Caso a chave de acesso seja validada, a
        a chave 'valida' recebe o valor True. Senao, a chave 'valida' recebe o valor False e a
        chave 'msg' recebe como valor a descricao das inconsistencias encontradas.
        :rtype: dict
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        campos = ['tipo', 'company_id', 'destinatario_id', 'nfe_processo_emissao', 'data_emissao',
                  'nfe_chave_acesso', 'modelo_nf_id', 'serie_nf', 'numero_nf', 'natureza_id',
                  'nf_propria', 'destinatario_endereco_id']
        dados_nf = self.read(cr, uid, nf[0] if isinstance(nf, (list, tuple)) else nf, campos,
                   context, load)
        dados_modelo = self.pool.get('fiscal.modelo.nf').read(
            cr, uid, dados_nf.get('modelo_nf_id'), ['fismnf_codigo'], context)
        if dados_nf.get('tipo') == 'S':
            pessoa_id = self.pool.get('res.company').read(
                cr, uid, dados_nf.get('company_id'), ['rescon_pessoa_id'], context,
                load).get('rescon_pessoa_id')
        else:
            natureza_dados = self.pool.get('fiscal.cfop.natureza').read(
                cr, uid, dados_nf.get('natureza_id'), ['cfop_id'], context, load)
            cfop_dados = self.pool.get('fiscal.cfop').read(
                cr, uid, natureza_dados.get('cfop_id'), ['movimento_tipo'], context)
            if cfop_dados.get('movimento_tipo') == 'D' or dados_nf.get('nf_propria'):  # Devolucao ou Proṕria
                pessoa_id = self.pool.get('res.company').read(
                    cr, uid, dados_nf.get('company_id'), ['rescon_pessoa_id'], context,
                    load).get('rescon_pessoa_id')
            else:  # Terceiros
                pessoa_id = dados_nf.get('destinatario_id')
        obj_pessoa = self.pool.get('adm.pessoa')
        context['super_read_adm_pessoa'] = True
        if context.get('terceiro'):
            pessoa_id = dados_nf.get('destinatario_id')

        dado_pessoa = obj_pessoa.read(
            cr, uid, pessoa_id, ['cpf_cnpj', 'endereco_ids', 'endereco_nf_id'], context, '_classic_write')
        considera_cnpj = (processo_emissao or dados_nf.get('nfe_processo_emissao')) in ('0', '3')
        estado = ''
        endereco_fiscal_ids = dado_pessoa.get('endereco_nf_id') and [dado_pessoa.get('endereco_nf_id')] or []
        for brw_endereco in self.pool.get('adm.pessoa.endereco').browse(
                cr, uid, endereco_fiscal_ids or dado_pessoa.get('endereco_ids'), context):
            if brw_endereco.endereco_tipo_id.name == 'Comercial':
                estado = brw_endereco.uf_id.sigla
                break
            if brw_endereco.endereco_tipo_id.name == 'Normal':
                estado = brw_endereco.uf_id.sigla
                continue
        uf = estado and UF_CODIGO[estado] or ''
        uf = str(uf)
        data_emissao = datetime.strptime(dados_nf.get('data_emissao'), '%Y-%m-%d %H:%M:%S')
        data_emissao = self.pool.get('adm.utils').admutl_datetime_to_timezone(
            cr, uid, data_emissao, formato='%Y-%m-%d %H:%M:%S')
        chave = chave or dados_nf.get('nfe_chave_acesso')
        nfe_valida = {'valida': True, 'msg': ''}
        if chave:
            chave = chave.replace(' ', '')
            nfe_valida = nfe_validacao.validar_chave_nfe(
            chave, uf, data_emissao, dado_pessoa.get('cpf_cnpj'), dados_modelo.get('fismnf_codigo'),
            dados_nf.get('serie_nf'), dados_nf.get('numero_nf'), considera_cnpj)

        return nfe_valida

    @log_debug
    def fisnfc_validar_nota_fiscal(self, cr, user, nf, context=None):
        """
        Valida as informacoes da nota fiscal antes de ser faturada.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota fiscal foi validada.
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True
        values = {}
        context.update({'verificar_nf': True})
        values['nfe_versao_processo_emissao'] = self._parametros(
            cr, user).fiscal.fispar_nfe_versao_xml

        # Regra E16a-40 - Operacao com nao contribuinte deve indicar operacao com consumidor final
        brw_nf = nf and self.browse(cr, user, nf, context)
        if brw_nf.modelo_nf_series_id and brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe:
            if brw_nf.destinatario_id:
                if brw_nf.destinatario_id.pessoa_tipo == 'J':
                    if not brw_nf.destinatario_id.admpes_jur_ie or \
                            brw_nf.destinatario_id.admpes_jur_ie.upper() in ('ISENTO', 'ISENTA'):
                        uf = ''
                        for brw_endereco in brw_nf.destinatario_id.endereco_ids:
                            if brw_endereco.endereco_tipo_id.name == 'Comercial':
                                uf = brw_endereco.uf_id.sigla
                                break
                            if brw_endereco.endereco_tipo_id.name == 'Normal':
                                uf = brw_endereco.uf_id.sigla
                                continue
                        if uf in ('AM','BA','CE','GO','MG','MS','MT','PA','PE','RN','SE','SP'):
                            if not self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                                    cr, user, brw_nf.destinatario_id.id, context=None) == 'F':
                                if not brw_nf.destinatario_id.admpes_jur_ie:
                                    raise osv.except_osv(
                                        _('Atencao!'),
                                        _('Favor informar IE do destinatario. Destinatario nao '
                                          'contribuinte e nao e operacao com consumidor final.'))

            if not brw_nf.nfe_chave_acesso:
                values['nfe_chave_acesso'] = self.fisnfc_monta_chave_acesso_nfe(
                    cr, user, nf, context)
        self.write(cr, user, nf, values, context)

        return super(fiscal_nf_capa, self).fisnfc_validar_nota_fiscal(cr, user, nf, context)

    def copy(self, cr, uid, nf_id, default=None, context=None):
        context = context and context.copy() or {}
        default = default and default.copy() or {}
        default.update({'nfe_chave_acesso': '', 'nfe_lote': '', 'nfe_protocolo': '',
                        'nfe_status': '', 'nfe_forma_emissao': '1', 'nfe_xml_gerado': '',
                        'nfe_xml_auto': '', 'fisnfc_nfe_status_correcao': '',
                        'fisnfc_nfe_xml_correcao': '', 'fisnfc_nfe_retorno_situacao_nfe': ''})

        # Status do ambiente de emissao de NF-e
        status = self.fisnfc_get_status_config_nfe(cr, uid, nf_id, context)
        default['fisnfc_nfe_status_configuracao_producao'] = 1 == status[0] and status[1] or ''
        default['fisnfc_nfe_status_configuracao_homologacao'] = 2 == status[0] and status[1] or ''

        return super(fiscal_nf_capa, self).copy(cr, uid, nf_id, default, context)

    def fisnfc_get_status_config_nfe(self, cr, user, nf_id, context=None):
        """
        Retorna o status do ambiente de emissao de uma nota fiscal eletronica.
        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type context: dict
        :return: o status do ambiente de emissao da NF-e
        :rtype: list
        """
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, nf_id, context)
        if brw_nf and brw_nf.modelo_nf_series_id.ambiente == 1:
            status_configuracao = 'Ambiente: PRODUCAO\n'
        else:
            status_configuracao = 'Ambiente: HOMOLOGACAO\n'

        if brw_nf and brw_nf.modelo_nf_series_id.tipo_contingencia:
            status_configuracao += '- %s\n' % brw_nf.modelo_nf_series_id.tipo_contingencia
        else:
            status_configuracao += 'Servidor: SEFAZ\n'
        status_configuracao += 'Venc. Certificado: '
        data_validade = self._parametros(cr, user).fiscal.data_validade
        status_configuracao += data_validade and data_validade[:10] or ''

        return brw_nf.modelo_nf_series_id.ambiente, status_configuracao

    def fisnfc_funcao_button_cancelar(self, cr, user, ids, context=None):
        """
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a nota fiscal foi cancelada
        :rtype: bool
        """
        context = context and context.copy() or {}
        dados_nf = self.read(
            cr, user, ids[0], ['fisnfc_modelo_serie', 'modelo_55', 'state', 'nf_propria',
                               'nfe_chave_acesso'], context, '_classic_write')
        super(fiscal_nf_capa, self).fisnfc_funcao_button_cancelar(cr, user, ids, context)
        if not dados_nf.get('nf_propria') or \
                not dados_nf.get('nfe_chave_acesso') or \
                not dados_nf.get('modelo_55'):
            return True

        # Cancelamento de tesouraria
        if dados_nf.get('state') == '3':  # Somente NF faturada realizou movimento financeiro
            self.fisnfc_estorna_movimento_tesouraria_mod2D(cr, user, dados_nf, context)

        # Realiza a inutilizacao da nota fiscal se nao houver correspondencia na SEFAZ
        processo, __ = self.verifica_status_nfe_sefaz(cr, user, ids[0], context)
        if processo.resposta.cStat.valor == u'217':  # Nota nao encontrada deve ser inutilizada
            self.fisnfc_inutilizar_nota_fiscal(
                cr, user, ids[0], 'Nota fiscal inutilizada por desistencia de compra.', context)
        elif processo.resposta.cStat.valor == u'100' and dados_nf.get('state') != '3':  # Nota autorizada na SEFAZ deve ser faturada
            raise osv.except_osv(_('Erro!'),
                                 _('A nota fiscal nao pode ser cancelada pois, se encontra faturada'
                                   'na SEFAZ. A nota deve ser faturada primeiro.'))
        elif processo.resposta.cStat.valor in (u'100', u'101'):  # Nota cancelada na SEFAZ. OK!
            return True
        else:  # Outros problemas
            if not processo.resposta.cStat.valor:
                raise osv.except_osv(_('Atencao!'),
                                     _('Falha no servidor da SEFAZ. Tente novamente mais tarde.'))
            else:
                raise osv.except_osv(_('Aviso.'),
                                     '%s - %s' % (processo.resposta.cStat.valor,
                                                  processo.resposta.xMotivo.valor))

        return True

    def fisnfc_button_importar_cte(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True
        nf_dados = self.read(cr, user, ids[0] or ids, ['fisnfc_cte_xml'], context)
        xml = base64.b64decode(nf_dados.get('fisnfc_cte_xml'))
        cte_id = self.pool.get('wizard.importar.cte').wimcte_processar_xml_cte(
            cr, user, xml, reprocessar=False, context=context)
        if cte_id and not self.search(
                cr, user, [('id', '=', cte_id), ('fisnfc_nf_frete_id', '!=', False)],
                context=context):
            self.write(cr, user, cte_id, {'fisnfc_nf_frete_id': ids[0]}, context)
        else:
            raise osv.except_osv(_('Atencao!'),
                                 _('Arquivo XML do CT-e ja referenciado.'))

        self.write(cr, user, ids, {'fisnfc_cte_xml': False}, context)

    def fisnfc_button_imprimir_carta_correcao(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, ids[0], context)
        company_id = brw_nf.company_id.id
        if brw_nf.nf_propria:
            logo = self.pool.get('res.company').browse(cr, user, company_id, context).logo
        else:
            logo = False
        if logo:
            # Trata a imagem (logomarca)
            logo = base64.decodestring(logo)
            logo = StringIO.StringIO(logo)
            logo.seek(0)
            max_size = 1920, 1920
            im = Image.open(logo)
            im = im.convert('P', palette=Image.WEB).convert('RGB')
            nova_logo = StringIO.StringIO()
            im.resize(max_size, Image.BILINEAR)
            im.save(nova_logo, format='JPEG')
            nova_logo.seek(0)
            logo = nova_logo
        procevento = ProcEventoNFe()
        procevento.nfe = NFe()
        procevento.nfe.infNFe.emit.xNome.valor = brw_nf.company_id.rescon_pessoa_id.name
        endereco = brw_nf.company_id.rescon_pessoa_id.endereco_nf_id
        procevento.nfe.infNFe.emit.enderEmit.xLgr.valor = endereco.logradouro_id.name
        procevento.nfe.infNFe.emit.enderEmit.nro.valor = endereco.numero
        procevento.nfe.infNFe.emit.enderEmit.xBairro.valor = endereco.bairro_id.name
        procevento.nfe.infNFe.emit.enderEmit.xMun.valor = endereco.cidade_id.name
        procevento.nfe.infNFe.emit.enderEmit.UF.valor = endereco.uf_id.sigla
        procevento.nfe.infNFe.emit.enderEmit.CEP.valor = re.sub('[^0-9]', '', endereco.cep)
        procevento.nfe.infNFe.total.ICMSTot.vNF.valor = '%.2f' % brw_nf.vr_total_nf
        procevento.nfe.infNFe.ide.cNF.valor = str(brw_nf.numero_nf).zfill(8)
        procevento.nfe.infNFe.ide.serie.valor = brw_nf.serie_nf
        procevento.nfe.infNFe.ide.dhEmi.valor = datetime.strptime(
            self.pool.get('adm.utils').admutl_datetime_to_timezone(
                cr, user, datetime.strptime(brw_nf.data_emissao, '%Y-%m-%d %H:%M:%S'),
                '%d-%m-%Y %T'), '%d-%m-%Y %H:%M:%S')
        procevento.retEnvEvento.xml = str(brw_nf.fisnfc_nfe_xml_correcao)
        procevento.envEvento.xml = str(sorted(filter(
            lambda f: '110110' in f.fiseve_tipo, brw_nf.fisnfc_evento_ids), key=lambda k: k.id,
            reverse=True)[0].fiseve_xml_env)
        cc = CartaCorrecaoReport([procevento], logo)
        cc_pdf = StringIO.StringIO()
        cc.generate_by(PDFGenerator, filename=cc_pdf)
        cc = cc_pdf.getvalue()
        cc_pdf.close()
        cc_id = self.pool.get('fiscal.export.nfe.osv').create(cr, user, {
            'file': procevento.retEnvEvento.infEvento.chNFe.valor + '.pdf',
            'xml_export': base64.encodestring(cc)}, context)

        return {'name': _('Impressao da Carta de Correcao'), 'view_type': 'form',
                'view_mode': 'form', 'res_id': cc_id, 'res_model': 'fiscal.export.nfe.osv',
                'view_id': False, 'target': 'new', 'type': 'ir.actions.act_window'}

    def fisnfc_onchange_fisnfc_nfe_indintermed(self, cr, user, ids, opcao, context=None):
        """limpa/libera os campos de CNPJ e identificador do intermediador em funcao da opcao"""
        context = context and context.copy() or {}
        if opcao != '1':
            return {
                'value': {
                    'fisnfc_nfe_intermediador_cnpj': False,
                    'fisnfc_nfe_intermediador_identificador': False
                }
            }
        return {}

    def fisnfc_onchange_nfe_indicador_presenca(self, cr, user, ids, indicador, context=None):
        """Ajusta a operacao com intermediador"""
        if indicador not in ('2', '3', '4', '9'):
            return {
                'value': {
                    'fisnfc_nfe_indintermed': '0'
                }
            }
        return {}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
