# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################
from openerp.osv import osv
from openerp.tools.translate import _


class fiscal_parametro(osv.Model):
    _name = 'fiscal.parametro'
    _inherit = 'fiscal.parametro'

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        raise osv.except_osv(_('Atencao'),
                             _('Nao e permitido excluir o parametro.'))

    def abrir_importacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {
            'name': _('Importacao de Certificado Digital'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'fiscal.import.certificado',
            'view_id': False,
            'target': 'new',
            'type': 'ir.actions.act_window',
        }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        args = [('company_id', '=', company_id)]
        _id = self.search(cr, uid, args, context=context)
        if _id:
            return _id[0]
        return super(fiscal_parametro, self).create(cr, uid, vals, context=context)

    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not isinstance(default, dict):
            default = {}
        default['company_id'] = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        return super(fiscal_parametro, self).copy(cr, uid, id, default, context=context)

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}

        # NF-e
        modelo_ids = self.pool.get('fiscal.modelo.nf.series').search(
            cr, user, [('parametro_id', '=', ids[0] if isinstance(ids, list) else ids)],
            context=context)
        for brw_parametro in self.browse(cr, user, ids if isinstance(ids, list) else [ids], context):
            versao_xml = vals.setdefault('fispar_nfe_versao_xml',
                                         brw_parametro.fispar_nfe_versao_xml)
            ambiente = vals.setdefault('fispar_nfe_ambiente', brw_parametro.fispar_nfe_ambiente)
            contingencia = vals.setdefault('fispar_nfe_contingencia',
                                           brw_parametro.fispar_nfe_contingencia)
            motivo = vals.setdefault('fispar_nfe_contingencia_motivo',
                                     brw_parametro.fispar_nfe_contingencia_motivo)
            data = vals.setdefault('fispar_nfe_contingencia_data',
                                   brw_parametro.fispar_nfe_contingencia_data)
            self.pool.get('fiscal.modelo.nf.series').write(
                cr, user, modelo_ids, {'versao_xml': versao_xml, 'ambiente': ambiente,
                                       'data_scan': data, 'motivo_scan': motivo,
                                       'tipo_contingencia': contingencia}, context)

        return super(fiscal_parametro, self).write(cr, user, ids, vals, context=context)

    def fispar_onchange_nfe_contingencia(self, cr, user, ids, empresa, contingencia, context=None):
        """
        Verifica se a contingencia pode ser utilizada pela UF.
        
        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs do formulario
        :param empresa: ID da empresa
        :param contingencia: tipo de contingencia: SVC-AN ou SVC-RS
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type empresa: long
        :type contingencia: bool
        :type context: dict
        :return: mensagem de aviso 
        :rtype: dict
 
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br> 
        """
        context = context and context.copy() or {}
        brw_empresa = self.pool.get('res.company').browse(
            cr, user, empresa, context)
        endereco = brw_empresa.rescon_pessoa_id.endereco_nf_id
        endereco = endereco or brw_empresa.rescon_pessoa_id.endereco_ids
        uf = endereco and endereco[0].uf_id if isinstance(endereco, list) else endereco.uf_id
        estado = uf and uf.name or 'Nao encontrada'
        uf = uf and uf.sigla or 'UF'
        msg = _('Contingenica %s nao esta relacionada ao estado %s.')
        result = {}
        if contingencia and contingencia == 'SVC-AN':
            if uf not in ('AC', 'AL', 'AP', 'DF', 'ES', 'MG', 'PB', 'RJ', 'RN', 'RO', 'RR', 'RS',
                          'SC', 'SE', 'SP', 'TO'):
                result = {'value': {'fispar_nfe_contingencia': False,
                                    'fispar_nfe_contingencia_motivo': False,
                                    'fispar_nfe_contingencia_data': False},
                          'warning': {'title': _('Aviso.'),
                                      'message': msg % (contingencia, '%s - %s' % (uf, estado))}}
        elif contingencia and contingencia == 'SVC-RS':
            if uf not in ('AM', 'BA', 'CE', 'GO', 'MA', 'MS', 'MT', 'PA', 'PE', 'PI', 'PR'):
                result = {'value': {'fispar_nfe_contingencia': False,
                                    'fispar_nfe_contingencia_motivo': False,
                                    'fispar_nfe_contingencia_data': False},
                          'warning': {'title': _('Aviso.'),
                                      'message': msg % (contingencia, '%s - %s' % (uf, estado))}}
        else:
            result = {'value': {'fispar_nfe_contingencia_motivo': False,
                                'fispar_nfe_contingencia_data': False}}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
