# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from datetime import date, datetime
from openerp.tools.translate import _
from openerp.osv.fields import float_round
from decimal import Decimal, ROUND_UP
from openerp.tools import config as openerp_config
from pynfe.nfe.manual_600.consrecinfe import ProcNFe
from ...adm.control.utils import capitalize_all


def log_debug(func):
    import time
    import logging
    no_logger = ['fiscal.utils']

    def wrapper(*args, **kwargs):
        if args[0]._name in no_logger and openerp_config.get('log_level').upper() == 'DEBUG':
            _logger = logging.getLogger(__name__)
            _logger.setLevel(logging.DEBUG)  # Enable DEBUG mode
        elif openerp_config.get('log_level').upper() == 'DEBUG':
            _logger = args[0]._logger
        else:
            return func(*args, **kwargs)
        start_time = time.time()
        result = func(*args, **kwargs)
        if _logger.isEnabledFor(logging.DEBUG):
            end_time = time.time()
            _logger.debug('%s :: %s@%s|%s (%.3fs)' % (
                func.__doc__ and func.__doc__.replace(
                    '\n', '', 1).strip().split('\n')[0] or '', func.func_name,
                datetime.fromtimestamp(start_time).strftime('%H:%M:%S'),
                datetime.fromtimestamp(end_time).strftime('%H:%M:%S'), end_time - start_time))

        return result

    return wrapper


class fiscal_utils(osv.Model):
    _name = 'fiscal.utils'
    _auto = False
    _tem_erro = False
    _mensagem_erro = u''

    def fisutl_get_erro(self):
        return self._mensagem_erro

    def fisutl_get_nf_names(self, cr, uid, nf_ids, context=None):
        # NAME DAS NF_IDS
        context = context and context.copy() or {}
        res_name_get = self.pool.get('fiscal.nf.capa').name_get(cr, uid, nf_ids, context) or [('', '')]
        return dict(res_name_get)

    def gerar_nf(self, cr, uid, company_id, destinatario_id, natureza_id=False, nf_condicao_ids=[], tipo='S',
                 modelo_nf_id=False, data_saida_entrada=False, end_entrega=False, end_coleta=False,
                 numero_nf=False, serie_nf=False, vendedor_id=False, vendedor_externo_id=False, vr_desconto=0,
                 vr_seguro=0, vr_despesas_acessorias=0, vr_frete=0, estoque_movimento_itens_ids=[],
                 nota_pedido_ids=[], finan_contrato_id=False, destinatario_endereco_id=False,
                 context=None, tipo_frete='F'):

        '''
        Funcao para geracao da capa da Nota Fiscal,retorna o id nf
        Obs: Em nf_condicao_ids, passar uma lista de tuplas para campo one2many com o parâmetro de create que e 0.
        Ex: [(0, 0, {'forma_pagto_id': 1, 'condicao_forma_id': 64, 'valor': 10.0}),
        (0, 0, {'forma_pagto_id': 7, 'condicao_forma_id': 56, 'valor': 15.0})]
        '''

        context = context and context.copy() or {}
        context['fiscal'] = True
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_fiscal_cfop_natureza = self.pool.get('fiscal.cfop.natureza')
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        obj_fiscal_parametro = self.pool.get('fiscal.parametro')
        erro = False

        parametro_id = obj_fiscal_parametro.search(cr, uid, [], context=context)
        if not parametro_id:
            erro = True
        else:
            tipo_nfe = False
            browse_parametro = obj_fiscal_parametro.browse(cr, uid, parametro_id[0])
            if browse_parametro:
                if context.get('nf_servico'):
                    if browse_parametro.fispar_servico_modelo_nf_series_id:
                        modelo_serie = browse_parametro.fispar_servico_modelo_nf_series_id.id
                        tipo_nfe = browse_parametro.fispar_servico_modelo_nf_series_id.\
                            modelo_nf_id.fismnf_tipo_nfe or False
                    else:
                        raise osv.except_osv(_('Atencao'),
                                             _('Informe um modelo padrao para Nota Fiscal de '
                                               'servico no parametro do Fiscal.'))
                elif context.get('faturar_pedido_nfce'):
                    modelo_serie = browse_parametro.fispar_nfce_modelo_series_id.id
                    tipo_nfe = True
                elif browse_parametro.modelo_nf_series_id:
                    modelo_serie = browse_parametro.modelo_nf_series_id.id
                    tipo_nfe = browse_parametro.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe \
                               or False
                else:
                    raise osv.except_osv(_('Atencao'), _('Informe um modelo padrao para Nota '
                                                         'Fiscal no parametro do Fiscal.'))
            else:
                erro = True
            if modelo_nf_id:
                modelo_serie = modelo_nf_id
                brw_modelo = self.pool.get('fiscal.modelo.nf.series').browse(cr, uid, modelo_nf_id,context=context)
                if brw_modelo and brw_modelo.modelo_nf_id:
                    tipo_nfe = brw_modelo.modelo_nf_id.fismnf_tipo_nfe  or False
            id_endereco = self.pool.get('adm.pessoa.endereco').search(cr, uid, [('pessoa_id', '=', destinatario_id)],
                                                                      context=context)
            data = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            vals = {'tipo': tipo,
                    'company_id': company_id,
                    'destinatario_id': destinatario_id,
                    'modelo_nf_series_id': modelo_serie,
                    'data_emissao': data,
                    'modelo_55': tipo_nfe if not context.get('faturar_pedido_nfce') else False,
                    'fisnfc_consumidor': context.get('faturar_pedido_nfce') or False,
                    'nfe_forma_impressao_danfe': '4' if context.get('faturar_pedido_nfce') else '1',
                    'nf_condicao_ids': nf_condicao_ids,
                    'endereco_entrega_id': end_entrega,
                    'endereco_coleta_id': end_coleta,
                    'natureza_id': natureza_id,
                    'data_saida_entrada': data_saida_entrada or data if not context.get('faturar_pedido_nfce') else False,
                    'vendedor_id': vendedor_id,
                    'vendedor_externo_id': vendedor_externo_id,
                    'vr_desconto': vr_desconto,
                    'vr_seguro': vr_seguro,
                    'vr_despesas_acessorias': vr_despesas_acessorias,
                    'vr_frete': vr_frete,
                    'tipo_frete': tipo_frete,
                    'frete': '0' if vr_frete else '9',
                    'estoque_movimento_itens_ids': estoque_movimento_itens_ids,
                    }

            if context.get('ctx_nf_propria'):
                vals['nf_propria'] = True
            if context.get('ctx_nf_servico'):
                vals['fisnfc_tipo_servico'] = True

            if nota_pedido_ids:
                if not isinstance(nota_pedido_ids, list):
                    nota_pedido_ids = [nota_pedido_ids]
                vals.update({'fisnfc_pedido_ids': [(6, 0, nota_pedido_ids)]})
            if tipo == 'E':
                vals['numero_nf'] = numero_nf
                vals['serie_nf'] = serie_nf
            if destinatario_endereco_id:
                vals['destinatario_endereco_id'] = destinatario_endereco_id
            elif id_endereco:
                vals['destinatario_endereco_id'] = id_endereco[0]
            else:
                if not context.get('nf_from_cupom') and not context.get('inutilizacao'):
                    if not context.get('faturar_pedido_nfce'):
                        # nota de consumidor e gerenciais(modelo >= 90 ) nao precisam de endereco
                        if not context.get('nota_fiscal_consumidor') and \
                            self.pool.get('fiscal.modelo.nf.series').browse(cr, uid, modelo_serie,
                                                        context).modelo_nf_id.fismnf_codigo < '90':
                            erro = True

            if finan_contrato_id:
                vals['fisnfc_contrato_id'] = finan_contrato_id
            try:
                #buscando funcionario
                funcionario_id = obj_rh_funcionario.get_id_func(cr, uid)
            except:
                erro = True if not context.get('nota_fiscal_consumidor') else False
            if not erro:
                #gerando nota fiscal
                nf_id = obj_fiscal_nf_capa.create(cr, uid, vals, context=context)

                fisnat_operacao_id = obj_fiscal_cfop_natureza.browse(cr, uid, natureza_id).fisnat_operacao_id.id
                if not fisnat_operacao_id:
                    raise osv.except_osv(_('Atencao'),
                                         _('Nos cadastros do fiscal, informe a operacao para a natureza da nota'))
        if erro:
            nf_id = False

        return nf_id

    def gerar_nf_itens(self, cr, uid, nf_id, produto_id, tipo='P', cupom_id=False, local_id=False,
                       produto_embalagem_id=False,
                       grade_id=False, quantidade=1, vr_desconto=0, vr_unitario=0, vr_total_item=0,
                       vendedor_id=False, vendedor_externo_id=False, item_preco_venda=0,
                       medidas_volume={}, context=None, sequencia=1):

        '''Funcao para a geracao de itens para a Nota Fiscal, retorna o id do item'''

        context = context and context.copy() or {}
        obj_mov_itens = self.pool.get('estoque.movimento.itens')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_estoque_movimento_capa = self.pool.get('estoque.movimento.capa')
        erro = False
        context['fiscal'] = True
        vals = {
            'nf_id': nf_id,
            'produto_id': produto_id,
            'local_id': local_id,
            'quantidade': quantidade,
            'vr_desconto': vr_desconto,
            'produto_embalagem_id': produto_embalagem_id,
            'grade_id': grade_id,
            'vendedor_id': vendedor_id or False,
            'vendedor_externo_id': vendedor_externo_id or False,
            'sequencia': sequencia
        }
        if cupom_id:
            if isinstance(cupom_id, list):
                cupom_id = cupom_id[0]
            vals['cupom_id'] = cupom_id
        if medidas_volume:
            vals.update(medidas_volume)
        #pegando informacao do natureza_id que está na capa da nota fiscal
        dados_nf = obj_fiscal_nf_capa.read(cr, uid, nf_id, ['tipo', 'natureza_id'], context=context,
                                           load='_classic_write')
        nf_tipo = dados_nf.get('tipo')
        natureza_id = dados_nf.get('natureza_id')
        cfop_id = self.pool.get('fiscal.cfop.natureza').read(cr, uid, natureza_id, ['cfop_id'],
                                               context=context,load='_classic_write').get('cfop_id')
        vals['cfop_id'] = cfop_id or False

        #Calculando o valor unitario pela embalagem do produto
        if not vr_unitario and not context.get('pcp_ordem_producao'):
            preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, produto_id, epc_grade_ids=grade_id)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            vr_unitario = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')
            if vr_unitario and produto_embalagem_id:
                embalagem_dados = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                                                                               produto_embalagem_id)
                embalagem = embalagem_dados.embalagem_id
                qt_emb = 1.0
                if embalagem:
                    qt_emb = embalagem.quantidade_na_embalagem or 1.0

                vr_unitario *= qt_emb

        vals['vr_unitario'] = vr_unitario
        if not vr_total_item:
            vr_total_item = float(float_round((quantidade * vr_unitario - vr_desconto), 2))

        vals['vr_total_item'] = vr_total_item
        if vals.get('estmvi_volume'):
            vals['vr_total_item'] *= vals['estmvi_volume']

        if nf_tipo == 'S':
            vals['vr_preco_venda'] = item_preco_venda

        #grade
        if not grade_id:
            grade_id = obj_estoque_produto_grade.search(cr, uid, [('produto_id', '=', produto_id)],
                                                        context=context)
            if grade_id:
                vals['grade_id'] = grade_id[0]
            else:
                erro = True

        if not erro:
            id_mov = obj_estoque_movimento_capa.search(cr, uid, [('nf_capa_id', '=', nf_id)],
                                                       context=context)
            #adicionando o movimento da capa
            if id_mov:
                vals['capa_id'] = id_mov[0]

            return obj_mov_itens.create(cr, uid, vals, context=context)

        return False

    def calcular_tributos(self, cr, uid, produto_id, dados_pessoa, tipo_tributacao, encargo_ids,
                          context=None):
        """ Retorna as tributacoes dos itens """
        context = context and context.copy() or {}
        ids_tri = []
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')

        # Verifica tributacao de entrada terceiro por fornecedor
        if tipo_tributacao == 'E' and (context.get('terceiro') or context.get('entrada_terceiros')
                                       or context.get('importacao_nfe') or context.get('proprio')):
            if dados_pessoa:
                fornecedor_ids = self.pool.get('financeiro.fornecedor').search(
                    cr, uid, [('pessoa_id', '=', dados_pessoa.get('id'))], context=context)
                tributacao_entrada_ids = []
                tipo_movimento_cfop = context.get('dados_cfop') and \
                                      context.get('dados_cfop').get('movimento_tipo') or ''
                if fornecedor_ids and tipo_movimento_cfop != 'D':
                    fornecedor_id = fornecedor_ids[0]
                    obj_fiscal_tributacao_entrada = self.pool.get('fiscal.tributacao.entrada')
                    tributacao_entrada_ids = obj_fiscal_tributacao_entrada.search(
                        cr, uid, [('fornecedor_id', '=', fornecedor_id),
                                  ('produto_id', '=', produto_id)], context=context)
                    if tributacao_entrada_ids:
                        tributacao_entrada_dados = obj_fiscal_tributacao_entrada.read(
                            cr, uid, tributacao_entrada_ids[0], ['tributacao_entrada_ids'],
                            context=context)
                        tributacao_entrada_encargo_ids = \
                            tributacao_entrada_dados.get('tributacao_entrada_ids')
                        tributos_entrada_dados = \
                            self.pool.get('fiscal.tributacao.entrada.encargos').read\
                                (cr, uid, tributacao_entrada_encargo_ids,
                                 ['tributacao_id', 'encargo_id'], context=context)
                        for t in tributos_entrada_dados:
                            if context.get('buscar_tributacao_devolucao'):
                                if t['encargo_id'][1].upper() in ('PIS', 'COFINS', 'IPI'):
                                    tributo_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                        cr, uid, [('produto_id', '=', produto_id),
                                                  ('tipo_tributacao', '=', 'T'),
                                                  ('encargo_id', '=', t['encargo_id'][0])],
                                        context=context)
                                    tributos_saida_dados = obj_fiscal_tributacao_encargos.read(
                                        cr, uid, tributo_encargo_ids, ['tributacao_id'],
                                        context=context)
                                    for t in tributos_saida_dados:
                                        ids_tri.append(t['tributacao_id'][0])
                                else:
                                    ids_tri.append(t['tributacao_id'][0])
                            else:
                                ids_tri.append(t['tributacao_id'][0])
                        context.update({'busca_fiscal_tributacao': True})
                elif tipo_movimento_cfop == 'D':  # Nota fiscal de devolucao
                    if encargo_ids:
                        for enc_id in encargo_ids:
                            tributo_entrada_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                cr, uid, [('produto_id', '=', produto_id),
                                          ('tipo_tributacao', '=', 'E'),
                                          ('encargo_id', '=', enc_id)], context=context)
                            if tributo_entrada_encargo_ids:
                                tributacao_entrada_ids.append(tributo_entrada_encargo_ids[0])
                        if tributacao_entrada_ids:
                            tributos_entrada_dados = obj_fiscal_tributacao_encargos.read(
                                cr, uid, tributacao_entrada_ids, ['tributacao_id', 'encargo_id'],
                                context=context)
                            for t in tributos_entrada_dados:
                                if context.get('buscar_tributacao_devolucao'):
                                    if t['encargo_id'][1].upper() in ('PIS', 'COFINS', 'IPI'):
                                        tributo_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                            cr, uid, [('produto_id', '=', produto_id),
                                                      ('tipo_tributacao', '=', 'T'),
                                                      ('encargo_id', '=', t['encargo_id'][0])],
                                            context=context)
                                        tributos_saida_dados = obj_fiscal_tributacao_encargos.read(
                                            cr, uid, tributo_encargo_ids, ['tributacao_id'],
                                            context=context)
                                        for t in tributos_saida_dados:
                                            ids_tri.append(t['tributacao_id'][0])
                                    else:
                                        ids_tri.append(t['tributacao_id'][0])
                                else:
                                    ids_tri.append(t['tributacao_id'][0])
                            context.update({'busca_fiscal_tributacao': True})
                if not tributacao_entrada_ids:
                    # Busca especifica por encargos eh utilizada quando for filtrar pelos encargos
                    # aplicados a natureza da operacao
                    if encargo_ids:
                        for enc_id in encargo_ids:
                            tributo_entrada_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                cr, uid, [('produto_id', '=', produto_id),
                                          ('tipo_tributacao', '=', tipo_tributacao),
                                          ('encargo_id', '=', enc_id)], context=context)
                            if not tributo_entrada_encargo_ids:
                                tributo_entrada_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                    cr, uid, [('produto_id', '=', produto_id),
                                              ('tipo_tributacao', '=', 'E'),
                                              ('encargo_id', '=', enc_id)], context=context)
                            if not tributo_entrada_encargo_ids:
                                tributo_entrada_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                    cr, uid, [('produto_id', '=', produto_id),
                                              ('tipo_tributacao', '=', 'T'),
                                              ('encargo_id', '=', enc_id)], context=context)

                            # Encontrou
                            if tributo_entrada_encargo_ids and tipo_movimento_cfop == 'D':
                                # CST oposto
                                brw_tributacao = self.pool.get('fiscal.tributacao.encargos').browse(
                                        cr, uid, tributo_entrada_encargo_ids[0], context)
                                cst_oposto_id = brw_tributacao and \
                                                brw_tributacao.cst_id.fiscst_cst_oposto_id.id
                                if not cst_oposto_id:
                                    if brw_tributacao.cst_id.fiscst_tipo_movimento in 'AE':
                                        cst_oposto_id = brw_tributacao.cst_id.id

                                # Recupera tributacao com CST oposto
                                entrada_encargo_ids = \
                                    self.pool.get('fiscal.tributacao').search(
                                        cr, uid, [('cst_id', '=', cst_oposto_id),
                                                  ('encargo_id', '=', enc_id)],
                                        context=context)
                                if entrada_encargo_ids:
                                    ids_tri.append(entrada_encargo_ids[0])
                            elif tributo_entrada_encargo_ids:
                                tributacao_entrada_ids.append(tributo_entrada_encargo_ids[0])
                    else:
                        tributacao_entrada_ids = obj_fiscal_tributacao_encargos.search(
                            cr, uid, [('produto_id', '=', produto_id),
                                      ('tipo_tributacao', '=', tipo_tributacao)], context=context)
                    if tributacao_entrada_ids:
                        tributos_entrada_dados = obj_fiscal_tributacao_encargos.read(
                            cr, uid, tributacao_entrada_ids, ['tributacao_id', 'encargo_id'],
                            context=context)
                        for t in tributos_entrada_dados:
                            if context.get('buscar_tributacao_devolucao'):
                                if t['encargo_id'][1].upper() in ('PIS', 'COFINS', 'IPI'):
                                    tributo_encargo_ids = obj_fiscal_tributacao_encargos.search(
                                        cr, uid, [('produto_id', '=', produto_id),
                                                  ('tipo_tributacao', '=', 'T'),
                                                  ('encargo_id', '=', t['encargo_id'][0])],
                                        context=context)
                                    tributos_saida_dados = obj_fiscal_tributacao_encargos.read(
                                        cr, uid, tributo_encargo_ids, ['tributacao_id'],
                                        context=context)
                                    for t in tributos_saida_dados:
                                        ids_tri.append(t['tributacao_id'][0])
                                else:
                                    ids_tri.append(t['tributacao_id'][0])
                            else:
                                ids_tri.append(t['tributacao_id'][0])
                        context.update({'busca_fiscal_tributacao': True})
        elif context.get('proprio'):
            # Busca especifica por encargos eh utilizada quando for filtrar pelos encargos
            # aplicados a natureza da operacao
            if encargo_ids:
                sql = """SELECT id, tipo_tributacao, encargo_id
                         FROM fiscal_tributacao_encargos
                         WHERE produto_id = %s
                           AND tipo_tributacao in ('%s', 'T')
                           AND encargo_id in (%s)
                      """ % (produto_id, tipo_tributacao, ','.join(map(lambda x: '%s' % x,
                                                                       encargo_ids)))
                cr.execute(sql)
                dados_trib = cr.dictfetchall()
                # deixa os ids das tributacoes do tipo passado caso tenha, senao usa do dipo 'T'
                tributacao_tipo_tributacao = filter(
                    lambda iit: iit.get('tipo_tributacao') == tipo_tributacao, dados_trib)
                usados_encargo_ids = []
                if tributacao_tipo_tributacao:
                    usados_encargo_ids = map(lambda it: it.get('encargo_id'),
                                             tributacao_tipo_tributacao)
                    map(lambda it: ids_tri.append(it.get('id')), tributacao_tipo_tributacao)
                for trib in filter(lambda iit: iit.get('tipo_tributacao') != tipo_tributacao
                                   and iit.get('encargo_id') not in usados_encargo_ids,
                                   dados_trib):
                    ids_tri.append(trib.get('id'))
            else:
                ids_tri = obj_fiscal_tributacao_encargos.search(
                    cr, uid, [('produto_id', '=', produto_id),
                              ('tipo_tributacao', '=', tipo_tributacao)], context=context)
            if not ids_tri:
                ids_tri = obj_fiscal_tributacao_encargos.search(
                    cr, uid, [('produto_id', '=', produto_id),
                              ('tipo_tributacao', '=', 'T')], context=context)

        return ids_tri

    def calcular_imposto(self, cr, uid, produto_id, encargo_id=False, dados_destinatario={},
                         tipo_tributacao='T', vr_produto=0, encargo_ids=[], context=None,
                         dados_natureza={}, vr_desconto=0):
        """
        Calcula impostos para o produto informado retornando os valores de cada encargo. Onde o
        ´tipo_tributacao´ pode ser 'T'=Todas/Consumidor final, 'P'=Produtor rural,
        'C'='Contribuinte' ou 'E'=Entradas.
        """
        context = context and context.copy() or {}
        res = []
        if not produto_id:
            return res

        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        obj_fiscal_cst = self.pool.get('fiscal.cst')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_fisnit = self.pool.get('fiscal.nf.item.tributacao')
        simples_nacional = obj_adm_pessoa.verificar_empresa_simples_nacional(cr, uid)
        cr.execute("""SELECT company_id, origem_mercadoria FROM estoque_produto WHERE id = %s""" % produto_id)
        _dados_produto = cr.dictfetchall()
        if _dados_produto and isinstance(_dados_produto, list):
            _dados_produto = _dados_produto[0]
        if _dados_produto and isinstance(_dados_produto, dict):
            empresa = _dados_produto.get('company_id')
            _origem_mercadoria = _dados_produto.get('origem_mercadoria')
        if not empresa:
            empresa = 0

        # Se nao for passado o valor do produto, busca
        if vr_produto == 0 and not context.get('onchange_terceiro') and not context.get(
                'nota_referenciada'):
            preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, produto_id, epc_company_id=empresa)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            vr_unitario = dados_preco_venda.get('preco_venda')
        else:
            vr_unitario = vr_produto

        context['dados_cfop'] = \
            context.get('dados_natureza') and context.get('dados_natureza').get('cfop_id') or {}
        ids_tri = self.calcular_tributos(
            cr, uid, produto_id, dados_destinatario, tipo_tributacao, encargo_ids, context)

        if not ids_tri:
            return res

        obj_fiscal_tributacao_simples_nacional = self.pool.get('fiscal.tributacao.simples.nacional')
        simples_id = False
        if simples_nacional:
            obj_fiscal_parametro = self.pool.get('fiscal.parametro')
            parametro_id = obj_fiscal_parametro.search(cr, uid, [], context=context)
            if not parametro_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao existe nenhum parametro fiscal cadastrado para a '
                                       'empresa!'))
            else:
                brw_par = obj_fiscal_parametro.browse(
                    cr, uid, parametro_id[0])
                receita_anual = brw_par.fispar_receita_anual or 0
                sublimite_estadual = brw_par.fispar_sublimite_estadual or 99999999
            simples_ids = obj_fiscal_tributacao_simples_nacional.search(
                cr, uid, [], context=context)
            if simples_ids:
                dados_simples = obj_fiscal_tributacao_simples_nacional.read(
                    cr, uid, simples_ids, ['fistsn_valor_faturamento'], context=context)
                simples_id = False
                list_receita = [(x['fistsn_valor_faturamento'], x['id']) for x in dados_simples]
                list_receita.sort()
                for r in list_receita:
                    if receita_anual <= r[0]:
                        simples_id = r[1]
                        break
                if not simples_id:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Nao existe tributacao cadastrada para "Simples '
                                           'Nacional" que se esteja na faixa de receita anual da '
                                           'empresa. Verifique o parametro do Fiscal.'))
            else:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao existe tributacao cadastrada para "Simples Nacional".'))
        csosn_id = False
        sql = """
              SELECT admpes_jur_tipo_atividade AS atividade
              FROM adm_pessoa            AS ap
              INNER JOIN res_company     AS rc ON rc.rescon_pessoa_id = ap.id
              INNER JOIN estoque_produto AS ep ON ep.company_id = rc.id
              WHERE ep.id = %s
              """ % produto_id
        cr.execute(sql)
        industria = cr.dictfetchall()[0].get('atividade') == 'I'
        _tem_modulo_ecf = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'ecf', context)
        # Se nao foi informado o encargo busca todos os encargos para o item
        if not encargo_id:
            sql = '''
                    SELECT  fte.id,
                            fte.modbcst,
                            fte.mva,
                            fte.fisten_aliquota_venda,
                            fte.tributacao_id
                    FROM fiscal_tributacao_encargos AS fte
                    LEFT JOIN fiscal_tributacao     AS ft ON ft.id = fte.tributacao_id
                    WHERE   fte.id IN (%s)
                  ''' % ','.join(map(lambda x: '%s' % x,  ids_tri))
            if _tem_modulo_ecf:
                sql = sql.replace('_tem_modulo_ecf', ', ft.tributacao_ecf         AS tributacao_na_ecf')
            else:
                sql.replace('_tem_modulo_ecf', '')
            cr.execute(sql)
            dados_tri_encargo = cr.dictfetchall()
            trib_encargo = {}
            map(lambda x: trib_encargo.update({x.get('id'):x}), dados_tri_encargo)
            tipo_movimento_cfop = ''
            if context.get('proprio') and not context.get('busca_fiscal_tributacao') and \
                    context.get('dados_natureza'):
                sql = '''
                    SELECT    movimento_tipo
                    FROM      fiscal_cfop_natureza AS fcn
                    LEFT JOIN fiscal_cfop          AS fc   ON fc.id =fcn.cfop_id
                    WHERE     fcn.id = %s
                 ''' % (context.get('dados_natureza').get('id'))
                cr.execute(sql)
                tipo_movimento_cfop = cr.dictfetchall()
                if tipo_movimento_cfop and isinstance(tipo_movimento_cfop, list):
                    tipo_movimento_cfop = tipo_movimento_cfop[0]
                if tipo_movimento_cfop and isinstance(tipo_movimento_cfop, dict):
                    tipo_movimento_cfop = tipo_movimento_cfop.get('movimento_tipo')

            for i in ids_tri:
                # Tipo de tributacao diferente de entrada por fornecedor
                if context.get('proprio') and not context.get('busca_fiscal_tributacao'):
                    if tipo_tributacao == 'E':
                        cst_id = False
                        sql = '''
                                SELECT  cst_id, encargo_id
                                FROM    fiscal_tributacao
                                WHERE   id = %s
                              ''' % (i)
                        cr.execute(sql)
                        fiscal_trib_dados = cr.dictfetchall()
                        if fiscal_trib_dados and isinstance(fiscal_trib_dados, list):
                            fiscal_trib_dados = fiscal_trib_dados[0]
                        if isinstance(fiscal_trib_dados, dict):
                            cst_id = fiscal_trib_dados.get('cst_id')
                            cst_id = cst_id[0] if isinstance(cst_id, (list, tuple)) else cst_id
                            encargo_id = fiscal_trib_dados.get('encargo_id')
                            encargo_id = encargo_id[0] if isinstance(encargo_id, (list, tuple)) \
                                else encargo_id

                        dados_tributacao_encargo = {}
                        if tipo_movimento_cfop == 'D' and context.get('nota_referenciada'):
                            sql = '''
                                    SELECT  id, modbcst, mva, fisten_aliquota_venda, tributacao_id
                                    FROM    fiscal_tributacao_encargos
                                    WHERE   cst_id =  %s
                                      AND   tributacao_id = %s
                                      AND   produto_id = %s
                                      AND   encargo_id = %s
                                      AND   tipo_tributacao != 'E'
                                  ''' % (cst_id, i, produto_id, encargo_id)
                            cr.execute(sql)
                            dados_tributacao_encargo = cr.dictfetchall()
                            base = obj_fiscal_tributacao.browse(cr, uid, i, context).base
                            aliquota = obj_fiscal_tributacao.browse(cr, uid, i, context).aliquota
                            cst_oposto_id, tributacao_oposta_id = self.pool.get('fiscal.utils').\
                                get_cst_oposto(cr, uid, cst_id, i, encargo_id, base, aliquota,
                                               context)
                            if dados_tributacao_encargo:
                                dados_tributacao_encargo[0]['tributacao_id'] = tributacao_oposta_id
                                dados_tributacao_encargo[0]['cst_id'] = cst_id
                            else:
                                break

                        else:
                            if not dados_tributacao_encargo:
                                sql = '''
                                        SELECT  id, modbcst, mva, fisten_aliquota_venda, tributacao_id
                                        FROM    fiscal_tributacao_encargos
                                        WHERE   cst_id =  %s
                                          AND   tributacao_id = %s
                                          AND   produto_id = %s
                                          AND   encargo_id = %s
                                          AND   tipo_tributacao = 'E'
                                      ''' % (cst_id, i, produto_id, encargo_id)
                                cr.execute(sql)
                                dados_tributacao_encargo = cr.dictfetchall()

                        if not dados_tributacao_encargo:
                            sql = '''
                                    SELECT     fte.id, 
                                               fte.modbcst, 
                                               fte.mva, 
                                               fte.fisten_aliquota_venda,
                                               fte.tributacao_id
                                    FROM       fiscal_tributacao_encargos AS fte
                                    INNER JOIN fiscal_cst                 AS fcst
                                            ON fcst.id = fte.cst_id
                                    WHERE      fte.cst_id =  %s
                                      AND      fte.tributacao_id = %s
                                      AND      fte.produto_id = %s
                                      AND      fte.encargo_id = %s
                                      AND      fte.tipo_tributacao = 'T'
                                      AND      fcst.fiscst_tipo_movimento = 'A' -- Somente para NF de entrada
                                  ''' % (cst_id, i, produto_id, encargo_id)
                            cr.execute(sql)
                            dados_tributacao_encargo = cr.dictfetchall()
                    else:
                        dados_tributacao_encargo = trib_encargo.get(i)

                    if dados_tributacao_encargo and isinstance(dados_tributacao_encargo, list):
                        dados_tributacao_encargo = dados_tributacao_encargo[0]

                    modbcst = dados_tributacao_encargo and dados_tributacao_encargo.get('modbcst')
                    if _tem_modulo_ecf:
                        tributacao_na_ecf = \
                            trib_encargo.setdefault(i, dict()).get('tributacao_na_ecf') or False
                    else:
                        tributacao_na_ecf = False
                    mva = 0
                    aliquota_venda = 0
                    if dados_tributacao_encargo:
                        mva = dados_tributacao_encargo.get('mva') or 0
                        aliquota_venda = dados_tributacao_encargo.get('fisten_aliquota_venda') or 0
                        id_t = dados_tributacao_encargo.get('tributacao_id')
                    else:
                        id_t = i
                else:
                    # Nota fiscal de terceiro
                    id_t = i
                    modbcst = ''
                    tributacao_na_ecf = False
                    encargo_tributacao_id = obj_fiscal_tributacao.read(cr, uid, id_t, ['encargo_id'],
                                           context=context, load='_classic_write').get('encargo_id')
                    tributacao_enc_ids = obj_fiscal_tributacao_encargos.search(
                        cr, uid, [('produto_id', '=', produto_id),
                                  ('encargo_id', '=', encargo_tributacao_id),
                                  ('tipo_tributacao', '=', 'E')], context=context)
                    if not tributacao_enc_ids:
                        tributacao_enc_ids = obj_fiscal_tributacao_encargos.search(
                            cr, uid, [('produto_id', '=', produto_id),
                                      ('encargo_id', '=', encargo_tributacao_id),
                                      ('tipo_tributacao', '=', 'T')], context=context)
                    if tributacao_enc_ids:
                        browse_tributacao_encargos = obj_fiscal_tributacao_encargos.browse(
                            cr, uid, tributacao_enc_ids[0], context)
                        mva = browse_tributacao_encargos.mva or 0
                        aliquota_venda = browse_tributacao_encargos.fisten_aliquota_venda or 0
                        modbcst = browse_tributacao_encargos.modbcst or ''
                    else:
                        mva = 0
                        aliquota_venda = 0

                if id_t:
                    sql = '''
                            SELECT ft.aliquota,
                                   ft.base,
                                   ft.dispositivo_fiscal,
                                   ft.encargo_id,
                                   ft.cst_id,
                                   fe.name,
                                   fe.sigla,
                                   fc.tipo
                            FROM fiscal_tributacao   AS ft
                            LEFT JOIN fiscal_cst     AS fc ON fc.id = ft.cst_id
                            LEFT JOIN fiscal_encargo AS fe on fe.id = ft.encargo_id
                            WHERE ft.id = %(id_t)s
                          ''' % locals()
                    if context.get('entrada_terceiros') or not context.get('nota_fiscal_saida') \
                            and (context.get('tipo_nota') and context.get('tipo_nota') != 'S'):
                        sql += """
                              AND fc.fiscst_tipo_movimento IN ('E', 'A')
                          """
                    cr.execute(sql)
                    dados_tributacao = cr.dictfetchall()
                    if not dados_tributacao:
                        continue

                    dados_tributacao = dados_tributacao[0]
                    id_enc = dados_tributacao.get('encargo_id')
                    sigla_encargo = dados_tributacao.get('sigla')
                    nome_enc = dados_tributacao.get('name')
                    id_cst = dados_tributacao.get('cst_id')
                    tipo_cst = dados_tributacao.get('tipo')
                    if simples_nacional and receita_anual < sublimite_estadual:
                        csosn_id = obj_fiscal_cst.browse(cr, uid, id_cst).fiscst_csosn_contribuinte_id
                        if csosn_id:
                            csosn_id = csosn_id.id
                        else:
                            if nome_enc.upper() == 'ICMS':
                                cst_codigo = obj_fiscal_cst.browse(cr, uid, id_cst).codigo
                                if context.get('valida_cupom'):
                                    raise Exception(_('Cupom CSON-') +
                                                    _('Nao foi informado o CSOSN para o CST ' +
                                                       cst_codigo + ' do ICMS.'))

                                raise osv.except_osv(_('Atencao!'),
                                                     _('Nao foi informado o CSOSN para o CST ' +
                                                       cst_codigo + ' do ICMS.'))

                    aliquota = dados_tributacao.get('aliquota') or 0
                    base = dados_tributacao.get('base')
                    dispositivo_fiscal = dados_tributacao.get('dispositivo_fiscal')
                    valor_base = obj_fisnit.fisnit_calcula_vr_base_calculo(
                        vr_unitario, percentual_base=base)

                    icms_ids = self.pool.get('fiscal.encargo').search(
                        cr, uid, [('sigla', 'ilike', 'ICMS')], context=context)
                    icms_id = icms_ids and icms_ids[0]

                    if dados_tributacao.get('encargo_id') == icms_id:  # ICMS
                        if tipo_cst in ('T', 'D'):  # Somente ICMS tributado ou Diferido
                            if dados_destinatario:
                                dados_uf_dest = self.pool.get('adm.pessoa.endereco').read(
                                    cr, uid, dados_destinatario.get('destinatario_endereco_id'),
                                    ['uf_id'], context, '_classic_write') if dados_destinatario.get(
                                    'destinatario_endereco_id') else {}
                                uf_id = dados_uf_dest.get('uf_id') or False
                            else:
                                uf_id = False
                            dados_natureza_cfop = False
                            if dados_natureza:
                                dados_natureza_cfop = self.pool.get('fiscal.cfop.natureza').read(
                                    cr, uid, dados_natureza.get('id'), ['cfop_id'], context,
                                    '_classic_write')

                            cfop_id = dados_natureza_cfop and dados_natureza_cfop.get('cfop_id')
                            dados_cfop = cfop_id and self.pool.get('fiscal.cfop').read(
                                cr, uid, cfop_id, ['codigo'], context)
                            cfop = dados_cfop and dados_cfop.get('codigo')
                            if dados_destinatario:
                                contribuinte = \
                                    self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                                        cr, uid, dados_destinatario.get('id'), context) == 'C'
                            else:
                                contribuinte = False

                            # Operacao interestadual p/ contribuinte
                            if cfop and cfop[:1] == '6' and uf_id:
                                op_inter_ids = \
                                    self.pool.get('fiscal.tributacao.interestadual').search(
                                        cr, uid, [('fistin_encargo', '=', icms_id),
                                                  ('fistin_uf', '=', uf_id)], context=context)
                                op_inter_id = op_inter_ids and op_inter_ids[0]
                                trib_inter_dados = op_inter_id and self.pool.get(
                                    'fiscal.tributacao.interestadual').read(
                                    cr, uid, op_inter_id, ['fistin_cod_tribut_interestadual_nac',
                                                           'fistin_cod_tribut_interestadual_imp'],
                                    context, '_classic_write') or {}
                                if _origem_mercadoria in ('1','2','6','7'):
                                    id_t = trib_inter_dados.get('fistin_cod_tribut_interestadual_imp') or id_t
                                else:
                                    id_t = trib_inter_dados.get('fistin_cod_tribut_interestadual_nac') or id_t
                                dados_trib = \
                                    id_t and self.pool.get('fiscal.tributacao').read(
                                        cr, uid, id_t, ['aliquota', 'cst_id'], context, '_classic_write')
                                aliquota = dados_trib and dados_trib.get('aliquota') or aliquota
                                id_cst = dados_trib and dados_trib.get('cst_id') or id_cst
                                cst_inter = id_cst and obj_fiscal_cst.read(cr, uid, id_cst, ['tipo'], context,
                                                                           '_classic_write',)
                                tipo_cst = cst_inter and cst_inter.get('tipo') or tipo_cst

                    valor_imposto = obj_fisnit.fisnit_calcula_vr_encargo(valor_base, aliquota)

                    # ICMS desonerado
                    vr_desonerado = 0
                    if dados_tributacao.get('encargo_id') == icms_id:
                        if dados_natureza and dados_natureza.get('fisnat_icms_desonerado') and \
                                aliquota:
                            vr_desonerado = valor_imposto
                        is_icms = True
                        if context.get('gerar_nf_cupom'):
                            mva = modbcst = False
                    else:
                        is_icms = False
                    if vr_desonerado and context.get('entrada_terceiros'):
                        vr_desonerado = 0
                    values = {
                        'encargo_id': id_enc, 'nome_encargo': nome_enc, 'sigla': sigla_encargo,
                        'percentual_base': base, 'aliquota': aliquota, 'valor_base': valor_base,
                        'valor_imposto': valor_imposto, 'cst_id': id_cst, 'tipo_cst': tipo_cst,
                        'tributacao_id': id_t, 'fisnit_vr_icms_desonerado': vr_desonerado,
                        'modbcst': is_icms and modbcst, 'dispositivo_fiscal': dispositivo_fiscal,
                        'csosn_id': csosn_id or False, 'mva': is_icms and mva,
                        'aliquota_venda': is_icms and aliquota_venda
                    }
                    if not context.get('terceiro'):
                        values.update(
                            self.pool.get('fiscal.nf.item.tributacao').fisnit_calcular_vr_icms_st_item(
                                valor_base, aliquota, aliquota_venda, mva))
                    if _tem_modulo_ecf:
                        values['tributacao_na_ecf'] = tributacao_na_ecf
                    res.append(values)
        else:
            # Se foi informado o encargo...
            for i in ids_tri:
                if context.get('proprio') and not context.get('busca_fiscal_tributacao'):
                    modbcst = obj_fiscal_tributacao_encargos.read(
                        cr, uid, i, ['modbcst'], context=context)['modbcst']
                    if _tem_modulo_ecf:
                        try:
                            tributacao_na_ecf = obj_fiscal_tributacao_encargos.browse(
                                cr, uid, i).tributacao_na_ecf
                        except:
                            tributacao_na_ecf = False
                    else:
                        tributacao_na_ecf = False
                    mva = obj_fiscal_tributacao_encargos.browse(cr, uid, i, context).mva
                    aliquota_venda = obj_fiscal_tributacao_encargos.browse(
                        cr, uid, i, context).fisten_aliquota_venda
                    id_t = obj_fiscal_tributacao_encargos.browse(
                        cr, uid, i, context).tributacao_id.id
                else:
                    id_t = i
                    modbcst = ''
                    tributacao_na_ecf = False
                    browse_tributacao = obj_fiscal_tributacao.browse(cr, uid, id_t)
                    encargo_tributacao_id = browse_tributacao.encargo_id.id
                    tributacao_enc_ids = obj_fiscal_tributacao_encargos.search(
                        cr, uid, [('produto_id', '=', produto_id),
                                  ('encargo_id', '=', encargo_tributacao_id),
                                  ('tipo_tributacao', '=', 'E')], context=context)
                    if not tributacao_enc_ids:
                        tributacao_enc_ids = obj_fiscal_tributacao_encargos.search(
                            cr, uid, [('produto_id', '=', produto_id),
                                      ('encargo_id', '=', encargo_tributacao_id),
                                      ('tipo_tributacao', '=', 'T')], context=context)
                    if tributacao_enc_ids:
                        browse_tributacao_encargos = obj_fiscal_tributacao_encargos.browse(
                            cr, uid, tributacao_enc_ids[0], context)
                        mva = browse_tributacao_encargos.mva or 0
                        aliquota_venda = browse_tributacao_encargos.fisten_aliquota_venda or 0
                        modbcst = browse_tributacao_encargos.modbcst or ''
                        tributacao_na_ecf = browse_tributacao_encargos.tributacao_na_ecf
                    else:
                        mva = 0
                        aliquota_venda = 0

                browse_tributacao = obj_fiscal_tributacao.browse(cr, uid, id_t, context)
                id_enc = browse_tributacao.encargo_id.id
                if encargo_id == id_enc:
                    sigla_encargo = browse_tributacao.encargo_id.sigla
                    nome_enc = browse_tributacao.encargo_id.name
                    id_cst = browse_tributacao.cst_id.id
                    tipo_cst = browse_tributacao.cst_id.tipo
                    if simples_nacional and receita_anual < sublimite_estadual:
                        csosn_id = obj_fiscal_cst.browse(
                            cr, uid, id_cst, context).fiscst_csosn_contribuinte_id  # para efeito informativo
                        if csosn_id:
                            csosn_id = csosn_id.id
                        else:
                            if nome_enc.upper() == 'ICMS':
                                cst_codigo = obj_fiscal_cst.browse(cr, uid, id_cst).codigo
                                raise osv.except_osv(_('Atencao!'),
                                                     _('Nao foi informado o CSOSN para o CST ' +
                                                       cst_codigo + ' do ICMS.'))

                    aliquota = browse_tributacao.aliquota
                    base = browse_tributacao.base
                    dispositivo_fiscal = browse_tributacao.dispositivo_fiscal
                    valor_base = obj_fisnit.fisnit_calcula_vr_base_calculo(
                        vr_unitario, percentual_base=base, vr_desconto=vr_desconto)

                    icms_ids = self.pool.get('fiscal.encargo').search(
                        cr, uid, [('sigla', 'ilike', 'ICMS')], context=context)
                    icms_id = icms_ids and icms_ids[0]

                    if browse_tributacao.encargo_id.id == icms_id:  # ICMS
                        if tipo_cst == 'T':  # Somente ICMS tributado
                            if dados_destinatario.get('destinatario_endereco_id'):
                                dados_uf_dest = self.pool.get('adm.pessoa.endereco').read(
                                    cr, uid, dados_destinatario.get('destinatario_endereco_id'),
                                    ['uf_id'], context, '_classic_write')
                                uf_id = dados_uf_dest and dados_uf_dest.get('uf_id')
                                dados_natureza_cfop = self.pool.get('fiscal.cfop.natureza').read(
                                    cr, uid, dados_natureza.get('id'), ['cfop_id'], context,
                                    '_classic_write')
                                cfop_id = dados_natureza_cfop and dados_natureza_cfop.get('cfop_id')
                                dados_cfop = cfop_id and self.pool.get('fiscal.cfop').read(
                                    cr, uid, cfop_id, ['codigo'], context)
                                cfop = dados_cfop and dados_cfop.get('codigo')
                                if cfop[:1] == '6':  # Operacao interestadual
                                    op_inter_ids = \
                                        self.pool.get('fiscal.tributacao.interestadual').search(
                                            cr, uid, [('fistin_encargo', '=', icms_id),
                                                      ('fistin_uf', '=', uf_id)], context=context)
                                    op_inter_id = op_inter_ids and op_inter_ids[0]
                                    trib_inter_dados = op_inter_id and self.pool.get(
                                        'fiscal.tributacao.interestadual').read(
                                        cr, uid, op_inter_id,
                                        ['fistin_cod_tribut_interestadual_nac'], context,
                                        '_classic_write') or {}
                                    id_t = \
                                        trib_inter_dados.get('fistin_cod_tribut_interestadual_nac')
                                    dados_trib = \
                                        id_t and self.pool.get('fiscal.tributacao').read(
                                            cr, uid, id_t, ['aliquota'], context)
                                    aliquota = dados_trib and dados_trib.get('aliquota') or aliquota

                    valor_imposto = obj_fisnit.fisnit_calcula_vr_encargo(valor_base, aliquota)

                    # ICMS desonerado
                    vr_desonerado = 0
                    if encargo_id == icms_id:
                        if dados_natureza.get('fisnat_icms_desonerado'):
                            cst40_ids = self.pool.get('fiscal.cst').search(
                                cr, uid, [('codigo', '=', '40')], context=context)
                            cst40_id = cst40_ids and cst40_ids[0]
                            tipo_cst = cst40_id and self.pool.get('fiscal.cst').read(
                                cr, uid, cst40_id, ['tipo'], context)
                            trib40_ids = self.pool.get('fiscal.tributacao').search(
                                cr, uid, [('encargo_id', '=', icms_id), ('cst_id', '=', cst40_id)],
                                context=context)
                            id_t = trib40_ids.__len__() == 1 and trib40_ids[0] or False
                            id_cst = cst40_id
                            vr_desonerado = valor_imposto
                        is_icms = True
                    else:
                        is_icms = False

                    values = {
                        'encargo_id': id_enc, 'nome_encargo': nome_enc, 'sigla': sigla_encargo,
                        'percentual_base': base, 'aliquota': aliquota, 'valor_base': valor_base,
                        'valor_imposto': valor_imposto, 'cst_id': id_cst, 'tipo_cst': tipo_cst,
                        'tributacao_id': id_t, 'fisnit_vr_icms_desonerado': vr_desonerado,
                        'modbcst': is_icms and modbcst, 'dispositivo_fiscal': dispositivo_fiscal,
                        'csosn_id': csosn_id or False, 'mva': is_icms and mva,
                        'aliquota_venda': is_icms and aliquota_venda
                    }
                    values.update(
                        self.pool.get('fiscal.nf.item.tributacao').fisnit_calcular_vr_icms_st_item(
                            valor_base, aliquota, aliquota_venda, mva))
                    if _tem_modulo_ecf:
                        values['tributacao_na_ecf']= tributacao_na_ecf
                    res.append(values)

        return res

    def get_cst_oposto(self, cr, uid, cst_id, tributacao_id, encargo_id, base, aliquota,
                       context=None):
        """Retorna o CST Oposto tributacao caso o tipo da nota seja de devolucao
           e o tipo de movimento do CST saja diferente de Ambos"""
        context = context and context.copy() or {}
        brw_cst = self.pool.get('fiscal.cst').browse(cr, uid, cst_id)
        if brw_cst.fiscst_tipo_movimento == 'A':
            return cst_id, tributacao_id
        cst_oposto_id = brw_cst.fiscst_cst_oposto_id
        if not cst_oposto_id:
            codigo = brw_cst.codigo
            sigla_encargo = brw_cst.encargo_id.sigla
            cst = sigla_encargo + ' ' + codigo
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha CST oposto cadastrado para o CST: %s' % cst))
        vals = self.pool.get('fiscal.tributacao').read(cr, uid, tributacao_id, context=context)
        if not base:
            base = vals.get('base')
        args = [('cst_id', '=', cst_oposto_id.id), ('encargo_id', '=', encargo_id),
                ('base', '=', base), ('aliquota', '=', aliquota)]
        tributacao_ids = self.pool.get('fiscal.tributacao').search(cr, uid, args, context=context)
        if not tributacao_ids:
            vals['cst_id'] = cst_oposto_id.id
            vals['encargo_id'] = encargo_id
            if vals.get('fistri_cst_tipo_movimento') == 'S':
                vals['fistri_cst_tipo_movimento'] = 'E'
            else:
                vals['fistri_cst_tipo_movimento'] = 'S'
            vals['produto_tributacao_ids'] = ''
            vals['aliquota'] = aliquota
            vals['name'] = 'CST %s %s' % (brw_cst.fiscst_cst_oposto_id.codigo,
                                          brw_cst.encargo_id.sigla)
            vals.pop('id')
            tributacao_ids = self.pool.get('fiscal.tributacao').create(
                cr, uid, vals, context=context)
        tributacao_id = tributacao_ids
        if isinstance(tributacao_id, list):
            tributacao_id=tributacao_id[0]
        return cst_oposto_id.id, tributacao_id

    @log_debug
    def verifica_encargos_obrigatorio_empresa(self, cr, uid, encargos_ids, context=None):

        '''Rertorna encargos faltantes. Return [(id,sigla)]'''
        context = context and context.copy() or {}
        res = []

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, tabela='fiscal.cfop.natureza.encargos', context=context)
        obj_encargo_empresa = self.pool.get('fiscal.encargo.empresa')
        #primeiro busca todos os encargos para a empresa corrente que sao obrigatorios
        encargo_emp_ids = obj_encargo_empresa.search(cr, uid, [('company_id', '=', company_id),
                                                                          ('fiseem_encargo_obrigatorio', '=', True)],
                                                                context=context)
        empresa_encargo_ids = []
        for e in obj_encargo_empresa.read(cr, uid, encargo_emp_ids, ['encargo_id'], context,
                                          '_classic_write'):#separa encargos obrigatorios
            empresa_encargo_ids.append(e.get('encargo_id'))
        # Verifica diferenca entre lista de encargos e os encargos obrigatorios para a empresa corrente
        encargos_faltantes = list(set.difference(set(empresa_encargo_ids), set(encargos_ids)))
        if encargos_faltantes:
            for enc in self.pool.get('fiscal.encargo').read(cr, uid, encargos_faltantes, ['sigla'],
                                                            context):
                res.append((enc.get('id'), enc.get('sigla')))

        return res

    @log_debug
    def fisutl_encargo_item_fora_natureza(self, cr, user, item_trib_ids, nat_encargo_ids,
                                     dict_encargos, dict_trib, context=None):
        """
        Retorna mesagem de erro depois de fazer as devidas verificacoes
        .Verifica se os encargos do item estao cadastrados na natureza

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item
        :param natureza: o ID da natureza da operacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type natureza: long
        :type context: dict
        :return: mensagem de erro ou mesagem vazia se nao houver erro
        :rtype: str
        """
        context = context and context.copy() or {}
        item_encagos = []
        for i in item_trib_ids:
            trib = dict_trib.get(i)
            encargo = dict_encargos.get(trib.get('encargo_id'))
            item_encagos.append((encargo.get('id'), encargo.get('sigla')))

        natureza_encargos = []
        for i in nat_encargo_ids:
            encargo = dict_encargos.get(i)
            natureza_encargos.append((encargo.get('id'), encargo.get('sigla')))

        encargos = list(set.difference(set(item_encagos), set(natureza_encargos)))
        msg = ''
        if encargos:
            msg = _('Os seguintes encargos nao estao cadastrados na Natureza:\n %s') % \
                  '\n'.join(map(lambda x: x[1], encargos))
        return msg

    @log_debug
    def fisutl_encargo_faltante_item(self, cr, user, item_trib_ids, produto, dict_encargos,
                                     dict_trib, company_id, context=None):
        """
        Retorna mesagem de erro depois de fazer as devidas verificacoes
        .Empresa sem encargos cadastrados
        .Se item de servico, verifica se tem ISS
        .Se o item possui os encargos obrigatorios da empresa
        .Se os encargos do item estao cadastrados na empresa

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item
        :param company_id: o ID da empresa 
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type company_id: long
        :type context: dict
        :return: mensagem de erro ou mesagem vazia se nao houver erro
        :rtype: str
        """
        context = context and context.copy() or {}
        obj_encargo_emp = self.pool.get('fiscal.encargo.empresa')
        context['sig_force_company'] = True
        encargo_emp_ids = obj_encargo_emp.search(cr, user, [('company_id', '=', company_id)],
                                                 context=context)
        if not encargo_emp_ids:
            return _('Nenhum encargo cadastrado para esta empresa em Fiscal->Cadastro->Empresa')

        encagos_empresa = []
        encagos_obrigatorios = []
        campos = ['encargo_id', 'fiseem_encargo_obrigatorio']
        for i in obj_encargo_emp.read(cr, user, encargo_emp_ids, campos, context, '_classic_write'):
            encargo = dict_encargos.get(i.get('encargo_id'))
            encagos_empresa.append((encargo.get('id'), encargo.get('sigla')))
            if i.get('fiseem_encargo_obrigatorio'):
                encagos_obrigatorios.append((encargo.get('id'), encargo.get('sigla')))

        item_encagos = []
        tem_iss = False
        for i in item_trib_ids:
            trib = dict_trib.get(i)
            encargo = dict_encargos.get(trib.get('encargo_id'))
            item_encagos.append((encargo.get('id'), encargo.get('sigla')))
            if encargo.get('sigla') == 'ISS':
                tem_iss = True

        if produto.get('tipo') == 'S' and not tem_iss:
            return _("O encargo ISS e obrigatorio para servico!")

        msg = ''
        # verifica se o item possui os encargos obrigatorios da empresa
        res = list(set.difference(set(encagos_obrigatorios), set(item_encagos)))
        if res:
            msg = _('Os seguintes encargos sao obrigatorios:\n%s') % \
                    '\n'.join(map(lambda x: x[1], res))

        # verifica se o item possui algum encargo que nao eh da empresa
        res = list(set.difference(set(item_encagos), set(encagos_empresa)))
        if res:
            if msg:
                msg += '\n'

            msg += _('Os seguintes encargos nao estao cadastrados nesta empresa:\n%s') % \
                  '\n'.join(map(lambda x: x[1], res))

        return msg

    @log_debug
    def fisutl_restricao_venda(self, cr, uid, dados_nf, context=None):
        context = context and context.copy() or {}
        self._tem_erro = False
        self._mensagem_erro = u''

        TIPO_DOCUMENTO = 'NFL'
        obj_venda_utils = self.pool.get('vendas.utils')
        id = dados_nf.get('id')
        pessoa_id = dados_nf.get('destinatario_id') or False
        state = dados_nf.get('state')

        venda_liberada = obj_venda_utils.venutl_documento_restricao_venda(cr, uid,
                              documento_id=id, pessoa_id=pessoa_id, documento_state=state,
            documento_tipo=TIPO_DOCUMENTO, context=context)

        if not venda_liberada:
            erro_venda = obj_venda_utils.venutl_get_erro()
            self._tem_erro = True
            self._mensagem_erro += erro_venda

        if self._tem_erro:
            return False
        else:
            return True

    @log_debug
    def fisutl_venda_liberacao_desconto(self, cr, uid, dados_nf, context=None):
        context = context and context.copy() or {}
        self._mensagem_erro = u''
        self._tem_erro = False

        # OBJETOS
        obj_liberar_desconto = self.pool.get('vendas.liberar.desconto')

        # NAME DOS PEDIDO_IDS
        dict_nf_names = self.fisutl_get_nf_names(cr, uid, dados_nf.get('id'), context)

        # CONTROLES
        erros_nf = u''
        info_nf_liberacao_desconto_criada = u''
        erro_nf_liberacao_desconto_nao_autorizada = u''

        for i in [dados_nf]:
            nf_id = i['id']
            nf_valor_total = i.get('vr_total_nf') or 0

            nf_name = (dict_nf_names.get(nf_id) or '') or _('** Nro NF nao encontrado **')

            # VALIDAÇÃO DE DESCONTOS DADOS NOS ITENS E NA CAPA.
            desconto_liberado = self.fisutl_validar_desconto_itens(
                cr, uid, nf_id, context=context
            )

            # *** Liberacao de Desconto ***
            liberacao_desconto_nf_ids = obj_liberar_desconto.search(
                cr, uid, [('venauv_nf_id', '=', nf_id)], context=context
            )

            if desconto_liberado:
                # SE NÃO HOUVER NENHUM PROBLEMA REFERENTE A DESCONTO E EXISTIR ALGUM REGISTRO DE LIBERACAO DE
                # DESCONTO, O MESMO SERÁ EXCLUIDO, POIS NÃO SERÁ MAIS NECESSARIO POR NÃO HAVER MAIS DESCONTO.
                if liberacao_desconto_nf_ids:
                    obj_liberar_desconto.unlink(cr, uid, liberacao_desconto_nf_ids, context)
                else:
                    continue
            else:
                if not liberacao_desconto_nf_ids:

                    erros_nf += self.fisutl_get_erro()

                    # CASO O DESCONTO EXCEDA O LIMITE NOS ITENS, O SISTEMA IRÁ CRIAR AUTOMATICAMENTE UMA LIBERAÇÃO
                    # DE DESCONTO NÃO AUTORIZADA, PARA QUE UM SUPERVISOR AUTORIZE.
                    vals_liberar_desconto = {
                        'venauv_nf_id': nf_id,
                        'venauv_autorizado': False,
                    }
                    obj_liberar_desconto.create(cr, uid, vals_liberar_desconto, context)
                    # NÃO RETIRAR O COMMIT, POIS HOUVE ERRO E SERÁ FEITO UM RAISE. POREM, É PRECISO
                    # CRIAR A LIBERAÇÃO DE DESCONTO.
                    cr.commit()
                    info_nf_liberacao_desconto_criada += '%s, ' % nf_name

                else:
                    args_liberacao_autorizada = [
                        ('venauv_autorizado', '=', True),
                        ('venauv_valor_liquido', '=', round(nf_valor_total, 2)),
                        ('id', 'in', liberacao_desconto_nf_ids)
                    ]
                    desconto_liberado_valor_liquido_ids = obj_liberar_desconto.search(
                        cr, uid, args_liberacao_autorizada, context=context
                    )
                    if desconto_liberado_valor_liquido_ids:
                        continue
                    else:
                        erros_nf += self.fisutl_get_erro()

                        erro_nf_liberacao_desconto_nao_autorizada += '%s, ' % nf_name
                        values = {
                            'venauv_autorizado': False
                        }
                        obj_liberar_desconto.write(cr, uid, liberacao_desconto_nf_ids, values, context)
                        cr.commit()
        if info_nf_liberacao_desconto_criada:
            erros_nf += _('** [CRIADA] - Liberacao de Desconto criada para as notas fiscais: %s') \
                        % info_nf_liberacao_desconto_criada

        if erro_nf_liberacao_desconto_nao_autorizada:
            erros_nf += _('* [DESCONTO NAO AUTORIZADO] Notas fiscais: %s') % \
                        erro_nf_liberacao_desconto_nao_autorizada

        if erros_nf:
            self._tem_erro = True
            self._mensagem_erro = erros_nf
            return False
        else:
            return True

    @log_debug
    def fisutl_validar_desconto_itens(self, cr, uid, nf_ids, context=None):
        """
        Funcao Valida:
            * Se o desconto dado aos itens e maior que o permitido.
        Retorna:
            * True - Descontos dentro do permitido. NF OK
            * False - Descontos acima do limite.
        """
        context = context and context.copy() or {}
        self._mensagem_erro = u''
        self._tem_erro = False

        if isinstance(nf_ids, (int, long)):
            nf_ids = [nf_ids]

        # OBJETOS
        obj_adm_utils = self.pool.get('adm.utils')
        obj_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_estoque_produto = self.pool.get('estoque.produto')
        load = '_classic_write'

        # NAME DAS NF_IDS
        dict_nf_names = self.fisutl_get_nf_names(cr, uid, nf_ids, context)

        # CONTROLE
        erro_nf_item_desconto = u''

        erro_nf_condicao_desconto = u''

        # CONTROLE CONDICAO PAGAMENTO SEM PERMISSAO DE DESCONTO
        nao_permite_desconto_condicao_ids = list()

        campos = ['vr_desconto', 'estoque_movimento_itens_ids', 'nf_condicao_ids']
        dados_nf = obj_nf_capa.read(cr, uid, nf_ids, campos, context)
        itens_ids = []
        condicoes_ids = []
        for i in dados_nf:
            itens_ids.extend(i.get('estoque_movimento_itens_ids'))
            condicoes_ids.extend(i.get('nf_condicao_ids'))

        dict_itens = {}
        campos = ['produto_id', 'vr_total_item', 'vr_desconto']
        map(lambda x: dict_itens.update({x.get('id'): x}),
            self.pool.get('estoque.movimento.itens').read(cr, uid, itens_ids, campos,
                                                          context=context, load=load))
        produto_ids = map(lambda x: x[1].get('produto_id'), dict_itens.items())
        descontos_prods = obj_estoque_produto.estpro_get_desconto_maximo(cr, uid, produto_ids,
                                                                         context)
        dict_item_name = dict(obj_estoque_produto.name_get(cr, uid, produto_ids, context)) or {}

        dict_condicoes = {}
        cond_forma_ids = []
        campos = ['condicao_forma_id']
        for i in self.pool.get('fiscal.condicao.pagamento').read(cr, uid, condicoes_ids, campos,
                                                                 context, load):
            cond_forma_ids.append(i.get('condicao_forma_id'))
            dict_condicoes.update({i.get('id'): i.get('condicao_forma_id')})

        dict_cond_forma = {}
        campos = ['admcof_aceita_desconto', 'name']
        map(lambda x: dict_cond_forma.update({x.get('id'): x}),
            self.pool.get('adm.condicao.forma').read(cr, uid, cond_forma_ids, campos, context))

        for nf in dados_nf:

            # CONTROLE ITEM
            erro_item_desconto = u''

            erro_condicao_desconto = u''

            # DADOS DA NOTA
            NF_ID = nf.get('id')
            NF_DESCONTO = nf.get('vr_desconto') or 0

            nf_name = (dict_nf_names.get(NF_ID) or '') or _('** Nro NF nao encontrado **')

            nf_itens_ids = nf.get('estoque_movimento_itens_ids')

            total_desconto_itens = 0

            if not nf_itens_ids:
                continue
            else:
                rateio_capa_itens = {}
                if NF_DESCONTO:
                    # SE HOUVER DESCONTO NA CAPA DO ORCAMENTO, RATEA-LO PARA O ITENS.
                    elemetos = map(lambda i: [i, dict_itens.get(i).get('vr_total_item') -
                                                 dict_itens.get(i).get('vr_desconto')], nf_itens_ids)
                    elementos_rateados = map(tuple, obj_adm_utils.efetuar_rateio(NF_DESCONTO, elemetos))
                    rateio_capa_itens = dict(elementos_rateados)

                # ITENS DA NF
                for i in nf_itens_ids:
                    item = dict_itens.get(i)
                    ITEM_NF_PRODUTO_ID = item.get('produto_id')
                    total_desconto_itens += item.get('vr_desconto') or 0
                    percentual_desconto_item = descontos_prods.get(ITEM_NF_PRODUTO_ID)
                    if percentual_desconto_item:
                        ITEM_NF_ID = i
                        ITEM_NF_VALOR_LIQUIDO = item.get('vr_total_item') or 0
                        ITEM_NF_DESCONTO = item.get('vr_desconto') or 0

                        valor_bruto = ITEM_NF_VALOR_LIQUIDO + ITEM_NF_DESCONTO
                        capa_desconto_rateado = rateio_capa_itens.get(ITEM_NF_ID) or 0
                        # DESCONTO TOTAL DADO NO ITEM (CAPA + DETALHE)
                        desconto_total_item = float(Decimal(str(
                            ITEM_NF_DESCONTO + capa_desconto_rateado)).quantize(
                            Decimal('.01'), rounding=ROUND_UP))

                        desconto_maximo_permitido = float(Decimal(str(
                            valor_bruto * percentual_desconto_item / 100)).quantize(
                            Decimal('.01'), rounding=ROUND_UP))
                        if desconto_total_item:
                            if desconto_total_item > desconto_maximo_permitido:
                                produto_name = (dict_item_name.get(ITEM_NF_PRODUTO_ID) or '').encode('utf-8') \
                                    or _('** Produto nao encontrado **')
                                erro_item_desconto += u'%s, ' % produto_name.decode('utf-8')

            if erro_item_desconto:
                erro_nf_item_desconto += _('%s - Itens: %s') % (nf_name, erro_item_desconto)

            if NF_DESCONTO or total_desconto_itens:
                names = []
                for i in nf.get('nf_condicao_ids'):
                    forma = dict_cond_forma.get(dict_condicoes.get(i))
                    if not forma.get('admcof_aceita_desconto'):
                        names.append(forma.get('name'))

                for name in names:
                    erro_condicao_desconto += u'%s, ' % name.decode('utf-8')

            if erro_condicao_desconto:
                erro_nf_condicao_desconto += _('%s - Condicoes: %s') % (nf_name,
                                                                        erro_condicao_desconto)

        if erro_nf_item_desconto:
            self._tem_erro = True
            self._mensagem_erro += _('* [DESCONTO ACIMA DO LIMITE]: %s ') % erro_nf_item_desconto

        if nao_permite_desconto_condicao_ids:
            self._tem_erro = True
            self._mensagem_erro += _('* [CONDICAO DE PAGAMENTO NAO PERMITE DESCONTO]: %s ') % \
                erro_nf_condicao_desconto

        if self._tem_erro:
            return False
        else:
            return True

    def fisutl_atualizar_tributacao_produtos(self, cr, uid, produto_dados, inclui_tributacao=False,
                                             context=None):
        """
        O metodo compara a tributacao cadastrada para os produtos com a tributacao fornecida pela
        Taxpertise, todos contidos em produto_dados. As diferencas sao atualizadas no cadastro dos
        produtos pelos dados da Taxpertise.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_dados: lista dos dados dos produtos com a tributacao cadastrada e a fornecida
        pela Taxpertise.
        :type produto_dados: list
        :param inclui_tributacao: Se verdadeiro, incluira na base de dados os dados referente a
        tributacao que nao foram encontrados. Se falso, apenas os atualizara, se encontrados.
        :type inclui_tributacao: bool
        :param context: dados contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        obj_fiscal_cst = self.pool.get('fiscal.cst')
        obj_fiscal_encargo = self.pool.get('fiscal.encargo')
        icms_ids = obj_fiscal_encargo.search(cr, uid, [('sigla', '=', 'ICMS')], context=context)
        pis_ids = obj_fiscal_encargo.search(cr, uid, [('sigla', '=', 'PIS')], context=context)
        cofins_ids = obj_fiscal_encargo.search(cr, uid, [('sigla', '=', 'COFINS')], context=context)
        for dado in produto_dados:
            produto_id = dado.get('estfit_produto_id')
            brw_estoque_produto = obj_estoque_produto.browse(cr, uid, produto_id, context)
            brw_fiscal_tributacao_encargos = brw_estoque_produto.tributacao_ids
            vals_produto = dict()
            list_tributacao_encargos = list()
            dict_tributacao_encargos_icms = dict()
            dict_tributacao_encargos_cont_icms = dict()
            dict_tributacao_encargos_pis_entrada = dict()
            dict_tributacao_encargos_pis_saida = dict()
            dict_tributacao_encargos_cofins_entrada = dict()
            dict_tributacao_encargos_cofins_saida = dict()
            args_tributacao_icms = [('encargo_id', 'in', icms_ids)]
            vals_tributacao_icms = {'encargo_id': icms_ids[0]}
            args_tributacao_cont_icms = [('encargo_id', 'in', icms_ids)]
            vals_tributacao_cont_icms = {'encargo_id': icms_ids[0]}
            args_tributacao_pis_entrada = [('encargo_id', 'in', pis_ids)]
            vals_tributacao_pis_entrada = {'encargo_id': pis_ids[0]}
            args_tributacao_pis_saida = [('encargo_id', 'in', pis_ids)]
            vals_tributacao_pis_saida = {'encargo_id': pis_ids[0]}
            args_tributacao_cofins_entrada = [('encargo_id', 'in', cofins_ids)]
            vals_tributacao_cofins_entrada = {'encargo_id': cofins_ids[0]}
            args_tributacao_cofins_saida = [('encargo_id', 'in', cofins_ids)]
            vals_tributacao_cofins_saida = {'encargo_id': cofins_ids[0]}

            if dado.get('estfit_novo_cest') and dado.get('estfit_cad_cest') != dado.get(
                    'estfit_novo_cest'):
                vals_produto['estpro_cest_id'] = dado.get('estfit_novo_cest_id')

            if dado.get('estfit_novo_ncm') and dado.get('estfit_cad_ncm') != dado.get(
                    'estfit_novo_ncm'):
                vals_produto['ncm_id'] = dado.get('estfit_novo_ncm_id')

            if dado.get('estfit_cad_pis_nat_rec') != dado.get('estfit_novo_pis_nat_rec'):
                vals_produto['estpro_pis_nat_rec'] = dado.get('estfit_novo_pis_nat_rec') or ''

            if dado.get('estfit_cad_cofins_nat_rec') != dado.get('estfit_novo_cofins_nat_rec'):
                vals_produto['estpro_cofins_nat_rec'] = dado.get('estfit_novo_cofins_nat_rec') or ''

            if not any([dado.get('estfit_novo_pis_nat_rec'),
                        dado.get('estfit_novo_cofins_nat_rec')]):
                vals_produto['estpro_nao_utiliza_nat_rec'] = True

            cst_ids = list()
            if dado.get('estfit_novo_icms_cst'):
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_novo_icms_cst')),
                              ('encargo_id', 'in', icms_ids),
                              ('fiscst_tipo_movimento', 'in', ('S', 'A'))], context=context)
                cst_ids and args_tributacao_icms.append(('cst_id', '=', cst_ids[0]))
                if cst_ids and dado.get('estfit_cad_icms_cst') != dado.get('estfit_novo_icms_cst'):
                    dict_tributacao_encargos_icms.update({'cst_id': cst_ids[0]})
                    vals_tributacao_icms.update({'cst_id': cst_ids[0]})

            if cst_ids and dado.get('estfit_cad_cod_ben_fis_uf') and dado.get(
                    'estfit_novo_cod_ben_fis_uf') and dado.get(
                    'estfit_cad_cod_ben_fis_uf') != dado.get('estfit_novo_cod_ben_fis_uf'):
                obj_fiscal_cst.write(cr, uid, cst_ids[0], {
                    'fiscst_codigo_beneficio': dado.get('estfit_novo_cod_ben_fis_uf')}, context)
            if not cst_ids and dado.get('estfit_cad_icms_cst') and dado.get(
                    'estfit_novo_cod_ben_fis_uf') and dado.get(
                    'estfit_cad_cod_ben_fis_uf') != dado.get('estfit_novo_cod_ben_fis_uf'):
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_cad_icms_cst')),
                              ('encargo_id', 'in', icms_ids),
                              ('fiscst_tipo_movimento', 'in', ('S', 'A'))], context=context)
                cst_ids and obj_fiscal_cst.write(cr, uid, cst_ids[0], {
                    'fiscst_codigo_beneficio': dado.get('estfit_novo_cod_ben_fis_uf')}, context)

            cont_cst_ids = list()
            if dado.get('estfit_novo_icms_cont_cst'):
                cont_cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_novo_icms_cont_cst')),
                              ('encargo_id', 'in', icms_ids),
                              ('fiscst_tipo_movimento', 'in', ('S', 'A'))], context=context)
                if cont_cst_ids:
                    dict_tributacao_encargos_cont_icms.update({'cst_id': cont_cst_ids[0]})
                    args_tributacao_cont_icms.append(('cst_id', '=', cont_cst_ids[0]))
                    vals_tributacao_cont_icms.update({'cst_id': cont_cst_ids[0]})

            if dado.get('estfit_novo_icms_aliquota'):
                args_tributacao_icms.append(
                    ('aliquota', '=', dado.get('estfit_novo_icms_aliquota')))
                vals_tributacao_icms.update({'aliquota': dado.get('estfit_novo_icms_aliquota')})
            if dado.get('estfit_novo_icms_cont_aliquota'):
                args_tributacao_cont_icms.append(
                    ('aliquota', '=', dado.get('estfit_novo_icms_cont_aliquota')))
                vals_tributacao_cont_icms.update(
                    {'aliquota': dado.get('estfit_novo_icms_cont_aliquota')})
            if dado.get('estfit_novo_icms_red_base_cont'):
                args_tributacao_cont_icms.append(
                    ('base', '=', dado.get('estfit_novo_icms_red_base_cont')))
                vals_tributacao_cont_icms.update(
                    {'base': dado.get('estfit_novo_icms_red_base_cont')})
            if dado.get('estfit_novo_icms_base') and dado.get('estfit_cad_icms_base') != \
                    dado.get('estfit_novo_icms_base'):
                args_tributacao_icms.append(('base', '=', dado.get('estfit_novo_icms_base')))
                vals_tributacao_icms.update({'base': dado.get('estfit_novo_icms_base')})
            if dado.get('estfit_novo_icms_fcp_aliquota') and dado.get(
                    'estfit_cad_icms_fcp_aliquota') != dado.get('estfit_novo_icms_fcp_aliquota'):
                vals_tributacao_icms.update({
                    'fistri_aliquota_fcp': dado.get('estfit_novo_icms_fcp_aliquota')})
            tributacao_id = False
            if len(args_tributacao_icms) >= 2 and cst_ids:
                tributacao_ids = obj_fiscal_tributacao.search(cr, uid, args_tributacao_icms,
                                                              context=context)
                if not tributacao_ids and inclui_tributacao:
                    vals_tributacao_icms.update(
                        {'name': 'ICMS CST %s' % dado.get('estfit_novo_icms_cst')})
                    tributacao_ids = [obj_fiscal_tributacao.create(cr, uid, vals_tributacao_icms,
                                                                   context)]
                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    dict_tributacao_encargos_icms.update({'tributacao_id': tributacao_id})
            cont_tributacao_id = False
            if len(args_tributacao_cont_icms) >= 2 and cont_cst_ids:
                tributacao_ids = obj_fiscal_tributacao.search(cr, uid, args_tributacao_cont_icms,
                                                              context=context)
                if not tributacao_ids and inclui_tributacao:
                    vals_tributacao_cont_icms.update(
                        {'name': 'ICMS CST %s' % dado.get('estfit_novo_icms_cont_cst')})
                    tributacao_ids = [obj_fiscal_tributacao.create(
                        cr, uid, vals_tributacao_cont_icms, context)]
                if tributacao_ids:
                    cont_tributacao_id = tributacao_ids[0]
                    dict_tributacao_encargos_cont_icms.update({'tributacao_id': cont_tributacao_id})
            tributacao_encargo_ids = filter(lambda te: te.tipo_tributacao == 'T'
                                            and te.encargo_id.id in icms_ids,
                                            brw_fiscal_tributacao_encargos)
            if dict_tributacao_encargos_icms and (tributacao_id or tributacao_encargo_ids):
                dict_tributacao_encargos_icms.update(
                    {'tributacao_id': tributacao_id or tributacao_encargo_ids[0].tributacao_id.id,
                     'encargo_id': icms_ids[0],
                     'tipo_tributacao': 'T'})
                if tributacao_encargo_ids:
                    list_tributacao_encargos.append((1, tributacao_encargo_ids[0].id,
                                                     dict_tributacao_encargos_icms))
                elif inclui_tributacao:
                    list_tributacao_encargos.append((0, 0, dict_tributacao_encargos_icms))
            cont_tributacao_encargo_ids = filter(lambda te: te.tipo_tributacao == 'C'
                                                            and te.encargo_id.id in icms_ids,
                                                 brw_fiscal_tributacao_encargos)
            if dict_tributacao_encargos_cont_icms and \
                    (cont_tributacao_id or cont_tributacao_encargo_ids):
                dict_tributacao_encargos_cont_icms.update(
                    {'tributacao_id': cont_tributacao_id or cont_tributacao_encargo_ids and
                                                    cont_tributacao_encargo_ids[0].tributacao_id.id,
                     'encargo_id': icms_ids[0],
                     'tipo_tributacao': 'C'})
                if cont_tributacao_encargo_ids:
                    list_tributacao_encargos.append((1, cont_tributacao_encargo_ids[0].id,
                                                     dict_tributacao_encargos_cont_icms))
                elif inclui_tributacao:
                    list_tributacao_encargos.append((0, 0, dict_tributacao_encargos_cont_icms))

            cst_ids = list()
            if dado.get('estfit_novo_pis_cst_entrada'):
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_novo_pis_cst_entrada')),
                              ('encargo_id', 'in', pis_ids),
                              ('fiscst_tipo_movimento', 'in', ('E', 'A'))],
                    context=context)
                cst_ids and args_tributacao_pis_entrada.append(('cst_id', '=', cst_ids[0]))
                if cst_ids and dado.get('estfit_cad_pis_cst_entrada') != dado.get(
                        'estfit_novo_pis_cst_entrada'):
                    dict_tributacao_encargos_pis_entrada.update({'cst_id': cst_ids[0]})
                    vals_tributacao_pis_entrada.update({'cst_id': cst_ids[0]})
            if dado.get('estfit_novo_pis_aliquota_entrada') \
                    and dado.get('estfit_cad_pis_aliquota_entrada') != dado.get(
                    'estfit_novo_pis_aliquota_entrada'):
                args_tributacao_pis_entrada.append(
                    ('aliquota', '=', dado.get('estfit_novo_pis_aliquota_entrada')))
                vals_tributacao_pis_entrada.update(
                    {'aliquota': dado.get('estfit_novo_pis_aliquota_entrada')})
            tributacao_id = False
            if len(args_tributacao_pis_entrada) >= 2 and cst_ids:
                tributacao_ids = obj_fiscal_tributacao.search(cr, uid, args_tributacao_pis_entrada,
                                                              context=context)
                if not tributacao_ids and inclui_tributacao:
                    vals_tributacao_pis_entrada.update(
                        {'name': 'PIS CST %s' % dado.get('estfit_novo_pis_cst_entrada')})
                    tributacao_ids = [obj_fiscal_tributacao.create(
                        cr, uid, vals_tributacao_pis_entrada, context)]
                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    dict_tributacao_encargos_pis_entrada.update(
                        {'tributacao_id': tributacao_id})
            tributacao_encargo_ids = filter(lambda te: te.tipo_tributacao == 'E'
                                            and te.encargo_id.id in pis_ids,
                                            brw_fiscal_tributacao_encargos)
            if dict_tributacao_encargos_pis_entrada and (tributacao_id or tributacao_encargo_ids):
                dict_tributacao_encargos_pis_entrada.update(
                    {'tributacao_id': tributacao_id or tributacao_encargo_ids[0].tributacao_id.id,
                     'encargo_id': pis_ids[0],
                     'tipo_tributacao': 'E'})
                if tributacao_encargo_ids:
                    list_tributacao_encargos.append((1, tributacao_encargo_ids[0].id,
                                                     dict_tributacao_encargos_pis_entrada))
                elif inclui_tributacao:
                    list_tributacao_encargos.append((0, 0, dict_tributacao_encargos_pis_entrada))

            cst_ids = list()
            if dado.get('estfit_novo_pis_cst_saida'):
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_novo_pis_cst_saida')),
                              ('encargo_id', 'in', pis_ids),
                              ('fiscst_tipo_movimento', 'in', ('S', 'A'))],
                    context=context)
                cst_ids and args_tributacao_pis_saida.append(('cst_id', '=', cst_ids[0]))
                if cst_ids and dado.get('estfit_cad_pis_cst_saida') != dado.get(
                        'estfit_novo_pis_cst_saida'):
                    dict_tributacao_encargos_pis_saida.update({'cst_id': cst_ids[0]})
                    vals_tributacao_pis_saida.update({'cst_id': cst_ids[0]})
            if dado.get('estfit_novo_pis_aliquota_saida') \
                    and dado.get('estfit_cad_pis_aliquota_saida') != dado.get(
                    'estfit_novo_pis_aliquota_saida'):
                args_tributacao_pis_saida.append(
                    ('aliquota', '=', dado.get('estfit_novo_pis_aliquota_saida')))
                vals_tributacao_pis_saida.update(
                    {'aliquota': dado.get('estfit_novo_pis_aliquota_saida')})
            tributacao_id = False
            if len(args_tributacao_pis_saida) >= 2 and cst_ids:
                tributacao_ids = obj_fiscal_tributacao.search(cr, uid, args_tributacao_pis_saida,
                                                              context=context)
                if not tributacao_ids and inclui_tributacao:
                    vals_tributacao_pis_saida.update(
                        {'name': 'PIS CST %s' % dado.get('estfit_novo_pis_cst_saida')})
                    tributacao_ids = [obj_fiscal_tributacao.create(
                        cr, uid, vals_tributacao_pis_saida, context)]
                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    dict_tributacao_encargos_pis_saida.update(
                        {'tributacao_id': tributacao_id})
            tributacao_encargo_ids = filter(lambda te: te.tipo_tributacao == 'T'
                                            and te.encargo_id.id in pis_ids,
                                            brw_fiscal_tributacao_encargos)
            if dict_tributacao_encargos_pis_saida and (tributacao_id or tributacao_encargo_ids):
                dict_tributacao_encargos_pis_saida.update(
                    {'tributacao_id': tributacao_id or tributacao_encargo_ids[0].tributacao_id.id,
                     'encargo_id': pis_ids[0],
                     'tipo_tributacao': 'T'})
                if tributacao_encargo_ids:
                    list_tributacao_encargos.append((1, tributacao_encargo_ids[0].id,
                                                     dict_tributacao_encargos_pis_saida))
                elif inclui_tributacao:
                    list_tributacao_encargos.append((0, 0, dict_tributacao_encargos_pis_saida))

            cst_ids = list()
            if dado.get('estfit_novo_cofins_cst_entrada'):
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_novo_cofins_cst_entrada')),
                              ('encargo_id', 'in', cofins_ids),
                              ('fiscst_tipo_movimento', 'in', ('E', 'A'))],
                    context=context)
                cst_ids and args_tributacao_cofins_entrada.append(('cst_id', '=', cst_ids[0]))
                if cst_ids and dado.get('estfit_cad_cofins_cst_entrada') != dado.get(
                        'estfit_novo_cofins_cst_entrada'):
                    dict_tributacao_encargos_cofins_entrada.update({'cst_id': cst_ids[0]})
                    vals_tributacao_cofins_entrada.update({'cst_id': cst_ids[0]})
            if dado.get('estfit_novo_cofins_aliquota_entrada') \
                    and dado.get('estfit_cad_cofins_aliquota_entrada') != dado.get(
                    'estfit_novo_cofins_aliquota_entrada'):
                args_tributacao_cofins_entrada.append(
                    ('aliquota', '=', dado.get('estfit_novo_cofins_aliquota_entrada')))
                vals_tributacao_cofins_entrada.update(
                    {'aliquota': dado.get('estfit_novo_cofins_aliquota_entrada')})
            tributacao_id = False
            if len(args_tributacao_cofins_entrada) >= 2 and cst_ids:
                tributacao_ids = obj_fiscal_tributacao.search(
                    cr, uid, args_tributacao_cofins_entrada, context=context)
                if not tributacao_ids and inclui_tributacao:
                    vals_tributacao_cofins_entrada.update(
                        {'name': 'COFINS CST %s' % dado.get('estfit_novo_cofins_cst_entrada')})
                    tributacao_ids = [obj_fiscal_tributacao.create(
                        cr, uid, vals_tributacao_cofins_entrada, context)]
                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    dict_tributacao_encargos_cofins_entrada.update(
                        {'tributacao_id': tributacao_id})
            tributacao_encargo_ids = filter(lambda te: te.tipo_tributacao == 'E'
                                            and te.encargo_id.id in cofins_ids,
                                            brw_fiscal_tributacao_encargos)
            if dict_tributacao_encargos_cofins_entrada and \
                    (tributacao_id or tributacao_encargo_ids):
                dict_tributacao_encargos_cofins_entrada.update(
                    {'tributacao_id': tributacao_id or tributacao_encargo_ids[0].tributacao_id.id,
                     'encargo_id': cofins_ids[0],
                     'tipo_tributacao': 'E'})
                if tributacao_encargo_ids:
                    list_tributacao_encargos.append((1, tributacao_encargo_ids[0].id,
                                                     dict_tributacao_encargos_cofins_entrada))
                elif inclui_tributacao:
                    list_tributacao_encargos.append((0, 0, dict_tributacao_encargos_cofins_entrada))

            cst_ids = list()
            if dado.get('estfit_novo_cofins_cst_saida'):
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('codigo', '=', dado.get('estfit_novo_cofins_cst_saida')),
                              ('encargo_id', 'in', cofins_ids),
                              ('fiscst_tipo_movimento', 'in', ('S', 'A'))],
                    context=context)
                cst_ids and args_tributacao_cofins_saida.append(('cst_id', '=', cst_ids[0]))
                if cst_ids and dado.get('estfit_cad_cofins_cst_saida') != dado.get(
                        'estfit_novo_cofins_cst_saida'):
                    dict_tributacao_encargos_cofins_saida.update({'cst_id': cst_ids[0]})
                    vals_tributacao_cofins_saida.update({'cst_id': cst_ids[0]})
            if dado.get('estfit_novo_cofins_aliquota_saida') \
                    and dado.get('estfit_cad_cofins_aliquota_saida') != dado.get(
                    'estfit_novo_cofins_aliquota_saida'):
                args_tributacao_cofins_saida.append(
                    ('aliquota', '=', dado.get('estfit_novo_cofins_aliquota_saida')))
                vals_tributacao_cofins_saida.update(
                    {'aliquota': dado.get('estfit_novo_cofins_aliquota_saida')})
            tributacao_id = False
            if len(args_tributacao_cofins_saida) >= 2 and cst_ids:
                tributacao_ids = obj_fiscal_tributacao.search(
                    cr, uid, args_tributacao_cofins_saida, context=context)
                if not tributacao_ids and inclui_tributacao:
                    vals_tributacao_cofins_saida.update(
                        {'name': 'COFINS CST %s' % dado.get('estfit_novo_cofins_cst_saida')})
                    tributacao_ids = [obj_fiscal_tributacao.create(
                        cr, uid, vals_tributacao_cofins_saida, context)]
                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    dict_tributacao_encargos_cofins_saida.update(
                        {'tributacao_id': tributacao_id})
            tributacao_encargo_ids = filter(lambda te: te.tipo_tributacao == 'T'
                                            and te.encargo_id.id in cofins_ids,
                                            brw_fiscal_tributacao_encargos)
            if dict_tributacao_encargos_cofins_saida and (tributacao_id or tributacao_encargo_ids):
                dict_tributacao_encargos_cofins_saida.update(
                    {'tributacao_id': tributacao_id or tributacao_encargo_ids[0].tributacao_id.id,
                     'encargo_id': cofins_ids[0],
                     'tipo_tributacao': 'T'})
                if tributacao_encargo_ids:
                    list_tributacao_encargos.append((1, tributacao_encargo_ids[0].id,
                                                     dict_tributacao_encargos_cofins_saida))
                elif inclui_tributacao:
                    list_tributacao_encargos.append((0, 0, dict_tributacao_encargos_cofins_saida))

            if list_tributacao_encargos:
                vals_produto.update({'tributacao_ids': list_tributacao_encargos})
            obj_estoque_produto.write(cr, uid, dado.get('estfit_produto_id'), vals_produto, context)
        return True

    def fisutl_importar_nfe_propria(self, cr, uid, xml, context=None):
        _spg = self.pool.get
        nfe = ProcNFe()
        nfe.xml = xml
        _cStat = nfe.protNFe.infProt.cStat.valor
        """ cStat
        100 - Autorizado o uso
        101 - Cancelamento autorizado
        102 - Inutilizacao de numero homologado
        110 - Denegado
        """
        _nro_nf = nfe.NFe.infNFe.ide.cNF.valor
        if _cStat not in ('100', '101', '102', '110'):
            return {'status': False, 'mensagem': 'Nf %s nao esta autorizada: Status: %s!' % (_nro_nf, _cStat)}
        _cnpj_emitente = nfe.NFe.infNFe.emit.CNPJ.valor
        _uf_emitente = nfe.NFe.infNFe.emit.enderEmit.UF.valor
        _uf_destinatario = nfe.NFe.infNFe.dest.enderDest.UF.valor
        _pessoa_ids = _spg('adm.pessoa').search(cr, uid, [('cpf_cnpj', '=', _cnpj_emitente)], context=context)
        if not _pessoa_ids:
            return {'status': False, 'mensagem': 'Cnpj nao cadastrado! cnpj: %s' % _cnpj_emitente}
        _pessoa_id = _pessoa_ids[0]
        company_ids = _spg('res.company').search(cr, uid, [('rescon_pessoa_id', '=', _pessoa_id)], context=context)
        if not company_ids:
            return {'status': False, 'mensagem': 'Empresa nao cadastrada para a pessoa_id: %s' % _pessoa_id}
        company_id = company_ids[0]
        context['sig_force_company'] = company_id
        _user_dados = _spg('res.users').read(cr, uid, uid, ['pessoa_id'], context=context)
        if not _user_dados:
            return {'status': False, 'mensagem': 'O Usuario %s nao tem uma pessoa_id!' % uid}
        _user_pessoa_id = _user_dados.get('pessoa_id')[0]
        _args = [('company_id', '=', company_id), ('pessoa', '=', _user_pessoa_id)]
        _func_ids = _spg('rh.funcionario').search(cr, uid, _args, context=context)
        if not _func_ids:
            _func_id = _spg('rh.funcionario').create(cr, uid, {'company_id': company_id, 'pessoa': _user_pessoa_id},
                                                   context=context)
        else:
            _func_id = _func_ids[0]
        _modelo_ids = _spg('fiscal.modelo.nf').search(cr, uid, [('fismnf_codigo', '=', '55')], context=context)
        if not _modelo_ids:
            return {'status': False, 'mensagem': 'Modelo de nf nao cadastrado: 55'}
        _modelo_id = _modelo_ids[0]
        _serie = nfe.NFe.infNFe.ide.serie.valor
        if _cStat == '102':  # Inutilizada
            res = _spg('mkp.utils').mkputl_inutilizar_nf(cr, uid, company_id, _nro_nf, _serie, context=context)
            return res
        _campos = ['id', 'condicao_pgto_transferencia_id','company_id','fispar_natureza_uf_dentro_venda_id',
                   'fispar_natureza_uf_fora_venda_id', 'fispar_natureza_uf_dentro_devolucao_id',
                   'fispar_natureza_uf_fora_devolucao_id', 'fispar_natureza_transferencia_saida_id']
        _dados_parametros = _spg('adm.utils').admutl_get_parametros(cr, uid, 'fiscal.parametro', _campos, context=context)
        _param_id = _dados_parametros.get('id')
        _param_company_id = _dados_parametros.get('company_id')
        args = [('modelo_nf_id', '=', _modelo_id), ('serie_nf', '=', _serie), ('parametro_id', '=', _param_id)]
        _modelo_nf_series_ids = _spg('fiscal.modelo.nf.series').search(cr, uid, args, context=context)
        if not _modelo_nf_series_ids:
            return {'status': False, 'mensagem': 'Serie %s nao cadastrada para a empresa_id: %s' % (_serie, _param_company_id)}
        _modelo_nf_series_id = _modelo_nf_series_ids[0]
        _natureza = nfe.NFe.infNFe.ide.natOp.valor
        _cfop = nfe.NFe.infNFe.det[0].prod.CFOP.valor
        _nat_ids = _spg('fiscal.cfop.natureza').search(cr, uid,
                                                       [('name', '=', _natureza), ('cfop_id', '=', _cfop)],
                                                       context=context)
        if not _nat_ids:
            return {'status': False, 'mensagem': 'Natureza nao cadastrada: %s e cfop: %s' % (_natureza, _cfop)}
        _nat_id = _nat_ids[0]
        brw_natureza = _spg('fiscal.cfop.natureza').browse(cr, uid, _nat_id, context=context)
        tipo_nat = brw_natureza.cfop_id.movimento_tipo
        _est_operacao_id = brw_natureza.fisnat_operacao_id.id
        _data_nf = nfe.NFe.infNFe.ide.dhEmi.valor
        _cnpj_destinatario = nfe.NFe.infNFe.dest.CPF.valor if nfe.NFe.infNFe.dest.CPF else nfe.NFe.infNFe.dest.CNPJ.valor
        _cliente_nome = nfe.NFe.infNFe.dest.xNome.valor
        _destinatario_ids = _spg('adm.pessoa').search(cr, uid, [('cpf_cnpj', '=', _cnpj_destinatario)], context=context)
        if not _destinatario_ids:
            _rua = nfe.NFe.infNFe.dest.enderDest.xLgr.valor
            _rua_nro = nfe.NFe.infNFe.dest.enderDest.nro.valor
            _rua_complemento = nfe.NFe.infNFe.dest.enderDest.xCpl.valor if nfe.NFe.infNFe.dest.enderDest.xCpl else ''
            _bairro = nfe.NFe.infNFe.dest.enderDest.xBairro.valor
            _cidade = nfe.NFe.infNFe.dest.enderDest.xMun.valor
            _cep = nfe.NFe.infNFe.dest.enderDest.CEP.valor
            _estado = nfe.NFe.infNFe.dest.enderDest.UF.valor
            _pais = nfe.NFe.infNFe.dest.enderDest.xPais.valor
            _ie = nfe.NFe.infNFe.dest.IE.valor if nfe.NFe.infNFe.dest.IE else ''
            _partner_id = _spg('res.partner').create(cr, uid, {'name': _cliente_nome}, context=context)
            if _cnpj_destinatario and len(_cnpj_destinatario) < 12:
                _tipo = 'F'
            else:
                _tipo = 'J'
            _vals_pessoa = {
                'partner_id': _partner_id,
                'pessoa_tipo': _tipo,
                'cpf_cnpj': _cnpj_destinatario,
            }
            if _ie:
                _vals_pessoa['admpes_jur_ie'] = _ie
            _destinatario_id = _spg('adm.pessoa').create(cr, uid, _vals_pessoa, context=context)
            if not _destinatario_id:
                return {'status': False, 'mensagem': 'Destinatario nao cadastrado: %s' % _cnpj_destinatario}
            _cep_ids = self.pool.get('adm.cep').search(cr, uid, [('cep', '=', _cep)], context=context)
            _pais_id = self.pool.get('adm.pais').search(cr, uid, [('name', '=', _pais)], context=context)[0]
            if _cep_ids:
                _cep_dados = self.pool.get('adm.cep').read(cr, uid, _cep_ids, [], context=context)[0]
                _pais_id = _cep_dados.get('pessoa_pais_id')[0]
                _estado_id = _cep_dados.get('pessoa_uf_id')[0]
                _localidade_id = _cep_dados.get('pessoa_cidade_id')[0]
                _bairro_id = _cep_dados.get('pessoa_bairro_id')
                if isinstance(_bairro_id, tuple):
                    _bairro_id = _bairro_id[0]
                _logradouro_id = _cep_dados.get('pessoa_logradouro_id')
                if isinstance(_logradouro_id, tuple):
                    _logradouro_id = _logradouro_id[0]
            else:
                _estado_id, _localidade_id, _bairro_id, _logradouro_id = self.mkputl_orders_get_dados_correios(
                    cr, uid, _cep, _pais_id, context=context)
                if _estado_id and _localidade_id and _pais_id:
                    _vals_cep = {
                        'cep': _cep,
                        'pessoa_bairro_id': _bairro_id,
                        'pessoa_cidade_id': _localidade_id,
                        'pessoa_logradouro_id': _logradouro_id,
                        'pessoa_pais_id': _pais_id,
                        'pessoa_uf_id': _estado_id
                    }
                    self.pool.get('adm.cep').create(cr, uid, _vals_cep, context=context)
            if not _logradouro_id:
                _logradouro = _rua
                _titulo = capitalize_all(_logradouro.split()[0])
                lista = _logradouro.split()[1:]
                _logradouro = ''
                for x in lista:
                    _logradouro += x + ' '
                _logradouro = capitalize_all(_logradouro.strip())
                logradouro = _spg('adm.utils').admutl_capitalizar_adm_nome(cr, uid, _logradouro, context=context)
                __logradouro_ids = _spg('adm.logradouro').search(cr, uid,
                                                           [('descricao', '=', logradouro), ('titulo', '=', _titulo)],
                                                           context=context)
                if __logradouro_ids:
                    _logradouro_id = __logradouro_ids[0]
                else:
                    _logradouro_id = _spg('adm.logradouro').create(cr, uid,
                                                                   {'descricao': logradouro, 'titulo': _titulo},
                                                                    context=context)
            if _cidade:
                localidade = _spg('adm.utils').admutl_capitalizar_adm_nome(cr, uid, _cidade,
                                                                                    context=context)
                __localidade_ids = _spg('adm.cidade').search(cr, uid, [('name', '=', localidade),
                                                                       ('uf_id', '=', _estado_id)],
                                                                        context=context)
                if __localidade_ids:
                    _localidade_id = __localidade_ids[0]
                else:
                    _localidade_id = _spg('adm.cidade').create(cr, uid, {'name': localidade, 'uf_id': _estado_id},
                                                         context=context)
            if _bairro:
                __obj_bairro = self.pool.get('adm.bairro')
                __bairro_ids = _spg('adm.bairro').search(cr, uid, [('name', '=', _bairro)], context=context)
                if __bairro_ids:
                    _bairro_id = __bairro_ids[0]
                else:
                    _bairro_id = _spg('adm.bairro').create(cr, uid, {'name': _bairro}, context=context)

            _endereco_tipo_id = \
                self.pool.get('adm.endereco.tipo').search(cr, uid, [('name', '=', 'Normal')], context=context)[0]
            _vals_endereco = {
                'cep': _cep,
                'endereco_tipo_id': _endereco_tipo_id,
                'complemento': _rua_complemento,
                'numero': _rua_nro,
                'pessoa_id': _destinatario_id,
                'cidade_id': _localidade_id,
                'logradouro_id': _logradouro_id,
                'pais_id': _pais_id,
                'uf_id': _estado_id,
                'bairro_id': _bairro_id,
            }
            _end_id = self.pool.get('adm.pessoa.endereco').create(cr, uid, _vals_endereco, context=context)
            _spg('adm.pessoa').write(cr, uid, _destinatario_id, {'endereco_nf_id': _end_id}, context=context)
        dados_pessoa = _spg('adm.pessoa').read(cr, uid, _destinatario_id, ['endereco_nf_id'], context=context)
        if dados_pessoa and dados_pessoa.get('endereco_nf_id'):
            _end_id = dados_pessoa.get('endereco_nf_id')[0]
        else:
            _end_id = ''
        vr_frete = nfe.NFe.infNFe.total.ICMSTot.vFrete.valor if nfe.NFe.infNFe.total.ICMSTot.vFrete else 0.00
        _vr_nota = nfe.NFe.infNFe.total.ICMSTot.vNF.valor
        _pedidos_ids = []
        _condicoes = []
        if tipo_nat == 'T':  # Nota de transferencia
            _condicao_forma_id = _dados_parametros.get('condicao_pgto_transferencia_id')
            if not _condicao_forma_id:
                return {'status': False, 'mensagem': 'Condicao/forma de pagamento na cadastrado no parametro'}
            dados_condicao_forma = _spg('adm.condicao.forma').read(cr, uid, _condicao_forma_id, [], context=context)
            _condicoes = [{
                'condicao_forma_id': dados_condicao_forma.get('id'),
                'forma_pagto_id': dados_condicao_forma.get('forma_pagamento_id')[0],
                'valor':_vr_nota,
                'parcelas': [{
                    'valor': _vr_nota,
                    'vencimento': _data_nf
                }]
            }]
        _vr_desconto = nfe.NFe.infNFe.total.ICMSTot.vDesc.valor
        mens_fiscais = nfe.NFe.infNFe.infAdic.infAdFisco
        mens_contribuinte = nfe.NFe.infNFe.infAdic.infCpl
        context['fatura_pedido'] = True
        context['fiscal'] = True
        context['proprio'] = True
        context['fisnfc_nao_faturar_sefaz'] = True
        context['fisnfc_importacao_nfce_do_supervisor'] = True
        if _cStat == 100:
            _state = '3'
        elif _cStat in (101, 135):
            _state = 'C'
        elif _cStat == 102:
            _state = 'I'
        elif _cStat == 110:
            _state = 'D'
        else:
            _state = '1'
        _tpNF = 'S' if nfe.NFe.infNFe.ide.tpNF.valor == 1 else 'E'
        vals = {'tipo': _tpNF,
                'fisnfc_tipo_servico': False,
                'company_id': company_id,
                'numero_nf': _nro_nf,
                'serie_nf': _serie,
                'nf_propria': True,
                'natureza_id': _nat_id,
                'modelo_nf_series_id': _modelo_nf_series_id,
                'modelo_nf_id': _modelo_id,
                'destinatario_id': _destinatario_id,
                'destinatario_endereco_id': _end_id,
                'data_emissao': _data_nf,
                'hora_emissao': _data_nf,
                'data_saida_entrada': _data_nf,
                'modelo_55': True,
                'fisnfc_consumidor': False,
                'gerar_xml': True,
                'nfe_forma_impressao_danfe': nfe.NFe.infNFe.ide.tpImp.valor,
                'nfe_processo_emissao': nfe.NFe.infNFe.ide.procEmi.valor,
                'fisnfc_nfe_indicador_presenca': nfe.NFe.infNFe.ide.indPres.valor,
                'nfe_versao_processo_emissao': nfe.NFe.infNFe.ide.verProc.valor,
                'nfe_forma_emissao': nfe.NFe.infNFe.ide.tpEmis,
                'finalidade_emissao': nfe.NFe.infNFe.ide.finNFe,
                'nfe_protocolo': nfe.protNFe.infProt.nProt.valor,
                'nfe_status': '%s - %s' % (nfe.protNFe.infProt.cStat.valor, nfe.protNFe.infProt.xMotivo.valor),
                'nfe_chave_acesso': nfe.protNFe.infProt.chNFe.valor,
                'nfe_lote': '',
                'vr_desconto': _vr_desconto,
                'vr_icms_substituicao': nfe.NFe.infNFe.total.ICMSTot.vST.valor,
                'vr_base_icms_substituicao': nfe.NFe.infNFe.total.ICMSTot.vBCST.valor,
                'vr_seguro': nfe.NFe.infNFe.total.ICMSTot.vSeg.valor,
                'vr_despesas_acessorias': nfe.NFe.infNFe.total.ICMSTot.vOutro.valor,
                'vr_frete': vr_frete,
                'vr_total_nf': _vr_nota,
                'frete': nfe.NFe.infNFe.transp.modFrete.valor,
                'fisnfc_infcomp_contribuinte': mens_contribuinte,
                'fisnfc_infcomp_fisco': mens_fiscais,
                'nfe_xml_gerado': nfe.NFe.xml,
                'nfe_xml_auto': _state == '3' and nfe.protNFe.xml,
                'nfe_xml_cancelamento': _state == 'C' and nfe.protNFe.xml,
                'state': _state,
                }
        if vr_frete > 0:
            vals['tipo_frete'] = 'D'
        if _state == 'C':
            vals['estoque_movimento_itens_ids'] = []
        _hoje = _spg('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(), formato='%Y_%m_%d_%H_%M_%s')
        args =[('numero_nf', '=', _nro_nf),
               ('serie_nf', '=', _serie),
               ('tipo', '=', _tpNF),
               ('company_id', '=', company_id),
               ('state', 'in', ('3', 'C', 'I', 'P', 'D'))]
        nf_ids = _spg('fiscal.nf.capa').search(cr, uid, args, context=context)
        if nf_ids:
            if tipo_nat == 'T' or _state != '3':
                return {'status': True, 'Mensagem': ''}
            dados_nf = _spg('fiscal.nf.capa').read(cr, uid, nf_ids[0], ['state','nf_condicao_ids'], context=context)
            if dados_nf.get('state') == _state:  # A nota já e existe e o status esta correto
                return {'status': True, 'Mensagem': ''}
            else: # nota ja existe, e os estados estão diferentes
                return {'status': False, 'Mensagem': 'Nota ja cadastrada com status diferente!'}
        cr.execute(''' BEGIN; SAVEPOINT importar_notas_propria_%s; ''' % _hoje)
        sql_rollback = """ROLLBACK TO SAVEPOINT importar_notas_propria_%s; """ % _hoje
        sql_release = """RELEASE SAVEPOINT importar_notas_propria_%s; """ % _hoje
        context['nf_capa_manutencao'] = True
        try:
            nf_id = _spg('fiscal.nf.capa').create(cr, uid, vals, context=context)
        except Exception as e:
            cr.execute(sql_rollback)
            return {'status': False, 'Mensagem': e}
        context.pop('nf_capa_manutencao')
        if not nf_id:
            cr.execute(sql_rollback)
            return {'status': False, 'Mensagem': _('Nao consegui criar a nota fiscal')}

        #### Criar capa do movimento do estoque
        vals = {'company_id': company_id,
                'nf_capa_id': nf_id,
                'funcionario_id': _func_id,
                'estmvc_operacao_id': _est_operacao_id,
                'data': _data_nf,
                'state': 'F' if _state == '3' else 'C',
        }
        mov_capa_id = _spg('estoque.movimento.capa').create(cr, uid, vals, context=context)

        ####   Criar itens do movimento do estoque e fiscal item tributacao
        nf_item_ids = list()
        itens_ids = list()
        # itens = dados.get('items')
        _total_itens = _vr_nota - vr_frete
        lista = []
        lista_frete = []
        # i = 0
        # for nfeItem in nfe.NFe.infNFe.det:
        #     i += 1
        #     lista.append([i, float(item.get('total_amount'))])
        #     lista_frete.append([i, float(item.get('total_amount'))])
        # valores_rateados_itens = _spg('adm.utils').efetuar_rateio(_vr_nota, lista)
        # valores_rateados_frete = _spg('adm.utils').efetuar_rateio(vr_frete, lista_frete)
        i = -1
        for nfeItem in nfe.NFe.infNFe.det:
            i += 1
            _prod_codigo = nfeItem.prod.cProd
            _prod_ean = nfeItem.prod.cEAN
            _qtd = nfeItem.prod.qCom
            _vl_unit = nfeItem.prod.vUnCom
            barra_ids = _spg('automacao.codigo.barra').search(cr, uid, [('codigo_barra', '=', _prod_ean)],
                                                              context=context)
            dados_barra = _spg('automacao.codigo.barra').read(cr, uid, barra_ids[0], ['produto_id', 'grade_id'],
                                                              context=context)
            _grade_id = [dados_barra.get('grade_id')[0]]
            _produto_id = [dados_barra.get('produto_id')[0]]

            args = [('company_id', '=', company_id), ('grade_id', '=', _grade_id), ('local_id', '=', _local_id)]
            _local_locacao_ids = _spg('estoque.local.locacao').search(cr, uid, args, context=context)
            if not _local_locacao_ids:
                return {'status': False, 'mensagem': 'Local locacao, nao cadastrado para o item: %s' % _prod_codigo}
            dados_local_locacao = _spg('estoque.local.locacao').read(cr, uid, _local_locacao_ids[0],
                                                                     ['estllc_produto_embalagem_id'],
                                                                     context=context)
            _item_vr_total = nfeItem.prod.vProd
            _item_vr_frete = nfeItem.prod.vFrete
            _cfop = nfeItem.prod.CFOP
            _cfop_ids = _spg('fiscal.cfop').search(cr, uid, [('codigo', '=', _cfop)], context=context)
            vals_custos = _spg('estoque.preco.custo').get_custos_do_dia(cr, uid, _produto_id, company_id,
                                                                    _data_nf, context)
            custos = vals_custos.get(_produto_id) or {}
            _custo_corrigido = (custos.get('estprc_custo_corrigido') or 0) * _qtd
            _ultimo_custo = (custos.get('estprc_ultimo_custo') or 0) * _qtd
            _custo_reposicao = (custos.get('estprc_custo_reposicao') or 0) * _qtd
            _custo_medio_contabil = (custos.get('estprc_custo_medio_contabil') or 0) * _qtd
            _vr_desconto_item = 0
            if i == 0:
                _vr_desconto_item = _vr_desconto
            vals = {
                # 'local_id': _local_id,
                # 'nf_id': nf_id,
                # 'sequencia': i + 1,
                # 'cfop_id': _cfop_ids[0],
                # 'capa_id': mov_capa_id,
                # 'natureza_id': _nat_id,
                # 'produto_id': _prod_id,
                # 'quantidade': _quantidade,
                # 'produto_embalagem_id': dados_local_locacao.get('estllc_produto_embalagem_id')[0],
                # 'grade_id': _grade_id,
                # 'vr_desconto': 0.0,
                # 'vr_desconto_rateado': _vr_desconto_item,
                # 'vr_despesas_rateado': 0.0,
                # 'estmvi_vr_despesas_extras_rateado': 0.0,
                # 'vr_preco_venda': _vr_total,
                # 'vr_unitario': _vr_total / item.get('quantity'),
                # 'vr_total_item': _vr_total,
                # 'vr_total_item_rateado': valores_rateados_itens[i][1],
                # 'vr_frete_rateado':  valores_rateados_frete[i][1],
                # 'estmvi_saidas_custo_corrigido_total': _custo_corrigido,
                # 'estmvi_saidas_custo_medio_contabil_total': _custo_medio_contabil,
                # 'estmvi_saidas_ultimo_custo_total': _ultimo_custo,
                # 'estmvi_saidas_custo_reposicao_total': _custo_reposicao,
            }
        #     context['atualizacao_inventario'] = True # contexto utilizdo para apenas criar sem fazer outros tratamentos
        #     _item_id = _spg('estoque.movimento.itens').create(cr, uid, vals, context=context)
        #     context.pop('atualizacao_inventario')
        #     itens_ids.append(_item_id)
        #     _cst_icms = _csosn = False
        #     _csosn_ids = []
        #     nat_rec_pis_cofins = ''
        #     for _encargo in item.get('fiscal_data').get('rules') :
        #         _name = _encargo.get('name', False)
        #         if _name == 'ICMS_SIMPLES':
        #             _csosn = _encargo.get('attributes').get('csosn', False)
        #             _csosn_ids = _spg('fiscal.csosn').search(cr, uid, [('codigo', '=', _csosn)], context=context)
        #             _enc_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'ICMS')], context=context)
        #         elif _name == 'PIS':
        #             nat_rec_pis_cofins = _spg('estoque.produto').read(cr, uid, _prod_id, ['estpro_pis_nat_rec'],
        #                                                               context).get('estpro_pis_nat_rec') or ''
        #         elif _name == 'COFINS':
        #             nat_rec_pis_cofins = _spg('estoque.produto').read(cr, uid, _prod_id, ['estpro_cofins_nat_rec'],
        #                                                               context).get('estpro_cofins_nat_rec') or ''
        #         else:
        #             continue
        #     if not _csosn_ids:
        #         return {'status': False,
        #                 'mensagem': 'Nao encontrei o csosn: %s' % _csosn}
        #     for _encargo in item.get('fiscal_data').get('rules'):
        #         _name = _encargo.get('name', False)
        #         _vbc = _encargo.get('attributes').get('vbc', False)
        #         _cst = _encargo.get('attributes').get('cst', False)
        #         _nat_rec_pis_cofins = ''
        #         if _name == 'ICMS_SIMPLES':
        #             args = [('fiscst_csosn_nao_contribuinte_id', '=', _csosn_ids[0])]
        #             _cst_ids = _spg('fiscal.cst').search(cr, uid, args, context=context)
        #             if not _cst_ids:
        #                 return {'status': False,
        #                     'mensagem': 'Nao encontrei cst para o csosn: %s' % _csosn}
        #             _cst = _spg('fiscal.cst').read(cr, uid, _cst_ids[0], ['codigo']).get('codigo')
        #             if not _cst:
        #                 return {'status': False,
        #                         'mensagem': 'Nao encontrei cst para o csosn: %s' % _csosn}
        #             _enc_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'ICMS')], context=context)
        #             if not _enc_ids:
        #                 return {'status': False,
        #                         'mensagem': 'Nao encontrei o encargo: %s' % 'ICMS'}
        #         elif _name == 'PIS':
        #             _vpis = _encargo.get('attributes').get('vpis', False)
        #             _ppis = _encargo.get('attributes').get('vpis', False)
        #             _enc_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'PIS')], context=context)
        #             if not _enc_ids:
        #                 return {'status': False,
        #                         'mensagem': 'Nao encontrei o encargo: %s' % 'PIS'}
        #         elif _name == 'COFINS':
        #             _vcofins = _encargo.get('attributes').get('vcofins', False)
        #             _pcofins = _encargo.get('attributes').get('vcofins', False)
        #             _enc_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'COFINS')], context=context)
        #             if not _enc_ids:
        #                 return {'status': False,
        #                         'mensagem': 'Nao encontrei o encargo: %s' % 'COFINS'}
        #         else:
        #             continue
        #         args = [('codigo','=',_cst),('encargo_id','=',_enc_ids[0])]
        #         _cst_ids = _spg('fiscal.cst').search(cr, uid, args, context=context)
        #         args =[
        #             ('encargo_id','=', _enc_ids[0]),
        #             ('produto_id','=', _prod_id),
        #             ('tipo_tributacao', '=', 'T'),
        #             # ('cst_id','=',_cst_ids[0])
        #         ]
        #         _tribut_encargos_ids = _spg('fiscal.tributacao.encargos').search(cr, uid, args, context=context)
        #         if not _tribut_encargos_ids:
        #             return {'status': False,
        #                     'mensagem': 'Nao encontrei o tributacao encargos para o produto_id: %s, encargo_id %s, cst_id: %s' % (_prod_id, _enc_ids[0], _cst_ids[0])}
        #         _tribut_id = _spg('fiscal.tributacao.encargos').read(cr, uid, _tribut_encargos_ids[0], ['tributacao_id'], context=context)['tributacao_id'][0]
        #         vals = {
        #             'movimento_itens_id': _item_id,
        #             'encargo_id': _enc_ids[0],
        #             'fisnit_capa_id': nf_id,
        #             'aliquota': 0.0,
        #             'vr_encargo': 0.0,
        #             'vr_base_calculo': valores_rateados_itens[i][1],
        #             'vr_reducao_base': 0.0,
        #             'fisnit_nat_rec_pis_cofins': nat_rec_pis_cofins,
        #             'fisnit_csosn_id': _csosn_ids[0],
        #             'cst_id': _cst_ids[0],
        #             'tributacao_id': _tribut_id,
        #         }
        #         _nf_item_id = _spg('fiscal.nf.item.tributacao').create(cr, uid, vals, context=context)
        #         nf_item_ids.append(_nf_item_id)
        # context['super_write_estoque_movimento_itens'] = True
        # context['fisnit_super_write'] = True
        # _spg('estoque.movimento.itens').write(cr, uid, itens_ids, {}, context=context)
        # self.pool.get('fiscal.nf.item.tributacao').write(cr, uid, nf_item_ids, {}, context=context)
        # context.pop('super_write_estoque_movimento_itens')
        # context.pop('fisnit_super_write')
        #
        # ### Inserir condicoes de pagamento
        # for _condicao in _condicoes:
        #     parcelas = _condicao.pop('parcelas')
        #     _condicao['capa_id'] = nf_id
        #     cond_pgto_id = _spg('fiscal.condicao.pagamento').create(cr, uid, _condicao, context=context)
        #     for _parcela in parcelas:
        #         _parcela['nf_condicao_id'] = cond_pgto_id
        #         _spg('fiscal.nf.parcelas').create(cr, uid, _parcela, context=context)
        # res = _spg('fiscal.nf.capa').write(cr, uid, [nf_id], {}, context=context)
        # try:
        #     res = _spg('fiscal.nf.capa').fisnfc_funcao_button_faturar(cr, uid, [nf_id], context)
        # except Exception as e:
        #     cr.execute(sql_rollback)
        #     return {'status': False, 'Mensagem': e}
        # if not res:
        #     cr.execute(sql_rollback)
        #     return {'status': False, 'Mensagem': 'Nao consegui faturar a nota fiscal'}
        # if _pedidos_ids:
        #     context['pedcap_write_state'] = True
        #     _spg('estoquepedidos.pedido.capa').write(cr, uid, _pedidos_ids, {'state': 'F'}, context=context)
        # cr.execute(sql_release)
        return {'status': True, 'Mensagem': ''}

# Ajusta os valores de atributos ´many2one´
def adjust_m2o(val):
    """ Recupera apenas o ID dos atributos ´many2one´ """
    for k, v in val.items():
        if k.endswith('_id') and isinstance(v, (list, tuple)):
            val[k] = v[0]
        elif k.endswith('_ids'):
            for l in v:
                if not isinstance(l, (list, tuple)):
                    v[v.index(l)] = [4, l, False]

    return val

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
