# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2016, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from SigUtils import datas
from decimal import Decimal, ROUND_HALF_DOWN
import datetime


# noinspection PyAbstractClass,PyPep8Naming
class adm_condicao(osv.Model):
    """
    Heranca da classe adm_condicao

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _inherit = 'adm.condicao'

    def gerar_parcelas(self, cr, user, condicao, valor, emissao=None, context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param condicao: ID da condicao de pagamento
        :param valor: valor a ser rateado entre as parcelas
        :param emissao: data de referencia para o calculo do vencimento das parcelas
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type condicao: long
        :type valor: float
        :type emissao: str
        :type context: dict
        :return: lista com os valores de vencimento e valor de cada parcela
        :rtype: list
        """

        context = context and context.copy() or {}
        f_vencimento = lambda e, p, t: datetime.datetime.strptime(
            datas.sigDatas(e).getCalculaPrazo(p, t), '%Y%m%d').strftime('%Y-%m-%d')
        f_parcelas = lambda v, e, p, t: [dict(
            valor=v, vencimento=f_vencimento(e, p, t)) for p in p]
        f_round = lambda f: float(Decimal(f).quantize(Decimal(10) ** -2, ROUND_HALF_DOWN))
        emissao = datetime.datetime.today().strftime('%Y-%m-%d') if not emissao else emissao
        condicao_dados = self.read(cr, user, condicao, [], context)

        # Gera as parcelas
        valor_parcela = f_round(valor * 1.0 / condicao_dados.get('qtde_parcelas'))
        dias_primeira_parcela = condicao_dados.get('dias_primeira_parcela')
        dias_incremento = condicao_dados.get('dias_incremento')
        qtde_parcelas = condicao_dados.get('qtde_parcelas')
        tipo_prazo = condicao_dados.get('tipo_prazo')
        parcelas = []
        # Primeira parcela
        parcelas.extend(f_parcelas(valor_parcela, emissao, [dias_primeira_parcela], tipo_prazo))
        data = parcelas[0].get('vencimento')
        # Demais parcelas
        for n in range(2, qtde_parcelas + 1):
            parcela = f_parcelas(valor_parcela, data, [dias_incremento], tipo_prazo)
            data = parcela[0].get('vencimento')
            parcelas.extend(parcela)
        # Ajusta o valor da ultima parcela
        if valor - valor_parcela * qtde_parcelas:
            parcelas[qtde_parcelas - 1]['valor'] = valor_parcela + f_round(
                valor - valor_parcela * qtde_parcelas)

        return parcelas

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
