# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from dateutil.parser import parse


class estoque_movimento_capa(osv.Model):
    """
    complemento do movimento do estoque(capa)
    """
    _inherit = 'estoque.movimento.capa'

    def estmvc_atualiza_data(self, cr, uid, nf_ids, vals_nf, context=None):
        """
        Método chamado na atuatilizacao de notas fiscais (write). Tem o objetivo de atualizar a
        data da capa do movimento baseado na data da nota fiscal, desde que, esse dado esteja para
        ser atualizado (vals_nf).
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param nf_ids: lista de inteiros, identificadores da tabel fiscal_nf_capa
        :type nf_ids: list
        :param vals_nf: conjunto de dados a serem atualizados nas notas fiscais
        :type vals_nf: dict
        :param context: conjunto de dados contextuais
        :type context: dict
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')

        # Verifica a data de movimento de acordo com o tipo da nota
        for dados_nf in self.pool.get('fiscal.nf.capa').read(
                cr, uid, nf_ids if isinstance(nf_ids, (list, tuple)) else [nf_ids], ['tipo'],
                context):
            data_mov_capa = ''
            if dados_nf.get('tipo') == 'E' and vals_nf.get('data_saida_entrada'):
                data_mov_capa = obj_adm_utils.admutl_datetime_to_timezone(
                    cr, uid, parse(vals_nf.get('data_saida_entrada')), '%Y-%m-%d')
            elif dados_nf.get('tipo') == 'S' and vals_nf.get('data_emissao'):
                data_mov_capa = obj_adm_utils.admutl_datetime_to_timezone(
                    cr, uid, parse(str(vals_nf.get('data_emissao'))), '%Y-%m-%d')
            if not data_mov_capa:
                continue
            capa_ids = self.search(cr, uid, [('nf_capa_id', '=', dados_nf.get('id'))],
                                   context=context)
            self.write(cr, uid, capa_ids, {'data': data_mov_capa}, context)
        return True
