# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import time
from SigUtils.datas import calcDiferenca
from openerp.tools.translate import _


class estoque_preco_custo(osv.Model):
    """
    Tabela de precos de custos dos produtos
    """
    _name = 'estoque.preco.custo'
    _inherit = 'estoque.preco.custo'

    def estprc_set_custos_do_dia(self, cr, uid, epc_nota_fiscal_id, context=None):
        """
        Esta Rotina e utilizada para re-calcular os custo do produto
    ######################################################################
    ###  Calculo dos Custos
    ###   ICMS_guia  --> Valores lancados nas Guias   ( Sem Recomposicao )
    ###   ICMS_recomposicao  --> Valores lancados nas Guias   ( Com Recomposicao )
    ###   Quantidade_Total_Item -> Quantidade na Embalagem * Quantidade Pedida
    ###   Qtd_estoque  --> Quantidade em estoque do Produto filtrando somente a Empresa.
    ###   nf_valor_bruto --> (Valor Unitário do item * Quantidade_Total_Item )
    ###   Estoque_Financeiro_Medio  --> Custo_Medio(Atual) * Quantidade em estoque
    ###   Empresa do Tipo
    ###     Comercio :
    ###         Valor_Total_do_Item = nf_valor_bruto - nf_desconto - ICMS_Item - PIS - Cofins +ICMS_Substituicao + nf_frete + nf_frete_icms + IPI
    ###     Industria :
    ###         Valor_Total_do_Item = nf_valor_bruto - nf_desconto - ICMS_Item - PIS - Cofins +ICMS_Substituicao + nf_frete + nf_frete_icms - IPI
    ###    Ultimo_Custo = Valor_Total_do_Item + ICMS_guia + ICMS_recomposicao
    ###    Custo_Medio  = (Ultimo_Custo + Estoque_Financeiro_Medio) / (Qtd_estoque + Quantidade_Total_Item)
    ###    Custo_Reposicao =  (Valor_Total_do_Item + ICMS_guia + nf_frete) / Quantidade_Total_Item
    ###    Custo_Corrigido
    ###      Compra a Prazo:
    ###             Juros = (Taxa_Juros * Prazo_Medio / 30 ) * Valor_Total_Item / 100
    ###      Compra a Vista :
    ###             Juros = 0
    ###      Custo_Anterior = Custo_Corrigido(Atual) * Qtd_estoque * Valor_Indexador
    ###      Custo_Atual = Valor_Total_do_Item - Juros + ICMS_da_guia + ICMS_Recomposicao
    ###      Custo_Corrigido = (Custo_Atual + Custo_Anterior) / (Quantidade_Total_Item +
    ###
    ###
    ###  Observacao do cálculo feita por  Luiz Fernando:
    ###  Segue o cálculo:
    ###  custo medio corrigido =(((custo na nf / qtd entrada) + (custo corr * indexador de hoje * saldo) ) / (qtd entrada + saldo)/ indexador de hoje)
    ###  O custo na nf e: (valor do item - desconto - icms +pis - cofins (+/-) ipi +frete - icms frete +icmsst + seguro + desp.acessorias)
    ###  Se a empresa que e nosso cliente for industria usar -ipi, para comercio usar +ipi
    ###  O custo da nf precisa ser deflacionado se a nota for a prazo, para isto, dividir os juros do parametro por 30 e multiplicar pelo prazo medio da nota.(taxa de juros)
    ###  O custo na nf entao será = custo na nf - (custo na nf * taxa de juros / 100)
    ###
    ######################################################################
        """

        context = context and context.copy() or {}
        if not epc_nota_fiscal_id: # se nota fiscal, nao altera custos
            return False

        # ___________________________ Objects ________________________________
        obj_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_fiscal_encargo = self.pool.get('fiscal.encargo')
        obj_guia = self.pool.get('fiscal.guia')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_produto = self.pool.get('estoque.produto')

        dados_nf_capa = obj_nf_capa.read(cr, uid, epc_nota_fiscal_id,
                                         ['vr_total_nf', 'vr_frete', 'vr_icms_frete', 'vr_desconto', 'condicao_pgto_id',
                                          'company_id', 'data_saida_entrada', 'nf_parcelas_ids',
                                          'fisnfc_parcelas_terceiros_ids', 'nf_propria','tipo_frete',
                                          'fisnfc_nf_frete_ids', 'natureza_id',
                                          'data_emissao'], context=context)
        data_custo = dados_nf_capa.get('data_saida_entrada')
        operacao_id = self.pool.get('fiscal.cfop.natureza').read(cr, uid,
                                                                dados_nf_capa.get('natureza_id')[0],
                                 ['fisnat_operacao_id'], context=context).get('fisnat_operacao_id')
        atualizar_custos = self.pool.get('estoque.natureza.operacao').read(cr, uid, operacao_id[0],
                        ['estnto_atualizar_custos'], context=context).get('estnto_atualizar_custos')
        ICMS_guia = 0
        guia_ids = obj_guia.search(cr, uid, [('nf_id', '=', epc_nota_fiscal_id)], context=context)

        for guia in guia_ids:
            ICMS_guia += obj_guia.read(cr, uid, guia, ['valor'], context=context)['valor']

        # IDs dos encargos
        icms_id = IPI_id = PIS_id = Cofins_id = None
        fiscal_encargo_ids = obj_fiscal_encargo.search(cr, uid, [], context=context)

        if fiscal_encargo_ids:
            for x in obj_fiscal_encargo.browse(cr, uid, fiscal_encargo_ids):
                if x.sigla == 'ICMS':
                    icms_id = x.id

                if x.sigla == 'IPI':
                    IPI_id = x.id

                if x.sigla == 'PIS':
                    PIS_id = x.id

                if x.sigla == 'COFINS':
                    Cofins_id = x.id

        estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [], context=context)

        if estoque_parametro_id:
            dados_estoque_parametro = obj_estoque_parametro.read(
                cr, uid, estoque_parametro_id[0], [
                    'juros', 'estpar_incluir_frete_no_custoreposicao',
                    'estpar_incluir_ipi_no_custoreposicao'], context=context)

            if isinstance(dados_estoque_parametro, (list, tuple)):
                dados_estoque_parametro = dados_estoque_parametro[0]

            if isinstance(dados_estoque_parametro, dict):
                incluir_frete_custo_reposicao = \
                    dados_estoque_parametro['estpar_incluir_frete_no_custoreposicao']
                incluir_ipi_custo_reposicao = \
                    dados_estoque_parametro['estpar_incluir_ipi_no_custoreposicao']
                juros_deflacao = dados_estoque_parametro['juros']
            else:
                juros_deflacao = 0
        else:
            juros_deflacao = 0
            incluir_frete_custo_reposicao = False
            incluir_ipi_custo_reposicao = False

        # Calcula a media de dias das parcelas para calcudo de deflacao
        if juros_deflacao:
            if dados_nf_capa['nf_propria']:
                parcelas = 'nf_parcelas_ids'
            else:
                parcelas = 'fisnfc_parcelas_terceiros_ids'

            browse_parcelas_dados = self.pool.get('fiscal.nf.parcelas').browse(
                cr, uid, dados_nf_capa[parcelas])
            data_inicio = dados_nf_capa['data_emissao']
            dias = 0

            for x in browse_parcelas_dados:
                data_fim = x.vencimento

                if data_fim and data_inicio:
                    dias += calcDiferenca(data_inicio, data_fim)
                    dias = 0 if dias < 0 else dias

                data_inicio = x.vencimento

            media_prazo = len(dados_nf_capa[parcelas]) and dias / len(dados_nf_capa[parcelas])
            vr_deflacao = media_prazo / (juros_deflacao / 100.)
        else:
            vr_deflacao = 0

        nf_item_estoque_ids = obj_movimento_itens.search(cr, uid,
                                              [('nf_id', '=', epc_nota_fiscal_id)], context=context)
        campos = ['produto_id', 'produto_embalagem_id', 'grade_id', 'quantidade', 'vr_total_item',
                  'vr_total_item_rateado',  'vr_despesas_rateado', 'vr_frete_rateado',
                  'vr_seguro_rateado', 'vr_desconto_rateado', 'estmvi_vr_despesas_extras_rateado',
                  'estmvi_vr_icms_frete_rateado']
        dados_itens = obj_movimento_itens.read(cr, uid, nf_item_estoque_ids, campos,
                                               context=context)
        dic_vals_frete_fora = {}
        valores_rateados_guia = []
        dict_icms_frete_rateado = {}

        if ICMS_guia > 0 or dados_nf_capa['tipo_frete'] == 'F':
            obj_adm_utils = self.pool.get('adm.utils')
            lista_valores_itens = []
            vals_frete_por_fora = []
            vals_frete_itens = []

            for item in dados_itens:
                vr_total_item_basico = item.get('vr_total_item') + \
                                       item.get('vr_despesas_rateado') + \
                                       item.get('vr_frete_rateado') + \
                                       item.get('vr_seguro_rateado') - \
                                       item.get('vr_desconto_rateado')
                lista_valores_itens.append([item.get('id'), vr_total_item_basico])
                vals_frete_por_fora.append([item.get('id'), vr_total_item_basico])
                vals_frete_itens.append([item.get('id'), vr_total_item_basico])

            if ICMS_guia > 0:
                valores_rateados_guia = obj_adm_utils.efetuar_rateio(ICMS_guia, lista_valores_itens)

            if dados_nf_capa['tipo_frete'] == 'F' and dados_nf_capa['vr_frete']:
                valores_frete_por_fora = obj_adm_utils.efetuar_rateio(dados_nf_capa['vr_frete'],
                                                                      vals_frete_por_fora)
                dic_vals_frete_fora = dict(valores_frete_por_fora)

                # ICMS frete
                if dados_nf_capa['fisnfc_nf_frete_ids']:
                    obj_adm_utils = self.pool.get('adm.utils')

                    brw_fiscal_frete = obj_nf_capa.browse(
                        cr, uid, dados_nf_capa['fisnfc_nf_frete_ids'])
                    vr_encargo_icms_frete = 0

                    for f in brw_fiscal_frete:
                        for t in f.fisnfc_frete_tributacoes_ids:
                            if t.encargo_id.sigla.upper() == 'ICMS':
                                vr_encargo_icms_frete += t.vr_encargo

                    if vr_encargo_icms_frete:
                        lista_icms_frete_rateado = \
                            obj_adm_utils.efetuar_rateio(vr_encargo_icms_frete, vals_frete_itens)
                        dict_icms_frete_rateado = dict(lista_icms_frete_rateado)

        nf_company_id = dados_nf_capa['company_id'][0]
        epc_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                              'estoque.preco.custo', context=context, user_company_id=nf_company_id)
        regime_tributario, industria_comercio = self._get_regime_atividade(cr, uid, nf_company_id,
                                                                           context=context)
        produto_ids = []
        grade_ids = []
        for i in dados_itens:
            produto_ids.append(i.get('produto_id')[0])
            grade_ids.append(i.get('grade_id')[0])

        qtd_embalagens = self.pool.get('estoque.utils').get_grade_qtd_embalagem(cr, grade_ids,
                                                                                context)
        dict_produto = {}
        map(lambda x: dict_produto.update({x.get('id'): x}),
            obj_estoque_produto.read(cr, uid, produto_ids, ['tipo'],context=context))
        saldos = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, grade_ids,
                                                         time.strftime('%Y-%m-%d'), context=context)
        dict_trib = self._get_dict_tributacoes(cr, uid, nf_item_estoque_ids, context=context)
        for dados_item in dados_itens:
            # Se o CFOP nao for de compra/venda "C", nao pode atualizar os custos
            produto_id = dados_item.get('produto_id')[0]
            grade_id = dados_item['grade_id'][0]
            dados_produto = dict_produto.get(produto_id)
            # Para servico, nao atualizar custos
            if dados_produto.get('tipo') == 'S' or not atualizar_custos:
                continue

            nf_item_id = dados_item.get('id')
            item_embalagem_id = dados_item['produto_embalagem_id'][0]
            item_quantidade = dados_item['quantidade']
            vr_total_item_rateado = dados_item.get('vr_total_item_rateado') or 0
            vr_despesas_extras_rateado = dados_item.get('estmvi_vr_despesas_extras_rateado') or 0
            vr_icms_frete_rateado = dict_icms_frete_rateado and \
                                    dict_icms_frete_rateado.get(nf_item_id) or 0

            # Se existe frete por fora inclui no calculo do custo do produto
            if dic_vals_frete_fora:
                vr_frete_fora_rateado = dic_vals_frete_fora.get(nf_item_id) or 0
                dados_item['vr_frete_rateado'] = vr_frete_fora_rateado
                vr_total_item_rateado +=  vr_frete_fora_rateado

            _CA_ids = self.search(
                cr, uid, [('company_id', '=', epc_company_id), ('produto_id', '=', produto_id),
                          ('estprc_data', '<=', data_custo)],
                order='estprc_data desc', limit=1, context=context)

            if not _CA_ids:
                dados_CA = {}.fromkeys(
                    ['estprc_custo_corrigido', 'estprc_custo_medio_contabil', 'estprc_ultimo_custo',
                     'estprc_custo_reposicao'], 0)
            else:
                dados_CA = self.read(
                    cr, uid, _CA_ids[0], ['estprc_custo_corrigido', 'estprc_custo_medio_contabil',
                                          'estprc_ultimo_custo', 'estprc_custo_reposicao'],
                    context=context)

            ICMS_Item = IPI = PIS = Cofins = ICMS_ST = FCP = 0.0
            dados_item_tributacao = dict_trib.get(nf_item_id)
            if dados_item_tributacao:
                for x in dados_item_tributacao:
                    vr_encargo_item = x.get('vr_encargo')
                    if not x.get('encargo_id'):
                        continue

                    if x['encargo_id'] == icms_id:
                        ICMS_Item = vr_encargo_item
                        if x.get('cst_id'):
                            nf_item_cst_id = x['cst_id']
                            browse_cst = self.pool.get('fiscal.cst').browse(cr, uid, nf_item_cst_id)
                            if browse_cst:
                                if browse_cst.tipo in ('U', 'TU'):
                                    if browse_cst.codigo in ['10', '30', '60', '70']:
                                        FCP = x.get('fisnit_vr_fcp_st') or 0
                                        ICMS_ST += x.get('vr_icms_substituicao') or 0
                                        ICMS_Item = 0

                    elif x['encargo_id'] == IPI_id:
                        IPI = vr_encargo_item
                    elif x['encargo_id'] == PIS_id:
                        PIS = vr_encargo_item
                    elif x['encargo_id'] == Cofins_id:
                        Cofins = vr_encargo_item

            emb_qtd = 1
            if item_embalagem_id:
                chave = '%s-%s' % (grade_id, item_embalagem_id)
                emb_qtd = qtd_embalagens.get(chave) and qtd_embalagens.get(chave).get('qtd') or 1

            Quantidade_Total_Item = emb_qtd * item_quantidade
            saldo = saldos.get(grade_id).get('saldo') - Quantidade_Total_Item

            # Como neste ponto os itens da nf já foram somados ao saldo, é necessário tirar a
            # quantidade da nota do saldo dos produtos para não calcular os custos medios errados
            saldo -= Quantidade_Total_Item
            if saldo < 0:
                saldo = 0

            if industria_comercio != 'C':#quando e industria o valor deve ser subtraido
                IPI *= -1

            vr_total_item_basico = dados_item.get('vr_total_item', 0.0) +\
                                   dados_item.get('vr_despesas_rateado', 0.0) + \
                                   dados_item.get('vr_frete_rateado', 0.0) + \
                                   dados_item.get('vr_seguro_rateado', 0.0) - \
                                   dados_item.get('vr_desconto_rateado', 0.0)

            if not incluir_frete_custo_reposicao:
                vr_total_item_basico -= dados_item.get('vr_frete_rateado')

            if incluir_ipi_custo_reposicao:
                vr_total_item_basico += IPI

            if ICMS_guia > 0:
                vr_icms_guia_rateado = filter(lambda i: (i[0] == dados_item['id']),
                                              valores_rateados_guia) or 0
            else:
                vr_icms_guia_rateado = 0

            if isinstance(vr_icms_guia_rateado, list):
                vr_icms_guia_rateado = vr_icms_guia_rateado[0][1]

            # Verificar Simples Nacional / Lucro Presumido ou Lucro Real
            if regime_tributario in 'SP':  # Simples Nacional ou Lucro Presumido
                ICMS_Item, PIS, Cofins, vr_icms_frete_rateado = 0, 0, 0, 0

            elif regime_tributario == 'R':  # Lucro Real
                vr_icms_frete_rateado = vr_icms_frete_rateado * -1
                PIS = PIS * -1
                Cofins = Cofins * -1
                ICMS_Item = ICMS_Item * -1
                if industria_comercio == 'I':
                    IPI = IPI * -1

            vr_ultimo_custo = (vr_total_item_rateado + ICMS_Item + vr_icms_guia_rateado + ICMS_ST +
                               FCP + PIS + vr_icms_frete_rateado + Cofins + IPI +
                               vr_despesas_extras_rateado) / Quantidade_Total_Item
            vr_medio_contabil = ((vr_ultimo_custo * Quantidade_Total_Item +
                                  dados_CA['estprc_custo_medio_contabil'] * saldo) /
                                  (Quantidade_Total_Item + saldo))
            vr_custo_reposicao = (vr_total_item_basico + ICMS_ST + FCP + vr_icms_guia_rateado +
                                  vr_despesas_extras_rateado) / Quantidade_Total_Item
            vr_custo_corrigido = (((vr_ultimo_custo * Quantidade_Total_Item) +
                                   (dados_CA['estprc_custo_corrigido'] * saldo)) /
                                   (Quantidade_Total_Item + saldo)) - vr_deflacao
            custo_id = self.create(cr, uid, {'company_id': epc_company_id,
                                             'produto_id': produto_id,
                                             'estprc_custo_corrigido': vr_custo_corrigido,
                                             'estprc_custo_medio_contabil': vr_medio_contabil,
                                             'estprc_ultimo_custo': vr_ultimo_custo,
                                             'estprc_custo_reposicao': vr_custo_reposicao,
                                             'estprc_data': data_custo},
                                   context)
            # Atualiza o ID do custo na tabela de movimentacao de itens
            context['super_write_estoque_movimento_itens'] = True
            obj_movimento_itens.write(cr, uid, nf_item_id,
                                      {'estmvi_entrada_preco_custo_id': custo_id}, context)

        return True

    def _get_dict_tributacoes(self, cr, uid, item_ids, context=None):
        'Pega os dados das tributacoes e retorna com a relacao dos ids dos movimentos dos itens'
        context = context and context.copy() or {}
        obj_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
        res = {}
        tributacao_ids = obj_nf_item_tributacao.search(cr, uid,
                                                       [('movimento_itens_id', 'in', item_ids)],
                                                       context=context)
        campos = ['encargo_id', 'vr_encargo', 'cst_id', 'vr_icms_substituicao',
                  'movimento_itens_id', 'fisnit_vr_fcp_st']
        for i in obj_nf_item_tributacao.read(cr, uid, tributacao_ids, campos, context=context,
                                             load='_classic_write'):
            item_id = i.get('movimento_itens_id')
            if not res.get(item_id):
                res[item_id] = [i]
            else:
                res[item_id].append(i)

        return res

    def _get_regime_atividade(self, cr, uid, nf_company_id, context=None):
        'Retorna o regime tributario e o tipo de atividade da empresa emissora da nota'
        context = context and context.copy() or {}
        obj_adm_empresa = self.pool.get('adm.empresa.objeto')
        obj_res_company = self.pool.get('res.company')
        obj_pessoa = self.pool.get('adm.pessoa')
        empresa_id = obj_res_company.search(cr, uid, [('id', '=', nf_company_id)], context=context)
        if not empresa_id:
            raise osv.except_osv(_('Atencao!'), _('Erro no cadastro da empresa!'))

        empresa_id = empresa_id[0]
        pessoa_id = obj_res_company.read(cr, uid, empresa_id, ['rescon_pessoa_id'], context=context)
        if not pessoa_id:
            raise osv.except_osv(_('Atencao'), _('Nao existe Pessoa relacionada a empresa.'))

        pessoa_id = pessoa_id['rescon_pessoa_id'][0]
        dados_pessoa = obj_pessoa.read(cr, uid, pessoa_id, ['admpes_jur_tipo_atividade',
                                                            'admpes_jur_regime_tributario'],
                                       context=context)
        if isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]

        regime_tributario = dados_pessoa.get('admpes_jur_regime_tributario') or ''
        industria_comercio = dados_pessoa.get('admpes_jur_tipo_atividade')
        if industria_comercio != 'I':
            industria_comercio = 'C'

        return regime_tributario, industria_comercio
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
