# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.tools import config as openerp_config
from ...control.utils import adjust_m2o, log_debug
from openerp.osv.fields import float_round
from openerp.tools.translate import _
from openerp.osv import osv
import logging
import time


class estoque_movimento_itens(osv.Model):
    """
    complemento do movimento do estoque
    """
    _inherit = 'estoque.movimento.itens'
    _logger = logging.getLogger(_inherit)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    def onchange_local_propria(self, cr, uid, ids, context=None):
        """
        Remove o item (produto/servico) no formulario de item de nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item da nota fiscal
        :param local: ID do local (estoque.local)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type local: long
        :type context: dict
        :return: item (produto_id) sem valor.
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'produto_id': False}}

    def onchange_local_produto(self, cr, uid, ids, local_id, produto_id=False, grade_id=False):
        return {'value': {'produto_id': False}}

    def estmvi_onchange_grade_propria(self, cr, user, ids, local_id, grade_id, context=None):
        """
        Recupera a embalagem em funcao do local e grade. Apresenta a embalagem se houver apenas uma.
        
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param local_id: o ID do local
        :param grade_id: o ID da grade
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type local_id: long
        :type grade_id: long
        :type context: dict
        :return: a embalagem do item.
        :rtype: dict
        """
        context = context and context.copy() or dict()
        locacao_ids = self.pool.get('estoque.local.locacao').search(
            cr, user, [('local_id', '=', local_id), ('grade_id', '=', grade_id)], context=context)
        locacao_id = locacao_ids and len(locacao_ids) == 1 and locacao_ids[0] or False
        embalagem_id = locacao_id and self.pool.get('estoque.local.locacao').read(
            cr, user, locacao_id, ['estllc_produto_embalagem_id'], context, '_classic_write').get(
            'estllc_produto_embalagem_id') or False
        values = {'produto_embalagem_id': embalagem_id}
        if grade_id:
            produto_id = self.pool.get('estoque.produto.grade').read(
                cr, user, grade_id, ['produto_id'], context, '_classic_write').get(
                'produto_id') or False
            values.update({'produto_id': produto_id})

        return {'value': values}

    def estmvi_onchange_grade(self, cr, uid, ids, produto_id, grade_id, local_id, pessoa_id=None,
                              natureza_id=None, company_id=None, context=None):
        context = context and context.copy() or {}
        val = {}
        if grade_id:
            dados_grade = self.pool.get('estoque.produto.grade').read(cr, uid, grade_id,
                                             ['produto_id'], context=context, load='_classic_write')
            if dados_grade.get('produto_id') != produto_id:
                produto_id = dados_grade.get('produto_id')
                val['produto_id'] = produto_id

            grade_embalagem_id = self.pool.get(
                'estoque.produto.grade').estgrd_get_embalagem_default(cr, uid,
                       produto_id=produto_id, local_id=local_id, grade_id=grade_id, context=context)
            if not grade_embalagem_id:
                return {'warning': {'title': _('Atencao'),
                                    'message': _('Produto sem embalagem cadastrada!')}}
            elif isinstance(grade_embalagem_id, (int, long)):
                val['produto_embalagem_id'] = grade_id and grade_embalagem_id

        else:
            val['produto_embalagem_id'] = False

        return {'value': val}

    def onchange_produto_embalagem_propria(self, cr, user, ids, company_id, local_id, nf_tipo,
                                           produto_id, embalagem_id, pessoa_id=False, quantidade=0,
                                           natureza_id=False, grade_id=False, vr_desconto=0,
                                           vr_total_item=0, context=None):
        """
        Recupera o saldo e quantidade reservada.
        
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param local_id: o ID do local
        :param grade_id: o ID da grade
        :param embalagem_id: o ID da embalagem
        :param quantidade: a quantidade informada 
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type local_id: long 
        :type grade_id: long
        :type embalagem_id: long
        :type quantidade: float
        :type context: dict
        :return: o saldo em estoque e a respectiva quantidade reservada
        :rtype: dict
        """
        context = context and context.copy() or dict()
        values = super(estoque_movimento_itens, self).onchange_produto_embalagem_propria(
            cr, user, ids, company_id, local_id, nf_tipo, produto_id, embalagem_id, pessoa_id,
            quantidade, natureza_id, grade_id, vr_desconto, vr_total_item, context)
        if 'warning' in values:
            values['value'].update(estmvi_saldo=0, estmvi_reservada=0)
            return values
        
        locacao_ids = self.pool.get('estoque.local.locacao').search(
            cr, user, [('local_id', '=', local_id), ('grade_id', '=', grade_id),
                       ('estllc_produto_embalagem_id', '=', embalagem_id)], context=context)
        locacao_dados = locacao_ids and self.pool.get('estoque.local.locacao').read(
            cr, user, locacao_ids[0], ['saldo', 'estllc_qtd_reservada'], context) or {}
        vr_preco_venda = produto_id and self.calcula_vr_produto_embalagem(
            cr, user, company_id, nf_tipo, produto_id, embalagem_id, pessoa_id, natureza_id,
            grade_id, quantidade, vr_desconto, vr_total_item, context) or {}
        values['value'].update(estmvi_saldo=locacao_dados.get('saldo'),
                               estmvi_reservada=locacao_dados.get('estllc_qtd_reservada'),
                               vr_unitario=vr_preco_venda.get('vr_unitario'))
        return values

    def onchange_produto_embalagem(self, cr, uid, ids, company_id, nf_tipo_movimentacao, produto_id,
                                   produto_embalagem_id, vr_preco_venda, pessoa_id=None,
                                   quantidade=0, natureza_id=False, grade_id=False, vr_desconto=0,
                                   vr_total_item=0, local_id=False, context=None):
        vals = {'estmvi_saldo': 0,
                'estmvi_reservada': 0}
        if not produto_embalagem_id:
            return {'value': vals}

        res = super(estoque_movimento_itens, self).onchange_produto_embalagem(cr, uid, ids,
                               company_id, nf_tipo_movimentacao, produto_id, produto_embalagem_id,
                               vr_preco_venda, pessoa_id=pessoa_id, quantidade=quantidade,
                               natureza_id=natureza_id, grade_id=grade_id, vr_desconto=vr_desconto,
                               vr_total_item=vr_total_item, local_id=local_id, context=context)
        vals.update(self.estmvi_get_saldo(cr, uid, local_id, produto_id, grade_id,
                                          produto_embalagem_id, context))
        if not res.get('value').get('vr_preco_venda') and res.get('value').get('vr_unitario'):
            res['value']['vr_preco_venda'] = res.get('value').get('vr_unitario')

        res['value'].update(vals)
        return res

    def estmvi_onchange_item_movimento_propria(self, cr, user, ids, comapny_id, item_id, grade_id,
                                               vr_total_item, destinatario_id, tributacoes_o2m,
                                               tipo_nf, natureza_id=None, context=None,
                                               dest_ender_id=False, vr_despesa=0,
                                               incide_despesa=False, vr_frete=0, vr_seguro=0,
                                               vr_desconto=0):
        """
        Recupera a grade do item (produto/servico) em funcao do local. Apresenta a grade se houver 
        apenas uma. Recupera as tributacoes do item.
        
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param item_id: o ID do item (produto/servico)
        :param destinatario_id: o ID do destinatario
        :param tributacoes_o2m: lista com o ID das tributacoes (one2many)
        :param natureza_id: o ID da natureza da operacao
        :param context: um dicionario padrao para valores contextuais
        :param dest_ender_id: ID do endereco do destinatario
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacoes_o2m: list
        :type natureza_id: long
        :type context: dict
        :type dest_ender_id: long
        :return: a grade do item.
        :rtype: dict
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        if grade_id and grade_id not in self.pool.get('estoque.produto.grade').search(
                cr, user, [('produto_id', '=', item_id)], context=context):
            grade_id = False
        if not grade_id:
            grade_ids = self.pool.get('estoque.produto.grade').search(
                cr, user, [('produto_id', '=', item_id)], context=context)
            grade_id = grade_ids and len(grade_ids) == 1 and grade_ids[0] or False

        # Cria as tributacoes novas
        tipo_tributacao = \
            destinatario_id and self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                cr, user, destinatario_id, context) or 'T'
        tipo_tributacao = 'T' if tipo_tributacao == 'F' else tipo_tributacao
        tipo_tributacao = 'E' if tipo_nf != 'S' else tipo_tributacao
        cfop_id = False
        if natureza_id and item_id:
            # Exclui as tributacoes antigas
            tributacoes_o2m = [[2, tributacao[1], False] for tributacao in filter(
                lambda f: f[0] in (1, 4), tributacoes_o2m)]

            obj_pessoa = self.pool.get('adm.pessoa')
            campos = ['tipo', 'produto_grade_ids', 'grade_usar', 'origem_mercadoria']
            dados_produto = self.pool.get('estoque.produto').read(cr, user, item_id, campos,
                                                                  context)
            campos = ['cfop_id', 'natureza_encargos_ids', 'tipo', 'fisnat_icms_desonerado']
            dados_natureza = self.pool.get('fiscal.cfop.natureza').read(cr, user, natureza_id,
                                                                        campos, context, load)
            campos = ['movimento_tipo', 'codigo', 'sigla']
            dados_cfop_nat = self.pool.get('fiscal.cfop').read(
                cr, user, dados_natureza.get('cfop_id'), campos, context)
            campos = ['encargo_id', 'sobrepor_tributacao_id', 'fiscne_sobrepoe_so_tributados']
            dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(
                cr, user, dados_natureza.get('natureza_encargos_ids'), campos, context, load)
            encargo_ids = map(lambda x: x.get('encargo_id'), dados_nat_encargo)
            dict_encargos_nat = {}
            map(lambda x: dict_encargos_nat.update({x.get('id'): x}),
                self.pool.get('fiscal.encargo').read(cr, user, encargo_ids, ['sigla'], context))
            campos = ['cpf_cnpj', 'endereco_ids', 'admpes_jur_tipo_atividade']
            dados_destinatario = destinatario_id and obj_pessoa.read(
                cr, user, destinatario_id, campos, context) or {}

            # Atualiza os dados do destinatario com o ID do endereco (capa da Nota Fiscal)
            dados_endereco = dest_ender_id and self.pool.get('adm.pessoa.endereco').read(
                cr, user, dest_ender_id, ['uf_id'], context, '_classic_write')
            uf_id = dados_endereco and dados_endereco.get('uf_id')
            dados_destinatario.update(dados_destinatario, destinatario_endereco_id=dest_ender_id)

            rescon_pessoa_id = self.pool.get('res.company').read(cr, user, comapny_id,
                                        ['rescon_pessoa_id'], context, load).get('rescon_pessoa_id')
            tipo_atividade = obj_pessoa.read(cr, user, rescon_pessoa_id,
                                       ['admpes_jur_tipo_atividade'], context).get('tipo_atividade')
            tributacoes_o2m.extend(self.calcular_tributo_itens(
                cr, user, dados_produto, dados_natureza,  dados_nat_encargo, dados_cfop_nat,
                encargo_ids, dados_destinatario, tipo_tributacao, vr_total_item, uf_id=uf_id,
                context=context))

            tributacoes_o2m = \
                self.pool.get('estoque.movimento.itens')._estmvi_ajusta_tributacao_propria(
                    cr, user, vr_total_item, vr_despesa, incide_despesa, vr_frete, vr_seguro,
                    vr_desconto, tributacoes_o2m, context, natureza_id)

            cfop_id = self.estmvi_get_cfop_id(
                cr, user, item_id, dados_produto.get('tipo'), tipo_nf, dados_cfop_nat,
                dados_nat_encargo, dict_encargos_nat, encargo_ids, dados_destinatario,
                tipo_atividade, context)

        values = {'grade_id': grade_id, 'tributacoes_ids': tributacoes_o2m}
        if cfop_id:
            values['cfop_id'] = cfop_id

        # Apenas para nota de saida e se a empresa comercializa combustivel/lubrificante.
        # Ver parametro do Estoque (estpar_comercializa_combustivel_lubrificante)
        if tipo_nf == 'S' and item_id:
            if not self.pool.get('adm.utils').admutl_get_parametros(
                    cr, user, 'estoque.parametro', ['estpar_comercializa_combustivel_lubrificante'],
                    context=context).get('estpar_comercializa_combustivel_lubrificante'):
                return {'value': values}

            # Cuidado aqui, pois algumas bases estao apontando para registros (many2one) que estao
            # devolvendo Codigo ANP = '00000000'. Se nao houver codigo assim (tudo zero) a condicao
            # abaixo vai funcionar, caso contrario, pode haver falha. O correto eh nao haver relacao
            # com a classe ´fiscal.cadastro.anp´ a menos que no parametro do Estoque a opcao
            # "Comercializa combustivel e lubrificantes" esteja marcada.
            codigo_anp_id = self.pool.get('estoque.produto').read(
                cr, user, item_id, ['estpro_codigo_anp_id'], context, load).get(
                'estpro_codigo_anp_id')
            dados_anp = codigo_anp_id and self.pool.get('fiscal.cadastro.anp').read(
                cr, user, codigo_anp_id, ['fisanp_codigo'], context) or {}
            if dados_anp and int(dados_anp.get('fisanp_codigo') or ''):
                company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                    cr, user, self._name, context=context)
                pessoa_id = self.pool.get('res.company').read(
                    cr, user, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                    'rescon_pessoa_id')
                dados_pessoas = self.pool.get('adm.pessoa').read(
                    cr, user, [destinatario_id, pessoa_id], ['uf'], context)

                # CFOP para oleos
                codigo_cfop = '5656'  # Ou 6656 para outros estados

                # Verifica se o Estado do destinatario eh diferente do emitente
                if dados_pessoas[0].get('uf') and dados_pessoas[1].get('uf'):
                    if dados_pessoas[0].get('uf') != dados_pessoas[1].get('uf'):
                        codigo_cfop = '6656'  # CFOP para oleos (outros estados)
                cfop_ids = self.pool.get('fiscal.cfop').search(
                    cr, user, [('codigo', '=', codigo_cfop)], context=context)
                if cfop_ids:
                    values['cfop_id'] = cfop_ids[0]

        return {'value': values}

    def estmvi_onchange_item_movimento(self, cr, uid, ids, empresa, natureza, nf_tipo, destinatario,
                                       item, local, preco_venda, total, nf_propria, nf_devolucao,
                                       endereco=False, grade_id=False, tributacao=None,
                                       context=None):
        context = context and context.copy() or {}
        load = '_classic_write'
        res = super(estoque_movimento_itens, self).estmvi_onchange_item_movimento(
            cr, uid, ids, empresa, natureza, nf_tipo, destinatario, item, local, preco_venda, total,
            nf_propria, nf_devolucao, endereco, grade_id, tributacao, context)
        if item and not natureza:
            res['warning'] = {'title': _('Atencao'),
                              'message': _('A natureza da operacao da capa deve ser informada '
                                           'antes de inserir o item, para o prenchimento '
                                           'automatico do CFOP do item.')}
            res['value']['produto_id'] = False
            return res
        res['value']['natureza_id'] = natureza  # Atualiza a natureza do item
        if ids:
            browse_nf = self.browse(cr, uid, ids[0])
            base_produto_id = browse_nf.produto_id and browse_nf.produto_id.id or False
            atual_igual_anterior = item == base_produto_id
        else:
            atual_igual_anterior = True
        if not item or not atual_igual_anterior:
            res['value'].update({}.fromkeys(
                ['vr_unitario', 'vr_total_item', 'vr_desconto_rateado', 'vr_total_item_rateado',
                 'quantidade', 'vr_preco_venda', 'vr_despesas_rateado'], 0))
            res['value'].update({}.fromkeys(['produto_embalagem_id', 'grade_id',
                                             'verifica_usa_grade', 'cfop_id'], ''))
            res['value']['tributacoes_ids'] = list()
        if item:
            # Verifica se o produto esta cadastrado para o local, caso nao esteja, e mostrado
            # um alerta informando.
            if local and nf_tipo == 'E':
                nf_item_local = self.pool.get('estoque.produto').estoque_check_produto_in_local(
                    cr, uid, company_id=empresa, produto_id=item, local_id=local,
                    context=context)
                if not nf_item_local:
                    # A mensagem deve ser somente um alerta.
                    res['warning'].update({
                        'title': _('Atencao!'),
                        'message': _('Este item nao encontra-se cadastrado para o local '
                                     'informado. Ao grava-lo, o local sera adicionado ao '
                                     'cadastro.')})
                    return res

            # Le os dados do item
            campos = ['tipo', 'produto_grade_ids', 'grade_usar', 'origem_mercadoria']
            dados_produto = self.pool.get('estoque.produto').read(cr, uid, item, campos, context)
            res['value']['verifica_usa_grade'] = dados_produto.get('grade_usar')

            obj_pessoa = self.pool.get('adm.pessoa')
            campos = ['cpf_cnpj', 'endereco_ids', 'admpes_jur_tipo_atividade']
            dados_destinatario = obj_pessoa.read(cr, uid, destinatario, campos, context)
            rescon_pessoa_id = self.pool.get('res.company').read(
                cr, uid, empresa, ['rescon_pessoa_id'], context, load).get('rescon_pessoa_id')
            tipo_atividade = obj_pessoa.read(
                cr, uid, rescon_pessoa_id, ['admpes_jur_tipo_atividade'], context).get(
                'tipo_atividade')
            campos = ['cfop_id', 'natureza_encargos_ids', 'tipo', 'fisnat_icms_desonerado']
            dados_natureza = self.pool.get('fiscal.cfop.natureza').read(
                cr, uid, natureza, campos, context, load)
            campos = ['movimento_tipo', 'codigo', 'sigla']
            dados_cfop_nat = self.pool.get('fiscal.cfop').read(
                cr, uid, dados_natureza.get('cfop_id'), campos, context)
            campos = ['encargo_id', 'sobrepor_tributacao_id', 'fiscne_sobrepoe_so_tributados']
            dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(
                cr, uid, dados_natureza.get('natureza_encargos_ids'), campos, context, load)
            encargo_ids = map(lambda x: x.get('encargo_id'), dados_nat_encargo)
            dict_encargos_nat = {}
            map(lambda x: dict_encargos_nat.update({x.get('id'): x}),
                self.pool.get('fiscal.encargo').read(cr, uid, encargo_ids, ['sigla'], context))

            # Se o item tiver somente uma grade, preencher campo se nao, o limpa
            produto_grade_ids = dados_produto.get('produto_grade_ids')
            if not grade_id or grade_id not in produto_grade_ids:
                if len(produto_grade_ids) == 1:
                    produto_grade_id = produto_grade_ids[0]
                    res['value']['grade_id'] = produto_grade_id
                else:
                    res['value']['grade_id'] = False
            else:
                res['value']['grade_id'] = grade_id

            # Se o item for produto e tiver somente uma embalagem, preencher o campo
            if dados_produto.get('tipo') == 'P':
                local_locacao_ids = self.pool.get('estoque.local.locacao').search(
                    cr, uid, [('local_id', '=', local), ('produto_id', '=', item)],
                    context=context)

                # Verifica se produto está cadastrado para o local selecionado
                if not local_locacao_ids:
                    res['warning'].update({
                        'title': _('Atencao!'),
                        'message': _('Item nao esta cadastrado para este local.')})
                    return res
            else:  # Servico
                embalagem_id = self.pool.get('adm.utils').admutl_get_parametros(
                    cr, uid, 'estoque.parametro', ['estpar_embalagem_servico_id'],
                    company_id=empresa, context=context).get('estpar_embalagem_servico_id')
                if not embalagem_id:
                    res['warning'].update({
                        'title': _('Atencao!'),
                        'message': _('Sem embalagem para servico no parametro do estoque')})
                produto_embalagem_ids = self.pool.get('estoque.produto.embalagem').search(
                    cr, uid, [('produto_id', '=', item),
                              ('embalagem_id', '=', embalagem_id)], context=context)
                if produto_embalagem_ids:
                    produto_embalagem_id = produto_embalagem_ids[0]
                else:
                    produto_embalagem_id = self.pool.get('estoque.produto.embalagem').create(
                        cr, uid, {'produto_id': item, 'embalagem_id': embalagem_id},
                        context)
                res['value']['produto_embalagem_id'] = produto_embalagem_id

            if natureza:
                if nf_propria:
                    context.update({'proprio': True})
                else:
                    context.update({'terceiro': True})

                cfop_id = self.estmvi_get_cfop_id(cr, uid, item, dados_produto.get('tipo'), nf_tipo,
                                                  dados_cfop_nat, dados_nat_encargo,
                                                  dict_encargos_nat, encargo_ids,
                                                  dados_destinatario, tipo_atividade, context)
                res['value']['cfop_id'] = cfop_id
                tipo_tributacao = ''
                if nf_tipo == 'S':
                    tipo_tributacao = 'T'
                elif nf_tipo == 'E':
                    tipo_tributacao = 'E'
                    if not nf_propria:
                        context.update({'terceiro': True, 'onchange_terceiro': True})

                res['value']['tributacoes_ids'] = self.calcular_tributo_itens(cr, uid,
                             dados_produto, dados_natureza, dados_nat_encargo, dados_cfop_nat,
                             encargo_ids, dados_destinatario, tipo_tributacao,
                             total, context=context)

                # Verifica se existem os encargos ICMS, PIS e COFINS para notas de entrada terceiro
                if context.get('entrada_terceiros'):
                    cr.execute("""SELECT id, sigla 
                                  FROM   fiscal_encargo 
                                  WHERE  sigla ilike 'ICMS'
                                     OR  sigla ilike 'PIS'
                                     OR  sigla ilike 'COFINS'""")
                    encargos = map(lambda m: m[2].get('encargo_id'), res['value']['tributacoes_ids'])
                    ausentes = [e[1] for e in cr.fetchall() if e[0] not in encargos]
                    ausentes = ausentes and _('Os seguintes encargos estao ausentes para o '
                                              'item informado:\n * %s') % '\n* '.join(ausentes)
                    res['warning'] = ausentes and {
                        'title': _('Aviso.'),
                        'message': ausentes
                    } or {}
            else:
                res['warning'] = {
                    'title': _('Atencao!'),
                    'message': _('A natureza da operacao da capa deve ser informada antes de '
                                 'inserir o item, para o prenchimento automatico do CFOP.')}
                return res

        return res

    def estmvi_onchange_grade_nf_terceiros(self, cr, uid, ids, produto_id, grade_id, local_id,
                                           vr_unitario, context=None):
        context = context and context.copy() or {}
        res = {'value':{}}
        if not grade_id:
            res['value']['produto_embalagem_id'] = ''
            return res

        dados_grade = self.pool.get('estoque.produto.grade').read(cr, uid, grade_id,
                                         ['produto_id'], context=context, load='_classic_write')
        if dados_grade.get('produto_id') != produto_id:
            produto_id = dados_grade.get('produto_id')
            res['value']['produto_id'] = produto_id

        grade_embalagem_id = self.pool.get('estoque.produto.grade').estgrd_get_embalagem_default(
            cr, uid, produto_id=produto_id, local_id=local_id, grade_id=grade_id, context=context)
        if not grade_embalagem_id:
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Produto sem embalagem cadastrada!')}
            return res
        else:
            if type(grade_embalagem_id) is list and len(grade_embalagem_id) == 1:
                grade_embalagem_id = grade_embalagem_id[0]

            res['value']['produto_embalagem_id'] = grade_embalagem_id
            qtde_embalagem = self.get_qtde_embalagem(cr, uid, grade_embalagem_id, context=context)
            res['value']['vr_unitario'] = vr_unitario * qtde_embalagem

        return res

    def estmvi_get_saldo(self, cr, uid, local_id, produto_id, grade_id, produto_emb_id,
                         context=None):
        """
        O metodo fara a busca do saldo em estoque de um produto, baseados no produto, grade e no
        local em que se encontra. Retorna dicionario com as informacoes de saldo e quantidade
        reservada
        @param cr: cursor do banco de dados
        @param uid: id do usuario
        @param local_id: id da tabela estoque_local
        @param produto_id: id da tabela estoque_produto
        @param grade_id: id da tabela estoque_produto_grade
        @param context: dicionario com valores contextuais
        """
        context = context and context.copy() or {}
        vals = {'estmvi_saldo': 0, 'estmvi_reservada': 0}
        if not all([local_id, produto_id, grade_id]):
            return vals
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        data = time.strftime('%Y-%m-%d')
        saldo = obj_estoque_produto_saldo.get_saldo_do_dia(
            cr, uid, grade_id, data, local_id, context=context)
        saldo = saldo.get(grade_id)
        if saldo:
            vals['estmvi_saldo'] = \
                saldo.get('saldo_na_embalagem').get(produto_emb_id).get('saldo') or 0
            vals['estmvi_reservada'] = \
                saldo.get('saldo_na_embalagem').get(produto_emb_id).get('qtd_reservada') or 0

        return vals

    def calcula_cfop_substituicao(self, cr, uid, produto_id, produto_tipo, tipo_nota, dados_cfop,
                                  encargo_ids, dados_pessoa, tipo_atividade, context=None):
        """
        Recupera o CFOP de substituicao tributaria.

        :param cr:
        :param uid:
        :param produto_id:
        :param produto_tipo: o tipo do item: "P"= Produto, "S"= Servico
        :param tipo_nota: o tipo da nota fiscal: "S"= Saida, "E"= Entrada
        :param natureza_id: o ID da natureza da operacao
        :param dados_pessoa: dados do destinatario (cliente/fornecedor)
        :param context:
        :type cr: object
        :type uid: int
        :type produto_id: long
        :type produto_tipo: str
        :type tipo_nota: str
        :type natureza_id: long
        :type dados_pessoa: dict
        :return: o ID do CFOP de substituicao.
        :rtype: long

        """
        context = context and context.copy() or {}
        cfop_codigo = dados_cfop.get('codigo') or ''

        if dados_cfop.get('movimento_tipo'):
            nf_cfop_codigo = dados_cfop.get('codigo') or ''
            if not encargo_ids:
                encargo_ids = []
            context['dados_cfop'] = dados_cfop
            tributacao_ids = self.pool.get('fiscal.utils').calcular_tributos(
                cr, uid, produto_id, dados_pessoa, 'T' if tipo_nota == 'S' else 'E', encargo_ids,
                context)
            dados_tributacao_ids = []
            campos = ['encargo_id', 'cst_id']
            load = '_classic_write'
            if context.get('terceiro') or context.get('entrada_terceiros') or \
                    context.get('busca_fiscal_tributacao'):
                dados_tributacao_ids = self.pool.get('fiscal.tributacao').read(
                    cr, uid, tributacao_ids, campos, context, load)

            if context.get('proprio') and tipo_nota == 'E' and not \
                    context.get('busca_fiscal_tributacao'):
                dados_tributacao_ids = self.pool.get('fiscal.tributacao').read(
                    cr, uid, tributacao_ids, campos, context, load)

            if context.get('proprio') and tipo_nota == 'S' and not \
                    context.get('busca_fiscal_tributacao'):
                dados_tributacao_ids = self.pool.get('fiscal.tributacao.encargos').read(
                    cr, uid, tributacao_ids, campos, context, load)
            if dados_tributacao_ids:
                _encargo_ids = []
                _cst_ids = []
                for i in dados_tributacao_ids:
                    _encargo_ids.append(i.get('encargo_id'))
                    _cst_ids.append(i.get('cst_id'))

                dict_encargos = {}
                map(lambda x: dict_encargos.update({x.get('id'): x}),
                    self.pool.get('fiscal.encargo').read(cr, uid, _encargo_ids, ['sigla'], context))

                dict_cst = {}
                map(lambda x: dict_cst.update({x.get('id'):x}),
                    self.pool.get('fiscal.cst').read(cr, uid, _cst_ids, ['tipo'], context))

                # Filtra as tributacoes que sao ICMS/ST
                dados_tributacao_encargos = []
                for i in dados_tributacao_ids:
                    encargo = dict_encargos.get(i.get('encargo_id'))
                    cst = dict_cst.get(i.get('cst_id'))
                    if (encargo.get('sigla') in ('ICMS') and cst.get('tipo') in ('U', 'TU')) or \
                                                                    encargo.get('sigla') in ('ISS'):
                        dados_tributacao_encargos.append(i)
                producao_propria = self.pool.get('estoque.produto').read(cr, uid, produto_id,
                                              ['producao_propria'], context).get('producao_propria')
                if dados_tributacao_encargos:
                    cfops_energia_eletrica = ['1153', '1207', '1251', '1252', '1253', '1254',
                                              '1255', '1256', '1257', '1305', '1355', '2153',
                                              '2207', '2251', '2252', '2253', '2254', '2255',
                                              '2256', '2257', '2305', '2355']
                    cfops_nao_calcular = ['1910', '2910', '1911', '2911', '5929', '6929', '1407', '2407', '3102']
                    cfops_nao_calcular += cfops_energia_eletrica

                    # Se for cfOp de bonificacao ele prevalecera sobre o de substituicao, e o de
                    # combustivel tambem. E se atividade nao for atacadista com o CFOP 5405
                    if (nf_cfop_codigo not in cfops_nao_calcular and
                                nf_cfop_codigo[1:3] not in ('65', '66')) and not \
                            (tipo_atividade == 'A' and nf_cfop_codigo == '5405'):
                        primeiro_digito_cfop = nf_cfop_codigo[:1]
                        if produto_tipo == 'S' and int(primeiro_digito_cfop) > 4:
                            cfop_codigo = primeiro_digito_cfop + '933'
                        else:
                            if (tipo_atividade == "C" or not tipo_atividade) and tipo_nota == 'S':
                                # 6202 = devolucao, nao troca o cfop
                                if dados_cfop.get('movimento_tipo') == 'T':
                                    if nf_cfop_codigo in ('5152', '5409', '6152', '6409'):
                                        cfop_codigo = primeiro_digito_cfop + '409'
                                elif primeiro_digito_cfop == '6' and nf_cfop_codigo not in ['6202']:
                                    cfop_codigo = primeiro_digito_cfop + '404'
                                else:
                                    # Verifica se CFOP eh de devolucao em nota fiscal de saida
                                    if nf_cfop_codigo[1:4] in ('202', '411'):
                                        cfop_codigo = primeiro_digito_cfop + '411'
                                    else:
                                        cfop_codigo = primeiro_digito_cfop + '405'
                                if producao_propria:
                                    cfop_codigo = self._estmvi_producao_propria_como_revenda(
                                        cr, uid, cfop_codigo, context)
                            else:
                                if nf_cfop_codigo[1:4] in ('202', '411'):  # Devolucao
                                    cfop_codigo = primeiro_digito_cfop + '411'
                                else:
                                    cfop_codigo = primeiro_digito_cfop + '403'
                elif producao_propria:
                    cfop_codigo = self._estmvi_producao_propria_como_revenda(
                        cr, uid, cfop_codigo, context)

        # Verifica se o CFOP esta cadastrado
        cfop_ids = self.pool.get('fiscal.cfop').search(
            cr, uid, [('codigo', '=', cfop_codigo)], context=context)
        if cfop_codigo and not cfop_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao existe CFOP %s cadastrado para substituicao tributaria.') %
                                 cfop_codigo)

        return cfop_codigo and cfop_ids[0] or dados_cfop.get('id')

    def calcula_vr_produto_embalagem(self, cr, uid, company_id, nf_tipo_movimentacao, produto_id,
                                     produto_embalagem_id, pessoa_id=None, natureza_id=False,
                                     grade_id=False, quantidade=0, vr_desconto=0, vr_total_item=0,
                                     context=None):
        context = context and context.copy() or {}
        val_produto_embalagem = {}.fromkeys(['vr_unitario'], 0)

        # Se for servico a embalagem jah esta definida no parametro ´estpar_embalagem_servico_id´
        # que aponta direto para a classe ´estoque_embalagem´ e nao ´estoque_produto_embalagem´
        item_dados = produto_id and self.pool.get('estoque.produto').read(
            cr, uid, produto_id, ['tipo'], context)
        tipo = item_dados and item_dados.get('tipo')
        if tipo == 'S':  # Item servico
            embalagem_id = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'estoque.parametro', ['estpar_embalagem_servico_id'],
                company_id=company_id, context=context).get('estpar_embalagem_servico_id')
            if not embalagem_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('Sem embalagem para servico no parametro do estoque.'))
            qtde_embalagem = self.pool.get('estoque.embalagem').read(
                cr, uid, embalagem_id, [], context).get('quantidade_na_embalagem') or 1
        else:
            qtde_embalagem = self.get_qtde_embalagem(cr, uid, produto_embalagem_id, context=context)

        if nf_tipo_movimentacao == 'S':
            cfop_id = natureza_id and self.pool.get('fiscal.cfop.natureza').browse(
                cr, uid, natureza_id).cfop_id.id
            preco_venda = cfop_id and self.estmvi_get_preco_unitario(
                cr, uid, [], produto_id, cfop_id, pessoa_id, company_id, grade_id=grade_id,
                context=context)
            val_produto_embalagem['vr_unitario'] = preco_venda * qtde_embalagem
        elif quantidade:
            vr_unitario = (float(vr_total_item) + float(vr_desconto)) / float(quantidade)
            val_produto_embalagem['vr_unitario'] = vr_unitario

        return val_produto_embalagem

    def get_valor_total(self, cr, user, ids, context=None):
        """
        Somatorio dos valores dos itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: o total (somatorio) dos valores dos itens
        :rtype: float
        """
        context = context and context.copy() or {}
        valores = self.pool.get('fiscal.nf.capa').resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', ids, ['vr_total_item'], context)

        return sum(map(lambda m: isinstance(m, dict) and m['vr_total_item'] or 0, valores))

    def onchange_despesas_acessorias(self, cr, uid, ids, vr_despesas,
                                     estoque_movimento_itens_ids, context=None):
        context = context and context.copy() or {}
        nf_id = context.get('nf_id', False)

        if isinstance(estoque_movimento_itens_ids, (list, tuple)):
            itens_ids = [x[1] for x in estoque_movimento_itens_ids]
            for x in estoque_movimento_itens_ids:
                if x[1]:
                    if x[1] in ids:
                        itens_ids.remove(x[1])
                    browse_itens = self.browse(cr, uid, itens_ids)
                    total_despesas = (sum(map(lambda d: d.vr_despesas_rateado or 0, browse_itens)))
                    total_despesas += vr_despesas
                    if total_despesas == 0:
                        self.pool.get('fiscal.nf.capa').write(
                            cr, uid, nf_id, {'fisnfc_despesa_acessoria_capa': True,
                                             'vr_despesas_acessorias': total_despesas}, context)
                    else:
                        self.pool.get('fiscal.nf.capa').write(
                            cr, uid, nf_id, {'vr_despesas_acessorias': total_despesas,
                                             'fisnfc_despesa_acessoria_capa': False}, context)

        return {'value': {}}

    def check_nf_item_local_produto(self, cr, uid, company_id, local_id, produto_id, grade_id,
                                    embalagem_id, context=None):
        # Verifica se o produto esta no local informado, caso nao esteja, e adiconado
        context = context and context.copy() or {}
        local_locacao_id = 0
        local_produto_ids = self.pool.get('estoque.produto').estoque_check_produto_in_local(
            cr, uid, company_id, produto_id, local_id, context)
        if not local_produto_ids:
            local_locacao_id = self.pool.get('estoque.local.locacao').create(
                cr, uid, {'local_id': local_id, 'produto_id': produto_id, 'company_id': company_id,
                          'grade_id': grade_id, 'estllc_produto_embalagem_id': embalagem_id},
                context)
        return local_locacao_id

    def calcular_tributo_itens(self, cr, uid, dados_produto, dados_natureza, dados_nat_encargo,
                               dados_cfop_nat, encargo_ids, dados_destinatario, tipo_tributacao,
                               vr_item_rateado, tributacoes_o2m=None, uf_id=False, context=None,
                               despesa_valor=0, incide_despesa=False):
        """
        Calcula a tributacao para o item.

        :param cr: cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param dados_produto: dados do item (produto/servico)
        :param dados_natureza: dados da natureza da operacao
        :param dados_nat_encargo: dados do encargo da natureza
        :param dados_cfop_nat: dados do CFOP
        :param encargo_ids: lista com o ID dos encargos (ICMS, PIS, COFINS, etc)
        :param dados_destinatario:  dados do destinatario da nota fiscal
        :param tipo_tributacao: ´E´=Entrada, ´S´=saida ou ´T´=Todos/consumidor final
        :param vr_item_rateado: valor rateado do item
        :param tributacoes_o2m: lista com os dados da tributacao do item (formato one2many)
        :param uf_id: o ID do estado (UF) do destinatario
        :param context: um dicionario padrao para valores contextuais
        :param despesa_valor: o valor da despesa (rateada) 
        :param incide_despesa: incide ou nao o valor da despesa (rateada) na base de calculo
        :type cr: object
        :type uid: int
        :type dados_produto: dict
        :type dados_natureza: dict
        :type dados_nat_encargo: dict
        :type dados_cfop_nat: dict
        :type encargo_ids: list
        :type dados_destinatario: dict
        :type tipo_tributacao: str
        :type vr_item_rateado: float
        :type tributacoes_o2m: list
        :type uf_id: long
        :type context: dict
        :type incide_despesa: bool
        :return: list
        :rtype: lista com os dados da tributacao do item com valores ajustados (one2many).
        """
        context = context and context.copy() or {}
        # Todo: Levar calculos e informacoes referente aos impostos dos itens para a classe
        # Todo: fiscal.nf.item.tributacao
        tributos = []
        if not tributacoes_o2m:
            tributacoes_o2m = []
        obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        obj_tributacao_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]
        aliquota_venda = percentual_mva = vr_icms_substituicao = vr_base_icms_substituicao = 0
        modbcst = ''
        item_trib_ids = []
        for i in tributacoes_o2m:
            if i[1]:
                item_trib_ids.append(i[1])
        campos = ['encargo_id', 'aliquota_venda', 'percentual_mva', 'modbcst',
                  'vr_icms_substituicao']
        dict_item_trib = {}
        _encargo_ids = []
        if item_trib_ids:
            for i in obj_tributacao_nf_item_tributacao.read(cr, uid, item_trib_ids, campos,
                                                            context):
                dict_item_trib.update({i.get('id'): i})
                _encargo_ids.append(i.get('encargo_id'))
        dict_encargos = {}
        if _encargo_ids:
            map(lambda x: dict_encargos.update({x.get('id'): x.get('sigla')}),
                self.pool.get('fiscal.encargo').read(cr, uid, _encargo_ids, ['sigla'], context))
        for i in tributacoes_o2m:
            if i[0] == 1 and isinstance(i[2], dict):
                dados_nf_item_tributacao = dict_item_trib.get(i[1])
                encargo_id = dados_nf_item_tributacao.get('encargo_id')
                if dict_encargos.get(encargo_id).upper() == 'ICMS':
                    if not i[2].get('aliquota_venda'):
                        aliquota_venda = dados_nf_item_tributacao.get('aliquota_venda')
                    if not i[2].get('percentual_mva'):
                        percentual_mva = dados_nf_item_tributacao.get('percentual_mva')
                    if not i[2].get('modbcst'):
                        modbcst = dados_nf_item_tributacao.get('modbcst')
                    if not i[2].get('vr_icms_substituicao'):
                        vr_icms_substituicao = dados_nf_item_tributacao.get('vr_icms_substituicao')
        context['dados_natureza'] = {'id': dados_natureza.get('id'),
                                     'cfop_id': {
                                         'movimento_tipo': dados_cfop_nat.get('movimento_tipo')}}
        impostos = self.pool.get('fiscal.utils').calcular_imposto(
            cr, uid, dados_produto.get('id'), dados_destinatario=dados_destinatario,
            tipo_tributacao=tipo_tributacao, vr_produto=vr_item_rateado, encargo_ids=encargo_ids,
            context=context, dados_natureza=dados_natureza)
        dict_tributacoes = {}
        sobre_trib_ids = []
        for i in dados_nat_encargo:
            i.get('sobrepor_tributacao_id') and sobre_trib_ids.append(
                                                                    i.get('sobrepor_tributacao_id'))

        cst_ids = []
        dict_sobrepor_trib = {}
        if sobre_trib_ids:
            for i in obj_fiscal_tributacao.read(cr, uid, sobre_trib_ids,
                                         ['cst_id', 'aliquota', 'base'], context, '_classic_write'):
                dict_sobrepor_trib.update({i.get('id'): i})
                i.get('cst_id') and cst_ids.append(i.get('cst_id'))

        for i in impostos:
            dict_tributacoes[i.get('tributacao_id')] = {
                'aliquota': i.get('aliquota'), 'id': i.get('tributacao_id'),
                'cst_id': i.get('cst_id'), 'base': i.get('valor_base')}
            cst_ids.append(i.get('cst_id'))

        dict_cst = {}
        map(lambda x: dict_cst.update({x.get('id'): x}),
            self.pool.get('fiscal.cst').read(cr, uid, cst_ids, ['tipo'], context))
        for i in dados_nat_encargo:
            sobrepor_trib_id = i.get('sobrepor_tributacao_id')
            for imp in impostos:
                if imp.get('encargo_id') == i.get('encargo_id'):
                    # Nao sera feita sobreposicao de tributacao para ICMS desonerado
                    if sobrepor_trib_id and not imp.get('fisnit_vr_icms_desonerado'):
                        tributacao_dados = dict_sobrepor_trib.get(sobrepor_trib_id)
                        if i.get('fiscne_sobrepoe_so_tributados'):
                            if imp['tipo_cst'] == 'T':
                                tributacao_id = sobrepor_trib_id
                                cst_id = tributacao_dados.get('cst_id') or False
                                imp['aliquota'] = tributacao_dados.get('aliquota') or 0
                                imp['percentual_base'] = tributacao_dados.get('base') or 0
                            else:
                                tributacao_id = imp.get('tributacao_id')
                                cst_id = imp['cst_id'] or False
                        else:
                            tributacao_id = sobrepor_trib_id
                            cst_id = tributacao_dados.get('cst_id') or False
                            imp['aliquota'] = tributacao_dados.get('aliquota') or 0
                            imp['percentual_base'] = tributacao_dados.get('base') or 0
                        imp['valor_base'] = vr_item_rateado * imp['percentual_base'] / 100.
                        imp['valor_imposto'] = imp['valor_base'] * imp['aliquota'] / 100.
                    else:
                        tributacao_id = imp['tributacao_id']
                        cst_id = imp['cst_id'] or False
                    dados_cst = dict_cst.get(cst_id)
                    tipo_cst = dados_cst.get('tipo')
                    if tipo_cst not in ('U', 'TU'):
                        imp['vr_base_icms_substituicao'] = 0

                    total_base = vr_item_rateado
                    tot_base_reduzida = total_base - total_base * imp['percentual_base'] / 100.

                    # Incide ou nao o valor da despesa rateada na base de calculo
                    tot_base_reduzida += despesa_valor if not incide_despesa else 0
                    if imp.get('encargo_id') != icms_id or not imp.get('aliquota') < 100:
                        tot_base_reduzida = 0
                    else:  # Recalcula o valor do encargo e ICMS desonerado (se houver)
                        if dados_natureza.get('fisnat_icms_desonerado'):
                            imp['valor_imposto'] = \
                                imp.get('aliquota') * (total_base - tot_base_reduzida) / 100.
                            imp['fisnit_vr_icms_desonerado'] = imp.get('valor_imposto')
                    total_imposto = imp['valor_imposto']

                    total_base -= tot_base_reduzida
                    tipo_substituicao = False
                    if cst_id:
                        if cst_id not in dict_cst:
                            dict_cst.update({cst_id: self.pool.get('fiscal.cst').read(
                                cr, uid, cst_id, ['tipo'], context)})
                        if dados_cst.get('tipo') in ('U', 'TU'):
                            tipo_substituicao = True
                        else:
                            imp['modbcst'] = ''
                            imp['tipo_cst'] = dados_cst.get('tipo')
                    if imp['nome_encargo'].upper() == 'ICMS':
                        percentual_mva = (percentual_mva or imp.get('mva')) or 0
                        aliquota_venda = aliquota_venda or imp.get('aliquota_venda') or 0
                    else:
                        percentual_mva = imp.get('mva') or 0
                        aliquota_venda = imp.get('aliquota_venda') or 0
                    if imp['modbcst'] == '4':
                        busca_info_st = \
                            obj_fiscal_nf_item_tributacao.fisnit_calcular_vr_icms_st_item(
                                round(imp['valor_base'], 2), imp['aliquota'], aliquota_venda,
                                percentual_mva)
                        vr_icms_substituicao = busca_info_st.get('vr_icms_substituicao') or 0
                        vr_base_icms_substituicao = \
                            busca_info_st.get('vr_base_icms_substituicao') or 0
                    valores = {'encargo_id': imp['encargo_id'], 'cst_id': cst_id,
                               'tributacao_id': tributacao_id, 'aliquota': imp['aliquota'],
                               'vr_base_calculo': round(total_base, 2),
                               'vr_encargo': round(total_imposto, 2),
                               'modbcst': (modbcst or imp['modbcst']) or 0,
                               'vr_reducao_base': round(tot_base_reduzida, 2),
                               'fisnit_csosn_id': imp.get('csosn_id'),
                               'vr_icms_substituicao': vr_icms_substituicao,
                               'vr_base_icms_substituicao': vr_base_icms_substituicao,
                               'percentual_mva': percentual_mva, 'aliquota_venda': aliquota_venda,
                               'tipo_substituicao': tipo_substituicao,
                               'fisnit_tipo_cst': dados_cst.get('tipo'),
                               'fisnit_vr_icms_desonerado': imp.get('fisnit_vr_icms_desonerado')}
                    tributo_existente = False
                    for tributo in tributacoes_o2m:
                        if tributo[0] == 4 or tributo[0] == 1:
                            encargo_salvo_id = dict_item_trib.get(tributo[1]).get('encargo_id')
                            if encargo_salvo_id == i.get('encargo_id'):
                                tributo_existente = True
                                tributos.append([1, tributo[1], valores])
                                break
                    if not tributo_existente:
                        tributos.append([0, False, valores])

        return tributos

    def read(self, cr, uid, ids, fields=[], context=None, load='_classic_read'):
        context = context and context.copy() or {}
        # Este contexto serve para melhorar a performace, tomar cuidado ao usa-lo
        if context.get('super_read_estoque_movimento_itens'):
            return super(estoque_movimento_itens, self).read(cr, uid, ids, fields, context=context,
                                                             load=load)

        res = super(estoque_movimento_itens, self).read(cr, uid, ids, fields, context, load)
        for movimento in isinstance(res, (list, tuple)) and res or [res]:
            if not all(['quantidade' in movimento, 'vr_desconto' in movimento,
                        'vr_total_item' in movimento]):
                continue
            ###################################
            # A utilizacao do metodo ´float_round´ por algum motivo causa erro de javascipt quando
            # um item jah persistido recebe um novo encargo. A sequencia de metodos float() e
            # float_round() foi substituida apenas por round() jah que o resultado (aparentemente)
            # eh o mesmo.
            #
            # movimento['vr_unitario'] = float(float_round(self.estmvi_get_valor_unitario(
            #     movimento.get('quantidade'), movimento.get('vr_desconto'),
            #     movimento.get('vr_total_item')), 6))
            ##################################
            movimento['vr_unitario'] = round(self.estmvi_get_valor_unitario(
                movimento.get('quantidade'), movimento.get('vr_desconto'),
                movimento.get('vr_total_item')), 6)

            movimento['vr_total_item'] = round((movimento.get('vr_unitario') *
                                                (movimento.get('quantidade') or 1)) -
                                               (movimento.get('vr_desconto') or 0), 2)

        if fields and 'estmvi_permissao_altera_preco' in fields:
            nota_entrada_devolucao = context.get('fisnfc_nf_devolucao') and \
                                     context.get('tipo_nota') == 'E' and \
                                     context.get('proprio')
            permite_alterar_preco = self.estmvi_get_permite_alterar_preco(cr, uid, context)
            if nota_entrada_devolucao:
                permite_alterar_preco = True

            for x in res if isinstance(res, list) else [res]:
                x['estmvi_permissao_altera_preco'] = permite_alterar_preco

        return res

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        parametros = ['fispar_sped_fiscal_e200_manual']
        fiscal_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', parametros, context=context)
        res = super(estoque_movimento_itens, self).default_get(cr, uid, fields_list, context)
        nota_entrada_devolucao = context.get('fisnfc_nf_devolucao') and \
            context.get('tipo_nota') == 'E' and context.get('proprio')
        permite_alterar_preco = self.estmvi_get_permite_alterar_preco(cr, uid, context)
        if nota_entrada_devolucao:
            permite_alterar_preco = True
        sequencia = len(self.search(cr, uid, [('nf_id', '=', context.get('nf_id') or 0)])) + 1
        res.update({'estmvi_permissao_altera_preco': permite_alterar_preco,
                    'estmvi_parametro_e200': fiscal_parametro_dados.get(
                    'fispar_sped_fiscal_e200_manual'), 'sequencia': sequencia})

        return res

    def estmvi_get_preco_unitario(self, cr, uid, ids, item_id, cfop_id, destinatario_id,
                                  company_id, grade_id=False, context=None):
        """
        Parametros: item_id - e o id do item que foi inserido na nota
                    cfop_id - eh verificado de acordo com a natureza da nota
                    destinatario_id - eh o destinatario da nota
                    company_id - empresa
        Funcao: verifica de acordo com o CFOP qual preco unitario deve-se
                retornar (Preco/Venda, Custo medio, custo reposicao, custo corrigido)
        Retorno: Retorna o preco unitario381

        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        obj_fiscal_cfop = self.pool.get('fiscal.cfop')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        fields_cfop_to_read = ['codigo', 'movimento_tipo']
        dados_cfop = obj_fiscal_cfop.read(cr, uid, cfop_id, fields_cfop_to_read, context=context)
        preco_unitario = 0.00
        cfop_codigo = dados_cfop.get('codigo', False)
        cfop_tipo_mov = dados_cfop.get('movimento_tipo', False)
        if all([cfop_codigo, cfop_tipo_mov]):
            if cfop_tipo_mov == 'C':
                preco = self.get_item_preco_venda(cr, uid, item_id, company_id, destinatario_id,
                                                  grade_id=grade_id, context=context).get(item_id)
                preco_unitario = preco.get(grade_id) or preco.get('preco_venda')
            else:
                dados_custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, item_id, company_id,
                                                                        epc_data=None, context=context)
                if dados_custo and dados_custo.get(item_id):
                    if cfop_tipo_mov == 'D':
                        #Caso a nota fiscal de entrada e importada este custo
                        #So sera calculado novamente caso a embalagem do item seja alterada
                        preco_unitario = dados_custo[item_id]['estprc_ultimo_custo']
                    elif cfop_tipo_mov == 'T':
                        preco_unitario = self.estmvi_get_preco_nf_transferencia(cr, uid, item_id,
                                                                                company_id, context)
                    else:
                        # 'dict_cfops' continuou para saber eque existe este tratamento, nao retirar
                        # esta redundancia(Felipe)
                        dict_cfops = {'5927': 'estprc_custo_medio_contabil'}
                        preco_unitario = dados_custo[item_id][dict_cfops.get(cfop_codigo,
                                                                     'estprc_custo_medio_contabil')]
        return preco_unitario

    # noinspection PyProtectedMember
    def estmvi_get_cfop_id(self, cr, uid, produto_id, produto_tipo, tipo_nota, dados_cfop_nat,
                           dados_nat_encargo, dict_encargos_nat, encargo_ids, dados_pessoa,
                           tipo_atividade, context=None):
        """
        Verifica se o movimento_tipo do cfop(fiscal.cfop) da natureza(fiscal.cfop.natureza)
        eh de compra e venda, se for, chama o metodo para verificar o cfop de substituicao para
        itens ST, se nao retorna o cfop cadastrado na natureza. (Implementacao via chamado 78823)

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param produto_id: ID do produto a verificar o cfop
        :param produto_tipo: Tipo do produto - Produto/Servico
        :param tipo_nota: Tipo da nota fiscal - Entrada/Saida
        :param natureza_id: ID da natureza informado na capa da nota
        :param dados_pessoa: dados do destinatario da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type produto_id: long
        :type produto_tipo: char
        :type tipo_nota: char
        :type natureza_id: long
        :type dados_pessoa: dict
        :type context: dict
        :rtype: long

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fistribenc = self.pool.get('fiscal.tributacao.encargos')

        result = dados_cfop_nat.get('id')
        cfop_tip_mov = dados_cfop_nat.get('movimento_tipo')
        cfop_codigo = dados_cfop_nat.get('codigo')
        natureza_encargos = filter(
            lambda i: dict_encargos_nat.get(i.get('encargo_id')).get('sigla') == 'ICMS' and
                      i.get('sobrepor_tributacao_id'), dados_nat_encargo)

        # Se natureza possui sobreposicao de tributos, prevalece o CFOP da natureza
        if natureza_encargos:
            if filter(lambda i: i.get('fiscne_sobrepoe_so_tributados') is True, natureza_encargos):
                entrada = tipo_nota == 'E'
                tributos = obj_fistribenc.fisten_get_tributos(
                    cr, uid, produto_id, dados_pessoa.get('id'), 'ICMS', entrada=entrada,
                    context=context)
                for tributo in obj_fistribenc.browse(cr, uid, tributos, context=context):
                    if tributo.cst_id.tipo == 'T':
                        return result
            else:
                return result

        # Os tipos de movimento são Venda/Cancelamento, Devolucao e Tranferencia
        if cfop_tip_mov in ('C', 'D', 'T') and cfop_codigo not in ['5405', '6405', '5403', '5910',
                                                                   '6403']:
            result = self.calcula_cfop_substituicao(cr, uid, produto_id, produto_tipo, tipo_nota,
                         dados_cfop_nat, encargo_ids, dados_pessoa, tipo_atividade, context=context)

        if not self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'estoque.parametro', ['estpar_comercializa_combustivel_lubrificante'],
                context=context).get('estpar_comercializa_combustivel_lubrificante'):
            return result
        codigo_anp_id = self.pool.get('estoque.produto').read(
            cr, uid, produto_id, ['estpro_codigo_anp_id'], context, '_classic_write').get(
            'estpro_codigo_anp_id')
        dados_anp = codigo_anp_id and self.pool.get('fiscal.cadastro.anp').read(
            cr, uid, codigo_anp_id, ['fisanp_codigo'], context) or {}
        if dados_anp and int(dados_anp.get('fisanp_codigo') or ''):
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
                cr, uid, self._name, context=context)
            dados_empresa = company_id and self.pool.get('res.company').read(
                cr, uid, company_id, ['rescon_pessoa_id'], context, '_classic_write') or {}
            company_id = dados_empresa.get('rescon_pessoa_id') or 0
            dados_uf = self.pool.get('adm.pessoa').read(
                cr, uid, [(dados_pessoa or {}).get('id'), company_id], ['uf'], context)

            # CFOP para oleos
            codigo_cfop = '5656'  # Ou 6656 para outros estados

            # Verifica se o Estado do destinatario eh diferente do emitente
            if len(dados_uf) > 1:
                if dados_uf[0].get('uf') != dados_uf[1].get('uf'):
                    codigo_cfop = '6656'  # CFOP para oleos (outros estados)
            cfop_ids = self.pool.get('fiscal.cfop').search(
                cr, uid, [('codigo', '=', codigo_cfop)], context=context)
            if cfop_ids:
                result = cfop_ids[0]

        return result

    def _estmvi_producao_propria_como_revenda(self, cr, user, cfop, context=None):
        """
         Na emissao de notas fiscais de saida, se o produto estiver marcado como "produção própria"
         ao definir o CFOP para o item, verifica se o parametro do Fiscal esta marcado como
         "Producao propria como revenda" para utilizar os CFOP definido, caso contrario, o CFOP
          sera alterado para 5101/6101 ou 5401/6401 (para item com substituicao tributaria)
        """
        context = context and context.copy() or {}
        cfop = cfop and cfop.__str__() or ''
        if cfop and not self._parametros(cr, user).fiscal.fispar_producao_propria_como_revenda:
            cfop = cfop[:2] + '01'

        return cfop

    def estmvi_onchange_quantidade_propria(self, cr, user, ids, quantidade, vr_unitario,
                                           vr_desconto, embalagem_id, context=None):
        """
        Calcula o valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param vr_unitario: o valor unitario do item
        :param vr_desconto: o valor do desconto do item
        :param embalagem_id: o ID da embalagem
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type vr_unitario: float
        :type vr_desconto: float
        :type embalagem_id: long
        :type context: dict
        :return: o valor total do item.
        :rtype: float
        """
        context = context and context.copy() or {}
        aviso = {}
        if quantidade <= 0:
            quantidade = 1

        # Verifica se a embalagem permite quantidade fracionada
        embalagem_id = embalagem_id and self.pool.get('estoque.produto.embalagem').read(
            cr, user, embalagem_id, ['embalagem_id'], context, '_classic_write').get(
            'embalagem_id') or False
        unidade_id = embalagem_id and self.pool.get('estoque.embalagem').read(
            cr, user, embalagem_id, ['unidade_id'], context, '_classic_write').get(
            'unidade_id') or False
        fracao = unidade_id and self.pool.get('estoque.embalagem.unidade').read(
            cr, user, unidade_id, ['fracao'], context).get('fracao') or False
        if not fracao and int(quantidade) != quantidade:
            quantidade = 1
            aviso = {'title': _('Aviso.'),
                     'message': _('Embalagem nao permite quantidade fracionada.')}
        quantidade = round(quantidade, 3)
        vr_unitario = round(vr_unitario, 6)
        vr_desconto = round(vr_desconto, 2)
        vr_total_item = round(quantidade * vr_unitario - vr_desconto, 2)
        return {'value': {'quantidade': quantidade, 'vr_unitario': vr_unitario,
                          'vr_desconto': vr_desconto, 'vr_total_item': vr_total_item},
                'warning': aviso}

    def estmvi_onchange_valor_unitario_propria(self, cr, user, ids, quantidade, vr_unitario,
                                               vr_desconto, context=None):
        """
        Calcula o valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param vr_unitario: o valor unitario do item
        :param vr_desconto: o valor do desconto rateado (proporcional ao desconto na capa)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type vr_unitario: float
        :type vr_desconto: float
        :type context: dict
        :return: o valor unitario do item.
        :rtype: float
        """
        context = context and context.copy() or {}

        return {'value': {'vr_total_item': quantidade * vr_unitario - vr_desconto}}

    def estmvi_onchange_desconto_propria(self, cr, user, ids, quantidade, vr_unitario, vr_desconto,
                                         context=None):
        """
        Calcula o valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param vr_unitario: o valor unitario do item
        :param vr_desconto: o valor do desconto rateado (proporcional ao desconto na capa)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type vr_unitario: float
        :type vr_desconto: float
        :type context: dict
        :return: o valor unitario do item.
        :rtype: float
        """
        context = context and context.copy() or {}
        aviso = {}

        # Nao permite valor de desconto negativo
        if vr_desconto < 0:
            vr_desconto = 0
            aviso = {'title': _('Aviso.'),
                     'message': _('Desconto nao permite valor negativo.')}

        quantidade = round(quantidade, 3)
        vr_unitario = round(vr_unitario, 6)
        vr_desconto = round(vr_desconto, 2)
        vr_total_item = round(quantidade * vr_unitario - vr_desconto, 2)

        return {'value': {'quantidade': quantidade, 'vr_unitario': vr_unitario,
                          'vr_desconto': vr_desconto, 'vr_total_item': vr_total_item},
                'warning': aviso}

    def estmvi_onchange_valor_total_item_propria(self, cr, user, ids, vr_item, vr_despesa,
                                                 incide_despesa, vr_frete, vr_seguro, vr_desconto,
                                                 tributacoes_o2m, context=None, natureza_id=None):
        """
        Ajusta os valores dos tributos em funcao do valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimento 
        :param vr_item: o valor total do item
        :param vr_despesa: o valor das despesas rateada do item
        :param incide_despesa: se o valor da despesa rateada incide ou nao na base de calculo.
        :param vr_frete: o valor do frete rateado do item
        :param vr_seguro: o valor do seguro rateado do item
        :param vr_desconto: o valor do desconto rateado do item
        :param tributacoes_o2m: lista com o ID das tributacoes do item (one2many)
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type vr_item: float
        :type vr_despesa: float
        :type incide_despesa: bool
        :type vr_frete: float
        :type vr_seguro: float
        :type vr_desconto: float
        :type tributacoes_o2m: list
        :type context: dict
        :type natureza_id: long
        :return: as tributacoes e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        self._estmvi_ajusta_tributacao_propria(
            cr, user, vr_item, vr_despesa, incide_despesa, vr_frete, vr_seguro, vr_desconto,
            tributacoes_o2m, context, natureza_id)
        importacao = self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, natureza_id, context).cfop_id.codigo[0] == '3'
        vr_encargo_importacao = 0
        for tributacao in self.resolve_2many_commands(
                cr, user, 'tributacoes_ids', tributacoes_o2m):
            tributacao = adjust_m2o(tributacao)
            encargo = self.pool.get('fiscal.encargo').read(
                cr, user, tributacao.get('encargo_id'), ['sigla'], context).get('sigla')
            if encargo.upper() == 'IPI'and importacao:
                vr_encargo_importacao += tributacao.get('vr_encargo')
            elif encargo.upper() == 'ICMS' and importacao:
                vr_encargo_importacao += tributacao.get('vr_encargo')
            elif encargo.upper() == 'II' and importacao:
                vr_encargo_importacao += tributacao.get('vr_encargo')

        return {
            'value': {
                'tributacoes_ids': tributacoes_o2m,
                'vr_total_item_rateado': vr_item + vr_despesa + vr_seguro + vr_frete - vr_desconto
                                         + vr_encargo_importacao
            }
        }

    def _estmvi_ajusta_tributacao_propria(self, cr, user, vr_item, vr_despesa, incide_despesa,
                                          vr_frete, vr_seguro, vr_desconto, tributacoes_o2m,
                                          context=None, natureza_id=None):
        """
        Ajusta os valores dos tributos em funcao do valor da despesa (rateada) do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimento 
        :param vr_item: o valor total do item
        :param vr_despesa: o valor das despesas rateada do item
        :param incide_despesa: se o valor da despesa rateada incide ou nao na base de calculo.
        :param vr_frete: o valor do frete rateado do item
        :param vr_seguro: o valor do seguro rateado do item
        :param vr_desconto: o valor do desconto rateado do item
        :param tributacoes_o2m: lista com o ID das tributacoes do item (one2many)
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type vr_item: float
        :type vr_despesa: float
        :type incide_despesa: bool
        :type vr_frete: float
        :type vr_seguro: float
        :type vr_desconto: float
        :type tributacoes_o2m: list
        :type context: dict
        :type natureza_id: long
        :return: as tributacoes e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        vr_base_calculo = vr_item + vr_despesa + vr_frete + vr_seguro - vr_desconto
        for tributacao in filter(lambda f: f[0] in (0, 1, 4), tributacoes_o2m):
            tributacao[2] = adjust_m2o(self.resolve_2many_commands(
                cr, user, 'tributacoes_ids', [tributacao])[0])
            tributacao_id = tributacao[2].get('tributacao_id')
            tributacao_id = \
                isinstance(tributacao_id, (list, tuple)) and tributacao_id[0] or tributacao_id
            tributacao[2].update(
                self.pool.get('fiscal.nf.item.tributacao').fisnit_get_tributacao_valor(
                    cr, user, tributacao_id, vr_base_calculo, context, vr_despesa, incide_despesa,
                    natureza_id))

            # Valores de ICMS/ST
            encargo_id = tributacao[2].setdefault('encargo_id', 0)
            icms_ids = encargo_id and self.pool.get('fiscal.encargo').search(
                cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
            icms_id = icms_ids and icms_ids[0]
            tributacao_dados = tributacao_id and self.pool.get('fiscal.tributacao').read(
                cr, user, tributacao_id, ['cst_id'], context, '_classic_write')
            cst_id = tributacao_dados and tributacao_dados.get('cst_id')
            cst_dados = cst_id and self.pool.get('fiscal.cst').read(
                cr, user, cst_id, ['tipo'], context) or {}
            tipo = cst_dados.get('tipo')
            if encargo_id == icms_id:
                if tipo in ('U', 'TU') and tributacao[2].get('aliquota'):
                    tributacao[2].update(
                        self.pool.get('fiscal.nf.item.tributacao').fisnit_calcular_vr_icms_st_item(
                            tributacao[2].setdefault('vr_base_calculo'),
                            tributacao[2].setdefault('aliquota', 0),
                            tributacao[2].setdefault('aliquota_venda', 0),
                            tributacao[2].setdefault('percentual_mva', 0)))
                else:
                    tributacao[2].update({'vr_icms_substituicao': 0, 'vr_base_icms_substituicao': 0,
                                          'aliquota_venda': 0})
            else:  # Encargo nao eh ICMS
                tributacao[2].update({'vr_icms_substituicao': 0, 'vr_base_icms_substituicao': 0,
                                      'aliquota_venda': 0})

            tributacao[0] = tributacao[0] and 1 or 0

        return tributacoes_o2m

    def _estmvi_ajusta_tributacao(self, cr, user, ids, tributacoes, despesa, desconto, seguro,
                                  frete, tipo, total, incide_despesa, context=None):
        """
        Ajusta os valores dos tributos em funcao do valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento (produto)
        :param tributacoes: lista com os dados das tributacoes (one2many)
        :param despesa: o valor das despesas acessorias rateada (proporcional as despesas na capa)
        :param desconto: desconto rateado
        :param seguro: o valor do seguro rateado (proporcional ao seguro na capa)
        :param frete: o valor do frete rateado (proporcional oa frete na capa)
        :param tipo: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param total: o valor total do item
        :param incide_despesa: se incidira ou nao o valor da despesa na base de calculo
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tributacoes: list
        :type seguro: float
        :type frete: float
        :type tipo: str
        :type total: float
        :type context: dict
        :return: o valor unitario do item e os valores da base de calculo e o valor do encargo em
         funcao do valor total do item.
        :rtype: dict
        """
        context = context and context.copy() or dict()
        frete = 0 if tipo == 'F' else frete  # Envolve o valor do frete somente se "por dentro"
        valor_rateado = total + despesa + seguro + frete - desconto
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', '=', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]

        # Ajusta o valor da base de calculo e do encargo das tributacoes
        for tributacao in filter(lambda f: f[0] in [0, 1, 4], tributacoes):
            tributacao[2] = adjust_m2o(self.resolve_o2m_commands_to_record_dicts(
                cr, user, 'tributacoes_ids', [tributacao], [], context)[0])
            tributacao[0] = tributacao[0] and 1 or 0
            tributacao_id = tributacao[2].get('tributacao_id')
            tributacao_id = \
                tributacao_id[0] if isinstance(tributacao_id, (tuple, list)) else tributacao_id
            dados_tributacao = self.pool.get('fiscal.tributacao').read(
                cr, user, tributacao_id, ['base', 'aliquota'], context, '_classic_write')
            valor_base = valor_rateado * dados_tributacao.get('base') / 100.
            vr_reducao_base = valor_rateado - valor_base

            # Remove o valor despesa da base de calculo e acrescenta na reducao se o valor da
            # despesa nao deve incidir na base
            vr_reducao_base += despesa if not incide_despesa else 0

            # Nao ha reducao se o encargo nao for ICMS
            if tributacao[2].get('encargo_id') != icms_id:
                vr_reducao_base = 0

            vr_encargo = valor_base * dados_tributacao.get('aliquota') / 100.
            tributacao[2] = dict(
                tributacao[2], vr_base_calculo=valor_rateado - vr_reducao_base,
                aliquota=dados_tributacao.get('aliquota'), vr_encargo=vr_encargo,
                vr_reducao_base=vr_reducao_base)
            if tributacao[2].get('modbcst') == '4':
                tributacao[2].update(
                    self.pool.get('fiscal.nf.item.tributacao').fisnit_calcular_vr_icms_st_item(
                        valor_rateado, tributacao[2].setdefault('aliquota', 0),
                        tributacao[2].setdefault('aliquota_venda', 0),
                        tributacao[2].setdefault('percentual_mva', 0)))

        return dict(value=dict(tributacoes_ids=tributacoes), context=context)

    def estmvi_onchange_desconto_terceiro(self, cr, user, ids, quantidade, desconto, total,
                                          context=None):
        """
        Calcula o valor unitario do item em funcao da quantidade, desconto e total do item (produto/
        servico).

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento (produto)
        :param quantidade: a quantidade de itens
        :param desconto: o valor do desconto rateado (proporcional ao desconto na capa)
        :param total: o valor total do item
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type desconto: float
        :type total: float
        :type context: dict
        :return: o valor unitario do item.
        :rtype: float
        """
        context = context and context.copy() or {}

        # Nao permite valor de desconto negativo
        if desconto < 0:
            desconto = 0
            aviso = {'title': _('Aviso.'), 'message': _('* Desconto nao permite valor negativo.')}
        else:
            aviso = {}

        vr_unitario = self.estmvi_get_valor_unitario(quantidade, desconto, total)

        return {'value': {'vr_unitario': vr_unitario, 'vr_desconto': desconto}, 'warning': aviso}

    def estmvi_onchange_quantidade_terceiro(self, cr, user, ids, quantidade, desconto, total,
                                            embalagem_id, context=None):
        """
        Calcula o valor unitario do item em funcao da quantidade, desconto e total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param desconto: o valor do desconto rateado (proporcional ao desconto na capa)
        :param total: o valor total do item
        :param embalagem_id: o ID da embalagem do item (estoque.produto.embalagem)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type desconto: float
        :type total: float
        :type embalagem_id: long
        :type context: dict
        :return: o valor unitario do item.
        :rtype: float
        """
        context = context and context.copy() or {}

        # Verifica se a embalagem permite quantidade fracionada
        brw_produto_embalagem = embalagem_id and self.pool.get('estoque.produto.embalagem').browse(
            cr, user, embalagem_id, context=context)
        if embalagem_id and not brw_produto_embalagem.embalagem_id.unidade_id.fracao:
            if int(quantidade) != quantidade:
                return {'value': {'quantidade': 1},
                        'warning': {'title': _('Aviso.'),
                                    'message': _('Unidade da embalagem nao permite quantidade '
                                                 'fracionada.')}}

        return {'value': {'vr_unitario': self.estmvi_get_valor_unitario(
            quantidade, desconto, total)}}

    def _get_values(self, cr, user, item_id, tributacoes_o2m, vr_total_item, vr_desconto,
                    vr_despesa, vr_seguro,  vr_frete, tipo_frete, context=None, natureza_id=None):
        """
        Retorna os valores da base de calculo e do encargo das tributacoes.
        
        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item_id: o ID do item (produto/servico)
        :param tributacoes_o2m: lista das tributacoes (formato one2many)
        :param vr_total_item: valor total do item
        :param vr_desconto: valor do desconto rateado (capa da nota fiscal)
        :param vr_despesa: valor total das despesas acessorias (capa da nota fiscal)
        :param vr_seguro: valor total do seguro
        :param vr_frete: valor do frete
        :param tipo_frete: tipo de frete: "D"= por dentro, "F"= por fora
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type item_id: long
        :type tributacoes_o2m: list
        :type vr_total_item: float
        :type vr_desconto: float
        :type vr_despesa: float
        :type vr_seguro: float
        :type vr_frete: float
        :type tipo_frete: str
        :type context: dict
        :type natureza_id: long
        :return: dicionario com os valores ajustados das tributacoes
        :rtype: dict
        """
        context = context and context.copy() or {}
        if tipo_frete == 'F':  # Envolve o valor do frete somente se "por dentro"
            vr_frete = 0.
        total = vr_total_item + vr_despesa + vr_seguro + vr_frete - vr_desconto

        # Ajusta o valor da base de calculo e do encargo das tributacoes
        for tributacao in filter(lambda f: f[0] in (0, 1, 4), tributacoes_o2m):
            tributacao[2] = adjust_m2o(self.resolve_2many_commands(
                cr, user, 'tributacoes_ids', [tributacao], [], context)[0])
            tributacao[0] = tributacao[0] and 1 or 0
            tributacao[2].update(
                self.pool.get('fiscal.nf.item.tributacao').calcula_tributacao_item(
                    cr, user, item_id, tributacao[2]['tributacao_id'], total, context, natureza_id))

        return {'value': {'tributacoes_ids': tributacoes_o2m}}

    def estmvi_onchange_total_item_terceiro(self, cr, user, ids, item_id, tributacoes_o2m,
                                            quantidade, vr_desconto, vr_total_item, vr_despesa,
                                            context=None):
        """
        Calcula o valor unitario do item em funcao da quantidade, desconto e total do item (produto/
        servico). Ajusta os valores dos tributos em funcao do valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimento
        :param item_id: o ID do item (produto/servico)
        :param tributacoes_o2m: lista com os dados das tributacoes (one2many)
        :param quantidade: a quantidade de itens
        :param vr_desconto: o desconto aplicado no item
        :param vr_total_item: o valor total do item
        :param vr_despesa: o valor da despesa rateada (capa da nota fiscal)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacoes_o2m: list
        :type quantidade: float
        :type vr_desconto: float
        :type vr_total_item: float
        :type vr_despesa: float
        :type context: dict
        :return: o valor unitario do item e os valores da base de calculo e o valor do encargo em
        funcao do valor total do item.
        :rtype: float
        """
        context = context and context.copy() or {}
        vr_total = vr_total_item + vr_despesa
        values = {'vr_unitario': self.estmvi_get_valor_unitario(quantidade, vr_desconto, vr_total),
                  'vr_total_item_rateado': vr_total}

        # Ajusta o valor das tributacoes
        for tributacao in filter(lambda f: f[0] in (0, 1, 4), tributacoes_o2m):
            tributacao[2] = adjust_m2o(self.resolve_2many_commands(
                cr, user, 'tributacoes_ids', [tributacao], [], context)[0])
            tributacao[0] = tributacao[0] and 1 or 0
            tributacao[2].update(
                self.pool.get('fiscal.nf.item.tributacao').calcula_tributacao_item(
                    cr, user, item_id, tributacao[2].get('tributacao_id'), vr_total, context,
                    movimento_id=ids and ids[0]))

        values.update(tributacoes_ids=tributacoes_o2m)

        return {'value': values}

    def estmvi_onchange_despesas_rateado_terceiro(self, cr, user, ids, item_id, tributacoes_o2m,
                                                  vr_total_item, vr_desconto, vr_despesa, vr_seguro,
                                                  vr_frete, tipo_frete, context=None):
        """
        Calcula o valor da base de calculo em funcao da quantidade, desconto e total do item.
        Ajusta os valores dos tributos em funcao do valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param item_id: o ID do item (produto/servico)
        :param tributacoes_o2m: lista com os dados das tributacoes (one2many)
        :param vr_total_item: o valor total do item
        :param vr_desconto: o valor do desconto rateado (capa da nota fiscal)
        :param vr_despesa: o valor das despesas acessorias rateada (capa da nota fiscal)
        :param vr_seguro: o valor do seguro rateado (capa da nota fiscal)
        :param vr_frete: o valor do frete rateado (capa da nota fiscal)
        :param tipo_frete: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacoes_o2m: list
        :type vr_desconto: float
        :type vr_despesa: float
        :type vr_seguro: float
        :type vr_frete: float
        :type tipo_frete: str
        :type context: dict
        :return: o valor unitario do item e os valores da base de calculo e o valor do encargo em
         funcao do valor total do item.
        :rtype: float
        """
        context = context and context.copy() or {}
        result = self._get_values(cr, user, item_id, tributacoes_o2m, vr_total_item, vr_desconto,
                                  vr_despesa, vr_seguro, vr_frete, tipo_frete, context)

        return result

    @staticmethod
    def estmvi_get_valor_unitario(quantidade, desconto, total):
        """
        Calcula o valor unitario do item em funcao da quantidade, desconte e valor total.

        :param quantidade: a quantidade de itens
        :param desconto: o valor do desconto
        :param total: o valor total do item
        :type quantidade: float
        :type desconto: float
        :type total: float
        :return: o valor unitario do item
        :rtype: float
        """
        if not quantidade:
            return 0

        return (total + desconto) / (quantidade * 1.)  # Sempre ´float´

    def estmvi_verifica_tributacao_interestadual(self, cr, user, tributacao, destinatario, natureza,
                                                 context=None):
        """
        Verifica se as tributacoes devem ser calculadas a partir de tributos interstaduais.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param tributacao: o ID da tributacao
        :param destinatario: ID do destinatario: fornecedor se entrada ou cliente se saida
        :param natureza: ID da natureza da operacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type tributacao: long
        :type destinatario: long
        :type natureza: long
        :return: verdadeiro se houver necessidade de utilizar tributos interestaduais.
        :rtype: bool
        """

        context = context and context.copy() or {}
        destinatario_endereco = destinatario and self.pool.get('adm.pessoa').admpes_endereco_fiscal(
            cr, user, destinatario, context)
        natureza_tipo = natureza and self.pool.get('fiscal.cfop.natureza').read(
            cr, user, natureza, ['tipo'], context).get('tipo')
        cst_tipo = self.pool.get('fiscal.tributacao').browse(
            cr, user, tributacao, context).cst_id.tipo

        return all([destinatario_endereco, cst_tipo == 'T', natureza_tipo == 'S'])

    def estmvi_calcula_vr_unitario_movimento(self, cr, uid, vr_unitario, produto_embalagem_id,
                                             context=None):
        """
        O metodo trata o valor unitario do item (vr_unitario) nas situacoes em que eh salvo no
        movimento (estoque_movimento_itens) e exibido em tela. No movimento, deve ser salvo o valor
        na unidade e em tela deve ser exibido o valor na embalagem.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param vr_unitario: valor da embalagem do item, como mostrado na tela.
        :type vr_unitario: float
        :param produto_embalagem_id: id da tabela estoque_produto_embalagem, a embalagem do produto.
        :type produto_embalagem_id: int
        :param context: dados de valores contextuais.
        :type context: dict
        :return: valor do item calculado.
        :rtype: float
        """
        context = context and context.copy() or {}
        quantidade_embalagem = self.pool.get('estoque.produto.embalagem').browse(
            cr, uid, produto_embalagem_id, context).embalagem_id.quantidade_na_embalagem or 1.0
        vr_unitario = float(float_round(vr_unitario / quantidade_embalagem, 6))
        return vr_unitario


    def _estmvi_create_fiscal(self, cr, uid, vals_itens, dados_nf, context):
        context = context and context.copy() or {}
        if not context.get('fiscal') and \
           not context.get('nota_fiscal_entrada') and \
           not context.get('nota_fiscal_saida') and \
           not context.get('terceiro') and \
           not context.get('importacao_nfe'):
            return True

        load = '_classic_write'
        obj_produto = self.pool.get('estoque.produto')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        pessoa_id = self.pool.get('res.company').read(cr, uid, company_id, ['rescon_pessoa_id'],
                                                      context, load).get('rescon_pessoa_id')
        context['super_read_adm_pessoa'] = True
        dados_pessoa = self.pool.get('adm.pessoa').read(cr, uid, pessoa_id,
                                                        ['admpes_jur_tipo_atividade'], context)
        tipo_atividade = dados_pessoa.get('admpes_jur_tipo_atividade')
        obj_est_parametro = self.pool.get('estoque.parametro')
        estoque_parametro_ids = obj_est_parametro.search(cr, uid, [('company_id', '=', company_id)],
                                                         context=context)
        if not estoque_parametro_ids:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao existe parametro do Estoque cadastrado para a'
                                   ' empresa corrente.'))

        embalagem_servico_id = obj_est_parametro.read(cr, uid, estoque_parametro_ids[0],
                                                     ['estpar_embalagem_servico_id'], context, load
                                                               ).get('estpar_embalagem_servico_id')
        produto_ids = []
        vr_produto_ids = []
        grade_ids = []
        for i in vals_itens:
            produto_ids.append(i[2].get('produto_id'))
            if not i[2].get('vr_unitario'): # vr_preco_venda
                vr_produto_ids.append(i[2].get('produto_id'))
                grade_ids.append(i[2].get('grade_id'))

        dict_produtos = {}
        campos = ['tipo', 'tributacao_ids', 'name', 'estpro_percentual_comissao', 'categoria_id']
        map(lambda x: dict_produtos.update({x.get('id'): x}),
            obj_produto.read(cr, uid, produto_ids, campos, context, load))
        precos = {}
        if vr_produto_ids:
            precos = self.get_item_preco_venda(cr, uid, vr_produto_ids, grade_id=grade_ids,
                                               company_id=dados_nf.get('company_id'),
                                               pessoa_id=dados_nf.get('destinatario_id'),
                                               context=context)
        devolucao = False
        if context.get('active_model') == 'fiscal.nf.capa' and context.get('active_id'):
            dev_natureza = self.pool.get('fiscal.nf.capa').read(
                cr, uid, context.get('active_id'), ['natureza_id'], context, load)
            dev_natureza_id = dev_natureza.get('natureza_id')
            dev_cfop_id = self.pool.get('fiscal.cfop.natureza').read(
                cr, uid, dev_natureza_id, ['cfop_id'], context, load).get('cfop_id')
            dev_cfop = self.pool.get('fiscal.cfop').read(
                cr, uid, dev_cfop_id, ['movimento_tipo'], context)

            # verifica se o item é de nota e se a nota eh de devolucao
            if dev_cfop and dev_cfop.get('movimento_tipo') == 'D':
                devolucao = True

        destinatario_endereco_id = dados_nf.get('destinatario_endereco_id', False)
        destinatario_id = dados_nf.get('destinatario_id')
        if destinatario_id and not destinatario_endereco_id:
            destinatario_endereco_ids = self.pool.get('adm.pessoa.endereco').search(cr, uid,
                                            [('pessoa_id', '=', destinatario_id)], context=context)
            destinatario_endereco_id = destinatario_endereco_ids and destinatario_endereco_ids[0]

        campos = ['pessoa_tipo', 'admpes_fis_produtor_rural', 'admpes_jur_ie', 'cpf_cnpj',
                  'admpes_jur_regime_tributario']
        dados_destinatario = {}
        destinatario_contribuinte = False
        if destinatario_id:
            # notas de cupom pode nao ter destinatario
            dados_destinatario = self.pool.get('adm.pessoa').read(cr, uid, destinatario_id, campos,
                                                              context)
            dados_destinatario.update(
                {'destinatario_endereco_id': destinatario_endereco_id or False})
            destinatario_contribuinte = all(
                [dados_destinatario.get('pessoa_tipo') == 'J',
                 dados_destinatario.get('cpf_cnpj'),
                 dados_destinatario.get('admpes_jur_ie'),
                 dados_destinatario.get('admpes_jur_regime_tributario') == 'R'])
        natureza_id = dados_nf.get('natureza_id')
        if not natureza_id:
            raise osv.except_osv(_('Atencao'),
                                 _('Para fazer a validacao informe o natureza da operacao.'))

        campos = ['cfop_id', 'natureza_encargos_ids', 'tipo']
        dados_natureza = self.pool.get('fiscal.cfop.natureza').read(cr, uid, natureza_id, campos,
                                                                    context, load)
        campos = ['movimento_tipo', 'codigo', 'sigla']
        dados_cfop_nat = self.pool.get('fiscal.cfop').read(cr, uid, dados_natureza.get('cfop_id'),
                                                           campos, context)
        campos = ['encargo_id', 'sobrepor_tributacao_id', 'fiscne_sobrepoe_so_tributados']
        dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(cr, uid,
                                 dados_natureza.get('natureza_encargos_ids'), campos, context, load)

        encargo_ids = map(lambda x: x.get('encargo_id'), dados_nat_encargo)
        dict_encargos_nat = {}
        map(lambda x: dict_encargos_nat.update({x.get('id'): x}),
            self.pool.get('fiscal.encargo').read(cr, uid, encargo_ids, ['sigla'], context))

        uf_id = False
        if destinatario_endereco_id:
            uf_id = self.pool.get('adm.pessoa.endereco').read(cr, uid, destinatario_endereco_id,
                                                          ['uf_id'], context, load).get('uf_id')
        for item in vals_itens:
            vals = item[2]
            item_local_id = vals.get('local_id')
            produto = dict_produtos.get(vals.get('produto_id'))
            if context.get('nota_fiscal_saida'):
                dados_comissao = self.estmvi_get_comissao(cr, uid, produto.get('id'),
                                                          vals.get('vr_total_item_rateado'),
                                                          vals.get('vendedor_id'), produto,
                                                          context)
                if dados_comissao:
                    percentual_comissao = float(float_round(
                        dados_comissao.get('estmvi_percentual_comissao'), 3))
                    vr_comissao = float(float_round(dados_comissao.get('estmvi_vr_comissao'), 6))
                    vals.update({'estmvi_percentual_comissao': percentual_comissao,
                                 'estmvi_vr_comissao': vr_comissao})

            grade_id = vals.get('grade_id')
            produto_embalagem_id = vals.get('produto_embalagem_id')
            # Tratamento para NF servico
            if not produto_embalagem_id and \
                    (context.get('nf_servico') or context.get('fatura_contrato')):
                if not embalagem_servico_id:
                    raise osv.except_osv(_('Atencao'),
                                         _('Embalagem para servico nao informada no '
                                           'parametro do estoque'))

                obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
                produto_embalagem_ids = obj_estoque_produto_embalagem.search(cr, uid, [
                    ('embalagem_id', '=', embalagem_servico_id),
                    ('produto_id', '=', vals.get('produto_id'))], context=context)
                if produto_embalagem_ids:
                    vals['produto_embalagem_id'] = produto_embalagem_ids[0]

                else:
                    produto_embalagem_id = obj_estoque_produto_embalagem.create(cr, uid,
                        {'embalagem_id': embalagem_servico_id,
                         'produto_id': vals.get('produto_id')}, context)
                    vals['produto_embalagem_id'] = produto_embalagem_id

            item_quantidade = vals.get('quantidade') or 0
            item_preco_venda = vals.get('vr_preco_venda') or 0
            item_desconto = vals.get('vr_desconto') or 0
            item_vr_unitario = vals.get('vr_unitario') or 0
            if item_quantidade <= 0 and not context.get('nota_referenciada'):
                raise osv.except_osv(_('Atencao!'),
                                     _('A quantidade informada para o produto deve ser maior que '
                                       'zero.'))

            if item_vr_unitario <= 0 and \
                    'atualizacao_inventario' not in context and \
                    'entrada_producao' not in context and \
                    not context.get('nota_referenciada') and \
                    not context.get('terceiro') and \
                    not context.get('importacao_nfe'):
                raise osv.except_osv(_('Atencao'),
                                     _('O valor unitario do item deve ser maior que zero!'))

            if item_desconto < 0:
                raise osv.except_osv(_('Atencao'),
                                     _('O valor desconto nao pode ser negativo, dever ser igual ou '
                                       'maior que zero.'))


            # a partir daqui somente itens com nota
            if dados_nf:
                if isinstance(dados_nf, dict):
                    if not vals.get('vendedor_id'):
                        vendedor_id = dados_nf.get('vendedor_id')
                        if vendedor_id:
                            vals['vendedor_id'] = vendedor_id

                    if not vals.get('vendedor_externo_id'):
                        vendedor_externo_id = dados_nf.get('vendedor_externo_id')
                        if vendedor_externo_id:
                            vals['vendedor_externo_id'] = vendedor_externo_id

                    nf_company_id = dados_nf.get('company_id') or 0
                    produto_tipo = produto.get('tipo')
                    if dados_nf.get('tipo') == 'S':
                        if not context.get('gerar_nf_cupom') and \
                                not context.get('fisnfc_nfe_de_nfce') and \
                                not context.get('fatura_pedido'):
                            if not dados_nf.get('fisnfc_nf_devolucao') and \
                                    not vals.get('estmvi_permissao_altera_preco'):
                                if item_preco_venda:
                                    # Foi retirada a multiplicacao pela qtd da embalagem neste
                                    # ponto, pois a quantidade do item ja esta multiplicada pela
                                    # quantidade na embalagem. Se em algum ponto ficar o valor
                                    # unitario errado, merece uma melhor analize para nao
                                    # ocorrer erro em outro ponto.
                                    produto_valor_unitario = item_preco_venda
                                else:
                                    preco = precos.get(produto.get('id')) or {}
                                    produto_valor_unitario = preco.get(grade_id) \
                                        or preco.get('preco_venda') \
                                        or item_vr_unitario or 0
                            else:
                                produto_valor_unitario = vals.get('vr_unitario') or 0
                        else:
                            produto_valor_unitario = vals.get('vr_unitario') or 0

                    else:
                        produto_valor_unitario = vals.get('vr_unitario') or 0
                        if not produto_valor_unitario and not context.get('nota_referenciada') \
                                and not context.get('terceiro') and not \
                                context.get('importacao_nfe'):
                            raise osv.except_osv(_('Atencao!'),
                                                 _('E necessario informar o valor unitario para o '
                                                   'item inserido.'))
                        if produto_tipo == 'P':
                            nf_capa_fornecedor_id = destinatario_id
                            # verifica se o produto possui o local informado, se nao tiver, adiciona o local para o produto
                            self.check_nf_item_local_produto(cr, uid, company_id=nf_company_id,
                                                             local_id=item_local_id,
                                                             produto_id=produto.get('id'),
                                                             grade_id=grade_id,
                                                             embalagem_id=produto_embalagem_id,
                                                             context=context)

                            if not devolucao:
                                # verifica se o fornecedor esta cadastrado para o produto, se nao estiver, realiza o cadastro
                                obj_produto.estoque_produto_in_fornecedor(cr, uid, company_id=nf_company_id,
                                                                              fornecedor_id=nf_capa_fornecedor_id,
                                                                              produto_id=produto.get('id'),
                                                                              produto_embalagem_id=produto_embalagem_id,
                                                                              context=context)

                    if not context.get('nota_referenciada') and not context.get('calculo_com_volume') \
                            and not context.get('pcp_ordem_producao') and 'fatura_pedido' not in context \
                            and not context.get('importacao_nfe'):
                        if vals.get('vr_total_item') and (context.get('gerar_nf_cupom') or
                                                          context.get('fisnfc_nfe_de_nfce')) \
                                or context.get('estmvi_nao_calcula_vr_total'):
                            valor_total_item = vals.get('vr_total_item')
                        else:
                            valor_total_item = (produto_valor_unitario * item_quantidade) - \
                                item_desconto
                        item_preco_venda = (valor_total_item + item_desconto) / item_quantidade
                        vals['vr_unitario'] = (vals.get('vr_unitario') or 0.0) or item_preco_venda
                        if not vals['vr_unitario'] > 0 and not context.get('terceiro'):
                            raise osv.except_osv(_('Atencao!'),
                                                 _('O valor unitario do item %s deve ser maior que '
                                                   'zero.' %
                                produto.get('name')))
                        if dados_nf.get('tipo') == 'S':
                            vals['vr_preco_venda'] = item_preco_venda
                            vals['vr_total_item'] = valor_total_item
                        else:
                            vals['vr_preco_venda'] = 0

                    # Calculo de tributacao do item
                    cfop_id = False
                    if not context.get('importacao_nfe') and \
                            not context.get('nota_referenciada') and \
                            not context.get('pcp_ordem_producao') and \
                            not vals.get('cfop_id'):
                        cfop_id = self.estmvi_get_cfop_id(
                            cr, uid, vals.get('produto_id'), produto_tipo, dados_nf.get('tipo'),
                            dados_cfop_nat, dados_nat_encargo, dict_encargos_nat, encargo_ids,
                            dados_destinatario, tipo_atividade, context=context)
                        vals['cfop_id'] = cfop_id
                    if context.get('fatura_pedido') or \
                            context.get('fatura_contrato') or context.get('pcp_ordem_producao') or \
                            context.get('kit_itens') or context.get('wizfni_nota_por_local') or \
                            context.get('nota_fiscal_consumidor'):
                        vr_rateado = vals.get('vr_total_item_rateado') or \
                                     vals.get('vr_total_item') or 0
                        if dados_nf.get('tipo') == 'S' and destinatario_contribuinte:
                            tipo_tributacao = 'C'
                        elif dados_nf.get('tipo') == 'S':
                            tipo_tributacao = 'T'
                        else:
                            tipo_tributacao = 'E'
                        if not context.get('gerar_nf_cupom') and not context.get('fisnfc_nfe_de_nfce') and not cfop_id:
                            cfop_id = self.estmvi_get_cfop_id(
                                cr, uid, vals.get('produto_id'), produto_tipo, dados_nf.get('tipo'),
                                dados_cfop_nat, dados_nat_encargo, dict_encargos_nat, encargo_ids,
                                dados_destinatario, tipo_atividade, context=context)
                            vals['cfop_id'] = cfop_id
                        if context.get('adicionar_pedido_compras'):
                            tipo_tributacao = 'E'
                        tributos = self.calcular_tributo_itens(
                            cr, uid, produto, dados_natureza, dados_nat_encargo, dados_cfop_nat,
                            encargo_ids, dados_destinatario, tipo_tributacao, vr_rateado, [], uf_id,
                            context)
                        if tributos:
                            vals['tributacoes_ids'] = tributos

            if context.get('calculo_com_volume') and not context.get('importacao_nfe'):
                if 'estmvi_volume' in vals:
                    volume = vals.get('estmvi_volume')
                    if not volume:
                        volume = \
                            self.pool.get('madeireira.utils').madutl_calcula_volume(
                                item_quantidade, vals.get('estmvi_comprimento', 0),
                                vals.get('estmvi_largura', 0), vals.get('estmvi_espessura', 0))
                    if volume:
                        vr_total_item = (item_vr_unitario * volume) - item_desconto
                        vals['vr_total_item'] = vr_total_item

        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if context.get('atualizacao_inventario'):
            return super(estoque_movimento_itens, self).create(cr, uid, vals, context)
        if vals.get('vr_unitario') and vals.get('produto_embalagem_id'):
            vals['vr_unitario'] = self.estmvi_calcula_vr_unitario_movimento(
                cr, uid, vals.get('vr_unitario'), vals.get('produto_embalagem_id'), context)
        nf_id = vals.get('nf_id')
        if vals.get('tributacoes_ids') and nf_id:
            for i in vals.get('tributacoes_ids'):
                # sera retirado no create da tributacao()
                i[2]['fisnit_capa_id'] = nf_id
        vals['sequencia'] = len(self.search(
            cr, uid, [('nf_id', '=', vals.get('nf_id') or 0)], context=context)) + 1
        res = super(estoque_movimento_itens, self).create(cr, uid, vals, context)
        vals.get('tributacoes_ids') and self.pool.get('fiscal.nf.item.tributacao').\
            fisnit_exclui_valor_icms_da_base_calculo_pis_cofins(cr, uid, res, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}

        # Para realizar o write direto sem fazer contas.
        if context.get('atualizacao_inventario') or context.get('super_write_estoque_movimento_itens'):
            res = super(estoque_movimento_itens, self).write(cr, uid, ids, vals, context)
            # O valor da comissao e seu percentual precisam ser calculados/atualizados
            if context.get('verificar_nf'):
                self.estmvi_atualiza_comissao(cr, uid, ids, context)
            vals.get('tributacoes_ids') and self.pool.get('fiscal.nf.item.tributacao'). \
                fisnit_exclui_valor_icms_da_base_calculo_pis_cofins(cr, uid, ids, context)
            return res

        if vals.get('vr_unitario') or vals.get('produto_embalagem_id'):
            brw_movimento_itens = self.browse(cr, uid, ids[0], context)
            vr_unitario = vals.get('vr_unitario') or brw_movimento_itens.vr_unitario
            if vals.get('produto_embalagem_id'):
                produto_embalagem_id = isinstance(vals.get('produto_embalagem_id'), (list, tuple)) \
                                       and vals.get('produto_embalagem_id')[0] or \
                                       vals.get('produto_embalagem_id')
                vals['produto_embalagem_id'] = produto_embalagem_id
            else:
                produto_embalagem_id = brw_movimento_itens.produto_embalagem_id.id
            vals['vr_unitario'] = self.estmvi_calcula_vr_unitario_movimento(
                cr, uid, vr_unitario, produto_embalagem_id, context)
        res = super(estoque_movimento_itens, self).write(cr, uid, ids, vals, context)

        # Codigo abaixo atualiza os campos funcionais da fiscal.nf.item.tributacao
        campos = ['tributacoes_ids']
        dados_item = self.read(cr, uid, ids, campos, context, '_classic_write')
        if isinstance(ids, (int, long)):
            dados_item = [dados_item]
        trib_ids = []
        for item in dados_item:
            item.get('tributacoes_ids') and trib_ids.extend(item.get('tributacoes_ids'))
        if trib_ids:
            context['fisnit_super_write'] = True
            self.pool.get('fiscal.nf.item.tributacao').write(cr, uid, trib_ids, {}, context=context)

        # Calculo da comissao nao deve ser feito somente para NF de saida. Deve ser feito tambem
        # para entrada propria, uma vez que, essa seja de devolução de uma venda em que houve
        # comissão. Por isso a remoção da verificação do contexto. A comissao só é calculada se
        # tiver vendedor_id.
        self.estmvi_atualiza_comissao(cr, uid, ids, context)
        vals.get('tributacoes_ids') and self.pool.get('fiscal.nf.item.tributacao'). \
            fisnit_exclui_valor_icms_da_base_calculo_pis_cofins(cr, uid, ids, context)
        return res

    def estmvi_atualiza_comissao(self, cr, uid, ids, context):
        campos = ['vr_total_item_rateado', 'produto_id', 'vendedor_id']
        dados_item = self.read(cr, uid, isinstance(ids, (int, long)) and [ids] or ids, campos,
                               context, '_classic_write')
        for mov_item in dados_item:
            dados_comissao = self.estmvi_get_comissao(
                cr, uid, mov_item.get('produto_id'), mov_item.get('vr_total_item_rateado'),
                mov_item.get('vendedor_id'), context=context)
            if not dados_comissao:
                continue
            percentual_comissao = float(float_round(dados_comissao.get(
                'estmvi_percentual_comissao'), 3))
            vr_comissao = float(float_round(dados_comissao.get('estmvi_vr_comissao'), 6))
            sql = """
                    UPDATE estoque_movimento_itens
                    SET estmvi_percentual_comissao = %.3f, estmvi_vr_comissao = %.6f
                    WHERE id = %d
                """ % (percentual_comissao, vr_comissao, mov_item.get('id'))
            cr.execute(sql)

    def estmvi_get_comissao(self, cr, uid, produto_id, vr_total_item_rateado, vendedor_id,
                            dados_produto={}, context=None):
        """
        O metodo calcula a porcentagem e o valor da comissao para insercao na tabela
        estoque_movimento_itens. A porcentagem da comissao eh encontrada no cadastro do produto,
        senão na categoria do mesmo, senão no cadastro do vendedor ou senão no parametro do estoque.
        O valor base para calculo da comissao eh referente ao total vendido de cada item.

        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param produto_id: id da tabela estoque_produto
        :param vr_total_item_rateado: float do total do item rateado
        :param vendedor_id: id da tabela vendas_vendedor

        :return: Dicionario com os valores do percentual e total da comissao
        """
        context = context and context.copy() or {}
        res = {}

        if not produto_id or not vr_total_item_rateado or not vendedor_id:
            return res

        load = '_classic_write'
        obj_produto = self.pool.get('estoque.produto')
        obj_adm_util = self.pool.get('adm.utils')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')

        campos = ['estpro_percentual_comissao', 'categoria_id']
        if not dados_produto:
            dados_produto = obj_produto.read(cr, uid, produto_id, campos, context, load)

        comissao = dados_produto.get('estpro_percentual_comissao') or 0.0

        categoria_id = dados_produto.get('categoria_id')
        if not comissao and categoria_id:
            obj_categoria = self.pool.get('estoque.categoria')
            categoria_ids = obj_categoria.search(cr, uid, [], context=context)
            dict_categoria = {}
            campos = ['estcat_percentual_comissao', 'parent_id']
            map(lambda x: dict_categoria.update({x.get('id'): x}),
                obj_categoria.read(cr, uid, categoria_ids, campos, context, load))
            while categoria_id:
                categoria = dict_categoria.get(categoria_id)
                categoria_id = categoria.get('parent_id')
                comissao = categoria.get('estcat_percentual_comissao') or 0.0
                if comissao:
                    break

        if not comissao:
            comissao = obj_vendas_vendedor.read(cr, uid, vendedor_id, ['percentual'], context
                                                ).get('percentual') or 0.0

        if not comissao:
            estoque_parametro_dados = obj_adm_util.admutl_get_parametros(cr, uid, 'estoque',
                                                                         'comissoes')
            comissao = estoque_parametro_dados.get('comissoes') or 0.0

        vr_comissao = (comissao / 100) * vr_total_item_rateado

        res.update({'estmvi_percentual_comissao': comissao,
                    'estmvi_vr_comissao': vr_comissao})

        return res

    # Verifica se a quantidade pedida e maior que o saldo disponivel
    @staticmethod
    def estmvi_valida_disponibilidade_saldo(saldo, dados_item):
        """
        Verifica a disponibilidade do saldo do item

        :param saldo: saldo do item
        :param brw_item: browse do item da nota (estoque.movimento.itens)
        :type saldo: float
        :type brw_item: browser
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if 0 > saldo - dados_item.get('quantidade') or 0:
            return False

        return True

    @staticmethod
    def estmvi_get_mensagem_saldo_indisp(produto_nome, local_nome, dados_item, saldo):
        """
        Retorna a mensagem de saldo indisponivel

        :param produto_nome: Nome do produto
        :param local_nome: Nome do local informado para o produto
        :param brw_item: browse do item da nota (estoque.movimento.itens)
        :param saldo: saldo do item
        :type produto_nome: str
        :type local_nome: str
        :type brw_item: browser
        :type saldo: float
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        msg = '* Saldo indisponivel para o produto: (%s) no local: (%s) - ' \
              'qtde. pedida: %.3f - disponivel: %.3f;' % (
                  produto_nome, local_nome, dados_item.get('quantidade') or 0, saldo)
        return msg

    @staticmethod
    def estmvi_get_query_item_requisitado(grade_ids, company_id, esboco=False):
        """
        Retorna a query para verificar a quantidade reservada do item para as notas fiscais de saida
        :param grade_ids: Os ids das grades para ser verificada
        :param company_id:
        :type grade_ids: list
        :type company_id: int
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if esboco:
            _status = "('3', 'C', 'D' )"
        else:
            _status = "( '1', '3', 'C', 'D' )"
        str_grade_ids = '('+','.join(str(e) for e in grade_ids)+')'
        # state da nf - 1-Esboço, 3-Faturada, C-Cancelada, D-Denegada
        _sql = """
                SELECT emi.grade_id,
                       emi.local_id,
                       Sum(emi.quantidade) AS quantidade
                FROM   estoque_movimento_itens      AS emi
                LEFT JOIN fiscal_nf_capa            AS fnc ON fnc.id = emi.nf_id
                LEFT JOIN fiscal_cfop_natureza      AS fcn ON fcn.id = fnc.natureza_id
                LEFT JOIN estoque_natureza_operacao AS eno ON eno.id = fcn.fisnat_operacao_id
                WHERE emi.grade_id IN {grade_ids}
                  AND fnc.state NOT IN {status}
                  AND fnc.tipo = 'S'
                  AND fnc.company_id = {company_id}
                  AND eno.estnto_atualizar_saldo = TRUE 
                GROUP BY emi.grade_id, emi.local_id
        """.format(grade_ids=str_grade_ids, status=_status, company_id=company_id)
        return _sql

    # noinspection PyMethodMayBeStatic
    def estmvi_onchange_local_terceiro(self, cr, user, ids, local, context=None):
        """
        Remove o item (produto/servico) no formulario de item de nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item da nota fiscal
        :param local: ID do local (estoque.local)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type local: long
        :type context: dict
        :return: item (produto_id) sem valor.
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'produto_id': False}}

    # vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:


    def estmvi_soma_ipi_base_pis_cofins(self, cr, uid, tributacoes_ids, vr_total_item, context=None):
        """
        Este metodo tem o objetivo de atualizar os valores do O2M tributacoes_ids, este ira
        chamar o metodo fisnit_soma_ipi_base_pis_cofins da classe fiscal.nf.item.tributacao
        que sera responsavel por somar o valor do ipi na base de calculo de pis/cofins conforme
        solicitacao do chamado 93158
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param tributacoes_ids: O2M contendo as tributacoes informadas para o item
        :param vr_total_item: valor total do item deduzido de descontos e acrescentado
                              de seguro e outras despesas
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type tributacoes_ids: O2m
        :type vr_total_item: float
        :type context: dict
        :rtype: Padrao O2M para persistencia

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        tributacoes = self.resolve_2many_commands(
            cr, uid, 'tributacoes_ids', tributacoes_ids, [], context)
        tributacoes = self.pool.get('fiscal.nf.item.tributacao'). \
            fisnit_soma_ipi_base_pis_cofins(cr, uid, tributacoes, vr_total_item, context=context)
        for tributo in tributacoes_ids:
            if tributo[0] in (0, 1, 4):
                if tributo[0] == 0:
                    encargo_id = tributo[2].get('encargo_id')
                    for i in filter(lambda x: x.get('encargo_id') == encargo_id, tributacoes):
                        tributo[2].update(i)
                else:
                    fisnit_id = tributo[1]
                    for i in filter(lambda x: x.get('id') == fisnit_id, tributacoes):
                        tributo[2].update(i)
                        if tributo[1] == 4:
                            tributo[1] = 1
        return tributacoes_ids

    # TODO: Mover para a classe ´estoque.produto´ e alterar o prefixo para ´estpro_´
    def estmvi_get_tributos_item(self, cr, user, item_id, natureza, dados_pessoa, tipo, valor,
                                 seguro=0, despesa=0, frete=0, desconto=0, context=None):
        """
        Recupera as tributacoes de um item (produto/servico).

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item_id: ID do item (produto ou servico)
        :param natureza: ID da natureza da operacao
        :param pessoa: ID do cliente ou fornecedor (destinatario_id)
        :param tipo: o tipo da tributacao: "T"= Todas/Consumidor final, "P"= Produtor rural,
        "C"= Contribuinte e "E"= Entradas
        :param valor: o valor total (liquido) do item
        :param seguro: o valor atribuido ao seguro
        :param despesa: o valor atribuido aas despesas acessorias
        :param frete: o valor atribuido ao frete (o valor deve ser zero se o frete for "por fora")
        :param desconto: o valor atribuido ao desconto
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item_id: long
        :type natureza: long
        :type pessoa: long
        :type tipo: str
        :type valor: float
        :type seguro: float
        :type despesa: float
        :type frete: float
        :type desconto: float
        :type context: dict
        :return: lista com os valores dos tributos para o item
        :rtype: list
        """
        context = context and context.copy() or {}
        result = list()
        obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
        cfop_natureza_dados = self.pool.get('fiscal.cfop.natureza').read(
            cr, user, natureza, ['natureza_encargos_ids', 'tipo'], context=context)
        natureza_encargo_ids = cfop_natureza_dados.get('natureza_encargos_ids') or []
        cfop_natureza_encargo_dados = self.pool.get('fiscal.cfop.natureza.encargos').read(
            cr, user, natureza_encargo_ids, ['encargo_id', 'sobrepor_tributacao_id',
                                             'fiscne_sobrepoe_so_tributados'], context,
            '_classic_write')
        for encargo in cfop_natureza_encargo_dados:
            for tributo in filter(
                    lambda f: f.get('encargo_id') == encargo.get('encargo_id'),
                    self.pool.get('fiscal.utils').calcular_imposto(
                        cr, user, item_id, dados_destinatario=dados_pessoa, tipo_tributacao=tipo,
                        vr_produto=valor, encargo_ids=[encargo.get('encargo_id', 0)],
                        context=context)):
                if encargo.get('sobrepor_tributacao_id'):
                    # TODO: Transformar em metodo
                    if encargo.get('fiscne_sobrepoe_so_tributados'):
                        if self.pool.get('fiscal.tributacao').browse(
                                cr, user, tributo.get('tributacao_id')).cst_id.tipo == 'T':
                            tributacao_id = encargo.get('sobrepor_tributacao_id')
                        else:
                            tributacao_id = tributo['tributacao_id']
                    else:
                        tributacao_id = encargo.get('sobrepor_tributacao_id')
                    tributacao_dados = self.pool.get('fiscal.tributacao').read(
                        cr, user, tributacao_id, ['aliquota', 'base', 'cst_id'], context,
                        '_classic_write')
                    tributo['aliquota'] = tributacao_dados.get('aliquota', 0)
                    tributo['percentual_base'] = tributacao_dados.get('base', 0)
                    tributo['valor_base'] = \
                        valor * tributo.get('percentual_base', 0) / 100. + seguro + despesa + \
                        frete - desconto
                    tributo['valor_imposto'] = \
                        tributo.get('valor_base', 0) * tributo.get('aliquota', 0) / 100.
                    tributo['cst_id'] = tributacao_dados.get('cst_id')
                else:
                    tributo['valor_base'] += seguro + despesa + frete - desconto
                    tributo['valor_imposto'] = \
                        tributo['valor_base'] * tributo.get('aliquota', 0) / 100.
                    tributacao_id = tributo.get('tributacao_id')
                tipo_cst = self.pool.get('fiscal.cst').browse(
                    cr, user, tributo.get('cst_id'), context).tipo
                endereco_ids = self.pool.get('adm.pessoa').read(
                    cr, user, dados_pessoa.get('id'), ['endereco_ids'], context).get('endereco_ids',
                                                                                     [])
                if endereco_ids and tipo_cst == 'T' and cfop_natureza_dados.get('tipo') == 'S':
                    # Assume aliquota interestadual quando existir
                    endereco_id = self.pool.get('adm.pessoa').admpes_endereco_fiscal(
                        cr, user, endereco_ids, context)
                    uf_id = self.pool.get('adm.pessoa.endereco').browse(
                        cr, user, endereco_id, context).uf_id.id
                    tributos_interestaduais = \
                        obj_fiscal_nf_item_tributacao.fisnit_calcula_tributos_interestaduais(
                            cr, user, item_id, uf_id, encargo.get('encargo_id'), valor, context)

                    # Ajusta os valores dos tributos
                    if tributos_interestaduais:
                        tributo['aliquota'] = tributos_interestaduais.get('aliquota') or 0
                        tributo['percentual_base'] = tributos_interestaduais.get('base') or 0
                        tributo['valor_base'] = tributos_interestaduais.get('vr_base_calculo') or 0
                        tributo['valor_imposto'] = tributos_interestaduais.get('vr_encargo') or 0
                        tributo['cst_id'] = tributos_interestaduais.get('cst_id')
                        tributacao_id = tributos_interestaduais.get('tributacao_id')

                total_base = tributo.get('valor_base') or 0
                total_imposto = tributo.get('valor_imposto') or 0
                tot_base_reduzida = valor - total_base
                tot_base_reduzida = tot_base_reduzida if tot_base_reduzida > 0 else 0
                tipo_substituicao = False
                tipo_cst = ''
                if tributo.get('cst_id'):
                    browse_fiscal_cst = self.pool.get('fiscal.cst').browse(
                        cr, user, tributo.get('cst_id'), context)
                    if browse_fiscal_cst.tipo == 'U':
                        if browse_fiscal_cst.codigo in ('10', '30', '60', '70'):
                            tipo_substituicao = True
                if tributo.get('nome_encargo', '').upper() == 'ICMS':
                    percentual_mva = tributo.get('mva') or 0
                    aliquota_venda = tributo.get('aliquota_venda') or 0
                else:
                    percentual_mva = tributo.get('mva') or 0
                    aliquota_venda = tributo.get('aliquota_venda') or 0
                if tributo['modbcst'] == '4':
                    busca_info_st = not context.get('terceiro') and \
                        obj_fiscal_nf_item_tributacao.fisnit_calcular_vr_icms_st_item(
                            round(total_base, 2), tributo.get('aliquota') or 0, aliquota_venda,
                            percentual_mva) or {}
                    vr_icms_substituicao = busca_info_st.get('vr_icms_substituicao') or 0
                    vr_base_icms_substituicao = \
                        busca_info_st.get('vr_base_icms_substituicao') or 0
                else:
                    vr_icms_substituicao = vr_base_icms_substituicao = 0
                result.append([0, False, {
                    'encargo_id': tributo.get('encargo_id'), 'cst_id': tributo.get('cst_id'),
                    'tributacao_id': tributacao_id, 'aliquota': tributo.get('aliquota'),
                    'vr_base_calculo': round(total_base, 2), 'vr_encargo': round(total_imposto, 2),
                    'modbcst': tributo.get('modbcst', ''), 'tipo_substituicao': tipo_substituicao,
                    'vr_reducao_base': round(tot_base_reduzida, 2), 'fisnit_tipo_cst': tipo_cst,
                    'fisnit_csosn_id': tributo.get('csosn_id'), 'percentual_mva': percentual_mva,
                    'vr_icms_substituicao': vr_icms_substituicao, 'aliquota_venda': aliquota_venda,
                    'vr_base_icms_substituicao': vr_base_icms_substituicao}])

        return result

    def estmvi_get_preco_nf_transferencia(self, cr, uid, produto_id, company_id, context):
        """
        Este metodo serve para busca o preco unitario do item quando a nota for de transferencia
        nesta situacao, o preco unitario na venda sera observado de acordo com o parametro
        estpar_preco_para_transferencia
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param produto_id: id do produto para buscar o preco
        :param company_id: id da empresa a verificar
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type produto_id: int | long
        :type company_id: int
        :type context: dict
        :return: valor unitario do item de acordo com o parametro estpar_preco_para_transferencia
        :rtype: float
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        preco_unitario = 0.00
        dados_custo = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id, company_id,
                                                                epc_data=None, context=context)
        if dados_custo and dados_custo.get(produto_id):
            preco_utilizar = obj_adm_utils.admutl_get_parametros(
                cr, uid, 'estoque.parametro', ['estpar_preco_para_transferencia'], context=context)
            if preco_utilizar:
                preco_utilizar = preco_utilizar['estpar_preco_para_transferencia']
            if not preco_utilizar or preco_utilizar == 'U':
                preco_unitario = dados_custo[produto_id]['estprc_ultimo_custo']
            elif preco_utilizar == 'M':
                preco_unitario = dados_custo[produto_id]['estprc_custo_medio_contabil']
            elif preco_utilizar == 'R':
                preco_unitario = dados_custo[produto_id]['estprc_custo_reposicao']
            elif preco_utilizar == 'C':
                preco_unitario = dados_custo[produto_id]['estprc_custo_corrigido']
            else:
                preco = self.get_item_preco_venda(cr, uid, produto_id, company_id,
                                                           context=context).get(produto_id) or {}
                preco_unitario = preco.get('preco_venda') or 0

        return preco_unitario

    def estmvi_onchange_natureza_propria(self, cr, user, ids, natureza_id, tributacoes_o2m,
                                         context=None):
        context = context and context.copy() or dict()

        # Desoneracao de ICMS
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]
        desoneracao = natureza_id and self.pool.get('fiscal.cfop.natureza').read(
            cr, user, natureza_id, ['fisnat_icms_desonerado'], context).get(
            'fisnat_icms_desonerado')
        for tributacao in tributacoes_o2m:
            tributacao_dados = self.resolve_2many_commands(
                cr, user, 'tributacoes_ids', [tributacao], ['vr_encargo'], context)
            tributacao_dados = tributacao_dados and tributacao_dados[0] or {}
            tributacao[2] = \
                tributacao[2] and tributacao[2].update(tributacao_dados) or tributacao_dados
            if desoneracao and tributacao[2].get('encargo_id') == icms_id:
                vr_desonerado = tributacao[2].get('vr_encargo') or 0
            else:
                vr_desonerado = 0
            tributacao[2].update(fisnit_vr_icms_desonerado=vr_desonerado)
            tributacao[0] = tributacao[2].get('id') and 1 or 0

        return {'value': {'tributacoes_ids': tributacoes_o2m}}

    def estmvi_onchange_cfop_id(self, cr, uid, ids, natureza_id, cfop_id, context=None):
        """
        O metodo valida se o CFOP da natureza da nota eh compativel com o CFOP do item, ou seja, se
        ambos sao para dentro ou para fora do estado. Metodo chamado apos inclusao de dados no campo
        'CFOP'.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param natureza_id: id da tabela fiscal_cfop_natureza.
        :type natureza_id: int
        :param cfop_id: id da tabela fiscal_cfop.
        :type cfop_id: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dados dos campos com seus respectivos valores e/ou mensagem de alerta ao usuario.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not natureza_id or not cfop_id:
            return res
        primeiro_digito_cfop = self.pool.get('fiscal.cfop').browse(cr, uid, cfop_id,
                                                                   context).codigo[0]
        brw_fiscal_cfop_natureza = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, natureza_id, context)
        primeiro_digito_cfop_natureza = brw_fiscal_cfop_natureza.cfop_id.codigo[0]
        if primeiro_digito_cfop == primeiro_digito_cfop_natureza:
            return res
        return {'value': {'cfop_id': False},
                'warning': {'title': _('Atencao'),
                            'message': _('CFOP invalido para a natureza da operacao:\n %s - %s' %
                                         (brw_fiscal_cfop_natureza.cfop_id.codigo,
                                          brw_fiscal_cfop_natureza.name))}}

    def unlink(self, cr, uid, ids, context=None):
        """
        Remove os registro de ajustes fiscais em funcao do movimento.
        
        :param cr: 
        :param uid: 
        :param ids: 
        :param context: 
        :type cr: 
        :type uid: object
        :type ids: list
        :type context: dict
        :return: verdadeiro se a exclusao do item de movimento foi realizada
        :rtype: bool
        """
        context = context and context.copy() or {}

        # Remove os registros de ajustes fiscais em funcao do movimento (complemento de arroz)
        ajuste_ids = self.pool.get('fiscal.ajuste.item').search(
            cr, uid, [('fisaji_movimento_item_id', 'in', ids)], context=context)
        self.pool.get('fiscal.ajuste.item').unlink(cr, uid, ajuste_ids, context)

        return super(estoque_movimento_itens, self).unlink(cr, uid, ids, context)

    def estmvi_onchange_tributacao(self, cr, user, ids, tributacao, context=None):
        # IMPORTANTE! Nao remover este metodo. Mesmo "vazio" resolve o problema de ´javascript´
        # relatado nos atendimentos 110575 e 110987.
        pass

        return dict()

    def estmvi_onchange_incide_despesa_bc_propria(self, cr, user, ids, vr_item, vr_despesa,
                                                  incide_despesa, vr_frete, vr_seguro, vr_desconto,
                                                  tributacoes_o2m, context=None, natureza_id=None):
        """
        Ajusta os valores dos tributos em funcao do valor da despesa (rateada) do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimento 
        :param vr_item: o valor total do item
        :param vr_despesa: o valor das despesas rateada do item
        :param incide_despesa: se o valor da despesa rateada incide ou nao na base de calculo.
        :param vr_frete: o valor do frete rateado do item
        :param vr_seguro: o valor do seguro rateado do item
        :param vr_desconto: o valor do desconto rateado do item
        :param tributacoes_o2m: lista com o ID das tributacoes do item (one2many)
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type vr_item: float
        :type vr_despesa: float
        :type incide_despesa: bool
        :type vr_frete: float
        :type vr_seguro: float
        :type vr_desconto: float
        :type tributacoes_o2m: list
        :type context: dict
        :type natureza_id: long
        :return: as tributacoes e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'tributacoes_ids': self._estmvi_ajusta_tributacao_propria(
            cr, user, vr_item, vr_despesa, incide_despesa, vr_frete, vr_seguro, vr_desconto,
            tributacoes_o2m, context, natureza_id)}}

    def estmvi_onchange_incide_despesa_bc(self, cr, user, ids, tributacoes, despesa, desconto,
                                          seguro, frete, tipo, total, incide_despesa, context=None):
        """
        Ajusta os valores dos tributos em funcao do valor total do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento (produto)
        :param tributacoes: lista com os dados das tributacoes (one2many)
        :param despesa: o valor das despesas acessorias rateada (proporcional as despesas na capa)
        :param desconto: desconto rateado
        :param seguro: o valor do seguro rateado (proporcional ao seguro na capa)
        :param frete: o valor do frete rateado (proporcional oa frete na capa)
        :param tipo: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param total: o valor total do item
        :param incide_despesa: se incidira ou nao o valor da despesa na base de calculo
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tributacoes: list
        :type seguro: float
        :type frete: float
        :type tipo: str
        :type total: float
        :type context: dict
        :return: o valor unitario do item e os valores da base de calculo e o valor do encargo em
         funcao do valor total do item.
        :rtype: dict
        """
        context = context and context.copy() or dict()

        return self._estmvi_ajusta_tributacao(
            cr, user, ids, tributacoes, despesa, desconto, seguro, frete, tipo, total,
            incide_despesa, context)

    def estmvi_onchange_despesa_rateado_propria(self, cr, user, ids, vr_item, vr_despesa,
                                                incide_despesa, vr_frete, vr_seguro, vr_desconto,
                                                tributacoes_o2m, context=None, natureza_id=None):
        """
        Ajusta os valores dos tributos em funcao do valor da despesa (rateada) do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimento 
        :param vr_item: o valor total do item
        :param vr_despesa: o valor das despesas rateada do item
        :param incide_despesa: se o valor da despesa rateada incide ou nao na base de calculo.
        :param vr_frete: o valor do frete rateado do item
        :param vr_seguro: o valor do seguro rateado do item
        :param vr_desconto: o valor do desconto rateado do item
        :param tributacoes_o2m: lista com o ID das tributacoes do item (one2many)
        :param context: um dicionario padrao para valores contextuais
        :param natureza_id: o ID da natureza da operacao
        :type cr: object
        :type user: int
        :type ids: list
        :type vr_item: float
        :type vr_despesa: float
        :type incide_despesa: bool
        :type vr_frete: float
        :type vr_seguro: float
        :type vr_desconto: float
        :type tributacoes_o2m: list
        :type context: dict
        :type natureza_id: long
        :return: as tributacoes e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'tributacoes_ids': self._estmvi_ajusta_tributacao_propria(
            cr, user, vr_item, vr_despesa, incide_despesa, vr_frete, vr_seguro, vr_desconto,
            tributacoes_o2m, context, natureza_id)}}

    def estmvi_check_icms_st(self, cr, user, ids, context=None):
        """
        Verifica se um item que eh tributado nao contem valor de base ou valor de substituicao.

        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de ID dos itens a serem verificados
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se o item tributado nao contem valor de base ou valor de substituicao.
        :rtype: bool
        """
        context = context and context.copy() or {}
        for item in self.browse(cr, user, isinstance(ids, (list, tuple)) and ids or [ids], context):
            for tributacao in item.tributacoes_ids:
                if tributacao.cst_id.tipo not in ('U', 'TU'):
                    if tributacao.vr_base_icms_substituicao or tributacao.vr_icms_substituicao:
                        return False

        return True

    @log_debug
    def estmvi_valida_quantidade(self, cr, uid, nf_id, context=None):
        """
        O metodo valida a quantidade informada nos itens da NF para que nao seja validada/faturada
        com valor menor ou igual a zero.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da Nota Fiscal.
        :type nf_id: int
        :param context: dados contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        nf_referencia_ids = self.pool.get('fiscal.nf.capa').read(
            cr, uid, nf_id, ['fisnfc_nf_referencia_ids'], context, '_classic_write').get(
            'fisnfc_nf_referencia_ids')
        # Nota Fiscal de complemento pode possuir itens com quantidade zero.
        if nf_referencia_ids:
            return True

        movimento_ids = self.search(cr, uid, [('nf_id', '=', nf_id), ('quantidade', '<=', 0)],
                                    context=context)
        if not movimento_ids:
            return True
        estoque_movimento_dados = self.read(cr, uid, movimento_ids, ['sequencia'], context)
        raise osv.except_osv(
            _('Atencao'),
            _('Os itens de "Sequencia" abaixo estao com quantidade invalida:\n' + ',\n'.join(map(
                lambda i: str(i.get('sequencia')), estoque_movimento_dados))))

    @staticmethod
    def estmvi_get_data_ultima_entrada(cr, local=False, produto=False, grade=False,
                                       company_id=False):
        """
        Recupera a data da ultima entrada de um produto (item). Operacao de movimento de compra.

        :param cr: cursor do banco de dados
        :param local: lista com ID do local a ser consultado
        :param produto: lista com ID dos produtos a serem consultados
        :param grade:  lista com o ID das grades dos produtos a serem consultados
        :param company_id: id da tabela res_company, identificador da empresa
        :type cr: cursor
        :type local: long|list[long]
        :type produto: long|list[long]
        :type grade: long|list[long]
        :type company_id: int
        :return: lista de dicionarios com os dados do produt, grade, local e ultima data de entrada
        :rtype: list[dict]
        """

        sql = """
                SELECT emi.produto_id
                     , emi.grade_id
                     , emi.local_id
                     , MAX(emc.data)                AS ult_entrada
                FROM   estoque_movimento_capa       AS emc
                INNER  JOIN estoque_movimento_itens AS emi ON emc.id = emi.capa_id
                WHERE  emc.nf_capa_id in (
                       SELECT id
                       FROM   fiscal_nf_capa
                       WHERE  state = '3' AND tipo = 'E'
                       )
            """
        if company_id:
            sql += """
                    AND emc.company_id = %s
                """ % company_id

        if local:
            sql += """
                  AND  emi.local_id = %s
                """ % local

        sql += """
                  AND  emc.estmvc_operacao_id IN (
                       SELECT id
                       FROM estoque_natureza_operacao
                       WHERE estnto_tipo_operacao = 'E'
                         AND estnto_atualizar_saldo = True
                         AND active = True
                         AND estnto_movimento_origem_id IN (
                            Select id from estoque_movimento_origem
                                WHERE estmog_codigo = 'N')
                       )
                  AND  emi.cfop_id IN (
                       SELECT id
                       FROM   fiscal_cfop
                       WHERE  movimento_tipo = 'C'
                       )
            """

        if produto:
            sql += """
                  AND  emi.produto_id IN (%s)
                """ % ', '.join(
                [str(_id) for _id in (produto if isinstance(produto, list) else [produto])])

        if grade:
            sql += """
                  AND  emi.grade_id IN (%s)
                """ % ', '.join(
                [str(_id) for _id in (grade if isinstance(grade, list) else [grade])])

        sql += """
                GROUP BY emi.local_id, emi.produto_id, emi.grade_id
            """
        cr.execute(sql)

        return cr.dictfetchall() or [{}]

    @staticmethod
    def estmvi_get_quantidade_entrada(cr, dt_inicio, dt_fim, local=False, company=False,
                                      produto=False, grade=False):
        """
        Recupera ultima quantidade de entrada para um determinado produto (item).  Operacao de
        movimento de compra.

        :param cr: cursor do banco de dados
        :param local: lista com ID do local a ser consultado
        :param company: id da tabela res_company, identificador da empresa
        :param produto: lista com ID dos produtos a serem consultados
        :param grade:  lista com o ID das grades dos produtos a serem consultados
        :param dt_inicio: data base do periodo de consulta
        :param dt_fim: data limite do periodo de consulta
        :type cr: cursor
        :type local: long|list[long]
        :type company: int
        :type produto: long|list[long]
        :type grade: long|list[long]
        :type dt_inicio: str
        :type dt_fim: str
        :return: lista de dicionarios com os dados do produt, grade, local e ultima data de entrada
        :rtype: list[dict]
        """

        sql = """
                SELECT emi.produto_id
                     , emi.grade_id
                     , emi.quantidade         AS quantidade
                FROM   estoque_movimento_itens     AS emi
                INNER  JOIN estoque_movimento_capa AS emc ON emi.capa_id = emc.id
                WHERE  emc.estmvc_origem = 'N'
            """

        if local:
            sql += """
                  AND  emi.local_id IN (%s)
                """ % ', '.join(
                [str(_id) for _id in (local if isinstance(local, list) else [local])])

        if company:
            sql += """
                  AND  emc.company_id IN (%s)
                """ % ', '.join(
                [str(_id) for _id in (company if isinstance(company, list) else [company])])

        sql += """
                  AND  emc.nf_capa_id IN (
                           SELECT id
                           FROM fiscal_nf_capa
                           WHERE state = '3' AND tipo = 'E'
                           )
                  AND  emc.estmvc_operacao_id in (
                           SELECT id
                           FROM   estoque_natureza_operacao
                           WHERE  estnto_tipo_operacao = 'E'
                             AND  estnto_atualizar_saldo = True
                             AND  active = True
                       )
                  AND  emc.data BETWEEN '%s' AND '%s'
                  AND  emi.cfop_id IN (
                       SELECT id
                       FROM   fiscal_cfop
                       WHERE  movimento_tipo = 'C'
                       )
            """ % (dt_inicio, dt_fim)

        if produto:
            sql += """
                  AND  emi.produto_id IN (%s)
                """ % ', '.join(
                [str(_id) for _id in (produto if isinstance(produto, list) else [produto])])

        if grade:
            sql += """
                  AND  emi.grade_id = (%s)
                """ % ', '.join(
                [str(_id) for _id in (grade if isinstance(grade, list) else [grade])])

        sql += """
                --GROUP BY emi.local_id, emi.produto_id, emi.grade_id
                ORDER BY emi.id desc
                LIMIT 1
            """

        cr.execute(sql)

        return cr.dictfetchall() or [{}]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
