# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estoque_movimento_itens(osv.Model):
    """
    PREFIXO: ESTMVI
    complemento do movimento do estoque

    """

    def _estmvi_get_dados_nf(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        parametros = ['fispar_sped_fiscal_e200_manual']
        fiscal_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', parametros, context=context)
        res = {}.fromkeys(
            ids, {'estmvi_state_nf': '', 'estmvi_item_nao_salvo': False, 'estmvi_tipo_nf': '',
                  'estmvi_parametro_e200': fiscal_parametro_dados.get(
                      'fispar_sped_fiscal_e200_manual')})
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')
        nf_ids = obj_fiscal_nf.search(cr, uid, [('estoque_movimento_itens_ids','in', ids)],
                                      context=context)
        dados_nf = obj_fiscal_nf.read(cr, uid, nf_ids, ['state','estoque_movimento_itens_ids',
                                      'tipo'], context=context, load='_classic_write')
        for nf in dados_nf:
            for id_ in ids:
                if id_ in nf.get('estoque_movimento_itens_ids'):
                    res[id_].update({'estmvi_state_nf': nf.get('state'),
                                     'estmvi_item_nao_salvo': False,
                                     'estmvi_tipo_nf': nf.get('tipo')})

        return res

    def estmvi_get_permite_alterar_preco(self, cr, uid, context=None):
        context = context and context.copy() or {}
        '''Verifica se o usuario logado tem permissao para alterar o preco de venda'''
        obj_ir_model_access = self.pool.get('ir.model.access')
        permite_acesso = obj_ir_model_access.\
            check_groups(cr, uid, 'estoque.group_estoque_alterar_preco')
        return permite_acesso

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(estoque_movimento_itens, self).default_get(cr, uid, fields, context)
        if context.get('tipo_nota'):
            res['estmvi_tipo_nf'] = context['tipo_nota']
        return res

    _name = 'estoque.movimento.itens'
    _inherit = 'estoque.movimento.itens'
    _columns = {
        'nf_id': fields.many2one("fiscal.nf.capa", 'Nota fiscal', ondelete="cascade", select=True), # id da capa da nf
        'sequencia': fields.integer('Sequencia', select=True), # Sequencia do item na nf
        'vr_unitario': fields.float('Vr unitario', digits=(15, 6), required=False), # Valor Unitario do item
        'vr_desconto': fields.float('Vr desconto', digits=(15, 2)),
        # Desconto total do item(multiplicado pela quantidade)
        'vr_total_item': fields.float('Vr total item', digits=(15, 2), required=False),
        # Valor Total do item(multiplicado pela quantidade), deduzido o desconto
        'vr_preco_venda': fields.float('Vr preco venda', digits=(15, 6), required=False),
        # Preco de venda do item no cadastro no momento da venda
        'cfop_id': fields.many2one("fiscal.cfop", 'Cfop', ondelete="restrict", select=True),
        'natureza_id': fields.many2one("fiscal.cfop.natureza", 'Natureza', ondelete="restrict", select=False),
        'tributacoes_ids': fields.one2many('fiscal.nf.item.tributacao', 'movimento_itens_id', 'Tributacoes'),
        'verifica_usa_grade': fields.boolean('verifica_usa_grade'),
        'vr_frete_rateado': fields.float('Vr frete rateado', digits=(15, 2)),
        'vr_seguro_rateado': fields.float('Vr seguro rateado', digits=(15, 2)),
        'vr_despesas_rateado': fields.float('Vr despesas rateado', digits=(15, 2)),
        'vr_desconto_rateado': fields.float('Vr desconto rateado', digits=(15, 2)),
        #Valor do desconto rateado ( desconto da nota fiscal(capa))
        'estmvi_vr_despesas_extras_rateado': fields.float('Despesas Extras', digits=(15, 2)),
        'estmvi_vr_icms_frete_rateado': fields.float('Vr. ICMS Frete rateado', digits=(15, 2)),
        # OBS para o campo 'vr_total_item_rateado':
        # Frete por dentro:
        # · Com tributacao - nao recebe rateio do frete
        # · Sem tributacao - recebe rateio do frete
        'vr_total_item_rateado': fields.float('Vr total item rateado', digits=(15, 2)), #Valor do item - rateado
        'vendedor_id': fields.many2one('vendas.vendedor', 'Vendedor', ondelete='restrict'),
        'vendedor_externo_id': fields.many2one('vendas.vendedor', 'Vendedor Externo', ondelete='restrict'),
        'fornecedor_id': fields.related('nf_id', 'destinatario_id', type="many2one", relation="adm.pessoa",
                                        store=False),
        'estmvi_item_nao_salvo': fields.function(_estmvi_get_dados_nf, type="boolean", method=True,
                                                 store=False, multi='dados_nf',
                                                 string='Item nao salvo'), #flag nao historiado, para verificar se item já foi lancado na base de dados

        'estmvi_permissao_altera_preco': fields.boolean('Permitido alterar preco venda'),  #flag nao historiado,para buscar acesso ao preco de venda
        'estmvi_tipo_nf': fields.function(_estmvi_get_dados_nf, type="char", size=1, method=True,
                        store=False, string='tipo nf', multi='dados_nf'),
        'estmvi_state_nf': fields.function(_estmvi_get_dados_nf, type="char", size=1, method=True,
                        store=False, string='State NF', multi='dados_nf'),
        'ajuste_fiscal_ids': fields.one2many('fiscal.ajuste.item', 'fisaji_movimento_item_id',
                                             string='Ajustes fiscais'),
        'estmvi_incide_despesa_bc': fields.boolean('Incide o valor da despesa na base de calculo',
                                                   help='Se marcado (padrao), o valor da despesa '
                                                        'sera incorporado ao valor da base de '
                                                        'calculo.'),
        'estmvi_sped_fiscal_e200_manual': fields.boolean(
            'Item gera apuracao p/ registro E200',
            help="Se marcado, o item gera no SPED Fiscal apuracao para o registro E200. "
                 "Os valores devem ser informados manualmente no arquivo gerado ou validador."),
        'estmvi_parametro_e200': fields.function(_estmvi_get_dados_nf, type='boolean', method=True,
                                                 string='Parametro registro E200',
                                                 multi='dados_nf'),
        }

    _defaults = {
        'estmvi_permissao_altera_preco': estmvi_get_permite_alterar_preco,
        'vr_frete_rateado': 0,
        'vr_seguro_rateado': 0,
        'vr_despesas_rateado': 0,
        'vr_desconto_rateado': 0,
        'estmvi_vr_despesas_extras_rateado': 0,
        'estmvi_vr_icms_frete_rateado': 0,
        'vr_total_item_rateado': 0,
        'estmvi_incide_despesa_bc': True,
        'natureza_id': lambda s, c, u, x: x.get('natureza_id') or False,
        'cfop_id': lambda s, c, u, x: x.get('natureza_id') and s.pool.get(
            'fiscal.cfop.natureza').read(c, u, x.get('natureza_id'), ['cfop_id'], x).get(
            'cfop_id')[0] or False
    }

    _order = 'nf_id, sequencia'

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
