# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2016, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class estoque_produto(osv.Model):
    """
    Heranca da classe ´estoque_produto´.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _inherit = 'estoque.produto'

    def estpro_get_dados_unidade_tributavel(self, cr, user, item, context=None):
        """
        Recupera os dados da unidade tributavel de um item (produto/servico).

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param item: ID do item (produto/servico)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type context: dict
        :return: dados da unidade tributavel do item.
        :rtype: dict

        """
        context = context and context.copy() or {}
        values={}
        if not item:
            return values
        brw_item = self.browse(cr, user, item, context)
        unidade_tributavel = brw_item.estpro_unidade_tributavel_id
        if not unidade_tributavel:
            return values
        values = {'unidade': unidade_tributavel.unidade,
                  'name': unidade_tributavel.name,
                  'fracao': unidade_tributavel.fracao}

        # Atualiza os dados da unidade tributavel do item
        values.update({
            'estpro_unidade_tributavel_id': unidade_tributavel.id,
            'estpro_fator_conversao_unidade_tributavel':
                brw_item.estpro_fator_conversao_unidade_tributavel or 0})

        return values

    def estpro_get_tributacao_entrada(self, cr, user, item, fornecedor, context=None):
        """
        Recupera as tributacoes de entrada para um item baseadas no fornecedor.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param item: ID do item (produto/servico)
        :param fornecedor: ID do fornecedor
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type fornecedor: long
        :type context: dict
        :return: lista com o ID das tributacoes
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        brw_tributacao_entrada = item and filter(
            lambda f: f.fornecedor_id.id == fornecedor and f.produto_id == item, self.browse(
                cr, user, item, context).tributacao_entrada_ids)

        return item and map(lambda m: m.tributacao_id.id, brw_tributacao_entrada) or []

    def estpro_get_tributacao_item(self, cr, user, item, tributacao_tipo, context=None):
        """
        Recupera as tributacoes de um item (produto/servico).

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param item: ID do item (produto/servico)
        :param tributacao_tipo: tipo da tributacao: T= todas/consumidor final; P= produtor rural;
        C= contribuinte ou E= entradas
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type tributacao_tipo: str
        :type context: dict
        :return: lista com o ID das tributacoes.
        :rtype: list

        """

        context = context and context.copy() or {}
        tributacao_item_ids = item and self.pool.get('fiscal.tributacao.encargos').search(
            cr, user, [('produto_id', '=', item), ('tipo_tributacao', '=', tributacao_tipo)],
            context=context)

        return tributacao_item_ids and map(lambda m: m.tributacao_id.id,
                                           self.pool.get('fiscal.tributacao.encargos').browse(
                                               cr, user, tributacao_item_ids, context)) or []

    def create(self, cr, uid, vals, context=None):
        """
        Criar o cadastro do produto, se as informacoes de pis e cofins forem validas

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param vals: Valores para criacao do produto
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type vals: dict
        :type context: dict
        :return: Id do item criado.
        :rtype: long

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = super(estoque_produto, self).create(cr, uid, vals, context)
        if res:
            self._estpro_valida_cst_pis_cofins(cr, uid, res, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        """
        Altera o cadastro do produto, se as informacoes de pis e cofins forem validas.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: Lista de ids dos itens a serem alterados
        :param vals: Valores para criacao do produto
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type vals: dict
        :type context: dict
        :return: True se os itens foram alterados
        :rtype: bool

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = super(estoque_produto, self).write(cr, uid, ids, vals, context)
        for produto_id in ids if isinstance(ids, list) else [ids]:
            self._estpro_valida_cst_pis_cofins(cr, uid, produto_id, context)

        return res

    def _estpro_valida_cst_pis_cofins(self, cr, uid, produto_id, context=None):
        """
        Metodo privado, utilizado apenas para centralizar as chamadas das validacoes do pis e do
        cofins realizadas pela classe fiscal.tributacao.encargos

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param produto_id: Id do produto a ser validado
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type produto_id: int | long
        :type context: dict
        :return: True se os itens foram alterados
        :raise: Caso o produto nao possua o cadastrado de pis/cofins validado, mostra na tela
                qual e o item

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        cst_valido = obj_fiscal_tributacao_encargos.valida_cst_pis_cofins(
            cr, uid, produto_id, context=context)
        natureza_pessoa_juridica = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', 'fispar_indicador_natureza_pessoa_juridica', context=context).get(
            'fispar_indicador_natureza_pessoa_juridica')
        for i in cst_valido.values():
            if not i and natureza_pessoa_juridica != '02':
                produto_codigo = self.browse(cr, uid, produto_id).codigo
                produto_descricao = self.browse(cr, uid, produto_id).name
                produto = produto_codigo + '-' + produto_descricao
                raise osv.except_osv(_('Atencao'),
                                     _('O produto %s possui CST de PIS diferente do CST de '
                                       'COFINS') % produto)

    @staticmethod
    def estpro_produto_nacional(origem_mercadoria):
        """
        Metodo retorna True caso a origem da mercadoria seja nacional
        :param origem_mercadoria: Uma string contendo o codigo da origem do produto
        :type origem_mercadoria: str
        :return: True se o produto for nacional
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        origens = ['0', '3', '4', '5', '8']  # Origens nacionais
        return origem_mercadoria in origens

    def estpro_verificar_encargo_obrigatorio(self, cr, user, item, context=None):
        """
        Verifica se os itens possuem todos os encargos que sao obrigatorios na empresa
        (fiscal_encargo_empresa).

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param item: ID do item (produto/servico)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type context: dict
        :return: verdadeiro se o item possue todos os encargos obrigatorios
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        company_id = self.pool.get('estoque.produto').browse(
            cr, user, item, context).company_id.id
        encargo_empresa_ids = self.pool.get('fiscal.encargo.empresa').search(
            cr, user, [('company_id', '=', company_id),
                       ('fiseem_encargo_obrigatorio', '=', True)], context=context)
        encargo_empresa_ids = map(lambda m: m.get('encargo_id'),
                                  self.pool.get('fiscal.encargo.empresa').read(
                                      cr, user, encargo_empresa_ids, ['encargo_id'], context,
                                      '_classic_write'))
        encargo_item_ids = self.pool.get('fiscal.tributacao.encargos').search(
            cr, user, [('produto_id', '=', item), ('tipo_tributacao', '=', 'T')], context=context)
        encargo_item_ids = map(lambda m: m.get('encargo_id'),
                               self.pool.get('fiscal.tributacao.encargos').read(
                                   cr, user, encargo_item_ids, ['encargo_id'], context,
                                   '_classic_write'))
        return not list(set.difference(set(encargo_empresa_ids), set(encargo_item_ids)))

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        return super(estoque_produto, self).name_search(
            cr, user, name, args, operator, context, limit)

    def copy(self, cr, uid, _id, default=None, context=None):
        context = context and context.copy() or {}
        default = default and default.copy() or {}
        default['tributacao_entrada_ids'] = list()
        return super(estoque_produto, self).copy(cr, uid, _id, default, context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
