# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


MERCADORIA_NACIONAL = [
    ('0', '0 - Nacional'),
    ('3', '3 - Nacional, mercadoria ou bem com Conteudo de Importacao superior a 40% e inferior ou igual a 70%'),
    ('4', '4 - Nacional, conforme processo produtivo basico dec 288/67'),
    ('5', '5 - Nacional, mercadoria ou bem com Conteudo de Importacao inferior ou igual a 40% (quarenta por cento)'),
    ('8', '8 - Nacional, mercadoria ou bem com conteudo de importacao superior a 70%')
]
MERCADORIA_ESTRANGEIRA = [
    ('1', '1 - Estrangeira - Importacao Direta'),
    ('2', '2 - Estrangeira - Adquirida no mercado interno'),
    ('6', '6 - Estrangeira - Importacao direta, sem similar nacional, constante em lista de Resolucao CAMEX'),
    ('7', '7 - Estrangeira - Adquirida no mercado interno, sem similar nacional, constante em lista de Resolucao CAMEX.'),
]

ORIGEM_MERCADORIAS = MERCADORIA_NACIONAL + MERCADORIA_ESTRANGEIRA
ORIGEM_MERCADORIAS.sort(key=lambda i: i[0])


class estoque_produto(osv.Model):
    _name = 'estoque.produto'
    _inherit = 'estoque.produto'
    _columns = {
        'ncm_id': fields.many2one('fiscal.ncm', 'NCM', ondelete="restrict", select=1),  # Nomenclatura comum do mercosul
        'codigo_ex_ncm': fields.char('Codigo ex ncm', size=3),  # Codigo de excessao do NCM conforme TIPI!
        'codigo_enq_ipi': fields.char('Codigo enq ipi', size=3, help="Codigo de enquadramento do IPI"), # Codigo de enquadramento do IPI\n
        'codigo_genero': fields.char('Codigo genero', size=2),  # Codigo do Genero de produto conforme ato Cotep 09/08!\n
        'codigo_tab_servicos_id': fields.many2one('fiscal.tabela.servico','Codigo tab servicos'), # Codigo de servicos comforne lei complementar 116/03\n
        'tributacao_ids': fields.one2many('fiscal.tributacao.encargos', 'produto_id', 'Tributacao'),
        'tribut_excecao_ids': fields.one2many('fiscal.tributacao.excecao', 'produto_id', 'Excecao'),
        'origem_mercadoria': fields.selection(ORIGEM_MERCADORIAS, "Origem da mercadoria", required=True, size=1),
        'tributacao_entrada_ids': fields.one2many('fiscal.tributacao.entrada', 'produto_id', 'Tributacao Entrada'),
        'estpro_codigo_anp_id': fields.many2one('fiscal.cadastro.anp','Codigo na ANP', help="Codigo do produto conforme Agencia Nacional de Petroleo"),
        'estpro_pis_nat_rec': fields.char('Natureza da Receita do PIS', help='Codigo da Natureza da Receita para efeitos de Pis', size=3),
        'estpro_cofins_nat_rec': fields.char('Natureza da Receita do COFINS', help='Codigo da Natureza da Receita para efeitos de Cofins', size=3),
        'estpro_fci': fields.char('FCI', size=36, help="Ficha de conteudo de importacao." ),
        # não pegar o estpro_cest_id diretamente da tabela, usar o método estpro_get_cest
        'estpro_cest_id': fields.many2one('fiscal.cest', 'Codigo CEST', ondelete='restrict', select=True, help='Codigo Especificador da Substituicao Tributaria'),
        'estpro_nao_utiliza_nat_rec': fields.boolean('Nao utiliza natureza da receita'),
        'estpro_unidade_tributavel_id': fields.many2one(
            'estoque.embalagem.unidade', 'Unidade tributavel', ondelete='restrict',
            help='Informar a unidade tributavel do item quando a tributacao incidente tem unidade '
                 'diferente da comercial (ex.: gas de cozinha (GLP)'),
        'estpro_fator_conversao_unidade_tributavel': fields.float(
            'Fator de conversao', digits=(10,6),
            help='Fator de conversao utilizado no calculo da quantidade tributada e valor '
                 'tributado do item.'),
        'estpro_med_codigo_anvisa': fields.char('Codigo anvisa', size=13, help="Codigo do produto na Anvisa."),
        'estpro_med_motivo_isencao': fields.text('Motivo de isencao'),  # Codigo de excessao do NCM conforme TIPI!
        'estpro_med_preco_maximo': fields.float('Preço máximo"', digits=(9, 2)),
        'estpro_nlote': fields.char('Numero do lote', size=20, help="Numero do lote de producao do produto"),
        'estpro_qlote': fields.float('Qtd de produtos por lote', digits=(8, 3)),
        'estpro_dfab': fields.date('Data de fabricacao', help='Data de fabricacao do lote'),
        'estpro_dval': fields.date('Data de validade', help='data de validade do lote'),
        'estpro_cagreg': fields.char('Codigo de Agregacao', size=20),
        'estpro_regime_especial': fields.char('Regime especial', size=20)
    }
    _defaults = {
        'origem_mercadoria': '0',
        'estpro_nao_utiliza_nat_rec': lambda *a: False,
    }
