# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class rel_registro_inventario_osv(osv.TransientModel):

    _inherit = 'rel.registro.inventario.osv'

    def get_ordem_listar(self, cr, uid, context=None):
        """
        Retorna lista com os tipos de ordenacao para listagem do inventario.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param context: um dicionario padrao para valores contextuais
        :return: lista com os tipos de precos a listar
        :rtype: list
        """
        context = context and context.copy() or dict()
        res = super(rel_registro_inventario_osv, self).get_ordem_listar(cr, uid, context)
        res.append(('T', 'Tributacao'))
        return res

    def set_ordem_listar(self, cr, uid, context=None):
        """
        Define o valor padrao da "orden a listar" para 'C', categoria.
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type context: dict
        :return: caracter 'C' que se refere a categoria.
        :rtype: str
        """
        context = context and context.copy() or dict()
        super(rel_registro_inventario_osv, self).set_ordem_listar(cr, uid, context)
        return 'T'

    _columns = {
        'encargo_id': fields.many2one('fiscal.encargo', 'Encargo'),
        'cst_id': fields.many2one('fiscal.cst', 'CST', domain="[('encargo_id', '=', encargo_id)]"),
        'tributacao_id': fields.many2one('fiscal.tributacao', 'Tributacao',
                                         domain="[('cst_id', '=', cst_id)]"),
        'ordem_listar': fields.selection(get_ordem_listar, 'Ordem a listar', size=1),
    }

    def estoque_inventario_get_dados(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = super(rel_registro_inventario_osv, self).estoque_inventario_get_dados(
            cr, uid, ids, context)
        inventario_contagem_dados = res.get('inventario_contagem_dados')
        produto_ids = list(set(map(lambda i: i.get('produto_id'), inventario_contagem_dados)))
        dict_tributacao_produto = dict()
        for tributacao in self.get_dados_tributacao_produto(cr, uid, produto_ids, context):
            dict_tributacao_produto[tributacao.get('produto_id')] = tributacao.get('tributacao_name')
        res['tributacao_produto_dados'] = dict_tributacao_produto
        res['ncm_produto_dados'] = self.get_dados_ncm_produto(cr, uid, produto_ids, context)
        return res

    def get_dados_tributacao_produto(self, cr, uid, produto_ids, context=None):
        context = context and context.copy() or {}
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))
        sql = """
            SELECT
                ep.id       AS produto_id,
                ft.name     AS tributacao_name
            FROM fiscal_tributacao_encargos fte
            INNER JOIN fiscal_tributacao ft
                ON ft.id = fte.tributacao_id
            INNER JOIN fiscal_encargo fe
                ON fe.id = fte.encargo_id
            INNER JOIN estoque_produto ep
                ON ep.id = fte.produto_id
            WHERE
                fte.tipo_tributacao = 'T'
                AND fe.sigla = 'ICMS'
                AND ep.id IN %s
        """ % produto_ids
        cr.execute(sql)
        tributacao_dados = cr.dictfetchall()
        return tributacao_dados

    def get_dados_ncm_produto(self, cr, uid, produto_ids, context=None):
        context = context and context.copy() or {}
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))
        sql = """
            SELECT
                ep.id       AS produto_id,
                ncm.codigo  AS ncm_codigo
            FROM estoque_produto ep
            INNER JOIN fiscal_ncm ncm
                ON ncm.id = ep.ncm_id
            WHERE
                ep.id IN %s
        """ % produto_ids
        cr.execute(sql)
        ncm_dados = cr.dictfetchall()
        res = dict()
        map(lambda i: res.update({i.get('produto_id'): i.get('ncm_codigo')}), ncm_dados)
        return res

    def get_dados_inventario_contagem(self, cr, uid, registro_inventario_id, inventario_id,
                                      local_id, context=None):
        context = context and context.copy()
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        args = list()
        brw_registro_inventario = self.browse(cr, uid, registro_inventario_id, context)
        dados = super(rel_registro_inventario_osv, self).get_dados_inventario_contagem(
            cr, uid, registro_inventario_id, inventario_id, local_id, context)
        if brw_registro_inventario.encargo_id:
            args.append(('encargo_id', '=', brw_registro_inventario.encargo_id.id))
        if brw_registro_inventario.cst_id:
            args.append(('cst_id', '=', brw_registro_inventario.cst_id.id))
        if brw_registro_inventario.tributacao_id:
            args.append(('tributacao_id', '=', brw_registro_inventario.tributacao_id.id))
        produto_ids = list()
        if args:
            args.append(('tipo_tributacao', '=', 'T'))
            tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(cr, uid, args,
                                                                           context=context)
            produto_ids = list(set(map(lambda p: p.produto_id.id,
                                       obj_fiscal_tributacao_encargos.browse(
                                           cr, uid, tributacao_encargo_ids, context))))
        if produto_ids:
            dados = filter(lambda p: p.get('produto_id') in produto_ids, dados)
        return dados
