# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _
import time

NAO_INCLUIR_NA_NOTA = '2'  # O valor do item eh zero e nao sera incluido na nota


class wizard_estoque_gerar_nf_perda(osv.TransientModel):
    _inherit = 'wizard.estoque.gerar.nf.perda'  # wizgnp

    def wignp_button_gerar_nf_perda(self, cr, user, ids, context=None):
        """
        Gera as notas fiscais de perda de inventario.
        
        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: long
        :type ids: list
        :type context: dict
        :return: verdadeiro se as notas de perda de inventario foram geradas
        :rtype: bool
        """
        context = context and context.copy() or dict()

        # Dados do formulario
        brw_nf_perda = self.browse(cr, user, ids[0], context)
        inventario_id = brw_nf_perda.wizgnp_inventario_id.id
        tipo_custo = brw_nf_perda.wizgnp_preco
        preco = brw_nf_perda.wizgnp_item_preco
        qtde = brw_nf_perda.wizgnp_qtde_itens

        # Verifica se jah foram geradas notas de perda
        if self.pool.get('estoque.inventario.capa').search(
                cr, user, [('id', '=', inventario_id), ('esticp_nf_gerada', '=', True)],
                context=context):
            raise osv.except_osv(_('Atencao!'),
                                 _('Ja foram geradas notas fiscais de perda para este inventario.'))

        self.wizgnp_gerar_nf_perda(cr, user, inventario_id, tipo_custo, preco, qtde, context)
        context['manutencao_inventario'] = True

        return self.pool.get('estoque.inventario.capa').write(
            cr, user, inventario_id, dict(esticp_nf_gerada=True), context)

    def _wizgnp_valida(self, cr, user, args, context=None):
        """
        Valida as informacoes antes de gerar a nota fiscal de perda de inventario.
        
        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param args: argumentos e valores utilizados na validacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: long
        :type args: dict
        :type context: dict
        :return: mensagem de erro se as informacoes para gerar a nota fiscal de perda de inventario 
        estiverem divergentes.
        :rtype: n/a
        """
        context = context and context.copy() or dict()

        # Verifica se o inventario jah tem NF gerada
        if self.pool.get('estoque.inventario.capa').search(
                cr, user, [('id', '=', args.get('inventario')), ('esticp_nf_gerada', '=', True)]):
            raise osv.except_osv(_('Atencao!'),
                                 _('Ja foi gerada nota fiscal para este inventario.'))

        fiscal_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'fiscal', ['fispar_natureza_nf_perda_id', 'fispar_condicao_pgto_nf_perda_id'],
            context)

        # Verifica se a natureza da operacao estah devidamente configurada no parametro do Fiscal
        if not fiscal_parametro.get('fispar_natureza_nf_perda_id'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A natureza a ser utilizada quando da emissao de nota fiscal '
                                   'atraves das perdas do inventario nao foi configurada no '
                                   'parametro do Fiscal.'))

        # Verifica se a condicao de pagamento para notas fiscais de perda estah devidamente
        # configurada no parametro do Fiscal
        if not fiscal_parametro.get('fispar_condicao_pgto_nf_perda_id'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A condicao de pagamento a ser utilizada quando da emissao de '
                                   'nota fiscal atraves das perdas do inventario nao foi '
                                   'configurada no parametro do Fiscal.'))

    def wizgnp_gerar_nf_perda(self, cr, user, inventario, tipo_custo, preco, qtde, context=None):
        """
        Gera as notas fiscais de perda de inventario.
        
        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param inventario: ID do inventario
        :param tipo_custo: tipo do custo para o item: PV=Preco de venda, UC=Ultimo custo, CM=Custo 
        medio contabil, e CR=Custo de reposicao
        :param preco: preco do item a considerar, 1=Considerar 0,01 ou 2=Nao incluir na nota fiscal
        :param qtde: quantidade maxima de itens por nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: Cursor
        :type user: long
        :type inventario: long
        :type context: dict
        :return: lista com o ID das notas fiscais geradas
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        context = dict(context, ctx_nf_propria=True, proprio=True, nota_fiscal_saida=True,
                       wizfni_nota_por_local=True)  # Cria tributacao dos itens
        nf_ids = list()  # Notas fiscais de perda

        # Verifica as informacoes
        self._wizgnp_valida(cr, user, dict(inventario=inventario), context)

        # Seleciona os itens que irao compor as notas fiscais
        itens = self._wizgnp_seleciona_itens(cr, user, inventario, tipo_custo, preco, context)

        # Parametro do fiscal
        fiscal_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'fiscal', ['fispar_natureza_nf_perda_id', 'fispar_condicao_pgto_nf_perda_id',
                                 'fispar_condicao_pgto_nf_perda_id'],
            context)

        # Natureza baseada no parametro do Fiscal
        brw_natureza = self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, fiscal_parametro.get('fispar_natureza_nf_perda_id'), context)

        # Condicao de pagamento baseada no parametro do Fiscal
        brw_condicao = self.pool.get('adm.condicao.forma').browse(
            cr, user, fiscal_parametro.get('fispar_condicao_pgto_nf_perda_id'), context)

        # Agrupa os itens na nota fiscal de acordo com a quantidade informada
        for grupo_itens in [itens[n:n + qtde] for n in xrange(0, len(itens), qtde)]:
            movimento_itens = list()  # Itens da nota fiscal
            total_geral_item = sequencia = 0
            for item, valor, embalagem_id in grupo_itens:
                sequencia += 1  # Sequencia do item na nota fiscal
                perda = abs(item.saldo_contado - item.saldo_cadastro)
                movimento_itens.append([0, False, dict(
                    produto_id=item.grade_id.produto_id.id, grade_id=item.grade_id.id,
                    local_id=item.local_id.id, quantidade=perda, vr_unitario=valor,
                    produto_embalagem_id=embalagem_id, vr_preco_venda=valor, sequencia=sequencia,
                    vr_total_item=valor * perda, vr_total_item_rateado=valor * perda,
                    cfop_id=brw_natureza.cfop_id.id, estmvi_permissao_altera_preco=True)])
                total_geral_item += valor * perda

            # Gera uma nota com a quantidade maxima de itens passada
            empresa = item.inventario_id.company_id
            condicao_pagamento = [[0, False, dict(condicao_forma_id=brw_condicao.adm_condicao_id.id,
                                                  forma_pagto_id=brw_condicao.forma_pagamento_id.id,
                                                  valor=total_geral_item)]]

            # Inclui as parcelas para a condicao de pagamento
            parcelas = [[0, False, p] for p in self.pool.get('adm.condicao').gerar_parcelas(
                cr, user, brw_condicao.adm_condicao_id.id, condicao_pagamento[0][2].get('valor'),
                time.strftime('%Y-%m-%d'), context)]
            condicao_pagamento[0][2]['nf_parcela_ids'] = parcelas

            nf_ids.append(self.pool.get('fiscal.utils').gerar_nf(
                cr, user, item.inventario_id.company_id.id,
                item.inventario_id.company_id.rescon_pessoa_id.id, natureza_id=brw_natureza.id,
                estoque_movimento_itens_ids=movimento_itens, nf_condicao_ids=condicao_pagamento,
                destinatario_endereco_id=empresa.rescon_pessoa_id.endereco_nf_id.id,
                context=context))

        return nf_ids

    def _wizgnp_seleciona_itens(self, cr, user, inventario, tipo_custo, preco, context=None):
        """
        Apenas os itens que farao parte da nota fiscal de perda de inventario.
        
        :param cr: 
        :type cr: 
        :param user: 
        :type user: 
        :param inventario: 
        :type inventario: 
        :param context: 
        :type context: 
        :return: lista com os itens que satisfazem as condicoes e seu respectivo valor e embalagem
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        result = list()

        # Itens do inventario com diferenca
        contagem_ids = self.pool.get('estoque.inventario.contagem').search(
            cr, user, [('inventario_id', '=', inventario)], context=context)
        itens = [(i.get('id'), i.get('produto_id')) for i in filter(
            lambda f: f.get('saldo_contado') < f.get('saldo_cadastro'),
            self.pool.get('estoque.inventario.contagem').read(
                cr, user, contagem_ids, ['saldo_contado', 'saldo_cadastro', 'produto_id'],
                context, '_classic_write'))]
        item_ids = [i[0] for i in itens]
        produto_ids = [i[1] for i in itens]

        # Itens que satisfazem as condicoes de preco
        if tipo_custo == 'PV':
            precos = self.pool.get('estoque.preco.venda').get_preco_venda(cr, user, produto_ids)
            get_preco = 'preco_venda'
        else:
            precos = self.pool.get('estoque.preco.custo').get_custos_do_dia(cr, user, produto_ids)
            get_preco = dict(UC='estprc_ultimo_custo', CC='estprc_custo_corrigido',
                             MC='estprc_custo_medio_contabil', CR='estprc_custo_reposicao')
            get_preco = get_preco.get(tipo_custo)

        for item in self.pool.get('estoque.inventario.contagem').browse(
                cr, user, item_ids, context):
            embalagem = filter(lambda f: f.local_id.id == item.local_id.id,
                               item.grade_id.local_locacao_ids)[0].estllc_produto_embalagem_id
            qtde_na_embalagem = embalagem.embalagem_id.quantidade_na_embalagem or 1.
            produto_preco = precos.get(item.grade_id.produto_id.id) or dict()

            # Valor unitario em funcao do custo
            if tipo_custo == 'MC':
                vr_unitario = item.estict_custo_medio_contabil
            elif tipo_custo == 'UC':
                    vr_unitario = item.estict_ultimo_custo
            else:
                vr_unitario = produto_preco.get(get_preco) or 0 * qtde_na_embalagem

            if not vr_unitario:
                if preco == NAO_INCLUIR_NA_NOTA:
                    continue  # Desconsidera o item
                else:
                    vr_unitario = .01  # Considera um centavo como valor do item

            result.append([item, vr_unitario, embalagem.id])

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
