# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""


# Funcoes da classe estoquepedidos_pedido_item
# noinspection PyUnusedLocal,PyIncorrectDocstring
def pedite_function_get_saldo(self, cr, user, ids, field_name, arg, context=None):
    """
    Recupera o saldo do item no local em funcao da grade.

    :param cr: o cursor do banco de dados
    :param user: o ID do usuario utilizado para a verificacao de perfil
    :param ids: lista com o ID dos itens de pedido
    :param field_name: o nome ou os nomes dos atributos (campos)
    :param arg: uma lista de argumentos utilizada na consulta
    :param context: um dicionario padrao para valores contextuais,
    :type cr: object
    :type user: int
    :type arg: list
    :type context: dict
    :return: dicionario com os valores de saldo de um item.
    :rtype: dict

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    context = context and context.copy() or {}
    result = dict().fromkeys(ids if isinstance(ids, list) else [ids], list())
    brw_item_pedido = self.browse(cr, user, ids[0] if isinstance(ids, list) else ids, context)
    result[brw_item_pedido.id] = self.pool.get('estoque.local.locacao').search(
            cr, user, [('grade_id', '=', brw_item_pedido.grade_id.id)], context=context)

    return result


# noinspection PyIncorrectDocstring
def pedite_function_get_ultima_compra(self, cr, user, ids, field_name, arg, context=None):
    """
    Recupera os valores da ultima compra efetuada para um item.

    :param cr: o cursor do banco de dados
    :param user: o ID do usuario utilizado para a verificacao de perfil
    :param ids: lista com o ID dos itens de pedido
    :param field_name: o nome ou os nomes dos atributos (campos)
    :param arg: uma lista de argumentos utilizada na consulta
    :param context: um dicionario padrao para valores contextuais,
    :type cr: object
    :type user: int
    :type arg: list
    :type context: dict
    :return: dicionario com os valores da ultima compra efetuada para um item.
    :rtype: dict

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    context = context and context.copy() or {}
    brw_item_pedido = self.browse(cr, user, ids if isinstance(ids, list) else [ids], context)
    grade_ids = map(lambda g: g.grade_id.id, brw_item_pedido)
    if not grade_ids:
        return dict().fromkeys(ids if isinstance(ids, list) else [ids], list())
    obj_compras_ultima_compra = self.pool.get('compras.ultima.compra')
    ultima_compra_ids = obj_compras_ultima_compra.search(
        cr, user, [('grade_id', 'in', grade_ids)], context=context)
    if not ultima_compra_ids:
        self.pedite_get_ultima_compra(cr, user, grade_ids, context)
    result = dict()
    for i in ids:
        result.update({i: self.pool.get('compras.ultima.compra').search(
            cr, user, [('grade_id', 'in', grade_ids)], context=context)})

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
