# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.tools import config as openerp_config
from ...control.utils import log_debug
from openerp.osv import osv
import logging

class estoquepedidos_pedido_capa(osv.Model):
    _inherit = 'estoquepedidos.pedido.capa'
    _logger = logging.getLogger(_inherit)
    _logger.level = logging._levelNames[openerp_config.get('log_level').upper()]

    @log_debug
    def pedcap_button_faturar_pedido_nfce(self, cr, user, ids, context=None):
        """
        Fatura o pedido de venda e gera a nota fiscal de consumidor eletronica.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do pedido
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se o pedido for faturado.
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['faturar_pedido_nfce'] = True
        context['nota_fiscal_consumidor'] = True
        dados_cliente = self.read(
            cr, user, ids[0], ['pessoa_id'], context, '_classic_write')
        if isinstance(dados_cliente, list):
            pessoa_id = dados_cliente[0].get('pessoa_id')
        else:
            pessoa_id = dados_cliente.get('pessoa_id')

        # Fatura o pedido e gera a NFC-e
        result = super(estoquepedidos_pedido_capa, self).button_faturar_pedido(
            cr, user, ids, context, pessoa_id)

        # Fatura a NFC-e
        if result:
            self._pedcap_faturar_nfce(cr, user, ids[0], context)

        return result

    @log_debug
    def _pedcap_faturar_nfce(self, cr, user, pedido_id, context=None):
        """
        Fatura a NFC-e gerada pelo pedido.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param pedido_id: o ID do pedido
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type pedido_id: long
        :type context: dict
        :return: dados da NFC-e
        :rtype: dict
        """
        context = context and context.copy() or {}
        nfce_dados = self.read(cr, user, pedido_id, ['pedcap_nf_ids'], context)
        nfce_dados = nfce_dados and nfce_dados[0]  # Necessario, pois sempre retorna uma lista :/
        values = {}
        for nfce_id in nfce_dados.get('pedcap_nf_ids') or []:
            values = self.pool.get('fiscal.nf.capa').fisnfc_funcao_button_faturar(
                cr, user, [nfce_id], context)

        return values

    def pedcap_cancela_nf_pedido_validado(self, cr, uid, ids, context=None):
        """
        Método chamado durante o processo de cancelamento da NF de venda. Caso o parâmetro do
        estoque 'estpar_cancel_nf_pedido_validado' esteja selecionado, os pedidos retornam ao
        estado de 'Validado'. São excluídas possíveis previsões de títulos para os pedidos como
        também os desvincula da NF.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuário
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        for pedido in self.read(cr, uid, ids, ['numero_pedido', 'data_emissao'], context):
            # Exclui previsoes
            titulo_ids = obj_financeiro_titulo.search(
                cr, uid, [('documento', '=', pedido.get('numero_pedido')),
                          ('data_emissao', '=', pedido.get('data_emissao')),
                          ('doc_tipo_id.sigla', '=', 'PRV')], context=context)
            titulo_ids and obj_financeiro_titulo.unlink(cr, uid, titulo_ids)

            self.button_validar(cr, uid, [pedido.get('id')], context)
        sql = """
            DELETE FROM fiscal_nf_pedidos WHERE pedido_id IN ({pedido_ids}) 
        """.format(pedido_ids=', '.join(map(lambda p: str(p), ids)))
        cr.execute(sql)
        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
