# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from datetime import date
from openerp.report import report_sxw
import subprocess


class estoquepedidos_pedido_capa(osv.Model):
    _name = 'estoquepedidos.pedido.capa'
    _inherit = 'estoquepedidos.pedido.capa'

    def abrir_nf_pedido(self, cr, uid, dados_pedidos, context=None):
        context = context and context.copy() or {}
        if isinstance(dados_pedidos[0], int):
            dados_pedidos = self.read(cr, uid, dados_pedidos[0], ["tipo", "pedcap_nf_ids"], context)
        pedido_tipo = dados_pedidos[0].get('tipo')
        nf_id = dados_pedidos[0].get('pedcap_nf_ids')[0] if dados_pedidos[0].get('pedcap_nf_ids') \
            else context.get('nf_id')
        if not nf_id:
            raise osv.except_osv(_('Atencao'), _('NF nao gerada'))

        nf_view = 'view_fiscal_nf_capa_form'
        if context.get('compra_nf_view_terceiro'):
            nf_view = 'view_fiscal_nf_capa_terceiro_form'

        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(cr, uid, 'fiscal', nf_view)
        view_id = reference and reference[1] or False

        ctx = False
        if pedido_tipo == 'V':
            ctx = True

        context.update({'ctx_serie_nf_saida': ctx, 'active_id': nf_id})
        action = {
            'name': _('Nota Fiscal Gerada'),
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': view_id,
            'res_model': 'fiscal.nf.capa',
            'res_id': nf_id,
            'type': 'ir.actions.act_window',
            'context': context,
        }

        return action

    def faturar_pedido_compra(self, cr, uid, nf_id, pedido_ids, context=None):
        """
        Metodo responsavel por alterar os status do pedido de compra para faturado quando
        as quantidades dos itens do pedido forem atendidas
        :param cr: Cursor do openerp
        :param uid: Usuario do openerp
        :param nf_id: ID da nota fiscal
        :param pedido_ids: Browse contendo os pedidos a serem verificados
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type nf_id: int | long
        :type pedido_ids: Browse
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        self.atualiza_quantidade_atendida_pedido_compra(cr, uid, nf_id, context=context)
        for pedido in pedido_ids:
            if not pedido.pedcap_divergencia_qtd:
                self.write(cr, uid, pedido.id, {'state': 'F'})

    def pedcap_faturar_varios_pedidos(self, cr, uid, ids, context=None):
        """
        Método utilizado para faturar varios pedidos de uma vez
        Só fatura pedidos que estão validados
        @param cr:
        @param uid:
        @param ids: Ids dos pedidos a faturar
        @param context:
        @return:
        """
        context = context and context.copy() or {}
        _erros = []
        for brw_pedido in self.browse(cr, uid, ids, context=context):
            if brw_pedido.state != 'V': # só considerar pedidos validados
                continue
            if brw_pedido.tipo != 'V': # só considerar pedidos de venda
                continue
            try:
                res = self.button_faturar_pedido(cr, uid, [brw_pedido.id], context=context)
            except Exception as e:
                if e.value:
                    _erros.append([brw_pedido.numero_pedido, e.value])
                    res = False
            if res:
                cr.execute(''' COMMIT; ''')
            else:
                cr.execute(''' ROLLBACK; ''')
        if _erros:
            _mensagens = ''
            for _erro in _erros:
                _mensagens += '%s %s\n' % (_erro[0], _erro[1])
            raise osv.except_osv('Foram encontrados os seguintes erros:', _mensagens)
        return True


    def atualiza_quantidade_atendida_pedido_compra(self, cr, uid, nf_id, context=None):
        """
        Este metodo sera responsavel por atualizar a quantidade atendida de acordo com a
        movimentacao dos itens da nota fiscal
        :param cr: Cursor do openerp
        :param uid: Usuario do openerp
        :param nf_id: ID da nota fiscal
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type nf_id: int | long
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        brw_nf = self.pool.get('fiscal.nf.capa').browse(cr, uid, nf_id, context=context)
        for item in brw_nf.estoque_movimento_itens_ids:
            quantidade = item.quantidade
            for pedido in brw_nf.fisnfc_pedido_ids:
                for item_pedido in filter(lambda i: i.grade_id == item.grade_id,
                                          pedido.pedido_item_ids):
                    qtd_necessaria = item_pedido.qtde_pedida - item_pedido.qtde_atendida
                    qtde_atendida = item_pedido.qtde_atendida
                    if qtd_necessaria > quantidade:
                        qtde_atendida += quantidade
                        quantidade = 0
                    else:
                        qtde_atendida += qtd_necessaria
                        quantidade -= qtd_necessaria
                    self.pool.get('estoquepedidos.pedido.item').write(
                        cr, uid, item_pedido.id, {'qtde_atendida': qtde_atendida},
                        context=context)
                    if quantidade == 0:
                        continue

    def pedcap_get_dado_extra(self, cr, uid, ids, context=None):
        """
        O metodo encontra os dados que se encontram nesse modulo, necessarios para a impressao
        dos pedidos.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario. Cada chave representa um id de um pedido. Cada valor dessa chave e um
        dicionario com os campos e seus respectivos valores.
        :rtype: dict
        """
        if not isinstance(ids, list):
            ids = [ids]
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).pedcap_get_dado_extra(cr, uid, ids, context)
        brw_pedido_capa = self.browse(cr, uid, ids, context)
        for pedido in brw_pedido_capa:
            if not pedido.pessoa_id or not pedido.pessoa_id.endereco_nf_id:
                continue
            endereco_nf = pedido.pessoa_id.endereco_nf_id
            vals = {
                'endereco_nf': '%s, %s' % (endereco_nf.logradouro_id.name,
                                          (endereco_nf.numero or '')),
                'bairro_nf': endereco_nf.bairro_id.name,
                'cidade_nf': endereco_nf.cidade_id.name,
                'uf_nf': endereco_nf.uf_id.sigla,
                'cep_nf': endereco_nf.cep
            }
            if pedido.id in res.keys():
                res[pedido.id].update(vals)
            else:
                res[pedido.id] = vals
        return res
