# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _


class financeiro_cliente(osv.Model):

    _inherit = 'financeiro.cliente'

    def onchange_endereco_nf(self, cr, uid, ids, endereco_nf_id, context=None):
        """
        Verifica se o endereco selecionado e um endereco válido para Nota Fiscal

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param endereco_nf_id: {int} ID do endereco selecionado.
        @return: endereco selecionado ou mensagem de erro da verificacao do en-
        dereco.
        """
        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pessoa_endereco = self.pool.get('adm.pessoa.endereco')

        if endereco_nf_id:
            msg = obj_pessoa_endereco.valida_endereco_nf(cr, uid, endereco_nf_id, context=context)

            if msg:
                return {
                    'value': {'endereco_nf_id': False},
                    'warning': {'title': _('Atencao'), 'message': msg}
                }

        return {'value': {'endereco_nf_id': endereco_nf_id}}

    def fincli_get_compra_dados(self, cr, uid, pessoa_id, context=None):
        """
        Metodo realiza busca de estatisticas no faturamento de notas fiscais a respeito de compras
        realizadas pelos clientes.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param pessoa_id: id da tabela adm_pessoa, identificador da pessoa.
        :type pessoa_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: dados com valores e datas das compras realizadas.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = dict()
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        sql = '''
              SELECT nf.data_emissao,
                     nf.vr_total_nf
              FROM fiscal_nf_capa nf
              LEFT JOIN fiscal_cfop_natureza fcn
                  ON fcn.id = nf.natureza_id
              LEFT JOIN fiscal_cfop fc
                  ON fcn.cfop_id = fc.id
              LEFT JOIN estoque_natureza_operacao eno
                  ON eno.id = fcn.fisnat_operacao_id
              WHERE nf.destinatario_id = %s
                AND nf.tipo = 'S'
                AND nf.state = '3'
                AND nf.company_id = %s
                AND fcn.tipo = 'S'
                AND fc.movimento_tipo = 'C'
                AND fc.tipo = 'S'
                AND eno.estnto_tipo_operacao = 'S'
                AND eno.estnto_gerar_financeiro IS TRUE
              ORDER BY nf.data_emissao
              ''' % (pessoa_id, company_id)
        cr.execute(sql)
        dados_nf = cr.dictfetchall()
        if not dados_nf:
            return res
        res.update({'primeira_compra_data': dados_nf[0].get('data_emissao'),
                    'primeira_compra_valor': dados_nf[0].get('vr_total_nf'),
                    'ultima_compra_data': dados_nf[-1].get('data_emissao'),
                    'ultima_compra_valor': dados_nf[-1].get('vr_total_nf')})

        dados_nf = sorted(dados_nf, key=lambda nf: nf.get('vr_total_nf'), reverse=True)
        media_compra_valor = round(
            sum(map(lambda nf: nf.get('vr_total_nf') or 0, dados_nf)), 2) / len(dados_nf)
        res.update({'maior_compra_data': dados_nf[0].get('data_emissao'),
                    'maior_compra_valor': dados_nf[0].get('vr_total_nf'),
                    'media_compra_valor': media_compra_valor})
        return res
