# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class wizard_gerencial_listagem_conferencia(osv.TransientModel):
    _name = 'wizard.gerencial.listagem.conferencia'
    _columns = {
        'wgecon_data_movimento_i': fields.date('Data inicial do movimento', required=True),
        'wgecon_data_movimento_f': fields.date('Data final do movimento', required=True),
    }

    def wgecon_button_print(self, cr, uid, ids, context=None):
        """
        Metodo de acao do botao 'Imprimir'
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: dados de acao para chamada da classe responsavel pela montagem e impressao do
        relatorio
        :rtype: dict
        """
        context = context and context.copy() or {}
        dados_wiz = self.read(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                   ['wgecon_data_movimento_i', 'wgecon_data_movimento_f'], context)
        data_movimento_i = dados_wiz.get('wgecon_data_movimento_i')
        data_movimento_f = dados_wiz.get('wgecon_data_movimento_f')
        dados = self.wgecon_get_dados(cr, uid, data_movimento_i, data_movimento_f, context)
        dados.update({'data_movimento_i': data_movimento_i, 'data_movimento_f': data_movimento_f})
        return {
            'type': 'ir.actions.report.xml',
            'datas': dados,
            'report_name': 'gerencial_listagem_conferencia',
            'name': 'listagem_para_conferencia',
        }

    def wgecon_get_dados(self, cr, uid, data_movimento_i, data_movimento_f, context=None):
        """
        O metodo reune todos os dados necessarios de nota fiscal, titulo, tesouraria e custificacao
        para serem processados pela classe do relatorio.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param data_movimento: dia do movimento a pesquisar
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :return: dicionario com identificao dos tipos de dados e seus respectivos valores
        :rtype: dict
        """
        context = context and context.copy() or {}
        dados = {'nf': {}, 'titulo': {}, 'tesouraria': {}, 'gerencial': {}}

        # Baseados nas NFs
        nf_dados = self.wgecon_get_dados_nota_fiscal(cr, uid, data_movimento_i, data_movimento_f, context)
        nf_ids = map(lambda nf: nf.get('nf_id'), nf_dados)
        nf_dados_gerencial = self.wgecon_get_dados_nota_fiscal_gerencial(cr, uid, nf_ids, context)
        nf_dados_titulo = self.wgecon_get_dados_nota_fiscal_titulo(cr, uid, nf_ids, context)
        nf_titulo_ids = map(lambda nf: nf.get('titulo_id'), nf_dados_titulo)
        nf_dados_titulo_gerencial = self.wgecon_get_dados_titulo_gerencial(cr, uid, nf_titulo_ids,
                                                                           context)
        nf_dados_titulo_tesouraria = self.wgecon_get_dados_titulo_tesouraria(cr, uid, nf_titulo_ids,
                                                                             context)
        nf_tesouraria_movimento_ids = map(lambda tt: tt.get('tesouraria_id'),
                                          nf_dados_titulo_tesouraria)
        nf_dados_tesouraria_gerencial = self.wgecon_get_dados_tesouraria_gerencial(
            cr, uid, nf_tesouraria_movimento_ids, context)
        nf_values = {'nivel1': nf_dados,
                     'nivel2_gerencial': nf_dados_gerencial,
                     'nivel2_titulo': nf_dados_titulo,
                     'nivel3_gerencial': nf_dados_titulo_gerencial,
                     'nivel3_tesouraria': nf_dados_titulo_tesouraria,
                     'nivel4': nf_dados_tesouraria_gerencial}
        dados['nf'].update(nf_values)

        # Baseados nos titulos sem NFs
        titulo_dados = self.wgecon_get_dados_titulo(cr, uid, data_movimento_i, data_movimento_f, context)
        titulo_ids = map(lambda t: t.get('titulo_id'), titulo_dados)
        titulo_dados_gerencial = self.wgecon_get_dados_titulo_gerencial(cr, uid, titulo_ids,
                                                                        context)
        titulo_dados_tesouraria = self.wgecon_get_dados_titulo_tesouraria(cr, uid, titulo_ids,
                                                                          context)
        titulo_tesouraria_movimento_ids = map(lambda tt: tt.get('tesouraria_id'),
                                              titulo_dados_tesouraria)
        titulo_dados_tesouraria_gerencial = self.wgecon_get_dados_tesouraria_gerencial(
            cr, uid, titulo_tesouraria_movimento_ids, context)
        titulo_values = {'nivel1': titulo_dados,
                         'nivel2_gerencial': titulo_dados_gerencial,
                         'nivel2_tesouraria': titulo_dados_tesouraria,
                         'nivel3_gerencial': titulo_dados_tesouraria_gerencial}
        dados['titulo'].update(titulo_values)

        # Baseados no movimento da tesouraria sem titulos
        tesouraria_movimento_dados = self.wgecon_get_dados_tesouraria(cr, uid, data_movimento_i, data_movimento_f,
                                                                      context)
        tesouraria_movimento_ids = map(lambda ft: ft.get('tesouraria_id'),
                                       tesouraria_movimento_dados)
        tesouraria_movimento_dados_gerencial = self.wgecon_get_dados_tesouraria_gerencial(
            cr, uid, tesouraria_movimento_ids, context)
        tesouraria_values = {
            'nivel1': tesouraria_movimento_dados,
            'nivel2': tesouraria_movimento_dados_gerencial
        }
        dados['tesouraria'].update(tesouraria_values)

        # Baseados no movimento gerencial sem NF, titulo e movimento da tesouraria
        gerencial_dados = self.wgecon_get_dados_gerencial(cr, uid, data_movimento_i, data_movimento_f, context)
        gerencial_values = {'nivel1': gerencial_dados}
        dados['gerencial'].update(gerencial_values)

        return dados

    def wgecon_get_dados_nota_fiscal(self, cr, uid, data_movimento_i, data_movimento_f, context=None):
        """
        Reune os dados das notas fiscais faturadas na data recebida (data_movimento)
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param data_movimento: dia do movimento a pesquisar
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados das notas fiscais
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'fiscal.nf.capa',
                                                                    context=context)
        query_nota_fiscal = """
            SELECT
                fnc.id AS nf_id,
                fnc.numero_nf AS numero, 
                fnc.tipo,
                fmn.fismnf_codigo AS modelo,
                fnc.serie_nf AS serie,
                ap.codigo::varchar || '-' || ap.nome AS cliente,
                CASE
                    WHEN fnc.tipo = 'E' AND NOT fnc.nf_propria
                        THEN fnc.data_saida_entrada
                    ELSE
                        fnc.data_emissao
                END AS data_emissao,
                fnc.vr_total_nf AS valor
            FROM
                fiscal_nf_capa fnc
            LEFT JOIN
                adm_pessoa ap ON ap.id = fnc.destinatario_id
            LEFT JOIN
                fiscal_modelo_nf fmn ON fmn.id = fnc.modelo_nf_id
            WHERE
                CASE
                    WHEN fnc.tipo = 'E' AND NOT fnc.nf_propria
                        THEN fnc.data_saida_entrada BETWEEN '{data_movimento_i} 00:00:00' 
                            AND '{data_movimento_f} 23:59:59'
                    ELSE
                        fnc.data_emissao BETWEEN '{data_movimento_i} 00:00:00' 
                            AND '{data_movimento_f} 23:59:59'
                END
                AND fnc.company_id = {company_id}
                AND fnc.state = '3'
            ORDER BY
                fnc.tipo, fnc.serie_nf, fnc.numero_nf
        """.format(data_movimento_i=data_movimento_i, data_movimento_f=data_movimento_f, company_id=company_id)
        cr.execute(query_nota_fiscal)
        dados_nf = cr.dictfetchall()
        return dados_nf

    def wgecon_get_dados_nota_fiscal_gerencial(self, cr, uid, nf_ids, context=None):
        """
        Reune os dados das custificacoes (gerencial_movimento) referente as notas fiscais
        recebidas (nf_ids)
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param nf_ids: lista de id da tabela fiscal_nf_capa
        :type nf_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados das custificacoes
        :rtype: list
        """
        context = context and context.copy() or {}
        if not nf_ids:
            return []
        query_nf_gerencial = """
            SELECT
                aec.codigo || '-' || aec.name AS centro_custo,
                ge.codigo || '-' || ge.name AS evento,
                gm.valor,
                gm.fingmo_historico AS historico,
                fnc.id AS nf_id
            FROM
                gerencial_movimento gm
            INNER JOIN
                adm_empresa_centrocusto aec ON aec.id = gm.centro_custo_id
            INNER JOIN
                gerencial_evento ge ON ge.id = gm.evento_id
            INNER JOIN
                fiscal_nf_capa fnc ON fnc.id = gm.fingmo_nf_id
            WHERE
                fnc.id IN ({nf_ids})
            order by aec.codigo, ge.codigo
        """.format(nf_ids=', '.join(map(lambda nf: str(nf), nf_ids)))
        cr.execute(query_nf_gerencial)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_nota_fiscal_titulo(self, cr, uid, nf_ids, context=None):
        """
        Reune os dados dos titulos (financeiro_titulo) referente as notas fiscais
        recebidas (nf_ids)
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param nf_ids: lista de id da tabela fiscal_nf_capa
        :type nf_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados dos titulos
        :rtype: list
        """
        context = context and context.copy() or {}
        if not nf_ids:
            return []
        query_nf_titulo = """
            SELECT
                ft.id AS titulo_id,
                ft.tipo_titulo AS tipo,
                ft.documento,
                ft.parcela,
                ft.data_prorrogacao AS vencimento,
                ft.vr_original AS valor,
                fnc.id AS nf_id
            FROM
                financeiro_titulo ft
            INNER JOIN
                financeiro_fatura ff ON ff.id = ft.fatura_id
            INNER JOIN
                fiscal_nf_capa fnc ON fnc.id = ff.nf_id
            WHERE
                fnc.id IN ({nf_ids})
        """.format(nf_ids=', '.join(map(lambda nf: str(nf), nf_ids)))
        cr.execute(query_nf_titulo)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_titulo_gerencial(self, cr, uid, titulo_ids, context=None):
        """
        Reune os dados das custificacoes (gerencial_movimento) referente aos titulos
        recebidos (titulo_ids)
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param titulo_ids: lista de id da tabela financeiro_titulo
        :type titulo_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados das custificacoes
        :rtype: list
        """
        context = context and context.copy() or {}
        if not titulo_ids:
            return []
        query_titulo_gerencial = """
            SELECT
                aec.codigo || '-' || aec.name AS centro_custo,
                ge.codigo || '-' || ge.name AS evento,
                gm.valor,
                gm.fingmo_historico AS historico,
                ft.id AS titulo_id
            FROM
                gerencial_movimento gm
            INNER JOIN
                adm_empresa_centrocusto aec ON aec.id = gm.centro_custo_id
            INNER JOIN
                gerencial_evento ge ON ge.id = gm.evento_id
            INNER JOIN
                financeiro_titulo ft ON ft.id = gm.titulo_id
            WHERE
                ft.id IN ({titulo_ids})    
        """.format(titulo_ids=', '.join(map(lambda ft: str(ft), titulo_ids)))
        cr.execute(query_titulo_gerencial)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_titulo_tesouraria(self, cr, uid, titulo_ids, context=None):
        """
        Reune os dados da tesouraria (financeiro_tesouraria_movimento) referente aos titulos
        recebidos (titulo_ids)
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param titulo_ids: lista de id da tabela financeiro_titulo
        :type titulo_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados da tesouraria
        :rtype: list
        """
        context = context and context.copy() or {}
        if not titulo_ids:
            return []
        query_titulo_tesouraria = """
            SELECT
                ftm.id AS tesouraria_id,
                ftm.debito_credito,
                ftc.apelido AS conta,
                ftm.valor,
                ftcd.apelido AS conta_destino,
                ftm.historico,
                ft.id AS titulo_id
            FROM
                financeiro_tesouraria_movimento ftm
            INNER JOIN
                financeiro_tesouraria_contas ftc ON ftc.id = ftm.conta_id
            LEFT JOIN
                financeiro_tesouraria_contas ftcd ON ftcd.id = ftm.conta_destino_id
            INNER JOIN
                financeiro_m2m_titulo_tesouraria tt ON tt.id = ftm.id
            INNER JOIN
                financeiro_titulo ft ON ft.id = tt.titulo_id
            WHERE
                ft.id IN ({titulo_ids})
        """.format(titulo_ids=', '.join(map(lambda ft: str(ft), titulo_ids)))
        cr.execute(query_titulo_tesouraria)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_tesouraria_gerencial(self, cr, uid, tesouraria_movimento_ids,
                                              context=None):
        """
        Reune os dados das custificacoes (gerencial_movimento) referente aos movimentos da
        tesouraria recebidos (tesouraria_movimento_ids)
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param tesouraria_movimento_ids: lista de id da tabela financeiro_tesouraria_movimento
        :type tesouraria_movimento_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados das custificacoes
        :rtype: list
        """
        context = context and context.copy() or {}
        if not tesouraria_movimento_ids:
            return []
        query_tesouraria_gerencial = """
            SELECT
                aec.codigo || '-' || aec.name AS centro_custo,
                ge.codigo || '-' || ge.name AS evento,
                gm.valor,
                gm.fingmo_historico AS historico,
                ftm.id AS tesouraria_id
            FROM
                gerencial_movimento gm
            INNER JOIN
                adm_empresa_centrocusto aec ON aec.id = gm.centro_custo_id
            INNER JOIN
                gerencial_evento ge ON ge.id = gm.evento_id
            INNER JOIN
                financeiro_tesouraria_movimento ftm ON ftm.id = gm.tesouraria_movimento_id
            WHERE
                ftm.id IN ({tesouraria_movimento_ids})
        """.format(tesouraria_movimento_ids=', '.join(map(lambda tm: str(tm),
                                                          tesouraria_movimento_ids)))
        cr.execute(query_tesouraria_gerencial)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_titulo(self, cr, uid, data_movimento_i, data_movimento_f, context=None):
        """
        Reune os dados dos titulos (financeiro_titulo) emitidos na data recebida (data_movimento).
        Esses nao estao relacionados a notas fiscais
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param data_movimento: dia de pesquisa da emissao do titulo
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados dos titulos
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'financeiro.titulo',
                                                                    context=context)
        query_titulo = """
            SELECT
                ft.id AS titulo_id,
                ft.tipo_titulo AS tipo,
                ft.documento,
                ft.parcela,
                ft.data_prorrogacao AS vencimento,
                ft.vr_original AS valor
            FROM
                financeiro_titulo ft
            LEFT JOIN
                financeiro_fatura ff ON ff.id = ft.fatura_id
            LEFT JOIN 
                documento_documento_tipo ddt on ddt.id = ft.doc_tipo_id
            WHERE
                ft.data_emissao BETWEEN '{data_movimento_i}' AND '{data_movimento_f}' 
                AND ft.company_id = {company_id}
                AND ff.nf_id IS NULL
                AND ddt.sigla <> 'PRV'
        """.format(data_movimento_i=data_movimento_i, data_movimento_f=data_movimento_f, company_id=company_id)
        cr.execute(query_titulo)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_tesouraria(self, cr, uid, data_movimento_i, data_movimento_f, context=None):
        """
        Reune os dados da tesouraria (financeiro_tesouraria_movimento) referente a data de
        movimento recebida (data_movimento). Esses movimentos nao estao relacionados a titulos.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param data_movimento: dia de movimento escolhido
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados da tesouraria
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'financeiro.tesouraria.contas', context=context)
        query_tesouraria = """
            SELECT
                ftm.id AS tesouraria_id,
                ftm.debito_credito,
                ftc.apelido AS conta,
                ftm.valor,
                ftcd.apelido AS conta_destino,
                ftm.historico
            FROM
                financeiro_tesouraria_movimento ftm
            INNER JOIN
                financeiro_tesouraria_contas ftc ON ftc.id = ftm.conta_id
            LEFT JOIN
                financeiro_tesouraria_contas ftcd ON ftcd.id = ftm.conta_destino_id
            WHERE
                ftc.company_id = {company_id}
                AND ftm.data_movimento BETWEEN '{data_movimento_i}' AND '{data_movimento_f}'
                AND ftm.id NOT IN (
                    SELECT tt1.id 
                    FROM financeiro_m2m_titulo_tesouraria tt1
                    INNER JOIN financeiro_tesouraria_movimento ftm1 ON tt1.id = ftm1.id
                    WHERE ftm1.data_movimento BETWEEN '{data_movimento_i}' AND '{data_movimento_f}'
                    )
            order by ftm.valor, ftm.id        
        """.format(data_movimento_i=data_movimento_i, data_movimento_f=data_movimento_f, company_id=company_id)
        cr.execute(query_tesouraria)
        dados = cr.dictfetchall()
        return dados

    def wgecon_get_dados_gerencial(self, cr, uid, data_movimento_i, data_movimento_f, context=None):
        """
        Reune os dados das custificacoes (gerencial_movimento) referente a data de movimento
        recebida (data_movimento). As custificacoes nao estao relacionadas a nota fiscal, titulo e
        movimento da tesouraria
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param data_movimento: dia de movimento escolhido
        :type data_movimento: str
        :param context: dados contextuais
        :type context: dict
        :return: lista dos dados das custificacoes
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'gerencial.movimento',
                                                                    context=context)
        requisicao = ''
        join_almoxarifado = ''
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'estoquealmoxarifado',
                                                                    context):
            requisicao = """, erc.numero AS requisicao"""
            join_almoxarifado = """
            LEFT JOIN
                almoxarifado_requisicao_capa erc ON erc.id = gm.fingmo_requisicao_id
            """
        query_gerencial = """
            SELECT
                aec.codigo || '-' || aec.name AS centro_custo,
                ge.codigo || '-' || ge.name AS evento,
                gm.valor,
                gm.fingmo_historico AS historico
                {requisicao}
            FROM
                gerencial_movimento gm
            INNER JOIN
                adm_empresa_centrocusto aec ON aec.id = gm.centro_custo_id
            INNER JOIN
                gerencial_evento ge ON ge.id = gm.evento_id
            {join_almoxarifado}
            WHERE
                gm.fingmo_data BETWEEN '{data_movimento_i}' AND '{data_movimento_f}' 
                AND gm.company_id = {company_id}
                AND gm.tesouraria_movimento_id IS NULL
                AND gm.titulo_id IS NULL
                AND gm.fingmo_nf_id IS NULL
        """.format(company_id=company_id, data_movimento_i=data_movimento_i, data_movimento_f=data_movimento_f,
                   requisicao=requisicao, join_almoxarifado=join_almoxarifado)
        cr.execute(query_gerencial)
        dados = cr.dictfetchall()
        return dados
