# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class estoque_produto(osv.Model):
    _name = 'estoque.produto'
    _inherit = 'estoque.produto'

    def ncm_tributo(self, cr, uid, ncm_id, produto_id, context=None):
        context = context and context.copy() or {}
        obj_ncm_tributacao = self.pool.get('fiscal.tributacao.encargos')
        obj_produto_tributos = self.pool.get('fiscal.tributacao.encargos')
        tributos_ids = obj_ncm_tributacao.search(cr, uid, [("ncm_id", "=", ncm_id)], context=context)
        ncm_tributos = obj_ncm_tributacao.read(cr, uid, tributos_ids, context=context)
        for x in ncm_tributos:
            obj_produto_tributos.create(cr, uid, {
                'encargo_id': x['encargo_id'] and x['encargo_id'][0],  # M2O
                'cst_id': x['cst_id'] and x['cst_id'][0],  # M2O
                'tributacao_id': x['tributacao_id'] and x['tributacao_id'][0],  # M2O
                'mva': x['mva'], 'valor_pauta': x['valor_pauta'], 'modbcst': x['modbcst'],
                'produto_id': produto_id, 'tipo_tributacao': x['tipo_tributacao'],
                'fisten_aliquota_venda': x['fisten_aliquota_venda']
            }, context=context)
        return True

    def button_tributos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        tributos_ids = self.read(cr, uid, ids, ['tributacao_ids'], context=context)[0]['tributacao_ids']
        objeto_tributo = self.pool.get('fiscal.tributacao.encargos')
        objeto_tributo.unlink(cr, uid, tributos_ids)

        ncm_id = self.read(cr, uid, ids, ['ncm_id'], context=context)[0]['ncm_id']

        if ncm_id:
            self.ncm_tributo(cr, uid, ncm_id[0], ids[0], context=context)
        return True

    def onchange_categoria(self, cr, uid, ids, categoria_id, ncm_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if ncm_id:
            return res
        if categoria_id:
            browse_categoria = self.pool.get('estoque.categoria').browse(cr, uid, categoria_id)
            res['margem_minima'] = browse_categoria.margem_minima or 0
            res['margem_ideal'] = browse_categoria.margem_ideal or 0
            res['margem_bruta'] = browse_categoria.margem_bruta or 0
            res['comprador_id'] = browse_categoria.comprador_id.id or False
            ncm_id = browse_categoria.ncm_id
            if ncm_id:
                ncm_id = ncm_id.id
                res['ncm_id'] = ncm_id
        return {'value': res}

    def estpro_get_cest(self, cr, uid, ids, context=None):
        """
        Metodo que retorna o id do cest a ser aplicado no produto.
        Verifica inicialmente no proprio produto, se não encontrar, pega o que está definido no ncm
        :param cr:
        :param uid:
        :param ids: lista de ids dos produtos
        :param context:
        :return: dicionario contendo id e cest_id ex.: {id:cest_id, id:cest_id, ....}
        """
        context = context and context.copy() or {}
        res = {}
        sql = """SELECT p.id, p.estpro_cest_id, n.fisnc_cest_id 
                 FROM estoque_produto AS p
                 LEFT join fiscal_ncm AS n ON n.id = p.ncm_id
                 WHERE p.id IN (%s)
              """ % ','.join(map(lambda x: '%s' % x, ids))
        cr.execute(sql)
        for item in cr.dictfetchall():
            res[item.get('id')] = item.get('estpro_cest_id') if item.get('estpro_cest_id') \
                             else item.get('fisnc_cest_id')
        return res

    def estpro_onchange_ncm(self, cr, uid, ids, ncm_id, cest_id, context=None):
        """
        O metodo vai atribuir um valor para o atributo estpro_cest_id baseado no codigo CEST
        incluido no cadastro do NCM.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: integer
        :param ncm_id: id da tabela fiscal_ncm.
        :type: integer
        :param cest_id: id da tabela fiscal_cest.
        :type: integer
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario com os campos e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not ncm_id or cest_id:
            return res
        brw_fiscal_ncm = self.pool.get('fiscal.ncm').browse(cr, uid, ncm_id, context)
        res['value'] = {'estpro_cest_id': brw_fiscal_ncm.fisnc_cest_id.id or False}
        return res

    def estpro_onchange_nao_utiliza_nat_rec(self, cr, uid, ids, nao_utiliza_nat_rec, context=None):
        context = context and context.copy()
        res = {}
        if nao_utiliza_nat_rec is False:
            return res
        res['value'] = {'estpro_pis_nat_rec': '', 'estpro_cofins_nat_rec': ''}
        return res

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(estoque_produto, self).default_get(cr, uid, fields_list, context)
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                    context=context)
        regime_tributario = self.pool.get('res.company').browse(
            cr, uid, company_id, context).rescon_pessoa_id.admpes_jur_regime_tributario
        if regime_tributario == 'S':
            res['estpro_nao_utiliza_nat_rec'] = True
        return res
