# -*- encoding: utf-8 -*-
##############################################################################
#                                                                            #
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca        #
#                    www.siginformatica.com.br luiz@siginformatica.com.br    #
#                                                                            #
##############################################################################

from openerp.osv import osv
from openerp.osv.fields import float_round


class gerencial_movimento(osv.Model):
    """
    Classe gerencial_movimento
    prefixo:fingmo_
    """

    _inherit = 'gerencial.movimento'

    def fingmo_get_valor_classificacao_origem(self, cr, uid, ids, context=None):
        """
        O metodo apura os valores das contas envolvidas no DRE. A apuracao e realizada no regime de
        fato gerador com classificacao de origem.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: valor total apurado das contas envolvidas no DRE.
        """
        context = context and context.copy() or {}
        vr_total = 0.0
        brw_gerencial_movimento = self.browse(cr, uid, ids, context)
        for movimento in brw_gerencial_movimento:
            titulo_ids = movimento.tesouraria_movimento_id.fintem_titulo_ids
            if not titulo_ids:
                vr_total += movimento.tesouraria_movimento_id.valor or 0
                continue

            for brw_titulo in titulo_ids:
                if not brw_titulo.fatura_id or not brw_titulo.fatura_id.nf_id:
                    if not brw_titulo.fintit_gerencialmov_ids:
                        vr_total += movimento.tesouraria_movimento_id.valor or 0
                        continue

                    for mov_tit in brw_titulo.fintit_gerencialmov_ids:
                        vr_total += mov_tit.valor or 0

                    continue

                nf_id = brw_titulo.fatura_id.nf_id.id
                vr_titulo = brw_titulo.vr_original or 0.0
                vr_nf = brw_titulo.fatura_id.nf_id.vr_total_nf or 0.0
                if round(vr_titulo, 2) != round(vr_nf, 2):
                    proporcao = vr_titulo / vr_nf
                    movimento_ids = self.search(cr, uid, [('fingmo_nf_id', '=', nf_id)],
                                                context=context)
                    sum_mov = sum(map(lambda i: i.valor * proporcao,
                                      self.browse(cr, uid, movimento_ids, context)))
                    vr_total += float(float_round(sum_mov, 2))
                else:
                    vr_total += vr_nf

        return vr_total

    def fingmo_get_gerencial_movimento_classificacao_origem_nivel_acima(self, cr, uid, ids,
                                                                        context=None):
        """
        O metodo busca o gerencial movimento dos titulos originados de notas fiscais.
        E usado para o regime de caixa com classificacao de origem.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: registros da tabela desse objeto.
        :rtype: list
        """
        context = context and context.copy() or {}
        gerencial_movimento_ids = []
        outra_conta_gerencial_movimento_ids = []
        brw_gerencial_movimento = self.browse(cr, uid, ids, context)
        for movimento in brw_gerencial_movimento:
            titulo_ids = movimento.tesouraria_movimento_id.fintem_titulo_ids
            if not titulo_ids:
                gerencial_movimento_ids.append(movimento.id)
                continue

            for brw_titulo in titulo_ids:
                if not brw_titulo.fatura_id or not brw_titulo.fatura_id.nf_id:
                    if not brw_titulo.fintit_gerencialmov_ids:
                        gerencial_movimento_ids.append(movimento.id)
                        continue

                    for mov_tit in brw_titulo.fintit_gerencialmov_ids:
                        gerencial_movimento_ids.append(mov_tit.id)

                    continue

                brw_nf = brw_titulo.fatura_id.nf_id
                if brw_nf.fisnfc_gerencial_movimento_ids:
                    gerencialmov_ids = filter(
                        lambda i: i.evento_id.id == movimento.evento_id.id and
                                  i.centro_custo_id.id == movimento.centro_custo_id.id,
                        brw_nf.fisnfc_gerencial_movimento_ids)
                    gerencial_movimento_ids.extend(map(lambda i: i.id, gerencialmov_ids))

                    outra_conta_gerencialmov_ids = filter(
                        lambda i: i.evento_id.id != movimento.evento_id.id or
                                  i.centro_custo_id.id != movimento.centro_custo_id.id,
                        brw_nf.fisnfc_gerencial_movimento_ids)
                    outra_conta_gerencial_movimento_ids.extend(map(lambda i: i.id,
                                                                   outra_conta_gerencialmov_ids))
        return list(set(gerencial_movimento_ids)), outra_conta_gerencial_movimento_ids
