# -*- coding: utf-8 -*-

from openerp.osv import osv


class vendas_apuracao_comissao_recebimento_osv(osv.TransientModel):

    _inherit = 'vendas.apuracao.comissao.recebimento.osv'

    def get_comissao_recebimento(self, cr, uid, interno_baixa_dict, externo_baixa_dict,
                                 context=None):
        """
        O metodo calcula o valor da comissao dos vendedores. Esse valor e proporcional pois e
        baseado no valor ja recebido diante do total do documento.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param interno_baixa_dict: dados da identificacao dos titulos e o respectivo valor recebido,
        referente aos vendedores internos
        :type interno_baixa_dict: dict
        :param externo_baixa_dict: dados da identificacao dos titulos e o respectivo valor recebido,
        referente aos vendedores externos
        :type externo_baixa_dict: dict
        :param context: dados contextuais
        :type context: dict
        :return: dados da identificacao do titulo e o respectivo valor da comissao
        :rtype: dict
        """
        context = context and context.copy() or {}
        titulo_ids = interno_baixa_dict.keys() + externo_baixa_dict.keys()
        titulo_ids = list(set(titulo_ids))
        if not titulo_ids:
            return interno_baixa_dict, externo_baixa_dict
        sql = """
            SELECT
                ft.id AS titulo_id,
                COALESCE(
                    (SELECT vr_total_nf FROM fiscal_nf_capa WHERE id IN (
                        SELECT nf_id FROM financeiro_fatura WHERE id = ft.fatura_id LIMIT 1)), 0) 
                    AS vr_nf,
                (SELECT sum(COALESCE(estmvi_vr_comissao, 0)) 
                 FROM estoque_movimento_itens WHERE nf_id IN (
                    SELECT nf_id FROM financeiro_fatura WHERE id = ft.fatura_id LIMIT 1))
                    AS vr_comissao
            FROM
                financeiro_titulo ft
            LEFT JOIN
                financeiro_fatura ff ON ff.id = ft.fatura_id
            WHERE
                ft.id IN ({titulo_ids})
                AND ff.nf_id IS NOT NULL   
        """.format(titulo_ids=', '.join(map(lambda t: str(t), titulo_ids)))
        cr.execute(sql)
        titulo_comissao_dados = cr.dictfetchall()
        for dado in titulo_comissao_dados:
            titulo_id = dado.get('titulo_id')
            if titulo_id in interno_baixa_dict:
                vr_comissao = round(
                    (interno_baixa_dict.get(titulo_id).get('valor') / dado.get('vr_nf')) * dado.get(
                        'vr_comissao'), 6)
                interno_baixa_dict[titulo_id].update({'vr_comissao': vr_comissao})
            if titulo_id in externo_baixa_dict:
                vr_comissao = round(
                    (externo_baixa_dict.get(titulo_id).get('valor') / dado.get('vr_nf')) * dado.get(
                        'vr_comissao'), 6)
                externo_baixa_dict[titulo_id].update({'vr_comissao': vr_comissao})
        return interno_baixa_dict, externo_baixa_dict

    def get_total_vendedor_forma_pagamento(self, cr, uid, data_ini, data_fim, vendedor_ids,
                                           vendedor_externo_ids, baixa_ids, context=None):
        """
        Metodo responsavel pelos dados de valores recebidos na baixa de titulos. Esses valores sao
        proporcionais baseados nas formas de pagamentos utilizadas nas vendas (NF) ou na
        baixa de titulos avulsos, com vendedor identificado.
        OBS.: METODO SOBRESCRITO NOS MODULOS fiscal e ECF, prevalecendo a hierarquia,
        respectivamente.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param data_ini: data inicial do filtro de pesquisa
        :type data_ini: str
        :param data_fim: data final do filtro de pesquisa
        :type data_fim: str
        :param vendedor_ids: identificadores dos vendedores internos, tabela vendas_vendedor
        :type vendedor_ids: list
        :param vendedor_externo_ids: identificadores dos vendedores externos, tabela vendas_vendedor
        :type vendedor_externo_ids: list
        :param baixa_ids: identificadores das baixas de titulos, tabela financeiro_titulo_baixa
        :type baixa_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: dados agrupados por vendedor, forma de pagamento e valores apurados
        :rtype: list
        """
        context = context and context.copy() or {}
        sql = """
            SELECT 
                forma_vendedor_id, 
                sum(vr_desconto) AS vr_desconto, 
                sum(vr_acrescimo_total) AS vr_acrescimo_total,
                sum(vr_bruto) AS vr_bruto, 
                sum(vr_liquido) AS vr_liquido, 
                nome_forma, 
                forma_pagamento_id, 
                nome_vendedor
            FROM (
                SELECT
                    (CASE 
                        WHEN titulo.vendedor_id IS NOT NULL 
                            THEN titulo.vendedor_id 
                        ELSE titulo.vendedor_externo_id 
                    END) AS forma_vendedor_id,
                    (
                        SELECT nome 
                        FROM adm_pessoa 
                        WHERE 
                            id IN (
                                SELECT pessoa_id 
                                FROM vendas_vendedor 
                                WHERE 
                                    CASE 
                                        WHEN titulo.vendedor_id IS NOT NULL 
                                            THEN id = titulo.vendedor_id 
                                        ELSE id = titulo.vendedor_externo_id 
                                    END
                                )
                    ) AS nome_vendedor,
                    round((COALESCE(baixa.valor_desconto, 0) / baixa.valor_pago) * 
                        baixa_forma.valor, 2) AS vr_desconto,
                    ROUND((baixa_forma.valor / baixa.valor_pago) * 
                        (baixa.valor_pago 
                        - COALESCE(baixa.valor_juros, 0) 
                        - COALESCE(baixa.valor_multa, 0)
                        - COALESCE(baixa.fintib_vr_acrescimo, 0) 
                        + COALESCE(baixa.valor_desconto, 0)), 2) AS vr_bruto,
                    baixa_forma.valor AS vr_liquido,
                    COALESCE(baixa.valor_juros, 0) + COALESCE(baixa.valor_multa, 0) 
                        + COALESCE(baixa.fintib_vr_acrescimo, 0) AS vr_acrescimo_total,
                    forma.name AS nome_forma,
                    forma.id AS forma_pagamento_id
                FROM
                    financeiro_titulo_baixa baixa
                INNER JOIN
                    financeiro_titulo titulo ON titulo.id = baixa.titulo_id
                INNER JOIN
                    financeiro_tesouraria_baixa_forma_pgto baixa_forma 
                    ON baixa.id = baixa_forma.tesouraria_baixa_id
                INNER JOIN
                    adm_forma_pgto forma ON forma.id = baixa_forma.forma_pgto_id
                LEFT JOIN
                    financeiro_fatura fatura ON fatura.id = titulo.fatura_id 
                WHERE
                    baixa.id IN ({baixa_ids})
                    AND baixa.data_pgto BETWEEN '{data_ini}' AND '{data_fim}'
                    AND (titulo.vendedor_id IN ({vendedor_ids}) 
                         OR titulo.vendedor_externo_id IN ({vendedor_externo_ids})
                         )
                    AND 
                        CASE WHEN titulo.fatura_id IS NOT NULL 
                            THEN fatura.nf_id IS NULL 
                            ELSE TRUE 
                        END
                GROUP BY
                    forma_vendedor_id,
                    forma.id, baixa.id , nome_vendedor, baixa_forma.valor
                UNION ALL
                SELECT
                    (CASE 
                        WHEN titulo.vendedor_id IS NOT NULL 
                            THEN titulo.vendedor_id 
                        ELSE titulo.vendedor_externo_id 
                    END) AS forma_vendedor_id,
                    (
                        SELECT nome 
                        FROM adm_pessoa 
                        WHERE 
                            id IN (
                                SELECT pessoa_id 
                                FROM vendas_vendedor 
                                WHERE 
                                    CASE 
                                        WHEN titulo.vendedor_id IS NOT NULL 
                                            THEN id = titulo.vendedor_id 
                                        ELSE id = titulo.vendedor_externo_id 
                                    END
                                )
                    ) AS nome_vendedor,
                    round((COALESCE(baixa.valor_desconto, 0) / baixa.valor_pago) * 
                        fiscal_condicao.valor, 2) AS vr_desconto,
                    ROUND((fiscal_condicao.valor / baixa.valor_pago) * 
                        (baixa.valor_pago 
                        - COALESCE(baixa.valor_juros, 0) 
                        - COALESCE(baixa.valor_multa, 0)
                        - COALESCE(baixa.fintib_vr_acrescimo, 0) 
                        + COALESCE(baixa.valor_desconto, 0)), 2) AS vr_bruto,
                    fiscal_condicao.valor AS vr_liquido,
                    round(COALESCE(baixa.valor_juros, 0) + COALESCE(baixa.valor_multa, 0) + 
                        COALESCE(baixa.fintib_vr_acrescimo, 0) / baixa.valor_pago * 
                        fiscal_condicao.valor, 2) AS vr_acrescimo_total,
                    forma.name AS nome_forma,
                    forma.id AS forma_pagamento_id
                FROM
                    financeiro_titulo_baixa baixa
                INNER JOIN
                    financeiro_titulo titulo ON titulo.id = baixa.titulo_id
                INNER JOIN
                    financeiro_fatura fatura 
                    ON fatura.id = titulo.fatura_id
                INNER JOIN
                    fiscal_nf_capa nota ON nota.id = fatura.nf_id
                INNER JOIN
                    fiscal_condicao_pagamento fiscal_condicao ON nota.id = fiscal_condicao.capa_id
                INNER JOIN
                    adm_forma_pgto forma ON forma.id = fiscal_condicao.forma_pagto_id
                WHERE
                    baixa.id IN ({baixa_ids})
                    AND baixa.data_pgto BETWEEN '{data_ini}' AND '{data_fim}'
                    AND (titulo.vendedor_id IN ({vendedor_ids}) 
                         OR titulo.vendedor_externo_id IN ({vendedor_externo_ids})
                         )
                GROUP BY
                    forma_vendedor_id,
                    forma.id, baixa.id , nome_vendedor, fiscal_condicao.valor
            ) AS tmp
            GROUP BY
                forma_vendedor_id,
                forma_pagamento_id,
                nome_forma,
                nome_vendedor
        """.format(baixa_ids=baixa_ids and ', '.join(map(lambda b: str(b), baixa_ids)) or '-1',
                   vendedor_ids=vendedor_ids and ', '.join(map(lambda v: str(v), vendedor_ids))
                   or '-1',
                   vendedor_externo_ids=vendedor_externo_ids and ', '.join(map(
                       lambda ve: str(ve), vendedor_externo_ids)) or '-1',
                   data_ini=data_ini,
                   data_fim=data_fim)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados
