# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from pynfe.xml_sped.base import *
from openerp.osv import osv
import base64

# Divisoes dos tipos complexos
RAIZ = '//EnviarLoteRpsEnvio'
RAIZ_LOTE = RAIZ + '/LoteRps'
RAIZ_LISTA = RAIZ_LOTE + '/ListaRps'
RAIZ_DECLARACAO = RAIZ_LISTA + '/Rps/InfDeclaracaoPrestacaoServico'
RAIZ_RPS_INFO = RAIZ_DECLARACAO + '/InfRps'
RAIZ_RPS_IDENTIFICACAO = RAIZ_RPS_INFO + '/IdentificacaoRps'
RAIZ_SERVICO = RAIZ_DECLARACAO + '/Servico'
RAIZ_VALOR = RAIZ_SERVICO + '/Valores'
RAIZ_PRESTADOR = RAIZ_DECLARACAO + '/Prestador'
RAIZ_TOMADOR = RAIZ_DECLARACAO + '/TomadorServico'


# noinspection PyPep8Naming
class rps_envio(XMLNFe):
    """
    Define a ´tag´ <EnviarLoteRpsEnvio>´ do arquivo xml.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        super(rps_envio, self).__init__()
        self.lote_rps = []

    @property  # Metodo ´get´
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '<?xml version="1.0" encoding="iso-8859-1"?>\n'
        xml += '<EnviarLoteRpsEnvio>\n'
        xml += ' <LoteRps>\n'

        for lote in self.lote_rps:
            xml += lote
        self.lote_rps = []

        xml += ' </LoteRps>\n'
        xml += '</EnviarLoteRpsEnvio>\n'

        return xml

    @xml.setter  # Metodo ´set´
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.lote_rps = self.le_grupo(RAIZ_LOTE, lote_rps)


# noinspection PyPep8Naming
class lote_rps(XMLNFe):
    """
    Define a ´tag´ <LoteRps>´ do arquivo xml.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        super(lote_rps, self).__init__()
        self.tsNumeroLote = TagInteiro(nome='NumeroLote', raiz=RAIZ_LOTE)
        self.tsCpf = TagCaracter(nome='Cpf', raiz=RAIZ_LOTE + '/CpfCnpj')
        self.tsCnpj = TagCaracter(nome='Cnpj', raiz=RAIZ_LOTE + '/CpfCnpj')
        self.tsInscricaoMunicipal = TagCaracter(nome='InscricaoMunicipal', raiz=RAIZ_LOTE)
        self.tsQuantidadeRps = TagInteiro(nome='QuantidadeRps', raiz=RAIZ_LOTE)
        self.lista_rps = []

    @property  # Metodo ´get´
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '  ' + self.tsNumeroLote.xml + '\n'

        # CPF ou CNPJ
        xml += '  <CpfCnpj>\n'

        if self.tsCpf.valor:
            xml += '   ' + self.tsCpf.xml + '\n'

        if self.tsCnpj.valor:
            xml += '   ' + self.tsCnpj.xml + '\n'

        xml += '  </CpfCnpj>\n'
        xml += '  ' + self.tsInscricaoMunicipal.xml + '\n'
        xml += '  ' + self.tsQuantidadeRps.xml + '\n'

        # Lista de RPS
        xml += '  <ListaRps>\n'

        for lista in self.lista_rps:
            xml += lista
        self.lista_rps = []

        xml += '  </ListaRps>\n'

        return xml

    @xml.setter  # Metodo ´set´
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tsNumeroLote = arquivo
            self.tsInscricaoMunicipal = arquivo
            self.tsQuantidadeRps = arquivo
            self.lista_rps = self.le_grupo(RAIZ_LISTA, lista_rps)


# noinspection PyPep8Naming
class lista_rps(XMLNFe):
    """
    Define a ´tag´ <ListaRps>´ do arquivo xml.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        super(lista_rps, self).__init__()
        self.tsNumeroRps = TagInteiro(nome='Numero', raiz=RAIZ_RPS_IDENTIFICACAO)
        self.tsSerieRps = TagInteiro(nome='Serie', obrigatorio=True, raiz=RAIZ_RPS_IDENTIFICACAO)
        self.tsTipoRps = TagInteiro(nome='Tipo', obrigatorio=True, raiz=RAIZ_RPS_IDENTIFICACAO)
        self.tsDataEmissao = TagData(nome='DataEmissao', raiz=RAIZ_RPS_INFO)
        self.tsStatus = TagInteiro(nome='Status', valor=1, raiz=RAIZ_RPS_INFO)
        self.tsCompetencia = TagData(nome='Competencia', raiz=RAIZ_DECLARACAO)
        self.tsRegimeEspecialTributacao = TagInteiro(
            nome='RegimeEspecialTributacao', raiz=RAIZ_DECLARACAO)
        self.tsOptanteSimplesNacional = TagInteiro(
            nome='OptanteSimplesNacional', raiz=RAIZ_DECLARACAO)
        self.tsIncentivoFiscal = TagInteiro(
            nome='IncentivoFiscal', raiz=RAIZ_DECLARACAO)
        self.servico = []
        self.prestador = []
        self.tomador = []

    @property  # Metodo ´get´
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '   <Rps>\n'
        xml += '    <InfDeclaracaoPrestacaoServico>\n'
        xml += '     <Rps>\n'
        xml += '      <InfRps>\n'
        xml += '       <IdentificacaoRps>\n'
        xml += '        ' + self.tsNumeroRps.xml + '\n'
        xml += '        ' + self.tsSerieRps.xml + '\n'
        xml += '        ' + self.tsTipoRps.xml + '\n'
        xml += '       </IdentificacaoRps>\n'
        xml += '       ' + self.tsDataEmissao.xml + '\n'
        xml += '       ' + self.tsStatus.xml + '\n'
        xml += '       <RpsSubstituido>\n'
        xml += '        <Numero />\n'
        xml += '        <Serie>1</Serie>\n'
        xml += '        <Tipo>1</Tipo>\n'
        xml += '        <Motivo />\n'
        xml += '       </RpsSubstituido>\n'
        xml += '      </InfRps>\n'
        xml += '     </Rps>\n'
        xml += '     ' + self.tsCompetencia.xml + '\n'

        # Servicos
        for s in self.servico:
            xml += s
        self.servico = []

        # Prestador
        for p in self.prestador:
            xml += p
        self.prestador = []

        # Tomador de servicos
        for t in self.tomador:
            xml += t
        self.tomador = []

        # Intermediario - Nao implementado
        xml += '     <Intermediario>\n'
        xml += '        <IdentificacaoIntermediario>\n'
        xml += '           <CpfCnpj>\n'
        xml += '             <Cpf></Cpf>\n'
        xml += '             <InscricaoMunicipal></InscricaoMunicipal>\n'
        xml += '           </CpfCnpj>\n'
        xml += '        </IdentificacaoIntermediario>\n'
        xml += '        <RazaoSocial></RazaoSocial>\n'
        xml += '     </Intermediario>\n'

        # Construcao civil - Nao implementado
        xml += '     <ConstrucaoCivil>\n'
        xml += '        <CodigoObra></CodigoObra>\n'
        xml += '        <Art></Art>\n'
        xml += '     </ConstrucaoCivil>\n'

        xml += '     ' + self.tsRegimeEspecialTributacao.xml + '\n'
        xml += '     ' + self.tsOptanteSimplesNacional.xml + '\n'
        xml += '     ' + self.tsIncentivoFiscal.xml + '\n'
        xml += '    </InfDeclaracaoPrestacaoServico>\n'
        xml += '   </Rps>\n'

        return xml

    @xml.setter  # Metodo ´set´
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tsNumeroRps = arquivo
            self.tsDataEmissao = arquivo
            self.tsCompetencia = arquivo
            self.tsRegimeEspecialTributacao = arquivo
            self.tsOptanteSimplesNacional = arquivo
            self.tsIncentivoFiscal = arquivo
            self.servico = self.le_grupo(RAIZ_SERVICO, servico)
            self.prestador = self.le_grupo(RAIZ_PRESTADOR, prestador)
            self.tomador = self.le_grupo(RAIZ_TOMADOR, tomador)


# noinspection PyPep8Naming
class servico(XMLNFe):
    """
    Define a ´tag´ <Servico>´ do arquivo xml.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        super(servico, self).__init__()
        self.tsValorServicos = TagDecimal(
            nome='ValorServicos', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorDeducoes = TagDecimal(
            nome='ValorDeducoes', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorPis = TagDecimal(
            nome='ValorPis', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorCofins = TagDecimal(
            nome='ValorCofins', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorInss = TagDecimal(
            nome='ValorInss', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorIr = TagDecimal(
            nome='ValorIr', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorCsll = TagDecimal(
            nome='ValorCsll', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsOutrasRetencoes = TagDecimal(
            nome='OutrasRetencoes', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsValorIss = TagDecimal(
            nome='ValorIss', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsAliquota = TagDecimal(
            nome='Aliquota', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsDescontoIncondicionado = TagDecimal(
            nome='DescontoIncondicionado', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsDescontoCondicionado = TagDecimal(
            nome='DescontoCondicionado', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=RAIZ_VALOR)
        self.tsIssRetido = TagInteiro(nome='IssRetido', valor=2, raiz=RAIZ_SERVICO)
        self.tsResponsavelRetencao = TagCaracter(nome='ResponsavelRetencao', raiz=RAIZ_SERVICO)
        self.tsCodAtividade = TagInteiro(nome='CodigoAtividade', raiz=RAIZ_SERVICO)
        self.tsCodigoCnae = TagInteiro(nome='CodigoCnae', raiz=RAIZ_SERVICO)
        self.tsDiscriminacao = TagCaracter(nome='Discriminacao', raiz=RAIZ_SERVICO)
        self.tsCodigoMunicipio = TagInteiro(nome='CodigoMunicipio', valor=3167202,
                                            raiz=RAIZ_SERVICO)
        self.tsCodigoPais = TagInteiro(nome='CodigoPais', valor=1058, raiz=RAIZ_SERVICO)
        self.tsExigibilidadeISS = TagInteiro(nome='ExigibilidadeISS', valor=1, raiz=RAIZ_SERVICO)
        self.tsMunicipioIncidencia = TagCaracter(nome='MunicipioIncidencia', raiz=RAIZ_SERVICO)
        self.tsNumeroProcesso = TagCaracter(nome='NumeroProcesso', raiz=RAIZ_SERVICO)

    @property  # Metodo ´get´
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '     <Servico>\n'
        xml += '      <Valores>\n'
        xml += '       ' + self.tsValorServicos.xml + '\n'
        xml += '       ' + self.tsValorDeducoes.xml + '\n'
        xml += '       ' + self.tsValorPis.xml + '\n'
        xml += '       ' + self.tsValorCofins.xml + '\n'
        xml += '       ' + self.tsValorInss.xml + '\n'
        xml += '       ' + self.tsValorIr.xml + '\n'
        xml += '       ' + self.tsValorCsll.xml + '\n'
        xml += '       ' + self.tsOutrasRetencoes.xml + '\n'
        xml += '       ' + self.tsValorIss.xml + '\n'
        xml += '       ' + self.tsAliquota.xml + '\n'
        xml += '       ' + self.tsDescontoIncondicionado.xml + '\n'
        xml += '       ' + self.tsDescontoCondicionado.xml + '\n'
        xml += '      </Valores>\n'
        xml += '      ' + self.tsIssRetido.xml + '\n'
        xml += '      ' + self.tsResponsavelRetencao.xml + '\n'
        xml += '      ' + self.tsCodAtividade.xml + '\n'
        xml += '      ' + self.tsCodigoCnae.xml + '\n'
        xml += '      ' + self.tsDiscriminacao.xml + '\n'
        xml += '      ' + self.tsCodigoMunicipio.xml + '\n'
        xml += '      ' + self.tsCodigoPais.xml + '\n'
        xml += '      ' + self.tsExigibilidadeISS.xml + '\n'
        xml += '      ' + self.tsMunicipioIncidencia.xml + '\n'
        xml += '      ' + self.tsNumeroProcesso.xml + '\n'
        xml += '     </Servico>\n'

        return xml

    @xml.setter  # Metodo ´set´
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tsValorServicos = arquivo
            self.tsValorDeducoes = arquivo
            self.tsValorPis = arquivo
            self.tsValorCofins = arquivo
            self.tsValorInss = arquivo
            self.tsValorIr = arquivo
            self.tsValorCsll = arquivo
            self.tsOutrasRetencoes = arquivo
            self.tsValorIss = arquivo
            self.tsAliquota = arquivo
            self.tsDescontoIncondicionado = arquivo
            self.tsDescontoCondicionado = arquivo
            self.tsIssRetido = arquivo
            self.tsResponsavelRetencao = arquivo
            self.tsCodAtividade = arquivo
            self.tsCodigoCnae = arquivo
            self.tsDiscriminacao = arquivo
            self.tsCodigoMunicipio = arquivo
            self.tsCodigoPais = arquivo
            self.tsExigibilidadeISS = arquivo
            self.tsMunicipioIncidencia = arquivo
            self.tsNumeroProcesso = arquivo


# noinspection PyPep8Naming
class prestador(XMLNFe):
    """
    Define a ´tag´ <Prestador>´ do arquivo xml.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        super(prestador, self).__init__()
        self.tsCpf = TagCaracter(nome='Cpf', raiz=RAIZ_PRESTADOR)
        self.tsCnpj = TagCaracter(nome='Cnpj', raiz=RAIZ_PRESTADOR)
        self.tsInscricaoMunicipal = TagCaracter(
            nome='InscricaoMunicipal', raiz=RAIZ_PRESTADOR)

    @property  # Metodo ´get´
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '     <Prestador>\n'

        # CPF ou CNPJ
        xml += '      <CpfCnpj>\n'

        if self.tsCpf.valor:
            xml += '      ' + self.tsCpf.xml + '\n'

        if self.tsCnpj.valor:
            xml += '      ' + self.tsCnpj.xml + '\n'

        xml += '      ' + self.tsInscricaoMunicipal.xml + '\n'
        xml += '      </CpfCnpj>\n'
        xml += '     </Prestador>\n'

        return xml

    @xml.setter  # Metodo ´set´
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tsCpf = arquivo
            self.tsCnpj = arquivo
            self.tsInscricaoMunicipal = arquivo


# noinspection PyPep8Naming
class tomador(XMLNFe):
    """
    Define a ´tag´ <Tomador>´ do arquivo xml.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        super(tomador, self).__init__()
        self.tsCpf = TagCaracter(nome='Cpf', raiz=RAIZ_TOMADOR)
        self.tsCnpj = TagCaracter(nome='Cnpj', raiz=RAIZ_TOMADOR)
        self.tsInscricaoMunicipal = TagCaracter(nome='InscricaoMunicipal', raiz=RAIZ_TOMADOR)
        self.tsRazaoSocial = TagCaracter(nome='RazaoSocial', raiz=RAIZ_TOMADOR)
        self.tsEndereco = TagCaracter(nome='Endereco', raiz=RAIZ_TOMADOR)
        self.tsNumero = TagCaracter(nome='Numero', raiz=RAIZ_TOMADOR)
        self.tsComplemento = TagCaracter(nome='Complemento', raiz=RAIZ_TOMADOR)
        self.tsBairro = TagCaracter(nome='Bairro', raiz=RAIZ_TOMADOR)
        self.tsCodigoMunicipio = TagInteiro(nome='CodigoMunicipio', raiz=RAIZ_TOMADOR)
        self.tsUf = TagCaracter(nome='Uf', raiz=RAIZ_TOMADOR)
        self.tsCodigoPais = TagInteiro(nome='CodigoPais', valor=1058, raiz=RAIZ_TOMADOR)
        self.tsCep = TagCaracter(nome='Cep', raiz=RAIZ_TOMADOR)
        self.tsTelefone = TagCaracter(nome='Telefone', raiz=RAIZ_TOMADOR)
        self.tsEmail = TagCaracter(nome='Email', raiz=RAIZ_TOMADOR)

    @property  # Metodo ´get´
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '     <TomadorServico>\n'
        xml += '      <IdentificacaoTomador>\n'

        # CPF ou CNPJ
        xml += '       <CpfCnpj>\n'

        if self.tsCpf.valor:
            xml += '       ' + self.tsCpf.xml + '\n'

        if self.tsCnpj.valor:
            xml += '       ' + self.tsCnpj.xml + '\n'

        xml += '       ' + self.tsInscricaoMunicipal.xml + '\n'
        xml += '       </CpfCnpj>\n'
        xml += '      </IdentificacaoTomador>\n'
        xml += '      ' + self.tsRazaoSocial.xml + '\n'
        xml += '      <Endereco>\n'
        xml += '       ' + self.tsEndereco.xml + '\n'
        xml += '       ' + self.tsNumero.xml + '\n'
        xml += '       ' + self.tsComplemento.xml + '\n'
        xml += '       ' + self.tsBairro.xml + '\n'
        xml += '       ' + self.tsCodigoMunicipio.xml + '\n'
        xml += '       ' + self.tsUf.xml + '\n'
        xml += '       ' + self.tsCodigoPais.xml + '\n'
        xml += '       ' + self.tsCep.xml + '\n'
        xml += '      </Endereco>\n'
        xml += '      <Contato>\n'
        xml += '       ' + self.tsTelefone.xml + '\n'
        xml += '       ' + self.tsEmail.xml + '\n'
        xml += '      </Contato>\n'
        xml += '     </TomadorServico>\n'

        return xml

    @xml.setter  # Metodo ´set´
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tsCpf = arquivo
            self.tsCnpj = arquivo
            self.tsInscricaoMunicipal = arquivo
            self.tsRazaoSocial = arquivo
            self.tsEndereco = arquivo
            self.tsNumero = arquivo
            self.tsComplemento = arquivo
            self.tsBairro = arquivo
            self.tsCodigoMunicipio = arquivo
            self.tsUf = arquivo
            self.tsCodigoPais = arquivo
            self.tsCep = arquivo
            self.tsTelefone = arquivo
            self.tsEmail = arquivo


# noinspection PyPep8Naming,PyAbstractClass
class vendas_wizard_exportar_rps(osv.TransientModel):
    """
    Exportacao (xml) das notas fiscais de servico sem numero mas com RPS.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'vendas.wizard.exportar.rps'  # vwerps
    _inherit = 'vendas.wizard.exportar.rps'  # sigaddons/fiscal/inherit/vendas/exportar_rps_data

    def button_selecionar(self, cr, uid, ids, context=None):
        # Remover notas selecionadas anteriormente
        _qtd_nf = 0
        brw_wiz = self.browse(cr, uid, ids[0], context=context)
        lista = []
        for x in brw_wiz.vwerps_nf_ids:
            lista.append((3, x.id, ''))
        # Incluir a nova selecao
        nf_ids = self.__vwern_fget_nf_servico_rps_ids(cr, uid, ids, context)
        for nf_id in nf_ids:
            _qtd_nf += 1
            lista.append((4, nf_id, ''))
            if _qtd_nf == 50:  # Maximo 50 nfs por lote(prefeitura de sete lagoas)
                break
        vals = {'vwerps_nf_ids': lista, 'state': 'S'}
        res = self.write(cr, uid, ids, vals, context=context)
        return res

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, context=context)
        __code = 'numero_lote_rps.fiscal'
        __name = 'Numero do lote do RPS para NFS-e'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 10,
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence').search(cr, uid, [('code', '=', __code), ('company_id', '=', company_id)], context=context)
        if not consulta:
            self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def __vwern_fget_nf_servico_rps_ids(self, cr, uid, ids, context=None):
        """
        Recupera o ID das notas fiscais de servico que tenham numero de RPS e nao tenha numero
        de nota.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type context: dict
        :return: lista com ID das notas fiscais de servico sem numeracao e com numero de RPS.
        :rtype: list[long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects _______________________________
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')

        context = context and context.copy() or {}

        dados_wiz = self.read(cr, uid, ids, [], context=context)
        _lote = 0
        for x in dados_wiz:
            _lote = x.get('vwers_nf_lote', 0)
        # Somente notas fiscais de servico sem numero e com numeracao de RPS
        args = [('tipo', '=', 'S'), ('fisnfc_rps_numero', '!=', False), ('state', '=', '2')]
        if _lote > 0:
            args.append(('fisnfc_rps_lote', '=', _lote))
        else:
            args.append('|')
            args.append(('fisnfc_rps_lote', '=', 0))
            args.append(('fisnfc_rps_lote', '=', False))
        result = obj_fiscal_nf_capa.search(cr, uid, args, context=context)

        return result

    def vwerps_button_exportar_rps(self, cr, uid, ids, context=None):
        """
        Cria o arquivo xml com dados das notas fiscais de servico sem numero mas com numeracao de
        RPS.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: IDs do fomrnulario de exportacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: verdadeiro se o arquivo xml foi criado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Recupera dados do formulario
        exportar_rps_dados = self.read(cr, uid, ids, ['vwerps_nf_ids', 'vwers_nf_lote'], context=context)

        if exportar_rps_dados and isinstance(exportar_rps_dados, list):
            exportar_rps_dados = exportar_rps_dados[0]

        # Lista de IDs das notas fiscais de servico
        nf_ids = exportar_rps_dados.get('vwerps_nf_ids')
        _lote = exportar_rps_dados.get('vwers_nf_lote')
        if not nf_ids:  # Não tem nota para exportar
            return True
        if _lote == 0:
            _lote = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, 'numero_lote_rps.fiscal')
            vals_nf = {'fisnfc_rps_lote': _lote}
            self.pool.get('fiscal.nf.capa').write(cr, uid, nf_ids, vals_nf, context=context)
            for id in nf_ids:
                self.pool.get('fiscal.nf.capa').message_post(cr, uid, id, 'NFS-e',
                                                         'Gerado Lote de RPS Nro: %s ' % _lote)


        # Gera o xml das notas fiscais de servico.
        xml = self._vwerps_gerar_xml_nfe_servico(cr, uid, _lote, nf_ids, context=context)

        # Cria o arquivo xml
        # Todo: verificar possibilidade de parametrizar o local onde o arquivo sera gerado
        caminho = '/tmp'
        arquivo = self._vwerps_gerar_nome_arquivo_xml(cr, uid, _lote, caminho, context=context)
        arquivo = open(arquivo, 'w')

        arquivo.write(xml)
        arquivo.close()

        # Retorna ao formulario (wizard)
        vals = dict(vwerps_arquivo=arquivo.name, vwerps_download=base64.encodestring(xml),
                    vwerps_mensagem='As notas fiscais foram exportadas com sucesso. Abaixo, o nome '
                                    'completo do arquivo de exportacao.')
        result = self.write(cr, uid, ids, vals, context)

        return result

    def _vwerps_gerar_nome_arquivo_xml(self, cr, uid, lote, caminho=None, context=None):
        """
        Gera o nome do arquivo que ira receber o xml de exportacao.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param caminho: caminho onde sera criado o arquivo
        :type cr: cursor
        :type uid: int
        :type caminho: str
        :return: nome completo do arquivo incluindo o caminho
        :rtype: str
        :todo: este metodo pode ser melhorado/ajustado para prever outras plataformas.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        import tempfile

        # ___________________________ Objects _______________________________
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')

        temp_dir = tempfile.gettempdir() if not caminho else caminho
        temp_dir += '/' if not temp_dir.endswith('/') else temp_dir  # linux somente ;)

        # Nome do arquivo baseado no numero da empresa
        result = temp_dir + str(int(lote)).zfill(3) + '_rps.xml'

        return result

    def _vwerps_gerar_xml_nfe_servico(self, cr, uid, lote, ids, context=None):
        """
        Gera o xml de nota fiscal eletronica da nota fiscal de servico.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com ID das notas de servico
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: xml de nota fiscal eletronica
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from unicodedata import normalize, category
        context = context and context.copy() or {}


        # ___________________________ Objects _______________________________
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_res_company = self.pool.get('res.company')
        obj_adm_uf = self.pool.get('adm.uf')
        obj_estoque_movimento_itens_ids = self.pool.get('estoque.movimento.itens')
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')

        # Grupos
        tag_importacao = rps_envio()
        tag_lote_rps = lote_rps()
        tag_lista_rps = lista_rps()
        tag_prestador = prestador()
        tag_servico = servico()
        tag_tomador = tomador()

        # Lote
        tag_lote_rps.tsNumeroLote.valor = lote  # Apenas um lote a ser enviado

        # Recupera dados da empresa
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, context=context)
        brw_res_company = obj_res_company.browse(cr, uid, company_id, context=context)
        brw_pessoa = brw_res_company.rescon_pessoa_id
        cpf_cnpj = filter(lambda f: f in '0123456789', brw_pessoa.cpf_cnpj)
        im = filter(lambda f: f.tipo_id.sigla.upper() == 'IM', brw_pessoa.documentos_ids)

        if im:
            im = im[0] if isinstance(im, list) else im
            im = filter(lambda f: f in '0123456789', im.numero)  # Somente algarismos
            tag_lote_rps.tsInscricaoMunicipal.valor = im
        else:
            tag_lote_rps.tsInscricaoMunicipal.valor = ''

        brw_nf_capa = obj_fiscal_nf_capa.browse(cr, uid, ids, context=context)
        tag_lote_rps.tsCpf.valor = cpf_cnpj if brw_pessoa.pessoa_tipo == 'F' else ''
        tag_lote_rps.tsCnpj.valor = cpf_cnpj if brw_pessoa.pessoa_tipo == 'J' else ''
        tag_lote_rps.tsQuantidadeRps.valor = len(brw_nf_capa)

        # Notas fiscais de servico / RPS
        for brw_nf in brw_nf_capa:
            endereco = obj_adm_pessoa.admpes_endereco_fiscal(
                cr, uid, brw_nf.destinatario_id.id, context).get(brw_nf.destinatario_id.id)
            uf = endereco.get('uf_sigla') or ''
            cep = codigo_ibge = numero = complemento = bairro = logradouro = ''

            if endereco.get('logradouro'):
                cep = filter(lambda f: f in '0123456789', endereco.get('cep') or '')
                codigo_ibge = endereco.get('codigo_ibge') or ''
                numero = filter(lambda f: f in '01234566789', endereco.get('numero') or '')
                complemento = filter(
                    lambda f: category(f) != 'Mn', normalize(
                        'NFKD', endereco.get('complemento') or u''))
                bairro = filter(
                    lambda f: category(f) != 'Mn', normalize(
                        'NFKD', endereco.get('bairro') or u''))
                logradouro = filter(
                    lambda f: category(f) != 'Mn', normalize(
                        'NFKD', endereco.get('logradouro') or u''))

            # Lista dos servicos executados (itens da nota)
            total_cofins = total_iss = total_pis = 0.0
            itens_ids = map(lambda m: m.id, brw_nf.estoque_movimento_itens_ids)

            descricao_itens = u''
            for item in obj_estoque_movimento_itens_ids.browse(cr, uid, itens_ids):
                descricao_itens += item.produto_id.name.strip() + u' '
                for tributo in item.tributacoes_ids:
                    if tributo.encargo_id.sigla.upper() == 'ISS':
                        total_iss += tributo.vr_encargo
                        aliquota_iss = tributo.aliquota
                        # Não informar Pis e Cofins, porque serão deduzidos do liquido da nf(serão retidos)
                        # if tributo.encargo_id.sigla.upper() == 'COFINS':
                        #     total_cofins += tributo.vr_encargo
                        # if tributo.encargo_id.sigla.upper() == 'PIS':
                        #     total_pis += tributo.vr_encargo

            tag_lista_rps.tsNumeroRps.valor = brw_nf.fisnfc_rps_numero
            tag_lista_rps.tsSerieRps.valor = brw_nf.serie_nf
            tag_lista_rps.tsTipoRps.valor = 1 if brw_nf.state != 'C' else 2  # 1=Sim 2=Nao
            tag_lista_rps.tsDataEmissao.valor = brw_nf.fisnfc_rps_data
            tag_lista_rps.tsCompetencia.valor = brw_nf.fisnfc_rps_data
            tag_lista_rps.tsRegimeEspecialTributacao.valor = 6  # TODO: Elaborar "de/para" com o atributo adm_pessoa.admpes_jur_tipo_empresa
            regime_tributario = brw_nf.destinatario_id.admpes_jur_regime_tributario
            simples_nacional = 1 if regime_tributario == 'S' else 2  # 1=Sim 2=Nao
            tag_lista_rps.tsOptanteSimplesNacional.valor = simples_nacional
            tag_lista_rps.tsIncentivoFiscal.valor = 2  # Sempre tributado (incentivo fiscal = Nao)

            # Valores dos servicos
            tag_servico.tsValorServicos.valor = '%.2f' % brw_nf.vr_total_nf
            tag_servico.tsValorDeducoes.valor = '%.2f' % sum(map(
                lambda deducoes: deducoes.valor or 0.0, brw_nf.deducao_itens_ids)) or 0.0
            tag_servico.tsValorPis.valor = '%.2f' % total_pis
            tag_servico.tsValorIss.valor = '%.2f' % total_iss
            tag_servico.tsAliquota.valor = '%.2f' % aliquota_iss
            tag_servico.tsValorCofins.valor = '%.2f' % total_cofins
            if brw_nf.company_id.rescon_codigo_atividade:
                tag_servico.tsCodAtividade.valor = brw_nf.company_id.rescon_codigo_atividade.strip()
            tag_servico.tsCodigoCnae.valor = filter(
                lambda f: f in '0123456789',
                brw_nf.company_id.rescon_pessoa_id.admpes_jur_cnae_id.admcna_codigo)
            discriminacao = brw_nf.fisnfc_discriminacao_servico or descricao_itens
            discriminacao = filter(
                lambda f: category(f) != 'Mn', normalize(
                    'NFKD', discriminacao or u''))
            tag_servico.tsDiscriminacao.valor = discriminacao
            tag_lista_rps.servico.append(tag_servico.xml)

            # Prestador de servicos
            tag_prestador.tsCpf.valor = tag_lote_rps.tsCpf.valor
            tag_prestador.tsCnpj.valor = tag_lote_rps.tsCnpj.valor
            tag_prestador.tsInscricaoMunicipal.valor = tag_lote_rps.tsInscricaoMunicipal.valor
            tag_lista_rps.prestador.append(tag_prestador.xml)

            # Tomador
            cpf_cnpj = filter(lambda f: f in '0123456789', brw_nf.destinatario_id.cpf_cnpj)
            tag_tomador.tsCpf.valor = cpf_cnpj if brw_nf.destinatario_id.pessoa_tipo == 'F' else ''
            tag_tomador.tsCnpj.valor = cpf_cnpj if brw_nf.destinatario_id.pessoa_tipo == 'J' else ''
            im = filter(
                lambda f: f.tipo_id.sigla == 'IM', brw_nf.destinatario_id.documentos_ids)
            if im:
                im = im[0] if isinstance(im, list) else im
                tag_tomador.tsInscricaoMunicipal.valor = filter(
                    lambda f: f in '0123456789', im.numero)  # Somente algarismos
            else:
                tag_tomador.tsInscricaoMunicipal.valor = ''

            tag_tomador.tsRazaoSocial.valor = filter(
                lambda c: category(c) != 'Mn', normalize('NFKD',
                                                         brw_nf.destinatario_id.name) or u'')
            tag_tomador.tsEndereco.valor = logradouro
            tag_tomador.tsNumero.valor = numero
            tag_tomador.tsComplemento.valor = complemento
            tag_tomador.tsBairro.valor = bairro
            tag_tomador.tsCodigoMunicipio.valor = codigo_ibge
            tag_tomador.tsUf.valor = uf
            tag_tomador.tsCep.valor = cep

            tag_lista_rps.tomador.append(tag_tomador.xml)
            tag_lote_rps.lista_rps.append(tag_lista_rps.xml)

        tag_importacao.lote_rps.append(tag_lote_rps.xml)

        return tag_importacao.xml

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
