# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.tools.translate import _
from openerp.osv import osv


# noinspection PyPep8Naming
class vendas_wizard_exportar_rps(osv.TransientModel):
    """
    Importacao de notas fiscais de servico com numero de arquivo csv.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'vendas.wizard.importar.nfe'  # vwinfe
    _inherit = 'vendas.wizard.importar.nfe'  # sigaddons/fiscal/inherit/vendas/inportar_nfe_data

    def vwinfe_button_importar_nfe(self, cr, uid, ids, context=None):
        """
        Importa os dados das notas fiscais de servico do arquivo .csf.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: IDs do fomrnulario de exportacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: formulario de importacao com os dados importados (notas fiscais de servico).
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from datetime import datetime

        context = context and context.copy() or {}

        # Recupera dados do formulario
        importar_nfe_dados = self.read(
            cr, uid, ids, ['vwinfe_arquivo', 'vwinfe_nf_servico_ids'], context=context)

        if importar_nfe_dados and isinstance(importar_nfe_dados, list):
            importar_nfe_dados = importar_nfe_dados[0]

            # Notas fiscais de servico (importadas do arquivo .csv)
            if not importar_nfe_dados.get('vwinfe_nf_servico_ids'):
                nf_servico_ids = map(lambda m: (0, 0, {
                    'vwinfi_nfe_numero': m.get('N\xfamero da nota'),
                    'vwinfi_nfe_numero_rps': m.get('N\xfamero do  RPS'),
                    'vwinfi_nfe_codigo_verificacao': m.get('C\xf3digo de verifica\xe7\xe3o'),
                    'vwinfi_nfe_state': m.get('Estado da nota'),
                    'vwinfi_nfe_data_emissao': m.get('Data e hora de emiss\xe3o'),
                    'vwinfi_nfe_valor_total': m.get('Valor total da nota')
                }), self._vwinfe_importar_arquivo_csv(
                    arquivo=importar_nfe_dados.get('vwinfe_arquivo'), binary=True))

                # Itens (notas fiscais importadas) e mensagem de aviso
                self.write(cr, uid, ids, dict(
                    vwinfe_nf_servico_ids=nf_servico_ids,
                    vwinfe_data_importacao=datetime.now() if nf_servico_ids else None,
                    vwinfe_mensagem=_('Aviso. Nenhuma nota fiscal de servico foi importada.') if not
                    nf_servico_ids else _('Importacao efetuada com sucesso.')), context)
            else:
                self.write(cr, uid, ids, dict(
                    vwinfe_data_importacao=datetime.now(), vwinfe_faturamento=False,
                    vwinfe_mensagem=None), context)

        view = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'fiscal', 'view_vendas_wizard_importar_nfe_form')
        view_id = view and view[1] or False
        result = dict(name='Importar NFe', view_type='form', view_mode='form', view_id=view_id,
                      res_model='vendas.wizard.importar.nfe', res_id=ids[0], target='new',
                      type='ir.actions.act_window', context=context)

        return result

    def vwinfe_button_faturar_nf_importada(self, cr, uid, ids, context=None):
        """
        Altera o estado (´state´) da nota fiscal de servico para 'Faturada' baseado no numero do
        RPS.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: IDs do fomrnulario de exportacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: formulario de importacao com os dados das notas fiscais de servico.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from datetime import datetime

        context = context and context.copy() or {}

        # Recupera dados do formulario
        importar_nfe_dados = self.read(cr, uid, ids, ['vwinfe_nf_servico_ids'], context=context)

        if importar_nfe_dados and isinstance(importar_nfe_dados, list):
            importar_nfe_dados = importar_nfe_dados[0]

            # Recupera as notas fiscais de servico baseadas no numero do RPS
            importar_nfe_dados = self._vwinfe_atualiza_item_nf_servico_rps(
                cr, uid, importar_nfe_dados.get('vwinfe_nf_servico_ids'), context)

            # Fatura as notas fiscais de servico (item de importacao atualizado)
            nf_servico_ids = self._vwinfe_faturar_nfe_importada(
                cr, uid, importar_nfe_dados, context)

            # Itens (notas fiscais importadas) e mensagem de aviso
            self.write(cr, uid, ids, dict(
                vwinfe_faturamento=True, vwinfe_nf_servico_ids=nf_servico_ids,
                vwinfe_mensagem=_('Aviso. Nenhuma nota fiscal de servico foi faturada.') if not
                nf_servico_ids else _('Faturamento efetuado.')), context)

        view = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'fiscal', 'view_vendas_wizard_importar_nfe_form')
        view_id = view and view[1] or False
        result = dict(name='Importar NFe', view_type='form', view_mode='form', view_id=view_id,
                      res_model='vendas.wizard.importar.nfe', res_id=ids[0], target='new',
                      type='ir.actions.act_window', context=context)

        return result

    def _vwinfe_atualiza_item_nf_servico_rps(self, cr, uid, ids, context=None):
        """
        Recupera as notas fiscais de servico baseado no numero do RPS.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: IDs dos itens de importacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: lista com os valores dos itens de importacao atualizada.
        :rtype: list[tuple]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # _________________________ Objects ______________________________________
        obj_wizard_importar_nfe = self.pool.get('vendas.wizard.importar.nfe.itens')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')

        result = []
        campos = ['vwinfi_nfe_numero_rps', 'vwinfi_nfe_numero', 'vwinfi_nfe_data_emissao']
        importar_nfe_dados = obj_wizard_importar_nfe.read(
            cr, uid, ids if isinstance(ids, list) else [ids], campos, context=context)

        # Notas fiscais de servico a serem faturadas (somente notas validadas)
        for item in importar_nfe_dados:
            args = [('fisnfc_rps_numero', '=', item.get('vwinfi_nfe_numero_rps')),
                    ('state', '=', '2')]
            nf_ids = obj_fiscal_nf_capa.search(cr, uid, args, context=context)

            # Guarda o ID da nota fiscal encontrada
            if nf_ids:
                item['nf_ids'] = nf_ids  # Nova chave do item de importacao com o ID da nota fiscal
                result.append((1, item.get('id'), item))  # Item de importacao a ser atualizado

        return result

    def _vwinfe_faturar_nfe_importada(self, cr, uid, dados, context=None):
        """
        Altera o estado da nota fiscal de servicao em funcao do numero do RPS. Fatura a nota fiscal.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param dados: lista com os dados de importacao (notas fiscais de servico com RPS)
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type dados: list[tuple]
        :type context: dict
        :return: lista atualizada comn os dados do item de importacao
        :rtype: list[tuple]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # _________________________ Objects ______________________________________
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')

        result = []  # Itens de importacao (atualizados)

        # Fatura as notas fiscais de servico com RPS
        for dado in dados:
            nf_ids = dado[2].get('nf_ids')  # Nova chave do item de importacao: ID da nota fiscal
            numero_nf = dado[2].get('vwinfi_nfe_numero')
            data_emissao = dado[2].get('vwinfi_nfe_data_emissao')

            # Atualiza os dados da nota fiscal
            obj_fiscal_nf_capa.write(cr, uid, nf_ids, dict(
                numero_nf=numero_nf, data_emissao=data_emissao, state='3'), context)

            # Altera o estado do item de importacao
            result.append((1, dado[1], dict(vwinfi_nfe_state='Faturada')))

        return result

    @staticmethod
    def _vwinfe_importar_arquivo_csv(arquivo='', separador=';', binary=False):
        """
        Gera o nome do arquivo que ira receber o xml de exportacao.

        :param arquivo: nome do arquivo a ser importado
        :param separador: caracter utilizado na separacao dos dados
        :type arquivo: str
        :type separador: str
        :return: lista de dicionario com os dados do arquivo csv.
        :rtype: list[dict]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        import csv
        import base64
        from tempfile import NamedTemporaryFile
        from os.path import isfile

        if binary:
            file_content = base64.decodestring(arquivo)
            file_csv = NamedTemporaryFile('w', delete=False)
            arquivo = file_csv.name
            file_csv.write(file_content)
            file_csv.close()

        if isfile(arquivo):
            with open(arquivo) as csvfile:
                result = map(lambda row: row, csv.DictReader(csvfile, delimiter=separador))
        else:
            result = []

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
