# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from funcoes import wizanv_get_total_geral_encargos
from openerp.osv.fields import float_round


class wizard_analise_venda(osv.TransientModel):

    def wizanv_get_tipo_documento(self, cr, uid, context):
        context = context and context.copy() or {}
        res = super(wizard_analise_venda, self).wizanv_get_tipo_documento(cr, uid, context)
        res.append(('N', 'Nota Fiscal'))
        return res

    _inherit = 'wizard.analise.venda'
    _columns = {
        'wizanv_documento': fields.selection(wizanv_get_tipo_documento, 'Documento', size=1,
                                             required=True),
        'wizanv_nf_id': fields.many2one('fiscal.nf.capa', 'Nota Fiscal', ondelete='cascade'),
        'wizanv_vr_encargos_total': fields.function(
            wizanv_get_total_geral_encargos, method=True, string='Total dos impostos',
            digits=(15, 2), type='float'),
    }

    def wizanv_button_analisa_venda(self, cr, uid, ids, context=None):
        """
        O metodo cria os dados para a tabela wizard_analise_venda_item.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            DELETE FROM wizard_analise_venda_item WHERE wizani_capa_id = %s
        """ % ids[0]
        cr.execute(sql)
        brw_analise_venda = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        tipo_documento = brw_analise_venda.wizanv_documento
        vals = {}
        if tipo_documento == 'P':
            list_vals = self.wizanv_trata_analise_pedido(cr, uid, brw_analise_venda, context)
            pedido_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if pedido_id:
                if self.pool.get('vendas.liberar.desconto').search(
                        cr, uid, [('venauv_pedido_id', '=', pedido_id),
                                  ('venauv_autorizado', '=', False)], limit=1, context=context):
                    vals.update({'wizanv_bloqueado_desconto': True})
                brw_pedido_capa = self.pool.get('estoquepedidos.pedido.capa').browse(
                    cr, uid, pedido_id, context)
                itens = brw_pedido_capa.pedido_item_ids
                vals.update({
                    'wizanv_vr_despesas': brw_pedido_capa.vr_despesas_acessorias or 0.0,
                    'wizanv_vr_desconto': (brw_pedido_capa.vr_desconto or 0.0) + sum(
                        map(lambda i: i.vr_desconto or 0.0, itens)),
                    'wizanv_vr_seguro': brw_pedido_capa.vr_seguro or 0.0,
                    'wizanv_vr_frete': brw_pedido_capa.vr_frete or 0.0,
                    'wizanv_vr_total_documento': brw_pedido_capa.valor_total or 0.0})
        elif tipo_documento == 'O':
            list_vals = self.wizanv_trata_analise_orcamento(cr, uid, brw_analise_venda, context)
            orcamento_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if orcamento_id:
                if self.pool.get('vendas.liberar.desconto').search(
                        cr, uid, [('venauv_orcamento_id', '=', orcamento_id),
                                  ('venauv_autorizado', '=', False)], limit=1, context=context):
                    vals.update({'wizanv_bloqueado_desconto': True})
                brw_orcamento_capa = self.pool.get('orcamentos.orcamento.capa').browse(
                    cr, uid, orcamento_id, context)
                itens = brw_orcamento_capa.orcamento_itens_ids
                vals.update({
                    'wizanv_vr_despesas': brw_orcamento_capa.orccap_vr_despesas_acessorias or 0.0,
                    'wizanv_vr_desconto': (brw_orcamento_capa.valor_desconto_total or 0.0) + sum(
                        map(lambda i: i.desconto_total or 0.0, itens)),
                    'wizanv_vr_seguro': 0.0,
                    'wizanv_vr_frete': brw_orcamento_capa.orccap_vr_frete or 0.0,
                    'wizanv_vr_total_documento': brw_orcamento_capa.valor_liquido or 0.0})
        else:
            list_vals = self.wizanv_trata_analise_nf(cr, uid, brw_analise_venda, context)
            nf_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if nf_id:
                if self.pool.get('vendas.liberar.desconto').search(
                        cr, uid, [('venauv_nf_id', '=', nf_id),
                                  ('venauv_autorizado', '=', False)], limit=1, context=context):
                    vals.update({'wizanv_bloqueado_desconto': True})
                brw_nf_capa = self.pool.get('fiscal.nf.capa').browse(cr, uid, nf_id, context)
                itens = brw_nf_capa.estoque_movimento_itens_ids
                vals.update({
                    'wizanv_vr_despesas': sum(map(lambda i: i.vr_despesas_rateado or 0.0, itens)),
                    'wizanv_vr_desconto': sum(map(lambda i: i.vr_desconto or 0.0, itens)) + sum(
                        map(lambda i: i.vr_desconto_rateado or 0.0, itens)),
                    'wizanv_vr_seguro': sum(map(lambda i: i.vr_seguro_rateado or 0.0, itens)),
                    'wizanv_vr_frete': sum(map(lambda i: i.vr_frete_rateado or 0.0, itens)),
                    'wizanv_vr_total_documento': brw_nf_capa.vr_total_nf or 0.0})
        if list_vals:
            vals.update({'wizanv_item_ids': list_vals})
        if vals:
            self.write(cr, uid, ids, vals, context)
        if context.get('active_model') and context.get('active_id'):
            obj_model_data = self.pool.get('ir.model.data')
            reference = obj_model_data.get_object_reference(
                cr, uid, 'vendas', 'wizard_analise_venda_form')
            view_id = reference and reference[1] or False
            return {'res_id': ids[0] if isinstance(ids, list) else ids,
                    'name': 'Analise de Venda',
                    'view_mode': 'form',
                    'res_model': 'wizard.analise.venda',
                    'view_id': [view_id],
                    'target': 'new',
                    'type': 'ir.actions.act_window',
                    'context': context
                    }
        return True

    def wizanv_onchange_pessoa(self, cr, uid, ids, pessoa_id, context=None):
        context = context and context.copy() or {}
        return {'value': {'wizanv_pedido_id': False, 'wizanv_orcamento_id': False,
                          'wizanv_nf_id': False}}

    def wizanv_onchange_tipo_documento(self, cr, uid, ids, tipo_documento, context=None):
        context = context and context.copy() or {}
        vals = {}
        if not tipo_documento:
            vals.update({'wizanv_pedido_id': False, 'wizanv_orcamento_id': False,
                         'wizanv_nf_id': False})
        if tipo_documento == 'P':
            vals.update({'wizanv_orcamento_id': False, 'wizanv_nf_id': False})
        if tipo_documento == 'O':
            vals.update({'wizanv_pedido_id': False, 'wizanv_nf_id': False})
        if tipo_documento == 'N':
            vals.update({'wizanv_pedido_id': False, 'wizanv_orcamento_id': False})
        return {'value': vals}

    def wizanv_onchange_id_documento(self, cr, uid, ids, pedido_id=False, orcamento_id=False,
                                     nf_id=False, context=None):
        context = context and context.copy() or {}
        vals = {'wizanv_id_documento_informado': False, 'wizanv_bloqueado_desconto': False}
        if any([pedido_id, orcamento_id, nf_id]):
            vals.update({'wizanv_id_documento_informado': True})
        obj_vendas_liberar_desconto = self.pool.get('vendas.liberar.desconto')
        if pedido_id:
            if obj_vendas_liberar_desconto.search(
                    cr, uid, [('venauv_pedido_id', '=', pedido_id),
                              ('venauv_autorizado', '=', False)], limit=1, context=context):
                vals.update({'wizanv_bloqueado_desconto': True})
        elif orcamento_id:
            if obj_vendas_liberar_desconto.search(
                    cr, uid, [('venauv_orcamento_id', '=', orcamento_id),
                              ('venauv_autorizado', '=', False)], limit=1, context=context):
                vals.update({'wizanv_bloqueado_desconto': True})
        elif nf_id:
            if obj_vendas_liberar_desconto.search(
                    cr, uid, [('venauv_nf_id', '=', nf_id),
                              ('venauv_autorizado', '=', False)], limit=1, context=context):
                vals.update({'wizanv_bloqueado_desconto': True})
        return {'value': vals}

    def wizanv_button_libera_desconto(self, cr, uid, ids, context=None):
        """
        O metodo realiza a liberacao de desconto para o documento.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso o documento tenha sido liberado. Senão , Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_analise_venda = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        obj_vendas_liberar_desconto = self.pool.get('vendas.liberar.desconto')
        if brw_analise_venda.wizanv_documento == 'P':
            pedido_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if not pedido_id:
                return False
            liberar_desconto_ids = obj_vendas_liberar_desconto.search(
                cr, uid, [('venauv_pedido_id', '=', pedido_id), ('venauv_autorizado', '=', False)],
                limit=1, context=context)
            if liberar_desconto_ids:
                obj_vendas_liberar_desconto.write(cr, uid, liberar_desconto_ids,
                                                  {'venauv_autorizado': True}, context)
        elif brw_analise_venda.wizanv_documento == 'O':
            orcamento_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if not orcamento_id:
                return False
            liberar_desconto_ids = obj_vendas_liberar_desconto.search(
                cr, uid, [('venauv_orcamento_id', '=', orcamento_id),
                          ('venauv_autorizado', '=', False)], limit=1, context=context)
            if liberar_desconto_ids:
                obj_vendas_liberar_desconto.write(cr, uid, liberar_desconto_ids,
                                                  {'venauv_autorizado': True}, context)
        else:
            nf_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if not nf_id:
                return False
            liberar_desconto_ids = obj_vendas_liberar_desconto.search(
                cr, uid, [('venauv_nf_id', '=', nf_id),
                          ('venauv_autorizado', '=', False)], limit=1, context=context)
            if liberar_desconto_ids:
                obj_vendas_liberar_desconto.write(cr, uid, liberar_desconto_ids,
                                                  {'venauv_autorizado': True}, context)
        self.write(cr, uid, ids, {'wizanv_bloqueado_desconto': False}, context)
        return True

    def wizanv_get_id_documento(self, cr, uid, brw_analise_venda, context=None):
        """
        O metodo identifica o registro do documento que sera feita a analise.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param brw_analise_venda: browse desse objeto.
        :type brw_analise_venda: object
        :param context: dados contextuais.
        :type context: dict
        :return: Inteiro, que representa id do registro. Senao, Falso.
        :rtype: int
        """
        context = context and context.copy() or {}
        if brw_analise_venda.wizanv_pedido_id:
            return brw_analise_venda.wizanv_pedido_id.id
        elif brw_analise_venda.wizanv_orcamento_id:
            return brw_analise_venda.wizanv_orcamento_id.id
        elif brw_analise_venda.wizanv_nf_id:
            return brw_analise_venda.wizanv_nf_id.id
        documento_id = False
        if brw_analise_venda.wizanv_documento == 'P':
            obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
            numero_pedido = '%' + brw_analise_venda.wizanv_numero_documento.zfill(10)
            pedido_ids = obj_estoquepedidos_pedido_capa.search(
                cr, uid, [('numero_pedido', 'like', numero_pedido), ('tipo', '=', 'V'),
                          ('state', '!=', 'C')], limit=1, context=context)
            if pedido_ids:
                documento_id = pedido_ids[0]
        elif brw_analise_venda.wizanv_documento == 'O':
            obj_orcamentos_orcamento_capa = self.pool.get('orcamentos.orcamento.capa')
            numero_orcamento = '%' + brw_analise_venda.wizanv_numero_documento.zfill(6)
            orcamento_ids = obj_orcamentos_orcamento_capa.search(
                cr, uid, [('documento', 'like', numero_orcamento), ('state', '!=', '5C')],
                limit=1, context=context)
            if orcamento_ids:
                documento_id = orcamento_ids[0]
        else:
            obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
            try:
                numero_nf = int(brw_analise_venda.wizanv_numero_documento)
            except Exception as e:
                return documento_id
            nf_ids = obj_fiscal_nf_capa.search(
                cr, uid, [('numero_nf', '=', numero_nf), ('tipo', '=', 'S'), ('state', '!=', 'C')],
                limit=1, context=context)
            if nf_ids:
                documento_id = nf_ids[0]
        return documento_id

    def wizanv_trata_analise_pedido(self, cr, uid, brw_analise_venda, context=None):
        """
        O metodo reune os dados referente ao pedido para que sejam incluidos na tabela
        wizard_analise_venda_item.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param brw_analise_venda: browse desse objeto.
        :type brw_analise_venda: object
        :param context: dados contextuais.
        :type context: dict
        :return: dados para insercao na tabela wizard_analise_venda_item.
        :rtype: list
        """
        context = context and context.copy() or {}
        list_vals = super(wizard_analise_venda, self).wizanv_trata_analise_pedido(
            cr, uid, brw_analise_venda, context)
        for val in list_vals:
            imposto_item_total = self.wizanv_get_total_encargos(
                cr, uid, val[2].get('wizani_produto_id'),
                val[2].get('wizani_preco_venda_item_total'))
            val[2]['wizani_vr_encargos_total'] = imposto_item_total
            vr_lucro = (val[2].get('wizani_vr_lucro') or 0) - imposto_item_total
            val[2]['wizani_vr_lucro'] = vr_lucro
            vr_item = val[2]['wizani_vr_total']
            val[2]['wizani_percentual_lucro'] = float_round(vr_lucro * 100 / vr_item, 2)
            # vr_contribuicao = (val[2].get('wizani_vr_contribuicao') or 0) - imposto_item_total
            # val[2]['wizani_vr_contribuicao'] = vr_contribuicao
            # val[2]['wizani_percentual_contribuicao'] = vr_contribuicao / vr_item * 100
        return list_vals

    def wizanv_trata_analise_orcamento(self, cr, uid, brw_analise_venda, context=None):
        """
        O metodo reune os dados referente ao orcamento para que sejam incluidos na tabela
        wizard_analise_venda_item.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param brw_analise_venda: browse desse objeto.
        :type brw_analise_venda: object
        :param context: dados contextuais.
        :type context: dict
        :return: dados para insercao na tabela wizard_analise_venda_item.
        :rtype: list
        """
        context = context and context.copy() or {}
        list_vals = super(wizard_analise_venda, self).wizanv_trata_analise_orcamento(
            cr, uid, brw_analise_venda, context)
        for val in list_vals:
            imposto_item_total = self.wizanv_get_total_encargos(
                cr, uid, val[2].get('wizani_produto_id'),
                val[2].get('wizani_preco_venda_item_total'))
            val[2]['wizani_vr_encargos_total'] = imposto_item_total
            vr_lucro = (val[2].get('wizani_vr_lucro') or 0) - imposto_item_total
            val[2]['wizani_vr_lucro'] = vr_lucro
            vr_item = val[2]['wizani_vr_total']
            val[2]['wizani_percentual_lucro'] = float_round(vr_lucro * 100 / vr_item, 2)
            # vr_contribuicao = (val[2].get('wizani_vr_contribuicao') or 0) - imposto_item_total
            # val[2]['wizani_vr_contribuicao'] = vr_contribuicao
            # val[2]['wizani_percentual_contribuicao'] = vr_contribuicao / vr_item * 100
        return list_vals

    def wizanv_trata_analise_nf(self, cr, uid, brw_analise_venda, context=None):
        """
        O metodo reune os dados referente a nota fiscal para que sejam incluidos na tabela
        wizard_analise_venda_item.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param brw_analise_venda: browse desse objeto.
        :type brw_analise_venda: object
        :param context: dados contextuais.
        :type context: dict
        :return: dados para insercao na tabela wizard_analise_venda_item.
        :rtype: list
        """
        context = context and context.copy() or {}
        list_vals = []
        obj_adm_utils = self.pool.get('adm.utils')
        brw_nf = brw_analise_venda.wizanv_nf_id
        if not brw_nf:
            obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
            try:
                numero_nf = int(brw_analise_venda.wizanv_numero_documento)
            except Exception as e:
                return list_vals
            nf_ids = obj_fiscal_nf_capa.search(
                cr, uid, [('numero_nf', '=', numero_nf), ('tipo', '=', 'S'), ('state', '!=', 'C'),
                          ('destinatario_id', '=', brw_analise_venda.wizanv_pessoa_id.id)],
                limit=1, context=context)
            if not nf_ids:
                raise osv.except_osv(_('Atencao'), _('Nota Fiscal nao encontrada.'))
            brw_nf = obj_fiscal_nf_capa.browse(cr, uid, nf_ids[0], context)
        item_ids = brw_nf.estoque_movimento_itens_ids
        total_nf = brw_nf.vr_total_nf
        total_itens = sum(map(lambda x: x.vr_total_item, item_ids))
        dif = total_nf - total_itens
        dict_rateio = {}
        if dif <= -0.01 or 0.01 < dif:
            lista_rateio = []
            for i in item_ids:
                lista_rateio.append([i.id, i.vr_total_item])

            dict_rateio = dict(obj_adm_utils.efetuar_rateio(dif, lista_rateio))

        produto_ids = list(set(map(lambda p: p.produto_id.id, item_ids)))
        custo_dados = self.wizanv_get_preco_custo(cr, uid, produto_ids, context)
        parametros = ['comissoes', 'custo_fixo']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', parametros, context=context)
        comissao = estoque_parametro_dados.get('comissoes') or 0.0
        custo_fixo = estoque_parametro_dados.get('custo_fixo') or 0.0
        for item in item_ids:
            produto_id = item.produto_id.id
            quantidade_item = item.quantidade
            quantidade_item_total = quantidade_item * (
                    item.produto_embalagem_id.embalagem_id.quantidade_na_embalagem or 1.0)
            custo_item_total = custo_dados.get(produto_id) * quantidade_item_total
            vr_total_item = (item.vr_total_item or 0) + (dict_rateio.get(item.id) or 0.0)
            imposto_item_total = self.wizanv_get_total_encargos(cr, uid, produto_id, vr_total_item,
                                                                context)
            comissao_item_total = float_round(comissao * vr_total_item / 100, 2)
            custo_fixo_item_total = float_round(custo_fixo * vr_total_item / 100, 2)
            lucro_item_total = float_round(vr_total_item - imposto_item_total - custo_item_total -
                                           comissao_item_total - custo_fixo_item_total, 2)
            agregado_item_total = vr_total_item - custo_item_total
            vals = {'wizani_codigo': item.produto_id.codigo,
                    'wizani_grade': item.grade_id.grade,
                    'wizani_descricao': item.grade_id.name,
                    'wizani_embalagem': item.produto_embalagem_id.embalagem_id.sigla,
                    'wizani_quantidade': item.quantidade,
                    'wizani_vr_total': vr_total_item,
                    'wizani_custo_total': custo_item_total,
                    'wizani_vr_encargos_total': imposto_item_total,
                    'wizani_vr_lucro': lucro_item_total,
                    'wizani_percentual_lucro': float_round(lucro_item_total * 100 / vr_total_item,
                                                           2),
                    'wizani_vr_contribuicao': agregado_item_total,
                    'wizani_percentual_contribuicao': agregado_item_total / vr_total_item * 100,
                    'wizani_custo_fixo': custo_fixo_item_total,
                    'wizani_vr_comissao': comissao_item_total,
                    'wizani_produto_id': produto_id,
                    'wizani_preco_venda_item_total': vr_total_item,
                    }
            list_vals.append((0, 0, vals))
        return list_vals

    def wizanv_get_total_encargos(self, cr, uid, produto_id, vr_total_item, context=None):
        """
        O metodo apura o valor total dos encargos (impostos) que incidem sobre o
        produto (produto_id), considerando se a empresa e debito/credito ou simples nacional.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_id: id da tabela estoque_produto, identificador do produto.
        :type produto_id: int
        :param vr_total_item: valor de venda do produto.
        :type vr_total_item: float
        :param context: dados contextuais.
        :type context: dict
        :return: valor total dos encargos.
        :rtype: float
        """
        context = context and context.copy() or {}
        if self.pool.get('adm.pessoa').verificar_empresa_simples_nacional(cr, uid):
            aliquota_dados = self.pool.get('fiscal.tributacao.simples.nacional').\
                fistsn_calcula_aliquota_imposto(cr, uid, [produto_id], context)
            vr_encargos = (aliquota_dados.get(produto_id) / 100.0) * vr_total_item
        else:
            obj_fiscal_utils = self.pool.get('fiscal.utils')
            context.update({'proprio': True})
            imposto_dados = obj_fiscal_utils.calcular_imposto(
                cr, uid, produto_id, vr_produto=vr_total_item, context=context)
            del context['proprio']
            vr_encargos = sum(map(lambda v: v.get('valor_imposto', 0), imposto_dados))
        return vr_encargos


class wizard_analise_venda_item(osv.TransientModel):

    _inherit = 'wizard.analise.venda.item'
    _columns = {
        'wizani_vr_encargos_total': fields.float('Imposto', digits=(15, 2)),
    }
