#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import osv
from openerp.tools.translate import _
from openerp.addons.web_sig.executor.descricao_funcoes import dic_funcoes


# noinspection PyPep8Naming,PyAbstractClass
class websig_executor_tela_osv(osv.TransientModel):
    _name = 'websig.executor.tela.osv'
    _inherit = 'websig.executor.tela.osv'

    def websig_exec_fis_002(self, cr, uid, ids, context=None):
        """
        Trocar CST do PIS/COFINS no movimento.
        Esta funcao troca os CSTs do PIS e do COFINS dos itens de movimento conforme periodo
        categoria e NCM informado.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com os IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: integer
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se os produtos sofreram alteracao da natureza ou ncm.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        # Recupera as informacoes do formulario
        executor_tela_id = ids[0] if isinstance(ids, list) else ids
        funcao_escolhida = eval(self.browse(cr, uid, executor_tela_id, context).dados)['fis_002']
        data_inicial = funcao_escolhida.get('campos').get('data_inicial').get('value', False)
        data_final = funcao_escolhida.get('campos').get('data_final').get('value', False)
        categoria_id = funcao_escolhida.get('campos').get('categoria_id').get('value', False)
        ncm_id = funcao_escolhida.get('campos').get('ncm_id').get('value', False)
        produto_id = funcao_escolhida.get('campos').get('produto_id').get('value', False)
        cst_pis_id = funcao_escolhida.get('campos').get('cst_pis_id').get('value', False)
        cst_cofins_id = funcao_escolhida.get('campos').get('cst_cofins_id').get('value', False)

        # Realiza as alteracao da natureza por categoria e/ou NCM
        _sql = """
SELECT        fisnit.id, fisenc.sigla
FROM          fiscal_nf_item_tributacao  AS fisnit
  INNER JOIN  estoque_movimento_itens    AS estmov ON estmov.id = fisnit.movimento_itens_id
  INNER JOIN  estoque_movimento_capa     AS estcap ON estcap.id = estmov.capa_id
  INNER JOIN  estoque_produto            AS estpro ON estpro.id = estmov.produto_id
  INNER JOIN  fiscal_encargo             AS fisenc ON fisenc.id = fisnit.encargo_id
  INNER JOIN  fiscal_cfop                AS fscfop ON fscfop.id = estmov.cfop_id
  INNER JOIN  fiscal_cst                 AS fiscst ON fiscst.id = fisnit.cst_id
WHERE         estpro.active              IS TRUE
  AND         fisenc.active              IS TRUE
  AND         fisenc.sigla               IN ('PIS', 'COFINS')
  AND         fiscst.fiscst_tipo_movimento = fscfop.tipo
  AND         estcap.data >= '%(data_inicial)s'
  AND         estcap.data <= '%(data_final)s'
"""
        if categoria_id:
            _sql += '  AND         estpro.categoria_id = %(categoria_id)s\n'
        if ncm_id:
            _sql += '  AND         estpro.ncm_id = %(ncm_id)s'
        if produto_id:
            _sql += '  AND         estpro.id = %(produto_id)s'
        _sql += ';'
        _sql %= locals()
        cr.execute(_sql)  # Executa a consulta
        movimento_dados = cr.dictfetchall()
        movimento_dados_pis = map(
            lambda m: m.get('id'), filter(lambda f: f.get('sigla') == 'PIS', movimento_dados))
        movimento_dados_cofins = map(
            lambda m: m.get('id'), filter(lambda f: f.get('sigla') == 'COFINS', movimento_dados))
        movimento_dados = len(movimento_dados)
        if movimento_dados_pis and cst_pis_id:
            _sql = """UPDATE  fiscal_nf_item_tributacao
                      SET     cst_id = %s
                      WHERE   id IN %s"""
            cr.execute(_sql % (str(cst_pis_id), str(tuple(movimento_dados_pis))))
        else:
            movimento_dados -= len(movimento_dados_pis)
        if movimento_dados_cofins and cst_cofins_id:
            _sql = """UPDATE  fiscal_nf_item_tributacao
                      SET     cst_id = %s
                      WHERE   id IN %s"""
            cr.execute(_sql % (str(cst_cofins_id),
                               str(tuple(movimento_dados_cofins))))
        else:
            movimento_dados -= len(movimento_dados_cofins)
        if movimento_dados:
            msg = _('Quantidade de itens alterados: %s' % movimento_dados)
            result = True
        else:
            msg = _('Nenhum item foi alterado.')
            result = False

        # Mostra mensagem
        msg = _('<strong>Concluido sem erro.</strong>') + '<br>' + msg
        self.write(cr, uid, ids, {'state': '3', 'mensagem': msg}, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
