# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


CFOP_MOVIMENTO_TIPO = [
    ('C', 'Compra/Venda'), ('D', 'Devolucao'), ('T', 'Transferencia'), ('E', 'Remessa'), ('R', 'Retorno'),
    ('U', 'Outras')
]


class fiscal_cfop(osv.Model):
    """
    Tabela de CFOPs
    prefixo: fiscfop_
    """
    from funcoes import _nome_completo

    _name = 'fiscal.cfop'
    _columns = {
        'descricao': fields.text('Descricao'), # Descricao detalhada do CFOP
        'tipo': fields.selection([('E', 'Entrada'), ('S', 'Saida')], 'Tipo', required=True, size=1),
        # Tipo do CFOP. E=Entrada, S=Saida
        'analitico_sintetico': fields.selection([('A', 'Analitico'), ('S', 'Sintetico')], 'Analitico Sintetico',
                                                required=True, size=1, select=1),
        'movimento_tipo': fields.selection(CFOP_MOVIMENTO_TIPO, 'Movimento tipo', required=True,
                                           size=1, select=True),
        'transacao_tipo': fields.selection([('C', 'Comercializacao'),
                                            ('I', 'Industrializacao'),
                                            ('M', 'Imobilizado'),
                                            ('S', 'Consumo'),
                                            ('O', 'Outros')], 'Operacao', required=True, size=1),
        'produto_servico_tipo': fields.selection([('P', 'Mercadoria / Produto'),
                                                  ('F', 'Servico de transporte'),
                                                  ('M', 'Servico de comunicacao'),
                                                  ('S', 'Servico'),
                                                  ('E', 'Energia eletrica'),
                                                  ('L', 'Combustivel / Lubrificante'),
                                                  ('O', 'Outros')], 'Transacao tipo', required=True, size=1),
        'aplicacao': fields.text('Aplicacao'), # como se aplica o cfop
        'cfop_natureza_ids': fields.one2many('fiscal.cfop.natureza', 'cfop_id', 'Natureza'),
        'codigo': fields.char('Codigo', size=4, required=True, select=True),
        'name': fields.function(_nome_completo, method=True, string='Nome', type='char', size=30, store=False),
        'parent_id': fields.many2one("fiscal.cfop", 'Codigo pai', ondelete="restrict", select=True), # id do CFOP pai
        'estoque_movimento_itens_ids': fields.one2many('estoque.movimento.itens', 'cfop_id', 'Itens'),
        'fiscfop_dapi': fields.integer('DAPI', help='Indica em qual linha do DAPI este CFOP sera '
                                                    'considerado'),
        'fiscfop_sped_contribuicoes': fields.boolean('Considerado no SPED Contribuicoes'),
    }
    _sql_constraints = [
        ('cfop_codigo_uniq', 'unique (codigo)', 'Codigo CFOP ja cadastrado!')
    ]
    _defaults = {
        'fiscfop_sped_contribuicoes': lambda *a: True,
    }
