# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_cfop_natureza(osv.Model):
    """
    Tabela de Naturezas do CFOP

    """

    _name = 'fiscal.cfop.natureza'
    _columns = {
        'cfop_id': fields.many2one(
            'fiscal.cfop', 'CFOP', ondelete='restrict', required=True, select=True,
            domain="[('analitico_sintetico', '=', 'A')]"),
        'name': fields.char('Nome', size=128, required=True),
        'fisnat_codigo_natureza_credito': fields.char(
            'Codigo da Natureza de Credito', size=2,
            help='Codigo da natureza de credito de acordo com a Tabela de Codigos e natureza de '
                 'credito (4.3.7)'),
        'active': fields.boolean('Ativo'),
        'tipo': fields.selection(
            [('E', 'Entrada'), ('S', 'Saida'), ], 'Tipo', required=True, size=1),
        'natureza_encargos_ids': fields.one2many(
            'fiscal.cfop.natureza.encargos', 'natureza_id', 'Encargos'),
        'estoque_movimento_itens_ids': fields.one2many(
            'estoque.movimento.itens', 'natureza_id', 'Itens'),
        'fna_evento_id': fields.many2one(
            'gerencial.evento', 'Evento', ondelete='cascade', select=True,
            help='O Evento deve ser informado para a custificacao.'),
        'fisnat_operacao_id': fields.many2one(
            'estoque.natureza.operacao', 'Operacao', required=True,
            domain="[('estnto_tipo_operacao','=',tipo)]"),
        'fisnat_natureza_fiscal': fields.char(
            'Natureza fiscal', size=60,
            help='Centro de custo a ser utilizado nas emissoes de nf/cupons.'
                 '\nEste centro de custo ira sobrepor ao cadastrado na categoria do produto.'),
        'fisnat_centro_custo_id': fields.many2one(
            'adm.empresa.centrocusto', 'Centro de Custo',
            help='Se o C. Custo informado, primeiramente e considerado para custificacao. Se nao, '
                 'verifica na categoria, e por ultimo no parametro.'),
        'fisnat_complemento_arroz': fields.boolean(
            'Complemento de arroz',
            help='Se marcado, indica que a nota fiscal sera uma nota de complemento de arroz.'),
        'fisnat_complemento_arroz_item_id': fields.many2one(
            'estoque.produto', 'Item de complemento de arroz'),
        'fisnat_icms_desonerado': fields.boolean('ICMS desonerado'),
        'fisnat_icms_desonerado_motivo': fields.selection(
            [('1', 'Taxi'), ('3', 'Uso na agropecuaria'), ('4', 'Frotista/Locadora'),
             ('5', 'Diplomatico/Consular'), ('6', 'Utilitarios e motocicletas da Amazonia'),
             ('7', 'SUFRAMA'), ('8', 'Venda a orgao publico'), ('9', 'Outros'),
             ('10', 'Deficiente condutor'), ('11', 'Deficiente nao condutor'),
             ('12', 'Orgao de fomento e desenvolvimento agropecuario')], 'Motivo da desoneracao'),
        'fisnat_codigo_ajuste_desonerado': fields.char('Codigo ajuste desoneracao', size=10,
                                                       oldname='fisnit_codigo_ajuste_desonerado'),
        'fisnat_calcula_difal': fields.boolean(
            'Calcula DIFAL (sem anuencia CONFAZ)',
            help='Se marcado, indica que o DIFAL sera calculado sempre quando houver venda de '
                 'mercadorias ou servicos para um estado diferente da origem.'),
        'fisnat_codigo_receita_uf_substituido': fields.char(
            'Codigo de receita', size=15,
            help='Codigo de receita referente a obrigacao, proprio da unidade da federacao do '
                 'contribuinte substituido.'),
        'fisnat_ir_retencao': fields.boolean('Retencao IR'),
        'fisnat_ir_aliquota': fields.float('Aliquota IR', digits=(4, 2)),
        'fisnat_dispositivo_fiscal': fields.text('Dispositivo fiscal'),
    }
    _defaults = {
        'active': lambda *a: True,
        'fisnat_complemento_arroz': False,
        'fisnat_icms_desonerado': False,
        'fisnat_calcula_difal': True,
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

