# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_cst(osv.Model):
    """
    Tabela de CSTs
    PREFIXO : FISCST
    """
    _name = 'fiscal.cst'
    _columns = {
        'codigo': fields.char('Codigo', size=2, required=True, select=True), # Codigo do CST
        'name': fields.char('Descricao', size=45, required=True), # Descricao do cst
        'tipo': fields.selection(
            [('T', 'Tributado'), ('U', 'Substituicao'), ('I', 'Isento'), ('O', 'Outros'), ('D', 'Diferido'),
             ('S', 'Servico'), ('TU', 'Tributado com substituicao')], 'Tipo', required=True, size=2),
        'encargo_id': fields.many2one("fiscal.encargo", 'Encargo', ondelete="cascade", required=True, select=True),
        'active': fields.boolean('Ativo'), # Indica que o cst esta ativo
        'tributo_simples_nac': fields.boolean('Simples Nacional',
                                              help='Tributacao exclusiva para empresas do Simples Nacional'),
        'tribut_excecao_ids': fields.one2many('fiscal.tributacao.excecao', 'cst_id', 'Excecao'),
        'tributacao_ids': fields.one2many('fiscal.tributacao', 'cst_id', 'Tributacao'),
        'produto_tributacao_ids': fields.one2many('fiscal.tributacao.encargos', 'cst_id', 'Tributacao'),
        'nf_item_tributacao_ids': fields.one2many('fiscal.tributacao.encargos', 'cst_id', 'Tributacao'),
        'fiscst_csosn_contribuinte_id': fields.many2one('fiscal.csosn', 'CSOSN contribuinte',
                                                        oldname='fiscst_csosn_id'),
        'fiscst_csosn_nao_contribuinte_id': fields.many2one('fiscal.csosn', 'CSOSN nao contribuinte'),
        'fiscst_tipo_movimento': fields.selection([('E', 'Entrada'), ('S', 'Saida'), ('A', 'Ambos')],
                                                  'Tipo de Movimento'),
        'fiscst_cst_oposto_id': fields.many2one('fiscal.cst', 'CST Oposto', domain="[('encargo_id','=',encargo_id)]",
                                                help="Este CST sera usado para entrada nas notas de devolucao."),
        'fiscst_codigo_beneficio': fields.char('Codigo Beneficio Fiscal UF', size=8),
        'fiscst_sigla_encargo': fields.related('encargo_id', 'sigla', type='char',
                                               relation='fiscal.encargo', string='Sigla encargo',
                                               readonly=True),
    }
    _defaults = {
        'active': lambda *a: True,
        'tributo_simples_nac': False,
    }
    _sql_constraints = [
        ('codigo_cst_uniq', 'unique (encargo_id, codigo)', 'O CST precisa ser unico para o encargo!'),
    ]


