# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2020 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2020, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import osv, fields


class fiscal_declaracao_importacao(osv.Model):
    # prefix: fisdim
    _name = 'fiscal.declaracao.importacao'
    _description = 'Declaracao de importacao'

    _columns = {
        'fisdim_nf_id': fields.many2one(
            'fiscal.nf.capa', 'Nota fiscal', ondelete='cascade', select=True),
        'fisdim_numero': fields.char('Numero', size=12, help='Numero do Documento de Importacao'),
        'fisdim_data': fields.date('Data', help='Data de registro da DI'),
        'fisdim_vr_despesa': fields.float('Despesas aduaneiras', digits=(15, 2), help='Valor das despesas aduaneiras'),
        'fisdim_vr_afrmm': fields.float('Valor da AFRMM', digits=(15, 2), help='Valor adicional ao frete para renovacao de marinha mercante'),
        'fisdim_local_desembaraco': fields.char('Local', size=60, help='Local do desembaraco aduaneiro'),
        'fisdim_uf_id': fields.many2one('adm.uf', 'UF', help='UF onde ocorreu o desembaraco aduaneiro'),
        'fisdim_exportador': fields.char('Codigo do exportador', size=60),
        'fisdim_data_desembaraco': fields.date('Data do desembaraco'),
        'fisdim_via_transporte': fields.selection([('1', 'Maritima'),('2', 'Fluvial'), ('3', 'Lacustre'), ('4', 'Aerea'), ('5', 'Postal'), ('6', 'Ferroviaria'), ('7', 'Rodoviaria'), ('8', 'Conduto/Rede Transmissao'), ('9', 'Meios proprios'), ('10', 'Entrada/Saida ficta'), ('11', 'Courier'), ('12', 'Handcarry')], 'Via de transporte', size=1, select=True, help='Via de transporte internacional informada na DI'),
        'fisdim_forma_importacao': fields.selection([('1', 'Importacao por conta propria'), ('2', 'Importacao por conta e ordem'), ('3', 'Importacao por encomenda')], 'Forma de importacao', size=1, select=True, help='Forma de importacao quanto a mediacao'),
        'fisdim_adicao_ids': fields.one2many('fiscal.declaracao.importacao.adicao', 'fisdia_di_id', 'Adicao', help='Adicoes')
    }

    _defaults = {
        'fisdim_vr_despesa': 0,
        'fisdim_vr_afrmm': 0,
        'fisdim_via_transporte': '1',
        'fisdim_forma_importacao': '1'
    }


class fiscal_declaracao_importacao_adicao(osv.Model):
    # prefix: fisdia
    _name = 'fiscal.declaracao.importacao.adicao'
    _description = 'Adicoes'

    _columns = {
        'fisdia_di_id': fields.many2one('fiscal.declaracao.importacao', 'Adicao', ondelete='cascade', select=True),
        'sequence': fields.integer('Sequencial', help='Numero sequencial do item dentro da adicao'),
        'fisdia_numero': fields.integer('Numero'),
        'fisdia_fabricante': fields.char('Codigo do fabricante', size=60, help='Codigo do fabricante estrangeiro. Normalmente o mesmo codigo do exportador'),
        'fisdia_vr_desconto': fields.float('Valor do desconto', digits=(15, 2), help='Valor do desconto do item da DI'),
        'fisdia_ato_concessorio': fields.integer('Drawback', help='Numero do ato concessorio de Drawback')
    }

    _order = 'sequence'
    _defaults = {
        'sequence': lambda s, c, u, x: s.search(c, u, [], count=True) + 1,
        'fisdia_numero': lambda s, c, u, x: (s.search(c, u, [], count=True) or 0) + 1,
        'fisdia_vr_desconto': 0
    }
