# -*- coding: utf-8 -*-

__author__ = 'marcilene.ribeiro'

from openerp.osv import osv, fields
from funcoes import _fisnfco_get_nf, _fisnfco_dias_de_ciencia, _fisnfco_get_dados

class fiscal_nfe_consumo(osv.Model):
    _name = 'fiscal.nfe.consumo'
    _inherit = ['mail.thread', 'ir.needaction_mixin']

    def _fisnfco_get_local_default(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = False
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id.id
        estoque_parametro_ids = obj_estoque_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)
        if estoque_parametro_ids:
            brw_parametro = obj_estoque_parametro.browse(cr, uid, estoque_parametro_ids[0])
            local_entrada_id = brw_parametro.estpar_local_compra_padrao_id
            res = local_entrada_id.id if local_entrada_id else False
        return res

    _rec_name = 'chave_nfe'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade'),
        'nsu': fields.integer('NSU'), #NSU – Numero Sequencial Único, IDENTITY da tabela;
        'tipo_documento': fields.selection([('N', 'NF-e'), ('C', 'Cancelamento'), ('E', 'Evento de CC-e')], 'Tipo',
                                           size=1),
        'data_recebimento': fields.date('Data Recebimento do Documento'),  #data de insercao da NF-e, Cancelamento ou CC-e ;

        # Identificacao do Emitente: UF, CNPJ/CPF, IE; Nome/Razao Social;
        'cpf_cnpj': fields.char('CPF/CNPJ', size=19, select=True), # Numero do Cpf ou CNPJ da pessoa
        'nome_razao_social': fields.char('Nome/Razao Social', size=60),  # Nome fantasia da empresa ou apelido da pessoa fisica
        'inscricao_estadual': fields.char('Inscricao Estadual', size=14),

        # Identificacao do Destinatário/Remetente: UF, CNPJ/CPF, IE; Nome/Razao Social;
        'cpf_cnpj_destinatario': fields.char('CPF/CNPJ', size=19, select=True), # Numero do Cpf ou CNPJ da pessoa
        'nome_razao_social_destinatario': fields.char('Nome/Razao Social', size=60),  # Nome fantasia da empresa ou apelido da pessoa fisica
        'inscricao_estadual_destinatario': fields.char('Inscricao Estadual', size=14),
        'chave_nfe': fields.char('Chave NF-e', size=44), #Chave de Acesso da NF-e;
        'fisnfco_numero_nf': fields.function(_fisnfco_get_dados, string='Numero da NF',
                                             method=True, store=True, type='integer', multi='dados'),
        'fisnfco_serie_nf': fields.function(_fisnfco_get_dados, string='Serie da NF',
                                             method=True, store=True, type='integer', multi='dados'),
        # Dados da NF-e ou do Cancelamento
        'data_emissao_cancelamento': fields.date('Data Emissao/Cancelamento'),  # Data de emissao da NF-e ou do Cancelamento da NF-e;
        'tipo_documento_fiscal': fields.selection([('0', 'Entrada'), ('1', 'Saida')], 'Tipo Documento Fiscal',
                                                  size=1),  #tpNF: Tipo do Documento Fiscal: 0=Entrada; 1=Saida;
        'finalidade_emissao': fields.selection([('1', 'Normal'), ('2', 'Complementar'), ('3', 'Ajuste')],
                                               'Finalidade emissao', size=1),  # finNFe: Finalidade da NF-e: 1=NF-e Normal; 2=NF-e Complementar; 3=NF-e Ajuste.
        'digest_value': fields.text('DigestValue'), #DigestValue: DigestValue da NF-e Autorizada;
        'situacao_nfe': fields.selection([('1', 'Autorizada'), ('2', 'Cancelada'), ('3', 'Denegada')], 'Situacao',
                                         size=1), # Situacao: 1=Autorizada; 2=Cancelada; 3=Denegada;
        'confirmacao_destinatario': fields.selection(
            [('0', 'Nao confirmada'), ('1', 'Confirmada'), ('2', 'Desconhecida'), ('3', 'Operacao nao Realizada')],
            'Confirmacao do Destinatario', size=1),  #Confirmacao Destinatário: 0=nao confirmada; 1=confirmada; 2=desconhecida; 3=Operacao nao Realizada;
        'data_hora_autorizacao': fields.datetime('Data Autorizacao'),  #dhRecbto: Data e Hora de autorizacao de uso da NF-e pela SEFAZ origem. Idem para a data de autorizacao do Cancelamento;
        'valor_total_nfe': fields.float('Valor Total NF-e', digits=(15, 2)),
        'fisnfco_nf_importada': fields.function(_fisnfco_get_nf, string='NF importada', method=True,
                                                store=False, type='boolean'),

        #Dados da Carta de Correcao
        'data_emissao_carta_correcao': fields.date('Data Emissao da Carta de correcao '),
        'codigo_evento': fields.char('Codigo do Evento', size=20),
        'sequencia_evento': fields.integer('Sequencial do Evento'),
        'descricao_evento': fields.char('Descricao Evento', size=20),  #descEvento: “Carta de Correcao” ou “Carta de Correcao”; Talvez seja desnecessário este campo
        'correcao': fields.text('Correcao'), # xCorrecao: Correcao a ser considerada.
        'data_hora_autorizacao_correcao': fields.datetime('Data/Hora da autorizacao de Carta de Correcao'),  #o dhRecbto: Data e Hora de autorizacao de uso da Carta de Correcao;
        'xml': fields.text('Arquivo XML'), #xml da nota
        'xml_file': fields.binary('Download arquivo xml'),
        'file': fields.char("Nome arquivo"),
        'state': fields.selection([('DI', 'Disponivel'), ('CI', 'Ciencia'),
                                   ('DO', 'Desconhecimento'), ('CO', 'Realizada'),
                                   ('OR', 'Nao Realizada'), ('BA', 'XML Baixado'),
                                   ('IM', 'XML Importado')], 'Status', size=2),
        'status_sefaz': fields.char('Status SEFAZ', size=60),
        'proc_resposta': fields.text('XML de resposta'),
        'fisnfco_natureza_id': fields.many2one('fiscal.cfop.natureza', 'Natureza'),
        'fisnfco_local_id': fields.many2one('estoque.local', 'Local'),
        'fisnfco_confirmou_operacao': fields.boolean('Realizou confirmacao'),
        'fisnfco_desconheceu': fields.boolean('Desconheceu operacao'),
        'fisnfco_danfe_nfe': fields.binary('Download Danfe da nota fiscal'),
        'fisnfco_nome_danfe': fields.char('Nome do danfe'),
        'fisnfco_data_ciencia_oper': fields.date('Data da ciencie da operacao'),
        'fisnfco_qtd_dias_de_ciencia': fields.function(_fisnfco_dias_de_ciencia, type='char',
                                                       method=True, store=False,
                                                       string='Dias de ciencia'),
        'active': fields.boolean('Ativo'),
    }
    _defaults = {
        'state': 'DI',
        'fisnfco_confirmou_operacao': False,
        'fisnfco_desconheceu': False,
        'fisnfco_local_id': _fisnfco_get_local_default,
        'active': True,
    }

    _order = 'id desc'

