#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from funcoes import fissup_get_parametros
from funcoes import fissup_check_tributacao_saida


class fiscal_sugestao_preco(osv.Model):
    """
    Classe responsavel pelos dados da capa da sugestao de de precos
    Esta classe foi transformada de uma TransienteModel para uma modo para
    atender solicitacao do chamado 91304
    :prefix: fissup
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """

    _name = 'fiscal.sugestao.preco'
    _columns = {
        # ------------ INTEGER ------------
        'id': fields.integer('Id', invisible=True),

        'name': fields.char('Descricao', size=60, required=True),
        # ------------ MANY2ONE ------------
        'fissup_local_id': fields.many2one('estoque.local', 'Local'),
        'fissup_pessoa_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'fissup_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'fissup_fornecedor_id': fields.many2one('financeiro.fornecedor', 'Fornecedor'),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", ),
        'fissup_nf_id': fields.many2one('fiscal.nf.capa', 'Nota Fiscal',
                                        domain="[('tipo', '=', 'E'), ('state', '=', '3')]",
                                        ondelete='cascade'),
        # ------------ DATE ---------------
        'fissup_data_entrada': fields.date('Data de entrada'),

        # ------------ SELECTION ----------
        'fissup_codigo': fields.selection([('I', 'Item'), ('F', 'Fabricante'), ('B', 'Barras')],
                                          'Codigo a listar', size=1, required=True),
        'fissup_opcoes': fields.selection(
            [('T', 'Total'), ('E', 'So entradas'), ('S', 'So a sugestao')],
            'Opcoes', size=1,
            required=True),
        'fissup_custo': fields.selection([('C', 'Corrigido'), ('R', 'Reposicao')], 'Custo a listar',
                                         size=1, required=True),

        # ------------ BOOLEANO ----------
        'fissup_listar_saldo_zero': fields.boolean('Lista itens com saldo zero'),
        'fissup_usar_margem_bruta_calculada': fields.boolean('Usa margem bruta calculada'),

        # ------------ FLOAT -------------
        'fissup_percentual_correcao': fields.float('Percentual para correcao'),


        # ----------- ONE2MANY -----------
        'fissup_sugestao_preco_itens_ids': fields.one2many(
            'fiscal.sugestao.preco.item', 'fisspi_capa_id', string='Itens da sugestao'),

        # ----------- FUNCITION -----------
        'fissup_custo_fixo': fields.function(
            fissup_get_parametros, string='Custo fixo', type='float', store=False, method=True,
            multi='parametros', digits=(5, 3)),
        'fissup_custo_para_preco_venda': fields.function(
            fissup_get_parametros, string='Custo preco venda', type='char', store=False,
            method=True, multi='parametros'),
        'fissup_comissao': fields.function(
            fissup_get_parametros, string='Comissao', type='float', store=False,
            method=True, multi='parametros', digits=(5, 3)),
        'fissup_string_custo': fields.function(
            fissup_get_parametros, string='String custo', type='char', store=False,
            method=True, multi='parametros'),
        'fissup_margem_parametro': fields.function(
            fissup_get_parametros, string='Margem do parametro', type='char', store=False,
            method=True, multi='parametros'),
        'fissup_tributacao_saida': fields.function(fissup_check_tributacao_saida, method=True,
                                                   string='Possui tributacao saida',
                                                   type='boolean'),
        'fissup_item_id': fields.many2one('estoque.produto', 'Item'),
    }

    _defaults = {
        'fissup_opcoes': 'E',
        'fissup_custo': 'R',
        'fissup_codigo': 'I',
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
