#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from funcoes import fisspi_get_codigo
from funcoes import fisspi_check_tributacao_saida


class fiscal_sugestao_preco_item(osv.Model):
    """
    Classe responsavel pelos itens da sugestao de preco
    :prefix: fisspi
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """

    _name = 'fiscal.sugestao.preco.item'
    _columns = {
        # ------------ MANY2ONE ------------
        'fisspi_capa_id': fields.many2one('fiscal.sugestao.preco', string='Sugestao de precos',
                                           ondelete='cascade', readonly=True),
        'fisspi_produto_id': fields.many2one('estoque.produto', string='Produto', readonly=True),
        'fisspi_grade_id': fields.many2one('estoque.produto.grade', string='Grade', readonly=True,
                                            ondelete='cascade'),
        'fisspi_estoque_embalagem_unidade_id': fields.many2one('estoque.embalagem.unidade',
                                                               string='Unidade', readonly=True,
                                                                ondelete='cascade'),
        'fisspi_estoque_embalagem_id': fields.many2one('estoque.embalagem', readonly=True,
                                                        ondelete='cascade', string='Embalagem'),
        'fisspi_nf_id': fields.many2one('fiscal.nf.capa', string='Nota fiscal', readonly=True,
                                        ondelete='cascade'),

        # ------------ RELATED ------------
        'fisspi_numero_nf': fields.related(
            'fisspi_nf_id', 'numero_nf', type="char", store=False, relation="fiscal.nf.capa",
            string='Numero da NF', readonly=True),
        'fisspi_descricao_produto': fields.related(
            'fisspi_produto_id', 'name', type="char", relation="estoque.produto", store=True,
            string='Descricao', readonly=True),
        'fisspi_grade_sigla': fields.related(
            'fisspi_grade_id', 'grade', type="char", relation="estoque.produto.grade", store=False,
            string='Grade', readonly=True),
        'fisspi_unidade_sigla': fields.related(
            'fisspi_estoque_embalagem_unidade_id', 'unidade', type="char", string='UND',
            store=False, relation="estoque.embalagem.unidade", readonly=True),

        # ------------ CHAR ---------------
        'fisspi_tipo_icms': fields.char(string='ICMS', readonly=True),

        # ------------ FLOAT --------------
        'fisspi_preco_custo_parametro': fields.float(string='Preco custo parametro', digits=(15, 5),
                                                     readonly=True),
        'fisspi_preco_custo_anterior': fields.float(string='Preco custo anterior', digits=(15, 5),
                                                    readonly=True),
        'fisspi_preco_custo_selecionado': fields.float(string='Preco custo selecionado',
                                                       digits=(15, 5), readonly=True),
        'fisspi_preco_venda': fields.float(string='Preco venda', digits=(14, 5), readonly=True),
        'fisspi_aliquota': fields.float(string='Aliquota', digits=(7, 4), readonly=True),
        'fisspi_qtd': fields.float(string='Quantidade', digits=(15, 2), readonly=True),
        'fisspi_margem_atual': fields.float(string='Margem atual', digits=(7, 2), readonly=True),
        'fisspi_margem_ideal': fields.float(string='Margem ideal', digits=(7, 2), readonly=True),
        'fisspi_margem_minima': fields.float(string='Margem minima', digits=(7, 2), readonly=True),
        'fisspi_margem_bruta': fields.float(string='Margem bruta', digits=(7, 2), readonly=True),
        'fisspi_sugestao': fields.float(string='Sugestao', digits=(7, 2)),
        'fisspi_quantidade_embalagem': fields.float(string='Embalagem', readonly=True,
                                                    digits=(8, 3)),

        # ------------ FUNCION --------------
        'fisspi_codigo': fields.function(fisspi_get_codigo, string='Codigo', readonly=True,
                                         method=True, store=False, type='char'),
        'fisspi_tributacao_saida': fields.function(fisspi_check_tributacao_saida, method=True,
                                                   string='Possui tributacao saida',
                                                   type='boolean'),

        # ------------ BOOLEAN --------------
        'fisspi_encargo_venda': fields.boolean(string="Possui encargo de venda")
    }
