# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################


from openerp.tools.translate import _
from datetime import datetime

# Funcoes da classe fiscal_apuracao_imposto
MES = {
    1: _('Janeiro'),
    2: _('Fevereiro'),
    3: _('Marco'),
    4: _('Abril'),
    5: _('Maio'),
    6: _('Junho'),
    7: _('Julho'),
    8: _('Agosto'),
    9: _('Setembro'),
    10: _('Outubro'),
    11: _('Novembro'),
    12: _('Dezembro')
}


def _get_name_imposto(self, cr, uid, ids, field, args, context=None):
    context = context and context.copy() or {}
    res = {}
    reads = self.read(cr, uid, ids, ['mes', 'ano'], context=context)
    if reads:
        for record in reads:
            ret = '%s / %s' % ( record.get('ano'), MES[record.get('mes', 1)])
            res[record['id']] = ret
    return res


def _fismnfs_get_tipo_modelo(self, cr, uid, ids, field, args, context=None):
    context = context and context.copy() or {}
    res = {}
    dados_mod_serie = self.read(cr, uid, ids, ['modelo_nf_id'],
                                context=context, load='_classic_write')
    modelo_ids = map(lambda x: x.get('modelo_nf_id'), dados_mod_serie)
    dict_modelo = {}
    for i in self.pool.get('fiscal.modelo.nf').read(cr, uid, modelo_ids, ['fismnf_tipo_nfe',
                                                               'fismnf_tipo_nfe_servico'], context):
        if i.get('fismnf_tipo_nfe'):
            dict_modelo[i.get('id')] = '1'
        elif i.get('fismnf_tipo_nfe_servico'):
            dict_modelo[i.get('id')] = '2'

    for i in dados_mod_serie:
        res[i.get('id')] = dict_modelo.get(i.get('modelo_nf_id')) or ''

    return res


def _fismnfs_get_mod_serie_ids(self, cr, uid, ids, context=None):
    context = context and context.copy() or {}
    res = self.pool.get('fiscal.modelo.nf.series').search(cr, uid,
                                                     [('modelo_nf_id', 'in', ids)],context=context)
    return res


# Funcoes da classe fiscal_cfop
def _nome_completo(self, cursor, user, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for NomeC in self.read(cursor, user, ids, ['codigo', 'descricao'], context=context):
        if NomeC.get('codigo') and NomeC.get('descricao'):
            res[NomeC.get('id')] = NomeC.get('codigo') + ' - ' + NomeC.get('descricao')
        elif NomeC.get('codigo'):
            res[NomeC.get('id')] = NomeC.get('codigo')
    return res


# Funcoes da classe fiscal_cfop_natureza_encargos
def _get_pis_cofins(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for e in self.browse(cr, uid, ids, context):
        res[e.id] = False
        if e.encargo_id:
            if e.encargo_id.sigla.upper() in ('PIS', 'COFINS'):
                res[e.id] = True
    return res


# Funcoes da classe fiscal_tributacao_simples_nacional
def _soma_total_aliquota(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for x in self.browse(cr, uid, ids):
        valor_total = x.fistsn_aliq_irpj + x.fistsn_aliq_csll + x.fistsn_aliq_cofins + \
                      x.fistsn_aliq_pis + x.fistsn_aliq_inss + x.fistsn_aliq_icms
        res[x.id] = valor_total
    return res


# Funcoes da classe fiscal_guia
def get_local_cobranca(self, cr, uid, context=None):
    context = context and context.copy() or {}
    id_par_fin = self.pool.get('financeiro.parametros').search(cr, uid, [], context=context)
    if id_par_fin:
        local_cobranca = \
            self.pool.get('financeiro.parametros').read(
                cr, uid, id_par_fin[0], ['local_cobranca_id'], context, '_classic_write')[
                'local_cobranca_id']
        return local_cobranca


def _get_simples_nacional(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for guia in self.browse(cr, uid, ids):
        verifica_simples_nacional = self.pool.get('adm.pessoa').verificar_empresa_simples_nacional(
            cr, uid)
        res[guia.id] = verifica_simples_nacional
    return res


# Funcoes da classe fiscal_condicao_pagamento
def get_valor_restante(self, cr, uid, context=None):
    context = context and context.copy() or {}
    valor_total = 0
    if context.get('vr_total_nf'):
        valor_total = context.get('vr_total_nf')
    if context.get('nf_condicao_ids'):
        nf_condicao_ids = context.get('nf_condicao_ids')
        for cond in nf_condicao_ids:
            # cond[0]==2 sao os registro que foram deletados
            if cond[0] != 2 and cond[1] and not cond[2]:
                valor_total -= self.browse(cr, uid, cond[1]).valor
            elif cond[2]:
                valor_total -= cond[2].get('valor')
    return valor_total


def _get_cartao_credito_debito(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for p in self.browse(cr, uid, ids, context):
        res[p.id] = False
        if p.forma_pagto_id:
            if p.forma_pagto_id.cartao_credito_debito:
                res[p.id] = True
    return res


def _get_forma_pagamento_cheque(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    if ids and not isinstance(ids, list):
        ids = [ids]
    if ids:
        browse_nf_condicao = self.browse(cr, uid, ids, context)
        for x in browse_nf_condicao:
            res[x.id] = x.forma_pagto_id.admfpg_documento_tipo_id.sigla == 'CHQ'
    return res


# Funcoes da classe fiscal_tributacao_encargos
def _get_empresa_regime(self, cr, uid, ids, context=None):
    context = context and context.copy() or {}
    simples_nacional = self.pool.get('adm.pessoa').verificar_empresa_simples_nacional(cr, uid)
    return simples_nacional


def _get_csosn(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for tri in self.browse(cr, uid, ids):
        res[tri.id] = ''
        if tri.cst_id:
            if tri.cst_id.fiscst_csosn_contribuinte_id:
                res[tri.id] = tri.cst_id.fiscst_csosn_contribuinte_id.name
    return res


# Funcoes da classe fiscal_nf_capa
def _fisnfc_get_nf_modelo_tipo(self, cr, uid, ids, field_name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for nf in self.browse(cr, uid, ids, context):
        res[nf.id] = {'fisnfc_nf_energia_eletrica': False, 'fisnfc_nf_agua_canalizada': False}
        if not nf.modelo_nf_id:
            continue
        if nf.modelo_nf_id.fismnf_tipo == 'E':
            res[nf.id]['fisnfc_nf_energia_eletrica'] = True
        elif nf.modelo_nf_id.fismnf_tipo == 'A':
            res[nf.id]['fisnfc_nf_agua_canalizada'] = True
    return res


def _fisnfc_get_total_servicos(self, cr, uid, ids, name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    for nf in self.browse(cr, uid, ids):
        valor_tot = 0
        for i in nf.estoque_movimento_itens_ids:
            if i.produto_id.tipo == 'S':
                valor_tot += i.vr_total_item
        res[nf.id] = valor_tot
    return res

# Comentei, porque não achei chamada do método em nenhum lugar
# def _fisnfc_get_nf_total_ipi(self, cr, uid, ids, name, arg, context=None):
#     context = context and context.copy() or {}
#     res = {}
#     for nf in self.browse(cr, uid, ids):
#         valor_tot = 0
#         for i in nf.estoque_movimento_itens_ids:
#             for x in i.tributacoes_ids:
#                 if x.encargo_id.sigla.upper() == 'IPI':
#                     valor_tot += x.vr_encargo
#         res[nf.id] = valor_tot
#     return res


def _fisnfc_get_nf_terceiro_modelo(self, cr, uid, context=None):
    context = context and context.copy() or {}
    modelo_serie_id = self._fisnfc_get_nf_modelo(cr, uid, context=context)
    modelo_id = []
    if modelo_serie_id:
        modelo_dados = self.pool.get('fiscal.modelo.nf.series').read(
            cr, uid, modelo_serie_id, ['modelo_nf_id'], context=context)
        if isinstance(modelo_dados, list):
            modelo_id = modelo_id[0]
        if isinstance(modelo_dados, dict):
            try:
                modelo_id = modelo_dados['modelo_nf_id'][0]
            except:
                pass

    return modelo_id


def _fisnfc_get_nf_modelo(self, cr, uid, context=None):
    context = context and context.copy() or {}
    modelo_id = False
    if not context.get('terceiro'):
        par = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal.parametro', [], context=context)
        if context.get('nf_servico'):
            modelo_id = par.get('fispar_servico_modelo_nf_series_id')
        elif context.get('nota_fiscal_consumidor') and not context.get('fisnfc_nfe_de_nfce'):
            modelo_id = par.get('fispar_nfce_modelo_series_id')
        else:
            modelo_id = par.get('modelo_nf_series_id')

    return modelo_id


def _fisnfc_get_tipo_frete(self, cr, uid, context=None):
    context = context and context.copy() or {}
    tipo = False
    if context.get('nota_fiscal_saida'):
        tipo = 'D'
    return tipo


def _fisnfc_get_msg(self, cr, uid, pessoa_ids, name, context):
    context = context and context.copy() or dict()
    if 'observacao_pessoa' not in name or not pessoa_ids:
        return {}

    sql = """
            SELECT fcmn.name      AS msg,
                   ap.observacoes,
                   ap.id          AS pessoa_id                
            FROM adm_pessoa                             AS ap 
            LEFT JOIN financeiro_cliente                AS fc   ON fc.pessoa_id = ap.id
            LEFT JOIN financeiro_clientes_mensagem      AS fcm  ON fcm.cliente_id = fc.id
            LEFT JOIN financeiro_clientes_mensagem_nome AS fcmn ON fcmn.id = fcm.mensagens_id
            WHERE ap.id IN (%s)
              AND fcm.modo_exibicao in ('V', 'A')
          """ % ','.join(map(lambda x: '%s' % x, pessoa_ids))
    cr.execute(sql)
    dict_msg = {}
    for i in cr.dictfetchall():
        pessoa_id = i.get('pessoa_id')
        if not dict_msg.get(pessoa_id):
            msg = i.get('observacoes') or ''
            if i.get('msg'):
                msg += '\n' + i.get('msg')

            dict_msg[pessoa_id] = msg

        elif i.get('msg'):
            dict_msg[pessoa_id] += '\n' + i.get('msg')

    return dict_msg


def _fisnfc_get_dict_titulos(self, cr, uid, ids, name, context=None):
    context = context and context.copy() or {}
    if 'fisnfc_titulos_ids' not in name:
        return {}

    res = {}
    sql = '''
            SELECT ft.id    AS titulo_id,
                   ff.nf_id AS nf_id
            FROM financeiro_titulo      AS ft
            LEFT JOIN financeiro_fatura AS ff ON ff.id = ft.fatura_id
            WHERE ff.nf_id IN (%s)
          ''' % ','.join(map(lambda x: '%s' % x, ids))
    cr.execute(sql)
    for dado in cr.dictfetchall():
        nf_id = dado.get('nf_id')
        if not res.get(nf_id):
            res[nf_id] = [dado.get('titulo_id')]
        else:
            res[nf_id].append(dado.get('titulo_id'))

    return res


def _fisnfc_get_dict_premite_alterar(self, cr, uid, ids, name, context=None):
    context = context and context.copy() or {}
    if 'fisnfc_permite_alterar_condicoes_pgto' not in name:
        return {}

    res = self.pool.get('estoque.parametro').estpar_get_permite_alterar_condicoes_pgto(cr, uid, ids,
                                                                                       context)
    return res


def _set_fisnfc_valida_item_rateado_nf(vals, nf, dict_itens, name, importacao=False):
    if 'fisnfc_valida_item_rateado_nf' not in name:
        return

    vr_total_item_ratedo = 0
    for i in nf.get('estoque_movimento_itens_ids'):
        item = dict_itens.get(i)
        vr_total_item_ratedo += item.get('vr_total_item_rateado') or 0

    total_rateado = (round(
        (nf.get('vr_icms_substituicao') or 0) +
        (not importacao and nf.get('vr_ipi') or 0) + (nf.get('fisnfc_valor_fcp_st') or 0) - nf.get(
            'fisnfc_valor_icms_desonerado') + vr_total_item_ratedo, 2))
    vals['fisnfc_valida_item_rateado_nf'] = nf.get('vr_total_nf') == total_rateado


def _set_fisnfc_imprimi_promissoria(vals, nf, dict_condicoes, forma_pgto_ids, name):
    if 'fisnfc_imprimi_promissoria' not in name:
        return

    imprimi_promissoria = any(map(lambda i: dict_condicoes.get(i) in forma_pgto_ids,
                                  nf.get('nf_condicao_ids')))
    imprimi_promissoria = [imprimi_promissoria, nf.get('state') == '3',
                           nf.get('nf_propria') is True,
                           nf.get('tipo') == 'S']
    vals['fisnfc_imprimi_promissoria'] = all(imprimi_promissoria)


def _set_observacao_pessoa(vals, nf, dict_msg, name):
    pessoa_id = nf.get('destinatario_id')
    if 'observacao_pessoa' not in name or not pessoa_id or not dict_msg.get(pessoa_id):
        return

    vals['observacao_pessoa'] = dict_msg.get(pessoa_id)


def _set_fisnfc_nf_devolucao(vals, nf, dados_cfop, name):
    if 'fisnfc_nf_devolucao' not in name:
        return

    devolucao = False
    if dados_cfop.get('movimento_tipo') == 'D':
        devolucao = True

    vals['fisnfc_nf_devolucao'] = devolucao


def _set_fisnfc_cfop_gnre(vals, dados_cfop, name):
    if 'fisnfc_cfop_gnre' not in name:
        return

    gnre = False
    if dados_cfop.get('codigo')[:1] != '2':
        gnre = True

    vals['fisnfc_cfop_gnre'] = gnre


def _set_fisnfc_modelo(vals, nf, dict_modelo, name):
    if 'fisnfc_modelo' not in name:
        return

    vals['fisnfc_modelo'] = dict_modelo.get(nf.get('modelo_nf_id')).get('fismnf_codigo') or ''


def _set_fisnfc_modelo_serie(vals, nf, dict_modelo, dict_modelo_serie, name):
    if 'fisnfc_modelo_serie' not in name:
        return

    modelo = dict_modelo.get(nf.get('modelo_nf_id'))
    mod_serie = ''
    if modelo and dict_modelo_serie.get(nf.get('modelo_nf_series_id')):
        mod_serie = modelo.get('fismnf_codigo') + \
                    dict_modelo_serie.get(nf.get('modelo_nf_series_id'))

    vals['fisnfc_modelo_serie'] = mod_serie


def _set_fisnfc_modelo_nf_tipo(vals, nf, dict_modelo, name):
    if 'fisnfc_modelo_nf_tipo' not in name:
        return

    modelo = dict_modelo.get(nf.get('modelo_nf_id'))
    vals['fisnfc_modelo_nf_tipo'] = modelo and modelo.get('fismnf_tipo')


def _set_name(vals, nf, dict_pessoas, name):
    if 'name' not in name:
        return

    nome = dict_pessoas.get(nf.get('destinatario_id')) or ' '
    if nf.get('numero_nf') == 0:
        nome = '%s - %s - %s ' % (nf.get('data_emissao'), nome, nf.get('vr_total_nf'))
    else:
        nome = '%s - %s - %s ' % (nf.get('numero_nf'), nf.get('serie_nf'), nome)

    vals['name'] = nome


def _set_fisnfc_parametro_danfe(vals, danf, name):
    if 'fisnfc_parametro_danfe' not in name:
        return

    vals['fisnfc_parametro_danfe'] = danf


def _set_fisnfc_ativar_entrega(vals, endereco_entrega, name):
    if 'fisnfc_ativar_entrega' not in name:
        return

    vals['fisnfc_ativar_entrega'] = endereco_entrega


def _set_fisnfc_ativar_coleta(vals, endereco_coleta, name):
    if 'fisnfc_ativar_coleta' not in name:
        return

    vals['fisnfc_ativar_coleta'] = endereco_coleta


def _set_fisnfc_titulos_ids(vals, id, dict_titulos, name):
    if 'fisnfc_titulos_ids' not in name:
        return []

    vals['fisnfc_titulos_ids'] = dict_titulos.get(id) or []


def _set_fisnfc_permite_alterar_condicoes_pgto(vals, id, dict_permite_alterar, name):
    if 'fisnfc_permite_alterar_condicoes_pgto' not in name:
        return

    vals['fisnfc_permite_alterar_condicoes_pgto'] = dict_permite_alterar.get(id)


def _set_fisnfc_calcula_peso_automaticamente(vals, parametro, name):
    if 'fisnfc_calcula_peso_automaticamente' not in name:
        return

    vals['fisnfc_calcula_peso_automaticamente'] = parametro.get(
        'fispar_calcula_peso_item_automatico')


def _set_fisnfc_permissao_voltar_esboco(vals, permisao_voltar, name):
    if 'fisnfc_permissao_voltar_esboco' not in name:
        return

    vals['fisnfc_permissao_voltar_esboco'] = permisao_voltar

def _set_fisnfc_link_xml(self, cr, uid, vals, nf, name, context=None):
    if 'fisnfc_link_xml' not in name:
        return
    if not (nf.get('nfe_chave_acesso') and nf.get('modelo_55')):
        vals['fisnfc_link_xml'] = ''
        return
    base = cr.dbname
    host = self.pool.get('ir.config_parameter').get_param(cr, uid, 'web.base.url')
    if host[-1:] == '/':
        host = host[:-1]
    _link = str('%s/nfe/xml/%s/l/%s-procNFe.xml' % (host, base, nf.get('nfe_chave_acesso')))

    vals['fisnfc_link_xml'] = _link

def _get_forma_pgto_ids(self, cr, uid, name, context=None):
    context = context and context.copy() or {}
    if 'fisnfc_imprimi_promissoria' not in name:
        return []

    obj_foma_pgto = self.pool.get('adm.forma.pgto')
    return obj_foma_pgto.search(cr, uid, [('admfpg_promissoria', '=', True)], context=context)


def _get_dict_itens(self, cr, uid, item_ids, name, context, load):
    context = context and context.copy() or {}
    if 'fisnfc_valida_item_rateado_nf' not in name:
        return {}

    res = {}
    campos = ['vr_total_item_rateado']
    for i in self.pool.get('estoque.movimento.itens').read(cr, uid, item_ids, campos, context,
                                                           load):
        res.update({i.get('id'): i})

    return res


def _get_dict_condicoes(self, cr, uid, condicao_ids, name, context, load):
    context = context and context.copy() or {}
    if 'fisnfc_valida_item_rateado_nf' not in name and 'fisnfc_imprimi_promissoria' not in name:
        return {}

    res = {}
    map(lambda x: res.update({x.get('id'):x.get('forma_pagto_id') or 0}),
        self.pool.get('fiscal.condicao.pagamento').read(cr, uid, condicao_ids, ['forma_pagto_id'],
                                                        context, load))
    return res


def _get_dict_natureza_cfop(self, cr, uid, natureza_ids, name, context, load):
    context = context and context.copy() or {}
    if 'fisnfc_nf_devolucao' not in name and 'fisnfc_cfop_gnre' not in name:
        return {}, {}

    dict_nat = {}
    cfop_ids = []
    for i in self.pool.get('fiscal.cfop.natureza').read(cr, uid, natureza_ids, ['cfop_id'], context,
                                                   load):
        dict_nat.update({i.get('id'): i.get('cfop_id')})
        cfop_ids.append(i.get('cfop_id'))

    dict_cfop = {}
    map(lambda x: dict_cfop.update({x.get('id'): x}),
        self.pool.get('fiscal.cfop').read(cr, uid, cfop_ids, ['movimento_tipo', 'codigo'], context))
    return dict_nat, dict_cfop


def _get_dict_modelo(self, cr, uid, modelo_ids, name, context):
    context = context and context.copy() or {}
    if 'fisnfc_modelo_serie' not in name and 'fisnfc_modelo_nf_tipo' not in name:
        return {}

    res = {}
    map(lambda x: res.update({x.get('id'): x}),
        self.pool.get('fiscal.modelo.nf').read(cr, uid, modelo_ids, ['fismnf_tipo',
                                                                     'fismnf_codigo'], context))
    return res


def _get_dict_modelo_serie(self, cr, uid, modelo_serie_ids, name, context):
    context = context and context.copy() or {}
    if 'fisnfc_modelo_serie' not in name:
        return {}

    res = {}
    map(lambda x: res.update({x.get('id'): x.get('serie_nf')}),
        self.pool.get('fiscal.modelo.nf.series').read(cr, uid, modelo_serie_ids, ['serie_nf'],
                                                      context))
    return res


def _get_dict_pessoas(self, cr, uid, pessoa_ids, name, context):
    context = context and context.copy() or {}
    if 'name' not in name or not pessoa_ids:
        return {}

    res = {}
    context['super_read_adm_pessoa'] = True
    map(lambda x: res.update({x.get('id'): x.get('nome')}),
        self.pool.get('adm.pessoa').read(cr, uid, pessoa_ids, ['nome'], context))

    return res


def _get_parametros_dee(self, cr, uid, name, context):
    context = context and context.copy() or {}
    if 'fisnfc_parametro_danfe' not in name and 'fisnfc_ativar_entrega' not in name \
            and 'fisnfc_ativar_coleta' not in name \
            and 'fisnfc_calcula_peso_automaticamente' not in name:
        return False, '', '', {}

    obj_parametro = self.pool.get('fiscal.parametro')
    fiscal_parametro_ids = obj_parametro.search(cr, uid, [], context=context)
    danf = False
    endereco_entrega = ''
    endereco_coleta = ''
    if fiscal_parametro_ids:
        campos = ['impressora_danfe', 'quantidade_vias_danfe', 'endereco_entrega',
                  'endereco_coleta', 'fispar_calcula_peso_item_automatico']
        parametro = obj_parametro.read(cr, uid, fiscal_parametro_ids[0], campos, context)
        endereco_entrega = parametro.get('endereco_entrega') or ''
        endereco_coleta = parametro.get('endereco_coleta') or ''
        # Verifica se existe configuracao para impressao do DANFE no parâmetro, se existir envia
        # direto para impressora informada.
        if parametro.get('impressora_danfe') and parametro.get('quantidade_vias_danfe') > 0:
            danf = True

    return danf, endereco_entrega, endereco_coleta, parametro


def _fisnfc_get_dados_no_store(self, cr, uid, ids, name, arq, context=None):
    """
    Calcula os valores dos campos funcionais nao estoriados
    """
    context = context and context.copy() or {}
    res = {}
    load = '_classic_write'
    forma_pgto_ids = _get_forma_pgto_ids(self, cr, uid, name, context)

    campos = ['estoque_movimento_itens_ids', 'modelo_nf_id', 'vr_icms_substituicao','numero_nf',
              'nf_condicao_ids', 'tipo', 'state', 'nf_propria', 'natureza_id', 'destinatario_id',
              'modelo_nf_series_id', 'vr_total_nf', 'data_emissao', 'serie_nf', 'nfe_chave_acesso',
              'fisnfc_valor_fcp_st', 'fisnfc_valor_icms_desonerado', 'vr_ipi', 'modelo_55']
    dados = self.read(cr, uid, ids, campos, context, load)
    item_ids = []
    modelo_ids = []
    modelo_serie_ids = []
    condicao_ids = []
    natureza_ids = []
    pessoa_ids = []
    for i in dados:
        item_ids.extend(i.get('estoque_movimento_itens_ids'))
        i.get('modelo_nf_id') and modelo_ids.append(i.get('modelo_nf_id'))
        i.get('modelo_nf_series_id') and modelo_serie_ids.append(i.get('modelo_nf_series_id'))
        condicao_ids.extend(i.get('nf_condicao_ids'))
        natureza_ids.append(i.get('natureza_id'))
        i.get('destinatario_id') and pessoa_ids.append(i.get('destinatario_id'))

    dict_itens = _get_dict_itens(self, cr, uid, item_ids, name, context, load)
    dict_condicoes = _get_dict_condicoes(self, cr, uid, condicao_ids, name, context, load)
    dict_natureza, dict_cfop = _get_dict_natureza_cfop(self, cr, uid, natureza_ids, name, context,
                                                       load)
    dict_modelo = _get_dict_modelo(self, cr, uid, modelo_ids, name, context)

    dict_modelo_serie = _get_dict_modelo_serie(self, cr, uid, modelo_serie_ids, name, context)
    dict_pessoas = _get_dict_pessoas(self, cr, uid, pessoa_ids, name, context)
    dict_msg = _fisnfc_get_msg(self, cr, uid, pessoa_ids, name, context)

    danf, endereco_entrega, endereco_coleta, parametro = _get_parametros_dee(self,cr, uid, name,
                                                                             context)

    dict_titulos = _fisnfc_get_dict_titulos(self, cr, uid, ids, name, context)
    dict_permite_alterar = _fisnfc_get_dict_premite_alterar(self, cr, uid, ids, name, context)
    permisao_voltar = self.fisnfc_permissao_para_voltar_esboco(cr, uid, name=name)
    for nf in dados:
        vals = {}
        dados_cfop = dict_cfop.get(dict_natureza.get(nf.get('natureza_id')))
        id = nf.get('id')
        importacao = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, nf.get('natureza_id'), context).cfop_id.codigo[0] == '3'
        _set_fisnfc_valida_item_rateado_nf(vals, nf, dict_itens, name, importacao)
        _set_fisnfc_imprimi_promissoria(vals, nf, dict_condicoes, forma_pgto_ids, name)
        _set_observacao_pessoa(vals, nf, dict_msg, name)
        _set_fisnfc_nf_devolucao(vals, nf, dados_cfop, name)
        _set_fisnfc_cfop_gnre(vals, dados_cfop, name)
        _set_fisnfc_modelo(vals, nf, dict_modelo, name)
        _set_fisnfc_modelo_serie(vals, nf, dict_modelo, dict_modelo_serie, name)
        _set_fisnfc_modelo_nf_tipo(vals, nf, dict_modelo, name)
        _set_name(vals, nf, dict_pessoas, name)
        _set_fisnfc_parametro_danfe(vals, danf, name)
        _set_fisnfc_ativar_entrega(vals, endereco_entrega, name)
        _set_fisnfc_ativar_coleta(vals, endereco_coleta, name)
        _set_fisnfc_titulos_ids(vals, id, dict_titulos, name)
        _set_fisnfc_permite_alterar_condicoes_pgto(vals, id, dict_permite_alterar, name)
        _set_fisnfc_calcula_peso_automaticamente(vals, parametro, name)
        _set_fisnfc_permissao_voltar_esboco(vals, permisao_voltar, name)
        _set_fisnfc_link_xml(self, cr, uid, vals, nf, name, context=context)
        res[id] = vals

    return res


def _fisnfc_get_dados(self, cr, uid, ids, name, arq, context=None):
    """
    Calcula os valores dos campos vr_total_nf, vr_ipi, vr_total_produtos, vr_total_servicos
    Estes campos sao calculados em uma so funcao pelo fato do campo vr_total_nf, precisar que todos
    os demais ja tenham sido calculados.
    """
    context = context and context.copy() or {}
    res = {}
    load = '_classic_write'
    obj_encargo = self.pool.get('fiscal.encargo')
    ipi_encargo_ids = obj_encargo.search(cr, uid, [('sigla', 'ilike', 'IPI')], context=context)
    icms_encargo_ids = obj_encargo.search(cr, uid, [('sigla', 'ilike', 'ICMS')], context=context)
    ii_encargo_ids = obj_encargo.search(cr, uid, [('sigla', 'ilike', 'II')], context=context)
    pis_encargo_ids = obj_encargo.search(cr, uid, [('sigla', 'ilike', 'PIS')], context=context)
    cofins_encargo_ids = obj_encargo.search(cr, uid, [('sigla', 'ilike', 'COFINS')], context=context)
    bonificacao_cfop_ids = self.pool.get('fiscal.cfop').search(
        cr, uid, [('codigo', 'in', ('1910', '2910', '1911', '2911'))], context=context)
    campos = ['estoque_movimento_itens_ids', 'tipo_frete', 'modelo_nf_id', 'vr_seguro', 'vr_frete',
              'vr_desconto', 'vr_despesas_acessorias', 'finalidade_emissao', 'vr_icms_substituicao',
              'deducao_itens_ids', 'nf_condicao_ids', 'tipo', 'state', 'nf_propria', 'natureza_id']
    sql = """
        SELECT
            nf.id AS nf_id,
            CASE
                WHEN fc.movimento_tipo = 'T' AND ape.cpf_cnpj != apd.cpf_cnpj 
                    AND left(ape.cpf_cnpj, 10) = left(apd.cpf_cnpj, 10)
                        THEN TRUE
                ELSE
                    FALSE
            END AS nf_transferencia
        FROM
            fiscal_nf_capa nf
        LEFT JOIN
            fiscal_cfop_natureza fcn ON fcn.id = nf.natureza_id
        LEFT JOIN
            fiscal_cfop fc ON fcn.cfop_id = fc.id
        LEFT JOIN
            adm_pessoa apd ON apd.id = nf.destinatario_id
        LEFT JOIN
            res_company rc ON rc.id = nf.company_id
        LEFT JOIN
            adm_pessoa ape ON ape.id = rc.rescon_pessoa_id
        WHERE
            nf.id IN ({nf_ids})
    """.format(nf_ids=ids and ', '.join(map(lambda n: str(n), ids)) or '-1')
    cr.execute(sql)
    nf_transferencia_list = cr.dictfetchall()
    nf_transferencia_dados = {}
    map(lambda n: nf_transferencia_dados.update({n.get('nf_id'): n.get('nf_transferencia')}),
        nf_transferencia_list)

    dados = self.read(cr, uid, ids, campos, context, load)
    item_ids = []
    deducao_ids = []
    modelo_ids = []
    condicao_ids = []
    for i in dados:
        item_ids.extend(i.get('estoque_movimento_itens_ids') or [])
        i.get('deducao_itens_ids') and deducao_ids.extend(i.get('deducao_itens_ids'))
        i.get('modelo_nf_id') and modelo_ids.append(i.get('modelo_nf_id'))
        i.get('nf_condicao_ids') and condicao_ids.extend(i.get('nf_condicao_ids'))
    dict_itens = {}
    produto_ids = []
    tributacao_ids = []
    context['super_read_estoque_movimento_itens'] = False
    campos = ['vr_total_item_rateado', 'produto_id', 'vr_total_item', 'tributacoes_ids', 'cfop_id',
              'estmvi_sped_fiscal_e200_manual']
    item_gera_e200 = False
    if item_ids:
        context_item = context.copy()
        context_item['super_read_estoque_movimento_itens'] = True
        for i in self.pool.get('estoque.movimento.itens').read(
                cr, uid, item_ids, campos, context_item, load):
            dict_itens.update({i.get('id'): i})
            produto_ids.append(i.get('produto_id'))
            tributacao_ids.extend(i.get('tributacoes_ids') or [])
            if i.get('estmvi_sped_fiscal_e200_manual'):
                item_gera_e200 = True

    dict_produtos = {}
    map(lambda x: dict_produtos.update({x.get('id'): x.get('tipo')}),
        self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['tipo'], context))
    dict_trib = {}
    cst_ids = []
    if tributacao_ids:
        campos = ['encargo_id', 'vr_encargo', 'cst_id', 'vr_base_calculo', 'vr_icms_substituicao',
                  'fisnit_vr_fcp_st', 'fisnit_vr_icms_desonerado']
        for i in self.pool.get('fiscal.nf.item.tributacao').read(
                cr, uid, tributacao_ids, campos, context, load):
            dict_trib.update({i.get('id'): i})
            i.get('cst_id') and cst_ids.append(i.get('cst_id'))
    dict_csts = {}
    if cst_ids:
        map(lambda x: dict_csts.update({x.get('id'): x.get('tipo')}),
            self.pool.get('fiscal.cst').read(cr, uid, cst_ids, ['tipo'], context))
    dict_modelo = {}
    if modelo_ids:
        map(lambda x: dict_modelo.update({x.get('id'):x.get('fismnf_tipo')}),
            self.pool.get('fiscal.modelo.nf').read(cr, uid, modelo_ids, ['fismnf_tipo'], context))
    dict_deducoes = {}
    if deducao_ids:
        map(lambda x: dict_deducoes.update({x.get('id'):x.get('valor') or 0}),
            self.pool.get('fiscal.deducoes.itens').read(cr, uid, deducao_ids, ['valor'], context))
    dict_condicoes = {}
    map(lambda x: dict_condicoes.update({x.get('id'):x.get('forma_pagto_id') or 0}),
        self.pool.get('fiscal.condicao.pagamento').read(cr, uid, condicao_ids, ['forma_pagto_id'],
                                                        context))
    vr_total_deducao = 0
    for nf in dados:
        importacao = nf['natureza_id'] and self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, nf['natureza_id'], context).cfop_id.codigo[0] == '3'
        importacao = nf['tipo'] == 'E' and nf['nf_propria'] and importacao
        vals = {}
        vr_total_produtos = vr_total_servicos = vr_ipi = vr_icms = vr_base_icms = 0
        vr_bonificacao = vr_fcp_st = vr_icms_desonerado = outros = 0

        for i in nf.get('estoque_movimento_itens_ids'):
            item = dict_itens.get(i)
            if dict_produtos.get(item.get('produto_id')) == 'P':
                vr_total_produtos += item.get('vr_total_item')
            else:
                vr_total_servicos += item.get('vr_total_item')
            if item.get('cfop_id') in bonificacao_cfop_ids:
                vr_bonificacao += item.get('vr_total_item_rateado')
            for x in item.get('tributacoes_ids'):
                trib = dict_trib.get(x)
                if trib.get('encargo_id') in ipi_encargo_ids:  # IPI
                    vr_ipi += trib.get('vr_encargo')
                    if item.get('cfop_id') in bonificacao_cfop_ids:
                        vr_bonificacao += trib.get('vr_encargo')
                elif trib.get('encargo_id') in icms_encargo_ids:  # ICMS
                    vr_icms_desonerado += trib.get('fisnit_vr_icms_desonerado') or 0
                    if dict_csts.get(trib.get('cst_id')) in 'TU':
                        if dict_csts.get(trib.get('cst_id')) == 'T':
                            vr_icms += trib.get('vr_encargo')
                            vr_base_icms += trib.get('vr_base_calculo')
                        vr_fcp_st += trib.get('fisnit_vr_fcp_st') or 0
                        if item.get('cfop_id') in bonificacao_cfop_ids:
                            vr_bonificacao += trib.get('fisnit_vr_fcp_st') or 0
                    if importacao:
                        outros += trib.get('vr_encargo')
                elif trib.get('encargo_id') in ii_encargo_ids:  # II
                    if importacao:
                        outros += trib.get('vr_encargo')
                elif trib.get('encargo_id') in ii_encargo_ids:  # II
                    if importacao:
                        outros += trib.get('vr_encargo')
                elif trib.get('encargo_id') in pis_encargo_ids:  # II
                    if importacao:
                        outros += trib.get('vr_encargo')
                elif trib.get('encargo_id') in cofins_encargo_ids:  # II
                    if importacao:
                        outros += trib.get('vr_encargo')
                if item.get('cfop_id') in bonificacao_cfop_ids:
                    vr_bonificacao += trib.get('vr_icms_substituicao')

        vals['fisnfc_valor_fcp_st'] = vr_fcp_st
        vals['vr_total_produtos'] = vr_total_produtos
        vals['vr_total_servicos'] = vr_total_servicos
        vals['vr_ipi'] = vr_ipi
        vals['vr_icms'] = vr_icms
        vals['vr_base_icms'] = vr_base_icms
        vals['fisnfc_valor_bonificado'] = vr_bonificacao
        vals['fisnfc_valor_icms_desonerado'] = vr_icms_desonerado
        vals['fisnfc_parametro_e200'] = item_gera_e200
        vals['fisnfc_nf_transferencia'] = nf_transferencia_dados.get(nf.get('id')) or False

        # Dados vr_total_nf
        if nf.get('tipo_frete') in ('', 'D') or dict_modelo.get(nf.get('modelo_nf_id')) == 'T':
            vr_frete = nf.get('vr_frete') or 0
        else:
            vr_frete = 0
        vr_seguro = nf.get('vr_seguro') or 0
        vr_desconto = (nf.get('vr_desconto') or 0)
        vr_desconto += vr_icms_desonerado  # ICMS desonerado somado ao desconto
        vr_despesas_acessorias = nf.get('vr_despesas_acessorias') or 0
        vr_icms_substituicao = nf.get('vr_icms_substituicao') or 0
        vr_deducao = sum(map(lambda x: dict_deducoes.get(x), nf.get('deducao_itens_ids'))) or 0
        vals['vr_total_deducao'] = vr_deducao
        if context.get('total_liquido'):
            vr_total_deducao = vr_deducao
        vr_total_nf = vr_total_produtos + vr_total_servicos
        vr_total_nf = (vr_total_nf + vr_seguro + vr_frete + vr_despesas_acessorias + vr_ipi +
                       vr_icms_substituicao - vr_total_deducao - vr_desconto + vr_fcp_st)
        vals['vr_total_nf'] = vr_total_nf + outros
        res[nf.get('id')] = vals.copy()

    return res


def _fisnfc_get_nf_exportacao(self, cr, user, ids, name, arg, context=None):
    """
    Identifica se a nota fiscal eh de exportacao.
    
    :param cr: a linha atual do cursor do banco de dados
    :param user: ID do usuario utilizado para a verificacao de perfil
    :param ids: lista dos IDs
    :param name: lista com o nome dos campos
    :param arg: dicionario com os argumentos e valores
    :param context: um dicionario padrao para valores contextuais
    :type cr: object
    :type user: int
    :type ids: list
    :type name: list
    :type arg: dict
    :type context: dict
    :return: verdadeiro se a nota fiscal for de exportacao (CFOP inicia com "7")
    :rtype: bool
    """
    context = context and context.copy() or dict()
    result = dict().fromkeys(ids, False)
    for nf in self.browse(cr, user, ids, context):
        result[nf.id] = nf.natureza_id and nf.natureza_id.cfop_id and \
                        nf.natureza_id.cfop_id.codigo[0] == '7'

    return result

def _fisnfc_get_nf_importacao(self, cr, user, ids, name, arg, context=None):
    """
    Identifica se a nota fiscal eh de importacao.

    :param cr: a linha atual do cursor do banco de dados
    :param user: ID do usuario utilizado para a verificacao de perfil
    :param ids: lista dos IDs
    :param name: lista com o nome dos campos
    :param arg: dicionario com os argumentos e valores
    :param context: um dicionario padrao para valores contextuais
    :type cr: object
    :type user: int
    :type ids: list
    :type name: list
    :type arg: dict
    :type context: dict
    :return: verdadeiro se a nota fiscal for de exportacao (CFOP inicia com "7")
    :rtype: bool
    """
    context = context and context.copy() or dict()
    result = dict().fromkeys(ids, False)
    for nf in self.browse(cr, user, ids, context):
        result[nf.id] = nf.natureza_id and nf.natureza_id.cfop_id and \
                        nf.natureza_id.cfop_id.codigo[0] == '3'

    return result


def fisnfc_get_nf_parcelas(self, cr, user, ids, name, arg, context=None):
    """
    Retorna o ID das parcelas em funcao das condicoes de pagamento.

    :param cr: a linha atual do cursor do banco de dados
    :param user: ID do usuario utilizado para a verificacao de perfil
    :param ids: lista dos IDs
    :param name: lista com o nome dos campos
    :param arg: dicionario com os argumentos e valores
    :param context: um dicionario padrao para valores contextuais
    :type cr: object
    :type user: int
    :type ids: list
    :type name: list
    :type arg: dict
    :type context: dict
    :return: lista com o ID das parcelas
    :rtype: list
    """

    context = context and context.copy() or {}
    result = {}
    parcelas = []
    dados_nf = self.read(cr, user, ids, ['nf_condicao_ids'], context)
    condicao_ids = []
    map(lambda x: condicao_ids.extend(x.get('nf_condicao_ids')), dados_nf)
    dict_condiao = {}
    map(lambda x: dict_condiao.update({x.get('id'): x}),
        self.pool.get('fiscal.condicao.pagamento').read(cr, user, condicao_ids, ['nf_parcela_ids'],
                                                        context))
    for nf in dados_nf:
        for condicao_id in nf.get('nf_condicao_ids'):
            condicao = dict_condiao.get(condicao_id)
            condicao.get('nf_parcela_ids') and parcelas.extend(condicao.get('nf_parcela_ids'))

        result[nf.get('id')] = parcelas

    return result


def fisnfc_set_nf_parcelas(self, cr, user, ids, name, values, arg, context=None):
    """
    Atualiza os valores das parcelas.

    :param cr: a linha atual do cursor do banco de dados
    :param user: ID do usuario utilizado para a verificacao de perfil
    :param ids: lista dos IDs
    :param name: nome dos campo
    :param values: lista com os valores do campo (one2many)
    :param arg: dicionario com os argumentos e valores
    :param context: um dicionario padrao para valores contextuais
    :type cr: object
    :type user: int
    :type ids: list
    :type name: str
    :type values: list
    :type arg: dict
    :type context: dict
    :return: verdadeiro se a atualizacao das parcelas foi efetuada com sucesso.
    :rtype: bool
    """

    for value in filter(lambda f: f[0] == 1, values):
        self.pool.get('fiscal.nf.parcelas').write(cr, user, value[1], value[2], context)

    return True


def _fisten_get_tipo_trib_cst(self, cr, uid, ids, name, arq, context=None):
    """
    Retorna o quais tipos de CST devem ser filtrados no domain, de acordo com
    o tipo de tributacao no cadastro do item
    """
    context = context and context.copy() or {}
    res = {}
    obj_fistribenc = self.pool.get('fiscal.tributacao.encargos')
    for i in self.browse(cr, uid, ids):
        res[i.id] = obj_fistribenc.fisten_get_tipo_trib_cst(cr, uid, ids, i.tipo_tributacao,
                                                            context)
    return res


def _get_trib_dados(self, cr, uid, dados_item_tribs, classe, campos=[], context=None):
    context = context and context.copy() or {}
    trib_classe_ids = {}
    classe_ids = []
    obj_classe = {'nf': self.pool.get('fiscal.nf.capa'),
                  'cst': self.pool.get('fiscal.cst'),
                  'mov_itens': self.pool.get('estoque.movimento.itens'),
                  'encargo': self.pool.get('fiscal.encargo'),
                  'tributacao': self.pool.get('fiscal.tributacao'),
                 }.get(classe)
    campo_id = {'nf': 'fisnit_capa_id',
                'cst': 'cst_id',
                'mov_itens': 'movimento_itens_id',
                'encargo': 'encargo_id',
                'tributacao': 'tributacao_id',
               }.get(classe)
    for trib in dados_item_tribs:
        classe_id = trib.get(campo_id)
        if classe_id:
            classe_ids.append(classe_id)
        if not trib_classe_ids.get(classe_id):
            trib_classe_ids[classe_id] = [trib.get('id')]
        else:
            trib_classe_ids[classe_id].append(trib.get('id'))

    dados_classe = obj_classe.read(cr, uid, classe_ids, campos, context=context,
                                   load='_classic_write')
    res = {}
    for dado_classe in dados_classe:
        for trib_id in trib_classe_ids.get(dado_classe.get('id')):
            res[trib_id] = dado_classe

    return res


def _fisnit_get_dados_fiscais(self, cr, uid, ids, name, arq, context=None):
    """
    Retorna qual tipo de CST devem ser filtrados no domain, de acordo com
    o tipo de nota fiscal ex:(Entrada, Saida)
    """
    context = context and context.copy() or {}
    res = {}
    if not ids:
        return res

    if not isinstance(ids, list):
        ids = [ids]

    campos = ['vr_base_calculo', 'vr_reducao_base', 'cst_id', 'movimento_itens_id', 'vr_encargo',
              'fisnit_capa_id', 'vr_icms_substituicao', 'encargo_id', 'tributacao_id',
              'vr_base_icms_substituicao']
    dados_item_tribs = self.read(cr, uid, ids, campos, context=context, load='_classic_write')
    campos_nf = ['natureza_id', 'vr_total_nf', 'tipo', 'destinatario_id']
    trib_nf_dados = _get_trib_dados(self, cr, uid, dados_item_tribs, 'nf', campos_nf,
                                    context=context)
    context_aux = context.copy()
    context_aux['super_read_estoque_movimento_itens'] = True
    campos_itens = ['vr_total_item_rateado', 'nf_id', 'cfop_id']
    trib_itens_dados = _get_trib_dados(self, cr, uid, dados_item_tribs, 'mov_itens', campos_itens,
                                       context=context_aux)
    trib_encargo_dados =_get_trib_dados(self, cr, uid, dados_item_tribs, 'encargo', ['sigla'],
                                        context=context)
    trib_cst_dados = _get_trib_dados(self, cr, uid, dados_item_tribs, 'cst', ['tipo', 'codigo'],
                                       context=context)
    for item_trib in dados_item_tribs:
        trib_id = item_trib.get('id')
        base_calculo = item_trib.get('vr_base_calculo')
        base_calculo_reduzida = item_trib.get('vr_reducao_base')
        total = base_calculo + base_calculo_reduzida
        tipo_cst = ''
        codigo_cst = ''
        if trib_cst_dados.get(trib_id):
            tipo_cst = trib_cst_dados.get(trib_id).get('tipo')
            codigo_cst = trib_cst_dados.get(trib_id).get('codigo')
        if item_trib.get('movimento_itens_id') and trib_nf_dados.get(trib_id):  # nota com itens
            cfop_id = trib_itens_dados.get(trib_id).get('cfop_id')
            vr_contabil = trib_itens_dados.get(trib_id).get('vr_total_item_rateado')
            tipo = trib_nf_dados[trib_id].get('tipo')
        elif trib_nf_dados.get(trib_id):  # nota sem itens
            cfop_id = self.pool.get('fiscal.cfop.natureza').browse(
                cr, uid, trib_nf_dados[trib_id].get('natureza_id'), context).cfop_id.id
            vr_contabil = trib_nf_dados[trib_id].get('vr_total_nf')
            tipo = trib_nf_dados[trib_id].get('tipo')

        else: # cupom
            cfop_id = trib_itens_dados.get(trib_id).get('cfop_id')
            vr_contabil = trib_itens_dados.get(trib_id).get('vr_total_item_rateado')
            tipo = 'S'

        dict_campos = {'fisnit_tipo_movimento_cst': tipo,
                       'fisnit_tipo_cst': tipo_cst,
                       'fisnit_vr_tributado': 0, 'fisnit_vr_isento': 0, 'fisnit_vr_outros': 0,
                       'fisnit_vr_diferido': 0, 'fisnit_vr_substituicao': 0,
                       'fisnit_vr_servico': 0, 'fisnit_vr_contabil': vr_contabil,
                       'fisnit_cfop_id': cfop_id, 'tipo_substituicao': False}
        if tipo_cst == 'T':  # Tributado
            dict_campos['fisnit_vr_tributado'] = base_calculo
            dict_campos['fisnit_vr_outros'] = base_calculo_reduzida

        elif tipo_cst in ('U', 'TU'):  # Substituicao
            dict_campos['fisnit_vr_substituicao'] = item_trib.get('vr_icms_substituicao')
            dict_campos['fisnit_vr_outros'] = total
            if item_trib.get('fisnit_capa_id') and \
                                         all([tipo == 'E', codigo_cst in ('10', '30', '60', '70')]):
                dict_campos['tipo_substituicao'] = True
            elif codigo_cst in ('10', '30', '60', '70'):
                dict_campos['tipo_substituicao'] = True
        elif tipo_cst == 'I':  # Isento
            dict_campos['fisnit_vr_isento'] = total
        elif tipo_cst == 'O':  # Outros
            dict_campos['fisnit_vr_outros'] = total
        elif tipo_cst == 'D':  # Diferido
            dict_campos['fisnit_vr_diferido'] = total
        elif tipo_cst == 'S':  # Servico
            dict_campos['fisnit_vr_servico'] = total
        if trib_encargo_dados.get(trib_id).get('sigla') == 'IPI':
            dict_campos['fisnit_vr_outros'] += item_trib.get('vr_encargo')

        dict_campos['fisnit_percentual_base_zero'] = True
        res[trib_id] = dict_campos

    return res


def _fisnfco_get_dados(self, cr, uid, ids, name, arq, context=None):
    context = context and context.copy() or {}
    res = {}
    for nf_consumo in self.read(cr, uid, ids, ['chave_nfe'], context=context):
        res[nf_consumo.get('id')] = nf_consumo.get('chave_nfe') and {
            'fisnfco_numero_nf': nf_consumo.get('chave_nfe')[25:34],
            'fisnfco_serie_nf': nf_consumo.get('chave_nfe')[22:25]
        } or {}
    return res


def _fisnfco_get_nf(self, cr, uid, ids, name, arq, context=None):
    """
    Este metodo retorna true se chave da nota ja existir na base dados ou false caso contrario
    """
    context = context and context.copy() or {}
    res = {}
    if len(ids) > 1:
        nfe_consumo_ids = str(tuple(ids))
    else:
        nfe_consumo_ids = str(tuple(ids)).replace(',', '')
    sql = """
        SELECT
            id,
            CASE
                WHEN chave_nfe IS NOT NULL
                    THEN (CASE
                            WHEN (
                                SELECT
                                    id
                                FROM
                                    fiscal_nf_capa
                                WHERE
                                    state != 'C'
                                    AND tipo = 'E'
                                    AND nfe_chave_acesso = chave_nfe
                                LIMIT 1
                            ) IS NOT NULL THEN True
                            WHEN (
                                SELECT
                                    id
                                FROM
                                    fiscal_import_nfe_osv
                                WHERE
                                    ide_id LIKE '%%' || chave_nfe
                                LIMIT 1
                            ) IS NOT NULL THEN True
                            ELSE False
                            END
                        )
                ELSE False
            END AS valor
        FROM
            fiscal_nfe_consumo
        WHERE
            id in %s
    """ % nfe_consumo_ids
    cr.execute(sql)
    nfe_consumo_dados = cr.dictfetchall()
    map(lambda i: res.update({i.get('id'): i.get('valor')}), nfe_consumo_dados)
    return res


def _fisnfco_dias_de_ciencia(self, cr, uid, ids, name, arq, context=None):
    context = context.copy() and context or {}
    res = {}
    hoje = datetime.today()
    if not ids:
        return res
    for i in self.browse(cr, uid, ids, context):
        if i.state == 'CI' and i.fisnfco_data_ciencia_oper:
            try:
                data_ciencia = datetime.strptime(i.fisnfco_data_ciencia_oper, '%Y-%m-%d')
                diferenca = hoje - data_ciencia
                dias = diferenca.days
                if dias > 1:
                    dias = str(dias) + " Dias"
                else:
                    dias = str(dias) + " Dia"
                res[i.id] = dias
            except:
                res[i.id] = False
        else:
            res[i.id] = False
    return res


def fistin_get_encargo(encargo):
    if isinstance(encargo, (list, tuple)):
        # se existe mais de um deve retornar o primeiro
        return encargo[0]
    else:
        return encargo


def fissup_get_parametros(self, cr, uid, ids, name, arq, context=None):
    """
    Funcao responsavel por retornar parametros necessarios para a sugestao de precos
    """
    context = context and context.copy() or {}
    res = {}
    parametros = self.fissup_get_parametros(cr, uid, context=None)
    for i in self.browse(cr, uid, ids, context=context):
        res[i.id] = parametros
    return res


def fissup_check_tributacao_saida(self, cr, uid, ids, field_name, arg, context=None):
    """
    Metodo do campo funcao 'fissup_tributacao_saida'. Seta True caso todos os itens tenha a
    tributacao de saida cadastrada e tenha selecionado para usar a margem bruta calculada.
    Senao, False.
    :param cr: cursor no banco de dados.
    :type cr: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type uid: int
    :param ids: lista de id da tabela desse objeto.
    :type ids: list
    :param field_name: descricao do campo funcao.
    :type field_name: str
    :param arg: lista de argumentos de pesquisa.
    :type arg: list
    :param context: dicionario com valores contextuais.
    :type context: dict
    :return: dicionario com id dos registros e seus respectivos valores.
    :rtype: dict
    """
    context = context and context.copy() or {}
    res = {}
    for sugestao in self.browse(cr, uid, ids, context):
        val = True
        itens_sem_tributacao_saida = filter(lambda i: not i.fisspi_tributacao_saida,
                                            sugestao.fissup_sugestao_preco_itens_ids)
        if not sugestao.fissup_usar_margem_bruta_calculada and itens_sem_tributacao_saida:
            val = False
        res[sugestao.id] = val
    return res


def fisspi_get_codigo(self, cr, uid, ids, name, arq, context=None):
    """
        Funcao responsavel por retornar qual o codigo do produto sera utilizado
        na sugestao de precos (I-Itens, F-Fabricante, B-Barras)
    """
    context = context and context.copy() or {}
    obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
    res = {}
    company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                context=context)
    for i in self.browse(cr, uid, ids, context=context):
        if i.fisspi_capa_id.fissup_codigo == 'I':
            res[i.id] = i.fisspi_produto_id.codigo
        elif i.fisspi_capa_id.fissup_codigo == 'F':
            res[i.id] = i.fisspi_grade_id.codigo_no_fabricante or ''
        else:
            res[i.id] = ''
            codigo_barra_ids = obj_automacao_codigo_barra.search(
                cr, uid, [('produto_id', '=', i.fisspi_produto_id.id),
                          ('grade_id', '=', i.fisspi_grade_id.id),
                          ('company_id', '=', company_id)], context=context)
            if not codigo_barra_ids:
                continue
            res[i.id] = obj_automacao_codigo_barra.browse(cr, uid, codigo_barra_ids[0],
                                                          context).codigo_barra or ''
    return res


def fisspi_check_tributacao_saida(self, cr, uid, ids, field_name, arg, context=None):
    """
    Metodo do campo funcao 'fisspi_tributacao_saida'. Seta True caso o item tenha a tributacao de
    saida cadastrada e tenha selecionado para usar a margem bruta calculada. Senao, False.
    :param cr: cursor no banco de dados.
    :type cr: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type uid: int
    :param ids: lista de id da tabela desse objeto.
    :type ids: list
    :param field_name: descricao do campo funcao.
    :type field_name: str
    :param arg: lista de argumentos de pesquisa.
    :type arg: list
    :param context: dicionario com valores contextuais.
    :type context: dict
    :return: dicionario com id dos registros e seus respectivos valores.
    :rtype: dict
    """
    context = context and context.copy() or {}
    res = {}
    obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
    encargo_ids = self.pool.get('fiscal.encargo').search(cr, uid, [('sigla', '=', 'ICMS')],
                                                         context=context)
    for item in self.browse(cr, uid, ids, context):
        val = True
        args_trib_enc = [('produto_id', '=', item.fisspi_produto_id.id),
                         ('encargo_id', 'in', encargo_ids),
                         ('tipo_tributacao', '=', 'T')]
        tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(cr, uid, args_trib_enc,
                                                                       context=context)
        if not tributacao_encargo_ids and not item.fisspi_capa_id.\
                fissup_usar_margem_bruta_calculada:
            val = False
        res[item.id] = val
    return res


def _fisnfc_get_vendedor_obrigatorio(self, cr, user, ids, name=None, arg=None, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos como tipo funcao.
    Retorna verdadeiro/falso em funcao do parametro informado no campo ´estpar_vendedor_obrigatorio´

    :param cr: a linha atual do cursor do banco de dados,
    :param user: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param context: um dicionario padrao para valores contextuais,
    :type cr: cursor
    :type user: int
    :type ids: list
    :type context: dict
    :return: valores ajustados dos campos.
    :rtype: bool
    """
    context = context and context.copy() or dict()
    if isinstance(ids, list):
        result = dict().fromkeys(ids, self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'estoque', [], context).get('estpar_vendedor_obrigatorio'))
    else:
        result = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'estoque', [], context).get('estpar_vendedor_obrigatorio')

    return result


def fisnfc_get_natureza_padrao(self, cr, uid, context=None):
    context = context and context.copy() or {}
    res = False
    fiscal_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
        cr, uid, 'fiscal', ['fispar_nfce_natureza_id'], context=context)
    if context.get('nota_fiscal_consumidor'):
        res = fiscal_parametro_dados.get('fispar_nfce_natureza_id') or False
    return res


# Funcoes da classe fiscal_parametro


def fispar_verifica_empresa_industria(self, cr, uid, ids, field_name, arg, context=None):
    context = context and context.copy() or {}
    res = {}
    brw_fiscal_parametro = self.browse(cr, uid, ids, context)
    map(lambda p: res.update(
        {p.id: p.company_id.rescon_pessoa_id.admpes_jur_tipo_atividade == 'I'}),
        brw_fiscal_parametro)
    return res


# Códigos da natureza de crédito e seus respectivos CFOP's de acordo com a tabela TABELA DE CÓDIGOS
# DA NATUREZA DO CRÉDITO (4.3.7)
fiscal_codigo_natureza_credito = {
    '01': ['1102', '1113', '1117', '1118', '1121', '1251', '1403', '2102', '2113', '2117', '2118',
           '2121', '2251', '2403', '3102', '3251'],
    '02': ['1101', '1111', '1116', '1120', '1122', '1126', '1128', '1401', '1556', '1651', '1653',
           '2101', '2111', '2116', '2120', '2122', '2126', '2401', '2556', '2651', '2653', '3101',
           '3126', '3651', '3653'],
    '03': ['1124', '1125', '1933', '2124', '2125', '2933'],
    '04': ['1252', '1253', '1254', '1255', '1256', '1257', '2252', '2253', '2254', '2255', '2256',
           '2257'],
    '07': ['1352', '1353', '1354', '1355', '1356', '1932', '2352', '2353', '2354', '2355', '2356',
           '2932', '3352', '3353', '3356'],
    '12': ['1201', '1202', '1410', '1411', '2201', '2202', '2410'],
    '14': ['1351', '2351', '3351']
}


#Códigos da natureza de crédito e descricao dos mesmos de acordo com a tabela TABELA DE CÓDIGOS DA
# NATUREZA DO CRÉDITO (4.3.7)
fiscal_codigo_naturez_credito_descricao = {'01': 'Aquisicao de Bens para Revenda',
                                           '02': 'Aquisicao de Bens Utilizados como Insumo',
                                           '03': 'Aquisicao de Servicos Utilizados como Insumo',
                                           '04': 'Despesas de Energia Eletrica',
                                           '07': 'Armazenagem de Mercadoria e Frete na Operacao de Venda',
                                           '12': 'Devolucao de Vendas Sujeitas a Incidência Nao-Cumulativa',
                                           '14': 'Atividade de Transporte de Cargas - Subcontratacao de Servicos de Transporte'
}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

