# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from datetime import datetime
import time
fmt = '%Y-%m-%d %H:%M:%S'


def _fisnfc_get_percentual_tempo(self, cr, user, ids, name, args, context=None):
    """
    Calcula o tempo minimo e maximo  para o envio de cancelamento e de contingencia de um NFC-e.

    :param self: a instancia da classe
    :param cr: o cursor do banco de dados
    :param user: o ID do usuario utilizado para a verificacao de perfil
    :param ids: lista com o ID da nota fiscal
    :param name: nome do campo
    :param args: argumentos passados para o metodo
    :param context: um dicionario padrao para valores contextuais
    :type self: object
    :type cr: object
    :type user: int
    :type ids: list
    :type name: list
    :type args: list
    :type context: dict
    :return: verdadeiro para os registro que tiverem o tempo excedido pelo prazo.
    :rtype: dict
    """
    context = context and context.copy() or {}
    parametro = self._parametros(cr, user)
    result = {}.fromkeys(ids, {}.fromkeys(name, False))
    time_cancelamento = parametro.fiscal.fispar_nfce_tempo_cancelamento
    time_min_cancel = parametro.fiscal.fispar_nfce_min_percent_tempo_cancelamento
    time_min_cancel *= time_cancelamento / 100.
    time_max_cancel = parametro.fiscal.fispar_nfce_max_percent_tempo_cancelamento
    time_max_cancel *= time_cancelamento / 100.
    time_contingencia = parametro.fiscal.fispar_nfce_tempo_envio_contingencia
    time_min_contingencia = parametro.fiscal.fispar_nfce_min_percent_tempo_envio_contingencia
    time_min_contingencia *= time_contingencia / 100.
    time_max_contingencia = parametro.fiscal.fispar_nfce_max_percent_tempo_envio_contingencia
    time_max_contingencia *= time_contingencia / 100.
    for nf in self.browse(cr, user, ids, context):
        dt_inicio = time.mktime(datetime.strptime(nf.create_date, fmt).timetuple())
        dt_fim = time.mktime(datetime.strptime(datetime.now().strftime(fmt), fmt).timetuple())
        time_mov = (dt_fim - dt_inicio) / 3600.
        limit_min_cancel = time_mov > time_min_cancel
        limit_max_cancel = time_mov > time_max_cancel
        limit_min_contingencia = time_mov > time_min_contingencia
        limit_max_contingencia = time_mov > time_max_contingencia
        result[nf.id] = {'fisnfc_nfce_min_percent_tempo_cancelamento': limit_min_cancel,
                         'fisnfc_nfce_max_percent_tempo_cancelamento': limit_max_cancel,
                         'fisnfc_nfce_min_percent_tempo_envio_contingencia': limit_min_contingencia,
                         'fisnfc_nfce_max_percent_tempo_envio_contingencia': limit_max_contingencia}

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
