# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.nfe.manual_600.consrecinfe import ProcNFe
from pynfe.nfce.manual_600.consrecinfce import ProcNFCe
from datetime import datetime
import time
import pytz
import xml.etree.ElementTree as ET

INDICADOR_PRESENCA = [('0', 'Nao se aplica (NF complementar ou de ajuste)'),
                      ('1', 'Operacao presencial'),
                      ('2', 'Operacao nao presencial, pela Internet'),
                      ('3', 'Operacao nao presencial, Teleatendimento'),
                      ('4', 'NFC-e em operacao com entrega em domicilio'),
                      ('5', 'Operacao presencial, fora do estabelecimento'),
                      ('9', 'Operacao nao presencial, outros')]


def _fisnfc_get_indicador_presenca(self, cr, user, context=None):
    context = context and context.copy() or {}
    if context.get('nota_fiscal_consumidor'):
        result = [x for x in INDICADOR_PRESENCA if x[0] in ('1', '4')]
    else:
        result = INDICADOR_PRESENCA

    return result


def _fisnfc_get_xml(self, cr, user, ids, name, args, context=None):
    """
    Gera o XML da nota fiscal
    """
    context = context and context.copy() or {}
    result = {}.fromkeys(ids, False)
    for dados_nf in self.read(
            cr, user, ids, ['nfe_xml_auto', 'nfe_xml_gerado', 'modelo_55', 'nfe_xml_cancelamento', 'state',
                            'nfe_chave_acesso', 'nfe_versao_processo_emissao', 'nfe_protocolo',
                            'nfe_status', 'data_emissao'], context):

        xml = ''
        try:
            if dados_nf.get('nfe_xml_auto') and 'infNFe' in dados_nf.get('nfe_xml_auto') and \
                    'protNFe' in dados_nf.get('nfe_xml_auto'):
                xml = dados_nf.get('nfe_xml_auto') or ''
                # if dados_nf.get('state') == 'C':
                #     nfe = ProcNFe() if dados_nf.get('modelo_55') else ProcNFCe()
                #     nfe.NFe.xml = xml
                #     nfe.cancNfe.xml = dados_nf.get('nfe_xml_cancelamento')
                #     xml = nfe.xml

            else:
                nfe = ProcNFe() if dados_nf.get('modelo_55') else ProcNFCe()
                nfe.NFe.xml = str(dados_nf.get('nfe_xml_gerado')) or nfe.NFe.xml
                xml_auto = dados_nf.get('nfe_xml_auto')
                if not xml_auto:
                    if not nfe.protNFe.infProt.cStat.valor:
                        ambiente = nfe.NFe.infNFe.ide.tpAmb.valor
                        processo = self.fisnfc_verificar_status_nfce(cr, user, dados_nf.get('nfe_chave_acesso'),
                                                                     ambiente)
                        if dados_nf.get('state') == '3':
                            nfe.protNFe.infProt.xml = processo.resposta.protNFe.infProt.xml
                        vals = {'nfe_xml_auto': nfe.protNFe.xml}
                        if not dados_nf.get('nfe_protocolo'):
                            if processo.resposta and processo.resposta.protNFe:
                                vals['nfe_protocolo'] = processo.resposta.protNFe.infProt.nProt.valor
                        self.write(cr, user, ids, vals, context)
                else:
                    if dados_nf.get('state') in ('3', 'C', 'D'):
                        nfe.protNFe.xml = xml_auto

                # if dados_nf.get('state') == 'C':
                #     nfe.cancNfe.xml = dados_nf.get('nfe_xml_cancelamento')

                xml = nfe.xml

        except Exception as err:
            pass
        else:
            if '<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="0.0"' in xml:
                xml = xml.replace('<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="0.0"',
                                '<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="4.00"')

            result[dados_nf['id']] = xml

    return result


def _fisnfc_get_tempo_movimentacao_sefaz(self, cr, user, ids, name, args, context=None):
    """
    Calcula o tempo de movimentacao/envio de notas para a SEFAZ.
    :param self:
    :param cr:
    :param user:
    :param ids:
    :param name:
    :param args:
    :param context:
    :type self:
    :type cr:
    :type user:
    :type ids:
    :type name:
    :type args:
    :type context:
    :return:
    :rtype: dict
    """
    context = context and context.copy() or {}
    result = {}.fromkeys(ids, 0)
    fmt = '%Y-%m-%d %H:%M:%S'
    for nf in self.read(cr, user, ids, ['create_date'], context):
        dt_inicio = time.mktime(datetime.strptime(nf.get('create_date'), fmt).timetuple())
        dt_fim = time.mktime(datetime.strptime(datetime.now().strftime(fmt), fmt).timetuple())
        result[nf.get('id')] = (dt_fim - dt_inicio) / 3600

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
