#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields



class fiscal_impressora(osv.Model):
    """
    Tabela de impressoras (impressao DANFE/NFC-e)
    """
    def _fisimp_impname(self, cr, user, ids, name, arg, context=None):
        """
        Descricao da impressora.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID dos registros
        :param name: nome dos campos
        :param arg: argumentos da funcao
        :param context: um dicionário padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type name: list
        :type arg: list
        :type context: dic
        :return: a descricao da impressora (marca + modelo)
        :rtype: dict
        """
        context = context and context.copy() or {}
        result = {}
        for impressora in self.read(
                cr, user, ids, ['fisimp_marca', 'fisimp_modelo', 'fisimp_observacao'], context):
            result[impressora.get('id')] = '%(fisimp_marca)s %(fisimp_modelo)s' % impressora
            if impressora.get('fisimp_observacao'):
                result[impressora.get('id')] += ' (%s)' % impressora.get('fisimp_observacao')

        return result

    _name = 'fiscal.nfce.impressora'
    _columns = {
        'parametro_id': fields.many2one("fiscal.parametro", 'Parametro Fiscal', ondelete="cascade"),
        'name': fields.function(_fisimp_impname, method=True, store=False, type="char",
                                string='Descricao'),
        'fisimp_marca': fields.char('Marca', size=20, required=True),
        'fisimp_modelo': fields.char('Modelo', size=20, required=True),
        'fisimp_negrito_liga': fields.char('Liga negrito', size=20),
        'fisimp_negrito_desliga': fields.char('Desliga negrito', size=20),
        'fisimp_sublinhado_liga': fields.char('Liga sublinhado', size=20),
        'fisimp_sublinhado_desliga': fields.char('Desliga sublinhado', size=20),
        'fisimp_expande_liga': fields.char('Liga fonte expandida', size=20),
        'fisimp_expande_desliga': fields.char('Desliga fonte expandida', size=20),
        'fisimp_condensado_liga': fields.char('Liga fonte condensada', size=20),
        'fisimp_condensado_desliga': fields.char('Desliga fonte condensada', size=20),
        'fisimp_corta_papel': fields.char('Corte de papel', size=20),
        'fisimp_espacamento_linha': fields.char('Espacamento entre linhas', size=20),
        'fisimp_qrcode':fields.char('QR Code', SIZE=20),
        'fisimp_observacao': fields.text('Observacao')
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
