# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields

STATUS_NOTA_FISCAL = [('1', 'Esboco'), ('2', 'Validada'), ('3', 'Faturada'), ('C', 'Cancelada'),
                      ('AI', 'Aguardando Inutilizacao'), ('2E', 'Validada com erro'),
                      ('I', 'Inutilizada'), ('EP', 'Em processamento'), ('P', 'Pendente'),
                      ('D', 'Denegada')]
TIPO_NOTA_FISCAL = [('E', 'Entrada'), ('S', 'Saida')]
FRETE_NOTA_FISCAL = [('0', 'Contratacao do frete por conta do remetente (CIF)'),
                     ('1', 'Contratacao do frete por conta do destinatario (FOB)'),
                     ('2', 'Contratacao do frete Por conta de terceiros'),
                     ('3', 'Transporte proprio por conta do remetente'),
                     ('4', 'Transporte proprio por conta do destinatario'),
                     ('9', 'Sem ocorrencia de transporte')]
TIPO_VENCIMENTO_FRETE =[('A', 'A Vista'), ('P', 'A Prazo')]
TIPO_FRETE =[('D', 'Por Dentro'), ('F', 'Por Fora')]
TIPO_PRAZO_NOTA_FISCAL =[('DD', 'Dias Direto'), ('DFS', 'Dias Fora a Semana'),
                         ('DFQ', 'Dias fora a quinzena'), ('DFM', 'Dias Fora o Mes')]
ENERGIA_GRUPO_TENSAO = [('01', 'A1-Alta Tensao(230 Kv ou mais)'),
                        ('02', 'A2-Alta Tensao(88 a 138Kv)'), ('03', 'A3-Alta Tensao(69Kv)'),
                        ('04', 'A3a-Alta Tensao(30Kv a 44Kv)'),
                        ('05', 'A4-Alta Tensao(2,3Kv a 25Kv)'),
                        ('06', 'AS-Alta Tensao subterraneo'), ('07', 'B1-Residencial'),
                        ('08', 'B1-Residencial Baixa renda'), ('09', 'B2-Rural'),
                        ('10', 'B2-Coop.de Eletrificacao Rural'),
                        ('11', 'B2-Servico Publico de irrigacao'), ('12', 'B3-Demais classes'),
                        ('13', 'B4a-Iluminacao Publica-Rede de distribuicao'),
                        ('14', 'B4b-Iluminacao Publica-Bulbo de lampad')]
ENERGIA_TIPO_LIGACAO =[('1', 'Monofasica'), ('2', 'Bifasica'), ('3', 'Trifasica')]

class fiscal_nf_capa(osv.Model):
    """
    Tabela com os dados de cabecalho (capa) da nota fiscal.
    PREFIXO: fisnfc_ - Todo campo novo deve ser iniciado com fisnfc. Ex.: fisnfc_nf_terceiro_id.
    """
    from funcoes import (_fisnfc_get_nf_modelo_tipo,
                         _fisnfc_get_nf_modelo,
                         _fisnfc_get_nf_terceiro_modelo,  _fisnfc_get_tipo_frete,
                         _fisnfc_get_dados, _fisnfc_get_dados_no_store,
                         fisnfc_get_nf_parcelas,
                         fisnfc_set_nf_parcelas,
                         _fisnfc_get_vendedor_obrigatorio,
                         _fisnfc_get_nf_exportacao, _fisnfc_get_nf_importacao,
                         fisnfc_get_natureza_padrao)

    def _fisnfc_nota_propria(self, cr, uid, context=None):
        """
        De acordo com o contexto ele define se o flag 'nf_proprio' e verdadeiro
        """

        context = context and context.copy() or {}
        tipo = False

        if context.get('proprio'):
            tipo = True

        return tipo

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        if not 'modelo_nf_series_id' in fields:
            fields.append('modelo_nf_series_id')
        res = super(fiscal_nf_capa, self).default_get(cr, uid, fields, context)
        res['state'] = '1'
        if context.get('nota_fiscal_entrada') or context.get('nf_frete'):
            res['tipo'] = 'E'
            if not res.get('nf_propria'):
                res['prazo_tipo'] = 'DD'
        if context.get('nota_fiscal_saida'):
            res['tipo'] = 'S'
            obj_fiscal_parametro = self.pool.get('fiscal.parametro')
            fiscal_parametro_ids = obj_fiscal_parametro.search(cr, uid, [], context=context)
            if fiscal_parametro_ids:
                res['frete'] = obj_fiscal_parametro.browse(cr, uid, fiscal_parametro_ids[0]).tipo_frete
        if context.get('nf_servico'):
            res['fisnfc_tipo_servico'] = True
        if context.get('valor_frete'):
            res['vr_frete'] = context.get('valor_frete')
        if not context.get('import_nfe') and not context.get('nota_referenciada'):
            if res.get('modelo_nf_series_id') and res.get('nf_propria') or \
                context.get('ctx_nf_propria'):
                nf_propria = self._fisnfc_nota_propria(cr, uid, context)
                vals = self.onchange_modelo_nf(
                    cr, uid, [], res['modelo_nf_series_id'], res['tipo'], nf_propria, context)
                res.update(vals.get('value') or {})
        res['fisnfc_imprimi_promissoria'] = False
        return res

    _name = 'fiscal.nf.capa'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _rec_name = 'name'
    _columns = {
        ######## _____________ integer ____________________
        'id': fields.integer(),
        'numero_nf': fields.integer('Numero nf', select=True, size=9), # Numero da NF.
        'quantidade_volume': fields.integer('Quantidade volume'), # Quantide de volumes destacado na nf
        'numero_conhecimento': fields.integer('Nr.Conhecimento '), #numero do conhecimento frete,
        'fisnfc_rps_numero': fields.integer('Numero RPS', select=True), #Numero RPS (Somente para NF de servico)
        'fisnfc_rps_lote': fields.integer('lote de RPS', select=True), #Numero do lote de RPS (Somente para NF de servico)

        ######## _____________ char ____________________
        'prazo': fields.char('Prazo', size=60, help="Prazo em dias, separado por ',' ou '/"),
        'marca': fields.char('Marca', size=15), # Marca
        'especie': fields.char('Especie', size=15), # Especie
        'serie_nf': fields.char('Serie nf', size=3, select=True), # Serie da NF.
        'subserie': fields.char('Subserie', size=3), # Sub serie da nf
        'fisnfc_placa_frete': fields.char('Placa', size=10), # Serie frete
        'serie_frete': fields.char('Serie nf', size=3, select=True), # Serie frete
        'numero_volume': fields.char('Numero volume', size=15), # Numero
        'fisnfc_infcomp_contribuinte': fields.text('Informacoes Complementares'),
        'fisnfc_infcomp_fisco': fields.text('Dispositivo Fiscal'),
        'fisnfc_discriminacao_servico': fields.text('Discriminacao Servico'),  #Discriminacao para Nota Fiscal Eletrônica do tipo servico
        ######## _____________ float ____________________

        'vr_frete': fields.float('Vr frete', digits=(14, 2)), # Valor do frete
        'vr_seguro': fields.float('Vr seguro', digits=(14, 2)), # Valor do seguro
        'peso_bruto': fields.float('Peso bruto (Kg)', digits=(15, 3)), # Peso Bruto
        'vr_desconto': fields.float('Vr desconto', digits=(14, 2)), # Total de desconto da NF
        'peso_liquido': fields.float('Peso liquido (Kg)', digits=(15, 3)), # Peso Liquido
        'vr_ipi_frete': fields.float('Valor IPI', digits=(15, 2)), # Valor total do IPI frete
        'vr_icms_frete': fields.float('Vr icms frete', digits=(14, 2)),
        'vr_aliquota_frete': fields.float('Aliquota', digits=(6, 3)),
        'vr_base_icms_frete': fields.float('Vr base icms frete', digits=(14, 2)),
        'vr_icms_substituicao': fields.float('Vr icms substituicao', digits=(14, 2)),  # valor do icms da substituicao tributaria
        'vr_despesas_acessorias': fields.float('Vr despesas acessorias', digits=(14, 2)),  # Valor das despesas acessorias
        'vr_base_icms_substituicao': fields.float('Vr base icms substituicao', digits=(14, 2)),  # Base de substituicao tributaria destacada na nf
        'fisnfc_vr_despesas_extras': fields.float('Despesas Extras', digits=(14, 2)),
        'fisnfc_valor_bonificado': fields.function(_fisnfc_get_dados, method=True, store=True,
                                                   string='Bonificacao', digits=(14, 2),
                                                   multi='dados'),

        ######## _____________ boolean ____________________
        'nf_propria': fields.boolean('Nota Fiscal propria'),
        'fisnfc_tipo_servico': fields.boolean('NF de Servico'),
        'fisnfc_recebimento_conferido': fields.boolean('Recebimento Conferido'),
        'fisnfc_despesa_acessoria_capa': fields.boolean('Flag despesa acessoria'),#flag para verificar se a despesa acessoria foi informada somente no item ou somente na capa

        ######## _____________ datetime ____________________
        'data_emissao': fields.datetime('Data Emissao', select=True), # Data de emissao da NF.
        'hora_emissao': fields.datetime('Hora Emissao'), # Horário da emissao da NF.
        'fisnfc_recebimento_data_hora': fields.datetime('Data Hora recebimento'),
        'data_saida_entrada': fields.datetime('Data Saida/Entrada', select=True), # Data de saida ou de entrada da nf

        ######## _____________ date ____________________
        'data_emissao_frete': fields.date('Data Emissao', select=True),  # Data Emissao Frete
        'data_vencimento_frete': fields.date('Data Vencimento', select=True),  # Data Vencimento Frete
        'fisnfc_previsao_entrega': fields.date('Previsao de entrega'),
        'fisnfc_rps_data': fields.date('Data do RPS'),

        ######## _____________ selection ____________________
        'tipo': fields.selection(TIPO_NOTA_FISCAL, 'Tipo', required=True, size=1, select=True),
        'state': fields.selection(
            STATUS_NOTA_FISCAL, 'Status', size=2, track_visibility='onchange', select=True),
        'frete': fields.selection(FRETE_NOTA_FISCAL, 'Frete', size=1),
        'prazo_tipo': fields.selection(TIPO_PRAZO_NOTA_FISCAL, 'Tipo de prazo', size=3),
        'tipo_frete': fields.selection(TIPO_FRETE, 'Tipo do Frete', size=1),
        'tipo_venc_frete': fields.selection(TIPO_VENCIMENTO_FRETE, 'Tipo Vencimento', size=1),
        'fisnfc_energia_tipo_ligacao': fields.selection(
            ENERGIA_TIPO_LIGACAO, 'Tipo de Ligacao', size=1),
        'fisnfc_energia_grupo_tensao': fields.selection(
            ENERGIA_GRUPO_TENSAO, 'Grupo de Tensao', size=2),

        ######## _____________ functions ____________________
        'vr_total_deducao': fields.function(_fisnfc_get_dados, method=True, string='Valor Total Deducao',
                                            type='float', digits=(15, 2), store=True, multi='dados'),  # O email da pessoa (um email apenas)
        'vr_total_produtos': fields.function(_fisnfc_get_dados, method=True, string='Valor total produtos',
                                             type='float', digits=(15, 2), store=True, multi='dados'), # Valor total dos produtos
        'vr_total_servicos': fields.function(_fisnfc_get_dados, method=True, string='Valor total servicos',
                                             type='float', digits=(15, 2), store=True, multi='dados'), # Valor total dos servicos
        'vr_total_nf': fields.function(_fisnfc_get_dados, method=True, string='Total NF', type='float',
                                       digits=(15, 2), store=True, multi='dados'), # Valor total da Nota Fiscal
        'fisnfc_nf_devolucao': fields.function(_fisnfc_get_dados_no_store, type="boolean", multi="dados_no",
                                               method=True, store=False, string='NF-Devolucao'),  #flag nao historiado, para verificacao de cfop de devolucao
        'fisnfc_nf_energia_eletrica': fields.function(
            _fisnfc_get_nf_modelo_tipo, type="boolean", method=True, string='NF-Energia Eletrica',
            multi='modelo_tipo'),
        # flag nao historiado, tipo verificacao codigo do modelo da nf  energia eletrica
        'fisnfc_nf_agua_canalizada': fields.function(
            _fisnfc_get_nf_modelo_tipo, type="boolean", method=True,
            string='NF-Agua Canalizada', multi='modelo_tipo'),
        # flag nao historiado, tipo verificacao codigo do modelo da nf de agua canalizada
        'fisnfc_parametro_danfe': fields.function(_fisnfc_get_dados_no_store, type="boolean", method=True, store=False,
                                                  string='Parametro DANFE', multi="dados_no"),  #flag nao historiado,para buscar o parâmetro do DANFE
        'fisnfc_cfop_gnre': fields.function(_fisnfc_get_dados_no_store, type="boolean", method=True, store=False,
                                            string='CFOP GNRE', multi="dados_no"), #flag nao historiado,para validar cfop x GNRE
        'fisnfc_modelo_nf_tipo': fields.function(_fisnfc_get_dados_no_store, type="char", size=1, method=True,
                                                 store=False, string='Modelo tipo', multi="dados_no"),  #flag nao historiado,para validar cfop tipo natureza
        'fisnfc_modelo_serie': fields.function(_fisnfc_get_dados_no_store, type="char", size=6, method=True,
                                                 store=False, string='Modelo e Serie', multi='dados_no'),  #flag nao historiado,para validar modelo serie
        'fisnfc_modelo': fields.function(_fisnfc_get_dados_no_store, type="char", size=2, method=True,
                                        store=False, string='Modelo', multi='dados_no'),  #flag nao historiado,para validar modelo
        'fisnfc_ativar_entrega': fields.function(_fisnfc_get_dados_no_store, type="boolean", method=True, store=False,
                                                 string='Entrega', multi='dados_no'), #flag nao historiado,para validar entrega
        'fisnfc_ativar_coleta': fields.function(_fisnfc_get_dados_no_store, type="boolean", method=True, store=False,
                                                string='Coleta', multi='dados_no'), #flag nao historiado,para validar coleta
        'name': fields.function(_fisnfc_get_dados_no_store, method=True, store=False, type="text",
                                string='Numero nf', multi='dados_no'),
        'vr_ipi': fields.function(_fisnfc_get_dados, method=True, string='Vr IPI', type='float', digits=(15, 2),
                                  store=True, multi='dados'), # Valor total do IPI
        'observacao_pessoa': fields.function(_fisnfc_get_dados_no_store, type="text", method=True,
                                             string="Observacao Cliente", store=False, multi='dados_no'),
        'vr_icms': fields.function(_fisnfc_get_dados, method=True, string='Vr ICMS', type='float', digits=(15, 2),
                                store=True, multi='dados'), # Valor do icms destacado na nf
        'vr_base_icms': fields.function(_fisnfc_get_dados, method=True, string='Vr ICMS', type='float', digits=(15, 2),
                                        store=True, multi='dados'),  # Base de icms destacada na nf
        'fisnfc_titulos_ids': fields.function(_fisnfc_get_dados_no_store, method=True, string='Titulos', type='one2many',
                                              relation='financeiro.titulo', store=False, multi='dados_no'),  # Titulos da da fatura da nota
        'fisnfc_permite_alterar_condicoes_pgto': fields.function(_fisnfc_get_dados_no_store,
                                                                'Permitir alterar condicoes de pagamento',
                                                                 method=True, type='boolean', store=False, multi='dados_no'),
        'fisnfc_imprimi_promissoria': fields.function(_fisnfc_get_dados_no_store, method=True,
                                                      string='Imprimi promissoria', type='boolean',
                                                      store=False, multi='dados_no'),
        'fisnfc_calcula_peso_automaticamente': fields.function(
            _fisnfc_get_dados_no_store, methodo=True, type='boolean', store=False, multi='dados_no'),
        'nf_parcelas_ids': fields.function(fnct=fisnfc_get_nf_parcelas, method=True,
                                           fnct_inv=fisnfc_set_nf_parcelas,
                                           string='Parcela', type='one2many',
                                           relation='fiscal.nf.parcelas'),
        'fisnfc_valida_item_rateado_nf': fields.function(_fisnfc_get_dados_no_store, method=True,
                                                     type='boolean', store=False, multi='dados_no'),

        'fisnfc_valor_fcp_st': fields.function(
            _fisnfc_get_dados, digits=(8, 2), method=True, string='Fundo de combate a pobreza',
            store=True, multi='dados'),
        'fisnfc_icms_desonerado': fields.boolean('Venda com desoneracao de ICMS'),
        'fisnfc_valor_icms_desonerado': fields.function(
            _fisnfc_get_dados, digits=(8, 2), method=True, string='Valor do ICMS desonerado',
            store=True, multi='dados'),
        'fisnfc_parametro_e200': fields.function(
            _fisnfc_get_dados, method=True, string='NF gera apuracao p/ registro E200',
            type='boolean', multi='dados'),
        'fisnfc_nf_transferencia': fields.function(_fisnfc_get_dados, method=True,
                                                   string='NF-Transferencia', type='boolean',
                                                   multi='dados'),

        # Possui forma de pagamento que imprimi promissoria
        ######## _____________ one2many ____________________
        'guia_ids': fields.one2many('fiscal.guia', 'nf_id', 'Guias'),
        'deducao_itens_ids': fields.one2many('fiscal.deducoes.itens', 'nf_id', 'Deducoes'),
        'tributacao_agrupar_ids': fields.one2many('fiscal.tributacao.agrupar.pgview', 'nf_id', 'Tributacao'),
        'estoque_movimento_capa_ids': fields.one2many('estoque.movimento.capa', 'nf_capa_id', 'Capa'),
        'estoque_movimento_itens_ids': fields.one2many('estoque.movimento.itens', 'nf_id', 'Nota Fiscal'),
        'nf_condicao_ids': fields.one2many('fiscal.condicao.pagamento', 'capa_id', 'Condicoes'),
        'fisnfc_parcelas_terceiros_ids': fields.one2many('fiscal.nf.parcelas', 'fisnfc_nf_terceiro_id', 'Parcelas'),  #campo referente as parcelas da nf de terceiros
        'fisnfc_nf_frete_ids': fields.one2many('fiscal.nf.capa', 'fisnfc_nf_frete_id', 'Fretes'),
        'fisnfc_frete_tributacoes_ids': fields.one2many('fiscal.nf.item.tributacao', 'fisnit_capa_id', 'Tributacoes'),
        'fisnfc_nf_referenciada_ids': fields.many2many('fiscal.nf.capa', 'fiscal_nf_referencia', 'id',
                                                       'nf_ref_id',
                                                       "Notas referenciadas"),  # NFc de saida de NFCe, ou entrada propria de devolucao
        'fisnfc_nf_referencia_ids': fields.many2many('fiscal.nf.capa', 'fiscal_nf_referencia', 'nf_ref_id',
                                                     'id',
                                                     "Notas de referencia"),  # NFc's e NFCe's que foram referenciadas por outras notas
        'fisnfc_gerencial_movimento_ids': fields.one2many('gerencial.movimento', 'fingmo_nf_id', 'Custificacao'),
        ######## _____________ many2one ____________________
        'pedido_id': fields.many2one("estoquepedidos.pedido.capa", 'Pedido Capa', ondelete="restrict", select=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True), # Empresa emitente do documento.
        'fisnfc_vendedor_obrigatorio': fields.function(
            _fisnfc_get_vendedor_obrigatorio, store=False, type='boolean',
            string='Vendedor obrigatorio'),
        'vendedor_id': fields.many2one("vendas.vendedor", 'Vendedor', ondelete="restrict", domain="[('tipo','=','I')]"),  # Vendedor
        'natureza_id': fields.many2one("fiscal.cfop.natureza", 'Natureza', ondelete="restrict", required=True),  # Id da natureza utilizada
        'modelo_nf_series_id': fields.many2one("fiscal.modelo.nf.series", 'Modelo NF', ondelete="restrict"),  # Id do modelo da nf
        'modelo_nf_id': fields.many2one("fiscal.modelo.nf", 'Modelo de NF', ondelete="restrict"),  # Id do modelo da nf para nota de terceiros
        'fisnfc_cod_consumo_id': fields.many2one("fiscal.codigo.consumo", 'Codigo de Consumo',
                                                 ondelete="restrict", select=True,
                                                 domain="[('fiscco_modelo_id','=',modelo_nf_id)]"),
        'tributacao_id': fields.many2one("fiscal.tributacao", 'Tributacao', ondelete="restrict", select=True),  #tributacao para o frete
        'modelo_nf_frete': fields.many2one('fiscal.modelo.nf', 'Modelo Frete', ondelete="restrict"),
        'destinatario_id': fields.many2one("adm.pessoa", 'Destinatario', ondelete="restrict"), # Pessoa destinatária.
        'transportador_id': fields.many2one("adm.transportador", 'Transportadora', ondelete="restrict"),  # id do transportador
        'endereco_coleta_id': fields.many2one("adm.pessoa.endereco", 'Endereco Coleta', ondelete="restrict",
                                              domain="[('pessoa_id', '=', destinatario_id)]"),  # Endereco de coleta da mercadoria.
        'endereco_entrega_id': fields.many2one("adm.pessoa.endereco", 'Endereco Entrega', ondelete="restrict",
                                               domain="[('pessoa_id', '=', destinatario_id)]"),  # Endereco de entrega da mercadoria.
        'vendedor_externo_id': fields.many2one("vendas.vendedor", 'Vendedor Externo', ondelete="restrict",
                                               domain="[('tipo','=','E')]"),
        'endereco_cobranca_id': fields.many2one("adm.pessoa.endereco", 'Endereco Cobranca', ondelete="restrict"),  # Endereco de entrega da mercadoria.
        'destinatario_endereco_id': fields.many2one("adm.pessoa.endereco", 'Endereco ', ondelete="restrict",
                                                    domain="[('pessoa_id', '=', destinatario_id)]"),  # Endereco do estabelecimento destinatário.
        'fisnfc_nf_frete_id': fields.many2one('fiscal.nf.capa', 'Id NF Frete'),
        'fisnfc_recebimento_usuario_id': fields.many2one('res.users', 'Recebimento Usuario'),
        'fisnfc_origem_uf_id': fields.many2one('adm.uf', 'Estado de origem'),
        'fisnfc_origem_cidade_id': fields.many2one(
            'adm.cidade', 'Cidade de origem', domain="[('uf_id', '=', fisnfc_origem_uf_id)]"),
        'fisnfc_destino_uf_id': fields.many2one('adm.uf', 'Estado de destino'),
        'fisnfc_destino_cidade_id': fields.many2one(
            'adm.cidade', 'Cidade de destino', domain="[('uf_id', '=', fisnfc_destino_uf_id)]"),

        ######## _____________ many2many ___________________
        'fisnfc_pedido_ids': fields.many2many('estoquepedidos.pedido.capa', 'fiscal_nf_pedidos', 'nf_id', 'pedido_id',
                                              oldname="nf_pedidos_ids" "Pedidos"),


        # RELATED
        'fisnfc_modelo_tipo': fields.related('modelo_nf_id', 'fismnf_tipo', type="char",
                                             relation="fiscal.modelo.nf", string="Tipo do modelo",
                                             store=False, required=False),
        # Exportacao
        'fisnfc_exportacao': fields.function(
            _fisnfc_get_nf_exportacao, type='boolean', method=True, invisible=True,
            string='Nota fiscal de exportacao'),
        'fisnfc_uf_saida_pais': fields.many2one(
            'adm.uf', 'UF de embarque', ondelete='restrict', select=True,
            help='UF de embarque ou de transposicao de fronteira (Nao aceita "EX").'),
        'fisnfc_local_exportacao': fields.char(
            'Local de embarque', size=60, help='Descricao do local de embarque ou de transposicao '
                                               'de fronteira.'),
        'fisnfc_local_despacho': fields.char(
            'Descricao do local de despacho', size=60,
            help='Informacao do recinto alfandegado. (Nao obrigatorio)'),

        # Empenho - Orgaos publicos e autarquias
        'fisnfc_numero_nota_empenho': fields.char('Numero da nota de empenho', size=17),
        'fisnfc_data_empenho': fields.date('Data do empenho'),
        'fisnfc_codigo_unidade_executora': fields.char('Codigo da unidade executora', size=10),
        'fisnfc_controle_onchange_parcelas': fields.function(
            _fisnfc_get_dados_no_store, methodo=True, type='char', store=False, size=1,
            multi='dados_no', string='Controle'),
        'fisnfc_cte_xml': fields.binary('Arquivo XML do CT-e'),
        'fisnfc_link_xml': fields.function(
            _fisnfc_get_dados_no_store, methodo=True, type='char', store=False, size=64,
            multi='dados_no', string='Link XML'),

        # Declaracao de importacao
        'fisnfc_di_ids': fields.one2many('fiscal.declaracao.importacao', 'fisdim_nf_id', 'Declaracao de importacao'),
        'fisnfc_importacao': fields.function(_fisnfc_get_nf_importacao, type='boolean', string='exportacao/importacao',
                                             help='Se marcado define a nota fiscal como sendo de exportacao/importacao'),
        'fisnfc_retencao_ir_valor': fields.float('IR Retido', digits=(9, 2)),
    }

    _order = 'data_emissao desc, serie_nf desc, numero_nf desc'
    _fields_no_store = ['fisnfc_valor_fcp_st']
    _defaults = {
        'data_emissao': lambda *a: fields.datetime.now(),
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'modelo_nf_series_id': _fisnfc_get_nf_modelo,
        'modelo_nf_id': _fisnfc_get_nf_terceiro_modelo,
        'nf_propria': _fisnfc_nota_propria,
        'frete': '9',
        'tipo_frete': _fisnfc_get_tipo_frete,
        'fisnfc_despesa_acessoria_capa': True,
        'fisnfc_recebimento_conferido': False,
        'fisnfc_vendedor_obrigatorio': _fisnfc_get_vendedor_obrigatorio,
        'natureza_id': fisnfc_get_natureza_padrao,
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:


