# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_nf_item_tributacao(osv.Model):
    """
    Tributacoes do item de nota fiscal
    """
    from funcoes import _fisnit_get_dados_fiscais

    _name = 'fiscal.nf.item.tributacao'  # fisnit_
    _rec_name = 'encargo_id'
    _columns = {
        'id': fields.integer('Id', readonly=True),
        'movimento_itens_id': fields.many2one(
            'estoque.movimento.itens', 'Item', ondelete='cascade', select=True),
        'encargo_id': fields.many2one(
            'fiscal.encargo', 'Encargo', ondelete='restrict', select=True),
        'cst_id': fields.many2one('fiscal.cst', 'Cst', ondelete='restrict', select=True),
        'tributacao_id': fields.many2one(
            'fiscal.tributacao', 'Tributacao', ondelete='restrict', select=True),
        'aliquota': fields.float('Aliquota', digits=(6, 3)),
        'vr_base_calculo': fields.float('Vr base calculo', digits=(15, 2)),
        'vr_encargo': fields.float('Vr encargo', digits=(15, 2)),
        'vr_reducao_base': fields.float('Vr reducao base', digits=(15, 2)),
        'retido': fields.boolean('Retido'),
        'percentual_mva': fields.float('Percentual mva', digits=(7, 4)),
        'modbcst': fields.selection(
            [('0', 'Preco tabelado'), ('1', 'Lista negativa'), ('2', 'Lista positiva'),
             ('3', 'Lista neutra'), ('4', 'Margem valor agregado'), ('5', 'Pauta'),
             ('6', 'Valor da operacao')],
            'Modal.determ. BC do ICMS ST',
            help='Modalidade de determinacao da Base de Calculo do ICMS de Substituicao Tributaria',
            size=1),
        'valor_pauta': fields.float('Valor Pauta', digits=(10, 2)),
        'vr_base_icms_substituicao': fields.float('Vr base icms substituicao', digits=(14, 2)),
        'vr_icms_substituicao': fields.float('Vr icms substituicao', digits=(14, 2)),
        'vr_icms_substituicao_guia': fields.float('Vr icms substituicao Guia', digits=(14, 2)),
        'aliquota_venda': fields.float(
            'Aliquota para venda', digits=(5, 2),
            help='Aliquota de ICMS na venda para verificacao de calculo do ICMS ST na compra.'),
        'tipo_substituicao': fields.function(
            _fisnit_get_dados_fiscais, type='boolean', method=True, store=True,
            string='Tipo Substituicao', multi='cst'),
        'fisnit_csosn_id': fields.many2one('fiscal.csosn', 'CSOSN'),
        'fisnit_nat_rec_pis_cofins': fields.char('Natureza da Receita PIS/COFINS', size=3),
        'fisnit_capa_id': fields.many2one(
            'fiscal.nf.capa', 'Id Nota Fiscal', ondelete='cascade', select=True),
        'fisnit_tipo_movimento_cst': fields.function(
            _fisnit_get_dados_fiscais, type='char', method=True, store=True, multi='cst',
            string='Tipo movimento cst'),
        'fisnit_tipo_cst': fields.function(
            _fisnit_get_dados_fiscais, type='char', method=True, store=True, multi='cst',
            string='Tipo cst'),
        'fisnit_vr_contabil': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True, multi='cst',
            string='Valor Contabil'),
        'fisnit_vr_tributado': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True,  multi='cst',
            string='Valor Tributado'),
        'fisnit_vr_isento': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True,  multi='cst',
            string='Valor Isento'),
        'fisnit_vr_outros': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True,  multi='cst',
            string='Valor outros'),
        'fisnit_vr_diferido': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True,  multi='cst',
            string='Valor diferido'),
        'fisnit_vr_substituicao': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True,  multi='cst',
            string='Valor substituicao'),
        'fisnit_vr_servico': fields.function(
            _fisnit_get_dados_fiscais, digits=(14, 2),  method=True, store=True,  multi='cst',
            string='Valor servico'),
        'fisnit_cfop_id': fields.function(
            _fisnit_get_dados_fiscais, type='many2one', method=True, store=True,  multi='cst',
            string='Cfop', relation='fiscal.cfop'),
        'fisnit_vr_difal_base': fields.float(digits=(15, 2), string='Valor da base do DIFAL'),
        'fisnit_vr_difal_uf_origem': fields.float(digits=(15, 2), string='Valor DIFAL UF origem'),
        'fisnit_vr_difal_uf_destino': fields.float(digits=(15, 2), string='Valor DIFAL UF destino'),
        'fisnit_vr_fcp': fields.float(digits=(15, 2), string='Valor do FCP'),
        'fisnit_vr_fcp_st': fields.float(digits=(15, 2), string='Valor do FCP/ST'),
        'fisnit_vr_icms_desonerado': fields.float('Valor do ICMS desonerado', digits=(15, 2)),
        'fisnit_percentual_base_zero': fields.function(
            _fisnit_get_dados_fiscais, type='boolean', multi='cst', string='% Base zero')
    }
    _sql_constraints = [('item_tributacao_uniq',
                         'unique (fisnit_capa_id, encargo_id, movimento_itens_id)',
                         'O item movimentado nao pode possuir encargos repetidos!\n'
                         'Caso esteja tentando exluir e incluir novamente, exclua salve e inclua!')]

    # Campos funcionais que nao serao calculados quando uma base de dados for carregada
    _fields_no_store = ['fisnit_vr_diferido', 'fisnit_vr_outros', 'fisnit_vr_contabil',
                        'fisnit_vr_substituicao', 'fisnit_vr_servico', 'fisnit_vr_tributado',
                        'fisnit_vr_isento', 'fisnit_tipo_movimento_cst', 'fisnit_tipo_cst',
                        'fisnit_cfop_id']

    _defaults = {'fisnit_vr_icms_desonerado': 0,
                 'fisnit_vr_difal_base': 0,
                 'fisnit_vr_difal_uf_origem': 0,
                 'fisnit_vr_difal_uf_destino': 0,
                 }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
