# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################
from openerp.osv import osv, fields
from openerp.tools.translate import _
from funcoes import fispar_verifica_empresa_industria


class fiscal_parametro(osv.Model):
    """
    Tabela e parametros para a nota fiscal.
    """

    def _fispar_get_contingencia_motivo(self, cr, user, context=None):
        return [('SVC-AN', 'Contingencia SVC-AN'), ('SVC-RS', 'Contingencia SVC-RS')]

    _name = 'fiscal.parametro'
    _rec_name = 'company_id'
    _description = 'Tabela de Parametros do modulo fiscal'
    _columns = {
        'id': fields.integer('id'),
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade', required=True,
                                      select=True),
        'fispar_natureza_uf_dentro_venda_id': fields.many2one('fiscal.cfop.natureza',
                                                              'Natureza de vendas dentro estado',
                                                              ondelete='restrict',
                                                              oldname="natureza_venda_id"),
        'fispar_natureza_uf_fora_venda_id': fields.many2one('fiscal.cfop.natureza',
                                                            'Natureza de vendas fora estado',
                                                            ondelete='restrict'),

        'fispar_natureza_uf_dentro_venda_servico_id': fields.many2one('fiscal.cfop.natureza',
                                                                      'Natureza de vendas Servico '
                                                                      'uf dentro padrao',
                                                                      ondelete='restrict'),
        'fispar_natureza_uf_fora_venda_servico_id': fields.many2one('fiscal.cfop.natureza',
                                                                    'Natureza de vendas Servico uf '
                                                                    'fora padrao',
                                                                    ondelete='restrict'),
        'fispar_natureza_transferencia_entrada_id': fields.many2one(
            'fiscal.cfop.natureza', 'Natureza para entrada por transferencia', ondelete='restrict',
            select=True),

        # Se marcado exibir este campo na tela de nf
        'endereco_entrega': fields.boolean('Ativar End.Entrega'),
        # Se marcado exibir este campo na tela de nf
        'endereco_coleta': fields.boolean('Ativar End.Coleta'),
        'fispar_nota_conjugada':fields.boolean('Nota Conjugada', help='Se marcado gera uma unica '
                                                                      'nota com serviço/produto. '
                                                                      'Caso contrario, gera nota '
                                                                      'de servico e outra nota com '
                                                                      'as mercadorias.'),
        'certificado_conteudo': fields.text('Certificado Conteudo'),
        'chave_conteudo': fields.text('chave'),
        'data_validade': fields.datetime('Data de Validade '),
        'proprietario_certificado': fields.char('Proprietario Certificado', size=50),
        'motivo_inutilizacao': fields.char('Motivo Inutilizacao', size=255),
        'versao_aplicativo_nfe': fields.char('Versao Aplicativo NF-e', size=20, readonly=True),
        'condicao_pgto_id': fields.many2one("adm.condicao.forma", 'Condicao pgto',
                                            ondelete="restrict"),
        'dt_fim_reg_ent_p1': fields.date('DATA FINAL REGISTRO ENTRADA P1'),
        'dt_fim_reg_ent_p1a': fields.date('DATA FINAL REGISTRO ENTRADA P1-A'),
        'folha_reg_ent_p1': fields.integer('FOLHA REGISTRO ENTRADA P1'),
        'folha_reg_ent_p1a': fields.integer('FOLHA REGISTRO ENTRADA P1-A'),
        'nro_folha_apuracao_icms': fields.integer('Nro Folha de Apuracao ICMS'),
        'periodo_apuracao_icms': fields.integer('Periodo de Apuracao ICMS'),
        'nro_folha_apuracao_ipi': fields.integer('Nro Folha de Apuracao IPI'),
        'periodo_apuracao_ipi': fields.date('Periodo de Apuracao IPi'),
        'saldo_credor_anterior_ipi': fields.float('Saldo Credor Anterior', digits=(14, 2)),
        # Parametro para sped nao remover
        'perfil_arquivo_sped': fields.selection([('A', 'Perfil A'), ('B', 'Perfil B'),
                                                 ('C', 'Perfil C')], 'Perfil de Arquivo SPED',
                                                size=1),
        'modelo_nf_series_id': fields.many2one('fiscal.modelo.nf.series', 'Modelo Padrao',
                                               ondelete='restrict'),
        'fispar_servico_modelo_nf_series_id': fields.many2one('fiscal.modelo.nf.series',
                                                              'Modelo Padrao de Servico',
                                                              ondelete='restrict'),
        # ID do modelo da nf
        'modelo_nf_series_ids': fields.one2many('fiscal.modelo.nf.series', 'parametro_id',
                                                'Modelo NF', ondelete='restrict'),
        # Parametros para SPED (nao remover)
        'dia_vencimento_obrigacao_icms': fields.integer('Dia Vencimento ICMS'),
        'codigo_receita': fields.char('Codigo da receita', size=6),
        'fispar_codigo_difal_origem': fields.char('Codigo do difal de origem', size=6),
        'fispar_codigo_difal_destino': fields.char('Codigo do difal de destino', size=6),
        'contribuinte_substituto': fields.boolean('Contribuinte Substituto'),
        'tributos_ids': fields.one2many('fiscal.tributacao.encargos', 'parametro_id', 'Tributos'),
        'tipo_frete': fields.selection([('0', 'Por conta do emitente'),
                                        ('1', 'Por conta do destinatario/remetente'),
                                        ('2', 'Por conta de terceiros'), ('9', 'Sem Frete')],
                                       'Tipo de frete', size=1),
        'impressora_danfe': fields.char('Impressora DANFE', size=30),
        'quantidade_vias_danfe': fields.integer('Quant. Vias DANFE'),
        'fispar_tamanho_danfe': fields.selection([('A4', 'A4'),('A5','A5')], 'Tamanho Danfe', size=2),
        'fispar_receita_anual': fields.float('Receita Anual', digits=(15, 2),
                                             help='Receita anual para fins de apuracao de imposto '
                                                  'para empresas enquadradas no Simples Nacional'),
        'fispar_sublimite_estadual': fields.float('Sublimite estadual', digits=(15, 2),
                                             help='Informe o sublimite estadual para desenquadramento '
                                                  'da empresa no Simples Nacional para os Estados(UFs)'),
        'fispar_base_reduzida': fields.selection([('I', 'Isentos'), ('O', 'Outros')],
                                                 'Base reduzida', size=1,
                                                 help='Nos livros fiscais, a base reduzida sera '
                                                      'consederada como:'),

        'fispar_regime_incidencia_tributaria': fields.selection(
            [('1', 'Nao Cumulativo'), ('2', 'Cumulativo'), ('3', 'Nao cumulativo e cumulativo')],
            'Incidencia Tributaria'),
        'fispar_metodo_apropriacao_creditos': fields.selection(
            [('1', 'Metodo de Apropriacao Direta'),
             ('2', ' Metodo de Rateio Proporcional (Receita Bruta)')],
            'Metodo de Apropriacao de Creditos'),
        'fispar_criterio_adotado': fields.selection([('1', 'Regime caixa'), ('2', 'Competencia'),
                                                     ('9', 'Competencia detalhado')],
                                                    'Criterio adotado'),
        'fispar_tipo_contribuicao_apuracao': fields.selection(
            [('1', 'Apuracao da Contribuicao Exclusivamente a Aliquota Basica'),
             ('2', ' Apuracao da Contribuicao a Aliquotas Especificas')],
            'Tipo de Contribuicao/Apuracao'),
        'fispar_cod_rec_pis': fields.char('Codigo receita PIS', size=6),
        'fispar_cod_rec_cofins': fields.char('Codigo receita COFINS', size=6),
        'fispar_natureza_transferencia_saida_id': fields.many2one(
            'fiscal.cfop.natureza', 'Natureza para saida por transferencia', ondelete='restrict',
            oldname='natureza_transferencia_id'),
        'fispar_indicador_nfe': fields.selection([('0', 'Todas as NF-e'),
                                                  ('1', 'Somente as NF-e que ainda nao tiveram '
                                                        'manifestacao do destinatario '
                                                        '(Desconhecimento da operacao, Operacao '
                                                        'nao Realizada ou Confirmacao da '
                                                        'Operacao)'),
                                                  ('2', 'Idem anterior, incluindo as NF-e que '
                                                        'tambem nao tiveram a Ciencia da '
                                                        'Operacao.')],
                                                 'Indicador de NF-e consultada', size=1),
        'fispar_indicador_emissor': fields.selection([('0', 'Todos os Emitentes / Remetentes'),
                                                      ('1', 'Somente as NF-e emitidas por '
                                                            'emissores/remetentes que nao tenham '
                                                            'a mesma raiz do CNPJ do destinatario '
                                                            '(para excluir as notas fiscais de '
                                                            'transferencia entre filiais).')],
                                                     'Indicador Emissor'),
        'fispar_ultimo_nsu': fields.char('Ultimo NSU', size=15,
                                            help='Numero Sequencial Unico de NF-e. Sera a partir '
                                                 'deste numero que a SEFAZ disponibilizara os '
                                                 'documentos (as notas fiscais emitidas).'),
        'fispar_propagar_informacoes_nf': fields.boolean('Propagar informacoes nota de '
                                                         'saida/servico para titulo',
                                                         help='Se marcado propaga as informacoes da'
                                                              ' nota fiscal de saida/servico para o'
                                                              'titulo'),
        'fispar_imprimir_qtde_embalagem_danfe': fields.boolean(
            'Imprimir quantidade embalagem no DANFE',
            help= 'Se marcado, a quantidade da embalagem selecionada para o produto sera impressa '
                  'a frente da descricao do produto no DANFE.'),
        'fispar_imprimir_qtde_embalagem_uni_danfe': fields.boolean(
            'Imprimir quantidade embalagem na unidade no DANFE',
            help= 'Se marcado, a quantidade da embalagem selecionada para o produto sera impressa '
                  'junto a unidade do produto no DANFE.'),
        'fispar_nfe_versao_xml': fields.selection([('4.00', 'Versao 4.00')], 'Versao do XML'),
        'fispar_nfe_ambiente': fields.selection([(1, 'Producao'), (2, 'Homologacao')], 'Ambiente'),
        'fispar_nfe_contingencia': fields.selection(_fispar_get_contingencia_motivo,
                                                    'Contingencia'),
        'fispar_nfe_contingencia_motivo': fields.char('Motivo da contingencia', size=256),
        'fispar_nfe_contingencia_data': fields.datetime('Inicio da contingencia'),
        'fispar_esboco_so_manager': fields.boolean(
            'So manager volta nf p/esboco'
        ),
        'fispar_numero_nf_no_titulo': fields.boolean('Numero da nf na obs do titulo',
                                                     help="Se marcado, o numero da nf será inserido nas observações do titulo"),

        # __ Parametros de para consulta dos tributos do IBPT __
        'fispar_ibpt_token': fields.text(string='Token',
                                         help="Token da empresa - Item adquirido no site De "
                                              "Olho no Imposto "
                                              "(http://deolhonoimposto.ibpt.org.br)"),
        'fispar_ibpt_cnpj': fields.char(string='CNPJ',
                                        help="CNPJ da empresa que esta fazendo as consultas"),
        'fispar_ibpt_uf': fields.char(string='UF', help='Sigla do estado para as consultas',
                                      size=2),
        'fispar_calcula_peso_item_automatico': fields.boolean(
            string='Calcula peso dos itens automaticamente',
            help='Se marcado, calcula o peso dos itens de forma automatica na nota fiscal de saida'),
        'fispar_enviar_email': fields.boolean('Eviar email em Homologacao',
                                              help='Permite o envio automatico de email ao '
                                                   'faturar uma nota quando o ambiente for de '
                                                   'homologacao.'),
        'fispar_indicador_natureza_pessoa_juridica': fields.selection(
            [('00', '00 - Pessoa Juridica em geral (nao participante de SCP como socia '
                    'ostensiva.)'),
             ('01', '01 - Sociedade cooperativa (nao participante de SCP como socia ostensiva.)'),
             ('02', '02 - Entidade sujeita ao PIS/PASEP exclusivamente com base na Folha de '
                    'Salarios.'),
             ('03', '03 - Pessoa juridica em geral participande de SCP como socia ostensiva.'),
             ('04', '04 - Sociedade cooperativa participante de SCP como socia ostensiva.'),
             ('05', '05 - Sociedade em Conta de Participacao - SCP.')],
            'Indicador da natureza da pessoa juridica', size=2),
        'fispar_cooperativa': fields.selection(
            [('00', '00-Nao e cooperativa'), ('01', '01-Prod.Agropecuaria'), ('02', '02-Consumo'),
             ('03', '03-Credito'), ('04', '04-Eletrif.Rural'), ('05', '05-Transp.Rod.Cargas'),
             ('06', '06-Medicos'), ('99', '99-Outras')], 'Cooperativa', size=2),
        'fispar_nfe_indicador_presenca': fields.selection([
            ('0', 'Nao se aplica (NF complementar ou de ajuste)'), ('1', 'Operacao presencial'),
            ('2', 'Operacao nao presencial, pela Internet'),
            ('3', 'Operacao nao presencial, Teleatendimento'),
            ('4', 'NFC-e em operacao com entrega em domicilio'),
            ('9', 'Operacao nao presencial, outros.')], 'Indicador de presenca do comprador',
            required=True, type='char', size=1,
            help='Indicador de presenca do comprador no estabelecimento comercial no momento da '
                 'operacao.'),

        # Notas fiscais de perda do inventario
        'fispar_natureza_nf_perda_id': fields.many2one(
            'fiscal.cfop.natureza', 'Natureza para notas fiscais de perda',
            help='Natureza a ser utilizada quando da emissao de nota fiscal atraves das perdas do '
                 'inventario.'),
        'fispar_condicao_pgto_nf_perda_id': fields.many2one(
            'adm.condicao.forma', 'Condicao de pagamento para notas fiscais de perda',
            help='Condicao de pagamento a ser utilizada quando da emissao de nota fiscal atraves '
                 'das perdas do inventario.'),
        'fispar_codigo_ajuste_icms_nf_arroz': fields.char(
            'Codigo de ajuste do ICMS NF arroz', size=8, oldname='fispar_codigo_ajuste_icms',
            help='O valor informado no campo deve existir na tabela de codigo do ajuste da '
                 'apuracao e deducao de cada Secretaria de Fazenda conforme a UF do declarante, '
                 'e deve existir na tabela de codigo do ajuste da apuracao e deducao, constante do '
                 'Item 5.1.1. do Ato COTEPE/ICMS.'),
        'fispar_codigo_ajuste_icms_simples_nac': fields.char(
            'Codigo de ajuste do ICMS p/ compras Simples Nac.', size=8,
            help='Codigo para credito de ICMS em compras de empresa simples nacional.\n O valor'
                 'informado no campo deve existir na tabela de codigo do ajuste da apuracao e '
                 'deducao de cada Secretaria de Fazenda conforme a UF do declarante e deve existir '
                 'na tabela de codigo do ajuste da apuracao e deducao, constante do Item 5.1.1 do '
                 'Ato COTEPE/ICMS.'),
        'fispar_codigo_ajuste_icms_st': fields.char('Codigo de ajuste do ICMS ST', size=10,
                                                    help="Débitos especiais; Op.ST; Resp.: "
                                                         "Própria; Apur.: Recolhimento Espontâneo;"
                                                         " Mercadoria; ST interna."),
        'fispar_sped_fiscal_e200_manual': fields.boolean(
            'Gera registro E200 p/ apuracao manual',
            help="Se marcado, o SPED Fiscal gera o registro E200 e seus filhos. O usuario deve "
                 "informar manualmente os valores apurados no arquivo gerado ou validador."),
        'fispar_gera_nfe_esboco': fields.boolean('Gerar NF-e em esboco',
                                                help='Se marcado, ao faturar um pedido, a NF-e sera'
                                                     ' gerada em estado de "Esboco".'),

        # NFC-e
        'fispar_nfce_modelo_series_id': fields.many2one(
            'fiscal.modelo.nf.series', 'Modelo padrao de NFC-e', ondelete='restrict',
            help='Modelo padrao para a nota fiscal de consumidor eletronica.'),
        'fispar_nfce_versao_qrcode': fields.selection(
            [(1, 'Versao 1'), (2, 'Versao 2')], 'Versao do QR Code', size=1,
            help='Numero da versao do QR Code utilizado na NFC-e'),
        'fispar_nfce_csc1': fields.char(
            'Identificacao do CSC 1', size=36,
            help='Codigo de Seguranca do Contribuinte - O CSC corresponde a um codigo de seguranca '
                 'alfanumerico (16 a 36 bytes) de conhecimento apenas da Secretaria da Fazenda da '
                 'Unidade Federada do emitente e do proprio contribuinte, tambem chamado de '
                 '“Token”.'),
        'fispar_nfce_csc2': fields.char(
            'Identificacao do CSC 2', size=36,
            help='Codigo de Seguranca do Contribuinte - O CSC corresponde a um codigo de seguranca '
                 'alfanumerico (16 a 36 bytes) de conhecimento apenas da Secretaria da Fazenda da '
                 'Unidade Federada do emitente e do proprio contribuinte, tambem chamado de '
                 '“Token”.'),
        'fispar_nfce_csc_homologacao': fields.char(
            'CSC de homologacao', size=36,
            help='Codigo de Seguranca do Contribuinte a ser utilizado no ambiente de homologacao '
                 'quando a SEFAZ Autorizadora nao permite a utilizacao dos mesmos CSCs do ambiente '
                 'de producao (CSC 1 ou CSC 2).'),
        'fispar_nfce_tentativa_envio': fields.integer(
            'Numero de tentativas de envio NFC-e',
            help='Informar o numero de tentativas de envio de uma NFC-e para a SEFAZ Autorizadora '
                 'antes de assumir como contingencia offline.'),
        'fispar_nfce_conexao_timeout': fields.integer(
            'Tempo de espera da conexao SEFAZ',
            help='Informar o tempo (em segundos) para aguardar resposta de conexao da SEFAZ '
                 'Autorizadora antes de cancelar a conexao por "timeout".'),
        'fispar_nfce_impressora_danfe': fields.char(
            'Nome da impressora para o DANFE/NFC-e', size=30,
            help='Nome da impressora utilizada na impressao do DANFE/NFC-e'),
        'fispar_nfce_impressora_orientacao': fields.selection(
            [('portrait', _('Retrato')), ('landscape', _('Paisagem'))], 'Modo de impressao'),
        'fispar_nfce_impressora_formato': fields.selection(
            [('A4', 'A4 - 210 x 297 mm'), ('A5', 'A5 - 148 x 210 mm'), ('BO', 'Bobina - 80 mm')],
            'Formato de impressao'),
        'fispar_nfce_impressora_marca': fields.many2one(
            'fiscal.nfce.impressora', 'Marca/modelo', ondelete='cascade',
            help='Modelo da impressora utilizada na impressao (Termica/Bobina)'),
        'fispar_nfce_impressora_ids': fields.one2many(
            'fiscal.nfce.impressora', 'parametro_id', 'Impressoras', ondelete='restrict'),
        'fispar_nfce_natureza_id': fields.many2one(
            'fiscal.cfop.natureza', 'Natureza para NFC-e',
            help='Natureza a ser utilizada na importacao da NFC-e.'),
        'fispar_nfce_tempo_cancelamento': fields.float(
            'Tempo de cancelamento', digits=(3, 2),  # hh:mm
            help='Tempo maximo definido pela SEFAZ para cancelamento de uma NFC-e.'),
        'fispar_nfce_min_percent_tempo_cancelamento': fields.integer(
            'Minimo percentual de tempo de cancelamento',
            help='Percentual minimo definido para o limite de tempo de cancelamento. O sistema ira '
                 'emitir um alerta no painel de NFC-e pendentes caso o tempo ultrapasse o limite '
                 'que foi estabelecido.'),
        'fispar_nfce_max_percent_tempo_cancelamento': fields.integer(
            'Maximo percentual de tempo de cancelamento',
            help='Percentual maximo definido para o limite de tempo de cancelamento. O sistema ira '
                 'emitir alerta no painel de NFC-e pendentes caso o tempo ultrapasse o limite que '
                 'foi estabelecido.'),
        'fispar_nfce_tempo_envio_contingencia': fields.float(
            'Tempo para envio de NFC-e em contingencia offline.', digits=(3, 2),  # hh:mm
            help='Tempo maximo definido pela SEFAZ para envio de uma NFC-e emitida em contingencia '
                 'offline.'),
        'fispar_nfce_min_percent_tempo_envio_contingencia': fields.integer(
            'Minimo percentual de tempo para envio em contingencia offline',
            help='Percentual definido para o limite de tempo de envio de contingencia offline. O '
                 'sistema ira emitir alerta no painel de NFC-e pendentes caso o tempo ultrapasse o '
                 'limite que foi estabelecido.'),
        'fispar_nfce_max_percent_tempo_envio_contingencia': fields.integer(
            'Maximo percentual de tempo para envio em contingencia offline',
            help='Percentual definido para o limite de tempo de envio de contingencia offline. O '
                 'sistema ira emitir alerta no painel de NFC-e pendentes caso o tempo ultrapasse o '
                 'limite que foi estabelecido.'),
        'fispar_nfce_contingencia_inicio': fields.datetime('Data de inicio'),
        'fispar_nfce_contingencia_fim': fields.datetime('Data de termino'),

        'fispar_nfe_cnpj_responsavel_tecnico': fields.char(
            'CNPJ', size=14, help='CNPJ da pessoa juridica responsavel pelo sistema utilizado na '
                                  'emissao do documento fiscal eletronico'),
        'fispar_nfe_contato': fields.char(
            'Contato', size=60, help='Nome da pessoa a ser contatada na empresa desenvolvedora do '
                                     'sistema utilizado na emissao do documento fiscal '
                                     'eletronico.'),
        'fispar_nfe_email': fields.char(
            'E-Mail', size=60, help='e-Mail da pessoa a ser contatada na empresa desenvolvedora do '
                                    'sistema.'),
        'fispar_nfe_fone': fields.char(
            'Telefone', size=14, help='telefone da pessoa a ser contatada na empresa '
                                      'desenvolvedora do sistema. Preencher com o codigo DDD + '
                                      'numero do telefone.'),
        'fispar_nfe_csrt_id': fields.integer(
            'Identificador do CSRT', help='Numero sequencial crescente de idenficacao do CSRT.'),
        'fispar_nfe_csrt': fields.char(
            'Numero do CSRT', size=36,
            help='O CSRT corresponde a um codigo de seguranca alfanumerico (16 a 36 bytes) de '
                 'conhecimento apenas da Secretaria da Fazenda da Unidade Federada do emitente e '
                 'da empresa responsavel pelo sistema emissor de NF-e.'),
        'fispar_producao_propria_como_revenda': fields.boolean(
            'Producao propria como revenda',
            help = 'Se marcado vende os produtos de producao propria com o CFOP de revenda de '
                   'mercadorias de terceiros.'),
        'fispar_empresa_industria': fields.function(fispar_verifica_empresa_industria, method=True,
                                                    string='Industria', type='boolean'),
        'fispar_classificacao_estabelecimento': fields.selection(
            [('00', '00 - Industrial - Transformacao'),
             ('01', '01 - Industrial - Beneficiamento'),
             ('02', '02 - Industrial - Montagem'),
             ('03', '03 - Industrial - Acondicionamento ou Reacondicionalmente'),
             ('04', '04 - Industrial - Renovacao ou Recondicionamento'),
             ('05', '05 - Equiparado a industrial - Por opcao'),
             ('06', '06 - Equiparado a industrial - Importacao Direta'),
             ('07', '07 - Equiparado a industrial - Por lei especifica'),
             ('08', '08 - Equiparado a industrial - Nao enquadrado nos codigos 05, 06 ou 07'),
             ('09', '09 - Outros')], 'Classificacao do estabelecimento', size=2,
            help='Classificacao do estabelecimento conforme contribuintes do IPI.'),
        'fispar_data_conciliacao': fields.date(_('Data de conciliacao')),
        'fispar_deduz_icms_do_pis_cofins': fields.boolean(
            'Deduz ICMS da base do PIS/COFINS',
            help=_('Se marcado, o valor do ICMS destacado e deduzido no valor da base de calculo do'
                   ' PIS/COFINS.'))
    }

    _sql_constraints = [
        ('company_id_uniq', 'unique (company_id)', 'Parametros para a empresa ja cadastrados!'),
        ('dia_vencimento_obrigacao_icms',
         'CHECK (dia_vencimento_obrigacao_icms>=1 and dia_vencimento_obrigacao_icms<=31)',
         'O dia para vencimento do ICMS informado deve ser de 1 a 31!'),
    ]
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'versao_aplicativo_nfe': 'SigERP 1.0',
        'perfil_arquivo_sped': 'B',
        'quantidade_vias_danfe': 1,
        'fispar_tamanho_danfe' : 'A4',
        'fispar_base_reduzida': 'I',
        'fispar_nota_conjugada': True,
        'fispar_numero_nf_no_titulo': True,
        'fispar_indicador_nfe': '0',
        'fispar_ultimo_nsu': '0'.zfill(15),
        'fispar_indicador_emissor': '0',
        'fispar_enviar_email': False,
        'fispar_sped_fiscal_e200_manual': False,
        'fispar_producao_propria_como_revenda': True,
        'fispar_sublimite_estadual': 3600000,

        # NF-e
        'fispar_nfe_versao_xml': '4.00',
        'fispar_nfe_ambiente': 1,
        'fispar_nfe_indicador_presenca': '1',
        'fispar_nfe_cnpj_responsavel_tecnico': '01526558000178',
        'fispar_nfe_contato': 'Luiz Fernando Maciel Franca',
        'fispar_nfe_email': 'luiz@siginformatica.com.br',
        'fispar_nfe_fone': '3121061700',
        'fispar_nfe_csrt_id': 1,
        'fispar_gera_nfe_esboco': False,

        # NFC-e
        'fispar_nfce_tentativa_envio': 5,  # Numero maximo de tentativas de envio para a SEFAZ
        'fispar_nfce_conexao_timeout': 10,  # Tempo de espera (timeout) de conexao com a SEFAZ
        'fispar_nfce_impressora_formato': 'A5',
        'fispar_nfce_impressora_orientacao': 'portrait',
        'fispar_nfce_tempo_cancelamento': 2.0,  # Duas horas
        'fispar_nfce_min_percent_tempo_cancelamento': 20,  # 20%
        'fispar_nfce_max_percent_tempo_cancelamento': 50,  # 50%
        'fispar_nfce_tempo_envio_contingencia': 24.0,  # Vinte e quatro horas
        'fispar_nfce_min_percent_tempo_envio_contingencia': 20,  # 20%
        'fispar_nfce_max_percent_tempo_envio_contingencia': 50,  # 50%
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
