# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_tributacao(osv.Model):
    """
    Tabela com os percentuais de tributacao para cada encargo(imposto)
    """
    _name = 'fiscal.tributacao'
    _rec_name = 'name'
    _columns = {
        'id': fields.integer('Codigo', readonly=True),
        'name': fields.char('Nome', size=40, required=True), # Descricao da tributacao
        'encargo_id': fields.many2one("fiscal.encargo", 'Encargo', ondelete="cascade", required=True, select=True),
        'cst_id': fields.many2one("fiscal.cst", 'Cst', ondelete="restrict", required=True, select=True),
        'base': fields.float('Base', digits=(9, 6)), # Percentual da base tributada\n
        'aliquota': fields.float('Aliquota', digits=(7, 4)), # Percentual de tributacao do encargo
        'active': fields.boolean('Ativo'), # Indica se a tributacao esta ativa
        'dispositivo_fiscal': fields.text('Dispositivo fiscal'),
        'produto_tributacao_ids': fields.one2many('fiscal.tributacao.encargos', 'tributacao_id', 'Tributacao'),
        'fistri_cst_tipo_movimento': fields.related(
            'cst_id', 'fiscst_tipo_movimento', type="selection", relation='fiscal.cst',
            string="CST Tipo Movimento", store=False, required=False,
            selection=[('E', 'Entrada'), ('S', 'Saida'), ('A', 'Ambos')]
        ),
        'fistri_cst_tipo': fields.related(
            'cst_id', 'tipo', type='selection', relation='fiscal.cst', string='Tipo do cst',
            store=False, required=False, selection=[('T', 'Tributado'), ('U', 'Substituicao'),
                                                    ('I', 'Isento'), ('O', 'Outros'),
                                                    ('D', 'Diferido'), ('S', 'Servico'),
                                                    ('TU', 'Tributado com substituicao')]),
        'fistri_aliquota_fcp': fields.float(
            string='Aliquota para FCP',
            help='Aliquota para calculo do Fundo de erradicacao da miseria, esta aliquota ira '
                 'identificar o valor que sera destinado ao fundo, de acordo com o total de '
                 'ICMS'),
        'fistri_encargo_sigla': fields.related('encargo_id', 'sigla', type='char',
                                               string='Sigla encargo', readonly=True, store=False),
        'fistri_codigo_beneficio': fields.char('Codigo de Beneficio Fiscal', size=10,
                                               help='Apenas para ICMS'),
    }
    _defaults = {
        'aliquota': lambda *a: 0,
        'active': lambda *a: True,
        'base': 100,
    }
