# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv, fields


class fiscal_tributacao_encargos(osv.Model):
    """
    Tabela de tributacao
    """

    from funcoes import _get_csosn, _get_empresa_regime, _fisten_get_tipo_trib_cst

    _name = 'fiscal.tributacao.encargos'
    _rec_name = 'tributacao_id'
    _columns = {
        'encargo_id': fields.many2one("fiscal.encargo", 'Encargo ', ondelete="cascade",
                                      required=True, select=True),
        'cst_id': fields.many2one("fiscal.cst", 'Cst ', ondelete="restrict", required=True,
                                  select=True,
                                  domain="[('encargo_id','=',encargo_id), ('fiscst_tipo_movimento', 'in', fisten_tip_trib_cst)]"), # id do cst
        'tributacao_id': fields.many2one("fiscal.tributacao", 'Tributacao', ondelete="restrict", required=True,
                                         select=True,
                                         domain="[('cst_id', '=', cst_id), ('encargo_id','=',encargo_id)]"),
        'mva': fields.float('MVA', digits=(5, 2), required=False),
        'valor_pauta': fields.float('Valor Pauta', digits=(10, 2), required=False),
        'modbcst': fields.selection([('0', 'Preco tabelado'),
                                     ('1', 'Lista negativa'),
                                     ('2', 'Lista positiva'),
                                     ('3', 'Lista neutra'),
                                     ('4', 'Margem valor agregado'),
                                     ('5', 'Pauta'),
                                     ('6', 'Valor da operacao')
                                    ], 'Modal.determ. BC do ICMS ST',
                                    help="Modalidade de determinacao da Base de Calculo do ICMS de Substituicao Tributaria",
                                    size=1),
        'flag_company_tributacao_simples': fields.boolean('regime_simples'),
        'ncm_id': fields.many2one("fiscal.ncm", 'ncm', ondelete="cascade", select=True), # id do NCM
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="cascade", select=True), # id do produto
        'parametro_id': fields.many2one("fiscal.parametro", 'Parametro Fiscal', ondelete="cascade", select=True),
        'tipo_tributacao': fields.selection(
            [('T', 'Todas / Cons.Final'), ('P', 'Produtor rural'), ('C', 'Contribuinte'), ('E', 'Entradas')],
            'Tipo de tributacao', help='Este tipo de tributacao identifica em quais operacoes inside esta tributacao',
            size=1),
        'fisten_aliquota_venda': fields.float('Aliquota para venda', digits=(5, 2),
                                              help='Aliquota de ICMS na venda para verificacao de calculo do ICMS ST na compra.'),
        'fisten_csosn': fields.function(_get_csosn, type="char", size=20, method=True, store=False,
                                        string='Tributacao CSOSN'), #retorna a tributacao csosn
        'fisten_tip_trib_cst': fields.function(_fisten_get_tipo_trib_cst, type='char', size=20,
                                               method=True, store=False,
                                               string='Tipo de tributacao do CST')
    }
    _defaults = {
        'flag_company_tributacao_simples': _get_empresa_regime,
        'tipo_tributacao': 'T',
    }
    _sql_constraints = [
        ('tributacao_produto_uniq', 'unique (encargo_id, tipo_tributacao, produto_id)',
         'Nao e possivel cadastrar mais de uma Tributacao com o mesmo Encargo e o mesmo Tipo de Tributacao'.decode(
             'utf-8')),
        ('tributacao_uniq', 'unique (encargo_id, parametro_id)',
         'Nao e possivel cadastrar mais de uma Tributacao com o mesmo Encargo para a mesma Empresa'.decode('utf-8')),
        ('tributacao_ncm_uniq', 'unique (encargo_id, tipo_tributacao, ncm_id)',
         'Nao e possivel cadastrar mais de uma Tributacao com o mesmo Encargo e Tipo Tributacao para o mesmo NCM'.decode(
             'utf-8')),
    ]        
