#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class pgview_fiscal_produto_por_tributacao(osv.Model):

    _name = 'estoque.produto.por.tributacao.pgview'
    _auto = False
    _columns = {
        'pgv_estppt_company_id': fields.many2one('res.company', 'Empresa'),
        'pgv_estppt_produto_id': fields.many2one('estoque.produto', 'ID Produto'),
        'pgv_estppt_produto_codigo': fields.char('Codigo', size=20),
        'pgv_estppt_produto_name': fields.char('Produto', size=60),
        'pgv_estppt_produto_categ_id': fields.many2one('estoque.categoria', 'ID Categoria'),
        'pgv_estppt_ncm_id': fields.many2one('fiscal.ncm', 'ID Ncm'),
        'pgv_estppt_ncm_codigo': fields.char('Ncm', size=11),
        'pgv_estppt_categoria_name': fields.char('Categoria', size=50),
        'pgv_estppt_tributacao_id': fields.many2one('fiscal.tributacao.encargos', 'Tributacao Encargo'),
        'pgv_estppt_tributacao_name': fields.char('Tributacao', size=40),
        'pgv_estppt_encargo_name': fields.char('Encargo', size=45),
        'pgv_estppt_cst_cod_name': fields.char('Cst', size=50),
    }

    def init(self, cr):
        sql = """
            CREATE OR REPLACE VIEW estoque_produto_por_tributacao_pgview AS(
                SELECT
                    Row_number() OVER (order by produto.id) AS id,
                    produto.company_id AS pgv_estppt_company_id,
                    produto.id AS pgv_estppt_produto_id,
                    produto.codigo AS pgv_estppt_produto_codigo,
                    produto.name AS pgv_estppt_produto_name,
                    produto.categoria_id AS pgv_estppt_produto_categ_id,
                    ncm.id AS pgv_estppt_ncm_id,
                    ncm.codigo AS pgv_estppt_ncm_codigo,
                    categoria.name AS pgv_estppt_categoria_name,
                    tributacao.id AS pgv_estppt_tributacao_id,
                    tributacao.name AS pgv_estppt_tributacao_name,
                    encargo.name AS pgv_estppt_encargo_name,
                    cst.codigo || ' - ' || cst.name AS pgv_estppt_cst_cod_name
                FROM estoque_produto produto
                    INNER JOIN fiscal_tributacao_encargos trib_encargo
                        ON trib_encargo.produto_id = produto.id
                    LEFT JOIN fiscal_cst cst
                        ON cst.id = trib_encargo.cst_id
                    LEFT JOIN fiscal_tributacao tributacao
                        ON tributacao.id = trib_encargo.tributacao_id
                    LEFT JOIN fiscal_encargo encargo
                        ON encargo.id = trib_encargo.encargo_id
                    LEFT JOIN fiscal_ncm ncm
                        ON ncm.id = produto.ncm_id
                    LEFT JOIN estoque_categoria categoria
                        ON categoria.id = produto.categoria_id
                WHERE  produto.tipo = 'P'
                GROUP BY
                    produto.company_id, produto.id, produto.codigo, produto.name, produto.categoria_id,
                    ncm.id, ncm.codigo, categoria.name, tributacao.id, tributacao.name, encargo.name,
                    pgv_estppt_cst_cod_name
                ORDER BY
                    produto.codigo, produto.name, tributacao.name, pgv_estppt_cst_cod_name, encargo.name
            );
        """
        drop_view_if_exists(cr, 'estoque_produto_por_tributacao_pgview')
        cr.execute(sql)