# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.__init__ import CODIGO_UF, UF_CODIGO
from geraldo.barcodes import BarCode
from geraldo import Report
from base import *
import locale

locale.setlocale(locale.LC_ALL, 'pt_BR.utf8')
BG_TAG = HexColor(0xAFEEEE)


class CartaCorrecaoReport(Report):
    def __init__(self, queryset=None, logo=None, **kwargs):
        super(CartaCorrecaoReport, self).__init__(queryset)
        self.print_if_empty = True
        self.additional_fonts = FONTS
        self.page_size = kwargs.get('page_size') or PAGE_SIZE  # default = A4
        self.margin_top = MARGIN_TOP
        self.margin_bottom = MARGIN_BOTTOM
        self.margin_left = MARGIN_LEFT
        self.margin_right = MARGIN_RIGHT

        # Divisoes do relatorio
        self.band_page_header = Header(logo)
        self.band_detail = Detail()
        self.band_page_footer = Footer()

    # noinspection PyProtectedMember
    def do_on_new_page(self, page, page_number, generator):
        if generator._current_page_number != 1:
            self.band_page_footer = generator._is_latest_page and Footer()


class Header(BandDamdfe):
    height = 3 * cm
    borders = dict(all=RoundRect(stroke_color=BLACK, stroke_width=.7, radius=3))

    def __init__(self, logo):
        super(Header, self).__init__()
        _logo = isinstance(logo, StringIO) and logo or logo and StringIO(LOGO.decode('base64'))
        self.add_image(_logo, top=.2, left=.2, width=5, height=5),
        self.add_field('nfe.infNFe.emit.xNome.valor', top=0, left=3,  width=12, style=COMPANY_NAME),
        fld = self.add_field('nfe.infNFe.emit.enderEmit.xLgr.valor', top=.4, left=3, width=12,
                             style=DEFAULT)
        fld.get_value = lambda i: ', '.join([i.nfe.infNFe.emit.enderEmit.xLgr.valor,
                                             i.nfe.infNFe.emit.enderEmit.nro.valor,
                                             i.nfe.infNFe.emit.enderEmit.xBairro.valor])
        fld = self.add_field('nfe.infNFe.emit.enderEmit.xMun.valor', top=.8, left=3, width=12,
                             style=DEFAULT)
        fld.get_value = lambda i: '{}&nbsp;&nbsp;CEP: {}-{}'.format(
            ' - '.join([i.nfe.infNFe.emit.enderEmit.xMun.valor,
                        i.nfe.infNFe.emit.enderEmit.UF.valor]),
            str(i.nfe.infNFe.emit.enderEmit.CEP.valor)[:5],
            str(i.nfe.infNFe.emit.enderEmit.CEP.valor)[-3:])
        self.elements.append(BarCode(
            type='Code128', top=1.7*cm, left=12.25*cm, height=.9*cm, width=.022*cm,
            get_value=lambda i: i.retEnvEvento.infEvento.chNFe.valor))
        self.add_field('retEnvEvento.infEvento.chNFe.valor', top=2.6, left=12.5, width=12)
        self.add_label('C A R T A &nbsp;&nbsp;D E &nbsp;&nbsp;C O R R E Ç Ã O', top=2, left=5.3,
                       width=20, style=COMPANY_NAME)


class Detail(BandDamdfe):
    height = .4 * cm
    # borders = dict(all=RoundRect(stroke_color=GREEN))

    def __init__(self):
        super(Detail, self).__init__()

        # Primeira linha
        self.add_tag(u'Identificação', 'retEnvEvento.infEvento.Id.valor', top=0, left=0, width=5,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD)
        orgao_value = lambda i: u'{} - {}'.format(
            i.retEnvEvento.cOrgao.valor, CODIGO_UF[int(i.retEnvEvento.cOrgao.valor)])
        self.add_tag(u'Orgão', 'retEnvEvento.cOrgao.valor', top=0, left=5, width=2, height=1,
                     border=BLACK, radius=3, stroke_width=.7, style=DEFAULT, get_value=orgao_value,
                     label_style=DEFAULT_BOLD)
        amb_value = lambda i: u'{} - {}'.format(i.retEnvEvento.tpAmb.valor, {
            1: u'Produção', 2: u'Homologação'}[i.retEnvEvento.tpAmb.valor])
        self.add_tag(u'Ambiente', 'retEnvEvento.tpAmb.valor', top=0, left=7, width=6, height=1,
                     border=BLACK, radius=3, stroke_width=.7, style=DEFAULT, get_value=amb_value,
                     label_style=DEFAULT_BOLD)
        self.add_tag(u'Protocolo de autorização', 'retEnvEvento.infEvento.nProt.valor', top=0,
                     left=13, width=7, height=1, border=BLACK, radius=3, stroke_width=.7,
                     style=DEFAULT, label_style=DEFAULT_BOLD)

        # Segunda linha
        cnpj_value = lambda i: u'{}.{}.{}/{}-{}'.format(
            *[str(i.retEnvEvento.infEvento.CNPJDest.valor).zfill(14)[x:x + y] for x, y in zip(
                (0, 2, 5, 8, 12), (2, 3, 3, 4, 2))])
        self.add_tag(u'CNPJ', 'retEnvEvento.infEvento.CNPJDest.valor', top=1, left=0, width=3,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     get_value=cnpj_value, label_style=DEFAULT_BOLD)
        chave_value = lambda i: u'{}-{}/{}-{}.{}.{}/{}-{}-{}-{}-{}.{}.{}-{}.{}.{}'.format(
            *[str(i.retEnvEvento.infEvento.chNFe.valor).zfill(44)[x:x + y] for x, y in zip(
                (0, 2, 4, 6, 8, 11, 14, 18, 20, 22, 25, 28, 31, 34, 37, 40),
                (2, 2, 2, 2, 3,  3,  4,  2,  2,  3,  3,  3,  3,  3,  3,  4))])
        self.add_tag(u'Chave de acesso', 'retEnvEvento.infEvento.chNFe.valor', top=1, left=3,
                     width=10, height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     get_value=chave_value, label_style=DEFAULT_BOLD)
        data_value = lambda i: u'{}/{}/{} {}:{}:{} {}'.format(
            i.retEnvEvento.infEvento.dhRegEvento.valor[8:10],
            i.retEnvEvento.infEvento.dhRegEvento.valor[5:7],
            i.retEnvEvento.infEvento.dhRegEvento.valor[:4],
            i.retEnvEvento.infEvento.dhRegEvento.valor[11:13],
            i.retEnvEvento.infEvento.dhRegEvento.valor[14:16],
            i.retEnvEvento.infEvento.dhRegEvento.valor[17:19],
            i.retEnvEvento.infEvento.dhRegEvento.valor[-6:])
        self.add_tag(u'Data de autorização', 'retEnvEvento.infEvento.dhRegEvento.valor', top=1,
                     left=13, width=7, height=1, border=BLACK, radius=3, stroke_width=.7,
                     style=DEFAULT, label_style=DEFAULT_BOLD, get_value=data_value)

        # Terceira linha
        self.add_tag(u'Código do evento', 'retEnvEvento.infEvento.tpEvento.valor', top=2, left=0,
                     width=7, height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD)
        self.add_tag(u'Sequencial do evento', 'retEnvEvento.infEvento.nSeqEvento.valor', top=2,
                     left=7, width=6, height=1, border=BLACK, radius=3, stroke_width=.7,
                     style=DEFAULT, label_style=DEFAULT_BOLD)
        self.add_tag(u'Versão do evento', 'retEnvEvento.versao.valor', top=2, left=13, width=7,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD)

        # Quarta linha
        self.add_tag(u'Nota Fiscal', 'nfe.infNFe.ide.cNF.valor', top=3, left=0, width=2,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD)
        self.add_tag(u'Série', 'nfe.infNFe.ide.serie.valor', top=3, left=2, width=1,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD)
        data_value = lambda i: i.nfe.infNFe.ide.dhEmi.valor.strftime('%d/%m/%Y %H:%M:%S')
        self.add_tag(u'Data de emissão', 'nfe.infNFe.ide.dhEmi.valor', top=3, left=3, width=4,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD, get_value=data_value)
        self.add_tag(u'Valor', 'nfe.infNFe.total.ICMSTot.vNF.valor', top=3, left=7, width=2,
                     height=1, border=BLACK, radius=3, stroke_width=.7, style=DEFAULT,
                     label_style=DEFAULT_BOLD)

        # Qunita linha
        self.add_tag(u'Texto da Carta de Correção', 'envEvento.infEvento.detEvento.xCorrecao.valor',
                     top=4, left=0, width=20, height=18.3, border=BLACK, radius=3, stroke_width=.7,
                     style=DEFAULT, label_style=DEFAULT_BOLD)


class Footer(BandDamdfe):
    height = 3.4 * cm
    # borders = dict(all=RoundRect(stroke_color=AQUA))

    def __init__(self):
        super(Footer, self).__init__()
        # Sexta linha
        self.add_tag(u'Condições de uso da Carta de Correção',
                     'envEvento.infEvento.detEvento.xCondUso.valor', top=0, left=0, width=20,
                     height=3.4, border=BLACK, radius=3, stroke_width=.7,
                     style=DEFAULT, label_style=DEFAULT_BOLD)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
