#!/usr/bin/python
# -*- encoding: utf-8 -*-

from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
import time
from openerp import pooler
from openerp.tools.translate import _
from datetime import datetime


class external_pdf(render):
    """
    Classe responsavel por renderizar o relatorio de sugestao de precos
    """

    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    """
    Classe responsavel pela escrita do relatorio de sugestao de precos
    """

    def create(self, cr, uid, ids, datas, context=None):
        """
        :param cr: Cursor do openerp
        :param uid: ID do usuario que esta executando a operacao
        :param ids: Ids do registro
        :param datas: Dicionario contendo os dados para trabalho do relatorio
        :param context: Dicionario contendo valore contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: list
        :type datas: dict
        :type context: dict
        """
        context = context and context.copy() or {}

        def center_title(E, C, D, T=160):
            title_c = (T - len(C)) / 2
            title = E
            for i in range(title_c - len(E)):
                title += ' '
            title += C
            for i in range(title_c - len(D)):
                title += ' '
            title += D
            return title

        def _repcus_escreve_relatorio_cab(string_custo, custo_selecionado):
            """
            Metodo responsavel por escrever o cabecalho do relatorio
            :param string_custo: String do Custo a ser impresso no relatorio
            :param custo_selecionado: Custo selecionado para o relatorio
            :type string_custo: str
            :type custo_selecionado: str
            """
            res_company_dados = pooler.get_pool(cr.dbname).get('res.company').read(cr, uid,
                                                                                   company_id,
                                                                                   ['name'],
                                                                                   context=context)
            nome_empresa = res_company_dados.get('name')
            data = time.strftime("%d/%m/%Y")
            docs = center_title(data, nome_empresa, 'Pagina: {{ Page }} ')

            listagem.informaCab('' + title)
            listagem.informaCab('')
            listagem.informaCab('' + docs)
            listagem.informaCab(
                '*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------*')
            cabecalho = '|' + formataString('CODIGO', 15, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('GR.', 3, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('DESCRICAO', 40, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('UND', 3, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('EMBAL.', 6, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('ICMS', 4, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('ALIQ.', 5, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('QTD', 7, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('%s' % string_custo, 11, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('C. ANTERIOR', 11, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('%s' % custo_selecionado, 11, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('P.VENDA', 7, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('ATUAL%', 6, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('IDEAL%', 6, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('MINIMA%', 7, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('BRUTA%', 6, ' ', 'C', 'N', 'N')
            cabecalho += '|' + formataString('SUGESTAO', 8, ' ', 'C', 'N', 'N')
            cabecalho += formataString('|', 1, ' ', 'E', 'N', 'N')
            listagem.informaCab(cabecalho)
            listagem.informaCab(
                '*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------*')
            listagem.rodapePadrao()

        def _repcus_escreve_relatorio(item):
            """
            Este metodo e responsavel por escrever os dados do relatorio
            :param item: Dicionario contendo os valores do registro
            """
            linha = formataString(item.get('fisspi_codigo')[:15] or '', 15, ' ', 'D', 'N',
                                  'S') + '|'
            linha += formataString(item.get('fisspi_grade_sigla') or '', 3, ' ', 'D', 'N',
                                   'S') + '|'
            linha += formataString(item.get('fisspi_descricao_produto') or '', 40, ' ', 'D', 'N',
                                   'S') + '|'
            linha += formataString(item.get('fisspi_unidade_sigla') or '', 3, ' ', 'D', 'N',
                                   'S') + '|'
            linha += formataString(item.get('fisspi_quantidade_embalagem') or '', 6, ' ', 'E', 'N',
                                   'S') + '|'
            linha += formataString(item.get('fisspi_tipo_icms'), 4, ' ', 'C', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_aliquota') or 0.0, 2), 5, ' ',
                                   'E', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_qtd') or 0.0, 2), 7, ' ',
                                   'E', 'N', 'S') + '|'
            linha += formataString(
                getNumeroEditado(item.get('fisspi_preco_custo_parametro') or 0.0, 2), 11, ' ', 'E',
                'N', 'S') + '|'
            linha += formataString(
                getNumeroEditado(item.get('fisspi_preco_custo_anterior') or 0.0, 2), 11, ' ', 'E',
                'N', 'S') + '|'
            linha += formataString(
                getNumeroEditado(item.get('fisspi_preco_custo_selecionado') or 0.0, 2), 11, ' ',
                'E', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_preco_venda') or 0.0, 2), 7,
                                   ' ',
                                   'E', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_margem_atual') or 0.0, 2), 6,
                                   ' ',
                                   'E', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_margem_ideal') or 0.0, 2), 6,
                                   ' ',
                                   'E', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_margem_minima') or 0.0, 2), 7,
                                   ' ',
                                   'E', 'N', 'S') + '|'
            linha += formataString(getNumeroEditado(item.get('fisspi_margem_bruta') or 0.0, 2), 6,
                                   ' ',
                                   'E', 'N', 'S') + '|'
            if item.get('fisspi_encargo_venda'):
                linha += formataString(getNumeroEditado(item.get('fisspi_sugestao') or 0.0, 2), 8,
                                       ' ', 'E', 'N', 'S') + '|'
            else:
                linha += formataString(getNumeroEditado(item.get('fisspi_sugestao') or 0.0, 2) +
                                       "*", 8, ' ', 'E', 'N', 'S') + '|'
            listagem.imprimeLinha('|' + linha)

        def _repcus_rodape_relatorio():
            """
            Este metodo e responsavel por escrever o rodape do relatorio
            """
            listagem.imprimeLinha(
                '*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------*')
            listagem.imprimeLinha(
                '                                                                                                                            ** Sig Informatica LTDA **       ')

        # Principal
        sugestao_preco = datas.get('sugestao')
        sugestao_preco_itens = datas.get('sugestao_itens')
        title = center_title('', _('Sugestao de Precos'), '')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='1')
        listagem.informaNome(pdf_string, _('Sugestao de Precos'))
        listagem.informaTamLinha(160)
        listagem.defineFonte(fonte='Courier', tam=7.8)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)
        custo_listar = sugestao_preco.get('fissup_custo')
        if custo_listar == 'R':
            custo_selecionado = 'REPOSICAO'
        else:
            custo_selecionado = 'CORRIGIDO'
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'wizard.sugestao.precos.osv', context=context)
        string_custo = sugestao_preco.get('fissup_string_custo')
        _repcus_escreve_relatorio_cab(string_custo, custo_selecionado)
        numero_nf = ''
        nf_ids = []
        for i in sugestao_preco_itens:
            if i.get('fisspi_nf_id'):
                nf_ids.append(i.get('fisspi_nf_id'))

        nf_fornecedor = {}
        if nf_ids:
            map(lambda x: nf_fornecedor.update({x.get('id'): x.get('destinatario_id')[1]}),
            pooler.get_pool(cr.dbname).get('fiscal.nf.capa').read(cr, uid, nf_ids,
                                                              ['destinatario_id'], context=context))
        primeira = True
        str_data = 'Sem data'
        if sugestao_preco.get('fissup_data_entrada'):
            str_data = datetime.strftime(datetime.strptime(sugestao_preco.get('fissup_data_entrada'),
                                                           '%Y-%m-%d'), '%d/%m/%Y')
        for item in sugestao_preco_itens:
            if numero_nf != item.get('fisspi_numero_nf'):
                numero_nf = item.get('fisspi_numero_nf')
                if numero_nf:
                    if not primeira:
                        listagem.imprimeLinha('|'.ljust(173,'-') + '|')

                    primeira = False
                    listagem.imprimeLinha((_('|Nota Fiscal: %s  Data de Entrada: %s') % (numero_nf, str_data)).ljust(173, ' ') + '|')
                    listagem.imprimeLinha((_('|Fornecedor: %s') % nf_fornecedor.get(
                                                  item.get('fisspi_nf_id'))).ljust(173, ' ') + '|')

                listagem.imprimeLinha('|'.ljust(173) + '|')
            _repcus_escreve_relatorio(item)
        _repcus_rodape_relatorio()
        listagem.finalizaImpressao()
        self.obj_pdf = external_pdf(pdf_string.getvalue())
        self.obj_pdf.render()
        pdf_string.close()
        return self.obj_pdf.pdf, 'pdf'


report_custom('report.sugestao_precos')
