#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.tools import mute_logger
from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from random import randint
from datetime import date


# noinspection PyUnresolvedReferences
class TestEstoqueInventarioContagem(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestEstoqueInventarioContagem, self).setUp()
        self.funcionario_id = self.registry('rh.funcionario').get_id_func(self.cr, self.uid)
        self.saldo = 10
        self.company_id = 1
        self.local_id = self.registry('estoque.local').create(
            self.cr, self.uid, self.get_vals_local())
        self.produto_id = self.registry('estoque.produto').create(
            self.cr, self.uid, self.get_vals_produto())
        self.grade_id = self.registry('estoque.produto.grade').create(
            self.cr, self.uid, self.get_vals_grade(self.produto_id))
        self.locacao_id = self.registry('estoque.local.locacao').create(
            self.cr, self.uid, self.get_vals_locacao(self.grade_id, self.local_id, self.produto_id))
        self.invcap = self.registry('estoque.inventario.capa')
        self.invcont = self.registry('estoque.inventario.contagem')
        wizgerarcontagem = self.registry('estoque.gerar.contagem.osv')
        descricao = 'teste' + str(randint(1, 1000))
        self.ano = str(date.today().year)
        self.mes = str(date.today().month).zfill(2)
        vals_inventario = dict(name=descricao, descricao=descricao,
                               local=self.local_id, mes=self.mes, ano=self.ano,
                               company_id=1)
        wizcontagem_id = wizgerarcontagem.create(self.cr, self.uid, vals_inventario, context={})
        wizgerarcontagem._gerar_inventario_capa(self.cr, self.uid, wizcontagem_id, context={})
        self.data_inventario = wizgerarcontagem._get_data_movimento(
            False, self.ano, self.mes)
        self.inventario_id = self.invcap.search(
            self.cr, self.uid, [('descricao', '=', descricao), ('ano', '=', self.ano),
                                ('mes', '=', self.mes)])[0]

    def test_salvar_contagem_sem_icms(self):
        vals = dict(inventario_id=self.inventario_id, produto_id=self.produto_id, grade_id=self.grade_id,
                    local_id=self.local_id, saldo_contado=self.saldo, saldo_cadastro=0)
        id_contagem = self.invcont.salvar_contagem(
            self.cr, self.uid, inventario_id=self.inventario_id, produto_id=self.produto_id,
            grade_id=self.grade_id, local_id=self.local_id, vals=vals)
        dados_contagem = self.invcont.browse(self.cr, self.uid, id_contagem)
        self.assertIsInstance(id_contagem, (int, long))
        self.assertFalse(dados_contagem.estict_tributacao_id.id)

    def test_salvar_contagem_com_icms(self):
        icms_id = self.registry('fiscal.encargo').search(self.cr, self.uid,
                                                         [('sigla', '=', 'ICMS')])[0]
        cst_id = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('codigo', '=', '00'), ('encargo_id', '=', icms_id)])[0]
        tributacao_id = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('cst_id', '=', cst_id)], limit=1)[0]
        vals_tribenc = dict(
            cst_id=cst_id, encargo_id=icms_id, produto_id=self.produto_id, tipo_tributacao='T',
            tributacao_id=tributacao_id)
        self.registry('fiscal.tributacao.encargos').create(self.cr, self.uid, vals_tribenc)
        vals_tribenc = dict(
            cst_id=cst_id, encargo_id=icms_id, produto_id=self.produto_id, tipo_tributacao='E',
            tributacao_id=tributacao_id)
        self.registry('fiscal.tributacao.encargos').create(self.cr, self.uid, vals_tribenc)
        vals = dict(inventario_id=self.inventario_id, produto_id=self.produto_id, grade_id=self.grade_id,
                    local_id=self.local_id, saldo_contado=self.saldo, saldo_cadastro=0)
        id_contagem = self.invcont.salvar_contagem(
            self.cr, self.uid, inventario_id=self.inventario_id, produto_id=self.produto_id,
            grade_id=self.grade_id, local_id=self.local_id, vals=vals)
        dados_contagem = self.invcont.browse(self.cr, self.uid, id_contagem)
        self.assertIsInstance(id_contagem, (int, long))
        self.assertEqual(dados_contagem.estict_tributacao_id.id, tributacao_id)

    def get_local_id(self, nome_local):
        return self.registry('estoque.local').search(self.cr, self.uid, [('name', '=', nome_local)])

    def get_vals_produto(self, grade_unica=True):
        nome_prod = 'teste' + str(randint(1, 1000))
        return dict(company_id=self.company_id, name=nome_prod, origem_mercadoria='0',
                    tipo_item='00', grade_usar=grade_unica)

    def get_vals_grade(self, produto_id, grade='UN'):
        nome_grade = 'teste' + str(randint(1, 1000))
        return dict(descricao=nome_grade, grade=grade, produto_id=produto_id)

    def get_vals_local(self):
        local = self.registry('estoque.local')
        nome_local = 'teste' + str(randint(1, 1000))
        centro_custo_id = self.registry('adm.empresa.centrocusto').search(
            self.cr, self.uid, [], limit=1)[0]
        local_maior_numero_id = local.search(self.cr, self.uid, [], order='numero desc', limit=1)[0]
        local_numero = local.browse(self.cr, self.uid, local_maior_numero_id).numero + 1

        return dict(centrocusto_id=centro_custo_id, company_id=self.company_id,
                    name=nome_local, numero=local_numero)

    def get_vals_locacao(self, grade_id, local_id, produto_id):
        return dict(company_id=self.company_id, grade_id=grade_id, local_id=local_id,
                    produto_id=produto_id)
