#!/usr/bin/python
# -*- encoding: utf-8 -*-
from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from datetime import datetime
from openerp.osv.osv import except_osv
from ...estoque.test.test_produto import t_criar_produto

class TestFiscalNfCapa(TransactionCase):
    """
    Classe de testes unitarios da class fiscal.nf
    """

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestFiscalNfCapa, self).setUp()
        self.agora = datetime.now().strftime('%Y-%m-%d %H:%m:%S')
        self.company_id = 1
        self.produto = self.registry('estoque.produto')
        self.nf = self.registry('fiscal.nf.capa')
        self.grade = self.registry('estoque.produto.grade')
        self.produto_id = self._criar_produto('DESTINATARIO DE TESTE ÁÇÃO%°*á')
        self.local_id = self._get_local('LOJA')[0]
        self.grade_id = self.grade.search(self.cr, self.uid,
                                          [('produto_id', '=', self.produto_id)])[0]
        self.produto_embalagem_id = self._cria_produto_embalagem()
        self.nf_id = self._cria_nota()
        self.item_ids = self.nf.read(self.cr, self.uid, self.nf_id,
                                     ['estoque_movimento_itens_ids']).get(
            'estoque_movimento_itens_ids')
        self.itens = self.registry('estoque.movimento.itens').read(
            self.cr, self.uid, self.item_ids, [])

    def test_fisnfc_onchange_movimento_itens_01(self):
        """
        Testa fisnfc_onchange_movimento_itens, sem a nota estar gravada
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [], []), {})

    def test_fisnfc_onchange_movimento_itens_02(self):
        """
        Testa fisnfc_onchange_movimento_itens, apenas com o valor do item
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [self.nf_id], self.itens), {'value': {'vr_total_nf': 10.0}})

    def test_fisnfc_onchange_movimento_itens_03(self):
        """
        Testa fisnfc_onchange_movimento_itens, Com o valor do item + valor do seguro
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        self.nf.write(self.cr, self.uid, self.nf_id, {'vr_seguro': 5.54})
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [self.nf_id], self.itens), {'value': {'vr_total_nf': 15.54}})

    def test_fisnfc_onchange_movimento_itens_04(self):
        """
        Testa fisnfc_onchange_movimento_itens, Com o valor do item + valor de frete
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        self.nf.write(self.cr, self.uid, self.nf_id, {'vr_frete': 5.54})
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [self.nf_id], self.itens), {'value': {'vr_total_nf': 15.54}})

    def test_fisnfc_onchange_movimento_itens_05(self):
        """
        Testa fisnfc_onchange_movimento_itens, Com o valor do item + valor de despesas
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        self.nf.write(self.cr, self.uid, self.nf_id, {'vr_despesas_acessorias': 5.54})
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [self.nf_id], self.itens), {'value': {'vr_total_nf': 15.54}})

    def test_fisnfc_onchange_movimento_itens_06(self):
        """
        Testa fisnfc_onchange_movimento_itens, Com o valor do item + valor de ipi
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        ipi_id = self._get_encargo('IPI')
        natureza_id = self._get_natureza('5102')
        brw_natureza = self.registry('fiscal.cfop.natureza').browse(self.cr, self.uid, natureza_id)
        nat_ipi = filter(lambda i: i.encargo_id.sigla == 'IPI', brw_natureza.natureza_encargos_ids)
        if not nat_ipi:
            self.registry('fiscal.cfop.natureza.encargos').create(
                self.cr, self.uid, {'encargo_id': ipi_id, 'natureza_id': natureza_id})
        cst_id = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('tipo', '=', 'T'), ('encargo_id', '=', ipi_id),
                                ('codigo', '=', '50')])[0]
        tributacao_id = self.registry('fiscal.tributacao').create(
            self.cr, self.uid,
            dict(cst_id=cst_id, encargo_id=ipi_id, name='TESTE TRIB 5 %', base=100, aliquota=5))
        fisnit = self.registry('fiscal.nf.item.tributacao')
        vr_base_calculo = fisnit.fisnit_calcula_vr_base_calculo(10)
        vr_encargo = fisnit.fisnit_calcula_vr_encargo(vr_base_calculo, 5)
        vals_tributacao = dict(aliquota=5, cst_id=cst_id, encargo_id=ipi_id,
                               tributacao_id=tributacao_id, vr_base_calculo=vr_base_calculo,
                               vr_encargo=vr_encargo, movimento_itens_id=self.item_ids[0])
        self.registry('estoque.movimento.itens').write(
            self.cr, self.uid, self.item_ids, dict(tributacoes_ids=[[0, False, vals_tributacao]]))
        self.nf.write(self.cr, self.uid, self.nf_id, {})
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [self.nf_id], self.itens), {'value': {'vr_total_nf': 10.50}})

    def test_fisnfc_onchange_movimento_itens_07(self):
        """
        Testa fisnfc_onchange_movimento_itens, Com o valor do item - valor do desconto
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        self.nf.write(self.cr, self.uid, self.nf_id, {'vr_desconto': 5.54})
        self.assertDictEqual(self.nf.fisnfc_onchange_movimento_itens(
            self.cr, self.uid, [self.nf_id], self.itens), {'value': {'vr_total_nf': 4.46}})

    def test_fisnfc_calcula_peso01(self):
        """
        Testa fisnfc_fisnfc_calcula_peso, quantidade item - 1
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        vals_produto_embalagem = dict(peso_liquido=10, peso_bruto=12)
        self.registry('estoque.produto.embalagem').write(
            self.cr, self.uid, self.produto_embalagem_id, vals_produto_embalagem)
        self.assertTrue(self.nf.fisnfc_atualiza_peso(self.cr, self.uid, self.nf_id))
        dados_nota = self.nf.read(self.cr, self.uid, self.nf_id, ['peso_liquido', 'peso_bruto'])
        self.assertEqual(dados_nota.get('peso_liquido'), 10)
        self.assertEqual(dados_nota.get('peso_bruto'), 12)

    def test_fisnfc_calcula_peso02(self):
        """
        Testa fisnfc_fisnfc_calcula_peso, quantidade item  - 2
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        vals_produto_embalagem = dict(peso_liquido=10, peso_bruto=12)
        self.registry('estoque.produto.embalagem').write(
            self.cr, self.uid, self.produto_embalagem_id, vals_produto_embalagem)
        self.registry('estoque.movimento.itens').write(self.cr, self.uid, self.item_ids,
                                                       vals=dict(quantidade=2))
        self.assertTrue(self.nf.fisnfc_atualiza_peso(self.cr, self.uid, self.nf_id))
        dados_nota = self.nf.read(self.cr, self.uid, self.nf_id, ['peso_liquido', 'peso_bruto'])
        self.assertEqual(dados_nota.get('peso_liquido'), 20)
        self.assertEqual(dados_nota.get('peso_bruto'), 24)

    def test_fisnfc_calcula_peso_manual03(self):
        """
        Testa fisnfc_fisnfc_calcula_peso, lancamento manual
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        vals_produto_embalagem = dict(peso_liquido=10, peso_bruto=12)
        self.registry('estoque.produto.embalagem').write(
            self.cr, self.uid, self.produto_embalagem_id, vals_produto_embalagem)
        self.registry('estoque.movimento.itens').write(self.cr, self.uid, self.item_ids,
                                                       vals=dict(quantidade=2))
        self.nf.write(self.cr, self.uid, self.nf_id, dict(peso_liquido=3, peso_bruto=4))
        dados_nota = self.nf.read(self.cr, self.uid, self.nf_id, ['peso_liquido', 'peso_bruto'])
        self.assertEqual(dados_nota.get('peso_liquido'), 3)
        self.assertEqual(dados_nota.get('peso_bruto'), 4)

    # Metodos para criar a nota
    def _cria_destinatario(self):
        pessoa = self.registry('adm.pessoa')
        vals = dict(pessoa_tipo='F', name='DESTINATARIO DE TESTE ÁÇÃO%°*á', cpf_cnpj='47078736770')
        return pessoa.create(self.cr, self.uid, vals)

    def _cria_produto_embalagem(self):
        embalagem_id = self.registry('estoque.embalagem').search(
            self.cr, self.uid, [('sigla', '=', 'UN')], limit=1)[0]
        vals = dict(embalagem_id=embalagem_id, produto_id=self.produto_id)
        return self.registry('estoque.produto.embalagem').create(
            self.cr, self.uid, vals)

    def _criar_produto(self, descricao):
        ncm_id = self.registry('fiscal.ncm').search(self.cr, self.uid, [], limit=1)[0]
        vals_produto = dict(name=descricao, ncm_id=ncm_id, tipo='P')
        return self.produto.create(self.cr, self.uid, vals_produto)

    def _get_local(self, nome_local):
        return self.registry('estoque.local').search(self.cr, self.uid, [('name', '=', nome_local)])

    def _cria_locacao(self):
        vals = dict(company_id=self.company_id, grade_id=self.grade_id, local_id=self.local_id,
                    produto_id=self.produto_id)
        return self.registry('estoque.local.locacao').create(self.cr, self.uid, vals)

    def _get_cfop(self, cfop):
        return self.registry('fiscal.cfop').search(self.cr, self.uid, [('codigo', '=', cfop)],
                                                   limit=1)[0]

    def _get_natureza(self, natureza):
        cfop_id = \
        self.registry('fiscal.cfop').search(self.cr, self.uid, [('codigo', '=', natureza)],
                                            limit=1)[0]
        return self.registry('fiscal.cfop.natureza').search(self.cr, self.uid,
                                                            [('cfop_id', '=', cfop_id)])[0]

    def _cria_nota(self):
        destinatario_id = self._cria_destinatario()
        icms_id = self._get_encargo('ICMS')
        cst_id = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('tipo', '=', 'T'), ('encargo_id', '=', icms_id),
                                ('codigo', '=', '00')])[0]
        tributacao_id = self.registry('fiscal.tributacao').create(
            self.cr, self.uid,
            dict(cst_id=cst_id, encargo_id=icms_id, name='TESTE TRIB 18 %', base=100, aliquota=18))
        fisnit = self.registry('fiscal.nf.item.tributacao')
        vr_base_calculo = fisnit.fisnit_calcula_vr_base_calculo(10)
        vr_encargo = fisnit.fisnit_calcula_vr_encargo(vr_base_calculo, 18)
        vals_tributacao = dict(aliquota=18, cst_id=cst_id, encargo_id=icms_id,

                               tributacao_id=tributacao_id, vr_base_calculo=vr_base_calculo,
                               vr_encargo=vr_encargo)
        vals_itens = dict(cfop_id=self._get_cfop('5102'), grade_id=self.grade_id,
                          produto_id=self.produto_id, local_id=self.local_id, quantidade=1.0,
                          produto_embalagem_id=self.produto_embalagem_id, vr_total_item=10,
                          tributacoes_ids=[[0, False, vals_tributacao]])
        condicao_forma_id = self.registry('adm.condicao.forma').search(
            self.cr, self.uid, [], limit=1)[0]
        forma_pagto_id = self.registry('adm.forma.pgto').search(self.cr, self.uid, [], limit=1)[0]
        vals_condicao = dict(condicao_forma_id=condicao_forma_id, forma_pagto_id=forma_pagto_id)
        vals = dict(company_id=self.company_id, tipo='S', destinatario_id=destinatario_id,
                    data_emissao=self.agora, natureza_id=self._get_natureza('5102'),
                    nfe_forma_impressao_danfe='1',
                    estoque_movimento_itens_ids=[[0, False, vals_itens]],
                    finalidade_emissao='1', nf_condicao_ids=[[0, False, vals_condicao]])
        nf_id = self.nf.create(self.cr, self.uid, vals)
        return nf_id

    def _get_encargo(self, sigla):
        return self.registry('fiscal.encargo').search(self.cr, self.uid, [('sigla', '=', sigla)])[0]