#!/usr/bin/python
# -*- coding: utf-8 -*-

__version__ = '1.0'
__author__ = 'thiago.rodrigues'
__email_author__ = 'thiago.rodrigues[at]siginformatica.com.br'

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from random import randint
from openerp.osv.osv import except_osv


class TestFiscalTributacao(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestFiscalTributacao, self).setUp()
        self.tributacao = self.registry('fiscal.tributacao')

    def test_create(self):
        vals = self.get_vals_tributacao('ICMS', '00', 12.0, 100.0)
        tributacao_id = self.tributacao.create(self.cr, self.uid, vals)
        brw_tributacao = self.tributacao.browse(self.cr, self.uid, tributacao_id)
        self.assertEqual(brw_tributacao.encargo_id.sigla, 'ICMS')
        self.assertEqual(brw_tributacao.cst_id.codigo, '00')
        self.assertEqual(brw_tributacao.base, 100)
        self.assertEqual(brw_tributacao.aliquota, 12.0)

    def test_create_com_aliquota_fem(self):
        vals = self.get_vals_tributacao('ICMS', '00', 12.0, 100.0)
        vals['fistri_aliquota_fcp'] = 2.0
        tributacao_id = self.tributacao.create(self.cr, self.uid, vals)
        brw_tributacao = self.tributacao.browse(self.cr, self.uid, tributacao_id)
        self.assertEqual(brw_tributacao.encargo_id.sigla, 'ICMS')
        self.assertEqual(brw_tributacao.cst_id.codigo, '00')
        self.assertEqual(brw_tributacao.base, 100)
        self.assertEqual(brw_tributacao.aliquota, 12.0)
        self.assertEqual(brw_tributacao.fistri_aliquota_fcp, 2.0)

    def test_unlink(self):
        vals = self.get_vals_tributacao('ICMS', '00', 12.0, 100.0)
        tributacao_id = self.tributacao.create(self.cr, self.uid, vals)
        with self.assertRaises(except_osv):
            self.tributacao.unlink(self.cr, self.uid, tributacao_id)

    def _criar_encargo(self):
        encargo_id = self.registry('fiscal.encargo').search(self.cr, self.uid,
                                                            [('sigla', '=', 'ICMS')], context={})
        if encargo_id:
            return encargo_id[0]

        return self.registry('fiscal.encargo').create(self.cr, self.uid, {'name': 'Icms',
                                                                          'sigla': 'ICMS'}, {})

    def test_fistri_onchange_encargo_com_encargo(self):
        vals = self.get_vals_tributacao('ICMS', '00', 12.0, 100.0)
        tributacao_id = self.tributacao.create(self.cr, self.uid, vals)
        encargo_id = self._criar_encargo()
        self.assertDictEqual(self.tributacao.fistri_onchange_encargo(
            self.cr, self.uid, [tributacao_id], encargo_id, context={}),
                             {'value': {'cst_id': False, 'fistri_encargo_sigla': u'ICMS'}})

    def test_fistri_onchange_encargo_sem_encargo(self):
        vals = self.get_vals_tributacao('ICMS', '00', 12.0, 100.0)
        tributacao_id = self.tributacao.create(self.cr, self.uid, vals)
        self.assertDictEqual(self.tributacao.fistri_onchange_encargo(
            self.cr, self.uid, [tributacao_id], False, context={}),
                             {'value': {'cst_id': False}})

    def test_fistri_onchange_cst(self):
        vals = self.get_vals_tributacao('ICMS', '00', 12.0, 100.0)
        tributacao_id = self.tributacao.create(self.cr, self.uid, vals)
        cst_id = self.tributacao.browse(self.cr, self.uid, tributacao_id).cst_id.id
        self.assertDictEqual(
            self.tributacao.fistri_onchange_cst(
                self.cr, self.uid, [tributacao_id], cst_id, context={}),
            {'value': {'fistri_cst_tipo_movimento': 'A', 'fistri_cst_tipo': 'T'}})

    def get_vals_tributacao(self, sigla_icms, cst, aliquota, base):
        encargo_id = self.registry('fiscal.encargo').search(self.cr, self.uid,
                                                            [('sigla', '=', sigla_icms.upper())])[0]
        cst_id = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', encargo_id), ('codigo', '=', cst)])[0]
        vals = dict(aliquota=aliquota, base=base, encargo_id=encargo_id, cst_id=cst_id,
                    name='TESTE TRIBUTACAO' + str(randint(1, 1000)))
        return vals