#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigaddons.config_teste import set_config
from openerp.tests.common import TransactionCase
import psycopg2


# noinspection PyUnresolvedReferences
class TestTributacaoInterEstadual(TransactionCase):
    """
    Classe de testes unitarios da class fiscal.tributacao.interestadual
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """

    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestTributacaoInterEstadual, self).setUp()
        self.fistrib_interestadual = self.registry('fiscal.tributacao.interestadual')
        self.uf = self.registry('adm.uf')
        self.encargo = self.registry('fiscal.encargo')
        self.fistrib = self.registry('fiscal.tributacao')
        self.cst = self.registry('fiscal.cst')
        vals = self.get_vals()
        self.fistrib_inter_id = self.fistrib_interestadual.create(self.cr, self.uid, vals)

    def get_vals(self):
        uf_id = self.uf.search(self.cr, self.uid, [('sigla', '=', 'SP')])[0]
        icms_id = self.encargo.search(self.cr, self.uid, [('sigla', '=', 'ICMS')])[0]
        cst_id = self.cst.search(self.cr, self.uid,
                                 [('tipo', '=', 'T'), ('encargo_id', '=', icms_id),
                                  ('codigo', '=', '00')])[0]
        trib_7_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=icms_id,
                                                                name='TESTE TRIB 7 %', base=100,
                                                                aliquota=7))
        trib_12_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=icms_id,
                                                                 name='TESTE TRIB 12 %', base=100,
                                                                 aliquota=12))
        trib_18_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=icms_id,
                                                                 name='TESTE TRIB 18 %', base=100,
                                                                 aliquota=18))
        return dict(fistin_uf=uf_id, fistin_encargo=icms_id,
                    fistin_cod_tribut_interestadual_nac=trib_7_id,
                    fistin_cod_tribut_interno=trib_12_id,
                    fistin_cod_tribut_interestadual_imp=trib_18_id)

    def test_create(self):
        self.fistrib_interestadual.unlink(self.cr, self.uid, self.fistrib_inter_id)
        vals = self.get_vals()
        self.assertIsInstance(self.fistrib_interestadual.create(self.cr, self.uid, vals),
                              (int, long))

    def test_create_duplicate(self):
        vals = self.get_vals()
        with self.assertRaises(psycopg2.IntegrityError):
            self.fistrib_interestadual.create(self.cr, self.uid, vals)

    def test_fistin_get_tributacao_interestadual(self):
        uf_id = self.uf.search(self.cr, self.uid, [('sigla', '=', 'SP')])[0]
        icms_id = self.encargo.search(self.cr, self.uid, [('sigla', '=', 'ICMS')])[0]
        self.assertIsInstance(
            self.fistrib_interestadual.fistin_get_tributacao_interestadual(
                self.cr, self.uid, uf_id, icms_id, produto_nacional=True), dict)

    def test_fistin_get_tributacao_interestadual_nacional(self):
        uf_id = self.uf.search(self.cr, self.uid, [('sigla', '=', 'SP')])[0]
        icms_id = self.encargo.search(self.cr, self.uid, [('sigla', '=', 'ICMS')])[0]
        trib_interestadual = self.fistrib_interestadual.fistin_get_tributacao_interestadual(
            self.cr, self.uid, uf_id, icms_id, produto_nacional=True)
        self.assertEqual(
            trib_interestadual.get('tributacao').aliquota, 7)

    def test_fistin_get_tributacao_interestadual_importado(self):
        uf_id = self.uf.search(self.cr, self.uid, [('sigla', '=', 'SP')])[0]
        icms_id = self.encargo.search(self.cr, self.uid, [('sigla', '=', 'ICMS')])[0]
        trib_interestadual = self.fistrib_interestadual.fistin_get_tributacao_interestadual(
            self.cr, self.uid, uf_id, icms_id, produto_nacional=False)
        self.assertEqual(
            trib_interestadual.get('tributacao').aliquota, 18)
