#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2016, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
import logging
__version__ = '1.0'
__date__ = '2016-07-11'
__author__ = 'pierre.barbosa'
__email__ = 'pierre.barbosa[at]siginformatica.com.br'


# noinspection PyUnresolvedReferences
class TestMovimentoItens(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestMovimentoItens, self).setUp()
        logging.getLogger('TestMovimentoItens').info('Inicio dos testes...')
        self._company_id = self.registry('adm.empresa.objeto').getEmpresa(self.cr, self.uid)

    def test_estmvi_get_cfop_id_1411(self):
        """
        Teste de ajuste de CFOP. O CFOP com natureza de operacao de devolucao 1202 deve ser
        ajustado para o 1411 para itens com substituicao tributaria.

        :return: n/a
        :rtype: n/a

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        natureza = self._criar_natureza_operacao_devolucao_1202()
        cfop_id = self.registry('estoque.movimento.itens').estmvi_get_cfop_id(
            self.cr, self.uid, self._criar_item_produto_st(), 'P', 'E', natureza,
            self._criar_pessoa_fisica(), dict(proprio=True, natureza_id=natureza))
        result = self.registry('fiscal.cfop').read(self.cr, self.uid, cfop_id, ['codigo'])
        self.assertEqual(result.get('codigo'), '1411')

    def test_estmvi_get_cfop_id_1202(self):
        """
        Teste de ajuste de CFOP. O CFOP com natureza de operacao de devolucao 1202 nao deve ser
        ajustado para o 1411 permanecendo 1202 para itens sem substituicao tributaria.

        :return: n/a
        :rtype: n/a

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        natureza = self._criar_natureza_operacao_devolucao_1202()
        cfop_id = self.registry('estoque.movimento.itens').estmvi_get_cfop_id(
            self.cr, self.uid, self._criar_item_produto(), 'P', 'E', natureza,
            self._criar_pessoa_fisica(), dict(proprio=True, natureza_id=natureza))
        result = self.registry('fiscal.cfop').read(self.cr, self.uid, cfop_id, ['codigo'])
        self.assertEqual(result.get('codigo'), '1202')

    def _criar_pessoa_fisica(self):
        """
        Cria uma pessoa fisica. Utiliza um CPF valido.

        :return: o ID da pessoa que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Se o CPF do cliente a ser criado jah existe, nao cria novo registro
        result = self.registry('adm.pessoa').search(
            self.cr, self.uid, [('cpf_cnpj', '=', '37371541120')], context=dict())
        if not result:
            result = self.registry('adm.pessoa').create(
                self.cr, self.uid, dict(pessoa_tipo='F', name='Pierre Geraldo Moura Barbosa',
                                        cpf_cnpj='37371541120', admpes_fis_sexo='M'))
        else:
            result = result[0]

        return result

    def _criar_natureza_operacao_devolucao_1202(self):
        """
        Cria uma natureza de operacao com CFOP de devolucao.

        :return: o ID da natureza da operacao que foi criada
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cfop_id = self._criar_CFOP_entrada_devolucao_1202()
        operacao_id = self._criar_operacao_entrada_devolucao()
        natureza_ids = self.registry('fiscal.cfop.natureza').search(
            self.cr, self.uid, [('cfop_id', '=', cfop_id), ('fisnat_operacao_id', '=', operacao_id),
                                ('tipo', '=', 'E')])
        if not natureza_ids:  # Cria a natureza de operacao se nao encontrou uma jah cadastrada
            pis_id = self._criar_encargo_PIS()
            cofins_id = self._criar_encargo_COFINS()
            icms_id = self._criar_encargo_ICMS()
            natureza_id = self.registry('fiscal.cfop.natureza').create(
                self.cr, self.uid, dict(cfop_id=cfop_id, tipo='E', fisnat_operacao_id=operacao_id,
                                        name='Devolucoes de vendas de mercadorias adiquiridas ou '
                                             'recebidas de terceiros',
                                        fisnat_natureza_fiscal='Devolucoes de mercadorias '
                                                               'recebidas de terceiros',
                                        natureza_encargos_ids=[[0, 0, dict(encargo_id=pis_id)],
                                                               [0, 0, dict(encargo_id=cofins_id)],
                                                               [0, 0, dict(encargo_id=icms_id)]]))
        else:
            natureza_id = natureza_ids[0]

        return natureza_id

    def _criar_encargo_PIS(self):
        """
        Cria o encargo PIS

        :param company_id: o ID da empresa
        :type company_id: long
        :return: o ID do encargo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        encargo_ids = self.registry('fiscal.encargo').search(
            self.cr, self.uid, [('sigla', '=', 'PIS')])
        if not encargo_ids:  # Cria o encargo se nao encontrou um cadastrado
            pis_id = self.registry('fiscal.encargo').create(
                self.cr, self.uid, dict(sigla='PIS', name='PIS', encargo_company_ids=[
                    [0, 0, dict(company_id=self._company_id)]]))
        else:
            pis_id = encargo_ids[0]

        return pis_id

    def _criar_encargo_COFINS(self):
        """
        Cria o encargo COFINS

        :return: o ID do encargo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        encargo_ids = self.registry('fiscal.encargo').search(
            self.cr, self.uid, [('sigla', '=', 'COFINS')])
        if not encargo_ids:  # Cria o encargo se nao encontrou um cadastrado
            cofins_id = self.registry('fiscal.encargo').create(
                self.cr, self.uid, dict(sigla='COFINS', name='COFINS', encargo_company_ids=[
                    [0, 0, dict(company_id=self._company_id)]]))
        else:
            cofins_id = encargo_ids[0]

        return cofins_id

    def _criar_encargo_ICMS(self):
        """
        Cria o encargo ICMS

        :return: o ID do encargo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        encargo_ids = self.registry('fiscal.encargo').search(
            self.cr, self.uid, [('sigla', '=', 'ICMS')])
        if not encargo_ids:  # Cria o encargo se nao encontrou um cadastrado
            icms_id = self.registry('fiscal.encargo').create(
                self.cr, self.uid, dict(sigla='ICMS', name='ICMS', encargo_company_ids=[
                    [0, 0, dict(company_id=self._company_id)]]))
        else:
            icms_id = encargo_ids[0]

        return icms_id

    def _criar_operacao_entrada_devolucao(self):
        """
        Cria a operacao de devolucao.

        :return: o ID da operacao que foi criada
        :rtype: long
        """
        operacao_ids = self.registry('estoque.natureza.operacao').search(
            self.cr, self.uid, [('name', '=', 'Devolucao'), ('estnto_tipo_operacao', '=', 'E')])
        if not operacao_ids:  # Cria a operacao se nao encontrar uma com a mesma descricao
            operacao_id = self.registry('estoque.natureza.operacao').create(
                self.cr, self.uid, dict(name='Devolucao', estnto_tipo_operacao='E'))
        else:
            operacao_id = operacao_ids[0]

        return operacao_id

    def _criar_CFOP_entrada_devolucao_1202(self):
        """
        Cria o CFOP de entrada de devolucao 1202 - Devolucoes de vendas de mercadorias adiquiridas
        ou recebidas de terceiros.

        :return: o ID do CFOP.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cfop_ids = self.registry('fiscal.cfop').search(self.cr, self.uid, [('codigo', '=', '1202')])
        if not cfop_ids:  # Cria o CFOP se nao encontrar um com o codigo 1202
            cfop_id = self.registry('fiscal.cfop').create(
                self.cr, self.uid, dict(codigo='1202', tipo='E', movimento_tipo='D',
                                        transacao_tipo='C', analitico_sintetico='A',
                                        descricao='Devolucoes de vendas de mercadorias adiquiridas '
                                                  'ou recebidas de terceiros',
                                        produto_servico_tipo='P'))
        else:
            cfop_id = cfop_ids[0]

        return cfop_id

    def _criar_item_produto(self):
        """
        Cria um item (produto) sem susbtituicao tributaria.

        :return: o ID do produto que foi criado
        :rtype:long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        descricao = 'Produto teste de uso interno (nao especificado)'
        item_ids = self.registry('estoque.produto').search(
            self.cr, self.uid, [('name', '=', descricao)])
        if not item_ids:  # Cria o produto se nao encontrar um com a mesma descricao
            pis_id = self._criar_encargo_PIS()
            cofins_id = self._criar_encargo_COFINS()
            icms_id = self._criar_encargo_ICMS()
            cst50_pis = self._criar_tributacao_cst50_PIS()
            cst50_cofins = self._criar_tributacao_cst50_COFINS()
            cst00_icms = self._criar_tributacao_cst00_ICMS()  # Tributado integralmente
            item_id = self.registry('estoque.produto').create(
                self.cr, self.uid, dict(name=descricao, ncm_id=self._criar_ncm(), tipo='P',
                                        tributacao_ids=[
                                            [0, 0, dict(cst_id=self._criar_CST50_PIS(),
                                                        encargo_id=pis_id,
                                                        tributacao_id=cst50_pis)],
                                            [0, 0, dict(cst_id=self._criar_CST50_COFINS(),
                                                        encargo_id=cofins_id,
                                                        tributacao_id=cst50_cofins)],
                                            [0, 0, dict(cst_id=self._criar_CST00_ICMS(),
                                                        encargo_id=icms_id,
                                                        tributacao_id=cst00_icms)]]))
        else:
            item_id = item_ids[0]

        return item_id

    def _criar_item_produto_st(self):
        """
        Cria um item (produto) com susbtituicao tributaria.

        :return: o ID do produto que foi criado
        :rtype:long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        descricao = 'Produto teste de uso interno (nao especificado)'
        item_ids = self.registry('estoque.produto').search(
            self.cr, self.uid, [('name', '=', descricao)])
        if not item_ids:  # Cria o produto se nao encontrar um com a mesma descricao
            pis_id = self._criar_encargo_PIS()
            cofins_id = self._criar_encargo_COFINS()
            icms_id = self._criar_encargo_ICMS()
            cst50_pis = self._criar_tributacao_cst50_PIS()
            cst50_cofins = self._criar_tributacao_cst50_COFINS()
            cst60_icms = self._criar_tributacao_cst60_ICMS()  # ICMS/ST
            item_id = self.registry('estoque.produto').create(
                self.cr, self.uid, dict(name=descricao, ncm_id=self._criar_ncm(), tipo='P',
                                        tributacao_ids=[
                                            [0, 0, dict(cst_id=self._criar_CST50_PIS(),
                                                        encargo_id=pis_id,
                                                        tributacao_id=cst50_pis)],
                                            [0, 0, dict(cst_id=self._criar_CST50_COFINS(),
                                                        encargo_id=cofins_id,
                                                        tributacao_id=cst50_cofins)],
                                            [0, 0, dict(cst_id=self._criar_CST60_ICMS(),
                                                        encargo_id=icms_id,
                                                        tributacao_id=cst60_icms)]]))
        else:
            item_id = item_ids[0]

        return item_id

    def _criar_CST00_ICMS(self):
        """
        Cria o CST 00-Tributado itegralmente.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        icms_id = self._criar_encargo_ICMS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('codigo', '=', '00'),
                                ('fiscst_tipo_movimento', 'in', ['E', 'A']), ('tipo', '=', 'T')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=icms_id, codigo='60', tipo='U',
                                        fiscst_tipo_movimento='E',
                                        name='Tributado integralmente'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_CST60_ICMS(self):
        """
        Cria o CST 60-ICMS cobrado anteriormente p/substit.tribut. o encargo ICMS.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        icms_id = self._criar_encargo_ICMS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('codigo', '=', '60'),
                                ('fiscst_tipo_movimento', 'in', ['E', 'A']), ('tipo', '=', 'U')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=icms_id, codigo='60', tipo='U',
                                        fiscst_tipo_movimento='E',
                                        name='ICMS cobrado anteriormente p/substit.tribut.'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_CST50_PIS(self):
        """
        Cria o CST 50-Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int para o encargo PIS.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pis_id = self._criar_encargo_PIS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', pis_id), ('codigo', '=', '50'),
                                ('fiscst_tipo_movimento', '=', 'E'), ('tipo', '=', 'T')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=pis_id, codigo='50', tipo='T',
                                        fiscst_tipo_movimento='E',
                                        name='Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_tributacao_cst00_ICMS(self):
        """
        Cria tributacao sem substituicao tributaria para o encargo ICMS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        icms_id = self._criar_encargo_ICMS()
        cst_id = self._criar_CST00_ICMS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(
                self.cr, self.uid, dict(encargo_id=icms_id, cst_id=cst_id))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_CST50_COFINS(self):
        """
        Cria o CST 50-Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int para o encargo COFINS.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cofins_id = self._criar_encargo_COFINS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', cofins_id), ('codigo', '=', '50'),
                                ('fiscst_tipo_movimento', '=', 'E'), ('tipo', '=', 'T')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=cofins_id, codigo='50', tipo='T',
                                        fiscst_tipo_movimento='E',
                                        name='Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_tributacao_cst60_ICMS(self):
        """
        Cria tributacao com substituicao tributaria para o encargo ICMS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        icms_id = self._criar_encargo_ICMS()
        cst_id = self._criar_CST60_ICMS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(
                self.cr, self.uid, dict(encargo_id=icms_id, cst_id=cst_id))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_tributacao_cst50_COFINS(self):
        """
        Cria tributacao para o encargo COFINS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cofins_id = self._criar_encargo_COFINS()
        cst_id = self._criar_CST50_COFINS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', cofins_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(
                self.cr, self.uid, dict(encargo_id=pis_id, cst_id=cst_id))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_tributacao_cst50_PIS(self):
        """
        Cria tributacao para o encargo PIS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pis_id = self._criar_encargo_PIS()
        cst_id = self._criar_CST50_PIS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', pis_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(self.cr, self.uid, dict(
                encargo_id=pis_id, cst_id=cst_id))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_ncm(self):
        """
        Cria NCM para um item (produto/servico).

        :return: o ID do NCM que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se o NCM do item existe, nao cria um novo
        ncm_ids = self.registry('fiscal.ncm').search(
            self.cr, self.uid, [('codigo', '=', '01010199')], context=dict())
        if not ncm_ids:
            ncm_id = self.registry('fiscal.ncm').create(
                self.cr, self.uid, dict(codigo='01010199', descricao='01010199'))
        else:
            ncm_id = ncm_ids[0]

        return ncm_id

    def tearDown(self):
        super(TestMovimentoItens, self).tearDown()
        logging.getLogger('TestMovimentoItens').info('Fim dos testes.')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
