#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.tools import mute_logger
from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from openerp.osv.orm import except_orm
from unittest2 import skip


class TestNcm(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestNcm, self).setUp()
        self.ncm = self.registry('fiscal.ncm')
        self.ncm_codigo = '94052000'
        self.token = 'HBjMa23juVp19PocGViAOtnTpKBRoMB3mC5aqEOAH9fY6dA5wLHy7vP9KjSzm_4F'
        self.token_invalido = '11111111111111111111111111111111'
        self.cnpj = '01526558000178'
        self.uf = 'MG'
        self.proxies = {'http': 'http://arizona:3128'}
        # Item abaixo utilizado quando a rede nao possui proxy
        # self.proxies = False
        self.ex = 0

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_token_invalido(self):
        res = self.ncm.get_tributo_ibpt(
            self.token_invalido, self.cnpj, self.ncm_codigo, self.uf, self.ex,
            proxies=self.proxies)
        self.assertEqual(res, {})

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_instance(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, proxies=self.proxies)
        self.assertIsInstance(res, dict, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_Estadual(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, proxies=self.proxies)
        self.assertIsInstance(res.get('Estadual'), float, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_Importado(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, proxies=self.proxies)
        self.assertIsInstance(res.get('Importado'), float, res.get('Importado'))

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_Nacional(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, proxies=self.proxies)
        self.assertIsInstance(res.get('Nacional'), float, res.get('Nacional'))

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_proxie_errado(self):
        proxies = {'http': 'http://teste:3128'}
        try:
            self.ncm.get_tributo_ibpt(
                self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, proxies=proxies)
        except:
            self.assertRaises(except_orm)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_com_excecao_ncm(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, '40131090', self.uf, ex=1, proxies=self.proxies)
        self.assertIsInstance(res, dict, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_produto_nao_encontrado(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, ex=2, proxies=self.proxies)
        self.assertEqual(res, {}, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_codigo_interno(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, '000001', proxies=self.proxies
        )
        self.assertIsInstance(res, dict, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_descricao(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, '0000001',
            descricao='PRODUTO DE TESTE', proxies=self.proxies
        )
        self.assertIsInstance(res, dict, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_unidade_medida(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, '0000001',
            descricao='PRODUTO DE TESTE', unidade_medida='KG', proxies=self.proxies
        )
        self.assertIsInstance(res, dict, res)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_valor(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, '0000001',
            descricao='PRODUTO DE TESTE', unidade_medida='KG', valor=10.5, proxies=self.proxies
        )
        self.assertIsInstance(res, dict)

    @mute_logger('openerp.osv.orm')
    def test_get_tributo_ibpt_gtin(self):
        res = self.ncm.get_tributo_ibpt(
            self.token, self.cnpj, self.ncm_codigo, self.uf, self.ex, '0000001',
            descricao='PRODUTO DE TESTE', unidade_medida='KG', valor=10.5, gtin='1',
            proxies=self.proxies
        )
        self.assertIsInstance(res, dict)

    @mute_logger('openerp.osv.orm')
    def test_atualiza_tributo_ibpt_ws_nacional(self):
        ncm_id = self.ncm.search(self.cr, self.uid, [('codigo', '=', '94052000')])[0]
        self.ncm.write(self.cr, self.uid, ncm_id, {'fisnc_aliq_ibpt_fed_nac': 0.0})
        res = self.ncm.atualiza_tributo_ibpt_ws(self.cr, self.uid, ncm_id, self.token, self.cnpj,
                                                self.uf, proxies=self.proxies)
        self.assertIsInstance(res, bool)
        ncm = self.ncm.browse(self.cr, self.uid, ncm_id)
        ncm_ibpt = self.ncm.get_tributo_ibpt(self.token, self.cnpj, self.ncm_codigo, self.uf,
                                             proxies=self.proxies)
        self.assertEqual(ncm_ibpt.get('Nacional'), ncm.fisnc_aliq_ibpt_fed_nac)

    @mute_logger('openerp.osv.orm')
    def test_atualiza_tributo_ibpt_ws_importado(self):
        ncm_id = self.ncm.search(self.cr, self.uid, [('codigo', '=', '94052000')])[0]
        self.ncm.write(self.cr, self.uid, ncm_id, {'fisnc_aliq_ibpt_fed_imp': 0.0})
        res = self.ncm.atualiza_tributo_ibpt_ws(self.cr, self.uid, ncm_id, self.token, self.cnpj,
                                                self.uf, proxies=self.proxies)
        self.assertIsInstance(res, bool)
        ncm = self.ncm.browse(self.cr, self.uid, ncm_id)
        ncm_ibpt = self.ncm.get_tributo_ibpt(self.token, self.cnpj, self.ncm_codigo, self.uf,
                                             proxies=self.proxies)
        self.assertEqual(ncm_ibpt.get('Importado'), ncm.fisnc_aliq_ibpt_fed_imp)

    @mute_logger('openerp.osv.orm')
    def test_atualiza_tributo_ibpt_ws_estadual(self):
        ncm_id = self.ncm.search(self.cr, self.uid, [('codigo', '=', '94052000')])[0]
        self.ncm.write(self.cr, self.uid, ncm_id, {'fisnc_aliq_ibpt_est': 0.0})
        res = self.ncm.atualiza_tributo_ibpt_ws(self.cr, self.uid, ncm_id, self.token, self.cnpj,
                                                self.uf, proxies=self.proxies)
        self.assertIsInstance(res, bool)
        ncm = self.ncm.browse(self.cr, self.uid, ncm_id)
        ncm_ibpt = self.ncm.get_tributo_ibpt(self.token, self.cnpj, self.ncm_codigo, self.uf,
                                             proxies=self.proxies)
        self.assertEqual(ncm_ibpt.get('Estadual'), ncm.fisnc_aliq_ibpt_est)

    @mute_logger('openerp.osv.orm')
    #@skip("Muita carga de dados")
    def test_atualiza_tributos_ibpt(self):
        parametro = self.registry('adm.utils').admutl_get_parametros(self.cr, self.uid, 'fiscal',
                                                                     ['id'])
        vals = {'fispar_ibpt_token': self.token, 'fispar_ibpt_cnpj': self.cnpj,
                'fispar_ibpt_uf': self.uf}
        self.registry('fiscal.parametro').write(self.cr, self.uid, parametro.get('id'), vals)
        vals = {'fisnc_aliq_ibpt_fed_imp': 0.0, 'fisnc_aliq_ibpt_fed_nac': 0.0,
                'fisnc_aliq_ibpt_est': 0.0}
        self.ncm.write(self.cr, self.uid, self.ncm.search(self.cr, self.uid, []), vals)
        res = self.ncm.atualiza_tributos_ibpt(self.cr, self.uid)
        self.assertIsInstance(res, bool)
        cont = 0
        for ncm_id in self.ncm.search(self.cr, self.uid, []):
            brw_ncm = self.ncm.browse(self.cr, self.uid, ncm_id)
            dados_ibpt = self.ncm.get_tributo_ibpt(
                self.token, self.cnpj, brw_ncm.codigo, self.uf, proxies=self.proxies)
            self.assertEqual(dados_ibpt.get('Estadual', 0.0), brw_ncm.fisnc_aliq_ibpt_est)
            self.assertEqual(dados_ibpt.get('Nacional', 0.0), brw_ncm.fisnc_aliq_ibpt_fed_nac)
            self.assertEqual(dados_ibpt.get('Importado', 0.0), brw_ncm.fisnc_aliq_ibpt_fed_imp)
            cont += 1
            if cont == 10:
                break

    @mute_logger('openerp.osv.orm')
    def test_atualiza_tributos_sem_token(self):
        fields = ['id', 'fispar_ibpt_token', 'fispar_ibpt_cnpj', 'fispar_ibpt_uf']
        parametro = self.registry('adm.utils').admutl_get_parametros(self.cr, self.uid, 'fiscal',
                                                                     fields)
        vals = {'fispar_ibpt_token': None, 'fispar_ibpt_cnpj': self.cnpj, 'fispar_ibpt_uf': self.uf}
        self.registry('fiscal.parametro').write(self.cr, self.uid, parametro.get('id'), vals)
        with self.assertRaises(except_orm):
            self.ncm.atualiza_tributos_ibpt(self.cr, self.uid)

    @mute_logger('openerp.osv.orm')
    def test_atualiza_tributos_sem_cnpj(self):
        fields = ['id', 'fispar_ibpt_token', 'fispar_ibpt_cnpj', 'fispar_ibpt_uf']
        parametro = self.registry('adm.utils').admutl_get_parametros(self.cr, self.uid, 'fiscal',
                                                                     fields)
        vals = {'fispar_ibpt_token': self.token, 'fispar_ibpt_cnpj': None,
                'fispar_ibpt_uf': self.uf}
        self.registry('fiscal.parametro').write(self.cr, self.uid, parametro.get('id'), vals)
        with self.assertRaises(except_orm):
            self.ncm.atualiza_tributos_ibpt(self.cr, self.uid)

    @mute_logger('openerp.osv.orm')
    def test_atualiza_tributos_sem_uf(self):
        fields = ['id', 'fispar_ibpt_token', 'fispar_ibpt_cnpj', 'fispar_ibpt_uf']
        parametro = self.registry('adm.utils').admutl_get_parametros(self.cr, self.uid, 'fiscal',
                                                                     fields)
        vals = {'fispar_ibpt_token': self.token, 'fispar_ibpt_cnpj': self.cnpj,
                'fispar_ibpt_uf': None}
        self.registry('fiscal.parametro').write(self.cr, self.uid, parametro.get('id'), vals)
        with self.assertRaises(except_orm):
            self.ncm.atualiza_tributos_ibpt(self.cr, self.uid)
