#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
from random import randint


# noinspection PyUnresolvedReferences
class TestFiscalNfItemTributacao(TransactionCase):
    """
    Classe de testes unitarios da class fiscal.tributacao.interestadual
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestFiscalNfItemTributacao, self).setUp()
        self.fisnit = self.registry('fiscal.nf.item.tributacao')
        self.uf = self.registry('adm.uf')
        self.encargo = self.registry('fiscal.encargo')
        self.fistrib_interestadual = self.registry('fiscal.tributacao.interestadual')
        self.fistrib = self.registry('fiscal.tributacao')
        self.cst = self.registry('fiscal.cst')

    def test_fisnit_calcula_icms_estado_origem_produto(self):
        uf_id = self.get_uf('SP')[0]
        encargo_id = self.get_encargo('ICMS')[0]
        vals_fistrib_inter = self.get_vals_trib_interestadual()
        self.fistrib_interestadual.create(self.cr, self.uid, vals_fistrib_inter)
        self.assertEqual(self.fisnit.fisnit_calcula_icms_estado_origem(
            self.cr, self.uid, uf_id, encargo_id, 0, True), 0)

    def test_fisnit_calcula_icms_estado_origem_produto_nacional(self):
        uf_id = self.get_uf('SP')[0]
        encargo_id = self.get_encargo('ICMS')[0]
        vals_fistrib_inter = self.get_vals_trib_interestadual()
        self.fistrib_interestadual.create(self.cr, self.uid, vals_fistrib_inter)
        self.assertEqual(self.fisnit.fisnit_calcula_icms_estado_origem(
            self.cr, self.uid, uf_id, encargo_id, 56.4, True), 3.948)

    def test_fisnit_calcula_icms_estado_origem_produto_importado(self):
        uf_id = self.get_uf('SP')[0]
        encargo_id = self.get_encargo('ICMS')[0]
        vals_fistrib_inter = self.get_vals_trib_interestadual()
        self.fistrib_interestadual.create(self.cr, self.uid, vals_fistrib_inter)
        self.assertEqual(self.fisnit.fisnit_calcula_icms_estado_origem(
            self.cr, self.uid, uf_id, encargo_id, 56.4, False), 10.152)

    def test_fisnit_calcula_tributos_interestaduais_vr_zero(self):
        uf_id = self.get_uf('SP')[0]
        encargo_id = self.get_encargo('ICMS')[0]
        vals_fistrib_inter = self.get_vals_trib_interestadual()
        self.fistrib_interestadual.create(self.cr, self.uid, vals_fistrib_inter)
        tributos = self.fisnit.fisnit_calcula_tributos_interestaduais(
            self.cr, self.uid, uf_id, encargo_id, 0.0, True
        )
        self.assertIsInstance(tributos, dict)
        self.assertEqual(tributos.get('vr_base_calculo'), 0.0)
        self.assertEqual(tributos.get('vr_encargo'), 0.0)
        self.assertEqual(tributos.get('aliquota'), 7.0)

    def test_fisnit_calcula_tributos_interestaduais_prod_nacional(self):
        uf_id = self.get_uf('SP')[0]
        encargo_id = self.get_encargo('ICMS')[0]
        vals_fistrib_inter = self.get_vals_trib_interestadual()
        self.fistrib_interestadual.create(self.cr, self.uid, vals_fistrib_inter)
        tributos = self.fisnit.fisnit_calcula_tributos_interestaduais(
            self.cr, self.uid, uf_id, encargo_id, 56.4, True
        )
        self.assertIsInstance(tributos, dict)
        self.assertEqual(tributos.get('vr_base_calculo'), 56.4)
        self.assertEqual(tributos.get('vr_encargo'), 3.948)
        self.assertEqual(tributos.get('aliquota'), 7.0)

    def test_fisnit_calcula_tributos_interestaduais_prod_importado(self):
        uf_id = self.get_uf('SP')[0]
        encargo_id = self.get_encargo('ICMS')[0]
        vals_fistrib_inter = self.get_vals_trib_interestadual()
        self.fistrib_interestadual.create(self.cr, self.uid, vals_fistrib_inter)
        tributos = self.fisnit.fisnit_calcula_tributos_interestaduais(
            self.cr, self.uid, uf_id, encargo_id, 56.4, False
        )
        self.assertIsInstance(tributos, dict)
        self.assertEqual(tributos.get('vr_base_calculo'), 56.4)
        self.assertEqual(tributos.get('vr_encargo'), 10.152)
        self.assertEqual(tributos.get('aliquota'), 18)

    def test_fisnit_calcula_vr_fcp(self):
        self.assertEqual(self.fisnit.fisnit_calcula_vr_fcp(145.68, 2.0), 2.91)
        self.assertEqual(self.fisnit.fisnit_calcula_vr_fcp(0, 2.0), 0)
        self.assertEqual(self.fisnit.fisnit_calcula_vr_fcp(145.68, 0), 0)
        self.assertEqual(self.fisnit.fisnit_calcula_vr_fcp(None, 2), 0)
        self.assertEqual(self.fisnit.fisnit_calcula_vr_fcp(145.68, None), 0)

    def test_fisnit_cfop_id(self):
        cfop_id = self.get_cfop('5102')
        vals_produto = dict(name="TESTE " + str(randint(1, 99999)))
        produto_id = self.registry('estoque.produto').create(self.cr, self.uid, vals_produto)
        grade_id = self.registry('estoque.produto.grade').search(
            self.cr, self.uid, [('produto_id', '=', produto_id)])[0]
        vals_produto_embalagem = dict(embalagem_id=self.get_embalagem('UN'), produto_id=produto_id)
        produto_embalagem_id = self.registry('estoque.produto.embalagem').create(
            self.cr, self.uid, vals_produto_embalagem)
        vals_item = dict(cfop_id=cfop_id, produto_id=produto_id, grade_id=grade_id,
                         produto_embalagem_id=produto_embalagem_id, quantidade=1)
        movimento_id = self.registry('estoque.movimento.itens').create(self.cr, self.uid, vals_item)
        encargo_id = self.get_encargo('ICMS')[0]
        cst_id = self.cst.search(self.cr, self.uid,
                                 [('tipo', '=', 'T'), ('encargo_id', '=', encargo_id),
                                  ('codigo', '=', '00')])[0]
        trib_7_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=encargo_id,
                                                                name='TESTE TRIB 7 %', base=100,
                                                                aliquota=7))
        vals_tributo = dict(movimento_itens_id=movimento_id, cst_id=cst_id, encargo_id=encargo_id,
                            tributacao_id=trib_7_id)
        tributo_id = self.fisnit.create(self.cr, self.uid, vals_tributo)
        brw_item = self.registry('estoque.movimento.itens').browse(self.cr, self.uid, movimento_id)
        brw_fisnit = self.fisnit.browse(self.cr, self.uid, tributo_id)
        self.assertEqual(brw_item.cfop_id.id, brw_fisnit.fisnit_cfop_id.id)
        cfop_id = self.get_cfop('1403')
        self.registry('estoque.movimento.itens').write(self.cr, self.uid, movimento_id,
                                                       {'cfop_id': cfop_id}, {})
        brw_item = self.registry('estoque.movimento.itens').browse(self.cr, self.uid, movimento_id)
        brw_fisnit = self.fisnit.browse(self.cr, self.uid, tributo_id)
        self.assertEqual(brw_item.cfop_id.id, brw_fisnit.fisnit_cfop_id.id)


    # Metodos para geracao de dados
    def get_uf(self, sigla):
        return self.uf.search(self.cr, self.uid, [('sigla', '=', sigla)])

    def get_encargo(self, sigla):
        return self.encargo.search(self.cr, self.uid, [('sigla', '=', sigla)])

    def get_cfop(self, cfop):
        return self.registry('fiscal.cfop').search(self.cr, self.uid, [('codigo', '=', cfop)])[0]

    def get_embalagem(self, sigla):
        return self.registry('estoque.embalagem').search(
            self.cr, self.uid, [('sigla', '=', sigla)])[0]

    def get_vals_trib_interestadual(self):
        uf_id = self.get_uf('SP')[0]
        icms_id = self.get_encargo('ICMS')[0]
        cst_id = self.cst.search(self.cr, self.uid,
                                 [('tipo', '=', 'T'), ('encargo_id', '=', icms_id),
                                  ('codigo', '=', '00')])[0]
        trib_7_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=icms_id,
                                                                name='TESTE TRIB 7 %', base=100,
                                                                aliquota=7))
        trib_12_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=icms_id,
                                                                 name='TESTE TRIB 12 %', base=100,
                                                                 aliquota=12))
        trib_18_id = self.fistrib.create(self.cr, self.uid, dict(cst_id=cst_id, encargo_id=icms_id,
                                                                 name='TESTE TRIB 18 %', base=100,
                                                                 aliquota=18))
        return dict(fistin_uf=uf_id, fistin_encargo=icms_id,
                    fistin_cod_tribut_interestadual_nac=trib_7_id,
                    fistin_cod_tribut_interno=trib_12_id,
                    fistin_cod_tribut_interestadual_imp=trib_18_id)
