#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------
                           
Copyleft (CC) 2016, Creative Commons
Author - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt-BR
================================================================================
"""
from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
import logging

__version__ = '1.0'
__date__ = '2016-07-28'
__author__ = 'pierre.barbosa'
__email__ = 'pierre.barbosa[at]siginformatica.com.br'


class TestValidarNotaFiscal(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestValidarNotaFiscal, self).setUp()
        logging.getLogger('TestValidarNotaFiscal').info('Inicio dos testes...')

        # Precondicoes para o teste
        self._company_id = self.registry('adm.empresa.objeto').getEmpresa(self.cr, self.uid)
        self.pessoa_id = self._criar_pessoa_fisica()
        self._criar_cliente()

    def test_validar_nota_fiscal_modelo_01(self):
        """
        Realiza o teste de verificacao/validacao de uma nota fiscal modelo 01.

        :return: n/a
        :rtype: n/a

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        nf_id = self._criar_nota_fiscal_modelo_01()
        self.assertTrue(self.registry('fiscal.nf.capa').fisnfc_validar_nota_fiscal(
            self.cr, self.uid, nf_id))

    def _criar_unidade(self):
        """
        Cria uma unidade para a embalagem.

        :return: o ID da unidade.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a embalagem existe, nao cria uma nova
        result = self.registry('estoque.embalagem.unidade').search(
            self.cr, self.uid, [('name', '=', 'UNIDADE')], context=dict())
        if not result:
            result = self.registry('estoque.embalagem.unidade').create(
                self.cr, self.uid, dict(name='UNIDADE', unidade='UN'))
        else:
            result = result[0]

        return result

    def _criar_embalagem(self):
        """
        Cria uma embalagem.

        :return: o ID da embalagem.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a embalagem existe, nao cria uma nova
        result = self.registry('estoque.embalagem').search(
            self.cr, self.uid, [('name', 'ilike', 'UNIDADE')], context=dict())
        if not result:
            unidade_id = self._criar_unidade()
            result = self.registry('estoque.embalagem').create(
                self.cr, self.uid, dict(name='UNIDADE', sigla='UN', unidade_id=unidade_id,
                                        quantidade_na_embalagem=1))
        else:
            result = result[0]

        return result

    def _criar_nota_fiscal_modelo_01(self):
        import time
        produto_id = self._criar_item_produto()
        grade_id = self.registry('estoque.produto.grade').search(
            self.cr, self.uid, [('produto_id', '=', produto_id)])[0]
        embalagem_id = self._criar_embalagem()
        produto_embalagem_ids = self.registry('estoque.produto.embalagem').search(
            self.cr, self.uid, [('produto_id','=', produto_id),
                                ('embalagem_id', '=', embalagem_id)])
        if not produto_embalagem_ids:
            produto_embalagem_id = self.registry('estoque.produto.embalagem').create(
                self.cr, self.uid, dict(produto_id=produto_id, embalagem_id=embalagem_id))
        else:
            produto_embalagem_id = produto_embalagem_ids[0]
        cfop_5405 = self._criar_CFOP_saida_venda_5405()

        # Cria a condicao de pagamento e a parcela
        forma_pgto_id = self._criar_forma_pagamento_dinheiro()
        condicao_id = self._criar_condicao_pagamento_dinheiro()
        parcela = [[0, 0, dict(vencimento=time.strftime('%Y-%m-%d'), valor=100.)]]
        condicao_pagamento = [
            [0, 0, dict(forma_pagto_id=forma_pgto_id, condicao_forma_id=condicao_id, valor=100.,
                        nf_parcela_ids=parcela)]]

        return self.registry('fiscal.nf.capa').create(
            self.cr, self.uid, dict(modelo_nf_series_id=self._criar_serie_numero_modelo_01(),
                                    tipo='S',
                                    natureza_id=self._criar_natureza_operacao_venda_5102(),
                                    destinatario_id=self.pessoa_id,
                                    data_emissao=time.strftime('%Y-%m-%d %H:%M:%S'),
                                    estoque_movimento_itens_ids=[
                                        [0, 0, dict(local_id=self._criar_local(),
                                                    produto_id=produto_id, grade_id=grade_id,
                                                    quantidade=10, vr_unitario=10.,
                                                    vr_total_item=100., cfop_id=cfop_5405,
                                                    produto_embalagem_id=produto_embalagem_id)]],
                                    nf_condicao_ids=condicao_pagamento, nf_propria=True,
                                    vr_total_nf=100.))

    def _criar_forma_pagamento_dinheiro(self):
        """
        Cria uma forma de pagamento (dinheiro).

        :return: o ID da forma de pagamento que foi criada
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a forma de pagamento existe, nao cria uma nova
        forma_pgto_ids = self.registry('adm.forma.pgto').search(
            self.cr, self.uid, [('name', '=', 'Dinheiro'), ('sigla', '=', 'DN')])
        if not forma_pgto_ids:
            forma_pgto_id = self.registry('adm.forma.pgto').create(
                self.cr, self.uid, dict(name='Dinheiro', sigla='DN'))
        else:
            forma_pgto_id = forma_pgto_ids[0]

        return forma_pgto_id

    def _criar_condicao_pagamento_dinheiro(self):
        """
        Cria a condicao de pagamento dinheiro ("a vista").

        :return: o ID da condicao de pagamento.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a condicao existir, nao cria
        forma_pgto_id = self._criar_forma_pagamento_dinheiro()
        condicao_id = self._criar_condicao_avista()
        condicao_forma_ids = self.registry('adm.condicao.forma').search(
            self.cr, self.uid, [('forma_pagamento_id', '=', forma_pgto_id),
                                ('adm_condicao_id', '=', condicao_id)])
        if not condicao_forma_ids:
            self.registry('adm.condicao.forma').create(
                self.cr, self.uid, dict(forma_pagamento_id=forma_pgto_id))
        else:
            condicao_forma_id = condicao_forma_ids[0]

        return condicao_forma_id

    def _criar_condicao_avista(self):
        """
        Cria condicao "a vista".

        :return: o ID da condicao que foi criada
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a condicao existir, nao cria
        condicao_ids = self.registry('adm.condicao').search(
            self.cr, self.uid, [('name', '=', 'A vista'), ('tipo', '=', 'V')])
        if not condicao_ids:
            condicao_id = self.registry('adm.condicao').create(
                self.cr, self.uid, dict(name='A vista', tipo='V'))
        else:
            condicao_id = condicao_ids[0]

        return condicao_id

    def _criar_local(self):
        """
        Cria um local para o item (produto/servico).

        :return: o ID do local que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se o local existe, nao cria um novo
        local_ids = self.registry('estoque.local').search(
            self.cr, self.uid, [('numero', '=', 1)], context=dict())
        if not local_ids:
            centro_custo_id = self._criar_centro_custo()
            local_id = self.registry('estoque.local').create(
                self.cr, self.uid, dict(name='LOJA', numero=1, centrocusto_id=centro_custo_id),
                dict())
        else:
            local_id = local_ids[0]

        return local_id

    def _criar_ncm(self):
        """
        Cria NCM para um item (produto/servico).

        :return: o ID do NCM que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se o NCM do item existe, nao cria um novo
        ncm_ids = self.registry('fiscal.ncm').search(
            self.cr, self.uid, [('codigo', '=', '01010199')], context=dict())
        if not ncm_ids:
            ncm_id = self.registry('fiscal.ncm').create(
                self.cr, self.uid, dict(codigo='01010199', descricao='01010199'))
        else:
            ncm_id = ncm_ids[0]

        return ncm_id

    def _criar_item_produto(self):
        """
        Cria um item (produto) sem susbtituicao tributaria.

        :return: o ID do produto que foi criado
        :rtype:long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        descricao = 'Produto teste de uso interno (nao especificado)'
        item_ids = self.registry('estoque.produto').search(
            self.cr, self.uid, [('name', '=', descricao)])
        if not item_ids:  # Cria o produto se nao encontrar um com a mesma descricao
            pis_id = self._criar_encargo_PIS()
            cofins_id = self._criar_encargo_COFINS()
            icms_id = self._criar_encargo_ICMS()
            cst50_pis = self._criar_tributacao_cst50_PIS()
            cst50_cofins = self._criar_tributacao_cst50_COFINS()
            cst00_icms = self._criar_tributacao_cst00_ICMS()  # Tributado integralmente
            item_id = self.registry('estoque.produto').create(
                self.cr, self.uid, dict(name=descricao, ncm_id=self._criar_ncm(), tipo='P',
                                        tributacao_ids=[
                                            [0, 0, dict(cst_id=self._criar_CST50_PIS(),
                                                        encargo_id=pis_id,
                                                        tributacao_id=cst50_pis)],
                                            [0, 0, dict(cst_id=self._criar_CST50_COFINS(),
                                                        encargo_id=cofins_id,
                                                        tributacao_id=cst50_cofins)],
                                            [0, 0, dict(cst_id=self._criar_CST00_ICMS(),
                                                        encargo_id=icms_id,
                                                        tributacao_id=cst00_icms)]]))
        else:
            item_id = item_ids[0]

        return item_id

    def _criar_CST50_COFINS(self):
        """
        Cria o CST 50-Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int para o encargo COFINS.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cofins_id = self._criar_encargo_COFINS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', cofins_id), ('codigo', '=', '50'),
                                ('fiscst_tipo_movimento', '=', 'E'), ('tipo', '=', 'T')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=cofins_id, codigo='50', tipo='T',
                                        fiscst_tipo_movimento='E',
                                        name='Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_CST00_ICMS(self):
        """
        Cria o CST 00-Tributado itegralmente.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        icms_id = self._criar_encargo_ICMS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('codigo', '=', '00'),
                                ('fiscst_tipo_movimento', 'in', ['E', 'A']), ('tipo', '=', 'T')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=icms_id, codigo='60', tipo='U',
                                        fiscst_tipo_movimento='E',
                                        name='Tributado integralmente'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_CST50_PIS(self):
        """
        Cria o CST 50-Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int para o encargo PIS.

        :return: o ID do CST que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pis_id = self._criar_encargo_PIS()
        cst_ids = self.registry('fiscal.cst').search(
            self.cr, self.uid, [('encargo_id', '=', pis_id), ('codigo', '=', '50'),
                                ('fiscst_tipo_movimento', '=', 'E'), ('tipo', '=', 'T')])
        if not cst_ids:  # Cria o CST se nao existir
            cst_id = self.registry('fiscal.cst').create(
                self.cr, self.uid, dict(encargo_id=pis_id, codigo='50', tipo='T',
                                        fiscst_tipo_movimento='E',
                                        name='Oper.c/Dir.Cred.Vinc.Exclus.Rec.Trib.Merc.Int'))
        else:
            cst_id = cst_ids[0]

        return cst_id

    def _criar_tributacao_cst00_ICMS(self):
        """
        Cria tributacao sem substituicao tributaria para o encargo ICMS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        icms_id = self._criar_encargo_ICMS()
        cst_id = self._criar_CST00_ICMS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', icms_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(
                self.cr, self.uid, dict(encargo_id=icms_id, cst_id=cst_id, aliquota=7.0))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_tributacao_cst50_COFINS(self):
        """
        Cria tributacao para o encargo COFINS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cofins_id = self._criar_encargo_COFINS()
        cst_id = self._criar_CST50_COFINS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', cofins_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(
                self.cr, self.uid, dict(encargo_id=cofins_id, cst_id=cst_id, aliquota=7.6))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_tributacao_cst50_PIS(self):
        """
        Cria tributacao para o encargo PIS.

        :return: o ID da tributacao que foi criada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pis_id = self._criar_encargo_PIS()
        cst_id = self._criar_CST50_PIS()
        tributacao_ids = self.registry('fiscal.tributacao').search(
            self.cr, self.uid, [('encargo_id', '=', pis_id), ('cst_id', '=', cst_id)])
        if not tributacao_ids:  # Cria a tributacao se nao existe
            tributacao_id = self.registry('fiscal.tributacao').create(self.cr, self.uid, dict(
                encargo_id=pis_id, cst_id=cst_id, aliquota=1.65))
        else:
            tributacao_id = tributacao_ids[0]

        return tributacao_id

    def _criar_pessoa_fisica(self):
        """
        Cria uma pessoa fisica. Utiliza um CPF valido.

        :return: o ID da pessoa que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Se o CPF do cliente a ser criado jah existe, nao cria novo registro
        pessoa_ids = self.registry('adm.pessoa').search(
            self.cr, self.uid, [('cpf_cnpj', '=', '37371541120')], context=dict())
        if not pessoa_ids:
            pessoa_id = self.registry('adm.pessoa').create(
                self.cr, self.uid, dict(pessoa_tipo='F', name='Pierre Geraldo Moura Barbosa',
                                        cpf_cnpj='37371541120', admpes_fis_sexo='M'))
        else:
            pessoa_id = pessoa_ids[0]

        return pessoa_id

    def _criar_cliente(self):
        """
        Cria um cliente.

        :return: o ID do cliente, tabela financeiro_cliente
        :rtype: long

        :author: Tiago Henrique Prates dos Santos <tiago.santos[at]siginformatica.com.br>
        """
        cliente_ids = self.registry('financeiro.cliente').search(
            self.cr, self.uid, [('pessoa_id', '=', self.pessoa_id)])
        if cliente_ids:
            return cliente_ids[0]
        return self.registry('financeiro.cliente').create(
            self.cr, self.uid, {'pessoa_id': self.pessoa_id, 'limite_credito': 100.0})

    def _criar_natureza_operacao_venda_5102(self):
        """
        Cria uma natureza de operacao de venda no estado.

        :return: o ID da natureza da operacao que foi criada
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cfop_id = self._criar_CFOP_saida_venda_5102()
        operacao_id = self._criar_operacao_vendas()
        natureza_ids = self.registry('fiscal.cfop.natureza').search(
            self.cr, self.uid, [('cfop_id', '=', cfop_id), ('fisnat_operacao_id', '=', operacao_id),
                                ('tipo', '=', 'S')])
        if not natureza_ids:  # Cria a natureza de operacao se nao encontrou uma jah cadastrada
            pis_id = self._criar_encargo_PIS()
            cofins_id = self._criar_encargo_COFINS()
            icms_id = self._criar_encargo_ICMS()
            natureza_id = self.registry('fiscal.cfop.natureza').create(
                self.cr, self.uid, dict(cfop_id=cfop_id, tipo='S', fisnat_operacao_id=operacao_id,
                                        name='Vendas no estado',
                                        fisnat_natureza_fiscal='Vendas no estado',
                                        natureza_encargos_ids=[[0, 0, dict(encargo_id=pis_id)],
                                                               [0, 0, dict(encargo_id=cofins_id)],
                                                               [0, 0, dict(encargo_id=icms_id)]]))
        else:
            natureza_id = natureza_ids[0]

        return natureza_id

    def _criar_encargo_PIS(self):
        """
        Cria o encargo PIS

        :return: o ID do encargo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        encargo_ids = self.registry('fiscal.encargo').search(
            self.cr, self.uid, [('sigla', '=', 'PIS')])
        if not encargo_ids:  # Cria o encargo se nao encontrou um cadastrado
            pis_id = self.registry('fiscal.encargo').create(
                self.cr, self.uid, dict(sigla='PIS', name='PIS', encargo_company_ids=[
                    [0, 0, dict(company_id=self._company_id)]]))
        else:
            pis_id = encargo_ids[0]

        return pis_id

    def _criar_encargo_COFINS(self):
        """
        Cria o encargo COFINS

        :return: o ID do encargo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        encargo_ids = self.registry('fiscal.encargo').search(
            self.cr, self.uid, [('sigla', '=', 'COFINS')])
        if not encargo_ids:  # Cria o encargo se nao encontrou um cadastrado
            cofins_id = self.registry('fiscal.encargo').create(
                self.cr, self.uid, dict(sigla='COFINS', name='COFINS', encargo_company_ids=[
                    [0, 0, dict(company_id=self._company_id)]]))
        else:
            cofins_id = encargo_ids[0]

        return cofins_id

    def _criar_encargo_ICMS(self):
        """
        Cria o encargo ICMS

        :return: o ID do encargo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        encargo_ids = self.registry('fiscal.encargo').search(
            self.cr, self.uid, [('sigla', '=', 'ICMS')])
        if not encargo_ids:  # Cria o encargo se nao encontrou um cadastrado
            icms_id = self.registry('fiscal.encargo').create(
                self.cr, self.uid, dict(sigla='ICMS', name='ICMS', encargo_company_ids=[
                    [0, 0, dict(company_id=self._company_id)]]))
        else:
            icms_id = encargo_ids[0]

        return icms_id

    def _criar_operacao_vendas(self):
        """
        Cria a operacao de venda.

        :return: o ID da operacao que foi criada
        :rtype: long
        """
        operacao_ids = self.registry('estoque.natureza.operacao').search(
            self.cr, self.uid, [('name', '=', 'Vendas'), ('estnto_tipo_operacao', '=', 'S')])
        if not operacao_ids:  # Cria a operacao se nao encontrar uma com a mesma descricao
            operacao_id = self.registry('estoque.natureza.operacao').create(
                self.cr, self.uid, dict(name='Vendas', estnto_tipo_operacao='S'))
        else:
            operacao_id = operacao_ids[0]

        return operacao_id

    def _criar_CFOP_saida_venda_5102(self):
        """
        Cria o CFOP de saida de venda de mercadoria.

        :return: o ID do CFOP.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cfop_ids = self.registry('fiscal.cfop').search(self.cr, self.uid, [('codigo', '=', '5102')])
        if not cfop_ids:  # Cria o CFOP se nao encontrar um com o codigo 5102
            cfop_id = self.registry('fiscal.cfop').create(
                self.cr, self.uid, dict(codigo='5102', tipo='S', movimento_tipo='C',
                                        analitico_sintetico='A', produto_servico_tipo='P',
                                        descricao='Vendas de mercadorias adquiridas ou recebidas '
                                                  'de terceiros', transacao_tipo='C'))
        else:
            cfop_id = cfop_ids[0]

        return cfop_id

    def _criar_CFOP_saida_venda_5405(self):
        """
        Cria o CFOP de saida de venda de mercadoria.

        :return: o ID do CFOP.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        cfop_ids = self.registry('fiscal.cfop').search(self.cr, self.uid, [('codigo', '=', '5405')])
        if not cfop_ids:  # Cria o CFOP se nao encontrar um com o codigo 5405
            cfop_id = self.registry('fiscal.cfop').create(
                self.cr, self.uid, dict(codigo='5405', tipo='S', movimento_tipo='C',
                                        analitico_sintetico='A', produto_servico_tipo='P',
                                        descricao='Vendas de mercadorias adquiridas ou recebida de '
                                                  'terceiros em operacao com mercadoria sujeita ao '
                                                  'regime de substituicao tributaria',
                                        transacao_tipo='C'))
        else:
            cfop_id = cfop_ids[0]

        return cfop_id

    def _criar_serie_numero_modelo_01(self):
        """
        Cria a serie '001' e sequencia de numeracao numeracao da nota fiscal baseada no modelo 01.

        :return: id do serie que foi criada para o modelo.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        modelo_nf_id = self._criar_modelo_01()
        serie_numero_ids = self.registry('fiscal.modelo.nf.series').search(
            self.cr, self.uid, [('modelo_nf_id', '=', modelo_nf_id), ('serie_nf', '=', '001')])
        if not serie_numero_ids:
            serie_numero_id = self.registry('fiscal.modelo.nf.series').create(
                # Os atributos relativos a versao do XML e o ambiente foram informados por serem
                # atributos obrigatorios da serie da nota mesmo nao sendo um modelo de NF-e :(
                self.cr, self.uid, dict(modelo_nf_id=modelo_nf_id, serie_nf='001', numero_nf=1,
                                        vesao_xml='3.10', ambiente=2))
        else:
            serie_numero_id = serie_numero_ids[0]

        return serie_numero_id

    def _criar_modelo_01(self):
        """
        Cria um modelo de nota fiscal para ambiente homologacao (testes).

        :return: o ID do modelo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        modelo_ids = self.registry('fiscal.modelo.nf').search(
            self.cr, self.uid, [('fismnf_codigo', '=', '01')])
        if not modelo_ids:
            modelo_id = self.registry('fiscal.modelo.nf').create(
                self.cr, self.uid, dict(fismnf_codigo='01', name='Nota fiscal'))
        else:
            modelo_id = modelo_ids[0]

        return modelo_id

    def tearDown(self):
        super(TestValidarNotaFiscal, self).tearDown()
        logging.getLogger('TestValidarNotaFiscal').info('Fim dos testes.')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
