# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.tools.translate import _
from openerp.osv import osv, fields
import base64


class fiscal_download_xml_nfe(osv.TransientModel):
    """
    Wizard.
    Baixa arquivo XML de nota fiscal eletronica pela chave de acesso
    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    _name = 'fiscal.download.xml.nfe'  # wizfdx
    _columns = {
        'wizfdx_chave_acesso': fields.char('Chave de acesso', size=44),
        'name': fields.function(lambda s, c, u, i, f, a, x: dict().fromkeys(
            i, s.browse(c, u, i[0], x).wizfdx_chave_acesso + '.xml'), type='char'),
        'wizfdx_arquivo_xml': fields.binary('Arquivo XML'),
    }

    def wizfdx_button_download_xml(self, cr, user, ids, context=None):
        """
        Baixar o arquivo XML da nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: IDs do formulario de consulta
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a baixa do arquivo xml foi efetuada com sucesso
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        brw_form = self.browse(cr, user, ids[0], context)
        if not brw_form.wizfdx_chave_acesso or len(brw_form.wizfdx_chave_acesso) != 44:
            raise osv.except_osv(_('Atencao!'),
                                 _('Chave de acesso da NF-e deve ter 44 caracteres.'))
        arquivo_xml = base64.b64encode(self.pool.get('fiscal.nfe.consumo').read(
            cr, user, self.wizfdx_download_xml(cr, user, brw_form.wizfdx_chave_acesso, context),
            ['proc_resposta'], context).get('proc_resposta') or str())
        if not arquivo_xml:
            raise osv.except_osv(_('Atencao!'),
                                 _('NF-e sem manifestacao do destinatario.'))

        return self.write(cr, user, ids, dict(wizfdx_arquivo_xml=arquivo_xml))

    def wizfdx_download_xml(self, cr, user, key, context=None):
        """
        Baixa o arquivo XML da nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param key: a sequencia numerica que corresponde a chave de acesso da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type key: str
        :type context: dic
        :return: o ID do arquivo da nota fiscal eletronica (manifestacao destinatario).
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        download_ids = self.pool.get('fiscal.nfe.consumo').search(
            cr, user, [('chave_nfe', '=', key)], context=context)
        if not download_ids:
            download_ids = [self.pool.get('fiscal.nfe.consumo').create(
                cr, user, dict(chave_nfe=key))]
        self.pool.get('fiscal.nfe.consumo').download_xml(cr, user, download_ids, context)

        return download_ids[0]

    def wizfdx_button_importar_nf(self, cr, user, ids, context=None):
        """
        Apresenta o formulario de importacao de nota fiscal eletronica.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param key: a sequencia numerica que corresponde a chave de acesso da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type key: str
        :type context: dic
        :return: o formulario para importacao da nota fiscal eletronica
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        brw_form = self.browse(cr, user, ids[0], context)

        context = context and context.copy() or {}
        res_id = context.get('active_id')
        values = dict(xml_file=brw_form.wizfdx_arquivo_xml)
        if res_id:
            self.pool.get('fiscal.import.nfe.osv').write(cr, user, res_id,  values, context)
        else:
            res_id = self.pool.get('fiscal.import.nfe.osv').create(cr, user, values, context)
        view = self.pool.get('ir.model.data').get_object_reference(
            cr, user, 'fiscal', 'fiscal_import_nfe_osv_view')

        return dict(view_type='form', view_mode='form', view_id=[view and view[1] or False],
                    res_model='fiscal.import.nfe.osv', type='ir.actions.act_window',
                    res_id=res_id, name=_('Importar NF-e'), target='current', context=context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
