# -*- coding: utf-8 -*-

__author__ = 'marcilene.ribeiro'

from openerp.osv import osv, fields
from openerp.tools.translate import _


# noinspection PyAbstractClass,PyPep8Naming
class fiscal_exibe_questao_osv(osv.TransientModel):
    _name = "fiscal.exibe.questao.osv"
    _columns = {
        'name': fields.text('Informacao'),
        'opcao_confirma': fields.boolean('Opcao Confirma'),
    }

    def button_calcular(self, cr, uid, ids, context=None):

        """
        Atribui o valor calculado para as tributações dos itens e o valor do ICMS ST

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type context: dict
        :param context:
        :return:
        """

        context = context and context.copy() or {}

        if context.get('verificar_nf'):
            nf_id = context.get('registro_id') if context.get('import_nfe', False) else \
                context.get('registro_id') or context.get('active_id') or False

            if nf_id:
                obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
                dados_fiscal_nf_capa = obj_fiscal_nf_capa.read(
                    cr, uid, nf_id, ['estoque_movimento_itens_ids', 'vr_icms_substituicao'], context=context)
                item_ids = dados_fiscal_nf_capa['estoque_movimento_itens_ids']
                vr_icms_substituicao = dados_fiscal_nf_capa['vr_icms_substituicao']
                verifica_substituicao = obj_fiscal_nf_capa.verifica_icms_substituicao(
                    cr, uid, item_ids, vr_icms_substituicao, context=context)

                if verifica_substituicao.get('contem_icms_substituicao'):
                    obj_fiscal_nf_capa.write(
                        cr, uid, nf_id, {
                            'vr_icms_substituicao': verifica_substituicao.get('total_icms_st', 0),
                            'vr_base_icms_substituicao': verifica_substituicao.get('total_base_icms_st', 0)
                        }, context=context)
                    if verifica_substituicao.get('itens_tributacao'):
                        obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')

                        for vals in verifica_substituicao['itens_tributacao']:
                            obj_fiscal_nf_item_tributacao.write(cr, uid, vals[0], vals[1], context=context)

                    context['recalcular_parcelas'] = True
                    cr.commit()
        return {}

    def button_ok(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        # Todo: Tentar melhorar o codigo, dividindo responsabilidades para cada funcao e tentar
        # Todo: diminuir a utilizacao dos contextos.
        if context.get('active_id', False) or context.get('registro_id', False):
            # Se o usuario entrar na nota atraves de outra tela, como a de cupom por exemplo,
            # nao utilizar o active_id pois ele sera da primeira tela e nao o da nota fiscal
            nf_id = context.get('registro_id') or context.get('active_id')

            if context.get('verificar_nf') and not context.get('cancelamento_terceiros') and \
                not context.get('cancelamento_nf'):
                obj_nf_capa = self.pool.get('fiscal.nf.capa')
                nf_dados = obj_nf_capa.read(
                    cr, uid, nf_id, ['estoque_movimento_itens_ids', 'vr_base_icms_substituicao',
                                     'vr_icms_substituicao', 'company_id'], context=context)

                if isinstance(nf_dados, list):
                    nf_dados = nf_dados[0]

                if isinstance(nf_dados, dict):
                    movimento_ids = nf_dados['estoque_movimento_itens_ids']
                    context['rateio_icms_substituicao'] = True
                    calcula_rateio_substituicao = obj_nf_capa.calcula_rateio_substituicao(
                        cr, uid, nf_dados['vr_base_icms_substituicao'],
                        nf_dados['vr_icms_substituicao'], movimento_ids, context=context)

                    if calcula_rateio_substituicao:
                        context['rateio_substituicao_conferido'] = True
                        obj_nf_capa.button_verificar_nf(cr, uid, nf_id, context=context)
                        mod_obj = self.pool.get('ir.model.data')
                        result = mod_obj.get_object_reference(
                            cr, uid, 'fiscal', 'view_fiscal_nf_capa_terceiro_form')
                        view_id = result and result[1] or False
                        del context['rateio_substituicao_conferido']

                        return {
                            'name': _('Nota Fiscal Entrada de Terceiros'),
                            'view_type': 'form',
                            'view_mode': 'form',
                            'view_id': [view_id],
                            'res_model': 'fiscal.nf.capa',
                            'res_id': nf_id,
                            'type': 'ir.actions.act_window',
                            'context': context,
                        }
                    else:
                        msg = _("Nao foi possivel realizar o rateio do ICMS ST. Verifique se as "
                                "tributacoes dos itens possuem tributacao com substituicao para os "
                                "'CST's 10, 30 e 70")
                        self.write(cr, uid, ids, {'name': msg, 'opcao_confirma': False}, context)

                        return {
                            'name': _('Informacao'),
                            'view_type': 'form',
                            'view_mode': 'form',
                            'res_id': ids[0],
                            'res_model': 'fiscal.exibe.questao.osv',
                            'view_id': False,
                            'context': context,
                            'target': 'new',
                            'type': 'ir.actions.act_window',
                        }

            if context.get('cancelamento_nf') or context.get('cancelamento_terceiros'):
                # context['cancelamento_confirmado'] = True
                obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
                brw_fiscal_nf_capa = obj_fiscal_nf_capa.browse(cr, uid, nf_id, context=context)
                state = brw_fiscal_nf_capa.state
                if context.get('terceiro'):
                    nfe = brw_fiscal_nf_capa.modelo_nf_id.fismnf_tipo_nfe
                else:
                    nfe = brw_fiscal_nf_capa.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe
                if not nfe or (state in ('1', '2', '2E') and nfe) \
                        or (nfe and context.get('terceiro')):
                    if brw_fiscal_nf_capa.state not in ('1', 'C', 'I') and nfe \
                            and not context.get('terceiro'):
                        # No cancelamento da nota fiscal eletronica, fazer a verificacao na SEFAZ
                        # para certificar-se de que a nota nao esta faturada. Se estiver, a nota
                        # fiscal devera ser faturada para entao ser cancelada.
                        result, filler = self.pool.get('fiscal.nf.capa').verifica_status_nfe_sefaz(
                            cr, uid, nf_id, context)
                        if result.resposta.cStat.valor == u'100':  # Autorizado o uso da NF-e
                            raise osv.except_osv(_('Atencao!'),
                                                 _('Nota ja autorizada na SEFAZ. A nota deve ser '
                                                   'faturada primeiro antes de ser cancelada.'))

                    self.pool.get('fiscal.nf.capa').button_cancelar(cr, uid, [nf_id], context)
                elif context.get('proprio'):
                    return dict(name=_('Cancelar NF-e'), view_type='form', view_mode='form',
                                res_model='wizard.cancelar.nfe', view_id=False, context=context,
                                target='new', type='ir.actions.act_window')

        return {}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
