# coding=utf-8


from openerp.osv import osv, fields
from openerp.osv.osv import except_osv
from pythonsped.sintegra import SintegraFile, SintegraRegister10, SintegraRegister11
from pythonsped.sintegra import SintegraRegister50, SintegraRegister53, SintegraRegister54
from pythonsped.sintegra import SintegraRegister55
from pythonsped.sintegra import SintegraRegister60A, SintegraRegister60D, SintegraRegister60I
from pythonsped.sintegra import SintegraRegister61, SintegraRegister70, SintegraRegister88EAN
from pythonsped.sintegra import SintegraRegister61R
from datetime import datetime
import re
import base64
import logging
# noinspection PyProtectedMember
from openerp.tools.translate import _
import time
from decimal import Decimal, ROUND_UP
from dateutil.parser import parse
from openerp.osv.fields import float_round
from re import sub as re_sub

global TOTAL_ENTRADAS
global TOTAL_SAIDAS
global TOTAL_CTES

READ_LOAD = '_classic_write'
_tbl_cod_ident_con = [
    # 9.1.1 - Tabela para preenchimento do campo 10:
    # Tabela de Codigo de identificacao da estrutura do arquivo magnetico entregue
    (1, 'Estrutura conforme Convenio ICMS 57/95, na versao estabelecida pelo Convenio ICMS 31/99 e '
        'com as alteracoes promovidas ate o Convenio ICMS 30/02.'),
    (2, 'Estrutura conforme Convenio ICMS 57/95, na versao estabelecida pelo Convenio ICMS 69/02 e '
        'com as alteracoes promovidas pelo Convenio ICMS 142/02.'),
    (3, 'Estrutura conforme Convenio ICMS 57/95, com as alteracoes promovidas pelo Convenio ICMS '
        '76/03.')]
_tbl_cod_ident_nat = [
    # 9.1.2 - Tabela para preenchimento do campo 11
    # Tabela para Codigo da identificacao da natureza das operacoes informadas
    (1, 'Interestaduais somente operacoes sujeitas ao regime de Substituicao Tributaria'),
    (2, 'Interestaduais - operacoes com ou sem Substituicao Tributaria'),
    (3, 'Totalidade das operacoes do informante')]
_tbl_cod_ident_fin = [
    # 9.1.3 - Tabela para preenchimento do campo 12:
    # Tabela de Finalidades da apresentacao do arquivo magnetico
    (1, 'Normal'),
    (2, 'Retificacao total de arquivo: substituicao total de informacoes prestadas pelo '
        'contribuinte referentes a este periodo'),
    (3, 'Retificacao aditiva de arquivo: acrescimo de informacao nao incluida em arquivos ja '
        'apresentados'),
    (5, 'Desfazimento: arquivo de informacao referente a operacoes/prestacoes nao efetivadas. '
        'Neste caso, o arquivo devera conter, alem dos registros tipo 10 e tipo 90, apenas os '
        'registros Referentes as operacoes/prestacoes nao efetivadas')]


# noinspection PyAbstractClass
class Registro50View(osv.TransientModel):
    _name = "sintegra.registro50"
    _description = "Agrupamento da Tributacao dos Itens da Nota Fiscal"
    _columns = {
        'id': fields.integer('ID'),
        'fisgersin_sintegra_id': fields.many2one('fiscal.gerar.sintegra.osv', 'Sintegra id'),
        'nf_id': fields.integer('NF ID'),
        'sint_reg50_id_company': fields.integer('ID Company'),
        'sint_reg50_cpf_cnpj': fields.char('CPF/CNPJ', size=14),
        'sint_reg50_ie': fields.char('Inscricao Estadual', size=14),
        'sint_reg50_data_emissao': fields.char('Data Emissao', size=8),
        'sint_reg50_uf_sigla': fields.char('UF', size=2),
        'sint_reg50_id_modelo': fields.integer('ID Modelo NF'),
        'sint_reg50_serie': fields.char('Serie', size=3),
        'sint_reg50_numero': fields.integer('Numero'),
        'sint_reg50_cfop': fields.char('CFOP', size=4),
        'sint_reg50_emitente': fields.char('Emitente', size=1),
        'sint_reg50_valor_total_nf': fields.float('Valor Total', digits=(14, 2)),
        'sint_reg50_valor_base_calculo': fields.float('Valor Base Calculo', digits=(14, 2)),
        'sint_reg50_valor_icms': fields.float('Valor ICMS', digits=(14, 2)),
        'sint_reg50_vr_base_icms_substituicao': fields.float('Valor base ICMS substituicao',
                                                             digits=(14, 2)),
        'sint_reg50_vr_icms_substituicao': fields.float('Valor ICMS substituicao', digits=(14, 2)),
        'sint_reg50_tipo_substituicao': fields.boolean('Substituicao'),
        'sint_reg50_valor_isentas_nao_tributadas': fields.float('Valor Isentas e Nao Tributadas',
                                                                digits=(14, 2)),
        'sint_reg50_vr_despesas_acessorias': fields.float('Valor despesas acessorias',
                                                          digits=(14, 2)),
        'sint_reg50_valor_outras': fields.float('Valor Outras', digits=(14, 2)),
        'sint_reg50_aliquota': fields.float('Aliquota', digits=(14, 2)),
        'sint_reg50_situacao': fields.char('Situacao', size=1),
        'sint_reg50_is_nfe': fields.boolean('Flag NFE'),
        'sint_reg50_nfe_chave_acesso': fields.char('NFE Chave Acesso', size=44),
        'sint_reg50_ipi_por_cfop': fields.float('Valor de IPI por CFOP', digits=(14, 12))
    }

    _order = 'sint_reg50_numero'

    @staticmethod
    def gera_registro_50(cr, uid, sintegra_capa_id, company_id, data_ini, data_fim,
                         simples_nacional, fispar_base_reduzida):
        if simples_nacional:
            query_sint_reg50_aliquota = "0               AS sint_reg50_aliquota"
            query_sint_reg50_valor_icms = "0.00          AS sint_reg50_valor_icms"
            query_sint_reg50_valor_base_calculo = "0.00  AS sint_reg50_valor_base_calculo"
            query_sint_reg50_valor_outros = """
                       coalesce(sum((SELECT coalesce(vr_base_calculo, 0)
                           FROM fiscal_nf_item_tributacao
                           WHERE fisnit_tipo_cst IN ('T', 'O', 'U')
                             AND movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
                        )), 0)              AS sint_reg50_valor_outras
            """
        else:
            query_sint_reg50_aliquota = """
                       COALESCE((SELECT aliquota
                        FROM fiscal_nf_item_tributacao
                        WHERE fisnit_capa_id = fisnfc.id
                              AND movimento_itens_id = estmvi.id
                              AND encargo_id = fisenc.id), 0)
                                                   AS sint_reg50_aliquota"""
            query_sint_reg50_valor_icms = """
                       sum(COALESCE((SELECT vr_encargo
                           FROM fiscal_nf_item_tributacao
                           WHERE fisnit_tipo_cst = 'T'
                             AND movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
                        ), 0.00))
                                                   AS sint_reg50_valor_icms
            """
            query_sint_reg50_valor_base_calculo = """
                       sum(COALESCE((SELECT vr_base_calculo
                           FROM fiscal_nf_item_tributacao
                           WHERE fisnit_tipo_cst = 'T'
                             AND movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
                        ), 0.00))
                                                    AS sint_reg50_valor_base_calculo
            """
            base_reduzida_outros = 0.00
            if fispar_base_reduzida == 'O':
                base_reduzida_outros = """
                        sum((SELECT coalesce(vr_reducao_base, 0)
                             FROM fiscal_nf_item_tributacao
                             WHERE movimento_itens_id = estmvi.id
                               AND fisnit_capa_id = fisnfc.id
                               AND encargo_id IN (SELECT id FROM fiscal_encargo
                                                  WHERE sigla = 'ICMS')
                               AND fisnit_cfop_id = fiscfop.id
                               AND aliquota = fisnit.aliquota
                               AND id = fisnit.id
                        ))
                        """
            query_sint_reg50_valor_outros = """
                       coalesce(sum((SELECT coalesce(vr_base_calculo, 0) 
                                            + coalesce(vr_icms_substituicao, 0)
                                            + coalesce(fisnit_vr_fcp_st, 0)
                           FROM fiscal_nf_item_tributacao
                           WHERE fisnit_tipo_cst IN ('O', 'U', 'TU')
                             AND movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
                        )), 0)
                       + {base_reduzida}
                          AS sint_reg50_valor_outras
            """.format(base_reduzida=base_reduzida_outros)
        base_reduzida_isentos = 0.00
        if fispar_base_reduzida == 'I':
            base_reduzida_isentos = """
                      sum((SELECT coalesce(vr_reducao_base, 0)
                           FROM fiscal_nf_item_tributacao
                           WHERE movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo
                                                WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
                        ))
            """
        cr.execute('delete from sintegra_registro50')
        sql = """
        -- noinspection SqlDialectInspection
        INSERT INTO sintegra_registro50(
                     fisgersin_sintegra_id                                                      -- 1
                  ,  create_uid
                  ,  nf_id
                  ,  sint_reg50_id_company
                  ,  sint_reg50_cpf_cnpj
                  ,  sint_reg50_ie
                  ,  sint_reg50_data_emissao
                  ,  sint_reg50_uf_sigla
                  ,  sint_reg50_id_modelo
                  ,  sint_reg50_serie
                  ,  sint_reg50_numero
                  ,  sint_reg50_cfop
                  ,  sint_reg50_emitente
                  ,  sint_reg50_valor_total_nf
                  ,  sint_reg50_valor_base_calculo                                              -- 2
                  ,  sint_reg50_valor_icms                                                      -- 3
                  ,  sint_reg50_valor_isentas_nao_tributadas
                  ,  sint_reg50_valor_outras                                                    -- 4
                  ,  sint_reg50_aliquota                                                        -- 5
                  ,  sint_reg50_situacao
                  ,  sint_reg50_is_nfe
                  ,  sint_reg50_vr_base_icms_substituicao
                  ,  sint_reg50_vr_icms_substituicao
                  ,  sint_reg50_tipo_substituicao
                  ,  sint_reg50_vr_despesas_acessorias
                  ,  sint_reg50_nfe_chave_acesso
                  ,  sint_reg50_ipi_por_cfop)
        SELECT {sintegra_capa_id}                  AS fisgersin_sintegra_id
             , {uid}                               AS create_uid
             , fisnfc.id                           AS nf_id
             , fisnfc.company_id                   AS sint_reg50_id_company
             , CASE WHEN admuf.sigla ILIKE 'ex' THEN ''
               ELSE admpes.cpf_cnpj END            AS sint_reg50_cpf_cnpj                      -- 02
             , CASE WHEN admpes.admpes_jur_ie IS NOT NULL AND NOT admuf.sigla ILIKE 'ex' 
               THEN admpes.admpes_jur_ie
               ELSE 'ISENTO' END                   AS sint_reg50_ie                            -- 03
             , CASE WHEN fisnfc.tipo = 'E' THEN
                   TO_CHAR(fisnfc.data_saida_entrada AT TIME ZONE 'UTC-3', 'YYYYMMDD')
               ELSE
                   TO_CHAR(fisnfc.data_emissao AT TIME ZONE 'UTC-3', 'YYYYMMDD')
               END                                 AS sint_reg50_data_emissao                  -- 04
             , admuf.sigla                         AS sint_reg50_uf_sigla                      -- 05
             , CASE WHEN fisnfc.nf_propria = TRUE THEN fismns.modelo_nf_id
               ELSE fismnf.id END                  AS sint_reg50_id_modelo
             , CASE WHEN fisnfc.serie_nf = '0' or fisnfc.serie_nf is null THEN ''
               ELSE fisnfc.serie_nf  END           AS sint_reg50_serie                         -- 07
             , fisnfc.numero_nf                    AS sint_reg50_numero                        -- 08
             , CASE
                WHEN fismnf.fismnf_tipo = 'T' THEN fiscfop2.codigo
                ELSE fiscfop.codigo
               END                                 AS sint_reg50_cfop                          -- 09
             , CASE WHEN fisnfc.nf_propria = true THEN 'P'
               ELSE 'T' END                        AS sint_reg50_emitente                      -- 10
             , COALESCE(CASE
                WHEN fismnf.fismnf_tipo != 'T' THEN
                       Sum(estmvi.vr_total_item
                       + estmvi.vr_despesas_rateado
                       + estmvi.vr_seguro_rateado
                       + estmvi.vr_frete_rateado - estmvi.vr_desconto_rateado)
                ELSE
                    fisnfc.vr_total_nf
               END, 0)                                 AS sint_reg50_valor_total_nf            -- 11
             , {vr_base_calculo}                                                               -- 12
             , {vr_icms}                                                                       -- 13
             ,coalesce(sum((SELECT coalesce(fisnit_vr_isento, 0)
                   FROM fiscal_nf_item_tributacao
                   WHERE fisnit_tipo_cst = 'I'
                         AND movimento_itens_id = estmvi.id
                         AND fisnit_capa_id = fisnfc.id
                         AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                         AND fisnit_cfop_id = fiscfop.id
                         AND aliquota = fisnit.aliquota
                         AND id = fisnit.id
                )), 0)
               + {base_reduzida}
                                                   AS sint_reg50_valor_isentas_nao_tributadas  -- 14
             ,{outros}                                                                         -- 15
             ,{aliquota}                                                                       -- 16
             , CASE 
                    WHEN fisnfc.state = 'C' THEN 'S'
                    WHEN fisnfc.state = 'D' THEN '2'
                    WHEN fisnfc.state = 'I' THEN '4'
                    ELSE 'N' 
               END                                 AS sint_reg50_situacao                      -- 17
             , CASE WHEN fisnfc.nf_propria = TRUE THEN fismns.flag_modelo_nfe
               ELSE fismnf.fismnf_tipo_nfe END     AS sint_reg50_is_nfe
             ,coalesce(sum((SELECT coalesce(vr_base_icms_substituicao, 0)
                           FROM fiscal_nf_item_tributacao
                           WHERE fisnit_tipo_cst IN ('O', 'U')
                             AND movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
             )), 0)                                AS sint_reg50_vr_base_icms_substituicao
             ,coalesce(sum((SELECT coalesce(vr_icms_substituicao, 0)
                           FROM fiscal_nf_item_tributacao
                           WHERE fisnit_tipo_cst IN ('O', 'U')
                             AND movimento_itens_id = estmvi.id
                             AND fisnit_capa_id = fisnfc.id
                             AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                             AND fisnit_cfop_id = fiscfop.id
                             AND aliquota = fisnit.aliquota
                             AND id = fisnit.id
             )), 0)                                AS sint_reg50_vr_icms_substituicao
             , CASE WHEN (SUM(CASE WHEN fisnit.tipo_substituicao = TRUE THEN
                                  fisnit.vr_icms_substituicao
                              ELSE 0.00 END)) > 0 THEN TRUE
               ELSE FALSE END                      AS sint_reg50_tipo_substituicao
             , fisnfc.vr_despesas_acessorias       AS sint_reg50_vr_despesas_acessorias
             , fisnfc.nfe_chave_acesso             AS sint_reg50_nfe_chave_acesso
             , fisnfc.vr_ipi                       AS sint_reg50_ipi_por_cfop
        FROM   fiscal_nf_capa fisnfc
               LEFT JOIN adm_pessoa                AS admpes
                    ON admpes.id = fisnfc.destinatario_id
               LEFT JOIN adm_pessoa_endereco       AS admpesend
                    ON fisnfc.destinatario_endereco_id = admpesend.id
               LEFT JOIN adm_uf admuf
                    ON admuf.id = admpesend.uf_id  
               LEFT JOIN estoque_movimento_itens   AS estmvi
                    ON fisnfc.id = estmvi.nf_id
              LEFT JOIN fiscal_nf_item_tributacao AS fisnit
                    ON fisnfc.id = fisnit.fisnit_capa_id
               LEFT JOIN fiscal_cst                AS fiscst
                    ON fiscst.id = fisnit.cst_id
               LEFT JOIN fiscal_cfop               AS fiscfop
                    ON fiscfop.id = estmvi.cfop_id
               LEFT JOIN fiscal_cfop_natureza      AS fisnat
                      ON fisnat.id = fisnfc.natureza_id
               LEFT JOIN fiscal_cfop               AS fiscfop2
                      ON fiscfop2.id = fisnat.cfop_id
               LEFT JOIN fiscal_encargo            AS fisenc
                    ON fisenc.id = fisnit.encargo_id
               LEFT JOIN fiscal_modelo_nf          AS fismnf
                    ON fismnf.id = fisnfc.modelo_nf_id
               LEFT JOIN fiscal_modelo_nf_series   AS fismns
                    ON fismns.id = fisnfc.modelo_nf_series_id
        WHERE  CASE WHEN fisnfc.tipo = 'E' THEN
                   fisnfc.data_saida_entrada BETWEEN '{dt_ini}' AND '{dt_fim}'
               ELSE fisnfc.data_emissao BETWEEN '{dt_ini}' AND '{dt_fim}' END
         AND   fisnfc.company_id = {company_id}
         AND   fisnfc.state = '3'
         OR fisnfc.state IN ( 'C', 'I', 'R', 'D' )
         AND   CASE WHEN fisnfc.tipo = 'E' THEN fisnfc.data_saida_entrada BETWEEN '{dt_ini}' AND '{dt_fim}'
               ELSE fisnfc.data_emissao BETWEEN '{dt_ini}' AND '{dt_fim}' END
         AND   fisnfc.company_id = {company_id}
        GROUP  BY fisnfc.company_id, fisnfc.id, admpes.cpf_cnpj, fismns.flag_modelo_nfe,
                  fismnf.fismnf_tipo_nfe, fisnfc.data_saida_entrada, admpes.admpes_jur_ie,
                  admuf.sigla, fismns.modelo_nf_id, fismnf.id, fiscfop.codigo, sint_reg50_aliquota,
                  fisenc.sigla, fiscfop2.codigo
        HAVING (fisnfc.state != 'C' AND fisenc.sigla = 'ICMS')
            OR (fisnfc.state = 'I')
            OR (fisnfc.state = 'C' AND fisnfc.numero_nf > 0 AND fisnfc.serie_nf != ''
                AND (fisnfc.nf_propria = TRUE -- Nota fiscal propria
                     AND (fismns.flag_modelo_nfe = FALSE
                          OR (fismns.flag_modelo_nfe = TRUE AND
                              fisnfc.nfe_chave_acesso IS NOT NULL
                             )
                         )
                    )
                 OR (fisnfc.nf_propria = FALSE -- Nota fiscal de terceiro
                     AND (fismnf.fismnf_tipo_nfe = FALSE
                          OR (fismnf.fismnf_tipo_nfe = TRUE
                              AND fisnfc.nfe_chave_acesso != '' )
                         )
                     AND fisnfc.state != 'C'
                     AND fisenc.sigla = 'ICMS'
                    )
               )
                OR (fisnfc.state != 'C'
                    AND fismnf.fismnf_tipo = 'T'
                    AND fisenc.sigla = 'ICMS')
        """.format(sintegra_capa_id=sintegra_capa_id, uid=uid,
                   vr_base_calculo=query_sint_reg50_valor_base_calculo,
                   vr_icms=query_sint_reg50_valor_icms, outros=query_sint_reg50_valor_outros,
                   aliquota=query_sint_reg50_aliquota, dt_ini=data_ini, dt_fim=data_fim,
                   company_id=company_id, base_reduzida=base_reduzida_isentos)
        cr.execute(sql)
        return True

_state_readonly = {'A': [('readonly', False)], 'E': [('readonly', True)]}


# noinspection PyAbstractClass,PyPep8Naming
class fiscal_gerar_sintegra_osv(osv.Model):
    _grade_ids = []

    _inherit = 'res.config.settings'

    _name = 'fiscal.gerar.sintegra.osv'
    _description = 'Geracao do Sped Fiscal'
    _logger = logging.getLogger(_name)
    _columns = {
        'state': fields.selection(string='Status',
                                  selection=[('A', 'Aguardando geracao do arquivo'),
                                             ('E', 'Arquivo gerado')], required=True, size=1),
        'inventario_id': fields.many2one("estoque.inventario.capa", 'Inventario',
                                         ondelete="cascade", select=True,
                                         domain="[('company_id', '=', company_id)]",
                                         states=_state_readonly),
        'data_ini': fields.date(string='Data Inicial', required=True, states=_state_readonly),
        'data_fim': fields.date(string='Data Final', required=True, states=_state_readonly),
        'gerar_registro60i': fields.boolean(string='Gerar Registro 60I'),
        'cod_ident_con': fields.selection(selection=_tbl_cod_ident_con,
                                          string='Codigo da identificacao do Convenio',
                                          size=1, required=True, states=_state_readonly),
        'cod_ident_nat': fields.selection(selection=_tbl_cod_ident_nat, states=_state_readonly,
                                          string='Codigo da identificacao da natureza das '
                                                 'operacoes informadas', size=1, required=True),
        'cod_ident_fin': fields.selection(selection=_tbl_cod_ident_fin,
                                          string='Codigo da finalidade do arquivo magnetico',
                                          size=1, required=True, states=_state_readonly),
        'anexo': fields.binary('Anexo'),
        'memo': fields.text('Memo'),
        'file_name': fields.char('Arquivo', size=20, readonly=False),
        'informacao': fields.text('Informacoes'),
        'registro50_ids': fields.one2many('sintegra.registro50', 'fisgersin_sintegra_id',
                                          'registro50_ids'),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="restrict", required=True,
                                      select=True),
    }
    _defaults = {
        'state': lambda *a: 'A',
        'cod_ident_con': lambda *a: 3,
        'cod_ident_nat': lambda *a: 3,
        'cod_ident_fin': lambda *a: 1,
        'gerar_registro60i': lambda *a: False,
        'file_name': lambda *a: 'sintegra.txt',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                                        s._name,
                                                                                        context=c),
    }

    def onchange_inventario(self, cr, uid, ids, inventario_id, context=None):
        context = context and context.copy() or {}
        res = {}
        obj_estoque_inventario_capa = self.pool.get('estoque.inventario.capa')

        if inventario_id:
            brw_inventario_capa = obj_estoque_inventario_capa.browse(
                cr, uid, inventario_id, context)
            data_livro_inventario = brw_inventario_capa.esticp_data_livro_inventario

            if not data_livro_inventario:
                res['warning'] = {'title': _('Atencao'),
                                  'message': _('Nao foi gerado o livro registro de inventario para:'
                                               ' %s' % brw_inventario_capa.name)}
                res.update({'value': {'inventario_id': False}})

        return res

    def _get_dados_registro10(self, cr, uid, company_id, context=None):
        """
        Obter os dados da empresa para geracao do Registro 10 do Sintegra
        """
        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_adm_empresa = self.pool.get('res.company')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_adm_parametro = self.pool.get('adm.parametro')

        data_browse_adm_empresa = obj_adm_empresa.browse(cr, uid, company_id)
        pessoa_id = data_browse_adm_empresa.rescon_pessoa_id.id

        # CNPJ da Empresa
        cgc = data_browse_adm_empresa.rescon_pessoa_id.cpf_cnpj

        if not cgc:
            raise except_osv(_('Atencao'), _('Empresa nao possui CPFJ cadastrado'))

        cgc = int(re.sub('[^0-9]', '', cgc))

        # Inscricao Estadual da Empresa
        ie = data_browse_adm_empresa.rescon_pessoa_id.admpes_jur_ie
        ie = 'ISENTO' if not ie else re.sub('[^0-9]', '', ie)

        # Nome do Contibuinte
        nome = data_browse_adm_empresa.rescon_pessoa_id.name

        # Informacoes do Socio Responsável
        par_company_ids = obj_adm_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)

        if not par_company_ids:
            raise except_osv(_('Atencao'), _('Parametros de administracao nao cadastrados para a '
                                             'Empresa.'))

        data_browse_adm_parametro = obj_adm_parametro.browse(cr, uid, par_company_ids[0])
        nome_socio = data_browse_adm_parametro.socio_id.name
        fone = data_browse_adm_parametro.socio_id.telefone or '0'
        fone = int(re.sub('[^0-9]', '', fone))

        # FAX
        fones = obj_adm_pessoa.get_telefones(cr, uid, [pessoa_id]).get(pessoa_id)

        if not fones:
            raise except_osv(_('Atencao'),
                             _('Numero do telefone do estabelecimento informante nao cadastrado'))
        fax = fones.get('Fax') or fones.get('Comercial') or fones.get('Celular') or ''

        if fax:
            fax = int(re.sub('[^0-9]', '', fax[0]))

        if not fax:
            raise except_osv(_('Atencao'),
                             _('Numero do telefone do estabelecimento informante nao cadastrado '
                               '(Fax ou Comercial)'))

        # Endereco
        endereco_fiscal = obj_adm_pessoa.admpes_endereco_fiscal(
            cr, uid, [pessoa_id], context=context)
        endereco_fiscal = endereco_fiscal.get(pessoa_id)

        if not endereco_fiscal:
            raise except_osv(_('Atencao'), _('Nao ha endereco cadastrado para a Empresa.'))

        municipio = endereco_fiscal.get('cidade')

        if not municipio:
            raise except_osv(_('Atencao'),
                             _('Municipio onde esta domiciliado o estabelecimento informante nao '
                               'cadastrado'))

        uf = endereco_fiscal.get('uf_sigla')

        if not uf:
            raise except_osv(_('Atencao'),
                             _('Unidade da Federacao referente ao Municipio nao cadastrado'))

        numero_endereco = endereco_fiscal.get('numero') or '0'
        complemento = endereco_fiscal.get('complemento') or ''
        separa_numero_letras = re.split('(\d+)', numero_endereco)

        for x in separa_numero_letras:
            if x:
                try:
                    numero_endereco = int(x)
                except ValueError:
                    complemento += x

        return dict(logradouro=endereco_fiscal.get('logradouro') or '', numero=numero_endereco,
                    complemento=complemento, bairro=endereco_fiscal.get('bairro', '') or '',
                    cep=re.sub('[^0-9]', '', endereco_fiscal.get('cep', '0')) or '0',
                    pessoa_id=pessoa_id, cgc=cgc, ie=ie, nome=nome, nome_socio=nome_socio,
                    municipio=municipio, uf=uf, fone=fone, fax=fax)

    def _get_dados_registro50(self, cr, uid, sintegra_id, company_id, data_ini, data_fim,
                              dt_ini, dt_fim, context=None):
        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_fiscal_modelo_nf = self.pool.get('fiscal.modelo.nf')

        sintegra_registro50 = self.pool.get('sintegra.registro50')
        mensagem = ''
        msg_50 = []

        fiscal_modelo_nf_ids = obj_fiscal_modelo_nf.search(
            cr, uid, [('fismnf_codigo', 'in', ["01", "1A", "1B", "06", "21", "22", "55"])],
            context=context)

        # data_ini = obj_adm_utils.admutl_converte_tz(cr, uid, data_ini, tz_destino='UTC',
        #                                             context=context)
        # data_fim = obj_adm_utils.admutl_converte_tz(cr, uid, data_fim, tz_destino='UTC',
        #                                             context=context)
        nf_ids = obj_fiscal_nf_capa.search(
            cr, uid, [('data_emissao', '>=', str(data_ini)), ('data_emissao', '<=', str(data_fim)),
                      ('modelo_nf_id', 'in', fiscal_modelo_nf_ids), ('company_id', '=', company_id),
                      ('state', 'in', ['3', 'C', 'I', 'R']),
                      ('nf_propria', '=', True)], context=context)
        nf_terceiro_ids = obj_fiscal_nf_capa.search(
            cr, uid, [('data_saida_entrada', '>=', str(data_ini)),
                      ('data_saida_entrada', '<=', str(data_fim)),
                      ('modelo_nf_id', 'in', fiscal_modelo_nf_ids), ('company_id', '=', company_id),
                      ('state', 'in', ['3', 'C', 'I', 'R']), ('tipo', '=', 'E'),
                      ('nf_propria', '=', False)], context=context)
        nf_ids.extend(nf_terceiro_ids)
        nf_ids = list(set(nf_ids))
        encargo_id = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', '=', 'ICMS')], context=context)
        cst_ids = self.pool.get('fiscal.cst').search(
            cr, uid, [('encargo_id', 'in', encargo_id)], context=context.update({
                'active_test': False
            }))

        produto_ids = []
        data = {
            'reg50': None,
            'reg54': None,
            'produto_ids': produto_ids
        }

        # Registro 50
        dreg50 = []  # Lista de dicionario com os dados do registro 50
        dreg53 = []  # Lista de dicionario com os dados do resgistro 53
        dreg70 = []  # Lista de dicionario com os dados do resgistro 70

        args_reg50 = [
            ('fisgersin_sintegra_id', '=', sintegra_id),
            # ('sint_reg50_data_emissao', '>=', dt_ini.strftime('%Y%m%d')),
            # ('sint_reg50_data_emissao', '<=', dt_fim.strftime('%Y%m%d')),
            ('sint_reg50_id_company', '=', company_id),
            ('sint_reg50_id_modelo', 'in', fiscal_modelo_nf_ids)
        ]
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        simples_nacional = obj_adm_pessoa.verificar_empresa_simples_nacional(cr, uid)
        reg50_ids = sintegra_registro50.search(cr, uid, args_reg50, context=context)
        dict_modelo_nf = dict()

        if reg50_ids:
            fields_to_read = [
                'nf_id',
                'sint_reg50_id_company', 'sint_reg50_cpf_cnpj', 'sint_reg50_ie',
                'sint_reg50_data_emissao', 'sint_reg50_uf_sigla', 'sint_reg50_id_modelo',
                'sint_reg50_serie', 'sint_reg50_numero', 'sint_reg50_cfop', 'sint_reg50_emitente',
                'sint_reg50_valor_total_nf', 'sint_reg50_valor_base_calculo',
                'sint_reg50_valor_icms', 'sint_reg50_valor_isentas_nao_tributadas',
                'sint_reg50_valor_outras', 'sint_reg50_aliquota', 'sint_reg50_situacao',
                'sint_reg50_tipo_substituicao', 'sint_reg50_vr_base_icms_substituicao',
                'sint_reg50_vr_icms_substituicao', 'sint_reg50_vr_despesas_acessorias',
                'sint_reg50_ipi_por_cfop'
            ]
            reg50_dados = sintegra_registro50.read(
                cr, uid, reg50_ids, fields_to_read, context=context)
            modelo_ids = map(lambda m: m['sint_reg50_id_modelo'], reg50_dados)
            modelo_nf_dados = obj_fiscal_modelo_nf.read(
                cr, uid, modelo_ids, ['fismnf_codigo'], context=context)
            dict_modelo_nf = dict(map(lambda m: (m['id'], m['fismnf_codigo']), modelo_nf_dados))
            dreg53 = filter(lambda f: f.get('sint_reg50_vr_icms_substituicao') > 0, reg50_dados)

            arg_70 = ('07', '08', '8B', '09', '10', '11', '26', '27', '57')
            modelo_nf_transportes_ids = obj_fiscal_modelo_nf.search(
                cr, uid, [('fismnf_codigo', 'in', arg_70)], context=context)
            dreg70_args_reg50 = [
                ('fisgersin_sintegra_id', '=', sintegra_id),
                # ('sint_reg50_data_emissao', '>=', dt_ini.strftime('%Y%m%d')),
                # ('sint_reg50_data_emissao', '<=', dt_fim.strftime('%Y%m%d')),
                ('sint_reg50_id_company', '=', company_id),
                ('sint_reg50_id_modelo', 'in', modelo_nf_transportes_ids)]
            dreg70_reg50_ids = sintegra_registro50.search(cr, uid, dreg70_args_reg50,
                                                          context=context)
            dreg70 = sintegra_registro50.read(cr, uid, dreg70_reg50_ids, fields_to_read,
                                              context=context)
            transp_modelo_nf_dados = obj_fiscal_modelo_nf.read(
                cr, uid, modelo_nf_transportes_ids, ['fismnf_codigo'], context=context)
            dict_modelo_nf.update(dict(map(lambda m: (m['id'], m['fismnf_codigo']),
                                           transp_modelo_nf_dados)))

            arg_50 = ('07', '08', '8B', '09', '10', '11', '26', '27', '57', '02', '2D')
            cupom_nf_ids = list()

            for i in reg50_dados:
                if i.get('nf_id') in cupom_nf_ids:
                    continue
                num_nf = str(i.get('sint_reg50_numero'))

                if len(num_nf) > 6:
                    i['sint_reg50_numero'] = int(num_nf[-6:])

                modelo = i.get('sint_reg50_id_modelo')
                modelo_nota = dict_modelo_nf.get(modelo) or ''

                if modelo_nota == '1B':
                    modelo_nota = '01'

                try:
                    int(modelo_nota)
                except:
                    if i['sint_reg50_numero'] not in msg_50:
                        mensagem += _('* Atencao : Modelo ' + modelo_nota +
                                      ' invalido para a nota de numero: ' +
                                      str(i['sint_reg50_numero']) + '  \n')
                        msg_50.append(i['sint_reg50_numero'])
                    continue

                modelo_fiscal = obj_fiscal_modelo_nf.verifica_codigo_modelo_fiscal(
                    cr, uid, modelo)

                if modelo_nota not in arg_50 and modelo_fiscal:
                    tipo_01 = '50'
                    cpf_cnpj_02 = i.get('sint_reg50_cpf_cnpj') or '0'
                    ie_03 = i.get('sint_reg50_ie')
                    ie_03 = obj_adm_utils.admutl_remove_caracter(ie_03, letras=True, numeros=True,
                                                                 espaco=False, outros=None)
                    data_emissao_04 = i.get('sint_reg50_data_emissao')
                    uf_05 = i.get('sint_reg50_uf_sigla') or ''
                    modelo_06 = modelo_nota
                    serie_07 = i.get('sint_reg50_serie') or ''
                    numero_08 = i.get('sint_reg50_numero')
                    cfop_09 = i.get('sint_reg50_cfop')
                    aliquota_16 = i.get('sint_reg50_aliquota')
                    if cfop_09 in ('5929', '6929'):
                        base_calculo_12 = valor_icms_13 = isenta_nao_tributada_14 = outras_15 = \
                            aliquota_16 = 0
                        cupom_nf_ids.append(i.get('nf_id'))
                        valor_total_11 = self.get_total_itens_por_cfop(
                            cr, uid, i.get('nf_id'), i.get('sint_reg50_cfop'),
                            i.get('sint_reg50_aliquota'), simples_nacional, context=context)
                        total_item_nf = round(valor_total_11 * 1e2, 2)
                    else:
                        valor_total_11 = self.get_total_itens_por_cfop(
                            cr, uid, i.get('nf_id'), i.get('sint_reg50_cfop'),
                            i.get('sint_reg50_aliquota'), simples_nacional, context=context)
                        # valor_total_11 += vr_ipi_por_cfop
                        if not simples_nacional:
                            base_calculo_12 = float(float_round(i.get(
                                'sint_reg50_valor_base_calculo') or 0.00, 2))
                        else:
                            base_calculo_12 = 0
                            aliquota_16 = 0
                        valor_icms_13 = float(float_round(i.get('sint_reg50_valor_icms') or 0.0, 2)) \
                            if not simples_nacional else 0
                        isenta_nao_tributada_14 = float(float_round(
                            i.get('sint_reg50_valor_isentas_nao_tributadas') or 0.0, 2))
                        vr_outras = (i.get('sint_reg50_valor_outras') or 0.00) + \
                            self.get_valor_ipi_reg50_outras(
                                cr, uid, i.get('nf_id'), i.get('sint_reg50_aliquota'),
                                i.get('sint_reg50_cfop'), context)
                        outras_15 = float(float_round(vr_outras, 2))
                        total_item_nf = round(valor_total_11 * 1e2, 2)
                    emitente_10 = i.get('sint_reg50_emitente')
                    situacao_17 = i.get('sint_reg50_situacao')
                    retval = dict(
                        tipo=tipo_01, cnpj=int(re.sub('[^0-9]', '', cpf_cnpj_02)), estadual=ie_03,
                        date=int(data_emissao_04), state=uf_05, modelo=int(modelo_06),
                        serial=serie_07, numero=numero_08, cfop=int(cfop_09), emitente=emitente_10,
                        total=int(total_item_nf), icms_base=int(base_calculo_12 * 1e2),
                        icms_total=int(valor_icms_13 * 1e2),
                        isenta=int(isenta_nao_tributada_14 * 1e2), outras=int(outras_15 * 1e2),
                        aliquota_icms=int(aliquota_16 * 1e2), situacao=situacao_17)
                    dreg50.append(retval)

        # Registro 53
        if dreg53:
            dreg53 = self._get_dados_registro53(cr, uid, dreg53, dict_modelo_nf)

        # Registro 54
        dreg54 = []

        # Registro 55
        dreg55 = self._get_dados_registro55(cr, nf_ids)

        if nf_ids:
            sql = """
                -- noinspection SqlDialectInspection
                SELECT estmvi.nf_id                               AS nf_id
                     , estmvi.grade_id                            AS grade_id
                     , COALESCE(estmvi.quantidade, 0)             AS quantidade
                     , COALESCE(estmvi.vr_desconto, 0) +
                       COALESCE(estmvi.vr_desconto_rateado, 0)    AS vr_desconto
                     , COALESCE(estmvi.vr_total_item, 0)          AS vr_total_item
                     , COALESCE(estmvi.vr_unitario, 0) * 
                       COALESCE(estmvi.quantidade, 0)             AS vr_bruto_item
                     , fisnit.aliquota                            AS aliquota
                     , fismnf.fismnf_tipo                         AS tipo_nf
                     , fisnfc.numero_nf                           AS numero_nf
                     , fismnf.id                                  AS modelo_nf_id
                     , fismnf.fismnf_codigo                       AS modelo
                     , CASE WHEN fisnfc.serie_nf = '0' THEN ''
                       ELSE fisnfc.serie_nf END                   AS serie_nf
                     , CASE WHEN admuf.sigla ILIKE 'ex' THEN ''
                       ELSE admpes.cpf_cnpj END                   AS cnpj
                     , fiscfop.codigo                             AS cfop
                     , CASE WHEN fiscst.tipo = 'T' THEN COALESCE(fisnit.vr_base_calculo, 0)
                       ELSE 0 END                                 AS vr_base_calculo
                     , CASE WHEN fisnit.vr_base_icms_substituicao > 0 THEN
                           fisnit.vr_base_icms_substituicao
                       ELSE 0 END                                 AS vr_base_substituicao
                     , produto.origem_mercadoria || fiscst.codigo AS origem_mercadoria_cst
                     , sequencia                                  AS sequencia
                     , estpro.codigo                              AS codigo_produto
                     , estpgr.grade                               AS grade
                     , LPAD(RIGHT(CAST(replace(produto.codigo, ' ', '') AS VARCHAR), 11) || '_' ||
                       TRIM(estpgr.grade), 14, '0')               AS item
                     , fiscst.fiscst_csosn_contribuinte_id        AS csosn_id
                     , CASE WHEN fisenc.sigla = 'ICMS' THEN fiscst.codigo
                       ELSE '' END                                AS cst_codigo
                     , CASE WHEN fisenc.sigla = 'ICMS' THEN
                           produto.origem_mercadoria || fiscst.codigo
                       ELSE '' END                                AS origem_mercadoria_cst
                     , CASE WHEN fisenc.sigla = 'IPI' THEN COALESCE(fisnit.vr_encargo, 0)
                       ELSE 0 END                                 AS valor_ipi
                FROM   estoque_movimento_itens                    AS estmvi
                LEFT   JOIN fiscal_nf_capa                        AS fisnfc
                       ON fisnfc.id = estmvi.nf_id
                LEFT   JOIN adm_pessoa                            AS admpes
                       ON admpes.id = fisnfc.destinatario_id
                LEFT   JOIN adm_pessoa_endereco                   AS admpesend
                       ON admpesend.id = fisnfc.destinatario_endereco_id 
                LEFT   JOIN adm_uf                                AS admuf
                       ON admuf.id = admpesend.uf_id
                LEFT   JOIN fiscal_cfop                           AS fiscfop
                       ON fiscfop.id = estmvi.cfop_id
                LEFT   JOIN estoque_produto                       AS produto
                       ON produto.id = estmvi.produto_id
                LEFT   JOIN fiscal_nf_item_tributacao             AS fisnit
                       ON fisnit.movimento_itens_id = estmvi.id
                LEFT   JOIN fiscal_cst                            AS fiscst
                       ON fiscst.id = fisnit.cst_id
                LEFT   JOIN fiscal_encargo                        AS fisenc
                       ON fisenc.id = fiscst.encargo_id
                LEFT   JOIN estoque_produto                       AS estpro
                       ON estpro.id = estmvi.produto_id
                LEFT   JOIN estoque_produto_grade                 AS estpgr
                       ON estpgr.id = estmvi.grade_id
                LEFT   JOIN fiscal_modelo_nf                      AS fismnf
                       ON fismnf.id = fisnfc.modelo_nf_id
                WHERE  fismnf.fismnf_codigo != '06'
                  AND  fisenc.id     IN %s
                  AND  estmvi.nf_id  IN %s
                  AND  fisnit.cst_id IN %s
                  AND  fiscfop.codigo NOT IN ('1206', '2206', '5206', '6206')
                ORDER  BY estmvi.nf_id, estmvi.sequencia ASC
                """ % (str(encargo_id).replace('[', '(').replace(']', ')'),
                       str(nf_ids).replace('[', '(').replace(']', ')'),
                       str(cst_ids).replace('[', '(').replace(']', ')'))
            cr.execute(sql)
            dados_54 = cr.dictfetchall()

            sql_vr_ipi = """
                -- noinspection SqlDialectInspection
                SELECT estmvi.nf_id
                     , fisnit.vr_encargo              AS vr_ipi
                     , LPAD(RIGHT(CAST(estpro.codigo  AS VARCHAR), 11) || '_' || 
                       TRIM(estprg.grade), 14, '0')   AS item
                FROM   estoque_movimento_itens        AS estmvi
                INNER  JOIN estoque_produto           AS estpro
                       ON estpro.id = estmvi.produto_id
                INNER  JOIN estoque_produto_grade     AS estprg
                       ON estprg.id = estmvi.grade_id
                INNER  JOIN fiscal_nf_item_tributacao AS fisnit
                       ON fisnit.movimento_itens_id = estmvi.id
                INNER  JOIN fiscal_cst                AS fiscst
                       ON fiscst.id = fisnit.cst_id
                INNER  JOIN fiscal_encargo            AS fisenc
                       ON fisenc.id = fiscst.encargo_id
                WHERE  estmvi.nf_id IN %s
                  AND  fisenc.sigla = 'IPI'
            """ % str(nf_ids).replace('[', '(').replace(']', ')')
            cr.execute(sql_vr_ipi)
            dados_vr_ipi_54 = cr.dictfetchall()
            obj_fiscal_csosn = self.pool.get('fiscal.csosn')
            for fisnfc_id in nf_ids:
                for i in filter(lambda x: x.get('nf_id') == fisnfc_id, dados_54):
                    if not i.get('cfop') in '5929' and '6929':
                        modelo_fiscal = obj_fiscal_modelo_nf.verifica_codigo_modelo_fiscal(
                            cr, uid, i.get('modelo_nf_id'))

                        if modelo_fiscal:
                            grade_id = i.get('grade_id') or False

                            # Lista com id dos produtos p/ geracao do registro 75
                            if grade_id not in self._grade_ids:
                                self._grade_ids.append(grade_id)

                            cst_csosn = i.get('cst_codigo')

                            if simples_nacional:
                                csosn_id = i.get('csosn_id')
                                if csosn_id:
                                    cst_csosn = obj_fiscal_csosn.read(cr, uid, csosn_id, ['codigo']).get('codigo')

                            if i.get('cst_codigo') != '':
                                numero_nf = i.get('numero_nf')
                                num_nf = str(i.get('numero_nf'))

                                if len(num_nf) > 6:
                                    numero_nf = int(num_nf[-6:])
                                cnpj = int(re.sub('[^0-9]', '', i.get('cnpj'))) if i.get('cnpj') else ''
                                nf_id = i.get('nf_id')
                                item = i.get('item') or ''
                                dados_valor_ipi = filter(
                                    lambda x: x.get('nf_id') == nf_id and x.get('item') == item,
                                    dados_vr_ipi_54)
                                valor_ipi = 0

                                if dados_valor_ipi:
                                    if isinstance(dados_valor_ipi, list):
                                        dados_valor_ipi = dados_valor_ipi[0]
                                    valor_ipi = dados_valor_ipi.get('vr_ipi') or 0

                                modelo = i.get('modelo')
                                if modelo == '1B':
                                    modelo = '01'

                                retval = dict(tipo=i.get('tipo_nf'), cnpj=cnpj,
                                              modelo=int(modelo), serial=i.get('serie_nf'),
                                              numero=numero_nf, cfop=int(i.get('cfop') or 0),
                                              cst='%03d' % int(cst_csosn), sequencia=i.get('sequencia'),
                                              codigo=item, quantidade=int(float_round(i.get('quantidade'), 3) * 1e3),
                                              valor=int(float_round(i.get('vr_bruto_item'), 2) * 1e2),
                                              vr_desconto=int(float_round(i.get('vr_desconto'), 2) * 1e2),
                                              base_calculo=int(float_round(i.get('vr_base_calculo'), 2) * 1e2
                                                               if not simples_nacional else 0.00 * 1e2),
                                              base_sub=int(float_round(i.get('vr_base_substituicao'), 2) * 1e2),
                                              ipi=int(valor_ipi * 1e2),
                                              aliquota=int(i.get('aliquota') * 1e2
                                                           if not simples_nacional else 0 * 1e2))
                                dreg54.append(retval)
                reg_54_frete = self.gera_registro_54_especifico(cr, uid, fisnfc_id, 991, context)
                if reg_54_frete:
                    dreg54.append(reg_54_frete)
                reg_54_seguro = self.gera_registro_54_especifico(cr, uid, fisnfc_id, 992, context)
                if reg_54_seguro:
                    dreg54.append(reg_54_seguro)
                reg_54_despesas_acessorias = self.gera_registro_54_especifico(cr, uid, fisnfc_id,
                                                                              999, context)
                if reg_54_despesas_acessorias:
                    dreg54.append(reg_54_despesas_acessorias)
        # Registro 70
        if dreg70:
            dreg70 = self._get_dados_registro70(cr, uid, dreg70, dict_modelo_nf)

        data.update({
            'reg50': dreg50,
            'reg53': dreg53,
            'reg55': dreg55,
            'reg54': dreg54,
            'reg70': dreg70,
            'mensagem_50': mensagem
        })

        return data

    def _get_dados_registro53(self, cr, uid, dados_registro_53, dict_modelo_nf):
        obj_adm_utils = self.pool.get('adm.utils')
        dreg53 = []
        models_no_53 = ['2D', '90', '91', '92', '93', '94', '95', '96', '97', '98', '99']
        for i in dados_registro_53:
            modelo = i.get('sint_reg50_id_modelo')

            if dict_modelo_nf.get(modelo) not in models_no_53:
                tipo_01 = '53'
                cpf_cnpj_02 = i.get('sint_reg50_cpf_cnpj')
                ie_03 = obj_adm_utils.admutl_remove_caracter(i.get('sint_reg50_ie'), letras=True,
                                                             numeros=True, espaco=False)
                data_emissao_04 = i.get('sint_reg50_data_emissao')
                uf_05 = i.get('sint_reg50_uf_sigla')
                modelo_06 = dict_modelo_nf.get(modelo)
                serie_07 = i.get('sint_reg50_serie')
                numero_08 = i.get('sint_reg50_numero')
                cfop_09 = i.get('sint_reg50_cfop')
                emitente_10 = i.get('sint_reg50_emitente')
                base_calculo_icms_subs_11 = i.get('sint_reg50_vr_base_icms_substituicao') or 0
                valor_icms_subs_retido_12 = Decimal(
                    str((i.get('sint_reg50_vr_icms_substituicao') or 0) * 1e2))
                valor_icms_subs_retido_12 = valor_icms_subs_retido_12.quantize(
                    Decimal('.01'), rounding=ROUND_UP)
                valor_despesas_acessorias_13 = i.get('sint_reg50_vr_despesas_acessorias') or 0
                situacao_14 = i.get('sint_reg50_situacao')
                codigo_antecipacao_15 = ' '
                brancos = '                             '

                if cpf_cnpj_02:
                    cpf_cnpj = int(re.sub('[^0-9]', '', cpf_cnpj_02))
                else:
                    cpf_cnpj = ''
                if modelo_06 == '1B':
                    modelo_06 = '01'

                retval = dict(tipo=tipo_01, cnpj=cpf_cnpj, estadual=ie_03, state=uf_05,
                              date=int(data_emissao_04), modelo=int(modelo_06), serial=serie_07,
                              numero=numero_08, cfop=int(cfop_09), emitente=emitente_10,
                              icms_base=int(base_calculo_icms_subs_11 * 1e2),
                              icms_retido=valor_icms_subs_retido_12,
                              despesas=int(valor_despesas_acessorias_13 * 1e2),
                              situacao=situacao_14, cod_antecipacao=codigo_antecipacao_15,
                              blank=brancos)
                dreg53.append(retval)

        return dreg53

    @staticmethod
    def _get_dados_registro55(cr, nf_ids):
        if not nf_ids:
            return list()
        sql = """
            SELECT
                ap.cpf_cnpj                                   AS cnpj,
                ap.admpes_jur_ie                              AS estadual,
                fg.data_pagamento                             AS date,
                ap.uf                                         AS state_dest,
                ab.admbco_codigo_bc                           AS banco,
                aba.admbag_codigo_agencia                     AS agencia,
                fg.autenticacao                               AS numero,
                fg.valor                                      AS valor,
                fg.data_vencimento                            AS vencimento,
                lpad(fg.mes, 2, '0') || lpad(fg.ano, 4, '0')  AS mes_ano,
                fg.convenio                                   AS convenio
            FROM
                fiscal_guia       fg
            LEFT JOIN
                fiscal_nf_capa    nf
                    ON nf.id = fg.nf_id
            LEFT JOIN
                adm_pessoa        ap
                    ON ap.id = nf.destinatario_id
            LEFT JOIN
                adm_banco_agencia aba
                    ON aba.id = fg.agencia_id
            LEFT JOIN
                adm_banco         ab
                    ON ab.id = aba.admbag_banco_id
            WHERE
                fg.nf_id IN (%s)
        """ % ', '.join(map(lambda i: str(i), nf_ids))
        cr.execute(sql)
        dreg55 = cr.dictfetchall()
        return dreg55

    def _get_dados_registro60(self, cr, uid, company_id, data_ini, data_fim, gerar60i=False,
                              context=None):
        context = context and context.copy() or {}
        data = {}
        dreg60m = []  # Dados do registro 60M
        dreg60a = []  # Dados do registro 60A
        dreg60d = []
        dreg60i = []
        datas_mapa_resumo = []
        mensagem = ''

        modulo_ecf_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'ecf', context={})

        if modulo_ecf_instalado:
            obj_ecf_mapa_resumo_capa = self.pool.get('ecf.mapa.resumo.capa')
            ecf_ids = self.pool.get('ecf.ecf').search(cr, uid, [('company_id', '=', company_id)])
            obj_ecf_mapa_resumo_capa_ids = obj_ecf_mapa_resumo_capa.search(
                cr, uid, [('data', '>=', data_ini.strftime('%Y-%m-%d')),
                          ('data', '<=', data_fim.strftime('%Y-%m-%d')),
                          ('ecf', 'in', ecf_ids)], context=context)
            data_mapa_resumo_capa = obj_ecf_mapa_resumo_capa.browse(
                cr, uid, obj_ecf_mapa_resumo_capa_ids, context=context)

            for odata in data_mapa_resumo_capa:
                dreg60m.append(dict(
                    id=odata.id, ecf_id=odata.ecf.id, printerserial=odata.ecf.nro_serie,
                    date=datetime.strptime(odata.data, '%Y-%m-%d').date(),
                    printerid=odata.ecf.caixa, coupon_start=odata.cupom_inicial,
                    coupon_end=odata.cupom_final, crz=odata.numero_z, cro=odata.cro,
                    period_total=round(odata.gt_final - odata.gt_inicial, 2), total=odata.gt_final))
                datas_mapa_resumo.append(odata.data)

            for odata in map(lambda itens: itens.item_ids, data_mapa_resumo_capa):
                reducaoz_sem_imposto = False
                lercapa = True  # Ler a capa somente 1 vez

                for i in odata:
                    data60 = datetime.strptime(i.capa_id.data, '%Y-%m-%d')
                    printerserial = i.capa_id.ecf.nro_serie

                    if lercapa:
                        dreg60a.append(dict(capa_id=i.capa_id.id, data=data60,
                                            printerserial=printerserial, tax='CANC',
                                            total=i.capa_id.cancelamentos or 0))

                        dreg60a.append(dict(capa_id=i.capa_id.id, data=data60,
                                            printerserial=printerserial, tax='DESC',
                                            total=i.capa_id.descontos or 0))

                    st = str(i.tipo)

                    if st == 'O':
                        st = 'N'

                    tributacao_ecf = int(i.aliquota * 1e2)
                    base_valor = 0.0

                    if i.imposto == 'C':  # ICMS
                        if st == 'T':
                            st = '%04d' % tributacao_ecf
                        else:
                            st = st.ljust(4)
                        base_valor = i.base_valor or 0.0

                    if str(i.imposto) == 'S':  # ISS
                        st = 'ISS '

                    if i.imposto not in ('C', 'S'):
                        reducaoz_sem_imposto = True

                    dreg60a.append(dict(capa_id=i.capa_id.id, data=data60,
                                        printerserial=printerserial, tax=st,
                                        total=base_valor))

                    lercapa = False

                if reducaoz_sem_imposto:
                    data_z = parse(i.capa_id.data).strftime('%d/%m/%Y')
                    mensagem += _('Reducao Z do dia %s, ECF %s, possui registro de tributacao sem '
                                  'imposto informado.\n' % (data_z, i.capa_id.ecf.caixa))

            sql_60d = """
                -- noinspection SqlDialectInspection
                SELECT reg_60d.sint_reg60d_data_cupom
                     , reg_60d.sint_reg60d_numero_serie
                     , reg_60d.sint_reg60d_item
                     , LPAD(RIGHT(CAST(produto.codigo          AS VARCHAR), 11) || '_' ||
                       TRIM(grade.grade), 14, '0')             AS produto_codigo
                     , grade.id                                AS grade_id
                     , SUM(reg_60d.sint_reg60d_quantidade)     AS sint_reg60d_quantidade
                     , SUM(reg_60d.sint_reg60d_valor_total)    AS sint_reg60d_valor_total
                     , reg_60d.sint_reg60d_base_icms
                     , reg_60d.sint_reg60d_tributacao_ecf
                     , reg_60d.sint_reg60d_tributacao_retag
                     , reg_60d.sint_reg60d_aliquota
                     , SUM(reg_60d.sint_reg60d_valor_icms)     AS sint_reg60d_valor_icms
                     , reg_60d.sint_reg60d_blank
                     , reg_60d.sint_reg60d_ecf_id
                FROM   sintegra_registro_60d_view              AS reg_60d
                LEFT   JOIN automacao_codigo_barra barra
                       ON barra.codigo_barra = reg_60d.sint_reg60d_item  
                 AND   barra.company_id = %s
                INNER  JOIN estoque_produto produto
                       ON produto.id = barra.produto_id
                LEFT   JOIN estoque_produto_grade grade
                       ON grade.id = barra.grade_id
                WHERE  reg_60d.sint_reg60d_data_cupom BETWEEN '%s' AND '%s'
                  AND  reg_60d.sint_reg60d_company_id = %s
                GROUP  BY reg_60d.sint_reg60d_data_cupom, reg_60d.sint_reg60d_numero_serie,
                          reg_60d.sint_reg60d_item, produto_codigo, grade.id,
                          reg_60d.sint_reg60d_base_icms, reg_60d.sint_reg60d_tributacao_ecf,
                          reg_60d.sint_reg60d_tributacao_retag, reg_60d.sint_reg60d_aliquota,
                          reg_60d.sint_reg60d_blank, reg_60d.sint_reg60d_ecf_id
            """ % (company_id, data_ini.strftime('%Y%m%d'), data_fim.strftime('%Y%m%d'), company_id)
            cr.execute(sql_60d)
            registro60d_dados = cr.dictfetchall()

            for i in registro60d_dados:
                if i.get('sint_reg60d_quantidade'):
                    data_cupom = i.get('sint_reg60d_data_cupom') or ''
                    numero_serie = i.get('sint_reg60d_numero_serie') or ''

                    # Lista com id dos produtos p/ geracao do registro 75
                    if not i.get('grade_id') in self._grade_ids:
                        self._grade_ids.append(i.get('grade_id'))

                    item = i.get('produto_codigo') or ''
                    quantidade = i.get('sint_reg60d_quantidade') or 0
                    valor_total = i.get('sint_reg60d_valor_total') or 0
                    base_icms = i.get('sint_reg60d_base_icms') or 0
                    tributacao_ecf = i.get('sint_reg60d_tributacao_ecf') or ''
                    tributacao_retag = i.get('sint_reg60d_tributacao_retag')
                    aliquota = i.get('sint_reg60d_aliquota') or 0
                    valor_icms = i.get('sint_reg60d_valor_icms') or 0
                    blank = i.get('sint_reg60d_blank')
                    ecf_id = i.get('sint_reg60d_ecf_id')
                    dreg60d.append(dict(
                        ecf_id=ecf_id, data_cupom=data_cupom, numero_serie=numero_serie, item=item,
                        quantidade=quantidade, valor_total=valor_total, base_icms=base_icms,
                        tributacao_ecf=tributacao_ecf, tributacao_retag=tributacao_retag,
                        aliquota=aliquota, valor_icms=valor_icms, blank=blank))

            if gerar60i:
                sql_60i = """
                -- noinspection SqlDialectInspection
                SELECT reg_60i.sint_reg60i_data_cupom
                     , LPAD(RIGHT(CAST(produto.codigo AS VARCHAR), 11) || '_' ||
                       TRIM(grade.grade), 14, '0')    AS produto_codigo
                     , grade.id                       AS grade_id
                     , reg_60i.sint_reg60i_numero_serie
                     , reg_60i.sint_reg60i_modelo
                     , reg_60i.sint_reg60i_coo
                     , reg_60i.sint_reg60i_sequencia
                     , reg_60i.sint_reg60i_item
                     , reg_60i.sint_reg60i_quantidade
                     , reg_60i.sint_reg60i_valor_total
                     , reg_60i.sint_reg60i_base
                     , reg_60i.sint_reg60i_tributacao_retag
                     , reg_60i.sint_reg60i_aliquota
                     , reg_60i.sint_reg60i_valor_icms
                     , reg_60i.sint_reg60i_blank
                     , reg_60i.sint_reg60i_ecf_id
                FROM   sintegra_registro_60i_view     AS reg_60i
                LEFT   JOIN automacao_codigo_barra barra
                       ON barra.codigo_barra = reg_60i.sint_reg60i_item
                INNER  JOIN estoque_produto produto
                       ON produto.id = barra.produto_id
                LEFT   JOIN estoque_produto_grade grade on grade.id = barra.grade_id
                WHERE  reg_60i.sint_reg60i_data_cupom BETWEEN '%s' AND  '%s'
                  AND  reg_60i.sint_reg60i_company_id = %s
                """ % (data_ini.strftime('%Y%m%d'), data_fim.strftime('%Y%m%d'), company_id)
                cr.execute(sql_60i)
                registro60i_dados = cr.dictfetchall()

                for i in registro60i_dados:
                    data_cupom = i.get('sint_reg60i_data_cupom') or ''
                    numero_serie = i.get('sint_reg60i_numero_serie') or ''
                    modelo = i.get('sint_reg60i_modelo') or ''
                    coo = i.get('sint_reg60i_coo') or 0
                    sequencia = i.get('sint_reg60i_sequencia') or 0
                    item = i.get('produto_codigo') or ''

                    if not i.get('grade_id') in self._grade_ids:
                        self._grade_ids.append(i.get('grade_id'))

                    quantidade = i.get('sint_reg60i_quantidade') or ''
                    valor_total = i.get('sint_reg60i_valor_total') or 0
                    base_icms = i.get('sint_reg60i_base') or 0
                    tributacao_ecf = i.get('sint_reg60i_tributacao_ecf') or ''
                    tributacao_retag = i.get('sint_reg60i_tributacao_retag') or ''
                    aliquota = i.get('sint_reg60i_aliquota') or 0
                    valor_icms = i.get('sint_reg60i_valor_icms') or 0
                    blank = i.get('sint_reg60i_blank') or ''
                    ecf_id = i.get('sint_reg60i_ecf_id')
                    dreg60i.append(dict(
                        ecf_id=ecf_id, data_cupom=data_cupom, numero_serie=numero_serie, coo=coo,
                        modelo=modelo, sequencia=sequencia, item=item, quantidade=quantidade,
                        valor_total=valor_total, base_icms=base_icms, tributacao_ecf=tributacao_ecf,
                        tributacao_retag=tributacao_retag, aliquota=aliquota, valor_icms=valor_icms,
                        blank=blank))

        data.update({
            'reg60m': dreg60m,
            'reg60a': dreg60a,
            'reg60d': dreg60d,
            'reg60i': dreg60i,
            'datas_mapa_resumo': datas_mapa_resumo,
            'mensagem': mensagem
        })
        return data

    def _get_dados_registro61(self, cr, uid, sintegra_id, company_id, data_ini, data_fim,
                              simples_nacional, context=None):
        """
        REGISTRO TIPO 61: Para os documentos fiscais descritos a seguir, quando nao emitidos por
        equipamento emissor de cupom fiscal: bilhete de passagem aquaviario (modelo 14), bilhete
        de passagem e nota de bagagem (modelo 15), bilhete de passagem Ferroviario (modelo 16),
        bilhete de passagem rodoviario (modelo 13) e nota fiscal de venda a consumidor (modelo 2),
        nota fiscal de produtor (modelo 4), para as Unidades da Federacao que nao o exigirem na
        forma prevista no item 11."""

        context = context and context.copy() or {}
        obj_fiscal_modelo_nf = self.pool.get('fiscal.modelo.nf')
        sintegra_registro50 = self.pool.get('sintegra.registro50')
        dreg61 = []
        fiscal_modelo_nf_ids = obj_fiscal_modelo_nf.search(
            cr, uid, [('fismnf_codigo', 'in', ['02', '04' '13', '14', '15', '16', '65'])],
            context=context)
        args_reg50 = [
            ('fisgersin_sintegra_id', '=', sintegra_id),
            # ('sint_reg50_data_emissao', '>=', data_ini.strftime('%Y%m%d')),
            # ('sint_reg50_data_emissao', '<=', data_fim.strftime('%Y%m%d')),
            ('sint_reg50_id_company', '=', company_id)]
        reg50_ids = sintegra_registro50.search(cr, uid, args_reg50, context=context)
        brw_sintegra_registro_50 = sintegra_registro50.browse(cr, uid, reg50_ids, context=context)
        nf_ids = [x.nf_id for x in brw_sintegra_registro_50]
        nf_ids = list(set(nf_ids))

        if not nf_ids:
            return dreg61, nf_ids, fiscal_modelo_nf_ids

        modelo_nf_dados = obj_fiscal_modelo_nf.read(
            cr, uid, fiscal_modelo_nf_ids, ['fismnf_codigo'], context=context)
        dict_modelo_nf = dict(map(lambda m: (m['id'], m['fismnf_codigo']), modelo_nf_dados))

        if simples_nacional:
            vr_base_calculo = """, 0.00 AS valor_base_calculo"""
            vr_outros = """
                , SUM(CASE WHEN fiscst.tipo IN ('T', 'O', 'U') AND fisnfc.state != 'I' 
                                THEN fisnit.vr_base_calculo
                           ELSE 0.00 END)                           AS valor_outras
            """
        else:
            vr_base_calculo = """
                , SUM(CASE 
                       WHEN fisnfc.state = 'I' THEN 0
                       ELSE fisnit.vr_base_calculo
                      END)                                     AS valor_base_calculo
            """
            vr_outros = """
                , SUM(CASE WHEN fiscst.tipo = 'O' AND fisnfc.state != 'I' 
                                THEN fisnit.vr_base_calculo
                           ELSE 0.00 END)                           AS valor_outras
            """

        sql = """
                -- noinspection SqlDialectInspection
                SELECT MIN(fisnfc.numero_nf)                        AS ordem_inicial
                     , MAX(fisnfc.numero_nf)                        AS ordem_final
                     , TO_CHAR (fisnfc.data_emissao, 'YYYYMMDD')    AS data_emissao
                     , fisnfc.serie_nf
                     , fisnfc.modelo_nf_id
                     , SUM(CASE 
                            WHEN fisnfc.state = 'I' THEN 0
                            ELSE fisnfc.vr_total_nf
                           END)                                     AS vr_total_nf
                     {vr_base_calculo}
                     , SUM(CASE WHEN fiscst.tipo = 'T' AND fisnfc.state != 'I' 
                                THEN fisnit.vr_encargo
                           ELSE 0.00
                           END)                                     AS vr_icms
                     , SUM(CASE WHEN fiscst.tipo = 'I' AND fisnfc.state != 'I' 
                                THEN fisnit.vr_encargo
                           ELSE 0.00 END)                           AS isento_nao_tributadas
                     {vr_outros}
                     , fisnit.aliquota                              AS aliquota
                     , SUM(CASE WHEN fisnfc.state = 'I' THEN 0
                                ELSE COALESCE(fisnit.vr_base_calculo, 0) + 
                                     COALESCE(fisnit.vr_reducao_base, 0)
                           END)                                     AS vr_total_item
                FROM   fiscal_nf_capa                               AS fisnfc
                LEFT   JOIN fiscal_nf_item_tributacao               AS fisnit
                       ON fisnit.fisnit_capa_id = fisnfc.id
                LEFT   JOIN fiscal_cst                              AS fiscst
                       ON fiscst.id = fisnit.cst_id
                LEFT   JOIN fiscal_encargo                          AS fisenc
                       ON fisenc.id = fisnit.encargo_id
                WHERE  fisenc.sigla = 'ICMS'
                  AND  fisnfc.id IN {nf_ids}
                  AND  fisnfc.modelo_nf_id in {modelo_nf_ids}
                GROUP  BY fisnfc.modelo_nf_id, fisnfc.data_emissao, fisnfc.serie_nf,
                          fisnit.aliquota, fisnfc.vr_total_nf
                ORDER  BY fisnfc.data_emissao
                """.format(nf_ids=(str(nf_ids).replace('[', '(').replace(']', ')')),
                           modelo_nf_ids=(str(fiscal_modelo_nf_ids).replace(
                               '[', '('). replace(']', ')')),
                           vr_base_calculo=vr_base_calculo,
                           vr_outros=vr_outros)

        cr.execute(sql)
        dados_reg_61 = cr.dictfetchall()

        # Filtra os modelos fiscais
        dados_reg_61 = filter(lambda f: obj_fiscal_modelo_nf.verifica_codigo_modelo_fiscal(
            cr, uid, f.get('modelo_nf_id')), dados_reg_61)

        for i in dados_reg_61:
            ordem_inicial = str(i.get('ordem_inicial'))

            if len(ordem_inicial) > 6:
                i['ordem_inicial'] = int(ordem_inicial[3:])

            ordem_final = str(i.get('ordem_final'))

            if len(ordem_final) > 6:
                i['ordem_final'] = int(ordem_final[3:])

            modelo = dict_modelo_nf.get(i.get('modelo_nf_id'))

            if modelo == '02':
                serie = 'D'
            else:
                serie = i.get('serie_nf')

            data_emissao = i.get('data_emissao')
            serie_nf = serie
            modelo_nf = modelo
            ordem_inicial = i.get('ordem_inicial')
            ordem_final = i.get('ordem_final')
            base_icms = i.get('valor_base_calculo')
            vr_icms = i.get('vr_icms')
            isento = i.get('isento_nao_tributadas')
            outras = i.get('valor_outras')
            aliquota = i.get('aliquota')
            valor_total = i.get('vr_total_item') or 0
            total_item_nf = Decimal(str(valor_total * 1e2)).quantize(
                Decimal('.01'), rounding=ROUND_UP)
            retval = dict(
                blank1=' ' * 28, data_emissao=int(data_emissao), modelo=int(modelo_nf),
                serie=serie_nf, subserie='  ', inicial_ordem=ordem_inicial, final_ordem=ordem_final,
                valor_total=int(total_item_nf), icms_base=int(base_icms * 1e2),
                icms_valor=int(vr_icms * 1e2), isento=int(isento * 1e2), outras=int(outras * 1e2),
                aliquota_icms=int(aliquota * 1e2), blank=' ')
            dreg61.append(retval)

        return dreg61, nf_ids, fiscal_modelo_nf_ids

    def _get_dados_registro61R(self, cr, uid, nf_ids, modelo_nf_ids, context=None):
        """
        Resumo Mensal por Item : Registro de mercadoria/produto ou serviço comercializados
        através de nota fiscal de venda a consumidor não emitida por ECF.
        """

        context = context and context.copy() or {}
        obj_fiscal_modelo_nf = self.pool.get('fiscal.modelo.nf')
        dreg61R = []
        if not nf_ids:
            return dreg61R
        sql = """
            SELECT 
                mes_ano_emissao, 
                codigo_produto, 
                grade_id, 
                sum(quantidade) AS quantidade, 
                sum(valor_bruto) AS valor_bruto, 
                sum(base_calculo_icms) AS base_calculo_icms, 
                aliquota_icms 
            FROM (
                SELECT 
                     TO_CHAR (fisnfc.data_emissao, 'MMYYYY')        AS mes_ano_emissao
                     , LPAD(RIGHT(replace(produto.codigo, ' ', ''), 11) || '_' || 
                       TRIM(grade.grade), 14, '0')                  AS codigo_produto
                     , grade.id                                     AS grade_id
                     , SUM(CASE 
                            WHEN fisnfc.state = 'I' THEN 0
                            ELSE movimento.quantidade
                           END)                                     AS quantidade
                     , SUM(CASE 
                            WHEN fisnfc.state = 'I' THEN 0
                            ELSE movimento.vr_total_item
                           END)                                     AS valor_bruto
                     , SUM(CASE 
                            WHEN fisnfc.state = 'I' THEN 0
                            ELSE fisnit.vr_base_calculo
                           END)                                     AS base_calculo_icms
                     , fisnit.aliquota                              AS aliquota_icms
                     
                FROM   fiscal_nf_capa                               AS fisnfc
                LEFT   JOIN estoque_movimento_itens                 AS movimento
                       ON fisnfc.id = movimento.nf_id
                LEFT   JOIN estoque_produto                         AS produto
                       ON produto.id = movimento.produto_id
                LEFT   JOIN estoque_produto_grade                   AS grade
                       ON grade.id = movimento.grade_id     
                LEFT   JOIN fiscal_nf_item_tributacao               AS fisnit
                       ON fisnit.fisnit_capa_id = fisnfc.id
                LEFT   JOIN fiscal_cst                              AS fiscst
                       ON fiscst.id = fisnit.cst_id
                LEFT   JOIN fiscal_encargo                          AS fisenc
                       ON fisenc.id = fisnit.encargo_id
                WHERE  fisenc.sigla = 'ICMS'
                  AND  fisnfc.state != 'I'
                  AND  fisnfc.id IN %s
                  AND  fisnfc.modelo_nf_id in %s
                GROUP  BY fisnfc.data_emissao, codigo_produto, grade.id, fisnit.aliquota
            ) AS tmp
            GROUP  BY mes_ano_emissao, codigo_produto, grade_id, aliquota_icms
            ORDER  BY mes_ano_emissao
        """ % ((str(nf_ids).replace('[', '(').replace(']', ')')),
               (str(modelo_nf_ids).replace('[', '('). replace(']', ')')))

        cr.execute(sql)
        dados_reg_61R = cr.dictfetchall()

        for i in dados_reg_61R:
            if i.get('grade_id') not in self._grade_ids:
                self._grade_ids.append(i.get('grade_id'))
            quantidade = Decimal(str((i.get('quantidade') or 0) * 1e3)).quantize(
                Decimal('.01'), rounding=ROUND_UP)
            valor_bruto = Decimal(str((i.get('valor_bruto') or 0) * 1e2)).quantize(
                Decimal('.01'), rounding=ROUND_UP)
            retval = {
                'branco': ' ' * 54, 'mes_ano_emissao': int(i.get('mes_ano_emissao')),
                'codigo_produto': i.get('codigo_produto'), 'quantidade': int(quantidade),
                'valor_bruto': int(valor_bruto),
                'base_calculo_icms': int((i.get('base_calculo_icms') or 0) * 1e2),
                'aliquota_icms': int((i.get('aliquota_icms') or 0) * 1e2)}
            dreg61R.append(retval)

        return dreg61R

    def _get_dados_registro70(self, cr, uid, dados_registro_70, dict_modelo_nf):
        dreg70 = []
        obj_fis_nf_item_trib = self.pool.get('fiscal.nf.item.tributacao')
        for i in dados_registro_70:
            base_calculo = 0.00
            valor_icms = 0.00
            isento_nao_tribut = 0.00
            outras = 0.00
            nf_id = i.get('nf_id')
            fisnit_ids = obj_fis_nf_item_trib.search(cr, uid, [('fisnit_capa_id', '=', nf_id)],
                                                     context=None)
            brw_fisnit = obj_fis_nf_item_trib.browse(cr, uid, fisnit_ids, context=None)
            encargo_dados = filter(lambda x: x.encargo_id.sigla == 'ICMS', brw_fisnit)
            for x in encargo_dados:
                base_calculo = float(float_round(x.vr_base_calculo or 0.00, 2))
                valor_icms = float(float_round(x.vr_encargo or 0.00, 2))
                isento_nao_tribut = x.vr_base_calculo if x.cst_id.tipo == 'I' else 0.00
                outras = x.vr_base_calculo if x.cst_id.tipo == 'O' else 0.00
            num_nf = str(i.get('sint_reg50_numero'))

            if len(num_nf) > 6:
                i['sint_reg50_numero'] = int(num_nf[-6:])

            modelo_nota = i.get('sint_reg50_id_modelo')
            cpf_cnpj_02 = i.get('sint_reg50_cpf_cnpj')
            ie_03 = i.get('sint_reg50_ie')
            data_emissao_04 = i.get('sint_reg50_data_emissao')
            uf_05 = i.get('sint_reg50_uf_sigla') or ''
            modelo_06 = dict_modelo_nf.get(modelo_nota)
            serie_07 = i.get('sint_reg50_serie')
            numero_08 = i.get('sint_reg50_numero')
            cfop_09 = i.get('sint_reg50_cfop')
            valor_total_11 = i.get('sint_reg50_valor_total_nf')
            base_calculo_12 = base_calculo
            valor_icms_13 = valor_icms
            isenta_nao_tributada_14 = isento_nao_tribut
            outras_15 = outras
            situacao_17 = i.get('sint_reg50_situacao')

            if cpf_cnpj_02:
                cnpj = int(re.sub('[^0-9]', '', cpf_cnpj_02))
            else:
                cnpj = ''

            retval = dict(cnpj=cnpj, estadual=ie_03, date=int(data_emissao_04), state=uf_05,
                          modelo=int(modelo_06), serie=serie_07, subserie='', numero=numero_08,
                          cfop=int(cfop_09), valor=int(float_round(valor_total_11, 2) * 1e2),
                          icms_base=int(round(float_round(base_calculo_12, 2) * 1e2)),
                          icms=int(float_round(valor_icms_13, 2) * 1e2),
                          isento=int(float_round(isenta_nao_tributada_14, 2) * 1e2),
                          outras=int(float_round(outras_15, 2) * 1e2), modalidade=0,
                          situacao=situacao_17)
            dreg70.append(retval)

        return dreg70

    def button_gerar(self, cr, uid, ids, context=None):
        self._logger.info('Gerando Sintegra...')
        _tInicio = datetime.now()
        global TOTAL_SAIDAS, TOTAL_ENTRADAS, TOTAL_CTES
        TOTAL_SAIDAS = TOTAL_ENTRADAS = TOTAL_CTES = 0.0

        context = context and context.copy() or {}

        if isinstance(ids, list):
            sintegra_id = ids[0]
        else:
            sintegra_id = ids

        self._grade_ids = []
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_adm_utils = self.pool.get('adm.utils')
        fiscal_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'fiscal', 'fispar_base_reduzida', context=context)
        fispar_base_reduzida = fiscal_parametro_dados.get('fispar_base_reduzida')
        simples_nacional = obj_adm_pessoa.verificar_empresa_simples_nacional(cr, uid)

        # Dados da tela
        self_data = self.read(cr, uid, ids, [], context=context)[0]
        mensagem = ''
        data_msg = []

        # Sintegra Class
        s = SintegraFile()

        # Empresa
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        # Data Inicio/Fim datetime format
        data_ini = datetime.strptime(self_data.get('data_ini') + ' 00:00:00', '%Y-%m-%d %H:%M:%S')
        data_utc_ini = self.pool.get('adm.utils').admutl_converte_tz(cr, uid, data_ini, 'America/Sao_Paulo', 'UTC', context=None)
        data_utc_ini = data_utc_ini.strftime("%Y-%m-%d %H:%M:%S")
        data_fim = datetime.strptime(self_data.get('data_fim') + ' 23:59:59', '%Y-%m-%d %H:%M:%S')
        data_utc_fim = self.pool.get('adm.utils').admutl_converte_tz(cr, uid, data_fim, 'America/Sao_Paulo', 'UTC', context=None)
        data_utc_fim = data_utc_fim.strftime("%Y-%m-%d %H:%M:%S")
        gerar_registro60i = self_data.get('gerar_registro60i')

        # Registro 10
        data_reg10 = self._get_dados_registro10(cr, uid, company_id)

        s.add(
            SintegraRegister10(
                data_reg10.get('cgc'), data_reg10.get('ie'), data_reg10.get('nome'),
                data_reg10.get('municipio'), data_reg10.get('uf'), data_reg10.get('fax'),
                int(data_ini.strftime('%Y%m%d')), int(data_fim.strftime('%Y%m%d')),
                '{0}{1}{2}'.format(self_data.get('cod_ident_con'), self_data.get('cod_ident_nat'),
                                   self_data.get('cod_ident_fin'))))

        # Registro 11
        logradouro = obj_adm_utils.admutl_remover_acentuacao(cr, uid, data_reg10.get('logradouro'))
        bairro = data_reg10.get('bairro').strip()
        s.add(SintegraRegister11(
            logradouro, int(re_sub('[^0-9]', '', '%s' % data_reg10.get('numero')) or 0),
            data_reg10.get('complemento'), bairro, int(data_reg10.get('cep')),
            data_reg10.get('nome_socio'), int(data_reg10.get('fone'))))

        # Registro 50
        obj_reg50 = self.pool.get('sintegra.registro50')
        obj_reg50.gera_registro_50(cr, uid, sintegra_id, company_id, data_utc_ini, data_utc_fim,
                                   simples_nacional, fispar_base_reduzida)

        # Atencao! reg50 contem os registros 50/53/54 e 70
        reg50 = self._get_dados_registro50(cr, uid, sintegra_id, company_id, data_utc_ini, data_utc_fim,
                                           data_ini, data_fim)

        if reg50.get('mensagem_50'):
            return self.write(cr, uid, ids, {
                'informacao': 'Nao foi possivel gerar o arquivo sintegra. \n' +
                              reg50['mensagem_50']}, context)

        data_reg50 = reg50.get('reg50')

        for d in data_reg50:
            s.add(SintegraRegister50(
                d.get('cnpj'), d.get('estadual').strip(), d.get('date'), d.get('state') or '',
                d.get('modelo'), d.get('serial'), d.get('numero'), d.get('cfop'), d.get('emitente'),
                d.get('total'), d.get('icms_base'), d.get('icms_total'), d.get('isenta'),
                d.get('outras'), d.get('aliquota_icms'), d.get('situacao')))
            if str(d.get('cfop'))[0] < '5':
                TOTAL_ENTRADAS += d.get('total')
            else:
                TOTAL_SAIDAS += d.get('total')
            if not d.get('cnpj') and str(d.get('cfop'))[0] not in ('3', '7'):
                mensagem += _('* Atencao : O destinatario da nota numero %s nao possui '
                              'cpf/cnpj cadastrado \n' % str(d.get('numero')))

        # Registro 53
        data_reg53 = reg50.get('reg53')
        for d in data_reg53:
            s.add(SintegraRegister53(
                d.get('cnpj') or 0, d.get('estadual').strip(), d.get('date'), d.get('state') or '',
                d.get('modelo'), d.get('serial'), d.get('numero'), d.get('cfop'), d.get('emitente'),
                d.get('icms_base'), d.get('icms_retido'), d.get('despesas'), d.get('situacao'),
                d.get('cod_antecipacao'), d.get('blank')))

        # Registro 54
        data_reg54 = reg50.get('reg54')
        for d in data_reg54:
            s.add(SintegraRegister54(
                d.get('cnpj') or 0, d.get('modelo'), d.get('serial'), d.get('numero'),
                d.get('cfop'), str(d.get('cst')), d.get('sequencia'), str(d.get('codigo')),
                d.get('quantidade'), d.get('valor'), d.get('vr_desconto'), d.get('base_calculo'),
                d.get('base_sub'), d.get('ipi'), d.get('aliquota')))

        brw_res_company = self.pool.get('res.company').browse(cr, uid, company_id, context)
        estado = brw_res_company.rescon_pessoa_id.uf
        cnpj = obj_adm_utils.admutl_remove_caracter(
            brw_res_company.rescon_pessoa_id.cpf_cnpj, letras=False, espaco=False)
        data_reg55 = reg50.get('reg55')
        for d in data_reg55:
            data_pagamento = parse(d.get('date')).strftime('%Y%m%d') if d.get('date') else 0
            cnpj = int(cnpj) if cnpj else 0
            data_vencimento = parse(d.get('vencimento')).strftime('%Y%m%d') if d.get('vencimento') \
                else 0
            s.add(SintegraRegister55(cnpj, d.get('estadual') or '',
                                     int(data_pagamento), estado or '',
                                     d.get('state_dest') or '', d.get('banco') or 0,
                                     d.get('agencia') or 0, d.get('numero') or '',
                                     int(d.get('valor') * 1e2), int(data_vencimento),
                                     int(d.get('mes_ano')), d.get('convenio') or ''))

        # Registro 60
        def get_tributacao_ecf(tributacao_ecf):
            situacao = 'N'

            if tributacao_ecf:
                obj_fiscal_cst = self.pool.get('fiscal.cst')
                obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
                tributacao_ids = obj_fiscal_tributacao.search(
                    cr, uid, [('tributacao_ecf', '=', tributacao_ecf)], context=context)

                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    tributacao_dados = obj_fiscal_tributacao.read(
                        cr, uid, tributacao_id, ['cst_id'], context=context, load=READ_LOAD)

                    if isinstance(tributacao_dados, list):
                        tributacao_dados = tributacao_dados[0]
                    cst_id = tributacao_dados.get('cst_id')

                    if cst_id:
                        cst_dados = obj_fiscal_cst.read(cr, uid, cst_id, ['tipo'], context=context)

                        if isinstance(cst_dados, list):
                            cst_dados = cst_dados[0]
                        cst_tipo = cst_dados.get('tipo')

                        if cst_tipo == 'T':  # Tributado
                            situacao = str(aliquota).rjust(4, '0')
                        elif cst_tipo == 'U':  # Substituicao
                            situacao = 'F'
                        elif cst_tipo == 'I':  # Isento
                            situacao = 'I'
                        elif cst_tipo == 'S':  # Servico
                            situacao = 'ISS'
                        elif cst_tipo == 'O':
                            situacao = 'N'

            return situacao

        data_reg60 = self._get_dados_registro60(
            cr, uid, company_id, data_ini, data_fim, gerar60i=gerar_registro60i)

        if 'mensagem' in data_reg60:
            mensagem += data_reg60.get('mensagem') or ''
            del data_reg60['mensagem']

        datas_mapa_resumo = data_reg60.get('datas_mapa_resumo')

        for dreg60m in data_reg60.get('reg60m'):
            data_mapa_resumo_capa = dreg60m.get('date')
            mapa_resumo_capa_id = dreg60m.get('id')
            ecf_id = dreg60m.get('ecf_id')
            s.add_fiscal_coupon(
                data_mapa_resumo_capa, dreg60m.get('printerserial'), dreg60m.get('printerid'),
                int(dreg60m.get('coupon_start')), int(dreg60m.get('coupon_end')),
                dreg60m.get('crz'), dreg60m.get('cro'),
                int((dreg60m.get('period_total') or 0) * 1e2),
                int((dreg60m.get('total') or 0) * 1e2))

            for dreg60a in filter(lambda a: a.get('capa_id') == mapa_resumo_capa_id,
                                  data_reg60.get('reg60a')):
                total = Decimal(str(dreg60a.get('total') * 1e2)).quantize(
                    Decimal('.01'), rounding=ROUND_UP)
                s.add(SintegraRegister60A(
                    'A', int(dreg60a.get('data').strftime('%Y%m%d')),
                    dreg60a.get('printerserial'),
                    dreg60a.get('tax'), int(total)))

            registro_60d = data_reg60.get('reg60d')

            for i in filter(lambda d: d.get('ecf_id') == ecf_id
                            and datetime.strptime(d.get('data_cupom'), '%Y%m%d').strftime(
                        '%Y-%m-%d') == str(data_mapa_resumo_capa), registro_60d):
                data = i.get('data_cupom')
                datad = datetime.strptime(data, '%Y%m%d')
                datad = datad.strftime('%Y-%m-%d')

                data = int(data)
                numero_serie = i.get('numero_serie')
                item = i.get('item')
                quantidade = int(i.get('quantidade') * 1e3)
                valor_total = int(i.get('valor_total') * 1e2)
                base_icms = int(i.get('base_icms') * 1e2)
                tributacao_retag = i.get('tributacao_retag')
                aliquota = int(i.get('aliquota') * 1e2)
                valor_icms = int(i.get('valor_icms') * 1e2)
                blank = i.get('blank')
                situacao_tributaria = get_tributacao_ecf(tributacao_retag)
                s.add(SintegraRegister60D(
                    'D', data, numero_serie, item, quantidade, valor_total,  base_icms,
                    situacao_tributaria, valor_icms, blank))

                if datad not in datas_mapa_resumo and datad not in data_msg:
                    mensagem += _('* Atencao : Nao existe reducao Z para o dia: ' +
                                  time.strftime("%d/%m/%Y", time.strptime(datad, "%Y-%m-%d")) +
                                  '  \n')
                    data_msg.append(datad)

            registro_60i = data_reg60.get('reg60i')

            for i in filter(lambda ii: ii.get('ecf_id') == ecf_id
                            and datetime.strptime(ii.get('data_cupom'), '%Y%m%d').strftime(
                        '%Y-%m-%d') == str(data_mapa_resumo_capa), registro_60i):
                data = i.get('data_cupom')

                data = int(data)
                numero_serie = i.get('numero_serie')
                modelo = i.get('modelo')
                coo = int(i.get('coo'))
                sequencia = int(i.get('sequencia'))
                item = i.get('item')
                quantidade = int(i.get('quantidade') * 1e3)
                valor_total = int(i.get('valor_total') * 1e2)
                base_icms = int(i.get('base_icms') * 1e2)
                tributacao_retag = i.get('tributacao_retag')
                aliquota = int(i.get('aliquota') * 1e2)
                valor_icms = int(i.get('valor_icms') * 1e2)
                blank = i.get('blank')
                situacao_tributaria = get_tributacao_ecf(tributacao_retag)
                s.add(SintegraRegister60I(
                    'I', data, numero_serie, modelo, coo, sequencia, item, quantidade,
                    valor_total, base_icms, situacao_tributaria, valor_icms, blank))

        # Registro 61 - Notas consumidor final
        data_reg_61, reg_61_nf_ids, reg_61_modelo_nf_ids = self._get_dados_registro61(
            cr, uid, sintegra_id, company_id, data_ini, data_fim, simples_nacional, context)

        for reg61 in data_reg_61:
            s.add(SintegraRegister61(
                reg61.get('branco'), reg61.get('data_emissao'), reg61.get('modelo'),
                reg61.get('serie'), reg61.get('subserie'), reg61.get('inicial_ordem'),
                reg61.get('final_ordem'), reg61.get('valor_total'), reg61.get('icms_base'),
                reg61.get('icms_valor'), reg61.get('isento'), reg61.get('outras'),
                reg61.get('aliquota_icms'), reg61.get('blank')))

        data_reg_61R = self._get_dados_registro61R(
            cr, uid, reg_61_nf_ids, reg_61_modelo_nf_ids, context)

        for reg61R in data_reg_61R:
            s.add(SintegraRegister61R(
                'R', reg61R.get('mes_ano_emissao'), reg61R.get('codigo_produto'),
                reg61R.get('quantidade'), reg61R.get('valor_bruto'),
                reg61R.get('base_calculo_icms'), reg61R.get('aliquota_icms'), reg61R.get('branco')))

        # Registro 70 - Informacoes de conhecimento de transportes
        data_reg70 = reg50.get('reg70')
        for d in data_reg70:
            s.add(SintegraRegister70(
                d.get('cnpj'), d.get('estadual').strip(), d.get('date'), d.get('state'), d.get('modelo'),
                d.get('serie'), d.get('subserie'), d.get('numero'), d.get('cfop'), d.get('valor'),
                d.get('icms_base'), d.get('icms'), d.get('isento'), d.get('outras'),
                d.get('modalidade'), d.get('situacao')))

        # Registro 74 - Inventario
        data_inventario_id = self_data.get('inventario_id')

        if data_inventario_id:
            inventario_id = data_inventario_id[0]
            estoque_parametros_dados = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'estoque', ['custo_inventario'], context=context)
            custo_inventario = estoque_parametros_dados.get('custo_inventario')
            _sql_registro_74 = """
                SELECT invcap.esticp_data_livro_inventario,
                       invofi.grade_id,
                       grade.grade,
                       produto.codigo,
                       Sum(saldo)                       AS saldo,
                       Sum(invofi.estiof_ultimo_custo)         AS ultimo_custo,
                       Sum(invofi.estiof_custo_medio_contabil) AS custo_medio_contabil
                FROM   estoque_inventario_capa invcap
                       INNER JOIN estoque_inventario_oficial invofi
                               ON invofi.capa_id = invcap.id
                       INNER JOIN estoque_produto_grade grade
                               ON grade.id = invofi.grade_id
                       INNER JOIN estoque_produto produto
                               ON produto.id = invofi.produto_id
                WHERE  invcap.id = {inventario_id}
                       AND saldo > 0
                GROUP  BY invcap.esticp_data_livro_inventario,
                          invofi.grade_id,
                          grade.grade,
                          produto.codigo
            """.format(inventario_id=inventario_id)
            cr.execute(_sql_registro_74)
            dados_inventario = cr.dictfetchall()
            for item_iventario in dados_inventario:
                if custo_inventario == 'U':
                    custo = item_iventario.get('ultimo_custo') or 0.0
                else:
                    custo = item_iventario.get('custo_medio_contabil') or 0.0
                grade_id = item_iventario.get('grade_id')
                if grade_id not in self._grade_ids:
                    self._grade_ids.append(grade_id)
                custo *= (item_iventario.get('saldo') or 0.0)
                custo = Decimal(str(custo * 1e2)).quantize(Decimal('.01'), rounding=ROUND_UP)
                saldo = Decimal(str(item_iventario.get('saldo') * 1e3)).quantize(
                    Decimal('.01'), rounding=ROUND_UP)
                produto_codigo = '{0:>s}_{1:>s}'.format(
                    item_iventario.get('codigo').replace(' ', '')[-11:],
                    item_iventario.get('grade').replace(' ', ''))
                s.add_inventory_item(
                    start=datetime.strptime(item_iventario.get('esticp_data_livro_inventario'),
                                            '%Y-%m-%d'),
                    product_code=produto_codigo,
                    product_quantity=int(saldo), total_product_value=int(custo), owner_code=1,
                    owner_cnpj=0, owner_state_registry='', state=data_reg10.get('uf'))

        def obter_menor_embalagem(produto):
            estoque_embalagens_d = dict(map(lambda m: (m.embalagem_id.quantidade_na_embalagem, m),
                                            produto.produto_embalagem_ids))
            if not estoque_embalagens_d:
                return estoque_embalagens_d
            return estoque_embalagens_d.get(min(estoque_embalagens_d))

        # Registro 75
        grade_ids = list(set(self._grade_ids))
        data_registro_75 = []
        produto_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, tabela='estoque.produto', context=context)
        if grade_ids:
            if None in grade_ids:
                grade_ids.remove(None)
            grade_ids = map(lambda x: '%s' % x, grade_ids)
            sql_registro_75 = """
                -- noinspection SqlDialectInspection
                SELECT DISTINCT estpro.id              AS produto_id
                     , estpro.codigo                   AS produto_codigo
                     , estpro.name                     AS nome_produto
                     , estprg.grade                    AS grade
                     , estprg.id                       AS grade_id
                     , fisncm.codigo                   AS ncm_codigo
                     , SUM(CASE WHEN fisenc.sigla = 'ICMS'
                                 AND fisten.tipo_tributacao = 'T'
                           THEN fistri.aliquota END)   AS aliquota_icms
                     , SUM(CASE WHEN fisenc.sigla = 'ICMS'
                                 AND fisten.tipo_tributacao = 'T'
                           THEN fistri.base END)       AS base_icms
                     , SUM(CASE WHEN fisenc.sigla = 'ICMS'
                                 AND fisten.tipo_tributacao = 'T'
                                 AND fiscst.tipo = 'T'
                           THEN 100 - fistri.base END) AS reducao_icms
                     , SUM(CASE WHEN fisenc.sigla = 'IPI'
                                 AND fisten.tipo_tributacao = 'T'
                           THEN fistri.aliquota END)   AS aliquota_ipi
                FROM   estoque_produto_grade           AS estprg
                LEFT   JOIN estoque_produto            AS estpro ON estpro.id = estprg.produto_id
                LEFT   JOIN fiscal_ncm                 AS fisncm ON fisncm.id = estpro.ncm_id
                LEFT   JOIN fiscal_tributacao_encargos AS fisten ON fisten.produto_id = estpro.id
                LEFT   JOIN fiscal_encargo             AS fisenc ON fisenc.id = fisten.encargo_id
                LEFT   JOIN fiscal_tributacao          AS fistri ON fistri.id = fisten.tributacao_id
                LEFT   JOIN fiscal_cst                 AS fiscst ON fiscst.id = fistri.cst_id
                LEFT   JOIN estoque_produto_embalagem  AS estpem ON estpem.produto_id = estpro.id
                WHERE  estpro.company_id = %s
                  AND  estprg.id IN (%s)
                GROUP  BY estpro.id, estpro.codigo, estpro.name, estprg.grade, estprg.id,
                          fisncm.codigo
            """ % (produto_company_id, ','.join(grade_ids))
            cr.execute(sql_registro_75)
            data_registro_75 = cr.dictfetchall()

        obj_estoque_produto = self.pool.get('estoque.produto')
        codigo_produto = unit_emb = None

        brw_estoque_produto_dict = dict()
        produto_ids = list(set(map(lambda p: p.get('produto_id'), data_registro_75)))
        map(lambda p: brw_estoque_produto_dict.update({p.id: p}), obj_estoque_produto.browse(
            cr, uid, produto_ids, context))
        for g in data_registro_75:
            emb = obter_menor_embalagem(brw_estoque_produto_dict.get(g.get('produto_id')))
            unit_emb = ''

            if emb:
                embalagem_id = emb.embalagem_id

                if embalagem_id:
                    unidade_id = embalagem_id.unidade_id

                    if unidade_id:
                        unit_emb = unidade_id.unidade or ''

            grade = (g.get('grade') or '').strip()
            codigo_produto = str(g.get('produto_codigo') or '').replace(' ', '')
            codigo_produto = (codigo_produto[-11:] + '_' + grade[:2]).strip().zfill(14)
            ncm = 0

            if g.get('ncm_codigo'):
                ncm = re.sub('[^0-9]', '0', g.get('ncm_codigo')) or 0
            else:
                mensagem += _('*Atencao: Nao existe NCM para o produto - ' +
                              g.get('produto_codigo') + ' - ' + g.get('nome_produto')) + '\n'

            s.add_product(start=data_ini, end=data_fim, product_code=codigo_produto, ncm=ncm,
                          desc=obj_adm_utils.admutl_remover_acentuacao(
                              cr, uid, g.get('nome_produto')).lstrip(),
                          unit=unit_emb, aliquota_ipi=int((g.get('aliquota_ipi') or 0) * 1e2),
                          aliquota_icms=int((g.get('aliquota_icms') or 0) * 1e2),
                          reducao_icms=int((g.get('reducao_icms') or 0) * 1e2),
                          base_icms=0)

        # Registro 88 - EAN
        grade_ids = list(set(map(lambda g_id: g_id.get('grade_id'), data_registro_75)))
        if len(grade_ids) == 1:
            grade_ids = str(tuple(grade_ids)).replace(',', '')
        elif len(grade_ids) > 1:
            grade_ids = str(tuple(grade_ids))
        dados_88ean = dict()
        if grade_ids:
            cr.execute("""
                -- noinspection SqlDialectInspection
                SELECT codigo_barra, grade_id
                FROM   automacao_codigo_barra
                WHERE  grade_id IN %s
            """ % grade_ids)
            dados_88 = cr.dictfetchall()
            map(lambda ean: dados_88ean.update({ean.get('grade_id'): ean.get('codigo_barra')}),
                dados_88)
        for g in data_registro_75:
            codigo_barra = dados_88ean.get(g.get('grade_id'))
            if not codigo_barra:
                continue
            s.add(SintegraRegister88EAN(
                'EAN', len(codigo_barra), codigo_produto,
                obj_adm_utils.admutl_remover_acentuacao(cr, uid, g.get('nome_produto')), unit_emb,
                codigo_barra, None))

        s.close()  # Fim geracao
        s.get_registers()
        registros_str = ''.join(map(lambda r: r.get_string(), s.get_registers()))
        regs = []

        for registro in s.get_registers():
            if registro.sintegra_number == 90:
                # noinspection PyProtectedMember
                msg = _('Total de registros tipo %s: %d') % (
                    registro._values.get('type'), int(registro._values.get('registers')))
                regs.append(msg)
                self._logger.info(msg)

        if mensagem:
            try:
                mensagem = mensagem.encode('utf-8', 'strict')
            except UnicodeDecodeError:
                mensagem = mensagem.decode('utf-8', 'strict')

        self.write(cr, uid, ids, {
            'anexo': base64.encodestring(registros_str),
            'state': 'E',
            'informacao': mensagem,
            'memo': '{0:>s}'.format('\n'.join(regs))}, context
        )
        self._grade_ids = []
        _tFim = datetime.now()
        self._logger.info('Fim da Geracao do Sintegra! (%s)' % str(_tFim - _tInicio))
        return True

    def gera_registro_54_especifico(self, cr, uid, nf_id, sequencia, context=None):
        context = context and context.copy() or {}
        res = False
        if isinstance(nf_id, (int, long)):
            nf_id = [nf_id]
        obj_fisnfc = self.pool.get('fiscal.nf.capa')
        for nf in obj_fisnfc.browse(cr, uid, nf_id, context=context):
            if sequencia == 991:
                valor = nf.vr_frete
            elif sequencia == 992:
                valor = nf.vr_seguro
            else:
                valor = nf.vr_despesas_acessorias
            if nf.tipo_frete == 'D' and valor > 0.00:
                cnpj = int(re.sub('[^0-9]', '', nf.destinatario_id.cpf_cnpj)) if nf.destinatario_id. \
                    cpf_cnpj else ''
                if nf.estoque_movimento_itens_ids:
                    cfop = iter(nf.estoque_movimento_itens_ids).next().cfop_id.codigo
                    if not cfop in ('5929', '6929'):
                        numero_nf = nf.numero_nf
                        if str(numero_nf).__len__() > 6:
                            numero_nf = int(str(numero_nf)[-6:])
                        serie_nf = nf.serie_nf
                        if serie_nf == '0':
                            serie_nf = ''
                        res = dict(
                            cnpj=cnpj,
                            modelo=int(nf.modelo_nf_id.fismnf_codigo),
                            serial=serie_nf,
                            numero=numero_nf,
                            cfop=int(cfop or 0),
                            cst='',
                            sequencia=sequencia,
                            codigo='',
                            quantidade=0,
                            valor=int(valor * 1e2),
                            vr_desconto=0,
                            base_calculo=0,
                            base_sub=0,
                            ipi=0,
                            aliquota=0,
                            )
        return res

    def get_total_itens_por_cfop(self, cr, uid, nf_id, cfop, aliquota, simples_nacional, context=None):
        """
        Metodo soma os valores referente a estoque_movimento_itens de uma nota fiscal e um cfop
        :param cr - Cursor do banco de dados
        :param uid - Usuario logado no sistema
        :param nf_id - id da nota fiscal a somar os valores
        :param cfop - cfop dos itens a somar
        :param aliquota - aliquota da movimentacao a somar
        :param context - dicionario de contextos
        :type cr: object
        :type uid: int
        :type nf_id: long
        :type cfop: str
        :type aliquota: float
        :type context: dict
        :rtype: float
        :autor: Thiago Abreu Rodrigues
        """
        obj_fiscal_cfop = self.pool.get('fiscal.cfop')
        obj_estmvi = self.pool.get('estoque.movimento.itens')
        obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
        cfop_id = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop)], context=context)
        mov_itens_ids = obj_estmvi.search(cr, uid,
                                          [('nf_id', '=', nf_id), ('cfop_id', 'in', cfop_id)],
                                          context=context)
        encargo_icms_id = self.pool.get('fiscal.encargo').search(cr, uid, [('sigla', '=', 'ICMS')],
                                                                 context=context)
        args_fisnit = [('movimento_itens_id', 'in', mov_itens_ids),
                       ('encargo_id', '=', encargo_icms_id[0])]
        if not simples_nacional:
            args_fisnit.append(('aliquota', '=', aliquota))
        item_tributacao_ids = obj_fiscal_nf_item_tributacao.search(
            cr, uid, args_fisnit, context=context)
        item_tributacao_dados = obj_fiscal_nf_item_tributacao.read(
            cr, uid, item_tributacao_ids, ['vr_icms_substituicao', 'vr_encargo', 'encargo_id',
                                           'movimento_itens_id', 'fisnit_vr_fcp_st'],
            context=context, load='_classic_write')
        # sobrescreve o valor contido em mov_itens_ids, para conter pesquisar apenas movimentacoes
        # que sejam de determinada aliquota
        mov_itens_ids = list(set(map(lambda i: i.get('movimento_itens_id'),
                                     item_tributacao_dados)))
        campos = ['vr_total_item_rateado', 'vr_despesas_rateado', 'vr_seguro_rateado', 'vr_frete_rateado',
                  'vr_desconto_rateado']
        valores_itens = obj_estmvi.read(cr, uid, mov_itens_ids, campos, load='_classic_write',
                                        context=context)
        vr_total_item = sum(map(lambda i: i.get('vr_total_item_rateado') or 0.0, valores_itens))
        vr_icms_st = 0.0
        vr_fcp_st = 0.0
        for tributacao in item_tributacao_dados:
            vr_icms_st += tributacao.get('vr_icms_substituicao') or 0.0
            vr_fcp_st += tributacao.get('fisnit_vr_fcp_st') or 0.0
        vr_ipi = 0.0
        if mov_itens_ids:
            vr_ipi = self.get_vr_ipi(cr, uid, movimento_item_ids=mov_itens_ids, context=context)
        vr_total = sum([vr_total_item, vr_icms_st, vr_ipi, vr_fcp_st])
        return float(float_round(vr_total, 2))

    def get_vr_ipi(self, cr, uid, movimento_item_ids, context):
        """
        Metodo retorna o total de ipi para os movimentos passados como parametro
        :param cr - Cursor do banco de dados
        :param uid - Usuario logado no sistema
        :param movimento_item_ids - lista de ids de estoque.movimento.itens
        :param context - dicionario de contextos
        :type cr: object
        :type uid: int
        :type movimento_item_ids: list
        :type context: dict
        :rtype: float
        :autor: Thiago Abreu Rodrigues
        """
        encargo_ipi_id = self.pool.get('fiscal.encargo').search(cr, uid, [('sigla', '=', 'IPI')],
                                                                context=context)
        obj_fisnit = self.pool.get('fiscal.nf.item.tributacao')
        args_fisnit = [('movimento_itens_id', 'in', movimento_item_ids),
                       ('encargo_id', '=', encargo_ipi_id[0])]
        fisnit_ids = obj_fisnit.search(cr, uid, args_fisnit, context=context)
        dados_ipi = obj_fisnit.read(cr, uid, fisnit_ids, ['vr_encargo'], context=context,
                                    load='_classic_write')
        vr_ipi = sum(map(lambda i: i.get('vr_encargo') or 0, dados_ipi))
        return float(float_round(vr_ipi, 2))

    def get_valor_ipi_reg50_outras(self, cr, uid, nf_id, aliquota_icms, cfop, context=None):
        """
        Metodo retorna o total de ipi para a coluna OUTROS do resgistro 50. E baseado na nota
        fiscal, aliquota do ICMS e CFOP.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_id: int
        :param aliquota_icms: percentual do imposto ICMS aplicado.
        :type aliquota_icms: float
        :param cfop: codigo fiscal de operacoes e prestacoes.
        :type cfop: str
        :param context: dados com valores contextuais.
        :type context: dict
        :return: valor total do IPI.
        :rtype: float
        """
        context = context and context.copy() or {}
        if not all([nf_id, isinstance(aliquota_icms, float), cfop]):
            return 0.0
        sql = """
            SELECT
                coalesce(sum((SELECT vr_encargo
                              FROM fiscal_nf_item_tributacao
                              WHERE
                                  encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                  AND movimento_itens_id = fnit.movimento_itens_id)), 0)   AS vr_ipi
            FROM fiscal_nf_item_tributacao fnit
            INNER JOIN fiscal_cfop fc
                ON fc.id = fnit.fisnit_cfop_id
            INNER JOIN fiscal_encargo fe
                ON fe.id = fnit.encargo_id
            WHERE
                fnit.fisnit_capa_id = {nf_id}
                AND fc.codigo = '{cfop}'
                AND fnit.aliquota = {aliquota}
                AND fe.sigla = 'ICMS'
        """.format(nf_id=nf_id, cfop=cfop, aliquota=aliquota_icms)
        cr.execute(sql)
        res = cr.fetchone()
        if res:
            res = res[0]
        else:
            res = 0.0
        return res



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
