# -*- encoding: utf-8 -*-


# noinspection PyUnusedLocal
def wizmai_get_tipo_trib_cst(self, cr, uid, ids, field_name, arg, context=None):
    """
    O metodo encontra quais tipos de CST devem ser filtrados no domain, de acordo com
    o tipo de tributacao.
    :param cr: cursor no banco de dados.
    :type: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type: integer
    :param ids: lista de id da tabela desse objeto.
    :type: list
    :param field_name: lista com nome dos atributos.
    :type: list
    :param arg: argumentos de pesquisa.
    :type: list
    :param context: dicionario com valores contextuais.
    :type: dict
    :return: lista de dicionarios com os atributos e seus respectivos valores.
    :rtype: list
    """
    context = context and context.copy() or {}
    res = {}
    obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
    for i in self.browse(cr, uid, ids):
        tipo_tributacao_cst = obj_fiscal_tributacao_encargos.fisten_get_tipo_trib_cst(
            cr, uid, ids, i.wizmai_tipo_tributacao, context)
        cst_substituicao = True if i.wizmai_cst_id.tipo == 'U' else False
        res[i.id] = {'wizmai_cst_substituicao': cst_substituicao,
                     'wizmai_tip_trib_cst': tipo_tributacao_cst}
    return res


def fisinfi_get_par_codigo_numerico(self, cr, uid, ids, field_name, arg, context=None):
    par = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque.parametro',
                                                           ['codigo_numerico'], context=context)
    res = {}
    for id in ids:
        res[id] = par.get('codigo_numerico')

    return res

def fisinfi_get_precos(self, cr, uid, ids, field_name, arg, context=None):
    """
    Pega os precos de venda e custo dos itens
    :param self:
    :param cr:
    :param uid:
    :param ids:
    :param field_name:
    :param arg:
    :param context:
    :return:
    """
    context = context and context.copy() or {}
    res = {}
    dados = self.read(cr, uid, ids, ['produto_id'], context, '_classic_write')
    produto_ids = []
    relacao = {}
    for i in dados:
        if i.get('produto_id'):
            produto_ids.append(i.get('produto_id'))
            relacao.update({i.get('id'): i.get('produto_id')})

    if context.get('produto_id'):
        produto_ids = [context.get('produto_id')]
        relacao = {ids[0]: context.get('produto_id')}

    preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid, produto_ids, context=context)
    preco_custo = self.pool.get('estoque.preco.custo').get_custos_do_dia(cr, uid, produto_ids, context=context)
    for i in ids:
        produto_id = relacao.get(i)
        res[i] = {'fisinfi_preco_venda': preco_venda and preco_venda.get(produto_id).get('preco_venda') or 0,
                  'fisinfi_preco_custo': preco_custo and preco_custo.get(produto_id).get('estprc_ultimo_custo') or 0}
    return res

# noinspection PyUnusedLocal
def fisinfi_get_produto_referenciado(self, cr, uid, ids, field_name, arg, context=None):
    """
    Mostra qual produto foi relacinado ao item do xml. Caso este jah esteja cadastrado no
    sistema:
    1 - Verifica se o usuario relacionou manualmente um produto na aba "Outras informacoes";
    2 - Verifica se o codigo de barras esta cadastrado;
    3 - Verifica se o codigo do fornecedor esta cadastrado.
    
    :param cr: cursor no banco de dados.
    :param uid: id da tabela res_users, identificador do usuario.
    :param ids: lista de id da tabela desse objeto.
    :param field_name: lista com nome dos atributos
    :param arg: argumentos de pesquisa.
    :param context: dicionario com valores contextuais.
    :type cr: object
    :type uid: int
    :type ids: list
    :type field_name: list
    :type arg: list
    :type context: dict
    :return: lista de dicionarios com os atributos e seus respectivos valores.
    :rtype: dict
    
    :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    """
    context = context and context.copy() or {}
    res = {}
    obj_estprocodfor = self.pool.get('estoque.produto.codigo.fornecedor')
    obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
    obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
    campos = {'produto_id': [], 'cean': [], 'cprod': [], 'nf_id': ['emit_pessoa_id', 'company_id']}
    for item in self.pool.get('adm.utils').admutl_read_browse(
            cr, uid, ids, self._name, campos, context=context):
        produto_id = False
        if item.get('produto_id'):
            produto_id = item.get('produto_id')
        else:
            cbarra_id = obj_automacao_codigo_barra.search(
                cr, uid, [('codigo_barra', '=', item.get('cean') or '')], context=context)
            for codigo_barra in obj_automacao_codigo_barra.browse(cr, uid, cbarra_id,
                                                                  context=context):
                produto_id = codigo_barra.produto_id.id
            if not produto_id:
                fornecedor_id = obj_financeiro_fornecedor.search(
                    cr, uid, [('pessoa_id', '=', item.get('nf_id').get('emit_pessoa_id')),
                              ('company_id', '=', item.get('nf_id').get('company_id'))],
                    context=context)[0]
                cfornec_id = obj_estprocodfor.search(
                    cr, uid, [('codigo_fornecedor', '=', item.get('cprod')),
                              ('fornecedor_id', '=', fornecedor_id)], context=context)
                for cod_fornecedor in obj_estprocodfor.browse(cr, uid, cfornec_id, context=context):
                    produto_id = cod_fornecedor.produto_id.id
        res[item.get('id')] = produto_id

    return res


def wizspc_get_produto(self, cr, uid, ids, field_name, arg, context=None):
    """
    O metodo encontra todos os produtos (estoque.produto) de acordo com os seguintes dados
    selecionaveis: ncm, categoria, comprador, fabricante e produto.
    :param cr: cursor no banco de dados.
    :type cr: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type uid: integer
    :param ids: lista de id da tabela desse objeto.
    :type ids: list
    :param context: dicionario com valores contextuais.
    :type context: dict
    :return: Dados com a identificacao do atributo funcional e seu respectivo valor.
    :rtype: dict
    """
    context = context and context.copy() or {}
    res = {}
    company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                context=context)
    for x in self.browse(cr, uid, ids, context):
        ncm_id = x.wizspc_ncm_id.id
        categoria_id = x.wizspc_categoria_id.id
        comprador_id = x.wizspc_comprador_id.id
        fabricante_id = x.wizspc_fabricante_id.id
        produto_id = x.wizspc_produto_id.id
        where = ''
        if ncm_id:
            where += ' AND ncm_id = %s' % ncm_id
        if categoria_id:
            where += ' AND categoria_id = %s' % categoria_id
        if comprador_id:
            where += ' AND comprador_id = %s' % comprador_id
        if fabricante_id:
            where += ' AND fabricante_id = %s' % fabricante_id
        if produto_id:
            where += ' AND id = %s' % produto_id
        sql = """
            SELECT
                id
            FROM
                estoque_produto
            WHERE
                active = True
                AND company_id = %s
        """ % company_id
        if where:
            sql += where
        cr.execute(sql)
        res[x.id] = map(lambda i: i.get('id'), cr.dictfetchall())
    return res
