# -*- coding: utf-8 -*-

import base64

from openerp.osv import osv, fields

# noinspection PyProtectedMember
from openerp.tools.translate import _
from pythonsped.efd import *
from datetime import datetime
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from SigUtils.formatastring import formataString
from sped_sql.sql_sped_fiscal import SqlSpedFiscal
from openerp.osv.fields import float_round
from collections import Counter
import logging
import time
_logger = logging.getLogger('fiscal.sped')


# noinspection PyPep8Naming
class groupby(dict):
    def __init__(self, seq, key=lambda x: x, **kwargs):
        super(groupby, self).__init__(**kwargs)
        for value in seq:
            k = key(value)

            self.setdefault(k, []).append(value)

    __iter__ = dict.iteritems


# noinspection PyProtectedMember,PyAbstractClass,PyPep8Naming,PyShadowingNames
class fiscal_sped(osv.TransientModel):
    domain_inventario_id = "[('company_id', '=', company_id), ('ano', '=', ano_inventario),\
     ('mes', '=', mes_inventario), ('esticp_data_livro_inventario', '!=', False)]"

    # Atributo utilizado via heranca no modulo patrimonio_fiscal (Bloco G)
    registro_0190_dados = list()

    _name = 'fiscal.sped'
    _columns = {
        'state': fields.selection(string='Status',
                                  selection=[('A', 'Aguardando geracao do arquivo'),
                                             ('E', 'Arquivo gerado')], required=True, size=1),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="restrict", required=True,
                                      select=True),  # id da empresa
        'dados': fields.binary('Clique para abrir o arquivo ', readonly=True),
        'tipo_arquivo': fields.selection([('0', 'Original'), ('1', 'Substituto')],
                                         'Finalidade do Arquivo',
                                         required=True),
        'informacao': fields.html('Informacoes'),
        'gerar_bloco_k': fields.boolean('Gerar bloco K'),
        'gerar_inventario': fields.boolean('Gerar Inventario'),
        'mes_inventario': fields.char('Mes Inventario', size=2),
        'ano_inventario': fields.char('Ano Inventario', size=4),
        'inventario_id': fields.many2one('estoque.inventario.capa', 'Inventario', ondelete='cascade',
                                         domain=domain_inventario_id),
        'file': fields.char('Nome do arquivo', size=20, readonly=False, required=True),
        'sped_gerado': fields.boolean('SPED gerado'),
        'sped_data_inicial': fields.date('Data Inicial', required=True),
        'sped_data_final': fields.date('Data Final', required=True),
        'motivo_inventario': fields.selection(
            [('01', '01-No final no periodo'),
             ('02', '02-Na mudanca de forma de tributacao da mercadoria (ICMS)'),
             ('03', '03-Na solicitacao da baixa cadastral, paralisacao  temporaria e outras '
                    'situacoes'),
             ('04', '04-Na alteracao de regime de pagamento – condicao  do contribuinte'),
             ('05', '05-Por determinacao dos fiscos')], 'Informe o motivo do Inventario'),
        'gerar_pis_cofins': fields.boolean('Gerar PIS/COFINS'),
    }
    _defaults = {
        'state': 'A',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                                        s._name,
                                                                                        context=c),
        'file': lambda *a: 'sped.txt',
        'tipo_arquivo': '0',
        'sped_gerado': False,
        'motivo_inventario': '01',
        'gerar_pis_cofins': lambda *a: True,
    }

    _mensagens = []

    def info_pessoa(self, cr, uid, pessoa_id, context=None):
        """
        Funcao para retornar informacoes da pessoa de acordo com o que eh disponibilizado.
        Se existir endereco de nota fiscal, retorna o endereco, se nao existir busca pelo
        primeiro endereco comercial e nao existindo o 'Comercial', pegará o primeiro endereco
        'Normal'. Tambem retorna CPF/CNPJ, cep, telefone (sem caracteres especiais), e todos os
        documentos da pessoa.
       """
        context = context and context.copy() or {}
        obj_pes = self.pool.get('adm.pessoa')
        obj_end = self.pool.get('adm.pessoa.endereco')
        obj_doc = self.pool.get('adm.pessoa.documento.dados')
        obj_tipo = self.pool.get('adm.pessoa.documento.tipo')
        vals = {}
        vals_end = {}
        read_pessoa = obj_pes.read(cr, uid, pessoa_id,
                                   ['name', 'cpf_cnpj', 'endereco_nf_id', 'endereco_ids',
                                    'telefone', 'email',
                                    'pessoa_tipo', 'telefones_ids', 'email_ids', 'admpes_jur_ie'],
                                   context=context)
        vals['nome'] = read_pessoa['name']
        vals['inscricao_estadual'] = read_pessoa['admpes_jur_ie']
        if read_pessoa['cpf_cnpj']:
            cpf_cnpj = read_pessoa['cpf_cnpj'].replace('.', '').replace('-', '').replace('/', '')
            if read_pessoa['pessoa_tipo'] == 'F':
                vals['cpf'] = cpf_cnpj
            else:
                vals['cnpj'] = cpf_cnpj

        if read_pessoa['endereco_nf_id']:
            end_pessoa = obj_end.read(cr, uid, read_pessoa['endereco_nf_id'][0], context=context)
            if end_pessoa['logradouro_id']:
                vals_end['logradouro'] = end_pessoa['logradouro_id'][1]
            if end_pessoa['numero']:
                vals_end['numero'] = end_pessoa['numero']
            if end_pessoa['complemento']:
                vals_end['complemento'] = end_pessoa['complemento']
            if end_pessoa['bairro_id']:
                vals_end['bairro'] = end_pessoa['bairro_id'][1]
            if end_pessoa['cidade_id']:
                vals_end['cidade'] = end_pessoa['cidade_id'][1]
                codigo_ibge = self.pool.get('adm.cidade').read(cr, uid, end_pessoa['cidade_id'][0],
                                                               ['codigo_ibge'], context=context)[
                    'codigo_ibge']
                if codigo_ibge:
                    vals_end['codigo_ibge'] = codigo_ibge
            if end_pessoa['uf_id']:
                sigla = self.pool.get('adm.uf').read(cr, uid, end_pessoa['uf_id'][0], ['sigla'],
                                                     context=context)['sigla']
                vals_end['uf'] = sigla
            if end_pessoa['cep']:
                vals_end['cep'] = end_pessoa['cep'].replace('.', '').replace('-', '')
            if end_pessoa['pais_id']:
                vals_end['pais'] = \
                    self.pool.get('adm.pais').read(cr, uid, end_pessoa['pais_id'][0], ['codigo_bc'],
                                                   context=context)[
                        'codigo_bc']

        else:
            enderecos = read_pessoa['endereco_ids']
            if enderecos:
                for i in enderecos:
                    end_pessoa = obj_end.read(cr, uid, i, context=context)
                    if end_pessoa['endereco_tipo_id']:
                        if end_pessoa['endereco_tipo_id'][1] == 'Comercial':
                            if end_pessoa['logradouro_id']:
                                vals_end['logradouro'] = end_pessoa['logradouro_id'][1]
                            if end_pessoa['numero']:
                                vals_end['numero'] = end_pessoa['numero']
                            if end_pessoa['complemento']:
                                vals_end['complemento'] = end_pessoa['complemento']
                            if end_pessoa['bairro_id']:
                                vals_end['bairro'] = end_pessoa['bairro_id'][1]
                            if end_pessoa['cidade_id']:
                                vals_end['cidade'] = end_pessoa['cidade_id'][1]
                                codigo_ibge = \
                                    self.pool.get('adm.cidade').read(cr, uid,
                                                                     end_pessoa['cidade_id'][0],
                                                                     ['codigo_ibge'],
                                                                     context=context)[
                                        'codigo_ibge']
                                if codigo_ibge:
                                    vals_end['codigo_ibge'] = codigo_ibge
                            if end_pessoa['uf_id']:
                                sigla = \
                                    self.pool.get('adm.uf').read(cr, uid, end_pessoa['uf_id'][0],
                                                                 ['sigla'], context=context)[
                                        'sigla']
                                vals_end['uf'] = sigla
                            if end_pessoa['cep']:
                                vals_end['cep'] = end_pessoa.replace('.', '').replace('-', '')
                            if end_pessoa['pais_id']:
                                vals_end['pais'] = \
                                    self.pool.get('adm.pais').read(cr, uid,
                                                                   end_pessoa['pais_id'][0],
                                                                   ['codigo_bc'], context=context)[
                                        'codigo_bc']
                            break

                        if end_pessoa['endereco_tipo_id'][1] == 'Normal':
                            if end_pessoa['logradouro_id']:
                                vals_end['logradouro'] = end_pessoa['logradouro_id'][1]
                            if end_pessoa['numero']:
                                vals_end['numero'] = end_pessoa['numero']
                            if end_pessoa['complemento']:
                                vals_end['complemento'] = end_pessoa['complemento']
                            if end_pessoa['bairro_id']:
                                vals_end['bairro'] = end_pessoa['bairro_id'][1]
                            if end_pessoa['cidade_id']:
                                vals_end['cidade'] = end_pessoa['cidade_id'][1]
                                codigo_ibge = \
                                    self.pool.get('adm.cidade').read(cr, uid,
                                                                     end_pessoa['cidade_id'][0],
                                                                     ['codigo_ibge'],
                                                                     context=context)[
                                        'codigo_ibge']
                                if codigo_ibge:
                                    vals_end['codigo_ibge'] = codigo_ibge
                            if end_pessoa['uf_id']:
                                sigla = \
                                    self.pool.get('adm.uf').read(cr, uid, end_pessoa['uf_id'][0],
                                                                 ['sigla'], context=context)[
                                        'sigla']
                                vals_end['uf'] = sigla
                            if end_pessoa['cep']:
                                vals_end['cep'] = end_pessoa['cep'].replace('.', '').replace('-',
                                                                                             '')
                            if end_pessoa['pais_id']:
                                vals_end['pais'] = \
                                    self.pool.get('adm.pais').read(cr, uid,
                                                                   end_pessoa['pais_id'][0],
                                                                   ['codigo_bc'], context=context)[
                                        'codigo_bc']
                            continue
        vals['endereco'] = vals_end
        # Documentos
        id_doc = obj_doc.search(cr, uid, [('pessoa_id', '=', pessoa_id)], context=context)
        docs = {}
        if id_doc:
            for i in id_doc:
                id_t = obj_doc.read(cr, uid, i, ['tipo_id'], context=context)['tipo_id'][0]
                sigla = obj_tipo.read(cr, uid, id_t, ['sigla'], context=context)['sigla']
                docs[sigla] = obj_doc.read(cr, uid, i, ['numero'], context=context)['numero']
        vals['documentos'] = docs
        # Telefones
        val_tel = {}
        if read_pessoa['telefones_ids']:
            tels = self.pool.get('adm.pessoa.telefone').read(cr, uid, read_pessoa['telefones_ids'],
                                                             context=context)
            for i in tels:
                val_tel[i['telefone_tipo_id'][1]] = '%010d' % int(
                    i['numero'].replace('(', '').replace(')', '').replace('-', ''))
        vals['telefones'] = val_tel

        # email
        val_email = {}
        if read_pessoa['email_ids']:
            emails = self.pool.get('adm.pessoa.email').read(cr, uid, read_pessoa['email_ids'],
                                                            context=context)
            for i in emails:
                val_email[i['email_tipo_id'][1]] = i['name']
        vals['emails'] = val_email

        return vals

    @staticmethod
    def formata_data(data):
        """
        Aceita-se para data tanto uma string no formato '%Y-%m-%d %H-%M-%s'
        ou um Datetime, para facilitar a formatacao.
        """
        if isinstance(data, datetime):
            return data.strftime('%d%m%Y')
        else:
            return datetime(int(data[:4]), int(data[5:7]),
                            int(data[8:10])).strftime('%d%m%Y')

    @staticmethod
    def formata_numero(value, size=9, dec=2, separador='.'):
        mask = '%.{dec}f'.replace('{dec}', str(dec))
        ret = mask % value if value else mask % 0.00
        ret = ret.rjust((size + dec + 1), '0') if size else ret
        if separador != '.':
            ret = ret.replace('.', separador)
        return ret

    @staticmethod
    def formata_para_digitos(digito):
        if digito:
            digito_formatado = ''
            for i in digito:
                if i.isdigit() or i.isalpha():
                    digito_formatado += i
        else:
            digito_formatado = ''
        return digito_formatado

    def filtra_ids_itens(self, cr, uid, obj_fiscal, movimento, perfil_sped, context=None):
        """
        Funcao: Retornar os ids dos movimentos que nao foram proprios e modelo 55
                Ira servir para filtro dos registros 0190 e 0200, para que nao
                seja gerado estes registros para itens que nao foram referenciados em outros blocos
        Parametros: obj_fiscal (fiscal.nf.capa)
                    movimento (Dicionario contendo os ids dos moviemntos de notas e cupons)
        """
        context = context and context.copy() or {}
        nota_ids = []
        # print '-----------------------INI FILTRO 0190'
        # print time.strftime('%Y-%m-%d %H:%M:%S')
        if movimento.get('movimento_notas') and perfil_sped != 'C':
            sql = """
                SELECT 
                    nf.id AS nf_id,
                    CASE 
                        WHEN modelo.fismnf_codigo IN ('21', '22') 
                            THEN TRUE 
                        ELSE 
                            FALSE 
                    END AS telecomunicacao,
                    nf.nf_propria,
                    CASE
                        WHEN
                            modelo.fismnf_tipo = 'E'
                                THEN TRUE
                        ELSE
                            FALSE
                    END AS energia_eletrica,
                    nf.modelo_55
                FROM
                    fiscal_nf_capa nf
                LEFT JOIN
                    fiscal_modelo_nf modelo ON modelo.id = nf.modelo_nf_id
                WHERE
                    nf.id in ({nf_ids})
            """.format(nf_ids=', '.join(map(lambda n: str(n), movimento.get('movimento_notas'))))
            cr.execute(sql)
            dados = cr.dictfetchall()
            for nf in dados:
                telecomunicacao = nf.get('telecomunicacao')
                if not any([nf.get('nf_propria'), nf.get('energia_eletrica'), telecomunicacao]):
                    nota_ids.append(nf.get('nf_id'))
                else:
                    if not any([nf.get('modelo_55'), nf.get('energia_eletrica'), telecomunicacao]):
                        nota_ids.append(nf.get('nf_id'))
        filtro = {
            'movimento_notas': nota_ids or [-1],  # Necessario na montagem das sentencas SQL
            'movimento_cupons': movimento.get('movimento_cupons') if perfil_sped != 'C' else list()
        }
        # print time.strftime('%Y-%m-%d %H:%M:%S')
        # print '-----------------------FIM FILTRO 0190'
        return filtro

    def get_modelo_codigo(self, cr, uid, modelo_id, context=None):
        context = context and context.copy() or {}
        obj_fiscal_modelo = self.pool.get('fiscal.modelo.nf')
        res = False
        if modelo_id:
            codigo_modelo = obj_fiscal_modelo.read(cr, uid, modelo_id, ['fismnf_codigo'],
                                                   context=context, load='_classic_write')
            if codigo_modelo:
                res = codigo_modelo.get('fismnf_codigo')
        return res

    def gerar_sped(self, cr, uid, ids, context=None):
        _logger.info('Gerando Sped ICMS/IPI...')
        _tInicio = datetime.now()
        # print '-----------------------INI BOTAO GERAR'
        # print time.strftime('%Y-%m-%d %H:%M:%S')
        context = context and context.copy() or {}
        val = self.read(cr, uid, ids[0], context=context)
        gerar_pis_cofins = val.get('gerar_pis_cofins')
        obj_emp = self.pool.get('res.company')
        obj_pes = self.pool.get('adm.pessoa')
        obj_adm_utl = self.pool.get('adm.utils')
        obj_par = self.pool.get('fiscal.parametro')
        obj_est_par = self.pool.get('estoque.parametro')
        obj_doc = self.pool.get('adm.pessoa.documento.dados')
        obj_tipo = self.pool.get('adm.pessoa.documento.tipo')
        obj_fiscal = self.pool.get('fiscal.nf.capa')
        obj_apuracao_imposto = self.pool.get('fiscal.apuracao.imposto')
        obj_deducoes = self.pool.get('fiscal.apuracao.imposto.deducoes')
        obj_outros_debitos = self.pool.get('fiscal.apuracao.imposto.outros.debitos')
        obj_outros_creditos = self.pool.get('fiscal.apuracao.imposto.outros.creditos')
        obj_estorno_debitos = self.pool.get('fiscal.apuracao.imposto.estorno.debitos')
        obj_estorno_creditos = self.pool.get('fiscal.apuracao.imposto.estorno.creditos')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        sped_data_inicial = val.get('sped_data_inicial')
        sped_data_final = val.get('sped_data_final') + ' 23:59:59'
        obj_sql = SqlSpedFiscal(cr, sped_data_inicial, sped_data_final, val['company_id'][0])
        if obj_sql.erro:
            raise osv.except_osv(_('Erro'), _(obj_sql.erro))

        list_fiscal_parametro = ['fispar_classificacao_estabelecimento']
        fiscal_parametro_dados = obj_adm_utl.admutl_get_parametros(
            cr, uid, 'fiscal', list_fiscal_parametro, context=context)

        DATE_SPED_INICIAL = parse(sped_data_inicial, yearfirst=True).date()
        DATE_SPED_FINAL = parse(sped_data_final, yearfirst=True).date()

        if DATE_SPED_FINAL < DATE_SPED_INICIAL:
            raise osv.except_osv('Atencao', u'A data inicial deve ser menor que a data final!')

        sped_mes_inicial = DATE_SPED_INICIAL.month
        sped_mes_final = DATE_SPED_FINAL.month
        sped_ano_inicial = DATE_SPED_INICIAL.year
        sped_ano_final = DATE_SPED_FINAL.year
        args_periodo_mes_e_ano = [
            ('mes', '>=', sped_mes_inicial), ('mes', '<=', sped_mes_final),
            ('ano', '>=', sped_ano_inicial), ('ano', '<=', sped_ano_final)
        ]

        msg = ''
        if isinstance(val['company_id'], tuple):
            val['company_id'] = val['company_id'][0]
        id_pessoa = obj_emp.read(cr, uid, val['company_id'], ['rescon_pessoa_id'], context=context)[
            'rescon_pessoa_id'][0]

        if DATE_SPED_INICIAL.year < 2011 or DATE_SPED_FINAL.year < 2011:
            raise osv.except_osv(_('Atencao'), _(
                'Ano informado invalido para leiaute do Ato Cotepe. O ano deve ser a partir '
                'de 2011.'))

        info_pessoa = self.info_pessoa(cr, uid, id_pessoa, context=context)
        read_pessoa = obj_pes.browse(cr, uid, id_pessoa)
        data_ini = sped_data_inicial
        data_fim = sped_data_final
        nome_empresa = read_pessoa.name
        tipo_atividade = read_pessoa.admpes_jur_tipo_atividade
        id_par = obj_par.search(cr, uid, [('company_id', '=', val['company_id'])], context=context)
        brw_fispar = obj_par.browse(cr, uid, id_par[0], context=context)
        dia_vencimento_icms = ''
        if not id_par:
            raise osv.except_osv(_('Atencao'), _('Parametro do Fiscal nao cadastrado.'))
        else:
            perfil_sped = \
                obj_par.read(cr, uid, id_par[0], ['perfil_arquivo_sped'], context=context)[
                    'perfil_arquivo_sped']
            contribuinte_substituto = \
                obj_par.read(cr, uid, id_par[0], ['contribuinte_substituto'], context=context)[
                    'contribuinte_substituto']
            dia_vencimento_icms = \
                obj_par.read(cr, uid, id_par[0], ['dia_vencimento_obrigacao_icms'],
                             context=context)[
                    'dia_vencimento_obrigacao_icms']
            codigo_receita = obj_par.read(cr, uid, id_par[0], ['codigo_receita'], context=context)[
                'codigo_receita']

        # Verifcando se existe os movimentos de nota
        filtro = {
            'sped_data_inicial': sped_data_inicial,
            'sped_data_final': sped_data_final,
            'company_id': str(val['company_id'])
        }
        movimento_notas = obj_sql.sql_consulta_movimento_notas(cr, filtro, perfil_sped)

        # Verificando se existe movimentacao de cupons
        dt_ini = DATE_SPED_INICIAL.strftime("%Y-%m-%d 00:00:00")
        dt_fim = DATE_SPED_FINAL.strftime("%Y-%m-%d 23:59:59")
        grupo_ipi_valores = []
        total_geral_debitos = 0.0
        total_geral_creditos = 0.0
        obj_adm_utils = self.pool.get('adm.utils')
        modulo_ecf_instalado = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'ecf',
                                                                           context=None)
        modulo_pcp_instalado = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'pcp',
                                                                           context={})
        movimento_cupons = ''
        if modulo_ecf_instalado:
            movimento_cupons = obj_sql.sql_consulta_movimento_cupons(cr, filtro)

        cod_versao = self.get_versao_layout(DATE_SPED_FINAL.year)
        if int(cod_versao) <= 15:
            movimento_eletronico_pagamento = obj_sql.sql_consulta_mov_cartao(
                cr, dt_ini, dt_fim, modulo_ecf_instalado, val['company_id'])
        else:
            movimento_eletronico_pagamento = obj_sql.sql_reg1601(
                cr, dt_ini, dt_fim, modulo_ecf_instalado, val['company_id'])
        apuracao_difal = []
        itens_movimentados = dict()
        ajustes_creditos = []
        sped = eval("layout_%s" % cod_versao[1:])  # Leiaute do SPED em funcao de sua versao
        # print time.strftime('%Y-%m-%d %H:%M:%S')
        # print '-----------------------FIM BOTAO GERAR'

        # Bloco 0
        reg_0000 = bloco_arquivo.registro_0000()
        reg_0001 = bloco_0.registro_0001()
        if cod_versao and int(cod_versao) >= 14:
            reg_0002 = sped.bloco_0.registro_0002()
        reg_0005 = bloco_0.registro_0005()
        reg_0100 = bloco_0.registro_0100()
        reg_0150 = bloco_0.registro_0150()
        reg_0190 = bloco_0.registro_0190()
        reg_0200 = sped.bloco_0.registro_0200()
        reg_0205 = bloco_0.registro_0205()
        reg_0400 = bloco_0.registro_0400()
        reg_0450 = bloco_0.registro_0450()
        lista_0460 = []

        # Bloco B
        reg_B001 = bloco_b.registro_B001()

        # Bloco c
        reg_C001 = bloco_c.registro_C001()
        reg_C100 = bloco_c.registro_C100()
        reg_C101 = bloco_c.registro_C101()
        reg_C110 = bloco_c.registro_C110()
        reg_C113 = bloco_c.registro_C113()
        reg_C140 = bloco_c.registro_C140()
        reg_C141 = bloco_c.registro_C141()
        reg_C170 = (cod_versao and int(cod_versao) >= 10) and sped.bloco_c.registro_C170() or \
            bloco_c.registro_C170()
        reg_C172 = bloco_c.registro_C172()
        if int(cod_versao) >= 15:
            reg_C181 = sped.bloco_c.registro_C181()
            reg_C186 = sped.bloco_c.registro_C186()
        reg_C190 = bloco_c.registro_C190()
        reg_C195 = bloco_c.registro_C195()
        reg_C197 = bloco_c.registro_C197()
        reg_C300 = bloco_c.registro_C300()
        reg_C310 = bloco_c.registro_C310()
        reg_C320 = bloco_c.registro_C320()
        reg_C321 = bloco_c.registro_C321()
        reg_C400 = bloco_c.registro_C400()
        reg_C405 = bloco_c.registro_C405()
        reg_C420 = bloco_c.registro_C420()
        reg_C425 = bloco_c.registro_C425()
        reg_C490 = bloco_c.registro_C490()
        reg_C500 = (cod_versao and int(cod_versao) >= 14) and bloco_c.registro_C500() or \
            sped.bloco_c.registro_C500()
        reg_C590 = bloco_c.registro_C590()

        # Instanciando bloco D
        reg_D001 = bloco_d.registro_D001()
        reg_D190 = bloco_d.registro_D190()
        reg_D195 = bloco_d.registro_D195()
        reg_D500 = bloco_d.registro_D500()
        reg_D590 = bloco_d.registro_D590()

        # Bloco E
        reg_E001 = bloco_e.registro_E001()
        reg_E100 = bloco_e.registro_E100()
        reg_E110 = bloco_e.registro_E110()
        reg_E116 = bloco_e.registro_E116()
        reg_E200 = bloco_e.registro_E200()
        reg_E210 = bloco_e.registro_E210()
        reg_E250 = bloco_e.registro_E250()
        reg_E300 = bloco_e.registro_E300()
        reg_E310 = sped.bloco_e.registro_E310()
        reg_E316 = bloco_e.registro_E316()
        reg_E500 = bloco_e.registro_E500()

        # Bloco H
        reg_H001 = bloco_h.registro_H001()
        reg_H005 = bloco_h.registro_H005()
        reg_H010 = bloco_h.registro_H010()

        # Bloco K
        reg_K001 = bloco_k.registro_K001()
        reg_K100 = bloco_k.registro_K100()
        reg_K200 = bloco_k.registro_K200()
        reg_K230 = bloco_k.registro_K230()
        reg_K235 = bloco_k.registro_K235()

        # Bloco 1
        reg_1001 = bloco_1.registro_1001()
        reg_1010 = (cod_versao and int(cod_versao) >= 10) and sped.bloco_1.registro_1010() or \
            bloco_1.registro_1010()
        if int(cod_versao) >= 15:
            reg_1250 = sped.bloco_1.registro_1250()
            reg_1255 = sped.bloco_1.registro_1255()
        if int(cod_versao) <= 15:
            reg_1600 = bloco_1.registro_1600()
        if int(cod_versao) >= 16:
            reg_1601 = sped.bloco_1.registro_1601()

        # Dados do Bloco 0
        def _registro0000():
            # print '-----------------------INI REG 0000'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """
            Gera dados do registro 0000 do sped fiscal
            """
            mensagem = ''
            cnpj = ''
            if 'cnpj' in info_pessoa or 'cpf' in info_pessoa:
                if 'cnpj' in info_pessoa:
                    cnpj = info_pessoa['cnpj']
                elif 'cpf' in info_pessoa['cpf']:
                    cnpj = ''
            else:
                cnpj = ''
            if 'uf' in info_pessoa['endereco']:

                uf = info_pessoa['endereco']['uf']
            else:
                mensagem += _("Nao existe endereco cadastrado para a empresa")
                uf = ''
            if read_pessoa.admpes_jur_ie:
                ie = self.formata_para_digitos(read_pessoa.admpes_jur_ie)
            else:
                ie = ''
                mensagem += _("Nao existe inscricao estadual cadastrada para a empresa")
            if 'codigo_ibge' in info_pessoa['endereco']:
                cod_mun = info_pessoa['endereco']['codigo_ibge']
            else:
                cod_mun = ''
            if 'IM' in info_pessoa['documentos']:
                im = info_pessoa['documentos']
            else:
                im = ''
            # noinspection PyShadowingNames
            perfil_sped = \
                obj_par.read(cr, uid, id_par[0], ['perfil_arquivo_sped'], context=context)[
                    'perfil_arquivo_sped']
            if read_pessoa.admpes_jur_tipo_atividade:
                if read_pessoa.admpes_jur_tipo_atividade == 'I':
                    ind_atividade = '0'
                else:
                    ind_atividade = '1'
            else:
                ind_atividade = ''

            reg_0000.COD_VER = cod_versao
            reg_0000.COD_FIN = val.get('tipo_arquivo')
            reg_0000.DT_INI = self.formata_data(data_ini)
            reg_0000.DT_FIN = self.formata_data(data_fim)
            reg_0000.NOME = read_pessoa.name
            reg_0000.CNPJ = cnpj
            reg_0000.UF = uf
            reg_0000.IE = ie
            reg_0000.COD_MUN = cod_mun
            reg_0000.IM = im
            reg_0000.SUFRAMA = read_pessoa.admpes_jur_suframa or ''
            reg_0000.IND_PERFIL = perfil_sped
            reg_0000.IND_ATIV = ind_atividade
            reg_0000.write_line()

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0000'
            return mensagem

        def _registro0001():
            # print '-----------------------INI REG 0001'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """Gera dados do registro 0001 do sped fiscal"""
            mensagem = ''
            dados = {
                'REG': '0001',
                'IND_MOV': '0'
            }
            reg_0001.REG = dados.get('REG')
            reg_0001.IND_MOV = dados.get('IND_MOV')
            reg_0001.write_line()

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0001'
            return mensagem

        def _registro0002():
            mensagem = ''
            classificacao_estabelecimento = fiscal_parametro_dados.get(
                'fispar_classificacao_estabelecimento')
            if not classificacao_estabelecimento:
                mensagem += "\n 'Classificacao do estabelecimento' nao cadastrada nos parametros " \
                            "do fiscal. \n"
                return mensagem
            reg_0002.REG = '0002'
            reg_0002.CLAS_ESTAB_IND = classificacao_estabelecimento
            reg_0002.write_line()
            return mensagem

        def _registro0005():
            # print '-----------------------INI REG 0005'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """Gera dados do registro 0005 do sped fiscal"""
            mensagem = ''
            cep = ''
            end = ''
            num = ''
            compl = ''
            bairro = ''
            fone = ''
            fax = ''
            email = ''
            if info_pessoa['endereco']:
                if 'cep' in info_pessoa['endereco']:
                    if len(info_pessoa['endereco']['cep']) == 8:
                        cep = info_pessoa['endereco'][
                            'cep']
                    else:
                        mensagem += _(
                            '* Informacao invalida para o campo CEP definido no endereco da '
                            'Empresa.\n')
                else:
                    mensagem += _('* Nao existe CEP definido no endereco da Empresa.\n')
                if 'logradouro' in info_pessoa['endereco']:
                    end = info_pessoa['endereco']['logradouro']
                else:
                    mensagem += _('* Logradouro da empresa nao cadastrado.\n')
                if 'numero' in info_pessoa['endereco']:
                    num = info_pessoa['endereco']['numero']
                if 'complemento' in info_pessoa['endereco']:
                    compl = info_pessoa['endereco']['complemento']
                if 'bairro' in info_pessoa['endereco']:
                    bairro = info_pessoa['endereco']['bairro']
                else:
                    mensagem += _('* Bairro da empresa nao informado.\n')
            if read_pessoa.telefone:
                fone = read_pessoa.telefone.replace('(', '').replace(')', '').replace('-', '')
            if 'telefones' in info_pessoa:
                for x in info_pessoa['telefones']:
                    if x == 'Fax' or x == 'FAX':
                        fax = info_pessoa['telefones'][x]
                        break
            if read_pessoa.email:
                email = read_pessoa.email

            dados = {
                'REG': '0005',
                'FANTASIA': nome_empresa,
                'CEP': cep,
                'END': end,
                'NUM': num,
                'COMPL': compl or '',
                'BAIRRO': bairro,
                'FONE': fone,
                'FAX': fax,
                'EMAIL': email,
            }
            reg_0005.REG = dados.get('REG')
            reg_0005.FANTASIA = dados.get('FANTASIA')
            reg_0005.CEP = dados.get('CEP')
            reg_0005.END = dados.get('END')
            reg_0005.NUM = dados.get('NUM')
            reg_0005.COMPL = dados.get('COMPL')
            reg_0005.BAIRRO = dados.get('BAIRRO')
            reg_0005.FONE = dados.get('FONE')
            reg_0005.FAX = dados.get('FAX')
            reg_0005.EMAIL = dados.get('EMAIL')
            reg_0001.registro_0005.append(reg_0005.write_line())

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0005'
            return mensagem

        def _registro0015():
            """
            Gera registro 0015 do sped fiscal
            """
            # print '-----------------------INI REG 0015'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            if contribuinte_substituto:
                documentos_ids = []
                for d in read_pessoa.documentos_ids:
                    documentos_ids.append(d.id)
                for i in documentos_ids:
                    id_t = obj_doc.read(cr, uid, i, ['tipo_id'], context=context)['tipo_id'][0]
                    sigla = obj_tipo.read(cr, uid, id_t, ['sigla'], context=context)['sigla']
                    if sigla == 'IE':
                        reg_0015 = bloco_0.registro_0015()
                        uf_st = read_pessoa.uf
                        ie_st = obj_doc.read(cr, uid, i, ['numero'], context=context)['numero']

                        dados = {
                            'REG': '0015',
                            'UF_ST': uf_st,
                            'IE_ST': ie_st
                        }
                        reg_0015.REG = dados.get('REG')
                        reg_0015.UF_ST = dados.get('UF_ST')
                        reg_0015.IE_ST = dados.get('IE_ST')
                        reg_0001.registro_0015.append(reg_0015.write_line())

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0015'
            return mensagem

        def _registro0100():
            # print '-----------------------INI REG 0100'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """
            Gerar registro 0100 do sped
            """
            mensagem = ''
            obj_adm_par = self.pool.get('adm.parametro')
            id_par_adm = obj_adm_par.search(cr, uid, [('company_id', '=', val['company_id'])],
                                            context=context)
            dados = {
                'REG': '0100',
                'NOME': '',
                'CPF': '',
                'CRC': '',
                'CNPJ': '',
                'CEP': '',
                'END': '',
                'NUM': '',
                'COMPL': '',
                'BAIRRO': '',
                'FONE': '',
                'FAX': '',
                'EMAIL': '',
                'COD_MUN': '',
            }
            if not id_par_adm:
                mensagem += _('* Nao existe existe parametro cadastrado em Administracao.\n')
            else:
                contador_id = \
                    obj_adm_par.read(cr, uid, id_par_adm[0], context=context)['contador_id'][0]
                if contador_id:
                    pessoa_contabil = self.info_pessoa(cr, uid, contador_id, context=context)
                    dados.update({'NOME': pessoa_contabil['nome']})
                    if 'cnpj' in pessoa_contabil:
                        dados.update({'CNPJ': pessoa_contabil['cnpj']})
                    if 'cpf' in pessoa_contabil:
                        dados.update({'CPF': pessoa_contabil['cpf']})
                    else:
                        mensagem += _('* Nao existe CPF cadastrado para o Contabilista.\n')

                    if not pessoa_contabil['documentos']:
                        mensagem += _('* Nao existe CRC cadastrado para o Contabilista.\n')
                    else:
                        if 'CRC' in pessoa_contabil['documentos']:
                            dados.update({'CRC': pessoa_contabil['documentos']['CRC']})
                        else:
                            mensagem += _('* Nao existe CRC cadastrado para o Contabilista.\n')

                    if pessoa_contabil['endereco']:
                        if 'cep' in pessoa_contabil['endereco']:
                            dados.update({'CEP': pessoa_contabil['endereco']['cep']})
                        else:
                            mensagem += _('* Nao existe CEP definido no endereco da Empresa.\n')
                        if 'logradouro' in pessoa_contabil['endereco']:
                            dados.update({'END': pessoa_contabil['endereco']['logradouro']})
                        if 'numero' in pessoa_contabil['endereco']:
                            dados.update({'NUM': pessoa_contabil['endereco']['numero']})
                        if 'complemento' in pessoa_contabil['endereco']:
                            dados.update({'COMPL': pessoa_contabil['endereco']['complemento']})
                        if 'bairro' in pessoa_contabil['endereco']:
                            dados.update({'BAIRRO': pessoa_contabil['endereco']['bairro']})
                        if 'codigo_ibge' in pessoa_contabil['endereco']:
                            dados.update({'COD_MUN': pessoa_contabil['endereco']['codigo_ibge']})
                    telefone_comum = \
                        obj_pes.read(cr, uid, contador_id, ['telefone'], context=context)[
                            'telefone']
                    if telefone_comum:
                        tel_comun = telefone_comum.replace('(', '').replace(')', '').replace('-',
                                                                                             '')
                        dados.update({'FONE': tel_comun})
                    if 'telefones' in pessoa_contabil:
                        for x in pessoa_contabil['telefones']:
                            if x == 'Fax' or x == 'FAX':
                                dados.update({'FAX': pessoa_contabil['telefones'][x]})
                                break
                    email_cont = obj_pes.read(cr, uid, contador_id, ['email'], context=context)[
                        'email']
                    if email_cont:
                        dados.update({'EMAIL': email_cont})
                else:
                    mensagem += _('* Contador nao informado no parametro de Administracao.\n')

                reg_0100.REG = dados.get('REG')
                reg_0100.NOME = dados.get('NOME')
                reg_0100.CPF = dados.get('CPF')
                reg_0100.CRC = dados.get('CRC')
                reg_0100.CNPJ = dados.get('CNPJ')
                reg_0100.CEP = dados.get('CEP')
                reg_0100.END = dados.get('END')
                reg_0100.NUM = dados.get('NUM')
                reg_0100.COMPL = dados.get('COMPL')
                reg_0100.BAIRRO = dados.get('BAIRRO')
                reg_0100.FONE = dados.get('FONE')
                reg_0100.FAX = dados.get('FAX')
                reg_0100.EMAIL = dados.get('EMAIL')
                reg_0100.COD_MUN = dados.get('COD_MUN')
                reg_0001.registro_0100.append(reg_0100.write_line())

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0100'
            return mensagem

        def _registro0150():
            # print '-----------------------INI REG 0150'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """
            O campo cod_part é composto pelo cod_part mais o id do logradouro pelo fato de estar
            ocorrendo chave duplicada quando existe movimento para mais de um endereco
            Verificar a possibidilidade de criar metodo
            """
            mensagem = ''

            # Movimento de operadoras de cartao
            pessoa_ids = []
            for dado_oper in movimento_eletronico_pagamento:
                if dado_oper.get('pessoa_id') in pessoa_ids:
                    continue
                pessoa_ids.append(dado_oper.get('pessoa_id'))
                reg_0150.REG = '0150'
                reg_0150.COD_PART = dado_oper.get('pessoa_id') or ''
                reg_0150.NOME = dado_oper.get('operadora')[:99] or ''
                reg_0150.COD_PAIS = dado_oper.get('cod_pais') or '1058'
                if dado_oper.get('pessoa_tipo') == 'F':
                    reg_0150.CNPJ = ''
                    reg_0150.CPF = dado_oper.get('cpf_cnpj') or ''
                elif dado_oper.get('pessoa_tipo') == 'J':
                    reg_0150.CNPJ = dado_oper.get('cpf_cnpj') or ''
                    reg_0150.CPF = ''
                else:
                    reg_0150.CNPJ = ''
                    reg_0150.CPF = ''
                _ie = dado_oper.get('ie') or ''
                if 'ISENT' in str(_ie).strip().upper():
                    _ie = ''
                reg_0150.IE = self.formata_para_digitos(_ie) or ''
                reg_0150.COD_MUN = dado_oper.get('cod_mun') or ''
                reg_0150.SUFRAMA = dado_oper.get('suframa') or ''
                reg_0150.END = dado_oper.get('endr') or ''
                reg_0150.NUM = dado_oper.get('num') or ''
                reg_0150.COMPL = dado_oper.get('compl') or ''
                reg_0150.BAIRRO = dado_oper.get('bairro') or ''
                reg_0001.registro_0150.append(reg_0150.write_line())

            # Movimento de clientes
            if movimento_notas:
                filtro_0150 = {
                    'movimento_notas': movimento_notas,
                    'company_id': val['company_id']
                }
            else:
                filtro_0150 = {
                    'movimento_notas': [(-1)],
                    'company_id': False
                }

            # Adiciona as notas referenciadas ao movimento de notas para incluir os participantes
            sql = """
                SELECT DISTINCT nf_ref_id AS id
                FROM fiscal_nf_referencia
                WHERE id IN ({nf_ids})
            """.format(nf_ids=', '.join(map(lambda n: str(n), movimento_notas)) or '-1')
            cr.execute(sql)
            part_nota_referecia = cr.dictfetchall()
            movimento_notas_part = []
            (movimento_notas and part_nota_referecia) and movimento_notas_part.extend(
                map(lambda n: n.get('id'), part_nota_referecia))
            if movimento_notas_part and movimento_notas:
                filtro_0150['movimento_notas'] = movimento_notas + movimento_notas_part

            filtro_0150.update({'periodo': {'dt_ini': dt_ini, 'dt_fin': dt_fim}})
            dados = obj_sql.sql_reg0150(cr, filtro_0150)
            for dado in dados:
                if dado.get('nome'):
                    ie = dado.get('ie') or ''
                    if 'ISENT' in str(ie).strip().upper():
                        ie = ''
                    reg_0150.REG = '0150'
                    reg_0150.COD_PART = dado.get('cod_part') or ''
                    reg_0150.NOME = dado.get('nome')[:99] or ''
                    reg_0150.COD_PAIS = dado.get('cod_pais') or '1058'
                    reg_0150.CNPJ = dado.get('cnpj') or ''
                    reg_0150.CPF = dado.get('cpf') or ''
                    reg_0150.IE = self.formata_para_digitos(ie) or ''
                    reg_0150.COD_MUN = dado.get('cod_mun') or ''
                    reg_0150.SUFRAMA = dado.get('suframa') or ''
                    reg_0150.END = dado.get('logradouro') or ''
                    reg_0150.NUM = dado.get('num') or ''
                    reg_0150.COMPL = dado.get('compl') or ''
                    reg_0150.BAIRRO = dado.get('bairro') or ''
                    reg_0001.registro_0150.append(reg_0150.write_line())

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0150'
            return mensagem

        def _registro0190():
            # print '-----------------------INI REG 0190'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            movimento = {'movimento_notas': movimento_notas,
                         'movimento_cupons': movimento_cupons}
            filtro = self.filtra_ids_itens(cr, uid, obj_fiscal, movimento, reg_0000.IND_PERFIL,
                                           context)
            if val['inventario_id']:
                inventario_id = val['inventario_id'][0]
            else:
                inventario_id = False
            filtro.update({'inventario_id': inventario_id})
            dados = obj_sql.sql_reg0190(cr, filtro, obj_adm_utils.admutl_check_modulo_instalado(
                cr, uid, 'ecf', context))
            self.registro_0190_dados = dados
            for dado in dados:
                reg_0190.REG = '0190'
                reg_0190.UNID = dado.get('unid')
                reg_0190.DESCR = dado.get('descr')
                reg_0001.registro_0190.append(reg_0190.write_line())

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0190'
            return mensagem

        produto_ids_0205 = list()

        def _registro0200():
            # print '-----------------------INI REG 0200'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            if val['inventario_id']:
                inventario_id = val['inventario_id'][0]
            else:
                inventario_id = False
            movimento = {'movimento_notas': movimento_notas,
                         'movimento_cupons': movimento_cupons}
            filtro = {}
            if reg_0000.IND_PERFIL != 'C':
                filtro = self.filtra_ids_itens(cr, uid, obj_fiscal, movimento, reg_0000.IND_PERFIL,
                                               context)
            filtro.update({'inventario_id': inventario_id})
            dados = obj_sql.sql_reg0200(cr, filtro, reg_0000.IND_PERFIL)
            contador = Counter()
            codigo_barra_ids = list(set(map(lambda cb: cb.get('cod_barra_id'),
                                            filter(lambda r: r.get('cod_barra_id'), dados))))
            codigo_barra_dados = self.pool.get(
                'automacao.codigo.barra').autcba_get_codigo_barra_formatado_gtin(
                cr, uid, codigo_barra_ids, context)
            for dado in dados:
                if dado.get('cod_item') in list(contador.elements()):
                    continue
                contador[dado.get('cod_item')] += 1
                codigo_barra = codigo_barra_dados.get(dado.get('cod_barra_id')) or ''
                itens_movimentados.update({dado.get('grade_id'): dado.get('cod_item')})
                reg_0200.REG = '0200'
                reg_0200.COD_ITEM = dado.get('cod_item') or ''
                descri_item = obj_adm_utl.admutl_remove_caracter(dado.get('descr_item'),
                                                                 letras=True, numeros=True,
                                                                 espaco=True)
                reg_0200.DESCR_ITEM = descri_item or ''
                reg_0200.COD_BARRA = '' if codigo_barra == 'SEM GTIN' else codigo_barra
                reg_0200.COD_ANT_ITEM = dado.get('cod_ant_item') or ''
                reg_0200.UNID_INV = dado.get('unid_inv') or ''
                reg_0200.TIPO_ITEM = dado.get('tipo_item') or ''
                reg_0200.COD_NCM = dado.get('cod_ncm') or ''
                reg_0200.EX_IPI = dado.get('ex_ipi') or ''
                reg_0200.COD_GEN = dado.get('cod_gen') or ''
                reg_0200.COD_LST = dado.get('cod_lst') or ''
                reg_0200.ALIQ_ICMS = dado.get('aliq_icms') or ''
                if cod_versao > '011':
                    reg_0200.CEST = dado.get('cest') or ''
                reg_0001.registro_0200.append(reg_0200.write_line())
                if dado.get('produto_id') not in produto_ids_0205:
                    _registro0205(dado.get('produto_id'), val.get('sped_data_inicial'),
                                  val.get('sped_data_final'))
                    produto_ids_0205.append(dado.get('produto_id'))

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0200'
            return mensagem, filtro

        def _registro0205(produto_id, sped_data_inicial, sped_data_final):
            # print '-----------------------INI REG 0205'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            dados = obj_sql.sql_reg0205(cr, produto_id, sped_data_inicial, sped_data_final)
            if not dados:
                return mensagem
            reg_0205.REG = '0205'
            reg_0205.DESCR_ANT_ITEM = dados.get('descr_ant_item')
            reg_0205.DT_INI = ''
            if dados.get('dt_ini'):
                reg_0205.DT_INI = parse(dados.get('dt_ini')).strftime('%d%m%Y')
            reg_0205.DT_FIM = parse(dados.get('dt_fim')).strftime('%d%m%Y')
            reg_0205.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0205'
            return mensagem

        def _registro0400():
            # print '-----------------------INI REG 0400'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            if reg_0000.IND_PERFIL != 'C':
                modelo_nf_ids = self.pool.get('fiscal.modelo.nf').search(
                    cr, uid, [("fismnf_codigo", "in", ('01', '1B', '04', '55'))],
                    context=context)
                args_movimento = [('nf_propria', '=', True),
                                  ('modelo_55', '=', False),
                                  ('id', 'in', movimento_notas),
                                  ('modelo_nf_id', 'in', modelo_nf_ids)]
                mov_modelo_01_ids = obj_fiscal_nf_capa.search(cr, uid, args_movimento,
                                                              context=context)
                if movimento_notas:
                    filtros_0400 = {
                        'movimento_notas': movimento_notas
                    }
                    if mov_modelo_01_ids:
                        filtros_0400['mov_modelo_01_ids'] = mov_modelo_01_ids
                    dados = obj_sql.sql_reg0400(cr, filtros_0400)
                    for dado in dados:
                        reg_0400.REG = '0400'
                        reg_0400.COD_NAT = dado.get('cod_nat') or ''
                        reg_0400.DESCR_NAT = dado.get('descr_nat') or ''
                        reg_0001.registro_0400.append(reg_0400.write_line())

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0400'
            return mensagem

        def _registro0450():
            # print '-----------------------INI REG 0450'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            observacoes = {}
            if movimento_notas:
                filtros_0450 = {
                    'movimento_notas': movimento_notas
                }
                dados = obj_sql.sql_reg0450(cr, filtros_0450)
                for dado in dados:
                    txt = obj_adm_utl.admutl_remove_caracter(obj_adm_utl.admutl_remover_acentuacao(
                        cr, False, dado.get('txt')), letras=True, numeros=True, espaco=True)
                    if txt and not txt.isspace():
                        txt = txt[0:255]
                        reg_0450.REG = '0450'
                        reg_0450.COD_INF = dado.get('cod_inf') or ''
                        reg_0450.TXT = txt or ''
                        reg_0001.registro_0450.append(reg_0450.write_line())
                        observacoes.update({dado.get('cod_inf'): txt})

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0450'
            return observacoes

        def _registro0460():
            # print '-----------------------INI REG 0460'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """
            Verifica no movimento passado como parametro do sql quando existe
            notas Substituicao Tributaria
            """
            mensagem = ''
            if movimento_notas:
                filtros_0460 = {
                    'movimento_notas': movimento_notas
                }
                dados = obj_sql.sql_reg0460(cr, filtros_0460)
                for dado in dados:
                    txt = obj_adm_utl.admutl_remove_caracter(obj_adm_utl.admutl_remover_acentuacao(
                        cr, False, dado.get('txt')), letras=True, numeros=True, espaco=True)
                    if txt and not txt.isspace():
                        reg_0460 = bloco_0.registro_0460()
                        reg_0460.REG = '0460'
                        reg_0460.COD_OBS = dado.get('cod_obs')
                        reg_0460.TXT = txt[0:255]
                        lista_0460.append(reg_0460)
                        reg_0460.write_line()

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 0460'
            return mensagem

        # Dados do Bloco B
        def _registroB001():
            reg_B001.REG = 'B001'
            reg_B001.IND_DAD = '1'
            reg_B001.write_line()
            return ''

        # Dados do Bloco C
        def _registroC001():
            movimentoC001 = False
            reg_C001.REG = 'C001'
            reg_C001.IND_MOV = '1'
            if not movimento_cupons and not movimento_notas:
                reg_C001.write_line()
            else:
                movimentoC001 = True

            return movimentoC001

        def _get_vl_nf_cupom(cfop=False, modelo=False, valor=0.00):
            if cfop in ('5929', '6929') and modelo == '55':
                valor = '0,00'
            else:
                valor = self.formata_numero(valor > 0 and valor or 0, size=1, separador=',')

            return valor

        def _registroC100():
            # print '-----------------------INI REG C100'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            """
            Registro C100 você escritura o valor total do documento e valor total da mercadoria como
            está na nota. No Registro C170 é a quantidade x valor do ítem, ou seja não soma o icms
            st. Esse só vai compor o valor total da operação no Registro C190,
            """
            global total_geral_creditos_C190
            global total_geral_debitos_C190
            total_geral_creditos_C190 = 0.0
            total_geral_debitos_C190 = 0.0
            mensagem = ''
            obj_nf = self.pool.get('fiscal.nf.capa')
            filho_dados = {}
            if movimento_notas:
                def _limpaC100():
                    reg_C100.REG = 'C100'
                    reg_C100.IND_OPER = ''
                    reg_C100.IND_EMIT = ''
                    reg_C100.COD_PART = ''
                    reg_C100.COD_MOD = ''
                    reg_C100.COD_SIT = ''
                    reg_C100.SER = ''
                    reg_C100.NUM_DOC = ''
                    reg_C100.CHV_NFE = ''
                    reg_C100.DT_DOC = ''
                    reg_C100.DT_E_S = ''
                    reg_C100.VL_DOC = ''
                    reg_C100.IND_PGTO = ''
                    reg_C100.VL_DESC = ''
                    reg_C100.VL_ABAT_NT = ''
                    reg_C100.VL_MERC = ''
                    reg_C100.IND_FRT = ''
                    reg_C100.VL_FRT = ''
                    reg_C100.VL_SEG = ''
                    reg_C100.VL_OUT_DA = ''
                    reg_C100.VL_BC_ICMS = ''
                    reg_C100.VL_ICMS = ''
                    reg_C100.VL_BC_ICMS_ST = ''
                    reg_C100.VL_ICMS_ST = ''
                    reg_C100.VL_IPI = ''
                    reg_C100.VL_PIS = ''
                    reg_C100.VL_COFINS = ''
                    reg_C100.VL_PIS_ST = ''
                    reg_C100.VL_COFINS_ST = ''

                tipo_empresa = read_pessoa.admpes_jur_tipo_atividade
                filtros_C100 = {
                    'movimento_notas': movimento_notas,
                    'tipo_empresa': tipo_empresa
                }
                dados = obj_sql.sql_regC100(cr, filtros_C100)
                dados_c100_descontos = obj_sql.sql_reg_C100_descontos(cr, movimento_notas)
                dados_c101 = obj_sql.sql_regC101(cr, movimento_notas)
                dados_c113 = obj_sql.sql_regC113(cr, movimento_notas)
                dados_c140 = obj_sql.sql_regC140(cr, movimento_notas)
                fatura_ids = list(set(map(lambda f: f.get('fatura_id'), dados_c140.values())))
                dados_c141 = fatura_ids and obj_sql.sql_regC141(cr, fatura_ids) or {}
                dados_c170 = obj_sql.sql_regC170(cr, movimento_notas)
                dados_c172 = obj_sql.sql_regC172(cr, movimento_notas)
                dados_c181 = {}
                dados_c186 = {}
                if int(cod_versao) >= 15:
                    movimento_notas_devolucao = map(lambda nd: nd.get('id'), filter(
                        lambda ndv: ndv.get('cod_mod') != '65', dados))
                    dados_c181 = obj_sql.sql_regC181(cr, {'nf_ids': movimento_notas_devolucao})
                    dados_c186 = obj_sql.sql_regC186(cr, {'nf_ids': movimento_notas_devolucao})
                    filho_dados.update({'c181': dados_c181})
                dados_c190 = obj_sql.sql_regC190(cr, movimento_notas)
                for dado in dados:
                    if reg_C001.IND_MOV == '1':
                        reg_C001.IND_MOV = '0'
                        reg_C001.write_line()
                    cfop = dado.get('cfop')
                    modelo = dado.get('cod_mod')
                    vl_desc = dados_c100_descontos.get(dado.get('id'))
                    vl_bc_icms_st = 0.00
                    vl_icms_st = 0.00
                    vl_merc = dado.get('vl_merc') or 0.0
                    vr_ipi = dado.get('vl_ipi') or 0.0
                    if read_pessoa.admpes_jur_tipo_atividade == 'I':
                        vl_bc_icms_st = dado.get('vl_bc_icms_st')
                        vl_icms_st = dado.get('vl_icms_st')
                    else:
                        vl_merc += dado.get('vl_icms_st') or 0.0
                    if read_pessoa.admpes_jur_tipo_atividade == 'C':
                        vl_merc += vr_ipi

                    _limpaC100()
                    reg_C100.REG = 'C100'
                    if dado.get('cod_sit') in ('02', '05', '04'):
                        reg_C100.IND_OPER = dado.get('ind_oper') or ''
                        reg_C100.IND_EMIT = dado.get('ind_emit') or ''
                        reg_C100.COD_MOD = dado.get('cod_mod') or ''
                        reg_C100.COD_SIT = dado.get('cod_sit') or ''
                        reg_C100.SER = dado.get('ser') or ''
                        reg_C100.NUM_DOC = dado.get('num_doc') or ''
                        if dado.get('cod_sit') == '05':
                            reg_C100.CHV_NFE = ''
                        else:
                            reg_C100.CHV_NFE = dado.get('chv_nfe') or ''
                        reg_C100.write_line()
                    else:
                        reg_C100.IND_OPER = dado.get('ind_oper') or ''
                        reg_C100.IND_EMIT = dado.get('ind_emit') or ''
                        reg_C100.COD_PART = dado.get('cod_part') or ''
                        reg_C100.COD_MOD = dado.get('cod_mod') or ''
                        # Nota fiscal de complemento Complemento
                        if dado.get('finalidade_emissao') == '2':
                            reg_C100.COD_SIT = '06'
                        else:
                            reg_C100.COD_SIT = dado.get('cod_sit') or ''
                        reg_C100.SER = dado.get('ser') or ''
                        reg_C100.NUM_DOC = dado.get('num_doc') or ''
                        reg_C100.CHV_NFE = dado.get('chv_nfe') or ''
                        reg_C100.DT_DOC = self.formata_data(dado.get('dt_doc')) \
                            if dado.get('dt_doc') else ''
                        reg_C100.DT_E_S = self.formata_data(dado.get('dt_e_s')) \
                            if dado.get('dt_e_s') else ''
                        reg_C100.VL_DOC = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_doc'))
                        reg_C100.IND_PGTO = dado.get('ind_pgto') or ''
                        reg_C100.VL_DESC = _get_vl_nf_cupom(cfop, modelo, vl_desc)
                        reg_C100.VL_ABAT_NT = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_abat_nt'))
                        reg_C100.VL_MERC = _get_vl_nf_cupom(cfop, modelo, vl_merc)
                        reg_C100.IND_FRT = dado.get('ind_frt') or ''
                        reg_C100.VL_FRT = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_frt'))
                        reg_C100.VL_SEG = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_seg'))
                        reg_C100.VL_OUT_DA = _get_vl_nf_cupom(cfop, modelo,
                                                              dado.get('vl_out_da'))
                        reg_C100.VL_BC_ICMS = _get_vl_nf_cupom(cfop, modelo,
                                                               dado.get('vl_bc_icms'))
                        reg_C100.VL_ICMS = _get_vl_nf_cupom(cfop, modelo,
                                                            dado.get('vl_icms'))
                        reg_C100.VL_BC_ICMS_ST = '' if modelo == '65' else _get_vl_nf_cupom(
                            cfop, modelo, vl_bc_icms_st)
                        reg_C100.VL_ICMS_ST = '' if modelo == '65' else _get_vl_nf_cupom(
                            cfop, modelo, vl_icms_st)
                        if modelo == '65':
                            reg_C100.VL_IPI = ''
                        elif read_pessoa.admpes_jur_tipo_atividade == "C":
                            reg_C100.VL_IPI = '0,00'
                        else:
                            reg_C100.VL_IPI = _get_vl_nf_cupom(cfop, modelo,
                                                               dado.get('vl_ipi'))
                        reg_C100.VL_PIS = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_pis')) \
                            if (gerar_pis_cofins and modelo != '65') else ''
                        reg_C100.VL_COFINS = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_cofins')) \
                            if (gerar_pis_cofins and modelo != '65') else ''
                        reg_C100.VL_PIS_ST = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_pis_st')) \
                            if (gerar_pis_cofins and modelo != '65') else ''
                        reg_C100.VL_COFINS_ST = _get_vl_nf_cupom(
                            cfop, modelo, dado.get('vl_cofins_st')) \
                            if (gerar_pis_cofins and modelo != '65') else ''
                        reg_C100.write_line()
                        # Chama metodo _registroC110
                        if reg_0000.IND_PERFIL != 'C' and modelo != '65':
                            _registroC101(dados_c101.get(dado.get('id')) or {})
                            if dado.get('txt'):
                                txt = u'%s' % unicode(dado.get('txt')).replace('\n', '').replace(
                                    '|', '').replace("\"", "")[0:255]
                                if txt:
                                    dados_C110 = {
                                        'REG': 'C110',
                                        'COD_INF': str(dado.get('id'))[-6:],
                                        'TXT_COMPL': txt
                                    }
                                    _registroC110(dados_C110)

                                    _registroC113(dados_c113.get(dado.get('id')) or [])

                            # Chama metodo _registroC114
                            if dado.get('cod_sit') == '08':
                                self.registroC114(cr, uid, dado.get('id'), context)

                            # Chama metodo _registroC140
                            if dado.get('cod_mod') == '01' and dados_c140.get(dado.get('id')):
                                _registroC140(dados_c140.get(dado.get('id')), dados_c141)

                            # Chama metodo _registroC170 se nota não e de entrada própria
                            if not dado.get('nf_propria'):
                                _registroC170(dados_c170.get(dado.get('id')) or [])
                                # Chama metodo _registroC172 se tipo = 1
                                if dado.get('tipo' == '1'):
                                    _registroC172(dados_c172.get(dado.get('id')))
                                _registroC181(dados_c181.get(dado.get('id')) or [])
                                _registroC186(dados_c186.get(dado.get('id')) or [])
                            else:
                                if dado.get('cod_mod') not in ('55', '65'):
                                    _registroC170(dados_c170.get(dado.get('id')))
                                    # Chama metodo _registroC172 se tipo = 1
                                    if dado.get('tipo' == '1'):
                                        _registroC172(dados_c172.get(dado.get('id')))
                                    _registroC181(dados_c181.get(dado.get('id')) or [])
                                    _registroC186(dados_c186.get(dado.get('id')) or [])

                        # Chama metodo _ registroC190
                        if dado.get('tipo') == 'E':
                            total_geral_creditos_C190 += _registroC190(
                                dados_c190.get(dado.get('id')) or [])
                        else:
                            total_geral_debitos_C190 += _registroC190(
                                dados_c190.get(dado.get('id')) or [], modelo)
                        _registroC195(dado.get('id'))

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C100'
            return mensagem, filho_dados

        def _registroC101(dado_c101):
            # print '-----------------------INI REG C101'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            if dado_c101.get('vl_icms_uf_dest') > 0.00 and reg_C100.IND_OPER == '1':
                reg_C101.REG = 'C101'
                reg_C101.VL_FCP_UF_DEST = '0,00'
                reg_C101.VL_ICMS_UF_DEST = self.formata_numero(dado_c101.get('vl_icms_uf_dest'),
                                                               size=1,
                                                               separador=',')
                reg_C101.VL_ICMS_UF_REM = self.formata_numero(dado_c101.get('vl_icms_uf_rem'),
                                                              size=1,
                                                              separador=',')
                apuracao_difal.append({reg_0000.UF: dado_c101.get('vl_icms_uf_rem')})
                apuracao_difal.append({dado_c101.get('uf'): dado_c101.get('vl_icms_uf_dest')})
                reg_C101.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C101'

        def _registroC110(dados):
            # print '-----------------------INI REG C110'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            txt = obj_adm_utl.admutl_remove_caracter(obj_adm_utl.admutl_remover_acentuacao(
                        cr, False, dados.get('TXT_COMPL')), letras=True, numeros=True, espaco=True)
            if txt and not txt.isspace():
                reg_C110.REG = dados.get('REG')
                reg_C110.COD_INF = dados.get('COD_INF')
                reg_C110.TXT_COMPL = txt[0:255]
                reg_C110.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C110'

        def _registroC113(dados_c113):
            # print '-----------------------INI REG C113'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            for dado in dados_c113:
                reg_C113.REG = 'C113'
                reg_C113.IND_OPER = dado.get('ind_oper') or ''
                reg_C113.IND_EMIT = dado.get('ind_emit') or ''
                reg_C113.COD_PART = dado.get('cod_part') or ''
                reg_C113.COD_MOD = dado.get('cod_mod') or ''
                reg_C113.SER = dado.get('ser') or ''
                reg_C113.SUB = dado.get('sub') or ''
                reg_C113.NUM_DOC = dado.get('num_doc') or ''
                reg_C113.DT_DOC = self.formata_data(dado.get('dt_doc')) if dado.get(
                    'dt_doc') else ''
                reg_C113.CHV_DOCe = dado.get('chv_doce') or ''
                reg_C113.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C113'

        def _registroC140(dado_c140, dado_c141):
            # print '-----------------------INI REG C140'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            desc_tit = obj_adm_utl.admutl_remove_caracter(dado_c140.get('desc_tit'), letras=True,
                                                          numeros=True, espaco=True)
            reg_C140.IND_EMIT = dado_c140.get('ind_emit') or ''
            reg_C140.IND_TIT = dado_c140.get('ind_tit') or ''
            reg_C140.DESC_TIT = desc_tit
            reg_C140.NUM_TIT = dado_c140.get('num_tit') or ''
            reg_C140.QTD_PARC = dado_c140.get('qtd_parc') or ''
            reg_C140.VL_TIT = self.formata_numero(dado_c140.get('vl_tit'), size=1, separador=',')
            reg_C140.write_line()

            # Chama registro C141
            dado_c141.get(dado_c140.get('fatura_id')) and _registroC141(dado_c141.get(dado_c140.get(
                'fatura_id')))
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C140'

        def _registroC141(dados_c141):
            for dado in dados_c141:
                reg_C141.NUM_PARC = 1 if dado.get('num_parc') == 0 else dado.get('num_parc') or '0'
                reg_C141.DT_VCTO = self.formata_data(dado.get('dt_vcto'))
                reg_C141.VL_PARC = self.formata_numero(dado.get('vl_parc'), size=1, separador=',')
                reg_C141.write_line()

        def _registroC170(dados_c170):
            # print '-----------------------INI REG C170'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # dados = sorted(dados_c170, key=lambda c: c.get('sequencia'))
            cod_cta_nf = ''
            if dados_c170:
                cod_cta_nf = self.sped_get_conta_contabil(
                    cr, uid, nf_id=dados_c170[0].get('nf_id'), context=context)
            for dado in dados_c170:
                vl_bc_icms_st = 0.00
                vl_icms_st = 0.00
                vl_bc_icms = dado.get('vl_bc_icms')
                vl_icms = dado.get('vl_icms')
                cst = dado.get('cst_icms')
                if cst in ('10', '30', '40', '60', '70', '90'):
                    if not read_pessoa.admpes_jur_tipo_atividade == 'I':
                        vl_bc_icms = 0.00
                        vl_icms = 0.00

                vl_item = dado.get('vl_item') or 0.0
                if read_pessoa.admpes_jur_tipo_atividade == 'I':
                    vl_bc_icms_st = dado.get('vl_bc_icms_st')
                    vl_icms_st = dado.get('vl_icms_st')
                else:
                    vl_item += dado.get('vl_icms_st') or 0

                reg_C170.REG = 'C170'
                reg_C170.NUM_ITEM = dado.get('sequencia') or ''
                reg_C170.COD_ITEM = dado.get('cod_item') or ''
                descr_compl = ''
                if dado.get('descr_compl'):
                    descr_compl = obj_adm_utl.admutl_remove_caracter(dado.get('descr_compl'),
                                                                     letras=True, numeros=True,
                                                                     espaco=True)
                reg_C170.DESCR_COMPL = descr_compl or ''
                reg_C170.QTD = self.formata_numero(dado.get('qtd'), size=1, separador=',',
                                                   dec=3) if dado.get(
                    'qtd') else ''
                reg_C170.UNID = dado.get('unid') or ''
                reg_C170.VL_ITEM = self.formata_numero(vl_item, size=1,
                                                       separador=',')
                reg_C170.VL_DESC = self.formata_numero(dado.get('vl_desc'), size=1,
                                                       separador=',')
                reg_C170.IND_MOV = '0'
                reg_C170.CST_ICMS = '0' + dado.get('cst_icms') if dado.get(
                    'cst_icms') else ''
                reg_C170.CFOP = dado.get('cfop') or ''
                reg_C170.COD_NAT = dado.get(
                    'cod_nat') or ''
                reg_C170.VL_BC_ICMS = self.formata_numero(vl_bc_icms, size=1,
                                                          separador=',')
                reg_C170.ALIQ_ICMS = self.formata_numero(dado.get('aliq_icms'), size=1,
                                                         separador=',')
                reg_C170.VL_ICMS = self.formata_numero(vl_icms, size=1,
                                                       separador=',')
                reg_C170.VL_BC_ICMS_ST = self.formata_numero(vl_bc_icms_st, size=1,
                                                             separador=',')
                reg_C170.ALIQ_ST = self.formata_numero(dado.get('aliq_st'), size=1,
                                                       separador=',')
                reg_C170.VL_ICMS_ST = self.formata_numero(vl_icms_st, size=1,
                                                          separador=',')
                reg_C170.IND_APUR = '0'
                reg_C170.CST_IPI = dado.get('cst_ipi') or ''
                reg_C170.COD_ENQ = ''
                reg_C170.VL_BC_IPI = self.formata_numero(dado.get('vl_bc_ipi'), size=1,
                                                         separador=',')
                reg_C170.ALIQ_IPI = self.formata_numero(dado.get('aliq_ipi'), size=1,
                                                        separador=',')
                reg_C170.VL_IPI = self.formata_numero(dado.get('vl_ipi'), size=1,
                                                      separador=',')
                reg_C170.CST_PIS = (dado.get('cst_pis') or '') if gerar_pis_cofins else ''
                reg_C170.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=1, separador=',') if gerar_pis_cofins else ''
                reg_C170.ALIQ_PIS = self.formata_numero(
                    dado.get('aliq_pis'), size=1, separador=',') if gerar_pis_cofins else ''
                reg_C170.QUANT_BC_PIS = ''
                reg_C170.VL_PIS = self.formata_numero(
                    dado.get('vl_pis'), size=1, separador=',') if gerar_pis_cofins else ''
                reg_C170.CST_COFINS = (dado.get('cst_cofins') or '') if gerar_pis_cofins else ''
                reg_C170.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=1, separador=',') if gerar_pis_cofins else ''
                reg_C170.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',') if gerar_pis_cofins else ''
                reg_C170.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=1, separador=',') if gerar_pis_cofins else ''
                reg_C170.COD_CTA = self.sped_get_conta_contabil(
                    cr, uid, nf_id=dado.get('nf_id'),
                    movimento_item_id=dado.get('movimento_item_id'),
                    context=context) or cod_cta_nf
                if cod_versao and int(cod_versao) >= 13:
                    reg_C170.VL_ABAT_NT = self.formata_numero(dado.get('vl_abat_nt'), size=1,
                                                              separador=',')
                reg_C170.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C170'

        def _registroC172(dados_c172):
            # print '-----------------------INI REG C172'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            for dado in dados_c172:
                reg_C172.REG = 'C172'
                reg_C172.VL_BC_ISSQN = self.formata_numero(dado.get('vl_bc_issqn'), size=1,
                                                           separador=',')
                reg_C172.ALIQ_ISSQN = self.formata_numero(dado.get('aliq_issqn'), size=1,
                                                          separador=',')
                reg_C172.VL_ISSQN = self.formata_numero(dado.get('vl_issqn'), size=1,
                                                        separador=',')
                reg_C172.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C172'

        def _registroC181(dados_c181):
            for dado in dados_c181:
                reg_C181.REG = 'C181'
                reg_C181.COD_MOT_REST_COMPL = dado.get('cod_mot_rest_compl')
                reg_C181.QUANT_CONV = self.formata_numero(dado.get('quant_conv'), size=1,
                                                          separador=',')
                reg_C181.UNID = dado.get('unid')
                reg_C181.COD_MOD_SAIDA = dado.get('cod_mod_saida') or ''
                reg_C181.SERIE_SAIDA = dado.get('cod_mod_saida') == '01' and dado.get(
                    'serie_saida') or ''
                reg_C181.ECF_FAB_SAIDA = ''
                reg_C181.NUM_DOC_SAIDA = dado.get('cod_mod_saida') == '01' and dado.get(
                    'num_doc_saida') or ''
                reg_C181.CHV_DFE_SAIDA = dado.get('chv_dfe_saida') or ''
                reg_C181.DT_DOC_SAIDA = dado.get('dt_doc_saida') and parse(dado.get(
                    'dt_doc_saida')).strftime('%d%m%Y') or ''
                reg_C181.NUM_ITEM_SAIDA = str(dado.get('num_item_saida') or '')
                reg_C181.VL_UNIT_CONV_SAIDA = ''
                # reg_C181.VL_UNIT_CONV_SAIDA = self.formata_numero(
                #     dado.get('vl_unit_conv_saida') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_ICMS_OP_ESTOQUE_CONV_SAIDA = ''
                # reg_C181.VL_UNIT_ICMS_OP_ESTOQUE_CONV_SAIDA = self.formata_numero(
                #     dado.get('vl_unit_icms_op_estoque_conv_saida') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_ICMS_ST_ESTOQUE_CONV_SAIDA = ''
                # reg_C181.VL_UNIT_ICMS_ST_ESTOQUE_CONV_SAIDA = self.formata_numero(
                #     dado.get('vl_unit_icms_st_estoque_conv_saida') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_FCP_ICMS_ST_ESTOQUE_CONV_SAIDA = ''
                # reg_C181.VL_UNIT_FCP_ICMS_ST_ESTOQUE_CONV_SAIDA = self.formata_numero(
                #     dado.get('vl_unit_fcp_st_estoque_conv_saida') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_ICMS_NA_OPERACAO_CONV_SAIDA = ''
                # reg_C181.VL_UNIT_ICMS_NA_OPERACAO_CONV_SAIDA = self.formata_numero(
                #     dado.get('vl_unit_icms_na_operacao_conv_saida') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_ICMS_OP_CONV_SAIDA = ''
                # reg_C181.VL_UNIT_ICMS_OP_CONV_SAIDA = self.formata_numero(
                #     dado.get('vl_unit_icms_op_conv_saida') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_ICMS_ST_CONV_REST = ''
                # reg_C181.VL_UNIT_ICMS_ST_CONV_REST = self.formata_numero(
                #     dado.get('vl_unit_icms_st_conv_rest') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_FCP_ST_CONV_REST = ''
                # reg_C181.VL_UNIT_FCP_ST_CONV_REST = self.formata_numero(
                #     dado.get('vl_unit_fcp_st_conv_rest') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_ICMS_ST_CONV_COMPL = ''
                # reg_C181.VL_UNIT_ICMS_ST_CONV_COMPL = self.formata_numero(
                #     dado.get('vl_unit_icms_st_conv_compl') or 0, size=1, separador=',')
                reg_C181.VL_UNIT_FCP_ST_CONV_COMPL = ''
                # reg_C181.VL_UNIT_FCP_ST_CONV_COMPL = self.formata_numero(
                #     dado.get('vl_unit_fcp_st_conv_compl') or 0, size=1, separador=',')
                reg_C181.write_line()

        def _registroC186(dados_c186):
            for dado in dados_c186:
                reg_C186.REG = 'C186'
                reg_C186.NUM_ITEM = str(dado.get('num_item') or '')
                reg_C186.COD_ITEM = dado.get('cod_item') or ''
                reg_C186.CST_ICMS = dado.get('cst_icms') and dado.get('cst_icms').zfill(3) or ''
                reg_C186.CFOP = dado.get('cfop') or ''
                reg_C186.COD_MOT_REST_COMPL = dado.get('cod_mot_rest_compl')
                reg_C186.QUANT_CONV = self.formata_numero(dado.get('quant_conv') or 0, size=1,
                                                          separador=',')
                reg_C186.UNID = dado.get('unid') or ''
                reg_C186.COD_MOD_ENTRADA = dado.get('cod_mod_entrada') or ''
                reg_C186.SERIE_ENTRADA = dado.get('cod_mod_entrada') != '55' and dado.get(
                    'serie_entrada') or ''
                reg_C186.NUM_DOC_ENTRADA = dado.get('cod_mod_entrada') != '55' and dado.get(
                    'num_doc_entrada') or ''
                reg_C186.CHV_DFE_ENTRADA = dado.get('chv_dfe_entrada') or ''
                reg_C186.DT_DOC_ENTRADA = dado.get('dt_doc_entrada') and parse(dado.get(
                    'dt_doc_entrada')).strftime('%d%m%Y') or ''
                reg_C186.NUM_ITEM_ENTRADA = str(dado.get('num_item_entrada') or '')
                reg_C186.VL_UNIT_CONV_ENTRADA = self.formata_numero(
                    dado.get('vl_unit_conv_entrada') or 0, size=1, separador=',')
                reg_C186.VL_UNIT_ICMS_OP_CONV_ENTRADA = self.formata_numero(
                    dado.get('vl_unit_icms_op_conv_entrada') or 0, size=1, separador=',')
                reg_C186.VL_UNIT_BC_ICMS_ST_CONV_ENTRADA = self.formata_numero(
                    dado.get('vl_unit_bc_icms_st_conv_entrada') or 0, size=1, separador=',')
                reg_C186.VL_UNIT_ICMS_ST_CONV_ENTRADA = self.formata_numero(
                    dado.get('vl_unit_icms_st_conv_entrada') or 0, size=1, separador=',')
                reg_C186.VL_UNIT_FCP_ST_CONV_ENTRADA = self.formata_numero(
                    dado.get('vl_unit_fcp_st_conv_entrada') or 0, size=1, separador=',')
                reg_C186.write_line()

        def _registroC190(dados_c190, modelo=False):
            # print '-----------------------INI REG C190'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            total_vlicms = 0
            cfop_temp = ''
            aliq_icms_temp = ''
            cst_id_temp = False
            for dado in dados_c190:
                cfop = dado.get('cfop')
                aliq_icms = dado.get('aliq_icms')
                cst_id = dado.get('cst_id')
                vr_ipi = 0.00
                if (cfop != cfop_temp) or (aliq_icms != aliq_icms_temp) or (cst_id != cst_id_temp):
                    dado_ipi = obj_sql.sql_get_ipi(cr, dado.get('nf_id'), cfop, aliq_icms,
                                                   dado.get('cst_id'))
                    for ipi in dado_ipi:
                        vr_ipi += ipi.get('vr_ipi')
                    cfop_temp = cfop
                    aliq_icms_temp = aliq_icms
                    cst_id_temp = cst_id
                vl_bc_icms_st = 0.00
                vl_icms_st = 0.00
                vl_bc_icms = dado.get('vl_bc_icms') or 0.00
                vl_icms = dado.get('vl_icms') if cfop not in ('5929', '6929') else 0.00
                cst = dado.get('cst_icms')
                if cst in ('10', '30', '40', '41', '50','51', '60', '70', '90'):
                    vl_bc_icms = 0.00
                    vl_icms = 0.00
                if read_pessoa.admpes_jur_tipo_atividade == 'I':
                    vl_bc_icms_st = dado.get('vl_bc_icms_st') or 0.00
                    vl_icms_st = dado.get('vl_icms_st') or 0.00
                reg_C190.REG = 'C190'
                reg_C190.CST_ICMS = '0' + dado.get('cst_icms') if dado.get('cst_icms') else ''
                reg_C190.CFOP = dado.get('cfop') or ''
                reg_C190.ALIQ_ICMS = self.formata_numero(dado.get('aliq_icms'), size=1,
                                                         separador=',')
                vl_opr = dado.get('vl_opr') or 0.00
                vl_opr += vr_ipi
                reg_C190.VL_OPR = _get_vl_nf_cupom(cfop, modelo, vl_opr)
                reg_C190.VL_BC_ICMS = _get_vl_nf_cupom(cfop, modelo, vl_bc_icms)
                reg_C190.VL_ICMS = _get_vl_nf_cupom(cfop, modelo, vl_icms)
                reg_C190.VL_BC_ICMS_ST = _get_vl_nf_cupom(cfop, modelo, vl_bc_icms_st)
                reg_C190.VL_ICMS_ST = _get_vl_nf_cupom(cfop, modelo, vl_icms_st)
                reg_C190.VL_RED_BC = _get_vl_nf_cupom(cfop, modelo,
                                                      dado.get('vl_red_bc') + vr_ipi) or ''
                if read_pessoa.admpes_jur_tipo_atividade == "C":
                    reg_C190.VL_IPI = '0,00'
                else:
                    reg_C190.VL_IPI = _get_vl_nf_cupom(cfop, modelo, vr_ipi)
                for reg_0460 in filter(lambda i: i.COD_OBS == str(dado.get('nf_id'))[-6:],
                                       lista_0460):
                    reg_C190.COD_OBS = reg_0460.COD_OBS
                reg_C190.write_line()
                total_vlicms += vl_icms

            # print '-----------------------FIM REG C190'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            return total_vlicms

        def _registroC195(nf_id):
            # print '-----------------------INI REG C195'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            for reg0460 in filter(lambda i: i.COD_OBS == str(nf_id)[-6:], lista_0460):
                reg_C195.COD_OBS = reg0460.COD_OBS
                reg_C195.write_line()
                _registroC197(nf_id)
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C195'

        def _registroC197(nf_id):
            # print '-----------------------INI REG C197'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            itens = filter(
                lambda i: i, self.pool.get('fiscal.nf.capa').browse(
                    cr, uid, nf_id, context=context).estoque_movimento_itens_ids)
            chaves = {}
            for item in itens:
                for ajuste in item.ajuste_fiscal_ids:
                    if not item.nf_id.fisnfc_valor_fcp_st or item.nf_id.nf_propria:
                        ajustes_creditos.append(ajuste.fisaji_valor_ajuste or 0.0)
                    codigo_ajuste = ajuste.fisaji_codigo_ajuste.zfill(10)
                    codigo_item = itens_movimentados.get(
                        ajuste.fisaji_movimento_item_id.grade_id.id) or ''
                    chave = codigo_ajuste + '/' + codigo_item
                    if chave in chaves:
                        chaves[chave]['vl_bc_icms'] += (ajuste.fisaji_valor_base_ajuste or 0.0)
                        chaves[chave]['aliq_icms'] += (ajuste.fisaji_aliquota_ajuste or 0.0)
                        chaves[chave]['vl_icms'] += (ajuste.fisaji_valor_ajuste or 0.0)
                        chaves[chave]['vl_outros'] += (ajuste.fisaji_valor_outros or 0.0)
                    else:
                        chaves.update({chave: {'vl_bc_icms': ajuste.fisaji_valor_base_ajuste or 0.0,
                                               'aliq_icms': ajuste.fisaji_aliquota_ajuste or 0.0,
                                               'vl_icms': ajuste.fisaji_valor_ajuste,
                                               'vl_outros': ajuste.fisaji_valor_outros or 0.0}})
            for ch in chaves:
                reg_C197.COD_AJ = ch.split('/')[0]
                reg_C197.COD_ITEM = ch.split('/')[1]
                reg_C197.VL_BC_ICMS = self.formata_numero(chaves.get(ch).get('vl_bc_icms'),
                                                          size=1, separador=',')
                reg_C197.ALIQ_ICMS = self.formata_numero(
                    chaves.get(ch).get('aliq_icms'), size=1, separador=',')
                reg_C197.VL_ICMS = self.formata_numero(chaves.get(ch).get('vl_icms'), size=1,
                                                       separador=',')
                reg_C197.VL_OUTROS = self.formata_numero(chaves.get(ch).get('vl_outros'), size=1,
                                                         separador=',')
                reg_C197.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C197'

        def _registroC300():
            # print '-----------------------INI REG C300'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            global total_geral_debitosC320
            total_geral_debitosC320 = 0.00
            filtro = {
                'data_ini': data_ini,
                'data_fin': data_fim
            }
            mov_2d = obj_sql.sql_consulta_mov_modelo_2d(cr, filtro)
            if mov_2d:
                dados = obj_sql.sql_regC300(cr, mov_2d)
                for dado in dados:
                    if reg_C001.IND_MOV == '1':
                        reg_C001.IND_MOV = '0'
                        reg_C001.write_line()
                    reg_C300.REG = 'C300'
                    reg_C300.COD_MOD = dado.get('cod_mod')
                    reg_C300.SER = dado.get('ser') or ''
                    reg_C300.SUB = dado.get('sub') or ''
                    reg_C300.NUM_DOC_INI = dado.get('num_doc_ini')
                    reg_C300.NUM_DOC_FIN = dado.get('num_doc_fin')
                    reg_C300.DT_DOC = self.formata_data(dado.get('dt_doc'))
                    reg_C300.VL_DOC = self.formata_numero(dado.get('vl_doc'), size=1,
                                                          separador=',') or '0,00'
                    reg_C300.VL_PIS = (self.formata_numero(
                        dado.get('vl_pis'), size=1, separador=',') or '0,00') if gerar_pis_cofins \
                        else ''
                    reg_C300.VL_COFINS = (self.formata_numero(
                        dado.get('vl_pis'), size=1, separador=',') or '0,00') if gerar_pis_cofins \
                        else ''
                    reg_C300.COD_CTA = ''
                    reg_C300.write_line()
                    _registroC310(dado.get('num_doc_ini'), dado.get('num_doc_fin'))
                    total_geral_debitosC320 += _registroC320(mov_2d, dado.get('dt_doc'))

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C300'
            return mensagem

        def _registroC310(num_nf_ini, num_nf_fin):
            # print '-----------------------INI REG C310'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            filtro = {'data_ini': data_ini, 'data_fim': data_fim,
                      'company_id': val.get('company_id'), 'num_nf_ini': num_nf_ini,
                      'num_nf_fin': num_nf_fin}
            dados = obj_sql.sql_regC310(cr, filtro)
            for dado in dados:
                reg_C310.REG = 'C310'
                reg_C310.NUM_DOC_CANC = dado.get('num_doc_canc')
                reg_C310.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C310'

        def _registroC320(mv2d_ids, data_emissao):
            # print '-----------------------INI REG C320'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            filtro = {
                'mov_2d_ids': mv2d_ids,
                'data_emissao': data_emissao
            }
            dados = obj_sql.sql_regC320(cr, filtro)
            deb_icms = 0.00
            if dados:
                for dado in dados:
                    cst = dado.get('cst_icms')
                    vl_bc_icms = dado.get('vl_bc_icms')
                    vl_icms = dado.get('vl_icms')
                    if cst in ('10', '30', '40', '60', '70'):
                        if not read_pessoa.admpes_jur_tipo_atividade == 'I':
                            vl_bc_icms = 0.00
                            vl_icms = 0.00
                    reg_C320.REG = 'C320'
                    reg_C320.CST_ICMS = str(dado.get('cst_icms')).zfill(3)
                    reg_C320.CFOP = dado.get('cfop')
                    reg_C320.ALIQ_ICMS = self.formata_numero(dado.get('aliq_icms'), size=1,
                                                             separador=',')
                    reg_C320.VL_OPR = self.formata_numero(dado.get('vl_opr'), size=1,
                                                          separador=',') or '0,00'
                    reg_C320.VL_BC_ICMS = self.formata_numero(vl_bc_icms, size=1,
                                                              separador=',') or '0,00'
                    reg_C320.VL_ICMS = self.formata_numero(vl_icms, size=1,
                                                           separador=',') or '0,00'
                    reg_C320.VL_RED_BC = self.formata_numero(dado.get('vl_red_bc'), size=1,
                                                             separador=',') or '0,00'
                    reg_C320.COD_OBS = ''
                    reg_C320.write_line()
                    deb_icms += dado.get('vl_icms')
                    _registroC321(mv2d_ids, data_emissao, cst)

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C320'
            return deb_icms

        def _registroC321(mov_2d_ids, data_emissao, cst):
            # print '-----------------------INI REG C321'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            filtro = {
                'mov_2d_ids': mov_2d_ids,
                'data_emissao': data_emissao,
                'cst': cst
            }
            dados = obj_sql.sql_regC321(cr, filtro)
            if dados:
                for dado in dados:
                    vl_bc_icms = dado.get('vl_bc_icms')
                    vl_icms = dado.get('vl_icms')
                    if cst in ('10', '30', '40', '60', '70'):
                        if not read_pessoa.admpes_jur_tipo_atividade == 'I':
                            vl_bc_icms = 0.00
                            vl_icms = 0.00
                    reg_C321.REG = 'C321'
                    reg_C321.COD_ITEM = dado.get('cod_item')
                    reg_C321.QTD = self.formata_numero(dado.get('qtd'), size=1,
                                                       separador=',')
                    reg_C321.UNID = dado.get('unid')
                    reg_C321.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=1,
                                                           separador=',') or '0,00'
                    reg_C321.VL_DESC = self.formata_numero(dado.get('vl_desc'), size=1,
                                                           separador=',') or '0,00'
                    reg_C321.VL_BC_ICMS = self.formata_numero(vl_bc_icms, size=1,
                                                              separador=',') or '0,00'
                    reg_C321.VL_ICMS = self.formata_numero(vl_icms, size=1,
                                                           separador=',') or '0,00'
                    reg_C321.VL_PIS = '0,00' if gerar_pis_cofins else ''
                    reg_C321.VL_COFINS = '0,00' if gerar_pis_cofins else ''
                    reg_C321.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C321'

        def _registroC400():
            while obj_sql.C400.get('executando'):
                pass
            if obj_sql.C400.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C400.get('erro')))
            global total_geral_debito_C490
            total_geral_debito_C490 = 0.00
            mensagem = ''
            for dado in obj_sql.C400.get('dados'):
                if reg_C001.IND_MOV == '1':
                    reg_C001.IND_MOV = '0'
                    reg_C001.write_line()
                reg_C400.REG = dado.get('reg')
                reg_C400.COD_MOD = dado.get('cod_mod')
                reg_C400.ECF_MOD = dado.get('ecf_mod')
                reg_C400.ECF_FAB = dado.get('ecf_fab')
                reg_C400.ECF_CX = dado.get('ecf_cx')
                reg_C400.write_line()
                total_geral_debito_C490 += _registroC405(dado.get('id'))

            return mensagem

        def _registroC405(ecf_id):
            while obj_sql.C405.get('executando'):
                pass
            if obj_sql.C405.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C405.get('erro')))
            tot_deb = 0.00
            dados = filter(lambda i: i.get('chave') == ecf_id, obj_sql.C405.get('dados'))
            for dado in dados:
                reg_C405.REG = dado.get('reg')
                reg_C405.DT_DOC = self.formata_data(dado.get('dt_doc'))
                if dado.get('cro'):
                    if len(str(dado.get('cro'))) <= 3:
                        reg_C405.CRO = str(dado.get('cro')).zfill(3)
                    else:
                        self.set_mensagem("REG C405: CRO nao deve conter mais que 3 digitos: Valor "
                                          "do campo: {cro}".format(cro=str(dado.get('cro'))))
                        reg_C405.CRO = str(dado.get('cro'))[0:2]
                else:
                    reg_C405.CRO = ''
                reg_C405.CRZ = dado.get('crz')
                if len(str(dado.get('num_coo_fin'))) <= 6:
                    reg_C405.NUM_COO_FIN = str(dado.get('num_coo_fin')).zfill(3)
                else:
                    self.set_mensagem('REG C405 -- Tamanho do COO invalido - {coo}'.format(
                        coo=dado.get('num_coo_fin')))
                reg_C405.GT_FIN = self.formata_numero(dado.get('gt_fin'), size=1,
                                                      separador=',')
                reg_C405.VL_BRT = self.formata_numero(dado.get('vl_brt'), size=1,
                                                      separador=',')
                reg_C405.write_line()

                # Chamando o metodo _registro C420 se venda bruta > 0
                if dado.get('vl_brt'):
                    dados_C420 = {
                        'id': dado.get('id'),
                        'acrescimos': dado.get('acrescimos'),
                        'descontos': dado.get('descontos'),
                        'cancelamentos': dado.get('cancelamentos')
                    }
                    _registroC420(dados_C420)
                    tot_deb += _registroC490(ecf_id, dado.get('dt_doc'))

            return tot_deb

        def _registroC420(dados_capa):
            while obj_sql.C420.get('executando'):
                pass
            if obj_sql.C420.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C420.get('erro')))
            z_id = dados_capa.get('id')
            if dados_capa.get('acrescimos'):
                reg_C420.REG = 'C420'
                reg_C420.COD_TOT_PAR = 'AT'
                reg_C420.VLR_ACUM_TOT = self.formata_numero(dados_capa.get('acrescimos'), size=0,
                                                            separador=',')
                reg_C420.NR_TOT = ''
                reg_C420.write_line()
            if dados_capa.get('descontos'):
                reg_C420.REG = 'C420'
                reg_C420.COD_TOT_PAR = 'DT'
                reg_C420.VLR_ACUM_TOT = self.formata_numero(dados_capa.get('descontos'), size=0,
                                                            separador=',')
                reg_C420.NR_TOT = ''
                reg_C420.write_line()
            if dados_capa.get('cancelamentos'):
                reg_C420.REG = 'C420'
                reg_C420.COD_TOT_PAR = 'Can-T'
                reg_C420.VLR_ACUM_TOT = self.formata_numero(dados_capa.get('cancelamentos'), size=0,
                                                            separador=',')
                reg_C420.NR_TOT = ''
                reg_C420.write_line()
            dados = filter(lambda i: i.get('chave') == z_id, obj_sql.C420.get('dados'))
            tot = 0
            for dado in dados:
                reg_C420.REG = dado.get('reg')
                cod_tot_par = ''
                if dado.get('imposto') == 'C':
                    if dado.get('tipo') == 'T':
                        tot += 1
                        aliquota = self.formata_numero(dado.get('aliquota'), size=0,
                                                       separador='').zfill(4)
                        cod_tot_par = str(tot).zfill(2) + 'T' + aliquota
                    elif dado.get('tipo') == 'F':
                        cod_tot_par = 'F1'
                    elif dado.get('tipo') == 'I':
                        cod_tot_par = 'I1'
                    elif dado.get('tipo') == 'O':
                        cod_tot_par = 'N1'
                    else:
                        cod_tot_par = 'D1'
                reg_C420.COD_TOT_PAR = cod_tot_par
                reg_C420.VLR_ACUM_TOT = self.formata_numero(dado.get('vlr_acum_tot'),
                                                            size=1, separador=',')
                if dado.get('tipo') == 'T':
                    nr_tot = tot
                else:
                    nr_tot = ''
                reg_C420.NR_TOT = nr_tot
                reg_C420.DESCR_NR_TOT = ''
                reg_C420.write_line()
                filtroC425 = {
                    'ecf_id': dado.get('ecf'),
                    'aliquota': dado.get('aliquota'),
                    'tipo': dado.get('tipo'),
                    'data': dado.get('data')
                }
                if reg_0000.IND_PERFIL != 'C':
                    _registroC425(filtroC425)

        def _registroC425(filtros):
            while obj_sql.C425.get('executando'):
                pass
            if obj_sql.C425.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C425.get('erro')))
            if perfil_sped == 'B':
                tipo = filtros.get('tipo')
                if tipo == 'F':
                    tipo = 'U'
                chave = '%s%s%s%s' % (filtros.get('ecf_id'), filtros.get('aliquota'),
                                      tipo, filtros.get('data'))
                dados = filter(lambda i: i.get('chave') == chave, obj_sql.C425.get('dados'))
                for dado in dados:
                    if all([dado.get('qtd'), dado.get('vl_item')]):
                        reg_C425.REG = dado.get('reg')
                        reg_C425.COD_ITEM = dado.get('cod_item')
                        reg_C425.QTD = self.formata_numero(dado.get('qtd'), size=1, dec=3,
                                                           separador=',')
                        reg_C425.UNID = dado.get('unid')
                        reg_C425.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=1,
                                                               separador=',')
                        reg_C425.VL_PIS = dado.get('vl_pis') if gerar_pis_cofins else ''
                        reg_C425.VL_COFINS = dado.get('vl_cofins') if gerar_pis_cofins else ''
                        reg_C425.write_line()

        def _registroC490(ecf_id, dt_doc):
            while obj_sql.C490.get('executando'):
                pass
            if obj_sql.C490.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C490.get('erro')))

            sql = """
                SELECT
                    Sum(base_valor) AS vr_base_calculo,
                    Sum(imposto_valor) AS vr_encargo,
                    aliquota
                FROM
                    ecf_mapa_resumo_itens emr
                INNER JOIN ecf_mapa_resumo_capa emc
                    ON emc.id = emr.capa_id
                WHERE
                    emc.data = '%s'
                    AND emc.ecf = %s
                    AND emr.tipo = 'T'
                    AND emr.base_valor > 0
                GROUP BY
                    aliquota
            """ % (dt_doc, ecf_id)
            cr.execute(sql)
            mapa_ecf_dados_dia = cr.dictfetchall()

            vl_icms_C490 = 0.00
            chave = '%s%s' % (ecf_id, dt_doc)
            dados = filter(lambda i: i.get('chave') == chave, obj_sql.C490.get('dados'))
            for dado in dados:
                if dado.get('vl_opr'):
                    mapa_ecf_aliquota = filter(
                        lambda a: a.get('aliquota') == dado.get('aliq_icms'), mapa_ecf_dados_dia)
                    if mapa_ecf_aliquota:
                        mapa_ecf_aliquota = mapa_ecf_aliquota[0]
                    else:
                        mapa_ecf_aliquota = {}
                    vr_base_calculo_icms = float(float_round(dado.get('vl_bc_icms'), 2))
                    if vr_base_calculo_icms != float(float_round(mapa_ecf_aliquota.get(
                            'vr_base_calculo') or 0, 2)) and dado.get('aliq_icms') == \
                            mapa_ecf_aliquota.get('aliquota'):
                        vr_base_calculo_icms = mapa_ecf_aliquota.get('vr_base_calculo')
                    vr_icms = float(float_round(dado.get('vl_icms'), 2))
                    if vr_icms != float(float_round(mapa_ecf_aliquota.get('vr_encargo') or 0, 2)) \
                            and dado.get('aliq_icms') == mapa_ecf_aliquota.get('aliquota'):
                        vr_icms = mapa_ecf_aliquota.get('vr_encargo')

                    reg_C490.REG = dado.get('reg')
                    reg_C490.CST_ICMS = str(dado.get('cst_icms')).zfill(3)
                    reg_C490.CFOP = dado.get('cfop')
                    reg_C490.ALIQ_ICMS = self.formata_numero(dado.get('aliq_icms'), size=1,
                                                             separador=',')
                    reg_C490.VL_OPR = self.formata_numero(dado.get('vl_opr'), size=0, separador=',')
                    reg_C490.VL_BC_ICMS = self.formata_numero(vr_base_calculo_icms, size=0,
                                                              separador=',')
                    reg_C490.VL_ICMS = self.formata_numero(vr_icms, size=0,
                                                           separador=',')
                    reg_C490.COD_OBS = dado.get('cod_obs')
                    reg_C490.write_line()
                    vl_icms_C490 += vr_icms

            return vl_icms_C490

        def _registroC500():
            # print '-----------------------INI REG C500'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            global total_geral_creditos_C590
            total_geral_creditos_C590 = 0.00
            mensagem = ''
            if movimento_notas:
                filtro = {'dt_ini': dt_ini,
                          'dt_fin': dt_fim}
                dados = obj_sql.sql_C500(cr, filtro)

                for dado in dados:
                    if reg_C001.IND_MOV == '1':
                        reg_C001.IND_MOV = '0'
                        reg_C001.write_line()
                    reg_C500.REG = dado.get('reg')
                    if dado.get('ind_oper') == 'E':
                        ind_oper = 0
                    else:
                        ind_oper = 1
                    reg_C500.IND_OPER = ind_oper
                    if dado.get('ind_emit'):
                        ind_emit = '0'
                    else:
                        ind_emit = '1'
                    reg_C500.IND_EMIT = ind_emit
                    reg_C500.COD_PART = dado.get('cod_part')
                    reg_C500.COD_MOD = str(dado.get('cod_mod')).zfill(2)
                    if dado.get('cod_sit') == '3':
                        cod_sit = '00'
                    else:
                        cod_sit = ''
                    reg_C500.COD_SIT = cod_sit
                    reg_C500.SER = dado.get('ser') or ''
                    reg_C500.SUB = dado.get('sub') or ''
                    reg_C500.COD_CONS = str(dado.get('cod_cons')).zfill(2) if dado.get('cod_cons') \
                        else ''
                    reg_C500.NUM_DOC = dado.get('num_doc')
                    reg_C500.DT_DOC = self.formata_data(dado.get('dt_doc'))
                    reg_C500.DT_E_S = self.formata_data(dado.get('dt_e_s'))
                    reg_C500.VL_DOC = self.formata_numero(dado.get('vl_doc'), separador=',', size=1)
                    reg_C500.VL_DESC = self.formata_numero(dado.get('vl_desc'), separador=',',
                                                           size=1)
                    reg_C500.VL_FORN = self.formata_numero(dado.get('vl_forn'), separador=',',
                                                           size=1)
                    reg_C500.VL_TERC = self.formata_numero(dado.get('vl_ter'), separador=',',
                                                           size=1)
                    reg_C500.VL_SERV_NT = self.formata_numero(dado.get('vl_serv_nt'), separador=',',
                                                              size=1)
                    reg_C500.VL_DA = self.formata_numero(dado.get('vl_da'), separador=',', size=1)
                    reg_C500.VL_BC_ICMS = self.formata_numero(dado.get('vl_bc_icms'), separador=',',
                                                              size=1)
                    reg_C500.VL_ICMS = self.formata_numero(dado.get('vl_icms'), separador=',',
                                                           size=1)
                    if read_pessoa.admpes_jur_tipo_atividade == "I":
                        reg_C500.VL_BC_ICMS_ST = self.formata_numero(dado.get('vl_bc_icms_st'),
                                                                     separador=',', size=1)
                        reg_C500.VL_ICMS_ST = self.formata_numero(dado.get('vl_icms_st'),
                                                                  separador=',', size=1)
                    else:
                        reg_C500.VL_BC_ICMS_ST = '0,00'
                        reg_C500.VL_ICMS_ST = '0,00'
                    reg_C500.COD_INF = dado.get('cod_inf') or ''
                    encargo_pis_cofins = obj_sql.sql_totaliza_pis_cofins(cr, dado.get('nf_capa_id'))
                    vl_pis = 0.00
                    vl_cofins = 0.00
                    for encargo in encargo_pis_cofins:
                        if encargo.get('encargo') == 'PIS':
                            vl_pis = encargo.get('vr_encargo')
                        elif encargo.get('encargo') == 'COFINS':
                            vl_cofins = encargo.get('vr_encargo')
                        else:
                            pass
                    reg_C500.VL_PIS = self.formata_numero(
                        vl_pis, separador=',', size=1) if gerar_pis_cofins else ''
                    reg_C500.VL_COFINS = self.formata_numero(
                        vl_cofins, separador=',', size=1) if gerar_pis_cofins else ''
                    reg_C500.TP_LIGACAO = dado.get('tp_ligacao') or ''
                    reg_C500.COD_GRUPO_TENSAO = str(dado.get('cod_grupo_tensao')).zfill(2) if \
                        dado.get('cod_grupo_tensao') else ''
                    if cod_versao and int(cod_versao) >= 14:
                        if reg_C500.COD_MOD == '66':
                            if not dado.get('chave_acesso'):
                                raise osv.except_osv(_('Atencao!'), ('Nota sem chave de acesso'))

                            reg_C500.CHV_DOCe = dado.get('chave_acesso') or ''
                            reg_C500.FIN_DOCe = '1'
                            if reg_C500.FIN_DOCe == '2':  # substituicao de nota
                                reg_C500.CHV_DOCe_REF = ''

                        if dado.get('tipo') == 'S':
                            codigo = 9  # Não Contribuinte
                            if dados.get('pessoa_tipo') == 'J':
                                codigo = 2  # Contribuinte Isento de Inscrição no Cadastro de
                                # Contribuintes do ICMS
                                if dados.get('inscricao_est'):
                                    codigo = 1  # Contribuinte do ICMS

                            reg_C500.IND_DEST = codigo  # valores validos [1,2,9]
                            if not dados.get('codigo_ibge'):
                                raise osv.except_osv(_('Atencao!'), _('Cidade do destinatario sem '
                                                                      'codigo IBGE cadastrado'))

                            reg_C500.COD_MUN_DEST = dados.get('codigo_ibge')

                    reg_C500.write_line()
                    total_geral_creditos_C590 += _registroC590(dado.get('nf_capa_id'))

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C500'
            return mensagem

        def _registroC590(nf_id):
            # print '-----------------------INI REG C590'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            dados = obj_sql.sql_C590(cr, nf_id)
            valor_icms = 0.00
            for dado in dados:
                reg_C590.REG = dado.get('reg')
                reg_C590.CST_ICMS = str(dado.get('cst_icms')).zfill(3)
                reg_C590.CFOP = dado.get('cfop')
                reg_C590.ALIQ_ICMS = self.formata_numero(dado.get('aliq_icms'), separador=',',
                                                         size=1)
                reg_C590.VL_OPR = self.formata_numero(dado.get('vl_opr'), separador=',', size=1)
                vl_bc_icms = dado.get('vl_bc_icms')
                vl_icms = dado.get('vl_icms') or 0
                if dado.get('cst_icms') in ('10', '30', '40', '60', '70', '90'):
                    if not read_pessoa.admpes_jur_tipo_atividade == 'I':
                        vl_bc_icms = 0.00
                        vl_icms = 0.00
                reg_C590.VL_BC_ICMS = self.formata_numero(vl_bc_icms, separador=',',
                                                          size=1)
                reg_C590.VL_ICMS = self.formata_numero(vl_icms, separador=',', size=1)
                reg_C590.VL_BC_ICMS_ST = self.formata_numero(dado.get('vl_bc_icms_st'),
                                                             separador=',', size=1)
                reg_C590.VL_ICMS_ST = self.formata_numero(dado.get('vl_icms_st'), separador=',',
                                                          size=1)
                reg_C590.VL_RED_BC = '0,00'
                if dado.get('cst_icms') in ['20', '70']:
                    reg_C590.VL_RED_BC = self.formata_numero(dado.get('vl_red_bc'),
                                                             separador=',', size=1)
                for reg0460 in filter(lambda i: i.COD_OBS == str(nf_id)[-6:], lista_0460):
                    reg_C590.COD_OBS = reg0460.COD_OBS

                reg_C590.write_line()
                valor_icms += vl_icms
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG C590'
            return valor_icms

        # Metodos do Bloco D
        def _registroD001():
            # print '-----------------------INI REG D001'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            reg_D001.REG = 'D001'
            reg_D001.IND_MOV = '1'
            movimentoD001 = False
            if movimento_notas:
                filtro = {
                    'movimento_notas': movimento_notas,
                    'dt_ini': dt_ini,
                    'dt_fim': dt_fim
                }
                dados_D100 = obj_sql.sql_D100(cr, filtro)
                dados_D500 = obj_sql.sql_D500(cr, filtro)
                if not any([dados_D100, dados_D500]):
                    reg_D001.write_line()
                else:
                    reg_D001.IND_MOV = '0'
                    reg_D001.write_line()
                    movimentoD001 = True
                    _registroD100(dados_D100)
                    _registroD500(dados_D500)
            else:
                reg_D001.write_line()

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG D001'
            return movimentoD001

        def _registroD100(dados):
            # print '-----------------------INI REG D100'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            global total_gera_creditosD190
            total_gera_creditosD190 = 0.00
            reg_D100 = bloco_d.registro_D100()
            if cod_versao == '012':
                reg_D100 = sped.bloco_d.registro_D100()
            for dado in dados:
                cod_cta = self.sped_get_conta_contabil(cr, uid, nf_id=dado.get('id'),
                                                       context=context)
                if cod_versao >= '012':
                    dado['cod_cta'] = cod_cta
                    reg_D100.registro = dado
                else:
                    reg_D100.REG = dado.get('reg')
                    reg_D100.IND_OPER = dado.get('ind_oper') or ''
                    reg_D100.IND_EMIT = dado.get('ind_emit')
                    reg_D100.COD_PART = dado.get('cod_part')
                    reg_D100.COD_MOD = dado.get('cod_mod')
                    reg_D100.COD_SIT = dado.get('cod_sit') or '00'
                    reg_D100.SER = dado.get('ser') or ''
                    reg_D100.SUB = dado.get('sub') or ''
                    reg_D100.NUM_DOC = dado.get('num_doc')
                    reg_D100.CHV_CTE = dado.get('chv_cte') or ''
                    reg_D100.DT_DOC = self.formata_data(dado.get('dt_doc')) or ''
                    reg_D100.DT_A_P = self.formata_data(dado.get('dt_a_p')) or ''
                    reg_D100.TP_CT_e = dado.get('tp_ct_e') or ''
                    reg_D100.CHV_CTE_REF = dado.get('chv_cte_ref')
                    reg_D100.VL_DOC = self.formata_numero(dado.get('vl_doc'), size=0, separador=',')
                    reg_D100.VL_DESC = self.formata_numero(dado.get('vl_desc'), size=0, separador=',')
                    reg_D100.IND_FRT = dado.get('ind_frt')  # Nao esta buscando da nota correta
                    reg_D100.VL_SERV = self.formata_numero(dado.get('vl_serv'), size=0, separador=',')
                    reg_D100.VL_BC_ICMS = self.formata_numero(dado.get('vl_bc'
                                                                       '_icms'), size=0,
                                                              separador=',')
                    reg_D100.VL_ICMS = self.formata_numero(dado.get('vl_icms'), size=0, separador=',')
                    reg_D100.VL_NT = self.formata_numero(dado.get('vl_nt'), size=0, separador=',')
                    if dado.get('txt'):
                        reg_D100.COD_INF = dado.get('cod_inf')
                    else:
                        reg_D100.COD_INF = ''
                    reg_D100.COD_CTA = cod_cta
                    reg_D100.write_line()
                total_gera_creditosD190 += _registroD190(dado.get('id'))
                _registroD195(dado.get('id'))
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG D100'

        def _registroD190(nf_id):
            # print '-----------------------INI REG D190'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # Todo: transformar este submetodo em um metodo para que outras classes possam enchergar
            dados = obj_sql.sql_D190(cr, nf_id)
            total_vlicms = 0
            for dado in dados:
                reg_D190.REG = dado.get('reg')
                # Esta fixo o primeiro 0, (alterar)
                reg_D190.CST_ICMS = dado.get('cst_icms').zfill(3) if dado.get('cst_icms') else ''
                reg_D190.CFOP = dado.get('cfop')
                reg_D190.ALIQ_ICMS = self.formata_numero(dado.get('aliq_icms'), size=1,
                                                         separador=',')
                reg_D190.VL_OPR = self.formata_numero(dado.get('vl_opr_total'), size=0,
                                                      separador=',')
                vl_bc_icms = dado.get('vl_bc_icms')
                vl_icms = dado.get('vl_icms')
                if dado.get('cst_icms') in ('10', '30', '40', '60', '70', '90'):
                    if not read_pessoa.admpes_jur_tipo_atividade == 'I':
                        vl_bc_icms = 0.00
                        vl_icms = 0.00
                reg_D190.VL_BC_ICMS = self.formata_numero(vl_bc_icms, size=0,
                                                          separador=',')
                reg_D190.VL_ICMS = self.formata_numero(vl_icms, size=0, separador=',')
                reg_D190.VL_RED_BC = self.formata_numero(dado.get('vl_red_bc'), size=0,
                                                         separador=',')
                reg_D190.COD_OBS = ''  # Necessario criar o registro 0460
                reg_D190.write_line()
                total_vlicms += dado.get('vl_icms')
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG D190'
            return total_vlicms

        def _registroD195(nf_id):
            for reg_0460 in filter(lambda i: i.COD_OBS == str(nf_id)[-6:], lista_0460):
                reg_D195.REG = 'D195'
                reg_D195.COD_OBS = reg_0460.COD_OBS
                reg_D195.TXT_COMPL = ''
                reg_D195.write_line()

        def _registroD500(dados):
            # print '-----------------------INI REG D500'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            for dado in dados:
                nf_id = dado.get('nf_id')
                if dado.get('cod_sit') == 'C':
                    reg_D500.REG = ''
                    reg_D500.COD_PART = ''
                    reg_D500.SUB = ''
                    reg_D500.DT_A_P = ''
                    reg_D500.VL_DOC = ''
                    reg_D500.VL_DESC = ''
                    reg_D500.VL_SERV = ''
                    reg_D500.VL_SERV_NT = ''
                    reg_D500.VL_TERC = ''
                    reg_D500.VL_DA = ''
                    reg_D500.VL_BC_ICMS = ''
                    reg_D500.VL_ICMS = ''
                    reg_D500.COD_INF = ''
                    reg_D500.VL_PIS = ''
                    reg_D500.VL_COFINS = ''
                    reg_D500.COD_CTA = ''
                    reg_D500.TP_ASSINANTE = ''
                else:
                    reg_D500.REG = dado.get('reg')
                    reg_D500.COD_PART = dado.get('cod_part') or ''
                    reg_D500.SUB = dado.get('sub') or ''
                    reg_D500.DT_A_P = self.formata_data(dado.get('dt_a_p')) or ''
                    reg_D500.VL_DOC = self.formata_numero(dado.get('vl_doc'), size=0, separador=',')
                    reg_D500.VL_DESC = self.formata_numero(dado.get('vl_desc'), size=0,
                                                           separador=',')
                    reg_D500.VL_SERV = self.formata_numero(dado.get('vl_serv'), size=0,
                                                           separador=',')
                    reg_D500.VL_SERV_NT = self.formata_numero(dado.get('vl_serv_nf', 0.00), size=0,
                                                              separador=',')
                    reg_D500.VL_TERC = self.formata_numero(dado.get('vl_terc', 0.00), size=0,
                                                           separador=',')
                    reg_D500.VL_DA = self.formata_numero(dado.get('vl_da', 0.00), size=0,
                                                         separador=',')
                    reg_D500.VL_BC_ICMS = self.formata_numero(dado.get('vl_bc_icms', 0.00), size=0,
                                                              separador=',')
                    reg_D500.VL_ICMS = self.formata_numero(dado.get('vl_icms', 0.00), size=0,
                                                           separador=',')
                    if dado.get('txt'):
                        reg_D500.COD_INF = dado.get('nf_id')
                    else:
                        reg_D500.COD_INF = ''
                    reg_D500.VL_PIS = self.formata_numero(
                        dado.get('vl_pis', 0.00), size=0, separador=',') if gerar_pis_cofins else ''
                    reg_D500.VL_COFINS = self.formata_numero(
                        dado.get('vl_cofins', 0.00), size=0, separador=',') if gerar_pis_cofins \
                        else ''
                    reg_D500.COD_CTA =  self.sped_get_conta_contabil(cr, uid, nf_id=nf_id,
                                                                     context=context)
                    reg_D500.TP_ASSINANTE = '1'
                reg_D500.IND_OPER = '0' if dado.get('ind_oper') == 'E' else '1'
                reg_D500.IND_EMIT = '1' if not dado.get('ind_emit') else '0'
                reg_D500.COD_MOD = dado.get('cod_mod') or ''
                reg_D500.COD_SIT = self.get_cod_sit(dado.get('cod_sit'))
                reg_D500.SER = dado.get('serie') or ''
                reg_D500.NUM_DOC = dado.get('num_doc') or ''
                reg_D500.DT_DOC = self.formata_data(dado.get('dt_doc')) or ''
                reg_D500.write_line()
                _registroD590(nf_id)
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG D500'

        def _registroD590(nf_id):
            # print '-----------------------INI REG D590'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            movimento_notas_0460 = {'movimento_notas': [nf_id]}
            cod_obs = obj_sql.sql_reg0460(cr, movimento_notas_0460) or {}
            if cod_obs and isinstance(cod_obs, list):
                cod_obs = cod_obs[0]
            cod_obs = cod_obs.get('cod_obs') or ''

            dados = obj_sql.sql_D590(cr, nf_id)
            for dado in dados:
                reg_D590.REG = dado.get('reg')
                reg_D590.CST_ICMS = dado.get('cst_icms') or ''
                reg_D590.CFOP = dado.get('cfop') or ''
                reg_D590.ALIQ_ICMS = dado.get('aliq_icms') or '0,00'
                reg_D590.VL_OPR = self.formata_numero(dado.get('vl_opr', 0.00), size=0,
                                                      separador=',')
                vl_bc_icms = dado.get('vl_bc_icms') or 0.0
                vl_icms = dado.get('vl_icms') or 0.0
                if dado.get('cst') in ('10', '30', '40', '60', '70', '90'):
                    if not read_pessoa.admpes_jur_tipo_atividade == 'I':
                        vl_bc_icms = 0.00
                        vl_icms = 0.00
                reg_D590.VL_BC_ICMS = self.formata_numero(vl_bc_icms, size=0,
                                                          separador=',')
                reg_D590.VL_ICMS = self.formata_numero(vl_icms, size=0,
                                                       separador=',')
                reg_D590.VL_BC_ICMS_UF = '0,00'
                reg_D590.VL_ICMS_UF = '0,00'
                reg_D590.VL_RED_BC = self.formata_numero(dado.get('vl_red_bc', 0.00), size=0,
                                                         separador=',')
                reg_D590.COD_OBS = cod_obs
                reg_D590.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG D590'

        # Metdodos do Bloco E
        def _registroE001():
            mensagem = ''
            reg_E001.REG = 'E001'
            reg_E001.IND_MOV = '0'
            reg_E001.write_line()

            return mensagem

        def _registroE100():
            mensagem = ''
            reg_E100.REG = 'E100'
            reg_E100.DT_INI = self.formata_data(data_ini)
            reg_E100.DT_FIN = self.formata_data(data_fim)
            reg_E100.write_line()

            return mensagem

        def _registroE110():
            # print '-----------------------INI REG E110'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            args_icms = [('tipo', '=', 'C'), ('company_id', '=', val['company_id'])]
            args_icms_anterior = [('tipo', '=', 'C')]
            args_icms.extend(args_periodo_mes_e_ano)
            mes_periodo_anterior = int(data_fim[5:7])
            ano_periodo_anterior = int(data_fim[:4])
            if mes_periodo_anterior == 1:
                mes_periodo_anterior = 12
                ano_periodo_anterior -= 1
            else:
                mes_periodo_anterior -= 1
            args_periodo_anterior = [('mes', '>=', mes_periodo_anterior),
                                     ('mes', '<=', mes_periodo_anterior),
                                     ('ano', '<=', ano_periodo_anterior),
                                     ('ano', '>=', ano_periodo_anterior),
                                     ('company_id', '=', val['company_id'])]
            args_icms_anterior.extend(args_periodo_anterior)
            ids_apuracao = obj_apuracao_imposto.search(cr, uid, args_icms, context=context)
            ids_apuracao_per_ant = obj_apuracao_imposto.search(cr, uid, args_icms_anterior,
                                                               context=context)
            table = 'fiscal_apuracao_imposto'
            valor_deducoes = valor_outros_debitos = valor_outros_creditos = 0
            valor_estorno_creditos = valor_estorno_debitos = valor_saldo_credor_ant = 0
            vl_aj_credito = sum(ajustes_creditos)  # Ajustes gerado pelo registro C197
            if ids_apuracao:
                read_saldo_credor_ant = obj_apuracao_imposto.read(cr, uid, ids_apuracao_per_ant,
                                                                  ['saldo_credor'], context=context)
                if read_saldo_credor_ant:
                    valor_saldo_credor_ant = read_saldo_credor_ant[0].get('saldo_credor') or 0
                valor_outros_creditos = obj_sql.sql_apuracoes(cr, ids_apuracao[0],
                                                              table + '_outros_creditos') or 0
                valor_outros_debitos = obj_sql.sql_apuracoes(cr, ids_apuracao[0],
                                                             table + '_outros_debitos') or 0
                valor_estorno_creditos = obj_sql.sql_apuracoes(cr, ids_apuracao[0],
                                                               table + '_estorno_creditos') or 0
                valor_estorno_debitos = obj_sql.sql_apuracoes(cr, ids_apuracao[0],
                                                              table + '_estorno_debitos') or 0
            if not movimentoD001:
                tot_creditosD190 = 0.00
            else:
                tot_creditosD190 = total_gera_creditosD190
            if not movimentoC001:
                tot_creditosC190 = 0.00
                tot_creditosC590 = 0.00
                tot_debitosC190 = 0.00
                tot_debitosC490 = 0.00
                tot_debitosC320 = 0.00
            else:
                tot_creditosC190 = total_geral_creditos_C190
                tot_creditosC590 = total_geral_creditos_C590
                tot_debitosC190 = total_geral_debitos_C190
                tot_debitosC320 = total_geral_debitosC320
                tot_debitosC490 = 0.00
                if movimento_cupons:
                    tot_debitosC490 = total_geral_debito_C490
            total_geral_creditos = \
                tot_creditosC190 + tot_creditosC590 + tot_creditosD190
            total_geral_debitos = tot_debitosC190 + tot_debitosC490 + tot_debitosC320
            if not ids_apuracao:
                self.set_mensagem(
                    '* Nao existe apuracao do ICMS entre a data inicial e data final informadas.')
            else:
                soma = total_geral_debitos + valor_outros_debitos + valor_estorno_creditos - (
                    total_geral_creditos + valor_estorno_debitos + valor_saldo_credor_ant +
                    valor_outros_creditos + vl_aj_credito)

                if soma >= 0:
                    saldo_apurado = soma
                    saldo_credor_transportar = 0.0
                else:
                    saldo_apurado = 0.0
                    saldo_credor_transportar = soma * -1
                icms_recolher = saldo_apurado - valor_deducoes
                if icms_recolher < 0:
                    icms_recolher = 0.0
                    saldo_credor_transportar = icms_recolher * -1
                for id_apuracao in ids_apuracao:
                    read_apuracao = obj_apuracao_imposto.read(cr, uid, id_apuracao,
                                                              ['deducoes_ids',
                                                               'outros_debitos_ids',
                                                               'estorno_creditos_ids',
                                                               'outros_creditos_ids',
                                                               'estorno_debitos_ids'],
                                                              context=context)
                    reg_E110.VL_TOT_DEBITOS = self.formata_numero(total_geral_debitos, size=1,
                                                                  separador=',')
                    reg_E110.VL_AJ_DEBITOS = '0,00'
                    reg_E110.VL_TOT_AJ_DEBITOS = self.formata_numero(valor_outros_debitos, size=1,
                                                                     separador=',')
                    reg_E110.VL_ESTORNOS_CRED = self.formata_numero(valor_estorno_creditos, size=1,
                                                                    separador=',')
                    reg_E110.VL_TOT_CREDITOS = self.formata_numero(total_geral_creditos, size=1,
                                                                   separador=',')
                    reg_E110.VL_AJ_CREDITOS = self.formata_numero(vl_aj_credito, size=1,
                                                                  separador=',')
                    reg_E110.VL_TOT_AJ_CREDITOS = self.formata_numero(valor_outros_creditos, size=1,
                                                                      separador=',')
                    reg_E110.VL_ESTORNOS_DEB = self.formata_numero(valor_estorno_debitos, size=1,
                                                                   separador=',')
                    reg_E110.VL_SLD_CREDOR_ANT = self.formata_numero(valor_saldo_credor_ant, size=1,
                                                                     separador=',')
                    reg_E110.VL_SLD_APURADO = self.formata_numero(saldo_apurado, size=1,
                                                                  separador=',')
                    reg_E110.VL_TOT_DED = self.formata_numero(valor_deducoes, size=1, separador=',')
                    reg_E110.VL_ICMS_RECOLHER = self.formata_numero(icms_recolher, size=1,
                                                                    separador=',')
                    reg_E110.VL_SLD_CREDOR_TRANSPORTAR = self.formata_numero(
                        saldo_credor_transportar, size=1, separador=',')
                    reg_E110.DEB_ESP = '0,00'
                    reg_E110.write_line()

                    dados_E111 = {
                        'read_apuracao': read_apuracao,
                    }
                    _registroE111(dados_E111)
                    mensagem += _registroE116(dia_vencimento_icms, icms_recolher)
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E110'
            return mensagem

        def _registroE111(dados):
            # print '-----------------------INI REG E111'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            global valor_outros_creditos_e111
            reg_E111 = bloco_e.registro_E111()
            read_apuracao = dados.get('read_apuracao')
            valor_deducoes = 0.00
            valor_outros_debitos = 0.00
            valor_outros_creditos = 0.00
            valor_estorno_creditos = 0.00
            valor_estorno_debitos = 0.00

            # Deducoes
            # XX049999 - Deducoes do imposto apurado na apuracao ICMS para a UF XX;
            for i in read_apuracao['deducoes_ids']:
                read_deducoes = obj_deducoes.read(cr, uid, i, ['valor', 'descricao'],
                                                  context=context)
                valor_deducoes += read_deducoes['valor']
                reg_E111.COD_AJ_APUR = reg_0000.UF + '049999'
                reg_E111.DESCR_COMPL_AJ = read_deducoes['descricao']
                reg_E111.VL_AJ_APUR = self.formata_numero(read_deducoes['valor'], size=1,
                                                          separador=',')
                reg_E111.write_line()

            # Outros Debitos
            # XX009999 - Outros debitos para ajuste de apuracao ICMS para a UF XX;
            for i in read_apuracao['outros_debitos_ids']:
                read_outros_debitos = obj_outros_debitos.read(cr, uid, i, ['valor', 'descricao'],
                                                              context=context)
                valor_outros_debitos += read_outros_debitos['valor']
                reg_E111.COD_AJ_APUR = reg_0000.UF + '009999'
                reg_E111.DESCR_COMPL_AJ = read_outros_debitos['descricao']
                reg_E111.VL_AJ_APUR = self.formata_numero(read_outros_debitos['valor'], size=1,
                                                          separador=',')
                reg_E111.write_line()

            # Outros Creditos
            # XX029999 - Outros creditos para ajuste de apuracao ICMS para a UF XX;
            for i in read_apuracao['outros_creditos_ids']:
                read_outros_creditos = obj_outros_creditos.read(
                    cr, uid, i, ['valor', 'descricao', 'fisapuoc_nf_id'], context=context)
                if read_outros_creditos.get('fisapuoc_nf_id'):
                    continue
                valor_outros_creditos += read_outros_creditos['valor']
                reg_E111.COD_AJ_APUR = reg_0000.UF + '029999'
                reg_E111.DESCR_COMPL_AJ = read_outros_creditos['descricao']
                reg_E111.VL_AJ_APUR = self.formata_numero(read_outros_creditos['valor'], size=1,
                                                          separador=',')
                reg_E111.write_line()

            # Estorno Creditos
            # XX019999 - Estorno de creditos para ajuste de apuracao ICMS para a UF XX;
            for i in read_apuracao['estorno_creditos_ids']:
                read_estorno_creditos = obj_estorno_creditos.read(cr, uid, i,
                                                                  ['valor', 'descricao'],
                                                                  context=context)
                valor_estorno_creditos += read_estorno_creditos['valor']
                reg_E111.COD_AJ_APUR = reg_0000.UF + '019999'
                reg_E111.DESCR_COMPL_AJ = read_estorno_creditos['descricao']
                reg_E111.VL_AJ_APUR = self.formata_numero(read_estorno_creditos['valor'], size=1,
                                                          separador=',')
                reg_E111.write_line()

            # Estorno Debitos
            # XX039999 - Estorno de debitos para ajuste de apuracao ICMS para a UF XX
            for i in read_apuracao['estorno_debitos_ids']:
                read_estorno_debitos = obj_estorno_debitos.read(cr, uid, i, ['valor', 'descricao'],
                                                                context=context)
                valor_estorno_debitos += read_estorno_debitos['valor']
                reg_E111.COD_AJ_APUR = reg_0000.UF + '039999'
                reg_E111.DESCR_COMPL_AJ = read_estorno_debitos['descricao']
                reg_E111.VL_AJ_APUR = self.formata_numero(read_estorno_debitos['valor'], size=1,
                                                          separador=',')
                reg_E111.write_line()
            valor_outros_creditos_e111 = valor_outros_creditos
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E111'

        def _registroE116(dia_vencimento_icms, icms_recolher):
            mensagem = ''
            if not dia_vencimento_icms:
                dia_vencimento_icms = ''
                mensagem += _(
                    '* DATA DE VENCIMENTO DO ICMS nao informado no parametro do Fiscal.\n')
            else:
                dia_vencimento_icms = str(dia_vencimento_icms)
                if not codigo_receita:
                    mensagem += _('* CODIGO DA RECEITA nao informado no parametro do Fiscal.\n')

            reg_E116.COD_OR = '000'
            mes = datetime.strptime(data_ini, '%Y-%m-%d') + relativedelta(months=1)
            reg_E116.VL_OR = self.formata_numero(icms_recolher, size=1, separador=',')
            reg_E116.DT_VCTO = formataString(dia_vencimento_icms, 2, '0', 'E') + '%s%s' % (
                str(mes.month).zfill(2), data_ini[:4]) if dia_vencimento_icms else ''
            reg_E116.COD_REC = codigo_receita or ''
            reg_E116.NUM_PROC = ''
            reg_E116.IND_PROC = ''
            reg_E116.PROC = ''
            reg_E116.TXT_COMPL = ''
            reg_E116.MES_REF = '%s%s' % (data_ini[5:7], data_ini[:4])
            reg_E116.write_line()

            return mensagem

        def _registroE200(data_inicial, data_final, tipo_atividade):
            # print '-----------------------INI REG E200'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            if not movimento_notas:
                return msg
            filtro = {'nf_ids': movimento_notas, 'tipo_atividade': tipo_atividade or 'NULL'}
            dados = obj_sql.sql_regE200(cr, filtro)
            # uf_dt_fin_dict = {}
            # if len(dados) == 1:
            #     dt_ini = parse(dados[0]['dt_ini']).strftime('%d%m%Y')
            #     uf_dt_fin_dict[dados[0].get('uf')] = {'dt_ini': dt_ini, 'dt_fin': dt_ini}
            # elif len(dados) > 1:
            #     uf_c = False
            #     for dado in dados:
            #         dado['dt_ini'] = parse(dado.get('dt_ini')).strftime('%d%m%Y')
            #         uf = dado.get('uf')
            #         if uf not in uf_dt_fin_dict:
            #             uf_dt_fin_dict.update({uf: {'dt_ini': dado.get('dt_ini')}})
            #         if uf_c and uf_c != uf:
            #             uf_dt_fin_dict[uf].update({'dt_fin': dado.get('dt_ini')})
            #         elif uf_c == uf:
            #             uf_dt_fin_dict[uf_c].update({'dt_fin': dado.get('dt_ini')})
            #         uf_c = uf
            # for dado in uf_dt_fin_dict:
            #     reg_E200.UF = dado
            #     reg_E200.DT_INI = uf_dt_fin_dict.get(dado).get('dt_ini')
            #     reg_E200.DT_FIN = uf_dt_fin_dict.get(dado).get('dt_fin')
            #     reg_E200.write_line()
            for dado in list(set(map(lambda i: i.get('uf'), dados))):
                reg_E200.UF = dado
                reg_E200.DT_INI = data_inicial.strftime('%d%m%Y')
                reg_E200.DT_FIN = data_final.strftime('%d%m%Y')
                reg_E200.write_line()
                mensagem += _registroE210(dado)
                mensagem += _registroE250(dado)
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E200'
            return mensagem

        def _registroE210(uf):
            # print '-----------------------INI REG E210'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            filtro = {'nf_ids': movimento_notas, 'uf': uf}
            dados = obj_sql.sql_regE210(cr, filtro)
            for dado in dados:
                vl_out_cred_st = dado.get('vl_out_cred_st') or 0
                vl_retencao_st = dado.get('vl_retencao_st') or 0
                vl_sld_cred_st_transportar = vl_out_cred_st - vl_retencao_st
                reg_E210.IND_MOV_ST = dado.get('ind_mov_st')
                reg_E210.VL_SLD_CRED_ANT_ST = '0,00'
                reg_E210.VL_DEVOL_ST = '0,00'
                reg_E210.VL_RESSARC_ST = '0,00'
                reg_E210.VL_OUT_CRED_ST = self.formata_numero(vl_out_cred_st, size=1,
                                                              separador=',')
                reg_E210.VL_AJ_CREDITOS_ST = '0,00'
                reg_E210.VL_RETENCAO_ST = self.formata_numero(vl_retencao_st, size=1,
                                                              separador=',')
                reg_E210.VL_OUT_DEB_ST = '0,00'
                reg_E210.VL_AJ_DEBITOS_ST = '0,00'
                reg_E210.VL_SLD_DEV_ANT_ST = '0,00'
                reg_E210.VL_DEDUCOES_ST = '0,00'
                reg_E210.VL_ICMS_RECOL_ST = '0,00'
                reg_E210.VL_SLD_CRED_ST_TRANSPORTAR = self.formata_numero(
                    vl_sld_cred_st_transportar, size=1, separador=',')
                reg_E210.DEB_ESP_ST = '0,00'
                reg_E210.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E210'
            return mensagem

        def _registroE250(uf):
            # print '-----------------------INI REG E250'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            filtro = {'nf_ids': movimento_notas, 'uf': uf}
            dados = obj_sql.sql_regE250(cr, filtro)
            for dado in dados:
                mes_ref = dado.get('mes_ref')
                reg_E250.COD_OR = dado.get('cod_or')
                reg_E250.VL_OR = '0,00'
                if dia_vencimento_icms:
                    vencimento = str(dia_vencimento_icms).zfill(2)
                    dt_vencimento = parse(data_ini) + relativedelta(months=1)
                    reg_E250.DT_VCTO = vencimento + str(dt_vencimento.month).zfill(2) + str(
                        dt_vencimento.year)
                reg_E250.COD_REC = uf == 'MG' and codigo_receita or (dado.get('cod_rec') or '')
                if sped_ano_inicial >= 2011 and sped_mes_inicial >= 1:
                    reg_E250.MES_REF = mes_ref if len(mes_ref) == 6 else '0' + mes_ref
                reg_E250.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E250'
            return mensagem

        def _registroE300():
            for estado in set([i.keys()[0] for i in apuracao_difal]):
                reg_E300.REG = 'E300'
                reg_E300.UF = estado
                reg_E300.DT_INI = self.formata_data(data_ini)
                reg_E300.DT_FIN = self.formata_data(data_fim)
                reg_E300.write_line()
                if cod_versao >= '011':
                    _registroE310_layout11(estado)
                else:
                    _registroE310(estado)

        def _registroE310_layout11(uf):
            reg_E310.REG = 'E310'
            reg_E310.IND_MOV_FCP_DIFAL = '1'
            reg_E310.VL_SLD_CRED_ANT_DIFAL = '0,00'
            reg_E310.VL_TOT_DEBITOS_DIFAL = self.formata_numero(sum(map(
                lambda i: i.get(uf) or 0.0, apuracao_difal)), size=1, separador=',')
            reg_E310.VL_OUT_DEB_DIFAL = '0,00'
            reg_E310.VL_TOT_CREDITOS_DIFAL = '0,00'
            reg_E310.VL_OUT_CRED_DIFAL = '0,00'
            reg_E310.VL_SLD_DEV_ANT_DIFAL = reg_E310.VL_TOT_DEBITOS_DIFAL
            reg_E310.VL_DEDUCOES_DIFAL = '0,00'
            reg_E310.VL_RECOL = reg_E310.VL_TOT_DEBITOS_DIFAL
            reg_E310.VL_SLD_CRED_TRANSPORTAR = '0,00'
            reg_E310.DEB_ESP_DIFAL = '0,00'
            reg_E310.VL_SLD_CRED_ANT_FCP = '0,00'
            reg_E310.VL_TOT_DEB_FCP = '0,00'
            reg_E310.VL_OUT_DEB_FCP = '0,00'
            reg_E310.VL_TOT_CRED_FCP = '0,00'
            reg_E310.VL_OUT_CRED_FCP = '0,00'
            reg_E310.VL_SLD_DEV_ANT_FCP = '0,00'
            reg_E310.VL_DEDUCOES_FCP = '0,00'
            reg_E310.VL_RECOL_FCP = '0,00'
            reg_E310.VL_SLD_CRED_TRANSPORTAR_FCP = '0,00'
            reg_E310.DEB_ESP_FCP = '0,00'
            reg_E310.write_line()
            _registroE316(uf)

        def _registroE310(uf):
            reg_E310.REG = 'E310'
            reg_E310.IND_MOV_DIFAL = '1'
            reg_E310.VL_SLD_CRED_ANT_DIFAL = '0,00'
            reg_E310.VL_TOT_DEBITOS_DIFAL = self.formata_numero(
                sum(map(lambda i: i.get(uf) or 0.0, apuracao_difal)), size=1, separador=',')
            reg_E310.VL_OUT_DEB_DIFAL = '0,00'
            reg_E310.VL_TOT_DEB_FCP = '0,00'
            reg_E310.VL_TOT_CREDITOS_DIFAL = '0,00'
            reg_E310.VL_TOT_CRED_FCP = '0,00'
            reg_E310.VL_OUT_CRED_DIFAL = '0,00'
            reg_E310.VL_SLD_DEV_ANT_DIFAL = reg_E310.VL_TOT_DEBITOS_DIFAL
            reg_E310.VL_DEDUCOES_DIFAL = '0,00'
            reg_E310.VL_RECOL = reg_E310.VL_TOT_DEBITOS_DIFAL
            reg_E310.VL_SLD_CRED_TRANSPORTAR = '0,00'
            reg_E310.DEB_ESP_DIFAL = '0,00'
            reg_E310.write_line()
            _registroE316(uf)

        def _registroE316(uf):
            reg_E316.COD_OR = '003'
            reg_E316.VL_OR = reg_E310.VL_RECOL
            mes = datetime.strptime(data_ini, '%Y-%m-%d') + relativedelta(months=1)
            reg_E316.DT_VCTO = formataString(dia_vencimento_icms, 2, '0', 'E') + '%s%s' % (
                str(mes.month).zfill(2), data_ini[:4]) if dia_vencimento_icms else ''
            if uf == reg_0000.UF:
                reg_E316.COD_REC = brw_fispar.fispar_codigo_difal_origem or ''
            else:
                reg_E316.COD_REC = brw_fispar.fispar_codigo_difal_destino or ''
            reg_E316.NUM_PROC = ''
            reg_E316.IND_PROC = ''
            reg_E316.PROC = ''
            reg_E316.TXT_COMPL = ''
            reg_E316.MES_REF = self.formata_data(data_ini)[2:]
            reg_E316.write_line()

        def _registroE500():
            # print '-----------------------INI REG E500'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            mensagem = ''
            if not tipo_atividade == 'I':  # Industria e equiparados
                # mensagem += _('* Apuracao do IPI nao lancada.\n')
                pass
            else:
                args_ipi = [('tipo', '=', 'I'), ('company_id', '=', val['company_id'])]
                args_ipi.extend(args_periodo_mes_e_ano)
                ids_apuracao_ipi = obj_apuracao_imposto.search(cr, uid, args_ipi)
                valor_outros_debitos_ipi = 0.0
                valor_outros_creditos_ipi = 0.0
                valor_estorno_creditos_ipi = 0.0
                valor_saldo_credor_ant_ipi = 0.0
                if ids_apuracao_ipi:
                    for id_apuracao_ipi in ids_apuracao_ipi:
                        valor_saldo_credor_ant_ipi += obj_apuracao_imposto.browse(
                            cr, uid, id_apuracao_ipi).saldo_credor
                        read_apuracao_ipi = obj_apuracao_imposto.read(cr, uid, id_apuracao_ipi,
                                                                      ['outros_debitos_ids',
                                                                       'estorno_creditos_ids',
                                                                       'outros_creditos_ids',
                                                                       'estorno_debitos_ids'],
                                                                      context=context)

                        browse_outros_debitos = obj_outros_debitos.browse(cr, uid,
                                                                          read_apuracao_ipi[
                                                                              'outros_debitos_ids'])
                        for i in browse_outros_debitos:
                            valor_outros_debitos_ipi += i.valor

                        browse_outros_creditos = obj_outros_creditos.browse(
                            cr, uid, read_apuracao_ipi['outros_creditos_ids'])
                        for i in browse_outros_creditos:
                            valor_outros_creditos_ipi += i.valor
                reg_E500.REG = 'E500'
                reg_E500.IND_APUR = '0'
                reg_E500.DT_INI = self.formata_data(data_ini)
                reg_E500.DT_FIN = self.formata_data(data_fim)
                reg_E500.write_line()
                _registroE510(movimento_notas)

                dadosE520 = {
                    'valor_outros_debitos_ipi': valor_outros_debitos_ipi,
                    'valor_estorno_creditos_ipi': valor_estorno_creditos_ipi,
                    'valor_saldo_credor_ant_ipi': valor_saldo_credor_ant_ipi,
                    'valor_outros_creditos_ipi': valor_outros_creditos_ipi,
                    'nf_ids': movimento_notas
                }
                _registroE520(dadosE520)

            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E500'
            return mensagem

        def _registroE510(nf_ids):
            # print '-----------------------INI REG E510'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            dados = obj_sql.sql_regE510(cr, nf_ids)
            for i in dados:
                reg_E510 = bloco_e.registro_E510()
                reg_E510.CFOP = i.get('cfop')
                reg_E510.CST_IPI = i.get('cst')
                reg_E510.VL_CONT_IPI = self.formata_numero(i.get('vl_cont_ipi') or 0, size=1,
                                                           separador=',')
                reg_E510.VL_BC_IPI = self.formata_numero(i.get('vl_bc_ipi') or 0, size=1,
                                                         separador=',')
                reg_E510.VL_IPI = self.formata_numero(i.get('vl_ipi') or 0, size=1, separador=',')
                reg_E510.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E510'

        def _registroE520(dados):
            # print '-----------------------INI REG E520'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            saldo_credor_transportar_ipi = 0.0
            valor_outros_debitos_ipi = dados.get('valor_outros_debitos_ipi')
            valor_estorno_creditos_ipi = dados.get('valor_estorno_creditos_ipi')
            valor_saldo_credor_ant_ipi = dados.get('valor_saldo_credor_ant_ipi')
            valor_outros_creditos_ipi = dados.get('valor_outros_creditos_ipi')
            soma_ipi = total_geral_debitos + valor_outros_debitos_ipi - (
                total_geral_creditos + valor_estorno_creditos_ipi + valor_saldo_credor_ant_ipi)
            saldo_devedor = 0
            if soma_ipi >= 0:
                saldo_devedor = soma_ipi
                saldo_credor_transportar_ipi = 0.0
            elif soma_ipi < 0:
                saldo_devedor = 0.0
                saldo_credor_transportar_ipi = soma_ipi * -1

            if saldo_devedor < 0:
                saldo_devedor = 0.0
                saldo_credor_transportar_ipi = saldo_devedor * -1

            dados = obj_sql.sql_regE520(cr, dados.get('nf_ids')) or {}
            if dados:
                dados = dados[0]

            reg_E520 = bloco_e.registro_E520()
            reg_E520.VL_SD_ANT_IPI = self.formata_numero(valor_saldo_credor_ant_ipi, size=1,
                                                         separador=',')
            reg_E520.VL_DEB_IPI = self.formata_numero(total_geral_debitos, size=1, separador=',')
            reg_E520.VL_CRED_IPI = self.formata_numero(dados.get('vl_cred_ipi'), size=1,
                                                       separador=',')
            reg_E520.VL_OD_IPI = self.formata_numero(valor_outros_debitos_ipi, size=1,
                                                     separador=',')
            reg_E520.VL_OC_IPI = self.formata_numero(valor_outros_creditos_ipi, size=1,
                                                     separador=',')
            reg_E520.VL_SC_IPI = self.formata_numero(dados.get('vl_cred_ipi'), size=1,
                                                     separador=',')
            reg_E520.VL_SD_IPI = self.formata_numero(saldo_devedor, size=1, separador=',')
            reg_E520.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG E520'

        # Metodos do Bloco H
        def _registroH001():
            mensagem = ''
            reg_H001.REG = 'H001'
            if not val['gerar_inventario'] and not pythonsped.efd.bloco_c.quantidade_registros.get(
                    'C186'):
                reg_H001.IND_MOV = '1'
                reg_H001.write_line()
            else:
                reg_H001.IND_MOV = '0'
                reg_H001.write_line()
                _registroH005()

            return mensagem

        def _registroH005():
            # print '-----------------------INI REG H005'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            _company_id = val['company_id']
            _inventario_capa_id = val.get('inventario_id')
            if _inventario_capa_id:
                _inventario_capa_id = _inventario_capa_id[0]
                args_company_id = [('company_id', '=', _company_id)]
                est_par_id = obj_est_par.search(cr, uid, args_company_id, context=context)
                read_est_par = obj_est_par.read(cr, uid, est_par_id, context=context)
                if read_est_par:
                    _custo_iventario = read_est_par[0]['custo_inventario']
                    dados = obj_sql.sql_regH005(cr, _custo_iventario, _inventario_capa_id)
                    if dados:
                        for dado in dados:
                            reg_H005.REG = 'H005'
                            reg_H005.DT_INV = self.formata_data(dado.get('dt_inv'))
                            reg_H005.VL_INV = self.formata_numero(dado.get('vl_inv'), size=1,
                                                                  separador=',')
                            reg_H005.MOT_INV = val['motivo_inventario']
                            reg_H005.write_line()
                            _registroH010(_custo_iventario, _inventario_capa_id)
                    else:
                        reg_H005.REG = 'H005'
                        reg_H005.DT_INV = self.formata_data(val.get('sped_data_final'))
                        reg_H005.VL_INV = '0'
                        reg_H005.MOT_INV = val['motivo_inventario']
                        reg_H005.write_line()
                        _registroH010(_custo_iventario, _inventario_capa_id)
            if sped_mes_inicial == 1 and pythonsped.efd.bloco_c.quantidade_registros.get(
                    'C186'):
                reg_H005.REG = 'H005'
                reg_H005.DT_INV = self.formata_data((parse(
                    sped_data_inicial) - relativedelta(days=1)).strftime('%Y-%m-%d'))
                reg_H005.VL_INV = self.formata_numero(0, size=1, separador=',')
                reg_H005.MOT_INV = '06'
                reg_H005.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG H005'

        def _registroH010(custo, inventario_id):
            # print '-----------------------INI REG H010'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            dados = obj_sql.sql_regH010(cr, custo, inventario_id)
            if dados:
                for dado in dados:
                    reg_H010.REG = 'H010'
                    reg_H010.COD_ITEM = dado.get('cod_item')
                    reg_H010.UNID = dado.get('unid')
                    reg_H010.QTD = self.formata_numero(dado.get('qtd'), size=3, separador=',',
                                                       dec=3)
                    reg_H010.VL_UNIT = self.formata_numero(dado.get('vl_unit'), size=6,
                                                           separador=',', dec=6)
                    qtd = float(float_round(dado.get('qtd'), 5))
                    vl_unit = float(float_round(dado.get('vl_unit'), 5))
                    vl_item = float(float_round(qtd * vl_unit, 2))
                    reg_H010.VL_ITEM = self.formata_numero(vl_item, size=2,
                                                           separador=',', dec=2)
                    reg_H010.IND_PROP = '0'
                    reg_H010.COD_PART = ''
                    reg_H010.TXT_COMPL = ''
                    if perfil_sped in ('A', 'B'):
                        reg_H010.COD_CTA = '3101'
                    else:
                        reg_H010.COD_CTA = ''
                    reg_H010.VL_ITEM_IR = self.formata_numero(vl_item, size=2,
                                                              separador=',', dec=2)
                    reg_H010.write_line()
                    if val['motivo_inventario'] != '01':
                        _registroH020(inventario_id, dado.get('grade_id'))
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG H010'

        def _registroH020(inventario_id, grade_id):
            # print '-----------------------INI REG H020'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            reg_H020 = bloco_h.registro_H020()
            produto_inventario = self.pool.get('estoque.inventario.oficial')
            item_inv_id = produto_inventario.search(
                cr, uid, [('capa_id', '=', inventario_id), ('grade_id', '=', grade_id)],
                context=context)[0]
            item = produto_inventario.browse(cr, uid, item_inv_id, context=context)
            if not item.saldo:
                self.set_mensagem("REG H020: Produto: {produto} saldo no inventario".format(
                                  produto=item.produto_id.codigo + ' - ' + item.produto_id.name))
                return

            if reg_H005.MOT_INV in ('02', '04'):
                encargo = filter(
                    lambda i: all([i.encargo_id.sigla == 'ICMS', i.tipo_tributacao == 'T']),
                    item.produto_id.tributacao_ids)
                tributo = None
                if encargo:
                    tributo = encargo[0].tributacao_id
                else:
                    self.set_mensagem(
                        "REG H020: Produto: {produto} sem tributacao Todos/Consumidor final"
                        .format(produto=item.produto_id.codigo + ' - ' + item.produto_id.name))
            else:
                tributo = item.estiof_tributacao_id
            if not tributo:
                self.set_mensagem("REG H020: Produto: {produto} sem tributacao informada no "
                                  "inventario".format(produto=item.produto_id.codigo + ' - ' + item
                                                      .produto_id.name))
                return
            reg_H020.REG = 'H020'
            cst = item.produto_id.origem_mercadoria + tributo.cst_id.codigo
            reg_H020.CST_ICMS = cst
            reg_H020.BC_ICMS = self.formata_numero(item.produto_id.preco_venda, size=2,
                                                   separador=',', dec=2)
            reg_H020.VL_ICMS = \
                self.formata_numero(
                    self.pool.get('fiscal.nf.item.tributacao').
                    fisnit_calcula_vr_encargo(item.produto_id.preco_venda, tributo.aliquota),
                    size=2, separador=',', dec=2)
            reg_H020.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG H020'

        # Metodos do Bloco K
        def _registroK001(gerar_bloco_k, filtro_0200):
            # print '-----------------------INI REG K001'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            reg_K001.REG = 'K001'
            reg_K001.IND_MOV = '1'
            filtro = {'data_ini': data_ini,
                      'data_fim': data_fim}
            filtro.update(filtro_0200)
            dados_K200 = False
            if gerar_bloco_k:
                dados_K200 = obj_sql.sql_K200(cr, filtro)
            if dados_K200:
                reg_K001.IND_MOV = '0'
                reg_K001.write_line()
                _registroK100()
                _registroK200(dados_K200)
                if modulo_pcp_instalado:
                    _registroK230()
            else:
                reg_K001.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG K001'

        def _registroK100():
            reg_K100.REG = 'K100'
            reg_K100.DT_INI = self.formata_data(data_ini)
            reg_K100.DT_FIN = self.formata_data(data_fim)
            reg_K100.write_line()

        def _registroK200(dados):
            # print '-----------------------INI REG K200'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
            saldos = obj_estoque_produto_saldo.get_saldo_do_dia(
                cr, uid, list(set(map(lambda g: g.get('grade_id'), dados))), data_fim,
                context=context)
            for dado in dados:
                grade_id = dado.get('grade_id')
                quantidade = saldos.get(grade_id).get('saldo') \
                    if saldos.get(grade_id).get('saldo') > 0 else 0
                reg_K200.REG = 'K200'
                reg_K200.DT_EST = self.formata_data(data_fim)
                reg_K200.COD_ITEM = dado.get('cod_item')
                reg_K200.QTD = self.formata_numero(quantidade, size=1, separador=',')
                reg_K200.IND_EST = '0'
                reg_K200.COD_PART = ''
                reg_K200.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG K200'

        def _registroK230():
            # print '-----------------------INI REG K230'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            filtro = {'data_ini': data_ini,
                      'data_fim': data_fim}
            dados = obj_sql.sql_K230(cr, filtro)
            for dado in dados:
                reg_K230.REG = 'K230'
                reg_K230.DT_INI_OP = self.formata_data(dado.get('dt_ini_op'))
                reg_K230.DT_FIN_OP = self.formata_data(dado.get('dt_fin_op'))
                reg_K230.COD_DOC_OP = dado.get('cod_doc_op')
                reg_K230.COD_ITEM = dado.get('cod_item')
                reg_K230.QTD_ENC = dado.get('qtd_enc')
                reg_K230.write_line()
                _registroK235(dado.get('ordem_producao_id'))
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG K230'

        def _registroK235(ordem_producao_id):
            # print '-----------------------INI REG K235'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            filtro = {
                'ordem_producao_id': ordem_producao_id
            }
            dados = obj_sql.sql_K235(cr, filtro)
            for dado in dados:
                reg_K235.REG = dado.get('reg')
                reg_K235.DT_SAIDA = self.formata_data(dado.get('dt_saida'))
                reg_K235.COD_ITEM = dado.get('cod_item')
                reg_K235.QTD = dado.get('qtd')
                reg_K235.COD_INS_SUBST = ''
                reg_K235.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG K235'

        # Metodos do Bloco 1
        def _registro1001():
            mensagem = ''
            reg_1001.IND_MOV = '0'
            reg_1001.write_line()
            _registro1010()

            return mensagem

        def _registro1010():
            tipo_cartao = 'N'
            if movimento_eletronico_pagamento:
                tipo_cartao = 'S'
            reg_1010.IND_EXP = 'N'
            reg_1010.IND_CCRF = 'N'
            args_icms = [('tipo', '=', 'C'), ('company_id', '=', val['company_id'])]
            args_icms.extend(args_periodo_mes_e_ano)
            ids_apuracao = obj_apuracao_imposto.search(cr, uid, args_icms, context=context)
            if ids_apuracao:
                if valor_outros_creditos_e111:
                    reg_1010.IND_CCRF = 'S'
            reg_1010.IND_COMB = 'N'
            reg_1010.IND_USINA = 'N'
            reg_1010.IND_VA = 'N'
            reg_1010.IND_EE = 'N'
            reg_1010.IND_CART = tipo_cartao
            reg_1010.ND_FORM = 'N'
            reg_1010.IND_AER = 'N'
            if cod_versao and int(cod_versao) >= 13:
                reg_1010.IND_GIAF1 = 'N'
                reg_1010.IND_GIAF3 = 'N'
                reg_1010.IND_GIAF4 = 'N'
                reg_1010.IND_REST_RESSARC_COMPL_ICMS = 'N'
            if cod_versao and int(cod_versao) >= 15 \
                    and pythonsped.efd.bloco_0.quantidade_registros.get('C181'):
                reg_1010.IND_REST_RESSARC_COMPL_ICMS = 'S'

            reg_1010.write_line()
            _registro1200()
            _registro1250()
            _registro1255()
            if int(cod_versao) <= 15:
                _registro1600()
            if int(cod_versao) >= 16:
                _registro1601()

        def _registro1200():
            """
            REGISTRO 1200: CONTROLE DE CREDITOS FISCAIS - ICMS.
            Este registro demonstra a conta corrente dos creditos fiscais de ICMS, controlados 
            extra-apuracao. Cada UF determinara a obrigatoriedade de apresentacao deste registro.
            
            Nro Campo        Descricao                                           Tipo Tam  Dec Obrig.
            -- ------------ --------------------------------------------------- ---- ---  --- ------
            01 REG          Texto fixo contendo "1200"                          C    004   -  O
            02 COD_AJ_APUR  Codigo de ajuste, conforme informado na Tabela      C    008*  -  O
                            indicada no item 5.1.1.                         
            03 SLD_CRED     Saldo de creditos fiscais de periodos anteriores    N     -   02  O
            04 CRED_APR     Total de credito apropriado no mes                  N     -   02  O
            05 CRED_RECEB   Total de creditos recebidos por transferência       N     -   02  O
            06 CRED_UTIL    Total de creditos utilizados no período             N     -   02  O
            07 SLD_CRED_FIM Saldo de credito fiscal acumulado a transportar     N     -   02  O
                            para o periodo seguinte                             
            =======================================================================================
            (*) Somente poderao ser utilizados os codigos nos quais o quarto caractere seja igual 
            a “9”
            """
            # print '-----------------------INI REG 1200'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            apuracao_icms_ids = obj_apuracao_imposto.search(
                cr, uid, [('mes', '=', sped_mes_inicial), ('tipo', '=', 'C'),
                          ('ano', '=', sped_ano_inicial),
                          ('company_id', '=', val.get('company_id'))])
            for apuracao in obj_apuracao_imposto.browse(cr, uid, apuracao_icms_ids, context):
                sld_cred = saldo_credor_transportar = 0
                for reg1200 in obj_sql.sql_reg1200(cr, apuracao.id):
                    reg_1200 = bloco_1.registro_1200()
                    reg_1200.COD_AJ_APUR = reg1200.get('codigo_ajuste') or ''
                    reg_1200.SLD_CRED = self.formata_numero(
                        sld_cred, size=1, separador=',')
                    reg_1200.CRED_APR = self.formata_numero(
                        reg1200.get('valor'), size=1, separador=',')
                    reg_1200.CRED_RECEB = '0,00'
                    reg_1200.CRED_UTIL = self.formata_numero(
                        reg1200.get('valor'), size=1, separador=',')
                    reg_1200.SLD_CRED_FIM = self.formata_numero(
                        saldo_credor_transportar, size=1, separador=',')
                    reg_1200.write_line()

                    _registro1210(apuracao.id, reg_1200.COD_AJ_APUR)
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 1200'

        def _registro1210(apuracao, cod_aj_apur):
            """
            REGISTRO 1210: UTILIZAÇÃO DE CRÉDITOS FISCAIS – ICMS.
            Este registro deve ser apresentado para detalhar a utilização de créditos fiscais de 
            ICMS no período. O somatório dos valores do campo 04 deste registro deve corresponder 
            ao informado no campo 06 do registro 1200
.
            Nro Campo        Descricao                                           Tipo Tam  Dec Obrig.
            -- ------------ --------------------------------------------------- ---- ---  --- ------
            01 REG          Texto fixo contendo "1210"                          C    004   -  O
            02 TIPO_UTIL    Tipo de utilizacao do credito, conforme tabela      C    004*  -  O
                            indicada no item 5.5.
            03 NR_DOC       Numero do documento utilizado na baixa de creditos  C     -    -  OC
            04 VL_CRED_UTIL Total de credito utilizado                          N     -   02  O
            05 CHV_DOCe     Chave do Documento Eletronico                       N    044*  -  OC
            =======================================================================================
            """
            # print '-----------------------INI REG 1210'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            for reg1210 in obj_sql.sql_reg1210(cr, apuracao, cod_aj_apur):
                reg_1210 = bloco_1.registro_1210()
                reg_1210.TIPO_UTIL = reg1210.get('tipo_util') or ''
                reg_1210.NR_DOC = reg1210.get('nr_doc') or ''
                reg_1210.VL_CRED_UTIL = self.formata_numero(
                    reg1210.get('vl_cred_util'), size=1, separador=',')
                reg_1210.CHV_DOCe = reg1210.get('chv_doc_e') or ''
                reg_1210.write_line()
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 1210'

        def _registro1250():
            # if not c100_registro_filho_dados.get('c181'):
            #     return
            if not pythonsped.efd.bloco_0.quantidade_registros.get('C181'):
                return
            reg_1250.REG = '1250'
            reg_1250.VL_CREDITO_ICMS_OP = '0,00'
            reg_1250.VL_ICMS_ST_REST = '0,00'
            reg_1250.VL_FCP_ST_REST = '0,00'
            reg_1250.VL_ICMS_ST_COMPL = '0,00'
            reg_1250.VL_FCP_ST_COMPL = '0,00'
            reg_1250.write_line()

        def _registro1255():
            # if not c100_registro_filho_dados.get('c181'):
            #     return
            # cod_mot_list = list(set(map(lambda cm: cm.get('cod_mot_rest_compl'),
            #                             c100_registro_filho_dados.get('c181'))))
            # for cod in cod_mot_list:
            if not pythonsped.efd.bloco_0.quantidade_registros.get('C181'):
                return
            reg_1255.REG = '1255'
            reg_1255.COD_MOT_REST_COMPL = 'MG500'
            reg_1255.VL_CREDITO_ICMS_OP_MOT = '0,00'
            reg_1255.VL_ICMS_ST_REST_MOT = '0,00'
            reg_1255.VL_FCP_ST_REST_MOT = '0,00'
            reg_1255.VL_ICMS_ST_COMPL_MOT = '0,00'
            reg_1255.VL_FCP_ST_COMPL_MOT = '0,00'
            reg_1255.write_line()

        def _registro1600():
            # print '-----------------------INI REG 1600'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            pessoa_ids = []
            for dado in movimento_eletronico_pagamento:
                if not dado.get('pessoa_id') in pessoa_ids:
                    reg_1600.REG = '1600'
                    reg_1600.COD_PART = dado.get('pessoa_id')
                    reg_1600.TOT_CREDITO = self.formata_numero(dado.get('vl_credito'),
                                                               separador=',',
                                                               size=1)
                    reg_1600.TOT_DEBITO = self.formata_numero(dado.get('vl_debito'), separador=',',
                                                              size=1)
                    reg_1600.write_line()
                    pessoa_ids.append(dado.get('pessoa_id'))
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 1600'

        def _registro1601():
            # print '-----------------------INI REG 1601'
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            pessoa_ids = []
            dados_1601 = obj_sql.sql_reg1601(
                cr, dt_ini, dt_fim, modulo_ecf_instalado, val['company_id'])
            for dado in dados_1601:
                if dado.get('pessoa_id') in pessoa_ids:
                    continue
                reg_1601.REG = '1601'
                reg_1601.COD_PART_IP = dado.get('pessoa_id')
                reg_1601.COD_PART_IT = ''
                reg_1601.TOT_VS = self.formata_numero(dado.get('valor'), separador=',', size=1)
                reg_1601.TOT_ISS = '0,00'
                reg_1601.TOT_OUTROS = '0,00'
                reg_1601.write_line()
                pessoa_ids.append(dado.get('pessoa_id'))
            # print time.strftime('%Y-%m-%d %H:%M:%S')
            # print '-----------------------FIM REG 1601'

        # print '-----------------------INI REG BLOCO G'
        # print time.strftime('%Y-%m-%d %H:%M:%S')
        msg += self.blocoG(cr, uid, sped_data_inicial, sped_data_final, val.get('company_id'),
                           args_periodo_mes_e_ano, sped_mes_inicial, sped_ano_inicial,
                           reg_0001, movimento_notas, context)
        # print time.strftime('%Y-%m-%d %H:%M:%S')
        # print '-----------------------FIM REG BLOCO G'

        # Bloco 0
        msg += _registro0000()
        msg += _registro0001()
        if read_pessoa.admpes_jur_tipo_atividade == 'I' and cod_versao and int(cod_versao) >= 14:
            msg += _registro0002()
        msg += _registro0005()
        msg += _registro0015()
        msg += _registro0100()
        msg += _registro0150()
        msg += self.bloco_g_registro0150(cr, uid, val.get('company_id'), obj_sql, movimento_notas,
                                         context)
        msg += _registro0190()
        msg += self.bloco_g_registro0190(cr, uid, reg_0001, context)
        msg_0200, filtro_0200 = _registro0200()
        msg += msg_0200
        msg += self.bloco_g_registro0200(cr, uid, reg_0000, obj_sql, sped_data_final,
                                         filtro_0200.get('movimento_notas') or [-1],
                                         context)
        msg += self.bloco_g_registro0300(cr, uid, val.get('company_id'), args_periodo_mes_e_ano,
                                         context)
        _registro0400()
        if reg_0000.IND_PERFIL != 'C':
            _registro0450()
        msg += _registro0460()

        msg += self.bloco_g_registro0500(cr, uid, val.get('company_id'), args_periodo_mes_e_ano,
                                         sped_data_inicial, context)
        msg += self.bloco_g_registro0600(cr, uid, val.get('company_id'), args_periodo_mes_e_ano,
                                         sped_data_inicial, context)

        # Bloco B
        if cod_versao and int(cod_versao) >= 13:
            msg += _registroB001()

        # Bloco C
        movimentoC001 = _registroC001()
        msg_c100, c100_registro_filho_dados = _registroC100()
        msg += msg_c100
        _registroC300()

        # Cupons
        if movimento_cupons:
            msg += _registroC400()
        if reg_0000.IND_PERFIL != 'C':
            _registroC500()

        # Bloco D
        movimentoD001 = _registroD001()

        # Bloco E
        msg += _registroE001()
        msg += _registroE100()
        msg += _registroE110()
        msg += _registroE200(DATE_SPED_INICIAL, DATE_SPED_FINAL, tipo_atividade)
        _registroE300()
        msg += _registroE500()

        # Chamando metodos do blco H
        msg += _registroH001()

        # Bloco K
        if DATE_SPED_FINAL.year > 2015:
            _registroK001(val['gerar_bloco_k'], filtro_0200)

        # Bloco 1
        msg += _registro1001()

        # Registro 9001  abertura do Bloco 9
        reg_9001 = bloco_9.registro_9001()

        # Indicador de movimento:0- Bloco com dados informados;1- Bloco sem dados informados.
        reg_9001.IND_MOV = '0'
        reg_9001.write_line()

        msg += ''.join(self.get_mensagem())
        if msg:
            try:
                self.write(cr, uid, ids, {'informacao': msg.encode('utf-8', 'strict')}, context)
            except UnicodeDecodeError:
                self.write(cr, uid, ids, {'informacao': msg.decode('utf-8', 'strict')}, context)
            finally:
                self._mensagens = []
        arq = bloco_arquivo()
        arquivo = arq.monta_arquivo()
        self.write(cr, uid, ids, {'dados': base64.encodestring(arquivo.encode('utf-8', 'strict')),
                                  'sped_gerado': True,
                                  'state': 'E'}, context)
        _tFim = datetime.now()
        _logger.info('Fim do Sped ICMS/IPI! (%s)' % str(_tFim - _tInicio))
        return True

    def registroC114(self, cr, uid, nf_id, context=None):
        """
        Metodo responsavel por gerar os registros C114. Cupons referenciados nas notas fiscais do
        registro C100.
        Metodo sobreescrito no modulo ECF.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da Nota Fiscal.
        :type nf_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: Mensagem da descricao de algum possivel problema perante a geracao.
        :rtype: str
        """
        return str()

    def bloco_g_registro0150(self, cr, uid, company_id, obj_sql_sped, movimento_notas,
                             context=None):
        return str()

    def bloco_g_registro0190(self, cr, uid, reg_0001, context=None):
        return str()

    def bloco_g_registro0200(self, cr, uid, reg_0000, obj_sql_sped, sped_data_final,
                             movimento_notas, context=None):
        return str()

    def bloco_g_registro0300(self, cr, uid, company_id, args_periodo_mes_e_ano, context=None):
        return str()

    def bloco_g_registro0500(self, cr, uid, company_id, args_periodo_mes_e_ano, sped_data_inicial,
                             context=None):
        return str()

    def bloco_g_registro0600(self, cr, uid, company_id, args_periodo_mes_e_ano,
                             sped_data_inicial, context=None):
        return str()

    def blocoG(self, cr, uid, sped_data_inicial, sped_data_final, company_id,
               args_periodo_mes_e_ano, sped_mes_inicial, sped_ano_inicial, reg_0001,
               movimento_notas, context=None):
        """
        Metodo responsavel pela dados do registro tipo G do SPED Fiscal. Para gera-los, o modulo
        'patrimonio_fiscal' deve estar instalado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param sped_data_inicial: dados da data de inicio da geracao do sped.
        :type sped_data_inicial: str
        :param sped_data_final: dados da data final da geracao do sped.
        :type sped_data_final: str
        :param company_id: identificador da empresa, id da tabela res_company.
        :param args_periodo_mes_e_ano: dados do periodo de pesquisa do CIAP.
        :type args_periodo_mes_e_ano: list
        :param sped_mes_inicial: numero inicial do mes de periodo do SPED.
        :type sped_mes_inicial: int
        :param sped_ano_inicial: numero inicial do ano de periodo do SPED.
        :type sped_ano_inicial: int
        :param reg_0000: dados do registro tipo 0000 do SPED.
        :type reg_0000: object
        :param reg_0001: dados do registro tipo 0001 do SPED.
        :type reg_0001: object
        :param movimento_notas: lista de id da tabela fiscal_nf_capa, identificadores das notas
        fiscais que serao consideradas no SPED.
        :type movimento_notas: list
        :param obj_sql_sped: instancia da classe SqlSpedFiscal.
        :type obj_sql_sped: object
        :param context: dados de valores contextuais.
        :type context: dict
        :return: Descricao de possiveis problemas encontrados na geracao do registro tipo G.
        :rtype: str
        """
        context = context and context.copy() or {}

        # Registro g001  abertura do Bloco g
        reg_G001 = bloco_g.registro_G001()
        reg_G001.IND_MOV = '1'
        reg_G001.write_line()

        return ''

    @staticmethod
    def get_versao_layout(ano):
        versao = {'2014': '008', '2015': '009', '2016': '010', '2017': '011', '2018': '012',
                  '2019': '013', '2020': '014', '2021': '015', '2022': '016', '2023': '017',
                  '2024': '018'}
        cod_versao = versao.get(str(ano))
        if not cod_versao:
            if ano < 2014:
                cod_versao = versao.get(str(2014))
            else:
                cod_versao = versao.get(str(2024))
        return cod_versao

    @staticmethod
    def get_cod_sit(codigo):
        if codigo == '3':
            return '00'
        elif codigo == 'C':
            return '02'
        elif codigo == 'I':
            return '05'
        else:
            return ''

    def sped_get_conta_contabil(self, cr, uid, nf_id=False, movimento_item_id=False,
                                natureza_id=False, cfop_id=False, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Encontra a conta contabil analitica para a natureza da nota fiscal (nf_id), movimento do
        item no estoque (movimento_item_id) ou da natureza da emissao (natureza_id), informados.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_id: int
        :param movimento_item_id: id da tabela estoque_movimento_itens, identificador do item
        movimentado no estoque.
        :param natureza_id: id da tabela fiscal_cfop_natureza, identificador da natureza da emissao
        da nota fiscal.
        :type natureza_id: int
        :type movimento_item_id: int
        :param cfop_id: id da tabela fiscal_cfop, idetificador do CFOP.
        :type cfop_id: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: codigo da conta contabil.
        :rtype: str
        """
        return str()

    def limpar_consulta(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.unlink(cr, uid, ids, context=context)
        return {'name': 'Gerar SPED',
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': False,
                'res_model': 'fiscal.sped',
                'target': 'inline',
                'type': 'ir.actions.act_window'}

    # noinspection PyUnusedLocal,PyMethodMayBeStatic
    def onchange_mes_inventario(self, cr, user, ids, month, context=None):
        """
        Acrescenta zeros a esquerda para valores do mes menores que dez. Valida o mes: 01 a 12.

        :param cr: a linha atual do cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de ID's
        :param month: mes de referencia
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type user: int
        :type ids: long|list[long]
        :type month: str
        :type context: dict
        :return: numero do mes de 01 a 12.
        :rtype str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        # Somente numeros de dois digitos de 01 a 12
        month = '' if not month or not month.isdigit() or month > '12' else str(month).zfill(2)[:2]
        result = dict(value=dict(mes_inventario=month))

        return result

    def get_mensagem(self):
        return self._mensagens

    def set_mensagem(self, mensagem):
        html = '<h4 style="color:#bf090d">' + mensagem + '</h4><br/>'
        self._mensagens.append(html)

fiscal_sped()
