# -*- coding: utf-8 -*-
##############################################################################
#   SPED CONTRIBUICOES
##############################################################################

from sped_sql.sql_sped_contribuicoes import SqlSpedContribuicoes
from pythonsped.contribuicoes import *
from openerp.tools.translate import _
from openerp.osv import osv, fields
from dateutil.parser import parse
from openerp import SUPERUSER_ID
from datetime import datetime
import base64
import logging

_logger = logging.getLogger('fiscal.sped.contribuicoes')


class groupby(dict):
    def __init__(self, seq, key=lambda x: x):
        super(groupby, self).__init__(seq, key=lambda x: x)
        for value in seq:
            k = key(value)
            self.setdefault(k, []).append(value)
    __iter__ = dict.iteritems


class fiscal_sped_contribuicoes(osv.TransientModel):
    def _get_company(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, context=context)
        return company_id or False

    def _get_companies(self, cr, uid, context=None):
        context = context and context.copy() or {}
        c = self._get_company(cr, uid, context)
        if c:
            return [c]
        return False

    def _get_cooperativa(self, cr, uid, context=None):
        context = context and context.copy() or {}
        natureza = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', ['fispar_indicador_natureza_pessoa_juridica'], context=context)

        return natureza == '01'

    _name = 'fiscal.sped.contribuicoes'
    _columns = {
        'state': fields.selection(string='Status',
                                  selection=[('A', 'Aguardando geracao do arquivo'),
                                             ('E', 'Arquivo gerado')], required=True, size=1),
        'company_ids': fields.many2many('res.company', 'res_company_sped_rel', 'id', 'company_id', 'Empresas',
                                        required=True, domain=[]),
        'dados': fields.binary('Clique para abrir o arquivo ', readonly=True),
        'tipo_arquivo': fields.selection([('0', 'Original'), ('1', 'Retificadora')], 'Finalidade do Arquivo',
                                         required=True),
        'informacao': fields.text('Informacoes'),
        'file': fields.char('Nome do arquivo', size=50, readonly=False, required=True),
        'sped_gerado': fields.boolean('SPED gerado'),
        'sped_data_inicial': fields.date('Data Inicial', required=True),
        'sped_data_final': fields.date('Data Final', required=True),
        'sped_pis_exclusao_geral': fields.float('Pis exclusao geral'),
        'sped_pis_exclusao_especifica': fields.float('Pis exclusao especifica'),
        'sped_cofins_exclusao_geral': fields.float('Cofins exclusao geral'),
        'sped_cofins_exclusao_especifica': fields.float('Cofins exclusao especifica'),
        'sped_empresa_cooperativa': fields.boolean(string='Cooperativa'),
        'sped_num_recibo_escrituracao_ant': fields.char('Recibo escrituracao anterior', size=41),

    }
    _defaults = {
        'state': 'A',
        'company_ids': _get_companies,
        'file': lambda *a: 'sped_contribuicoes.txt',
        'tipo_arquivo': '0',
        'sped_gerado': False,
        'sped_empresa_cooperativa': _get_cooperativa

    }

    @staticmethod
    def formata_data(data):
        """
        Aceita-se para data tanto uma string no formato '%Y-%m-%d %H-%M-%s'
        ou um Datetime, para facilitar a formatacao.
        """
        if isinstance(data, datetime):
            return data.strftime('%d%m%Y')
        else:
            return datetime(int(data[:4]), int(data[5:7]),
                            int(data[8:10])).strftime('%d%m%Y')

    def formata_numero(self, value, size=9, dec=2, separador='.'):
        mask = '%.{dec}f'.replace('{dec}', str(dec))
        ret = mask % value if value else mask % 0.00
        ret = ret.rjust((size + dec + 1), '0') if size else ret
        if separador != '.':
            ret = ret.replace('.', separador)
        return ret

    def remove_ponto(self, texto):
        texto = texto.replace('.', '').replace('/', '').replace('-', '')
        return texto

    def formata_para_digitos(self, digito):
        if digito:
            digito_formatado = ''
            for i in digito:
                if i.isdigit() or i.isalpha():
                    digito_formatado += i
        else:
            digito_formatado = ''
        return digito_formatado

    def filtra_ids_itens(self, cr, uid, obj_fiscal, movimento, context):
        """
        Funcao: Retornar os ids dos movimentos que nao foram proprios e modelo 55
                Ira servir para filtro dos registros 0190 e 0200, para que nao
                seja gerado estes registros para itens que nao foram referenciados em outros blocos
        Parametros: obj_fiscal (fiscal.nf.capa)
                    movimento (Dicionario contendo os ids dos moviemntos de notas e cupons)
        """
        context = context and context.copy() or {}
        movimento_notas = movimento.get('movimento_notas')
        movimento_cupons = movimento.get('movimento_cupons')
        campos = ['id', 'nf_propria', 'tipo', 'modelo_55']
        read_fiscal_nf_capa = obj_fiscal.read(cr, uid, movimento_notas, campos, context=context)
        notas_id = []
        cupons_id = []

        if not movimento_notas:
            notas_id.append(-1)
        else:
            for i in read_fiscal_nf_capa:
                if not i.get('nf_propria'):
                    notas_id.append(i.get('id'))
                else:
                    if not i.get('modelo_55'):
                        notas_id.append(i.get('id'))
            if not notas_id:
                notas_id.append(-1)
        if not movimento_cupons:
            pass
        else:
            cupons_id = movimento_cupons
        filtro = {
            'movimento_notas': notas_id,
            'movimento_cupons': cupons_id
        }
        return filtro

    def gera_natureza(self, cr, cfop):
        fiscal_codigo_natureza_credito = {
            '01': ['1102', '1113', '1117', '1118', '1121', '1251', '1403', '2102', '2113', '2117', '2118',
                   '2121', '2251', '2403', '3102', '3251'],
            '02': ['1101', '1111', '1116', '1120', '1122', '1126', '1128', '1401', '1556', '1651', '1653',
                   '2101', '2111', '2116', '2120', '2122', '2126', '2401', '2556', '2651', '2653', '3101',
                   '3126', '3651', '3653'],
            '03': ['1124', '1125', '1933', '2124', '2125', '2933'],
            '04': ['1252', '1253', '1254', '1255', '1256', '1257', '2252', '2253', '2254', '2255', '2256',
                   '2257'],
            '07': ['1352', '1353', '1354', '1355', '1356', '1932', '2352', '2353', '2354', '2355', '2356',
                   '2932', '3352', '3353', '3356'],
            '12': ['1201', '1202', '1410', '1411', '2201', '2202', '2410'],
            '14': ['1351', '2351', '3351']
        }
        fiscal_codigo_naturez_credito_descricao = {
            '01': 'Aquisicao de Bens para Revenda',
            '02': 'Aquisicao de Bens Utilizados como Insumo',
            '03': 'Aquisicao de Servicos Utilizados como Insumo',
            '04': 'Despesas de Energia Eletrica',
            '07': 'Armazenagem de Mercadoria e Frete na Operacao de Venda',
            '12': 'Devolucao de Vendas Sujeitas a Incidência Nao-Cumulativa',
            '14': 'Atividade de Transporte de Cargas - Subcontratacao de Servicos de Transporte'
        }
        dados = {}
        for codigo in fiscal_codigo_natureza_credito:
            if cfop in fiscal_codigo_natureza_credito.get(codigo):
                dados = {
                    'codigo': codigo,
                    'descricao': fiscal_codigo_naturez_credito_descricao.get(codigo)
                }
        return dados

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context.copy() or {}
        res = super(fiscal_sped_contribuicoes, self).read(cr, SUPERUSER_ID, ids, fields, context,
                                                          load)

        return res

    @staticmethod
    def sped_contribuicoes_get_versao_layout(ano):
        versao = {'2018': '004', '2019': '005', '2020': '006', '2021': '006'}
        ano_versao = versao.get(str(ano))
        if not ano_versao:
            if ano < 2018:
                ano_versao = '003'
            if ano >= 2021:
                ano_versao = '006'
        return ano_versao

    def gerar_sped(self, cr, uid, ids, context=None):
        """ Gera SPED Contribuicoes
        """
        _tInicio = datetime.now()
        _logger.info('Gerando Sped Contribuicoes...')
        global mensagem
        global CREDITO_PIS
        global CREDITO_COFINS
        global DEBITO_PIS
        global DEBITO_COFINS
        global MODULO_ECF_INSTALADO
        context = context and context.copy() or {}
        mensagem = ''
        CREDITO_PIS = 0.00
        CREDITO_COFINS = 0.00
        DEBITO_PIS = 0.00
        DEBITO_COFINS = 0.00
        if ids and isinstance(ids, list):
            ids = ids[0]
        browse_sped = self.browse(cr, uid, ids, context)
        data_inicial = browse_sped.sped_data_inicial
        data_final = browse_sped.sped_data_final + ' 23:59:59'
        nf_nao_fiscais = []
        modelos_nao_fiscais_ids = self.pool.get('fiscal.modelo.nf').search(
            cr, uid, [('fismnf_codigo', '>=', '90')], context=context
        )
        modelos_nao_fiscais_nf_series_ids = self.pool.get('fiscal.modelo.nf.series').search(
            cr, uid, [('modelo_nf_id', 'in', modelos_nao_fiscais_ids)], context=context
        )
        args_nao_fiscais = [('nf_propria', '=', True),
                            ('modelo_nf_series_id', 'in', modelos_nao_fiscais_nf_series_ids),
                            ('data_emissao', '>=', data_inicial),
                            ('data_emissao', '<=', data_final)]
        nf_nao_fiscais_propria = self.pool.get('fiscal.nf.capa').search(
            cr, uid, args_nao_fiscais, context=context
        )
        args_nao_fiscais = [('nf_propria', '=', False),
                            ('modelo_nf_id', 'in', modelos_nao_fiscais_ids),
                            ('data_saida_entrada', '>=', data_inicial),
                            ('data_saida_entrada', '<=', data_final)]
        nf_nao_fiscais_terceiro = self.pool.get('fiscal.nf.capa').search(
            cr, uid, args_nao_fiscais, context=context
        )
        nf_nao_fiscais.extend(nf_nao_fiscais_propria)
        nf_nao_fiscais.extend(nf_nao_fiscais_terceiro)
        MODULO_ECF_INSTALADO = self.pool.get("adm.utils").admutl_check_modulo_instalado(
            cr, uid, 'ecf', context=context)

        obj_company = self.pool.get('res.company')
        obj_adm_utl = self.pool.get('adm.utils')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_adm_par = self.pool.get('adm.parametro')
        obj_fiscal_parametro = self.pool.get('fiscal.parametro')
        company_ids = [x.id for x in browse_sped.company_ids] # Empresas selecionadas
        obj_sql = SqlSpedContribuicoes(cr, company_ids, data_inicial, data_final, nf_nao_fiscais)
        admpes_jur_tipo_atividade = ''
        empresas_cnpj = {}
        sql = """
            SELECT
                empresa.id AS id,
                pessoa.cpf_cnpj AS cnpj,
                pessoa.admpes_jur_tipo_atividade
            FROM
                res_company empresa
            LEFT JOIN
                adm_pessoa pessoa ON empresa.rescon_pessoa_id = pessoa.id
            WHERE
                empresa.id IN ({company_ids}) 
        """.format(company_ids=', '.join(map(lambda c: str(c), company_ids)))
        cr.execute(sql)
        pessoa_empresa_dados = cr.dictfetchall()
        for empresa in pessoa_empresa_dados:
            empresas_cnpj.update({empresa.get('id'): empresa.get('cnpj')})
            admpes_jur_tipo_atividade = empresa.get('admpes_jur_tipo_atividade')
        modulo_ecf_instalado = obj_adm_utl.admutl_check_modulo_instalado(cr, uid, 'ecf',
                                                                         context=None)
        natureza_fiscal_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'contabil.natureza.fiscal', context=context)
        contabil_fiscal = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'contabil_fiscal', context=context)
        contabil_conta_dict = {}
        if contabil_fiscal:
            sql = """
                SELECT id
                FROM contabil_conta
            """
            cr.execute(sql)
            contabil_conta_dados = cr.dictfetchall()
            contabil_conta_ids = map(lambda cc: cc.get('id'), contabil_conta_dados)
            contabil_conta_dados = self.pool.get('contabil.conta').read(cr, uid, contabil_conta_ids,
                                                                        ['name', 'conta'], context)
            map(lambda c: contabil_conta_dict.update({c.get('id'): c}), contabil_conta_dados)

        data_inicial_date = parse(data_inicial, yearfirst=True).date()
        data_final_date = parse(data_final, yearfirst=True).date()
        cod_versao = self.sped_contribuicoes_get_versao_layout(data_final_date.year)
        if data_inicial_date.year == 2018 and 6 <= data_inicial_date.month <= 12:
            cod_versao = '004'
        if (data_inicial_date.month < 6 and data_inicial_date.year <= 2018) or \
                (data_final_date.month < 6 and data_final_date.year <= 2018):
            cod_versao = '003'
        if data_inicial_date.year < 2012 or data_final_date.year < 2012:
            cod_versao = '002'
        sped = eval("layout_%s" % cod_versao)  # Leiaute do SPED em funcao de sua versao

        reg_0000 = bloco_arquivo.registro_0000()
        reg_0001 = bloco_0.registro_0001()
        reg_0100 = bloco_0.registro_0100()
        reg_0140 = bloco_0.registro_0140()
        reg_0150 = bloco_0.registro_0150()
        reg_0190 = bloco_0.registro_0190()
        reg_0200 = bloco_0.registro_0200()
        reg_0400 = bloco_0.registro_0400()
        reg_0450 = bloco_0.registro_0450()

        # Instanciando bloco A
        reg_A001 = bloco_a.registro_A001()
        reg_A010 = bloco_a.registro_A010()
        reg_A100 = bloco_a.registro_A100()
        reg_A110 = bloco_a.registro_A110()
        reg_A170 = bloco_a.registro_A170()

        # Instanciando bloco C
        reg_C001 = bloco_c.registro_C001()
        reg_C010 = bloco_c.registro_C010()
        reg_C100 = bloco_c.registro_C100()
        reg_C110 = bloco_c.registro_C110()
        reg_C170 = bloco_c.registro_C170()
        reg_C175 = bloco_c.registro_C175()
        reg_C380 = bloco_c.registro_C380()
        reg_C381 = bloco_c.registro_C381()
        reg_C385 = bloco_c.registro_C385()
        reg_C400 = bloco_c.registro_C400()
        reg_C405 = bloco_c.registro_C405()
        reg_C481 = bloco_c.registro_C481()
        reg_C485 = bloco_c.registro_C485()
        reg_C500 = sped.bloco_c.registro_C500()
        reg_C501 = bloco_c.registro_C501()
        reg_C505 = bloco_c.registro_C505()

        # Instanciando bloco D
        reg_D001 = bloco_d.registro_D001()
        reg_D010 = bloco_d.registro_D010()
        reg_D100 = bloco_d.registro_D100()
        reg_D101 = bloco_d.registro_D101()
        reg_D105 = bloco_d.registro_D105()
        reg_D201 = bloco_d.registro_D201()
        reg_D205 = bloco_d.registro_D205()
        reg_D500 = bloco_d.registro_D500()
        reg_D501 = bloco_d.registro_D501()
        reg_D505 = bloco_d.registro_D505()

        # Instanciando bloco F
        reg_F001 = bloco_f.registro_F001()

        # Instanciando bloco M
        reg_M001 = bloco_m.registro_M001()
        reg_M100 = bloco_m.registro_M100()
        reg_M105 = bloco_m.registro_M105()
        reg_M200 = bloco_m.registro_M200()
        reg_M205 = bloco_m.registro_M205()
        reg_M210 = sped.bloco_m.registro_M210()
        reg_M211 = bloco_m.registro_M211()
        reg_M400 = bloco_m.registro_M400()
        reg_M410 = bloco_m.registro_M410()
        reg_M500 = bloco_m.registro_M500()
        reg_M505 = bloco_m.registro_M505()
        reg_M600 = bloco_m.registro_M600()
        reg_M605 = bloco_m.registro_M605()
        reg_M610 = sped.bloco_m.registro_M610()
        reg_M611 = bloco_m.registro_M611()
        reg_M800 = bloco_m.registro_M800()
        reg_M810 = bloco_m.registro_M810()

        # Instanciando bloco P
        reg_P001 = bloco_p.registro_P001()

        # Instanciando bloco 1
        reg_1001 = bloco_1.registro_1001()

        # Instanciando bloco 9
        reg_9001 = bloco_9.registro_9001()

        tipo_sped = 'C'  # C=Contribuicoes, F=Fiscal
        if data_final_date < data_inicial_date:
            raise osv.except_osv(_('Atencao!'),
                                 _('A data inicial deve ser menor que a data final!'))
        browse_empresa_corrente_id = self.pool.get('res.users').browse(cr, uid, uid,
                                                                       context).company_id
        empresa_corrente_id = browse_empresa_corrente_id.id
        mensagem = ''
        id_pessoa = browse_empresa_corrente_id.rescon_pessoa_id and \
                    browse_empresa_corrente_id.rescon_pessoa_id.id or False
        if data_inicial_date.year < 2011 or data_final_date.year < 2011:
            raise osv.except_osv(_('Atencao!'),
                                 _('Ano informado inválido para leiaute do Ato Cotepe. O ano '
                                   'deve ser a partir de 2011.'))
        browse_pessoa = obj_adm_pessoa.browse(cr, uid, id_pessoa, context)
        if not obj_fiscal_parametro.search(cr, uid, [('company_id', '=', empresa_corrente_id)],
                                           context=context):
            raise osv.except_osv(_('Atencao!'), _('Parametro do Fiscal nao cadastrado!'))
        campos_fiscal_parametro = ['fispar_regime_incidencia_tributaria',
                                   'fispar_indicador_natureza_pessoa_juridica',
                                   'fispar_regime_incidencia_tributaria',
                                   'fispar_metodo_apropriacao_creditos',
                                   'fispar_tipo_contribuicao_apuracao',
                                   'fispar_cod_rec_pis',
                                   'fispar_cod_rec_cofins',
                                   'cupom_nota_natureza_id']
        fiscal_parametro_dados = obj_adm_utl.admutl_get_parametros(
            cr, uid, 'fiscal', campos_fiscal_parametro, context=context)
        regime_incidencia = fiscal_parametro_dados.get('fispar_regime_incidencia_tributaria')
        data_ini = data_inicial
        data_fim = data_final

        def _movimento_notas():
            # Movimentos no mes
            filtro_notas = {
                'sped_data_inicial': data_inicial,
                'sped_data_final': data_final,
                'company_id': company_ids,
                'tipo_sped': tipo_sped
            }
            mov_notas = obj_sql.sql_consulta_movimento_notas(cr, filtro_notas)

            return mov_notas

        def _movimento_cupons():
            filtro_cupons = {
                'sped_data_inicial': data_inicial,
                'sped_data_final': data_final,
                'company_id': company_ids,
                'tipo_sped': tipo_sped
            }
            mov_cupons = obj_sql.sql_consulta_movimento_cupons(cr, filtro_cupons)

            return mov_cupons

        mov_notas = _movimento_notas()
        if MODULO_ECF_INSTALADO:
            mov_cupons = _movimento_cupons()
        else:
            mov_cupons = []

        def _movimento_notas_energia():
            filtro = {
                'data_ini': data_ini,
                'data_fin': data_fim
            }
            movimento_notas_energia = obj_sql.sql_consulta_nf_energia(cr, filtro)

            return movimento_notas_energia

        mov_notas_energia = _movimento_notas_energia()

        def _movimento_notas_2d():
            filtro = {
                'data_ini': data_ini,
                'data_fin': data_fim
            }
            movimento_notas_mod_2d = obj_sql.sql_consulta_mov_modelo_2d(cr, filtro)

            return movimento_notas_mod_2d

        mov_nf_mod_2d = _movimento_notas_2d()

        self.sped_get_conta_contabil_nf_e_movimento(
            cr, uid, mov_notas + mov_notas_energia + mov_nf_mod_2d, context)
        self.sped_get_conta_contabil_natureza(cr, uid, context)
        self.sped_get_conta_contabil_cfop(cr, uid, context)

        # Registro 0000 - Abertura do arquivo
        def _registro0000():
            # print 'REGISTRO 0000'
            global mensagem
            nome_empresa = browse_pessoa.name
            endereco_empresa = obj_adm_pessoa.admpes_endereco_fiscal(
                cr, uid, id_pessoa, context=context)
            if endereco_empresa.get(id_pessoa):
                uf_empresa = endereco_empresa.get(id_pessoa).get('uf_sigla')
                codigo_ibge = endereco_empresa.get(id_pessoa).get('codigo_ibge')
            else:
                uf_empresa = ''
                codigo_ibge = ''
            numero_recibo_anterior = ''
            if browse_sped.tipo_arquivo == '1':
                numero_recibo_anterior = browse_sped.sped_num_recibo_escrituracao_ant or ''
            reg_0000.COD_VER = cod_versao
            reg_0000.TIPO_ESCRIT = browse_sped.tipo_arquivo
            reg_0000.IND_SIT_ESP = ''
            reg_0000.NUM_REC_ANTERIOR = numero_recibo_anterior
            reg_0000.DT_INI = self.formata_data(data_ini)
            reg_0000.DT_FIN = self.formata_data(data_fim)
            reg_0000.NOME = nome_empresa
            reg_0000.CNPJ = browse_pessoa.cpf_cnpj or ''
            reg_0000.UF = uf_empresa or ''
            reg_0000.COD_MUN = codigo_ibge or ''
            reg_0000.SUFRAMA = browse_pessoa.admpes_jur_suframa or ''
            reg_0000.IND_NAT_PJ = fiscal_parametro_dados.get(
                'fispar_indicador_natureza_pessoa_juridica') or ''
            reg_0000.NOME = nome_empresa
            if browse_pessoa.cpf_cnpj:
                reg_0000.CNPJ = self.remove_ponto(browse_pessoa.cpf_cnpj)
            else:
                mensagem += _('* Nao existe informacao de CNPJ cadastrado para a Empresa.\n')
            if not endereco_empresa:
                mensagem += _('* Nao existe Endereco cadastrado para o Empresa.\n')
            if not reg_0000.UF:
                mensagem += _('* Nao existe UF no endereco informado.\n')
            if not codigo_ibge:
                mensagem += _('* Nao existe Codigo do Municipio no endereco informado para a '
                              'empresa.\n')
            if browse_pessoa.admpes_jur_suframa:
                reg_0000.SUFRAMA = browse_pessoa.admpes_jur_suframa
            if browse_pessoa.admpes_jur_tipo_atividade:
                if browse_pessoa.admpes_jur_tipo_atividade == 'I':
                    reg_0000.IND_ATIV = '0'
                elif browse_pessoa.admpes_jur_tipo_atividade == 'S':
                    reg_0000.IND_ATIV = '1'
                elif browse_pessoa.admpes_jur_tipo_atividade in ('C','A'):
                    reg_0000.IND_ATIV = '2'
                elif browse_pessoa.admpes_jur_tipo_atividade == 'F':
                    reg_0000.IND_ATIV = '3'
                elif browse_pessoa.admpes_jur_tipo_atividade == 'M':
                    reg_0000.IND_ATIV = '4'
                else:
                    reg_0000.IND_ATIV = '9'
            else:
                mensagem += _('* Nao existe classificacao tributaria definida para a Empresa.\n')
            reg_0000.write_line()

        # Registro 0001
        def _registro0001():
            # print 'REGISTRO 0001'
            reg_0001.IND_MOV = '0'
            reg_0001.write_line()

        # Registro 0100 - Dados do contabilista
        def _registro0100():
            # print 'REGISTRO 0100'
            global mensagem
            id_par_adm = obj_adm_par.search(
                cr, uid, [('company_id', '=', empresa_corrente_id)], context=context)
            if not id_par_adm:
                mensagem += _('* Nao existe existe parametro cadastrado em Administracao.\n')
            else:
                contador_id = obj_adm_par.read(
                    cr, uid, id_par_adm[0], ['contador_id'], context=context,
                    load='_classic_write').get('contador_id')
                if contador_id:
                    pessoa_contabil = self.pool.get('fiscal.sped').info_pessoa(
                        cr, uid, contador_id, context=context)
                    reg_0100.NOME = pessoa_contabil['nome']  # Nome contador
                    if 'cnpj' in pessoa_contabil:
                        reg_0100.CNPJ = pessoa_contabil['cnpj']
                    if 'cpf' in pessoa_contabil:
                        reg_0100.CPF = pessoa_contabil['cpf']
                    else:
                        mensagem += _('* Nao existe CPF cadastrado para o Contabilista.\n')
                    if not pessoa_contabil['documentos']:
                        mensagem += _('* Nao existe CRC cadastrado para o Contabilista.\n')
                    else:
                        if 'CRC' in pessoa_contabil['documentos']:
                            reg_0100.CRC = pessoa_contabil['documentos']['CRC']
                        else:
                            mensagem += _('* Nao existe CRC cadastrado para o Contabilista.\n')
                    if pessoa_contabil['endereco']:
                        if 'cep' in pessoa_contabil['endereco']:
                            reg_0100.CEP = pessoa_contabil['endereco']['cep']
                        else:
                            mensagem += _('* Nao existe CEP definido no endereco da Empresa.\n')
                        if 'logradouro' in pessoa_contabil['endereco']:
                            reg_0100.END = pessoa_contabil['endereco']['logradouro']
                        if 'numero' in pessoa_contabil['endereco']:
                            reg_0100.NUM = pessoa_contabil['endereco']['numero']
                        if 'complemento' in pessoa_contabil['endereco']:
                            reg_0100.COMPL = pessoa_contabil['endereco']['complemento']
                        if 'bairro' in pessoa_contabil['endereco']:
                            reg_0100.BAIRRO = pessoa_contabil['endereco']['bairro']
                        if 'codigo_ibge' in pessoa_contabil['endereco']:
                            reg_0100.COD_MUN = pessoa_contabil['endereco']['codigo_ibge']
                    telefone_comum = obj_adm_pessoa.read(
                        cr, uid, contador_id, ['telefone'], context=context)['telefone']
                    if telefone_comum:
                        reg_0100.FONE = telefone_comum.replace(
                            '(', '').replace(')', '').replace('-', '')
                    if 'telefones' in pessoa_contabil:
                        for x in pessoa_contabil['telefones']:
                            if x.upper() == 'FAX':
                                reg_0100.FAX = pessoa_contabil['telefones'][x]
                                break
                    email_cont = obj_adm_pessoa.read(
                        cr, uid, contador_id, ['email'], context=context)['email']
                    if email_cont:
                        reg_0100.EMAIL = email_cont
                else:
                    mensagem += _('* Contador nao informado no parametro de Administracao.\n')
                reg_0001.registro_0100.append(reg_0100.write_line())

        # Registro 0110 - Regimes de apuracao da contribuicao social
        def _registro0110():
            # print 'REGISTRO 0110'
            reg_0110 = bloco_0.registro_0110()
            reg_0110.COD_INC_TRIB = fiscal_parametro_dados.get(
                'fispar_regime_incidencia_tributaria') or ''
            reg_0110.IND_APRO_CRED = fiscal_parametro_dados.get(
                'fispar_metodo_apropriacao_creditos') or ''
            reg_0110.COD_TIPO_CONT = fiscal_parametro_dados.get(
                'fispar_tipo_contribuicao_apuracao') or ''
            reg_0110.IND_REG_CUM = ''
            reg_0110.write_line()

        # Registro 0140 - Tabela de cadastro de estabelecimento
        def _registro0140():
            # print 'REGISTRO 0140'
            global mensagem
            dados = obj_sql.sql_0140(cr, company_ids)
            for dado in dados:
                emp_id = dado.get('cod_est')
                reg_0140.REG = '0140'
                reg_0140.COD_EST = str(dado.get('cod_est')).zfill(3)
                reg_0140.NOME = dado.get('nome') or ''
                reg_0140.CNPJ = dado.get('cnpj') or ''
                reg_0140.UF = dado.get('uf') or ''
                reg_0140.IE = dado.get('ie') or ''
                reg_0140.COD_MUN = dado.get('cod_mun') or ''
                reg_0140.IM = dado.get('im') or ''
                reg_0140.SUFRAMA = dado.get('suframa') or ''
                reg_0140.write_line()
                _registro0150(emp_id)
                _registro0190(emp_id)
                _registro0200(emp_id)
                _registro0400(emp_id)
                _registro0450(emp_id)

        def _registro0150(emp_id):
            """
            O campo cod_part é composto pelo cod_part mais o id do logradouro pelo fato de estar
            ocorrendo chave duplicada quando existe movimento para mais de um endereco
            Verificar a possibidilidade de criar metodo
            """
            # print 'REGISTRO 0150'
            movimento_cartao = obj_sql.sql_consulta_mov_cartao(cr, data_ini, data_fim,
                                                               modulo_ecf_instalado, emp_id)

            # Movimento de operadoras de cartao
            pessoa_ids = []
            for dado_oper in movimento_cartao:
                if dado_oper.get('pessoa_id') in pessoa_ids:
                    continue
                pessoa_ids.append(dado_oper.get('pessoa_id'))
                reg_0150.REG = '0150'
                reg_0150.COD_PART = dado_oper.get('pessoa_id') or ''
                reg_0150.NOME = dado_oper.get('operadora')[:99] or ''
                reg_0150.COD_PAIS = dado_oper.get('cod_pais') or '1058'
                if dado_oper.get('pessoa_tipo') == 'F':
                    reg_0150.CNPJ = ''
                    reg_0150.CPF = dado_oper.get('cpf_cnpj') or ''
                elif dado_oper.get('pessoa_tipo') == 'J':
                    reg_0150.CNPJ = dado_oper.get('cpf_cnpj') or ''
                    reg_0150.CPF = ''
                else:
                    reg_0150.CNPJ = ''
                    reg_0150.CPF = ''
                _ie = dado_oper.get('ie') or ''
                if 'ISENT' in str(_ie).strip().upper():
                    _ie = ''
                reg_0150.IE = self.formata_para_digitos(_ie) or ''
                reg_0150.COD_MUN = dado_oper.get('cod_mun') or ''
                reg_0150.SUFRAMA = dado_oper.get('suframa') or ''
                reg_0150.END = dado_oper.get('endr') or ''
                reg_0150.NUM = dado_oper.get('num') or ''
                reg_0150.COMPL = dado_oper.get('compl') or ''
                reg_0150.BAIRRO = dado_oper.get('bairro') or ''
                reg_0150.write_line()

            # Movimento de clientes
            if mov_notas:
                filtro_0150 = {
                    'movimento_notas': mov_notas,
                    'company_id': emp_id
                }
            else:
                filtro_0150 = {
                    'movimento_notas': [(-1)],
                    'company_id': False
                }
            dados = obj_sql.sql_reg0150(cr, filtro_0150, nf_nao_fiscais)
            for dado in dados:
                if dado.get('nome'):
                    ie = dado.get('ie') or ''
                    if 'ISENT' in str(ie).strip().upper():
                        ie = ''
                    reg_0150.REG = '0150'
                    reg_0150.COD_PART = dado.get('cod_part') or ''
                    reg_0150.NOME = dado.get('nome')[:99] or ''
                    reg_0150.COD_PAIS = dado.get('cod_pais') or '1058'
                    reg_0150.CNPJ = dado.get('cnpj') or ''
                    reg_0150.CPF = dado.get('cpf') or ''
                    reg_0150.IE = self.formata_para_digitos(ie) or ''
                    reg_0150.COD_MUN = dado.get('cod_mun') or ''
                    reg_0150.SUFRAMA = dado.get('suframa') or ''
                    reg_0150.END = dado.get('logradouro') or ''
                    reg_0150.NUM = dado.get('num') or ''
                    reg_0150.COMPL = dado.get('compl') or ''
                    reg_0150.BAIRRO = dado.get('bairro') or ''
                    reg_0150.write_line()

        # Registro 0190 - Identificacao das unidades de medida
        def _registro0190(company_id):
            # print 'REGISTRO 0190'
            dados = obj_sql.sql_reg0190(cr, company_id, mov_notas, mov_cupons, nf_nao_fiscais,
                                        modulo_ecf_instalado)
            for dado in dados:
                if dado.get('unid'):
                    reg_0190.REG = '0190'
                    reg_0190.UNID = dado.get('unid')
                    reg_0190.DESCR = dado.get('descr')
                    reg_0190.write_line()

        # Registro 0200
        def _registro0200(company_id):
            # print 'REGISTRO 0200'
            dados = obj_sql.sql_reg0200(cr, company_id, mov_notas, mov_cupons, nf_nao_fiscais,
                                        modulo_ecf_instalado)
            codigo_barra_ids = list(set(map(lambda cb: cb.get('cod_barra_id'),
                                            filter(lambda r: r.get('cod_barra_id'), dados))))
            codigo_barra_dados = self.pool.get(
                'automacao.codigo.barra').autcba_get_codigo_barra_formatado_gtin(
                cr, uid, codigo_barra_ids, context)
            for dado in dados:
                codigo_barra = codigo_barra_dados.get(dado.get('cod_barra_id')) or ''
                reg_0200.REG = '0200'
                reg_0200.COD_ITEM = dado.get('cod_item') or ''
                descr_item = ''
                if dado.get('descr_item'):
                    descr_item = obj_adm_utl.admutl_remove_caracter(dado.get('descr_item'),
                                                                    letras=True, numeros=True,
                                                                    espaco=True)
                reg_0200.DESCR_ITEM = descr_item or ''
                reg_0200.COD_BARRA = '' if codigo_barra == 'SEM GTIN' else codigo_barra
                reg_0200.COD_ANT_ITEM = dado.get('cod_ant_item') or ''
                reg_0200.UNID_INV = dado.get('unid_inv') or ''
                reg_0200.TIPO_ITEM = dado.get('tipo_item') or ''
                reg_0200.COD_NCM = (dado.get('cod_ncm') or '').zfill(8)
                reg_0200.EX_IPI = dado.get('ex_ipi') or ''
                reg_0200.COD_GEN = dado.get('cod_gen') or ''
                reg_0200.COD_LST = dado.get('cod_lst') or ''
                reg_0200.ALIQ_ICMS = dado.get('aliq_icms') or ''
                reg_0200.write_line()

        # Registro 0400 - Tabela de Natureza da Operacao/Prestacao
        def _registro0400(company_id):
            # print 'REGISTRO 0400'
            global mensagem
            if mov_notas:
                filtros_0400 = {
                        'movimento_notas': mov_notas,
                        'tipo_sped': 'P',
                        'company_id': company_id
                    }
                dados = obj_sql.sql_reg0400(cr, filtros_0400, nf_nao_fiscais)
                for dado in dados:
                    reg_0400.REG = '0400'
                    reg_0400.COD_NAT = dado.get('cod_nat') or ''
                    reg_0400.DESCR_NAT = dado.get('descr_nat') or ''
                    reg_0400.write_line()

        # Registro 0450 - Tabela de informacao complementar do documento fiscal
        def _registro0450(emp_id):
            # print 'REGISTRO 0450'
            if mov_notas:
                filtros_0450 = {
                    'movimento_notas': mov_notas,
                    'company_id': emp_id
                }
                dados = obj_sql.sql_reg0450(cr, filtros_0450, nf_nao_fiscais)
                for dado in dados:
                    txt = obj_adm_utl.admutl_remove_caracter(dado.get('txt'), letras=True,
                                                             numeros=True, espaco=True)
                    if txt and not txt.isspace():
                        txt = txt[0:255]
                        reg_0450.REG = '0450'
                        reg_0450.COD_INF = dado.get('cod_inf') or ''
                        reg_0450.TXT = txt
                        reg_0450.write_line()

        # Bloco A
        # Registro A001
        def _registroA001():
            # print 'REGISTRO A001'
            reg_A001.REG = 'A001'
            reg_A001.IND_MOV = '1'
            if mov_notas:
                filtro = {
                    'movimento_notas': mov_notas,
                }
                dados = obj_sql.sql_consulta_nf_servico(cr, filtro)
                if dados:
                    reg_A001.IND_MOV = '0'
                    reg_A001.write_line()
                    _registroA010()
                else:
                    reg_A001.write_line()
            else:
                reg_A001.write_line()

        # Registro A010 - Identificacao do estabelecimento
        def _registroA010():
            # print 'REGISTRO A010'
            for dado in empresas_cnpj:
                reg_A010.CNPJ = self.remove_ponto(empresas_cnpj.get(dado) or '')
                reg_A010.write_line()
                _registroA100(dado)

        # Registro A100 - Documento - nota fiscal de servico
        def _registroA100(company_id):
            # print 'REGISTRO A100'
            if mov_notas:
                filtro = {
                    'movimento_notas': mov_notas,
                    'company_id': company_id
                }
                dados = obj_sql.sql_A100(cr, filtro)
                for dado in dados:
                    reg_A100 = bloco_a.registro_A100()
                    reg_A100.IND_OPER = dado.get('ind_oper') or ''
                    reg_A100.IND_EMIT = dado.get('ind_emit') or ''
                    reg_A100.SER = dado.get('ser') or ''
                    reg_A100.SUB = dado.get('sub') or ''
                    reg_A100.NUM_DOC = dado.get('num_doc') or ''
                    reg_A100.COD_SIT = dado.get('cod_sit') or ''
                    reg_A100.COD_PART = dado.get('cod_part') or ''

                    # Somente serao preenchidos se COD_SIT for diferente de 02 (documento cancelado)
                    if reg_A100.COD_SIT != '02':
                        reg_A100.CHV_NFSE = ''
                        reg_A100.DT_DOC = self.formata_data(dado.get('dt_doc')) or ''
                        reg_A100.DT_EXE_SERV = ''
                        reg_A100.VL_DOC = self.formata_numero(
                            dado.get('vl_doc'), size=1, separador=',') or '0,00'
                        reg_A100.IND_PGTO = dado.get('ind_pgto') or ''
                        reg_A100.VL_DESC = self.formata_numero(
                            dado.get('vl_desc'), size=1, separador=',') or '0,00'
                        reg_A100.VL_BC_PIS = self.formata_numero(
                            dado.get('vl_bc_pis'), size=1, separador=',') or '0,00'
                        reg_A100.VL_PIS = self.formata_numero(
                            dado.get('vl_pis'), size=1, separador=',') or '0,00'
                        reg_A100.VL_BC_COFINS = self.formata_numero(
                            dado.get('vl_bc_cofins'), size=1, separador=',') or '0,00'
                        reg_A100.VL_COFINS = self.formata_numero(
                            dado.get('vl_cofins'), size=1, separador=',') or '0,00'
                        reg_A100.VL_PIS_RET = '0,00'
                        reg_A100.VL_COFINS_RET = '0,00'
                        reg_A100.VL_ISS = self.formata_numero(
                            dado.get('vl_iss'), size=1, separador=',') or '0,00'
                    reg_A100.write_line()
                    if dado.get('txt'):
                        _registroA110(dado.get('nf_id'), dado.get('txt'))
                    _registroA170(dado.get('nf_id'))

        # Registro A110 -  Complemento do documento - Informacao complementar da NF
        def _registroA110(nf_id, txt):
            # print 'REGISTRO A110'
            reg_A110.REG = 'A110'
            reg_A110.COD_INF = nf_id
            reg_A110.TXT_COMPL = txt[0:254]
            reg_A110.write_line()

        # Registro A170: Complemento do documento - Itens do documento
        def _registroA170(nf_id):
            # print 'REGISTRO A170'
            global CREDITO_PIS
            global CREDITO_COFINS
            global DEBITO_PIS
            global DEBITO_COFINS
            cst_saida = ['01', '20', '03', '04', '05', '06', '07', '08', '09', '49']
            codigos = ['50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63', '64', '65',
                       '66']
            dados = obj_sql.sql_A170(cr, nf_id)
            cod_cta_nf = self.sped_get_conta_contabil(cr, uid, nf_id=nf_id, context=context)
            for dado in dados:
                reg_A170.NUM_ITEM = dado.get('num_item')
                reg_A170.COD_ITEM = dado.get('cod_item')
                reg_A170.DESCR_COMPL = dado.get('descr_compl')
                reg_A170.VL_ITEM = self.formata_numero(
                    dado.get('vl_item'), size=1, separador=',') or '0,00'
                reg_A170.VL_DESC = self.formata_numero(
                    dado.get('vl_desc'), size=1, separador=',') or '0,00'
                reg_A170.NAT_BC_CRED = ''
                cst = dado.get('cst_pis') or dado.get('cst_cofins')
                if cst and cst in codigos and dado.get('nat_bc_cred'):
                    reg_A170.NAT_BC_CRED = dado.get('nat_bc_cred')
                reg_A170.IND_ORIG_CRED = dado.get('ind_orig_cred')
                reg_A170.CST_PIS = dado.get('cst_pis') or ''
                reg_A170.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=1, separador=',') or '0,00'
                reg_A170.ALIQ_PIS = self.formata_numero(
                    dado.get('aliq_pis'), size=1, separador=',') or '0,00'
                reg_A170.VL_PIS = self.formata_numero(
                    dado.get('vl_pis'), size=1, separador=',') or '0,00'
                reg_A170.CST_COFINS = dado.get('cst_cofins') or ''
                reg_A170.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=1, separador=',') or '0,00'
                reg_A170.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',') or '0,00'
                reg_A170.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=1, separador=',') or '0,00'
                reg_A170.COD_CTA = self.sped_get_conta_contabil(
                    cr, uid, nf_id=nf_id, movimento_item_id=dado.get('movimento_item_id'),
                    context=context) or cod_cta_nf
                reg_A170.COD_CCUS = ''
                if dado.get('cst_pis') in cst_saida:
                    if dado.get('vl_pis'):
                        DEBITO_PIS += dado.get('vl_pis') or 0.00
                else:
                    if dado.get('vl_pis'):
                        CREDITO_PIS += dado.get('vl_pis') or 0.00
                if dado.get('cst_cofins') in cst_saida:
                    DEBITO_COFINS += dado.get('vl_cofins') or 0.00
                else:
                    CREDITO_COFINS += dado.get('vl_cofins') or 0.00
                reg_A170.write_line()

        # Bloco C
        # Registro C001 - Abertura do Bloco C
        def _registroC001():
            # print 'REGISTRO C001'
            if mov_notas or mov_cupons:
                reg_C001.IND_MOV = '0'
                reg_C001.write_line()
                _registroC010()
            else:
                reg_C001.IND_MOV = '1'
                reg_C001.write_line()

        # Registro C010
        def _registroC010():
            # print 'REGISTRO C010'
            for dado in empresas_cnpj:
                reg_C010.CNPJ = self.remove_ponto(empresas_cnpj.get(dado) or '')
                reg_C010.IND_ESCRI = '2'
                reg_C010.write_line()
                _registroC100(dado)
                _registroC380(dado)
                _registroC400(dado)
                _registroC500(dado)

        def _get_vl_nf_cupom(cfop=False, modelo=False, valor=0.00):
            if cfop == '5929' and modelo == '55':
                valor = '0,00'
            else:
                valor = self.formata_numero(valor, size=1, separador=',')
            return valor

        # Registro C100
        def _registroC100(company_id):
            """
            Registro C100 você escritura o valor total do documento e valor total da mercadoria como
            está na nota. No Registro C170 é a quantidade x valor do ítem, ou seja não soma o icms
            st. Esse só vai compor o valor total da operação no Registro C190,
            """
            # print 'REGISTRO C100'
            if mov_notas:
                def _limpaC100():
                    reg_C100.REG = 'C100'
                    reg_C100.IND_OPER = ''
                    reg_C100.IND_EMIT = ''
                    reg_C100.COD_PART = ''
                    reg_C100.COD_MOD = ''
                    reg_C100.COD_SIT = ''
                    reg_C100.SER = ''
                    reg_C100.NUM_DOC = ''
                    reg_C100.CHV_NFE = ''
                    reg_C100.DT_DOC = ''
                    reg_C100.DT_E_S = ''
                    reg_C100.VL_DOC = ''
                    reg_C100.IND_PGTO = ''
                    reg_C100.VL_DESC = ''
                    reg_C100.VL_ABAT_NT = ''
                    reg_C100.VL_MERC = ''
                    reg_C100.IND_FRT = ''
                    reg_C100.VL_FRT = ''
                    reg_C100.VL_SEG = ''
                    reg_C100.VL_OUT_DA = ''
                    reg_C100.VL_BC_ICMS = ''
                    reg_C100.VL_ICMS = ''
                    reg_C100.VL_BC_ICMS_ST = ''
                    reg_C100.VL_ICMS_ST = ''
                    reg_C100.VL_IPI = ''
                    reg_C100.VL_PIS = ''
                    reg_C100.VL_COFINS = ''
                    reg_C100.VL_PIS_ST = ''
                    reg_C100.VL_COFINS_ST = ''
                filtros_C100 = {
                    'movimento_notas': mov_notas,
                    'company_id': company_id
                }
                dados = obj_sql.sql_regC100(cr, filtros_C100)
                dados_c170 = obj_sql.sql_regC170(cr, mov_notas)
                modelo_65_nf_ids = map(lambda m: m.get('id'),
                                       filter(lambda c: c.get('cod_mod') == '65', dados))
                filtro_c175 = {'contabil_fiscal': contabil_fiscal, 'nf_ids': modelo_65_nf_ids}
                if filtro_c175.get('contabil_fiscal'):
                    filtro_c175.update({'natureza_fiscal_company_id': natureza_fiscal_company_id})
                dados_c175 = obj_sql.sql_regC175(cr, filtro_c175) if modelo_65_nf_ids else {}

                for dado in dados:
                    des_itens = obj_sql.sql_reg_C100_descontos(cr, dado.get('id'))
                    vl_desc = des_itens[0].get('desc_item') or 0.00
                    vl_bc_icms_st = dado.get('vl_bc_icms_st')
                    vl_icms_st = dado.get('vl_icms_st')
                    _limpaC100()
                    reg_C100.REG = 'C100'
                    cfop = dado.get('cfop')
                    modelo = dado.get('cod_mod')
                    if dado.get('cod_sit') in ('02', '04', '05'):
                        reg_C100.IND_OPER = dado.get('ind_oper') or ''
                        reg_C100.IND_EMIT = dado.get('ind_emit') or ''
                        reg_C100.COD_MOD = dado.get('cod_mod') or ''
                        reg_C100.COD_SIT = dado.get('cod_sit') or ''
                        reg_C100.SER = dado.get('ser') or ''
                        reg_C100.NUM_DOC = dado.get('num_doc') or ''
                        if dado.get('cod_sit') == '05':
                            reg_C100.CHV_NFE = ''
                        else:
                            reg_C100.CHV_NFE = dado.get('chv_nfe') or ''
                        reg_C100.write_line()
                    else:
                        reg_C100.IND_OPER = dado.get('ind_oper') or ''
                        reg_C100.IND_EMIT = dado.get('ind_emit') or ''
                        reg_C100.COD_PART = dado.get('cod_part') or ''
                        reg_C100.COD_MOD = dado.get('cod_mod') or ''
                        reg_C100.COD_SIT = dado.get('cod_sit') or ''
                        reg_C100.SER = dado.get('ser') or ''
                        reg_C100.NUM_DOC = dado.get('num_doc') or ''
                        reg_C100.CHV_NFE = dado.get('chv_nfe') or ''
                        reg_C100.DT_DOC = self.formata_data(dado.get('dt_doc')) \
                            if dado.get('dt_doc') else ''
                        reg_C100.DT_E_S = self.formata_data(dado.get('dt_e_s')) \
                            if dado.get('dt_e_s') else ''
                        reg_C100.VL_DOC = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_doc'))
                        reg_C100.IND_PGTO = dado.get('ind_pgto') or ''
                        reg_C100.VL_DESC = _get_vl_nf_cupom(cfop, modelo, vl_desc)
                        reg_C100.VL_ABAT_NT = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_abat_nt'))
                        reg_C100.VL_MERC = _get_vl_nf_cupom(cfop, modelo,
                                                            dado.get('vl_merc'))
                        reg_C100.IND_FRT = dado.get('ind_frt') or ''
                        reg_C100.VL_FRT = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_frt'))
                        reg_C100.VL_SEG = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_seg'))
                        reg_C100.VL_OUT_DA = _get_vl_nf_cupom(cfop, modelo,
                                                              dado.get('vl_out_da'))
                        reg_C100.VL_BC_ICMS = _get_vl_nf_cupom(cfop, modelo,
                                                               dado.get('vl_bc_icms'))
                        reg_C100.VL_ICMS = _get_vl_nf_cupom(cfop, modelo,
                                                            dado.get('vl_icms'))
                        reg_C100.VL_BC_ICMS_ST = _get_vl_nf_cupom(cfop, modelo,
                                                                  vl_bc_icms_st)
                        reg_C100.VL_ICMS_ST = _get_vl_nf_cupom(cfop, modelo,
                                                               vl_icms_st)
                        reg_C100.VL_IPI = _get_vl_nf_cupom(cfop, modelo,
                                                           dado.get('vl_ipi'))
                        reg_C100.VL_PIS = _get_vl_nf_cupom(cfop, modelo, dado.get('vl_pis'))
                        reg_C100.VL_COFINS = _get_vl_nf_cupom(cfop, modelo,
                                                              dado.get('vl_cofins'))
                        reg_C100.VL_PIS_ST = _get_vl_nf_cupom(cfop, modelo,
                                                              dado.get('vl_pis_st'))
                        reg_C100.VL_COFINS_ST = _get_vl_nf_cupom(cfop, modelo,
                                                                 dado.get('vl_cofins_st'))
                        reg_C100.write_line()
                        if dado.get('txt') and modelo != '65':
                            dados_C110 = {
                                'REG': 'C110',
                                'COD_INF': str(dado.get('id'))[-6:],
                                'TXT_COMPL': dado.get('txt')
                            }
                            _registroC110(dados_C110)
                        if modelo != '65':
                            _registroC170(dados_c170.get(dado.get('id')))
                        if modelo == '65':
                            _registroC175(dados_c175.get(dado.get('id')) or [])

        # Registro C110
        def _registroC110(dados):
            # print 'REGISTRO C110'
            txt = obj_adm_utl.admutl_remove_caracter(dados.get('TXT_COMPL'), letras=True,
                                                     numeros=True, espaco=True)
            if txt and not txt.isspace():
                txt = txt[0:255]
                reg_C110.COD_INF = dados.get('COD_INF') or ''
                reg_C110.TXT_COMPL = txt
                reg_C110.write_line()

        # Registro C170
        def _registroC170(dados_c170):
            # print 'REGISTRO C170'
            global CREDITO_PIS
            global CREDITO_COFINS
            global DEBITO_PIS
            global DEBITO_COFINS
            fisnit = self.pool.get('fiscal.nf.item.tributacao')
            cst_saida = ['01', '20', '03', '04', '05', '06', '07', '08', '09', '49']
            desc_itens = 0.00
            cod_cta_nf = self.sped_get_conta_contabil(
                cr, uid, nf_id=dados_c170[0].get('nf_id') if dados_c170 else False, context=context)
            for dado in dados_c170:
                cfop = dado.get('cfop')
                vl_bc_icms = dado.get('vl_bc_icms')
                vl_icms = dado.get('vl_icms')
                if dado.get('vl_icms_desonerado'):
                    vl_bc_icms = 0.00
                    vl_icms = 0.00
                cst = dado.get('cst_icms')
                if cst in ('10', '30', '40', '51', '60', '70', '90'):
                    if not admpes_jur_tipo_atividade == 'I':
                        vl_bc_icms = 0.00
                        vl_icms = 0.00
                vl_bc_icms_st = dado.get('vl_bc_icms_st')
                vl_icms_st = dado.get('vl_icms_st')
                reg_C170.REG = 'C170'
                reg_C170.NUM_ITEM = dado.get('sequencia') or ''
                reg_C170.COD_ITEM = dado.get('cod_item') or ''
                descr_compl = ''
                if dado.get('descr_compl'):
                    descr_compl = obj_adm_utl.admutl_remove_caracter(
                        dado.get('descr_compl'), letras=True, numeros=True, espaco=True)
                reg_C170.DESCR_COMPL = descr_compl or ''
                reg_C170.QTD = self.formata_numero(
                    dado.get('qtd'), size=1, separador=',', dec=3) if dado.get('qtd') else ''
                reg_C170.UNID = dado.get('unid') or ''
                reg_C170.VL_ITEM = self.formata_numero(
                    (dado.get('vl_item') or 0) - (dado.get('vl_icms_desonerado') or 0),
                    size=1, separador=',')  if cfop != '5929' else '0,00'
                reg_C170.VL_DESC = self.formata_numero(
                    dado.get('vl_desc'), size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.IND_MOV = '0'
                reg_C170.CST_ICMS = '0' + dado.get('cst_icms') if dado.get('cst_icms') else ''
                reg_C170.CFOP = dado.get('cfop') or ''
                reg_C170.COD_NAT = dado.get('cod_nat') or ''
                reg_C170.VL_BC_ICMS = self.formata_numero(
                    vl_bc_icms, size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.ALIQ_ICMS = self.formata_numero(
                    dado.get('aliq_icms'), size=1, separador=',')
                reg_C170.VL_ICMS = self.formata_numero(
                    vl_icms, size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.VL_BC_ICMS_ST = self.formata_numero(
                    vl_bc_icms_st, size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.ALIQ_ST = self.formata_numero(
                    dado.get('aliq_st'), size=1, separador=',')
                reg_C170.VL_ICMS_ST = self.formata_numero(
                    vl_icms_st, size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.IND_APUR = '0'
                reg_C170.CST_IPI = dado.get('cst_ipi') or ''
                reg_C170.COD_ENQ = ''
                reg_C170.VL_BC_IPI = self.formata_numero(
                    dado.get('vl_bc_ipi'), size=1,separador=',') if cfop != '5929' else '0,00'
                reg_C170.ALIQ_IPI = self.formata_numero(
                    dado.get('aliq_ipi'), size=1, separador=',')
                reg_C170.VL_IPI = self.formata_numero(
                    dado.get('vl_ipi'), size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.CST_PIS = dado.get('cst_pis') or ''

                # Tratamento para cst 05 conforme Pag.73 Topico 4 do Guia pratico EFD
                # contribuicoes versao 1.21
                # http://www1.receita.fazenda.gov.br/sistemas/efd-contribuicoes/download/download.htm
                if dado.get('cst_pis') == '05':
                    reg_C170.VL_BC_PIS = '0,00'
                    reg_C170.VL_BC_COFINS = '0,00'
                    vl_pis = 0
                    vl_cofins = 0
                else:
                    reg_C170.VL_BC_PIS = self.formata_numero(
                        dado.get('vl_bc_pis'), size=1, separador=',') if cfop != '5929' else '0,00'
                    reg_C170.VL_BC_COFINS = self.formata_numero(
                        dado.get('vl_bc_cofins'), size=1, separador=',') if cfop != '5929' \
                        else '0,00'
                    vl_pis = float(fisnit._float_persist('vr_encargo', dado.get('vl_pis')))
                    vl_cofins = float(fisnit._float_persist('vr_encargo', dado.get('vl_cofins')))
                reg_C170.ALIQ_PIS = self.formata_numero(
                    dado.get('aliq_pis'), size=1, separador=',')
                reg_C170.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',')
                reg_C170.QUANT_BC_PIS = ''
                reg_C170.VL_PIS = self.formata_numero(
                    vl_pis, size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.CST_COFINS = dado.get('cst_cofins') or ''
                reg_C170.VL_COFINS = self.formata_numero(
                    vl_cofins, size=1, separador=',') if cfop != '5929' else '0,00'
                reg_C170.COD_CTA = self.sped_get_conta_contabil(
                    cr, uid, movimento_item_id=dado.get('movimento_item_id'),
                    context=context) or cod_cta_nf
                if not isinstance(dado.get('vl_desc'), float) or cfop == '5929':
                    vl_desc = 0.00
                else:
                    vl_desc = dado.get('vl_desc') or 0.00
                desc_itens += vl_desc
                if dado.get('cst_pis') in cst_saida:
                    if dado.get('vl_pis') and cfop != '5929':
                        DEBITO_PIS += dado.get('vl_pis') or 0.00
                else:
                    if dado.get('vl_pis') and cfop != '5929':
                        CREDITO_PIS += dado.get('vl_pis') or 0.00
                if dado.get('cst_cofins') in cst_saida and cfop != '5929':
                    DEBITO_COFINS += dado.get('vl_cofins') or 0.00
                elif cfop != '5929':
                    CREDITO_COFINS += dado.get('vl_cofins') or 0.00
                reg_C170.write_line()

        # Registro C175
        def _registroC175(dados_c175):
            # print 'REGISTRO C175'
            for dado in dados_c175:
                cst = dado.get('cst') or ''
                vl_bc_pis = dado.get('vl_bc') or 0
                vl_bc_cofins = dado.get('vl_bc') or 0
                vl_pis = dado.get('vl_pis') or 0
                vl_cofins = dado.get('vl_pis') and (dado.get('vl_cofins') or 0) or 0
                if cst == '05':
                    vl_bc_cofins = 0
                    vl_bc_pis = 0
                    vl_pis = 0
                    vl_cofins = 0

                reg_C175.REG = 'C175'
                reg_C175.CFOP = dado.get('cfop')
                reg_C175.VL_OPR = self.formata_numero(dado.get('vl_opr') or 0, size=1,
                                                      separador=',')
                reg_C175.VL_DESC = self.formata_numero(dado.get('vl_desc') or 0, size=1,
                                                       separador=',')
                reg_C175.CST_PIS = cst
                reg_C175.VL_BC_PIS = self.formata_numero(vl_bc_pis, size=1, separador=',')
                reg_C175.ALIQ_PIS = self.formata_numero(dado.get('aliq_pis') or 0, size=1,
                                                        separador=',', dec=4)
                reg_C175.QUANT_BC_PIS = ''
                reg_C175.ALIQ_PIS_QUANT = ''
                reg_C175.VL_PIS = self.formata_numero(vl_pis, size=1, separador=',')
                reg_C175.CST_COFINS = cst
                reg_C175.VL_BC_COFINS = self.formata_numero(vl_bc_cofins, size=1, separador=',')
                reg_C175.ALIQ_COFINS = self.formata_numero(dado.get('aliq_cofins') or 0, size=1,
                                                      separador=',', dec=4)
                reg_C175.QUANT_BC_COFINS = ''
                reg_C175.ALIQ_COFINS_QUANT = ''
                reg_C175.VL_COFINS = self.formata_numero(vl_cofins, size=1, separador=',')
                cod_cta = dado.get('cod_cta') or False
                reg_C175.COD_CTA = contabil_conta_dict.get(cod_cta).get('conta') if cod_cta else ''
                reg_C175.INFO_COMPL = ''
                reg_C175.write_line()

        # Registro C380
        def _registroC380(company_id):
            # print 'REGISTRO C380'
            if mov_nf_mod_2d:
                filtro = {'movimento_notas': mov_nf_mod_2d,
                          'company_id': company_id}
                cod_ctas = list()
                map(lambda n: cod_ctas.append(
                    self.sped_get_conta_contabil(cr, uid, nf_id=n, context=context)),
                    mov_nf_mod_2d)
                cod_ctas = list(set(cod_ctas))
                dados = obj_sql.sql_C380(cr, filtro)
                for dado in dados:
                    if dado.get('dt_doc_ini'):
                        reg_C380.COD_MOD = '02'
                        reg_C380.DT_DOC_INI = self.formata_data(dado.get('dt_doc_ini'))
                        reg_C380.DT_DOC_FIN = self.formata_data(dado.get('dt_doc_fin'))
                        reg_C380.NUM_DOC_INI = dado.get('num_doc_ini')
                        reg_C380.NUM_DOC_FIN = dado.get('num_doc_fin')
                        reg_C380.VL_DOC = self.formata_numero(
                            dado.get('vl_doc'), size=1, separador=',')
                        reg_C380.VL_DOC_CANC = self.formata_numero(
                            dado.get('vl_doc_canc'), size=1, separador=',')
                        reg_C380.write_line()
                        _registroC381(mov_nf_mod_2d, cod_ctas)
                        _registroC385(mov_nf_mod_2d, cod_ctas)

        def _registroC381(filtro, cod_ctas):
            # print 'REGISTRO C381'
            global DEBITO_PIS
            dados = obj_sql.sql_C381(cr, filtro)
            cont = 0
            for dado in dados:
                reg_C381.CST_PIS = dado.get('cst_pis')
                reg_C381.COD_ITEM = dado.get('cod_item')
                reg_C381.VL_ITEM = self.formata_numero(
                    dado.get('vl_item'), size=1, separador=',')
                reg_C381.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=1, separador=',')
                reg_C381.ALIQ_PIS = self.formata_numero(
                    dado.get('aliq_pis'), size=1, separador=',')
                reg_C381.QUANT_BC_PIS = ''
                reg_C381.ALIQ_PIS_QUANT = ''
                reg_C381.VL_PIS = self.formata_numero(dado.get('vl_pis'), size=1, separador=',')
                try:
                    cod_cta = cod_ctas[cont]
                    cont += 1
                except:
                    cont = 0
                    cod_cta = cod_ctas[cont]
                    cont += 1
                reg_C381.COD_CTA = cod_cta
                if dado.get('vl_pis'):
                    DEBITO_PIS += dado.get('vl_pis') or 0.00
                reg_C381.write_line()

        # Registro C385
        def _registroC385(filtro, cod_ctas):
            # print 'REGISTRO C385'
            global DEBITO_COFINS
            dados = obj_sql.sql_C385(cr, filtro)
            cont = 0
            for dado in dados:
                reg_C385.CST_COFINS = dado.get('cst_cofins')
                reg_C385.COD_ITEM = dado.get('cod_item')
                reg_C385.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=1, separador=',')
                reg_C385.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=1, separador=',')
                reg_C385.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',')
                reg_C385.QUANT_BC_COFINS = ''
                reg_C385.ALIQ_COFINS_QUANT = ''
                reg_C385.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=1, separador=',')
                try:
                    cod_cta = cod_ctas[cont]
                    cont += 1
                except:
                    cont = 0
                    cod_cta = cod_ctas[cont]
                    cont += 1
                reg_C385.COD_CTA = cod_cta
                if dado.get('vl_cofins'):
                    DEBITO_COFINS += dado.get('vl_cofins') or 0.00
                reg_C385.write_line()

        # Registro C400
        def _registroC400(company_id):
            # print 'REGISTRO C400'
            if mov_cupons:
                filtro = {
                    'dt_ini': data_ini,
                    'dt_fim': data_fim,
                    'company_id': company_id
                }
                dados = obj_sql.sql_regC400(cr, filtro)
                for dado in dados:
                    reg_C400.REG = dado.get('reg')
                    reg_C400.COD_MOD = dado.get('cod_mod')
                    reg_C400.ECF_MOD = dado.get('ecf_mod')
                    reg_C400.ECF_FAB = dado.get('ecf_fab')
                    reg_C400.ECF_CX = dado.get('ecf_cx')
                    reg_C400.write_line()
                    _registroC405(dado.get('id'))

        # Registro C405
        def _registroC405(ecf_id):
            # print 'REGISTRO C405'
            filtros = {
                'dt_ini': data_ini,
                'dt_fin': data_fim,
                'ecf_id': ecf_id
            }
            mensagem = ''
            dados = obj_sql.sql_regC405(cr, filtros)
            for dado in dados:
                reg_C405.REG = dado.get('reg')
                reg_C405.DT_DOC = self.formata_data(dado.get('dt_doc'))
                if dado.get('cro'):
                    if len(str(dado.get('cro'))) <= 3:
                        reg_C405.CRO = str(dado.get('cro')).zfill(3)
                    else:
                        reg_C405.CRO = str(dado.get('cro', ''))
                else:
                    reg_C405.CRO = ''
                reg_C405.CRZ = dado.get('crz')
                if len(str(dado.get('num_coo_fin'))) <= 6:
                    reg_C405.NUM_COO_FIN = str(dado.get('num_coo_fin')).zfill(3)
                else:
                    mensagem += _('*Tamanho do COO invalido') + str(dado.get('num_coo_fin')) + '.\n'
                reg_C405.GT_FIN = self.formata_numero(
                    dado.get('gt_fin'), size=1, separador=',')
                reg_C405.VL_BRT = self.formata_numero(
                    dado.get('vl_brt'), size=1, separador=',')
                reg_C405.write_line()
                _registroC481(ecf_id, dado.get('dt_doc'))
                _registroC485(ecf_id, dado.get('dt_doc'))

        # Registro C481
        def _registroC481(ecf_id, dt_doc):
            # print 'REGISTRO C481'
            fisnit = self.pool.get('fiscal.nf.item.tributacao')
            while obj_sql.C481.get('executando'):
                pass
            if obj_sql.C481.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C481.get('erro')))
            global DEBITO_PIS
            chave = str(ecf_id) + str(dt_doc)
            dados = filter(lambda i: str(i.get('ecf_id')) + str(i.get('data')) == chave,
                           obj_sql.C481.get('dados'))
            for dado in dados:
                if dado.get('vl_item'):
                    reg_C481.CST_PIS = dado.get('cst_pis_cofins') or ''
                    reg_C481.VL_ITEM = self.formata_numero(
                        dado.get('vl_item'), size=1, separador=',')
                    # Tratamento para cst 05 conforme Pag.73 Topico 4 do Guia pratico EFD
                    # contribuicoes versao 1.21
                    # http://www1.receita.fazenda.gov.br/sistemas/efd-contribuicoes/download/download.htm
                    if dado.get('cst_pis_cofins') == '05':
                        reg_C481.VL_BC_PIS = '0,00'
                    else:
                        reg_C481.VL_BC_PIS = self.formata_numero(
                            dado.get('vl_bc_pis_cofins'), size=1, separador=',')
                    reg_C481.ALIQ_PIS = self.formata_numero(
                        dado.get('aliq_pis_cofins'), size=1, separador=',')
                    reg_C481.QUANT_BC_PIS = ''
                    reg_C481.ALIQ_PIS_QUANT = ''
                    vl_pis = float(fisnit._float_persist('vr_encargo', dado.get('vl_pis_cofins')))
                    reg_C481.VL_PIS = self.formata_numero(vl_pis, size=1, separador=',')
                    reg_C481.COD_ITEM = dado.get('cod_item')
                    reg_C481.COD_CTA = self.sped_get_conta_contabil(
                        cr, uid, cfop_id=dado.get('cfop_id'), context=context)
                    if dado.get('vl_pis'):
                        DEBITO_PIS += dado.get('vl_pis_cofins') or 0.00
                    reg_C481.write_line()

        # Registro C485
        def _registroC485(ecf_id, dt_doc):
            # print 'REGISTRO C485'
            fisnit = self.pool.get('fiscal.nf.item.tributacao')
            while obj_sql.C485.get('executando'):
                pass
            if obj_sql.C485.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.C485.get('erro')))
            global DEBITO_COFINS
            chave = str(ecf_id) + str(dt_doc)
            dados = filter(lambda i: str(i.get('ecf_id')) + str(i.get('data')) == chave,
                           obj_sql.C485.get('dados'))
            for dado in dados:
                if dado.get('vl_item'):
                    reg_C485.CST_COFINS = dado.get('cst_pis_cofins') or ''
                    reg_C485.VL_ITEM = self.formata_numero(
                        dado.get('vl_item'), size=1, separador=',')
                    # Tratamento para cst 05 conforme Pag.73 Topico 4 do Guia pratico EFD
                    # contribuicoes versao 1.21
                    # http://www1.receita.fazenda.gov.br/sistemas/efd-contribuicoes/download/download.htm
                    if dado.get('cst_pis_cofins') == '05':
                        reg_C485.VL_BC_COFINS = '0,00'
                    else:
                        reg_C485.VL_BC_COFINS = self.formata_numero(
                            dado.get('vl_bc_pis_cofins'), size=1, separador=',')
                    reg_C485.ALIQ_COFINS = self.formata_numero(
                        dado.get('aliq_pis_cofins'), size=1, separador=',')
                    reg_C485.QUANT_BC_COFINS = ''
                    reg_C485.ALIQ_COFINS_QUANT = ''
                    vl_cofins = \
                        float(fisnit._float_persist('vr_encargo', dado.get('vl_pis_cofins')))
                    reg_C485.VL_COFINS = self.formata_numero(vl_cofins, size=1, separador=',')
                    reg_C485.COD_ITEM = dado.get('cod_item')
                    reg_C485.COD_CTA = self.sped_get_conta_contabil(
                        cr, uid, cfop_id=dado.get('cfop_id'), context=context)
                    if dado.get('vl_pis_cofins'):
                        DEBITO_COFINS += dado.get('vl_pis_cofins') or 0.00
                    reg_C485.write_line()

        # Registro C500
        def _registroC500(company_id):
            # print 'REGISTRO C500'
            if mov_notas:
                if mov_notas_energia:
                    filtro = {
                        'movimento_notas_energia': mov_notas_energia,
                        'company_id': company_id
                    }
                    dados = obj_sql.sql_C500(cr, filtro)
                    for dado in dados:
                        if dado.get('cod_sit') in ('02', '05'):
                            reg_C500.COD_MOD = dado.get('cod_mod') or ''
                            reg_C500.COD_SIT = dado.get('cod_sit') or ''
                            reg_C500.SER = dado.get('ser') or ''
                            reg_C500.SUB = dado.get('sub') or ''
                            reg_C500.NUM_DOC = dado.get('num_doc') or ''
                            reg_C500.write_line()
                        else:
                            reg_C500.COD_PART = dado.get('cod_part') or ''
                            reg_C500.COD_MOD = dado.get('cod_mod') or ''
                            reg_C500.COD_SIT = dado.get('cod_sit') or ''
                            reg_C500.SER = dado.get('ser') or ''
                            reg_C500.SUB = dado.get('sub') or ''
                            reg_C500.NUM_DOC = dado.get('num_doc') or ''
                            reg_C500.DT_DOC = self.formata_data(
                                dado.get('dt_doc')) if dado.get('dt_doc') else ''
                            reg_C500.DT_ENT = self.formata_data(
                                dado.get('dt_doc')) if dado.get('dt_doc') else ''
                            reg_C500.VL_DOC = self.formata_numero(
                                dado.get('vl_doc'), size=1, separador=',')
                            reg_C500.VL_ICMS = self.formata_numero(
                                dado.get('vl_icms'), size=1, separador=',')
                            if dado.get('txt'):
                                reg_C500.COD_INF = dado.get('id')
                            reg_C500.VL_PIS = self.formata_numero(
                                dado.get('vl_pis'), size=1, separador=',')
                            reg_C500.VL_COFINS = self.formata_numero(
                                dado.get('vl_cofins'), size=1, separador=',')
                            if cod_versao and int(cod_versao) >= 6:
                                reg_C500.CHV_DOCe = dado.get('chv_doce') or ''
                            reg_C500.write_line()
                            if isinstance(dado.get('id'), int):
                                cod_cta = self.sped_get_conta_contabil(
                                    cr, uid, nf_id=dado.get('id'), context=context)
                                _registroC501(dado.get('id'), cod_cta)
                                _registroC505(dado.get('id'), cod_cta)

        # Registro C501
        def _registroC501(nf_id, cod_cta=''):
            # print 'REGISTRO C501'
            global CREDITO_PIS
            global DEBITO_PIS
            cst_saida = ['01', '20', '03', '04', '05', '06', '07', '08', '09', '49']
            dados = obj_sql.sql_C501(cr, nf_id)
            for dado in dados:
                reg_C501.CST_PIS = dado.get('cst_pis') or ''
                reg_C501.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=1, separador=',')
                reg_C501.NAT_BC_CRED = '04'
                reg_C501.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=1, separador=',')
                reg_C501.ALIQ_PIS = self.formata_numero(dado.get('aliq_pis'), size=1, separador=',')
                reg_C501.VL_PIS = self.formata_numero(dado.get('vl_pis'), size=1, separador=',')
                reg_C501.COD_CTA = cod_cta
                if dado.get('cst_pis') in cst_saida:
                    if dado.get('vl_pis'):
                        DEBITO_PIS += dado.get('vl_pis') or 0.00
                else:
                    if dado.get('vl_pis'):
                        CREDITO_PIS += dado.get('vl_pis') or 0.00
                reg_C501.write_line()

        # Registro C505
        def _registroC505(nf_id, cod_cta=''):
            # print 'REGISTRO C505'
            global CREDITO_COFINS
            global DEBITO_COFINS
            cst_saida = ['01', '20', '03', '04', '05', '06', '07', '08', '09', '49']
            dados = obj_sql.sql_C505(cr, nf_id)
            for dado in dados:
                reg_C505.CST_COFINS = dado.get('cst_cofins') or ''
                reg_C505.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=1, separador=',')
                reg_C505.NAT_BC_CRED = '04'
                reg_C505.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=1, separador=',')
                reg_C505.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',')
                reg_C505.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=1, separador=',')
                reg_C505.COD_CTA = cod_cta
                if dado.get('cst_cofins') in cst_saida:
                    DEBITO_COFINS += dado.get('vl_cofins') or 0.00
                else:
                    CREDITO_COFINS += dado.get('vl_cofins') or 0.00
                reg_C505.write_line()

        # Bloco D
        # Registro D001
        def _registroD001():
            # print 'REGISTRO D001'
            reg_D001.REG = 'D001'
            reg_D001.IND_MOV = '1'
            if mov_notas:
                _registroD010()
                if reg_D001.IND_MOV == '1':  # Se passou por D100 e D500 e nao tem movimento
                    reg_D001.write_line()
            else:
                reg_D001.write_line()

        # Registro D010
        def _registroD010():
            # print 'REGISTRO D010'
            for emp in empresas_cnpj:
                reg_D010.CNPJ = self.remove_ponto(empresas_cnpj.get(emp) or '')
                _registroD100(emp)
                _registroD500(emp)

        # Registro D0100
        def _registroD100(company_id):
            # print 'REGISTRO D100'
            dados = obj_sql.sql_D100(cr, mov_notas, company_id)
            if dados:
                if reg_D001.IND_MOV == '1':
                    reg_D001.IND_MOV = '0'
                    reg_D001.write_line()
                reg_D010.write_line()
            for dado in dados:
                reg_D100.REG = dado.get('reg')
                reg_D100.IND_OPER = dado.get('ind_oper') or ''
                reg_D100.IND_EMIT = dado.get('ind_emit')
                reg_D100.COD_PART = dado.get('cod_part')
                reg_D100.COD_MOD = dado.get('cod_mod')
                reg_D100.COD_SIT = dado.get('cod_sit') or '00'
                reg_D100.SER = dado.get('ser') or ''
                reg_D100.SUB = dado.get('sub') or ''
                reg_D100.NUM_DOC = dado.get('num_doc')
                reg_D100.CHV_CTE = dado.get('chv_cte') or ''
                reg_D100.DT_DOC = self.formata_data(dado.get('dt_doc')) or ''
                reg_D100.DT_A_P = self.formata_data(dado.get('dt_a_p')) or ''
                reg_D100.TP_CT_e = dado.get('tp_ct_e') or ''
                reg_D100.CHV_CTE_REF = dado.get('chv_cte_ref')
                reg_D100.VL_DOC = self.formata_numero(dado.get('vl_doc'), size=0, separador=',')
                reg_D100.VL_DESC = self.formata_numero(dado.get('vl_desc'), size=0, separador=',')
                reg_D100.IND_FRT = dado.get('ind_frt')
                reg_D100.VL_SERV = self.formata_numero(dado.get('vl_serv'), size=0, separador=',')
                reg_D100.VL_BC_ICMS = self.formata_numero(
                    dado.get('vl_bc_icms'), size=0, separador=',')
                reg_D100.VL_ICMS = self.formata_numero(dado.get('vl_icms'), size=0, separador=',')
                reg_D100.VL_NT = self.formata_numero(dado.get('vl_nt'), size=0, separador=',')
                if dado.get('txt'):
                    reg_D100.COD_INF = dado.get('nf_id')
                else:
                    reg_D100.COD_INF = ''
                cod_cta = self.sped_get_conta_contabil(
                    cr, uid, nf_id=dado.get('nf_id'), context=context)
                reg_D100.COD_CTA = cod_cta
                reg_D100.write_line()
                _registroD101(dado.get('nf_id'), cod_cta)
                _registroD105(dado.get('nf_id'), cod_cta)

        # Registro D101
        def _registroD101(nf_id, cod_cta=''):
            # print 'REGISTRO D101'
            global CREDITO_PIS
            dados = obj_sql.sql_D101(cr, nf_id)
            for dado in dados:
                reg_D101.REG = 'D101'
                if dado.get('vl_pis') > 0:
                    CREDITO_PIS += dado.get('vl_pis') or 0.00
                    reg_D101.IND_NAT_FRT = '2'
                else:
                    reg_D101.IND_NAT_FRT = '3'
                reg_D101.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=0, separador=',')
                reg_D101.CST_PIS = dado.get('cst_pis')
                reg_D101.NAT_BC_CRED = dado.get('codigo_natureza_credito')
                reg_D101.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=0, separador=',')
                reg_D101.ALIQ_PIS = self.formata_numero(dado.get('aliq_pis'), size=0, separador=',')
                reg_D101.VL_PIS = self.formata_numero(dado.get('vl_pis'), size=0, separador=',')
                reg_D101.COD_CTA = cod_cta
                reg_D101.write_line()

        # Registro D105
        def _registroD105(nf_id, cod_cta=''):
            # print 'REGISTRO D105'
            global CREDITO_COFINS
            dados = obj_sql.sql_D105(cr, nf_id)
            for dado in dados:
                reg_D105.REG = 'D105'
                if dado.get('vl_cofins') > 0:
                    reg_D105.IND_NAT_FRT = '2'
                    CREDITO_COFINS += dado.get('vl_cofins') or 0.00
                else:
                    reg_D105.IND_NAT_FRT = '3'
                reg_D105.VL_ITEM = self.formata_numero(dado.get('vl_item'), size=0, separador=',')
                reg_D105.CST_COFINS = dado.get('cst_cofins')
                reg_D105.NAT_BC_CRED = dado.get('codigo_natureza_credito')
                reg_D105.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=0, separador=',')
                reg_D105.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=0, separador=',')
                reg_D105.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=0, separador=',')
                reg_D105.COD_CTA = cod_cta
                reg_D105.write_line()

        # Registro D200
        def _registroD200():
            # print 'REGISTRO D200'
            if mov_notas:
                dados = obj_sql.sql_D200(cr, mov_notas)
                for dado in dados:
                    if dado.get('dt_doc_ini'):
                        reg_D200 = bloco_d.registro_D200()
                        reg_D200.COD_MOD = dado.get('cod_mod') or ''
                        reg_D200.COD_SIT = dado.get('cod_sit') or ''
                        reg_D200.SER = dado.get('ser') or ''
                        reg_D200.SUB = dado.get('sub') or ''
                        reg_D200.NUM_DOC_INI = dado.get('num_doc_ini') or ''
                        reg_D200.NUM_DOC_FIN = dado.get('num_doc_fin') or ''
                        reg_D200.CFOP = dado.get('cfop') or ''
                        reg_D200.DT_REF = self.formata_data(
                            dado.get('dt_ref')) if dado.get('dt_ref') else ''
                        reg_D200.VL_DOC = self.formata_numero(
                            dado.get('vl_doc'), size=1, separador=',') if dado.get('vl_doc') else ''
                        reg_D200.VL_DESC = self.formata_numero(
                            dado.get('vl_desc'), size=1, separador=',') if dado.get('vl_desc') \
                            else ''
                        reg_D200.write_line()

        # Registro D201
        def _registroD201():
            # print 'REGISTRO D201'
            dados = obj_sql.sql_D201(cr, filtro=False)
            for dado in dados:
                reg_D201.CST_PIS = dado.get('cst_pis') or ''
                reg_D201.VL_ITEM = self.formata_numero(
                    dado.get('vl_item'), size=1, separador=',') if dado.get('vl_item') else '0,00'
                reg_D201.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=1, separador=',') if dado.get('vl_bc_pis') \
                    else '0,00'
                reg_D201.ALIQ_PIS = self.formata_numero(
                    dado.get('aliq_pis'), size=1, separador=',') if dado.get('aliq_pis') else '0,00'
                reg_D201.VL_PIS = self.formata_numero(
                    dado.get('vl_pis'), size=1, separador=',') if dado.get('vl_pis') else '0,00'
                reg_D201.COD_CTA = self.sped_get_conta_contabil(
                    cr, uid, movimento_item_id=dado.get('movimento_item_id'), context=context)
                reg_D201.write_line()

        # Registro D205
        def _registroD205():
            # print 'REGISTRO D205'
            dados = obj_sql.sql_D205(cr, filtro=False)
            for dado in dados:
                reg_D205.CST_COFINS = dado.get('cst_cofins') or ''
                reg_D205.VL_ITEM = self.formata_numero(
                    dado.get('vl_item'), size=1, separador=',') if dado.get('vl_item') else '0,00'
                reg_D205.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=1, separador=',') if dado.get('vl_bc_cofins') \
                    else '0,00'
                reg_D205.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',') if dado.get('aliq_cofins') \
                    else '0,00'
                reg_D205.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=1, separador=',') if dado.get('vl_cofins') \
                    else '0,00'
                reg_D205.COD_CTA = self.sped_get_conta_contabil(
                    cr, uid, movimento_item_id=dado.get('movimento_item_id'), context=context)
                reg_D205.write_line()

        # Registro D500
        def _registroD500(company_id):
            # print 'REGISTRO D500'
            dados = obj_sql.sql_D500(cr, data_ini, data_fim, company_id)
            for dado in dados:
                if reg_D001.IND_MOV == '1':
                    reg_D001.IND_MOV = '0'
                    reg_D001.write_line()
                    reg_D010.write_line()
                reg_D500.REG = 'D500'
                reg_D500.IND_OPER = '0'
                reg_D500.IND_EMIT = dado.get('ind_emit')
                reg_D500.COD_PART = dado.get('cod_part')
                reg_D500.COD_MOD = dado.get('cod_mod')
                reg_D500.COD_SIT = dado.get('cod_sit')
                reg_D500.SER = dado.get('ser') or ''
                reg_D500.SUB = dado.get('sub') or ''
                reg_D500.NUM_DOC = dado.get('num_doc')
                reg_D500.DT_DOC = self.formata_data(dado.get('dt_doc'))
                reg_D500.DT_A_P = self.formata_data(dado.get('dt_a_p'))
                reg_D500.VL_DOC = self.formata_numero(dado.get('vl_doc'), size=1, separador=',')
                reg_D500.VL_DESC = self.formata_numero(dado.get('vl_desc'), size=1, separador=',')
                reg_D500.VL_SERV = self.formata_numero(dado.get('vl_serv'), size=1, separador=',')
                reg_D500.VL_SERV_NT = self.formata_numero(
                    dado.get('vl_serv_nt'), size=1, separador=',')
                reg_D500.VL_TERC = self.formata_numero(dado.get('vl_terc'), size=1, separador=',')
                reg_D500.VL_DA = self.formata_numero(dado.get('vl_da'), size=1, separador=',')
                reg_D500.VL_BC_ICMS = self.formata_numero(
                    dado.get('vl_icms'), size=1, separador=',')
                reg_D500.VL_ICMS = self.formata_numero(dado.get('vl_icms'), size=1, separador=',')
                if dado.get('txt'):
                    reg_D500.COD_INF = dado.get('nf_id')
                else:
                    reg_D500.COD_INF = ''
                reg_D500.VL_PIS = self.formata_numero(dado.get('vl_pis'), size=1, separador=',')
                reg_D500.VL_COFINS = self.formata_numero(
                    dado.get('vl_cofins'), size=1, separador=',')
                reg_D500.write_line()
                cod_cta = self.sped_get_conta_contabil(
                    cr, uid, nf_id=dado.get('nf_id'), context=context)
                _registroD501(dado.get('nf_id'), cod_cta)
                _registroD505(dado.get('nf_id'), cod_cta)

        # Registro D501
        def _registroD501(nf_id, cod_cta=''):
            # print 'REGISTRO D501'
            dados = obj_sql.sql_D501_D505(cr, nf_id, 'PIS')
            for dado in dados:
                if 49 < int(dado.get('cst')) < 57:
                    reg_D501.REG = 'D501'
                    reg_D501.CST_PIS = dado.get('cst')
                    reg_D501.VL_ITEM = self.formata_numero(
                        dado.get('vr_total_produtos'), size=1, separador=',')
                    reg_D501.NAT_BC_CRED = dado.get('codigo_natureza_credito') or '03'
                    reg_D501.VL_BC_PIS = self.formata_numero(
                        dado.get('vr_base_calculo'), size=1, separador=',')
                    reg_D501.ALIQ_PIS = self.formata_numero(
                        dado.get('aliquota'), size=1, separador=',')
                    reg_D501.VL_PIS = self.formata_numero(
                        dado.get('vr_encargo'), size=1, separador=',')
                    reg_D501.COD_CTA = cod_cta
                    reg_D501.write_line()

        # Registro D505
        def _registroD505(nf_id, cod_cta=''):
            # print 'REGISTRO D505'
            dados = obj_sql.sql_D501_D505(cr, nf_id, 'COFINS')
            for dado in dados:
                if 49 < int(dado.get('cst')) < 57:
                    reg_D505.REG = 'D505'
                    reg_D505.CST_COFINS = dado.get('cst')
                    reg_D505.VL_ITEM = self.formata_numero(
                        dado.get('vr_total_produtos'), size=1, separador=',')
                    reg_D505.NAT_BC_CRED = dado.get('codigo_natureza_credito') or '03'
                    reg_D505.VL_BC_COFINS = self.formata_numero(
                        dado.get('vr_base_calculo'), size=1, separador=',')
                    reg_D505.ALIQ_COFINS = self.formata_numero(
                        dado.get('aliquota'), size=1, separador=',')
                    reg_D505.VL_COFINS = self.formata_numero(
                        dado.get('vr_encargo'), size=1, separador=',')
                    reg_D505.COD_CTA = cod_cta
                    reg_D505.write_line()

        # Bloco F
        # Registro F001
        def _registroF001():
            # print 'REGISTRO F001'
            reg_F001.IND_MOV = '1'
            reg_F001.write_line()

        # Bloco M
        # Registro M001
        def _registroM001(possui_movimento):
            # print 'REGISTRO M001'
            MODULO_ECF_INSTALADO = self.pool.get("adm.utils").admutl_check_modulo_instalado(
                cr, uid, 'ecf', context={})
            filtro = {
                    'data_ini': data_ini,
                    'data_fim': data_fim,
                    'ecf_instalado': MODULO_ECF_INSTALADO
                }
            # dadosM200 = obj_sql.sql_M200(cr, filtro)
            if possui_movimento:
                reg_M001.IND_MOV = '0'
                reg_M001.write_line()
                if not regime_incidencia == '2':
                    _registroM100()
                _registroM200()
                _registroM400()
                if not regime_incidencia == '2':
                    _registroM500()
                _registroM600()
                _registroM800()
            else:
                reg_M001.IND_MOV = '1'
                reg_M001.write_line()

        # Registro M0100
        def _registroM100():
            # print 'REGISTRO M100'
            while obj_sql.M100.get('executando'):
                pass
            if obj_sql.M100.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.M100.get('erro')))
            global mensagem
            global CREDITO_PIS
            global DEBITO_PIS
            DEBITO_PIS = 0.00
            CREDITO_PIS = 0.00
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'encargo': 'PIS',
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids
            }
            total_debitos = obj_sql.sql_total_debito(cr, filtro, nf_nao_fiscais)
            for debito in total_debitos:
                DEBITO_PIS = debito.get('vl_tot_cont_nc_per') or 0.00
            for dado in obj_sql.M100.get('dados'):
                vl_cred = round((dado.get('vl_bc_pis') or 0.0) *
                                (dado.get('aliq_pis') or 0.0) / 100, 2)
                CREDITO_PIS += vl_cred
                cst = dado.get('cst_codigo', "")
                cod_cred = ''
                if str(cst).isdigit():
                    if int(cst) == 53:
                        cod_cred = '201'
                    elif 59 < int(cst) < 67:
                        cod_cred = '106'
                    else:
                        cod_cred = '101'
                reg_M100.COD_CRED = cod_cred
                reg_M100.IND_CRED_ORI = '0'
                reg_M100.VL_BC_PIS = self.formata_numero(
                    dado.get('vl_bc_pis'), size=1, separador=',') or '0,00'
                reg_M100.ALIQ_PIS = self.formata_numero(
                    dado.get('aliq_pis'), size=1, separador=',')
                reg_M100.QUANT_BC_PIS = '0,000'
                reg_M100.ALIQ_PIS_QUANT = ''
                reg_M100.VL_CRED = self.formata_numero(vl_cred, size=1, separador=',') or '0,00'
                reg_M100.VL_AJUS_ACRES = '0,00'
                reg_M100.VL_AJUS_REDUC = '0,00'
                reg_M100.VL_CRED_DIF = '0,00'
                reg_M100.VL_CRED_DISP = self.formata_numero(
                    vl_cred, size=1, separador=',') or '0,00'
                if vl_cred > DEBITO_PIS:
                    reg_M100.IND_DESC_CRED = '1'
                    reg_M100.VL_CRED_DESC = self.formata_numero(
                        DEBITO_PIS, size=1, separador=',') or '0,00'
                    sld_cred = vl_cred - DEBITO_PIS
                    reg_M100.SLD_CRED = self.formata_numero(sld_cred, size=1, separador=',')
                else:
                    reg_M100.IND_DESC_CRED = '0'
                    vl_cred_desc = vl_cred
                    reg_M100.VL_CRED_DESC = self.formata_numero(
                        vl_cred_desc, size=1, separador=',') or '0,00'
                    reg_M100.SLD_CRED = '0,00'
                reg_M100.write_line()
                _registroM105(dado.get('aliq_pis'), dado.get('cst_codigo'), cod_cred)

        # Registro M105
        def _registroM105(aliquota, cst, cod_cred):
            # print 'REGISTRO M105'
            while obj_sql.M105.get('executando'):
                pass
            if obj_sql.M105.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.M105.get('erro')))
            global mensagem
            for dado in filter(
                    lambda i: i.get('aliquota') == aliquota and (
                                cst == i.get('cst_pis') if cod_cred == '201' else True),
                    obj_sql.M105.get('dados')):
                cst_pis = int(dado.get('cst_pis'))
                if 49 < cst_pis < 67:
                    reg_M105.NAT_BC_CRED = dado.get('codigo_natureza_credito') or ''
                    reg_M105.CST_PIS = cst_pis or ''
                    reg_M105.VL_BC_PIS_TOT = self.formata_numero(
                        dado.get('vl_bc_pis_tot'), size=1, separador=',') or ''
                    reg_M105.VL_BC_PIS_CUM = ''
                    reg_M105.VL_BC_PIS_NC = self.formata_numero(
                        dado.get('vl_bc_pis_tot'), size=1, separador=',') or ''
                    reg_M105.VL_BC_PIS = reg_M105.VL_BC_PIS_NC if \
                        dado.get('cst_pis') in ('50', '51', '52', '60', '61', '62') or \
                        cod_cred == '201' else 0
                    reg_M105.QUANT_BC_PIS_TOT = ''
                    reg_M105.QUANT_BC_PIS = ''
                    reg_M105.DESC_CRED = ''
                    reg_M105.write_line()

        # Registro M200
        def _registroM200():
            # print 'REGISTRO M200'
            if regime_incidencia in ('1', '3'):
                _registroM200_nao_cumulativo()
            else:
                _registroM200_cumulativo()

        def _registroM200_nao_cumulativo():
            global DEBITO_PIS
            global CREDITO_PIS
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids
            }
            dados_m210 = obj_sql.sql_M210(cr, filtro, nf_nao_fiscais)
            dados_m200 = filter(lambda m: m.get('aliq_pis') and m.get('cst_pis') != '05' and m.get(
                'vl_cont_apur') > 0, dados_m210)
            aliquota_total = sum(map(lambda m: m.get('aliq_pis'), dados_m200))
            base_calculo_total = sum(map(lambda m: m.get('vl_bc_cont'), dados_m200))
            vl_tot_cont_nc_per = (aliquota_total / 100) * base_calculo_total
            if DEBITO_PIS > CREDITO_PIS:
                vl_tot_cred_desc = CREDITO_PIS
            else:
                vl_tot_cred_desc = DEBITO_PIS

            vl_tot_cont_nc_dev = round(vl_tot_cont_nc_per - vl_tot_cred_desc, 2)
            if vl_tot_cont_nc_dev < 0:
                vl_tot_cont_nc_dev = 0

            reg_M200.VL_TOT_CONT_NC_PER = self.formata_numero(
                vl_tot_cont_nc_per, size=1, separador=',') or '0,00'
            reg_M200.VL_TOT_CRED_DESC = self.formata_numero(
                vl_tot_cred_desc, size=1, separador=',') or '0,00'
            reg_M200.VL_TOT_CRED_DESC_ANT = '0,00'
            reg_M200.VL_TOT_CONT_NC_DEV = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M200.VL_RET_NC = '0,00'
            reg_M200.VL_OUT_DED_NC = '0,00'
            reg_M200.VL_CONT_NC_REC = self.formata_numero(vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M200.VL_TOT_CONT_CUM_PER = '0,00'
            reg_M200.VL_RET_CUM = '0,00'
            reg_M200.VL_OUT_DED_CUM = '0,00'
            reg_M200.VL_CONT_CUM_REC = '0,00'
            reg_M200.VL_TOT_CONT_REC = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M200.write_line()
            if vl_tot_cont_nc_dev > 0:
                campo = '08'
                vl_debito = vl_tot_cont_nc_dev
                _registroM205(campo, vl_debito)
            _registroM210(dados_m210)

        def _registroM200_cumulativo():
            """
            1-REG
            2-VL_TOT_CONT_NEC_PER
            3-VL_TOT_CRED_DESC
            4-VL_TOT_CRED_DESC_ANT
            5-VL_TOT_CONT_NC_DEV
            6-VL_RET_NC
            7-VL_OUT_DED_NC
            8-VL_CONT_NC_REC
            9-VL_TOT_CONT_CUM_PER
            10-VL_RET_CUM
            11-VL_OUT_DED_CUM
            12-VL_CONT_CUM_REC
            13-VL_TOT_CONT_REC
            """
            global DEBITO_PIS
            global CREDITO_PIS
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids
            }
            dados_m210 = obj_sql.sql_M210(cr, filtro, nf_nao_fiscais)
            vl_contribuicao_apurada = sum(map(
                lambda m: m.get('vl_bc_cont') * m.get('aliq_pis') / 100.0, dados_m210))
            vl_tot_cont_nc_dev = 0.0
            vl_ret_cum = 0.0
            vl_out_deb_cum = 0.00
            vl_tot_cont_cum_per = vl_contribuicao_apurada
            vl_cont_cum_rec = vl_tot_cont_cum_per - vl_ret_cum - vl_out_deb_cum
            vl_tot_cred_desc = 0.00
            vl_cont_nc_rec = 0.00
            vl_tot_cont_rec = vl_cont_nc_rec + vl_cont_cum_rec
            reg_M200.VL_TOT_CONT_NC_PER = '0,00'
            reg_M200.VL_TOT_CRED_DESC = self.formata_numero(vl_tot_cred_desc, size=1, separador=',')
            reg_M200.VL_TOT_CRED_DESC_ANT = '0,00'
            reg_M200.VL_TOT_CONT_NC_DEV = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M200.VL_RET_NC = '0,00'
            reg_M200.VL_OUT_DED_NC = '0,00'
            reg_M200.VL_CONT_NC_REC = self.formata_numero(vl_cont_nc_rec, size=1, separador=',')
            reg_M200.VL_TOT_CONT_CUM_PER = self.formata_numero(
                vl_contribuicao_apurada, size=1, separador=',')
            reg_M200.VL_RET_CUM = self.formata_numero(vl_ret_cum, size=1, separador=',')
            reg_M200.VL_OUT_DED_CUM = self.formata_numero(vl_out_deb_cum, size=1, separador=',')
            reg_M200.VL_CONT_CUM_REC = self.formata_numero(vl_cont_cum_rec, size=1, separador=',')
            reg_M200.VL_TOT_CONT_REC = self.formata_numero(vl_tot_cont_rec, size=1, separador=',')
            reg_M200.write_line()
            if vl_cont_nc_rec > 0 or vl_cont_cum_rec > 0:
                if vl_cont_nc_rec > 0:
                    campo = '08'
                    vl_debito = vl_cont_nc_rec
                    _registroM205(campo, vl_debito)
                if vl_cont_cum_rec > 0:
                    campo = '12'
                    vl_debito = vl_cont_cum_rec
                    _registroM205(campo, vl_debito)
            _registroM210(dados_m210)

        # Registro M205
        def _registroM205(campo, vl_debito):
            # print 'REGISTRO M205'
            cod_rec = fiscal_parametro_dados.get('fispar_cod_rec_pis') or ''
            reg_M205.REG = 'M205'
            reg_M205.NUM_CAMPO = campo or ''
            reg_M205.COD_REC = cod_rec
            reg_M205.VL_DEBITO = self.formata_numero(vl_debito, size=1, separador=',') or '0,00'
            reg_M205.write_line()

        # Registro M210
        def _registroM210(dados):
            # print 'REGISTRO M210'
            global mensagem
            if regime_incidencia == '1':
                m210_cod_cont = '01'
            elif regime_incidencia == '2':
                m210_cod_cont = '51'
            elif regime_incidencia == '3':
                m210_cod_cont = '01'
            else:
                m210_cod_cont = ''
                mensagem += '* Regime de incidencia tributaria nao informado no parametro do ' \
                            'Fiscal \n'
            for dado in dados:
                if dado.get('cst_pis') in ('01', '02', '03', '04', '05'):
                    if dado.get('aliq_pis') or dado.get('aliq_pis') == 0:
                        if dado.get('aliq_pis') == 0:
                            cod_cont = '02'
                        elif dado.get('aliq_pis') == 0.65 and dado.get('cst_pis') == '05':
                            cod_cont = '31'
                        else:
                            cod_cont = m210_cod_cont
                        reg_M210.COD_CONT = cod_cont
                        reg_M210.VL_REC_BRT = self.formata_numero(
                            dado.get('vl_rec_brt'), size=1, separador=',') if dado.get(
                            'vl_rec_brt') else ''
                        reg_M210.VL_BC_CONT = self.formata_numero(
                            dado.get('vl_bc_cont'), size=1, separador=',') if dado.get(
                            'vl_bc_cont') else '0,00'
                        if cod_versao and int(cod_versao) >= 5:
                            reg_M210.VL_AJUS_ACRES_BC_PIS = '0,00'
                            reg_M210.VL_AJUS_REDUC_BC_PIS = '0,00'
                            reg_M210.VL_BC_CONT_AJUS = reg_M210.VL_BC_CONT
                        if dado.get('aliq_pis') == 0:
                            aliq_pis = '0,00'
                        elif dado.get('aliq_pis') > 0:
                            aliq_pis = self.formata_numero(dado.get('aliq_pis'), size=1,
                                                           separador=',')
                        else:
                            aliq_pis = ''
                        vl_cont_apur = 0.00
                        if dado.get('vl_bc_cont') and dado.get('aliq_pis') > 0:
                            vl_cont_apur = dado.get('vl_bc_cont') * dado.get('aliq_pis') / 100
                        reg_M210.ALIQ_PIS =  aliq_pis
                        reg_M210.QUANT_BC_PIS = ''
                        reg_M210.ALIQ_PIS_QUANT = ''
                        reg_M210.VL_CONT_APUR = self.formata_numero(
                            vl_cont_apur, size=1, separador=',') or '0.00'
                        reg_M210.VL_AJUS_ACRES = '0,00'
                        reg_M210.VL_AJUS_REDUC = '0,00'
                        reg_M210.VL_CONT_DIFER = '0,00'
                        reg_M210.VL_CONT_DIFER_ANT = '0,00'
                        reg_M210.VL_CONT_PER = self.formata_numero(
                            vl_cont_apur, size=1, separador=',') or '0.00'
                        reg_M210.write_line()
                        cooperativa = browse_sped.sped_empresa_cooperativa
                        if cooperativa:
                            _registroM211(dado.get('vl_rec_brt'))

        # Registro M211
        def _registroM211(vl_rec_brt):
            # print 'REGISTRO M211'
            parametros_fiscal = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'fiscal', ['fispar_indicador_natureza_pessoa_juridica',
                                    'fispar_cooperativa'], context=context)
            if parametros_fiscal.get('fispar_indicador_natureza_pessoa_juridica') == '01':
                vl_exc_coop_gerp = browse_sped.sped_pis_exclusao_geral
                vl_exc_esp_coop = browse_sped.sped_pis_exclusao_especifica
                reg_M211.REG = 'M211'
                reg_M211.IND_TIP_COOP = parametros_fiscal.get('fispar_cooperativa') or str()
                reg_M211.VL_BC_CONT_ANT_EXC_COOP = self.formata_numero(
                    vl_rec_brt, size=1, separador=',') or '0,00'
                reg_M211.VL_EXC_COOP_GER = self.formata_numero(
                    vl_exc_coop_gerp, size=1, separador=',') or '0,00'
                reg_M211.VL_EXC_ESP_COOP = self.formata_numero(
                    vl_exc_esp_coop, size=1, separador=',') or '0,00'
                vl_bc_cont = vl_rec_brt - vl_exc_coop_gerp - vl_exc_esp_coop
                reg_M211.VL_BC_CONT = self.formata_numero(
                    vl_bc_cont, size=1, separador=',') or '0,00'
                reg_M211.write_line()

        # Registro M400
        def _registroM400():
            # print 'REGISTRO M400'
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids,
                'contabil_fiscal': contabil_fiscal,
                'coo_natureza': fiscal_parametro_dados.get('cupom_nota_natureza_id') or 0
            }
            if filtro.get('contabil_fiscal'):
                filtro.update({'natureza_fiscal_company_id': natureza_fiscal_company_id})
            # csts validos ('04', '06', '07', '08', '09')
            dados = obj_sql.sql_M400(cr, filtro)
            dados_m400 = {}
            list_chave = []
            for dado in dados:
                chave = dado.get('cst_pis') + str(dado.get('cod_cta') or '')
                if chave not in list_chave:
                    dados_m400.update({chave: dado.copy()})
                    list_chave.append(chave)
                else:
                    dados_m400[chave]['vl_tot_rec'] += (dado.get('vl_tot_rec') or 0.0)
            for dado in dados_m400.values():
                reg_M400.CST_PIS = dado.get('cst_pis') or ''
                reg_M400.VL_TOT_REC = self.formata_numero(
                    dado.get('vl_tot_rec'), size=1, separador=',') if dado.get('vl_tot_rec') else ''
                cod_cta = dado.get('cod_cta')
                reg_M400.COD_CTA = contabil_conta_dict.get(cod_cta).get('conta') if cod_cta else ''
                reg_M400.DESC_COMPL = contabil_conta_dict.get(cod_cta).get(
                    'name') if cod_cta else ''
                reg_M400.write_line()
                if cod_cta:
                    dados_m410 = filter(lambda m: m.get('cst_pis') == dado.get('cst_pis')
                                        and m.get('cod_cta') == cod_cta, dados)
                else:
                    dados_m410 = filter(lambda m: m.get('cst_pis') == dado.get('cst_pis'), dados)
                _registroM410(dados_m410)

        # Registro M410
        def _registroM410(dados_m410=[]):
            # print 'REGISTRO M410'
            global mensagem
            for dado in dados_m410:
                nat_rec = dado.get('nat_rec') or ''
                if nat_rec:
                    nat_rec = nat_rec.zfill(3)
                reg_M410.NAT_REC = nat_rec
                reg_M410.VL_REC = self.formata_numero(
                    dado.get('vl_tot_rec'), size=1, separador=',') or '0,00'
                cod_cta = dado.get('cod_cta')
                reg_M410.COD_CTA = contabil_conta_dict.get(cod_cta).get('conta') if cod_cta else ''
                reg_M410.DESC_COMPL = contabil_conta_dict.get(cod_cta).get(
                    'name') if cod_cta else ''
                reg_M410.write_line()

        # Registro M500
        def _registroM500():
            # print 'REGISTRO M500'
            while obj_sql.M500.get('executando'):
                pass
            if obj_sql.M500.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.M500.get('erro')))
            global mensagem
            global CREDITO_COFINS
            global DEBITO_COFINS
            DEBITO_COFINS = 0.00
            CREDITO_COFINS = 0.00
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'encargo': 'COFINS',
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids
            }
            total_debitos = obj_sql.sql_total_debito(cr, filtro, nf_nao_fiscais)
            for debito in total_debitos:
                DEBITO_COFINS = debito.get('vl_tot_cont_nc_per') or 0.00
            for dado in obj_sql.M500.get('dados'):
                CREDITO_COFINS += (dado.get('vl_cred') or 0.0)
                cst = dado.get('cst_codigo', "")
                cod_cred = ''
                if str(cst).isdigit():
                    if int(cst) == 53:
                        cod_cred = '201'
                    elif 59 < int(cst) < 67:
                        cod_cred = '106'
                    else:
                        cod_cred = '101'
                reg_M500.COD_CRED = cod_cred
                reg_M500.IND_CRED_ORI = '0'
                reg_M500.VL_BC_COFINS = self.formata_numero(
                    dado.get('vl_bc_cofins'), size=1, separador=',') or '0,00'
                reg_M500.ALIQ_COFINS = self.formata_numero(
                    dado.get('aliq_cofins'), size=1, separador=',')
                reg_M500.QUANT_BC_COFINS = '0,000'
                reg_M500.ALIQ_COFINS_QUANT = ''
                reg_M500.VL_CRED = self.formata_numero(
                    dado.get('vl_cred'), size=1, separador=',') or '0,00'
                reg_M500.VL_AJUS_ACRES = '0,00'
                reg_M500.VL_AJUS_REDUC = '0,00'
                reg_M500.VL_CRED_DIFER = '0,00'
                reg_M500.VL_CRED_DISP = self.formata_numero(
                    dado.get('vl_cred'), size=1, separador=',') or '0,00'
                if dado.get('vl_cred') > DEBITO_COFINS:
                    reg_M500.IND_DESC_CRED = '1'
                    reg_M500.VL_CRED_DESC = self.formata_numero(
                        DEBITO_COFINS, size=1, separador=',') or '0,00'
                    sld_cred = dado.get('vl_cred') - DEBITO_COFINS
                    reg_M500.SLD_CRED = self.formata_numero(sld_cred, size=1, separador=',')
                else:
                    reg_M500.IND_DESC_CRED = '0'
                    vl_cred_desc = dado.get('vl_cred')
                    reg_M500.VL_CRED_DESC = self.formata_numero(
                        vl_cred_desc, size=1, separador=',') or '0,00'
                    reg_M500.SLD_CRED = '0,00'
                reg_M500.write_line()
                _registroM505(dado.get('aliq_cofins'), dado.get('cst_codigo'), cod_cred)

        # Registro M505
        def _registroM505(aliquota, cst, cod_cred):
            # print 'REGISTRO M505'
            while obj_sql.M505.get('executando'):
                pass
            if obj_sql.M505.get('erro'):
                raise osv.except_osv(_('Erro'), _(obj_sql.M505.get('erro')))
            global mensagem
            for dado in filter(
                    lambda i: i.get('aliquota') == aliquota and (
                               cst == i.get('cst_cofins') if cod_cred == '201' else True),
                    obj_sql.M505.get('dados')):
                cst_cofins = int(dado.get('cst_cofins'))
                if 49 < cst_cofins < 67:
                    reg_M505.NAT_BC_CRED = dado.get('codigo_natureza_credito') or ''
                    reg_M505.CST_COFINS = cst_cofins or ''
                    reg_M505.VL_BC_COFINS_TOT = self.formata_numero(
                        dado.get('vl_bc_cofins_tot'), size=1, separador=',') or ''
                    reg_M505.VL_BC_COFINS_CUM = ''
                    reg_M505.VL_BC_COFINS_NC = self.formata_numero(
                        dado.get('vl_bc_cofins_tot'), size=1, separador=',') or ''
                    reg_M505.VL_BC_COFINS = reg_M505.VL_BC_COFINS_NC if \
                        dado.get('cst_cofins') in ('50', '51' '52', '60', '61', '62') or \
                        cod_cred == '201' else 0
                    reg_M505.QUANT_BC_COFINS_TOT = ''
                    reg_M505.QUANT_BC_COFINS = ''
                    reg_M505.DESC_CRED = ''
                    reg_M505.write_line()

        # Registro M600
        def _registroM600():
            # print 'REGISTRO M600'
            if regime_incidencia in ('1', '3'):
                _registroM600_nao_cumulativo()
            else:
                _registroM600_cumulativo()

        def _registroM600_nao_cumulativo():
            global DEBITO_COFINS
            global CREDITO_COFINS
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids
            }
            dados_m610 = obj_sql.sql_M610(cr, filtro, nf_nao_fiscais)

            dados_m600 = filter(lambda m: m.get('aliq_cofins') and m.get('cst_cofins') != '05' and
                                          m.get('vl_cont_apur') > 0,
                                dados_m610)
            aliquota_total = sum(map(lambda m: m.get('aliq_cofins'), dados_m600))
            base_calculo_total = sum(map(lambda m: m.get('vl_bc_cont'), dados_m600))
            vl_tot_cont_nc_per = (aliquota_total / 100) * base_calculo_total

            if DEBITO_COFINS > CREDITO_COFINS:
                # DEBITO_COFINS - CREDITO_COFINS
                vl_tot_cont_nc_dev = vl_tot_cont_nc_per - CREDITO_COFINS
                vl_tot_cred_desc = CREDITO_COFINS
            else:
                vl_tot_cont_nc_dev = 0.00
                vl_tot_cred_desc = DEBITO_COFINS
            reg_M600.VL_TOT_CONT_NC_PER = self.formata_numero(vl_tot_cont_nc_per, size=1,
                                                              separador=',') or '0,00'
            reg_M600.VL_TOT_CRED_DESC = self.formata_numero(
                vl_tot_cred_desc, size=1, separador=',') or '0,00'
            reg_M600.VL_TOT_CRED_DESC_ANT = '0,00'
            reg_M600.VL_TOT_CONT_NC_DEV = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M600.VL_RET_NC = '0,00'
            reg_M600.VL_OUT_DED_NC = '0,00'
            reg_M600.VL_CONT_NC_REC = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M600.VL_TOT_CONT_CUM_PER = '0,00'
            reg_M600.VL_RET_CUM = '0,00'
            reg_M600.VL_OUT_DED_CUM = '0,00'
            reg_M600.VL_CONT_CUM_REC = '0,00'
            reg_M600.VL_TOT_CONT_REC = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M600.write_line()
            if vl_tot_cont_nc_dev > 0:
                campo = '08'
                vl_debito = vl_tot_cont_nc_dev
                _registroM605(campo, vl_debito)
            _registroM610(dados_m610)

        def _registroM600_cumulativo():
            """
            1-REG
            2-VL_TOT_CONT_NEC_PER
            3-VL_TOT_CRED_DESC
            4-VL_TOT_CRED_DESC_ANT
            5-VL_TOT_CONT_NC_DEV
            6-VL_RET_NC
            7-VL_OUT_DED_NC
            8-VL_CONT_NC_REC
            9-VL_TOT_CONT_CUM_PER
            10-VL_RET_CUM
            11-VL_OUT_DED_CUM
            12-VL_CONT_CUM_REC
            13-VL_TOT_CONT_REC
            """
            global DEBITO_COFINS
            global CREDITO_COFINS
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids
            }
            dados_m610 = obj_sql.sql_M610(cr, filtro, nf_nao_fiscais)
            vl_contribuicao_apurada = sum(map(
                lambda m: m.get('vl_bc_cont') * m.get('aliq_cofins') / 100.0, dados_m610))
            vl_tot_cont_nc_dev = 0.0
            vl_ret_cum = 0.0
            vl_out_deb_cum = 0.00
            vl_tot_cont_cum_per = vl_contribuicao_apurada
            vl_cont_cum_rec = vl_tot_cont_cum_per - vl_ret_cum - vl_out_deb_cum
            vl_tot_cred_desc = 0.00
            vl_cont_nc_rec = 0.00
            vl_tot_cont_rec = vl_cont_nc_rec + vl_cont_cum_rec
            reg_M600.VL_TOT_CONT_NC_PER = '0,00'
            reg_M600.VL_TOT_CRED_DESC = self.formata_numero(vl_tot_cred_desc, size=1, separador=',')
            reg_M600.VL_TOT_CRED_DESC_ANT = '0,00'
            reg_M600.VL_TOT_CONT_NC_DEV = self.formata_numero(
                vl_tot_cont_nc_dev, size=1, separador=',')
            reg_M600.VL_RET_NC = '0,00'
            reg_M600.VL_OUT_DED_NC = '0,00'
            reg_M600.VL_CONT_NC_REC = self.formata_numero(vl_cont_nc_rec, size=1, separador=',')
            reg_M600.VL_TOT_CONT_CUM_PER = self.formata_numero(
                vl_contribuicao_apurada, size=1, separador=',')
            reg_M600.VL_RET_CUM = self.formata_numero(vl_ret_cum, size=1, separador=',')
            reg_M600.VL_OUT_DED_CUM = self.formata_numero(vl_out_deb_cum, size=1, separador=',')
            reg_M600.VL_CONT_CUM_REC = self.formata_numero(vl_cont_cum_rec, size=1, separador=',')
            reg_M600.VL_TOT_CONT_REC = self.formata_numero(vl_tot_cont_rec, size=1, separador=',')
            reg_M600.write_line()
            if vl_cont_nc_rec > 0 or vl_cont_cum_rec > 0:
                if vl_cont_nc_rec > 0:
                    campo = '08'
                    vl_debito = vl_cont_nc_rec
                    _registroM605(campo, vl_debito)
                if vl_cont_cum_rec > 0:
                    campo = '12'
                    vl_debito = vl_cont_cum_rec
                    _registroM605(campo, vl_debito)
            _registroM610(dados_m610)

        # Registro M605
        def _registroM605(campo, vl_debito):
            # print 'REGISTRO M605'
            cod_rec = fiscal_parametro_dados.get('fispar_cod_rec_cofins') or ''
            reg_M605.REG = 'M605'
            reg_M605.NUM_CAMPO = campo or ''
            reg_M605.COD_REC = cod_rec
            reg_M605.VL_DEBITO = self.formata_numero(vl_debito, size=1, separador=',') or '0,00'
            reg_M605.write_line()

        # Registro M610
        def _registroM610(dados):
            # print 'REGISTRO M610'
            global mensagem
            if regime_incidencia == '1':
                m610_cod_cont = '01'
            elif regime_incidencia == '2':
                m610_cod_cont = '51'
            elif regime_incidencia == '3':
                m610_cod_cont = '01'
            else:
                m610_cod_cont = ''
                mensagem += _('* Regime de incidencia tributaria nao informado no parametro do '
                              'Fiscal \n')
            for dado in dados:
                if dado.get('cst_cofins') in ('01', '02', '03', '04', '05'):
                    if dado.get('aliq_cofins') or dado.get('aliq_cofins') == 0:
                        if dado.get('aliq_cofins') == 0:
                            cod_cont = '02'
                        elif dado.get('aliq_cofins') == 3.0 and dado.get('cst_cofins') == '05':
                            cod_cont = '31'
                        else:
                            cod_cont = m610_cod_cont
                        reg_M610.COD_CONT = cod_cont
                        reg_M610.VL_REC_BRT = self.formata_numero(
                            dado.get('vl_rec_brt'), size=1, separador=',') if dado.get(
                            'vl_rec_brt') else ''
                        reg_M610.VL_BC_CONT = self.formata_numero(
                            dado.get('vl_bc_cont'), size=1, separador=',') if dado.get(
                            'vl_bc_cont') else '0,00'
                        if dado.get('aliq_cofins') == 0:
                            aliq_cofins = '0,00'
                        elif dado.get('aliq_cofins') > 0:
                            aliq_cofins = self.formata_numero(dado.get('aliq_cofins'), size=1,
                                                           separador=',')
                        else:
                            aliq_cofins = ''
                        vl_cont_apur = 0.00
                        if dado.get('vl_bc_cont') and dado.get('aliq_cofins') > 0:
                            vl_cont_apur = dado.get('vl_bc_cont') * dado.get('aliq_cofins') / 100
                        if cod_versao and int(cod_versao) >= 5:
                            reg_M610.VL_AJUS_ACRES_BC_COFINS = '0,00'
                            reg_M610.VL_AJUS_REDUC_BC_COFINS = '0,00'
                            reg_M610.VL_BC_CONT_AJUS = self.formata_numero(
                                dado.get('vl_bc_cont'), size=1, separador=',') if dado.get(
                                'vl_bc_cont') else '0,00'
                        reg_M610.ALIQ_COFINS = aliq_cofins
                        reg_M610.QUANT_BC_COFINS = ''
                        reg_M610.ALIQ_COFINS_QUANT = ''
                        reg_M610.VL_CONT_APUR = self.formata_numero(
                            vl_cont_apur, size=1, separador=',') or '0.00'
                        reg_M610.VL_AJUS_ACRES = '0,00'
                        reg_M610.VL_AJUS_REDUC = '0,00'
                        reg_M610.VL_CONT_DIFER = '0,00'
                        reg_M610.VL_CONT_DIFER_ANT = '0,00'
                        reg_M610.VL_CONT_PER = self.formata_numero(
                            vl_cont_apur, size=1, separador=',') or '0.00'
                        reg_M610.write_line()
                        cooperativa = browse_sped.sped_empresa_cooperativa
                        if cooperativa:
                            _registroM611(dado.get('vl_rec_brt'))

        # Registro M611
        def _registroM611(vl_rec_brt):
            # print 'REGISTRO M611'
            parametros_fiscal = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'fiscal', ['fispar_indicador_natureza_pessoa_juridica',
                                    'fispar_cooperativa'], context=context)
            if parametros_fiscal.get('fispar_indicador_natureza_pessoa_juridica') == '01':
                vl_exc_coop_gerp = browse_sped.sped_cofins_exclusao_geral or 0.00
                vl_exc_esp_coop = browse_sped.sped_cofins_exclusao_especifica or 0.00
                reg_M611.REG = 'M611'
                reg_M611.IND_TIP_COOP = parametros_fiscal.get('fispar_cooperativa') or str()
                reg_M611.VL_BC_CONT_ANT_EXC_COOP = self.formata_numero(
                    vl_rec_brt, size=1, separador=',') or '0,00'
                reg_M611.VL_EXC_COOP_GER = self.formata_numero(
                    vl_exc_coop_gerp, size=1, separador=',') or '0,00'
                reg_M611.VL_EXC_ESP_COOP = self.formata_numero(
                    vl_exc_esp_coop, size=1, separador=',') or '0,00'
                vl_bc_cont = vl_rec_brt - vl_exc_coop_gerp - vl_exc_esp_coop
                reg_M611.VL_BC_CONT = self.formata_numero(
                    vl_bc_cont, size=1, separador=',') or '0,00'
                reg_M611.write_line()

        # Registro M800
        def _registroM800():
            # print 'REGISTRO M800'
            filtro = {
                'data_ini': data_ini,
                'data_fim': data_fim,
                'ecf_instalado': MODULO_ECF_INSTALADO,
                'company_ids': company_ids,
                'contabil_fiscal': contabil_fiscal,
                'coo_natureza': fiscal_parametro_dados.get('cupom_nota_natureza_id') or 0
            }
            if filtro.get('contabil_fiscal'):
                filtro.update({'natureza_fiscal_company_id': natureza_fiscal_company_id})
            # csts validos ('04', '06', '07', '08', '09')
            dados = obj_sql.sql_M800(cr, filtro)
            dados_m800 = {}
            list_chave = []
            for dado in dados:
                chave = dado.get('cst_cofins') + str(dado.get('cod_cta') or '')
                if chave not in list_chave:
                    dados_m800.update({chave: dado.copy()})
                    list_chave.append(chave)
                else:
                    dados_m800[chave]['vl_tot_rec'] += (dado.get('vl_tot_rec') or 0.0)
            for dado in dados_m800.values():
                reg_M800.CST_COFINS = dado.get('cst_cofins') or ''
                reg_M800.VL_TOT_REC = self.formata_numero(
                    dado.get('vl_tot_rec'), size=1, separador=',') if dado.get('vl_tot_rec') else ''
                cod_cta = dado.get('cod_cta')
                reg_M800.COD_CTA = contabil_conta_dict.get(cod_cta).get('conta') if cod_cta else ''
                reg_M800.DESC_COMPL = contabil_conta_dict.get(cod_cta).get(
                    'name') if cod_cta else ''
                reg_M800.write_line()
                if cod_cta:
                    dados_m810 = filter(lambda m: m.get('cst_cofins') == dado.get('cst_cofins')
                                        and m.get('cod_cta') == cod_cta, dados)
                else:
                    dados_m810 = filter(lambda m: m.get('cst_cofins') == dado.get('cst_cofins'),
                                        dados)
                _registroM810(dados_m810)

        # Registro M810
        def _registroM810(dados_m810=[]):
            # print 'REGISTRO M810'
            global mensagem
            for dado in dados_m810:
                if dado.get('vl_tot_rec') > 0:
                    nat_rec = dado.get('nat_rec') or ''
                    if nat_rec:
                        nat_rec = nat_rec.zfill(3)
                    reg_M810.NAT_REC = nat_rec
                    reg_M810.VL_REC = self.formata_numero(
                        dado.get('vl_tot_rec'), size=1, separador=',') if dado.get(
                        'vl_tot_rec') else ''
                    cod_cta = dado.get('cod_cta')
                    reg_M810.COD_CTA = contabil_conta_dict.get(cod_cta).get(
                        'conta') if cod_cta else ''
                    reg_M810.DESC_COMPL = contabil_conta_dict.get(cod_cta).get(
                        'name') if cod_cta else ''
                    reg_M810.write_line()

        # Registro P001
        def _registroP001():
            # print 'REGISTRO P001'
            reg_P001.IND_MOV = '1'
            reg_P001.write_line()

        # Registro 1001
        def _registro1001():
            # print 'REGISTRO 1001'
            reg_1001.IND_MOV = '1'
            reg_1001.write_line()

        # Registro 9001
        def _registro9001():
            # print 'REGISTRO 9001'
            reg_9001.IND_MOV = '0'
            reg_9001.write_line()

        # Bloco 0
        _registro0000()
        _registro0001()
        _registro0100()
        _registro0110()
        _registro0140()

        # Bloco A
        _registroA001()

        # Bloco C
        _registroC001()

        # Bloco D
        _registroD001()

        # Bloco F
        _registroF001()

        # Bloco M
        _registroM001(any([mov_notas, mov_cupons]))

        # Bloco P
        _registroP001()

        # Bloco 1
        _registro1001()

        # Bloco 9
        _registro9001()

        self.sped_gera_registro_0500(cr, uid, context)

        # Apropriacao de credito
        if not fiscal_parametro_dados.get('fispar_regime_incidencia_tributaria'):
            mensagem += _('* Regime de incidencia tributaria nao informada no parametro do '
                          'Fiscal.\n')
        if not fiscal_parametro_dados.get('fispar_metodo_apropriacao_creditos'):
            mensagem += _('* Metodo de apropriacao de creditos nao informado no parametro do '
                          'Fiscal.\n')
        if not fiscal_parametro_dados.get('fispar_tipo_contribuicao_apuracao'):
            mensagem += _('* Tipo de Contribuicao nao informado no parametro do Fiscal..\n')
        if mensagem:
            try:
                self.write(
                    cr, uid, ids, {'informacao': mensagem.encode('utf-8', 'strict')}, context)
            except UnicodeDecodeError:
                self.write(
                    cr, uid, ids, {'informacao': mensagem.decode('utf-8', 'strict')}, context)
        arq = bloco_arquivo()
        arquivo = arq.monta_arquivo()
        self.write(cr, uid, ids, {
            'dados': base64.encodestring(arquivo.encode('utf-8', 'strict')),
            'sped_gerado': True,
            'state': 'E'}, context)
        _tFim = datetime.now()
        _logger.info('Fim do Sped Contribuicoes! (%s)' % str(_tFim - _tInicio))
        return True

    def limpar_consulta(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.unlink(cr, uid, ids, context=context)
        return {'name': 'Gerar SPED Contribuicoes',
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': False,
                'res_model': 'fiscal.sped.contribuicoes',
                'target': 'inline',
                'type': 'ir.actions.act_window',
        }

    def sped_get_conta_contabil(self, cr, uid, nf_id=False, movimento_item_id=False,
                                natureza_id=False, cfop_id=False, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Encontra a conta contabil analitica para a natureza da nota fiscal (nf_id), movimento do
        item no estoque (movimento_item_id) ou da natureza da emissao (natureza_id), informados.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_id: id da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_id: int
        :param movimento_item_id: id da tabela estoque_movimento_itens, identificador do item
        movimentado no estoque.
        :param natureza_id: id da tabela fiscal_cfop_natureza, identificador da natureza da emissao
        da nota fiscal.
        :type natureza_id: int
        :type movimento_item_id: int
        :param cfop_id: id da tabela fiscal_cfop, idetificador do CFOP.
        :type cfop_id: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: codigo da conta contabil.
        :rtype: str
        """
        return str()

    def sped_get_conta_contabil_nf_e_movimento(self, cr, uid, nf_ids, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Encontra a conta contabil analitica para a natureza das notas fiscais (nf_ids) e movimento
        do item no estoque.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param nf_ids: ids da tabela fiscal_nf_capa, identificador da nota fiscal.
        :type nf_ids: list
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        return True

    def sped_get_conta_contabil_natureza(self, cr, uid, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Encontra a conta contabil analitica para a natureza da emissao (natureza_id).
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        return True

    def sped_get_conta_contabil_cfop(self, cr, uid, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Encontra a conta contabil analitica para os CFOPs
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        return True

    def sped_gera_registro_0500(self, cr, uid, context=None):
        """
        Metodo sobreescrito no modulo contabil_fiscal.
        Cria os registros 0500 para a apuração do periodo.
        :param cr: cursor no banco da dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados com valores contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
