# -*- coding: utf-8 -*-

from openerp.osv import fields, osv
from openerp.tools.translate import _


class wizard_alterar_preco_venda_fiscal(osv.TransientModel):
    _name = 'wizard.alterar.preco.venda.fiscal'
    _description = 'Alterar preco de venda'

    _columns = {
        'login': fields.char('Login', size=30, required=True),
        'senha': fields.char('Senha', size=30, required=True),
        'produto_id': fields.many2one('estoque.produto', 'Produto'),
        'preco_venda': fields.float('Preco', digits=(9, 2)),
        'altera': fields.boolean('pode alterar Preco de venda'),
        'texto': fields.text('log'),
        'altera_login': fields.boolean('precisa alterar login e senha'),
    }
    _defaults = {'altera': lambda *a: False,
                 'altera_login': lambda *a: True,
    }

    def default_get(self, cr, uid, fields_list, context=None):
        """
        Recupera o ID do produto e seu respectivo preco de venda.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param fields_list: lista com os atributos da classe (campos da tabela)
        :param context: um dicionario padrao para valores contextuais
        :param fields_list:
        :param context:
        :type cr: object
        :type uid: int
        :type fields_list: list
        :type context: dict
        :return: dicionario com os valores do ID do produto e valor do preco de venda.
        :rtype: dict
        """

        context = context and context.copy() or {}
        res = super(wizard_alterar_preco_venda_fiscal, self).default_get(
            cr, uid, fields_list, context)
        item_id = context.get('active_id', False)

        if item_id:
            produto = self.pool.get('estoque.movimento.itens').read(
                cr, uid, item_id, ['produto_id', 'vr_unitario'], context=context)

            if isinstance(produto, list):
                produto = produto[0]

            if produto and produto.get('produto_id', False):
                res['produto_id'] = produto['produto_id'][0]
                res['preco_venda'] = produto['vr_unitario']

        return res

    def onchange_dados(self, cr, uid, ids, login, senha, context=None):
        """
        Ao alterar o usuario e senha no formulario de alteracao do valor do preco de venda, mostra
        mensagem de erro de usuario se este nao tiver a permissao correta.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param login: a identificacao do usuario
        :param senha: a senha do usuario
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type login: str
        :type senha: str
        :type context: dict
        :return: a mensagem de erro para usuario nao cadastrado ou sem permissao para alterar o
        preco de venda
        :rtype: dict
        """

        context = context and context.copy() or {}
        val = {'altera': False, 'altera_login': True, 'texto': False}

        if login and senha:
            usuario = self.pool.get('res.users').search(
                cr, uid, [('login', '=', login), ('password', '=', senha)], context=context)

            if usuario:
                usuario = usuario[0]
                res = self.pool.get('ir.model.access').check_groups(
                    cr, usuario, 'estoque.group_estoque_alterar_preco')

                if res:
                    val['altera'] = True
                else:
                    val['texto'] = _('Usuario sem permissao')
            else:
                val['texto'] = _('Usuario nao cadastrado')

        return {'value': val}

    def altera_preco_venda(self, cr, uid, ids, context=None):
        """
        Altera o preco de venda de um produto.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: Nao se aplica. Apos alterar o preco de venda, fecha o formulario de alteracao.
        :rtype: object
        """

        context = context and context.copy() or {}

        # Contexto criado pois o metodo 'write' usa um metodo onchange que recalcula o preco de
        # venda
        context['preco_venda_fiscal'] = True

        preco_venda = self.read(cr, uid, ids, ['preco_venda'], context=context)

        if not preco_venda:
            return {'type': 'ir.actions.act_window_close'}

        preco_venda = preco_venda[0]['preco_venda']
        item_id = context.get('active_id', False)

        if not item_id:
            return {'type': 'ir.actions.act_window_close'}

        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        movimento_itens_dados = obj_estoque_movimento_itens.read(
            cr, uid, item_id, ['quantidade', 'vr_desconto'], context=context)
        vr_total_item = (movimento_itens_dados.get('quantidade') or 0) * preco_venda
        if movimento_itens_dados.get('vr_desconto'):
            vr_total_item -= movimento_itens_dados.get('vr_desconto')
        obj_estoque_movimento_itens.write(
            cr, uid, item_id, {'vr_preco_venda': preco_venda, 'vr_total_item': vr_total_item},
            context=context)

        return {'type': 'ir.actions.act_window_close'}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
