# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from datetime import date, datetime
from openerp.osv import osv, fields
from openerp.report.interface import report_int
from openerp.report.render import render
try:
    import StringIO  # Python 2
except ImportError:
    from io import StringIO  # Python 3
from openerp import pooler
from openerp.tools.translate import _
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_relatorio_apuracao_icms1(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        pdf_file = StringIO.StringIO()
        self.db_name = pooler.get_pool(cr.dbname)
        company_id = self.db_name.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'wizard.apuracao.icms')
        brw_empresa = self.db_name.get('res.company').browse(cr, uid, company_id)
        simples_nacional = pooler.get_pool(cr.dbname).get(
            'adm.pessoa').verificar_empresa_simples_nacional(cr, uid)
        if company_id:
            empresa_name = brw_empresa.name or ''
            n_cnpj = brw_empresa.rescon_pessoa_id.cpf_cnpj or ''
            ie = brw_empresa.rescon_pessoa_id.admpes_jur_ie or ''
            nome_empresa = formataString(empresa_name, 60, usaTrim='N', forcado='S')
            cnpj = formataString(n_cnpj, 90,  usaTrim='N', forcado='S')
            ie = formataString(ie, 15, usaTrim='N', forcado='S')
            if datas.get('mes') < 10:
                datas['mes'] = '0' + str(datas.get('mes'))
            periodo = formataString(str(datas['mes']) + '/' + 
                                    str(datas['ano']), 18, usaTrim='N', forcado='S')
            listagem = Impressora()
            listagem.defineFormato()
            listagem.informaNome(pdf_file, 'Apuracao de ICMS')
            listagem.informaTamLinha(120)
            listagem.defineMargem(1, 1, 1, 1)
            valores = {
                'entrada': self.relat_entrada(cr, uid, ids, datas, listagem, nome_empresa, cnpj,
                                              ie, periodo, company_id, simples_nacional,
                                              context),
                'saida': self.relat_saida(cr, uid, ids, datas, listagem, nome_empresa, cnpj,
                                          ie, periodo, company_id, simples_nacional, context)
            }

            # Imprime o cabecalho do resumo da apuracao do imposto
            self.relat_resumo_cabecalho(cr, uid, listagem, nome_empresa, cnpj, ie, periodo)
            total_debito = self.relat_debito(
                cr, uid, ids, datas, listagem, valores['saida'], context)
            total_credito = self.relat_credito(
                cr, uid, ids, datas, listagem, valores['entrada'], context)
            valor_trasport = self.relat_saldo(
                cr, uid, ids, datas, listagem, valores, total_debito, total_credito, context={})
            self.relat_complementares(cr, uid, ids, datas, listagem, valores, periodo, context)
            listagem.finalizaImpressao()
            self.db_name.get('fiscal.apuracao.imposto').write(
                cr, uid, datas['apuracao_icms_id'], {'saldo_credor': valor_trasport}, context)

            # Cria o registro do proximo mes/ano
            ano = datas['periodo_final'][:4]
            mes = int(datas['periodo_final'][5:7])
            tipo = datas['tipo']
            apuracao = '%s-%s-01' % (ano, mes)

            if not self.db_name.get('fiscal.apuracao.imposto').search(
                cr, uid, [('mes', '=', mes), ('ano', '=', ano), ('tipo', '=', tipo)], 
                    context=context):
                self.db_name.get('fiscal.apuracao.imposto').create(
                    cr, uid, dict(tipo=tipo, ano=ano, mes=mes, data_apuracao=apuracao,
                                  saldo_credor=0.0), context=context)

            self.obj = external_pdf(pdf_file.getvalue())
            self.obj.render()
            pdf_file.close()

            return (self.obj.pdf, 'pdf')

    def relat_entrada(self, cr, uid, ids, datas, listagem, nome_empresa, cnpj, ie, periodo,
                      company_id, simples_nacional, context=None):
        context = context and context.copy() or {}

        listagem.informaCab('*' + '-' * 146 + '*')
        listagem.informaCab(_('|                                             RESUMO DA APURACAO DO '
                              'IMPOSTO                                                             '
                              '           |'))
        listagem.informaCab(
            _('|FIRMA: %s|') % nome_empresa.ljust(139))
        listagem.informaCab(_('|                                                                   '
                              '                                                                    '
                              '           |'))
        listagem.informaCab(_('|INSCR.ESTADUAL: ') + ie + _('               CNPJ(MF): ') + cnpj + 
                            '|')
        listagem.informaCab(
            _('|FOLHA: {{ Page }}                 MES OU PERIODO/ANO: ') + periodo + 
            '                                                                                 |')
        listagem.informaCab(_('|                                                                   '
                              '                                                                    '
                              '           |'))
        listagem.informaCab(_('|------------------------------------------------------------- ENTRA'
                              'DAS ----------------------------------------------------------------'
                              '-----------|'))
        listagem.informaCab(_('|     CODIFICACAO   |               |                               '
                              '   ICMS - VALORES FISCAIS                                           '
                              '           |'))
        listagem.informaCab(_('|-------------------|    VALORES    |-------------------------------'
                              '--------------------------------------------------------------------'
                              '-----------|'))
        listagem.informaCab(_('|                   |               |            OPERACOES COM CREDI'
                              'TO DO IMPOSTO         |         OPERACOES SEM CREDITO DO IMPOSTO    '
                              '           |'))
        listagem.informaCab(_('| CONTABIL | FISCAL |   CONTABEIS   |-------------------------------'
                              '--------------------------------------------------------------------'
                              '-----------|'))
        listagem.informaCab(_('|          |        |               |      BASE DE CALCULO     |    '
                              ' IMPOSTO CREDITADO    |  ISENTOS OU NAO TRIBUTADAS   |          OUTR'
                              'AS         |'))
        listagem.informaCab(_('|' + '-' * 146 + '|'))
        sql_modelos_ids = """
        SELECT id
        FROM   fiscal_modelo_nf
        WHERE  fismnf_codigo LIKE '9%'
        """
        cr.execute(sql_modelos_ids)
        dados_modelos = cr.dictfetchall()
        modelos_ids = map(lambda x: x.get('id'), dados_modelos)
        if not modelos_ids:
            modelos_ids = [-1]
        modelos_ids = str(modelos_ids).replace('[','(').replace(']',')')
        inicio, fim = datas['periodo_inicial'], datas['periodo_final']
        sql = """
        SELECT codigo
             , SUM(vr_contab)                                       AS vr_contab
             , SUM(vr_frete_rateado)                                AS vr_frete_rateado
             , SUM(vr_icms_st_rateado)                              AS vr_icms_st_rateado
             , SUM(vr_base)                                         AS vr_base
             , SUM(vr_reducao_base)                                 AS vr_reducao_base
             , SUM(vr_seguro_rateado)                               AS vr_seguro_rateado
             , SUM(vr_desconto_rateado)                             AS vr_desconto_rateado
             , SUM(vr_despesas_rateado)                             AS vr_despesas_rateado
             , SUM(ImpCred)                                         AS ImpCred
             , SUM(ImpIsen)                                         AS ImpIsen
             , SUM(ImpOut)                                          AS ImpOut
        FROM(SELECT cf.codigo
                  , SUM(COALESCE(it.vr_total_item_rateado, 0)  +
                        COALESCE(it_tr.vr_icms_substituicao, 0) +
                        COALESCE(it_tr.fisnit_vr_fcp_st, 0))        AS vr_contab
                  , SUM(COALESCE (it.vr_frete_rateado, 0))          AS vr_frete_rateado
                  , SUM(COALESCE (it_tr.vr_icms_substituicao, 0))   AS vr_icms_st_rateado
                  , SUM(CASE 
                            WHEN nf.nf_propria IS FALSE
                              AND destinatario.admpes_jur_regime_tributario = 'S'
                              AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                              AND it_cst.tipo = 'T'
                              AND it_tr.vr_encargo > 0 
                              THEN 0
                             WHEN it_cst.tipo = 'T' AND NOT %(simples_nacional)s
                                THEN it_tr.vr_base_calculo
                             ELSE 0 END)                            AS vr_base
                  , SUM(COALESCE (it_tr.vr_reducao_base, 0))        AS vr_reducao_base
                  , SUM(COALESCE (it.vr_seguro_rateado, 0))         AS vr_seguro_rateado
                  , Sum(COALESCE (it.vr_desconto_rateado, 0))       AS vr_desconto_rateado
                  , SUM(COALESCE (it.vr_despesas_rateado, 0))       AS vr_despesas_rateado
                  , SUM(CASE WHEN nf.nf_propria IS FALSE
                              AND destinatario.admpes_jur_regime_tributario = 'S'
                              AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                              AND it_cst.tipo = 'T'
                              AND it_tr.vr_encargo > 0 THEN 0
                             WHEN it_cst.tipo = 'T' AND NOT %(simples_nacional)s
                             THEN COALESCE (it_tr.vr_encargo, 0) 
                             ELSE 0 END)                            AS ImpCred
                  , SUM(CASE WHEN it_cst.tipo = 'I' 
                             THEN COALESCE (it_tr.vr_base_calculo, 0) 
                             ELSE 0 END)                            AS ImpIsen
                  , SUM(CASE WHEN nf.nf_propria IS FALSE
                              AND destinatario.admpes_jur_regime_tributario = 'S'
                              AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                              AND it_cst.tipo = 'T'
                              AND it_tr.vr_encargo > 0 THEN COALESCE (it_tr.vr_base_calculo, 0)
                             WHEN it_cst.tipo = 'T' AND %(simples_nacional)s
                                 THEN COALESCE (it_tr.vr_base_calculo, 0)
                             WHEN it_cst.tipo IN ('O', 'D') THEN COALESCE (it_tr.vr_base_calculo, 0)
                             WHEN it_cst.tipo IN ('U', 'TU') THEN COALESCE (it_tr.vr_base_calculo, 0) +
                                                                  COALESCE (it_tr.vr_icms_substituicao, 0) +
                                                                  COALESCE (it_tr.fisnit_vr_fcp_st, 0)
                             ELSE 0 END)                            AS ImpOut
             FROM fiscal_nf_capa                                    AS nf
             INNER JOIN estoque_movimento_itens                     AS it
                     ON it.nf_id = nf.id
             INNER JOIN fiscal_nf_item_tributacao                   AS it_tr
                     ON it_tr.movimento_itens_id = it.id
             INNER JOIN fiscal_cst                                  AS it_cst
                     ON it_cst.id = it_tr.cst_id
             INNER JOIN fiscal_cfop                                 AS cf
                     ON cf.id = it.cfop_id
             INNER JOIN fiscal_encargo                              AS enc
                     ON enc.id = it_tr.encargo_id
             LEFT  JOIN adm_pessoa                                  AS destinatario
                     ON nf.destinatario_id = destinatario.id
             LEFT  JOIN res_company                                 AS empresa
                     ON empresa.id = nf.company_id
             LEFT  JOIN adm_pessoa                                  AS pessoa_empresa
                     ON empresa.rescon_pessoa_id = pessoa_empresa.id
             WHERE it.nf_id = nf.id
               AND nf.company_id = %(company_id)s
               AND it.id = it_tr.movimento_itens_id
               AND it_tr.cst_id = it_cst.id
               AND it_cst.encargo_id = enc.id
               AND enc.sigla = 'ICMS'
               AND it.cfop_id = cf.id
               AND nf.tipo = 'E'
               AND nf.state = '3'
               AND nf.modelo_nf_id NOT IN %(modelos_ids)s
               AND nf.data_saida_entrada AT TIME ZONE 'UTC' >= '%(inicio)s 00:00:00'
               AND nf.data_saida_entrada AT TIME ZONE 'UTC' < '%(fim)s 23:59:59'
               AND cf.produto_servico_tipo != 'F' -- Para nao duplicar movimento de Frete lancado depois da chegada da nota
             GROUP BY cf.codigo
             UNION ALL
             SELECT cf.codigo
                  , SUM(CASE WHEN enc.sigla = 'IPI' THEN it_tr.vr_encargo
                        ELSE 0 END)                                 AS vr_contab
                  , 0                                               AS vr_frete_rateado
                  , 0                                               AS vr_icms_st_rateado
                  , 0                                               AS vr_base
                  , 0                                               AS vr_reducao_base
                  , 0                                               AS vr_seguro_rateado
                  , 0                                               AS vr_desconto_rateado
                  , 0                                               AS vr_despesas_rateado
                  , 0                                               AS ImpCred
                  , 0                                               AS ImpIsen
                  , SUM(CASE WHEN enc.sigla = 'IPI' THEN it_tr.vr_encargo
                        ELSE 0 END)                                 AS ImpOut
             FROM fiscal_nf_capa                                    AS nf
             INNER JOIN estoque_movimento_itens                     AS it
                     ON it.nf_id = nf.id
             INNER JOIN fiscal_nf_item_tributacao                   AS it_tr
                     ON it_tr.movimento_itens_id = it.id
             INNER JOIN fiscal_cst                                  AS it_cst
                     ON it_cst.id = it_tr.cst_id
             INNER JOIN fiscal_cfop                                 AS cf
                     ON cf.id = it.cfop_id
             INNER JOIN fiscal_encargo                              AS enc
                     ON enc.id = it_tr.encargo_id
             WHERE it.nf_id = nf.id
               AND nf.company_id = %(company_id)s
               AND it.id = it_tr.movimento_itens_id
               AND it_tr.cst_id = it_cst.id
               AND it_cst.encargo_id = enc.id
               AND enc.sigla = 'IPI'
               AND it.cfop_id = cf.id
               AND nf.tipo = 'E'
               AND nf.state = '3'
               AND nf.modelo_nf_id NOT IN %(modelos_ids)s
               AND nf.data_saida_entrada AT TIME ZONE 'UTC' >= '%(inicio)s 00:00:00'
               AND nf.data_saida_entrada AT TIME ZONE 'UTC' <= '%(fim)s 23:59:59'
             GROUP BY cf.codigo
             ORDER BY codigo)                                       AS TMP
        GROUP BY codigo"""
        sql_frete = """
        UNION ALL
        SELECT cfop.codigo
             , SUM(COALESCE(nf_capa.vr_total_nf, 0))                AS vr_contab
             , 0                                                    AS vr_frete_rateado
             , SUM(COALESCE(tributacao.vr_icms_substituicao, 0))    AS vr_icms_st_rateado
             , SUM(CASE 
                    WHEN nf_capa.nf_propria IS FALSE
                    AND destinatario.admpes_jur_regime_tributario = 'S'
                    AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                    AND tributacao.tipo = 'T'
                    AND tributacao.vr_encargo > 0 
                        THEN 0
                    WHEN tributacao.tipo = 'T' AND NOT %(simples_nacional)s 
                        THEN COALESCE(tributacao.vr_base_calculo, 0)
                   ELSE 0 END)                                      AS vr_base
             , SUM(COALESCE (tributacao.vr_reducao_base, 0))        AS vr_reducao_base
             , 0                                                    AS vr_seguro_rateado
             , 0                                                    AS vr_desconto_rateado
             , 0                                                    AS vr_despesas_rateado
             , SUM(CASE WHEN nf_capa.nf_propria IS FALSE
                    AND destinatario.admpes_jur_regime_tributario = 'S'
                    AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                    AND tributacao.tipo = 'T'
                    AND tributacao.vr_encargo > 0 THEN 0
                   WHEN tributacao.tipo = 'T' THEN COALESCE(tributacao.vr_encargo, 0)
                   ELSE 0 END)                                      AS ImpCred
             , SUM(CASE WHEN tributacao.tipo = 'I' THEN COALESCE(tributacao.vr_base_calculo, 0)
                   ELSE 0 END)                                      AS ImpIsen
             , SUM(CASE WHEN nf_capa.nf_propria IS FALSE
                            AND destinatario.admpes_jur_regime_tributario = 'S'
                            AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                            AND tributacao.tipo = 'T'
                            AND tributacao.vr_encargo > 0 
                            THEN COALESCE (tributacao.vr_base_calculo, 0)
                        WHEN tributacao.tipo = 'T' AND %(simples_nacional)s
                            THEN COALESCE (tributacao.vr_base_calculo, 0)
                        WHEN tributacao.tipo IN ('U', 'O') THEN COALESCE(tributacao.vr_base_calculo, 0) +
                                                                COALESCE(tributacao.vr_icms_substituicao, 0)
                        WHEN tributacao.tipo = 'TU' THEN COALESCE(tributacao.vr_base_calculo, 0)
                        ELSE 0 END)                                 AS ImpOut
        FROM fiscal_nf_capa nf_capa
        INNER JOIN fiscal_cfop cfop
                ON cfop.id = (SELECT cfop_id
                              FROM   fiscal_cfop_natureza
                              WHERE  id = nf_capa.natureza_id)
        INNER JOIN fiscal_tributacao_agrupar_pgview                 AS tributacao
                ON tributacao.nf_id = nf_capa.id
        INNER JOIN fiscal_encargo                                   AS encargo
                ON encargo.id = tributacao.encargo_id
        LEFT  JOIN adm_pessoa                                       AS destinatario
                ON nf_capa.destinatario_id = destinatario.id
        LEFT  JOIN res_company                                      AS empresa
                ON empresa.id = nf_capa.company_id
        LEFT  JOIN adm_pessoa                                       AS pessoa_empresa
                ON empresa.rescon_pessoa_id = pessoa_empresa.id
        WHERE cfop.produto_servico_tipo = 'F'
          AND nf_capa.state = '3'
          AND nf_capa.data_saida_entrada AT TIME ZONE 'UTC' >= '%(inicio)s 00:00:00'
          AND nf_capa.data_saida_entrada AT TIME ZONE 'UTC' <= '%(fim)s 23:59:59'
          AND encargo.sigla = 'ICMS'
          AND nf_capa.company_id = %(company_id)s
        GROUP BY cfop.codigo
        """
        sql += sql_frete
        cr.execute(sql % locals())
        datas['movimento'] = cr.dictfetchall()

        # Valor total para CFOP começado com 1
        cfop1_contab = cfop1_base = cfop1_impcred = cfop1_isen = cfop1_outras = 0

        # Valor total para CFOP começado com 2
        cfop2_contab = cfop2_base = cfop2_impcred = cfop2_isen = cfop2_outras = 0

        # Valor total para CFOP começado com 3
        cfop3_contab = cfop3_base = cfop3_impcred = cfop3_isen = cfop3_outras = 0
        
        fispar_base_reduzida = self.db_name.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal.parametro', ['fispar_base_reduzida'], company_id=company_id,
            context=context).get('fispar_base_reduzida')
        for i in datas['movimento']:
            valor_contabil = i.get('vr_contab') or 0
            vr_base = i.get('vr_base') or 0
            contabil_total = valor_contabil
            vr_base = vr_base
            vr_isento = i['impisen'] or 0
            vr_outros = i['impout'] or 0
            base_reduzida = i['vr_reducao_base'] or 0
            if fispar_base_reduzida == 'I':
                vr_isento += base_reduzida
            else:
                vr_outros += base_reduzida
            linha = ['|', formataString('', 10, ' ', 'E', 'N', 'S'), '|',
                     formataString(i['codigo'], 8, ' ', 'E', 'N', 'S'), '|',
                     formataString('%1.2f' % contabil_total, 15, ' ', 'E', 'N', 'S'), '|',
                     formataString('%1.2f' % vr_base, 26, ' ', 'E', 'N', 'S'), '|',
                     formataString('%1.2f' % i['impcred'], 26, ' ', 'E', 'N', 'S'), '|',
                     formataString('%1.2f' % vr_isento, 30, ' ', 'E', 'N', 'S'), '|',
                     formataString('%1.2f' % vr_outros, 25, ' ', 'E', 'N', 'S'), '|']
            listagem.imprimeLinha(''.join(linha))
            if i['codigo'][0] == '1':
                cfop1_contab += contabil_total
                cfop1_base += i['vr_base']
                cfop1_impcred += i['impcred']
                cfop1_isen += vr_isento
                cfop1_outras += vr_outros
            elif i['codigo'][0] == '2':
                cfop2_contab += contabil_total
                cfop2_base += i['vr_base']
                cfop2_impcred += i['impcred']
                cfop2_isen += vr_isento
                cfop2_outras += vr_outros
            elif i['codigo'][0] == '3':
                cfop3_contab += contabil_total
                cfop3_base += i['vr_base']
                cfop3_impcred += i['impcred']
                cfop3_isen += vr_isento
                cfop3_outras += vr_outros

        # Formatacao dos valores totais do CFOP 1
        cfop1_contab = formataString('%1.2f' % cfop1_contab, 15, ' ', 'E', 'N', 'S')
        cfop1_base = formataString('%1.2f' % cfop1_base, 26, ' ', 'E', 'N', 'S')
        cfop1_impcred = formataString('%1.2f' % cfop1_impcred, 26, ' ', 'E', 'N', 'S')
        cfop1_isen = formataString('%1.2f' % cfop1_isen, 30, ' ', 'E', 'N', 'S')
        cfop1_outras = formataString('%1.2f' % cfop1_outras, 25, ' ', 'E', 'N', 'S')
        
        # Formatacao dos valores totais do CFOP 2
        cfop2_contab = formataString('%1.2f' % cfop2_contab, 15, ' ', 'E', 'N', 'S')
        cfop2_base = formataString('%1.2f' % cfop2_base, 26, ' ', 'E', 'N', 'S')
        cfop2_impcred = formataString('%1.2f' % cfop2_impcred, 26, ' ', 'E', 'N', 'S')
        cfop2_isen = formataString('%1.2f' % cfop2_isen, 30, ' ', 'E', 'N', 'S')
        cfop2_outras = formataString('%1.2f' % cfop2_outras, 25, ' ', 'E', 'N', 'S')
        
        # Formatação dos valores totais do CFOP 2
        cfop3_contab = formataString('%1.2f' % cfop3_contab, 15, ' ', 'E', 'N', 'S')
        cfop3_base = formataString('%1.2f' % cfop3_base, 26, ' ', 'E', 'N', 'S')
        cfop3_impcred = formataString('%1.2f' % cfop3_impcred, 26, ' ', 'E', 'N', 'S')
        cfop3_isen = formataString('%1.2f' % cfop3_isen, 30, ' ', 'E', 'N', 'S')
        cfop3_outras = formataString('%1.2f' % cfop3_outras, 25, ' ', 'E', 'N', 'S')

        total_contab = '%1.2f' % (float(cfop1_contab) + float(cfop2_contab) + float(cfop3_contab))
        total_base = '%1.2f' % (float(cfop1_base) + float(cfop2_base) + float(cfop3_base))
        total_impcred = '%1.2f' % (float(cfop1_impcred) + float(cfop2_impcred) +
                                   float(cfop3_impcred))
        total_isen = '%1.2f' % (float(cfop1_isen) + float(cfop2_isen) + float(cfop3_isen))
        total_outras = '%1.2f' % (float(cfop1_outras) + float(cfop2_outras) + float(cfop3_outras))

        # Formatacao dos valores totais de todos CFOP
        total_contab = formataString(total_contab, 15, ' ', 'E', 'N', 'S')
        total_base = formataString(total_base, 26, ' ', 'E', 'N', 'S')
        total_impcred = formataString(total_impcred, 26, ' ', 'E', 'N', 'S')
        total_isen = formataString(total_isen, 30, ' ', 'E', 'N', 'S')
        total_outras = formataString(total_outras, 25, ' ', 'E', 'N', 'S')

        listagem.imprimeLinha('|          |        |               |                          |'
                              '                          |                              |      '
                              '                   |')
        listagem.imprimeLinha(
            _('|SUBTOTAIS ENTRADAS |               |                          |                '
              '          |                              |                         |'))
        listagem.imprimeLinha('|                   |               |                          |'
                              '                          |                              |      '
                              '                   |')
        listagem.imprimeLinha(_('| 1.00 DO ESTADO    |') + cfop1_contab + '|' + cfop1_base + 
                              '|' + cfop1_impcred + '|' + cfop1_isen + '|' + cfop1_outras + '|')
        listagem.imprimeLinha('|                   |               |                          |'
                              '                          |                              |      '
                              '                   |')
        listagem.imprimeLinha(_('| 2.00 DE OUTROS    |' + cfop2_contab + '|' + cfop2_base + '|' + 
                                cfop2_impcred + '|' + cfop2_isen + '|' + cfop2_outras + '|'))
        listagem.imprimeLinha(_('|     ESTADOS       |               |                          |'
                                '                          |                              |      '
                                '                   |'))
        listagem.imprimeLinha('|                   |               |                          |'
                              '                          |                              |      '
                              '                   |')
        listagem.imprimeLinha(_('| 3.00 DO EXTERIOR  |' + cfop3_contab + '|' + cfop3_base + '|' + 
                                cfop3_impcred + '|' + cfop3_isen + '|' + cfop3_outras + '|'))
        listagem.imprimeLinha('|                   |               |                          |'
                              '                          |                              |      '
                              '                   |')
        listagem.imprimeLinha('|                   |               |                          |'
                              '                          |                              |      '
                              '                   |')
        listagem.imprimeLinha(_('|     TOTAIS        |') + total_contab + '|' + total_base + '|' + 
                              total_impcred + '|' + total_isen + '|' + total_outras + '|')
        listagem.imprimeLinha('*' + '-' * 146 + '*')

        return {'total_contab': float(total_contab), 'total_base': float(total_base),
                'total_imp_cred_deb': float(total_impcred),
                'total_isen': float(total_isen), 'total_outras': float(total_outras)}

    def relat_saida(self, cr, uid, ids, datas, listagem, nome_empresa, cnpj, ie, periodo,
                    company_id, simples_nacional, context=None):
        """
        Relatorio de apuracao de icms de saida
        """
        context = context and context.copy() or {}
        line_end = '*' + '-' * 146 + '*'
        line_separator = '|' + ' ' * 146 + '|'
        listagem.cabecalhos = []
        listagem.informaCab(line_end)
        listagem.informaCab('|' + _('RESUMO DA APURACAO DO IMPOSTO').center(146) + '|')
        listagem.informaCab(_('|FIRMA: ') + nome_empresa.ljust(139) + '|')
        listagem.informaCab(line_separator)
        listagem.informaCab(_('|INSCR. ESTADUAL: ') + ie + ' ' * 14 + _('CNPJ(MF): ') + cnpj + '|')
        listagem.informaCab(_('|FOLHA: {{ Page }}                 MES OU PERIODO/ANO: ') +
                            periodo + ' ' * 82 + '|')
        listagem.informaCab(line_separator)
        listagem.informaCab('|' + _(' SAIDAS ').center(146, '-') + '|')
        listagem.informaCab(_('|     CODIFICACAO   |               |                               '
                              '   ICMS - VALORES FISCAIS                                           '
                              '           |'))
        listagem.informaCab(_('|-------------------|    VALORES    |-------------------------------'
                              '--------------------------------------------------------------------'
                              '-----------|'))
        listagem.informaCab(_('|          |        |               |            OPERACOES COM DEBIT'
                              'O DO IMPOSTO          |         OPERACOES SEM DEBITO DO IMPOSTO     '
                              '           |'))
        listagem.informaCab(_('| CONTABIL | FISCAL |   CONTABEIS   |-------------------------------'
                              '--------------------------------------------------------------------'
                              '-----------|'))
        listagem.informaCab(_('|          |        |               |      BASE DE CALCULO     |    '
                              ' IMPOSTO DEBITADO     |  ISENTOS OU NAO TRIBUTADAS   |          OUTR'
                              'AS         |'))
        listagem.informaCab(line_end)
        listagem.imprimeLinha('{{ Page }}')
        sql_modelos_ids = """
        SELECT id
        FROM   fiscal_modelo_nf
        WHERE  fismnf_codigo LIKE '9%'
        """
        cr.execute(sql_modelos_ids)
        dados_modelos = cr.dictfetchall()
        modelos_ids = map(lambda x: x.get('id'), dados_modelos)
        if not modelos_ids:
            modelos_ids = [-1]
        modelos_ids = str(modelos_ids).replace('[', '(').replace(']', ')')
        inicio, fim = datas.get('periodo_inicial'), datas.get('periodo_final')
        sql = """
        SELECT codigo
             , movimento_tipo                                             AS movimento_tipo
             , vr_contabil                                                AS vr_contab
             , base_icms                                                  AS vr_base
             , vr_icms                                                    AS ImpCred
             , isentos                                                    AS ImpIsen
             , outros                                                     AS ImpOut
             , vr_reducao_base                                            AS vr_reducao_base
             , vr_despesas_acessorias                                     AS vr_despesas_acessorias
        FROM (SELECT cfop.codigo                                          AS codigo
                   , cfop.movimento_tipo                                  AS movimento_tipo
                   , SUM(COALESCE(emi.vr_total_item_rateado, 0)  +
                         COALESCE(nf_item_trib.vr_icms_substituicao, 0) +
                         COALESCE(nf_item_trib.fisnit_vr_fcp_st, 0))      AS vr_contabil
                   , SUM(CASE WHEN cst.tipo = 'T' AND NOT %(simples_nacional)s
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0)
                         ELSE 0 END)                                      AS base_icms
                   , SUM(CASE WHEN cst.tipo = 'T' AND NOT %(simples_nacional)s
                         THEN COALESCE (nf_item_trib.vr_encargo, 0) 
                         ELSE 0 END)                                      AS vr_icms
                   , 0                                                    AS vr_ipi
                   , SUM(COALESCE(nf_item_trib.vr_reducao_base, 0))       AS vr_reducao_base
                   , SUM(CASE WHEN cst.tipo = 'I' 
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0)
                         ELSE 0 END)                                      AS isentos
                   , SUM(CASE WHEN cst.tipo = 'O' 
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0)
                         WHEN cst.tipo = 'U' 
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0) +
                              COALESCE (nf_item_trib.vr_icms_substituicao, 0)
                         WHEN cst.tipo = 'TU' 
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0)
                         WHEN cst.tipo = 'T' AND %(simples_nacional)s
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0)
                         ELSE 0 END)                                      AS outros
                   , SUM(CASE WHEN cst.tipo = 'U' 
                         THEN COALESCE (nf_item_trib.vr_base_calculo, 0)
                         ELSE 0 END)                                      AS substituicao
                   , SUM(CASE WHEN retido IS TRUE 
                         THEN COALESCE (nf_item_trib.vr_icms_substituicao, 0)
                         ELSE 0 END)                                      AS icms_retido
                   , SUM(COALESCE (emi.vr_desconto_rateado, 0))           AS valor_desconto
                   , SUM(COALESCE (emi.vr_frete_rateado, 0))              AS valor_frete
                   , SUM(COALESCE (emi.vr_seguro_rateado, 0))             AS vr_seguro
                   , SUM(COALESCE (emi.vr_despesas_rateado, 0))           AS vr_despesas_acessorias
              FROM fiscal_nf_capa                                         AS nf_capa
              LEFT JOIN adm_pessoa_endereco                               AS endereco
                     ON endereco.id = nf_capa.destinatario_endereco_id
              LEFT JOIN adm_uf                                            AS uf
                     ON uf.id = endereco.uf_id
              LEFT JOIN estoque_movimento_itens                           AS emi
                     ON emi.nf_id = nf_capa.id
              LEFT JOIN fiscal_nf_item_tributacao                         AS nf_item_trib
                     ON nf_item_trib.movimento_itens_id = emi.id
              LEFT JOIN fiscal_encargo                                    AS fe
                     ON fe.id = nf_item_trib.encargo_id
              LEFT JOIN fiscal_cst                                        AS cst
                     ON cst.id = nf_item_trib.cst_id
              LEFT JOIN fiscal_cfop                                       AS cfop
                     ON cfop.id = emi.cfop_id
              WHERE  nf_capa.tipo = 'S'
                AND  nf_capa.state = '3'
                AND  nf_capa.modelo_nf_id NOT IN %(modelos_ids)s
                AND nf_capa.data_emissao AT TIME ZONE 'UTC' >= '%(inicio)s 00:00:00'
                AND nf_capa.data_emissao AT TIME ZONE 'UTC' <= '%(fim)s 23:59:59'
                AND  nf_capa.company_id = %(company_id)s
                AND  fe.sigla = 'ICMS'
                AND  cfop.codigo NOT IN ('5929', '6929')
              GROUP  BY cfop.codigo, cfop.movimento_tipo
              UNION  ALL
              SELECT cfop.codigo                                          AS codigo
                   , cfop.movimento_tipo                                  AS movimento_tipo
                   , 0                                                    AS vr_contabil
                   , 0                                                    AS base_icms
                   , 0                                                    AS vr_icms
                   , SUM(CASE WHEN fe.sigla = 'IPI' 
                         THEN nf_item_trib.vr_encargo
                         ELSE 0 END)                                      AS vr_ipi
                   , 0                                                    AS isentos
                   , SUM(CASE WHEN fe.sigla = 'IPI' 
                         THEN nf_item_trib.vr_encargo
                         ELSE 0 END)                                      AS outros
                   , 0                                                    AS vr_reducao_base
                   , 0                                                    AS substituicao
                   , 0                                                    AS icms_retido
                   , 0                                                    AS valor_desconto
                   , 0                                                    AS valor_frete
                   , 0                                                    AS vr_seguro
                   , 0                                                    AS vr_despesas_acessorias
              FROM fiscal_nf_capa                                         AS nf_capa
              INNER JOIN estoque_movimento_itens                          AS emi
                      ON emi.nf_id = nf_capa.id
              INNER JOIN fiscal_nf_item_tributacao                        AS nf_item_trib
                      ON nf_item_trib.movimento_itens_id = emi.id
              INNER JOIN fiscal_encargo                                   AS fe
                      ON fe.id = nf_item_trib.encargo_id
              INNER JOIN fiscal_cfop                                      AS cfop
                      ON cfop.id = emi.cfop_id
              WHERE nf_capa.tipo = 'S'
                AND nf_capa.state = '3'
                AND nf_capa.modelo_nf_id NOT IN %(modelos_ids)s
                AND nf_capa.data_emissao AT TIME ZONE 'UTC' >= '%(inicio)s 00:00:00'
                AND nf_capa.data_emissao AT TIME ZONE 'UTC' <= '%(fim)s 23:59:59'
                AND nf_capa.company_id = %(company_id)s
                AND fe.sigla = 'IPI'
                AND cfop.codigo NOT IN ('5929', '6929')
              GROUP BY cfop.codigo, cfop.movimento_tipo)                  AS TMP
        """
        ecf_instalado = self.db_name.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'ecf', context)
        if ecf_instalado:
            sql += """
        UNION ALL
        SELECT CASE WHEN mapa_itens.tipo = 'F'
               THEN '5405' :: text
               ELSE '5102' :: text END                                    AS codigo
             , 'C'                                                        AS movimento_tipo
             , SUM(COALESCE(mapa_itens.base_valor, 0))                    AS vr_contab
             , SUM(CASE WHEN mapa_itens.tipo = 'T' AND NOT %(simples_nacional)s
                   THEN COALESCE(mapa_itens.base_valor, 0)
                   ELSE 0 END)                                            AS vr_base
             , SUM(CASE WHEN mapa_itens.tipo = 'T' AND NOT %(simples_nacional)s
               THEN COALESCE(mapa_itens.imposto_valor, 0)
               ELSE 0 END)                                                AS ImpCred
             , SUM(CASE WHEN mapa_itens.tipo = 'I'
               THEN COALESCE(mapa_itens.base_valor, 0)
               ELSE 0 END)                                                AS isentos
             , SUM(CASE WHEN mapa_itens.tipo IN ('O', 'F')
               THEN COALESCE(mapa_itens.base_valor, 0)
               WHEN mapa_itens.tipo = 'TU'
               THEN COALESCE(mapa_itens.base_valor, 0)
               WHEN mapa_itens.tipo = 'T' AND %(simples_nacional)s
               THEN COALESCE(mapa_itens.base_valor, 0) 
               ELSE 0 END)                                                AS ImpOut
             , SUM(COALESCE(mapa_itens.base_reduzida, 0))                 AS vr_reducao_base
             , 0                                                          AS vr_despesas_acessorias
        FROM  ecf_mapa_resumo_capa                                        AS mapa_capa
        INNER JOIN ecf_ecf                                                AS ecf
                ON ecf.id = mapa_capa.ecf
        INNER JOIN ecf_mapa_resumo_itens                                  AS mapa_itens
                ON mapa_itens.capa_id = mapa_capa.id
               AND mapa_capa.data >= '%(inicio)s'
               AND mapa_capa.data <= '%(fim)s'
        WHERE ecf.company_id = %(company_id)s
        GROUP BY codigo
        """
        sql = sql % locals()
        sql_final = """
        SELECT codigo                      
             , movimento_tipo                                             AS movimento_tipo
             , SUM(vr_contab)                                             AS vr_contab
             , SUM(vr_base)                                               AS vr_base
             , SUM(ImpCred)                                               AS ImpCred
             , SUM(ImpIsen)                                               AS ImpIsen
             , SUM(ImpOut)                                                AS ImpOut
             , SUM(vr_reducao_base)                                       AS vr_reducao_base
             , SUM(vr_despesas_acessorias)                                AS vr_despesas_acessorias
        FROM (%(sql)s)                                                    AS TMP
        GROUP BY codigo, movimento_tipo
        ORDER BY codigo
        """
        cr.execute(sql_final % locals())
        datas['movimento'] = cr.dictfetchall()

        # Zera os valores
        cfop5_contab = cfop5_base = cfop5_impcred = cfop5_isen = cfop5_outras = 0
        cfop6_contab = cfop6_base = cfop6_impcred = cfop6_isen = cfop6_outras = 0
        cfop7_contab = cfop7_base = cfop7_impcred = cfop7_isen = cfop7_outras = 0

        line_separator = '|          |        |               |                          |      ' \
                         '                    |                              |                  ' \
                         '       |'
        fispar_base_reduzida = self.db_name.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal.parametro', ['fispar_base_reduzida'], company_id=company_id,
            context=context).get('fispar_base_reduzida')
        for i in datas['movimento']:
            # Determina o tipo de movimento em funcao do CFOP
            vr_isento = i['impisen'] or 0
            vr_outros = i['impout']
            base_reduzida = i.get('vr_reducao_base')
            if fispar_base_reduzida == 'I':
                vr_isento += base_reduzida
            else:
                vr_outros += base_reduzida

            linha = ['|%s|' % formataString('', 10, ' ', 'E', 'N', 'S')]
            linha.append('%s|' % formataString(i['codigo'], 8, ' ', 'E', 'N', 'S'))
            linha.append('%s|' % formataString('%1.2f' % i['vr_contab'], 15, ' ', 'E', 'N', 'S'))
            linha.append('%s|' % formataString('%1.2f' % i['vr_base'], 26, ' ', 'E', 'N', 'S'))
            linha.append('%s|' % formataString('%1.2f' % i['impcred'], 26, ' ', 'E', 'N', 'S'))
            linha.append('%s|' % formataString('%1.2f' % vr_isento, 30, ' ', 'E', 'N', 'S'))
            linha.append('%s|' % formataString('%1.2f' % vr_outros, 25, ' ', 'E', 'N', 'S'))
            listagem.imprimeLinha(''.join(linha))
            if i['codigo'][0] == '5':
                cfop5_contab += i['vr_contab']
                cfop5_base += i['vr_base']
                cfop5_impcred += i['impcred']
                cfop5_isen += vr_isento
                cfop5_outras += vr_outros
            elif i['codigo'][0] == '6':
                cfop6_contab += i['vr_contab']
                cfop6_base += i['vr_base']
                cfop6_impcred += i['impcred']
                cfop6_isen += vr_isento
                cfop6_outras += vr_outros
            elif i['codigo'][0] == '7':
                cfop7_contab += i['vr_contab']
                cfop7_base += i['vr_base']
                cfop7_impcred += i['impcred']
                cfop7_isen += vr_isento
                cfop7_outras += vr_outros

        # Formatacao dos valores totais de todos os CFOPs
        total_contab = ('%1.2f' % (cfop5_contab + cfop6_contab + cfop7_contab)).rjust(15)
        total_base = ('%1.2f' % (cfop5_base + cfop6_base + cfop7_base)).rjust(26)
        total_impdeb = ('%1.2f' % (cfop5_impcred + cfop6_impcred + cfop7_impcred)).rjust(26)
        total_isen = ('%1.2f' % (cfop5_isen + cfop6_isen + cfop7_isen)).rjust(30)
        total_outras = ('%1.2f' % (cfop5_outras + cfop6_outras + cfop7_outras)).rjust(25)

        # Formatacao dos valores totais do CFOP 1
        cfop5_contab = ('%1.2f' % cfop5_contab).rjust(15)
        cfop5_base = ('%1.2f' % cfop5_base).rjust(26)
        cfop5_impcred = ('%1.2f' % cfop5_impcred).rjust(26)
        cfop5_isen = ('%1.2f' % cfop5_isen).rjust(30)
        cfop5_outras = ('%1.2f' % cfop5_outras).rjust(25)

        # Formatacao dos valores totais do CFOP 2
        cfop6_contab = ('%1.2f' % cfop6_contab).rjust(15)
        cfop6_base = ('%1.2f' % cfop6_base).rjust(26)
        cfop6_impcred = ('%1.2f' % cfop6_impcred).rjust(26)
        cfop6_isen = ('%1.2f' % cfop6_isen).rjust(30)
        cfop6_outras = ('%1.2f' % cfop6_outras).rjust(25)

        # Formatacao dos valores totais do CFOP 2
        cfop7_contab = ('%1.2f' % cfop7_contab).rjust(15)
        cfop7_base = ('%1.2f' % cfop7_base).rjust(26)
        cfop7_impcred = ('%1.2f' % cfop7_impcred).rjust(26)
        cfop7_isen = ('%1.2f' % cfop7_isen).rjust(30)
        cfop7_outras = ('%1.2f' % cfop7_outras).rjust(25)

        listagem.imprimeLinha(line_separator)
        listagem.imprimeLinha(_('|SUBTOTAIS SAIDAS   |               |                          |  '
                                '                        |                              |          '
                                '               |'))
        listagem.imprimeLinha(line_separator)
        listagem.imprimeLinha(_('| 5.00 PARA O ESTADO|') + cfop5_contab + '|' + cfop5_base + '|' +
                              cfop5_impcred + '|' + cfop5_isen + '|' + cfop5_outras + '|')
        listagem.imprimeLinha(line_separator)
        listagem.imprimeLinha(_('| 6.00 PARA OUTROS  |') + cfop6_contab + '|' + cfop6_base + '|' +
                              cfop6_impcred + '|' + cfop6_isen + '|' + cfop6_outras + '|')
        listagem.imprimeLinha(_('|      ESTADOS      |               |                          |  '
                                '                        |                              |          '
                                '               |'))
        listagem.imprimeLinha(line_separator)
        listagem.imprimeLinha(_('| 7.00 PARA O       |') + cfop7_contab + '|' + cfop7_base + '|' +
                              cfop7_impcred + '|' + cfop7_isen + '|' + cfop7_outras + '|')
        listagem.imprimeLinha(_('|      EXTERIOR     |               |                          |  '
                                '                        |                              |          '
                                '               |'))
        listagem.imprimeLinha(line_separator)
        listagem.imprimeLinha(_('|     TOTAIS        |') + total_contab + '|' + total_base + '|' +
                              total_impdeb + '|' + total_isen + '|' + total_outras + '|')
        listagem.imprimeLinha(line_end)

        return {'total_contab': float(total_contab), 'total_base': float(total_base),
                'total_imp_cred_deb': float(total_impdeb),
                'total_isen': float(total_isen), 'total_outras': float(total_outras)}

    def relat_resumo_cabecalho(self, cr, uid, listagem, nome_empresa, cnpj, ie, periodo):
        listagem.cabecalhos = []
        listagem.informaCab('*' + '-' * 146 + '*')
        listagem.informaCab(_('|                                             RESUMO DA APURACAO DO '
                              'IMPOSTO                                                             '
                              '           |'))
        listagem.informaCab(
            _('|FIRMA: %s|') % nome_empresa.ljust(139))
        listagem.informaCab('|' + ' ' * 146 + '|')
        listagem.informaCab(_('|INSCR.ESTADUAL: ') + ie + _('               CNPJ(MF): ') + 
                            cnpj + '|')
        listagem.informaCab(_('|FOLHA: {{ Page }}                 MES OU PERIODO/ANO: ') + 
                            periodo + ' ' * 80 + '|')
        listagem.informaCab('|' + '-' * 146 + '|')

    def relat_debito(self, cr, uid, ids, datas, listagem, valores, context=None):
        context = context and context.copy() or {}
        somas = formataString('' + str('%1.2f' % valores['total_imp_cred_deb']), 22, ' ', 'E', 'N',
                              'S')
        vr_total = 0
        listagem.imprimeLinha('{{ Page }}')
        listagem.imprimeLinha(
            _('|                                  DEBITO DO IMPOSTO                                '
              '              |                     VALORES                    |'))
        listagem.imprimeLinha('|' + '-' * 145 + '|')
        listagem.imprimeLinha(_('|                                                                 '
                                '                                |     COLUNA AUXILIAR     |       '
                                ' SOMAS         |'))
        listagem.imprimeLinha('|                                                                   '
                              '                              |-------------------------|-----------'
                              '-----------|')
        listagem.imprimeLinha('|                                                                   '
                              '                              |                         |           '
                              '           |')
        listagem.imprimeLinha(_('| 001 - POR SAIDAS/PRESTACOES COM DEBITO DO IMPOSTO               '
                                '                                |                         |') + 
                              somas + '|')
        listagem.imprimeLinha(_('| 002 - OUTROS DEBITOS (DISCRIMINAR ABAIXO)                       '
                                '                                |                         |       '
                                '               |'))
        if datas['outros_debitos_ids']:
            vr_total += self.listar_descricao_valor(
                cr, uid, listagem, datas['outros_debitos_ids'], 
                'fiscal.apuracao.imposto.outros.debitos', context=context)
        listagem.imprimeLinha('|                                                                   '
                              '                              |                         |           '
                              '           |')
        listagem.imprimeLinha(_('| 003 - ESTORNO DE CREDITOS (DISCRIMINAR ABAIXO)                  '
                                '                                |                         |       '
                                '               |'))
        if datas['estorno_creditos_ids']:
            vr_total += self.listar_descricao_valor(
                cr, uid, listagem, datas['estorno_creditos_ids'],
                'fiscal.apuracao.imposto.estorno.creditos', context=context)
        listagem.imprimeLinha('|                                                                   '
                              '                              |                         |           '
                              '           |')
        vr_total += float(somas)
        vr_total = formataString('' + str('%1.2f' % vr_total), 22, ' ', 'E', 'N', 'S')
        listagem.imprimeLinha(_('| 004 - TOTAL                                                     '
                                '                                |                         |') + 
                              vr_total + '|')
        
        return float(vr_total)

    def relat_credito(self, cr, uid, ids, datas, listagem, valores, context=None):
        context = context and context.copy() or {}
        somas = formataString('' + str('%1.2f' % valores['total_imp_cred_deb']), 22, ' ', 'E', 'N',
                              'S')
        vr_total = 0
        listagem.imprimeLinha('|-------------------------------------------------------------------'
                              '--------------------------------------------------------------------'
                              '-----------|')
        listagem.imprimeLinha(_('|                                  CREDITO DO IMPOSTO             '
                                '                                |                         |       '
                                '               |'))
        listagem.imprimeLinha('|-------------------------------------------------------------------'
                              '--------------------------------------------------------------------'
                              '-----------|')
        listagem.imprimeLinha(_('| 005 - POR ENTRADAS/AQUISICOES COM CREDITO DO IMPOSTO            '
                                '                                |                         |') + 
                              somas + '|')
        listagem.imprimeLinha(_('| 006 - OUTROS CREDITOS (DISCRIMINAR ABAIXO)                      '
                                '                                |                         |       '
                                '               |'))
        if datas['outros_creditos_ids']:
            vr_total += self.listar_descricao_valor(
                cr, uid, listagem, datas['outros_creditos_ids'],
                'fiscal.apuracao.imposto.outros.creditos', context=context)
        listagem.imprimeLinha('|                                                                   '
                              '                              |                         |           '
                              '           |')
        listagem.imprimeLinha(_('| 007 - ESTORNO DE DEBITOS (DISCRIMINAR ABAIXO)                   '
                                '                                |                         |       '
                                '               |'))
        if datas['estorno_debitos_ids']:
            vr_total += self.listar_descricao_valor(
                cr, uid, listagem, datas['estorno_debitos_ids'],
                'fiscal.apuracao.imposto.estorno.debitos', context=context)
        vr_total += float(somas)
        listagem.imprimeLinha(_('|                                                                 '
                                '                                |                         |       '
                                '               |'))
        svr_total = formataString('' + str('%1.2f' % vr_total), 22, ' ', 'E', 'N', 'S')
        listagem.imprimeLinha(_('| 008 - SUBTOTAL                                                  '
                                '                                |                         |') + 
                              svr_total + '|')
        svr_total = formataString('%1.2f' % datas['saldo_periodo_anterior'], 22, ' ', 'E', 'N', 'S')
        listagem.imprimeLinha(_('| 009 - SALDO CREDOR DO PERIODO ANTERIOR                          '
                                '                                |                         |') + 
                              svr_total + '|')
        vr_total += float(datas['saldo_periodo_anterior'])
        svr_total = formataString('' + str('%1.2f' % vr_total), 22, ' ', 'E', 'N', 'S')
        listagem.imprimeLinha(_('| 010 - TOTAL                                                     '
                                '                                |                         |') +
                              svr_total + '|')
        
        return float(vr_total)

    def relat_saldo(self, cr, uid, ids, datas, listagem, valores, total_debito, total_credito,
                    context=None):
        context = context and context.copy() or {}
        total_deducoes = self.total_deducoes(cr, uid, listagem, datas['deducoes_ids'],
                                             context=context)
        vr_total_debito = vr_total_credito = 0
        listagem.imprimeLinha('|' + '-' * 146 + '|')
        listagem.imprimeLinha(_('|                                  APURACAO DE SALDO              '
                                '                                |                         |       '
                                '               |'))
        listagem.imprimeLinha('|' + '-' * 146 + '|')
        if total_debito > total_credito:
            vr_total_debito = total_debito - total_credito - total_deducoes
        vr_total_debito = formataString('' + str('%1.2f' % vr_total_debito), 22, ' ', 'E', 'N', 'S')
        listagem.imprimeLinha(_('| 011 - SALDO DEVEDOR (DEBITO MENOS CREDITO)                      '
                                '                                |                         |') + 
                              vr_total_debito + '|')
        listagem.imprimeLinha(_('| 012 - DEDUCOES (DISCRIMINAR ABAIXO)                             '
                                '                                |                         |       '
                                '               |'))
        if datas['deducoes_ids']:
            self.listar_descricao_valor(
                cr, uid, listagem, datas['deducoes_ids'], 'fiscal.apuracao.imposto.deducoes', 
                context=context)
        listagem.imprimeLinha('|                                                                   '
                              '                              |                         |           '
                              '           |')
        listagem.imprimeLinha(_('| 013 - IMPOSTO A RECOLHER                                        '
                                '                                |                         |       '
                                '               |'))
        if total_debito < total_credito:
            vr_total_credito = total_credito - total_debito + total_deducoes
        vr_total_credito = formataString(
            '' + str('%1.2f' % vr_total_credito), 22, ' ', 'E', 'N', 'S')
        listagem.imprimeLinha(_('| 014 - SALDO CREDOR (CREDITO MENOS DEBITO)                       '
                                '                                |                         |') + 
                              vr_total_credito + _('|'))
        listagem.imprimeLinha(_('|       A TRANSPORTAR PARA O PERIODO SEGUINTE                     '
                                '                                                                  '
                                '               |'))
        listagem.imprimeLinha('|' + '-' * 146 + '|')

        return vr_total_credito

    def relat_complementares(self, cr, uid, ids, datas, listagem, valores, periodo, context=None):
        context = context and context.copy() or {}
        listagem.imprimeLinha(_('|                                  INFORMACOES COMPLEMENTARES     '
                                '                                                                  '
                                '               |'))
        listagem.imprimeLinha('|' + '-' * 146 + '|')
        listagem.imprimeLinha('|' + ' ' * 146 + '|')
        listagem.imprimeLinha(_('|               GUIAS DE RECOLHIMENTO(DAE)                        '
                                '                             GUIA DE INFORMACAO (DAPI)            '
                                '               |'))
        listagem.imprimeLinha('|' + ' ' * 146 + '|')
        listagem.imprimeLinha(_('|            NUMERO                 DATA         VALOR            '
                                ' ORGAO ARRECADADOR        DATA DA ENTREGA     LOCAL DA ENTREGA (BA'
                                'NCO/REPARTICAO)|'))
        listagem.imprimeLinha('|' + ' ' * 146 + '|')
        mes = periodo[0:2]
        ano = periodo[3:7]
        data_inicial = ano + "-" + mes + "-" + "01"
        if mes == '12':
            mes = 0
        mes = int(mes) + 1
        data_final = ano + "-" + str(mes) + "-" + "01"
        obj_fiscal_guia = self.db_name.get('fiscal.guia')
        guia_ids = obj_fiscal_guia.search(
            cr, uid, [('data_vencimento', '>=', data_inicial), 
                      ('data_vencimento', '<', data_final)], context=context)
        if guia_ids:
            guia_dados = obj_fiscal_guia.read(cr, uid, guia_ids, context=context)
            for i in guia_dados:
                numero = formataString('' + i['convenio'], 30, ' ', 'C', 'N', 'S')
                data_vencimento = formataString(
                    '' + datetime.strptime(i['data_vencimento'], "%Y-%m-%d").__format__("%d-%m-%Y"),
                    12, ' ', 'C', 'N',
                    'S')
                valor = formataString('' + getNumeroEditado(i['valor'], 2), 13, ' ', 'C', 'N', 'S')
                orgao_arrecadador = ""
                if i.get('pessoa_id', False):
                    orgao_arrecadador = i['pessoa_id'][1]
                orgao_arrecadador = formataString('' + orgao_arrecadador, 31, ' ', 'C', 'N', 'S')
                data_entrega = ""
                if i.get('data_pagamento', False):
                    data_entrega = datetime.strptime(
                        i['data_pagamento'], "%Y-%m-%d").__format__("%d-%m-%Y")
                data_entrega = formataString('' + data_entrega, 17, ' ', 'C', 'N', 'S')
                local_entrega = formataString('' + i['local_cobranca_id'][1], 38, ' ', 'C', 'N',
                                              'S')
                listagem.imprimeLinha('|' + numero + ' ' + data_vencimento + ' ' + valor + ' ' + 
                                      orgao_arrecadador + ' ' + data_entrega + ' ' + 
                                      local_entrega + '|')
        listagem.imprimeLinha('|' + ' ' * 146 + '|')
        listagem.imprimeLinha(_('|               GUIA NACIONAL DE ARRECADACAO (GNRE)               '
                                '                                                                  '
                                '               |'))
        listagem.imprimeLinha(_('|      Fornecedor              Docum.    Data pagto       Valor   '
                                ' bco   |      Fornecedor              Docum.    Data pagto       V'
                                'alor    bco    |'))
        if guia_ids:
            i = 0
            linha_imprimi = ''
            browse_fiscal_guia = obj_fiscal_guia.browse(cr, uid, guia_ids)
            for x in browse_fiscal_guia:
                ja_imprimiu = False
                i += 1
                fornecedor = x.nf_id.destinatario_id.name if x.nf_id.destinatario_id else ''
                documento = x.autenticacao or ''
                data_pagamento = x.data_pagamento or ''
                valor = x.valor or 0.0
                agencia = x.agencia_id
                if agencia:
                    banco = agencia.admbag_banco_id.admbco_codigo_bc
                else:
                    banco = ''
                fornecedor = formataString('' + fornecedor, 30, ' ', 'C', 'N', 'S')
                documento = formataString('' + documento, 7, ' ', 'D', 'N', 'S')
                if data_pagamento:
                    data_vencimento = formataString(
                        '' + datetime.strptime(data_pagamento, "%Y-%m-%d").__format__("%d-%m-%Y"),
                        10, ' ', 'C', 'N', 'S')
                else:
                    data_vencimento = formataString('' + '', 10, ' ', 'C', 'N', 'S')
                valor = formataString('' + getNumeroEditado(valor, 2), 13, ' ', 'C', 'N', 'S')
                banco = formataString('' + str(banco), 6, ' ', 'D', 'N', 'S')
                linha = _('|') + fornecedor + ' ' + documento + ' ' + data_vencimento + '   ' + \
                        valor + ' ' + banco
                linha_imprimi += linha
                if len(browse_fiscal_guia) == 1:
                    ja_imprimiu = True
                    listagem.imprimeLinha(linha_imprimi + '                                        '
                                                          '                                  |')
                if i == 2:
                    ja_imprimiu = True
                    listagem.imprimeLinha(linha_imprimi + ' |')
                    i = 0
                    linha_imprimi = ''
                if x == browse_fiscal_guia[-1] and not ja_imprimiu:
                    listagem.imprimeLinha(linha_imprimi + '                                        '
                                                          '                                 |')
                listagem.imprimeLinha('|' + ' ' * 146 + '|')
        if datas.get('observacao'):
            listagem.imprimeLinha(_('| OBSERVACOES:                                                '
                                    '                                                              '
                                    '                       |'))
            tamanho_linha = 146
            word = datas['observacao'].split(' ')
            frase = ""
            cont = 0
            len_word = len(word)
            campo = 0
            for j in word:
                cont += 1
                size_i = len(j)
                if size_i + campo < tamanho_linha:
                    frase = frase + j + ' '
                    campo = len(frase)
                elif size_i + campo > tamanho_linha:
                    frase = formataString('' + frase, tamanho_linha, ' ', 'D', 'N', 'S')
                    listagem.imprimeLinha('|' + frase + '|')
                    frase = ''
                    frase = frase + j + ' '
                    campo = len(frase)
                if cont == len_word:
                    frase = formataString('' + frase, tamanho_linha, ' ', 'D', 'N', 'S')
                    listagem.imprimeLinha('|' + frase + '|')
        else:
            listagem.imprimeLinha('|' + ' ' * 146 + '|')
        listagem.imprimeLinha('*' + '-' * 146 + '*')

    def listar_descricao_valor(self, cr, uid, listagem, datas, tabela, context=None):
        context = context and context.copy() or {}
        obj_outros_debitos = self.db_name.get(tabela)

        # Receber datas contendo uma lista com os IDs
        outros_debitos = obj_outros_debitos.read(cr, uid, datas, context=context)
        valor_total = 0
        if outros_debitos:
            cont = 1
            iterable = len(outros_debitos)
            for i in outros_debitos:
                valor_total += i['valor']
                descricao = formataString(' ' * 7 + i['descricao'], 97, ' ', 'D', 'N', 'S')
                valor = formataString(' ' * 7 + str('%1.2f' % i['valor']), 25, ' ', 'E', 'N', 'S')
                if cont < iterable:
                    listagem.imprimeLinha('|' + descricao + '|' + valor + '|' + ' ' * 22 + '|')
                elif cont == iterable:
                    valor_total = formataString(
                        '' + str('%1.2f' % valor_total), 22, ' ', 'E', 'N', 'S')
                    listagem.imprimeLinha('|' + descricao + '|' + valor + '|' + valor_total + '|')
                cont += 1
        return float(valor_total)

    def total_deducoes(self, cr, uid, listagem, datas, context=None):
        context = context and context.copy() or {}
        obj_outros_debitos = self.db_name.get('fiscal.apuracao.imposto.deducoes')

        # Receber datas contendo {'outros_debitos_ids': [1, 2, 3]}
        outros_debitos = obj_outros_debitos.read(cr, uid, datas, context=context)
        valor_total = 0
        if outros_debitos:
            for i in outros_debitos:
                valor_total += i['valor']

        return float(valor_total)
report_relatorio_apuracao_icms1('report.relatorio.apuracao.icms1')


class wizard_apuracao_icms_capa(osv.TransientModel):
    def parametro_id(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_parametro = self.pool.get('adm.paginas')
        parametro_id = obj_parametro.search(cr, uid, [], context=context)
        if parametro_id:
            return parametro_id[0]
        else:
            return []

    def folha_parametro(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = {}
        parametro_id = self.parametro_id(cr, uid, ids, context)
        company_id = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
        obj_pagina = self.pool.get('adm.paginas')
        id_pagina = obj_pagina.search(
            cr, uid, [('company_id', '=', company_id), ('tipo', '=', 'wizard_apuracao_icms_capa')],
            context=context)
        if id_pagina:
            id_dados = obj_pagina.read(cr, uid, parametro_id, ['inicial'], context=context)
            grava = id_dados['inicial']
        else:
            vals = {'data': date.today(),
                    'inicial': '1',
                    'tipo': 'wizard_apuracao_icms_capa'
            }
            grava = obj_pagina.create(cr, uid, vals, context=context)

        return grava

    def periodo_parametro(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        parametro_id = self.parametro_id(cr, uid, ids, context)
        company_id = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
        obj_pagina = self.pool.get('adm.paginas')
        id_pagina = obj_pagina.search(
            cr, uid, [('company_id', '=', company_id), ('tipo', '=', 'wizard_apuracao_icms_capa')],
            context=context)
        if id_pagina:
            id_data = obj_pagina.read(cr, uid, parametro_id, ['data'], context=context)
            data = id_data['data']
            return data

    _name = 'wizard.apuracao.icms'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade'),
        'tipo': fields.char('tipo', size=1),
        'apuracao_icms_id': fields.many2one(
            'fiscal.apuracao.imposto', 'Apuracao ICMS',
            domain="[('tipo', '=', 'C'), ('company_id', '=', company_id)]"),
        'folha': fields.integer('Folha'),
        'saldo_periodo_anterior': fields.float('Saldo Anterior', digits=(16, 2)),
        'dia_inicial': fields.integer('Data inicial'),
        'dia_final': fields.integer('Data final'),
        }
    _defaults = {
        'folha': folha_parametro,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c)
        }

    def apuracao_icms_get_ultimo_dia(self, cr, uid, apuracao_icms_id, context=None):
        context = context and context.copy() or {}
        if not apuracao_icms_id:
            return 0
        brw_fiscal_apuracao_imposto = self.pool.get('fiscal.apuracao.imposto').browse(
            cr, uid, apuracao_icms_id, context)
        data_inicial_apuracao = brw_fiscal_apuracao_imposto.data_apuracao
        ultimo_dia_valido = ((parse(data_inicial_apuracao) + relativedelta(months=1)) -
                             relativedelta(days=1)).day
        return ultimo_dia_valido


    def apuracao_icms_onchange_dia(self, cr, uid, ids, dia_inicial, dia_final, apuracao_icms_id,
                                   context=None):
        context = context and context.copy() or {}
        res = dict()
        msg = str()
        vals = dict()
        if dia_inicial < 1 or dia_inicial > 31:
            msg += _('Dia inicial invalido.\n')
            vals.update({'dia_inicial': 1})
        if dia_final < 1 or dia_final > 31:
            msg += _('Dia final invalido.\n')
            vals.update({'dia_final': 1})
        if dia_inicial > dia_final:
            msg += _('Dia inicial deve ser menor ou igual ao dia final.\n')
            vals.update({'dia_inicial': 1})
        ultimo_dia_valido = self.apuracao_icms_get_ultimo_dia(cr, uid, apuracao_icms_id, context)
        if apuracao_icms_id and not msg and (dia_inicial or dia_final):
            if dia_inicial > ultimo_dia_valido:
                msg += _('Dia inicial deve ser menor ou igual a %d.\n' % ultimo_dia_valido)
                vals.update({'dia_inicial': ultimo_dia_valido})
            if dia_final > ultimo_dia_valido:
                msg += _('Dia final deve ser menor ou igual a %d.\n' % ultimo_dia_valido)
                vals.update({'dia_final': ultimo_dia_valido})
        if msg and (vals.get('dia_inicial') == 1 or dia_inicial == 1) \
                and (vals.get('dia_final') == ultimo_dia_valido or dia_final == ultimo_dia_valido):
            vals.update({'saldo_periodo_anterior': self.apuracao_icms_get_saldo_anterior(
                cr, uid, apuracao_icms_id, context)})
        if ultimo_dia_valido and (dia_inicial != 1 and vals.get('dia_inicial') != 1) or \
                (vals.get('dia_final') != ultimo_dia_valido and dia_final != ultimo_dia_valido):
            vals.update({'saldo_periodo_anterior': 0.0})
        else:
            vals.update({'saldo_periodo_anterior': self.apuracao_icms_get_saldo_anterior(
                cr, uid, apuracao_icms_id, context)})
        if vals:
            res['value'] = vals
        if msg:
            res['warning'] = {'title': _('Atencao'), 'message': msg}
        return res

    def onchange_saldo_periodo_anterior(self, cr, uid, ids, id_apuracao, context=None):
        context = context and context.copy() or {}
        res = {'value': {'saldo_periodo_anterior': 0}}
        if not id_apuracao:
            return res
        ultimo_dia = self.apuracao_icms_get_ultimo_dia(cr, uid, id_apuracao, context)
        res['value'].update(
            {'dia_inicial': 1, 'dia_final': ultimo_dia,
             'saldo_periodo_anterior': self.apuracao_icms_get_saldo_anterior(
                 cr, uid, id_apuracao, context)
             }
        )
        return res

    def apuracao_icms_get_saldo_anterior(self, cr, uid, apuracao_icms_id, context=None):
        context = context and context.copy() or {}
        saldo_anterior = 0.0
        if not apuracao_icms_id:
            return saldo_anterior
        obj_fiscal_apuracao_imposto = self.pool.get('fiscal.apuracao.imposto')
        data_apuracao = obj_fiscal_apuracao_imposto.read(
                cr, uid, apuracao_icms_id, ['data_apuracao'], context=context).get('data_apuracao')
        apuracao_imposto_anterior_id = False
        if data_apuracao:
            apuracao_imposto_anterior_id = obj_fiscal_apuracao_imposto.search(
                cr, uid, [('tipo', '=', 'C'), ('data_apuracao', '<', data_apuracao)],
                context=context, order='data_apuracao desc', limit=1)

        if apuracao_imposto_anterior_id:
            saldo_anterior = obj_fiscal_apuracao_imposto.read(
                cr, uid, apuracao_imposto_anterior_id[0], ['saldo_credor'], context=context).get(
                'saldo_credor') or 0.0
        return saldo_anterior

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        from dateutil.relativedelta import relativedelta
        apuracao_icms_id = self.read(cr, uid, ids[0], context=context)
        icms_id = apuracao_icms_id.get('apuracao_icms_id')
        if isinstance(icms_id, tuple):
            icms_id = icms_id[0]
        datas = {}
        datas.update(self.pool.get('fiscal.apuracao.imposto').read(
            cr, uid, icms_id, context=context))
        if datas.get('mes') < 10:
            datas['mes'] = '0' + str(datas.get('mes'))
        datas['folha'] = apuracao_icms_id['folha']
        datas['saldo_periodo_anterior'] = apuracao_icms_id['saldo_periodo_anterior']
        datas['apuracao_icms_id'] = icms_id

        if not datas['ano']:
            raise osv.except_osv(_('Atencao!'), _('Informe o ano para apuracao.'))
        elif not datas['mes']:
            raise osv.except_osv(_('Atencao!'), _('Informe o mes para apuracao.'))
        elif not datas['folha']:
            raise osv.except_osv(_('Atencao!'), _('Informe a folha.'))
        else:
            datas['periodo_inicial'] = str(datas['ano']) + "-" + str(datas['mes']) + "-" + \
                                       str(apuracao_icms_id.get('dia_inicial'))
            datas['periodo_final'] = (parse(str(datas['ano']) + "-" + str(datas['mes']) + "-" +
                                            str(apuracao_icms_id.get('dia_final')))).strftime('%Y-%m-%d')
            datas.pop('name')
            datas.pop('id')

        return {'type': 'ir.actions.report.xml', 'report_name': 'relatorio.apuracao.icms1',
                'datas': datas}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
