# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
from openerp import pooler
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_fiscal_apuracao_preco_ncm(report_int):
    def create(self, cr, uid, ids, data, context=None):
        context = context and context.copy() or {}
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.informaNome(pdf_string, _('Listagem de NCM - Fiscal'))
        listagem.informaTamLinha(150)
        listagem.rodapePadrao()

        # VARIAVEIS GLOBAIS
        self.db_name = pooler.get_pool(cr.dbname) # PERMITE ACESSO AS TABELAS.
        self.traco_linha = 145 # QUANTIDADE DE CARACTERES NA HORIZONTAL QUE SERÃO ESCRITOS.
        self.space = ' ' * self.traco_linha # ESPAÇO DE LINHA.

        # COLUNAS DO RELATORIO
        self.col_produto = 68
        self.col_ncm = 8
        self.col_cst_codigo = 3
        self.col_encarago_sigla = 7
        self.col_tributacao = 30
        self.col_tributacao_base = 6
        self.col_tributacao_aliquota = 8
        self.col_cst_tipo = 8

        self.tracos = _('|') + (self.col_produto * '-') + \
                      _('|') + (self.col_ncm * '-') + \
                      _('|') + (self.col_cst_codigo * '-') + \
                      _('|') + (self.col_encarago_sigla * '-') + \
                      _('|') + (self.col_tributacao * '-') + \
                      _('|') + (self.col_tributacao_base * '-') + \
                      _('|') + (self.col_tributacao_aliquota * '-') + \
                      _('|') + (self.col_cst_tipo * '-') + _('|')

        self.tracos_fiscal_branco = _('|') + (self.col_tributacao * ' ') + \
                                    _('|') + (self.col_cst_codigo * ' ') + \
                                    _('|') + (self.col_encarago_sigla * ' ') + \
                                    _('|') + (self.col_tributacao_base * ' ') + \
                                    _('|') + (self.col_tributacao_aliquota * ' ') + \
                                    _('|') + (self.col_cst_tipo * ' ') + _('|')

        self.tracos_fiscal_traco = _('|') + (self.col_produto * ' ') + \
                                   _('|') + (self.col_ncm * ' ') + \
                                   _('|') + (self.col_cst_codigo * '-') + \
                                   _('|') + (self.col_encarago_sigla * '-') + \
                                   _('|') + (self.col_tributacao * '-') + \
                                   _('|') + (self.col_tributacao_base * '-') + \
                                   _('|') + (self.col_tributacao_aliquota * '-') + \
                                   _('|') + (self.col_cst_tipo * '-') + _('|')


        # IMPRESSÃO DO RELATORIO (CABEÇALHO)
        self.define_cabecalho(listagem, data, context)
        self.listar_registros(cr, listagem=listagem, in_produto_ids=data.get('produto_ids'), context=context)

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return (self.obj.pdf, 'pdf')

    def define_cabecalho(self, listagem, data, context=None):
        context = context and context.copy() or {}
        str_titulo = formataString(_('APURACAO DE PRECO E NCM'), self.traco_linha, ' ', 'C', 'N', 'S')
        str_company_name = formataString(_('EMPRESA: ') + data.get('company_name') or '', self.traco_linha, ' ', 'D', 'N',
                                         'S')
        str_company_inscricao_est = _('INSCRICAO ESTADUAL: ') + str(data.get('company_inscricao_est')) or ''
        str_company_ie = formataString(str_company_inscricao_est, len(str_company_inscricao_est), ' ', 'D', 'N', 'S')
        str_company_cnpj = formataString(_('          CNPJ: ') + data.get('company_cnpj') or '',
                                         self.traco_linha - len(str_company_ie), ' ', 'D', 'N', 'S')
        str_folha = formataString(_('Folha: {{ Page , 5}}'), self.traco_linha + 8, ' ', 'E', 'N', 'S')

        str_produto = formataString(_('COD. - DESCRICAO DO PRODUTO'), self.col_produto, ' ', 'D', 'N', 'S')
        str_ncm = formataString(_('NCM'), self.col_ncm, ' ', 'D', 'N', 'S')
        str_cst_codigo = formataString(_('CST'), self.col_cst_codigo, ' ', 'D', 'N', 'S')
        str_encargo_sigla = formataString(_('ENCARGO'), self.col_encarago_sigla, ' ', 'D', 'N', 'S')
        str_tributacao = formataString(_('TRIBUTACAO'), self.col_tributacao, ' ', 'D', 'N', 'S')
        str_base = formataString(_('BASE'), self.col_tributacao_base, ' ', 'D', 'N', 'S')
        str_aliquota = formataString(_('ALIQUOTA'), self.col_tributacao_aliquota, ' ', 'D', 'N', 'S')
        str_cst_tipo = formataString(_('TIPO'), self.col_cst_tipo, ' ', 'D', 'N', 'S')

        listagem.informaCab(_('*') + ('-' * self.traco_linha) + '*')
        listagem.informaCab(_('|') + str_titulo + _('|'))
        listagem.informaCab(_('|') + self.space + _('|'))
        listagem.informaCab(_('|') + str_company_name + _('|'))
        listagem.informaCab(_('|') + self.space + _('|'))
        listagem.informaCab(_('|') + str_company_ie + str_company_cnpj + _('|'))
        listagem.informaCab(_('|') + self.space + _('|'))
        listagem.informaCab(_('|') + str_folha + _('|'))
        listagem.informaCab(_('|') + self.space + _('|'))
        listagem.informaCab(_('|') + ('-' * self.traco_linha) + _('|'))
        listagem.informaCab(
            _('|') + str_produto + _('|') + str_ncm + _('|') + str_cst_codigo + _('|') + str_encargo_sigla + _(
                _('|')) + str_tributacao + _('|') + str_base + _('|') + str_aliquota + _('|') + str_cst_tipo + _('|'))
        listagem.informaCab(_('|') + ('-' * self.traco_linha) + _('|'))
        listagem.informaCab(_('|') + self.space + _('|'))

    def get_produto_names(self, cr, produto_ids):
        """
        funcao que retorna o nome dos produtos.
        É utilizada para que o consumo de memoria nao seja alto na funcao get_registros.
        A funcao get_registros possui o id do produto e realiza a consulta do nome no retorno desse metodo.
        returna {id: name} caso haja resultados, caso contrario, retorna {}
        """
        # cada item da lista e convertido para str pois se a lista for transformada em str e conter
        # um long, ficará 'IDL' (o id seguindo da letra L), que irá gerar erro no sql.
        str_produto_ids = map(str, produto_ids)
        str_produto_ids = str(str_produto_ids)
        str_produto_ids = str_produto_ids.replace('[', '(').replace(']', ')')
        sql = """
            select
                produto.id,
                produto.name
            from
                estoque_produto produto
            where
                produto.id in %s
        """ % str_produto_ids
        cr.execute(sql)
        registros = cr.fetchall()
        if registros:
            return dict(registros)
        else:
            return {}

    def get_produto_codigo(self, cr, produto_ids):
        str_produto_ids = map(str, produto_ids)
        str_produto_ids = str(str_produto_ids)
        str_produto_ids = str_produto_ids.replace('[', '(').replace(']', ')')
        sql = """
            select
                produto.id,
                produto.codigo
            from
                estoque_produto produto
            where
                produto.id in %s
        """ % str_produto_ids
        cr.execute(sql)
        registros = cr.fetchall()
        if registros:
            return dict(registros)
        else:
            return {}

    def get_produto_ncm(self, cr, produto_ids):
        str_produto_ids = map(str, produto_ids)
        str_produto_ids = str(str_produto_ids)
        str_produto_ids = str_produto_ids.replace('[', '(').replace(']', ')')
        sql = """
            select
                produto.id,
                ncm.codigo
            from
                estoque_produto produto,
                fiscal_ncm as ncm
            where
                produto.ncm_id = ncm.id and
                produto.id in %s
        """ % str_produto_ids
        cr.execute(sql)
        registros = cr.fetchall()
        if registros:
            return dict(registros)
        else:
            return {}

    def get_registros(self, cr, in_produto_ids=[]):

        select_from = """
            select
                produto.id as produto_id,
                ncm.codigo as ncm_codigo,
                cst.codigo as cst_codigo,
                tributacao.name as tributacao,
                tributacao.base as tributacao_base,
                tributacao.aliquota as tributacao_aliquota,
                encargo.sigla as encargo_sigla,
                cst.tipo as cst_tipo

            from
                estoque_produto produto,
                fiscal_ncm ncm,
                fiscal_tributacao_encargos trib_encargo,
                fiscal_tributacao tributacao,
                fiscal_encargo encargo,
                fiscal_cst cst """

        where = """
            where
                produto.ncm_id = ncm.id and
                trib_encargo.produto_id = produto.id and
                trib_encargo.tributacao_id = tributacao.id and
                trib_encargo.encargo_id = encargo.id and
                trib_encargo.cst_id = cst.id """

        if in_produto_ids:
            in_produto_ids = map(str, in_produto_ids)
            str_in_produto_ids = str(in_produto_ids)
            str_in_produto_ids = str_in_produto_ids.replace('[', '(').replace(']', ')')
            where += """
                and produto.id in %s
            """ % str_in_produto_ids

        group_order_by = """
            group by
                produto.id,
                produto.name,
                ncm.codigo,
                cst.codigo,
                tributacao.name,
                tributacao.base,
                tributacao.aliquota,
                encargo.sigla,
                cst.tipo

            order by
                produto.name
        """
        sql = select_from + where + group_order_by
        cr.execute(sql)
        registros = cr.dictfetchall()
        if registros:
            new_registro = {}
            for i in registros:
                produto_id = i['produto_id']
                del i['produto_id']
                if new_registro.get(produto_id):
                    new_registro[produto_id].append(i)
                else:
                    new_registro.update({produto_id: [i]})
                registros.remove(i)
            return new_registro
        else:
            return {}

    def listar_registros(self, cr, listagem, in_produto_ids, context=None):
        context = context and context.copy() or {}
        registro_produtos = self.get_registros(cr, in_produto_ids=in_produto_ids)
        if registro_produtos:
            produto_ids = registro_produtos.keys()
            if produto_ids:
                dict_produto_codigo = self.get_produto_codigo(cr, produto_ids)
                dict_produto_names = self.get_produto_names(cr, produto_ids)
                dict_produto_ncm = self.get_produto_ncm(cr, produto_ids)
                cst_tipo_descricao = dict([('T', 'Tribut.'), ('U', 'Subst.'), ('I', 'Isento'), ('O', 'Outros')])

                for key in registro_produtos.keys():

                    produto_id = key
                    produto_codigo = dict_produto_codigo.get(produto_id) or ''
                    produto_ncm = dict_produto_ncm.get(produto_id) or ''
                    produto_name = dict_produto_names.get(produto_id) or ''
                    del dict_produto_codigo[produto_id]
                    del dict_produto_names[produto_id]
                    del dict_produto_ncm[produto_id]

                    key_registros = registro_produtos[key]
                    if key_registros:
                        listagem.imprimeLinha(self.tracos)
                        total_listagem = len(key_registros)
                        listados = 1
                        for i in key_registros:

                            if listados != 1:
                                produto_codigo = ''
                                produto_name = ''
                                produto_ncm = ''

                            produto_cod_name = produto_codigo + ' - ' + produto_name if produto_codigo and produto_name else ''

                            str_produto_name = formataString(produto_cod_name, self.col_produto, ' ', 'D', 'S', 'S')
                            str_ncm = formataString(produto_ncm, self.col_ncm, ' ', 'D', 'S', 'S')

                            cst_codigo = i.get('cst_codigo') or ''
                            if cst_codigo:
                                cst_codigo = '0' + cst_codigo
                            tributacao = i.get('tributacao') or ''
                            tributacao_base = i.get('tributacao_base') or ''
                            tributacao_aliquota = i.get('tributacao_aliquota') or ''
                            encargo_sigla = i.get('encargo_sigla') or ''
                            cst_tipo = i.get('cst_tipo') or ''

                            cst_tipo = cst_tipo_descricao.get(cst_tipo) or 'Outros'

                            str_cst_codigo = formataString(cst_codigo, self.col_cst_codigo, ' ', 'D', 'N', 'S')
                            str_tributacao = formataString(tributacao, self.col_tributacao, ' ', 'D', 'N', 'S')
                            str_encargo_sigla = formataString(encargo_sigla, self.col_encarago_sigla, ' ', 'D', 'N',
                                                              'S')
                            str_tributacao_base = formataString(tributacao_base, self.col_tributacao_base, ' ', 'D',
                                                                'N', 'S')
                            str_tributacao_aliquota = formataString(tributacao_aliquota, self.col_tributacao_aliquota,
                                                                    ' ', 'D', 'N', 'S')
                            str_cst_tipo = formataString(cst_tipo, self.col_cst_tipo, ' ', 'D', 'N', 'S')
                            # listagem dos registros
                            listagem.imprimeLinha(
                                _('|') + str_produto_name + _('|') + str_ncm + _('|') + str_cst_codigo + _(
                                    _('|')) + str_encargo_sigla + _('|') + str_tributacao + _(
                                    _('|')) + str_tributacao_base + _('|') + str_tributacao_aliquota + _(
                                    _('|')) + str_cst_tipo + _('|'))
                            if listados != total_listagem:
                                listagem.imprimeLinha(self.tracos_fiscal_traco)
                            listados += 1
                    del registro_produtos[key]
        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
        return True


report_fiscal_apuracao_preco_ncm('report.fiscal_apuracao_preco_ncm')


class wiz_apuracao_preco_ncm(osv.TransientModel):
    _name = 'wiz.apuracao.ncm'
    _columns = {
        'wfis_apncm_company_id': fields.many2one('res.company', "Empresa", required=True),
        'wfis_apncm_ncm_id': fields.many2one('fiscal.ncm', "NCM"),
        'wfis_apncm_categoria_id': fields.many2one('estoque.categoria', "Categoria")
    }
    _defaults = {
        'wfis_apncm_company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(
            cr, uid, uid, context=c).company_id.id,
    }

    def button_wfis_apncm_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        data = {}
        apuracao_preco_ncm_dados = self.read(cr, uid, ids,
                                             ['wfis_apncm_company_id', 'wfis_apncm_ncm_id', 'wfis_apncm_categoria_id'], context=context)
        if isinstance(apuracao_preco_ncm_dados, list):
            apuracao_preco_ncm_dados = apuracao_preco_ncm_dados[0]
        if isinstance(apuracao_preco_ncm_dados, dict):
            company_id = apuracao_preco_ncm_dados.get('wfis_apncm_company_id') and \
                         apuracao_preco_ncm_dados['wfis_apncm_company_id'][0] or False
            if not company_id:
                raise osv.except_osv(_('Atencao'), _(
                    'Nenhuma empresa selecionada para a impressao do Relatorio de Apuracao de Precos e NCM'))
            else:
                browse_company = self.pool.get("res.company").browse(cr, uid, company_id)
                company_name = browse_company.name or ''
                company_cnpj = browse_company.rescon_pessoa_id.cpf_cnpj or ''
                company_inscricao_est = browse_company.rescon_pessoa_id.admpes_jur_ie or ''

                ncm_id = apuracao_preco_ncm_dados.get('wfis_apncm_ncm_id') and \
                         apuracao_preco_ncm_dados['wfis_apncm_ncm_id'][0] or False
                produto_categoria_id = apuracao_preco_ncm_dados.get('wfis_apncm_categoria_id') and \
                                       apuracao_preco_ncm_dados['wfis_apncm_categoria_id'][0] or False
                args_produto = []

                if ncm_id:
                    args_produto.append(('ncm_id', '=', ncm_id))
                if produto_categoria_id:
                    categoria_ids = self.pool.get('estoque.categoria').search(cr, uid, [
                        ('parent_id', 'child_of', produto_categoria_id)], context=context)
                    if categoria_ids:
                        args_produto.append(('categoria_id', 'in', categoria_ids))
                if args_produto:
                    produto_ids = self.pool.get('estoque.produto').search(cr, uid, args_produto, context=context)
                    if not produto_ids:
                        raise osv.except_osv(_('Atencao'), _('Nenhum resultado encontrado para os criteritos informados!'))
                else:
                    produto_ids = []
                data = {'company_name': company_name, 'company_cnpj': company_cnpj,
                        'company_inscricao_est': company_inscricao_est, 'produto_ids': produto_ids}

        return {'type': 'ir.actions.report.xml', 'report_name': 'fiscal_apuracao_preco_ncm', 'datas': data}
