#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.report.render import render
from openerp.osv import osv, fields
from openerp import pooler
from openerp.tools.translate import _
from SigUtils.impressao import Impressora
from SigUtils.formatastring import formataString
from SigUtils.datas import getDataNoFormato
import StringIO
from openerp.report.interface import report_int
import time
from SigUtils.numero import getNumeroEditado
from openerp import SUPERUSER_ID
import gc


ecf_instalado = False
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class wizard_apuracao_pis_cofins(osv.TransientModel):
    opcao = [('R', 'Resumo'),
             ('N', 'Notas / Cupons'),
             ('I', 'Itens')]
    _name = 'wizard.apuracao.pis.cofins'
    _columns = {
        'company_ids': fields.many2many('res.company', 'res_company_apur_rel', 'id', 'company_id',
                                        'Empresas', required=True, domain=[]),
        'wiz_data_ini': fields.date('Data inicial', required=True),
        'wiz_data_fin': fields.date('Data final', required=True),
        'wiz_client_fornec': fields.many2one('adm.pessoa', 'Cliente/Fornecedor'),
        'wiz_nf_numero': fields.integer('Nota'),
        'wiz_nf_serie': fields.char('Serie', size=3),
        'wiz_opcao': fields.selection(opcao, 'Opcao', required=True),
        'wiz_entrada': fields.boolean('Entradas'),
        'wiz_saida': fields.boolean('Saida'),
        'wiz_cupons': fields.boolean('Cupons'),
        'wiz_cst': fields.text('CST', help='CST de Pis/Cofins'),
        'wiz_cfop_ids': fields.many2many('fiscal.cfop', 'm2m_apuracao_pis_cofins_cfop',
                                         'apuracao_id', 'cfop_id', 'CFOP'),
        'wiz_nat_rec': fields.char('Natureza de PIS/COFINS', size=3),
        'wiz_item_sem_nat_rec': fields.boolean('Itens sem natureza receita',
                                               help='Imprimir os itens que estao sem a natureza da '
                                                    'receita do PIS e/ou COFINS'),
        'wiz_item_nat_rec_diferente': fields.boolean(
            'Itens natureza receita diferentes', help='Imprime os itens que estao com a natureza '
                                                      'da receita do PIS/COFINS diferentes'),
        'wiz_cfop_sped_contribuicoes': fields.boolean(
            'CFOP SPED Contribuicoes', help='Lista somente CFOPs considerados no SPED '
                                            'Contribuicoes.'),
    }

    _defaults = {
        'wiz_entrada': True,
        'wiz_saida': True,
        'wiz_cupons': True,
        'wiz_opcao': 'R',
        'wiz_item_sem_nat_rec': False,
        'wiz_item_nat_rec_diferente': False,
    }

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        return super(wizard_apuracao_pis_cofins, self).read(cr, SUPERUSER_ID, ids, fields, context,
                                                            load)

    def button_imprimir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        apuracao_piscofins = self.read(cr, SUPERUSER_ID, ids, [], context=context)
        if isinstance(apuracao_piscofins, (list, tuple)):
            if apuracao_piscofins:
                apuracao_piscofins = apuracao_piscofins[0]

        if apuracao_piscofins.get('wiz_data_ini') > apuracao_piscofins.get('wiz_data_fin'):
            raise osv.except_osv(_('Atencao'),
                                 _('Data incial nao pode ser maior que data final!'))

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'wizard.apuracao.pis.cofins',
            'datas': apuracao_piscofins
        }


class report_apuracao_pis_cofins(report_int):

    def center_title(self, E, C, D, T=134):
        title_c = (T - len(C)) / 2
        title = E
        for i in range(title_c - len(E)):
            title += ' '
        title += C
        for i in range(title_c - len(D)):
            title += ' '
        if (T - len(C)) % 2 > 0:
            D = ' ' + D
        title += D
        return title

    @staticmethod
    def getFormatador(caracter='-', tamanho=144):
        return caracter * tamanho

    def create(self, cr, uid, ids, datas, context=None):
        global ecf_instalado
        context = context and context.copy() or {}
        ecf_instalado = pooler.get_pool(cr.dbname).get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'ecf', context)
        data_inicio = datas.get('wiz_data_ini')
        data_final = datas.get('wiz_data_fin')
        imprimir_item_sem_nat_rec = datas.get('wiz_item_sem_nat_rec')
        imprimir_item_nat_rec_diferentes = datas.get('wiz_item_nat_rec_diferente')
        dt_ini = getDataNoFormato(data_inicio, 'N', '/')
        dt_fin = getDataNoFormato(data_final, 'N', '/')
        entrada = datas.get('wiz_entrada')
        saida = datas.get('wiz_saida')
        cupons = datas.get('wiz_cupons')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Apuracao de Pis Cofins'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)
        listagem.defineFonte(tam=6.5)
        listagem.informaRodape(self.getFormatador(caracter='_', tamanho=144))
        listagem.informaRodape(self.getFormatador(caracter=' ', tamanho=55) +
                               'Sig Informatica Ltda.')
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                                     'wizard.apuracao.pis.cofins',
                                                                                     context=context)
        company_ids = datas.get('company_ids')
        try:
            empresa_nome = pooler.get_pool(cr.dbname).get('res.company').browse(cr, uid,
                                                                                company_id).name
        except AttributeError:
            empresa_nome = ' '
        data_relatorio = time.strftime('%d/%m/%Y')
        titulo_empresa = self.center_title('|', _(empresa_nome), '|', 146)
        opcao = datas.get('wiz_opcao')
        listagem.informaCab(_('*' + self.getFormatador(tamanho=144) + '*'))
        listagem.informaCab(titulo_empresa)
        titulo = self.center_title(_('|' + data_relatorio), _('Apuracao de Pis/Cofins'),
                                   _('      Pagina: {{ Page }}' + '|'), 155)
        listagem.informaCab(titulo)
        listagem.informaCab('|' + ' ' * 144 + '|')
        periodo = self.center_title('|' + 'Periodo de: %s a %s' % (dt_ini, dt_fin), ' ', '|', 146)
        listagem.informaCab(periodo)
        if opcao == 'N':
            listagem.informaCab(_('*' + self.getFormatador(tamanho=144) + '*'))
            cabecalho = formataString('|Cod.', 6, ' ', 'D', 'N', 'N')
            cabecalho += formataString('Cliente/Fornecedor', 35, ' ', 'D', 'N', 'N')
            cabecalho += formataString('Cnae', 6, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Numero', 8, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Ser', 5, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Emissao', 11, ' ', 'C', 'N', 'N')
            cabecalho += formataString('CFOP', 6, ' ', 'D', 'N', 'N')
            cabecalho += formataString('Valor', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Valor item', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('%Pis', 5, ' ', 'C', 'N', 'N')
            cabecalho += formataString('VL.PIS', 8, ' ', 'C', 'N', 'N')
            cabecalho += formataString('%COF.', 5, ' ', 'C', 'N', 'N')
            cabecalho += formataString('VL.COFINS', 12, ' ', 'C', 'N', 'N')
            listagem.informaCab(cabecalho + ' ' * 14 + '|')
            listagem.informaCab(_('*' + self.getFormatador(tamanho=144) + '*'))
            listagem.informaCab('|' + ' ' * 144 + '|')
            res = self.relat_notas_cupons_itens(cr, uid, listagem, data_inicio, data_final, entrada,
                                                saida, cupons, company_ids, datas, itens=False,
                                                resumo=False)
        elif opcao == 'I':
            listagem.informaCab(_('*' + self.getFormatador(tamanho=144) + '*'))
            cabecalho = formataString('|Cod.', 7, ' ', 'D', 'N', 'N')
            cabecalho += formataString('Cliente/Fornecedor', 33, ' ', 'D', 'N', 'N')
            cabecalho += formataString('Cnae', 6, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Numero', 8, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Serie', 5, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Emissao', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('CFOP', 5, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Valor', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Valor.Itens', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('%Pis', 6, ' ', 'C', 'N', 'N')
            cabecalho += formataString('VL.PIS', 9, ' ', 'C', 'N', 'N')
            cabecalho += formataString('%COF.', 6, ' ', 'C', 'N', 'N')
            cabecalho += formataString('VL.COFINS', 9, ' ', 'C', 'N', 'N')
            listagem.informaCab(cabecalho + '               |')

            cabecalho = formataString('| Cod.Item', 10, ' ', 'E', 'N', 'N')
            cabecalho += formataString('GR', 4, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Descricao', 40, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Ncm', 9, ' ', 'C', 'N', 'N')
            cabecalho += formataString(' ', 8, ' ', 'C', 'N', 'N')
            cabecalho += formataString('CFOP', 5, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Valor', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('Valor.Item', 12, ' ', 'C', 'N', 'N')
            cabecalho += formataString('%Pis', 6, ' ', 'C', 'N', 'N')
            cabecalho += formataString('VL.PIS', 9, ' ', 'C', 'N', 'N')
            cabecalho += formataString('%COF.', 6, ' ', 'C', 'N', 'N')
            cabecalho += formataString('VL.COFINS', 9, ' ', 'C', 'N', 'N')
            listagem.informaCab(cabecalho + '               |')
            listagem.informaCab(_('*' + self.getFormatador(tamanho=144) + '*'))
            listagem.informaCab('|' + ' ' * 144 + '|')
            res = self.relat_notas_cupons_itens(
                cr, uid, listagem, data_inicio, data_final, entrada, saida, cupons, company_ids,
                datas, itens=True, resumo=False,
                imprimir_item_sem_nat_rec=imprimir_item_sem_nat_rec,
                imprimir_item_nat_rec_diferentes=imprimir_item_nat_rec_diferentes)
        else:
            res = self.relat_notas_cupons_itens(cr, uid, listagem, data_inicio, data_final, entrada,
                                                saida, cupons, company_ids, datas, itens=False,
                                                resumo=True)
        movimento_item_ids = list()
        new_res = list()
        for dado in res:
            if dado.get('mov_item_id') and dado.get('mov_item_id') in movimento_item_ids:
                continue
            movimento_item_ids.append(dado.get('mov_item_id'))
            new_res.append(dado)
        if new_res:
            res = new_res
        self.relat_totais_cfop(cr, uid, res, listagem, context)
        self.relat_totais_cst_nat(res, listagem)
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return (self.obj.pdf, 'pdf')

    def relat_totais_cfop(self, cr, uid, res, listagem, context=None):
        context = context and context.copy() or {}
        separador = _('*' + self.getFormatador(caracter='-', tamanho=144) + '*')
        titulo = self.center_title('|', 'TOTAIS POR CFOP', '|', 146)
        listagem.imprimeLinha(separador)
        listagem.imprimeLinha(titulo)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        cabecalho_coluna = formataString('CFOP', 20, ' ', 'C', 'N', 'N')
        cabecalho_coluna += formataString('Valor Total', 20, ' ', 'E', 'N', 'N')
        cabecalho_coluna += formataString('Valor Total Item', 20, ' ', 'E', 'N', 'N')
        cabecalho_coluna += formataString('Base PIS', 20, ' ', 'E', 'N', 'N')
        cabecalho_coluna += formataString('Valor PIS', 15, ' ', 'E', 'N', 'N')
        cabecalho_coluna += formataString('Base COFINS', 20, ' ', 'E', 'N', 'N')
        cabecalho_coluna += formataString('Valor COFINS', 15, ' ', 'E', 'N', 'N')
        cabecalho_coluna = '|' + cabecalho_coluna + self.getFormatador(caracter=' ',
                                                                       tamanho=14) + '|'
        listagem.imprimeLinha(cabecalho_coluna)
        tot_ent_cfop = 0.00
        tot_ent_base_pis_cfop = 0.00
        tot_ent_pis_cfop = 0.00
        tot_ent_item_cfop = 0.00
        tot_ent_base_cofins_cfop = 0.00
        tot_ent_cofins_cfop = 0.00
        tot_sai_cfop = 0.00
        tot_sai_item_cfop = 0.00
        tot_sai_base_pis_cfop = 0.00
        tot_sai_pis_cfop = 0.00
        tot_sai_base_cofins_cfop = 0.00
        tot_sai_cofins_cfop = 0.00
        cfops = list(set(map(lambda x: x.get('cfop'), res)))
        for cfop in cfops:
            vl_tot = sum(map(lambda x: x.get('vr_total_item_rateado') or 0.0    if x.get('cfop') == cfop
                                                                                else 0.0, res))
            vl_tot_item = sum(map(lambda x: x.get('vr_total_item') or 0.0       if x.get('cfop') == cfop
                                                                                else 0.0, res))
            vl_base_pis = sum(
                map(lambda x: x.get('vr_base_calculo_pis') or 0.0 if x.get('cfop') == cfop
                else 0.0, res))
            vl_base_cofins = sum(
                map(lambda x: x.get('vr_base_calculo_cofins') or 0.0 if x.get('cfop') == cfop
                else 0.0, res))
            vl_imp_pis = sum(map(lambda x: x.get('vr_encargo_pis') or 0.0       if x.get('cfop') == cfop
                                                                                else 0.0, res))
            vl_imp_cofins = sum(map(lambda x: x.get('vr_encargo_cofins') or 0.0 if x.get('cfop') == cfop
                                                                                else 0.0, res))
            linha = '|'
            linha += formataString(cfop, 20, ' ', 'C', 'N', 'N')
            linha += formataString(getNumeroEditado(vl_tot), 20, ' ', 'E', 'N', 'N')
            linha += formataString(getNumeroEditado(vl_tot_item), 20, ' ', 'E', 'N', 'N')
            linha += formataString(getNumeroEditado(vl_base_pis), 20, ' ', 'E', 'N', 'N')
            linha += formataString(getNumeroEditado(vl_imp_pis), 15, ' ', 'E', 'N', 'N')
            linha += formataString(getNumeroEditado(vl_base_cofins), 20, ' ', 'E', 'N', 'N')
            linha += formataString(getNumeroEditado(vl_imp_cofins), 15, ' ', 'E', 'N', 'N')
            linha += self.getFormatador(caracter=' ', tamanho=14) + '|'
            if int(cfop[0]) < 5:
                tot_ent_cfop += vl_tot
                tot_ent_item_cfop += vl_tot_item
                tot_ent_pis_cfop += vl_imp_pis
                tot_ent_cofins_cfop += vl_imp_cofins
                tot_ent_base_pis_cfop += vl_base_pis
                tot_ent_base_cofins_cfop += vl_base_cofins
            else:
                tot_sai_cfop += vl_tot
                tot_sai_item_cfop += vl_tot_item
                tot_sai_pis_cfop += vl_imp_pis
                tot_sai_cofins_cfop += vl_imp_cofins
                tot_sai_base_pis_cfop += vl_base_pis
                tot_sai_base_cofins_cfop += vl_base_cofins
            listagem.imprimeLinha(linha)
            listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
            listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        totais_cfop = '|'
        totais_cfop += formataString('Entradas: ', 20, ' ', 'E', 'N', 'N')
        totais_cfop += formataString(getNumeroEditado(tot_ent_cfop), 20, ' ', 'E', 'N', 'N')
        totais_cfop += formataString(getNumeroEditado(tot_ent_item_cfop), 20, ' ', 'E', 'N',
                                     'N')
        totais_cfop += formataString(getNumeroEditado(tot_ent_base_pis_cfop), 20, ' ', 'E', 'N',
                                     'N')
        totais_cfop += formataString(getNumeroEditado(tot_ent_pis_cfop), 15, ' ', 'E', 'N', 'N')
        totais_cfop += formataString(getNumeroEditado(tot_ent_base_cofins_cfop), 20, ' ', 'E', 'N',
                                     'N')
        totais_cfop += formataString(getNumeroEditado(tot_ent_cofins_cfop), 15, ' ', 'E', 'N',
                                     'N')
        totais_cfop += self.getFormatador(caracter=' ', tamanho=14) + '|'
        listagem.imprimeLinha(totais_cfop)
        totais_cfop = '|'
        totais_cfop += formataString('Saidas: ', 20, ' ', 'E', 'N', 'N')
        totais_cfop += formataString(getNumeroEditado(tot_sai_cfop), 20, ' ', 'E', 'N', 'N')
        totais_cfop += formataString(getNumeroEditado(tot_sai_item_cfop), 20, ' ', 'E', 'N',
                                     'N')
        totais_cfop += formataString(getNumeroEditado(tot_sai_base_pis_cfop), 20, ' ', 'E', 'N',
                                     'N')
        totais_cfop += formataString(getNumeroEditado(tot_sai_pis_cfop), 15, ' ', 'E', 'N', 'N')
        totais_cfop += formataString(getNumeroEditado(tot_sai_base_cofins_cfop), 20, ' ', 'E', 'N',
                                     'N')
        totais_cfop += formataString(getNumeroEditado(tot_sai_cofins_cfop), 15, ' ', 'E', 'N',
                                     'N')
        totais_cfop += self.getFormatador(caracter=' ', tamanho=14) + '|'
        listagem.imprimeLinha(totais_cfop)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')

    def relat_totais_cst_nat(self, res, listagem):
        # Cabecalho
        bytes_pis = 20
        separador = _('*' + self.getFormatador(tamanho=144) + '*')
        underline = '|'
        underline += formataString(self.getFormatador(caracter='_', tamanho=144), 110, ' ', 'C',
                                   'N', 'N')
        underline += '|'
        listagem.imprimeLinha(separador)
        cabecalho = self.center_title('|', 'Resumo de PIS agrupado por CST/Natureza', '|', 146)
        cabecalho_dados = '|'
        cabecalho_dados += formataString('CST', bytes_pis, ' ', 'C', 'N', 'N')
        cabecalho_dados += formataString('Nat.Rec', bytes_pis, ' ', 'C', 'N', 'N')
        cabecalho_dados += formataString('Valor contabil', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += formataString('Valor do item', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += formataString('Base', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += formataString('Imposto', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(cabecalho)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha(cabecalho_dados)
        vl_cont_res_pis = 0.00
        vl_item_res_pis = 0.00
        vl_base_res_pis = 0.00
        vl_encargo_res_pis = 0.00
        vl_cont_ent_pis = 0.00
        vl_item_ent_pis = 0.00
        vl_base_ent_pis = 0.00
        vl_encargo_ent_pis = 0.00
        vl_cont_sai_pis = 0.00
        vl_item_sai_pis = 0.00
        vl_base_sai_pis = 0.00
        vl_encargo_sai_pis = 0.00

        # Resumo de pis
        csts = sorted(list(set(map(lambda x: x.get('cst_codigo'), res))))
        cst_antigo = ''
        print_cst = ''
        for cst in csts:
            print_cst = cst
            if vl_cont_res_pis > 0.00:
                linha_res_pis = '|'
                linha_res_pis += formataString('Tot. CST %s', 40, ' ', 'C', 'N', 'N') % cst_antigo
                linha_res_pis += formataString(getNumeroEditado(vl_cont_res_pis), 20, ' ', 'E', 'N', 'N')
                linha_res_pis += formataString(getNumeroEditado(vl_item_res_pis), 20, ' ', 'E', 'N', 'N')
                linha_res_pis += formataString(getNumeroEditado(vl_base_res_pis), 20, ' ', 'E', 'N', 'N')
                linha_res_pis += formataString(getNumeroEditado(vl_encargo_res_pis), 20, ' ', 'E', 'N', 'N')
                linha_res_pis += self.getFormatador(caracter=' ', tamanho=24) + '|'
                listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
                listagem.imprimeLinha(linha_res_pis)
                listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
                listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
                vl_cont_res_pis = 0.00
                vl_item_res_pis = 0.00
                vl_base_res_pis = 0.00
                vl_encargo_res_pis = 0.00
            naturezas = sorted((set(map(lambda x: x.get('rec_nat') if x.get('cst_codigo') == cst
                                                                else '0', res))))
            for nat in naturezas:
                linha = '|'
                if cst_antigo != cst:
                    linha += formataString(cst or '', bytes_pis, ' ', 'C', 'N', 'N')
                    cst_antigo = cst
                else:
                    linha += formataString(' ', bytes_pis, ' ', 'C', 'N', 'N')
                valor_contabil = sum(map(lambda x: x.get('vr_total_item_rateado') or 0.0 if x.get('cst_codigo') == cst
                                                                                        and x.get('rec_nat') == nat
                                                                                       else 0.0, res))
                if valor_contabil > 0:
                    valor_total_item = sum(map(lambda x: x.get('vr_total_item') or 0.0 if x.get('cst_codigo') == cst
                                                                                      and x.get('rec_nat') == nat
                                                                                     else 0.0, res))
                    base_calculo = sum(map(lambda x: x.get('vr_base_calculo_pis') or 0.0 if x.get('cst_codigo') == cst
                                                                                    and x.get('rec_nat') == nat
                                                                                   else 0.0, res))
                    encargo = sum(map(lambda x: x.get('vr_encargo_pis') or 0.0 if x.get('cst_codigo') == cst
                                                                              and x.get('rec_nat') == nat
                                                                             else 0.0, res))
                    linha += formataString(nat or ' ', bytes_pis, ' ', 'C',
                                           'N', 'N')
                    linha += formataString(getNumeroEditado(valor_contabil),
                                           bytes_pis, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(valor_total_item),
                                           bytes_pis, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(base_calculo), bytes_pis,
                                           ' ',
                                           'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(encargo), bytes_pis,
                                           ' ', 'E', 'N', 'N')
                    vl_cont_res_pis += float(valor_contabil)
                    vl_item_res_pis += float(valor_total_item)
                    vl_base_res_pis += float(base_calculo)
                    vl_encargo_res_pis += float(encargo)
                    try:
                        if cst.isdigit():
                            if int(cst) > 49:
                                vl_cont_ent_pis += float(valor_contabil)
                                vl_item_ent_pis += float(valor_total_item)
                                vl_base_ent_pis += float(base_calculo)
                                vl_encargo_ent_pis += float(encargo)
                            else:
                                vl_cont_sai_pis += float(valor_contabil)
                                vl_item_sai_pis += float(valor_total_item)
                                vl_base_sai_pis += float(base_calculo)
                                vl_encargo_sai_pis += float(encargo)
                    except ValueError:
                        pass
                    linha += self.getFormatador(caracter=' ', tamanho=24) + '|'
                listagem.imprimeLinha(linha)
        linha_res_pis = '|'
        linha_res_pis += formataString('Tot. CST %s', 40, ' ', 'C', 'N', 'N') % print_cst
        linha_res_pis += formataString(getNumeroEditado(vl_cont_res_pis), 20, ' ', 'E', 'N', 'N')
        linha_res_pis += formataString(getNumeroEditado(vl_item_res_pis), 20, ' ', 'E', 'N', 'N')
        linha_res_pis += formataString(getNumeroEditado(vl_base_res_pis), 20, ' ', 'E', 'N', 'N')
        linha_res_pis += formataString(getNumeroEditado(vl_encargo_res_pis), 20, ' ', 'E', 'N', 'N')
        linha_res_pis += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha(linha_res_pis)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha(separador)
        totais_pis = self.center_title('|', 'TOTAIS DE PIS', '|', 146)
        listagem.imprimeLinha(totais_pis)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        linha_tot_pis = '|'
        linha_tot_pis += formataString('Entradas: ', 40, ' ', 'C', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_cont_ent_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_item_ent_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_base_ent_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_encargo_ent_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(linha_tot_pis)
        linha_tot_pis = '|'
        linha_tot_pis += formataString('Saidas: ', 40, ' ', 'C', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_cont_sai_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_item_sai_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_base_sai_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(vl_encargo_sai_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(linha_tot_pis)
        result_cont_pis = (vl_cont_ent_pis - vl_cont_sai_pis).__abs__()
        result_item_pis = (vl_item_ent_pis - vl_item_sai_pis).__abs__()
        result_base_pis = (vl_base_ent_pis - vl_base_sai_pis).__abs__()
        result_encargo_pis = (vl_encargo_ent_pis - vl_encargo_sai_pis).__abs__()
        linha_tot_pis = '|'
        linha_tot_pis += formataString('Resultado: ', 40, ' ', 'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(result_cont_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(result_item_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(result_base_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += formataString(getNumeroEditado(result_encargo_pis), 20, ' ',
                                       'E', 'N', 'N')
        linha_tot_pis += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(linha_tot_pis)
        listagem.imprimeLinha(separador)

        #Resumo de cofins
        listagem.imprimeLinha('{{ Page }}')
        cabecalho = self.center_title('|', 'Resumo de COFINS agrupado por CST/Natureza', '|', 146)
        listagem.imprimeLinha(separador)
        cabecalho_dados = '|'
        cabecalho_dados += formataString('CST', bytes_pis, ' ', 'C', 'N', 'N')
        cabecalho_dados += formataString('Nat.Rec', bytes_pis, ' ', 'C', 'N', 'N')
        cabecalho_dados += formataString('Valor contabil', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += formataString('Valor do item', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += formataString('Base', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += formataString('Imposto', bytes_pis, ' ', 'E', 'N', 'N')
        cabecalho_dados += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(cabecalho)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha(cabecalho_dados)
        cst_cofins = ''
        vl_cont_res_cofins = 0.00
        vl_item_res_cofins = 0.00
        vl_base_res_cofins = 0.00
        vl_encargo_res_cofins = 0.00
        vl_cont_ent_cofins = 0.00
        vl_item_ent_cofins = 0.00
        vl_base_ent_cofins = 0.00
        vl_encargo_ent_cofins = 0.00
        vl_cont_sai_cofins = 0.00
        vl_item_sai_cofins = 0.00
        vl_base_sai_cofins = 0.00
        vl_encargo_sai_cofins = 0.00

        csts = sorted(list(set(map(lambda x: x.get('cst_codigo'), res))))
        cst_antigo = ''
        print_cst  = ''
        for cst in csts:
            print_cst = cst
            if vl_cont_res_cofins > 0.00:
                linha_res_cofins = '|'
                linha_res_cofins += formataString('Tot. CST %s', 40, ' ', 'C', 'N', 'N') % cst_antigo
                linha_res_cofins += formataString(getNumeroEditado(vl_cont_res_cofins), 20, ' ', 'E', 'N', 'N')
                linha_res_cofins += formataString(getNumeroEditado(vl_item_res_cofins), 20, ' ', 'E', 'N', 'N')
                linha_res_cofins += formataString(getNumeroEditado(vl_base_res_cofins), 20, ' ', 'E', 'N', 'N')
                linha_res_cofins += formataString(getNumeroEditado(vl_encargo_res_cofins), 20, ' ', 'E', 'N', 'N')
                linha_res_cofins += self.getFormatador(caracter=' ', tamanho=24) + '|'
                listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
                listagem.imprimeLinha(linha_res_cofins)
                listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
                listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
                vl_cont_res_cofins = 0.00
                vl_item_res_cofins = 0.00
                vl_base_res_cofins = 0.00
                vl_encargo_res_cofins = 0.00
            naturezas = sorted((set(map(lambda x: x.get('rec_nat') if x.get('cst_codigo') == cst
                                                                else '0', res))))
            for nat in naturezas:
                linha = '|'
                if cst_antigo != cst:
                    linha += formataString(cst or '', bytes_pis, ' ', 'C', 'N', 'N')
                    cst_antigo = cst
                else:
                    linha += formataString(' ', bytes_pis, ' ', 'C', 'N', 'N')
                valor_contabil = sum(map(lambda x: x.get('vr_total_item_rateado') or 0.0 if x.get('cst_codigo') == cst
                                                                                       and x.get('rec_nat') == nat
                                                                                      else 0.0, res))
                if valor_contabil > 0:
                    valor_total_item = sum(map(lambda x: x.get('vr_total_item') or 0.0 if x.get('cst_codigo') == cst
                                                                                      and x.get('rec_nat') == nat
                                                                                     else 0.0, res))
                    base_calculo = sum(map(lambda x: x.get('vr_base_calculo_cofins') or 0.0 if x.get('cst_codigo') == cst
                                                                                    and x.get('rec_nat') == nat
                                                                                   else 0.0, res))
                    encargo = sum(map(lambda x: x.get('vr_encargo_cofins') or 0.0 if x.get('cst_codigo') == cst
                                                                                 and x.get('rec_nat') == nat
                                                                                else 0.0, res))
                    linha += formataString(nat or ' ', bytes_pis, ' ', 'C',
                                           'N', 'N')
                    linha += formataString(getNumeroEditado(valor_contabil),
                                           bytes_pis, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(valor_total_item),
                                           bytes_pis, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(base_calculo), bytes_pis,
                                           ' ',
                                           'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(encargo), bytes_pis,
                                           ' ', 'E', 'N', 'N')
                    vl_cont_res_cofins += float(valor_contabil)
                    vl_item_res_cofins += float(valor_total_item)
                    vl_base_res_cofins += float(base_calculo)
                    vl_encargo_res_cofins += float(encargo)
                    try:
                        if cst.isdigit():
                            if int(cst) > 49:
                                vl_cont_ent_cofins += float(valor_contabil)
                                vl_item_ent_cofins += float(valor_total_item)
                                vl_base_ent_cofins += float(base_calculo)
                                vl_encargo_ent_cofins += float(encargo)
                            else:
                                vl_cont_sai_cofins += float(valor_contabil)
                                vl_item_sai_cofins += float(valor_total_item)
                                vl_base_sai_cofins += float(base_calculo)
                                vl_encargo_sai_cofins += float(encargo)
                    except ValueError:
                        pass
                    linha += self.getFormatador(caracter=' ', tamanho=24) + '|'
                listagem.imprimeLinha(linha)
        linha_res_cofins = '|'
        linha_res_cofins += formataString('Tot. CST %s', 40, ' ', 'C', 'N', 'N') % print_cst
        linha_res_cofins += formataString(getNumeroEditado(vl_cont_res_cofins), 20, ' ', 'E', 'N', 'N')
        linha_res_cofins += formataString(getNumeroEditado(vl_item_res_cofins), 20, ' ', 'E', 'N', 'N')
        linha_res_cofins += formataString(getNumeroEditado(vl_base_res_cofins), 20, ' ', 'E', 'N', 'N')
        linha_res_cofins += formataString(getNumeroEditado(vl_encargo_res_cofins), 20, ' ', 'E', 'N', 'N')

        linha_res_cofins += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha(linha_res_cofins)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        listagem.imprimeLinha(separador)
        totais_cofins = self.center_title('|', 'TOTAIS DE COFINS', '|', 146)
        listagem.imprimeLinha(totais_cofins)
        listagem.imprimeLinha('|' + self.getFormatador(caracter=' ', tamanho=144) + '|')
        linha_tot_cofins = '|'
        linha_tot_cofins += formataString('Entradas: ', 40, ' ', 'C', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_cont_ent_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_item_ent_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_base_ent_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_encargo_ent_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(linha_tot_cofins)
        linha_tot_cofins = '|'
        linha_tot_cofins += formataString('Saidas: ', 40, ' ', 'C', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_cont_sai_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_item_sai_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_base_sai_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(vl_encargo_sai_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(linha_tot_cofins)
        result_cont_cofins = (vl_cont_ent_cofins - vl_cont_sai_cofins).__abs__()
        result_item_cofins = (vl_item_ent_cofins - vl_item_sai_cofins).__abs__()
        result_base_cofins = (vl_base_ent_cofins - vl_base_sai_cofins).__abs__()
        result_encargo_cofins = (vl_encargo_ent_cofins - vl_encargo_sai_cofins).__abs__()
        linha_tot_cofins = '|'
        linha_tot_cofins += formataString('Resultado: ', 40, ' ', 'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(result_cont_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(result_item_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(result_base_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += formataString(getNumeroEditado(result_encargo_cofins), 20, ' ',
                                          'E', 'N', 'N')
        linha_tot_cofins += self.getFormatador(caracter=' ', tamanho=24) + '|'
        listagem.imprimeLinha(linha_tot_cofins)
        listagem.imprimeLinha(separador)

        return True

    def consulta_nota(self, cr, uid, data_ini, data_fim, tipo_mov, company_ids, filtro,
                      imprimir_item_nat_rec_diferentes=False, imprimir_item_sem_nat_rec=False):
        sql = '''
                SELECT id
                FROM   fiscal_encargo
                WHERE  sigla IN ( 'PIS', 'COFINS' )
            '''
        cr.execute(sql)
        encargo_ids = ', '.join(map(lambda x: str(x.get('id')), cr.dictfetchall()))
        query_tipo = """AND fnc.tipo = '%s' """ % tipo_mov
        client_fornec = filtro.get('wiz_client_fornec')
        nf = filtro.get('wiz_nf_numero')
        serie = filtro.get('wiz_nf_serie')

        cfop_ids = []
        query_cfop_id = ''
        query_cfop_natureza_id = ''
        if filtro.get('wiz_cfop_ids'):
            if len(filtro.get('wiz_cfop_ids')) == 1:
                cfop_ids = str(tuple(filtro.get('wiz_cfop_ids'))).replace(',', '')
            else:
                cfop_ids = str(tuple(filtro.get('wiz_cfop_ids')))
            natureza_ids = pooler.get_pool(cr.dbname).get('fiscal.cfop.natureza').search(cr, uid, [
                ('cfop_id', 'in', filtro.get('wiz_cfop_ids'))])
            query_cfop_id = """AND emi.cfop_id IN %s """ % cfop_ids
            if natureza_ids:
                query_cfop_natureza_id = """
                AND fnc.natureza_id IN {natureza_ids}
                """.format(natureza_ids=str(natureza_ids).replace('[', '(').replace(']', ')'))
        query_cupom = ''
        if ecf_instalado:
            query_cupom = '''AND emi.cupom_id IS NULL '''

        # Filtro de cliente/fornecedor
        query_pessoa_id = ''' '''
        if client_fornec:
            pessoa_id = client_fornec[0]
            query_pessoa_id = """AND fnc.destinatario_id = %d """ % pessoa_id

        # Filtro de Nota Fiscal
        query_nf = ''
        if nf > 0:
            query_nf = """AND fnc.numero_nf = %d
                  AND fnc.serie_nf = '%s' """ % (nf, serie)

        # Filtro de busca de CFOPs que sao considerados no SPED Contribuicoes
        cfop_considerado_sped_contribuicoes = ''
        if filtro.get('wiz_cfop_sped_contribuicoes'):
            cfop_considerado_sped_contribuicoes = """
                            AND cfop.fiscfop_sped_contribuicoes IS TRUE            
            """

        item_nat_rec_diferente = ''
        if imprimir_item_nat_rec_diferentes:
            item_nat_rec_diferente = self.get_query_item_nat_rec_diferente()

        item_sem_nat_rec = ''
        if imprimir_item_sem_nat_rec:
            item_sem_nat_rec = self.get_query_item_sem_nat_rec()

        sql = """
        SELECT id,
               codigo_cliente                                 AS codigo_cliente,
               client_fornec                                  AS client_fornec,
               cnae          		                          AS cnae,
               numero                                         AS numero,
               serie                                          AS serie,
               data_emissao                                   AS data_emissao,
               cfop                                           AS cfop,
               modelo_tipo                                    AS modelo_tipo,
               sum(vr_total_nota)                             AS vr_total_nota,
               sum(vr_total_produtos)                         AS vr_total_produtos,
               aliq_pis                                       AS aliq_pis,
               sum(pis)                                       AS pis,
               aliq_cofins                                    AS aliq_cofins,
               sum(cofins)                                    AS cofins
        FROM (
                SELECT id,
                       codigo_cliente                                 AS codigo_cliente,
                       client_fornec                                  AS client_fornec,
                       cnae          		                          AS cnae,
                       numero                                         AS numero,
                       serie                                          AS serie,
                       data_emissao                                   AS data_emissao,
                       cfop                                           AS cfop,
                       modelo_tipo                                    AS modelo_tipo,
                       vr_total_nota                                  AS vr_total_nota,
                       vr_total_produtos                              AS vr_total_produtos,
                       sum(aliq_pis)                                  AS aliq_pis,
                       sum(pis)                                       AS pis,
                       sum(aliq_cofins)                               AS aliq_cofins,
                       sum(cofins)                                    AS cofins
                FROM (
                        SELECT fnc.id,
                               emi.id	                                      AS emi_id,
                               admp.codigo                                    AS codigo_cliente,
                               admp.nome                                      AS client_fornec,
                               cnae.admcna_codigo                             AS cnae,
                               fnc.numero_nf                                  AS numero,
                               fnc.serie_nf                                   AS serie,
                               CASE
                                 WHEN fnc.tipo = 'E' THEN fnc.data_saida_entrada
                                 ELSE fnc.data_emissao
                               END                                            AS data_emissao,
                               cfop.codigo                                    AS cfop,
                               modelo.fismnf_tipo                             AS modelo_tipo,
                               emi.vr_total_item_rateado                      AS vr_total_nota,
                               emi.vr_total_item - 
                                   (SELECT sum(COALESCE(fisnit_vr_icms_desonerado, 0))
                                    FROM fiscal_nf_item_tributacao
                                    WHERE movimento_itens_id = emi.id)
                                                                              AS vr_total_produtos,
                               CASE WHEN encargo.sigla = 'PIS'
                                    THEN fnit.aliquota
                               END                                            AS aliq_pis,
                               CASE WHEN encargo.sigla = 'PIS'
                                    THEN Sum(fnit.vr_encargo)
                               END                                            AS pis,
                               CASE WHEN encargo.sigla = 'COFINS'
                                    THEN fnit.aliquota
                               END                                            AS aliq_cofins,
                               CASE WHEN encargo.sigla = 'COFINS'
                                    THEN Sum(fnit.vr_encargo)
                               END                                            AS cofins
                        FROM   fiscal_nf_capa                AS fnc
                        LEFT  JOIN adm_pessoa                AS admp    ON admp.id = fnc.destinatario_id
                        LEFT  JOIN adm_cnae                  AS cnae    ON cnae.id = admp.admpes_jur_cnae_id
                        INNER JOIN fiscal_cfop_natureza      AS fcn     ON fcn.id = fnc.natureza_id
                        INNER JOIN fiscal_nf_item_tributacao AS fnit    ON fnit.fisnit_capa_id = fnc.id
                        INNER JOIN estoque_movimento_itens   AS emi     ON emi.id = fnit.movimento_itens_id
                        INNER JOIN fiscal_cfop               AS cfop    ON cfop.id = emi.cfop_id
                        INNER JOIN fiscal_cst                AS cst     ON cst.id = fnit.cst_id
                        INNER JOIN fiscal_encargo            AS encargo ON encargo.id = fnit.encargo_id
                        INNER JOIN fiscal_modelo_nf          AS modelo  ON modelo.id = fnc.modelo_nf_id
                        LEFT  JOIN estoque_produto           AS ep      ON ep.id = emi.produto_id
                        WHERE
                          CASE
                            WHEN fnc.tipo = 'E' THEN fnc.data_saida_entrada BETWEEN '{data_ini}' AND '{data_fim} 23:59:59'
                            ELSE fnc.data_emissao BETWEEN '{data_ini}' AND '{data_fim} 23:59:59'
                          END
                          AND fnc.state = '3'
                          AND fnc.company_id IN {company_ids}
                          AND fnit.encargo_id IN ({encargo_ids})
                          AND modelo.fismnf_codigo NOT LIKE '9%'
                          {query_cupom}
                          {query_tipo}
                          {query_nf}
                          {query_pessoa_id}
                          {query_cfop_id}
                          {cfop_sped_contribuicoes}
                          AND cfop.codigo NOT IN ('5929', '6929')
                          {item_nat_rec_diferente}
                          {item_sem_nat_rec}
                        GROUP  BY fnc.id,
                                  emi.id,
                                  fnc.numero_nf,
                                  fnc.serie_nf,
                                  admp.codigo,
                                  admp.nome,
                                  cnae.admcna_codigo,
                                  cfop.codigo,
                                  modelo.fismnf_tipo,
                                  fnc.vr_total_nf,
                                  fnc.vr_total_produtos,
                                  fnit.aliquota,
                                  aliq_pis,
                                  aliq_cofins,
                                  fnc.data_emissao,
                                  encargo.sigla
                ) AS tmp
                GROUP  BY id,
                          emi_id,
                          codigo_cliente,
                          client_fornec,
                          cnae,
                          numero,
                          serie,
                          data_emissao,
                          cfop,
                          modelo_tipo,
                          vr_total_nota,
                          vr_total_produtos
          ) AS tmp2
        GROUP  BY id,
              codigo_cliente,
              client_fornec,
              cnae,
              aliq_pis,
              aliq_cofins,
              numero,
              serie,
              data_emissao,
              cfop,
              modelo_tipo
        ORDER  BY cfop asc,
                  id ASC
                """.format(data_ini=data_ini,
                           data_fim=data_fim,
                           company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                           encargo_ids=encargo_ids,
                           query_cupom=query_cupom,
                           query_tipo=query_tipo,
                           query_nf=query_nf,
                           query_pessoa_id=query_pessoa_id,
                           query_cfop_id=query_cfop_id,
                           cfop_sped_contribuicoes=cfop_considerado_sped_contribuicoes,
                           item_nat_rec_diferente=item_nat_rec_diferente,
                           item_sem_nat_rec=item_sem_nat_rec)
        cr.execute(sql)
        dados = cr.dictfetchall()

        # notas que nao possuem itens ex:(nota com frete vinculado a nota)
        sql = '''
            SELECT id,
                   codigo_cliente                                 AS codigo_cliente,
                   client_fornec                                  AS client_fornec,
                   cnae          		                      AS cnae,
                   numero                                         AS numero,
                   serie                                          AS serie,
                   data_emissao                                   AS data_emissao,
                   cfop                                           AS cfop,
                   vr_total_nota                                  AS vr_total_nota,
                   vr_total_produtos                              AS vr_total_produtos,
                   sum(aliq_pis)                                  AS aliq_pis,
                   sum(pis)                                       AS pis,
                   sum(aliq_cofins)                               AS aliq_cofins,
                   sum(cofins)                                    AS cofins
            FROM (
                SELECT fnc.id,
                       admp.codigo                                    AS codigo_cliente,
                       admp.nome                                      AS client_fornec,
                       cnae.admcna_codigo                             AS cnae,
                       fnc.numero_nf                                  AS numero,
                       fnc.serie_nf                                   AS serie,
                       fnc.data_emissao                               AS data_emissao,
                       cfop.codigo                                    AS cfop,
                       fnc.vr_total_nf                                AS vr_total_nota,
                       fnc.vr_total_nf                                AS vr_total_produtos,
                       CASE WHEN encargo.sigla = 'PIS'
                            THEN fnit.aliquota
                       END                                            AS aliq_pis,
                       CASE WHEN encargo.sigla = 'PIS'
                            THEN Sum(fnit.vr_encargo)
                       END                                            AS pis,
                       CASE WHEN encargo.sigla = 'COFINS'
                            THEN fnit.aliquota
                       END                                            AS aliq_cofins,
                       CASE WHEN encargo.sigla = 'COFINS'
                            THEN Sum(fnit.vr_encargo)
                       END                                            AS cofins
                FROM   fiscal_nf_capa                AS fnc
                INNER JOIN adm_pessoa                AS admp    ON admp.id = fnc.destinatario_id
                LEFT  JOIN adm_cnae                  AS cnae    ON cnae.id = admp.admpes_jur_cnae_id
                INNER JOIN fiscal_cfop_natureza      AS fcn     ON fcn.id = fnc.natureza_id
                INNER JOIN fiscal_cfop               AS cfop    ON cfop.id = fcn.cfop_id
                INNER JOIN fiscal_nf_item_tributacao AS fnit    ON fnit.fisnit_capa_id = fnc.id
                INNER JOIN fiscal_cst                AS cst     ON cst.id = fnit.cst_id
                INNER JOIN fiscal_encargo            AS encargo ON encargo.id = fnit.encargo_id
                WHERE
                CASE
                  WHEN fnc.tipo = 'S' THEN
                       fnc.data_emissao BETWEEN '{data_ini}' AND '{data_fim} 23:59:59'
                  ELSE fnc.data_saida_entrada BETWEEN '{data_ini}' AND '{data_fim} 23:59:59'
                END
                  AND fnit.movimento_itens_id IS NULL
                  AND fnc.state = '3'
                  AND fnc.company_id IN {company_ids}
                  AND fnit.encargo_id IN ({encargo_ids})
                  {query_tipo}
                  {query_nf}
                  {query_pessoa_id}
                  {query_cfop_natureza_id}
                  {cfop_sped_contribuicoes}
                GROUP  BY fnc.id,
                      fnc.numero_nf,
                      fnc.serie_nf,
                      admp.codigo,
                      admp.nome,
                      cnae.admcna_codigo,
                      cfop.codigo,
                      fnc.vr_total_nf,
                      fnc.vr_total_produtos,
                      aliq_pis,
                      aliq_cofins,
                      fnc.data_emissao,
                      encargo.sigla
                  ) AS tmp
            GROUP  BY id,
                  codigo_cliente,
                  client_fornec,
                  cnae,
                  numero,
                  serie,
                  data_emissao,
                  cfop,
                  vr_total_nota,
                  vr_total_produtos
            ORDER  BY cfop asc,
                  id ASC
                '''.format(data_ini=data_ini,
                           data_fim=data_fim,
                           company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                           encargo_ids=encargo_ids,
                           query_tipo=query_tipo,
                           query_nf=query_nf,
                           query_pessoa_id=query_pessoa_id,
                           query_cfop_natureza_id=query_cfop_natureza_id,
                           cfop_sped_contribuicoes=cfop_considerado_sped_contribuicoes)
        if filtro.get('wiz_opcao') != 'I':
            cr.execute(sql)
            dados.extend(cr.dictfetchall())
        dados = sorted(dados, key=lambda x: (
            x.get('numero'), x.get('serie'), x.get('aliq_pis'), x.get('aliq_dofins')))
        return dados

    def consulta_cupons(self, cr, uid, data_ini, data_fim, company_ids, cfop_ids=[],
                        cfop_considerado_sped_contribuicoes=False,
                        imprimir_item_nat_rec_diferentes=False, imprimir_item_sem_nat_rec=False):
        sql = '''
                SELECT id
                FROM   fiscal_encargo
                WHERE  sigla IN ( 'PIS', 'COFINS' )
            '''
        cr.execute(sql)
        encargo_ids = ', '.join(map(lambda x: str(x.get('id')), cr.dictfetchall()))
        query_cfop = ''
        if cfop_ids:
            if len(cfop_ids) == 1:
                cfop_ids = str(tuple(cfop_ids)).replace(',', '')
            else:
                cfop_ids = str(tuple(cfop_ids))
            query_cfop = 'AND fc.id IN {cfop_id}'.format(cfop_id=cfop_ids)
        # Filtro de busca de CFOPs que sao considerados no SPED Contribuicoes
        cfop_sped_contribuicoes = ''
        if cfop_considerado_sped_contribuicoes:
            cfop_sped_contribuicoes = """
                                AND fc.fiscfop_sped_contribuicoes IS TRUE            
            """

        item_nat_rec_diferente = ''
        if imprimir_item_nat_rec_diferentes:
            item_nat_rec_diferente = self.get_query_item_nat_rec_diferente()

        item_sem_nat_rec = ''
        if imprimir_item_sem_nat_rec:
            item_sem_nat_rec = self.get_query_item_sem_nat_rec()

        sql = """
            SELECT ecf_id           AS ecf_id,
                   ecf              AS ecf,
                   data             AS data,
                   cfop             AS cfop,
                   sum(vr_total)    AS vr_total,
                   sum(vr_total_item) AS vr_total_item,
                   COALESCE(aliq_pis, 0.0) AS aliq_pis,
                   Sum(pis)         AS pis,
                   COALESCE(aliq_cofins, 0.0) AS aliq_cofins,
                   Sum(cofins)      AS cofins
            FROM (

                    SELECT ecf_id2           AS ecf_id,
                           ecf2              AS ecf,
                           data2             AS data,
                           cfop2             AS cfop,
                           vr_total2         AS vr_total,
                           vr_total_item2    AS vr_total_item,
                           Sum(aliq_pis2)    AS aliq_pis,
                           Sum(pis2)         AS pis,
                           Sum(aliq_cofins2) AS aliq_cofins,
                           Sum(cofins2)      AS cofins
                    FROM ( SELECT ee.id                                             AS ecf_id2,
                                  ee.caixa                                          AS ecf2,
                                  ecc.date_cupom                                    AS data2,
                                  fc.codigo                                         AS cfop2,
                                  Sum(CASE WHEN ecc.tipo_cupom = 'C'
                                           THEN -emi.vr_total_item_rateado
                                           ELSE emi.vr_total_item_rateado
                                      END)                                          AS vr_total2,
                                  Sum(CASE WHEN ecc.tipo_cupom = 'C'
                                           THEN -emi.vr_total_item
                                           ELSE emi.vr_total_item
                                      END)                                          AS vr_total_item2,
                                  CASE WHEN fe.sigla = 'PIS'
                                       THEN COALESCE(fnit.aliquota, 0.0)
                                       ELSE 0.0
                                  END                                               AS aliq_pis2,
                                  Sum(CASE WHEN fe.sigla = 'PIS' AND ecc.tipo_cupom = 'C'
                                           THEN COALESCE(-fnit.vr_encargo, 0.0)
                                           WHEN fe.sigla = 'PIS'
                                           THEN COALESCE(fnit.vr_encargo, 0.0)
                                           ELSE 0.0
                                       END)                                         AS pis2,
                                  CASE WHEN fe.sigla = 'COFINS'
                                       THEN COALESCE(fnit.aliquota, 0.0)
                                       ELSE 0.0
                                  END                                               AS aliq_cofins2,
                                  Sum(CASE WHEN fe.sigla = 'COFINS' AND ecc.tipo_cupom = 'C'
                                           THEN COALESCE(-fnit.vr_encargo, 0.0)
                                           WHEN fe.sigla = 'COFINS'
                                           THEN COALESCE(fnit.vr_encargo, 0.0)
                                           ELSE 0.0
                                       END)                                         AS cofins2
                           FROM   estoque_movimento_itens       AS emi
                           INNER JOIN estoque_movimento_capa    AS emc  ON emc.id =  emi.capa_id
                           INNER JOIN ecf_cupons_capa           AS ecc  ON ecc.id = emi.cupom_id
                           INNER JOIN ecf_ecf                   AS ee   ON ee.id = ecc.ecf_id
                           INNER JOIN fiscal_cfop               AS fc   ON fc.id = emi.cfop_id
                           INNER JOIN fiscal_nf_item_tributacao AS fnit ON fnit.movimento_itens_id = emi.id
                           INNER JOIN fiscal_encargo            AS fe   ON fe.id = fnit.encargo_id
                           LEFT  JOIN estoque_produto           AS ep   ON ep.id = emi.produto_id
                           WHERE  emc.data >= '{data_ini}'
                              AND emc.data <= '{data_fim}'
                              AND emi.nf_id IS NULL
                              AND fnit.encargo_id IN ({encargo_ids})
                              AND ecc.tipo_cupom IN ('V', 'C')
                              AND ecc.state = 'V'
                              AND not (emi.item_cancelado is True AND  ecc.tipo_cupom = 'V')
                              AND fc.codigo not in ('5929', '6929') {cfop_id}
                              AND emc.company_id IN {company_ids}
                              {cfop_sped_contribuicoes}
                              {item_nat_rec_diferente}
                              {item_sem_nat_rec}
                           GROUP  BY ee.id,
                                     aliq_pis2,
                                     aliq_cofins2,
                                     data2,
                                     cfop2,
                                     fe.sigla,
                                     ecc.tipo_cupom
                       ) AS tmp
                    GROUP  BY ecf_id,
                              ecf,
                              data2,
                              cfop2,
                              vr_total2,
                              vr_total_item2

                ) as tpm2
            GROUP  BY ecf_id,
                      ecf,
                      data,
                      cfop,
                      aliq_pis,
                      aliq_cofins
        """.format(data_ini=data_ini,
                   data_fim=data_fim,
                   company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                   encargo_ids=encargo_ids, cfop_id=query_cfop,
                   cfop_sped_contribuicoes=cfop_sped_contribuicoes,
                   item_nat_rec_diferente=item_nat_rec_diferente,
                   item_sem_nat_rec=item_sem_nat_rec)
        cr.execute(sql)
        dados = cr.dictfetchall()
        dados = sorted(dados, key=lambda x: (
            x.get('ecf'), x.get('data'), x.get('aliq_pis'), x.get('aliq_dofins'), x.get('cfop')))
        return dados

    def _consulta_itens_nota(self, cr, nota_ids, cst, natureza, aliq_pis, aliq_cofins, cfop_ids,
                             data_min, data_max, company_ids, cfop_considerado_sped_contribuicoes,
                             imprimir_item_nat_rec_diferentes=False,
                             imprimir_item_sem_nat_rec=False):
        nota_ids = nota_ids.replace('[', '').replace(']', '')
        aliq_pis = aliq_pis.replace('[', '').replace(']', '')
        aliq_cofins = aliq_cofins.replace('[', '').replace(']', '')
        data_min = data_min[0:10] + ' 00:00:00'
        # Filtro cfop
        query_cfop = ''
        if cfop_ids:
            if len(cfop_ids) == 1:
                cfop_ids = str(tuple(cfop_ids)).replace(',', '')
            else:
                cfop_ids = str(tuple(cfop_ids))
            query_cfop = '''AND cfop.id IN %s''' % (cfop_ids)

        # Filtro cst
        query_cst = ''
        if cst:
            if len(cst) == 1:
                cst = str(tuple(cst)).replace(',', '')
            else:
                cst = str(tuple(cst))
            query_cst = """AND cst.codigo IN %s """ % cst

        # Filtro de busca de CFOPs que sao considerados no SPED Contribuicoes
        cfop_sped_contribuicoes = ''
        if cfop_considerado_sped_contribuicoes:
            cfop_sped_contribuicoes = """
                            AND cfop.fiscfop_sped_contribuicoes IS TRUE            
            """

        sql = '''
                    SELECT id
                    FROM   fiscal_encargo
                    WHERE  sigla IN ( 'PIS', 'COFINS' )
                '''
        cr.execute(sql)
        encargo_ids = ', '.join(map(lambda x: str(x.get('id')), cr.dictfetchall()))

        item_nat_rec_diferentes = ''
        if imprimir_item_nat_rec_diferentes:
            item_nat_rec_diferentes = self.get_query_item_nat_rec_diferente()

        item_sem_nat_rec = ''
        if imprimir_item_sem_nat_rec:
            item_sem_nat_rec = self.get_query_item_sem_nat_rec()

        sql = """
        SELECT nf_id,
               mov_item_id,
               data_emissao,
               codigo                         AS codigo,
               ncm                            AS ncm,
               cfop                           AS cfop,
               digito                         AS digito,
               grade                          AS grade,
               descricao                      AS descricao,
               SUM(vr_total_item_rateado)     AS vr_total_item_rateado,
               SUM(vr_total_item)             AS vr_total_item,
               False                          AS sem_item
        FROM (
                SELECT nf_id,
                       mov_item_id,
                       data_emissao,
                       codigo                         AS codigo,
                       ncm                            AS ncm,
                       cfop                           AS cfop,
                       digito                         AS digito,
                       grade                          AS grade,
                       descricao                      AS descricao,
                       vr_total_item_rateado          AS vr_total_item_rateado,
                       vr_total_item                  AS vr_total_item
                FROM (
                        SELECT fnc.id                                                  AS nf_id,
                               emi.id                                                  AS mov_item_id,
                               CASE
                                 WHEN fnc.tipo = 'E' THEN fnc.data_saida_entrada
                                 ELSE fnc.data_emissao
                               END                                                     AS data_emissao,
                               ep.codigo                                               AS codigo,
                               ncm.codigo                                              AS ncm,
                               cfop.codigo                                             AS cfop,
                               ep.digito                                               AS digito,
                               epg.grade                                               AS grade,
                               ep.name                                                 AS descricao,
                               SUM(emi.vr_total_item_rateado) 	                       AS vr_total_item_rateado,
                               SUM(emi.vr_total_item) - 
                                   (SELECT sum(COALESCE(fisnit_vr_icms_desonerado, 0))
                                    FROM fiscal_nf_item_tributacao
                                    WHERE movimento_itens_id = emi.id)                 AS vr_total_item
                        FROM   estoque_movimento_itens       AS emi
                        INNER JOIN estoque_produto           AS ep   ON ep.id = emi.produto_id
                        INNER JOIN estoque_produto_grade     AS epg  ON epg.id = emi.grade_id
                        INNER JOIN fiscal_nf_capa            AS fnc  ON fnc.id = emi.nf_id
                        LEFT  JOIN fiscal_ncm                AS ncm  ON ncm.id = ep.ncm_id
                        INNER JOIN fiscal_cfop               AS cfop ON cfop.id = emi.cfop_id
                        LEFT  JOIN fiscal_nf_item_tributacao AS fnit ON emi.id = fnit.movimento_itens_id
                        LEFT  JOIN fiscal_encargo            AS fe   ON fe.id = fnit.encargo_id
                        WHERE
                           CASE
                             WHEN fnc.tipo = 'E' THEN fnc.data_saida_entrada BETWEEN '{data_min}' AND '{data_max}'
                             ELSE fnc.data_emissao BETWEEN '{data_min}' AND '{data_max}'
                           END
                          AND fnc.id IN ({nota_ids})
                          AND fe.sigla = 'PIS'
                          {query_cfop}
                          AND fnc.company_id in {company_ids}
                          {cfop_sped_contribuicoes}
                          {item_nat_rec_diferentes}
                          {item_sem_nat_rec}
                        GROUP  BY fnc.id,
                                  emi.id,
                                  fnc.data_emissao,
                                  ep.codigo,
                                  ncm.codigo,
                                  cfop.codigo,
                                  ep.digito,
                                  epg.grade,
                                  ep.name,
                                  emi.vr_total_item_rateado,
                                  emi.vr_total_item
                    ) AS tmp
                GROUP  BY  nf_id,
                           mov_item_id,
                           data_emissao,
                           codigo,
                           ncm,
                           cfop,
                           digito,
                           grade,
                           descricao,
                           vr_total_item_rateado,
                           vr_total_item
            ) AS tmp2
        GROUP  BY  nf_id,
                   mov_item_id,
                   data_emissao,
                   codigo,
                   ncm,
                   cfop,
                   digito,
                   grade,
                   descricao
        UNION ALL

        ----------- NOTAS SEM ITENS -------------
        SELECT nf_id,
               mov_item_id,
               data_emissao,
               ''                    AS codigo,
               ''                    AS ncm,
               cfop                  AS cfop,
               ''                    AS digito,
               ''                    AS grade,
               ''                    AS descricao,
               vr_total_item_rateado AS vr_total_item_rateado,
               vr_total_item         AS vr_total_item,
               true                  AS sem_item
        FROM   (SELECT mov_item_id,
                       nf_id,
                       data_emissao,
                       cfop                       AS cfop,
                       cst_codigo                 AS cst_codigo,
                       Sum(vr_total_item_rateado) AS vr_total_item_rateado,
                       Sum(vr_total_item)         AS vr_total_item,
                       Sum(vr_base_calculo)       AS vr_base_calculo,
                       aliquota_pis               AS aliquota_pis,
                       aliquota_cofins            AS aliquota_cofins,
                       Sum(vr_encargo_pis)        AS vr_encargo_pis,
                       Sum(vr_encargo_cofins)     AS vr_encargo_cofins,
                       rec_nat                    AS rec_nat
                FROM   (SELECT mov_item_id,
                               nf_id,
                               data_emissao,
                               cfop                   AS cfop,
                               cst_codigo             AS cst_codigo,
                               vr_total_item_rateado  AS vr_total_item_rateado,
                               vr_total_item          AS vr_total_item,
                               vr_base_calculo        AS vr_base_calculo,
                               Sum(aliquota_pis)      AS aliquota_pis,
                               Sum(aliquota_cofins)   AS aliquota_cofins,
                               Sum(vr_encargo_pis)    AS vr_encargo_pis,
                               Sum(vr_encargo_cofins) AS vr_encargo_cofins,
                               rec_nat                AS rec_nat
                        FROM   (SELECT fnit.movimento_itens_id        AS mov_item_id,
                                       fnc.id                         AS nf_id,
                                       fnc.data_emissao               AS data_emissao,
                                       cfop.codigo                    AS cfop,
                                       cst.codigo                     AS cst_codigo,
                                       fnc.vr_total_nf                AS
                                       vr_total_item_rateado,
                                       fnc.vr_total_nf                AS vr_total_item,
                                       fnit.vr_base_calculo           AS vr_base_calculo
                                       ,
                                       fe.sigla
                                       AS encargo_sigla,
                                       CASE
                                         WHEN fe.sigla = 'PIS'
                                              AND fnit.aliquota IN ( {aliq_pis} ) THEN
                                         fnit.aliquota
                                       END                            AS aliquota_pis,
                                       CASE
                                         WHEN fe.sigla = 'COFINS'
                                              AND fnit.aliquota IN ( {aliq_cofins} ) THEN
                                         fnit.aliquota
                                       END                            AS aliquota_cofins
                                       ,
                                       CASE
                                         WHEN fe.sigla = 'PIS'
                                              AND fnit.aliquota IN ( {aliq_pis} ) THEN
                                         Sum(fnit.vr_encargo)
                                       END                            AS vr_encargo_pis,
                                       CASE
                                         WHEN fe.sigla = 'COFINS'
                                              AND fnit.aliquota IN ( {aliq_cofins} ) THEN
                                         Sum(fnit.vr_encargo)
                                       END                            AS
                                       vr_encargo_cofins,
                                       fnit.fisnit_nat_rec_pis_cofins AS rec_nat
                                FROM   fiscal_nf_item_tributacao AS fnit
                                       INNER JOIN fiscal_nf_capa AS fnc
                                               ON fnc.id = fnit.fisnit_capa_id
                                       INNER JOIN fiscal_cfop_natureza AS fcn
                                               ON fcn.id = fnc.natureza_id
                                       INNER JOIN fiscal_cfop AS cfop
                                               ON cfop.id = fcn.cfop_id
                                       INNER JOIN fiscal_cst AS cst
                                               ON cst.id = fnit.cst_id
                                       INNER JOIN fiscal_encargo AS fe
                                               ON fe.id = fnit.encargo_id
                          WHERE fnc.data_emissao >= '{data_min}'
                                AND fnc.data_emissao <= '{data_max}'
                                       AND fnit.fisnit_capa_id IN ({nota_ids})
                                       AND fnit.movimento_itens_id IS NULL
                                       AND fnit.encargo_id IN ({encargo_ids})
                                       AND fnc.company_id IN {company_ids}
                                       {query_cfop}
                                       {query_cst}
                                       {cfop_sped_contribuicoes}
                                GROUP  BY fnit.movimento_itens_id,
                                          fnc.id,
                                          fnc.data_emissao,
                                          cfop,
                                          cst_codigo,
                                          fnit.vr_encargo,
                                          aliquota,
                                          rec_nat,
                                          fe.sigla,
                                          fnc.vr_total_nf,
                                          fnc.vr_total_produtos,
                                          fnit.vr_base_calculo) AS tmp3
                        GROUP  BY nf_id,
                                  mov_item_id,
                                  data_emissao,
                                  cfop,
                                  cst_codigo,
                                  vr_total_item_rateado,
                                  vr_total_item,
                                  vr_base_calculo,
                                  rec_nat) AS tmp4
                GROUP  BY nf_id,
                          mov_item_id,
                          data_emissao,
                          cfop,
                          cst_codigo,
                          rec_nat,
                          aliquota_pis,
                          aliquota_cofins) AS tmp5

        ORDER BY nf_id, data_emissao, cfop, descricao, grade
            """.format(aliq_pis=aliq_pis,
                       aliq_cofins=aliq_cofins,
                       nota_ids=nota_ids,
                       encargo_ids=encargo_ids,
                       query_cst=query_cst,
                       query_cfop=query_cfop,
                       data_min=data_min,
                       data_max=data_max,
                       company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                       cfop_sped_contribuicoes=cfop_sped_contribuicoes,
                       item_nat_rec_diferentes=item_nat_rec_diferentes,
                       item_sem_nat_rec=item_sem_nat_rec)
        cr.execute(sql)
        dados = cr.dictfetchall()
        res = {}
        for i in dados:
            if not res.get(i.get('nf_id')):
                res[i.get('nf_id')] = [i]
            else:
                res[i.get('nf_id')].append(i)

        return res

    def _contulta_itens_cupom(self, cr, ecf_id, aliq_pis, aliq_cofins, natureza, cst, cfop,
                              data_cupom, company_ids, imprimir_item_nat_rec_diferentes=False,
                              imprimir_item_sem_nat_rec=False):
        ecf_id = ecf_id.replace('[', '').replace(']', '')
        aliq_pis = aliq_pis.replace('[', '').replace(']', '')
        aliq_cofins = aliq_cofins.replace('[', '').replace(']', '')
        cfop = cfop.replace('[', '').replace(']', '')
        data_cupom = data_cupom.replace('[', '').replace(']', '')
        # Filtro da natureza
        query_natureza = ''
        if natureza:
            query_natureza = """AND fnit.fisnit_nat_rec_pis_cofins = '%s' """ % natureza
        # Filtro cst
        query_cst = ''
        if cst:
            if len(cst) == 1:
                cst = str(tuple(cst)).replace(',', '')
            else:
                cst = str(tuple(cst))
            query_cst = """AND cst.codigo IN %s """ % cst

        sql = '''
                SELECT id
                FROM   fiscal_encargo
                WHERE  sigla IN ( 'PIS', 'COFINS' )
            '''
        cr.execute(sql)
        encargo_ids = ', '.join(map(lambda x: str(x.get('id')), cr.dictfetchall()))

        item_nat_rec_diferentes = ''
        if imprimir_item_nat_rec_diferentes:
            item_nat_rec_diferentes = self.get_query_item_nat_rec_diferente()

        item_sem_nat_rec = ''
        if imprimir_item_sem_nat_rec:
            item_sem_nat_rec = """
                AND fnit.fisnit_nat_rec_pis_cofins IN (' ', '0', '00', '000', NULL)
            """

        sql = """
        SELECT ecf_id,
               data,
               codigo                     AS codigo,
               ncm                        AS ncm,
               cfop                       AS cfop,
               cst_codigo 			      AS cst_codigo,
               digito                     AS digito,
               grade                      AS grade,
               descricao                  AS descricao,
               SUM(vr_total_item_rateado) AS vr_total_item_rateado,
               SUM(vr_total_item)         AS vr_total_item,
               SUM(vr_base_calculo)       AS vr_base_calculo,
               aliquota_pis               AS aliquota_pis,
               SUM(vr_encargo_pis)        AS vr_encargo_pis,
               aliquota_cofins            AS aliquota_cofins,
               SUM(vr_encargo_cofins)     AS vr_encargo_cofins,
               rec_nat                    AS rec_nat
        FROM (

                SELECT ecf_id,
                       data,
                       codigo                 AS codigo,
                       ncm                    AS ncm,
                       cfop                   AS cfop,
                       cst_codigo 			  AS cst_codigo,
                       digito                 AS digito,
                       grade                  AS grade,
                       descricao              AS descricao,
                       vr_total_item_rateado  AS vr_total_item_rateado,
                       vr_total_item          AS vr_total_item,
                       vr_base_calculo        AS vr_base_calculo,
                       SUM(aliquota_pis)      AS aliquota_pis,
                       SUM(vr_encargo_pis)    AS vr_encargo_pis,
                       SUM(aliquota_cofins)   AS aliquota_cofins,
                       SUM(vr_encargo_cofins) AS vr_encargo_cofins,
                       rec_nat                AS rec_nat
                FROM (  SELECT ecc.ecf_id,
                               ecc.tipo_cupom,
                               emi.item_cancelado,
                               emc.data,
                               ep.codigo                                               AS codigo,
                               ncm.codigo                                              AS ncm,
                               cfop.codigo                                             AS cfop,
                               cst.codigo 					                           AS cst_codigo,
                               ep.digito                                               AS digito,
                               epg.grade                                               AS grade,
                               ep.name                                                 AS descricao,
                               SUM(CASE WHEN ecc.tipo_cupom = 'C'
                                        THEN -emi.vr_total_item_rateado
                                        ELSE emi.vr_total_item_rateado
                                   END)     	                    		           AS vr_total_item_rateado,
                               SUM(CASE WHEN ecc.tipo_cupom = 'C'
                                        THEN -emi.vr_total_item
                                        ELSE emi.vr_total_item
                                   END)     	                    		           AS vr_total_item,
                               SUM(CASE WHEN ecc.tipo_cupom = 'C'
                                        THEN -fnit.vr_base_calculo
                                        ELSE fnit.vr_base_calculo
                                   END)     	                    		           AS vr_base_calculo,
                               fe.sigla						                           AS encargo_sigla,
                               CASE WHEN fe.sigla = 'PIS'        AND fnit.aliquota in ({aliq_pis})
                                    THEN fnit.aliquota
                               END						                               AS aliquota_pis,
                               CASE WHEN fe.sigla = 'COFINS'     AND fnit.aliquota in ({aliq_cofins})
                                    THEN fnit.aliquota
                               END               				                       AS aliquota_cofins,
                               SUM(CASE WHEN fe.sigla = 'PIS'    AND fnit.aliquota in ({aliq_pis})
                                                                 AND ecc.tipo_cupom = 'C'
                                        THEN -fnit.vr_encargo
                                        WHEN fe.sigla = 'PIS'    AND fnit.aliquota in ({aliq_pis})
                                        THEN fnit.vr_encargo
                                   END)					                               AS vr_encargo_pis,
                               SUM(CASE WHEN fe.sigla = 'COFINS' AND fnit.aliquota in ({aliq_cofins})
                                                                 AND ecc.tipo_cupom = 'C'
                                        THEN -fnit.vr_encargo
                                        WHEN fe.sigla = 'COFINS' AND fnit.aliquota in ({aliq_cofins})
                                        THEN fnit.vr_encargo
                                   END)           				                       AS vr_encargo_cofins,
                               fnit.fisnit_nat_rec_pis_cofins			               AS rec_nat
                        FROM   estoque_movimento_itens       AS emi
                        INNER JOIN estoque_movimento_capa    AS emc  ON emc.id = emi.capa_id
                        INNER JOIN estoque_produto           AS ep   ON ep.id = emi.produto_id
                        INNER JOIN estoque_produto_grade     AS epg  ON epg.id = emi.grade_id
                        LEFT  JOIN fiscal_ncm                AS ncm  ON ncm.id = ep.ncm_id
                        INNER JOIN fiscal_cfop               AS cfop ON cfop.id = emi.cfop_id
                        INNER JOIN fiscal_nf_item_tributacao AS fnit ON fnit.movimento_itens_id = emi.id
                        INNER JOIN ecf_cupons_capa           AS ecc  ON ecc.id = emi.cupom_id
                        INNER JOIN fiscal_cst                AS cst  ON cst.id = fnit.cst_id
                        INNER JOIN fiscal_encargo            AS fe   ON fe.id = fnit.encargo_id
                        WHERE emc.data in ({data_cupom})
                          AND ecc.company_id in {company_ids}
                          AND ecc.ecf_id in ({ecf_id})
                          AND fnit.encargo_id IN ({encargo_ids})
                          {query_natureza}
                          {query_cst}
                          AND cfop.codigo in ({cfop})
                          {item_nat_rec_diferentes}
                          {item_sem_nat_rec}
                        GROUP  BY ecc.ecf_id,
                                  emc.data,
                                  ep.codigo,
                                  ep.digito,
                                  epg.grade,
                                  ep.name,
                                  cst.codigo,
                                  ncm.codigo,
                                  cfop.codigo,
                                  fnit.fisnit_nat_rec_pis_cofins,
                                  fe.sigla,
                                  fnit.aliquota,
                                  ecc.tipo_cupom,
                                  emi.item_cancelado
                        HAVING (ecc.tipo_cupom = 'V' AND emi.item_cancelado = FALSE )
                            OR (ecc.tipo_cupom = 'C')
                       ) AS tmp
                GROUP BY ecf_id,
                         data,
                         codigo,
                         ncm,
                         cfop,
                         cst_codigo,
                         digito,
                         grade,
                         descricao,
                         vr_total_item_rateado,
                         vr_total_item,
                         vr_base_calculo,
                         rec_nat
               ) AS tmp2
        GROUP BY ecf_id,
                 data,
                 codigo,
                 ncm,
                 cfop,
                 cst_codigo,
                 digito,
                 grade,
                 descricao,
                 rec_nat,
                 aliquota_pis,
                 aliquota_cofins
        ORDER BY ecf_id, data, cfop, aliquota_pis, aliquota_cofins, descricao, grade
        """.format(aliq_pis=aliq_pis,
                   aliq_cofins=aliq_cofins,
                   ecf_id=ecf_id,
                   encargo_ids=encargo_ids,
                   data_cupom=data_cupom,
                   query_natureza=query_natureza,
                   query_cst=query_cst,
                   cfop=cfop,
                   company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                   item_nat_rec_diferentes=item_nat_rec_diferentes,
                   item_sem_nat_rec=item_sem_nat_rec)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def get_tributos_nf(self, cr, nf_ids):
        sql = """
                SELECT fisnit.fisnit_capa_id AS nf_capa,
                       fisnit.movimento_itens_id,
                       fisnit.vr_base_calculo AS vr_base_calculo,
                       fisnit.vr_encargo,
                       fisnit.aliquota,
                       fisnit.fisnit_nat_rec_pis_cofins,
                       fisenc.sigla,
                       fiscst.codigo  AS cst,
                       fisnit.vr_icms_substituicao,
                       fisnit.fisnit_capa_id,
                       cfop.codigo    AS cfop
                FROM   fiscal_nf_item_tributacao fisnit
                       LEFT JOIN fiscal_encargo fisenc
                               ON fisenc.id = fisnit.encargo_id
                       LEFT JOIN fiscal_cst fiscst
                               ON fiscst.id = fisnit.cst_id
                       LEFT JOIN fiscal_cfop cfop
                               ON cfop.id = fisnit.fisnit_cfop_id
                WHERE  fisnit.fisnit_capa_id in ({nf_id})
        """.format(nf_id=','.join(map(lambda x: "%s" % x, eval(nf_ids))))
        cr.execute(sql)
        dados = cr.dictfetchall()
        res = {}
        for i in dados:
            if not res.get(i.get('nf_capa')):
                res[i.get('nf_capa')] = [i]
            else:
                res[i.get('nf_capa')].append(i)

        return res

    def relat_notas_cupons_itens(self, cr, uid, listagem, data_ini, data_fim, entrada, saida, cupom,
                                 company_ids, datas, itens, resumo,
                                 imprimir_item_sem_nat_rec=False,
                                 imprimir_item_nat_rec_diferentes=False):
        natureza = datas.get('wiz_nat_rec')
        cst = datas.get('wiz_cst')
        if cst:
            cst = filter(lambda i: i, str(cst).split(' '))
        cfop_considerado_sped_contribuicoes = datas.get('wiz_cfop_sped_contribuicoes')

        def _relat_nota_itens(dados_nota, itens, resumo, cfop_ids,
                              imprimir_item_sem_nat_rec=False,
                              imprimir_item_nat_rec_diferentes=False):
            lista_itens = []
            ids_nota = []
            aliq_pis = []
            aliq_cofins = []
            data_emissao = []
            for i in dados_nota:
                ids_nota.append(i.get('id'))
                aliq_pis.append(i.get('aliq_pis') or 0.0)
                aliq_cofins.append(i.get('aliq_cofins') or 0.0)
                data_emissao.append(i.get('data_emissao') or '')

            ids_nota = str(list(set(ids_nota)))
            aliq_pis = str(list(set(aliq_pis)))
            aliq_cofins = str(list(set(aliq_cofins)))
            data_min = min(data_emissao)
            data_max = max(data_emissao)
            dados_itens_notas = self._consulta_itens_nota(cr, ids_nota, cst, natureza, aliq_pis,
                                                         aliq_cofins, cfop_ids, data_min, data_max,
                                                         company_ids,
                                                         cfop_considerado_sped_contribuicoes,
                                                         imprimir_item_nat_rec_diferentes,
                                                         imprimir_item_sem_nat_rec)

            dados_tributos = self.get_tributos_nf(cr, ids_nota)
            for dado in dados_nota:
                nf_id = dado.get('id')
                modelo_tipo = dado.get('modelo_tipo')
                aliq_pis = dado.get('aliq_pis')
                aliq_cofins = dado.get('aliq_cofins')
                codigo_cliente = dado.get('codigo_cliente')
                cliente_fornec = (dado.get('client_fornec') or '')[:32]
                cnae = dado.get('cnae')
                if not dado.get('cane'):
                    cnae = '0'.zfill(6)

                tributos = dados_tributos.get(nf_id)
                dados_pis = filter(lambda i: i.get('sigla') == 'PIS' and i.get(
                    'aliquota') == aliq_pis, tributos)
                dados_cofins = filter(lambda i: i.get('sigla') == 'COFINS' and i.get(
                    'aliquota') == aliq_cofins, tributos)
                if natureza:
                    dados_pis = filter(lambda i: i.get('fisnit_nat_rec_pis_cofins') == natureza,
                                       dados_pis)
                    dados_cofins = filter(lambda i: i.get('fisnit_nat_rec_pis_cofins') == natureza,
                                          dados_cofins)
                if cst:
                    dados_pis = filter(lambda i: i.get('cst') in cst, dados_pis)
                    dados_cofins = filter(lambda i: i.get('cst') in cst, dados_cofins)

                gera_nf_comunicacao = True
                if modelo_tipo == 'C':
                    for pis in dados_pis:
                        if pis.get('cst') and not 49 < int(pis.get('cst')) < 57:
                            gera_nf_comunicacao = False
                    for pis in dados_pis:
                        if pis.get('cst') and not 49 < int(pis.get('cst')) < 57:
                            gera_nf_comunicacao = False
                    if not gera_nf_comunicacao:
                        continue
                movimento_itens_ids = map(lambda i: i.get('movimento_itens_id'), dados_pis)
                movimento_itens_ids.extend(map(lambda i: i.get('movimento_itens_id'),
                                               dados_cofins))
                fisnit_capa_ids = map(lambda i: i.get('fisnit_capa_id'), dados_pis)
                fisnit_capa_ids.extend(map(lambda i: i.get('fisnit_capa_id'), dados_cofins))
                icms_st = sum(map(lambda i: i.get('vr_icms_substituicao') or 0.00,
                                  filter(lambda x: x.get(
                                      'movimento_itens_id') in movimento_itens_ids and x.get(
                                      'sigla') == 'ICMS' and x.get('cfop') == dado.get('cfop'),
                                         tributos)))
                vr_ipi = sum(map(lambda i: i.get('vr_encargo'), filter(lambda x: x.get(
                    'movimento_itens_id') in movimento_itens_ids and x.get('sigla') == 'IPI',
                                                                       tributos))) or 0.00
                vr_total_nota = dado.get('vr_total_nota') or 0.0
                vr_total_nota += icms_st + vr_ipi
                numero_nf = dado.get('numero')
                serie_nf = dado.get('serie')
                cfop = dado.get('cfop')

                vr_total_nota_item = dado.get('vr_total_produtos') or 0.0
                data_emissao = dado.get('data_emissao')
                data_emissao = data_emissao[:10]
                dados_itens_nota = dados_itens_notas.get(nf_id) or []
                dados_sem_itens = filter(lambda i: i.get('nf_id') == nf_id and not i.get(
                    'mov_item_id'), dados_itens_nota)
                list_itens_impressao = list()
                if not itens and dados_sem_itens:
                    for dado_sem_item in dados_sem_itens:
                        sem_item_nf_id = dado_sem_item.get('fisnit_capa_id')
                        lista_pis = filter(lambda i: i.get('nf_id') == sem_item_nf_id,
                                           dados_pis)
                        lista_cofins = filter(lambda i: i.get('nf_id') == sem_item_nf_id,
                                              dados_cofins)
                        vr_pis = 0
                        vr_cofins = 0
                        cst_pis = ''
                        nat_rec_pis = ''
                        vr_base_calculo_cofins = 0
                        vr_base_calculo_pis = 0
                        icms_st_item = sum(map(lambda i: i.get('vr_icms_substituicao') or 0.00,
                                               filter(lambda x: x.get('nf_id') == sem_item_nf_id
                                                                and x.get('sigla') == 'ICMS',
                                                      tributos)))
                        vr_ipi_item = sum(map(lambda i: i.get('vr_encargo') or 0.00,
                                              filter(lambda x: x.get('nf_id') == sem_item_nf_id
                                                               and x.get('sigla') == 'IPI', tributos
                                              )))
                        for encargo_pis in lista_pis:
                            aliq_pis = encargo_pis.get('aliquota')
                            vr_pis = float(fields.float_round(encargo_pis.get('vr_encargo'), 2))
                            cst_pis = encargo_pis.get('cst')
                            nat_rec_pis = encargo_pis.get('fisnit_nat_rec_pis_cofins')
                            vr_base_calculo_pis = encargo_pis.get('vr_base_calculo')
                            if encargo_pis.get('cst') == '05':
                                vr_base_calculo_pis = 0.00
                                vr_pis = 0.00

                        for encargo_cofins in lista_cofins:
                            aliq_cofins = encargo_cofins.get('aliquota')
                            vr_cofins = float(fields.float_round(encargo_cofins.get('vr_encargo'),
                                                                 2))
                            vr_base_calculo_cofins = encargo_cofins.get('vr_base_calculo')
                            if encargo_cofins.get('cst') == '05':
                                vr_base_calculo_cofins = 0.00
                                vr_cofins = 0.00
                        dado_sem_item['vr_encargo_pis'] = vr_pis
                        dado_sem_item['vr_encargo_cofins'] = vr_cofins
                        dado_sem_item['cst_codigo'] = cst_pis
                        dado_sem_item['rec_nat'] = nat_rec_pis or ''
                        dado_sem_item['cfop_id'] = dado_sem_item.get('cfop_id')
                        dado_sem_item['vr_base_calculo_pis'] = vr_base_calculo_pis
                        dado_sem_item['vr_base_calculo_cofins'] = vr_base_calculo_cofins
                        dado_sem_item['vr_total_item_rateado'] += icms_st_item + vr_ipi_item
                        dado_sem_item['vr_total_item'] = dado_sem_item.get('vr_total_item') or 0.00
                        lista_itens.append(dado_sem_item)
                else:
                    dados_itens = filter(
                        lambda i: i.get('nf_id') == nf_id and i.get('cfop') == cfop and i.get(
                            'mov_item_id') in movimento_itens_ids, dados_itens_nota)
                    for dado_item in dados_itens:
                        movimento_item_id = dado_item.get('mov_item_id')
                        icms_st_item = sum(map(lambda i: i.get('vr_icms_substituicao') or 0.00,
                                           filter(lambda x: x.get(
                                               'movimento_itens_id') == movimento_item_id
                                           and x.get('sigla') == 'ICMS', tributos)))
                        vr_ipi_item = sum(map(lambda i: i.get('vr_encargo') or 0.00,
                                          filter(lambda x: x.get(
                                              'movimento_itens_id') == movimento_item_id
                                                           and x.get('sigla') == 'IPI', tributos)))
                        vr_pis = 0
                        vr_cofins = 0
                        cst_pis = ''
                        cst_cofins = ''
                        nat_rec_pis = ''
                        nat_rec_cofins = ''
                        vr_base_calculo_cofins = 0
                        vr_base_calculo_pis = 0
                        lista_pis = filter(lambda i: i.get(
                            'movimento_itens_id') == movimento_item_id, dados_pis)
                        lista_cofins = filter(lambda i: i.get(
                            'movimento_itens_id') == movimento_item_id, dados_cofins)
                        for encargo_pis in lista_pis:
                            aliq_pis = encargo_pis.get('aliquota')
                            vr_pis = float(fields.float_round(encargo_pis.get('vr_encargo'), 2))
                            cst_pis = encargo_pis.get('cst')
                            nat_rec_pis = encargo_pis.get('fisnit_nat_rec_pis_cofins') or ''
                            vr_base_calculo_pis = encargo_pis.get('vr_base_calculo')
                            if cst_pis == '05':
                                vr_base_calculo_pis = 0.00
                                vr_pis = 0.00
                        for encargo_cofins in lista_cofins:
                            aliq_cofins = encargo_cofins.get('aliquota')
                            vr_cofins = float(fields.float_round(encargo_cofins.get('vr_encargo'),
                                                                 2))
                            cst_cofins = encargo_cofins.get('cst')
                            nat_rec_cofins = encargo_cofins.get('fisnit_nat_rec_pis_cofins') or ''
                            vr_base_calculo_cofins = encargo_cofins.get('vr_base_calculo')
                            if cst_cofins == '05':
                                vr_base_calculo_cofins = 0.00
                                vr_cofins = 0.00

                        dado_item['vr_encargo_pis'] = vr_pis
                        dado_item['vr_encargo_cofins'] = vr_cofins
                        dado_item['cst_codigo'] = cst_pis
                        dado_item['rec_nat'] = nat_rec_pis
                        dado_item['vr_base_calculo_pis'] = vr_base_calculo_pis
                        dado_item['vr_base_calculo_cofins'] = vr_base_calculo_cofins
                        if not dado_item.get('vr_total_item_rateado'):
                            dado_item['vr_total_item_rateado'] = icms_st_item + vr_ipi_item
                        else:
                            dado_item['vr_total_item_rateado'] += icms_st_item + vr_ipi_item
                        if not itens:
                            lista_itens.append(dado_item)
                            continue
                        cod_item = int(dado_item.get('codigo') or 0)
                        digito = dado_item.get('digito') or ''
                        codigo = str(str(cod_item) + '/' + digito)
                        grade = dado_item.get('grade')
                        descricao = dado_item.get('descricao')[:39]
                        ncm = dado_item.get('ncm') or ''
                        cfop = dado_item.get('cfop')
                        valor = (dado_item.get('vr_total_item_rateado') or 0.0)
                        valor_item = dado_item.get('vr_total_item') or 0.0
                        pis = getNumeroEditado(vr_pis or 0.0)
                        cofins = getNumeroEditado(vr_cofins or 0.0)
                        linha = '|'
                        linha += formataString(codigo[:10], 10, ' ', 'E', 'N', 'N')
                        linha += formataString(grade, 4, ' ', 'C', 'N', 'N')
                        linha += formataString(descricao, 40, ' ', 'D', 'N', 'N')
                        linha += formataString(ncm, 9, ' ', 'C', 'N', 'N')
                        linha += formataString(' ', 8, ' ', 'C', 'N', 'N')
                        linha += formataString(cfop, 5, ' ', 'C', 'N', 'N')
                        linha += formataString(getNumeroEditado(valor), 12, ' ', 'E', 'N', 'N')
                        linha += formataString(getNumeroEditado(valor_item), 12, ' ', 'E', 'N', 'N')
                        linha += formataString(aliq_pis, 6, ' ', 'E', 'N', 'N')
                        linha += formataString(getNumeroEditado(pis), 9, ' ', 'E', 'N', 'N')
                        linha += formataString(aliq_cofins, 6, ' ', 'E', 'N', 'N')
                        linha += formataString(getNumeroEditado(cofins), 9, ' ', 'E', 'N', 'N')
                        linha += formataString(nat_rec_pis, 4, ' ', 'E', 'N', 'N')
                        linha += formataString(cst_pis, 3, ' ', 'E', 'N', 'N')
                        linha += formataString(nat_rec_cofins, 4, ' ', 'E', 'N', 'N')
                        linha += formataString(cst_cofins, 3, ' ', 'E', 'N', 'N')
                        list_itens_impressao.append(linha + '|')
                        lista_itens.append(dado_item)

                if not resumo and dado.get('id') in fisnit_capa_ids:
                    listagem.imprimeLinha('|' + ' ' * 144 + '|')
                    linha = formataString('|' + str(codigo_cliente), 7, ' ', 'D', 'N', 'N')
                    linha += formataString(cliente_fornec, 33, ' ', 'D', 'N', 'N')
                    linha += formataString(cnae, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(numero_nf, 8, ' ', 'E', 'N', 'N')
                    linha += formataString(serie_nf, 5, ' ', 'E', 'N', 'N')
                    linha += formataString(getDataNoFormato(data_emissao, 'N', '/'),
                                           12, ' ', 'E', 'N', 'N')
                    linha += formataString(cfop, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(vr_total_nota), 12, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(vr_total_nota_item), 12, ' ', 'E', 'N', 'N')
                    linha += formataString(aliq_pis, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(dado.get('pis') or 0), 9, ' ', 'E', 'N', 'N')
                    linha += formataString(aliq_cofins, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(dado.get('cofins')), 9, ' ', 'E', 'N', 'N')
                    listagem.imprimeLinha(linha + '              |')

                for item_impressao in list_itens_impressao:
                    listagem.imprimeLinha(item_impressao)
            return lista_itens

        def _relat_cupom_itens(dados_ecf, itens, resumo, imprimir_item_sem_nat_rec=False,
                               imprimir_item_nat_rec_diferentes=False):
            lista_itens = []
            listagem.informaCab(_('*' + self.getFormatador(tamanho=144) + '*'))
            ecf_id =      str(list(set(map(lambda x: x.get('ecf_id')      or 0.0, dados_ecf))))
            aliq_pis =    str(list(set(map(lambda x: x.get('aliq_pis')    or 0.0, dados_ecf))))
            aliq_cofins = str(list(set(map(lambda x: x.get('aliq_cofins') or 0.0, dados_ecf))))
            cfop =        str(list(set(map(lambda x: str(x.get('cfop'))   or 0.0, dados_ecf))))
            data_cupom =  str(list(set(map(lambda x: str(x.get('data'))   or 0.0, dados_ecf))))
            dados_item_cupons = self._contulta_itens_cupom(cr, ecf_id, aliq_pis, aliq_cofins,
                                                           natureza, cst, cfop, data_cupom,
                                                           company_ids,
                                                           imprimir_item_nat_rec_diferentes,
                                                           imprimir_item_sem_nat_rec)
            dict_ecf_data_cfop_aliqP_aliqC = {}
            for item in dados_item_cupons:
                item['vr_base_calculo_pis'] = item.get('vr_base_calculo')
                item['vr_base_calculo_cofins'] = item.get('vr_base_calculo')
                ecf_data_cfop_aliqP_aliqC = str(item.get('ecf_id')) + str(item.get('data')) + str(item.get('cfop')) \
                                          + str(item.get('aliquota_pis')) + str(item.get('aliquota_cofins'))
                # imprime a linha referente a ecf
                if not ecf_data_cfop_aliqP_aliqC in dict_ecf_data_cfop_aliqP_aliqC:
                    if dict_ecf_data_cfop_aliqP_aliqC and itens:
                        listagem.imprimeLinha('|' + ' ' * 144 + '|')
                        listagem.imprimeLinha('|' + ' ' * 144 + '|')
                    dict_ecf_data_cfop_aliqP_aliqC[ecf_data_cfop_aliqP_aliqC] = True
                    ecf_id = item.get('ecf_id')
                    aliq_pis = item.get('aliquota_pis') or 0.00
                    aliq_cofins = item.get('aliquota_cofins') or 0.00
                    cfop = item.get('cfop')
                    data_cupom = item.get('data')
                    dado = filter(lambda x: x.get('ecf_id') == ecf_id
                                        and x.get('data') == data_cupom
                                        and x.get('cfop') == cfop
                                        and x.get('aliq_pis') == aliq_pis
                                        and x.get('aliq_cofins') == aliq_cofins, dados_ecf)
                    # retira os movimentos de cupons venda e cancelamento referente
                    if not dado:
                        continue
                    dado = dado[0]
                    if (not dado.get('vr_total') or dado.get('vr_total') < 0.00001) \
                            and dado.get('vr_total') == dado.get('vr_total_item_rateado'):
                        continue
                    # se o filtro do relatorio dor resumo, nao imprime apartir daqui
                    if resumo:
                        lista_itens.append(item)
                        continue
                    ecf = str(dado.get('ecf')).zfill(3)
                    linha = formataString('|Total ecf: ' + ecf, 17, ' ', 'D', 'N', 'N')
                    linha += formataString(getDataNoFormato(dado.get('data'), 'N', '/'),
                                           54, ' ', 'E', 'N', 'N')
                    linha += formataString(dado.get('cfop'), 6, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(dado.get('vr_total') or 0.00), 12, ' ', 'E',
                                           'N',
                                           'N')
                    linha += formataString(getNumeroEditado(dado.get('vr_total_item') or 0.00), 12, ' ',
                                           'E',
                                           'N',
                                           'N')
                    linha += formataString(aliq_pis, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(dado.get('pis') or 0.00), 9, ' ', 'E', 'N',
                                           'N')
                    linha += formataString(aliq_cofins, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(dado.get('cofins') or 0.00), 9, ' ', 'E',
                                           'N',
                                           'N')
                    linha += '{:<14}|'.format('')
                    listagem.imprimeLinha(linha)

                if not item.get('vr_total_item_rateado') or \
                                item.get('vr_total_item_rateado') < 0.00001:
                    continue

                lista_itens.append(item)
                # quando o filtro do relatorio fora para nota e cupom, nao mostra os itens
                if not itens:
                    continue
                # itens do cupom
                cod_item = int(item.get('codigo'))
                digito = item.get('digito') or ''
                codigo = str(str(cod_item) + '/' + digito)
                grade = item.get('grade')
                descricao = item.get('descricao')
                ncm = item.get('ncm') or ''
                cfop = item.get('cfop')
                valor = item.get('vr_total_item_rateado') or 0.0
                valor_item = item.get('vr_total_item') or 0.0
                aliq_pis = item.get('aliquota_pis') or 0
                pis = getNumeroEditado(item.get('vr_encargo_pis') or 0.0)
                aliq_cofins = item.get('aliquota_cofins') or 0
                cofins = getNumeroEditado(item.get('vr_encargo_cofins') or 0.0)
                rec_nat_pis = item.get('rec_nat') or ''
                cst_pis = item.get('cst_codigo') or ''
                rec_nat_cofins = item.get('rec_nat') or ''
                cst_cofins = item.get('cst_codigo') or ''
                linha = '|'
                linha += formataString(codigo[:10], 10, ' ', 'E', 'N', 'N')
                linha += formataString(grade, 4, ' ', 'C', 'N', 'N')
                linha += formataString(descricao[:39], 40, ' ', 'D', 'N', 'N')
                linha += formataString(ncm, 9, ' ', 'C', 'N', 'N')
                linha += formataString(' ', 8, ' ', 'C', 'N', 'N')
                linha += formataString(cfop, 5, ' ', 'C', 'N', 'N')
                linha += formataString(getNumeroEditado(valor), 12, ' ', 'E', 'N', 'N')
                linha += formataString(getNumeroEditado(valor_item), 12, ' ', 'E', 'N', 'N')
                linha += formataString(aliq_pis, 6, ' ', 'E', 'N', 'N')
                linha += formataString(getNumeroEditado(pis), 9, ' ', 'E', 'N', 'N')
                linha += formataString(aliq_cofins, 6, ' ', 'E', 'N', 'N')
                linha += formataString(getNumeroEditado(cofins), 9, ' ', 'E', 'N', 'N')
                linha += formataString(rec_nat_pis, 4, ' ', 'E', 'N', 'N')
                linha += formataString(cst_pis, 3, ' ', 'E', 'N', 'N')
                linha += formataString(rec_nat_cofins, 4, ' ', 'E', 'N', 'N')
                linha += formataString(cst_cofins, 3, ' ', 'E', 'N', 'N')
                linha += '|'
                listagem.imprimeLinha(linha)
            return lista_itens

        res = []
        cfop_ids = datas.get('wiz_cfop_ids') or []
        if entrada:
            dados = self.consulta_nota(
                cr, uid, data_ini, data_fim, tipo_mov='E', company_ids=company_ids, filtro=datas,
                imprimir_item_nat_rec_diferentes=imprimir_item_nat_rec_diferentes,
                imprimir_item_sem_nat_rec=imprimir_item_sem_nat_rec)
            gc.collect()
            if dados:
                res.extend(_relat_nota_itens(dados, itens, resumo, cfop_ids,
                                             imprimir_item_sem_nat_rec,
                                             imprimir_item_nat_rec_diferentes))
            gc.collect()
        if saida:
            dados = self.consulta_nota(
                cr, uid, data_ini, data_fim, tipo_mov='S', company_ids=company_ids, filtro=datas,
                imprimir_item_nat_rec_diferentes=imprimir_item_nat_rec_diferentes,
                imprimir_item_sem_nat_rec=imprimir_item_sem_nat_rec)
            gc.collect()
            if dados:
                res.extend(_relat_nota_itens(dados, itens, resumo, cfop_ids,
                                             imprimir_item_sem_nat_rec,
                                             imprimir_item_nat_rec_diferentes))
            gc.collect()
        if cupom and ecf_instalado:
            dados = self.consulta_cupons(cr, uid, data_ini, data_fim, company_ids, cfop_ids,
                                         cfop_considerado_sped_contribuicoes,
                                         imprimir_item_nat_rec_diferentes,
                                         imprimir_item_sem_nat_rec)
            gc.collect()
            if dados:
                res.extend(_relat_cupom_itens(dados, itens, resumo, imprimir_item_sem_nat_rec,
                                              imprimir_item_nat_rec_diferentes))
            gc.collect()
        return res

    def get_query_item_nat_rec_diferente(self):
        sql = """
                AND (ep.estpro_pis_nat_rec != ep.estpro_cofins_nat_rec OR 
                        (
                            SELECT fisnit_nat_rec_pis_cofins 
                            FROM fiscal_nf_item_tributacao 
                            WHERE 
                                movimento_itens_id = emi.id 
                                AND encargo_id IN (
                                   SELECT id FROM fiscal_encargo WHERE sigla = 'PIS')
                        ) != 
                        (
                            SELECT fisnit_nat_rec_pis_cofins 
                            FROM fiscal_nf_item_tributacao 
                            WHERE 
                                movimento_itens_id = emi.id 
                                AND encargo_id IN (
                                   SELECT id FROM fiscal_encargo WHERE sigla = 'COFINS')
                        )
                    )
            """
        return sql

    def get_query_item_sem_nat_rec(self):
        sql = """
                AND (fnit.fisnit_nat_rec_pis_cofins IN (' ', '0', '00', '000') OR fnit.fisnit_nat_rec_pis_cofins IS NULL)
            """
        return sql


report_apuracao_pis_cofins('report.wizard.apuracao.pis.cofins')
