# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from xml.dom.minidom import parseString
from openerp.tools.translate import _
from openerp.osv import fields, osv
from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400
from datetime import datetime


class wizard_carta_correcao(osv.TransientModel):
    _name = 'wizard.carta.correcao'
    _description = 'Carta de Correcao'

    _columns = {
        'name': fields.char('Motivo', required=True, size=1000, help="Motivo da Correcao"),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        id_s = context['active_id']
        obj_nf = self.pool.get('fiscal.nf.capa')
        if isinstance(id_s, (tuple, list)):
            id_s = id_s[0]
        brw_nf = obj_nf.browse(cr, uid, id_s)
        company_id = brw_nf.company_id.id
        state = brw_nf.state
        tipo = brw_nf.tipo
        if (tipo == 'S' or (tipo == 'E' and brw_nf.nf_propria)) and state == '3':
            obj_parametro = self.pool.get('fiscal.parametro')
            id_p = obj_parametro.search(cr, uid, [], context=context)
            if id_p:
                vals_p = obj_parametro.read(cr, uid, id_p[0], context=context)
            else:
                raise osv.except_osv(_('Atencao!'),
                                     _('Parametro do Fiscal nao cadastrado.'))

            # Nao eh possivel emitir CCE em contingencia
            if vals_p.get('fispar_nfe_contingencia'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao e possivel emitir Carta de Correcao Eletronica em '
                                       'contingencia.'))

            obj_pessoa = self.pool.get('adm.pessoa')
            obj_pessoa_end = self.pool.get('adm.pessoa.endereco')
            pessoa_id = self.pool.get('res.company').browse(cr, uid, company_id).rescon_pessoa_id.id
            if not pessoa_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao existe uma pessoa associada a empresa cadastrada.'))
            enderecos = obj_pessoa.read(
                cr, uid, pessoa_id, ['endereco_ids'], context).get('endereco_ids') or []
            cnpj_emissor = obj_pessoa.browse(cr, uid, pessoa_id).cpf_cnpj
            cnpj_emissor = cnpj_emissor.replace('.', '').replace('/', '').replace('-', '')
            estado = False
            for x in obj_pessoa_end.browse(cr, uid, enderecos, context):
                if x.endereco_tipo_id.name == 'Comercial':
                    estado = x.uf_id.sigla
                    break
                if x.endereco_tipo_id.name == 'Normal':
                    estado = x.uf_id.sigla
                    continue
            if not estado:
                raise osv.except_osv(_('Atencao!'),
                                     _('Cadastre um endereco valido para a empresa.'))
            vals_nf = obj_nf.read(
                cr, uid, id_s, ['modelo_nf_series_id', 'nfe_chave_acesso'], context,
                '_classic_write')
            try:
                carta_correcao = self.remove_accents(vals['name'])
            except UnicodeEncodeError:
                carta_correcao = self.remove_accents(vals['name'].encode('utf-8'))
            mod_nf_id = vals_nf.get('modelo_nf_series_id')
            read_mod = self.pool.get('fiscal.modelo.nf.series').read(
                cr, uid, mod_nf_id, ['versao_xml', 'ambiente', 'tipo_contingencia'], context)

            # O evento CC-e (Carta de correcao eletronica) nao estah disponivel em ambiente SCAN
            if read_mod.get('tipo_contigencia'):
                raise osv.except_osv(_('Atencao!'),
                                      _('Emissao da CC-e para notas emitidas em contingencia nao '
                                        'esta habilitada pela SEFAZ. Nao sera possivel emissao de '
                                        'carta de correcao para este tipo de emissao de NF-e.'))
            if len(carta_correcao) < 15:
                raise osv.except_osv(_('Atencao!'),
                                     _('A carta de correcao deve ter no minimo 15 caracteres.'))
            elif len(carta_correcao) > 1000:
                raise osv.except_osv(_('Atencao!'),
                                     _('A carta de correcao nao deve ultrapassar 1000 caracteres.'))
            sequencia = 1
            if vals_nf.get('nfe_chave_acesso'):
                while True:
                    nfe = nfe_400
                    nfe = nfe(certify=vals_p.get('certificado_conteudo'),
                              key=vals_p.get('chave_conteudo'),
                              environment=brw_nf.modelo_nf_series_id.ambiente, state=estado,
                              contingency=brw_nf.modelo_nf_series_id.tipo_contingencia)
                    processo, __ = nfe.emitir_carta_correcao(
                        brw_nf.nfe_chave_acesso, cnpj_emissor, carta_correcao, sequencia)
                    if processo.resposta.cStat.valor == u'128':  # Lote processado
                        evento = processo.resposta.infEvento
                        if evento.cStat.valor == u'135':
                            dhrecbto = datetime.strptime(
                                evento.dhRegEvento.valor[:-6], '%Y-%m-%dT%H:%M:%S')
                            dhrecbto = self.pool.get('adm.utils').admutl_converte_tz(
                                cr, uid, dhrecbto, 'America/Sao_Paulo', 'UTC', context=context)
                            self.pool.get('fiscal.evento').create(cr, uid, {
                                'fiseve_xml_env': processo.envio.xml,
                                'fiseve_xml_ret': processo.resposta.xml,
                                'fiseve_tipo': '00110110',  # Carta de correcao
                                'fiseve_data': dhrecbto or datetime.now().strftime(
                                    '%Y-%m-%d %H:%M:%S'),
                                'fiseve_status': evento.cStat.valor,
                                'fiseve_motivo': evento.xMotivo.valor,
                                'fiseve_lote': processo.resposta.idLote.valor,
                                'fiseve_protocolo': evento.nProt.valor,
                                'fiseve_nf_id': brw_nf.id,
                            })
                            xml = parseString(processo.resposta.xml).toprettyxml('  ', '\n')
                            protocolo = processo.resposta.infEvento.Id.valor[2:]
                            obj_nf.write(
                                cr, uid, id_s, dict(fisnfc_nfe_xml_correcao=processo.resposta.xml,
                                                    fisnfc_nfe_protocolo_correcao=protocolo,
                                                    fisnfc_nfe_status_correcao='%s - %s' % (
                                                        evento.cStat.valor, evento.xMotivo.valor)),
                                context)
                            obj_nf.message_post(
                                cr, uid, id_s, _('Carta de correcao enviada. (sequencia %s)' %
                                                 str(sequencia).zfill(2)),
                                'CC-e %s' % protocolo if protocolo else 'NF-e %s-%s' % (
                                    brw_nf.numero_nf, brw_nf.serie_nf))
                            break
                        elif evento.cStat.valor == u'573':
                            sequencia += 1
                            if sequencia > 19:
                                raise osv.except_osv(_('Erro!'),
                                                     _('Numero de Cartas de Correcao excedido. '
                                                       'Maximo de 20.'))
                        else:
                            raise osv.except_osv(_('Erro!'), '%s - %s' % (
                                evento.cStat.valor, evento.xMotivo.valor))
                    else:
                        if not processo.resposta.cStat.valor:
                            raise osv.except_osv(_('Erro!'), self.remove_accents(
                                processo.resposta.reason))
                        else:
                            raise osv.except_osv(_('Erro!'), '%s - %s' % (
                                processo.resposta.cStat.valor, processo.resposta.xMotivo.valor))
            else:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nota fiscal sem chave de acesso.'))

        return super(wizard_carta_correcao, self).create(cr, uid, vals, context=context)

    @staticmethod
    def remove_accents(text):
        from unicodedata import normalize, category
        unicode_string = unicode(text, 'utf-8') if type(text) == str else text
        text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

        return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

    def confirmar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {'type': 'ir.actions.act_window_close'}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
