# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_documento_emitido(osv.Model):
    # prefixo: wizdoe_

    _name = 'wizard.documento.emitido'
    _columns = {
        'wizdoe_data_inicial': fields.date('Data inicial', required=True),
        'wizdoe_data_final': fields.date('Data final', required=True),
        'wizdoe_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wizdoe_documento_tipo_id': fields.many2one('documento.documento.tipo', 'Tipo documento'),
        'wizdoe_listar_venda': fields.boolean('Listar apenas vendas'),
        'company_id': fields.many2one('res.company', 'Empresa'),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
    }

    def wizdoe_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados = self.wizdoe_get_dado(cr, uid, ids, context)
        if not dados:
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para impressao'))
        brw_documento_emitido = self.browse(cr, uid, ids[0], context)
        data_inicial = brw_documento_emitido.wizdoe_data_inicial
        data_final = brw_documento_emitido.wizdoe_data_final
        pessoa_nome = brw_documento_emitido.wizdoe_pessoa_id.nome or ''
        tipo_documento = brw_documento_emitido.wizdoe_documento_tipo_id.name or ''
        listar_venda = brw_documento_emitido.wizdoe_listar_venda or False
        if listar_venda:
            listar_venda = 'Sim'
        else:
            listar_venda = 'Nao'
        dados_completo = {
            'dados': dados.get('dados') or [],
            'dados_resumo_tipo_documento': dados.get('dados_resumo_tipo_documento') or [],
            'data_inicial': data_inicial,
            'data_final': data_final,
            'pessoa_nome': pessoa_nome,
            'tipo_documento': tipo_documento,
            'listar_venda': listar_venda
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'documento_emitido',
            'datas': dados_completo,
        }

    def wizdoe_get_dado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = {}
        brw_wizard_documento_emitido = self.browse(cr, uid, ids[0], context)
        data_inicial = brw_wizard_documento_emitido.wizdoe_data_inicial
        data_final = brw_wizard_documento_emitido.wizdoe_data_final
        pessoa_id = brw_wizard_documento_emitido.wizdoe_pessoa_id.id or False
        documento_tipo_id = brw_wizard_documento_emitido.wizdoe_documento_tipo_id.id or False
        company_id = brw_wizard_documento_emitido.company_id.id
        company_ids = self.pool.get('res.company')._get_company_children(cr, uid, company_id)
        if len(company_ids) == 1:
            company_ids = str(tuple(company_ids)).replace(',', '')
        else:
            company_ids = str(tuple(company_ids))
        listar_venda = brw_wizard_documento_emitido.wizdoe_listar_venda or False
        sql = """
            SELECT
                fnc.destinatario_id                       AS pessoa_id,
                ap.nome                                   AS nome_pessoa,
                'NF'                                      AS doc,
                fnc.numero_nf                             AS numero_documento,
                fnc.serie_nf                              AS serie,
                fnc.vr_total_produtos                     AS vr_produto,
                fnc.vr_desconto                           AS vr_desconto,
                fnc.vr_total_nf                           AS vr_total,
                fnc.data_emissao                          AS data_emissao,
                ddt.descricao                             AS titulo_tipo_documento,
                ft.documento                              AS titulo_documento,
                ft.parcela                                AS titulo_parcela,
                ft.data_prorrogacao                       AS titulo_data_vencimento,
                ft.vr_original                            AS titulo_valor,
                CASE
                    WHEN ft.state = 'A' THEN 'Aberto'
                    WHEN ft.state = 'B' THEN 'Baixado'
                    WHEN ft.state = 'D' THEN 'Devolvido'
                    WHEN ft.state = 'R' THEN 'Reapresentado'
                    WHEN ft.state = 'C' THEN 'Cancelado'
                    ELSe ''
                END                                       AS titulo_state,
                CASE
                    WHEN afp.cartao_credito_debito = True AND
                        afp.admfpg_cartao_pessoa_id IS NOT NULL
                        THEN (SELECT codigo || '-' || nome
                              FROM adm_pessoa
                              WHERE id = afp.admfpg_cartao_pessoa_id
                                    AND ft.pessoa_id = id)
                    ELSE ''
                END                                       AS titulo_operadora,
                CASE
                    WHEN ft.id IS NOT NULL AND ft.fintit_nro_autorizacao_cartao IS NOT NULL
                        THEN ft.fintit_nro_autorizacao_cartao
                    ELSE ''
                END                                       AS titulo_nro_autorizacao,
                (
                    SELECT tipo
                    FROM adm_condicao
                    WHERE
                        id IN (
                            SELECT adm_condicao_id
                            FROM adm_condicao_forma
                            WHERE
                                id IN (
                                    SELECT condicao_forma_id
                                    FROM fiscal_condicao_pagamento
                                    WHERE id = fcp.id
                                    )
                            )
                )                                         AS tipo_pagamento,
                fnc.state                                 AS state,
                ft.id                                     AS titulo_id
            FROM fiscal_nf_capa fnc
            INNER JOIN adm_pessoa ap
                ON fnc.destinatario_id = ap.id
            LEFT JOIN financeiro_fatura ff
                ON ff.nf_id = fnc.id
            LEFT JOIN financeiro_titulo ft
                ON ff.id = ft.fatura_id
            LEFT JOIN documento_documento_tipo ddt
                ON ddt.id = ft.doc_tipo_id
            INNER JOIN fiscal_condicao_pagamento fcp
                ON fcp.capa_id = fnc.id
            INNER JOIN adm_forma_pgto afp
                ON afp.id = fcp.forma_pagto_id
            WHERE
                fnc.company_id IN %s
                AND fnc.tipo = 'S'
                AND fnc.state = '3'
                AND fnc.data_emissao >= '%s 00:00:00'
                AND fnc.data_emissao <= '%s 23:59:59'
        """ % (company_ids, data_inicial, data_final)
        if pessoa_id:
            sql += """
                AND fnc.destinatario_id = %s
            """ % pessoa_id
        if documento_tipo_id:
            sql += """
                AND ddt.id = %s
            """ % documento_tipo_id
        if listar_venda:
            sql += """
                AND fnc.natureza_id IN (
                    SELECT id
                    FROM fiscal_cfop_natureza
                    WHERE
                        tipo = 'S'
                        AND cfop_id IN (
                            SELECT id
                            FROM fiscal_cfop
                            WHERE
                                tipo = 'S'
                                AND movimento_tipo = 'C'
                            )
                        AND fisnat_operacao_id IN (
                            SELECT id
                            FROM estoque_natureza_operacao
                            WHERE
                                estnto_tipo_operacao = 'S'
                                AND estnto_gerar_financeiro = True
                                AND estnto_gerar_estatistica = True
                            )
                    )
            """
        sql += """
            ORDER BY /*fnc.data_emissao,*/ fnc.numero_nf, titulo_operadora
        """
        cr.execute(sql)
        dados = cr.dictfetchall()
        sql_resumo_tipo_documento = self.wizdoe_get_resumo_tipo_documento(cr, uid, ids, context)
        cr.execute(sql_resumo_tipo_documento)
        dados_resumo_tipo_documento = cr.dictfetchall()
        res.update({'dados': dados})
        res.update({'dados_resumo_tipo_documento': dados_resumo_tipo_documento})
        return res

    def wizdoe_get_resumo_tipo_documento(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_wizard_documento_emitido = self.browse(cr, uid, ids[0], context)
        data_inicial = brw_wizard_documento_emitido.wizdoe_data_inicial
        data_final = brw_wizard_documento_emitido.wizdoe_data_final
        pessoa_id = brw_wizard_documento_emitido.wizdoe_pessoa_id.id or False
        documento_tipo_id = brw_wizard_documento_emitido.wizdoe_documento_tipo_id.id or False
        listar_venda = brw_wizard_documento_emitido.wizdoe_listar_venda or False
        company_id = brw_wizard_documento_emitido.company_id.id
        company_ids = list(set(self.pool.get('res.company')._get_company_children(
            cr, uid, company_id)))
        if len(company_ids) == 1:
            company_ids = str(tuple(company_ids)).replace(',', '')
        else:
            company_ids = str(tuple(company_ids))
        sql = """
            SELECT
                round(Sum(fcp.valor) + ((fnc.vr_desconto * Sum(fcp.valor))/fnc.vr_total_nf),
                      2)                                  AS vr_produto,
                round(((fnc.vr_desconto * Sum(fcp.valor))/fnc.vr_total_nf), 2)
                                                          AS vr_desconto,
                Sum(fcp.valor)                            AS vr_total,
                (SELECT descricao
                 FROM documento_documento_tipo
                 WHERE id IN (SELECT admfpg_documento_tipo_id FROM adm_forma_pgto
                              WHERE id = fcp.forma_pagto_id))
                                                          AS titulo_tipo_documento,
                (
                    SELECT tipo
                    FROM adm_condicao
                    WHERE
                        id IN (
                            SELECT adm_condicao_id
                            FROM adm_condicao_forma
                            WHERE
                                id IN (
                                    SELECT condicao_forma_id
                                    FROM fiscal_condicao_pagamento
                                    WHERE id = fcp.id
                                    )
                            )
                )                                         AS tipo_pagamento
            FROM fiscal_nf_capa fnc
            INNER JOIN fiscal_condicao_pagamento fcp
                ON fcp.capa_id = fnc.id
            WHERE
                fnc.company_id IN %s
                AND fnc.tipo = 'S'
                AND fnc.state = '3'
                AND fnc.data_emissao >= '%s 00:00:00'
                AND fnc.data_emissao <= '%s 23:59:59'
        """ % (company_ids, data_inicial, data_final)
        if pessoa_id:
            sql += """
                AND fnc.destinatario_id = %s
            """ % pessoa_id
        if documento_tipo_id:
            sql += """
                AND fcp.forma_pagto_id IN (SELECT id FROM adm_forma_pgto WHERE
                                           admfpg_documento_tipo_id = %s)
            """ % documento_tipo_id
        if listar_venda:
            sql += """
                AND fnc.natureza_id IN (
                    SELECT id
                    FROM fiscal_cfop_natureza
                    WHERE
                        tipo = 'S'
                        AND cfop_id IN (
                            SELECT id
                            FROM fiscal_cfop
                            WHERE
                                tipo = 'S'
                                AND movimento_tipo = 'C'
                            )
                        AND fisnat_operacao_id IN (
                            SELECT id
                            FROM estoque_natureza_operacao
                            WHERE
                                estnto_tipo_operacao = 'S'
                                AND estnto_gerar_financeiro = True
                                AND estnto_gerar_estatistica = True
                            )
                    )
            """
        sql += """
            GROUP BY tipo_pagamento, titulo_tipo_documento, fnc.vr_total_nf, fnc.vr_desconto
        """
        return sql
