# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
import base64
from zipfile import ZipFile
import subprocess
from datetime import datetime
import time
import os


class wizard_exportar_xml_nfe(osv.TransientModel):
    # prefixo: wexxml_

    _name = 'wizard.exportar.xml.nfe'
    _columns = {
        'wexxml_data_inicial': fields.date('Data inicial', required=True),
        'wexxml_data_final': fields.date('Data final', required=True),
        'wexxml_arquivo': fields.binary('Download do arquivo'),
        'wexxml_nome_arquivo': fields.char('Nome arquivo', size=20),
        'wexxml_nfe': fields.boolean(
            'NF-e', help='Se marcado, o XML de NF-e (modelo 55) sera exportado.'),
        'wexxml_nfe_terceiro': fields.boolean(
            'NF-e de terceiros', help='Se marcado, sera exportado tambem o XML de NF-e de compra.'),
        'wexxml_canc_export_fatur': fields.boolean(
            'P/Cancel.Exp.Faturada', help='Se marcado, para as notas canceladas, vai exportar 2 arquivos, se nao marcado, exporta somente o xml de cancelamento'),
        'wexxml_nfce': fields.boolean(
            'NFC-e', help='Se marcado, o XML da NFC-e (modelo 65) sera exportado.'),
        'wexxml_operacao': fields.selection([('T', 'Todas'), ('E', 'Entradas'), ('S', 'Saidas')], 'Operacao', size=1,
                                             required=True),
        'wexxml_texto': fields.text('Notas Fiscais nao exportadas.', readonly=True),
        'wexxml_status': fields.selection([('T', 'Todas'), ('3', 'Faturadas'), ('C', 'Canceladas')],
                                          'Status notas', size=1, required=True),
    }
    _defaults = {
        'wexxml_nome_arquivo': 'NFE.zip',
        'wexxml_nfe': True,
        'wexxml_nfe_terceiro': False,
        'wexxml_nfce': True,
        'wexxml_operacao': 'T',
        'wexxml_canc_export_fatur': True,
    }

    def wexxml_button_exportar(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Exportar'. Encontra as notas fiscais a partir dos filtros selecionados.
        Essas sao passadas como parametro para o metodo 'wexxml_exporta_xml_nfe' que, por sua vez,
        vai exportar o xml de cada uma.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista da id da tabela dessa classe.
        :param context: dados de valores contextuais.
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: Acao para chamada de uma nova janela, caso alguma nota fiscal tenha sido exportada.
        Senao, raise, para informar que os filtros nao encontrou nenhuma.
        :rtype: bool
        """
        context = context and context.copy() or {}
        exportacao = self.browse(cr, uid, ids[0], context)
        status = exportacao.wexxml_status
        if exportacao.wexxml_nfe:
            modelo_nf_ids = self.pool.get('fiscal.modelo.nf').search(
                cr, uid, [('fismnf_codigo', 'in', ('55', '57')), ('fismnf_tipo_nfe', '=', True)],
                context=context)
        else:
            modelo_nf_ids = []
        if exportacao.wexxml_nfce:  # Inclui NFC-e
            modelo_nf_ids += self.pool.get('fiscal.modelo.nf').search(
                cr, uid, [('fismnf_codigo', '=', '65')], context=context)
        modelo_nf_series_ids = self.pool.get('fiscal.modelo.nf.series').search(
            cr, uid, [('modelo_nf_id', 'in', modelo_nf_ids)], context=context)

        convert_tz = self.pool.get('adm.utils').admutl_converte_tz
        data_inicial = datetime.strptime(exportacao.wexxml_data_inicial + ' 00:00:00', '%Y-%m-%d %H:%M:%S')
        data_inicial = str(convert_tz(cr, uid, data_inicial, tz_origem=context.get('tz'),
                                      tz_destino='UTC', context=context))[:19]
        data_final = datetime.strptime(exportacao.wexxml_data_final + ' 23:59:59', '%Y-%m-%d %H:%M:%S')
        data_final = str(convert_tz(cr, uid, data_final, tz_origem=context.get('tz'),
                                    tz_destino='UTC', context=context))[:19]
        args_propria = [('data_emissao', '>=', data_inicial),
                        ('data_emissao', '<=', data_final),
                        ('nf_propria', '=', True),
                        ('modelo_nf_series_id', 'in', modelo_nf_series_ids)]
        if status != 'T':
            args_propria.append(('state', '=', status))
        if exportacao.wexxml_operacao != 'T':
            args_propria.append(('tipo', '=', exportacao.wexxml_operacao))
        nf_ids = self.pool.get('fiscal.nf.capa').search(cr, uid, args_propria, context=context)
        if exportacao.wexxml_nfe_terceiro:
            args_terceiro = [('data_saida_entrada', '>=', data_inicial),
                             ('data_saida_entrada', '<=', data_final),
                             ('nf_propria', '=', False),
                             ('modelo_nf_id', 'in', modelo_nf_ids)]
            if status != 'T':
                args_terceiro.append(('state', '=', status))
            if exportacao.wexxml_operacao != 'T':
                args_terceiro.append(('tipo', '=', exportacao.wexxml_operacao))
            nf_ids += self.pool.get('fiscal.nf.capa').search(
                cr, uid, args_terceiro, context=context)
        if not nf_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhuma Nota Fiscal encontrada para os '
                                                 'filtros selecionados.'))
        dados = self.wexxml_exporta_xml_nfe(cr, uid, nf_ids, context)
        self.write(cr, uid, exportacao.id,
                   {'wexxml_arquivo': dados.get('dados'), 'wexxml_texto': dados.get('msg')},
                   context)
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'fiscal', 'view_wizard_exportar_xml_nfe_exportados_form')
        view_id = reference and reference[1] or False

        return {'name': 'XML exportados', 'view_type': 'form', 'view_mode': 'form',
                'res_model': self._name, 'view_id': [view_id], 'target': 'new',
                'type': 'ir.actions.act_window'}

    def wexxml_exporta_xml_nfe(self, cr, uid, nf_ids, context=None):
        """
        O metodo exporta para um arquivo compactado (tar.gz) todos os arquivos xml originados das
        notas fiscais (nf_ids).
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param nf_ids: lista de id da tabela fiscal_nf_capa.
        :param context: dados de valores contextuais.
        :type cr: object
        :type uid: int
        :type nf_ids: list
        :type context: dict
        :return: Dados do arquivo compactado e descricao das notas nao exportadas.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        nfe_xml_chave_dados = self.wexxml_get_chave_xml_nfe(cr, uid, nf_ids, context)
        diretorio_arquivo = '/tmp/NFE_%s.zip' % time.strftime('%d_%m_%Y_%H_%M_%S')
        arquivo_zipado = ZipFile(diretorio_arquivo, 'w')
        nf_fields = ['nf_propria', 'numero_nf', 'serie_nf', 'state', 'nfe_xml_proc',
                     'nfe_xml_cancelamento', 'nfe_chave_cte', 'fisnfc_cte_xml', 'nfe_xml_gerado']

        msg = []
        tam_nf_ids = len(nf_ids) / 10 + 1
        for indice in range(tam_nf_ids):
            if not nf_ids[indice*10: (indice+1)*10]:
                continue

            diretorios = []
            for nf in obj_fiscal_nf_capa.read(cr, uid, nf_ids[indice*10: (indice+1)*10], nf_fields, context=context):
                nf_id = nf.get('id')
                state_nf = nf.get('state')
                nfe_xml = ''
                if nf.get('nfe_chave_cte'):
                    nfe_xml = nf.get('fisnfc_cte_xml')
                elif state_nf in ('3', 'C'):
                    if nf.get('nfe_xml_proc'):
                        nfe_xml = nf.get('nfe_xml_proc')
                    else:
                        nfe_xml = nf.get('nfe_xml_gerado')
                if not nfe_xml:
                    msg.append('%s/%s' % (str(nf.get('numero_nf')), nf.get('serie_nf')))
                    continue

                name_nfe_xml = nfe_xml_chave_dados.get(nf_id)
                if not nf.get('nf_propria'):
                    name_nfe_xml = name_nfe_xml.replace('-procNFe.xml', '_terceiro.xml')
                try:
                    diretorio_nfe_tmp = '/tmp/' + name_nfe_xml
                    arquivo = open(diretorio_nfe_tmp, 'wb')
                except:
                    raise osv.except_osv(_('Atencao'), _('Diretorio nao existe ou sem permissao para '
                                                         'gravacao.'))
                try:
                    name_nfe_xml = base64.encodestring(nfe_xml.encode('utf-8', 'strict'))
                    name_nfe_xml = base64.decodestring(name_nfe_xml)
                except:
                    name_nfe_xml = nfe_xml
                arquivo.write(name_nfe_xml)
                arquivo.close()
                if nfe_xml_chave_dados.get(nf_id)[20:22] == '55':
                    archname = 'NF-e/'
                elif nfe_xml_chave_dados.get(nf_id)[20:22] == '57':
                    archname = 'CT-e/'
                else:
                    archname = 'NFC-e/'
                archname += nfe_xml_chave_dados.get(nf_id)
                arquivo_zipado.write(diretorio_nfe_tmp, archname)
                diretorios.append(diretorio_nfe_tmp)
                if state_nf == 'C':
                    name_nfe_xml = nfe_xml_chave_dados.get(nf_id).replace('-procNFe.xml', '-procEventoNFe.xml')
                    diretorio_nfe_tmp = '/tmp/' + name_nfe_xml
                    arquivo = open(diretorio_nfe_tmp, 'wb')
                    xml_canc = nf.get('nfe_xml_cancelamento')
                    try:
                        xml = base64.encodestring(xml_canc.encode('utf-8', 'strict'))
                        xml = base64.decodestring(xml)
                    except:
                        xml = xml_canc
                    if not xml:
                        msg.append('%s/%s sem xml de Cancelamento' % (str(nf.get('numero_nf')), nf.get('serie_nf')))
                        continue
                    arquivo.write(xml)
                    arquivo.close()
                    if nfe_xml_chave_dados.get(nf_id)[20:22] == '55':
                        archname = 'NF-e/'
                    elif nfe_xml_chave_dados.get(nf_id)[20:22] == '57':
                        archname = 'CT-e/'
                    else:
                        archname = 'NFC-e/'
                    archname += name_nfe_xml
                    arquivo_zipado.write(diretorio_nfe_tmp, archname)
                    diretorios.append(diretorio_nfe_tmp)

            os.system('rm ' + ' '.join(diretorios))

        msg = msg and _('As Notas Fiscais Eletronicas de numero/serie a seguir nao possuem xml:\n' +
                        ', '.join(msg)) or ''
        arquivo_zipado.close()
        arquivo_zipado = open(diretorio_arquivo, 'r')
        dados_arquivo = base64.encodestring(arquivo_zipado.read())
        arquivo_zipado.close()
        subprocess.call(['rm', diretorio_arquivo])
        return {'dados': dados_arquivo, 'msg': msg}

    def wexxml_get_chave_xml_nfe(self, cr, uid, nf_ids, context=None):
        """
        O metodo reune em um conjunto de dados os identificadores de cada nota fiscal recebida em
        nf_ids.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param nf_ids: lista de id da tabela fiscal_nf_capa.
        :param context: dados de valores contextuais.
        :type cr: object
        :type uid: int
        :type nf_ids: list
        :type context: dict
        :return: dicionario, tendo como chave o id da nota fiscal e seu valor eh uma sequencia de
        caracteres que a identifica.
        :rtype: dict
        """
        if not nf_ids:
            raise osv.except_osv(_('Atencao'), _('Nao e possivel montar a chave. Nenhuma Nota '
                                                 'Fiscal informada.'))
        context = context and context.copy() or {}
        res = {}
        obj_fiscal_modelo_nf_series = self.pool.get('fiscal.modelo.nf.series')
        dados_nf_capa = self.pool.get('fiscal.nf.capa').read(
            cr, uid, nf_ids, ['nfe_chave_acesso', 'numero_nf', 'serie_nf', 'modelo_nf_series_id', 'nfe_chave_cte'],
            context=context, load='_classic_write')
        for nf in dados_nf_capa:
            chave = nf.get('nfe_chave_acesso') or nf.get('nfe_chave_cte')
            if chave:
                chave += '-procNFe.xml'
            else:
                numero_nf = 0
                serie_nf = '0'
                if nf.get('modelo_nf_series_id'):
                    browse_modelo_nf_series = obj_fiscal_modelo_nf_series.browse(
                        cr, uid, nf.get('modelo_nf_series_id'), context=context)
                    serie_nf = browse_modelo_nf_series.serie_nf
                    numero_nf = browse_modelo_nf_series.numero_nf
                chave = '%041d%s.xml' % (numero_nf, serie_nf.rjust(3, '0'))
            res[nf.get('id')] = chave

        return res

    def wexxml_onchange_nfe(self, cr, user, ids, state, context=None):
        """
        Habilita ou desabilita a opcao de NF-e de terceiro
        :param cr: cursor no banco de dados.
        :param user: id da tabela res_users, identificador do usuario.
        :param ids: lista de ID do formulario.
        :param state: estado da selecao de notas fiscais eletronicas (habilitado/desabilitado)
        :param context: dados de valores contextuais.
        :type cr:
        :type user:
        :type ids:
        :type state:
        :type context:
        :return:
        :rtype:
        """
        if not state:
            return {'value': {'wexxml_nfe_terceiro': state}}

        return {}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
