# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from dateutil.parser import parse
import time
from SigUtils.impressao import formataString
from SigUtils.numero import getNumeroEditado
from random import randint
import base64


class wizard_gerar_dapi(osv.TransientModel):
    # prefixo: wizdapi_
    _name = 'wizard.gerar.dapi'
    _columns = {
        'wizdapi_substituicao': fields.boolean('Substitui DAPI ja entregue'),
        'wizdapi_periodo_inicial': fields.date('Data inicial', required=True),
        'wizdapi_periodo_final': fields.date('Data final', required=True),
        'wizdapi_data_limite': fields.date('Data limite para pagamento'),
        'wizdapi_regime_recolhimento': fields.selection(
            [('01', 'Debito/Credito'), ('03', 'Isento ou Imune'), ('29', 'Micro produtor-PJ'),
             ('30', 'Produto pequeno porte'), ('40', 'Microempresa'),
             ('41', 'Microempresa inscricao coletiva'), ('42', 'EPP-faixa1'), ('43', 'EPP-faixa2'),
             ('44', 'EPP-faixa3'), ('45', 'EPP-faixa4'), ('46', 'EPP-faixa5'), ('47', 'EPP-faixa6'),
             ('48', 'EPP-faixa7'), ('49', 'EPP-faixa8'), ('50', 'EPP-faixa9'),
             ('51', 'EPP-faixa10')], 'Regime de recolhimento', size=2),
        'wizdapi_escrituracao_centralizada': fields.boolean('Escrituracao centralizada'),
        'wizdapi_fundese': fields.boolean('Optante pelo FUNDESE'),
        'wizdapi_arquivo_dapi': fields.binary('Arquivo DAPI', readonly=True),
        'wizdapi_nome_arquivo_dapi': fields.char('Nome do arquivo', size=50),
    }

    _defaults = {
        'wizdapi_nome_arquivo_dapi': 'DAPI.txt',
    }

    def wizdapi_button_print(self, cr, uid, ids, context=None):
        """
        O metodo reune os dados necessarios para que sejam utilizados na impressao do arquivo em pdf
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_gerar_dapi = self.browse(cr, uid, ids, context)
        dados = {}
        for dapi in brw_gerar_dapi:
            movimento_dados = self.wizdapi_get_movimento_dados(
                cr, uid, dapi.wizdapi_periodo_inicial, dapi.wizdapi_periodo_final, context)
            desc_regime_recolhimento = self._wizdapi_get_regime_recolhimento(
                cr, uid, dapi.wizdapi_regime_recolhimento, context)
            data_limite = dapi.wizdapi_data_limite or ''
            if data_limite:
                data_limite = parse(data_limite).strftime('%d/%m/%Y')
            dados.update({'substituicao': 'S' if dapi.wizdapi_substituicao else 'N',
                          'periodo_inicial': parse(dapi.wizdapi_periodo_inicial).strftime(
                              '%d/%m/%Y'),
                          'periodo_final': parse(dapi.wizdapi_periodo_final).strftime('%d/%m/%Y'),
                          'data_limite': data_limite,
                          'regime_recolhimento': desc_regime_recolhimento,
                          'regime_recolhimento_codigo': dapi.wizdapi_regime_recolhimento,
                          'escrituracao_centralizada': 'S' if dapi.wizdapi_escrituracao_centralizada
                          else 'N',
                          'fundese': 'S' if dapi.wizdapi_fundese else 'N',
                          'movimento_dados': movimento_dados})
        self.wizdapi_gera_arquivo_dapi(cr, uid, ids, dados, context)
        return {'type': 'ir.actions.report.xml',
                'report_name': 'gerar_dapi',
                'datas': dados}

    def _wizdapi_get_regime_recolhimento(self, cr, uid, regime_recolhimento, context=None):
        """
        O metodo recebe um char que identifica o tipo do regime de recolhimento e retorna a sua
        descricao completa.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param regime_recolhimento: char que identifica o tipo do regime de recolhimento.
        :type: char
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: string com a descricao completa.
        :rtype: string
        """
        context = context and context.copy() or {}
        res = ''
        if not regime_recolhimento:
            return res
        if regime_recolhimento == '01':
            res = 'Debito/Credito'
        elif regime_recolhimento == '03':
            res = 'Isento ou Imune'
        elif regime_recolhimento == '29':
            res = 'Micro produtor-PJ'
        elif regime_recolhimento == '30':
            res = 'Produto pequeno porte'
        elif regime_recolhimento == '40':
            res = 'Microempresa'
        elif regime_recolhimento == '41':
            res = 'Microempresa inscricao coletiva'
        elif regime_recolhimento == '42':
            res = 'EPP-faixa1'
        elif regime_recolhimento == '43':
            res = 'EPP-faixa2'
        elif regime_recolhimento == '44':
            res = 'EPP-faixa3'
        elif regime_recolhimento == '45':
            res = 'EPP-faixa4'
        elif regime_recolhimento == '46':
            res = 'EPP-faixa5'
        elif regime_recolhimento == '47':
            res = 'EPP-faixa6'
        elif regime_recolhimento == '48':
            res = 'EPP-faixa7'
        elif regime_recolhimento == '49':
            res = 'EPP-faixa8'
        elif regime_recolhimento == '50':
            res = 'EPP-faixa9'
        elif regime_recolhimento == '51':
            res = 'EPP-faixa10'
        return res

    def wizdapi_get_movimento_dados(self, cr, uid, periodo_inicial, periodo_final, context=None):
        """
        O metodo busca o movimento de itens do estoque para as notas fiscais faturadas de entrada
        e saida. Esses dizem respeito somente ao ICMS.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param periodo_inicial: data inicial para a busca do movimento.
        :type: datetime
        :param periodo_final: data final para a busca do movimento.
        :type: datetime
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dados da movimentacao de acordo com o periodo definido.
        :rtype: list[dict]
        """
        context = context and context.copy() or {}
        sql = """
            SELECT
                cfop.fiscfop_dapi                         AS dapi,
                cfop.tipo                                 AS cfop_tipo,
                CASE
                    WHEN cfop.tipo = 'E' AND cfop.movimento_tipo = 'C'
                        THEN 'Compras'
                    WHEN cfop.tipo = 'S' AND cfop.movimento_tipo = 'C'
                        THEN 'Vendas'
                    WHEN cfop.movimento_tipo = 'D'
                        THEN 'Devolucao'
                    WHEN cfop.movimento_tipo = 'T'
                        THEN 'Transferencia'
                    WHEN cfop.movimento_tipo = 'E'
                        THEN 'Remessa'
                    WHEN cfop.movimento_tipo = 'R'
                        THEN 'Retorno'
                    WHEN cfop.movimento_tipo = 'U'
                        THEN 'Outras'
                END                                       AS cfop_movimento,
                CASE
                    WHEN cfop.produto_servico_tipo = 'P'
                        THEN 'Mercadoria / Produto'
                    WHEN cfop.produto_servico_tipo = 'F'
                        THEN 'Transporte'
                    WHEN cfop.produto_servico_tipo = 'M'
                        THEN 'Comunicacao'
                    WHEN cfop.produto_servico_tipo = 'S'
                        THEN 'Servico'
                    WHEN cfop.produto_servico_tipo = 'E'
                        THEN 'Energia Eletrica'
                    WHEN cfop.produto_servico_tipo = 'L'
                        THEN 'Combustivel / Lubrificante'
                    WHEN cfop.produto_servico_tipo = 'O'
                        THEN 'Outros'
                END				                          AS cfop_transacao_des,
                cfop.produto_servico_tipo                 AS cfop_transacao,
                Sum(movimento.vr_total_item_rateado) + COALESCE(Sum(
                    (
                    SELECT Sum(vr_encargo) 
                    FROM fiscal_nf_item_tributacao 
                    WHERE 
                        movimento_itens_id = movimento.id 
                        AND encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                    )
                ), 0) AS vr_contabil,
                Sum(item_tributacao.vr_base_calculo)      AS vr_base_calculo,
                Sum(item_tributacao.vr_encargo)           AS vr_encargo,
                0			                  AS vr_imposto_sem_ap_credito,
                Sum(
                    CASE
                        WHEN cst.tipo = 'I' THEN movimento.vr_total_item_rateado
                        ELSE 0
                    END
                )                                         AS vr_isento_nao_tributado,
                Sum(item_tributacao.vr_reducao_base)
                                              AS vr_base_reduzida,
                Sum(
                    CASE
                        WHEN cst.tipo = 'D' THEN movimento.vr_total_item_rateado
                        ELSE 0
                    END
                )                                         AS vr_diferido,
                0                                         AS vr_suspenso,
                Sum(item_tributacao.vr_icms_substituicao)
                                               AS vr_icms_substituicao,
                Sum(
                    CASE
                        WHEN cst.tipo = 'O' THEN movimento.vr_total_item_rateado
                        ELSE 0
                    END
                )                                         AS vr_outros,
                Sum(COALESCE(item_tributacao.fisnit_vr_fcp_st, 0))        AS vr_fcp
            FROM
                estoque_movimento_itens movimento
            INNER JOIN fiscal_nf_capa nf
                ON nf.id = movimento.nf_id
            INNER JOIN fiscal_nf_item_tributacao item_tributacao
                ON movimento.id = item_tributacao.movimento_itens_id
            INNER JOIN fiscal_cfop cfop
                ON cfop.id = movimento.cfop_id
            INNER JOIN fiscal_encargo encargo
                ON encargo.id = item_tributacao.encargo_id
            INNER JOIN fiscal_cst cst
                ON cst.id = item_tributacao.cst_id
            WHERE
                CASE
                    WHEN nf.tipo = 'E'
                        THEN nf.data_saida_entrada BETWEEN '{periodo_inicial}' AND '{periodo_final}'
                    ELSE nf.data_emissao BETWEEN '{periodo_inicial}' AND '{periodo_final}'
                END
                AND nf.state = '3'
                AND encargo.sigla = 'ICMS'
                AND cfop.fiscfop_dapi IS NOT NULL
                AND cfop.codigo NOT IN ('5929', '6929')
            GROUP BY
                cfop.fiscfop_dapi,
                cfop.tipo,
                cfop.produto_servico_tipo,
                cfop.movimento_tipo
        """.format(periodo_inicial=periodo_inicial+' 00:00:00',
                   periodo_final=periodo_final+' 23:59:59')
        cr.execute(sql)
        return cr.dictfetchall()

    def wizdapi_gera_arquivo_dapi(self, cr, uid, ids, dados, context=None):
        """
        O  metodo gera o aquivo da DAPI, em formato txt, para download. Seu leiaute e definido na
        Secretaria da Fazenda.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, indentificador do usuario.
        :type: integer
        :param dados: dicionario com os dados necessarios para a geracao do aqruivo.
        :type: dict
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True
        :rtype: bool
        """
        context = context and context.copy() or {}

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'wizard.gerar.dapi',
                                                                    context=context)
        brw_res_company = self.pool.get('res.company').browse(cr, uid, company_id, context=context)
        nome_empresa = brw_res_company.name
        ie = brw_res_company.rescon_pessoa_id.admpes_jur_ie or ''
        cnpj = brw_res_company.rescon_pessoa_id.cpf_cnpj
        cidade = brw_res_company.rescon_pessoa_id.cidade or ''
        uf = brw_res_company.rescon_pessoa_id.uf

        substituicao = dados.get('substituicao')
        periodo_inicial = dados.get('periodo_inicial')
        periodo_final = dados.get('periodo_final')
        data_limite = dados.get('data_limite')
        regime_recolhimento_codigo = dados.get('regime_recolhimento_codigo')
        escrituracao_centralizada = dados.get('escrituracao_centralizada')
        fundese = dados.get('fundese')
        movimento_dados = dados.get('movimento_dados')
        if not movimento_dados:
            return True
        movimento_dados = sorted(movimento_dados, key=lambda i: i.get('dapi'))

        numero_total_linhas = 0
        lb = '\r\n'
        nome_arquivo = 'dapi_%s.txt' % time.strftime('%H%M%S')
        f_dapi = open('/tmp/%s' % nome_arquivo, 'wb')

        dado_comum = formataString(ie, 13, ' ', 'D', 'N', 'S')
        dado_comum += formataString(periodo_inicial[6:10], 4, ' ', 'D', 'N', 'S')
        dado_comum += formataString(periodo_inicial[3:5], 2, ' ', 'D', 'N', 'S')
        dado_comum += formataString(periodo_final[:2], 2, ' ', 'D', 'N', 'S')
        dado_comum += formataString(periodo_inicial[:2], 2, ' ', 'D', 'N', 'S')

        # LINHAS 00

        linha00 = formataString('00', 2, ' ', 'D', 'N', 'S')
        linha00 += dado_comum
        linha00 += formataString('01', 2, ' ', 'D', 'N', 'S')
        linha00 += formataString(substituicao, 1, ' ', 'D', 'N', 'S')
        linha00 += formataString('', 7, ' ', 'D', 'N', 'S')
        linha00 += formataString('', 2, ' ', 'D', 'N', 'S')
        linha00 += formataString(regime_recolhimento_codigo, 2, ' ', 'D', 'N', 'S')
        linha00 += formataString('', 1, ' ', 'D', 'N', 'S')
        linha00 += formataString(data_limite, 10, ' ', 'D', 'N', 'S')
        linha00 += formataString(fundese, 1, ' ', 'D', 'N', 'S')
        linha00 += formataString('', 3, ' ', 'D', 'N', 'S')
        linha00 += lb
        numero_total_linhas += 1
        f_dapi.write(linha00)

        # LINHAS 10

        list_campos = ['', 'vr_contabil', 'vr_base_calculo', 'vr_encargo',
                       'vr_imposto_sem_ap_credito', 'vr_isento_nao_tributado', '',
                       'vr_base_reduzida', 'vr_diferido', 'vr_suspenso', 'vr_icms_substituicao',
                       'vr_outros']
        list_dapi = list(set(map(lambda i: i.get('dapi'), movimento_dados)))
        list_dapi.extend([25, 35, 42, 43, 51, 59, 64, 65])
        list_dapi.sort()
        list_subtotal = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        list_total = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        valor_total_geral = 0
        for dapi in list_dapi:
            if dapi in (25, 35, 42, 51, 59, 64):
                for i in range(1, 12):
                    valor = list_subtotal[i]
                    if i == 6 or not valor:
                        continue
                    linha10 = formataString('10', 2, ' ', 'D', 'N', 'S')
                    linha10 += dado_comum
                    linha10 += formataString(dapi, 3, '0', 'E', 'N', 'S')
                    linha10 += formataString(i, 2, '0', 'E', 'N', 'S')
                    linha10 += formataString(getNumeroEditado(
                        valor, dec_point='', thousands_sep=''), 15, '0', 'E', 'N', 'S')
                    linha10 += lb
                    numero_total_linhas += 1
                    f_dapi.write(linha10)
                list_subtotal = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
            if dapi in (43, 65):
                for i in range(1, 12):
                    valor = list_total[i]
                    if i == 6 or not valor:
                        continue
                    linha10 = formataString('10', 2, ' ', 'D', 'N', 'S')
                    linha10 += dado_comum
                    linha10 += formataString(dapi, 3, '0', 'E', 'N', 'S')
                    linha10 += formataString(i, 2, '0', 'E', 'N', 'S')
                    linha10 += formataString(getNumeroEditado(
                        valor, dec_point='', thousands_sep=''), 15, '0', 'E', 'N', 'S')
                    linha10 += lb
                    numero_total_linhas += 1
                    f_dapi.write(linha10)
                valor_total_geral += sum(list_total)
                list_total = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
            dado = filter(lambda i: i.get('dapi') == dapi, movimento_dados)
            for i in range(1, 12):
                valor = sum(map(lambda v: v.get(list_campos[i]) or 0, dado))
                list_subtotal[i] += valor
                list_total[i] += valor
                if i == 6 or not valor:
                    continue
                linha10 = formataString('10', 2, ' ', 'D', 'N', 'S')
                linha10 += dado_comum
                linha10 += formataString(dapi, 3, '0', 'E', 'N', 'S')
                linha10 += formataString(i, 2, '0', 'E', 'N', 'S')
                linha10 += formataString(getNumeroEditado(valor, dec_point='', thousands_sep=''),
                                         15, '0', 'E', 'N', 'S')
                linha10 += lb
                numero_total_linhas += 1
                f_dapi.write(linha10)

        # LINHAS 99

        numero_total_linhas += 1
        linha99 = formataString('99', 2, ' ', 'D', 'N', 'S')
        linha99 += dado_comum
        linha99 += formataString('0', 3, '0', 'D', 'N', 'S')
        linha99 += formataString('0', 3, '0', 'D', 'N', 'S')
        linha99 += formataString('0', 3, '0', 'D', 'N', 'S')
        linha99 += formataString(numero_total_linhas, 4, '0', 'E', 'N', 'S')
        linha99 += formataString(getNumeroEditado(
            valor_total_geral, dec_point='', thousands_sep=''), 20, '0', 'E', 'N', 'S')
        linha99 += lb
        f_dapi.write(linha99)

        f_dapi.close()
        f_dapi = open('/tmp/'+nome_arquivo)
        dapi = f_dapi.readlines()
        f_dapi.close()
        dapi = ''.join(map(str, dapi))

        vals = {
            'wizdapi_arquivo_dapi': base64.encodestring(dapi.encode('utf-8', 'strict')),
        }
        self.write(cr, uid, ids, vals, context)
        return True
