# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from calendar import isleap


class wizard_gerar_destda(osv.TransientModel):
    # prefixo: wizdes_
    _name = 'wizard.gerar.destda'
    _columns = {
        'wizdes_mes': fields.selection([('01', 'Janeiro'),
                                        ('02', 'Fevereiro'),
                                        ('03', 'Marco'),
                                        ('04', 'Abril'),
                                        ('05', 'Maio'),
                                        ('06', 'Junho'),
                                        ('07', 'Julho'),
                                        ('08', 'Agosto'),
                                        ('09', 'Setembro'),
                                        ('10', 'Outubro'),
                                        ('11', 'Novembro'),
                                        ('12', 'Dezembro')], 'Mes', size=2),
        'wizdes_ano': fields.char('Ano', size=4),
        'wizdes_icms_st': fields.float('ICMS de ST', digits=(15, 2), readonly=True),
        'wizdes_icms_ativo_fixo': fields.float('ICMS ativo fixo', digits=(15, 2), readonly=True),
        'wizdes_icms_uso_consumo': fields.float('ICMS uso e consumo', digits=(15, 2),
                                                readonly=True),
        'wizdes_difal_ids': fields.one2many('wizard.gerar.destda.difal', 'wizdif_capa_id', 'Itens'),
    }

    def wizdes_button_find(self, cr, uid, ids, context=None):
        """
        O metodo insere na tabela wizard_gerar_destda_item os dados encontrados na tabela
        fiscal_guia
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True
        :rtype: bool
        """
        context = context and context.copy() or {}
        cr.execute('DELETE FROM wizard_gerar_destda_difal')
        obj_fiscal_guia = self.pool.get('fiscal.guia')
        nf_ids = self.pool.get('fiscal.nf.capa').search(cr, uid, [('state', '=', '3')],
                                                        context=context)
        for destda in self.browse(cr, uid, ids, context):
            args = [('nf_id', 'in', nf_ids)]
            mes = destda.wizdes_mes
            if mes:
                args.append(('mes', '=', mes))
            ano = destda.wizdes_ano
            if ano:
                args.append(('ano', '=', ano))
            guia_ids = obj_fiscal_guia.search(cr, uid, args, context=context)
            brw_fiscal_guia = obj_fiscal_guia.browse(cr, uid, guia_ids, context)
            icms_st = filter(lambda i: not i.guia_recomp_aliq, brw_fiscal_guia)
            icms_ativo_fixo = filter(lambda i: i.guia_recomp_aliq and
                                              i.nf_id.natureza_id.cfop_id.transacao_tipo == 'M',
                                     brw_fiscal_guia)
            icms_uso_consumo = filter(lambda i: i.guia_recomp_aliq and
                                              i.nf_id.natureza_id.cfop_id.transacao_tipo == 'S',
                                     brw_fiscal_guia)
            vals = {
                'wizdes_icms_st': sum(map(lambda i: i.valor, icms_st)),
                'wizdes_icms_ativo_fixo': sum(map(lambda i: i.valor, icms_ativo_fixo)),
                'wizdes_icms_uso_consumo': sum(map(lambda i: i.valor, icms_uso_consumo)),
                'wizdes_difal_ids': self._wizdes_get_valor_difal(cr, uid, destda.id, mes, ano,
                                                                 context)
            }
            self.write(cr, uid, destda.id, vals, context)
        return True

    def _wizdes_get_valor_difal(self, cr, uid, id, mes, ano, context=None):
        """
        O metodo encontra o valor total referente ao Difal, agrupado por estado de origem e destino.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identficador do usuario.
        :type: integer
        :param id: id da tabela desse objeto.
        :type: integer
        :param mes: caracter que representa um dos meses do ano.
        :type: char
        :param ano: caracter que respresenta um ano.
        :type: char
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: lista com os dados que serao incluidos na tabela wizard_gerar_destda_difal.
        :rtype: list
        """
        context = context and context.copy() or {}
        res = []
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        estado_origem = self.pool.get('res.company').browse(
            cr, uid, company_id, context).rescon_pessoa_id.uf
        ultimo_dia = self.wizdes_get_ultimo_dia_mes_ano(cr, uid, mes, ano, context)
        sql = """
            SELECT
                ap.uf                                   AS wizdif_estado,
                Sum(fnit.fisnit_vr_difal_uf_destino)    AS wizdif_valor
            FROM fiscal_nf_item_tributacao fnit
            INNER JOIN fiscal_nf_capa nf
                ON nf.id = fnit.fisnit_capa_id
            INNER JOIN adm_pessoa ap
                ON ap.id = nf.destinatario_id
            WHERE
                fnit.fisnit_vr_difal_uf_destino > 0
                AND nf.state = '3'
                AND (nf.data_emissao BETWEEN '{ano}-{mes}-01 00:00:00' AND
                                             '{ano}-{mes}-{ultimo_dia} 23:59:59'
                    OR
                    nf.data_saida_entrada BETWEEN '{ano}-{mes}-01 00:00:00' AND
                                             '{ano}-{mes}-{ultimo_dia} 23:59:59'
                )
            GROUP BY ap.uf
        """.format(ano=ano, mes=mes, ultimo_dia=ultimo_dia)
        cr.execute(sql)
        dados_destino = cr.dictfetchall()
        if not dados_destino:
            return res
        sql = """
            SELECT
                '{estado_origem}'                         AS wizdif_estado,
                Sum(fnit.fisnit_vr_difal_uf_origem)       AS wizdif_valor
            FROM fiscal_nf_item_tributacao fnit
            INNER JOIN fiscal_nf_capa nf
                ON nf.id = fnit.fisnit_capa_id
            WHERE
                fnit.fisnit_vr_difal_uf_origem > 0
                AND nf.state = '3'
                AND (nf.data_emissao BETWEEN '{ano}-{mes}-01 00:00:00' AND
                                             '{ano}-{mes}-{ultimo_dia} 23:59:59'
                    OR
                    nf.data_saida_entrada BETWEEN '{ano}-{mes}-01 00:00:00' AND
                                             '{ano}-{mes}-{ultimo_dia} 23:59:59'
                )
        """.format(ano=ano, mes=mes, ultimo_dia=ultimo_dia, estado_origem=estado_origem)
        cr.execute(sql)
        dados_origem = cr.dictfetchall()
        dados_destino.extend(dados_origem)
        map(lambda i: res.append((0, 0, i)), dados_destino)
        return res

    def wizdes_get_ultimo_dia_mes_ano(self, cr, uid, mes, ano, context=None):
        """
        O metodo recebe strings que representam algum mes e ano para encontrar o ultimo dia
        referente a eles. Identifica, inclusive, se o ano e bissexto.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param mes: caracter que representa um dos meses do ano.
        :type: char
        :param ano: caracter que respresenta um ano.
        :type: char
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: caracter que representa o ultimo dia do mes e ano recebidos.
        :rtype: char
        """
        context = context and context.copy() or {}
        if mes in ('1', '3', '5', '7', '8', '10', '12'):
            ultimo_dia = '31'
        elif mes != '2':
            ultimo_dia = '30'
        else:
            ultimo_dia = '28'
            if isleap(int(ano)):
                ultimo_dia = '29'
        return ultimo_dia

    def wizdes_button_print(self, cr, uid, ids, context=None):
        """
        O metodo reune todos os dados num dicionario. Esses serao recebidos na classe do relatorio
        e serao processados para gerar o arquivo pdf.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario com dados para a chamada do relatorio.
        :rtype: dict
        """
        context = context and context.copy() or {}
        dados = {}
        for destda in self.browse(cr, uid, ids, context):
            dados.update({
                'mes': destda.wizdes_mes,
                'ano': destda.wizdes_ano,
                'icms_st': destda.wizdes_icms_st or 0,
                'icms_ativo_fixo': destda.wizdes_icms_ativo_fixo or 0,
                'icms_uso_consumo': destda.wizdes_icms_uso_consumo or 0,
                'difal_ids': map(lambda i: i.id, destda.wizdes_difal_ids)
            })
        return {'type': 'ir.actions.report.xml',
                'report_name': 'listagem_destda',
                'datas': dados}


class wizard_gerar_destda_difal(osv.TransientModel):
    # prefixo: wizdif_
    _name = 'wizard.gerar.destda.difal'
    _columns = {
        'wizdif_capa_id': fields.many2one('wizard.gerar.destda', 'Capa'),
        'wizdif_estado': fields.char('Estado', size=2),
        'wizdif_valor': fields.float('Valor', digits=(15, 2)),
    }