# -*- coding: utf-8 -*-

from openerp.tools.translate import _
from openerp.osv import osv, fields
import xml.etree.ElementTree as ET
import base64
from dateutil.parser import parse
from datetime import timedelta
import time
import os
import logging
import psycopg2
from openerp.tools.config import config
""" Nao remover import threading, está sendo usando dentro de uma string com exec()"""
import threading


class wizard_importar_cte(osv.TransientModel):
    def __init__(self, pool, cr):
        super(wizard_importar_cte, self).__init__(pool, cr)
        self._erro, self._warn, self._info = [], [], []

    _name = 'wizard.importar.cte'
    _logger = logging.getLogger(_name)
    _columns = {
        'wimcte_arquivo': fields.binary('Arquivo XML do CT-e'),
        'wimcte_diretorio': fields.char('Diretorio dos XMLs dos CT-e', size=128),
        'wimcte_importado': fields.boolean('Arquivo importado'),
        'wimcte_observacao': fields.text(_('Observacoes'), readonly=True),
        'wimcte_data_entrada_saida': fields.selection([('H', 'Hoje'),('D', 'Data emissao')], 'Data de entrada/Saida',
                                                      help='Data a ser considerada para entrada do Cte.', required=True),
        'wimcte_state_cte': fields.selection([('E', 'Esboco'), ('F', 'Faturado')], 'Status do CTE',
                                                      help='Status do CTE apos a importacao', required=True),
        'wimcte_custificar': fields.boolean('Custificar cte'),
        'wimcte_reprocessar': fields.boolean('Reprocessar cte', help="Se marcado e o CTE ja tiver sido importado, excluir e importa novamente, caso contrario, ignora"),
    }
    _defaults = {
        'wimcte_importado': False,
        'wimcte_data_entrada_saida': 'H',
        'wimcte_state_cte': 'E',
        'wimcte_custificar': True,
    }
    def _wimcte_importar_um_cte(self, cr, uid, ids, xml, arq, context=None):
        context = context and context.copy() or {}
        cte_id = False
        try:
            dados = self.read(cr, uid, ids[0], ['wimcte_data_entrada_saida',
                                                'wimcte_state_cte',
                                                'wimcte_custificar',
                                                'wimcte_reprocessar'],
                                           context=context)
            if not dados.get('wimcte_custificar'):
                context['gerencial_nao_custificar'] = True
            cte_id = self.wimcte_processar_xml_cte(cr, uid, xml,
                                                   dados.get('wimcte_data_entrada_saida'),
                                                   dados.get('wimcte_state_cte'), 
                                                   reprocessar=dados.get('wimcte_reprocessar'),
                                                   context=context)
            cr.commit()
        except:
            pass
        if not cte_id:
            self._erro.append('Arquivo nao importado: %s' % arq)
            cr.rollback()
            return False
        self._wimcte_validar_um_cte(cr, uid, ids, cte_id, arq, context=context)
        self._wimcte_faturar_um_cte(cr, uid, ids, cte_id, arq, context=context)
        return cte_id

    def _wimcte_validar_um_cte(self, cr, uid, ids, cte_id, arq, context=None):
        _spg = self.pool.get
        _cond_while = True
        while _cond_while:
            try:
                importar_cte_dados = self.read(cr, uid, ids[0], ['wimcte_state_cte'], context)
                if importar_cte_dados.get('wimcte_state_cte') == 'F':
                    dados_cte = _spg('fiscal.nf.capa').read(cr, uid, cte_id, ['state'], context=context)
                    if dados_cte.get('state') == '1':
                        context['nota_referenciada'] = True
                        result = _spg('fiscal.nf.capa').button_validar(cr, uid, [cte_id], context)
                        cr.commit()
                _cond_while = False
            except Exception as e:
                self._erro.append('Arquivo %s nao validado, id cte: %s - %s' % (arq, cte_id, e))
                cr.rollback()
        return

    def _wimcte_faturar_um_cte(self, cr, uid, ids, cte_id, arq, context=None):
        _spg = self.pool.get
        _cond_while = True
        while _cond_while:
            try:
                importar_cte_dados = self.read(cr, uid, ids[0], ['wimcte_state_cte'], context)
                if importar_cte_dados.get('wimcte_state_cte') == 'F':
                    dados_cte = _spg('fiscal.nf.capa').read(cr, uid, cte_id, ['state'], context=context)
                    if dados_cte.get('state') == '2':
                        context['nota_referenciada'] = True
                        result = _spg('fiscal.nf.capa').button_faturar(cr, uid, [cte_id], context)
                        cr.commit()
                _cond_while = False
            except Exception as e:
                self._erro.append('Arquivo %s nao faturado, id cte: %s - %s' % (arq, cte_id, e))
                cr.rollback()
        return

    def wimcte_botao_importar(self, cr, uid, ids, context=None):
        """
        Metodo chamado pelo clique do botao 'Importar'
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: lista com mensagens de erro e avisos
        :rtype: list
        """
        def __inicio__(x):
            pass

        inicio = time.time()
        context = context and context.copy() or {}
        importar_cte_dados = self.read(cr, uid, ids[0], ['wimcte_arquivo', 'wimcte_diretorio'], context)
        if importar_cte_dados.get('wimcte_arquivo'):
            xml = base64.b64decode(importar_cte_dados.get('wimcte_arquivo'))
            cte_id = self._wimcte_importar_um_cte(cr, uid, ids, xml, '', context=context)
            erros = self._erro + self._warn
            if erros:
                self.write(cr, uid, ids, {'wimcte_observacao':  '- ' + '\n- '.join(erros)}, context)
            elif cte_id:
                self.write(cr, uid, ids, {'wimcte_importado': True})
            else:  # Erro inesperado
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao foi possivel importar o arquivo XML.'))

            return cte_id
        else:
            _diretorio = importar_cte_dados.get('wimcte_diretorio')
            if _diretorio:
                novaLista = []
                try:
                    lista = os.listdir(_diretorio)
                    for linha in lista:
                        if linha[-3:] != 'xml':
                            continue
                        novaLista.append(linha)
                except Exception as e:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Erro na importacao dos XMLs %s') % e)
                for x in range(3):
                    x += 1
                    exec("T%s = threading.Thread(target=__inicio__,args=(1,))" % x)
                    exec("_cr%s = self.pool.db.cursor()" % x)
                if novaLista:
                    i = 0
                    for linha in novaLista:
                        i += 1
                        th = 0
                        while True:
                            th += 1
                            if th > 3:
                                time.sleep(0.1)
                                th = 1
                            exec ("_thread = T%s.isAlive()" % th)
                            if _thread:  # Se a Tread T? estiver em execução tentar a proxima thread
                                continue
                            break
                        _nomearq = _diretorio + '/' + linha
                        _arq = open(_nomearq, 'r')
                        xml = _arq.read()
                        _arq.close()
                        self._logger.info('Importando CTEs:  %s/%s %s' % (i, len(novaLista), linha))
                        exec("T%s = threading.Thread(target=self._wimcte_importar_um_cte, args=(_cr%s, uid, ids, xml, linha, context))" % (th, th))
                        exec('T%s.start()' % th)
                th = 0
                while True:
                    th += 1
                    if th > 3:
                        break
                    exec("_thread = T%s.isAlive()" % th)
                    if _thread:
                        time.sleep(0.1)
                        th -= 1
                        continue
                    exec("_cr%s.close()" % th)

                if self._erro:
                    self.write(cr, uid, ids, {'wimcte_observacao': 'Arquivos com problemas:' + '\n'.join(self._erro)}, context)
                    erros = self._erro + self._warn
                    if erros:
                        self.write(cr, uid, ids, {'wimcte_observacao': '- ' + '\n- '.join(erros)}, context)
                    self.write(cr, uid, ids, {'wimcte_importado': True})
                self._logger.info('Fim de importacao de CTe!')

            else:
                raise osv.except_osv(_('Atencao!'),
                                     _('Precisa informar um XML ou um diretorio de XMLs'))
            fim = time.time()
            tempo = '%f' % (fim - inicio)
            print('Fim de importacao - Tempo: %s' % tempo)
            return True

    def wimcte_processar_xml_cte(self, cr, uid, xml, tipo_data='D', status='E', reprocessar=False, context=None):
        """
        Metodo responsavel por processar o xml e gerar a nota fiscal de entrada de terceiros
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param xml: arquivo XML do CT-e
        :type xml: str
        :param context: dados contextuais
        :type context: dict
        :return: o ID da NF de terceiros (CT-e)
        :rtype: long
        """
        context = context and context.copy() or {}
        vals_nf = {'tipo': 'E', 'nf_propria': False, 'modelo_55': True}
        msg_arquivo_invalido = _('O arquivo informado nao e um arquivo CT-e valido')
        try:
            xml_cte = ET.fromstring(xml)
        except Exception:
            raise osv.except_osv(_('Atencao!'), msg_arquivo_invalido)

        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_fiscal_modelo_nf = self.pool.get('fiscal.modelo.nf')
        obj_fiscal_cfop = self.pool.get('fiscal.cfop')
        obj_fiscal_cfop_natureza = self.pool.get('fiscal.cfop.natureza')
        obj_fiscal_encargo = self.pool.get('fiscal.encargo')
        obj_fiscal_cst = self.pool.get('fiscal.cst')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        obj_adm_cidade = self.pool.get('adm.cidade')
        obj_adm_uf = self.pool.get('adm.uf')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')

        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', ['cadastro_automatico_fornecedor'], context=context)
        ns = {'cte': 'http://www.portalfiscal.inf.br/cte'}

        # FIX: Validar o arquivo XML pelo XSD
        if xml_cte.tag[len(ns['cte']) + 2:] != 'cteProc':
            xml_cte = xml_cte.find('.//cte:cteProc', ns) or {}
        try:
            infcte = xml_cte.find('.//cte:infCte', ns)
        except Exception:
            raise osv.except_osv(_('Atencao!'), msg_arquivo_invalido)

        ide = infcte.find('cte:ide', ns)
        emit = infcte.find('cte:emit', ns)
        vprest = infcte.find('cte:vPrest', ns)
        enderemit = emit.find('cte:enderEmit', ns)
        infctenorm = infcte.find('cte:infCTeNorm', ns)
        infdoc = infctenorm.find('cte:infDoc', ns)
        imp = infcte.find('cte:imp', ns)

        chave_cte = infcte.attrib['Id'][3:]
        numero_nf = int(ide.find('cte:nCT', ns).text)
        serie = ide.find('cte:serie', ns).text
        str_data = ide.find('cte:dhEmi', ns).text
        horas = str_data[20:22]
        minutos = str_data[23:25]
        op = str_data[19:20]
        if horas:
            dif = timedelta(hours=int(horas), minutes=int(minutos))
            # calcula a data em UTC
            if op == '-':
                str_data = str(parse(str_data) + dif)[:19]
            elif op == '+':
                str_data = str(parse(str_data) - dif)[:19]

        data_emissao = str_data
        ide_cmunini = int(ide.find('cte:cMunIni', ns).text)
        ide_cmunfim = int(ide.find('cte:cMunFim', ns).text)
        ide_ufini = ide.find('cte:UFIni', ns).text
        ide_uffim = ide.find('cte:UFFim', ns).text
        mod = ide.find('cte:mod', ns).text
        emit_cnpj = emit.find('cte:CNPJ', ns).text
        emit_nome = emit.find('cte:xNome', ns).text
        emit_nome_fantasia = emit.find('cte:xFant', ns)
        emit_nome_fantasia = emit_nome_fantasia and emit_nome_fantasia.text
        emit_ie = emit.find('cte:IE', ns).text
        enderemit_uf = enderemit.find('cte:UF', ns).text
        enderemit_xmun = enderemit.find('cte:xMun', ns).text
        enderemit_xbairro = enderemit.find('cte:xBairro', ns).text
        enderemit_xlgr = enderemit.find('cte:xLgr', ns).text
        enderemit_cep = enderemit.find('cte:CEP', ns) != None and \
                        enderemit.find('cte:CEP', ns).text or ''
        enderemit_nro = enderemit.find('cte:nro', ns).text
        enderemit_cmun = enderemit.find('cte:cMun', ns).text
        valor_receber = float(vprest.find('cte:vRec', ns).text)
        versao_emissao = infcte.attrib.get('versao')

        # Modelo NF para CT-e
        modelo_nf_ids = obj_fiscal_modelo_nf.search(
            cr, uid, [('fismnf_codigo', '=', mod)], context=context)
        not modelo_nf_ids and self._erro.append(
            _('Modelo de nota fiscal %s nao cadastrado para CT-e') % mod)
        modelo_nf_id = modelo_nf_ids[0]
        vals_nf.update({'modelo_nf_id': modelo_nf_id})

        # Pessoa (destinatario/fornecedor)
        pessoa_ids = obj_adm_pessoa.search(cr, uid, [('cpf_cnpj', '=', emit_cnpj)], context=context)
        pessoa_id = pessoa_ids and pessoa_ids[0] or False
        if not pessoa_id:
            vals_pessoa = {'cpf_cnpj': emit_cnpj, 'pessoa_tipo': 'J', 'name': emit_nome,
                           'fantasia_apelido': emit_nome_fantasia, 'admpes_jur_ie': emit_ie}
            pessoa_id = obj_adm_pessoa.create(cr, uid, vals_pessoa, context)

        # Existencia da NF (CT-e) na base
        args_nf = [('numero_nf', '=', numero_nf), ('serie_nf', '=', serie),
                   ('data_emissao', '>=', data_emissao[:10] + ' 00:00:00'),
                   ('data_emissao', '<=', data_emissao[:10] + ' 23:59:59'),
                   ('state', '!=', 'C'), ('destinatario_id', '=', pessoa_id), ('tipo', '=', 'E')]
        cte_ids = obj_fiscal_nf_capa.search(cr, uid, args_nf, context=context)
        if cte_ids:
            if reprocessar:
                context['nf_capa_manutencao'] = True
                res = obj_fiscal_nf_capa.unlink(cr, uid, cte_ids, context=context)
                context.pop('nf_capa_manutencao')
            else:
                self._info.append(_('A nota fiscal de numero %s ja esta cadastrada') % numero_nf)
                return cte_ids[0]

        # Fornecedor
        fornecedor_ids = obj_financeiro_fornecedor.search(
            cr, uid, [('pessoa_id', '=', pessoa_id)], context={'active_test': False})
        fornecedor_id = fornecedor_ids and fornecedor_ids[0] or False
        if fornecedor_id:
            fornecedor_ativo = obj_financeiro_fornecedor.read(
                cr, uid, fornecedor_id, ['active'], {'active_test': False}).get('active')
            (
                not fornecedor_ativo
                and financeiro_parametro_dados.get('cadastro_automatico_fornecedor')
            ) and obj_financeiro_fornecedor.write(cr, uid, fornecedor_id, {'active': True},
                                                  context={'active_test': False}) \
                or (not fornecedor_ativo and self._warn.append(
                _('Fornecedor %s (CNPJ %s) encontra-se inativo') % (emit_nome, emit_cnpj)))
        else:
            financeiro_parametro_dados.get('cadastro_automatico_fornecedor') \
                and obj_financeiro_fornecedor.create(cr, uid, {'pessoa_id': pessoa_id}, context) \
                or self._warn.append(_('Fornecedor %s (CNPJ %s) nao esta cadastrado') % (
                    emit_nome, emit_cnpj))

        # Endereco
        endereco_id = obj_adm_pessoa_endereco.str2address(
            cr, uid, pessoa_id, 'Brasil', enderemit_uf, enderemit_xmun, enderemit_xbairro,
            enderemit_xlgr, enderemit_cep, enderemit_nro, IntCodigoIBGE=int(enderemit_cmun),
            context=context)
        # Atualiza o endereco da NF-e no cadastro de pessoa
        dados_pessoa = obj_adm_pessoa.read(cr, uid, pessoa_id, ['endereco_nf_id'], context=context)['endereco_nf_id']
        if dados_pessoa and dados_pessoa[0] == endereco_id:
            pass
        else:
            endereco_id and obj_adm_pessoa.write(cr, uid, pessoa_id, {'endereco_nf_id': endereco_id}, context)

        # CFOP e natureza fiscal
        cfop_xml = ide.find('cte:CFOP', ns).text
        if cfop_xml.startswith('6'):
            cfop = '2353'
        else:
            cfop = '1353'
        cfop_ids = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop)], context=context)
        not cfop_ids and self._warn.append(_('CFOP para entrada %s, nao cadastrado') % cfop)
        cfop_natureza_ids = obj_fiscal_cfop_natureza.search(
            cr, uid, [('cfop_id', 'in', cfop_ids)], context=context)
        not cfop_natureza_ids and self._warn.append(
            _('Natureza fiscal nao cadastrada para o CFOP %s') % cfop)
        vals_nf.update({'natureza_id': cfop_natureza_ids and cfop_natureza_ids[0] or False})

        # Estado e cidade de origem e destino
        origem_uf_ids = obj_adm_uf.search(cr, uid, [('sigla', '=', ide_ufini)], context=context)
        origem_uf_id = origem_uf_ids and origem_uf_ids[0] or False
        not origem_uf_ids and self._warn.append(
            _('Estado de origem %s nao cadastrado') % ide_ufini)
        origem_cidade_ids = obj_adm_cidade.search(
            cr, uid, [('codigo_ibge', '=', ide_cmunini)], context=context)
        origem_cidade_id = origem_cidade_ids and origem_cidade_ids[0] or False
        not origem_cidade_ids and self._warn.append(
            _('Cidade de origem %s nao cadastrada') % ide_ufini)
        destino_uf_ids = obj_adm_uf.search(cr, uid, [('sigla', '=', ide_uffim)], context=context)
        destino_uf_id = destino_uf_ids and destino_uf_ids[0] or False
        not destino_uf_ids and self._warn.append(
            _('Estado de destino %s nao cadastrado') % ide_ufini)
        destino_cidade_ids = obj_adm_cidade.search(
            cr, uid, [('codigo_ibge', '=', ide_cmunfim)], context=context)
        destino_cidade_id = destino_cidade_ids and destino_cidade_ids[0] or False
        not destino_cidade_ids and self._warn.append(
            _('Cidade de destino %s nao cadastrada') % ide_ufini)
        _dt_entrada = time.strftime('%Y-%m-%d %H:%M:%S')
        if tipo_data == 'D':
            _dt_entrada = data_emissao
        vals_nf.update({
            'nfe_chave_cte': chave_cte, 'numero_nf': numero_nf, 'serie_nf': serie,
            'data_emissao': data_emissao, 'destinatario_id': pessoa_id,
            'destinatario_endereco_id': endereco_id,
            'data_saida_entrada': _dt_entrada,
            'fisnfc_origem_uf_id': origem_uf_id, 'fisnfc_origem_cidade_id': origem_cidade_id,
            'fisnfc_destino_uf_id': destino_uf_id, 'fisnfc_destino_cidade_id': destino_cidade_id,
            'nfe_versao_processo_emissao': versao_emissao,
            'fisnfc_cte_xml': xml,
        })

        # Impostos
        icms_encargo_id = obj_fiscal_encargo.search(
            cr, uid, [('sigla', '=', 'ICMS')], context=context)[0]
        tributacoes_ids = []
        for imposto in imp.getchildren():
            for imp in imposto.getchildren():
                cst = imp.find('cte:CST', ns).text
                vr_base_calculo = imp.find('cte:vBC', ns)
                vr_base_calculo = float(vr_base_calculo.text) if isinstance(vr_base_calculo, ET.Element) else 0
                aliquota = imp.find('cte:pICMS', ns)
                aliquota = float(aliquota.text) if isinstance(aliquota, ET.Element) else 0
                vr_imposto = imp.find('cte:vICMS', ns)
                vr_imposto = float(vr_imposto.text) if isinstance(vr_imposto, ET.Element) else 0
                encargo_id = False
                encargo = ''
                if 'ICMS' in imposto.tag:
                    encargo_id = icms_encargo_id
                    encargo = 'ICMS'
                cst_ids = obj_fiscal_cst.search(
                    cr, uid, [('encargo_id', '=', encargo_id), ('codigo', '=', cst)],
                    context=context)
                not cst_ids and self._warn.append(
                    _('CST nao cadastrado para o codigo %s e encargo %s') % (cst, encargo))
                tributacao_ids = obj_fiscal_tributacao.search(
                    cr, uid, [('encargo_id', '=', encargo_id), ('cst_id', 'in', cst_ids),
                              ('aliquota', '=', aliquota)], context=context)
                not tributacao_ids and self._warn.append(
                    _('Tributacao nao cadastrada para o encargo %s, CST %s e aliquota %s') %
                    (encargo, cst, aliquota))
                vals_tributacao = {
                    'encargo_id': encargo_id,
                    'cst_id': cst_ids and cst_ids[0] or False,
                    'tributacao_id': tributacao_ids and tributacao_ids[0] or False,
                    'aliquota': aliquota,
                    'vr_base_calculo': vr_base_calculo,
                    'vr_encargo': vr_imposto
                }
                company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
                pessoa_id = self.pool.get('res.company').read(
                    cr, uid, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                    'rescon_pessoa_id')
                _fields = ['admpes_jur_regime_tributario', 'fispar_sublimite_estadual', 'fispar_receita_anual']
                dados_par = self.pool.get('adm.pessoa').read(cr, uid, pessoa_id, _fields, context)
                simples = False
                if (dados_par.get('admpes_jur_regime_tributario') == 'S' and
                   dados_par.get('fispar_receita_anual') < dados_par.get('fispar_sublimite_estadual')):
                       simples = True
                if simples: #Para empresas do simples, não tributar o icms
                    cst_ids = obj_fiscal_cst.search(
                        cr, uid, [('encargo_id', '=', encargo_id), ('codigo', '=', '90')],
                        context=context)
                    not cst_ids and self._warn.append(
                        _('CST nao cadastrado para o codigo %s e encargo %s') % ('90', encargo))
                    tributacao_ids = obj_fiscal_tributacao.search(
                        cr, uid, [('encargo_id', '=', encargo_id), ('cst_id', 'in', cst_ids)], context=context)
                    vals_tributacao.update({
                        'cst_id': cst_ids and cst_ids[0] or False,
                        'tributacao_id': tributacao_ids and tributacao_ids[0] or False,
                        'aliquota': 0,
                        'vr_encargo': 0,
                    })
                tributacoes_ids.append((0, 0, vals_tributacao))
                break
            break
        tributacoes_ids and vals_nf.update({'fisnfc_frete_tributacoes_ids': tributacoes_ids})

        # Valor a receber
        vals_nf.update({
            'fisnfc_parcelas_terceiros_ids': [(0, 0, {'vencimento': data_emissao[:10],
                                                      'valor': valor_receber})]})
        context.update({'nf_frete': True, 'valor_frete': valor_receber, 'terceiro': True})
        res = obj_fiscal_nf_capa.create(cr, uid, vals_nf, context)
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

