# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.tools.translate import _
from pynfe.nfce.manual_600.NFCe import NFCe as nfce
from openerp.osv import fields, osv
from unicodedata import normalize
from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400
from datetime import datetime
import re


class wizard_inutilizar_faixa_nfe(osv.TransientModel):
    _name = 'wizard.inutilizar.faixa.nfe'
    _description = 'Inutilizar NF-e por faixa de numeracao'

    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', required=True),
        'name': fields.char('Justificativa', size=255, required=True,
                            help="Justificativa da Inutilizacao"),
        'modelo_id': fields.many2one('fiscal.modelo.nf.series', 'Modelo da NF', required=True),
        'faixa_inicial': fields.char('Faixa Inicial', size=8, required=True),
        'faixa_final': fields.char('Faixa Final', size=8, required=True),
        'inutilizado': fields.boolean('inutilizado'),
        'informacao': fields.text('Informacao'),
    }

    _defaults = {
        'company_id': lambda s, cr, u, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, u, s._name, context=c)
    }

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.financeiro = None
            self.fiscal = None
            self.estoque = None
            self.adm = None
            self.rh = None
            self.__dict__.update(entries)

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or dict()
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in ['financeiro', 'fiscal', 'estoque', 'adm',
                                                         'rh']])
        return self.ParamToObject(**parametros)

    def wifnfe_button_confirmar(self, cr, user, ids, context=None):
        """
        Confirma a inutilizacao de uma faixa de numeracao de NF-e.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a faixa foi inutilizada
        :rtype: bool
        """

        return self.inutilizar_faixa_nfe(cr, user, ids, context)

    def inutilizar_faixa_nfe(self, cr, user, ids, context=None):
        """
        Inutiliza uma faixa de numeracao de NF-e.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a faixa foi inutilizada
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['inutilizacao'] = True
        parametro = self._parametros(cr, user)
        if not parametro.fiscal.modelo_nf_series_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Informe um modelo padrao de Nota Fiscal Eletronica no '
                                   'parametro do Fiscal.'))

        # Recupera a primeira natureza da operacao pois na inutilizacao isso eh indiferente
        natureza_ids = self.pool.get('fiscal.cfop.natureza').search(cr, user, [], context=context)
        natureza_id = natureza_ids and natureza_ids[0]
        if not natureza_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao existe nenhuma natureza da operacao cadastrada.'))
        company_id = self.pool.get('res.users').browse(
            cr, user, user, context=context).company_id.id
        pessoa_id = self.pool.get('res.company').read(
            cr, user, company_id, ['rescon_pessoa_id'], context=context)['rescon_pessoa_id'][0]
        brw_pessoa = self.pool.get('adm.pessoa').browse(cr, user, pessoa_id, context)
        estado = False
        for endereco in brw_pessoa.endereco_ids:
            if endereco.endereco_tipo_id.name == 'Comercial':
                estado = endereco.uf_id.sigla
                break
            if endereco.endereco_tipo_id.name == 'Normal':
                estado = endereco.uf_id.sigla
                continue
        if not estado:
            raise osv.except_osv(_('Atencao!'),
                                 _('Cadastre um endereco valido para a empresa.'))
        cnpj = re.sub('[^0-9]', '', brw_pessoa.cpf_cnpj) if brw_pessoa.cpf_cnpj else ''
        if not cnpj:
            raise osv.except_osv(_('Atencao!'), _('Nao existe CNPJ cadastro para o emitente.'))
        brw_inutilizacao = self.browse(cr, user, ids[0], context)
        ini = brw_inutilizacao.faixa_inicial or 1
        fim = brw_inutilizacao.faixa_final or 1
        justificativa = normalize('NFKD', brw_inutilizacao.name).encode('ASCII', 'ignore')
        if len(justificativa) < 15:
            raise osv.except_osv(_('Atencao!'),
                                 _('A justificativa deve ter no minimo 15 caracteres.'))
        modelo = brw_inutilizacao.modelo_id
        serie = modelo and modelo.serie_nf
        ano = datetime.now().strftime(u'%y')
        try:
            if modelo.modelo_nf_id.fismnf_codigo == u'55':
                nfe = nfe_400
                nfe = nfe(certify=parametro.fiscal.certificado_conteudo,
                          key=parametro.fiscal.chave_conteudo,
                          environment=parametro.fiscal.fispar_nfe_ambiente, state=estado,
                          contingency=parametro.fiscal.fispar_nfe_contingencia)
                nfe.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
                nfe.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
                processo = nfe.inutilizar_faixa_numeracao(ano, cnpj, serie, ini, fim, justificativa)
            elif modelo.modelo_nf_id.fismnf_codigo == u'65':
                nfc_e = nfce(
                    parametro.fiscal.certificado_conteudo,  parametro.fiscal.chave_conteudo,
                    modelo.ambiente, estado, False)
                nfc_e.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio or 5
                nfc_e.process.timeout = parametro.fiscal.fispar_nfce_conexao_timeout or 10
                processo = nfc_e.process.inutilizar_faixa_numeracao(ano, cnpj, serie, ini, fim, justificativa)
        except Exception as err:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro de conexao com servidor. \n%s') % err)
        if processo.resposta.infInut.cStat.valor not in (u'102', u'563'):
            raise osv.except_osv(_('Aviso.'),
                                 '%s - %s' % (processo.resposta.infInut.cStat.valor,
                                              processo.resposta.infInut.xMotivo.valor))
        elif processo.resposta.infInut.cStat.valor == u'563':
            numero_protocolo = re.sub('[^0-9]', '', processo.resposta.infInut.xMotivo.valor)
        else:
            numero_protocolo = processo.resposta.infInut.nProt.valor
        informacao = ''
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        for numero_nf in range(int(ini), int(fim) + 1):
            nf_ids = obj_fiscal_nf_capa.search(
                    cr, user, [('modelo_nf_series_id', '=', modelo.id), ('numero_nf', '=', numero_nf),
                               ('serie_nf', '=', serie)], context=context)
            vals_nf = {
                'nfe_protocolo': numero_protocolo,
                'state': 'I', 'nf_propria': True
            }
            if nf_ids:
                nf_id = nf_ids[0]
                vals_nf['nfe_status'] = '102 - Inutilizacao de numero homologado'
            else:
                nf_id = self.pool.get('fiscal.utils').gerar_nf(
                    cr, user, company_id=company_id, destinatario_id=False, natureza_id=natureza_id,
                    tipo='S', modelo_nf_id=modelo.id, numero_nf=numero_nf, serie_nf=serie,
                    context=context)
                vals_nf.update({
                    'nfe_status': '102 - Inutilizacao de numero homologado',
                    'modelo_55': True if modelo.modelo_nf_id.fismnf_codigo == u'55' else False,
                    'fisnfc_consumidor': True if modelo.modelo_nf_id.fismnf_codigo == u'65' else False,
                    'numero_nf': numero_nf, 'serie_nf': serie})
            obj_fiscal_nf_capa.write(cr, user, nf_id, vals_nf, context=context)
            informacao += _('* Nota fiscal inutilizada de numero %s e serie %s\n' %
                            (numero_nf, serie))
        self.write(cr, user, ids, {'inutilizado': True, 'informacao': informacao}, context)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
