# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from datetime import date
from openerp.tools.translate import _


class wizard_listagem_item_tributacao(osv.TransientModel):
    _name = 'wizard.listagem.item.tributacao'
    _description = 'Listagem de itens e suas tributacoes'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa'),
        'data_inicial': fields.date('Data Inicial', required=True),
        'data_final': fields.date('Data Final', required=True),
        'tipo_nf': fields.selection([('E', 'Entrada'),
                                     ('S', 'Saida'),
                                     ('C', 'Cupom')], 'Tipo da nota', size=1),
        'produto_id': fields.many2one('estoque.produto', 'Produto'),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade'),
        'encargo_id': fields.many2one('fiscal.encargo', 'Encargo'),
        'wizlit_cst_id': fields.many2one('fiscal.cst', 'CST'),
        'wizlit_cfop_ids': fields.many2many('fiscal.cfop', 'item_tributacao_m2m_fiscal_cfop',
                                            'item_tributacao_id', 'cfop_id', 'CFOP'),
        'nf_id': fields.many2one('fiscal.nf.capa', 'Nota Fiscal'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Pessoa', ondelete="restrict"),
        'tipo_movimento': fields.selection([('C', 'Compra/Venda'),
                                            ('D', 'Devolucao'),
                                            ('T', 'Transferencia'),
                                            ('E', 'Remessa'),
                                            ('R', 'Retorno'),
                                            ('U', 'Outras')], 'Tipo Movimento', size=1),
        'listar_todos_encargos': fields.boolean(
            'Listar todos os encargos', help='Lista todos os encargos do item que atende a '
                                             'selecao dos filtros.'),
    }

    _defaults = {
        'data_inicial': date.today().strftime('%Y-%m-%d'),
        'data_final': date.today().strftime('%Y-%m-%d'),
        'tipo_nf': 'E',
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id,
        }

    def onchange_wizlit_produto_id(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'grade_id': ''}}
        if not produto_id:
            return res

        grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid,
                                                 [('produto_id', '=', produto_id)], context=context)
        if len(grade_ids) == 1:
            res['value']['grade_id'] = grade_ids[0]

        return res

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_item_tributacao = self.browse(cr, uid, ids[0], context=context)
        if brw_item_tributacao.data_inicial > brw_item_tributacao.data_final:
            raise osv.except_osv(_('Atencao!'),
                                 _('A data inicial nao poder ser maior que a data final.'))
        datas = {
            'company_id': brw_item_tributacao.company_id.id,
            'data_inicial': brw_item_tributacao.data_inicial,
            'data_final': brw_item_tributacao.data_final,
            'tipo_nf': brw_item_tributacao.tipo_nf,
            'produto_id': brw_item_tributacao.produto_id.id,
            'grade_id': brw_item_tributacao.grade_id.id,
            'encargo_id': brw_item_tributacao.encargo_id.id,
            'nf_id': brw_item_tributacao.nf_id.id,
            'pessoa_id': brw_item_tributacao.pessoa_id.id,
            'tipo_movimento': brw_item_tributacao.tipo_movimento,
            'cst_id': brw_item_tributacao.wizlit_cst_id.id,
            'cfop_ids': map(lambda cfop: cfop.id, brw_item_tributacao.wizlit_cfop_ids),
            'dados': self.wizlit_get_dados(cr, brw_item_tributacao)
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_item_tributacao',
            'datas': datas,
        }

    def wizlit_get_dados(self, cr, data, context=None):
        """
        Retorna os dados da consulta SQL em funcao dos parametros do formulario (Listagem de itens 
        e tributacao). -- Nota fiscal --
        
        :param cr: o cursor do banco de dados
        :param data: dodos do formulario (browser)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type data: object.browser
        :type context: dict
        :return: lista de dicionarios com o resultado da consulta
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()
        sql = """
        -- noinspection SqlDialectInspectionForFile
        SELECT      'NF-' || fnc.id           AS documento_id,
                    ap.nome                   AS pessoa_nome,
                    fcn.name                  AS natureza_nome,
                    fc.movimento_tipo         AS cfop_movimento_tipo,
                    fc.codigo                 AS cfop_codigo,
                    'NF' || fnc.numero_nf     AS documento_numero,
                    fnc.serie_nf              AS documento_comp,
                    emc.data                  AS documento_data,
                    fnc.tipo                  AS documento_tipo,
                    ep.codigo                 AS produto_codigo,
                    ep.name                   AS produto_nome,
                    epg.descricao             AS grade_descricao,
                    fe.sigla                  AS encargo_sigla,
                    fct.codigo                AS cst_codigo,
                    emi.vr_total_item_rateado AS item_vr_total,
                    emi.quantidade            AS quantidade,
                    fnit.vr_base_calculo      AS vr_base_calculo,
                    fnit.aliquota             AS aliquota,
                    fnit.vr_encargo           AS vr_encargo,
                    emi.id                    AS emi_id,
                    fnit.vr_base_icms_substituicao
                                              AS vr_base_icms_st,
                    fnit.vr_icms_substituicao AS vr_icms_st
        FROM        estoque_movimento_itens   AS emi
        INNER JOIN  estoque_movimento_capa    AS emc  ON  emc.id = emi.capa_id
        INNER JOIN  fiscal_nf_capa            AS fnc  ON  fnc.id = emi.nf_id
        INNER JOIN  fiscal_nf_item_tributacao AS fnit ON  fnit.movimento_itens_id = emi.id
        INNER JOIN  fiscal_cfop               AS fc   ON  fc.id = emi.cfop_id
        INNER JOIN  fiscal_cst                AS fct  ON  fct.id = fnit.cst_id
        INNER JOIN  fiscal_encargo            AS fe   ON  fe.id = fnit.encargo_id
        INNER JOIN  estoque_produto           AS ep   ON  ep.id = emi.produto_id
        INNER JOIN  estoque_produto_grade     AS epg  ON  epg.id = emi.grade_id
        LEFT JOIN   fiscal_cfop_natureza      AS fcn  ON   fcn.id = fnc.natureza_id
        LEFT JOIN   adm_pessoa                AS ap   ON   ap.id = fnc.destinatario_id
        WHERE       emc.data >= '%s'
          AND       emc.data <= '%s'
        """ % (data.data_inicial, data.data_final)

        if data.pessoa_id:
            sql += """
          AND       fnc.destinatario_id = %s
        """ % data.pessoa_id.id
        if data.tipo_nf:
            sql += """
          AND       fnc.tipo = '%s'
        """ % data.tipo_nf
        if data.tipo_movimento:
            sql += """
          AND       fc.movimento_tipo = '%s'
        """ % data.tipo_movimento
        if data.company_id:
            sql += """
          AND       emc.company_id = %s
        """ % data.company_id.id
        if data.produto_id:
            sql += """
          AND       emi.produto_id = %s
        """ % data.produto_id.id
        if data.grade_id:
            sql += """
          AND       emi.grade_id = %s
        """ % data.grade_id.id
        if data.encargo_id:
            if not data.listar_todos_encargos:
                sql += """
                          AND       fnit.encargo_id = %s
                """ % data.encargo_id.id
        if data.nf_id:
            sql += """
          AND       emi.nf_id = %s
        """ % data.nf_id.id
        if data.wizlit_cst_id:
            if not data.listar_todos_encargos:
                sql += """
                  AND       fnit.cst_id = %s
                """ % data.wizlit_cst_id.id
            else:
                sql += """
                  AND       CASE 
                                WHEN fnit.encargo_id = {encargo_id} THEN fnit.cst_id = {cst_id} 
                                ELSE fnit.encargo_id != {encargo_id} AND 
                                    (SELECT TRUE 
                                     FROM fiscal_nf_item_tributacao 
                                     WHERE 
                                        movimento_itens_id = fnit.movimento_itens_id 
                                        AND encargo_id = {encargo_id} 
                                        AND cst_id = {cst_id})
                            END
                """.format(encargo_id=data.encargo_id.id, cst_id=data.wizlit_cst_id.id)
        if data.wizlit_cfop_ids:
            sql += """
          AND       fc.id IN (%s)
        """ % ', '.join(map(lambda cfop: str(cfop.id), data.wizlit_cfop_ids))
        sql += """
        ORDER BY    emi.nf_id ASC, emi.id,  fnit.encargo_id
        """
        cr.execute(sql)
        result = cr.dictfetchall()

        return result

    def wizlit_onchange_encargo(self, cr, user, ids, context=None):
        """
        Limpa o campo de CST no formulario.
         
        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: valor do campo CST em branco.
        :rtype: dict
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or dict()

        return dict(value=dict(wizlit_cst_id=False))

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
