# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.addons.fiscal.model.nf_capa_data import STATUS_NOTA_FISCAL
from SigUtils.formatastring import formataString
from openerp.report.interface import report_int
from SigUtils.impressao import Impressora
from openerp.report.render import render
from openerp.tools.translate import _
from openerp.osv import osv, fields
from openerp import pooler
from datetime import date, datetime
import StringIO
import locale
import time

TIPO_RESUMO = {'E': 'Entradas', 'S': 'Saidas'}
TIPO_MOVIMENTO = {'C': _('Compra/Venda'), 'D': _('Devolucao'), 'T': _('Transferencia'),
                  'E': _('Remessa'), 'R': _('Retorno'), 'U': _('Outras')}


# noinspection PyPep8Naming
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):
    def __init__(self, name):
        super(report_custom, self).__init__(name)
        self._new_line = '{: >179}'.format('')
        self._trace_line = '{:->181}'.format('')
        self._detail1 = ''  # 1th detail line
        self._detail2 = ''  # 2nd detail line
        self._dateFmt = locale.nl_langinfo(locale.D_FMT)
        self._moneyFmt = lambda f: locale.format('%.2f', f or 0, grouping=True)

    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato()
        listagem.informaNome(pdf_string, _('Listagem de NF'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.3, 0.3, 0.3, 0.3)
        listagem.defineFonte(tam=5.32)

        # Define o formato de data e numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))  # Normalmente pt_BR
        self._dateFmt = locale.nl_langinfo(locale.D_FMT)
        self._moneyFmt = lambda f: locale.format('%.2f', f or 0, grouping=True)

        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'wizard.listagem.nf', context=context)
        res_company_dados = pooler.get_pool(cr.dbname).get('res.company').read(
            cr, uid, company_id, ['name'], context=context)

        # Impressao das linhas do cabecalho
        self._report_header(listagem, datas.get('data_cab_inicial'), datas.get('data_cab_final'),
                            res_company_dados.get('name'), datas.get('imprimir_notas'),
                            datas.get('listar_encargos'), datas.get('cab_filtro'))

        # Impressao do detalhe do relatorio
        list_nf = []
        total_notas = 0.0
        nf_summary = {}
        cfop_summary = {}
        fispar_base_reduzida = pooler.get_pool(cr.dbname).get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', 'fispar_base_reduzida', context=context).get('fispar_base_reduzida')

        listar_encargos = datas.get('listar_encargos')
        encargo_ids = datas.get('encargo_ids')
        imprimir_resumo = datas.get('imprimir_resumo')
        imprimir_notas = datas.get('imprimir_notas')
        nf_state = dict(STATUS_NOTA_FISCAL)
        dados_nota = self._get_dados_nota(cr, uid, ids, datas, context)
        for nota in dados_nota:
            # Informacoes de notas fiscais
            if nota.get('nf_id') and nota.get('nf_id') not in list_nf:
                list_nf.append(nota.get('nf_id'))
                if datas.get('cfop_ids'):
                    vr_total_nf = nota.get('total_cfop') or 0.0
                else:
                    vr_total_nf = nota.get('vr_total_nf') or 0.0
                total_notas += vr_total_nf
                pessoa_name = str(nota.get('pessoa_codigo') or '') + '-' + \
                              (nota.get('pessoa_name') or '')
                movimento = TIPO_MOVIMENTO.get(nota.get('tipo_movimento')) or _('Outras')
                data_nf = nota.get('tipo_nf') == 'E' and nota.get('data_saida_entrada')
                data_nf = data_nf or nota.get('data_emissao')
                data_nf = time.strftime(self._dateFmt, time.strptime(
                    data_nf[:19], "%Y-%m-%d %H:%M:%S")) or ''

                # Impressao da linha detalhe de notas fiscais
                imprimir_notas and listagem.imprimeLinha(self._detail1.format(
                    nota.get('company_id'),
                    data_nf, nota.get('tipo_nf'), nota.get('modelo_nf') or '', nota.get('serie_nf') or '',
                             nota.get('numero_nf') or '', pessoa_name, self._moneyFmt(vr_total_nf),
                             nota.get('natureza_name') or '', movimento, nf_state.get(nota.get('state_nf'))))

            # Informacoes dos encargos da nota fiscal
            if listar_encargos and nota.get('codigo_cfop') and nota.get('encargo_id') in \
                    encargo_ids and nota.get('encargo_nome'):
                encargo = nota.get('encargo_nome') or ' '
                chave = encargo + '-' + nota.get('codigo_cfop') or ''
                vr_isento = nota.get('fisnit_vr_isento') or 0
                vr_outros = nota.get('fisnit_vr_outros') or 0
                if fispar_base_reduzida == 'I':
                    vr_isento += (nota.get('vr_reducao_base') or 0)
                else:
                    vr_outros += (nota.get('vr_reducao_base') or 0)
                vals = dict(fisnit_vr_tributado=nota.get('fisnit_vr_tributado') or 0,
                            aliquota=nota.get('aliquota') or 0,
                            vr_encargo=nota.get('vr_encargo') or 0,
                            fisnit_vr_isento=vr_isento,
                            fisnit_vr_outros=vr_outros,
                            fisnit_vr_diferido=nota.get('fisnit_vr_diferido') or 0,
                            base_substituicao=nota.get('base_substituicao') or 0,
                            fisnit_vr_substituicao=nota.get('fisnit_vr_substituicao') or 0,
                            fisnit_vr_fcp=nota.get('fisnit_vr_fcp') or 0,
                            fisnit_vr_icms_desonerado=nota.get('fisnit_vr_icms_desonerado') or 0,
                            fisnit_vr_servico=nota.get('fisnit_vr_servico') or 0)

                if not nf_summary.get(encargo):
                    nf_summary[encargo] = vals.copy()
                else:
                    nf_summary[encargo]['fisnit_vr_tributado'] += \
                        nota.get('fisnit_vr_tributado') or 0
                    nf_summary[encargo]['vr_encargo'] += \
                        nota.get('vr_encargo') or 0
                    nf_summary[encargo]['fisnit_vr_isento'] += vr_isento
                    nf_summary[encargo]['fisnit_vr_outros'] += vr_outros
                    nf_summary[encargo]['fisnit_vr_diferido'] += \
                        nota.get('fisnit_vr_diferido') or 0
                    nf_summary[encargo]['base_substituicao'] += \
                        nota.get('base_substituicao') or 0
                    nf_summary[encargo]['fisnit_vr_substituicao'] += \
                        nota.get('fisnit_vr_substituicao') or 0
                    nf_summary[encargo]['fisnit_vr_fcp'] += \
                        nota.get('fisnit_vr_fcp') or 0
                    nf_summary[encargo]['fisnit_vr_icms_desonerado'] += \
                        nota.get('fisnit_vr_icms_desonerado') or 0
                    nf_summary[encargo]['fisnit_vr_servico'] += \
                        nota.get('fisnit_vr_servico') or 0

                if not cfop_summary.get(chave):
                    cfop_summary[chave] = vals.copy()
                else:
                    cfop_summary[chave]['fisnit_vr_tributado'] += \
                        nota.get('fisnit_vr_tributado') or 0
                    cfop_summary[chave]['vr_encargo'] += \
                        nota.get('vr_encargo') or 0
                    cfop_summary[chave]['fisnit_vr_isento'] += vr_isento
                    cfop_summary[chave]['fisnit_vr_outros'] += vr_outros
                    cfop_summary[chave]['fisnit_vr_diferido'] += \
                        nota.get('fisnit_vr_diferido') or 0
                    cfop_summary[chave]['base_substituicao'] += \
                        nota.get('base_substituicao') or 0
                    cfop_summary[chave]['fisnit_vr_substituicao'] += \
                        nota.get('fisnit_vr_substituicao') or 0
                    cfop_summary[chave]['fisnit_vr_fcp'] += \
                        nota.get('fisnit_vr_fcp') or 0
                    cfop_summary[chave]['fisnit_vr_icms_desonerado'] += \
                        nota.get('fisnit_vr_icms_desonerado') or 0
                    cfop_summary[chave]['fisnit_vr_servico'] += \
                        nota.get('fisnit_vr_servico') or 0

                # Impressao da linha detalhe dos encargos
                listagem.imprimeLinha(self._detail2.format('',
                    '', '',encargo, nota.get('codigo_cfop') or '',
                    self._moneyFmt(nota.get('fisnit_vr_tributado')),
                    self._moneyFmt(nota.get('aliquota')), self._moneyFmt(nota.get('vr_encargo')),
                    self._moneyFmt(vr_isento), self._moneyFmt(vr_outros),
                    self._moneyFmt(nota.get('fisnit_vr_icms_desonerado')),
                    self._moneyFmt(nota.get('fisnit_vr_fcp')),
                    self._moneyFmt(nota.get('fisnit_vr_diferido')),
                    self._moneyFmt(nota.get('base_substituicao')),
                    self._moneyFmt(nota.get('fisnit_vr_substituicao')),
                    self._moneyFmt(nota.get('fisnit_vr_servico'))))
            list_nf.append(nota.get('nf_id'))

        # Total das notas fiscais
        imprimir_notas and listagem.imprimeLinha(self._trace_line)
        imprimir_notas and listagem.imprimeLinha('|{:<80}{:>14} {:/>83} |'.format(
            _('T O T A L   N O T A S'), self._moneyFmt(total_notas), ''))
        imprimir_notas and listagem.imprimeLinha(self._trace_line)

        if listar_encargos:
            for i in [1, 2]:  # 1= imprime total de encargos por CFOP, 2= imprime total de encargos
                if i == 1:
                    listagem.imprimeLinha('| {:^177} |'.format(
                        _(' T O T A L   E N C A R G O S   P O R   C F O P ')))
                else:
                    listagem.imprimeLinha('| {:^177} |'.format(
                        _(' T O T A L   E N C A R G O S ')))
                listagem.imprimeLinha(self._detail2.format('',
                    '', '', _('ENCARGO'), _('CFOP') if i == 1 else '', _('VR BASE'), '', _('IMPOSTO'),
                    _('ISENTOS'), _('OUTROS'), _('DISONERADO'), _('VALOR FCP'), _('DIFERIDO'),
                    _('BASE ST'), _('VALOR ST'), _('VR SERVICO')).upper())
                listagem.imprimeLinha(self._trace_line)
                encargo_antigo = ''
                chaves = sorted(cfop_summary.keys())
                for chave in chaves:  # Evita registros repetidos para o segundo total
                    encargo, codigo_cfop = chave.split('-')
                    if i == 2:
                        if encargo_antigo == encargo:
                            continue
                        encargo_antigo = encargo
                    nota = nf_summary.get(encargo)
                    if i == 1:
                        nota = cfop_summary.get(chave)
                    listagem.imprimeLinha(self._detail2.format('',
                        '', '',encargo, codigo_cfop if i == 1 else '',  # Com ou sem o codigo do CFOP
                        self._moneyFmt(nota.get('fisnit_vr_tributado')),
                        '',  # Aliquota nao eh mostrada
                        self._moneyFmt(nota.get('vr_encargo')),
                        self._moneyFmt(nota.get('fisnit_vr_isento')),
                        self._moneyFmt(nota.get('fisnit_vr_outros')),
                        self._moneyFmt(nota.get('fisnit_vr_icms_desonerado')),
                        self._moneyFmt(nota.get('fisnit_vr_fcp')),
                        self._moneyFmt(nota.get('fisnit_vr_diferido')),
                        self._moneyFmt(nota.get('base_substituicao')),
                        self._moneyFmt(nota.get('fisnit_vr_substituicao')),
                        self._moneyFmt(nota.get('fisnit_vr_servico'))))
                listagem.imprimeLinha(self._trace_line)

        # Resumo das notas fiscais
        self.trata_resumo(cr, uid, imprimir_resumo, dados_nota, listagem, context)

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return (self.obj.pdf, 'pdf')

    def _report_header(self, report, dt_start, dt_end, company, imp_nf=True, imp_enc=False, filtro=False):
        report.informaCab(u'{:^174}'.format(_('LISTAGEM DE NF')))
        report.informaCab(self._new_line)
        report.informaCab(self._new_line)
        report.informaCab(u'{:<12}{:^150}{:>12}'.format(
            time.strftime("%d/%m/%Y"), company, _('Pagina: {{ Page }}')))
        report.informaCab(self._new_line)
        report.informaCab(u'{} {} a {}'.format(_('Periodo de'), time.strftime(
            "%d/%m/%Y", time.strptime(dt_start, "%Y-%m-%d")), time.strftime(
            "%d/%m/%Y", time.strptime(dt_end, "%Y-%m-%d"))))
        if filtro:
            report.informaCab(u'Filtros: %s' % filtro)
        report.informaCab(self._trace_line)

        # Define linha de abecalho/detalhe para informacoes de notas fiscais
        self._detail1 = u'| {:0>3} {:<10}{:>4} {:>7} {:>5} {:>12} {:<43.43} {:>12.12} {:<45.45} {:<15} {:<13}|'
        imp_nf and report.informaCab(self._detail1.format(_('EMP'),
            _('DATA'), _('TIPO'), _('MODELO'), _('SERIE'), _('NUMERO NF'), _('PARTICIPANTE'), _('VALOR TOTAL'),
            _('NATUREZA DA OPERACAO'), _('TIPO MOVIMENTO'), _('STATUS')))

        # Define linha de cabecalho/detalhe para informacoes dos encargos das notas fiscais
        self._detail2 = u'| {:>3} {:<10}{:>4} {:>7} {:>5} {:>12} {:>12} {:>12} {:>12} {:>12} {:>13} {:>12} ' \
                        '{:>12} {:>12} {:>13} {:>13}|'
        imp_enc and report.informaCab(self._detail2.format('',
            '', '', _('ENCARGO'), _('CFOP'), _('VR BASE'), _('ALIQUOTA'), _('IMPOSTO'), _('ISENTOS'),
            _('OUTROS'), _('DESONERADO'), _('VALOR FCP'), _('DIFERIDO'), _('BASE ST'),
            _('VALOR ST'), _('VR SERVICO')))
        imp_nf and report.informaCab(self._trace_line)
        report.rodapePadrao()


    def _get_dados_nota(self, cr, user, ids, datas, context=None):
        context = context and context.copy() or {}
        data_inicial = datas.get('data_inicial')
        data_final = datas.get('data_final')
        context.update({'search_normal': True})
        cfop_ids = datas.get('cfop_ids')
        pessoa_ids = datas.get('pessoa_ids')
        serie = datas.get('serie')
        tipo = datas.get('tipo')
        tipo_movimento = datas.get('tipo_movimento')
        natureza_id = datas.get('natureza_id')
        state = datas.get('state')
        encargo_ids = datas.get('encargo_ids')
        ordem_listar = datas.get('ordem_listar')
        tipo_pgto = datas.get('tipo_pagamento')
        modelo_nf_id = datas.get('modelo_nf_id')
        imprimir_nf_propria = datas.get('imprimir_nf_propria')
        exceto_grupo_90 = datas.get('exceto_grupo_90')
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, user, 'wizard.listagem.nf', context=context)
        company_ids = map(lambda x: '%s' % x,
                          pooler.get_pool(cr.dbname).get('res.company')._get_company_children(cr, user, company_id))
        obj_fiscal_cfop = pooler.get_pool(cr.dbname).get('fiscal.cfop')
        cond_state = ''
        if state:
            cond_state = """AND fnc.state = '%s' """ % state
        total_cfop = ''
        cond_cfop_id = ''
        if cfop_ids:
            if len(cfop_ids) == 1:
                str_cfop_ids = str(tuple(cfop_ids)).replace(',', '')
            else:
                str_cfop_ids = str(tuple(cfop_ids))
            total_cfop += """
                    SUM(CASE WHEN fnit.fisnit_cfop_id IN %s
                             THEN fnit.fisnit_vr_contabil
                             ELSE 0
                        END)                             AS total_cfop,
            """ % str_cfop_ids
            cond_cfop_id = """AND fnit.fisnit_cfop_id IN %s """ % str_cfop_ids
        cond_tipo = ''
        cond_data = ''
        tipo_cfop = list(set(map(lambda c: c.tipo,
                                 obj_fiscal_cfop.browse(cr, user, cfop_ids, context))))
        if len(tipo_cfop) == 1:
            if tipo_cfop[0] == 'E':
                cond_data += """
                    fnc.data_saida_entrada BETWEEN '%s' AND '%s'
                """ % (data_inicial, data_final)
                if not tipo:
                    cond_tipo += """AND fnc.tipo = 'E' """
            else:
                cond_data += """
                    fnc.data_emissao BETWEEN '{data_inicial}' AND '{data_final}'
                """.format(data_inicial=data_inicial, data_final=data_final)
                if not tipo:
                    cond_tipo += """AND fnc.tipo = 'S' """
        else:
            cond_data += """
                CASE
                    WHEN fnc.tipo = 'E'
                        THEN fnc.data_saida_entrada BETWEEN '{data_inicial}'
                             AND '{data_final}'
                    ELSE
                        fnc.data_emissao BETWEEN '{data_inicial}'
                            AND '{data_final}'
                END
            """.format(data_inicial=data_inicial, data_final=data_final)

        cond_company_id = """AND fnc.company_id IN (%s) """ % ','.join(company_ids)
        cond_destinatario_id = ''
        if pessoa_ids:
            cond_destinatario_id += """AND fnc.destinatario_id IN %s """ % str(pessoa_ids).replace(
                '[', '(').replace(']', ')')
        cond_encargo = ''
        if datas.get('listar_encargos') and encargo_ids:
            encargo_ids = ', '.join(map(lambda x: str(x), encargo_ids))
            cond_encargo += """AND fe.compoe_valor_doc = True"""
        cond_natureza_id = ''
        if natureza_id:
            cond_natureza_id += """AND fnc.natureza_id = %s """ % natureza_id
        cond_serie_nf = ''
        if serie:
            # não preencher com Zeros pois existe séries de 1 e 2 caracteres EX: 5, 9 , 10, 05, 003
            cond_serie_nf += """AND fnc.serie_nf = '%s' """ % str(serie)

        if tipo:
            cond_tipo += """AND fnc.tipo = '%s' """ % tipo
        cond_tipo_pgto = ''
        inner_join_condicao_pgtop = ''
        if tipo_pgto:
            cr.execute("""-- noinspection SqlDialectInspectionForFile
            SELECT fcp.id
            FROM fiscal_condicao_pagamento AS fcp
            INNER JOIN adm_condicao_forma  AS acf ON acf.id = fcp.condicao_forma_id
            INNER JOIN adm_condicao        AS ac  ON ac.id = acf.forma_pagamento_id
            WHERE ac.tipo = '%s' """ % tipo_pgto)
            dados_condicao_pgto = cr.dictfetchall()
            if dados_condicao_pgto:
                condicao_pgto_ids = ', '.join(map(lambda x: str(x.get('id')), dados_condicao_pgto))
                cond_tipo_pgto += """AND fcp.id in (%s) """ % condicao_pgto_ids
                inner_join_condicao_pgtop += """LEFT JOIN fiscal_condicao_pagamento AS fcp  ON
                fcp.capa_id = fnc.id """
        cond_movimento_tipo = ''
        if tipo_movimento:
            cond_movimento_tipo += """AND cfop.movimento_tipo = '%s' """ % tipo_movimento
        cond_modelo_nf_id = ''
        if modelo_nf_id:
            cond_modelo_nf_id = """
                AND fnc.modelo_nf_id = %s
            """ % modelo_nf_id

        cond_nf_propria = ''
        if imprimir_nf_propria:
            cond_nf_propria = """
                AND fnc.nf_propria IS TRUE
            """

        cond_mais_filtro = datas.get('mais_filtro')

        cond_exceto_grupo_90 = ''
        if exceto_grupo_90:
            cond_exceto_grupo_90 = """
                AND fnc.modelo_nf_id NOT IN (
                    SELECT id FROM fiscal_modelo_nf WHERE fismnf_codigo BETWEEN '90' AND '99'
                )
            """

        if ordem_listar == 1:  # Por data
            order_by = 'fnc.data_saida_entrada, fnc.data_emissao, '

        elif ordem_listar == 2:  # Por nf(serie + numero)
            order_by = 'fnc.modelo_nf_id, fnc.serie_nf, fnc.numero_nf, '
        elif ordem_listar == 3:  # por
            order_by = 'ap.nome, '
        else:
            order_by = 'fnc.id, '

        sql = """
            -- noinspection SqlDialectInspectionForFile
            SELECT fnc.id                           AS nf_id,
                   fnc.company_id,
                   CASE
                        WHEN fe.sigla = 'ICMS' AND fnc.finalidade_emissao = '2' THEN
                            sum(
                                (SELECT SUM(coalesce(vr_total_item_rateado, 0.0)) 
                                 FROM estoque_movimento_itens 
                                 WHERE nf_id = fnc.id) +
                                coalesce(fnit.vr_icms_substituicao, 0) +
                                COALESCE(fnit.fisnit_vr_fcp_st, 0)
                            )
                        WHEN fe.sigla = 'IPI' AND fe.compoe_valor_doc 
                             AND fnc.finalidade_emissao = '2' THEN
                            sum(coalesce(fnit.vr_encargo, 0))
                        ELSE
                            fnc.vr_total_nf
                   END AS vr_total_nf,
                   fnc.tipo                         AS tipo_nf,
                   fnc.data_emissao,
                   fnc.data_saida_entrada,
                   CASE
                       WHEN fnc.tipo = 'E' AND NOT fnc.nf_propria
                           THEN (SELECT fismnf_codigo FROM fiscal_modelo_nf 
                                 WHERE id = fnc.modelo_nf_id)
                       ELSE
                           (SELECT fismnf_codigo FROM fiscal_modelo_nf 
                            WHERE id IN (SELECT modelo_nf_id 
                                         FROM fiscal_modelo_nf_series 
                                         WHERE id = fnc.modelo_nf_series_id 
                                         LIMIT 1)
                                         )
                   END                              AS modelo_nf,
                   fnc.serie_nf,
                   fnc.numero_nf,
                   fnc.fisnfc_valor_icms_desonerado AS vr_desonerado,
                   fnc.fisnfc_valor_fcp_st          AS vr_fcp,
                   fnc.state                        AS state_nf,
                   fnit.fisnit_cfop_id,
                   cfop.codigo                      AS codigo_cfop,
                   cfop.produto_servico_tipo        AS servico_tipo,
                   cfop.movimento_tipo              AS tipo_movimento,
                   {total_cfop}
                   ap.nome                          AS pessoa_name,
                   ap.codigo                        AS pessoa_codigo,
                   fcn.name                         AS natureza_name,
                   fe.sigla                         AS encargo_nome,
                   fnit.encargo_id                  AS encargo_id,
                   fnit.aliquota                    AS aliquota,
                   SUM(CASE WHEN cst.tipo = 'T' THEN fnit.fisnit_vr_tributado ELSE 0 END )
                                                    AS fisnit_vr_tributado,
                   Sum(CASE
                       WHEN cst.tipo = 'I'
                           THEN COALESCE (fnit.vr_base_calculo, 0.0)
                       ELSE 0
                   END)                             AS fisnit_vr_isento,
                   Sum(CASE
                           WHEN cst.tipo IN ('O', 'U', 'TU')
                               THEN COALESCE (fnit.vr_base_calculo, 0.0)
                           ELSE 0
                       END)                         AS fisnit_vr_outros,
                   SUM(fnit.fisnit_vr_diferido)     AS fisnit_vr_diferido,
                   Sum(CASE
                           WHEN cst.tipo IN ('U', 'TU')
                           THEN COALESCE (fnit.vr_base_icms_substituicao, 0.0) 
                           ELSE 0
                       END)                         AS base_substituicao,
                   Sum(CASE
                           WHEN cst.tipo IN ('U', 'TU')
                           THEN COALESCE (fnit.vr_icms_substituicao, 0.0)
                           ELSE 0
                       END)                         AS fisnit_vr_substituicao,
                   Sum(CASE
                           WHEN cst.tipo IN ('U', 'TU')
                           THEN COALESCE (fnit.fisnit_vr_fcp_st, 0)
                           ELSE 0
                       END)                         AS fisnit_vr_fcp,
                   Sum(CASE
                           WHEN cst.tipo IN ('U', 'TU')
                           THEN COALESCE (fnit.fisnit_vr_icms_desonerado, 0)
                           ELSE 0
                       END)                         AS fisnit_vr_icms_desonerado,
                   SUM(fnit.fisnit_vr_servico)      AS fisnit_vr_servico,
                   SUM(CASE
                           WHEN cst.tipo = 'T'
                               THEN fnit.vr_encargo
                           WHEN cst.tipo != 'T' AND fe.sigla = 'IPI'
                               THEN fnit.vr_encargo
                           ELSE 0
                       END)                         AS vr_encargo,
                   SUM(COALESCE(fnit.vr_reducao_base, 0.0))
                                                    AS vr_reducao_base,
                   SUM(fnit.vr_icms_substituicao)   AS vr_icms_substituicao
            FROM fiscal_nf_capa                     AS fnc
            LEFT JOIN fiscal_nf_item_tributacao     AS fnit ON fnit.fisnit_capa_id = fnc.id
            LEFT JOIN fiscal_cst                    AS cst  ON cst.id = fnit.cst_id
            LEFT JOIN fiscal_cfop_natureza          AS fcn  ON fcn.id = fnc.natureza_id
            LEFT JOIN fiscal_cfop                   AS cfop ON cfop.id = fnit.fisnit_cfop_id
            LEFT JOIN fiscal_encargo                AS fe   ON fe.id = fnit.encargo_id
            LEFT JOIN adm_pessoa                    AS ap   ON ap.id = fnc.destinatario_id
            {inner_join_condicao_pgtop}
            WHERE {cond_data}
                  and fnc.numero_nf > 0 
                  {cond_state}
                  {cond_destinatario_id}
                  {cond_cfop_id}
                  {cond_natureza_id}
                  {cond_serie_nf}
                  {cond_tipo}
                  {cond_tipo_pgto}
                  {cond_movimento_tipo}
                  {cond_company_id}
                  {cond_modelo_nf_id}
                  {cond_nf_propria}
                  {cond_exceto_grupo_90}
                  {cond_mais_filtro}
            GROUP BY fnc.id, fnit.fisnit_cfop_id, cfop.codigo, ap.nome, ap.codigo,
                     fcn.name, cfop.movimento_tipo, cfop.produto_servico_tipo, fnc.tipo,
                     fnc.data_emissao, fnc.data_saida_entrada, fnc.serie_nf, fnc.numero_nf,
                     fe.sigla, fnit.encargo_id, fnit.aliquota, fe.compoe_valor_doc, fnc.company_id
            ORDER BY fnc.company_id, {order_by} vr_total_nf desc, fe.sigla
        """.format(total_cfop=total_cfop, cond_encargo=cond_encargo, cond_state=cond_state,
                   cond_company_id=cond_company_id, cond_destinatario_id=cond_destinatario_id,
                   cond_cfop_id=cond_cfop_id, cond_natureza_id=cond_natureza_id,
                   cond_serie_nf=cond_serie_nf, cond_tipo=cond_tipo, cond_tipo_pgto=cond_tipo_pgto,
                   cond_data=cond_data, cond_movimento_tipo=cond_movimento_tipo, order_by=order_by,
                   inner_join_condicao_pgtop=inner_join_condicao_pgtop,
                   cond_modelo_nf_id=cond_modelo_nf_id, cond_nf_propria=cond_nf_propria,
                   cond_mais_filtro=cond_mais_filtro,
                   cond_exceto_grupo_90=cond_exceto_grupo_90)
        cr.execute(sql)
        dados_nota = cr.dictfetchall()
        if not state or state in ('C', 'I'):
            parametros = dict(cond_data=cond_data, cond_state=cond_state,
                              cond_destinatario_id=cond_destinatario_id, cond_cfop_id=cond_cfop_id,
                              cond_natureza_id=cond_natureza_id, cond_serie_nf=cond_serie_nf,
                              cond_tipo=cond_tipo, cond_company_id=cond_company_id,
                              order_by=order_by, cond_modelo_nf_id=cond_modelo_nf_id,
                              cond_exceto_grupo_90=cond_exceto_grupo_90)
            dados_nota.extend(self.get_canceladas_inutilizadas(cr, parametros))

        return dados_nota

    @staticmethod
    def get_canceladas_inutilizadas(cr, parametros):
        sql = """
        -- noinspection SqlDialectInspectionForFile
        SELECT fnc.id                           AS nf_id,
               fnc.vr_total_nf,
               fnc.tipo                         AS tipo_nf,
               fnc.data_emissao,
               fnc.data_saida_entrada,
               CASE
                   WHEN fnc.tipo = 'E' AND NOT fnc.nf_propria
                       THEN (SELECT fismnf_codigo FROM fiscal_modelo_nf 
                             WHERE id = fnc.modelo_nf_id)
                   ELSE
                       (SELECT fismnf_codigo FROM fiscal_modelo_nf 
                        WHERE id IN (SELECT modelo_nf_id 
                                     FROM fiscal_modelo_nf_series 
                                     WHERE id = fnc.modelo_nf_series_id 
                                     LIMIT 1)
                                     )
               END                              AS modelo_nf,
               fnc.serie_nf,
               fnc.numero_nf,
               fnc.state                        AS state_nf,
               ap.nome                          AS pessoa_name,
               ap.codigo                        AS pessoa_codigo,
               fcn.name                         AS natureza_name,
               0                                AS codigo_cfop
        FROM fiscal_nf_capa                  AS fnc
        LEFT JOIN fiscal_nf_item_tributacao  AS fnit ON fnit.fisnit_capa_id = fnc.id
        INNER JOIN fiscal_cfop_natureza      AS fcn  ON fcn.id = fnc.natureza_id
        INNER JOIN adm_pessoa                AS ap   ON ap.id = fnc.destinatario_id
        WHERE {cond_data}
          AND fnc.state IN ('C', 'I')
          and fnc.numero_nf > 0 
          {cond_state}
          {cond_destinatario_id}
          {cond_cfop_id}
          {cond_natureza_id}
          {cond_serie_nf}
          {cond_tipo}
          {cond_company_id}
          {cond_modelo_nf_id}
          {cond_exceto_grupo_90}
        GROUP BY fnc.id,
                 fnc.vr_total_nf,
                 ap.nome,
                 ap.codigo,
                 fcn.name,
                 fnc.tipo,
                 fnc.data_emissao,
                 fnc.data_saida_entrada,
                 fnc.serie_nf,
                 fnc.numero_nf
        ORDER BY {order_by}
        """.format(cond_data=parametros.get('cond_data'), cond_state=parametros.get('cond_state'),
                   cond_destinatario_id=parametros.get('cond_destinatario_id'),
                   cond_natureza_id=parametros.get('cond_natureza_id'),
                   cond_serie_nf=parametros.get('cond_serie_nf'),
                   cond_cfop_id=parametros.get('cond_cfop_id'),
                   cond_tipo=parametros.get('cond_tipo'),
                   cond_company_id=parametros.get('cond_company_id'),
                   order_by=parametros.get('order_by')[:-2],
                   cond_modelo_nf_id=parametros.get('cond_modelo_nf_id'),
                   cond_exceto_grupo_90=parametros.get('cond_exceto_grupo_90'))
        cr.execute(sql)
        dados_nota = cr.dictfetchall()
        return dados_nota

    def trata_resumo(self, cr, uid, imprimir_resumo, dados_nota, listagem, context=None):
        context = context and context.copy() or {}
        if not imprimir_resumo:
            return False
        linha = ''
        nf_ids = list(set(map(lambda n: n.get('nf_id'), dados_nota)))
        str_nf_ids = str(nf_ids).replace('[', '(').replace(']', ')')
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'wizard.listagem.nf', context=context)
        company_ids = map(lambda x: '%s' % x,
                          pooler.get_pool(cr.dbname).get('res.company')._get_company_children(cr, uid, company_id))

        cond_company_id = """AND nf.company_id IN (%s) """ % ','.join(company_ids)
        sql = """
        SELECT COUNT(numero_nf) AS qtd_nf, 
               company_id,
               modelo,
               serie,
               MIN(inicial)     AS inicial,
               MAX(final)       AS final,
               SUM(total)       AS total,
               tipo,
               movimento_tipo   
    FROM (  SELECT nf.numero_nf,
                   nf.company_id,
                   modelo.fismnf_codigo AS modelo,
                   nf.serie_nf          AS serie,
                   nf.numero_nf   AS inicial,
                   nf.numero_nf   AS final,
                   nf.vr_total_nf  AS total,
                   cfop.tipo            AS tipo,
                   cfop.movimento_tipo  AS movimento_tipo
            FROM fiscal_nf_capa AS nf
            INNER JOIN fiscal_modelo_nf    AS modelo   ON modelo.id = nf.modelo_nf_id
            LEFT JOIN fiscal_cfop_natureza AS natureza ON natureza.id = nf.natureza_id
            LEFT JOIN fiscal_cfop          AS cfop     ON natureza.cfop_id = cfop.id
            WHERE nf.id IN %s
              %s             
            GROUP BY nf.numero_nf ,modelo.fismnf_codigo, nf.numero_nf, nf.serie_nf, cfop.tipo, 
                     cfop.movimento_tipo, nf.company_id, nf.vr_total_nf
                     ) as TMP  
                               
    GROUP BY modelo, serie, tipo, movimento_tipo, company_id    
    ORDER BY company_id, modelo, serie, tipo, movimento_tipo
        """ % (nf_ids and str_nf_ids or '(-1)', cond_company_id)
        cr.execute(sql)
        resumo_dados = cr.dictfetchall()

        listagem.imprimeLinha(self._new_line)
        listagem.imprimeLinha(_('RESUMO:'))
        listagem.imprimeLinha(self._new_line)
        modelo_serie_anterior = {}
        detail = '{}: {:0>3} {}: {:<2} {}: {:<3} {}: {:<12} {}: {:<12.12} {}:{:>12.12} {}: {:>4}'.decode('utf-8')
        modelo_antigo = False
        dict_qtd = {}
        for i in resumo_dados:
            chave = i.get('modelo') + '-' + i.get('serie')
            if not chave in dict_qtd:
                dict_qtd[chave] = i.get('qtd_nf')
            else:
                dict_qtd[chave] += i.get('qtd_nf')

        _chaveAtual = _chaveUltima = _resumoAnterior = ''
        _detalheVazio = 57 * ' '
        _detalheEntrada = _detalheSaida = _detalheVazio
        ient = 1
        isai = 2
        novo_resumo_dados = resumo_dados
        for resumo in resumo_dados:
            _chaveAtual = '%s-%s-%s' % (resumo.get('company_id'), resumo.get('modelo'), resumo.get('serie'))
            if _chaveAtual != _chaveUltima:
                ient = 1
                isai = 2
            _chaveUltima = _chaveAtual
            if resumo.get('tipo') == 'E':
                ient += 2
                resumo['ordem'] = _chaveAtual + '-{:>03}'.format(ient)
            else:
                isai += 2
                resumo['ordem'] = _chaveAtual + '-{:>03}'.format(isai)
        _chaveAtual = _chaveUltima = _resumoAnterior = ''
        resumo_dados = sorted(novo_resumo_dados, key=lambda o: o.get('ordem'))
        for resumo in resumo_dados:
            _chaveAtual = '%s-%s-%s' % (resumo.get('company_id'), resumo.get('modelo'), resumo.get('serie'))
            if _chaveAtual != _chaveUltima and _chaveUltima != '':
                if _detalheSaida != _detalheVazio or _detalheEntrada != _detalheVazio:
                    listagem.imprimeLinha(_detalheEntrada + _detalheSaida)

                    # Tratamento do total por grupo
                    chave_ultima_tipo = _chaveUltima.split('-')
                    modelo_serie_ent_resumo_dados = filter(
                        lambda x: x.get('modelo') == chave_ultima_tipo[1]
                        and x.get('serie') == chave_ultima_tipo[2]
                        and x.get('tipo') == 'E', resumo_dados)
                    modelo_serie_sai_resumo_dados = filter(
                        lambda x: x.get('modelo') == chave_ultima_tipo[1]
                                  and x.get('serie') == chave_ultima_tipo[2]
                                  and x.get('tipo') == 'S', resumo_dados)
                    if modelo_serie_ent_resumo_dados and not modelo_serie_sai_resumo_dados:
                        linha_total_grupo = '{:<24}: {:>15}'.format(
                            'Total',
                            self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                                   modelo_serie_ent_resumo_dados))),
                        )
                        listagem.imprimeLinha(linha_total_grupo)
                    elif modelo_serie_sai_resumo_dados and not modelo_serie_ent_resumo_dados:
                        linha_total_grupo = '{:<24}: {:>15}'.format(
                            'Total',
                            self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                                   modelo_serie_sai_resumo_dados))),
                        )
                        listagem.imprimeLinha(_detalheVazio + linha_total_grupo)
                    elif modelo_serie_ent_resumo_dados and modelo_serie_sai_resumo_dados:
                        linha_total_grupo = '{:<24}: {:>15}                {:<24}: {:>15}'.format(
                            'Total',
                            self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                                   modelo_serie_ent_resumo_dados))),
                            'Total',
                            self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                                   modelo_serie_sai_resumo_dados))),
                        )
                        listagem.imprimeLinha(linha_total_grupo)

                    _detalheEntrada = _detalheSaida = _detalheVazio
                self.trata_situacao_nota_fiscal(cr, str_nf_ids, _resumoAnterior, listagem)
            _chaveUltima = _chaveAtual
            _resumoAnterior = resumo
            modelo_serie = resumo.get('modelo') + '-' + resumo.get('serie')
            if modelo_antigo != modelo_serie:
                linha and listagem.imprimeLinha(self._trace_line)
                modelo_serie_resumo_dados = filter(
                    lambda x: x.get('modelo') == resumo.get('modelo')
                              and x.get('serie') == resumo.get('serie'), resumo_dados)
                inicial = min(map(lambda n: n.get('inicial'), modelo_serie_resumo_dados))
                final = max(map(lambda n: n.get('final'), modelo_serie_resumo_dados))
                modelo_serie_total = sum(map(lambda t: t.get('total'), modelo_serie_resumo_dados))
                listagem.imprimeLinha(detail.format(_('Empresa'), resumo.get('company_id'),
                    _('Modelo'), resumo.get('modelo'), _('Serie'), resumo.get('serie'),
                    _('Inicial'), str(inicial), _('Final'), str(final), _('Total'),
                    self._moneyFmt(modelo_serie_total or 0), _('  QTd.NFs'), dict_qtd.get(modelo_serie)))
                modelo_antigo = modelo_serie
                modelo_serie_anterior.update({'modelo': resumo.get('modelo'),
                                              'serie': resumo.get('serie')})

            linha = '{:<10} '.format(TIPO_RESUMO.get(resumo.get('tipo')))
            if TIPO_MOVIMENTO.get(resumo.get('movimento_tipo')):
                nome = TIPO_MOVIMENTO.get(resumo.get('movimento_tipo'))
                linha += '{:<13}: {:>15}                '.format(nome, self._moneyFmt(resumo.get('total')))
            if resumo.get('tipo') == 'E':
                if _detalheEntrada == _detalheVazio:
                    _detalheEntrada = linha
                else:
                    listagem.imprimeLinha(_detalheEntrada + _detalheSaida)
                    _detalheEntrada = linha
                    _detalheSaida = _detalheVazio
            else:
                if _detalheSaida == _detalheVazio:
                    _detalheSaida = linha
                else:
                    listagem.imprimeLinha(_detalheEntrada + _detalheSaida)
                    _detalheEntrada = _detalheVazio
                    _detalheSaida = linha
        if _detalheSaida != _detalheVazio or _detalheEntrada != _detalheVazio:
            listagem.imprimeLinha(_detalheEntrada + _detalheSaida)
            _detalheEntrada = _detalheSaida = _detalheVazio

            # Tratamento do total por grupo
            modelo_serie_ent_resumo_dados = filter(
                lambda x: x.get('modelo') == resumo.get('modelo')
                          and x.get('serie') == resumo.get('serie')
                          and x.get('tipo') == 'E', resumo_dados)
            modelo_serie_sai_resumo_dados = filter(
                lambda x: x.get('modelo') == resumo.get('modelo')
                          and x.get('serie') == resumo.get('serie')
                          and x.get('tipo') == 'S', resumo_dados)
            if modelo_serie_ent_resumo_dados and not modelo_serie_sai_resumo_dados:
                linha_total_grupo = '{:<24}: {:>15}'.format(
                    'Total',
                    self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                           modelo_serie_ent_resumo_dados))),
                )
                listagem.imprimeLinha(linha_total_grupo)
            elif modelo_serie_sai_resumo_dados and not modelo_serie_ent_resumo_dados:
                linha_total_grupo = '{:<24}: {:>15}'.format(
                    'Total',
                    self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                           modelo_serie_sai_resumo_dados))),
                )
                listagem.imprimeLinha(_detalheVazio + linha_total_grupo)
            elif modelo_serie_ent_resumo_dados and modelo_serie_sai_resumo_dados:
                linha_total_grupo = '{:<24}: {:>15}                {:<24}: {:>15}'.format(
                    'Total',
                    self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                           modelo_serie_ent_resumo_dados))),
                    'Total',
                    self._moneyFmt(sum(map(lambda mstr: mstr.get('total'),
                                           modelo_serie_sai_resumo_dados))),
                )
                listagem.imprimeLinha(linha_total_grupo)

        self.trata_situacao_nota_fiscal(cr, str_nf_ids, _resumoAnterior, listagem)

        # Tratamento do total geral
        listagem.imprimeLinha('')
        ent_resumo_dados = filter(
            lambda x: x.get('tipo') == 'E', resumo_dados)
        sai_resumo_dados = filter(
            lambda x: x.get('tipo') == 'S', resumo_dados)
        linha_total_grupo = '{:<24}: {:>15}                {:<24}: {:>15}'.format(
            'Total Geral Entradas',
            self._moneyFmt(sum(map(lambda mstr: mstr.get('total'), ent_resumo_dados))),
            'Total Geral Saidas',
            self._moneyFmt(sum(map(lambda mstr: mstr.get('total'), sai_resumo_dados))),
        )
        listagem.imprimeLinha(linha_total_grupo)
        return True

    @staticmethod
    def trata_situacao_nota_fiscal(cr, str_nf_ids, resumo_dados, listagem):
        # NFs faltantes
        sql = """
            SELECT numero_nf
            FROM fiscal_nf_capa
            WHERE 
                id IN %s
                AND nf_propria IS TRUE 
                AND modelo_nf_id IN (
                    SELECT id 
                    FROM fiscal_modelo_nf 
                    WHERE fismnf_codigo = '%s') 
                AND serie_nf = '%s'
            ORDER BY numero_nf
        """ % (str_nf_ids, resumo_dados.get('modelo'), resumo_dados.get('serie'))
        cr.execute(sql)
        saida_nfs = map(lambda n: n.get('numero_nf'), cr.dictfetchall())
        nf_inicial = saida_nfs and saida_nfs[0] or 0
        nf_final = saida_nfs and saida_nfs[-1] or 0
        faltantes = []
        for nf in range(nf_inicial, nf_final and (nf_final + 1) or 0):
            if nf in saida_nfs:
                continue
            faltantes.append(nf)
        if faltantes:
            linha = formataString('Faltas:', 11, ' ', 'D', 'S', 'S') + ' '
            while faltantes:
                linha += formataString(
                    ', '.join(map(lambda nff: str(nff), faltantes[:20])),
                    139, ' ', 'D', 'S', 'S')
                listagem.imprimeLinha(linha)
                linha = ' ' * 12
                faltantes = faltantes[20:]

        # NFs canceladas
        sql = """
            SELECT numero_nf::varchar
            FROM fiscal_nf_capa
            WHERE 
                id IN %s
                AND state = 'C'
                AND modelo_nf_id IN (
                    SELECT id 
                    FROM fiscal_modelo_nf 
                    WHERE fismnf_codigo = '%s') 
                AND serie_nf = '%s'
            ORDER BY numero_nf
        """ % (str_nf_ids, resumo_dados.get('modelo'), resumo_dados.get('serie'))
        cr.execute(sql)
        cancelada_nfs = list(set(map(lambda n: n.get('numero_nf'), cr.dictfetchall())))
        if cancelada_nfs:
            linha = formataString('Canceladas:', 11, ' ', 'D', 'S', 'S') + ' '
            while cancelada_nfs:
                linha += formataString(', '.join(cancelada_nfs[:20]), 139, ' ', 'D',
                                       'S', 'S')
                listagem.imprimeLinha(linha)
                linha = ' ' * 12
                cancelada_nfs = cancelada_nfs[20:]

        # NFs denegadas
        sql = """
            SELECT numero_nf::varchar
            FROM fiscal_nf_capa
            WHERE 
                id IN %s
                AND state = 'D'
                AND modelo_nf_id IN (
                    SELECT id 
                    FROM fiscal_modelo_nf 
                    WHERE fismnf_codigo = '%s') 
                AND serie_nf = '%s'
            ORDER BY numero_nf
        """ % (str_nf_ids, resumo_dados.get('modelo'), resumo_dados.get('serie'))
        cr.execute(sql)
        denegada_nfs = map(lambda n: n.get('numero_nf'), cr.dictfetchall())
        if denegada_nfs:
            linha = formataString('Denegadas:', 11, ' ', 'D', 'S', 'S') + ' '
            while denegada_nfs:
                linha += formataString(', '.join(denegada_nfs[:20]), 139, ' ', 'D', 'S', 'S')
                listagem.imprimeLinha(linha)
                linha = ' ' * 12
                denegada_nfs = denegada_nfs[20:]

        return True


report_custom('report.listagem.nf')


class wizard_listagem_nf(osv.TransientModel):
    _name = 'wizard.listagem.nf'
    _description = 'Listagem de Notas Fiscais'
    _columns = {
        'data_inicial': fields.date('Data Inicial', required=True),
        'data_final': fields.date('Data Final', required=True),
        'cfop_ids': fields.many2many('fiscal.cfop', 'listagem_nf_m2m_fiscal_cfop', 'listagem_nf_id',
                                     'cfop_id', 'CFOP', oldname='cfop'),
        'pessoa_ids': fields.many2many('adm.pessoa', 'listagem_nf_m2m_adm_pessoa', 'listagem_nf_id',
                                       'pessoa_id', 'Cliente/Fornecedor'),
        'serie': fields.char('Serie', size=3),
        'tipo': fields.selection([('E', 'Entradas'), ('S', 'Saidas')], 'Tipo'),
        'tipo_movimento': fields.selection([('C', 'Compra/Venda'),
                                            ('D', 'Devolucao'),
                                            ('T', 'Transferencia'),
                                            ('E', 'Remessa'),
                                            ('R', 'Retorno'),
                                            ('U', 'Outras')], 'Tipo Movimento', size=1),
        'natureza_id': fields.many2one('fiscal.cfop.natureza', 'Natureza',
                                       oldname='fisemi_natureza_id'),
        'listar_encargos': fields.boolean('Listar Encargos'),
        'state': fields.selection(STATUS_NOTA_FISCAL, 'Status', size=2),
        'encargo_ids': fields.many2many('fiscal.encargo', 'wizard_listagem_nf_encargo',
                                        'listagem_id', 'encargo_id', 'Encargos'),
        'ordem_listar': fields.selection([(1, 'Data'),
                                          (2, 'Nota fiscal'),
                                          (3, 'Emitente'),
                                          (4, 'Por gravacao')], 'Ordem a listar',  size=1),
        'tipo_pagamento': fields.selection([('V', 'A vista'),
                                            ('C', 'A prazo'),
                                            ('S', 'Sem condicoes')], 'Tipo de pagamento',  size=1),
        'modelo_nf_id': fields.many2one('fiscal.modelo.nf', 'Modelo NF', ondelete="restrict"),
        'imprimir_resumo': fields.boolean('Imprimir resumo'),
        'imprimir_notas': fields.boolean('Imprimir notas'),
        'imprimir_nf_propria': fields.boolean('Notas proprias'),
        'exceto_grupo_90': fields.boolean('Exceto grupo 90',
                                          help='Desconsidera as notas fiscais cujo modelo seja '
                                               'igual ou maior a 90.')
    }

    _defaults = {
        'data_inicial': date.today().strftime('%Y-%m-%d'),
        'data_final': date.today().strftime('%Y-%m-%d'),
        'imprimir_resumo': True,
        'imprimir_notas': True,
        'imprimir_nf_propria': False,
        'exceto_grupo_90': True,
    }

    def onchange_encargos(self, cr, uid, ids, listar_encargos, context=None):
        """
        Preenche o campo 'encargo_ids' com todos os encargos cadastrados na empresa
        """
        if not listar_encargos:
            return {}
        obj_fiscal_encargo = self.pool.get('fiscal.encargo.empresa')
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'wizard.listagem.nf', context=context)
        encargo_empresa_ids = obj_fiscal_encargo.search(
            cr, uid, [('company_id', '=', company_id)], order='encargo_id', context=context)

        encargo_empresa_dados = obj_fiscal_encargo.read(
            cr, uid, encargo_empresa_ids, ['encargo_id'], context=context)
        encargo_ids = map(lambda x: x.get('encargo_id')[0], encargo_empresa_dados)
        res = dict(value={'encargo_ids': [[6, False, encargo_ids]]})

        return res

    def wiz_acrescentar_filtro(self, cr, user, ids, context=None):
        filtro = cab_filtro = ''
        return cab_filtro, filtro


    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        x = self.browse(cr, uid, ids[0], context)
        if x.data_inicial > x.data_final:
            raise osv.except_osv(_('Atencao!'),
                                 _('A Data inicial nao poder ser maior que a Data final.'))
        encargo_ids = map(lambda y: y.id, x.encargo_ids)
        _inic = datetime.strptime("%s 00:00:00" % x.data_inicial, "%Y-%m-%d %H:%M:%S")
        _inic = self.pool.get('adm.utils').admutl_converte_tz(cr, uid, _inic, 'America/Sao_Paulo', 'UTC', context=None)
        _inic = _inic.strftime("%Y-%m-%d %H:%M:%S")
        # _inic = self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid, _inic, formato=False)
        # _final = datetime.strptime("%s 23:59:59" % x.data_final, "%Y-%m-%d %H:%M:%S")
        # _final = self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid, _final, formato=False)
        _final = datetime.strptime("%s 23:59:59" % x.data_final, "%Y-%m-%d %H:%M:%S")
        _final = self.pool.get('adm.utils').admutl_converte_tz(cr, uid, _final, 'America/Sao_Paulo', 'UTC', context=None)
        _final = _final.strftime("%Y-%m-%d %H:%M:%S")

        _cab_filtro, _filtro = self.wiz_acrescentar_filtro(cr, uid, ids, context=context)
        datas = dict(data_inicial=_inic,
                     data_final=_final,
                     data_cab_inicial=x.data_inicial,
                     data_cab_final=x.data_final,
                     cfop_ids=map(lambda c: c.id, x.cfop_ids),
                     pessoa_ids=map(lambda p: p.id, x.pessoa_ids),
                     serie=x.serie,
                     tipo=x.tipo,
                     tipo_movimento=x.tipo_movimento,
                     natureza_id=x.natureza_id.id,
                     listar_encargos=x.listar_encargos,
                     state=x.state,
                     encargo_ids=encargo_ids,
                     ordem_listar=x.ordem_listar,
                     tipo_pagamento=x.tipo_pagamento,
                     modelo_nf_id=x.modelo_nf_id.id,
                     imprimir_resumo=x.imprimir_resumo,
                     imprimir_notas=x.imprimir_notas,
                     imprimir_nf_propria=x.imprimir_nf_propria,
                     mais_filtro=_filtro,
                     cab_filtro=_cab_filtro,
                     exceto_grupo_90=x.exceto_grupo_90)

        return dict(type='ir.actions.report.xml', report_name='listagem.nf', datas=datas)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
