# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
import time
from datetime import datetime


class wizard_posicao_estoque(osv.TransientModel):
    # prefixo: wizpes

    _name = 'wizard.posicao.estoque'

    _columns = {
        'wizpes_local_id': fields.many2one('estoque.local', 'Local'),
        'wizpes_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizpes_primeira_categoria': fields.boolean('So primeira categoria'),
        'wizpes_item_ids': fields.one2many('wizard.posicao.estoque.itens', 'wizpei_capa_id',
                                           'Posicao de Estoque'),
        'wizpei_vr_lucratividade_total': fields.float('Vr. Lucratividade Total', digits=(14,2)),
        'wizpei_preco_sem_saldo_total': fields.float('Preco sem saldo Total', digits=(14,2)),

    }

    def get_dados(self, cr, uid, local_ids, categoria_id, context):
        context = context and context.copy() or {}
        cond_categoria = ''
        if categoria_id:
            categoria_ids = [categoria_id]
            categoria_ids.extend(self.pool.get('estoque.categoria').search(cr, uid,
                                    [('parent_id', 'child_of', categoria_id)], context=context))
            cond_categoria = 'AND ec.id IN (%s)' % ','.join(map(lambda x: '%s' % x, categoria_ids))

        sql = """
            SELECT ec.id           AS categoria_id,
                   ec.parent_id    AS parent_id,
                   ec.name         AS categoria_nome,
                   ec.conta        AS nivel,
                   ep.id           AS produto_id,
                   epg.id          AS grade_id
                   
            FROM estoque_categoria           AS ec 
            INNER JOIN estoque_produto       AS ep  ON ep.categoria_id = ec.id
            INNER JOIN estoque_produto_grade AS epg ON epg.produto_id = ep.id
            INNER JOIN estoque_local_locacao AS ell ON ell.grade_id = epg.id
            INNER JOIN estoque_local         AS el  ON el.id = ell.local_id
            WHERE ell.local_id IN ({locais})
              {cond_categoria}
            ORDER BY ec.conta, el.name
              """.format(locais=','.join(map(lambda x: "%s" % x, local_ids)),
                         cond_categoria=cond_categoria)
        try:
            cr.execute(sql)
        except Exception as e:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro na consulta: %s') % str(e))

        return cr.dictfetchall()

    def get_vr_lucratividade(self, cr, uid, imposto, preco_venda, custos, parametros, context=None):
        """ A coluna lucratividade é o a coluna preco de venda - a coluna do custo(o que está 
           definido no parametro p/preco de venda) - impostos(considerar tributação para cada 
           encargo para consumidor final) - comissao (% do parametro * coluna preco de venda / 100) 
           - custo-fixo (percnetual do parametro * preco de venda / 100), 
           tudo isto dividido pela coluna preco de venda e depois dividido por 100
           Mas esta funcao volta o valor do lucro para depois calcular a porcentagem"""

        context = context and context.copy() or {}
        par_custo = parametros.get('estpar_custo_para_precovenda')
        if par_custo == 'U':
            custo = custos.get('estprc_ultimo_custo') or 0
        elif par_custo == 'M':
            custo = custos.get('estprc_custo_medio_contabil') or 0
        elif par_custo == 'R':
            custo = custos.get('estprc_custo_reposicao') or 0
        elif par_custo == 'C':
            custo = custos.get('estprc_custo_corrigido') or 0
        else:
            custo = 0

        comissao = (parametros.get('comissoes') or 0) * preco_venda / 100.0
        custo_fixo = (parametros.get('custo_fixo') or 0) * preco_venda / 100.0
        return (preco_venda - custo - imposto - comissao - custo_fixo)

    def wizpes_button_search(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        filtros = self.read(cr, uid, ids[0], [], context=context, load='_classic_write')
        local_ids = filtros.get('wizpes_local_id') and [filtros.get('wizpes_local_id')] or []
        categoria_id = filtros.get('wizpes_categoria_id')
        primeira_categoria = filtros.get('wizpes_primeira_categoria')
        if not local_ids:
            local_ids = self.pool.get('estoque.local').search(cr, uid,
                                                 [('excluir_consultas','=',False)], context=context)

        dados = self.get_dados(cr, uid, local_ids, categoria_id, context)
        if not dados:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha registro para os filtros selecionados!'))

        produto_ids = []
        grade_ids = []
        for i in dados:
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))

        precos = self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid, produto_ids,
                                                           epc_grade_ids=grade_ids, context=context)
        saldos = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(cr, uid, grade_ids,
                                     datetime.now().date(),
                                     eps_local_id=filtros.get('wizpes_local_id'), context=context)
        custos = self.pool.get('estoque.preco.custo').get_custos_do_dia(cr, uid, produto_ids,
                                                                        context=context)
        aux = {}
        nivel = ''
        impostos = self.pool.get('fiscal.tributacao.encargos').fisten_getvalor_impostos(cr, uid,
                                                                         grade_ids, precos, context)
        parametros = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque.parametro',
                       ['estpar_custo_para_precovenda', 'comissoes', 'custo_fixo'], context=context)
        for i in dados:
            produto_id = i.get('produto_id')

            grade_id = i.get('grade_id')
            saldo  = saldos.get(grade_id).get('disponivel')
            if saldo <= 0:
                continue

            preco_venda_produto = precos.get(produto_id)
            preco_venda_sem_saldo = 0
            if preco_venda_produto:
                preco_venda_sem_saldo = preco_venda_produto.get(grade_id) or \
                               preco_venda_produto.get('preco_venda')

            custo_produto = custos.get(produto_id) or {}
            custo_corrigido = (custo_produto.get('estprc_custo_corrigido') or 0) * saldo
            ultimo_custo = (custo_produto.get('estprc_ultimo_custo') or 0) * saldo
            custo_reposicao = (custo_produto.get('estprc_custo_reposicao') or 0) * saldo
            custo_medio = (custo_produto.get('estprc_custo_medio_contabil') or 0) * saldo
            if primeira_categoria:
                if not i.get('parent_id'):
                    nivel = i.get('nivel')
            else:
                nivel = i.get('nivel')

            imposto = impostos.get(grade_id) and impostos.get(grade_id).get('total') or 0
            vr_lucratividade = self.get_vr_lucratividade(cr, uid, imposto, preco_venda_sem_saldo,
                                                   custo_produto, parametros, context)
            preco_venda = preco_venda_sem_saldo * saldo
            if nivel not in aux:
                aux[nivel] = {'wizpei_categoria_id': i.get('categoria_id'),
                              'wizpei_preco_venda': preco_venda,
                              'wizpei_custo_corrigido':custo_corrigido,
                              'wizpei_ultimo_custo':ultimo_custo,
                              'wizpei_custo_reposicao': custo_reposicao,
                              'wizpei_custo_medio': custo_medio,
                              'vr_lucratividade': vr_lucratividade,
                              'total_preco_sem_saldo': preco_venda_sem_saldo}
            else:
                aux[nivel]['wizpei_preco_venda'] += preco_venda
                aux[nivel]['wizpei_custo_corrigido'] += custo_corrigido
                aux[nivel]['wizpei_ultimo_custo'] += ultimo_custo
                aux[nivel]['wizpei_custo_reposicao'] += custo_reposicao
                aux[nivel]['wizpei_custo_medio'] += custo_medio
                aux[nivel]['vr_lucratividade'] += vr_lucratividade
                aux[nivel]['total_preco_sem_saldo'] += preco_venda_sem_saldo

        vals_capa = {'wizpes_item_ids': []}
        vr_lucratividade_total = 0
        preco_sem_saldo_total = 0
        teve_item = False
        for nivel, vals in aux.items():
            vr_lucratividade_total += vals.get('vr_lucratividade')
            preco_sem_saldo_total += vals.get('total_preco_sem_saldo')
            try:
                lucratividade = vals.pop('vr_lucratividade') / vals.pop('total_preco_sem_saldo') * 100.0
            except:
                lucratividade = 100.0
            vals['wizpei_conta'] = nivel
            vals['wizpei_lucratividade'] =  lucratividade
            vals_capa['wizpes_item_ids'].append((0, 0, vals))
            teve_item = True

        vals_capa.update({'wizpei_vr_lucratividade_total': vr_lucratividade_total,
                          'wizpei_preco_sem_saldo_total': preco_sem_saldo_total})
        if not teve_item:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha registro para os filtros selecionados!'))

        self.write(cr, uid, ids, vals_capa, context=context)

        return True

    def wizpes_button_new(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_itens = self.pool.get('wizard.posicao.estoque.itens')
        itens_ids = obj_itens.search(cr, uid, [], context=context)
        obj_itens.unlink(cr, uid, itens_ids, context=context)
        return True

    def wizpes_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        estrutura, dados = self.get_dados_rel(cr, uid, ids, context)
        context['estrutura'] = estrutura
        context['dados'] = dados
        return self.pool.get('wizard.relatorios.impressao').get_tela_relatorios(cr, uid, context)

    def get_dados_rel(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        pagina = {'relpag_nome_rel': _('Posicao de Estoque'),
                  'relpag_tam_linha': 170,
                  'relpag_fonte_tamanho': 8.4,
                  'relpag_margem_esquerda': 0.5,
                  'relpag_margem_direita': 0.5,
                  'relpag_margem_superior': 0.5,
                  'relpag_margem_inferior': 0.5,
                  'relpag_cabecalhos': [{'relcab_name': 'cab1',
                                         'relcab_colunas': [{'relcol_name': 'data',
                                                             'relcol_dados_tam': 16},
                                                            {'relcol_name': 'titulo',
                                                             'relcol_dados_tam': 135,
                                                             'relcol_dados_alinhamento': 'C'},
                                                            {'relcol_name': 'pagina',
                                                             'relcol_dados_tam': 9}]},
                                        {'relcab_name': 'cab2',
                                         'relcab_colunas': [{'relcol_name': 'empresa',
                                                             'relcol_dados_tam': 168,}],},
                                        {'relcab_name': 'cab_col',
                                         'relcab_divide_linha': True,
                                         'relcab_cor': 'Blue',
                                         'relcab_separa_coluna': '|',
                                         'relcab_colunas': [{'relcol_name': 'col1',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 60},
                                                            {'relcol_name': 'col2',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 17},
                                                            {'relcol_name': 'col3',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 17},
                                                            {'relcol_name': 'col4',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 17},
                                                            {'relcol_name': 'col5',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 17},
                                                            {'relcol_name': 'col6',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 17},
                                                            {'relcol_name': 'col7',
                                                             'relcol_dados_alinhamento': 'C',
                                                             'relcol_dados_tam': 17},]
                                         },],
                  'relpag_linhas': [{'rellin_name': 'linha1',
                                     'rellin_separa_coluna': '|',
                                     'rellin_colunas': [{'relcol_name': 'categoria',
                                                         'relcol_dados_alinhamento': 'D',
                                                         'relcol_dados_tam': 60},
                                                        {'relcol_name': 'preco_venda',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 17},
                                                        {'relcol_name': 'ultimo_custo',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 17},
                                                        {'relcol_name': 'custo_medio',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 17},
                                                        {'relcol_name': 'custo_corrigido',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 17},
                                                        {'relcol_name': 'custo_reposicao',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 17},
                                                        {'relcol_name': 'lucratividade',
                                                         'relcol_dados_alinhamento': 'E',
                                                         'relcol_dados_tam': 17},]
                                     },],
                  }
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        nome_empresa = self.pool.get("res.company").read(cr, uid, company_id, ['name'],
                                                         context).get('name')
        dados = {'cab1': {'data': _('Data: ') + time.strftime('%d/%m/%Y'),
                          'titulo': _('Posicao de Estoque'),
                          'pagina': '|pag|'},
                 'cab2': {'empresa': nome_empresa},
                 'cab_col': {'col1': _('CATEGORIA'),
                             'col2': _('PRECO VENDA'),
                             'col3': _('ULTIMO CUSTO'),
                             'col5': _('CUSTO MEDIO'),
                             'col4': _('CUSTO CORRIGIDO'),
                             'col6': _('CUSTO REPOSICAO'),
                             'col7': _('% LUCRATIVIDADE')},
                 }
        linhas = []
        for i in self.read(cr, uid, ids, [], context=context):
            if i.get('wizpes_local_id'):
                pagina['relpag_cabecalhos'] = pagina.get('relpag_cabecalhos')[:-1] + \
                                              [{'relcab_name': 'cab3',
                                                'relcab_colunas': [{'relcol_name': 'local',
                                                                    'relcol_dados_tam': 170,}],},] + \
                                              pagina.get('relpag_cabecalhos')[-1:]
                dados['cab3'] = {'local': i.get('wizpes_local_id')[1]}
            filtro_categ = ''
            if i.get('wizpes_categoria_id'):
                filtro_categ += i.get('wizpes_categoria_id')[1] + '        '

            pagina['relpag_cabecalhos'] = pagina.get('relpag_cabecalhos')[:-1] + \
                                          [{'relcab_name': 'cab4',
                                            'relcab_divide_linha': True,
                                            'relcab_colunas': [{'relcol_name': 'so_primeira',
                                                                'relcol_dados_tam': 170,}],},] + \
                                          pagina.get('relpag_cabecalhos')[-1:]
            dados['cab4'] = {'so_primeira': filtro_categ + 'SO PRIMEIRA CATEGORIA: %s' %
                                            (i.get('wizpes_primeira_categoria') and 'SIM' or 'NAO')}
            for item in self.pool.get('wizard.posicao.estoque.itens').read(cr, uid,
                                                             i.get('wizpes_item_ids'), [], context):
                categoria = '%s-%s' % (item.get('wizpei_conta'), item.get('wizpei_categoria_id')[1])
                linhas.append({'linha1': {'categoria': categoria,
                                          'preco_venda': item.get('wizpei_preco_venda'),
                                          'ultimo_custo': item.get('wizpei_ultimo_custo'),
                                          'custo_medio': item.get('wizpei_custo_medio'),
                                          'custo_corrigido': item.get('wizpei_custo_corrigido'),
                                          'custo_reposicao': item.get('wizpei_custo_reposicao'),
                                          'lucratividade': item.get('wizpei_lucratividade')}})
        dados['linhas'] = linhas
        return pagina, dados

class wizard_posicao_estoque_itens(osv.TransientModel):

    _name = 'wizard.posicao.estoque.itens'

    _columns = {
        'wizpei_conta': fields.char('Nivel', size=30),
        'wizpei_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizpei_preco_venda': fields.float('Preco venda', digits=(14,2)),
        'wizpei_ultimo_custo': fields.float('Ultimo Custo', digits=(14,2)),
        'wizpei_custo_medio': fields.float('Custo Medio', digits=(14,2)),
        'wizpei_custo_corrigido': fields.float('Custo Corrigido', digits=(14,2)),
        'wizpei_custo_reposicao': fields.float('Custo de Reposicao', digits=(14,2)),
        'wizpei_lucratividade': fields.float('Lucratividade(%)', digits=(14,2)),
        'wizpei_capa_id': fields.many2one('wizard.posicao.estoque', 'Capa'),
    }

    _order = 'wizpei_conta'
