# -*- coding: utf-8 -*-

__author__ = 'marcilene.ribeiro'

from pynfe.nfe.manifestacao_destinatario.nfedistribuicaoDFe import ResNFe
from openerp.tools.translate import _
from openerp.osv import fields, osv
from datetime import date, datetime
from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400


class consulta_manifestacao_destinatario(osv.TransientModel):
    _name = 'consulta.manifestacao.destinatario'
    _description = 'Manifesto de destinatario'

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(consulta_manifestacao_destinatario, self).default_get(cr, uid, fields, context)
        obj_fiscal_parametro = self.pool.get('fiscal.parametro')
        fiscal_parametro_ids = obj_fiscal_parametro.search(cr, uid, [], context=context)
        if fiscal_parametro_ids:
            res['indicador_emissor'] = obj_fiscal_parametro.browse(
                cr, uid, fiscal_parametro_ids[0]).fispar_indicador_emissor
            res['indicador_nfe'] = obj_fiscal_parametro.browse(
                cr, uid, fiscal_parametro_ids[0]).fispar_indicador_nfe
        return res

    _columns = {
        'name': fields.char('Justificativa', size=255, help="Justificativa"),
        'indicador_nfe': fields.selection([
            ('0', 'Todas as NF-e'),
            ('1', 'Somente as NF-e que ainda nao tiveram manifestacao do destinatario '
                  '(Desconhecimento da operacao, Operacao nao Realizada ou Confirmacao da '
                  'Operacao)'),
            ('2', 'Idem anterior, incluindo as NF-e que tambem nao tiveram a Ciencia da Operacao.')
        ], 'Indicador de NF-e consultada', size=1, required=True),
        'indicador_emissor': fields.selection([
            ('0', 'Todos os Emitentes / Remetentes'),
            ('1', 'Somente as NF-e emitidas por emissores/remetentes que nao tenham a mesma raiz '
                  'do CNPJ do destinatario (para excluir as notas fiscais de transferencia entre '
                  'filiais).')
        ], 'Indicador do Emissor da NF-e', size=1, required=True),
        'descricao_evento': fields.char('Descricao Evento', size=60),
        'informacao': fields.text('Log'),
        'empresa_atual': fields.boolean('Consultar somente para esta empresa',
                                        help='Realiza a baixa das notas fiscais somente para a '
                                             'empresa cujo CNPJ seja igual ao do destinatario da '
                                             'nota.')
    }
    _defaults = {'empresa_atual': True}

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.financeiro = None
            self.fiscal = None
            self.estoque = None
            self.adm = None
            self.rh = None
            self.__dict__.update(entries)

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or dict()
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company, context=context))) for mod in ['fiscal']])
        return self.ParamToObject(**parametros)

    def consultar_notas(self, cr, user, ids, context=None, company_ids=None):
        """
        Consulta as notas fiscais de manifestacao do destinatario.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: IDs do formulario de consulta
        :param context: um dicionario padrao para valores contextuais
        :param company_ids: identificadores das empresas nas quais o manifesto sera executado
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :type company_ids: list
        :return: verdadeiro se a consulta e recuperacao das notas fiscais para manifestacao foram
        realizadas com sucesso.
        :rtype: bool
        """
        import re
        context = context and context.copy() or {}
        context['sig_force_company'] = True  # Considera cada empresa separadamente
        empresa_ids = isinstance(company_ids, list) and company_ids or []
        empresa_ids = empresa_ids or self.pool.get('res.company').search(
            cr, user, [], context=context)

        # O manisfesto realizado automaticamente (cron) pode definir em quais empresas sera
        # executado. Isso eh feito para que notas fiscais de uma empresa nao seja consultada pelo
        # manifesto em outra.
        if not context.get('verificacao_automatica_manifesto'):
            brw_consumo = self.browse(cr, user, ids and ids[0], context)
            empresa_ids = brw_consumo and not brw_consumo.empresa_atual and empresa_ids or filter(
                lambda f: f == self.pool.get('res.users').browse(
                    cr, user, user, context).company_id.id, empresa_ids)

        for brw_empresa in self.pool.get('res.company').browse(cr, user, empresa_ids, context):
            parametro = self._parametros(cr, user, brw_empresa.id, {'sig_force_company': True})
            if not hasattr(parametro.fiscal, 'data_validade'):
                continue
            if datetime.now().strftime('%Y-%m-%d %H:%M:%S') > parametro.fiscal.data_validade:
                if context.get('verificacao_automatica_manifesto'):
                    continue
                raise osv.except_osv(_('Atencao!'),
                                     _('Certificado digital vencido!'))

            _empresa_id = brw_empresa.id
            _cpf_cnpj = re.sub('[^0-9]', '', brw_empresa.rescon_pessoa_id.cpf_cnpj or str())
            _destinatario_razao_social=brw_empresa.rescon_pessoa_id.name
            _destinatario_ie = brw_empresa.rescon_pessoa_id.admpes_jur_ie

            # Determina a UF em funcao do endereco da empresa (preferencialmente comercial)
            endereco = filter(lambda f: f.endereco_tipo_id.name == 'Comercial',
                              brw_empresa.rescon_pessoa_id.endereco_ids)
            endereco = endereco or filter(lambda f: f.endereco_tipo_id.name == 'Normal',
                                          brw_empresa.rescon_pessoa_id.endereco_ids)
            uf = endereco and endereco[0].uf_id.sigla or None

            # O ´cron´ recupera todas as notas de todos os remetentes
            if context.get('verificacao_automatica_manifesto'):  # Cron
                if not uf:
                    continue  # Nao recupera notas fiscais de empresas sem UF
            else:  # Selecao do usuario pelo formulario
                if not uf:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Cadastre um endereco valido para a empresa.'))

            # Recupera as notas fiscais
            ultimo_nsu = parametro.fiscal.fispar_ultimo_nsu
            bloco_nsu = 0
            while True:  # Enquanto houver notas fiscais (retorno = 138)
                nfe = nfe_400
                nfe = nfe(certify=parametro.fiscal.certificado_conteudo,
                          key=parametro.fiscal.chave_conteudo,
                          environment=parametro.fiscal.fispar_nfe_ambiente, state=uf,
                          contingency=False)
                processo, __ = nfe.process.consultar_notas_destinatario(_cpf_cnpj, ultimo_nsu)
                if processo.resposta.cStat.valor == u'138':  # Documento localizado
                    for lote in processo.resposta.loteDistDFeInt:
                        ultimo_nsu = lote.NSU.valor
                        if lote.schema.valor.startswith('resNFe'):
                            bloco_nsu += 1
                            self._criar_nfe_consumo(
                                cr, user, lote, _empresa_id, _cpf_cnpj, _destinatario_razao_social,
                                _destinatario_ie, context)
                        elif lote.schema.valor.startswith('resEvento'):
                            continue  # Ainda nao implementado.
                        elif lote.schema.valor.startswith('procNFe'):
                            continue  # Ainda nao implementado.
                        elif lote.schema.valor.startswith('procEvento'):
                            continue  # Ainda nao implementado
                else:
                    break
            if not bloco_nsu:
                if processo.resposta.cStat.valor == u'138':
                    log = _('Nenhum documento localizado')
                else:
                    if not processo.resposta.cStat.valor:
                        processo.resposta.cStat.valor = unicode(processo.resposta.status)
                        processo.resposta.xMotivo.valor = processo.resposta.reason
                    log = 'Erro %s - %s' % (processo.resposta.cStat.valor,
                                            processo.resposta.xMotivo.valor)
            else:
                log = _('%s Documentos localizados') % bloco_nsu
            self.write(cr, user, ids, dict(informacao=log), context)

            # Guarda o ultimo NSU
            if int(ultimo_nsu) > int(parametro.fiscal.fispar_ultimo_nsu):
                self.pool.get('fiscal.parametro').write(
                    cr, user, [parametro.fiscal.id], dict(
                        company_id=brw_empresa.id, fispar_ultimo_nsu=ultimo_nsu), context)

        return True

    def _cmdes_get_local_id(self, cr, uid, company_id, context):
        context = context and context.copy() or {}
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        context['sig_force_company'] = True
        par_ids = obj_estoque_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)
        if par_ids:
            brw_parametro = obj_estoque_parametro.browse(cr, uid, par_ids[0])
            res = brw_parametro.estpar_local_compra_padrao_id.id or False

        return res
    def _criar_nfe_consumo(self, cr, user, lote, empresa, cpf_cnpj, razao_social, ie, context=None):
        context = context and context.copy() or dict()
        nf = ResNFe()  # Nota fiscal
        nf.decodifica_nfe('/tmp/', lote.docZip.valor)
        nfe_ids = self.pool.get('fiscal.nfe.consumo').search(
            cr, user, [('nsu', '=', lote.NSU.valor), ('company_id', '=', empresa)], context=context)
        if not nfe_ids:
            local_id = self._cmdes_get_local_id(cr, user, empresa, context)
            values = dict(
                nsu=lote.NSU.valor, tipo_documento='N', chave_nfe=nf.chNFe.valor,
                data_recebimento=date.today(), inscricao_estadual=nf.IE.valor,
                nome_razao_social=nf.xNome.valor, cpf_cnpj_destinatario=cpf_cnpj,
                nome_razao_social_destinatario=razao_social, inscricao_estadual_destinatario=ie,
                data_hora_autorizacao=nf.dhRecbto.valor, company_id=empresa,
                situacao_nfe=str(nf.cSitNFe.valor), tipo_documento_fiscal=str(nf.tpNF.valor),
                cpf_cnpj=nf.CPF.valor or nf.CNPJ.valor, data_emissao_cancelamento=nf.dhEmi.valor,
                valor_total_nfe=float(nf.vNF.valor), digest_value=nf.digVal.valor,
                fisnfco_local_id=local_id)
            if nf.cSitNFe.valor == 1:  # Registra na manifestacao somente se estiver autorizada
                self.pool.get('fiscal.nfe.consumo').create(cr, user, values, context)
        else:
            if nf.cSitNFe.valor != 1:  # Retira da lista de manifestacao se nao estiver autorizada
                self.pool.get('fiscal.nfe.consumo').unlink(cr, user, nfe_ids, context)

        return lote.NSU.valor

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
