# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from funcoes import wizmai_get_tipo_trib_cst
from openerp.tools.translate import _


class wizard_manutencao_tributacao(osv.TransientModel):
    # prefixo: wizmat_
    _name = 'wizard.manutencao.tributacao'
    _columns = {
        'wizmat_ncm_id': fields.many2one('fiscal.ncm', 'NCM'),
        'wizmat_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizmat_comprador_id': fields.many2one('compras.comprador', 'Comprador'),
        'wizmat_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'wizmat_tributacao_id': fields.many2one('fiscal.tributacao', 'Tributacao'),
        'wizmat_cst_id': fields.many2one('fiscal.cst', 'CST'),
        'wizmat_tributacao_ids': fields.one2many('wizard.manutencao.tributacao.item',
                                                 'wizmai_capa_id', 'Tributacoes'),
        'wizmat_produto_ids': fields.one2many('wizard.manutencao.tributacao.produto',
                                              'wizmap_capa_id', 'Produtos'),
    }

    def wizmat_button_find(self, cr, uid, ids, context=None):
        """
        O metodo encontra todos os produtos (estoque.produto) de acordo com os seguintes dados
        selecionaveis: ncm, categoria, comprador, fabricante e tributacao.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: True
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_manutencao_tributacao = self.browse(cr, uid, ids, context)
        cr.execute("""DELETE FROM wizard_manutencao_tributacao_produto WHERE create_uid = %s""" %
                   uid)
        obj_estoque_categoria = self.pool.get('estoque.categoria')
        for x in brw_manutencao_tributacao:
            ncm_id = x.wizmat_ncm_id.id
            categoria_id = x.wizmat_categoria_id.id
            comprador_id = x.wizmat_comprador_id.id
            fabricante_id = x.wizmat_fabricante_id.id
            tributacao_id = x.wizmat_tributacao_id
            cst_id = x.wizmat_cst_id.id
            where = ''
            if ncm_id:
                where += ' AND ep.ncm_id = %s' % ncm_id
            if categoria_id:
                categoria_ids = obj_estoque_categoria.search(
                    cr, uid, [('parent_id', 'child_of', categoria_id)], context=context)
                where += ' AND ep.categoria_id IN (%s)' % ', '.join(map(lambda c: str(c),
                                                                        categoria_ids))
            if comprador_id:
                where += ' AND ep.comprador_id = %s' % comprador_id
            if fabricante_id:
                where += ' AND ep.fabricante_id = %s' % fabricante_id
            if tributacao_id:
                produto_ids = filter(lambda i: i.produto_id.id,
                                     tributacao_id.produto_tributacao_ids)
                produto_ids = map(lambda i: i.produto_id.id, produto_ids) or [-1]
                if len(produto_ids) == 1:
                    produto_ids = str(tuple(produto_ids)).replace(',', '')
                else:
                    produto_ids = str(tuple(produto_ids))
                where += ' AND ep.id IN %s' % produto_ids
            inner_join = ''
            if cst_id:
                inner_join = """
                INNER JOIN fiscal_tributacao_encargos fte
                    ON ep.id = fte.produto_id AND fte.cst_id = %s
                """ % cst_id
            sql = """
                SELECT
                    ep.id, ep.codigo, ep.digito, ep.categoria_id
                FROM
                    estoque_produto ep
                    %s
                WHERE
                    ep.active = True
            """ % inner_join
            if where:
                sql += where
            cr.execute(sql)
            produto_dados = cr.dictfetchall()
            vals = []
            for produto in produto_dados:
                vals_dict = {
                    'wizmap_capa_id': x.id,
                    'wizmap_codigo': produto.get('codigo'),
                    'wizmap_digito': produto.get('digito'),
                    'wizmap_categoria_id': produto.get('categoria_id'),
                    'wizmap_produto_id': produto.get('id'),
                }
                vals.append((0, 0, vals_dict))
            self.write(cr, uid, x.id, {'wizmat_produto_ids': vals}, context)
        return True

    def wizmat_button_substituir_tributacao(self, cr, uid, ids, context=None):
        """
        O metodo substitui os dados da tributacao (fiscal.tributacao.encargos) para os produtos que
        possuam o tipo de tributacao e encargo compativeis.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario para a chamada de uma nova view, onde contem a informacao de que os
        produtos foram ou nao atualizados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        tributacao_atualizada = False
        brw_manutencao_tributacao = self.browse(cr, uid, ids, context)
        for manutencao in brw_manutencao_tributacao:
            tributacao_ids = manutencao.wizmat_tributacao_ids
            if not tributacao_ids:
                raise osv.except_osv(_('Atencao'), _('Nenhuma tributacao foi informada.'))
            tributacao_keys = {}
            map(lambda i: tributacao_keys.update({
                i.wizmai_tipo_tributacao + str(i.wizmai_encargo_id.id):
                {'cst_id': i.wizmai_cst_id.id, 'tributacao_id': i.wizmai_tributacao_id.id,
                 'modbcst': i.wizmai_modbcst, 'mva': i.wizmai_mva,
                 'fisten_aliquota_venda': i.wizmai_aliquota_venda,
                 'valor_pauta': i.wizmai_valor_pauta}}), tributacao_ids)
            tipo_tributacao_encargo_produto = []
            produto_ids = map(lambda i: i.wizmap_produto_id.id, manutencao.wizmat_produto_ids)
            for trib in tributacao_keys.keys():
                for produto_id in produto_ids:
                    tipo_tributacao_encargo_produto.append(trib + str(produto_id))
            tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(
                cr, uid, [('produto_id', 'in', produto_ids)], context=context)
            brw_fiscal_tributacao_encargos = obj_fiscal_tributacao_encargos.browse(
                cr, uid, tributacao_encargo_ids, context)
            nao_atualizado_produto_ids = []
            if not tributacao_encargo_ids:
                nao_atualizado_produto_ids = produto_ids
            for tributacao_encargo in brw_fiscal_tributacao_encargos:
                str_tributacao_encargo = tributacao_encargo.tipo_tributacao + str(
                    tributacao_encargo.encargo_id.id)
                str_tributacao_encargo_produto = str_tributacao_encargo + str(
                    tributacao_encargo.produto_id.id)
                if str_tributacao_encargo_produto in tipo_tributacao_encargo_produto:
                    tipo_tributacao_encargo_produto.remove(str_tributacao_encargo_produto)
                if tributacao_keys.get(str_tributacao_encargo):
                    vals = tributacao_keys.get(str_tributacao_encargo)
                    if 'produto_id' in vals:
                        del vals['produto_id']
                    if 'encargo_id' in vals:
                        del vals['encargo_id']
                    if 'tipo_tributacao' in vals:
                        del vals['tipo_tributacao']
                    obj_fiscal_tributacao_encargos.write(cr, uid, tributacao_encargo.id, vals,
                                                         context)
                    tributacao_atualizada = True
                else:
                    nao_atualizado_produto_ids.append(tributacao_encargo.produto_id.id)
            for produto_id in list(set(nao_atualizado_produto_ids)):
                for tributacao in tributacao_keys:
                    vals = tributacao_keys.get(tributacao)
                    vals.update({'encargo_id': int(tributacao[1]),
                                 'tipo_tributacao': tributacao[0],
                                 'produto_id': produto_id})
                    trib_enc_ids = obj_fiscal_tributacao_encargos.search(
                        cr, uid, [('encargo_id', '=', vals.get('encargo_id')),
                                  ('tipo_tributacao', '=', vals.get('tipo_tributacao')),
                                  ('produto_id', '=', vals.get('produto_id'))], context=context)
                    if trib_enc_ids:
                        obj_fiscal_tributacao_encargos.unlink(cr, uid, trib_enc_ids, context)
                    obj_fiscal_tributacao_encargos.create(cr, uid, vals, context)
                tributacao_atualizada = True
            if not tipo_tributacao_encargo_produto or nao_atualizado_produto_ids:
                continue
            for dado in tipo_tributacao_encargo_produto:
                tipo_tributacao = dado[0]
                encargo_id = dado[1]
                produto_id = int(dado[2:])
                vals = tributacao_keys.get(tipo_tributacao + encargo_id)
                vals.update({'encargo_id': int(encargo_id),
                             'tipo_tributacao': tipo_tributacao,
                             'produto_id': produto_id})
                obj_fiscal_tributacao_encargos.create(cr, uid, vals, context)
                tributacao_atualizada = True
        obj_model_data = self.pool.get('ir.model.data')
        vals = {
            'name': 'Atencao',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.manutencao.tributacao',
            'target': 'new',
            'type': 'ir.actions.act_window',
        }
        if tributacao_atualizada:
            reference = obj_model_data.get_object_reference(
                cr, uid, 'fiscal', 'view_wizard_manutencao_tributacao_atualizado_form')
            view_id = reference and reference[1] or False
            vals.update({'view_id': [view_id]})
            return vals
        else:
            reference = obj_model_data.get_object_reference(
                cr, uid, 'fiscal', 'view_wizard_manutencao_tributacao_nao_atualizado_form')
            view_id = reference and reference[1] or False
            vals.update({'view_id': [view_id]})
            return vals


class wizard_manutencao_tributacao_item(osv.TransientModel):
    # prefixo: wizmai_
    _name = 'wizard.manutencao.tributacao.item'
    _columns = {
        'wizmai_capa_id': fields.many2one('wizard.manutencao.tributacao', 'Capa'),
        'wizmai_tipo_tributacao': fields.selection(
            [('T', 'Todas / Cons.Final'), ('P', 'Produtor rural'), ('C', 'Contribuinte'),
             ('E', 'Entradas')], 'Tipo de tributacao', size=1, required=True),
        'wizmai_encargo_id': fields.many2one('fiscal.encargo', 'Encargo ', required=True),
        'wizmai_cst_id': fields.many2one(
            'fiscal.cst', 'Cst', required=True,
            domain="[('encargo_id', '=', wizmai_encargo_id), "
                   "('fiscst_tipo_movimento', 'in', wizmai_tip_trib_cst)]"),
        'wizmai_tributacao_id': fields.many2one("fiscal.tributacao", 'Tributacao', required=True,
                                                domain="[('cst_id', '=', wizmai_cst_id), "
                                                       "('encargo_id', '=', wizmai_encargo_id)]"),
        'wizmai_modbcst': fields.selection([('0', 'Preco tabelado'),
                                            ('1', 'Lista negativa'),
                                            ('2', 'Lista positiva'),
                                            ('3', 'Lista neutra'),
                                            ('4', 'Margem valor agregado'),
                                            ('5', 'Pauta'),
                                            ('6', 'Valor da operacao')],
                                           'Modal.determ. BC do ICMS ST', size=1),
        'wizmai_mva': fields.float('MVA', digits=(5, 2)),
        'wizmai_aliquota_venda': fields.float('Aliquota para venda', digits=(5, 2)),
        'wizmai_valor_pauta': fields.float('Valor Pauta', digits=(10, 2)),
        'wizmai_tip_trib_cst': fields.function(wizmai_get_tipo_trib_cst, type='char', size=20,
                                               method=True, store=False,
                                               string='Tipo de tributacao do CST', multi='cst'),
        'wizmai_cst_substituicao': fields.function(wizmai_get_tipo_trib_cst, type='boolean',
                                                   method=True, store=False,
                                                   string='CST substituicao', multi='cst'),
    }
    _sql_constraints = [
        ('tipo_tributacao_encargo_uniq',
         'unique (wizmai_tipo_tributacao, wizmai_encargo_id, wizmai_capa_id)',
         _('O tipo de tributacao e o encargo devem ser unicos'))
    ]

    def wizmai_onchange_tipo_tributacao(self, cr, uid, ids, tipo_tributacao, context=None):
        """
        O metodo altera os dados de atributos de acordo com o tipo_tributacao recebido.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param tipo_tributacao: letra para representar o tipo de tributacao.
        'T' = Todas / Consumidor final, 'P' = Produtor rural, 'C' = Contribuinte, 'E' = Entradas.
        :type: char
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario com os novos valores dos atributos.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        return {
            'value': {
                'wizmai_cst_id': False,
                'wizmai_tributacao_id': False,
                'wizmai_tip_trib_cst': obj_fiscal_tributacao_encargos.fisten_get_tipo_trib_cst(
                    cr, uid, ids, tipo_tributacao, context)
            }
        }

    def wizmai_onchange_cst(self, cr, uid, ids, cst_id, context=None):
        """
        O metodo altera os dados de atributos de acordo com o id do cst recebido. Caso ele nao seja
        do tipo substituicao, deve zerar os valor para modbcst, mva, aliquota de venda e valor da
        pauta.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param cst_id: id da tabela fiscal_cst.
        :type: integer
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario com os novos valores dos atributos.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = {}
        brw_fiscal_cst = self.pool.get('fiscal.cst').browse(cr, uid, cst_id, context)
        cst_substituicao = True if brw_fiscal_cst.tipo == 'U' else False
        vals = {'wizmai_cst_substituicao': cst_substituicao}
        if not cst_substituicao:
            vals.update({'wizmai_modbcst': '', 'wizmai_mva': 0.0, 'wizmai_aliquota_venda': 0.0,
                         'wizmai_valor_pauta': 0.0})
        res['value'] = vals
        return res


class wizard_manutencao_tributacao_produto(osv.TransientModel):
    # prefixo: wizmap_
    _name = 'wizard.manutencao.tributacao.produto'
    _columns = {
        'wizmap_capa_id': fields.many2one('wizard.manutencao.tributacao', 'Capa'),
        'wizmap_codigo': fields.char('Codigo', size=20),
        'wizmap_digito': fields.char('Digito', size=1),
        'wizmap_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizmap_produto_id': fields.many2one('estoque.produto', 'Descricao'),
    }
