# -*- coding: utf-8 -*-
# $Id$

from openerp.tools.translate import _
from openerp.osv import fields, osv
from dateutil.parser import parse

__author__ = 'marcilene.ribeiro'
__maintainer__ = "Pierre Barbosa"
__email_maintainer__ = "pierre.barbosa[at]siginformatica.com.br"
__last_update__ = "2019-05-31"


# Ajusta os valores de atributos ´many2one´
def adjust_m2o(vals):
    """ Recupera apenas o ID dos atributos ´many2one´ """
    for val in isinstance(vals, (list, tuple)) and vals or [vals]:
        for k, v in val.items():
            if k.endswith('_id') and isinstance(v, (list, tuple)):
                val[k] = v[0]
            elif k.endswith('_ids'):
                for l in v:
                    if not isinstance(l, (list, tuple)):
                        v[v.index(l)] = [4, l, False]

    return vals


# noinspection PyPep8Naming,PyAbstractClass
class wizard_nf_complemento(osv.TransientModel):
    """
    Cria notas fiscais complementares
    """

    _name = 'wizard.nf.complemento'  # wiznfc
    _description = 'Gera nota fiscal complementar'
    _rec_name = 'nf_referenciada_id'
    _columns = {
        'nf_referenciada_id': fields.many2one('fiscal.nf.capa', 'Nota fiscal referenciada',
                                              required=True),
        'movimento_item_ids': fields.one2many('wizard.nf.movimento.itens',
                                              'complemento_id', 'Item'),
        'wiznfc_tipo': fields.selection([('E', 'Entrada'), ('S', 'Saida')], 'Tipo da nota fiscal',
                                        help='Tipo da nota fiscal de complemento: entrada ou '
                                             'saida.')
    }
    _defaults = {
        'wiznfc_tipo': lambda self, cr, user, context: context.get('tipo', 'S')  # Saida eh o padrao
    }

    def _wiznfc_get_itens_nota_referenciada(self, cr, uid, nf_id, context=None):
        """
        Retorna os itens da nota fiscal referenciada.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nota fiscal de referencia
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf_id: long
        :type context: dict
        :return: os itens da nota fiscal referenciada.
        :rtype: list
        """
        context = context and context.copy() or {}
        itens_complemento = []
        context.update({'nota_referenciada': True})
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', '=', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]
        for brw_item in self.pool.get('fiscal.nf.capa').browse(
                cr, uid, nf_id, context).estoque_movimento_itens_ids:
            brw_tributacao = filter(lambda f: f.encargo_id.id == icms_id, brw_item.tributacoes_ids)
            brw_tributacao = brw_tributacao[0]
            vals_itens = {'produto_id': brw_item.produto_id.id, 'movimento_item_id': brw_item.id,
                          'produto_embalagem_id': brw_item.produto_embalagem_id.id,
                          'local_id': brw_item.local_id.id, 'cfop_id': brw_item.cfop_id.id,
                          'wiznmi_encargo_id': brw_tributacao.encargo_id.id,
                          'wiznmi_cst_id': brw_tributacao.cst_id.id,
                          'wiznmi_tributacao_id': brw_tributacao.tributacao_id.id,
                          'grade_id': brw_item.grade_id.id,
                          'tipo_substituicao': self._wiznfc_get_tributacao_st(
                              cr, uid, map(lambda m: m.id, brw_item.tributacoes_ids), context)}
            itens_complemento.append([0, False, vals_itens])

        return itens_complemento

    def _wiznfc_get_tributacao_st(self, cr, user, ids, context=None):
        """
        Item com substituicao tributaria.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com ID da tributacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return:
        :rtype:
        """
        # noinspection PyProtectedMember
        return bool(filter(lambda f: f.get('tipo_substituicao'),
                           self.pool.get('fiscal.nf.item.tributacao')._fisnit_get_dados_fiscais(
                               cr, user, ids, None, None, context).values()))

    # noinspection PyMethodMayBeStatic
    def wiznfc_onchange_tipo(self, cr, user, ids, tipo, context=None):
        """
        Remove a nota fiscal selecionada sempre que houver troca do tipo de nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: o ID do formulario de nota fiscal de complemento
        :param tipo: o tipo da nota fiscal ("E"= entrada ou "S"= saida)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tipo: str
        :type context: dict
        :return: nenhuma nota fiscal selecionada.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        return dict(value=dict(nf_referenciada_id=False))

    def onchange_nf_referenciada(self, cr, uid, ids, nf_id, context=None):
        """
        Retorna os itens da nota fiscal referenciada e seus respectivos valores.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da nota fiscal de referencia
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: os itens da nota fiscal referenciada.
        :rtype: list[tuple]
        """
        context = context and context.copy() or {}

        return dict(value=dict(movimento_item_ids=self._wiznfc_get_itens_nota_referenciada(
            cr, uid, nf_id, context)))

    def button_gerar_nf_complementar(self, cr, uid, ids, context=None):
        """
        Cria uma nota fiscal referenciada.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da nota fiscal de referencia
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: formulario com os dados da nota complementar.
        :rtype: dict
        """
        context = context and context.copy() or {}
        complemento_nf_id = self.gerar_nf_complementar(cr, uid, ids, context)
        nf_complementar_dados = self.pool.get('fiscal.nf.capa').read(
            cr, uid, complemento_nf_id, ['nf_propria'], context, '_classic_write')
        # Retorna a ´view´ correta
        view = nf_complementar_dados.get('nf_propria') and 'view_fiscal_nf_capa_form'
        view = view or 'view_fiscal_nf_capa_terceiro_form'
        result = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'fiscal', view)
        view_id = result and result[1] or False

        return dict(name='Nota fiscal referenciada', view_type='form', view_mode='form',
                    view_id=[view_id], res_model='fiscal.nf.capa', res_id=complemento_nf_id,
                    type='ir.actions.act_window', context=context)

    def gerar_nf_complementar(self, cr, uid, ids, context=None):
        """
        Cria a nota fiscal complementar.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da wizard
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: os formulario com os dados da nota fiscal complementar.
        :rtype: object
        """
        context = context and context.copy() or {}
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')
        context.update({'nota_referenciada': True, 'proprio': True})
        dados_wizard = self.read(cr, uid, ids, [], context=context, load='_classic_write')[0]
        nf_id = dados_wizard.get('nf_referenciada_id')
        if dados_wizard.get('wiznfc_tipo') == 'S':
            context['nota_fiscal_saida'] = True
        else:
            context['nota_fiscal_entrada'] = True
        fields2read = ['quantidade_volume', 'numero_conhecimento', 'prazo', 'marca', 'especie',
                       'serie_frete', 'numero_volume', 'fisnfc_infcomp_contribuinte', 'tipo',
                       'fisnfc_infcomp_fisco', 'fisnfc_cod_consumo_id', 'modelo_55', 'state',
                       'gerar_xml', 'nf_propria', 'fisnfc_tipo_servico', 'data_emissao', 'frete',
                       'hora_emissao', 'daeta_emissao_frete', 'data_vencimento_frete', 'prazo_tipo',
                       'tipo_frete', 'tipo_venc_frete', 'fisnfc_energia_tipo_ligacao', 'company_id',
                       'vendedor_id', 'natureza_id', 'fisnfc_energia_grupo_tensao', 'modelo_nf_id',
                       'modelo_nf_series_id', 'tributacao_id', 'modelo_nf_frete', 'destinatario_id',
                       'transportador_id', 'endereco_coleta_id', 'endereco_entrega_id',
                       'vendedor_externo_id', 'endereco_cobranca_id', 'destinatario_endereco_id',
                       'fisnfc_nf_frete_id', 'nfe_processo_emissao', 'nfe_forma_impressao_danfe',
                       'nfe_versao_processo_emissao', 'nfe_forma_emissao', 'serie_nf', 'numero_nf',
                       'fisnfc_consumidor']
        dados_nf_capa = obj_fiscal_nf.read(cr, uid, nf_id, fields2read, context, '_classic_write')
        propria = dados_wizard.get('wiznfc_tipo') == 'E' or dados_nf_capa.get('nf_propria')
        modelo_padrao = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', ['modelo_nf_series_id'], context=context).get('modelo_nf_series_id')
        vals_itens = list()
        tot_base_icms_substituicao = tot_icms_substituicao = 0
        obj_wiz_mov_itens = self.pool.get('wizard.nf.movimento.itens')

        # Recupera as informacoes dos itens da nota referenciada
        for brw_item in obj_wiz_mov_itens.browse(
                cr, uid, dados_wizard.get('movimento_item_ids'), context):
            item_comp = {'produto_id': brw_item.produto_id.id, 'quantidade': brw_item.quantidade,
                         'vr_total_item': brw_item.vr_total_item, 'local_id': brw_item.local_id.id,
                         'vr_total_item_rateado': brw_item.vr_total_item, 'nf_id': nf_id,
                         'cfop_id': brw_item.cfop_id.id, 'grade_id': brw_item.grade_id.id,
                         'produto_embalagem_id': brw_item.produto_embalagem_id.id}
            icms = filter(lambda f: f.encargo_id.sigla.upper() == 'ICMS',
                          brw_item.movimento_item_id.tributacoes_ids)
            icms_id = icms and icms[0].encargo_id.id

            # Gera os itens da nota de complemento somente se houver alguma alteracao nos valores
            if any([brw_item.vr_total_item, brw_item.vr_icms, brw_item.aliquota,
                    brw_item.quantidade, brw_item.vr_base_calculo,
                    brw_item.vr_base_icms_substituicao, brw_item.vr_icms_substituicao]):
                # Ajuste dos valores da tributacao do ICMS
                flds = ['encargo_id', 'cst_id', 'tributacao_id', 'aliquota', 'vr_base_calculo',
                        'vr_encargo', 'modbcst', 'vr_reducao_base', 'fisnit_csosn_id', 'cfop_id',
                        'vr_icms_substituicao', 'vr_base_icms_substituicao',  'tipo_substituicao',
                        'fisnit_tipo_cst']
                tributacoes = \
                    adjust_m2o(self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                        cr, uid, 'tributacoes_ids', map(
                            lambda m: m.id, brw_item.movimento_item_id.tributacoes_ids), flds))
                map(lambda m: m.pop('id'), tributacoes)  # O ID nao eh utilizado. Remove!
                for tributacao in tributacoes:
                    is_icms = tributacao['encargo_id'] == icms_id
                    if is_icms:
                        tributacao['cst_id'] = is_icms and brw_item.wiznmi_cst_id.id
                        tributacao['tributacao_id'] = is_icms and brw_item.wiznmi_tributacao_id.id
                        tributacao['cfop_id'] = is_icms and brw_item.cfop_id.id
                    tributacao['aliquota'] = is_icms and brw_item.aliquota
                    tributacao['vr_base_calculo'] = is_icms and brw_item.vr_base_calculo
                    tributacao['vr_encargo'] = is_icms and brw_item.vr_icms
                    tributacao['vr_icms_substituicao'] = is_icms and brw_item.vr_icms_substituicao
                    tributacao['vr_base_icms_substituicao'] = \
                        is_icms and brw_item.vr_base_icms_substituicao
                tributacoes = [[0, False, tributacao] for tributacao in tributacoes]
                tot_icms_substituicao += brw_item.vr_icms_substituicao or 0
                tot_base_icms_substituicao += brw_item.vr_base_icms_substituicao or 0

                # Inclusao do item na nota de complemento
                vals_itens.append([0, False, dict(item_comp, tributacoes_ids=tributacoes)])

        if not vals_itens:
            raise osv.except_osv(_('Atencao!'),
                                 _('Necessario informar pelo menos uma alteracao para que seja '
                                   'gerada uma nota fiscal complementar. As alteracoes podem ser '
                                   'no valor total do item, valor do ICMS, aliquota do ICMS, '
                                   'quantidade do item, valor base do ICMS da substituicao ou '
                                   'valor do ICMS da substituicao.'))

        # O ID da nota de referencia nao sera utilizado na criacao da nova nota
        del dados_nf_capa['id']

        if not dados_nf_capa.get('data_emissao'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Nota fiscal de referencia sem data de emissao.'))
        emissao = parse(dados_nf_capa.get('data_emissao')).strftime('%d-%m-%Y')
        dados_nf_capa['fisnfc_infcomp_contribuinte'] = '' if not dados_nf_capa.get(
            'fisnfc_infcomp_contribuinte') else unicode(
            dados_nf_capa.get('fisnfc_infcomp_contribuinte'))
        dados_nf_capa['fisnfc_infcomp_contribuinte'] += '\nNota fiscal referenciada: \nNumero: ' \
                                                        '%s. Serie: %s. Data de emissao: %s.\n' % (
            dados_nf_capa.get('numero_nf'), dados_nf_capa.get('serie_nf'), emissao)
        del dados_nf_capa['serie_nf']
        del dados_nf_capa['numero_nf']

        # Cria a nota fiscal complementar
        dados_nf_complementear = dict(
            dados_nf_capa, state='1', estoque_movimento_itens_ids=vals_itens, nf_propria=propria,
            fisnfc_nf_referencia_ids=[[6, False, [nf_id]]], vr_icms_substituicao=tot_icms_substituicao,
            vr_base_icms_substituicao=tot_base_icms_substituicao, modelo_nf_series_id=modelo_padrao)
        if dados_wizard.get('wiznfc_tipo') != 'E':
            modelo_padrao = dados_nf_capa.get('modelo_nf_series_id')
        vals_aux = obj_fiscal_nf.dados_onchange_modelo(
            cr, uid, modelo_padrao, propria, context=context)
        dados_nf_complementear.update(vals_aux)
        if propria:
            # Dados de NF-e
            if dados_nf_complementear.get('modelo_55') or \
                    dados_nf_complementear.get('fisnfc_consumidor'):
                dados_nf_complementear.update(
                    {'nfe_forma_impressao_danfe': '1', 'nfe_forma_emissao': '1',
                     'finalidade_emissao': '2'})
            if dados_nf_complementear.get('fisnfc_consumidor'):
                dados_nf_complementear.update({
                    'fisnfc_consumidor': False, 'modelo_55': True
                })
        complemento_nf_id = obj_fiscal_nf.create(cr, uid, dados_nf_complementear, context=context)

        # Inclui a referencia da nota fiscal de complemento
        obj_fiscal_nf.write(
            cr, uid, nf_id, dict(fisnfc_nf_referencia_ids=[[6, False, [complemento_nf_id]]]), context)

        return complemento_nf_id


# noinspection PyAbstractClass,PyPep8Naming
class wizard_nf_movimento_itens(osv.TransientModel):
    """
    Itens de movimento da nota fiscal complementar.
    """
    _name = 'wizard.nf.movimento.itens'  # prefixo: wiznmi

    _description = 'Itens da nota fiscal de complemento'
    _columns = {
        'produto_id': fields.many2one('estoque.produto', 'Produto', ondelete="restrict",
                                      required=True),
        'produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                                ondelete='restrict',
                                                domain="[('produto_id', '=', produto_id)]"),
        'quantidade': fields.float('Quantidade', digits=(12, 3)),
        'vr_total_item': fields.float('Valor total', digits=(15, 2)),
        'vr_icms': fields.float('Valor ICMS', digits=(15, 2)),
        'vr_base_calculo': fields.float('Valor Base Calculo', digits=(15, 2)),
        'vr_base_icms_substituicao': fields.float('Valor base do ICMS da substituicao',
                                                  digits=(14, 2)),
        'vr_icms_substituicao': fields.float('Valor do ICMS da substituicao', digits=(14, 2)),
        'aliquota': fields.float('Aliquota ICMS', digits=(15, 2)),
        'movimento_item_id': fields.many2one('estoque.movimento.itens', 'Item'),
        'complemento_id': fields.many2one('wizard.nf.complemento', 'Complemento'),
        'local_id': fields.many2one('estoque.local', 'Local', ondelete="restrict"),
        'cfop_id': fields.many2one('fiscal.cfop', 'CFOP'),
        'wiznmi_encargo_id': fields.many2one('fiscal.encargo', 'Encargo', readonly=1),
        'wiznmi_cst_id': fields.many2one('fiscal.cst', 'CST'),
        'wiznmi_tributacao_id': fields.many2one('fiscal.tributacao', 'Tributacao'),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade',
                                    domain="[('produto_id', '=', produto_id)]"),
        'tipo_substituicao': fields.boolean('Item com ST')
    }
    _defaults = {
        'tipo_substituicao': False,
        'wiznmi_encargo_id': lambda s, c, u, x: s.pool.get('fiscal.encargo').search(
            c, u, [('sigla', '=', 'ICMS')], context=x)[0],
    }

    def wiznmi_onchange_cst(self, cr, user, ids, cst_id, context=None):
        context = context and context.copy() or {}
        tributacao_ids = self.pool.get('fiscal.tributacao').search(
            cr, user, [('cst_id', '=', cst_id)], context=context)
        tributacao_id = len(tributacao_ids) == 1 and tributacao_ids[0]

        return {'value': {
            'wiznmi_tributacao_id': tributacao_id,
            'tipo_substituicao': self.pool.get('fiscal.cst').browse(
                cr, user, cst_id, context=context).tipo in ('U', 'TU')
        }}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
