#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from openerp.report.render import render
from openerp import pooler
from SigUtils.impressao import Impressora
from SigUtils.formatastring import formataString
from SigUtils.numero import getNumeroEditado
import StringIO
from openerp.report.interface import report_int
import time

READ_LOAD = '_classic_write'


class estoque_produto_report_capa(osv.TransientModel):
    domain_tributacao = "[('encargo_id', '=', wizpptc_encargo_id)]"
    args_tipo_trib = [('T', 'Todas / Cons.Final'),
                      ('P', 'Produtor rural'),
                      ('C', 'Contribuinte'),
                      ('E', 'Entradas')]
    _name = 'wizard.produto.por.tributacao.capa'
    _columns = {
        'wizpptc_company_id': fields.many2one('res.company', 'Empresa'),
        'wizpptc_encargo_id': fields.many2one('fiscal.encargo', 'Encargo'),
        'wizpptc_tributacao_id': fields.many2one("fiscal.tributacao", 'Tributacao',
                                                 domain=domain_tributacao),
        'wizpptc_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizpptc_local_id': fields.many2one("estoque.local", 'Local'),
        'wizpptc_fornecedor_id': fields.many2one("financeiro.fornecedor", 'Fornecedor'),
        'wizpptc_ncm_id': fields.many2one("fiscal.ncm", 'Ncm'),
        'wizpptc_tipo_trib': fields.selection(args_tipo_trib, 'Tipo de tributacao'),
        'wizpptc_nat_rec': fields.char('Natureza', size=3),
        'wizpptc_comprador_id': fields.many2one('compras.comprador', 'Comprador'),
        'wizpptc_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'wizpptc_listagem_ids': fields.one2many(
            'wizard.produto.por.tributacao.item', 'wizppti_capa_id', 'Tributacaoes'
        ),
        'wizpptc_consulta_realizada': fields.boolean('Consulta realizada'),
        'state': fields.selection(string='Status', selection=[('N', 'Aguardando consulta'),
                                                              ('R', 'Consulta realizada')],
                                  required=True, size=1),
        'wizpptc_condicao': fields.selection([('1', 'Todos'),
                                              ('2', 'Ativos'),
                                              ('3', 'Inativos')], 'Condicao do produto'),
    }
    _defaults = {
        'wizpptc_company_id': lambda s, cr, uid, context: s.pool.get(
            'adm.empresa.objeto').getEmpresa(cr, uid),
        'wizpptc_consulta_realizada': False,
        'state': 'N'
    }

    def wizpptc_button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        w = self.read(cr, uid, ids[0], [], context=context, load=READ_LOAD)
        filtro = dict(
            wiz_id=w['id'], company_id=w.get('wizpptc_company_id'),
            tributacao_id=w.get('wizpptc_tributacao_id'), tipo_trib=w.get('wizpptc_tipo_trib'),
            categoria_id=w.get('wizpptc_categoria_id'), local_id=w.get('wizpptc_local_id'),
            fornecedor_id=w.get('wizpptc_fornecedor_id'), ncm_id=w.get('wizpptc_ncm_id'),
            encargo_id=w.get('wizpptc_encargo_id'), registro_ids=w.get('wizpptc_listagem_ids'),
            nat_rec=w.get('wizpptc_nat_rec'), comprador_id=w.get('wizpptc_comprador_id'),
            fabricante_id=w.get('wizpptc_fabricante_id'),
            condicao=w.get('wizpptc_condicao')
        )
        self.alimenta_wizard_dados(cr, uid, w['id'], filtro, context)
        self.write(cr, uid, ids, {'wizpptc_consulta_realizada': True, 'state': 'R'}, context)

        return True

    def alimenta_wizard_dados(self, cr, uid, capa_id, filtros, context=None):
        context = context and context.copy() or dict()
        if filtros.get('company_id'):
            query_company = 'AND ep.company_id = %d' % filtros.get('company_id')
        else:
            query_company = ''
        if filtros.get('encargo_id'):
            query_encargo = 'AND fte.encargo_id = %d' % filtros.get('encargo_id')
        else:
            query_encargo = ''
        if filtros.get('tributacao_id'):
            query_tributacao = 'AND ft.id = %d' % filtros.get('tributacao_id')
        else:
            query_tributacao = ''
        if filtros.get('ncm_id'):
            query_ncm = 'AND ep.ncm_id = %d' % filtros.get('ncm_id')
        else:
            query_ncm = ''
        if filtros.get('categoria_id'):
            categoria_ids = self.pool.get('estoque.categoria').search(
                cr, uid, [('parent_id', 'child_of', filtros.get('categoria_id'))])
            if len(categoria_ids) == 1:
                categoria_ids = str(tuple(categoria_ids)).replace(',', '')
            else:
                categoria_ids = str(tuple(categoria_ids))
            query_categoria = 'AND ep.categoria_id IN %s' % categoria_ids
        else:
            query_categoria = ''
        if filtros.get('local_id'):
            query_local = """AND ep.id IN (
                                 SELECT     epg.produto_id 
                                 FROM       estoque_produto_grade AS epg
                                 INNER JOIN estoque_local_locacao AS ell
                                         ON ell.grade_id = epg.id
                                 WHERE      ell.local_id = %s)""" % filtros.get('local_id')
        else:
            query_local = ''
        if filtros.get('fornecedor_id'):
            query_fornecedor = """AND ep.id IN (
                                      SELECT produto_id
                                      FROM   estoque_fornecedor 
                                      WHERE  fornecedor_id = %d)""" % filtros.get('fornecedor_id')
        else:
            query_fornecedor = ''
        if filtros.get('tipo_trib'):
            query_tipo_trib = "AND fte.tipo_tributacao = '%s'" % filtros.get('tipo_trib')
        else:
            query_tipo_trib = ''
        if filtros.get('nat_rec'):
            query_nat_rec = \
                "AND estpro_pis_nat_rec = '%s' OR estpro_cofins_nat_rec = '%s'" % (
                    filtros.get('nat_rec'), filtros.get('nat_rec'))
        else:
            query_nat_rec = ''
        if filtros.get('comprador_id'):
            query_comprador = 'AND ep.comprador_id = %s' % filtros.get('comprador_id')
        else:
            query_comprador = ''
        if filtros.get('fabricante_id'):
            query_fabricante = 'AND adm.id = %s' % filtros.get('fabricante_id')
        else:
            query_fabricante = ''

        query_condicao = ''
        if filtros.get('condicao') == '2':
            query_condicao = 'AND ep.active = TRUE'
        elif filtros.get('condicao') == '3':
            query_condicao = 'AND ep.active = FALSE'

        sql = """
        TRUNCATE TABLE wizard_produto_por_tributacao_item;
        INSERT INTO wizard_produto_por_tributacao_item
                    (create_uid,
                     wizppti_capa_id,
                     wizppti_produto_codigo,
                     wizppti_cest,
                     wizppti_produto_name,
                     wizppti_ncm_codigo,
                     wizppti_encargo_name,
                     wizppti_tributacao_name,
                     wizppti_cst_cod_name,
                     wizppti_preco_venda,
                     wizppti_nat_rec,
                     wizppti_tipo_tributacao)
        SELECT * FROM(
        SELECT %(uid)d                                 AS create_uid
              ,%(capa_id)d                             AS capa_id
              ,ep.codigo                               AS codigo 
              ,CASE WHEN ep.estpro_cest_id IS NOT NULL THEN 
                    fc.fiscest_codigo 
               ELSE fc2.fiscest_codigo END             AS cest
              ,ep.name                                 AS descricao
              ,ncm.codigo                              AS ncm
              ,fe.sigla                                AS encargo
              ,ft.name                                 AS tributacao
              ,cst.codigo                              AS cst
              ,epv.preco_venda                         AS preco_venda
              ,CASE WHEN fe.sigla = 'PIS'              THEN ep.estpro_pis_nat_rec 
                    WHEN fe.sigla = 'COFINS'           THEN ep.estpro_cofins_nat_rec 
               ELSE '' END                             AS natureza
              ,CASE WHEN fte.tipo_tributacao = 'E'     THEN 'Entradas' 
                    WHEN fte.tipo_tributacao = 'T'     THEN 'Todas/Cons.Final' 
                    WHEN fte.tipo_tributacao = 'P'     THEN 'Produtor rual' 
              ELSE 'Contribuinte' END                  AS tipo_tributacao
        FROM  estoque_produto                          AS ep 
              LEFT  JOIN fiscal_cest                   AS fc 
                      ON fc.id = ep.estpro_cest_id 
              LEFT  JOIN fiscal_ncm                    AS ncm 
                      ON ncm.id = ep.ncm_id 
              LEFT  JOIN fiscal_cest                   AS fc2 
                      ON fc2.id = ncm.fisnc_cest_id 
              INNER JOIN fiscal_tributacao_encargos    AS fte 
                      ON fte.produto_id = ep.id 
                     %(query_encargo)s
                     %(query_tipo_trib)s
              INNER JOIN fiscal_encargo                AS fe 
                      ON fe.id = fte.encargo_id 
              INNER JOIN fiscal_tributacao             AS ft 
                      ON ft.id = fte.tributacao_id 
                     %(query_tributacao)s
              INNER JOIN fiscal_cst                    AS cst 
                      ON cst.id = fte.cst_id 
              LEFT  JOIN estoque_preco_venda           AS epv 
                      ON epv.produto_id = ep.id
              LEFT  JOIN compras_comprador             AS cc
                      ON cc.id = ep.comprador_id
              LEFT  JOIN adm_pessoa                    AS adm
                      ON adm.id = ep.fabricante_id
        WHERE ep.tipo = 'P'
          %(query_company)s
          %(query_ncm)s
          %(query_categoria)s
          %(query_local)s
          %(query_fornecedor)s
          %(query_encargo)s
          %(query_tributacao)s
          %(query_tipo_trib)s
          %(query_nat_rec)s
          %(query_comprador)s
          %(query_fabricante)s
          %(query_condicao)s
          AND (epv.create_date = (
              SELECT MAX(create_date)
              FROM   estoque_preco_venda
              WHERE  produto_id = ep.id)
           OR epv.create_date IS NULL   
          ) 
        ORDER BY ep.codigo, fe.sigla)                  AS tmp""" % locals()
        cr.execute(sql)

        return True

    def wizpptc_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if ids and isinstance(ids, list):
            ids = ids[0]
        wiz_dados = self.read(cr, uid, ids, [], context=context, load=READ_LOAD)
        try:
            filtro = dict(
                wiz_id=wiz_dados.get('id'), company_id=wiz_dados.get('wizpptc_company_id'),
                tributacao_id=wiz_dados.get('wizpptc_tributacao_id'),
                categoria_id=wiz_dados.get('wizpptc_categoria_id'),
                local_id=wiz_dados.get('wizpptc_local_id'),
                fornecedor_id=wiz_dados.get('wizpptc_fornecedor_id'),
                ncm_id=wiz_dados.get('wizpptc_ncm_id'),
                encargo_id=wiz_dados.get('wizpptc_encargo_id'),
                comprador_id=wiz_dados.get('wizpptc_comprador_id'),
                fabricante_id=wiz_dados.get('wizpptc_fabricante_id'),
                nat_rec =wiz_dados.get('wizpptc_nat_rec'),
                tipo_trib=wiz_dados.get('wizpptc_tipo_trib'),
                condicao=wiz_dados.get('wizpptc_condicao')
            )
            self.alimenta_wizard_dados(cr, uid, wiz_dados.get('id'), filtro, context)
            obj_wiz_item = self.pool.get('wizard.produto.por.tributacao.item')
            datas = obj_wiz_item.wizpptc_button_print(cr, uid, ids)
            self.write(cr, uid, ids, {'state': 'R'}, context)
        except AttributeError:
            datas = {}
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'wizard.produto.por.tributacao.item',
            'datas': {},
            }

    def wizpptc_nova_consulta(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.unlink(cr, uid, ids, context=context)
        return {'name': 'Itens por Tributacao',
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': False,
                'res_model': 'wizard.produto.por.tributacao.capa',
                'target': 'inline',
                'type': 'ir.actions.act_window',
                }


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class estoque_produto_report_view(osv.TransientModel):
    _name = 'wizard.produto.por.tributacao.item'
    _description = "Produtos por Tributacao"
    _rec_name = 'wizppti_produto_name'
    _columns = {
        'wizppti_capa_id': fields.many2one('wizard.produto.por.tributacao.capa', 'Capa'),
        'wizppti_produto_codigo': fields.char('Codigo', size=20),
        'wizppti_produto_name': fields.char('Descricao', size=60),
        'wizppti_tributacao_name': fields.char('Tributacao', size=40),
        'wizppti_cst_cod_name': fields.char('Cst', size=50),
        'wizppti_nat_rec': fields.char('Nat', size=3),
        'wizppti_tipo_tributacao': fields.char('Tipo/Trib', size=17),
        'wizppti_encargo_name': fields.char('Encargo', size=45),
        'wizppti_ncm_codigo': fields.char('Ncm', size=11),
        'wizppti_preco_venda': fields.float('P.Venda', digits=(10, 2)),
        'wizppti_cest': fields.char('CEST', size=7),
    }
    _order = 'wizppti_produto_codigo, wizppti_produto_name'

    def wizpptc_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if ids and isinstance(ids, list):
            ids = ids[0]

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'wizard.produto.por.tributacao.item',
            'datas': {},
        }


class report_produto_por_tributacao_item(report_int):

    def center_title(self, E, C, D, T=155):
        title_c = (T - len(C)) / 2
        title = E
        for i in range(title_c - len(E)):
            title += ' '
        title += C
        for i in range(title_c - len(D)):
            title += ' '
        title += D
        return title

    def get_dados_wizard(self, cr, capa_id):
        sql = """
                SELECT wizppti_produto_codigo           AS cod_prod,
                       wizppti_produto_name             AS prod_name,
                       wizppti_ncm_codigo               AS ncm,
                       wizppti_encargo_name             AS encargo,
                       wizppti_tributacao_name          AS tributacao,
                       wizppti_cst_cod_name             AS cst,
                       wizppti_nat_rec                  AS nat_rec,
                       wizppti_tipo_tributacao          AS tipo_tributacao,
                       wizppti_preco_venda              AS preco_venda,
                       wizppti_cest                     AS cest
                FROM   wizard_produto_por_tributacao_item
                WHERE  wizppti_capa_id = %s
                ORDER BY wizppti_produto_codigo,
                         wizppti_nat_rec,
                         wizppti_encargo_name  ASC
        """ % capa_id
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        capa_id = context.get('active_id')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Itens por Tributacao'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(1, 1, 1, 1)
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                        'wizard.apuracao.pis.cofins', context=context)
        try:
            empresa_nome = pooler.get_pool(cr.dbname).get('res.company').browse(cr, uid, company_id).name
        except AttributeError:
            empresa_nome = ' '
        data_relat = time.strftime('%d/%m/%Y')
        titulo_empresa = self.center_title(' ', _(empresa_nome), '')
        listagem.informaCab(titulo_empresa)
        titulo = self.center_title(_(data_relat), _('Itens por tributacao'),
                                   _('Pagina: {{ Page }}'))
        listagem.informaCab(titulo)
        #listagem.informaCab(
        #    _('*---------------------------------------------------------------------------------------------------------------------------------*'))
        listagem.informaCab('--')
        cabecalho = formataString(' Cod.Produto',  14, ' ', 'D', 'N', 'N')
        cabecalho += formataString('Descricao',  40, ' ', 'D', 'N', 'N')
        cabecalho += formataString('NCM', 9, ' ', 'C', 'N', 'N')
        cabecalho += formataString('Encargo',  8, ' ', 'C', 'N', 'N')
        cabecalho += formataString('Tributacao', 23, ' ', 'E', 'N', 'N')
        cabecalho += formataString('TIP/TRIB',  21, ' ', 'C', 'N', 'N')
        cabecalho += formataString('Nat', 6, ' ', 'E', 'N', 'N')
        cabecalho += formataString('CST',  6, ' ', 'E', 'N', 'N')
        cabecalho += formataString('CEST',  9, ' ', 'E', 'N', 'N')
        cabecalho += formataString('Preco Venda',  13, ' ', 'E', 'N', 'N')
        listagem.informaCab(cabecalho)
        listagem.informaCab('--')
        listagem.informaCab(" ")
        if capa_id:
            dados = self.get_dados_wizard(cr, capa_id)
            if dados:
                cod_item = ''
                for dado in dados:
                    if cod_item == dado.get('cod_prod'):
                        linha = formataString(' ',  14, ' ', 'D', 'N', 'N')
                    else:
                        if not dado.get('cod_prod'):
                            cod_item = ''
                        else:
                            cod_item = dado.get('cod_prod')[:14]
                        linha = formataString(' ' + cod_item,  14, ' ',
                                              'D', 'N', 'N')
                    if not dado.get('nat_rec'):
                        nat_rec = ''
                    else:
                        nat_rec = dado.get('nat_rec')[:5]
                    if not dado.get('ncm'):
                        ncm = ''
                    else:
                        ncm = dado.get('ncm')[:9]
                    tipo_tributacao = dado.get('tipo_tributacao')
                    linha += formataString(dado.get('prod_name')[:39] or '',  40, ' ', 'D', 'N',
                                           'N')
                    linha += formataString(ncm,  9, ' ', 'C', 'N', 'N')
                    linha += formataString(dado.get('encargo')[:7] or '',  8, ' ', 'E', 'N', 'N')
                    linha += formataString(dado.get('tributacao')[:19] or '', 23, ' ', 'E', 'N',
                                           'N')
                    linha += formataString(tipo_tributacao or '',  21, ' ', 'E', 'N',
                                           'N')
                    linha += formataString(nat_rec, 6, ' ', 'E', 'N', 'N')
                    linha += formataString(dado.get('cst')[:5] or '',  6, ' ', 'E', 'N', 'N')
                    linha += formataString(dado.get('cest') or '',  9, ' ', 'E', 'N', 'N')
                    linha += formataString(getNumeroEditado(dado.get('preco_venda') or 0.00), 13,
                                           ' ', 'E', 'N', 'N')
                    listagem.imprimeLinha(linha)
                    listagem.imprimeLinha(" ")
                    cod_item = dado.get('cod_prod')
        listagem.rodapePadrao()
        listagem.imprimeLinha('')
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return (self.obj.pdf, 'pdf')
report_produto_por_tributacao_item('report.wizard.produto.por.tributacao.item')
