#!/usr/bin/python
# -*- coding: utf-8 -*-
from openerp import pooler
import StringIO
from openerp.osv import osv, fields
from openerp.report.render import render
from dateutil.parser import parse
from openerp.report.interface import report_int
from SigUtils.impressao import Impressora
from SigUtils.formatastring import formataString
from openerp.tools.translate import _
from decimal import Decimal, ROUND_UP
NOME_ARQUIVO_DEFAULT = 'FISCAL_REGISTRO_ENTRADA'


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_registro_entrada(report_int):

    @staticmethod
    def remove_accents(text):
        from unicodedata import normalize, category
        unicode_string = unicode(text, 'utf-8') if type(text) == str else text
        text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

        return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

    def create(self, cr, uid, ids, data, context=None):
        """
        ATENÇÃO: AO REALIZAR ALGUMA ALTERAÇÃO NO RELATORIO, LANÇAR NOTAS E CONFERIR OS VALORES SE ESTAO CORRETOS.

        É DESTACADO NO LIVRO:
            . FRETE POR FORA;
            . IPI DO FRETE POR FORA;
            . GNRE;
            . BASE DE CALCULO ST E VALOR ICMS ST INSERIDOS NA CAPA DA NF;
            . REDUÇÃO DE BASE DE CALCULO;

        COMERCIO
        NO RELATORIO P1A:
            COLUNA (VALOR CONTABIL) == (COLUNA BASE CALCULO)

        INDUSTRIA
        NO RELATORIO P1:
            COLUNA (VALOR CONTABIL) == COLUNA (BASE DE CALCULO DO ICMS) + COLUNA (IMPOSTO CREDITADO DO IPI)

        * EM CASO DE DUVIDAS PROCURAR: LUIS (O PAI DA CRIANÇA)
        """

        context = context and context.copy() or {}
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.informaNome(pdf_string, _('Registro de Entrada'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.3, 0.3, 0.3, 0.3)

        # CONSTANTES DA VARIÁVEL DATA
        FOLHA = data.get('folha') or 0
        DATA_FINAL = str(data.get('data_final') or '')
        DATA_INICIAL = str(data.get('data_inicial') or '')
        COMPANY_ID = data.get('company_id')
        COMPANY_NAME = \
            (data.get('company_name') or '').encode('utf-8') or _('** Empresa nao encontrada **')
        COMPANY_CNPJ = str(data.get('company_cnpj')) or ''
        COMPANY_TIPO_ATIVIDADE = (data.get('company_tipo_atividade') or '').upper()
        COMPANY_INSCRICAO_ESTADUAL = str(data.get('company_inscricao_est')) or ''

        TIPO_COMERCIO = 'C'  # EMPRESA DO TIPO COMERCIO
        self.P1A = 'P1A'  # Comercio
        self.P1 = 'P1'  # Industria

        if COMPANY_TIPO_ATIVIDADE == TIPO_COMERCIO:
            print "____________________REGISTRO ENTRADA - TIPO: COMERCIO___________________________"
            self.str_modelo_relatorio = 'MODELO P1/A'
            self.modelo_relatorio = self.P1A
            self.traco_linha = 150  # QUANTIDADE DE CARACTERES NA HORIZONTAL QUE SERÃO ESCRITOS.
            listagem.defineFonte(tam=6.35)
        else:
            print "____________________REGISTRO ENTRADA - TIPO: INDUSTRIA__________________________"
            self.str_modelo_relatorio = 'MODELO P1'
            self.modelo_relatorio = self.P1
            self.traco_linha = 155  # QUANTIDADE DE CARACTERES NA HORIZONTAL QUE SERÃO ESCRITOS.
            listagem.defineFonte(tam=6.15)

        # CONTROLES GLOBAIS
        self.space = ' ' * self.traco_linha  # ESPAÇO DE LINHA.
        self.db_name = pooler.get_pool(cr.dbname)  # PERMITE ACESSO AS TABELAS.

        # DEFINE O CABEÇALHO DO RELATORIO
        self.define_cabecalho(
            listagem=listagem,
            company_name=COMPANY_NAME,
            company_cnpj=COMPANY_CNPJ,
            company_ie=COMPANY_INSCRICAO_ESTADUAL,
            data_inicial=DATA_INICIAL,
            data_final=DATA_FINAL,
            folha=FOLHA,
        )
        # LISTAGEM DO MOVIMENTO
        resumo_estado = self.listar_movimentos(
            cr, uid,listagem, DATA_INICIAL, DATA_FINAL, COMPANY_ID, COMPANY_TIPO_ATIVIDADE, context)
        if resumo_estado:
            # RESUMO POR ESTADO
            self.resumo_por_estado(listagem, resumo_estado)

        # ATUALIZA A PAGINA E A DATA
        db_name = pooler.get_pool(cr.dbname)
        folha = listagem.getPage()
        db_name.get('adm.paginas').admpag_set_pagina(
            cr, uid, DATA_INICIAL, folha, 'registro_entrada', context)

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return self.obj.pdf, 'pdf'

    def define_cabecalho(self, listagem, company_name, company_cnpj, company_ie, data_inicial,
                         data_final, folha):
        menu_codigo_valores_fiscais = 45
        traco_menu_fiscal = '-' * menu_codigo_valores_fiscais
        linha = self.traco_linha - menu_codigo_valores_fiscais
        space_menu_fiscal = menu_codigo_valores_fiscais - 1
        space_linha = ' ' * linha
        listagem.setPage(folha)

        # DADOS DA EMPRESA
        str_titulo = formataString(_('REGISTRO DE ENTRADA'), linha -
                                   len(self.str_modelo_relatorio), ' ','C','N','S')
        str_company_name = formataString(_('EMPRESA: ') + company_name, linha, ' ','D','N','S')
        str_company_inscricao_est = _('INSCRICAO ESTADUAL: ') + company_ie
        str_company_ie = formataString(str_company_inscricao_est,
                                       len(str_company_inscricao_est), ' ','D','N','S')
        str_company_cnpj = formataString(_('          CNPJ: ') + company_cnpj, linha -
                                         len(str_company_ie), ' ','D','N','S')
        str_periodo = _('PERIODO DE: %s a %s') % (data_inicial, data_final)
        str_periodo = formataString(str_periodo, linha, ' ','D','N','S')
        str_folha = formataString(_('Folha: {{ Page , 5 }}'), linha + 9, ' ','E','N','S')

        # MENU FISCAL COM OS CODIGOS INFORMANDO A OPERAÇÃO
        str_menu_fiscal_titulo = formataString(_('( * ) Codigo de Valores Fiscais'),
                                               space_menu_fiscal, ' ','C','N','S')
        str_menu_traco = formataString(traco_menu_fiscal, space_menu_fiscal, ' ','D','N','S')
        str_menu_fiscal_cod_1 = formataString(_('1 - Operacao com Credito do Imposto'),
                                              space_menu_fiscal, ' ','D','N','S')
        str_menu_fiscal_cod_2 = formataString(_('2 - Operacao sem Credito do Imposto-Isenta'),
                                              space_menu_fiscal, ' ','D','N','S')
        str_menu_fiscal_cod_2_continue = formataString(_('    ou nao-tributadas'),
                                                       space_menu_fiscal, ' ','D','N','S')
        str_menu_fiscal_cod_3 = formataString(_('3 - Operacao com Substituicao Tributaria'),
                                              space_menu_fiscal, ' ','D','N','S')
        str_menu_fiscal_cod_4 = formataString(_('4 - Operacao sem Credito do Imposto - Outras'),
                                              space_menu_fiscal, ' ','D','N','S')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab('|' + str_titulo + self.str_modelo_relatorio + '|' +
                            str_menu_fiscal_titulo + '|')
        listagem.informaCab('|' + space_linha + '|' + str_menu_traco + '|')
        listagem.informaCab('|' + str_company_name + '|' + str_menu_fiscal_cod_1 + '|')
        listagem.informaCab('|' + space_linha + '|' + str_menu_fiscal_cod_2 + '|')
        listagem.informaCab('|' + str_company_ie + str_company_cnpj + '|' +
                            str_menu_fiscal_cod_2_continue + '|')
        listagem.informaCab('|' + str_folha + '|' + str_menu_fiscal_cod_3 + '|' )
        listagem.informaCab('|' + str_periodo + '|' + str_menu_fiscal_cod_4 + '|')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        if self.modelo_relatorio == self.P1A:  # Comercio
            listagem.informaCab(_('|          |             DOCUMENTOS FISCAIS              |            |  CODIFICACAO  |        VALORES FISCAIS                     |                   |'))
            listagem.informaCab(_('| DATA DE  |---------------------------------------------|   VALOR    |---------------|--------------------------------------------|                   |'))
            listagem.informaCab(_('|          |    |SERIE|         |DATA DO   |CODIGO  | UF |            |        |      |ICMS|COD| BASE DE CALCULO  |     |  IMPOSTO |    OBSERVACOES    |'))
            listagem.informaCab(_('| ENTRADA  |ESPE|SUB- | NUMERO  |          |        |    |  CONTABIL  |CONTABIL|FISCAL|    |   |       OU         |ALIQ.|          |                   |'))
            listagem.informaCab(_('|          |CIE |SERIE|         |DOCUMENTO |EMITENTE|ORIG|            |        |      |IPI |(*)| VALOR DA OPERACAO|     | CREDITADO|                   |'))
        else:  # Industria - Modelo P1
            listagem.informaCab(_('|          |          DOCUMENTOS FISCAIS          |            |CODIFICACAO|        ICMS VALORES FISCAIS    |   IPI VALORES FISCAIS    |                    |'))
            listagem.informaCab(_('| DATA DE  |--------------------------------------|    VALOR   |-----------|--------------------------------|--------------------------|                    |'))
            listagem.informaCab(_('|          |ES-|    |       |DATA DO   |CODIGO|   |            |    |      |   |  BASE DE   |     | IMPOSTO |   |  BASE DE   | IMPOSTO |                    |'))
            listagem.informaCab(_('|          |PE-|SE- |NUMERO |          |      |   |            |CON-|FISCAL|COD|  CALCULO   |ALIQ.|         |COD|  CALCULO   |         |     OBSERVACOES    |'))
            listagem.informaCab(_('| ENTRADA  |CIE|RIE |       |DOCUMENTO | EMI- |UF |  CONTABIL  |TA- |      |(*)|  VALOR DA  |     |CREDITADO|(*)|  VALOR DA  |CREDITADO|                    |'))
            listagem.informaCab(_('|          |   |    |       |          |TENTE |   |            |BIL |      |   |  OPERACAO  |     |         |   |  OPERACAO  |         |                    |'))
        listagem.informaRodape('*' + '-' * self.traco_linha + '*')

        return True

    def resumo_por_estado(self, listagem, resumo_uf):
        col_resumo_uf = 21
        col_resumo_valor_contabil = 16
        col_resumo_base_icms = 15
        col_resumo_valor_icms = 12
        col_resumo_base_ipi = 12
        col_resumo_valor_ipi = 11
        col_resumo_isento = 13
        col_resumo_outros = 15
        if self.modelo_relatorio == self.P1A:
            col_resumo_substituicao = 27
        else:
            col_resumo_substituicao = 32
        listagem.cabecalhos = []  # APAGA O CABEÇALHO
        str_titulo_registro = formataString(_('*** REGISTRO DE ENTRADAS ***'), self.traco_linha -
                                            len(self.str_modelo_relatorio), ' ','C','N','S')
        str_resumo_estado = formataString(_('RESUMO POR ESTADO'), self.traco_linha, ' ','C','N','S')
        str_folha = formataString(_('Folha: {{ Page , 5 }}'), self.traco_linha + 9, ' ','E','N','S')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab('|' + str_titulo_registro + self.str_modelo_relatorio + '|')
        listagem.informaCab('|' + self.space + '|')
        listagem.informaCab('|' + str_folha + '|')
        listagem.informaCab('|' + str_resumo_estado + '|')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        if self.modelo_relatorio == self.P1A:
            listagem.informaCab(_('|        ESTADO       |   VALORES      |    BASE DE    |    VALOR   |   BASE DE  |   VALOR   |   ISENTOS   |    OUTROS     |       SUBSTITUICAO        |'))
            listagem.informaCab(_('|                     |   CONTABEIS    |    C. ICMS    |    ICMS    |    C.IPI   |   I P I   |   TRIBUTAD  |               |                           |'))
        else:
            listagem.informaCab(_('|        ESTADO       |   VALORES      |    BASE DE    |    VALOR   |   BASE DE  |   VALOR   |   ISENTOS   |    OUTROS     |          SUBSTITUICAO          |'))
            listagem.informaCab(_('|                     |   CONTABEIS    |    C. ICMS    |    ICMS    |    C.IPI   |   I P I   |   TRIBUTAD  |               |                                |'))
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.imprimeLinha('{{ Page }}')
        resumo_total_valor_contabil = 0
        resumo_total_base_icms = 0
        resumo_total_valor_icms = 0
        resumo_total_base_ipi = 0
        resumo_total_valor_ipi = 0
        resumo_total_isento = 0
        resumo_total_outros = 0
        resumo_total_substituicao = 0
        for i in resumo_uf:
            # PODE HAVER CHAVE COM VAZIO.
            if i:
                resumo_valor_icms = float(
                    Decimal(str(resumo_uf[i].get('resumo_valor_icms') or 0)).quantize(
                        Decimal('.01'), rounding=ROUND_UP))
                ESTADO = str(i) # i e unicode
                RESUMO_VALOR_CONTABIL = resumo_uf[i].get('resumo_valor_contabil') or 0
                RESUMO_BASE_ICMS = resumo_uf[i].get('resumo_base_icms') or 0
                RESUMO_VALOR_ICMS = resumo_valor_icms
                RESUMO_BASE_IPI = resumo_uf[i].get('resumo_base_ipi') or 0
                RESUMO_VALOR_IPI = resumo_uf[i].get('resumo_valor_ipi') or 0
                RESUMO_ISENTO = resumo_uf[i].get('resumo_isento') or 0
                RESUMO_OUTROS = resumo_uf[i].get('resumo_outros') or 0
                RESUMO_SUBSTITUICAO = resumo_uf[i].get('resumo_substituicao') or 0
                str_resumo_uf = formataString(ESTADO, col_resumo_uf, ' ', 'C', 'S', 'S')
                str_resumo_valor_contabil = formataString(
                    '%.2f' % RESUMO_VALOR_CONTABIL, col_resumo_valor_contabil, ' ', 'E', 'S', 'S')
                str_resumo_base_icms = formataString(
                    '%.2f' % RESUMO_BASE_ICMS, col_resumo_base_icms, ' ', 'E', 'S', 'S')
                str_resumo_valor_icms = formataString(
                    '%.2f' % RESUMO_VALOR_ICMS, col_resumo_valor_icms, ' ', 'E', 'S', 'S')
                str_resumo_base_ipi = formataString(
                    '%.2f' % RESUMO_BASE_IPI, col_resumo_base_ipi, ' ', 'E', 'S', 'S')
                str_resumo_valor_ipi = formataString(
                    '%.2f' % RESUMO_VALOR_IPI, col_resumo_valor_ipi, ' ', 'E', 'S', 'S')
                str_resumo_isento = formataString(
                    '%.2f' % RESUMO_ISENTO, col_resumo_isento, ' ', 'E', 'S', 'S')
                str_resumo_outros = formataString(
                    '%.2f' % RESUMO_OUTROS, col_resumo_outros, ' ', 'E', 'S', 'S')
                str_resumo_substituicao = formataString(
                    '%.2f' % RESUMO_SUBSTITUICAO, col_resumo_substituicao, ' ', 'E', 'S', 'S')

                listagem.imprimeLinha(
                    '|' + str_resumo_uf + '|' + str_resumo_valor_contabil + '|' +
                    str_resumo_base_icms + '|' + str_resumo_valor_icms + '|' + str_resumo_base_ipi +
                    '|' + str_resumo_valor_ipi + '|' + str_resumo_isento + '|' + str_resumo_outros +
                    '|' + str_resumo_substituicao + '|')

                # TOTAIS DOS RESUMOS POR ESTADOS
                resumo_total_valor_contabil += RESUMO_VALOR_CONTABIL
                resumo_total_base_icms += RESUMO_BASE_ICMS
                resumo_total_valor_icms += RESUMO_VALOR_ICMS
                resumo_total_base_ipi += RESUMO_BASE_IPI
                resumo_total_valor_ipi += RESUMO_VALOR_IPI
                resumo_total_isento += RESUMO_ISENTO
                resumo_total_outros += RESUMO_OUTROS
                resumo_total_substituicao += RESUMO_SUBSTITUICAO

        # FORMATAÇÃO PARA IMPRESSÃO
        str_uf_space = formataString(_('TOTAIS'), col_resumo_uf, ' ', 'D', 'S', 'S')
        str_total_valor_contabil = formataString(
            '%.2f' % resumo_total_valor_contabil, col_resumo_valor_contabil, ' ', 'E', 'S', 'S')
        str_total_base_icms = formataString(
            '%.2f' % resumo_total_base_icms, col_resumo_base_icms, ' ', 'E', 'S', 'S')
        str_total_valor_icms = formataString(
            '%.2f' % resumo_total_valor_icms, col_resumo_valor_icms, ' ', 'E', 'S', 'S')
        str_total_base_ipi = formataString(
            '%.2f' % resumo_total_base_ipi, col_resumo_base_ipi, ' ', 'E', 'S', 'S')
        str_total_valor_ipi = formataString(
            '%.2f' % resumo_total_valor_ipi, col_resumo_valor_ipi, ' ', 'E', 'S', 'S')
        str_total_isento = formataString(
            '%.2f' % resumo_total_isento, col_resumo_isento, ' ', 'E', 'S', 'S')
        str_total_outros = formataString(
            '%.2f' % resumo_total_outros, col_resumo_outros, ' ', 'E', 'S', 'S')
        str_total_substituicao = formataString(
            '%.2f' % resumo_total_substituicao, col_resumo_substituicao, ' ', 'E', 'S', 'S')
        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
        listagem.imprimeLinha(
            '|' + str_uf_space + ' ' + str_total_valor_contabil + ' ' + str_total_base_icms +
            ' ' + str_total_valor_icms + ' ' + str_total_base_ipi + ' ' + str_total_valor_ipi +
            ' ' + str_total_isento + ' ' + str_total_outros + ' ' + str_total_substituicao + '|')
        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')

        return True

    def get_movimento_data(self, cr, uid, data_inicial, data_final, company_id, tipo_atividade,
                           simples_nacional):
        print "BUSCAR NOTAS ENTRE AS DATAS %s e %s" % (data_inicial, data_final)
        if tipo_atividade == 'I':
            filtro_aliquota = """
                CASE
                    WHEN nf.nf_propria IS FALSE
                        AND ap.admpes_jur_regime_tributario = 'S'
                        AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                        AND cst.tipo = 'T'
                        AND fnit.vr_encargo > 0
                        AND encargo.sigla = 'ICMS'
                    THEN 0.00
                    ELSE fnit.aliquota
                END                                             AS relat_aliquota,
            """
        else:
            filtro_aliquota = """
                   CASE
                       WHEN nf.nf_propria IS FALSE
                           AND ap.admpes_jur_regime_tributario = 'S'
                           AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                           AND cst.tipo = 'T'
                           AND fnit.vr_encargo > 0
                           AND encargo.sigla = 'ICMS'
                       THEN 0.00
                       WHEN cst.tipo = 'T' AND encargo.sigla != 'IPI'
                       THEN fnit.aliquota
                       ELSE 0
                   END                                          AS relat_aliquota,
            """
        notas_sql = """
        SELECT destinatario_id,
               relat_data_entrada,
               modelo_nf_id,
               relat_especie,
               relat_serie,
               relat_numero,
               relat_data_documento,
               relat_codigo_emitente,
               relat_uf,
               SUM(relat_valor_contabil)                   AS relat_valor_contabil,
               relat_contabil,
               relat_cfop_codigo,
               relat_encargo_sigla,
               relat_cod_fiscal,
               SUM(relat_base_calculo)                     AS relat_base_calculo,
               relat_aliquota,
               SUM(relat_imposto_creditado)                AS relat_imposto_creditado,
               uf_resumo,
               nf_id,
               nf_propria,
               data_ordenacao,
               item_base_icms_subst,
               tipo_frete,
               relat_valor_frete,
               nf_state,
               nfe_chave_acesso,
               relat_observacao_fisco,
               nfe_modelo,
               finalidade_emissao,
               SUM(item_valor_icms_subst)                  AS item_valor_icms_subst,
               SUM(item_valor_icms_desonerado)             AS item_valor_icms_desonerado,
               SUM(item_valor_reducao_base)                AS item_valor_reducao_base,
               SUM(item_valor_desconto)                    AS item_valor_desconto,
               SUM(item_valor_desconto_rateado)            AS item_valor_desconto_rateado,
               SUM(item_valor_frete_rateado)               AS item_valor_frete_rateado,
               SUM(item_valor_seguro_rateado)              AS item_valor_seguro_rateado,
               SUM(item_valor_despesas_acessorias_rateado) AS item_valor_despesas_acessorias_rateado
        FROM (
            SELECT nf.destinatario_id                           AS destinatario_id,
                   FORMAT('%s/%s/%s',
                          EXTRACT( DAY FROM nf.data_saida_entrada AT TIME ZONE 'UTC'),
                          EXTRACT( MONTH FROM nf.data_saida_entrada AT TIME ZONE 'UTC'),
                          EXTRACT( YEAR FROM nf.data_saida_entrada AT TIME ZONE 'UTC'))
                                                                AS relat_data_entrada,
                   CASE WHEN fnm.id IS NOT NULL
                        THEN fnm.id
                        WHEN fnms.modelo_nf_id IS NOT NULL
                        THEN fnms.modelo_nf_id
                        ELSE NULL
                   END                                          AS modelo_nf_id,
                   fnm.fismnf_tipo                              AS relat_especie,
                   nf.serie_nf                                  AS relat_serie,
                   nf.numero_nf                                 AS relat_numero,
                   nf.data_emissao                              AS relat_data_documento,
                   ap.codigo                                    AS relat_codigo_emitente,
                   au.sigla                                     AS relat_uf,
                   CASE
                        WHEN encargo.sigla = 'ICMS' THEN
                            SUM(coalesce(emi.vr_total_item_rateado, 0) +
                                coalesce(fnit.vr_icms_substituicao, 0) +
                                COALESCE(fnit.fisnit_vr_fcp_st, 0))
                        WHEN encargo.sigla = 'IPI' AND encargo.compoe_valor_doc THEN
                            SUM(coalesce(fnit.vr_encargo, 0))
                        ELSE
                            0
                   END                                          AS relat_valor_contabil,
                   ''                                           AS relat_contabil,
                   cfop.codigo                                  AS relat_cfop_codigo,
                   encargo.sigla                                AS relat_encargo_sigla,
                   -- Quando o tipo de atividade for diferente de industria o codigo para ipi eh 4
                   CASE
                        WHEN nf.nf_propria IS FALSE
                            AND ap.admpes_jur_regime_tributario = 'S'
                            AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                            AND cst.tipo = 'T'
                            AND fnit.vr_encargo > 0
                            AND encargo.sigla = 'ICMS'
                        THEN 4
                        WHEN encargo.sigla = 'IPI' and  cst.tipo = 'T' AND '{tipo_atividade}' != 'I'
                        THEN 4
                        WHEN cst.tipo = 'T' AND NOT {simples_nacional}
                        THEN 1
                        WHEN cst.tipo = 'I'
                        THEN 2
                        WHEN cst.tipo IN ('U', 'TU')
                        THEN 3
                        WHEN cst.tipo IN ( 'O', 'D', 'S') OR (cst.tipo = 'T' AND {simples_nacional})
                        THEN 4
                        ELSE 0
                   END                                          AS relat_cod_fiscal,
                   SUM(COALESCE (fnit.vr_base_calculo, 0) + COALESCE(fnit.fisnit_vr_fcp_st, 0) +
                       CASE WHEN cst.tipo IN ('U', 'TU')
                                THEN COALESCE (fnit.vr_reducao_base, 0)
                            ELSE 0
                       END
                       )                                        AS relat_base_calculo,
                   {aliquota}
                   SUM(
                       CASE
                           WHEN nf.nf_propria IS FALSE
                               AND ap.admpes_jur_regime_tributario = 'S'
                               AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                               AND cst.tipo = 'T'
                               AND fnit.vr_encargo > 0
                               AND encargo.sigla = 'ICMS'
                           THEN 0.00
                           ELSE COALESCE (fnit.vr_encargo, 0)
                       END
                   )                                            AS relat_imposto_creditado,
                   au.sigla                                     AS uf_resumo,
                   nf.id                                        AS nf_id,
                   nf.nf_propria                                AS nf_propria,
                   FORMAT('%s/%s/%s',
                          EXTRACT( DAY FROM nf.data_saida_entrada AT TIME ZONE 'UTC'),
                          EXTRACT( MONTH FROM nf.data_saida_entrada AT TIME ZONE 'UTC'),
                          EXTRACT( YEAR FROM nf.data_saida_entrada AT TIME ZONE 'UTC'))
                                                                AS data_ordenacao,
                   nf.vr_base_icms_substituicao                 AS item_base_icms_subst,
                   nf.tipo_frete,
                   nf.vr_frete                                  AS relat_valor_frete,
                   nf.state                                     AS nf_state,
                   nf.nfe_chave_acesso                          AS nfe_chave_acesso,
                   nf.fisnfc_infcomp_fisco                      AS relat_observacao_fisco,
                   fnm.fismnf_tipo_nfe                          AS nfe_modelo,
                   nf.finalidade_emissao                        AS finalidade_emissao,
                   SUM(COALESCE (fnit.vr_icms_substituicao, 0)) AS item_valor_icms_subst,
                   SUM(COALESCE (fnit.fisnit_vr_icms_desonerado, 0))    AS item_valor_icms_desonerado,
                   SUM(COALESCE (fnit.vr_reducao_base, 0))              AS item_valor_reducao_base,
                   SUM(COALESCE (emi.vr_desconto, 0))                   AS item_valor_desconto,
                   SUM(COALESCE (emi.vr_desconto_rateado, 0))           AS item_valor_desconto_rateado,
                   SUM(COALESCE (emi.vr_frete_rateado, 0))              AS item_valor_frete_rateado,
                   SUM(COALESCE (emi.vr_seguro_rateado, 0))             AS item_valor_seguro_rateado,
                   SUM(COALESCE (emi.vr_despesas_rateado, 0))           AS item_valor_despesas_acessorias_rateado
            FROM   fiscal_nf_capa                                       AS nf
            INNER JOIN adm_pessoa                                       AS ap
                    ON ap.id = nf.destinatario_id
            INNER JOIN adm_pessoa_endereco                              AS ape
                    ON ape.id = nf.destinatario_endereco_id
            INNER JOIN adm_uf                                           AS au      
                    ON au.id = ape.uf_id
            LEFT  JOIN fiscal_modelo_nf                                 AS fnm
                    ON fnm.id = nf.modelo_nf_id
            LEFT  JOIN fiscal_modelo_nf_series                          AS fnms
                    ON fnms.id = nf.modelo_nf_series_id
            LEFT  JOIN estoque_movimento_itens                          AS emi
                    ON emi.nf_id = nf.id
            LEFT  JOIN fiscal_cfop                                      AS cfop
                    ON cfop.id = emi.cfop_id
            LEFT  JOIN fiscal_nf_item_tributacao                        AS fnit
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo                                   AS encargo
                    ON encargo.id = fnit.encargo_id
            LEFT  JOIN fiscal_cst                                       AS cst
                    ON cst.id = fnit.cst_id
            LEFT  JOIN res_company                                      AS empresa
                    ON empresa.id = nf.company_id
            LEFT  JOIN adm_pessoa                                       AS pessoa_empresa
                    ON empresa.rescon_pessoa_id = pessoa_empresa.id
            WHERE  nf.state IN ( '3', 'C', 'I' )
              AND  nf.tipo = 'E'
              AND  au.id = ape.uf_id
              AND  nf.data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '{data_inicio}' AND '{data_fim}'
              AND  encargo.sigla IN('ICMS', 'IPI')
              AND  nf.company_id = {company_id}
              AND  fnm.fismnf_tipo != 'T'
            GROUP  BY nf.destinatario_id, nf.data_saida_entrada, fnm.fismnf_tipo, fnm.id,
                      fnms.modelo_nf_id, nf.serie_nf, nf.numero_nf, nf.data_emissao, ap.codigo,
                      au.sigla, relat_contabil, cfop.id, encargo.sigla, cst.tipo, relat_aliquota,
                      nf.id, nf.nf_propria, nf.state, nf.vr_base_icms_substituicao, nf.tipo_frete,
                      nf.vr_frete, nf.vr_aliquota_frete, nf.vr_icms_frete, nf.vr_ipi_frete,
                      nf.numero_conhecimento, nf.serie_frete, nf.data_emissao_frete,
                      nf.nfe_chave_acesso, nf.fisnfc_infcomp_fisco, fnm.fismnf_tipo_nfe,
                      encargo.compoe_valor_doc, relat_cod_fiscal, finalidade_emissao
            HAVING (nf.state != 'C'
               AND (encargo.sigla = 'ICMS'
                OR (encargo.sigla = 'IPI'
               AND SUM(fnit.vr_encargo) > 0 )))
                OR (nf.state = 'C'
               AND nf.nf_propria = TRUE
               AND ((nf.modelo_55= FALSE AND nf.numero_nf > 0 AND nf.serie_nf <> '' )
                OR ( nf.modelo_55 = TRUE AND nf.nfe_chave_acesso != '' ) ) )
            ) AS tmp
        GROUP BY destinatario_id, relat_data_entrada, modelo_nf_id, relat_especie, relat_serie,
                 relat_numero, relat_data_documento, relat_codigo_emitente, relat_uf, relat_contabil, 
                 relat_cfop_codigo, relat_encargo_sigla, relat_cod_fiscal, relat_aliquota, uf_resumo,
                 nf_id, nf_propria, data_ordenacao, item_base_icms_subst, tipo_frete, 
                 relat_valor_frete, nf_state, nfe_chave_acesso, relat_observacao_fisco, nfe_modelo,
                 finalidade_emissao
        ORDER BY relat_data_entrada, relat_codigo_emitente, relat_numero, relat_serie,
                 relat_cod_fiscal
        """.format(tipo_atividade=tipo_atividade, simples_nacional=simples_nacional,
                   data_inicio=data_inicial, data_fim=data_final, company_id=company_id,
                   aliquota=filtro_aliquota)
        cr.execute(notas_sql)
        nota_dados = cr.dictfetchall()
        sql_notas_sem_itens = '''
            SELECT nf.destinatario_id                               AS destinatario_id,
                   FORMAT('%s/%s/%s',
                          EXTRACT( DAY FROM nf.data_saida_entrada   AT TIME ZONE 'UTC'),
                          EXTRACT( MONTH FROM nf.data_saida_entrada AT TIME ZONE 'UTC'),
                          EXTRACT( YEAR FROM nf.data_saida_entrada  AT TIME ZONE 'UTC'))
                                                                    AS relat_data_entrada,
                   CASE WHEN fnm.id IS NOT NULL
                        THEN fnm.id
                        WHEN fnms.modelo_nf_id IS NOT NULL
                        THEN fnms.modelo_nf_id
                        ELSE NULL
                   END                                              AS modelo_nf_id,
                   fnm.fismnf_tipo                                  AS relat_especie,
                   nf.serie_nf                                      AS relat_serie,
                   nf.numero_nf                                     AS relat_numero,
                   nf.data_emissao                                  AS relat_data_documento,
                   ap.codigo                                        AS relat_codigo_emitente,
                   au.sigla                                         AS relat_uf,
                   SUM(nf.vr_total_nf)                              AS relat_valor_contabil,
                   ''                                               AS relat_contabil,
                   cfop.codigo                                      AS relat_cfop_codigo,
                   encargo.sigla                                    AS relat_encargo_sigla,
                   -- Quando o tipo de atividade for diferente de industria o codigo para ipi eh 4
                   CASE WHEN encargo.sigla = 'IPI' AND  cst.tipo = 'T' AND '{tipo_atividade}' != 'I'
                        THEN 4
                        WHEN cst.tipo = 'T' AND NOT {simples_nacional}
                        THEN 1
                        WHEN cst.tipo = 'I'
                        THEN 2
                        WHEN cst.tipo IN ('U', 'TU')
                        THEN 3
                        WHEN cst.tipo IN ( 'O', 'D', 'S') OR (cst.tipo = 'T' AND {simples_nacional})
                        THEN 4
                        ELSE 0
                   END                                              AS relat_cod_fiscal,
                   SUM(fnit.vr_base_calculo)                        AS relat_base_calculo,
                   fnit.aliquota                                    AS relat_aliquota,
                   SUM(fnit.vr_encargo)                             AS relat_imposto_creditado,
                   nf.tipo_frete,
                   nf.vr_frete                                      AS relat_valor_frete,
                   au.sigla                                         AS uf_resumo,
                   nf.id                                            AS nf_id,
                   nf.nf_propria                                    AS nf_propria,
                   FORMAT('%s/%s/%s',
                          EXTRACT( DAY FROM nf.data_saida_entrada   AT TIME ZONE 'UTC'),
                          EXTRACT( MONTH FROM nf.data_saida_entrada AT TIME ZONE 'UTC'),
                          EXTRACT( YEAR FROM nf.data_saida_entrada  AT TIME ZONE 'UTC'))
                                                                    AS data_ordenacao,
                   nf.vr_base_icms_substituicao                     AS item_base_icms_subst,
                   nf.state                                         AS nf_state,
                   nf.nfe_chave_acesso                              AS nfe_chave_acesso,
                   nf.fisnfc_infcomp_fisco                          AS relat_observacao_fisco,
                   fnm.fismnf_tipo_nfe                              AS nfe_modelo,
                   SUM(fnit.vr_icms_substituicao)                   AS item_valor_icms_subst,
                   SUM(fnit.vr_reducao_base)                        AS item_valor_reducao_base,
                   SUM(fnit.fisnit_vr_icms_desonerado)              AS item_valor_icms_desonerado,
                   0                                                AS item_valor_desconto,
                   0                                                AS item_valor_desconto_rateado,
                   0                                                AS item_valor_frete_rateado,
                   0                                                AS item_valor_seguro_rateado,
                   0                                                AS item_valor_despesas_acessorias_rateado,
                   nf.finalidade_emissao                            AS finalidade_emissao
            FROM   fiscal_nf_capa                                   AS nf
            INNER JOIN fiscal_cfop_natureza                         AS fcn
                  ON fcn.id = nf.natureza_id
            INNER JOIN adm_pessoa                                   AS ap
                  ON ap.id = nf.destinatario_id
            LEFT  JOIN adm_pessoa_endereco                          AS ape
                  ON ape.id = nf.destinatario_endereco_id
            LEFT  JOIN adm_uf                                       AS au  
                  ON au.id = ape.uf_id
            LEFT  JOIN fiscal_modelo_nf                             AS fnm
                  ON fnm.id = nf.modelo_nf_id
            LEFT  JOIN fiscal_modelo_nf_series                      AS fnms
                  ON fnms.id = nf.modelo_nf_series_id
            LEFT  JOIN fiscal_cfop                                  AS cfop
                  ON cfop.id = fcn.cfop_id
            INNER JOIN fiscal_nf_item_tributacao                    AS fnit
                  ON fnit.fisnit_capa_id = nf.id
            INNER JOIN fiscal_encargo                               AS encargo
                  ON encargo.id = fnit.encargo_id
            LEFT  JOIN fiscal_cst                                   AS cst
                  ON cst.id = fnit.cst_id
            WHERE nf.data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '{data_inicio}' AND '{data_fim}'
              AND nf.state IN ( '3', 'C', 'I' )
              AND nf.tipo = 'E'
              AND encargo.sigla IN ('ICMS', 'IPI')
              AND nf.company_id = {company_id}
              AND fnm.fismnf_tipo = 'T'
            GROUP BY nf.destinatario_id, nf.data_saida_entrada, fnm.fismnf_tipo, fnm.id,
                     fnms.modelo_nf_id, nf.serie_nf, nf.numero_nf, nf.data_emissao, ap.codigo,
                     au.sigla, relat_contabil, cfop.id, encargo.sigla, cst.tipo, fnit.aliquota,
                     nf.id, nf.nf_propria, nf.state, nf.vr_base_icms_substituicao, nf.tipo_frete,
                     nf.vr_frete, nf.vr_aliquota_frete, nf.vr_icms_frete, nf.vr_ipi_frete,
                     nf.numero_conhecimento, nf.serie_frete, nf.data_emissao_frete,
                     nf.nfe_chave_acesso, nf.fisnfc_infcomp_fisco, fnm.fismnf_tipo_nfe, 
                     nf.finalidade_emissao
            HAVING (nf.state != 'C'
               AND (encargo.sigla = 'ICMS'
                OR (encargo.sigla = 'IPI'
               AND SUM(fnit.vr_encargo) > 0 )))
                OR (nf.state = 'C'
               AND nf.nf_propria = TRUE
               AND ((nf.modelo_55= FALSE AND nf.numero_nf > 0 AND nf.serie_nf <> '' )
                OR (nf.modelo_55 = TRUE AND nf.nfe_chave_acesso != '' ) ) )
            ORDER  BY relat_data_entrada, ap.codigo, nf.numero_nf, nf.serie_nf, relat_cod_fiscal
        '''.format(tipo_atividade=tipo_atividade, simples_nacional=simples_nacional,
                   data_inicio=data_inicial, data_fim=data_final, company_id=company_id)
        cr.execute(sql_notas_sem_itens)
        notas_sem_itens_dados = cr.dictfetchall()
        if notas_sem_itens_dados:
            nota_dados.extend(notas_sem_itens_dados)
        obj_fiscal_modelo_nf = pooler.get_pool(cr.dbname).get('fiscal.modelo.nf')
        nota_dados = filter(lambda i: obj_fiscal_modelo_nf.verifica_codigo_modelo_fiscal(
            cr, uid, i.get('modelo_nf_id')), nota_dados)

        return nota_dados

    def get_nf_guia(self, cr, nf_ids):
        nf_ids = map(str, nf_ids)
        str_nf_ids = str(nf_ids).replace('[', '(').replace(']', ')')
        guia_sql = """
            SELECT nf.id           AS nf_id,
                   SUM(guia.base)  AS nf_guia_base,
                   SUM(guia.valor) AS nf_guia_valor
            FROM   fiscal_guia guia
            INNER JOIN fiscal_nf_capa nf 
                    ON guia.nf_id = nf.id
            WHERE nf.id IN %s
            GROUP BY nf.id
        """ % str_nf_ids
        cr.execute(guia_sql)
        guia_dados = cr.dictfetchall()
        guia_dados_organizados = {}
        if guia_dados:
            for i in guia_dados:
                nf_id = i.get('nf_id')
                guia_base = i.get('nf_guia_base') or 0
                guia_valor = i.get('nf_guia_valor') or 0
                guia_dados_organizados.update({
                    nf_id: {
                        'nf_guia_base': guia_base,
                        'nf_guia_valor': guia_valor
                    }
                })

        return guia_dados_organizados

    def processar_dados_impressao(self, cr, uid, data_inicial, data_final, company_id,
                                  tipo_atividade, simples_nacional):
        dados_sql = self.get_movimento_data(
            cr, uid, data_inicial, data_final, company_id, tipo_atividade, simples_nacional)
        dados_repetidos = {}
        dados_organizados = {}
        indice = 1
        converte_tz = pooler.get_pool(cr.dbname).get('adm.utils').admutl_converte_tz
        for i in dados_sql:
            data_ordenacao = i.get('data_ordenacao') or ''
            RELAT_ESPECIE = i.get('relat_especie') or '' # ** COLUNA ESPECIE **
            RELAT_SERIE = i.get('relat_serie') or '' # ** COLUNA SUBSÉRIE **
            RELAT_NUMERO = i.get('relat_numero') or ''# ** COLUNA NUMERO **
            RELAT_DATA_DOCUMENTO = converte_tz(
                cr, uid, parse(i.get('relat_data_documento')), 'UTC', 'America/Sao_Paulo',
                {}).strftime('%d/%m/%Y') if i.get('relat_data_documento') else ''
            # ** COLUNA DATA DOCUMENTO **
            RELAT_CODIGO_EMITENTE = i.get('relat_codigo_emitente') or ''# ** COLUNA CODIGO EMITENTE **
            RELAT_UF = (i.get('relat_uf') or '').upper() # ** COLUNA UF **
            if i.get('relat_valor_contabil'):
                i['relat_valor_contabil'] = sum(
                    map(lambda x: x.get('relat_valor_contabil')
                        if x.get('relat_valor_contabil') > 0 and
                        x.get('relat_numero') == i.get('relat_numero') and
                        x.get('relat_serie') == i.get('relat_serie') and
                        x.get('destinatario_id') == i.get('destinatario_id') else 0, dados_sql))
            else:
                i['relat_valor_contabil'] = 0
            linha_relatorio = '%s%s%s%s%s%s%s' % (
                data_ordenacao, RELAT_ESPECIE, RELAT_SERIE, RELAT_NUMERO, RELAT_DATA_DOCUMENTO,
                RELAT_CODIGO_EMITENTE, RELAT_UF)
            key = hash(linha_relatorio)
            if key in dados_repetidos:
                # SE HÁ MAIS DE UMA LINHA PARA A MEMSA NOTA,
                # OS DADOS ABAIXO NÃOS SERÃO NECESSÁRIOS SEREM EXIBIDOS NOVAMENTE.
                i['relat_data_entrada'] = ''
                i['relat_especie'] = ''
                i['relat_serie'] = ''
                i['relat_numero'] = ''
                i['relat_data_documento'] = ''
                i['relat_codigo_emitente'] = ''
                i['relat_uf'] = ''
                i['relat_valor_contabil'] = sum(
                    map(lambda x: x.get('relat_valor_contabil')
                        if x.get('relat_valor_contabil') > 0 and
                        x.get('relat_numero') == i.get('relat_numero') and
                        x.get('relat_serie') == i.get('relat_serie') and
                        x.get('destinatario_id') == i.get('destinatario_id') and
                        x.get('relat_encargo_sigla') == 'IPI' and
                        x.get('finalidade_emissao') == '2' else 0, dados_sql))
                dados_repetidos[key] += 1
            else:
                dados_repetidos[key] = 1
            i['hash_key'] = key
            if data_ordenacao in dados_organizados:
                dados_organizados[data_ordenacao].append(i)
            else:
                dados_organizados[data_ordenacao] = [i]
            indice += 1
        for key in dados_repetidos:
            value = dados_repetidos[key]
            for data in dados_organizados:
                dados = filter(lambda i: i.get('hash_key') == key, dados_organizados[data])
                for q in dados:
                    q['total_registros_por_nota'] = value

        return dados_organizados

    def listar_movimentos(self, cr, uid, listagem, data_inicial, data_final, company_id,
                          tipo_atividade, context=None):
        context = context and context.copy() or {}
        if self.modelo_relatorio == self.P1A:
            col_data_entrada = 10
            col_especie = 4
            col_serie = 5
            col_numero = 9
            col_data_documento = 10
            col_emitente = 8
            col_uf = 4

            # GRUPO
            col_grupo_doc_fiscais = col_especie + col_serie + col_numero + col_data_documento + \
                                    col_emitente + col_uf

            col_valor_contabil = 12
            col_contabil = 8
            col_fiscal = 6
            col_encargo_sigla = 4

            # GRUPO
            col_grupo_codificacao = col_contabil + col_fiscal

            # APURAÇÃO ICMS
            col_codigo_menu_fiscal = 3
            col_base_calculo = 18
            col_aliquota = 5
            col_imposto_creditado = 10
            col_obs = 19

            col_ipi_cod = 3
            col_ipi_base_calculo = 12
            col_ipi_imposto_creditado = 9

            # COLUNA DOS TOTAIS
            col_totais_name = col_data_entrada + col_grupo_doc_fiscais + 6
            col_totais_space1 = \
                col_grupo_codificacao + col_encargo_sigla + col_codigo_menu_fiscal + 3
            col_totais_space2 = col_aliquota
        else:
            col_data_entrada = 10
            col_especie = 3
            col_serie = 4
            col_numero = 7
            col_data_documento = 10
            col_emitente = 6
            col_uf = 3

            # GRUPO
            col_grupo_doc_fiscais = \
                col_especie + col_serie + col_numero + col_data_documento + col_emitente + col_uf

            col_valor_contabil = 12
            col_contabil = 4
            col_fiscal = 6
            col_encargo_sigla = 4

            # GRUPO
            col_grupo_codificacao = col_contabil + col_fiscal

            # APURAÇÃO ICMS
            col_codigo_menu_fiscal = 3
            col_base_calculo = 12
            col_aliquota = 5
            col_imposto_creditado = 9

            # APURAÇÃO IPI
            col_ipi_cod = 3
            col_ipi_base_calculo = 12
            col_ipi_imposto_creditado = 9
            col_obs = 20

            # COLUNA DOS TOTAIS
            col_totais_name = col_data_entrada + col_grupo_doc_fiscais + 6
            col_totais_space1 = col_grupo_codificacao + col_codigo_menu_fiscal + 2
            col_totais_space2 = col_aliquota

        obj_adm_pessoa = pooler.get_pool(cr.dbname).get('adm.pessoa')
        is_simples_nacional = obj_adm_pessoa.verificar_empresa_simples_nacional(cr, uid)
        dados_nota = self.processar_dados_impressao(
            cr, uid, data_inicial, data_final, company_id, tipo_atividade, is_simples_nacional)
        keys = dados_nota.keys()
        resumo_estado = {}
        par_base_reduzida = self.db_name.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal.parametro', ['fispar_base_reduzida'], company_id=company_id,
            context=context).get('fispar_base_reduzida')
        converte_tz = pooler.get_pool(cr.dbname).get('adm.utils').admutl_converte_tz
        if keys:
            keys = sorted(keys, key=lambda k: int(k.replace('/', '')))
            total_valor_contabil = 0
            total_base_calculo = 0
            total_imposto_creditado = 0
            total_ipi_base_calculo = 0
            total_imposto_creditado_ipi = 0
            total_cod_icms_1 = 0
            total_cod_icms_2 = 0
            total_cod_icms_3 = 0
            total_cod_icms_4 = 0
            total_cod_ipi_1 = 0
            total_cod_ipi_2 = 0
            total_cod_ipi_3 = 0
            total_cod_ipi_4 = 0
            nf_ids = []
            for key in keys:
                nf_ids = filter(None, map(lambda i: i.get('nf_id') or False, dados_nota[key]))

            # DADOS DA GUIA DA NOTA FISCAL
            if nf_ids:
                dict_guia_dados = self.get_nf_guia(cr, nf_ids)
            else:
                dict_guia_dados = {}

            list_nf = []
            indice_geral = 0
            observacao_fisco = ''
            for key in keys:
                indice = {}
                valor_indice = 0
                numero = 0
                nfs_com_observacao_fisco = []
                for i in dados_nota[key]:
                    nf_id = i.get('nf_id')
                    if nf_id not in list_nf:
                        list_nf.append(nf_id)
                    if nf_id and numero != nf_id:
                        if self.modelo_relatorio == self.P1A:
                            listagem.imprimeLinha('|----------|----|-----|---------|--------'
                                                  '--|--------|----|------------|--------'
                                                  '|------|----|---|------------------|--'
                                                  '---|----------|-------------------|')
                        else:
                            listagem.imprimeLinha('|----------|---|----|-------|----------|--'
                                                  '----|---|------------|----|------|---|-'
                                                  '-----------|-----|---------|---|------'
                                                  '------|---------|-------------------'
                                                  '-|')
                        numero = nf_id
                    valot_total_icms_st = 0

                    # DADOS PRINCIPAIS
                    RELAT_DATA_ENTRADA = i.get('relat_data_entrada') or ''
                    RELAT_ESPECIE = i.get('relat_especie') or ''
                    RELAT_SERIE = i.get('relat_serie') or ''
                    RELAT_NUMERO = i.get('relat_numero') or ''
                    RELAT_DATA_DOCUMENTO = converte_tz(
                        cr, uid, parse(i.get('relat_data_documento')), 'UTC', 'America/Sao_Paulo',
                        {}).strftime('%d/%m/%Y') if i.get('relat_data_documento') else ''
                    RELAT_CODIGO_EMITENTE = i.get('relat_codigo_emitente') or ''
                    RELAT_UF = i.get('relat_uf') or '' #
                    RELAT_VALOR_CONTABIL = i.get('relat_valor_contabil') or 0
                    RELAT_CONTABIL = i.get('relat_contabil') or ''
                    RELAT_CFOP_CODIGO = i.get('relat_cfop_codigo') or ''
                    RELAT_ENCARGO_SIGLA = i.get('relat_encargo_sigla') or ''
                    relat_cod_fiscal = i.get('relat_cod_fiscal')
                    if is_simples_nacional and relat_cod_fiscal == 1:
                        relat_cod_fiscal = 4
                    RELAT_COD_FISCAL = relat_cod_fiscal or '' # ** COLUNA COD(*)
                    RELAT_VALOR_BASE_CALCULO = i.get('relat_base_calculo') or 0

                    # Notas de frete (Conhecimento de Transporte)
                    if RELAT_ESPECIE == 'T':
                        RELAT_VALOR_BASE_CALCULO = RELAT_VALOR_CONTABIL

                    RELAT_VALOR_ALIQUOTA = i.get('relat_aliquota') or 0
                    RELAT_VALOR_IMPOSTO_CREDITADO = i.get('relat_imposto_creditado') or 0
                    REALAT_OBS = i.get('relat_observacao') or ''
                    UF_RESUMO = i.get('uf_resumo') or ''
                    NF_ID = i.get('nf_id')
                    NF_STATE = i.get('nf_state')
                    TOTAL_REGISTROS_POR_NOTA = i.get('total_registros_por_nota') or 0
                    HASH_KEY = i.get('hash_key')

                    # Tributacao do item
                    ITEM_TRIBUT_VALOR_ICMS_ST = i.get('item_valor_icms_subst') or 0
                    ITEM_TRIBUT_BASE_ICMS_ST = i.get('item_base_icms_subst') or 0
                    ITEM_TRIBUT_VALOR_REDUCAO_BASE = i.get('item_valor_reducao_base') or 0

                    # Registro inserido manualmente
                    REGISTRO_MANUAL = i.get('registro_manual')

                    # DADOS DA GUIA DA NF
                    dict_nf_guia = dict_guia_dados.get(NF_ID)
                    if dict_nf_guia:
                        NF_GUIA_BASE = dict_nf_guia.get('nf_guia_base') or 0
                        NF_GUIA_VALOR = dict_nf_guia.get('nf_guia_valor') or 0
                    else:
                        NF_GUIA_BASE = 0
                        NF_GUIA_VALOR = 0

                    # Controle de indices
                    if HASH_KEY in indice:
                        indice[HASH_KEY] += 1
                    else:
                        indice[HASH_KEY] = 0
                    ultimo_registro = indice[HASH_KEY] + 1 == TOTAL_REGISTROS_POR_NOTA
                    next_index = valor_indice

                    # Valor contabil a ser utilizado
                    valor_contabil = RELAT_VALOR_CONTABIL

                    observacao = REALAT_OBS
                    valor_base_calculo = RELAT_VALOR_BASE_CALCULO + ITEM_TRIBUT_VALOR_ICMS_ST
                    codigo_fiscal = RELAT_COD_FISCAL
                    valor_imposto_creditado = RELAT_VALOR_IMPOSTO_CREDITADO
                    especie = RELAT_ESPECIE
                    aliquota = RELAT_VALOR_ALIQUOTA

                    if not REGISTRO_MANUAL:
                        if RELAT_ESPECIE == 'T':
                            especie = 'CTR'
                        elif RELAT_ESPECIE == 'N':
                            especie = 'NF'
                        else:
                            especie = ''

                    # Encargo ipi
                    valor_base_calculo_ipi = 0
                    if RELAT_ENCARGO_SIGLA == 'IPI':
                        if self.modelo_relatorio == self.P1A:
                            valor_base_calculo = valor_imposto_creditado
                            valor_imposto_creditado = 0
                            codigo_fiscal = 4
                            if not observacao:
                                observacao = '** IPI'
                        valor_base_calculo_ipi = valor_base_calculo
                    elif RELAT_ENCARGO_SIGLA == 'ICMS':
                        if codigo_fiscal != 1:
                            # Itens nao tributados nao agregam valor no imposto creditado
                            valor_imposto_creditado = 0
                        valot_total_icms_st += ITEM_TRIBUT_VALOR_ICMS_ST

                    # Reducao de base de calculo
                    if ITEM_TRIBUT_VALOR_REDUCAO_BASE:
                        if RELAT_COD_FISCAL == 1 or (RELAT_COD_FISCAL == 4 and is_simples_nacional):
                            next_index += 1
                            if par_base_reduzida == 'I':
                                reducao_codigo_fiscal = 2
                            else:
                                reducao_codigo_fiscal = 4

                            dados_nota[key].insert(next_index, {
                                'relat_encargo_sigla': RELAT_ENCARGO_SIGLA,
                                'relat_base_calculo': ITEM_TRIBUT_VALOR_REDUCAO_BASE,
                                'relat_observacao':  _('** VR REDUCAO BC'),
                                'uf_resumo': UF_RESUMO,
                                'relat_cod_fiscal': reducao_codigo_fiscal,
                            })

                    # Valor do icms st informado na capa da nota fiscal
                    if valot_total_icms_st and NF_STATE == '3':
                        next_index += 1
                        dados_nota[key].insert(next_index, {
                            'registro_manual': True,
                            'nf_id': nf_id,
                            'nova_linha': True
                        })

                    # ** ATENÇÃO **
                    # CRIAÇÃO DE UM NOVO REGISTRO PARA SER LISTADO NO RELATORIO.
                    if ultimo_registro and NF_STATE == '3':
                        # LANÇAMENTO DA GNRE NA COLUNA OBERSAÇÕES
                        if NF_GUIA_BASE or NF_GUIA_VALOR:
                            next_index += 1
                            dados_nota[key].insert(next_index, {
                                'relat_observacao': 'GNRE BC: %.2f' % NF_GUIA_BASE,
                            })
                            next_index += 1
                            dados_nota[key].insert(next_index, {
                                'relat_observacao': 'GNRE ICMS: %.2f' % NF_GUIA_VALOR,
                            })

                    if NF_STATE == 'C':  # Nota cancelada
                        observacao = 'Cancelada'
                        valor_contabil = 0
                        valor_base_calculo = 0
                        codigo_fiscal = ''
                        valor_imposto_creditado = 0
                    elif NF_STATE == 'I':  # Nota inutilizada
                        observacao = 'Inutilizada'

                    reg_ipi = ''
                    ipi_cod_fiscal = ''
                    ipi_imposto_creditado = 0
                    if self.modelo_relatorio == self.P1A:
                        # COLUNA NÃO EXISTE NO RELATORIO P1
                        str_relat_encargo_sigla = formataString(
                            RELAT_ENCARGO_SIGLA, col_encargo_sigla, ' ', 'E', 'S', 'S')
                        reg_encargo_sigla = '|' + str_relat_encargo_sigla
                    else:
                        reg_encargo_sigla = ''
                        if RELAT_ENCARGO_SIGLA == 'IPI':
                            ipi_cod_fiscal = codigo_fiscal
                            ipi_imposto_creditado = valor_imposto_creditado
                            valor_base_calculo_ipi = 0
                            valor_base_calculo = 0
                            valor_imposto_creditado = 0
                            aliquota = 0
                            codigo_fiscal = ''
                        str_relat_ipi_cod_fiscal = formataString(
                            ipi_cod_fiscal, col_ipi_cod, ' ', 'C', 'S', 'S')
                        str_relat_ipi_base_calculo = formataString(
                            '%.2f' % valor_base_calculo_ipi,
                            col_ipi_base_calculo, ' ', 'E', 'S', 'S')
                        str_relat_ipi_imposto_creditado = formataString(
                            '%.2f' % ipi_imposto_creditado,
                            col_ipi_imposto_creditado, ' ', 'E', 'S', 'S')
                        reg_ipi = '|' + str_relat_ipi_cod_fiscal + '|' + \
                                  str_relat_ipi_base_calculo + '|' + str_relat_ipi_imposto_creditado

                    str_valor_contabil = ''
                    # valor contabil pode ser uma string vazia quando for para nao aparecer
                    if valor_contabil:
                        str_valor_contabil = '%.2f' % valor_contabil
                    str_base_calculo = '%.2f' % valor_base_calculo
                    if not valor_base_calculo and i.get('nova_linha'):
                        str_base_calculo = ''
                    str_aliquota = '%.2f' % aliquota
                    if not aliquota and i.get('nova_linha'):
                        str_aliquota = ''
                    str_imposto_creditado = '%.2f' % valor_imposto_creditado
                    if not valor_imposto_creditado and i.get('nova_linha'):
                        str_imposto_creditado = ''
                    if i.get('relat_observacao_fisco') and nf_id not in nfs_com_observacao_fisco:
                        nfs_com_observacao_fisco.append(nf_id)
                        if (len(observacao) + len(i.get('relat_observacao_fisco'))) > 17:
                            observacao_fisco = i.get('relat_observacao_fisco')
                        else:
                            observacao += '. %s' % i.get('relat_observacao_fisco')
                    str_relat_data_entrada = formataString(
                        RELAT_DATA_ENTRADA, col_data_entrada, ' ', 'E', 'S', 'S')
                    str_relat_especie = formataString(especie, col_especie, ' ', 'E', 'S', 'S')
                    str_relat_serie = formataString(RELAT_SERIE, col_serie, ' ', 'E', 'S', 'S')
                    str_relat_numero = formataString(RELAT_NUMERO, col_numero, ' ', 'E', 'S', 'S')
                    str_relat_data_documento = formataString(
                        RELAT_DATA_DOCUMENTO, col_data_documento, ' ', 'E', 'S', 'S')
                    str_relat_codigo_emitente = formataString(
                        RELAT_CODIGO_EMITENTE, col_emitente, ' ', 'E', 'S', 'S')
                    str_relat_uf = formataString(RELAT_UF, col_uf, ' ', 'E', 'S', 'S')
                    str_relat_valor_contabil = formataString(
                        str_valor_contabil, col_valor_contabil, ' ', 'E', 'S', 'S')
                    str_relat_contabil = formataString(
                        RELAT_CONTABIL, col_contabil, ' ', 'E', 'S', 'S')
                    str_relat_cfop_codigo = formataString(
                        RELAT_CFOP_CODIGO, col_fiscal, ' ', 'E', 'S', 'S')
                    str_relat_cod_fiscal = formataString(
                        codigo_fiscal, col_codigo_menu_fiscal, ' ', 'C', 'S', 'S')
                    str_relat_base_calculo = formataString(
                        str_base_calculo, col_base_calculo, ' ', 'E', 'S', 'S')
                    str_relat_aliquota = formataString(
                        str_aliquota, col_aliquota, ' ', 'E', 'S', 'S')
                    str_relat_imposto_creditado = formataString\
                        (str_imposto_creditado, col_imposto_creditado, ' ', 'E', 'S', 'S')
                    cont_obs = 0
                    str_obs = []
                    if not observacao and observacao_fisco:
                        if observacao_fisco:
                            str_obs = observacao_fisco.split(' ')
                            observacao_fisco = ''

                        for i in str_obs:
                            if len(observacao)  + 1 + len(i) < 19:
                                cont_obs += 1
                                observacao += ' ' + i
                            else:
                                break

                    str_relat_obs = formataString(observacao, col_obs, ' ', 'D', 'S', 'S')
                    registro = '|' + str_relat_data_entrada +  '|' + str_relat_especie + \
                               '|' + str_relat_serie + '|' + str_relat_numero + \
                               '|' + str_relat_data_documento + '|' + str_relat_codigo_emitente + \
                               '|' + str_relat_uf + '|' + str_relat_valor_contabil + \
                               '|' + str_relat_contabil + '|' + str_relat_cfop_codigo + \
                                reg_encargo_sigla + '|' + str_relat_cod_fiscal + \
                               '|' + str_relat_base_calculo +  '|' + str_relat_aliquota + \
                               '|' + str_relat_imposto_creditado + reg_ipi + '|' + str_relat_obs + \
                               '|'

                    # IMPRESSÃO DAS NOTAS FISCAIS
                    if any(registro.replace('|', '').split(' ')):
                        listagem.imprimeLinha(registro)

                    if str_obs and len(str_obs) != cont_obs:
                        observacao = ''
                        for i in range(cont_obs, len(str_obs)):
                            if len(observacao) + 1 + len(str_obs[i]) < 20:
                                observacao += observacao and ' ' + str_obs[i] or '|' + str_obs[i]
                                observacao = self.remove_accents(observacao)
                            else:
                                if self.modelo_relatorio == self.P1A:
                                    linha = '{:<11}{:<5}{:<6}{:<10}{:<11}{:<9}{:<5}{:<13}{:<9}{:<7}{:<5}{:<4}{:<19}{:<6}{:<11}{:<20}|'.\
                                        format('|','|','|','|','|','|','|','|','|','|','|','|','|','|','|', observacao.encode('utf-8'))
                                else:
                                    linha = '{:<11}{:<4}{:<5}{:<8}{:<11}{:<7}{:<4}{:<13}{:<5}{:<7}{:<4}{:<13}{:<6}{:<10}{:<4}{:<13}{:<10}{:<20}|'.\
                                        format('|','|','|','|','|','|','|','|','|','|','|','|','|','|','|','|','|', observacao.encode('utf-8'))

                                listagem.imprimeLinha(linha)
                                observacao = '|' + str_obs[i]

                        if observacao:
                            if self.modelo_relatorio == self.P1A:
                                linha = '{:<11}{:<5}{:<6}{:<10}{:<11}{:<9}{:<5}{:<13}{:<9}{:<7}{:<5}{:<4}{:<19}{:<6}{:<11}{:<20}|'.\
                                    format('|','|','|','|','|','|','|','|','|','|','|','|','|','|','|', observacao.encode('utf-8'))
                            else:
                                linha = '{:<11}{:<4}{:<5}{:<8}{:<11}{:<7}{:<4}{:<13}{:<5}{:<7}{:<4}{:<13}{:<6}{:<10}{:<4}{:<13}{:<10}{:<20}|'.\
                                    format('|','|','|','|','|','|','|','|','|','|','|','|','|','|','|','|','|', observacao.encode('utf-8'))

                            listagem.imprimeLinha(linha)

                    # TOTAIS
                    total_valor_contabil += valor_contabil or 0
                    total_base_calculo += valor_base_calculo
                    total_imposto_creditado += valor_imposto_creditado
                    total_ipi_base_calculo += valor_base_calculo_ipi
                    total_imposto_creditado_ipi += ipi_imposto_creditado

                    if not UF_RESUMO in resumo_estado:
                        resumo_estado[UF_RESUMO] = {
                            'resumo_valor_contabil': 0,
                            'resumo_base_icms': 0,
                            'resumo_valor_icms': 0,
                            'resumo_base_ipi': 0,
                            'resumo_valor_ipi': 0,
                            'resumo_isento': 0,
                            'resumo_outros': 0,
                            'resumo_substituicao': 0
                        }
                    resumo_estado[UF_RESUMO]['resumo_valor_contabil'] += valor_contabil or 0

                    # TOTAIS DOS CODIGOS FISCAIS E RESUMO POR ESTADOS
                    if RELAT_ENCARGO_SIGLA == 'ICMS':
                        if is_simples_nacional:
                            if codigo_fiscal == 1:
                                codigo_fiscal = 4
                        if codigo_fiscal == 1:
                            # TOTAL DO ICMS CODIGO 1
                            total_cod_icms_1 += valor_base_calculo
                            # RESUMO POR ESTADO (ICMS)
                            resumo_estado[UF_RESUMO]['resumo_base_icms'] += valor_base_calculo
                            # RELAT_VALOR_IMPOSTO_CREDITADO
                            resumo_estado[UF_RESUMO]['resumo_valor_icms'] += \
                                RELAT_VALOR_IMPOSTO_CREDITADO
                        elif codigo_fiscal == 2:
                            total_cod_icms_2 += valor_base_calculo
                            resumo_estado[UF_RESUMO]['resumo_isento'] += valor_base_calculo

                        elif codigo_fiscal == 3:
                            total_cod_icms_3 += valor_base_calculo
                            resumo_estado[UF_RESUMO]['resumo_substituicao'] += valor_base_calculo

                        elif codigo_fiscal == 4:
                            total_cod_icms_4 += valor_base_calculo
                            resumo_estado[UF_RESUMO]['resumo_outros'] += (valor_base_calculo or
                                                                          valor_contabil)
                    else:
                        if self.modelo_relatorio == self.P1:
                            resumo_estado[UF_RESUMO]['resumo_base_ipi'] += valor_base_calculo_ipi
                            resumo_estado[UF_RESUMO]['resumo_valor_ipi'] += ipi_imposto_creditado
                        if codigo_fiscal == 1:
                            total_cod_ipi_1 += valor_base_calculo_ipi
                        elif codigo_fiscal == 2:
                            total_cod_ipi_2 += valor_base_calculo_ipi
                        elif codigo_fiscal == 3:
                            total_cod_ipi_3 += valor_base_calculo_ipi
                        elif codigo_fiscal == 4:
                            total_cod_ipi_4 += valor_base_calculo_ipi
                            if self.modelo_relatorio == self.P1A:
                                resumo_estado[UF_RESUMO]['resumo_outros'] += valor_base_calculo_ipi

                    valor_indice += 1
                    indice_geral += 1
                indice_geral += 1

            # ESPAÇOS ENTRE OS TOTAIS
            str_totais = formataString(_('TOTAIS:'), col_totais_name, ' ', 'D', 'S', 'S')
            str_totais_space1 = formataString('', col_totais_space1, ' ', 'E', 'S', 'S')
            str_totais_space2 = formataString('', col_totais_space2, ' ', 'E', 'S', 'S')
            str_totais_space3 = formataString('', col_obs, ' ', 'E', 'S', 'S')

            # VALORES TOTAIS
            str_total_valor_contabil = formataString(
                '%.2f' % total_valor_contabil, col_valor_contabil, ' ', 'E', 'S', 'S')
            str_total_base_calculo = formataString(
                '%.2f' % total_base_calculo, col_base_calculo, ' ', 'E', 'S', 'S')
            if total_imposto_creditado < 0:
                total_imposto_creditado = 0
            str_total_imposto_creditado = formataString(
                '%.2f' % total_imposto_creditado, col_imposto_creditado, ' ', 'E', 'S', 'S')

            if self.modelo_relatorio == self.P1:
                str_totais_space3 = formataString('', col_ipi_cod, ' ', 'E', 'S', 'S')
                str_total_ipi_base_calculo = formataString(
                    '%.2f' % total_ipi_base_calculo, col_ipi_base_calculo, ' ', 'E', 'S', 'S')
                str_total_ipi_imposto_creditado = formataString(
                    '%.2f' % total_imposto_creditado_ipi,
                    col_ipi_imposto_creditado + 1, ' ', 'E', 'S', 'S')
                str_totais_obs = formataString('', col_obs + 1, ' ', 'E', 'S', 'S')

                str_totais_relatorio_p1 = ' ' + str_total_ipi_base_calculo + \
                                          str_total_ipi_imposto_creditado + str_totais_obs
            else:
                str_totais_relatorio_p1 = ''

            # IMPRESSÃO DOS TOTAIS (VALOR CONTABIL, BASE CALCULO, IMPOSTO CREDITADO)
            listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
            listagem.imprimeLinha(
                '|' + str_totais + ' ' + str_total_valor_contabil + ' ' + str_totais_space1 +
                ' ' + str_total_base_calculo + ' ' + str_totais_space2 + ' ' +
                str_total_imposto_creditado + ' ' + str_totais_space3 + str_totais_relatorio_p1 +
                '|'
            )
            listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
            if self.modelo_relatorio == self.P1A:
                # TOTAIS DE ICMS E IPI NOS CODIGOS FISCAIS
                col_total_menu_fiscal_right_space = \
                    col_totais_name + col_valor_contabil + col_grupo_codificacao + 3
                col_total_menu_fiscal_left_space = \
                    col_aliquota + col_imposto_creditado + col_obs + 2
                totais_icms_ipi = []
                if total_cod_icms_1:
                    totais_icms_ipi.append(('ICMS 1', total_cod_icms_1))
                if total_cod_icms_2:
                    totais_icms_ipi.append(('ICMS 2', total_cod_icms_2))
                if total_cod_icms_3:
                    totais_icms_ipi.append(('ICMS 3', total_cod_icms_3))
                if total_cod_icms_4:
                    totais_icms_ipi.append(('ICMS 4', total_cod_icms_4))
                if total_cod_ipi_1:
                    totais_icms_ipi.append(('IPI  1', total_cod_ipi_1))
                if total_cod_ipi_2:
                    totais_icms_ipi.append(('IPI  2', total_cod_ipi_2))
                if total_cod_ipi_3:
                    totais_icms_ipi.append(('IPI  3', total_cod_ipi_3))
                if total_cod_ipi_4:
                    totais_icms_ipi.append(('IPI  4', total_cod_ipi_4))
                str_left_space = formataString(
                    '', col_total_menu_fiscal_left_space, ' ', 'E', 'S', 'S')
                str_right_space = formataString(
                    '', col_total_menu_fiscal_right_space, ' ', 'E', 'S', 'S')
                for i in totais_icms_ipi:
                    label = i[0]
                    value = i[1]
                    str_label = formataString(
                        label, col_encargo_sigla + col_codigo_menu_fiscal + 1, ' ', 'C', 'S', 'S')
                    str_value = formataString(
                        '%.2f' % value, col_base_calculo, ' ', 'E', 'S', 'S')
                    listagem.imprimeLinha(
                        '|' + str_right_space + ' ' + str_label + ' ' + str_value + ' ' +
                        str_left_space + '|')
                if totais_icms_ipi:
                    listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')

        return resumo_estado

report_registro_entrada('report.%s' % NOME_ARQUIVO_DEFAULT)


class wizard_registro_entrada(osv.TransientModel):
    def wreg_ent_adm_pagina_id(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = \
            self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'adm.paginas', context=context)
        obj_adm_pagina = self.pool.get('adm.paginas')
        adm_pagina_ids = obj_adm_pagina.search(
            cr, uid, [('company_id', '=', company_id), ('tipo', '=', 'registro_entrada')],
            context=context)
        if adm_pagina_ids:
            return max(adm_pagina_ids)
        else:
            return 0

    def wreg_ent_adm_pagina_dados(self, cr, uid, context=None):
        context = context and context.copy() or {}
        res = ''
        adm_pagina_id = self.wreg_ent_adm_pagina_id(cr, uid, context)
        if adm_pagina_id:
            if context.get('wreg_ent_folha'):
                field_to_read = 'final'
                nao_encontrado = 0
            else:
                field_to_read = 'data'
                nao_encontrado = ''
            adm_pagina_dados = self.pool.get('adm.paginas').read(
                cr, uid, adm_pagina_id, [field_to_read], context=context)
            if adm_pagina_dados and isinstance(adm_pagina_dados, list):
                adm_pagina_dados = adm_pagina_dados[0]
            if isinstance(adm_pagina_dados, dict):
                res = adm_pagina_dados.get(field_to_read) or nao_encontrado
        return res

    _name = 'wizard.registro.entrada'
    _description = 'Registro de Entradas'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa'),
        'wreg_ent_data_ini': fields.date('Data Inicio', required=True,
                                         help='Data inicial geracao relatorio'),
        'wreg_ent_data_fim': fields.date('Data Fim', required=True,
                                         help='Data final geracao relatorio'),
        'wreg_ent_folha': fields.integer('Folha', help='Informe a folha inicial')
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(
            cr, uid, uid).company_id.id,
        'wreg_ent_folha': lambda s, cr, uid, c: s.wreg_ent_adm_pagina_dados(
            cr, uid, {'wreg_ent_folha': True}),
        'wreg_ent_data_ini': lambda s, cr, uid, c: s.wreg_ent_adm_pagina_dados(
            cr, uid, {'wreg_ent_data_ini': True}),
    }

    def wreg_ent_convert_str_2_date(self, str_date, name_date):
        try:
            date_converted = parse(str_date, yearfirst=True).date()
            return date_converted
        except Exception:
            raise osv.except_osv('Data Invalida', 'A data %s informada e invalida!'%name_date)

    def wreg_ent_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        data = {}
        fields_to_read = [
            'company_id', 'wreg_ent_data_ini', 'wreg_ent_data_fim', 'wreg_ent_folha'
        ]
        wizard_entrada = self.read(cr, uid, ids, fields_to_read, context, '_classic_write')
        if isinstance(wizard_entrada, list):
            wizard_entrada = wizard_entrada[0]
        if isinstance(wizard_entrada, dict):
            folha = wizard_entrada.get('wreg_ent_folha') or 0
            data_inicial = wizard_entrada.get('wreg_ent_data_ini')
            data_final = wizard_entrada.get('wreg_ent_data_fim')
            company_id = wizard_entrada.get('company_id')
            date_inicial = self.wreg_ent_convert_str_2_date(data_inicial, name_date='inicial')
            date_final = self.wreg_ent_convert_str_2_date(data_final, name_date='final')
            if date_inicial > date_final:
                raise osv.except_osv(_('Datas Invalidas'),
                                     _('A data inicial e maior que a data final!'))
            elif not company_id:
                raise osv.except_osv(_('Atencao'),
                                     _('A empresa nao foi informada!'))
            else:
                browse_company = self.pool.get("res.company").browse(cr, uid, company_id)
                company_name = browse_company.name
                company_cnpj = browse_company.rescon_pessoa_id.cpf_cnpj
                company_inscricao_est = browse_company.rescon_pessoa_id.admpes_jur_ie
                company_tipo_atividade = browse_company.rescon_pessoa_id.admpes_jur_tipo_atividade

                # QUANDO RELATORIO É IMPRESSO, UTILIZA FOLHA + 1,
                # CASO O USUARIO INFORME UMA FOLHA IRÁ COMEÇAR COM A NUMERAÇÃO ESCOLHIDA PELO USUÁRIO.
                if folha:
                    folha -= 1

                data.update({
                    'company_id': company_id,
                    'company_name': company_name,
                    'company_cnpj': company_cnpj,
                    'company_tipo_atividade': company_tipo_atividade,
                    'company_inscricao_est': company_inscricao_est,
                    'data_inicial': date_inicial.strftime('%Y-%m-%d') + ' 00:00:00',
                    'data_final': date_final.strftime('%Y-%m-%d') + ' 23:59:59',
                    'folha': folha
                })

        return {'type': 'ir.actions.report.xml', 'report_name': NOME_ARQUIVO_DEFAULT, 'datas': data}

    def wreg_onchange_data_inicial(self, cr, uid, ids, data_inicial, context=None):
        context = context and context.copy() or {}
        pagina = self.pool.get('adm.paginas').admpag_get_pagina(
            cr, uid, data=data_inicial, tipo='registro_entrada', context=context)

        return {'value': {'wreg_ent_folha': pagina}}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
