# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.parser import parse
from datetime import date
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils import datas
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
from openerp import pooler
from openerp.tools.translate import _

# Variaveis globais
global_db_name = ''
global_listagem = ''
global_pdf_string = ''
global_resumo_estados_ecf = ''
global_ecf_instalado = False


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_fiscal_registro_saida(report_int):
    # Registro de saida (definicao de colunas)
    # Coluna documentos fiscais
    col_doc_fiscais = 26

    # Coluna codificacao
    col_codificao = 12

    # Colunas dentro de operacoes com/sem debito/imposto
    col_isentas = 14
    col_substituicao = 11
    col_outras = 11

    # Coluna observacao
    col_obs = 26

    # Colunas dentro de documentos fiscais
    col_especie = 5
    col_serie = 5
    col_numero = 7
    col_dia = 2
    col_uf = 3

    # Coluna valor contabil
    col_vr_contabil = 10

    # Colunas dentro de codificacao
    col_contabil = 6
    col_fiscal = 5

    # Colunas dentro de operacoes com debito imposto
    col_base_calculo = 11
    col_aliquota = 5
    col_imposto_deb = 10

    def define_cabecalho(self, listagem, data):
        self.str_modelo_registro = _('MODELO P2/A')
        str_titulo = formataString(_('REGISTRO DE SAIDA'), self.traco_linha -
                                   len(self.str_modelo_registro), ' ', 'C', 'N', 'S')
        str_company_name = formataString(_('EMPRESA: ') + data.get('company_name') or '',
                                         self.traco_linha, ' ', 'D', 'N', 'S')
        str_company_inscricao_est = \
            _('INSCRICAO ESTADUAL: ') + str(data.get('company_inscricao_est')) or ''
        str_company_ie = formataString(str_company_inscricao_est,
                                       len(str_company_inscricao_est), ' ', 'D', 'N', 'S')
        str_company_cnpj = formataString(_('          CNPJ: ') + data.get('company_cnpj') or '',
                                         self.traco_linha - len(str_company_ie), ' ', 'D', 'N', 'S')
        str_periodo = _('PERIODO DE: %s a %s') % (data.get('data_inicial_n') or '',
                                                  data.get('data_final_n') or '')
        str_periodo = formataString(str_periodo, self.traco_linha, ' ', 'D', 'N', 'S')
        str_folha = formataString(_('Folha: {{ Page }}'), self.traco_linha + 9, ' ', 'E', 'N', 'S')
        listagem._Impressora__pagina = self.folha
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab('|' + str_titulo + self.str_modelo_registro + '|')
        listagem.informaCab('|' + self.space + '|')
        listagem.informaCab('|' + str_company_name + '|')
        listagem.informaCab('|' + self.space + '|')
        listagem.informaCab('|' + str_company_ie + str_company_cnpj + '|')
        listagem.informaCab('|' + str_folha + '|')
        listagem.informaCab('|' + str_periodo + '|')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(
            _('|     DOCUMENTOS  FISCAIS  |          |CODIFICACAO |                    '
              ' VALORES FISCAIS                               |                          |'))
        listagem.informaCab('|--------------------------|          |------------|-----------------'
                            '--------------------------------------------------|                  '
                            '        |')
        listagem.informaCab(
            _('|     |SERIE|       | D|UF |  VALOR   |      |     |OPERACOES COM DEBITO IMPOSTO|  '
              ' OPERACOES S/DEBITO IMPOSTO         |                          |'))
        listagem.informaCab(
            _('|ESPE |SUB- | NUMERO| I|   |          |CONTA-|FIS- |----------------------------|--'
              '------------------------------------|                          |'))
        listagem.informaCab(
            _('|CIE  |SERIE|       | A|DE | CONTABIL |BIL   |CAL  |  BASE DE  |     | IMPOSTO  | '
              ' ISENTAS OU  | SUBSTIT.  |           |       OBSERVACOES        |'))
        listagem.informaCab(
            _('|     |     |       |  |ST |          |      |     |  CALCULO  |ALIQ.| DEBITADO '
              '|NAO TRIBUTADAS| TRIBUTARIA|   OUTRAS  |                          |'))
        listagem.informaCab('|-----|-----|-------|--|---|----------|------|-----|-----------|-----'
                            '|----------|--------------|-----------|-----------|------------------'
                            '--------|')
        listagem.informaCab('|     |     |       |  |   |          |      |     |           |     '
                            '|          |              |           |           |                  '
                            '        |')
        listagem.informaRodape('*-----------------------------------------------------------------'
                               '------------------------------------------------------------------'
                               '--------------*')

    def listagem_movimentos(self, cr, uid, listagem, data_inicial, data_final, totais_por_nota,
                            totais_por_cupons, movimentos, fiscal_parametro_dados, context):
        resumo_estados = {}
        total_valor_contabil = total_base_calculo = total_imposto_debitado = total_isentas = 0
        total_substituicao = total_outras = numero_nf = 0
        lista_dados = []
        lista_datas = []
        nf_ids = []
        tipo_tributacao = None
        obj_adm_pessoa = pooler.get_pool(cr.dbname).get('adm.pessoa')
        obj_ecf_cupom_capa = pooler.get_pool(cr.dbname).get('ecf.cupons.capa')
        calc_fis_nf_trib = \
            pooler.get_pool(cr.dbname).get('fiscal.nf.item.tributacao')._float_persist
        is_simples_nacional = obj_adm_pessoa.verificar_empresa_simples_nacional(cr, uid)
        resumo = []
        for i in movimentos:
            nf_id = i.get('nf_id')
            cfop = i.get('cfop')
            if cfop in ('5929', '6929') and nf_id in nf_ids:
                continue
            if nf_id not in nf_ids:
                nf_ids.append(nf_id)
            if i.get('state') == 'C':
                if numero_nf == i.get('numero_nf'):
                    continue
                else:
                    numero_nf = i.get('numero_nf')
            obs = ''
            tipo_tributacao = i.get('tipo_tributacao', '')
            valor_contabil = base_calculo = aliquota = imposto_debitado = isentas = substituicao = 0
            outras = 0
            if i.get('sigla') and i.get('sigla').upper() == 'IPI':
                outras = float(calc_fis_nf_trib('vr_encargo', i.get('vr_imposto')))
                if isinstance(totais_por_nota, list):
                    totais_por_nota[0]['outros'] += outras
                else:
                    totais_por_nota['outros'] += outras
                valor_contabil = i.get('vr_ipi') or 0
            elif i.get('state') == '3':  # Se a nota fiscal estiver faturada
                valor_contabil = (i.get('valor_contabil') or 0) + \
                                 (i.get('vr_icms_substituicao') or 0) + (i.get('vr_ipi') or 0)
                aliquota = i.get('aliquota', 0)
                if tipo_tributacao == 'T':
                    base_calculo = i.get('base_calculo', 0)
                    if base_calculo != valor_contabil:
                        if fiscal_parametro_dados.get('fispar_base_reduzida') != 'I':
                            outras = i.get('reducao_base') or 0
                            valor_contabil -= outras
                        else:
                            isentas += i.get('reducao_base') or 0
                    if is_simples_nacional:
                        outras += base_calculo
                        totais_por_nota[0]['outros'] += base_calculo
                        valor_contabil -= outras
                    imposto_debitado = i.get('vr_imposto') or 0
                elif tipo_tributacao == 'I':
                    isentas = i.get('valor_contabil') or 0
                    if fiscal_parametro_dados.get('fispar_base_reduzida') == 'I':
                        isentas += (i.get('reducao_base') or 0)
                elif tipo_tributacao == 'U':
                    substituicao = (i.get('valor_contabil') or 0) + \
                                   (i.get('vr_icms_substituicao') or 0)
                    if fiscal_parametro_dados.get('fispar_base_reduzida') != 'I':
                        outras = i.get('reducao_base') or 0
                        valor_contabil -= outras
                        substituicao -= outras
                    else:
                        isentas += i.get('reducao_base') or 0
                elif tipo_tributacao == 'TU':
                    substituicao += (i.get('base_calculo') or 0)
                    if fiscal_parametro_dados.get('fispar_base_reduzida') != 'I':
                        outras = i.get('reducao_base') or 0
                        valor_contabil -= outras
                    else:
                        isentas += i.get('reducao_base') or 0
                else:
                    outras = i.get('valor_contabil') or 0
                    valor_contabil -= outras

                valor_contabil += outras
                if is_simples_nacional:
                    aliquota = base_calculo = imposto_debitado = 0
            cfop = i.get('cfop')
            if cfop in ('5929', '6929'):
                valor_contabil = base_calculo = outras = isentas = imposto_debitado = 0
                aliquota = substituicao = 0
            if i.get('state') == 'I':  # Se a nota fiscal estiver inutilizada
                especie = formataString('NF', self.col_especie, ' ', 'E', 'S', 'S')
                serie = formataString(i.get('serie_nf', '') or i.get('subserie', ''),
                                      self.col_serie, ' ', 'C', 'S', 'S')
                numero = formataString(i.get('numero_nf'), self.col_numero, ' ', 'E', 'S', 'S')
                dia = formataString(str(parse(i.get('dia')).day) or '', self.col_dia, ' ', 'E', 'S',
                                    'S')
                uf = formataString(i.get('uf') or '', self.col_uf, ' ', 'E', 'S', 'S')
                str_valor_contabil = formataString('', self.col_vr_contabil, ' ', 'E', 'S', 'S')
                contabil = formataString('', self.col_contabil, ' ', 'E', 'S', 'S')
                fiscal = formataString('', self.col_fiscal, ' ', 'E', 'S', 'S')
                str_base_calculo = formataString('', self.col_base_calculo, ' ', 'E', 'S', 'S')
                str_aliquota = formataString('', self.col_aliquota, ' ', 'E', 'S', 'S')
                str_imposto_debitado = formataString('', self.col_imposto_deb, ' ', 'E', 'S', 'S')
                str_isentas = formataString('', self.col_isentas, ' ', 'E', 'S', 'S')
                str_substituicao = formataString('', self.col_substituicao, ' ', 'E', 'S', 'S')
                str_outras = formataString('', self.col_outras, ' ', 'E', 'S', 'S')
            else:
                especie = formataString('NF', self.col_especie, ' ', 'E', 'S', 'S')
                serie = formataString(i.get('serie_nf', '') or i.get('subserie_nf', ''),
                                      self.col_serie, ' ', 'C', 'S', 'S')
                numero = formataString(i.get('numero_nf', ''), self.col_numero, ' ', 'E', 'S', 'S')
                dia = formataString(str(parse(i.get('dia')).day) or '', self.col_dia, ' ', 'E', 'S',
                                    'S')
                uf = formataString(i.get('uf') or '', self.col_uf, ' ', 'E', 'S', 'S')
                str_valor_contabil = formataString('%.2f' % valor_contabil, self.col_vr_contabil,
                                                   ' ', 'E', 'S', 'S')
                contabil = formataString(i.get('contabil', ''), self.col_contabil, ' ', 'E', 'S',
                                         'S')
                fiscal = formataString(i.get('cfop') or '', self.col_fiscal, ' ', 'E', 'S', 'S')
                str_base_calculo = formataString('%.2f' % base_calculo, self.col_base_calculo, ' ',
                                                 'E', 'S', 'S')
                str_aliquota = formataString('%.2f' % aliquota, self.col_aliquota, ' ', 'E', 'S',
                                             'S')
                str_imposto_debitado = formataString('%.2f' % imposto_debitado,
                                                     self.col_imposto_deb, ' ', 'E', 'S', 'S')
                str_isentas = formataString('%.2f' % isentas, self.col_isentas, ' ', 'E', 'S', 'S')
                str_substituicao = formataString('%.2f' % substituicao, self.col_substituicao, ' ',
                                                 'E', 'S', 'S')
                str_outras = formataString('%.2f' % outras, self.col_outras, ' ', 'E', 'S', 'S')
            if i.get('state') == 'C':
                obs = '- Cancelada'
            elif i.get('state') == 'I':
                obs = '- Inutilizada'
            elif self.ecf_instalado and i.get('cfop') in ('5929', '6929'):
                cupom_ids = obj_ecf_cupom_capa.search(
                    cr, uid, [('nf_id', '=', i.get('nf_id'))], context=context)
                brw_ecf_cupom = obj_ecf_cupom_capa.browse(cr, uid, cupom_ids)
                count = len(brw_ecf_cupom)
                for cupom in brw_ecf_cupom:
                    obs += 'Cupom:%s ECF:%s' % (cupom.coo, cupom.ecf_id.caixa)
                    count -= 1
                    if count:
                        obs += ', '
            elif i.get('info_complementar'):
                vr_reducao_base = i.get('vr_base_icms_substituicao', 0)
                if any([vr_reducao_base, i.get('vr_icms_substituicao')]):
                    obs = '* B.Calc.Subst: %s, Val ou ICMS Subst: %s' % (
                    vr_reducao_base, i.get('vr_icms_substituicao', 0))
            if dia:
                dia = dia.replace('-', '')
                dia = int(dia)
            observacao_fisco = ''
            if i.get('observacao_fisco'):
                if (len(obs) + len(i.get('observacao_fisco'))) > 52:
                    observacao_fisco = i.get('observacao_fisco')
                else:
                    obs += '. %s' % i.get('observacao_fisco')
            lista_dados.append([{'especie': especie, 'serie': serie, 'numero': numero,
                                 'dia': dia, 'uf': uf, 'str_valor_contabil': str_valor_contabil,
                                 'contabil': contabil, 'fiscal': fiscal,
                                 'str_base_calculo': str_base_calculo,
                                 'str_aliquota': str_aliquota,
                                 'str_imposto_debitado': str_imposto_debitado,
                                 'str_isentas': str_isentas,
                                 'str_substituicao': str_substituicao, 'str_outras': str_outras,
                                 'obs': obs, 'obs_fisco': observacao_fisco}])
            if dia and not dia in lista_datas:
                lista_datas.append(dia)

            # Calcula os totais
            total_valor_contabil += round(valor_contabil, 2)
            total_base_calculo += base_calculo
            total_imposto_debitado += imposto_debitado
            total_isentas += isentas
            total_substituicao += substituicao
            total_outras += outras

            if not resumo_estados.get(uf):
                resumo_estados[uf] = {}.fromkeys(
                    ['resumo_valor_contabil', 'base_icms', 'valor_icms', 'base_ipi', 'valor_ipi',
                     'isento', 'outros', 'substituicao', 'icms_retido'], 0)
            resumo_estados[uf]['resumo_valor_contabil'] += valor_contabil
            if i.get('sigla') == 'ICMS':
                resumo_estados[uf]['base_icms'] += base_calculo
                resumo_estados[uf]['valor_icms'] += imposto_debitado
                if is_simples_nacional:
                    resumo_estados[uf]['outros'] += (i.get('base_calculo') or 0
                                                     if tipo_tributacao == 'T' else 0)
                if base_calculo != valor_contabil:
                    if fiscal_parametro_dados.get('fispar_base_reduzida') != 'I':
                        resumo_estados[uf]['outros'] += i.get('reducao_base') or 0
                    else:
                        resumo_estados[uf]['isento'] += i.get('reducao_base') or 0
            elif i.get('sigla') == 'IPI':
                resumo_estados[uf]['base_ipi'] += base_calculo
                resumo_estados[uf]['valor_ipi'] += i.get('vr_ipi') or 0
                resumo_estados[uf]['outros'] += outras
            if tipo_tributacao == 'I':
                resumo_estados[uf]['isento'] += isentas
            elif tipo_tributacao == 'O':
                resumo_estados[uf]['outros'] += outras
            elif tipo_tributacao in ('U', 'TU'):
                resumo_estados[uf]['substituicao'] += substituicao

            # O ICMS retido nao recebe nenhum valor pois ainda nao foi lancado no sistema
            resumo_estados[uf]['icms_retido'] += 0

        if resumo_estados:
            resumo.append(resumo_estados)

        # Tratamento valores cupons ECF
        # TODO: Transformar e metodo
        if self.ecf_instalado:
            obj_mapa_resumo_capa = self.db_name.get('ecf.mapa.resumo.capa')
            ecf_company_id = self.db_name.get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'ecf.ecf', context=context)
            ecf_ids = self.db_name.get('ecf.ecf').search(
                cr, uid, [('company_id', '=', ecf_company_id)], context=context)
            data_inicial = \
                date(int(data_inicial[6:10]), int(data_inicial[3:5]), int(data_inicial[0:2]))
            data_final = date(int(data_final[6:10]), int(data_final[3:5]), int(data_final[0:2]))
            ids_mapa_resumo_capa = obj_mapa_resumo_capa.search(
                cr, uid, [('data', '>=', data_inicial), ('data', '<=', data_final),
                          ('ecf', 'in', ecf_ids)], context=context)
            obj_mapa_resumo_itens = self.db_name.get('ecf.mapa.resumo.itens')
            ids_mapa_resumo_itens = obj_mapa_resumo_itens.search(
                cr, uid, [('capa_id', 'in', ids_mapa_resumo_capa)], context=context)
            self.resumo_estados_ecf = {}
            if ids_mapa_resumo_itens:
                id_vr_encargo = isentas = tributada = diferido = substituicao = outros = 0
                base_calc = iss = 0
                tributacao = obj_mapa_resumo_itens.read(
                    cr, uid, ids_mapa_resumo_itens, ['imposto', 'tipo', 'imposto_valor', 'aliquota',
                                                     'base_valor','base_reduzida'], context=context)
                a = []
                valor_ct = 0
                for y in tributacao:
                    vr_encargo = y.get('imposto_valor')
                    vr_base = y.get('base_valor')
                    id_vr_encargo += y.get('imposto_valor')
                    valor_ct += (y.get('base_valor', 0) + y.get('base_reduzida', 0))
                    if y.get('imposto') == 'C':
                        cst = y.get('tipo')
                        if cst == 'I':
                            isentas += vr_base
                        if cst == 'T':
                            tributada += vr_encargo
                        if cst in ('F', 'TU'):
                            substituicao += vr_base
                        if cst == 'O':
                            outros += vr_base
                        if cst == 'D':
                            diferido += vr_base
                        if y.get('aliquota') not in a:
                            a.append(y.get('aliquota'))
                        base_calc += y.get('base_valor')
                    elif y.get('imposto') == 'S':
                        iss += vr_encargo
                for r in ids_mapa_resumo_capa:
                    # Classe ´ecf.mapa.resumo.capa´
                    dados_mapa_resumo_capa = obj_mapa_resumo_capa.read(
                        cr, uid, [r], ['ecf', 'coo', 'gt_final', 'cancelamentos', 'numero_z',
                                       'data', 'cupom_inicial', 'cupom_final'], context=context)
                    if isinstance(dados_mapa_resumo_capa, list):
                        dados_mapa_resumo_capa = dados_mapa_resumo_capa[0]
                    uf = 'MG'
                    numero_cupom = dados_mapa_resumo_capa.get('coo') or ''
                    if numero_cupom:
                        id_cupom = self.db_name.get('ecf.cupons.capa').search(
                            cr, uid, [('coo', '=', numero_cupom)], context=context)
                        if id_cupom:
                            dados_ecf_cupons_capa = self.db_name.get('ecf.cupons.capa').read(
                                cr, uid, id_cupom, ['cliente_id'], context=context)
                            if dados_ecf_cupons_capa and isinstance(dados_ecf_cupons_capa, list):
                                dados_ecf_cupons_capa = dados_ecf_cupons_capa[0]
                            cliente_id = dados_ecf_cupons_capa.get('cliente_id')
                            if cliente_id:
                                dados_adm_pessoa = self.db_name.get('adm.pessoa').read(
                                    cr, uid, cliente_id[0], ['uf'], context=context)
                                if isinstance(dados_adm_pessoa, list):
                                    dados_adm_pessoa = dados_adm_pessoa[0]
                                uf = dados_adm_pessoa.get('uf') or 'MG'

                    # Classe ´ecf.ecf´
                    especie = formataString('ECF', self.col_especie, ' ', 'E', 'S', 'S')
                    serie = formataString('', self.col_serie, ' ', 'C', 'S', 'S')
                    numero = formataString(dados_mapa_resumo_capa.get('coo') or '',
                                           self.col_numero, ' ', 'E', 'S', 'S')
                    uf = formataString(uf or '', self.col_uf, ' ', 'E', 'S', 'S')
                    contabil = formataString('', self.col_contabil, ' ', 'E', 'S', 'S')
                    fiscal = formataString('', self.col_fiscal, ' ', 'E', 'S', 'S')
                    obs_red_z = ' GT:  %s' % dados_mapa_resumo_capa.get('gt_final')
                    obs = formataString(obs_red_z, self.col_obs, ' ', 'D', 'S', 'S')
                    dia = dados_mapa_resumo_capa.get('data')
                    if dia:
                        dia = dia.replace('-', '')
                        dia = int(dia)
                    ids_itens = obj_mapa_resumo_itens.search(
                        cr, uid, [('capa_id', '=', r)], context=context)
                    if ids_itens:
                        dados_mapa_resumo_itens = self.db_name.get('ecf.mapa.resumo.itens').read(
                            cr, uid, ids_itens, ['imposto_valor', 'base_valor', 'base_reduzida',
                                                 'aliquota', 'tipo'], context=context)
                        for item in dados_mapa_resumo_itens:
                            if item.get('base_valor'):
                                if item.get('tipo') == 'F':
                                    cfop = '5405'
                                else:
                                    cfop = '5102'
                                fiscal = formataString(cfop, self.col_fiscal, ' ', 'E', 'S', 'S')
                            str_imposto_debitado = formataString(' ', self.col_imposto_deb, ' ',
                                                                 'E', 'S', 'S')
                            str_valor_contabil = \
                                formataString('%.2f' % (item.get('base_valor') +
                                                        item.get('base_reduzida')),
                                              self.col_vr_contabil, ' ', 'E', 'S', 'S')
                            str_base_calculo = formataString('0.00', self.col_base_calculo,
                                                             ' ', 'E', 'S', 'S')
                            str_aliquota = formataString('%.2f' % item.get('aliquota'),
                                                         self.col_aliquota, ' ', 'E', 'S', 'S')
                            str_isentas = \
                                formataString('0.00', self.col_isentas, ' ', 'E', 'S', 'S')
                            str_outras = formataString('0.00', self.col_outras, ' ', 'E', 'S', 'S')
                            str_substituicao = \
                                formataString('0.00', self.col_substituicao, ' ', 'E', 'S', 'S')
                            if item.get('tipo') == "T":
                                str_base_calculo = \
                                    formataString('%.2f' % item.get('base_valor'),
                                                  self.col_base_calculo, ' ', 'E', 'S', 'S')
                                str_imposto_debitado = \
                                    formataString('%.2f' % item.get('imposto_valor'),
                                                  self.col_imposto_deb, ' ', 'E', 'S', 'S')
                            elif item.get('tipo') == "I":
                                str_isentas = \
                                    formataString('%.2f' % item.get('base_valor'),
                                                  self.col_isentas, ' ', 'E', 'S', 'S')
                            elif item.get('tipo') == "O":
                                str_outras = \
                                    formataString('%.2f' % item.get('base_valor'),
                                                  self.col_outras, ' ', 'E', 'S', 'S')
                            else:
                                str_substituicao = \
                                    formataString('%.2f' % item.get('base_valor'),
                                                  self.col_substituicao, ' ', 'E', 'S', 'S')
                            lista_dados.append([{
                                'especie': especie, 'serie': serie, 'numero': numero, 'dia': dia,
                                'uf': uf, 'str_valor_contabil': str_valor_contabil,
                                'contabil': contabil, 'fiscal': fiscal,
                                'str_base_calculo': str_base_calculo, 'str_aliquota': str_aliquota,
                                'str_imposto_debitado': str_imposto_debitado,
                                'str_isentas': str_isentas, 'str_substituicao': str_substituicao,
                                'str_outras': str_outras, 'obs': obs
                            }])
                    else:
                        str_valor_contabil = \
                            formataString('', self.col_vr_contabil, ' ', 'E', 'S', 'S')
                        str_base_calculo = \
                            formataString('', self.col_base_calculo, ' ', 'E', 'S', 'S')
                        str_imposto_debitado = \
                            formataString('', self.col_imposto_deb, ' ', 'E', 'S', 'S')
                        str_aliquota = formataString('', self.col_aliquota, ' ', 'E', 'S', 'S')
                        str_isentas = \
                            formataString('', self.col_isentas, ' ', 'E', 'S', 'S')
                        str_substituicao = \
                            formataString('', self.col_substituicao, ' ', 'E', 'S', 'S')
                        str_outras = formataString('', self.col_outras, ' ', 'E', 'S', 'S')
                        lista_dados.append([{
                            'especie': especie, 'serie': serie, 'numero': numero, 'dia': dia,
                            'uf': uf, 'str_valor_contabil': str_valor_contabil,
                            'contabil': contabil, 'fiscal': fiscal,
                            'str_base_calculo': str_base_calculo, 'str_aliquota': str_aliquota,
                            'str_imposto_debitado': str_imposto_debitado,
                            'str_isentas': str_isentas, 'str_substituicao': str_substituicao,
                            'str_outras': str_outras, 'obs': obs
                        }])
                    if dia and not dia in lista_datas:
                        lista_datas.append(dia)
                    ids_mapa_resumo_itens = self.db_name.get('ecf.mapa.resumo.itens').search(
                        cr, uid, [('capa_id', '=', r)], context=context)
                    if ids_mapa_resumo_itens:
                        for c in self.db_name.get('ecf.mapa.resumo.itens').browse(
                                cr, uid, ids_mapa_resumo_itens, context):
                            if c.imposto == 'C':
                                if not self.resumo_estados_ecf.get(uf):
                                    self.resumo_estados_ecf[uf] = {}.fromkeys(
                                        ['resumo_valor_contabil', 'base_icms', 'valor_icms',
                                         'isento', 'outros', 'substituicao', 'icms_retido', 'ecf'],
                                        0)
                                self.resumo_estados_ecf[uf]['ecf'] = True
                                self.resumo_estados_ecf[uf]['resumo_valor_contabil'] += \
                                    c.base_valor + c.base_reduzida
                                self.resumo_estados_ecf[uf]['base_icms'] += c.base_valor
                                self.resumo_estados_ecf[uf]['valor_icms'] += \
                                    c.base_valor * c.aliquota / 100.
                                if c.tipo == 'I':
                                    self.resumo_estados_ecf[uf]['isento'] += isentas
                                elif tipo_tributacao == 'O':
                                    self.resumo_estados_ecf[uf]['outros'] += outros
                                elif tipo_tributacao == 'F':
                                    self.resumo_estados_ecf[uf]['substituicao'] += substituicao
                                self.resumo_estados_ecf[uf]['icms_retido'] += 0

        lista_datas.sort()
        for data in lista_datas:
            for l in lista_dados:
                for dado in l:
                    if data == dado.get('dia'):
                        dia = dado.get('dia')
                        dia = str(dia)
                        marcador = dado.get('obs')
                        if len(marcador) > 26:
                            dado['obs'] = marcador[:26]
                        elif len(marcador) < 26:
                            dado['obs'] += (26 - len(marcador)) * ' '
                        dado['obs'] = \
                            formataString(dado.get('obs'), self.col_obs, ' ', 'D', 'S', 'S')
                        if len(dia) > 2:
                            dia = dia[6:]
                        elif len(dia) <2:
                            dia = '0' + dia
                        listagem.imprimeLinha(
                            '|' + dado.get('especie') + '|' + dado.get('serie') + '|' +
                            dado.get('numero') + '|' + formataString(dia, 2, ' ') + '|' +
                            dado.get('uf') + '|' + dado.get('str_valor_contabil') + '|' +
                            dado.get('contabil') + '|' + dado.get('fiscal') + '|' +
                            dado.get('str_base_calculo') + '|' + dado.get('str_aliquota') + '|' +
                            dado.get('str_imposto_debitado') + '|' + dado.get('str_isentas') + '|' +
                            dado.get('str_substituicao') + '|' + dado.get('str_outras') + '|' +
                            dado.get('obs') + '|')
                        linha_marcador = False
                        linha = ''
                        if len(marcador) > 26:
                            dado['obs'] = marcador[26:]
                            while len(dado.get('obs')):
                                linha = '|' + (self.col_especie * ' ') + '|' + (
                                    self.col_serie * ' ') + '|' + (self.col_numero * ' ') + '|' + (
                                            self.col_dia * ' ') + '|' + (self.col_uf * ' ') + \
                                        '|' + (self.col_vr_contabil * ' ') + '|' + (
                                            self.col_contabil * ' ') + '|' + (
                                            self.col_fiscal * ' ') + '|' + (
                                            self.col_base_calculo * ' ') + \
                                        '|' + (self.col_aliquota * ' ') + '|' + (
                                            self.col_imposto_deb * ' ') + '|' + (
                                            self.col_isentas * ' ') + \
                                        '|' + (self.col_substituicao * ' ') + '|' + (
                                            self.col_outras * ' ') + '|'
                                obs = formataString(dado.get('obs')[:26],
                                                    self.col_obs, ' ', 'D', 'S', 'S')
                                listagem.imprimeLinha(linha + obs + '|')
                                dado['obs'] = dado['obs'][26:]
                            linha_marcador = True
                        obs_fisco = dado.get('obs_fisco')
                        while obs_fisco:
                            listagem.imprimeLinha('|' + formataString(
                                obs_fisco[:145], 145, ' ', 'D', 'S', 'S') + '|')
                            obs_fisco = obs_fisco[145:]
                        if linha_marcador:
                            listagem.imprimeLinha(linha + 26 * ' ' + '|')

        # Formata os totais
        total_geral_contabil = total_geral_base_icms = total_geral_valor_icms = 0
        total_geral_base_ipi = total_geral_valor_ipi = total_geral_isentos = total_geral_outros = 0
        total_geral_subtituicao = total_geral_icms_retido = 0
        for r in resumo:
            for uf in r.keys() if isinstance(r, dict) else []:
                x = r.get(uf) or {}
                total_geral_contabil += x.get('resumo_valor_contabil') or 0
                total_geral_base_icms += x.get('base_icms') or 0
                total_geral_valor_icms += x.get('valor_icms') or 0
                total_geral_base_ipi += x.get('base_ipi') or 0
                total_geral_valor_ipi += x.get('valor_ipi') or 0
                total_geral_isentos += x.get('isento') or 0
                total_geral_outros += x.get('outros') or 0
                total_geral_subtituicao += (x.get('substituicao') or 0) + \
                                           (x.get('icms_retido') or 0)
                total_geral_icms_retido += x.get('icms_retido') or 0
        if self.ecf_instalado:
            for j in totais_por_cupons:
                total_geral_contabil += j.get('vr_contabil') or 0
                total_geral_base_icms += j.get('base_icms') or 0
                total_geral_valor_icms += j.get('vr_icms') or 0
                total_geral_base_ipi += 0
                total_geral_valor_ipi += j.get('vr_ipi') or 0
                total_geral_isentos += j.get('isentos') or 0
                total_geral_outros += j.get('outros') or 0
                total_geral_subtituicao += j.get('substituicao') or 0
                total_geral_icms_retido += j.get('icms_retido') or 0
        if is_simples_nacional:
            total_geral_base_icms = 0
            total_geral_valor_icms = 0

        str_total = formataString('TOTAL', self.col_doc_fiscais, ' ', 'C', 'S', 'S')
        str_total_valor_contabil = formataString('%.2f' % total_geral_contabil,
                                                 self.col_vr_contabil, ' ', 'E', 'S', 'S')
        str_total_base_calculo = formataString('%.2f' % total_geral_base_icms,
                                               self.col_base_calculo, ' ', 'E', 'S', 'S')
        str_total_imposto_debitado = formataString('%.2f' % total_geral_valor_icms,
                                                   self.col_imposto_deb, ' ', 'E', 'S', 'S')
        str_total_isentas = formataString('%.2f' % total_geral_isentos, self.col_isentas, ' ', 'E',
                                          'S', 'S')
        str_total_substituicao = formataString('%.2f' % total_geral_subtituicao,
                                               self.col_substituicao, ' ', 'E', 'S', 'S')
        str_total_outras = formataString('%.2f' % total_geral_outros, self.col_outras, ' ', 'E', 'S', 'S')

        # Imprime os totais NF
        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
        listagem.imprimeLinha('|' + str_total + '|' + str_total_valor_contabil + ' ' +
                              (' ' * self.col_codificao) + ' ' + str_total_base_calculo + ' ' +
                              (' ' * self.col_aliquota) + ' ' + str_total_imposto_debitado + ' ' +
                              str_total_isentas + ' ' + str_total_substituicao + ' ' +
                              str_total_outras + '|' + (' ' * self.col_obs) + '|')
        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')
        if resumo_estados and self.ecf_instalado:
            global global_resumo_estados_ecf
            global_resumo_estados_ecf = self.resumo_estados_ecf

        return resumo

    def create(self, cr, uid, ids, data, context=None):
        self.db_name = pooler.get_pool(cr.dbname)  # Permite acesso as tabelas
        self.traco_linha = 145
        self.space = ' ' * self.traco_linha
        self.str_modelo_registro = 'MODELO P2/A'  # Modelo do relatorio
        self.folha = data.get('folha') or 1  # Numero da folha a ser impresso
        self.ecf_instalado = data.get('ecf_instalado')

        obj_adm_utils = pooler.get_pool(cr.dbname).get('adm.utils')
        fields_fiscal_parametro = ['fispar_base_reduzida']
        fiscal_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'fiscal', fields_fiscal_parametro, context=context)

        context = context and context.copy() or {}
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.informaNome(pdf_string, _('Registro de Saida'))
        listagem.defineMargem(1, 1, 1, 1)
        listagem.informaTamLinha(150)
        self.define_cabecalho(listagem, data)
        totais_por_nota = data.get('totais_por_nota')
        totais_por_cupom = data.get('totais_por_cupom')
        tipo_atividade = data.get('company_tipo_atividade')
        movimento = data.get('movimento')
        resumo = self.listagem_movimentos(
            cr, uid, listagem, data.get('data_inicial_n'), data.get('data_final_n'), totais_por_nota,
            totais_por_cupom, movimento, fiscal_parametro_dados, context)

        # Resumo por estado (defincao de colunas)
        self.col_resumo_estado = 21
        self.col_resumo_contabil = 13
        self.col_resumo_base_icms = 14
        self.col_resumo_valor_icms = 11
        self.col_resumo_base_ipi = 12
        self.col_resumo_valor_ipi = 11
        self.col_resumo_isento = 13
        self.col_resumo_outros = 12
        self.col_resumo_subst = 12
        self.col_resumo_icms_retido = 17

        # Busca resumos
        data_inicial_i = data.get('data_inicial_i')
        data_final_i = data.get('data_final_i')
        totais_por_nota = data.get('totais_por_nota')
        totais_por_cupom = data.get('totais_por_cupom')

        # Impressao do relatorio (movimentos)
        listagem._Impressora__pagina += 1

        # Impressao do relatorio (resumo por estado)
        resumo_por_estado = self.get_resumo_por_estado(
            cr, data_inicial_i, data_final_i, data.get('company_id'))
        simples_nacional = pooler.get_pool(cr.dbname).get('adm.pessoa').\
            verificar_empresa_simples_nacional(cr, uid)
        self.resumo_por_estado(listagem, resumo, resumo_por_estado, totais_por_nota,
                               totais_por_cupom, simples_nacional, fiscal_parametro_dados,
                               tipo_atividade)

        # Atualiza numero da folha e data final da classe ´adm.paginas´
        data_inicial = data.get('data_inicial_i')[:10]
        self.db_name.get('adm.paginas').admpag_set_pagina(
            cr, uid, data_inicial=data_inicial, pag_final=listagem._Impressora__pagina - 1,
            tipo='registro_saida', context=context)

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return (self.obj.pdf, 'pdf')

    def get_resumo_por_estado(self, cr, data_inicio_i, data_fim_i, company_id):
        sql = """
                -- noinspection SqlDialectInspection
                SELECT sigla,
                       COALESCE(SUM(vr_contabil), 0)        AS vr_contabil,
                       COALESCE(SUM(base_icms), 0)          AS base_icms,
                       COALESCE(SUM(vr_icms), 0)            AS vr_icms,
                       COALESCE(SUM(vr_ipi), 0)             AS vr_ipi,
                       COALESCE(SUM(isentos), 0)            AS isentos,
                       COALESCE(SUM(outros), 0)             AS outros,
                       COALESCE(SUM(substituicao), 0)       AS substituicao,
                       COALESCE(SUM(icms_retido), 0)        AS icms_retido,
                       COALESCE(SUM(vr_reducao_base), 0)    AS vr_reducao_base
                FROM   (
                       -- NOTAS --
                       SELECT uf.sigla,
                              SUM(emi.vr_total_item_rateado +
                                  COALESCE(nf_item_trib.vr_icms_substituicao, 0)) 
                                                                      AS vr_contabil,
                              SUM(CASE
                                    WHEN cst.tipo = 'T' THEN nf_item_trib.vr_base_calculo
                                    ELSE 0
                                  END)                                AS base_icms,
                              SUM(CASE
                                    WHEN cst.tipo = 'T' THEN nf_item_trib.vr_encargo
                                    ELSE 0
                                  END)                                AS vr_icms,
                              0                                    AS vr_ipi,
                              SUM(CASE
                                    WHEN cst.tipo = 'I' THEN emi.vr_total_item_rateado
                                    ELSE 0
                                  END)                                AS isentos,
                              SUM(CASE
                                    WHEN cst.tipo = 'O' THEN emi.vr_total_item_rateado
		                            WHEN cst.tipo = 'T' THEN nf_item_trib.vr_reducao_base
                                    ELSE 0
                                  END)                                AS outros,
                              SUM(CASE
                                    WHEN cst.tipo = 'U'
                                        THEN emi.vr_total_item_rateado +
                                             COALESCE(nf_item_trib.vr_icms_substituicao, 0)
                                    WHEN cst.tipo = 'TU' THEN nf_item_trib.vr_base_calculo
                                    ELSE 0
                                  END)                                AS substituicao,
                              SUM(CASE WHEN cst.tipo IN ('T', 'TU', 'U')
                                  THEN nf_item_trib.vr_reducao_base
                                  ELSE 0 END)                         AS vr_reducao_base,
                              SUM(nf_item_trib.vr_icms_substituicao)  AS icms_retido,
                              SUM(emi.vr_desconto_rateado)            AS valor_desconto,
                              SUM(emi.vr_frete_rateado)               AS valor_frete,
                              SUM(emi.vr_seguro_rateado)              AS vr_seguro,
                              SUM(emi.vr_despesas_rateado)            AS vr_despesas_acessorias
                        FROM   fiscal_nf_capa                         AS nf_capa
                               INNER JOIN fiscal_modelo_nf_series     AS modelo_nf_series
                                       ON modelo_nf_series.id = nf_capa.modelo_nf_series_id
                               INNER JOIN fiscal_modelo_nf            AS fismod
                                       ON fismod.id = modelo_nf_series.modelo_nf_id
                               INNER JOIN adm_pessoa_endereco endereco
                                       ON endereco.id = nf_capa.destinatario_endereco_id
                               INNER JOIN adm_uf uf
                                       ON uf.id = endereco.uf_id
                               INNER JOIN estoque_movimento_itens emi
                                       ON emi.nf_id = nf_capa.id
                               INNER JOIN fiscal_nf_item_tributacao nf_item_trib
                                       ON nf_item_trib.movimento_itens_id = emi.id
                               INNER JOIN fiscal_encargo fe
                                       ON fe.id = nf_item_trib.encargo_id
                               INNER JOIN fiscal_cst cst
                                       ON cst.id = nf_item_trib.cst_id
                               INNER JOIN fiscal_cfop_natureza natureza
                                       ON natureza.id = nf_capa.natureza_id
                        WHERE  nf_capa.tipo = 'S'
                               AND nf_capa.state = '3'
                               AND nf_capa.data_emissao >= '%s'
                               AND nf_capa.data_emissao <= '%s'
                               AND fismod.fismnf_codigo < '91'
                               AND fe.sigla = 'ICMS'
                               AND natureza.cfop_id IN (SELECT id
                                                        FROM   fiscal_cfop
                                                        WHERE  codigo NOT IN ( '5929', '6929' ))
                               AND nf_capa.company_id = %s
                        GROUP  BY uf.sigla
        """ % (data_inicio_i, data_fim_i, company_id)
        if self.ecf_instalado:
            sql += """
                    UNION ALL
                        -- CUPONS --
                        SELECT 'MG'                       AS sigla,
                               SUM(mapa_itens.base_valor) AS vr_contabil,
                               SUM(CASE
                                     WHEN mapa_itens.tipo = 'T' THEN mapa_itens.base_valor
                                     ELSE 0
                                   END)                   AS base_icms,
                               SUM(CASE
                                     WHEN mapa_itens.tipo = 'T' THEN mapa_itens.imposto_valor
                                     ELSE 0
                                   END)                   AS vr_icms,
                               0                          AS vr_ipi,
                               SUM(CASE
                                     WHEN mapa_itens.tipo = 'I' THEN mapa_itens.base_valor
                                     ELSE 0
                                   END)                   AS isentos,
                               SUM(CASE
                                     WHEN mapa_itens.tipo = 'O' THEN mapa_itens.base_valor
                                     ELSE 0
                                   END)                   AS outros,
                               SUM(CASE
                                     WHEN mapa_itens.tipo IN ('F', 'TU') THEN mapa_itens.base_valor
                                     ELSE 0
                                   END)                   AS substituicao,
                               SUM(CASE WHEN mapa_itens.tipo IN ('T', 'U', 'TU')
                                   THEN mapa_itens.base_reduzida
                                   ELSE 0 END)            AS vr_reducao_base,
                               0                          AS icms_retido,
                               0                          AS valor_desconto,
                               0                          AS valor_frete,
                               0                          AS vr_seguro,
                               0                          AS vr_despesas_acessorias
                        FROM   ecf_mapa_resumo_capa mapa_capa
                               INNER JOIN ecf_mapa_resumo_itens mapa_itens
                                       ON mapa_itens.capa_id = mapa_capa.id
                                          AND mapa_capa.data >= '%s'
                                          AND mapa_capa.data <= '%s'
                               INNER JOIN ecf_cupons_capa cupons_capa
                                       ON cupons_capa.id = mapa_capa.ecfmrc_cupom_id
                                          AND cupons_capa.company_id = %s
        """ % (data_inicio_i, data_fim_i, company_id)
        sql += """
                    ) AS tmp
                GROUP  BY sigla
        """
        cr.execute(sql)
        dados = cr.dictfetchall()

        return dados

    def resumo_por_estado(self, listagem, resumos, resumo_por_estado, totais_por_nota,
                          totais_por_cupom, simples_nacional, fiscal_parametro_dados,
                          tipo_ativade='C'):
        total_base_icms_notas = total_valor_icms_notas = total_base_ipi_notas = 0
        total_valor_ipi_notas = total_isentos_notas = total_outros_notas = 0
        total_substituicao_notas = total_icms_retido_notas = 0
        listagem.cabecalhos = []  # Apaga o cabecalho

        # Novo cabecalho
        str_titulo_registro = formataString(_('*** REGISTRO DE SAIDAS ***'),
                                            self.traco_linha - len(self.str_modelo_registro), ' ',
                                            'C', 'N', 'S')
        str_resumo_estado = formataString(_('RESUMO POR ESTADO'),
                                          self.traco_linha, ' ', 'C', 'N', 'S')
        str_folha = formataString(_('Folha: ') + str(listagem._Impressora__pagina),
                                  self.traco_linha, ' ', 'E', 'N', 'S')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab('|' + str_titulo_registro + self.str_modelo_registro + '|')
        listagem.informaCab('|' + self.space + '|')
        listagem.informaCab('|' + str_folha + '|')
        listagem.informaCab('|' + str_resumo_estado + '|')
        listagem.informaCab('*' + ('-' * self.traco_linha) + '*')
        listagem.informaCab(
            _('| ESTADO              | VALORES     |   BASE DE    |   VALOR   |   BASE DE  |   '
              'VALOR   |   ISENTOS   |   OUTROS   |SUBSTITUICAO|      ICMS       |'))
        listagem.informaCab(
            _('|                     | CONTABEIS   |   C. ICMS    |   ICMS    |   C.IPI    |   '
              'I P I   |   TRIBUTAD  |            |            |     RETIDO      |'))
        listagem.informaCab(
            _('|---------------------|-------------|--------------|-----------|------------|-------'
              '----|-------------|------------|------------|-----------------|'))
        listagem.imprimeLinha('{{ Page }}')
        total_contabil_notas = 0

        # Formatacao
        for resumo in resumos:
            for i in resumo.keys():
                total_contabil_cupons = total_base_icms_cupons = total_valor_icms_cupons = 0
                total_isentos_cupons = total_outros_cupons = total_subtituicao_cupons = 0
                total_icms_retido_cupons = 0
                self.resumo_estados_ecf = global_resumo_estados_ecf or {}
                if resumo[i].get('ecf'):
                    for x in self.resumo_estados_ecf.keys():
                        if x == i:
                            total_contabil_notas += resumo[i].get('resumo_valor_contabil') - \
                                                    self.resumo_estados_ecf[x].get(
                                'resumo_valor_contabil')
                            total_contabil_cupons += \
                                self.resumo_estados_ecf[x].get('resumo_valor_contabil')
                            if resumo[i].get('base_icms') != 0 and not simples_nacional:
                                total_base_icms_notas += resumo[i].get('base_icms') - \
                                                         self.resumo_estados_ecf[x].get('base_icms')
                            else:
                                total_base_icms_notas += 0
                            total_base_icms_cupons += self.resumo_estados_ecf[x].get('base_icms')
                            if resumo[i].get('valor_icms') != 0 and not simples_nacional:
                                total_valor_icms_notas += \
                                    resumo[i].get('valor_icms') - \
                                    self.resumo_estados_ecf[x].get('valor_icms')
                            else:
                                total_valor_icms_notas += 0
                            total_valor_icms_cupons += self.resumo_estados_ecf[x].get('valor_icms')
                            total_base_ipi_notas += resumo[i].get('base_ipi', 0)
                            total_valor_ipi_notas += resumo[i].get('base_ipi', 0)
                            if resumo[i].get('isento') != 0:
                                total_isentos_notas += resumo[i].get('isento') - \
                                                       self.resumo_estados_ecf[x].get('isento')
                            else:
                                total_isentos_notas += 0
                            total_isentos_cupons += self.resumo_estados_ecf[x].get('isento')
                            if resumo[i].get('outros') != 0:
                                total_outros_notas += resumo[i].get('outros') - \
                                                      self.resumo_estados_ecf[x].get('outros')
                            else:
                                total_outros_notas += 0
                            if simples_nacional and resumo[i].get('base_icms') != 0:
                                total_outros_notas += resumo[i].get('base_icms') - \
                                                      self.resumo_estados_ecf[x].get('base_icms')
                            total_outros_cupons += self.resumo_estados_ecf[x].get('outros')
                            if resumo[i].get('substituicao') != 0:
                                total_substituicao_notas += \
                                    resumo[i].get('substituicao') - \
                                    self.resumo_estados_ecf[x].get('substituicao')
                            else:
                                total_substituicao_notas += 0
                            total_subtituicao_cupons += \
                                self.resumo_estados_ecf[x].get('substituicao')
                            total_icms_retido_notas += resumo[i].get('icms_retido') - \
                                                       self.resumo_estados_ecf[x].get('icms_retido')
                            total_icms_retido_cupons += \
                                self.resumo_estados_ecf[x].get('icms_retido')
                        else:
                            total_contabil_cupons += \
                                self.resumo_estados_ecf[x].get('resumo_valor_contabil')
                            total_base_icms_cupons += self.resumo_estados_ecf[x].get('base_icms')
                            total_valor_icms_cupons += self.resumo_estados_ecf[x].get('valor_icms')
                            total_isentos_cupons += self.resumo_estados_ecf[x].get('isento')
                            total_outros_cupons += self.resumo_estados_ecf[x].get('outros')
                            total_subtituicao_cupons += \
                                self.resumo_estados_ecf[x].get('substituicao')
                            total_icms_retido_cupons += \
                                self.resumo_estados_ecf[x].get('icms_retido')
                else:
                    total_contabil_notas += resumo[i].get('resumo_valor_contabil')
                    if not simples_nacional:
                        total_base_icms_notas += resumo[i].get('base_icms')
                        total_valor_icms_notas += resumo[i].get('valor_icms')
                    else:
                        total_outros_notas += resumo[i].get('base_icms')
                    total_base_ipi_notas += resumo[i].get('base_ipi', 0)
                    total_valor_ipi_notas += resumo[i].get('base_ipi', 0)
                    total_isentos_notas += resumo[i].get('isento')
                    total_outros_notas += resumo[i].get('outros')
                    total_substituicao_notas += resumo[i].get('substituicao')
                    total_icms_retido_notas += resumo[i].get('icms_retido')

        # Impressao do resumo por estado
        total_geral_contabil = total_geral_base_icms = total_geral_valor_icms = 0
        total_geral_base_ipi = total_geral_valor_ipi = total_geral_isentos = total_geral_outros = 0
        total_geral_substituicao = total_geral_icms_retido = 0
        for r in resumos:
            for uf in r.keys():
                x = r.get(uf)
                str_estado = \
                    formataString(uf, self.col_resumo_estado, ' ', 'D', 'N', 'S')
                str_resumo_contabil = formataString(
                    '%.2f' % x.get('resumo_valor_contabil'), self.col_resumo_contabil, ' ', 'E',
                    'N', 'S')
                total_geral_contabil += x.get('resumo_valor_contabil')
                str_base_icms = formataString(
                    '%.2f' % x.get('base_icms'), self.col_resumo_base_icms, ' ', 'E', 'N', 'S')
                total_geral_base_icms += x.get('base_icms')
                str_valor_icms = formataString(
                    '%.2f' % x.get('valor_icms'), self.col_resumo_valor_icms, ' ', 'E', 'N',
                    'S')
                total_geral_valor_icms += x.get('valor_icms')
                str_base_ipi = formataString(
                    '%.2f' % x.get('base_ipi'), self.col_resumo_base_ipi, ' ', 'E', 'N', 'S')
                total_geral_base_ipi += x.get('base_ipi')
                str_valor_ipi = formataString(
                    '%.2f' % x.get('valor_ipi'), self.col_resumo_valor_ipi, ' ', 'E', 'N',
                    'S')
                total_geral_valor_ipi += x.get('valor_ipi')
                str_resumo_isento = formataString(
                    '%.2f' % x.get('isento'), self.col_resumo_isento, ' ', 'E', 'N', 'S')
                total_geral_isentos += x.get('isento')
                str_resumo_outro = formataString(
                    '%.2f' % x.get('outros'), self.col_resumo_outros, ' ', 'E', 'N', 'S')
                total_geral_outros += x.get('outros')
                str_resumo_substituicao = formataString(
                    '%.2f' % x.get('substituicao'), self.col_resumo_subst, ' ', 'E', 'N', 'S')
                total_geral_substituicao += x.get('substituicao')
                str_icms_retido = formataString(
                    '%.2f' % x.get('icms_retido'), self.col_resumo_icms_retido, ' ', 'E', 'N',
                    'S')
                total_geral_icms_retido += x.get('icms_retido')
                listagem.imprimeLinha(
                    '|' + str_estado + '|' + str_resumo_contabil + '|' + str_base_icms + '|' +
                    str_valor_icms + '|' + str_base_ipi + '|' + str_valor_ipi + '|' +
                    str_resumo_isento + '|' + str_resumo_outro + '|' + str_resumo_substituicao +
                    '|' + str_icms_retido + '|')

                # Linha que separa os estados diferentes (fica dentro do grid dos estados)
                listagem.imprimeLinha(
                    '|---------------------|-------------|--------------|-----------|----------'
                    '--|-----------|-------------|------------|------------|-----------------|')

        # Formatacao do total das notas
        str_geral_contabil_notas = formataString(
            '%.2f' % total_geral_contabil, self.col_resumo_contabil, ' ', 'E', 'N', 'S')
        str_geral_base_icms_notas = formataString(
            '%.2f' % total_geral_base_icms, self.col_resumo_base_icms, ' ', 'E', 'N', 'S')
        str_geral_valor_icms_notas = formataString(
            '%.2f' % total_geral_valor_icms, self.col_resumo_valor_icms, ' ', 'E', 'N', 'S')
        str_geral_base_ipi_notas = formataString(
            '%.2f' % total_geral_base_ipi, self.col_resumo_base_ipi, ' ', 'E', 'N', 'S')
        str_geral_valor_ipi_notas = formataString(
            '%.2f' % total_geral_valor_ipi, self.col_resumo_valor_ipi, ' ', 'E', 'N', 'S')
        str_geral_isentos_notas = formataString(
            '%.2f' % total_geral_isentos, self.col_resumo_isento, ' ', 'E', 'N', 'S')
        str_geral_outros_notas = formataString(
            '%.2f' % total_geral_outros, self.col_resumo_outros, ' ', 'E', 'N', 'S')
        str_geral_subtituicao_notas = formataString(
            '%.2f' % total_geral_substituicao, self.col_resumo_subst, ' ', 'E', 'N', 'S')
        str_geral_icms_retido_notas = formataString(
            '%.2f' % total_geral_icms_retido, self.col_resumo_icms_retido, ' ', 'E', 'N', 'S')

        # Formatacao do total dos cupons
        str_geral_contabil_cupons = str_geral_base_icms_cupons = str_geral_valor_icms_cupons = 0
        str_geral_base_ipi_cupons = str_geral_valor_ipi_cupons = str_geral_isentos_cupons = 0
        str_geral_outros_cupons = str_geral_substituicao_cupons = str_geral_icms_retido_cupons = 0
        if self.ecf_instalado:
            for k in totais_por_cupom:
                total_geral_contabil += k.get('vr_contabil') or 0
                base_icms = k.get('base_icms') or 0
                vr_icms = k.get('vr_icms') or 0
                outros = k.get('outros') or 0
                if simples_nacional:
                    total_geral_outros += k.get('base_icms') or 0
                    outros += base_icms
                    base_icms = vr_icms = 0
                else:
                    total_geral_base_icms += k.get('base_icms') or 0
                    total_geral_valor_icms += k.get('vr_icms') or 0
                total_geral_base_ipi += k.get('vr_ipi') or 0
                total_geral_valor_ipi += k.get('vr_ipi') or 0
                total_geral_isentos += k.get('isentos') or 0
                total_geral_outros += k.get('outros') or 0
                total_geral_substituicao += k.get('substituicao') or 0
                total_geral_icms_retido += k.get('icms_retido') or 0

                str_geral_contabil_cupons = formataString(
                    '%.2f' % (k.get('vr_contabil') or 0), self.col_resumo_contabil, ' ', 'E', 'N',
                    'S')
                str_geral_base_icms_cupons = formataString(
                    '%.2f' % base_icms, self.col_resumo_base_icms, ' ', 'E', 'N', 'S')
                str_geral_valor_icms_cupons = formataString(
                    '%.2f' % vr_icms, self.col_resumo_valor_icms, ' ', 'E', 'N', 'S')
                str_geral_base_ipi_cupons = formataString(
                    '0.00', self.col_resumo_base_ipi, ' ', 'E', 'N', 'S')
                str_geral_valor_ipi_cupons = formataString(
                    '0.00', self.col_resumo_valor_ipi, ' ', 'E', 'N', 'S')
                str_geral_isentos_cupons = formataString(
                    '%.2f' % (k.get('isentos') or 0), self.col_resumo_isento, ' ', 'E', 'N', 'S')
                str_geral_outros_cupons = formataString(
                    '%.2f' % outros, self.col_resumo_outros, ' ', 'E', 'N', 'S')
                str_geral_substituicao_cupons = formataString(
                    '%.2f' % (k.get('substituicao') or 0), self.col_resumo_subst, ' ', 'E', 'N',
                    'S')
                str_geral_icms_retido_cupons = formataString(
                    '%.2f' % (k.get('icms_retido') or 0), self.col_resumo_icms_retido, ' ', 'E',
                    'N', 'S')
        if simples_nacional:
            total_geral_base_icms = total_geral_valor_icms = 0

        # Formatacao dos total geral
        str_geral_contabil = formataString(
            '%.2f' % total_geral_contabil, self.col_resumo_contabil, ' ', 'E', 'N', 'S')
        str_geral_base_icms = formataString(
            '%.2f' % total_geral_base_icms, self.col_resumo_base_icms, ' ', 'E', 'N', 'S')
        str_geral_valor_icms = formataString(
            '%.2f' % total_geral_valor_icms, self.col_resumo_valor_icms, ' ', 'E', 'N', 'S')
        str_geral_base_ipi =  formataString(
            '%.2f' % total_geral_base_ipi, self.col_resumo_base_ipi, ' ', 'E', 'N', 'S')
        str_geral_valor_ipi = formataString(
            '%.2f' % total_geral_valor_ipi, self.col_resumo_valor_ipi, ' ', 'E', 'N', 'S')
        str_geral_isentos = formataString(
            '%.2f' % total_geral_isentos, self.col_resumo_isento, ' ', 'E', 'N', 'S')
        str_geral_outros =  formataString(
            '%.2f' % total_geral_outros, self.col_resumo_outros, ' ', 'E', 'N', 'S')
        str_geral_subtituicao = formataString(
            '%.2f' % total_geral_substituicao, self.col_resumo_subst, ' ', 'E', 'N', 'S')
        str_geral_icms_retido = formataString(
            '%.2f' % total_geral_icms_retido, self.col_resumo_icms_retido, ' ', 'E', 'N', 'S')

        # Formatacao dos nomes dos totalizadores
        str_total_notas = formataString(
            _('TOTAL DE NOTAS  -->'), self.col_resumo_estado, ' ', 'D', 'N', 'S')
        str_total_cupons = formataString(
            _('TOTAL DE CUPONS -->'), self.col_resumo_estado, ' ', 'D', 'N', 'S')
        str_total_geral = formataString(
            _('TOTAL GERAL     -->'), self.col_resumo_estado, ' ', 'D', 'N', 'S')
        listagem.imprimeLinha(
            '|' + str_total_notas + '|' + str_geral_contabil_notas + '|' +
            str_geral_base_icms_notas + '|' + str_geral_valor_icms_notas + '|' +
            str_geral_base_ipi_notas + '|' + str_geral_valor_ipi_notas + '|' +
            str_geral_isentos_notas + '|' + str_geral_outros_notas + '|' +
            str_geral_subtituicao_notas + '|' + str_geral_icms_retido_notas + '|')
        if self.ecf_instalado:
            listagem.imprimeLinha(
                '|' + str_total_cupons + '|' + str_geral_contabil_cupons + '|' +
                str_geral_base_icms_cupons + '|' + str_geral_valor_icms_cupons + '|' +
                str_geral_base_ipi_cupons + '|' + str_geral_valor_ipi_cupons + '|' +
                str_geral_isentos_cupons + '|' + str_geral_outros_cupons + '|' +
                str_geral_substituicao_cupons + '|' + str_geral_icms_retido_cupons + '|')
        listagem.imprimeLinha(
            '|' + str_total_geral + '|' + str_geral_contabil + '|' + str_geral_base_icms + '|' +
            str_geral_valor_icms + '|' + str_geral_base_ipi + '|' + str_geral_valor_ipi + '|' +
            str_geral_isentos + '|' + str_geral_outros + '|' + str_geral_subtituicao + '|' +
            str_geral_icms_retido + '|')
        listagem.imprimeLinha('*' + ('-' * self.traco_linha) + '*')


report_fiscal_registro_saida('report.fiscal_registro_saida')


# noinspection PyAttributeOutsideInit
class wizard_registro_saida(osv.TransientModel):
    _name = 'wizard.registro.saida'
    _description = 'Registro de Saida'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa'),
        'data_ini': fields.date('Data Inicial', help='Data inicial geracao relatorio'),
        'data_fim': fields.date('Data Final', help='Data final geracao relatorio'),
        'folha': fields.integer('Folha', help='Informe a folha inicial')
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(
            cr, uid, uid).company_id.id
    }

    def onchange_data_inicial(self, cr, uid, ids, data_inicial, context=None):
        context = context and context.copy() or {}
        pagina = self.pool.get('adm.paginas').admpag_get_pagina(
            cr, uid, data=data_inicial, tipo='registro_saida', context=context)

        return {'value': {'folha': pagina}}

    @staticmethod
    def get_totais_nota(cr, data_inicio, data_fim, company_id):
        sql = """
-- noinspection SqlDialectInspection
SELECT vr_contabil,
       base_icms,
       vr_icms,
       vr_ipi,
       isentos,
       COALESCE(outros, 0) AS outros,
       substituicao,
       icms_retido,
       vr_reducao_base
FROM   (SELECT SUM(emi.vr_total_item_rateado) 
                                                                        AS vr_contabil,
               SUM(CASE WHEN cst.tipo = 'T' THEN nf_item_trib.vr_base_calculo
                   ELSE 0 END)                                          AS base_icms,
               SUM(CASE WHEN cst.tipo = 'T' THEN nf_item_trib.vr_encargo
                   ELSE 0 END)                                          AS vr_icms,
               SUM((SELECT vr_encargo
                    FROM   fiscal_nf_item_tributacao
                    WHERE  movimento_itens_id = emi.id
                           AND encargo_id = (SELECT id
                                             FROM   fiscal_encargo
                                             WHERE  sigla = 'IPI')))    AS vr_ipi,
               SUM(CASE WHEN cst.tipo = 'I' THEN emi.vr_total_item_rateado
                     ELSE 0 END)                                        AS isentos,
               SUM(CASE WHEN cst.tipo = 'O' THEN COALESCE(emi.vr_total_item_rateado)
                        WHEN cst.tipo = 'T' THEN COALESCE(nf_item_trib.vr_reducao_base, 0)
                   ELSE 0 END)                                          AS outros,
               SUM(CASE WHEN cst.tipo IN ('T', 'TU', 'U') THEN nf_item_trib.vr_reducao_base
                   ELSE 0 END)                                          AS vr_reducao_base,
               SUM(CASE
                       WHEN cst.tipo = 'U' THEN emi.vr_total_item_rateado
                       WHEN cst.tipo = 'TU' THEN nf_item_trib.vr_base_calculo
                   ELSE 0 END)                                          AS substituicao,
               SUM(nf_item_trib.vr_icms_substituicao)                   AS icms_retido,
               SUM(emi.vr_desconto_rateado)                             AS valor_desconto,
               SUM(emi.vr_frete_rateado)                                AS valor_frete,
               SUM(emi.vr_seguro_rateado)                               AS vr_seguro,
               SUM(emi.vr_despesas_rateado)                             AS vr_despesas_acessorias
       FROM    fiscal_nf_capa                                           AS nf_capa
       INNER JOIN fiscal_modelo_nf_series                               AS modelo_nf_series
               ON modelo_nf_series.id = nf_capa.modelo_nf_series_id
       INNER JOIN fiscal_modelo_nf                                      AS fismod
               ON fismod.id = modelo_nf_series.modelo_nf_id
       INNER JOIN adm_pessoa_endereco                                   AS endereco
               ON endereco.id = nf_capa.destinatario_endereco_id
       INNER JOIN adm_uf                                                AS uf
               ON uf.id = endereco.uf_id
       INNER JOIN estoque_movimento_itens                               AS emi
               ON emi.nf_id = nf_capa.id
       INNER JOIN fiscal_nf_item_tributacao                             AS nf_item_trib
               ON nf_item_trib.movimento_itens_id = emi.id
       INNER JOIN fiscal_encargo                                        AS fe
               ON fe.id = nf_item_trib.encargo_id
       INNER JOIN fiscal_cst                                            AS cst
               ON cst.id = nf_item_trib.cst_id
       INNER JOIN fiscal_cfop_natureza                                  AS natureza
               ON natureza.id = nf_capa.natureza_id
       WHERE  nf_capa.tipo = 'S'
         AND nf_capa.state = '3'
         AND nf_capa.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
         AND fismod.fismnf_codigo < '91'
         AND fe.sigla = 'ICMS'
         AND natureza.cfop_id IN (SELECT id
                                  FROM   fiscal_cfop
                                  WHERE  codigo NOT IN ( '5929', '6929' ))
         AND nf_capa.company_id = %s
       )                                                                AS TMP
        """ % (data_inicio, data_fim, company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()

        return dados

    @staticmethod
    def get_totais_cupons(cr, data_inicial_i, data_final_i, company_id):
        res = []
        sql = """
        -- noinspection SqlDialectInspection
        SELECT SUM(mapa_itens.base_valor) AS vr_contabil,
               SUM(CASE WHEN mapa_itens.tipo = 'T' THEN mapa_itens.base_valor
                   ELSE 0 END)                      AS base_icms,
               SUM(CASE WHEN mapa_itens.tipo = 'T' THEN mapa_itens.imposto_valor
                   ELSE 0 END)                      AS vr_icms,
               0                                    AS vr_ipi,
               SUM(CASE WHEN mapa_itens.tipo = 'I' THEN mapa_itens.base_valor
                   ELSE 0 END)                      AS isentos,
               SUM(CASE WHEN mapa_itens.tipo = 'O' THEN mapa_itens.base_valor
                   ELSE 0 END)                      AS outros,
               SUM(CASE WHEN mapa_itens.tipo IN ('F', 'TU') THEN mapa_itens.base_valor
                   ELSE 0 END)                      AS substituicao,
               SUM(CASE WHEN mapa_itens.tipo IN ('T', 'F') THEN mapa_itens.base_reduzida
                   ELSE 0 END)                      AS vr_reducao_base,
               0                                    AS icms_retido
        FROM   ecf_mapa_resumo_capa mapa_capa
        INNER JOIN ecf_mapa_resumo_itens            AS mapa_itens
                ON mapa_itens.capa_id = mapa_capa.id
               AND mapa_capa.data AT TIME ZONE 'UTC' >= '%s'
               AND mapa_capa.data AT TIME ZONE 'UTC' <= '%s'
        INNER JOIN ecf_ecf                          AS ecf
                ON ecf.id = mapa_capa.ecf
        WHERE  ecf.company_id = %s """ % (data_inicial_i, data_final_i, company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if dados:
            res = dados

        return res

    def get_movimentos(self, cr, uid, data_inicial, data_final, company_id, natureza_atividade,
                       context=None):
        res = []
        if data_inicial and data_final:
            """
            CST - 00, 10, 20, 70 (TRIBUTADOS)
            CST - 30, 40 (ISENTOS)
            CST - 60 (SUBSTITUICAO)
            CST - RESTANTE (OUTRAS)
            """
            sql = """
            -- noinspection SqlDialectInspection
            SELECT nf.id                                            AS nf_id,
                   nf.serie_nf                                      AS serie_nf,
                   nf.subserie                                      AS subserie_nf,
                   nf.numero_nf                                     AS numero_nf,
                   nf.modelo_nf_id                                  AS modelo_nf_id,
                   FORMAT('%s-%s-%s', EXTRACT( YEAR FROM nf.data_emissao AT TIME ZONE 'UTC'),
                                      EXTRACT( MONTH FROM nf.data_emissao AT TIME ZONE 'UTC'),
                                      EXTRACT( DAY FROM nf.data_emissao AT TIME ZONE 'UTC'))
                                                                    AS dia,
                   uf.sigla                                         AS uf,
                   CASE WHEN encargo.sigla = 'IPI' 
                        THEN SUM(nf_item_tribut.vr_encargo) 
                   ELSE 0 END                                       AS vr_ipi,
                   SUM(mov_itens.vr_total_item_rateado)
                                                                    AS valor_contabil,
                   SUM(mov_itens.vr_desconto_rateado)               AS valor_desconto,
                   SUM(mov_itens.vr_frete_rateado)                  AS valor_frete,
                   SUM(mov_itens.vr_seguro_rateado)                 AS vr_seguro,
                   SUM(mov_itens.vr_despesas_rateado)               AS vr_despesas_acessorias,
                   ''                                               AS contabil,
                   CASE WHEN nf.state = 'D' THEN cfop2.codigo ELSE cfop.codigo END
                                                                    AS cfop,
                   cfop.movimento_tipo                              AS cfop_movimento_tipo,
                   CASE WHEN cfop.codigo IN ('5929', '6929') THEN 0
                   ELSE SUM(nf_item_tribut.vr_base_calculo)  END    AS base_calculo,
                   nf.fisnfc_infcomp_fisco                          AS info_complementar,
                   SUM(nf_item_tribut.vr_icms_substituicao)         AS vr_icms_substituicao,
                   SUM(nf_item_tribut.vr_base_icms_substituicao)    AS vr_base_icms_substituicao,
                   cst.tipo                                         AS tipo_tributacao,
                   SUM(nf_item_tribut.vr_reducao_base)              AS reducao_base,
                   encargo.sigla                                    AS sigla,
                   nf_item_tribut.aliquota                          AS aliquota,
                   SUM(nf_item_tribut.vr_encargo)                   AS vr_imposto,
                   nf.state                                         AS state,
                   nf.nfe_chave_acesso,
                   nf.fisnfc_infcomp_fisco                          AS observacao_fisco,
                   modelo_nf_series.flag_modelo_nfe

            FROM   fiscal_nf_capa                                   AS nf
            LEFT JOIN  fiscal_modelo_nf_series                      AS modelo_nf_series
                   ON  modelo_nf_series.id = nf.modelo_nf_series_id
            LEFT JOIN  fiscal_modelo_nf                             AS fismod
                   ON  fismod.id = modelo_nf_series.modelo_nf_id
            LEFT JOIN  adm_pessoa_endereco                          AS endereco
                   ON  endereco.id = nf.destinatario_endereco_id
            LEFT JOIN  adm_uf                                       AS uf
                   ON  uf.id = endereco.uf_id
            LEFT JOIN  estoque_movimento_itens                      AS mov_itens
                   ON  mov_itens.nf_id = nf.id
            LEFT JOIN  fiscal_cfop                                  AS cfop
                   ON  cfop.id = mov_itens.cfop_id
            LEFT JOIN  fiscal_nf_item_tributacao                    AS nf_item_tribut
                   ON  nf_item_tribut.movimento_itens_id = mov_itens.id
            LEFT JOIN  fiscal_encargo                               AS encargo
                   ON  encargo.id = nf_item_tribut.encargo_id
            LEFT JOIN  fiscal_cst                                   AS cst
                   ON  cst.id = nf_item_tribut.cst_id
            LEFT JOIN  fiscal_cfop_natureza                         AS natureza
                   ON  natureza.id = nf.natureza_id
            LEFT JOIN  fiscal_cfop                                  AS cfop2
                   ON  natureza.cfop_id = cfop2.id 
            WHERE  nf.state IN ('3', 'C', 'I', 'D')
              AND  nf.tipo = 'S'
              AND  nf.data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
              AND  nf.company_id = {company_id}
              AND  fismod.fismnf_codigo < '91'
            GROUP  BY nf.id, nf.serie_nf, nf.subserie, nf.numero_nf, nf.modelo_nf_id,
                   nf.data_emissao, uf.sigla, cfop.codigo, cfop.movimento_tipo,
                   nf.fisnfc_infcomp_fisco, nf.data_emissao, cst.tipo, nf_item_tribut.aliquota,
                   nf.state, encargo.sigla, nf.nfe_chave_acesso, nf.fisnfc_infcomp_fisco,
                   modelo_nf_series.flag_modelo_nfe, cfop2.codigo
            HAVING (nf.state != 'C' AND encargo.sigla IN ('ICMS')) 
                OR nf.state = 'D'
                OR (nf.state = 'C'  AND (modelo_nf_series.flag_modelo_nfe = FALSE
                                     OR (modelo_nf_series.flag_modelo_nfe = TRUE
                                    AND nf.nfe_chave_acesso != '')))
            UNION ALL
            SELECT nf.id                                            AS nf_id,
                   nf.serie_nf                                      AS serie_nf,
                   nf.subserie                                      AS subserie_nf,
                   nf.numero_nf                                     AS numero_nf,
                   nf.modelo_nf_id                                  AS modelo_nf_id,
                   FORMAT('%s-%s-%s', EXTRACT( YEAR FROM nf.data_emissao AT TIME ZONE 'UTC'),
                                      EXTRACT( MONTH FROM nf.data_emissao AT TIME ZONE 'UTC'),
                                      EXTRACT( DAY FROM nf.data_emissao AT TIME ZONE 'UTC'))
                                                                    AS dia,
                   uf.sigla                                         AS uf,
                   CASE WHEN encargo.sigla = 'IPI' 
                        THEN SUM(nf_item_tribut.vr_encargo) 
                   ELSE 0 END                                       AS vr_ipi,
                   SUM(mov_itens.vr_total_item_rateado)
                                                                    AS valor_contabil,
                   SUM(mov_itens.vr_desconto_rateado)               AS valor_desconto,
                   SUM(mov_itens.vr_frete_rateado)                  AS valor_frete,
                   SUM(mov_itens.vr_seguro_rateado)                 AS vr_seguro,
                   SUM(mov_itens.vr_despesas_rateado)               AS vr_despesas_acessorias,
                   ''                                               AS contabil,
                   cfop.codigo                                      AS cfop,
                   cfop.movimento_tipo                              AS cfop_movimento_tipo,
                   CASE WHEN cfop.codigo IN ('5929', '6929') THEN 0
                   ELSE SUM(nf_item_tribut.vr_base_calculo)  END    AS base_calculo,
                   nf.fisnfc_infcomp_fisco                          AS info_complementar,
                   SUM(nf_item_tribut.vr_icms_substituicao)         AS vr_icms_substituicao,
                   SUM(nf_item_tribut.vr_base_icms_substituicao)    AS vr_base_icms_substituicao,
                   cst.tipo                                         AS tipo_tributacao,
                   SUM(nf_item_tribut.vr_reducao_base)              AS reducao_base,
                   encargo.sigla                                    AS sigla,
                   {vr_aliquota}                                    AS aliquota,
                   SUM(nf_item_tribut.vr_encargo)                   AS vr_imposto,
                   nf.state                                         AS state,
                   nf.nfe_chave_acesso,
                   nf.fisnfc_infcomp_fisco                          AS observacao_fisco,
                   modelo_nf_series.flag_modelo_nfe

            FROM   fiscal_nf_capa                                   AS nf
            LEFT JOIN  fiscal_modelo_nf_series                      AS modelo_nf_series
                   ON  modelo_nf_series.id = nf.modelo_nf_series_id
            LEFT JOIN  fiscal_modelo_nf                             AS fismod
                   ON  fismod.id = modelo_nf_series.modelo_nf_id
            LEFT JOIN  adm_pessoa_endereco                          AS endereco
                   ON  endereco.id = nf.destinatario_endereco_id
            LEFT JOIN  adm_uf                                       AS uf
                   ON  uf.id = endereco.uf_id
            LEFT JOIN  estoque_movimento_itens                      AS mov_itens
                   ON  mov_itens.nf_id = nf.id
            LEFT JOIN  fiscal_cfop                                  AS cfop
                   ON  cfop.id = mov_itens.cfop_id
            LEFT JOIN  fiscal_nf_item_tributacao                    AS nf_item_tribut
                   ON  nf_item_tribut.movimento_itens_id = mov_itens.id
            LEFT JOIN  fiscal_encargo                               AS encargo
                   ON  encargo.id = nf_item_tribut.encargo_id
            LEFT JOIN  fiscal_cst                                   AS cst
                   ON  cst.id = nf_item_tribut.cst_id
            WHERE  nf.state IN ('3', 'C', 'I', 'D')
              AND  nf.tipo = 'S'
              AND  nf.data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
              AND  nf.company_id = {company_id}
              AND  fismod.fismnf_codigo < '91'
            GROUP  BY nf.id, nf.serie_nf, nf.subserie, nf.numero_nf, nf.modelo_nf_id,
                   nf.data_emissao, uf.sigla, cfop.codigo, cfop.movimento_tipo,
                   nf.fisnfc_infcomp_fisco, nf.data_emissao, cst.tipo, {group_aliquota}
                   nf.state, encargo.sigla, nf.nfe_chave_acesso, nf.fisnfc_infcomp_fisco,
                   modelo_nf_series.flag_modelo_nfe
            HAVING (nf.state != 'C' AND encargo.sigla IN ('IPI'))
                OR (nf.state = 'C'  AND (modelo_nf_series.flag_modelo_nfe = FALSE
                                     OR (modelo_nf_series.flag_modelo_nfe = TRUE
                                    AND nf.nfe_chave_acesso != '')))
            ORDER  BY dia, serie_nf ASC, numero_nf, sigla;
            """.format(data_inicial=data_inicial, data_final=data_final, company_id=company_id,
                       vr_aliquota='nf_item_tribut.aliquota' if natureza_atividade == 'I' else 0,
                       group_aliquota='nf_item_tribut.aliquota,' if natureza_atividade == 'I'
                       else ''
                       )
            cr.execute(sql)
            res = cr.dictfetchall()
        obj_fiscal_modelo_nf = pooler.get_pool(cr.dbname).get('fiscal.modelo.nf')
        res = filter(lambda i: obj_fiscal_modelo_nf.verifica_codigo_modelo_fiscal(
            cr, uid, i.get('modelo_nf_id')), res)

        return res

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        ecf_instalado = obj_adm_utils.admutl_check_modulo_instalado(
            cr, uid, 'ecf', context=context)
        wizard_saida = self.read(cr, uid, ids[0] if isinstance(ids, list) else ids, [],
                                 context=context)
        if wizard_saida.get('data_ini') > wizard_saida.get('data_fim'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A data inicial nao poder ser maior que a data final.'))
        # try:
        #     data_inicial_nf = wizard_saida.get('data_ini') + ' 00:00:00'
        #     data_final_nf = wizard_saida.get('data_fim') + ' 23:59:59'
        # except:
        #     raise osv.except_osv(_('Atencao!'),
        #                          _('FISCAL - REGISTRO DE SAIDA: Ocorreu um erro ao converter '
        #                            'as datas informadas para o relatorio.'))

        data_inicial_i = parse(wizard_saida.get('data_ini'),
                             yearfirst=True).strftime('%Y/%m/%d') + ' 00:00:00'
        data_final_i = parse(wizard_saida.get('data_fim'),
                           yearfirst=True).strftime('%Y/%m/%d') + ' 23:59:59'
        data_inicial_n = parse(wizard_saida.get('data_ini'),
                             yearfirst=True).strftime('%d/%m/%Y') + ' 00:00:00'
        data_final_n = parse(wizard_saida.get('data_fim'),
                           yearfirst=True).strftime('%d/%m/%Y') + ' 23:59:59'

        company_id = wizard_saida.get('company_id')
        if isinstance(company_id, tuple):
            company_id = company_id[0]
        if not company_id:
            raise osv.except_osv(_('Atencao'), _('A empresa nao foi informada!'))
        browse_company = self.pool.get("res.company").browse(cr, uid, company_id)
        company_name = browse_company.name
        company_cnpj = browse_company.rescon_pessoa_id.cpf_cnpj
        company_inscricao_est = browse_company.rescon_pessoa_id.admpes_jur_ie
        company_tipo_atividade = browse_company.rescon_pessoa_id.admpes_jur_tipo_atividade
        data = {'company_id': company_id,
                'company_name': company_name, 'company_cnpj': company_cnpj,
                'company_inscricao_est': company_inscricao_est,
                'data_inicial_i': data_inicial_i, 'data_final_i': data_final_i, 'folha': 0,
                'data_inicial_n': data_inicial_n, 'data_final_n': data_final_n,
                'ecf_instalado': ecf_instalado,
                'company_tipo_atividade': company_tipo_atividade}
        sql = """
              -- noinspection SqlDialectInspection
              SELECT id
              FROM   fiscal_nf_capa
              WHERE  data_emissao AT TIME ZONE 'UTC' BETWEEN '{0:s}' AND '{1:s}'
                AND  state IN ( '3', 'C', 'I' )
                AND  tipo = 'S'
              """.format(data_inicial_i, data_final_i)
        if ecf_instalado:
            sql += """
              UNION

              SELECT id
              FROM ecf_cupons_capa
              WHERE date_cupom AT TIME ZONE 'UTC' >= '{0:s}'
                AND date_cupom AT TIME ZONE 'UTC' <= '{1:s}'
            """.format(data_inicial_i, data_final_i)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum registro encontrado para este periodo.'))

        # Impressao do relatorio (cabecalho)
        totais_por_nota = self.get_totais_nota(cr, data_inicial_i, data_final_i, company_id)
        totais_por_cupom = self.get_totais_cupons(
            cr, data_inicial_i, data_final_i, company_id) if ecf_instalado else None
        data['totais_por_nota'] = totais_por_nota
        data['totais_por_cupom'] = totais_por_cupom
        data['movimento'] = self.get_movimentos(
            cr, uid, data_inicial_i, data_final_i, company_id, company_tipo_atividade,
            context=None)

        return {'type': 'ir.actions.report.xml', 'report_name': 'fiscal_registro_saida',
                'datas': data}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
