#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from geraldo import Report, ReportBand, SystemField, Label, ObjectValue, DetailBand
from geraldo.utils import cm, TA_CENTER, TA_RIGHT, TA_LEFT
from geraldo.generators import PDFGenerator
from openerp.report.render import render
import StringIO
from openerp.report.interface import report_int
from SigUtils.numero import getNumeroEditado
from dateutil.parser import parse

#Variaveis globais

EMPRESA = ''
PERIODO = ''


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        dados_entrada_por_uf = datas.get('dados_entrada_por_uf')
        dados_saida_por_uf = datas.get('dados_saida_por_uf')
        dados_entrada_por_cfop = datas.get('dados_entrada_por_cfop')
        dados_saida_por_cfop = datas.get('dados_saida_por_cfop')
        rel_entrada_por_uf = EntradaPorUf(queryset=dados_entrada_por_uf)
        rel_saida_por_uf = SaidaPorUf(queryset=dados_saida_por_uf)
        rel_entrada_por_cfop = EntradaPorCfop(queryset=dados_entrada_por_cfop)
        rel_saida_por_cfop = SaidaPorCfop(queryset=dados_saida_por_cfop)
        arquivo = StringIO.StringIO()
        canvas_ent_por_uf = rel_entrada_por_uf.generate_by(PDFGenerator, filename=arquivo,
                                                           return_canvas=True)
        canvas_sai_por_uf = rel_saida_por_uf.generate_by(PDFGenerator, canvas=canvas_ent_por_uf,
                                                         return_canvas=True)
        canvas_ent_por_cfop = rel_entrada_por_cfop.generate_by(PDFGenerator,
                                                               canvas=canvas_sai_por_uf,
                                                               return_canvas=True)
        rel_saida_por_cfop.generate_by(PDFGenerator, canvas=canvas_ent_por_cfop)
        self.obj = external_pdf(arquivo.getvalue())
        self.obj.render()
        arquivo.close()
        return (self.obj.pdf, 'pdf')

report_custom('report.damef')


class wizard_resumo_damef(osv.TransientModel):
    _name = 'wizard.resumo.damef'
    _columns = {
        'wizresdam_data_inicio': fields.date("Data inicio", required=True),
        'wizresdam_data_fim': fields.date("Data final", required=True),
    }

    def button_gerar_damef(self, cr, uid, ids, context):
        global EMPRESA
        global PERIODO
        context = context and context.copy() or {}
        if ids and isinstance(ids, list):
            ids = ids[0]
        dados = self.read(cr, uid, ids, [], context=context)
        dt_ini = dados.get('wizresdam_data_inicio')
        dt_fim = dados.get('wizresdam_data_fim')
        periodo = "Periodo: {data_ini} - {data_fim}".\
            format(data_ini=parse(dt_ini).strftime('%d/%m/%Y'),
                   data_fim=parse(dt_fim).strftime('%d/%m/%Y'))
        PERIODO = periodo
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self, context=context)
        rescon_pessoa = self.pool.get('res.company').read(cr, uid, company_id,
                                                          ['rescon_pessoa_id'], context=context,
                                                          load='_classic_read')
        if isinstance(rescon_pessoa.get('rescon_pessoa_id'), tuple):
            EMPRESA = rescon_pessoa.get('rescon_pessoa_id')[1]
        dt_fim += ' 23:59:59'

        sql_modelos_ids = """
            SELECT id
            FROM   fiscal_modelo_nf
            WHERE  fismnf_codigo LIKE '9%'
        """
        cr.execute(sql_modelos_ids)
        dados_modelos = cr.dictfetchall()
        modelo_nf_ids = map(lambda x: x.get('id'), dados_modelos)
        if not modelo_nf_ids:
            modelo_nf_ids = [-1]

        entrada_por_uf = self.get_operacoes_entrada_por_uf(cr, dt_ini, dt_fim, company_id,
                                                           modelo_nf_ids)
        saida_por_uf = self.get_operacoes_saida_por_uf(cr, uid, dt_ini, dt_fim, company_id,
                                                       modelo_nf_ids)
        entrada_por_cfop = self.get_operacoes_entrada_por_cfop(cr, uid, dt_ini, dt_fim, company_id,
                                                               modelo_nf_ids)
        saida_por_cfop = self.get_operacoes_saida_por_cfop(cr, uid, dt_ini, dt_fim, company_id,
                                                           modelo_nf_ids)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'damef',
            'datas': {'dados_entrada_por_uf': entrada_por_uf,
                      'dados_saida_por_uf': saida_por_uf,
                      'dados_entrada_por_cfop': entrada_por_cfop,
                      'dados_saida_por_cfop': saida_por_cfop}
        }

    @staticmethod
    def get_operacoes_entrada_por_uf(cr, data_inicio, data_fim, company_id, modelo_nf_ids):
        sql = """
                SELECT  uf                          AS uf,
                       SUM(vr_contabil) + SUM(ipi) AS vr_contabil,
                       SUM(vr_base_calculo)        AS vr_base_calculo,
                       SUM(isento)                 AS isento,
                       SUM(substituicao)           AS substituicao,
                       SUM(outros)                 AS outros,
                       SUM(ipi)                    AS ipi
                FROM   (SELECT uf.sigla
                               || '-'
                               || uf.name AS uf,
                               CASE
                                 WHEN fe.sigla = 'ICMS' THEN
                                 SUM(mov_itens.vr_total_item_rateado)
                                 + SUM(it_trib.vr_icms_substituicao)
                                 ELSE 0.00
                               END        AS vr_contabil,
                               CASE
                                 WHEN cst.tipo = 'T'
                                      AND fe.sigla = 'ICMS' THEN SUM(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END        AS vr_base_calculo,
                               CASE
                                 WHEN cst.tipo = 'I'
                                      AND fe.sigla = 'ICMS' THEN SUM(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END        AS isento,
                               CASE
                                 WHEN cst.tipo IN ('U', 'TU')
                                      AND fe.sigla = 'ICMS' THEN
                                 SUM(it_trib.vr_base_calculo)
                                 + SUM(it_trib.vr_icms_substituicao)
                                 + SUM(vr_reducao_base)
                                 ELSE 0.00
                               END        AS substituicao,
                               CASE
                                 WHEN cst.tipo IN ('O', 'D', 'S') AND fe.sigla = 'ICMS'
                                     THEN SUM(it_trib.vr_base_calculo)
                                 WHEN fe.sigla = 'IPI'
                                     THEN SUM(it_trib.vr_encargo)
                                 WHEN cst.tipo = 'T' AND fe.sigla = 'ICMS'
                                     THEN SUM(it_trib.vr_reducao_base)
                                 ELSE 0.00
                               END        AS outros,
                               CASE
                                 WHEN fe.sigla = 'IPI' THEN SUM(it_trib.vr_encargo)
                                 ELSE 0.00
                               END        AS ipi
                        FROM   estoque_movimento_itens mov_itens
                               inner join fiscal_nf_capa nf_capa
                                       ON nf_capa.id = mov_itens.nf_id
                               inner join fiscal_nf_item_tributacao it_trib
                                       ON it_trib.movimento_itens_id = mov_itens.id
                               inner join adm_pessoa_endereco endereco
                                       ON endereco.id = nf_capa.destinatario_endereco_id
                               inner join adm_uf uf
                                       ON uf.id = endereco.uf_id
                               inner join fiscal_encargo fe
                                       ON fe.id = it_trib.encargo_id
                                          AND fe.sigla IN ( 'ICMS', 'IPI' )
                               inner join fiscal_cst cst
                                       ON cst.id = it_trib.cst_id
                        WHERE  nf_capa.tipo = 'E'
                               AND nf_capa.data_saida_entrada BETWEEN
                                   '{data_inicio}' AND '{data_fim}'
                               AND nf_capa.state = '3'
                               AND nf_capa.company_id = {company_id}
                               AND mov_itens.cfop_id NOT IN (SELECT id FROM fiscal_cfop
                                                             WHERE produto_servico_tipo = 'F')
                               AND nf_capa.modelo_nf_id NOT IN ({modelo_nf_ids})
                        GROUP  BY uf.sigla,
                                  uf.name,
                                  cst.tipo,
                                  fe.sigla

                        UNION ALL

                        SELECT
                               uf.sigla || '-' || uf.name AS uf,
                               Sum(COALESCE(nf_capa.vr_total_nf, 0.0))             AS vr_contabil,
                               Sum(CASE
                                     WHEN tributacao.tipo = 'T'
                                     THEN COALESCE(tributacao.vr_base_calculo, 0.0)
                                     ELSE 0.00
                                   END)                                            AS vr_base_calculo,
                               Sum(CASE
                                     WHEN tributacao.tipo = 'I' THEN
                                     COALESCE(tributacao.vr_base_calculo, 0.0)
                                     ELSE 0
                                   END)                                            AS isento,
                               Sum(CASE
                                     WHEN tributacao.tipo = 'U'
                                     THEN COALESCE(tributacao.vr_base_calculo, 0.0) +
                                          COALESCE(tributacao.vr_icms_substituicao, 0.0) +
                                          COALESCE(tributacao.vr_reducao_base, 0.0)
                                     ELSE 0
                                   END)                                            AS substituicao,
                               Sum(CASE
                                     WHEN tributacao.tipo = 'O'
                                     THEN COALESCE(tributacao.vr_base_calculo, 0.0)
                                     ELSE 0
                                   END)                                            AS outros,
                               CASE
                                 WHEN encargo.sigla = 'IPI' THEN SUM(tributacao.vr_encargo)
                                 ELSE 0.00
                               END                                                 AS ipi
                        FROM   fiscal_nf_capa nf_capa
                               INNER JOIN adm_pessoa_endereco endereco
                                       ON endereco.id = nf_capa.destinatario_endereco_id
                               INNER JOIN adm_uf uf
                                       ON uf.id = endereco.uf_id
                               INNER JOIN fiscal_cfop cfop
                                       ON cfop.id = (SELECT cfop_id
                                                     FROM   fiscal_cfop_natureza
                                                     WHERE  id = nf_capa.natureza_id)
                               INNER JOIN fiscal_tributacao_agrupar_pgview tributacao
                                       ON tributacao.nf_id = nf_capa.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                        WHERE  cfop.produto_servico_tipo = 'F'
                               AND nf_capa.state = '3'
                               AND nf_capa.data_saida_entrada >= '{data_inicio}'
                               AND nf_capa.data_saida_entrada < '{data_fim}'
                               AND encargo.sigla = 'ICMS'
                               AND nf_capa.company_id = {company_id}
                               AND nf_capa.modelo_nf_id NOT IN ({modelo_nf_ids})
                        GROUP  BY uf.sigla,
                                  uf.name,
                                  encargo.sigla

                                  ) AS tmp
                GROUP  BY uf
        """
        sql = sql.format(data_inicio=data_inicio, data_fim=data_fim, company_id=company_id,
                         modelo_nf_ids=', '.join(map(lambda m: str(m), modelo_nf_ids)))
        cr.execute(sql)
        return cr.dictfetchall()

    def get_operacoes_saida_por_uf(self, cr, uid, data_inicio, data_fim, company_id, modelo_nf_ids):
        obj_adm_utl = self.pool.get('adm.utils')
        ecf_instaldo = obj_adm_utl.admutl_check_modulo_instalado(cr, uid, 'ecf', context={})
        uf_empresa = self.pool.get('res.company').browse(cr, uid, company_id).rescon_pessoa_id.uf
        fiscal_parametro_dados = obj_adm_utl.admutl_get_parametros(
            cr, uid, 'fiscal', 'fispar_base_reduzida')
        fispar_base_reduzida = fiscal_parametro_dados.get('fispar_base_reduzida')

        #Sql para as notas de saida
        sql_notas_saida = """
                          SELECT uf.sigla
                               || '-'
                               || uf.name AS uf,
                               CASE
                                 WHEN fe.sigla = 'ICMS' THEN
                                 SUM(mov_itens.vr_total_item_rateado)
                                 + SUM(it_trib.vr_icms_substituicao)
                                 ELSE 0.00
                               END        AS vr_contabil,
                               CASE
                                 WHEN cst.tipo = 'T'
                                      AND fe.sigla = 'ICMS' THEN SUM(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END        AS vr_base_calculo,
                               CASE
                                 WHEN cst.tipo = 'I'
                                      AND fe.sigla = 'ICMS' THEN SUM(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END        AS isento,
                               CASE
                                 WHEN cst.tipo = 'U'
                                      AND fe.sigla = 'ICMS' THEN
                                 SUM(it_trib.vr_base_calculo)
                                 + SUM(it_trib.vr_icms_substituicao)
                                 ELSE 0.00
                               END        AS substituicao,
                               CASE
                                 WHEN cst.tipo = 'O'
                                      AND fe.sigla = 'ICMS' THEN SUM(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END        AS outros,
                               CASE
                                 WHEN cst.tipo IN ('T', 'TU', 'U')
                                      AND fe.sigla = 'ICMS' THEN SUM(it_trib.vr_reducao_base)
                                 ELSE 0.00
                               END        AS vr_reducao_base,
                               CASE
                                 WHEN fe.sigla = 'IPI' THEN SUM(it_trib.vr_encargo)
                                 ELSE 0.00
                               END        AS ipi
                        FROM   estoque_movimento_itens mov_itens
                               inner join fiscal_nf_capa nf_capa
                                       ON nf_capa.id = mov_itens.nf_id
                               inner join fiscal_nf_item_tributacao it_trib
                                       ON it_trib.movimento_itens_id = mov_itens.id
                               inner join adm_pessoa_endereco endereco
                                       ON endereco.id = nf_capa.destinatario_endereco_id
                               inner join adm_uf uf
                                       ON uf.id = endereco.uf_id
                               inner join fiscal_encargo fe
                                       ON fe.id = it_trib.encargo_id
                                          AND fe.sigla IN ( 'ICMS', 'IPI' )
                               inner join fiscal_cst cst
                                       ON cst.id = it_trib.cst_id
                        WHERE  nf_capa.tipo = 'S'
                               AND nf_capa.data_emissao BETWEEN
                                   '{data_inicio}' AND '{data_fim}'
                               AND nf_capa.state = '3'
                               AND nf_capa.company_id = {company_id}
                               AND mov_itens.cfop_id NOT IN (SELECT id
                                                             FROM fiscal_cfop
                                                             WHERE codigo IN ('5929', '6929'))
                               AND nf_capa.modelo_nf_id NOT IN ({modelo_nf_ids})
                        GROUP  BY uf.sigla,
                                  uf.name,
                                  cst.tipo,
                                  fe.sigla
        """
        sql_notas_saida = sql_notas_saida.format(
            data_inicio=data_inicio, data_fim=data_fim, company_id=company_id,
            modelo_nf_ids=', '.join(map(lambda m: str(m), modelo_nf_ids)))
        #Sql para os cupons fiscais
        sql_saida_cupons = """
                            SELECT (SELECT sigla
                                           || '-'
                                           || NAME
                                    FROM   adm_uf
                                    WHERE  id = (SELECT MIN(id)
                                                 FROM adm_uf
                                                 WHERE sigla = '{uf_empresa}')
                                   )                               AS uf,
                                   SUM(COALESCE(mapa_itens.base_valor, 0.0))
                                                                   AS vr_contabil,
                                   SUM(CASE
                                     WHEN mapa_itens.tipo = 'T' THEN COALESCE(mapa_itens.base_valor, 0.0)
                                     ELSE 0
                                   END)                            AS vr_base_calculo,
                                   SUM(CASE
                                     WHEN mapa_itens.tipo = 'I' THEN COALESCE(mapa_itens.base_valor, 0.0)
                                     ELSE 0
                                   END)                            AS isento,
                                   SUM(CASE
                                     WHEN mapa_itens.tipo = 'F'
                                     THEN COALESCE(mapa_itens.base_valor, 0.0)
                                     ELSE 0
                                   END)                            AS substituicao,
                                   SUM(CASE
                                     WHEN mapa_itens.tipo = 'O'
                                     THEN COALESCE(mapa_itens.base_valor, 0.0)
                                     ELSE 0
                                   END)                             AS outros,
                                   SUM(CASE
                                     WHEN mapa_itens.tipo IN ('T','F')
                                     THEN COALESCE(mapa_itens.base_reduzida, 0.0)
                                     ELSE 0
                                   END)                            AS vr_reducao_base,
                                   0.00                             AS ipi
                            FROM   ecf_mapa_resumo_capa mapa_capa
                                   inner join ecf_ecf ecf
                                           ON ecf.id = mapa_capa.ecf
                                   inner join ecf_mapa_resumo_itens mapa_itens
                                           ON mapa_itens.capa_id = mapa_capa.id
                                              AND mapa_capa.data >= '{data_inicio}'
                                              AND mapa_capa.data <= '{data_fim}'
                            WHERE ecf.company_id = {company_id}
                            GROUP  BY uf
        """
        sql_saida_cupons = sql_saida_cupons.format(data_inicio=data_inicio, data_fim=data_fim,
                                                   company_id=company_id, uf_empresa=uf_empresa)
        if ecf_instaldo:
            sql = """
                SELECT uf                          AS uf,
                       SUM(vr_contabil) + SUM(ipi) AS vr_contabil,
                       SUM(vr_base_calculo)        AS vr_base_calculo,
                       SUM(isento)                 AS isento,
                       SUM(substituicao)           AS substituicao,
                       SUM(outros)                 AS outros,
                       SUM(vr_reducao_base)        AS vr_reducao_base,
                       SUM(ipi)                    AS ipi
                FROM   (
                  {sql_nota}
                UNION ALL
                  {sql_cupons}
                ) AS tmp
                GROUP  BY uf
            """
            sql = sql.format(sql_nota=sql_notas_saida, sql_cupons=sql_saida_cupons,
                             company_id=company_id)
        else:
            sql = """
                SELECT uf                          AS uf,
                       SUM(vr_contabil) + SUM(ipi) AS vr_contabil,
                       SUM(vr_base_calculo)        AS vr_base_calculo,
                       SUM(isento)                 AS isento,
                       SUM(substituicao)           AS substituicao,
                       SUM(outros)                 AS outros,
                       SUM(vr_reducao_base)        AS vr_reducao_base,
                       SUM(ipi)                    AS ipi
                FROM   ({sql_nota}) AS tmp
                GROUP  BY uf
            """
            sql = sql.format(sql_nota=sql_notas_saida, company_id=company_id)
        cr.execute(sql)
        dados_saida = cr.dictfetchall()
        for dado in dados_saida:
            if fispar_base_reduzida == 'I':
                dado['isento'] += (dado.get('vr_reducao_base') or 0)
            else:
                dado['outros'] += (dado.get('vr_reducao_base') or 0)
        return dados_saida

    def get_operacoes_entrada_por_cfop(self, cr, uid, data_inicio, data_fim, company_id,
                                       modelo_nf_ids):
        sql = """
            SELECT cfop                        AS cfop,
                   Sum(vr_contabil) + Sum(ipi) AS vr_contabil,
                   Sum(vr_base_calculo)        AS vr_base_calculo,
                   Sum(vr_encargo)             AS imposto,
                   Sum(isento)                 AS isento,
                   Sum(outros) + Sum(ipi)      AS outros,
                   Sum(substituicao)           AS substituicao
            FROM   (SELECT CASE
                             WHEN fe.sigla = 'ICMS' THEN
                             Sum(mov_itens.vr_total_item_rateado)
                             + Sum(it_trib.vr_icms_substituicao)
                             ELSE 0.00
                           END         AS vr_contabil,
                           CASE
                             WHEN fe.sigla = 'ICMS' THEN Sum(it_trib.vr_encargo)
                             ELSE 0.00
                           END         AS vr_encargo,
                           CASE
                             WHEN cst.tipo = 'T'
                                  AND fe.sigla = 'ICMS' THEN Sum(it_trib.vr_base_calculo)
                             ELSE 0.00
                           END         AS vr_base_calculo,
                           CASE
                             WHEN cst.tipo = 'I'
                                  AND fe.sigla = 'ICMS' THEN Sum(it_trib.vr_base_calculo)
                             ELSE 0.00
                           END         AS isento,
                           CASE
                             WHEN cst.tipo IN ('U', 'TU')
                                  AND fe.sigla = 'ICMS' THEN
                             Sum(it_trib.vr_base_calculo)
                             + Sum(it_trib.vr_icms_substituicao)
                             + Sum(vr_reducao_base)
                             ELSE 0.00
                           END         AS substituicao,
                           CASE
                                 WHEN cst.tipo IN ('O', 'D', 'S') AND fe.sigla = 'ICMS'
                                     THEN SUM(it_trib.vr_base_calculo)
                                 WHEN cst.tipo = 'T' AND fe.sigla = 'ICMS'
                                     THEN SUM(it_trib.vr_reducao_base)
                                 ELSE 0.00
                           END         AS outros,
                           CASE
                             WHEN fe.sigla = 'IPI' THEN Sum(it_trib.vr_encargo)
                             ELSE 0.00
                           END         AS ipi,
                           cfop.codigo ||'-'|| cfop.descricao AS cfop
                    FROM   estoque_movimento_itens mov_itens
                           INNER JOIN fiscal_nf_capa nf_capa
                                   ON nf_capa.id = mov_itens.nf_id
                           INNER JOIN fiscal_nf_item_tributacao it_trib
                                   ON it_trib.movimento_itens_id = mov_itens.id
                           INNER JOIN adm_pessoa_endereco endereco
                                   ON endereco.id = nf_capa.destinatario_endereco_id
                           INNER JOIN adm_uf uf
                                   ON uf.id = endereco.uf_id
                           INNER JOIN fiscal_encargo fe
                                   ON fe.id = it_trib.encargo_id
                                      AND fe.sigla IN ( 'ICMS', 'IPI' )
                           INNER JOIN fiscal_cst cst
                                   ON cst.id = it_trib.cst_id
                           INNER JOIN fiscal_cfop cfop
                                   ON cfop.id = mov_itens.cfop_id
                    WHERE  nf_capa.tipo = 'E'
                           AND nf_capa.data_saida_entrada BETWEEN
                               '{data_inicio}' AND '{data_fim}'
                           AND nf_capa.state = '3'
                           AND nf_capa.company_id = {company_id}
                           AND cfop.produto_servico_tipo != 'F'
                           AND nf_capa.modelo_nf_id NOT IN ({modelo_nf_ids})
                    GROUP  BY uf.sigla,
                              uf.NAME,
                              cst.tipo,
                              fe.sigla,
                              cfop.codigo,
                              cfop.descricao

                    UNION ALL

                    SELECT
                           Sum(COALESCE(nf_capa.vr_total_nf, 0.0))             AS vr_contabil,
                           Sum(CASE
                                 WHEN tributacao.tipo = 'T'
                                 THEN COALESCE(tributacao.vr_encargo, 0.0)
                                 ELSE 0
                               END)                                            AS vr_encargo,
                           Sum(CASE
                                 WHEN tributacao.tipo = 'T'
                                 THEN COALESCE(tributacao.vr_base_calculo, 0.0)
                                 ELSE 0.00
                               END)                                            AS vr_base_calculo,
                           Sum(CASE
                                 WHEN tributacao.tipo = 'I' THEN
                                 COALESCE(tributacao.vr_base_calculo, 0.0)
                                 ELSE 0
                               END)                                            AS isento,
                           Sum(CASE
                                 WHEN tributacao.tipo = 'U'
                                 THEN COALESCE(tributacao.vr_base_calculo, 0.0) +
                                      COALESCE(tributacao.vr_icms_substituicao, 0.0) +
                                      COALESCE(tributacao.vr_reducao_base, 0.0)
                                 ELSE 0
                               END)                                            AS substituicao,
                           Sum(CASE
                                 WHEN tributacao.tipo = 'O'
                                 THEN COALESCE(tributacao.vr_base_calculo, 0.0)
                                 ELSE 0
                               END)                                            AS outros,
                           0.00                                                AS ipi,
                           cfop.codigo ||'-'|| cfop.descricao                  AS cfop
                    FROM   fiscal_nf_capa nf_capa
                           INNER JOIN fiscal_cfop cfop
                                   ON cfop.id = (SELECT cfop_id
                                                 FROM   fiscal_cfop_natureza
                                                 WHERE  id = nf_capa.natureza_id)
                           INNER JOIN fiscal_tributacao_agrupar_pgview tributacao
                                   ON tributacao.nf_id = nf_capa.id
                           INNER JOIN fiscal_encargo encargo
                                   ON encargo.id = tributacao.encargo_id
                    WHERE  cfop.produto_servico_tipo = 'F'
                           AND nf_capa.state = '3'
                           AND nf_capa.data_saida_entrada >= '{data_inicio}'
                           AND nf_capa.data_saida_entrada < '{data_fim}'
                           AND encargo.sigla = 'ICMS'
                           AND nf_capa.company_id = {company_id}
                           AND nf_capa.modelo_nf_id NOT IN ({modelo_nf_ids})
                    GROUP  BY cfop.codigo,
                              cfop.descricao

                                            ) AS tmp
            GROUP  BY cfop

        """
        sql = sql.format(data_inicio=data_inicio, data_fim=data_fim, company_id=company_id,
                         modelo_nf_ids=', '.join(map(lambda m: str(m), modelo_nf_ids)))
        cr.execute(sql)
        return cr.dictfetchall()

    def get_operacoes_saida_por_cfop(self, cr, uid, data_inicio, data_fim, company_id,
                                     modelo_nf_ids):
        obj_adm_utl = self.pool.get('adm.utils')
        ecf_instaldo = obj_adm_utl.admutl_check_modulo_instalado(cr, uid, 'ecf', context={})
        sql_notas = """
                        SELECT cfop.codigo
                               || '-'
                               || cfop.descricao AS cfop,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN fe.sigla = 'ICMS' THEN Sum(mov_itens.vr_total_item_rateado)
                                                             + Sum(it_trib.vr_icms_substituicao)
                                 ELSE 0.00
                               END               AS vr_contabil,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN fe.sigla = 'IPI' THEN Sum(it_trib.vr_encargo)
                                 ELSE 0.00
                               END               AS ipi,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN cst.tipo = 'T'
                                      AND fe.sigla = 'ICMS' THEN Sum(it_trib.vr_base_calculo)
                                                                 + Sum(vr_reducao_base)
                                 ELSE 0.00
                               END               AS vr_base_calculo,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN cst.tipo = 'T' THEN Sum(it_trib.vr_encargo)
                                 ELSE 0.00
                               END               AS imposto,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN cst.tipo = 'I'
                                      AND fe.sigla = 'ICMS' THEN Sum(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END               AS isento,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN cst.tipo = 'U'
                                      AND fe.sigla = 'ICMS' THEN Sum(it_trib.vr_base_calculo)
                                                                 + Sum(it_trib.vr_icms_substituicao)
                                                                 + Sum(vr_reducao_base)
                                 ELSE 0.00
                               END               AS substituicao,
                               CASE
                                 WHEN cfop.codigo IN ('5929', '6929') THEN 0.00
                                 WHEN cst.tipo = 'O'
                                      AND fe.sigla = 'ICMS' THEN Sum(it_trib.vr_base_calculo)
                                 ELSE 0.00
                               END               AS outros
                        FROM   estoque_movimento_itens mov_itens
                               INNER JOIN fiscal_nf_capa nf_capa
                                       ON nf_capa.id = mov_itens.nf_id
                               INNER JOIN fiscal_nf_item_tributacao it_trib
                                       ON it_trib.movimento_itens_id = mov_itens.id
                               INNER JOIN adm_pessoa_endereco endereco
                                       ON endereco.id = nf_capa.destinatario_endereco_id
                               INNER JOIN adm_uf uf
                                       ON uf.id = endereco.uf_id
                               INNER JOIN fiscal_encargo fe
                                       ON fe.id = it_trib.encargo_id
                                          AND fe.sigla IN ( 'ICMS', 'IPI' )
                               INNER JOIN fiscal_cst cst
                                       ON cst.id = it_trib.cst_id
                               INNER JOIN fiscal_cfop cfop
                                       ON cfop.id = mov_itens.cfop_id
                        WHERE  nf_capa.tipo = 'S'
                               AND nf_capa.data_emissao BETWEEN '{data_ini}' AND '{data_fim}'
                               AND nf_capa.state = '3'
                               AND nf_capa.company_id = {company_id}
                               AND nf_capa.modelo_nf_id NOT IN ({modelo_nf_ids})
                        GROUP  BY uf.sigla,
                                  uf.NAME,
                                  cst.tipo,
                                  fe.sigla,
                                  cfop.codigo,
                                  cfop.descricao
        """
        sql_notas = sql_notas.format(data_ini=data_inicio, data_fim=data_fim, company_id=company_id,
                                     modelo_nf_ids=', '.join(map(lambda m: str(m), modelo_nf_ids)))
        sql_cupons = """
                        SELECT CASE
                                 WHEN mapa_itens.tipo = 'F' THEN (SELECT codigo || '-' || descricao
                                                                  FROM   fiscal_cfop
                                                                  WHERE  codigo = '5405')
                                 ELSE (SELECT codigo || '-' || descricao
                                       FROM   fiscal_cfop
                                       WHERE  codigo = '5102')
                               END                             AS cfop,
                               SUM(COALESCE(mapa_itens.base_valor, 0.0))
                                                               AS vr_contabil,
                               0.00                            AS ipi,
                               SUM(CASE
                                 WHEN mapa_itens.tipo = 'T' THEN COALESCE(mapa_itens.base_valor, 0.0)
                                 ELSE 0
                               END)                            AS vr_base_calculo,
                               SUM(CASE
                                 WHEN mapa_itens.tipo = 'T' THEN COALESCE(mapa_itens.imposto_valor, 0.0)
                                 ELSE 0
                               END)                            AS imposto,
                               SUM(CASE
                                 WHEN mapa_itens.tipo = 'I' THEN COALESCE(mapa_itens.base_valor, 0.0)
                                 ELSE 0
                               END)                            AS isento,
                               SUM(CASE
                                 WHEN mapa_itens.tipo = 'F'
                                 THEN COALESCE(mapa_itens.base_valor, 0.0)
                                 ELSE 0
                               END)                            AS substituicao,
                               SUM(CASE
                                 WHEN mapa_itens.tipo = 'O'
                                 THEN COALESCE(mapa_itens.base_valor, 0.0)
                                 ELSE 0
                               END)                             AS outros
                        FROM   ecf_mapa_resumo_capa mapa_capa
                               inner join ecf_ecf ecf
                                       ON ecf.id = mapa_capa.ecf
                               inner join ecf_mapa_resumo_itens mapa_itens
                                       ON mapa_itens.capa_id = mapa_capa.id
                                          AND mapa_capa.data >= '{data_ini}'
                                          AND mapa_capa.data <= '{data_fim}'
                        WHERE ecf.company_id = {company_id}
                        GROUP  BY cfop
        """
        sql_cupons = sql_cupons.format(data_ini=data_inicio, data_fim=data_fim,
                                       company_id=company_id)
        if ecf_instaldo:
            sql = """
                SELECT cfop                                         AS cfop,
                       SUM(vr_contabil)                             AS vr_contabil,
                       SUM(vr_base_calculo)                         AS vr_base_calculo,
                       SUM(isento)                                  AS isento,
                       SUM(substituicao)                            AS substituicao,
                       SUM(outros) + SUM(ipi)                       AS outros,
                       SUM(imposto)                                 AS imposto
                FROM   (
                  {sql_nota}
                UNION ALL
                  {sql_cupons}
                ) AS tmp
                GROUP  BY cfop
            """
            sql = sql.format(sql_nota=sql_notas, sql_cupons=sql_cupons, company_id=company_id)
        else:
            sql = """
                SELECT cfop                                         AS cfop,
                       SUM(vr_contabil)                             AS vr_contabil,
                       SUM(vr_base_calculo)                         AS vr_base_calculo,
                       SUM(isento)                                  AS isento,
                       SUM(substituicao)                            AS substituicao,
                       SUM(outros) + SUM(ipi)                       AS outros,
                       SUM(imposto)                                 AS imposto
                FROM   ({sql_nota}) AS tmp
                GROUP  BY cfop
            """
            sql = sql.format(sql_nota=sql_notas, company_id=company_id)
        cr.execute(sql)
        return cr.dictfetchall()


class EntradaPorUf(Report):
    print_if_empty = True
    title = 'DAMEF'

    class band_page_header(ReportBand):
        BAND_WIDTH = 17*cm
        height = 2.0*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        def get_empresa(self, generator):
            self.text = EMPRESA

        def get_periodo(self, generator):
            self.text = PERIODO

        elements = [
            SystemField(expression='%(report_title)s', top=0.1*cm, left=0, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 8,
                               'alignment': TA_CENTER}),
            Label(text=EMPRESA, top=0.9*cm, left=0, width=BAND_WIDTH,
                  style={'fontName': 'Helvetica-Bold', 'fontSize': 8, 'alignment': TA_CENTER},
                  before_print=get_empresa),
            SystemField(expression=u'Pag. %(page_number)d de %(page_count)d', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm, width=BAND_WIDTH, left=0.1*cm,
                        style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            Label(text=_('DETALHAMENTO DAS OPERACOES POR UNIDADE DA FEDERACAO - Entrada'),
                  top=0.8*cm, width=BAND_WIDTH, before_print=traduz_label,
                  style={'alignment': TA_CENTER, 'fontSize': 8, 'fontName': 'Helvetica-Bold'},),
            Label(text=PERIODO, top=1.6*cm, width=BAND_WIDTH, left=0.1*cm,
                  style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'},
                  before_print=get_periodo),
            Label(text="UF", top=1.5*cm, left=0.3*cm, style={'fontSize': 8,
                                                             'fontName': 'Helvetica-Bold'}),
            Label(text=_("Valor contabil"), top=1.5*cm, left=1.0*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                  before_print=traduz_label),
            Label(text="Base calculo", top=1.5*cm, left=3.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Isento", top=1.5*cm, left=5.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text=_("Substituicao"), top=1.5*cm, left=8.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                  before_print=traduz_label),
            Label(text="Outros", top=1.5*cm, left=10.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="IPI", top=1.5*cm, left=12.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            ]
        borders = {'all': True}

    class band_detail(DetailBand):
        height = 0.7*cm
        BAND_WIDTH = 17*cm
        elements = [
            ObjectValue(expression='uf', left=0.3*cm, style={'fontSize': 8}),
            ObjectValue(expression='vr_contabil', left=1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('vr_contabil', '0.00'))),
            ObjectValue(expression='vr_base_calculo', left=3.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('vr_base_calculo'))),
            ObjectValue(expression='isento', left=5.5*cm, style={'fontSize': 8,
                                                                 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('isento'))),
            ObjectValue(expression='substituicao', left=8.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('substituicao'))),
            ObjectValue(expression='outros', left=10.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('outros'))),
            ObjectValue(expression='ipi', left=12.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('ipi'))),
        ]
        borders = {'left': True, 'right': True}

    class band_summary(ReportBand):
        height = 0.5*cm

        def do_before_print(self, generator):
            dados = generator.report.queryset
            if generator._current_object:
                generator._current_object['tot_vr_contab'] = sum(map(lambda i: i.get('vr_contabil'),
                                                                     dados))
                generator._current_object['tot_vr_base'] = sum(map(lambda i: i.get('vr_base_calculo'),
                                                                   dados))
                generator._current_object['tot_isento'] = sum(map(lambda i: i.get('isento'), dados))
                generator._current_object['tot_st'] = sum(map(lambda i: i.get('substituicao'), dados))
                generator._current_object['tot_out'] = sum(map(lambda i: i.get('outros'), dados))
                generator._current_object['tot_ipi'] = sum(map(lambda i: i.get('ipi'), dados))
            else:
                dict_current_object = {'tot_vr_contab': 0,
                                       'tot_vr_base': 0,
                                       'tot_isento': 0,
                                       'tot_st': 0,
                                       'tot_out': 0,
                                       'tot_ipi': 0}
                generator._current_object = dict_current_object

        elements = [
            Label(text="Total geral: ", left=0.3*cm, style={'fontSize': 8,
                                                            'fontName': 'Helvetica-Bold'}),
            ObjectValue(expression='tot_vr_contab', left=1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_contab', ' '))),
            ObjectValue(expression='tot_vr_base', left=3.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_base', ' '))),
            ObjectValue(expression='tot_isento', left=5.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_isento', ' '))),
            ObjectValue(expression='tot_st', left=8.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_st', ' '))),
            ObjectValue(expression='tot_out', left=10.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_out', ' '))),
            ObjectValue(expression='tot_ipi', left=12.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_ipi', ' ')))
        ]
        borders = {'all': True}

    class band_page_footer(ReportBand):
        height = 0.5*cm
        BAND_WIDTH = 17*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        elements = [
            Label(text=_('Sig Solucoes em Informatica'), top=0.1*cm, before_print=traduz_label),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            ]
        borders = {'top': True}


class SaidaPorUf(Report):
    title = 'DAMEF'
    print_if_empty = True

    class band_page_header(ReportBand):
        BAND_WIDTH = 17*cm
        height = 2.0*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        def get_empresa(self, generator):
            self.text = EMPRESA

        def get_periodo(self, generator):
            self.text = PERIODO
        elements = [
            SystemField(expression='%(report_title)s', top=0.1*cm, left=0, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 8,
                               'alignment': TA_CENTER}),
            Label(text=EMPRESA, top=0.9*cm, left=0, width=BAND_WIDTH,
                  style={'fontName': 'Helvetica-Bold', 'fontSize': 8, 'alignment': TA_CENTER},
                  before_print=get_empresa),
            SystemField(expression=u'Pag. %(page_number)d de %(page_count)d', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm, width=BAND_WIDTH, left=0.1*cm,
                        style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            Label(text=_('DETALHAMENTO DAS OPERACOES POR UNIDADE DA FEDERACAO - Saida'),
                  top=0.8*cm, width=BAND_WIDTH, before_print=traduz_label,
                  style={'alignment': TA_CENTER, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            Label(text=PERIODO, top=1.6*cm, width=BAND_WIDTH, left=0.1*cm,
                  style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'},
                  before_print=get_periodo),
            Label(text="UF", top=1.5*cm, left=0.3*cm, style={'fontSize': 8,
                                                             'fontName': 'Helvetica-Bold'}),
            Label(text=_("Valor contabil"), top=1.5*cm, left=1.0*cm, before_print=traduz_label,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Base calculo", top=1.5*cm, left=3.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Isento", top=1.5*cm, left=5.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text=_("Substituicao"), top=1.5*cm, left=8.5*cm, before_print=traduz_label,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Outros", top=1.5*cm, left=10.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="IPI", top=1.5*cm, left=12.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            ]
        borders = {'all': True}

    class band_detail(DetailBand):
        height = 0.7*cm
        BAND_WIDTH = 17*cm
        elements = [
            ObjectValue(expression='uf', left=0.3*cm, style={'fontSize': 8}),
            ObjectValue(expression='vr_contabil', left=1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get(
                            'vr_contabil') or 0)),
            ObjectValue(expression='vr_base_calculo', left=3.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get(
                            'vr_base_calculo') or 0)),
            ObjectValue(expression='isento', left=5.5*cm, style={'fontSize': 8,
                                                                 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('isento') or 0)),
            ObjectValue(expression='substituicao', left=8.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get(
                            'substituicao') or 0)),
            ObjectValue(expression='outros', left=10.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('outros') or 0)),
            ObjectValue(expression='ipi', left=12.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('ipi') or 0)),
            ]
        borders = {'left': True, 'right': True}

    class band_summary(ReportBand):
        height = 0.5*cm

        def do_before_print(self, generator):
            dados = generator.report.queryset
            if generator._current_object:
                generator._current_object['tot_vr_contab'] = sum(map(
                    lambda i: i.get('vr_contabil') or 0, dados))
                generator._current_object['tot_vr_base'] = sum(map(
                    lambda i: i.get('vr_base_calculo') or 0, dados))
                generator._current_object['tot_isento'] = sum(map(
                    lambda i: i.get('isento') or 0, dados))
                generator._current_object['tot_st'] = sum(map(
                    lambda i: i.get('substituicao') or 0, dados))
                generator._current_object['tot_out'] = sum(map(
                    lambda i: i.get('outros') or 0, dados))
                generator._current_object['tot_ipi'] = sum(map(
                    lambda i: i.get('ipi') or 0, dados))
            else:
                dict_current_object = {'tot_vr_contab': 0,
                                       'tot_vr_base': 0,
                                       'tot_isento': 0,
                                       'tot_st': 0,
                                       'tot_out': 0,
                                       'tot_ipi': 0}
                generator._current_object = dict_current_object

        elements = [
            Label(text="Total geral: ", left=0.3*cm, style={'fontSize': 8,
                                                            'fontName': 'Helvetica-Bold'}),
            ObjectValue(expression='tot_vr_contab', left=1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_contab'))),
            ObjectValue(expression='tot_vr_base', left=3.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_base'))),
            ObjectValue(expression='tot_isento', left=5.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_isento'))),
            ObjectValue(expression='tot_st', left=8.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_st'))),
            ObjectValue(expression='tot_out', left=10.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_out'))),
            ObjectValue(expression='tot_ipi', left=12.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_ipi')))
        ]
        borders = {'all': True}

    class band_page_footer(ReportBand):
        height = 0.5*cm
        BAND_WIDTH = 17*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        elements = [
            Label(text=_('Sig Solucoes em Informatica'), top=0.1*cm, before_print=traduz_label),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            ]
        borders = {'top': True}


class EntradaPorCfop(Report):
    title = 'DAMEF'
    print_if_empty = True

    class band_page_header(ReportBand):
        BAND_WIDTH = 17*cm
        height = 2.0*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        def get_empresa(self, generator):
            self.text = EMPRESA

        def get_periodo(self, generator):
            self.text = PERIODO

        elements = [
            SystemField(expression='%(report_title)s', top=0.1*cm, left=0, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 8,
                               'alignment': TA_CENTER}),
            Label(text=EMPRESA, top=0.9*cm, left=0, width=BAND_WIDTH,
                  style={'fontName': 'Helvetica-Bold', 'fontSize': 8, 'alignment': TA_CENTER},
                  before_print=get_empresa),
            SystemField(expression=u'Pag. %(page_number)d de %(page_count)d', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm, width=BAND_WIDTH, left=0.1*cm,
                        style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            SystemField(expression=_('Entradas'),
                        top=0.8*cm, width=BAND_WIDTH,
                        style={'alignment': TA_CENTER, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            Label(text=PERIODO, top=1.6*cm, width=BAND_WIDTH, left=0.1*cm,
                  style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'},
                  before_print=get_periodo),
            Label(text="CFOP", top=1.5*cm, left=0.1*cm, style={'fontSize': 8,
                                                               'fontName': 'Helvetica-Bold'}),
            Label(text=_("Valor contabil"), top=1.5*cm, left=3.0*cm, before_print=traduz_label,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Base calculo", top=1.5*cm, left=5.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Imposto", top=1.5*cm, left=7.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Isentos", top=1.5*cm, left=9.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Outros", top=1.5*cm, left=11.1*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text=_("Subst.Tributaria"), top=1.5*cm, left=13.5*cm, before_print=traduz_label,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            ]
        borders = {'all': True}

    class band_detail(DetailBand):
        height = 0.9*cm
        BAND_WIDTH = 17*cm
        elements = [
            ObjectValue(expression='cfop', left=0.1*cm, style={'fontSize': 7.5},
                        get_value=lambda instance: instance.get('cfop')[:42],
                        width=6.5*cm),
            ObjectValue(expression='vr_contabil', left=3.0*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('vr_contabil'))),
            ObjectValue(expression='vr_base_calculo', left=5.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('vr_base_calculo'))),
            ObjectValue(expression='imposto', left=7.5*cm, style={'fontSize': 8,
                                                                 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('imposto'))),
            ObjectValue(expression='isento', left=9.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('isento'))),
            ObjectValue(expression='outros', left=11.1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('outros'))),
            ObjectValue(expression='substituicao', left=13.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('substituicao'))),
            ]
        borders = {'left': True, 'right': True}

    class band_summary(ReportBand):
        height = 0.5*cm

        def do_before_print(self, generator):
            dados = generator.report.queryset
            if generator._current_object:
                generator._current_object['tot_vr_contab'] = sum(map(lambda i: i.get('vr_contabil'),
                                                                     dados))
                generator._current_object['tot_vr_base'] = sum(map(lambda i: i.get('vr_base_calculo'),
                                                                   dados))
                generator._current_object['tot_imposto'] = sum(map(lambda i: i.get('imposto'), dados))
                generator._current_object['tot_isento'] = sum(map(lambda i: i.get('isento'), dados))
                generator._current_object['tot_out'] = sum(map(lambda i: i.get('outros'), dados))
                generator._current_object['tot_st'] = sum(map(lambda i: i.get('substituicao'), dados))
            else:
                dict_current_object = {'tot_vr_contab': 0,
                                       'tot_vr_base': 0,
                                       'tot_imposto': 0,
                                       'tot_isento': 0,
                                       'tot_out': 0,
                                       'tot_st': 0}
                generator._current_object = dict_current_object

        elements = [
            Label(text="Total geral: ", left=0.1*cm, style={'fontSize': 8,
                                                            'fontName': 'Helvetica-Bold'}),
            ObjectValue(expression='tot_vr_contab', left=3.0*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_contab'))),
            ObjectValue(expression='tot_vr_base', left=5.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_base'))),
            ObjectValue(expression='tot_imposto', left=7.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_imposto'))),
            ObjectValue(expression='tot_isento', left=9.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_isento'))),
            ObjectValue(expression='tot_out', left=11.1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_out'))),
            ObjectValue(expression='tot_st', left=13.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_st')))
        ]
        borders = {'all': True}

    class band_page_footer(ReportBand):
        height = 0.5*cm
        BAND_WIDTH = 17*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        elements = [
            Label(text=_('Sig Solucoes em Informatica'), top=0.1*cm, before_print=traduz_label),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            ]
        borders = {'top': True}


class SaidaPorCfop(Report):
    title = 'DAMEF'
    print_if_empty = True

    class band_page_header(ReportBand):
        BAND_WIDTH = 17*cm
        height = 2.0*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        def get_empresa(self, generator):
            self.text = EMPRESA

        def get_periodo(self, generator):
            self.text = PERIODO

        elements = [
            SystemField(expression='%(report_title)s', top=0.1*cm, left=0, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 8,
                               'alignment': TA_CENTER}),
            Label(text=EMPRESA, top=0.9*cm, left=0, width=BAND_WIDTH,
                  style={'fontName': 'Helvetica-Bold', 'fontSize': 8, 'alignment': TA_CENTER},
                  before_print=get_empresa),
            SystemField(expression=u'Pag. %(page_number)d de %(page_count)d', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm, width=BAND_WIDTH, left=0.1*cm,
                        style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            Label(text=_('Saidas'), top=0.8*cm, width=BAND_WIDTH, before_print=traduz_label,
                  style={'alignment': TA_CENTER, 'fontSize': 8, 'fontName': 'Helvetica-Bold'}),
            Label(text=PERIODO, top=1.6*cm, width=BAND_WIDTH, left=0.1*cm,
                  style={'alignment': TA_LEFT, 'fontSize': 8, 'fontName': 'Helvetica-Bold'},
                  before_print=get_periodo),
            Label(text="CFOP", top=1.5*cm, left=0.1*cm, style={'fontSize': 8,
                                                               'fontName': 'Helvetica-Bold'}),
            Label(text=_("Valor contabil"), top=1.5*cm, left=3.0*cm, before_print=traduz_label,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Base calculo", top=1.5*cm, left=5.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Imposto", top=1.5*cm, left=7.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Isentos", top=1.5*cm, left=9.5*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text="Outros", top=1.5*cm, left=11.1*cm,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'}),
            Label(text=_("Subst.Tributaria"), top=1.5*cm, left=13.5*cm, before_print=traduz_label,
                  style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},)
            ]
        borders = {'all': True}

    class band_detail(DetailBand):
        height = 0.9*cm
        BAND_WIDTH = 17*cm
        elements = [
            ObjectValue(expression='cfop', left=0.1*cm, style={'fontSize': 7.5},
                        get_value=lambda instance: instance.get('cfop')[:42], width=6.5*cm),
            ObjectValue(expression='vr_contabil', left=3.0*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get(
                            'vr_contabil') or 0)),
            ObjectValue(expression='vr_base_calculo', left=5.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get(
                            'vr_base_calculo') or 0)),
            ObjectValue(expression='imposto', left=7.5*cm, style={'fontSize': 8,
                                                                 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('imposto') or 0)),
            ObjectValue(expression='isento', left=9.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('isento') or 0)),
            ObjectValue(expression='outros', left=11.1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get('outros') or 0)),
            ObjectValue(expression='substituicao', left=13.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT},
                        get_value=lambda instance: getNumeroEditado(instance.get(
                            'substituicao') or 0)),
            ]
        borders = {'left': True, 'right': True}

    class band_summary(ReportBand):
        height = 0.5*cm

        def do_before_print(self, generator):
            dados = generator.report.queryset
            if generator._current_object:
                generator._current_object['tot_vr_contab'] = sum(map(
                    lambda i: i.get('vr_contabil') or 0, dados))
                generator._current_object['tot_vr_base'] = sum(map(
                    lambda i: i.get('vr_base_calculo') or 0, dados))
                generator._current_object['tot_imposto'] = sum(map(
                    lambda i: i.get('imposto') or 0, dados))
                generator._current_object['tot_isento'] = sum(map(
                    lambda i: i.get('isento') or 0, dados))
                generator._current_object['tot_out'] = sum(map(
                    lambda i: i.get('outros') or 0, dados))
                generator._current_object['tot_st'] = sum(map(
                    lambda i: i.get('substituicao') or 0, dados))
            else:
                dict_current_object = {'tot_vr_contab': 0,
                                       'tot_vr_base': 0,
                                       'tot_imposto': 0,
                                       'tot_isento': 0,
                                       'tot_out': 0,
                                       'tot_st': 0}
                generator._current_object = dict_current_object

        elements = [
            Label(text="Total geral: ", left=0.1*cm, style={'fontSize': 8,
                                                            'fontName': 'Helvetica-Bold'}),
            ObjectValue(expression='tot_vr_contab', left=3.0*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_contab'))),
            ObjectValue(expression='tot_vr_base', left=5.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_vr_base'))),
            ObjectValue(expression='tot_imposto', left=7.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_imposto'))),
            ObjectValue(expression='tot_isento', left=9.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_isento'))),
            ObjectValue(expression='tot_out', left=11.1*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_out'))),
            ObjectValue(expression='tot_st', left=13.5*cm,
                        style={'fontSize': 8, 'alignment': TA_RIGHT, 'fontName': 'Helvetica-Bold'},
                        get_value=lambda i: getNumeroEditado(i.get('tot_st')))
        ]
        borders = {'all': True}


    class band_page_footer(ReportBand):
        height = 0.5*cm
        BAND_WIDTH = 17*cm

        def traduz_label(self, generator, context={'lang': 'pt_BR'}):
            self.text = _(self.text)

        elements = [
            Label(text=_('Sig Solucoes em Informatica'), top=0.1*cm, before_print=traduz_label),
            SystemField(expression='%(now:%d/%m/%Y)s', top=0.1*cm,
                        width=BAND_WIDTH, style={'alignment': TA_RIGHT}),
            ]
        borders = {'top': True}
