# -*- coding: utf-8 -*-
from openerp.osv import fields, osv
from openerp.tools.translate import _
import time

__author__ = 'marcilene.ribeiro'
__lastupdate__ = '2018-07-13'
__mantainner__ = 'Pierre Barbosa - pierre.barbosa[at]siginformatica.com.br'


# noinspection PyPep8Naming
class wizard_selecao_nf(osv.TransientModel):
    _name = 'wizard.selecao.nf'  # wizsnf
    _description = 'Selecionar notas ficais de devolucao entre outras'
    _columns = {
        'wizselnf_importar_itens': fields.boolean(
            string='Importar itens', help='Se esta opcao estiver marcada ira importar todos os '
                                          'itens dos documentos selecionados'),
        'nf_ids': fields.many2many('fiscal.nf.capa', 'fiscal_nf_selecao', 'selecao_id', 'nf_id',
                                   "Notas Fiscais"),
    }

    _defaults = {
        'wizselnf_importar_itens': True
    }

    def confirmar_selecao(self, cr, uid, ids, context=None):
        """
        Cria os itens da nova nota baseados nos itens das notas selecionadas.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :param context: dict
        :return: sempre verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        # Este contexto 'super_write_fiscal_nf_capa' faz o super do write da nota diretamente para
        # fazer apenas as alteracoes passadas no vals da funcao write
        context['super_write_fiscal_nf_capa'] = True
        context.update({'nota_fiscal_saida': True, 'proprio': True, 'wizard_selecao_nf': True})

        # Dados da selecao
        selecao_dados = self.read(cr, uid, ids, ['nf_ids'], context=context)

        # Lista das notas fiscais selecionadas
        if selecao_dados and isinstance(selecao_dados, list):
            selecao_dados = selecao_dados[0]
        nf_ids = selecao_dados.get('nf_ids')
        selecao_id = ids[0] if ids and isinstance(ids, (list, tuple)) else ids

        # Se opcao wizselnf_importar_itens desmarcada, nao devera importar os itens dos documentos
        importar_itens = self.browse(cr, uid, selecao_id, context).wizselnf_importar_itens
        result = self._wizsnf_criar_nf(
            cr, uid, context.get('active_id'), nf_ids, importar_itens, context=context)

        self._wizsnf_atualiza_nf_referencia(cr, uid, context.get('active_id'), nf_ids, context)

        return result

    def _wizsnf_criar_nf(self, cr, uid, nf_id, nf_ids, importar_itens, context=None):
        """
        Cria nova nota fiscal baseado nas notas fiscais selecionadas.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nova nota fiscal
        :param nf_ids: lista com o ID das notas fiscais selecionadas
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type nf_id: long
        :type nf_ids: list
        :param context: dict
        :return:
        :rtype: long
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_adm_utils = self.pool.get('adm.utils')
        fields_estoque_parametro = ['estpar_local_entrada_devol_padrao_id',
                                    'estpar_local_compra_padrao_id']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', fields_estoque_parametro, context=context)
        devolucao_local_id = estoque_parametro_dados.get('estpar_local_entrada_devol_padrao_id')
        entrada_padrao_local_id = estoque_parametro_dados.get('estpar_local_compra_padrao_id')
        nf_itens = []  # Novos itens

        # Totais da capa da nota
        nf_dados = obj_fiscal_nf_capa.read(
            cr, uid, nf_id, ['vr_total_produtos', 'vr_total_servicos', 'vr_ipi',
                             'vr_icms_substituicao', 'vr_base_icms_substituicao',
                             'fisnfc_valor_fcp_st', 'fisnfc_valor_icms_desonerado',
                             'estoque_movimento_itens_ids'], context)
        vr_total_produtos = nf_dados.get('vr_total_produtos') or 0
        vr_total_servicos = nf_dados.get('vr_total_servicos') or 0
        vr_total_base_icms = nf_dados.get('vr_base_icms_substituicao') or 0
        vr_total_icms_substituicao = nf_dados.get('vr_icms_substituicao') or 0
        vr_total_ipi = nf_dados.get('vr_ipi') or 0
        vr_total_fcp_st = nf_dados.get('fisnfc_valor_fcp_st') or 0
        vr_total_icms_desonerado = nf_dados.get('fisnfc_valor_icms_desonerado') or 0
        vr_frete = vr_seguro = vr_despesas = vr_desconto = 0
        tipo_frete = 'F'

        # Numero sequencial do item na nota fiscal
        sequencia = nf_dados.get('estoque_movimento_itens_ids').__len__() + 1

        str_informacao_complementar = ''
        contador_nf = 1
        for brw_nf_capa in obj_fiscal_nf_capa.browse(cr, uid, nf_ids, context):
            for brw_item_nf in importar_itens and brw_nf_capa.estoque_movimento_itens_ids or []:
                brw_nf_capa_criada = obj_fiscal_nf_capa.browse(cr, uid, nf_id, context)
                codigo_cfop = brw_nf_capa_criada.natureza_id.cfop_id.codigo or False
                brw_tributacoes = brw_item_nf.tributacoes_ids
                for trib in brw_tributacoes:
                    if trib.encargo_id.sigla == 'ICMS':
                        vr_total_base_icms += trib.vr_base_icms_substituicao
                        vr_total_icms_substituicao += trib.vr_icms_substituicao
                        if trib.cst_id.tipo in ('U', 'TU'):  # U = Subs. tributaria
                            if codigo_cfop:
                                if codigo_cfop[1:] == '201':
                                    codigo_cfop = codigo_cfop[:1] + '410'
                                elif codigo_cfop[1:] == '202':
                                    codigo_cfop = codigo_cfop[:1] + '411'
                            vr_total_fcp_st += trib.fisnit_vr_fcp_st
                        vr_total_icms_desonerado += trib.fisnit_vr_icms_desonerado
                    elif trib.encargo_id.sigla == 'IPI':
                        vr_total_ipi += trib.vr_encargo

                cfop_id = self.pool.get('fiscal.cfop').search(
                    cr, uid, [('codigo', '=', codigo_cfop)], context=context)
                if cfop_id:
                    cfop_id = cfop_id[0]

                # Acrescenta as tributacoes do item
                tributacao_ids = self._wizsnf_criar_itens_tributacao(
                    cr, uid, nf_id, brw_nf_capa.id, brw_item_nf.id, context)

                frete_rateado = brw_item_nf.vr_frete_rateado
                # caso o frete seja por fora, nao coloca frete rateado para os itens
                if brw_nf_capa.tipo_frete == 'F':
                    frete_rateado = 0

                vals_itens = {'local_id': devolucao_local_id or entrada_padrao_local_id,
                              'produto_id': brw_item_nf.produto_id.id,
                              'grade_id': brw_item_nf.grade_id.id, 'sequencia': sequencia,
                              'natureza_id': brw_item_nf.natureza_id.id,
                              'vr_total_item': brw_item_nf.vr_total_item,
                              'vr_desconto_rateado': brw_item_nf.vr_desconto_rateado,
                              'vr_despesas_rateado': brw_item_nf.vr_despesas_rateado,
                              'vr_seguro_rateado': brw_item_nf.vr_seguro_rateado,
                              'vr_frete_rateado': frete_rateado,
                              'vr_total_item_rateado': brw_item_nf.vr_total_item_rateado,
                              'quantidade': brw_item_nf.quantidade,
                              'vr_unitario': brw_item_nf.vr_total_item / brw_item_nf.quantidade,
                              'produto_embalagem_id': brw_item_nf.produto_embalagem_id.id,
                              'cfop_id': cfop_id, 'tributacoes_ids': tributacao_ids,
                              'vendedor_id': brw_item_nf.vendedor_id.id if brw_item_nf.vendedor_id
                              else False,
                              'estmvi_vr_comissao': brw_item_nf.estmvi_vr_comissao or 0,}
                nf_itens.append([0, False, vals_itens])
                sequencia += 1

                # Se for devolucao e o local nao estiver cadastrado para o item, entao cadastra
                if not self.pool.get('estoque.local.locacao').search(
                    cr, uid, [('produto_id', '=', vals_itens['produto_id']),
                              ('grade_id', '=', vals_itens['grade_id']),
                              ('local_id', '=', vals_itens['local_id']),], context=context):

                    self.pool.get('estoque.local.locacao').create(cr, uid, {
                        'produto_id': vals_itens['produto_id'],
                        'grade_id': vals_itens['grade_id'],
                        'local_id': vals_itens['local_id'],
                        'estllc_produto_embalagem_id': vals_itens['produto_embalagem_id'],
                    }, context)

            # Soma os dados das notas escolhidas para a nova nota
            vr_total_produtos += importar_itens and brw_nf_capa.vr_total_produtos
            vr_total_servicos += importar_itens and brw_nf_capa.vr_total_servicos

            # Define o tipo de frete
            if brw_nf_capa.tipo_frete == 'D':
                vr_frete += brw_nf_capa.vr_frete
                tipo_frete = 'D'

            vr_seguro += brw_nf_capa.vr_seguro
            vr_despesas += brw_nf_capa.vr_despesas_acessorias
            vr_desconto += brw_nf_capa.vr_desconto
            serie = brw_nf_capa.serie_nf
            numero = brw_nf_capa.numero_nf
            data_emissao = brw_nf_capa.data_emissao and time.strftime(
                '%d-%m-%Y', time.strptime(brw_nf_capa.data_emissao, '%Y-%m-%d %H:%M:%S')) or ''
            str_informacao_complementar += 'Numero: %s. Serie: %s. Emissao: %s. ' % \
                                           (numero, serie, data_emissao)
            if contador_nf == 2:
                str_informacao_complementar += '\n'
                contador_nf = 0
            contador_nf += 1
        if str_informacao_complementar:
            str_informacao_complementar = 'Notas fiscais referenciadas: \n' + \
                                          str_informacao_complementar

        # Atualiza os dados da nota fiscal
        vals = {'vr_total_produtos': vr_total_produtos, 'vr_total_servicos': vr_total_servicos,
                'vr_icms_substituicao': vr_total_icms_substituicao,
                'vr_base_icms_substituicao': vr_total_base_icms,
                'estoque_movimento_itens_ids': nf_itens, 'vr_frete': vr_frete,
                'vr_seguro': vr_seguro, 'vr_despesas_acessorias': vr_despesas,
                'vr_desconto': vr_desconto, 'vr_ipi': vr_total_ipi,
                'fisnfc_valor_fcp_st': vr_total_fcp_st, 'frete': '0' if vr_frete else '9',
                'tipo_frete': tipo_frete, 'fisnfc_valor_icms_desonerado': vr_total_icms_desonerado,
                'fisnfc_infcomp_contribuinte': str_informacao_complementar}
        result = obj_fiscal_nf_capa.write(cr, uid, nf_id, vals, context)

        return result

    def _wizsnf_criar_itens_tributacao(self, cr, uid, nf_capa_id, nf_id, item_id, context):
        """
        Cria os itens de tributacao baseado no item da nota fiscal selecionada.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nota fiscal selecionada
        :param item_id: ID do item da nota fiscal selecionada
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type nf_id: long
        :type item_id: long
        :type context: dict
        :return: lista com os novos itens de tributacao
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')

        # Recupera os itens de tributacao
        args = [('fisnit_capa_id', '=', nf_id), ('movimento_itens_id', '=', item_id)]
        trib_item_ids = obj_fiscal_nf_item_tributacao.search(cr, uid, args, context=context)
        campos = ['vr_base_icms_substituicao', 'aliquota', 'vr_base_calculo', 'percentual_mva',
                  'cst_id', 'vr_encargo', 'tributacao_id', 'vr_icms_substituicao', 'modbcst',
                  'vr_reducao_base', 'tipo_substituicao', 'aliquota_venda', 'fisnit_csosn',
                  'encargo_id', 'fisnit_vr_fcp_st', 'fisnit_vr_icms_desonerado']

        item_tributacao_dados = obj_fiscal_nf_item_tributacao.read(
            cr, uid, trib_item_ids, campos, context=context, load='_classic_write')
        natureza_id = self.pool.get('fiscal.nf.capa').browse(
            cr, uid, nf_capa_id, context).natureza_id
        if not natureza_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Selecione uma natureza para a nota de entrada!'))
        cst_ids = []
        if natureza_id.cfop_id.movimento_tipo == 'D':
            lista_nf_item_tributacao = []
            for item_trib in item_tributacao_dados:
                cst_ids.append(item_trib.get('cst_id'))
                base = obj_fiscal_tributacao.browse(
                    cr, uid, item_trib.get('tributacao_id'), context).base or 0
                cst_id, tributacao_id = self.pool.get('fiscal.utils').get_cst_oposto(
                    cr, uid, item_trib.get('cst_id'), item_trib.get('tributacao_id'),
                    item_trib.get('encargo_id'), base, item_trib.get('aliquota'), context)

                # Ajusta as tributacoes se a natureza da NF exige sobreposicao
                encargo_id = item_trib.get('encargo_id')
                sobrepor = filter(lambda f: f.encargo_id.id == encargo_id and
                                            f.sobrepor_tributacao_id,
                                  natureza_id.natureza_encargos_ids)
                cst_id = sobrepor and sobrepor[0].sobrepor_tributacao_id.cst_id.id or cst_id
                tributacao_id = sobrepor and sobrepor[0].sobrepor_tributacao_id.id or tributacao_id

                item_trib['cst_id'] = cst_id
                dados_cst = cst_id and self.pool.get('fiscal.cst').read(
                    cr, uid, cst_id, ['tipo'], context)
                isento = dados_cst and dados_cst.get('tipo') == 'I'
                substituicao = dados_cst and dados_cst.get('tipo') in ('U', 'TU')
                item_trib['tributacao_id'] = tributacao_id
                tributacao_dados = self.pool.get('fiscal.tributacao').read(
                    cr, uid, tributacao_id, ['aliquota', 'base'], context)
                item_trib['aliquota'] = tributacao_dados and tributacao_dados.get('aliquota') or 0
                base = tributacao_dados and tributacao_dados.get('base') or 0
                vr_encargo = item_trib['vr_base_calculo'] * item_trib['aliquota'] / 100.
                item_trib['vr_encargo'] = vr_encargo
                vr_base = base * item_trib['vr_base_calculo'] / 100.
                vr_base_reduzida = item_trib['vr_base_calculo'] - vr_base
                vr_base_reduzida = 0 if vr_base_reduzida < 0 else vr_base_reduzida
                item_trib['vr_reducao_base'] = vr_base_reduzida
                if isento:  # Zera a base de calculo se ISENTO
                    item_trib['vr_base_calculo'] = 0
                if not substituicao:  # Zera a base de ICMS/ST se nao for substituicao tributaria
                    item_trib['vr_base_icms_substituicao'] = 0
                lista_nf_item_tributacao.append(item_trib)
                if item_trib.get('id'):
                    del (item_trib['id'])
            item_tributacao_dados = lista_nf_item_tributacao
        else:
            msg = []
            for cst in self.pool.get('fiscal.cst').read(
                    cr, uid, cst_ids, ['fiscst_tipo_movimento', 'codigo', 'name'], context=context):
                if cst.get('fiscst_tipo_movimento') == 'S':
                    msg.append('%s-%s' % (cst.get('codigo'), cst.get('name')))
            if msg:
                nome_item = self.pool.get('estoque.movimento.itens').read(
                    cr, uid, item_id, ['produto_id'], context).get('produto_id')[1]
                raise osv.except_osv(_('Atencao!'),
                                     _('Nota de entrada com CST de saida:\n Item: %s\n\n%s') %
                                     (nome_item, '\n'.join(msg)))
        if not isinstance(item_tributacao_dados, list):
            item_tributacao_dados = [item_tributacao_dados]

        # Itens de tributacao com a correlacao do CST
        result = map(lambda m: [0, False, dict(m, cst_id=self._wizsnf_correlacao_cst(
            cr, uid, m.get('cst_id'), context))], item_tributacao_dados)

        return result

    def _wizsnf_correlacao_cst(self, cr, uid, cst, context=None):
        """
        Realiza a troca do CST.

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param cst: ID do CST
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type cst: long
        :param context: dict
        :return: dicionario com o ID do CST de correlacao: {'cst_id': id}
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_fiscal_cst_correlacao = self.pool.get('fiscal.cst.correlacao')
        args = [('cst_entrada_id', '=', cst)]
        correlacao_cst_id = obj_fiscal_cst_correlacao.search(cr, uid, args, context=context)
        if correlacao_cst_id:
            correlacao_cst_id = correlacao_cst_id[0]  # Pode haver mais de uma correlacao
        else:
            return cst

        # Procura pelo CST na tabela de correlacao
        fiscal_cst_correlacao_dados = obj_fiscal_cst_correlacao.read(
            cr, uid, correlacao_cst_id, ['cst_saida_id'], context=context, load='_classic_write')

        if fiscal_cst_correlacao_dados and isinstance(fiscal_cst_correlacao_dados, (list, tuple)):
            fiscal_cst_correlacao_dados = fiscal_cst_correlacao_dados[0]
            cst = fiscal_cst_correlacao_dados.get('cst_saida_id') or cst  # Troca o CST

        return cst

    def _wizsnf_correlacao_cfop(self, cr, uid, cfop, tipo, context=None):
        """
        Sugere a troca de CFOP na importacao dos itens de notas fiscais.

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param cfop: codigo do cfop
        :param tipo: tipo ou lista com tipos de CST das tributacoes do item
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type cfop: str
        :type tipo: str
        :param context: dict
        :return: o ID do CFOP da correlacao entrada/saida para itens de produtos.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        # Ajusta o CFOP do item entrada/saida
        if cfop.startswith('1'):  # Entrada dentro do estado/UF
            if tipo.upper() in ['TU', 'U']:  # Tributado
                cfop = '5411'
            elif tipo.upper() in ['T', 'I', 'O']:  # Nao tributado
                cfop = '5202'
            else:  # Servico
                cfop = None
        elif cfop.startswith('2'):  # Entrada fora do estado/UF
            if tipo.upper() in ['TU', 'U']:  # Tributado
                cfop = '6411'
            elif tipo.upper() in ['T', 'I', 'O']:  # Nao tributado
                cfop = '6202'
            else:  # Servico
                cfop = None

        # Recupera o ID do CFOP de correlacao
        args = [('codigo', '=', cfop)]
        result = self.pool.get('fiscal.cfop').search(cr, uid, args, context=context)
        result = result[0] if result else 0

        return result

    def _wizsnf_atualiza_nf_referencia(self, cr, uid, nf_id, nf_ids, context):
        """
        Gera as notas fiscais referenciadas para devolucao

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nova nota fiscal
        :param nf_ids: lista com o ID das notas fiscais selecionadas
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type nf_id: long
        :type nf_ids: list[long]
        :param context: dict
        :rtype: boolean
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not nf_ids:
            return
        obj_fnc = self.pool.get('fiscal.nf.capa')
        for i in nf_ids:
            dados = obj_fnc.read(cr, uid, i, ['fisnfc_nf_referencia_ids'], context)
            vals = {'fisnfc_nf_referencia_ids': [[6, False, [nf_id]]]}
            if dados.get('fisnfc_nf_referencia_ids'):
                vals['fisnfc_nf_referencia_ids'][0][2].extend(dados.get('fisnfc_nf_referencia_ids'))

            obj_fnc.write(cr, uid, i, vals, context)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
