# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from funcoes import wizspc_get_produto


class wizard_substitui_natureza_receita_pis_cofins(osv.TransientModel):
    # prefixo: wizspc_

    _name = 'wizard.substitui.natureza.receita.pis.cofins'

    _columns = {
        'wizspc_data_inicial': fields.date('Data incial'),
        'wizspc_data_final': fields.date('Data final'),
        'wizspc_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'wizspc_ncm_id': fields.many2one('fiscal.ncm', 'NCM'),
        'wizspc_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizspc_comprador_id': fields.many2one('compras.comprador', 'Comprador'),
        'wizspc_fabricante_id': fields.many2one('adm.pessoa', 'Fabricante'),
        'wizspc_produto_ids': fields.function(wizspc_get_produto, method=True, string='Produtos',
                                              type='one2many', relation='estoque.produto'),
        'wizspc_executado': fields.boolean('Executado'),
    }
    _defaults = {
        'wizspc_executado': False
    }

    def wizspc_button_find(self, cr, uid, ids, context=None):
        """
        O metodo forca a execucao do metodo funcional 'wizspc_get_produto' que trara para a tela os
        produtos encontrados.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'wizspc_executado': False}, context)
        return True

    def wizspc_button_substitui_natureza_receita_movimento(self, cr, uid, ids, context=None):
        """
        O metodo substitui o valor na coluna 'fisnit_nat_rec_pis_cofins' da tabela
        'fiscal_nf_item_tributacao' pelo valor que se encontra registrado no cadastro do produto
        ('estoque_produto'), nas colunas 'estpro_cofins_nat_rec' e 'estpro_pis_nat_rec'.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: Dados com acao para abertura de uma nova janela para exibir ao usuario se atualizou
        ou nao nao alguma movimentacao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = False
        brw_produto_ids = list()
        data_inicial = ''
        data_final = ''
        for x in self.browse(cr, uid, ids, context):
            brw_produto_ids.extend(x.wizspc_produto_ids)
            data_inicial = x.wizspc_data_inicial
            data_final = x.wizspc_data_final
        brw_produto_ids = list(set(brw_produto_ids))
        for produto_id in brw_produto_ids:
            if not any([produto_id.estpro_pis_nat_rec, produto_id.estpro_cofins_nat_rec]):
                continue
            sql = """
                SELECT
                    fnit.id  AS nf_item_tributacao_id,
                    fe.sigla AS sigla
                FROM
                    estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo fe
                    ON fe.id = fnit.encargo_id
                WHERE
                    emi.produto_id = %s
                    AND fe.sigla IN ('PIS', 'COFINS')
            """ % (produto_id.id)
            if data_inicial:
                sql += """
                    AND emc.data >= '%s'
                """ % data_inicial
            if data_final:
                sql += """
                    AND emc.data <= '%s'
                """ % data_final
            cr.execute(sql)
            nf_item_tributacao_dados = cr.dictfetchall()
            item_tributacao_pis_dados = filter(lambda i: i.get('sigla') == 'PIS',
                                               nf_item_tributacao_dados)
            item_tributacao_cofins_dados = filter(lambda i: i.get('sigla') == 'COFINS',
                                                  nf_item_tributacao_dados)
            sql = """
                UPDATE
                    fiscal_nf_item_tributacao
                SET
                    fisnit_nat_rec_pis_cofins = '%s'
                WHERE
                    id IN %s
            """
            if item_tributacao_pis_dados and produto_id.estpro_pis_nat_rec:
                nf_item_tributacao_ids = map(lambda i: i.get('nf_item_tributacao_id'),
                                             item_tributacao_pis_dados)
                if len(nf_item_tributacao_ids) == 1:
                    nf_item_tributacao_ids = str(tuple(nf_item_tributacao_ids)).replace(',', '')
                elif len(nf_item_tributacao_ids) > 1:
                    nf_item_tributacao_ids = str(tuple(nf_item_tributacao_ids))
                sql_pis = sql % (produto_id.estpro_pis_nat_rec, nf_item_tributacao_ids)
                cr.execute(sql_pis)
                res = True
            if item_tributacao_cofins_dados and produto_id.estpro_cofins_nat_rec:
                nf_item_tributacao_ids = map(lambda i: i.get('nf_item_tributacao_id'),
                                             item_tributacao_cofins_dados)
                if len(nf_item_tributacao_ids) == 1:
                    nf_item_tributacao_ids = str(tuple(nf_item_tributacao_ids)).replace(',', '')
                elif len(nf_item_tributacao_ids) > 1:
                    nf_item_tributacao_ids = str(tuple(nf_item_tributacao_ids))
                sql_cofins = sql % (produto_id.estpro_cofins_nat_rec, nf_item_tributacao_ids)
                cr.execute(sql_cofins)
                res = True
        self.write(cr, uid, ids, {'wizspc_executado': True}, context)
        obj_model_data = self.pool.get('ir.model.data')
        vals = {
            'name': 'Atencao',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.substitui.natureza.receita.pis.cofins',
            'target': 'new',
            'type': 'ir.actions.act_window',
        }
        if res:
            reference = obj_model_data.get_object_reference(
                cr, uid, 'fiscal', 'view_wizard_substitui_natureza_receita_pis_cofins_form_sim')
            view_id = reference and reference[1] or False
            vals.update({'view_id': [view_id]})
            return vals
        else:
            reference = obj_model_data.get_object_reference(
                cr, uid, 'fiscal', 'view_wizard_substitui_natureza_receita_pis_cofins_form_nao')
            view_id = reference and reference[1] or False
            vals.update({'view_id': [view_id]})
            return vals
