# -*- encoding: utf-8 -*-
from openerp.osv import osv, fields
from datetime import date
from openerp.tools.translate import _


class estoque_produto_osv(osv.TransientModel):

    def get_data_pedido(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_fornecedor = self.pool.get('estoque.fornecedor')

        args = [('produto_id', '=', produto_id)]
        fornecedor_ids = obj_estoque_fornecedor.search(cr, uid, args, context=context)
        list_datas = []
        for fornecedor in obj_estoque_fornecedor.browse(cr, uid, fornecedor_ids, context):
            if not fornecedor.fornecedor_id.pessoa_id:
                continue
            list_datas.append(obj_adm_utils.ultima_data_pedido(
                cr, uid, ids, produto_id, fornecedor.fornecedor_id.pessoa_id.id))
        list_datas.sort(reverse=True)
        return list_datas[0] if list_datas else False

    def onchange_produto(self, cr, uid, ids, produto_id, custo, tipo, context=None):
        """
        Recupera os dados do produto para simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} ID do produto,
        @param custo: {char} custo a considerar no calculo da simulacao.
               'U' = 'ultimo_custo', 'M' = 'custo_medio_contabil', 'R' = 'custo_reposicao' e
               'C' = 'custo_corrigido'
        @param tipo:
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} dicionario com os valores do produto.

        @change: 16.01.2014 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        read_load = '_classic_write'
        result = {'value': {'produto_id': produto_id}}

        # Mostra a grade do produto se for grade unica
        campos = ['grade_usar']
        dados_produto = obj_estoque_produto.read(
            cr, uid, produto_id, campos, context=context, load=read_load)

        if dados_produto and isinstance(dados_produto, list):
            dados_produto = dados_produto[0]

        if dados_produto and isinstance(dados_produto, dict):
            args = [('produto_id', '=', produto_id)]
            grade_ids = obj_estoque_produto_grade.search(cr, uid, args, context=context)
            grade_id = grade_ids[0] if grade_ids else False

            if dados_produto.get('grade_usar', False) and len(grade_ids) != 1:                      # Produto com uma ou mais grades
                grade_id = False
        else:
            grade_id = False

        result['value']['grade_id'] = grade_id

        # Mostra a embalagem do produto se for unica
        args = [('produto_id', '=', produto_id)]
        embalagem_ids = obj_estoque_produto_embalagem.search(cr, uid, args, context=context)
        embalagem_id = embalagem_ids[0] if embalagem_ids else False

        if len(embalagem_ids) != 1:
            embalagem_id = False

        result['value']['embalagem_id'] = embalagem_id
        result['value']['custo'] = custo

        context['tipo'] = tipo                                                                      # Tipo: ´´V´´ = Venda ou ´´C´´ = Compra

        result['value'].update(self.dados_simulacao_get(
            cr, uid, produto_id, grade_id, embalagem_id, custo, context))

        return result

    def onchange_grade(self, cr, uid, ids, produto_id, grade_id, custo, tipo, context=None):
        """
        Recupera os dados do produto para simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} ID do produto,
        @param grade_id: {int} ID da grade do produto,
        @param custo: {char} custo a considerar no calculo da simulacao.
               'U' = 'ultimo_custo', 'M' = 'custo_medio_contabil', 'R' = 'custo_reposicao' e
               'C' = 'custo_corrigido'
        @param tipo:
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} dicionario com os valores do produto.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        result = {'value': {'produto_id': produto_id, 'grade_id': grade_id}}

        # Mostra a embalagem do produto se for unica
        args = [('produto_id', '=', produto_id)]
        embalagem_ids = obj_estoque_produto_embalagem.search(cr, uid, args, context=context)
        embalagem_id = embalagem_ids[0] if embalagem_ids else False

        if len(embalagem_ids) != 1:
            embalagem_id = False

        result['value']['embalagem_id'] = embalagem_id
        result['value']['custo'] = custo

        context['tipo'] = tipo                                                                      # Tipo: ´´V´´ = Venda ou ´´C´´ = Compra

        result['value'].update(self.dados_simulacao_get(
            cr, uid, produto_id, grade_id, embalagem_id, custo, context))

        return result

    def onchange_embalagem(self, cr, uid, ids, produto_id, grade_id, embalagem_id, custo, tipo,
                           context=None):
        """
        Recupera os dados do produto para simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} ID do produto,
        @param grade_id: {int} ID da grade do produto,
        @param embalagem_id: {int} ID da embalagem,
        @param custo: {char} custo a considerar no calculo da simulacao.
               'U' = 'ultimo_custo', 'M' = 'custo_medio_contabil', 'R' = 'custo_reposicao' e
               'C' = 'custo_corrigido'
        @param tipo:
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} dicionario com os valores do produto.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {'produto_id': produto_id, 'grade_id': grade_id,
                            'embalagem_id': embalagem_id, 'custo': custo}}

        context['tipo'] = tipo                                                                      # Tipo: ´´V´´ = Venda ou ´´C´´ = Compra

        result['value'].update(self.dados_simulacao_get(
            cr, uid, produto_id, grade_id, embalagem_id, custo, context))

        return result

    def onchange_custo(self, cr, uid, ids, produto_id, grade_id, embalagem_id, custo, tipo,
                       context=None):
        """
        Recupera os dados do produto para simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} ID do produto,
        @param grade_id: {int} ID da grade do produto,
        @param embalagem_id: {int} ID da embalagem,
        @param custo: {char} custo a considerar no calculo da simulacao.
               'U' = 'ultimo_custo', 'M' = 'custo_medio_contabil', 'R' = 'custo_reposicao' e
               'C' = 'custo_corrigido'
        @param tipo:
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} dicionario com os valores do produto.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {'produto_id': produto_id, 'grade_id': grade_id,
                            'embalagem_id': embalagem_id, 'custo': custo}}

        context['tipo'] = tipo  # Tipo: ´´V´´ = Venda ou ´´C´´ = Compra

        result['value'].update(self.dados_simulacao_get(
            cr, uid, produto_id, grade_id, embalagem_id, custo, context))

        return result

    def dados_simulacao_get(self, cr, uid, produto_id, grade_id, embalagem_id,  custo, context):
        """
        Recupera os dados do produto necessarios para os calculos da simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param produto_id: {int} ID do produto,
        @param grade_id: {int} ID da grade do produto,
        @param embalagem_id {int} ID da embalagem do produto,
        @param custo: {char} custo a considerar no calculo da simulacao.
               'U' = 'ultimo_custo', 'M' = 'custo_medio_contabil', 'R' = 'custo_reposicao' e
               'C' = 'custo_corrigido'
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return {dict} dicionario com os valores a serem utilizados na simulacao.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects ________________________________
        obj_est_prod = self.pool.get('estoque.produto')
        obj_prod_emb = self.pool.get('estoque.produto.embalagem')
        obj_emb = self.pool.get('estoque.embalagem')
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        obj_fiscal_produto_tribut_osv = self.pool.get('fiscal.produto.tribut.osv')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_saldo = self.pool.get('estoque.produto.saldo')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        obj_preco_venda = self.pool.get('estoque.preco.venda')
        obj_adm_moeda = self.pool.get('adm.moeda')
        obj_estoque_utils = self.pool.get('estoque.utils')

        result = {}
        read_load = '_classic_write'

        # Verifica os parametros
        if not all((produto_id, grade_id, embalagem_id, custo)):
            return result

        # TODO: Deve ser movido para ´´wizard´´ proprio de simulacao de compra
        # Verifica se o tipo de simulacao e compra ou venda
        if context.get('tipo', 'V') == 'C':
            list_movimento_itens = []
            args = [('produto_id', '=', produto_id), ('nf_id', '!=', False)]
            movimento_itens_ids = obj_estoque_movimento_itens.search(
                cr, uid, args, order='create_date DESC', context=context)

            if movimento_itens_ids:
                browse_movimento_itens = obj_estoque_movimento_itens.browse(
                    cr, uid, movimento_itens_ids)
                for item in browse_movimento_itens:
                    if item.natureza_id:
                        if item.natureza_id.tipo == 'E':
                            list_movimento_itens.append(item.id)

            result['estoque_movimento_itens_ids'] = list_movimento_itens

        # TODO: Deve ser movido para o modulo de concorrencia utilizando heranca
        if obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'concorrencia', context):
            # ___________________________ Objects ________________________________
            obj_concorrencia_concorrente_item = self.pool.get('concorrencia.concorrente.item')
            obj_estoque_produto_concorrente_osv = self.pool.get('estoque.produto.concorrente.osv')

            itens = []
            args = [('produto_id', '=', produto_id)]
            concorrente_item_ids = obj_estoque_produto_concorrente_osv.search(cr, uid, args, context=context)
            obj_estoque_produto_concorrente_osv.unlink(cr, uid, concorrente_item_ids)

            concorrente_item_ids = obj_concorrencia_concorrente_item.search(cr, uid, args, context=context)
            brw_concorrente_item = obj_concorrencia_concorrente_item.browse(
                cr, uid, concorrente_item_ids)

            for item in brw_concorrente_item:
                vals_concorrente = {'company_id': item.company_id.id, 'data': item.data or '',
                                    'preco': item.preco or 0, 'produto_id': item.produto_id.id,
                                    'grade_id': item.grade_id.id,
                                    'pessoa_id': item.concorrente_id.pessoa_id.id,
                                    'usa_grade_ou_nao': item.usa_grade_ou_nao or False}

                itens.append(obj_estoque_produto_concorrente_osv.create(
                    cr, uid, vals_concorrente, context))

            result['concorrente_item_ids'] = itens

        # Margens do produto
        margens = obj_est_prod.estpro_get_margens(cr, uid, produto_id)
        result['marg_minima'] = margens.get(produto_id, {}).get('margem_minima', 0)
        result['marg_bruta'] = margens.get(produto_id, {}).get('margem_bruta', 0)
        result['marg_ideal'] = margens.get(produto_id, {}).get('margem_ideal', 0)

        # Saldo do produto
        result.update(obj_saldo.get_saldo_do_dia(cr, uid, grade_id, str(date.today()),
                                                 False, context=context).get(grade_id, {}))

        # TODO: Deve ser movido para ´´wizard´´ proprio de simulacao de compra
        # Verifica se o tipo de simulacao e compra ou venda
        if context.get('tipo', 'V') == 'C':
            saldo_total = obj_saldo.get_saldo_do_dia(
                cr, uid, grade_id, str(date.today()), False, context=context)
            result['saldo_total'] = saldo_total.get(grade_id).get('saldo', 0)

        # Quantidade na embalagem
        quant_emb = 1                                                                               # Padrao = 1
        campos = ['embalagem_id']
        dados_embalagem = obj_prod_emb.read(cr, uid, embalagem_id, campos, context=context, load=read_load)

        if dados_embalagem and isinstance(dados_embalagem, list):
            dados_embalagem = dados_embalagem[0]

        if dados_embalagem and isinstance(dados_embalagem, dict):
            id_prod_emb = dados_embalagem.get('embalagem_id', 0)

            campos = ['quantidade_na_embalagem']
            dados_embalagem = obj_emb.read(cr, uid, id_prod_emb, campos, context=context, load=read_load)

            if dados_embalagem and isinstance(dados_embalagem, list):
                dados_embalagem = dados_embalagem[0]

            if dados_embalagem and isinstance(dados_embalagem, dict):
                quant_emb = dados_embalagem.get('quantidade_na_embalagem', 0)

        # Preco custo
        registro_preco_custo = obj_estoque_preco_custo.get_custos_do_dia(
            cr, uid, produto_id).get(produto_id, {})
        ultimo_custo = registro_preco_custo.get('estprc_ultimo_custo', 0) * quant_emb
        custo_medio = registro_preco_custo.get('estprc_custo_medio_contabil', 0) * quant_emb
        custo_corrigido = registro_preco_custo.get('estprc_custo_corrigido', 0) * quant_emb
        custo_reposicao = registro_preco_custo.get('estprc_custo_reposicao', 0) * quant_emb
        result['ultimo_custo'] = ultimo_custo
        result['custo_medio'] = custo_medio
        result['custo_corrigido'] = custo_corrigido
        result['custo_reposicao'] = custo_reposicao

        # Preco venda
        preco_venda = obj_preco_venda.get_preco_venda(cr, uid, produto_id, epc_grade_ids=grade_id)
        dados_preco_venda = preco_venda.get(produto_id) or {}
        valor_preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
            'preco_venda')
        preco_venda = valor_preco_venda * quant_emb
        result['preco'] = preco_venda

        juros = 0.0
        comissoes = 0.0
        custo_fixo = 0.0
        moeda_valor_ids = []

        # Parametros
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)  # Necessario para busca dos parametros da empresa
        parametro_ids = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
        parametro_id = parametro_ids[0] if parametro_ids else False
        campos = ['juros', 'comissoes', 'custo_fixo', 'indexador_id']
        dados_parametro = obj_estoque_parametro.read(cr, uid, parametro_id, campos, context=context, load=read_load)

        if dados_parametro and isinstance(dados_parametro, list):
            dados_parametro = dados_parametro[0]

        if dados_parametro and isinstance(dados_parametro, dict):
            juros = dados_parametro.get('juros', 0)
            comissoes = dados_parametro.get('comissoes', 0)
            custo_fixo = dados_parametro.get('custo_fixo', 0)
            indexador_id = dados_parametro.get('indexador_id', False)

            if indexador_id:
                args = [('id', '=', indexador_id)]
                moeda_valor_ids = obj_adm_moeda.search(cr, uid, args, context=context)

        # Data da ultima movimentacao (compra) do produto
        ult_comp = self.get_data_pedido(cr, uid, False, produto_id)
        result['ultima_compra'] = ult_comp

        result['juros'] = juros
        result['comissoes'] = comissoes
        result['custo_fixo'] = custo_fixo
        tipo_tributacao = 'T'

        # TODO: Deve ser movido para ´´wizard´´ proprio de simulacao de compra
        # Verifica se o tipo de simulacao e compra ou venda
        if context.get('tipo', 'V') == 'C':
            tipo_tributacao = 'E'

        parametro = obj_estoque_parametro.browse(cr, uid, parametro_id)

        # TODO: Deve ser movido para ´´wizard´´ proprio de simulacao de compra
        # Verifica se o tipo de simulacao e compra ou venda
        if context.get('tipo', 'V') == 'C':
            custo = parametro.estpar_custo_para_precovenda

        if custo == 'U':
            preco_custo = ultimo_custo
        elif custo == 'C':
            if not moeda_valor_ids:
                msg = _('Idexador de moedas nao esta corretamente cadastrado.')
                result['warning'] = {'title': _('Aviso.'), 'message': msg}

            preco_custo = custo_corrigido
        elif custo == 'M':
            preco_custo = custo_medio
        elif custo == 'R':
            preco_custo = custo_reposicao
        else:
            preco_custo = 0

        # TODO: Verificar a possibilidade de NAO utilizar a tabela de tributos (obj_fiscal_produto_tribut_osv) por ser uma tabela temporaria
        # Tibutacoes
        ids_fiscal_produto_tribut_osv = obj_fiscal_produto_tribut_osv.search(cr, uid, [], context=context)
        obj_fiscal_produto_tribut_osv.unlink(cr, uid, ids_fiscal_produto_tribut_osv)

        result['fiscal_tributacao_ids'] = []
        total_encargos = 0
        values = {}

        args = [('produto_id', '=', produto_id), ('tipo_tributacao', '=', tipo_tributacao)]
        tributacao_ids = obj_fiscal_tributacao_encargos.search(cr, uid, args, context=context)
        browse_tributacao_encargos = obj_fiscal_tributacao_encargos.browse(cr, uid, tributacao_ids)

        # TODO: Retornar os valores diretamente sem uso da tabela temporaria
        for encargos in browse_tributacao_encargos:
            aliquota_tributacao = encargos.tributacao_id.aliquota or 0.0
            base = encargos.tributacao_id.base or 0.0
            encargo = aliquota_tributacao * base / 100
            total_encargos += encargo

            values['produto_id'] = produto_id
            values['encargo_id'] = encargos.encargo_id.id
            values['cst_id'] = encargos.cst_id.id
            values['tributacao_id'] = encargos.tributacao_id.id
            values['mva'] = encargos.mva
            values['valor_pauta'] = encargos.valor_pauta
            values['modbcst'] = encargos.modbcst
            values['codigo_cst'] = encargos.cst_id.codigo
            values['aliquota'] = encargos.tributacao_id.aliquota
            values['base'] = encargos.tributacao_id.base
            tributacao_osv_id = obj_fiscal_produto_tribut_osv.create(cr, uid, values, context)
            result['fiscal_tributacao_ids'].append(tributacao_osv_id.__int__())

        # Margem contribuicao
        total_encargos += comissoes + custo_fixo
        result['margem_contrib'] = obj_estoque_utils.estutl_calcular_margem(
            preco_venda, total_encargos, preco_custo)

        # Margem praticada
        total_encargos -= custo_fixo
        result['margem_praticada'] = obj_estoque_utils.estutl_calcular_margem(
            preco_venda, total_encargos, preco_custo)

        return result

    def button_calcular_simulacao(self, cr, uid, ids, context=None):
        """
        Calculo do lucro em funcao do preco.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return:

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = False

        if not ids:
            return result

        dados_simulacao = self.read(cr, uid, ids, ['simulacao_tipo'], context=context)

        if dados_simulacao and isinstance(dados_simulacao, list):
            dados_simulacao = dados_simulacao[0]

        if dados_simulacao and isinstance(dados_simulacao, dict):
            if dados_simulacao.get('simulacao_tipo') == 'P':                                        # Preco em funcao do lucro
                values = self._calcula_preco_venda(cr, uid, ids, context)
                values = values.get(ids[0], {})
            elif dados_simulacao.get('simulacao_tipo') == 'L':                                      # % lucro em funcao do preco
                values = self._calcula_percentual_lucro(cr, uid, ids, context)
                values = values.get(ids[0], {})
            else:
                values = {}

            result = self.write(cr, uid, ids, values, context)

        return result

    def onchange_simulacao_tipo(self, cr, uid, ids, tipo, context=None):
        """
        Verifica qual sera o tipo de simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param tipo: {char} o tipo da simulacao. "P" = Preco ou "L" = % de Lucro,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} dicionario com os valores de preco de venda ou % de lucro zerado em funcao
        do tipo de simulacao.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if tipo == 'P':
            values = {'lucros': 0}
        elif tipo == 'L':
            values = {'preco_de_venda': 0}
        else:
            values = {}

        result = self.write(cr, uid, ids, values, context)

        return result

    def _calcula_preco_venda(self, cr, uid, ids, context=None):
        """
        Calculo do preco de venda.
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_utils = self.pool.get('estoque.utils')

        result = {}

        for simulacao in self.browse(cr, uid, ids, context=context):
            custo = simulacao.selecionar_custo

            if custo == 'U':
                preco_custo = simulacao.ultimo_custo
            elif custo == 'C':
                preco_custo = simulacao.custo_corrigido
            elif custo == 'M':
                preco_custo = simulacao.custo_medio
            elif custo == 'R':
                preco_custo = simulacao.custo_reposicao
            else:
                preco_custo = 0

            comissao = simulacao.comissoes
            custo_fixo = simulacao.custo_fixo
            encargos = sum(map(lambda t: t.tributacao_id.aliquota * t.tributacao_id.base / 100,
                               simulacao.fiscal_tributacao_ids))
            encargos += comissao + custo_fixo
            lucros = simulacao.lucros                                                                # % Lucros

            preco_venda = obj_estoque_utils.estutl_calcular_preco_de_venda(
                preco_custo, encargos, lucros)

            result.update({simulacao.id: {'preco_de_venda': preco_venda}})

        return result

    def _calcula_percentual_lucro(self, cr, uid, ids, context=None):

        """
        Calculo do % de lucro.
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_utils = self.pool.get('estoque.utils')

        result = {}

        for simulacao in self.browse(cr, uid, ids, context=context):
            custo = simulacao.selecionar_custo

            if custo == 'U':
                preco_custo = simulacao.ultimo_custo
            elif custo == 'C':
                preco_custo = simulacao.custo_corrigido
            elif custo == 'M':
                preco_custo = simulacao.custo_medio
            elif custo == 'R':
                preco_custo = simulacao.custo_reposicao
            else:
                preco_custo = 0

            comissao = simulacao.comissoes
            custo_fixo = simulacao.custo_fixo
            encargos = sum(map(lambda t: t.tributacao_id.aliquota * t.tributacao_id.base / 100,
                               simulacao.fiscal_tributacao_ids))
            encargos += comissao + custo_fixo
            preco_de_venda = simulacao.preco_de_venda
            lucros = obj_estoque_utils.estutl_calcular_margem(preco_de_venda, encargos, preco_custo)

            result.update({simulacao.id: {'lucros': lucros}})

        return result

    # TODO: Deve ser movido para ´´wizard´´ proprio de simulacao de compra
    # def button_encargo(self, cr, uid, ids, context=None):
    #     self.write(cr, uid, ids, {'button_encargo': True})
    #     return True

    def _get_local(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        estoque_parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
        local_venda_padrao_id = obj_estoque_parametro.read(cr, uid, estoque_parametro_id, ['local_venda_padrao_id'], context=context)
        if local_venda_padrao_id and isinstance(local_venda_padrao_id, list):
            local_venda_padrao_id = local_venda_padrao_id[0].get('local_venda_padrao_id')
            if isinstance(local_venda_padrao_id, tuple):
                local_venda_padrao_id = local_venda_padrao_id[0]
        return local_venda_padrao_id

    def _get_concorrentes(self, cr, uid, ids, name, arg, context=None):
        res = {}
        context = context and context.copy() or {}
        obj_concorrencia_concorrente_item = self.pool.get('concorrencia.concorrente.item')
        browse_produto_osv = self.browse(cr, uid, ids[0])
        produto_id = browse_produto_osv.produto_id.id
        itens = []
        modulo_concorrencia_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'concorrencia', context={})
        if modulo_concorrencia_instalado:
            obj_estoque_produto_concorrente_osv = self.pool.get('estoque.produto.concorrente.osv')
            concorrente_item_ids = obj_estoque_produto_concorrente_osv.search(cr, uid,
                                                                              [('produto_id', '=', produto_id)],
                                                                              context=context)
            if concorrente_item_ids:
                obj_estoque_produto_concorrente_osv.unlink(cr, uid, concorrente_item_ids)
            concorrente_item_ids = obj_concorrencia_concorrente_item.search(cr, uid, [('produto_id', '=', produto_id)],
                                                                            context=context)
            browse_concorrente_item = obj_concorrencia_concorrente_item.browse(cr, uid, concorrente_item_ids)
            for item in browse_concorrente_item:
                vals_concorrente = {
                    'produto_osv_id': ids[0],
                    'company_id': item.company_id.id,
                    'data': item.data or '',
                    'preco': item.preco or 0,
                    'produto_id': item.produto_id.id,
                    'grade_id': item.grade_id.id,
                    'pessoa_id': item.concorrente_id.pessoa_id.id,
                    'usa_grade_ou_nao': item.usa_grade_ou_nao or False,
                }
                id_concorrente = obj_estoque_produto_concorrente_osv.create(cr, uid, vals_concorrente, context)
                itens.append(id_concorrente)
        res[ids[0]] = itens
        return res

    def _get_movimentos(self, cr, uid, ids, name, arg, context=None):
        res = {}
        context = context and context.copy() or {}
        list_movimento_itens = []
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        browse_produto_osv = self.browse(cr, uid, ids[0])
        produto_id = browse_produto_osv.produto_id.id

        movimento_itens_ids = obj_estoque_movimento_itens.search(cr, uid, [('produto_id', '=', produto_id),
                                                                           ('nf_id', '!=', False)],
                                                                 order='create_date DESC', context=context)
        if movimento_itens_ids:
            browse_movimento_itens = obj_estoque_movimento_itens.browse(cr, uid, movimento_itens_ids)
            for x in browse_movimento_itens:
                if x.natureza_id:
                    if x.natureza_id.tipo == 'E':
                        list_movimento_itens.append(x.id)
        res[ids[0]] = list_movimento_itens
        return res

    def default_get(self, cr, uid, fields, context=None):
        context = context and context.copy() or {}
        res = super(estoque_produto_osv, self).default_get(cr, uid, fields, context)
        if context.get('compra'):
            res['tipo'] = 'C'
        else:
            res['tipo'] = 'V'
        modulo_concorrencia_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'concorrencia', context={})
        if modulo_concorrencia_instalado:
            res['modulo_concorrencia_instalado'] = True
        obj_adm_utils = self.pool.get('adm.utils')
        dados_parametro = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque',
                                                              ['estpar_custo_para_precovenda'])
        custo_precovenda = dados_parametro.get('estpar_custo_para_precovenda')
        if custo_precovenda:
            res['selecionar_custo'] = custo_precovenda
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}

        produto_id = vals.get('produto_id', False)
        grade_id = vals.get('grade_id', False)
        embalagem_id = vals.get('embalagem_id', False)
        custo = vals.get('selecionar_custo', 'C')

        vals.update(self.dados_simulacao_get(cr, uid, produto_id, grade_id, embalagem_id, custo,
                                              context))
        vals['fiscal_tributacao_ids'] = map(lambda v: (4, v), vals.get('fiscal_tributacao_ids', []))
        result = super(estoque_produto_osv, self).create(cr, uid, vals, context)

        return result

    _name = 'estoque.produto.osv'
    _columns = {
        'company_id': fields.many2one("res.company", 'Empresa ', ondelete="cascade", required=True, select=True), # id da empresa
        'codigo': fields.char('Codigo', size=20, required=False, select=True), # Codigo de identificacao do produto
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="restrict", required=True, select=True,
                                      domain="[('company_id', '=', company_id)]"), # Nome Produto
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', required=True, ondelete="restrict", select=True,
                                    domain="[('produto_id', '=', produto_id)]"),
        'local_id': fields.many2one("estoque.local", 'Local', ondelete="restrict", required=True, select=True),
        # Id do local requisitado
        'embalagem_id': fields.many2one("estoque.produto.embalagem", 'Embalagem', ondelete="restrict", required=True,
                                        domain="[('produto_id', '=', produto_id)]", select=True), # identificacao da embalagem / produto\n
        'marg_ideal': fields.float('Margem Ideal', digits=(7, 3)),
        # Percentual da margem de lucro que o cliente deseja obter na venda do produto. Sera utilizado para calcular o preco de venda. Informar margem liquida, sem encargos e impostos.
        'marg_minima': fields.float('Margem Minima', digits=(7, 3)),
        # Percentual da margem minima de lucro que o cliente deseja objeter na venda deste produto. Sera utilizado em relatorio para analise e correcao de preco de venda
        'marg_bruta': fields.float('Margem Bruta', digits=(7, 3)),
        # Percentual de margem bruta. Nao tem importancia dentro do sistema. Simplismente informativa.
        'ultimo_custo': fields.float('Ultimo custo', digits=(7, 2)),
        'custo_medio': fields.float('Custo medio', digits=(7, 2)),
        'custo_corrigido': fields.float('Custo corrigido', digits=(7, 2)),
        'custo_reposicao': fields.float('Custo reposicao', digits=(7, 2)),
        'saldo': fields.float('Saldo'),
        'saldo_total': fields.float('Saldo total'),
        'ultima_compra': fields.date('Ultima compra', select=True),
        'preco': fields.float('Preco de venda', digits=(7, 3)),
        'juros': fields.float('Juros', digits=(7, 2)),
        'comissoes': fields.float('Comissoes', digits=(7, 3)),
        'custo_fixo': fields.float('Custo fixo', digits=(7, 3)),
        'base_tributada': fields.float('Base tributada', digits=(7, 3)),
        'prazo_de_venda': fields.integer('Prazo de venda'),
        'preco_de_venda': fields.float('Preco de venda', digits=(7, 3)),
        'lucros': fields.float('% Lucro', digits=(7, 3)),
        'selecionar_custo': fields.selection([('U', 'Ultimo custo'), ('C', 'Custo corrigido'),
                                              ('M', 'Medio contabil'), ('R', 'Reposicao')],
                                             'Custo', size=1, required=True),
        'button_preco': fields.boolean('libera preco'),
        'button_encargo': fields.boolean('libera encargo'),
        'button_margem': fields.boolean('libera margem'),
        'margem_praticada': fields.float('Margem praticada', digits=(7, 3)),
        'margem_contrib': fields.float('Margem contribuicao', digits=(7, 3)),
        'fiscal_tributacao_ids': fields.one2many(
            'fiscal.produto.tribut.osv', 'produto_osv_id', 'Tributacao'),
        'tipo': fields.selection([('C', 'Compra'), ('V', 'Venda')], 'Tipo', size=1),                # Simulacao de compras (C), simulacao de venda (V)
        'porcentagem_frete': fields.float('% Frete', digits=(7, 3)),
        'valor_frete': fields.float('Valor Frete', digits=(15, 3)),
        'porcentagem_ipi': fields.float('% IPI', digits=(7, 3)),
        'valor_ipi': fields.float('Valor IPI', digits=(15, 3)),
        'porcentagem_icms': fields.float('% ICMS', digits=(7, 3)),
        'porcentagem_base_icms': fields.float('% base ICMS', digits=(7, 3)),
        'valor_icms': fields.float('Valor ICMS', digits=(15, 3)),
        'porcentagem_pis': fields.float('% PIS', digits=(7, 3)),
        'valor_pis': fields.float('Valor PIS', digits=(15, 3)),
        'porcentagem_cofins': fields.float('% COFINS', digits=(7, 3)),
        'valor_cofins': fields.float('Valor COFINS', digits=(15, 3)),
        'preco_sugestao': fields.float('Sugestao', digits=(15, 3)),
        'concorrente_item_ids': fields.function(_get_concorrentes, method=True, string="Itens", type="one2many",
                                                relation="estoque.produto.concorrente.osv", store=False),
        'estoque_movimento_itens_ids': fields.function(_get_movimentos, method=True, string="Itens", type="one2many",
                                                       relation="estoque.movimento.itens", store=False),
        'modulo_concorrencia_instalado': fields.boolean('Concorrencia instalado'),
        'simulacao_tipo': fields.selection([('P', 'Preco'), ('L', 'Lucro')], size=1, required=True,
                                           string='Tipo de simulacao'),
    }
    _order = 'ultima_compra desc'
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto', context=c),
        'local_id': _get_local,
        'button_preco': lambda *a: False,
        'button_encargo': lambda *a: False,
        'button_margem': lambda *a: False,
        'modulo_concorrencia_instalado': False,
        'simulacao_tipo': lambda *a: 'P',
    }

    def button_calcular(self, cr, uid, ids, context=None):
        obj_estoque_produto = self.pool.get('estoque.produto')

        context = context and context.copy() or {}
        if isinstance(ids, (list, tuple)):
            ids = ids[0]
        browse_produto_osv = self.browse(cr, uid, ids)
        preco_compra_calculo = browse_produto_osv.preco_de_venda or 0.0
        porcentagem_frete = browse_produto_osv.porcentagem_frete or 0.0
        porcentagem_ipi = browse_produto_osv.porcentagem_ipi or 0.0
        porcentagem_icms = browse_produto_osv.porcentagem_icms or 0.0
        porcentagem_base_icms = browse_produto_osv.porcentagem_base_icms or 0.0
        porcentagem_pis = browse_produto_osv.porcentagem_pis or 0.0
        porcentagem_cofins = browse_produto_osv.porcentagem_cofins or 0.0
        prazo_medio = browse_produto_osv.prazo_de_venda or 0
        produto_id = browse_produto_osv.produto_id.id
        juros = browse_produto_osv.juros or 0.0
        custo_fixo = browse_produto_osv.custo_fixo or 0.0
        comissao = browse_produto_osv.comissoes or 0.0
        embalagem_id = browse_produto_osv.embalagem_id.id
        preco_de_venda = browse_produto_osv.preco or 0.0
        if not preco_compra_calculo:
            return False
        else:
            valor_frete = (porcentagem_frete * preco_compra_calculo) / 100
            valor_ipi = (porcentagem_ipi * preco_compra_calculo) / 100
            if porcentagem_base_icms:
                porcentagem_icms = (porcentagem_icms * porcentagem_base_icms) / 100
            valor_icms = (porcentagem_icms * preco_compra_calculo) / 100
            valor_pis = (porcentagem_pis * preco_compra_calculo) / 100
            valor_cofins = (porcentagem_cofins * preco_compra_calculo) / 100

            fabricante_id = browse_produto_osv.produto_id.fabricante_id
            if fabricante_id:
                tipo_atividade = fabricante_id.admpes_jur_tipo_atividade
                if tipo_atividade != 'I':
                    porcentagem_ipi = 0.0
            taxa_juros = (juros / 30) * prazo_medio
            custos = custo_fixo + comissao
            encargos = porcentagem_ipi + porcentagem_icms + porcentagem_pis + porcentagem_cofins + taxa_juros
            encargos = (encargos * preco_compra_calculo) / 100
            preco_compra_calculo -= encargos
            custos = (custos * preco_compra_calculo) / 100
            preco_compra_calculo += custos

            porcentagem_lucro = ((preco_de_venda - encargos - custos - preco_compra_calculo) / preco_de_venda) * 100
            preco_sugestao = obj_estoque_produto.estpro_calcular_preco_venda(cr, uid, produto_id, embalagem_id, 0,
                                                                             preco_compra_calculo, [], 0, False,
                                                                             context=context)

            vals = {
                'valor_frete': valor_frete,
                'valor_ipi': valor_ipi,
                'valor_icms': valor_icms,
                'valor_pis': valor_pis,
                'valor_cofins': valor_cofins,
                'preco_sugestao': preco_sugestao,
                'lucros': porcentagem_lucro,
            }
            self.write(cr, uid, ids, vals, context)
            return True

    def atualiza_dados_readonly(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = {}
        if ids:
            obj_est_prod = self.pool.get('estoque.produto')
            obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
            obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
            obj_estoque_embalagem = self.pool.get('estoque.embalagem')
            obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
            obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
            obj_preco_venda = self.pool.get('estoque.preco.venda')
            obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
            obj_fiscal_produto_tribut_osv = self.pool.get('fiscal.produto.tribut.osv')
            obj_adm_utils = self.pool.get('adm.utils')

            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto', context=context)
            browse_simulacao = self.browse(cr, uid, ids[0])
            produto_id = browse_simulacao.produto_id.id
            embalagem_id = browse_simulacao.embalagem_id.id
            grade_id = browse_simulacao.grade_id.id
            selecionar_custo = browse_simulacao.selecionar_custo or ''
            tipo = browse_simulacao.tipo or ''
            margens = obj_est_prod.estpro_get_margens(cr, uid, produto_id)
            margens = margens.get(produto_id)

            if embalagem_id:
                id_prod_emb = obj_estoque_produto_embalagem.read(cr, uid, embalagem_id,
                                                                 ['embalagem_id'], context=context)
                if id_prod_emb:
                    id_prod_emb = id_prod_emb['embalagem_id'][0]
                quant_emb = obj_estoque_embalagem.read(cr, uid, id_prod_emb,
                                                       ['quantidade_na_embalagem'], context=context)
                if quant_emb:
                    quant_emb = quant_emb['quantidade_na_embalagem']
            else:
                quant_emb = 1

            registro_precocusto = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id,
                                                                            company_id)
            if registro_precocusto.get(produto_id):
                registro_precocusto = registro_precocusto.get(produto_id)
            else:
                registro_precocusto = {}

            custo = registro_precocusto.get('estprc_ultimo_custo', 0.0) * quant_emb
            custo_medio = registro_precocusto.get('estprc_custo_medio_contabil', 0.0) * quant_emb
            custo_corrigido = registro_precocusto.get('estprc_custo_corrigido', 0.0) * quant_emb
            custo_reposicao = registro_precocusto.get('estprc_custo_reposicao', 0.0) * quant_emb

            preco_venda = obj_preco_venda.get_preco_venda(cr, uid, produto_id,
                                                          epc_company_id=company_id,
                                                          epc_grade_ids=grade_id)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            registro_preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')
            preco_venda = registro_preco_venda * quant_emb

            args = [('produto_id', '=', produto_id), ('grade_id', '=', grade_id),
                    ('estllc_produto_embalagem_id', '=', embalagem_id),
                    ('company_id', '=', company_id)]
            local_id = obj_estoque_local_locacao.search(cr, uid, args, context=context)

            if local_id:
                local_id = local_id[0]

            saldo = False

            if isinstance(grade_id, list) and len(grade_id) == 1:
                grade_id = grade_id[0]

            if isinstance(grade_id, (int, long)):
                saldo = obj_estoque_produto_saldo.get_saldo_do_dia(
                    cr, uid, grade_id, str(date.today()), local_id, context=context)
                saldo = saldo.get(grade_id).get('saldo')

            ult_comp = self.get_data_pedido(cr, uid, ids, produto_id)

            if tipo == 'C':
                tipo_tributacao = 'E'
            else:
                tipo_tributacao = 'T'

            args = [('produto_id', '=', produto_id), ('tipo_tributacao', '=', tipo_tributacao)]
            tributacao_ids = obj_fiscal_tributacao_encargos.search(cr, uid, args, context=context)
            browse_tributacao_encargos = obj_fiscal_tributacao_encargos.browse(
                cr, uid, tributacao_ids)
            fiscal_tributacao_ids = []
            soma_vr_encargo = 0
            values = {}
            args = [('produto_osv_id', 'in', ids)]
            tributacao_osv_ids = obj_fiscal_produto_tribut_osv.search(cr, uid, args, context=context)

            if tributacao_osv_ids:
                obj_fiscal_produto_tribut_osv.unlink(cr, uid, tributacao_osv_ids)

            for i in browse_tributacao_encargos:
                aliquota_tributacao = i.tributacao_id.aliquota or 0.0
                base = i.tributacao_id.base or 0.0
                encargo = (aliquota_tributacao * base) / 100
                soma_vr_encargo += encargo

                values['produto_id'] = produto_id
                values['encargo_id'] = i.encargo_id.id
                values['cst_id'] = i.cst_id.id
                values['tributacao_id'] = i.tributacao_id.id
                values['mva'] = i.mva
                values['valor_pauta'] = i.valor_pauta
                values['modbcst'] = i.modbcst
                values['codigo_cst'] = i.cst_id.codigo
                values['aliquota'] = i.tributacao_id.aliquota
                values['base'] = i.tributacao_id.base
                values['produto_osv_id'] = ids[0]
                tributacao_osv_id = obj_fiscal_produto_tribut_osv.create(cr, uid, values, context)
                fiscal_tributacao_ids.append(tributacao_osv_id)

            soma_vr_encargo = (soma_vr_encargo * preco_venda) / 100

            dados_parametro = obj_adm_utils.admutl_get_parametros(
                cr, uid, 'estoque', ['comissoes', 'custo_fixo', 'estpar_custo_para_precovenda'])

            if tipo == 'C':
                custo_para_precovenda = dados_parametro.get('estpar_custo_para_precovenda')
                if custo_para_precovenda == 'U':
                    custo_para_precovenda = 'estprc_ultimo_custo'
                elif custo_para_precovenda == 'M':
                    custo_para_precovenda = 'estprc_custo_medio_contabil'
                elif custo_para_precovenda == 'R':
                    custo_para_precovenda = 'esstprc_custo_reposicao'
                elif custo_para_precovenda == 'C':
                    custo_para_precovenda = 'estprc_custo_corrigido'

                if custo_para_precovenda:
                    preco_custo = registro_precocusto.get(custo_para_precovenda, 0.0)
                else:
                    preco_custo = 0.0
            else:
                if selecionar_custo == 'U':
                    preco_custo = custo or 0.0
                elif selecionar_custo == 'C':
                    preco_custo = custo_corrigido or 0.0
                elif selecionar_custo == 'M':
                    preco_custo = custo_medio or 0.0
                elif selecionar_custo == 'R':
                    preco_custo = custo_reposicao or 0.0

            comissoes = dados_parametro.get('comissoes') or 0
            custo_fixo = dados_parametro.get('custo_fixo') or 0
            b_comissao = (comissoes * preco_venda) / 100
            b_custo_fixo = (custo_fixo * preco_venda) / 100
            margem_praticada = 0
            margem_contrib = 0
            if preco_venda > 0 and selecionar_custo:
                    margem_praticada = (preco_venda - soma_vr_encargo - b_comissao - b_custo_fixo -
                                        preco_custo) / preco_venda * 100
                    margem_contrib = (preco_venda - soma_vr_encargo - b_comissao -
                                      preco_custo) / preco_venda * 100
            vals = {
                'button_preco': True, 'button_margem': False, 'lucros': 0, 'preco_de_venda': 0,
                'marg_minima': margens.get('marg_minima'),
                'marg_bruta': margens.get('marg_bruta'), 'marg_ideal': margens.get('marg_ideal'),
                'ultimo_custo': custo, 'custo_medio': custo_medio, 'custo_corrigido': custo_corrigido,
                'saldo': saldo, 'ultima_compra': ult_comp, 'preco': preco_venda,
                'margem_praticada': margem_praticada, 'margem_contrib': margem_contrib,
            }
        return vals

class estoque_produto_concorrente_osv(osv.TransientModel):
    """
    Cadastro de itens de um concorrente
    """
    _name = 'estoque.produto.concorrente.osv'
    _columns = {
        'produto_osv_id': fields.many2one('estoque.produto.osv', 'Simulacao'),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # Id da empresa
        'data': fields.date('Data', required=True), # data da coleta de precos
        'preco': fields.float('Preco', digits=(14, 2), required=True), # preco praticado pelo concorrente
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="cascade", required=True, select=True,
                                      domain="[('company_id', '=', company_id)]"), # Id do produto
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', ondelete="cascade", required=False, select=True,
                                    domain="[('produto_id', '=', produto_id)]"), # id da grade/produto
        'pessoa_id': fields.many2one("adm.pessoa", 'Concorrente', ondelete="cascade", required=True, select=True,
                                     domain="[('company_id', '=', company_id)]"), # id do concorrente
        'usa_grade_ou_nao': fields.boolean('usa grade ou nao')
    }
    _order = 'data desc'


