# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from dateutil.parser import parse
from openerp.tools.translate import _


class wizard_verifica_sped(osv.TransientModel):

    _name = 'wizard.verifica.sped'
    _columns = {
        'data_inicial': fields.date('Data inicial', required=True),
        'data_final': fields.date('Data final', required=True)
    }

    def button_print_inconsistencia_sped(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        verifica_sped_id = ids[0] if isinstance(ids, (list, tuple)) else ids
        brw_verifica_sped = self.browse(cr, uid, verifica_sped_id, context)
        data_inicial = brw_verifica_sped.data_inicial
        data_final = brw_verifica_sped.data_final

        dados_inconsistencia_sped = self.get_dados_inconsistencia_sped(
            cr, uid, data_inicial, data_final, context)

        if not any(dados_inconsistencia_sped.values()):
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para o periodo selecionado.'))

        dados = {'data_inicial': parse(data_inicial).strftime('%d/%m/%Y'),
                 'data_final': parse(data_final).strftime('%d/%m/%Y')}

        dados.update(dados_inconsistencia_sped)

        return {'type': 'ir.actions.report.xml',
                'report_name': 'verifica_inconsistencia_sped',
                'datas': dados}

    def get_dados_inconsistencia_sped(self, cr, uid, data_inicial, data_final, context=None):
        context = context and context.copy() or {}

        # CST de PIS diferente do COFINS no movimento
        query_cst_pist_diferente_cst_cofins = self.get_query_cst_pis_diferente_cst_cofins(
                cr, uid, data_inicial, data_final, context)
        cr.execute(query_cst_pist_diferente_cst_cofins)
        dados_cst_pis_diferente_cst_cofins = cr.dictfetchall()

        # Tributacao divergente do CST (exemplo CST50 e tributacao OUTROS)
        query_tributacao_diferente_cst = self.get_query_tributacao_diferente_cst(
                cr, uid, data_inicial, data_final, context)
        cr.execute(query_tributacao_diferente_cst)
        dados_tributacao_diferente_cst = cr.dictfetchall()

        # CST de entrada para movimento de saida
        query_cst_entrada_movimento_saida = self.get_query_cst_entrada_movimento_saida(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_cst_entrada_movimento_saida)
        dados_cst_entrada_movimento_saida = cr.dictfetchall()

        # CST de saida para movimento de entrada
        query_cst_saida_movimento_entrada = self.get_query_cst_saida_movimento_entrada(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_cst_saida_movimento_entrada)
        dados_cst_saida_movimento_entrada = cr.dictfetchall()

        # Soma das bases de cálculo batem com o total da nota fiscal
        query_diferenca_base_calculo_total_nf = self.get_query_diferenca_base_calculo_total_nf(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_diferenca_base_calculo_total_nf)
        dados_diferenca_base_calculo_total_nf = cr.dictfetchall()

        # CST nao tributado com valor (base e imposto de PIS/COFINS)
        query_cst_nao_tributado_com_valor = self.get_query_cst_nao_tributado_com_valor(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_cst_nao_tributado_com_valor)
        dados_cst_nao_tributado_com_valor = cr.dictfetchall()

        # CFOP nao tributado gerando valor (base e imposto de PIS/COFINS)
        query_cfop_nao_tributado_com_valor_piscofins = self.\
            get_query_cfop_nao_tributado_com_valor_piscofins(cr, uid, data_inicial, data_final,
                                                             context)
        cr.execute(query_cfop_nao_tributado_com_valor_piscofins)
        dados_cfop_nao_tributado_com_valor_piscofins = cr.dictfetchall()

        # CFOP de bonificacao com CST diferente de 49 ou 98
        query_cfop_bonificado_cst_diferente = self.get_query_cfop_bonificado_cst_diferente(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_cfop_bonificado_cst_diferente)
        dados_cfop_bonificado_cst_diferente = cr.dictfetchall()

        # Produto movimentado com grade de outro
        query_produto_com_grade_distinta = self.get_query_produto_com_grade_distinta(
            cr, uid, data_inicial, data_final, context)
        cr.execute(query_produto_com_grade_distinta)
        dados_produto_com_grade_distinta = cr.dictfetchall()

        dados = {
            'cst_pis_diferente_cst_cofins': dados_cst_pis_diferente_cst_cofins,
            'tributacao_diferente_cst': dados_tributacao_diferente_cst,
            'cst_entrada_movimento_saida': dados_cst_entrada_movimento_saida,
            'cst_saida_movimento_entrada': dados_cst_saida_movimento_entrada,
            'diferenca_base_calculo_total_nf': dados_diferenca_base_calculo_total_nf,
            'cst_nao_tributado_com_valor': dados_cst_nao_tributado_com_valor,
            'cfop_nao_tributado_com_valor': dados_cfop_nao_tributado_com_valor_piscofins,
            'cfop_bonificado_cst_diferente': dados_cfop_bonificado_cst_diferente,
            'produto_com_grade_distinta': dados_produto_com_grade_distinta
        }

        return dados

    def get_query_inherit_dados(self, cr, uid, sql, context=None):
        return sql

    def get_query_cst_pis_diferente_cst_cofins(self, cr, uid, data_inicial, data_final,
                                               context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT *
            FROM (
                SELECT
                    nf.numero_nf    AS numero_nf,
                    nf.tipo         AS tipo_nf,
                    emi.id          AS movimento,
                    (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                    AS produto,
                    (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                    AS grade,
                    count(DISTINCT fc.codigo) AS qtd
                FROM estoque_movimento_itens emi
                    INNER JOIN estoque_movimento_capa emc
                        ON emc.id = emi.capa_id
                    INNER JOIN fiscal_nf_item_tributacao fnit
                        ON emi.id = fnit.movimento_itens_id
                    INNER JOIN fiscal_encargo fe
                        ON fe.id = fnit.encargo_id
                    INNER JOIN fiscal_cst fc
                        ON fc.id = fnit.cst_id
                    LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
                WHERE
                    emc.company_id = %s
                    AND emc.state = 'F'
                    AND emi.nf_id IS NOT NULL
                    AND emc.data BETWEEN '%s' AND '%s'
                    AND fe.sigla IN ('PIS', 'COFINS')
                GROUP BY numero_nf, tipo_nf, movimento, produto, grade
            ) AS tmp
            WHERE qtd > 1
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_cst_nao_tributado_com_valor(self, cr, uid, data_inicial, data_final,
                                              context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                emi.id          AS movimento,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo fe
                    ON fe.id = fnit.encargo_id
                INNER JOIN fiscal_cst fc
                    ON fc.id = fnit.cst_id
                LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emi.nf_id IS NOT NULL
                AND emc.data BETWEEN '%s' AND '%s'
                AND fe.sigla IN ('PIS', 'COFINS')
                AND fc.codigo IN ('49','98','05','06', '07', '08', '09', '70')
                AND fnit.vr_encargo > 0
                AND fnit.vr_base_calculo > 0
            GROUP BY numero_nf, tipo_nf, movimento, produto, grade
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_cfop_nao_tributado_com_valor_piscofins(self, cr, uid, data_inicial, data_final,
                                                context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                emi.id          AS movimento,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo fe
                    ON fe.id = fnit.encargo_id
                INNER JOIN fiscal_cfop fc
                    ON fc.id = emi.cfop_id
                LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
                LEFT JOIN fiscal_cfop_natureza fcn
                    ON fcn.id = nf.natureza_id
                LEFT JOIN fiscal_cfop fc2
                    ON fcn.cfop_id = fc2.id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emi.nf_id IS NOT NULL
                AND emc.data BETWEEN '%s' AND '%s'
                AND fe.sigla IN ('PIS', 'COFINS')
                AND fnit.vr_encargo > 0
                AND fnit.vr_base_calculo > 0
                AND (fc.codigo IN ('1407', '1556', '1910', '1911', '2911', '1949', '2949', '2407', '2556', '2910', 
                                   '5927', '6927', '1551', '2551', '5910', '6910') 
                     OR fc2.codigo IN ('1556', '1407'))
            GROUP BY numero_nf, tipo_nf, movimento, produto, grade
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_cfop_bonificado_cst_diferente(self, cr, uid, data_inicial, data_final,
                                                context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                emi.id          AS movimento,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo fe
                    ON fe.id = fnit.encargo_id
                INNER JOIN fiscal_cst fc
                    ON fc.id = fnit.cst_id
                INNER JOIN fiscal_cfop cfop
                    ON cfop.id = emi.cfop_id
                LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emi.nf_id IS NOT NULL
                AND emc.data BETWEEN '%s' AND '%s'
                AND fe.sigla IN ('PIS', 'COFINS')
                AND fc.codigo NOT IN ('49','98')
                AND cfop.codigo IN ('1910', '2910', '1911', '2911')
                --AND fnit.vr_encargo > 0
                --AND fnit.vr_base_calculo > 0
            GROUP BY numero_nf, tipo_nf, movimento, produto, grade
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_tributacao_diferente_cst(self, cr, uid, data_inicial, data_final, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_tributacao ft
                    ON ft.id = fnit.tributacao_id
                INNER JOIN fiscal_cst fc
                    ON fc.id = fnit.cst_id
                LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emi.nf_id IS NOT NULL
                AND fc.codigo != (SELECT codigo FROM fiscal_cst WHERE id = ft.cst_id)
                AND emc.data BETWEEN '%s' AND '%s'
            GROUP BY numero_nf, tipo_nf, produto, grade
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_cst_entrada_movimento_saida(self, cr, uid, data_inicial, data_final,
                                              context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_cst fc
                    ON fc.id = fnit.cst_id
                INNER JOIN estoque_natureza_operacao eno
                    ON eno.id = emc.estmvc_operacao_id
                LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emc.data BETWEEN '%s' AND '%s'
                AND emi.nf_id IS NOT NULL
                AND fc.fiscst_tipo_movimento = 'E'
                AND eno.estnto_tipo_operacao = 'S'
            GROUP BY numero_nf, tipo_nf, produto, grade
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_cst_saida_movimento_entrada(self, cr, uid, data_inicial, data_final,
                                              context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                (SELECT descricao FROM estoque_produto_grade WHERE id = emi.grade_id)
                                AS grade
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                INNER JOIN fiscal_nf_item_tributacao fnit
                    ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_cst fc
                    ON fc.id = fnit.cst_id
                INNER JOIN estoque_natureza_operacao eno
                    ON eno.id = emc.estmvc_operacao_id
                LEFT JOIN fiscal_nf_capa nf ON nf.id = emi.nf_id
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emc.data BETWEEN '%s' AND '%s'
                AND emi.nf_id IS NOT NULL
                AND fc.fiscst_tipo_movimento = 'S'
                AND eno.estnto_tipo_operacao = 'E'
            GROUP BY numero_nf, tipo_nf, produto, grade
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_diferenca_base_calculo_total_nf(self, cr, uid, data_inicial, data_final,
                                                  context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'fiscal.nf.capa', context=context)
        sql = """
            SELECT *
            FROM (
                SELECT
                 nf.numero_nf              AS numero_nf,
                 nf.tipo                   AS tipo_nf,
                 nf.nf_propria             AS nf_propria,
                 vr_total_nf               AS valor_nf,
                 (SELECT
                      sum(vr_base_calculo + vr_icms_substituicao + vr_reducao_base + 
                          COALESCE(fisnit_vr_fcp_st, 0.0) - COALESCE(fisnit_vr_icms_desonerado, 0))
                  FROM fiscal_nf_item_tributacao
                  WHERE id IN (SELECT tributacao_id
                               FROM (SELECT
                                         DISTINCT min(fnit.id) as tributacao_id,
                                         fnit.movimento_itens_id as movimento_id
                                     FROM fiscal_nf_item_tributacao fnit
                                     WHERE
                                         fnit.fisnit_capa_id = nf.id
                                          AND fnit.encargo_id IN (SELECT id
                                                                  FROM fiscal_encargo
                                                                  WHERE sigla = 'ICMS')
                                     GROUP BY movimento_id) as tmp
                               )
                 )
                 + COALESCE(nf.vr_ipi, 0)  AS valor_tributacao
                FROM fiscal_nf_capa nf
                LEFT JOIN fiscal_cfop_natureza nat
                    ON nat.id = nf.natureza_id
                LEFT JOIN fiscal_cfop fc
                    ON nat.cfop_id = fc.id
                WHERE
                 CASE
                     WHEN nf.tipo = 'E'
                         THEN nf.data_saida_entrada BETWEEN '{data_inicial} 00:00:00'
                             AND '{data_final} 23:59:59'
                     ELSE nf.data_emissao BETWEEN '{data_inicial} 00:00:00'
                         AND '{data_final} 23:59:59'
                 END
                 AND nf.company_id = {company_id}
                 AND nf.state = '3'
                 AND fc.codigo NOT IN ('5929', '6929')
            ) AS tmp
            WHERE
                valor_nf != valor_tributacao
            ORDER BY numero_nf, tipo_nf, nf_propria
        """.format(company_id=company_id, data_inicial=data_inicial, data_final=data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql

    def get_query_produto_com_grade_distinta(self, cr, uid, data_inicial, data_final, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.movimento.capa', context=context)
        sql = """
            SELECT
                nf.numero_nf    AS numero_nf,
                nf.tipo         AS tipo_nf,
                (SELECT codigo || '-' || name FROM estoque_produto WHERE id = emi.produto_id)
                                AS produto,
                emi.sequencia   AS sequencia
            FROM estoque_movimento_itens emi
                INNER JOIN estoque_movimento_capa emc
                    ON emc.id = emi.capa_id
                LEFT JOIN fiscal_nf_capa nf 
                    ON nf.id = emi.nf_id
                LEFT JOIN estoque_produto_grade epg
                    ON emi.grade_id = epg.id
                LEFT JOIN estoque_produto ep
                    ON ep.id = emi.produto_id 
            WHERE
                emc.company_id = %s
                AND emc.state = 'F'
                AND emi.nf_id IS NOT NULL
                AND emc.data BETWEEN '%s' AND '%s'
                AND epg.produto_id != ep.id
            GROUP BY numero_nf, tipo_nf, produto, sequencia
        """ % (company_id, data_inicial, data_final)
        sql = self.get_query_inherit_dados(cr, uid, sql, context)
        return sql
