# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
import urllib2
from openerp.tools import config
from datetime import datetime
import xml.etree.ElementTree as ET
from dateutil.relativedelta import relativedelta
import urllib3
import requests
import logging
import base64
import time


class alertafiscal_utils(osv.Model):
    _name = 'alertafiscal.utils'

    def alfutl_consulta_webservice(self, cr, uid, dt_filtro=False, ean=False, ncm=False,
                                   codigo_interno=False, context=None):
        """
        Metodo responsavel por realizar a consulta com o webservice alertafiscal.
        Sempre deve ser usado como consulta somente um dos parametros disponiveis.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param dt_filtro: ultima data de atualizacao da tributacao junto ao alertafiscal.
        :type dt_filtro: str
        :param ean: sequencia de caracteres referente a um codigo de barras.
        :type ean: str
        :param ncm: sequencia de caracteres referente a um NCM.
        :type ncm: str
        :param codigo_interno: sequencia de caracteres referente ao codigo de um produto.
        :type codigo_interno: str
        :param context: dados contextuais.
        :type context: dict
        :return: Xml com os dados procesados pelo webservice, caso a consulta seja realizada
        corretamente. Senao, mensagem que informara possivel problema/erro.
        """
        obj_adm_utils = self.pool.get('adm.utils')
        list_parametros = ['estpar_alertafiscal_idcliente', 'estpar_alertafiscal_token']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', list_parametros, context=context)
        if not all(estoque_parametro_dados.values()):
            raise osv.except_osv(_('Atencao'), _("Cadastre todos os dados da aba 'Alerta Fsical' "
                                                 "nos parametros do estoque."))
        parametros = [dt_filtro, ean, ncm, codigo_interno]
        if not any(parametros):
            raise osv.except_osv(_('Atencao'), _('Deve ser informado uma data de atualizacao, '
                                                 'codigo de barras, codigo interno ou NCM.'))
        parametros_escolhidos = filter(lambda p: p, parametros)
        if len(parametros_escolhidos) > 1:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao deve ser usado mais de um parametro de consulta. '
                                   'Escolhidos:\n' + ',\n'.join(parametros_escolhidos)))
        req_header, req_footer = self.alfutl_get_requisicao_soap_default(cr, uid, context)
        req_body = """
            <ConsultarTributacoesMG xmlns="http://soap.alertafiscalintranet.com.br">
                <objConsulta>
                    <Id>0</Id>
                    <Token />
                    {ean}
                    {codigo_interno}
                    {ncm}
                    {dt_filtro}
                </objConsulta>
            </ConsultarTributacoesMG>
        """.format(ean='<Ean>%s</Ean>' % ean if ean else '<Ean />',
                   codigo_interno='<CodigoInterno>%s</CodigoInterno>' % codigo_interno if
                   codigo_interno else '<CodigoInterno />',
                   ncm='<Ncm>%s</Ncm>' % ncm if ncm else '<Ncm />',
                   dt_filtro='<DataFiltro>%s</DataFiltro>' % dt_filtro if dt_filtro else
                   '<DataFiltro>0001-01-01T00:00:00.000-00:00</DataFiltro>')
        requisicao = req_header + req_body + req_footer
        requisicao = requisicao.replace('\n', '')
        requisicao = requisicao.replace('  ', '')
        proxy = config.options.get('http_proxy')
        if proxy:
            proxy_handler = urllib2.ProxyHandler({'http': proxy})
            opener = urllib2.build_opener(proxy_handler)
            urllib2.install_opener(opener)
        url = 'http://soap.alertafiscalintranet.com.br/WSIntegracaoSoap.asmx'
        headers = {"SOAPAction": "http://soap.alertafiscalintranet.com.br/ConsultarTributacoesMG",
                   "Content-Type": "text/xml;charset=UTF-8"
                   }
        res = urllib2.Request(url, requisicao, headers)
        try:
            res = urllib2.urlopen(res)
        except Exception as e:
            msg = ''
            if e.code == 305:
                msg += str(e) + '.\nVerifique as configuracoes de proxy.'
            else:
                msg = 'Nenhuma tributacao encontrada.'
            raise osv.except_osv(_('Atencao'), msg)
        str_xml = res.read()
        return str_xml

    def alfutl_solicita_cadastro_produto(self, cr, uid, context=None):
        """
        Metodo chamado pelo agendador (cron) que fara a inclusao no Alerta Fiscal para os produtos
        criados no dia corrente. O limite de produtos é de 500, diario.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso os produtos tenham sido criados. Senao, falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        produto_ids = self.pool.get('estoque.produto').search(
            cr, uid, [('create_date', '>=', str(datetime.today().date()) + ' 00:00:00')],
            limit=500, context=context)
        codigo_barra_ids = obj_automacao_codigo_barra.search(
            cr, uid, [('produto_id', 'in', produto_ids)], context=context)
        if not codigo_barra_ids:
            return False
        req_header, req_footer = self.alfutl_get_requisicao_soap_default(cr, uid, context)
        body = """
            <SolicitarCadastroProduto xmlns="http://soap.alertafiscalintranet.com.br">
                <Ean>{codigo_barra}</Ean>
                <DescricaoCompleta>{descricao}</DescricaoCompleta>
                <MarcaProduto />
                <DetalheProduto />
                <ConteudoProduto />
                <MedidaProduto />
                <ProdutoImportado>{importado}</ProdutoImportado>
            </SolicitarCadastroProduto>
        """
        req_body = """"""
        for cb in obj_automacao_codigo_barra.browse(cr, uid, codigo_barra_ids, context):
            req_body += body.format(
                codigo_barra=cb.codigo_barra, descricao=cb.grade_id.name or cb.grade_id.descricao,
                importado=str(True if cb.produto_id.origem_mercadoria in ['1', '2', '6', '7'] else
                              False).lower())
        requisicao = req_header + req_body + req_footer
        requisicao = requisicao.replace('\n', '').replace('  ', '')
        url = 'http://soap.alertafiscalintranet.com.br/WsSolicitacaoCadastro.asmx'
        http = urllib3.PoolManager()
        res = http.request('GET', url)
        if res.status != 200:
            return False
        headers = {"SOAPAction": "http://soap.alertafiscalintranet.com.br/SolicitarCadastroProduto",
                   "Content-Type": "text/xml;charset=UTF-8"
                   }
        response = requests.post(url, data=requisicao, headers=headers)
        str_xml = response.content
        if not str_xml:
            return False
        str_xml = str_xml.replace(' xmlns="http://soap.alertafiscalintranet.com.br"', '')
        file_data = ET.XML(str_xml)
        dados = []
        msg = ''
        for segmento in file_data:
            xmldict = XmlDictConfig(segmento)
            if segmento.tag == 'erro':
                raise osv.except_osv(_('Atencao'), segmento.text)
            if xmldict.get('SolicitarCadastroProdutoResponse', {}):
                dados.append(xmldict.get('SolicitarCadastroProdutoResponse', {}).get(
                    'SolicitarCadastroProdutoResult', {}) or False)
            if xmldict.get('{http://schemas.xmlsoap.org/soap/envelope/}Fault'):
                msg += xmldict.get('{http://schemas.xmlsoap.org/soap/envelope/}Fault').get(
                    'faultstring') + '\n'
        logger = logging.getLogger(self._name)
        if msg:
            logger.warning('Alerta Fiscal - Solicitacao de Cadastro: ' + msg)
        elif all(dados):
            logger.info('Alerta Fiscal - Solicitacao de Cadastro: Incluido com sucesso.')
        return all(dados)

    def alfutl_consulta_codigo_iterno_atualizado(self, cr, uid, context=None):
        """
        Metodo chamado pelo agendador (cron) que fara a consulta no Alerta Fiscal para os produtos
        atualizados no dia anterior.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso tenha produtos atualizados. Senao, falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        req_header, req_footer = self.alfutl_get_requisicao_soap_default(cr, uid, context)
        req_body = """
            <ConsultarCodigosInternosAtualizados xmlns="http://soap.alertafiscalintranet.com.br">
                <objConsulta>
                    <Id>0</Id>
                    <Token />
                    <Ean />
                    <Ncm />
                    <CodigoInterno />
                    <DataFiltro>{data_filtro}T00:00:00.000-00:00</DataFiltro>
                </objConsulta>
            </ConsultarCodigosInternosAtualizados>
        """.format(data_filtro=str(datetime.today().date() - relativedelta(days=1)))
        requisicao = req_header + req_body + req_footer
        requisicao = requisicao.replace('\n', '').replace('  ', '')
        proxy = config.options.get('http_proxy')
        if proxy:
            proxy_handler = urllib2.ProxyHandler({'http': proxy})
            opener = urllib2.build_opener(proxy_handler)
            urllib2.install_opener(opener)
        url = 'http://soap.alertafiscalintranet.com.br/WSIntegracaoSoap.asmx'
        headers = {
            "SOAPAction": "http://soap.alertafiscalintranet.com.br/ConsultarCodigosInternosAtual"
                          "izados",
            "Content-Type": "text/xml;charset=UTF-8"}
        res = urllib2.Request(url, requisicao, headers)
        try:
            res = urllib2.urlopen(res)
        except Exception as e:
            msg = ''
            if e.code == 305:
                msg += ' Verifique as configuracoes de proxy.'
            else:
                msg += ' Nenhuma atualizacao disponivel de acordo com os filtros aplicados.'
            raise osv.except_osv(_('Atencao'), str(e) + msg)
        str_xml = res.read()
        if not str_xml:
            return False
        str_xml = str_xml.replace(' xmlns="http://soap.alertafiscalintranet.com.br"', '')
        file_data = ET.XML(str_xml)
        dados = []
        for segmento in file_data:
            xmldict = XmlDictConfig(segmento)
            if segmento.tag == 'erro':
                raise osv.except_osv(_('Atencao'), segmento.text)
            dados += (xmldict.get('ConsultarCodigosInternosAtualizadosResponse', {}).get(
                'ConsultarCodigosInternosAtualizadosResult', {}). get('CodigosInternos', {}) or
                      '').split(';')
        if not dados:
            return False
        obj_importa_xml = self.pool.get('wizard.importa.xml.retorno.alertafiscal')
        context.update({'alertafiscal_produto_id': True, 'alertafiscal_codigo_interno_list': dados})
        list_produto_dados = []
        for produto in dados:
            # OBSERVACAO NA DOCUMENTACAO DO ALERTA FISCAL:
            # Recomendamos fortemente que, caso sejam executados acessos simultâneos em lote, haja
            # um intervalo de 1 (UM) segundo entre as requisicoes. Isto garante que nao haja falhas
            # nas respostas do servidor ou problemas de sincronismo de acessos.
            time.sleep(1)
            str_xml_dados = self.alfutl_consulta_webservice(
                cr, uid, codigo_interno=produto, context=context)
            if not str_xml_dados:
                continue
            xml_dados = base64.b64encode(str_xml_dados)
            list_xml_dados = obj_importa_xml.retalf_converte_xml_dados(cr, uid, xml_dados,
                                                                       context)
            list_xml_dados[0].update({'CodigoInterno': produto})
            list_produto_dados += list_xml_dados
        del context['alertafiscal_produto_id']
        estoque_fiscal_id = obj_importa_xml.retalf_importa_retorno_revisao(
            cr, uid, list_produto_dados, context)
        return self.pool.get('estoque.fiscal.capa').write(
            cr, uid, estoque_fiscal_id,
            {'estfca_observacao': 'Códigos internos dos produtos atualizados:\n' + ' / '.join(map(
                lambda i: str(i), dados))})

    def alfutl_get_requisicao_soap_default(self, cr, uid, context=None):
        """
        Monta os dados padrao da requisicao SOAP.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dados da requisicao SOAP
        :rtype: str
        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        list_parametros = ['estpar_alertafiscal_idcliente', 'estpar_alertafiscal_token']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', list_parametros, context=context)
        if not all(estoque_parametro_dados.values()):
            raise osv.except_osv(_('Atencao'), _("Cadastre todos os dados da aba 'Alerta Fsical' "
                                                 "nos parametros do estoque."))
        req_header = """<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"><soap:Header><ValidationSoapHeader xmlns="http://soap.alertafiscalintranet.com.br"><Id>{id_cliente}</Id><Token>{token}</Token></ValidationSoapHeader></soap:Header><soap:Body>""".\
            format(id_cliente=estoque_parametro_dados.get('estpar_alertafiscal_idcliente'),
                   token=estoque_parametro_dados.get('estpar_alertafiscal_token'))
        req_footer = """</soap:Body></soap:Envelope>"""
        return req_header, req_footer


class XmlListConfig(list):
    def __init__(self, aList):
        for element in aList:
            if element:
                # treat like dict
                if len(element) == 1 or element[0].tag != element[1].tag:
                    self.append(XmlDictConfig(element))
                # treat like list
                elif element[0].tag == element[1].tag:
                    self.append(XmlListConfig(element))
            elif element.text:
                text = element.text.strip()
                if text:
                    self.append(text)


class XmlDictConfig(dict):
    """
    Example usage:

    >>> tree = ElementTree.parse('your_file.xml')
    >>> root = tree.getroot()
    >>> xmldict = XmlDictConfig(root)

    Or, if you want to use an XML string:

    >>> root = ElementTree.XML(xml_string)
    >>> xmldict = XmlDictConfig(root)

    And then use xmldict for what it is... a dict.
    """

    def __init__(self, parent_element):
        if parent_element.items():
            self.update(dict(parent_element.items()))
        for element in parent_element:
            if len(element):
                # treat like dict - we assume that if the first two tags
                # in a series are different, then they are all different.
                if len(element) == 1 or element[0].tag != element[1].tag:
                    aDict = XmlDictConfig(element)
                # treat like list - we assume that if the first two tags
                # in a series are the same, then the rest are the same.
                else:
                    # here, we put the list in dictionary; the key is the
                    # tag name the list elements all share in common, and
                    # the value is the list itself
                    aDict = {element[0].tag: XmlListConfig(element)}
                # if the tag has attributes, add those to the dict
                if element.items():
                    aDict.update(dict(element.items()))
                self.update({element.tag: aDict})
            # this assumes that if you've got an attribute in a tag,
            # you won't be having any text. This may or may not be a
            # good idea -- time will tell. It works for the way we are
            # currently doing XML configuration files...
            elif element.items():
                self.update({element.tag: dict(element.items())})
            # finally, if there are no child tags and no attributes, extract
            # the text
            else:
                self.update({element.tag: element.text})

