# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import xlwt
import time
from openerp.tools.translate import _


class wizard_exporta_planilha_alertafiscal(osv.TransientModel):

    _name = 'wizard.exporta.planilha.alertafiscal'
    _columns = {
        'local_id': fields.many2one('estoque.local', 'Local'),
        'planilha': fields.binary('Planilha'),
        'nome_planilha': fields.char('Nome planilha', size=16),
    }

    def default_get(self, cr, uid, fields_list, context=None):
        """
        Metodo responsavel por atribuir valores padrao aos atributos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista com a descricao dos atributos.
        :type fields_list: list
        :param context: valores contextuais.
        :type context: dict
        :return: dados com a descricao dos atributos e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(wizard_exporta_planilha_alertafiscal, self).default_get(cr, uid, fields_list, context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'local_venda_padrao_id', context=context)
        res['local_id'] = estoque_parametro_dados.get('local_venda_padrao_id')
        res['nome_planilha'] = 'alertafiscal.xls'
        return res

    def button_exporta_planilha_alertafiscal(self, cr, uid, ids, context=None):
        """
        Metodo do botao para processo de geracao da planilha alertafiscal.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: valores contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        alertafiscal_id = ids[0] if isinstance(ids, (list, tuple)) else ids
        local_id = self.browse(cr, uid, alertafiscal_id, context).local_id.id or False
        print '------------------GET_DADOS'
        dados = self.get_dados_alertafiscal(cr, uid, ids, context)
        if not dados:
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para o local selecionado.'))
        print '------------------GERA_PLANILHA'
        xls_dados = self.gera_planilha_alertafiscal(cr, uid, local_id, dados, context)
        if xls_dados:
            self.write(cr, uid, ids, {'planilha': xls_dados}, context)
        return True

    def get_dados_alertafiscal(self, cr, uid, ids, context=None):
        """
        O metodo reune todos os dados necessarios dos produtos para a exportacao da planilha
        alertafiscal.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: valores contextuais.
        :type context: dict
        :return: dados dos produtos para a planilha alertafiscal.
        :rtype: list
        """
        context = context and context.copy() or {}
        alertafiscal_id = ids[0] if isinstance(ids, (list, tuple)) else ids
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                    context=context)

        local = ''
        inner_locacao = ''
        local_id = self.browse(cr, uid, alertafiscal_id, context).local_id.id or False
        if local_id:
            local = """
                AND ell.local_id = %s
            """ % local_id
            inner_locacao = """
            INNER JOIN estoque_local_locacao ell
                ON ep.id = ell.produto_id AND epg.id = ell.grade_id
        """

        sql = """
            SELECT
                DISTINCT
                ep.id               AS produto_id,
                epg.id              AS grade_id,
                ep.codigo           AS CodigoInterno,
                acb.codigo_barra    AS CodigoEan,
                CASE
                    WHEN substr(acb.codigo_barra, 1, 13)::bigint = ep.codigo::bigint
                        THEN 1
                    ELSE 0
                END                 AS CodigoEanInterno,
                ''                  AS Setor,
                CASE
                    WHEN (SELECT estpar_descricao_grade FROM estoque_parametro WHERE company_id = 1) = 'P'
                        THEN ep.name || '-' || epg.descricao
                    ELSE
                        epg.descricao
                END                 AS DescricaoProduto,
                fn.codigo           AS NCM,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'PIS')
                     )
                )                   AS AliquotaPis,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'COFINS')
                     )
                )                   AS AliquotaCofins,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla IN ('PIS', 'COFINS'))
                     )
                )                   AS CstPisCofinsSaida,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla IN ('PIS', 'COFINS'))
                     )
                )                   AS CstPisCofinsEntrada,
                CASE
                    WHEN ep.estpro_pis_nat_rec IS NOT NULL THEN ep.estpro_pis_nat_rec
                    WHEN ep.estpro_cofins_nat_rec IS NOT NULL THEN ep.estpro_cofins_nat_rec
                    ELSE ''
                END                 AS NaturezaReceitaPisCofins,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS AliquotaIcmsSaida,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS CstIcmsSaida,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'U' THEN 1
                    ELSE 0
                END                 AS SubstituicaoTributaria,
                0                   AS CestaBasica,
                CASE
                    WHEN ep.origem_mercadoria IN ('1','2','6','7')
                        THEN 1
                    ELSE 0
                END                 AS Importado,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'I' THEN 1
                    ELSE 0
                END                 AS Isento,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'D' THEN 1
                    ELSE 0
                END                 AS Diferido,
                ''                  AS TipoLista,
                CASE
                    WHEN ep.estpro_cest_id IS NOT NULL THEN
                        (SELECT fiscest_codigo
                         FROM fiscal_cest
                         WHERE id = ep.estpro_cest_id)
                    WHEN fn.fisnc_cest_id IS NOT NULL THEN
                        (SELECT fiscest_codigo
                         FROM fiscal_cest
                         WHERE id = fn.fisnc_cest_id)
                    ELSE ''
                END                 AS CEST
            FROM estoque_produto ep
            INNER JOIN estoque_produto_grade epg
                ON ep.id = epg.produto_id
            INNER JOIN automacao_codigo_barra acb
                ON ep.id = acb.produto_id AND epg.id = acb.grade_id
            INNER JOIN fiscal_ncm fn
                ON fn.id = ep.ncm_id
            {inner_locacao}
            WHERE
                ep.active IS TRUE
                AND ep.company_id = {company_id}
                {local}
        """.format(inner_locacao=inner_locacao, company_id=company_id, local=local)
        cr.execute(sql)
        dados = cr.dictfetchall()

        return dados

    def gera_planilha_alertafiscal(self, cr, uid, local_id, dados, context=None):
        """
        O metodo exporta os dados dos produtos para uma planilha do excel.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param local_id: id da tabela estoque_local, identificador do local dos produtos.
        :type local_id: int
        :param dados: dados os produtos que formarao a planilha.
        :type dados: list
        :param context: valores contextuais.
        :type context: dict
        :return: dados da planilha excel.
        :rtype: str
        """
        context = context and context.copy() or {}
        produto_ids = list(set(map(lambda i: i.get('produto_id'), dados)))
        grade_ids = list(set(map(lambda i: i.get('grade_id'), dados)))
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        ecf_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'ecf', context)
        print '------------------GET_QTD_VENDIDA'
        if ecf_instalado:
            quantidade_vendida_dados = self.get_movimento_quantidade_vendida_ecf(
                cr, uid, company_id, local_id, context=context)
        else:
            quantidade_vendida_dados = self.get_movimento_quantidade_vendida(
                cr, uid, company_id, local_id, context=context)

        print '------------------GET_PRECO_VENDA'
        preco_venda_dados = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, epc_produto_ids=produto_ids, epc_grade_ids=grade_ids, context=context)

        print '------------------GET_PRECO_CUSTO'
        preco_custo_dados = self.pool.get('estoque.preco.custo').get_custos_do_dia(
            cr, uid, epc_produto_ids=produto_ids)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_custo_para_precovenda', context=context)
        tipo_custo = estoque_parametro_dados.get('estpar_custo_para_precovenda') or ''
        if tipo_custo == 'U':
            tipo_custo = 'estprc_ultimo_custo'
        elif tipo_custo == 'M':
            tipo_custo = 'estprc_custo_medio_contabil'
        elif tipo_custo == 'R':
            tipo_custo = 'estprc_custo_reposicao'
        elif tipo_custo == 'C':
            tipo_custo = 'estprc_custo_corrigido'

        # Criacao de Planilha Excel
        wb = xlwt.Workbook(encoding='utf8')
        ws = wb.add_sheet('ALERTAFISCAL')
        rotulos = ['CodigoInterno', 'CodigoEan', 'CodigoEan_Interno', 'Setor', 'Descricao_Produto',
                   'Qtd_Vendida', 'Preco_Medio_Custo', 'Preco_Medio_Venda', 'NCM', 'Aliquota_Pis',
                   'Aliquota_Cofins', 'CST_PisCofins_Saida', 'CST_PisCofins_Entrada',
                   'Natureza_Receita_PisCofins', 'Aliquota_Icms_Saida', 'Cst_Icms_Saida',
                   'Tipo_Lista', 'Substituicao_Tributaria', 'Cesta_Basica', 'Importado', 'Isento',
                   'Diferido']  # , 'CEST']
        for i in range(0, 22):
            ws.write(0, i, rotulos[i])
        linha = 1
        quantidade_vendida_produto = {}
        map(lambda q: quantidade_vendida_produto.update(
            {str(q.get('produto_id')) + str(q.get('grade_id')): q.get('quantidade')}),
            quantidade_vendida_dados)
        for dado in dados:
            print '------------------WRITE_PLAN'
            chave = str(dado.get('produto_id')) + str(dado.get('grade_id'))
            quantidade_vendida = quantidade_vendida_produto.get(chave) or 0.0
            preco_venda_produto = preco_venda_dados.get(dado.get('produto_id'))
            preco_venda = 0
            if preco_venda_produto:
                preco_venda = preco_venda_produto.get(dado.get('grade_id')) or \
                              preco_venda_produto.get('preco_venda')

            preco_custo_produto = preco_custo_dados.get(dado.get('produto_id'))
            preco_custo = 0
            if preco_custo_produto:
                preco_custo = preco_custo_produto.get(tipo_custo)

            coluna = 0
            ws.write(linha, coluna, dado.get('CodigoInterno'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('CodigoEan'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('CodigoEanInterno'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('Setor'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('DescricaoProduto'.lower()))
            coluna += 1
            ws.write(linha, coluna, quantidade_vendida)
            coluna += 1
            ws.write(linha, coluna, preco_custo)
            coluna += 1
            ws.write(linha, coluna, preco_venda)
            coluna += 1
            ws.write(linha, coluna, dado.get('NCM'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('AliquotaPis'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('AliquotaCofins'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('CstPisCofinsSaida'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('CstPisCofinsEntrada'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('NaturezaReceitaPisCofins'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('AliquotaIcmsSaida'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('CstIcmsSaida'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('TipoLista'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('SubstituicaoTributaria'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('CestaBasica'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('Importado'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('Isento'.lower()))
            coluna += 1
            ws.write(linha, coluna, dado.get('Diferido'.lower()))
            coluna += 1
            # ws.write(linha, coluna, dado.get('CEST'.lower()))
            # coluna += 1

            linha += 1

        filename = 'alertafiscal-%s.xls' % time.strftime('%Y%m%d%H%M%S')
        wb.save('/tmp/'+filename)
        res = open('/tmp/%s' % filename, 'rb').read().encode('base64')
        return res

    def get_movimento_quantidade_vendida(self, cr, uid, company_id, local_id=False,
                                         data_inicial=False, data_final=False, context=None):
        """
        O metodo reune por produto e grade a quantidade vendida desses.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_ids: lista de id da tabela estoque_produto, identificadores dos produtos.
        :type produto_ids: list
        :param grade_ids: lista de id da tabela estoque_produto_grade, identificadores das grades
        dos produtos.
        :param company_id: id da tabela res_company, identificador da empresa.
        :type company_id: int
        :param local_id: id da tabela estoque_local, identificador do local dos produtos.
        :type local_id: int
        :param data_inicial: data inicial (aaaa-mm-dd) de movimento.
        :type data_inicial: str
        :param data_final: data final (aaaa-mm-dd) de movimento.
        :type data_final: str
        :param context: valores contextuais.
        :type context: dict
        :return: dados da identificacao do produto, da grade e a quantidade vendida.
        :rtype: list
        """
        context = context and context.copy() or {}
        sql = """
            SELECT
                Sum(emi.quantidade) AS quantidade,
                emi.produto_id      AS produto_id,
                emi.grade_id        AS grade_id
            FROM estoque_movimento_itens emi
            INNER JOIN estoque_movimento_capa emc
                ON emc.id = emi.capa_id
            INNER JOIN fiscal_nf_capa nf
                ON nf.id = emi.nf_id
            INNER JOIN estoque_produto ep
                ON ep.id = emi.produto_id
            LEFT JOIN fiscal_cfop fc
                ON fc.id = emi.cfop_id
            WHERE
                emc.company_id = {company_id}
                AND ep.active IS TRUE
        """.format(company_id=company_id)
        if data_inicial:
            sql += """
                AND emc.data >= '%s'
            """  % data_inicial
        if data_final:
            sql += """
                AND emc.data <= '%s'
            """ % data_final
        if local_id:
            sql += """
                AND emi.local_id = %s
            """ % local_id
        sql += """
                AND emc.state = 'F'
                AND estmvc_operacao_id IN (SELECT id
                                           FROM estoque_natureza_operacao
                                           WHERE
                                               estnto_atualizar_saldo = TRUE
                                               AND estnto_tipo_operacao = 'S')
                AND nf.state = '3'
                AND nf.tipo = 'S'
                AND emi.cfop_id in (SELECT id
                                    FROM fiscal_cfop
                                    WHERE
                                        tipo = 'S'
                                        AND movimento_tipo = 'C')
            GROUP BY
                emi.produto_id,
                emi.grade_id
        """
        cr.execute(sql)
        res = cr.dictfetchall()
        return res

    def get_movimento_quantidade_vendida_ecf(self, cr, uid, company_id, local_id=False,
                                             data_inicial=False, data_final=False, context=None):
        """
        O metodo reune por produto e grade a quantidade vendida desses quando o modulo ECF esta
        instalado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_ids: lista de id da tabela estoque_produto, identificadores dos produtos.
        :type produto_ids: list
        :param grade_ids: lista de id da tabela estoque_produto_grade, identificadores das grades
        dos produtos.
        :param company_id: id da tabela res_company, identificador da empresa.
        :type company_id: int
        :param local_id: id da tabela estoque_local, identificador do local dos produtos.
        :type local_id: int
        :param data_inicial: data inicial (aaaa-mm-dd) de movimento.
        :type data_inicial: str
        :param data_final: data final (aaaa-mm-dd) de movimento.
        :type data_final: str
        :param context: valores contextuais.
        :type context: dict
        :return: dados da identificacao do produto, da grade e a quantidade vendida.
        :rtype: list
        """
        context = context and context.copy() or {}
        sql = """
            SELECT
                Sum(
                    CASE
                        WHEN ecc.tipo_cupom = 'C'
                            THEN emi.quantidade * -1
                        ELSE emi.quantidade
                    END
                ) AS quantidade,
                emi.produto_id AS produto_id,
                emi.grade_id   AS grade_id
            FROM estoque_movimento_itens emi
            INNER JOIN estoque_movimento_capa emc
                ON emc.id = emi.capa_id
            INNER JOIN estoque_produto ep
                ON ep.id = emi.produto_id
            LEFT JOIN fiscal_cfop fc
                ON fc.id = emi.cfop_id
            LEFT JOIN ecf_cupons_capa AS ecc ON ecc.id = emi.cupom_id
            WHERE
                emc.company_id = {company_id}
                AND ep.active IS TRUE
        """.format(company_id=company_id)
        if data_inicial:
            sql += """
                AND emc.data >= '%s'
            """ % data_inicial
        if data_final:
            sql += """
                AND emc.data <= '%s'
            """ % data_final
        if local_id:
            sql += """
                AND emi.local_id = %s
            """ % local_id
        sql += """
                AND emc.state = 'F'
                AND estmvc_operacao_id IN (SELECT id
                                           FROM estoque_natureza_operacao
                                           WHERE
                                               estnto_atualizar_saldo = TRUE
                                               AND estnto_tipo_operacao = 'S')
                AND emi.cfop_id in (SELECT id
                                    FROM fiscal_cfop
                                    WHERE
                                        tipo = 'S'
                                        AND movimento_tipo = 'C')
                AND not (emi.item_cancelado is True AND  ecc.tipo_cupom = 'V')
            GROUP BY
                emi.produto_id,
                emi.grade_id
        """
        cr.execute(sql)
        res = cr.dictfetchall()
        return res
