# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
import requests
import json
import re
import time
from openerp import tools
import logging
logger = logging.getLogger('IMendes')

class fiscal_imendes_utils(osv.Model):

    _name = 'fiscal.imendes.utils'

    def imeutl_get_parametro(self, cr, uid, context=None):
        context = context and context.copy() or {}
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_imendes_senha', 'estpar_imendes_qtd_produto'],
            context=context)
        if not estoque_parametro_dados.get('estpar_imendes_senha'):
            raise osv.except_osv(_('Atencao'), _('Cadastre a senha nos parametros do estoque, aba '
                                                 '"IMendes".'))
        # login = str(re.sub('[^0-9]', '', self.pool.get('res.users').browse(
        #     cr, uid, uid, context).company_id.rescon_pessoa_id.cpf_cnpj))
        # Segundo a IMendes, o CNPJ para conexao sempre sera o da SIG, por isso hardcode
        login = '01526558000178'
        return {'headers': {'login': login,
                            'senha': str(estoque_parametro_dados.get('estpar_imendes_senha')),
                            'Content-Type': 'application/json; charset=utf-8'},
                'quantidade_envio': estoque_parametro_dados.get('estpar_imendes_qtd_produto'),
                'url_padrao': 'http://consultatributos.com.br:8080/api/v1/public/',
                'proxies': {'url': tools.config.options.get('http_proxy')} if
                tools.config.options.get('http_proxy') else {}
        }

    def imeutl_verifica_status_cliente(self, cr, uid, context=None):
        context = context and context.copy() or {}
        parametros = self.imeutl_get_parametro(cr, uid, context)
        resposta = requests.get(
            url=parametros.get('url_padrao') + 'GetStatusCliente/' + parametros.get('headers').get(
                'login'),
            headers=parametros.get('headers'),
            proxies=parametros.get('proxies'),
        )
        res = json.loads(resposta.content)
        mensagem = ''
        try:
            if not res.get('ret').get('sucesso'):
                mensagem = res.get('ret').get('mensagem')
        except:
            if not res.get('sucesso'):
                mensagem = res.get('mensagem')
        if mensagem:
            raise osv.except_osv(_('Atencao'), mensagem)
        return True

    def imeutl_get_dado_webservice(self, cr, uid, context=None):
        """
        Metodo chamado pelo agendador (cron). Responsável por chamar os métodos para realizacao do
        do processo de comunicacao e importacao dos dados para o sistema.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados contextuais.
        :type context: dict
        :return: Dados dos produtos e suas tributacoes
        :rtype: list
        """
        context = context and context.copy() or {}
        dados_retorno = self.imeutl_consulta_produto(cr, uid, context=context)
        if dados_retorno:
            self.imeutl_importa_retorno_revisao(cr, uid, dados_retorno, context)
        return True

    def imeutl_consulta_produto(self, cr, uid, produto_id=False, context=None):
        """
        O metodo fara a consulta no webservice da IMendes para o produto informado (produto_id) ou
        os produtos ainda nao consultados. O limite de produtos é de 2000, por consulta.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_id: id da tabela estoque_produto, identificador do produto
        :type produto_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: Dados dos produtos e suas tributacoes
        :rtype: list
        """
        context = context and context.copy() or {}

        parametros = self.imeutl_get_parametro(cr, uid, context)
        quantidade_envio = parametros.get('quantidade_envio') or 2000
        if produto_id:
            quantidade_envio = 1
        self.imeutl_verifica_status_cliente(cr, uid, context)

        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        if produto_id:
            produto_ids = [produto_id]
        else:
            produto_ids = self.pool.get('estoque.produto').search(
                cr, uid, [('estpro_imendes_enviado', '=', False)],
                limit=quantidade_envio, order='estpro_regime_especial,name', context=context)
        if not produto_ids:
            return
        codigo_barra_ids = obj_automacao_codigo_barra.search(
            cr, uid, [('produto_id', 'in', produto_ids)], limit=quantidade_envio, context=context,
            order='codigo_barra DESC')
        if not codigo_barra_ids:
            raise osv.except_osv(_('Atencao'), _('Codigo de barras nao cadastrado para o produto_id: %s!') % produto_ids[0])
        brwCodigoBarras = obj_automacao_codigo_barra.browse(cr, uid, codigo_barra_ids, context)
        brw_adm_pessoa = self.pool.get('res.users').browse(
            cr, uid, uid, context).company_id.rescon_pessoa_id
        cnae = brw_adm_pessoa.admpes_jur_cnae_id.admcna_codigo
        _grupo = []
        obj_adm_utils = self.pool.get('adm.utils')
        for regimeEspecial in list(set([i.produto_id.estpro_regime_especial or '' for i in brwCodigoBarras])):
            produto_ids = []
            dict_envio = {
                'cabecalho': {
                    'cnpj': str(re.sub('[^0-9]', '', brw_adm_pessoa.cpf_cnpj)),
                    'cnae': str(re.sub('[^0-9]', '', cnae)) if cnae else '',
                    'crt': 1 if brw_adm_pessoa.admpes_jur_regime_tributario == 'S' else 3,
                    'regimeEspecial': str(regimeEspecial),
                    'codFaixa': self.imeutl_get_codigo_faixa_enquadramento(
                        cr, uid, brw_adm_pessoa.admpes_jur_regime_tributario, context),
                    'regimeTrib': str(brw_adm_pessoa.admpes_jur_regime_tributario),
                    'amb': 2
                },
                'uf': ['MG'],
            }

            list_dado_produto = []
            for cb in brwCodigoBarras:
                if (cb.produto_id.estpro_regime_especial or '') != regimeEspecial:
                    continue
                produto_ids.append(cb.produto_id.id)
                descricao = obj_adm_utils.admutl_remover_acentuacao(cr, uid, cb.grade_id.name[:60])
                descricao = obj_adm_utils.admutl_remove_caracter(descricao)
                list_dado_produto.append(
                    {'codigo': str(cb.codigo_barra[-13:]),
                     'descricao': str(descricao),
                     'tipoCodigo': 1 if cb.autcba_tipo_barra == 'OT' else 0
                     }
                )
            if not produto_ids:
                continue
            dict_envio.update({'produto': list_dado_produto})
            logger.info('Envio: %s' % str(dict_envio))
            resposta = requests.post(
                url=parametros.get('url_padrao') + 'RegrasFiscais',
                data=str(dict_envio),
                headers=parametros.get('headers'),
                proxies=parametros.get('proxies'),
                timeout=180
            )
            res = json.loads(resposta.content)
            logger.info('Retorno: %s' % res)
            if resposta.ok and not res.get('grupo') and res.get('cabecalho', {}).get('mensagem') == 'OK':
                self.imeutl_atualiza_produto_pesquisado(cr, uid, produto_ids, context)

            if not res.get('grupo') and res.get('cabecalho', {}).get('mensagem') == 'OK':
                raise osv.except_osv(_('Atencao'), _('Nenhuma tributacao encontrada.'))

            if not res.get('grupo') and res.get('cabecalho') \
                    and res.get('cabecalho').get('mensagem') != 'OK':
                raise osv.except_osv(_('Atencao'), res.get('cabecalho').get('mensagem'))

            if res.get('message'):
                raise osv.except_osv(_('Atencao'), res.get('message'))

            self.imeutl_atualiza_produto_pesquisado(cr, uid, produto_ids, context)
            _grupo.extend(res.get('grupo'))
        return _grupo

    def imeutl_get_access_history(self, cr, uid, context=None):
        """
        O metodo fara o consumo da API ENVIARECEBEDADOS acessando o metodo HISTORICOACESSO.
        Esse fornece dados de resumo de acesso, detalhes de cada acesso com quantidade de produtos
        enviados e retornados e produtos devolvidos com a identificacao dos motivos.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: identificador do usuario na transacao, id da tabela res_users
        :param context: dados contextuais
        :type context: dict
        :return: dados do historico de acesso
        :rtype: dict
        """
        context = context and context.copy()
        self.imeutl_verifica_status_cliente(cr, uid, context)
        parametros = self.imeutl_get_parametro(cr, uid, context)
        brw_adm_pessoa = self.pool.get('res.users').browse(
            cr, uid, uid, context).company_id.rescon_pessoa_id
        dados = {
            'nomeServico': 'HISTORICOACESSO',
            'dados': str(re.sub('[^0-9]', '', brw_adm_pessoa.cpf_cnpj)),
        }
        resposta = requests.post(
            url=parametros.get('url_padrao') + 'EnviaRecebeDados',
            data=str(dados),
            headers=parametros.get('headers'),
            proxies=parametros.get('proxies'),
            timeout=180,
        )
        res = json.loads(resposta.content)
        return res

    def imeutl_get_codigo_faixa_enquadramento(self, cr, uid, regime_tributario, context=None):
        context = context and context.copy() or {}
        cod_faixa = 0
        # Lucro Presumido
        if regime_tributario == 'P':
            cod_faixa = 98
        # Lucro Real
        elif regime_tributario == 'R':
            cod_faixa = 99
        else:
            # Simples Nacional
            receita_anual = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'fiscal', 'fispar_receita_anual', context=context).get(
                'fispar_receita_anual') or 0
            if receita_anual <= 180000:
                cod_faixa = 101
            elif 180000.01 <= receita_anual <= 360000:
                cod_faixa = 102
            elif 360000.01 <= receita_anual <= 720000:
                cod_faixa = 103
            elif 720000.01 <= receita_anual <= 1800000:
                cod_faixa = 104
            elif 1800000.01 <= receita_anual <= 3600000:
                cod_faixa = 105
            elif 3600000.01 <= receita_anual <= 4800000:
                cod_faixa = 105
        return cod_faixa

    def imeutl_importa_retorno_revisao(self, cr, uid, dados_retorno, context):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                    context=context)
        # simples_nacional = self.pool.get('res.company').browse(
        #     cr, uid, company_id, context).rescon_pessoa_id.admpes_jur_regime_tributario == 'S'
        # obj_estoque_produto = self.pool.get('estoque.produto')
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        obj_fiscal_ncm = self.pool.get('fiscal.ncm')
        obj_fiscal_cest = self.pool.get('fiscal.cest')
        # obj_adm_utils = self.pool.get('adm.utils')
        # estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
        #     cr, uid, 'estoque', ['codigo_numerico', 'estpar_tamanho_codigo'], context=context)

        list_vals_itens = list()
        # if context.get('imendes_produto_id'):
        #     produto_ids = [context.get('imendes_produto_id')]
        # elif context.get('imendes_ncm_id'):
        #     produto_ids = obj_estoque_produto.search(
        #         cr, uid, [('ncm_id', '=', context.get('imendes_ncm_id'))], context=context)
        #     dados_retorno = [dados_retorno[0]]
        # else:
        #     codigo_interno_list = map(lambda p: p.get('RevisaoFiscal').get('CodigoInterno'),
        #                               dados_retorno)
        #     produto_ids = obj_estoque_produto.search(
        #         cr, uid, [('codigo', 'in', codigo_interno_list)], context=context)
        codigo_barra_list = []
        map(lambda p: codigo_barra_list.extend(p.get('produto')), dados_retorno)
        codigo_barra_list = list(set(map(lambda p: p.zfill(14), codigo_barra_list)))
        codigo_barra_ids = obj_automacao_codigo_barra.search(
            cr, uid, [('codigo_barra', 'in', codigo_barra_list)], context=context)
        automacao_codigo_barra_dados = obj_automacao_codigo_barra.read(
            cr, uid, codigo_barra_ids, ['produto_id'], context, '_classic_write')
        produto_ids = list(set(map(lambda p: p.get('produto_id'), automacao_codigo_barra_dados)))
        cadastro_produto_dados = self.imeutl_get_produto_dados(cr, uid, produto_ids, context)
        if not cadastro_produto_dados:
            return False
        # segmento_codigo_interno_list = []
        # segmento_codigo_ean_list = []
        for grupo in dados_retorno:
            list_codigo_ean = grupo.get('produto') or []
            pis_cofins_dados = grupo.get('piscofins')
            novo_pis_cofins_cst_saida = pis_cofins_dados.get('cstSai') or ''
            # if simples_nacional and novo_pis_cofins_cst_saida in ('01', '06'):
            #     novo_pis_cofins_cst_saida = '49'
            ipi_dados = grupo.get('ipi')
            # So MG
            icms_dados = grupo.get('regra')[0]

            # produto_dados = []
            # if context.get('imendes_produto_id'):
            #     produto_dados = [cadastro_produto_dados[0]]
            # elif context.get('imendes_ncm_id'):
            #     produto_dados = cadastro_produto_dados
            # else:
            #     if estoque_parametro_dados.get('codigo_numerico'):
            #         codigo_interno = codigo_interno.zfill(estoque_parametro_dados.get(
            #             'estpar_tamanho_codigo'))
            #     if codigo_interno and codigo_ean:
            #         codigo_ean = codigo_ean.zfill(14)
            #         produto_dados = filter(
            #             lambda p: p.get('codigo_interno') == codigo_interno
            #             and p.get('codigo_ean') == codigo_ean, cadastro_produto_dados)
            #     elif codigo_interno:
            #         produto_dados = filter(
            #             lambda p: p.get('codigo_interno') == codigo_interno, cadastro_produto_dados)
            #     elif codigo_ean:
            #         codigo_ean = codigo_ean.zfill(14)
            #         produto_dados = filter(
            #             lambda p: p.get('codigo_ean') == codigo_ean, cadastro_produto_dados)
            list_codigo_ean_14 = map(lambda cb: cb.zfill(14), list_codigo_ean)
            produto_dados = filter(
                lambda p: p.get('codigo_ean') in list_codigo_ean_14, cadastro_produto_dados)
            if not produto_dados:
                continue

            novo_ncm = grupo.get('ncm').replace('.', '')
            novo_ncm_id = obj_fiscal_ncm.search(cr, uid, [('codigo', '=', novo_ncm)], limit=1,
                                                context=context)
            novo_ncm_id = novo_ncm_id[0] if novo_ncm_id else False
            if not novo_ncm_id and novo_ncm:
                novo_ncm_id = obj_fiscal_ncm.create(cr, uid, {'codigo': novo_ncm}, context)

            cest_codigo = (icms_dados.get('cest') or grupo.get('cest')).replace('.', '')
            novo_cest_id = obj_fiscal_cest.search(cr, uid, [('fiscest_codigo', '=', cest_codigo)],
                                                  limit=1, context=context)
            novo_cest_id = novo_cest_id[0] if novo_cest_id else False
            if not novo_cest_id and cest_codigo:
                novo_cest_id = obj_fiscal_cest.create(cr, uid, {'fiscest_codigo': cest_codigo},
                                                      context)

            aliquota_icms = icms_dados.get('pICMSPDV')

            novo_icms_cst = (icms_dados.get('cst') or icms_dados.get('csosn'))[1:3]
            novo_icms_base = 100.0
            if novo_icms_cst in ('20', '70'):
                novo_icms_base = novo_icms_base - (icms_dados.get('reducaobcicms') or 0)
                aliquota_icms = icms_dados.get('aliqicms')

            for dado in produto_dados:
                # if codigo_interno and codigo_interno in segmento_codigo_interno_list:
                #     continue
                # if codigo_ean and codigo_ean in segmento_codigo_ean_list:
                #     continue
                # if codigo_interno:
                #     segmento_codigo_interno_list.append(codigo_interno)
                # if codigo_ean:
                #     segmento_codigo_ean_list.append(codigo_ean)
                aliquota_fcp = icms_dados.get('fcp')
                if aliquota_fcp and aliquota_icms:
                    aliquota_icms -= aliquota_fcp
                vals_itens = {
                    'estfit_produto_id': dado.get('produto_id'),
                    'estfit_cad_ncm': dado.get('ncm'),
                    'estfit_cad_cest': dado.get('cest'),
                    'estfit_cad_icms_cst': dado.get('cst_icms_saida'),
                    'estfit_cad_icms_aliquota': dado.get('aliquota_icms_saida'),
                    'estfit_cad_icms_fcp_aliquota': dado.get('aliquota_icms_fcp'),
                    'estfit_cad_icms_base': dado.get('base_icms_saida'),
                    'estfit_cad_pis_cst_entrada': dado.get('cst_pis_cofins_entrada'),
                    'estfit_cad_pis_cst_saida': dado.get('cst_pis_cofins_saida'),
                    'estfit_cad_pis_aliquota_entrada': dado.get('aliquota_pis_entrada'),
                    'estfit_cad_pis_aliquota_saida': dado.get('aliquota_pis_saida'),
                    'estfit_cad_pis_nat_rec': dado.get('natureza_receita_pis_cofins'),
                    'estfit_cad_cofins_cst_entrada': dado.get('cst_pis_cofins_entrada'),
                    'estfit_cad_cofins_cst_saida': dado.get('cst_pis_cofins_saida'),
                    'estfit_cad_cofins_aliquota_entrada': dado.get('aliquota_cofins_entrada'),
                    'estfit_cad_cofins_aliquota_saida': dado.get('aliquota_cofins_saida'),
                    'estfit_cad_cofins_nat_rec': dado.get('natureza_receita_pis_cofins'),
                    'estfit_cad_cod_ben_fis_uf': dado.get('codigo_beneficio_fiscal_uf'),

                    'estfit_novo_ncm': grupo.get('ncm').replace('.', ''),
                    'estfit_novo_cest': cest_codigo,
                    'estfit_novo_icms_cst': novo_icms_cst,
                    'estfit_novo_icms_cont_cst': ''[1:3],
                    'estfit_novo_icms_aliquota': aliquota_icms,
                    'estfit_novo_icms_fcp_aliquota': aliquota_fcp,
                    'estfit_novo_icms_cont_aliquota': 0,
                    'estfit_novo_icms_base': novo_icms_base,
                    'estfit_novo_icms_red_base_cont': icms_dados.get('reducaobcicms') or 0,
                    'estfit_novo_cod_ben_fis_uf': icms_dados.get('codBenef') or '',
                    'estfit_novo_pis_cst_entrada': (pis_cofins_dados.get('cstEnt') or ''),
                    'estfit_novo_pis_cst_saida': novo_pis_cofins_cst_saida,
                    'estfit_novo_pis_aliquota_entrada': pis_cofins_dados.get('aliqPIS') or 0,
                    'estfit_novo_pis_aliquota_saida': pis_cofins_dados.get('aliqPIS') or 0,
                    'estfit_novo_pis_nat_rec': pis_cofins_dados.get('nri') or '',
                    'estfit_novo_cofins_cst_entrada': (pis_cofins_dados.get('cstEnt') or ''),
                    'estfit_novo_cofins_cst_saida': novo_pis_cofins_cst_saida,
                    'estfit_novo_cofins_aliquota_entrada': pis_cofins_dados.get('aliqCOFINS') or 0,
                    'estfit_novo_cofins_aliquota_saida': pis_cofins_dados.get('aliqCOFINS') or 0,
                    'estfit_novo_cofins_nat_rec': pis_cofins_dados.get('nri') or '',

                    'estfit_novo_icms_tribut_id': self.imeutl_get_nova_tributacao_id(
                        cr, uid, 'icms', (icms_dados.get('cst') or '')[1:3], 0,
                        icms_dados.get('aliqicms') or 0, context),
                    'estfit_novo_icms_cont_tribut_id': self.imeutl_get_nova_tributacao_id(
                        cr, uid, 'icms', ''[1:3], 0, 0, context),
                    'estfit_novo_pis_tribut_id': self.imeutl_get_nova_tributacao_id(
                        cr, uid, 'pis', novo_pis_cofins_cst_saida, 0,
                        pis_cofins_dados.get('aliqPIS') or 0, context),
                    'estfit_novo_cofins_tribut_id': self.imeutl_get_nova_tributacao_id(
                        cr, uid, 'cofins', novo_pis_cofins_cst_saida, 0,
                        pis_cofins_dados.get('aliqCOFINS') or 0, context),
                }
                novo_cest_id and vals_itens.update({'estfit_novo_cest_id': novo_cest_id})
                novo_ncm_id and vals_itens.update({'estfit_novo_ncm_id': novo_ncm_id})
                list_vals_itens.append((0, 0, vals_itens))
        if not list_vals_itens:
            return False
        vals = {'estfca_data': time.strftime('%Y-%m-%d'), 'estfca_origem': 'IMendes',
                'estfca_item_ids': list_vals_itens}
        return self.pool.get('estoque.fiscal.capa').create(cr, uid, vals, context)

    def imeutl_get_nova_tributacao_id(self, cr, uid, encargo, cst, base, aliquota, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT ft.id
            FROM fiscal_tributacao ft
            INNER JOIN fiscal_encargo fe
                ON fe.id = ft.encargo_id
            INNER JOIN fiscal_cst fc
                ON fc.id = ft.cst_id
            WHERE
                fe.sigla = '%s'
                AND fc.codigo = '%s'
                AND ft.aliquota = %s
        """ % (encargo.upper(), cst, aliquota)
        if base:
            sql += """
                AND ft.base = %s
            """ % base
        sql += """
            LIMIT 1
        """
        cr.execute(sql)
        tributacao_id = cr.fetchone()
        return tributacao_id or False

    def imeutl_get_produto_dados(self, cr, uid, produto_ids, context=None):
        """
        O metodo reune todos os dados necessarios do cadastro dos produtos para a importacao da
        revisao fiscal.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: valores contextuais.
        :type context: dict
        :return: dados do cadastro dos produtos.
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                    context=context)
        if not produto_ids:
            return list()
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))

        sql = """
            SELECT
                DISTINCT
                ep.id               AS produto_id,
                epg.id              AS grade_id,
                ep.codigo           AS codigo_interno,
                acb.codigo_barra    AS codigo_ean,
                CASE
                    WHEN substr(acb.codigo_barra, 1, 13)::bigint = ep.codigo::bigint
                        THEN 1
                    ELSE 0
                END                 AS codigo_ean_interno,
                CASE
                    WHEN (SELECT estpar_descricao_grade 
                          FROM estoque_parametro WHERE company_id = 1) = 'P'
                        THEN ep.name || '-' || epg.descricao
                    ELSE
                        epg.descricao
                END                 AS descricao_produto,
                fn.codigo           AS ncm,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'PIS')
                     )
                )                   AS aliquota_pis_saida,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'PIS')
                     )
                )                   AS aliquota_pis_entrada,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'COFINS')
                     )
                )                   AS aliquota_cofins_saida,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'COFINS')
                     )
                )                   AS aliquota_cofins_entrada,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla IN ('PIS', 'COFINS'))
                     )
                )                   AS cst_pis_cofins_saida,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla IN ('PIS', 'COFINS'))
                     )
                )                   AS cst_pis_cofins_entrada,
                CASE
                    WHEN ep.estpro_pis_nat_rec IS NOT NULL THEN ep.estpro_pis_nat_rec
                    WHEN ep.estpro_cofins_nat_rec IS NOT NULL THEN ep.estpro_cofins_nat_rec
                    ELSE ''
                END                 AS natureza_receita_pis_cofins,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS aliquota_icms_saida,
                (SELECT fistri_aliquota_fcp
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS aliquota_icms_fcp,
                (SELECT base
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS base_icms_saida,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS cst_icms_saida,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'U' THEN 1
                    ELSE 0
                END                 AS substituicao_tributaria,
                0                   AS cesta_basica,
                CASE
                    WHEN ep.origem_mercadoria IN ('1','2','6','7')
                        THEN 1
                    ELSE 0
                END                 AS importado,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'I' THEN 1
                    ELSE 0
                END                 AS isento,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'D' THEN 1
                    ELSE 0
                END                 AS diferido,
                ''                  AS tipo_lista,
                (SELECT fiscest_codigo
                 FROM fiscal_cest
                 WHERE id = ep.estpro_cest_id)
                                    AS cest,
                (SELECT fiscst_codigo_beneficio
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS codigo_beneficio_fiscal_uf
            FROM estoque_produto ep
            INNER JOIN estoque_produto_grade epg
                ON ep.id = epg.produto_id
            INNER JOIN automacao_codigo_barra acb
                ON ep.id = acb.produto_id AND epg.id = acb.grade_id
            INNER JOIN fiscal_ncm fn
                ON fn.id = ep.ncm_id
            WHERE
                ep.id IN {produto_ids}
                AND ep.company_id = {company_id}
        """.format(produto_ids=produto_ids, company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()

        return dados

    def imeutl_atualiza_produto_pesquisado(self, cr, uid, produto_ids, context=None):
        """
        O metodo atualiza todos os produtos recebidos (produto_ids) informando que foram
        consultados junto ao sistema IMendes.
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param produto_ids: lista de registros da tabela estoque_produto
        :type produto_ids: list
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            UPDATE 
                estoque_produto 
            SET 
                write_date = '{data_atualizacao}', 
                write_uid = {uid}, 
                estpro_imendes_enviado = TRUE 
            WHERE
                id IN ({produto_ids})
        """.format(data_atualizacao=time.strftime('%Y-%m-%d %H:%M:%S'), uid=uid,
                   produto_ids=', '.join(map(lambda p: str(p), produto_ids)))
        cr.execute(sql)
        cr.commit()
        return True

