# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_imendes_historico_acesso_resumo(osv.TransientModel):

    _name = 'wizard.imendes.historico.acesso.resumo'
    _columns = {
        'data_primeiro_consumo': fields.date('Data primeiro consumo'),
        'data_ultimo_consumo': fields.date('Data ultimo consumo'),
        'produto_pendente_interno': fields.integer(
            'Produtos pendentes (codigo interno)', help='Numero de produtos pendentes sendo codigo '
                                                        'interno'),
        'produto_pendente_ean': fields.integer(
            'Produtos pendentes (EAN)', help='Numero de produtos pendentes sendo codigo de barras'),
        'produto_pendente_devolvido': fields.integer(
            'Produtos devolvidos',
            help='Produtos que nao serao processados de acordo com a observacao: '
                 'DI – Descricao insuficiente, SC – Produto sem codigo e '
                 'CV – Codigo de barras com dígito verificador inválido'),
        'produto_pendente_data_inicio': fields.date(
            'Data de produto pendente',
            help='Data mais antiga que um produto se encontra pendente'),
        'detalhe_ids': fields.one2many('wizard.imendes.historico.acesso.detalhe', 'resumo_id',
                                       'Detalhes'),
        'devolvido_ids': fields.one2many('wizard.imendes.historico.acesso.devolvido', 'resumo_id',
                                         'Devolvidos'),
    }

    def button_get_access_history(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados = self.pool.get('fiscal.imendes.utils').imeutl_get_access_history(cr, uid, context)
        if not dados:
            raise osv.except_osv(_('Atencao'), _('Cliente nao posssui historico de acesso.'))
        resumo_dados = dados.get('resumo')
        detalhe_dados = dados.get('detalhes')
        devolvido_dados = dados.get('prodDevolvidos')
        vals_resumo = {
            'data_primeiro_consumo': resumo_dados.get('dataPrimeiroConsumo'),
            'data_ultimo_consumo': resumo_dados.get('dataUltimoConsumo'),
            'produto_pendente_interno': resumo_dados.get('produtosPendentes_Interno'),
            'produto_pendente_ean': resumo_dados.get('produtosPendentes_EAN'),
            'produto_pendente_devolvido': resumo_dados.get('produtosPendentes_Devolvidos'),
            'produto_pendente_data_inicio': resumo_dados.get('produtosPendentes_DataInicio')
        }
        detalhe_ids = []
        for detalhe in detalhe_dados:
            vals_detalhe = {
                'data_acesso': detalhe.get('data'),
                'metodo_executado': detalhe.get('metodo'),
                'mensagem_recebida': detalhe.get('msg'),
                'qtd_requisicao_dia': detalhe.get('requisicoes'),
                'qtd_produto_enviado': detalhe.get('enviados'),
                'qtd_produto_retornado': detalhe.get('retornados')
            }
            detalhe_ids.append((0, 0, vals_detalhe))
        detalhe_ids and vals_resumo.update({'detalhe_ids': detalhe_ids})
        devolvido_ids = []
        for devolvido in devolvido_dados:
            vals_devolvido = {
                'identificacao_devolvido': devolvido.get('id'),
                'codigo_produto': devolvido.get('codigo'),
                'descricao_produto': devolvido.get('descricao'),
                'data_inclusao': devolvido.get('dtinclusao'),
                'data_devolucao': devolvido.get('dtdevolucao'),
                'motivo_devolucao': devolvido.get('motivodevolucao'),
            }
            devolvido_ids.append((0, 0, vals_devolvido))
        devolvido_ids and vals_resumo.update({'devolvido_ids': devolvido_ids})
        self.write(cr, uid, ids, vals_resumo, context)
        return True


class wizard_imendes_historico_acesso_detalhe(osv.TransientModel):

    _name = 'wizard.imendes.historico.acesso.detalhe'
    _columns = {
        'resumo_id': fields.many2one('wizard.imendes.historico.acesso.resumo', 'Resumo',
                                     ondelete='cascade', required=True),
        'data_acesso': fields.date('Data de acesso'),
        'metodo_executado': fields.char('Servico consumido', size=80),
        'mensagem_recebida': fields.char('Mensagem de retorno', size=80),
        'qtd_requisicao_dia': fields.integer('Requisicoes realizadas'),
        'qtd_produto_enviado': fields.integer('Produtos enviados'),
        'qtd_produto_retornado': fields.integer('Produtos retornados')
    }


class wizard_imendes_historico_acesso_devolvido(osv.TransientModel):

    _name = 'wizard.imendes.historico.acesso.devolvido'
    _columns = {
        'resumo_id': fields.many2one('wizard.imendes.historico.acesso.resumo', 'Resumo',
                                     ondelete='cascade', required=True),
        'identificacao_devolvido': fields.integer('Id produto Imendes'),
        'codigo_produto': fields.char('Codigo produto', size=14),
        'descricao_produto': fields.char('Descricao', size=60),
        'data_inclusao': fields.date('Data inclusao'),
        'data_devolucao': fields.date('Date devolucao'),
        'motivo_devolucao': fields.selection(
            [('DI', 'Descricao insuficiente'), ('SC', 'Produto sem codigo'),
             ('CV', 'Codigo de barras com digito verificador invalido')], size=2,
            string='Motivo devolucao'),
    }
