# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import datetime

__author__ = 'felipe.franca'
__lastupdate__ = '2017-07-07T09:19:57 <pierre.barbosa>'


class wizard_fiscal_nf_itens_local(osv.TransientModel):
    _name = "wizard.fiscal.nf.itens.local"  # wizfni
    _columns = {
        'wizfni_company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade",
                                             required=True),
        'wizfni_tipo': fields.selection([('E', 'Entrada'), ('S', 'Saida')],
                                         required=True, string="Entrada/Saida", size=1),
        'wizfni_natureza_id': fields.many2one("fiscal.cfop.natureza", 'Natureza', required=True,
                                              domain="[('tipo', '=', wizfni_tipo)]"),
        'wizfni_cliente_id': fields.many2one("adm.pessoa", 'Cliente', required=True),
        'wizfni_endereco_id': fields.many2one("adm.pessoa.endereco", 'Endereco',
                                              domain="[('pessoa_id', '=', wizfni_cliente_id)]",
                                              required=True),
        'wizfni_local_id': fields.many2one("estoque.local", 'Local', required=True),
        'wizfni_fator': fields.integer("Fator p/divisao de custo"),
        'wizfni_entrada_local_id': fields.many2one("estoque.local", 'Local entrada'),
        'wizfni_max_itens': fields.integer("Itens por nota", required=True,
                                           help="Maximo de itens por nota(Maximo=900, Minimo=1)"),
        'wizfni_preco': fields.selection([('PV', 'Preco de venda'), ('UC', 'Ultimo custo'),
                                          ('CC', 'Custo corrigido'), ('CM', 'Custo medio')],
                                         required=True, string="Preco / custo", size=2),
        'state': fields.selection([('1', 'Esboco'), ('2', 'Criada')], 'Status', size=1,),

    }
    _defaults = {
        'wizfni_max_itens': 100,
        'wizfni_company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'wizfni_fator': 1,
        'state': '1',
        'wizfni_tipo' : 'S',
    }

    def wizfni_button_gerar_nota(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_adm_empresa = self.pool.get('adm.empresa.objeto')
        obj_movimento_itens = self.pool.get('estoque.movimento.itens')
        company_id = obj_adm_empresa.getEmpresa(cr, uid, tabela="fiscal.nf.capa", context=context)
        eps_company_id = obj_adm_empresa.getEmpresa(cr, uid, tabela="estoque.produto.saldo",
                                                    context=context)
        context_nota = context.copy()
        context_nota.update({'ctx_nf_propria': True,
                             'proprio': True,
                             'nota_fiscal_saida': True,
                             'wizfni_nota_por_local': True,  # Cria tributacao dos itens
                            })
        context['proprio'] = True
        for brw_wiz in self.browse(cr, uid, ids, context=context):
            _fator = brw_wiz.wizfni_fator
            if _fator < 1:
                _fator = 1
            mov_local_id = local_id = brw_wiz.wizfni_local_id.id
            if brw_wiz.wizfni_tipo == 'E':
                mov_local_id = brw_wiz.wizfni_entrada_local_id.id
                context_nota.pop('nota_fiscal_saida')
                context['nota_fiscal_entrada'] = True
                context_nota['nota_fiscal_entrada'] = True

            maximo_por_nota = brw_wiz.wizfni_max_itens
            locacao_ids = self.pool.get('estoque.local.locacao').search(cr, uid,
                                      [('local_id', '=', local_id)],
                                                                        context=context)
            dados_locacao = self.pool.get('estoque.local.locacao').read(
                cr, uid, locacao_ids, ['grade_id', 'produto_id', 'estllc_produto_embalagem_id'],
                context=context, load='_classic_write')
            grade_ids = map(lambda x: x.get('grade_id'), dados_locacao)
            produto_ids = map(lambda x: x.get('produto_id'), dados_locacao)
            grade_embalagem = {}
            map(lambda x: grade_embalagem.update({x.get('grade_id'):
                                                  x.get('estllc_produto_embalagem_id')}),
                dados_locacao)
            saldos = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                cr, uid, grade_ids, str(datetime.now().date()) + " 23:59:59",
                eps_local_id=local_id, company_id=eps_company_id, context=context)
            movimento_itens = []
            if brw_wiz.wizfni_preco == 'PV':
                precos = self.pool.get('estoque.preco.venda').get_preco_venda(
                    cr, uid, produto_ids) or dict()
                get_preco = 'preco_venda'
            else:
                precos = self.pool.get('estoque.preco.custo').get_custos_do_dia(
                    cr, uid, produto_ids) or dict()
                get_preco = dict(UC='estprc_ultimo_custo', CC='estprc_custo_corrigido',
                                 CM='estprc_custo_medio_contabil').get(brw_wiz.wizfni_preco)
            cont = sequencia = 0
            criou_nota = False
            grade_qtd_na_embalagem = self.pool.get('estoque.utils').get_grade_qtd_embalagem(
                cr, grade_ids, context=context)
            for grade in self.pool.get('estoque.produto.grade').read(
                    cr, uid, grade_ids, ['produto_id'], context=context, load='_classic_write'):
                grade_id = grade.get('id')
                produto_emb_id = grade_embalagem.get(grade_id)
                chave = '%s-%s' % (grade_id, produto_emb_id)
                saldo = saldos.get(grade_id)
                saldo = saldo and saldo.get('saldo_na_embalagem')
                saldo = saldo and saldo.get(produto_emb_id)
                saldo = saldo and saldo.get('disponivel')
                produto_id = grade.get('produto_id')
                qtd_na_embalagem = grade_qtd_na_embalagem.get(chave).get('qtd') if grade_qtd_na_embalagem.get(chave) else 1
                produto_preco = precos.get(produto_id) or dict()
                vr_unitario = (produto_preco.get(get_preco) or 0 * qtd_na_embalagem) / _fator
                vr_total_item = saldo and vr_unitario * saldo or 0

                # Apenas os itens com total do item maior que zero
                if not vr_total_item > 0:
                    continue

                sequencia += 1
                movimento_itens.append([0, False, {
                    'produto_id': produto_id, 'grade_id': grade_id, 'local_id': mov_local_id,
                    'produto_embalagem_id': produto_emb_id, 'quantidade': saldo or 0,
                    'vr_unitario': vr_unitario, 'vr_preco_venda': vr_unitario, 'sequencia': sequencia,
                    'vr_total_item': vr_total_item, 'vr_total_item_rateado': vr_total_item,
                    # Nao permite calcular o ´vr_unitario´ no ´create´ do item
                    'estmvi_permissao_altera_preco': True
                }])
                cont += 1
                if cont < maximo_por_nota:
                    continue
                cont = 0

                # Gera uma nota com o a quantidade maxima de itens passada
                self.pool.get('fiscal.utils').gerar_nf(
                    cr, uid, company_id, brw_wiz.wizfni_cliente_id.id,
                    tipo=brw_wiz.wizfni_tipo,
                    natureza_id=brw_wiz.wizfni_natureza_id.id,
                    estoque_movimento_itens_ids=movimento_itens,
                    destinatario_endereco_id=brw_wiz.wizfni_endereco_id.id, context=context_nota)

                criou_nota = True
                movimento_itens = []
            if movimento_itens:
                self.pool.get('fiscal.utils').gerar_nf(
                    cr, uid, company_id, brw_wiz.wizfni_cliente_id.id,
                    tipo=brw_wiz.wizfni_tipo,
                    natureza_id=brw_wiz.wizfni_natureza_id.id,
                    estoque_movimento_itens_ids=movimento_itens,
                    destinatario_endereco_id=brw_wiz.wizfni_endereco_id.id, context=context_nota)
                criou_nota = True

        if not criou_nota:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha itens com saldo disponivel neste local!'))
        self.write(cr, uid, ids, {'state': '2'}, context=context)

        return True

    def onchange_cliente_id(self, cr, uid, ids, cliente_id, natureza_id, context=None):
        context = context and context.copy() or {}
        if not cliente_id:
            return {'value': {'wizfni_endereco_id': False}}

        if not natureza_id:
            return {'value': {'wizfni_cliente_id': False},
                    'warning': {'title': _('Atencao!'),
                                'message': _('Escolha uma natureza')}}

        obj_adm_empresa = self.pool.get('adm.empresa.objeto')
        vals={}
        warning = {}
        tipo = 'S'  # nota fiscal de saida
        endereco = self.pool.get('adm.pessoa').read(
            cr, uid, cliente_id, ['endereco_nf_id', 'endereco_ids'], context, '_classic_write')
        endereco_id = False
        if endereco:
            endereco_id = endereco.get('endereco_nf_id')
            if not endereco_id:
                endereco_id = endereco.get('endereco_ids')
                if endereco_id:
                    endereco_id = endereco_id[0]

        vals['wizfni_endereco_id'] = endereco_id
        tipo_movimento_cfop = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, natureza_id, context).cfop_id.movimento_tipo
        # Verifica restricao de clientes somente para natureza onde o CFOP tenha o tipo de
        #  movimento "compra/venda"
        fc_company_id = obj_adm_empresa.getEmpresa(cr, uid, tabela="financeiro.cliente",
                                                   context=context)
        if tipo_movimento_cfop == 'C':
            if self.pool.get('fiscal.nf.capa').verificar_incluir_cli_for(cr, uid, tipo, cliente_id,
                                                        fc_company_id, 'verificar',context=context):
                pessoa_com_restricao = \
                    self.pool.get('financeiro').finutl_verificar_restricao_cliente(
                        cr, uid, finutl_pessoa_id=cliente_id, context=context)

                if pessoa_com_restricao:
                    msg = _(' Cliente possui restricoes de venda.')
                    if 'message' in warning:
                        warning['message'] += msg
                    else:
                        warning = {'title': _('Atencao!'), 'message': msg}
            else:
                warning['title'] = _('Atencao!')
                nome_pessoa = self.pool.get('adm.pessoa').browse(
                    cr, uid, cliente_id, context).nome
                warning['message'] = _('%s nao e um cliente.' % nome_pessoa)
                vals['wizfni_cliente_id'] = False

        dados_pessoa = self.pool.get('adm.pessoa').read(cr, uid, cliente_id,  ['cpf_cnpj'],
                                                        context=context, load='_classic_write')
        if not dados_pessoa.get('cpf_cnpj'):
            vals['wizfni_cliente_id'] = False
            warning = {'title': _('Atencao!'), 'message': _('Destinatario nao possui CPF/CNPJ.')}

        return {
            'value': vals,
            'warning': warning
        }

    def onchange_maximo(self, cr, uid, ids, maximo, context=None):
        context = context and context.copy() or {}
        if maximo > 900:
            return {
                'value': {'wizfni_max_itens': 900},
                'warning': {'title': _('Atencao!'),
                            'message': 'O maximo de itens por nota e 900!'}
            }
        elif maximo < 1:
            return {
                'value': {'wizfni_max_itens': 1},
                'warning': {'title': _('Atencao!'),
                            'message': 'O minimo de itens por nota e 1!'}
            }
        return {}
