# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import fields
from openerp.osv import osv


class fluxocaixa_previsao(osv.Model):
    """
    Tabela com as previsoes de despesas e receitas. Sao documentos que foram
    efetivados, mas nao chegaram a empresa (agua, luz, etc.).
    """
    _name = 'fluxocaixa.previsao'
    _columns = {
        # ___________________________ Integer ________________________________
        'dia': fields.integer('Dia', required=True),  # Dia em que ocorre a previsao

        # ___________________________ Many2One _______________________________
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True,
                                      select=True),  # Id da empresa
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente/Fornecedor', ondelete="restrict",
                                     select=True, required=True),  # Id da pessoa

        # ___________________________ One2Many _______________________________
        'titulo_ids': fields.one2many('financeiro.titulo', 'previsao_id', 'Titulo'),

        # ___________________________ Float __________________________________
        'valor': fields.float('Valor', digits=(9, 2), required=True),  # Valor da previsao

        # ___________________________ Date ___________________________________
        'periodo_i': fields.date('Validade inicial'),  # Data inicial de validade da previsao
        'periodo_f': fields.date('Validade final'),  # Data final de validade da previsao

        # ___________________________ Char ___________________________________
        'descricao': fields.char('Descricao', size=40, required=True), # Descricao da previsao

        # ___________________________ Selection ______________________________
        'tipo': fields.selection([('R', 'Receber'), ('P', 'Pagar')], 'Tipo', required=True, size=1),
        # Fix: trocar para o nome correto: incidencia
        'insidencia': fields.selection([('D', 'Diario'), ('S', 'Semanal'), ('Q', 'Quinzenal'),
                                        ('M', 'Mensal'), ('T', 'Trimestral'), ('U', 'unico')], 'Incidencia',
                                       required=True, size=1),
        'dia_semana': fields.selection([('', ''), ('0', 'Segunda'), ('1', 'Terca'), ('2', 'Quarta'),
                                        ('3', 'Quinta'), ('4', 'Sexta'), ('5', 'Sabado'),
                                        ('6', 'Domingo')], 'Dia da Semana', size=1),
        'tipo_dia': fields.selection([('', ''), ('D', 'Dia direto'), ('U', 'dia util')], 'Tipo dia',
                                     size=1),
        'origem': fields.selection([('S', 'Sistema'), ('U', 'Usuario')], 'Origem', size=1),
        'mes': fields.selection([('1', '1'), ('2', '2'), ('3', '3')], 'Mes', size=1,
                              attrs="{'invisible': [('insidencia','!=','T')], 'required': [('insidencia','=','T')]}",
                              help="O mês deve ser 1, 2 ou 3. Primeiro, segundo ou terceiro mês do trimestre."),

        # ___________________________ Boolean ________________________________
        'antecipa_feriado': fields.boolean('Antecipa feriado'),  # Flag que indica se o vencimento sera antecipado se ocorrer em feriados
        'sabado': fields.boolean('Sabado'),  # Valido apenas para periodo diario. Indica se a previsao tambem acontecera no dia de sabado
        'domingo': fields.boolean('Domingo'),  # Valido apenas para periodo diario. Indica se a previsao tambem acontecera no dia de domingo
    }
    _defaults = {
        'antecipa_feriado': lambda *a: False,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'insidencia': lambda *a: 'M',  # Fix: Trocar para o nome correto: incidencia
        'tipo_dia': lambda *a: 'D',
        'origem': lambda *a: 'U',
    }

