# -*- coding: utf-8 -*-


from openerp import pooler
from openerp.osv import osv, fields
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
from SigUtils import datas
from openerp.tools.translate import _
import time
from datetime import date
from dateutil.relativedelta import relativedelta
from dateutil.parser import parse


class wiz_fluxocaixa_posicao_financeira_capa(osv.TransientModel):
    _name = 'wiz.fluxocaixa.posicao.financeira.capa'
    _rec_name = 'conta_id'
    _description = 'Posicao Financeira'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', required=True),
        'saldo': fields.float('Saldo Inicial', digits=(15, 2), required=True),
        'data_ini': fields.date('Periodo Inicial', ),
        'data_fim': fields.date('Periodo Final', ),
        'corpo_ids': fields.one2many('wiz.fluxocaixa.posicao.financeira.corpos', 'capa_id', 'Posicoes'),
        'movimento_ids': fields.one2many('wiz.fluxocaixa.posicao.financeira.movimento', 'capa_id', 'Posicoes'),
        'state': fields.char('Status', size=1),
        'conta_id': fields.many2one("financeiro.tesouraria.contas", 'Conta', ondelete="restrict"),
        'considerar_disponibilidade': fields.boolean('Considerar Disponibilidade'),
        'considerar_previsao': fields.boolean('Considerar Previsao'),
        'titulo_vencido_ids': fields.one2many(
            'wiz.fluxocaixa.titulo.vencido', 'fluxocaixa_posicao_financeira_id',
            'Titulos vencidos'),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id,
        'state': lambda *a: '0',
        'considerar_previsao':True,
        'considerar_disponibilidade':True,
        'data_ini': time.strftime('%Y-%m-%d'),
        'data_fim': time.strftime('%Y-%m-%d'),
    }

    def _verifica_disp(self, cr, uid, data, context=None):
        """
        Verifica Saldo disponivel de acordo com a data informada
        """

        context = context and context.copy() or {}
        obj_conta = self.pool.get("financeiro.tesouraria.contas")
        company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id.id
        id_conta = obj_conta.search(cr, uid, [('company_id', '=', company_id),('integra_fluxo','=',True)], context=context)

        # Localizando as contas filhas
        contas = []

        for i in id_conta:
            contas += [i]
            a = obj_conta.search(cr, uid, [('parent_id', '=', i)], context=context)

            if not a == []:
                contas += a
                # Subtraindo data inicial para o dia anterior

        y, m, d = (int(x) for x in data.split('-'))
        data_ant = date(y, m, d)

        # Calcula o saldo do dia
        obj_financeiro_tesouraria_saldos = self.pool.get('financeiro.tesouraria.saldos')
        saldo_dia = 0

        for i in contas:
            saldo_dia += obj_financeiro_tesouraria_saldos.get_saldo_do_dia(
                cr, uid, i, str(data_ant))

        return saldo_dia

    def onchange_calcula_disponibilidade(self, cr, uid, ids, considerar_disponibilidade,
                                         data_ini, context=None):
        """
        Calcula a disponibilidade de acordo com data infomada
        """

        context = context and context.copy() or {}
        vr_disponivel = 0
        warning = {}
        if not data_ini:
            return {'warning': {'title': _('Atencao!'),
                                'message': _('Data inicial vazia')}}

        data = (parse(data_ini) - relativedelta(days=1)).strftime('%Y-%m-%d')

        if considerar_disponibilidade:
            vr_disponivel = self._verifica_disp(cr, uid, data)
        else:
            considerar_disponibilidade = False

        return {'value': {'saldo': vr_disponivel,
                          'considerar_disponibilidade': considerar_disponibilidade},
                'warning': warning}

    def onchange_data_ini(self, cr, uid, ids, data_ini, conta_id, context=None):
        context = context and context.copy() or {}
        res = {}

        if data_ini:
            if conta_id:
                data_dia_ant = datas.calcVencimento(data_ini, -1)
                saldo = self.pool.get('financeiro.tesouraria.saldos').get_saldo_do_dia(cr, uid,
                                                                                       conta_id,
                                                                                       data_dia_ant)
                res['value'] = {'saldo': saldo}
            else:
                warning = {}
                warning['message'] = _('Selecione uma conta!')
                res = {'warning': warning, 'value': {'data_ini': False}}

        return res

    def button_limpar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {
            'saldo': False,
            'data_ini': False,
            'data_fim': False,
            'corpo_ids': [],
            'state': '0',
            'considerar_previsao': True,
            'considerar_disponibilidade':False,
        }, context=context)
        obj_corpos = self.pool.get('wiz.fluxocaixa.posicao.financeira.corpos')
        id_s = obj_corpos.search(cr, uid, [], context=context)
        obj_corpos.unlink(cr, uid, id_s)

        return True

    def button_limpar_tesouraria(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {
            'saldo': False,
            'conta_id': False,
            'data_ini': False,
            'data_fim': False,
            'corpo_ids': [],
            'state': '0',
        }, context)
        obj_movimentos = self.pool.get('wiz.fluxocaixa.posicao.financeira.movimento')
        id_s = obj_movimentos.search(cr, uid, [], context=context)
        obj_movimentos.unlink(cr, uid, id_s, context=context)

        return True

    def get_titulo_receber_vencido(self, cr, uid, context=None):
        """
        O metodo encontra os titulos a receber que estao vencidos. Nao considera previsao
        financeira.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: lista de dicionarios com os dados dos titulos.
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'financeiro.titulo',
                                                                    context=context)
        sql_titulo_vencido = """
            SELECT
                titulo.parcela          AS parcela,
                titulo.pessoa_id        AS pessoa_id,
                titulo.saldo            AS saldo,
                titulo.data_prorrogacao AS data_prorrogacao,
                titulo.documento        AS documento
            FROM
                financeiro_titulo titulo
            WHERE
                state = 'A'
                AND titulo.tipo_titulo = 'R'
                AND titulo.data_prorrogacao < CURRENT_DATE
                AND titulo.company_id = {company_id}
                AND titulo.doc_tipo_id NOT IN (
                    SELECT id FROM documento_documento_tipo WHERE sigla = 'PRV')
        """.format(company_id=company_id)
        cr.execute(sql_titulo_vencido)
        titulo_dados = list()
        map(lambda i: titulo_dados.append((0, 0, i)), cr.dictfetchall())
        return titulo_dados

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_wiz_fluxocaixa_titulo_vencido = self.pool.get('wiz.fluxocaixa.titulo.vencido')
        vals_read = self.read(cr, uid, ids[0], [], context=context)

        # Datas originais do periodo, serao usadas para limitar a insercao de itens (titulos)
        data_ini_o = vals_read.get('data_ini')
        data_fim_o = vals_read.get('data_fim')
        data_ini = vals_read.get('data_ini')
        data_fim = vals_read.get('data_fim')
        data_atual = str(date.today())

        if data_ini < data_atual:
            raise osv.except_osv(_('Atencao'), _('Data inicial anterior ao do dia da emissao'))
        if data_fim < data_ini:
            raise osv.except_osv(_('Atencao'), _('Data  final maior que data inicial'))

        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')

        # Buscar prazo para liberacao que tenha o maior valor
        local_natureza_ids = obj_financeiro_local_natureza.search(
            cr, uid, [('finlon_prazo_liberacao', '>', 0)], limit=1,
            order='finlon_prazo_liberacao DESC', context=context)

        if local_natureza_ids:
            prazo_liberacao = obj_financeiro_local_natureza.browse(
                cr, uid, local_natureza_ids[0]).finlon_prazo_liberacao

            if prazo_liberacao:
                # Aplica o prazo para liberacao nos periodos inicial e final
                data_ini = parse(data_ini, yearfirst=True)
                data_ini -= relativedelta(days=prazo_liberacao)
                data_ini = data_ini.strftime('%Y-%m-%d')
                data_fim = parse(data_fim, yearfirst=True)
                data_fim += relativedelta(days=prazo_liberacao)
                data_fim = data_fim.strftime('%Y-%m-%d')

        company_id = vals_read['company_id'][0]
        args = [('data', '>=', data_ini), ('data', '<=', data_fim), ('company_id', '=', company_id)]

        # Naturezas de cobranca sem recebimento antecipado
        natureza_cobranca_ids = self.pool.get('financeiro.natureza.cobranca').search(
            cr, uid, [('recebimento_antecipado', '=', True)], context=context)

        periodo_titulo = "AND titulo.data_prorrogacao BETWEEN '%s' AND '%s'" % (data_ini, data_fim)
        natureza_cobranca = ''
        if len(natureza_cobranca_ids) == 1:
            natureza_cobranca = 'AND (titulo.natureza_cobranca_id not IN %s or titulo.natureza_cobranca_id isnull)' % str(tuple(
                natureza_cobranca_ids)).replace(',', '')
        elif len(natureza_cobranca_ids) > 1:
            natureza_cobranca = 'AND (titulo.natureza_cobranca_id not IN %s or titulo.natureza_cobranca_id isnull)' % str(tuple(
                natureza_cobranca_ids))
        sql_titulo = """
            SELECT
                titulo.company_id as company_id,
                titulo.pessoa_id as pessoa_id,
                titulo.doc_tipo_id as doc_tipo_id,
                titulo.tipo_titulo as tipo,
                titulo.saldo as valor,
                titulo.data_prorrogacao as data,
                titulo.documento as historico,
                titulo.natureza_cobranca_id as natureza_cobranca_id,
                titulo.local_cobranca_id as local_cobranca_id
            FROM
                financeiro_titulo titulo
            WHERE
                state = 'A'
                {periodo_titulo}
                AND titulo.company_id = {company_id}
                {natureza_cobranca_id}
        """.format(periodo_titulo=periodo_titulo, company_id=company_id,
                   natureza_cobranca_id=natureza_cobranca)

        sql_tesouraria_movimento = """
            UNION ALL
            SELECT
                conta.company_id as company_id,
                null as pessoa_id,
                null as doc_tipo_id,
                movimento.debito_credito as tipo,
                movimento.valor as valor,
                movimento.data_movimento as data,
                movimento.historico as historico,
                null as natureza_cobranca_id,
                null as local_cobranca_id
            FROM
                financeiro_tesouraria_movimento movimento
            INNER JOIN
                financeiro_tesouraria_contas conta ON conta.id = movimento.conta_id
            WHERE
                state = 'A'
                AND conta.integra_fluxo = True
                {periodo_tesouraria}
                AND conta.company_id = {company_id}
            ORDER BY
                data
        """
        if data_ini < time.strftime('%Y-%m-%d') > data_fim:
            sql_tesouraria_movimento = ''
        elif data_ini < time.strftime('%Y-%m-%d'):
            data_ini = time.strftime('%Y-%m-%d')
        periodo_tesouraria = "AND movimento.data_movimento BETWEEN '%s' AND '%s'" % (data_ini,
                                                                                     data_fim)
        sql = sql_titulo + sql_tesouraria_movimento
        sql = sql.format(periodo_tesouraria=periodo_tesouraria, company_id=company_id)
        cr.execute(sql)
        vals = cr.dictfetchall()

        # if natureza_cobranca_ids:
        #     sql_titulo = sql_titulo.replace(
        #         natureza_cobranca, 'AND titulo.natureza_cobranca_id IS NULL')
        #     cr.execute(sql_titulo)
        #     vals.extend(cr.dictfetchall())

        if not vals:
            raise osv.except_osv(_('Atencao'), _('Nao ha informacoes para exibir neste periodo.'))

        lista_com_integra_fluxo = []

        for i in vals:
            # loc_cobranca_id = i.get('local_cobranca_id') or []

            # brw_loc_cobranca = self.pool.get('financeiro.local.cobranca').browse(
            #     cr, uid, loc_cobranca_id, context=context)

            # # Se titulo, deve ser considerado somente os que integram fluxo
            # if brw_loc_cobranca and not brw_loc_cobranca.conta_id.integra_fluxo and \
            #         i.get('pessoa_id'):
            #     continue
            if i.get('natureza_cobranca_id'):
                args = [('finlon_local_cobranca_id', '=', i.get('local_cobranca_id')),
                        ('finlon_natureza_cobranca_id', '=', i.get('natureza_cobranca_id'))]
                local_natureza = obj_financeiro_local_natureza.search(
                    cr, uid, args, context=context)

                if local_natureza:
                    if isinstance(local_natureza, list):
                        local_natureza=local_natureza[0]

                    bwr_loc_nat = obj_financeiro_local_natureza.browse(cr, uid, local_natureza)
                    prazo_lib = bwr_loc_nat.finlon_prazo_liberacao

                    if not prazo_lib or prazo_lib == 0:
                        if data_ini_o <= i.get('data') and i.get('data') <= data_fim_o:
                            lista_com_integra_fluxo.append(i)
                    else:
                        if data_ini_o <= (parse(i.get('data'), yearfirst=True) + relativedelta(days=prazo_lib)).strftime('%Y-%m-%d') and \
                                         (parse(i.get('data'), yearfirst=True) + relativedelta(days=prazo_lib)).strftime('%Y-%m-%d') <= data_fim_o:
                            lista_com_integra_fluxo.append(i)
            else:
                if data_ini_o <= i.get('data') and i.get('data') <= data_fim_o:
                    lista_com_integra_fluxo.append(i)

        vals = lista_com_integra_fluxo
        fluxo_caixa_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'fluxocaixa', context)
        list_doc_tipo_ids = []

        if fluxo_caixa_instalado:
            obj_fluxocaixa_excluir = self.pool.get('fluxocaixa.excluir')
            fluxocaixa_excluir_ids = obj_fluxocaixa_excluir.search(
                cr, uid, [('company_id', '=', company_id)])

            if fluxocaixa_excluir_ids:
                browse_fluxocaixa_excluir = obj_fluxocaixa_excluir.browse(
                    cr, uid, fluxocaixa_excluir_ids)

                for x in browse_fluxocaixa_excluir:
                    list_doc_tipo_ids.append(x.doc_tipo_id.id)

        obj_documento_tipo =  pooler.get_pool(cr.dbname).get('documento.documento.tipo')
        tipo_previsao_id = obj_documento_tipo.search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)

        if tipo_previsao_id:
            tipo_previsao_id = tipo_previsao_id[0]

        obj_corpos = self.pool.get('wiz.fluxocaixa.posicao.financeira.corpos')
        corpos = obj_corpos.search(cr, uid, [], context=context)

        if corpos:
            obj_corpos.unlink(cr, uid, corpos)

        # Filtra os titulos que possuem natureza de cobranca
        financeiro_titulos_1 = filter(lambda x: x.get('natureza_cobranca_id'), vals)

        # Filtra os titulos que nao possuem natureza de cobranca
        financeiro_titulos_2 = filter(lambda x: not x.get('natureza_cobranca_id'),
                                      vals)

        # Filtra os titulos que possuem natureza de cobranca e que nao estao com recebimento
        # antecipado selecionado
        financeiro_titulos_1 = filter(
            lambda x: x.get('natureza_cobranca_id') in natureza_cobranca_ids,
            financeiro_titulos_1)

        financeiro_titulos_2.extend(financeiro_titulos_1)
        vals = financeiro_titulos_2
        dados = list()

        vals = sorted(vals, key=lambda v: v['data'])
        vals = self.wizposf_incrementa_prazo_liberacao(cr, uid, vals, context)
        for x in vals:
            data_vencimento = x.get('data')
            if data_ini_o <= data_vencimento <= data_fim_o:
                tipo_doc_id = x.get('doc_tipo_id')
                is_previsao = True if tipo_previsao_id == tipo_doc_id else False

                if (vals_read['considerar_previsao'] and is_previsao) or not is_previsao:
                    if tipo_doc_id not in list_doc_tipo_ids:
                        tipo = x.get('tipo')
                        dado = {
                            'capa_id': ids[0],
                            'pessoa_id': x.get('pessoa_id'),
                            'data': data_vencimento,
                            'documento': x['historico'],
                            'vr_debito': x.get('valor') if tipo in 'DP' else 0.0,
                            'vr_credito': x.get('valor') if tipo not in 'DP' else 0.0,
                            'tipo_previsao':is_previsao,
                        }

                        dados.append((0, 0, dado))

        saldo = vals_read['saldo']
        dados = sorted(dados, key=lambda k: k[2]['data'])

        # Registra o saldo e organiza a sequencia dos registros
        for val in dados:
            saldo += val[2].get('vr_credito') - val[2].get('vr_debito')
            dados[dados.index(val)][2]['saldo'] = saldo

        wiz_titulo_vencido_ids = obj_wiz_fluxocaixa_titulo_vencido.search(
            cr, uid, [('fluxocaixa_posicao_financeira_id', 'in', ids)], context=context)
        obj_wiz_fluxocaixa_titulo_vencido.unlink(cr, uid, wiz_titulo_vencido_ids, context)

        titulo_dados = self.get_titulo_receber_vencido(cr, uid, context)

        self.write(cr, uid, ids, {'state': '1', 'corpo_ids': dados,
                                  'titulo_vencido_ids': titulo_dados}, context)

        return True

    def button_titulo_receber_atraso(self, cr, uid, ids, context=None):
        """
        O metodo executa acao para abertura de uma nova janela. Nessa eh possivel selecionar os
        titulos a receber que estao vencidos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com dados da acao
        :rtype: dict
        """
        context = context and context.copy() or {}
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fluxocaixa', 'view_wizard_posicao_financeira_titulo_vencido_form')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Titulos a receber em atraso',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'new'
        }

    def button_tratar_titulo_receber_atraso(self, cr, uid, ids, context=None):
        """
        O metodo trata se todos os titulos serao ou nao selecionados.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Dicionario com dados para execucao de uma acao que abrira novamente a mesma janela
        ja com os registros selecionados ou nao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_wiz_fluxocaixa_titulo_vencido = self.pool.get('wiz.fluxocaixa.titulo.vencido')
        for x in self.browse(cr, uid, ids, context):
            titulo_vencido_ids = map(lambda i: i.id, x.titulo_vencido_ids)
            selecionado = False
            if context.get('marcar_todos'):
                selecionado = True
            obj_wiz_fluxocaixa_titulo_vencido.write(cr, uid, titulo_vencido_ids,
                                                    {'selecionado': selecionado}, context)
        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fluxocaixa', 'view_wizard_posicao_financeira_titulo_vencido_form')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Titulos a receber em atraso',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'new'
        }

    def button_confirma_titulo_receber_atraso(self, cr, uid, ids, context=None):
        """
        O metodo cria um novo registro em 'wiz_fluxocaixa_posicao_financeira_corpos', com data para
        o ultimo dia do periodo, com o total dos titulos selecionados.
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Dicionario com dados para execucao de uma acao que abrira novamente o mesmo
        formulario ja com a nova linha inserida.
        :rtype: dict
        """
        context = context and context.copy() or {}
        for x in self.browse(cr, uid, ids, context):
            if not x.titulo_vencido_ids:
                continue

            sql = """DELETE FROM
                         wiz_fluxocaixa_posicao_financeira_corpos
                     WHERE
                         documento = 'TITULOS EM ATRASO' AND capa_id = %s
            """
            cr.execute(sql, [x.id])

            titulo_vencido_selecionado_ids = filter(lambda i: i.selecionado, x.titulo_vencido_ids)
            if not titulo_vencido_selecionado_ids:
                continue
            vr_credito = sum(map(lambda i: i.saldo, titulo_vencido_selecionado_ids))
            saldo = vr_credito
            if x.corpo_ids:
                saldo += x.corpo_ids[-1].saldo
            dado = {
                'data': x.data_fim,
                'documento': 'TITULOS EM ATRASO',
                'vr_debito': 0.0,
                'vr_credito': vr_credito,
                'saldo': saldo
            }
            self.write(cr, uid, x.id, {'corpo_ids': [(0, 0, dado)]}, context)

        reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fluxocaixa', 'wizard_posicao_financeira_view')
        view_id = reference and reference[1] or False
        return {
            'res_id': ids[0],
            'name': 'Analitico',
            'view_mode': 'form',
            'res_model': self._name,
            'view_id': [view_id],
            'type': 'ir.actions.act_window',
            'target': 'inline'
        }

    def wizposf_incrementa_prazo_liberacao(self, cr, uid, vals, context):
        """
        Este methodo incrementa na data os dias para liberacao ,cadastrados
        em fiscal.local.natureza, e posteriormente retorna
        o vals contendo um lista de dicionarios ordenado pela nova data.
        E importante que os valores do vals venham ja ordernados pela
        chave data, pois isso ira garantir a listagem correta em Tela.

        :param cr:  [Object] Cursor do banco de dados
        :param uid: [Long]   Id do usuario open que fez a solicitacao
        :param vals:[list[dict]]  Lista de dicionarios contendo os valores de
        financeiro.analitico.pgview
        :param context: [dict]dicionario contendo os context
        :return: Lista de dicionarios contendo os valores vindos de vals, porem com a
        data incrementada pelos dias de liberacao
        """
        context = context and context.copy() or {}
        obj_financeiro_local_natureza = self.pool.get('financeiro.local.natureza')
        for x in vals:
            data_vencimento = x.get('data')
            natureza_cobranca_id = x.get('natureza_cobranca_id')
            if isinstance(natureza_cobranca_id, (list, tuple)):
                natureza_cobranca_id = natureza_cobranca_id[0]
            local_cobranca_id = x.get('local_cobranca_id')
            if isinstance(local_cobranca_id, (list, tuple)):
                local_cobranca_id = local_cobranca_id[0]
            if natureza_cobranca_id and local_cobranca_id and data_vencimento:
                prazo_liberacao = 0
                local_natureza_ids = obj_financeiro_local_natureza.search(
                    cr, uid, [('finlon_local_cobranca_id', '=', local_cobranca_id),
                              ('finlon_natureza_cobranca_id', '=', natureza_cobranca_id)],
                    context=context)
                if local_natureza_ids:
                    prazo_liberacao = obj_financeiro_local_natureza.browse(
                        cr, uid, local_natureza_ids[0], context=context).finlon_prazo_liberacao
                data_vencimento = parse(data_vencimento, yearfirst=True)
                data_vencimento += relativedelta(days=prazo_liberacao)
                data_vencimento = data_vencimento.strftime('%Y-%m-%d')
                x.update({'data': data_vencimento})
        return sorted(vals, key=lambda v: v['data'])

    def button_pesquisar_tesouraria(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        vals_browse = self.browse(cr, uid, ids)

        # Verifica valores do periodo
        if vals_browse.data_fim < vals_browse.data_ini:
            raise osv.except_osv(_('Atencao'), _('Data final maior que data inicial'))

        # Recupera os titulos
        obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
        movimento_ids = obj_financeiro_tesouraria_movimento.search(
            cr, uid, [('data_movimento', '>=', vals_browse.data_ini),
                      ('data_movimento', '<=', vals_browse.data_fim),
                      ('conta_id', '=', vals_browse.conta_id.id), ('state', '=', 'A')],
            order='data_movimento asc', context=context)

        if not movimento_ids:
            raise osv.except_osv(_('Atencao'), _('Nao ha informacoes para exibir neste periodo.'))

        browse_movimentacoes = obj_financeiro_tesouraria_movimento.browse(cr, uid, movimento_ids)
        obj_corpos = self.pool.get('wiz.fluxocaixa.posicao.financeira.movimento')

        # Elimina os registros da ultima consulta
        obj_corpos.unlink(cr, uid, obj_corpos.search(cr, uid, [], context=context))

        data_dia_ant = datas.calcVencimento(vals_browse.data_ini, -1)
        saldo = self.pool.get('financeiro.tesouraria.saldos').get_saldo_do_dia(
            cr, uid, vals_browse.conta_id.id, data_dia_ant)

        for x in browse_movimentacoes:
            if x.debito_credito == 'D':
                saldo -= x.valor
                x.valor = (x.valor) * (-1)
            elif x.debito_credito == 'C':
                saldo += x.valor

            vals = {'capa_id': ids,
                    'historico': x.historico,
                    'data': x.data_movimento,
                    'valor': x.valor,
                    'saldo': saldo,
                    'debito_credito': x.debito_credito
            }
            obj_corpos.create(cr, uid, vals, context)

        self.write(cr, uid, ids, {'state': '1'}, context)

        return True

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], [], context=context)

        if not vals['corpo_ids']:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao existe informacoes para ser impressas.'))

        datas = {
            'company_id': vals['company_id'][0],
            'titulo_movimento': 'T'
        }

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'posicao_financeira',
            'datas': datas,
        }

    def button_print_movimento(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = self.read(cr, uid, ids[0], [], context=context)
        datas = {
            'company_id': vals['company_id'][0],
            'titulo_movimento': 'M'
        }

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'posicao_financeira',
            'datas': datas,
        }


# noinspection PyAbstractClass,PyPep8Naming
class wiz_fluxocaixa_posicao_financeira_corpos(osv.TransientModel):

    def _get_dados(self, cr, uid, ids, name, arg, context=None):
        """
        Campos do tipo funcao nao sao ordenados
        """
        result = {}.fromkeys(ids, False)
        context = context and context.copy() or {}

        for item in self.browse(cr, uid, ids):
            vals = {
                'f_pessoa_id': item.pessoa_id.id,
                'f_data': item.data,
                'f_documento': item.documento,
                'f_vr_debito': item.vr_debito,
                'f_vr_credito': item.vr_credito,
                'f_saldo': item.saldo,
                }
            result[item.id] = vals

        return result

    _name = 'wiz.fluxocaixa.posicao.financeira.corpos'
    _rec_name = 'capa_id'
    _description = 'Fluxo de Caixa Analitico - Detalhes'
    _columns = {
        'id': fields.integer('Id', readonly=True),
        'tipo_titulo': fields.char('Tipo', size=1),
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente/Fornecedor'),
        'f_pessoa_id': fields.function(_get_dados, method=True, string='Cliente/Fornecedor',
                                       type='many2one', relation='adm.pessoa',  multi=True),
        'data': fields.date('Data Vencimento'),
        'f_data': fields.function(_get_dados, method=True, string='Data de vencimento', type='date',
                                  multi=True),
        'documento': fields.text('Documento'),
        'f_documento': fields.function(_get_dados, method=True, string='Documento', type='text',
                                       multi=True),
        'vr_debito': fields.float('Valor', digits=(15, 2)),
        'f_vr_debito': fields.function(_get_dados, method=True, string='Debito', type='float',
                                       digits=(15, 2), multi=True),
        'vr_credito': fields.float('Valor', digits=(15, 2)),
        'f_vr_credito': fields.function(_get_dados, method=True, string='Credito', type='float',
                                        digits=(15, 2), multi=True),

        'saldo': fields.float('saldo', digits=(15, 2)),
        'f_saldo': fields.function(_get_dados, method=True, string='Saldo', type='float',
                                   digits=(15, 2), multi=True),
        'capa_id': fields.many2one('wiz.fluxocaixa.posicao.financeira.capa',
                                   'Fluxo de Caixa Analitico - Detalhes'),
        'doc_tipo_id': fields.many2one("documento.documento.tipo", 'Tipo de documento'),
        'tipo_previsao':fields.boolean('Tipo Previsao'),
        'sequencia': fields.integer('Sequencia de impressao'),
    }
    _order = 'data, id'

class wiz_fluxocaixa_posicao_financeira_movimento(osv.TransientModel):
    _name = 'wiz.fluxocaixa.posicao.financeira.movimento'
    _rec_name = 'capa_id'
    _description = 'Extrato Financeiro - Detalhes Movimentacoes'
    _columns = {
        'data': fields.date('Data Movimentacao'),
        'debito_credito': fields.selection(
            [('D', 'Debito'), ('C', 'Credito')], 'Debito/Credito', required=True, size=1),
        'historico': fields.text('Historico'),
        'saldo': fields.float('saldo', digits=(15, 2)),
        'valor': fields.float('valor', digits=(15, 2)),
        'capa_id': fields.many2one('wiz.fluxocaixa.posicao.financeira.capa',
                                   'Extrato Financeiro - Detalhes'),

    }
    _order = 'data'


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        def center_title(E, C, D, T=160):
            title_c = (T - len(C)) / 2
            title = E

            for i in range(title_c - len(E)):
                title += ' '

            title += C

            for i in range(title_c - len(D)):
                title += ' '

            title += D

            return title

        if datas['titulo_movimento'] == 'T':
            pdf_string = StringIO.StringIO()
            listagem = Impressora()
            listagem.defineFormato(orientacao='1')
            listagem.informaNome(pdf_string, _('Posicao Financeira: Analitico'))
            listagem.informaTamLinha(115)
            obj_posicao_financeira_capa = pooler.get_pool(cr.dbname).get(
                'wiz.fluxocaixa.posicao.financeira.capa')
            obj_posicao_financeira_corpos = pooler.get_pool(cr.dbname).get(
                'wiz.fluxocaixa.posicao.financeira.corpos')
            dados_posicao_financeira_capa = obj_posicao_financeira_capa.read(cr, uid, ids[0], context=context)
            company_id = dados_posicao_financeira_capa.get('company_id')

            if isinstance(company_id, tuple):
                company_id = company_id[0]
            empresa = pooler.get_pool(cr.dbname).get('res.company').read(cr, uid, company_id, ['name'], context=context)
            empresa = empresa.get('name')

            dados_posicao_financeira_corpos = obj_posicao_financeira_corpos.read(
                cr, uid, dados_posicao_financeira_capa['corpo_ids'],
                ['pessoa_id', 'data', 'documento', 'vr_debito', 'vr_credito', 'doc_tipo_id',
                 'tipo_previsao', 'tipo_titulo'], context=context)

            # Ordena os itens
            dados_posicao_financeira_corpos = sorted(
                dados_posicao_financeira_corpos, key=lambda k: (k['data']))

            data_ini = dados_posicao_financeira_capa['data_ini']
            data_fim = dados_posicao_financeira_capa['data_fim']
            data_ini = time.strftime("%d/%m/%Y", time.strptime(data_ini, "%Y-%m-%d"))
            data_fim = time.strftime("%d/%m/%Y", time.strptime(data_fim, "%Y-%m-%d"))
            folha = center_title(_('Pagina: {{ Page }} '), '', '', 0)
            listagem.informaCab(
                _('Empresa: ') + empresa + _('                                                                                                                                ') + folha )
            listagem.informaCab('')
            listagem.informaCab(
                _('Posicao Financeira                                                                Analitico                                                  Periodo: ') + data_ini + _(' a ') + data_fim)
            listagem.informaCab('--')
            listagem.informaCab(
                _('PRV  Cliente/Fornecedor                      Data        Tipo     Documento/Historico                                                              Valor          Saldo                   '))
            listagem.informaCab('--')
            listagem.informaCab('')
            listagem.informaRodape('--')
            listagem.informaRodape(
            '                                                                                                              Sig Informatica Ltda')

            disponivel = dados_posicao_financeira_capa['saldo']
            string = formataString(_('     Disponivel: ' + getNumeroEditado(disponivel)),
                                   38, ' ', 'E', 'N', 'S')
            listagem.imprimeLinha(string)
            listagem.imprimeLinha('')

            for i in dados_posicao_financeira_corpos:
                valor = i.get('vr_debito') if i.get('vr_debito') != 0 else i.get('vr_credito')
                if i.get('vr_credito') > 0:
                    tipo = 'Credito'
                    disponivel += valor
                else:
                    tipo = 'Debito'
                    disponivel -= valor

                linha = formataString(' * ' if i['tipo_previsao'] else '  ', 4, ' ', 'D', 'N', 'S')
                linha += formataString(i['pessoa_id'][1] if i['pessoa_id'] else ' ', 40, ' ', 'D', 'N', 'S')
                linha += ' '
                linha += formataString(i['data'][8:10] + i['data'][4:8] + i['data'][0:4], 11, ' ', 'D', 'N', 'S')
                linha += ' '
                linha += formataString(tipo, 8, ' ', 'D', 'N', 'S')
                linha += ' '
                linha += formataString(i['documento'], 71, ' ', 'D', 'N', 'S')
                linha += ' '
                linha += formataString(getNumeroEditado(valor), 15, ' ', 'E', 'N', 'S')
                linha += ' '
                linha += formataString(getNumeroEditado(disponivel), 18, ' ', 'E', 'N', 'S')
                linha += ' '
                listagem.imprimeLinha(linha)
                listagem.imprimeLinha('')

            listagem.finalizaImpressao()
            self.obj = external_pdf(pdf_string.getvalue())
            self.obj.render()
            pdf_string.close()

            return self.obj.pdf, 'pdf'

        # Relatorio de movimentacoes
        if datas['titulo_movimento'] == 'M':
            title = center_title('', _('EXTRATO FINANCEIRO'), '', 150)
            pdf_string = StringIO.StringIO()
            listagem = Impressora()
            listagem.defineFormato(orientacao='0')
            listagem.informaNome(pdf_string, _('Extrato Financeiro'))
            listagem.informaTamLinha(160)
            obj_posicao_financeira_capa = pooler.get_pool(cr.dbname).get(
                'wiz.fluxocaixa.posicao.financeira.capa')
            obj_posicao_financeira_movimentos = pooler.get_pool(cr.dbname).get(
                'wiz.fluxocaixa.posicao.financeira.movimento')
            dados_posicao_financeira_capa = obj_posicao_financeira_capa.read(cr, uid, ids[0], context=context)
            company_id = dados_posicao_financeira_capa.get('company_id')
            if isinstance(company_id, tuple):
                company_id = company_id[0]
            empresa = pooler.get_pool(cr.dbname).get('res.company').read(cr, uid, company_id, ['name'], context=context)
            empresa = formataString(empresa.get('name'), 58, ' ', )
            dados_posicao_financeira_movimentos = obj_posicao_financeira_movimentos.read(
                cr, uid, dados_posicao_financeira_capa['movimento_ids'],
                ['historico', 'data', 'valor', 'saldo'])
            disponivel = dados_posicao_financeira_capa['saldo']
            saldo_titulo = formataString(getNumeroEditado(dados_posicao_financeira_capa['saldo']), 14, ' ', 'E', 'N',
                                         'S')
            saldo = center_title(_('SALDO INICIAL: '), '', saldo_titulo, 0)
            conta_id = dados_posicao_financeira_capa.get('conta_id')
            if isinstance(conta_id, tuple):
                conta_id = conta_id[0]
            conta = pooler.get_pool(cr.dbname).get('financeiro.tesouraria.contas').read(cr, uid, conta_id, ['apelido'], context=context)[
                'apelido']
            conta = formataString(conta, 148, '', 'D', 'N', 'S')
            data_ini = dados_posicao_financeira_capa['data_ini']
            data_ini = data_ini[8:10] + data_ini[4:8] + data_ini[0:4]
            data_fim = dados_posicao_financeira_capa['data_fim']
            data_fim = data_fim[8:10] + data_fim[4:8] + data_fim[0:4]
            periodo = center_title(_('PERIODO INICIAL: ') + data_ini, _('PERIODO FINAL: ') + data_fim, '', 154)
            folha = center_title(_('FOLHA: {{ Page }} '), '', '', 0)
            listagem.informaCab(
                _('------------------------------------------------------------------------------------------------------------------------------------------------------------|'))
            listagem.informaCab('|' + title + '     |')
            listagem.informaCab(
                _('|                                                                                                                                                           |'))
            listagem.informaCab(
                _('|EMPRESA: ' + empresa + '                                                                               ') + folha + _('|'))
            listagem.informaCab(_('|CONTA: ') + conta + _('|'))
            listagem.informaCab(_('|') + periodo + _('  |'))
            listagem.informaCab(
                _('|-----------------------------------------------------------------------------------------------------------------------------------------------------------|'))
            listagem.informaCab(
                _('|                                                     Historico                                                 | DATA MOVIMENTACAO |   VALOR   |   SALDO   |'))
            listagem.informaCab(
                _('|-----------------------------------------------------------------------------------------------------------------------------------------------------------|'))
            listagem.informaCab(
                _('|                                                                                                                              ') + saldo + _('|'))
            listagem.informaCab(
                _('|                                                                                                                                                           |'))
            listagem.rodapePadrao()

            for i in dados_posicao_financeira_movimentos:
                i['historico'] = i.get('historico').replace('\n', ' ')
                len_historico = len(i.get('historico') or '')
                linha = formataString(i['historico'][0:111], 111, ' ', 'D', 'N', 'S') + '|'
                linha += ' '

                if len_historico >= 111 and len_historico < 222:
                    linha += '                  |           |           |'
                    listagem.imprimeLinha('|' + linha)
                    linha = formataString(i['historico'][111:221], 111, ' ', 'D', 'N', 'S') + '|'
                    linha += ' '

                    if len_historico >= 124:
                        linha += '                  |           |           |'
                        listagem.imprimeLinha('|' + linha)
                        linha = formataString(i['historico'][221:331], 111, ' ', 'D', 'N', 'S') + '|'
                        linha += ' '

                linha += formataString(i['data'][8:10] + i['data'][4:8] + i['data'][0:4], 18, ' ', 'C', 'N', 'S') + '|'
                linha += ' '
                linha += formataString(getNumeroEditado(i['valor']), 10, ' ', 'E', 'N', 'S') + '|'
                linha += ' '
                disponivel += i['valor']
                linha += formataString(getNumeroEditado(disponivel), 10, ' ', 'E', 'N', 'S') + '|'
                linha += ' '
                listagem.imprimeLinha('|' + linha)

            listagem.imprimeLinha(
                _('|------------------------------------------------------------------------------------------------------------------------------------------------------------'))
            listagem.imprimeLinha(' ')
            listagem.finalizaImpressao()
            self.obj = external_pdf(pdf_string.getvalue())
            self.obj.render()
            pdf_string.close()

            return self.obj.pdf, 'pdf'

report_custom('report.posicao_financeira')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
