# -*- encoding: utf-8 -*-

import openerp.addons.web.http as http
from logging import getLogger
import json
from ..web_sig.control.utils import websig_MySock
_debug = True

_logger = getLogger("gesbas")

class SeuControler(http.Controller):
    _cp_path = '/gesbas'

    @http.httprequest
    def gesbas_set_acessos(self, req, s_action=None, **kw):
        """
        Inserie acessos ao sigerp
        recebe em req.params(dicionario):
        base: nome do banco de dados
        data: data dos acessos formato YYYY-MM-DD
        qtd: quantidade de acessos
        retorna:  {status:'', mensagem:''}
           status 0 se tiver erro e a
           status 1 se deu certo
           qdo hover erro retorna tambem a mensagem de erro
        layout para dados
         - sem versao: data_hora;db;qtd acessos
         - versao 1: versao;data;hora;db;qtdacessos
        """
        try:
            _base = req.params.get('base') or ''
            _host = req.params.get('host') or ''
            _port = req.params.get('port') or ''
            _user = req.params.get('user') or ''
            _senha = req.params.get('senha') or ''
            _dados = req.params.get('dados') or ''
            if not _dados:
                return json.dumps({'status': '0', 'mensagem': 'Nao informado dados a incluir'})
            if _dados[0] == '1':  # Versao 1
                try:
                    _dados = _dados.replace('%3B', ';')
                    _versao, _data, _hora, _base_acessos, _qtd = _dados.split(';')
                    _data_hora = _data + ' ' + _hora
                    _qtd = int(_qtd)
                except:
                    return json.dumps({'status': '0', 'mensagem': 'Dados informados invalidos: %s' % _dados})
            else:  # Sem versao
                try:
                    _data_hora, _base_acessos, _qtd = _dados.split(';')
                    _data_hora = _data_hora.replace('_', ' ') + ':00'
                    _data_hora = _data_hora[:13]+':' + _data_hora[14:]
                    _qtd = int(_qtd)
                except:
                    return json.dumps({'status': '0', 'mensagem': 'Dados informados invalidos: %s' % _dados})
            sock = websig_MySock(host=_host, port=_port, base=_base, user=_user, senha=_senha)
            _args = [('gesbas_clie_base', '=', _base_acessos)]
            _base_ids = sock.execute('gesbas.cliente', 'search', _args)
            if not _base_ids:
                return json.dumps({'status': '0', 'mensagem': 'Base nao cadastrada'})
            _base_data = sock.execute('gesbas.cliente', 'read', _base_ids, ['gesbas_clie_pessoa_id'])
            _pessoa_id = _base_data[0].get('gesbas_clie_pessoa_id', [])[0]
            _args = [('gesbas_aces_data_hora', '=', _data_hora), ('gesbas_aces_pessoa_id', '=', _pessoa_id)]
            _gesbas_ids = sock.execute('gesbas.acessos', 'search', _args)
            if _gesbas_ids:
                sock.execute('gesbas.acessos', 'unlink', _gesbas_ids)
            vals = {'gesbas_aces_data_hora': _data_hora,
                    'gesbas_aces_pessoa_id': _pessoa_id,
                    'gesbas_aces_base': _base_acessos,
                    'gesbas_aces_quantidade': _qtd}
            res = sock.execute('gesbas.acessos', 'create', vals)
            if not res:
                return json.dumps({'status': '0', 'mensagem': 'Nao consegui incluir a informacao'})
            return json.dumps({'status': '1', 'mensagem': ''})
        except Exception as e:
            try:
                if e.faultCode:
                    return json.dumps({'status': '0', 'mensagem': 'Erro: %s' % e.faultCode})
            except:
                pass
            try:
                if e.message:
                    return json.dumps({'status': '0', 'mensagem': 'Erro: %s' % e.message})
            except:
                pass
            return json.dumps({'status': '0', 'mensagem': 'Erro: %s' % e})

    @http.httprequest
    def gesbas_set_estatisticas(self, req, s_action=None, **kw):
        """
        Inserie estatisticas de uso de servidores sigerp
        recebe em req.params(dicionario):
        base: nome do banco de dados
        data: data dos acessos formato YYYY-MM-DD
        qtd: quantidade de acessos
        retorna:  {status:'', mensagem:''}
           status 0 se tiver erro e a
           status 1 se deu certo
           qdo hover erro retorna tambem a mensagem de erro
        """
        if _debug:
            _logger.info('====== gesbas_set_estatisticas %s' % req.params.get('dados'))
        try:

            _base = req.params.get('base') or ''
            _host = req.params.get('host') or ''
            _port = req.params.get('port') or ''
            _user = req.params.get('user') or ''
            _senha = req.params.get('senha') or ''
            _dados = req.params.get('dados') or ''
            if not _dados:
                if _debug:
                    _logger.info('======== erro: Nao informado dados a incluir ')
                return json.dumps({'status': '0', 'mensagem': 'Nao informado dados a incluir'})
            try:
                _dados = _dados.split(';')
                if _dados[0] == '2':
                    _servidor = _dados[1]
                    _data = _dados[2]
                    _cpu_cores = _dados[3]
                    _cpu_uso_1 = float(_dados[4])
                    _cpu_uso_5 = float(_dados[5])
                    _cpu_uso_15 = float(_dados[6])
                    _cpu_avg_1 = float(_dados[7])
                    _cpu_avg_5 = float(_dados[8])
                    _cpu_avg_15 = float(_dados[9])
                    _dsk_total = _dados[10]
                    _dsk_perc = float(_dados[11])
                    _dsk_read_count = _dados[12]
                    _dsk_write_count = _dados[13]
                    _dsk_read_bytes = _dados[14]
                    _dsk_write_bytes = _dados[15]
                else:
                    if _debug:
                        _logger.info('======== erro: Layout desconhecido ')
                    return json.dumps({'status': '0', 'mensagem': 'Layout desconhecido %s' % _dados[0]})
            except:
                if _debug:
                    _logger.info('======== erro: Dados informados invalidos ')
                return json.dumps({'status': '0', 'mensagem': 'Dados informados invalidos: %s' % _dados})
            sock = websig_MySock(host=_host, port=_port, base=_base, user=_user, senha=_senha)
            _args = [('name', '=', _servidor), ('gesbas_serv_data_hora', '=', _data)]
            _gesbas_ids = sock.execute('gesbas.servidores', 'search', _args)
            if _gesbas_ids:
                if _debug:
                    _logger.info('======== info: Ja cadastrado, vou excluir para incluir novamente: %s' % _args)
                sock.execute('gesbas.servidores', 'unlink', _gesbas_ids)
            vals = {
                'name': _servidor,
                'gesbas_serv_data_hora': _data,
                'gesbas_serv_cpu_cores': _cpu_cores,
                'gesbas_serv_cpu_1_min': _cpu_uso_1,
                'gesbas_serv_cpu_5_min': _cpu_uso_5,
                'gesbas_serv_cpu_15_min': _cpu_uso_15,
                'gesbas_serv_avg_1_min': _cpu_avg_1,
                'gesbas_serv_avg_5_min': _cpu_avg_5,
                'gesbas_serv_avg_15_min': _cpu_avg_15,
                'gesbas_serv_dsk_uso': _dsk_total,
                'gesbas_serv_dsk_per': _dsk_perc,
                'gesbas_serv_dsk_read_count': _dsk_read_count,
                'gesbas_serv_dsk_read_bytes': _dsk_read_bytes,
                'gesbas_serv_dsk_write_count': _dsk_write_count,
                'gesbas_serv_dsk_write_bytes': _dsk_write_bytes,
            }
            res = sock.execute('gesbas.servidores', 'create', vals)
            if not res:
                if _debug:
                    _logger.info('======== erro: Nao consegui incluir a informacao ')
                return json.dumps({'status': '0', 'mensagem': 'Nao consegui incluir a informacao'})
            if _debug:
                _logger.info('======== Ok,registrado')
            return json.dumps({'status': '1', 'mensagem': ''})
        except Exception as e:
            if _debug:
                _logger.info('======== erro: Exception ')
            try:
                if e.faultCode:
                    return json.dumps({'status': '0', 'mensagem': 'Erro: %s' % e.faultCode})
            except:
                pass
            try:
                if e.message:
                    return json.dumps({'status': '0', 'mensagem': 'Erro: %s' % e.message})
            except:
                pass
            return json.dumps({'status': '0', 'mensagem': 'Erro: %s' % e})
