# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _

READ_LOAD = '_classic_write'


class kit_produto(osv.Model):
    _name = 'kit.produto'
    _inherit = 'kit.produto'

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.browse(cr, uid, ids, context=context)
        res = []
        for record in reads:
            _nome = record.kitpro_produto_id.name
            res.append((record['id'], _nome))
        return res

    def verifica_codigo_barra_embalagem(self, cr, uid, ids, vals, context=None):
        "Não deixa criar ou salvar uma embalagem que não possui codigo de barras"
        context = context and context.copy() or {}
        if not vals.get('kitpro_produto_embalagem_id'):
            return True

        campos = ['kitpro_item_produto_id', 'kitpro_grade_id']

        dados = [vals]
        if ids:
            dados = self.read(cr, uid, ids, campos, context=context)

        for i in dados:
            produto_id = i.get('kitpro_item_produto_id')
            if isinstance(produto_id, (list, tuple)):
                produto_name = produto_id[1]
                produto_id = produto_id[0]
            else:
                dados = self.pool.get('estoque.produto').read(cr, uid, produto_id, ['name',
                                                                       'codigo'], context=context)
                produto_name = '%s - %s' % (dados.get('codigo'), dados.get('name'))

            grade_id = i.get('kitpro_grade_id')
            if not isinstance(grade_id, int):
                grade_name = grade_id[1]
                grade_id = grade_id[0]
            else:
                grade_name = self.pool.get('estoque.produto.grade').read(cr, uid, grade_id,
                                                              ['name'], context=context).get('name')

            prod_emb_id = vals.get('kitpro_produto_embalagem_id')
            if not isinstance(prod_emb_id, int):
                prod_emb_id = prod_emb_id[0]

            args = [('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id),
                    ('produto_embalagem_id', '=', prod_emb_id)]
            if not self.pool.get('automacao.codigo.barra').search(cr, uid, args, context=context):
                nome_emb = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                         vals.get('kitpro_produto_embalagem_id'), context=context).embalagem_id.name
                raise osv.except_osv(_('Atencao!'),
                                     _('A embalagem nao esta cadastrada em um codigo de barras:\nProduto: %s\n'
                                       'Grade: %s\nEmbalagem: %s') % (produto_name,
                                                                      grade_name,
                                                                      nome_emb)
                                     )
        return True

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        # O KIT NAO PODE SER USADO COMO ITEM DELE PROPRIO SENAO GERA RECURSIVIDADE
        kit_id = vals.get('kitpro_produto_id')
        kit_item_id = vals.get('kitpro_item_produto_id')

        if kit_id and kit_item_id:
            if kit_id == kit_item_id:
                raise osv.except_osv('Atencao', 'O KIT nao pode ser utilizado como item dentro do proprio kit!')

        self.verifica_codigo_barra_embalagem(cr, uid, [], vals, context=context)
        res = super(kit_produto, self).create(cr, uid, vals, context=context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        # O KIT NAO PODE SER USADO COMO ITEM DELE PROPRIO SENAO GERA RECURSIVIDADE
        vls_kit_id = vals.get('kitpro_produto_id')
        vls_kit_item_id = vals.get('kitpro_item_produto_id')

        if vls_kit_id and vls_kit_item_id:
            if vls_kit_id == vls_kit_item_id:
                raise osv.except_osv('Atencao', 'O KIT nao pode ser utilizado como item dentro do proprio kit!')
        else:
            kit_produto_dados = self.read(
                cr, uid, ids, ['kitpro_produto_id', 'kitpro_item_produto_id'], context, READ_LOAD
            )
            for i in kit_produto_dados:
                kit_id = vls_kit_id or i.get('kitpro_produto_id')
                kit_item_id = vls_kit_item_id or i.get('kitpro_item_produto_id')
                if kit_id == kit_item_id:
                    raise osv.except_osv('Atencao', 'O KIT nao pode ser utilizado como item dentro do proprio kit!')

        self.verifica_codigo_barra_embalagem(cr, uid, ids, vals, context=context)
        res = super(kit_produto, self).write(cr, uid, ids, vals, context)
        return res

    def onchange_kitpro_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'kitpro_grade_id': ''}}
        if not produto_id:
            return res

        produto = self.pool.get('estoque.produto').read(cr, uid, produto_id, ['codigo', 'name',
                                             'kit_de_produtos', 'preco_venda_ids'], context=context)

        if produto_id in ids or produto.get('kit_de_produtos'):
            res['value']['kitpro_item_produto_id'] = ''
            res['warning'] = {'title': _('Atencao!'),
                              'message': _("O item do kit nao pode ser kit")}

        if not produto.get('preco_venda_ids'):
            res['value']['kitpro_item_produto_id'] = ''
            res['warning'] = {'title': _('Atencao!'),
                              'message': _("O item '%s-%s' nao possui preco") %
                                         (produto.get('codigo'),produto.get('name'))
                              }
            return res

        grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid,
                                             [('produto_id', '=', produto_id)], context=context)
        if len(grade_ids) == 1:
            res['value']['kitpro_grade_id'] = grade_ids[0]

        return res

    def onchange_kitpro_grade(self, cr, uid, ids, produto_id, grade_id, preco_kit, context=None):
        context = context and context.copy() or {}
        vals = {'kitpro_produto_embalagem_id': False}
        if not grade_id or not produto_id:
            return {'value': vals}

        args = [('produto_id', '=', produto_id)]
        prod_emb_ids = self.pool.get('estoque.produto.embalagem').search(cr, uid, args,
                                                                         context=context)
        if len(prod_emb_ids) == 1:
            vals['kitpro_produto_embalagem_id'] = prod_emb_ids[0]

        return {'value': vals}

    def onchange_kitpro_quantidade(self, cr, uid, ids, quantidade, context=None):
        context = context and context.copy() or {}
        res = {'value': {}}
        if quantidade == 0:
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Quantidade nao pode ser 0(Zero)')}
            res['value']['kitpro_quantidade'] = 1

        return res

    def kitpro_get_preco_venda(self, cr, uid, produto_kit_id, context=None):
        context = context and context.copy() or {}
        produto_kit_id = isinstance(produto_kit_id, list) and  produto_kit_id[0] or produto_kit_id
        kit_produto_ids = self.search(cr, uid, [('kitpro_produto_id', '=', produto_kit_id)],
                                      context=context)
        preco_venda_kit = 0.0
        if not kit_produto_ids:
            return preco_venda_kit

        dadpos_kit = self.read(cr, uid, kit_produto_ids, ['kitpro_item_produto_id',
                     'kitpro_grade_id', 'kitpro_quantidade'], context=context,load='_classic_write')
        produto_ids = []
        grade_ids = []
        for i in dadpos_kit:
            produto_ids.append(i.get('kitpro_item_produto_id'))
            grade_ids.append(i.get('kitpro_grade_id'))

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        precos_venda = self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid, produto_ids,
                                                                          epc_company_id=company_id,
                                                                          epc_grade_ids=grade_ids,
                                                                          context=context)
        for item_do_kit in dadpos_kit:
            produto_id = item_do_kit.get('kitpro_item_produto_id')
            grade_id = item_do_kit.get('kitpro_grade_id')
            quantidade = item_do_kit.get('kitpro_quantidade')
            dados_preco_venda = precos_venda.get(produto_id) or {}
            preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get('preco_venda')
            preco_venda *= quantidade
            preco_venda_kit += preco_venda

        return preco_venda_kit

    def kitpro_button_show_kit(self, cr, uid, ids, context=None):
        """
        Metodo do botao 'Formulario Kit'. Exibe para edicao o formulario do produto kit.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: action para chamada do formulario do produto kit.
        """
        context = context and context.copy() or {}
        kitpro_produto_id = self.read(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                      ['kitpro_produto_id'], context, '_classic_write').get(
            'kitpro_produto_id')
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'estoque', 'view_estoque_produto_form')
        view_id = reference and reference[1] or False
        return {
            'name': 'Kit',
            'type': 'ir.actions.act_window',
            'view_id': [view_id],
            'view_mode': 'form',
            'res_id': kitpro_produto_id,
            'res_model': 'estoque.produto',
            'target': 'new'
        }

