# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.addons.kit.model.funcoes import get_preco_total_venda_itens_kit
from decimal import *
TWOPLACES = Decimal(10) ** -2

class estoque_produto(osv.Model):

    _inherit = 'estoque.produto'

    def check_inativar_produto(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'active' in vals and not vals.get('active'):
            kit_produto_ids = self.pool.get('kit.produto').search(cr, uid,
                                           [('kitpro_item_produto_id', 'in', ids)], context=context)
            kit_ids = map(lambda x: x.get('kitpro_produto_id'),
                          self.pool.get('kit.produto').read(cr, uid, kit_produto_ids, ['kitpro_produto_id'],
                                    context=context, load='_classic_write'))
            for i in self.read(cr, uid, kit_ids, ['active', 'name', 'codigo'], context=context):
                if i.get('active'):
                    name = '%s-%s' % (i.get('codigo'), i.get('name'))
                    raise osv.except_osv(_('Atencao!'),
                                         _('Nao e possivel initivar este produto, ele esta '
                                           'cadastrado no kit: %s') % name)
        return True

    def verificar_valor_total_itens_kit(self, cr, uid, ids, vals={}, context=None):
        context = context and context.copy() or {}
        if not isinstance(ids, list):
            ids = [ids]

        if vals and not vals.get('kit_de_produtos'):
            return

        for i in self.read(cr, uid, ids, ['kit_de_produtos', 'produto_kit_ids'], context=context):
            if not i.get('kit_de_produtos'):
                continue

            itens = filter(lambda x: x.get('kitpro_preco_venda') <= 0,
                           self.pool.get('kit.produto').read(cr, uid, i.get('produto_kit_ids'),
                                ['kitpro_preco_venda', 'kitpro_item_produto_id'], context=context))
            if itens:
                raise osv.except_osv(_('Atencao!'),
                                    _('Preco total de venda deve ser maior que Zero dos itens:\n%s')
                                     % '\n'.join(map(lambda x: x.get('kitpro_item_produto_id')[1],
                                                     itens)))

        return True

    def create(self, cr, user, vals, context=None):
        context = context and context.copy() or {}
        res = super(estoque_produto, self).create(cr, user, vals, context)
        self.verificar_valor_total_itens_kit(cr, user, res,  context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        self.check_inativar_produto(cr, uid, ids, vals, context)
        res = super(estoque_produto, self).write(cr, uid, ids, vals, context=context)
        self.verificar_valor_total_itens_kit(cr, uid, ids, context)
        return res

    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('nao_pesquisa_kit'):
            args.append(('kit_de_produtos', '=', False))

        return super(estoque_produto, self).search(cr, user, args, offset, limit, order, context,
                                                   count)

    def get_kits_com_preco_total(self, cr, uid, ids, preco_venda_ids, produto_kit_ids,
                                 context=None):
        context = context and context.copy() or {}
        preco_venda = filter(lambda i: i[0] == 0, preco_venda_ids)
        if not preco_venda:
            return []

        campos = ['kitpro_produto_id', 'kitpro_quantidade', 'kitpro_grade_id',
                  'kitpro_item_produto_id', 'kitpro_produto_embalagem_id']
        dados_itens = self.resolve_2many_commands(cr, uid, 'produto_kit_ids',
                                                  filter(lambda i: i[0] in [0, 1, 4],
                                                         produto_kit_ids), campos, context=context)
        preco_venda = preco_venda[0][2].get('preco_venda')
        cont = 1
        for i in dados_itens:
            i['id_'] = cont  # indice de identificacao do item
            cont += 1
            if not i.get('kitpro_produto_id'):
                i['kitpro_produto_id'] = ids

        totais = get_preco_total_venda_itens_kit(self, cr, uid, [], context=context,
                                                 onchage=dados_itens, onchange_preco=preco_venda)
        kits = []
        for i in dados_itens:
            cod = 0
            id = False
            if i.get('id'):
                cod = 1
                id = i.pop('id')

            i['kitpro_preco_venda'] = totais.get(i.get('id_'))
            del i['id_']
            kits.append([cod, id, i])

    def onchange_preco_venda(self, cr, uid, ids, preco_venda_ids, produto_kit_ids, context=None):
        """
        Ao alterar preco_venda_ids este metodo sera chamado para alterar os valores do itens do kit
        :param cr: Cursor do banco de dados
        :param uid: Usuario que solicitou a operacao
        :param ids: Lista de ids dos registros
        :param preco_venda_ids: Lista contendo os dados do preco no formato One2Many
        :param produto_kit_ids: Lista contendo os dados dos itens do kit no formato One2Many
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: int | long
        :type preco_venda_ids: list
        :type produto_kit_ids: list
        :type context: dict
        :return: Retorna um dicionario contendo as alteracoes para produto_kit_ids
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not produto_kit_ids or not all([preco_venda_ids,
                                           filter(lambda i: i[0] == 0, preco_venda_ids)]):
            return {}

        kits = self.get_kits_com_preco_total(cr, uid, ids, preco_venda_ids, produto_kit_ids,
                                             context)
        return {'value': {'produto_kit_ids': kits}}

    def button_atualizar_kit(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        kit_produto_ids = []
        map(lambda x: kit_produto_ids.extend(x.get('produto_kit_ids')),
                      self.read(cr, uid, ids, ['produto_kit_ids'], context=context))
        context['preco_apenas_da_classe_preco_venda'] = True
        if not kit_produto_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Kit sem itens!'))

        for i in self.pool.get('kit.produto').read(cr, uid, kit_produto_ids, ['kitpro_quantidade'],
                                                      context):
            vals = {'kitpro_quantidade': i.get('kitpro_quantidade')}
            # recalcula o preco
            self.pool.get('kit.produto').write(cr, uid, i.get('id'), vals, context=context)

        return True

    def onchange_preco_kit(self, cr, uid, ids, preco_venda_ids, produto_kit_ids, context=None):
        """
        Ao alterar preco_kit este metodo sera chamado para alterar os valores do itens do kit
        :param cr: Cursor do banco de dados
        :param uid: Usuario que solicitou a operacao
        :param ids: Lista de ids dos registros
        :param preco_kit: String informando se o preco do kit e a soma dos itens ou nao
        :param produto_kit_ids: Lista contendo os dados dos itens do kit no formato One2Many
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: int | long
        :type preco_kit: str
        :type produto_kit_ids: list
        :type context: dict
        :return: Retorna um dicionario contendo as alteracoes para produto_kit_ids
        :rtype: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        kits = self.get_kits_com_preco_total(cr, uid, ids, preco_venda_ids, produto_kit_ids,
                                             context)
        if not kits:
            return {}

        return {'value': {'produto_kit_ids': kits}}

    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not default:
            default = {}
        default.update({
            'produto_kit_ids': [],
            'estpro_contem_kit_ids': [],
        })
        res = super(estoque_produto, self).copy(cr, uid, id, default, context)
        return res
