# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2016 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2016, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


# noinspection PyAbstractClass,PyPep8Naming
class estoque_movimento_itens(osv.Model):
    """
    Heranca da classe ´estoque_movimento_itens´.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _inherit = 'estoque.movimento.itens'

    def estmvi_onchange_item_movimento(self, cr, uid, ids, empresa, natureza, nf_tipo, destinatario,
                                       item, local, preco_venda, total, nf_propria, nf_devolucao,
                                       endereco=False, grade_id=False, tributacao=None,
                                       context=None):
        """
        Recupera a grade do produto (item) se existir apenas uma.

        :param cr: curor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento (produto)
        :param empresa: ID da empresa
        :param natureza: ID da natureza da operacao
        :param nf_tipo: tipo da nota fiscal: "E" para entrada/compra ou "S" para saida/venda
        :param destinatario: ID do destinatario: fornecedor se entrada ou cliente se saida
        :param item: ID do produto
        :param local: ID do local do produto
        :param preco_venda: preco de venda do produto
        :param total: valor total do produto
        :param nf_propria: identificador para notas fiscais proprias
        :param nf_devolucao: identificador para notas fiscais de devolucao
        :param endereco: ID do endereco do destinatario
        :param grade_id: ID da grade do item
        :param tributacao: lista com o ID das tributacoes (one2many)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type empresa: long
        :type natureza: long
        :type nf_tipo: str
        :type destinatario: long
        :type item: long
        :type local: long
        :type preco_venda: float
        :type total: float
        :type nf_propria: bool
        :type nf_devolucao: bool
        :type endereco: long
        :type grade_id: long
        :type tributacao: list
        :type context: dict
        :return: valor da grade para ser mostrada no formulario
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(estoque_movimento_itens, self).estmvi_onchange_item_movimento(
            cr, uid, ids, empresa, natureza, nf_tipo, destinatario, item, local, preco_venda, total,
            nf_propria, nf_devolucao, endereco, grade_id, tributacao, context)
        if not item:
            return res
        if not nf_propria:  # Entrada de terceiro
            context['super_estoque_produto_search'] = True
            if self.pool.get('estoque.produto').search(
                    cr, uid, [('id', 'in', [item]), ('kit_de_produtos', '=', True)], context=context):
                return dict(warning=dict(title=_('Atencao!'),
                                         message=('O item informado e um Kit. Nao e possivel dar '
                                                  'entrada em itens Kit.!')),
                            value=dict(produto_id=False))
        if nf_tipo == 'S':  # Saida propria
            if self.pool.get('estoque.produto').browse(
                    cr, uid, item, context=context).kit_de_produtos:
                mensagem = _('Este e um item de KIT e deve ser informado na aba "Kit por '
                             'itens"')
                warning = {'title': _('Aviso.'), 'message': mensagem}
                return {'value': dict(produto_id=False), 'warning': warning}

        return res

    def estmvi_onchange_item_movimento_propria(self, cr, user, ids, comapny_id, item_id, grade_id,
                                               vr_total_item, destinatario_id, tributacoes_o2m,
                                               tipo_nf, natureza_id=None, context=None,
                                               dest_ender_id=False, vr_despesa=0,
                                               incide_despesa=False, vr_frete=0, vr_seguro=0,
                                               vr_desconto=0):
        """
        Recupera a grade do item (produto/servico) em funcao do local. Apresenta a grade se houver 
        apenas uma. Recupera as tributacoes do item.
        Sobrescricao modulo kit.
        
        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do item de movimentacao
        :param item_id: o ID do item (produto/servico)
        :param destinatario_id: o ID do destinatario
        :param tributacoes_o2m: lista com o ID das tributacoes (one2many)
        :param natureza_id: o ID da natureza da operacao
        :param context: um dicionario padrao para valores contextuais
        :param dest_ender_id: ID do endereco do destinatario
        :type cr: object
        :type user: int
        :type ids: list
        :type item_id: long
        :type tributacoes_o2m: list
        :type natureza_id: long
        :type context: dict
        :type dest_ender_id: long
        :return: a grade do item.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(estoque_movimento_itens, self).estmvi_onchange_item_movimento_propria(
            cr, user, ids, comapny_id, item_id, grade_id, vr_total_item, destinatario_id,
            tributacoes_o2m, tipo_nf, natureza_id, context, dest_ender_id, vr_despesa,
            incide_despesa, vr_frete, vr_seguro, vr_desconto)
        if not item_id:
            return res
        if self.pool.get('estoque.produto').browse(
                cr, user, item_id, context=context).kit_de_produtos:
            mensagem = _('Este e um item de KIT e deve ser informado na aba "Kit por '
                         'itens"')
            warning = {'title': _('Aviso.'), 'message': mensagem}
            return {'value': dict(produto_id=False), 'warning': warning}
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
