#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
import time


class fiscal_sugestao_preco_item(osv.Model):
    _inherit = 'fiscal.sugestao.preco.item'

    def fisspi_get_dados_entrada(self, cr, sugestao_capa, company_id, context=None):
        """
        Metodo responsavel por executar o sql de consulta dos dados referente a entrada na sugestao
        :param cr: Cursor do banco de dados
        :param sugestao_capa: Browser de fiscal.sugestao.preco
        :param company_id: ID da empresa que esta executando o processo
        :param context: dados contextuais
        :type cr: Cursor
        :type sugestao_capa: Browse
        :type company_id: int
        :type context: dict
        :rtype: dict
        """
        res = super(fiscal_sugestao_preco_item, self).fisspi_get_dados_entrada(
            cr, sugestao_capa, company_id, context)
        if not sugestao_capa.fissup_listar_kits:
            return res
        sql = """
                SELECT DISTINCT kp.kitpro_produto_id       
                                                  AS produto_id,
                       (SELECT min(id) FROM estoque_produto_grade 
                        WHERE produto_id = kp.kitpro_produto_id)
                                                  AS grade_id,
                       (SELECT min(unidade_id) FROM estoque_embalagem WHERE id IN (
                           SELECT min(embalagem_id) FROM estoque_produto_embalagem 
                           WHERE produto_id = kp.kitpro_produto_id)
                        )                         AS fisspi_estoque_embalagem_unidade_id,
                       (SELECT min(embalagem_id) FROM estoque_produto_embalagem 
                        WHERE produto_id = kp.kitpro_produto_id)
                                                  AS embalagem_id,
                       TRUE                       AS kit_de_produtos
                FROM   kit_produto                AS kp
                INNER  JOIN estoque_produto       AS ep  ON ep.id = kp.kitpro_produto_id
                WHERE  kp.kitpro_item_produto_id IN ({produto_ids})
                       AND kp.kitpro_grade_id IN ({grade_ids})
                       AND (ep.preco_kit is null OR ep.preco_kit != 'S') 
                ORDER  BY produto_id;
        """.format(produto_ids=', '.join(list(set(map(lambda p: str(p.get('produto_id')), res)))),
                   grade_ids=', '.join(list(set(map(lambda p: str(p.get('grade_id')), res)))))
        cr.execute(sql)
        res_kit = cr.dictfetchall()
        return res_kit + res

    def fisspi_get_total_sugestao(self, cr, uid, sugestao_capa, context):
        """
        Metodo responsavel por executar o sql de consulta dos dados referente ao filtro
        sugestao ou total
        :param cr: Cursor do banco de dados
        :param uid: id da tabela res_users, identificador do usuario.
        :param sugestao_capa: Browser de fiscal.sugestao.preco
        :param context: dados contextuais
        :type cr: Cursor
        :type uid: int
        :type sugestao_capa: Browse
        :type context: dict
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(fiscal_sugestao_preco_item, self).fisspi_get_total_sugestao(
            cr, uid, sugestao_capa, context)
        if not sugestao_capa.fissup_listar_kits:
            return res
        sql = """
                SELECT DISTINCT kp.kitpro_produto_id       
                                                  AS produto_id,
                       (SELECT min(id) FROM estoque_produto_grade 
                        WHERE produto_id = kp.kitpro_produto_id)
                                                  AS grade_id,
                       (SELECT min(unidade_id) FROM estoque_embalagem WHERE id IN (
                           SELECT min(embalagem_id) FROM estoque_produto_embalagem 
                           WHERE produto_id = kp.kitpro_produto_id)
                        )                         AS fisspi_estoque_embalagem_unidade_id,
                       (SELECT min(embalagem_id) FROM estoque_produto_embalagem 
                        WHERE produto_id = kp.kitpro_produto_id)
                                                  AS embalagem_id,
                       TRUE                       AS kit_de_produtos,
                       ep.margem_minima           AS margem_minima,
                       ep.margem_ideal            AS margem_ideal,
                       ep.margem_bruta            AS margem_bruta
                FROM   kit_produto                AS kp
                INNER  JOIN estoque_produto       AS ep  ON ep.id = kp.kitpro_produto_id
                WHERE  kp.kitpro_item_produto_id IN ({produto_ids})
                       AND kp.kitpro_grade_id IN ({grade_ids})
                       AND (ep.preco_kit is null OR ep.preco_kit != 'S') 
                ORDER  BY produto_id;
        """.format(produto_ids=', '.join(list(set(map(lambda p: str(p.get('produto_id')), res)))),
                   grade_ids=', '.join(list(set(map(lambda p: str(p.get('grade_id')), res)))))
        cr.execute(sql)
        res_kit = cr.dictfetchall()
        return res_kit + res
