# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _

def get_kit_id_do_produto(self, cr, uid, ids, context=None):
    '''Retorna uma lista com os ids da classe kit.produto
       self = classe estoque.produto'''
    aux_ids = []
    for i in self.pool.get('estoque.produto').read(cr, uid, ids, ['kit_de_produtos', 'preco_kit'],
                                                   context=context):
        # retira os produtos que nao sao kits ou o preco eh a soma dos itens
        if i.get('kit_de_produtos'):
            aux_ids.append(i.get('id'))

    res = self.pool.get('kit.produto').search(cr, uid, [('kitpro_produto_id', 'in', aux_ids)],
                                              context=context)
    return res

def get_kit_id_do_preco_venda(self, cr, uid, ids,  context=None):
    '''Retorna uma lista com os ids da classe kit.produto
       self = classe estoque.preco.custo'''
    produto_ids = map(lambda x: x.get('produto_id'), self.read(cr, uid, ids, ['produto_id'],
                                                            context=context, load='_classic_write'))
    res = []
    for i in self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['preco_kit',
                                                   'kit_de_produtos'], context=context):
        # retira os produtos que nao sao kits ou cujo o preco eh a soma dos itens do kit
        if not i.get('kit_de_produtos'):
            item_kit_ids = self.pool.get('kit.produto').search(cr, uid,
                                     [('kitpro_item_produto_id','=', i.get('id'))], context=context)
            produto_ids.remove(i.get('id'))
            if item_kit_ids:
                produto_ids.append(self.pool.get('kit.produto').read(cr, uid, item_kit_ids[0],
                                             ['kitpro_produto_id'], context=context,
                                             load='_classic_write').get('kitpro_produto_id'))

    if not produto_ids:
        return []

    res.extend(self.pool.get('kit.produto').search(cr, uid, [('kitpro_produto_id', 'in', produto_ids)],
                                                   context=context))
    return res

def get_kit_id_do_item_kit(self, cr, uid, ids, context=None):
    '''Retorna uma lista com os ids da classe kit.produto
       self = classe kit.produto'''
    produto_ids = list(set(map(lambda x: x.get('kitpro_produto_id'),
                               self.read(cr, uid, ids, ['kitpro_produto_id'], context=context,
                                         load='_classic_write'))))
    if False in produto_ids:
        produto_ids.remove(False)

    if not produto_ids:
        return []

    for i in self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['preco_kit',
                                                   'kit_de_produtos'], context=context):
        # retira os produtos que nao sao kits ou cujo o preco eh a soma dos itens do kit
        if not i.get('kit_de_produtos'): # preco eh a soma dos itens
            produto_ids.remove(i.get('id'))

    if not produto_ids:
        return []

    res = []
    item_ids = self.search(cr, uid, [('kitpro_produto_id', 'in', produto_ids)], context=context)
    for i in self.read(cr, uid, item_ids, ['kitpro_produto_embalagem_id'], context=context,
                       load='_classic_write'):
        if i.get('kitpro_produto_embalagem_id'):
            res.append(i.get('id'))

    return res

def get_kit_id_do_embalagem(self, cr, uid, ids, context=None):
    '''Retorna uma lista com os ids da classe kit.produto
       self = classe estoque.embalagem'''
    prod_emb_ids = self.pool.get('estoque.produto.embalagem').search(cr, uid,
                                                       [('embalagem_id','in',ids)], context=context)
    if not prod_emb_ids:
        return []

    kit_prod_ids = self.pool.get('kit.produto').search(cr, uid,
                             [('kitpro_produto_embalagem_id', 'in', prod_emb_ids)], context=context)

    produto_ids = map(lambda x: x.get('kitpro_produto_id'),
                     self.pool.get('kit.produto').read(cr, uid, kit_prod_ids, ['kitpro_produto_id'],
                                                        context=context, load='_classic_write'))
    for i in self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['preco_kit',
                                                   'kit_de_produtos'], context=context):
        # retira os produtos que nao sao kits ou cujo o preco eh a soma dos itens do kit
        if not i.get('kit_de_produtos'): # preco eh a soma dos itens
            produto_ids.remove(i.get('id'))

    if not produto_ids:
        return []

    res = self.pool.get('kit.produto').search(cr, uid, [('kitpro_produto_id', 'in', produto_ids)],
                                              context=context)
    return res

def get_preco_total_venda_itens_kit(self, cr, uid, ids, context=None, onchage=None, onchange_preco=0):
    """Calcula o preco total para os itens do kit, soma dos itens ou preco no kit, rateado para os 
       itens caso o preco seja no kit"""
    context = context and context.copy() or {}
    res_precos = {}
    campos = ['kitpro_produto_id', 'kitpro_quantidade', 'kitpro_grade_id',
              'kitpro_item_produto_id', 'kitpro_produto_embalagem_id']

    if ids:
        dados_kit_itens = self.pool.get('kit.produto').read(cr, uid, ids, campos, context)
    else:
        dados_kit_itens = onchage

    if not dados_kit_itens:
        return {}

    produto_ids = []
    kit_produto_ids = []
    kit_grade_ids = []
    dict_kit_itens = {}
    for i in dados_kit_itens:
        produto_id = i.get('kitpro_produto_id')
        if not isinstance(produto_id, int):
            produto_id = produto_id[0]

        i['kitpro_produto_id'] = produto_id
        if produto_id:
            produto_ids.append(produto_id)
        kit_produto_id = i.get('kitpro_item_produto_id')
        if not isinstance(kit_produto_id, int):
            kit_produto_id = kit_produto_id[0]

        i['kitpro_item_produto_id'] = kit_produto_id
        kit_produto_ids.append(kit_produto_id)
        kit_grade_id = i.get('kitpro_grade_id')
        if not isinstance(kit_grade_id, int):
            kit_grade_id = kit_grade_id[0]

        i['kitpro_grade_id'] = kit_grade_id
        kit_grade_ids.append(kit_grade_id)
        kit_emb_id = i.get('kitpro_produto_embalagem_id')
        if not isinstance(kit_emb_id, int):
            kit_emb_id = kit_emb_id[0]

        i['kitpro_produto_embalagem_id'] = kit_emb_id
        if ids:
            dict_kit_itens[i.get('id')] = i
        else:
            dict_kit_itens[i.get('id_')] = i

    dict_produto = {}
    campos = ['preco_kit', 'produto_kit_ids']
    map(lambda x: dict_produto.update({x.get('id'): x}),
        self.pool.get('estoque.produto').read(cr, uid, produto_ids, campos, context=context))
    obj_preco_venda = self.pool.get('estoque.preco.venda')
    preco_kits = obj_preco_venda.get_preco_venda(cr, uid, produto_ids, context=context)
    preco_itens = obj_preco_venda.get_preco_venda(cr, uid, kit_produto_ids,
                                                  epc_grade_ids=kit_grade_ids, context=context)
    qtd_embalagens = self.pool.get('estoque.utils').get_grade_qtd_embalagem(cr, kit_grade_ids,
                                                                            context=context)
    for kit_id, dados in dict_produto.items():
        preco_kit = (preco_kits.get(kit_id) or {}).get('preco_venda') or 0
        if onchange_preco:
            preco_kit = onchange_preco

        lista_rateio = []
        produto_kit_ids = ids and dados.get('produto_kit_ids') or map(lambda x: x.get('id_'),
                                                                      dados_kit_itens)
        msg = []
        msg_preco = []
        for kit_item_id in produto_kit_ids:
            if not dict_kit_itens.get(kit_item_id):
                continue

            produto_id = dict_kit_itens.get(kit_item_id).get('kitpro_item_produto_id')
            grade_id = dict_kit_itens.get(kit_item_id).get('kitpro_grade_id')
            prd_emb_id = dict_kit_itens.get(kit_item_id).get('kitpro_produto_embalagem_id')
            chave = '%s-%s' % (grade_id, prd_emb_id)
            # Tratamento com o contexto 'kitpro_funcional_preco_venda' para evitar raise na chamada
            # desse metodo pelo atributo funcional kitpro_preco_venda
            if not qtd_embalagens.get(chave) and not context.get('kitpro_funcional_preco_venda'):
                dados = self.pool.get('estoque.produto').read(cr, uid, produto_id,
                                                              ['name', 'codigo'], context=context)
                msg.append('%s - %s' % (dados.get('codigo'), dados.get('name')))
                continue

            qtd_emb = qtd_embalagens.get(chave) and qtd_embalagens.get(chave).get('qtd') or 0
            preco = dict_kit_itens.get(kit_item_id).get('kitpro_quantidade') * \
                    ((preco_itens.get(produto_id) or {}).get(grade_id) or
                     (preco_itens.get(produto_id) or {}).get('preco_venda') or 0) * qtd_emb
            # Tratamento com o contexto 'kitpro_funcional_preco_venda' para evitar raise na chamada
            # desse metodo pelo atributo funcional kitpro_preco_venda
            if preco <= 0 and not context.get('kitpro_funcional_preco_venda'):
                dados = self.pool.get('estoque.produto').read(cr, uid, produto_id,
                                                              ['name', 'codigo'], context=context)
                msg_preco.append('%s - %s' % (dados.get('codigo'), dados.get('name')))
                continue

            if  dados.get('preco_kit') == 'K': # preco no kit
                lista_rateio.append([kit_item_id, preco])

            elif dados.get('preco_kit') == 'S': # soma dos itens
                res_precos[kit_item_id] = preco

        if msg:
            kit_name = dict(self.pool.get('estoque.produto').name_get(cr, uid, [kit_id],context)
                            ).get(kit_id)
            raise osv.except_osv(_('Atencao!'),
                                 _('Os itens do kit %s nao possuem embalagem informada no kit: \n%s') %
                                 (kit_name, '\n'.join(msg)))
        if msg_preco:
            raise osv.except_osv(_('Atencao!'),
                                 _('Os itens nao possuem preco ou quantidade:\n %s') %
                                 '\n'.join(msg_preco))

        if lista_rateio:
            self.pool.get('adm.utils').efetuar_rateio(preco_kit, lista_rateio)
            for i in lista_rateio:
                res_precos[i[0]] =  i[1]

    return res_precos

def get_preco_venda_itens_kit(self, cr, uid, ids, field, args, context=None):
    context = context and context.copy() or {}
    # Atributos funcionais cujo parametro 'store' possui regras, como o kitpro_preco_venda, eh usado
    # o usuario admin (id = 1) para os calculos. Isso eh tratado no orm, veja arquivo orm.py nas
    # linhas 4695 e 4721. Como em nossos modulos possui regras proprias para uso da empresa
    # (company_id), aqui forco novamente o usuario que esta realizando as transacoes.
    if context.get('uid') and context.get('uid') != uid:
        uid = context.get('uid')
    context.update({'preco_apenas_da_classe_preco_venda': True,
                    'kitpro_funcional_preco_venda': True})
    return get_preco_total_venda_itens_kit(self, cr, uid, ids, context=context)
