# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import time
from openerp.tools.translate import _


class wizard_kit_listagem(osv.TransientModel):

    _name = 'wizard.kit.listagem'
    _columns = {
        'wkitli_produto_id': fields.many2one('estoque.produto', 'Kit',
                                             domain="[('kit_de_produtos', '=', True)]"),
        'wkitli_tipo_preco': fields.selection([('K', 'Preco no kit'), ('S', 'Soma dos Itens')],
                                              'Tipo preco', size=1),
        'wkitli_situacao': fields.selection([('A', 'Ativo'), ('I', 'Inativo')], 'Situacao', size=1),
    }

    def wkitli_button_print(self, cr, uid, ids, context=None):
        """
        Metodo de acao do botao 'Imprimir'. Reune os dados necessarios que serao passados para a
        classe de impressao.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: dados de acao para chamada da classe de impressao.
        :rtype: dict
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'estoque.produto', context=context)
        brw_wizard_kit_listagem = self.browse(
            cr, uid, ids[0] if isinstance(ids, (list, tuple)) else ids, context)
        sql = """
            SELECT
                epg.id                  AS kit_grade_id,
                ep.codigo               AS kit_codigo,
                epg.grade               AS kit_grade,
                epg.descricao           AS kit_descricao,
                ep.name                 AS kit_name,
                epg1.id                 AS item_grade_id,
                ep1.codigo              AS item_codigo,
                epg1.grade              AS item_grade,
                epg1.descricao          AS item_descricao,
                ep1.name                AS item_name,
                ki.kitpro_quantidade    AS qtd_no_kit
            FROM
                estoque_produto ep
            LEFT JOIN
                estoque_produto_grade epg   ON ep.id = epg.produto_id
            LEFT JOIN
                kit_produto ki              ON ep.id = ki.kitpro_produto_id
            LEFT JOIN
                estoque_produto ep1         ON ep1.id = ki.kitpro_item_produto_id
            LEFT JOIN
                estoque_produto_grade epg1  ON epg1.id = ki.kitpro_grade_id
            WHERE
                ep.company_id = %s
                AND ep.kit_de_produtos IS TRUE
        """ % company_id
        if brw_wizard_kit_listagem.wkitli_produto_id:
            sql += """
                AND ep.id = %s
            """ % brw_wizard_kit_listagem.wkitli_produto_id.id
        if brw_wizard_kit_listagem.wkitli_tipo_preco:
            sql += """
                AND ep.preco_kit = '%s'
            """ % brw_wizard_kit_listagem.wkitli_tipo_preco
        if brw_wizard_kit_listagem.wkitli_situacao:
            sql += """
                AND ep.active = %s
            """ % True if brw_wizard_kit_listagem.wkitli_situacao == 'A' else False
        sql += """
            ORDER BY ep.codigo, ep1.codigo
        """
        cr.execute(sql)
        kit_dados = cr.dictfetchall()
        if not kit_dados:
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para os filtros selecionados.'))
        grade_ids = []
        for dado in kit_dados:
            if dado.get('kit_grade_id'):
                grade_ids.append(dado.get('kit_grade_id'))
            if dado.get('item_grade_id'):
                grade_ids.append(dado.get('item_grade_id'))
        grade_ids = list(set(grade_ids))
        saldos = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
            cr, uid, grade_ids, time.strftime('%Y-%m-%d'), company_id=company_id, context=context)
        dados = {'kit_dados': kit_dados, 'company_id': company_id, 'saldos': saldos}
        if brw_wizard_kit_listagem.wkitli_produto_id:
            dados.update({'produto': '%s - %s' % (brw_wizard_kit_listagem.wkitli_produto_id.codigo,
                                                  brw_wizard_kit_listagem.wkitli_produto_id.name)})
        if brw_wizard_kit_listagem.wkitli_tipo_preco:
            dados.update({
                'tipo_preco': 'Preco no kit' if brw_wizard_kit_listagem.wkitli_produto_id == 'K'
                else 'Soma dos itens'})
        if brw_wizard_kit_listagem.wkitli_situacao:
            dados.update({'situacao': 'Ativos' if brw_wizard_kit_listagem.wkitli_situacao is True
                          else 'Inativos'})
        return {
            'type': 'ir.actions.report.xml',
            'name': 'Relacao de kits',
            'report_name': 'listagem_relacao_kits',
            'datas': dados
        }

