# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from xml.etree.ElementTree import Element, SubElement, tostring

READ_LOAD = '_classic_write'


class wizard_kit_categoria(osv.TransientModel):

    def wkitcat_get_local_padrao(self, cr, uid, context=None):
        context = context and context.copy() or {}
        parametro_dados = self.pool.get('estoque.parametro').estpar_get_fields(
            cr, uid, fields_to_read='local_venda_padrao_id'
        )
        if isinstance(parametro_dados, list):
            parametro_dados = parametro_dados[0]

        return parametro_dados.get('local_venda_padrao_id') or False

    def default_get(self, cr, uid, fields, context):
        res = super(wizard_kit_categoria, self).default_get(cr, uid, fields, context)
        if fields:
            locais = filter(lambda i: 'local_' in i, fields)
            if locais:
                local_padrao = self.wkitcat_get_local_padrao(cr, uid, context={})
                res.update({}.fromkeys(locais, local_padrao))
        return res

    _name = 'wizard.kit.categoria'
    _description = 'Wizard Kit Categoria'
    _columns = {
        'local_1_id': fields.many2one('estoque.local', 'Local', required=True),
        'produto_1_id': fields.many2one('estoque.produto', 'Produto', required=True),
        'grade_1_id': fields.many2one('estoque.produto.grade', 'Grade', required=True),
    }
    _defaults = {
        'local_1_id': wkitcat_get_local_padrao
    }

    def wkitcat_onchange_local(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = {}
        field_produto = context.get('ctx_wkitcat_field_produto')
        field_grade = context.get('ctx_wkitcat_field_grade')
        if field_produto:
            res[field_produto] = False
        if field_grade:
            res[field_grade] = False
        return {'value': res}

    def wkicat_onchange_produto(self, cr, uid, ids, produto_id, context=None):
        context = context and context.copy() or {}
        res = {}
        field_grade = context.get('ctx_wkitcat_field_grade')
        if not produto_id:
            if field_grade:
                res[field_grade] = False
        else:
            grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid, [('produto_id', '=', produto_id)],
                                                                      context=context)
            if grade_ids and field_grade:
                res[field_grade] = grade_ids[0]
        return {'value': res}

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        wizard_fields = self._columns.keys()
        wizard_vals = filter(lambda i: i[0] in wizard_fields, vals.items())
        res = super(wizard_kit_categoria, self).create(cr, uid, dict(wizard_vals), context)

        acitve_model = context.get('active_model')
        active_id = context.get('active_id')

        kit_categoria_modulo = (context.get('ctx_kit_categoria_modulo') or '').lower()

        if not kit_categoria_modulo:
            raise osv.except_osv('Atencao', 'Modulo para inserir os produtos nao foi expecificado no contexto!')

        if acitve_model == 'kit.categoria.movimento' and active_id:

            obj_categoria_movimento = self.pool.get(acitve_model)

            fields_to_read = [
                'kitcatmov_kit_id', 'kitcatmov_quantidade', 'kitcatmov_nf_id', 'kitcatmov_pedido_id',
                'kitcatmov_orcamento_id'
            ]
            kitcateg_mov_dados = obj_categoria_movimento.read(cr, uid, active_id, fields_to_read, context=context, load=READ_LOAD)
            if isinstance(kitcateg_mov_dados, list):
                kitcateg_mov_dados = kitcateg_mov_dados[0]

            kit_mov_id = kitcateg_mov_dados['id']
            kit_mov_produto_id = kitcateg_mov_dados['kitcatmov_kit_id']
            kit_mov_qtde = kitcateg_mov_dados['kitcatmov_quantidade']
            kit_mov_nf_id = kitcateg_mov_dados['kitcatmov_nf_id']
            kit_mov_pedido_id = kitcateg_mov_dados['kitcatmov_pedido_id']
            kit_mov_orcamento_id = kitcateg_mov_dados['kitcatmov_orcamento_id']

            if kit_categoria_modulo == 'fiscal':
                obj_item = self.pool.get('estoque.movimento.itens')
                wizard_vals = {'nf_id': kit_mov_nf_id, 'estmvi_kitcat_mov_id': kit_mov_id}

            elif kit_categoria_modulo == 'pedido':
                obj_item = self.pool.get('estoquepedidos.pedido.item')
                wizard_vals = {'pedido_id': kit_mov_pedido_id, 'pedite_kitcat_mov_id': kit_mov_id}

            else:
                obj_item = self.pool.get('orcamentos.orcamento.itens')
                wizard_vals = {'orcamento_id': kit_mov_orcamento_id, 'orcori_kitcat_mov_id': kit_mov_id}

            obj_preco_venda = self.pool.get('estoque.preco.venda')
            obj_local_locacao = self.pool.get('estoque.local.locacao')

            produto_browser = self.pool.get('estoque.produto').browse

            # KIT CATEGORIA - (CATEGORIA x QUANTIDADE)
            browse_produto = produto_browser(cr, uid, kit_mov_produto_id)
            record_kit_categoria = browse_produto.estpro_kit_categoria_ids
            dict_categ_qtde = dict(map(lambda i: (i.kitcat_categoria_id.id, i.kitcat_quantidade), record_kit_categoria))

            # PRODUTOS A SEREM LANCADOS - (PRODUTO x CATEGORIA)
            produto_ids = map(lambda j: j[1], filter(lambda i: 'produto_' in i[0], vals.items()))
            grade_ids = map(lambda j: j[1], filter(lambda i: 'grade_' in i[0], vals.items()))
            browse_produto2 = produto_browser(cr, uid, produto_ids)
            dict_produto_categoria = dict(map(lambda i: (i.id, i.categoria_id.id), browse_produto2))

            # COMPANY_ID PARA ESTOQUE.PRODUTO
            produto_company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, tabela='estoque.produto', context=context)

            dict_produto_name = self.pool.get('estoque.utils').estutl_dict_produto_name(cr, uid, produto_ids, context)

            dict_preco_venda = obj_preco_venda.get_preco_venda(
                cr, uid,
                epc_produto_ids=produto_ids,
                epc_company_id=produto_company_id,
                epc_grade_ids=grade_ids,
                epc_cliente_id=None,
                context=context
            )

            movimento_lancado_ids = []

            for i in range(1, (vals.__len__() / 3 + 1)):

                local_id = vals['local_%d_id' % i]
                produto_id = vals['produto_%d_id' % i]
                grade_id = vals['grade_%d_id' % i]

                produto_categoria_id = dict_produto_categoria.get(produto_id)
                qtde_na_categoria = dict_categ_qtde.get(produto_categoria_id) or 1

                quantidade_final = qtde_na_categoria * kit_mov_qtde

                dados_preco_venda = dict_preco_venda.get(produto_id) or {}
                preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                    'preco_venda')

                args_local_local = [
                    ('local_id', '=', local_id), ('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id)
                ]
                locacao_ids = obj_local_locacao.search(cr, uid, args_local_local, context=context)
                if not locacao_ids:
                    embalagem_id = False
                else:
                    locacao_dados = obj_local_locacao.read(
                        cr, uid, locacao_ids[0], ['estllc_produto_embalagem_id'], context=context,
                        load=READ_LOAD
                    )
                    if isinstance(locacao_dados, list):
                        locacao_dados = locacao_dados[0]

                    embalagem_id = locacao_dados['estllc_produto_embalagem_id']

                if not embalagem_id:
                    name = dict_produto_name.get(produto_id) or '** Nome do produto nao encontrado **'
                    raise osv.except_osv(
                        'Atencao',
                        'Nenhuma embalagem encontra-se cadastrada para a Grade e Local do produto - %s escolhido.'
                        % name
                    )

                if kit_categoria_modulo == 'fiscal':
                    wizard_vals.update({
                        'local_id': local_id,
                        'produto_id': produto_id,
                        'grade_id': grade_id,
                        'produto_embalagem_id': embalagem_id,
                        'quantidade': quantidade_final,
                        'vr_unitario': preco_venda,
                        'vr_preco_venda': preco_venda,
                        'vr_total_item': quantidade_final * preco_venda,

                    })
                elif kit_categoria_modulo == 'pedido':
                    wizard_vals.update({
                        'local_id': local_id,
                        'produto_id': produto_id,
                        'grade_id': grade_id,
                        'produto_embalagem_id': embalagem_id,
                        'qtde_pedida': quantidade_final,
                        'vr_unitario': preco_venda,
                        'pei_preco_venda': preco_venda,
                    })
                else:
                    wizard_vals.update({
                        'local_id': local_id,
                        'produto_id': produto_id,
                        'grade_id': grade_id,
                        'produto_embalagem_id': embalagem_id,
                        'quantidade': quantidade_final,
                        'valor_unitario_bruto': preco_venda,
                        'orcori_preco_venda': preco_venda,
                    })

                print wizard_vals

                obj_item.create(cr, uid, wizard_vals, context)
                movimento_lancado_ids.append(kit_mov_id)

            if movimento_lancado_ids:
                obj_categoria_movimento.write(cr, uid, movimento_lancado_ids, {'kitcatmov_lancado': True}, context)

        return res

    def wkitcat_inserir_produtos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return True

    def fields_view_get(self, cr, user, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        context = context and context.copy() or {}
        res = super(wizard_kit_categoria, self).fields_view_get(cr, user, view_id, view_type, context, toolbar, submenu)

        if res.get('type') == 'form' and res.get('model') == 'wizard.kit.categoria':

            active_id = context.get('active_id')
            active_model = context.get('active_model')

            if active_model == 'kit.categoria.movimento':

                fields = self.fields_get(cr, user, context=context)

                relation_local = fields['local_1_id']
                relation_produto = fields['produto_1_id']
                relation_grade = fields['grade_1_id']

                browse_kit_categoria_movimento = self.pool.get(active_model).browse(cr, user, active_id)

                record_kit_categ = browse_kit_categoria_movimento.kitcatmov_kit_id.estpro_kit_categoria_ids
                produto_kit_nome = browse_kit_categoria_movimento.kitcatmov_kit_id.name

                if record_kit_categ:

                    obj_categoria = self.pool.get('estoque.categoria')
                    new_fields = {}
                    list_fields = []

                    # CRIACAO DOS CAMPOS DINAMICOS
                    for num, i in enumerate(record_kit_categ, start=1):

                        categoria_id = i.kitcat_categoria_id.id
                        categoria_nome = obj_categoria.name_get(cr, user, [categoria_id], context=context)[0]

                        label_local = 'local_%d_id' % num
                        label_produto = 'produto_%d_id' % num
                        label_grade = 'grade_%d_id' % num

                        new_fields[label_local] = relation_local.copy()
                        new_fields[label_produto] = relation_produto.copy()
                        new_fields[label_grade] = relation_grade.copy()

                        # MUDA O LABEL DO PRODUTO PARA O NOME DA CATEGORIA
                        new_fields[label_produto]['string'] = categoria_nome[1]

                        # DOMAIN
                        new_fields[label_produto]['domain'] = [('categoria_id', '=', categoria_id)]

                        # ORDEM PARA APARECER NA TELA (PRODUTO E QUANTIDADE)
                        list_fields.append((label_local, label_produto, label_grade))

                    if list_fields:
                        form = Element('form', attrib={'string': 'Kit Categoria'})
                        group = SubElement(form, 'group', attrib={'col': '6', 'colspan': '6'})

                        h2 = SubElement(group, 'h2')
                        SubElement(h2, 'label', attrib={'string': produto_kit_nome, 'colspan': '6'})
                        SubElement(group, 'newline')

                        group_local = SubElement(group, 'group', attrib={'col': '2', 'colspan': '2'})
                        group_produto = SubElement(group, 'group', attrib={'col': '2', 'colspan': '2'})
                        group_grade = SubElement(group, 'group', attrib={'col': '2', 'colspan': '2'})

                        atributos = {
                            'required': '1',
                            'nolabel': '1',
                            'colspan': '4',
                            'modifiers': "{&quot;required&quot;: true}",
                        }

                        atributos_local = atributos.copy()
                        atributos_produto = atributos.copy()
                        atributos_grade = atributos.copy()

                        atributos_local['on_change'] = "wkitcat_onchange_local(context)"

                        # EXIBICAO DOS CAMPOS DINAMICOS
                        for l, p, g in list_fields:

                            # LOCAL
                            SubElement(group_local, 'label', attrib={'for': l, 'colspan': '4'})
                            atributos_local.update({
                                'name': l,
                                'context': "{'ctx_wkitcat_field_produto': '%s', 'ctx_wkitcat_field_grade': '%s'}" % (p, g)
                            })
                            SubElement(group_local, 'field', attrib=atributos_local)

                            # PRODUTO
                            SubElement(group_produto, 'label', attrib={'for': p, 'colspan': '4'})
                            atributos_produto.update({
                                'name': p,
                                'on_change': "wkicat_onchange_produto(%s, context)" % p,
                                'context': "{'local_id': %s, 'ctx_wkitcat_field_grade': '%s'}" % (l, g)
                            })
                            SubElement(group_produto, 'field', attrib=atributos_produto)

                            # GRADE
                            SubElement(group_grade, 'label', attrib={'for': g, 'colspan': '4'})
                            atributos_grade.update({
                                'name': g,
                                'domain': "[('produto_id', '=', %s)]" % p
                            })
                            SubElement(group_grade, 'field', attrib=atributos_grade)

                        atributos_button = {
                            'string': 'Inserir Produtos',
                            'icon': 'gtk-ok',
                            'name': 'wkitcat_inserir_produtos',
                            'type': 'object'
                        }
                        SubElement(group, 'button', atributos_button)

                        info = "Apos confirmar o lancamentos dos itens atualize a pagina."

                        SubElement(group, 'label', attrib={'string': info})

                        txt = tostring(form)

                        # print parseString(txt).toprettyxml()

                        res['fields'].update(new_fields)
                        res['arch'] = txt.replace('amp;', '')

        return res
