# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _


class madeireira_utils(osv.Model):
    # prefixo: madutl_

    _name = 'madeireira.utils'

    def madutl_verifica_medidas(self, cr, uid, produto_embalagem_id, comprimento, largura,
                                espessura, context=None):
        """
        Verifica se as medidas de comprimento, largura e espessura estao conforme a embalagem
        (M3-metro cubico, M2-metro quadrado ou ML-metro linear).
        """
        context = context and context.copy() or dict()
        if comprimento > 0 and largura > 0 and espessura > 0:
            return True
        if not produto_embalagem_id:
            return True
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        sigla = ''
        browse_produto_embalagem = obj_estoque_produto_embalagem.browse(
            cr, uid, produto_embalagem_id, context)
        embalagem_id = browse_produto_embalagem.embalagem_id
        if embalagem_id:
            sigla = embalagem_id.sigla or ''
        if sigla != 'M3' and sigla != 'M2' and sigla != 'ML':
            return True

        msg = ''
        if not comprimento > 0:
            msg = _('Para o item %s, o valor do comprimento nao e valido.')
        elif sigla == 'M3':
            if not largura > 0:
                msg += _('Para o item %s, o valor da largura nao e valido.')
            elif not espessura > 0:
                msg += _('Para o item %s, o valor da espessura nao e valido.')
        elif sigla == 'M2':
            if not largura > 0:
                msg += _('Para o item %s, o valor da largura nao e valido.')

        return msg and msg % browse_produto_embalagem.produto_id.name or True

    @staticmethod
    def madutl_calcula_volume(quantidade, comprimento=0, largura=0, espessura=0,
                              objeto=None, campo=None):
        """
        Este metodo tem como objetivo realizar o calculo do volume de itens referente
        a madeireira

        :param quantidade: float contendo a quantidade do item
        :param comprimento: float contendo o comprimento do item
        :param largura: float contendo a largura do item
        :param espessura: float contendo a espessura do item
        :param objeto: Instancia de classe contendo o atributo a utilizar o calculo de acordo com
        o que deve ser persistido no banco
        :param campo: field do objeto passado como parametro. Quando objeto e campo passados como
         parametro, sera verificado atraves do metodo _float_persist quantos digits o campo possui
         e assim realizar o calculo de acordo com o valor que sera persistido pelo ORM do openerp.
        :type: float
        :type: float
        :type: float
        :type: float
        :type: object
        :type: string
        :rtype: float
        """
        if comprimento <= 0:
            return 0.0

        # Calculo do volume
        if not espessura > 0 and not largura > 0:  # Metro linear
            volume = quantidade * comprimento
        elif not espessura > 0:  # Metro quadrado
            volume = quantidade * comprimento * largura / 100.
        else:  # Metro cubico
            volume = quantidade * comprimento * largura / 100. * espessura / 100.

        if objeto and campo:
            # noinspection PyProtectedMember
            volume = float(objeto._float_persist(campo, volume))

        return volume

    def madutl_get_valor_com_fator(self, cr, uid, produto_id, vr_variacao, pessoa_id=False,
                                   context=None):

        """
        O metodo verifica a existencia de variacao por variacao para o produto, se sim, busca
        pelo comprimento e produto informados. Caso encontre, eh feita a multiplicacao do preco de
        venda (valor_unitario) pelo fator, retornando o resultado dessa multiplicacao que eh o novo
        valor unitario do item
        """

        context = context and context.copy() or {}
        valor = 0
        if not produto_id:
            return valor
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                    context=context)
        cliente_id = False
        if pessoa_id:
            cliente_ids = self.pool.get('financeiro.cliente').search(
                cr, uid, [('pessoa_id', '=', pessoa_id)], context=context)
            cliente_id = cliente_ids and cliente_ids[0] or False
        produto_preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, epc_produto_ids=produto_id, epc_company_id=company_id,
            epc_cliente_id=cliente_id, context=context)
        dados_preco_venda = produto_preco_venda.get(produto_id) or {}
        valor = dados_preco_venda.get('preco_venda')
        obj_estoque_produto = self.pool.get('estoque.produto')
        variacao = obj_estoque_produto.browse(
            cr, uid, produto_id).estpro_variacao
        if variacao != 'S':
            obj_madeireira_produto_variacao = self.pool.get('madeireira.produto.variacao')
            madeira_variacao_ids = obj_madeireira_produto_variacao.search(cr, uid, [
                ('madpro_produto_id', '=', produto_id),
                ('madpro_variacao', '=', vr_variacao)], context=context)
            if madeira_variacao_ids:
                fator = obj_madeireira_produto_variacao.browse(
                    cr, uid, madeira_variacao_ids[0]).madpro_fator
                valor *= fator
        return valor

    def madpro_valida_variacao(self, cr, uid, produto_id, comprimento=0, espessura=0, context=None):
        """
                Funcao para validar a variacao do produto.
                :param cr:
                :param uid:
                :param produto_id:
                :param comprimento:
                :param espessura:
                :param context:
                :return:dict
            """
        context = context and context.copy() or {}
        vals = {'vr_variacao': 0}
        if produto_id:
            obj_estoque_produto = self.pool.get('estoque.produto')
            browse_produto = obj_estoque_produto.browse(cr, uid, produto_id)
            variacao = browse_produto.estpro_variacao
            descricao = browse_produto.name
            if variacao not in ('S',False):
                obj_madeireira_produto_variacao = self.pool.get('madeireira.produto.variacao')
                if variacao == 'C':
                    vr_variacao = comprimento
                    msg_variacao = _('O comprimento ' + str(comprimento) + \
                                     ' nao cadastrado para o produto ' + descricao + '!')
                elif variacao == 'E':
                    vr_variacao = espessura
                    msg_variacao = _('A espessura ' + str(espessura) + \
                                     ' nao cadastrada para o produto ' + descricao + '!')
                madeira_variacao_ids = obj_madeireira_produto_variacao.search(cr, uid, [
                    ('madpro_produto_id', '=', produto_id),
                    ('madpro_variacao', '=', vr_variacao)], context=context)
                vals['vr_variacao'] = vr_variacao
                if not madeira_variacao_ids:
                    vals_value = {'comprimento': comprimento, 'espessura': espessura}
                    if (variacao == 'C' and comprimento > 0) or (variacao == 'E' and espessura > 0):
                        warning = {'title': 'Atencao',
                                   'message': msg_variacao}
                        if variacao == 'C':
                            vals_value['comprimento'] = 0
                        elif variacao == 'E':
                            vals_value['espessura'] = 0
                        vals = {'value': vals_value, 'warning': warning}

        return vals

    def madutl_get_fator_variacao(self, cr, user, item, variacao, context=None):
        """
        Retorna o fator da variacao de comprimento ou espessura do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item
        :param variacao: o comprimento ou a espessura a ser verificado
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type variacao: float
        :type context: dict
        :return: o fator da variacao de comprimento ou espessura.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        variacao_ids = self.pool.get('madeireira.produto.variacao').search(
            cr, user, [('madpro_produto_id', '=', item), ('madpro_variacao', '=', variacao)],
            context=context)
        variacao_dados = variacao_ids and self.pool.get('madeireira.produto.variacao').read(
            cr, user, variacao_ids[0], ['madpro_fator'], context)

        return variacao_dados and variacao_dados.get('madpro_fator')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
