# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv


class estoquepedidos_pedido_item(osv.Model):

    _inherit = 'estoquepedidos.pedido.item'

    def init(self, cr):
        cr.execute("""delete
                      from ir_act_report_xml
                      where model = 'estoquepedidos.pedido.capa' and
                      report_rml != 'addons/madeireira/report/rel_impressao_estoquepedidos.rml'
                      """)

    def pedite_onchange_volume(self, cr, uid, ids, produto_id, qtde_pedida, vr_unitario,
                               vr_desconto, comprimento, largura, espessura, pessoa_id,
                               context=None):
        context = context and context.copy() or {}
        obj_madeireira_utils = self.pool.get('madeireira.utils')
        volume = obj_madeireira_utils.madutl_calcula_volume(
            qtde_pedida, comprimento, largura, espessura, self,
            'pedite_volume')
        qtde_pedida = volume or qtde_pedida

        if produto_id:
            valida_variacao = obj_madeireira_utils.madpro_valida_variacao(
                cr, uid, produto_id, comprimento, espessura, context)

            if valida_variacao.get('warning'):
                valida = valida_variacao['value']
                value = {'pedite_comprimento': valida['comprimento'],
                         'pedite_espessura': valida['espessura']}
                valida_variacao['value'] = value
                return valida_variacao
            else:
                vr_variacao = valida_variacao.get('vr_variacao')

                if vr_unitario:
                    vr_unitario = self.pool.get('madeireira.utils').madutl_get_valor_com_fator(
                        cr, uid, produto_id, vr_variacao, pessoa_id, context)

        total_item = (vr_unitario * qtde_pedida) - vr_desconto
        res = {'valor_total': total_item, 'pedite_volume': volume,
               'vr_unitario': vr_unitario}

        return {'value': res}

    def pedite_onchange_medida(self, cr, uid, ids, pedite_medida_id, context=None):
        context = context and context.copy() or {}
        res = {'value': {'pedite_comprimento': 0, 'pedite_largura': 0, 'pedite_espessura': 0}}

        if pedite_medida_id:
            brw_madeireira_medida = self.pool.get('madeireira.medida').browse(
                cr, uid, pedite_medida_id, context)
            comprimento = brw_madeireira_medida.madmed_comprimento
            largura = brw_madeireira_medida.madmed_largura
            espessura = brw_madeireira_medida.madmed_espessura
            vals = {'pedite_comprimento': comprimento, 'pedite_largura': largura,
                    'pedite_espessura': espessura}
            res['value'] = vals

        return res

    @staticmethod
    def pedite_valida_disponibilidade_saldo(saldo, qtd_reservada, brw_pedido_item, qtd_embalagem):
        """
        Verifica a disponibilidade do saldo do item

        :param saldo: saldo do item
        :param qtd_reservada: qtd_reservada_do_item
        :param brw_pedido_item: browse do item da nota (estoquepedidos.pedido.item)
        :param qtd_embalagem: Quantidade da embalagem do item
        :type saldo: float
        :type qtd_reservada: float
        :type brw_pedido_item: browser
        :type qtd_embalagem: float
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if 0 > saldo - (qtd_reservada + (brw_pedido_item.pedite_volume * qtd_embalagem) or 0):
            return False

        return True

    @staticmethod
    def pedite_get_mensagem_saldo_indisp(produto, local, item, quantidade):
        """
        Retorna a mensagem de saldo indisponivel

        :param produto: nome do produto
        :param local: nome do local informado para o produto
        :param item: browse do item da nota (estoquepedidos.pedido.item)
        :param quantidade: quantidade disponivel no estoque
        :type produto: str
        :type local: str
        :type item: browser
        :type quantidade: float
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """

        solicitado = item.pedite_volume
        msg = '* Saldo indisponivel para %(produto)s em %(local)s.\n' \
              'Quantidade solicidada: %(solicitado)f \n' \
              'Quantidade disponivel: %(quantidade)f' % locals()

        return msg

    @staticmethod
    def pedite_get_query_item_requisitado_pedido(grade_ids, company_id, esboco=False):
        """
        Retorna a query para verificar a quantidade reservada do item para os pedidos de venda
        :param grade_ids: Os ids das grades para ser verificada
        :param company_id:
        :type grade_ids: list
        :type company_id: int
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """

        if esboco:
            _state = "('F', 'C')"
        else:
            _state = "('R', 'F', 'C')"
        str_grade_ids = '(' + ','.join(str(e) for e in grade_ids) + ')'
        _sql = """
                SELECT grade_id,
                       local_id,
                       Sum(quantidade) AS quantidade
                FROM   (SELECT grade_id,
                               local_id,
                               CASE
                                 WHEN COALESCE(pedite_volume, 0.0) > 0 THEN pedite_volume
                                 ELSE qtde_pedida
                               END AS quantidade
                        FROM   estoquepedidos_pedido_item i
                               LEFT JOIN estoquepedidos_pedido_capa c
                                      ON c.id = i.pedido_id
                        WHERE  grade_id IN {grade_ids}
                               AND state NOT IN {state}
                               AND tipo = 'V'
                               AND company_id = {company_id}) AS TMP
                GROUP  BY grade_id,
                          local_id
        """.format(grade_ids=str_grade_ids, state=_state, company_id=company_id)

        return _sql
