# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
import logging

class mkp_acoes(osv.Model):
    _inherit = 'mkp.acoes'
    _logger = logging.getLogger(_inherit)

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de ações aguardando execução.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :return: dominio utilizado para contar ações pendentes.

        :see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        :copyright: OpenERP SA.
        """
        context = context and context.copy() or {}
        return [('state', '!=', 'EX')]

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        if not 'mkpaco_prioridade' in vals:
            vals['mkpaco_prioridade'] = 1
        res = super(mkp_acoes, self).write(cr, user, ids, vals, context=context)
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        ## Verificar se já existe acção registrada para o objeto, se existir não incluir mais
        _obj = vals.get('mkpaco_objeto') or False
        _acao = vals.get('mkpaco_acao') or False
        _id = vals.get('mkpaco_id') or False
        args = [('mkpaco_objeto', '=', _obj), ('mkpaco_acao', '=', _acao), ('mkpaco_id', '=', _id)]
        context.update({'sig_force_company': True})
        _ids = self.search(cr, uid, args, context=context)
        del context['sig_force_company']
        if _ids:  # Já existe acao para o objeto, para o id informado, apenas alterar para executar novamente
            vals['mkpaco_mensagem'] = ''
            vals['state'] = 'AE'
            res = self.write(cr, uid, _ids, vals, context=context)
        else:
            res = super(mkp_acoes, self).create(cr, uid, vals, context=context)
        return res

    def btn_mkpaco_reference(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not ids:
            return True
        if isinstance(ids, list):
            _acao_id = ids[0]
        else:
            _acao_id = ids
        _brw_acoes = self.browse(cr, uid, _acao_id, context=context)
        view_id = False
        link = {
            'view_type': 'form',
            'view_mode': 'form',
            'res_id': _brw_acoes.mkpaco_id,
            'view_id': view_id,
            'target': 'new',
            'type': 'ir.actions.act_window'
        }
        _code_mkp = _brw_acoes.mkpaco_conta_id.mkpcta_integrador
        if _brw_acoes.mkpaco_objeto in ('A', 'S'):
            link['res_model'] = 'mkp.anuncios'
            link['name'] = _('Anuncio')
            return link
        return True

    def btn_mkpaco_executar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        _obj_anuncio = self.pool.get('mkp.anuncios')
        _obj_mensagem = self.pool.get('mkp.mensagens')
        _obj_prod_imagem = self.pool.get('estoque_img.produto')
        # Não fazer o browser de todos os ids, para que no momento da execucao
        # peque os dados mais atualizados.
        # Por exemplo: tem muitos anuncios para atualizar e no meio deles tem para criar,
        # enquanto é processada a lista, alguem pode mandar processar manualmente o anuncio
        # que estava para criar, e ai, qdo a lista chega naquele anuncio para criar,
        # cria novamente, ficando em duplicidade no MKP.
        # com browser individual, apesar de demorar mais, garante que qdo chegar no
        # anuncio que era para criar e ja está criado, o ação vai ser de apenas atualizar.
        for acao_id in ids:
            brw_acao = self.browse(cr, uid, acao_id, context=context)
            if brw_acao.mkpaco_conta_id.mkpcta_parar_comunicacao or \
               brw_acao.mkpaco_conta_id.mkpcta_marketplace_id.mkpmkp_parar_comunicacao:
                self.write(cr, uid, brw_acao.id, {'mkpaco_mensagem': 'Comunicacao parada'}, context=context)
                cr.commit()
                continue
            try:
                resultado = {'status': False, 'mensagem': 'Nao executado'}
                if brw_acao.mkpaco_objeto == 'A':  # Anuncios
                    if brw_acao.mkpaco_acao == 'A':
                        _obj_id = brw_acao.mkpaco_id
                        _mkp_integrador = brw_acao.mkpaco_conta_id.mkpcta_integrador
                        resultado = _obj_anuncio.mkpanu_atualizar_anuncio_mkp(cr, uid, _obj_id, _mkp_integrador, context=context)
                    else:
                        continue
                # todo Luiz Criar método separado para atualizar apenas preço
                elif brw_acao.mkpaco_objeto == 'P':  # Precos
                    if brw_acao.mkpaco_acao == 'A':
                        _obj_id = brw_acao.mkpaco_id
                        _mkp_integrador = brw_acao.mkpaco_conta_id.mkpcta_integrador
                        resultado = _obj_anuncio.mkpanu_atualizar_preco_mkp(cr, uid, _obj_id, _mkp_integrador,
                                                                              context=context)
                    else:
                        continue
                elif brw_acao.mkpaco_objeto == 'M':  # Mensagens
                    if brw_acao.mkpaco_acao == 'A':
                        resultado = _obj_mensagem.mkpmen_atualizar_mensagem_mkp(cr, uid, brw_acao.mkpaco_id, context=context)
                    else:
                        continue
                elif brw_acao.mkpaco_objeto == 'S':  # Saldos
                    if brw_acao.mkpaco_acao == 'A':
                        _mkp_integrador = brw_acao.mkpaco_conta_id.mkpcta_integrador
                        res = _obj_anuncio.mkpanu_atualizar_saldo(cr, uid, brw_acao.mkpaco_id,
                                                                  brw_acao.mkpaco_conta_id.id, _mkp_integrador,
                                                                  context=context)
                        if res:
                            resultado = res
                    else:
                        continue
                else:
                    continue
                if not resultado.get('status'):
                    _mens = resultado.get('mensagem')
                    if resultado.get('cause') and isinstance(resultado.get('cause'), list):
                        for _men_cause in resultado.get('cause'):
                            _mens += ' / ' + _men_cause.get('message') or _men_cause.get('code')
                    _prioridade = (brw_acao.mkpaco_prioridade + 1) if brw_acao.mkpaco_prioridade < 9 else 9
                    self.write(cr, uid, brw_acao.id, {'state': 'ER', 'mkpaco_mensagem': _mens, 'mkpaco_prioridade': _prioridade}, context=context)
                else:
                    if brw_acao.mkpaco_objeto == 'M':
                        self.unlink(cr, uid, brw_acao.id, context=context)
                    else:
                        self.write(cr, uid, brw_acao.id, {'state': 'EX', 'mkpaco_mensagem': ''}, context=context)
                cr.commit()
            except Exception as e:
                try:
                    _mens = e.message.message
                except:
                    try:
                        _mens = e.message if e.message else e.value
                    except:
                        _mens = e
                try:
                    if brw_acao.mkpaco_objeto == 'M':
                        self.unlink(cr, uid, brw_acao.id, context=context)
                    else:
                        self.write(cr, uid, brw_acao.id,
                            {'state': 'ER', 'mkpaco_mensagem': _mens, 'mkpaco_prioridade': 9}, context=context)
                except:
                    pass
        return True

    def mkpaco_cron_executar_acoes(self, cr, uid, context=None):
        """
        Método chamado pelo cron para executar acoes no mercado livre
        @param cr:
        @param uid:
        @param context:
        @return: Sempre True
        """
        context = context and context.copy() or {}
        context.update({'sig_force_company': True})
        _acao = 'AE'
        while True:
            acoes_ids = self.search(cr, uid, [('state', '=', _acao), ('mkpaco_prioridade', '<', 9)],
                                limit=80, context=context)
            if acoes_ids:
                self.btn_mkpaco_executar(cr, uid, acoes_ids, context=context)
            if _acao == 'AE' and len(acoes_ids) < 80:
                _acao = 'ER'
            else:
                break
        return True

    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
        if not ('mkpaco_conta_id' in str(args) or context.get('sig_force_company')):
            _company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, user, 'mkp.contas', context=context)
            conta_ids = self.pool.get('mkp.contas').search(cr, user, [('company_id', '=', _company_id)],
                                                           context=context)
            if conta_ids:
                args.append(('mkpaco_conta_id', 'in', tuple(conta_ids)))
        res = super(mkp_acoes, self).search(cr, user, args, offset=offset, limit=limit, order=order,
                                                context=context, count=count)
        return res
