# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from SigUtils import datas
from datetime import datetime
from dateutil.relativedelta import relativedelta
from openerp.tools.translate import _

import logging
_logger = logging.getLogger(__name__)

class mkp_notificacoes(osv.Model):
    _inherit = "mkp.notificacoes"

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de notificações aguardando execução.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :return: dominio utilizado para contar notificações pendentes.

        :see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        :copyright: OpenERP SA.
        """
        context = context and context.copy() or {}
        return [('state', '!=', 'P')]


    def _mkpnot_processar_notificacao(self, cr, uid, codigo_mkp, notificacao_tipo, dados_notificacao, context=None):
        """
        Metodo a ser sobrescrito por cada Marketplace para tratamento especializado
        :param cr:
        :param uid:
        :param codigo_mkp: Codigo do Marketplace
        :param notificacao_tipo: Tipo de notificacao 'messages', 'itens', 'orders', etc...
        :param dados_notificacao: Dados resultado do read de um registro do marketplace, com todos os atributos
        :param context:
        :return:
        """
        self._logger.info('Processando notificacao! Mkp: %s, Tipo: %s, Dados:%s' % (codigo_mkp, notificacao_tipo,
                                                                                    dados_notificacao))
        # x = self.pool.get('mkp.merliv').mkpMerLiv_set_order_faturado(cr, uid, '2212088697', 1, False, [26559], context=None) # usando apenas para teste
        res = False
        return res

    def _mkpnot_executar_notificacao(self, cr, uid, dados_notificacao, context=None):
        """
        Método chamado para executar uma notificacao
        @param cr:
        @param uid:
        @param dados_notificacao: dicionario resultado de um read de uma notificacao
        @param context:
        @return: True
        """

        def _marcar_como_executado():
            self.write(cr, uid, dados_notificacao.get('id'), {'state': 'P', 'mkpnot_mensagem': ''}, context=context)
            return True

        def _marcar_como_problema(msg):
            try:
                i = int(dados_notificacao.get('mkpnot_attempts'))
                i += 1
                if i > 9:
                    i = 9
            except:
                i = 1
            vals = {
                'state': 'E',
                'mkpnot_mensagem': msg,
                'mkpnot_attempts': i,
            }
            self.write(cr, uid, dados_notificacao.get('id'), vals, context=context)
            return True

        _hoje = self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(),
                                                                       formato='%Y_%m_%d_%H_%M_%s')
        cr.execute(''' BEGIN; SAVEPOINT notificacoes_mkp_%s; ''' % _hoje)
        _item = False
        topic = dados_notificacao.get('mkpnot_topic')
        _conta_id = dados_notificacao.get('mkpnot_conta_id')
        obj_mkp = self.pool.get('mkp.marketplaces')
        codigo_mkp = obj_mkp.mkpmkp_get_codigo(cr, uid, _conta_id, context=None)
        _mensagem = ''
        _item = self._mkpnot_processar_notificacao(cr, uid, codigo_mkp, topic, dados_notificacao, context=context)
        if isinstance(_item, dict):
            if _item.get('status'):
                _item = True
            else:
                _mensagem = _item.get('mensagem')
                _item = False
        if _item:
            _marcar_como_executado()
        else:
            _marcar_como_problema(_mensagem)
        cr.execute(''' COMMIT; ''')
        return True

    def mkpnot_cron_executar_notificacoes(self, cr, uid, context=None):
        """
        Método chamado pelo cron para executar notificacoes recebidas do mercado livre
        @param cr:
        @param uid:
        @param context:
        @return: Sempre True
        """
        def _processar_notificacoes():
            for dados_notificacoes in cr.dictfetchall():
                _conta_id = dados_notificacoes.get('mkpnot_conta_id')
                try:
                    if _spg('mkp.contas').mkpcta_verificar_se_comunica(cr, uid, _conta_id, context=context):
                        self._mkpnot_executar_notificacao(cr, uid, dados_notificacoes, context=context)
                    else:
                        vals = {
                            'mkpnot_mensagem': 'Sem Comunicacao',
                        }
                        self.write(cr, uid, dados_notificacoes.get('id'), vals, context=context)
                except Exception as e:
                    _logger.error(_("%s. Erro na execucao da notificacao: %s") % (e, dados_notificacoes.get('id')))

        context = context and context.copy() or {}
        _spg = self.pool.get
        _conta_proc = {}

        """
        Processar notificacoes recebidas
        """
        _sql = """select * from mkp_notificacoes where state = 'R' and mkpnot_attempts <> '9' limit 80"""
        cr.execute(_sql)
        _processar_notificacoes()

        """
        Reprocessar notificacoes com erros a mais de 10 minutos
        """
        agora = datetime.now()
        agora_10 = agora - relativedelta(minutes=10)
        agora_10 = agora_10.strftime('%Y-%m-%d %H:%M:%S')
        _sql = """select * from mkp_notificacoes where state = 'E' and mkpnot_attempts <> '9' and 'write_date' > '%s' limit 80""" % agora_10
        cr.execute(_sql)
        _processar_notificacoes()

        """
        Reprocessar notifcacoes com erros a mais de 1 hora
        """
        agora = datetime.now()
        agora_60 = agora - relativedelta(minutes=60)
        agora_60 = agora_60.strftime('%Y-%m-%d %H:%M:%S')
        _sql = """select * from mkp_notificacoes where state = 'E' and mkpnot_attempts = '9' and 'write_date' > '%s' limit 80""" % agora_60
        cr.execute(_sql)
        _processar_notificacoes()
        return True

    def mkpnot_cron_excluir_notificacoes(self, cr, uid, prazo=None, context=None):
        """
        Método chamado pelo cron para excluir notificacoes processadas a algum tempo
        @param cr:
        @param uid:
        @param context:
        @return: Sempre True
        """
        context = context and context.copy() or {}
        try:
            prazo = int(prazo)
        except:
            prazo = 7
        _hoje = datas. getDataDeHoje('I')
        _data_fim = datas.calcVencimento(_hoje, (prazo * -1))
        _sql = "delete from mkp_notificacoes where write_date < '%s' and state = 'P'" % (_data_fim[:4]+'-'+_data_fim[4:6]+'-'+_data_fim[6:])
        cr.execute(_sql)
        _data = datas.calcVencimento(datas. getDataDeHoje('I'), -3)
        _data = _data[:4] + '-' + _data[4:6] + '-' + _data[6:] + ' 00:00:00'
        _sql = "delete from mkp_notificacoes where write_date < '%s' and state = 'E'" % _data
        cr.execute(_sql)
        return True


    def btn_mkpnot_executar(self, cr, uid, ids, context=None):
        _sql = """select * from mkp_notificacoes where id in %s""" % ids
        _sql = _sql.replace('[', '(').replace(']', ')')
        cr.execute(_sql)
        for dados_notificacoes in cr.dictfetchall():
            try:
                self._mkpnot_executar_notificacao(cr, uid, dados_notificacoes, context=context)
            except Exception as e:
                _logger.error(_("%s. Erro na execucao da notificacao: %s") % (e, dados_notificacoes.get('id')))
        return True

    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
        if not 'mkpnot_conta_id' in str(args):
            _company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, user, 'mkp.contas', context=context)
            conta_ids = self.pool.get('mkp.contas').search(cr, user, [('company_id', '=', _company_id)],
                                                           context=context)
            if conta_ids:
                args.append(('mkpnot_conta_id', 'in', tuple(conta_ids)))
        res = super(mkp_notificacoes, self).search(cr, user, args, offset=offset, limit=limit, order=order,
                                            context=context, count=count)
        return res
