# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import logging
import ConfigParser
from openerp.tools import config
from openerp.tools.translate import _
from SigUtils.cep import Correios
from ...adm.control.utils import capitalize_all
from SigUtils import sigerp
from HTMLParser import HTMLParser
from datetime import datetime
from pynfe.nfe.manual_600.nf_e400 import NFe as nfe_400
import re
import json


class MLStripper(HTMLParser):
    def __init__(self):
        self.reset()
        self.fed = []
    def handle_data(self, d):
        self.fed.append(d)
    def get_data(self):
        return ''.join(self.fed)

def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    return s.get_data()


class configuracao(object):
    def __init__(self, mkp, secao_id):
        self.secao_id = secao_id
        self.arq = '%s/%s/mkp.config' % (config.get('addons_path'), mkp)
        self.cfg = ConfigParser.ConfigParser()
        self.autorizado = ''
        self.secreto = ''
        self.partner_id = ''
        self.partner_key = ''
        self.shop_id = ''
        self.user_id = ''
        self.conexao = ''
        self.db_password = ''
        self.db_host = ''
        self.db_port = ''
        self.db_user = ''
        self.server = ''
        self.base = ''
        self.redirect = ''
        self.conta_id = ''
        self.app_id = ''
        return

    def lerConfiguracao(self):
        self.cfg.read(self.arq)
        self.server = self.cfg.get(self.secao_id, 'server') if self.cfg.has_option(self.secao_id, 'server') else ''
        self.base = self.cfg.get(self.secao_id, 'base') if self.cfg.has_option(self.secao_id, 'base') else ''
        self.secreto = self.cfg.get(self.secao_id, 'secreto') if self.cfg.has_option(self.secao_id, 'secreto') else ''
        self.redirect = self.cfg.get(self.secao_id, 'redirect') if self.cfg.has_option(self.secao_id, 'redirect') else ''
        self.conta_id = self.cfg.get(self.secao_id, 'conta_id') if self.cfg.has_option(self.secao_id, 'conta_id') else ''
        self.autorizado = self.cfg.get(self.secao_id, 'autorizado') if self.cfg.has_option(self.secao_id, 'autorizado') else ''
        self.user_id = self.cfg.get(self.secao_id, 'user_id') if self.cfg.has_option(self.secao_id, 'user_id') else ''
        self.shop_id = self.cfg.get(self.secao_id, 'shop_id') if self.cfg.has_option(self.secao_id, 'shop_id') else ''
        self.app_id = self.cfg.get(self.secao_id, 'app_id') if self.cfg.has_option(self.secao_id, 'app_id') else ''
        self.partner_id = self.cfg.get(self.secao_id, 'partner_id') if self.cfg.has_option(self.secao_id, 'partner_id') else ''
        self.partner_key = self.cfg.get(self.secao_id, 'partner_key') if self.cfg.has_option(self.secao_id, 'partner_key') else ''
        self.conexao = self.cfg.get(self.secao_id, 'conexao') if self.cfg.has_option(self.secao_id, 'conexao') else ''
        if self.conexao:
            _conexao = sigerp._crypto_sig('d', self.conexao)
            _conexao = json.loads(_conexao)
            self.db_user = _conexao.get('db_user')
            self.db_password = _conexao.get('db_password')
            self.db_host = _conexao.get('db_host')
            self.db_port = _conexao.get('db_port')
        return

    def gravarConfiguracao(self):
        if self.cfg.has_section(self.secao_id):
            self.cfg.remove_section(self.secao_id)
        self.cfg.add_section(self.secao_id)
        if self.server:
            self.cfg.set(self.secao_id, 'server', self.server)
        if self.base:
            self.cfg.set(self.secao_id, 'base', self.base)
        if self.secreto:
            self.cfg.set(self.secao_id, 'secreto', self.secreto)
        if self.redirect:
            self.cfg.set(self.secao_id, 'redirect', self.redirect)
        if self.user_id:
            self.cfg.set(self.secao_id, 'user_id', self.user_id)
        if self.shop_id:
            self.cfg.set(self.secao_id, 'shop_id', self.shop_id)
        if self.conta_id:
            self.cfg.set(self.secao_id, 'conta_id', self.conta_id)
        if self.app_id:
            self.cfg.set(self.secao_id, 'app_id', self.app_id)
        if self.partner_id:
            self.cfg.set(self.secao_id, 'partner_id', self.partner_id)
        if self.partner_key:
            self.cfg.set(self.secao_id, 'partner_key', self.partner_key)
        if self.autorizado:
            self.cfg.set(self.secao_id, 'autorizado', self.autorizado)
        if self.db_user:
            _conexao = {
                'db_user': self.db_user,
                'db_password': self.db_password,
                'db_host': self.db_host,
                'db_port': self.db_port,
            }
            _conexao = sigerp._crypto_sig('e', json.dumps(_conexao))
            self.cfg.set(self.secao_id, 'conexao', _conexao)
        try:
            f = open(self.arq, 'a')
            self.cfg.write(f)
            f.close()
        except Exception as e:
            _mensagem = _('Erro salvando arquivo de configuracao %s-> %s') % (self.arq, str(e))
            raise osv.except_osv(_('Atencao'), _mensagem)
        return


class mkp_utils(osv.Model):
    _name = 'mkp.utils'
    _auto = False
    _tem_erro = False
    _mensagem_erro = ''
    _logger = logging.getLogger(_name)

    def _mkputl_get_kit_itens(self, cr, uid, produto_id, context=None):
        """
        Método a ser sobrescrito par buscar os itens que compoem o kit
        :param self:
        :param cr:
        :param uid:
        :param produto_id:
        :param context:
        :return:
        """
        lista = []
        return lista

    def _mkputl_get_prodkit_contem_item(self, cr, uid, produto_ids, context=None):
        """
        Metodo para retornar produtos que estão contindos no kit do produto kit
            será sobreposto qdo tiver kit instalado
        :param cr:
        :param uid:
        :param produto_ids: ids dos produtos
        :param context:
        :return: lista de ids de produtos que estão contidos em kit dos produtos fornecidos
        """
        lista = []
        return lista

    def mkputl_inutilizar_nf(self, cr, uid, company_id, nro_nf, serie, context=None):
        _spg = self.pool.get
        res = {'status': True, 'mensagem': ''}
        ini = nro_nf
        fim = nro_nf
        justificativa = 'Nota fiscal inutilizada por desistencia de compra'
        if len(justificativa) < 15:
            return {'status': False, 'mensagem': _('A justificativa deve ter no minimo 15 caracteres.')}
        ano = datetime.now().strftime(u'%y')
        par_ids = _spg('fiscal.parametro').search(cr, uid, [('company_id', '=', company_id)], context=context)
        if par_ids:
            par_id = par_ids[0]
        brw_parametro = _spg('fiscal.parametro').browse(cr, uid, par_id, context=context)
        _cnpj = re.sub('[^0-9]', '', brw_parametro.company_id.rescon_pessoa_id.cpf_cnpj)
        try:
            nfe = nfe_400
            nfe = nfe(certify=brw_parametro.certificado_conteudo,
                      key=brw_parametro.chave_conteudo,
                      environment=brw_parametro.fispar_nfe_ambiente, state=brw_parametro.company_id.rescon_pessoa_id.uf,
                      contingency=brw_parametro.fispar_nfe_contingencia)
            processo, __ = nfe.inutilizar_faixa_numeracao(
                ano, _cnpj, serie, ini, fim, justificativa)
        except Exception as err:
            return {'status': False, 'mensagem': _('Erro de conexao com servidor. \n%s') % err}
        if processo.resposta.infInut.cStat.valor not in (u'102', u'563'):
            return {'status': False,
                    'mensagem': '%s - %s' % (processo.resposta.infInut.cStat.valor,
                                              processo.resposta.infInut.xMotivo.valor)}
        elif processo.resposta.infInut.cStat.valor == u'563':
            numero_protocolo = re.sub('[^0-9]', '', processo.resposta.infInut.xMotivo.valor)
        else:
            numero_protocolo = processo.resposta.infInut.nProt.valor
        informacao = ''
        nf_ids = _spg('fiscal.nf.capa').search(
                cr, uid, [('modelo_nf_series_id', '=', brw_parametro.modelo_nf_series_id.id), ('numero_nf', '=', nro_nf),
                           ('serie_nf', '=', serie)], context=context)
        vals_nf = {
            'nfe_protocolo': numero_protocolo,
            'state': 'I', 'nf_propria': True
        }
        context['inutilizacao'] = True
        if nf_ids:
            nf_id = nf_ids[0]
            vals_nf['nfe_status'] = '102 - Inutilizacao de numero homologado'
        else:
            nf_id = _spg('fiscal.utils').gerar_nf(
                cr, uid, company_id=company_id, destinatario_id=False, natureza_id=brw_parametro.fispar_natureza_uf_dentro_venda_id.id,
                tipo='S', modelo_nf_id=brw_parametro.modelo_nf_series_id.id, numero_nf=nro_nf, serie_nf=serie,
                context=context)
            vals_nf.update({
                # 'nfe_status': '%s - %s' % (processo.resposta.infInut.cStat.valor,
                #                            processo.resposta.infInut.xMotivo.valor),
                'nfe_statue': '102 - Inutilizacao de numero homologado',
                'modelo_55': True if brw_parametro.modelo_nf_series_id.modelo_nf_id.fismnf_codigo == u'55' else False,
                'fisnfc_consumidor': True if brw_parametro.modelo_nf_series_id.modelo_nf_id.fismnf_codigo == u'65' else False,
                'numero_nf': nro_nf, 'serie_nf': serie})

        _spg('fiscal.nf.capa').write(cr, uid, nf_id, vals_nf, context=context)
        return True

    def _mkputl_monta_dic_item_pedido(self, cr, uid, prod_id, grade_id, local_id, emb_id, qtd, valor, anuncio_id, context=None):
        """
        Metodo criado para ser sobrescrito qdo utilizar o Kit
        :param cr:
        :param uid:
        :param prod_id:
        :param grade_id:
        :param local_id:
        :param emb_id:
        :param qtd:
        :param valor:
        :param anuncio_id:
        :return: dicionario com os dados para inserir o item no pedido
        """
        res = {}
        res['produto_id'] = prod_id
        res['grade_id'] = grade_id
        res['local_id'] = local_id
        res['vr_unitario'] = valor
        res['qtde_pedida'] = qtd
        res['pedite_anuncio_id'] = anuncio_id
        res['produto_embalagem_id'] = emb_id
        return [(0,0, res)]

    def _mkputl_get_pesquisa_no_mkp(self, cr, uid, brw_anuncio):
        """
        método a ser sobrescrito pelos métodos de cada marketplace
        @param brw_anuncio: browser de um anuncio
          ex.:{576: u'MLB887521819', 565: u'MLB872477007', 542: u'MLB851019359'}
        @return: dicionario que contem dicionario com o resultado da pesquisa
          Ex.: {u'MLB851019359': {'estoque': 63, 'mercado_envio': True, 'parcelas': 'parcelas Com Juros  Com MercadoPago',
                                  'local_vendedor': u'Sete Lagoas (Minas Gerais)', 'qtd_vendido': 231, 'anuncio_tipo': u'gold_special',
                                  'foto': 'xx', 'state': 'A', 'frete_gratis': False, 'preco': 14.49,
                                  'titulo': u'Afinador Digital Crom\xe1tico Clip Viol\xe3o Viola Guitarra Cavaco',
                                  'descricao_anuncio': u'Afinador Digital Crom\xe1tico Clip Viol\xe3o Viola Guitarra Cavaco',
                                  'qtd_visitas': 3654}}

        """
        return {}


    def _mkputl_anuncio_preview(self, cr, uid, brw_mkp_produto, anuncio_id, preview_dict, context=None):
        """
        Metodo para ser sobrescrito em cada marketplace para tratamento especifico da seção dos anuncios que são separados em cada marketplace
        :param cr:
        :param uid:
        :param brw_mkp_produto:
        :param anuncio_id:
        :param preview_dict:
        :param context:
        :return: dicionario: preview_dict
        """
        return preview_dict

    def mkputl_preview(self, cr, uid, prd_mkp_id, anuncio_id=False, context=None):
        """
        Método para apurar preview de produto no marketpace ou de um anuncio individual
        :param cr:
        :param uid:
        :param prd_mkp_id: id do produto no marketplace
        "param anuncio_id: id do anuncio
        :param context:
        :return: string: preview
        """
        def _get_descricao():
            _corpo = brw_modelo_secao.mkpmsc_corpo
            if brw_modelo_secao.mkpmsc_descricao_produto:
                _corpo = brw_mkp_produto.mkpromkp_produto_id.estpro_detalhamento
                if brw_modelo_secao.mkpmsc_modelo_id.mkpmod_tipo == 'H':
                    return _corpo
                if _corpo:
                    if not '\n' in _corpo:
                        _corpo = _corpo.replace('<br>', '\n')
                    _corpo = strip_tags(_corpo)
            return _corpo

        context = context and context.copy() or {}
        brw_mkp_produto = self.pool.get('mkp.produto.marketplaces').browse(cr, uid, prd_mkp_id, context=context)
        preview_dict = {
            'cabecalho': brw_mkp_produto.mkpromkp_modelo_id.mkpmod_cabecalho,
            'rodape': brw_mkp_produto.mkpromkp_modelo_id.mkpmod_rodape,
            'secoes': {}
        }
        if not brw_mkp_produto.mkpromkp_modelo_id:
            raise osv.except_osv(_('Atencao'), _('Nao ha modelo de anuncio registrado para o Magalu!'))
        for brw_modelo_secao in brw_mkp_produto.mkpromkp_modelo_id.mkpmod_secao_ids:
            secao = {
                'name': brw_modelo_secao.name,
                'sequencia': brw_modelo_secao.sequence,
                'cabecalho': brw_modelo_secao.mkpmsc_cab_secao,
                'corpo': _get_descricao()
            }
            idx = brw_modelo_secao.id
            preview_dict['secoes'][idx] = secao
        for brw_mkp_secao in brw_mkp_produto.mkpromkp_secoes_ids:
            idx = brw_mkp_secao.mkppromkpsc_secao_id.id
            secao = preview_dict.get('secoes').get(idx) or {'corpo': ''}
            if brw_mkp_secao.mkppromkpsc_secao_texto:
                if brw_mkp_secao.mkppromkpsc_sobrepoe_acrescenta == 'S':
                    secao['corpo'] = brw_mkp_secao.mkppromkpsc_secao_texto
                else:
                    if secao['corpo']:
                        secao['corpo'] = secao['corpo'] + brw_mkp_secao.mkppromkpsc_secao_texto
                    else:
                        secao['corpo'] = brw_mkp_secao.mkppromkpsc_secao_texto
                preview_dict['secoes'][idx] = secao
        if anuncio_id:
            preview_dict = self._mkputl_anuncio_preview(cr, uid, brw_mkp_produto, anuncio_id, preview_dict, context=context)
        texto = u''
        if preview_dict.get('cabecalho'):
            texto += preview_dict.get('cabecalho')
        secoes = preview_dict.get('secoes')
        sequencias = []
        for secao in secoes:
            if secoes.get(secao).get('corpo'):
                sequencias.append(secoes.get(secao).get('sequencia'))
        sequencias.sort()
        for sequencia in sequencias:
            if not sequencia:
                continue
            for secao in secoes:
                if secoes.get(secao).get('sequencia') == sequencia:
                    texto += secoes.get(secao).get('cabecalho')
                    texto += secoes.get(secao).get('corpo')
                    break
        if preview_dict.get('rodape'):
            texto += preview_dict.get('rodape')
        return texto

    def mkputl_orders_pesquisar_incluir_pais(self, cr, uid, pais, context=None):
        context = context and context.copy() or {}
        pais_id = self.pool.get('adm.pais').search(cr, uid, [('name', '=', pais)], context=context)[0]
        return pais_id

    def mkputl_orders_pesquisar_incluir_uf(self, cr, uid, pais_id, estado, context=None):
        context = context and context.copy() or {}
        __obj_uf = self.pool.get('adm.uf')
        __estado_ids = __obj_uf.search(cr, uid, [('sigla', '=', estado)], context=context)
        if __estado_ids:
            _estado_id = __estado_ids[0]
        else:
            _estado_id = __obj_uf.create(cr, uid, {'name': estado, 'pais_id': pais_id}, context=context)
        return _estado_id

    def mkputl_orders_pesquisar_incluir_localidade(self, cr, uid, localidade, estado_id, context=None):
        context = context and context.copy() or {}
        __obj_cidade = self.pool.get('adm.cidade')
        localidade = self.pool.get('adm.utils').admutl_capitalizar_adm_nome(cr, uid, localidade, context=context)
        __localidade_ids = __obj_cidade.search(cr, uid, [('name', '=', localidade), ('uf_id', '=', estado_id)],
                                               context=context)
        if __localidade_ids:
            _localidade_id = __localidade_ids[0]
        else:
            _localidade_id = __obj_cidade.create(cr, uid, {'name': localidade, 'uf_id': estado_id}, context=context)
        return _localidade_id

    def mkputl_orders_pesquisar_incluir_bairro(self, cr, uid, bairro, context=None):
        context = context and context.copy() or {}
        __obj_bairro = self.pool.get('adm.bairro')
        __bairro_ids = __obj_bairro.search(cr, uid, [('name', '=', bairro)], context=context)
        if __bairro_ids:
            _bairro_id = __bairro_ids[0]
        else:
            _bairro_id = __obj_bairro.create(cr, uid, {'name': bairro}, context=context)
        return _bairro_id

    def mkputl_orders_pesquisar_incluir_logradouro(self, cr, uid, titulo, logradouro, context=None):
        context = context and context.copy() or {}
        __obj_logradouro = self.pool.get('adm.logradouro')
        logradouro = self.pool.get('adm.utils').admutl_capitalizar_adm_nome(cr, uid, logradouro, context=context)
        __logradouro_ids = __obj_logradouro.search(cr, uid, [('descricao', '=', logradouro), ('titulo', '=', titulo)],
                                                   context=context)
        if __logradouro_ids:
            _logradouro_id = __logradouro_ids[0]
        else:
            _logradouro_id = __obj_logradouro.create(cr, uid, {'descricao': logradouro, 'titulo': titulo},
                                                     context=context)
        return _logradouro_id

    def mkputl_orders_get_dados_correios(self, cr, uid, cep, pais_id, context=None):
        context = context and context.copy() or {}
        try:
            _cep_correios = Correios(config.options.get('http_proxy')).consulta(str(cep), primeiro=False)
            if isinstance(_cep_correios, list):
                _cep_correios = _cep_correios[0]
        except:
            _cep_correios = False
        _estado_id = False
        _localidade_id = False
        _bairro_id = False
        _logradouro_id = False
        if _cep_correios:
            _localidade = _cep_correios.get('localidade') or ''
            _bairro = _cep_correios.get('bairro') or ''
            _estado = _cep_correios.get('uf') or ''
            if _estado == '\n':
                _estado = ''
            if _estado:
                _estado_id = self.mkputl_orders_pesquisar_incluir_uf(cr, uid, pais_id, _estado, context=context)
                if _localidade == '\n':
                    _localidade = ''
                if _localidade:
                    _localidade_id = self.mkputl_orders_pesquisar_incluir_localidade(cr, uid, _localidade,
                                                                                             _estado_id,
                                                                                             context=context)
            if _bairro == '\n' or _bairro == '' or _bairro == u'\xa0' or _bairro == ' ':
                _bairro = 'Bairro'
            _bairro_id = self.mkputl_orders_pesquisar_incluir_bairro(cr, uid, _bairro, context=context)
            _logradouro = _cep_correios.get('logradouroDNEC') or _cep_correios.get('logradouroTexto') or _cep_correios.get('logradouroTextoAdicional') or ''
            if ' - ' in _logradouro:
                _logradouro = _logradouro.split(' - ')[0].strip()
            if _logradouro == '\n':
                _logradouro = ''
            if _logradouro:
                _titulo = _logradouro.split()[0]
                lista = _logradouro.split()[1:]
                _logradouro = ''
                for x in lista:
                    _logradouro += x + ' '
                _logradouro = _logradouro.strip()
                _logradouro_id = self.mkputl_orders_pesquisar_incluir_logradouro(cr, uid, _titulo, _logradouro,
                                                                                         context=context)
        return _estado_id, _localidade_id, _bairro_id, _logradouro_id

    def __mkputl_cadastrar_cep(self, cr, uid, cep, context=None):
        context = context and context.copy() or {}
        _cep_ids = self.pool.get('adm.cep').search(cr, uid, [('cep', '=', cep)], context=context)
        _pais_id = self.mkputl_orders_pesquisar_incluir_pais(cr, uid, 'Brasil', context=context)
        if _cep_ids:
            _cep_dados = self.pool.get('adm.cep').read(cr, uid, _cep_ids, [], context=context)[0]
            _pais_id = _cep_dados.get('pessoa_pais_id')[0]
            _estado_id = _cep_dados.get('pessoa_uf_id')[0]
            _localidade_id = _cep_dados.get('pessoa_cidade_id')[0]
            _bairro_id = _cep_dados.get('pessoa_bairro_id')
            if isinstance(_bairro_id, tuple):
                _bairro_id = _bairro_id[0]
            _logradouro_id = _cep_dados.get('pessoa_logradouro_id')
            if isinstance(_logradouro_id, tuple):
                _logradouro_id = _logradouro_id[0]
        else:
            _estado_id, _localidade_id, _bairro_id, _logradouro_id = self.mkputl_orders_get_dados_correios(
                cr, uid, cep, _pais_id, context=context)
            if _estado_id and _localidade_id and _pais_id:
                _vals_cep = {
                    'cep': cep,
                    'pessoa_bairro_id': _bairro_id,
                    'pessoa_cidade_id': _localidade_id,
                    'pessoa_logradouro_id': _logradouro_id,
                    'pessoa_pais_id': _pais_id,
                    'pessoa_uf_id': _estado_id
                }
                self.pool.get('adm.cep').create(cr, uid, _vals_cep, context=context)
        return _pais_id, _estado_id, _localidade_id, _bairro_id, _logradouro_id

    def __mkputl_cadastrar_logradouro(self, cr, uid, rua, context=None):
        context = context and context.copy() or {}
        _logradouro = rua
        _titulo = capitalize_all(_logradouro.split()[0])
        lista = _logradouro.split()[1:]
        _logradouro = ''
        for x in lista:
            _logradouro += x + ' '
        _logradouro = capitalize_all(_logradouro.strip())
        _logradouro_id = self.mkputl_orders_pesquisar_incluir_logradouro(cr, uid, _titulo, _logradouro,
                                                                         context=context)
        return _logradouro_id

    def __mkputl_cadastrar_email(self, cr, uid, pessoa_id, email_tipo_id, email, context=None):
        context = context and context.copy() or {}
        args = [('email_tipo_id', '=', email_tipo_id), ('pessoa_id', '=', pessoa_id)]
        email_ids = self.pool.get('adm.pessoa.email').search(cr, uid, args, context=context)
        _vals_email = {
            'email_tipo_id': email_tipo_id,
            'name': email,
            'pessoa_id': pessoa_id,
        }
        try:  # Problemas em email, pode deixar passar
            if email_ids:
                self.pool.get('adm.pessoa.email').write(cr, uid, email_ids[0], _vals_email, context=context)
            else:
                self.pool.get('adm.pessoa.email').create(cr, uid, _vals_email, context=context)
        except:
            pass

    def __mkputl_cadastrar_fone(self, cr, uid, pessoa_id, ddd, fone, context=None):
        context = context and context.copy() or {}
        if fone[1] == '*':
            return True
        if int(fone[1]) < 7:
            _name = 'Residencial'
        else:
            _name = 'Celular'
        _tipo_fone = \
            self.pool.get('adm.telefone.tipo').search(cr, uid, [('name', '=', _name)], context=context)[0]
        _vals_fone = {
            'ddd': ddd,
            'pessoa_id': pessoa_id,
            'telefone_tipo_id': _tipo_fone,
            'numero': fone
        }
        args = [('telefone_tipo_id', '=', _tipo_fone), ('pessoa_id', '=', pessoa_id)]
        tel_ids = self.pool.get('adm.pessoa.telefone').search(cr, uid, args, context=context)
        if tel_ids:
            self.pool.get('adm.pessoa.telefone').write(cr, uid, tel_ids[0], _vals_fone, context=context)
        else:
            self.pool.get('adm.pessoa.telefone').create(cr, uid, _vals_fone, context=context)
        return True

    def mkputl_orders_alterar_pessoa(self, cr, uid, pessoa_id, nome, cpf, id_externo, ie, cep, rua, rua_nro,
                                     rua_complemento, ddd1, fone1, ddd2, fone2, tipo_email, email, context=None,
                                     **outros_dados):
        obj_pessoa = self.pool.get('adm.pessoa')
        context = context and context.copy() or {}
        brw_pessoa = obj_pessoa.browse(cr, uid, pessoa_id, context=context)
        if nome.strip() and not '***' in nome:
            self.pool.get('res.partner').write(cr, uid, brw_pessoa.partner_id.id, {'name': nome}, context=context)
        if not cpf:
            cpf = re.sub('[^0-9]', '', brw_pessoa.cpf_cnpj)
        if cpf and len(cpf) < 12:
            _tipo = 'F'
        else:
            _tipo = 'J'
        if '***' in cpf:
            _tipo = brw_pessoa.pessoa_tipo
            cpf = brw_pessoa.cpf_cnpj
        _email_tipo_ids = self.pool.get('adm.email.tipo').search(cr, uid, [('name', '=', tipo_email)], context=context)
        _vals_pessoa = {
            'partner_id': brw_pessoa.partner_id.id,
            'pessoa_tipo': _tipo,
            'cpf_cnpj': cpf,
            'admpes_id_externo': id_externo,
        }
        if ie:
            _vals_pessoa['admpes_jur_ie'] = ie

        obj_pessoa.write(cr, uid, pessoa_id, _vals_pessoa, context=context)
        if _email_tipo_ids and email:
            self.__mkputl_cadastrar_email(cr, uid, pessoa_id, _email_tipo_ids[0], email, context=context)
        if fone1:
            self.__mkputl_cadastrar_fone(cr, uid, pessoa_id, ddd1, fone1, context=context)
        if fone2:
            self.__mkputl_cadastrar_fone(cr, uid, pessoa_id, ddd2, fone2, context=context)
        if cep:
            _pais_id, _estado_id, _localidade_id, _bairro_id, _logradouro_id = self.__mkputl_cadastrar_cep(cr, uid, cep,
                                                                                                       context=context)
        else:
            _pais_id = _estado_id = _localidade_id = _bairro_id = _logradouro_id = None
        if not _logradouro_id and rua and not "***" in rua:
            _logradouro_id = self.__mkputl_cadastrar_logradouro(cr, uid, rua, context=context)
        if not _pais_id and outros_dados.get('pais'):
            _pais_id = self.mkputl_orders_pesquisar_incluir_pais(cr, uid, outros_dados.get('pais'), context=context)
        if outros_dados.get('estado'):
            _estado_id = self.mkputl_orders_pesquisar_incluir_uf(cr, uid, _pais_id, outros_dados.get('estado'),
                                                                 context=context)
        if outros_dados.get('cidade') and not '***' in outros_dados.get('cidade'):
            _localidade_id = self.mkputl_orders_pesquisar_incluir_localidade(cr, uid, outros_dados.get('cidade'),
                                                                             _estado_id, context=context)
        if outros_dados.get('bairro') and not "***" in outros_dados.get('bairro'):
            _bairro_id = self.mkputl_orders_pesquisar_incluir_bairro(cr, uid, outros_dados.get('bairro'), context=context)

        _endereco_tipo_id = \
            self.pool.get('adm.endereco.tipo').search(cr, uid, [('name', '=', 'Normal')], context=context)[0]
        if cep:
            _vals_endereco = {
                'cep': cep,
                'endereco_tipo_id': _endereco_tipo_id,
                'complemento': rua_complemento,
                'numero': rua_nro,
                'pessoa_id': pessoa_id,
                'cidade_id': _localidade_id,
                'logradouro_id': _logradouro_id,
                'pais_id': _pais_id,
                'uf_id': _estado_id,
                'bairro_id': _bairro_id,
            }
            args = [('endereco_tipo_id', '=',  _endereco_tipo_id),
                    ('pessoa_id', '=', pessoa_id),
                    ('pais_id', '=',_pais_id),
                    ('uf_id', '=', _estado_id),
                    ('cidade_id', '=', _localidade_id),
                    ('logradouro_id', '=', _logradouro_id)]
            end_ids = self.pool.get('adm.pessoa.endereco').search(cr, uid, args, context=context)
            if end_ids:
                _vals_endereco['endereco_nf_id'] = end_ids[0]
                self.pool.get('adm.pessoa.endereco').write(cr, uid, end_ids[0], _vals_endereco, context=context)
            else:
                _end_id = self.pool.get('adm.pessoa.endereco').create(cr, uid, _vals_endereco, context=context)
                obj_pessoa.write(cr, uid, pessoa_id, {'endereco_nf_id': _end_id}, context=context)
        return pessoa_id

    def mkputl_orders_incluir_pessoa(self, cr, uid, nome, cpf, id_externo, apelido, ie, cep, rua, rua_nro,
                                     rua_complemento, ddd1, fone1, ddd2, fone2, tipo_email, email, context=None, **outros_dados):
        context = context and context.copy() or {}
        obj_pessoa = self.pool.get('adm.pessoa')
        _partner_id = self.pool.get('res.partner').create(cr, uid, {'name': nome}, context=context)
        if not cpf:
            _tipo = 'F'
        elif len(cpf) < 12:
            _tipo = 'F'
        else:
            _tipo = 'J'
        _email_tipo_ids = self.pool.get('adm.email.tipo').search(cr, uid, [('name', '=', tipo_email)], context=context)
        _vals_pessoa = {
            'partner_id': _partner_id,
            'pessoa_tipo': _tipo,
            'cpf_cnpj': cpf,
            'fantasia_apelido': apelido,
            'admpes_id_externo': id_externo,
        }
        if ie:
            _vals_pessoa['admpes_jur_ie'] = ie
        _pessoa_id = obj_pessoa.create(cr, uid, _vals_pessoa, context=context)
        if _email_tipo_ids and email:
            self.__mkputl_cadastrar_email(cr, uid, _pessoa_id, _email_tipo_ids[0], email, context=context)
        if fone1:
            self.__mkputl_cadastrar_fone(cr, uid, _pessoa_id, ddd1, fone1, context=context)
        if fone2:
            self.__mkputl_cadastrar_fone(cr, uid, _pessoa_id, ddd2, fone2, context=context)
        _pais_id, _estado_id, _localidade_id, _bairro_id, _logradouro_id = self.__mkputl_cadastrar_cep(cr, uid, cep,
                                                                                                       context=context)
        if not _logradouro_id and rua:
            _logradouro_id = self.__mkputl_cadastrar_logradouro(cr, uid, rua, context=context)
        if not _pais_id and outros_dados.get('pais'):
            _pais_id = self.mkputl_orders_pesquisar_incluir_pais(cr, uid, outros_dados.get('pais'), context=context)
        if outros_dados.get('estado'):
            _estado_id = self.mkputl_orders_pesquisar_incluir_uf(cr, uid, _pais_id, outros_dados.get('estado'),
                                                                 context=context)
        if outros_dados.get('cidade') and not '***' in outros_dados.get('cidade'):
            _localidade_id = self.mkputl_orders_pesquisar_incluir_localidade(cr, uid, outros_dados.get('cidade'),
                                                                             _estado_id, context=context)
        if outros_dados.get('bairro') and not "***" in outros_dados.get('bairro'):
            _bairro_id = self.mkputl_orders_pesquisar_incluir_bairro(cr, uid, outros_dados.get('bairro'), context=context)

        _endereco_tipo_id = \
            self.pool.get('adm.endereco.tipo').search(cr, uid, [('name', '=', 'Normal')], context=context)[0]
        _vals_endereco = {
            'cep': cep,
            'endereco_tipo_id': _endereco_tipo_id,
            'complemento': rua_complemento,
            'numero': rua_nro,
            'pessoa_id': _pessoa_id,
            'cidade_id': _localidade_id,
            'logradouro_id': _logradouro_id,
            'pais_id': _pais_id,
            'uf_id': _estado_id,
            'bairro_id': _bairro_id,
        }
        _end_id = self.pool.get('adm.pessoa.endereco').create(cr, uid, _vals_endereco, context=context)
        obj_pessoa.write(cr, uid, _pessoa_id, {'endereco_nf_id': _end_id}, context=context)
        return _pessoa_id

    def mkputl_faturar_no_mkp_selecionado(self, cr, uid, mkp_integrador, conta_id, ped_externo, chave, nf_ids, context=None):
        """
        Metodo a ser sobrescrito por cada marketplace
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param conta_id:
        :param ped_externo:
        :param chave:
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        res = True
        return res

    def mkputl_faturar_no_mkp(self, cr, uid, nf_ids, context=None):
        context = context and context.copy() or {}
        _sql = """SELECT id, fisnfc_mkp_nro_pedido, nfe_chave_acesso, numero_nf
                  FROM   fiscal_nf_capa
                  WHERE  id in (%s)""" % tuple(x for x in nf_ids)
        cr.execute(_sql)
        nf_dados = cr.dictfetchall()
        res = True
        for nf_dado in nf_dados:
            if nf_dado.get('fisnfc_mkp_nro_pedido'):
                _ped_externo = nf_dado.get('fisnfc_mkp_nro_pedido')
                _chave = nf_dado.get('nfe_chave_acesso')
                if _chave:
                    _sql = """select id, pedcap_conta_id, pedcap_nro_pedido_externo
                                from estoquepedidos_pedido_capa 
                                where  pedcap_nro_pedido_externo = '%s'
                                """ % _ped_externo
                    cr.execute(_sql)
                    ped_dados = cr.dictfetchall()
                    for dados_pedido in ped_dados:
                        _conta_id = dados_pedido.get('pedcap_conta_id')
                        _sql = """ select mkpcta_integrador from mkp_contas where id = %s""" % _conta_id
                        cr.execute(_sql)
                        contas_dados = cr.dictfetchall()
                        for conta_dados in contas_dados:
                            res = self.mkputl_faturar_no_mkp_selecionado(cr, uid,
                                                                        conta_dados.get('mkpcta_integrador'),
                                                                        _conta_id,
                                                                        _ped_externo,
                                                                        _chave,
                                                                        nf_ids,
                                                                        context=context)
        return res

    def mkputl_entregar_no_mkp(self, cr, uid, brw_encomenda, context=None):
        """
        Metodo utilizado para informar aos marketplaces que o pedido foi entregue ao cliente
        Deve ser sobrescrito em cada marketplace
        :param cr:
        :param uid:
        :param brw_encomenda:
        :param context:
        :return:
        """
        return True


    def mkputl_postar_no_mkp(self, cr, uid, mkp_integrador, _conta_id, _ped_id, codigo_rastreio, data, context=None):
        """
        Metodo utilizado para informar aos marketplaces que o pedido foi postado.
        Deve ser sobrescrito em cada marketplace
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param _conta_id:
        :param _ped_id:
        :param data:
        :param context:
        :return:
        """
        return True

    def mkputl_get_etiqueta_no_mkp(self, cr, uid, mkp_integrador, conta_id, pedido_ids, context=None):
        """
        Metodo para buscar etiqueta no marketplace, deve ser sobrescrito por cada marketplace
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param conta_id:
        :param pedido_ids:
        :param context:
        :return: {status: True/False, 'mensagem': '', 'dados': ''}
        """
        context = context and context.copy() or {}
        return {'status': True, 'mensagem': '', 'dados': ''}

    def atualizar_corpo(self, cr, uid, context=None):
        _obj_produto = self.pool.get('estoque.produto')
        prod_ids = _obj_produto.search(cr, uid, [], context={})
        for dados_produto in _obj_produto.read(cr, uid, prod_ids, [], context={}):
            _prod = dados_produto.get('produto_id')
            t = 0
            _corpo = ''
            while t == 3:
                if t == 0:  # Prestashop
                    _obj_anuncio = self.pool.get('mkp.anuncios')
                    _obj_secao = 'brw_anuncio.mkppsanu_secao_ids'
                    for _anuncio_id in _obj_anuncio.search(cr, uid, [('mkpanu_produto_id', '=', _prod)], context={}):
                        brw_anuncio = _obj_anuncio.browser(cr, uid, _anuncio_id, context={})
                        for brw_secao in brw_anuncio.mkppsanu_secao_ids:
                            if not brw_secao.mkppsans_secao_id.mkpmsc_descricao_produto:
                                continue
                            _corpo = brw_secao.mkppsans_secao_texto
                            break
                        if _corpo:
                            break
                    if _corpo:
                        t = 3
                        continue
                    t += 1
                    continue
                elif t == 1:  # Skyhub
                    _obj_anuncio = self.pool.get('mkp.anuncios')
                    _obj_secao = 'brw_anuncio.mkpanu_secao_ids'
                    for _anuncio_id in _obj_anuncio.search(cr, uid, [('mkpanu_produto_id', '=', _prod)], context={}):
                        brw_anuncio = _obj_anuncio.browser(cr, uid, _anuncio_id, context={})
                        for brw_secao in brw_anuncio.mkpanu_secao_ids:
                            if not brw_secao.mkpshans_secao_id.mkpmsc_descricao_produto:
                                continue
                            _corpo = brw_secao.mkpshans_secao_texto
                            break
                        if _corpo:
                            break
                    if _corpo:
                        t = 3
                        continue
                    t += 1
                    continue
                elif t == 2:  # M.L
                    _obj_anuncio = self.pool.get('mkp.anuncios')
                    _obj_secao = 'brw_anuncio.mkpanu_secao_ids'
                    for _anuncio_id in _obj_anuncio.search(cr, uid, [('mkpanu_produto_id', '=', _prod)], context={}):
                        brw_anuncio = _obj_anuncio.browser(cr, uid, _anuncio_id, context={})
                        for brw_secao in brw_anuncio.mkpanu_secao_ids:
                            if not brw_secao.mkpans_secao_id.mkpmsc_descricao_produto:
                                continue
                            _corpo = brw_secao.mkpans_secao_texto
                            break
                        if _corpo:
                            break
                    if _corpo:
                        t = 3
                        continue
                    t += 1
                    continue

            # if _corpo:
            #     _obj_produto.write(cr, uid, _prod, {'estpro_detalhamento': _corpo})
            continue
        return True

    def mkputl_atualizar_saldo_anuncios(self, cr, uid, produto_ids, context=None):
        """
        Metodo utilizado para atualizar saldo dos anuncios, recebe lista de produtos,
            procura kits que contenham algum dos itens e
            procura kits que condeftenham algum dos itens e
            procuta os anuncios dos itens
        :param cr:
        :param uid:
        :param produto_ids: lista de itens
        :param context:
        :return: Sempre retorna True

        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        if not produto_ids:
            return True
        if not isinstance(produto_ids, (tuple, list)):
            produto_ids = [produto_ids]
        lista = self._mkputl_get_prodkit_contem_item(cr, uid, produto_ids, context=context)
        if lista:
            produto_ids += lista
        str_prod_ids = str(produto_ids).replace('L', '')
        sql = """select 
                    MA.id, mkpanu_produto_id, mkpanu_quantidade, mkpanu_mercadofull, mkpcta_local_id, 
                    mkpcta_saldo_percentual, MC.company_id
                 from mkp_anuncios MA
                 left join mkp_contas MC on MC.id = MA.mkpanu_conta_id
                 where mkpanu_produto_id in %s""" % str_prod_ids
        cr.execute(sql.replace('[', '(').replace(']',')'))
        for dados in cr.dictfetchall():
            if dados.get('mkpanu_mercadofull'):
                continue
            _produto_id = dados.get('mkpanu_produto_id')
            _local_id = dados.get('mkpcta_local_id')
            _perc_saldo = dados.get('mkpcta_saldo_percentual')
            if dados.get('company_id'):
                context['company_id'] = dados.get('company_id')
            _saldo = int(_spg('estoque.produto').estpro_get_saldo(cr, uid, [_produto_id], pesquisar="D",
                                                                  local=_local_id, context=context)[_produto_id])
            if _perc_saldo > 0 or _perc_saldo < 100:
                _saldo = int(_saldo * _perc_saldo / 100)
            _saldo = int(_saldo / dados.get('mkpanu_quantidade'))
            if _saldo < 0:
                _saldo = 0
            _spg('mkp.anuncios').write(cr, uid, dados.get('id'), {'mkpanu_saldo': _saldo}, context=context)
        return True

    def mkputl_atualizar_preco_anuncios(self, cr, uid, produto_ids, context=None):
        """
        Metodo utilizado para atualizar precos dos anuncios, recebe lista de produtos,
            procura kits que contenham algum dos itens e
            procuta os anuncios dos itens
        :param cr:
        :param uid:
        :param produto_ids: lista de itens
        :param context:
        :return: Sempre retorna True

        """
        _spg = self.pool.get
        if not isinstance(produto_ids, (tuple, list)):
            produto_ids = [produto_ids]
        lista = self._mkputl_get_prodkit_contem_item(cr, uid, produto_ids, context=context)
        if lista:
            produto_ids += lista
        args = [('mkpanu_produto_id', 'in', produto_ids)]
        anuncio_ids = _spg('mkp.anuncios').search(cr, uid, args, context=context)
        for brw_anuncio in _spg('mkp.anuncios').browse(cr, uid, anuncio_ids, context=context):
            _produto_id = brw_anuncio.mkpanu_produto_id.id
            _local_id = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
            _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
            vals_acoes = {
                'mkpaco_conta_id': brw_anuncio.mkpanu_conta_id.id,
                'mkpaco_objeto': 'P',
                'mkpaco_acao': 'A',
                'mkpaco_id': brw_anuncio.id,
            }
            _spg('mkp.acoes').create(cr, uid, vals_acoes, context=context)

        return True
