# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from dateutil.parser import parse


class wizard_mkp_avaliacao_venda(osv.TransientModel):
    _name = 'wizard.mkp.avaliacao.venda'
    _columns = {
        'wizavv_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'wizavv_data_ini': fields.date('Data inicial'),
        'wizavv_data_fin': fields.date('Data inicial'),
        'wizavv_nf_id': fields.many2one('fiscal.nf.capa', 'Nota Fiscal',
                                        domain="[('tipo', '=', 'S'), ('state', '=', '3')]"),
        'wizavv_categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'wizavv_agrupar': fields.selection([('P', 'Produto'), ('D', 'Data'), ('C', 'Categoria')],
                                           size=1, string='Agrupar por'),
        'wizavv_custo': fields.selection(
            [('M', 'Medio'), ('C', 'Corrigido'), ('U', 'Ultimo Custo')], size=1, string='Custo'),
    }
    _defaults = {
        'wizavv_custo': 'M',
    }

    def wizavv_button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        avaliacao_venda_dados = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                            context)
        data_ini = avaliacao_venda_dados.wizavv_data_ini
        data_fin = avaliacao_venda_dados.wizavv_data_fin
        produto = avaliacao_venda_dados.wizavv_produto_id
        if data_ini and data_fin and (data_fin < data_ini):
            raise osv.except_osv(_('Atencao'), 'Data final menor que a inicial.')
        dados = {}
        produto and dados.update({'produto': produto.codigo + ' - ' + produto.name})
        data_ini and dados.update({'data_ini': parse(data_ini).strftime('%d-%m-%Y')})
        data_fin and dados.update({'data_fim': parse(data_fin).strftime('%d-%m-%Y')})
        avaliacao_venda_dados.wizavv_nf_id and dados.update(
            {'nf': str(avaliacao_venda_dados.wizavv_nf_id.numero_nf)})
        avaliacao_venda_dados.wizavv_categoria_id and dados.update(
            {'categoria': avaliacao_venda_dados.wizavv_categoria_id.conta + ' - ' +
                avaliacao_venda_dados.wizavv_categoria_id.name})
        avaliacao_venda_dados.wizavv_agrupar and dados.update(
            {'agrupar': avaliacao_venda_dados.wizavv_agrupar})
        avaliacao_venda_dados.wizavv_custo and dados.update(
            {'custo': avaliacao_venda_dados.wizavv_custo})
        relatorio_dados = self.wizavv_get_dados(cr, uid, avaliacao_venda_dados, context)
        relatorio_produto_dados = self.wizavv_get_produto_dados(cr, uid, avaliacao_venda_dados,
                                                                context)
        nf_ids = list(set(map(lambda n: n.get('nf_id'), relatorio_produto_dados)))
        relatorio_comissao_mkp_dados = self.wizavv_get_comissao_mkp_dados(cr, uid, nf_ids, context)
        (not avaliacao_venda_dados.wizavv_agrupar or avaliacao_venda_dados.wizavv_agrupar in (
            'P', 'C')) and dados.update(
            {'comissao_mkp_por_produto': self.wizavv_get_comissao_mkp_por_produto(
                cr, uid, nf_ids, relatorio_comissao_mkp_dados, relatorio_produto_dados, context)})
        if not relatorio_dados:
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para gerar o relatorio.'))
        dados.update({'dados': relatorio_dados, 'produto_dados': relatorio_produto_dados,
                      'comissao_mkp_dados': relatorio_comissao_mkp_dados})
        produto_ids = []
        categoria_produto_dict = {}
        data_produto_dict = {}
        if not avaliacao_venda_dados.wizavv_agrupar:
            produto_ids = list(set(map(lambda p: p.get('produto_id'), relatorio_dados)))
        elif avaliacao_venda_dados.wizavv_agrupar == 'P':
            produto_ids = map(lambda p: p.get('produto_id'), relatorio_dados)
        elif avaliacao_venda_dados.wizavv_agrupar == 'C':
            categoria_ids = map(lambda c: c.get('categoria_id'), relatorio_dados)
            estoque_categoria_dados = self.pool.get('estoque.categoria').read(
                cr, uid, categoria_ids, ['produto_ids'], context)
            for categoria in estoque_categoria_dados:
                categoria_produto_dict.update({categoria.get('id'): categoria.get('produto_ids')})
        elif avaliacao_venda_dados.wizavv_agrupar == 'D':
            for data in relatorio_produto_dados:
                if data.get('data') not in data_produto_dict:
                    data_produto_dict.update({data.get('data'): [data.get('produto_id')]})
                else:
                    data_produto_dict[data.get('data')].append(data.get('produto_id'))
        simples_nacional = self.pool.get('adm.pessoa').verificar_empresa_simples_nacional(cr, uid)
        simples_nacional and dados.update({'simples_nacional': True})
        tributacao_simples_nacional = simples_nacional and self.pool.get(
            'fiscal.tributacao.simples.nacional').fistsn_calcula_aliquota_imposto(
            cr, uid, produto_ids, context) or {}
        categoria_produto_dict and dados.update({'categoria_produto_dict': categoria_produto_dict})
        data_produto_dict and dados.update({'data_produto_dict': data_produto_dict})
        tributacao_simples_nacional and dados.update(
            {'tributacao_simples_nacional': tributacao_simples_nacional})
        estoque_parametro = ['custo_fixo']
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', estoque_parametro, context=context)
        estoque_parametro_dados.get('custo_fixo') and dados.update(
            {'custo_fixo': estoque_parametro_dados.get('custo_fixo') or 0})

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'mkp_avaliacao_venda',
            'datas': dados,
        }

    def wizavv_get_dados(self, cr, uid, avaliacao_venda_dados, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'fiscal.nf.capa',
                                                                    context=context)
        cond_produto = ''
        if avaliacao_venda_dados.wizavv_produto_id:
            cond_produto = """
                AND produto.id = %s
            """ % avaliacao_venda_dados.wizavv_produto_id.id
        cond_data_ini = ''
        if avaliacao_venda_dados.wizavv_data_ini:
            cond_data_ini = """
                AND nf.data_emissao >= '%s 00:00:00'
            """ % avaliacao_venda_dados.wizavv_data_ini
        cond_data_fin = ''
        if avaliacao_venda_dados.wizavv_data_fin:
            cond_data_fin = """
                AND nf.data_emissao <= '%s 23:59:59'
            """ % avaliacao_venda_dados.wizavv_data_fin
        cond_nf = ''
        if avaliacao_venda_dados.wizavv_nf_id:
            cond_nf = """
                AND nf.id = %s
            """ % avaliacao_venda_dados.wizavv_nf_id.id
        cond_categoria = ''
        if avaliacao_venda_dados.wizavv_categoria_id:
            cond_categoria = """
                AND categoria.id = %s
            """ % avaliacao_venda_dados.wizavv_categoria_id.id
        tipo_custo = avaliacao_venda_dados.wizavv_custo or ''

        sel_produto = """
                produto.id					                    AS produto_id,
                produto.codigo || ' - ' || produto.name 	    AS produto,
        """

        sel_grade = """
                grade.grade					                    AS grade,
        """

        sel_data = """
                to_char(nf.data_emissao, 'YYYY-mm-dd')::date	AS data,
        """

        sel_categoria = """
                categoria.id                                    AS categoria_id,
                categoria.conta || ' - ' || categoria.name	    AS categoria,
        """

        sel_nf = """
                nf.numero_nf					                AS nf,
                nf.id                                           AS nf_id,
        """

        agrupar = avaliacao_venda_dados.wizavv_agrupar
        agrupamento = """
                produto.id, grade.id, nf.id, categoria.id
        """
        if agrupar == 'P':
            agrupamento = """
                produto.id, categoria.id
            """
            sel_data = ''
            sel_nf = ''
            sel_grade = ''
        elif agrupar == 'D':
            agrupamento = """
                to_char(nf.data_emissao, 'YYYY-mm-dd')::date
            """
            sel_produto = ''
            sel_categoria = ''
            sel_nf = ''
            sel_grade = ''
        elif agrupar == 'C':
            agrupamento = """
                categoria.id
            """
            sel_produto = ''
            sel_data = ''
            sel_nf = ''
            sel_grade = ''

        ordenacao = ''
        if not agrupar:
            ordenacao = """
            ORDER BY nf.numero_nf
            """

        sql = """
            SELECT
                {sel_produto}
                {sel_grade}
                {sel_data}
                {sel_categoria}
                {sel_nf}
                sum(movimento.vr_total_item_rateado)		    AS vr_vendido,
                sum(movimento.vr_frete_rateado)			        AS vr_frete,
                sum(CASE
                        WHEN '{tipo_custo}' = 'M' 
                            THEN COALESCE(movimento.estmvi_saidas_custo_medio_contabil_total, 0)
                        WHEN '{tipo_custo}' = 'C' 
                            THEN COALESCE(movimento.estmvi_saidas_custo_corrigido_total, 0)
                        WHEN '{tipo_custo}' = 'U' 
                            THEN COALESCE(movimento.estmvi_saidas_ultimo_custo_total, 0)
                        ELSE 0
                    END
                    )                                           AS vr_custo,
                sum((
                    SELECT SUM(vr_encargo) 
                    FROM fiscal_nf_item_tributacao 
                    WHERE 
                        fisnit_capa_id = nf.id 
                        AND movimento_itens_id IN (
                            SELECT id 
                            FROM estoque_movimento_itens 
                            WHERE 
                                produto_id = produto.id 
                                AND nf_id = nf.id
                            )
                ))						                        AS vr_imposto
            FROM
                fiscal_nf_capa nf
            LEFT JOIN
                estoque_movimento_itens movimento ON nf.id = movimento.nf_id
            LEFT JOIN
                estoque_produto produto ON produto.id = movimento.produto_id
            LEFT JOIN
                estoque_produto_grade grade ON grade.id = movimento.grade_id
            LEFT JOIN
                estoque_categoria categoria ON categoria.id = produto.categoria_id
            LEFT JOIN
                fiscal_cfop_natureza natureza ON natureza.id = nf.natureza_id
            LEFT JOIN
                fiscal_cfop cfop ON natureza.cfop_id = cfop.id
            LEFT JOIN 
                estoque_movimento_capa capa ON capa.nf_capa_id = nf.id
            LEFT JOIN
                estoque_natureza_operacao operacao ON operacao.id = capa.estmvc_operacao_id
            WHERE
                nf.company_id = {company_id}
                {cond_data_ini}
                {cond_data_fin}
                AND nf.tipo = 'S'
                AND nf.state = '3'
                AND cfop.tipo = 'S'
                AND cfop.movimento_tipo = 'C'
                AND operacao.estnto_tipo_operacao = 'S'
                AND operacao.estnto_atualizar_saldo IS TRUE
                AND operacao.estnto_gerar_financeiro IS TRUE
                {cond_nf}
                {cond_produto}
                {cond_categoria}
            GROUP BY
                {agrupamento}
            {ordenacao}
        """.format(company_id=company_id, cond_data_ini=cond_data_ini, cond_data_fin=cond_data_fin,
                   cond_produto=cond_produto, cond_nf=cond_nf, cond_categoria=cond_categoria,
                   tipo_custo=tipo_custo, agrupamento=agrupamento, sel_produto=sel_produto,
                   sel_data=sel_data, sel_categoria=sel_categoria, sel_nf=sel_nf,
                   sel_grade=sel_grade, ordenacao=ordenacao)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def wizavv_get_produto_dados(self, cr, uid, avaliacao_venda_dados, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'fiscal.nf.capa',
                                                                    context=context)
        cond_data_ini = ''
        if avaliacao_venda_dados.wizavv_data_ini:
            cond_data_ini = """
                AND nf.data_emissao >= '%s 00:00:00'
            """ % avaliacao_venda_dados.wizavv_data_ini
        cond_data_fin = ''
        if avaliacao_venda_dados.wizavv_data_fin:
            cond_data_fin = """
                AND nf.data_emissao <= '%s 23:59:59'
            """ % avaliacao_venda_dados.wizavv_data_fin
        cond_nf = ''
        if avaliacao_venda_dados.wizavv_nf_id:
            cond_nf = """
                AND nf.id = %s
            """ % avaliacao_venda_dados.wizavv_nf_id.id
        cond_categoria = ''
        if avaliacao_venda_dados.wizavv_categoria_id:
            cond_categoria = """
                AND categoria.id = %s
            """ % avaliacao_venda_dados.wizavv_categoria_id.id

        sel_produto = """
                produto.id					                    AS produto_id,
        """

        sel_data = """
                to_char(nf.data_emissao, 'YYYY-mm-dd')::date	AS data,
        """

        sel_categoria = """
                categoria.id                                    AS categoria_id,
        """

        sel_nf = """
                nf.numero_nf					                AS nf,
        """

        agrupamento = """
                produto.id, nf.id, categoria.id
        """

        sql = """
            SELECT
                {sel_produto}
                {sel_data}
                {sel_categoria}
                {sel_nf}
                nf.id                                           AS nf_id,
                sum(movimento.vr_total_item_rateado)		    AS vr_vendido
            FROM
                fiscal_nf_capa nf
            LEFT JOIN
                estoque_movimento_itens movimento ON nf.id = movimento.nf_id
            LEFT JOIN
                estoque_produto produto ON produto.id = movimento.produto_id
            LEFT JOIN
                estoque_categoria categoria ON categoria.id = produto.categoria_id
            LEFT JOIN
                fiscal_cfop_natureza natureza ON natureza.id = nf.natureza_id
            LEFT JOIN
                fiscal_cfop cfop ON natureza.cfop_id = cfop.id
            LEFT JOIN 
                estoque_movimento_capa capa ON capa.nf_capa_id = nf.id
            LEFT JOIN
                estoque_natureza_operacao operacao ON operacao.id = capa.estmvc_operacao_id
            WHERE
                nf.company_id = {company_id}
                {cond_data_ini}
                {cond_data_fin}
                AND nf.tipo = 'S'
                AND nf.state = '3'
                AND cfop.tipo = 'S'
                AND cfop.movimento_tipo = 'C'
                AND operacao.estnto_tipo_operacao = 'S'
                AND operacao.estnto_atualizar_saldo IS TRUE
                AND operacao.estnto_gerar_financeiro IS TRUE
                {cond_nf}
                {cond_categoria}
            GROUP BY
                {agrupamento}
        """.format(company_id=company_id, cond_data_ini=cond_data_ini, cond_data_fin=cond_data_fin,
                   cond_nf=cond_nf, cond_categoria=cond_categoria,
                   agrupamento=agrupamento, sel_produto=sel_produto,
                   sel_data=sel_data, sel_categoria=sel_categoria, sel_nf=sel_nf)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def wizavv_get_comissao_mkp_dados(self, cr, uid, nf_ids, context=None):
        context = context and context.copy() or {}
        if not nf_ids:
            return {}
        sql = """
            SELECT 
                nf_pedido.nf_id                 AS nf_id,
                sum(COALESCE(condicao.pedcdc_mkp_tarifa, 0.00)) AS vr_tarifa
            FROM
                fiscal_nf_pedidos nf_pedido
            LEFT JOIN
                estoquepedidos_pedido_condicao condicao ON nf_pedido.pedido_id = condicao.capa_id
            WHERE
                nf_pedido.nf_id IN %s
            GROUP BY
                nf_pedido.nf_id 
        """ % str(tuple(nf_ids))
        cr.execute(sql)
        dados = cr.fetchall()
        dados = dict(dados)
        return dados

    def wizavv_get_comissao_mkp_por_produto(self, cr, uid, nf_ids, dict_comissao_nf,
                                            dict_produto_nf, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        res = {}
        for nf_id in nf_ids:
            if not dict_comissao_nf.get(nf_id):
                res.update({nf_id: {}})
                continue
            nf_dados = filter(lambda n: n.get('nf_id') == nf_id, dict_produto_nf)
            list_produto_valor_rateado = obj_adm_utils.efetuar_rateio(
                dict_comissao_nf.get(nf_id),
                map(lambda d: [d.get('produto_id'), d.get('vr_vendido')], nf_dados))
            # produto_ids = []
            # dict_produto_valor = {}
            # for produto in list_produto_valor_rateado:
            #     if produto[0] in dict_produto_valor:
            #         dict_produto_valor[produto[0]] += produto[1]
            #     else:
            #         dict_produto_valor.update({produto[0]: produto[1]})
            #     produto_ids.append(produto[0])
            # res.update({nf_id: dict_produto_valor})
            res.update({nf_id: dict(list_produto_valor_rateado)})
        return res
