# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################
import time
import base64
# from openerp.tools.translate import _
import json
import logging
# from openerp.osv.fields import float_round
import requests
import zipfile
import os
from urllib import urlencode
from integrar_mkp_classes import mkp_objeto

MAG_STATUS = [
    '0-A processar',
    '1-Aguardando aprovacao',
    '2-Divergencias',
    '3-Reprovado',
    '4-Aguardando confirmacao',
    '5-Publicado'
]
MAG_STATUS_CON = {
    400: 'Requisicao mal formatada',
    401: 'Usuario nao autorizado',
    403: 'Acesso nao autorizado',
    405: 'Operacao nao suportada',
    409: 'Recurso em conflito',
    415: 'Content-type invalido',
    422: 'Nao foi possivel processar as instrucoes da requisicao',
    429: 'Requisicao excde a quantidade maxima de chamadas a API',
    500: 'Erro na API',
}

class magalu_mkp_objeto(mkp_objeto):
    def __init__(self):
        super(magalu_mkp_objeto, self).__init__()
        if self.hp == 'p':
            self.api_path = 'https://in.integracommerce.com.br'
        else:
            self.api_path = 'https://api.integracommerce.com.br'
        return

    @staticmethod
    def _get_erro(mensagem):
        try:
            _mens = json.loads(mensagem)
        except:
            return 'Erro nao identificado!'
        ret = '%s\r\n' % _mens.get('Message')
        for x in _mens.get('Errors'):
            ret += '%s-%s\r\n' % (x.get('Field'), x.get('Message'))
        return ret

    def comunicar(self, metodo, uri, dados=None, params={}, headers={}):
        def _comunicar():
            if self.logs_habilitar:
                self._gerar_logs('Magalu.%s-> URI:%s -> DADOS:%s -> HEADER:%s' % (metodo, uri, str(dados), headers))
            response = requests.request(metodo, uri, json=dados, params=params, headers=headers)
            if self.logs_habilitar:
                self._gerar_logs('Magalu.resposta--> %s-%s ' % (response.status_code, response.content))
            return response

        """ Magalu não usa token, é usuario e senha """
        _basic = '%s:%s' % (self.token_cliente.strip(), self.token_passwd.strip())
        _basic = base64.b64encode(_basic)
        headers.update({
            'cache-control': 'no-cache',
            'authorization': 'Basic %s' % _basic,
        })
        uri = self._make_path(uri, params)
        metodo = metodo.upper()
        response = _comunicar()
        if response.status_code == 429:
            time.sleep(3)
        return response

    def get_anuncio(self, codigo_anuncio, variacoes):
        """
        Método a ser sobrescrito pelos metodos de cada parketplace
        :param codigo_anuncio: codigo que identifica o anuncio no mkp é o código do produto no erp ex: 000372
        :param variacoes:  Lista de codigos que identifica as variacoes no mkp, é a grade no erp ex: UN, VM
        :return:
        """
        res = super(magalu_mkp_objeto, self).get_anuncio(self, codigo_anuncio)
        _url = '/api/Product/%s' % codigo_anuncio
        res = self.comunicar('get', _url, dados=None, params={}, headers={})
        if res.status_code > 399:
            self.sts_status = False
            self.sts_mensagem = res.content
            return
        _produto = json.loads(res.content)
        _skus = {}
        for variacao in variacoes:
            _url = '/api/Sku/%s' % variacao
            res = self.comunicar('get', _url, dados=None, params={}, headers={})
            if res.status_code > 399:
                _skus[variacao] = ''
                continue
            _skus[variacao] = json.loads(res.content)
        _produto['skus'] = _skus
        self.sts_status = True
        self.sts_mensagem = ''
        self.dados = _produto
        return

    def set_anuncio(self, dados):
        res = super(magalu_mkp_objeto, self).set_anuncio(self, dados)
        return res


    def get_categorias(self):
        _url = '/api/Category'
        _level = 0
        _categorias = []
        while _level < 4:
            dados = {
                'level': 0,
                'page': 0,
                'perPage': 100000,
            }
            res = self.comunicar('get', _url, dados, params={}, headers={})
            if res.status_code > 399:
                self.sts_status = False
                self.sts_mensagem = res.content
                return
            _categorias.append(json.loads(res.content))
            _level += 1
        self.sts_status = True
        self.sts_mensagem = ''
        self.dados = _categorias
        return

    def set_categorias(self, codigo, categoria, categoria_pai=None):
        if not categoria:
            self.sts_status = False
            self.sts_mensagem = 'Faltou a descricao da categoria'
            return
        uri = '/api/Category'
        dados = {
            'Id': codigo or id,
            'Name': categoria,
            'ParentId': categoria_pai,
        }
        response = self.comunicar('post', uri, [dados])
        if response.status_code > 399:
            if response.content:
                _mens = json.loads(response.content).get('error')
            else:
                _mens = response.content
            self.sts_status = False
            self.sts_mensagem = _mens
            return
        self.sts_status = True
        self.sts_mensagem = ''
        self.dados = {}
        return

    def set_preco(self, anuncio, preco):
        vals = []
        vals.append({
            'IdSku': anuncio,
            'SalePrice': preco,
            'ListPrice': round((preco / 0.9), 2),
        })
        _url = 'api/Price'
        response = self.comunicar('put', _url, vals, params={}, headers={})
        if response.status_code > 399:
            try:
                _mens = self._get_erro(response.content)
            except:
                _mens = MAG_STATUS_CON.get(response.status_code, 'Erro nao identificado')
            self.sts_status = False
            self.sts_mensagem = _mens
            return
        self.sts_status = True
        self.sts_mensagem = ''
        self.dados = {}
        return

    def set_saldo(self, anuncio, saldo):
        vals = [{'IdSku': anuncio, 'Quantity': saldo}]
        _url = '/api/Stock'
        response = self.comunicar('put', _url, vals, params={}, headers={})
        if response.status_code > 399:
            self.sts_status = False
            self.sts_mensagem = MAG_STATUS_CON.get(response.status_code, 'Erro nao identificado')
            return
        self.sts_status = True
        self.sts_mensagem = ''
        self.dados = {}
        return


#
#     def mkpMagalu_set_anuncio(self, cr, uid, brw_anuncio, context=None):
#
#         def _atualizar_variacoes():
#             variacoes_atuais = {}
#             for brw_var in brw_anuncio.mkpanu_variacoes_ids:
#                 variacoes_atuais[brw_var.mkpanuv_grade_id.id] = brw_var.mkpanuv_id_no_mkp
#                 if brw_var.mkpanuv_id_no_mkp == vals.get('IdSku'):
#                     return
#             if variacoes_atuais.get(brw_grade.id):  # Já tem
#                 return
#
#             vals_var = {
#                 'mkpanuv_anuncio_id': brw_anuncio.id,
#                 'mkpanuv_grade_id': brw_grade.id,
#                 'mkpanuv_id_no_mkp': vals.get('IdSku'),
#             }
#             _spg('mkp.anuncios.var').create(cr, uid, vals_var, context=context)
#             return
#
#         context = context and context.copy() or {}
#         _spg = self.pool.get
#         mkp_con = Magalu(cr, uid, brw_anuncio.mkpanu_conta_id.id)
#         args = [
#             ('mkpromkp_produto_id', '=', brw_anuncio.mkpanu_produto_id.id),
#             ('mkpromkp_marketplace_id', '=', brw_anuncio.mkpanu_marketplace_id.id)
#         ]
#         mkpromkp_ids = _spg('mkp.produto.marketplaces').search(cr, uid, args, context=context)
#         if not mkpromkp_ids:
#             return {'status': False, 'mensagem': 'Marketplace nao cadastrado!'}
#
#         brw_mkpromkp = _spg('mkp.produto.marketplaces').browse(cr, uid, mkpromkp_ids[0], context=context)
#
#         _atributos = []
#         _marca = ''
#         for brw_atributo in brw_mkpromkp.mkpromkp_produto_id.estpro_atributo_ids:
#             _valor_att = brw_atributo.estatp_atributo_valor
#             if brw_atributo.estatp_tem_unidade:
#                 _valor_att = '%s%s' % (brw_atributo.estatp_atributo_valor, brw_atributo.estatp_atributo_unidade_id.name)
#             if not _valor_att:
#                 return {'status': False,
#                         'mensagem': 'Item %s com atributos sem valor!' % brw_atributo.estatp_produto_id.codigo}
#
#             _dic = {'Name': brw_atributo.estatp_atributo_id.name, 'Value': _valor_att}
#             if brw_atributo.estatp_atributo_id.name.lower() == 'marca':
#                 _marca = _valor_att
#             _atributos.append(_dic)
#         _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
#         vals = {
#             'IdProduct': brw_anuncio.mkpanu_produto_id.codigo,
#             'Code': brw_anuncio.mkpanu_produto_id.codigo,
#             'Name': brw_anuncio.mkpanu_title,
#             'Brand': _marca or 'OEM',
#             'NbmOrigin': '0' if brw_anuncio.mkpanu_produto_id.origem_mercadoria in ('0', '3', '4', '5', '8') else '1',
#             'NbmNumber': brw_anuncio.mkpanu_produto_id.ncm_id.codigo
#         }
#         _tempo = brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_prazo
#         if brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_prazo_tipo == 'D':
#             _tempo = int(_tempo / 30)
#         elif brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_prazo_tipo == 'A':
#             _tempo = _tempo * 12
#         vals['WarrantyTime'] = _tempo
#         vals['Active'] = True
#         _cat_code = brw_mkpromkp.mkpromkp_categoria_id.mkpcat_codigo
#         _cat_name = brw_mkpromkp.mkpromkp_categoria_id.name
#         _cat_pai = brw_mkpromkp.mkpromkp_categoria_id.parent_id.mkpcat_codigo
#         vals['Categories'] = [{
#             'Id': _cat_code,
#             'Name': _cat_name,
#             'ParentId': _cat_pai
#         }]
#         if brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_responsavel:
#             _g = "do vendedor" if brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_responsavel == 'L' else "de fábrica"
#             _tp = {'D': 'dias', 'M': 'meses', 'A': 'anos'}
#             _gar = '%s Prazo:%s %s' % (_g,
#                                        brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_prazo,
#                                        _tp.get(brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia_prazo_tipo))
#         else:
#             _gar = brw_anuncio.mkpanu_produto_id.estpro_mkp_garantia
#         _atributos.append({'Name': 'Garantia', 'Value': _gar})
#         vals['Attributes'] = _atributos
#         _url = '/api/Product'
#         if brw_anuncio.mkpanu_codigo_anuncio:
#             response = mkp_con.magalu_comunicar('put', _url, vals)
#         else:
#             response = mkp_con.magalu_comunicar('post', _url, vals)
#         if response.status_code == 400:
#             response = mkp_con.magalu_comunicar('put', _url, vals)
#         if response.status_code > 399:
#             return {'status': False, 'mensagem': MAG_STATUS_CON.get(response.status_code, 'Erro nao identificado')}
#
#         for brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
#             _imagens = []
#             _imagens_comuns = []
#             vals = {
#                 'IdProduct': brw_anuncio.mkpanu_produto_id.codigo
#             }
#             for brw_imagens in brw_mkpromkp.mkpromkp_imagens_ids:
#                 if brw_grade.grade != 'UN':
#                     if not brw_imagens.mkprmkimg_imagem_produto_id.estimg_grade_id:
#                         _imagens_comuns.append(brw_imagens.mkprmkimg_endereco_mkp)
#                         continue
#                     if brw_imagens.mkprmkimg_imagem_produto_id.estimg_grade_id.id != brw_grade.id:
#                         continue
#                 _imagens.append(brw_imagens.mkprmkimg_endereco_mkp)
#                 self._logger.info('imagem: seq %s emdereco %s' % (brw_imagens.sequence, brw_imagens.mkprmkimg_endereco_mkp))
#             if _imagens_comuns:
#                 _imagens += _imagens_comuns
#             if not _imagens:
#                 return {'status': False,
#                         'mensagem': 'Item %s-%s sem fotos!' % (brw_grade.produto_id.codigo, brw_grade.grade)}
#             _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
#             _ean = False
#             for brw_cod_barra in brw_anuncio.mkpanu_produto_id.automacao_ids:
#                 if not brw_cod_barra.active:
#                     continue
#                 if brw_cod_barra.grade_id.id != brw_grade.id:
#                     continue
#                 if brw_cod_barra.autcba_tipo_barra == '13':
#                     _ean = brw_cod_barra.codigo_barra[1:]
#
#             vals['IdSku'] = '%s-%s' % (brw_grade.produto_id.codigo, brw_grade.grade)
#             vals['IdSkuErp'] = '%s-%s' % (brw_grade.produto_id.codigo, brw_grade.grade)
#             vals['Name'] = brw_anuncio.mkpanu_title
#             if brw_anuncio.mkpanu_produto_id.grade_usar:
#                 vals['Name'] = '%s - %s' % (brw_anuncio.mkpanu_title, brw_grade.descricao)
#                 i = 0
#                 for __att in _atributos:
#                     if __att.get('Name') == brw_anuncio.mkpanu_produto_id.estpro_grade_descricao:
#                         _atributos.pop(i)
#                         break
#                     i += 1
#                 _dic = {'Name': brw_anuncio.mkpanu_produto_id.estpro_grade_descricao, 'Value': brw_grade.descricao}
#                 _atributos.append(_dic)
#
#             vals['Description'] = brw_anuncio.mkpanu_descricao_html
#             _peso = 0.3
#             _altura = 0.07
#             _largura = 0.10
#             _comprimento = 0.10
#             for brw_embalagem in brw_anuncio.mkpanu_produto_id.produto_embalagem_ids:  # pegar APENAS A PRIMEIRA
#                 _peso = brw_embalagem.peso_bruto
#                 _altura = round(brw_embalagem.altura / 100, 2)
#                 _largura = round(brw_embalagem.largura / 100, 2)
#                 _comprimento = round(brw_embalagem.profundidade / 100, 2)
#                 break
#             vals['Height'] = _altura
#             vals['Width'] = _largura
#             vals['Length'] = _comprimento
#             vals['Weight'] = _peso
#
#             vals['CodeEan'] = _ean
#             vals['CodeNcm'] = brw_anuncio.mkpanu_produto_id.ncm_id.codigo
#             if brw_anuncio.mkpanu_produto_id.grade_usar:
#                 vals['Variation'] = brw_grade.descricao
#             vals['Status'] = True
#             vals['Price'] = {
#                 'SalePrice': _valor,
#                 'ListPrice': float_round((_valor / 0.9), 2),
#             }
#             if brw_anuncio.mkpanu_produto_id.grade_usar:
#                 _x = _spg('estoque.produto.grade').estpgr_get_saldo(cr, uid, [brw_grade.id], 'D', _local,
#                                                                     context=context)
#                 _saldo = int(_x.get(brw_grade.id) / brw_anuncio.mkpanu_quantidade) if _x.get(brw_grade.id) > 0 else 0
#             else:
#                 _saldo = brw_anuncio.mkpanu_saldo
#             vals['StockQuantity'] = _saldo
#             _imagens = [x.replace('https', 'http') for x in _imagens]
#             vals['MainImageUrl'] = _imagens[0]
#             self._logger.info('imagem principal %s' % _imagens[0])
#             _imagens.pop(0)
#             if _imagens:
#                 vals['UrlImages'] = _imagens
#             vals['Attributes'] = _atributos
#             _codeGrade = ''
#             for brw_var in brw_anuncio.mkpanu_variacoes_ids:
#                 if brw_var.mkpanuv_grade_id.id == brw_grade.id:
#                     _codeGrade = '%s-%s' % (brw_anuncio.mkpanu_produto_id.codigo, brw_grade.grade)
#                     break
#             _url = '/api/Sku'
#             if _codeGrade:
#                 response = mkp_con.magalu_comunicar('put', _url, vals)
#             else:
#                 response = mkp_con.magalu_comunicar('post', _url, vals)
#             if response.status_code == 400:
#                 response = mkp_con.magalu_comunicar('put', _url, vals)
#             if response.status_code > 399:
#                 _mens = 'Erro: %s-%s' % (response.status_code, _get_erro(response.content))
#                 return {'status': False, 'mensagem': _mens}
#             # res = json.loads(response.content)
#             vals_anuncio = {}
#             vals_anuncio['mkpanu_codigo_anuncio'] = brw_anuncio.mkpanu_produto_id.codigo
#             if brw_anuncio.mkpanu_produto_id.grade_usar:
#                 _atualizar_variacoes()
#             _spg('mkp.anuncios').write(cr, uid, brw_anuncio.id, vals_anuncio, context=context)
#         return {'status': True, 'mensagem': ''}
#
#
#     def mkpMagalu_pesquisar_orders_queues(self, cr, uid, conta_id, context=None):
#         context = context and context.copy() or {}
#         mkp_con = Magalu(cr, uid, conta_id)
#         _url = 'api/OrderQueue'
#         for _status in ('NEW', 'APPROVED', 'CANCELED'):
#             response = mkp_con.magalu_comunicar('get', _url, params={'Status': _status})
#             if response.status_code > 399:
#                 return {'status': False, 'mensagem': json.loads(response.content).get('message')}
#             res = json.loads(response.content)
#             if res.get('Total') == 0:
#                 continue
#             for _order in res.get('OrderQueues'):
#                 args = [('mkpnot_conta_id', '=', conta_id), ('mkpnot_resource', '=', '/orders/%s' % _order.get('IdOrder'))]
#                 not_ids = self.pool.get('mkp.notificacoes').search(cr, uid, args, context=context)
#                 if not_ids:
#                     vals = {
#                         'state': 'R',
#                     }
#                     self.pool.get('mkp.notificacoes').write(cr, uid, not_ids, vals, context=context)
#                 else:
#                     vals = {
#                     'mkpnot_conta_id': conta_id,
#                     'mkpnot_resource': '/orders/%s' % _order.get('IdOrder'),
#                     'mkpnot_topic': 'orders',
#                     'mkpnot_attempts': 1,
#                     'state': 'R',
#                     }
#                     res = self.pool.get('mkp.notificacoes').create(cr, uid, vals, context=context)
#                     vals = [{"Id": _order.get('Id')}]
#                 response = mkp_con.magalu_comunicar('put', _url, vals)
#                 if response.status_code > 399:
#                     return {'status': False, 'mensagem': json.loads(response.content).get('message')}
#         return True
#
#     def mkpMagalu_get_orders(self, cr, uid, pedido, conta_id, context=None):
#         """
#         Método destinado a buscar um pedido no Magalu
#         @param cr:
#         @param uid:
#         @param pedido: Número do pedido no Magalu
#         @param context:
#         @return: Sempre True
#         """
#         context = context and context.copy() or {}
#         mkp_con = Magalu(cr, uid, conta_id)
#         _url = 'api/Order/%s' % pedido
#         response = mkp_con.magalu_comunicar('get', _url)
#         if response.status_code > 399:
#             if response.status_code == 404:
#                 return {'status': False, 'mensagem': 'Pagina nao encontrada!'}
#             _json = json.loads(response.content)
#             _causa = ' '
#             for x in _json.get('errors'):
#                 """Item nao sobreu alteracao"""
#                 if response.status_code == 422 and x.get('detail'):
#                     return {'status': False, 'mensagem': x.get('detail')}
#                 if isinstance(x, unicode):
#                     _causa = '%s %s\n' % (_causa, x)
#                 else:
#                     _causa = '%s%s\n' % (_causa, x.get('detail'))
#             _mens = _causa
#             return {'status': False, 'mensagem': _mens}
#         dados = json.loads(response.text)
#         return {'status': True, 'mensagem': '', 'pedido': dados}
#
#     def mkpMagalu_set_orders_status(self, cr, uid, order, conta_id, status, context=None):
#         """
#         Método destinado a buscar um pedido no Magalu
#         @param cr:
#         @param uid:
#         @param pedido: Número do pedido no Magalu
#         @param context:
#         @return: Sempre True
#         """
#         context = context and context.copy() or {}
#         mkp_con = Magalu(cr, uid, conta_id)
#         _url = 'api/Order'
#         vals = {
#             "IdOrder": order,
#             'OrderStatus': status,
#         }
#         response = mkp_con.magalu_comunicar('put', _url, vals)
#         if response.status_code > 399:
#             if response.status_code == 404:
#                 return {'status': False, 'mensagem': 'Pagina nao encontrada!'}
#         return True
#
#     def mkpMagalu_set_order_faturado(self, cr, uid, _order, _conta_id, brw_nf, context=None):
#         context = context and context.copy() or {}
#         con_magalu = Magalu(cr, uid, _conta_id)
#         recurso = 'api/Order'
#         data = {
#             "IdOrder": _order,
#             "OrderStatus": 'INVOICED',
#             "InvoicedNumber": brw_nf.numero_nf,
#             "InvoicedLine": brw_nf.serie_nf,
#             "InvoicedIssueDate": brw_nf.data_emissao,
#             "InvoicedKey": brw_nf.nfe_chave_acesso,
#             "InvoicedDanfeXml": brw_nf.nfe_xml_proc
#         }
#         response = con_magalu.magalu_comunicar('put', recurso, dados=data)
#         _json = False
#         if response.content:
#             _json = json.loads(response.content)
#         if response.status_code > 399:
#             if _json:
#                 _mens = str(_json.get('detail')) + '\n'
#             else:
#                 _mens = 'Erro: %s Sem conteudo!' % response.status_code
#             return {
#                 'status': False,
#                 'mensagem': _mens
#             }
#         ret = {'status': True, 'mensagem': ''}
#         return ret
#
#     def mkpMagalu_get_etiqueta(self, cr, uid, order, conta_id, context):
#         context = context and context.copy() or {}
#         con_magalu = Magalu(cr, uid, conta_id)
#         recurso = 'api/Order/ShippingLabels'
#         data = {
#             "Format": 'ZPL',
#             "Orders": [order],
#         }
#         response = con_magalu.magalu_comunicar('post', recurso, dados=data)
#         _resp = json.loads(response.content)
#         if response.status_code > 399:
#             _mens = str(_resp.get('detail')) + '\n'
#             return {
#                 'status': False,
#                 'mensagem': _mens
#             }
#         for etiq in _resp:
#             _url = etiq.get('Url')
#             _track = etiq.get('Orders')[0].get('TrackingCode')
#         dados = requests.request('get', _url)
#         path_zip = '/tmp/etiqueta-magalu-%s.zip' % _track
#         f = open(path_zip, 'w')
#         f.write(dados.content)
#         f.close()
#         path = '/tmp/etiqueta-magalu-%s' % _track
#         try:
#             fantasy_zip = zipfile.ZipFile(path_zip)
#             fantasy_zip.extractall(path)
#             fantasy_zip.close()
#         except Exception as e:
#             try:
#                 f = open(path_zip, 'r')
#                 _etiq = json.loads(f.read())
#                 f.close()
#                 raise Exception('Erro na leitura da etiqueta\n%s' % _etiq.get('message'))
#             except Exception as e:
#                 raise osv.except_osv(_('Atencao!'),
#                                  _('Nao consegui extrair a etiqueta:\n\n%s') % e)
#         f_in = open('%s/etiquetas.zpl' % path, 'r')
#         fi = f_in.readlines()
#         f_in.close()
#         out_path = '/tmp/etiqueta-magalu-%s.zpl' % str(_track)
#         f = open(out_path, 'w')
#         f.writelines(fi)
#         f.close()
#         os.system('rm -rf ' + path_zip)
#         os.system('rm -rf ' + path)
#         return {'status': True, 'mensagem': '', 'etiqueta': out_path, 'track_no': _track}