# -*- encoding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
# #############################################################################

from openerp.osv import osv, fields
import logging


class mkp_contas(osv.Model):
    """
    Tabela de acoes a serem executads no mercado Livre

    prefixo: mkpaco_
    """

    _name = 'mkp.contas'
    _roteavel = True
    _logger = logging.getLogger(_name)
    _columns = {
        'mkpcta_marketplace_id': fields.many2one('mkp.marketplaces', 'Marketplace', required=True, select=True),
        'mkpcta_integrador': fields.related('mkpcta_marketplace_id', 'mkpmkp_codigo', type="char",
                                            string='Integrador', store=True, size=6, select=True),
        'name': fields.char('Descricao da conta', required=True, size=20),
        'mkpcta_saldo_percentual': fields.integer('Saldo disponivel',
                                                  help='Percentual de saldo disponivel para o Marketplace'),
        'mkpcta_local_id': fields.many2one('estoque.local', 'Local de venda', required=True,
                                           help='Local de venda para o Marketplace'),
        'mkpcta_host_erp': fields.char('Host(ERP)', size=256, help='Endereço do servidor SigERP'),
        'mkpcta_banco_erp': fields.char('Banco(ERP)', size=20, help='Banco do SigERP'),
        'mkpcta_rastreamento_servico_id': fields.many2one('rastreio.servicos', 'Servico de rastreamento', select=True),
        'mkpcta_operacao_frete_id': fields.many2one('financeiro.operacoes', 'Operacao para frete(a pagar)',
                                                    help="Identifique a operacao a ser usada para o frete a pagar ao Marketplace\nNao esqueça de classificar os custos se for utilizar"),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # id da empresa\n
        'mkpcta_fulfillment_empresa_id': fields.many2one("res.company", 'Empresa para Fulfillment', ondelete="cascade", select=True, oldname='mkpcta_mercadofull_empresa_id'),
        'mkpcta_fulfillment_local_id': fields.many2one('estoque.local', 'Local para Fulfillment', ondelete="cascade", select=True),
        'mkpcta_fulfillment_cnpj': fields.char('CNPJ do Fulfillment', size=15),
        'mkpcta_full_modelo_90_nf_series': fields.many2one("fiscal.modelo.nf.series", 'Modelo NF Full', ondelete="restrict", select=True,
                                                        help='Modelo 90 de nf para as notas de movimento no local do full, se nao informar usara o primeiro que encontrar',
                                                        ),
        'mkpcta_faturar_pelo_mkp': fields.boolean('Faturar pelo Mkp', help='Se marcado, o faturamento será pelo MKP, desmarcado, o faturamento será pelo ERP!'),
        'mkpcta_modelo_nf_series': fields.many2one("fiscal.modelo.nf.series", 'Modelo NF', ondelete="restrict", select=True),
        'mkpcta_producao': fields.boolean('Producao'),
        'mkpcta_parar_comunicacao': fields.boolean('Parar comunicacao', help="Se marcado esta conta noa envia nada para o MKP!"),
        'mkpcta_mkp_id_conta': fields.char('ID da conta no mkp', size=32),
        'mkpcta_mkp_cnpj_mkp': fields.char('CNPJ do mkp', size=15),
        'mkpcta_mkp_cnpj_financeiro': fields.char('CNPJ do intermediador financeiro', size=15, oldname='mkpcta_mkp_cnpj_finaceiro'),
        'mkpcta_fator_ajuste_pv': fields.float('Fator ajuste no preco', digits=(2,6)),
        'mkpcta_qtd_maxima': fields.integer('Qtd.Maxima', help="Quantidade maxima permitida por pedido."),
        'mkpcta_anuncio_ids': fields.one2many('mkp.anuncios', 'mkpanu_conta_id', 'Anuncios'),
    }
    _defaults = {
        'mkpcta_saldo_percentual': 100,
        'mkpcta_qtd_maxima': 0,
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'mkpcta_producao': True,
        'mkpcta_fator_ajuste_pv': 0.00,
    }