# -*- coding: utf-8 -*-
from openerp.osv import fields, osv
from openerp.osv.fields import float_round
from datetime import date, datetime
import base64
from SigUtils import calcDigito
from openerp.tools.translate import _
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from pynfe.nfe.manual_600.consrecinfe import ProcNFe

# Ajusta os valores de atributos ´many2one´
def adjust_m2o(val):
    """ Recupera apenas o ID dos atributos ´many2one´ """
    for k, v in val.items():
        if k.endswith('_id') and isinstance(v, (list, tuple)):
            val[k] = v[0]
        elif k.endswith('_ids'):
            for l in v:
                if not isinstance(l, (list, tuple)):
                    v[v.index(l)] = [4, l, False]

    return val


# TODO: Mover para ´import_nfe_data´
c_uf = [('11', 'Rondonia'), ('12', 'Acre'), ('13', 'Amazonas'), ('14', 'Roraima'), ('15', 'Para'),
        ('16', 'Amapa'), ('17', 'Tocantins'), ('21', 'Maranhao'), ('22', 'Piaui'), ('23', 'Ceara'),
        ('24', 'Rio Grande do Norte'), ('25', 'Paraiba'), ('26', 'Pernambuco'), ('27', 'Alagoas'),
        ('28', 'Sergipe'), ('29', 'Bahia'), ('31', 'Minas Gerais'), ('32', 'Espirito Santo'),
        ('33', 'Rio de Janeiro'), ('35', 'Sao Paulo'), ('41', 'Parana'), ('42', 'Santa Catarina'),
        ('43', 'Rio Grande do Sul'), ('50', 'Mato Grosso do Sul'), ('51', 'Mato Grosso'),
        ('52', 'Goias'), ('53', 'Distrito Federal')]
tp_emis = [('1', 'Normal'), ('2', 'Contingencia'), ('3', 'Contingencia SCAN'),
           ('4', 'Contingencia DPEC'), ('5', 'Contingencia FS-DA'), ('6', 'Contingência SVC-AN'),
           ('7', 'Contingência SVC-RS'), ('9', 'Contingência off-line da NFC-e')]
tp_imp = [('0', 'Sem geração de DANFE'), ('1', 'Retrato'), ('2', 'Paisagem'),
          ('3', 'DANFE Simplificado'), ('4', 'DANFE NFC-e'),
          ('5', 'DANFE NFC-e em mensagem eletrônica')]
proc_emi = [('0', 'emissao de NF-e com aplicativo do contribuinte'),
            ('1', 'emissao de NF-e avulsa pelo Fisco'),
            ('2', 'emissao de NF-e avulsa, pelo contribuinte com seu certificado digital, atraves '
                  'do site do Fisco'),
            ('3', 'emissao NF-e pelo contribuinte com aplicativo fornecido pelo Fisco')]
fin_nfe = [('1', 'NF-e normal'), ('2', 'NF-e complementar'), ('3', 'NF-e de ajuste'),
           ('4', 'Devolucao de Mercadoria')]
mod_frete = [('0', 'Por conta do emitente'), ('1', 'Por conta do destinatario/remetente'),
             ('2', 'Por conta de terceiros'), ('9', 'Sem Frete')]
tp_amb = [('1', 'Producao'), ('2', 'Homologacao')]
ind_pag = [('0', 'pagamento à vista'), ('1', 'pagamento à prazo'), ('2', 'outros')]
tp_nf = [('0', 'Entrada'), ('1', 'Saida')]
percent = lambda v, w: v * w / 100.  # Calculo do percentual


class mkp_import_nfe_osv(osv.TransientModel):
    def __init__(self, pool, cr):
        super(mkp_import_nfe_osv, self).__init__(pool, cr)
        self._erro, self._info = [], []  # Lista de erros e avisos

    def _get_local_entrada(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = False
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id.id
        estoque_parametro_ids = obj_estoque_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)
        if estoque_parametro_ids:
            browse_parametro = obj_estoque_parametro.browse(cr, uid, estoque_parametro_ids[0])
            local_entrada_id = browse_parametro.estpar_local_compra_padrao_id
            res = local_entrada_id.id if local_entrada_id else False
        return res

    _name = "mkp.import.nfe.osv"
    _description = "Importacao de Nota Fiscal Eletronica(mkp)"
    _rec_name = 'xml_file'
    _transient = False
    _columns = {
        'xml_file': fields.binary('Arquivo NF-e', filters='*.xml',
                                  filename='Nota Fiscal Eletronica'),
        'company_id': fields.many2one("res.company", 'Empresa', required=True),
        'natureza_id': fields.many2one("fiscal.cfop.natureza", 'Natureza',
                                       domain=[('tipo', '=', 'E')]),
        'local_id': fields.many2one("estoque.local", 'Local'),
        'prod_ids': fields.one2many('mkp.import.nfe.itens.osv', 'nf_id',
                                    'Produtos e Servicos da NF-e'),
        'log_err': fields.text('Erro'),
        'check_err': fields.boolean('Log', readonly=True),
        'xml_proc': fields.text('Xml'),
        'protocolo': fields.char('Numero do protocolo', size=15),
        'informacao_complementar': fields.text('Informacao complementar'),
        'informacao_complementar_fisco': fields.text('Informacao complementar Fisco'),
        'parcela_ids': fields.one2many('mkp.import.nfe.parcelas', 'nf_id', 'Parcelas'),
        'modfrete': fields.selection(mod_frete, 'Frete', size=1),
        'user_id': fields.many2one('res.users', 'Usuario', readonly=True),
        'log_info': fields.char('Log', size=50),

        # Tag ide
        'ide_id': fields.char('Id', size=60, readonly=True),
        'ide_cuf': fields.selection(c_uf, 'cUF', readonly=True),
        'ide_cnf': fields.char('cNF', size=8, readonly=True),
        'ide_natop': fields.char('natOp', size=60, readonly=True),
        'ide_indpag': fields.selection(ind_pag, 'indPag', readonly=True),
        'ide_mod': fields.char('mod', size=2, readonly=True),
        'ide_serie': fields.char('serie', size=3, readonly=True),
        'ide_nnf': fields.char('nNF', size=9, readonly=True),
        'ide_dhemi': fields.datetime('dhEmi', readonly=True),
        'ide_dhsaient': fields.datetime('dhSaiEnt'),
        'ide_tpnf': fields.selection(tp_nf, 'tpNf', readonly=True),
        'ide_cmunfg': fields.char('cMuniFG', size=7, readonly=True),
        'ide_tpimp': fields.selection(tp_imp, 'tpImp', readonly=True),
        'ide_tpemis': fields.selection(tp_emis, 'tpEmis', readonly=True),
        'ide_cdv': fields.char('cDV', size=1, readonly=True),
        'ide_tpamb': fields.selection(tp_amb, 'tpAmb', readonly=True),
        'ide_finnfe': fields.selection(fin_nfe, 'finNFe', readonly=True),
        'ide_procemi': fields.selection(proc_emi, 'procEmi', readonly=True),
        'ide_verproc': fields.char('verProc', size=10, readonly=True),

        # Tag emit
        'emit_ie': fields.char('IE', size=14, readonly=True),
        'emit_cnpj': fields.char('CNPJ', size=14, readonly=True),
        'emit_cpf': fields.char('CPF', size=11, readonly=True),
        'emit_crt': fields.char('CTR', size=5, readonly=True),
        'emit_pessoa_id': fields.many2one('adm.pessoa', 'xNome'),
        'emit_enderemit_xlgr': fields.char('xLgr', size=60, readonly=True),
        'emit_enderemit_xcpl': fields.char('xcpl', size=60, readonly=True),
        'emit_enderemit_nro': fields.char('nro', size=10, readonly=True),
        'emit_enderemit_xbairro': fields.char('xBairro', size=40, readonly=True),
        'emit_enderemit_cmun': fields.char('cMun', size=10, readonly=True),
        'emit_enderemit_xmun': fields.char('xMun', size=60, readonly=True),
        'emit_enderemit_uf': fields.char('UF', size=2, readonly=True),
        'emit_enderemit_cep': fields.char('CEP', size=8, readonly=True),
        'emit_enderemit_cpais': fields.char('cPais', size=4, readonly=True),
        'emit_enderemit_xpais': fields.char('xPais', size=30, readonly=True),
        'emit_enderemit_fone': fields.char('fone', size=14, readonly=True),
        'emit_im': fields.char('im', size=15, readonly=True),
        'emit_cnae': fields.char('cnae', size=7, readonly=True),

        # Tag dest
        'dest_ie': fields.char('IE', size=14, readonly=True),
        'dest_cnpj': fields.char('CNPJ', size=14, readonly=True),
        'dest_cpf': fields.char('CPF', size=11, readonly=True),
        'dest_crt': fields.char('CTR', size=5, readonly=True),
        'dest_pessoa_id': fields.many2one('adm.pessoa', 'xNome'),
        'dest_enderdest_xlgr': fields.char('xLgr', size=60, readonly=True),
        'dest_enderdest_xcpl': fields.char('xcpl', size=60, readonly=True),
        'dest_enderdest_nro': fields.char('nro', size=10, readonly=True),
        'dest_enderdest_xbairro': fields.char('xBairro', size=40, readonly=True),
        'dest_enderdest_cmun': fields.char('cMun', size=10, readonly=True),
        'dest_enderdest_xmun': fields.char('xMun', size=60, readonly=True),
        'dest_enderdest_uf': fields.char('UF', size=2, readonly=True),
        'dest_enderdest_cep': fields.char('CEP', size=8, readonly=True),
        'dest_enderdest_cpais': fields.char('cPais', size=4, readonly=True),
        'dest_enderdest_xpais': fields.char('xPais', size=30, readonly=True),
        'dest_enderdest_fone': fields.char('fone', size=14, readonly=True),
        'dest_im': fields.char('im', size=15, readonly=True),
        'dest_cnae': fields.char('cnae', size=7, readonly=True),

        # Tag total/ICMSTot
        'total_icmstot_vbc': fields.float('vBC', digits=(15, 2), help="Valor da Base de calculo"),
        'total_icmstot_vseg': fields.float('vSeg', digits=(15, 2), help="Valor do Seguro"),
        'total_icmstot_vst': fields.float('vST', digits=(15, 2), help="Valor da Substituicao"),
        'total_icmstot_vnf': fields.float('vNF', digits=(15, 2), help="Valor da Nota Fisal"),
        'total_icmstot_vprod': fields.float('vProd', digits=(15, 2), help="Total dos produtos"),
        'total_icmstot_vpis': fields.float('vPIS', digits=(15, 2), help="Valor do PIS"),
        'total_icmstot_vii': fields.float('vII', digits=(15, 2),
                                          help="Valor do Imposto de Importacao"),
        'total_icmstot_voutro': fields.float('vOutro', digits=(15, 2), help="Valor de outros"),
        'total_icmstot_vicms': fields.float('vICMS', digits=(15, 2), help="Valor do ICMS"),
        'total_icmstot_vfrete': fields.float('vFrete', digits=(15, 2), help="Valor do frete"),
        'total_icmstot_vdesc': fields.float('vDesc', digits=(15, 2), help="Valor do desconto"),
        'total_icmstot_vcofins': fields.float('vCOFINS', digits=(15, 2), help="Valor do COFINS"),
        'total_icmstot_vipi': fields.float('vIPI', digits=(15, 2), help="Valor do IPI"),
        'total_icmstot_vbcst': fields.float('vBCST', digits=(15, 2),
                                            help="Valor da Base de Calculo de Substituicao"),
        'total_icmstot_vfcpst': fields.float('vFCPST', digits=(15, 2),
                                             help='Valor do Fundo de Combate a Pobreza'),
        'total_icmstot_vfcpstret': fields.float('vFCPSTRet', digits=(15, 2),
                                                help='Valor do Fundo de Combate a Pobreza'),
        'total_icmstot_vicmsdeson': fields.float('vICMSDeson', digits=(15, 2),
                                                 help='Valor do ICMS desonerado'),

        # Tag total/ISSQNtot
        'total_issqntot_vserv': fields.float('vServ', digits=(15, 2), help='Valor dos servicos'),
        'total_issqntot_vbc': fields.float('vBC', digits=(15, 2), help='Valor da base de calculo'),
        'total_issqntot_viss': fields.float('vISS', digits=(15, 2), help='Valor do ISS'),
        'total_issqntot_vpis': fields.float('vPIS', digits=(15, 2), help='Valor do PIS'),
        'total_issqntot_vcofins': fields.float('vCOFINS', digits=(15, 2), help='Valor do COFINS'),
    }

    _defaults = {
        'check_err': lambda *a: False,
        'company_id': lambda s, cr, u, c: s.pool.get('res.users').browse(cr, u, u, c).company_id.id,
        'local_id': _get_local_entrada,
        'user_id': lambda obj, cr, uid, context: uid
    }

    def impnfe_onchange_natureza(self, cr, uid, ids, natureza_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if not natureza_id:
            return res
        cfop_movimento_tipo = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, natureza_id, context).cfop_id.movimento_tipo
        devolucao_local_id = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_local_entrada_devol_padrao_id', context=context).get(
            'estpar_local_entrada_devol_padrao_id')
        (cfop_movimento_tipo == 'D' and devolucao_local_id) and res.update(
            {'value': {'local_id': devolucao_local_id}})
        return res

    def _consultar_embalagem(self, cr, user, item, qtde, fornecedor, context):
        """
        Recupera a embalagem do item.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item
        :param qtde: a quantidade (do item) informada
        :param fornecedor: o ID do fornecedor (´destinatario_id´)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type qtde: float
        :type fornecedor: long
        :type context: dict
        :return: o ID da embalagem.
        :rtype: long
        """
        context = context and context.copy() or {}
        embalagem_id = self._get_embalagem_fornecedor(cr, user, item, fornecedor, context)
        embalagem_id = embalagem_id or self._get_embalagem_local(cr, user, item, qtde, context)

        return embalagem_id

    def _get_embalagem_local(self, cr, uid, produto_id, qtd_prod, context=None):
        """
        Retorna o ID da embalagem para o produto x local_entrada_padrao
        """
        context = context and context.copy() or {}
        res = False
        if not produto_id:
            return res
        estoque_parametro_ids = self.pool.get('estoque.parametro').search(
            cr, uid, [], context=context)
        if estoque_parametro_ids:
            local_padrao_id = self.pool.get('estoque.parametro').browse(
                cr, uid, estoque_parametro_ids[0]).estpar_local_compra_padrao_id
            if local_padrao_id:
                obj_local_locacao = self.pool.get('estoque.local.locacao')
                local_locacao_id = obj_local_locacao.search(
                    cr, uid, [('produto_id', '=', produto_id),
                              ('local_id', '=', local_padrao_id.id)], context=context)
                if local_locacao_id:
                    browse_local_locacao = obj_local_locacao.browse(cr, uid, local_locacao_id[0])
                    brw_embalagem = \
                        browse_local_locacao.estllc_produto_embalagem_id.embalagem_id if \
                            browse_local_locacao.estllc_produto_embalagem_id and \
                            browse_local_locacao.estllc_produto_embalagem_id.embalagem_id else None
                    embalagem_id = False
                    if brw_embalagem:
                        embalagem_id = brw_embalagem.id
                        if not brw_embalagem.unidade_id.fracao and \
                                int(float(qtd_prod)) != float(qtd_prod):
                            embalagem_id = False

                    return embalagem_id
        return res

    def _get_embalagem_fornecedor(self, cr, user, item, fornecedor, context=None):
        """
        Recupera o ID da embalagem utilizada pelo fornecedor.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param item: o ID do item
        :param fornecedor: o ID do fornecedor
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type item: long
        :type fornecedor: long
        :type context: dict
        :return: o ID da embalagem.
        :rtype: long
        """
        context = context and context.copy() or {}
        estoque_fornecedor_ids = self.pool.get('estoque.fornecedor').search(
            cr, user, [('fornecedor_id', '=', fornecedor), ('produto_id', '=', item)],
            context=context)
        estoque_fornecedor_dados = self.pool.get('estoque.fornecedor').read(
            cr, user, estoque_fornecedor_ids and estoque_fornecedor_ids[0], ['embalagem_id'],
            context, '_classic_write')

        return estoque_fornecedor_dados and estoque_fornecedor_dados.get('embalagem_id') or 0

    def verificar_existencia_nf(self, cr, uid, company_id, destinatario_id, numero, serie,
                                data_emissao, context=None):
        """
        Veifica se a nota fiscal jah existe.

        :param cr: o cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param company_id: o ID da empresa
        :param destinatario_id: o ID do destinatario
        :param numero: o numero da nota fiscal
        :param serie: a serie da nota fiscal
        :param data_emissao: a data de emissao da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type company_id: long
        :type destinatario_id: long
        :type numero: int
        :type serie: str
        :type data_emissao: str
        :type context: dict

        :return: dicionario com a descricao do erro quando a nota fiscal jah existe.
        :rtype: dict
        """

        context = context and context.copy() or {}
        vals = {}
        # partner_cnpj = self.pool.get('res.company').browse(
        #     cr, uid, company_id, context).rescon_pessoa_id.cpf_cnpj
        campos = {'rescon_pessoa_id': ['cpf_cnpj']}
        partner_cnpj = self.pool.get('adm.utils').admutl_read_browse(
            cr, uid, company_id, 'res.company', campos, context=context)[0].get(
            'rescon_pessoa_id').get('cpf_cnpj')
        if partner_cnpj:
            nf_lancada_args = [('numero_nf', '=', numero), ('serie_nf', '=', serie),
                               ('data_emissao', '>=', data_emissao[:10] + ' 00:00:00'),
                               ('data_emissao', '<=', data_emissao[:10] + ' 23:59:59'),
                               ('state', '!=', 'C'), ('destinatario_id', '=', destinatario_id),
                               ('tipo', '=', 'E')]
            nf_lancada_ids = self.pool.get('fiscal.nf.capa').search(
                cr, uid, nf_lancada_args, context=context)
            if nf_lancada_ids:
                log_err = _('A Nota fiscal de numero %s ja esta cadastrada.\n') % numero or ''
                vals['log_err'] = log_err
        return vals

    def create(self, cr, uid, vals, context=None):
        _id = super(mkp_import_nfe_osv, self).create(cr, uid, vals, context)
        if not vals.get('xml_file'):
            return _id
        self.upload_xml(cr, uid, _id, context)

        return _id

    def button_upload_xml(self, cr, user, ids, context=None):
        """
        Realiza a carga do arquivo XML

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a carga do arquivo XML foi realizada com sucesso
        :rtype: bool
        """

        return True

    def upload_xml(self, cr, user, ids, context=None):
        """
        Realiza a carga (upload) do arquivo XML atribuindo os valores das ´tag´ aos respectivos
        valores da nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a carga do arquivo XML foi realizada com sucesso
        :rtype: bool
        """
        import xml.etree.ElementTree as ET
        context = context and context.copy() or {}

        # O contexto 'nao_pesquisa_kit' impede que colque kit na importação da nota
        context.update({'carregando_xml': True, 'entrada_terceiros': True,
                        'nao_pesquisa_kit': True})
        vals = self.read(
            cr, user, ids[0] if isinstance(ids, list) else ids, [], context, '_classic_write')

        # Carrega o XML
        # FIX: Deve ser validado pelo arquivo de esquema XSD -- procNFe_v4.00.xsd
        vals['xml_proc'] = base64.b64decode(vals['xml_file'] or '')
        try:
            xml_nfe = ET.fromstring(vals['xml_proc'])
        except Exception as err:
            raise osv.except_osv(_('Atencao!'),
                                 _('O arquivo NF-e nao foi informado ou nao e um arquivo valido.'))

        if not vals.get('natureza_id'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A Natureza da Operacao deve ser informada.'))
        if not vals.get('local_id'):
            raise osv.except_osv(_('Atencao!'),
                                 _('O Local nao foi informado.'))

        ns = {'nfe': 'http://www.portalfiscal.inf.br/nfe'}

        # TODO: Validar o XML pelo arquivo .XSD
        if xml_nfe.tag[len(ns['nfe']) + 2:] != 'nfeProc':  # root = nfeProc
            xml_nfe = xml_nfe.find('.//nfe:nfeProc', ns) or {}

        # Tag ´<infNFe>´ e ´<ide>´ - Identificacao da NF-e
        infnfe = xml_nfe.find('.//nfe:infNFe', ns)
        ide = infnfe.find('nfe:ide', ns)
        vals['ide_id'] = infnfe.get('Id')
        vals.update(dict([('ide_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                          for e in ide.getchildren()]))

        # Tag ´<protNFe>´ e ´<infProt>´ - Identificacao do protocolo
        protnfe = xml_nfe.find('nfe:protNFe', ns)
        infprot = protnfe.find('nfe:infProt', ns)

        # Tag ´<cobr> - Cobrancas (parcelas de pagamento)
        cobr = infnfe.find('.//nfe:cobr', ns)

        # Tag ´<total>´ - Totais da nota fiscal
        total = infnfe.find('nfe:total', ns)
        icmstot = total.find('nfe:ICMSTot', ns)  # Produtos
        vals.update(dict([('total_icmstot_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                          for e in icmstot.getchildren()]))

        creditos = ['total_icmstot_vprod', 'total_icmstot_vst', 'total_icmstot_vfcpst',
                    'total_icmstot_vfrete', 'total_icmstot_vseg', 'total_icmstot_voutro',
                    'total_icmstot_vii', 'total_icmstot_vipi', 'total_icmstot_vipidevol']
        debitos = ['total_icmstot_vdesc']
        valor_total = 0
        for k, v in vals.items():
            if 'total_icmstot' in k:
                valor_total += float(v) if k in creditos else 0
                valor_total -= float(v) if k in debitos else 0
        desonerar = True if round(valor_total, 2) != round(float(vals['total_icmstot_vnf']), 2) else False
        issqntot = total.find('nfe:ISSQNtot', ns)  # Servicos
        if issqntot:
            vals.update(dict([('total_issqntot_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                              for e in issqntot.getchildren()]))

        vals['protocolo'] = infprot.find('nfe:nProt', ns).text

        # Data e hora de emissao
        emissao = ide.find('nfe:dEmi', ns)  # <-- Nao deveria aceitar v2.00 mas...
        emissao = isinstance(emissao, ET.Element) and emissao.text or ide.find('nfe:dhEmi', ns)
        emissao = isinstance(emissao, ET.Element) and emissao.text or emissao
        emissao = emissao and parse(emissao)
        try:
            tz = 24 - (emissao.tzinfo._offset.seconds / 3600)
        except:
            tz = 3
        emissao += relativedelta(hours=tz)
        vals['ide_dhemi'] = emissao.strftime('%Y-%m-%d %H:%M:%S%z')
        if 'ide_demi' in vals:
            del vals['ide_demi']

        # Condicoes de pagamento 0-Pagamento "a vista", 1-Pagamento a prazo, 2-Outros
        if vals.get('ide_indpag') == '0':
            vals['parcela_ids'] = [[0, False, {'valor': vals['total_icmstot_vnf'],
                                               'vencimento': vals['ide_dhemi'][:10]}]]
        elif cobr:
            # atencao, ao modificar este ponto verificar e a(s) parcela(s) da nota foram importadas
            # corretamente
            vals['parcela_ids'] = [
                [0, False, {'valor': e.find('nfe:vDup', ns) not in (False, None) and float(
                    e.find('nfe:vDup', ns).text),
                            'vencimento': e.find('nfe:dVenc', ns) not in (False, None) and e.find(
                                'nfe:dVenc', ns).text or False}]
                for e in (cobr.findall('nfe:dup', ns) if cobr is not None else [])]

        # Tag ´<emit>´ - Identificacao do emitente da nota fiscal
        vals.update(dict([('emit_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                          for e in infnfe.find('nfe:emit', ns).getchildren()]))
        # Endereco do emitente
        vals.update(dict([('emit_enderemit_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                          for e in infnfe.find('nfe:emit', ns).find('nfe:enderEmit',
                                                                    ns).getchildren()]))

        # Tag ´<dest>´ - Identificacao do destinatario da nota fiscal
        vals.update(dict([('dest_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                          for e in infnfe.find('nfe:dest', ns).getchildren()]))
        # Endereco do destinatario
        vals.update(dict([('dest_enderdest_%s' % e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                          for e in infnfe.find('nfe:dest', ns).find('nfe:enderDest',
                                                                    ns).getchildren()]))

        # Dados da pessoa
        if vals.get('dest_cnpj') and vals.get('dest_xnome'):
            cnae_ids = self.pool.get('adm.cnae').search(
                cr, user, [('admcna_codigo', '=', vals.get('dest_cnae'))], context=context)
            if cnae_ids:
                vals_pessoa = {'admpes_jur_cnae_id': cnae_ids[0]}
            else:
                vals_pessoa = {}
            cpf_cnpj = vals.get('dest_cnpj')
            vals_pessoa.update({'cpf_cnpj': vals.get('dest_cnpj'), 'pessoa_tipo': 'J',
                                'name': vals.get('dest_xnome'),
                                'fantasia_apelido': vals.get('dest_xfant'),
                                'admpes_jur_ie': vals.get('dest_ie')})
        elif vals.get('dest_cpf') and vals.get('dest_xnome'):
            cpf_cnpj = vals.get('dest_cpf')
            vals_pessoa = {'cpf_cnpj': vals.get('dest_cpf'), 'pessoa_tipo': 'F',
                           'name': vals.get('dest_xnome')}
        else:
            cpf_cnpj = ''
            vals_pessoa = {'name': vals.get('dest_xnome'), 'pessoa_tipo': 'J'}

        if cpf_cnpj:
            pessoa_ids = self.pool.get('adm.pessoa').search(
                cr, user, [('cpf_cnpj', '=', cpf_cnpj[:14])], context=context)
        else:
            args = [('name', '=', vals.get('dest_xnome'))]
            pessoa_ids = self.pool.get('adm.pessoa').search(
                cr, user, args, context=context)
        log_err = ''
        if not pessoa_ids:  # Se nao existir a pessoa, cadastra
            pessoa_id = self.pool.get('adm.pessoa').create(
                cr, user, vals_pessoa, context=context)
            log_err += _('Pessoa %d - %s/%s cadastrada com sucesso.\n') % (
                pessoa_id, vals['dest_cnpj'], vals['dest_xnome'])
        else:
            pessoa_id = pessoa_ids[0]
        fornecedor_ids = self.pool.get('financeiro.fornecedor').search(
            cr, user, [('pessoa_id', '=', pessoa_id), ('company_id', '=', vals['company_id'])],
            context={'active_test': False})
        fornecedor_id = fornecedor_ids and fornecedor_ids[0]

        # Verifica se o fornecedor estah inativo e se o parametro permite cadastro automatico
        if fornecedor_id:
            if not self.pool.get('financeiro.fornecedor').read(
                    cr, user, fornecedor_id, ['active'], {'active_test': False}).get('active'):
                if self.pool.get('adm.utils').admutl_get_parametros(
                        cr, user, 'financeiro', []).get('cadastro_automatico_fornecedor'):
                    self.pool.get('financeiro.fornecedor').write(
                        cr, user, fornecedor_id, {'active': True}, context={'active_test': False})
                else:  # Se estiver inativo e o parametro nao permitir cadastro automatico
                    raise osv.except_osv(_('Atencao!'),
                                         _('Fornecedor %s (CNPJ %s) encontra-se inativo.') % (
                                             vals.get('dest_xnome'), vals.get('dest_cnpj')))
        else:
            if self.pool.get('adm.utils').admutl_get_parametros(
                    cr, user, 'financeiro', []).get('cadastro_automatico_fornecedor'):
                fornecedor_id = self.pool.get('financeiro.fornecedor').create(
                    cr, user, {'pessoa_id': pessoa_id, 'company_id': vals.get('company_id')},
                    context)
            else:  # Se fornecedor nao estah cadastrado e nao permitir cadastro automatico
                raise osv.except_osv(_('Atencao!'),
                                     _('Fornecedor %s (CNPJ %s) nao esta cadastrado.') % (
                                         vals.get('dest_xnome'), vals.get('dest_cnpj')))

        endereco_id = False
        nome_pais = vals.get('dest_enderdest_xpais') or 'Brasil'
        if not vals.get('dest_enderdest_xpais') and vals.get('dest_enderdest_cpais'):
            obj_adm_pais = self.pool.get('adm.pais')
            codigo_pais = vals['dest_enderdest_cpais']
            pais_id = obj_adm_pais.search(
                cr, user, [('codigo_bc', '=', codigo_pais)], context=context)
            if pais_id:
                nome_pais = obj_adm_pais.browse(cr, user, pais_id[0], context).name
        vals['dest_enderdest_xpais'] = nome_pais
        if pessoa_id:
            try:
                endereco_id = self.pool.get('adm.pessoa.endereco').str2address(
                    cr, user, pessoa_id, nome_pais, vals.get('dest_enderdest_uf'),
                    vals.get('dest_enderdest_xmun'), vals.get('dest_enderdest_xbairro'),
                    vals.get('dest_enderdest_xlgr'), vals.get('dest_enderdest_cep'),
                    vals.get('dest_enderdest_nro'), vals.get('dest_enderdest_xcpl'),
                    IntCodigoIBGE=int(vals.get('dest_enderdest_cmun')), context=context)
            except:
                log_err += _('Erro ao tentar cadastrar endereco.\n')
                cr.rollback()

        if endereco_id:
            # Atualiza o endereco da NF-e no cadastro de pessoa
            self.pool.get('adm.pessoa').write(cr, user, pessoa_id, {'endereco_nf_id': endereco_id},
                                              context)

            # Verificacao de CFOP fora do estado
            self.pool.get('fiscal.nf.capa').verifica_cfop_fora_estado(
                cr, user, vals['company_id'], vals['natureza_id'], endereco_id, 'E', context=context)

        vals['dest_pessoa_id'] = pessoa_id
        if pessoa_id:
            dh_emissao = vals.get('ide_dhemi')
            pesquisa_nf_existe = self.verificar_existencia_nf(
                cr, user, vals['company_id'], pessoa_id, vals.get('ide_nnf'), vals.get('ide_serie'),
                dh_emissao)
            if pesquisa_nf_existe.get('log_err'):
                raise osv.except_osv(_('Atencao!'), pesquisa_nf_existe['log_err'])

        # Tag ´<infCpl>´ - Informacoes adicionais/complementares
        infcpl = infnfe.find('.//nfe:infCpl', ns)
        if infcpl is not None:
            vals['informacao_complementar'] = infcpl.text

        # Tag ´<infAdFisco>´ - Informacoes adicionais/complementares fisco
        infAdFisco = infnfe.find('.//nfe:infAdFisco', ns)
        if infAdFisco is not None:
            vals['informacao_complementar_fisco'] = infAdFisco.text

        # Tag ´<transp>´ - Frete
        mod_frete = infnfe.find('.//nfe:modFrete', ns).text
        vals['modfrete'] = {'3': '0', '4': '1'}.get(mod_frete, mod_frete) or '9'

        # TODO: Transformar em metodo
        # Tag ´<det>´ - Itens da NF
        vals['prod_ids'] = []
        # cont_itens = 1
        tot_serv = 0
        for det in infnfe.findall('nfe:det', ns):
            # Tag ´<prod>´ - O item (produto ou servico)
            vals_itens = dict([(e.tag[len(ns['nfe']) + 2:].lower(), e.text)
                               for e in det.find('nfe:prod', ns).getchildren()])
            item_id = grade_id = False
            cfop_ids = self.pool.get('fiscal.cfop').search(cr, user,
                                                           [('codigo', '=', vals_itens.get('cfop') or '')],
                                                           context=context)
            if cfop_ids:
                prod_serv_tipo = self.pool.get('fiscal.cfop').read(cr, user, cfop_ids[0],
                                                                   ['produto_servico_tipo'], context).get(
                    'produto_servico_tipo')
                if prod_serv_tipo == 'S':
                    tot_serv += float(vals_itens.get('vprod') or vals_itens.get('vserv') or 0)

            # Recupera o item do estoque
            cb_lido = vals_itens.get('cean')
            if cb_lido == 'SEM GTIN':
                cb_lido = ''
            if (cb_lido or '').isdigit():
                cb_lido = str(int(vals_itens.get('cean') or '0') or '')
            codigo_barra_ids = self.pool.get('automacao.codigo.barra').search(cr, user,
                                                                              [('codigo_barra', '=', cb_lido)],
                                                                              context=context)
            codigo_fornecedor_ids = self.pool.get('estoque.produto.codigo.fornecedor').search(
                cr, user, [('codigo_fornecedor', '=', vals_itens.get('cprod')),
                           ('fornecedor_id', '=', fornecedor_id)], context=context)
            if codigo_barra_ids:
                automacao_dados = self.pool.get('automacao.codigo.barra').read(
                    cr, user, codigo_barra_ids[0], ['produto_id', 'grade_id'], context,
                    '_classic_write')
                item_id = automacao_dados.get('produto_id')
                grade_id = automacao_dados.get('grade_id')

                # Se eh KIT desconsidera-o pelo codigo de barras e assume o codigo do fornecedor
                if self.pool.get('estoque.produto').read(
                        cr, user, item_id, ['kit_de_produtos'], context).get('kit_de_produtos'):
                    item_id = False
            else:
                if not cb_lido:
                    vals_itens['cean'] = None

            if codigo_fornecedor_ids and not item_id:
                fornecedor_dados = self.pool.get('estoque.produto.codigo.fornecedor').browse(
                    cr, user, codigo_fornecedor_ids[0], context)
                item_id = fornecedor_dados.produto_id.id if fornecedor_dados.produto_id.active \
                    else False
                grade_id = fornecedor_dados.grade_id.id if \
                    fornecedor_dados.grade_id.active and item_id else False
            else:
                # Recupera o item do XML. O item deve ser cadastrado
                vals_itens.update({'prod_nao_cadastrado': not item_id, 'cadastrar': not item_id,
                                   'cadastrar_tributacao_entrada': not item_id})

            categoria_id = item_id and self.pool.get('estoque.produto').read(
                cr, user, item_id, ['categoria_id'], context, '_classic_write').get('categoria_id')
            vals_itens.update({'produto_id': item_id, 'grade_id': grade_id, 'encargo_ids': [],
                               'categoria_id': categoria_id,
                               'codigo_alfa_numerico': vals_itens.get('cprod')})
            vbc = float(vals_itens.get('vprod', 0)) + float(vals_itens.get('voutro', 0)) + \
                  float(vals_itens.get('vfrete', 0)) - float(vals_itens.get('vdesc', 0))
            natureza_id = vals.get('natureza_id')

            # Tag ´<imposto>´ - Os impostos (encargos) do item
            encargo_dados = {}
            encargo_anterior = None
            for imposto in det.find('nfe:imposto', ns).getchildren():  # Consolidacao dos encargos
                encargo = imposto.tag[len(ns['nfe']) + 2:]
                if encargo.startswith('PIS'):  # Pode ser PISNT, PISST
                    encargo = 'PIS'
                elif encargo.startswith('COFINS'):  # Pode ser COFINSNT, COFINSST
                    encargo = 'COFINS'
                elif encargo.startswith('ICMS'):  # Pode ser ICMSNT, ICMSST
                    encargo = 'ICMS'
                elif encargo.startswith('IPI'):  # Pode ser IPINT, IPIST
                    encargo = 'IPI'
                else:
                    pass  # Pode ser um encargo nao cadastrado
                encargo_dados[encargo] = dict([(e.tag[len(ns['nfe']) + 2:].lower(),
                                                e.text) for e in imposto.iter()])

            for encargo in encargo_dados:
                encargo_ids = self.pool.get('fiscal.encargo').search(
                    cr, user, [('sigla', '=', encargo)], context=context)
                if not encargo_ids:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Encargo %s nao cadastrado!' % encargo))

                encargo_id = encargo_ids[0]
                encargos = {
                    'vr_base_calculo': encargo_dados.get(encargo).get('vbc') or 0,
                    'cst': encargo_dados.get(encargo).get('cst') or False,
                    'csosn': encargo_dados.get(encargo).get('csosn') or False,
                    'encargo_id': encargo_id,
                }
                if encargo == 'ICMS':
                    encargos['aliquota'] = encargo_dados.get(encargo).get('picms') or 0
                    encargos['vr_encargo'] = encargo_dados.get(encargo).get('vicms') or 0
                    if not encargo_dados.get(encargo).get('cst') and encargo_dados.get(encargo).get('picms'):
                        encargos['cst'] = '00'
                if encargo == 'II':
                    encargos['aliquota'] = False
                    encargos['vr_encargo'] = encargo_dados.get(encargo).get('vii') or 0
                if encargo == 'IPI':
                    encargos['aliquota'] = encargo_dados.get(encargo).get('pipi') or 0
                    encargos['vr_encargo'] = encargo_dados.get(encargo).get('vipi') or 0
                if encargo == 'COFINS':
                    encargos['aliquota'] = encargo_dados.get(encargo).get('pcofins') or 0
                    encargos['vr_encargo'] = encargo_dados.get(encargo).get('vcofins') or 0
                if encargo == 'PIS':
                    encargos['aliquota'] = encargo_dados.get(encargo).get('ppis') or 0
                    encargos['vr_encargo'] = encargo_dados.get(encargo).get('vpis') or 0
                cst_id = False
                if encargos.get('cst'):
                    cst_ids = self.pool.get('fiscal.cst').search(
                        cr, user, [('codigo', '=', encargos.get('cst')), ('encargo_id', '=', encargo_id)],
                        context=context)
                    if cst_ids:
                        cst_id = cst_ids[0]
                        encargos['cst_id'] = cst_id
                args = [('encargo_id', '=', encargo_id)]
                if cst_id:
                    args.append(('cst_id', '=', cst_id))
                if encargos.get('aliquota'):
                    args.append(('aliquota', '=', encargos.get('aliquota')))
                tributacao_ids = self.pool.get('fiscal.tributacao').search(cr, user, args, context=context)
                if tributacao_ids:
                    tributacao_id = tributacao_ids[0]
                    encargos['tributacao_id'] = tributacao_id
                else:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Tributacao nao cadastrada para o encargo:%s Trib:%s.' % (encargo, args)))
                vals_itens['encargo_ids'].append([0, False, encargos])

            # Os dados do item ´<prod>´ e dos encargos ´<imposto>´ jah foram recuperados do XML
            vals_itens['local_id'] = vals['local_id']

            if item_id:
                vals_itens['embalagem_id'] = self._consultar_embalagem(
                    cr, user, item_id, vals_itens.get('qcom'), fornecedor_id, context)
                brw_estoque_produto = self.pool.get('estoque.produto').browse(
                    cr, user, item_id, context)
                vals_itens.update({'pis_nat_rec': brw_estoque_produto.estpro_pis_nat_rec,
                                   'cofins_nat_rec': brw_estoque_produto.estpro_cofins_nat_rec})

            vals_itens['item_editado'] = self.pool.get('mkp.import.nfe.itens.osv').validar_itens(
                cr, user, vals_itens['encargo_ids'], vals_itens.get('embalagem_id', False), context)
            vals_itens['sequencia'] = int(det.attrib.get('nItem'))
            # cont_itens += 1
            vals['prod_ids'].append([0, False, vals_itens])

        # no caso onde vier servico e nao tiver issqn, esta ficando sem valor o total de
        # servicos
        if tot_serv and not vals.get('total_issqntot_vserv'):
            vals['total_issqntot_vserv'] = tot_serv

        # Exclui ´tag´ nao utilizada
        map(lambda m: m in vals and vals.pop(m),
            ['emit_xfant', 'emit_xnome', 'emit_iest', 'ide_xjust', 'ide_dhcont'])

        # Ultimas atribuicoes: data e hora de entrada/saida da nota fiscal e ´´log´´ de erro
        vals['ide_dhsaient'] = datetime.now()
        vals['log_err'] = log_err

        return self.write(cr, user, ids, vals, context)

    def _impnfe_get_cfop_entrada(self, cr, user, natureza, cst, context=None):
        """
        Determina o codigo de CFOP de entrada.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param natureza: o ID da natureza da operacao
        :param cst: o numero do CST
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type natureza: long
        :type cst: str
        :type context: dict
        :return: o CFOP de entrada.
        :rtype: str
        """
        context = context and context.copy() or {}
        cfop = self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, natureza, context).cfop_id.codigo
        cfop_entrada = cfop

        # TODO: Elaborar uma classe de correlacao assim como eh feito para CST
        if cfop not in ('1910', '2910', '1911', '2911'):
            if cst in ('10', '30', '60', '70'):
                if cfop[1:] == '101':
                    cfop_entrada = cfop[0] + '401'
                elif cfop[1:] in ['102', '405']:
                    cfop_entrada = cfop[0] + '403'
                elif cfop[1:] == '551':
                    cfop_entrada = cfop[0] + '406'
                elif cfop[1:] == '556':
                    cfop_entrada = cfop[0] + '407'

        return cfop_entrada


    def bt_import_nfe(self, cr, uid, ids, context=None):
        """
        BOTAO DE IMPORTAÇÃO DA NFE
        """
        context = context and context.copy() or {}
        brw_nfe = self.browse(cr, uid, ids[0], context)  # Dados do formulario de importacao
        self._info, self._erro = [], []

        def _infnfe_verifica_erro_critico():
            """
            Verifica se existe algum erro que impede a importacao da nota fiscal eletronica.

            :return: verdadeiro se nao houver impedimento para importacao da NF-e
            :rtype: bool
            :raise: mensagem de erro do impedimento de importacao da NF-e.
            """
            title = _('Atencao!')

            # Erros criticos a serem verificados
            try:
                if not brw_nfe.company_id:
                    erro = _('Nao foi possivel localizar a empresa para realizar a importacao.')
                elif not brw_nfe.prod_ids:
                    erro = _('A importacao da nota fiscal nao pode ser realizada. Nao existem itens.')
                elif not self.pool.get('fiscal.cfop.natureza').browse(
                        cr, uid, brw_nfe.natureza_id.id, context).fisnat_operacao_id:
                    erro = _('Cadastre a operacao para a natureza da operacao informada.')
                else:
                    erro = None
                if erro:
                    raise osv.except_osv(title, erro)
            except:
                raise osv.except_osv(title, _('Nao foi possivel importar o XML da nota fiscal.'))

            return True

        def _infnfe_log_erro():
            """
            Verifica se existe algum erro que impede a importacao da nota fiscal eletronica.

            :return: verdadeiro se nao houver impedimento para importacao da NF-e
            :rtype: bool
            """

            # Cadastra o fornecedor caso nao encontre na tabela de fornecedores
            if not self.pool.get('financeiro.fornecedor').search(
                    cr, uid, [('pessoa_id', '=', brw_nfe.dest_pessoa_id.id),
                              ('company_id', '=', brw_nfe.company_id.id)],
                    context=context):
                self.pool.get('financeiro.fornecedor').create(
                    cr, uid,
                    {'pessoa_id': brw_nfe.dest_pessoa_id.id, 'company_id': brw_nfe.company_id.id}, context)
                self._info.append(_('%s cadastrado como fornecedor.') % brw_nfe.dest_pessoa_id.name)

            # Verifica se a nota fiscal esta cadastrada
            pesquisa_nf_existe = self.verificar_existencia_nf(
                cr, uid, brw_nfe.company_id.id, brw_nfe.dest_pessoa_id.id, brw_nfe.ide_nnf,
                brw_nfe.ide_serie, brw_nfe.ide_dhemi, context)
            if pesquisa_nf_existe.get('log_err'):
                raise osv.except_osv(_('Atencao!'), pesquisa_nf_existe['log_err'])

            # Somente funcionarios podem realizar a importacao de notas fiscais
            if not self.pool.get('rh.funcionario').get_id_func(cr, uid):
                self._erro.append(_('Usuario nao esta cadastrado como funcionario.'))

            # Verifica itens sem embalagem
            item_sem_embalagem = filter(lambda f: not f.embalagem_id, brw_nfe.prod_ids)[:5]
            if item_sem_embalagem:
                raise osv.except_osv(_('Atencao!'),
                                     _('Itens sem embalagem: %s') % ''.join(map(
                                         lambda m: '\n* %s - %s' % ((m.cean or m.cprod), m.xprod),
                                         item_sem_embalagem)))

            # Verifica itens sem CFOP
            item_sem_cfop = filter(lambda f: not f.cfop, brw_nfe.prod_ids)[:5]
            if item_sem_cfop:
                raise osv.except_osv(_('Atencao!'),
                                     _('Itens sem CFOP: %s') % ''.join(map(
                                         lambda m: '\n* %s - %s' % ((m.cean or m.cprod), m.xprod),
                                         item_sem_cfop)))

            # Verifica itens sem local informado
            item_sem_local = filter(lambda f: not f.local_id, brw_nfe.prod_ids)[:5]
            if item_sem_local:
                raise osv.except_osv(_('Atencao!'),
                                     _('Itens sem local informado: %s') % ''.join(map(
                                         lambda m: '\n* %s - %s' % ((m.cean or m.cprod), m.xprod),
                                         item_sem_local)))

            # Verifica se o tipo de movimento do CST eh de saida (deve ser entrada ou ambos)
            # for item in brw_nfe.prod_ids:
            #     for brw_encargo in item.encargo_ids:
            #         if not self.pool.get('fiscal.cst').fiscst_check_tipo_movimento(
            #                 cr, uid, brw_encargo.cst, brw_encargo.encargo_id.id, ['A', 'E'],
            #                 context):
            #             campos = {'seq': item.sequencia, 'codigo': item.cean or item.cprod,
            #                       'nome': item.xprod, 'encargo': brw_encargo.encargo_id.sigla}
            #             raise osv.except_osv(_('Atencao!'),
            #                                  _('O item %(seq)s. %(codigo)s - %(nome)s contem CST '
            #                                    'de saida para o encargo %(encargo)s') % campos)

            # Verifica se CST de PIS e COFINS sao diferentes (devem ser iguais)
            for item in brw_nfe.prod_ids:
                encargos = filter(
                    lambda f: f.encargo_id.sigla in ('PIS', 'COFINS'), item.encargo_ids)
                if encargos.__len__() == 2:  # Existe PIS e COFINS
                    if encargos[0].cst != encargos[1].cst:
                        campos = {'seq': item.sequencia, 'codigo': item.cean or item.cprod,
                                  'nome': item.xprod}
                        raise osv.except_osv(_('Atencao!'),
                                             _('O CST de PIS difere do COFINS para o item %(seq)s. '
                                               '%(codigo)s - %(nome)s') % campos)

            return True if not self._erro else False

        context['importacao_nfe'] = True
        context['import_nfe'] = True
        context['nao_pesquisa_kit'] = True
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        # Verifica erros criticos
        _infnfe_verifica_erro_critico()

        # Aviso (log) de importacao. Pode impedir a importacao da nota fiscal eletronica
        _infnfe_log_erro()

        # Endereco da NF-e no Cadastro de Pessoa
        endereco_id = self.pool.get('adm.pessoa.endereco').str2address(
            cr, uid, brw_nfe.dest_pessoa_id.id, brw_nfe.dest_enderdest_xpais,
            brw_nfe.dest_enderdest_uf, brw_nfe.dest_enderdest_xmun, brw_nfe.dest_enderdest_xbairro,
            brw_nfe.dest_enderdest_xlgr, brw_nfe.dest_enderdest_cep, brw_nfe.dest_enderdest_nro,
            brw_nfe.dest_enderdest_xcpl, IntCodigoIBGE=int(brw_nfe.dest_enderdest_cmun or 0),
            context=context)
        self.pool.get('adm.pessoa').write(
            cr, uid, brw_nfe.dest_pessoa_id.id, {'endereco_nf_id': endereco_id}, context)

        # ID do modelo da NF-e
        ide_mod_nf = self.pool.get('fiscal.modelo.nf').search(cr, uid, [
            ('fismnf_codigo', '=', brw_nfe.ide_mod),
            ('active', '=', True)], context=context)
        ide_mod = ide_mod_nf[0] if ide_mod_nf else False

        # Itens da nota fiscal
        itens = []
        modulo_serie_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'estoqueserie', context)
        fornecedor_id = self.pool.get('financeiro.fornecedor').search(
            cr, uid, [('pessoa_id', '=', brw_nfe.dest_pessoa_id.id),
                      ('company_id', '=', brw_nfe.company_id.id)], context=context)[0]

        # TODO: Transformar em metodo
        cfop_outros = True  # Se o tipo do item no CFOP for outros, o total da nota pode ser zero
        total = 0  # Total da nota
        total_despesa = total_seguro = total_frete = total_base_icms_st = 0  # Totais na capa
        seq_itens = 1
        produto_ids = []
        msg = []
        for i in brw_nfe.prod_ids:
            if i.produto_id:
                produto_ids.append(i.produto_id.id)
                if i.grade_id and i.grade_id.produto_id.id != i.produto_id.id:
                    msg.append('Produto: %s - %s - Grade: %s' % (i.sequencia,
                                                                 i.produto_id.name,
                                                                 i.grade_id.descricao))
        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('Os produtos a seguir estao com a grade incorreta: \n%s') %
                                 '\n'.join(msg))

        sem_kit_produto_ids = []
        if produto_ids:  # este search retira os kits pelo contexto 'nao_pesquisa_kit'
            sem_kit_produto_ids = self.pool.get('estoque.produto').search(cr, uid,
                                                                          [('id', 'in', produto_ids)], context=context)
        msg_inativo = []
        msg_kit = []
        msg_barra = []
        msg_fornecedor = []
        itens_serao_criados = str()
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        for brw_item in sorted(brw_nfe.prod_ids, key=lambda x: x.sequencia):
            produto_id = brw_item.produto_id and brw_item.produto_id.id  # Item correlacionado
            grade_id = brw_item.grade_id and brw_item.grade_id.id  # Grade do item correlacionado
            if produto_id and not grade_id and brw_item.usar_grade:
                grade_ids = obj_estoque_produto_grade.search(
                    cr, uid, [('produto_id', '=', produto_id), ('grade', '=', brw_item.grade)],
                    context=context)
                if grade_ids:
                    grade_id = grade_ids[0]
                else:
                    grade_id = obj_estoque_produto_grade.create(
                        cr, uid, {'produto_id': produto_id, 'grade': brw_item.grade,
                                  'descricao': brw_item.grade}, context)

            # Nao permite item sem grade
            if not brw_item.prod_nao_cadastrado and not grade_id:
                raise osv.except_osv(_('Atencao!'),
                                     _('O item %s - %s encontra-se sem a grade. Favor '
                                       'informar.') % (brw_item.sequencia,
                                                       brw_item.produto_id.name or brw_item.xprod))
            embalagem_id = brw_item.embalagem_id.id
            if produto_id and produto_id not in sem_kit_produto_ids:
                context_active = context.copy()
                context_active['active_test'] = False
                if self.pool.get('estoque.produto').search(cr, uid, [('id', '=', produto_id),
                                                                     ('active', '=', False)], context=context_active):
                    msg_inativo.append(_('Sequencia: %s') % brw_item.sequencia)
                    continue
                msg_kit.append(_('Sequencia: %s') % brw_item.sequencia)
                continue

            # Se existir codigo de barras ou codigo de fornecedor no xml e o item nao foi
            # correlacionado, recupera o item pelo codigo de barras ou codigo do fornecedor
            if not produto_id and brw_item.cean:
                # Procura item pelo codigo de barras
                produto_id, grade_id = self._impnfe_get_item_codigo_barra(cr, uid, brw_item.cean,
                                                                          context)
                if produto_id:
                    if produto_id not in self.pool.get('estoque.produto').search(
                            cr, uid, [('id', '=', produto_id)], context=context):
                        msg_barra.append(_('Sequencia: %s') % brw_item.sequencia)
                        continue

            if not produto_id and brw_item.cprod:
                # Procura item pelo codigo de fornecedor
                produto_id, grade_id, self._impnfe_get_item_codigo_fornecedor(
                    cr, uid, brw_item.cprod, fornecedor_id, context)
                if produto_id:
                    if produto_id not in self.pool.get('estoque.produto').search(
                            cr, uid, [('id', '=', produto_id)], context=context):
                        msg_fornecedor.append(_('Sequencia: %s') % brw_item.sequencia)
                        continue

            if not produto_id:
                if context.get('impnfe_autorizada_criacao_itens'):
                    # Cadastra o item se nao foi encontrado ou correlacionado
                    produto_id, grade_id = self._impnfe_item_create(
                        cr, uid, brw_nfe.id, brw_item.id, context)
                else:
                    itens_serao_criados += '%s - %s\n' % (brw_item.sequencia, brw_item.xprod)
                    continue

            # Cadastro do item para o fornecedor (codigo do item no fornecedor)
            if brw_item.cprod and not self.pool.get('estoque.produto.codigo.fornecedor').search(
                    cr, uid, [('fornecedor_id', '=', fornecedor_id),
                              ('codigo_fornecedor', '=', brw_item.cprod)], context=context):
                self.pool.get('estoque.produto.codigo.fornecedor').create(
                    cr, uid, {'fornecedor_id': fornecedor_id, 'codigo_fornecedor': brw_item.cprod,
                              'grade_id': grade_id, 'produto_id': produto_id}, context)

            item_vr_icms_desonerado = 0
            item_tributacao_ids = []
            trib_encargos = []

            # TODO: Transformar em metodo
            # Encargos do item
            _temII = False
            for brw_encargo in brw_item.encargo_ids:
                if brw_encargo.encargo_id.sigla == 'II':
                    _temII = True
                # Verifica a tributacao no cadastro de produto
                encargo_sigla = brw_encargo.encargo_id.sigla
                # if not self.pool.get('fiscal.cst').search(
                #         cr, uid, [('fiscst_tipo_movimento', 'in', ('E', 'A')),
                #                   ('codigo', '=', brw_encargo.cst),
                #                   ('encargo_id', '=', brw_encargo.encargo_id.id)],
                #         context=context, count=True):
                #     self._erro.append(_('O CST do encargo %s do produto %s-%s nao pode ser de '
                #                         'saida.') % (encargo_sigla,
                #                                      brw_item.cean or brw_item.cprod,
                #                                      brw_item.xprod))
                #     continue

                # Verifica a tributacao no cadastro de produto
                if not brw_encargo.tributacao_id:
                    self._erro.append(_('Tributacao do produto %s-%s nao foi informada para o '
                                        'encargo %s.') % (brw_item.cean or brw_item.cprod,
                                                          brw_item.xprod,
                                                          encargo_sigla))
                    continue

                if encargo_sigla != brw_encargo.tributacao_id.encargo_id.sigla:
                    self._erro.append(_('No encargo %s do item de sequencia %s, a tributacao e '
                                        'do encargo %s') % (encargo_sigla,
                                                            brw_item.sequencia,
                                                            brw_encargo.tributacao_id.encargo_id.sigla))
                    continue

                cst_ids = self.pool.get('fiscal.cst').search(
                    cr, uid, [('encargo_id', '=', brw_encargo.encargo_id.id),
                              ('codigo', '=', brw_encargo.cst)], context=context)
                cst_id = cst_ids and cst_ids[0] or False
                produto_encargo_ids = self.pool.get('fiscal.tributacao.encargos').search(
                    cr, uid, [('produto_id', '=', produto_id), ('tipo_tributacao', '=', 'E'),
                              ('encargo_id', '=', brw_encargo.encargo_id.id)], context=context)
                brw_tributacao_encargo = self.pool.get('fiscal.tributacao.encargos').browse(
                    cr, uid, produto_encargo_ids)
                tributacao_dados = {'tipo_tributacao': 'E', 'produto_id': produto_id,
                                    'encargo_id': brw_encargo.encargo_id.id,
                                    'tributacao_id': brw_encargo.tributacao_id.id, 'cst_id': cst_id,
                                    'mva': brw_encargo.percentual_mva,
                                    'modbcst': brw_encargo.modbcst,
                                    'valor_pauta': brw_encargo.valor_pauta,
                                    'fisten_aliquota_venda': brw_encargo.aliquota_venda}
                if brw_tributacao_encargo:
                    for encargo in brw_tributacao_encargo:
                        if encargo.tipo_tributacao == 'E':
                            # Altera a ultima tributacao de entrada
                            trib_encargos.append([1, encargo.id, tributacao_dados])
                        else:
                            # se nao existe uma tributacao para o produto, cria.
                            trib_encargos.append([0, False, tributacao_dados])
                else:
                    # Cria tributacao se nao existe uma para o produto
                    trib_encargos.append([0, False, tributacao_dados])

                # Adiciona encargos da nota
                if encargo_sigla == 'PIS':
                    rec_pis_cofins = brw_item.pis_nat_rec
                elif encargo_sigla == 'COFINS':
                    rec_pis_cofins = brw_item.cofins_nat_rec
                else:
                    rec_pis_cofins = False
                tipo_substituicao = False
                vr_icms_substituicao = brw_encargo.vr_icms_substituicao
                if encargo_sigla == 'ICMS':
                    if cst_id:
                        dados_cst = self.pool.get('fiscal.cst').read(
                            cr, uid, cst_id, ['tipo', 'codigo'], context)
                        if dados_cst.get('tipo') == 'U':
                            if dados_cst.get('codigo') in ('10', '30', '60', '70'):
                                tipo_substituicao = True
                    item_vr_icms_desonerado = brw_encargo.vr_icms_desonerado or 0
                item_tributacao_ids.append(
                    [0, False, {'encargo_id': brw_encargo.encargo_id.id, 'cst_id': cst_id,
                                'aliquota': brw_encargo.aliquota,
                                'vr_base_calculo': brw_encargo.vr_base_calculo,
                                'tributacao_id': brw_encargo.tributacao_id.id,
                                'vr_encargo': brw_encargo.vr_encargo,
                                'percentual_mva': brw_encargo.percentual_mva, 'retido': False,
                                'vr_reducao_base': brw_encargo.vr_reducao_base,
                                'aliquota_venda': brw_encargo.aliquota_venda,
                                'valor_pauta': brw_encargo.valor_pauta,
                                'modbcst': brw_encargo.modbcst,
                                'mkpnit_nat_rec_pis_cofins': rec_pis_cofins,
                                'vr_icms_substituicao': vr_icms_substituicao,
                                'tipo_substituicao': tipo_substituicao,
                                'vr_base_icms_substituicao': brw_encargo.vr_base_icms_substituicao,
                                'mkpnit_vr_difal_uf_origem': brw_encargo.vr_difal_uf_origem,
                                'mkpnit_vr_difal_uf_destino': brw_encargo.vr_difal_uf_destino,
                                'mkpnit_vr_fcp': brw_encargo.vr_fcp,
                                'mkpnit_vr_fcp_st': brw_encargo.vr_fcp_st,
                                'mkpnit_vr_icms_desonerado': brw_encargo.vr_icms_desonerado}])

            # Verifica obrigatoridade de encargos nos itens
            faltantes = self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(
                cr, uid, map(lambda m: m.encargo_id and m.encargo_id.id, brw_item.encargo_ids),
                context)
            if faltantes:
                self._erro.append(_('Os seguintes encargos obrigatorios para esta empresa devem '
                                    'ser inseridos no item %s:\n%s') %
                                  (brw_item.xprod, '\n'.join(['\t* %s' % f[1] for f in faltantes])))

            # Verifica o tipo de CFOP. Se diferente de "outros" a nota nao podera ser zero
            if not brw_item.cfop:
                self._erro.append(_('Informe CFOP para o item %s.') % (
                    brw_item.cean or brw_item.cprod, brw_item.xprod))
            cfop_ids = self.pool.get('fiscal.cfop').search(
                cr, uid, [('codigo', '=', brw_item.cfop)], context=context)
            cfop_id = cfop_ids[0] if cfop_ids else False
            if cfop_id and self.pool.get('fiscal.cfop').read(
                    cr, uid, cfop_id, ['movimento_tipo'], context).get('movimento_tipo') != 'U':
                cfop_outros = False

            # Ativa novamente ja que esta sendo importado
            produto_inativo_ids = self.pool.get('estoque.produto').search(
                cr, uid, [('id', '=', produto_id), ('active', '=', False)], context=context)
            if produto_inativo_ids:
                self.pool.get('estoque.produto').write(
                    cr, uid, produto_inativo_ids[0], {'active': True}, context=context)

            if isinstance(brw_item.qcom, basestring):
                brw_item.qcom = brw_item.qcom.replace(',', '.')

            # Total da nota
            total_servico = brw_nfe.total_issqntot_vserv
            total_produto = brw_nfe.total_icmstot_vprod - brw_nfe.total_icmstot_vdesc
            total_nf = total_produto + total_servico  # Total da nota para calculo do rateio
            quantidade = float(brw_item.qcom) or 0
            vr_unitario = float(brw_item.vuncom) or 0
            vr_desconto = float(brw_item.vdesc) or 0
            vr_total_item = quantidade * vr_unitario - vr_desconto
            vr_despesas_rateado = float(brw_item.voutro) or 0
            vr_frete_rateado = 0
            if total_nf and brw_nfe.total_icmstot_vfrete:
                vr_frete_rateado = vr_total_item / total_nf * brw_nfe.total_icmstot_vfrete
            vr_seguro_rateado = 0
            if total_nf and brw_nfe.total_icmstot_vseg:
                vr_seguro_rateado = vr_total_item / total_nf * brw_nfe.total_icmstot_vseg
            vr_total_item_rateado = \
                vr_total_item + vr_despesas_rateado + vr_frete_rateado + vr_seguro_rateado - \
                item_vr_icms_desonerado
            if _temII:
                for brw_encargo in brw_item.encargo_ids:
                    if brw_encargo.encargo_id.sigla in ('IPI', 'II', 'ICMS'):
                        vr_total_item_rateado += brw_encargo.vr_encargo
            total += vr_total_item_rateado
            total_despesa += vr_despesas_rateado
            total_frete += vr_frete_rateado
            total_seguro += vr_seguro_rateado
            total_base_icms_st += sum([e.vr_base_icms_substituicao
                                       if e.encargo_id.sigla.upper() == 'ICMS'
                                       else 0 for e in brw_item.encargo_ids])

            # Se nao encontrou a embalagem informada do item, entao cadastra
            produto_embalagem_ids = obj_produto_embalagem.search(
                cr, uid, [('produto_id', '=', produto_id), ('embalagem_id', '=', embalagem_id)])
            if not produto_embalagem_ids:
                produto_embalagem_id = obj_produto_embalagem.create(
                    cr, uid, {'produto_id': produto_id, 'embalagem_id': embalagem_id}, context)
            else:
                produto_embalagem_id = produto_embalagem_ids[0]

            # Associa Local/locacao se nao existir
            if not self.pool.get('estoque.local.locacao').search(
                    cr, uid, [('grade_id', '=', grade_id), ('local_id', '=', brw_item.local_id.id),
                              ('produto_id', '=', produto_id)], context=context):
                self.pool.get('estoque.local.locacao').create(
                    cr, uid, {'grade_id': grade_id, 'local_id': brw_item.local_id.id,
                              'produto_id': produto_id,
                              'estllc_produto_embalagem_id': produto_embalagem_id}, context)

            vals_nf_itens = {'produto_id': produto_id, 'grade_id': grade_id or brw_item.grade_id.id,
                             'local_id': brw_item.local_id.id,
                             'produto_embalagem_id': produto_embalagem_id, 'cfop_id': cfop_id,
                             'quantidade': quantidade, 'vr_total_item': vr_total_item,
                             'vr_unitario': vr_unitario, 'tributacoes_ids': item_tributacao_ids,
                             'vr_frete_rateado': vr_frete_rateado, 'vr_desconto': vr_desconto,
                             'natureza_id': brw_nfe.natureza_id.id,
                             'vr_total_item_rateado': vr_total_item_rateado,
                             'vr_despesas_rateado': vr_despesas_rateado,
                             'vr_seguro_rateado': vr_seguro_rateado, 'sequencia': seq_itens}
            seq_itens += 1

            # TODO: Levar para o modulo estoqueserie
            # if modulo_serie_instalado and brw_item.winf_item_serie_ids:
            #     series = map(lambda m: m.wseinf_numero_serie, brw_item.winf_item_serie_ids)
            #     vals_nf_itens.update({'produto_series': series,
            #                           'serie_tipo_controle': brw_item.winf_item_controle_serie})
            itens.append([0, False, vals_nf_itens])

        # Ajuste do rateio
        diferenca = total
        diferenca -= sum(map(lambda m: round(m[2]['vr_total_item_rateado'], 2), itens))
        if itens and (diferenca <= -0.01 or 0.01 <= diferenca):
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item')))
            itens[index][2]['vr_total_item_rateado'] += diferenca

        if itens_serao_criados:
            msg_itens_criados = 'Os itens abaixo de sequencia e descricao, respectivamente, ' \
                                'serao cadastrados:\n ' + itens_serao_criados
            self.write(cr, uid, ids, {'log_err': msg_itens_criados}, context)
            reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fiscal', 'mkp_import_nfe_osv_cria_itens_view')
            view_id = reference and reference[1] or False
            return {
                'name': 'Importar NF-e',
                'res_id': ids[0],
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'mkp.import.nfe.osv',
                'view_id': [view_id],
                'target': 'new',
                'type': 'ir.actions.act_window',
                'context': context
            }
        msg_erro = []
        if msg_inativo:
            msg_erro.append(_('Produtos correlacionados estao inativos nos itens de sequencia: %s')
                            % ',\n'.join(msg_inativo))
        if msg_kit:
            msg_erro.append(_('Os produtos correlacionados sao kits nos itens de sequencia: %s')
                            % ',\n'.join(msg_kit))
        if msg_barra:
            msg_erro.append(_('Os codigos de barras sao de kit nos itens de sequencia: %s')
                            % ',\n'.join(msg_barra))
        if msg_fornecedor:
            msg_erro.append(_('Os produtos cadastrados no fornecedor sao kits nos itens de '
                              'sequencia: %s') % ',\n'.join(msg_fornecedor))
        if msg_erro:
            raise osv.except_osv(_('Atencao!'),
                                 '\n\n'.join(msg_erro))

        if self._erro:
            self._erro.append(_('\n * Nao foi possivel importar a nota fiscal.'))
            cr.rollback()  # Desfaz procedimentos
            self.write(
                cr, uid, ids, {'check_err': True, 'log_err': '\n'.join(
                    [self._info and _('[A V I S O S]')] + self._info + self._erro and
                    ['\n[E R R O S ]'] + self._erro), 'log_info': _('Nao foi possivel importar a '
                                                                    'NF-e.')}, context)
            return False

        # Cria a nota fiscal
        for item in itens:
            # TODO: Mover para a classe ´estoqueserie.entrada´ com heranca na criacao da nota
            if modulo_serie_instalado:
                item = item[2] or {}
                series = []
                produto_id = item.get('produto_id')
                msg_serie_erro = ''
                erro_numero_serie_invalido = ''
                erro_quantidade_numero_serie = ''
                serie_dados_entrada = {}
                serie_tipo_controle = item.get('serie_tipo_controle')
                list_series = item.get('produto_series') or []
                produto_name = dict(self.pool.get('estoque.produto').name_get(
                    cr, uid, map(lambda m: m.produto_id.id,
                                 filter(lambda w: w.produto_id, brw_nfe.prod_ids)), context) or
                                    [('', '')])

                if list_series and serie_tipo_controle in ('NS', 'NC'):
                    quantidade_item = item.get('quantidade')
                    series = self.pool.get('serie.utils').serutl_generate_range_serie(
                        list_series) or []
                    name = (produto_name.get(produto_id) or '') or _('** Produto nao encontrado **')

                    if not series:
                        serie_utils_erro = self.pool.get('serie.utils').serutl_get_erro()
                        erro_numero_serie_invalido += '%s - %s, ' % (name, serie_utils_erro)
                    elif len(series) != quantidade_item:
                        erro_quantidade_numero_serie += '%s, ' % name
                        msg_serie_erro += _('Quantidade de numero de serie inseridas difere da '
                                            'quantidade nos itens: %s' %
                                            erro_quantidade_numero_serie)
                        raise osv.except_osv(_('Atencao!'), msg_serie_erro)

                # Remove as chaves ´produto_series´ e ´serie_tipo_controle´. Sao utilizadas
                # temporariamente no dicionario
                if 'produto_series' in item:
                    del item['produto_series']
                if 'serie_tipo_controle' in item:
                    del item['serie_tipo_controle']

                id_mov = ''
                if id_mov:
                    # Dados das series para serem gravados.
                    serie_dados_entrada.update({int(id_mov): series})

                if erro_numero_serie_invalido:
                    msg_serie_erro += _('Numero de serie invalido para os itens: %s' %
                                        erro_numero_serie_invalido)
                    raise osv.except_osv(_('Atencao!'), msg_serie_erro)
                else:
                    # Serie entrada
                    for movimento_id in serie_dados_entrada:
                        movimento_series = serie_dados_entrada[movimento_id]
                        for serie in movimento_series:
                            self.pool.get('estoqueserie.entrada').create(
                                cr, uid,
                                {'eseent_numero_serie': serie, 'eseent_mov_item_id': movimento_id},
                                context=context)
        parcelas = []
        if filter(lambda f: f.cfop[-3:] != '910', brw_nfe.prod_ids):
            for brw_parcela in brw_nfe.parcela_ids:
                if brw_parcela.vencimento and brw_parcela.valor:
                    parcelas.append([0, False, {'valor': brw_parcela.valor,
                                                'vencimento': brw_parcela.vencimento}])

        procnfe = ProcNFe()
        procnfe.xml = base64.decodestring(brw_nfe.xml_file)
        _dados_parametros = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'fiscal.parametro', ['id'],
                                                                             context=context)
        _param_id = _dados_parametros.get('id')
        args = [('modelo_nf_id', '=', ide_mod), ('serie_nf', '=', brw_nfe.ide_serie), ('parametro_id', '=', _param_id)]
        _modelo_nf_series_ids = self.pool.get('fiscal.modelo.nf.series').search(cr, uid, args, context=context)
        if not _modelo_nf_series_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Serie %s nao cadastrada para a empresa_id: %s' % (brw_nfe.ide_serie,
                                                                                     brw_nfe.company_id.id)))
        _modelo_nf_series_id = _modelo_nf_series_ids[0]
        vals = {
            'tipo': 'E',
            'company_id': brw_nfe.company_id.id,
            'modelo_nf_id': ide_mod,
            'modelo_nf_series_id': _modelo_nf_series_id,
            'status': '3',
            'nf_propria': True,
            'destinatario_id': brw_nfe.dest_pessoa_id.id,
            'data_saida_entrada': brw_nfe.ide_dhemi,
            'data_emissao': brw_nfe.ide_dhemi,
            'destinatario_endereco_id': endereco_id,
            'estoque_movimento_itens_ids': itens,
            'natureza_id': brw_nfe.natureza_id.id,
            'numero_nf': brw_nfe.ide_nnf,
            'serie_nf': brw_nfe.ide_serie,
            'vr_base_icms': brw_nfe.total_icmstot_vbc,
            'vr_icms': brw_nfe.total_icmstot_vicms,
            'vr_base_icms_substituicao': total_base_icms_st,
            'vr_icms_substituicao': brw_nfe.total_icmstot_vst,
            'vr_frete': total_frete,
            'vr_total_servicos': brw_nfe.total_issqntot_vserv,
            'nfe_xml_gerado': procnfe.NFe.xml,
            'vr_total_produtos': brw_nfe.total_icmstot_vprod,
            'vr_seguro': total_seguro,
            'vr_despesas_acessorias': total_despesa,
            'vr_ipi': brw_nfe.total_icmstot_vipi,
            'nfe_xml_auto': procnfe.protNFe.xml,
            'nfe_chave_acesso': brw_nfe.ide_id[3:],
            'nfe_protocolo': brw_nfe.protocolo,
            'vr_total_nf': brw_nfe.total_icmstot_vnf,
            'modelo_55': True,
            'prazo_tipo': 'DD',
            'prazo': '0',
            'frete': brw_nfe.modfrete, 'tipo_frete': 'D',
            'fisnfc_infcomp_contribuinte': brw_nfe.informacao_complementar,
            'fisnfc_infcomp_fisco': brw_nfe.informacao_fisco,
            'fisnfc_parcelas_terceiros_ids': parcelas,
            'fisnfc_tipo_servico': brw_nfe.natureza_id.cfop_id.produto_servico_tipo == 'S'
        }
        context['nf_capa_manutencao'] = True
        nf_id = self.pool.get('fiscal.nf.capa').create(cr, uid, vals, context=context)

        # Exclui os dados da importacao que nao sao mais necessarios
        self.unlink(cr, uid, ids, context)

        # Mostra a nota fiscal que foi importada
        result = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'fiscal', 'view_fiscal_nf_capa_terceiro_form')
        view_id = result and result[1] or False

        return {'name': 'Nota fiscal importada', 'view_type': 'form', 'view_mode': 'form',
                'view_id': [view_id], 'res_model': 'fiscal.nf.capa', 'res_id': nf_id,
                'type': 'ir.actions.act_window', 'context': {'import_nfe': True, 'terceiro': True}}

    def _impnfe_item_create(self, cr, user, nf, item, context=None):
        context = context and context.copy() or {}
        brw_nfe = self.pool.get('mkp.import.nfe.osv').browse(cr, user, nf, context)
        brw_item = filter(lambda f: f.id == item, brw_nfe.prod_ids)[0]
        ncm_ids = self.pool.get("fiscal.ncm").search(
            cr, user, [('codigo', '=', brw_item.ncm)], context=context)
        if not ncm_ids:
            ncm_id = self.pool.get("fiscal.ncm").create(
                cr, user, {'codigo': brw_item.ncm, 'descricao': brw_item.ncm}, context)
        else:
            ncm_id = ncm_ids[0]
        if self.pool.get('estoque.produto').search(
                cr, user, [('name', '=', brw_item.xprod),
                           ('company_id', '=', brw_nfe.company_id.id)], context=context):
            if brw_item.cean:
                # tratamento para descricao do item maior que 60 caracteres
                if len(brw_item.xprod) == 60:
                    brw_item.xprod = \
                        brw_item.xprod[:len(brw_item.xprod) - len(brw_item.cean) - 1]

                brw_item.xprod = brw_item.xprod + ' ' + brw_item.cean
            elif brw_item.cprod:
                # tratamento para descricao do item maior que 60 caracteres
                if len(brw_item.xprod) == 60:
                    brw_item.xprod = \
                        brw_item.xprod[:len(brw_item.xprod) - len(brw_item.cprod) - 1]

                brw_item.xprod = brw_item.xprod + ' ' + brw_item.cprod
        if brw_item.ncm == '99':  # NCM identifica um servico
            produto_tipo = 'S'
            tipo_item = '09'
        else:  # NCM identifica um produto
            produto_tipo = 'P'
            tipo_item = '00'
        dados = {'company_id': brw_nfe.company_id.id, 'name': brw_item.xprod, 'ncm_id': ncm_id,
                 'tipo_item': tipo_item, 'tipo': produto_tipo, 'grade_usar': brw_item.usar_grade,
                 'estpro_pis_nat_rec': brw_item.pis_nat_rec,
                 'categoria_id': brw_item.categoria_id.id,
                 'estpro_cofins_nat_rec': brw_item.cofins_nat_rec}

        # Codigo e digito em funcao do parametro do estoque
        digito = False
        if self.pool.get('adm.utils').admutl_get_parametros(cr, user, 'estoque', ['codigo_numerico']
                                                            ).get('codigo_numerico'):
            codigo = self.pool.get('adm.utils').get_sequence(cr, user, 'estoque.produto',
                                                             'estoque.produto')
            if codigo and self.pool.get('adm.utils').admutl_get_parametros(
                    cr, user, 'estoque', ['usa_digito']).get('usa_digito'):
                digito = calcDigito.ean(codigo)
        else:
            codigo = brw_item.cprod

        dados.update({'codigo': codigo, 'digito': digito})

        # TODO: Mover para a classe ´estoque.serie´ com heranca no evento ´create´
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(
                cr, user, 'estoqueserie', context):
            dados.update({'estpro_nserie_tipo': brw_item.winf_item_controle_serie})

        # Pesquisa a embalagem. Se nao encontrou, cadastra
        if not brw_item.embalagem_id:
            embalagem_ids = self.pool.get('estoque.embalagem').search(
                cr, user, [('sigla', '=', brw_item.ucom)], context=context)
            if not embalagem_ids:
                # Se nao encontrou a unidade, realiza o seu cadastro
                unidade = brw_item.ucom.upper()
                unidade_ids = self.pool.get('estoque.embalagem.unidade').search(
                    cr, user, [('unidade', '=', unidade)], context=context)
                if not unidade_ids:
                    unidade_id = self.pool.get('estoque.embalagem.unidade').create(
                        cr, user, {'unidade': unidade, 'name': unidade + '_'}, context)
                else:
                    unidade_id = unidade_ids[0]

                embalagem_id = self.pool.get('estoque.embalagem').create(
                    cr, user,
                    {'name': brw_item.ucom, 'sigla': brw_item.ucom, 'unidade_id': unidade_id,
                     'quantidade_na_embalagem': 1}, context)
            else:
                embalagem_id = embalagem_ids[0]
        else:
            embalagem_id = brw_item.embalagem_id.id

        dados['produto_embalagem_ids'] = [[0, False, {'embalagem_id': embalagem_id}]]
        if brw_item.usar_grade:
            dados['produto_grade_ids'] = [[0, False,
                                           {'grade': brw_item.grade, 'descricao': brw_item.grade}]]

        # Cadastro do item
        produto_id = self.pool.get('estoque.produto').create(cr, user, dados, context=context)
        brw_produto = self.pool.get('estoque.produto').browse(cr, user, produto_id, context=context)
        grade_ids = self.pool.get('estoque.produto.grade').search(
            cr, user, [('produto_id', '=', produto_id)], context=context)
        grade_id = grade_ids[0]
        if brw_item.cean:
            if str(brw_item.cean[-1:]) == str(calcDigito.ean(brw_item.cean[:-1])):
                codigo_barra_p = '%014d' % int(brw_item.cean)

                # Verifica  se o codigo eh inativo e o ativa novamente
                barra_inativo_id = self.pool.get('automacao.codigo.barra').search(
                    cr, user, [('codigo_barra', '=', codigo_barra_p),
                               ('active', '=', False)], context=context)
                if barra_inativo_id:
                    self.pool.get('automacao.codigo.barra').write(
                        cr, user, barra_inativo_id[0], {'active': True}, context)
                codigo_barras_ids = self.pool.get('automacao.codigo.barra').search(
                    cr, user, [('codigo_barra', '=', codigo_barra_p)],
                    context=context)
                if not codigo_barras_ids:
                    vals_bar = {'codigo_barra': codigo_barra_p, 'grade_id': grade_id,
                                'produto_embalagem_id': brw_produto.produto_embalagem_ids[0].id,
                                'ultima_data_movimentacao': date.today().strftime('%Y-%m-%d'),
                                'produto_id': produto_id}
                    self.pool.get('automacao.codigo.barra').create(
                        cr, user, vals_bar, context)
        else:
            self._info.append(_('O item %s-%s sera cadastrado sem codigo de barras. O codigo de '
                                'barras podera ser informado diretamente no cadastro do '
                                'produto.') % (brw_item.cean or brw_item.cprod, brw_item.xprod))

        return produto_id, grade_id

    def _impnfe_get_item_codigo_barra(self, cr, user, ean, context=None):
        """
        Consulta um item pelo codigo de barras e retorna as informacos do item, grade e embalagem.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ean: o codigo de barras que representa o item de consulta
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ean: str
        :type context: dict
        :return: o ID do item e da grade.
        :rtype: tuple
        """
        context = context and context.copy() or {}
        codigo_barras_ids = self.pool.get('automacao.codigo.barra').search(
            cr, user, [('codigo_barra', '=', ean)], context=context)
        if codigo_barras_ids:
            # read foi colocado para ficar mais rapida a funcao
            dados_cb = self.pool.get('automacao.codigo.barra').read(cr, user, codigo_barras_ids[0],
                                                                    ['produto_id', 'grade_id'], context,
                                                                    load='_classic_write')
            grade_id = dados_cb.get('grade_id')
            produto_id = dados_cb.get('produto_id')
        else:
            produto_id = grade_id = False

        return produto_id, grade_id

    def _impnfe_get_item_codigo_fornecedor(self, cr, user, codigo, fornecedor, context=None):
        """
        Consulta um item pelo codigo do fornecedor e retorna as informacoes do item, grade e
        embalagem.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param codigo: o codigo que representa o item para um determinado fornecedor
        :param fornecedor: o ID do fornecedor
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type codigo: str
        :type fornecedor: long
        :type context: dict
        :return: o ID do item e da grade
        :rtype: tuple
        """
        fornecedor_ids = self.pool.get('estoque.produto.codigo.fornecedor').search(
            cr, user, [('codigo_fornecedor', '=', codigo), ('fornecedor_id', '=', fornecedor)],
            context=context)
        if fornecedor_ids:
            brw_produto_codigo_fornecedor = \
                self.pool.get('estoque.produto.codigo.fornecedor').browse(
                    cr, user, fornecedor_ids[0], context)
            produto_id = brw_produto_codigo_fornecedor.produto_id.id
            grade_id = brw_produto_codigo_fornecedor.grade_id.id
        else:
            produto_id = grade_id = False

        return produto_id, grade_id

    # def _check_tipo_movimento_cst(self, cr, uid, brw_item, context=None):
    #     """
    #     O metodo calcula o valor total da bonificacao da nota fiscal informada.
    #
    #     :param cr: cursor no banco de dados
    #     :param uid: id da tabela res_users, identificador do usuario
    #     :param brw_item: brwser do item a ser importado
    #     :param context: dicionario com valores contextuais
    #     :type cr: object
    #     :type uid: int
    #     :type brw_item: Browse
    #     :type context: dict
    #     :return o valor total da bonificacao da nota fiscal.
    #     :rtype: float
    #     :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
    #     """
    #     context = context and context.copy() or {}
    #     res = []
    #     obj_cst = self.pool.get('fiscal.cst')
    #     csts_invalido = filter(lambda i: obj_cst.fiscst_get_tipo_movimento(
    #         cr, uid, i.cst, i.encargo_id.id, context) == 'S', brw_item.encargo_ids)
    #     msg = 'O item {item} possui CST de saida para o encargo {encargo}'
    #     map(lambda i: res.append(msg.format(item='%s-%s' % (
    #         i.item_id.cean or i.item_id.cprod, i.item_id.xprod), encargo=i.encargo_id.sigla)),
    #         csts_invalido)
    #     return res


# TODO: Mover para ´import_nfe_itens_data´
class mkp_import_nfe_itens_osv(osv.Model):
    _name = 'mkp.import.nfe.itens.osv'
    _description = 'Itens da NFe'
    _rec_name = 'nf_id'
    _transient = False

    def fisinfi_get_produto_referenciado(self, cr, uid, ids, field_name, arg, context=None):
        """
        Mostra qual produto foi relacinado ao item do xml. Caso este jah esteja cadastrado no
        sistema:
        1 - Verifica se o usuario relacionou manualmente um produto na aba "Outras informacoes";
        2 - Verifica se o codigo de barras esta cadastrado;
        3 - Verifica se o codigo do fornecedor esta cadastrado.

        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param field_name: lista com nome dos atributos
        :param arg: argumentos de pesquisa.
        :param context: dicionario com valores contextuais.
        :type cr: object
        :type uid: int
        :type ids: list
        :type field_name: list
        :type arg: list
        :type context: dict
        :return: lista de dicionarios com os atributos e seus respectivos valores.
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = {}
        obj_estprocodfor = self.pool.get('estoque.produto.codigo.fornecedor')
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        campos = {'produto_id': [], 'cean': [], 'cprod': [], 'nf_id': ['dest_pessoa_id', 'company_id']}
        for item in self.pool.get('adm.utils').admutl_read_browse(
                cr, uid, ids, self._name, campos, context=context):
            produto_id = False
            if item.get('produto_id'):
                produto_id = item.get('produto_id')
            else:
                cbarra_id = obj_automacao_codigo_barra.search(
                    cr, uid, [('codigo_barra', '=', item.get('cean') or '')], context=context)
                for codigo_barra in obj_automacao_codigo_barra.browse(cr, uid, cbarra_id,
                                                                      context=context):
                    produto_id = codigo_barra.produto_id.id
                if not produto_id:
                    fornecedor_id = obj_financeiro_fornecedor.search(
                        cr, uid, [('pessoa_id', '=', item.get('nf_id').get('dest_pessoa_id')),
                                  ('company_id', '=', item.get('nf_id').get('company_id'))],
                        context=context)[0]
                    cfornec_id = obj_estprocodfor.search(
                        cr, uid, [('codigo_fornecedor', '=', item.get('cprod')),
                                  ('fornecedor_id', '=', fornecedor_id)], context=context)
                    for cod_fornecedor in obj_estprocodfor.browse(cr, uid, cfornec_id, context=context):
                        produto_id = cod_fornecedor.produto_id.id
            res[item.get('id')] = produto_id

        return res

    def fisinfi_get_par_codigo_numerico(self, cr, uid, ids, field_name, arg, context=None):
        par = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque.parametro',
                                                               ['codigo_numerico'], context=context)
        res = {}
        for id in ids:
            res[id] = par.get('codigo_numerico')

        return res

    _columns = {
        'sequencia': fields.integer('Sequencia'),  # Sequencia do item na nf
        'nf_id': fields.many2one('mkp.import.nfe.osv', 'NF Capa', ondelete="cascade",
                                 select=True),
        'par_codigo_numerico': fields.function(fisinfi_get_par_codigo_numerico,
                                               string='Parametro Codigo numerico', type='boolean', method=True),
        'cprod': fields.char('cProd', size=60, help="Codigo do item (tam=60)"),
        'cean': fields.char('cEAN', size=14, readonly=True, help="Codigo de barras (tam=2,13,14)"),
        'xprod': fields.char('xProd', size=60, help="Descricao do item (tam=60)"),
        'ncm': fields.char('NCM', size=8, readonly=True,
                           help="Codigo NCM com 8 digitos ou 2 digitos (genero) (tam=2,8)"),
        'extipi': fields.char('EXTIPI', size=3, readonly=True, help="EX_TIPI (tam=2-3)"),
        'cfop': fields.char('CFOP', size=14,
                            help="Codigo Fiscal de Operacoes e Prestacoes (tam=4)"),
        'ucom': fields.char('uCom', size=6, readonly=False, help="Unidade comercial (tam=1-6)"),
        'qcom': fields.char('qCom', size=20, help="Quantidade comercial (tam=0-4)"),
        'vuncom': fields.char('vUnCom', size=18, readonly=True,
                              help="Valor Unitario de Comercializacao (tam=21,10)"),
        'vprod': fields.char('vProd', size=18, readonly=True, help="Valor total bruto (tam=15,2)"),
        'ceantrib': fields.char('cEANTrib', size=14, readonly=True,
                                help="Codigo de barras (tam=2,13,14)"),
        'utrib': fields.char('uTrib', size=6, readonly=True, help="Unidade tributavel (tam=1-6)"),
        'qtrib': fields.char('qTrib', size=20, readonly=True,
                             help="Quantidade tributavel (tam=15,4)"),
        'vuntrib': fields.char('vUnTrib', size=18, readonly=True,
                               help="Valor Unitario de tributacao (tam=21,10)"),
        'vfrete': fields.char('vFrete', size=18, readonly=True,
                              help="Valor total do frete (tam=15,2)"),
        'vseg': fields.char('vSeg', size=18, readonly=True,
                            help="Valor total do seguro (tam=15,2)"),
        'vdesc': fields.char('vDesc', size=18, readonly=True, help="Valor do desconto (tam=15,2)"),
        'voutro': fields.char('vOutro', size=18, readonly=True,
                              help="Outras despesas acessorias (tam=15,2)"),
        'indtot': fields.selection([('0', 'Nao compoe o valor total da NF-e'),
                                    ('1', 'Compoe o valor total da NF-e')], 'indTot', readonly=True,
                                   help="Indica se valor do item entra no valor total da NF-e"),
        'genero': fields.char('genero', size=6, readonly=True, help="Unidade tributavel (tam=1-6)"),
        'encargo_ids': fields.one2many('mkp.import.nfe.encargos.osv', 'item_id',
                                       'Itens da NF-e'),
        'usar_grade': fields.boolean('Usa grade'),
        'grade': fields.char('Nova grade', size=30),
        'erro': fields.boolean('Erro de importacao'),
        'prod_nao_cadastrado': fields.boolean('Produto nao cadastrado'),
        'cadastrar': fields.boolean('Cadastrar produto'),
        'produto_id': fields.many2one("estoque.produto", 'Produto'),
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade'),
        'cadastrar_tributacao_entrada': fields.boolean('Cadastrar tributacao'),
        'flag_grade_unica': fields.boolean('Grade unica'),
        'item_editado': fields.boolean('Item editado'),
        'categoria_id': fields.many2one('estoque.categoria', 'Categoria'),
        'embalagem_id': fields.many2one("estoque.embalagem", 'Embalagem'),
        'pis_nat_rec': fields.char('Natureza da receita do PIS', size=3,
                                   help='Codigo da natureza da receita para efeitos de PIS'),
        'cofins_nat_rec': fields.char('Natureza da Receita do COFINS', size=3,
                                      help='Codigo da natureza da receita para efeitos de COFINS'),
        'descricao_alterada_60': fields.boolean('Acima de 60'),
        'local_id': fields.many2one("estoque.local", 'Local', required=True),
        'usa_codigo_numerico': fields.boolean('Usa codigo numerico'),
        'codigo_alfa_numerico': fields.char('Codigo', size=20),
        'produto_referencia_id': fields.function(
            fisinfi_get_produto_referenciado, string='Produto relacionado', type='many2one',
            relation='estoque.produto', method=True),
    }

    _defaults = {
        'usar_grade': lambda *a: False,
        'cadastrar': lambda *a: False,
        'cadastrar_tributacao_entrada': False,
        'item_editado': False,
        'erro': False,
        'descricao_alterada_60': False
    }

    def validar_itens(self, cr, uid, encargo_ids, embalagem_id, context=None):
        """
        Verifica as informacoes relacionadas aos itens e retorna se estao preenchidos corretamente.
        """

        context = context and context.copy() or {}
        if not embalagem_id:
            return False
        res = False
        tributacao_ids = []

        # Verificacao de tributacao no cadastro de produto
        for y in encargo_ids:
            if y[2] and y[2].get('tributacao_id'):
                tributacao_id = y[2].get('tributacao_id')
                if isinstance(y[2].get('tributacao_id'), (list, tuple)):
                    tributacao_id = y[2].get('tributacao_id')[0]
                tributacao_ids.append(tributacao_id)
            if y[1]:
                obj_mkp_import_encargos = self.pool.get('mkp.import.nfe.encargos.osv')
                brw_import_encargos = obj_mkp_import_encargos.browse(cr, uid, y[1])
                try:
                    tributacao_id = brw_import_encargos.tributacao_id.id
                except AttributeError:
                    tributacao_id = False
                if tributacao_id:
                    tributacao_ids.append(tributacao_id)

        # Verificacao de obrigatoriedade encargos dos itens
        encargos_tributos = []
        browse_tributacao = self.pool.get('fiscal.tributacao').browse(
            cr, uid, tributacao_ids, context)
        for t in browse_tributacao:
            encargos_tributos.append(t.encargo_id.id)
        encargos_faltantes = self.pool.get('fiscal.utils'). \
            verifica_encargos_obrigatorio_empresa(cr, uid, encargos_tributos, context=context)
        if not encargos_faltantes and not context.get('carregando_xml'):
            res = True
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if 'nitemped' in vals:
            del vals['nitemped']
        if 'xped' in vals:
            del vals['xped']
        tamanho_xprod = len(vals.get('xprod'))
        produto_novo = vals.get('prod_nao_cadastrado')
        if tamanho_xprod > 60 and produto_novo:
            vals['descricao_alterada_60'] = True

        return super(mkp_import_nfe_itens_osv, self).create(cr, uid, vals, context=context)

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if context:
            dados = self.read(
                cr, uid, ids[0], ['usar_grade', 'grade', 'produto_id', 'cadastrar',
                                  'prod_nao_cadastrado'],
                context, '_classic_write')
            if dados['usar_grade'] != vals.get('usar_grade', True) or \
                    dados['grade'] != vals.get('grade', True):
                cadastrar = vals['cadastrar'] if 'cadastrar' in vals else dados['cadastrar']
                produto_id = vals['produto_id'] if 'produto_id' in vals else dados['produto_id']
                vals['prod_nao_cadastrado'] = False if produto_id else True

                if cadastrar and not produto_id:
                    vals['prod_nao_cadastrado'] = True
                    vals['cadastrar_tributacao_entrada'] = True
                vals['cadastrar_tributacao_entrada'] = False if 'encargo_ids' in vals else True
        if 'nitemped' in vals:
            del vals['nitemped']
        if 'xped' in vals:
            del vals['xped']
        return super(mkp_import_nfe_itens_osv, self).write(cr, uid, ids, vals, context=context)

    def onchange_cfop(self, cr, uid, ids, cfop, uf_emitente, context=None):
        context = context and context.copy() or {}
        res = {}
        if not cfop:
            return res
        obj_fiscal_cfop = self.pool.get('fiscal.cfop')
        cfop_ids = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop)], context=context)
        if not cfop_ids:
            warning = {'title': _('Aviso.'), 'message': _('Este CFOP nao esta cadastrado.')}
            return {'value': {'cfop': ''}, 'warning': warning}
        brw_fiscal_cfop = obj_fiscal_cfop.browse(cr, uid, cfop_ids[0], context)
        msg = str()
        if brw_fiscal_cfop.tipo == 'S':
            msg += _('CFOP deve ser de entrada.')
        elif cfop[0] != '1' and uf_emitente == 'MG':
            msg += _('CFOP invalido para operacoes dentro do estado.\n')
        elif cfop[0] != '2' and uf_emitente != 'MG':
            msg += _('CFOP invalido para operacoes fora do estado.\n')
        if msg:
            res['value'] = {'cfop': ''}
            res['warning'] = {'title': _('Atencao'), 'message': msg}
        return res

    def verifica_alteracao(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id,
                           grade_id, qcom, vprod, embalagem_id, context=None):
        context = context and context.copy() or {}
        values = {}
        warning = {}
        if isinstance(qcom, basestring):
            qcom = qcom.replace(',', '.')
        if embalagem_id and qcom:
            if not self.pool.get('estoque.embalagem').browse(
                    cr, uid, embalagem_id, context).unidade_id.fracao:
                if int(float(qcom)) != float(qcom):
                    values['embalagem_id'] = False
                    warning = {'title': _('Atencao!'),
                               'message': _('A unidade da embalagem nao permite quantidade '
                                            'fracionada.')}
        if float(qcom) > 0:
            vuncom = float(vprod) / float(qcom)
            values['vuncom'] = str(vuncom)
        else:
            warning = {'title': _('Aviso.'),
                       'message': _('A quantidade do item deve ser maior que zero.')}
        validar_itens = self.validar_itens(cr, uid, encargo_ids, embalagem_id, context)
        values['item_editado'] = validar_itens
        values['qcom'] = qcom
        return {'value': values, 'warning': warning}

    def onchange_xprod(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id, qcom,
                       vprod, embalagem_id, context=None):
        context = context and context.copy() or {}
        return self.verifica_alteracao(cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                                       qcom, vprod, embalagem_id, context=context)

    def onchange_embalagem_id(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                              qcom, vprod, embalagem_id, context=None):
        context = context and context.copy() or {}
        return self.verifica_alteracao(cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                                       qcom, vprod, embalagem_id, context=context)

    def onchange_ucom(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id, qcom,
                      vprod, embalagem_id, context=None):
        context = context and context.copy() or {}
        return self.verifica_alteracao(cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                                       qcom, vprod, embalagem_id, context=context)

    def onchange_qcom(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id, qcom,
                      vprod, embalagem_id, context=None):
        context = context and context.copy() or {}
        if not qcom and qcom is not 0:
            return {}

        try:
            float(qcom)
        except:
            return {'value': {'qcom': ''},
                    'warning': {'title': _('Atencao!'),
                                'message': _('Valor invalido para Quant. Comercial.')
                                }
                    }

        return self.verifica_alteracao(cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                                       qcom, vprod, embalagem_id, context=context)

    def impnfe_onchange_encargo_ids(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                                    qcom, vprod, embalagem_id, natureza_id, cfop, context=None):
        context = context and context.copy() or {}
        result = self.verifica_alteracao(
            cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id, qcom, vprod, embalagem_id,
            context=context)
        encargo_dados = filter(lambda f: self.pool.get('fiscal.encargo').browse(
            cr, uid, adjust_m2o(f).get('encargo_id'), context).sigla == 'ICMS',
                               self.resolve_2many_commands(
                                   cr, uid, 'encargo_ids', filter(
                                       lambda f: f[0] in (0, 1), encargo_ids) or [],
                                   context=context))
        if not encargo_dados:
            return result
        encargo = adjust_m2o(encargo_dados[0])
        tributacao_dados = encargo.get('tributacao_id') and self.pool.get('fiscal.tributacao').read(
            cr, uid, encargo.get('tributacao_id'), ['cst_id'], context, '_classic_write')
        cst_id = tributacao_dados and tributacao_dados.get('cst_id')
        cst_dados = cst_id and self.pool.get('fiscal.cst').read(
            cr, uid, cst_id, ['tipo', 'codigo'], context)
        cst_tipo = cst_dados and cst_dados.get('tipo')
        if cfop not in ('1910', '2910', '1911', '2911'):
            if cst_tipo in ('U', 'TU'):
                result['value']['cfop'] = \
                    self.pool.get('mkp.import.nfe.osv')._impnfe_get_cfop_entrada(
                        cr, uid, natureza_id, encargo.get('cst'), context)
            else:
                natureza_dados = natureza_id and self.pool.get('fiscal.cfop.natureza').read(
                    cr, uid, natureza_id, ['cfop_id'], context, '_classic_write')
                cfop_id = natureza_dados and natureza_dados.get('cfop_id')
                cfop_dados = cfop_id and self.pool.get('fiscal.cfop').read(
                    cr, uid, cfop_id, ['codigo'], context)
                cfop = cfop_dados and cfop_dados.get('codigo')
                result['value']['cfop'] = cfop

        return result

    def onchange_grade_id(self, cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id,
                          qcom, vprod, embalagem_id, local, context=None):
        context = context and context.copy() or {}
        result = self.verifica_alteracao(
            cr, uid, ids, encargo_ids, xprod, ucom, produto_id, grade_id, qcom, vprod, embalagem_id,
            context)
        if grade_id:
            brw_grade = self.pool.get('estoque.produto.grade').browse(
                cr, uid, grade_id, context)
            result['value']['categoria_id'] = brw_grade.produto_id.categoria_id.id
            if not produto_id:
                result['value']['produto_id'] = brw_grade.produto_id.id
                result['value']['grade_id'] = grade_id
            else:
                if brw_grade.produto_id.id != produto_id:
                    result['value']['grade_id'] = False
                    result['value']['categoria_id'] = False
                    result['warning'] = {'title': _('Aviso.'),
                                         'message': _('Codigo de barra da grade nao corresponde '
                                                      'produto.')}
                else:
                    if not embalagem_id:
                        locacao_ids = self.pool.get('estoque.local.locacao').search(
                            cr, uid, [('grade_id', '=', grade_id), ('local_id', '=', local)],
                            context=context)
                        locacao_dados = locacao_ids and self.pool.get('estoque.local.locacao').read(
                            cr, uid, locacao_ids[0], ['estllc_produto_embalagem_id'], context,
                            '_classic_write')
                        if locacao_dados:
                            embalagem_dados = self.pool.get('estoque.produto.embalagem').read(
                                cr, uid, locacao_dados.get('estllc_produto_embalagem_id'),
                                ['embalagem_id'], context, '_classic_write')
                            result['value']['embalagem_id'] = \
                                embalagem_dados.get('embalagem_id')
        else:
            result['value'] = dict(result['value'], categoria_id=False)

        return result

    def onchange_usa_grade(self, cr, uid, ids, usar_grade):
        vals = {}
        if usar_grade:
            vals.update({'grade_id': False})
        else:
            vals.update({'grade': False})

        return {'value': vals}

    def onchange_produto(self, cr, uid, ids, produto_id, dest_pessoa_id, indtot, encargo_ids,
                         context=None, grade_id=False):
        context = context and context.copy() or {}
        context['terceiro'] = True
        context['item_id'] = ids[0]
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, uid, [('sigla', '=', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]

        if not produto_id:
            return {'value': {'grade_id': False, 'categoria_id': False}}

        # Dados do item
        item_dados = self.read(cr, uid, ids[0], ['vprod', 'voutro', 'vfrete', 'vdesc'], context)

        # Valores de frete que compoe o valor da nota devem entrar no calculo da base
        if indtot == '1':
            context['importa_nf_frete_compoe_valor'] = float(item_dados.get('vfrete') or 0)
        value = {}.fromkeys(['grade_id', 'flag_grade_unica'], False)

        if not all((produto_id, dest_pessoa_id)):
            return value
        produto_dados = self.pool.get('estoque.produto').read(
            cr, uid, produto_id, ['produto_grade_ids', 'grade_usar', 'tributacao_entrada_ids',
                                  'tributacao_ids', 'estpro_pis_nat_rec', 'estpro_cofins_nat_rec',
                                  'categoria_id'], context, '_classic_write')
        if not produto_dados.get('grade_usar'):
            value['flag_grade_unica'] = True
            value['grade'] = False
        produto_grade_ids = produto_dados.get('produto_grade_ids') or []

        # Informa a grade do produto somente se houver apenas uma
        value['grade_id'] = not grade_id and \
                            produto_grade_ids[0] if len(produto_grade_ids) == 1 else grade_id
        value.update({'pis_nat_rec': produto_dados.get('estpro_pis_nat_rec'),
                      'categoria_id': produto_dados.get('categoria_id'),
                      'cofins_nat_rec': produto_dados.get('estpro_cofins_nat_rec')})

        # Valor da base de calculo
        vbc = float(item_dados.get('vprod') or 0) + float(item_dados.get('voutro') or 0) + \
              float(item_dados.get('vfrete') or 0) - float(item_dados.get('vdesc') or 0)

        # Encargos do item
        encargos = [[encargo[0] == 4 and 1 or encargo[0], encargo[1],
                     adjust_m2o(self.resolve_2many_commands(
                         cr, uid, 'encargo_ids', [encargo[0] not in (2, 3, 5) and encargo or
                                                  encargo[1]], context=context)[0])]
                    for encargo in encargo_ids]
        tributacao_encargo_ids = self.pool.get('fiscal.tributacao.encargos').search(
            cr, uid, [('produto_id', '=', produto_id), ('tipo_tributacao', '=', 'E')],
            context=context)
        encargo_ids = map(
            lambda m: m['encargo_id'], self.pool.get('fiscal.tributacao.encargos').read(
                cr, uid, tributacao_encargo_ids, ['encargo_id'], context, '_classic_write'))
        encargo_ids = tributacao_encargo_ids + self.pool.get('fiscal.tributacao.encargos').search(
            cr, uid, [('produto_id', '=', produto_id), ('tipo_tributacao', '=', 'T'),
                      ('encargo_id', 'not in', encargo_ids)], context=context)
        context['super_read_adm_pessoa'] = True
        campos = ['pessoa_tipo', 'admpes_jur_ie', 'admpes_fis_produtor_rural']
        dados_pessoa = self.pool.get('adm.pessoa').read(cr, uid, dest_pessoa_id, campos, context)
        for encargo in self.pool.get('fiscal.tributacao.encargos').read(
                cr, uid, encargo_ids, ['encargo_id'], context, '_classic_write'):
            encargo_id = encargo.get('encargo_id')
            imposto = self.pool.get('fiscal.utils').calcular_imposto(
                cr, uid, produto_id, False, dados_pessoa, 'E',
                float(item_dados.get('vprod')), encargo_ids=[encargo_id], context=context)
            imposto = filter(lambda f: f['encargo_id'] == encargo_id, imposto) or {}
            imposto = imposto and imposto[0] or {}
            cst_dados = imposto and self.pool.get('fiscal.cst').read(
                cr, uid, imposto.get('cst_id'), ['codigo', 'tipo'], context)
            encargo = {'encargo_id': encargo_id, 'cst': cst_dados.get('codigo') or str(),
                       'aliquota': imposto.get('aliquota') or 0, 'vr_base_calculo': vbc,
                       'tributacao_id': imposto.get('tributacao_id') or 0,
                       'vr_encargo': percent(vbc, imposto.setdefault('aliquota', 0))}
            antigo = filter(lambda f: f[2].get('encargo_id') == encargo_id, encargos)

            # Calcula reducao, se houver
            dados_tributacao = self.pool.get('fiscal.tributacao').read(
                cr, uid, imposto.get('tributacao_id'), ['base'], context)
            if dados_tributacao.get('base') < 100:
                encargo['vr_reducao_base'] = vbc - percent(vbc, dados_tributacao.get('base'))
                encargo['vr_base_calculo'] -= encargo['vr_reducao_base']
                encargo['vr_encargo'] = percent(encargo['vr_base_calculo'], imposto.get('aliquota'))
            else:
                encargo['vr_reducao_base'] = 0

            if not antigo:
                encargos += [[0, False, encargo]]
            else:
                if antigo[0][0] in (2, 3, 5):  # Retoma o valor do encargo encontrado (sobreposicao)
                    antigo[0][0] = 1
                    antigo[0][2] = encargo
                else:
                    if not cst_dados.get('tipo') in ['U', 'TU']:
                        encargo['modbcst'] = imposto.get('modbcst') or ''
                        encargo['percentual_mva'] = imposto.get('mva') or 0
                        encargo['aliquota_venda'] = imposto.get('aliquota_venda') or 0
                        encargo['vr_icms_substituicao'] = imposto.get('vr_icms_substituicao') or 0
                        antigo[0][2].update(encargo)

        value['encargo_ids'] = encargos
        value['item_editado'] = True

        return {'value': value}


# TODO: Mover para ´import_nfe_encargos_data´
class mkp_import_nfe_encargos_osv(osv.Model):
    _name = 'mkp.import.nfe.encargos.osv'
    _description = 'Encargos da NFe'
    _rec_name = 'item_id'
    _transient = False
    _columns = {
        'id': fields.integer('id', readonly=True),
        'item_id': fields.many2one('mkp.import.nfe.itens.osv', 'Item', ondelete="cascade"),
        'encargo_id': fields.many2one("fiscal.encargo", 'Encargo', required=True),
        'aliquota': fields.float('Aliquota', digits=(7, 4)),
        'vr_base_calculo': fields.float('Vr base calculo', digits=(15, 2)),
        'vr_encargo': fields.float('Vr encargo', digits=(15, 2)),
        'vr_reducao_base': fields.float('Vr reducao base', digits=(15, 2)),
        'cst': fields.char('CST', size=2, required=False),
        'percentual_mva': fields.float('Percentual mva', digits=(7, 4)),
        'valor_pauta': fields.float('Valor Pauta', digits=(10, 2)),
        'modbcst': fields.selection([
            ('0', 'Preco tabelado'),
            ('1', 'Lista negativa'),
            ('2', 'Lista positiva'),
            ('3', 'Lista neutra'),
            ('4', 'Margem valor agregado'),
            ('5', 'Pauta'),
            ('6', 'Valor da operacao')],
            'Modal.determ. BC do ICMS ST',
            help="Modalidade de determinacao da Base de Calculo do ICMS de "
                 "Substituicao Tributaria", size=1),
        'aliquota_venda': fields.float('Aliquota para venda', digits=(5, 2),
                                       help='Aliquota de ICMS na venda para verificacao de calculo '
                                            'do ICMS/ST na compra.'),
        'tributacao_id': fields.many2one("fiscal.tributacao", 'Tributacao Entrada',
                                         ondelete="restrict", required=False,
                                         domain="[('encargo_id','=',encargo_id)]"),
        'cadastrar_tributacao': fields.boolean('Cadastrar Tributacao'),
        'vr_icms_substituicao': fields.float('Vr ICMS Substituicao', digits=(15, 2)),
        'vr_base_icms_substituicao': fields.float('Vr Base ICMS Substituicao', digits=(15, 2)),
        'vr_difal_uf_origem': fields.float(digits=(15, 2), string='Valor DIFAL UF origem'),
        'vr_difal_uf_destino': fields.float(digits=(15, 2), string='Valor DIFAL UF destino'),
        'vr_fcp': fields.float(digits=(15, 2), string='FCP'),
        'vr_fcp_st': fields.float(digits=(15, 2), string='FCP/ST'),
        'vr_icms_desonerado': fields.float(digits=(15, 2), string='Valor ICMS desonerado')
    }
    _defaults = {
        'cadastrar_tributacao': False,
    }

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'encargo_id' in vals:
            if isinstance(vals['encargo_id'], (list, tuple)):
                vals['encargo_id'] = vals['encargo_id'][0]
        if 'tributacao_id' in vals:
            if isinstance(vals['tributacao_id'], (list, tuple)):
                vals['tributacao_id'] = vals['tributacao_id'][0]
        if ids and isinstance(ids, list):
            ids = ids[0]

        # Se a aliquota da tributacao for igual a selecionada pelo usuario, serah mantido o valor
        # total do encargo que veio no XML.
        if 'tributacao_id' in vals and ids:
            if vals['tributacao_id']:
                browse_encargos_tributacao = self.browse(cr, uid, ids, context)
                if not context.get('alteracao_outros_itens'):
                    codigo_cst = browse_encargos_tributacao.cst or ''
                    obj_import_nfe = self.pool.get('mkp.import.nfe.osv')
                    nf_import_id = browse_encargos_tributacao.item_id.nf_id.id
                    prod_ids = obj_import_nfe.read(
                        cr, uid, nf_import_id, ['prod_ids'], context=context)
                    if isinstance(prod_ids, list):
                        prod_ids = prod_ids[0]
                    if isinstance(prod_ids, dict):
                        prod_ids = prod_ids['prod_ids']
                    encargo_tributacao_ids = self.search(
                        cr, uid, [('encargo_id', '=', browse_encargos_tributacao.encargo_id.id),
                                  ('cst', '=', codigo_cst), ('item_id', 'in', prod_ids)],
                        context=context)
                    brw_tributacoes_itens = self.browse(cr, uid, encargo_tributacao_ids)
                    for x in brw_tributacoes_itens:
                        if not x.tributacao_id:
                            self.write(
                                cr, uid, x.id,
                                {'tributacao_id': vals['tributacao_id'], 'cst': codigo_cst},
                                context={'alteracao_outros_itens': True})

        return super(mkp_import_nfe_encargos_osv, self).write(
            cr, uid, ids, vals, context=context)


    def impnfe_onchange_tributacao(self, cr, uid, ids, tributacao_id, vprod, vfrete, vseg, voutro,
                                   vdesc, aliquota, context=None):
        context = context and context.copy() or {}
        vals = {}
        if tributacao_id:
            vals.update(self.calcula_tributacao_item(
                cr, uid, tributacao_id, float(vprod), float(vfrete), float(vseg), float(voutro),
                float(vdesc), aliquota, context))

        return {'value': vals}

    def onchange_encargo_id(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {'value': {'cst': ''}}

    def onchange_cst(self, cr, uid, ids, encargo_id, cst, context=None):
        """
        Ao alterar a informacao do CST o campo tributacao_id eh limpo
        """
        context = context and context.copy() or {}
        vals = {'tributacao_id': None}
        if cst == '60':
            vals['aliquota'] = 0.00
        if not encargo_id or not cst:
            return {'value': vals}
        cst_ids = self.pool.get('fiscal.cst').search(
            cr, uid, [('encargo_id', '=', encargo_id), ('fiscst_tipo_movimento', 'in', ('A', 'E')),
                      ('codigo', '=', cst)], context=context)
        if not cst_ids:
            vals['cst'] = ''
            return {'value': vals,
                    'warning': {'title': _('Atencao!'),
                                'message': _('Nao existe cst de entrada com o codigo informado')}}
        cst_ids = cst_ids[0]
        tributacao_ids = self.pool.get('fiscal.tributacao').search(
            cr, uid, [('cst_id', '=', cst_ids), ('encargo_id', '=', encargo_id)], context=context)
        if len(tributacao_ids) == 1:
            vals['tributacao_id'] = tributacao_ids[0]

        return {'value': vals}

    def impnfe_onchange_aliquota(self, cr, user, ids, base, aliquota, context=None):
        """
        Calcula o valor do encargo em funcao dos valores da base de calculo e aliquota.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da tributacao
        :param base: valor da base de calculo
        :param aliquota: valor % da aliquota
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type base: float
        :type aliquota: float
        :type context: dict
        :return: o valor do encargo em funcao da base de calculo e aliquota
        :rtype: dict
        """
        context = context and context.copy() or {}

        return {'value': {'vr_encargo': percent(base, aliquota)}}

    def impnfe_onchange_base_calculo(self, cr, user, ids, vr_prod, vr_outro, vr_frete, vr_desc,
                                     aliquota, tributacao_id, context=None):
        """
        Ajusta o valor do encargo em funcao do valor da base de calculo.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da tributacao
        :param vr_prod: o valor do item
        :param vr_outro: o valor informado para outras despesas
        :param vr_frete: o valor do frete
        :param vr_desc: o valor do desconto
        :param aliquota: o valor da aliquota
        :param tributacao_id: o ID da tributacao do encargo
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type vr_prod: str
        :type vr_outro: str
        :type vr_frete: str
        :type vr_desc: str
        :type aliquota: float
        :type tributacao_id: long
        :type context: dict
        :return: o valor ajustado do encargo e da reducao de base (se houver)
        :rtype: dict
        """
        context = context and context.copy() or {}
        vr_base = float(vr_prod) + float(vr_outro) + float(vr_frete) - float(vr_desc)
        dados_tributacao = self.pool.get('fiscal.tributacao').read(
            cr, user, tributacao_id, ['base'], context)
        if dados_tributacao.get('base') < 100:
            vr_reducao_base = vr_base - percent(vr_base, dados_tributacao.get('base'))
            vr_base -= vr_reducao_base
            vr_encargo = percent(vr_base, aliquota)
        else:
            vr_reducao_base = 0
            vr_encargo = percent(vr_base, aliquota)

        return {'value': {'vr_encargo': vr_encargo, 'vr_reducao_base': vr_reducao_base}}


class mkp_import_nfe_parcelas(osv.Model):
    """
    Condicoes de pagamento importadas
    """

    _name = 'mkp.import.nfe.parcelas'
    _order = 'vencimento'
    _transient = False
    _columns = {
        'valor': fields.float('Valor', digits=(14, 2)),
        'vencimento': fields.date('Vencimento'),
        'nf_id': fields.many2one('mkp.import.nfe.osv', 'NF Capa', ondelete="cascade",
                                 select=True)
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
