# -*- encoding: utf-8 -*-
import re
from urllib import urlencode
import json
import requests
from openerp import pooler
from openerp.tools.translate import _
from SigUtils.geralogs import geraLogs
from openerp.tools import config


"""
Cartoes para testes
Diners	3612 673574 0137
Hipercard	6062 8267 8627 6634
Elo	5067 2686 5051 7446
Cartão MercadoLivre	5300 3272 2151 3800
"""

class Meli(object):
    def logged_client(f):
        """Decorator for User Credentials checking."""
        def inner(self,*args, **kargs):
            if ((self.client_id is None) or (self.client_secret is None)):
                raise Exception(_("Nao posso acessar o M.L. parametros do M.L nao informados!"))
            return f(self, *args, **kargs)
        return inner

    def __init__(self, cr=False, uid=False, conta_id=False, company_id=False):
        self.API_ROOT_URL = 'https://api.mercadolibre.com'
        self.SDK_VERSION = 'SIG.1.01'
        self.AUTH_URL = 'https://auth.mercadolibre.com/authorization'
        self.AUTH_URL = "http://auth.mercadolibre.com/authorization"
        self.OAUTH_URL = '/oauth/token'
        self.cr = cr
        self.uid = uid
        self.client_id = ''
        self.client_secret = ''
        self.access_token = ''
        self.refresh_token = ''
        self.code = ''
        self.redirect_url = ''
        self.seller = ''
        self.habilitar_log = False
        self.conta_id = ''
        self.comunicar = True
        if cr:
            _poolget = pooler.get_pool(cr.dbname).get
            mkp_ids = _poolget('mkp.marketplaces').search(cr, uid, [('mkpmkp_codigo', '=', 'MerLiv')])
            if mkp_ids:
                mkp_ids = mkp_ids[0]
                dados_marketplace = _poolget('mkp.marketplaces').read(cr, uid, mkp_ids, [])
                if dados_marketplace.get('mkpmkp_parar_comunicacao'):
                    self.comunicar = False
                if not conta_id:
                    conta_id = _poolget('mkp.contas').search(cr, uid, [('mkpcta_marketplace_id', '=', mkp_ids)])
                    if conta_id:
                        conta_id = conta_id[0]
                if conta_id:
                    self.conta_id = conta_id
                    dados_conta = _poolget('mkp.contas').read(cr, uid, conta_id, [])
                    if dados_conta.get('mkpcta_parar_comunicacao'):
                        self.comunicar = False
                    self.client_id = dados_conta.get('mkpcta_ml_app_id')
                    self.client_secret = dados_conta.get('mkpcta_ml_secret_key')
                    self.access_token = dados_conta.get('mkpcta_ml_token')
                    self.refresh_token = dados_conta.get('mkpcta_ml_refresh_token')
                    self.redirect_url = dados_conta.get('mkpcta_ml_redirect_url')
                    self.code = dados_conta.get('mkpcta_ml_code')
                    self.seller = dados_conta.get('mkpcta_ml_user_id')
                    self.habilitar_log = dados_marketplace.get('mkpmkp_habilita_log')

    def _get_arq_log(self):
        _arq_log = 'mkp_merliv_' + self.cr.dbname
        return _arq_log

    def _gerar_logs(self, mens):
        _diretorio = config.get('logfile')
        if _diretorio:
            _lista = _diretorio.split('/')
            _lista.pop(len(_lista) -1)
            _diretorio = '/'.join(_lista)
        else:
            _diretorio = '/var/log/openerp'
        _arq_log = self._get_arq_log()
        try:
            geraLogs(_arq_log, mens, 'S', _diretorio)
        except:
            try:
                geraLogs(_arq_log, mens.encode('ASCII', 'ignore'), 'S', _diretorio)
            except:
                geraLogs(_arq_log, 'Nao consegui registrar o log', 'S', _diretorio)
        return True


    #AUTH METHODS
    def auth_url(self):
        params = {
            'client_id': self.client_id,
            'response_type': 'code',
            'redirect_uri': self.redirect_url
        }
        url = self.AUTH_URL + '?' + urlencode(params)
        return url

    def authorize(self):
        params = {
            'grant_type': 'authorization_code',
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'code': self.code,
            'redirect_uri': self.redirect_url
        }
        headers = {
            'Accept': 'application/json',
            'User-Agent': self.SDK_VERSION,
            'Content-type': 'application/json'
        }
        uri = self.make_path(self.OAUTH_URL)
        if self.habilitar_log:
            self._gerar_logs('Meli.Authorize-> %s -> %s' % (uri, params))

        response = requests.post(uri, params=urlencode(params), headers=headers)
        if self.habilitar_log:
            if response.status_code == 200:
                self._gerar_logs('Meli.resposta--> %s ' % response.content)
            else:
                self._gerar_logs('Meli.authorize--> Code: %s-%s ' % (response.status_code, response.content))

        if response.status_code == requests.codes.ok:
            response_info = response.json()
            self.access_token = response_info['access_token']
            if 'refresh_token' in response_info:
                self.refresh_token = response_info['refresh_token']
            else:
                self.refresh_token = ''  # offline_access not set up
            return self.access_token
        else:
            response.raise_for_status()

    def get_refresh_token(self):
        if self.refresh_token:
            params = {
                'grant_type': 'refresh_token',
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'refresh_token': self.refresh_token
            }
            headers = {
                'Accept': 'application/json',
                'User-Agent': self.SDK_VERSION,
                'Content-type':'application/json'
            }
            uri = self.make_path(self.OAUTH_URL)

            if self.habilitar_log:
                self._gerar_logs('Meli.get_refresh_token-> %s -> %s' % (uri, params))
            response = requests.post(uri, params=urlencode(params), headers=headers, data=params)
            if self.habilitar_log:
                if response.status_code == 200:
                    self._gerar_logs('Meli.resposta--> %s ' % response.content)
                else:
                    msg = response
                    try:
                        msg = '%s - %s' % (response, response.content)
                    except:
                        pass
                    self._gerar_logs('Meli.get_refresh_token--> %s ' % msg)

            if response.status_code == requests.codes.ok:
                response_info = response.json()
                self.access_token = response_info['access_token']
                self.refresh_token = response_info['refresh_token']
                if self.cr:
                    vals = {
                        'mkpcta_ml_token': self.access_token,
                        'mkpcta_ml_refresh_token': self.refresh_token
                    }
                    pooler.get_pool(self.cr.dbname).get('mkp.contas').write(self.cr, self.uid, self.conta_id, vals)
                return self.access_token, self.refresh_token
            else:
                return (False, False)
        else:
            self._gerar_logs('Meli.resposta--> refresh token nao cadastrado!')
            return (False, False)

    # REQUEST METHODS
    @logged_client
    def get(self, path, params={}, headers={}):
        def comunicar():
            i = 0
            while True:
                i += 1
                try:
                    if self.habilitar_log:
                        self._gerar_logs('Meli.get-> %s -> %s' % (uri, params))
                    response = requests.get(uri, params=urlencode(params), headers=headers)
                    if self.habilitar_log:
                        if response.status_code == 200:
                            self._gerar_logs('Meli.resposta--> %s ' % response.content)
                        else:
                            self._gerar_logs('Meli.get--> Uri:%s Params:%s Resposta:%s-%s ' % (uri,
                                                                                               params,
                                                                                               response.status_code,
                                                                                               response.content))
                    break
                except Exception as e:
                    print '' \
                          '' \
                          '' \
                          'erro no while ->', e
            return response

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"message":"Comunicacao parada"}'
            return resp

        headers.update(
            {
                'Accept': 'application/json',
                'User-Agent': self.SDK_VERSION,
                'Content-type': 'application/json',
                'Authorization': 'Bearer %s' % self.access_token,
            }
        )
        uri = self.make_path(path)
        response = comunicar()
        if response.status_code == 401:
            if response.content:
                _res = eval(response.content)
                if _res.get('message') == 'invalid_token' or _res.get('message') == 'expired_token':
                    _access_token, refresh_token = self.get_refresh_token()
                    if _access_token:
                        headers['Authorization'] = 'Bearer %s' % self.access_token
                        response = comunicar()
        return response

    @logged_client
    def post(self, path, body=None, params={}, headers={}):
        def comunicar():
            if self.habilitar_log:
                self._gerar_logs('Meli.post-> %s -> %s -> %s' % (uri, params, body))
            response = requests.post(uri, data=body, params=urlencode(params), headers=headers)
            if self.habilitar_log:
                if response.status_code in (200, 201):
                    self._gerar_logs('Meli.resposta--> %s ' % response.content)
                else:
                    self._gerar_logs('Meli.post--> Uri:%s Params:%s Body:%s Resposta:%s-%s' % (uri,
                                                                                               params,
                                                                                               body,
                                                                                               response.status_code,
                                                                                               response.content))
            return response

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"message":"Comunicacao parada"}'
            return resp

        if not headers.get('Accept'):
            headers['Accept'] = 'application/json'
        if not headers.get('User-Agent'):
            headers['User-Agent'] = self.SDK_VERSION
        if not headers.get('Content-Type'):
            headers['Content-type'] = 'application/json'
        headers['Authorization'] = 'Bearer %s' % self.access_token

        uri = self.make_path(path)
        if not uri.endswith('invoice_data'):
            if body:
                body = json.dumps(body)

        response = comunicar()
        if response.status_code == 401:
            _res = eval(response.content)
            if _res.get('message') == 'invalid_token':
                self.get_refresh_token()
                headers['Authorization'] = 'Bearer %s' % self.access_token
                response = comunicar()
        return response

    @logged_client
    def upload(self, path, files, params={}):
        def comunicar():
            if self.habilitar_log:
                self._gerar_logs('Meli.upload-> %s -> %s' % (uri, params))
            response = requests.post(uri, files=files, params=urlencode(params), headers=headers)
            if self.habilitar_log:
                if response.status_code == 200:
                    self._gerar_logs('Meli.resposta--> %s ' % response.content)
                else:
                    self._gerar_logs('Meli.upload--> Uri:%s Params:%s Resposta:%s ' % (uri, params, response.content))
            return response

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"message":"Comunicacao parada"}'
            return resp

        headers = {
            'Accept': 'application/json',
            'User-Agent': self.SDK_VERSION,
            'Content-type': 'multipart/form-data',
            'Authorization': 'Bearer %s' % self.access_token,
        }
        response = self.options('users/me', {'access_token': self.access_token})
        if response.status_code == 401:
            _res = eval(response.content)
            if _res.get('message') == 'invalid_token':
                self.get_refresh_token()
                headers['Authorization'] = 'Bearer %s' % self.access_token
                response = comunicar()
        headers = {
            'Authorization': 'Bearer %s' % self.access_token,
        }
        uri = self.make_path(path)
        response = comunicar()
        return response

    @logged_client
    def put(self, path, body=None, params={}):
        def comunicar():
            if self.habilitar_log:
                self._gerar_logs('Meli.put-> %s -> %s -> %s' % (uri, params, _body))
            response = requests.put(uri, data=_body, params=urlencode(params), headers=headers)
            if self.habilitar_log:
                if response.status_code == 200:
                    self._gerar_logs('Meli.resposta--> %s ' % response.content)
                else:
                    self._gerar_logs('Meli.put--> Uri:%s Params:%s Body:%s Resposta:%s-%s' % (uri,
                                                                                              params,
                                                                                              _body,
                                                                                              response.status_code,
                                                                                              response.content))
            return response

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"message":"Comunicacao parada"}'
            return resp

        headers = {
            'Accept': 'application/json',
            'User-Agent': self.SDK_VERSION,
            'Content-type': 'application/json',
            'Authorization': 'Bearer %s' % self.access_token,
        }
        uri = self.make_path(path)
        _body = json.dumps(body)
        response = comunicar()
        if response.status_code == 401:
            _res = eval(response.content)
            if _res.get('message') == 'invalid_token':
                self.get_refresh_token()
                headers['Authorization'] = 'Bearer %s' % self.access_token
                response = comunicar()
        return response

    def delete(self, path, params={}):

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"message":"Comunicacao parada"}'
            return resp

        headers = {
            'Accept': 'application/json',
            'User-Agent': self.SDK_VERSION,
            'Content-type': 'application/json',
            'Authorization': 'Bearer %s' % self.access_token,
        }
        uri = self.make_path(path)
        if self.habilitar_log:
            self._gerar_logs('Meli.delete-> %s -> %s' % (uri, params))
        response = requests.delete(uri, params=params, headers=headers)
        if self.habilitar_log:
            if response.status_code == 200:
                self._gerar_logs('Meli.resposta--> %s ' % response.content)
            else:
                self._gerar_logs('Meli.delete--> Uri:%s Params:%s Resposta:%s' % (uri, params, response.content))
        return response

    def options(self, path, params={}):
        headers = {
            'Accept': 'application/json',
            'User-Agent': self.SDK_VERSION,
            'Content-type': 'application/json',
            'Authorization': 'Bearer %s' % self.access_token,
        }
        uri = self.make_path(path)
        if self.habilitar_log:
            self._gerar_logs('Meli.options-> %s -> %s' % (uri, params))
        response = requests.options(uri, params=urlencode(params), headers=headers)
        if self.habilitar_log:
            if response.status_code == 200:
                self._gerar_logs('Meli.resposta--> %s ' % response.content)
            else:
                self._gerar_logs('Meli.options--> Uri:%s Params:%s Resposta:%s ' % (uri, params, response.content))
        return response

    def make_path(self, path, params={}):
        # Making Path and add a leading / if not exist
        if not (re.search("^http", path)):
            if not (re.search("^\/", path)):
                path = "/" + path
            path = self.API_ROOT_URL + path
        if params:
            path = path + "?" + urlencode(params)
        return path

