# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import logging


class mkpml_pagamentos(osv.Model):
    """
    Classe para receber as notificacoes de pagamento do mercado livre
    prefixo mkpmlpag_
    """

    from funcoes import  _mkpmlpag_get_dados
    _name = "mkpml.pagamentos"
    _logger = logging.getLogger(_name)
    _columns = {
        'id': fields.integer('id', readonly=True),
        'mkpmlpag_marketplace_id': fields.many2one('mkp.marketplaces', 'Marketplace', required=True, select=True),
        'mkpmlpag_conta_id': fields.many2one('mkp.contas', 'Conta', required=True, select=True,
                                             domain="[('mkpcta_marketplace_id', '=', mkpmlpag_marketplace_id)]"),
        'write_date': fields.datetime('Ultima alteracao', readonly=True),
        'mkpmlpag_payment_code': fields.char('Codigo do pagamento', size=12, select=True,
                                            help='Numero do pagamento no Mercado Pago'),
        'mkpmlpag_ml_order': fields.char('Nro pedido ML', size=12, select=True,
                                            help='Numero do pedido no mercado livre'),
        'mkpmlpag_valor_itens': fields.float('Total pago', digits=(8,2)),
        'mkpmlpag_frete_recebido': fields.float('Frete recebido', digits=(8,2)),
        'mkpmlpag_frete_custo': fields.float('Custo do frete', digits=(8, 2)),
        'mkpmlpag_tarifa_ml': fields.float('Tarifa do M.L.', digits=(8, 2)),
        'mkpmlpag_total_liquido': fields.float('Total liquido', digits=(8, 2)),
        'mkpmlpag_pedido_id': fields.many2one('estoquepedidos.pedido.capa', 'Pedido', select=True),
        'mkpmlpag_status_pedido': fields.function(_mkpmlpag_get_dados, method=True, string='Estado', multi="dados",
                                             type='char', size=15, store=False),

        'mkpmlpag_item_titulo_id': fields.function(_mkpmlpag_get_dados, method=True, string='Titulo de venda', multi="dados",
                                             type='many2one', relation='financeiro.titulo', store=False),
        'mkpmlpag_item_titulo_valor': fields.function(_mkpmlpag_get_dados, method=True, string='Vl.Tit.Item', multi="dados",
                                             type='float', store=False),
        'mkpmlpag_item_titulo_state': fields.function(_mkpmlpag_get_dados, method=True, string='Estado', multi="dados",
                                             type='char', store=False),
        'mkpmlpag_frete_rec_titulo_id': fields.function(_mkpmlpag_get_dados, method=True, string='Titulo frete a receber', multi="dados",
                                             type='many2one', relation='financeiro.titulo', store=False),
        'mkpmlpag_frete_rec_titulo_valor': fields.function(_mkpmlpag_get_dados, method=True, string='Vl.Tit.Frete Rec.', multi="dados",
                                             type='float', store=False),
        'mkpmlpag_frete_rec_titulo_state': fields.function(_mkpmlpag_get_dados, method=True, string='Estado', multi="dados",
                                             type='char', store=False),
        'mkpmlpag_frete_pag_titulo_id': fields.function(_mkpmlpag_get_dados, method=True, string='Titulo custo do frete', multi="dados",
                                             type='many2one', relation='financeiro.titulo', store=False),
        'mkpmlpag_frete_pag_titulo_valor': fields.function(_mkpmlpag_get_dados, method=True, string='Vl.Tit.Frete Pag', multi="dados",
                                             type='float', store=False),
        'mkpmlpag_frete_pag_titulo_state': fields.function(_mkpmlpag_get_dados, method=True, string='Estado', multi="dados",
                                             type='char', store=False),
        'mkpmlpag_tarifa_titulo_id': fields.function(_mkpmlpag_get_dados, method=True, string='Titulo tarifa M.L.', multi="dados",
                                             type='many2one', relation='financeiro.titulo', store=False),
        'mkpmlpag_tarifa_titulo_valor': fields.function(_mkpmlpag_get_dados, method=True, string='Vl.Tit.Tarifa', multi="dados",
                                                   type='float', store=False),
        'mkpmlpag_tarifa_titulo_state': fields.function(_mkpmlpag_get_dados, method=True, string='Estado', multi="dados",
                                                        type='char', store=False),
    }


    _sql_constraints = [
        ('codigo_uniq', 'unique(mkpmlpag_payment_code)', 'Codigo de pagamento ja existe!'),
    ]
    _order = 'write_date desc'
