# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################
from openerp.osv import osv
from openerp.tools.translate import _
import json
import logging
import requests
from ..olist import Olist
from unicodedata import normalize


# Canais de venda com lista de lojas
_canal = {
    'amazon': [],
    'zoom': [],
    'madeiramadeira': [],
    'mercadolivre': [],
    'b2w': [],
    # 'b2w': ['americanas', 'shoptime', 'submarino'],
    'cnova': [],
    'magazineluiza': [],
    'carrefour': [],
    'clickspace': [],
    'consultaremedios': [],
    'ecommerce': [],
}

def get_prices(valor):
    precos = []
    for key in _canal.keys():
        _d = {
            "channel_slug": key,
            "currency": "BRL",
            "minimum_quantity": 1,
            "offer": valor,
            "value": valor
        }
        _stp = []
        for _loja in _canal.get(key):
            _stp.append(
                {
                    "store_slug": _loja,
                    "offer": valor,
                    "value": valor,
                }
            )
        if _stp:
            _d['store_prices'] = _stp
        precos.append(_d)
    return precos

# Classe de alto nivel para comunicação com o Mercado Livre
class mkp_Olist(osv.Model):
    _name = 'mkp.olist'
    _auto = False
    _tem_erro = False
    _mensagem_erro = ''
    _logger = logging.getLogger(_name)

    def mkpOlist_get_seller_id(self, cr, uid, conta_id, context=None):
        if conta_id:
            ol_con = Olist(cr, uid, conta_id)
            _url = 'sellers/'
            response = ol_con.olist_comunicar('get', _url)
            if response.status_code > 399:
                return {
                    'status': False,
                    'mensagem': json.loads(response.content).get('message')
                }
            res = json.loads(response.content)
            _seller_id = res.get('results')[0].get('id')
            return _seller_id
        return False

    def mkpOlist_autenticar(self, cr, uid, conta_id):
        olist = Olist(cr, uid, conta_id)
        _url = olist.auth_url()
        return _url

    def mkpOlist_get_anuncio(self, cr, uid, conta_id, codigo_anuncio, context=None):
        context = context and context.copy() or {}
        ol_con = Olist(cr, uid, conta_id)
        if codigo_anuncio:
            _url = 'seller-products/' + str(codigo_anuncio)
            response = ol_con.olist_comunicar('get', _url)
            if response.status_code > 399:
                return {
                    'status': False,
                    'mensagem': json.loads(response.content).get('message')
                }
            dados_anuncio_Olist = json.loads(response.content)
            return dados_anuncio_Olist
        return False

    def mkpOlist_set_shipments(self, cr, uid, conta_id, seller_id, package_id, context=None):
        ol_con = Olist(cr, uid, conta_id)
        vals = {
            'seller': str(seller_id),
            'packages': [str(package_id)],
        }
        resource = '/shipments/'
        response = ol_con.olist_comunicar('POST', resource, dados=vals)
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('message')}
        try:
            res = json.loads(response.content)
        except:
            return {'status': False, 'mensagem': response.content}
        return {'status': True, 'mensagem': '', 'shipments': res}

    def mkpOlist_get_packages(self, cr, uid, id_pacote, conta_id, context=None):
        if not conta_id:
            return {'status': False, 'mensagem': 'sem conta_id'}
        ol_con = Olist(cr, uid, conta_id)
        resource = '/packages/%s' % id_pacote
        response = ol_con.olist_comunicar('get', resource)
        if response.status_code > 399:
            return {
                'status': False,
                'mensagem': json.loads(response.content).get('message')
            }
        res = json.loads(response.content)
        return {'status': True, 'mensagem': '', 'packages': res}

    def mkpOlist_get_shippment(self, cr, uid, id_shippment, conta_id, context=None):
        if not conta_id:
            return {'status': False, 'mensagem': 'sem conta_id'}
        ol_con = Olist(cr, uid, conta_id)
        resource = '/shipments/%s' % id_shippment
        response = ol_con.olist_comunicar('get', resource)
        if response.status_code > 399:
            return {
                'status': False,
                'mensagem': json.loads(response.content).get('message')
            }
        res = json.loads(response.content)
        return {'status': True, 'mensagem': '', 'packages': res}

    def _mkpOlist_get_emb_produto(self, brw_anuncio, context=None):
        _peso = 0.3
        _altura = 6
        _largura = 11
        _comprimento = 16
        for brw_embalagem in brw_anuncio.mkpanu_produto_id.produto_embalagem_ids:  # pegar APENAS A PRIMEIRA
            _peso = brw_embalagem.peso_bruto
            _altura = brw_embalagem.altura
            _largura = brw_embalagem.largura
            _comprimento = brw_embalagem.profundidade
            break
        _peso = _peso * 1000
        _max = brw_anuncio.mkpanu_qtd_maxima or 1
        _emb_produto = {
            "height_unit": "cm",
            "height_value": _altura,
            "length_unit": "cm",
            "length_value": _comprimento,
            "weight_unit": "g",
            "weight_value": _peso,
            "width_unit": "cm",
            "width_value": _largura,
        }
        _emb_package = _emb_produto.copy()
        _emb_package["capacity"] = _max
        if _peso < 50:
            _emb_package['weight_value'] = 50
        return _emb_produto, _emb_package

    def mkpOlist_set_anuncio(self, cr, uid, brw_anuncio, context=None):
        def __atualizar_variacoes(sku):
            vals_var = {
                'mkpanuv_anuncio_id': brw_anuncio.id,
                'mkpanuv_grade_id': brw_grade.id,
                'mkpanuv_id_no_mkp': sku
            }
            args = [('mkpanuv_anuncio_id', '=', brw_anuncio.id),
                    ('mkpanuv_grade_id', '=', brw_grade.id)]
            var_ids = _spg('mkp.anuncios.var').search(cr, uid, args, context=context)
            if var_ids:
                return  # Já existe
            else:
                _spg('mkp.anuncios.var').create(cr, uid, vals_var, context=context)
            return

        def __atualizar_por_gtin():
            _url = 'seller-products/'
            response = ol_con.olist_comunicar('GET', _url)
            if response.status_code < 400:
                res = json.loads(response.content)
                for _item in res.get('results'):
                    if _item.get('gtin') != _gtin:
                        continue
                    _sku = _item.get('sku')
                    _group = _item.get('group')
                    _vals = {
                        'mkpanu_codigo_anuncio': _group,
                        'mkpanu_necessita_atualizar': False
                    }
                    self.pool.get('mkp.anuncios').write(cr, uid, brw_anuncio.id, _vals, context=context)
                    if brw_anuncio.mkpanu_produto_id.grade_usar:
                        __atualizar_variacoes(_sku)
                    break
            return

        context = context and context.copy() or {}
        _spg = self.pool.get
        ol_con = Olist(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        _picture_produts = []
        _img_idx = 0
        _ajustar_imagens = False
        _products = []
        _atributos = []
        _marca = ''
        for brw_atributos in brw_anuncio.mkpanu_produto_id.estpro_atributo_ids:
            if brw_atributos.estatp_tem_unidade:
                if not brw_atributos.estatp_atributo_unidade_id.name:
                    return {
                        'status': False,
                        'mensagem': _('Atributo "%s" sem unidade cadastrada!') % brw_atributos.estatp_atributo_id.name
                    }
                _at_vl = '%s %s' % (brw_atributos.estatp_atributo_valor, brw_atributos.estatp_atributo_unidade_id.name)
            _at_vl = brw_atributos.estatp_atributo_valor
            if not _at_vl:
                continue
            _attr = {
                'attribute_name': str(brw_atributos.estatp_atributo_id.name),
                'attribute_value': str(_at_vl),
                # 'category_attribute_id': None,
            }
            if brw_atributos.estatp_atributo_id.name == 'Marca':
                _marca = str(brw_atributos.estatp_atributo_valor)
            _atributos.append(_attr)

        _controle = {}
        _group = ''
        vals_anuncio = {}
        for brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
            _sku = brw_anuncio.mkpanu_produto_id.codigo + '-' + brw_grade.grade
            _controle[_sku] = {'fotos': {}}
            _controle[_sku]['grade_id'] = brw_grade.id
            _controle[_sku]['grade'] = brw_grade.grade
            produto = {}
            _gtin = False
            for brw_cbar in brw_grade.codigo_barra_ids:
                if brw_cbar.autcba_tipo_barra == '13' and brw_cbar.active:
                    if not _gtin:
                        _gtin = brw_cbar.codigo_barra
                    if brw_cbar.emite_etiqueta:
                        _gtin = brw_cbar.codigo_barra
                        break
            if not _gtin:  # Não enviar sem gtin
                continue
            _fotos = []
            _seq = 0
            for brw_fotos in brw_grade.produto_id.estpro_imagem_ids:
                if brw_fotos.estimg_grade_id and brw_fotos.estimg_grade_id.id != brw_grade.id:
                    continue
                _foto_id = brw_fotos.id
                _produto_id = brw_fotos.estimg_produto_id.id
                _mkp_id = brw_anuncio.mkpanu_marketplace_id.id
                context['sig_force_company'] = True
                _produto_mkp_id = _spg('mkp.produto.marketplaces').search(cr, uid, [
                    ('mkpromkp_produto_id', '=', _produto_id),
                    ('mkpromkp_marketplace_id', '=', _mkp_id),
                    ('company_id', '=', brw_anuncio.company_id.id)
                ], context=context)
                context.pop('sig_force_company')
                if not _produto_mkp_id:
                    return {
                        'status': False,
                        'mensagem': _('Item nao cadastrado no produto maketplace!')
                    }
                _produto_mkp_id = _produto_mkp_id[0]
                _prod_mkp_img_ids = _spg('mkp.produto.marketplace.imagens').search(cr, uid, [
                    ('mkprmkimg_produto_mkp_id', '=', _produto_mkp_id ),
                    ('mkprmkimg_imagem_produto_id', '=', _foto_id)
                ])
                if not _prod_mkp_img_ids:
                    continue
                _dados_prod_mkp_img = _spg('mkp.produto.marketplace.imagens').read(cr, uid,
                                                                                   _prod_mkp_img_ids[0],
                                                                                   ['mkprmkimg_endereco_mkp'],
                                                                                   context=context)
                _fotos.append({'order': _seq, 'url': str(_dados_prod_mkp_img.get('mkprmkimg_endereco_mkp'))})
                _controle[_sku]['fotos'][_seq] = _prod_mkp_img_ids[0]
                _seq += 1
            _emb_produto, _emb_package = self._mkpOlist_get_emb_produto(brw_anuncio, context=context)
            _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
            _prices = get_prices(_valor)
            _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
            context['company_id'] = brw_anuncio.company_id.id
            _x = _spg('estoque.produto.grade').estpgr_get_saldo(cr, uid, [brw_grade.id], 'D', _local, context=context)
            _saldo = int(_x.get(brw_grade.id) / brw_anuncio.mkpanu_quantidade)
            if _saldo < 0:
                _saldo = 0
            if brw_anuncio.mkpanu_estado == 'P':
                _saldo = 0
            produto['attributes'] = _atributos
            produto['availability_days'] = 0
            produto['brand'] = _marca
            produto['categories'] = []
            produto['description'] = normalize('NFKD', brw_anuncio.mkpanu_descricao_texto).encode('ASCII', 'ignore')
            produto['gtin'] = str(_gtin)
            produto['name'] = normalize('NFKD', brw_anuncio.mkpanu_title).encode('ASCII', 'ignore')
            produto['origin'] = 'SigERP'
            produto['package_measures'] = [_emb_package]
            produto['photos'] = _fotos
            produto['prices'] = _prices
            produto['product_code'] = str(_sku)
            produto['product_measures'] = [_emb_produto]
            produto['seller_id'] = str(brw_anuncio.mkpanu_conta_id.mkpcta_ol_seller_id)
            produto['stock'] = [{
                'availability_days': 0,
                'quantity': _saldo
            }]
            produto['tags'] = []
            if brw_anuncio.mkpanu_produto_id.grade_usar:
                if _group:
                    produto['group'] = _group

            _codigo_anuncio = brw_anuncio.mkpanu_codigo_anuncio
            if brw_anuncio.mkpanu_produto_id.grade_usar:
                _codigo_anuncio = ''
                args = [('mkpanuv_anuncio_id', '=', brw_anuncio.id),
                        ('mkpanuv_grade_id', '=', brw_grade.id)]
                var_ids = _spg('mkp.anuncios.var').search(cr, uid, args, context=context)
                if var_ids:
                    dados_var = _spg('mkp.anuncios.var').read(cr, uid, var_ids[0], ['mkpanuv_id_no_mkp'], context=context)
                    if dados_var:
                        _codigo_anuncio = dados_var.get('mkpanuv_id_no_mkp')

            if _codigo_anuncio:
                """ removendo campos que nao podem ser alterados """
                produto.pop('gtin')
                produto.pop('product_code')
                produto.pop('seller_id')

                _url = 'seller-products/%s' % brw_anuncio.mkpanu_codigo_anuncio
                response = ol_con.olist_comunicar('patch', _url, dados=produto)
                if response.status_code > 399:
                    _json = json.loads(response.content)
                    _mens = str(_json.get('detail')) + '\n'
                    return {
                        'status': False,
                        'mensagem': _mens
                    }
                res = json.loads(response.content)
                _group = res.get('group')
                vals_anuncio = {
                    'mkpanu_necessita_atualizar': False,
                    'mkpanu_estado_mkp': res.get('status'),
                    'mkpanu_anuncio_endereco': 'https://app.olist.com/products/edit/%s' % _codigo_anuncio
                }
                if brw_anuncio.mkpanu_produto_id.grade_usar:
                    vals_anuncio['mkpanu_codigo_anuncio'] = _group
                    __atualizar_variacoes(res.get('sku'))
            else:
                _url = 'seller-products/'
                response = ol_con.olist_comunicar('post', _url, dados=produto)
                if response.status_code > 399:
                    _json = json.loads(response.content)
                    if _json.get('detail'):
                        _mens = str(_json.get('detail')) + '\n'
                    else:
                        _mens = json.dumps(_json) + '\n'
                    if 'GTIN is in use' in _mens:
                        __atualizar_por_gtin()
                    return {
                        'status': False,
                        'mensagem': _mens
                    }
                res = json.loads(response.content)
                _group = res.get('group')
                vals_anuncio = {
                    'mkpanu_necessita_atualizar': False,
                    'mkpanu_codigo_anuncio': res.get('sku'),
                    'mkpanu_estado_mkp': res.get('status'),
                    'mkpanu_anuncio_endereco': 'https://app.olist.com/products/edit/%s' % res.get('sku')
                }
                if brw_anuncio.mkpanu_produto_id.grade_usar:
                    vals_anuncio['mkpanu_codigo_anuncio'] = _group
                    __atualizar_variacoes(res.get('sku'))

            if res.get('status') == 'created':
                _url = 'seller-products/%s/' % res.get('sku')
                _body = {
                    'status': 'moderation'
                }
                response = ol_con.olist_comunicar('patch', _url, dados=_body)
                if response.status_code > 399:
                    _json = json.loads(response.content)
                    _mens = str(_json.get('detail')) + '\n'
                    return {
                        'status': False,
                        'mensagem': _mens
                    }
                vals_anuncio['mkpanu_estado_mkp'] = res.get('status')
        self.pool.get('mkp.anuncios').write(cr, uid, brw_anuncio.id, vals_anuncio, context=context)
        return {
            'status': True,
            'mensagem': ''
        }

    def mkpOlist_atualizar_saldo(self, cr, uid, anuncio_id, context=None):
        """
        Inclui ou altera saldo do anuncio no site do oLIST
        @param cr:
        @param uid:
        @param anuncio_id: ids do anuncio alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        brw_anuncio = _spg('mkp.anuncios').browse(cr, uid, anuncio_id, context=context)
        context['company_id'] = brw_anuncio.company_id.id
        ol_con = Olist(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
        _mens = ''
        for brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
            path = '/seller-products/%s' % brw_anuncio.mkpanu_codigo_anuncio
            if brw_anuncio.mkpanu_produto_id.grade_usar:
                if not brw_grade.estpgr_anuncio_var_ids:
                    _mens = _('Item %s, grade %s nao encontrado no Mkp!') % (brw_anuncio.mkpanu_produto_id.codigo, brw_grade.grade)
                    continue
                for brw_var in brw_grade.estpgr_anuncio_var_ids:
                    if brw_var.mkpanuv_anuncio_id.id != brw_anuncio.id:
                        continue
                    path = '/seller-products/%s' % brw_var.mkpanuv_id_no_mkp
                    break
            context['company_id'] = brw_anuncio.company_id.id
            _x = _spg('estoque.produto.grade').estpgr_get_saldo(cr, uid, [brw_grade.id], 'D', _local, context=context)
            _saldo = int(_x.get(brw_grade.id) / brw_anuncio.mkpanu_quantidade)
            if _saldo < 0:
                _saldo = 0
            if brw_anuncio.mkpanu_estado == 'P':
                _saldo = 0
            _emb_produto, _emb_package = self._mkpOlist_get_emb_produto(brw_anuncio, context=context)
            body = {
                "stock": [{
                    "availability_days": 0,
                    "quantity": _saldo
                }],
                "package_measures": [_emb_package]
            }
            resp = ol_con.olist_comunicar('patch', path, dados=body)
            res = json.loads(resp.content)
            if resp.status_code > 399:
                if resp.status_code == 404:
                    _mens = _('Item %s, grade %s nao encontrado!') % (brw_anuncio.mkpanu_produto_id.codigo, brw_grade.grade)
                else:
                    _mens = str(res.get('detail')) + '\n'
        if _mens:
            ret = {'status': False, 'mensagem': _mens}
        else:
            ret = {'status': True, 'mensagem': ''}
        return ret

    def mkpOlist_atualizar_preco_mkp(self, cr, uid, anuncio_id, code_mkp, context=None):
        context = context and context.copy() or {}
        _spg = self.pool.get
        brw_anuncio = _spg('mkp.anuncios').browse(cr, uid, anuncio_id, context=context)
        ol_con = Olist(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        _mens = ''
        for brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
            if brw_anuncio.mkpanu_produto_id.grade_usar:
                if not brw_grade.estpgr_anuncio_var_ids:
                    _mens = _('Item %s, grade %s nao encontrado no Mkp!') % (brw_anuncio.mkpanu_produto_id.codigo, brw_grade.grade)
                    continue
                for brw_var in brw_grade.estpgr_anuncio_var_ids:
                    if brw_var.mkpanuv_anuncio_id.id != brw_anuncio.id:
                        continue
                    path = '/seller-products/%s' % brw_var.mkpanuv_id_no_mkp
                    break
            else:
                path = '/seller-products/%s' % brw_anuncio.mkpanu_codigo_anuncio
            _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
            _prices = get_prices(_valor)
            _emb_produto, _emb_package = self._mkpOlist_get_emb_produto(brw_anuncio, context=context)
            body = {
                "prices": _prices,
                "package_measures": [_emb_package]
            }
            resp = ol_con.olist_comunicar('patch', path, dados=body)
            res = json.loads(resp.content)
            if resp.status_code > 399:
                if resp.status_code == 404:
                    _mens = _('Item %s, grade %s nao encontrado!') % (brw_anuncio.mkpanu_produto_id.codigo, brw_grade.grade)
                else:
                    _mens = str(res.get('detail')) + '\n'
        if _mens:
            ret = {'status': False, 'mensagem': _mens}
        else:
            ret = {'status': True, 'mensagem': ''}
        return ret

    def mkpOlist_pesquisar_orders(self, cr, uid, conta_id, context=None):
        context = context and context.copy() or {}
        olist_con = Olist(cr, uid, conta_id)
        params = {'status': 'approved', 'canceled': False}
        response = olist_con.olist_comunicar('get', 'seller-orders', params=params)
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('message')}
        if not response.content:
            return False
        obj_utils = self.pool.get('mkp.utils')
        _peds = json.loads(response.content)
        if _peds.get('count') > 0:
            for _ped in _peds.get('results'):
                args = [('mkpnot_conta_id', '=', conta_id), ('mkpnot_resource', '=', '/seller-orders/%s/' % _ped
                                                             .get('code'))]
                not_ids = self.pool.get('mkp.notificacoes').search(cr, uid, args, context=context)
                if not_ids:
                    vals = {
                        'state': 'R',
                    }
                    self.pool.get('mkp.notificacoes').write(cr, uid, not_ids, vals, context=context)
                else:
                    vals = {
                    'mkpnot_conta_id': conta_id,
                    'mkpnot_resource': '/seller-orders/%s/' % _ped.get('code'),
                    'mkpnot_topic': 'orders',
                    'mkpnot_attempts': 1,
                    'state': 'R',
                    }
                    res = self.pool.get('mkp.notificacoes').create(cr, uid, vals, context=context)
        return True


    def mkpOlist_get_orders(self, cr, uid, order_list, conta_id):
        self._logger.info('Buscando orders na Olist!')
        con_olist = Olist(cr, uid, conta_id)
        path = 'seller-orders/%s/' % order_list
        response = con_olist.olist_comunicar('get', path)
        try:
            _json = json.loads(response.content)
        except:
            return {'status': False, 'mensagem': 'response.content'}
        if response.status_code > 399:
            _mens = str(_json.get('detail')) + '\n'
            return {
                'status': False,
                'mensagem': _mens
            }
        ret = {'status': True, 'mensagem': '', 'orders': _json}
        return ret

    def mkpOlist_set_order_faturado(self, cr, uid, _order, _conta_id, link, context=None):
        context = context and context.copy() or {}
        con_olist = Olist(cr, uid, _conta_id)
        recurso = 'seller-orders/%s/invoice' % _order

        data = {
                  "url": link
        }
        response = con_olist.olist_comunicar('put', recurso, dados=data)
        if response.status_code == 304:  # Já enviado
            return {'status': True, 'mensagem': ''}
        _json = json.loads(response.content)
        if response.status_code > 399:
            _mens = str(_json.get('detail')) + '\n'
            _mens = _mens.replace('The seller order is already invoiced', _('Ja esta faturado no mkp!'))
            return {
                'status': False,
                'mensagem': _mens
            }
        ret = {'status': True, 'mensagem': ''}
        return ret

    def mkpOlist_verificar_pacotes(self, cr, uid, conta_id, _ped, context=None):
        def _buscar(resource):
            ol_con = Olist(cr, uid, conta_id)
            response = ol_con.olist_comunicar('GET', resource, dados={})
            return response

        resource = '/packages/?seller_order_code=%s' % _ped
        response = _buscar(resource)
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('message')}
        res = {}
        lista = []
        while True:
            try:
                resultado = json.loads(response.content)
                if resultado.get('count') == 0:
                    return {'status': False, 'mensagem': _('Nenhum pacote encontrado!')}
                for x in resultado.get('results'):
                    if x.get('seller_order_code') == _ped:
                        lista.append(x)
                if resultado.get('next'):
                    response = _buscar(resultado.get('next'))
                    if response.status_code > 399:
                        return {'status': False, 'mensagem': json.loads(response.content).get('message')}
                else:
                    break
            except:
                return {'status': False, 'mensagem': response.content}
        return {'status': True, 'mensagem': '', 'shipments': lista}

    def mkpOlist_get_etiqueta_mkp(self, cr, uid, conta_id, pkg, context=None):
        context = context and context.copy() or {}
        _order = False
        for plp in pkg.get('shipments'):
            _url = plp.get('zpl_label_url')
            _track = plp.get('tracking_code')
            if not _url:
                return {'status': False, 'mensagem': _('A etiqueta ainda nao esta disponivel')}
            if not _order:
                _order = plp.get('seller_order_code')
                path_zpl = '/tmp/etiqueta-olist-%s.zpl' % _order
                f = open(path_zpl, 'w')
            response = requests.get(_url)
            if response.status_code > 399:
                return {'status': False, 'mensagem': json.loads(response.content).get('message')}
            dados = response.text
            f.write(dados)
        f.close()
        return {'status': True, 'mensagem': '', 'arquivo': path_zpl}