#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from urllib import urlencode
import json
import requests
from openerp import pooler
import logging
from openerp.tools.translate import _
try:
    from prestapyt import PrestaShopWebServiceDict
except:
    pass

_logger = logging.getLogger('Ptsp')

"""
Para acessar
http://P9DGBI7SSYXAS49KXKXKCPI863N5LL51@www.solear.net.br/loja/api/categories
http://P9DGBI7SSYXAS49KXKXKCPI863N5LL51@www.solear.net.br/loja/api/categories/1
http://P9DGBI7SSYXAS49KXKXKCPI863N5LL51@www.solear.net.br/loja/api/products
http://P9DGBI7SSYXAS49KXKXKCPI863N5LL51@www.solear.net.br/loja/api/products/1
http://P9DGBI7SSYXAS49KXKXKCPI863N5LL51@www.solear.net.br/loja/api/orders


"""

class Ptsp(object):
    def logged_client(f):
        """Decorator for User Credentials checking."""
        def inner(self,*args, **kargs):
            if ((self.chave is None) or (self.site_url is None)):
                raise Exception, _("Nao posso acessar o Prestashop parametros do Prestashop nao informados!")
            return f(self,*args,**kargs)
        return inner

    def __init__(self, cr=False, uid=False, conta_id=False, company_id=False):
        self.site_url = ''
        self.cr = cr
        self.uid = uid
        self.chave = ''
        self.habilitar_log = False
        self.conta_id = ''
        self.comunicar = True
        if cr:
            _poolget = pooler.get_pool(cr.dbname).get
            mkp_ids = _poolget('mkp.marketplaces').search(cr, uid, [('mkpmkp_codigo', '=', 'PrtShp')])
            if mkp_ids:
                mkp_ids = mkp_ids[0]
                dados_marketplace = _poolget('mkp.marketplaces').read(cr, uid, mkp_ids, [])
                if dados_marketplace.get('mkpmkp_parar_comunicacao'):
                    self.comunicar = False
                if not conta_id:
                    conta_id = _poolget('mkp.contas').search(cr, uid, [('mkpcta_marketplace_id', '=', mkp_ids)])
                    if conta_id:
                        conta_id = conta_id[0]
                if conta_id:
                    self.conta_id = conta_id
                    dados_conta = _poolget('mkp.contas').read(cr, uid, conta_id, [])
                    if dados_conta.get('mkpcta_parar_comunicacao'):
                        self.comunicar = False
                    self.chave = dados_conta.get('mkpcta_ps_chave_id')
                    self.site_url = dados_conta.get('mkpcta_ps_site')+'/api'
                    self.habilitar_log = dados_marketplace.get('mkpmkp_habilita_log')

    # def get_url(self, path):
    #     x = '//%s@' % self.chave
    #     _url = self.site_url.replace('//', x)
    #     if _url[-1:] != '/':
    #         _url += '/'
    #     _url += path
    #     return _url

    # REQUEST METHODS
    @logged_client
    def get(self, recurso, recurso_id, options=None):

        if not self.comunicar:
            return {'status': False, 'mensagem': 'Comunicacao parada'}

        status = True
        message = ''
        resposta = ''
        prestashop = PrestaShopWebServiceDict(self.site_url, self.chave)
        if self.habilitar_log:
            _logger.info('Ptsp.get-> %s -> %s -> %s' % (self.site_url, recurso, recurso_id))
        try:
            resposta = prestashop.get(recurso, recurso_id, options=options)
        except Exception as e:
            status = False
            message = str(e)
        if status:
            if self.habilitar_log:
                _logger.info('Ptsp.resposta--> %s ' % resposta)
        return {'status': status, 'message': message, 'resposta': resposta}

    @logged_client
    def set(self, recurso, dados=None, codigo=None, files=None):

        if not self.comunicar:
            return {'status': False, 'mensagem': 'Comunicacao parada'}

        status = True
        message = ''
        resposta = ''
        prestashop = PrestaShopWebServiceDict(self.site_url, self.chave)
        add_edit = 'edit' if codigo else 'add'
        if self.habilitar_log:
            _logger.info('Ptsp.%s-> %s -> %s -> %s' % (add_edit, self.site_url, recurso, dados or file))
        try:
            resposta = prestashop.add(recurso, content=dados, files=files) if add_edit == 'add' else prestashop.edit(recurso, dados)
        except Exception as e:
            status = False
            message = str(e)
        if status:
            if self.habilitar_log:
                _logger.info('Ptsp.resposta--> %s ' % resposta)
        return {'status': status, 'message': message, 'resposta': resposta}

    @logged_client
    def upload(self, path, files, params={}):
        def comunicar():
            if self.habilitar_log:
                _logger.info('Ptsp.upload-> %s -> %s' % (uri, params))
            response =  requests.post(uri, files=files, params=urlencode(params), headers=headers)
            if self.habilitar_log:
                if response.status_code == 200:
                    _logger.info('Ptsp.resposta--> %s ' % response.content)
                else:
                    _logger.info('Ptsp.resposta--> %s ' % response)
            elif response.content > 399:
                print '== upload uri->', uri, '== files->', files, '== Params->', params
            return response

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"error":"Comunicacao parada"}'
            return resp

        headers = {'Accept': 'application/json', 'User-Agent':self.SDK_VERSION, 'Content-type':'multipart/form-data'}
        response = self.options('users/me', {'access_token': self.access_token})
        if response.status_code == 401:
            _res = eval(response.content)
            if _res.get('message') == 'invalid_token':
                self.get_refresh_token()
                if params.get('access_token'):
                    params['access_token'] = self.access_token
        headers = {}
        uri = self.make_path(path)
        response = comunicar()
        return response

    @logged_client
    def put(self, path, body=None, params={}):
        def comunicar():
            if self.habilitar_log:
                _logger.info('Ptsp.put-> %s -> %s' % (uri, params))
            response = requests.put(uri, data=_body, params=urlencode(params), headers=headers)
            if self.habilitar_log:
                if response.status_code == 200:
                    _logger.info('Ptsp.resposta--> %s ' % response.content)
                else:
                    _logger.info('Ptsp.resposta--> %s ' % response)
            elif response.status_code > 399:
                print '== Put uri->', uri, '== Data->', _body, '== Params->', params
                print response.content
            return response

        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"error":"Comunicacao parada"}'
            return resp

        headers = {'Accept': 'application/json', 'User-Agent':self.SDK_VERSION, 'Content-type':'application/json'}
        uri = self.make_path(path)
        # if body:
        #     _body = json.dumps(body)
        _body = json.dumps(body)
        response = comunicar()
        if response.status_code == 401:
            _res = eval(response.content)
            if _res.get('message') == 'invalid_token':
                self.get_refresh_token()
                if params.get('access_token'):
                    params['access_token'] = self.access_token
                    response = comunicar()
        return response

    def delete(self, path, params={}):
        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"error":"Comunicacao parada"}'
            return resp
        headers = {'Accept': 'application/json', 'User-Agent':self.SDK_VERSION, 'Content-type':'application/json'}
        uri = self.make_path(path)
        if self.habilitar_log:
            _logger.info('Ptsp.delete-> %s -> %s' % (uri, params))
        response = requests.delete(uri, params=params, headers=headers)
        if self.habilitar_log:
            if response.status_code == 200:
                _logger.info('Ptsp.resposta--> %s ' % response.content)
            else:
                _logger.info('Ptsp.resposta--> %s ' % response)
        return response

    def search(self, recurso, options=None):
        status = True
        message = ''
        resposta = ''
        prestashop = PrestaShopWebServiceDict(self.site_url, self.chave)
        if self.habilitar_log:
            _logger.info('Ptsp.search-> %s -> %s' % (recurso, options))
        try:
            resposta = prestashop.search(recurso, options)
        except Exception as e:
            status = False
            message = e
        if status:
            if self.habilitar_log:
                _logger.info('Ptsp.resposta--> %s ' % resposta)
        return {'status': status, 'message': message, 'resposta': resposta}

    def options(self, path, params={}):
        headers = {'Accept': 'application/json', 'User-Agent':self.SDK_VERSION, 'Content-type':'application/json'}
        uri = self.make_path(path)
        if self.habilitar_log:
            _logger.info('Ptsp.options-> %s -> %s' % (uri, params))
        response = requests.options(uri, params=urlencode(params), headers=headers)
        if self.habilitar_log:
            if response.status_code == 200:
                _logger.info('Ptsp.resposta--> %s ' % response.content)
            else:
                _logger.info('Ptsp.resposta--> %s ' % response)
        return response

    def make_path(self, path, params={}):
        # Making Path and add a leading / if not exist
        if not (re.search("^http", path)):
            if not (re.search("^\/", path)):
                path = "/" + path
            path = self.API_ROOT_URL + path
        if params:
            path = path + "?" + urlencode(params)
        return path



# #!/usr/bin/python
# # -*- coding: utf-8 -*-
#
# from prestapyt import PrestaShopWebService
# from xml.etree import ElementTree
#
#
# prestashop = PrestaShopWebService('http://www.solear.net.br/loja/api', 'P9DGBI7SSYXAS49KXKXKCPI863N5LL51')
# prestashop.debug = True
#
# prestashop.get('')
# address = """
# <prestashop xmlns:xlink="http://www.w3.org/1999/xlink">
# <category>
# <active>1</active>
# <name><language id="1">Jackets Exp test</language><language id="2">Giacche exp prova</language></name>
# <link_rewrite><language id="1">1</language><language id="2">1</language></link_rewrite>
# </category>
# </prestashop>
# """
# prestashop.add('categories', address)