# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################
from openerp.osv import osv
from openerp.tools.translate import _
import logging
from ..controller import Shopee
import datetime
import requests
import zipfile
import json
import time
import base64
import os
from SigUtils import numero


#  Classe de alto nivel para comunicação com o Mercado Livre
class mkp_Shopee(osv.Model):
    _name = 'mkp.shopee'
    _auto = False
    _tem_erro = False
    _mensagem_erro = ''
    _logger = logging.getLogger(_name)

    def mkpShopee_get_set_address_shop(self, cr, uid, conta_id, context=None):
        def _setar_address():
            path = '/api/v2/logistics/set_address_config'
            body = {
                'show_pickup_address': True,
                'address_type_config': {
                    'address_id': _address_id,
                    'address_type': ["PICKUP_ADDRESS", "RETURN_ADDRESS"],
                }
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('errors'):
                if resp.get('message'):
                    ret = {'status': False, 'mensagem': resp.get('message'), 'address': []}
                else:
                    ret = {'status': False, 'mensagem': resp.get('errors'), 'address': []}
            else:
                ret = {'status': True, 'mensagem': ''}
            return ret

        con_shopee = Shopee(cr, uid, conta_id)

        path = '/api/v2/logistics/get_address_list'
        body = {
        }
        resp = con_shopee.shopee_comunicar('get', path, body)
        if resp.get('error'):
            if resp.get('message'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'address': []}
            else:
                ret = {'status': False, 'mensagem': resp.get('error'), 'address': []}
            return ret
        if not (resp.get('response') or resp.get('response').get('address_list')):
            ret = {'status': False, 'mensagem': _('Endereco nao encontrado!')}
            return ret
        _address_id = resp.get('response').get('address_list')[0].get('address_id')
        _pickup_address = resp.get('response').get('show_pickup_address')
        if not _pickup_address:
            ret = _setar_address()
            if not ret.get('status'):
                return ret
        ret = {'status': True, 'mensagem': '', 'address': _address_id}
        return ret

    # Todo: Ok
    def mkpShopee_logistico_setar(self, cr, uid, conta_id, channel_id, habilitar, context=None):
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/logistics/update_channel'
        body = {
            "logistics_channel_id": int(channel_id),
            "enabled": habilitar,
        }
        resp = con_shopee.shopee_comunicar('post', path, body)
        if resp.get('error'):
            if resp.get('error') == 'logistics.require_pickup_address':
                ret = self.mkpShopee_get_set_address_shop(cr, uid, conta_id, context=None)
                if not ret.get('status'):
                    return ret
            if resp.get('message'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'logisticos': []}
            else:
                ret = {'status': False, 'mensagem': resp.get('error'), 'logisticos': []}
        else:
            ret = {'status': True, 'mensagem': '', 'logisticos': resp.get('order_list')}
        return ret

    # Todo: Ok
    def mkpShopee_autenticar(self, cr, uid, conta_id):
        con_shopee = Shopee(cr, uid, conta_id)
        _url = con_shopee.auth_url()
        return _url

    # Todo: Verificar
    def mkpShopee_get_orders(self, cr, uid, order_list, conta_id):
        self._logger.info(_('Buscando orders na Shopee!'))
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/order/get_order_detail'
        body = {
            'order_sn_list': order_list,
            'response_optional_fields': [
                'buyer_user_id,buyer_username,estimated_shipping_fee,recipient_address,actual_shipping_fee ,goods_to_declare,note,note_update_time,item_list,pay_time,dropshipper,dropshipper_phone,split_up,buyer_cancel_reason,cancel_by,cancel_reason,actual_shipping_fee_confirmed,buyer_cpf_id,fulfillment_flag,pickup_done_time,package_list,shipping_carrier,payment_method,total_amount,buyer_username,invoice_data, checkout_shipping_carrier, reverse_shipping_fee'
            ]
        }
        resp = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
        if resp.get('errors'):
            if resp.get('message'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'orders': []}
            else:
                ret = {'status': False, 'mensagem': resp.get('errors'), 'orders': []}
        else:
            ret = {'status': True, 'mensagem': '', 'orders': resp.get('response').get('order_list')}
        return ret

    # Todo: Ok
    def mkpShopee_get_categorias(self, cr, uid, conta_id):
        self._logger.info('Buscando categorias na Shopee!')
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/product/get_category'
        body = {
            'language': 'pt-br',
        }
        resp = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('message'), 'categorias': []}
        else:
            ret = {'status': True, 'mensagem': '', 'categorias': resp.get('response')}
        return ret

    # Todo: Ok
    def mkpShopee_get_categorias_atributos(self, cr, uid, mkp_categoria_code, context=None):
        self._logger.info(_('Buscando atributos da categorias no Shopee!'))
        path = '/api/v2/product/get_attributes'
        con_shopee = Shopee(cr, uid)
        body = {
            'category_id': int(mkp_categoria_code),
            'language': 'pt-br',
        }
        resp = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('message'), 'atributos': []}
        else:
            ret = {'status': True, 'mensagem': '', 'atributos': resp.get('response')}
        return ret

    # Todo: Ok
    def mkpShopee_get_logistics(self, cr, uid, conta_id):
        self._logger.info(_('Buscando dados logisticos na Shopee!'))
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/logistics/get_channel_list'
        body = {
        }
        resp = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
        # tratar retorno
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('message'), 'logisticos': []}
        else:
            ret = {'status': True, 'mensagem': '', 'logisticos': resp.get('response')}
        return ret

    # Todo: Verificar
    def mkpShopee_atualizar_saldo(self, cr, uid, anuncio_id, context=None):
        """
        Inclui ou altera saldo do anuncio no site do Shopee
        @param cr:
        @param uid:
        @param anuncio_id: ids do anuncio alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        brw_anuncio = _spg('mkp.anuncios').browse(cr, uid, anuncio_id, context=context)
        context['company_id'] = brw_anuncio.company_id.id
        con_shopee = Shopee(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
        path = '/api/v2/product/update_stock'
        if not brw_anuncio.mkpanu_produto_id.grade_usar:  # produtos sem grade
            if not brw_anuncio.mkpanu_codigo_anuncio:
                return {'status': False, 'mensagem': _('Produto nao cadastrado no mkp!'), 'item': []}
            body = {
                'item_id': int(brw_anuncio.mkpanu_codigo_anuncio),
                'stock_list': [{
                    'model_id': 0,
                    'seller_stock': [{
                        'stock': int(brw_anuncio.mkpanu_saldo),
                        }],
                }]
            }
            if brw_anuncio.mkpanu_estado == 'P':
                body['stock'] = 0
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'stock': []}
            else:
                ret = {'status': True, 'mensagem': '', 'stock': brw_anuncio.mkpanu_codigo_anuncio}
            return ret
        body = {
            'item_id': int(brw_anuncio.mkpanu_codigo_anuncio),
            }
        _lista = []
        for brw_variacoes in brw_anuncio.mkpanu_variacoes_ids:
            _grade_id = brw_variacoes.mkpanuv_grade_id.id
            context['company_id'] = brw_anuncio.company_id.id
            _x = _spg('estoque.produto.grade').estpgr_get_saldo(cr, uid, [_grade_id], 'D', _local, context=context)
            if _x.get(_grade_id) == 0:
                _saldo_zero = True
            _lista.append({
                'model_id': int(brw_variacoes.mkpanuv_id_no_mkp),
                    'seller_stock': [{
                        'stock': int(_x.get(_grade_id) / brw_anuncio.mkpanu_quantidade) if _x.get(_grade_id) > 0 else 0,
                        }],
            })
            # if brw_anuncio.mkpanu_estado == 'P':
            #     body['stock'] = 0
        body.update({'stock_list': _lista})
        # todo: confirmar se o método é o post
        resp = con_shopee.shopee_comunicar('post', path, body)
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('messsage'), 'item': []}
        else:
            ret = {'status': True, 'mensagem': ''}
        if not brw_anuncio.mkpanu_variacoes_ids:
            ret = {'status': False, 'mensagem': _('Item com grade, mas sem variacoes cadastradas!'), 'item': []}
        return ret

    # Todo: Verificar
    def mkpShopee_atualizar_preco(self, cr, uid, anuncio_id, context=None):
        """
        Altera preco do anuncio no site do Shopee
        @param cr:
        @param uid:
        @param anuncio_id: ids do anuncio alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """

        context = context and context.copy() or {}
        _spg = self.pool.get
        brw_anuncio = _spg('mkp.anuncios').browse(cr, uid, anuncio_id, context=context)
        con_shopee = Shopee(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
        _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
        path = '/api/v2/product/update_price'
        if not brw_anuncio.mkpanu_produto_id.grade_usar:  # produtos sem grade
            body = {
                'item_id': int(brw_anuncio.mkpanu_codigo_anuncio),
                'price_list': [{
                    'model_id': 0,
                    'original_price': float(_valor),
                }]
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message')}
            else:
                ret = {'status': True, 'mensagem': '', 'item': brw_anuncio.mkpanu_codigo_anuncio}
            return ret
        body = {
            'item_id': int(brw_anuncio.mkpanu_codigo_anuncio),
        }
        _lista = []
        for brw_variacoes in brw_anuncio.mkpanu_variacoes_ids:
            _grade_id = brw_variacoes.mkpanuv_grade_id.id
            context['company_id'] = brw_anuncio.company_id.id
            _x = _spg('estoque.produto.grade').estpgr_get_saldo(cr, uid, [_grade_id], 'D', _local, context=context)
            if _x.get(_grade_id) == 0:
                _saldo_zero = True
            _lista.append({
                'model_id': int(brw_variacoes.mkpanuv_id_no_mkp),
                'original_price': float(_valor),
            })
        body.update({'price_list': _lista})
        #todo: confirmar se o método é o post
        resp = con_shopee.shopee_comunicar('post', path, body)
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('message')}
        else:
            ret = {'status': True, 'mensagem': ''}
        # if not brw_anuncio.mkpanu_variacoes_ids:
        #     dados_anuncio = _spg('mkp.shopee').mkpShopee_get_anuncio(cr, uid,
        #                                              brw_anuncio.mkpanu_conta_id.id,
        #                                              brw_anuncio.mkpanu_codigo_anuncio,
        #                                              context=context)
        return ret


    # todo: verificar os metodos de variaçoes no shopee
    def mkpShopee_set_variacoes(self, cr, uid, brw_anuncio, anuncio_code, context=None):
        _spg = self.pool.get
        context = context and context.copy() or {}
        con_shopee = Shopee(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        variacoes_atualizar = {}
        variacoes_existentes = {}
        vals_criar_option = []
        vals_criar_model = []
        vals_alterar = []
        vals_alterar_com_novo = []
        for brw_variacoes in brw_anuncio.mkpanu_variacoes_ids:
            variacoes_existentes.update({
                brw_variacoes.mkpanuv_grade_id.id: {
                    'id_no_mkp': brw_variacoes.mkpanuv_id_no_mkp,
                    'id_no_erp': brw_variacoes.id,
                }
            })
        _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
        i = 0
        for _brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
            _sku = '%s-%s' % (brw_anuncio.mkpanu_produto_id.codigo, _brw_grade.grade)
            _grade_ids = [_brw_grade.id]
            context['company_id'] = brw_anuncio.company_id.id
            _x = _spg('estoque.produto.grade').estpgr_get_saldo(cr, uid, _grade_ids, 'D', _local)
            _saldo = int(_x.get(_brw_grade.id) / brw_anuncio.mkpanu_quantidade)
            var = {
                'image': {'image_id': ''},
                'seller_stock': [{
                    # 'location_id': '',
                    'stock': _saldo,
                }],
                'original_price': brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido,
                'model_sku': _sku,
                'option': _brw_grade.descricao,
                'tem_reg_variacao': False,
                'grade_id': _brw_grade.id,
            }
            if _brw_grade.id in variacoes_existentes.keys():
                var['tem_reg_variacao'] = True
                var['id_no_mkp'] = variacoes_existentes.get(_brw_grade.id).get('id_no_mkp')
                var['var_id_no_erp'] = variacoes_existentes.get(_brw_grade.id).get('id_no_erp')
            variacoes_atualizar[_brw_grade.descricao] = var
            vals_alterar_com_novo.append({'option': var.get('option')})
            if var.get('id_no_mkp'):
                vals_alterar.append({'option': var.get('option')})
                i += 1
            else:
                vals_criar_option.append({'option': var.get('option')})
                vals_criar_model.append({
                    'tier_index': [i],
                    'seller_stock': var.get('seller_stock'),
                    'original_price': var.get('original_price'),
                    'model_sku': var.get('model_sku'),
                })
                i += 1

        if vals_alterar and vals_criar_option:
            vals_alterar = vals_alterar_com_novo
            vals_criar_option = []

        if vals_alterar:

            """Alterar variacoes ja cadastradas"""
            path = '/api/v2/product/update_tier_variation'
            body = {
                'item_id': int(anuncio_code),
                'tier_variation': [{
                    'name': brw_anuncio.mkpanu_produto_id.estpro_grade_descricao,
                    'option_list': vals_alterar,
                }],
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                # ret = {'status': False, 'mensagem': resp.get('message'), 'variacoes': []}
                return resp

            if vals_criar_model:
                """Criar modelo(nova grade)"""
                path = '/api/v2/product/add_model'
                body = {
                    'item_id': int(anuncio_code),
                    'model_list': vals_criar_model,
                }
                resp = con_shopee.shopee_comunicar('post', path, body)
                if resp.get('error'):
                    # ret = {'status': False, 'mensagem': resp.get('message'), 'variacoes': []}
                    return resp
                resp_var = resp.get('response').get('model')
                i = 0
                for x in vals_alterar:
                    _grade = x.get('option')
                    for y in resp_var:
                        if y.get('tier_index')[0] != i:
                            continue
                        variacoes_atualizar[_grade]['id_no_mkp'] = y.get('model_id')
                        break
                    i += 1

        if vals_criar_option:
            """Criar variacoes novas"""
            path = '/api/v2/product/init_tier_variation'
            body = {
                'item_id': int(anuncio_code),
                'tier_variation': [{
                    'name': brw_anuncio.mkpanu_produto_id.estpro_grade_descricao,
                    'option_list': vals_criar_option,
                }],
                'model': vals_criar_model,
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                # ret = {'status': False, 'mensagem': resp.get('message'), 'variacoes': []}
                return resp
            resp_var = resp.get('response').get('model')
            i = 0
            for x in vals_criar_option:
                _grade = x.get('option')
                variacoes_atualizar[_grade]['id_no_mkp'] = resp_var[i].get('model_id')
                i += 1

        """Criar e atualizar registros de variacoes"""
        for _grade in variacoes_atualizar.keys():
            var = variacoes_atualizar.get(_grade)
            vals_vars = {
                'mkpanuv_anuncio_id': brw_anuncio.id,
                'mkpanuv_grade_id': var.get('grade_id'),
                'mkpanuv_id_no_mkp': var.get('id_no_mkp'),
            }
            if var.get('tem_reg_variacao'):
                _spg('mkp.anuncios.var').write(cr, uid, var.get('var_id_no_erp'), vals_vars, context=context)
            else:
                _spg('mkp.anuncios.var').create(cr, uid, vals_vars, context=context)
        """Excluir variacoes existentes no mkp"""
        for idx in variacoes_existentes.keys():
            x = variacoes_existentes.get(idx)
            _achei = False
            for idx in variacoes_atualizar.keys():
                y = variacoes_atualizar.get(idx)
                if x.get('id_no_erp') == y.get('var_id_no_erp'):
                    _achei = True
                    break
            if not _achei:
                _spg('mkp.anuncios.var').unlink(cr, uid, x.get('id_no_erp'), context=context)
        resp = self.mkpShopee_atualizar_saldo(cr, uid, brw_anuncio.id, context=context)
        return resp

    # Todo: Ok
    def mkpShopee_set_imagem(self, cr, uid, conta_id, imagem, context=None):
        _spg = self.pool.get
        context = context and context.copy() or {}
        _arq = '/tmp/shopee_%s.jpg' % time.time()
        f = open(_arq, 'wb')
        f.write(base64.b64decode(imagem))
        f.close()
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/media_space/upload_image'
        files = {
            'image': open(_arq, 'rb'),
            'Content-Disposition': 'form-data; name="image"; filename="%s"' % _arq,
            'Content-Type': 'image/jpeg',
        }
        resp = con_shopee.shopee_comunicar('upload', path, files)
        os.remove(_arq)
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('message'), 'images': []}
        else:
            ret = {'status': True, 'mensagem': '', 'images': resp.get('response')}
        return ret

    def mkpShopee_set_get_brand(self, cr, uid, conta_id, categoria, marca, image, context=None):
        _spg = self.pool.get
        context = context and context.copy() or {}
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/product/get_brand_list'
        _offset = 0
        while True:
            body = {
                'page_size': 100,
                'status': 1,
                'category_id': categoria,
                'offset': _offset,
                'language': 'pt-br',
            }
            resp = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
            if resp.get('error'):
                if resp.get('message'):
                    ret = {'status': False, 'mensagem': resp.get('message'), 'brands': {}}
                else:
                    ret = {'status': False, 'mensagem': resp.get('errors'), 'brands': {}}
                return ret
            for lista in resp.get('response').get('brand_list'):
                if lista.get('original_brand_name').lower() == marca.lower():
                    _dMarca = {'brand_id': lista.get('brand_id'),
                               'original_brand_name':  lista.get('original_brand_name')}
                    ret = {'status': True, 'mensagem': '', 'brands': _dMarca}
                    return ret
            if resp.get('response').get('has_next_page'):
                _offset = resp.get('response').get('next_offset')
                continue
            break
        path = '/api/v2/product/register_brand'
        body = {
            'original_brand_name': marca,
            'category_list': [categoria],
            'brand_country': 'pt-br',
            'product_image': {'image_id_list': [image]},
        }
        resp = con_shopee.shopee_comunicar('post', path, body)
        if resp.get('error'):
            if resp.get('message'):
                # ret = {'status': False,
                       # 'mensagem': resp.get('message'),
                       # 'brands': {}}
                ret = {'status': True,
                      'mensagem': '',
                      'brands': {'brand_id': 0, 'original_brand_name': 'NoBrand'}}
            else:
                ret = {'status': False, 'mensagem': resp.get('errors'), 'brands': {}}
            return ret
        _dMarca = {'brand_id': resp.get('response').get('brand_id'),
                   'original_brand_name': resp.get('response').get('original_brand_name')}
        ret = {'status': True, 'mensagem': '', 'brands': _dMarca}
        return ret

    def _mkpShopee_verificar_cadastrar_imagem(self, cr, uid, brw_anuncio, context=None):
        _spg = self.pool.get
        _picture_produts = []
        _pictures_grades = {}
        _img_comum = []
        args = [('mkpromkp_produto_id', '=', brw_anuncio.mkpanu_produto_id.id),
                ('mkpromkp_marketplace_id', '=', brw_anuncio.mkpanu_marketplace_id.id),
                ('company_id', '=', brw_anuncio.company_id.id)
                ]
        context['sig_force_company'] = True
        mkpromkp_ids = _spg('mkp.produto.marketplaces').search(cr, uid, args, context=context)
        context.pop('sig_force_company')
        brw_mkpromkp = _spg('mkp.produto.marketplaces').browse(cr, uid, mkpromkp_ids[0], context=context) # Embora se mkpromkp_ids, será sempre lista de 1
        for brw_imagens in brw_mkpromkp.mkpromkp_imagens_ids:
            if not brw_imagens.mkprmkimg_codigo_no_mkp:
                resp = self.mkpShopee_set_imagem(cr, uid, brw_anuncio.mkpanu_conta_id.id,
                                                 brw_imagens.mkprmkimg_foto, context=context)
                if resp.get('status'):
                    _image_codigo_shopee = resp.get('images').get('image_info').get('image_id')
                    _image_url_shopee = ''
                    for _linha in resp.get('images').get('image_info').get('image_url_list'):
                        if _linha.get('image_url_region') == 'BR':
                            _image_url_shopee = _linha.get('image_url')
                            break
                    _picture_produts.append( _image_codigo_shopee)
                    _dados_img = {
                        'mkprmkimg_codigo_no_mkp': _image_codigo_shopee,
                        'mkprmkimg_endereco_mkp': _image_url_shopee,
                    }
                    _spg('mkp.produto.marketplace.imagens').write(cr, uid, brw_imagens.id, _dados_img, context=context)
                else:
                    continue
            else:
                _image_codigo_shopee = str(brw_imagens.mkprmkimg_codigo_no_mkp)
                _picture_produts.append(_image_codigo_shopee)
            if brw_anuncio.mkpanu_produto_id.grade_usar:
                _gr = brw_imagens.mkprmkimg_imagem_produto_id.estimg_grade_id.grade
                if not _gr:  # imagem é é do produto e não da grade
                    _img_comum.append(_image_codigo_shopee)
                    continue
                _img_gr = _pictures_grades.get(_gr, [])
                _img_gr.append(_image_codigo_shopee)
                _pictures_grades[_gr] = _img_gr
        return _picture_produts, _pictures_grades, _img_comum

    def _mkpShopee_get_sku_produto(self, cr, uid, brw_anuncio, context=None):
        """
        Formata o sku de um produto
        :param cr:
        :param uid:
        :param brw_anuncio:
        :param context:
        :return: {sku_produto: ___, sku_grades :{'GR':_sku_grade, 'GR': _sku_grade}}
        """
        if not brw_anuncio.mkpanu_produto_id.grade_usar:
            _sku = '%s-UN' % brw_anuncio.mkpanu_produto_id.codigo
            return {'sku_produto': _sku, 'sku_grades': {}}
        grades = {}
        for brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
            _sku = brw_anuncio.mkpanu_produto_id.codigo
            _sku_grade = '%s-%s' % (brw_anuncio.mkpanu_produto_id.codigo, brw_grade.grade)
            grades.update({brw_grade.grade: _sku_grade})
        return {'sku_produto': _sku, 'sku_grades': grades}

    # Todo: Ok
    def mkpShopee_set_anuncio(self, cr, uid, brw_anuncio, context=None):
        if brw_anuncio.mkpanu_codigo_anuncio:
            _funcao = _('Alterar')
            _obj = _('Anuncio')
            _cod = brw_anuncio.mkpanu_codigo_anuncio
        else:
            _funcao = _('Criar')
            _obj = _('Item')
            _cod = brw_anuncio.mkpanu_produto_id.codigo
        self._logger.info(_('%s anuncio na Shopee! %s: %s') % (_funcao, _obj, _cod))
        _spg = self.pool.get
        context = context and context.copy() or {}
        con_shopee = Shopee(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        if brw_anuncio.mkpanu_codigo_anuncio:
            path = '/api/v2/product/update_item'
        else:
            path = '/api/v2/product/add_item'
        mkp_id = brw_anuncio.mkpanu_marketplace_id.id
        args = [('mkpromkp_produto_id', '=', brw_anuncio.mkpanu_produto_id.id),
                ('mkpromkp_marketplace_id', '=', mkp_id),
                ('company_id', '=', brw_anuncio.company_id.id)
                ]
        context['sig_force_company'] = True
        mkp_produto_mkp_ids = _spg('mkp.produto.marketplaces').search(cr, uid, args, context=context)
        context.pop('sig_force_company')
        if not mkp_produto_mkp_ids:
            return {'status': True}  # Não tem anuncio para atualizar
        brw_mkp_produto_mkp = _spg('mkp.produto.marketplaces').browse(cr, uid, mkp_produto_mkp_ids, context=context)[0]
        if not brw_mkp_produto_mkp.mkpromkp_categoria_id:
            return {'status': False, 'mensagem': _('Nao tem categoria cadastrada no produto para o Marketplace')}
        _categ_id = brw_mkp_produto_mkp.mkpromkp_categoria_id.mkpcat_codigo
        _name = brw_anuncio.mkpanu_title
        _descricao = brw_anuncio.mkpanu_descricao_texto
        if not brw_anuncio.mkpanu_conta_id.mkpcta_producao:
            if len(_descricao) > 500:
                _descricao = _descricao[:499]
        _price = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
        _peso = 0.3
        _altura = 7
        _largura = 10
        _comprimento = 10
        for brw_embalagem in brw_anuncio.mkpanu_produto_id.produto_embalagem_ids:  # pegar APENAS A PRIMEIRA
            _peso = brw_embalagem.peso_bruto
            _altura = brw_embalagem.altura
            _largura = brw_embalagem.largura
            _comprimento = brw_embalagem.profundidade
            break
        _picture_produts, _pictures_grades, _img_comum = self._mkpShopee_verificar_cadastrar_imagem(cr, uid, brw_anuncio, context=context)
        if brw_anuncio.mkpanu_produto_id.grade_usar and not _pictures_grades:
            return {'status': False,
                    'mensagem': _('O produto %s nao tem fotos das grades') % brw_anuncio.mkpanu_produto_id.codigo}
        for _img_idx in _pictures_grades.keys():
            _pictures_grades[_img_idx] = _pictures_grades.get(_img_idx) + _img_comum
        if not brw_anuncio.mkpanu_conta_id.mkpcta_shopee_logistico_id:
            return {'status': False,
                    'mensagem': _('Operador logistico nao cadastrado na conta do mkp!')}
        _logistics = {
            'logistic_id': int(brw_anuncio.mkpanu_conta_id.mkpcta_shopee_logistico_id.mkplog_codigo),
            'enabled': True,
        }
        _atributos = []
        _marca = 'OEM'
        for brw_atributo in brw_anuncio.mkpanu_produto_id.estpro_atributo_ids:
            if brw_atributo.estatp_atributo_id.name.lower() == 'marca':
                _marca = str(brw_atributo.estatp_atributo_valor)

        ret_brand = self.mkpShopee_set_get_brand(cr, uid, brw_anuncio.mkpanu_conta_id.id, int(_categ_id), _marca,
                                                 _picture_produts[0], context=context)
        if not ret_brand.get('status'):
            return ret_brand
        _sku = brw_anuncio.mkpanu_produto_id.codigo
        body = {
            'original_price': float(_price),
            'description': _descricao,
            'weight': float(_peso),
            'item_name': _name,
            'item_status': "NORMAL",
            'dimension': {
                'package_height': int(_altura),
                'package_length': int(_comprimento),
                'package_width': int(_largura),
            },
            'logistic_info': [_logistics],
            'attribute_list': _atributos,
            'category_id': int(_categ_id),
            'image': {'image_id_list': _picture_produts},
            'brand': ret_brand.get('brands'),
            'seller_stock': [{
                # 'location_id': '',
                'stock': 0,
            }],
            'item_sku': self._mkpShopee_get_sku_produto(cr, uid, brw_anuncio, context=context).get('sku_produto'),
        }
        if not brw_anuncio.mkpanu_produto_id.grade_usar:
            _saldo = brw_anuncio.mkpanu_saldo
            body.update({
                'seller_stock': [{
                    # 'location_id': '',
                    'stock': _saldo,
                }]
            })
        if brw_anuncio.mkpanu_codigo_anuncio:
            body.update({
                'item_id': int(brw_anuncio.mkpanu_codigo_anuncio),
            })
        resp_item = con_shopee.shopee_comunicar('post', path, body)
        if resp_item.get('error'):
            ret = {'status': False, 'mensagem': resp_item.get('message'), 'item': []}
            return ret
        _item_shopee = resp_item.get('response').get('item_id')
        ret = {'status': True, 'mensagem': '', 'item': _item_shopee}
        vals_anuncio = {}
        vals_anuncio['mkpanu_codigo_anuncio'] = _item_shopee
        _spg('mkp.anuncios').write(cr, uid, brw_anuncio.id, vals_anuncio, context=context)
        cr.commit()
        if brw_anuncio.mkpanu_produto_id.grade_usar:
            anuncio_code = _item_shopee
            resp = self.mkpShopee_set_variacoes(cr, uid, brw_anuncio, anuncio_code, context=context)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'variacoes': []}
        return ret

    def mkpShopee_set_order_faturado(self, cr, uid, order, conta_id, context=None):
        context = context and context.copy() or {}
        _spg = self.pool.get
        ped_id = _spg('estoquepedidos.pedido.capa').search(cr, uid, [('pedcap_nro_pedido_externo', '=', order)],
                                                           context=context)
        brw_ped = _spg('estoquepedidos.pedido.capa').browse(cr, uid, ped_id[0], context=context)
        _itens = []
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/order/upload_invoice_doc'
        for brw_nf in brw_ped.pedcap_nf_ids:
            _nome_arquivo = '/tmp/%s.xml' % brw_nf.nfe_chave_acesso
            fs = open(_nome_arquivo, 'w')
            fs.write(brw_nf.nfe_xml_proc)
            fs.close()
            payload = {
                'order_sn': order,
                'file_type': '4',
            }
            files = {
                'file': open(_nome_arquivo, 'rb'),
            }
            resp = con_shopee.shopee_comunicar('upload', path, files, payload=payload)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message')}
            else:
                ret = {'status': True, 'mensagem': ''}
            if os.path.exists(_nome_arquivo):
                os.system('rm -rf %s' % _nome_arquivo)
        return ret


    # Todo: Verificar
    def mkpShopee_get_anuncio(self, cr, uid, conta_id, item_code, context=None):
        self._logger.info(_('Buscando anuncios na Shopee!'))
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/product/get_item_base_info'
        body = {
            'item_id_list': [int(item_code)],
            'need_tax_info': False,
            'need_complaint_policy': False,
        }
        resp = con_shopee.shopee_comunicar('get', path, body)
        if resp.get('error'):
            ret = {'status': False, 'mensagem': resp.get('message'), 'item': []}
        path = '/api/v2/product/get_item_extra_info'
        body = {
            'item_id_list': [int(item_code)],
        }
        resp_extra = con_shopee.shopee_comunicar('get', path, body)
        if resp_extra.get('error'):
            ret = {'status': False, 'mensagem': resp_extra.get('message'), 'item': []}
        _item = resp.get('response').get('item_list')[0]
        if resp_extra.get('response') and resp_extra.get('response').get('item_list'):
            _item.update(resp_extra.get('response').get('item_list')[0])
        else:
            _item.update({'sale': 0})
        ret = {'status': True, 'mensagem': '', 'item': _item}
        return ret

    # Todo: Verificar
    def mkpShopee_get_etiqueta(self, cr, uid, order, conta_id, context):
        def _get_shipping_parameter(_order):
            path = '/api/v2/logistics/get_shipping_parameter'
            body = {
                'order_sn': _order,
            }
            resp_par = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
            if resp_par.get('error'):
                ret = {'status': False, 'mensagem': resp_par.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp_par}

        def _ship_order(_order, dropoff, pickup, non_integrated, resp_par):
            # Iniciando a logistica
            path = '/api/v2/logistics/ship_order'
            body = {
                'order_sn': _order,
            }
            if dropoff:
                body.update({
                    "dropoff": {
                        "tracking_number": "",
                        "branch_id": 0,
                        "sender_real_name": ""
                    }
                })
            if pickup:
                address_id = 0
                pickup_time_id = ''
                for address in resp_par.get('response').get('pickup').get('address_list'):
                    if address.get('zipcode') != company_cep:
                        continue
                    address_id = address.get('address_id')
                    for tml in address.get('time_slot_list'):
                        pickup_time_id = tml.get('pickup_time_id')
                        break
                body.update({
                    "pickup": {
                        "tracking_number": "",
                        "address_id": address_id,
                        "pickup_time_id": pickup_time_id,
                    }
                })
            if non_integrated:
                body.update({
                    "non_integrated": {
                        "tracking_number": "",
                    }
                })
            resp = con_shopee.shopee_comunicar('post', path, body, headers={})
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        def _update_shipping_order(_order, resp_par):
            path = '/api/v2/logistics/ship_order'
            body = {
                'order_sn': _order,
            }
            address_id = 0
            pickup_time_id = ''
            for address in resp_par.get('response').get('pickup').get('address_list'):
                if address.get('zipcode') != company_cep:
                    continue
                address_id = address.get('address_id')
                for tml in address.get('time_slot_list'):
                    pickup_time_id = tml.get('pickup_time_id')
                    break
            body.update({
                "pickup": {
                    "address_id": address_id,
                    "pickup_time_id": pickup_time_id,
                }
            })
            resp = con_shopee.shopee_comunicar('post', path, body, headers={})
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        def _get_track_number(_order):
            path = '/api/v2/logistics/get_tracking_number'
            body = {
                'order_sn': _order,
                'response_optional_fields': 'plp_number,first_mile_tracking_number,last_mile_tracking_number',
            }
            resp = con_shopee.shopee_comunicar('get', path, body, no_headers=True)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        def _get_shipping_document_parameter(_order):
            path = '/api/v2/logistics/get_shipping_document_parameter'
            body = {
                'order_list': [{
                    'order_sn': _order,
                }]
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        def _create_shipping_document(_order, tracking_number, shipping_document_type):
            path = '/api/v2/logistics/create_shipping_document'
            body = {
                'order_list': [{
                    'order_sn': _order,
                    'tracking_number': tracking_number,
                    'shipping_document_type': shipping_document_type,
                    # 'package_number': ''
                }],
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        def _get_shipping_document_result(_order, shipping_document_type):
            path = '/api/v2/logistics/get_shipping_document_result'
            body = {
                'order_list': [{
                    'order_sn': _order,
                    'shipping_document_type': shipping_document_type,
                    # 'package_number': ''
                }],
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        def _download_shipping_document(_order, shipping_document_type):
            path = '/api/v2/logistics/download_shipping_document'
            body = {
                'shipping_document_type': shipping_document_type,
                'order_list': [{
                    'order_sn': _order,
                    # 'package_number': ''
                }],
            }
            resp = con_shopee.shopee_comunicar('post', path, body)
            if resp.get('error'):
                ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
                return ret
            return {'status': True, 'resposta': resp}

        self._logger.info(_('Buscando etiqueta na Shopee!'))
        brw_conta = self.pool.get('mkp.contas').browse(cr, uid, conta_id, context=context)
        company_cep = brw_conta.company_id.rescon_pessoa_id.endereco_nf_id.cep.replace('-', '').replace('.', '')
        con_shopee = Shopee(cr, uid, conta_id)

        ret = _get_shipping_parameter(order)
        if not ret.get('status'):
            return ret
        resp_par = ret.get('resposta')

        _dropoff = True if resp_par.get('response').get('dropoff') else False
        _pickup = True if resp_par.get('response').get('pickup') else False
        _non_integrated = True if resp_par.get('response').get('non_integrated') else False
        ret = _ship_order(order, _dropoff, _pickup, _non_integrated, resp_par)
        # if not ret.get('status'):
        #     return ret
        resp = ret.get('resposta')

        # if _pickup:
        #     ret = _update_shipping_order(order, resp_par)
        #     if not ret.get('status'):
        #         return ret
        #     resp = ret.get('resposta')

        ret = _get_track_number(order)
        if not ret.get('status'):
            return ret
        resp = ret.get('resposta')
        _tracking_no = resp.get('response').get('tracking_number')

        ret = _get_shipping_document_parameter(order)
        if not ret.get('status'):
            return ret
        resp = ret.get('resposta')
        _shipping_document_type = resp.get('response').get('result_list')[0].get('suggest_shipping_document_type')

        ret = _create_shipping_document(order, _tracking_no, _shipping_document_type)
        if not ret.get('status'):
            return ret
        resp = ret.get('resposta')

        ret = _get_shipping_document_result(order, _shipping_document_type)
        if not ret.get('status'):
            return ret
        resp = ret.get('resposta')

        ret = _download_shipping_document(order, _shipping_document_type)
        if not ret.get('status'):
            return ret
        resp = ret.get('resposta')
        dados = resp.get('dados')
        try:
            arqz = '/tmp/%s.zip' % _tracking_no
            self._logger.info(_('Criando arquivo: %s' % arqz))
            f = open(arqz, 'wb')
            f.write(dados)
            f.close()
        except Exception as e:
            self._logger.error(_('Nao consegui salvar arquivo de etiqueta: %s' % e))
        arqd = '/tmp/%s' % _tracking_no
        try:
            fantasy_zip = zipfile.ZipFile(arqz)
            fantasy_zip.extractall(arqd)
            fantasy_zip.close()
        except Exception as e:
            pass
        os.remove(arqz)
        try:
            f_in = open('%s/thermal_zpl_shipping_label.txt' % arqd, 'r')
            fi = f_in.readlines()
            f_in.close()
            _nome = '%s/thermal_zpl_shipping_label.txt' % arqd
        except:
            f_in = open('%s/shipping_label.txt' % arqd, 'r')
            fi = f_in.readlines()
            f_in.close()
            _nome = '%s/shipping_label.txt' % arqd
        try:
            os.remove(_nome)
            os.rmdir(arqd)
        except:
            pass
        out_path = '/tmp/etiqueta-%s.txt' % _tracking_no
        f = open(out_path, 'w')
        f.writelines(fi)
        f.close()

        # i = 0
        # while True:
        #     i += 1
        #     path = '/api/v1/logistics/init'
        #     body = {
        #         "ordersn": order,
        #         'dropoff': {},
        #     }
        #     resp = con_shopee.shopee_comunicar(path, body)
        #     if resp.get('error'):
        #         ret = {'status': False, 'mensagem': resp.get('message'), 'etiqueta': []}
        #         return ret
        #     if resp.get('tracking_number'):
        #         _tracking_no = resp.get('tracking_number')
        #     else:
        #         if i < 10:
        #             self._logger.info(_('Tentando novamente!'))
        #             time.sleep(1)
        #             continue
        #     path = '/api/v1/logistics/airway_bill/get_mass'
        #     body = {
        #         'ordersn_list': [order],
        #     }
        #     resp = con_shopee.shopee_comunicar(path, body)
        #     if resp.get('result').get('errors'):
        #         ret = {'status': False, 'mensagem': _('Nao consegui pegar etiqueta de postagem'), 'etiqueta': []}
        #         return ret
        #     else:
        #         break
        # arq = resp.get('result').get('airway_bills')
        # if isinstance(arq, list):
        #     arq = arq[0]
        # _url = arq.get('airway_bill')
        # dados = requests.get(_url)
        # data = datetime.datetime.now()
        # path_zip = '/tmp/etiqueta-shopee-%s.zip' % data
        # path = '/tmp/etiqueta-%s' % data
        # try:
        #     fantasy_zip = zipfile.ZipFile(path_zip)
        #     fantasy_zip.extractall(path)
        #     fantasy_zip.close()
        # except Exception as e:
        #     try:
        #         f = open(path_zip, 'r')
        #         _etiq = json.loads(f.read())
        #         f.close()
        #         if _etiq.get('error') == 'labels_generation_locked':
        #             ret = {'status': False, 'mensagem': _('labels_generation_locked'), 'etiqueta': []}
        #             return ret
        #         elif _etiq.get('error') == 'not_found':
        #             ret = {'status': False, 'mensagem': _('A etiqueta ainda nao foi liberada pelo Shopee.-not_found!'), 'etiqueta': []}
        #             return ret
        #         else:
        #             ret = {'status': False, 'mensagem': _('Erro na leitura da etiqueta\n%s') % _etiq.get('message'), 'etiqueta': []}
        #             return ret
        #     except Exception as e:
        #         if e.message == 'labels_generation_locked':
        #             ret = {'status': False, 'mensagem': _('A etiqueta ainda nao foi liberada pelo Shopee.!'), 'etiqueta': []}
        #             return ret
        #         ret = {'status': False, 'mensagem': (_('Nao consegui extrair a etiqueta:\n\n%s') % e), 'etiqueta': []}
        #         return ret
        # os.remove(path_zip)
        # f_in = open('%s/shipping_label.txt' % path, 'r')
        # fi = f_in.readlines()
        # f_in.close()
        # os.remove('%s/shipping_label.txt' % path)
        # out_path = '/tmp/etiqueta-%s.txt' % str(data).replace(' ', '_')
        # f = open(out_path, 'w')
        # f.writelines(fi)
        # f.close()
        # try:
        #     f_in = open('%s/content_declaration_file.pdf' % path, 'r')
        #     fi = f_in.readlines()
        #     f_in.close()
        #     os.remove('%s/content_declaration_file.pdf' % path)
        #     outC_path = '/tmp/Controle-%s.pdf' % str(data).replace(' ', '_').replace(':', '_')
        #     f = open(outC_path, 'w')
        #     f.writelines(fi)
        #     f.close()
        # except:
        #     outC_path = ''
        ret = {'status': True,
               'mensagem': '',
               'etiqueta': out_path,
               'controle': '',
               'trackno': _tracking_no}
        return ret

    # Todo: Verificar
    def mkpShopee_pesquisar_orders(self, cr, uid, conta_id, context=None):
        context = context and context.copy() or {}
        _spg = self.pool.get
        con_shopee = Shopee(cr, uid, conta_id)
        path = '/api/v2/order/get_order_list'
        _cursor = 0
        body = {
            'time_range_field': 'update_time',
            'time_from': int(time.time() - 3600),
            'time_to': int(time.time()),
            'page_size': 100,
        }
        while True:
            body.update({'cursor': _cursor})
            _cursor += 100
            resp = con_shopee.shopee_comunicar('get', path, body)
            if not (resp and resp.get('response') and resp.get('response').get('order_list')):
                return True
            for order in resp.get('response').get('order_list'):
                _order_id = order.get('order_sn')
                _resourse = '/orders/%s' % _order_id
                args = [
                    ('mkpnot_conta_id', '=', conta_id),
                    ('mkpnot_resource', '=', _resourse)
                ]
                not_ids = _spg('mkp.notificacoes').search(cr, uid, args, context=context)
                if not_ids:
                    vals = {'state': 'R'}
                    res = _spg('mkp.notificacoes').write(cr, uid, not_ids[0], vals, context=context)
                else:
                    vals = {
                        'mkpnot_conta_id': conta_id,
                        'mkpnot_resource': _resourse,
                        'mkpnot_topic': 'orders',
                        'mkpnot_attempts': 1,
                        'mkpnot_loja': con_shopee.shop_id,
                        'state': 'R',
                    }
                    res = _spg('mkp.notificacoes').create(cr, uid, vals, context=context)
            if not resp.get('response').get('more'):
                break
        return True
