# -*- encoding: utf-8 -*-

import json
import xmlrpclib, socket
from logging import getLogger
import requests
import time
import hmac
import hashlib
from openerp import pooler
from openerp.tools.translate import _
import openerp.addons.web.http as http
from ..marketplace.control.utils import configuracao
from SigUtils.geralogs import geraLogs
from SigUtils.sigerp import db_sig
from openerp.tools import config


_logger = getLogger("Shopee")
_senha = 'mkp_remoto_77813'


def _remover_acentuacao(text, cod='utf-8'):
    """
    Funcao Wrapper para remover acentuacao
    :param text:texto onde sera feita a remocao da acentuacao
    :param cod:Tipo de codificacao
    return:string
    """

    from unicodedata import normalize, category
    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        return normalize('NFKD', text).encode('ASCII', 'ignore')
    return unicode(filter(lambda c: category(c) != 'Mn',
                          normalize('NFKD', unicode_string.decode(cod))))


class Shopee(object):
    def logged_client(f):
        """Decorator for User Credentials checking."""

        def inner(self, *args, **kargs):
            if ((self.client_id is None) or (self.client_secret is None)):
                raise Exception(_("Nao posso acessar o Shopee, parametros do Shopee nao informados!"))
            return f(self, *args, **kargs)

        return inner

    def __init__(self, cr=False, uid=False, conta_id=False, producao=True):
        self.API_ROOT_URL_TESTE = 'https://partner.test-stable.shopeemobile.com'  # Sandbox
        self.API_ROOT_URL = 'https://partner.shopeemobile.com'  # Producao
        self.API_ROOT_URL = 'https://openplatform.shopee.com.br'  # Producao
        self.SDK_VERSION = 'SIG.1.01'
        self.AUTH_URL = '/api/v2/shop/auth_partner'
        self.cr = cr
        self.uid = uid
        self.shop_id = ''
        self.partner_id = ''
        self.partner_key = ''
        self.redirect_url = ''
        self.access_token = ''
        self.refresh_token = ''
        self.code = ''
        self.habilitar_log = False
        self.conta_id = ''
        self.comunicar = True
        if cr:
            _poolget = pooler.get_pool(cr.dbname).get
            mkp_ids = _poolget('mkp.marketplaces').search(cr, uid, [('mkpmkp_codigo', '=', 'Shopee')])
            if mkp_ids:
                mkp_ids = mkp_ids[0]
                dados_marketplace = _poolget('mkp.marketplaces').read(cr, uid, mkp_ids, [])
                if dados_marketplace.get('mkpmkp_parar_comunicacao'):
                    self.comunicar = False
                if not conta_id:
                    conta_id = _poolget('mkp.contas').search(cr, uid, [('mkpcta_marketplace_id', '=', mkp_ids)])
                    if conta_id:
                        conta_id = conta_id[0]
                if conta_id:
                    self.conta_id = conta_id
                    dados_conta = _poolget('mkp.contas').read(cr, uid, conta_id, [])
                    if dados_conta.get('mkpcta_parar_comunicacao'):
                        self.comunicar = False
                    self.shop_id = str(dados_conta.get('mkpcta_shopee_shop_id'))
                    self.partner_id = str(dados_conta.get('mkpcta_shopee_partner_id'))
                    self.partner_key = str(dados_conta.get('mkpcta_shopee_partner_key'))
                    self.redirect_url = dados_conta.get('mkpcta_shopee_redirect_url')
                    self.code = dados_conta.get('mkpcta_shopee_code')
                    self.habilitar_log = dados_marketplace.get('mkpmkp_habilita_log')
                    self.access_token = dados_conta.get('mkpcta_shopee_token')
                    self.refresh_token = dados_conta.get('mkpcta_shopee_refresh_token')
                    if not dados_conta.get('mkpcta_producao'):
                        self.API_ROOT_URL = self.API_ROOT_URL_TESTE
        else:
            if not producao:
                self.API_ROOT_URL = self.API_ROOT_URL_TESTE

    def _get_arq_log(self):
        _arq_log = 'mkp_shopee_' + self.cr.dbname
        return _arq_log

    def _gerar_logs(self, mens):
        _diretorio = config.get('logfile')
        if _diretorio:
            _lista = _diretorio.split('/')
            _lista.pop(len(_lista) -1)
            _diretorio = '/'.join(_lista)
        else:
            _diretorio = '/var/log/openerp'
        _arq_log = self._get_arq_log()
        try:
            geraLogs(_arq_log, mens, 'S', _diretorio)
        except:
            try:
                geraLogs(_arq_log, mens.encode('ASCII', 'ignore'), 'S', _diretorio)
            except:
                geraLogs(_arq_log, 'Nao consegui registrar o log', 'S', _diretorio)
        return True


    # AUTH METHODS
    def auth_url(self):
        timest = int(time.time())
        base_string = "%s%s%s" % (self.partner_id, self.AUTH_URL, timest)
        sign = hmac.new(self.partner_key, base_string, hashlib.sha256).hexdigest()
        url = self.API_ROOT_URL + self.AUTH_URL + "?partner_id=%s&timestamp=%s&sign=%s&redirect=%s" \
              % (self.partner_id, timest, sign, self.redirect_url)
        return url

    def shopee_comunicar(self, metodo, path, body, params={}, headers={}, no_headers=False, payload={}):
        _spg = pooler.get_pool
        _metodo = metodo.upper()

        def comunicar():
            _body = body
            if _metodo == 'GET':
                params.update(_body)
                _body = {}
            if self.habilitar_log:
                self._gerar_logs('Shopee.%s -> PATH:%s -> BODY:%s -> PARAMS:%s-> HEADERS:%s' % (_metodo, url, _body, params, headers))
            if _metodo == 'UPLOAD':
                if payload:
                    response = requests.request('POST', url, files=_body, data=payload, params=params, headers=headers)
                else:
                    response = requests.request('POST', url, files=_body, params=params, headers=headers)
            else:
                session = requests.Session()
                req = requests.Request(_metodo, url, json=_body, params=params)
                prepped = req.prepare()
                if no_headers:  # Qdo não tiver headers, usar o que está informando, se tiver
                    prepped.headers = headers
                response = session.send(prepped)
            if self.habilitar_log:
                self._gerar_logs('Status:%s Resposta: %s' % (response.status_code, response.text))
            return response

        while True:
            timest = int(time.time())
            url = self.API_ROOT_URL + path
            # if 'token' in url or 'upload_image' in url:
            if 'token' in url:
                base_string = "%s%s%s" % (int(self.partner_id), path, timest)
            else:
                base_string = "%s%s%s%s%s" % (int(self.partner_id), path, timest, self.access_token, int(self.shop_id))
            sign = hmac.new(self.partner_key, base_string, hashlib.sha256).hexdigest()
            default_parameter = {
                'partner_id': int(self.partner_id),
                'sign': sign,
                'timestamp': timest,
            }
            if not 'token' in url:
                default_parameter['access_token'] = self.access_token
                default_parameter['shop_id'] = int(self.shop_id)

            if not params:
                params = {}
            params.update(default_parameter)
            resp = comunicar()
            if resp.status_code == 200:
                try:
                    ret = json.loads(resp.content)
                    break
                except:
                    ret = {'dados': resp.content}
                    break
            elif resp.status_code == 403 and \
                 json.loads(resp.content).get('error') == 'error_auth' and \
                 json.loads(resp.content).get('message') == 'Invalid access_token.':
                    resp = self.get_refresh_token()
                    if not resp:
                        ret = {'error': 'token_invalido', 'message': 'Refresh Token invalido!'}
                        break
            else:
                try:
                    _mens = json.loads(resp.content).get('message')
                except:
                    _mens = resp.content
                ret = {'error': resp.status_code, 'message': _mens}
                break
        return ret

    def authorize(self):
        body = {
            'code': self.code,
            'partner_id': int(self.partner_id),
            'shop_id': int(self.shop_id),
        }
        path = '/api/v2/auth/token/get'
        response = self.shopee_comunicar('post', path, body)
        if response.get('error'):
            _logger.info(_('Shopee: Nao consegui pegar o token. Erro: %s') % response.get('message'))
            return False
        else:
            self.access_token = response['access_token']
            if 'refresh_token' in response:
                self.refresh_token = response['refresh_token']
            else:
                self.refresh_token = ''  # offline_access not set up
            return self.access_token

    def get_refresh_token(self):
        body = {
            'refresh_token': self.refresh_token,
            'partner_id': int(self.partner_id),
            'shop_id': int(self.shop_id),
        }
        path = '/api/v2/auth/access_token/get'
        response = self.shopee_comunicar('post', path, body)
        if response.get('error'):
            _logger.info(_('Shopee: Nao consegui pegar o token. Erro: %s') % response.get('message'))
            return False
        else:
            self.access_token = response['access_token']
            if 'refresh_token' in response:
                self.refresh_token = response['refresh_token']
            else:
                self.refresh_token = ''  # offline_access not set up
            vals = {
                'mkpcta_shopee_token': self.access_token,
                'mkpcta_shopee_refresh_token': self.refresh_token
            }
            _poolget = pooler.get_pool(self.cr.dbname).get
            _poolget('mkp.contas').write(self.cr, self.uid, self.conta_id, vals)
            self.cr.commit()
            return self.access_token

class sc(http.Controller):
    _cp_path = '/mkp_shopee'

    def teste(self, req, s_action=None, **kw):
        _logger.info('=============================== teste %s' % req)
        # return 'testado IP:%s' % req.httprequest.headers.environ.get('REMOTE_ADDR')
        # sys.exit(1)
        return 'Ok'

    def verify_push_msg(self, url, request_body, partner_key, authorization):
        base_string = '%s|%s' % (url, request_body)
        cal_auth = hmac.new(partner_key, base_string.encode(), hashlib.sha256).hexdigest()
        if cal_auth != authorization:
            return False
        else:
            return True


    @http.httprequest
    def authorize(self, req, s_action=None, **kw):
        _shop_id = req.params.get('shop_id')
        _tg_code = req.params.get('code')
        _logger.info('Shopee - Autorizando: %s' % _shop_id)
        cfg = configuracao('marketplace_shopee', _shop_id)
        cfg.lerConfiguracao()
        _server = str(cfg.server)
        _base = str(cfg.base)
        template = "<html><header>Retorno do Shopee</header><body>Nao consegui autorizar!<br>%s</body></html>" % str(
            req.params)
        _logger.info('=== mkp_shopee/authorize === %s' % str(req.params))
        if _tg_code:
            if _server:
                _logger.info('%s/xmlrpc/common' % _server)
                sock_common = xmlrpclib.ServerProxy('%s/xmlrpc/common' % _server)
                socket.setdefaulttimeout(3)
                uid = sock_common.login(_base, 'mkp_remoto', _senha)
                sock = xmlrpclib.ServerProxy('%s/xmlrpc/object' % _server)
                for _conta_id in sock.execute(_base, uid, _senha, 'mkp.contas', 'search',
                                              [('mkpcta_shopee_shop_id', '=', _shop_id)]):
                    dados_marketplace = sock.execute(_base, uid, _senha, 'mkp.contas', 'read', _conta_id, [])
                    _prod = True
                    if dados_marketplace:
                        _prod = dados_marketplace.get('mkpcta_producao')
                    sh_con = Shopee(producao=_prod)
                    sh_con.code = req.params.get('code')
                    sh_con.partner_id = cfg.partner_id
                    sh_con.shop_id = _shop_id
                    sh_con.partner_key = cfg.partner_key
                    sh_con.authorize()
                    vals = {
                        'mkpcta_shopee_code': _tg_code,
                        'mkpcta_shopee_token': sh_con.access_token,
                        'mkpcta_shopee_refresh_token': sh_con.refresh_token
                    }
                    sock.execute(_base, uid, _senha, 'mkp.contas', 'write', _conta_id, vals)
                    _logger.info('Code: %s' % str(_tg_code))
                    template = """
                    <html>
                       <header>Retorno do Shopee</header>
                       <body>Code:%s<br>Access token: %s<br>Refresh token:%s<br></body>
                    </html>""" % (_tg_code, sh_con.access_token, sh_con.refresh_token)
            else:
                _logger.info('Não encontrei servidor no arquivo de shopee.config para a loja: %s') % str(_shop_id)
        return template

    @http.httprequest
    def notifications(self, req, s_action=None, **kw):
        _logger.info('--------------------- notificacoes: %s ' % str(req.httprequest.data))
        dados_post = json.loads(req.httprequest.data)
        if dados_post.get('code') in (0, 1):  # Somente teste de conexao
            _logger.info('Notificacao processada!-1')
            return 'Ok'
        _code = dados_post.get('code')
        _shop_id = dados_post.get('shop_id')
        if _shop_id:
            cfg_shopee = configuracao('marketplace_shopee', str(_shop_id))
            cfg_shopee.lerConfiguracao()
            _url = req.httprequest.base_url
            _authorization = req.httprequest.environ.get('HTTP_AUTHORIZATION')
            _partner_key = cfg_shopee.partner_key
            # res = self.verify_push_msg(_url, req.httprequest.data, _partner_key, _authorization)
            _base = str(cfg_shopee.base)
            cfg = configuracao('marketplace', _base)
            cfg.lerConfiguracao()
            _db_user = cfg.db_user
            _db_password = cfg.db_password
            _db_host = cfg.db_host
            _db_port = cfg.db_port
            arg_db = {}
            if _db_host:
                arg_db.update({'host': _db_host})
            else:
                arg_db.update({'host': 'localhost'})
            if _db_port:
                arg_db.update({'port': _db_port or '5431'})
            if _db_user:
                arg_db.update({'user': _db_user})
            if _db_password:
                arg_db.update({'pwd': _db_password})
            _db = db_sig(_base, **arg_db)
            _fields = ['id']
            _where = [('login', '=', 'mkp_remoto')]
            dados_users = _db.dbsSelect('res_users', _fields, _where)
            _uid = False
            if dados_users:
                _uid = dados_users[0].get('id')
                if _uid:
                    if _code == 3:
                        _topic = 'orders'
                        _resource = '/%s/%s' % (_topic, dados_post.get('data').get('ordersn'))
                    else:
                        _db.conexao.close()
                        return 'Ok'
                    _db.uid = _uid
                    _where = [
                        ('mkpnot_resource', '=', _resource),
                        ('mkpnot_loja', '=', _shop_id)
                    ]
                    dados_not = _db.dbsSelect('mkp_notificacoes', ['id'], _where)
                    _not_id = False
                    if dados_not:
                        _not_id = dados_not[0].get('id')
                    ## Gravar as notificacoes
                    if _not_id:
                        vals = {
                            'mkpnot_attempts': 1,
                            'state': 'R',
                        }
                        where = [('id', '=', _not_id)]
                        x = _db.dbsUpdate('mkp_notificacoes', vals, where)
                        if x:
                            _db.conexao.commit()
                        else:
                            _db.conexao.roolback()
                    else:
                        vals = {
                            'mkpnot_conta_id': cfg_shopee.conta_id,
                            'mkpnot_resource': _resource,
                            'mkpnot_loja': _shop_id,
                            'mkpnot_topic': _topic,
                            'mkpnot_attempts': 1,
                            'state': 'R',
                        }
                        x = _db.dbsInsert('mkp_notificacoes', vals)
                        if not x:
                            _db.conexao.roolback()
                        else:
                            _db.conexao.commit()
                    _logger.info('Notificacao processada!-2')
                    _db.conexao.close()
                    return 'OK'  # nunca retorne booleano, vai dar erro de https
            _db.conexao.close()
        _logger.info('Nao consegui pegar o shop_id no Mkp Shopee!')
        return 'Ok'  # nunca retorne booleano, vai dar erro de https
