# -*- encoding: utf-8 -*-
# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from ...adm.control.utils import capitalize_all
import re
import logging
from openerp.tools.translate import _
import subprocess
from openerp.tools import config
from SigUtils.formatastring import formataString
from datetime import datetime
from pynfe.nfe.manual_600.consrecinfe import ProcNFe
from pynfe.nfe.manual_600.carta_correcao import ProcEventoNFe
from dateutil.parser import parse

class mkp_utils(osv.Model):
    _inherit = 'mkp.utils'
    _logger = logging.getLogger(_inherit)

    def mkputl_entregar_no_mkp(self, cr, uid, brw_encomenda, context=None):
        """
        Metodo utilizado para informar aos marketplaces que o pedido foi entregue ao cliente
        Deve ser sobrescrito em cada marketplace
        :param cr:
        :param uid:
        :param brw_encomenda:
        :param context:
        :return:
        """
        if brw_encomenda.rasenc_pedido_id.pedcap_conta_id.mkpcta_integrador != 'SkyHub':
            return super(mkp_utils, self).mkputl_entregar_no_mkp(cr, uid, brw_encomenda, context=context)
        _conta_id = brw_encomenda.rasenc_pedido_id.pedcap_conta_id.id
        _order = brw_encomenda.rasenc_pedido_id.pedcap_nro_pedido_externo
        if not _order:
            return True
        if _order:
            self.pool.get('mkp.skyhub').mkpSkyHub_entregar_pedido(cr, uid, _conta_id, _order, context=context)
        return True

    def mkputl_postar_no_mkp(self, cr, uid, mkp_integrador, _conta_id, _ped_id, codigo_rastreio, data, context=None):
        """
        Metodo utilizado para informar aos marketplaces que o pedido foi postado.
        Deve ser sobrescrito em cada marketplace
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param _conta_id:
        :param _ped_id:
        :param data:
        :param context:
        :return:
        """
        if mkp_integrador != 'SkyHub':
            return super(mkp_utils, self).mkputl_postar_no_mkp(cr, uid, mkp_integrador, _conta_id, _ped_id, codigo_rastreio, data, context=context)
        dados_ped = self.pool.get('estoquepedidos.pedido.capa').read(cr, uid, _ped_id,
                                                                  ['pedcap_nro_pedido_externo'], context)
        _order = dados_ped[0].get('pedcap_nro_pedido_externo')
        if _order:
            res = self.pool.get('mkp.skyhub').mkpSkyHub_postar_pedido(cr, uid, _conta_id, _order, codigo_rastreio, context=context)
        return True


    def mkputl_cron_pesquisar_nfe_ret_simb_skyhub(self, cr, uid, context=None):
        """
        Método chamado pelo cron para pesquisar NFes de retorno simbolico do fulfillment da Americanas
        @param cr:
        @param uid:
        @param context:
        @return: Sempre True
        """
        context = context and context.copy() or {}
        obj_contas = self.pool.get('mkp.contas')
        conta_ids = obj_contas.search(cr, uid, [('mkpcta_integrador', '=', 'SkyHub')], context=context)
        if conta_ids:
            for brw_conta in obj_contas.browse(cr, uid, conta_ids, context=context):
                _conta_id = brw_conta.id
                _mkp_codigo = brw_conta.mkpcta_marketplace_id.mkpmkp_codigo
                _cnpj_ful = brw_conta.mkpcta_fulfillment_cnpj
                _company_ful = brw_conta.mkpcta_fulfillment_empresa_id.id
                self._mkputl_pesquisar_nfe_ret_simbol_mkp_skyhub(cr, uid, _conta_id, _cnpj_ful, _company_ful,
                                                                 context=context)
        return True


    def _mkputl_importar_uma_nota(self, cr, uid, xml, conta_id, context=None):
        """
        Importa nota de retorno simbolico emitida pela americanas
        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        nfe = ProcNFe()
        nfe.xml = xml
        _nro_nf = nfe.NFe.infNFe.ide.nNF.valor
        _serie = nfe.NFe.infNFe.ide.serie.txt
        _company_id = context.get('sig_force_company')
        _cnpj = nfe.NFe.infNFe.emit.CNPJ.txt
        _pessoa_ids = _spg('adm.pessoa').search(cr, uid, [('cpf_cnpj', '=', _cnpj)], context=context)
        if not _pessoa_ids:
            return {'status': False, 'mensagem': _('Cnpj nao cadastrado! cnpj: %s') % _cnpj}
        _pessoa_id = _pessoa_ids[0]
        args = [('company_id', '=', _company_id),
                ('destinatario_id', '=', _pessoa_id),
                ('serie_nf', '=', _serie),
                ('numero_nf', '=', +_nro_nf)]
        _nf_ids = _spg('fiscal.nf.capa').search(cr, uid, args, context=context)
        if _nf_ids:
            return {'status': True, 'mensagem': '', 'nf_id': _nf_ids[0]}
        brw_pessoa = _spg('adm.pessoa').browse(cr, uid, _pessoa_id, context=context)
        user_ids = _spg('res.users').search(cr, uid, [('login', '=', 'mkp_user_nf')], context=context)
        if not user_ids:
            return {'status': False, 'mensagem': _('Usuario nao cadastrado para o login: mkp_user_nf')}

        uid = user_ids[0]
        _user_dados = _spg('res.users').read(cr, uid, uid, ['pessoa_id'], context=context)
        if not _user_dados:
            return {'status': False, 'mensagem': 'O Usuario %s nao tem uma pessoa_id!' % uid}
        _user_pessoa_id = _user_dados.get('pessoa_id')[0]
        _args = [('company_id', '=', context.get('sig_force_company')), ('pessoa', '=', _user_pessoa_id)]
        _func_ids = _spg('rh.funcionario').search(cr, uid, _args, context=context)
        if not _func_ids:
            _func_id = _spg('rh.funcionario').create(cr, uid, {'company_id': _company_id, 'pessoa': _user_pessoa_id},
                                                   context=context)
        else:
            _func_id = _func_ids[0]
        _modelo_ids = _spg('fiscal.modelo.nf').search(cr, uid, [('fismnf_codigo', '=', '55'), ('active', '=', True)],
                                                      context=context)
        if not _modelo_ids:
            return {'status': False, 'mensagem': _('Modelo de nf nao cadastrado: 55')}
        _modelo_id = _modelo_ids[0]
        _natureza = nfe.NFe.infNFe.ide.natOp.txt
        _dt_emis = str(parse(nfe.NFe.infNFe.ide.dhEmi.txt))
        _dt_emis = _spg('adm.utils').admutl_converte_date_tz(_dt_emis)
        _cfop = nfe.NFe.infNFe.det[0].prod.CFOP.txt
        if _cfop == '5907':
            _cfop = '1907'
        elif _cfop == '6907':
            _cfop = '2907'
        _cfop_ids = _spg('fiscal.cfop').search(cr, uid, [('codigo', '=', _cfop)], context=context)
        _nat_ids = _spg('fiscal.cfop.natureza').search(cr, uid, [('fisnat_natureza_fiscal', '=', _natureza),
                                                                 ('cfop_id', '=', _cfop_ids[0])], context=context)
        if not _nat_ids:
            return {'status': False, 'mensagem': _('Natureza nao cadastrada: "%s" e cfop: %s') % (_natureza, _cfop)}
        brw_natureza = _spg('fiscal.cfop.natureza').browse(cr, uid, _nat_ids[0], context=context)
        _campos = ['id', 'condicao_pgto_transferencia_id','company_id','fispar_natureza_uf_dentro_venda_id',
                   'fispar_natureza_uf_fora_venda_id', 'fispar_natureza_uf_dentro_devolucao_id',
                   'fispar_natureza_uf_fora_devolucao_id', 'fispar_natureza_transferencia_saida_id']
        _dados_parametros = _spg('adm.utils').admutl_get_parametros(cr, uid, 'fiscal.parametro', _campos, context=context)
        _param_id = _dados_parametros.get('id')
        _param_company_id = _dados_parametros.get('company_id')
        _nf_ref_chave = nfe.NFe.infNFe.ide.NFref[0].refNFe.txt
        _nf_ref_nro = _nf_ref_chave[28:34]
        args = [('nfe_chave_acesso', '=', _nf_ref_chave)]
        _nf_ref_ids = _spg('fiscal.nf.capa').search(cr, uid, args, context=context)
        if not _nf_ref_ids:
            return {'status': False,
                    'mensagem': _('Nota fiscal referenciada nao encontrada! Chave: %s') % _nf_ref_chave}
        vals = {'tipo': 'E',
                'fisnfc_tipo_servico': False,
                'company_id': _company_id,
                'numero_nf': _nro_nf,
                'serie_nf': _serie,
                'nf_propria': False,
                'natureza_id': _nat_ids[0],
                'modelo_nf_id': _modelo_id,
                'destinatario_id': _pessoa_id,
                'destinatario_endereco_id': brw_pessoa.endereco_nf_id.id,
                'data_emissao': _dt_emis[:19],
                'hora_emissao': _dt_emis[:19],
                'data_saida_entrada': _dt_emis[:19],
                'modelo_55': True,
                'fisnfc_consumidor': False,
                'gerar_xml': False,
                'nfe_forma_impressao_danfe': '1',
                'nfe_processo_emissao': '0',
                'fisnfc_nfe_indicador_presenca': '0',
                'nfe_versao_processo_emissao': '',
                'nfe_forma_emissao': nfe.NFe.infNFe.ide.tpEmis.txt,
                'finalidade_emissao': nfe.NFe.infNFe.ide.finNFe.txt,
                'nfe_protocolo': nfe.protNFe.infProt.nProt.txt,
                'nfe_status': '%s - %s' % (nfe.protNFe.infProt.cStat.txt, nfe.protNFe.infProt.xMotivo.txt),
                'nfe_chave_acesso': nfe.protNFe.infProt.chNFe.txt,
                'vr_desconto': nfe.NFe.infNFe.total.ICMSTot.vDesc.valor.real,
                'vr_icms_substituicao': 0.0,
                'vr_base_icms_substituicao': 0.0,
                'vr_seguro': 0.0,
                'vr_despesas_acessorias': 0.0,
                'vr_frete': 0.0,
                'vr_total_nf': nfe.NFe.infNFe.total.ICMSTot.vNF.valor.real,
                'frete': '0',
                'fisnfc_infcomp_contribuinte': nfe.NFe.infNFe.infAdic.infCpl.txt,
                'fisnfc_infcomp_fisco': nfe.NFe.infNFe.infAdic.infAdFisco.txt,
                'state': '3',
                'nfe_xml_auto': xml,
                }
        context['nf_capa_manutencao'] = True
        try:
            nf_id = _spg('fiscal.nf.capa').create(cr, uid, vals, context=context)
        except Exception as e:
            return {'status': False, 'Mensagem': e}
        context.pop('nf_capa_manutencao')
        if not nf_id:
            return {'status': False, 'Mensagem': _('Nao consegui criar a nota fiscal que referencia a nf: %s') % _nf_ref_nro}
        res = _spg('fiscal.nf.capa').write(cr, uid, _nf_ref_ids, {'fisnfc_nf_referencia_id': nf_id}, context=context)
        #### Criar capa do moviment do estoque
        vals = {'company_id': _company_id,
                'nf_capa_id': nf_id,
                'funcionario_id': _func_id,
                'estmvc_operacao_id': brw_natureza.fisnat_operacao_id.id,
                'data': _dt_emis[:10],
                'state': 'F',
        }
        mov_capa_id = _spg('estoque.movimento.capa').create(cr, uid, vals, context=context)

        ####   Criar itens do movimento do estoque e fiscal item tributacao
        nf_item_ids = list()
        itens_ids = list()
        _total_itens = nfe.NFe.infNFe.total.ICMSTot.vNF.valor
        i = 0
        _sequencia = 0
        for item_nf in nfe.NFe.infNFe.det:
            i += 1
            _ean = item_nf.prod.cEAN.txt
            barra_ids = _spg('automacao.codigo.barra').search(cr, uid, [('codigo_barra', '=', _ean)],
                                                              context=context)
            if not barra_ids:
                return {'status': False,
                        'mensagem': _('Codigo de barras %s da nf %s nao cadastrado!') % (_ean, _nf_ref_nro)}
            dados_barra = _spg('automacao.codigo.barra').read(cr, uid, barra_ids[0], ['grade_id','produto_id'],
                                                              context=context)
            _grade_id = dados_barra.get('grade_id')[0]
            _prod_id = dados_barra.get('produto_id')[0]
            _campos = ['codigo', 'estpro_cofins_nat_rec']
            _dados_produto = _spg('estoque.produto').read(cr, uid, _prod_id, _campos, context=context)
            _codigo = _dados_produto.get('codigo')
            nat_rec_pis_cofins = _dados_produto.get('estpro_cofins_nat_rec')
            dados_conta = _spg('mkp.contas').read(cr, uid, conta_id, ['mkpcta_fulfillment_local_id'], context=context)
            _local_id = dados_conta.get('mkpcta_fulfillment_local_id')[0]
            args = [('company_id', '=', _company_id), ('grade_id', '=', _grade_id), ('local_id', '=', _local_id)]
            _local_locacao_ids = _spg('estoque.local.locacao').search(cr, uid, args, context=context)
            if not _local_locacao_ids:
                return {'status': False, 'mensagem': 'Local locacao, nao cadastrado para o item: %s' % _codigo}
            dados_local_locacao = _spg('estoque.local.locacao').read(cr, uid, _local_locacao_ids[0],
                                                                     ['estllc_produto_embalagem_id'],
                                                                     context=context)
            _vr_total = item_nf.prod.vProd.valor
            _cfop_ids = _spg('fiscal.cfop').search(cr, uid, [('codigo', '=', _cfop)], context=context)
            vals_custos = _spg('estoque.preco.custo').get_custos_do_dia(cr, uid, _prod_id, _company_id,
                                                                    _dt_emis[:10], context)
            custos = vals_custos.get(_prod_id) or {}
            _quantidade = int(item_nf.prod.qCom.valor.real)
            _custo_corrigido = (custos.get('estprc_custo_corrigido') or 0) * _quantidade
            _ultimo_custo = (custos.get('estprc_ultimo_custo') or 0) * _quantidade
            _custo_reposicao = (custos.get('estprc_custo_reposicao') or 0) * _quantidade
            _custo_medio_contabil = (custos.get('estprc_custo_medio_contabil') or 0) * _quantidade
            _vr_desconto_item = 0
            _sequencia += 1
            vals = {
                'local_id': _local_id,
                'nf_id': nf_id,
                'sequencia': _sequencia,
                'cfop_id': _cfop_ids[0],
                'capa_id': mov_capa_id,
                'natureza_id': _nat_ids[0],
                'produto_id': _prod_id,
                'quantidade': _quantidade,
                'produto_embalagem_id': dados_local_locacao.get('estllc_produto_embalagem_id')[0],
                'grade_id': _grade_id,
                'vr_desconto': 0.0,
                'vr_desconto_rateado': _vr_desconto_item,
                'vr_despesas_rateado': 0.0,
                'estmvi_vr_despesas_extras_rateado': 0.0,
                'vr_preco_venda': _vr_total,
                'vr_unitario': _vr_total / _quantidade,
                'vr_total_item': _vr_total,
                'vr_total_item_rateado': _vr_total,
                'vr_frete_rateado':  0.0,
                'estmvi_saidas_custo_corrigido_total': _custo_corrigido,
                'estmvi_saidas_custo_medio_contabil_total': _custo_medio_contabil,
                'estmvi_saidas_ultimo_custo_total': _ultimo_custo,
                'estmvi_saidas_custo_reposicao_total': _custo_reposicao,
            }
            context['atualizacao_inventario'] = True  # contexto utilizado para apenas criar sem fazer outros tratamentos
            _item_id = _spg('estoque.movimento.itens').create(cr, uid, vals, context=context)
            context.pop('atualizacao_inventario')
            itens_ids.append(_item_id)
            for encargo in brw_natureza.natureza_encargos_ids:
                vals = {
                    'movimento_itens_id': _item_id,
                    'encargo_id': encargo.encargo_id.id,
                    'fisnit_capa_id': nf_id,
                    'aliquota': 0.0,
                    'vr_encargo': 0.0,
                    'vr_base_calculo': _vr_total,
                    'vr_reducao_base': 0.0,
                    'fisnit_nat_rec_pis_cofins': nat_rec_pis_cofins,
                    'cst_id': encargo.sobrepor_tributacao_id.cst_id.id,
                    'tributacao_id': encargo.sobrepor_tributacao_id.id,
                }
                _nf_item_id = _spg('fiscal.nf.item.tributacao').create(cr, uid, vals, context=context)
                nf_item_ids.append(_nf_item_id)
        context['super_write_estoque_movimento_itens'] = True
        context['fisnit_super_write'] = True
        res = _spg('estoque.movimento.itens').write(cr, uid, itens_ids, {}, context=context)
        res = _spg('fiscal.nf.item.tributacao').write(cr, uid, nf_item_ids, {}, context=context)
        context.pop('super_write_estoque_movimento_itens')
        context.pop('fisnit_super_write')
        res = _spg('fiscal.nf.capa').write(cr, uid, [nf_id], {}, context=context)
        return {'status': True, 'Mensagem': '', 'nf_id': nf_id}

    def _mkputl_pesquisar_nfe_ret_simbol_mkp_skyhub(self, cr, uid, conta_id, cnpj_ful, company_ful_id, context=None):
        """
        Metodo a ser sobrescrito para cada market place
        :param cr:
        :param uid:
        :param conta_id: id da conta no marketplace
        :param codigo_mkp: codigo de identificação do marketplace
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        self._logger.info('Inicio de pesquisa de NFes retorno simbolico SkyHub')
        context['sig_force_company'] = company_ful_id
        args = [('fisnfc_mkp_conta_id', '=', conta_id), ('fisnfc_numero_nota_empenho', '=', None),
                ('state', '=', '3')]
        nf_ids = _spg('fiscal.nf.capa').search(cr, uid, args, context=context)
        _campos = ['nfe_chave_acesso', 'numero_nf']
        for dados_nf in _spg('fiscal.nf.capa').read(cr, uid, nf_ids, _campos, context=context):
            _chave_nfe = dados_nf.get('nfe_chave_acesso')
            dados_nfe_skyhub = _spg('mkp.skyhub').mkpSkyHub_get_nf_ret_simbol_mkp(cr, uid, conta_id, cnpj_ful,
                                                                                           _chave_nfe, context=context)
            if not dados_nfe_skyhub:
                continue
            if not dados_nfe_skyhub.get('nfes'):
                continue
            xml = dados_nfe_skyhub.get('nfes')[0].get('xml_content')
            _hoje = _spg('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(),
                                                                  formato='%Y_%m_%d_%H_%M_%s')
            cr.execute(''' BEGIN; SAVEPOINT importar_notas_sky_%s; ''' % _hoje)
            sql_rollback = """ROLLBACK TO SAVEPOINT importar_notas_sky_%s; """ % _hoje
            sql_release = """RELEASE SAVEPOINT importar_notas_sky_%s; """ % _hoje
            res = self._mkputl_importar_uma_nota(cr, uid, xml, conta_id, context=context)
            if res.get('status'):
                vals = {'fisnfc_numero_nota_empenho': str(res.get('numero_nf'))}
                _spg('fiscal.nf.capa').write(cr, uid, dados_nf.get('id'), vals, context=context)
                cr.execute(sql_release)
                cr.commit()
            else:
                cr.execute(sql_rollback)

        self._logger.info('Fim de pesquisa de pedidos SkyHub')
        return True


    def mkputl_cron_pesquisar_pedidos_skyhub(self, cr, uid, context=None):
        """
        Método chamado pelo cron para pesquisar pedidos nos marketplaces
        @param cr:
        @param uid:
        @param context:
        @return: Sempre True
        """
        context = context and context.copy() or {}
        obj_contas = self.pool.get('mkp.contas')
        for company_id in self.pool.get('res.company').search(cr, uid, [], context=context):
            context['sig_force_company'] = True
            args = [('mkpcta_integrador', '=', 'SkyHub'), ('company_id', '=', company_id)]
            conta_ids = obj_contas.search(cr, uid, args, context=context)
            for brw_conta in obj_contas.browse(cr, uid, conta_ids, context=context):
                if brw_conta.mkpcta_parar_comunicacao or \
                   brw_conta.mkpcta_marketplace_id.mkpmkp_parar_comunicacao:
                    continue
                _conta_id = brw_conta.id
                _mkp_codigo = brw_conta.mkpcta_marketplace_id.mkpmkp_codigo
                self._mkputl_pesquisar_pedido_mkp_skyhub(cr, uid, _conta_id, _mkp_codigo, context=context)
        return True

    def _mkputl_pesquisar_pedido_mkp_skyhub(self, cr, uid, conta_id, codigo_mkp, context=None):
        """
        Metodo a ser sobrescrito para cada market place
        :param cr:
        :param uid:
        :param conta_id: id da conta no marketplace
        :param codigo_mkp: codigo de identificação do marketplace
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        self._logger.info('Inicio de pesquisa de pedidos SkyHub')
        while True:
            ped = self.pool.get('mkp.skyhub').mkpSkyHub_pesquisar_orders(cr, uid, conta_id, context=context)
            if not ped:
                break
        self._logger.info('Fim de pesquisa de pedidos SkyHub')
        return True


    def mkputl_importar_um_pedido_skyhub(self, cr, uid, ped, conta_id, context=None):
        def _mkputl_processar_ful():
            """
            Para fulfillment, se o pedido for do fulfillment, se estiver valido e se for para faturar pelo ERP,
            faturar o pedido, faturar a nf e enviar para o MKP
            """
            if _full and not brw_conta.mkpcta_faturar_pelo_mkp:
                _pedido_ids = obj_pedido.search(cr, uid, [('pedcap_nro_pedido_externo', '=', _sh_venda_id),
                                                          ('tipo', '=', 'V')], context=context)
                _dados_pedido = obj_pedido.read(cr, uid, _pedido_ids[0], ['state'], context=context)
                if _dados_pedido and _dados_pedido[0].get('state') == 'V':
                    obj_pedido.button_faturar_pedido(cr, uid, _pedido_ids, context=context)
                nf_ids = obj_nf_capa.search(cr, uid, [('fisnfc_pedido_ids', '=', _pedido_ids[0])], context=context)
                if not nf_ids:
                    return _pedido_id
                res = obj_nf_capa.read(cr, uid, nf_ids[0], ['state'], context=context)
                if not res:
                    return _pedido_id
                if res.get('state') == '1':  # Esboco -> Validar nf
                    obj_nf_capa.button_verificar_nf(cr, uid, nf_ids[0], context)
                    cr.commit()
                    res = obj_nf_capa.read(cr, uid, nf_ids[0], ['state'], context=context)
                if res.get('state') == 'EP':  # Lote em processamento -> processar lote
                    obj_nf_capa.fisnfc_button_processar_lote(cr, uid, nf_ids, context=context)
                    cr.commit()
                    res = obj_nf_capa.read(cr, uid, nf_ids[0], ['state'], context=context)
                if res.get('state') in ('2', '2E'):
                    obj_nf_capa.fisnfc_funcao_button_faturar(cr, uid, nf_ids, context)
                    cr.commit()
                    res = obj_nf_capa.read(cr, uid, nf_ids[0], ['state'], context=context)
                if res.get('state') == '3':
                    self.pool.get('mkp.utils').mkputl_faturar_no_mkp(cr, uid, [nf_ids[0]], context=context)
            return

        context = context and context.copy() or {}
        self._logger.info('Importando o pedido: %s' % ped.get('code'))
        _venda = ped
        obj_pedido = self.pool.get('estoquepedidos.pedido.capa')
        obj_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_local_locacao = self.pool.get('estoque.local.locacao')
        obj_produto = self.pool.get('estoque.produto')
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_contas = self.pool.get('mkp.contas')
        obj_mkputils = self.pool.get('mkp.utils')
        brw_conta = obj_contas.browse(cr, uid, conta_id, context=context)
        context['sig_force_company'] = brw_conta.company_id.id
        _local_id = brw_conta.mkpcta_local_id.id
        _hoje = self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(),
                                                                       formato='%Y_%m_%d_%H_%M_%s')
        cr.execute(''' BEGIN; SAVEPOINT importar_pedido_sh_%s; ''' % _hoje)
        _sh_venda_id = _venda.get('code')
        _comentario = ''
        _status_sh_name = _venda.get('status').get('type')
        _data_venda = _venda.get('placed_at')[:10]
        _lista_itens = []
        _qtd_venda = 0
        _vr_acrescimo = ped.get('interest')
        if ped.get('shipping_method') and ped.get('shipping_method') == 'B2W Fulfillment':
            _full = True
        else:
            _full = False
        context['sig_force_company'] = brw_conta.company_id.id
        if _full:
            if brw_conta.mkpcta_fulfillment_empresa_id:
                context['sig_force_company'] = brw_conta.mkpcta_fulfillment_empresa_id.id
                _local_id = brw_conta.mkpcta_fulfillment_local_id.id
        for _item in _venda.get('items'):
            _pedido_item = {}
            _sku = _item.get('id')
            _sku_code, _sku_grade = _sku.split('-')
            _produto_ids = obj_produto.search(cr, uid, [('codigo', '=', _sku_code)], context=context)
            if not _produto_ids:
                continue
            _produto_id = _produto_ids[0]
            _args = [('mkpanu_produto_id', '=', _produto_id),('mkpanu_conta_id', '=', conta_id)]
            _anuncio_ids = self.pool.get('mkp.anuncios').search(cr, uid, _args, context=context)
            if not _anuncio_ids:
                continue
            _frete_receita = 0
            _campos = ['mkpanu_valor_frete', 'mkpanu_frete', 'mkpanu_anuncio_tipo_id', 'mkpanu_quantidade']
            _dados_anuncio = self.pool.get('mkp.anuncios').read(cr, uid, _anuncio_ids[0], _campos, context=context)
            _grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid, [('produto_id', '=', _produto_id),
                                                                                 ('grade', '=', _sku_grade)],
                                                                       context=context)
            _qtd_anuncio = _dados_anuncio.get('mkpanu_quantidade', 1)
            if _qtd_anuncio == 0:
                _qtd_anuncio = 1
            _qtd_venda =int((_item.get('qty') or 1) * _qtd_anuncio)
            _valor_total_item = (float(_item.get('special_price')) / _qtd_anuncio) * _qtd_venda
            _l_vl = _valor_total_item / _qtd_venda
            _args = [('grade_id', '=', _grade_ids[0]), ('local_id', '=', _local_id)]
            _emb_ids = obj_local_locacao.search(cr, uid, _args, context=context)
            _emb_id = \
                obj_local_locacao.read(cr, uid, _emb_ids[0], ['estllc_produto_embalagem_id'],
                                       context=context)['estllc_produto_embalagem_id'][0]
            _pedido_item = self.pool.get('mkp.utils')._mkputl_monta_dic_item_pedido(cr, uid, _produto_id,
                                                                                    _grade_ids[0], _local_id, _emb_id,
                                                                                    _qtd_venda, _l_vl, _anuncio_ids[0])
            for x in _pedido_item:
                _lista_itens.append(x)
        _preco_total = float(_venda.get('total_ordered') or 1)
        _cliente_sh_email = _venda.get('customer').get('email')
        _cliente_sh_fone_1 = _cliente_sh_fone_2 = _cliente_sh_ddd_1 =  _cliente_sh_ddd_2 = False
        i = 0
        for _fone in _venda.get('customer').get('phones'):
            i += 1
            if i > 2:
                break
            _ddd = ''
            _fone = _fone.strip().replace('-', '').replace('.','').replace('/','').replace(' ','')
            while True: # retirar zeros à esquerda
                if _fone[:1] == '0':
                    _fone = _fone[1:]
                else:
                    break
            if len(_fone) > 8:
                ddd = _fone[:2]
                _nro_f = _fone[2:]
            if i == 1:
                _cliente_sh_ddd_1 = ddd
                _cliente_sh_fone_1 = _fone
            else:
                _cliente_sh_ddd_2 = ddd
                _cliente_sh_fone_2 = _fone
        _cliente_nome = _venda.get('billing_address').get('full_name') or _venda.get('customer').get('name')
        _cliente_nome = capitalize_all(_cliente_nome)
        _cliente_cpf = re.sub('[^0-9]', '', _venda.get('customer').get('vat_number') or '')
        _cliente_cep = _cliente_rua = _cliente_rua_nro = _cliente_rua_complemento = ''
        _cliente_cep = _venda.get('billing_address').get('postcode') or ''
        _cliente_rua = _venda.get('billing_address').get('street') or ''
        _cliente_rua_nro = _venda.get('billing_address').get('number') or ''
        _cliente_rua_complemento = _venda.get('billing_address').get('complement') or ''
        _cliente_bairro = _venda.get('billing_address').get('neighborhood') or ''
        _pedido_ids = obj_pedido.search(cr, uid, [('pedcap_nro_pedido_externo', '=', _sh_venda_id), ('tipo', '=', 'V')], context=context)
        _pedido_id = False
        _status_pedido_erp = False
        _pessoa_ids = False
        if _cliente_cpf:
            _pessoa_ids = obj_pessoa.search(cr, uid, [('cpf_cnpj', '=', _cliente_cpf)], context=context)
        _pais_id = False
        if _pessoa_ids:
            _pessoa_id = obj_mkputils.mkputl_orders_alterar_pessoa(cr, uid, _pessoa_ids[0], _cliente_nome, _cliente_cpf,
                                                                   '', '', _cliente_cep, _cliente_rua,
                                                                   _cliente_rua_nro, _cliente_rua_complemento,
                                                                   _cliente_sh_ddd_1, _cliente_sh_fone_1,
                                                                   _cliente_sh_ddd_2, _cliente_sh_fone_2, 'Comum',
                                                                   _cliente_sh_email, context=context)
        else:
            # TODO FAZER TRATAMENTO PARA ALTERAR OS DADOS DO CLIENTE POIS PODE TER TIDO ALTERAÇÃO NO Marketplace
            _pessoa_id = self.mkputl_orders_incluir_pessoa(cr, uid, _cliente_nome, _cliente_cpf, '',
                                                           '', '', _cliente_cep, _cliente_rua,
                                                           _cliente_rua_nro, _cliente_rua_complemento, _cliente_sh_ddd_1,
                                                           _cliente_sh_fone_1, _cliente_sh_ddd_2, _cliente_sh_fone_2,
                                                           'Comum', _cliente_sh_email, context=context)
        if _pedido_ids:  # O nro pedido do Marketplace é unico, portanto se tiver pedido so tem 1
            _pedido_id = _pedido_ids[0]
            _dados_pedido = obj_pedido.read(cr, uid, [_pedido_id], ['state'], context=context)
            _status_pedido_erp = _dados_pedido[0].get('state')
            if _status_pedido_erp == 'V': # Se o pedido estiver validado, voltar para esboco
                obj_pedido.button_rascunho(cr, uid, [_pedido_id], context=context)
                _status_pedido_erp = 'R'
            if _status_pedido_erp not in ('R', 'C'):  # O pedido não está mais em esboço ou cancelado, não deve ser alterado
                _mkputl_processar_ful()
                return _pedido_id
        _condicao_forma_id = False
        if ped.get('import_info') and ped.get('import_info').get('ss_name'):
            _point_sale = ped.get('import_info').get('ss_name')
        else:
            _point_sale = 'B2W'
        _tipo_anuncio = ''
        _vencimento = _data_venda
        _forma = []
        _condicao_forma_id = brw_conta.mkpcta_marketplace_id.mkpmkp_sh_condicao_pgto_id.id
        _taxa_tr_percentual = brw_conta.mkpcta_sh_comisao_mkp
        _taxa_tr_fixo = 0
        brw_condicao = brw_conta.mkpcta_marketplace_id.mkpmkp_sh_condicao_pgto_id
        for brw_adicional in brw_conta.mkpcta_marketplace_id.mkpmkp_mark_adicional_ids:
            if brw_adicional.mkpmkpadc_apelido == _point_sale:
                brw_condicao = brw_adicional.mkpmkpadc_condicao_pgto_id
                _taxa_tr_fixo = brw_adicional.mkpmkpadc_adicional_fixo
                _taxa_tr_percentual = brw_adicional.mkpmkpadc_adicional_percentual
                _forma1 = {
                    'forma_pagto_id': brw_adicional.mkpmkpadc_condicao_pgto_id.forma_pagamento_id.id,
                    'condicao_forma_id': brw_adicional.mkpmkpadc_condicao_pgto_id.id,
                    'pedido_parcela_ids': [(0, 0, {'vr_parcela': _preco_total, 'data_vencimento': _vencimento, })],
                    'valor': _preco_total,
                    'pedcdc_mkp_tarifa': (_taxa_tr_percentual * _preco_total / 100) + _taxa_tr_fixo,
                }
                _forma.append((0,0, _forma1))
                break

        dados_pessoa = self.pool.get('adm.pessoa').read(cr, uid, _pessoa_id, ['endereco_nf_id'], context=context)
        if dados_pessoa and dados_pessoa.get('endereco_nf_id'):
            _end_id = dados_pessoa.get('endereco_nf_id')[0]
        else:
            _end_id = ''
        vals = {
            'observacao': _comentario,
            'data_emissao': _data_venda,
            'tipo': 'V',
            'pessoa_id': _pessoa_id,
            'pedcap_endereco_id': _end_id,
            'pedcap_mkp_status': _status_sh_name,
            'pedcap_nro_pedido_externo': _sh_venda_id,
            'pedcap_endereco_eletronico': brw_conta.mkpcta_sh_site + 'orders/%s' % _sh_venda_id,
            'pedcap_mkp_tipo_anuncio': _tipo_anuncio,
            'vr_frete': _venda.get('shipping_cost'),
            'vr_desconto': 0.00,
            'vr_seguro': 0.00,
            'vr_despesas_acessorias': _vr_acrescimo,
            'pedcap_conta_id': conta_id,
            'pedcap_full': _full,
        }

        if not _pedido_id:
            if _anuncio_ids:
                vals['pedido_item_ids'] = _lista_itens
            if _forma:
                vals['pedido_condicao_ids'] = _forma
            _pedido_id = obj_pedido.create(cr, uid, vals, context=context)
            obj_pedido.write(cr, uid, [_pedido_id], {}, context=context)
            cr.execute(''' COMMIT; ''')  # Comito, porque se houver algum problema na validação, não perco o pedido
            cr.execute(''' BEGIN; SAVEPOINT importar_pedido_ml_%s; ''' % _hoje)
        else:
            if _status_sh_name == 'SHIPPED' and _status_pedido_erp in ('R', 'V'):
                obj_pedido.write(cr, uid, [_pedido_id], {'state': 'F'}, context=context)
            elif _status_sh_name != 'CANCELED' and _status_pedido_erp == 'C':
                return False # Marcar com problema
            elif _status_sh_name != 'CANCELED':
                if _status_pedido_erp == 'R':
                    _condicao_ids = self.pool.get('estoquepedidos.pedido.condicao').search(cr, uid, [('capa_id', '=', _pedido_id)], context=context)
                    if _condicao_ids and _forma:
                        self.pool.get('estoquepedidos.pedido.condicao').unlink(cr, uid, _condicao_ids, context=context)
                    vals['pedido_condicao_ids'] = _forma
                    vals['state'] = 'R'
                    context['salvar_forcado'] = True
                    res = obj_pedido.write(cr, uid, [_pedido_id], vals, context=context)
                    cr.execute(''' COMMIT; ''')  # Comito, porque se houver algum problema na validação, não perco o pedido
                    cr.execute(''' BEGIN; SAVEPOINT importar_pedido_ml_%s; ''' % _hoje)
        if _status_sh_name == 'APPROVED':
            try:
                obj_pedido.button_validar(cr, uid, [_pedido_id], context=context)
            except Exception as e:
                pass
        elif _status_sh_name == 'CANCELED' and _status_pedido_erp != 'C':
            if _status_pedido_erp != 'F':
                obj_pedido.button_cancelar(cr, uid, [_pedido_id], context=context)
            else:
                cr.execute(''' COMMIT; ''')
                return False # Marcar com problema, pedido faturado no ERP e cancelado mo MKP
        _encomenda = False
        if _venda.get('shipments'):
            for ship in _venda.get('shipments'):
                for track in ship.get('tracks'):
                    _encomenda = track.get('code')
                    try:
                        _servico_id = brw_conta.mkpcta_rastreamento_servico_id.id
                    except:
                        _servico_id = False
                    if _encomenda:
                        sql = "select id from rastreio_encomendas where name = '%s'" % _encomenda
                        cr.execute(sql)
                        _encomenda_id = cr.fetchone()
                        if _encomenda_id:
                            _encomenda_id = _encomenda_id[0]
                        else:
                            vals_encomenda = dict(
                                name=_encomenda,
                                rasenc_pedido_id=_pedido_id,
                                rasenc_pessoa_id=_pessoa_id,
                                rasenc_compra_venda='V',
                            )
                            if _servico_id:
                                vals_encomenda['rasenc_servicos_ids'] = [(4, _servico_id,)]
                            _encomenda_id = self.pool.get('rastreio.encomendas').create(cr, uid, vals_encomenda, context=context)
        if _encomenda:
            vals_ped = {'pedcap_encomenda_id': _encomenda_id}
            res = obj_pedido.write(cr, uid, [_pedido_id], vals_ped, context=context)
        cr.execute(''' COMMIT; ''')
        _mkputl_processar_ful()
        return _pedido_id

    def mkputl_faturar_no_mkp_selecionado(self, cr, uid, mkp_integrador, conta_id, ped_externo, chave, nf_ids, context=None):
        """
        Metodo sobrescrito ppelo marketplace especifico
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param conta_id:
        :param ped_externo:
        :param chave:
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        if mkp_integrador == 'SkyHub':
            for brw_nf in self.pool.get('fiscal.nf.capa').browse(cr, uid, nf_ids, context=context):
                if (brw_nf.fisnfc_mkp_conta_id and brw_nf.fisnfc_mkp_conta_id.mkpcta_fulfillment_empresa_id and
                    brw_nf.fisnfc_mkp_conta_id.mkpcta_fulfillment_empresa_id.id == brw_nf.company_id.id):
                    # Nota do fulfillment
                    _xml = brw_nf.nfe_xml_proc
                    _cnpj_cd = brw_nf.fisnfc_mkp_conta_id.mkpcta_fulfillment_cnpj
                    res = self.pool.get('mkp.skyhub').mkpSkyHub_full_enviar_xml(cr, uid, conta_id, _xml, _cnpj_cd,
                                                                                context=context)
                else:
                    # Nota do ERP
                    data = brw_nf.data_saida_entrada
                    res = self.pool.get('mkp.skyhub').mkpSkyHub_set_order_faturado(cr, uid, ped_externo, conta_id,
                                                                                   chave, data, context=context)
                for brw_itens in brw_nf.estoque_movimento_itens_ids:
                    for brw_anuncio in  brw_itens.produto_id.estpro_anuncios_ids:
                        if brw_anuncio.mkpanu_conta_id.id == conta_id:
                            _qtd = brw_anuncio.mkpanu_qtd_vendido + brw_itens.quantidade
                            vals = {'mkpanu_qtd_vendido': _qtd}
                            self.pool.get('mkp.anuncios').write(cr, uid, brw_anuncio.id, vals, context=context)
                            break
        else:
            res = super(mkp_utils, self).mkputl_faturar_no_mkp_selecionado(cr, uid, mkp_integrador, conta_id, ped_externo,
                                                                          chave, nf_ids, context=context)
        return res

    def mkputl_get_etiqueta_no_mkp(self, cr, uid, mkp_integrador, conta_id, pedido_ids, context=None):
        """
        Metodo para buscar etiqueta no marketplace, deve ser sobrescrito por cada marketplace
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param conta_id:
        :param pedido_ids:
        :param context:
        :return: {status: True/False, 'mensagem': '', 'dados': ''}
        """
        context = context and context.copy() or {}
        if mkp_integrador != 'SkyHub':
            res = super(mkp_utils, self).mkputl_get_etiqueta_no_mkp(cr, uid, mkp_integrador, conta_id, pedido_ids, context=context)
            return res
        for i in self.pool.get('estoquepedidos.pedido.capa').read(cr, uid, pedido_ids,
                                                                  ['pedcap_nro_pedido_externo', 'numero_pedido',
                                                                   'pedcap_nf_ids', 'pedcap_encomenda_id'], context):
            _ped = i.get('pedcap_nro_pedido_externo')
            _encomenda_id = i.get('pedcap_encomenda_id')
            if isinstance(_encomenda_id, tuple):
                _encomenda_id = _encomenda_id[0]
            if not _ped:
                return {'status': False, 'mensagem': 'Nao encontrei numero externo do pedido', 'dados': ''}
            _ped_local, _ped_nro = _ped.split('-')
            _plp = self.pool.get('mkp.skyhub').mkpSkyHub_get_set_plp(cr, uid, conta_id, _ped_nro, context=context)
            if not _plp:
                return {'status': False, 'mensagem': 'Nao consegui pegar a PLP', 'dados': ''}
            if isinstance(_plp, dict):
                return {'status': False, 'mensagem': 'Nao consegui pegar a PLP', 'dados': ''}
            res = self.pool.get('mkp.skyhub').mkpSkyHub_get_etiqueta_mkp(cr, uid, conta_id, _plp, context=context)
            if not res:
                return {'status': False, 'mensagem': 'Nao consegui pegar a Etiqueta', 'dados': ''}
            _nfe = ''
            for nf_id in i.get('pedcap_nf_ids'):
                _nfe = str(self.pool.get('fiscal.nf.capa').read(cr, uid, nf_id, ['numero_nf'], context=context)['numero_nf'])
            for _resumo in res.get('plp').get('resumoServicos'):
                _plp_servico_nome = _resumo.get('nomeServico')
                _plp_servico_cod = _resumo.get('codServico')
            for _docs in res.get('docsExternos'):
                if _docs.get('docExterno') != _ped_nro:
                    continue
                _data_prometida = _docs.get('dtPrometida')
                _telefone_contato = re.sub('[^0-9]', '', _docs.get('telefoneContato'))
                _vlEntrega = _docs.get('vlEntrega')
                _tpEntrega = _docs.get('tpEntrega')
                _numNotaFiscal = _docs.get('numNotaFiscal')
                _marca = _docs.get('marca')
                _codCliente = _docs.get('codCliente')
                _servicoAdicional = _docs.get('servicoAdicional')
                _pesoTotal = _docs.get('pesoTotal')
                _telefoneEmbarcador = _docs.get('telefoneEmbarcador')
                _tpServico = _docs.get('tpServico')
                _cartaoPostagem = _docs.get('cartaoPostagem')
                _contrato = _docs.get('numeroContratoTransp')
                _qtdVolumes = str(_docs.get('qtVolumes'))

                _rem_complemento = _docs.get('remetente').get('enderecoComplemento')
                _rem_cep = _docs.get('remetente').get('enderecoCep')
                _rem_cep = re.sub('[^0-9]', '', _rem_cep)
                _rem_cep = formataString(_rem_cep, 8, '0', 'E')
                _rem_cep_e = '%s.%s-%s' % (_rem_cep[:2], _rem_cep[2:5], _rem_cep[5:])
                _rem_nome = _docs.get('remetente').get('nome')
                _rem_cidade = _docs.get('remetente').get('enderecoCidade')
                _rem_logradouro = _docs.get('remetente').get('enderecoLogradouro')
                _rem_numero = _docs.get('remetente').get('enderecoNumero')
                _rem_bairro = _docs.get('remetente').get('enderecoBairro')
                _rem_uf = _docs.get('remetente').get('enderecoUf')

                _dest_complemento = _docs.get('destinatario').get('enderecoComplemento')
                _dest_cep = str(_docs.get('destinatario').get('enderecoCep'))
                _dest_cep = re.sub('[^0-9]', '', _dest_cep)
                _dest_cep = formataString(_dest_cep, 8, '0', 'E')
                _dest_cep_e = '%s.%s-%s' % (_dest_cep[:2], _dest_cep[2:5], _dest_cep[5:])
                _dest_nome = _docs.get('destinatario').get('nome')
                _dest_cidade = _docs.get('destinatario').get('enderecoCidade')
                _dest_logradouro = _docs.get('destinatario').get('enderecoLogradouro')
                _dest_numero = _docs.get('destinatario').get('enderecoNumero')
                _dest_bairro = _docs.get('destinatario').get('enderecoBairro')
                _dest_uf = _docs.get('destinatario').get('enderecoUf')

                _volumes = []
                for _awbs in _docs.get('awbs'):
                    _posicaoVolume = str(_awbs.get('posicaoVolume'))
                    _codigoAwb = _awbs.get('codigoAwb')
                    _lista_itens = []
                    for _itens in _awbs.get('itens'):
                        it = {
                            'quantidade': _itens.get('quantidade'),
                            'descricao' : _itens.get('descricao'),
                            'peso': _itens.get('peso')
                        }
                        _lista_itens.append(it)
                    # importar etiqueta
                    _arq = '%s/marketplace_skyhub/static/etiqueta_b2w.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _etiq = f.read()
                    f.close()
                    # importar logo da empresa
                    _arq = '%s/marketplace_skyhub/static/entregas_empresa.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _lg_empresa = f.read()
                    f.close()
                    # importar logo da americanas
                    _arq = '%s/marketplace_skyhub/static/entregas_americanas.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _lg_americanas = f.read()
                    f.close()
                    # importar logo da submarino
                    _arq = '%s/marketplace_skyhub/static/entregas_submarino.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _lg_submarino = f.read()
                    f.close()
                    # importar logo da shoptime
                    _arq = '%s/marketplace_skyhub/static/entregas_shoptime.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _lg_shoptime = f.read()
                    f.close()
                    # importar logo PAC
                    _arq = '%s/marketplace_skyhub/static/entregas_pac.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _lg_pac = f.read()
                    f.close()
                    # importar logo sedex
                    _arq = '%s/marketplace_skyhub/static/entregas_sedex.txt' % config.get('addons_path')
                    f = open(_arq, 'r')
                    _lg_sedex = f.read()
                    f.close()
                    _etiq = _etiq.replace('{logo_empresa}', _lg_empresa)
                    if 'PAC' in _plp_servico_nome:
                        _etiq = _etiq.replace('{logo_servico}', _lg_pac)
                    else:
                        _etiq = _etiq.replace('{logo_servico}', _lg_sedex)
                    if _marca == 'SUBA':
                        _etiq = _etiq.replace('{logo_entrega}', _lg_submarino)
                    elif _marca == 'ACOM':
                        _etiq = _etiq.replace('{logo_entrega}', _lg_americanas)
                    elif _marca == 'SHOP':
                        _etiq = _etiq.replace('{logo_entrega}', _lg_shoptime)
                    _etiq = _etiq.replace('{nro_nf}', _numNotaFiscal)
                    _etiq = _etiq.replace('{pedido}', _ped_nro)
                    _etiq = _etiq.replace('{plp}', _plp)
                    _etiq = _etiq.replace('{vol_vol}', _posicaoVolume)
                    _etiq = _etiq.replace('{vol_tot}', _qtdVolumes)
                    _etiq = _etiq.replace('{contrato}', _contrato)
                    _etiq = _etiq.replace('{enc1}', _codigoAwb[:8])
                    _etiq = _etiq.replace('{enc2}', _codigoAwb[8:])
                    _etiq = _etiq.replace('{encomenda}', _codigoAwb)
                    _etiq = _etiq.replace('{dest_nome}', _dest_nome)
                    _end = '%s, %s' % (_dest_logradouro, _dest_numero)
                    _etiq = _etiq.replace('{dest_end}', _end)
                    if _dest_complemento:
                        _bai = "%s - %s" % (_dest_complemento, _dest_bairro)
                        _etiq = _etiq.replace('{dest_bairro}', _bai)
                    else:
                        _etiq = _etiq.replace('{dest_bairro}', _dest_bairro)
                    _etiq = _etiq.replace('{dest_cidade}', _dest_cidade)
                    _etiq = _etiq.replace('{dest_cep_e}', _dest_cep_e)
                    _etiq = _etiq.replace('{dest_cep}', _dest_cep)
                    _etiq = _etiq.replace('{dest_uf}', _dest_uf)
                    _etiq = _etiq.replace('{rem_nome}', _rem_nome)
                    _end = '%s, %s' % (_rem_logradouro, _rem_numero)
                    _etiq = _etiq.replace('{rem_end}', _end)
                    if _rem_complemento:
                        _bai = "%s - %s" % (_rem_complemento, _rem_bairro)
                        _etiq = _etiq.replace('{rem_bairro}', _bai)
                    else:
                        _etiq = _etiq.replace('{rem_bairro}', _rem_bairro)
                    _etiq = _etiq.replace('{rem_cidade}', _rem_cidade)
                    _etiq = _etiq.replace('{rem_cep_e}', _rem_cep_e)
                    _etiq = _etiq.replace('{rem_cep}', _rem_cep)
                    _etiq = _etiq.replace('{rem_uf}', _rem_uf)
                    _etiq = _etiq.replace('{pve}',  i.get('numero_pedido'))
                    _etiq = _etiq.replace('{nfe}', _nfe)
                    _etiq = _etiq.replace('{skus1}', '')
                    _etiq = _etiq.replace('{skus2}', '')
                    _etiq = _etiq.replace('{skus3}', '')
                    t = 0
                    for x in _dest_cep:
                        t += int(x)
                    if t < 11:
                        dv = 10 - t
                    else:
                        dv = int(str(t)[:1]+'0') + 10 - t
                    _qr_code = _dest_cep + '00000' + _rem_cep + '00000' + str(dv) + '51' + _codigoAwb + _servicoAdicional
                    _qr_code += _cartaoPostagem + _plp_servico_cod + '00'
                    _qr_code += formataString(_dest_numero or '', 5, ' ', 'D', 'N', 'S')
                    _qr_code += formataString(_dest_complemento or '', 20, ' ', 'D', 'N', 'S')
                    _vl = str(_vlEntrega)
                    if '.' in _vl:
                        _vl = _vl.split('.')[0]
                    _qr_code += formataString(_vl, 5, '0', 'E')
                    _qr_code += formataString(_telefone_contato, 12, '0', 'E')
                    _qr_code += '-00.000000-00.000000|' + i.get('numero_pedido') + _nfe
                    _etiq = _etiq.replace('{qr_code}', _qr_code)

                    printer = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'estoque',
                                                                               ['estpar_impressora_termica', 'estpar_impressora_termica_servidor'],
                                                                               context=context)
                    if printer:
                        servidor = printer.get('estpar_impressora_termica_servidor')
                        printer = printer.get('estpar_impressora_termica')
                    else:
                        return {'status': False, 'mensagem': _('Cadastrar impressora no parametro do "Estoque" na aba '
                                               '"Automacao"'), 'dados': ''}
                    _arq = '/tmp/etiqueta_%s.txt' % _codigoAwb
                    f = open(_arq, 'wb')
                    f.write(_etiq)
                    f.close()
                    try:
                        if servidor:
                            subprocess.call(["lp", '-o raw', "-n1", '-h%s' % servidor, '-d%s' % printer, _arq])
                        else:
                            subprocess.call(["lp", '-o raw', "-n1", '-d%s' % printer, _arq])
                    except Exception as e:
                        return {'status': False, 'mensagem': _('Erro ao imprimir etiqueta de postagem %s') % e,
                                'dados': ''}
            vals = {'name': _codigoAwb}
            self.pool.get('rastreio.encomendas').write(cr, uid, _encomenda_id, vals, context=context)
            self.mkputl_faturar_no_mkp(cr, uid, [nf_id], context=context)
        # return {'status': True, 'mensagem': '', 'dados': _codigoAwb}
        return {'status': True, 'mensagem': '', 'dados': {'track_no': _codigoAwb, 'encomenda_id': _encomenda_id}}

    def mkputl_atualizar_anuncio_skyhub(self, cr, uid, id, context=None):
        """
        Inclui ou altera anuncio no Skyhub
        @param cr:
        @param uid:
        @param id: id do anuncio a ser alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """
        context = context and context.copy() or {}
        obj_SkyHub = self.pool.get('mkp.skyhub')
        obj_anuncios = self.pool.get('mkp.anuncios')
        brw_anuncio = obj_anuncios.browse(cr, uid, id, context=context)
        if not brw_anuncio: # anuncio nao existe mais. Retornar True para que a ação seja encerrada
            return {'status': True, 'mensagem': ''}
        try:
            _codigo = brw_anuncio.mkpanu_codigo_anuncio
        except: # Erro porque o id não existe
            return {'status': True, 'mensagem': ''}
        # _sku = brw_anuncio.mkpanu_produto_id.codigo
        # if not brw_anuncio.mkpanu_produto_id.grade_usar:
        #     _sku += '-UN'
        # dados_anuncio = obj_SkyHub.mkpSkyHub_get_produto(cr, uid, brw_anuncio.mkpanu_conta_id.id, _sku, context=context)
        # if dados_anuncio.get('status'):
        #     vals = {'mkpanu_qtd_vendido': dados_anuncio.get('Product').get('quantity_sold')}
        #     obj_anuncios.write(cr, uid, id, vals, context=context)
        res = obj_SkyHub.mkpSkyHub_set_anuncios(cr, uid, brw_anuncio, context=context)
        if res and res.get('status') == False:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao consegui atualizar o anuncio!\n%s' % res.get('mensagem')))
        vals = {}
        if not _codigo: # Qdo for inclusao, o id no erp é o mesmo codigo que foi usado no marketplace
            vals['mkpanu_codigo_anuncio'] =  id
        vals['mkpanu_necessita_atualizar'] = False
        obj_anuncios.write(cr, uid, id, vals, context=context)
        return res

