# -*- encoding: utf-8 -*-
import re
from urllib import urlencode
import json
import requests
from openerp import pooler
import logging
from openerp.tools.translate import _
import time
from SigUtils.geralogs import geraLogs
from openerp.tools import config

_logger = logging.getLogger('SkyHub')


class SkyHub(object):
    def logged_client(f):
        """Decorator for User Credentials checking."""
        def inner(self, *args, **kargs):
            if ((self.user_email is None) or (self.app_key is None)):
                raise Exception(_("Nao posso acessar o SkyHub. parametros do SkyHub nao informados!"))
            return f(self, *args, **kargs)
        return inner

    def __init__(self, cr=False, uid=False, conta_id=False, company_id=False):
        self.API_ROOT_URL = 'https://api.skyhub.com.br'
        self.cr = cr
        self.uid = uid
        self.user_email = ''
        self.app_key = ''
        self.account_manager_key = ''
        self.habilitar_log = False
        self.conta_id = ''
        self.comunicar = True
        if cr:
            _poolget = pooler.get_pool(cr.dbname).get
            mkp_ids = _poolget('mkp.marketplaces').search(cr, uid, [('mkpmkp_codigo', '=', 'SkyHub')])
            if mkp_ids:
                mkp_ids = mkp_ids[0]
                dados_marketplace = _poolget('mkp.marketplaces').read(cr, uid, mkp_ids, [])
                if dados_marketplace.get('mkpmkp_parar_comunicacao'):
                    self.comunicar = False
                if not conta_id:
                    conta_id = _poolget('mkp.contas').search(cr, uid, [('mkpcta_marketplace_id', '=', mkp_ids)])
                    if conta_id:
                        conta_id = conta_id[0]
                if conta_id:
                    self.conta_id = conta_id
                    dados_conta = _poolget('mkp.contas').read(cr, uid, conta_id, [])
                    if dados_conta.get('mkpcta_parar_comunicacao'):
                        self.comunicar = False
                    self.user_email = dados_conta.get('mkpcta_sh_user_email')
                    self.app_key = dados_conta.get('mkpcta_sh_api_key')
                    self.account_manager_key = dados_conta.get('mkpcta_sh_account_manager_key', '')
                    self.habilitar_log = dados_marketplace.get('mkpmkp_habilita_log')

    def _get_arq_log(self):
        _arq_log = 'mkp_skyhub_' + self.cr.dbname
        return _arq_log

    def _gerar_logs(self, mens):
        _diretorio = config.get('logfile')
        if _diretorio:
            _lista = _diretorio.split('/')
            _lista.pop(len(_lista) -1)
            _diretorio = '/'.join(_lista)
        else:
            _diretorio = '/var/log/openerp'
        _arq_log = self._get_arq_log()
        try:
            geraLogs(_arq_log, mens, 'S', _diretorio)
        except:
            try:
                geraLogs(_arq_log, mens.encode('ASCII', 'ignore'), 'S', _diretorio)
            except:
                geraLogs(_arq_log, 'Nao consegui registrar o log', 'S', _diretorio)
        return True


    def _comunicar_com_SkyHub(self, metodo, uri, params, body):
        headers = self._monta_header()
        if not self.comunicar:
            class resp(object):
                status_code = 999
                content = '{"message":"Comunicacao parada"}'
            return resp
        while True:
            if self.habilitar_log:
                self._gerar_logs('%s-> URL:%s Params: %s Body: %s Header: %s' % (metodo.upper(), str(uri),
                                                                                          str(params), str(body),
                                                                                          str(headers)))
            if metodo == 'get':
                response = requests.get(uri, params=urlencode(params), headers=headers)
            elif metodo == 'post':
                response = requests.post(uri, data=body, params=urlencode(params), headers=headers)
            elif metodo == 'put':
                response = requests.put(uri, data=body, params=urlencode(params), headers=headers)
            elif metodo == 'delete':
                response = requests.delete(uri, headers=headers)
            if self.habilitar_log:
                self._gerar_logs('Status:%s Resposta: %s' % (response.status_code, response.text))
            if response.status_code != 429:
                break
            time.sleep(1)
        return response

    # REQUEST METHODS
    @logged_client
    def get(self, path, params={}):
        uri = self.make_path(path)
        response = self._comunicar_com_SkyHub('get', uri, params, {})
        return response

    @logged_client
    def post(self, path, body={}, params={}):
        uri = self.make_path(path)
        if body:
            body = json.dumps(body)
        response = self._comunicar_com_SkyHub('post', uri, params, body)
        return response

    @logged_client
    def put(self, path, mkp_integrador, body=None):
        if path[-1:] == '/':
            uri = self.make_path(path) + str(mkp_integrador)
        else:
            uri = self.make_path(path) + '/' + str(mkp_integrador)
        _body = json.dumps(body)
        response = self._comunicar_com_SkyHub('put', uri, {}, _body)
        return response

    def delete(self, path, mkp_integrador):
        uri = self.make_path(path) + '/' + str(mkp_integrador)
        response = self._comunicar_com_SkyHub('delete', uri, {}, {})
        return response

    def make_path(self, path, params={}):
        # Making Path and add a leading / if not exist
        if not (re.search("^http", path)):
            if not (re.search("^\/", path)):
                path = "/" + path
            path = self.API_ROOT_URL + path
        if params:
            path = path + "?" + urlencode(params)
        return path

    def _monta_header(self):
        headers = {
            'X-User-Email': self.user_email,
            'x-Api-Key': self.app_key,
            'x-accountmanager-key': self.account_manager_key,
            'Accept': 'application/json;charset=UTF-8',
            'Content-Type': 'application/json'
        }
        return headers
