# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################
from openerp.osv import osv
from openerp.tools.translate import _
import json
import logging
from datetime import datetime, timedelta
from ..tray import Tray

ML_STATUS = {
    'A': 'active',
    'P': 'paused',
    'F': 'closed'
}

# Classe de alto nivel para comunicação com a Tray
class mkp_Tray(osv.Model):
    _name = 'mkp.tray'
    _auto = False
    _tem_erro = False
    _mensagem_erro = ''
    _logger = logging.getLogger(_name)

    # def mkpTray_autenticar(self, cr, uid, conta_id):
    #     tray = Tray(cr, uid, conta_id)
    #     _url = tray.auth_url()
    #     return _url

    def _mkpTray_get_id_status(self, cr, uid, conta_id, status):
        tray = Tray(cr, uid, conta_id)
        recurso = 'orders/statuses'
        page = 0
        while True:
            page += 1
            params = {'limit': '50', 'page': str(page)}
            response = tray.get(recurso, params)
            if response.status_code > 399:
                return {'status': False, 'mensagem': 'Nao consegui pegar status na Tray'}
            if not response.content:
                return {'status': False, 'mensagem': 'Nao consegui pegar status na Tray'}
            _dados_status = response.json()
            if not _dados_status.get('OrderStatuses'):
                break
            for _dst in _dados_status.get('OrderStatuses'):
                if _dst.get('OrderStatus').get('status') == status:
                    status_id = _dst.get('OrderStatus').get('id')
                    return {'status': True, 'mensagem': '', 'status_id': status_id}
        return {'status': False, 'mensagem': 'Status nao encontrado'}

    def _mkpTray_set_status_order(self, cr, uid, conta_id, status, pedido_id, context=None):
        context = context and context.copy() or {}
        tray = Tray(cr, uid, conta_id)
        if status == 'CANCELADO':
            recurso = 'orders/cancel/%s' % pedido_id
            response = tray.put(recurso,  {}, {})
            if response.status_code > 399:
                return {'status': False, 'mensagem': json.loads(response.content).get('message')}
            if not response.content:
                return {'status': False, 'mensagem': 'Resposta da Tray nao identificada'}
            return {'status': True, 'mensagem': ''}
        res = self._mkpTray_get_id_status(cr, uid, conta_id, status)
        if res.get('status') != True:
            return res
        status_id = res.get('status_id')
        _dados = {'Order': {'status_id': status_id}}
        recurso = 'orders/%s' % pedido_id
        response = tray.put(recurso,  _dados, {})
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('message')}
        if not response.content:
            return {'status': False, 'mensagem': 'Resposta da Tray nao identificada'}
        return {'status': True, 'mensagem': ''}

    def _mkpTray_set_codigo_rastreio(self, cr, uid, conta_id, order, codigo_rastreio, data, context=None):
        context = context and context.copy() or {}
        tray = Tray(cr, uid, conta_id)
        recurso = 'orders/%s' % order
        data = {
            'Order': {
                'sending_code': codigo_rastreio,
                'sending_date': data,
            }
        }
        response = tray.put(recurso, data, {})
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('message')}
        if not response.content:
            return {'status': False, 'mensagem': 'Resposta da Tray nao identificada'}
        return {'status': True, 'mensagem': ''}


    def _mkpTray_get_pedido_interno_tray(self, cr, uid, pedido_externo=None, pedido_id=None, context=None):
        context = context and context.copy() or {}
        _obj_pedidos = self.pool.get('estoquepedidos.pedido.capa')
        if pedido_externo:
            ped_ids = _obj_pedidos.search(cr, uid, [('pedcap_nro_pedido_externo', '=', pedido_externo)], context=context)
            if ped_ids:
                _ped_id = ped_ids[0]
            else:
                return {'status': False, 'mensagem': 'Nao encontrei o pedido externo informado: %s' % pedido_externo}
        else:
            _ped_id = pedido_id
        dados_pedido = _obj_pedidos.read(cr, uid, _ped_id, ['pedcap_endereco_eletronico'], context=context)
        if isinstance(dados_pedido, list):
            dados_pedido=dados_pedido[0]
        x = dados_pedido.get('pedcap_endereco_eletronico').split('/')
        if x and isinstance(x, list) and len(x) > 1:
            y = x[-1].split('=')
            if y and isinstance(y, list) and len(y) > 1:
                _order = y[1]
                return {'status': True, 'mensagem': '', 'order': _order}
        return {'status': False, 'mensagem': 'Nao consegui identificar o pedido:%s' % dados_pedido.get('pedcap_endereco_eletronico')}

    def mkpTray_cancelar_pedido(self, cr, uid, conta_id, pedido_externo, context=None):
        context = context and context.copy() or {}

        res = self._mkpTray_get_pedido_interno_tray(cr, uid, pedido_externo=pedido_externo, context=context)
        if res.get('status') == True:
            _order = res.get('order')
        else:
            return res
        res = self._mkpTray_set_status_order(cr, uid, conta_id, 'CANCELADO', _order, context=context)
        return res

    def mkpTray_get_atributos(self, cr, uid, context=None):
        self._logger.info('Buscando Atributos na Tray!')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        _url = '/products/properties'
        mkp_con = Tray(cr, uid, company_id=company_id)
        params = {'access_token': mkp_con.access_token}
        xml_str = mkp_con.get(_url, params)
        if xml_str.status_code > 399:
            self._logger.info('Nao consegui conectar na Tray')
            return False
        return xml_str.json()

    def mkpTray_get_categorias(self, cr, uid, context=None):
        self._logger.info('Buscando categorias na Tray!')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        _url = '/categories/tree'
        mkp_con = Tray(cr, uid, company_id=company_id)
        params = {'access_token': mkp_con.access_token}
        xml_str = mkp_con.get(_url, params)
        if xml_str.status_code > 399:
            self._logger.info('Nao consegui conectar na Tray')
            return False
        return xml_str.json()

    def mkpTray_set_categories(self, cr, uid, conta_id, codigo, categoria, categoria_pai, meta_keywords=None,
                               meta_descricao=None, property=None, context=None):
        context = context and context.copy() or {}
        if not categoria:
            return {'status': False, 'mensagem': 'Faltou a descricao da categoria'}
        mkp_con = Tray(cr, uid, conta_id)
        data = {
            'Category':
                    {
                        'name': categoria,
                        'description': categoria,
                        'slug': categoria.lower().replace(' ', '-'),
                        'has_acceptance_term': '0',
                    }
        }
        if meta_descricao or meta_keywords:
            meta = {'metatag':{}}
            if meta_keywords:
                meta['metatag']['keywords'] = meta_keywords
            if meta_descricao:
                meta['metatag']['description'] = meta_descricao
            data['Category'] = meta
        if categoria_pai:
            data['Category']['parent_id'] = categoria_pai
        # if property:
        #     str_pro = '['
        #     for x in property:
        #         str_pro += str(x) + ','
        #     str_pro = str_pro[:-1] + ']'
        #     data['Category']['property'] = str_pro
        recurso = 'categories'
        if codigo:
            recurso = 'categories/%s' % codigo
            dados_Tray = mkp_con.put(recurso,  data, params={})
        else:
            dados_Tray = mkp_con.post(recurso, data, params={})
        if dados_Tray.status_code in (200, 201):
            mkp_id = json.loads(dados_Tray.content).get('id')
            return {'status': True, 'mensagem': '', 'id': mkp_id}
        try:
            mens = str(json.loads(dados_Tray.content).get('causes'))
        except:
            mens = ''
        return {'status': False, 'mensagem': mens}

    def mkpTray_set_anuncios(self, cr, uid, brw_anuncio, context=None):
        context = context and context.copy() or {}
        obj_mkpromkp = self.pool.get('mkp.produto.marketplaces')
        obj_grade = self.pool.get('estoque.produto.grade')
        obj_anuncios_var = self.pool.get('mkp.anuncios.var')
        mkp_con = Tray(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        params = {}
        vals = {}
        _picture_produts = []
        _pictures_grades = {}
        _img_comum = []

        args = [('mkpromkp_produto_id', '=', brw_anuncio.mkpanu_produto_id.id), ('mkpromkp_marketplace_id', '=', brw_anuncio.mkpanu_marketplace_id.id)]
        mkpromkp_ids = obj_mkpromkp.search(cr, uid, args, context=context)
        if not mkpromkp_ids:
            return {'status': False, 'mensagem': 'Produto %s nao cadastrado no marketplace!' %  brw_anuncio.mkpanu_produto_id.codigo}

        brw_mkpromkp = obj_mkpromkp.browse(cr, uid, mkpromkp_ids[0], context=context) # Embora se mkpromkp_ids, será sempre lista de 1
        for brw_imagens in brw_mkpromkp.mkpromkp_imagens_ids:
            _picture_produts.append(brw_imagens.mkprmkimg_endereco_mkp)
            if brw_anuncio.mkpanu_produto_id.grade_usar:
                _gr = brw_imagens.mkprmkimg_imagem_produto_id.estimg_grade_id.grade
                if not _gr: # imagem é é do produto e não da grade
                    _img_comum.append(brw_imagens.mkprmkimg_endereco_mkp)
                    continue
                _img_gr = _pictures_grades.get(_gr, [])
                if not brw_imagens.mkprmkimg_endereco_mkp: # Para Tray tem que ter endereço da imagem
                    continue
                _img_gr.append(brw_imagens.mkprmkimg_endereco_mkp)
                _pictures_grades[_gr]= _img_gr
        i = 0
        for _img in _picture_produts:
            i += 1
            if i > 6: # só 6 imagens
                break
            name = 'picture_source_%s' % i
            vals[name] = _img
        for _img_idx in _pictures_grades.keys():
            _pictures_grades[_img_idx] = _pictures_grades.get(_img_idx) + _img_comum
        vals['name'] = brw_anuncio.mkpanu_title
        vals['description'] = brw_mkpromkp.mkpromkp_anuncio_preview_html
        if brw_anuncio.mkpanu_estado == 'A':
            vals['available'] = '1'
        else:
            vals['available'] = '0'
        _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
        _peso = 0.3
        _altura = 6
        _largura = 11
        _comprimento = 16
        _marca = _modelo = False

        _atributos = []
        for brw_atributo in brw_anuncio.mkpanu_produto_id.estpro_atributo_ids:
            if brw_atributo.estatp_atributo_id.name == 'Marca':
                _marca = brw_atributo.estatp_atributo_valor
                continue
            if brw_atributo.estatp_atributo_id.name == 'Modelo':
                _modelo = brw_atributo.estatp_atributo_valor
                continue
            for brw_mkp_atributo in brw_atributo.estatp_atributo_id.estatr_mkp_atributos_ids:
                if brw_mkp_atributo.mkpatr_marketplace_id.id == brw_anuncio.mkpanu_marketplace_id.id:
                    if not brw_mkp_atributo.mkpatr_id_no_mkp:
                        break
                    _valor_att = brw_atributo.estatp_atributo_valor
                    if brw_atributo.estatp_tem_unidade:
                        _valor_att = '%s%s' % (brw_atributo.estatp_atributo_valor, brw_atributo.estatp_atributo_unidade_id.name)
                    _dic = {'property_id': brw_mkp_atributo.mkpatr_id_no_mkp, 'value': _valor_att}
                    _atributos.append(_dic)
                    break
        for brw_embalagem in brw_anuncio.mkpanu_produto_id.produto_embalagem_ids: # pegar APENAS A PRIMEIRA
            _peso = brw_embalagem.peso_bruto or 0.3
            _altura = brw_embalagem.altura or 6
            _largura = brw_embalagem.largura or 11
            _comprimento = brw_embalagem.profundidade or 16
            break
        vals['price'] = _valor
        vals['weight'] = _peso * 1000
        vals['height'] = _altura
        vals['width'] = _largura
        vals['length'] = _comprimento
        if _marca:
            vals['brand'] = _marca
        if _modelo:
            vals['model'] = _modelo
        vals['virtual_product'] = '0'
        vals['ncm'] = brw_anuncio.mkpanu_produto_id.ncm_id.codigo
        vals['category_id'] = brw_mkpromkp.mkpromkp_categoria_id.mkpcat_codigo
        vals['reference'] = '%s-UN' % brw_anuncio.mkpanu_produto_id.codigo
        _sku = vals.get('reference')
        vals['stock'] = brw_anuncio.mkpanu_saldo
        for brw_cod_barra in brw_anuncio.mkpanu_produto_id.automacao_ids:
            if brw_cod_barra.autcba_tipo_barra == '13':
                 _barra = brw_cod_barra.codigo_barra[1:]
                 vals['ean'] = _barra
                 break
        res = {'status': True, 'mensagem': ''}
        vals = {'Product': vals}
        _url = 'products'
        if not brw_anuncio.mkpanu_codigo_anuncio:
            response = mkp_con.post(_url, vals, params)
        else:
            _url = 'products/%s' % brw_anuncio.mkpanu_codigo_anuncio
            response = mkp_con.put(_url,  vals, params)
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('error')}
        else:
            mkp_id = json.loads(response.content).get('id')
            res['id'] = mkp_id
        if brw_anuncio.mkpanu_produto_id.grade_usar:
            vals_grade = {}
            vals['reference'] = brw_anuncio.mkpanu_produto_id.codigo
            _sku = vals.get('reference')
            _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
            for _brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
                context['company_id'] = brw_anuncio.company_id.id
                _x = obj_grade.estpgr_get_saldo(cr, uid, [_brw_grade.id], 'D', _local, context=context)
                vals_grade['reference'] = '%s-%s' % (brw_anuncio.mkpanu_produto_id.codigo, _brw_grade.grade)
                vals_grade['stock'] = int(_x.get(_brw_grade.id) / brw_anuncio.mkpanu_quantidade)
                vals_grade['price'] = _valor
                vals_grade['Sku'] = []
                vals_grade['Sku'].append({})
                vals_grade['Sku'][0]['type'] = brw_anuncio.mkpanu_produto_id.estpro_grade_descricao
                vals_grade['Sku'][0]['value'] = _brw_grade.descricao
                for brw_cod_barra in brw_anuncio.mkpanu_produto_id.automacao_ids:
                    if _brw_grade.id == brw_cod_barra.grade_id.id:
                        if brw_cod_barra.autcba_tipo_barra == '13':
                            _barra = brw_cod_barra.codigo_barra[1:]
                            vals_grade['ean'] = _barra
                            break
                i = 0
                if not _pictures_grades:
                    return {'status': False, 'mensagem': 'O produto %s nao tem fotos das grades'  % brw_anuncio.mkpanu_produto_id.codigo}
                if _pictures_grades.get(_brw_grade.grade):
                    for _img in _pictures_grades.get(_brw_grade.grade):
                        i + 1
                        if i > 6:  # só 6 imagens
                            break
                        name = 'picture_source_%s' % i
                        vals_grade[name] = _img

                vals_grade['images'] = _pictures_grades.get(_brw_grade.grade) or []
                vals_grade['product_id'] = mkp_id
                args = [('mkpanuv_anuncio_id', '=', brw_anuncio.id),('mkpanuv_grade_id', '=', _brw_grade.id)]
                var_ids = obj_anuncios_var.search(cr, uid, args, context=context)
                id_mkp_var = False
                _url = 'products/variants'
                if var_ids:
                    dados_var = obj_anuncios_var.read(cr, uid, var_ids[0], [], context=context)
                    id_mkp_var = dados_var.get('mkpanuv_id_no_mkp')
                    _url = 'products/variants/%s' % id_mkp_var
                    response_var = mkp_con.put(_url, vals_grade, params)
                else:
                    response_var = mkp_con.post(_url, vals_grade, params)
                if response_var.status_code < 400:
                    id_mkp_var = json.loads(response_var.content).get('id')
                vals_var = {
                    'mkpanuv_anuncio_id': brw_anuncio.id,
                    'mkpanuv_grade_id': _brw_grade.id,
                    'mkpanuv_id_no_mkp': id_mkp_var,
                    'mkpanuv_valor': _valor,
                }
                if var_ids:
                    obj_anuncios_var.write(cr, uid, var_ids, vals_var, context=context)
                else:
                    obj_anuncios_var.create(cr, uid, vals_var, context=context)
        if _atributos:
            _url = 'products/%s/properties' % mkp_id
            response = mkp_con.post(_url, _atributos, params)
        return res

    def mkpTray_set_preco(self, cr, uid, brw_anuncio, context=None):
        context = context and context.copy() or {}
        _spg = self.pool.get
        mkp_con = Tray(cr, uid, brw_anuncio.mkpanu_conta_id.id)
        params = {}
        vals = {}
        _valor = brw_anuncio.mkpanu_valor_promocao or brw_anuncio.mkpanu_valor or brw_anuncio.mkpanu_valor_sugerido
        vals['price'] = _valor
        res = {'status': True, 'mensagem': ''}
        vals = {'Product': vals}
        _url = 'products'
        if not brw_anuncio.mkpanu_codigo_anuncio:
            return {'status': False, 'mensagem': 'Produto nao cadstrado no MKP!'}
        else:
            _url = 'products/%s' % brw_anuncio.mkpanu_codigo_anuncio
            response = mkp_con.put(_url,  vals, params)
        if response.status_code > 399:
            return {'status': False, 'mensagem': json.loads(response.content).get('error')}
        else:
            mkp_id = json.loads(response.content).get('id')
            res['id'] = mkp_id
        if brw_anuncio.mkpanu_produto_id.grade_usar:
            vals_grade = {}
            vals['reference'] = brw_anuncio.mkpanu_produto_id.codigo
            _sku = vals.get('reference')
            _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
            for _brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
                vals_grade['price'] = _valor
                args = [('mkpanuv_anuncio_id', '=', brw_anuncio.id),('mkpanuv_grade_id', '=', _brw_grade.id)]
                var_ids = _spg('mkp.anuncios.var').search(cr, uid, args, context=context)
                id_mkp_var = False
                _url = 'products/variants'
                if not var_ids:
                    continue
                dados_var = _spg('mkp.anuncios.var').read(cr, uid, var_ids[0], [], context=context)
                id_mkp_var = dados_var.get('mkpanuv_id_no_mkp')
                _url = 'products/variants/%s' % id_mkp_var
                response_var = mkp_con.put(_url, vals_grade, params)
                if response_var.status_code < 400:
                    id_mkp_var = json.loads(response_var.content).get('id')
                vals_var = {
                    'mkpanuv_anuncio_id': brw_anuncio.id,
                    'mkpanuv_grade_id': _brw_grade.id,
                    'mkpanuv_id_no_mkp': id_mkp_var,
                    'mkpanuv_valor': _valor,
                }
                _spg('mkp.anuncios.var').write(cr, uid, var_ids, vals_var, context=context)
        return res

    def mkpTray_atualizar_saldo(self, cr, uid, anun_id, conta_id, context=None):
        """
        Inclui ou altera saldo dos anuncio no site da Tray
        @param cr:
        @param uid:
        @param prod_id: ids do produto a ser alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """

        context = context and context.copy() or {}
        res = {'status': True, 'mensagem': ''}
        obj_anuncio = self.pool.get('mkp.anuncios')
        brw_anuncio = obj_anuncio.browse(cr, uid, anun_id, context=context)
        obj_grade = self.pool.get('estoque.produto.grade')
        mkp_con = Tray(cr, uid, conta_id)
        _local = brw_anuncio.mkpanu_conta_id.mkpcta_local_id.id
        # Verificar situacao do anuncio no Marketplace
        if not brw_anuncio.mkpanu_codigo_anuncio:
            res = {'status': True, 'mensagem': ''}
            return res
        _product = {}
        if brw_anuncio.mkpanu_produto_id.grade_usar:
            _sku = brw_anuncio.mkpanu_produto_id.codigo
            grades = []
            for brw_grade in brw_anuncio.mkpanu_produto_id.produto_grade_ids:
                vals_grade = {}
                context['company_id'] = brw_anuncio.company_id.id
                _x = obj_grade.estpgr_get_saldo(cr, uid, [brw_grade.id], 'D', _local, context=context)
                vals_grade['stock'] = int(_x.get(brw_grade.id) / brw_anuncio.mkpanu_quantidade) if _x.get(brw_grade.id) > 0 else 0
                vals_grade['product_id'] = brw_anuncio.mkpanu_codigo_anuncio
                for brw_var in brw_anuncio.mkpanu_variacoes_ids:
                    if brw_var.mkpanuv_grade_id.id == brw_grade.id:
                        _mkp_id = brw_var.mkpanuv_id_no_mkp
                        _url = 'products/variants/%s' % _mkp_id
                        response_var = mkp_con.put(_url, vals_grade, params={})
                        vals_grade = {}
                        break
                if vals_grade: # tem grade que não existe no mkp
                    resx = self.mkpTray_set_anuncios(cr, uid, brw_anuncio, context=context)
        else:
            _mkp_id =  brw_anuncio.mkpanu_codigo_anuncio
            _product['stock'] = brw_anuncio.mkpanu_saldo
            vals = {'Product': _product}
            _url = 'products/%s' % _mkp_id
            response = mkp_con.put(_url, vals, params={})
            if response.status_code == 400:
                return {'status': False, 'mensagem': 'O codigo do anuncio na Tray e invalido!'}
            if response.status_code > 399:
                return {'status': False, 'mensagem': str(json.loads(response.content).get('causes'))}
        return res

    def mkpTray_get_order(self, cr, uid, pedido, conta_id, context=None):
        """
        Método destinado a buscar um pedido na Trau.
        @param cr:
        @param uid:
        @param pedido: Número do pedido no M.L.
        @param context:
        @return: Sempre True
        """
        context = context and context.copy() or {}
        mkp_con = Tray(cr, uid, conta_id)
        headers = {}
        params = {'access_token': mkp_con.access_token}
        _url = 'orders/%s/complete' % pedido
        xml_str = mkp_con.get(_url, params)
        if xml_str.status_code > 399:
            self._logger.info('Nao consegui conectar na Tray')
            return False
        # return xml_str.json()
        dados = json.loads(xml_str.text)
        return dados

    def mkpTray_pesquisar_orders(self, cr, uid, conta_id, context=None):
        context = context and context.copy() or {}
        mkp_con = Tray(cr, uid, conta_id)
        obj_pedido = self.pool.get('estoquepedidos.pedido.capa')
        obj_notificacoes = self.pool.get('mkp.notificacoes')
        _hoje = self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(), formato='%Y-%m-%d')
        _dt_inicio = datetime.now() - timedelta(days=20)
        _dt_inicio = self.pool.get('adm.utils').admutl_datetime_to_timezone(cr, uid, _dt_inicio, formato='%Y-%m-%d')
        page = 0
        while True:
            page += 1
            # params={'limit': '50', 'page': str(page), 'modified': '%s,%s' % (_dt_inicio, _hoje)}
            params={'limit': '50', 'page': str(page), 'modified': '%s' % (_hoje)}
            xml_str = mkp_con.get('orders', params)
            if xml_str.status_code > 399:
                self._logger.info('Nao consegui conectar na Tray')
                return False
            dados_mkp = json.loads(xml_str.text)
            if not dados_mkp.get('Orders'):
                break
            for order in dados_mkp.get('Orders'):
                _tr_venda_id = _tr_tray_venda_id = order.get('Order').get('id')
                _point_sale = order.get('Order').get('point_sale')
                if order.get('Order').get('external_code'):
                    _tr_venda_id = '%s-%s' % (_point_sale, order.get('Order').get('external_code'))
                args = [
                    ('pedcap_nro_pedido_externo', '=', _tr_venda_id),
                    ('tipo', '=', 'V'),
                    ('pedcap_mkp_status', '=', order.get('Order').get('status'))
                ]
                _pedido_ids = obj_pedido.search(cr, uid, args, context=context)
                if _pedido_ids:
                    continue
                _resourse = '/orders/%s' % _tr_tray_venda_id
                args = [('mkpnot_conta_id','=', conta_id), ('mkpnot_resource','=',_resourse), ('state','in',('R', 'E'))]
                not_ids = obj_notificacoes.search(cr, uid, args, context=context)
                if not not_ids:
                    vals = {
                        'mkpnot_conta_id': conta_id,
                        'mkpnot_resource': _resourse,
                        'mkpnot_topic': 'orders',
                        'mkpnot_attempts': 1,
                        'state': 'R',
                    }
                    res = obj_notificacoes.create(cr, uid, vals, context=context)
        return True
    
    def mkpTray_set_order_faturado(self, cr, uid, _order, _conta_id, chave, nf_ids, context=None):
        context = context and context.copy() or {}
        res = self._mkpTray_get_pedido_interno_tray(cr, uid, pedido_externo=_order, context=context)
        if res.get('status') != True:
            return res
        _order = res.get('order')
        res = self._mkpTray_set_status_order(cr, uid, _conta_id, 'AGUARDANDO ENVIO', _order, context=context)
        if res.get('status') != True:
            return res
        for dados_nf in self.pool.get('fiscal.nf.capa').read(cr, uid, nf_ids, [], context=context):
            recurso = 'orders/%s/invoices' % _order
            data = {
                'issue_date': dados_nf.get('data_emissao')[:10],
                'number': dados_nf.get('numero_nf'),
                'serie': dados_nf.get('serie_nf'),
                'value': dados_nf.get('vr_total_nf'),
                'key': dados_nf.get('nfe_chave_acesso'),
                'xml_danfe': dados_nf.get('nfe_xml_proc'),
            }
            mkp_con = Tray(cr, uid, _conta_id)
            response = mkp_con.post(recurso, data, {})
            if response.status_code == 422:
                data.pop('status')
                response = mkp_con.post(recurso, data, {})
            if response.status_code > 399:
                return {'status': False, 'mensagem': json.loads(response.content)}
        return {'status': True, 'mensagem': ''}


    def mkpTray_get_anuncio(self, cr, uid, conta_id, codigo_anuncio, context=None):
        context = context and context.copy() or {}
        mkp_con = Tray(cr, uid, conta_id)
        params = {'access_token': mkp_con.access_token}
        if codigo_anuncio:
            _url = 'products/' +str(codigo_anuncio)
            response = mkp_con.get(_url, params)
            if response.status_code > 399:
                return {'status': False, 'mensagem': json.loads(response.content).get('message')}
            dados_anuncio_Tray = json.loads(response.content)
            return dados_anuncio_Tray
        return False


    # def mkpTray_atualizar_mensagem(self, cr, uid, brw_mensagem, context=None):
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, brw_mensagem.mkpmen_conta_id.id)
    #     params = {'access_token': mkp_con.access_token}
    #     _url = 'messages'
    #     _ml_meu_user_id = brw_mensagem.mkpmen_conta_id.mkpcta_ml_user_id
    #     _ml_cli_user_id = brw_mensagem.mkpmen_pedido_id.pessoa_id.admpes_id_externo
    #     if not _ml_cli_user_id:  # Não é cliente do M.L. não precisa fazer nada
    #         return {'status': True, 'mensagem': ''}
    #     _order_id = brw_mensagem.mkpmen_pedido_id.pedcap_nro_pedido_externo
    #     _mens = brw_mensagem.name
    #     vals = {
    #         'from': {'user_id': _ml_meu_user_id},
    #         'to': [{
    #             'user_id': _ml_cli_user_id,
    #             'resource': 'orders',
    #             'resource_id': _order_id,
    #             'site_id': 'MLB',
    #         }],
    #         'text': {'plain': _mens}
    #     }
    #     response = mkp_con.post(_url, vals, params)
    #     if response.status_code > 399:
    #         return {'status': False, 'mensagem': json.loads(response.content).get('message')}
    #     res = {'status': True, 'mensagem': ''}
    #     message_id = json.loads(response.content)[0].get('message_id')
    #     res['message_id'] = message_id
    #     return res

    # def mkpTray_get_shipment(self, cr, uid, _ship_id, _conta_id, context=None):
    #     mkp_con = Tray(cr, uid, _conta_id)
    #     _url = '/shipments/%s' % _ship_id
    #     params = {'access_token': mkp_con.access_token}
    #     headers = {}
    #     response = mkp_con.get(_url, params, headers=headers)
    #     if response.status_code > 399:
    #         return False
    #     dados = json.loads(response.content)
    #     return dados

    # def mkpTray_get_etiqueta(self, cr, uid, _ship_ids, conta_id, context=None):
    #     if not _ship_ids:
    #         return ''
    # 
    #     if not isinstance(_ship_ids, list):
    #         _ship_ids = [_ship_ids]
    #     'https://api.mercadolibre.com/shipment_labels?shipment_ids=27651568876&response_type=zpl2&access_token=APP_USR-5212872253144318-072009-4c11a5aceb86ccd7b9a083f297e596ee-77813143'
    #     _url = 'shipment_labels?shipment_ids=%s&response_type=zpl2' % ','.join(
    #                                                               map(lambda x: '%s' % x,_ship_ids))
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token}
    #     headers = {}
    #     try:
    #         response = mkp_con.get(_url, params, headers=headers)
    #     except Exception as e:
    #         raise osv.except_osv(_('Atencao!'),
    #                              _('Erro de comunicacao com o Mercado Livre:\n\n%s') % e)
    # 
    #     data = datetime.now()
    #     path_zip = '/tmp/etiqueta-%s.zip' % data
    #     f = open(path_zip, 'w')
    #     f.write(response.content)
    #     f.close()
    #     path = '/tmp/etiqueta-%s' % data
    #     try:
    #         fantasy_zip = zipfile.ZipFile(path_zip)
    #         fantasy_zip.extractall(path)
    #         fantasy_zip.close()
    #     except Exception as e:
    #         try:
    #             f = open(path_zip, 'r')
    #             _etiq = json.loads(f.read())
    #             f.close()
    #             if _etiq.get('error') == 'labels_generation_locked':
    #                 raise Exception('labels_generation_locked')
    #             elif _etiq.get('error') == 'not_found':
    #                 raise Exception(_('A etiqueta ainda nao foi liberada pelo M.L.-not_found!'))
    #             else:
    #                 raise Exception('Erro na leitura da etiqueta\n%s' % _etiq.get('message'))
    #         except Exception as e:
    #             if e.message == 'labels_generation_locked':
    #                 raise osv.except_osv(_('Atencao!'),
    #                      _('A etiqueta ainda nao foi liberada pelo M.L.!'))
    #             raise osv.except_osv(_('Atencao!'),
    #                              _('Nao consegui extrair a etiqueta:\n\n%s') % e)
    #     f_in = open('%s/Etiqueta de envio.txt' % path, 'r')
    #     fi = f_in.readlines()
    #     f_in.close()
    #     out_path = '/tmp/etiqueta-%s.txt' % str(data).replace(' ', '_')
    #     f = open(out_path, 'w')
    #     f.writelines(fi)
    #     f.close()
    #     return out_path
    #     # return '%s/Etiqueta de envio.txt' % path

    # def mkpTray_get_custo_frete(self, cr, uid, anuncio, conta_id, context=None):
    #     """
    #     Método destinado a buscar o custo do frete de um anuncio
    #     @param cr:
    #     @param uid:
    #     @param anuncio: codigo do anuncio no mercado livre MLB????????????
    #     @param context:
    #     @return: Sempre True
    #     """
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token, 'seller': mkp_con.seller}
    #     _url = 'items/%s/shipping_options/free' % anuncio
    #     xml_str = mkp_con.get(_url, {})
    #     if xml_str.status_code > 399:
    #         return 0.00
    #     dados = json.loads(xml_str.text)
    #     idx = dados.get('coverage').keys()
    #     return dados.get('coverage').get(idx[0]).get('list_cost')

    # def mkpTray_get_shipment_orders(self, cr, uid, _ship_id, conta_id, context=None):
    #     """
    #     get ordens de uma etiqueta
    #     :param cr:
    #     :param uid:
    #     :param _ship_id: Codigo da encomenda
    #     :param context:
    #     :return: lista com numero das ordens(pedidos)
    #     """
    #     mkp_con = Tray(cr, uid, conta_id)
    #     _url = '/shipments/%s/items' % _ship_id
    #     params = {'access_token': mkp_con.access_token}
    #     headers = {}
    #     response = mkp_con.get(_url, params, headers=headers)
    #     dados = json.loads(response.content)
    #     _orders = []
    #     for x in dados:
    #         _orders.append(x.get('order_id'))
    #     return _orders



    # def mkpTray_get_payments(self, cr, uid, payment, conta_id, context=None):
    #     """
    #     Método destinado a buscar os recebimentos no mercado pago
    #     @param cr:
    #     @param uid:
    #     @param payment: Codigo do pagamento no mercado pago
    #     @param context:
    #     @return: Json com resultado da pesquisa de pagamentos no mercado livre
    #     """
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token}
    #     _url = 'collections/%s' % payment
    #     xml_str = mkp_con.get(_url, params)
    #     dados_collections = json.loads(xml_str.text)
    #     res = {
    #             'pedido': dados_collections.get('order_id'),
    #             'Venda': dados_collections.get('transaction_amount'),
    #             'Tarifa_ml': dados_collections.get('marketplace_fee'),
    #             'Tarifa_mp': dados_collections.get('mercadopago_fee'),
    #             'Frete': dados_collections.get('shipping_cost'),
    #         }
    #     return res

    # def mkpTray_get_pergunta(self, cr, uid, pergunta, conta_id, context=None):
    #     """
    #     Método destinado a buscar um pedido no M.L.
    #     @param cr:
    #     @param uid:
    #     @param pedido: Número do pedido no M.L.
    #     @param context:
    #     @return: Sempre True
    #     """
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token}
    #     _url = 'questions/%s' % pergunta
    #     xml_str = mkp_con.get(_url, params)
    #     dados = json.loads(xml_str.text)
    #     return dados

    # def mkpTray_get_mensagem(self, cr, uid, mensagem, conta_id, context=None):
    #     """
    #     Método destinado a buscar uma mensagem de pedido no M.L.
    #     @param cr:
    #     @param uid:
    #     @param pedido: Número da mensagem no M.L.
    #     @param context:
    #     @return: Sempre True
    #     """
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token}
    #     _url = 'messages/%s' % mensagem
    #     xml_str = mkp_con.get(_url, params)
    #     dados = json.loads(xml_str.text)
    #     return dados

    # def mkpTray_get_usuario(self, cr, uid, ml_user_id, conta_id, context=None):
    #     """
    #     Método destinado a buscar um usuario no M.L.
    #     @param cr:
    #     @param uid:
    #     @param ml_user_id: Número do usuario no M.L.
    #     @param context:
    #     @return: Sempre True
    #     """
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token}
    #     _url = 'users/%s' % ml_user_id
    #     xml_str = mkp_con.get(_url, params)
    #     dados = json.loads(xml_str.text)
    #     return dados

    # def MkpTray_pesquisar_anuncio(self, cr, uid, conta_id, context=None):
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token, 'seller': mkp_con.seller}
    #     _url = 'orders/search/recent'
    #     self._logger.info('Pesquisando pedidos')
    #     _offset = 0
    #     _laco = 1
    #     res = []
    #     while True:
    #         try:
    #             params['offset'] = _offset
    #             xml_str = mkp_con.get(_url, params)
    #             dados = json.loads(xml_str.text)
    #             _total_pedidos = dados.get('paging').get('total')
    #             _qtd_lacos = (_total_pedidos / 50) + 1
    #             if _total_pedidos in (50, 100, 150, 200, 250, 300):
    #                 _qtd_lacos -= 1
    #         except:
    #             self._logger.info('Nao consegui obter a pesquisa de pedidos!')
    #             return {}
    #         if dados.get('status', False):
    #             self._logger.info(
    #                 'Erro ML: %s-%s' % (dados.get('status', 'NI'), dados.get('message', 'Erro nao identificado!')))
    #             return True
    # 
    #         for _venda in dados.get('results'):
    #             res.append(_venda)
    #         _offset += 50
    #         _laco += 1
    #         if _laco > _qtd_lacos:
    #             break
    #     return res

    # def mkpTray_set_resposta(self, cr, uid, pergunta_ml_id, resposta, conta_id, context=None):
    #     """
    #     Método destinado a responder uma pergunta no M.L.
    #     @param cr:
    #     @param uid:
    #     @param pedido: Número do pedido no M.L.
    #     @param context:
    #     @return: Sempre True
    #     """
    #     mkp_con = Tray(cr, uid, conta_id)
    #     params = {'access_token': mkp_con.access_token}
    #     vals = { 'question_id': pergunta_ml_id, 'text': resposta}
    #     _url = 'answers'
    #     response = mkp_con.post(_url, vals, params)
    #     res = json.loads(response.content)
    #     if response.status_code > 399:
    #         return False
    #     else:
    #         return True

    # def estimg_imagem_cadastrar_ml(self, cr, uid, imagem_id, force = False, context={}):
    #     """
    #     Função para cadastrar imagens no ML se não existir ou para retornar o id da imagem
    #     @param cr:
    #     @param uid:
    #     @param imagem_id: integer. id da imagen do produto ou da grade no sigerp
    #     @param context:
    #     @return: char Codigo da imagem no ML
    #     """
    #     def _set_imagem(_imagem, _img_id):
    #         _arq = '/tmp/ml_%s.jpg' % time.time()
    #         f=open(_arq, 'wb')
    #         f.write(base64.b64decode(_imagem))
    #         f.close()
    #         params = {'access_token': mkp_con.access_token}
    #         response = mkp_con.upload('pictures', {'file': open(_arq, 'rb')}, params)
    #         if response.status_code > 399:
    #             return False
    #         res = json.loads(response.content)
    #         _img_ml_id = res.get('id')
    #         _img_size = res.get('max_size')
    #         _img_url = ''
    #         for x in res.get('variations'):
    #             if x.get('size') == _img_size:
    #                 _img_url = x.get('url')
    #                 break
    #         context['ml_atualizado'] = True
    #         obj_img.write(cr, uid, _img_id, {'mkprmkimg_codigo_no_mkp':_img_ml_id, 'mkprmkimg_endereco_mkp': _img_url},
    #                               context=context)
    #         return _img_ml_id
    # 
    #     def _get_imagem_no_ml(id_imagem_no_ml):
    #         response = mkp_con.get('pictures/%s' % id_imagem_no_ml)
    #         if response.status_code > 399: #Not found
    #             return False
    #         else:
    #             return True
    # 
    #     def _get_imagem():
    #         args = [('mkprmkimg_imagem_produto_id', '=',imagem_id)]
    #         mkp_produto_marketplace_imagens_ids = self.pool.get('mkp.produto.marketplace.imagens').search(cr, uid, args, context=context)
    #         res = False
    #         for dados_imagem in obj_img.read(cr, uid, mkp_produto_marketplace_imagens_ids, ['mkprmkimg_codigo_no_mkp', 'mkprmkimg_foto', 'mkprmkimg_mkp_codigo'], context=context):
    #             if dados_imagem.get('mkprmkimg_mkp_codigo') == 'Tray':
    #                 if not force:
    #                     id_imagem_no_ml = dados_imagem.get('mkprmkimg_codigo_no_mkp') or False
    #                     if id_imagem_no_ml and _get_imagem_no_ml(id_imagem_no_ml):
    #                         return id_imagem_no_ml
    #                 res = _set_imagem(dados_imagem.get('mkprmkimg_foto'), dados_imagem.get('id'))
    #         return res
    # 
    #     context = context and context.copy() or {}
    #     mkp_con = Tray(cr, uid)
    #     obj_img = self.pool.get('mkp.produto.marketplace.imagens')
    #     imagem_no_ml = _get_imagem()
    #     return imagem_no_ml


# class mkpTray_pesquisar_anuncio(object):
#     def __init__(self, cr, uid, conta_id, dt_inicio, dt_hoje):
#         self.iniciou = False
#         self.terminou = False
#         self.anuncios = []
#         self.anuncios_com_problemas = []
#         self.dict_resposta = {}
#         self.mkp_con = Tray(cr, uid, conta_id)
#         self.dt_inicio = dt_inicio
#         self.dt_hoje = dt_hoje
#         self._codigos = ''
#         return
# 
#     def pesquisar_no_ml(self):
#         str_codigos = self._codigos
#         _url = '/items?ids=%s' % str_codigos
#         xml_str = self.mkp_con.get(_url)
#         if xml_str.status_code > 399:
#             anuncios = str_codigos.split(',')
#             for a in anuncios:
#                 self.anuncios_com_problemas.append(a)
#             return True
# 
#         res_url = {}
#         for res in json.loads(xml_str.text):
#             res = res.get('body')
#             res_url = {}
#             res_url['descricao_anuncio'] = res.get('title', 'Nao consegui identificar')
#             _state = res.get('status')
#             res_url['qtd_vendido'] = res.get('sold_quantity', 0)
#             res_url['preco'] = res.get('price', 0)
#             _mp = res.get('accepts_mercadopago', False)
#             c_s_juros = res.get('listing_type_id')
#             _parcela = 'parcelas '
#             res_url['anuncio_tipo'] = res.get('listing_type_id')
#             if c_s_juros == 'gold_pro':
#                 _parcela += 'Sem Juros '
#             else:
#                 _parcela += 'Com Juros '
#             if _mp:
#                 _parcela += ' Com MercadoPago'
#             res_url['parcelas'] = _parcela
#             res_url['estoque'] = res.get('available_quantity', 0)
#             res_url['frete_gratis'] = False
#             if res.get('shipping', False):
#                 res_url['frete_gratis'] = res.get('shipping').get('free_shipping', False)
#             res_url['local_vendedor'] = ''
#             if res.get('seller_address', False):
#                 if res.get('seller_address').get('city', False) and res.get('seller_address').get('state', False):
#                     res_url['local_vendedor'] = '%s (%s)' % (
#                         res.get('seller_address').get('city').get('name', ''),
#                         res.get('seller_address').get('state').get('name', ''))
#             if _state == 'active':
#                 res_url['state'] = 'A'
#             elif _state == 'paused':
#                 res_url['state'] = 'P'
#             elif _state == 'closed':
#                 res_url['state'] = 'F'
#             else:
#                 res_url['state'] = 'C'
#             res_url['mercado_envio'] = False
#             res_url['titulo'] = res.get('title')
#             try:
#                 if res.get('shipping').get('mode') == 'me2':
#                     res_url['mercado_envio'] = True
#             except:
#                 pass
#             _id_anuncio = res.get('id', '').strip()
#             try:
#                 res_url['foto'] = urllib2.urlopen(res.get('thumbnail')).read().encode('base64')
#             except:
#                 # self._logger.info('Nao consegui tratar a foto do anuncio %s' % _id_anuncio)
#                 pass
#             self.dict_resposta[_id_anuncio] = res_url
#         # Pesquisar visitas
#         # _anuncios = str([str(x) for x in self.dict_resposta.keys()]).replace("'", '').replace(' ', '')[1:-1]
#         _url = '/items/visits?ids=%s&date_from=%sT00:00:00.000-03:00&date_to=%sT23:59:59.999-03:00#options' % (
#             str_codigos, self.dt_inicio, self.dt_hoje)
#         xml_str = self.mkp_con.get(_url)
#         if xml_str.status_code < 400:
#             for res in json.loads(xml_str.text):
#                 _codigo_anuncio = res.get('item_id')
#                 res_vis = {'qtd_visitas': res.get('total_visits', 0)}
#                 if res_url.get(_codigo_anuncio):
#                     res_url[_codigo_anuncio].update(res_vis)
#         return True
# 
#     def registrar(self, anun):
#         if not isinstance(anun, (tuple,list)):
#             self.anuncios.append(anun)
#         else:
#             for a in anun:
#                 self.anuncios.append(a)
#         return True
# 
#     def iniciar(self):
#         self.iniciou = True
#         while True:
#             qtd_anuncios = len(self.anuncios)
#             qtd_anuncios_com_problema = len(self.anuncios_com_problemas)
#             if qtd_anuncios == 0 and qtd_anuncios_com_problema == 0:
#                 break
#             if qtd_anuncios == 0:
#                for a in self.anuncios_com_problemas:
#                    self.anuncios.append(a)
#                qtd_anuncios = len(self.anuncios)
#             if qtd_anuncios > 20:
#                 qtd_anuncios = 20
#             codigos = ''
#             for i in range(qtd_anuncios):
#                 codigos += self.anuncios.pop(0) + ','
#             codigos = codigos[:-1]
#             self._codigos = codigos
#             thread.start_new_thread(self.pesquisar_no_ml, ())
#             # self._pesquisar_ml(codigos)
#         return True
# 
#     def retornar(self):
#         if self.terminou == True:
#             return False
#         res = {}
#         for key in self.dict_resposta.keys():
#             res[key] = self.dict_resposta.get(key)
#             self.dict_resposta.__delitem__(key)
#             if not (self.dict_resposta or self.anuncios):
#                 self.terminou = True
#                 break
#         return res


