# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from ...adm.control.utils import capitalize_all
import re
import logging
from openerp.tools.translate import _
from datetime import datetime


class mkp_utils(osv.Model):
    _inherit = 'mkp.utils'
    _logger = logging.getLogger(_inherit)

    def mkputl_postar_no_mkp(self, cr, uid, mkp_integrador, _conta_id, _ped_id, codigo_rastreio, data, context=None):
        """
        Metodo utilizado para informar aos marketplaces que o pedido foi postado.
        Deve ser sobrescrito em cada marketplace
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param _conta_id:
        :param _ped_id:
        :param data:
        :param context:
        :return:
        """
        _spg = self.pool.get
        if mkp_integrador != 'Tray':
            return super(mkp_utils, self).mkputl_postar_no_mkp(cr, uid, mkp_integrador, _conta_id, _ped_id, codigo_rastreio, data, context=context)
        res = _spg('mkp.tray')._mkpTray_get_pedido_interno_tray(cr, uid, pedido_id=_ped_id, context=context)
        if res.get('status') != True:
            return True
        _order = res.get('order')
        res = _spg('mkp.tray')._mkpTray_set_status_order(cr, uid, _conta_id, 'ENVIADO', _order, context=context)
        if data:
            data = data[:10]
        else:
            data = _spg('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(), formato='%Y-%m-%d')
        res = _spg('mkp.tray')._mkpTray_set_codigo_rastreio(cr, uid, _conta_id, _order, codigo_rastreio, data, context=context)
        return True

    def mkputl_entregar_no_mkp(self, cr, uid, brw_encomenda, context=None):
        """
        Metodo utilizado para informar aos marketplaces que o pedido foi entregue ao cliente
        Deve ser sobrescrito em cada marketplace
        :param cr:
        :param uid:
        :param brw_encomenda:
        :param context:
        :return:
        """
        _spg = self.pool.get
        if brw_encomenda.rasenc_pedido_id.pedcap_conta_id.mkpcta_integrador != 'Tray':
            return super(mkp_utils, self).mkputl_entregar_no_mkp(cr, uid, brw_encomenda, context=context)
        _conta_id = brw_encomenda.rasenc_pedido_id.pedcap_conta_id.id
        _ped_id = brw_encomenda.rasenc_pedido_id.id
        if not _ped_id:
            return True

        res = _spg('mkp.tray')._mkpTray_get_pedido_interno_tray(cr, uid, pedido_id=_ped_id, context=context)
        if res.get('status') != True:
            return True
        _order = res.get('order')
        res = _spg('mkp.tray')._mkpTray_set_status_order(cr, uid, _conta_id, 'FINALIZADO', _order,
                                                                  context=context)
        return True

    def mkputl_importar_um_pedido_tray(self, cr, uid, ped, conta_id, context=None):
        _spg = self.pool.get
        context = context and context.copy() or {}
        _venda = ped.get('Order')
        _point_sale = _venda.get('point_sale')
        _tr_venda_id = _tr_tray_venda_id = _venda.get('id')
        if _venda.get('external_code'):
            _tr_venda_id = '%s-%s' % (_point_sale, _venda.get('external_code'))
        elif _venda.get('store_note'):
            _tr_venda_id = _venda.get('store_note')
        self._logger.info('Importando o pedido: %s' % _tr_venda_id)
        brw_conta = _spg('mkp.contas').browse(cr, uid, conta_id, context=context)
        context['sig_force_company'] = brw_conta.company_id.id
        _local_id = brw_conta.mkpcta_local_id.id
        _hoje = _spg('adm.utils').admutl_datetime_to_timezone(cr, uid, datetime.now(),
                                                                       formato='%Y_%m_%d_%H_%M_%s')
        cr.execute(''' BEGIN; SAVEPOINT importar_pedido_tray_%s; ''' % _hoje)
        _comentario = _venda.get('shipment')
        _status_tr_name = _venda.get('status')
        _data_venda = _venda.get('date')
        _lista_itens = []
        _qtd_venda = 0
        _juros = _venda.get('taxes', 0.0)
        for _item in _venda.get('ProductsSold'):
            _pedido_item = {}
            _sku = _item.get('ProductsSold').get('reference')
            _sku_code, _sku_grade = _sku.split('-')
            _produto_ids = _spg('estoque.produto').search(cr, uid, [('codigo', '=', _sku_code)], context=context)
            if not _produto_ids:
                continue
            _produto_id = _produto_ids[0]
            _args = [('mkpanu_produto_id', '=', _produto_id), ('mkpanu_conta_id', '=', conta_id)]
            _anuncio_ids = _spg('mkp.anuncios').search(cr, uid, _args, context=context)
            if not _anuncio_ids:
                continue
            _frete_receita = 0
            _campos = ['mkpanu_valor_frete', 'mkpanu_frete', 'mkpanu_anuncio_tipo_id', 'mkpanu_quantidade']
            _dados_anuncio = _spg('mkp.anuncios').read(cr, uid, _anuncio_ids[0], _campos, context=context)
            _grade_ids = _spg('estoque.produto.grade').search(cr, uid, [('produto_id', '=', _produto_id),
                                                                                 ('grade', '=', _sku_grade)],
                                                                       context=context)
            if not _grade_ids:
                return {'status': False, 'mensagem': 'Grade do produto %s nao encontrada' % _sku}
            _qtd_anuncio = _dados_anuncio.get('mkpanu_quantidade', 1)
            if _qtd_anuncio == 0:
                _qtd_anuncio = 1
            _qtd_venda =int(_item.get('ProductsSold').get('quantity') * _qtd_anuncio)
            _valor_total_item = (float(_item.get('ProductsSold').get('price')) / _qtd_anuncio) * _qtd_venda
            _l_vl = _valor_total_item / _qtd_venda
            _args = [('grade_id', '=', _grade_ids[0]), ('local_id', '=', _local_id)]
            _emb_ids = _spg('estoque.local.locacao').search(cr, uid, _args, context=context)
            _emb_id = \
                _spg('estoque.local.locacao').read(cr, uid, _emb_ids[0], ['estllc_produto_embalagem_id'],
                                       context=context)['estllc_produto_embalagem_id'][0]
            _pedido_item = _spg('mkp.utils')._mkputl_monta_dic_item_pedido(cr, uid, _produto_id,
                                                                                    _grade_ids[0], _local_id, _emb_id,
                                                                                    _qtd_venda, _l_vl, _anuncio_ids[0])
            for x in _pedido_item:
                _lista_itens.append(x)
        _preco_total = float(_venda.get('total'))
        _cliente_tr_email = _venda.get('Customer').get('email')
        _cliente_tr_fone_1 = _cliente_tr_fone_2 = _cliente_tr_ddd_1 =  _cliente_tr_ddd_2 = False
        i = 0
        _fone = _venda.get('Customer').get('phone')
        ddd = ''
        if _fone:
            _fone = _fone.strip().replace('-', '').replace('.','').replace('/','').replace(' ','')
            while True: # retirar zeros à esquerda
                if _fone[:1] == '0':
                    _fone = _fone[1:]
                else:
                    break
            if len(_fone) > 8:
                ddd = _fone[:2]
                _nro_f = _fone[2:]
            _cliente_tr_ddd_1 = ddd
            _cliente_tr_fone_1 = _fone
        _fone = _venda.get('Customer').get('cellphone')
        if _fone:
            _fone = _fone.strip().replace('-', '').replace('.', '').replace('/', '').replace(' ', '')
            while True:  # retirar zeros à esquerda
                if _fone[:1] == '0':
                    _fone = _fone[1:]
                else:
                    break
            if len(_fone) > 8:
                ddd = _fone[:2]
                _nro_f = _fone[2:]
            _cliente_tr_ddd_2 = ddd
            _cliente_tr_fone_2 = _fone
        if _venda.get('Customer').get('company_name'):
            _cliente_nome = _venda.get('Customer').get('company_name')
            _cliente_nome = capitalize_all(_cliente_nome)
            _cliente_cpf = re.sub('[^0-9]', '', _venda.get('Customer').get('cnpj') or '')
            _cliente_ie = _venda.get('Customer').get('state_inscription')
        else:
            _cliente_nome = _venda.get('Customer').get('name')
            _cliente_nome = capitalize_all(_cliente_nome)
            _cliente_cpf = re.sub('[^0-9]', '', _venda.get('Customer').get('cpf') or '')
            _cliente_ie = ''
            if not _cliente_cpf:
                _cliente_cpf = re.sub('[^0-9]', '', _venda.get('Customer').get('cnpj') or '')
        _cliente_cep = _cliente_rua = _cliente_rua_nro = _cliente_rua_complemento = ''
        _cliente_cep = _venda.get('Customer').get('zip_code') or ''
        _cliente_rua = _venda.get('Customer').get('address') or ''
        _cliente_rua_nro = _venda.get('Customer').get('number') or ''
        _cliente_rua_complemento = _venda.get('Customer').get('complement') or ''
        _cliente_bairro = _venda.get('Customer').get('neighborhood') or ''
        args = [('pedcap_integrador', '=', 'Tray'), ('pedcap_mkp_carrinho_compras', '=', _tr_tray_venda_id), ('tipo', '=', 'V')]
        _pedido_ids = _spg('estoquepedidos.pedido.capa').search(cr, uid, args, context=context)
        _pedido_id = False
        _status_pedido_erp = False
        _pessoa_ids = False
        if _cliente_cpf:
            _pessoa_ids = _spg('adm.pessoa').search(cr, uid, [('cpf_cnpj', '=', _cliente_cpf)], context=context)
        _pais_id = False
        if _pessoa_ids:
            _pessoa_id = _spg('mkp.utils').mkputl_orders_alterar_pessoa(cr, uid, _pessoa_ids[0], _cliente_nome, _cliente_cpf,
                                                                   '', _cliente_ie, _cliente_cep, _cliente_rua,
                                                                   _cliente_rua_nro, _cliente_rua_complemento,
                                                                   _cliente_tr_ddd_1, _cliente_tr_fone_1,
                                                                   _cliente_tr_ddd_2, _cliente_tr_fone_2, 'Comum',
                                                                   _cliente_tr_email, context=context)
        else:
            # TODO FAZER TRATAMENTO PARA ALTERAR OS DADOS DO CLIENTE POIS PODE TER TIDO ALTERAÇÃO NO Marketplace
            _pessoa_id = _spg('mkp.utils').mkputl_orders_incluir_pessoa(cr, uid, _cliente_nome, _cliente_cpf, '',
                                                           '', _cliente_ie, _cliente_cep, _cliente_rua,
                                                           _cliente_rua_nro, _cliente_rua_complemento, _cliente_tr_ddd_1,
                                                           _cliente_tr_fone_1, _cliente_tr_ddd_2, _cliente_tr_fone_2,
                                                           'Comum', _cliente_tr_email, context=context)
        if _pedido_ids:  # O nro pedido do Marketplace é unico, portanto se tiver pedido so tem 1
            _pedido_id = _pedido_ids[0]
            _dados_pedido = _spg('estoquepedidos.pedido.capa').read(cr, uid, [_pedido_id], ['state'], context=context)
            _status_pedido_erp = _dados_pedido[0].get('state')
            if _status_pedido_erp == 'V': # Se o pedido estiver validado, voltar para esboco
                _spg('estoquepedidos.pedido.capa').button_rascunho(cr, uid, [_pedido_id], context=context)
                _status_pedido_erp = 'R'
            if _status_pedido_erp not in ('R', 'C'):  # O pedido não está mais em esboço ou cancelado, não deve ser alterado
                vals = {
                    'pedcap_mkp_status': _status_tr_name,
                }
                _spg('estoquepedidos.pedido.capa').write(cr, uid, [_pedido_id], vals, context=context)
                cr.execute(''' COMMIT; ''')  # Comito, porque se houver algum problema na validação, não perco o pedido
                return {'status': True, 'mensagem': '', 'pedido': _pedido_id}
        _condicao_forma_id = False

        _tipo_anuncio = ''
        _vencimento = _data_venda
        _forma = []
        _condicao_forma_id = brw_conta.mkpcta_marketplace_id.mkpmkp_tr_condicao_pgto_id.id
        _taxa_tr_fixo = 0
        _taxa_tr_percentual = brw_conta.mkpcta_tr_comisao_mkp
        if _point_sale != 'LOJA VIRTUAL':
            for brw_adicional in brw_conta.mkpcta_marketplace_id.mkpmkp_mark_adicional_ids:
                if brw_adicional.mkpmkpadc_apelido == _point_sale:
                    brw_condicao = brw_adicional.mkpmkpadc_condicao_pgto_id
                    _taxa_tr_fixo = brw_adicional.mkpmkpadc_adicional_fixo
                    _taxa_tr_percentual = brw_adicional.mkpmkpadc_adicional_percentual
                    _forma1 = {
                        'forma_pagto_id': brw_adicional.mkpmkpadc_condicao_pgto_id.forma_pagamento_id.id,
                        'condicao_forma_id': brw_adicional.mkpmkpadc_condicao_pgto_id.id,
                        'pedido_parcela_ids': [(0, 0, {'vr_parcela': _preco_total, 'data_vencimento': _vencimento, })],
                        'valor': _preco_total,
                        'pedcdc_mkp_tarifa': (_taxa_tr_percentual * _preco_total / 100) + _taxa_tr_fixo,
                    }
                    _forma.append((0, 0, _forma1))
        else:
            _forma1 = {
                'forma_pagto_id': brw_conta.mkpcta_marketplace_id.mkpmkp_tr_condicao_pgto_id.forma_pagamento_id.id,
                'condicao_forma_id': brw_conta.mkpcta_marketplace_id.mkpmkp_tr_condicao_pgto_id.adm_condicao_id.id,
                'pedido_parcela_ids': [(0, 0, {'vr_parcela': _preco_total, 'data_vencimento': _vencimento,})],
                'valor': _preco_total,
                'pedcdc_mkp_tarifa': (_taxa_tr_percentual * _preco_total / 100) * _taxa_tr_fixo,
            }
            _forma.append((0, 0, _forma1))

        dados_pessoa = _spg('adm.pessoa').read(cr, uid, _pessoa_id, ['endereco_nf_id'], context=context)
        if dados_pessoa and dados_pessoa.get('endereco_nf_id'):
            _end_id = dados_pessoa.get('endereco_nf_id')[0]
        else:
            _end_id = ''
        _desconto = float(_venda.get('payment_method_rate'))
        if _desconto:
            if _desconto > 0:
                _juros = _desconto
                _desconto = 0
            else:
                _desconto = _desconto * -1
        if float(_venda.get('discount')):
            _desconto += float(_venda.get('discount'))
        _vr_frete = _venda.get('shipping_cost', 0.0)
        if _venda.get('shipment_value'):  # frete de outros marketplaces, não Tray
            _vr_frete = _venda.get('shipment_value')
        vals = {
            'observacao': _comentario,
            'data_emissao': _data_venda,
            'tipo': 'V',
            'pessoa_id': _pessoa_id,
            'pedcap_endereco_id': _end_id,
            'pedcap_mkp_status': _status_tr_name,
            'pedcap_nro_pedido_externo': _tr_venda_id,
            'pedcap_endereco_eletronico': brw_conta.mkpcta_tr_site + '/adm/pedidos/detalhe_pedido.php?id_pedido=%s' % _tr_tray_venda_id,
            'pedcap_mkp_tipo_anuncio': _tipo_anuncio,
            'vr_frete': _vr_frete,
            'vr_desconto': _desconto,
            'vr_seguro': 0.00,
            'vr_despesas_acessorias': _juros,
            'pedcap_conta_id': conta_id,
            'pedcap_mkp_carrinho_compras': _tr_tray_venda_id,
            'pedcap_mkp_shipping': _venda.get('shipment'),
        }

        if not _pedido_id:
            if _anuncio_ids:
                vals['pedido_item_ids'] = _lista_itens
            if _forma:
                vals['pedido_condicao_ids'] = _forma
            _pedido_id = _spg('estoquepedidos.pedido.capa').create(cr, uid, vals, context=context)
            _spg('estoquepedidos.pedido.capa').write(cr, uid, [_pedido_id], {}, context=context)
            cr.execute(''' COMMIT; ''')  # Comito, porque se houver algum problema na validação, não perco o pedido
            cr.execute(''' BEGIN; SAVEPOINT importar_pedido_tray_%s; ''' % _hoje)
        else:
            if _status_tr_name == 'ENVIADO' and _status_pedido_erp in ('R', 'V'):
                _spg('estoquepedidos.pedido.capa').write(cr, uid, [_pedido_id], {'state': 'F'}, context=context)
            elif _status_tr_name == 'A ENVIAR' and _status_pedido_erp == 'C':
                vals['state'] = 'R'
                context['salvar_forcado'] = True
                res = _spg('estoquepedidos.pedido.capa').write(cr, uid, [_pedido_id], vals, context=context)
            elif _status_tr_name != 'CANCELADO' and _status_tr_name != 'CANCELADO AUT' and _status_pedido_erp == 'C':
                return {'status': False, 'mensagem': 'O pedido esta cancelado no ERP mas nao esta no MKP'}
            elif _status_tr_name != 'CANCELADO':
                _condicao_ids = _spg('estoquepedidos.pedido.condicao').search(cr, uid, [('capa_id', '=', _pedido_id)], context=context)
                if _condicao_ids and _forma:
                    _spg('estoquepedidos.pedido.condicao').unlink(cr, uid, _condicao_ids, context=context)
                vals['pedido_condicao_ids'] = _forma
                vals['state'] = 'R'
                context['salvar_forcado'] = True
                res = _spg('estoquepedidos.pedido.capa').write(cr, uid, [_pedido_id], vals, context=context)
                cr.execute(''' COMMIT; ''')  # Comito, porque se houver algum problema na validação, não perco o pedido
                cr.execute(''' BEGIN; SAVEPOINT importar_pedido_tray_%s; ''' % _hoje)
        if _status_tr_name in ('A ENVIAR', 'A ENVIAR YAPAY', 'A ENVIAR VINDI', 'AGUARDANDO ENVIO'):
            try:
                _spg('estoquepedidos.pedido.capa').button_validar(cr, uid, [_pedido_id], context=context)
            except:
                pass
        elif _status_tr_name in ('CANCELADO', 'CANCELADO AUT') and _status_pedido_erp != 'C':
            if _status_pedido_erp != 'F':
                _spg('estoquepedidos.pedido.capa').button_cancelar(cr, uid, [_pedido_id], context=context)
            else:
                cr.execute(''' COMMIT; ''')
                return {'status': True, 'mensagem': 'Pedido Faturado no ERP e cancelado no MKP'}
        _encomenda = False
        cr.execute(''' COMMIT; ''')
        # self.pedcap_ver_se_atualiza_saldo_anuncios(cr, uid, [_pedido_id], context=context)
        return {'status': True, 'mensagem': '', 'pedido': _pedido_id}


    def mkputl_cron_pesquisar_pedidos_tray(self, cr, uid, context=None):
        """
        Método chamado pelo cron para pesquisar pedidos nos marketplaces
        @param cr:
        @param uid:
        @param context:
        @return: Sempre True
        """
        context = context and context.copy() or {}
        obj_contas = self.pool.get('mkp.contas')
        for company_id in self.pool.get('res.company').search(cr, uid, [], context=context):
            context['sig_force_company'] = True
            args = [('mkpcta_integrador', '=', 'Tray'), ('company_id', '=', company_id)]
            conta_ids = obj_contas.search(cr, uid, args, context=context)
            for brw_conta in obj_contas.browse(cr, uid, conta_ids, context=context):
                if brw_conta.mkpcta_parar_comunicacao or \
                   brw_conta.mkpcta_marketplace_id.mkpmkp_parar_comunicacao:
                    continue
                _conta_id = brw_conta.id
                _mkp_codigo = brw_conta.mkpcta_marketplace_id.mkpmkp_codigo
                self._mkputl_pesquisar_pedido_mkp_tray(cr, uid, _conta_id, _mkp_codigo, context=context)
        return True

    def _mkputl_pesquisar_pedido_mkp_tray(self, cr, uid, conta_id, codigo_mkp, context=None):
        """
        Metodo a ser sobrescrito para cada market place
        :param cr:
        :param uid:
        :param conta_id: id da conta no marketplace
        :param codigo_mkp: codigo de identificação do marketplace
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        self._logger.info('Inicio de pesquisa de pedidos Tray')
        ped = self.pool.get('mkp.tray').mkpTray_pesquisar_orders(cr, uid, conta_id, context=context)
        self._logger.info('Fim de pesquisa de pedidos Tray')
        return True


    def mkputl_faturar_no_mkp_selecionado(self, cr, uid, mkp_integrador, conta_id, ped_externo, chave, nf_ids, context=None):
        """
        Metodo sobrescrito ppelo marketplace especifico
        :param cr:
        :param uid:
        :param mkp_integrador:
        :param conta_id:
        :param ped_externo:
        :param chave:
        :param context:
        :return:
        """
        context = context and context.copy() or {}
        if mkp_integrador == 'Tray':
            res = self.pool.get('mkp.tray').mkpTray_set_order_faturado(cr, uid, ped_externo, conta_id,
                                                                     chave, nf_ids, context=context)
            # if not res.get('status'):
            #     mensagem = 'Nao consegui faturar no marketplace! \n %s' % res.get('mensagem')
            #     raise osv.except_osv(_('Atencao'), _(mensagem))
        else:
            res = super(mkp_utils, self).mkputl_faturar_no_mkp_selecionado(cr, uid, mkp_integrador, conta_id, ped_externo,
                                                                          chave, nf_ids, context=context)
        return res

    def mkputl_atualizar_anuncio_tray(self, cr, uid, id, context=None):
        """
        Inclui ou altera anuncio no Tray
        @param cr:
        @param uid:
        @param id: id do anuncio a ser alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """
        context = context and context.copy() or {}
        obj_Tray = self.pool.get('mkp.tray')
        obj_anuncios = self.pool.get('mkp.anuncios')
        brw_anuncio = obj_anuncios.browse(cr, uid, id, context=context)
        if not brw_anuncio: # anuncio nao existe mais. Retornar True para que a ação seja encerrada
            return {'status': True, 'mensagem': ''}
        try:
            _codigo = brw_anuncio.mkpanu_codigo_anuncio
        except: # Erro porque o id não existe
            return {'status': True, 'mensagem': ''}
        dados_anuncio = obj_Tray.mkpTray_get_anuncio(cr, uid, brw_anuncio.mkpanu_conta_id.id, _codigo, context=context)
        if dados_anuncio:
            vals = {'mkpanu_qtd_vendido': dados_anuncio.get('Product').get('quantity_sold'),
                    'mkpanu_necessita_atualizar': False}
            obj_anuncios.write(cr, uid, id, vals, context=context)
        res = obj_Tray.mkpTray_set_anuncios(cr, uid, brw_anuncio, context=context)
        if res and res.get('status') == False:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao consegui atualizar o anuncio!\n%s' % res.get('mensagem')))
        _id_mkp_prod = res.get('id')
        _loja = brw_anuncio.mkpanu_conta_id.mkpcta_tr_id_loja
        _url = brw_anuncio.mkpanu_conta_id.mkpcta_tr_site + '/loja/produto.php?loja=%s&IdProd=%s'
        _url = _url % (_loja, _id_mkp_prod)
        vals = {
            'mkpanu_codigo_anuncio': _id_mkp_prod,
            'mkpanu_anuncio_endereco': _url,
            'mkpanu_necessita_atualizar': False,
        }
        obj_anuncios.write(cr, uid, id, vals, context=context)

        return res

    def mkputl_atualizar_preco_tray(self, cr, uid, id, context=None):
        """
        altera precos no Tray
        @param cr:
        @param uid:
        @param id: id do anuncio a ser alterado
        @param context:
        @return: dicionario contendo id do anuncio e response ex.:{status: True/False, 'mensagem': 'Mensagem'}
                 response. quando response.raise_for_status() for None, tá tudo certo. Para pegar o erro use response.status_code
        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        brw_anuncio = _spg('mkp.anuncios').browse(cr, uid, id, context=context)
        if not brw_anuncio: # anuncio nao existe mais. Retornar True para que a ação seja encerrada
            return {'status': True, 'mensagem': ''}
        try:
            _codigo = brw_anuncio.mkpanu_codigo_anuncio
        except: # Erro porque o id não existe
            return {'status': True, 'mensagem': ''}
        res = _spg('mkp.tray').mkpTray_set_preco(cr, uid, brw_anuncio, context=context)
        if res and res.get('status') == False:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao consegui atualizar o preco!\n%s' % res.get('mensagem')))
        return res

