# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.xml_sped.__init__ import UF_CODIGO
from pynfe.mdfe.manual_300.MDFe import AutXML
from pynfe.mdfe.manual_300.MDFe import MDFe
from openerp.tools.translate import _
from datetime import datetime
from openerp.osv import osv
import re

__version__ = u'SIG-ERP 9J1'


class mdfe(osv.Model):
    """
    Classe Manifesto de Documento Fiscal Eletronico
    """
    _inherit = 'mdfe'  # mdfe

    def __init__(self, pool, cr):
        super(mdfe, self).__init__(pool, cr)
        self._erro, self._warn, self._info = [], [], []

    def mdfe_get_chave_acesso(self, cr, user, ids, context=None):
        """Monta a chave de acesso do MDF-e"""
        mdfe = self.browse(cr, user, ids, context)
        _ids = ids[:] if isinstance(ids, (list, tuple)) else [ids]
        result = {}.fromkeys(_ids, '')
        for mdfe in self.browse(cr, user, _ids, context):
            chave = str(
                UF_CODIGO.get(mdfe.company_id.rescon_pessoa_id.admpes_endereco.uf_id.sigla))
            chave += mdfe.mdfe_data_emissao[2:4] + mdfe.mdfe_data_emissao[5:7]
            chave += re.sub('[^0-9]', '', mdfe.company_id.rescon_pessoa_id.cpf_cnpj).zfill(14)
            chave += '58'
            chave += mdfe.mdfe_serie.zfill(3)
            chave += mdfe.mdfe_numero.__str__().zfill(9)
            chave += '2' if mdfe.mdfe_emissao_contingencia else '1'
            chave += mdfe.id.__str__().zfill(8)
            chave += self._mod11(chave).__str__()
            result[mdfe.id] = chave

        return result if isinstance(ids, (list, tuple)) else result[ids]

    @staticmethod
    def _mod11(sequencia):
        """
        Realiza o calculo do modulo 11. O módulo 11 eh calculado multiplicando-se cada algarismo
        pela sequencia de multiplicadores 2,3,4,5,6,7,8,9,2,3, ... posicionados da direita para a
        esquerda.
        :param sequencia: sequencia de numeros utilizada no calculo do modulo-11
        :type sequencia: int|long|string
        :return: digito verificador
        :rtype: int
        """
        sequencia = ''.join(filter(lambda f: f in '0123456789', str(sequencia)))  # Algarismos
        soma = 0
        peso = 2
        for nr in reversed(sequencia):
            soma += int(nr) * peso
            peso += 1
            if peso > 9:
                peso = 2
        result = 11 - soma % 11
        result = 0 if result > 9 else result

        return result

    def _mdfe_validar_tipo_emitente(self, cr, user, mdfe, context=None):
        """Validacoes do tipo de emitente - """
        context = context and context.copy() or {}

    def _mdfe_validar_carregamento_posterior(self, cr, user, mdfe, context=None):
        """Validacoes do carregamento posterior"""
        context = context and context.copy() or {}

    def _mdfe_validar_documento_transportado(self, cr, user, mdfe, context=None):
        """Validacoes dos documentos transportados"""
        context = context and context.copy() or {}

    def _mdfe_validar_emitente(self, cr, user, mdfe, context=None):
        """Validacoes do emitente"""
        context = context and context.copy() or {}
        self._mdfe_validar_tipo_emitente(cr, user, mdfe, context)
        if mdfe.company_id.rescon_pessoa_id.pessoa_tipo == 'J':
            if '919' < mdfe.mdfe_serie > '970':
                self._erro.append(_('A Serie informada NAO pode estar na faixa de 920 a 969.'))
        else:
            if not '919' < mdfe.mdfe_serie > '969':
                self._erro.append(_('A Serie informada deve estar na faixa de 920 A 969.'))

    def _mdfe_validar_data_emissao(self, cr, user, mdfe, context=None):
        """Validacoes da data de emissao"""
        context = context and context.copy() or {}
        if mdfe.mdfe_data_emissao[2:4] < '12':
            self._erro.append(_('Ano da data de emissao nao pode ser inferior a 2012.'))

    def _mdfe_validar_banco_dados(self, cr, user, mdfe, context=None):
        """Validacoes do Banco de Dados"""
        context = context and context.copy() or {}

    def _mdfe_validar_autorizado_xml(self, cr, user, mdfe, context=None):
        """Validacoes dos autorizados a download do XML"""
        context = context and context.copy() or {}

    def _mdfe_validar_qr_code(self, cr, user, mdfe, context=None):
        """Validacoes do QR Code"""
        context = context and context.copy() or {}

    def _mdfe_validar_responsavel_tecnico(self, cr, user, mdfe, context=None):
        """Validacoes do responsavel tecnico"""
        context = context and context.copy() or {}

    def mdfe_validar(self, cr, user, ids, context=None):
        """Valida um ou mais MDF-e indicando avisos e erros ocorridos durante o processo de
        validacao"""
        _ids = isinstance(ids, (list, tuple)) and ids[:] or [ids]
        for _mdfe in self.browse(cr, user, _ids, context):
            self._mdfe_validar_carregamento_posterior(cr, user, _mdfe, context)
            self._mdfe_validar_documento_transportado(cr, user, _mdfe, context)
            self._mdfe_validar_emitente(cr, user, _mdfe, context)
            self._mdfe_validar_data_emissao(cr, user, _mdfe, context)
            self._mdfe_validar_banco_dados(cr, user, _mdfe, context)
            self._mdfe_validar_autorizado_xml(cr, user, _mdfe, context)
            self._mdfe_validar_qr_code(cr, user, _mdfe, context)
            self._mdfe_validar_responsavel_tecnico(cr, user, _mdfe, context)

    def _mdfe_tag_ide(self, cr, user, mdfe, obj, context=None):
        """Identificacao do MDF-e - infMDFe.ide"""
        context = context and context.copy() or {}
        obj.infMDFe.ide.cUF.valor = UF_CODIGO.get(
            mdfe.company_id.rescon_pessoa_id.admpes_endereco.uf_id.sigla)
        obj.infMDFe.ide.tpAmb.valor = mdfe.parametro().fiscal.fispar_mdfe_ambiente
        obj.infMDFe.ide.tpEmit.valor = mdfe.mdfe_tipo_emitente
        obj.infMDFe.ide.tpTransp.valor = mdfe.parametro().fiscal.fispar_mdfe_tipo_transportador
        obj.infMDFe.ide.mod.valor = u'58'
        obj.infMDFe.ide.serie.valor = int(mdfe.mdfe_serie)
        obj.infMDFe.ide.nMDF.valor = mdfe.mdfe_numero
        obj.infMDFe.ide.cMDF.valor = mdfe.id.__str__().zfill(8)
        obj.infMDFe.ide.cDV.valor = mdfe.mdfe_chave_acesso[-1]
        obj.infMDFe.ide.modal.valor = 1
        obj.infMDFe.ide.dhEmi.valor = datetime.strptime(
            self.pool.get('adm.utils').admutl_datetime_to_timezone(
                cr, user, datetime.strptime(mdfe.mdfe_data_emissao, '%Y-%m-%d %H:%M:%S'),
                '%d-%m-%Y %T'), '%d-%m-%Y %H:%M:%S')
        obj.infMDFe.ide.tpEmis.valor = 2 if mdfe.mdfe_emissao_contingencia else 1
        obj.infMDFe.ide.procEmi.valor = u'0'
        obj.infMDFe.ide.verProc.valor = __version__
        obj.infMDFe.ide.dhIniViagem.valor = mdfe.mdfe_data_inicio_viagem

    def _mdfe_tag_emit(self, cr, user, mdfe, obj, context=None):
        """Identificacao do emitente do Manifesto"""
        context = context and context.copy() or {}
        obj.infMDFe.emit.cnpj.valor = \
            re.sub('[^0-9]', '', mdfe.company_id.rescon_pessoa_id.cpf_cnpj)
        obj.infMDFe.emit.ie.valor = mdfe.company_id.rescon_pessoa_id.admpes_jur_ie
        obj.infMDFe.emit.xNome.valor = mdfe.company_id.rescon_pessoa_id.name
        obj.infMDFe.emit.xFant.valor = mdfe.company_id.rescon_pessoa_id.fantasia_apelido

        endereco = mdfe.company_id.rescon_pessoa_id.admpes_endereco
        obj.infMDFe.emit.enderEmit.xLgr.valor = endereco.logradouro_id.descricao
        obj.infMDFe.emit.enderEmit.nro.valor = endereco.numero
        obj.infMDFe.emit.enderEmit.xCpl.valor = endereco.complemento
        obj.infMDFe.emit.enderEmit.xBairro.valor = endereco.bairro_id.name
        obj.infMDFe.emit.enderEmit.cMun.valor = endereco.cidade_id.codigo_ibge.__str__()
        obj.infMDFe.emit.enderEmit.xMun.valor = endereco.cidade_id.name
        obj.infMDFe.emit.enderEmit.cep.valor = re.sub('[^0-9]', '', endereco.cep)
        obj.infMDFe.emit.enderEmit.uf.valor = endereco.uf_id.sigla
        obj.infMDFe.emit.enderEmit.fone.valor = u''
        obj.infMDFe.emit.enderEmit.email.valor = u''

    def _mdfe_tag_infdoc(self, cr, user, mdfe, obj, context=None):
        """Informacoes dos Documentos Fiscais vinculados ao Manifesto"""
        context = context and context.copy() or {}
        pass

    def _mdfe_tag_seg(self, cr, user, mdfe, obj, context=None):
        """Informacoes de seguro da carga - Nao implementado"""
        context = context and context.copy() or {}
        pass

    def _mdfe_tag_tot(self, cr, user, mdfe, obj, context=None):
        """Totalizadores da carga transportada e seus documentos fiscais"""
        context = context and context.copy() or {}
        obj.infMDFe.tot.qCTe.valor = 0
        obj.infMDFe.tot.qNFe.valor = 0
        obj.infMDFe.tot.qMDFe.valor = 0  # Somente modalidade aquaviario
        obj.infMDFe.tot.vCarga.valor = 0.00
        obj.infMDFe.tot.cUnid.valor = u'KG'  # Unidade padrao
        obj.infMDFe.tot.qCarga.valor = 0.000

    def _mdfe_tag_lacres(self, cr, user, mdfe, obj, context=None):
        """Lacres do MDF-e"""
        context = context and context.copy() or {}

    def _mdfe_tag_autxml(self, cr, user, mdfe, obj, context=None):
        """Autorizados para download do XML do DF-e"""
        context = context and context.copy() or {}
        autorizado = AutXML()
        autorizado.cnpj.valor = u'01526558000178'  # SIG Informatica
        obj.infMDFe.autXML.append(autorizado)

    def _mdfe_tag_infadic(self, cr, user, mdfe, obj, context=None):
        """Informacoes adicionais"""
        context = context and context.copy() or {}
        obj.infMDFe.infAdic.infAdFisco.valor = mdfe.mdfe_informacao_fisco
        obj.infMDFe.infAdic.infCpl.valor = mdfe.mdfe_informacao_contribuinte

    def _mdfe_tag_infresptec(self, cr, user, mdfe, obj, context=None):
        """Informacoes do Responsavel Tecnico pela emissao do DF-e"""
        context = context and context.copy() or {}
        parametro_fiscal = mdfe.parametro().fiscal
        obj.infMDFe.infRespTec.cnpj.valor = parametro_fiscal.fispar_nfe_cnpj_responsavel_tecnico
        obj.infMDFe.infRespTec.xContato.valor = parametro_fiscal.fispar_nfe_contato
        obj.infMDFe.infRespTec.email.valor = parametro_fiscal.fispar_nfe_email
        obj.infMDFe.infRespTec.fone.valor = parametro_fiscal.fispar_nfe_fone
        obj.infMDFe.infRespTec.idCSRT.valor = parametro_fiscal.fispar_nfe_csrt_id
        obj.infMDFe.infRespTec.hashCSRT.valor = parametro_fiscal.fispar_nfe_csrt

    def _mdfe_tag_infmdfe(self, cr, user, mdfe, obj, context=None):
        """Informacoes do MDF-e - InfMDFe"""
        self._mdfe_tag_ide(cr, user, mdfe, obj, context)
        self._mdfe_tag_emit(cr, user, mdfe, obj, context)
        self._mdfe_tag_infdoc(cr, user, mdfe, obj, context)
        self._mdfe_tag_seg(cr, user, mdfe, obj, context)  # Seguro da carga. Nao implementado
        self._mdfe_tag_tot(cr, user, mdfe, obj, context)
        self._mdfe_tag_lacres(cr, user, mdfe, obj, context)
        self._mdfe_tag_autxml(cr, user, mdfe, obj, context)
        self._mdfe_tag_infadic(cr, user, mdfe, obj, context)
        self._mdfe_tag_infresptec(cr, user, mdfe, obj, context)
        obj.infMDFe.Id.valor = u'MDFe' + mdfe.mdfe_chave_acesso

    def _mdfe_tag_infmdfesupl(self, cr, user, mdfe, obj, context=None):
        """Informacoes suplementares do MDF-e"""
        context = context and context.copy() or {}
        uri = u'https://dfe-portal.svrs.rs.gov.br/mdfe/qrCode?chMDFe={}&tpAmb={}'
        uri = uri.format(mdfe.mdfe_chave_acesso, mdfe.parametro().fiscal.fispar_mdfe_ambiente)
        obj.infMDFeSupl.qrCodMDFe.valor = uri

    def mdfe_gerar(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        _ids = isinstance(ids, (list, tuple)) and ids[:] or [ids]
        result = {}.fromkeys(_ids, None)
        for mdfe in self.browse(cr, user, _ids, context):
            obj = MDFe()
            self._mdfe_tag_infmdfe(cr, user, mdfe, obj, context)
            self._mdfe_tag_infmdfesupl(cr, user, mdfe, obj, context)
            result[mdfe.id] = obj

        return isinstance(ids, (list, tuple)) and result or result[ids]

    def mdfe_onchange_emissao_contingencia(self, cr, user, ids, contingencia, chave, context=None):
        """Recupera os dados do veiculo para mostrar na aba ´´Rodoviario/Veiculo de tracao´´."""
        if not chave:
            return {}
        chave = bytearray(chave[:-1], 'utf-8')
        chave[34] = ord('2') if contingencia else ord('1')
        chave = str(chave)
        chave += self._mod11(chave).__str__()

        return {'value': {'mdfe_chave_acesso': chave}}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
