# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from pynfe.mdfe.manual_300.MDFe import InfMunCarrega, InfMunDescarga, InfNFe, InfPercurso
from pynfe.mdfe.manual_300.modalRodoviario import ModalRodo, Disp, Condutor, VeicReboque
from pynfe.mdfe.manual_300.MDFe import Lacres as Lacre, AutXML
from pynfe.mdfe.manual_300.processador import ProcessadorMDFe
from ..report.danfe_rodo import DAMDFeRodo as DAMDFE
from openerp.tools.translate import _
from datetime import datetime
from openerp.osv import osv
import base64
import time
import mdfe
import re


class mdfe_modal_rodoviario(mdfe.mdfe):
    """
    Classe Modalidade Rodoviario
    """
    _inherit = 'mdfe.modal.rodoviario'  # mmodr

    def mmodr_get_totalizadores(self, cr, user, ids, municipio_ids, context=None):
        """
        Totaliza a quantidade de NF-e relacionadas no Manifesto.

        :param cr: cursor no banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario
        :param municipio_ids: lista com o ID dos municipios de descarregamento (many2many)
        :param context: dicionario com valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type municipio_ids: list
        :type context: dict
        :return: o somatorio dos valores liquidos e brutos das NF relacionadas
        :rtype: dict
        """
        context = context and context.copy() or dict()
        values = {
            'value': {
                'mmodr_qtde_total_cte': self.mmodr_get_qtde_total_cte(
                    cr, user, municipio_ids, context),
                'mmodr_qtde_total_nfe': self.mmodr_get_qtde_total_nfe(
                    cr, user, municipio_ids, context),
                'mmodr_qtde_total_nf': self.mmodr_get_qtde_total_nf(
                    cr, user, municipio_ids, context),
                'mmodr_valor_total': self.mmodr_get_valor_total_mercadoria(
                    cr, user, municipio_ids, context),
                'mmodr_peso_bruto_total': self.mmodr_get_peso_bruto_total(
                    cr, user, municipio_ids, context),
            }
        }

        return values

    def mmodr_get_qtde_total_cte(self, cr, user, municipio_ids, context=None):
        return sum([len(
            self.pool.get('mdfe.municipio.descarregamento').resolve_2many_commands(
                cr, user, 'mdfemd_cte_referencia_ids', m.get('mdfemd_cte_referencia_ids') or '',
                context)) for m in self.resolve_2many_commands(
            cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids, [], context)])

    def mmodr_get_qtde_total_nfe(self, cr, user, municipio_ids, context=None):
        return sum([len(
            self.pool.get('mdfe.municipio.descarregamento').resolve_2many_commands(
                cr, user, 'mdfemd_nfe_referencia_ids', m.get('mdfemd_nfe_referencia_ids') or '',
                context)) for m in self.resolve_2many_commands(
            cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids, [], context)])

    def mmodr_get_qtde_total_nf(self, cr, user, municipio_ids, context=None):
        return sum([len(
            self.pool.get('mdfe.municipio.descarregamento').resolve_2many_commands(
                cr, user, 'mdfemd_nf_referencia_ids', m.get('mdfemd_nf_referencia_ids') or '',
                context)) for m in self.resolve_2many_commands(
            cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids, [], context)])

    def mmodr_get_valor_total_mercadoria(self, cr, user, municipio_ids, context=None):
        context = context and context.copy() or {}
        _municipio_ids = municipio_ids and municipio_ids[:] or []
        carga_propria = self.parametro(cr, user).fiscal.fispar_mdfe_tipo_emitente == '2'

        def get_ids(command):
            result = command[0][2] if command and isinstance(command[0], list) else command
            return result

        total_cte = not carga_propria and sum(
            [sum(map(lambda s: s and s['vr_total_nf'] or 0, self.pool.get('fiscal.nf.capa').read(
                cr, user, get_ids(m['mdfemd_cte_referencia_ids']), ['vr_total_nf'], context)))
             for m in self.resolve_2many_commands(
                cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids,
                ['mdfemd_cte_referencia_ids'], context)])
        total_nfe = carga_propria and sum(
            [sum(map(lambda s: s and s['vr_total_nf'] or 0, self.pool.get('fiscal.nf.capa').read(
                cr, user, get_ids(m['mdfemd_nfe_referencia_ids']), ['vr_total_nf'], context)))
             for m in self.resolve_2many_commands(
                cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids,
                ['mdfemd_nfe_referencia_ids'], context)])
        total_nf = carga_propria and sum(
            [sum(map(lambda s: s and s['vr_total_nf'] or 0, self.pool.get('fiscal.nf.capa').read(
                cr, user, get_ids(m['mdfemd_nf_referencia_ids']), ['vr_total_nf'], context)))
             for m in self.resolve_2many_commands(
                cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids,
                ['mdfemd_nf_referencia_ids'], context)])

        return total_cte + total_nfe + total_nf

    def mmodr_get_peso_bruto_total(self, cr, user, municipio_ids, context=None):
        context = context and context.copy() or {}
        _municipio_ids = municipio_ids and municipio_ids[:] or []
        carga_propria = self.parametro(cr, user).fiscal.fispar_mdfe_tipo_emitente == '2'

        def get_ids(command):
            result = command[0][2] if command and isinstance(command[0], list) else command
            return result

        peso_cte = not carga_propria and sum(
            [sum(map(lambda s: s and s['peso_bruto'] or 0, self.pool.get('fiscal.nf.capa').read(
                cr, user, get_ids(m['mdfemd_cte_referencia_ids']), ['peso_bruto'], context)))
             for m in self.resolve_2many_commands(
                cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids,
                ['mdfemd_cte_referencia_ids'], context)])
        peso_nfe = carga_propria and sum(
            [sum(map(lambda s: s and s['peso_bruto'] or 0, self.pool.get('fiscal.nf.capa').read(
                cr, user, get_ids(m['mdfemd_nfe_referencia_ids']), ['peso_bruto'], context)))
             for m in self.resolve_2many_commands(
                cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids,
                ['mdfemd_nfe_referencia_ids'], context)])
        peso_nf = carga_propria and sum(
            [sum(map(lambda s: s and s['peso_bruto'] or 0, self.pool.get('fiscal.nf.capa').read(
                cr, user, get_ids(m['mdfemd_nf_referencia_ids']), ['peso_bruto'], context)))
             for m in self.resolve_2many_commands(
                cr, user, 'mmodr_municipio_descarrega_ids', municipio_ids,
                ['mdfemd_nf_referencia_ids'], context)])

        return peso_cte + peso_nfe + peso_nf

    def mmodr_onchange_veiculo(self, cr, user, ids, veiculo_id, context=None):
        """Recupera os dados do veiculo para mostrar na aba ´´Rodoviario/Veiculo de tracao´´."""
        context = context and context.copy() or {}
        _fields = ['veivei_tara', 'veivei_tipo_carroceria', 'veivei_uf_id', 'veivei_renavan',
                   'carga_maxima_peso', 'veivei_tipo_rodado', 'veivei_capacidade_m3',
                   'veivei_pessoa_id']
        if veiculo_id:
           values = self.pool.get('veiculos.veiculo').read(
               cr, user, veiculo_id, _fields, context, '_classic_write')
        else:
            values = {}.fromkeys(_fields, False)

        return {'value': values}

    def _mmodr_validar_proprietario(self, cr, user, mmodr, context=None):
        """Validacao das informacoes de proprietario do veiculo (Veiculo de tracao)"""
        context = context and context.copy() or {}
        if mmodr.mmodr_veiculo_proprietario:
            if mmodr.veivei_pessoa_id.id:
                proprietario = mmodr.veivei_pessoa_id
                if not proprietario.cpf_cnpj:
                    self._erro.append(_('Proprietario do veiculo sem informacao de CPF ou CNPJ'))
                if not proprietario.admpes_tipo_proprietario:
                    self._erro.append(_('Proprietario do veiculo sem informacao do tipo: TAC ou '
                                        'outros'))
                if not proprietario.name:
                    self._erro.append(_('Nome do proprietario do veiculo deve ser informado'))
                if proprietario.admpes_rntrc:
                    if len(proprietario.admpes_rntrc) != 8:
                        self._warn.append(_('RNTRC do proprietario do veiculo deve conter sempre 8 '
                                            'caracteres.'))
            else:
                self._erro.append(_('O proprietario do veiculo deve ser informado quando nao e o '
                                    'mesmo emitente do MDF-e'))

    def _mmodr_validar_condutor(self, cr, user, mmodr, context=None):
        """Valida as informacoes dos condutores do veiculo"""
        if not len(mmodr.veivei_condutor_ids):
            self._erro.append(_('Ao menos um condutor para o veiculo de tracao deve ser informado'))
        if filter(lambda f: not f, mmodr.veivei_condutor_ids):
            self._erro.append(_('Existe condutor sem o CPF informado'))
        if len(mmodr.veivei_condutor_ids) > 10:
            self._warn.append(_('O numero maximo de 10 (dez) condutores foi ultrapassado'))

    def _mmodr_validar_veiculo_tracao(self, cr, user, mmodr, context=None):
        """Validacoes do Modal (Rodoviario)"""
        context = context and context.copy() or {}
        if not mmodr.mmodr_veiculo_id.id:
            self._erro.append(_('A Placa do veiculo de tracao nao foi informada'))
        if not mmodr.veivei_uf_id.id:
            self._erro.append(_('A UF do veiculo de tracao nao foi informada'))
        if not mmodr.veivei_renavam:
            self._info.append(_('Se desejar, voce pode informar o numero do RENAVAN cadastrado '
                                'para o veiculo de tracao'))
        if not mmodr.veivei_tara:
            self._erro.append(_('A tara do veiculo de tracao deve ser informada'))
        if not mmodr.veivei_tipo_carroceria:
            self._erro.append(_('O tipo de carroceria para o veiculo de tracao deve ser informado'))
        if not mmodr.veivei_tipo_rodado:
            self._erro.append(_('O tipo de rodado para o veiculo de tracao deve ser informado'))
        self._mmodr_validar_proprietario(cr, user, mmodr, context)
        self._mmodr_validar_condutor(cr, user, mmodr, context)

    def _mmodr_validar_reboque(self, cr, user, mmodr, context=None):
        """Validacao das informacoes do reboque"""
        if len(mmodr.mmodr_reboque_ids) > 3:
            self._warn.append(_('O numero maximo de 3 (tres) reboques foi ultrapassado'))

    def _mmodr_validar_vale_pedagio(self, cr, user, mmodr, context=None):
        """Validacao do vale-pedagio"""
        if filter(lambda f: not f.mdfevp_empresa_fornecedora_id.cpf_cnpj,
                  mmodr.mmodr_vale_pedagio_ids):
            self._erro.append(_('O CNPJ da empresa fornecedora do Vale-Pedagio deve ser informado'))
        if filter(lambda f: not f.mdfevp_responsavel_pagamento_id.cpf_cnpj,
                  mmodr.mmodr_vale_pedagio_ids):
            self._erro.append(_('O CPF ou CNPJ do responsavel pelo pagamento do Vale-Pedagio deve '
                                'ser informado'))
        if filter(lambda f: not f.mdfevp_numero_comprovante_compra, mmodr.mmodr_vale_pedagio_ids):
            self._erro.append(_('O numero do comprovante de compra do Vale-Pedagio deve ser '
                                'informado'))
        if filter(lambda f: not f.mdfevp_valor or f.mdfevp_valor < 0, mmodr.mmodr_vale_pedagio_ids):
            self._warn.append(_('O valor do Vale-Pedagio deve ser informado ou maior que zero'))

    def _mmodr_validar_local_carregamento(self, cr, user, mmodr, context=None):
        """Validacao do local de carregamento"""
        if not mmodr.mmodr_uf_carrega_id.id:
            self._erro.append(_('A UF do local de carregamento deve ser informada'))
        if mmodr.mmodr_municipio_carrega_ids:
            if len(mmodr.mmodr_municipio_carrega_ids) > len(set(
                    [m.codigo_ibge for m in mmodr.mmodr_municipio_carrega_ids])):
                self._warn.append(_('Existem municipios com o mesmo Codigo IBGE na lista de '
                                    'de municipios do local de CARREGAMENTO'))
            if filter(lambda f: f.uf_id.id != mmodr.mmodr_uf_carrega_id.id,
                      mmodr.mmodr_municipio_carrega_ids):
                self._warn.append(_('Existem municipios na lista de municipios do local de '
                                    'CARREGAMENTO que nao pertencem a UF de carregamento'))
            if filter(lambda f: not f.codigo_ibge, mmodr.mmodr_municipio_carrega_ids):
                self._warn.append(_('Existem municipios na lista de municipios do local de '
                                    'CARREGAMENTO sem o Codigo IBGE'))
            if len(mmodr.mmodr_municipio_carrega_ids) > 50:
                self._warn.append(_('O numero maximo de 50 (cinquenta) municipios foi ultrapassado '
                                    'na lista de municipios do local de CARREGAMENTO'))
        else:
            self._warn.append(_('Um municipio de CARREGAMENTO deve ser informado'))

    def _mmodr_validar_local_descarregamento(self, cr, user, mmodr, context=None):
        """Validacao do local de descarregamento"""
        if not mmodr.mmodr_uf_descarregamento_id.id:
            self._erro.append(_('A UF do local de descarregamento deve ser informada'))
        if mmodr.mmodr_municipio_descarrega_ids:
            if len(mmodr.mmodr_municipio_descarrega_ids) > len(set(
                    [m.mdfemd_municipio_id.codigo_ibge
                     for m in mmodr.mmodr_municipio_descarrega_ids])):
                self._warn.append(_('Existem municipios com o mesmo Codigo IBGE na lista de '
                                    'de municipios do local de DESCARREGAMENTO'))
            if filter(lambda f: f.mdfemd_uf_id.id != mmodr.mmodr_uf_descarregamento_id.id,
                      mmodr.mmodr_municipio_descarrega_ids):
                self._warn.append(_('Existem municipios na lista de municipios do local de '
                                    'DESCARREGAMENTO que nao pertencem a UF de descarregamento'))
            if filter(lambda f: not f.mdfemd_municipio_id.codigo_ibge,
                      mmodr.mmodr_municipio_descarrega_ids):
                self._warn.append(_('Existem municipios na lista de municipios do local de '
                                    'DESCARREGAMENTO sem o Codigo IBGE'))
            if len(mmodr.mmodr_municipio_descarrega_ids) > 100:
                self._warn.append(_('O numero maximo de 100 (cem) municipios foi ultrapassado '
                                    'na lista de municipios do local de DESCARREGAMENTO'))
            if map(lambda m: m, mmodr.mmodr_municipio_descarrega_ids):
                pass
        else:
            self._warn.append(_('Um municipio de DESCARREGAMENTO deve ser informado'))

    def _mmodr_validar_percurso(self, cr, user, mmodr, context=None):
        """Validacao das informacoes do percurso"""
        context = context and context.copy() or {}
        if any([f.mdfru_uf_id.id == mmodr.mmodr_uf_carrega_id.id for f in mmodr.mmodr_rel_uf_ids]):
            self._erro.append(_('UF do percurso ja informada na UF de CARREGAMENTO'))
        if any([f.mdfru_uf_id.id == mmodr.mmodr_uf_descarregamento_id.id for f in mmodr.mmodr_rel_uf_ids]):
            self._erro.append(_('UF do percurso ja informada na UF de DESCARREGAMENTO'))
        if len(mmodr.mmodr_rel_uf_ids) > 25:
            self._warn.append(_('O numero maximo de 25 (vinte e cinco) UFs de percurso foi '
                                'ultrapassado'))

    def _mmodr_validar_carregamento(self, cr, user, mmodr, context=None):
        """Validacao do carregamento"""
        context = context and context.copy() or {}
        self._mmodr_validar_local_carregamento(cr, user, mmodr, context)
        self._mmodr_validar_percurso(cr, user, mmodr, context)
        self._mmodr_validar_local_descarregamento(cr, user, mmodr, context)

    def _mmodr_validar_rodoviario(self, cr, user, mmodr, context=None):
        """Validacao das informacoes rodoviarias"""
        context = context and context.copy() or {}
        if mmodr.mmodr_rntrc:
            if len(mmodr.mmodr_rntrc) != 8:
                self._warn.append(_('O numero RNTRC (Rodoviario) deve conter sempre 8 caracteres.'))
        self._mmodr_validar_veiculo_tracao(cr, user, mmodr, context)
        self._mmodr_validar_reboque(cr, user, mmodr, context)
        self._mmodr_validar_vale_pedagio(cr, user, mmodr, context)

    def _mmodr_validar_modal_rodoviario(self, cr, user, mmodr, context=None):
        """Valicacao do Modal Rodoviario"""
        self._mmodr_validar_carregamento(cr, user, mmodr, context)
        self._mmodr_validar_rodoviario(cr, user, mmodr, context)

    def mmodr_button_validar(self, cr, user, ids, context=None):
        """Validacao do Modal Rodoviario"""
        context = context and context.copy() or {}
        self._erro, self._warn, self._info = [], [], []  # Inicializa valores para validacoes
        self.mdfe_validar(cr, user, ids[0], context)  # Validacoes gerais do Manifesto
        _mmodr = self.browse(cr, user, ids[0], context)
        self._mmodr_validar_modal_rodoviario(cr, user, _mmodr, context)  # Validacoes do Modal
        if self._erro:
            raise osv.except_osv(_('Atencao!'), self._erro.pop())  # <--.  Para o Manifesto, ambos
        elif self._warn:                                           #     > sao considerados erro!
            raise osv.except_osv(_('Aviso.'), self._warn.pop())    # <--´
        else:
            self._info.append(_('MDF-e validado.'))
            self.write(cr, user, ids[0], {
                'state': '2', 'mdfe_chave_acesso': self.mdfe_get_chave_acesso(
                    cr, user, ids[0], context)
            }, context)

    def mmodr_button_autorizar(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        _mdfe = self.mmodr_gerar(cr, user, ids[0], context)
        values = {}
        try:
            _mdfe.infMDFe.infModal.modalRodo.validar()
            _mdfe.validar()
            process = self.mmodr_autorizar_mdfe(cr, user, ids, _mdfe, context)
            if process.resposta.protMDFe.infProt.nProt.valor:
                values['mdfe_xml'] = _mdfe.xml
                values['mdfe_protocolo_autorizacao'] = process.resposta.protMDFe.infProt.nProt.valor
                values['mdfe_xml_autorizacao'] = process.resposta.protMDFe.xml
                values['state'] = '3'  # Autorizado
            else:
                values['state'] = 'R'  # Rejeitado
            values['mdfe_status_sefaz'] = '%s-%s' % (
                process.resposta.protMDFe.infProt.cStat.valor,
                process.resposta.protMDFe.infProt.xMotivo.valor)
        except Exception as error:
            raise osv.except_osv(
                _('Atencao!'),
                _('Nao foi possivel autorizar o Manifesto. Houve um erro durante o processo.\n\n'
                  '---\n' + error.__str__()))
        finally:
            self.write(cr, user, ids, values, context)

    def mmodr_autorizar_mdfe(self, cr, user, ids, mdfe, context=None):
        context = context and context.copy() or {}
        process = self.mmodr_enviar_lote(cr, user, ids[0], mdfe, context)
        if process.resposta.cStat.valor == u'103':
            tentativa = 5
            while tentativa:
                time.sleep(1)  # MOC v-6.0 sugere 15s no minimo
                process = self.mmodr_consultar_recibo(
                    cr, user, ids[0], process.resposta.cUF.valor,
                    process.resposta.infRec.nRec.valor, context)
                if process.resposta.cStat.valor == u'105':  # Lote em processamento
                    tentativa -= 1
                    continue
                break
        dhrecbto = process.resposta.protMDFe.infProt.dhRecbto.valor
        dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(cr, user,
            datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
            'America/Sao_Paulo', 'UTC')
        self.pool.get('fiscal.evento').create(cr, user, {
            'fiseve_xml_env': process.envio.xml,
            'fiseve_xml_ret': process.resposta.xml,
            'fiseve_tipo': '00110100',  # Autorizacao do MDF-e
            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'fiseve_status': process.resposta.protMDFe.infProt.cStat.valor,
            'fiseve_motivo': process.resposta.protMDFe.infProt.xMotivo.valor,
            'fiseve_lote': process.resposta.nRec.valor,
            'fiseve_protocolo': process.resposta.protMDFe.infProt.nProt.valor,
            'mdfe_rodo_id': ids[0],
        })

        return process

    def mmodr_enviar_lote(self, cr, user, mdfe_id, lote, context):
        context = context and context.copy() or {}
        cert = self.parametro(cr, user).fiscal.certificado_conteudo
        key = self.parametro(cr, user).fiscal.chave_conteudo
        ambient = int(self.parametro(cr, user).fiscal.fispar_mdfe_ambiente)
        mdfe = lote[0] if isinstance(lote, (list, tuple)) else lote  # Assume apenas um MDF-e
        state = int(mdfe.infMDFe.Id.valor[4:6])
        process = ProcessadorMDFe(cert, key, ambient, state).enviar_lote(lote)
        dhrecbto = process.resposta.infRec.dhRecbto.valor
        dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(cr, user,
            datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
            'America/Sao_Paulo', 'UTC')
        self.pool.get('fiscal.evento').create(cr, user, {
            'fiseve_xml_env': process.envio.xml,
            'fiseve_xml_ret': process.resposta.xml,
            'fiseve_tipo': '00999102',  # Enviar lote
            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'fiseve_status': process.resposta.cStat.valor,
            'fiseve_motivo': process.resposta.xMotivo.valor,
            'fiseve_lote': process.resposta.infRec and process.resposta.infRec.nRec.valor,
            'fiseve_protocolo': False,  # No envio do lote ainda nao se tem o protocolo
            'mdfe_rodo_id': mdfe_id,
        })

        return process

    def mmodr_consultar_recibo(self, cr, user, mdfe_id, uf, recibo, context=None):
        context = context and context.copy() or {}
        cert = self.parametro(cr, user).fiscal.certificado_conteudo
        key = self.parametro(cr, user).fiscal.chave_conteudo
        ambient = int(self.parametro(cr, user).fiscal.fispar_mdfe_ambiente)
        process = ProcessadorMDFe(cert, key, ambient, uf).consultar_recibo(recibo)
        dhrecbto = process.resposta.protMDFe.infProt.dhRecbto.valor
        dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(cr, user,
            datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
            'America/Sao_Paulo', 'UTC')
        self.pool.get('fiscal.evento').create(cr, user, {
            'fiseve_xml_env': process.envio.xml,
            'fiseve_xml_ret': process.resposta.xml,
            'fiseve_tipo': '00999103',  # Consultar Manifesto por lote/recibo
            'fiseve_data': dhrecbto,
            'fiseve_status': process.resposta.cStat.valor,
            'fiseve_motivo': process.resposta.xMotivo.valor,
            'fiseve_lote': process.resposta.nRec.valor,
            'fiseve_protocolo': process.resposta.protMDFe.infProt.nProt.valor,
            'mdfe_rodo_id': mdfe_id,
        })

        return process

    def mmodr_button_esboco(self, cr, user, ids, context=None):
        context = context and context.copy() or {}

        return self.write(cr, user, ids, {'state': '1'}, context)

    def mmodr_button_cancelar(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        res_id = self.pool.get('mdfe.cancelamento.osv').create(
            cr, user, {'mdfe_id': ids[0], 'modal': 'rodo'}, context)
        context = context and context.copy() or {}
        ref = self.pool.get('ir.model.data').get_object_reference(
             cr, user, 'mdfe', 'view_mdfe_cancelamento_osv_form')
        view_id = ref and ref[1] or False
        context['rodo'] = True  # Habilita botao para cancelamento de Modal Rodoviario
        return {
            'name': _('Cancelamento de MDF-e'),
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': view_id,
            'res_id': res_id,
            'target': 'new',
            'res_model': 'mdfe.cancelamento.osv',
            'type': 'ir.actions.act_window',
            'context': context}

    def mmodr_cancelar_mdfe(self, cr, user, chave, protocolo, motivo, context):
        context = context and context.copy() or {}
        cert = self.parametro(cr, user).fiscal.certificado_conteudo
        key = self.parametro(cr, user).fiscal.chave_conteudo
        ambient = int(self.parametro(cr, user).fiscal.fispar_mdfe_ambiente)
        state = int(chave[:2])

        return ProcessadorMDFe(cert, key, ambient, state).cancelar_mdfe(
            chave, protocolo, motivo)

    def mmodr_button_consultar_mdfe(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        key = self.read(cr, user, ids[0], ['mdfe_chave_acesso'], context)['mdfe_chave_acesso']
        process = self.mmodr_consultar_mdfe(cr, user, key, context)
        dhrecbto = process.resposta.protMDFe.infProt.dhRecbto.valor
        dhrecbto = dhrecbto and self.pool.get('adm.utils').admutl_converte_tz(cr, user,
            datetime.strptime(dhrecbto.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
            'America/Sao_Paulo', 'UTC')
        self.pool.get('fiscal.evento').create(cr, user, {
            'fiseve_xml_env': process.envio.xml,
            'fiseve_xml_ret': process.resposta.xml,
            'fiseve_tipo': '00999101',  # Consultar MDF-e por chave de acesso
            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'fiseve_status': process.resposta.cStat.valor,
            'fiseve_motivo': process.resposta.xMotivo.valor,
            'fiseve_lote': False,
            'fiseve_protocolo': process.resposta.protMDFe.infProt.nProt.valor,
            'mdfe_rodo_id': ids[0],
        })

    def mmodr_consultar_mdfe(self, cr, user, chave, context=None):
        context = context and context.copy() or {}
        cert = self.parametro(cr, user).fiscal.certificado_conteudo
        key = self.parametro(cr, user).fiscal.chave_conteudo
        ambient = int(self.parametro(cr, user).fiscal.fispar_mdfe_ambiente)
        state = int(chave[:2])

        return ProcessadorMDFe(cert, key, ambient, state).consultar_mdfe(chave)

    def mmodr_button_encerrar(self, cr, user, ids, context=None):
        from pynfe.xml_sped.__init__ import UF_CODIGO
        context = context and context.copy() or {}
        mdfe_dados = self.read(cr, user, ids[0], [
            'mdfe_chave_acesso', 'mdfe_protocolo_autorizacao', 'mmodr_uf_descarregamento_id',
            'mmodr_municipio_descarrega_ids'], context, '_classic_write')
        prot = mdfe_dados['mdfe_protocolo_autorizacao']
        key = mdfe_dados['mdfe_chave_acesso']
        uf = UF_CODIGO[self.pool.get('adm.uf').read(
            cr, user, mdfe_dados['mmodr_uf_descarregamento_id'], ['sigla'], context)['sigla']]
        mun_id = mdfe_dados['mmodr_municipio_descarrega_ids'].pop()
        municipio_dados = self.pool.get('mdfe.municipio.descarregamento').read(
            cr, user, mun_id, ['mdfemd_municipio_id'], context, '_classic_write')
        mun_id = municipio_dados and municipio_dados['mdfemd_municipio_id']
        codigo_ibge = mun_id and self.pool.get('adm.cidade').read(
            cr, user, mun_id, ['codigo_ibge'], context)['codigo_ibge']
        process = self.mmodr_encerrar_mdfe(cr, user, key, prot, uf, codigo_ibge, context)
        dhregevento = process.resposta.infEvento.dhRegEvento.valor
        dhregevento = dhregevento and self.pool.get('adm.utils').admutl_converte_tz(
            cr, user, datetime.strptime(dhregevento.__str__()[:19], '%Y-%m-%d %H:%M:%S'),
            'America/Sao_Paulo', 'UTC')
        self.pool.get('fiscal.evento').create(cr, user, {
            'fiseve_xml_env': process.envio.xml,
            'fiseve_xml_ret': process.resposta.xml,
            'fiseve_tipo': '58110112',  # Encerramento do Manifesto
            'fiseve_data': dhregevento or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'fiseve_status': process.resposta.infEvento.cStat.valor,
            'fiseve_motivo': process.resposta.infEvento.xMotivo.valor,
            'fiseve_lote': False,
            'fiseve_protocolo': process.resposta.infEvento.nProt.valor,
            'mdfe_rodo_id': ids[0],
        })
        if process.resposta.infEvento.cStat.valor == u'135':
            self.write(cr, user, ids, {
                'mmodr_protocolo_encerramento': process.resposta.infEvento.nProt.valor,
                'mmodr_xml_encerramento': process.resposta.infEvento.xml,
                'mdfe_status_sefaz': '%s-%s' % (process.resposta.infEvento.cStat.valor,
                                                process.resposta.infEvento.xMotivo.valor),
                'state': 'E'
            }, context)
        return True

    def mmodr_encerrar_mdfe(self, cr, user, chave, protocolo, uf, mun, context=None):
        context = context and context.copy() or {}
        cert = self.parametro(cr, user).fiscal.certificado_conteudo
        key = self.parametro(cr, user).fiscal.chave_conteudo
        ambient = int(self.parametro(cr, user).fiscal.fispar_mdfe_ambiente)
        state = int(chave[:2])

        return ProcessadorMDFe(cert, key, ambient, state).encerrar_mdfe(chave, protocolo, uf, mun)

    def mmodr_button_exportar_xml(self, cr, user, ids, context=None):
        from pynfe.mdfe.manual_300.procMDFe import ProcMDFe
        context = context and context.copy() or {}
        procmdfe = ProcMDFe()
        mdfe_dados = self.read(
            cr, user, ids[0], ['mdfe_xml', 'mdfe_xml_autorizacao'], context)
        procmdfe.mdfe.xml = mdfe_dados['mdfe_xml'].encode('utf-8')
        procmdfe.protMDFe.xml = mdfe_dados['mdfe_xml_autorizacao']
        res_id = self.pool.get('mdfe.damdfe.osv').create(cr, user, {
            'name': procmdfe.mdfe.infMDFe.Id.valor[4:] + '.xml',
            'report': base64.encodestring(procmdfe.xml)}, context)

        return {'name': _('Exportar XML do MDF-e'), 'view_type': 'form',
                'view_mode': 'form', 'res_id': res_id, 'res_model': 'mdfe.damdfe.osv',
                'view_id': False, 'target': 'new', 'type': 'ir.actions.act_window'}

    def mmodr_button_damdfe(self, cr, user, ids, context=None):
        from pynfe.mdfe.manual_300.procMDFe import ProcMDFe
        context = context and context.copy() or {}

        def get_logo_damdfe(image):
            from PIL import Image
            import StringIO

            logo = base64.decodestring(image)
            logo = StringIO.StringIO(logo)
            logo.seek(0)
            max_size = 1920, 1920
            im = Image.open(logo)
            im = im.convert('P', palette=Image.WEB).convert('RGB')
            nova_logo = StringIO.StringIO()
            im.resize(max_size, Image.BILINEAR)
            im.save(nova_logo, format='JPEG')
            nova_logo.seek(0)

            return nova_logo

        company_id = self.read(
            cr, user, ids[0], ['company_id'], context, '_classic_write')['company_id']
        logo = get_logo_damdfe(self.pool.get('res.company').read(
            cr, user, company_id, ['logo'], context)['logo'])
        procmdfe = ProcMDFe()
        mdfe_dados = self.read(
            cr, user, ids[0], ['mdfe_xml', 'mdfe_xml_autorizacao'], context)
        procmdfe.mdfe.xml = mdfe_dados['mdfe_xml'].encode('utf-8')
        procmdfe.protMDFe.xml = mdfe_dados['mdfe_xml_autorizacao']
        res_id = self._mmodr_gerar_danfe(cr, user, [procmdfe], logo, context)

        return {'name': _('DAMDFE - Documento Auxiliar de MDF-e'), 'view_type': 'form',
                'view_mode': 'form', 'res_id': res_id, 'res_model': 'mdfe.damdfe.osv',
                'view_id': False, 'target': 'new', 'type': 'ir.actions.act_window'}

    def _mmodr_gerar_danfe(self, cr, user, procmdfe, logo, context=None):
        context = context and context.copy() or {}
        from geraldo.generators import PDFGenerator
        from StringIO import StringIO
        damdfe = DAMDFE(procmdfe, logo)
        pdf = StringIO()
        damdfe.generate_by(PDFGenerator, filename=pdf)
        damdfe = pdf.getvalue()
        pdf.close()

        return self.pool.get('mdfe.damdfe.osv').create(cr, user, {
            'name': procmdfe[0].mdfe.infMDFe.Id.valor[4:] + '.pdf',
            'report': base64.encodestring(damdfe)}, context)

    def _mmodr_tag_ide(self, cr, user, mmodr, obj, context=None):
        """Identificacao do MDF-e - infMDFe.ide"""
        context = context and context.copy() or {}
        obj.infMDFe.ide.ufIni.valor = mmodr.mmodr_uf_carrega_id.sigla
        obj.infMDFe.ide.ufFim.valor = mmodr.mmodr_uf_descarregamento_id.sigla

        # Carregamento   /// Todo: transformar em metodo
        for carrega in mmodr.mmodr_municipio_carrega_ids[:50]:
            infMunCarrega = InfMunCarrega()
            infMunCarrega.xMunCarrega.valor = carrega.name
            infMunCarrega.cMunCarrega.valor = carrega.codigo_ibge.__str__()
            obj.infMDFe.ide.infMunCarrega.append(infMunCarrega)

        # Percurso (opcional) /// Todo: transformar em metodo
        for percurso in sorted(mmodr.mmodr_rel_uf_ids[:25], key=lambda k: k.sequence):
            infPercurso = InfPercurso()
            infPercurso.ufPer.valor = percurso.mdfru_uf_id.sigla
            obj.infMDFe.ide.infPercurso.append(infPercurso)

    def _mmodr_tag_infdoc(self, cr, user, mmodr, obj, context=None):
        """Informacoes dos Documentos Fiscais vinculados ao Manifesto"""
        context = context and context.copy() or {}

        # Descarregamento   /// Todo: transformar em metodo
        for descarga in mmodr.mmodr_municipio_descarrega_ids[:100]:
            infMunDescarga = InfMunDescarga()
            infMunDescarga.cMunDescarga.valor = descarga.mdfemd_municipio_id.codigo_ibge.__str__()
            infMunDescarga.xMunDescarga.valor = descarga.mdfemd_municipio_id.name
            for nfe in descarga.mdfemd_nfe_referencia_ids:
                infNFe = InfNFe()
                infNFe.chNFe.valor = nfe.nfe_chave_acesso
                infMunDescarga.infNFe.append(infNFe)
            obj.infMDFe.infDoc.infMunDescarga.append(infMunDescarga)

    def _mmodr_tag_tot(self, cr, user, mmodr, obj, context=None):
        """Totalizadores da carga transportada e seus documentos fiscais"""
        context = context and context.copy() or {}
        obj.infMDFe.tot.qCTe.valor = mmodr.mmodr_qtde_total_cte  # Somente tpTransp = 1
        obj.infMDFe.tot.qNFe.valor = mmodr.mmodr_qtde_total_nfe
        obj.infMDFe.tot.qMDFe.valor = 0  # Somente modalidade aquaviario
        obj.infMDFe.tot.vCarga.valor = '%.2f' % mmodr.mmodr_valor_total
        obj.infMDFe.tot.cUnid.valor = mmodr.mmodr_unidade_medida_id.unidade == 'KG' and '01' or '02'
        obj.infMDFe.tot.qCarga.valor = '%.4f' % mmodr.mmodr_peso_bruto_total

    def _mmodr_tag_lacres(self, cr, user, mmodr, obj, context=None):
        """Lacres do MDF-e"""
        context = context and context.copy() or {}
        for l in mmodr.mmodr_lacre_ids:
            lacre = Lacre()
            lacre.nLacre.valor = l.mdfel_numero
            obj.infMDFe.lacres.append(lacre)

    def _mmodr_tag_autxml(self, cr, user, mmodr, obj, context=None):
        """Autorizados para download do XML do DF-e"""
        context = context and context.copy() or {}
        autorizado = AutXML()
        for a in mmodr.mmodr_autorizado_download_xml_ids:
            if a.pessoa_tipo == 'J':
                autorizado.cnpj.valor = re.sub('[^0-9]', '', a.cpf_cnpj)
            else:
                autorizado.cpf.valor = re.sub('[^0-9]', '', a.cpf_cnpj)
            obj.infMDFe.autXML.append(autorizado)

    def _mmodr_tag_valeped(self, cr, user, mmodr, obj, context=None):
        """Informacoes de Vale Pedagio"""
        context = context and context.copy() or {}
        for v in mmodr.mmodr_vale_pedagio_ids:
            disp = Disp()
            disp.cnpjForn.valor = re.sub('[^0-9]', '', v.mdfevp_empresa_fornecedora_id.cpf_cnpj)
            if v.mdfevp_responsavel_pagamento_id.pessoa_tipo == 'J':
                disp.cnpjPg.valor = re.sub('[^0-9]', '', v.mdfevp_responsavel_pagamento_id.cpf_cnpj)
            else:
                disp.cpfPg.valor = re.sub('[^0-9]', '', v.mdfevp_responsavel_pagamento_id.cpf_cnpj)
            disp.nCompra.valor = v.mdfevp_numero_comprovante_compra
            disp.vValePed.valor = '%.2f' % v.mdfevp_valor
            obj.infMDFe.infModal.modalRodo.infANTT.valePed.disp.append(disp)

    def _mmodr_tag_infantt(self, cr, user, mmodr, obj, context=None):
        """Grupo de informacoes para agencia reguladora"""
        context = context and context.copy() or {}
        obj.infMDFe.infModal.modalRodo.infANTT.rntrc.valor = mmodr.mmodr_rntrc
        self._mmodr_tag_valeped(cr, user, mmodr, obj, context)

    def _mmodr_tag_prop(self, cr, user, mmodr, obj, context=None):
        """Proprietario do veiculo. Somente quando veiculo NAO pertencer ao emitente"""
        context = context and context.copy() or {}
        if not mmodr.mmodr_veiculo_proprietario:
            return
        proprietario = mmodr.mmodr_veiculo_id.veivei_pessoa_id
        endereco = proprietario.admpes_endereco
        if proprietario.pessoa_tipo == 'J':
            obj.infMDFe.infModal.modalRodo.veicTracao.prop.cnpj.valor = \
                re.sub('[^0-9]', '', proprietario.cpf_cnpj)
        else:
            obj.infMDFe.infModal.modalRodo.veicTracao.prop.cpf.valor = \
                re.sub('[^0-9]', '', proprietario.cpf_cnpj)
        obj.infMDFe.infModal.modalRodo.veicTracao.prop.rntrc.valor = proprietario.admpes_rntrc
        obj.infMDFe.infModal.modalRodo.veicTracao.prop.xNome.valor = proprietario.name
        obj.infMDFe.infModal.modalRodo.veicTracao.prop.ie.valor = proprietario.admpes_jur_ie
        obj.infMDFe.infModal.modalRodo.veicTracao.prop.uf.valor = endereco.uf_id.sigla
        obj.infMDFe.infModal.modalRodo.veicTracao.prop.tpProp.valor = \
            proprietario.admpes_tipo_proprietario

    def _mmodr_tag_veictracao_condutor(self, cr, user, mmodr, obj, context=None):
        """Informacoes do condutor do veiculo"""
        context = context and context.copy() or {}
        for condutor in mmodr.mmodr_veiculo_id.veivei_condutor_ids[:10]:
            motorista = Condutor()
            motorista.xNome.valor = condutor.pessoa_id.name
            motorista.cpf.valor = re.sub('[^0-9]', '', condutor.pessoa_id.cpf_cnpj)
            obj.infMDFe.infModal.modalRodo.veicTracao.condutor.append(motorista)

    def _mmodr_tag_veictracao(self, cr, user, mmodr, obj, context=None):
        """Dados do veiculo de tracao"""
        veiculo = mmodr.mmodr_veiculo_id
        obj.infMDFe.infModal.modalRodo.veicTracao.cInt.valor = str(veiculo.id).zfill(10)
        obj.infMDFe.infModal.modalRodo.veicTracao.placa.valor = veiculo.placa
        obj.infMDFe.infModal.modalRodo.veicTracao.tara.valor = int(veiculo.veivei_tara)
        obj.infMDFe.infModal.modalRodo.veicTracao.capKg.valor = int(veiculo.carga_maxima_peso)
        obj.infMDFe.infModal.modalRodo.veicTracao.capM3.valor = int(veiculo.veivei_capacidade_m3)
        if mmodr.mmodr_veiculo_proprietario:
            self._mmodr_tag_prop(cr, user, mmodr, obj, context)
        self._mmodr_tag_veictracao_condutor(cr, user, mmodr, obj, context)
        obj.infMDFe.infModal.modalRodo.veicTracao.tpRod.valor = veiculo.veivei_tipo_rodado
        obj.infMDFe.infModal.modalRodo.veicTracao.tpCar.valor = veiculo.veivei_tipo_carroceria
        obj.infMDFe.infModal.modalRodo.veicTracao.uf.valor = veiculo.veivei_uf_id.sigla

    def _mmodr_tag_veicreboque(self, cr, user, mmodr, obj, context=None):
        """Dados do reboque """
        context = context and context.copy() or {}
        reboque = VeicReboque()
        for veiculo in mmodr.mmodr_reboque_ids[:3]:
            reboque.cInt.valor = str(veiculo.id).zfill(10)
            reboque.placa.valor = veiculo.placa
            reboque.renavan.valor = veiculo.veivei_renavam
            reboque.tara.valor = int(veiculo.veivei_tara)
            reboque.capKg.valor = int(veiculo.carga_maxima_peso)
            reboque.capM3.valor = int(veiculo.veivei_capacidade_m3)
            if veiculo.veivei_pessoa_id:  # O emitente nao eh o proprietario do reboque
                if veiculo.veivei_pessoa_id.pessoa_tipo == 'J':
                    reboque.prop.cnpj.valor = \
                        re.sub('[^0-9]', '', veiculo.veivei_pessoa_id.cpf_cnpj)
                else:
                    reboque.prop.cpf.valor = re.sub('[^0-9]', '', veiculo.veivei_pessoa_id.cpf_cnpj)
                reboque.prop.rntrc.valor = veiculo.veivei_pessoa_id.admpes_rntrc
                reboque.prop.xNome.valor = veiculo.veivei_pessoa_id.name
                reboque.prop.ie.valor = veiculo.veivei_pessoa_id.admpes_jur_ie
                reboque.prop.uf.valor = veiculo.veivei_pessoa_id.admpes_endereco.uf_id.sigla
                reboque.prop.tpProp.valor = veiculo.veivei_pessoa_id.admpes_tipo_proprietario
            reboque.tpCar.valor = veiculo.veivei_tipo_carroceria
            reboque.uf.valor = veiculo.veivei_uf_id.sigla

    def _mmodr_tag_infmodal(self, cr, user, mmodr, obj, context=None):
        """Informacoes do modal - Modalidade Rodoviario"""
        context = context and context.copy() or {}
        modal = ModalRodo()
        self._mmodr_tag_infantt(cr, user, mmodr, obj, context)
        self._mmodr_tag_veictracao(cr, user, mmodr, obj, context)
        self._mmodr_tag_veicreboque(cr, user, mmodr, obj, context)
        modal.codAgPorto.valor = mmodr.mmodr_cod_agendamento_porto
        # self._mmodr_tag_lacrodo(cr, user, mmodr, obj, context)  # Opcional - Nao implementado

    def _mmodr_tag_infmdfe(self, cr, user, mmodr, obj, context=None):
        """Informacoes do MDF-e - InfMDFe"""
        self._mmodr_tag_ide(cr, user, mmodr, obj, context)
        self._mmodr_tag_infdoc(cr, user, mmodr, obj, context)
        self._mmodr_tag_tot(cr, user, mmodr, obj, context)
        self._mmodr_tag_lacres(cr, user, mmodr, obj, context)
        self._mmodr_tag_autxml(cr, user, mmodr, obj, context)

    def mmodr_gerar(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        _ids = isinstance(ids, (list, tuple)) and ids[:] or [ids]
        result = {}.fromkeys(_ids, None)
        for mmodr in self.browse(cr, user, _ids, context):
            obj = mmodr.mdfe_gerar()[mmodr.id]
            self._mmodr_tag_infmdfe(cr, user, mmodr, obj, context)
            self._mmodr_tag_infmodal(cr, user, mmodr, obj, context)
            result[mmodr.id] = obj

        return isinstance(ids, (list, tuple)) and result or result[ids]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
