# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import fields
from openerp.osv import osv


class veiculos_veiculo(osv.Model):
    """
    Classe veiculos
    """
    def veivei_get_tipo_carroceria(self, cr, user, context=None):
        context = context and context.copy() or {}
        return [('00', 'Nao aplicavel'), ('01', 'Aberta'), ('02', 'Fechada/Bau'),
                ('03', 'Granelera'), ('04', 'Porta container'), ('05', 'Sider')]

    def veivei_get_tipo_rodado(self, cr, user, context=None):
        context = context and context.copy() or {}
        return [('01', 'Truck'), ('02', 'Toco'), ('03', 'Cavalo mecanico'), ('04', 'VAN'),
                ('05', 'Utilitario'), ('06', 'Outro')]

    def veivei_get_capacidade_m3(self, cr, user, ids, name, arg, context=None):
        """
        Calcula volume em funcao da altura, largura e comprimento do veiculo.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param name: nome do campo,
        :param arg: lista dos argumentos a serem utilizados,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: integer
        :type ids: list
        :type name: list
        :type arg: list
        :return: Capacidade de carga do veiculo em m3 (metros cubicos).
        :rtype: float
        """
        context = context and context.copy() or {}
        result = {}
        for veiculo in self.browse(
                cr, user, ids if isinstance(ids, (list, tuple)) else [ids], context):
            altura = veiculo.carga_altura or 0
            largura = veiculo.carga_largura or 0
            comprimento = veiculo.carga_comprimento or 0
            result[veiculo.id] = altura * largura * comprimento

        return result

    _inherit = 'veiculos.veiculo'  #veivei
    _columns = {
        'veivei_uf_id': fields.many2one(
            'adm.uf', 'UF', ondelete='set null', select=True,
            help='UF de licenciamento do veiculo.'),
        'veivei_tara': fields.float(
            'Tara', digits=(8, 2), help='Peso do veiculo vazio, sem considerar a carga.'),
        'veivei_tipo_carroceria': fields.selection(
            veivei_get_tipo_carroceria, 'Tipo de carroceria', size=2, oldname='carroceria_tipo',
            help=''),
        'veivei_tipo_rodado': fields.selection(
            veivei_get_tipo_rodado, string='Tipo rodado', size=2,
            help=''),
        'veivei_renavam': fields.char(
            'RENAVAM', size=11, help='Numero do Registro Nacional de Veiculos Automotores'),
        'veivei_reboque_id': fields.many2one(
            'veiculos.veiculo', 'Reboque', ondelete='set null'),
        'veivei_reboque_ids': fields.one2many(
            'veiculos.veiculo', 'veivei_reboque_id',
            help='Veiculo de carga, sem meio proprio de tracao destinado a ser engatado atras de '
                 'um veiculo automotor.'),
        'veivei_capacidade_m3': fields.function(
            veivei_get_capacidade_m3, method=True, digits=(15, 6), string='Volume (m3)',
            oldname='cubagem', help=''),
        'veivei_condutor_ids': fields.many2many(
            'veiculos.motorista', 'mdfe_condutor_m2m_veiculo_motorista', 'veiculo_id',
            'condutor_id', 'Condutor', help='')
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
