# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
# noinspection PyProtectedMember
from openerp.tools.translate import _
from openerp.osv import fields
from datetime import datetime
from openerp.osv import osv


# noinspection PyAbstractClass,PyPep8Naming,PyProtectedMember
class mdfe(osv.Model):
    """
    Classe Manifesto Eletronico de Documentos Fiscais
    """
    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.__dict__.update(entries)

    def parametro(self, cr, uid, company=False, context=None):
        context = context and context.copy() or {}
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in [
                               'financeiro', 'fiscal', 'estoque', 'adm', 'rh']])

        return self.ParamToObject(**parametros)

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def mdfe_get_state(self, cr, user, context=None):
        context = context and context.copy() or {}
        return [('1', _('Esboco')), ('2', _('Validado')), ('3', 'Autorizado'), ('C', 'Cancelado'),
                ('E', 'Encerrado'), ('R', 'Rejeitado')]

    # noinspection PyUnusedLocal
    def mdfe_get_descricao(self, cr, user, ids, name, args, context=None):
        context = context and context.copy() or {}
        result = {}
        for r in self.browse(cr, user, ids, context):
            if r.mdfe_numero:
                descricao = u'Modal Rodoviario n° %d/%s ' % (r.mdfe_numero, r.mdfe_serie)
                if r.mdfe_emissao_contingencia:
                    descricao += _('(Contingencia)')
                descricao = descricao.strip()
            else:
                descricao = _('Rodoviario (esboco)')
            result[r.id] = descricao

        return result

    def _get_default_mdfe_serie(self, cr, user, context=None):
        context = context and context.copy() or {}
        serie_id = self.parametro(cr, user).fiscal.fispar_mdfe_modelo_serie_id
        if not serie_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Modelo MDF-e nao esta cadastrada no parametro do fiscal'))

        return self.pool.get('fiscal.modelo.nf.series').read(cr, user, serie_id, ['serie_nf'],
                                                                                context)['serie_nf']

    _name = 'mdfe'  # Prefix: mdfe
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _rec_name = 'mdfe_descricao'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade', required=True),
        'mdfe_descricao': fields.function(mdfe_get_descricao, type='char', size=30),
        'mdfe_modelo_serie_id': fields.many2one(
           'fiscal.modelo.nf.series', 'Modelo', ondelete='restrict', readonly=True),
        'mdfe_numero': fields.integer('Numero', required=True, select=True),
        'mdfe_serie': fields.char('Serie', size=3, required=True),
        'mdfe_data_emissao': fields.datetime('Data de emissao', select=True),
        'mdfe_data_inicio_viagem': fields.datetime('Data de inicio de viagem'),
        'mdfe_chave_acesso': fields.char('Chave de acesso', size=44, select=True),
        'mdfe_emissao_contingencia': fields.boolean(
            'Contingencia', help='Se marcado, informa que a forma de autorizacao do MDF-e sera '
                                 'realizada em contingencia.'),
        'mdfe_tipo_emitente': fields.function(
            lambda s, c, u, i, n, a, x: dict([
                (i, s.parametro(c, u).fiscal.fispar_mdfe_tipo_emitente) for i in i]),
            type='char', size=1, string='Tipo de emitente'),
        'mdfe_informacao_fisco': fields.text('Informacoes adicionais do fisco', size=2000),
        'mdfe_informacao_contribuinte': fields.text('Informacoes do complementares', size=5000),
        'mdfe_protocolo_autorizacao': fields.char('Protocolo Autorizacao', size=15),
        'mdfe_protocolo_cancelamento': fields.char('Protocolo Cancelamento', size=15),
        'mdfe_status_sefaz': fields.char('Status', size=255),
        'mdfe_xml': fields.text('XML', help='XML do MDF-e'),
        'mdfe_xml_autorizacao': fields.text(
            'XML Autorizacao', help='XML do protocolo de autorizacao'),
        'mdfe_xml_cancelamento': fields.text(
            'XML Cancelamento', help='XML do protocolo de cancelamento'),
        'mdfe_evento_ids': fields.one2many('fiscal.evento', 'mdfe_rodo_id', 'Evento'),
        'state': fields.selection(mdfe_get_state, 'Status', select=True),
        'active': fields.boolean('Ativo')
    }
    _defaults = {
        'company_id': lambda s, c, u, x: s.pool.get('adm.empresa.objeto').getEmpresa(
            c, u, s._name, context=x),
        'mdfe_modelo_serie_id': lambda s, c, u, x: s.parametro(
            c, u).fiscal.fispar_mdfe_modelo_serie_id,
        'mdfe_serie': _get_default_mdfe_serie,
        'mdfe_tipo_emitente': lambda s, c, u, x: s.parametro(c, u).fiscal.fispar_mdfe_tipo_emitente,
        'mdfe_data_emissao': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
        'mdfe_emissao_contingencia': False,
        'state': '1',
        'active': True
    }

    def create(self, cr, user, vals, context=None):
        """Atribui a numeracao e serie do Manifesto"""
        context = context and context.copy() or {}
        numeracao = self.pool.get('fiscal.modelo.nf.series').read(
            cr, user, vals['mdfe_modelo_serie_id'], ['numero_nf', 'serie_nf'], context)
        vals['mdfe_numero'] = numeracao['numero_nf']
        vals['mdfe_serie'] = numeracao['serie_nf']
        self.pool.get('fiscal.modelo.nf.series').write(cr, user, vals['mdfe_modelo_serie_id'], {
                'numero_nf': vals['mdfe_numero'] + 1
        }, context)

        return super(mdfe, self).create(cr, user, vals, context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
