# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from funcoes_modal_rodoviario import _mmodr_total_fornecimento
from openerp.osv import fields
from openerp.osv import osv


class mdfe_modal_rodo(osv.Model):
    """
    Classe MDF-e - Modalidade Rodoviario
    """
    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.__dict__.update(entries)

    def parametro(self, cr, uid, company=False, context=None):
        context = context and context.copy() or {}
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in [
                               'financeiro', 'fiscal', 'estoque', 'adm', 'rh']])

        return self.ParamToObject(**parametros)

    _name = 'mdfe.modal.rodoviario'  # Prefix: mmodr
    _inherit = 'mdfe'
    _inherits = {
        'veiculos.veiculo': 'mmodr_veiculo_id'
    }
    _columns = {
        'mmodr_rntrc': fields.char(
            'RNTRC', size=8, help='Registro Nacional dos Transportadores de Cargas'),
        'mmodr_ciot': fields.char(
            'CIOT', size=12, help='Codigo Identificador da Operacao de Transporte'),
        'mmodr_cod_agendamento_porto': fields.char(
            'Codigo de agendamento do porto', size=16, help='Codigo de agendamento do porto'),

        'mmodr_uf_carrega_id': fields.many2one(
            'adm.uf', 'UF carregamento', ondelete='restrict',
            help='UF do local de carregamento'),
        'mmodr_municipio_carrega_ids': fields.many2many(
            'adm.cidade', 'mmodr_municipio_carrega_m2m_adm_cidade', 'id', 'municipio_id',
            'Municipio', help='Lista dos municipios de carregamento. O primeiro da lista sera a '
                              'origem do carregamento. (Max. 50)'),
        'mmodr_uf_descarregamento_id': fields.many2one(
            'adm.uf', 'UF descarregamento', ondelete='restrict',
            help='UF de descarregamento.'),
        'mmodr_municipio_descarrega_ids': fields.one2many(
            'mdfe.municipio.descarregamento', 'mdfemd_rodo_id', 'Municipio de descarregamento',
            ondelete='set null', help='Lista dos municipios de descarregamento. (Max. 100)'),
        'mmodr_veiculo_proprietario': fields.boolean(
            'Veiculo proprietario', help='Se marcado, informa que o proprietario do veiculo NAO e '
                                         'o mesmo emitente do MDF-e (ex.: TAC - Transportador '
                                         'Autonomo de Cargas)'),
        'mmodr_veiculo_id': fields.many2one(
            'veiculos.veiculo', string='Veiculo', required=True, ondelete='restrict',
            help='Veiculo de tracao. O veiculo utilizado no transporte da carga.'),
        'mmodr_reboque_proprietario': fields.boolean(
            'Reboque proprietario', help='Se marcado, informa que o proprietario do veiculo '
                                         '(reboque) NAO e o mesmo emitente do MDF-e.'),
        'mmodr_reboque_ids': fields.many2many(
            'veiculos.veiculo', 'mmodr_reboque_m2m_veiculos_veiculo', 'reboque_id',
            'veiculo_id', 'Reboque', help='Veiculo de carga, sem meio proprio de tracao destinado '
                                          'a ser engatado atras de um veiculo automotor.'),
        'mmodr_vale_pedagio_ids': fields.one2many(
            'mdfe.vale.pedagio', 'mmodr_id', 'Vale-Pedagio', ondelete='set null'),
        'mmodr_qtde_total_cte': fields.function(
            _mmodr_total_fornecimento, type='integer', string='Quantidade total CT-e', multi='sum',
            help='Quantidade total de CT-e relacionados no manifesto.'),
        'mmodr_qtde_total_nfe': fields.function(
            _mmodr_total_fornecimento, type='integer', string='Quantidade total NF-e', multi='sum',
            help='Quantidade total de NF-e relacionadas no manifesto.'),
        'mmodr_qtde_total_nf': fields.function(
           _mmodr_total_fornecimento, type='integer', string='Quantidade total NF', multi='sum',
            help='Quantidade total de notas fiscais (mod. 01 e A1) relacionadas no manifesto.'),
        'mmodr_valor_total': fields.function(
            _mmodr_total_fornecimento, digits=(10, 2), string='Valor total', multi='sum',
            help='Valor total da mercadoria transportada'),
        'mmodr_unidade_medida_id': fields.many2one(
            'estoque.embalagem.unidade', 'Unidade de medida', ondelete='set null'),
        'mmodr_peso_bruto_total': fields.function(
            _mmodr_total_fornecimento, type='integer', string='Peso bruto total', multi='sum'),
        'mmodr_lacre_ids': fields.one2many('mdfe.lacre', 'mdfel_modal_rodoviario_id', 'Lacre'),
        'mmodr_autorizado_download_xml_ids': fields.many2many(
            'adm.pessoa', 'mmodr_autorizado_download_xml_m2m_adm_pessoa', 'id', 'pessoa_id',
            'Autorizado para download do XML'),
        'mmodr_protocolo_encerramento': fields.char('Protocolo Encerramento', size=15),
        'mmodr_xml_encerramento': fields.text('XML do protocolo de encerramento'),
        'mmodr_rel_uf_ids': fields.one2many(
            'mdfe.rel.rodo.uf', 'mdfru_rodo_id', 'Municipio de Percurso',
            help='Lista das UFs de percurso da carga. Nao e necessaria a repeticao de UF do local '
                 'de carregamento.'
        ),
    }
    _defaults = {
        'mmodr_veiculo_proprietario': False,
        'mmodr_reboque_proprietario': False,
        'mmodr_unidade_medida_id': lambda s, c, u, x: s.pool.get(
            'estoque.embalagem.unidade').search(
            c, u, [('unidade', '=', 'KG')], context=x) and s.pool.get(
            'estoque.embalagem.unidade').search(c, u, [('unidade', '=', 'KG')], context=x)[0]
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
