# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from geraldo import Report, SubReport, SystemField, RoundRect
from geraldo.barcodes import BarCode
from base import *
import locale

locale.setlocale(locale.LC_ALL, 'pt_BR.utf8')
BG_TAG = HexColor(0xAFEEEE)


class DAMDFeRodo(Report):
    def __init__(self, queryset=None, logo=None, **kwargs):
        super(DAMDFeRodo, self).__init__(queryset)
        self.print_if_empty = True
        self.additional_fonts = FONTS
        self.page_size = kwargs.get('page_size') or PAGE_SIZE  # default = A5
        self.margin_top = MARGIN_TOP
        self.margin_bottom = MARGIN_BOTTOM
        self.margin_left = MARGIN_LEFT
        self.margin_right = MARGIN_RIGHT
        self.qrcode = queryset[0].mdfe.infMDFeSupl.qrCodMDFe.valor

        # Divisoes do DAMDFE
        contingency = self._check_contingency()
        self.band_page_header = Header(self.qrcode, logo, contingency)
        self.band_detail = Detail(contingency)
        self.band_page_footer = Footer()
        self.band_begin = Begin()
        self.band_summary = Summary()

    # Contingencia
    def _check_contingency(self):
        """Verifica se o MDF-e foi emitido em contingencia"""
        return self.queryset[0].mdfe.infMDFe.ide.tpEmis.valor == 2

    # noinspection PyProtectedMember
    def do_on_new_page(self, page, page_number, generator):
        if generator._current_page_number != 1:
            self.band_page_footer = generator._is_latest_page and Footer()


class BeginCondutor(BandDamdfe):
    height = .4 * cm

    def __init__(self):
        super(BeginCondutor, self).__init__()
        self.elements = [Condutor()]


class Begin(BandDamdfe):
    visible = True
    height = .4*cm
    # borders = dict(all=RoundRect(stroke_color=FUCHSIA))

    def __init__(self):
        super(Begin, self).__init__()
        self.add_label('Veículo', top=.4, left=0, width=1.2, style=COMPANY_NAME)
        self.elements.append(Line(top=1.6*cm, left=5*cm, right=2, bottom=1.6*cm))
        tag = self.add_tag('Placa', 'mdfe.infMDFe.infModal.modalRodo.veicTracao.placa.valor',
                           top=1.2, left=0, width=1.4, style=DEFAULT_BOLD)
        tag[1].style = DEFAULT
        tag = self.add_tag('RNTRC', 'mdfe.infMDFe.infModal.modalRodo.veicTracao.prop.rntrc.valor',
                           top=1.2, left=1.7, width=1.4, style=DEFAULT_BOLD)
        tag[1].style = DEFAULT
        self.child_bands = [BeginCondutor()]


class Summary(BandDamdfe):
    visible = True
    height = .4*cm
    # borders = dict(all=RoundRect(stroke_color=PURPLE))
    # auto_expand_height = True

    def __init__(self):
        super(Summary, self).__init__()
        self.add_label('Observação', top=1, left=0, width=3, style=COMPANY_NAME)
        self.elements.append(Line(top=1.5*cm, left=WIDTH_PORTRAIT, right=0, bottom=1.5*cm))
        self.add_field('mdfe.infMDFe.infAdic.infCpl.valor', top=1.6, left=0, width=WIDTH_PORTRAIT,
                       style=DEFAULT)


class Header(BandDamdfe):
    height = 10*cm
    # borders = dict(all=RoundRect(stroke_color=YELLOW))

    @staticmethod
    def _qrcode(value):
        image = StringIO()
        qr = qrcode.QRCode(version=1, error_correction=qrcode.constants.ERROR_CORRECT_L)
        qr.add_data(value)
        qr.make()
        img = qr.make_image()
        img.save(image)

        return image

    def protocolo_autorizacao(self, contingency=False):
        if not contingency:
            tag = self.add_tag('Protocolo de Autorização', 'protMDFe.infProt.dhRecbto.valor', top=8,
                               left=0, width=10, height=1, style=DEFAULT)
            tag[1].style = DEFAULT_BOLD
            tag[2].get_value = lambda i: '{} - {}'.format(
                i.protMDFe.infProt.nProt.valor,  i.protMDFe.infProt.dhRecbto.valor.strftime(
                    '%d/%m/%Y %H:%M:%S'))
            return
        self.add_label('Protocolo de Autorização', top=8, left=0, width=8, style=DEFAULT_BOLD)
        self.elements.append(Rect(top=8.5*cm, left=0, width=8*cm, height=1*cm, stroke_color=BLACK,
                                  stroke_width=5, fill=True, fill_color=BLACK, radius=0))
        fld = self.add_field('mdfe.infMDFe.ide.dhEmi.valor', top=8.5, left=0, width=8, height=1,
                             style=DEFAULT_WHITE)
        fld.get_value = lambda i: \
            u'<strong>EMISSAO EM CONTINGÊNCIA:</strong> Obrigatória a autorização em 168 horas ' \
            u'após esta data de emissão ({})'.format(
                i.mdfe.infMDFe.ide.dhEmi.valor.strftime('%d/%m/%Y %H:%M'))

    def modelo_rodoviario_carga(self, contingency=False):
        self.add_label('Modelo Rodoviário de Carga', top=5, left=0, width=5, style=COMPANY_NAME)
        _, _, fld = self.add_tag('Qtde. CT-e', 'mdfe.infMDFe.tot.qCTe.valor', top=5.6, left=0,
                                 width=1.4, height=1, style=DEFAULT, fill=SILVER)
        fld.style = DEFAULT_BOLD
        _, _, fld = self.add_tag('Qtde. NF-e', 'mdfe.infMDFe.tot.qNFe.valor', top=5.6, left=1.5,
                                 width=1.4, height=1, style=DEFAULT, fill=SILVER)
        fld.style = DEFAULT_BOLD
        _, lbl, fld = self.add_tag('Peso total', 'mdfe.infMDFe.tot.qCarga.valor', top=5.6, left=3,
                                   width=3, height=1, style=DEFAULT, fill=SILVER)
        fld.get_value = lambda i: '{:.3f} {:}'.format(
            float(i.mdfe.infMDFe.tot.qCarga.valor), {'01': 'Kg', '02': 'Ton'}.get(
                i.mdfe.infMDFe.tot.cUnid.valor))
        fld.style = DEFAULT_BOLD
        self.protocolo_autorizacao(contingency)

    def controle_fisco(self):
        self.add_label('CONTROLE DO FISCO', top=5, left=WIDTH_PORTRAIT/2/cm, width=3,
                       style=DEFAULT_BOLD)
        self.elements.append(BarCode(
            type='Code128', top=5.5*cm, left=WIDTH_PORTRAIT/2-17, height=1.7*cm, width=.035*cm,
            get_value=lambda i: i.mdfe.infMDFe.Id.valor[4:]))
        tag = self.add_tag('Chave de acesso', 'mdfe.infMDFe.Id.valor', top=8, left=WIDTH_PORTRAIT/2/cm,
                           width=10, height=1, style=DEFAULT)
        tag[1].style = DEFAULT_BOLD
        tag[2].get_value = lambda i: ' '.join([
            i.mdfe.infMDFe.Id.valor[4:][x:x + y] for x, y in zip(range(0, 44, 4), (4, )*11)])
        fld = self.add_field('mdfe.infMDFeSupl.qrCodMDFe.valor', top=9, left=tag[0].left/cm + .05,
                             width=10, style=DEFAULT)
        fld.get_value = lambda i: 'Consulte em: <strong>' + \
                                  i.mdfe.infMDFeSupl.qrCodMDFe.valor[
                                  :i.mdfe.infMDFeSupl.qrCodMDFe.valor.index(
                                      '?')] + '</strong>'

    def __init__(self, qr_code, logo, contingency=False):
        super(Header, self).__init__()
        _logo = isinstance(logo, StringIO) and logo or logo and StringIO(LOGO.decode('base64'))
        self.add_image(_logo, top=0, left=0, width=5, height=5),
        self.add_image(self._qrcode(qr_code), top=0, left=16.5, width=6, height=6),
        self.add_field('mdfe.infMDFe.emit.xNome.valor', top=0, left=3,  width=12,
                       style=COMPANY_NAME),
        fld = self.add_field('mdfe.infMDFe.emit.enderEmit.xLgr.valor', top=.4, left=3, width=12,
                             style=DEFAULT)
        fld.get_value = lambda i: ', '.join([i.mdfe.infMDFe.emit.enderEmit.xLgr.valor,
                                             i.mdfe.infMDFe.emit.enderEmit.nro.valor,
                                             i.mdfe.infMDFe.emit.enderEmit.xBairro.valor])
        fld = self.add_field('mdfe.infMDFe.emit.enderEmit.xMun.valor', top=.8, left=3, width=12,
                             style=DEFAULT)
        fld.get_value = lambda i: '{}&nbsp;&nbsp;CEP: {}-{}'.format(
            ' - '.join([i.mdfe.infMDFe.emit.enderEmit.xMun.valor,
                        i.mdfe.infMDFe.emit.enderEmit.uf.valor]),
            str(i.mdfe.infMDFe.emit.enderEmit.cep.valor)[:5],
            str(i.mdfe.infMDFe.emit.enderEmit.cep.valor)[-3])
        fld = self.add_field('mdfe.infMDFe.emit.cnpj.valor', top=1.2, left=3, width=12,
                             style=DEFAULT)
        fld.get_value = lambda i: ' '.join(['<strong>CNPJ: </strong> {}.{}.{}/{}-{}'.format(
            *[str(i.mdfe.infMDFe.emit.cnpj.valor).zfill(14)[x:x + y] for x, y in zip(
                (0, 2, 5, 8, 12), (2, 3, 3, 4, 2))]), '&nbsp;<strong>IE: </strong>{}'.format(
            i.mdfe.infMDFe.emit.ie.valor), '&nbsp;<strong>RNTRC: </strong>{}'.format(
            i.mdfe.infMDFe.infModal.modalRodo.infANTT.rntrc.valor)])
        self.add_label('<strong>DAMDFE</strong> - Documento Auxiliar de Manifesto Eletrônico de '
                       'Documentos Fiscais', top=3, left=0, width=12, style=DEFAULT_BOLD)
        self.add_tag('Modelo', 'mdfe.infMDFe.ide.mod.valor', top=3.6, left=0, width=1.2, height=1,
                     style=DEFAULT, fill=BG_TAG)
        self.add_tag('Série', 'mdfe.infMDFe.ide.serie.valor', top=3.6, left=1.2, width=1.2,
                     height=1, style=DEFAULT, fill=BG_TAG)
        self.add_tag('Número', 'mdfe.infMDFe.ide.nMDF.valor', top=3.6, left=2.4, width=1.4,
                     height=1, style=DEFAULT, fill=BG_TAG)
        self.elements.extend([
            Rect(top=3.6*cm, left=3.9*cm, height=1*cm, width=1.1*cm, stroke=False, stroke_width=0,
                 stroke_color=BG_TAG, fill=True, fill_color=BG_TAG),
            Label(text='FL', top=3.6*cm, left=3.9*cm, width=1.1*cm, padding_left=2,
                  style=DEFAULT),
            SystemField(expression='%(page_number)d/%(page_count)d', top=4.1*cm, left=3.9*cm,
                        padding_left=2, style=DEFAULT)])
        _, _, fld = self.add_tag('Data e hora de emissão', 'mdfe.infMDFe.ide.dhEmi.valor', top=3.6,
                                 left=5.1, width=3, height=1, style=DEFAULT, fill=BG_TAG)
        fld.style = DEFAULT_BOLD
        fld.get_value = lambda i: '{} {}'.format(
            i.mdfe.infMDFe.ide.dhEmi.valor.strftime('%d/%m/%Y'),
            i.mdfe.infMDFe.ide.dhEmi.valor.strftime('%H:%M:%S'))
        _, _, fld = self.add_tag('UF Carreg.', 'mdfe.infMDFe.ide.ufIni.valor', top=3.6,
                                 left=8.2, width=1.4, height=1, style=DEFAULT, fill=BG_TAG)
        fld.style = DEFAULT_BOLD
        _, _, fld = self.add_tag('UF Descarreg.', 'mdfe.infMDFe.ide.ufFim.valor', top=3.6, left=9.7,
                                 width=1.7, height=1, style=DEFAULT, fill=BG_TAG)
        fld.style = DEFAULT_BOLD

        self.modelo_rodoviario_carga(contingency)
        self.controle_fisco()


class DetailValePedagio(BandDamdfe):
    def __init__(self):
        super(DetailValePedagio, self).__init__()
        self.elements = [ValePedagio()]


class DetailComposicaoCarga(BandDamdfe):
    def __init__(self, contingency=True):
        super(DetailComposicaoCarga, self).__init__()
        self.elements = contingency and [ComposicaoCarga()] or []


class Detail(BandDamdfe):
    height = .4 * cm
    # borders = dict(all=RoundRect(stroke_color=GREEN))

    def __init__(self, contingency=False):
        super(Detail, self).__init__()
        self.child_bands = [
            DetailValePedagio(),
            DetailComposicaoCarga(contingency)
        ]


class Footer(BandDamdfe):
    height = .4 * cm
    # borders = dict(all=RoundRect(stroke_color=AQUA))

    def __init__(self):
        super(Footer, self).__init__()


class Condutor(SubReport):
    def __init__(self):
        super(Condutor, self).__init__()
        # self.queryset_string = '%(object)s.infMDFe.infModal.modalRodo.veicTracao.condutor'
        self.get_queryset = lambda cls, parent: \
            parent.mdfe.infMDFe.infModal.modalRodo.veicTracao.condutor[:10]

    # noinspection PyPep8Naming
    class band_header(BandDamdfe):
        height = .4*cm
        # borders = dict(all=RoundRect(stroke_color=OLIVE))

        def __init__(self):
            super(Condutor.band_header, self).__init__()
            self.add_label('Condutor', top=0, left=WIDTH_PORTRAIT/2/cm, width=1.6,
                           style=COMPANY_NAME)
            self.add_label('CPF', top=.8, left=WIDTH_PORTRAIT/2/cm, width=2.5, style=DEFAULT)
            self.add_label('Nome', top=.8, left=WIDTH_PORTRAIT/2/cm+2.5, width=8, style=DEFAULT)
            self.elements.append(Line(top=1.2*cm, left=20*cm, right=10*cm, bottom=1.2*cm))

    # noinspection PyPep8Naming
    class band_detail(BandDamdfe):
        height = .4 * cm
        # borders = dict(all=RoundRect(stroke_color=LIME))

        def __init__(self):
            super(Condutor.band_detail, self).__init__()
            fld = self.add_field('cpf.valor', top=.9, left=WIDTH_PORTRAIT/2/cm, width=2.5,
                                 style=DEFAULT_BOLD)
            fld.get_value = lambda i: '{}.{}.{}-{}'.format(
                *[str(i.cpf.valor).zfill(11)[x:x + y] for x, y in zip(
                    (0, 3, 6, 9, 11), (3, 3, 3, 2))])
            self.add_field('xNome.valor', top=.9, left=WIDTH_PORTRAIT/2/cm+2.5, width=8,
                           style=DEFAULT_BOLD)


class ValePedagio(SubReport):
    # borders = dict(all=RoundRect(stroke_color=BLUE))

    def __init__(self):
        super(ValePedagio, self).__init__()
        self.get_queryset = lambda cls, parent: \
            parent.mdfe.infMDFe.infModal.modalRodo.infANTT.valePed.disp

    # noinspection PyPep8Naming
    class band_header(BandDamdfe):
        height = .4 * cm
        # borders = dict(all=RoundRect(stroke_color=RED))

        def __init__(self):
            super(ValePedagio.band_header, self).__init__()
            self.add_label('Vale Pedágio', top=1, left=0, width=3, style=COMPANY_NAME)
            self.add_label('Responsável CPF/CNPJ', top=1.7, left=0, width=3.2, style=DEFAULT_BOLD)
            self.add_label('Fornecedor CNPJ', top=1.7, left=3.3, width=2.7, style=DEFAULT_BOLD)
            self.add_label('N° do comprovante', top=1.7, left=6.2, width=3.2, style=DEFAULT_BOLD)
            self.elements.append(Line(top=2.1*cm, left=9.5*cm, right=0, bottom=2.1*cm))

    # noinspection PyPep8Naming
    class band_detail(BandDamdfe):
        height = .4 * cm
        # borders = dict(all=RoundRect(stroke_color=NAVY))

        def __init__(self):
            super(ValePedagio.band_detail, self).__init__()
            fld = self.add_field('cnpjPg.valor', top=1.8, left=0, width=3.2, style=DEFAULT)
            fld.get_value = lambda i: i.cnpjPg.valor and '{}.{}.{}/{}-{}'.format(
                *[str(i.cnpjPg.valor).zfill(14)[x:x + y] for x, y in zip(
                    (0, 2, 5, 8, 12), (2, 3, 3, 4, 2))]) or '{}.{}.{}-{}'.format(
                *[str(i.cpfPg.valor).zfill(11)[x:x + y] for x, y in zip(
                    (0, 3, 6, 9, 11), (3, 3, 3, 2))])
            fld = self.add_field('cnpjForn.valor', top=1.8, left=3.3, width=2.7, style=DEFAULT)
            fld.get_value = lambda i: '{}.{}.{}/{}-{}'.format(
                *[str(i.cnpjForn.valor).zfill(14)[x:x + y] for x, y in zip(
                    (0, 2, 5, 8, 12), (2, 3, 3, 4, 2))])
            self.add_field('nCompra.valor', top=1.8, left=6.2, width=3.2, style=DEFAULT)


class ComposicaoCarga(SubReport):
    # borders = dict(all=RoundRect(stroke_color=SILVER))

    def __init__(self):
        super(ComposicaoCarga, self).__init__()
        self.get_queryset = lambda cls, parent: parent.mdfe.infMDFe.infDoc.infMunDescarga

    # noinspection PyPep8Naming
    class band_header(BandDamdfe):
        height = .4 * cm
        # borders = dict(all=RoundRect(stroke_color=RED))

        def __init__(self):
            super(ComposicaoCarga.band_header, self).__init__()
            self.add_label('Informações da composição da carga', top=1.5, left=0, width=6,
                           style=COMPANY_NAME)
            self.add_label('Informação dos documentos fiscais vinculados ao manifesto', top=2.2,
                           left=0, width=8, style=DEFAULT_BOLD)
            self.add_label('Informações da unidade de transporte', top=2.2, left=8.3, width=6,
                           style=DEFAULT_BOLD)
            self.add_label('Informações da unidade de carga', top=2.2, left=14.6, width=6,
                           style=DEFAULT_BOLD)
            self.elements.append(Line(top=2.6*cm, left=WIDTH_PORTRAIT, right=0, bottom=2.6*cm))

    # noinspection PyPep8Naming
    class band_detail(BandDamdfe):
        height = .4 * cm
        # borders = dict(all=RoundRect(stroke_color=NAVY))

        def __init__(self):
            super(ComposicaoCarga.band_detail, self).__init__()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
