# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.osv.fields import float_round
from datetime import date
from openerp.tools.translate import _
import time


class orcamentos_orcamento_itens(osv.Model):
    _name = 'orcamentos.orcamento.itens'
    _inherit = 'orcamentos.orcamento.itens'

    def default_get(self, cr, uid, fields_list, context=None):
        """
        Metodo busca valores padrao para os atributos da classe.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista com a descricao dos atributos.
        :type fields_list: list
        :param context: dados com valores contextuais.
        :type context: dict
        :return: dados com a descricao dos atributos e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(orcamentos_orcamento_itens, self).default_get(cr, uid, fields_list, context)
        res['orcori_altera_vr_unitario'] = self.pool.get('ir.model.access').check_groups(
            cr, uid, 'estoque.group_estoque_alterar_preco')
        return res

    def get_local_embalagem(self, cr, uid, company_id, local_id, produto_id, grade_id, context=None):
        context = context and context.copy() or {}
        produto_embalagem_id = False
        obj_local_locacao = self.pool.get('estoque.local.locacao')
        local_locacao_ids = obj_local_locacao.search(cr, uid, [('company_id', '=', company_id),
                                                               ('local_id', '=', local_id),
                                                               ('produto_id', '=', produto_id),
                                                               ('grade_id', '=', grade_id)],
                                                     context=context)
        if local_locacao_ids:
            local_locacao_id = local_locacao_ids[0]
            local_locacao_dados = obj_local_locacao.read(cr, uid, local_locacao_id,
                                                         ['estllc_produto_embalagem_id'],
                                                         context=context)
            if isinstance(local_locacao_dados, list) and local_locacao_dados:
                local_locacao_dados = local_locacao_dados[0]
            if isinstance(local_locacao_dados, dict):
                produto_embalagem_id = local_locacao_dados.get('estllc_produto_embalagem_id') and \
                                       local_locacao_dados['estllc_produto_embalagem_id'][0]
                if produto_embalagem_id:
                    return produto_embalagem_id
        produto_embalagem_ids = self.pool.get('estoque.produto.embalagem').search(cr, uid,
                                                                                  [('produto_id',
                                                                                    '=',
                                                                                    produto_id)],
                                                                                  context=context)
        if produto_embalagem_ids and len(produto_embalagem_ids) == 1:
            produto_embalagem_id = produto_embalagem_ids[0]
        return produto_embalagem_id

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if not len(ids):
            return []
        reads = self.browse(cr, uid, ids, context=context)
        res = []
        for record in reads:
            _nome = record.produto_id.name
            res.append((record['id'], _nome))
        return res

    def write(self, cr, uid, ids, values, context=None):
        def _orcori_write_madeireira(cr, uid, ids, vals, context):
            obj_madeireira_utils = self.pool.get('madeireira.utils')
            if isinstance(ids, list):
                ids = ids[0]
            fields = ['produto_id', 'orcori_comprimento', 'produto_embalagem_id', 'orcori_largura',
                      'orcori_espessura']
            dados_orcamento_item = self.read(cr, uid, ids, fields, context=context)
            produto_embalagem_id = dados_orcamento_item.get('produto_embalagem_id') or False
            if produto_embalagem_id and isinstance(produto_embalagem_id, (tuple, list)):
                produto_embalagem_id = produto_embalagem_id[0]
            if 'orcori_comprimento' in vals:
                orcori_comprimento = vals.get('orcori_comprimento')
            else:
                orcori_comprimento = dados_orcamento_item.get('orcori_comprimento')
            if 'orcori_largura' in vals:
                orcori_largura = vals.get('orcori_largura')
            else:
                orcori_largura = dados_orcamento_item.get('orcori_largura')
            if 'orcori_espessura' in vals:
                orcori_espessura = vals.get('orcori_espessura')
            else:
                orcori_espessura = dados_orcamento_item.get('orcori_espessura')
            res = obj_madeireira_utils.madutl_verifica_medidas(cr, uid, produto_embalagem_id,
                                                               orcori_comprimento,
                                                               orcori_largura,
                                                               orcori_espessura)
            if isinstance(res,basestring):
                raise osv.except_osv(_('Atencao'), res)

            produto_id = vals.get('produto_id')
            if not produto_id:
                produto_id = dados_orcamento_item.get('produto_id')
            if isinstance(produto_id, (list, tuple)):
                produto_id = produto_id[0]
            if not orcori_comprimento:
                orcori_comprimento = dados_orcamento_item.get('orcori_comprimento')
            if not vals.get('valor_unitario_bruto'):
                if produto_id:
                    obj_madeireira_utils = self.pool.get('madeireira.utils')
                    valida_variacao = obj_madeireira_utils.madpro_valida_variacao(cr, uid, produto_id,
                                                                                  orcori_comprimento,
                                                                                  orcori_espessura,
                                                                                  context)
                    if valida_variacao.get('warning'):
                        mensagem = valida_variacao['warning']['message']
                        raise osv.except_osv(_('Atencao'), mensagem)
                    else:
                        vr_variacao = valida_variacao.get('vr_variacao')
                pessoa_id = self.browse(
                    cr, uid, ids[0] if isinstance(ids, list) else ids, context).orcamento_id.\
                    pessoa_id.id or False
                vals['valor_unitario_bruto'] = obj_madeireira_utils.madutl_get_valor_com_fator(
                    cr, uid, produto_id, vr_variacao or 0, pessoa_id, context
                )
            return vals

        context = context and context.copy() or {}
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'madeireira', context):
            values = _orcori_write_madeireira(cr, uid, ids, values, context)
        if isinstance(ids, (int, long)):
            ids = [ids]
        if values.get('local_id'):
            local_id = values['local_id']
        else:
            local_id = False
        if values.get('produto_id'):
            produto_id = values['produto_id']
        else:
            produto_id = False
        if values.get('grade_id'):
            grade_id = values['grade_id']
        else:
            grade_id = False
        if values.get('produto_embalagem_id'):
            produto_embalagem_id = values['produto_embalagem_id']
        else:
            produto_embalagem_id = False
        if values.get('quantidade'):
            quantidade = values['quantidade']
        else:
            quantidade = False
        if values.get('desconto_total'):
            desconto = values['desconto_total']
        else:
            desconto = False
        if values.get('orcori_preco_venda'):
            preco_venda = values['orcori_preco_venda']
        else:
            preco_venda = False
        res_write = []
        browse_orcamento_item = self.browse(cr, uid, ids)
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_embalagem = self.pool.get('estoque.embalagem')
        obj_estoque_preco = self.pool.get('estoque.preco.venda')
        obj_orcamentos_orcamento_capa = self.pool.get('orcamentos.orcamento.capa')
        orcamento_capa_dados = self.read(cr, uid, ids, ['orcamento_id'], context=context)
        ids_orcamento_capa = []
        for x in orcamento_capa_dados:
            orcamento_id = x.get('orcamento_id')
            if isinstance(orcamento_id, tuple):
                orcamento_id = orcamento_id[0]
            ids_orcamento_capa.append(orcamento_id)
        ids_orcamento_itens = self.search(cr, uid, [('orcamento_id', 'in', ids_orcamento_capa)], context=context)
        itens_excluidos = []
        estado = values.get('state')
        orcamento_itens_dados = self.read(cr, uid, ids_orcamento_itens, ['state'], context=context)
        if estado == 'excluido':
            for i in orcamento_itens_dados:
                if i.get('state') == 'excluido':
                    itens_excluidos.append(i.get('state'))
            itens_excluidos.append(estado)
        if len(ids_orcamento_itens) == 1 and estado == 'excluido' or len(
                ids_orcamento_itens) == len(itens_excluidos):
            obj_orcamentos_orcamento_capa.write(cr, uid, ids_orcamento_capa,
                                                {'percentual_desconto': 0.0,
                                                 'valor_desconto_total': 0.0}, context)
        for i in browse_orcamento_item:
            pessoa_id = i.orcamento_id.pessoa_id.id or None
            cliente_id = None
            if pessoa_id:
                cliente_id = self.pool.get('financeiro.cliente').search(cr, uid, [
                    ('pessoa_id', '=', pessoa_id)], context=context)
            if cliente_id:
                cliente_id = cliente_id[0]
            id = i.id
            orcamento_capa_company_id = i.orcamento_id.company_id.id
            item_local_id = local_id or i.local_id.id
            item_produto_id = produto_id or i.produto_id.id
            item_grade_id = grade_id or i.grade_id.id
            item_produto_embalagem_id = produto_embalagem_id or i.produto_embalagem_id.id
            item_quantidade = quantidade or i.quantidade or 0
            item_desconto = desconto or i.desconto_total or 0
            if not preco_venda or i.valor_unitario_bruto or i.orcori_preco_venda:
                item_preco = preco_venda or i.valor_unitario_bruto or i.orcori_preco_venda
            else:
                preco_venda = obj_estoque_preco.get_preco_venda(cr, uid, item_produto_id,
                                                                epc_company_id=orcamento_capa_company_id,
                                                                epc_grade_ids=item_grade_id,
                                                                epc_cliente_id=cliente_id)
                dados_preco_venda = preco_venda.get(item_produto_id) or {}
                item_preco = dados_preco_venda.get(item_grade_id) or dados_preco_venda.get(
                    'preco_venda')
                values['orcori_preco_venda'] = item_preco
            if values.get('quantidade'):
                if not quantidade > 0:
                    raise osv.except_osv(_('Atencao'),
                                         _('A quantidade informada deve ser maior que zero!'))
            if item_desconto and not item_desconto > 0:
                raise osv.except_osv(_('Atencao'),
                                     _('O desconto informado deve ser maior que zero!'))
            embalagem_quantidade = 1
            produto_dados = obj_estoque_produto.read(cr, uid, item_produto_id, ['tipo'],
                                                     context=context)
            if produto_dados and isinstance(produto_dados, list):
                produto_dados = produto_dados[0]
            if isinstance(produto_dados, dict):
                produto_tipo = produto_dados.get('tipo') or ''
                if produto_tipo == 'P':
                    if not item_grade_id:
                        raise osv.except_osv(_('Atencao'),
                                             _(
                                                 'A grade do produto deve ser lancada para o item no orcamento!'))
                    if not item_produto_embalagem_id:
                        raise osv.except_osv(_('Atencao'),
                                             _(
                                                 'A embalagem do produto deve ser lancada para o item no orcamento!'))
                    else:
                        produto_embalagem_dados = obj_produto_embalagem.read(cr, uid,
                                                                             item_produto_embalagem_id,
                                                                             ['embalagem_id'], context=context)
                        if isinstance(produto_embalagem_dados, list):
                            produto_embalagem_dados = produto_embalagem_dados[0]
                        if isinstance(produto_embalagem_dados, dict):
                            embalagem_id = produto_embalagem_dados.get('embalagem_id') and \
                                           produto_embalagem_dados['embalagem_id'][0] or False
                            if not embalagem_id:
                                raise osv.except_osv(_('Atencao'),
                                                     _(
                                                         'Nenhuma embalagem foi encontrada associada ao produto!'))
                            else:
                                embalagem_dados = obj_embalagem.read(cr, uid, embalagem_id,
                                                                     ['quantidade_na_embalagem'], context=context)
                                if isinstance(embalagem_dados, list):
                                    embalagem_dados = embalagem_dados[0]
                                if isinstance(embalagem_dados, dict):
                                    embalagem_quantidade = embalagem_dados.get(
                                        'quantidade_na_embalagem') or 1
            saida_produto_local = obj_estoque_produto.estoque_check_produto_in_local(cr, uid,
                                                                                     company_id=orcamento_capa_company_id,
                                                                                     produto_id=item_produto_id,
                                                                                     local_id=item_local_id, context=context)
            if not saida_produto_local:
                raise osv.except_osv(_('Atencao'), _(
                    'O produto nao encontra-se cadastrado para o local informado!'))
                ## se a alteracao nao vier do wizard, será recalculado o valor unitario.
            if not context.get('orcamento_preco_venda'):
                if not values.get('valor_unitario_bruto'):
                    values['valor_unitario_bruto'] = item_preco * embalagem_quantidade
            else:
                values['valor_unitario_bruto'] = item_preco
                values['orcori_preco_venda'] = item_preco / embalagem_quantidade
            res = super(orcamentos_orcamento_itens, self).write(cr, uid, id, values, context)
            res_write.append(res)
        return all(res_write)

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        browse_orcamento_item = self.browse(cr, uid, ids)
        orcamento_ids = filter(lambda i: i.orcamento_id.state != '1E', browse_orcamento_item)
        if orcamento_ids:
            raise osv.except_osv(_('Atencao'),
                                 _(
                                     'O item nao pode ser excluido pois o orcamento nao encontra-se em Esboco!'))
        return super(orcamentos_orcamento_itens, self).unlink(cr, uid, ids, context)

    def create(self, cr, uid, vals, context=None):
        def _orcori_create_madeireira(cr, uid, vals, context):
            obj_madeireira_utils = self.pool.get('madeireira.utils')
            produto_embalagem_id = vals.get('produto_embalagem_id')
            orcori_comprimento = vals.get('orcori_comprimento') or 0
            orcori_largura = vals.get('orcori_largura') or 0
            orcori_espessura = vals.get('orcori_espessura') or 0
            res = obj_madeireira_utils.madutl_verifica_medidas(cr, uid, produto_embalagem_id,
                                                               orcori_comprimento,
                                                               orcori_largura,
                                                               orcori_espessura)
            if isinstance(res,basestring):
                raise osv.except_osv(_('Atencao'), res)
            if not vals.get('valor_unitario_bruto'):
                if vals.get('produto_id'):
                    obj_madeireira_utils = self.pool.get('madeireira.utils')
                    valida_variacao = obj_madeireira_utils.madpro_valida_variacao(cr, uid,
                                                                                  vals.get('produto_id'),
                                                                                  orcori_comprimento,
                                                                                  orcori_espessura,
                                                                                  context)
                    if valida_variacao.get('warning'):
                        mensagem = valida_variacao['warning']['message']
                        raise osv.except_osv(_('Atencao'), mensagem)
                    else:
                        vr_variacao = valida_variacao.get('vr_variacao')
                pessoa_id = self.pool.get('orcamentos.orcamento.capa').browse(
                    cr, uid, vals.get('orcamento_id'), context).pessoa_id.id or False
                vals['valor_unitario_bruto'] = obj_madeireira_utils.madutl_get_valor_com_fator(
                    cr, uid, vals.get('produto_id'), vr_variacao or 0, pessoa_id, context
                )
            return vals

        def _orcori_create_software(cr, uid, vals):
        # METODO CRIADO PARA SALVAR A QUANTIDADE SE ELA ESTIVER READONLY
            o_tabela = self.pool.get('software.tabela.servicos')
            o_orc_capa = self.pool.get('orcamentos.orcamento.capa')
            company_id = \
                o_orc_capa.read(cr, uid, vals['orcamento_id'], ['company_id'], context=context)['company_id'][0]
            ps_id = o_tabela.search(cr, uid, [
                ('servico_id', '=', vals['produto_id']),
                ('company_id', '=', company_id)
            ], context=context)
            if ps_id:
                qtda_horas = o_tabela.read(cr, uid, ps_id[0], ['qtd_horas'], context=context)['qtd_horas']
                vals['quantidade'] = qtda_horas
            return vals

        context = context and context.copy() or {}
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'madeireira', context):
            vals = _orcori_create_madeireira(cr, uid, vals, context=context)
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'software', context=context):
            vals = _orcori_create_software(cr, uid, vals)
        orcamento_id = vals.get('orcamento_id')
        local_id = vals.get('local_id')
        produto_id = vals.get('produto_id')
        grade_id = vals.get('grade_id')
        produto_embalagem_id = vals.get('produto_embalagem_id')
        quantidade = vals.get('quantidade') or 0
        preco_venda = vals.get('orcori_preco_venda') or 0
        desconto = vals.get('desconto_total') or 0
        if produto_embalagem_id:
            brw_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem').browse(cr,
                                                                                              uid,
                                                                                              produto_embalagem_id)
            if brw_estoque_produto_embalagem and brw_estoque_produto_embalagem.embalagem_id:
                quantidade_na_embalagem = brw_estoque_produto_embalagem.embalagem_id.quantidade_na_embalagem
        else:
            quantidade_na_embalagem = 1
        if not vals.get('valor_unitario_bruto'):
            vals['valor_unitario_bruto'] = preco_venda
        if not quantidade > 0:
            raise osv.except_osv(_('Atencao'), _('A quantidade informada deve ser maior que zero!'))
        if desconto and not desconto > 0:
            raise osv.except_osv(_('Atencao'), _('O desconto informado deve ser maior que zero!'))
        obj_estoque_produto = self.pool.get('estoque.produto')
        if orcamento_id:
            browse_orcamento = self.pool.get('orcamentos.orcamento.capa').browse(cr, uid,
                                                                                 orcamento_id)
            company_id = browse_orcamento.company_id.id
            if company_id:
                saida_produto_local = obj_estoque_produto.estoque_check_produto_in_local(cr, uid,
                                                                                         company_id=company_id,
                                                                                         produto_id=produto_id,
                                                                                         local_id=local_id, context=context)
                if not saida_produto_local:
                    nome_produto = obj_estoque_produto.browse(cr, uid, produto_id,
                                                              context).name or ''
                    raise osv.except_osv(_('Atencao'), _(
                        'O produto %s nao encontra-se cadastrado para o local informado!' % nome_produto))
        produto_dados = self.pool.get('estoque.produto').read(cr, uid, produto_id, ['tipo'],
                                                              context=context)
        if produto_dados and isinstance(produto_dados, list):
            produto_dados = produto_dados[0]
        if isinstance(produto_dados, dict):
            produto_tipo = produto_dados.get('tipo') or ''
            if produto_tipo == 'P':
                if not grade_id:
                    raise osv.except_osv(_('Atencao'),
                                         _(
                                             'A grade do produto deve ser lancada para o item no orcamento!'))
                if not produto_embalagem_id:
                    raise osv.except_osv(_('Atencao'),
                                         _(
                                             'A embalagem do produto deve ser lancada para o item no orcamento!'))
        if not vals.get('valor_unitario_bruto'):
            vals['valor_unitario_bruto'] = quantidade_na_embalagem * preco_venda
        return super(orcamentos_orcamento_itens, self).create(cr, uid, vals, context=context)

    def orcori_check_preco_venda(self, cr, uid, ids, context=None):
        """
        O metodo valida se todos os itens de um orcamento possuem preco de venda
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param ids: lista de id da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: raise, caso algum item esteja sem preco. Senao, Verdadeiro
        """
        context = context and context.copy() or {}
        msg = ''
        for item in self.browse(cr, uid, ids, context):
            if item.valor_unitario_bruto or item.orcori_preco_venda:
                continue
            msg += '%s - %s\n' % (item.produto_id.codigo, item.produto_id.name)
        if msg:
            raise osv.except_osv(_('Atencao'), _('O preco de venda nao encontra-se cadastrado para'
                                                 ' os produtos abaixo:\n' + msg))
        return True

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        if not fields:
            fields = []
        else:
            if not 'orcori_preco_venda' in fields:
                fields.append('orcori_preco_venda')
            if not 'local_id' in fields:
                fields.append('local_id')
            if not 'grade_id' in fields:
                fields.append('grade_id')
            if not 'produto_id' in fields:
                fields.append('produto_id')
            if not 'valor_total' in fields:
                fields.append('valor_total')
            if not 'produto_embalagem_id' in fields:
                fields.append('produto_embalagem_id')
            if not 'quantidade' in fields:
                fields.append('quantidade')
            if not 'percentual_desconto_item' in fields:
                fields.append('percentual_desconto_item')
            if not 'desconto_total' in fields:
                fields.append('desconto_total')
        res = super(orcamentos_orcamento_itens, self).read(cr, uid, ids, fields, context=context, load=load)
        if 'produto_embalagem_id' in res:
            produto_embalagem_id = res.get('produto_embalagem_id')
            if isinstance(produto_embalagem_id, (list, tuple)):
                produto_embalagem_id = produto_embalagem_id[0]
            if produto_embalagem_id:
                quantidade_embalagem = obj_estoque_produto_embalagem.browse(cr, uid,
                                                                            produto_embalagem_id).embalagem_id.quantidade_na_embalagem
                if quantidade_embalagem:
                    res['quantidade'] = res.get('quantidade') / quantidade_embalagem
        return res

    def orcori_ratear_despesa_itens(self, cr, uid, itens, despesas, context=None):
        """
        Rateia as despesas da capa do orcamento no itens proporcionalmente ao total de cada item.
        :param cr: 
        :param uid: 
        :param itens: lista no formato de campos one2many [[1,1,{...}], [0, False, {...}], ...]
        :param despesas: valor das respesas que serah rateada nos itens
        :param context: 
        :return: dict
        """
        context = context and context.copy() or {}
        item_ids = []
        ids_valor = []
        ids_desp = []
        total_itens = 0
        total_despesas_itens = 0
        for i in itens:
            if i[0] not in (0, 1, 4):
                continue

            if i[0] == 4:
                i[0] = 1

            if not i[2]:
                item_ids.append(i[1])
                ids_desp.append(i[1])
                ids_valor.append(i[1])
            else:
                if 'valor_total' not in i[2]:
                    item_ids.append(i[1])
                    ids_valor.append(i[1])
                else:
                    total_itens += i[2].get('valor_total') or 0

                if 'orcori_despesas' not in i[2]:
                    i[1] not in item_ids and item_ids.append(i[1])
                    ids_desp.append(i[1])
                else:
                    total_despesas_itens += i[2].get('orcori_despesas') or 0

        dict_itens = {}
        for i in self.read(cr, uid, item_ids, ['valor_total', 'orcori_despesas'], context,
                           '_classic_write'):
            dict_itens[i.get('id')] = i
            if i.get('id') in ids_valor:
                total_itens += i.get('valor_total')
            if i.get('id') in ids_desp:
                total_despesas_itens += i.get('orcori_despesas')

        dif_desp = despesas - total_despesas_itens
        # controle para nao recalcular quando a despesas eh alterada no item
        if -0.01 < dif_desp and dif_desp < 0.01:
            return {}

        maior_item = []
        percentual = despesas / total_itens
        total_calculado = 0
        # rateia o valor da despesa proporcional ao valor total do item
        for item in itens:
            if item[0] not in (0, 1, 4):
                continue

            valor_total = item[2] and item[2].get('valor_total') or \
                          dict_itens.get(item[1]).get('valor_total')
            despesa_item = float_round((valor_total * percentual), 2)
            total_calculado += despesa_item
            if not item[2]:
                item[2] = {'orcori_despesas': despesa_item}
            else:
                item[2]['orcori_despesas'] = despesa_item

            if not maior_item or maior_item[2].get('orcori_despesas') < despesa_item:
                maior_item = item

        dif = despesas - total_calculado
        # joga a diferenca do calculo do rateio no item de maior valor
        if dif < -0.009 or 0.009 < dif:
            maior_item[2]['orcori_despesas'] += dif

        return {'orcamento_itens_ids': itens}

    def get_preco_venda_embalagem(self, cr, uid, company_id, produto_id, grade_id,
                                  produto_embalagem_id, pessoa_id=None, context=None):
        context = context and context.copy() or {}
        cliente_id = None
        if pessoa_id:
            cliente_id = self.pool.get('financeiro.cliente').search(cr, uid,
                                                                    [('pessoa_id', '=', pessoa_id)], context=context)
            if cliente_id:
                cliente_id = cliente_id[0]
                ## BUSCA O VALOR UNITARIO DO PRODUTO
        preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid, produto_id,
                                                                           epc_company_id=company_id,
                                                                           epc_grade_ids=grade_id,
                                                                           epc_cliente_id=cliente_id)
        dados_preco_venda = preco_venda.get(produto_id) or {}
        preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
            'preco_venda')
        valor_unitario = preco_venda
        ## SE ALGUMA EMBALAGEM FOR ESCOLHIDA
        if produto_embalagem_id:
            ## BUSCA A QUANTIDADE PRESENTE NA EMBALAGEM
            qtde_embalagem = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                                                                               produto_embalagem_id).embalagem_id.quantidade_na_embalagem
            valor_unitario = preco_venda * (qtde_embalagem or 1)
        return valor_unitario

    def onchange_local_orc_item(self, cr, uid, ids, local_id, produto_id, grade_id):
        vals = {}.fromkeys(['produto_id', 'grade_id', 'produto_embalagem_id'],False)
        vals.update({}.fromkeys(
            ['quantidade', 'valor_unitario_bruto', 'percentual_desconto_item', 'desconto_total',
             'valor_total', 'orcori_preco_venda', 'orcori_saldo'], 0))
        return {'value': vals}

    def orcori_get_saldo(self, cr, uid, local_id, produto_id, grade_id, produto_emb_id,
                         context=None):
        """
        O metodo fara a busca do saldo em estoque de um produto, baseados no produto, grade e no
        local em que se encontra. Retorna dicionario com as informacoes de saldo e quantidade
        reservada
        @param cr: cursor do banco de dados
        @param uid: id do usuario
        @param local_id: id da tabela estoque_local
        @param produto_id: id da tabela estoque_produto
        @param grade_id: id da tabela estoque_produto_grade
        @param context: dicionario com valores contextuais
        """
        context = context and context.copy() or {}
        vals = {'orcori_saldo': 0, 'orcori_reservada': 0}
        if not all([local_id, produto_id, grade_id]):
            return vals
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')
        data = time.strftime('%Y-%m-%d')
        saldo = obj_estoque_produto_saldo.get_saldo_do_dia(cr, uid, grade_id, data, local_id,
                                                           context=context)
        saldo = saldo.get(grade_id).get('saldo_na_embalagem').get(produto_emb_id)
        if saldo:
            vals['orcori_saldo'] = saldo.get('saldo') or 0
            vals['orcori_reservada'] = saldo.get('qtd_reservada')
        return vals

    def onchange_produto_orc_item(self, cr, uid, ids, company_id, local_id, produto_id, grade_id,
                                  orcori_preco_venda,
                                  pessoa_id=None, context=None):
        context = context and context.copy() or {}
        warning = {}
        if not produto_id:
            vals = {}.fromkeys(['grade_id', 'produto_embalagem_id'], False)
            vals.update({}.fromkeys(
                ['quantidade', 'valor_unitario_bruto', 'percentual_desconto_item', 'desconto_total',
                 'valor_total',
                 'orcori_preco_venda'], 0))
        else:
            vals = {}
            obj_adm_util = self.pool.get('adm.utils')
            ## busca o preco de venda do produto, 1º no campo orcori_preco_venda, 2º cadastro do produto
            cliente_id = None
            if pessoa_id:
                cliente_id = self.pool.get('financeiro.cliente').search(cr, uid, [
                    ('pessoa_id', '=', pessoa_id)], context=context)
                if cliente_id:
                    cliente_id = cliente_id[0]
            preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, produto_id, epc_company_id=company_id, epc_grade_ids=grade_id,
                epc_cliente_id=cliente_id, context=context)
            if orcori_preco_venda != preco_venda:
                dados_preco_venda = preco_venda.get(produto_id) or {}
                preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                    'preco_venda')
            else:
                preco_venda = orcori_preco_venda
            if not preco_venda:
                warning = {'title': _('Atencao'),
                           'message': _('Nao ha Preco de Venda  cadastrado para o item')}
            vals['valor_unitario_bruto'] = preco_venda
            vals['orcori_preco_venda'] = preco_venda
            vals['quantidade'] = 0.0
            grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid, [
                ('produto_id', '=', produto_id)], context=context)
            _brw_produto = self.pool.get('estoque.produto').browse(cr, uid, produto_id)
            modulo_kit_instalado = obj_adm_util.admutl_check_modulo_instalado(cr, uid, 'kit',
                                                                              context)
            if modulo_kit_instalado:
                kit_de_produto = _brw_produto.kit_de_produtos
                if kit_de_produto:
                    vals = {}.fromkeys(['produto_id', 'grade_id', 'produto_embalagem_id'], False)
                    mensagem = _('\n Este e um item kit e deve ser informado na aba "Kit por '
                                 'itens"')
                    if warning:
                        warning['message'] += mensagem
                    else:
                        warning['title'] = _('Atencao')
                        warning['message'] = mensagem
                    return {'value': vals, 'warning': warning}
            vals['orcori_foto'] = _brw_produto.estpro_foto
            if _brw_produto.tipo == 'S': # Para servico
                _emb = obj_adm_util.admutl_get_parametros(cr, uid, 'estoque',
                                                          ['estpar_embalagem_servico_id'])
                _emb_id = False
                for x in _brw_produto.produto_embalagem_ids:
                    if x.embalagem_id.id == _emb['estpar_embalagem_servico_id']:
                        _emb_id = x.id
                        break
                if _emb_id:
                    vals['produto_embalagem_id'] = _emb_id
                else: # Se nao existir embalagem cadastrada para o servico, cadastrar
                    vals['produto_embalagem_id'] = self.pool.get(
                        'estoque.produto.embalagem').create(cr, uid,
                                                            {'active': True,
                                                             'produto_id': produto_id,
                                                             'embalagem_id':
                                                                 _emb[
                                                                     'estpar_embalagem_servico_id']}, context)

            if len(grade_ids) == 1:
                grade_id = grade_ids[0]
                vals['grade_id'] = grade_id
                produto_embalagem_id = self.get_local_embalagem(cr, uid, company_id, local_id,
                                                                produto_id, grade_id)
                if produto_embalagem_id:
                    vals['produto_embalagem_id'] = produto_embalagem_id
                    embalagem_quantidade = self.pool.get('estoque.produto.embalagem').browse(cr,
                                                                                             uid,
                                                                                             produto_embalagem_id).embalagem_id.quantidade_na_embalagem
                    vals['valor_unitario_bruto'] = preco_venda * embalagem_quantidade
                else:
                    vals['produto_embalagem_id'] = False
            else:
                vals['grade_id'] = ''
                vals['produto_embalagem_id'] = ''
        if 'produtos_similares' in context and produto_id:
            brw_similaridade = _brw_produto.produto_similaridade_ids
            simi_ids=[]
            for simi in brw_similaridade:
                simi_ids.append(simi.id)
            vals['orcori_prod_similares'] = simi_ids
        return {'value': vals, 'warning': warning}

    def onchange_grade_orc_item(self, cr, uid, ids, company_id, local_id, produto_id, grade_id,
                                produto_embalagem_id, pessoa_id=False,context=None):
        context = context and context.copy() or {}
        vals = {}
        if not grade_id:
            return {'value': vals}
        cliente_id = None
        if pessoa_id:
            cliente_id = self.pool.get('financeiro.cliente').search(cr, uid, [
                ('pessoa_id', '=', pessoa_id)], context=context)
            if cliente_id:
                cliente_id = cliente_id[0]
        produto_embalagem_id = False
        if company_id and local_id and produto_id and grade_id and not produto_embalagem_id:
            produto_embalagem_id = self.get_local_embalagem(cr, uid, company_id, local_id,
                                                            produto_id, grade_id)
            if produto_embalagem_id:
                vals['produto_embalagem_id'] = produto_embalagem_id or ''

        if produto_id:
            preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, produto_id, epc_company_id=company_id, epc_grade_ids=grade_id, epc_cliente_id=cliente_id)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')
            vals.update({'valor_unitario_bruto': preco_venda})

        return {'value': vals}

    def onchange_embalagem_orc_item(self, cr, uid, ids, company_id, produto_id, grade_id,
                                    produto_embalagem_id, quantidade,
                                    desconto_total, orcori_preco_venda, pessoa_id=None,
                                    local_id=False, context=None):
        context = context and context.copy() or {}
        def _orcori_onchange_embalagem_orc_item_madeireira_inicial(cr, uid, produto_id,
                                                                   produto_embalagem_id,
                                                                   orcori_preco_venda, context):
            obj_madeireira_utils = self.pool.get('madeireira.utils')
            sigla = ''
            if produto_embalagem_id:
                obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
                browse_produto_embalagem = obj_estoque_produto_embalagem.browse(cr, uid,
                                                                                produto_embalagem_id)
                embalagem_id = browse_produto_embalagem.embalagem_id
                if embalagem_id:
                    sigla = embalagem_id.sigla or ''
            comprimento = context.get('orcori_comprimento') or 0
            largura = context.get('orcori_largura') or 0
            espessura = context.get('orcori_espessura') or 0
            if sigla == 'M2':
                espessura = 0
            elif sigla == 'ML':
                largura = espessura = 0
            elif sigla != 'M3':
                comprimento = largura = espessura = 0

            if orcori_preco_venda:
                valida_variacao = obj_madeireira_utils.madpro_valida_variacao(cr, uid, produto_id,
                                                                              comprimento,
                                                                              espessura,
                                                                              context)
                if not valida_variacao.get('warning'):
                    vr_variacao = valida_variacao.get('vr_variacao')
                    orcori_preco_venda = obj_madeireira_utils.madutl_get_valor_com_fator(
                    cr, uid, produto_id, vr_variacao or 0, pessoa_id, context
                )
            return orcori_preco_venda, sigla, comprimento, largura, espessura

        def _orcori_onchange_embalagem_orc_item_madeireira_final(res, sigla, volume, comprimento,
                                                                 largura, espessura):
            res['value']['orcori_sigla_embalagem'] = sigla
            res['value']['orcori_volume'] = volume
            res['value']['orcori_comprimento'] = comprimento
            res['value']['orcori_largura'] = largura
            res['value']['orcori_espessura'] = espessura
            return res

        madeireira_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid,
                                                                                        'madeireira',
                                                                                        context)

        quantidade = self._get_qtd_madeireira(cr, uid, ids, quantidade, context)
        if madeireira_instalado:
            orcori_preco_venda, sigla, comprimento, largura, espessura = (
                _orcori_onchange_embalagem_orc_item_madeireira_inicial(cr, uid, produto_id,
                                                 produto_embalagem_id, orcori_preco_venda, context))
        vals = {}
        cliente_id = None
        if pessoa_id:
            cliente_id = self.pool.get('financeiro.cliente').search(cr, uid,
                                                                    [('pessoa_id', '=', pessoa_id)], context=context)
            if cliente_id:
                cliente_id = cliente_id[0]
        preco_venda = orcori_preco_venda
        if produto_embalagem_id:
            if isinstance(produto_embalagem_id, (list, tuple)):
                produto_embalagem_id = produto_embalagem_id[0]
            embalagem_quantidade = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                                                                                     produto_embalagem_id).embalagem_id.quantidade_na_embalagem
            preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, produto_id, epc_company_id=company_id, epc_grade_ids=grade_id,
                epc_cliente_id=cliente_id)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')
            if not preco_venda:
                preco_venda = orcori_preco_venda

            preco_venda *= embalagem_quantidade
        else:
            vals.update({'valor_unitario_bruto': preco_venda})

        valor_total = self._get_valor_total(cr, uid, ids, quantidade, preco_venda, desconto_total,
                                            context)
        vals.update({'valor_total': valor_total,
                     'valor_unitario_bruto': preco_venda})
        vals.update(self.orcori_get_saldo(cr, uid, local_id, produto_id, grade_id,
                                          produto_embalagem_id, context=context))
        res = {'value': vals}
        if madeireira_instalado:
            volume = quantidade
            res = _orcori_onchange_embalagem_orc_item_madeireira_final(res, sigla, volume,
                                                                    comprimento, largura, espessura)

        return res

    def _get_qtd_madeireira(self, cr, uid, ids, quantidade, context):
        context = context and context.copy() or {}
        if not self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'madeireira',
                                                                        context):
            return quantidade

        obj_madeireira_utils = self.pool.get('madeireira.utils')
        comprimento = context.get('orcori_comprimento') or 0
        largura = context.get('orcori_largura') or 0
        espessura = context.get('orcori_espessura') or 0
        volume = obj_madeireira_utils.madutl_calcula_volume(quantidade, comprimento, largura,
                                                            espessura)
        quantidade = volume or quantidade
        return quantidade

    def _get_valor_total(self, cr, uid, ids, quantidade, valor_unitario, desconto_total,
                         context=None):
        """Calcula o total do item"""
        context = context and context.copy() or {}
        quantidade = self._get_qtd_madeireira(cr, uid, ids, quantidade, context)

        return (valor_unitario * quantidade) - desconto_total

    def onchange_desconto(self, cr, uid, ids, quantidade, valor_unitario, desconto_total,
                          context=None):

        context = context and context.copy() or {}
        vals = {}
        valor_total = self._get_valor_total(cr, uid, ids, quantidade, valor_unitario,
                                            desconto_total, context)
        if not desconto_total:
            vals.update({'percentual_desconto_item': 0})

        vals.update({'valor_total': valor_total})
        return {'value': vals}

    def onchange_desconto_percentual(self, cr, uid, ids, quantidade, valor_unitario,
                                     percentual_desconto_item, context=None):
        context = context and context.copy() or {}
        desconto_total_aux = (valor_unitario * quantidade) * percentual_desconto_item / 100.0
        valor_total = self._get_valor_total(cr, uid, ids, quantidade, valor_unitario,
                                            desconto_total_aux, context)
        return {'value': {'desconto_total': desconto_total_aux,
                          'valor_total': valor_total}}

    def onchange_quantidade(self, cr, uid, ids, grade_id, produto_id, local_id, valor_unitario,
                            quantidade, desconto_total, produto_embalagem_id, pessoa_id,
                            context=None):
        def _orcori_onchange_quantidade_madeireira(cr, uid, res, produto_id, valor_unitario_bruto,
                                                   quantidade, desconto_total, context):
            obj_madeireira_utils = self.pool.get('madeireira.utils')
            res['value']['orcori_volume'] = quantidade
            if valor_unitario_bruto and produto_id:
                valida_variacao = obj_madeireira_utils.madpro_valida_variacao(cr, uid, produto_id,
                                                             context.get('orcori_comprimento') or 0,
                                                             context.get('orcori_espessura') or 0,
                                                                              context)
                if valida_variacao.get('warning'):
                    valida = valida_variacao['value']
                    value = {'orcori_comprimento': valida['comprimento'],
                             'orcori_espessura': valida['espessura']}
                    valida_variacao['value'] = value
                    return valida_variacao
                else:
                    vr_variacao = valida_variacao.get('vr_variacao')

                res['value']['vr_unitario'] = obj_madeireira_utils.madutl_get_valor_com_fator(
                    cr, uid, produto_id, vr_variacao, pessoa_id, context
                )
            return res

        context = context and context.copy() or {}
        warning = {}
        vals = {}
        fracao_unidade = False
        mensagem = ''
        if produto_embalagem_id:
            fracao_unidade = self.pool.get('estoque.produto.embalagem').browse(cr, uid,
                                                                               produto_embalagem_id).embalagem_id.unidade_id.fracao or False
        if not quantidade:
            return {'value': {'valor_total': 0}}

        if not fracao_unidade and (quantidade != int(quantidade)):
            mensagem = _('A embalagem nao aceita fracionamento. \n')
            vals['quantidade'] = 0.0

        valor_total = self._get_valor_total(cr, uid, ids, quantidade, valor_unitario,
                                            desconto_total, context)
        vals.update({'valor_total': valor_total})
        if local_id and produto_id:
            _tipo = self.pool.get('estoque.produto').browse(cr, uid, produto_id).tipo
            if _tipo == 'S':
                return {'value': vals, 'warning': warning}
            data = date.today()
            obj_estoque_local = self.pool.get('estoque.local')
            aceita_saldo_negativo = obj_estoque_local.browse(cr, uid,
                                                             local_id).aceita_saldo_negativo
            saldo = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(cr, uid, grade_id, data, local_id,
                                                                            context=context)
            saldo = saldo.get(grade_id).get('saldo') if grade_id else 0
            if saldo <= 0:
                if not aceita_saldo_negativo:
                    if mensagem:
                        mensagem += _('Nao ha saldo disponivel.\n')
                    else:
                        mensagem = _('Nao ha saldo disponivel.\n')
                else:
                    if mensagem:
                        mensagem += _('O saldo do produto e: ') + str(saldo)
                    else:
                        mensagem = _('O saldo do produto e: ') + str(saldo)
        if mensagem:
            warning = {'title': _("Atencao"), 'message': _(mensagem)}
        res = {'value': vals, 'warning': warning}
        if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'madeireira', context):
            res = _orcori_onchange_quantidade_madeireira(cr, uid, res, produto_id, valor_unitario,
                                                      quantidade, desconto_total, context)
        return res

    def orcori_onchange_valor_unitario_bruto(self, cr, uid, ids, quantidade, desconto,
                                             valor_unitario, context=None):
        """
        Metodo invocado na atribuicao de valor ao atributo 'valor_unitario_bruto' (valor_unitario).
        Calcula o valor total do item, no momento da alteracao, considerando o valor unitario,
        quantidade e desconto.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param quantidade: quantidade do item no orcamento.
        :type quantidade: float
        :param desconto: valor do desconto do item no orcamento.
        :type desconto: float
        :param valor_unitario: valor de venda do item no orcamento.
        :type valor_unitario: float
        :param context: dados com valores contextuais.
        :type context: dict
        :return: dados com a descricao do atributo e seu respectivo valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        vals = {'valor_total': 0.0}
        if not quantidade or not valor_unitario:
            return {'value': vals}

        valor_total = self._get_valor_total(cr, uid, ids, quantidade, valor_unitario, desconto,
                                            context)
        vals['valor_total'] = valor_total
        return {'value': vals}

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        obj_orcamento_capa = self.pool.get('orcamentos.orcamento.capa')
        geral_orcamento_ids = []
        args1 = []
        if args:
            for arg in args:
                if 'orcori_state' in arg:
                    orcamento_ids = obj_orcamento_capa.search(cr, uid, [('state', '=', arg[2])], context=context)
                    geral_orcamento_ids.extend(orcamento_ids)
                if 'create_date' in arg and '=' in arg:
                    data_criacao = arg[2][:10]
                    args.extend([['create_date', '>=', data_criacao + ' 00:00:00'],
                                 ['create_date', '<=', data_criacao + ' 23:59:59']])
        if geral_orcamento_ids:
            for arg in args:
                if arg != '|' and 'orcori_state' not in arg and ('create_date' and '=') not in arg:
                    args1.append(arg)
            args1.append(['orcamento_id', 'in', geral_orcamento_ids])
            args = args1
        else:
            for arg in args:
                if arg != '|' and ('create_date' and '=') not in arg:
                    args1.append(arg)
            args = args1
        res = super(orcamentos_orcamento_itens, self).search(cr, uid, args, offset, limit, order,
                                                             context, count)
        return res

