# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import _orccap_get_valor_liquido
from funcoes import _orccap_prioridade
from funcoes import _orccap_get_observacao_cliente
from funcoes import _orccap_get_valor_ipi_total
from funcoes import _orcap_get_altera_prazo_validade_padrao
from funcoes import _orcap_get_altera_prazo_validade
from funcoes import _orccap_check_nao_imprimir_valor_item

class orcamentos_orcamento_capa(osv.Model):
    """
    Tabela de orcamentos

    prefixo: orccap_
    """
    from funcoes import _orccap_get_vendedor_obrigatorio

    _name = 'orcamentos.orcamento.capa'
    _rec_name = 'documento'
    _inherit = ['ir.needaction_mixin', 'mail.thread']
    _roteavel = True
    _columns = {
        'create_date': fields.datetime('Data', readonly=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # Id da empresa
        'documento': fields.char('Documento', size=15, required=False, select=True),
        # Numero do orcamento:\nCriar Sequence ORC-AAAA-999999
        'validade': fields.date('Validade'), # Validade em dias\n
        'previsao_aprovacao': fields.date('Previsao Aprovacao'),
        'vendedor_id': fields.many2one("vendas.vendedor", 'Vendedor', ondelete="restrict", select=True),
        'vendedor_externo_id': fields.many2one("vendas.vendedor", 'Vendedor Externo', ondelete="restrict", select=True),
        'orccap_origem_id': fields.many2one('orcamentos.orcamento.origem', 'Origem',
                                            help='Identifica a origem da venda'),
        'valor_liquido': fields.function(_orccap_get_valor_liquido, method=True, string='Valor Total', type='float',
                                         store=True),
        'valor_desconto_total': fields.float('Desconto Total', digits=(15, 2)), # Valor desconto total do orcamento
        'aos_cuidados': fields.char('Aos Cuidados', size=45), # indica aos cuidados de quem deve ser enviado o orcamento
        'nome_cliente': fields.char('Nome Cliente', size=45), # Nome do cliente, quando ele nao estiver cadastrado
        'pessoa_id': fields.many2one("adm.pessoa", 'Cliente', ondelete="restrict", required=False, select=True),
        'observacao': fields.text('Observacao'), # Observacoes complementares
        'orcamento_itens_ids': fields.one2many('orcamentos.orcamento.itens', 'orcamento_id', 'Itens', required=True),
        'orcamento_condicao_ids': fields.one2many('orcamentos.orcamento.condicao', 'capa_id', 'Condicao'),
        'pedido_id': fields.many2one('estoquepedidos.pedido.capa', 'Pedido', ondelete="restrict", required=False,
                                     select=True),
        'altera_prazo_validade': fields.function(_orcap_get_altera_prazo_validade, method=True,
                                                 string='Altera prazo validade', type='boolean', store=False),
        'state': fields.selection([
                                      ('1E', 'Esboco'),
                                      ('2V', 'Validado'),
                                      ('3A', 'Aprovado'),
                                      ('4G', 'Gerado Pedido'),
                                      ('5C', 'Cancelado'),
                                  ], string='Estado', readonly=False, size=2),
        'condicao_pagto_id': fields.many2one('adm.condicao', 'Condicao de Pagamento', required=False,
                                             ondelete="restrict"),
        'percentual_desconto': fields.float('% Desconto', digits=(3, 2), help="Percentual de desconto no Orcamento."),
        'prioridade': fields.function(_orccap_prioridade, type='integer', method=True, string="Prioridade",
                                      store=False),
        'valor_acrescimo_total': fields.float('Acrescimo', digits=(15, 2), help="Valor do acrescimo do orcamento."),
        'orccap_cor': fields.integer('Cor'),
        'observacao_pessoa': fields.function(_orccap_get_observacao_cliente, type="text", method=True,
                                             string="Observacao Cliente", store=False),
        'orccap_valor_ipi': fields.function(_orccap_get_valor_ipi_total, method=True, string='Valor do IPI',
                                            type='float', store=False),
        'orccap_nao_imprimir_valor_item': fields.function(_orccap_check_nao_imprimir_valor_item,
                                                          string="Nao imprimir valor item",
                                                          method=True, type='boolean', store=False),
        'orccap_vr_despesas_acessorias': fields.float('Despesas Acessorias', digits=(15, 2),
                                               help="Valor das despesas Acessorias"), # Valor das despesas acessorias
        'orccap_vr_frete': fields.float('Valor do frete', digits=(14, 2)),
        'orccap_cancelamento_id': fields.many2one('venda.motivo.cancelamento',
                                                  'Motivo cancelamento'),
        'orccap_cancelamento_complemento': fields.text('Complemento cancelamento'),
        'orccap_data_entrega': fields.date('Data de entrega'),
        'orccap_endereco_id': fields.many2one("adm.pessoa.endereco", 'Endereco ',
                                              ondelete="restrict",
                                              domain="[('pessoa_id', '=', pessoa_id)]"),
        'orccap_vendedor_obrigatorio': fields.function(
            _orccap_get_vendedor_obrigatorio, store=False, type='boolean',
            string='Vendedor obrigatorio')
    }
    _order = 'documento desc'
    _defaults = {
        'state': lambda *a: '1E',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'orccap_cor': 0,
        'altera_prazo_validade': _orcap_get_altera_prazo_validade_padrao,
        'orccap_vendedor_obrigatorio': _orccap_get_vendedor_obrigatorio
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

