# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class orcamentos_orcamento_itens(osv.Model):
    """
    Tabela que contem os items dos orcamentos
    PREFIXO PARA CRIAÇÃO DE CAMPOS: orcori_
    """
    from funcoes import _orcori_get_total_itens, _orcori_get_foto, _orcori_get_local_venda_padrao
    from funcoes import _orcori_get_saldo_produto, _orcori_get_prod_similares, _orcori_get_par_similaridade
    from funcoes import default_get_orcori_par_similaridade, _orcori_get_state
    from funcoes import orcori_get_permissao_vr_unitario
    _name = 'orcamentos.orcamento.itens'
    _rec_name = 'produto_id'
    _columns = {
        'create_date': fields.datetime('Data criacao'),
        'orcamento_id': fields.many2one("orcamentos.orcamento.capa", 'Orcamento', ondelete="cascade", required=True,
                                        select=True), # Id do orcamento
        'produto_id': fields.many2one("estoque.produto", 'Produto', ondelete="restrict", required=True), # Id do produto
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', ondelete="restrict", select=True,
                                    domain="[('produto_id', '=', produto_id)]"), # Codigo da grade/produto
        'local_id': fields.many2one("estoque.local", 'Local', ondelete="restrict", select=True, required=True),
        'produto_embalagem_id': fields.many2one("estoque.produto.embalagem", 'Embalagem', ondelete="restrict",
                                                select=True, domain="[('produto_id', '=', produto_id)]"),
        'quantidade': fields.float('Quantidade', digits=(13, 3), required=True), # Quantidade orcada
        'valor_unitario_bruto': fields.float('Valor Unit. Bruto', digits=(15, 2), required=True),
        'desconto_total': fields.float('Desconto Total', digits=(15, 2)), # Valor do desconto total
        'orcori_despesas': fields.float('Despesas', digits=(15, 2),
                                        help="Valor das despesas do item"),
        'valor_total': fields.function(_orcori_get_total_itens, method=True, string='Valor Total', type='float',
                                       digits=(15, 2), store=True),
        'orcori_preco_venda': fields.float('Preco Venda', digits=(15, 2), oldname='ori_preco_venda'),
        'percentual_desconto_item': fields.float('% Desconto', digits=(3, 2), help="Percentual do Desconto do Item."),
        'orcori_foto': fields.function(_orcori_get_foto, method=True, string='Foto', type="binary", store=False),
        'orcori_saldo': fields.function(_orcori_get_saldo_produto, method=True, string='Saldo',
                                        type='float', digits=(15, 3), store=False, multi='saldo'),
        'orcori_reservada': fields.function(_orcori_get_saldo_produto, method=True,
                                            string='Qtd. reservada', type='float', digits=(15, 3),
                                            store=False, multi='saldo'),
        'orcori_prod_similares': fields.function(_orcori_get_prod_similares, method=True,
                                                string='Produtos Similares', relation='estoque.produto.similaridade',type='one2many', store=False),
        'orcori_par_similaridade': fields.function(_orcori_get_par_similaridade, method=True,
                                                string='Similaridade', type='boolean', store=False),
        'orcori_observacao': fields.text('Observacao'),
        'orcori_state': fields.function(_orcori_get_state, method=True, string='Estado',
                                        type='char', store=False),
        'orcori_altera_vr_unitario': fields.function(orcori_get_permissao_vr_unitario, method=True,
                                                     string='Permissao valor unitario',
                                                     type='boolean'),
    }
    _defaults = {
        'local_id': _orcori_get_local_venda_padrao,
        'orcori_par_similaridade': default_get_orcori_par_similaridade
    }

