# -*- encoding: utf-8 -*-

from SigUtils.impressao import Impressora
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
from openerp import pooler
import time
import locale
from openerp.osv import osv, fields
# noinspection PyProtectedMember
from openerp.tools.translate import _
from dateutil.parser import parse


# noinspection PyPep8Naming
class external_pdf(render):

    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming,PyAttributeOutsideInit
class report_custom(report_int):

    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_orcamento = pooler.get_pool(cr.dbname).get('orcamentos.orcamento.capa')

        # Dados do formulario.
        empresa = datas['form']['company_id'] or ''
        pessoa = datas['form']['pessoa_id'] or ''
        vendedor = datas['form']['vendedor_id'] or ''
        data_fim = datas['form']['data_final'] or ''
        if data_fim:
            data_fim = parse(data_fim).strftime('%d/%m/%Y')
        data_ini = datas['form']['data_ini'] or ''
        if data_ini:
            data_ini = parse(data_ini).strftime('%d/%m/%Y')
        state = datas['form']['state'] or ''
        validade_fim = datas['form']['validade_final'] or ''
        if validade_fim:
            validade_fim = parse(validade_fim).strftime('%d/%m/%Y')
        validade_ini = datas['form']['validade_ini'] or ''
        if validade_ini:
            validade_ini = parse(validade_ini).strftime('%d/%m/%Y')
        prev_aprov_fim = datas['form']['prev_aprov_final'] or ''
        if prev_aprov_fim:
            prev_aprov_fim = parse(prev_aprov_fim).strftime('%d/%m/%Y')
        prev_aprov_ini = datas['form']['prev_aprov_ini'] or ''
        if prev_aprov_ini:
            prev_aprov_ini = parse(prev_aprov_ini).strftime('%d/%m/%Y')

        # Argumentos da consulta
        args = []

        if empresa and isinstance(empresa, list):
            empresa_id, empresa = empresa
            args += [('company_id', '=', empresa_id)]

        if pessoa and isinstance(pessoa, list):
            pessoa_id, pessoa = pessoa
            args += [('pessoa_id', '=', pessoa_id)]

        if vendedor and isinstance(vendedor, list):
            vendedor_id, vendedor = vendedor
            args += [('vendedor_id', '=', vendedor_id)]

        args += [('state', '=', state)] if state else []
        args += [('create_date', '>=', data_ini)] if data_ini else []
        args += [('create_date', '<=', data_fim)] if data_fim else []
        args += [('validade', '>=', validade_ini)] if validade_ini else []
        args += [('validade', '<=', validade_fim)] if validade_fim else []
        args += [('previsao_aprovacao', '>=', prev_aprov_ini)] if prev_aprov_ini else []
        args += [('previsao_aprovacao', '<=', prev_aprov_fim)] if prev_aprov_fim else []
        orcamento_ids = obj_orcamento.search(cr, uid, args, context=context)

        pdf_file = StringIO.StringIO()
        listagem = Impressora()
        titulo = _('LISTAGEM DE ORCAMENTOS')[:180]
        det_format = '{:<15} {:<35} {:<15} {:^10} {:^10} {:^12} {:<15} {:<15} {:>15} {:<25} {:<13}'
        det_format = det_format.decode('utf-8')
        listagem.defineFormato(orientacao='1')
        listagem.informaNome(pdf_file, titulo)
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)
        listagem.defineFonte(tam=7)
        if state == '1E':
            state = _('Esboco')
        elif state == '2V':
            state = _('Validado')
        elif state == '3A':
            state = _('Aprovado')
        elif state == '4G':
            state = 'Pedido gerado'
        elif state == '5C':
            state = 'Cancelado'

        inf1 = '{:<50}                              {:<50}'.format(
            'Empresa: %s' % empresa.encode('utf-8'), 'Cliente: %s' % pessoa.encode('utf-8'))
        inf2 = '{:<50}                             {:<50}'.format(
            'Vendedor: %s' % vendedor.encode('utf-8'), 'Status: %s' % state.encode('utf-8'))
        inf3 = '{:<50}                             {:<50}'.format(
            'Data inicial: %s' % data_ini, 'Data final: %s' % data_fim)
        inf4 = '{:<50}                             {:<50}'.format(
            'Validade inicial: %s' % validade_ini, 'Validade final: %s' % validade_fim)
        inf5 = '{:<50}                             {:<50}'.format(
            _('Previsao') + ' aprovado inicial: %s' % prev_aprov_ini,
            _('Previsao') + ' aprovado final: %s' % prev_aprov_fim)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        listagem.informaCab('{:^180} {:>10}'.format(titulo, 'Pagina: ' + '{{ Page }}'))
        listagem.informaCab('')
        listagem.informaCab('{:<6} {:<10}'.format(_('Data:'), time.strftime('%d/%m/%Y')))
        listagem.informaCab(inf1)
        listagem.informaCab(inf2)
        listagem.informaCab(inf3)
        listagem.informaCab(inf4)
        listagem.informaCab(inf5)
        listagem.informaCab('=' * 190)
        listagem.informaCab(det_format.format(
            _('NUMERO'), _('CLIENTE'), _('TELEFONE'), _('DATA'), _('VALIDADE'), _('PREV. APROV.'),
            _('VENDEDOR'), _('AOS CUIDADOS'), _('VALOR TOTAL'), _('MOTIVO CANCELAMENTO'),
            _('STATUS')))
        listagem.informaCab('=' * 190)
        listagem.rodapePadrao()
        valor_total = 0
        status = {'1E': _('ESBOCO'), '2V': _('VALIDADO'), '3A': _('APROVADO'),
                  '5C': _('CANCELADO'), '4G': _('PEDIDO GERADO')}
        for brw_orcamento in obj_orcamento.browse(cr, uid, orcamento_ids, context=context):
            # Cliente
            cliente = brw_orcamento.pessoa_id and brw_orcamento.pessoa_id.name or \
                      brw_orcamento.nome_cliente or str()

            # Datas
            data_criacao = data_validade = data_aprovacao = str()
            if brw_orcamento.create_date:
                data_criacao = time.strftime(
                    "%d/%m/%Y", time.strptime(brw_orcamento.create_date[:10], "%Y-%m-%d"))
            if brw_orcamento.validade:
                data_validade = time.strftime(
                    "%d/%m/%Y", time.strptime(brw_orcamento.validade, "%Y-%m-%d"))
            if brw_orcamento.previsao_aprovacao:
                data_aprovacao = time.strftime(
                    "%d/%m/%Y", time.strptime(brw_orcamento.previsao_aprovacao, "%Y-%m-%d"))

            # Vendedor
            vendedor = brw_orcamento.vendedor_id and brw_orcamento.vendedor_id.name or str()

            # Motivo do cancelamento
            motivo = brw_orcamento.orccap_cancelamento_id and \
                     brw_orcamento.orccap_cancelamento_id.name or str()

            # Aos cuidados de...
            aos_cuidados = brw_orcamento.aos_cuidados or str()

            # Telefone
            telefone = brw_orcamento.pessoa_id and brw_orcamento.pessoa_id.telefone or str()

            listagem.imprimeLinha(det_format.format(
                brw_orcamento.documento[:18], cliente[:35], telefone, data_criacao, data_validade,
                data_aprovacao, vendedor[:15], aos_cuidados[:15],
                locale.format('%.2f', brw_orcamento.valor_liquido, grouping=True), motivo[:25],
                status.get(brw_orcamento.state, '')))
            listagem.imprimeLinha(190 * '-')
            valor_total += brw_orcamento.valor_liquido

        listagem.imprimeLinha(190 * '=')
        listagem.imprimeLinha('{:>134}{:>16}'.format(
            _('Total:'), locale.format('%.2f', valor_total, grouping=True)))
        listagem.imprimeLinha(190 * '=')
        listagem.finalizaImpressao()

        self.obj = external_pdf(pdf_file.getvalue())
        self.obj.render()
        pdf_file.close()

        return self.obj.pdf, 'pdf'

report_custom('report.listagens_de_orcamentos')


# noinspection PyAbstractClass,PyPep8Naming
class wizard_listagem_orcamento(osv.TransientModel):
    _name = "wizard.listagem.orcamento"
    _columns = {
        'company_id': fields.many2one("res.company", "Empresa", required=True),
        'pessoa_id': fields.many2one("adm.pessoa", "Cliente"),
        'vendedor_id': fields.many2one("vendas.vendedor", "Vendedor"),
        'data_ini': fields.date("Data Inicial"),
        'data_final': fields.date("Data Final"),
        'state': fields.selection(
            [('1E', 'ESBOCO'), ('2V', 'VALIDADO'), ('3A', 'APROVADO'), ('4G', 'PEDIDO GERADO'),
             ('5C', 'CANCELADO')], "Status"),
        'validade_ini': fields.date("Validade Inicial"),
        'validade_final': fields.date("Validade Final"),
        'prev_aprov_ini': fields.date("Previsao Aprovado Inicial"),
        'prev_aprov_final': fields.date("Previsao Aprovado Final"),
        }

    _defaults = {
        'company_id': lambda s, c, u, x: s.pool.get('res.users').browse(c, u, u).company_id.id
    }

    def check_report(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados = self.read(cr, uid, ids, [], context=context)

        if dados and isinstance(dados, list):
            dados = dados[0]

        if dados.get('data_ini') > dados.get('data_final'):
            raise osv.except_osv(_('Atencao'), _('Data inicial nao pode ser maior que a data '
                                                 'final'))
        if dados.get('validade_ini') > dados.get('validade_final'):
            raise osv.except_osv(_('Atencao'), _('Validade inicial nao pode ser maior que a '
                                                 'validade final'))
        if dados.get('prev_aprov_ini') > dados.get('prev_aprov_final'):
            raise osv.except_osv(_('Atencao'),
                                 _('Previsao aprovacao inicial nao pode ser maior que a previsao '
                                   'aprovacao final'))
        datas = {
            'ids': ids,
            'model': 'rat.rat',
            'form': dados
        }

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagens_de_orcamentos',
            'datas': datas,
            }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
