#-*- encoding: utf-8 -*-

from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
from openerp import pooler
import time
from openerp.osv import osv, fields
from openerp.tools.translate import _


_form = """<?xml version="1.0"?>
<form string="listagens_de_orcamentos">
    <field name="company_id" colspan="4" />
    <newline/>

    <field name="pessoa_id"/>
    
    <field name="vendedor_id"/>

    <field name="data_ini" />

    <field name="data_final" />

    <field name="state" />

    <field name="validade_ini" />

    <field name="validade_final" />

    <field name="prev_aprov_ini" />

    <field name="prev_aprov_final" />

</form>"""

_fields = {
    'company_id': {'string': 'Empresa', 'type': 'many2one', 'relation': "res.company", 'required': True},
    'pessoa_id': {'string': 'Cliente', 'type': 'many2one', 'relation': 'adm.pessoa'},
    'vendedor_id': {'string': 'Vendedor', 'type': 'many2one', 'relation': 'vendas.vendedor'},
    'data_ini': {'string': 'Data Inicial', 'type': 'date'},
    'data_final': {'string': 'Data Final', 'type': 'date'},
    'state': {'string': 'Status', 'type': 'selection',
              'selection': [('1E', 'ESBOCO'), ('3A', 'APROVADO'), ('cancelado', 'CANCELADO'),
                            ('gerado', 'GERADO PEDIDO'), ('todos', 'TODOS')], 'default': lambda *a: 'todos'},
    'validade_ini': {'string': 'Validade Inicial', 'type': 'date'},
    'validade_final': {'string': 'Validade Final', 'type': 'date'},
    'prev_aprov_ini': {'string': 'Previsao Aprovado Inicial', 'type': 'date'},
    'prev_aprov_final': {'string': 'Previsao Aprovado Final', 'type': 'date'},
}


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        pdf_file = StringIO.StringIO()
        empresa = datas['form']['company_id']
        pessoa = datas['form']['pessoa_id']
        vendedor = datas['form']['vendedor_id']
        data_fim = datas['form']['data_final']
        data_ini = datas['form']['data_ini']
        state = datas['form']['state']
        validade_fim = datas['form']['validade_final']
        validade_ini = datas['form']['validade_ini']
        prev_aprov_fim = datas['form']['prev_aprov_final']
        prev_aprov_ini = datas['form']['prev_aprov_ini']

        def center_title(E, C, D, T=200):
            title_c = (T - len(C)) / 2
            title = E
            for i in range(title_c - len(E)):
                title += ' '
            title += C
            for i in range(title_c - len(D)):
                title += ' '
            title += D
            return title

        obj_emp = pooler.get_pool(cr.dbname).get("res.company")
        nome_empresa = obj_emp.read(cr, uid, empresa, ['name'], context=context)['name']
        if not nome_empresa:
            raise osv.except_osv(_('Atencao'), _('Empresa nao selecionada!'))

        obj_req = pooler.get_pool(cr.dbname).get('orcamentos.orcamento.capa')

        argumentos = [('company_id', '=', empresa)]

        if state == 'todos':
            state = _('TODOS')
        if state == '1E':
            state = _('ESBOCO')
            argumentos.append(('state', '=', '1E'))
        if state == '3A':
            state = _('APROVADO')
            argumentos.append(('state', '=', '3A'))
        if state == '5C':
            state = _('CANCELADO')
            argumentos.append(('state', '=', '5C'))
        if state == '4G':
            state = _('GERADO PEDIDO')
            argumentos.append(('state', '=', '4G'))

        if data_ini != False and data_fim != False:
            data_ini = time.strftime("%d-%m-%Y", time.strptime(datas['form']['data_ini'], "%Y-%m-%d"))
            argumentos.append(('create_date', '>=', datas['form']['data_ini']))

            data_fim = time.strftime("%d-%m-%Y", time.strptime(datas['form']['data_final'], "%Y-%m-%d"))
            argumentos.append(('create_date', '<=', datas['form']['data_final']))

        if validade_ini != False and validade_fim != False:
            validade_ini = time.strftime("%d-%m-%Y", time.strptime(datas['form']['validade_ini'], "%Y-%m-%d"))
            argumentos.append(('validade', '>=', datas['form']['validade_ini']))

            validade_fim = time.strftime("%d-%m-%Y", time.strptime(datas['form']['validade_final'], "%Y-%m-%d"))
            argumentos.append(('validade', '<=', datas['form']['validade_final']))

        if prev_aprov_ini != False and validade_fim != False:
            prev_aprov_ini = time.strftime("%d-%m-%Y", time.strptime(datas['form']['prev_aprov_ini'], "%Y-%m-%d"))
            argumentos.append(('previsao_aprovacao', '>=', datas['form']['validade_ini']))

            prev_aprov_fim = time.strftime("%d-%m-%Y", time.strptime(datas['form']['prev_aprov_final'], "%Y-%m-%d"))
            argumentos.append(('previsao_aprovacao', '<=', datas['form']['prev_aprov_final']))

        ids_req = obj_req.search(cr, uid, argumentos, context=None)

        listagem = Impressora()
        listagem.defineFormato(orientacao='1')
        listagem.informaNome(pdf_file, _('LISTAGENS de ORCAMENTOS'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(1, 1, 1, 1)
        listagem.informaCab(center_title(_('LISTAGENS ORCAMENTOS'), nome_empresa, _('Pagina:  {{ Page }}')))
        listagem.informaCab(' ')
        listagem.informaCab(_('Data: ') + time.strftime('%d/%m/%Y'))
        listagem.informaCab(' ')
        listagem.informaCab(_('Status: '))
        listagem.informaCab(' ')
        listagem.informaCab('--')
        cab = formataString(_('NUMERO'), 18, ' ', 'D', 'S', 'S')
        cab += formataString(_('EMPRESA'), 26, ' ', 'D', 'S', 'S')
        cab += formataString(_('CLIENTE'), 41, ' ', 'D', 'S', 'S')
        cab += formataString(_('TELEFONE'), 15, ' ', 'D', 'S', 'S')
        cab += formataString(_('DATA'), 12, ' ', 'D', 'S', 'S')
        cab += formataString(_('VALIDADE'), 12, ' ', 'D', 'S', 'S')
        cab += formataString(_('PREV. APROV. '), 13, ' ', 'D', 'S', 'S')
        cab += formataString(_('VENDEDOR'), 15, '  ', 'D', 'S', 'S')
        cab += formataString(_('VALOR TOTAL'), 17, ' ', 'D', 'S', 'S')
        cab += formataString(_('STATUS'), 15, '  ', 'D', 'S', 'S')
        listagem.informaCab(cab)
        listagem.informaCab('--')
        listagem.informaCab(' ')
        listagem.rodapePadrao()

        for req in obj_req.browse(cr, uid, ids_req):
            linha = formataString(req.documento, 17, ' ', 'D', 'S', 'S')
            linha += ' '
            linha += formataString(req.company_id.name, 25, ' ', 'D', 'S', 'S')
            linha += ' '
            # Cliente
            if req.pessoa_id.name:
                linha += formataString(req.pessoa_id.name, 40, ' ', 'D', 'S', 'S')
            elif req.nome_cliente:
                linha += formataString(req.nome_cliente, 40, ' ', 'D', 'S', 'S')
            else:
                linha += formataString('-', 40, ' ', 'D', 'S', 'S')
            linha += ' '
            # Telefone
            if req.pessoa_id.telefone == None:
                linha += formataString('-', 14, ' ', 'D', 'S', 'S')
            else:
                linha += formataString(req.pessoa_id.telefone, 14, ' ', 'D', 'S', 'S')
            linha += ' '
            # Data
            linha += formataString(time.strftime('%d/%m/%Y'), 11, ' ', 'D', 'S', 'S')
            linha += ' '
            # Validacao
            if req.validade == False:
                linha += formataString('-', 11, ' ', 'D', 'S', 'S')
            else:
                data_validade = time.strftime("%d/%m/%Y", time.strptime(req.validade, "%Y-%m-%d"))
                linha += formataString(data_validade, 11, ' ', 'D', 'S', 'S')
            linha += ' '
            # aprovacao
            if req.previsao_aprovacao == False:
                linha += formataString('-', 12, ' ', 'C', 'S', 'S')
            else:
                data_aprovacao = time.strftime("%d/%m/%Y", time.strptime(req.previsao_aprovacao, "%Y-%m-%d"))
                linha += formataString(data_aprovacao, 12, ' ', 'D', 'S', 'S')
            linha += ' '
            # vendedor
            if req.vendedor_id.name == None:
                linha += formataString('-', 15, ' ', 'D', 'S', 'S')
            else:
                linha += formataString(req.vendedor_id.name, 15, ' ', 'D', 'S', 'S')
            linha += ' '
            # valor_liquido
            if req.valor_liquido == False:
                linha += formataString('-', 17, ' ', 'E', 'S', 'S')
            else:
                linha += formataString(getNumeroEditado(req.valor_liquido), 17, ' ', 'E', 'S', 'S')
            linha += ' '

            if req.state == 'todos':
                states = _('TODOS')
            elif req.state == '1E':
                states = _('ESBOCO')
            elif req.state == '3A':
                states = _('APROVADO')
            elif req.state == '5C':
                states = _('CANCELADO')
            elif req.state == '4G':
                states = _('GERADO PEDIDO')
            else:
                states = '-'
            linha += formataString(states, 13, ' ', 'E', 'S', 'S')
            linha += ' '

            listagem.imprimeLinha(linha)

        listagem.informaCab('--')
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_file.getvalue())
        self.obj.render()
        pdf_file.close()
        return (self.obj.pdf, 'pdf')


report_custom('report.listagens_de_orcamentos')
