# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class oservico_requisicao_itens(osv.Model):
    _name = 'oservico.requisicao.itens'
    _inherit = 'oservico.requisicao.itens'

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if not 'osvrqi_quantidade' in vals or vals.get('osvrqi_quantidade') <= 0:
            raise osv.except_osv(_('Atencao'), _('Valor invalido para a quantidade do item da '
                                                 'requisicao'))
        if vals.get('osvrqi_capa_id', False):
            brw_requisicao = self.pool.get('oservico.requisicao.capa').browse(cr, uid, vals['osvrqi_capa_id'],
                                                                              context=context)
            if brw_requisicao.state in ('N', 'C'):
                raise osv.except_osv(_('Atencao'), _('Nao posso alterar Requisicao cancelada ou Impressa.'))
            if brw_requisicao.osvrqc_os_id.state in ('C', 'F'):
                raise osv.except_osv(_('Atencao'), _('Nao posso alterar Requisicao de O.S Cancelada ou Fechada.'))
            result = super(oservico_requisicao_itens, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'osvrqi_quantidade' in vals and vals.get('osvrqi_quantidade') <= 0:
            raise osv.except_osv(_('Atencao'), _('Valor invalido para a quantidade do item da '
                                                 'requisicao'))
        if not isinstance(ids, list):
            ids = [ids]
        brw_requisicao = self.browse(cr, uid, ids, context=context)
        for x in brw_requisicao:
            if x.osvrqi_capa_id.state in ('N', 'C'):
                raise osv.except_osv(_('Atencao'), _('Nao posso alterar Requisicao cancelada ou Impressa.'))
            if x.osvrqi_capa_id.osvrqc_os_id.state in ('C', 'F'):
                raise osv.except_osv(_('Atencao'), _('Nao posso alterar Requisicao de O.S Cancelada ou Fechada.'))
        result = super(oservico_requisicao_itens, self).write(cr, uid, ids, vals, context)
        return result

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for brw_requisicao in self.browse(cr, uid, ids, context=context):
            if brw_requisicao.osvrqi_capa_id.state in ('N', 'C'):
                raise osv.except_osv(_('Atencao'), _('Nao posso alterar Requisicao cancelada ou Entregue.'))
            if brw_requisicao.osvrqi_capa_id.osvrqc_os_id.state in ('C', 'F'):
                raise osv.except_osv(_('Atencao'), _('Nao posso alterar Requisicao de O.S Cancelada ou Fechada.'))
        result = super(oservico_requisicao_itens, self).unlink(cr, uid, ids, context)
        return result

    def osvrqi_onchange_produto(self, cr, uid, ids, osvrqi_produto_id, osvrqi_grade_id, context=None):
        context = context and context.copy() or {}
        res = {'osvrqi_preco_unitario': 0, 'osvrqi_grade_id': ''}
        if osvrqi_produto_id:
            preco = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, osvrqi_produto_id, epc_grade_ids=osvrqi_grade_id)
            if preco:
                dados_preco_venda = preco.get(osvrqi_produto_id) or {}
                preco = dados_preco_venda.get(osvrqi_grade_id) or dados_preco_venda.get(
                    'preco_venda')
                res['osvrqi_preco_unitario'] = preco
            grade_ids = self.pool.get('estoque.produto.grade').search(cr, uid, [('produto_id', '=', osvrqi_produto_id)],
                                                                      context=context)
            if len(grade_ids) == 1:
                grade_id = grade_ids[0]
                res['osvrqi_grade_id'] = grade_id
        return {'value': res}

    def osvrqi_onchange_grade(self, cr, uid, ids, osvrqi_produto_id, osvrqi_grade_id, context=None):
        context = context and context.copy() or {}
        res = {'osvrqi_preco_unitario': 0}
        cliente_id = False
        if context.get('pessoa_id'):
            cliente_id = self.pool.get('financeiro.cliente').search(
                cr, uid, [('pessoa_id', '=', context.get('pessoa_id'))], context=context)
            if cliente_id:
                cliente_id = cliente_id[0]
        if osvrqi_produto_id and osvrqi_grade_id:
            preco = self.pool.get('estoque.preco.venda').get_preco_venda(
                cr, uid, osvrqi_produto_id, epc_grade_ids=osvrqi_grade_id,
                epc_cliente_id=cliente_id)
            if preco:
                dados_preco_venda = preco.get(osvrqi_produto_id) or {}
                preco = dados_preco_venda.get(osvrqi_grade_id) or dados_preco_venda.get(
                    'preco_venda')
                res['osvrqi_preco_unitario'] = preco
        return {'value': res}

    def osvrqi_onchange_quantidade(self, cr, uid, ids, local_id, produto_id, grade_id,
                                   quantidade, context=None):
        context = context and context.copy() or {}
        res = {}
        quantidade_valida = self._osvrqi_valida_quantidade(cr, uid, local_id, produto_id, grade_id,
                                                           quantidade, context)
        if not quantidade_valida:
            res['warning'] = {'title': _('Atencao'),
                              'message': _('Esta unidade nao aceita valor fracionado')}
            res['value'] = {'osvrqi_quantidade': 0.0}
        return res

    def _osvrqi_valida_quantidade(self, cr, uid, local_id, produto_id, grade_id, quantidade,
                                  context=None):
        context = context and context.copy() or {}
        res = True
        if isinstance(quantidade, int):
            return res
        if not all([local_id, produto_id, grade_id]):
            return res
        args = [('local_id', '=', local_id), ('produto_id', '=', produto_id),
                ('grade_id', '=', grade_id)]
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        locacao_ids = obj_estoque_local_locacao.search(cr, uid, args, context=context)
        if not locacao_ids:
            return res
        produto_embalagem_id = obj_estoque_local_locacao.browse(
            cr, uid, locacao_ids[0]).estllc_produto_embalagem_id
        if not produto_embalagem_id:
            return res
        fracao = produto_embalagem_id.embalagem_id.unidade_id.fracao
        if isinstance(quantidade, float) and not fracao:
            res = False
        return res
