# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import time


class oservico_os(osv.Model):
    from funcoes import _osvos_get_total, _osvos_get_total_servicos, _osvos_get_total_pecas, \
        _osvsrv_verifica_veiculo_instalado, _osvos_get_adm_mensagem_padrao

    _name = 'oservico.os'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _description = 'Ordens de Servico'
    _rec_name = 'osvos_numero'
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', required=True, readonly=True, select=True),
        'osvos_numero': fields.char('Numero da OS', size=11, readonly=True, select=True),
        'osvos_data_abertura': fields.datetime('Data da abertura', required=True, readonly=True, select=True),
        'osvos_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', required=True, select=True), # Cliente
        'osvos_telefone': fields.many2one('adm.pessoa.telefone', 'Telefone', select=True),
        'osvos_data_fechamento': fields.datetime('Data do fechamento', readonly=True, select=True),
        'osvos_fechamento_user_id': fields.many2one('res.users', 'Usuario que fechou', readonly=True),
        'osvos_data_entrega': fields.date('Data da entrega'),
        'osvos_data_previsao': fields.date('Previsao de Entrega', select=True),
        'osvos_execucao': fields.selection([('I', 'Interna'), ('E', 'Externa')], 'Execucao', required=True, size=1),
        'osvos_defeito_relatado': fields.text('Defeito relatado', required=True), # Defeito informado pelo cliente\n
        'osvos_defeito_constatado': fields.text('Defeito constatado'), # Defeito identificado pelo tecnico/mecanico
        'osvos_observacoes': fields.text('Informacoes complementares',
                                         help="Observacoes ou outras informacoes complementares"),
        'osvos_garantia': fields.boolean('Garantia', help='Informa se a OS e de garantia(nao sera cobrado)'),
        'state': fields.selection([('E', 'Esboco'), ('A', 'Aberta'), ('C', 'Cancelada'), ('F', 'Fechada')], 'Estado',
                                  size=1), # Estado da OS E=Esboco, A=Aberta, C=Cancelada, F=Fechada
        'osvos_desconto_servico': fields.float('Desconto no servico', digits=(8, 2)),
        'osvos_desconto_pecas': fields.float('Desconto em pecas', digits=(8, 2)),
        'osvos_valor_total': fields.function(_osvos_get_total, type='float', method=True, string='Total O.S.',
                                             store=False),
        'osvos_mensagem_padrao': fields.function(_osvos_get_adm_mensagem_padrao, method=True,
                                                 string='Mensagem padrao', type='text',
                                                 store=False),
        'osvos_requisicao_ids': fields.one2many('oservico.requisicao.capa', 'osvrqc_os_id', 'Requisicoes'),
        'osvos_servicos_ids': fields.one2many('oservico.servicos', 'osvsrv_os_id', 'Servicos'),
        'osvos_pecas_ids': fields.one2many('oservico.requisicao.pgview', 'os_id', 'Pecas'),
        'osvos_condicao_ids': fields.one2many('oservico.os.condicao', 'osvocd_capa_id', 'Condicoes'),
        'osvos_parcela_ids': fields.one2many('oservico.os.parcela', 'osvocp_os_id', 'Parcela'),
        'osvos_total_servicos': fields.function(_osvos_get_total_servicos, type='float', method=True,
                                                string='Total Servicos', store=False),
        'osvos_total_pecas': fields.function(_osvos_get_total_pecas, type='float', method=True, string='Total Pecas',
                                             store=False),
        'osvos_equip_serie': fields.char('Nro Serie', size=30, select=True),
        'osvos_equip_ano': fields.integer('Ano'),
        'osvos_chamado': fields.integer('Chamado'),
        'osvos_descricao_equipamento': fields.char("Equipamento", size=60),
        'osvos_veiculo_instalado': fields.function(_osvsrv_verifica_veiculo_instalado, method=True,
                                                   string="Veiculo instalado", type="boolean",
                                                   store=False),
        'osvos_categoria_id': fields.many2one('oservico.categoria', 'Categoria',
                                              select=True, ondelete="restrict"),
        'osvos_faturamento_sob_demanda': fields.boolean('Faturamento sob demanda',
                                                        help='Se marcado, ao gerar pedido, armazena'
                                                             ' o total do apontamento multiplicado '
                                                             'pelo valor da hora.'),
        'osvos_setor_id': fields.many2one('oservico.setor', 'Setor', ondelete='cascade',
                                          select=True),
        'osvos_centro_custo_id': fields.many2one(
            'oservico.centro.custo', 'Centro de custos', ondelete='cascade', select=True,
            domain="[('osvcco_setor_id', '=', osvos_setor_id)]", write=False),
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'osvos_execucao': lambda *a: 'I',
        'osvos_garantia': lambda *a: False,
        'osvos_data_abertura': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'osvos_desconto_pecas': lambda *a: 0,
        'osvos_desconto_servico': lambda *a: 0,
        'state': lambda *a: 'E',
        'osvos_faturamento_sob_demanda': lambda *a: False,
    }
    _order = 'osvos_numero desc'

    _sql_constraints = [
        ('osvos_numero_uniq', 'unique (company_id,osvos_numero)', 'O Numero da OS precisa ser unico!'),
    ]
