# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp import pooler
import time
from openerp.tools.translate import _


class patrimonio_bens(osv.Model):
    """
    Cadastro de bens - Prefixo: patbem_
    """
    _name = 'patrimonio.bens'
    _inherit = 'patrimonio.bens'

    def onchange_grupo(self, cr, uid, ids, grupo_digitado_id, context=None):
        context = context and context.copy() or {}
        if not grupo_digitado_id:
            return {'value': {}}
        oPar = self.pool.get('patrimonio.grupos')
        _percentual = oPar.read(cr, uid, [grupo_digitado_id], ['percentual', ], context=context)[0]['percentual']
        warning = {}
        value = {}
        if ids:
            grupo_atual_id = self.read(cr, uid, ids, fields=['grupos_id', ], context=context)
            grupo_atual_id = grupo_atual_id[0]['grupos_id'][0]
            if grupo_atual_id == grupo_digitado_id:
                return {'warning': warning}
            warning = {'title': _('Atencao')}
            warning['message'] = _('O percentual deste grupo e diferente!\n Percentual:') + str(
                _percentual) + _('\n Verifique a possibilidade de alteracao!')
        else:
            value['percentual'] = _percentual
        return {'value': value, 'warning': warning}

    def onchange_data_aquisicao(self, cr, uid, ids, data_aquisicao, context=None):
        context = context and context.copy() or {}
        value = {}
        warning = {}
        if not ids:
            value['data_implantacao_saldo'] = data_aquisicao
        ano = data_aquisicao[:4]
        mes = data_aquisicao[5:7]
        resMoeda = self.pool.get('adm.moeda.valor').achar_valor_moeda(cr, uid, 'UPF', ano, mes, operador='=',
                                                                      vlDefault=1.00, context=context)
        if resMoeda['status'] == False:
            warning = {'title': _('Atencao'), 'message': resMoeda['mensagem']}
        return {'value': value, 'warning': warning}

    def onchange_valor_aquisicao(self, cr, uid, ids, valor_aquisicao):
        value = {}
        if not ids:
            value['valor_a_depreciar'] = valor_aquisicao
        return {'value': value}

    def create(self, cr, uid, vals, context=None):
        # def _patbem_create_patrimonio_fiscal(cr, uid, vals, result):
        #     """
        #     @param cr: {cursor} a linha atual do cursor do banco de dados,
        #     @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        #     @param vals: {dict} os campos e seus respectivos valores,
        #     @param context: {dict} um dicionário padrao para valores contextuais,
        #     @return:
        #     """
        #     # ___________________________ Objects ________________________________
        #     obj_patrimoniofiscal_ciap = self.pool.get('patrimoniofiscal.ciap')
        #     if all(('data_implantacao_saldo' in vals,  'credito_icms' in vals,
        #             'qtd_parcelas' in vals, vals['credito_icms'] > 0)):
        #         obj_patrimoniofiscal_ciap.criar_ciap(cr, uid, result,
        #                                              vals['data_implantacao_saldo'],
        #                                              vals['credito_icms'],
        #                                              vals['qtd_parcelas'])
        #     return True

        context = context and context.copy() or {}
        data_aquisicao = vals.get('data_aquisicao', False)
        if data_aquisicao:
            resMoeda = self.pool.get('adm.moeda.valor').achar_valor_moeda(cr, uid, 'UPF', data_aquisicao[:4],
                                                                          data_aquisicao[5:7], operador='=',
                                                                          vlDefault=1.00, context=context)
            if not resMoeda['status']:
                raise osv.except_osv(_('Atencao'), (resMoeda['mensagem']))
        result = super(patrimonio_bens, self).create(cr, uid, vals, context=context)
        if 'localizacao_id' in vals:
            if vals['localizacao_id']:
                log = {'bem_id':result, 'localizacao_id': vals['localizacao_id']}
                self.pool.get('patrimonio.bens.historico.local').create(cr, uid, log, context)
        # if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'patrimonio_fiscal', context):
        #      _patbem_create_patrimonio_fiscal(cr, uid, vals, result)
        if 'codigo_bem' not in vals or vals.get('codigo_bem') <= 0:
            raise osv.except_osv(_('Atencao'), _('Valor invalido para codigo do bem'))
        return result

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('codigo_bem') and vals.get('codigo_bem') <= 0:
            raise osv.except_osv(_('Atencao'), _('Valor invalido para codigo do bem'))
        result = super(patrimonio_bens, self).write(cr, uid, ids, vals, context)
        if 'localizacao_id' in vals:
            if vals['localizacao_id']:
                bem_id = ids[0] if isinstance(ids,list) else ids
                args = [('bem_id', '=', bem_id), ('data_final', '=',False)]
                logObj = self.pool.get('patrimonio.bens.historico.local')
                logIds = logObj.search(cr, uid, args, context=context)
                logObj.write(cr, uid, logIds, {'data_final': time.strftime('%Y-%m-%d')}, context)
                log = {'bem_id': bem_id, 'localizacao_id': vals['localizacao_id']}
                logObj.create(cr, uid, log, context)
        return result

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        context['sig_search_sem_filhos'] = True
        result = super(patrimonio_bens, self).search(cr, uid, args, offset, limit, order, context, count)
        return result

    def patbem_onchange_codigo_bem(self, cr, uid, ids, codigo_bem, context=None):
        context = context and context.copy() or {}
        res = {}
        if not codigo_bem:
            return res
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        patrimonio_bem_ids = self.search(cr, uid, [('codigo_bem', '=', codigo_bem),
                                                   ('company_id', '=', company_id),
                                                   ('id', 'not in', ids)], context=context)
        if patrimonio_bem_ids:
            res['warning'] = {'title': _('Atencao'), 'message': _('Codigo ja cadastrado para outro '
                                                                  'bem')}
            res['value'] = {'codigo_bem': False}
        return res

    def name_search(self, cr, uid, name='', args=[], operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        if name.isdigit():
            ids = self.search(cr, uid, [('codigo_bem', '=', int(name))] + args, limit=limit,
                              context=context)
        else:
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit,
                              context=context)
        return self.name_get(cr, uid, ids, context)

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        brw_patrimonio_bens = self.browse(cr, uid, ids, context)
        for bem in brw_patrimonio_bens:
            res.append((bem.id, '%s - %s' % (bem.codigo_bem, bem.name)))
        return res
