# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import patbem_get_qtd_parcelas
from funcoes import patben_verifica_item_baixado

TIPO_EMITENTE = [('0', 'Emissao propria'), ('1', 'Terceiros')]
TIPO_UTILIZACAO = [('1', 'Comercio - Area operacional'),
                   ('2', 'Comercio - Area administrativa'),
                   ('3', 'Industria - Area produtiva'),
                   ('4', 'Industria - Area apoio a producao'),
                   ('5', 'Industria - Area administrativa')]


class patrimonio_bens(osv.Model):
    """
    Classe patrimonio_bens

    @change: 30.04.2013 - pierre.barbosa

    prefixo: patben_
    """

    _name = 'patrimonio.bens'
    _roteavel = True



    def get_uso_bem_id(self, cr, uid, ids, field_name, args, context=None):
        context = context and context.copy() or {}
        res = {}
        if 'view_patrimonio_bens_search' in context: # é executado somente na pesquisa por agrupamento
            for id in ids:
                pat_uso_id = self.pool.get('patrimonio.uso').search(cr, uid, [('patuso_bens_id','=', id),
                                                                              ('patuso_data_final', '=', None)],
                                                                    context=context)
                if pat_uso_id:
                    if isinstance(pat_uso_id, list):
                        pat_uso_id=pat_uso_id[0]
                    bwr_pat_uso = self.pool.get('patrimonio.uso').browse(cr, uid, pat_uso_id)
                    res[id] = bwr_pat_uso.patuso_funcionario_id.id
                else:
                    res[id] = None
        else:
            for brw_bem in self.browse(cr, uid, ids):
                patuso_uso_ids = brw_bem.patuso_uso_ids
                if patuso_uso_ids:
                    for paruso in patuso_uso_ids:
                        if not paruso.patuso_data_final:
                            funcionario_id = paruso.patuso_funcionario_id.id
                            res[brw_bem.id] = funcionario_id
                        else:
                            res[brw_bem.id] = None
                else:
                    res[brw_bem.id] = None
        return res

    _columns = {
        # ___________________________ Integer ________________________________
        'id': fields.integer('id'),
        'codigo_bem': fields.integer('Codigo do bem', required=True, # Codigo que identifica o bem
                                     select=True),
        'numero_nf': fields.integer('Numero da nf'), # Numero da NF de aquisicao
        'percentual': fields.integer('Percentual', # Percentual de depreciacao do bem
                                     help='Percentual anual de depreciacao do bem'),
        'sequencia_item_nf': fields.integer('Sequência do item na nf'), # Sequencia do item na nf de entrada
        'qtd_parcelas': fields.integer('Qtd de parcelas'), # Quantidade de parcelas para apropriacao

        # ___________________________ One2Many _______________________________
        'patuso_uso_ids': fields.one2many('patrimonio.uso', 'patuso_bens_id',
                                          'Uso'),
        'custo_complementar_ids': fields.one2many('patrimonio.custos.complementares',
                                                  'bem_id',
                                                  'Custos complementares'),
        'log_localizacao_ids': fields.one2many('patrimonio.bens.historico.local',
                                               'bem_id',
                                               'Historico de localizacoes'),
        'movimento_ids': fields.one2many('patrimonio.movimento', 'bem_id',
                                         'Movimento'),
        'bens_ids': fields.one2many('patrimonio.bens', 'parent_id',
                                    'Bens'),
        'manutencoes_ids': fields.one2many('patrimonio.manutencoes', 'bens_id',
                                           'Manutencoes'),

        # ___________________________ Many2One _______________________________
        'fornecedor_id': fields.many2one("adm.pessoa", 'Fornecedor', # ID do fornecedor do Bem
                                         ondelete="restrict", select=True),
        'company_id': fields.many2one("res.company", 'Empresa',
                                      ondelete="cascade", required=True, # ID da empresa
                                      select=True),
        'localizacao_id': fields.many2one("patrimonio.localizacao", 'Localizacao', # Codigo de identificacao do Bem
                                          ondelete="restrict", select=True),
        'parent_id': fields.many2one("patrimonio.bens", 'Patrimonio Pai', # Codigo do Bem principal
                                     ondelete="restrict", select=True),
        'grupos_id': fields.many2one("patrimonio.grupos", 'Grupo', # Grupo ao qual o Bem pertence
                                     ondelete="restrict", required=True,
                                     select=True),

        # ___________________________ Date ___________________________________
        'data_baixa': fields.date('Data da baixa'), # Data de baixa do Bem
        'data_aquisicao': fields.date('Data de aquisicao'), # Data de aquisicao do Bem
        'data_inicio_depreciacao': fields.date('Data de inicio da depreciacao'), # Data de inicio de depreciacao
        'data_doc': fields.date('Data do documento'), # Data do documento de entrada
        'data_garantia': fields.date('Data de garantia', # Data final da garantia do Bem
                                     help="Data do termino da garantia"),
        'data_implantacao_saldo': fields.date('Data da implantacao', # Data da implantacao do saldo
                                              required=True),

        # ___________________________ Char ___________________________________
        'numero_serie': fields.char('Numero de Serie', size=30),
        'serie_nf': fields.char('Serie da NF', size=3),
        'chave_nfe': fields.char('Chave da NFe', size=44,
                                 help='Chave da NFe de entrada'),
        'modelo_nf': fields.char('Modelo da NF', size=2),
        'codigo_item': fields.char('Codigo do item no estoque', size=15),

        # ___________________________ Text ___________________________________
        'name': fields.text('Descricao do bem', required=True),
        'observacoes': fields.text('Observacoes'), # Observacoes complementares
        'descricao_funcao': fields.text('Descricao da funcao',
                                        help="Descricao da funcao do bem na atividade do estabelecimento.\nEsta descricao sera informada no SPED."),

        # ___________________________ Float __________________________________
        'valor_a_depreciar': fields.float('Valor a depreciar', digits=(12, 2)),
        'depreciacao_acumulada': fields.float('Depreciacao acumulada',
                                              digits=(12, 2)),
        'credito_icms': fields.float('Credito de ICMS', digits=(12, 2),
                                     help="Valor do credito do ICMS a ser apropriado!"),
        'valor_aquisicao': fields.float('Valor de aquisicao', digits=(12, 2),
                                        required=True),

        # ___________________________ Selection ______________________________
        'emitente': fields.selection(TIPO_EMITENTE, 'Emitente da nf', size=1),
        # Identifica o emitente do documento fiscal
        'area_utilizacao': fields.selection(TIPO_UTILIZACAO, 'Area de utilizacao', # Area de utilizacao do bem
                                            size=1),
        #____________________________ Function _______________________________
        'uso_bem_id': fields.function(get_uso_bem_id, method=True, string='Uso', type='many2one',relation='rh.funcionario', store=True), # id do funcionario que esta com o bem
        'patben_baixado': fields.function(patben_verifica_item_baixado, method=True,
                                          string='Baixado', type='boolean', store=True),


    }
    def _check_recursion(self, cr, uid, ids, context=None, parent=None):
        context = context and context.copy() or {}
        res = super(patrimonio_bens, self)._check_recursion(cr, uid, ids, context, parent)
        return res

    _sql_constraints = [
        ('codigo_bem_uniq', 'unique (company_id, codigo_bem)', 'O codigo do bem precisa ser unico na empresa.')
    ]

    _constraints = [
        (_check_recursion, 'Error! You cannot create recursive directories.', ['parent_id'])
    ]

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'qtd_parcelas': patbem_get_qtd_parcelas,
        'patben_baixado': lambda *a: False,
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4

