# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import osv
from openerp.tools.translate import _

# __________________________ patrimonio.bens _________________________________
def patbem_get_qtd_parcelas(self, cr, uid, ids, context=None):
    """
    Numero de parcelas

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {int} o numero de parcelas ou mensagem de erro.
    """

    context = context and context.copy() or {}

    if isinstance(ids, (int, long)):
        ids = [ids]

    # __________________________ Objects _____________________________________
    obj_parametro = self.pool.get('adm.parametro')

    parametro_ids = obj_parametro.search(cr, uid, [], context=context)

    if parametro_ids:
        campos = ['admpar_pat_qtd_parcelas']

        parametro_dados = obj_parametro.read(cr, uid, parametro_ids, campos, context=context)

        if parametro_dados and isinstance(parametro_dados, list):
            parametro_dados = parametro_dados[0]

        if parametro_dados and isinstance(parametro_dados, dict):
            return parametro_dados.get('admpar_pat_qtd_parcelas') or 0
        else:
            return 0
    else:
        raise osv.except_osv(_('Atencao'), _('Parametro da Administracao nao cadastrado.'))


def patbem_verifica_recursao(self, cr, uid, ids, context=None):
    """
    Metodo para Impedir o cadastramento de bens recursivos

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {boolean} Verdadeiro se houver recursividade.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if isinstance(ids, (int, long)):
        ids = [ids]

    level = 100

    while len(ids):
        _ids = ', '.join(map(str, ids))
        _sql = "select distinct parent_id from patrimonio_bens where id in (%s)"
        cr.execute(_sql % _ids)

        ids = filter(None, map(lambda x: x[0], cr.fetchall()))

        if not level:
            return False

        level -= 1

    return True


def patben_verifica_item_baixado(self, cr, uid, ids, field_name, args, context=None):
    """
    O método verifica se há 'data_baixa' para cada registro recebido, setando True para sim e False
    para não ao atributo 'patben_baixado'.
    :param cr: cursor no banco de dados
    :param uid: id da tabela res_users, identificador do usuário
    :param ids: lista de id da tabela patrimonio_bens
    :param field_name: string do nome do atributo/campo
    :param args: lista de argumentos
    :param context: dicionário com valores contextuais
    :return: dicionário com chave e valor representado o id do bem e um booleano, respectivamente.
    """
    context = context and context.copy() or {}
    res = {}
    brw_patrimonio_bens = self.browse(cr, uid, ids, context)
    for bem in brw_patrimonio_bens:
        if bem.data_baixa:
            res[bem.id] = True
        else:
            res[bem.id] = False
    return res


# __________________________ patrimonio.grupos _______________________________
def patgrp_verifica_vida_util(self, cr, uid, ids, context=None):
    """
    Verifica se a quantidade de meses informada para vida util do bem e inferior
    a um mes.

    @param cr:
    @param uid:
    @param ids:
    @param context:
    @return:

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    @since: 13.05.2013
    """

    context = context and context.copy() or {}

    if isinstance(ids, (int, long)):
        ids = [ids]

    for grupo in self.browse(cr, uid, ids):
        if grupo.vida_util < 1:
            return False

    return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
