# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class patrimonio_grupos(osv.Model):
    """
    Tabela de Grupos de Bens.

    @change: 13.05.2013 - pierre.barbosa
    """
    _name = 'patrimonio.grupos'                                                     # patgrp

    from funcoes import patgrp_verifica_vida_util
    _roteavel = True
    _columns = {
        # ___________________________ Many2One _______________________________
        'company_id': fields.many2one("res.company", 'Empresa',                     # ID da empresa
                                      ondelete="cascade", required=True,
                                      select=True),
        'conta_depreciacao_id': fields.many2one("contabil.conta",                   # ID da con ta de depreciacao
                                                'Conta depreciacao',
                                                ondelete="restrict",
                                                select=True),
        'conta_depreciacao_acumul_id': fields.many2one("contabil.conta",            # Conta de depreciacao acumulada
                                                       'Conta depreciacao acumulada',
                                                       ondelete="restrict",
                                                       select = True),
        'conta_aquisicao_id': fields.many2one("contabil.conta",                     # Conta de aquisicao
                                              'Conta aquisicao',
                                              ondelete="restrict",
                                              select=True),
        'conta_custo_baixa': fields.many2one("contabil.conta",                      # Conta para custo de baixa
                                             'Conta custo baixa',
                                             ondelete="restrict",
                                             select=True),

        # ___________________________ One2Many _______________________________
        'bens_ids': fields.one2many('patrimonio.bens', 'grupos_id', 'Bens'),

        # ___________________________ Char ___________________________________
        'sigla': fields.char('Sigla', size=3, required=True, select=True),          # Sigla para identificacao do grupo de bens
        'name': fields.char('Descricao', size=40, required=True),                   # Descricao do Grupo de bens

        # ___________________________ Float __________________________________
        'percentual': fields.float('Percentual',                                    # Percentual de depreciacao
                                   help='Percentual anual de depreciacao',
                                   digits=(10, 0)),

        # ___________________________ Integer ________________________________
        'vida_util': fields.integer('Vida util',                                    # Vida util do bem em numero de meses
                                    help='Vida util do bem em meses'),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
        'vida_util': 12,                                                             # Pardrao = 1 ano = 12 meses
    }

    _sql_constraints = [('sigla_uniq', 'unique (company_id, sigla)',
                         'A sigla precisa ser unica na empresa!')]

    _constraints = [(patgrp_verifica_vida_util,
                     'A vida util de um bem nao pode ser inferior a um mes.',
                     ['Vida util'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
