# -*- encoding: utf-8 -*-

from openerp.report.interface import report_int
from openerp.report.render import render
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
import StringIO
from openerp import pooler
import time
from SigUtils.formatastring import formataString
from openerp.osv import osv, fields
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        def center_title(E, C, D, T=160):
            title_c = (T - len(C)) / 2
            title = E
            for i in range(title_c - len(E)):
                title += ' '
            title += C
            for i in range(title_c - len(D)):
                title += ' '
            title += D
            return title

        context = context and context.copy() or {}
        title = center_title('', _('BAIXA DE PATRIMONIO'), '')
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Listagem de Baixa de Patrimonio'))
        listagem.informaTamLinha(160)

        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                    'wizard.listagem.item.tributacao', context=context)
        nome_empresa = pooler.get_pool(cr.dbname).get('res.company').browse(cr, uid, company_id).name
        data = time.strftime("%d/%m/%Y")
        docs = center_title(data, nome_empresa, _('Pagina: {{ Page }} '))

        data_inicial = datas.get('data_inicial')
        data_final = datas.get('data_final')

        obj_patrimonio_bens = pooler.get_pool(cr.dbname).get('patrimonio.bens')
        obj_patrimonio_ciap = pooler.get_pool(cr.dbname).get('patrimoniofiscal.ciap')
        patrimonio_ids = obj_patrimonio_bens.search(cr, uid, [('data_baixa', '>=', data_inicial),
                                                              ('data_baixa', '<=', data_final)], context=context)

        listagem.informaCab('' + title)
        listagem.informaCab('')
        listagem.informaCab('')
        listagem.informaCab('' + docs)
        listagem.informaCab('  ')
        listagem.informaCab(
            _(' Periodo: ') + time.strftime("%d/%m/%Y", time.strptime(data_inicial, "%Y-%m-%d")) + ' a ' + time.strftime(
                "%d/%m/%Y", time.strptime(data_final, "%Y-%m-%d")))
        listagem.informaCab(
            '|-----------------------------------------------------------------------------------------------------------------------------------------------------------|')
        listagem.informaCab(
            _('|   CODIGO   |                       DESCRICAO                        |   DATA    |   VALOR   |   CREDITO   |  DATA DA  |  ICMS A   |    ICMS    |   ICMS   |'))
        listagem.informaCab(
            _('| PATRIMONIO |                       PATRIMONIO                       | AQUISICAO | AQUISICAO |     ICMS    |   BAIXA   | APROPRIAR | APROPRIADO | RESTANTE |'))
        listagem.informaCab(
            '|-----------------------------------------------------------------------------------------------------------------------------------------------------------|')
        listagem.informaCab(
            '|                                                                                                                                                           |')
        listagem.rodapePadrao()

        if patrimonio_ids:
            dados = obj_patrimonio_bens.browse(cr, uid, patrimonio_ids, context)

            for i in dados:

                ciap_ids = obj_patrimonio_ciap.search(cr, uid, [('bem_id', '=', i.id)], offset=0, limit=None,
                                                      order='parcela', context=context)
                dados_ciap = obj_patrimonio_ciap.browse(cr, uid, ciap_ids)

                ano_baixa = i.data_baixa[:4]
                mes_baixa = i.data_baixa[5:7]

                icms_a_apropriar = 0
                icms_apropriado = 0
                icms_restante = 0

                antes_baixa = True

                for x in dados_ciap:
                    if antes_baixa:
                        icms_a_apropriar += x.valor_apropriacao
                        icms_apropriado += x.valor_apropriacao * x.ciap_controle_id.coeficiente
                    if not antes_baixa:
                        icms_restante += x.valor_apropriacao
                    if ano_baixa == x.ano and mes_baixa == x.baixa:
                        antes_baixa = False

                linha = '|'
                linha += formataString(i.codigo_bem or 0, 12, ' ', 'E', 'S', 'S')
                linha += '|'
                linha += formataString(i.name or '', 56, ' ', 'D', 'S', 'S')
                linha += '|'
                linha += formataString(time.strftime("%d/%m/%Y", time.strptime(i.data_aquisicao, "%Y-%m-%d")) or '', 11,
                                       ' ', 'D', 'S', 'S')
                linha += '|'
                linha += formataString(getNumeroEditado(i.valor_aquisicao or 0.0, 2), 11, ' ', 'E', 'S', 'S')
                linha += '|'
                linha += formataString(getNumeroEditado(i.credito_icms or 0.0, 2), 13, ' ', 'E', 'S', 'S')
                linha += '|'
                linha += formataString(time.strftime("%d/%m/%Y", time.strptime(i.data_baixa, "%Y-%m-%d")) or '', 11,
                                       ' ', 'D', 'S', 'S')
                linha += '|'
                linha += formataString(getNumeroEditado(icms_a_apropriar or 0.0, 2), 11, ' ', 'E', 'S', 'S')
                linha += '|'
                linha += formataString(getNumeroEditado(icms_apropriado or 0.0, 2), 12, ' ', 'E', 'S', 'S')
                linha += '|'
                linha += formataString(getNumeroEditado(icms_restante or 0.0, 2), 10, ' ', 'E', 'S', 'S')
                linha += '|'

                listagem.imprimeLinha(linha)

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return (self.obj.pdf, 'pdf')


report_custom('report.listagem_baixa_patrimonio')
