# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.addons.adm import messagebox
from openerp.tools.translate import _


class wizard_depreciacao_bem(osv.TransientModel):
    """
    Calculo de depreciacao de bens.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    @since: 13.05.2013
    """

    _name = 'wizard.depreciacao.bem'                                                # wizdep
    _inherit = 'wizard.depreciacao.bem'                                             # patrimonio/wizard/depreciacao_bem_data

    def wizdep_button_calcular(self, cr, uid, ids, context=None):
        """
        Calcula depreciacao dos bens.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return:

        @change: 13.05.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}
        from calendar import monthrange

        # ___________________________ Objects ________________________________
        obj_depreciacao_bens = self.pool.get('wizard.depreciacao.bem')
        obj_patrimonio_bens = self.pool.get('patrimonio.bens')
        obj_patrimonio_movimento = self.pool.get('patrimonio.movimento')
        obj_adm_parametro = self.pool.get('adm.parametro')

        campos = ['company_id', 'ano', 'mes']
        dados_bem = obj_depreciacao_bens.read(cr, uid, ids, campos, context=context)
        if dados_bem and isinstance(dados_bem, list):
            dados_bem = dados_bem[0]
        msg = _('Nao foi possivel calcular a depreciacao')
        if dados_bem and isinstance(dados_bem, dict):
            company_id = dados_bem.get('company_id')[0]
            ano = dados_bem.get('ano') or False
            mes = '{0:02d}'.format(int(dados_bem.get('mes') or False))

            # Datas parametro para calculo da depreciacao.
            data_inicio = '%d-%02d-%02d' % (int(ano), 1, 1)
            data_fim = '%d-%02d-%02d' % (int(ano), int(mes),
                                         monthrange(int(ano), int(mes))[1])

            args = [('company_id', '=', company_id),
                    ('data_aquisicao', '>=', data_inicio),
                    ('data_aquisicao', '<=', data_fim)]

            bem_ids = obj_patrimonio_bens.search(cr, uid, args, context=context)

            # Todos os bens da empresa adquiridos no periodo
            for bem in obj_patrimonio_bens.browse(cr, uid, bem_ids):
                # TODO: Ver a possibilidade de verificar tambem o mes em ´´args[]´´
                if bem.data_baixa and bem.data_baixa[
                                      5:7] == mes:                   # Ignora bens com baixa no mesmo mes do calculo
                    continue

                # Depreciacoes anteriores
                depreciacao_anterior_mes = 0                                        # Percentual de depreciacao anterior utilizada para correcao
                depreciacao_id = False                                              # ID do movimento anterior

                for movimento in bem.movimento_ids:                                 # Movimentos do Bem
                    if movimento.mes == mes and movimento.ano == ano:
                        depreciacao_id = movimento.id
                        depreciacao_anterior_mes = movimento.depreciacao_mes        # Antiga depreciacao no mes
                        break

                depreciacao_atual_mes = bem.percentual / 12 * \
                                        bem.valor_aquisicao / 100                   # Percentual de depreciacao no mes (atual)

                dados_patrimonio_movimento = {
                    'company_id': company_id,
                    'bem_id': bem.id,
                    'ano': ano,
                    'mes': mes,
                    'depreciacao_mes': depreciacao_atual_mes
                }

                if depreciacao_id:                                                  # Corrige o movimento
                    obj_patrimonio_movimento.write(cr, uid, depreciacao_id,
                                                   dados_patrimonio_movimento,
                                                   context)
                else:                                                               # Registra novo movimento
                    obj_patrimonio_movimento.create(cr, uid,
                                                    dados_patrimonio_movimento,
                                                    context)

                # Calculo e ajuste de depreciacao acumulada
                depreciacao_acumulada = bem.depreciacao_acumulada - \
                                        depreciacao_anterior_mes + \
                                        depreciacao_atual_mes

                # Se o valor da depreciacao acumulada for maior que o valor do
                # Bem a depreciar, sera calculada apenas a diferenca entre o
                # valor do Bem a depreciar e o valor da depreciacao acumulada
                # anterior
                if depreciacao_acumulada > bem.valor_a_depreciar:
                    depreciacao_excesso = bem.depreciacao_acumulada - \
                                          bem.valor_depreciar


                    # Corrige o movimento
                    corrigido = depreciacao_atual_mes - depreciacao_excesso
                    dados_patrimonio_movimento['depreciacao_mes'] = corrigido
                    obj_patrimonio_movimento.write(cr, uid, depreciacao_id,
                                                   dados_patrimonio_movimento,
                                                   context)

                dados_patrimonio_bens = {
                    'depreciacao_acumulada': depreciacao_acumulada
                }

                obj_patrimonio_bens.write(cr, uid, bem.id,
                                          dados_patrimonio_bens, context)

            #Atualiza Parametro com mes e ano da ultima depreciacao
            parametro_ids = obj_adm_parametro.search(cr, uid, [], context=context)

            if parametro_ids:
                dados = {'admpar_pat_ano': ano, 'admpar_pat_mes': mes}
                obj_adm_parametro.write(cr, uid, parametro_ids, dados, context)
            msg = u'Calculado %d itens.' % len(dados_bem)
        else:
            raise osv.except_osv(_('Atencao'), _('Nao há dados para efetuar o cálculo.'))

        return messagebox(msg)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
