# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from SigUtils import numero
from openerp.tools.translate import _


class patrimoniofiscal_ciap(osv.Model):
    """
    Tabela de controle das amortizacoes

    @change: 30.04.2013 - pierre.barbosa
    """

    _name = 'patrimoniofiscal.ciap'
    _inherit = 'patrimoniofiscal.ciap'

    def create(self, cr, uid, vals, context=None):
        """
        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param vals: {dict} os campos e seus respectivos valores,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if 'mes' in vals:
            vals['mes'] = '%02d' % int(vals.get('mes'))

        args = [('company_id', '=', vals.get('company_id')),
                ('bem_id', '=', vals.get('bem_id')),
                ('ano', '=', vals.get('ano')),
                ('mes', '=', vals.get('mes'))]

        ciap_ids = self.search(cr, uid, args, context=context)

        if ciap_ids:
            msg = _('Ja existe CIAP para este bem neste ano/mes.')
            raise osv.except_osv(_('Atencao'), msg)

        result = super(patrimoniofiscal_ciap, self).create(cr, uid, vals, context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        """

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids:
        @param vals: {dict} os campos e seus respectivos valores,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        if 'mes' in vals:
            vals['mes'] = '%02d' % int(vals.get('mes'))

        return super(patrimoniofiscal_ciap, self).write(cr, uid, ids, vals, context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param args: {list} argumentos da pesquisa,
        @param offset:
        @param limit: {int} numero máximo de registro a ser retornado,
        @param order: {char} ordem de classificacao dos registros,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @param count: {boolean} se verdadeiro somente o numero de registros afe-
        tados será retornado,
        @return: {list} list dos ID's dos registros que satisfizeram os argumen-
        tos da pesquisa ou o numero de registros afetados (se count=True).

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        context['sig_search_sem_filhos'] = True

        return super(patrimoniofiscal_ciap, self).search(cr, uid, args, offset, limit, order, context, count)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
