# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
from calendar import monthrange


class patrimoniofiscal_ciap_controle(osv.Model):
    """
    Tabela de controle de apropriacao do ciap

    @change: 30.04.2013 - pierre.barbosa
    """
    _name = 'patrimoniofiscal.ciap_controle'
    _inherit = 'patrimoniofiscal.ciap_controle'

    def patfis_onchange_calcular_coeficiente(self, cr, uid, ids,
                                             saidas_tributadas, saidas_total):
        """

        @param cr:
        @param uid:
        @param ids:
        @param saidas_tributadas:
        @param saidas_total:
        @return:
        """

        if isinstance(ids, (int, long)):
            ids = [ids]

        if saidas_tributadas == 0 or saidas_total == 0:
            coeficiente = 0.0
        else:
            coeficiente = saidas_tributadas / saidas_total

        return {'value': {'coeficiente': coeficiente}, 'warning': {}}

    def patfis_calcularIcmsAquisicao(self, cr, uid, ano, mes, context=None):
        """

        @param cr:
        @param uid:
        @param ano:
        @param mes:
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        # ___________________________ Objects ________________________________
        context = context and context.copy() or {}
        obj_patrimonio_bens = self.pool.get('patrimonio.bens')
        obj_custos = self.pool.get('patrimonio.custos.complementares')

        _dataFinal = '%04d-%02d-%02d' % (ano, int(mes),
                                         monthrange(ano, int(mes))[1])

        args = [('data_aquisicao', '<=', _dataFinal)]
        bem_ids = obj_patrimonio_bens.search(cr, uid, args, context=context)

        _valor = 0.00
        campos = ['credito_icms', 'data_baixa']
        _bens_dados = obj_patrimonio_bens.read(cr, uid, bem_ids, campos, context=context)

        # TODO: Alterar a rotina para utilizar o metodo ´´browse`` da classe
        campos = ['icms']
        if _bens_dados and isinstance(_bens_dados, list):
            for _bem in _bens_dados:
                if _bem.get('data_baixa') and _bem.get('data_baixa') <= _dataFinal:
                    continue

                _valor += _bem.get('credito_icms')

                args = [('bem_id', '=', _bem.get('id'))]
                custo_ids = obj_custos.search(cr, uid, args, context=context)

                if custo_ids:
                    for custo_id in custo_ids:
                        icms_dados = obj_custos.read(cr, uid, custo_id, campos, context=context)
                        if icms_dados and isinstance(icms_dados, list):
                            icms_dados = icms_dados[0]

                        if icms_dados and isinstance(icms_dados, dict):
                            _valor += icms_dados.get('icms') or 0

        return _valor

    def patfis_atualizarControle(self, cr, uid, ids, context=None):
        """

        @param cr:
        @param uid:
        @param ids:
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_ciap = self.pool.get('patrimoniofiscal.ciap')
        obj_bens = self.pool.get('patrimonio.bens')

        for item in self.browse(cr, uid, ids):
            _ano = int(item.ano)
            _mes = int(item.mes)
            _empresa_id = item.company_id.id

            args = [('company_id', '=', _empresa_id),
                    ('ano', '=', '%04d' % _ano),
                    ('mes', '=', '%02d' % _mes)]
            ciap_ids = obj_ciap.search(cr, uid, args, context=context)

            for ciap_id in ciap_ids:
                _ciap_dados = obj_ciap.read(cr, uid, ciap_id, context=context)

                _bem_dados = obj_bens.read(cr, uid, _ciap_dados.get('bem_id')[0], context=context)

                if _bem_dados and isinstance(_bem_dados, list):
                    _bem_dados = _bem_dados[0]

                if _bem_dados and isinstance(_bem_dados, dict):
                    _primeiro_dia_mes = '%04d-%02d-01' % (_ano, _mes)

                    if _bem_dados.get('data_baixa') < _primeiro_dia_mes:
                        vals = {'ciap_controle_id': False}
                    else:
                        vals = {'ciap_controle_id': item.id}

                    obj_ciap.write(cr, uid, ciap_id, vals, context)

        return {}

    def create(self, cr, uid, vals, context=None):
        """

        @param cr:
        @param uid:
        @param vals:
        @param context:
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}
        if 'mes' in vals:
            vals['mes'] = '%02d' % int(vals.get('mes'))

        if all(('saidas_tributadas' in vals, 'saidas_total' in vals,
                vals.get('saidas_total') != 0)):
            vals['coeficiente'] = float(vals.get('saidas_tributadas'))
            vals['coeficiente'] /= float(vals.get('saidas_total'))

        if not 'coeficiente' in vals:
            vals['coeficiente'] = 0.00

        vals['saldo_acumulado'] = 0.00
        vals['total_apropriar'] = 0.00

        _ids = super(patrimoniofiscal_ciap_controle, self).create(cr, uid,
                                                                  vals,
                                                                  context=context)
        self.patfis_atualizarControle(cr, uid, _ids, context=context)

        campos = ['saldo_acumulado']
        controle_dados = self.read(cr, uid, _ids, campos, context=context)
        if controle_dados and isinstance(controle_dados, list):
            controle_dados = controle_dados[0]

        if controle_dados and isinstance(controle_dados, dict):
            vals['saldo_acumulado'] = controle_dados.get('saldo_acumulado')
            vals['total_apropriar'] = vals['coeficiente'] * vals['saldo_acumulado']

            super(patrimoniofiscal_ciap_controle, self).write(cr, uid, _ids, vals, context=context)

        return _ids

    def write(self, cr, uid, ids, vals, context=None):
        """

        @param cr:
        @param uid:
        @param ids:
        @param vals:
        @param context:
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        if 'mes' in vals:
            vals['mes'] = '%02d' % int(vals.get('mes'))

        ciap_dados = self.read(cr, uid, ids, context=context)

        if ciap_dados and isinstance(ciap_dados, list):
            ciap_dados = ciap_dados[0]

        if ciap_dados and isinstance(ciap_dados, dict):
            if not 'saidas_tributadas' in vals:
                vals['saidas_tributadas'] = ciap_dados.get('saidas_tributadas')

            if not ('saidas_total' in vals):
                vals['saidas_total'] = ciap_dados.get('saidas_total')

            if not ('saldo_acumulado' in vals):
                vals['saldo_acumulado'] = ciap_dados.get('saldo_acumulado')

        vals['coeficiente'] = float(vals['saidas_tributadas'])
        vals['coeficiente'] /= float(vals['saidas_total'])

        if not 'coeficiente' in vals:
            vals['coeficiente'] = 0.00

        saldo_acumulado = self._patfis_get_saldo(cr, uid, ids,
                                                 'saldo_acumulado', None)
        vals['total_apropriar'] = vals['coeficiente'] * saldo_acumulado[ids[0]]
        result = super(patrimoniofiscal_ciap_controle, self).write(cr, uid, ids, vals, context)
        self.patfis_atualizarControle(cr, uid, ids)

        return result

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """

        @param cr:
        @param uid:
        @param args:
        @param offset:
        @param limit:
        @param order:
        @param context:
        @param count:
        @return:

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        context['sig_search_sem_filhos'] = True

        return super(patrimoniofiscal_ciap_controle, self).search(cr, uid,
                                                                  args, offset,
                                                                  limit, order,
                                                                  context,
                                                                  count)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
