# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from pythonsped.efd import *
from collections import Counter
from dateutil.parser import parse


class fiscal_sped(osv.TransientModel):

    _inherit = 'fiscal.sped'
    blocoG_nf_ids = list()
    blocoG_itens = dict()
    pessoa_ids_0150 = list()

    # @tempo_exec
    def blocoG(self, cr, uid, sped_data_inicial, sped_data_final, company_id,
               args_periodo_mes_e_ano, sped_mes_inicial, sped_ano_inicial, reg_0001,
               movimento_notas, context=None):
        """
        Metodo responsavel pela dados do registro tipo G do SPED Fiscal. Para gera-los, o modulo
        'patrimonio_fiscal' deve estar instalado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param sped_data_inicial: dados da data de inicio da geracao do sped.
        :type sped_data_inicial: str
        :param sped_data_final: dados da data final da geracao do sped.
        :type sped_data_final: str
        :param company_id: identificador da empresa, id da tabela res_company.
        :param args_periodo_mes_e_ano: dados do periodo de pesquisa do CIAP.
        :type args_periodo_mes_e_ano: list
        :param sped_mes_inicial: numero inicial do mes de periodo do SPED.
        :type sped_mes_inicial: int
        :param sped_ano_inicial: numero inicial do ano de periodo do SPED.
        :type sped_ano_inicial: int
        :param reg_0000: dados do registro tipo 0000 do SPED.
        :type reg_0000: object
        :param reg_0001: dados do registro tipo 0001 do SPED.
        :type reg_0001: object
        :param movimento_notas: lista de id da tabela fiscal_nf_capa, identificadores das notas
        fiscais que serao consideradas no SPED.
        :type movimento_notas: list
        :param obj_sql_sped: instancia da classe SqlSpedFiscal.
        :type obj_sql_sped: object
        :param context: dados de valores contextuais.
        :type context: dict
        :return: Descricao de possiveis problemas encontrados na geracao do registro tipo G.
        :rtype: str
        """
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        mensagem = ''

        cod_versao = self.get_versao_layout(parse(sped_data_final, yearfirst=True).date().year)
        sped = eval("layout_%s" % cod_versao[1:])  # Leiaute do SPED em funcao de sua versao

        # PATRIMÔNIO CIAP

        # BLOCO G – CONTROLE DO CRÉDITO DE ICMS DO ATIVO PERMANENTE CIAP
        obj_patfiscal_ciap = self.pool.get('patrimoniofiscal.ciap')
        obj_controle = self.pool.get('patrimoniofiscal.ciap_controle')
        args = [('company_id', '=', company_id)]
        args.extend(args_periodo_mes_e_ano)

        new_args = list()
        new_args2 = list()
        for arg in args:
            arg = list(arg)
            if 'mes' not in arg and 'ano' not in arg:
                new_args.append(arg)
                new_args2.append(arg)
                continue
            mes_ano = str(arg[2])
            new_args.append([arg[0], arg[1], mes_ano])
            if len(mes_ano) == 1:
                mes_ano = '0' + mes_ano
            new_args2.append([arg[0], arg[1], mes_ano])

        ciap_ids = obj_patfiscal_ciap.search(cr, uid, args, context=context)
        if not ciap_ids:
            ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args, context=context)
            if not ciap_ids:
                ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args2, context=context)
        brw_pat_ciap = obj_patfiscal_ciap.browse(cr, uid, ciap_ids, context=context)

        controle_ids = obj_controle.search(cr, uid, args, context=context)
        if not controle_ids:
            controle_ids = obj_controle.search(cr, uid, new_args, context=context)
            if not controle_ids:
                controle_ids = obj_controle.search(cr, uid, new_args2, context=context)

        # REGISTRO G001  ABERTURA DO BLOCO G
        reg_G001 = bloco_g.registro_G001()

        if not obj_adm_utils.admutl_get_parametros(
                cr, uid, 'adm', 'admpar_pat_gerar_ciap', context=context).get(
                'admpar_pat_gerar_ciap') or (not ciap_ids and not controle_ids):
            reg_G001.IND_MOV = '1'
            reg_G001.write_line()
            return mensagem

        reg_G001.IND_MOV = '0'
        reg_G001.write_line()

        # REGISTRO G110 – ICMS – ATIVO PERMANENTE – CIAP

        valor_apropriacao = sum(map(lambda pc: pc.valor_apropriacao or 0,
                                    brw_pat_ciap))
        if sped_mes_inicial == 1:
            ano_anterior = sped_ano_inicial - 1
        else:
            ano_anterior = sped_ano_inicial

        if not controle_ids:
            reg_G110 = bloco_g.registro_G110()
            reg_G110.DT_INI = ''
            reg_G110.DT_FIN = ''
            reg_G110.SALDO_IN_ICMS = self.formata_numero(0, size=1, separador=',')  # saldo_in_icms
            reg_G110.SOM_PARC = self.formata_numero(0, size=1, separador=',')
            reg_G110.VL_TRIB_EXP = self.formata_numero(0, size=1, separador=',')
            reg_G110.VL_TOTAL = self.formata_numero(0, size=1, separador=',')
            reg_G110.IND_PER_SAI = self.formata_numero(0, size=1, separador=',')
            reg_G110.ICMS_APROP = self.formata_numero(0, size=1, separador=',')
            reg_G110.SOM_ICMS_OC = '0,00'
            reg_G110.write_line()
            mensagem += _('Nao foi lancado o controle mensal do CIAP.\n')
        for controle in obj_controle.browse(cr, uid, controle_ids, context):
            brw_ciap_controle = filter(lambda ciap: ciap.ciap_controle_id.id == controle.id,
                                       brw_pat_ciap)
            if not brw_ciap_controle:
                brw_ciap_controle = brw_pat_ciap
            saldo_in_icms = sum(map(lambda ciap: ciap.bem_id.credito_icms or 0.0,
                                    brw_ciap_controle))
            reg_G110 = bloco_g.registro_G110()
            reg_G110.DT_INI = self.formata_data(sped_data_inicial)
            reg_G110.DT_FIN = self.formata_data(sped_data_final)
            reg_G110.SALDO_IN_ICMS = self.formata_numero(saldo_in_icms, size=1,
                                                         separador=',')  # saldo_in_icms
            reg_G110.SOM_PARC = self.formata_numero(valor_apropriacao, size=1,
                                                    separador=',')
            reg_G110.VL_TRIB_EXP = self.formata_numero(
                controle.saidas_tributadas,
                size=1, separador=',')
            reg_G110.VL_TOTAL = self.formata_numero(controle.saidas_total,
                                                    size=1,
                                                    separador=',')
            reg_G110.IND_PER_SAI = self.formata_numero(controle.coeficiente,
                                                       size=1, dec=8,
                                                       separador=',')
            reg_G110.ICMS_APROP = self.formata_numero(0, size=1, separador=',')
            if controle.saidas_total > 0:
                reg_G110.ICMS_APROP = self.formata_numero(
                    valor_apropriacao * (
                        controle.saidas_tributadas / controle.saidas_total),
                    size=1,
                    separador=',')

            reg_G110.SOM_ICMS_OC = '0,00'
            reg_G110.write_line()

        for i in brw_pat_ciap:
            if not i.bem_id:
                continue
            if i.bem_id.patbem_item_id:
                self.blocoG_itens.update(
                    {i.bem_id.patbem_item_id.produto_id.id: i.bem_id.patbem_item_id.grade_id.id})
            else:
                produto_ids = obj_estoque_produto.search(
                    cr, uid, [('codigo', '=', i.bem_id.codigo_item)], limit=1, context=context)
                grade_ids = obj_estoque_produto_grade.search(
                    cr, uid, [('produto_id', 'in', produto_ids)], limit=1, context=context)
                if produto_ids and grade_ids:
                    self.blocoG_itens.update({produto_ids[0]: grade_ids[0]})
            if i.bem_id.patbem_nota_fiscal_id:
                if i.bem_id.patbem_nota_fiscal_id.id not in movimento_notas:
                    self.blocoG_nf_ids.append(i.bem_id.patbem_nota_fiscal_id.id)
            else:
                nf_id = obj_fiscal_nf_capa.search(
                    cr, uid, [('numero_nf', '=', i.bem_id.numero_nf),
                              ('serie_nf', '=', i.bem_id.serie_nf), ('tipo', '=', 'E')],
                    limit=1, context=context)
                if nf_id:
                    if nf_id[0] not in movimento_notas:
                        self.blocoG_nf_ids.append(nf_id[0])

            if i.bem_id.grupos_id:
                if i.bem_id.grupos_id.conta_aquisicao_id:
                    # REGISTRO 0500 PLANO DE CONTAS CONTÁBEIS
                    if not i.bem_id.grupos_id.conta_aquisicao_id.conta_tipo:
                        mensagem += _('* TIPO DA CONTA nao informada.\n')
                    if not i.bem_id.grupos_id.conta_aquisicao_id.grau:
                        mensagem += _('* NIVEL DA CONTA nao informada.\n')
                    if not i.bem_id.grupos_id.conta_aquisicao_id.name:
                        mensagem += _('* DESCRICAO DA CONTA nao informada.\n')
                else:
                    mensagem += _('* CONTA DE AQUISICAO nao informada para o grupo.\n')
            else:
                mensagem += _('* GRUPO qual o bem pertence nao informado.\n')

            # REGISTRO G125 – MOVIMENTAÇÃO DE BEM OU COMPONENTE DO ATIVO IMOBILIZADO
            read_custo_complementar = self.pool.get(
                'patrimonio.custos.complementares').read(
                cr, uid, map(lambda cc: cc.id, i.bem_id.custo_complementar_ids), context=context)
            vl_imob_icms_st, vl_imob_icms_frt, vl_imob_icms_dif = '0,00', '0,00', '0,00'
            vl_imob_icms_op = self.formata_numero(i.bem_id.credito_icms, size=1, separador=',')
            for x in read_custo_complementar:
                sigla = \
                    self.pool.get('patrimonio.tipo.custos.complementares').read(
                        cr, uid, x['tipo_id'][0], ['sigla'], context=context)['sigla']
                if sigla == 'ISU':
                    vl_imob_icms_st = self.formata_numero(x['icms'], size=1,
                                                          separador=',')
                if sigla == 'FRT':
                    vl_imob_icms_frt = self.formata_numero(x['icms'], size=1,
                                                           separador=',')
                if sigla == 'DIF':
                    vl_imob_icms_dif = self.formata_numero(x['icms'], size=1,
                                                           separador=',')

            if i.bem_id.qtd_parcelas == 1:  # aquisicao do bem
                # BUSCAR DATA DE SAIDA_ENTRADA SE POSSUIR NOTA FISCAL, SENAO, DA
                # AQUISICAO
                if i.bem_id.patbem_nota_fiscal_id:
                    dt_mov = self.formata_data(
                        i.bem_id.patbem_nota_fiscal_id.data_saida_entrada[:10])
                else:
                    dt_mov = self.formata_data(i.bem_id.data_aquisicao)
                tipo_mov = 'IM'
            else:
                dt_mov = self.formata_data(sped_data_inicial)
                tipo_mov = 'SI'

            reg_G125 = bloco_g.registro_G125()
            reg_G125.COD_IND_BEM = i.bem_id.codigo_bem
            reg_G125.DT_MOV = dt_mov
            reg_G125.TIPO_MOV = tipo_mov
            reg_G125.VL_IMOB_ICMS_OP = vl_imob_icms_op
            reg_G125.VL_IMOB_ICMS_ST = vl_imob_icms_st
            reg_G125.VL_IMOB_ICMS_FRT = vl_imob_icms_frt
            reg_G125.VL_IMOB_ICMS_DIF = vl_imob_icms_dif
            reg_G125.NUM_PARC = i.parcela
            reg_G125.VL_PARC_PASS = self.formata_numero(i.valor_apropriacao, size=1,
                                                        separador=',')
            reg_G125.write_line()

            if i.bem_id.qtd_parcelas != 1:

                reg_G130 = (cod_versao and int(cod_versao) >= 14) and bloco_g.registro_G130() or \
                    sped.bloco_g.registro_G130()

                if i.bem_id.patbem_nota_fiscal_id:
                    emitente = 0 if i.bem_id.patbem_nota_fiscal_id.nf_propria else 1
                    fornecedor = i.bem_id.patbem_nota_fiscal_id.destinatario_id
                    numero_nf = i.bem_id.patbem_nota_fiscal_id.numero_nf
                    reg_G130.IND_EMIT = emitente
                    cod_part = str(fornecedor.codigo) + (
                        str(i.bem_id.patbem_nota_fiscal_id.destinatario_endereco_id.id) or '')
                    if context.get('patfis_cod_part'):
                        cod_part = '@##@' + cod_part + '@##@'
                    reg_G130.COD_PART = cod_part
                    reg_G130.COD_MOD = i.bem_id.patbem_nota_fiscal_id.modelo_nf_id.fismnf_codigo
                    reg_G130.SERIE = i.bem_id.patbem_nota_fiscal_id.serie_nf or ''
                    reg_G130.NUM_DOC = numero_nf
                    reg_G130.CHV_NFE_CTE = i.bem_id.patbem_nota_fiscal_id.nfe_chave_acesso or ''
                    reg_G130.DT_DOC = self.formata_data(
                        i.bem_id.patbem_nota_fiscal_id.data_emissao[:10])
                else:
                    self.pessoa_ids_0150.append(i.bem_id.fornecedor_id.id)
                    emitente = i.bem_id.emitente
                    fornecedor = i.bem_id.fornecedor_id
                    numero_nf = i.bem_id.numero_nf
                    reg_G130.IND_EMIT = emitente
                    cod_part = fornecedor.cpf_cnpj
                    if context.get('patfis_cod_part'):
                        cod_part = '@##@' + cod_part + '@##@'
                    reg_G130.COD_PART = obj_adm_utils.admutl_remove_caracter(cod_part, numeros=True)
                    reg_G130.COD_MOD = i.bem_id.modelo_nf
                    reg_G130.SERIE = i.bem_id.serie_nf or ''
                    reg_G130.NUM_DOC = numero_nf
                    reg_G130.CHV_NFE_CTE = i.bem_id.chave_nfe or ''
                    reg_G130.DT_DOC = self.formata_data(i.bem_id.data_aquisicao)
                reg_G130.write_line()

                if not emitente:
                    mensagem += _('* TIPO DO EMITENTE nao informado em bens.\n')
                if not fornecedor:
                    mensagem += _('* FORNECEDOR DO BEM nao informado.\n')
                if not numero_nf:
                    mensagem += _('* NUMERO DA NF nao informado para o bem.\n')
                if not dt_mov:
                    mensagem += _(
                        '* DATA DA EMISSAO do documento fiscal do bem nao informado.\n')

                # REGISTRO G140 – IDENTIFICAÇÃO DO ITEM DO DOCUMENTO FISCAL
                reg_G140 = (cod_versao and int(cod_versao) >= 14) and bloco_g.registro_G140() or \
                    sped.bloco_g.registro_G140()
                if i.bem_id.patbem_item_id:
                    sequencia = str(i.bem_id.patbem_item_id.sequencia)
                    reg_G140.NUM_ITEM = sequencia
                    if i.bem_id.patbem_item_id.produto_id.grade_usar:
                        codigo_item = i.bem_id.patbem_item_id.produto_id.codigo + ' ' + \
                            i.bem_id.patbem_item_id.grade_id.grade + ' ' + \
                            i.bem_id.patbem_item_id.produto_embalagem_id.estpre_unidade_produto
                        reg_G140.COD_ITEM = codigo_item
                    else:
                        codigo_item = i.bem_id.patbem_item_id.produto_id.codigo + ' ' + \
                            i.bem_id.patbem_item_id.produto_embalagem_id.estpre_unidade_produto
                        reg_G140.COD_ITEM = codigo_item
                else:
                    sequencia = str(i.bem_id.sequencia_item_nf or '')
                    codigo_item = i.bem_id.codigo_item or ''
                    reg_G140.NUM_ITEM = sequencia
                    reg_G140.COD_ITEM = codigo_item

                item = i.bem_id.patbem_item_id
                trib_icms = False
                for trib in (item.tributacoes_ids if item else []):
                    if trib.encargo_id.sigla == 'ICMS':
                        trib_icms = trib
                        break

                if item and trib_icms:
                    total = item.vr_total_item_rateado
                    frete = item.vr_frete_rateado or 0
                    vr_icms = trib_icms.vr_encargo or 0
                    icms_frete = 0
                    if total > 0:
                        icms_frete = frete * vr_icms / total

                    if cod_versao and int(cod_versao) >= 14:
                        reg_G140.QTDE = self.formata_numero(item.quantidade, dec=5, separador=',')
                        reg_G140.UNID = i.bem_id.patbem_item_id.produto_embalagem_id.\
                            estpre_unidade_produto
                        reg_G140.VL_ICMS_OP_APLICADO = self.formata_numero(i.bem_id.credito_icms or 0,
                                                                           dec=2, separador=',')
                        reg_G140.VL_ICMS_ST_APLICADO = self.formata_numero(
                            trib_icms.vr_icms_substituicao or 0, dec=2, separador=',')
                        reg_G140.VL_ICMS_FRT_APLICADO = self.formata_numero(icms_frete, dec=2,
                                                                            separador=',')
                        reg_G140.VL_ICMS_DIF_APLICADO = self.formata_numero(0, dec=2, separador=',')

                reg_G140.write_line()

                if not sequencia:
                    mensagem += _('* SEQUENCIA DO ITEM do documento fiscal do bem "%s" nao infor'
                                  'mado.\n' % i.bem_id.name)
                if not codigo_item:
                    mensagem += _('* CODIGO DO ITEM do documento fiscal do bem "%s" nao infor'
                                  'mado.\n' % i.bem_id.name)
        return mensagem

    def bloco_g_registro0150(self, cr, uid, company_id, obj_sql_sped, movimento_notas,
                             context=None):
        context = context and context.copy() or {}
        mensagem = ''
        nf_pessoa_ids = list(set(map(
            lambda d: d.get('destinatario_id'), self.pool.get('fiscal.nf.capa').read(
                cr, uid, movimento_notas, ['destinatario_id'], context=context,
                load='_classic_write'))))

        reg_0150 = bloco_0.registro_0150()

        if self.blocoG_nf_ids:
            filtro_0150 = {
                'movimento_notas': self.blocoG_nf_ids,
                'company_id': company_id
            }
        else:
            filtro_0150 = {
                'movimento_notas': [(-1)],
                'company_id': False
            }
        dados = obj_sql_sped.sql_reg0150(cr, filtro_0150)
        for dado in dados:
            if not dado.get('nome'):
                continue
            if dado.get('pessoa_id') in nf_pessoa_ids:
                continue
            ie = dado.get('ie') or ''
            if 'ISENT' in str(ie).strip().upper():
                ie = ''
            reg_0150.REG = '0150'
            reg_0150.COD_PART = dado.get('cod_part') or ''
            reg_0150.NOME = dado.get('nome')[:99] or ''
            reg_0150.COD_PAIS = dado.get('cod_pais') or '1058'
            reg_0150.CNPJ = dado.get('cnpj') or ''
            reg_0150.CPF = dado.get('cpf') or ''
            reg_0150.IE = self.formata_para_digitos(ie) or ''
            reg_0150.COD_MUN = dado.get('cod_mun') or ''
            reg_0150.SUFRAMA = dado.get('suframa') or ''
            reg_0150.END = dado.get('logradouro') or ''
            reg_0150.NUM = dado.get('num') or ''
            reg_0150.COMPL = dado.get('compl') or ''
            reg_0150.BAIRRO = dado.get('bairro') or ''
            reg_0150.write_line()
        for pessoa in self.pool.get('adm.pessoa').browse(cr, uid, self.pessoa_ids_0150, context):
            cod_part = pessoa.cpf_cnpj
            if context.get('patfis_cod_part'):
                cod_part = '@##@' + cod_part + '@##@'

            reg_0150.REG = '0150'
            reg_0150.COD_PART = cod_part or ''
            reg_0150.NOME = pessoa.nome or ''
            reg_0150.COD_PAIS = pessoa.endereco_ids[0].pais_id.codigo_bc if pessoa.endereco_ids \
                else '1058'
            reg_0150.CNPJ = pessoa.cpf_cnpj if pessoa.pessoa_tipo == 'J' else ''
            reg_0150.CPF = pessoa.cpf_cnpj if pessoa.pessoa_tipo == 'F' else ''
            reg_0150.IE = self.formata_para_digitos(pessoa.admpes_jur_ie
                                                    if pessoa.pessoa_tipo == 'J' else '')
            reg_0150.COD_MUN = pessoa.endereco_ids[0].cidade_id.codigo_ibge if pessoa.endereco_ids \
                else ''
            reg_0150.SUFRAMA = pessoa.admpes_jur_suframa or ''
            reg_0150.END = pessoa.endereco_ids[0].logradouro_id.name if pessoa.endereco_ids \
                else ''
            reg_0150.NUM = pessoa.endereco_ids[0].numero if pessoa.endereco_ids \
                else ''
            reg_0150.COMPL = pessoa.endereco_ids[0].complemento if pessoa.endereco_ids \
                else ''
            reg_0150.BAIRRO = pessoa.endereco_ids[0].bairro_id.name if pessoa.endereco_ids \
                else ''
            reg_0150.write_line()
        return mensagem

    def bloco_g_registro0200(self, cr, uid, reg_0000,  obj_sql_sped, sped_data_final,
                             movimento_notas, context=None):
        context = context and context.copy() or {}
        mensagem = ''
        obj_fiscal = self.pool.get('fiscal.nf.capa')
        obj_adm_utl = self.pool.get('adm.utils')
        cod_versao = self.get_versao_layout(parse(sped_data_final, yearfirst=True).date().year)
        sped = eval("layout_%s" % cod_versao[1:])  # Leiaute do SPED em funcao de sua versao
        reg_0200 = sped.bloco_0.registro_0200()
        movimento = {'movimento_notas': self.blocoG_nf_ids,
                     'movimento_cupons': []}
        filtro = {}
        if reg_0000.IND_PERFIL != 'C':
            filtro = self.filtra_ids_itens(cr, uid, obj_fiscal, movimento, reg_0000.IND_PERFIL,
                                           context)
        filtro.update({'inventario_id': False})
        dados = obj_sql_sped.sql_reg0200(cr, filtro, reg_0000.IND_PERFIL)
        dados = filter(lambda d: d.get('produto_id') in self.blocoG_itens.keys()
                       and d.get('grade_id') in self.blocoG_itens.values(), dados)
        contador = Counter()
        contador_cod_item = Counter()

        sql = """
            SELECT DISTINCT grade_id FROM estoque_movimento_itens WHERE nf_id IN (%s) 
        """ % ', '.join(map(lambda n: str(n), movimento_notas))
        grade_ids_0200 = []
        if movimento_notas:
            cr.execute(sql)
            grade_ids_0200 = map(lambda g: g.get('grade_id'), cr.dictfetchall())

        codigo_barra_ids = list(set(map(lambda cb: cb.get('cod_barra_id'),
                                        filter(lambda r: r.get('cod_barra_id'), dados))))
        codigo_barra_dados = self.pool.get(
            'automacao.codigo.barra').autcba_get_codigo_barra_formatado_gtin(
            cr, uid, codigo_barra_ids, context)

        for dado in dados:
            if dado.get('grade_id') not in list(contador.elements()) \
                    or dado.get('grade_id') not in grade_ids_0200:
                contador[dado.get('grade_id')] += 1
            if dado.get('cod_item') in list(contador_cod_item.elements()):
                continue
            contador_cod_item[dado.get('cod_item')] += 1
            codigo_barra = codigo_barra_dados.get(dado.get('cod_barra_id')) or ''
            reg_0200.REG = '0200'
            reg_0200.COD_ITEM = dado.get('cod_item') or ''
            descri_item = obj_adm_utl.admutl_remove_caracter(dado.get('descr_item'),
                                                             letras=True, numeros=True,
                                                             espaco=True)
            reg_0200.DESCR_ITEM = descri_item or ''
            reg_0200.COD_BARRA = '' if codigo_barra == 'SEM GTIN' else codigo_barra
            reg_0200.COD_ANT_ITEM = dado.get('cod_ant_item') or ''
            reg_0200.UNID_INV = dado.get('unid_inv') or ''
            reg_0200.TIPO_ITEM = dado.get('tipo_item') or ''
            reg_0200.COD_NCM = dado.get('cod_ncm') or ''
            reg_0200.EX_IPI = dado.get('ex_ipi') or ''
            reg_0200.COD_GEN = dado.get('cod_gen') or ''
            reg_0200.COD_LST = dado.get('cod_lst') or ''
            reg_0200.ALIQ_ICMS = dado.get('aliq_icms') or ''
            if cod_versao == '011':
                reg_0200.CEST = dado.get('cest') or ''
            reg_0200.write_line()

        itens_nao_gerados_grade_ids = filter(lambda it: it not in list(contador.elements()),
                                             self.blocoG_itens.values())
        for grade in self.pool.get('estoque.produto.grade').browse(
                cr, uid, itens_nao_gerados_grade_ids, context):
            if grade.id in grade_ids_0200:
                continue
            descri_item = obj_adm_utl.admutl_remove_caracter(grade.name, letras=True, numeros=True,
                                                             espaco=True)
            if grade.produto_id.tipo == 'S':
                cod_gen = '00'
            elif grade.produto_id.tipo == 'P' and grade.produto_id.ncm_id:
                cod_gen = grade.produto_id.ncm_id.codigo[:2]
            else:
                cod_gen = ''

            if grade.produto_id.estpro_cest_id:
                cest = grade.produto_id.estpro_cest_id.fiscest_codigo
            elif grade.produto_id.ncm_id.fisnc_cest_id:
                cest = grade.produto_id.ncm_id.fisnc_cest_id.fiscest_codigo
            else:
                cest = ''

            codigo_barra = ''
            if grade.codigo_barra_ids:
                codigo_barra = grade.codigo_barra_ids[0].autcba_get_codigo_barra_formatado_gtin().\
                    get(grade.codigo_barra_ids[0].id)

            if grade.grade_usar:
                codigo_item = grade.produto_id.codigo + ' ' + grade.grade + ' ' + \
                    grade.produto_id.estpro_unidade_inventario
            else:
                codigo_item = grade.produto_id.codigo + ' ' + \
                    grade.produto_id.estpro_unidade_inventario

            reg_0200.REG = '0200'
            reg_0200.COD_ITEM = codigo_item
            reg_0200.DESCR_ITEM = descri_item or ''
            reg_0200.COD_BARRA = '' if codigo_barra == 'SEM GTIN' else codigo_barra
            reg_0200.COD_ANT_ITEM = ''
            reg_0200.UNID_INV = grade.produto_id.estpro_unidade_inventario or ''
            reg_0200.TIPO_ITEM = grade.produto_id.tipo_item or ''
            reg_0200.COD_NCM = obj_adm_utl.admutl_remove_caracter(
                grade.produto_id.ncm_id.codigo if grade.produto_id.ncm_id else '', numeros=True)
            reg_0200.EX_IPI = ''
            reg_0200.COD_GEN = cod_gen
            reg_0200.COD_LST = ''
            reg_0200.ALIQ_ICMS = ''
            if cod_versao == '011':
                reg_0200.CEST = cest
            reg_0200.write_line()
        return mensagem

    def bloco_g_registro0190(self, cr, uid, reg_0001, context=None):
        mensagem = ''
        reg_0190 = bloco_0.registro_0190()
        produto_ids = self.blocoG_itens.keys()
        grade_ids = self.blocoG_itens.values()
        if not produto_ids or not grade_ids:
            return mensagem
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))
        if len(grade_ids) == 1:
            grade_ids = str(tuple(grade_ids)).replace(',', '')
        else:
            grade_ids = str(tuple(grade_ids))
        sql_0190 = """
            SELECT DISTINCT produto.estpro_unidade_inventario                    AS unid,
                            (SELECT name
                             FROM   estoque_embalagem_unidade
                             WHERE  unidade = produto.estpro_unidade_inventario) AS descr
            FROM   estoque_produto produto
                   INNER JOIN estoque_produto_grade grade
                           ON produto.id = grade.produto_id
            WHERE
                   produto.id IN %s
                   AND grade.id IN %s
        """ % (produto_ids, grade_ids)
        cr.execute(sql_0190)
        dados = cr.dictfetchall()
        for dado in dados:
            if filter(lambda p: p.get('unid') == dado.get('unid')
                      and p.get('descr') == dado.get('descr'), self.registro_0190_dados):
                continue
            reg_0190.REG = '0190'
            reg_0190.UNID = dado.get('unid')
            reg_0190.DESCR = dado.get('descr')
            reg_0190.write_line()
        return mensagem

    def bloco_g_registro0300(self, cr, uid, company_id, args_periodo_mes_e_ano, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        mensagem = ''

        # PATRIMÔNIO CIAP

        obj_patfiscal_ciap = self.pool.get('patrimoniofiscal.ciap')
        args = [('company_id', '=', company_id)]
        args.extend(args_periodo_mes_e_ano)

        new_args = list()
        new_args2 = list()
        for arg in args:
            arg = list(arg)
            if 'mes' not in arg and 'ano' not in arg:
                new_args.append(arg)
                new_args2.append(arg)
                continue
            mes_ano = str(arg[2])
            new_args.append([arg[0], arg[1], mes_ano])
            if len(mes_ano) == 1:
                mes_ano = '0' + mes_ano
            new_args2.append([arg[0], arg[1], mes_ano])

        ciap_ids = obj_patfiscal_ciap.search(cr, uid, args, context=context)
        if not ciap_ids:
            ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args, context=context)
            if not ciap_ids:
                ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args2, context=context)

        if not obj_adm_utils.admutl_get_parametros(
                cr, uid, 'adm', 'admpar_pat_gerar_ciap', context=context).get(
                'admpar_pat_gerar_ciap') or not ciap_ids:
            return mensagem

        brw_pat_ciap = obj_patfiscal_ciap.browse(cr, uid, ciap_ids, context=context)

        for i in brw_pat_ciap:
            if not i.bem_id:
                continue
            # Verificacao identificacao do tipo de mercadoria:-bem
            if not i.bem_id.parent_id:
                ident_merc = '1'
                cod_prnc = ''
            else:  # componente
                ident_merc = '2'
                cod_prnc = i.bem_id.parent_id.id
            if i.bem_id.grupos_id:
                if i.bem_id.grupos_id.conta_aquisicao_id:
                    cod_cta = i.bem_id.grupos_id.conta_aquisicao_id.conta.replace(".", "")
                else:
                    cod_cta = ''
            else:
                cod_cta = ''
            # REGISTRO 0300 CADASTRO DE BENS OU COMPONENTES DO ATIVO IMOBILIZADO
            reg_0300 = bloco_0.registro_0300()
            # Codigo individualizado do bem ou componente adotado no controle
            # patrimonial do estabelecimento informante
            reg_0300.COD_IND_BEM = i.bem_id.codigo_bem
            # Identificacao do tipo de mercadoria:  1 = bem; 2 = componente.
            reg_0300.IDENT_MERC = ident_merc
            # Descricao do bem ou componente (modelo, marca e outras caracteristicas
            # necessárias a sua individualizacao)
            reg_0300.DESCR_ITEM = i.bem_id.name
            # Codigo de cadastro do bem principal nos casos em que o bem ou componente
            # ( campo 02) esteja vinculado a um bem principal.
            reg_0300.COD_PRNC = cod_prnc
            # Codigo da conta analitica de contabilizacao do bem ou componente
            # (campo 06 do Registro 0500).
            reg_0300.COD_CTA = cod_cta
            # Numero total de parcelas a serem apropriadas, segundo a legislacao de cada
            # unidade federada.
            reg_0300.NR_PARC = i.bem_id.qtd_parcelas
            reg_0300.write_line()
            self.bloco_g_registro0305(cr, uid, i.bem_id.id, context)

        return mensagem

    def bloco_g_registro0305(self, cr, uid, bem_id, context=None):
        context = context and context.copy() or {}
        mensagem = ''

        if not bem_id:
            return mensagem

        brw_patrimonio_bens = self.pool.get('patrimonio.bens').browse(cr, uid, bem_id, context)

        # Verificacao identificacao do tipo de mercadoria:-bem
        if not brw_patrimonio_bens.parent_id:
            ident_merc = '1'
        else:  # componente
            ident_merc = '2'

        vida_util = ''
        if brw_patrimonio_bens.grupos_id:
            vida_util = brw_patrimonio_bens.grupos_id.vida_util

        if ident_merc == '1':  # verificar, pois nao existe info. no sistema
            # REGISTRO 0305  INFORMAÇÃO SOBRE A UTILIZAÇÃO DO BEM
            reg_0305 = bloco_0.registro_0305()
            # Codigo do centro de custo onde o bem está sendo ou será utilizado
            # (campo 03 do Registro 0600)
            reg_0305.COD_CCUS = brw_patrimonio_bens.area_utilizacao
            # Descricao sucinta da funcao do bem na atividade do estabelecimento
            reg_0305.FUNC = brw_patrimonio_bens.descricao_funcao
            # Vida util estimada do bem, em numero de meses
            reg_0305.VIDA_UTIL = vida_util
            reg_0305.write_line()
        return mensagem

    def bloco_g_registro0500(self, cr, uid, company_id, args_periodo_mes_e_ano, sped_data_inicial,
                             context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        mensagem = ''

        # PATRIMÔNIO CIAP

        obj_patfiscal_ciap = self.pool.get('patrimoniofiscal.ciap')
        args = [('company_id', '=', company_id)]
        args.extend(args_periodo_mes_e_ano)

        new_args = list()
        new_args2 = list()
        for arg in args:
            arg = list(arg)
            if 'mes' not in arg and 'ano' not in arg:
                new_args.append(arg)
                new_args2.append(arg)
                continue
            mes_ano = str(arg[2])
            new_args.append([arg[0], arg[1], mes_ano])
            if len(mes_ano) == 1:
                mes_ano = '0' + mes_ano
            new_args2.append([arg[0], arg[1], mes_ano])

        ciap_ids = obj_patfiscal_ciap.search(cr, uid, args, context=context)
        if not ciap_ids:
            ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args, context=context)
            if not ciap_ids:
                ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args2, context=context)

        if not obj_adm_utils.admutl_get_parametros(
                cr, uid, 'adm', 'admpar_pat_gerar_ciap', context=context).get(
                'admpar_pat_gerar_ciap') or not ciap_ids:
            return mensagem

        brw_pat_ciap = obj_patfiscal_ciap.browse(cr, uid, ciap_ids, context=context)

        conta_aquisicao_ids = list()
        for i in brw_pat_ciap:
            if not i.bem_id:
                continue

            if not i.bem_id.grupos_id:
                continue

            if not i.bem_id.grupos_id.conta_aquisicao_id:
                continue

            if i.bem_id.grupos_id.conta_aquisicao_id.id in conta_aquisicao_ids:
                continue
            conta_aquisicao_ids.append(i.bem_id.grupos_id.conta_aquisicao_id.id)

            cod_cta = i.bem_id.grupos_id.conta_aquisicao_id.conta.replace(".", "")
            cod_nat_cc = i.bem_id.grupos_id.conta_aquisicao_id.natureza
            if cod_nat_cc == 'A':
                cod_nat_cc = '01'
            elif cod_nat_cc == 'P':
                cod_nat_cc = '02'
            elif cod_nat_cc == 'L':
                cod_nat_cc = '03'
            elif cod_nat_cc == 'R':
                cod_nat_cc = '04'
            elif cod_nat_cc == 'C':
                cod_nat_cc = '05'
            else:
                cod_nat_cc = '09'

            # REGISTRO 0500 PLANO DE CONTAS CONTÁBEIS
            if not i.bem_id.grupos_id.conta_aquisicao_id.conta_tipo:
                mensagem += _('* TIPO DA CONTA nao informada.\n')
            if not i.bem_id.grupos_id.conta_aquisicao_id.grau:
                mensagem += _('* NIVEL DA CONTA nao informada.\n')
            if not i.bem_id.grupos_id.conta_aquisicao_id.name:
                mensagem += _('* DESCRICAO DA CONTA nao informada.\n')

            reg_0500 = bloco_0.registro_0500()
            reg_0500.DT_ALT = self.formata_data(
                sped_data_inicial)  # Data da inclusao/alteracao
            # Codigo da natureza da conta/grupo de contas:01 - Contas de ativo;
            # 02 - Contas de passivo;03 - Patrimônio liquido;
            # 04 - Contas de movimento_notasado;05 - Contas de compensacao;
            # 09 - Outras.
            reg_0500.COD_NAT_CC = cod_nat_cc
            # Indicador do tipo de conta: S - Sintetica (grupo de contas);
            # A - Analitica (conta).
            reg_0500.IND_CTA = i.bem_id.grupos_id.conta_aquisicao_id.conta_tipo
            # Nivel da conta analitica/grupo de contas.
            reg_0500.NIVEL = i.bem_id.grupos_id.conta_aquisicao_id.grau
            # Codigo da conta analitica/grupo de contas.
            reg_0500.COD_CTA = cod_cta
            # Nome da conta analitica/grupo de contas.
            reg_0500.NOME_CTA = i.bem_id.grupos_id.conta_aquisicao_id.name
            reg_0500.write_line()

        return mensagem

    def bloco_g_registro0600(self, cr, uid, company_id, args_periodo_mes_e_ano, sped_data_inicial,
                             context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        mensagem = ''

        # PATRIMÔNIO CIAP

        obj_patfiscal_ciap = self.pool.get('patrimoniofiscal.ciap')
        args = [('company_id', '=', company_id)]
        args.extend(args_periodo_mes_e_ano)

        new_args = list()
        new_args2 = list()
        for arg in args:
            arg = list(arg)
            if 'mes' not in arg and 'ano' not in arg:
                new_args.append(arg)
                new_args2.append(arg)
                continue
            mes_ano = str(arg[2])
            new_args.append([arg[0], arg[1], mes_ano])
            if len(mes_ano) == 1:
                mes_ano = '0' + mes_ano
            new_args2.append([arg[0], arg[1], mes_ano])

        ciap_ids = obj_patfiscal_ciap.search(cr, uid, args, context=context)
        if not ciap_ids:
            ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args, context=context)
            if not ciap_ids:
                ciap_ids = obj_patfiscal_ciap.search(cr, uid, new_args2, context=context)

        if not obj_adm_utils.admutl_get_parametros(
                cr, uid, 'adm', 'admpar_pat_gerar_ciap', context=context).get(
                'admpar_pat_gerar_ciap') or not ciap_ids:
            return mensagem

        brw_pat_ciap = obj_patfiscal_ciap.browse(cr, uid, ciap_ids, context=context)

        list_area_utilizacao = list()
        for i in brw_pat_ciap:
            if not i.bem_id:
                continue

            if i.bem_id.area_utilizacao in list_area_utilizacao:
                continue
            list_area_utilizacao.append(i.bem_id.area_utilizacao)

            # Verificacao identificacao do tipo de mercadoria:-bem
            if not i.bem_id.parent_id:
                ident_merc = '1'
            else:  # componente
                ident_merc = '2'

            if ident_merc == '1':  # verificar, pois nao existe info. no sistema
                # REGISTRO 0600: CENTRO DE CUSTOS
                areas = {'1': 'Area operacional',
                         '2': 'Area administrativa',
                         '3': 'Area Produtiva',
                         '4': 'Area de apoio a producao',
                         '5': 'Area administrativa'}
                reg_0600 = bloco_0.registro_0600()
                reg_0600.DT_ALT = self.formata_data(
                    sped_data_inicial)  # Data da inclusao/alteracao.
                reg_0600.COD_CCUS = i.bem_id.area_utilizacao  # Codigo do centro de custos.
                reg_0600.CCUS = areas[i.bem_id.area_utilizacao]
                # Nome do centro de custos.
                reg_0600.write_line()
        return mensagem
