# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from SigUtils import numero
from openerp.tools.translate import _


class patrimonio_bens(osv.Model):
    _inherit = 'patrimonio.bens'


    def patfis_criar_ciap(self, cr, uid, bem_id, data_implantacao_saldo,
                          credito_icms, qtd_parcelas):
        """
        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificação de perfil,
        @param bem_id: {int} ID de identificação do bem (patrimonio)
        @param data_implantacao_saldo:
        @param credito_icms:
        @param qtd_parcelas:
        @return: {dict} o número

        @since: 30.04.2013
        @change: 30.04.2013 - pierre.barbosa
        """

        # ___________________________ Objects ________________________________
        obj_adm_parametro = self.pool.get('adm.parametro')
        obj_empresa = self.pool.get('adm.empresa.objeto')
        obj_moeda_valor = self.pool.get('adm.moeda.valor')
        obj_patrimoniofiscal_ciap = self.pool.get('patrimoniofiscal.ciap')

        id_parametro = obj_adm_parametro.search(cr, uid, [])

        if not id_parametro:
            msg = 'Nao foi cadastrado parametros para esta empresa!'
            raise osv.except_osv('Atencao!', msg)

        if qtd_parcelas <= 0:
            raise osv.except_osv(
                _('Atencao'),
                _('A "Qtd de parcelas", na aba "Informacoes complementares", deve ser maior que '
                  'zero.'))

        empresa_id = obj_empresa.getEmpresa(cr, uid, 'patrimoniofiscal.ciap')
        movimento = {'company_id': empresa_id, 'bem_id': bem_id}

        ano = int(data_implantacao_saldo[:4])
        mes = int(data_implantacao_saldo[5:7])
        resMoeda = obj_moeda_valor.achar_valor_moeda(cr, uid, 'UPF', ano,
                                                     '%02d' % mes, operador='=',
                                                     vlDefault='1.00')
        if not resMoeda.get('status'):
            msg = 'Erro ao retornar o valor da moeda.'
            raise osv.except_osv('Atencao!', resMoeda.get('mensagem') or msg)

        _valorEmUPF = float(credito_icms) / float(resMoeda.get('valor'))
        _valorParcelaEmUPF = float(numero.getNumeroEditado(_valorEmUPF /
                                                           int(qtd_parcelas), 3,
                                                           '.', '')[:-1])
        _valorPrimeiraEmUPF = float(numero.getNumeroEditado(_valorEmUPF - (
            _valorParcelaEmUPF * (int(qtd_parcelas) - 1)), 3, '.', '')[:-1])

        for parcela in range(qtd_parcelas):
            movimento['ano'] = ano
            movimento['mes'] = '%02d' % mes
            movimento['parcela'] = parcela + 1

            if parcela == 0:
                movimento['valor_parcela_upf'] = _valorPrimeiraEmUPF
            else:
                movimento['valor_parcela_upf'] = _valorParcelaEmUPF

            _valor = float(movimento['valor_parcela_upf']) * \
                     float(resMoeda['valor'])
            movimento['valor_apropriacao'] = _valor
            args = [('bem_id', '=', bem_id),
                    ('ano', '=', movimento.get('ano')),
                    ('mes', '=', movimento.get('mes'))]

            mov_id = obj_patrimoniofiscal_ciap.search(cr, uid, args)

            if not mov_id:
                obj_patrimoniofiscal_ciap.create(cr, uid, movimento)
            else:
                obj_patrimoniofiscal_ciap.write(cr, uid, mov_id, movimento)

            mes += 1
            if mes > 12:
                ano += 1
                mes = 1

        return {}


    def patfis_button_gerar_ciap(self, cr, uid, ids, context=None):
        """

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos ID's
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {boolean} verdadeiro sempre.

        @change: 30.04.2013 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_empresa_objeto = self.pool.get('adm.empresa.objeto')

        # Estas duas linhas vao gerar o ciap para todos os bens, se quizer ape-
        # nas para o bem selecionado, retire-a.
        company_id = obj_empresa_objeto.getEmpresa(cr, uid, self._name, context=context)
        ids = self.search(cr, uid, [('company_id', '=', company_id)], context=context)

        for _id in ids:
            campos = ['data_implantacao_saldo', 'credito_icms', 'qtd_parcelas']
            ciap_dados = self.read(cr, uid, _id, campos, context=context)

            if ciap_dados and isinstance(ciap_dados, list):
                ciap_dados = ciap_dados[0]

            if ciap_dados and isinstance(ciap_dados, dict):
                if ciap_dados.get('credito_icms') <= 0:
                    raise osv.except_osv(_('Atencao'),
                                         _('O valor do "Credito de ICMS" deve ser maior que zero.'))
                self.patfis_criar_ciap(cr, uid, _id,
                                       ciap_dados.get('data_implantacao_saldo'),
                                       ciap_dados.get('credito_icms'),
                                       ciap_dados.get('qtd_parcelas'))

        return True

