# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class patrimoniofiscal_ciap_controle(osv.Model):
    """
    Tabela de controle de apropriacao do ciap

    @change: 29.04.2013 - pierre.barbosa
    """

    _name = 'patrimoniofiscal.ciap_controle'
    _rec_name = 'mes'

    from funcoes import _patfis_get_saldo

    _columns = {
        # ___________________________ Many2One _______________________________
        'company_id': fields.many2one('res.company', 'Empresa',                     # ID da Empresa
                                      ondelete='cascade', required=True,
                                      select=True),

        # ___________________________ One2Many _______________________________
        'ciap_bens_ids': fields.one2many('patrimoniofiscal.ciap',
                                         'ciap_controle_id', 'Detalhamento'),

        # ___________________________ Char ___________________________________
        'ano': fields.char('Ano', size=4, required=True),                           # Ano de apropriacao
        'mes': fields.char('Mes', size=2, required=True),                           # Mes de apropriacao

        # ___________________________ Float __________________________________
        'saidas_tributadas': fields.float('Saidas tributadas', digits=(12, 2),      # Valor da receita tributada
                                          required=True),
        'saidas_total': fields.float('Total das saidas', digits=(12, 2),
                                     required=True), # Valor total da receita
        'coeficiente': fields.float('Coeficiente', digits=(10, 8),                  # Coeficiente para apropriacao (receitas tributadas / total),
                                    readonly=True),
        'total_apropriar': fields.float('Total a apropriar', digits=(12, 2),        # Valor total a ser apropriado
                                        readonly=True),

        # ___________________________ Integer ________________________________
        'nota_fiscal': fields.integer('Nota fiscal',                                # Numero a nota fiscal
                                      help='Numero da nota fiscal de apropriacao do ICMS'),
        'serie': fields.integer('Serie'),                                           # Serie da nota fiscal

        # ___________________________ Date ___________________________________
        'data_emissao': fields.date('Data de emissao'),                             # Data de emissao da nota fiscal

        # ___________________________ Function _______________________________
        'saldo_acumulado':  fields.function(_patfis_get_saldo, method=True,         # Saldo acumulado para calculo da apropriacao
                                            string='Saldo acumulado',
                                            type='float', store=True),
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c:
        s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }

    _sql_constraints = [
        ('ciap_controle_unique', 'unique (company_id, ano, mes)',
         'Nao pode haver mais de um movimento no mesmo mes.'),
    ]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4

