# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class patrimoniofiscal_ciap(osv.Model):
    """
    Tabela de controle das amortizacoes

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    @since: 29.04.2013
    """

    from funcoes import patfis_get_upf

    _name = 'patrimoniofiscal.ciap'
    _rec_name = 'bem_id'

    _columns = {
        # ___________________________ Many2One _______________________________
        'company_id': fields.many2one('res.company', 'Empresa',                     # ID da empresa
                                      ondelete='cascade', required=True,
                                      select=True),
        'bem_id': fields.many2one('patrimonio.bens', 'Bem',                         # ID do bem
                                  ondelete='restrict', required=True,
                                  select=True),
        'ciap_controle_id': fields.many2one('patrimoniofiscal.ciap_controle',             # ID do CIAP (controle)
                                            'Controle', select=True,
                                            ondelete='set null'),

        # ___________________________ Char ___________________________________
        'ano': fields.char('Ano', size=4, required=True),                           # Ano de apropriacao
        'mes': fields.char('Mês', size=2, required=True),                           # Mês de apropriacao

        # ___________________________ Integer ________________________________
        'parcela': fields.integer('Parcela', required=True),                        # Parcela da amortizacao

        # ___________________________ Float __________________________________
        'valor_apropriacao': fields.float('ICMS', digits=(12, 2)),                  # Valor do ICMS da apropriacao
        'valor_parcela_upf': fields.float('Valor parcela em UPF',                   # Valor da parcela em UPF
                                          digits=(12, 2), required=True),

        # ___________________________ Function _______________________________
        'valor_upf': fields.function(patfis_get_upf, method=True, store=False,
                                     string='Valor UPF', type='float', )
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c:
        s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }

    _sql_constraints = [
        ('ciap_movimento_unique', 'unique (company_id, bem_id, ano, mes)',
         'Nao pode haver mais de um movimento no mesmo mes.'),
    ]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4


