# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class pcp_ficha_tecnica_capa(osv.Model):
    """
    Cadastro da Ficha Tecnica
    """
    _name = 'pcp.ficha.tecnica.capa'
    _inherit = 'pcp.ficha.tecnica.capa'

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(pcp_ficha_tecnica_capa, self).default_get(cr, uid, fields_list, context)
        campos_financeiro = ['usar_centro_custos_e_eventos']
        financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', campos_financeiro, context=context)
        if financeiro_parametro_dados.get('usar_centro_custos_e_eventos'):
            res['pcpftc_utiliza_custo_evento'] = True
        return res

    def pcpftc_onchange_produto_unidade_embalagem(self, cr, uid, ids, produto_embalagem_id,
                                                  quantidade, context=None):
        context = context and context.copy() or {}
        res = {}
        if not produto_embalagem_id:
            return {'value': {'pcpftc_quantidade_unidade': 0}}
        brw_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        aceita_fracao = brw_estoque_produto_embalagem.embalagem_id.unidade_id.fracao
        if not aceita_fracao and quantidade != int(quantidade):
            res['value'] = {'pcpftc_quantidade_unidade': 0}
            res['warning'] = {'title': _('Atencao'),
                              'message': _("A 'Embalagem (opcional)' nao aceita fracionamento")}
        return res

    def onchange_produto_id(self, cr, uid, ids, pcpftc_produto_id, context=None):
        context = context and context.copy() or {}
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        grade_ids = obj_estoque_produto_grade.search(cr, uid, [('produto_id', '=', pcpftc_produto_id)], context=context)
        vals = {'pcpftc_produto_peso_embalagem_id': False,
                'pcpftc_grade_id': False}
        if len(grade_ids) == 1:
            vals['pcpftc_grade_id'] = grade_ids[0]
            return {'value': vals}

        return {'value': vals}

    def pcpftc_onchange_grade_id(self, cr, uid, ids, pcpftc_grade_id, context=None):
        context = context and context.copy() or {}
        vals = {'pcpftc_produto_peso_embalagem_id': False}
        if not pcpftc_grade_id:
            return {'value': vals}

        obj_locacao = self.pool.get('estoque.local.locacao')
        locacao_ids = obj_locacao.search(cr, uid, [('grade_id', '=', pcpftc_grade_id)], context=context)
        if not locacao_ids:
            return {'value': vals}

        dados_locacao = obj_locacao.read(cr, uid, locacao_ids, ['estllc_produto_embalagem_id'], context)
        if len(dados_locacao) == 1:
            vals['pcpftc_produto_peso_embalagem_id'] = dados_locacao[0].get('estllc_produto_embalagem_id')
            return {'value': vals}

        return {'value': vals}

    def pcpftc_onchange_pcpftc_quantidade_peso(self, cr, uid, ids, peso, unidade):
        """
        Calcula o valor do peso referente a cada unidade.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param peso: {float} valor do peso,
        @param unidade: {int} o numero de unidades,
        @return: {dict} o valor do peso para cada unidade.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {'pcpftc_peso_unidade': 0.0}}

        if unidade:
            result['value']['pcpftc_peso_unidade'] = self._calcular_peso_unidade(peso, unidade)

        return result

    def pcpftc_onchange_pcpftc_ficha_tecnica_embalagem_ids(self, cr, uid, ids, embalagem_ids):
        """
        Calcula o valor do peso referente a cada unidade.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param embalagem_ids: {int} o numero de unidades,
        @return: {dict} o valor do peso para cada unidade.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        embalagem_ids = filter(lambda embalagem: embalagem[0] == 4, embalagem_ids)
        embalagem_ids = map(lambda embalagem: embalagem[1], embalagem_ids)
        result = {'value': {
            'pcpftc_valor_embalagem': self._pcpftc_calcular_total_embalagem(
                cr, uid, embalagem_ids)}
        }

        return result

    def pcpftc_onchange_pcpftc_quantidade_unidade(self, cr, uid, ids, peso, unidade,
                                                  produto_embalagem_id, context=None):
        """
        Calcula o valor do peso referente a cada unidade.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param peso: {float} valor do peso,
        @param unidade: {int} o numero de unidades,
        @return: {dict} o valor do peso para cada unidade.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {'pcpftc_peso_unidade': 0.0}}

        if unidade:
            result['value']['pcpftc_peso_unidade'] = self._calcular_peso_unidade(peso, unidade)

        if not produto_embalagem_id:
            result['value'].update({'pcpftc_quantidade_unidade': 0})
            return result

        brw_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem').browse(
            cr, uid, produto_embalagem_id, context)
        aceita_fracao = brw_estoque_produto_embalagem.embalagem_id.unidade_id.fracao
        if not aceita_fracao and unidade != int(unidade):
            result['value'].update({'pcpftc_quantidade_unidade': 0})
            result['warning'] = {'title': _('Atencao'),
                                 'message': _("A 'Embalagem (opcional)' nao aceita fracionamento")}

        return result

    def _pcpftc_calcular_total_embalagem(self, cr, uid, embalagem_ids, context=None):
        """
        Calcula o valor total do custo das embalagens utilizadas na ficha tecnica.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param embalagem_ids: {list} lista com os IDs dos produtos onde tipo_item deve ser igual a
        ´´2 - Embalagem´´,
        @return: {float} somatorio do preco de venda das embalagens.
        @see _pcpftc_fields_function (pcp/model/funcoes.py)

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pcp_util = self.pool.get('pcp.util')

        context.update({'pcpftc_total_embalagem_padrao': True})
        result = 0.0
        dict_custo_total_por_embalagem = obj_pcp_util.pcputl_get_custo_unitario_total(
            cr, uid, 'E', embalagem_ids, context)
        for embalagem_id in embalagem_ids:
            result += dict_custo_total_por_embalagem.get(embalagem_id) or 0

        return result

    @staticmethod
    def _calcular_peso_unidade(peso, unidade):
        """
        Calcula o peso referente a cada unidade (geralmente do produto).

        @param peso: {float} valor do peso,
        @param unidade: {int} o numero de unidades.
        @return: {float} o peso de cada unidade.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = 0.0

        if unidade:  # Evita divisao por zero. ;)
            result = peso/unidade

        return result

    def pcpftc_button_salvar(self, cr, uid, ids, context=None):
        res = self.write(cr, uid, ids, {}, context)
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('pcpftc_quantidade_peso') == 0:
            raise osv.except_osv(_('Atencao'), _('Informe uma quantidade valida'))
        obj_estoque_produto = self.pool.get('estoque.produto')
        active = vals.get('active') or False
        produto_id = vals.get('pcpftc_produto_id')
        browse_estoque_produto = obj_estoque_produto.browse(cr, uid, produto_id)
        producao_propria = browse_estoque_produto.producao_propria or False
        tipo_item = browse_estoque_produto.tipo_item
        # se ficha estiver ativa (active = True) e produto estiver 'producao_propria' = False, deve ser
        # 'producao_propria' = True
        # o produto (tipo_item) nao pode ser materia prima ('01')
        if tipo_item != '01':
            if active:
                if not producao_propria:
                    obj_estoque_produto.write(cr, uid, produto_id, {'producao_propria': True}, context)
            else:
                if producao_propria:
                    obj_estoque_produto.write(cr, uid, produto_id, {'producao_propria': False}, context)
        res = super(pcp_ficha_tecnica_capa, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'pcpftc_quantidade_peso' in vals and vals.get('pcpftc_quantidade_peso') == 0:
            raise osv.except_osv(_('Atencao'), _('Informe uma quantidade valida'))
        browse_pcp_ficha_tecnica_capa = self.browse(cr, uid, ids[0])
        obj_estoque_produto = self.pool.get('estoque.produto')
        if vals.get('active'):
            active = vals.get('active') or False
        else:
            active = browse_pcp_ficha_tecnica_capa.active or False
        produto_id = vals.get('pcpftc_produto_id') or False
        if produto_id:
            produto_original_id = browse_pcp_ficha_tecnica_capa.pcpftc_produto_id.id
            obj_estoque_produto.write(cr, uid, produto_original_id, {'producao_propria': False}, context)

            browse_estoque_produto = obj_estoque_produto.browse(cr, uid, produto_id)
            producao_propria = obj_estoque_produto.browse(cr, uid, produto_id).producao_propria or False
            tipo_item = browse_estoque_produto.tipo_item
            # se ficha estiver ativa (active = True) e produto estiver 'producao_propria' = False, deve ser
            # 'producao_propria' = True
            # o produto (tipo_item) nao pode ser materia prima ('01')
            if tipo_item != '01':
                if active:
                    if not producao_propria:
                        obj_estoque_produto.write(cr, uid, produto_id, {'producao_propria': True}, context)
                else:
                    if producao_propria:
                        obj_estoque_produto.write(cr, uid, produto_id, {'producao_propria': False}, context)
        res = super(pcp_ficha_tecnica_capa, self).write(cr, uid, ids, vals, context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_estoque_produto = self.pool.get('estoque.produto')
        browse_pcp_ficha_tecnica_capa = self.browse(cr, uid, ids)
        for x in browse_pcp_ficha_tecnica_capa:
            produto_id = x.pcpftc_produto_id.id
            obj_estoque_produto.write(cr, uid, produto_id, {'producao_propria': False}, context)
        res = super(pcp_ficha_tecnica_capa, self).unlink(cr, uid, ids, context)
        return res

    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if context.get('pcpftc_button_duplicar'):
            return super(pcp_ficha_tecnica_capa, self).copy(cr, uid, id, default, context)

        raise osv.except_osv(_('Atencao!'),
                             _('Para duplicar a Ficha Tecnica, '
                               '\nutilize o outro botão "Duplicar",'
                               '\nao lado do "Atualizar!"'))

    def pcpftc_button_duplicar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_capa = self.browse(cr, uid, ids[0], context)
        context.update({'pcpftc_button_duplicar': True,
                        'pcpftc_produto_id': brw_capa.pcpftc_produto_id.id})

        mod_obj = self.pool.get('ir.model.data')
        result = mod_obj.get_object_reference(cr, uid, 'pcp', 'view_wiz_duplicar_ficha_form')
        view_id = result and result[1] or False
        return {
            'name': 'Copiar Ficha Tecnica',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'wizard.duplicar.ficha',
            'view_id': view_id,
            'target': 'new',
            'context': context,
            'type': 'ir.actions.act_window',
        }
