#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
from datetime import datetime, date
from openerp.tools.translate import _
from lxml import etree

PCPOPR_STATE_ESBOCO = 'E'
PCPOPR_STATE_ABERTO = 'A'
PCPOPR_STATE_NF_GERADA = 'N'
PCPOPR_STATE_FECHADO = 'F'
PCPOPR_STATE_CANCELADO = 'C'
PCPOPR_CODE_SEQUENCE = 'numero_pcp_ordem_producao'

READ_LOAD = '_classic_write'


class pcp_ordem_producao(osv.Model):
    _inherit = 'pcp.ordem.producao'  # Prefixo: pcpopr_

    def pcpopr_onchange_quantidade_unidade(self, cr, uid, ids, ficha_id, unidade, context=None):
        """
        Calcula a quantidade de receitas em funcao do peso informado.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} ID da ficha tecnica do produto,
        @param unidade: {float} a quantidade em peso utilizada no calculo da quantidade de receita,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {dict} dicionario com os valores dos campos de quantidade peso e quantidade
        unidade com valor zero se houver informacao para o campo quantidade de receita.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {}, 'warning': {}}

        if not ficha_id:
            return result

        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')

        campos = ['pcpftc_quantidade_unidade']
        dados_ficha_tecnica = obj_pcp_ficha_tecnica_capa.read(cr, uid, ficha_id, campos, context=context)

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, list):
            dados_ficha_tecnica = dados_ficha_tecnica[0]

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, dict):
            quantidade_unidade = float(dados_ficha_tecnica.get('pcpftc_quantidade_unidade', 0.0))

            if quantidade_unidade:  # Evita divisao por zero. ;)
                unidade = float(unidade)
                result['value'] = {'pcpopr_quantidade': unidade / quantidade_unidade}

        return result

    def pcpopr_onchange_quantidade_peso(self, cr, uid, ids, ficha_id, peso, context=None):
        """
        Calcula a quantidade de receitas em funcao do peso informado.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} ID da ficha tecnica do produto,
        @param peso: {float} a quantidade em peso utilizada no calculo da quantidade de receita,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {dict} dicionario com os valores dos campos de quantidade peso e quantidade
        unidade com valor zero se houver informacao para o campo quantidade de receita.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {}

        if not ficha_id:
            return result

        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')

        campos = ['pcpftc_quantidade_peso']
        dados_ficha_tecnica = obj_pcp_ficha_tecnica_capa.read(cr, uid, ficha_id, campos, context=context)

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, list):
            dados_ficha_tecnica = dados_ficha_tecnica[0]

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, dict):
            quantidade_peso = float(dados_ficha_tecnica.get('pcpftc_quantidade_peso', 0.0))

            if quantidade_peso:  # Evita divisao por zero. ;)
                peso = float(peso)
                result = {'value': {'pcpopr_quantidade': peso / quantidade_peso}}

        return result

    def onchange_ficha_tecnica(self, cr, uid, ids, ficha_tecnica_id, context=None):
        context = context and context.copy() or {}
        vals = {'value': {}}
        if ficha_tecnica_id:
            obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
            brw_ficha_tecnica = obj_pcp_ficha_tecnica_capa.browse(cr, uid, ficha_tecnica_id)
            quant_unidade = brw_ficha_tecnica.pcpftc_quantidade_unidade
            vals['value'] = {'pcp_flag_uni_quantidade': quant_unidade}
        return vals

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        __name = 'Ordem Producao'
        vals = {
            'name': __name,
            'code': PCPOPR_CODE_SEQUENCE,
            'padding': 6,
            'company_id': company_id,
            'prefix': 'OP',
            'implementation': 'no_gap',
        }
        obj_ir_sequence_type = self.pool.get('ir.sequence.type')
        consulta = obj_ir_sequence_type.search(cr, uid, [('code', '=', PCPOPR_CODE_SEQUENCE)],
                                               context=context)
        if not consulta:
            obj_ir_sequence_type.create(cr, uid, {'name': __name, 'code': PCPOPR_CODE_SEQUENCE},
                                        context)
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        return True

    def pcpopr_gerar_nf_capa(self, cr, uid, pcp_company_id, pcp_destinatario_id, pcp_tipo, pcp_nf_numero=False,
                             pcp_nf_serie=False, pcp_nf_total=0, context=None):
        context = context and context.copy() or {}
        res = False
        context.update({'ordem_producao': pcp_tipo})
        obj_fiscal_parametro = self.pool.get('fiscal.parametro')
        obj_estoque_movimento_capa = self.pool.get('estoque.movimento.capa')
        fispar_ids = obj_fiscal_parametro.search(cr, uid, [('company_id', '=', pcp_company_id)], context=context)
        if not fispar_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum parametro do fiscal encontra-se cadastrado para a empresa.'))
        else:
            if pcp_tipo == 'E':
                field_fispar_natureza = 'fispar_pcpopr_natureza_entrada_id'
            else:
                field_fispar_natureza = 'fispar_pcpopr_natureza_saida_id'

            fispar_dados = obj_fiscal_parametro.read(
                cr, uid, fispar_ids[0], [field_fispar_natureza, 'condicao_pgto_transferencia_id'],
                context=context, load=READ_LOAD
            )

            if fispar_dados and isinstance(fispar_dados, list):
                fispar_dados = fispar_dados[0]

            pcp_natureza_id = fispar_dados.get(field_fispar_natureza)
            if not pcp_natureza_id:
                erro_natureza = _('Natureza de %s para Nota Fiscal nao cadastrada no parametro do fiscal') \
                                % (_('entrada') if pcp_tipo == 'E' else _('saida'))
                raise osv.except_osv(
                    _('Atencao'),
                    erro_natureza
                )

            if not fispar_dados.get('condicao_pgto_transferencia_id'):
                erro_condicao_tranferencia = _("* A CONDICAO DE PAGAMENTO PARA TRANSFERENCIA nao "
                                               "encontra-se cadastrada no parametro do fiscal. ")
                raise osv.except_osv(
                    _('Atencao'), erro_condicao_tranferencia)
            else:
                condicao_transferencia_id = fispar_dados['condicao_pgto_transferencia_id']
                obj_adm_condicao_forma = self.pool.get('adm.condicao.forma')
                browse_condicao_forma = obj_adm_condicao_forma.browse(cr, uid, condicao_transferencia_id)
                nf_condicao_ids = [(0, 0, {'forma_pagto_id': browse_condicao_forma.forma_pagamento_id.id,
                                           'condicao_forma_id': condicao_transferencia_id,
                                           'valor': pcp_nf_total
                                           })]

            nf_id = self.pool.get('fiscal.utils').gerar_nf(
                cr, uid,
                company_id=pcp_company_id,
                destinatario_id=pcp_destinatario_id,
                natureza_id=pcp_natureza_id,
                nf_condicao_ids=nf_condicao_ids,
                tipo=pcp_tipo,
                modelo_nf_id=False,
                end_entrega=False,
                end_coleta=False,
                numero_nf=pcp_nf_numero,
                serie_nf=pcp_nf_serie,
                context=context
            )

            pessoa_id = self.pool.get('res.users').browse(cr, uid, uid).pessoa_id.id
            if not pessoa_id:
                raise osv.except_osv(_('Atencao'), _('Pessoa nao cadastrada para o usuario!'))
            obj_rh_funcionario = self.pool.get('rh.funcionario')
            func_id = obj_rh_funcionario.search(cr, uid, [('pessoa', '=', pessoa_id)], context=context)
            if not func_id:
                raise osv.except_osv(_('Atencao'), _('Voce tem que se cadastrar como um Funcionario...'))
            res = nf_id
            # refresh nas funções da nf
            self.pool.get('fiscal.nf.capa').write(cr, uid, nf_id, {}, context=context)
        return res

    def pcpopr_gerar_nf_itens(self, cr, uid, pcp_nf_id, pcp_company_id, pcp_produto_id, pcp_grade_id, pcp_embalagem_id,
                              pcp_quantidade, pcp_vr_unitario=0, pcp_preco_venda=0, context=None):

        context = context and context.copy() or {}
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        estpar_ids = obj_estoque_parametro.search(cr, uid, [('company_id', '=', pcp_company_id)], context=context)
        if not estpar_ids:
            raise osv.except_osv(_('Atencao'), _('Nenhum parametro do estoque encontra-se cadastrado para a empresa.'))
        else:
            estpar_dados = obj_estoque_parametro.read(
                cr, uid, estpar_ids[0], ['local_venda_padrao_id'], context=context, load=READ_LOAD
            )
            if estpar_dados and isinstance(estpar_dados, list):
                estpar_dados = estpar_dados[0]

            local_venda_padrao_id = estpar_dados.get('local_venda_padrao_id')
            if not local_venda_padrao_id:
                raise osv.except_osv(_('Atencao'), _('Local de venda padrao nao cadastrado no parametro do estoque'))

            self.pool.get('fiscal.utils').gerar_nf_itens(
                cr, uid,
                nf_id=pcp_nf_id,
                produto_id=pcp_produto_id,
                tipo='P',
                local_id=local_venda_padrao_id,
                produto_embalagem_id=pcp_embalagem_id,
                grade_id=pcp_grade_id,
                quantidade=pcp_quantidade,
                vr_unitario=pcp_vr_unitario,
                item_preco_venda=pcp_preco_venda,
                context=context
            )
            # refresh nas funções da nf
            self.pool.get('fiscal.nf.capa').write(cr, uid, pcp_nf_id, {}, context=context)
        return True

    ## __________________________ BUTTONS __________________________________
    def pcpopr_button_abrir(self, cr, uid, ids, context=None):

        context = context and context.copy() or {}
        values = {
            'pcpopr_inicio': datetime.now(),
            'state': PCPOPR_STATE_ABERTO
        }
        id = ids
        if isinstance(ids, list) and ids:
            id = ids[0]
        obj_ficha_tecnica = self.pool.get('pcp.ficha.tecnica.capa')
        ficha_tecnica_id = self.browse(cr, uid, id).pcpopr_ficha_tecnica_capa_id
        if ficha_tecnica_id:
            obj_ficha_tecnica_itens = self.pool.get('pcp.ficha.tecnica.itens')

            ficha_tecnica_itens = obj_ficha_tecnica.browse(cr, uid, ficha_tecnica_id.id). \
                pcpftc_ficha_tecnica_itens_ids
            msg = ''
            for x in ficha_tecnica_itens:
                obj_ficha_tecnica_itens.write(cr, uid, [x.id], {}, context)
                if x.pcpfti_custo_total <= 0:
                    msg += '* ' + x.pcpfti_produto_id.codigo + ' - ' + \
                           x.pcpfti_produto_id.name + ' \n'
            if msg:
                msg = _('Os seguintes produtos nao possuem custo: \n' + msg)
                raise osv.except_osv(_('Atencao'),
                                     msg)

        res = self.write(cr, uid, ids, values, context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_pcp_gerar_requisicao_automatica'])
        gerar_requisicao_automatica = estoque_parametro_dados.get(
            'estpar_pcp_gerar_requisicao_automatica')
        if gerar_requisicao_automatica:
            self.pcpopr_button_gerar_requisicoes(cr, uid, ids, context)
        return res

    def pcpopr_button_gerar_nf(self, cr, uid, ids, context=None):
        values = {}

        if isinstance(ids, (int, long)):
            ids = [ids]
        context = context and context.copy() or {}
        context['ctx_nf_propria'] = True
        context.update({'proprio': True, 'pcp_ordem_producao': True})


        # FIELDS TO READ
        field_numero = 'pcpopr_numero'
        field_quantidade = 'pcpopr_quantidade'
        field_nf_saida_id = 'pcpopr_nf_saida_id'
        field_nf_entrada_id = 'pcpopr_nf_entrada_id'
        field_ficha_tec_id = 'pcpopr_ficha_tecnica_capa_id'
        field_quantidade_produzida = 'pcpopr_quantidade_produzida'
        field_requisicao_ids = 'pcpopr_requisicao_ids'

        fields_to_read = [
            field_nf_entrada_id, field_nf_saida_id, field_ficha_tec_id, field_quantidade_produzida,
            field_numero, field_quantidade, field_requisicao_ids
        ]

        ordem_producao_dados = self.read(cr, uid, ids, fields_to_read, context=context, load=READ_LOAD)
        obj_company = self.pool.get('res.company')
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        company_id = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
        company_pessoa_id = obj_company.browse(cr, uid, company_id, context=context).rescon_pessoa_id.id
        company_endereco_id = obj_adm_pessoa_endereco.search(cr, uid, [('pessoa_id', '=', company_pessoa_id)],
                                                             context=context)
        if not company_endereco_id:
            raise osv.except_osv(
                _('Atencao'), _('Endereco nao cadastrado para a empresa.'))

        # VALIDACAO SE A QUANTIDADE PRODUZIDA E QUANTIDADE FOI INFORMADA
        erro_ordem_producao = ''
        ordens_sem_quantidade = ''
        ordens_sem_quantidade_produzida = ''
        for ordem_prod in ordem_producao_dados:
            numero = ordem_prod.get(field_numero) or _('** Numero nao encontrado **')
            quantidade = ordem_prod.get(field_quantidade) or 0
            quantidade_produzida = ordem_prod.get(field_quantidade_produzida) or 0

            if not quantidade > 0:
                ordens_sem_quantidade += '%s, ' % numero

            if not quantidade_produzida > 0:
                ordens_sem_quantidade_produzida += '%s, ' % numero

        if ordens_sem_quantidade:
            erro_ordem_producao += _('Quantidade nao informada nas Ordens de Producao: %s; ') % ordens_sem_quantidade
        if ordens_sem_quantidade_produzida:
            erro_ordem_producao += _('Quantidade a ser produzida nao informada nas Ordens de Producao: %s') % \
                                   ordens_sem_quantidade_produzida

        if erro_ordem_producao:
            raise osv.except_osv(_('Erro ao fechar a Ordem de Producao'), erro_ordem_producao)
        else:
            total_nf_entrada = 0
            total_nf_saida = 0
            for ordem_prod in ordem_producao_dados:
                op_id = ordem_prod['id']
                op_numero = ordem_prod.get(field_numero) or '** Numero nao encontrado **'
                op_nf_saida_id = ordem_prod.get(field_nf_saida_id)
                op_ficha_tec_id = ordem_prod.get(field_ficha_tec_id)
                op_nf_entrada_id = ordem_prod.get(field_nf_entrada_id)
                op_quantidade_produzida = ordem_prod.get(field_quantidade_produzida)
                op_requisicao_ids = ordem_prod.get(field_requisicao_ids)

                # ____ DADOS DA FICHA TECNICA _____
                fields_to_read_ficha_tec = [
                    'pcpftc_produto_id', 'pcpftc_grade_id', 'pcpftc_produto_peso_embalagem_id',
                    'pcpftc_quantidade_peso', 'pcpftc_ficha_tecnica_itens_ids'
                ]
                ficha_tec_dados = self.pool.get('pcp.ficha.tecnica.capa').read(
                    cr, uid, op_ficha_tec_id, fields_to_read_ficha_tec, context=context, load=READ_LOAD
                )
                if ficha_tec_dados and isinstance(ficha_tec_dados, list):
                    ficha_tec_dados = ficha_tec_dados[0]

                ft_produto_id = ficha_tec_dados.get('pcpftc_produto_id')
                ft_grade_id = ficha_tec_dados.get('pcpftc_grade_id')
                ft_embalagem_id = ficha_tec_dados.get('pcpftc_produto_peso_embalagem_id')
                ft_quantidade = ficha_tec_dados.get('pcpftc_quantidade_peso')
                ft_item_ids = ficha_tec_dados.get('pcpftc_ficha_tecnica_itens_ids')
                if not ft_item_ids:
                    raise osv.except_osv(
                        _('Atencao'), _('Ficha Tecnica sem itens cadastrados para a Ordem de Producao: %s') % op_numero
                    )

                fator = op_quantidade_produzida / ft_quantidade

                # ____ DADOS DOS ITENS DA FICHA TECNICA ____
                fields_to_read_ficha_tec_itens = [
                    'pcpfti_produto_id', 'pcpfti_grade_id', 'pcpfti_produto_embalagem_id', 'pcpfti_quantidade',
                    'pcpfti_custo_total'
                ]
                item_dados = self.pool.get('pcp.ficha.tecnica.itens').read(
                    cr, uid, ft_item_ids, fields_to_read_ficha_tec_itens, context=context, load=READ_LOAD
                )

                requisicao_capa_dados = obj_almoxarifado_requisicao_capa.browse(
                    cr, uid, op_requisicao_ids)
                requisicao_item_dados = []

                for requisicao in requisicao_capa_dados:
                    requisicao_item_dados.extend(requisicao.requisicao_item_ids)
                for i in requisicao_item_dados:
                    produto_id = i.produto_id.id
                    quantidade = i.quantidade_requisitada
                    dados_custo = filter(lambda i: i.get('pcpfti_produto_id') == produto_id,
                                         item_dados)
                    ficha_item_qtd = 1
                    if dados_custo:
                        ficha_item_qtd = dados_custo[0].get('pcpfti_quantidade') or 1
                        ficha_item_custo = dados_custo[0].get('pcpfti_custo_total') or 0
                    else:
                        preco_custo_dic = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id)
                        dados_custo = preco_custo_dic.get(produto_id) or {}
                        ficha_item_custo = dados_custo.get('estprc_ultimo_custo') or 0

                    total_custo = (quantidade * ficha_item_custo) / ficha_item_qtd
                    vr_unitario = 0
                    if quantidade > 0:
                        vr_unitario = total_custo / quantidade
                    quantidade_utilizada = quantidade * fator
                    total_nf_saida += round((vr_unitario * quantidade_utilizada), 2)

                custo_total = sum(map(
                    lambda fi: fi.get('pcpfti_custo_total') or 0, item_dados
                ))

                total_nf_entrada += (custo_total / ft_quantidade) * (ft_quantidade * fator)
                if not op_nf_entrada_id:
                    context.update({'nota_fiscal_entrada': True})
                    op_nf_entrada_id = self.pcpopr_gerar_nf_capa(
                        cr, uid,
                        pcp_company_id=company_id,
                        pcp_destinatario_id=company_pessoa_id,
                        pcp_tipo='E',
                        pcp_nf_numero=False,
                        pcp_nf_serie=False,
                        pcp_nf_total=total_nf_entrada,
                        context=context
                    )
                    if op_nf_entrada_id:
                        values['pcpopr_nf_entrada_id'] = op_nf_entrada_id

                        custo_total = sum(map(
                            lambda fi: fi.get('pcpfti_custo_total') or 0, item_dados
                        ))

                        self.pcpopr_gerar_nf_itens(
                            cr, uid,
                            pcp_nf_id=op_nf_entrada_id,
                            pcp_company_id=company_id,
                            pcp_produto_id=ft_produto_id,
                            pcp_grade_id=ft_grade_id,
                            pcp_embalagem_id=ft_embalagem_id,
                            pcp_quantidade=ft_quantidade * fator,
                            pcp_vr_unitario=custo_total / ft_quantidade,
                            pcp_preco_venda=0,
                            context=context
                        )

                if not op_nf_saida_id:
                    # GERA UMA NOTA FISCAL DE SAIDA PARA AS MATERIAS PRIMAS QUE FORAM UTILIZADAS NA PRODUCAO
                    context.update({'nota_fiscal_saida': True})
                    op_nf_saida_id = self.pcpopr_gerar_nf_capa(
                        cr, uid,
                        pcp_company_id=company_id,
                        pcp_destinatario_id=company_pessoa_id,
                        pcp_tipo='S',
                        pcp_nf_total=total_nf_saida,
                        context=context
                    )
                    if op_nf_saida_id:
                        # Nota fiscal de saida deve ser gerada a partir das requisicoes
                        values['pcpopr_nf_saida_id'] = op_nf_saida_id
                        requisicao_capa_dados = obj_almoxarifado_requisicao_capa.browse(
                            cr, uid, op_requisicao_ids)
                        requisicao_item_dados = []
                        for requisicao in requisicao_capa_dados:
                            requisicao_item_dados.extend(requisicao.requisicao_item_ids)

                        for i in requisicao_item_dados:
                            produto_id = i.produto_id.id
                            grade_id = i.grade_id.id
                            embalagem_id = i.produto_embalagem_id.id
                            quantidade = i.quantidade_requisitada
                            dados_custo = filter(lambda i: i.get('pcpfti_produto_id') == produto_id,
                                                 item_dados)
                            ficha_item_qtd = 1
                            if dados_custo:
                                ficha_item_qtd = dados_custo[0].get('pcpfti_quantidade') or 1
                                ficha_item_custo = dados_custo[0].get('pcpfti_custo_total') or 0
                            else:
                                preco_custo_dic = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_id)
                                ficha_item_custo = preco_custo_dic[produto_id].get('estprc_ultimo_custo') or 0
                            custo_total = (quantidade * ficha_item_custo) / ficha_item_qtd
                            vr_unitario = 0
                            if quantidade > 0:
                                vr_unitario = custo_total / quantidade

                            self.pcpopr_gerar_nf_itens(
                                cr, uid,
                                pcp_nf_id=op_nf_saida_id,
                                pcp_company_id=company_id,
                                pcp_produto_id=produto_id,
                                pcp_grade_id=grade_id,
                                pcp_embalagem_id=embalagem_id,
                                pcp_quantidade=quantidade,
                                pcp_vr_unitario=vr_unitario,
                                pcp_preco_venda=vr_unitario,
                                context=context
                            )
                if op_nf_entrada_id and op_nf_saida_id:
                    values.update({
                        'state': PCPOPR_STATE_NF_GERADA
                    })

                self.write(cr, uid, [op_id], values, context=context)
        return True

    def pcpopr_button_gerar_requisicoes(self, cr, uid, ids, context=None):
        """
        Gera ou altera requisicoes baseado nas embalagens utilizadas na ordem de producao,
        materias-primas e insumos da ficha tecnica do produdo.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {int} o numero da requisicao que foi alterada ou incluida.

        @change: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_res_users = self.pool.get('res.users')
        obj_rh_funcionario = self.pool.get('rh.funcionario')

        # Verifica se usuario esta cadastrado com funcionario
        pessoa_id = obj_res_users.browse(cr, uid, uid).pessoa_id.id

        if pessoa_id:
            funcionario_id = obj_rh_funcionario.search(cr, uid, [('pessoa', '=', pessoa_id)], context=context)
            if not funcionario_id:
                raise osv.except_osv(_('Atencao'), _('O usuario deve estar cadastrado como '
                                                     'funcionario'))
            else:
                funcionario_id = funcionario_id[0]
        else:
            funcionario_id = False

        result = map(lambda res: res, self._pcpopr_gerar_requisicao(
            cr, uid, ids, funcionario_id, context))

        return result

    def _pcpopr_gerar_requisicao(self, cr, uid, ids, funcionario_id, context=None):
        """
        Gera ou altera requisicoes baseado nas embalagens utilizadas na ordem de producao,
        materias-primas e insumos da ficha tecnica do produdo.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param funcionario_id: {int} ID do funcionario,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {list} lista com o ID da requisicao que foi alterada ou incluida ou mensagem de
        erro se nao existir um local para almoxarifado cadastrado no parametro do estoque.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        obj_estoquealmoxarifado_requisicao_item = self.pool.get('estoquealmoxarifado.requisicao.item')

        # Parametros do PCP
        dados_parametro = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_local_almoxarifado_pcp_id')
        local_almoxarifado_pcp_id = dados_parametro.get('estpar_local_almoxarifado_pcp_id', False)

        if not local_almoxarifado_pcp_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Local para almoxarifado nao esta cadastrado no parametro do '
                                   'estoque'))
        result = []

        for brw_ordem_producao in self.browse(cr, uid, ids, context):
            quantidade_receitas = brw_ordem_producao.pcpopr_quantidade
            brw_ficha_tecnica = brw_ordem_producao.pcpopr_ficha_tecnica_capa_id
            company_id = brw_ficha_tecnica.company_id.id

            # Consolida os itens das requisicoes diferente de esboco e que nao estao canceladas
            args = [('almrqc_ordem_producao_id', '=', brw_ordem_producao.id),
                    ('state', 'not in', ['R', 'C']),  # Diferentes de esboco (R) e cancelado (C)
                    ('almrqc_flag_exibir_requisicao', '=', True),
                    ('almrqc_devolucao', '=', False)]
            requisicao_ids = obj_almoxarifado_requisicao_capa.search(cr, uid, args, context=context)
            args = [('requisicao_id', 'in', requisicao_ids)]
            item_ids = obj_estoquealmoxarifado_requisicao_item.search(cr, uid, args, context=context)
            insumos = {}  # Materia-prima/insumos & embalagens

            for item in obj_estoquealmoxarifado_requisicao_item.browse(
                    cr, uid, item_ids, context):
                produto_id = item.produto_id.id
                grade_id = item.grade_id.id
                embalagem_id = item.produto_embalagem_id.id
                quantidade = item.quantidade_requisitada / quantidade_receitas

                chave = produto_id, grade_id, embalagem_id

                if chave not in insumos:
                    insumos[chave] = quantidade
                else:
                    insumos[chave] += quantidade

            # Calculo da diferenca (materia-prima/insumos)
            dados_insumo = []

            for insumo in brw_ficha_tecnica.pcpftc_ficha_tecnica_itens_ids:
                produto_id = insumo.pcpfti_produto_id.id
                grade_id = insumo.pcpfti_grade_id.id
                embalagem_id = insumo.pcpfti_produto_embalagem_id.id
                quantidade = insumo.pcpfti_quantidade
                chave = produto_id, grade_id, embalagem_id
                dados = {
                    'produto_id': produto_id,
                    'grade_id': grade_id,
                    'local_id': local_almoxarifado_pcp_id,
                    'produto_embalagem_id': embalagem_id
                }

                if chave in insumos:
                    if quantidade - insumos.get(chave, 0.0) > 0:  # Diferenca
                        quantidade = (quantidade - insumos.get(chave, 0.0)) * quantidade_receitas
                        dados['quantidade_requisitada'] = quantidade
                        dados_insumo.append(dados)
                else:  # Novo
                    quantidade *= quantidade_receitas
                    dados['quantidade_requisitada'] = quantidade
                    dados_insumo.append(dados)

            # Calculo da diferenca (embalagens)
            for embalagem in brw_ordem_producao.pcpopr_embalagem_ids:
                produto_id = embalagem.pcpope_produto_id.id
                grade_id = embalagem.pcpope_grade_id.id
                quantidade = embalagem.pcpope_quantidade / quantidade_receitas
                dados = {
                    'produto_id': produto_id,
                    'grade_id': grade_id,
                    'local_id': local_almoxarifado_pcp_id,
                }

                # Embalagem do local
                local_locacao = filter(lambda ll: ll.local_id.id == local_almoxarifado_pcp_id,
                                       embalagem.pcpope_grade_id.local_locacao_ids)

                if local_locacao and len(local_locacao) == 1:
                    produto_embalagem_id = local_locacao[0].estllc_produto_embalagem_id.id
                    dados['produto_embalagem_id'] = produto_embalagem_id
                    chave = produto_id, grade_id, produto_embalagem_id
                else:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Nao existe embalagem com o local equivalente ao local '
                                           'do almoxarifado.'))

                if chave in insumos:
                    if quantidade - insumos.get(chave, 0.0) > 0:  # Diferenca
                        quantidade = (quantidade - insumos.get(chave, 0.0)) * quantidade_receitas
                        dados['quantidade_requisitada'] = quantidade
                        dados_insumo.append(dados)
                else:  # Novo
                    quantidade *= quantidade_receitas
                    dados['quantidade_requisitada'] = quantidade
                    dados_insumo.append(dados)

            # Verifica se existe requisicao em estado de esboco
            args = [('almrqc_ordem_producao_id', '=', brw_ordem_producao.id),
                    ('state', '=', 'R')]
            requisicao_ids = obj_almoxarifado_requisicao_capa.search(cr, uid, args, context=context)
            requisicao_id = False

            if not requisicao_ids and dados_insumo:  # Cria nova requisicao
                dados_requisicao = {
                    'company_id': company_id, 'local_id': local_almoxarifado_pcp_id,
                    'data_emissao': datetime.today(), 'funcionario_id': funcionario_id,
                    'almrqc_ordem_producao_id': brw_ordem_producao.id,
                    'observacao': brw_ficha_tecnica.pcpftc_observacao,
                    'centrocusto_id': brw_ficha_tecnica.pcpftc_centro_custo_id.id or False
                }
                requisicao_id = obj_almoxarifado_requisicao_capa.create(
                    cr, uid, dados_requisicao, context)
            elif dados_insumo:  # Atualiza com os novos valores.
                requisicao_id = requisicao_ids[0]
                args = [('requisicao_id', '=', requisicao_id)]
                insumo_ids = obj_estoquealmoxarifado_requisicao_item.search(cr, uid, args, context=context)
                obj_estoquealmoxarifado_requisicao_item.unlink(cr, uid, insumo_ids)

            if requisicao_id and requisicao_id not in result:  # Nao duplica ;)
                result.append(requisicao_id)

            # Itens da requisicao (materia-prima/insumos & embalagens)
            for data in dados_insumo:
                data['requisicao_id'] = requisicao_id
                obj_estoquealmoxarifado_requisicao_item.create(cr, uid, data, context)

        return result

    def _pcpopr_verificar_requisicoes(self, cr, uid, ids, context=None):
        """
        Verifica se todas as requisicoes se encontram no estado de entregue ou cancelada.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs das requisicoes,
        @return: {boolean} verdadeiro se todas as requisicoes estiverem com o estado de entregue
        ou cancelado.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # ___________________________ Objects ________________________________
        context = context and context.copy() or {}
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        args = [('id', 'in', ids), ('state', 'not in', ['E', 'C'])]
        requisicao_ids = obj_almoxarifado_requisicao_capa.search(cr, uid, args, context=context)

        result = False if requisicao_ids else True

        return result

    def _pcpopr_verificar_itens(self, cr, uid, ficha_id, requisicao_ids, context=None):
        """
        Verifica se todos os itens (materia-prima/insumos e embalagens) estao contemplados nas
        requisicoes entregues.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ficha_id: {int} ID da ficha tecnica do produto,
        @param requisicao_ids: {list} lista dos IDs das requisicoes,
        @return: {boolean} verdadeiro se todos os itens estiverem contemplados nas requisicoes.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_estoquealmoxarifado_requisicao_item = self.pool.get('estoquealmoxarifado.requisicao.item')
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')

        result = True  # Assume que todos os itens estao contemplados

        # Consolida itens da ficha tecnica
        brw_ficha_tecnica = obj_pcp_ficha_tecnica_capa.browse(cr, uid, ficha_id)
        ficha_tecnica_itens = map(  # Materia-prima/insumos
                                    lambda fti: [fti.pcpfti_produto_id.id, fti.pcpfti_grade_id.id],
                                    brw_ficha_tecnica.pcpftc_ficha_tecnica_itens_ids)
        ficha_tecnica_embalagem = map(  # Embalagens
                                        lambda fte: [fte.pcpfte_produto_id.id, fte.pcpfte_grade_id.id],
                                        brw_ficha_tecnica.pcpftc_ficha_tecnica_embalagem_ids)

        # Consolida itens das requisicoes. Materia-prima, insumos e embalagens
        args = [('requisicao_id', 'in', requisicao_ids)]
        requisicao_item_ids = obj_estoquealmoxarifado_requisicao_item.search(cr, uid, args, context=context)
        brw_requisicao_item = filter(lambda i: i.produto_id.tipo_item != '02',
                                     obj_estoquealmoxarifado_requisicao_item.browse(
                                         cr, uid, requisicao_item_ids))
        brw_requisicao_embalagem = filter(lambda i: i.produto_id is not None,
                                          obj_estoquealmoxarifado_requisicao_item.browse(
                                              cr, uid, requisicao_item_ids))

        requisicao_itens = map(  # Materia-prima/insumos
                                 lambda rqi: [rqi.produto_id.id, rqi.grade_id.id], brw_requisicao_item)

        requisicao_embalagem = map(  # Embalagens
                                     lambda rqi: [rqi.produto_id.id, rqi.grade_id.id], brw_requisicao_embalagem)

        # Faz a verificacao da materia-prima
        for item in ficha_tecnica_itens:
            if item not in requisicao_itens:  # Item nao esta contido nas requisicoes
                result = False
                break

        for embalagem in ficha_tecnica_embalagem:
            if embalagem in requisicao_embalagem:  # Embalagem esta contida nas requisicoes
                result = True
                break
            else:
                result = False

        return result

    def pcpopr_button_fechado(self, cr, uid, ids, context=None):
        """
        Realiza o fechamento da ordem de producao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {boolean} verdadeiro se a ordem de producao foi fechada.

        @change: 06.03.2014 - pierre.barbosa
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')

        result = False
        campos = ['pcpopr_quantidade_produzida', 'pcpopr_requisicao_ids', 'pcpopr_embalagem_ids',
                  'pcpopr_ficha_tecnica_capa_id']
        dados_ordem_producao = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_ordem_producao and isinstance(dados_ordem_producao, list):
            dados_ordem_producao = dados_ordem_producao[0]

        if dados_ordem_producao and isinstance(dados_ordem_producao, dict):
            quantidade_produzida = dados_ordem_producao.get('pcpopr_quantidade_produzida', 0.0)
            requisicao_ids = dados_ordem_producao.get('pcpopr_requisicao_ids', [])
            embalagem_ids = dados_ordem_producao.get('pcpopr_embalagem_ids', [])
            ficha_tecnica_id = dados_ordem_producao.get('pcpopr_ficha_tecnica_capa_id', False)

            # Verifica se existe quantidade produzida
            if not quantidade_produzida or quantidade_produzida < 0:
                raise osv.except_osv(_('Atencao'), _('Informe a quantidade produzida'))

            # Verifica se as requisicoes estao todas entregues ou canceladas
            if not self._pcpopr_verificar_requisicoes(cr, uid, requisicao_ids, context=context):
                msg = _('As requisicoes devem estar no estado "Entregue" ou "Cancelada".')
                raise osv.except_osv(_('Atencao!'), msg)

            # Verifica se todos os itens de materia-prima/insumos e embalagens estao contemplados
            # nas requisicoes (entregues)
            args = [('id', 'in', requisicao_ids), ('state', '=', 'E')]  # Somente as entregues
            requisicao_ids = obj_almoxarifado_requisicao_capa.search(cr, uid, args, context=context)

            if not self._pcpopr_verificar_itens(cr, uid, ficha_tecnica_id, requisicao_ids, context=context):
                msg = _('As requisicoes devem contemplar todos os itens da ficha tecnica.')
                raise osv.except_osv(_('Atencao!'), msg)

            values = {'state': PCPOPR_STATE_FECHADO, 'pcpopr_termino': datetime.now()}
            campos = ['estpar_pcp_gerar_nota_fiscal', 'estpar_local_compra_padrao_id']
            estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
                cr, uid, 'estoque', campos, context=context)

            # Verifica o local de compra (entrada) padrao
            local_compra_padrao = estoque_parametro_dados.get('estpar_local_compra_padrao_id')

            if not local_compra_padrao:
                msg = _('Local para entrada nao informado no parametro do estoque.')
                raise osv.except_osv(_('Atencao!'), msg)

            # Caso o parametro de gerar nota fiscal nao esteja marcado, as ordens de producao,
            # quando fechadas, devem realizar a entrada de producao automatica do produto acabado.
            # Nao gera notas fiscais.
            pcp_gerar_nota = estoque_parametro_dados.get('estpar_pcp_gerar_nota_fiscal', False)

            if not pcp_gerar_nota:
                self.cria_movimento_atualiza_saldo(cr, uid, ids, context=context)

            result = self.write(cr, uid, ids, values, context=context)

            # Atualiza os custos caso o parametro estpar_atualizar_custo_fechamentoOP
            # esteja marcado
            self._pcpopr_atualizar_preco_custo(cr, uid, ids, context)

            # Alterar almrqc_flag_exibir_requisicao para false, para que as requisicoes nao sejam
            # vistas mais na devolucao de requisicoes
            for requisicao_id in requisicao_ids:
                vals_req_capa = {'almrqc_flag_exibir_requisicao': False}
                obj_almoxarifado_requisicao_capa.write(cr, uid, requisicao_id, vals_req_capa,
                                                       context)

        return result

    def _pcpopr_atualizar_preco_custo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_adm_utils = self.pool.get('adm.utils')
        obj_pcp_util = self.pool.get('pcp.util')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        parametro = ['admpar_siggestao_servidor', 'admpar_siggestao_diretorio',
                     'admpar_siggestao_pv_exportar']
        adm_parametro = obj_adm_utils.admutl_get_parametros(cr, uid, 'adm', parametro, context=context)
        parametro = ['estpar_atualizar_custo_fechamentoOP']
        estoque_parametro = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', parametro,
                                                                context=context)
        exportar = adm_parametro.get('admpar_siggestao_pv_exportar')
        servidor = adm_parametro.get('admpar_siggestao_servidor')
        diretorio = adm_parametro.get('admpar_siggestao_diretorio')
        atualizar_custo_ordem_producao = estoque_parametro.get(
            'estpar_atualizar_custo_fechamentoOP')
        if not atualizar_custo_ordem_producao:
            return False
        brw_pcp_ordem_producao = self.browse(cr, uid, ids, context=context)
        # Contexto para atualizacao somente do preco de custo
        context['PC'] = True
        for ordem in brw_pcp_ordem_producao:
            ficha_id = ordem.pcpopr_ficha_tecnica_capa_id
            item_ids = ficha_id.pcpftc_ficha_tecnica_itens_ids
            embalagem_ids = ordem.pcpopr_embalagem_ids
            quantidade_produzida = ordem.pcpopr_quantidade_produzida or 1.00
            custo_total = sum(map(lambda item: item.pcpfti_custo_total or 0.0, item_ids)) * \
                ordem.pcpopr_quantidade
            custo_total_embalagem = sum(map(lambda emb: emb.pcpope_custo_total or 0.0,
                                            embalagem_ids))
            custo_total += custo_total_embalagem
            custo_unitario = custo_total / quantidade_produzida
            produto_id = ficha_id.pcpftc_produto_id.id
            preco_venda = obj_estoque_preco_venda.get_preco_venda(cr, uid, produto_id)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            preco_venda = dados_preco_venda.get('preco_venda')
            # Exporta para SigGestao
            if exportar:
                obj_pcp_util.pcputl_atualizar_pvc_siggestao(cr, uid, produto_id, servidor,
                                                            diretorio, exportar, preco_venda,
                                                            custo_unitario, context)
            # Atualiza no SigERP
            else:
                obj_pcp_util.pcputl_atualizar_pvc_sigerp(cr, uid, produto_id, custo_unitario,
                                                         preco_venda, context)
        return True

    def cria_movimento_atualiza_saldo(self, cr, uid, ids, context=None):

        """
        O metodo realiza a entrada de producao. Usado para quando nao ha geracao de notas
        fiscais na ordem de producao
        """

        context = context and context.copy() or {}
        obj_estoque_movimento_capa = self.pool.get('estoque.movimento.capa')
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_adm_utils = self.pool.get('adm.utils')
        ordem_producao_dados = self.browse(cr, uid, ids)
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_local_compra_padrao_id'], context=context)
        local_id = estoque_parametro_dados.get('estpar_local_compra_padrao_id')
        pessoa_id = self.pool.get('res.users').browse(cr, uid, uid).pessoa_id.id
        company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id.id
        if not pessoa_id:
            raise osv.except_osv(_('Atencao'), _('Pessoa nao cadastrada para o usuario!'))
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        func_id = obj_rh_funcionario.search(cr, uid, [('pessoa', '=', pessoa_id)], context=context)
        if not func_id:
            raise osv.except_osv(_('Atencao'), _('Voce tem que se cadastrar como um Funcionario...'))
        else:
            func_id = func_id[0]
        for op in ordem_producao_dados:
            context['entrada_producao'] = True
            vals_mov_capa = {'company_id': company_id,
                             'funcionario_id': func_id,
                             'estmvc_ordem_producao_id': op.id,
                             'data': date.today()}
            movimento_capa_id = obj_estoque_movimento_capa.create(cr, uid, vals_mov_capa, context)
            vals = {
                'produto_id': op.pcpopr_ficha_tecnica_capa_id.pcpftc_produto_id.id,
                'grade_id': op.pcpopr_ficha_tecnica_capa_id.pcpftc_grade_id.id,
                'produto_embalagem_id': op.pcpopr_ficha_tecnica_capa_id.pcpftc_produto_peso_embalagem_id
                    .id,
                'quantidade': op.pcpopr_quantidade_produzida,
                'local_id': local_id,
                'capa_id': movimento_capa_id,
            }
            obj_estoque_movimento_itens.create(cr, uid, vals, context)
            obj_estoque_movimento_capa.estmvc_button_efetivado(cr, uid, movimento_capa_id, context)
        return True

    def pcpopr_button_cancelar(self, cr, uid, ids, context=None):

        context = context and context.copy() or {}
        if not isinstance(ids, list):
            ids = [ids]
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        requisicao_ids = obj_almoxarifado_requisicao_capa.search(
            cr, uid, [('almrqc_ordem_producao_id', 'in', ids), ('almrqc_devolucao', '=', False)],
            context=context)
        for req in obj_almoxarifado_requisicao_capa.read(cr, uid, requisicao_ids, ['state'],
                                                         context):
            if req.get('state') == 'C':
                continue
            if not obj_almoxarifado_requisicao_capa.search(
                    cr, uid, [('almrqc_devolucao', '=', True),
                              ('almrqc_requisicao_origem_id', '=', req.get('id')),
                              ('state', '=', 'E')], context=context):
                raise osv.except_osv(_('Atenção'), _('Para cancelar a ordem de produção, e '
                                                     'necessario devolver todas as requisicoes.'))

        res = self.write(cr, uid, ids, {'state': 'C'}, context)

        return res

    ## ___________________________ METODOS ORM _______________________________

    def copy(self, cr, uid, id, default=None, context=None):
        context = context and context.copy() or {}
        if not isinstance(default, dict):
            default = {}

        default['pcpopr_inicio'] = False
        default['pcpopr_termino'] = False
        default['pcpopr_nf_saida_id'] = False
        default['pcpopr_nf_entrada_id'] = False
        default['pcpopr_quantidade_produzida'] = 0

        res = super(pcp_ordem_producao, self).copy(cr, uid, id, default, context)

        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'pcpopr_quantidade' in vals:
            quantidade_receitas = vals.get('pcpopr_quantidade')
            if not quantidade_receitas or quantidade_receitas < 0:
                raise osv.except_osv(_('Atencao'), _('Informe a quantidade de receitas'))
        res = super(pcp_ordem_producao, self).write(cr, uid, ids, vals, context)
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        quantidade_receitas = vals.get('pcpopr_quantidade')
        if not quantidade_receitas or quantidade_receitas < 0:
            raise osv.except_osv(_('Atencao'), _('Informe a quantidade de receitas'))
        numero_ordem_producao = self.pool.get('adm.utils').get_sequence(cr, uid, self._name, PCPOPR_CODE_SEQUENCE)

        vals['pcpopr_numero'] = numero_ordem_producao

        ordem_producao_id = super(pcp_ordem_producao, self).create(cr, uid, vals, context)
        return ordem_producao_id

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        res = super(pcp_ordem_producao, self).fields_view_get(cr, uid, view_id, view_type, context,
                                                              toolbar, submenu)
        obj_adm_utils = self.pool.get('adm.utils')
        estoque_parametro_dados = obj_adm_utils. \
            admutl_get_parametros(cr, uid, 'estoque', ['estpar_pcp_gerar_nota_fiscal'], context=context)
        pcp_gerar_nota = estoque_parametro_dados.get('estpar_pcp_gerar_nota_fiscal')
        if not pcp_gerar_nota and res.get('type') == 'form':
            doc = etree.XML(res['arch'])
            doc.xpath("//field[@name='state']")[0].set('statusbar_visible', "E,A,F,C")
            res['arch'] = etree.tostring(doc)
        return res
