# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from decimal import Decimal

class pcp_ordem_producao_embalagem(osv.Model):
    """
    Embalagens utilizadas pela ordem de producao.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'pcp.ordem.producao.embalagem'
    _inherit = 'pcp.ordem.producao.embalagem'

    def pcpope_onchange_grade_id(self, cr, uid, ids, produto_id, grade_id, quantidade, context=None):
        """
        Calcula a quantidade de embalagem. A quantidade sera a divisao entre a quantidade produzida
        e a capacidade na embalagem.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} ID do produto (tipo_item = '02 - Embalagem'),
        @param grade_id: {int} ID da grade do produto,
        @param quantidade: {float} quantidade de receitas.
        @return: {dict} o valor sugerido para quantidade de embalagens.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_ficha_tecnica_embalagem = self.pool.get('pcp.ficha.tecnica.embalagem')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')

        result = {'value': {'pcpope_quantidade': 0.0}}

        # Recupera a quantidade da embalagem selecionada para o calculo da quantidade de embalagem
        # necessaria (sugerida)
        args = [('pcpfte_produto_id', '=', produto_id), ('pcpfte_grade_id', '=', grade_id)]
        embalagem_ids = obj_ficha_tecnica_embalagem.search(cr, uid, args, context=context)
        campos = ['pcpfte_quantidade']
        dados_embalagem = obj_ficha_tecnica_embalagem.read(cr, uid, embalagem_ids, campos, context=context)

        if dados_embalagem and isinstance(dados_embalagem, list):
            dados_embalagem = dados_embalagem[0]

        if dados_embalagem and isinstance(dados_embalagem, dict):
            quantidade_embalagem = dados_embalagem.get('pcpfte_quantidade', 0.0)

            #Verifica se a unidade eh fracionada ou nao
            dados_estoque_produto_grade = obj_estoque_produto_grade.read(cr, uid, grade_id,
                                                                         ['local_locacao_ids'], context=context
                                                                         )
            # dados_estoque_produto_grade esta recebendo uma lista de dicionarios apenas com um
            # index. E em outro momento recebe um dicionario. Para nao ter problema com acesso aos
            # dados retornados, caso o seja uma lista o retorno o mesmo e tratado para dicionario.
            if isinstance(dados_estoque_produto_grade, list):
                dados_estoque_produto_grade = dados_estoque_produto_grade[0]

            local_locacao_ids = dados_estoque_produto_grade['local_locacao_ids']
            embalagem_unidade_fracionada = False
            if local_locacao_ids:
                brw_estoque_local_locacao = obj_estoque_local_locacao.browse(cr, uid,
                                                                             local_locacao_ids[0], context=context)
                embalagem_unidade_fracionada = brw_estoque_local_locacao. \
                    estllc_produto_embalagem_id.embalagem_id.unidade_id.fracao

            if quantidade_embalagem:  # Evita divisao por zero! ;)

                quantidade_sugerida = quantidade * quantidade_embalagem
                if not embalagem_unidade_fracionada:
                    quantidade_sugerida = Decimal(str(quantidade_sugerida)).quantize(exp=3, rounding='ROUND_UP')
                result['value']['pcpope_quantidade'] = quantidade_sugerida

        return result

    def pcpope_onchange_produto_id(self, cr, uid, ids, produto_id, context=None):
        """
        Atribui a mesma grade do produto da ficha tecnica.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} ID do produto (tipo_item = '02 - Embalagem'),
        @return: {dict} a grade (unica) do produto.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')

        result = {'value': {'pcpope_grade_id': False}}
        args = [('produto_id', '=', produto_id)]
        grade_ids = obj_estoque_produto_grade.search(cr, uid, args, context=context)

        # Se for grade unica, mostra a grade.
        if isinstance(grade_ids, list) and len(grade_ids) == 1:
            result['value']['pcpope_grade_id'] = grade_ids[0]

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
