# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import socket
from openerp.tools.translate import _
from unicodedata import normalize
import time
from SigUtils.formatastring import formataString
from datetime import datetime
READ_LOAD = '_classic_write'
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from decimal import Decimal


class pcp_util(osv.Model):

    _name = 'pcp.util'
    _auto = False

    def pcputl_realiza_processo_producao(self, cr, uid, context=None):
        """
        Metodo chamado pelo agendador (cron). Identifica os produtos (ficha tecnica) vendidos no
        dia e gera a producao desses, passando pelo processo de forma completa.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_pcp_ordem_producao = self.pool.get('pcp.ordem.producao')
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
        obj_almoxarifado_requisicao_capa = self.pool.get('almoxarifado.requisicao.capa')
        obj_wizard_almoxarifado_entrega = self.pool.get('wizard.almoxarifado.entrega')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_pcp_ficha_tecnica_embalagem = self.pool.get('pcp.ficha.tecnica.embalagem')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        campos_estoque = ['estpar_pcp_gerar_nota_fiscal', 'estpar_local_almoxarifado_pcp_id']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', campos_estoque, context=context)
        campos_financeiro = ['usar_centro_custos_e_eventos']
        financeiro_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'financeiro', campos_financeiro, context=context)
        local_id = estoque_parametro_dados.get('estpar_local_almoxarifado_pcp_id')
        if not local_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre o almoxarifado do pcp nos parametros do '
                                                 'estoque.'))
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'pcp.ordem.producao',
                                                                    context=context)
        data_hoje = time.strftime('%Y-%m-%d')
        funcionario_id = self.pool.get('rh.funcionario').get_id_func(cr, uid, context)

        # Como esse metodo sera executado pelo cron, e preciso forcar a empresa encontrada para
        # garantir que o processo seja totalmente referente a ela.
        context.update({'sig_force_company': True})

        # Encontra ordens de producao que possivelmente ja foram criadas pelo cron no periodo.
        # Isso e feito para evitar duplicacao caso o agendador seja alterado manualmente,
        # por exemplo.
        ordem_producao_ids = obj_pcp_ordem_producao.search(
            cr, uid, [('pcpopr_inicio', '>=', str(parse(data_hoje) - relativedelta(days=5))),
                      ('pcpopr_termino', '<=', data_hoje + ' 23:59:59'),
                      ('state', 'in', ('F', 'N')), ('pcpopr_gerado_automatico', '=', True),
                      ('company_id', '=', company_id),
                      ('pcpopr_responsavel_id', '=', funcionario_id)],
            context=context)
        ficha_tecnica_ids = map(lambda f: f.get('pcpopr_ficha_tecnica_capa_id'),
                                obj_pcp_ordem_producao.read(
                                    cr, uid, ordem_producao_ids, ['pcpopr_ficha_tecnica_capa_id'],
                                    context, '_classic_write'))

        # Encontra ordens de producao que possivelmente ja foram criadas pelo cron no dia.
        # Isso e feito para evitar duplicacao caso o agendador seja alterado manualmente,
        # por exemplo.
        hoje_ordem_producao_ids = obj_pcp_ordem_producao.search(
            cr, uid, [('pcpopr_inicio', '>=', data_hoje + ' 00:00:00'),
                      ('pcpopr_termino', '<=', data_hoje + ' 23:59:59'),
                      ('state', 'in', ('F', 'N')), ('pcpopr_gerado_automatico', '=', True),
                      ('company_id', '=', company_id),
                      ('pcpopr_responsavel_id', '=', funcionario_id)],
            context=context)
        hoje_ficha_tecnica_ids = map(lambda f: f.get('pcpopr_ficha_tecnica_capa_id'),
                                     obj_pcp_ordem_producao.read(
                                         cr, uid, hoje_ordem_producao_ids,
                                         ['pcpopr_ficha_tecnica_capa_id'], context,
                                         '_classic_write'))
        # Encontra o movimento de venda - no periodo entre a data atual mais 5 dias atras - para os
        # produtos que sao ficha tecnica.
        sql = """
            SELECT
                ep.name                     AS descricao,
                sum(emi.quantidade)         AS quantidade_vendida,
                pft.id                      AS ficha_tecnica_id
            FROM estoque_movimento_itens emi
            INNER JOIN estoque_movimento_capa emc
                ON emc.id = emi.capa_id
            INNER JOIN estoque_natureza_operacao eno
                ON eno.id = emc.estmvc_operacao_id
            INNER JOIN pcp_ficha_tecnica_capa pft
                ON pft.pcpftc_grade_id = emi.grade_id
            LEFT JOIN estoque_produto ep
                ON ep.id = emi.produto_id
            WHERE
                emc.data BETWEEN CURRENT_DATE - 5 AND CURRENT_DATE
                AND emc.company_id = {company_id}
                AND emc.state = 'F'
                AND eno.estnto_tipo_operacao = 'S'
                AND eno.estnto_atualizar_saldo IS TRUE
                AND eno.estnto_gerar_financeiro IS TRUE
                AND emi.cfop_id IN (SELECT id 
                                    FROM fiscal_cfop WHERE tipo = 'S')
            GROUP BY
                ficha_tecnica_id, descricao
        """.format(company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()

        sql = sql.replace('emc.data BETWEEN CURRENT_DATE - 5 AND CURRENT_DATE',
                          'emc.data = CURRENT_DATE')
        cr.execute(sql)
        dados_hoje = cr.dictfetchall()

        msg = ''
        for dado in dados:
            ficha_tecnica_id = dado.get('ficha_tecnica_id')
            if ficha_tecnica_id in hoje_ficha_tecnica_ids:
                continue
            if ficha_tecnica_id in ficha_tecnica_ids:
                ficha_venda_hoje = filter(lambda x: x.get('ficha_tecnica_id') == ficha_tecnica_id,
                                          dados_hoje)
                if not ficha_venda_hoje:
                    continue
                dado = ficha_venda_hoje[0]
            ficha_tecnica_dados = obj_pcp_ficha_tecnica_capa.read(
                cr, uid, ficha_tecnica_id, ['pcpftc_quantidade_peso', 'pcpftc_centro_custo_id'],
                context, '_classic_write')
            if financeiro_parametro_dados.get('usar_centro_custos_e_eventos') and not \
                    ficha_tecnica_dados.get('pcpftc_centro_custo_id'):
                msg += dado.get('descricao') + '\n'
                continue
            vals_ordem_producao = {
                'company_id': company_id,
                'pcpopr_responsavel_id': funcionario_id,
                'pcpopr_ficha_tecnica_capa_id': ficha_tecnica_id,
                'pcpopr_quantidade': dado.get('quantidade_vendida') / ficha_tecnica_dados.get(
                    'pcpftc_quantidade_peso'),
                'pcpopr_quantidade_produzida': dado.get('quantidade_vendida'),
                'name': dado.get('descricao'),
                'pcpopr_gerado_automatico': True
            }
            ordem_producao_id = obj_pcp_ordem_producao.create(cr, uid, vals_ordem_producao, context)
            pcp_embalagem_ids = obj_pcp_ficha_tecnica_embalagem.search(
                cr, uid, [('pcpfte_capa_id', '=', ficha_tecnica_id)], context=context)
            pcp_embalagem_vals = []
            if pcp_embalagem_ids:
                for embalagem in obj_pcp_ficha_tecnica_embalagem.browse(cr, uid, pcp_embalagem_ids,
                                                                        context):
                    local_locacao_ids = obj_estoque_local_locacao.search(
                        cr, uid, [('grade_id', '=', embalagem.pcpfte_grade_id.id),
                                  ('local_id', '=', local_id)], context=context)
                    embalagem_unidade_fracionada = False
                    if local_locacao_ids:
                        brw_estoque_local_locacao = obj_estoque_local_locacao.browse(
                            cr, uid, local_locacao_ids[0], context=context)
                        embalagem_unidade_fracionada = brw_estoque_local_locacao. \
                            estllc_produto_embalagem_id.embalagem_id.unidade_id.fracao
                    quantidade = vals_ordem_producao.get(
                        'pcpopr_quantidade') * embalagem.pcpfte_quantidade
                    if not embalagem_unidade_fracionada:
                        quantidade = Decimal(str(quantidade)).quantize(exp=3, rounding='ROUND_UP')
                    vals = {'pcpope_produto_id': embalagem.pcpfte_produto_id.id,
                            'pcpope_grade_id': embalagem.pcpfte_grade_id.id,
                            'pcpope_quantidade': quantidade}
                    pcp_embalagem_vals.append([0, 0, vals])
            if pcp_embalagem_vals:
                obj_pcp_ordem_producao.write(cr, uid, ordem_producao_id,
                                             {'pcpopr_embalagem_ids': pcp_embalagem_vals}, context)
            obj_pcp_ordem_producao.pcpopr_button_abrir(cr, uid, [ordem_producao_id], context)
            requisicao_ids = obj_almoxarifado_requisicao_capa.search(
                cr, uid, [('almrqc_ordem_producao_id', '=', ordem_producao_id)], context=context)
            if not requisicao_ids:
                requisicao_ids = obj_pcp_ordem_producao.pcpopr_button_gerar_requisicoes(
                    cr, uid, [ordem_producao_id], context)
            for requisicao_id in requisicao_ids:
                obj_almoxarifado_requisicao_capa.button_requerer(cr, uid, [requisicao_id], context)
                obj_almoxarifado_requisicao_capa.button_aprovar(cr, uid, [requisicao_id], context)
                obj_almoxarifado_requisicao_capa.button_separar(cr, uid, [requisicao_id], context)
                wizard_almoxarifado_entrega_id = obj_wizard_almoxarifado_entrega.create(
                    cr, uid, {'recebeu_id': funcionario_id,
                              'data': time.strftime('%Y-%m-%d %H:%M:%S')}, context)
                context.update({'active_id': requisicao_id})
                obj_wizard_almoxarifado_entrega.button_wizard_entregar(
                    cr, uid, [wizard_almoxarifado_entrega_id], context)
            obj_pcp_ordem_producao.pcpopr_button_fechado(cr, uid, [ordem_producao_id], context)
            if estoque_parametro_dados.get('estpar_pcp_gerar_nota_fiscal'):
                obj_pcp_ordem_producao.pcpopr_button_gerar_nf(cr, uid, [ordem_producao_id], context)
        del context['sig_force_company']
        if msg:
            raise osv.except_osv(_('Atencao'), _('As fichas tecnicas a seguir nao possuem centro '
                                                 'de custo cadastrado: \n' + msg))
        return True

    def pcputl_verificar_materia_prima(self, cr, uid, data_inicial, data_final, context=None):
        """
        O metodo busca todos os itens e suas quantidades planejada, existente e faltante das
        fichas tecnicas dos planejamentos de producao, de acordo com o periodo informado.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param data_inicial: data inicial do periodo de busca
        @param data_final: data final do periodo de busca
        @param context: dicionario com valores contextuais
        @return: Lista de dicionarios com os dados de produto, grade, produto embalagem,
        quantidade planejada, quantidade existente e quantidade faltante
        """
        context = context and context.copy() or {}
        if not data_inicial or not data_final:
            return []
        estoque_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_local_almoxarifado_pcp_id'])
        local_id = estoque_parametro.get('estpar_local_almoxarifado_pcp_id')
        if not local_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre o almoxarifado do pcp nos parametros do '
                                                 'estoque'))
        sql = """
            SELECT fti.pcpfti_produto_id                              AS produto_id,
                   fti.pcpfti_grade_id                                AS grade_id,
                   fti.pcpfti_produto_embalagem_id                    AS produto_embalagem_id,
                   Sum(fti.pcpfti_quantidade * plp.pljprd_quantidade) AS quantidade_planejada
            FROM   pcp_planejamento_producao plp
                   INNER JOIN pcp_ficha_tecnica_capa ftc
                           ON plp.pljprd_ficha_tecnica_id = ftc.id
                   INNER JOIN pcp_ficha_tecnica_itens fti
                           ON fti.pcpfti_capa_id = ftc.id
            WHERE  plp.pljprd_data >= '%s'
                   AND plp.pljprd_data <= '%s'
            GROUP  BY fti.pcpfti_produto_id, fti.pcpfti_grade_id, fti.pcpfti_produto_embalagem_id
            UNION
            SELECT fte.pcpfte_produto_id AS produto_id,
                   fte.pcpfte_grade_id   AS grade_id,
                   null                  AS produto_embalagem_id,
                   Sum(plp.pljprd_quantidade * fte.pcpfte_quantidade) AS quantidade_planejada
            FROM   pcp_planejamento_producao plp
                   INNER JOIN pcp_ficha_tecnica_capa ftc
                           ON plp.pljprd_ficha_tecnica_id = ftc.id
                   INNER JOIN pcp_ficha_tecnica_embalagem fte
                           ON fte.pcpfte_capa_id = ftc.id
            WHERE  plp.pljprd_data >= '%s'
                   AND plp.pljprd_data <= '%s'
            GROUP  BY fte.pcpfte_produto_id, fte.pcpfte_grade_id
        """ % (data_inicial, data_final, data_inicial, data_final)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            return dados
        grade_ids = map(lambda i: i.get('grade_id'), dados)
        saldo_produtos = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
            cr, uid, grade_ids, time.strftime('%Y-%m-%d'), local_id, context=context)
        for dado in dados:
            saldo = saldo_produtos.get(dado.get('grade_id')).get('saldo')
            quantidade_planejada = dado.get('quantidade_planejada') or 0.0
            dado['quantidade_existente'] = saldo
            faltante = saldo - quantidade_planejada
            if saldo < 0 or faltante < 0:
                dado['quantidade_faltante'] = faltante * -1
            else:
                dado['quantidade_faltante'] = 0.0
        return dados

    def pcputl_atualizar_pvc_siggestao(self, cr, uid, produto_id, servidor, diretorio, exportar,
                                       preco_venda, custo_total, context=None):

        """
        Exporta para o SigGestao os valores de preco de venda e/ou preco de custo
        @produto_id: id da tabela estoque_produto
        @servidor: ip ou dominio do servidor siggestao
        @diretorio: diretorio onde esta instalado o siggestao
        @exportar: comando a ser executado no siggestao para exportacao dos precos
        @preco_venda: valor do preco de venda
        @custo_total: valor do custo
        @return: True
        """

        context = context and context.copy() or {}
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        address = ((servidor, 5019))
        try:
            client_socket.connect(address)
        except Exception as e:
            mensagem = str(e)
            raise osv.except_osv(_('Atencao'), _('Erro na conexao com o servidor:') + str(
                servidor) + '\nMensagem:' + normalize('NFKD', mensagem.decode(
                    'utf-8')).encode('ASCII', 'ignore'))
        client_socket.send('ESTA AI?')
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), mensagem)
        client_socket.send('DIR:' + diretorio)
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), _("Diretorio '%s' nao encontrado" % diretorio))
        c = exportar
        _arquivoCMD = c[c.find('{') + 1:c.find('}')].strip()
        if not _arquivoCMD:
            a = str(time.time())
            n = a.find('.')
            _arquivoCMD = 'pv' + a[n - 5:n] + a[n + 1:n + 2] + '.cmd'
        admpar_siggestao_pv_exportar = c[:c.find('{')] + _arquivoCMD + c[c.find('}') + 1:]
        client_socket.send('ARQ NOME:' + diretorio + '/' + _arquivoCMD)
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), mensagem)
        arquivo = []
        if produto_id:
            codigo = self.pool.get('estoque.produto').browse(
                cr, uid, produto_id, context).codigo or 0
            novo_preco = preco_venda or 0
            # contexto para atualizar somente o preco de custo
            if novo_preco > 0 and context.get('PV'):
                linha = 'PV'
                linha += formataString(codigo, 10, '0', 'E')
                linha += '     '
                valor = "%.2f" % novo_preco
                linha += formataString(str(valor).replace('.', '').replace(',', ''), 12, '0', 'E')
                linha += str(uid).rjust(10, '0')
                arquivo.append(linha)

            indexador = {}
            parametro_indexador = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'estoque', 'indexador_id')
            indexador_id = parametro_indexador.get('indexador_id')
            if indexador_id:
                sigla = self.pool.get('adm.moeda').browse(cr, uid, indexador_id).sigla
                dia_atual = time.strftime('%d')
                mes_atual = time.strftime('%m')
                ano_atual = time.strftime('%Y')
                indexador = self.pool.get('adm.moeda.valor').achar_valor_moeda(
                    cr, uid, sigla, ano_atual, mes_atual, dia_atual, '<=', context=context)
            valor_indexador = indexador.get('valor') or 0
            valor_indexador = float(valor_indexador) or 1.0
            if context.get('PC'):
                tuple_custo = ('U', 'C', 'R', 'M')
            else:
                tuple_custo = ()
            for tipo_custo in tuple_custo:

                if tipo_custo == 'C':
                    custo = custo_total / valor_indexador
                else:
                    custo = custo_total

                linha_custo = 'PC'
                linha_custo += formataString(codigo, 10, '0', 'E')
                linha_custo += '     '
                linha_custo += formataString(str(custo).replace('.', '').replace(',', ''), 12, '0',
                                             'E')
                linha_custo += str(uid).rjust(10, '0')
                linha_custo += tipo_custo

                arquivo.append(linha_custo)

        for linha in arquivo:
            comando = 'ARQ DADOS:' + linha
            client_socket.send(comando)
            mensagem = client_socket.recv(2048)
            if not mensagem.startswith('OK'):
                raise osv.except_osv(_('Atencao'), mensagem)
        client_socket.send('ARQ FECHAR:')
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            raise osv.except_osv(_('Atencao'), mensagem)
        comando = 'EXECUTAR:' + admpar_siggestao_pv_exportar
        client_socket.send(comando)
        mensagem = client_socket.recv(2048)
        if not mensagem.startswith('OK'):
            if mensagem.startswith('ERRO!'):
                mensagem = _('Sem permissao para execucao')
            raise osv.except_osv(_('Atencao'), mensagem)
        return True

    def pcputl_atualizar_pvc_sigerp(self, cr, uid, produto_id, valor_custo, preco_venda,
                                    context=None):

        """
        Atualiza o preco de venda e de custo para os produtos no sigerp
        @produto_id: id da tabela estoque_produto
        @valor_custo: valor do custo do produto
        @preco_venda: valor do preco de venda do produto
        @return: True
        """

        context = context and context.copy() or {}
        obj_adm_empresa_objeto = self.pool.get('adm.empresa.objeto')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')

        # company_id definida para o modelo estoque_produto
        company_id = obj_adm_empresa_objeto.getEmpresa(cr, uid, 'estoque.produto', context=context)
        # contexto para atualizar o preco de custo
        if context.get('PC'):
            preco_custo_values = {'estprc_data': str(datetime.now()),
                                  'estprc_custo_corrigido': valor_custo,
                                  'estprc_custo_medio_contabil': valor_custo,
                                  'estprc_ultimo_custo': valor_custo,
                                  'estprc_custo_reposicao': valor_custo,
                                  'produto_id': produto_id, 'company_id': company_id,
                                  'active': True}

            obj_estoque_preco_custo.create(cr, uid, preco_custo_values, context)
        # contexto para atualizar o preco de venda
        if context.get('PV'):
            preco_venda_values = {'company_id': company_id, 'produto_id': produto_id,
                                  'preco_venda': preco_venda}

            obj_estoque_preco_venda.create(cr, uid, preco_venda_values, context)
        return True

    def pcputl_gerar_venda_diaria(self, cr, uid, data_inicial=None, data_final=None,
                                  produto_id=None, local_id=None, context=None):
        """
        O metodo calcula a venda diaria (entradas - perdas).
        O consumo de cada local e calculado pelo resultado entre a quantidade de entrada de
        transferencia (requisicao de ressuprimento) menos a perda de procucao lancada.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param data_inicial: data inicial do periodo de movimentacao do produto
        @param data_final: data final do periodo de movimentacao do produto
        @param produto_id: id do produto movimentado
        @param context: dicionario com valores contextuais
        @return: lista de dicionarios contendo o id do produto e produto embalagem, alem da
        quantidade da venda. Senao, lista vazia
        """
        context = context and context.copy() or {}
        sql = """
            SELECT emi.produto_id,
                   emi.produto_embalagem_id,
                   ( Sum(CASE
                           WHEN emo.estmog_codigo in ( 'S', 'N' ) THEN emi.quantidade
                           ELSE 0
                         END) - Sum(CASE
                                      WHEN emo.estmog_codigo = 'E'
                                           AND emc.state = 'F' THEN emi.quantidade
                                      ELSE 0
                                    END) ) AS quantidade
            FROM   estoque_movimento_itens emi
                   INNER JOIN estoque_movimento_capa emc
                           ON emi.capa_id = emc.id
                   INNER JOIN estoque_natureza_operacao eno
                           ON emc.estmvc_operacao_id = eno.id
                   LEFT JOIN estoque_movimento_origem emo
                           ON emo.id = eno.estnto_movimento_origem_id
            WHERE  emo.estmog_codigo IN ( 'S', 'E', 'N' )
                   AND eno.estnto_atualizar_saldo = true
        """
        if produto_id:
            sql += """
                AND emi.produto_id = %s
            """ % produto_id
        if local_id:
            sql += """
                AND emi.local_id = %s
            """ % local_id
        if data_inicial:
            sql += """
                AND emc.data >= '%s'
            """ % data_inicial
        if data_final:
            sql += """
                AND emc.data <= '%s'
            """ % data_final
        sql += """
            GROUP  BY emi.produto_id,
                      emi.produto_embalagem_id
        """
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def pcputl_get_custo_unitario_total(self, cr, uid, tipo, item_embalagem_ids, context=None):
        """
        O método calcula o custo unitário total para os insumos ou para as embalagens da ficha
        técnica, de acordo com o 'tipo' definido.
        @param cr: cursor no banco de dados
        @param uid: id da tabala res_users, identificador do usuario
        @param tipo: char que identifica se o calculo será para o custo dos insumos ou das
        embalagens. Portanto, 'I' para insumos e 'E' para embalagens.
        @param item_embalagem_ids: lista de ids da tabela pcp_ficha_tecnica_itens ou da tabela
        pcp_ficha_tecnica_embalagem_data
        @return: Dicionário. As chaves representam os ids dos insumos/embalagens. O valor das chaves
        representam o custo unitário total
        """
        context = context and context.copy() or {}
        res = {}.fromkeys(item_embalagem_ids, 0)
        if not tipo or not item_embalagem_ids:
            return res
        # CUSTO PARA PRECO DE VENDA NO PARAMETRO
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, tabela='estoque', campos='estpar_custo_para_precovenda', context=context
        )
        custo_para_preco_venda = estoque_parametro_dados.get('estpar_custo_para_precovenda')

        # FUNCAO PARA BUSCAR O PRECO DE CUSTO DO DIA
        funct_get_custos_do_dia = self.pool.get('estoque.preco.custo').get_custos_do_dia

        # TIPO INSUMOS
        if tipo == 'I':
            obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
            obj_pcp_ficha_tecnica_itens = self.pool.get('pcp.ficha.tecnica.itens')
            ficha_tecnica_item_dados = obj_pcp_ficha_tecnica_itens.read(
                cr, uid, item_embalagem_ids, ['pcpfti_produto_id', 'pcpfti_quantidade'], context=context,
                load=READ_LOAD)

            produto_ids = filter(None, map(lambda item: item.get('pcpfti_produto_id'),
                                           ficha_tecnica_item_dados))

            # VERIFICA SE EXISTE FICHA TECNICA PARA OS INSUMOS (PRODUTOS)
            ficha_tecnica_ids = obj_pcp_ficha_tecnica_capa.search(
                cr, uid, [('pcpftc_produto_id', 'in', produto_ids)], context=context)
            ficha_tecnica_dados = obj_pcp_ficha_tecnica_capa.read(
                cr, uid, ficha_tecnica_ids,
                ['pcpftc_produto_id', 'pcpftc_custo_total', 'pcpftc_quantidade_peso'], context=context,
                load=READ_LOAD)
            dict_produto_de_ficha = {}
            for x in ficha_tecnica_dados:
                produto_id = x.get('pcpftc_produto_id')
                custo_total_produto_ficha = x.get('pcpftc_custo_total') or 0
                quantidade_peso = x.get('pcpftc_quantidade_peso')
                dict_produto_de_ficha.update(
                    {produto_id: custo_total_produto_ficha / quantidade_peso})

            dict_produto_custos = funct_get_custos_do_dia(cr, uid, produto_ids)

            for i in ficha_tecnica_item_dados:
                produto_id = i.get('pcpfti_produto_id')
                quantidade = i.get('pcpfti_quantidade') or 0
                if produto_id in dict_produto_de_ficha:
                    custo = dict_produto_de_ficha.get(produto_id)
                else:

                    dict_custo = dict_produto_custos.get(produto_id) or {}

                    if custo_para_preco_venda == 'U':
                        custo = dict_custo.get('estprc_ultimo_custo') or 0
                    elif custo_para_preco_venda == 'M':
                        custo = dict_custo.get('estprc_custo_medio_contabil') or 0
                    elif custo_para_preco_venda == 'C':
                        custo = dict_custo.get('estprc_custo_corrigido') or 0
                    elif custo_para_preco_venda == 'R':
                        custo = dict_custo.get('estprc_custo_reposicao') or 0
                    else:
                        custo = 0
                custo_total = custo * quantidade
                res[i.get('id')] = custo_total

        # TIPO EMBALAGENS
        elif tipo == 'E':
            fields_to_read = ['pcpfte_produto_id', 'pcpfte_quantidade', 'pcpfte_considerar_padrao']
            ficha_tecnica_embalagem_dados = self.pool.get('pcp.ficha.tecnica.embalagem').read(
                cr, uid, item_embalagem_ids, fields_to_read, context=context, load=READ_LOAD)
            if context.get('pcpftc_total_embalagem_padrao'):
                ficha_tecnica_embalagem_dados = filter(lambda i: i.get('pcpfte_considerar_padrao'),
                                                       ficha_tecnica_embalagem_dados)
            produto_ids = map(lambda item: item.get('pcpfte_produto_id'),
                              ficha_tecnica_embalagem_dados)

            dict_produto_custos = funct_get_custos_do_dia(cr, uid, produto_ids)

            for i in ficha_tecnica_embalagem_dados:

                produto_id = i.get('pcpfte_produto_id')
                quantidade = i.get('pcpfte_quantidade') or 0
                dict_custo = dict_produto_custos.get(produto_id) or {}

                if custo_para_preco_venda == 'U':
                    custo = dict_custo.get('estprc_ultimo_custo') or 0
                elif custo_para_preco_venda == 'M':
                    custo = dict_custo.get('estprc_custo_medio_contabil') or 0
                elif custo_para_preco_venda == 'C':
                    custo = dict_custo.get('estprc_custo_corrigido') or 0
                elif custo_para_preco_venda == 'R':
                    custo = dict_custo.get('estprc_custo_reposicao') or 0
                else:
                    custo = 0
                custo_total = custo * quantidade
                res[i.get('id')] = custo_total
        return res

