# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class estoque_produto(osv.Model):
    """
    Filtra as embalagens da ficha tecnica do produto.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.produto'
    _inherit = 'estoque.produto'

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        """
        Dominio para as embalagens utilizadas na ficha tecnica do produto.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param name: {str} nome do objeto (modelo) utilizado na pesquisa,
        @param args: {list} lista de argumentos para a pesquisa,
        @param operator: {str} operador utilizado na consulta dos registros,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @param limit: {int} numero de registros para ser retornado,
        @return: {list} lista com as embalagens utilizadas na ficha tecnica do produto.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        result = super(estoque_produto, self).name_search(
            cr, uid, name=name, args=args, operator=operator, context=context, limit=0)

        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')

        # Filtra as embalagens da ficha tecnica
        ficha_tecnica_id = context.get('ficha_tecnica', False)

        if ficha_tecnica_id:
            brw_ficha_tecnica = obj_pcp_ficha_tecnica_capa.browse(cr, uid, ficha_tecnica_id)
            embalagem_ids = map(lambda embalagem: embalagem.pcpfte_produto_id.id,
                                brw_ficha_tecnica.pcpftc_ficha_tecnica_embalagem_ids)
            result = filter(lambda embalagem: embalagem[0] in embalagem_ids, result)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
