#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
import time


class estoqueressuprimento_requisicao_capa(osv.Model):
    _name = 'estoqueressuprimento.requisicao.capa'
    _inherit = 'estoqueressuprimento.requisicao.capa'

    def button_entregue(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = super(estoqueressuprimento_requisicao_capa, self).button_entregue(cr, uid, ids, context)
        self.atualizar_custos(cr, uid, ids, context)
        return res

    def atualizar_custos(self, cr, uid, ids, context=None):
        """
        O metodo realiza a atualizacao dos custos do produto destino, incluidos na requisicao a
        partir da transferencia para producao.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param context: dicionario com valores contextuais.
        :return: booleano True.
        """
        context = context and context.copy() or {}

        obj_preco_custo = self.pool.get('estoque.preco.custo')
        obj_adm_util = self.pool.get('adm.utils')
        obj_estoque_produto_saldo = self.pool.get('estoque.produto.saldo')

        estoque_parametro_dados = obj_adm_util.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_custo_entrada_producao', context=context)
        custo_entrada_producao = estoque_parametro_dados.get('estpar_custo_entrada_producao')
        if not custo_entrada_producao:
            raise osv.except_osv(_('Atencao'), _('O custo para entrada de producao nao esta '
                                                 'cadastrado nos parametros do estoque'))
        if custo_entrada_producao == 'U':
            custo_entrada_producao = 'estprc_ultimo_custo'
        elif custo_entrada_producao == 'M':
            custo_entrada_producao = 'estprc_custo_medio_contabil'
        elif custo_entrada_producao == 'R':
            custo_entrada_producao = 'estprc_custo_reposicao'
        else:
            custo_entrada_producao = 'estprc_custo_corrigido'

        for requisicao in self.browse(cr, uid, ids, context=context):
            for item in requisicao.requisicao_itens_ids:
                if not (item.etrrqi_destino_produto_id and item.produto_id):
                    continue
                destino_item_quantidade = item.etrrqi_destino_quantidade
                emb_qtd = item.embalagem_id.embalagem_id.quantidade_na_embalagem or 1
                qtd_total_item = emb_qtd * destino_item_quantidade
                company_id = requisicao.company_id.id
                destino_produto_id = item.etrrqi_destino_produto_id.id
                destino_grade_id = item.etrrqi_destino_grade_id.id
                origem_produto_id = item.produto_id.id

                # Custos produto de origem
                custos_origem = obj_preco_custo.get_custos_do_dia(
                    cr, uid, origem_produto_id, company_id, context=context)
                custos_origem = custos_origem.get(origem_produto_id) or {}
                vr_custo_entrada_producao = custos_origem.get(custo_entrada_producao) or 0.0
                vr_total_item = vr_custo_entrada_producao

                # Custos produto de destino
                custos = obj_preco_custo.get_custos_do_dia(cr, uid, destino_produto_id,
                                                           company_id, context=context)
                custos = custos.get(destino_produto_id) or {}
                custo_medio_contabil = custos.get('estprc_custo_medio_contabil') or 0.0
                custo_corrigido = custos.get('estprc_custo_corrigido') or 0.0

                qtd_estoque = obj_estoque_produto_saldo.get_saldo_do_dia(
                    cr, uid, destino_grade_id, time.strftime('%Y-%m-%d'),
                    context=context)
                qtd_estoque = qtd_estoque.get(destino_grade_id).get('saldo')

                novo_ultimo_custo = vr_total_item / qtd_total_item
                novo_custo_medio_contabil = ((
                    novo_ultimo_custo * qtd_total_item) + (
                    custo_medio_contabil * qtd_estoque)) / (qtd_total_item + qtd_estoque)
                novo_custo_reposicao = vr_total_item / qtd_total_item
                novo_custo_corrigido = ((
                    novo_ultimo_custo * qtd_total_item) + (
                    custo_corrigido * qtd_estoque)) / (qtd_total_item + qtd_estoque)

                obj_preco_custo.create(cr, uid, {'estprc_custo_corrigido': novo_custo_corrigido,
                                                 'estprc_custo_medio_contabil': novo_custo_medio_contabil,
                                                 'estprc_ultimo_custo': novo_ultimo_custo,
                                                 'estprc_custo_reposicao': novo_custo_reposicao,
                                                 'produto_id': destino_produto_id,
                                                 'company_id': company_id,
                                                 'active': True,
                                                 'estprc_data': time.strftime('%Y-%m-%d %H:%M:%S')},
                                       context=context)
        return True


    def _get_produto_grade_ids(self, dados_itens_req, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        produto_ids, grade_ids = super(estoqueressuprimento_requisicao_capa,
                                       self)._get_produto_grade_ids(dados_itens_req, context)
        for i in dados_itens_req:
            if i.get('etrrqi_destino_produto_id'):
                produto_ids.append(i.get('etrrqi_destino_produto_id'))
            if i.get('etrrqi_destino_grade_id'):
                grade_ids.append(i.get('etrrqi_destino_grade_id'))

        return list(set(produto_ids)), list(set(grade_ids))

    def _get_campos_itens(self, context=None):
        context = context and context.copy() or {}
        campos = ['etrrqi_destino_produto_id', 'etrrqi_destino_grade_id',
                  'etrrqi_destino_quantidade']
        campos.extend(super(estoqueressuprimento_requisicao_capa, self)._get_campos_itens(context))
        return campos

    def _get_embalagem(self, tipo_movimento, item, dict_emb, context=None):
        context = context and context.copy() or {}
        res = super(estoqueressuprimento_requisicao_capa, self)._get_embalagem(tipo_movimento, item,
                                                                               dict_emb, context)
        if tipo_movimento != 'E': # E=entrada, S=saida
            return res

        if item.get('etrrqi_destino_produto_id'):
            res =  dict_emb.get(item.get('etrrqi_destino_produto_id'))

        return res

    def _get_produto_grade_id(self, tipo_movimento, item, context=None):
        context = context and context.copy() or {}
        res = super(estoqueressuprimento_requisicao_capa, self)._get_produto_grade_id(
                                                                      tipo_movimento, item, context)
        if tipo_movimento != 'E' or not all(
                (item.get('etrrqi_destino_produto_id'), item.get('etrrqi_destino_grade_id'))):
            return res

        return item.get('etrrqi_destino_produto_id'), item.get('etrrqi_destino_grade_id')

    def _get_qtd_item(self, tipo_movimento, item, qtd_emb, context=None):
        'Ha uma heranca para este metodo em PCP'
        context = context and context.copy() or {}
        res = super(estoqueressuprimento_requisicao_capa, self)._get_qtd_item(tipo_movimento, item,
                                                                              qtd_emb, context)
        if tipo_movimento != 'E': # E=entrada, S=saida
            return res

        if item.get('etrrqi_destino_quantidade'):
            res = item.get('etrrqi_destino_quantidade')

        return res
