# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from funcoes import _pcpftc_fields_function, _pcpftc_get_quantidade_unidade_peso, \
                    pcpftc_get_ficha_capa_id_custo, pcpftc_get_ficha_capa_id_sub_receita


class pcp_ficha_tecnica_capa(osv.Model):
    """
    Cadastro da Ficha Tecnica
    """

    _name = 'pcp.ficha.tecnica.capa'
    _inherits = {'estoque.produto': 'pcpftc_produto_id'}
    _roteavel = True
    _columns = {
        # ___________________________ Integer ________________________________
        'id': fields.integer(),
        'pcpftc_qtd_segunda': fields.float('Quantidade segunda', digits=(12, 3)),
        'pcpftc_qtd_terca': fields.float('Quantidade terca', digits=(12, 3)),
        'pcpftc_qtd_quarta': fields.float('Quantidade quarta', digits=(12, 3)),
        'pcpftc_qtd_quinta': fields.float('Quantidade quinta', digits=(12, 3)),
        'pcpftc_qtd_sexta': fields.float('Quantidade sexta', digits=(12, 3)),
        'pcpftc_qtd_sabado': fields.float('Quantidade sabado', digits=(12, 3)),
        'pcpftc_qtd_domingo': fields.float('Quantidade domingo', digits=(12, 3)),

        # ___________________________ Many2One _______________________________
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade', required=True,
                                      select=True),
        'pcpftc_produto_id': fields.many2one('estoque.produto', 'Produto', ondelete='restrict',
                                             required=True,
                                             domain="[('tipo_item','in',['03','04','05','06'])]"),
        'pcpftc_grade_id': fields.many2one(  # Id da grade do produto a ser produzido
            'estoque.produto.grade', 'Grade', ondelete='restrict',
                      domain="[('produto_id', '=', pcpftc_produto_id)]", required=True, select=True,),
        'pcpftc_produto_peso_embalagem_id': fields.many2one(
            'estoque.produto.embalagem', 'Embalagem (peso)', ondelete="restrict", required=True,
            select=True, oldname='pcpftc_produto_embalagem_id',
            domain="[('produto_id', '=', pcpftc_produto_id)]"),
        'pcpftc_centro_custo_id': fields.many2one('adm.empresa.centrocusto', 'Centro de custo'),
        'pcpftc_produto_unidade_embalagem_id': fields.many2one('estoque.produto.embalagem',
                                                               'Embalagem (opcional)',
                                                               domain="[('produto_id', '=', pcpftc_produto_id)]"),


        # ___________________________ One2Many _______________________________
        'pcpftc_ficha_tecnica_etapas_ids': fields.one2many('pcp.ficha.tecnica.etapas',
                                                           'pcpfte_capa_id', 'Etapas'),
        'pcpftc_ficha_tecnica_itens_ids': fields.one2many('pcp.ficha.tecnica.itens',
                                                          'pcpfti_capa_id', 'Materia prima/insumo'),
        'pcpftc_ordem_producao_ids': fields.one2many('pcp.ordem.producao',
                                                     'pcpopr_ficha_tecnica_capa_id',
                                                     'Ordens de producao'),
        'pcpftc_ficha_tecnica_embalagem_ids': fields.one2many('pcp.ficha.tecnica.embalagem',
                                                              'pcpfte_capa_id',
                                                              'Embalagens'),

        # ___________________________ Float __________________________________
        'pcpftc_quantidade_peso': fields.float('Quantidade (peso)', digits=(12, 3), required=True,
                                               help="Quantidade a produzir",
                                               oldname='pcpftc_quantidade'),
        'pcpftc_quantidade_unidade': fields.float('Quantidade (opcional)', digits=(12, 3)),

        # ___________________________ Text ___________________________________
        'pcpftc_observacao': fields.text('Informacoes complementares'),

        # ___________________________ Function________________________________
        'pcpftc_utiliza_custo_evento': fields.function(
            _pcpftc_fields_function, method=True, string='Utiliza centro de custos e eventos',
            type='boolean', multi=True),
        'pcpftc_custo_total': fields.function(_pcpftc_fields_function, string="Custo total",
                                              digits=(12, 3), method=True,
                                              store={'estoque.preco.custo': (pcpftc_get_ficha_capa_id_custo, [], 10),
                                                     'pcp.ficha.tecnica.capa': (pcpftc_get_ficha_capa_id_sub_receita, [], 10)},
                                              multi=True),
        'pcpftc_custo_unitario': fields.function(_pcpftc_fields_function, string="Custo total",
                                                 digits=(12, 3), method=True, store=True,
                                                 multi=True),
        'pcpftc_peso_unidade': fields.function(_pcpftc_fields_function, string='Peso unidade',
                                               digits=(12, 3), method=True, multi=True),
        'pcpftc_valor_embalagem': fields.function(_pcpftc_fields_function, string='Valor embalagem',
                                                  digits=(12, 3), method=True, multi=True),
        'pcpftc_qtd_und_domingo': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_domingo': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_und_segunda': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_segunda': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_und_terca': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_terca': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_und_quarta': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_quarta': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_und_quinta': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_quinta': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_und_sexta': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_sexta': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_und_sabado': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),
        'pcpftc_qtd_peso_sabado': fields.function(
            _pcpftc_get_quantidade_unidade_peso, method=True, digits=(12, 3), type='float',
            store=False, multi='unidade_peso'),

        # ___________________________ Boolean ________________________________
        'pcpftc_domingo': fields.boolean('Domingo'),
        'pcpftc_segunda': fields.boolean('Segunda-feira'),
        'pcpftc_terca': fields.boolean('Terca-feira'),
        'pcpftc_quarta': fields.boolean('Quarta-feira'),
        'pcpftc_quinta': fields.boolean('Quinta-feira'),
        'pcpftc_sexta': fields.boolean('Sexta-feira'),
        'pcpftc_sabado': fields.boolean('Sabado'),
        'active': fields.boolean('Ativo')
    }

    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
        'active': True,
        'pcpftc_domingo': False,
        'pcpftc_segunda': False,
        'pcpftc_terca': False,
        'pcpftc_quarta': False,
        'pcpftc_quinta': False,
        'pcpftc_sexta': False,
        'pcpftc_sabado': False
    }

    _sql_constraints = [
        ('pcpftc_produto_uniq', 'unique (pcpftc_produto_id, pcpftc_grade_id)',
         'Cada ficha tecnica deve possuir um produto diferente'),
    ]
