# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from dateutil.parser import parse

# _______________________ pcp.ficha.tecnica.embalagem ________________________
def _pcpfte_fields_function(self, cr, uid, ids, field_name, args, context=None):
    """
    Trata os campos do tipo ´´function´´ da classe pcp_ficha_tecnica_embalagem

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param field_name: {list} lista com o nome dos campos,
    @param args: {dict} dicionario com os argumentos e valores,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {dict} valores ajustados dos campos.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}.fromkeys(ids, False)

    for brw_registro in self.browse(cr, uid, ids, context):
        try:
            brw_produto = brw_registro.pcpfte_produto_id

            if field_name == 'name':
                valor = '%s - %s' % (brw_produto.codigo, brw_produto.name)
            else:
                valor = False
        except:
            valor = False

        result[brw_registro.id] = valor

    return result


# _______________________ pcp.ordem.producao.embalagem _______________________
def pcpope_validar_quantidade(self, cr, uid, ids):
    """
    Valida a quantidade informada para a embalagem.

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @return: {boolean} verdadeiro se a quantidade for maior que zero.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = not any(filter(lambda op: op.pcpope_quantidade <= 0, self.browse(cr, uid, ids)))

    return result


# _______________________ pcp.balanco.producao _______________________________
# Balanco de producao
def _pcpbpr_fields_function(self, cr, uid, ids, field_name, args, context=None):
    """
    Trata os campos do tipo ´´function´´ da classe pcp_balanco_producao

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param field_name: {list} lista com o nome dos campos,
    @param args: {dict} dicionario com os argumentos e valores,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {dict} valores ajustados dos campos.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}.fromkeys(ids, False)

    # ___________________________ Objects ________________________________
    obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
    obj_pcp_ordem_producao = self.poo.get('estoque.movimento.itens')

    for brw_registro in self.browse(cr, uid, ids, context):
        try:
            brw_produto = brw_registro.pcpbpr_ficha_tecnica_id.pcpftc_produto_id
            ficha_tecnica_id = brw_registro.pcpbpr_ficha_tecnica_id.id
            produto_id = brw_produto.id

            if field_name == 'pcpbpr_preco_venda':
                valor = obj_estoque_preco_venda.get_preco_venda_1(cr, uid, produto_id)
                valor = valor.get(produto_id, 0.0)
            elif field_name == 'pcpbpr_producao':
                args = [('pcpopr_ficha_tecnica_capa_id', '=', ficha_tecnica_id)]
                ordem_producao_ids = obj_pcp_ordem_producao.search(cr, uid, args, context=context)

                # Dados da ordem de producao
                campos = ['pcpopr_quantidade_produzida']
                dados_ordem_producao = obj_pcp_ordem_producao.read(
                    cr, uid, ordem_producao_ids, campos, context=context)

                # Somatorio das quantidades produzidas (ordem de producao)
                if dados_ordem_producao and isinstance(dados_ordem_producao, list):
                    valor = sum(map(lambda dop: dop.get('pcpopr_quantidade_produzida', 0.0),
                                    dados_ordem_producao))
                else:
                    valor = dados_ordem_producao.get('pcpopr_quantidade_produzida', 0.0)
            else:
                valor = False
        except:
            valor = False

        result[brw_registro.id] = valor

    return result

#Funcoes da classe pcp_ficha_tecnica_itens
READ_LOAD = '_classic_write'


def _get_pcpfti_custo_total(self, cr, uid, ids, fields, args, context=None):
    """

    :param cr:
    :param uid:
    :param ids:
    :param fields:
    :param args:
    :param context:
    :type cr:
    :type uid:
    :type ids:
    :type fields:
    :type args:
    :type context:
    :return:
    :rtype:
    """

    context = context and context.copy() or {}
    result = {}.fromkeys(ids, 0.0)

    # Recupera no parametro do Estoque o tipo de custo a ser utilizado no calculo do custo total
    estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
        cr, uid, tabela='estoque', campos='estpar_custo_para_precovenda', context=context)
    tipo_custo = {
        'U': 'estprc_ultimo_custo',
        'M': 'estprc_custo_medio_contabil',
        'C': 'estprc_custo_corrigido',
        'R': 'estprc_custo_reposicao',
    }.get(estoque_parametro_dados.get('estpar_custo_para_precovenda'))

    # Recupera os dados dos itens da ficha tecnica
    campos = ['pcpfti_produto_id', 'pcpfti_quantidade']
    item_dados = self.read(cr, uid, ids, campos, context, '_classic_write')
    dado_item_dict = {}

    sql = """
        SELECT 
            DISTINCT 
            i.id AS id,
            c.id AS ficha_id
        FROM pcp_ficha_tecnica_capa c
        INNER JOIN pcp_ficha_tecnica_itens i
            ON c.pcpftc_produto_id = i.pcpfti_produto_id
        WHERE 
            i.pcpfti_produto_id IN (%s) 
            AND i.id IN (%s)
            AND c.active IS TRUE
    """ % (', '.join(map(lambda i: str(i.get('pcpfti_produto_id')), item_dados)),
           ', '.join(map(lambda i: str(i), ids)))
    cr.execute(sql)
    dados_capa_item = cr.dictfetchall()
    capa_item = {}
    for dado in dados_capa_item:
        capa_item.update({dado.get('id'): dado.get('ficha_id')})

    for item in item_dados:
        quantidade = item.get('pcpfti_quantidade')
        # Se o item for uma ficha tecnica soma os custos dos itens da ficha
        dado_item_dict[item.get('id')] = {}
        if capa_item.get(item.get('id')):
            ficha_id = capa_item.get(item.get('id'))
            dado_item_dict.get(item.get('id')).update(
                {'ficha_id': ficha_id, 'quantidade': quantidade})
        else:
            dado_item_dict.get(item.get('id')).update({'produto_id': item.get('pcpfti_produto_id'),
                                                       'quantidade': quantidade})

    ficha_tecnica_capa_dados = self.pool.get('pcp.ficha.tecnica.capa').read(
        cr, uid, map(lambda i: i.get('ficha_id'), dados_capa_item),
        ['pcpftc_custo_total', 'pcpftc_quantidade_peso'], context=context)
    custo_ficha_dict = {}
    for capa in ficha_tecnica_capa_dados:
        custo_ficha_dict[capa.get('id')] = capa

    dado_item_produto_id = filter(lambda i: i.get('produto_id'), dado_item_dict.values())
    produto_ids = map(lambda i: i.get('produto_id'), dado_item_produto_id)
    custo_item = self.pool.get('estoque.preco.custo').get_custos_do_dia(
        cr, uid, epc_produto_ids=produto_ids, context=context)

    for item_id in dado_item_dict.keys():
        if dado_item_dict.get(item_id).get('ficha_id'):
            custo_ficha = custo_ficha_dict.get(dado_item_dict.get(item_id).get('ficha_id')).get(
                'pcpftc_custo_total')
            qtd_ficha = custo_ficha_dict.get(dado_item_dict.get(item_id).get('ficha_id')).get(
                'pcpftc_quantidade_peso')
            custo = custo_ficha / qtd_ficha
        else:
            custo = custo_item.get(dado_item_dict.get(item_id).get('produto_id'), {}).get(
                tipo_custo, 0)
        result[item_id] = custo * dado_item_dict.get(item_id).get('quantidade')

    return result


def _pcpopr_get_requisicao_automatica(self, cr, uid, ids, field_name, args, context=None):
    """
    O metodo verifica nos parametros do estoque se na aba 'Pcp' esta definido que sera gerado
    requisicao de almoxarifado automatica ou nao
    :param cr: cursor no banco de dados
    :param uid: resgistro da tabela res_users, identificador do usuario
    :param ids: lista de registros da tabela referente
    :param field_name: nome da coluna na tabela (atributo)
    :param args:
    :param context: dicionario com valores contextuais
    :return: dicionario, suas chaves representam os ids da tabela e os valores das chaves
    representam o valor do parametro
    """
    context = context and context.copy() or {}
    res = {}
    estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
        cr, uid, 'estoque', ['estpar_pcp_gerar_requisicao_automatica'], context=context)
    gerar_requisicao_automatica = estoque_parametro_dados.get(
        'estpar_pcp_gerar_requisicao_automatica')
    brw_ordem_producao = self.pool.get('pcp.ordem.producao').browse(cr, uid, ids, context=context)
    for x in brw_ordem_producao:
        res[x.id] = gerar_requisicao_automatica
    return res


def _pcpopr_get_qtd_a_produzir(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]

    res = {}.fromkeys(ids, 0)

    browse_ordem_producao = self.browse(cr, uid, ids, context=context)
    for i in browse_ordem_producao:
        ordem_id = i.id
        quantidade = i.pcpopr_quantidade or 0

        record_ficha_tecnica = i.pcpopr_ficha_tecnica_capa_id
        ficha_tecnica_qtde = record_ficha_tecnica and record_ficha_tecnica.pcpftc_quantidade_peso or 0

        res[ordem_id] = quantidade * ficha_tecnica_qtde

    return res


def _pcpopr_get_percentual_producao(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]

    res = {}.fromkeys(ids, 0)

    fields_to_read = ['pcpopr_quantidade_a_produzir', 'pcpopr_quantidade_produzida']
    ordem_producao_dados = self.read(cr, uid, ids, fields_to_read, context=context)
    for i in ordem_producao_dados:
        ordem_id = i.get('id')
        quantidade_a_ser_produzida = i.get('pcpopr_quantidade_a_produzir')
        quantidade_produzida = i.get('pcpopr_quantidade_produzida')
        if quantidade_a_ser_produzida:
            percentual = quantidade_produzida * 100 / quantidade_a_ser_produzida
        else:
            percentual = 0

        res[ordem_id] = percentual
    return res


def _pcpopr_get_parametro_gerar_nota(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    obj_adm_utils = self.pool.get('adm.utils')
    estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque',
                                                                  ['estpar_pcp_gerar_nota_fiscal'],
                                                                  context=context)
    pcp_gerar_nota = estoque_parametro_dados.get('estpar_pcp_gerar_nota_fiscal')
    browse_ordem_producao = self.browse(cr, uid, ids)
    for ordem_producao in browse_ordem_producao:
        res[ordem_producao.id] = pcp_gerar_nota
    return res

# Funcoes da classe pcp_ordem_producao_embalagem


def _pcpope_get_custo_total(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = dict()
    if isinstance(ids, (int, long)):
        ids = [ids]
    fields_to_read = ['pcpope_produto_id', 'pcpope_quantidade']
    ordem_producao_embalagem_dados = self.read(cr, uid, ids, fields_to_read, context=context,
                                               load=READ_LOAD)
    produto_ids = map(lambda item: item.get('pcpope_produto_id'),
                      ordem_producao_embalagem_dados)

    dict_produto_custos = self.pool.get('estoque.preco.custo').get_custos_do_dia(
        cr, uid, produto_ids)

    estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
        cr, uid, tabela='estoque', campos='estpar_custo_para_precovenda', context=context)
    custo_para_preco_venda = estoque_parametro_dados.get('estpar_custo_para_precovenda')

    for i in ordem_producao_embalagem_dados:

        produto_id = i.get('pcpope_produto_id')
        quantidade = i.get('pcpope_quantidade') or 0
        dict_custo = dict_produto_custos.get(produto_id) or {}

        if custo_para_preco_venda == 'U':
            custo = dict_custo.get('estprc_ultimo_custo') or 0
        elif custo_para_preco_venda == 'M':
            custo = dict_custo.get('estprc_custo_medio_contabil') or 0
        elif custo_para_preco_venda == 'C':
            custo = dict_custo.get('estprc_custo_corrigido') or 0
        elif custo_para_preco_venda == 'R':
            custo = dict_custo.get('estprc_custo_reposicao') or 0
        else:
            custo = 0
        custo_total = custo * quantidade
        res[i.get('id')] = custo_total
    return res


# Funcoes da classe pcp_ficha_tecnica_capa


def _pcpftc_fields_function(self, cr, uid, ids, field_name, args, context=None):
    """
    Manipulacao de campos do tipo ´´function´´.

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param field_name: {list} lista com o nome dos campos,
    @param args: {dict} dicionario com os argumentos e valores,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {dict} valores das chaves com os totais calculados.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}
    campos_financeiro = ['usar_centro_custos_e_eventos']
    financeiro_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
        cr, uid, 'financeiro', campos_financeiro, context=context)
    for brw_ficha_tecnica in self.browse(cr, uid, ids, context):
        ficha_tecnica_id = brw_ficha_tecnica.id
        if not result.get(ficha_tecnica_id):
            result[ficha_tecnica_id] = {'pcpftc_custo_total': 0.0, 'pcpftc_peso_unidade': 0.0}

        custo_total = 0
        peso = brw_ficha_tecnica.pcpftc_quantidade_peso
        unidade = brw_ficha_tecnica.pcpftc_quantidade_unidade

        if unidade:  # Evita divisao por zero. ;)
            result[ficha_tecnica_id]['pcpftc_peso_unidade'] = peso / unidade
        else:
            result[ficha_tecnica_id]['pcpftc_peso_unidade'] = 0.0

        for ficha_tecnica_item in brw_ficha_tecnica.pcpftc_ficha_tecnica_itens_ids:
            fti_custo_total = ficha_tecnica_item.pcpfti_custo_total or 0
            custo_total += fti_custo_total

        result[ficha_tecnica_id]['pcpftc_custo_total'] = custo_total

        # Valor total das embalagens
        embalagem_ids = map(lambda embalagem: embalagem.id,
                            brw_ficha_tecnica.pcpftc_ficha_tecnica_embalagem_ids)
        result[ficha_tecnica_id]['pcpftc_valor_embalagem'] = self._pcpftc_calcular_total_embalagem(
            cr, uid, embalagem_ids)

        if financeiro_parametro_dados.get('usar_centro_custos_e_eventos'):
            result[ficha_tecnica_id]['pcpftc_utiliza_custo_evento'] = True

    return result


def _pcpftc_get_quantidade_unidade_peso(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    brw_ficha_tecnica_capa = self.browse(cr, uid, ids, context)
    vals_und = {}
    vals_peso = {}
    for x in brw_ficha_tecnica_capa:
        quantidade_unidade = x.pcpftc_quantidade_unidade or 0
        quantidade_peso = x.pcpftc_quantidade_peso or 0
        und_domingo = peso_domingo = und_segunda = peso_segunda = und_terca = peso_terca = \
            und_quarta = peso_quarta = und_quinta = peso_quinta = und_sexta = peso_sexta = \
            und_sabado = peso_sabado = 0
        vals_und.update({'pcpftc_qtd_und_domingo': und_domingo,
                         'pcpftc_qtd_und_segunda': und_segunda,
                         'pcpftc_qtd_und_terca': und_terca,
                         'pcpftc_qtd_und_quarta': und_quarta,
                         'pcpftc_qtd_und_quinta': und_quinta,
                         'pcpftc_qtd_und_sexta': und_sexta,
                         'pcpftc_qtd_und_sabado': und_sabado,
                         })
        vals_peso.update({'pcpftc_qtd_peso_domingo': peso_domingo,
                          'pcpftc_qtd_peso_segunda': peso_segunda,
                          'pcpftc_qtd_peso_terca': peso_terca,
                          'pcpftc_qtd_peso_quarta': peso_quarta,
                          'pcpftc_qtd_peso_quinta': peso_quinta,
                          'pcpftc_qtd_peso_sexta': peso_sexta,
                          'pcpftc_qtd_peso_sabado': peso_sabado
                          })
        if quantidade_unidade > 0:
            und_domingo = x.pcpftc_qtd_domingo * quantidade_unidade
            und_segunda = x.pcpftc_qtd_segunda * quantidade_unidade
            und_terca = x.pcpftc_qtd_terca * quantidade_unidade
            und_quarta = x.pcpftc_qtd_quarta * quantidade_unidade
            und_quinta = x.pcpftc_qtd_quinta * quantidade_unidade
            und_sexta = x.pcpftc_qtd_sexta * quantidade_unidade
            und_sabado = x.pcpftc_qtd_sabado * quantidade_unidade

            vals_und.update({'pcpftc_qtd_und_domingo': und_domingo,
                             'pcpftc_qtd_und_segunda': und_segunda,
                             'pcpftc_qtd_und_terca': und_terca,
                             'pcpftc_qtd_und_quarta': und_quarta,
                             'pcpftc_qtd_und_quinta': und_quinta,
                             'pcpftc_qtd_und_sexta': und_sexta,
                             'pcpftc_qtd_und_sabado': und_sabado
                             })
        if quantidade_peso > 0:
            peso_domingo = x.pcpftc_qtd_domingo * quantidade_peso
            peso_segunda = x.pcpftc_qtd_segunda * quantidade_peso
            peso_terca = x.pcpftc_qtd_terca * quantidade_peso
            peso_quarta = x.pcpftc_qtd_quarta * quantidade_peso
            peso_quinta = x.pcpftc_qtd_quinta * quantidade_peso
            peso_sexta = x.pcpftc_qtd_sexta * quantidade_peso
            peso_sabado = x.pcpftc_qtd_sabado * quantidade_peso

            vals_peso.update({'pcpftc_qtd_peso_domingo': peso_domingo,
                              'pcpftc_qtd_peso_segunda': peso_segunda,
                              'pcpftc_qtd_peso_terca': peso_terca,
                              'pcpftc_qtd_peso_quarta': peso_quarta,
                              'pcpftc_qtd_peso_quinta': peso_quinta,
                              'pcpftc_qtd_peso_sexta': peso_sexta,
                              'pcpftc_qtd_peso_sabado': peso_sabado
                              })
        vals_und.update(vals_peso)
        res[x.id] = vals_und
    return res


# Funcoes da classe pcp_planejamento_producao


def _get_name_produto(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    brw_planejamento_producao = self.browse(cr, uid, ids, context)
    for x in brw_planejamento_producao:
        res[x.id] = x.pljprd_ficha_tecnica_id.pcpftc_produto_id.name
    return res


# Funcoes da classe wiz_pcp_gerar_ordem_producao
# Funcoes da classe pcp_planejamento_producao
def _get_categoria_produto(self, cr, uid, ids, field_name, args, context=None):
    """
    Manipulacao de campos do tipo ´´function´´.

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param field_name: {list} lista com o nome dos campos,
    @param args: {dict} dicionario com os argumentos e valores,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {dict} dicionario com o nome da categoria.

    @author: Dener Rodrigues <denner.silva[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}
    res = {}

    if isinstance(field_name, list):
        field_name = field_name[0]

    if field_name == 'pljprd_categoria':
        obj_planejamento_producao = self.pool.get('pcp.planejamento.producao')
        brw_planejamento_producao = obj_planejamento_producao.browse(cr, uid, ids, context=context)
        for x in brw_planejamento_producao:
            categoria_id = x.pljprd_ficha_tecnica_id.pcpftc_produto_id.categoria_id
            if categoria_id:
                res[x.id] = categoria_id.name

    elif field_name == 'wizgoi_categoria':
        obj_ordem_producao_item = self.pool.get('wiz.pcp.gerar.ordem.producao.item')
        brw_ordem_producao_item = obj_ordem_producao_item.browse(cr, uid, ids, context=context)
        for x in brw_ordem_producao_item:
            categoria_id = x.wizgoi_ficha_tecnica_id.pcpftc_produto_id.categoria_id
            if categoria_id:
                res[x.id] = categoria_id.name

    elif field_name == 'wizlmi_categoria':
        obj_listar_material_item = self.pool.get('wiz.pcp.listar.material.item')
        brw_listar_material_item = obj_listar_material_item.browse(cr, uid, ids, context=context)
        for x in brw_listar_material_item:
            categoria_id = x.wizlmi_produto_id.categoria_id
            if categoria_id:
                res[x.id] = categoria_id.name

    return res

def get_unidade_peso(self, cr, uid, id, ficha_tecnica_id, data, res, context=None):
    context = context and context.copy() or {}
    obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
    campos = ['pcpftc_qtd_und_segunda', 'pcpftc_qtd_peso_segunda',
              'pcpftc_qtd_und_terca', 'pcpftc_qtd_peso_terca',
              'pcpftc_qtd_und_quarta', 'pcpftc_qtd_peso_quarta',
              'pcpftc_qtd_und_quinta', 'pcpftc_qtd_peso_quinta',
              'pcpftc_qtd_und_sexta', 'pcpftc_qtd_peso_sexta',
              'pcpftc_qtd_und_sabado', 'pcpftc_qtd_peso_sabado',
              'pcpftc_qtd_und_domingo', 'pcpftc_qtd_peso_domingo']
    ficha_tecnica_dados = obj_pcp_ficha_tecnica_capa.read(cr, uid, ficha_tecnica_id, campos, context=context)
    if isinstance(ficha_tecnica_dados, list):
        ficha_tecnica_dados = ficha_tecnica_dados[0]
    dia_semana = parse(data).date().weekday()
    qtd_peso = 0.0
    qtd_unidade = 0.0
    if dia_semana == 0:  # Segunda
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_segunda')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_segunda')
    elif dia_semana == 1:  # Terca
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_terca')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_terca')
    elif dia_semana == 2:  # Quarta
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_quarta')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_quarta')
    elif dia_semana == 3:  # Quinta
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_quinta')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_quinta')
    elif dia_semana == 4:  # Sexta
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_sexta')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_sexta')
    elif dia_semana == 5:  # Sabado
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_sabado')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_sabado')
    elif dia_semana == 0:  # Domingo
        qtd_unidade = ficha_tecnica_dados.get('pcpftc_qtd_und_domingo')
        qtd_peso = ficha_tecnica_dados.get('pcpftc_qtd_peso_domingo')
    res[id] = {'pljprd_qtd_unidade': qtd_unidade, 'pljprd_qtd_peso': qtd_peso}
    return res

def _pljprd_get_unidade_peso(self, cr, uid, ids, field_name, args, context=None):
    context = context and context.copy() or {}
    res = {}
    if ids and not isinstance(ids, list):
        ids = [ids]
    brw_planejamento_producao = self.browse(cr, uid, ids, context)
    for x in brw_planejamento_producao:
        ficha_tecnica_id = x.pljprd_ficha_tecnica_id.id
        data = x.pljprd_data
        res = get_unidade_peso(self, cr, uid, x.id, ficha_tecnica_id, data, res, context)
    return res


# Funcoes da classe pcp_ficha_tecnica_embalagem_data


def _get_pcpfte_get_custo(self, cr, uid, ids, field, args, context=None):
    context = context and context.copy() or {}
    if isinstance(ids, (int, long)):
        ids = [ids]
    res = self.pool.get('pcp.util').pcputl_get_custo_unitario_total(cr, uid, 'E', ids, context)
    return res

def _get_fichas_pai_ids(self, cr, uid, ficha_ids, context=None):
    '''
    Procuras se ha fichas tecnicas pai para os ids de fichas tecnicas passados.
    '''
    context = context and context.copy() or {}
    if not ficha_ids:
        return []

    dados_capa = self.pool.get('pcp.ficha.tecnica.capa').read(cr, uid, ficha_ids,
                                     ['pcpftc_produto_id'], load='_classic_write', context=context)
    produto_ids = map(lambda x: x.get('pcpftc_produto_id'), dados_capa)
    obj_ficha_itens = self.pool.get('pcp.ficha.tecnica.itens')
    ficha_itens_ids = obj_ficha_itens.search(cr, uid, [('pcpfti_produto_id','in',produto_ids)],
                                           context=context)
    if not ficha_itens_ids: # nao possui ficha tecnica pai
        return []

    res = list(set(map(lambda x: x.get('pcpfti_capa_id'), obj_ficha_itens.read(cr, uid,
                       ficha_itens_ids,['pcpfti_capa_id'],context=context, load='_classic_write'))))
    # funcao recursiva que vai procurando se a ficha possui pai
    res.extend(_get_fichas_pai_ids(self, cr, uid, res, context=context))
    return res

def pcpftc_get_ficha_capa_id_custo(self, cr, uid, ids, context=None):
    '''
    Funcao que retorna o uma lista de ids da classe pcp.ficha.tecnica.capa onde o custo de um
    produto que eh item de uma ficha tecnica foi alterado
    :param ids: ids da classe estoque.preco.custo
    :return: lista com ids da classe 'pcp.ficha.tecnica.capa'
    '''
    context = context and context.copy() or {}
    dados_preco_custo = self.read(cr, uid, ids, ['produto_id'], load='_classic_write',
                                  context=context)
    produto_ids = map(lambda x: x.get('produto_id'), dados_preco_custo)
    obj_ficha_itens = self.pool.get('pcp.ficha.tecnica.itens')
    ficha_itens_ids = obj_ficha_itens.search(cr, uid,  [('pcpfti_produto_id','in',produto_ids)],
                                             context=context)
    res = list(set(map(lambda x: x.get('pcpfti_capa_id'), obj_ficha_itens.read(cr, uid,
                       ficha_itens_ids,['pcpfti_capa_id'],context=context, load='_classic_write'))))
    res.extend(_get_fichas_pai_ids(self, cr, uid, res, context=context))
    return res

def pcpftc_get_ficha_capa_id_sub_receita(self, cr, uid, ids, context=None):
    '''
    Esta funcao busca verifica se a ficha tecnica possi ficha pai e retorna os ids da pai
    :param ids: ids da classe 'pcp.ficha.tecnica.capa'(filhas)
    :return: lista de ids classe 'pcp.ficha.tecnica.capa'(pai)
    '''
    context = context and context.copy() or {}
    res = ids
    res.extend(_get_fichas_pai_ids(self, cr, uid, ids, context=context))
    return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
