# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class pcp_ordem_producao(osv.Model):

    from funcoes import _pcpopr_get_qtd_a_produzir, _pcpopr_get_percentual_producao
    from funcoes import _pcpopr_get_parametro_gerar_nota, _pcpopr_get_requisicao_automatica

    _name = 'pcp.ordem.producao'  # Prefixo: pcpopr_
    _description = 'Ordem de Producao'

    _columns = {
        # ___________________________ Many2One _______________________________
        'company_id': fields.many2one('res.company', 'Empresa', ondelete="cascade", required=True,
                                      select=1),
        'pcpopr_responsavel_id': fields.many2one("rh.funcionario", 'Responsavel',
                                                 ondelete="restrict", required=True, select=1),
        'pcpopr_ficha_tecnica_capa_id': fields.many2one("pcp.ficha.tecnica.capa", 'Ficha tecnica',
                                                        ondelete="restrict",
                                                        required=True, select=True),
        'pcpopr_nf_entrada_id': fields.many2one('fiscal.nf.capa', 'Nota Fiscal Entrada',
                                                readonly=True, ondelete="set null"),
        'pcpopr_nf_saida_id': fields.many2one('fiscal.nf.capa', 'Nota Fiscal Saida', readonly=True,
                                              ondelete="set null"),

        # ___________________________ One2Many _______________________________
        'pcpopr_requisicao_ids': fields.one2many('almoxarifado.requisicao.capa',
                                                 'almrqc_ordem_producao_id', 'Requisicoes'),
        # Produtos do tipo ´´Embalagem´´ (estoque_produto.tipo_item = '02 - Embalagem')
        'pcpopr_embalagem_ids': fields.one2many('pcp.ordem.producao.embalagem',
                                                'pcpope_ordem_producao_id', 'Embalagens'),

        # ___________________________ Integer ________________________________
        'pcpopr_quantidade_unidade': fields.integer('Quantidade unidade'),

        # ___________________________ Char ___________________________________
        'name': fields.char('Descricao', size=45, required=True, select=1),
        'pcpopr_numero': fields.char('Numero', size=15, required=False, select=1, readonly=True),

        # ___________________________ Float __________________________________
        'pcpopr_quantidade': fields.float('Quantidade de receitas', digits=(12, 3), required=True),
        'pcpopr_quantidade_produzida': fields.float('Quantidade produzida', digits=(12, 3)),
        'pcpopr_quantidade_peso': fields.float('Quantidade peso', digits=(12, 3)),

        # ___________________________ Function _______________________________
        'pcpopr_quantidade_a_produzir': fields.function(_pcpopr_get_qtd_a_produzir,
                                                        string='Quantidade a Produzir', store=True,
                                                        method=True, digits=(12, 3)),
        'pcpopr_percentual_producao': fields.function(_pcpopr_get_percentual_producao,
                                                      string="Percentual Producao", store=True,
                                                      method=True, digits=(12, 3)),
        'pcpopr_gerar_nota': fields.function(_pcpopr_get_parametro_gerar_nota, method=True,
                                             string='Gerar Nota Fiscal', type='boolean'),
        'pcpopr_gerar_requisicao_automatica': fields.function(
            _pcpopr_get_requisicao_automatica, method=True, string='Gerar requisicao automatica',
            type='boolean', store=False),

        # ___________________________ Datetime _______________________________
        'pcpopr_inicio': fields.datetime('Inicio', readonly=True),
        'pcpopr_termino': fields.datetime('Fim', readonly=True),

        # ___________________________ Selection ______________________________
        'state': fields.selection([('E', 'Esboco'), ('A', 'Aberta'), ('F', 'Fechado'),
                                   ('N', 'Notas Fiscais Geradas'), ('C', 'Cancelada')],
                                  'Estado', size=1),
        'pcp_flag_uni_quantidade':fields.integer('Flag quant.unidade'),

        # ___________________________ Boolean ________________________________
        'pcpopr_gerado_automatico': fields.boolean('Gerado automatico'),
    }

    _order = 'pcpopr_numero desc'

    _sql_constraints = [('pcpopr_numero_uniq', 'unique (pcpopr_numero)',
                         'O numero precisa ser unico.')]

    _defaults = {
        'state': lambda *a: 'E',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, s._name, context=c),
    }
