# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class pgview_relatorio_sugestao_producao(osv.Model):
    """
    Visao com os dados necessarios para impressao do relatorio de sugestao de producao.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'view.pcp.relatorio.sugestao.producao'
    _auto = False

    _columns = {
        # ___________________________ Integer ________________________________
        'id': fields.integer('id'),
        'pcprpp_ficha_tecnica_id': fields.integer('Ficha tecnica'),
        'pcprpp_categoria_id': fields.integer('Categoria'),
        'pcprpp_categoria_master_id': fields.integer('Categoria master'),
        'pcprpp_categoria_pai_id': fields.integer('Categoria pai'),

        # ___________________________ Char ___________________________________
        'pcprpp_produto': fields.char('Produto', size=60),
        'pcprpp_categoria': fields.char('Categoria', size=50),
        'pcprpp_categoria_pai': fields.char('Categoria pai', size=50),
        'pcprpp_nivel': fields.char('Nivel'),
        'pcprpp_conta': fields.char('Conta', size=20),

        # ___________________________ Boolean ________________________________
        'pcprpp_segunda': fields.boolean('Segunda'),
        'pcprpp_terca': fields.boolean('Terca'),
        'pcprpp_quarta': fields.boolean('Quarta'),
        'pcprpp_quinta': fields.boolean('Quinta'),
        'pcprpp_sexta': fields.boolean('Sexta'),
        'pcprpp_sabado': fields.boolean('Sabado'),
        'pcprpp_domingo': fields.boolean('Domingo')
    }

    _order = 'pcprpp_conta'

    @staticmethod
    def init(cr):
        """
        Visao para mostrar as fichas tecnicas dos produtos baseados na hierarquia das
        categorias

        @type cr: openerp.sql_db.Cursor
        @return: N/A

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        drop_view_if_exists(cr, 'view_pcp_relatorio_sugestao_producao')
        cr.execute(
            """
            CREATE VIEW view_pcp_relatorio_sugestao_producao AS
            WITH RECURSIVE estoque_categoria_rec      AS (
              (
                (
                  SELECT ec.id,
                         ec.name                      AS categoria,
                         ec.parent_id,
                         ec_p.name                    AS categoria_pai,
                         ep.name                      AS produto,
                         ftc.id                       AS ficha_tecnica_id,
                         array[ec.id]                 AS nivel,
                         ftc.pcpftc_segunda           As segunda,
                         ftc.pcpftc_terca             As terca,
                         ftc.pcpftc_quarta            As quarta,
                         ftc.pcpftc_quinta            As quinta,
                         ftc.pcpftc_sexta             As sexta,
                         ftc.pcpftc_sabado            As sabado,
                         ftc.pcpftc_domingo           AS domingo,
                         ec.id                        AS categoria_master_id,
                         ec.conta                     AS conta
                  FROM estoque_categoria              AS ec
                  LEFT  JOIN estoque_categoria        AS ec_p ON ec_p.id = ec.parent_id
                  LEFT  JOIN estoque_produto          AS ep   ON ep.categoria_id = ec.id
                  LEFT  JOIN pcp_ficha_tecnica_capa   AS ftc  ON ftc.pcpftc_produto_id = ep.id
                  WHERE ec.parent_id IS NULL
                )
              )
              UNION
              (
                  SELECT ec.id,
                         ec.categoria,
                         ec.parent_id,
                         ec.categoria_pai,
                         ec.produto,
                         ec.ficha_tecnica_id,
                         (rec.nivel || ec.id),
                         ec.segunda,
                         ec.terca,
                         ec.quarta,
                         ec.quinta,
                         ec.sexta,
                         ec.sabado,
                         ec.domingo,
                         rec.categoria_master_id,
                         ec.conta
                  FROM
                  (
                    SELECT ec.id,
                           ec.name                    AS categoria,
                           ep.name                    AS produto,
                           ec_p.name                  AS categoria_pai,
                           ec.parent_id,
                           ftc.id                     AS ficha_tecnica_id,
                         ftc.pcpftc_segunda           As segunda,
                         ftc.pcpftc_terca             As terca,
                         ftc.pcpftc_quarta            As quarta,
                         ftc.pcpftc_quinta            As quinta,
                         ftc.pcpftc_sexta             As sexta,
                         ftc.pcpftc_sabado            As sabado,
                         ftc.pcpftc_domingo           AS domingo,
                         ec.conta                     AS conta
                    FROM estoque_categoria            AS ec
                    LEFT  JOIN estoque_categoria      AS ec_p ON ec_p.id = ec.parent_id
                    LEFT  JOIN estoque_produto        AS ep  ON ep.categoria_id = ec.id
                    LEFT  JOIN pcp_ficha_tecnica_capa AS ftc ON ftc.pcpftc_produto_id = ep.id
                    WHERE ec.parent_id IS NOT NULL
                  )                                   AS ec,
                          estoque_categoria_rec       AS rec
                  WHERE ec.parent_id = rec.id
              )
            )

            -- Resultado ordenado pelo nivel (hierarquia das categorias) --
            SELECT   ROW_NUMBER() OVER ()             AS id,
                     categoria_master_id              AS pcprpp_categoria_master_id,
                     id                               AS pcprpp_categoria_id,
                     categoria                        AS pcprpp_categoria,
                     CASE WHEN parent_id IS NULL THEN
                         CAST(id AS INT) ELSE
                         CAST(parent_id AS INT) END   AS pcprpp_categoria_pai_id,
                     categoria_pai                    AS pcprpp_categoria_pai,
                     ficha_tecnica_id                 AS pcprpp_ficha_tecnica_id,
                     produto                          AS pcprpp_produto,
                     segunda                          As pcprpp_segunda,
                     terca                            As pcprpp_terca,
                     quarta                           As pcprpp_quarta,
                     quinta                           As pcprpp_quinta,
                     sexta                            As pcprpp_sexta,
                     sabado                           As pcprpp_sabado,
                     domingo                          AS pcprpp_domingo,
                     nivel                            AS pcprpp_nivel,
                     conta                            AS pcprpp_conta
            FROM     estoque_categoria_rec
            WHERE    ficha_tecnica_id IS NOT NULL
            ORDER BY nivel;
            """
        )

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
