#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
from pytz import timezone
from SigUtils.numero import getNumeroEditado
from openerp.report import report_sxw

class rel_curva_abc(report_sxw.rml_parse):
    def __init__(self, cr, uid,name, context=None):
        context = context and context.copy() or {}
        super(rel_curva_abc, self).__init__(cr, uid, name, context)
        # Data de hoje
        tz = context.get('tz') or 'Brazil/East'
        hoje = datetime.now(timezone('UTC')).astimezone(timezone(tz))

        # Nome da empresa
        empresa_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        empresa = self.pool.get('res.company').read(cr, uid, empresa_id, ['name'], context=context)
        empresa = empresa[0].get('name') if isinstance(empresa, list) else empresa.get('name')

        # Categoria
        report_id = context.get('active_id', False)
        campos = ['categoria_ids', 'data_inicial', 'data_final', 'tipo', 'classificar',
                  'agrupar_categoria']
        dados_relatorio = self.pool.get('pcp.wizard.relatorio.curva.abc').read(
            cr, uid, report_id, campos, context=context)

        obj_view_pcp_relatorio_curva_abc = self.pool.get('view.pcp.relatorio.curva.abc')

        args = []  # Sem argumento. Todas as fichas tecnicas de todas as categorias

        # Filtro categorias
        if dados_relatorio.get('categoria_ids'):
            args.append(('pcpabc_categoria_id', '=', dados_relatorio.get('categoria_ids')[0]))

        rel_ids = obj_view_pcp_relatorio_curva_abc.search(
                cr, uid, args, context=context)

        itens_relatorio = obj_view_pcp_relatorio_curva_abc.browse(
            cr, uid, rel_ids)

        quantidade_total = 0
        valor_total = 0
        acumula_dict = {}
        acumulado = 0
        percente_vlr_dict = {}
        percente_vlr = 0
        individual_percente_dict = {}
        valor_total_preco_custo = 0
        for i in itens_relatorio:
            quantidade_total += i.pcpabc_quantidade
            if dados_relatorio.get('agrupar_categoria'):
                valor_total += i.pcpabc_vr_total
                valor_total_preco_custo += i.pcpabc_vr_custo_total
                acumulado += i.pcpabc_vr_total
                acumula_dict[i.id] = acumulado
            else:
                valor_total += i.pcpabc_valor * i.pcpabc_quantidade
                valor_total_preco_custo += i.pcpabc_ultimo_custo * i.pcpabc_quantidade
                acumulado += i.pcpabc_valor * i.pcpabc_quantidade
                acumula_dict[i.id] = acumulado

        for p in itens_relatorio:
            # Verificar classificacao escolhida
            if int(dados_relatorio.get('classificar')) == 1:  # Classificacao por custo
                classificacao = 'Custo'
                if valor_total_preco_custo:
                    if not dados_relatorio.get('agrupar_categoria'):
                        individual_percente_dict[p.id] = ((p.pcpabc_ultimo_custo * p.pcpabc_quantidade)
                                                          * 100) / valor_total_preco_custo
                        percente_vlr += ((
                                         p.pcpabc_ultimo_custo * p.pcpabc_quantidade)
                                         * 100) / valor_total_preco_custo
                        percente_vlr_dict[p.id] = percente_vlr
                    else:
                        individual_percente_dict[p.id] = (p.pcpabc_vr_custo_total *
                                                          100) / valor_total_preco_custo
                        percente_vlr += (p.pcpabc_vr_custo_total * 100) / valor_total_preco_custo
                        percente_vlr_dict[p.id] = percente_vlr

            elif int(dados_relatorio.get('classificar')) == 2:  # Classificacao por venda
                classificacao = 'Venda'
                if valor_total:
                    if not dados_relatorio.get('agrupar_categoria'):
                        individual_percente_dict[p.id] = ((p.pcpabc_valor * p.pcpabc_quantidade)
                                                          * 100) / valor_total
                        percente_vlr += ((p.pcpabc_valor * p.pcpabc_quantidade) * 100) / valor_total
                        percente_vlr_dict[p.id] = percente_vlr
                    else:
                        individual_percente_dict[p.id] = (p.pcpabc_vr_total * 100) / valor_total
                        percente_vlr += (p.pcpabc_vr_total * 100) / valor_total
                        percente_vlr_dict[p.id] = percente_vlr

            else:  # Classificacao por quantidade
                classificacao = 'Quant.'
                if quantidade_total:
                    individual_percente_dict[p.id] = (p.pcpabc_quantidade * 100) / quantidade_total
                    percente_vlr += (p.pcpabc_quantidade * 100) / quantidade_total
                    percente_vlr_dict[p.id] = percente_vlr


        if dados_relatorio and isinstance(dados_relatorio, list):
            dados_relatorio = dados_relatorio[0]

        if dados_relatorio and isinstance(dados_relatorio, dict):
            categoria = dados_relatorio.get('categoria_ids') or ('', 'Todas')
            ficha_tecnica_ids = dados_relatorio.get('ficha_tecnica_ids', [])
            data_inicial = dados_relatorio.get('data_inicial')
            data_final = dados_relatorio.get('data_final')
            tipo = dados_relatorio.get('tipo')
            if tipo == 'E':
                tipo = 'Perda'
                titulo = 'Curva ABC - Perda'
            else:
                tipo = 'Producao'
                titulo = 'Curva ABC - %s' % tipo
            self.localcontext.update({'categoria': categoria,
                                      'ficha_tecnica': ficha_tecnica_ids,
                                      'data_inicial': data_inicial,
                                      'data_final': data_final,
                                      'quantidade_produzida_total': quantidade_total,
                                      'valor_total': getNumeroEditado(valor_total),
                                      'titulo': titulo,
                                      'tipo': tipo,
                                      'acumula_dict': acumula_dict,
                                      'percente_vlr_dict': percente_vlr_dict,
                                      'valor_total_preco_custo': getNumeroEditado(valor_total_preco_custo),
                                      'classificacao': classificacao,
                                      'individual_percente_dict': individual_percente_dict,
                                      'getNumeroEditado': getNumeroEditado,
                                      'agrupar_categoria': dados_relatorio.get('agrupar_categoria')})

        self.localcontext.update({'hoje': hoje, 'empresa': empresa})
        self.context = context


report_sxw.report_sxw('report.rel_curva_abc', 'view.pcp.relatorio.curva.abc',
                      'addons/pcp/report/rel_curva_abc.rml',
                      parser=rel_curva_abc, header=False)

