# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields


class wizard_consulta_venda_diaria(osv.TransientModel):
    # prefixo: wizcvd_

    _name = 'wizard.consulta.venda.diaria'

    _columns = {
        'wizcvd_ficha_tecnica_id': fields.many2one('pcp.ficha.tecnica.capa', 'Ficha tecnica'),
        'wizcvd_local_id': fields.many2one('estoque.local', 'Local'),
        'wizcvd_data_inicial': fields.date('Data inicial'),
        'wizcvd_data_final': fields.date('Data final'),
        'wizcvd_item_ids': fields.one2many('wizard.consulta.venda.diaria.item',
                                           'wizcvi_consulta_venda_diaria_id', 'Itens'),
    }

    def wizcvd_button_pesquisar(self, cr, uid, ids, context=None):
        """
        O metodo realiza a busca dos produtos e de seus dados de acordo com os filtros informados
        para a geracao da venda diaria.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param ids: lista de ids do objeto
        @param context: dicionario com valores contextuais
        @return: True
        """
        context = context and context.copy() or {}
        obj_pcp_util = self.pool.get('pcp.util')
        obj_wizard_consulta_venda_diaria_item = self.pool.get('wizard.consulta.venda.diaria.item')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')
        estoque_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_custo_entrada_producao'], context=context)
        custo_entrada_producao = estoque_parametro.get('estpar_custo_entrada_producao')
        if custo_entrada_producao == 'U':
            custo_entrada_producao = 'estprc_ultimo_custo'
        if custo_entrada_producao == 'C':
            custo_entrada_producao = 'estprc_custo_medio_contabil'
        if custo_entrada_producao == 'O':
            custo_entrada_producao = 'estprc_custo_corrigido'
        if custo_entrada_producao == 'R':
            custo_entrada_producao = 'estprc_custo_reposicao'
        brw_consulta_venda_diaria = self.browse(cr, uid, ids, context=context)
        cr.execute('DELETE FROM wizard_consulta_venda_diaria_item')
        for x in brw_consulta_venda_diaria:
            produto_id = None
            ficha_tecnica_id = x.wizcvd_ficha_tecnica_id
            if ficha_tecnica_id:
                produto_id = ficha_tecnica_id.pcpftc_produto_id.id
            local_id = x.wizcvd_local_id.id
            data_inicial = x.wizcvd_data_inicial
            data_final = x.wizcvd_data_final
            dados = obj_pcp_util.pcputl_gerar_venda_diaria(cr, uid, data_inicial, data_final,
                                                           produto_id, local_id, context)
            produto_ids = map(lambda i: i.get('produto_id'), dados)
            custos = obj_estoque_preco_custo.get_custos_do_dia(cr, uid, produto_ids,
                                                               context=context)
            precos_venda = obj_estoque_preco_venda.get_preco_venda(cr, uid, produto_ids,
                                                                   context=context)
            for d in dados:
                produto_id = d.get('produto_id')
                produto_embalagem_id = d.get('produto_embalagem_id')
                quantidade = d.get('quantidade')
                valor_custo = 0.0
                custo = custos.get(produto_id)
                if custo:
                    valor_custo = custo.get(custo_entrada_producao) or 0.0
                dados_preco_venda = precos_venda.get(produto_id) or {}
                preco_venda = dados_preco_venda.get('preco_venda')
                valor_agregado = preco_venda - valor_custo
                vals = {
                    'wizcvi_consulta_venda_diaria_id': x.id,
                    'wizcvi_produto_id': produto_id,
                    'wizcvi_produto_embalagem_id': produto_embalagem_id,
                    'wizcvi_quantidade': quantidade,
                    'wizcvi_preco_custo': valor_custo,
                    'wizcvi_preco_venda': preco_venda,
                    'wizcvi_valor_agregado': valor_agregado
                }
                obj_wizard_consulta_venda_diaria_item.create(cr, uid, vals, context)
        return True

    def wizcvd_button_imprimir(self, cr, uid, ids, context=None):
        """
        O metodo realiza a chamada para uma classe de relatorio. Essa procesa os dados passados e
        gera a impressao de um arquivo pdf.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param ids: lista de ids do objeto
        @param context: dicionario com valores contextuais
        @return: dicionario com os dados da classe e de impressao do relatorio
        """
        context = context and context.copy() or {}
        return {'type': 'ir.actions.report.xml',
                'report_name': 'venda_diaria',
                'datas': {'id': ids[0]}
                }



class wizard_consulta_venda_diaria_item(osv.TransientModel):
    # prefixo: wizcvi_

    _name = 'wizard.consulta.venda.diaria.item'

    _columns = {
        'wizcvi_consulta_venda_diaria_id': fields.many2one('wizard.consulta.venda.diaria',
                                                           'Venda diaria'),
        'wizcvi_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'wizcvi_produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem'),
        'wizcvi_quantidade': fields.float('Quantidade', digits=(12, 3)),
        'wizcvi_preco_custo': fields.float('Preco custo', digits=(15, 2)),
        'wizcvi_preco_venda': fields.float('Preco venda', digits=(14, 5)),
        'wizcvi_valor_agregado': fields.float('Valor agregado', digits=(15, 2)),
    }
