#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
READ_LOAD = '_classic_write'


class wizard_ficha_tecnica_consulta(osv.TransientModel):

    _name = 'wizard.ficha.tecnica.consulta'
    _columns = {
        'wftc_ficha_tecnica_id': fields.many2one('pcp.ficha.tecnica.capa', string='Ficha Tecnica', ondelete='cascade'),
        'wftc_categoria_id': fields.many2one('estoque.categoria', string='Categoria', ondelete='cascade'),
        'wftc_item_ids': fields.one2many('wizard.ficha.tecnica.consulta.item', 'wftci_capa_id', 'Itens'),
        'wftc_imprimir': fields.boolean('Imprimir'),
    }
    _defaults = {
        'wftc_imprimir': False
    }
    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if not 'nao_limpa_itens' in context:
            vals['wftc_item_ids'] = ''
            vals['wftc_imprimir'] = False
            itens_ids = self.pool.get('wizard.ficha.tecnica.consulta.item').search(cr, uid, [], context=context)
            self.pool.get('wizard.ficha.tecnica.consulta.item').unlink(cr, uid, itens_ids)
        res = super(wizard_ficha_tecnica_consulta, self).write(cr, uid, ids,vals, context)
        return res

    def wtfc_onchange_ficha_tecnica_id(self, cr, uid, ids, ficha_tecnica_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if ficha_tecnica_id:
            res['wftc_categoria_id'] = False
        return {'value': res}

    def wtfc_onchange_categoria_id(self, cr, uid, ids, categoria_id, context=None):
        context = context and context.copy() or {}
        res = {}
        if categoria_id:
            res['wftc_ficha_tecnica_id'] = False
        return {'value': res}

    def wftc_get_categoria_filhas(self, cr, uid, categoria_id, context=None):
        context = context and context.copy() or {}
        args = [('parent_id', 'child_of', categoria_id)]
        categoria_ids = self.pool.get('estoque.categoria').search(cr, uid, args, context=context)
        return categoria_ids

    def wftc_get_custo_total(self, cr, uid, ficha_tecnica_item_ids=[],
                             ficha_tecnica_embalagem_ids=[], context=None):
        context = context and context.copy() or {}
        custo_total = 0
        if not any([ficha_tecnica_item_ids, ficha_tecnica_embalagem_ids]):
            return custo_total

        # Custo itens
        if isinstance(ficha_tecnica_item_ids, (int, long)):
            ficha_tecnica_item_ids = [ficha_tecnica_item_ids]

        ficha_item_dados = self.pool.get('pcp.ficha.tecnica.itens').read(
            cr, uid, ficha_tecnica_item_ids, ['pcpfti_custo_total'], context=context, load=READ_LOAD
        )
        custo_item = sum(map(lambda i: i.get('pcpfti_custo_total') or 0, ficha_item_dados))

        # Custo embalagens
        if isinstance(ficha_tecnica_embalagem_ids, (int, long)):
            ficha_tecnica_embalagem_ids = [ficha_tecnica_embalagem_ids]
        context.update({'pcpftc_total_embalagem_padrao': True})
        ficha_embalagem_dados = self.pool.get('pcp.ficha.tecnica.embalagem').read(
            cr, uid, ficha_tecnica_embalagem_ids, ['pcpfte_custo_total'], context=context, load=READ_LOAD
        )
        custo_embalagem = sum(map(lambda i: i.get('pcpfte_custo_total') or 0,
                                  ficha_embalagem_dados))

        # Custo total
        custo_total = custo_item + custo_embalagem
        return custo_total

    def wftc_button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        # OBJETOS
        obj_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
        obj_wizard_item = self.pool.get('wizard.ficha.tecnica.consulta.item')
        obj_estoque_produto = self.pool.get('estoque.produto')

        wizard_ftc_dados = self.read(
            cr, uid,
            ids,
            ['wftc_ficha_tecnica_id', 'wftc_categoria_id', 'wftc_item_ids'],
            context=context,
            load=READ_LOAD
        )
        for i in wizard_ftc_dados:

            wiz_id = i['id']
            wiz_ficha_tecnica_id = i.get('wftc_ficha_tecnica_id')
            wiz_categoria_id = i.get('wftc_categoria_id')
            wiz_item_ids = i.get('wftc_item_ids')


            if wiz_item_ids:
                obj_wizard_item.unlink(cr, uid, wiz_item_ids, context)

            args = []
            if wiz_categoria_id:
                categoria_ids = self.wftc_get_categoria_filhas(cr, uid, categoria_id=wiz_categoria_id, context=context)
                produto_ids = self.pool.get('estoque.produto').search(
                    cr, uid, [('categoria_id', 'in', categoria_ids)], context=context
                )
                if not produto_ids:
                    produto_ids = [0]
                args.append(('pcpftc_produto_id', 'in', produto_ids))

            if wiz_ficha_tecnica_id:
                args.append(('id', '=', wiz_ficha_tecnica_id))

            ficha_tecnica_capa_ids = obj_ficha_tecnica_capa.search(cr, uid, args=args, context=context)
            if not ficha_tecnica_capa_ids:
                raise osv.except_osv(_('Atencao'), _('Nenhum registro encontrado com os filtros informados!'))
            else:

                obj_ficha_tecnica_item = self.pool.get('pcp.ficha.tecnica.itens')
                obj_ficha_tecnica_embalagem = self.pool.get('pcp.ficha.tecnica.embalagem')
                obj_preco_venda = self.pool.get('estoque.preco.venda')

                funct_set_wizard_ficha_item = obj_wizard_item.set_ficha_tecnica_consulta_item

                fields_to_read = ['pcpftc_produto_id', 'pcpftc_grade_id', 'pcpftc_quantidade_peso']
                ficha_tecnica_capa_dados = obj_ficha_tecnica_capa.read(
                    cr, uid, ficha_tecnica_capa_ids, fields_to_read, context=context, load=READ_LOAD
                )

                ficha_produto_ids = filter(
                    None, map(lambda item: item.get('pcpftc_produto_id'), ficha_tecnica_capa_dados)
                )
                ficha_grade_ids = list(set(map(lambda item: item.get('pcpftc_grade_id'),
                                               ficha_tecnica_capa_dados)))
                dict_preco_venda = obj_preco_venda.get_preco_venda(
                    cr, uid, ficha_produto_ids, epc_grade_ids=ficha_grade_ids, context=context)

                for i in ficha_tecnica_capa_dados:

                    id = i['id']

                    # BUSCA O CUSTO TOTAL DAS FICHAS TECNICAS
                    ficha_tecnica_item_ids = obj_ficha_tecnica_item.search(
                        cr, uid, [('pcpfti_capa_id', '=', id)], context=context)
                    ficha_tecnica_embalagem_ids = obj_ficha_tecnica_embalagem.search(
                        cr, uid, [('pcpfte_capa_id', '=', id)], context=context)
                    custo_total = self.wftc_get_custo_total(
                        cr, uid, ficha_tecnica_item_ids, ficha_tecnica_embalagem_ids,
                        context=context)

                    produto_id = i.get('pcpftc_produto_id')
                    grade_id = i.get('pcpftc_grade_id')
                    quantidade = i.get('pcpftc_quantidade_peso')

                    custo_unitario = custo_total / quantidade

                    dados_preco_venda = dict_preco_venda.get(produto_id) or {}
                    preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                        'preco_venda')

                    dados_produto_margem = obj_estoque_produto.estpro_get_margens(cr, uid, produto_id)
                    if preco_venda <= 0:
                        margem_atual=0
                    else:
                        margem_atual = ((preco_venda - custo_unitario) / preco_venda) * 100
                    margem_ideal = dados_produto_margem.get(produto_id).get('margem_ideal') or 0
                    novo_preco = custo_unitario / ((100 - margem_ideal) / 100)

                    funct_set_wizard_ficha_item(
                        cr, uid,
                        capa_id=wiz_id,
                        produto_id=produto_id,
                        grade_id=grade_id,
                        custo_unitario=custo_unitario,
                        preco_venda=preco_venda,
                        preco_sugerido=novo_preco,
                        margem_atual=margem_atual,
                        margem_sugerida=margem_ideal,
                        context=context
                    )
        context['nao_limpa_itens']= True
        self.write(cr, uid, ids, {'wftc_imprimir':True}, context)
        return True

    def wftc_button_imprimir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_ficha_tec = self.browse(cr, uid, ids)
        if brw_ficha_tec and isinstance(brw_ficha_tec,list):
            brw_ficha_tec=brw_ficha_tec[0]
        lista_brw_item = brw_ficha_tec.wftc_item_ids or False
        if not lista_brw_item:
            raise osv.except_osv(_('Atencao'), _('Nenhum registro encontrado com os filtros informados!'))
        lista_itens = []
        for item in lista_brw_item:
            dados = {'produto': self.pool.get('estoque.produto').name_get(cr, uid, [item.wftci_produto_id._id])[0][1],
                     'grade': str(item.wftci_grade_id.grade),
                     'custo_unitario': item.wftci_custo_unitario,
                     'margem_atual': item.wftci_margem_atual,
                     'preco_atual': item.wftci_preco_venda,
                     'margem_sugerida': item.wftci_margem_sugerida,
                     'preco_sugerido': item.wftci_preco_sugerido,
                    }
            lista_itens.append(dados)
        return {
                'type': 'ir.actions.report.xml',
                'report_name': 'listagem_ficha_tecnica',
                'datas': {'lista_itens': lista_itens}
                }
        self.pool.get('estoque.produto.grade')._estpgr_get_name(crm, uid, 3621)


class wizard_ficha_tecnica_consulta_item(osv.TransientModel):

    _name = 'wizard.ficha.tecnica.consulta.item'
    _columns = {
        'wftci_capa_id': fields.many2one('wizard.ficha.tecnica.consulta', 'Capa ID', required=True, ondelete='cascade'),
        'wftci_produto_id': fields.many2one('estoque.produto', 'Produto', required=True, ondelete='cascade'),
        'wftci_grade_id': fields.many2one('estoque.produto.grade', 'Grade', required=True, ondelete='cascade'),
        'wftci_custo_unitario': fields.float('Custo Unitario', digits=(12, 3), required=True),
        'wftci_preco_venda': fields.float('Preco de Venda Atual', digits=(12, 2), required=True),
        'wftci_preco_sugerido': fields.float('Preco de Venda Sugerido', digits=(12, 2), required=True),
        'wftci_margem_atual': fields.float('Margem Atual(%)', digits=(12, 2), required=True),
        'wftci_margem_sugerida': fields.float('Margem Sugerida(%)', digits=(12, 2), required=True),
    }

    def set_ficha_tecnica_consulta_item(self, cr, uid, capa_id, produto_id, grade_id, custo_unitario, preco_venda,
                                        preco_sugerido, margem_atual, margem_sugerida, context=None):
        context = context and context.copy() or {}
        vals = {
            'wftci_capa_id': capa_id,
            'wftci_produto_id': produto_id,
            'wftci_grade_id': grade_id,
            'wftci_custo_unitario': custo_unitario,
            'wftci_preco_venda': preco_venda,
            'wftci_preco_sugerido': preco_sugerido,
            'wftci_margem_atual': margem_atual,
            'wftci_margem_sugerida': margem_sugerida,
        }
        res = super(wizard_ficha_tecnica_consulta_item, self).create(cr, uid, vals, context)
        return res
