# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from openerp.addons.pcp.model.funcoes import _get_categoria_produto


# noinspection PyAbstractClass,PyPep8Naming
class wiz_pcp_gerar_ordem_producao(osv.TransientModel):
    # prefixo: wizgop_

    _name = 'wiz.pcp.gerar.ordem.producao'

    _columns = {
        'wizgop_data': fields.date('Data planejamento', required=True),
        'wizgop_gerar_ordem_producao_ids': fields.one2many('wiz.pcp.gerar.ordem.producao.item',
                                                           'wizgoi_capa_id', 'Itens'),
        'wizgop_ordem_producao_gerada': fields.boolean('Ordens de producao geradas'),
        'wizgop_categoria_id': fields.many2one('estoque.categoria', 'Categoria', select=True),
    }

    _defaults = {
        'wizgop_ordem_producao_gerada': lambda *a: False,
    }

    def wizgop_button_pesquisar(self, cr, uid, ids, context=None):
        """
        O metodo insere na tabela wiz_pcp_gerar_ordem_producao_item (TransientModel) todos os
        planejamentos (pcp_planejamento_producao) para a data selecionada.

        :param cr: cursor do banco de dados
        :param uid: id do usuario responsavel pela transacao
        :param ids: lista de id da tabela do objeto
        :param context: dicionario com valores contextuais
        :return: True
        """
        context = context and context.copy() or {}
        cr.execute('DELETE FROM wiz_pcp_gerar_ordem_producao_item')
        obj_wiz_pcp_gerar_ordem_producao_item = self.pool.get('wiz.pcp.gerar.ordem.producao.item')
        brw_gerar_ordem_producao = self.browse(cr, uid, ids, context)
        for x in brw_gerar_ordem_producao:
            data = x.wizgop_data
            categoria_id = x.wizgop_categoria_id
            sql = """
                SELECT id                       AS wizgoi_planejamento_producao_id,
                       pljprd_ficha_tecnica_id  AS wizgoi_ficha_tecnica_id,
                       pljprd_data              AS wizgoi_data,
                       pljprd_quantidade        AS wizgoi_quantidade,
                       pljprd_ordem_producao_id AS wizgoi_ordem_producao_id,
                       pljprd_categoria         AS wizgoi_categoria, (
                       CASE
                              WHEN pljprd_ordem_producao_id > 0 THEN TRUE
                              ELSE FALSE
                       END) AS wizgoi_planejamento_selecionado
                FROM   pcp_planejamento_producao
                WHERE  pljprd_data = '%s'
                       """ % data
            if categoria_id:
                sql += "AND pljprd_categoria = '%s' " % categoria_id.name
            cr.execute(sql)
            dados = cr.dictfetchall()
            for y in dados:
                y.update({'wizgoi_capa_id': x.id})
                obj_wiz_pcp_gerar_ordem_producao_item.create(cr, uid, y, context=context)
        return True

    def wizgop_button_confirmar(self, cr, uid, ids, context=None):
        """
        O metodo cria ordens de producao (pcp_ordem_producao) a partir dos dados dos planejamentos
        selecionados.

        :param cr: cursor do banco de dados
        :param uid: id do usuario responsavel pela transacao
        :param ids: lista de id da tabela do objeto
        :param context: dicionario com valores contextuais
        :return: True
        """

        context = context and context.copy() or {}
        obj_pcp_ordem_producao = self.pool.get('pcp.ordem.producao')
        obj_pcp_planejamento_producao = self.pool.get('pcp.planejamento.producao')
        brw_gerar_ordem_producao = self.browse(cr, uid, ids, context=context)
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        res_users_dados = self.pool.get('res.users').read(cr, uid, [uid], ['pessoa_id'], context=context)

        for x in res_users_dados:
            pessoa_id = x.get('pessoa_id')

        if isinstance(pessoa_id, tuple):
            pessoa_id = pessoa_id[0]

        funcionario_id = obj_rh_funcionario.search(cr, uid, [('pessoa', '=', pessoa_id)], context=context)

        if not funcionario_id:
            raise osv.except_osv(_('Atencao'), _('Usuario nao cadastrado como funcionario'))

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'pcp.ordem.producao', context=context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_pcp_incluir_op_aberta'])
        incluir_op_aberta = estoque_parametro_dados.get('estpar_pcp_incluir_op_aberta')

        for x in brw_gerar_ordem_producao:
            brw_gerar_ordem_producao_itens = x.wizgop_gerar_ordem_producao_ids
            brw_gerar_ordem_producao_itens = filter(lambda i: not i.wizgoi_ordem_producao_id,
                                                    brw_gerar_ordem_producao_itens)

            if not brw_gerar_ordem_producao_itens:
                raise osv.except_osv(_('Atencao'), _('Planejamento(s) ja possui(em) ordem de '
                                                     'producao'))

            brw_gerar_ordem_producao_itens = filter(
                lambda i: i.wizgoi_planejamento_selecionado and not i.wizgoi_ordem_producao_id,
                brw_gerar_ordem_producao_itens)

            if not brw_gerar_ordem_producao_itens:
                raise osv.except_osv(_('Atencao'), _('Selecione algum planejamento'))

            brw_gerar_ordem_producao_itens = filter(lambda i: not i.wizgoi_ordem_producao_id,
                                                    brw_gerar_ordem_producao_itens)

            if not brw_gerar_ordem_producao_itens:
                raise osv.except_osv(_('Atencao'), _('Planejamento(s) ja possui(em) ordem de '
                                                     'producao'))
            ordem_producao_id = False

            for y in brw_gerar_ordem_producao_itens:
                vals = {
                    'company_id': company_id,
                    'state': 'E',
                    'pcpopr_responsavel_id': funcionario_id[0],
                    'pcpopr_ficha_tecnica_capa_id': y.wizgoi_ficha_tecnica_id.id,
                    'name': y.wizgoi_ficha_tecnica_id.pcpftc_produto_id.name,
                    'pcpopr_quantidade': y.wizgoi_quantidade,
                    'pcpopr_gerado_automatico': True,
                }
                ordem_producao_id = obj_pcp_ordem_producao.create(cr, uid, vals, context)

                if ordem_producao_id:
                    planejamento_producao_id = y.wizgoi_planejamento_producao_id.id
                    val = {'pljprd_ordem_producao_id': ordem_producao_id}
                    obj_pcp_planejamento_producao.write(
                        cr, uid, planejamento_producao_id, val, context)

                    if incluir_op_aberta:
                        obj_pcp_ordem_producao.pcpopr_button_abrir(
                            cr, uid, [ordem_producao_id], context)

            if ordem_producao_id:
                self.write(cr, uid, x.id, {'wizgop_ordem_producao_gerada': True}, context)

        return True

    def wizgop_button_selecionar_todos(self, cr, uid, ids, context=None):
        """
        Seleciona todos os planejamentos de producao.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario responsavel pela transacao
        :param ids: lista com ID da tabela do objeto
        :param context: dicionario com valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se todos os planejamentos foram marcados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects ________________________________
        obj_ordem_producao_item = self.pool.get('wiz.pcp.gerar.ordem.producao.item')

        context = context and context.copy() or {}
        brw_gerar_ordem_producao = self.browse(
            cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        planejamento_ids = map(
            lambda m: m.id, brw_gerar_ordem_producao.wizgop_gerar_ordem_producao_ids)
        result = obj_ordem_producao_item.wizgoi_selecionar_planejamento(
            cr, uid, planejamento_ids, context)

        return result


# noinspection PyAbstractClass,PyAbstractClass
class wiz_pcp_gerar_ordem_producao_item(osv.TransientModel):
    # prefixo: wizgoi_

    _name = 'wiz.pcp.gerar.ordem.producao.item'

    _columns = {
        'wizgoi_capa_id': fields.many2one('wiz.pcp.gerar.ordem.producao', 'Capa'),
        'wizgoi_planejamento_producao_id': fields.many2one('pcp.planejamento.producao',
                                                           'Planejamento producao'),
        'wizgoi_ficha_tecnica_id': fields.many2one('pcp.ficha.tecnica.capa', 'Ficha tecnica'),
        'wizgoi_data': fields.date('Data'),
        'wizgoi_quantidade': fields.float('Quantidade', digits=(12, 3)),
        'wizgoi_ordem_producao_id': fields.many2one('pcp.ordem.producao', 'Ordem de producao'),
        'wizgoi_planejamento_selecionado': fields.boolean('Selecionado'),
        'wizgoi_categoria': fields.function(_get_categoria_produto, method=True,
                                            string='Categoria', type='char', store=True),
    }

    def wizgoi_selecionar_planejamento(self, cr, uid, ids, context=None):
        """
        Marca os planejamentos de producao como selecionado.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario responsavel pela transacao
        :param ids: lista com ID da tabela do objeto
        :param context: dicionario com valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se todos os planejamentos foram marcados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        return self.write(cr, uid, ids, {'wizgoi_planejamento_selecionado': True}, context=context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
