#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import date

READ_LOAD = '_classic_write'


class wizard_ordem_producao_consulta(osv.TransientModel):

    _name = 'wizard.ordem.producao.consulta'
    _columns = {
        # ___________________________ Date ___________________________________
        'wopc_data_inicial': fields.date('Data Inicial', required=True),
        'wopc_data_final': fields.date('Data Final', required=True),

        # ___________________________ Many2One _______________________________
        'wopc_ficha_tecnica_capa_id': fields.many2one("pcp.ficha.tecnica.capa", 'Ficha tecnica'),

        # ___________________________ One2Many _______________________________
        'wopc_item_ids': fields.one2many('wizard.ordem.producao.consulta.item', 'wopci_capa_id',
                                         'Ordens de Producao'),

        # ___________________________ Boolean ________________________________
        'wizopc_habilita_impressao': fields.boolean('Habilitar impressao da consulta')
    }

    _defaults = {
        'wopc_data_inicial': date.today().strftime('%Y-%m-%d'),
        'wopc_data_final': date.today().strftime('%Y-%m-%d'),
        'wizopc_habilita_impressao': False
    }

    def wizopc_button_imprimir_relatorio(self, cr, uid, ids, context=None):
        """
        Realiza a impressao do relatorio de consulta da ordem de producao. Relatorio comparativo.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} mensagem de aviso se nao houver ficha tecnica.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = {'warning': {'title': _('Aviso'),
                              'message': _('Nenhuma ficha tecnica para imprimir.')}}
        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_ordem_producao_consulta_item = self.pool.get('wizard.ordem.producao.consulta.item')

        campos = ['wopc_ficha_tecnica_capa_id']
        dados_relatorio = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_relatorio and isinstance(dados_relatorio, list):
            dados_relatorio = dados_relatorio[0]

        if dados_relatorio and isinstance(dados_relatorio, dict):
            # Se nao for informada nenhuma ficha tecnica, todas as fichas tecnicas deverao ser
            # consideradas
            args = []  # Sem argumento. Todas as fichas tecnicas de todas as categorias

            # Filtro de ficha tecnica
            ficha_tecnica_id = dados_relatorio.get('wopc_ficha_tecnica_capa_id', False)
            if ficha_tecnica_id:
                args.append(('wopci_capa_id', '=', dados_relatorio.get('id', False)))
                args.append(('wopci_ficha_tecnica_id', '=', ficha_tecnica_id))

            if obj_ordem_producao_consulta_item.search(cr, uid, args, context=context):
                # Impressao do relatorio
                result = {'type': 'ir.actions.report.xml', 'report_name': 'ordem_producao_consulta'}
            else:
                raise osv.except_osv(_('Atencao!'), _('Nenhuma ficha tecnica para imprimir.'))

            # Desabilita o botao de impressao
            self.write(cr, uid, ids, {'wizopc_habilita_impressao': False}, context)

        return result

    def wopc_button_pesquisar(self, cr, uid, ids, context=None):
        """
        Realiza a pesquisa das ordens de producao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {boolean} verdadeiro.
        """

        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        fields_to_read = ['wopc_data_inicial', 'wopc_data_final', 'wopc_item_ids',
                          'wopc_ficha_tecnica_capa_id']
        wizard_capa_dados = self.read(cr, uid, ids, fields_to_read, context=context, load=READ_LOAD)

        obj_wiz_item = self.pool.get('wizard.ordem.producao.consulta.item')

        for i in wizard_capa_dados:
            wiz_capa_id = i['id']
            data_inicial = i.get('wopc_data_inicial')
            data_final = i.get('wopc_data_final')
            ficha_tecnica_capa_id = i.get('wopc_ficha_tecnica_capa_id')
            item_ids = i.get('wopc_item_ids')

            if item_ids:
                obj_wiz_item.unlink(cr, uid, item_ids, context)

            dados = obj_wiz_item.wopci_get_dados(cr, data_inicial, data_final,
                                                 ficha_tecnica_capa_id)
            habilita_impressao = True if dados else False  # Habilita ou nao a impressao

            for ordem in dados:
                ficha_tecnica_id = ordem.get('ficha_tecnica_id')
                qtde_a_produzir = ordem.get('qtde_a_produzir') or 0
                qtde_produzida = ordem.get('qtde_produzida') or 0
                diferenca = ordem.get('diferenca') or 0
                vendas = ordem.get('vendas') or 0
                perdas_lancadas = ordem.get('perdas_lancadas') or 0

                vals = {
                    'wopci_capa_id': wiz_capa_id,
                    'wopci_ficha_tecnica_id': ficha_tecnica_id,
                    'wopci_qtde_a_produzir': qtde_a_produzir,
                    'wopci_qtde_produzida': qtde_produzida,
                    'wopci_producao_diferenca': diferenca,
                    'wopci_perdas_lancadas': perdas_lancadas,
                    'wopci_vendas': vendas,
                }

                obj_wiz_item.create(cr, uid, vals, context)

            # Habilita a impressao da consulta
            self.write(cr, uid, ids, {'wizopc_habilita_impressao': habilita_impressao}, context)

        return True


class wizard_ordem_producao_item(osv.TransientModel):
    def wopci_get_valores_calculados(self, cr, uid, ids, field_name, args, context=None):
        """
        Calcula o total de producao e o total percentual de producao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param field_name: {char} nome do atributo da classe (nome do campo),
        @param args: {list} lista de argumentos,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} valores do total de producao: total, percentual e diferenca (percentual).
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        values = {'wopci_total': 0, 'wopci_total_percentual': 0,
                  'wopci_producao_diferenca_percentual': 0}

        result = {}.fromkeys(ids, values)

        fields_to_read = ['wopci_qtde_produzida', 'wopci_qtde_a_produzir',
                          'wopci_producao_diferenca', 'wopci_perdas_lancadas', 'wopci_vendas']

        for item in self.read(cr, uid, ids, fields_to_read, context=context):
            item_id = item['id']
            vendas = float(item.get('wopci_vendas', 0))
            diferenca = float(item.get('wopci_producao_diferenca', 0))
            qtde_a_produzir = float(item.get('wopci_qtde_a_produzir', 0))
            qtde_produzida = float(item.get('wopci_qtde_produzida', 0))
            perdas_lancadas = float(item.get('wopci_perdas_lancadas', 0))

            if qtde_a_produzir:  # Evita divisao por zero ;)
                percentual = diferenca * 100 / qtde_a_produzir
                result[item_id]['wopci_producao_diferenca_percentual'] = percentual

            if qtde_produzida:  # Evita divisao por zero ;)
                total = qtde_produzida - perdas_lancadas - vendas
                percentual_total = total * 100 / qtde_produzida
                result[item_id]['wopci_total'] = total
                result[item_id]['wopci_total_percentual'] = percentual_total

        return result

    _name = 'wizard.ordem.producao.consulta.item'
    _columns = {
        # ___________________________ Integer ________________________________
        'wopci_qtde_a_produzir': fields.integer('Qtde a Produzir'),
        'wopci_qtde_produzida': fields.integer('Qtde Produzida'),
        'wopci_producao_diferenca': fields.integer('Diferenca Producao'),
        'wopci_perdas_lancadas': fields.integer('Perdas Lancadas'),
        'wopci_vendas': fields.integer('Vendas'),

        # ___________________________ Many2One _______________________________
        'wopci_capa_id': fields.many2one('wizard.ordem.producao.consulta', 'Capa ID'),
        'wopci_ficha_tecnica_id': fields.many2one('pcp.ficha.tecnica.capa', 'Ficha Tecnica'),

        # ___________________________ Function _______________________________
        'wopci_producao_diferenca_percentual': fields.function(wopci_get_valores_calculados,
                                                               string="Diferenca (%)", method=True,
                                                               digits=(5, 2),
                                                               multi='valores_calculados'),
        'wopci_total': fields.function(wopci_get_valores_calculados, string="Total", method=True,
                                       digits=(12, 2), multi='valores_calculados'),
        'wopci_total_percentual': fields.function(wopci_get_valores_calculados, string="Total (%)",
                                                  method=True, digits=(12, 2),
                                                  multi='valores_calculados')
    }

    def wopci_get_dados(self, cr, data_inicial, data_final, ficha_tecnica_capa_id):
        tupla_dados = (data_inicial, data_final, data_inicial, data_final, data_inicial, data_final)
        sql = """
            SELECT
               ordem.pcpopr_ficha_tecnica_capa_id                                               AS ficha_tecnica_id,
               ficha.pcpftc_produto_id                                                          AS produto_id,
               SUM(ordem.pcpopr_quantidade_a_produzir)                                          AS qtde_a_produzir,
               SUM(ordem.pcpopr_quantidade_produzida)                                           AS qtde_produzida,
               SUM(ordem.pcpopr_quantidade_a_produzir) - SUM(ordem.pcpopr_quantidade_produzida) AS diferenca,
               (
                    SELECT
                        SUM(mov_item.quantidade)
                    FROM
                        estoque_movimento_itens mov_item
                        INNER JOIN fiscal_cfop cfop ON cfop.id = mov_item.cfop_id
                        INNER JOIN estoque_movimento_capa mov_capa  ON mov_capa.id = mov_item.capa_id
                        LEFT JOIN fiscal_nf_capa nf ON nf.id = mov_item.nf_id
                        LEFT JOIN ecf_cupons_capa cupom ON cupom.id = mov_item.cupom_id
                    WHERE
                        mov_item.produto_id = ficha.pcpftc_produto_id
                        AND cfop.tipo = 'S'
                        AND cfop.movimento_tipo = 'C'
                        AND mov_capa.data >= '%s'
                        AND mov_capa.data <= '%s'
                        AND ((nf.state = '3' and mov_item.cupom_id is null) or cupom.state = 'V')
               ) AS vendas,
               (
                    SELECT     SUM(mov_item2.quantidade)
                    FROM       estoque_movimento_itens mov_item2
                    INNER JOIN estoque_movimento_capa mov_capa2 ON mov_capa2.id = mov_item2.capa_id
                    LEFT JOIN estoque_natureza_operacao eno ON eno.id = mov_capa2.estmvc_operacao_id
                    LEFT JOIN estoque_movimento_origem emo ON emo.id = eno.estnto_movimento_origem_id

                    WHERE
                        emo.estmog_codigo = 'E'
                        AND mov_item2.produto_id = ficha.pcpftc_produto_id
                        AND mov_capa2.data >= '%s' AND mov_capa2.data <= '%s'
               ) AS perdas_lancadas
            FROM
               pcp_ordem_producao ordem
            INNER JOIN
               pcp_ficha_tecnica_capa ficha
              ON ficha.id = ordem.pcpopr_ficha_tecnica_capa_id
            WHERE
               ordem.pcpopr_inicio >= '%s 00:00:00'
               and ordem.pcpopr_termino <= '%s 23:59:59'
        """ % tupla_dados
        if ficha_tecnica_capa_id:
            sql += """
                AND ordem.pcpopr_ficha_tecnica_capa_id = %s
            """ % ficha_tecnica_capa_id
        sql += """
            GROUP BY
               ordem.pcpopr_ficha_tecnica_capa_id,
               ficha.pcpftc_produto_id
        """
        cr.execute(sql)
        res = cr.dictfetchall()

        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
