# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
import time
from openerp.tools.translate import _
from openerp.addons.pcp.model.funcoes import _get_categoria_produto


class wiz_pcp_listar_material(osv.TransientModel):
    # prefixo: wizplm_

    _name = 'wiz.pcp.listar.material'

    _columns = {
        'wizplm_data_inicial': fields.date('Data inicial'),
        'wizplm_data_final': fields.date('Data final'),
        'wizplm_solicitacao_compra_gerada': fields.boolean('Solicitacao de compra gerada'),
        'wizplm_item_ids': fields.one2many('wiz.pcp.listar.material.item', 'wizlmi_capa_id',
                                           'Itens'),
        'wizplm_categoria_id': fields.many2one('estoque.categoria', 'Categoria', select=True),
    }

    _defaults = {
        'wizplm_data_inicial': lambda *a: time.strftime('%Y-%m-%d'),
        'wizplm_data_final': lambda *a: time.strftime('%Y-%m-%d')
    }

    def wizplm_button_pesquisar(self, cr, uid, ids, context=None):
        """
        O metodo busca todos os itens faltantes (itens cuja quantidade planejada para producao
        ultrapassa o saldo disponivel) das fichas tecnicas dos planejamentos de producao, de acordo
        com o periodo informado.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param ids: lista de ids da classe do metodo
        @param context: dicionario com valores contextuais
        @return: True
        """
        context = context and context.copy() or {}
        estoque_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_local_almoxarifado_pcp_id'])
        local_id = estoque_parametro.get('estpar_local_almoxarifado_pcp_id')
        if not local_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre o almoxarifado do pcp nos parametros do '
                                                 'estoque'))
        cr.execute("""DELETE FROM wiz_pcp_listar_material_item""")
        obj_pcp_util = self.pool.get('pcp.util')
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        obj_wiz_pcp_listar_material_item = self.pool.get('wiz.pcp.listar.material.item')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                                    'estoque.local.locacao', context=context)
        brw_listar_material = self.browse(cr, uid, ids, context)
        for x in brw_listar_material:
            data_inicial = x.wizplm_data_inicial
            data_final = x.wizplm_data_final
            categoria = x.wizplm_categoria_id
            dados = obj_pcp_util.pcputl_verificar_materia_prima(cr, uid, data_inicial, data_final,
                                                                context)
            categorias_filhas_ids = []
            if categoria:
                categorias_filhas_ids = [i.id for i in categoria.child_ids]
            for dado in dados:
                if not dado.get('quantidade_faltante'):
                    continue
                produto_embalagem_id = dado.get('produto_embalagem_id')
                if not produto_embalagem_id:
                    local_locacao_ids = obj_estoque_local_locacao.search(
                        cr, uid, [('company_id', '=', company_id),
                                  ('produto_id', '=', dado.get('produto_id')),
                                  ('grade_id', '=', dado.get('grade_id')),
                                  ('local_id', '=', local_id)], context=context)
                    if local_locacao_ids:
                        produto_embalagem_id = obj_estoque_local_locacao.browse(
                            cr, uid, local_locacao_ids[0]).estllc_produto_embalagem_id.id
                vals = {
                    'wizlmi_capa_id': x.id,
                    'wizlmi_produto_id': dado.get('produto_id'),
                    'wizlmi_grade_id': dado.get('grade_id'),
                    'wizlmi_produto_embalagem_id': produto_embalagem_id,
                    'wizlmi_qtd_existente': dado.get('quantidade_existente'),
                    'wizlmi_qtd_faltante': dado.get('quantidade_faltante')
                }
                if not categoria:
                    obj_wiz_pcp_listar_material_item.create(cr, uid, vals, context=context)
                else:
                    produto_id = dado.get('produto_id')
                    brw_estoque_produto = obj_estoque_produto.browse(cr, uid, produto_id)
                    if brw_estoque_produto:
                        if brw_estoque_produto.categoria_id:
                            if brw_estoque_produto.categoria_id.id in categorias_filhas_ids \
                                    or brw_estoque_produto.categoria_id.id == categoria.id:
                                obj_wiz_pcp_listar_material_item.create(cr, uid, vals,
                                                                        context=context)
        return True

    def wizplm_button_gerar_solicitacao(self, cr, uid, ids, context=None):
        """
        O metodo cria um registro na tabela compras_solicitacao_capa para os itens que possui
        quantidade faltante.
        @param cr: cursor do banco de dados
        @param uid: id do usuario responsavel pela transacao
        @param ids: lista de ids da classe do metodo
        @param context: dicionario com valores contextuais
        @return: True
        """
        context = context and context.copy() or {}
        obj_compra_solicitacao_capa = self.pool.get('compras.solicitacao.capa')
        brw_listar_material = self.browse(cr, uid, ids, context)
        fields_solicitacao = ['state', 'data_hora_solicitacao', 'company_id']
        vals_solicitacao = obj_compra_solicitacao_capa.default_get(cr, uid, fields_solicitacao,
                                                                   context)
        for x in brw_listar_material:
            brw_listar_material_item = x.wizplm_item_ids
            list_solicitacao_item = []
            for item in brw_listar_material_item:
                if not item.wizlmi_produto_embalagem_id:
                    continue
                vals_solicitacao_item = {
                    'produto_id': item.wizlmi_produto_id.id,
                    'grade_id': item.wizlmi_grade_id.id,
                    'comsit_produto_embalagem_id': item.wizlmi_produto_embalagem_id.id,
                    'quantidade': item.wizlmi_qtd_faltante,
                }
                list_solicitacao_item.append((0, 0, vals_solicitacao_item))
            vals_solicitacao['solicitacao_item_ids'] = list_solicitacao_item
            solicitacao_compra_id = obj_compra_solicitacao_capa.create(cr, uid, vals_solicitacao,
                                                                       context)
            if solicitacao_compra_id:
                self.write(cr, uid, x.id, {'wizplm_solicitacao_compra_gerada': True}, context)
        return True

    def wizplm_button_imprimir(self, cr, uid, ids, context=None):
        """
        Metodo responsavel por chamar a classe do relatorio que ira imprimir um arquivo pdf com
        as informacoes dos materiais faltantes
        @return: Dicionario com acao para a classe do relatorio, os dados referentes e o nome.
        """
        context = context and context.copy() or {}
        datas = {
            'id': ids[0],
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_material_faltante',
            'datas': datas,
        }



class wiz_pcp_listar_material_item(osv.TransientModel):
    # prefixo: wizlmi_

    _name = 'wiz.pcp.listar.material.item'

    _columns = {
        'wizlmi_capa_id': fields.many2one('wiz.pcp.listar.material', 'Capa'),
        'wizlmi_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'wizlmi_grade_id': fields.many2one('estoque.produto.grade', 'Grade'),
        'wizlmi_produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem'),
        'wizlmi_qtd_existente': fields.float('Quantidade existente', digits=(14, 2)),
        'wizlmi_qtd_faltante': fields.float('Quantidade faltante', digits=(14, 2)),
        'wizlmi_categoria': fields.function(_get_categoria_produto, method=True,
                                            string='Categoria', type='char', store=True),
    }
