#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields


class wizard_pcp_transferencia_producao(osv.TransientModel):
    """
    prefixo: wptp
    """

    _name = 'wizard.pcp.transferencia.producao'

    _columns = {
        'wptp_origem_produto_id': fields.many2one('pcp.conversao.itens', 'Produto de origem',
                                                  required=True, ondelete='restrict'),

        'wptp_destino_produto_id': fields.many2one('pcp.conversao.capa', 'Produto de destino',
                                                   required=True, ondelete='restrict'),

        'wptp_quantidade': fields.float('Quantidade', digits=(8, 3), required=True),

        'wptp_local_saida_id': fields.many2one('estoque.local', 'Local saida', select=True,
                                               required=True, ondelete="restrict"),
        'wptp_mensagem': fields.text('Mensagem', readonly=True)
    }

    _defaults = {
        'wptp_mensagem': None
    }

    def wptp_gerar_requisicao(self, cr, uid, transferencia_id, context=None):
        """
        Método responsável por gerar requisição em estoqueressuprimento.

        :param cr:
        :param uid:
        :param transferencia_id: ID do formulario de tranferencia de producao
        :param context:
        :return: o ID da requisicao que foi criada.
        :rtype: long
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_requisicao_capa = self.pool.get('estoqueressuprimento.requisicao.capa')
        obj_local_locacao = self.pool.get('estoque.local.locacao')

        brw_transferencia = self.browse(cr, uid, transferencia_id, context=context)
        origem_produto_id = brw_transferencia.wptp_origem_produto_id.pcpcvi_produto_id.id
        origem_grade_id = brw_transferencia.wptp_origem_produto_id.pcpcvi_grade_id.id
        company_id = brw_transferencia.wptp_origem_produto_id.pcpcvi_produto_id.company_id.id
        local_entrada_id = obj_adm_utils.admutl_get_parametros(
            cr,uid,'estoque','estpar_local_almoxarifado_pcp_id')['estpar_local_almoxarifado_pcp_id']
        local_saida_id = brw_transferencia.wptp_local_saida_id.id
        locacao_id = obj_local_locacao.search(
            cr, uid, [('local_id', '=', local_saida_id), ('produto_id', '=', origem_produto_id),
                      ('grade_id', '=', origem_grade_id), ('company_id', '=', company_id)],
            context=context)
        produto_origem_embalagem_id = obj_local_locacao.read(
            cr, uid, locacao_id, ['estllc_produto_embalagem_id'], context=context)

        if produto_origem_embalagem_id and isinstance(produto_origem_embalagem_id, list):
            produto_origem_embalagem_id = produto_origem_embalagem_id[0]

        if produto_origem_embalagem_id and isinstance(produto_origem_embalagem_id, dict):
            produto_origem_embalagem_id = \
                produto_origem_embalagem_id.get('estllc_produto_embalagem_id')

        if produto_origem_embalagem_id and isinstance(produto_origem_embalagem_id, tuple):
            produto_origem_embalagem_id = produto_origem_embalagem_id[0]

        fator_conversao = brw_transferencia.wptp_origem_produto_id.pcpcvi_fator_conversao
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                        tabela='wizard.pcp.transferencia.producao', context=context)
        qtd_requisitada = brw_transferencia.wptp_quantidade
        saldo = self.pool.get('estoqueressuprimento.requisicao.itens').get_saldo_local(cr, uid, [],
                       company_id, local_saida_id, origem_produto_id, origem_grade_id,
                       produto_origem_embalagem_id, quantidade_requisitada=qtd_requisitada,
                                                                       context=context).get('saldo')
        requisicao_itens = dict(
            produto_id=origem_produto_id,
            grade_id=origem_grade_id,
            embalagem_id=produto_origem_embalagem_id,
            etrrqi_destino_produto_id=brw_transferencia.wptp_destino_produto_id.pcpcvc_produto_id.id,
            etrrqi_destino_grade_id=brw_transferencia.wptp_destino_produto_id.pcpcvc_grade_id.id,
            quantidade_requisitada=qtd_requisitada,
            etrrqi_destino_quantidade=qtd_requisitada * fator_conversao,
            saldo=saldo)
        requisicao_capa = dict(tipo='QR',
                               local_entrada_id=local_entrada_id,
                               local_saida_id=local_saida_id,
                               requisicao_itens_ids=[(0, 0, requisicao_itens)])

        return obj_requisicao_capa.create(cr, uid, requisicao_capa, context=context)

    def wptp_button_confirmar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        requisicao_dados = self.pool.get('estoqueressuprimento.requisicao.capa').read(
            cr, uid, self.wptp_gerar_requisicao(cr, uid, ids[0], context), ['numero'], context)

        if requisicao_dados:
            mensagem = 'Transferencia efetuada com sucesso. Requisicao %s criada.' % \
                       requisicao_dados.get('numero')
        else:
            mensagem = 'Nao foi possivel efetuar a transferencia.'

        self.write(cr, uid, ids[0], dict(wptp_mensagem=mensagem), context)
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'pcp', 'wizard_pcp_trans_producao_form')

        return dict(name='Transferencia para producao', view_type='form', view_mode='form',
                    res_model=self._name, view_id=[reference and reference[1] or False],
                    res_id=ids[0], type='ir.actions.act_window')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
