# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
import time
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from datetime import date, datetime, timedelta


class wiz_pcp_planejamento_producao_automatico(osv.TransientModel):
    # prefixo: wizppa_
    from funcoes import _wizppi_confirmar
    _name = 'wiz.pcp.planejamento.producao.automatico'
    _columns = {
        'id': fields.integer('Id'),
        'wizppa_data_inicio': fields.date('Data inicio', required=True),
        'wizppa_data_fim': fields.date('Data fim', required=True),
        'wizppa_sobrepoe_planejamento': fields.boolean('Sobrepor planejamento'),
        'wizppa_categoria_id': fields.many2one('estoque.categoria', 'Categoria', select=True),
        'wizppa_planejamento_producao_ids': fields.one2many('wiz.pcp.planejamento.producao.automatico.itens', 'wizppi_capa_id','Ficha tecnica'),
        'wizppa_confirmar': fields.function(_wizppi_confirmar, method=True, string='Confirmar', type='char', store=False, )
        }

    def wizppa_button_pesquisar(self, cr, uid, ids, context=None):
        """
        O metodo busca todos os resgistros (ids) da tabela pcp_planejamento_producao de acordo com
        o periodo informado.
        """
        context = context and context.copy() or {}
        dias_semana = {1:[], 2:[], 3:[], 4:[], 5:[], 6:[], 7:[]} # 1=segunda, 2=terca, ...,7=domingo
        nome_dia_semana = {1: 'pcpftc_segunda', 2: 'pcpftc_terca',  3: 'pcpftc_quarta', 4: 'pcpftc_quinta',
                           5: 'pcpftc_sexta',   6: 'pcpftc_sabado', 7: 'pcpftc_domingo'}
        nome_dia_qtd_semana = {1: 'pcpftc_qtd_segunda', 2: 'pcpftc_qtd_terca', 3: 'pcpftc_qtd_quarta',
                               4: 'pcpftc_qtd_quinta',  5: 'pcpftc_qtd_sexta', 6: 'pcpftc_qtd_sabado',
                               7: 'pcpftc_qtd_domingo'}
        itens_ids = self.pool.get('wiz.pcp.planejamento.producao.automatico.itens').search(cr, uid, [], context=context)
        self.pool.get('wiz.pcp.planejamento.producao.automatico.itens').unlink(cr, uid, itens_ids)
        brw_capa = self.browse(cr, uid, ids)
        if isinstance(brw_capa, list):
            brw_capa = brw_capa[0]
        categoria = brw_capa.wizppa_categoria_id.id
        data_inicio = data = datetime.strptime(brw_capa.wizppa_data_inicio, "%Y-%m-%d").date()
        data_fim = datetime.strptime(brw_capa.wizppa_data_fim, "%Y-%m-%d").date()
        if not brw_capa.wizppa_sobrepoe_planejamento:
            sql_planejamento = '''
                                SELECT pljprd_data
                                FROM pcp_planejamento_producao
                                WHERE pljprd_data >= '%s'
                                  AND pljprd_data <= '%s'
                                ORDER BY pljprd_data DESC
                                LIMIT 1
                               ''' % (str(data_inicio), str(data_fim))
            cr.execute(sql_planejamento)
            dados_planejamento = cr.dictfetchall()
            if dados_planejamento:
                data_aviso = str(dados_planejamento[0].get('pljprd_data')).split('-')
                data_aviso = data_aviso[2] + '/' + data_aviso[1] + '/' + data_aviso[0]
                raise osv.except_osv(_('Atencao'),
                                     _('Ja existe planejamento de producao ate a data: %s' % (data_aviso) ) )
        cont = 1
        while data <= data_fim:   # separa as datas por dia da semana
            dias_semana[data.isoweekday()].append(str(data))
            data += timedelta(days=cont)
        condicao = ''' '''
        if categoria:
            condicao += '''
              AND ep.categoria_id = %s ''' % (categoria)
        flag1 = False
        for i in dias_semana:
            if dias_semana.get(i):
                if not flag1:
                    flag1 = True
                    condicao +='''
                   AND (ftc.%s = True ''' % (nome_dia_semana.get(i))
                else:
                    condicao +='''
                   OR ftc.%s = True ''' % (nome_dia_semana.get(i))
        if flag1:
            condicao += ''' ) '''
        sql = """
            SELECT ftc.id,
                   ftc.pcpftc_segunda, ftc.pcpftc_qtd_segunda,
                   ftc.pcpftc_terca,   ftc.pcpftc_qtd_terca,
                   ftc.pcpftc_quarta,  ftc.pcpftc_qtd_quarta,
                   ftc.pcpftc_quinta,  ftc.pcpftc_qtd_quinta,
                   ftc.pcpftc_sexta,   ftc.pcpftc_qtd_sexta,
                   ftc.pcpftc_sabado,  ftc.pcpftc_qtd_sabado,
                   ftc.pcpftc_domingo, ftc.pcpftc_qtd_domingo
            FROM pcp_ficha_tecnica_capa AS ftc
            LEFT JOIN estoque_produto   AS ep ON ep.id = ftc.pcpftc_produto_id
            WHERE ftc.active = TRUE
              %s """ % (condicao)
        cr.execute(sql)
        dados_ficha = cr.dictfetchall()
        for dia in dias_semana:
            if not dias_semana.get(dia):
                continue
            for data in dias_semana.get(dia):
                for ficha in dados_ficha:
                    if not ficha.get(nome_dia_semana.get(dia)):
                        continue
                    quantidade = ficha.get(nome_dia_qtd_semana.get(dia))
                    vals_itens = {'wizppi_capa_id': ids[0],
                                  'wizppi_ficha_tecnica_id': ficha.get('id'),
                                  'wizppi_data': data,
                                  'wizppi_quantidade': quantidade,
                    }
                    self.pool.get('wiz.pcp.planejamento.producao.automatico.itens').create(cr, uid, vals_itens, context)
        return True

    def wizppa_onchange_periodo(self, cr, uid, ids, data_inicial, data_final, categoria_id, sobrepoe_planejamento,context=None):
        """
        O metodo verifica se há planejamento de producao, de acordo com o periodo informado.
        """
        context = context and context.copy() or {}
        res = {}
        if sobrepoe_planejamento:
            return res
        obj_pcp_planejamento_producao = self.pool.get('pcp.planejamento.producao')
        if data_inicial and data_final:
            args = [('pljprd_data', '>=', data_inicial), ('pljprd_data', '<=', data_final)]
            if categoria_id:
                obj_estoque_categoria = self.pool.get('estoque.categoria')
                read_estoque_categoria = obj_estoque_categoria.read(cr, uid, categoria_id, context=context)
                if read_estoque_categoria:
                    categoria = read_estoque_categoria.get('name')
                    args.append(('pljprd_categoria', '=', categoria))
            planejamento_producao_ids = obj_pcp_planejamento_producao.search(cr, uid, args, context=context)
            if planejamento_producao_ids:
                data_maior_periodo = obj_pcp_planejamento_producao.browse(
                    cr, uid, planejamento_producao_ids[-1]).pljprd_data
                data_maior_periodo = time.strftime('%d/%m/%Y', time.strptime(data_maior_periodo,
                                                                             '%Y-%m-%d'))
                res['warning'] = {
                    'title': _('Atencao'),
                    'message': _('Ja existe planejamento de producao ate a data %s' %
                                 data_maior_periodo)
                }
        return res

    def wizppa_button_confirmar(self, cr, uid, ids, context=None):
        """
        De acordo com o periodo informado para wizard de planejamento, por dia, varrer todas as
        fichas tecnicas selecionadas, uma a uma, verificar se coincide com o dia da semana setado
        para a ficha tecnica e, inserir registro a registro, na classe de pcp_planejamento_producao
        os atributos ficha tecnica, data e quantidade necessarios.
        Se "Sobrepor Planejamento" estiver marcado ==> Atualizar os planejamentos existentes de
        acordo com as novas quantidades. Senao se "Sobrepor Planejamento" nao estiver marcado ==>
        Manter os planejamentos existentes e gerar somente os planejamentos para os demais dias.
        """
        context = context and context.copy() or {}
        obj_pcp_planejamento_producao = self.pool.get('pcp.planejamento.producao')
        brw_planejamento_producao_automatico = self.browse(cr, uid, ids[0], context)
        itens_ids = []
        if not brw_planejamento_producao_automatico.wizppa_planejamento_producao_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nunhuma ficha tecnica selecionada'))
        for item in brw_planejamento_producao_automatico.wizppa_planejamento_producao_ids:
            itens_ids.append(item.id)
            args = [('pljprd_ficha_tecnica_id', '=',item.wizppi_ficha_tecnica_id.id),
                    ('pljprd_data', '=', item.wizppi_data)]
            plan_ids = obj_pcp_planejamento_producao.search(cr, uid, args, context=context)
            vals_planejamento = {'pljprd_ficha_tecnica_id': item.wizppi_ficha_tecnica_id.id,
                                 'pljprd_data': item.wizppi_data,
                                 'pljprd_quantidade': item.wizppi_quantidade,
                                }
            if not plan_ids:
                obj_pcp_planejamento_producao.create(cr, uid, vals_planejamento, context)
            else:
                obj_pcp_planejamento_producao.write(cr, uid, plan_ids[0], vals_planejamento, context)
        self.pool.get('wiz.pcp.planejamento.producao.automatico.itens').unlink(cr, uid, itens_ids)
        vals_limpar ={'wizppa_sobrepoe_planejamento': False,
                      'wizppa_categoria_id': '',
                      }
        self.write(cr, uid, ids[0], vals_limpar, context)
        return True

class wiz_pcp_planejamento_producao_automatico_itens(osv.TransientModel):
    # prefixo: wizppi
    from funcoes import _wizppi_get_categoria_produto, _wizppi_pljprd_get_unidade_peso
    _name = 'wiz.pcp.planejamento.producao.automatico.itens'

    _columns = {
        'wizppi_capa_id': fields.many2one('wiz.pcp.planejamento.producao.automatico',
                                                   required=True),
        'wizppi_categoria': fields.function(_wizppi_get_categoria_produto, method=True,
                                            string='Categoria', type='char', store=True),
        'wizppi_ficha_tecnica_id': fields.many2one('pcp.ficha.tecnica.capa', 'Ficha tecnica',
                                                   required=True),
        'wizppi_data': fields.date('Data', required=True),
        'wizppi_quantidade': fields.float('Quantidade', digits=(12, 3), required=True),

        'wizppi_qtd_unidade': fields.function(_wizppi_pljprd_get_unidade_peso, method=True,
                                              string='Unidade', type='float', digits=(12, 3),
                                              store=False),
        'wizppi_qtd_peso': fields.function(_wizppi_pljprd_get_unidade_peso, method=True,
                                           string='Peso(kg)', type='float', digits=(12, 3),
                                           store=False),
        }
