#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _

READ_LOAD = '_classic_write'

class pcp_wizard_relatorio_comparativo(osv.TransientModel):
    """
    Formulario com os parametros para impressao do relatorio de comparativo.
    """

    _name = 'pcp.wizard.relatorio.comparativo'  #

    _columns = {
        'data_inicial': fields.date('Data inicial', required=True),
        'data_final': fields.date('Data final', required=True),
        'categoria_ids': fields.many2one('estoque.categoria', string='Categoria',
                                         ondelete='cascade'),
        'ficha_tecnica_ids': fields.many2many('pcp.ficha.tecnica.capa',
                                              'pcp_ficha_tecnica_capa_relatorio_comparativo',
                                              'comparativo_id', 'pcp_ficha_tecnica_capa_id',
                                              'Ficha tecnica'),
    }

    def pcpwrc_button_imprimir_relatorio(self, cr, uid, ids, context=None):
        """
        Realiza a impressao do relatorio de planejamento de producao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} mensagem de aviso se nao houver ficha tecnica.

        """
        result = {'warning': {'title': _('Aviso'),
                              'message': _('Nenhuma comparativo para imprimir.')}}
        data = {}  # Dados utilizados para impressao do relatorio
        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_view_pcp_relatorio_comparativo = self.pool.get('view.pcp.relatorio.comparativo')

        campos = ['ficha_tecnica_ids', 'categoria_ids', 'data_inicial', 'data_final']
        dados_relatorio = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

         #  ____________________________Verificar datas informadas ________________________________
        if dados_relatorio[0]['data_final'] < dados_relatorio[0]['data_inicial']:
            raise osv.except_osv(_('Atencao'),
                                 _('Data final menor que a data inicial'))

        if dados_relatorio and isinstance(dados_relatorio, list):
            dados_relatorio = dados_relatorio[0]

        if dados_relatorio and isinstance(dados_relatorio, dict):
            # Se nao for informada nenhuma ficha tecnica, todas as fichas tecnicas deverao ser
            # consideradas, o mesmo para categoria
            args = []  # Sem argumento. Todas as fichas tecnicas de todas as categorias


            # Filtro categorias
            categoria_ids_selec = dados_relatorio['categoria_ids']
            if categoria_ids_selec:
                args.append(('pcprcp_categoria_id', '=', dados_relatorio['categoria_ids']))

            # Filtro de ficha tecnica
            ficha_tecnica_ids = dados_relatorio.get('ficha_tecnica_ids', [])
            if ficha_tecnica_ids:
                args.append(('pcprcp_ficha_tecnica_id', 'in', ficha_tecnica_ids))

            cr.execute(
                   """
                    CREATE or replace view view_pcp_relatorio_comparativo AS
                         (
                          SELECT ep.id, sum(CASE WHEN emo.estmog_codigo = 'P' THEN emi.quantidade ELSE 0 END) as pcprcp_quantidade_produzida,
                                 sum(CASE WHEN emo.estmog_codigo = 'P' THEN emi.vr_unitario ELSE 0 END) as pcprcp_valor,
                                 sum(CASE WHEN emo.estmog_codigo = 'E' THEN emi.quantidade ELSE 0 END) as pcprcp_quantidade_perda,
                                 ep.codigo  AS  pcprcp_codigo,
                                 ep.name    AS  pcprcp_produto,
                                 ftc.id     AS  pcprcp_ficha_tecnica_id,
                                 ep.categoria_id as pcprcp_categoria_id
                         FROM estoque_movimento_itens AS emi
                         INNER JOIN estoque_produto AS ep ON ep.id = emi.produto_id
                         INNER JOIN pcp_ficha_tecnica_capa  AS ftc  ON ftc.pcpftc_produto_id = ep.id
                         LEFT JOIN  estoque_movimento_capa emc ON emi.capa_id = emc.id
                         LEFT JOIN estoque_categoria AS ec ON ec.id = ep.categoria_id
                         LEFT JOIN estoque_natureza_operacao eno ON eno.id = emc.estmvc_operacao_id
                         LEFT JOIN estoque_movimento_origem emo ON emo.id = eno.estnto_movimento_origem_id
                         WHERE emi.create_date >= '%s' and emi.create_date <= '%s 23:59'
                         GROUP BY ep.name, ep.id, ftc.id
                         );

                        """ % (dados_relatorio['data_inicial'], dados_relatorio['data_final']))


            # Lista dos ID's
            data['ids'] = obj_view_pcp_relatorio_comparativo.search(cr, uid, args, context=context)



            if data['ids']:
                # Impressao do relatorio
                result = {'type': 'ir.actions.report.xml', 'report_name': 'rel_comparativo',
                          'datas': data}

            else:
                raise osv.except_osv(_('Atencao!'), _('Nenhuma relatorio para imprimir.'))

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
