#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from openerp.tools import drop_view_if_exists

READ_LOAD = '_classic_write'

class pcp_wizard_relatorio_curva_abc(osv.TransientModel):
    """
    Formulario com os parametros para impressao do relatorio de curva abc.
    """

    _name = 'pcp.wizard.relatorio.curva.abc'  #

    _columns = {
        'tipo': fields.selection((('E', 'Perda'), ('P', 'Producao')), 'Tipo', required=True),
        'classificar': fields.selection(((1, 'Custo'), (2, 'Venda'), (3, 'Quantidade')),
                                        'Classificar por', required=True),
        'data_inicial': fields.date('Data inicial', required=True),
        'data_final': fields.date('Data final', required=True),
        'categoria_ids': fields.many2one('estoque.categoria', string='Categoria',
                                         ondelete='cascade'),
        'agrupar_categoria': fields.boolean('Agrupar por categoria'),

    }

    def pcpwrc_button_imprimir_relatorio(self, cr, uid, ids, context=None):
        """
        Realiza a impressao do relatorio de curva abc.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} mensagem de aviso se nao houver ficha tecnica.

        """
        result = {'warning': {'title': _('Aviso'),
                              'message': _('Nao a dados para ser imprisso.')}}
        data = {}  # Dados utilizados para impressao do relatorio
        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_view_pcp_relatorio_curva_abc = self.pool.get('view.pcp.relatorio.curva.abc')

        campos = []
        dados_relatorio = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

         #  ____________________________Verificar datas informadas ________________________________
        if dados_relatorio[0]['data_final'] < dados_relatorio[0]['data_inicial']:
            raise osv.except_osv(_('Atencao'),
                                 _('Data final menor que a data inicial'))

        if dados_relatorio and isinstance(dados_relatorio, list):
            dados_relatorio = dados_relatorio[0]

        if dados_relatorio and isinstance(dados_relatorio, dict):
            # Se nao for informada nenhuma ficha tecnica, todas as fichas tecnicas deverao ser
            # consideradas, o mesmo para categoria
            args = []  # Sem argumento. Todas as fichas tecnicas de todas as categorias

            if dados_relatorio.get('agrupar_categoria'):
                sql = """ CREATE or replace view view_pcp_relatorio_curva_abc  AS
                         (
                            SELECT id,
                                   Sum(pcpabc_valor * pcpabc_quantidade) AS pcpabc_vr_total,
                                   Sum(pcpabc_ultimo_custo * pcpabc_quantidade) AS pcpabc_vr_custo_total,
                                   Sum(pcpabc_valor) AS pcpabc_valor,
                                   Sum(pcpabc_ultimo_custo) AS pcpabc_ultimo_custo,
                                   Sum(pcpabc_valor_acumulado) AS pcpabc_valor_acumulado,
                                   Sum(pcpabc_quantidade) AS pcpabc_quantidade,
                                   pcpabc_codigo,
                                   pcpabc_produto,
                                   pcpabc_categoria_id,
                                   pcpabc_unidade
                            FROM
                              (SELECT ec.id AS id,
                                      sum(emi.vr_preco_venda * emi.quantidade) / sum(emi.quantidade) AS pcpabc_valor,
                                      sum(emi.estmvi_saidas_ultimo_custo_total * emi.quantidade) / sum(emi.quantidade) AS pcpabc_ultimo_custo,
                                      sum(emi.vr_unitario) AS pcpabc_valor_acumulado,
                                      sum(emi.quantidade) AS pcpabc_quantidade,
                                      ec.controle AS pcpabc_codigo,
                                      ec.name AS pcpabc_produto,
                                      ep.categoria_id AS pcpabc_categoria_id,
                                      ec.controle AS pcpabc_unidade
                               FROM estoque_movimento_itens AS emi
                               INNER JOIN estoque_produto AS ep ON ep.id = emi.produto_id
                               INNER JOIN pcp_ficha_tecnica_capa AS ftc ON ftc.pcpftc_produto_id = ep.id
                               LEFT JOIN estoque_produto_embalagem AS epe ON epe.produto_id = ep.id
                               LEFT JOIN estoque_embalagem AS ee ON ee.id = epe.embalagem_id
                               LEFT JOIN estoque_movimento_capa emc ON emi.capa_id = emc.id
                               LEFT JOIN estoque_categoria AS ec ON ec.id = ep.categoria_id
                               LEFT JOIN estoque_natureza_operacao eno ON eno.id = emc.estmvc_operacao_id
                               LEFT JOIN estoque_movimento_origem emo ON emo.id = eno.estnto_movimento_origem_id
                               WHERE emi.create_date >= '%s'
                                 AND emi.create_date <= '%s 23:59'
                                 AND emo.estmog_codigo = '%s'
                                 AND ep.categoria_id IS NOT NULL
                               GROUP BY ec.name,
                                        ep.categoria_id,
                                        ec.id,
                                        ec.controle,
                                        emi.vr_total_item) AS tmp
                            GROUP BY id,
                                     pcpabc_codigo,
                                     pcpabc_produto,
                                     pcpabc_categoria_id,
                                     pcpabc_unidade
                         );
                     """ % (dados_relatorio['data_inicial'], dados_relatorio['data_final'],
                            dados_relatorio['tipo'])
            else:
                sql = """ CREATE or replace view view_pcp_relatorio_curva_abc  AS
                         (
                          SELECT ep.id,
                             0 AS pcpabc_vr_total,
                             0 AS pcpabc_vr_custo_total,
                             sum(emi.vr_preco_venda * emi.quantidade) / sum(emi.quantidade) as pcpabc_valor,
                             sum(emi.estmvi_saidas_ultimo_custo_total * emi.quantidade) / sum(emi.quantidade) as pcpabc_ultimo_custo,
                             sum(emi.quantidade)
                             as pcpabc_quantidade,
                             ep.codigo  AS  pcpabc_codigo,
                             ep.name    AS  pcpabc_produto,
                             ee.sigla as pcpabc_unidade,
                             ftc.id     AS  pcprcp_ficha_tecnica_id,
                             ep.categoria_id as pcpabc_categoria_id
                         FROM estoque_movimento_itens AS emi
                         INNER JOIN estoque_produto AS ep ON ep.id = emi.produto_id
                         INNER JOIN pcp_ficha_tecnica_capa  AS ftc  ON ftc.pcpftc_produto_id = ep.id
                         LEFT JOIN estoque_produto_embalagem AS epe ON epe.produto_id = ep.id
                         LEFT JOIN estoque_embalagem AS ee ON ee.id = epe.embalagem_id
                         LEFT JOIN estoque_movimento_capa emc ON emi.capa_id = emc.id
                         LEFT JOIN estoque_categoria AS ec ON ec.id = ep.categoria_id
                         LEFT JOIN estoque_natureza_operacao eno ON eno.id = emc.estmvc_operacao_id
                         LEFT JOIN estoque_movimento_origem emo ON emo.id = eno.estnto_movimento_origem_id
                         WHERE emi.create_date >= '%s' and emi.create_date <= '%s 23:59'
                         AND emo.estmog_codigo = '%s'
                         GROUP BY ep.name, ep.id, ee.sigla, ftc.id
                         );
                        """ % (dados_relatorio['data_inicial'], dados_relatorio['data_final'],
                               dados_relatorio['tipo'])
            # Filtro categorias
            categoria_id_selec = dados_relatorio['categoria_ids']
            if categoria_id_selec:
                args.append(('pcpabc_categoria_id', '=', dados_relatorio['categoria_ids']))

            drop_view_if_exists(cr, 'view_pcp_relatorio_curva_abc')
            cr.execute(sql)
            # Lista dos ID's
            data['ids'] = obj_view_pcp_relatorio_curva_abc.search(
                cr, uid, args, context=context)

            if data['ids']:
                # Impressao do relatorio
                result = {'type': 'ir.actions.report.xml', 'report_name': 'rel_curva_abc',
                          'datas': data}

            else:
                raise osv.except_osv(_('Atencao!'), _('Nenhuma relatorio para imprimir.'))
        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
