# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class procedimentos_atividade(osv.Model):
    """
        Tabela de atividades, e o POP propriamente dito.
    """
    # prefixo: prcatv_
    from funcoes import _prcatv_get_tempo
    _name = 'procedimentos.atividade'
    _columns = {
        'id': fields.integer('Id'),
        'name': fields.char('Descricao', size=60, required=True, select=True),
        # Nome da atividade\n
        'active': fields.boolean('Ativo'),
        'prcatv_gerar_processo': fields.boolean('Gerar processo'),
        'responsavel_id': fields.many2one("adm.empresa.centrocusto", 'Departamento',
                                          ondelete="restrict", required=True, select=True),
        # Id da funcao responsavel pela atividade\n
        'codigo': fields.char('Padrao Nro', size=10, required=False, select=True),
        # Codigo padrao do POP
        'data_criacao': fields.date('Data da criacao'),  # Data da criacao do POP
        'data_revisao': fields.date('Data da revisao'),  # Data da revisao do POP
        'revisao': fields.char('Revisao', size=10),  # Identificacao da revisao
        'frequencia': fields.selection([('D', 'Diaria'), ('S', 'Semanal'), ('Q', 'Quinzenal'),
                                        ('M', 'Mensal'), ('B', 'Bimestral'), ('T', 'Trimestral'),
                                        ('U', 'Quadrimestral'), ('R', 'Semestral'), ('A', 'Anual'),
                                        ('E', 'Esporadico')], 'Frequencia', size=1, help="""-> Bimestral: O mes deve ser 1 ou 2. Primeiro ou segundo mes do bimestre.
                                 \n-> Trimestral: O mes deve ser 1, 2 ou 3.  Primeiro, segundo ou terceiro  mes do trimestre.
                                 \n-> Quadrimestral: O mes deve ser 1, 2, 3 ou 4.  Primeiro, segundo, terceiro ou quarto mes do quadrimestre.
                                 \n-> Semestral: O mes deve ser 1, 2, 3, 4, 5 ou 6. Primeiro, segundo, terceiro, quarto, quinto ou sexto  mes do semestre.
                                 \n-> Anual: Qual o mes do ano. De 1 a 12."""),
        # Frequencia com que deve ser realizada a atividade\nD=Diaria\nS=Semanal\nQ=Quinzenal\n
        # M=Mensal\nB=Bimestral\nT=Trimestral\nU=qUadrimestral\nR=Semestral\nA=Anual\nE=Exporadica
        'dia': fields.integer('Dia'),  # dia em que deve desenvolver a atividade\nExemplo:\n
        # Se semanal 1 = primeiro dia da semana\nSe Quinzenal 1 = aprimeiro dia da quinzena\n
        'observacoes': fields.text('Observacoes'),  # Observacoes complementares\n
        'acoes_corretivas': fields.text('Acoes Corretivas'),  # Acoes Corretivas\n
        'passo_ids': fields.one2many('procedimentos.passo', 'atividade_id', 'Passos'),
        # Passos a serem desenvolvidos nesta atividade
        'recurso_ids': fields.one2many('procedimentos.recurso.pop', 'atividade_id', 'Recursos'),
        # recursos utilizados nesta atividade
        'prcatv_mes': fields.integer('Mes', help='Variavel conforme log na tela'),
        'prcatv_funcionario_ids': fields.many2many(
            'rh.funcionario', 'procedimento_atividade_funcionario', 'procedimento_atividade_id',
            'funcionario_id', 'Funcionarios'),
        'prcatv_tipo_tempo': fields.selection([('M', 'Minutos'), ('H', 'Horas'), ('D', 'Dias')], size=1,
                                              string='Periodo tempo p/execucao'),
        'prcatv_tempo_previsto': fields.function(_prcatv_get_tempo, method=True, string='Tempo previsto', multi="dados",
                                                 type='integer', store=False),
    }
    _defaults = {
        'active': lambda *args: True,
        'prcatv_gerar_processo': lambda *args: False,
        'prcatv_tipo_tempo': 'H',
    }
