# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from lxml import etree
from mx.DateTime import now
import time
from openerp.tools.translate import _
from openerp.osv import fields, osv

# The hard-coded super-user id (a.k.a. administrator, or root user).
SUPERUSER_ID = 1


class task(osv.Model):
    _name = "project_br.task"
    _inherit = "project_br.task"

    def prjtsk_onchange_data_inicio_previsto(self, cr, uid, ids, data_inicio_previsto,
                                             context=None):
        """
        Metodo chamado ao alterar dados no campo 'data_inicio_previsto'. Atribui o valor desse ao
        campo 'date_start'.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param data_inicio_previsto: data de previsão para inicio de uma tarefa.
        :type data_inicio_previsto: str
        :param context: dicionario com valores contextuais.
        :type context: dict
        """
        context = context and context.copy() or {}
        res = {}
        if not data_inicio_previsto:
            return res
        return {'value': {'date_start': data_inicio_previsto}}

    def onchange_projeto(self, cr, uid, ids, project_br_id, context=None):
        '''Quando o projeto eh selecionado preenche automaticamente o campo pessoa_id
           que eh respectivamente o cliente do projeto, se nao existir cliente selecionado
           retorna falso'''
        context = context and context.copy() or {}
        val = {}
        val['pessoa_id'] = 0
        if project_br_id:
            obj_projeto = self.pool.get('project_br.project_br')
            cliente = obj_projeto.read(cr, uid, project_br_id, ['pessoa_id'], context=context)['pessoa_id']
            if cliente:
                val['project_br_id'] = project_br_id
                val['pessoa_id'] = cliente[0]
        return {'value': val}

    def _str_get(self, task, level=0, border='***', context=None):
        context = context and context.copy() or {}
        return border + ' ' + (task.user_id and task.user_id.name.upper() or '') + (
            level and (': L' + str(level)) or '') + (
                   ' - %.1fh / %.1fh' % (task.effective_hours or 0.0, task.planned_hours)) + ' ' + border + '\n' + \
               border[0] + ' ' + (task.name or '') + '\n' + \
               (task.description or '') + '\n\n'

    def onchange_planned(self, cr, uid, ids, planned, effective=0.0):
        return {'value': {'remaining_hours': planned - effective}}

    def copy_data(self, cr, uid, id, default={}, context=None):
        context = context and context.copy() or {}
        default = default or {}
        default['work_ids'] = []
        return super(task, self).copy_data(cr, uid, id, default, context)

    # Override view according to the company definition
    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        obj_tm = self.pool.get('res.users').browse(cr, SUPERUSER_ID, uid,
                                                   context=context).company_id.project_br_time_mode
        tm = obj_tm or 'Hours'

        res = super(task, self).fields_view_get(cr, uid, view_id, view_type, context, toolbar,
                                                submenu=submenu)

        if tm in ['Hours', 'Hour', 'hours', 'hour']:
            return res

        eview = etree.fromstring(res['arch'])

        def _check_rec(eview):
            if eview.attrib.get('widget', '') == 'float_time':
                eview.set('widget', 'float')
            for child in eview:
                _check_rec(child)
            return True

        _check_rec(eview)

        res['arch'] = etree.tostring(eview)

        for f in res['fields']:
            if 'Hours' in res['fields'][f]['string']:
                res['fields'][f]['string'] = res['fields'][f]['string'].replace('Hours', tm)
        return res

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not args:
            args = []
        context = context and context.copy() or {}
        obj_project_br = self.pool.get('project_br.project_br')
        if 'alltask' in context or context.get('mytask', False) or \
                context.get('tarefas_nao_concluidas', False) or \
                context.get('atendimentos_project_br', False):
                # or context.get('todos_projetos', False):
            argumentos = []
            for arg in args:
                if arg[0] != 'project_br_id':
                    argumentos.append(arg)
                    continue
                elif arg[0] == 'project_br_id':
                    if arg[1] == '=' and isinstance(arg[2], int):
                        argumentos.append(arg)
                        continue
                    else:
                        project_ids = obj_project_br.search(cr, uid, [('name', arg[1], arg[2])],
                                                            context=context)
                        argumentos.append(['project_br_id', 'in', project_ids])
                        continue
                else:
                    project_ids = obj_project_br.search(cr, uid, [('name', 'ilike', arg[2])],
                                                        context=context)
                ids = []
                while (project_ids != None) and (project_ids != []):
                    ids = ids + project_ids
                    project_ids = obj_project_br.search(cr, uid, [('parent_id', 'in', project_ids)],
                                                        context=context)
                argumentos.append(('project_br_id', 'in', ids))
            res = super(task, self).search(cr, uid, argumentos, offset, limit, order, context,
                                           count)
        else:
            res = super(task, self).search(cr, uid, args, offset, limit, order, context, count)
        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        res = super(task, self).read(cr, uid, ids, fields=fields, context=context, load=load)
        return res

    def do_close(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        # request = self.pool.get('res.request')
        tasks = [self.browse(cr, uid, ids)]
        for task in tasks:
            if isinstance(task, list):
                task = task[0]
            project_br = task.project_br_id

            if project_br and project_br.warn_manager and project_br.manager.id:
                task_user_ids = self.pool.get('res.users').search(cr, uid, [('pessoa_id', '=', task.user_id.id)],
                                                                  context=context)
                manager_user_ids = self.pool.get('res.users').search(cr, uid,
                                                                     [('pessoa_id', '=', project_br.manager.id)],
                                                                     context=context)
                # if task_user_ids and manager_user_ids:
                #     request.create(cr, uid, {
                #         'name': _("Task '%s' closed") % task.name,
                #         'body': 'Projeto: %d-%s  \nTarefa: %d-%s  \nUsuario: %d-%s' % (
                #             project_br.id, project_br.name, task.id, task.name, project_br.pessoa_uid, task.user_id.name),
                #         'state': 'waiting',
                #         'act_from': task_user_ids[0],
                #         'act_to': manager_user_ids[0],
                #     }, context)
            dados_hora = self._hours_get(cr, uid, [task.id], None, None, {})
            dados_hora = dados_hora.get(task.id)
            hora_gasta = dados_hora.get('effective_hours') or 0.0
            self.write(cr, uid, [task.id], {
                'state': 'done',
                'date_close': time.strftime('%Y-%m-%d %H:%M:%S'),
                'remaining_hours': task.planned_hours - hora_gasta}, context)

            if task.parent_id and task.parent_id.state in ('pending', 'draft'):
                reopen = True
                for child in task.parent_id.child_ids:
                    if child.id != task.id and child.state not in ('done', 'cancelled'):
                        reopen = False
                if reopen:
                    self.do_reopen(cr, uid, [task.parent_id.id], context)
        return True

    def do_reopen(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        # request = self.pool.get('res.request')
        tasks = self.browse(cr, uid, ids)
        for task in tasks:
            project_br = task.project_br_id
            if project_br and project_br.warn_manager and project_br.manager.id:
                task_user_ids = self.pool.get('res.users').search(cr, uid, [('pessoa_id', '=', task.user_id.id)],
                                                                  context=context)
                manager_user_ids = self.pool.get('res.users').search(cr, uid,
                                                                     [('pessoa_id', '=', project_br.manager.id)],
                                                                     context=context)
                # if task_user_ids and manager_user_ids:
                #     request.create(cr, uid, {
                #         'name': _("Task '%s' re-opened") % task.name,
                #         'body': 'Projeto: %d-%s  \nTarefa: %d-%s  \nUsuario: %d-%s' % (
                #             project_br.id, project_br.name, task.id, task.name, project_br.pessoa_uid, task.user_id.name),
                #         'state': 'waiting',
                #         'act_from': manager_user_ids[0],
                #         'act_to': task_user_ids[0],
                #     }, context)

            self.write(cr, uid, [task.id], {'state': 'open', 'date_close': False}, context)
        return True

    def do_delegado(self, cr, uid, ids, *args):
        return {
            'view_type': 'form',
            "view_mode": 'form',
            'res_model': 'project_br.task.delegate',
            'type': 'ir.actions.act_window',
            'ref': 'action_task_delegate',
            'target': 'new',
        }

    def do_concluido(self, cr, uid, ids, *args):
        return {
            'view_type': 'form',
            "view_mode": 'form',
            'res_model': 'project_br.task.close',
            'type': 'ir.actions.act_window',
            'ref': 'action_task_close',
            'target': 'new',
        }

    def do_cancel(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        # request = self.pool.get('res.request')
        tasks = self.browse(cr, uid, ids)
        for task in tasks:
            project_br = task.project_br_id
            # if project_br.warn_manager and project_br.manager and (project_br.manager.id != uid):
            #     request.create(cr, uid, {
            #         'name': _("Task '%s' cancelled") % task.name,
            #         'state': 'waiting',
            #         'act_from': uid,
            #         'act_to': project_br.pessoa_uid,
            #         'ref_doc1': 'project_br.task,%d' % task.id,
            #         'ref_doc2': 'project_br.project_br,%d' % project_br.id,
            #     }, context)
            self.write(cr, uid, [task.id], {
                'state': 'cancelled',
                'remaining_hours': task.planned_hours - task.total_hours,
                'date_close': False}, context)
        return True

    def do_open(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        tasks = self.browse(cr, uid, ids)
        for t in tasks:
            self.write(cr, uid, [t.id],
                       {'state': 'open', 'date_close': False, 'date_start': time.strftime('%Y-%m-%d %H:%M:%S')}, context)
        return True

    def do_draft(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'draft'}, context)
        return True

    def do_pending(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'pending'}, context)
        return True

    # ***********************************************************************************************************************************

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        if not vals:
            vals = {}
        if context.get('__copy_data_seen') and vals.get('planned_hours'):
            vals['remaining_hours'] = vals.get('planned_hours') or 0.0
        # ***************************************ja estava no create********************************************************************************************
        if vals.get('project_br_id', False) and not vals.get('pessoa_id', False):
            project_br_id = vals.get('project_br_id', 0)
            vals['pessoa_id'] = \
                self.pool.get('project_br.project_br').read(cr, uid, project_br_id, ['pessoa_id'], context=context)[
                    'pessoa_id'][0]
        return super(task, self).create(cr, uid, vals, context=context)

    # ***********************************************************************************************************************************

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        brw_task = self.browse(cr, uid, ids)
        if brw_task and isinstance(brw_task, list):
            brw_task = brw_task[0]
        prjprj_avisar_atribuido = brw_task.project_br_id.prjprj_avisar_atribuido
        if 'user_id' in vals:
            if prjprj_avisar_atribuido:
                user_id = vals.get('user_id')
                subject = str(brw_task.id) + ' - ' + brw_task.name
                mail_mail = self.pool.get('mail.mail')
                email = self.pool.get('adm.pessoa').browse(cr, uid, user_id, context=context).email
                project_email = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'adm',
                                                                                 'admpar_prj_email_projeto').get(
                    'admpar_prj_email_projeto')
                if project_email:
                    project_name = brw_task.project_br_id.name
                    horas_planejadas = brw_task.planned_hours
                    body_html = u"""<p><b>Nro.Tarefa:</b> %s<br/>
                                       <b>Projeto:</b> %s<br/>
                                       <b>Sumário da tarefa:</b> %s<br/>
                                       <b>Horas planejadas:</b> %s</p>""" % (
                        str(brw_task.id), project_name, brw_task.name, horas_planejadas)
                    dict_mail = {'email_from': project_email, 'email_to': email, 'subject': subject,
                                 'body_html': body_html}
                    mail_id = mail_mail.create(cr, uid, dict_mail, context)
                    mail_mail.send(cr, uid, [mail_id])
        if vals.get('user_id'):
            try:
                user_ids = self.pool.get('res.users').search(cr, uid, [('pessoa_id', '=', vals['user_id'])],
                                                             context=context)
                vals['pessoa_uid'] = user_ids[0]
            except:
                vals['pessoa_uid'] = 0
        if vals.get('project_br_id', False) and not vals.get('pessoa_id', False):
            project_br_id = vals.get('project_br_id', 0)
            vals['pessoa_id'] = \
                self.pool.get('project_br.project_br').read(cr, uid, project_br_id, ['pessoa_id'], context=context)[
                    'pessoa_id'][0]
        return super(task, self).write(cr, uid, ids, vals, context=context)

