# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from mx import DateTime
from mx.DateTime import now
import time
from openerp.tools.translate import _
import sys
from openerp.osv import fields, osv
from openerp.tools.translate import _


class project_br(osv.Model):

    from funcoes import _progress_rate, _complete_name, _get_user
    from funcoes import _verifica_recursao, _prjprj_get_hora, _prjprj_get_datas

    _name = "project_br.project_br"
    _description = "project_br"
    _columns = {

        ## _____________________ char ________________________________
        'id': fields.integer('ID', readonly=True),
        'name': fields.char("project_br Name", size=128, required=True),
        'notes': fields.text('Notes', help="Internal description of the project_br."),
        'warn_header': fields.text('Mail Header', help="Header added at the beginning of the email for the warning "
                                                       "message sent to the customer when a task is closed."),
        'warn_footer': fields.text('Mail Footer', help="Footer added at the beginning of the email for the warning "
                                                       "message sent to the customer when a task is closed."),
        ## _____________________ integer ________________________________
        'priority': fields.integer('Sequence'),
        'pessoa_uid': fields.integer('Usuario'), # pessoa_id do user_id - Atualizado automaticamente
        'prjprj_quantidade_deslocamento': fields.integer('Quantidade deslocamento'),

        ## _____________________ boolean ________________________________
        'active': fields.boolean('Active'),
        'warn_customer': fields.boolean('Warn Pessoa', help="If you check this, the user will have a popup when "
                                                            "closing a task that propose a message to send by email "
                                                            "to the customer."),
        'warn_manager': fields.boolean('Warn Manager', help="If you check this field, the project_br manager will "
                                                            "receive a request each time a task is completed by his "
                                                            "team."),
        'prjprj_faturar': fields.boolean('Faturar', help="Se marcado, ao fechar o projeto, estara disponivel para faturar"),
        'prjprj_avisar_atribuido': fields.boolean('Avisar atribuido', help="Se marcado, dispara email com informações da tarefa, posicionando ao atribuído que existe tarefa para ele."),

        ## _____________________ date ________________________________
        'date_end': fields.date('Expected End'),
        'date_start': fields.date('Starting Date'),
        'data_conclusao': fields.date('Data Conclusao'),

        ## _____________________ selection ________________________________
        'state': fields.selection([('template', 'Template'), ('open', 'Running'), ('pending', 'Pending'),
                                   ('cancelled', 'Cancelled'), ('done', 'Done')], 'State', required=True, readonly=True),

        ## _____________________ float ________________________________
        'prjprj_tempo_deslocamento': fields.float('Tempo de Deslocamento', digits=(5, 2),
                                                  help="Tempo de deslocamento de para atender ao cliente. Apenas um "
                                                       "trecho ida ou volta."),
        'prjprj_valor_hora': fields.float('Valor hora', digits=(5, 2)),

        ## _____________________ function ________________________________
        'total_hours': fields.function(_progress_rate, multi="progress", method=True,
                                       string='Total Time', help="Sum of total hours of all tasks "
                                                                 "related to this project_br.",
                                       type='float'),
        'progress_rate': fields.function(_progress_rate, multi="progress", method=True,
                                         string='Progress', help="Percent of tasks closed "
                                                                 "according to the total of tasks "
                                                                 "todo.",
                                         type='float'),
        'complete_name': fields.function(_complete_name, method=True, string="project_br Name",
                                         type='char', size=128),
        'planned_hours': fields.function(_progress_rate, multi="progress", method=True,
                                         string='Planned Time',
                                         help="Sum of planned hours of all tasks related to this "
                                              "project_br.",
                                         type='float'),
        'effective_hours': fields.function(_progress_rate, multi="progress", method=True,
                                           string='Time Spent',
                                           help="Sum of spent hours of all tasks related to this "
                                                "project_br.",
                                           type='float'),
        'prjprj_hora_planejada_formatada': fields.function(_prjprj_get_hora, multi="hora",
                                                           method=True, string='Planned Time',
                                                           help="Sum of planned hours of all "
                                                                "tasks related to this project_br",
                                                           type='char'),
        'prjprj_hora_efetiva_formatada': fields.function(_prjprj_get_hora, multi="hora",
                                                         method=True, string='Time Spent',
                                                         help="Sum of spent hours of all tasks "
                                                              "related to this project_br.",
                                                         type='char'),
        'prjprj_data_atendimento_ultimo': fields.function(
            _prjprj_get_datas, method=True, string='Data ultimo atendimento', type='datetime',
            multi='data_atendimento', store=False),
        'prjprj_data_atendimento_proximo': fields.function(
            _prjprj_get_datas, method=True, string='Data proximo atendimento', type='datetime',
            multi='data_atendimento', store=False),

        ## _____________________ many2one ________________________________
        'manager': fields.many2one('adm.pessoa', 'project_br Manager', domain="[('pessoa_tipo','=','F')]"),
        'pessoa_id': fields.many2one('adm.pessoa', 'Cliente', select=1),
        'parent_id': fields.many2one('project_br.project_br', 'Parent project_br',
                                     help="If you have [?] in the name, it means there are no analytic account linked "
                                          "to project_br."),
        'prjprj_contato_id': fields.many2one('adm.pessoa', 'Contato',
                                        context="{'contatos_de':pessoa_id}", ondelete="restrict"),
        'company_id': fields.many2one('res.company', 'Empresa', required=True),
        'endereco_id': fields.many2one('adm.pessoa.endereco', 'Endereco'),

        ## _____________________ one2many ________________________________
        'tasks': fields.one2many('project_br.task', 'project_br_id', "project_br tasks"),
        'child_id': fields.one2many('project_br.project_br', 'parent_id', 'Subproject_br'),

        ## _____________________ many2many ________________________________
        'members': fields.many2many('adm.pessoa', 'project_br_user_rel', 'project_br_id', 'uid', 'project_br Members',
                                    domain="[('pessoa_tipo','=','F')]", help="project_br's member. Not used in any "
                                                                             "computation, just for information "
                                                                             "purpose."),
    }

    _defaults = {
        'active': lambda * a: True,
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(cr, uid, uid).company_id.id,
        'manager': _get_user,
        'priority': lambda * a: 1,
        'date_start': lambda * a: time.strftime('%Y-%m-%d'),
        'state': lambda * a: 'open'
    }
    _order = "name,parent_id,date_start,priority"

    _constraints = [
        (_verifica_recursao, 'Voce nao pode cadastrar um projeto que possua o projeto pai como ele mesmo!', ['parent_id'])
    ]


