# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from mx import DateTime
from mx.DateTime import now
import time
from openerp.tools.translate import _
import sys
from openerp.osv import fields, osv
from openerp.tools.translate import _
class task(osv.Model):

    from funcoes import _history_get, _hours_get, _get_user, _default_project_br
    _name = "project_br.task"
    _description = "Tasks"
    _date_name = "date_start"

    _columns = {
        'active': fields.boolean('Active'),
        'name': fields.char('Task summary', size=128, required=True),
        'description': fields.text('Description'),
        'priority' : fields.selection([('4', 'Very Low'), ('3', 'Low'), ('2', 'Medium'), ('1', 'Urgent'), ('0', 'Very urgent')], 'Importance'),
        'sequence': fields.integer('Sequence'),
        'type': fields.many2one('project_br.task.type', 'Type'),
        'state': fields.selection([('draft', 'Draft'), ('open', 'In Progress'), ('pending', 'Pending'), ('cancelled', 'Cancelled'), ('done', 'Done')], 'Status', readonly=True, required=True),
        'date_start': fields.datetime('Starting Date'),
        'date_deadline': fields.datetime('Deadline'),
        'date_close': fields.datetime('Date Closed', readonly=True),
        'project_br_id': fields.many2one('project_br.project_br', 'project_br', ondelete='cascade', domain="[('pessoa_id','!=',False)]",
            help="If you have [?] in the project_br name, it means there are no analytic account linked to this project_br."),
        'parent_id': fields.many2one('project_br.task', 'Parent Task'),
        'child_ids': fields.one2many('project_br.task', 'parent_id', 'Delegated Tasks'),
        'history': fields.function(_history_get, method=True, string="Task Details", type="text"),
        'notes': fields.text('Notes'),
        'planned_hours': fields.float('Planned Hours', required=True, help='Estimated time to do the task, usually set by the project_br manager when the task is in draft state.'),
        'effective_hours': fields.function(_hours_get, method=True, string='Hours Spent',
                                           type='float', digits=(16, 4), multi='hours',
                                           store=True, help="Computed using the sum of the task "
                                                            "work done."),
        'remaining_hours': fields.float('Remaining Hours', digits=(16, 4), help="Total remaining time, can be re-estimated periodically by the assignee of the task."),
        'total_hours': fields.function(_hours_get, method=True, string='Total Hours', multi='hours', store=True, help="Computed as: Time Spent + Remaining Time."),
        'progress': fields.function(_hours_get, method=True, string='Progress (%)', multi='hours', store=True, help="Computed as: Time Spent / Total Time."),
        'delay_hours': fields.function(_hours_get, method=True, string='Delay Hours',
                                       type='float', digits=(16, 4), multi='hours', store=True,
                                       help="Computed as: Total Time - Estimated Time. It gives the"
                                            " difference of the time estimated by the project_br "
                                            "manager and the real time to close the task."),
        'user_id': fields.many2one('adm.pessoa', 'Assigned to', domain="[('pessoa_tipo','=','F')]"),
        'pessoa_uid': fields.integer('Usuario'), # pessoa_id do user_id - Atualizado automaticamente
        'delegated_user_id': fields.related('child_ids', 'user_id', type='many2one', relation='adm.pessoa', string='Delegated To'), #para uso wizard/task_delegate
        'pessoa_id': fields.many2one('adm.pessoa', 'Pessoa'),
        'work_ids': fields.one2many('project_br.task.work', 'task_id', 'Work done'),
        'data_inicio_previsto': fields.datetime('Inicio Previsto'),
        'prjtsk_a_faturar': fields.boolean('A Faturar', help="Se marcado, a tarefa tem suas horas faturadas no Projeto."),
        'prjtsk_deslocamento': fields.boolean('Deslocamento', help="Se marcado indica que a tarefa está planejada com deslocamento do consultor"),
        'prjtsk_horas_a_deduzir':fields.float('Horas a deduzir',digits=(16, 4),
                                              help="Total de horas a deduzir do total de horas faturadas do projeto."),
        'prjtsk_considerar_tempo_minimo': fields.boolean('Considerar tempo minimo',
                                                         help="Se marcado, o tempo minimo a ser "
                                                              "cobrado e considerado"),
    }
    _defaults = {
        'user_id': _get_user,
        'state': lambda * a: 'draft',
        'priority': lambda * a: '2',
        'progress': lambda * a: 0,
        'sequence': lambda * a: 10,
        'active': lambda * a: True,
        'data_inicio_previsto': lambda * a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'project_br_id': _default_project_br,
        'prjtsk_a_faturar': True,
        'prjtsk_deslocamento': False
    }
    _order = "project_br_id,sequence,data_inicio_previsto,date_start,date_close,state"

