# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from datetime import datetime, timedelta
import logging
import time
import re


class rastreio_encomendas(osv.Model):
    _inherit = 'rastreio.encomendas'
    _logger = logging.getLogger(_inherit)

    def _rasenc_get_destino(self, cr, uid, pessoa_id, context=None):
        if not pessoa_id:
            return ''
        brw_pessoa = self.pool.get('adm.pessoa').browse(cr, uid, pessoa_id, context=context)
        try:
            destino = '%s/%s' % (brw_pessoa.cidade[:30], brw_pessoa.uf)
        except:
            destino = ''
        return destino

    def create(self, cr, uid, vals, context=None):
        vals['name'] = vals.get('name').upper().strip()
        vals['name'] = re.sub('[^0-9A-Z]', '', vals['name'])
        if vals.get('rasenc_pessoa_id'):
            vals['rasenc_destino'] = self._rasenc_get_destino(cr, uid, vals.get('rasenc_pessoa_id'), context=context)
        res = super(rastreio_encomendas, self).create(cr, uid, vals, context=context)
        vals_evento = {
            'raseve_encomenda_id': res,
            'raseve_situacao': 'Registrado',
            'raseve_data_hora': time.strftime('%Y-%m-%d %H:%M:%S')
        }
        self.pool.get('rastreio.eventos').create(cr, uid, vals_evento, context=context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        if vals.get('rasenc_pedido_id'):
            _dados_pedido = self.pool.get('estoquepedidos.pedido.capa').read(cr, uid, vals.get('rasenc_pedido_id'),
                                                                             ['tipo', 'pessoa_id'], context=context)
            if not isinstance(_dados_pedido, (list, tuple)):
                _dados_pedido = [_dados_pedido]
            for _pedido in _dados_pedido:
                vals['rasenc_compra_venda'] = _pedido.get('tipo')
                vals['rasenc_pessoa_id'] = _pedido.get('pessoa_id')[0]
                if vals.get('rasenc_pessoa_id'):
                    vals['rasenc_destino'] = self._rasenc_get_destino(cr, uid, vals.get('rasenc_pessoa_id'),
                                                                      context=context)
        if vals.get('rasenc_nf_id'):
            _dados_nf = self.pool.get('fiscal.nf.capa').read(cr, uid, vals.get('rasenc_nf_id'),
                                                                             ['tipo', 'destinatario_id'], context=context)
            if not isinstance(_dados_nf, (list, tuple)):
                _dados_nf = [_dados_nf]
            for _nf in _dados_nf:
                vals['rasenc_compra_venda'] = 'C' if _nf.get('tipo') == 'E' else 'V'
                vals['rasenc_pessoa_id'] = _nf.get('destinatario_id')[0]
                if vals.get('rasenc_pessoa_id'):
                    vals['rasenc_destino'] = self._rasenc_get_destino(cr, uid, vals.get('rasenc_pessoa_id'),
                                                                      context=context)
        if vals.get('name'):
            vals['name'] = vals.get('name').upper()
        res = super(rastreio_encomendas, self).write(cr, uid, ids, vals, context=context)
        return res

    def rasenc_rastrear_varios_itens(self, cr, uid, pacotes, context=None):
        def __alterar_codigo_rastreio(encomenda):
            track_number = encomenda.numero
            encomenda_id = pacotes.get(servico_code).get(track_number).get('enc_id')
            dados_encomenda = self.read(cr, uid, encomenda_id, ['rasenc_observacoes'], context=context)
            vals = {
                'name': encomenda.codigo_rastreio_novo,
            }
            if dados_encomenda.get('rasenc_observacoes'):
                vals['rasenc_observacoes'] = '%s\n%s' % (dados_encomenda.get('rasenc_observacoes'), encomenda.numero)
            else:
                vals['rasenc_observacoes'] = encomenda.numero
            self.write(cr, uid, [encomenda_id], vals, context=context)

        def __incluir_evento(track_number, eventos):
            for evento in eventos:
                _data_hora_br = evento.get('data_hora').replace('T', ' ')
                _data_hora_utc = str(self.pool.get('adm.utils').admutl_converte_tz(cr, uid, datetime.strptime(_data_hora_br, '%Y-%m-%d %H:%M:%S'), tz_destino='UTC'))
                _situacao = evento.get('situacao')
                encomenda_id = pacotes.get(servico_code).get(track_number).get('enc_id')
                servico_id = pacotes.get(servico_code).get(track_number).get('serv_id')
                sql = """
                  select id from rastreio_eventos
                      where raseve_encomenda_id = %s and
                            raseve_data_hora = '%s' and
                            raseve_servico_id = %s and
                            raseve_situacao = '%s'""" % \
                      (encomenda_id, _data_hora_utc, servico_id, _situacao)
                cr.execute(sql)
                _evento = cr.fetchone()
                if _evento and _evento[0]:
                    pass
                else:
                    vals = {
                        'raseve_encomenda_id': encomenda_id,
                        'raseve_data_hora': _data_hora_utc,
                        'raseve_pais': evento.get('pais'),
                        'raseve_estado': evento.get('estado'),
                        'raseve_cidade': evento.get('cidade'),
                        'raseve_agencia': evento.get('agencia'),
                        'raseve_situacao': evento.get('situacao'),
                        'raseve_observacao': evento.get('observacao'),
                        # 'raseve_evento': _txt.replace('None', '').replace('  ', ' '),
                        'raseve_servico_id': servico_id,
                    }
                    self.pool.get('rastreio.eventos').create(cr, uid, vals, context=context)
                    vals = {'rasenc_pendente_visualizar': True}
                    if evento.get('fim'):
                        vals['rasenc_entregue'] = True
                    self.write(cr, uid, [encomenda_id], vals, context=context)
            return True

        # resultado get_rastreio, dicionario com data_hora,pais,estado,cidade,agencia,situacao,observacao
        for servico_code in pacotes.keys():
            tracks = []
            for track_number in pacotes.get(servico_code).keys():
                tracks.append(track_number)
            if len(tracks) == 1:
                res = self.pool.get('rastreio.servicos').rassvr_getRastreioUm(cr, uid, servico_code, tracks[0],
                                                                              context=context)
            else:
                res = self.pool.get('rastreio.servicos').rassvr_getRastreioVarios(cr, uid, servico_code, tracks, context=context)
            if res and res.get('dados'):
                for codigo in res.get('dados').keys():
                    even = res.get('dados').get(codigo)
                    __incluir_evento(codigo, even)

    def rasenc_cron_rastrear(self, cr, uid, context=None):
        sql = """
            select ec.id, ec.name, svr.rassvr_codigo, m2m.servico_id from rastreio_encomendas_m2m_servicos m2m
              left join rastreio_encomendas ec on ec.id = m2m.encomenda_id
              left join rastreio_servicos svr on svr.id = m2m.servico_id
              where rasenc_entregue = False
        """
        cr.execute(sql)
        _encomendas = cr.dictfetchall()
        pacotes = {}
        for _dEncomenda in _encomendas:
            if not pacotes.get(_dEncomenda.get('rassvr_codigo'), False):
                pacotes[_dEncomenda.get('rassvr_codigo')] = {}
            pacotes[_dEncomenda.get('rassvr_codigo')][_dEncomenda.get('name')] = {
                'enc_id': _dEncomenda.get('id'),
                'serv_id': _dEncomenda.get('servico_id')
            }
        if pacotes:
            self.rasenc_rastrear_varios_itens(cr, uid, pacotes, context=context)
        return True

    def rasenc_cron_limpar(self, cr, uid, prazo, context=None):
        _data = str(datetime.now() - timedelta(days=prazo))[:10]
        sql = """
            select id from rastreio_encomendas
              where rasenc_entregue = True and rasenc_data_ultimo_evento < '%s'
        """ % _data
        cr.execute(sql)
        _encomenda_ids = tuple(map(lambda x: x[0], cr.fetchall()))
        if _encomenda_ids:
            sql = """
                 delete from rastreio_encomendas where id in %s
            """ % str(_encomenda_ids)
            cr.execute(sql)
        return True

    def rasenc_button_rastrear(self, cr, uid, ids, context=None):
        pacotes = {}
        for brw_encomenda in self.browse(cr, uid, ids, context=context):
            for brw_servico in brw_encomenda.rasenc_servicos_ids:
                if not pacotes.get(brw_servico.rassvr_codigo, False):
                    pacotes[brw_servico.rassvr_codigo] = {}
                pacotes[brw_servico.rassvr_codigo][brw_encomenda.name] = {'enc_id': brw_encomenda.id, 'serv_id': brw_servico.id}
        self.rasenc_rastrear_varios_itens(cr, uid, pacotes, context=context)
        return True

