#-*- encoding: utf-8 -*-
"""
API Python Rastreio - Rastreamento de Encomendas
Copyright (C) 2012  Leandro T. de Souza <leandrotoledo@member.fsf.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from openerp import pooler
from openerp.tools.translate import _
from openerp.osv import osv
import json
import requests
import time
import base64
import re
import logging
_logger = logging.getLogger('Correios')

padrao_correios = r'^[A-Z]{2}\d{9}[A-Z]{2}$'


class Correios(object):

    def __init__(self, cr, uid, context=None):
        self.cr = cr
        self.uid = uid
        self.context = context or {}
        _pooler = pooler.get_pool(self.cr.dbname).get

        par_ids = _pooler('rastreio.correios.parametro').search(cr, uid, [], limit=1, context=self.context)
        parBrw = _pooler('rastreio.correios.parametro').browse(cr, uid, par_ids[0], context=self.context)
        self.username = parBrw.rascor_usuario
        self.password = parBrw.rascor_senha
        self.codigoAcesso = parBrw.rascor_codigo_acesso
        self.cartaoPostagem = parBrw.rascor_cartao_postagem
        self.token = parBrw.rascor_token
        self.param_id = parBrw.id
        self.baseUrl = 'https://api.correios.com.br'

        # retorno
        self.status = True
        self.mensagem = ''
        self.dados = {}
        self.httpErro = None

        if not any([self.username, self.password, self.codigoAcesso]):
            self.status = False
            self.mensagem = _('Falta a configuracao dos parametros dos Correios!')
        if not self.token:
            self.getToken()
        self.status = True
        self.dados = {}

    def correiosComunica(self, url, metodo, json=None, data=None, params=None, headers=None):
        def comunicar():
            try:
                _logger.info('===> Enviar(%s) Headers: %s Json: %s   Data:%s   Params:%s' % (metodo, headers, json, data, params))
                response = requests.request(metodo, uri, json=json, data=data, params=params, headers=headers)
                self.httpErro = response.status_code
                _logger.info('==> Resposta: %s, %s' % (response.status_code, response.text))
                return response
            except:
                pass

        if not params:
            params = {}
        if not json:
            json = {}
        if not headers:
            headers = {}

        uri = self.baseUrl + url
        metodo = metodo.upper()
        t = 0
        while True:
            response = comunicar()
            if response.status_code == 429:
                time.sleep(1)
            elif response.status_code == 403:
                t += 1
                if t == 1:
                    self.getToken()
                else:
                    break
            else:
                break
        return response

    def getToken(self):
        url = '/token/v1/autentica/cartaopostagem'
        _basic = '%s:%s' % (self.username.strip(), self.codigoAcesso.strip())
        headers = {
            'Authorization': 'Basic %s' % base64.b64encode(_basic)
        }
        vals = {
            "numero": self.cartaoPostagem,
        }

        res = self.correiosComunica(url, 'POST', json=vals, headers=headers)
        if res.ok:
            _dados = json.loads(res.text)
            self.token = _dados.get('token')
            _pooler = pooler.get_pool(self.cr.dbname).get
            vals = {
                'rascor_token': self.token
            }
            _pooler('rastreio.correios.parametro').write(self.cr, self.uid, [self.param_id], vals, context=self.context)
            self.status = True
            self.dados = {}
            return
        self.status = False
        self.mensagem = _('Nao consegui pegar o token dos Correios!')

    def getRastroUm(self, codigo_objeto):
        url = '/srorastro/v1/objetos/%s' % codigo_objeto
        headers = {
            'Authorization': 'Bearer %s' % self.token
        }
        vals = {
            "resultado": 'T',
        }
        res = self.correiosComunica(url, 'GET', headers=headers, params=vals)
        if res.ok:
            dados = json.loads(res.text)
            self.status = True
            self.dados = dados
            return
        self.status = False
        self.mensagem = _('Nao consegui pegar o rastreio!')
        return

    def getRastroVarios(self, lista_objetos):
        def getLista(_codigos):
            _codigos = _codigos[:-1]
            vals = {
                'codigosObjetos': _codigos,
                "resultado": 'T',
            }
            res = self.correiosComunica(url, 'GET', headers=headers, params=vals)
            if not res.ok:
                self.status = False
                self.mensagem = _('Nao consegui pegar o rastreio!')
                return
            for l in json.loads(res.text).get('objetos'):
                dados.append(l)

        url = '/srorastro/v1/objetos'
        headers = {
            'Authorization': 'Bearer %s' % self.token
        }
        dados = []
        i = 0
        _codigos = ''
        for x in lista_objetos:
            _codigos += x + ','
            i += 1
            if i < 50:
                continue
            getLista(_codigos)
            i = 0
            _codigos = ''
        if _codigos:
            getLista(_codigos)
        self.status = True
        self.dados = dados
        return


class rastreio_correios(osv.Model):
    _inherit = 'rastreio.servicos'

    def rassvr_getRastreioUm(self, cr, uid, codTranp, objeto, context=None):
        res = super(rastreio_correios, self).rassvr_getRastreioUm(cr, uid, codTranp, objeto, context=context)
        if codTranp != 'CorBra':
            return res
        if not re.match(padrao_correios, objeto):
            return {'status': False, 'mensagem': _('Codigo de objeto invalido para os Correios')}
        c = Correios(cr, uid, context=context)
        if not c.status:
            return {'status': c.status, 'mensagem': c.mensagem}
        c.getRastroUm(objeto)
        if not c.status:
            return {'status': c.status, 'mensagem': c.mensagem}
        dados = {}
        for linha in c.dados.get('objetos'):
            if linha.get('mensagem'):
                dados[c.dados.get('codObjeto')] = []
                continue
            eventos = []
            for evento in linha.get('eventos'):
                if evento.get('unidade').get('tipo') == u'País':
                    _pais = evento.get('unidade').get('nome')
                else:
                    _pais = ''
                if evento.get('codigo') in (u'BDE', u'BDI', u'BDR') and evento.get('tipo') in ('00', '01'):
                    _fim = True
                else:
                    _fim = False
                obs = ''
                if evento.get('unidadeDestino'):
                    _end = evento.get('unidadeDestino').get('endereco')
                    if _end:
                        obs = 'Encaminhado p/%s-%s' % (_end.get('cidade'), _end.get('uf'))
                eventos.append({
                    'data_hora': evento.get('dtHrCriado'),
                    'situacao': evento.get('descricao'),
                    'pais': _pais,
                    'estado': evento.get('unidade').get('endereco').get('uf'),
                    'cidade': evento.get('unidade').get('endereco').get('cidade'),
                    'fim': _fim,
                    # 'agencia': c.dados.get('objetos').get(''),
                    'observacao': obs,
                })
            dados[linha.get('codObjeto')] = eventos
        return {'status': True, 'dados': dados}

    def rassvr_getRastreioVarios(self, cr, uid, codTranp, listaObjetos, context=None):
        res = super(rastreio_correios, self).rassvr_getRastreioVarios(cr, uid, codTranp, listaObjetos, context=context)
        if codTranp != 'CorBra':
            return res
        c = Correios(cr, uid, context=context)
        if not c.status:
            return {'status': c.status, 'mensagem': c.mensagem}
        _novaLista = []
        for x in listaObjetos:
            if not re.match(padrao_correios, x):
                continue
            _novaLista.append(x)
        listaObjetos = _novaLista

        c.getRastroVarios(listaObjetos)
        if not c.status:
            return {'status': c.status, 'mensagem': c.mensagem}
        dados = {}
        for linha in c.dados:
            if linha.get('mensagem'):
                dados[linha.get('codObjeto')] = []
                continue
            eventos = []
            for evento in linha.get('eventos'):
                if evento.get('unidade').get('tipo') == u'País':
                    _pais = evento.get('unidade').get('nome')
                else:
                    _pais = ''
                if evento.get('codigo') in (u'BDE', u'BDI', u'BDR') and evento.get('tipo') in ('00', '01'):
                    _fim = True
                else:
                    _fim = False
                eventos.append({
                    'data_hora': evento.get('dtHrCriado'),
                    'situacao': evento.get('descricao'),
                    'pais': _pais,
                    'estado': evento.get('unidade').get('endereco').get('uf'),
                    'cidade': evento.get('unidade').get('endereco').get('cidade'),
                    'fim': _fim
                })
            dados[linha.get('codObjeto')] = eventos
        return {'status': True, 'dados': dados}

