# -*- encoding: utf-8 -*-

from openerp.osv import fields, osv
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from openerp.tools.translate import _


class rat_rat(osv.Model):

    _inherit = "rat.rat"

    def set_finalizar(self, cr, uid, ids, context=None):

        """
        Realiza a alteracao do estado do rat para 'Finalizado' ('done'), caso o rat possua
        atividades com usuario cadastrado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """

        context = context and context.copy() or {}
        res = super(rat_rat, self).set_finalizar(cr, uid, ids, context)
        if res is not True:
            return res
        for rat in self.browse(cr, uid, ids, context):
            brw_faturar_task_ids = map(lambda i: i.atividade_id,
                                       filter(lambda j: j.atividade_id.prjtsk_a_faturar,
                                              rat.atividade_rat_ids))
            if not brw_faturar_task_ids:
                continue
            valor_hora = brw_faturar_task_ids[0].project_br_id.prjprj_valor_hora or 0
            tempo_total_gasto = sum(map(lambda i: i.effective_hours or 0, brw_faturar_task_ids))
            valor_total = valor_hora * tempo_total_gasto
            if not valor_total:
                continue
            self.ratrat_cria_previsao_financeira(cr, uid, rat.id, valor_total, context)
        return res

    def ratrat_reabrir_rat(self, cr, uid, ids, context=None):
        """
        Reabre o RAT ja concluido.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = super(rat_rat, self).ratrat_reabrir_rat(cr, uid, ids, context)
        self.ratrat_deleta_previsao_financeira(cr, uid, ids, context)
        return res

    def ratrat_cria_previsao_financeira(self, cr, uid, rat_id, valor_total, context=None):
        """
        O metodo cria a previsao financeira referente as tarefas do rat que serao faturadas.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param rat_id: id da tabela desse objeto.
        :type rat_id: int
        :param valor_total: total a ser faturado no rat.
        :type valor_total: float
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: id da tabela financeiro_titulo, identificador da previsao financeira.
        :rtype: int
        """
        context = context and context.copy() or {}
        prazo_rat = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', 'finpar_prazo_rat', context=context).get('finpar_prazo_rat')
        doc_tipo_ids = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)
        brw_rat = self.browse(cr, uid, rat_id, context)
        vencimento = parse(brw_rat.data_fim) + relativedelta(days=prazo_rat)
        vencimento = vencimento.strftime('%Y-%m-%d')
        context['fintit_titulo_sem_custificacao'] = True
        res = self.pool.get('financeiro').financ_titulo_entrada(
            cr, uid, brw_rat.cliente_id.id, brw_rat.data_fim[:10], vencimento,
            str(brw_rat.id)+'RAT', 1, valor_total, 'A', 'R', fult_tit_doc_tipo_id=doc_tipo_ids[0],
            context=context)
        return res

    def ratrat_deleta_previsao_financeira(self, cr, uid, ids, context=None):
        """
        O metodo deleta as previsoes financeiras referente as tarefas dos rats que serao reabertos.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso possua previsões e elas sejam excluidas. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False
        obj_financeiro_titulo = self.pool.get('financeiro.titulo')
        brw_rat = self.browse(cr, uid, ids, context)
        list_documentos = list()
        map(lambda i: list_documentos.append(str(i.id)+'RAT'), brw_rat)
        pessoa_ids = list()
        map(lambda i: pessoa_ids.append(i.cliente_id.id), brw_rat)
        doc_tipo_ids = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'PRV')], context=context)
        previsao_titulo_ids = obj_financeiro_titulo.search(
            cr, uid, [('documento', 'in', list_documentos), ('tipo_titulo', '=', 'R'),
                      ('doc_tipo_id', 'in', doc_tipo_ids), ('pessoa_id', 'in', pessoa_ids)],
            context=context)
        if not previsao_titulo_ids:
            return res
        res = obj_financeiro_titulo.unlink(cr, uid, previsao_titulo_ids, context)
        return res

    def set_avancar_setor(self, cr, uid, ids, context=None):
        """
        O metodo disponibiliza o rat para os usuarios que pertencem ao proximo setor da sequencia.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso o rat seja atualizado para o proximo setor. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = super(rat_rat, self).set_avancar_setor(cr, uid, ids, context)
        rat_id = ids[0] if isinstance(ids, list) else ids
        pedido_ids = self.pool.get('estoquepedidos.pedido.capa').search(
            cr, uid, [('observacao', '=', 'Numero do RAT: %s.' % rat_id)])
        proximo_setor_id = self.read(cr, uid, rat_id, ['ratrat_proximo_setor_id'], context,
                                     '_classic_write').get('ratrat_proximo_setor_id')
        if not proximo_setor_id and not pedido_ids:
            self.ratrat_deleta_previsao_financeira(cr, uid, ids, context)
            self.ratrat_cria_pedido_venda(cr, uid, ids, context)
        return res

    def ratrat_cria_pedido_venda(self, cr, uid, ids, context=None):
        """
        O metodo cria o pedido de venda validado para o RAT.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso o pedido seja criado e validado. Senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False

        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_estoquepedidos_pedido_condicao = self.pool.get('estoquepedidos.pedido.condicao')
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_project_br_task = self.pool.get('project_br.task')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')

        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['local_venda_padrao_id', 'estpar_embalagem_servico_id'], context)
        local_id = estoque_parametro_dados.get('local_venda_padrao_id')
        embalagem_id = estoque_parametro_dados.get('estpar_embalagem_servico_id')
        if not embalagem_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre a "Embalagem para servicos" nos '
                                                 'parametros do estoque.'))

        forma_pagamento_ids = self.pool.get('adm.forma.pgto').search(
            cr, uid, [('sigla', '=', 'DN')], context=context)
        condicao_forma_ids = self.pool.get('adm.condicao.forma').search(
            cr, uid, [('forma_pagamento_id', 'in', forma_pagamento_ids)], context=context)

        msg = ''
        for rat in self.browse(cr, uid, ids, context):
            endereco_id = False
            pessoa_endereco_ids = obj_adm_pessoa_endereco.search(
                cr, uid, [('pessoa_id', '=', rat.cliente_id.id)], context=context)
            if pessoa_endereco_ids:
                endereco_id = pessoa_endereco_ids[0]

            vals_pedido = {'pessoa_id': rat.cliente_id.id, 'pedcap_endereco_id': endereco_id,
                           'tipo': 'V', 'observacao': 'Numero do RAT: %s.' % rat.id}
            pedido_id = obj_estoquepedidos_pedido_capa.create(cr, uid, vals_pedido, context)
            for item in rat.atividade_rat_ids:

                vendedor_ids = obj_vendas_vendedor.search(
                    cr, uid, [('pessoa_id', '=', item.user_id.pessoa_id.id)])
                if not vendedor_ids:
                    vendedor_id = obj_vendas_vendedor.create(
                        cr, uid, {'pessoa_id': item.user_id.pessoa_id.id}, context)
                else:
                    vendedor_id = vendedor_ids[0]

                if not item.atividade_id.prjtsk_a_faturar:
                    continue
                if not item.atividade_id.prjtsk_produto_id:
                    msg += '%s, ' % item.atividade_id.id
                    continue
                grade_ids = obj_estoque_produto_grade.search(
                    cr, uid,
                    [('produto_id', '=', item.atividade_id.prjtsk_produto_id.id)],
                    context=context)
                produto_embalagem_ids = obj_estoque_produto_embalagem.search(
                    cr, uid,
                    [('produto_id', '=', item.atividade_id.prjtsk_produto_id.id)],
                    context=context)
                if produto_embalagem_ids:
                    produto_embalagem_id = produto_embalagem_ids[0]
                else:
                    produto_embalagem_id = obj_estoque_produto_embalagem.create(
                        cr, uid,
                        {'produto_id': item.atividade_id.prjtsk_produto_id.id,
                         'embalagem_id': embalagem_id}, context)
                quantidade = item.rat_id.ratrat_tempo_atendimento

                vals_pedido_item = {
                    'pedido_id': pedido_id,
                    'qtde_pedida': quantidade,
                    'vr_unitario': item.atividade_id.project_br_id.prjprj_valor_hora,
                    'produto_id': item.atividade_id.prjtsk_produto_id.id,
                    'grade_id': grade_ids[0],
                    'local_id': local_id,
                    'produto_embalagem_id': produto_embalagem_id,
                    'vendedor_id': vendedor_id}
                pedido_item_id = obj_estoquepedidos_pedido_item.create(
                    cr, uid, vals_pedido_item, context)
                obj_project_br_task.write(
                    cr, uid, item.atividade_id.id,
                    {'prjtsk_pedido_item_id': pedido_item_id}, context)

            obj_estoquepedidos_pedido_capa.write(cr, uid, pedido_id, {}, context)
            vr_total_pedido = obj_estoquepedidos_pedido_capa.browse(cr, uid, pedido_id,
                                                                    context).valor_total or 0
            obj_estoquepedidos_pedido_condicao.create(
                cr, uid, {'capa_id': pedido_id, 'forma_pagto_id': forma_pagamento_ids[0],
                          'condicao_forma_id': condicao_forma_ids[0], 'valor': vr_total_pedido},
                context)
            obj_estoquepedidos_pedido_capa.calcular_parcelas_pedido(cr, uid, pedido_id, context)
            validado = obj_estoquepedidos_pedido_capa.button_validar(cr, uid, pedido_id, context)
            if validado:
                res = True

        if msg:
            raise osv.except_osv(_('Atencao'), _('As tarefas de numeros a seguir nao possuem '
                                                 'servico cadastrado: %s.' % msg[:-2]))
        return res
