# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_rat_faturamento(osv.TransientModel):
    # prefixo: wratfa_

    _name = 'wizard.rat.faturamento'

    _columns = {
        'wratfa_data_inicial': fields.date('Data inicial', required=True),
        'wratfa_data_final': fields.date('Data final', required=True),
        'wratfa_item_ids': fields.one2many('wizard.rat.faturamento.item', 'wratfi_capa_id',
                                           'Itens'),
        'wratfa_pesquisado': fields.boolean('Pesquisado'),
        'wratfa_faturado': fields.boolean('Faturado'),
    }

    _defaults = {
        'wratfa_pesquisado': lambda *a: False,
        'wratfa_faturado': lambda *a: False
    }

    def wratfa_button_find(self, cr, uid, ids, context=None):
        """
        O metodo encontra todos os rats (rat_rat) que se encontram dentro do periodo informado e
        que possuem tarefas a faturar.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso encontre algum rat, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False
        cr.execute('DELETE FROM wizard_rat_faturamento_item')
        for x in self.browse(cr, uid, ids, context):
            sql = """
                SELECT
                    rr.id                                                    AS wratfi_rat_id,
                    rr.cliente_id                                            AS wratfi_pessoa_id,
                    rr.data_inicio                                           AS wratfi_data_inicio,
                    rr.data_fim                                              AS wratfi_data_fim,
                    rr.user_id                                               AS wratfi_user_id,
                    round(Sum(pt.effective_hours * pb.prjprj_valor_hora), 2) AS wratfi_valor
                FROM rat_rat rr
                INNER JOIN rat_atividade ra
                    ON rr.id = ra.rat_id
                INNER JOIN project_br_task pt
                    ON ra.atividade_id = pt.id
                INNER JOIN project_br_project_br pb
                    ON pb.id = pt.project_br_id
                WHERE
                    rr.state = 'done'
                    AND rr.data_inicio >= '%s 00:00:00'
                    AND rr.data_fim <= '%s 23:59:59'
                    AND pt.prjtsk_a_faturar = TRUE
                    AND (pt.prjtsk_pedido_item_id IS NULL OR pt.prjtsk_pedido_item_id IN (
                        SELECT id FROM estoquepedidos_pedido_item WHERE pedido_id IN (
                            SELECT id FROM estoquepedidos_pedido_capa WHERE state = 'C')))
                GROUP BY
                    rr.id, rr.cliente_id, rr.data_inicio, rr.data_fim, rr.user_id
            """ % (x.wratfa_data_inicial, x.wratfa_data_final)
            cr.execute(sql)
            rat_dados = cr.dictfetchall()
            if not rat_dados:
                continue
            vals = []
            map(lambda i: vals.append((0, 0, i)), rat_dados)
            res = self.write(cr, uid, x.id, {'wratfa_item_ids': vals, 'wratfa_pesquisado': True},
                             context)
        return res

    def wratfa_button_faturar(self, cr, uid, ids, context=None):
        """
        O metodo vai gerar os pedidos de venda por cliente para todos os rats encontrados.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso gere os pedidos de venda, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}

        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_project_br_task = self.pool.get('project_br.task')
        obj_vendas_vendedor = self.pool.get('vendas.vendedor')

        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', ['local_venda_padrao_id', 'estpar_embalagem_servico_id'], context)
        local_id = estoque_parametro_dados.get('local_venda_padrao_id')
        embalagem_id = estoque_parametro_dados.get('estpar_embalagem_servico_id')
        if not embalagem_id:
            raise osv.except_osv(_('Atencao'), _('Cadastre a "Embalagem para servicos" nos '
                                                 'parametros do estoque.'))
        res = False
        msg = ''
        for x in self.browse(cr, uid, ids, context):
            pessoa_ids = list(set(map(lambda i: i.wratfi_pessoa_id.id, x.wratfa_item_ids)))
            for pessoa_id in pessoa_ids:
                rat_faturamento_itens = filter(lambda i: i.wratfi_pessoa_id.id == pessoa_id,
                                               x.wratfa_item_ids)
                endereco_id = False
                pessoa_endereco_ids = obj_adm_pessoa_endereco.search(
                    cr, uid, [('pessoa_id', '=', pessoa_id)], context=context)
                if pessoa_endereco_ids:
                    endereco_id = pessoa_endereco_ids[0]
                vals_pedido = {'pessoa_id': pessoa_id, 'pedcap_endereco_id': endereco_id,
                               'tipo': 'V'}
                pedido_id = obj_estoquepedidos_pedido_capa.create(cr, uid, vals_pedido, context)
                obs = ''
                for item in rat_faturamento_itens:
                    rat_atividade_ids = item.wratfi_rat_id.atividade_rat_ids
                    obs += '%s, ' % item.wratfi_rat_id.id

                    vendedor_id = False
                    vendedor_ids = obj_vendas_vendedor.search(
                        cr, uid, [('pessoa_id', '=', item.wratfi_user_id.pessoa_id.id)])
                    if not vendedor_ids:
                        vendedor_id = obj_vendas_vendedor.create(
                            cr, uid, {'pessoa_id': item.wratfi_user_id.pessoa_id.id}, context)
                    else:
                        vendedor_id = vendedor_ids[0]

                    for atividade in rat_atividade_ids:
                        if not atividade.atividade_id.prjtsk_a_faturar:
                            continue
                        if not atividade.atividade_id.prjtsk_produto_id:
                            msg += '%s, ' % atividade.atividade_id.id
                            continue
                        grade_ids = obj_estoque_produto_grade.search(
                            cr, uid,
                            [('produto_id', '=', atividade.atividade_id.prjtsk_produto_id.id)],
                            context=context)
                        produto_embalagem_ids = obj_estoque_produto_embalagem.search(
                            cr, uid,
                            [('produto_id', '=', atividade.atividade_id.prjtsk_produto_id.id)],
                            context=context)
                        if produto_embalagem_ids:
                            produto_embalagem_id = produto_embalagem_ids[0]
                        else:
                            produto_embalagem_id = obj_estoque_produto_embalagem.create(
                                cr, uid,
                                {'produto_id': atividade.atividade_id.prjtsk_produto_id.id,
                                 'embalagem_id': embalagem_id}, context)
                        # quantidade = atividade.atividade_id.effective_hours
                        quantidade = item.wratfi_rat_id.ratrat_tempo_atendimento

                        vals_pedido_item = {
                            'pedido_id': pedido_id,
                            'qtde_pedida': quantidade,
                            'vr_unitario': atividade.atividade_id.project_br_id.prjprj_valor_hora,
                            'produto_id': atividade.atividade_id.prjtsk_produto_id.id,
                            'grade_id': grade_ids[0],
                            'local_id': local_id,
                            'produto_embalagem_id': produto_embalagem_id,
                            'vendedor_id': vendedor_id}
                        pedido_item_id = obj_estoquepedidos_pedido_item.create(
                            cr, uid, vals_pedido_item, context)
                        obj_project_br_task.write(
                            cr, uid, atividade.atividade_id.id,
                            {'prjtsk_pedido_item_id': pedido_item_id}, context)

                if obs:
                    obj_estoquepedidos_pedido_capa.write(
                        cr, uid, pedido_id, {'observacao': 'Numero dos RATs: %s.' % obs[:-2]},
                        context)
        if msg:
            raise osv.except_osv(_('Atencao'), _('As tarefas de numeros a seguir nao possuem '
                                                 'servico cadastrado: %s.' % msg[:-2]))
        self.write(cr, uid, ids, {'wratfa_faturado': True}, context)
        return res


class wizard_rat_faturamento_item(osv.TransientModel):
    # prefixo: wratfi_

    _name = 'wizard.rat.faturamento.item'

    _columns = {
        'wratfi_capa_id': fields.many2one('wizard.rat.faturamento', 'Capa'),
        'wratfi_rat_id': fields.many2one('rat.rat', 'Numero'),
        'wratfi_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),
        'wratfi_data_inicio': fields.datetime('Data inicio'),
        'wratfi_data_fim': fields.datetime('Data fim'),
        'wratfi_user_id': fields.many2one('res.users', 'Consultor'),
        'wratfi_valor': fields.float('Valor', digits=(15, 2)),
    }