# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _
import time


class recebimento_divergencia_nf(osv.Model):
    """
    Divergencia no recebimento de notas fiscais.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.divergencia.nf'  # recdnf
    # Veja: recebimento/control/divergencia_nf, openerp/addons/base/ir/ir.needaction
    _inherit = ['recebimento.divergencia.nf', 'ir.needaction_mixin']

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de ordens de servico com ´´status´´ igual a "divergente"
        no menu Ordem de servico.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} dominio utilizado para contar as Ordens de Servico com
        ´status´  igual a 'aberta'.

        @attention: e necessario que a classe "ir.needaction_mixin" seja car-
        regada em "_inherit". exemplo: _inherit = '[ir.needaction_mixin]'
        @warning: nao coloque prefixo ou troque o nome deste metodo!

        @see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        @copyright: OpenERP SA.
        """

        context = context and context.copy() or {}
        return [('state', 'in', ['DV'])]

    def recdnf_aceitar_divergencia(self, cr, uid, ids, context=None):
        """
        Aceita as divergencias na nota fiscal.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {boolean} verdadeiro ou mensagem de aviso de que o motivo de
        aceite da divergecia deve ser informado.
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')

        campos = ['observacao', 'nf_id']

        dados_divergencia = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_divergencia and isinstance(dados_divergencia, list):
            dados_divergencia = dados_divergencia[0]

        if dados_divergencia and isinstance(dados_divergencia, dict):
            # Verifica o campo observacao. Nao pode ser vazio se divergencia
            # esta sendo aceita ou recusada.
            observacao = dados_divergencia.get('observacao') or ''

            if not len(observacao.strip()):
                msg = _('O campo de observacao deve conter o motivo de aceite da divergencia.')

                raise osv.except_osv(_('Atencao!'), msg)

            dados = {'state': 'A',  # Divergencias aceitas
                     'data_acao': time.strftime('%Y-%m-%d %H:%M:%S'),
                     'usuario_autorizador_id': uid}

            self.write(cr, uid, ids, dados, context)

            # Ajusta o ´´status´´ da nota fiscal. Troca de "divergente" para "validada"
            nf_id = dados_divergencia.get('nf_id')
            context['salvar_forcado'] = True  # Forca a alteracao da nota fiscal
            obj_fiscal_nf.write(cr, uid, nf_id, {'state': '2'}, context)

        return True

    def recdnf_recusar_divergencia(self, cr, uid, ids, context=None):
        """
        Recusa divergencias na nota fiscal.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {boolean} verdadeiro ou mensagem de aviso de que o motivo de
        recusa da divergencia deve ser informado.
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        campos = ['observacao', 'nf_id']

        dados_divergencia = self.read(cr, uid, ids, campos, context=context)

        if dados_divergencia and isinstance(dados_divergencia, list):
            dados_divergencia = dados_divergencia[0]

        if dados_divergencia and isinstance(dados_divergencia, dict):
            # Verifica o campo observacao. Nao pode ser vazio se divergencia
            # esta sendo aceita ou recusada.
            if not dados_divergencia.get('observacao'):
                msg = _('O campo de observacao deve conter o motivo de recusa da divergencia.')

                raise osv.except_osv(_('Atencao!'), msg)

            # Nota fiscal recusada
            dados = {'state': 'R', 'data_acao': time.strftime('%Y-%m-%d %H:%M:%S'),
                     'usuario_autorizador_id': uid}

            self.write(cr, uid, ids, dados, context=context)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
