# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class recebimento_itens(osv.Model):
    """
    Itens de recebimento.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.itens'  # recite
    _inherit = 'recebimento.itens'  # recebimento/control/recebimento_itens

    @staticmethod
    def recite_onchange_qtde_contada(cr, uid, ids, qtde_nf, qtde_contada, qtde_avaria):
        """
        Verifica se soma das quantidades contada e avaria ultrapassam a quanti-
        dade total registrada na nota fiscal.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuario utilizado para a verificacao de per-
        fil,
        @param ids: {list} lista dos IDs,
        @param qtde_nf:  {int} a quantidade total do item na nota fiscal,
        @param qtde_contada: {int} a quantidade que foi informada na contagem,
        @param qtde_avaria: {int} a quantidade contada de itens avariados,
        @return:

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {}

        if qtde_nf != qtde_contada + qtde_avaria:
            titulo = 'Aviso.'
            msg = 'A soma das quantidades (contada + avariada) difere da quantidade na nota fiscal.'

            result = {'warning': {'title': titulo, 'message': msg}}

        return result

    @staticmethod
    def recite_onchange_qtde_avaria(cr, uid, ids, qtde_nf, qtde_contada, qtde_avaria):
        """
        Verifica se soma das quantidades contada e avaria ultrapassam a quanti-
        dade total registrada na nota fiscal.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param qtde_nf:  {int} a quantidade total do item na nota fiscal,
        @param qtde_contada: {int} a quantidade que foi informada na contagem,
        @param qtde_avaria: {int} a quantidade contada de itens avariados,
        @return: {dict} valor do ´status´ do item de recebimento.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {}

        if qtde_nf != qtde_contada + qtde_avaria:
            titulo = 'Aviso.'
            msg = 'A soma das quantidades (contada + avaria) difere do total da nota fiscal.'

            result = {'warning': {'title': titulo, 'message': msg}}

        return result

    def recite_verifica_divergencias(self, cr, uid, ids, context=None):
        """
        Verifica se ha divergencia na contagem dos itens.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} lista com os IDs dos itens com divergencia.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = []  # Lista de itens divergentes

        for item in self.browse(cr, uid, ids):
            if item.quantidade != (item.qtde_contada + item.qtde_avaria):
                result.append(item.id)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
