# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class fiscal_nf_capa(osv.Model):
    """
    Verificacao de notas fiscais.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'fiscal.nf.capa'  # fisnfc
    _inherit = 'fiscal.nf.capa'  # fiscal/control/nf_capa

    # TODO: Ajustar o nome do metodo incluindo o prefixo.
    def button_verificar_nf(self, cr, uid, nf, context=None):
        """
        Verifica (por heranca) e faz o registro das divergencias da nota fiscal
        em relacao aos pedidos.
        Apenas para notas fiscais de entrada.

        @see: fiscal/control/nf_capa.py
        @attention: heranca da classe ´fiscal.nf.capa´

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        obj_recebimento_divergencia_nf = self.pool.get('recebimento.divergencia.nf')
        obj_recebimento_divergencia_itens = self.pool.get('recebimento.divergencia.itens')

        nf = nf[0] if isinstance(nf, list) else nf

        # Faz verificacao normal
        super(fiscal_nf_capa, self).button_verificar_nf(cr, uid, nf, context)

        # Contexto utilizado na identificacao de rotinas que envolvem o modulo de Recebimento
        context['recebimento'] = True

        # Recupera e consolida os itens da NF
        brw_nf = self.browse(cr, uid, nf, context)

        if brw_nf.tipo == 'E':
            itens_nf = {}

            for item in brw_nf.estoque_movimento_itens_ids:
                produto_id = item.produto_id.id
                grade_id = item.grade_id.id

                if itens_nf.get((produto_id, grade_id)):
                    itens_nf[(produto_id, grade_id)][0] += item.quantidade
                    itens_nf[(produto_id, grade_id)][1] += item.vr_total_item
                else:
                    itens_nf[(produto_id, grade_id)] = [item.quantidade, item.vr_total_item,
                                                        item.cfop_id.codigo]

            # Recupera e consolida os itens do pedido
            itens_pedido = {}

            for pedido in brw_nf.fisnfc_pedido_ids:
                for item_pedido in pedido.pedido_item_ids:
                    produto_id = item_pedido.produto_id.id
                    grade_id = item_pedido.grade_id.id
                    quantidade = item_pedido.qtde_pedida
                    valor = item_pedido.valor_total

                    if itens_pedido.get((produto_id, grade_id)):
                        itens_pedido[(produto_id, grade_id)][0] += quantidade
                        itens_pedido[(produto_id, grade_id)][1] += valor
                    else:
                        itens_pedido[(produto_id, grade_id)] = [quantidade, valor]

            # Verifica se houve algum tipo de divergencia
            divergencias = []  # lista com os IDs das Notas Fiscais com divergencia

            for item in itens_nf:
                if itens_pedido.get(item):
                    # Divergencia na quantidade NF x pedido
                    if itens_nf[item][0] != itens_pedido[item][0]:
                        divergencias.append({'produto_id': item[0], 'grade_id': item[1],
                                             'qtde_pedida': itens_pedido[item][0],
                                             'valor_item_pedido': itens_pedido[item][1],
                                             'qtde_nf': itens_nf[item][0],
                                             'valor_item_nf': itens_nf[item][1],
                                             'tipo_divergencia': 'DV'})
                else:
                    # TODO: O codigo CFOP de bonificacao deve ser parametro do sistema
                    # Excedente se nao for NF de bonificacao (CFOP = 1910)
                    if itens_nf[item][2] not in ('1910', '2910', '1911', '2911'):
                        divergencias.append({'produto_id': item[0], 'grade_id': item[1],
                                             'qtde_pedida': False, 'valor_item_pedido': False,
                                             'qtde_nf': itens_nf[item][0],
                                             'valor_item_nf': itens_nf[item][1],
                                             'tipo_divergencia': 'E'})

            # Verifica se falta na Nota
            for item in itens_pedido:
                if not itens_nf.get(item):  # Falta
                    divergencias.append({'produto_id': item[0], 'grade_id': item[1],
                                         'qtde_pedida': itens_pedido[item][0],
                                         'valor_item_pedido': itens_pedido[item][1],
                                         'qtde_nf': False, 'valor_item_nf': False,
                                         'tipo_divergencia': 'F'})

            # Se houver divergencia grava as divergencias e muda o status da nota fiscal
            if divergencias:
                divergencia_id = obj_recebimento_divergencia_nf.create(
                    cr, uid, {'nf_id': brw_nf.id}, context)

                for divergencia in divergencias:
                    dados = {'divergencia_nf_id': divergencia_id,
                             'produto_id': divergencia.get('produto_id'),
                             'grade_id': divergencia.get('grade_id'),
                             'qtde_pedida': divergencia.get('qtde_pedida'),
                             'qtde_nf': divergencia.get('qtde_nf'),  # Transformada em unidades
                             'valor_item_pedido': divergencia.get('valor_item_pedido'),
                             'valor_item_nf': divergencia.get('valor_item_nf'),
                             'tipo_divergencia': divergencia.get('tipo_divergencia')}
                    obj_recebimento_divergencia_itens.create(cr, uid, dados, context)

                msg = 'Nota fiscal de entrada nr %s de %s verificada com ' \
                      'divergencia.' % (brw_nf.numero_nf, brw_nf.data_emissao)
                self.log(cr, uid, brw_nf.id, msg)

                dados = {'state': 'DV'}
                obj_fiscal_nf_capa.write(cr, uid, brw_nf.id, dados, context)

        return True

    def fisnfc_button_recusar_nf(self, cr, uid, ids, context=None):
        """
        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @return: {boolean} verdadeiro se o ´´status´´  da nota fiscal foi alterado para "R"
        (recusada).

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids, {'state': 'R'}, context)

        return result

    def fisnfc_button_divergencia_nf(self, cr, uid, ids, context=None):
        """
        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @return: {boolean} verdadeiro se o ´status´  da nota fiscal foi alterado para "DV"
        (divergencia).

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids, {'state': 'DV'}, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
