# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""


# _______________________ os (ordem de servico) ______________________________
def recosv_tipo_padrao(self, cr, uid, context=None):
    """
    Define o tipo da OS / REC - Recebimento

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param context: {dict} um dicionario padrao para valores contextuais,

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    args = [('codigo', '=', 'REC')]
    os_tipo_id = self.pool.get('os.tipo').search(cr, uid, args, context=context)

    if os_tipo_id:
        return os_tipo_id[0]  # Tipo padrao = "Recebimento"

    return []


def recosv_codigo_padrao(self, cr, uid, context=None):
    """
    Define o codigo do tipo da ordem de servico / REC

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param context: {dict} um dicionario padrao para valores contextuais,

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    context = context and context.copy() or {}

    return 'REC'


def recosv_lista_nf(self, cr, uid, ids, name, arg, context=None):
    """
    Monta a lista dos numeros das Notas Fiscais

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param name: {list} lista com o nome dos campos,
    @param arg: {dict} dicionario com os argumentos e valores,
    @param context: {dict} um dicionario padrao para valores contextuais,

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    # ___________________________ Objects ________________________________
    obj_nf_capa = self.pool.get('fiscal.nf.capa')

    lista = ''
    nf_ids = self.read(cr, uid, ids[0], ['nf_ids'], context=context)
    nr_notas = obj_nf_capa.read(cr, uid, nf_ids['nf_ids'], ['numero_nf'], context=context)

    for nr_nota in nr_notas:
        lista += str(nr_nota['numero_nf']) + ', '

    return {ids[0]: lista[:len(lista) - 2]}


def recosv_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @param name: {list} lista com o nome dos campos,
    @param arg: {dict} dicionario com os argumentos e valores,
    @param context: {dict} um dicionario padrao para valores contextuais,
    @return: {dict} valores das chaves representando registros divergentes e
    nao divergentes.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for os in self.browse(cr, uid, ids):
        try:
            if name == 'recosv_recebimento_ok':  # Verifica se todos os itens foram recebidos (Ok)
                if not os.recebimento_item_ids or \
                        filter(lambda i: i.state != 'O',
                               os.recebimento_item_ids):
                    valor = False
                else:
                    valor = True
            elif name == 'recosv_divergencia':
                if os.recebimento_item_ids and filter(lambda i: i.state == 'DV',
                                                      os.recebimento_item_ids):
                    valor = True
                else:
                    valor = False
            elif name == 'lista_nf':  # Lista com os numeros das Notas Fiscais (compra)
                valor = ', '.join(map(lambda nf: str(nf.numero_nf), os.nf_ids))
            elif name == 'recosv_mesmo_conferente':
                valor = False
                campos = ['conferente_id', 'contagem', 'conferente_anterior_id']
                dados_os = self.read(cr, uid, os.id, campos, context=context, load='_classic_write')

                if dados_os and isinstance(dados_os, list):
                    dados_os = dados_os[0]

                if dados_os and isinstance(dados_os, dict):
                    conferente_id = dados_os.get('conferente_id')
                    conferente_anterior_id = dados_os.get('conferente_anterior_id')
                    contagem = dados_os.get('contagem', 0)
                    valor = conferente_id == conferente_anterior_id if contagem > 1 else False
            else:
                valor = None
        except Exception as e:
            valor = '[erro] %s' % e

        result[os.id] = valor

    return result


def recosv_validar_box(self, cr, uid, ids):
    """
    Valida o nro do box de recebimento

    @param cr: {cursor} a linha atual do cursor do banco de dados,
    @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    @param ids: {list} lista dos IDs,
    @return: {boolean} verdadeiro se box e valido.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    for os in self.browse(cr, uid, ids):
        if os.tipo_id.codigo == 'REC' and os.box < 1:
            return False

    return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
