# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class recebimento_os(osv.Model):
    """
    Ordem de servico de recebimento.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.os'  # recosv
    _inherit = 'recebimento.os'  # recebimento/inherit/os_data

    def recosv_lista_nf(self, cr, uid, ids, name, arg, context=None):
        """
        Monta a lista dos numeros das notas fiscais.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param name: {list} lista com o nome dos campos,
        :param arg: {dict} dicionario com os argumentos e valores,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {char} numero das Notas Fiscais utilizadas pelo recebimento separadas por virgula.
        :todo: Contemplar a montagem da lista para varios IDs.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')

        lista = ''
        nf_ids = self.read(cr, uid, ids[0], ['nf_ids'], context=context)

        for nr_nota in obj_fiscal_nf.browse(cr, uid, nf_ids, context=context):
            lista += str(nr_nota.numero_nf) + ', '

        result = {ids[0]: lista[:len(lista)-2]}

        return result

    def recosv_onchange_coferente_id(self, cr, uid, ids, conferente_id, anterior_id, contagem):
        """
        Verifica se o conferente atual da ordem de servico e o mesmo da contagem anterior

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param conferente_id: {int} ID do conferente atual
        :param anterior_id: {int} ID do conferente da contagem anterior
        :param contagem: {int} sequencia da contagem.
        :return: mensgagem informativa se os conferentes (atual e anterior) forem iguais.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = {'value': {'recosv_mesmo_conferente': False}}

        if contagem > 1 and conferente_id == anterior_id:
            result['value']['recosv_mesmo_conferente'] = True

        return result

    def recosv_onchange_box(self, cr, uid, ids, box, context=None):
        """
        Verifica se o ´´Box´´ ja esta sendo utilizado em outra ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param box: {char} valor atribuido ao ´´Box´´ que sera utilizado na verificacao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :attention: ´verifica_box_msg´ pode ser atribuido como ´False´ para que a mensagem de aviso
        nao seja mostrada caso haja alguma ordem de servico que esteja utilizano o ´´Box´´,
        :return: {boolean} verdadeiro ou mensagem de aviso de que o ´´Box´´ esta
        sendo utilizado por outra ordem de servico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        if self.recosv_box_em_uso(cr, uid, ids, box, context):
            msg = 'O Box de recebimento %s, encontra-se em uso por outra ordem de servico.' % box

            if not 'verifica_box_msg' in context or \
                    context.get('verifica_box_msg'):
                return {'warning': {'title': 'Aviso.', 'message': msg}}

        return True

    def recosv_box_em_uso(self, cr, uid, ids, box, context=None):
        """
        Verifica se o ´Box´ esta sendo utilizado por outras Ordens de Servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param box: {char} valor atribuido ao ´Box´ que sera utilizado na verificacao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} IDs das ordem de servico que estao utilizando o ´Box´.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        args = [('box', '=', box),
                ('state', 'in', ['esboco', 'aberta', 'criada'])]

        if ids:
            args.append(('id', 'not in', ids))  # Nao considera a propria ordem de servico

        return self.search(cr, uid, args, context=context)

    def osvosv_abrir_os(self, cr, uid, ids, context=None):
        """
        Abrir ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} status da ordem de servico aberta.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('recebimento.os.historico')

        for os in filter(lambda os: os.state == 'criada', self.browse(cr, uid, ids)):
            if os.tipo_id.codigo == 'REC':
                # Se nao existir responsavel a OS nao podera ser aberta
                if not os.responsavel_ids:
                    msg = _('Deve existir ao menos um responsavel pela execucao da ordem de '
                            'servico.')

                    raise osv.except_osv(_('Atencao!'), msg)

                # Abre contagem de recebimento
                self.recosv_abrir_contagem(cr, uid, ids, context)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'abrir',  context)

        return super(recebimento_os, self).osvosv_abrir_os(cr, uid, ids)

    def recosv_abrir_contagem(self, cr, uid, ids, context=None):
        """
        Abre nova contagem de recebimento.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {long} numero da ordem de servico cuja contagem foi aberta.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('recebimento.os.historico')
        obj_recebimento_itens = self.pool.get('recebimento.itens')
        os = None

        for os in self.browse(cr, uid, ids):
            contagem = os.contagem + 1

            # Inicia a contagem dos itens nao OK.
            ids = map(lambda rec: rec.id, filter(lambda i: i.state != 'O', os.recebimento_item_ids))

            dados = {'qtde_contada': 0, 'qtde_avaria': 0, 'data_validade': None,
                     'state': 'C'}

            if obj_recebimento_itens.write(cr, uid, ids, dados, context):
                # Abre a contagem
                dados = {'contagem': contagem, 'contagem_status': 'A'}

                if contagem > 1:  # Atualiza o conferente (anterior)
                    dados['conferente_anterior_id'] = os.conferente_id.id

                if self.write(cr, uid, [os.id], dados, context):
                    # Registro do historico
                    obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'abrir contagem', context)

        return os.contagem

    def recosv_fechar_contagem(self, cr, uid, ids, context=None):
        """
        Fecha contagem de recebimento.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} lista com o ID da ordem de servico cuja contagem foi fechada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_recebimento_itens = self.pool.get('recebimento.itens')
        obj_os_historico = self.pool.get('recebimento.os.historico')

        for os in self.browse(cr, uid, ids):
            # Verifica se ha divergencia para alterar o status do item.
            for item in os.recebimento_item_ids:
                if item.state == 'O':
                    continue

                if obj_recebimento_itens.recite_verifica_divergencias(cr, uid, item.id, context):
                    status = 'DV'
                else:
                    status = 'O'

                dados = {'state': status}
                # Altera o status dos itens divergentes
                obj_recebimento_itens.write(cr, uid, item.id, dados, context)

            # Fecha a contagem
            dados = {'contagem_status': 'F'}

            if self.write(cr, uid, os.id, dados, context):
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'fechar contagem', context)

        return ids

    def osvosv_cancelar_os(self, cr, uid, ids, context=None):
        """
        Cancelar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} status da ordem de servico cancelada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')
        obj_os_historico = self.pool.get('recebimento.os.historico')

        for os in self.browse(cr, uid, ids):
            if os.tipo_id.codigo == 'REC':

                # Libera as NF para faturamento.
                for nf_id in os.nf_ids:
                    if os.state not in ('fechada', 'cancelada'):
                        _id = [nf_id.id]
                        dados = {'recebimento_id': False}
                        context['salvar_forcado'] = True

                        obj_fiscal_nf.write(cr, uid, _id, dados, context)
                    else:
                        msg = _('Ordem de Recebimento nao pode ser cancelada.')

                        raise osv.except_osv(_('Atencao!'), msg)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'cancelar', context)

        return super(recebimento_os, self).osvosv_cancelar_os(cr, uid, ids, context)

    def osvosv_criar_os(self, cr, uid, ids, context=None):
        """
        Criar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} uma nova ordem de servico (criada).

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')
        obj_recebimento_itens = self.pool.get('recebimento.itens')
        obj_os_historico = self.pool.get('recebimento.os.historico')

        for os in self.browse(cr, uid, ids):
            if os.tipo_id.codigo == 'REC':
                # Se nao existir N.F. a O.S. nao podera ser criada
                if not os.nf_ids:
                    msg = _('Informe ao menos uma nota fiscal utilizada no recebimento.')

                    raise osv.except_osv(_('Atencao!'), msg)

                # Verifica se existem O.S. utilizando as mesmas notas fiscais
                # for nf_id in os.nf_ids:
                args = [('id', 'in', map(lambda nf: nf.id, os.nf_ids)),
                        ('recebimento_id', '!=', os.id),
                        ('state', 'not in', ['esboco',  'fechada', 'cancelada'])]

                if obj_fiscal_nf.search(cr, uid, args, context=context):
                    msg = _('Uma das notas fiscais utilizada no recebimento ja esta sendo '
                            'utilizada por outra ordem de servico.')

                    raise osv.except_osv(_('Atencao!'), msg)

                # Recupera e consolida os itens da nota fiscal
                itens_nf = {}

                for nf_id in os.nf_ids:
                    for item in nf_id.estoque_movimento_itens_ids:
                        produto_id = item.produto_id.id
                        grade_id = item.grade_id.id
                        embalagem = item.produto_embalagem_id
                        qtde_na_embalagem = embalagem.embalagem_id.quantidade_na_embalagem
                        quantidade = item.quantidade * qtde_na_embalagem
                        chave = (produto_id, grade_id)

                        if chave in itens_nf:
                            itens_nf[chave] += quantidade
                        else:
                            itens_nf[chave] = quantidade

                # Monta a lista de recebimento
                PRODUTO = 0
                GRADE = 1
                for item in itens_nf:
                    dados = {'recebimento_id': os.id,  # Nr da ordem de servico de Recebimento
                             'produto_id': item[PRODUTO],  # Produto
                             'grade_id': item[GRADE],  # Grade
                             'quantidade': itens_nf[item]}  # Quantidate do item de recebimento

                    context['recebimento'] = True

                    obj_recebimento_itens.create(cr, uid, dados, context)

                # Grava o nr. do recebimento nas Notas Fiscais
                context = {'salvar_forcado': True}  # Obriga a gravacao
                for nf_id in os.nf_ids:
                    dados = {'recebimento_id': os.id}
                    obj_fiscal_nf.write(cr, uid, nf_id.id, dados, context)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'criar', context)

        return super(recebimento_os, self).osvosv_criar_os(cr, uid, ids, context)

    def osvosv_button_esboco_os(self, cr, uid, ids, context=None):
        """
        Retorno da ordem de servico para a condicao de esboco.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} retorno da ordem de servico a condicao de esboco.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')
        obj_recebimento_itens = self.pool.get('recebimento.itens')
        obj_os_historico = self.pool.get('recebimento.os.historico')

        for os in filter(lambda os: os.tipo_id.codigo == 'REC', self.browse(cr, uid, ids)):
            # Inativa todos os itens de recebimento
            item_ids = map(lambda item: item.id, os.recebimento_item_ids)
            obj_recebimento_itens.write(cr, uid, item_ids, {'active': False}, context)

            # Remove o nr. do recebimento das notas fiscais
            context = {'salvar_forcado': True}  # Obriga a gravacao
            nf_ids = map(lambda nf: nf.id, os.nf_ids)
            obj_fiscal_nf.write(cr, uid, nf_ids, {'recebimento_id': False}, context)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'esboco', context)

        return super(recebimento_os, self).osvosv_button_esboco_os(cr, uid, ids, context)

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Fechar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} status da ordem de servico fechada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('recebimento.os.historico')
        obj_recebimento_itens = self.pool.get('recebimento.itens')

        for os in self.browse(cr, uid, ids):
            if os.tipo_id.codigo == 'REC':
                # Verifica se todos os itens estao com contagem "OK"
                item_ids = map(lambda item: item.id, os.recebimento_item_ids)

                if obj_recebimento_itens.recite_verifica_divergencias(cr, uid, item_ids):
                    msg = _('A ordem de servico Nr %s nao pode ser fechada. Existem divergencias '
                            'nos itens de recebimento.' % os.id)
                    raise osv.except_osv(_('Atencao!'), msg)

            # Fecha a ordem de servico
            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'fechar', context)
            self.recosv_fechar_contagem(cr, uid, os.id, context)  # Tudo Ok. Fecha a contagem

        return super(recebimento_os, self).osvosv_fechar_os(cr, uid, ids, context)

    def osvosv_imprimir_os(self, cr, uid, ids, context=None):
        """
        Imprimir ordem de servico - Mapa de conferencia cega.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} o relatorio da ordem de servico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('recebimento.os.historico')

        for os in self.browse(cr, uid, ids):
            if os.tipo_id.codigo == 'REC':
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'imprimir', context)

                if context.get('divergencia'):
                    return {
                        'type': 'ir.actions.report.xml',
                        'report_name': 'recebimento.os',
                        'datas': {'ids': [os.id]},
                        'context': {'divergencia': 'D I V E R G E N C I A'}
                    }
                else:
                    return {
                        'type': 'ir.actions.report.xml',
                        'report_name': 'recebimento.os',
                        'datas': {'ids': [os.id]},
                        'context': {'divergencia': False}
                    }

        return super(recebimento_os, self).osvosv_imprimir_os(cr, uid, ids, context)

    # _________________________ ORM Methods __________________________________
    def create(self, cr, uid, values, context=None):
        """
        Registra acao "esboco" no historico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param values: {dict} o nome dos campos e seus respectivos valores,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {int} o ID do novo registro criado

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        _id = super(recebimento_os, self).create(cr, uid, values, context)

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('recebimento.os.historico')

        obj_os_historico.osvhis_registra_acao(cr, uid, _id, 'esboco', context)

        return _id

    def default_get(self, cr, uid, fields, context):
        """
        Valor padrao para o campo ´status_nf_recebimento´.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param fields: {dict} dicionario com o nome dos campos e respectivas esruturas de
        construcao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} novos campos criados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects ________________________________
        obj_estoque_parametro = self.pool.get('estoque.parametro')

        result = super(recebimento_os, self).default_get(cr, uid, fields, context=context)
        parametro_id = obj_estoque_parametro.search(cr, uid, [], context=context)
        campos = ['estpar_status_nf']
        dados_parametro = obj_estoque_parametro.read(cr, uid, parametro_id,
                                                     campos, context=context)
        if dados_parametro and isinstance(dados_parametro, list):
            dados_parametro = dados_parametro[0]

        if dados_parametro and isinstance(dados_parametro, dict):
            status = dados_parametro.get('estpar_status_nf') or '2'

            result['status_nf_recebimento'] = status

        return result

    def fields_get(self, cr, uid, fields=None, context=None):
        """
        Acrescenta o campo status_nf_recebimento utilizado no filtro das Notas
        Fiscais de Recebimento (compra).

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param fields: {dict} dicionario com o nome dos campos e respectivas esruturas de
        construcao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} novos campos criados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = super(recebimento_os, self).fields_get(cr, uid, fields, context)

        result['status_nf_recebimento'] = {'selectable': True, 'type': 'char', 'size': 2,
                                           'string': 'Status Recebimento'}
        return result

    def read(self, cr, uid, ids, fields=None, context=None, load="_classic_read"):
        """
        Recupera o valor padrao para o campo ´status_nf_recebimento´.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param fields: {dict} dicionario com o nome dos campos e seus respectivos valores,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :param load:

        :return: {dict} valor do ´status´ de recebimento.
        """

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_parametro = self.pool.get('estoque.parametro')

        results = super(recebimento_os, self).read(cr, uid, ids, fields, context=context)

        for result in results:
            if 'status_nf_recebimento' not in result:
                parametro_id = obj_estoque_parametro.search(cr, uid, [], context=context)
                campos = ['estpar_status_nf']
                dados_parametro = obj_estoque_parametro.read(cr, uid,
                                                             parametro_id,
                                                             campos, context=context)
                if dados_parametro and isinstance(dados_parametro, list):
                    dados_parametro = dados_parametro[0]

                if dados_parametro and isinstance(dados_parametro, dict):
                    status = dados_parametro.get('estpar_status_nf') or '2'

                    result['status_nf_recebimento'] = status

        return results

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
