# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields

# ´´Status´´ da contagem
STATUS_CONTAGEM = [('A', 'Aberta'), ('F', 'Fechada')]


class recebimento_os(osv.Model):
    """
    Ordem de servico de recebimento.
    @todo: Incluir o prefixo no nome dos metodos da classe.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    # Funcoes utilizadas
    from funcoes import recosv_codigo_padrao, recosv_tipo_padrao
    from funcoes import recosv_fields_function, recosv_validar_box

    _name = 'recebimento.os'  # recosv
    _inherit = 'os'  # recebimento/os/model/os_data

    _columns = {
        # ___________________________ Many2One _______________________________
        # Apos 2a. contagem nao pode ser igual ao conferente
        'conferente_anterior_id': fields.many2one('res.users', 'Conferente anterior'),

        # ___________________________ Many2Many ______________________________
        'responsavel_ids': fields.many2many('res.users', 'recebimento_m2m_user', 'recebimento_id',
                                            'user_id', 'Responsaveis',
                                            help='Responsaveis pela execucao da ordem de servico.'),
        'nf_ids': fields.many2many('fiscal.nf.capa', 'recebimento_m2m_nf', 'recebimento_id',
                                   'nf_id', 'Nota fiscal',
                                   help='Notas fiscais utilizadas no recebimento.'),

        # ___________________________ One2Many _______________________________
        'recebimento_item_ids': fields.one2many('recebimento.itens', 'recebimento_id',
                                                'Item de recebimento',
                                                help='Itens de recebimento.'),

        # ___________________________ Char ___________________________________
        'box': fields.char('Box', size=2, required=True,
                           help='Box/local utilizado no recebimento das mercadorias.'),

        # ___________________________ Function _______________________________
        'lista_nf': fields.function(recosv_fields_function, type='text', method=True,
                                    help='# Relacao das notas fiscais envolvidas no recebimento.',
                                    string='Notas fiscais'),
        'recosv_recebimento_ok': fields.function(recosv_fields_function, type='boolean', method=True,
                                                 help='Se marcado informa que todos as mercadorias foram recebidas.',
                                                 string='Recebimento OK'),
        'recosv_divergencia': fields.function(recosv_fields_function,
                                              type='boolean', method=True,
                                              help='Se marcado informa que existe divergencia no recebimento.',
                                              string='Divergencia'),
        # Verdadeiro se o conferente for o mesmo da contagem anterior
        'recosv_mesmo_conferente': fields.function(recosv_fields_function, method=True,
                                                   type='boolean',  string='Mesmo conferente'),

        # ___________________________ Integer ________________________________
        'contagem': fields.integer('Contagem', help='Numero atual da contagem.'),

        # ___________________________ Selection ______________________________
        'contagem_status': fields.selection(STATUS_CONTAGEM, 'Status',
                                            help='O estado da contagem,'),

        # ___________________________ Boolean ________________________________
        'pendente': fields.boolean(string='Pendente de autorizacao',
                                   help='Se marcado informa que existe pendencia na ordem de servico.')

    }

    # Valores padrao
    _defaults = {
        'tipo_id': recosv_tipo_padrao,  # "REC" - Recebimento
        'tipo': recosv_codigo_padrao
    }

    # Obrigatoriedade na informacao do nr do ´´Box´´ (local de recebimento)
    _constraints = [(recosv_validar_box,
                     'O nr do box deve ser especificado.', ['Box (local de recebimento)'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
