# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""


# _______________________ recebimento.itens __________________________________
def recite_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: {cursor} a linha atual do cursor do banco de dados,
    :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    :param ids: {list} lista dos IDs,
    :param name: {list} lista com o nome dos campos,
    :param arg: {dict} dicionario com os argumentos e valores,
    :param context: {dict} um dicionario padrao para valores contextuais,
    :return: {dict} valores ajustados dos campos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    # ___________________________ Objects ________________________________
    obj_codigo_barra = self.pool.get('automacao.codigo.barra')
    obj_adm_utils = self.pool.get('adm.utils')

    result = {}

    # Contexto para mostrar produto baseado no parametro de descricao da grade
    campos = ['estpar_descricao_grade']
    adm_utils_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', campos, context=context)

    for brw_recebimento_item in self.browse(cr, uid, ids):
        try:
            if name == 'codigo':
                value = brw_recebimento_item.produto_id.codigo
            elif name == 'codigo_barra':  # Codigo de barra do produto relacionado a embalagem.
                embalagem_id = brw_recebimento_item.produto_embalagem_id.id
                args = [('produto_id', '=', brw_recebimento_item.produto_id.id),
                        ('grade_id', '=', brw_recebimento_item.grade_id.id),
                        ('produto_embalagem_id', '=', embalagem_id)]
                ids = obj_codigo_barra.search(cr, uid, args, context=context)
                if ids:
                    browse_cb = obj_codigo_barra.browse(cr, uid, ids)
                    value = browse_cb[0].codigo_barra
                else:
                    value = None
            elif name == 'os_contagem_status':
                # Status da contagem da ordem de servico (aberta/fechada)
                value = brw_recebimento_item.recebimento_id.contagem_status
            elif name == 'recite_grade_sigla':
                value = brw_recebimento_item.grade_id.grade
            elif name == 'recite_produto':
                codigo = brw_recebimento_item.produto_id.codigo or ''
                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    produto = brw_recebimento_item.grade_id.descricao or ''
                else:  # Produto + grade
                    produto = brw_recebimento_item.produto_id.name or ''
                value = "%s%s%s" % (codigo, ' - ' if codigo else '', produto)
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_recebimento_item.id] = value

    return result


def recite_field_selection_state(self, cr, uid, context=None):
    """
    ´´Status´´ do item de recebimento.

    :param cr: {cursor} a linha atual do cursor do banco de dados,
    :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    :param context: {dict} um dicionario padrao para valores contextuais,
    :return: {list} valores de selecao do ´´status´´.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>

    """

    context = context and context.copy() or {}

    result = [('C', 'Contagem'), ('O', 'OK'), ('DV', 'Divergente')]

    return result


# _______________________ recebimento.divergencia.itens ______________________
def recdit_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: {cursor} a linha atual do cursor do banco de dados,
    :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    :param ids: {list} lista dos IDs,
    :param name: {list} lista com o nome dos campos,
    :param arg: {dict} dicionario com os argumentos e valores,
    :param context: {dict} um dicionario padrao para valores contextuais,
    :return: {dict} valores ajustados dos campos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for brw_recebimento_divergencia_item in self.browse(cr, uid, ids):
        try:
            if name == 'codigo':
                value = brw_recebimento_divergencia_item.produto_id.codigo
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_recebimento_divergencia_item.id] = value

    return result


def recdit_field_selection_divergencia(self, cr, uid, context=None):
    """
    Tipos de divergencia do item da nota fiscal.

    :param cr: {cursor} a linha atual do cursor do banco de dados,
    :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    :param context: {dict} um dicionario padrao para valores contextuais,
    :return: {list} valores de selecao dos tipos de divergencia.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    result = [('DV', 'Divergencia'), ('E', 'Excede na nota fiscal'), ('F', 'Falta na nota fiscal')]

    return result


# _______________________ recebimento.divergencia.nf _________________________
def recdnf_field_selection_state(self, cr, uid, context=None):
    """
    ´´Status´´ da nota fiscal.

    :param cr: {cursor} a linha atual do cursor do banco de dados,
    :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    :param context: {dict} um dicionario padrao para valores contextuais,
    :return: {list} valores de ´´status´´ da nota fiscal.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}
    result = [('DV', 'Divergencia'), ('A', 'Aceita'), ('R', 'Recusada')]

    return result


def recdnf_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: {cursor} a linha atual do cursor do banco de dados,
    :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
    :param ids: {list} lista dos IDs,
    :param name: {list} lista com o nome dos campos,
    :param arg: {dict} dicionario com os argumentos e valores,
    :param context: {dict} um dicionario padrao para valores contextuais,
    :return: {dict} valores das chaves representando registros divergentes e
    nao divergentes.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for divergencia in self.browse(cr, uid, ids, context):
        try:
            if name == 'numero':  # ID da nota fiscal
                value = divergencia.nf_id.id
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[divergencia.id] = value

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
