# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class recebimento_itens(osv.Model):
    """
    Itens de Recebimento.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.itens'  # recite

    from funcoes import recite_fields_function, recite_field_selection_state

    _columns = {
        # ___________________________ Many2One _______________________________
        'produto_id': fields.many2one('estoque.produto', 'Produto', select=True),  # Produto
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade', required=True, select=True,
                                    ondelete='restrict'),
        'usuario_id': fields.many2one('res.users', 'Responsavel pela digitacao',
                                      help='Usuario responsavel pela digitacao do recebimento.'),

        # ___________________________ Integer ________________________________
        'quantidade': fields.integer('Quantidade recebida'),  # Quantidade total dos itens das NFs
        'qtde_contada': fields.integer('Quantidade contada',
                                       help='Quantidade informada na contagem. Sempre em unidades simples.'),
        'qtde_avaria': fields.integer('Quantidade avariada',
                                      help='Quantidade avariados informada na contagem. Sempre em unidades simples.'),

        # ___________________________ Date ___________________________________
        # Data da validade do produto informada na contagem
        'data_validade': fields.date('Data de validade'),

        # ___________________________ Datetime _______________________________
        # Data da digitacao do recebimento "cego"
        'data_digitacao': fields.date('Data da digitacao'),

        # ___________________________ Function _______________________________
        'codigo': fields.function(recite_fields_function, method=True, type='char',
                                  string='Codigo'),
        'os_contagem_status': fields.function(recite_fields_function, method=True, type='char',
                                              size=1, string='Status da contagem'),
        'recite_grade_sigla': fields.function(recite_fields_function, method=True, type='char',
                                              size=2, string='Grade'),  # Sigla da grade
        'recite_produto': fields.function(recite_fields_function, method=True, type='char',
                                          size=60, string='Produto'),

        # ___________________________ Selection ______________________________
        'state': fields.selection(recite_field_selection_state,  # Status do recebimento
                                  'Status do item'),

        # ___________________________ Boolean ________________________________
        'active': fields.boolean('Ativo')
    }

    # Ordem
    _order = 'produto_id'

    # Valores padrao
    _defaults = {
        'state': lambda * args: 'C',  # Item em contagem
        'active': lambda * args: True
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
