# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _
import time


class os(osv.Model):
    """
    Ordem de servico.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'os'  # osvosv
    _inherit = ['os', 'ir.needaction_mixin', 'mail.thread']

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de ordens de servico com ´status´ igual a 'aberta'
        no menu 'Ordem de servico'.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} dominio utilizado para contar as Ordens de servico com ´status´  igual a
        'aberta'.

        :attention: e necessario que a classe "ir.needaction_mixin" seja carregada em "_inherit".
        Exemplo: _inherit = '[ir.needaction_mixin]'
        :warning: nao coloque prefixo ou troque o nome deste metodo!

        :see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        :copyright: OpenERP SA.
        """

        context = context and context.copy() or {}
        return [('state', 'in', ['aberta'])]

    def osvosv_atualizar_os(self, cr, uid, ids, context=None):
        """
        Atualizar os dados da ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a ordem de severvico foi atualizada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        return self.write(cr, uid, ids, {}, context)

    def osvosv_abrir_os(self, cr, uid, ids, context=None):
        """
        Abrir ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a ordem de servico foi aberta.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        return self.osvosv_registra_status_os(cr, uid, ids, 'aberta', context)

    def osvosv_cancelar_os(self, cr, uid, ids, context=None):
        """
        Cancelar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a ordem de servico foi cancelada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        return self.osvosv_registra_status_os(cr, uid, ids, 'cancelada', context)

    def osvosv_criar_os(self, cr, uid, ids, context=None):
        """
        Criar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a ordem de servico foi criada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        return self.osvosv_registra_status_os(cr, uid, ids, 'criada', context)

    def osvosv_button_esboco_os(self, cr, uid, ids, context=None):
        """
        Retorno da ordem de servico a condicao de esboco.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a ordem de servico retornou a condicao de esboco.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        return self.osvosv_registra_status_os(cr, uid, ids, 'esboco', context)

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Fechar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a ordem de servico foi fechada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        return self.osvosv_registra_status_os(cr, uid, ids, 'fechada', context)

    def osvosv_imprimir_os(self, cr, uid, ids, context=None):
        """
        Imprimir ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} o relatorio da ordem de servico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        return self.osvosv_registra_status_os(cr, uid, ids, 'impressa', context)

    def osvosv_registra_status_os(self, cr, uid, ids, status=None, context=None):
        """
        Registro do status da ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param status: {char} o estado da ordem de servico: aberta, fechada, cancelada, etc.
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se o registro do ´status´ da ordem de ser-
        vico foi realizado.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Dados da ordem de servico
        dados = {'active': True}

        agora = time.strftime('%Y-%m-%d %H:%M:%S')
        if status == 'aberta':
            dados['data_abertura'] = agora
        elif status == 'criada':
            dados['data_criacao'] = agora
        elif status == 'esboco':  # Retorno da ordem de servico a condicao de esboco
            dados['data_criacao'] = False
        elif status == 'fechada':
            dados['data_fechamento'] = agora
        elif status == 'cancelada':
            dados['data_cancelamento'] = agora

        dados['state'] = status
        os_lista = []  # Lista com os IDs das ordens de servico cujo ´status´ foi alterado

        for brw_os in self.browse(cr, uid, ids):
            if status:
                if self.write(cr, uid, brw_os.id, dados, context):
                    os_lista.append(brw_os.id)

        return all(os_lista)

    # _________________________ ORM Methods __________________________________
    def copy(self, cr, uid, ids, defaults=None, context=None):
        """
        Nao permite duplicar ordens de servico

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} uma mensagem informando que ordens de servico nao podem
        ser duplicadas.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        msg = _('Ordem de servico nao pode ser duplicada.')

        raise osv.except_osv(_('Atencao!'), msg)

    def unlink(self, cr, uid, ids, context=None):
        """
        Permitir exclusao de ordens de servico apenas se o status for 'esboco'

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} uma mensagem informando que a ordem de servico nao pode ser excluida.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        for brw_os in self.browse(cr, uid, ids, context):
            if brw_os.state != 'esboco':
                msg = _('Ordem de servico nao pode ser excluida, apenas cancelada.')

                raise osv.except_osv(_('Atencao!'), msg)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
