# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
from datetime import date
from funcoes import rclrcl_get_nf_produto_ids


class reclamacoes_reclamacao(osv.Model):

    _name = 'reclamacoes.reclamacao'
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _columns = {
        'id': fields.integer('Controle', readonly=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True, select=True), # Id da empresa
        'state': fields.selection([('E', 'Esboco'), ('P', 'Pendente'), ('C', 'Cancelada'), ('F', 'Finalizada')],
                                 'Estado'),
        'rclrcl_reclamacao': fields.text('Reclamacao', required=True),
        'rclrcl_pessoa_id': fields.many2one('adm.pessoa', 'Pessoa', ondelete='restrict', required=True),
        'rclrcl_obs': fields.text('Informacoes complementares'),
        'rclrcl_compra_venda': fields.selection([('E', 'Compra'), ('S', 'Venda')], 'Reclamacao de', required=True),
        'rclrcl_nota_fiscal_origem_id': fields.many2one('fiscal.nf.capa', 'Nota fiscal origem', required=True,
                                                        ondelete="restrict", domain="[('destinatario_id', '=', rclrcl_pessoa_id),('tipo', '=', rclrcl_compra_venda)]"),
        'rclrcl_nota_fiscal_devolucao_id': fields.many2one('fiscal.nf.capa', 'Nota fiscal Devolucao',
                                                        ondelete="restrict", domain="[('destinatario_id', '=', rclrcl_pessoa_id),('tipo', '!=', rclrcl_compra_venda)]"),
        'create_date': fields.datetime('Data de criacao', readonly=True),
        'rclrcl_proximo_passo': fields.date('Proximo passo'),
        'rclrcl_pedido_origem_id': fields.many2one('estoquepedidos.pedido.capa', 'Pedido de origem', ondelete="restrict",
                                                   domain="[('pessoa_id', '=', rclrcl_pessoa_id),('tipo', '=', rclrcl_compra_venda_pedido_invertida)]"),
        'rclrcl_compra_venda_pedido_invertida': fields.selection([('C', 'Compra'), ('V', 'Venda')], 'Inverso:', required=True),
        'rclrcl_nota_fiscal_nova_id':  fields.many2one('fiscal.nf.capa', 'Nota fiscal substituicao',
                                                       help='Nova nota fiscal para envio da troca de mercadoria',
                                                       ondelete="restrict",
                                                       domain="[('destinatario_id', '=', rclrcl_pessoa_id),('tipo', '=', rclrcl_compra_venda)]"),
        'rclrcl_reclamacao_externa': fields.char('Reclamacao externa', size=256, help='Endereco da reclamacao em sites externos'),
        'rclrcl_passos_ids': fields.one2many('reclamacoes.passos', 'rclpas_reclamacao_id', 'Passos'),
        'rclrcl_nf_produto_ids': fields.function(rclrcl_get_nf_produto_ids, method=True,
                                                 string='Produtos NF',
                                                 relation='estoque.produto',
                                                 type='one2many'),
        'rclrcl_nf_produto_id': fields.many2one('estoque.produto', 'Produto'),
        'rclrcl_devolvido': fields.boolean('Pedido Devolvido'),
        'rclrcl_substituido': fields.boolean('Pedido Substituido', help='Marque qdo for troca de produto'),
        'rclrcl_perdido': fields.boolean('Pedido Perdido', help='Marque esta opção qdo reembolsar o cliente sem o retorno dos produtos'),
        'rckrcl_motivo_id': fields.many2one('reclamacoes.motivos', 'Motivo', help='Motivo da reclamacao', select=True),
    }
    _defaults = {
        'state': 'E',
        'company_id': lambda s, cr, uid, c: s.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, s._name, context=c),
    }
    _order = 'id desc'
