# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import re
from openerp.tools.translate import _
from SigUtils.formatastring import formataString
from SigUtils.pis import Pis


class rh_funcionario(osv.Model):
    _inherit = 'rh.funcionario'

    def get_id_func(self, cr, uid, context=None):
        context = context and context.copy() or {}
        id = False
        usuario = self.pool.get('res.users').read(cr, uid, uid, ['pessoa_id'], context=context)
        if usuario:
            if usuario['pessoa_id']:
                pessoa_id = self.pool.get('rh.funcionario').search(cr, uid, [('pessoa', '=', usuario['pessoa_id'][0])], context=context)
                if pessoa_id:
                    id = pessoa_id[0]
        return id

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        ecf_operador = vals.get('ecf_operador')
        if ecf_operador and ecf_operador > 0:
            ids = self.search(cr, uid, [('ecf_operador', '=', ecf_operador)], context=context)
            if ids:
                raise osv.except_osv(_('Atencao'), _('Ecf operador deve ser unico.'))
        if context.get('install_mode') and context['install_mode']:
            args = [('pessoa', '=', vals['pessoa']), ('company_id', '=', vals['company_id'])]
            _id = self.search(cr, uid, args, context=context)
            if _id:
                return _id[0]
        if vals.get('rhfunc_pis'):
            if not Pis().validate(vals.get('rhfunc_pis')):
                raise osv.except_osv(_('Atencao'), _('Codigo do PIS invalido!'))
            vals['rhfunc_pis'] = Pis().verificarTamanho(vals.get('rhfunc_pis'))
        res = super(rh_funcionario, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('ecf_operador'):
            ecf_operador = vals.get('ecf_operador')
            dados = self.browse(cr, uid, ids)
            for i in dados:
                if ecf_operador and ecf_operador > 0:
                    rh_funcionario_id = self.search(cr, uid, [('ecf_operador', '=', ecf_operador)], context=context)
                    if rh_funcionario_id:
                        raise osv.except_osv(_('Atencao'), _('Ecf operador deve ser unico.'))
        if vals.get('rhfunc_pis'):
            if not Pis().validate(vals.get('rhfunc_pis')):
                raise osv.except_osv(_('Atencao'), _('Codigo do PIS invalido!'))
            vals['rhfunc_pis'] = Pis().verificarTamanho(vals.get('rhfunc_pis'))
        res = super(rh_funcionario, self).write(cr, uid, ids, vals, context)
        return res

    def rhfunc_onchange_matricula(self, cr, uid, ids, matricula):
        """
        Verifica a matricula do funcionario.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param matricula: {int} numero de matricula do funcionario.
        @return: {dict} mensagem de erro se a matricula for invalida.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': {'matricula': matricula}}

        if matricula > 999999999:
            result['value']['matricula'] = False
            result['warning'] = {'title': _('Aviso.'),
                                 'message': _('Matricula informada para funcionario nao e valida.')}

        return result

    def onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        pessoa_ids = obj_pessoa.search(cr, uid,
                                       [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar), ('pessoa_tipo', '=', pessoa_tipo)],
                                       context=context)
        if pessoa_ids:
            funcionario_id = obj_rh_funcionario.search(cr, uid, [('pessoa', 'in', pessoa_ids), ('active', '=', True)],
                                                       context=context)
            if funcionario_id:
                warning = {'title': _('Atencao')}
                warning['message'] =_('Funcionario ja cadastrado!')
                return {'warning': warning, 'value': {'cpf_cnpj': ''}}
            dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
            if isinstance(dados_pessoa, list):
                dados_pessoa = dados_pessoa[0]
            endereco_ids = []
            for dados_endereco in dados_pessoa.endereco_ids:
                enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                             'active': dados_endereco.active,
                             "cep": dados_endereco.cep,
                             "pais_id": dados_endereco.pais_id.id,
                             "uf_id": dados_endereco.uf_id.id,
                             "cidade_id": dados_endereco.cidade_id.id,
                             "bairro_id": dados_endereco.bairro_id.id,
                             "logradouro_id": dados_endereco.logradouro_id.id,
                             "numero": dados_endereco.numero,
                             "complemento": dados_endereco.complemento,
                             "observacao": dados_endereco.observacao,
                }
                endereco_ids.append((1, dados_endereco.id, enderecos))
            email_ids = []
            for dados_emails in dados_pessoa.email_ids:
                emails = {
                    "email_tipo_id": dados_emails.email_tipo_id.id,
                    "name": dados_emails.name,
                    "observacao": dados_emails.observacao,
                }
                email_ids.append((1, dados_emails.id, emails))
            telefones_ids = []
            for dados_telefones in dados_pessoa.telefones_ids:
                telefones = {
                    "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                    "ddi": dados_telefones.ddi,
                    "ddd": dados_telefones.ddd,
                    "numero": dados_telefones.numero,
                    "observacoes": dados_telefones.observacoes,
                }
                telefones_ids.append((1, dados_telefones.id, telefones))
            res = {'value': {'cpf_cnpj': cpf_cnpj,
                             'name': dados_pessoa.name,
                             'codigo': dados_pessoa.codigo,
                             'fantasia_apelido': dados_pessoa.fantasia_apelido,
                             'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                             'admpes_fis_rg_orgao_emissor': dados_pessoa.admpes_fis_rg_orgao_emissor,
                             'admpes_fis_rg_data_emisssao': dados_pessoa.admpes_fis_rg_data_emisssao,
                             'admpes_fis_cnh_nro': dados_pessoa.admpes_fis_cnh_nro,
                             'admpes_fis_cnh_orgao_emissor': dados_pessoa.admpes_fis_cnh_orgao_emissor,
                             'admpes_fis_cnh_validade': dados_pessoa.admpes_fis_cnh_validade,
                             'admpes_fis_cnh_categoria': dados_pessoa.admpes_fis_cnh_categoria,
                             'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                             'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                             'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                             "foto": dados_pessoa.foto,
                             'endereco_ids': endereco_ids,
                             'email_ids': email_ids,
                             'telefones_ids': telefones_ids,
                             'pessoa': dados_pessoa.id,
            }
            }
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        new_args = []
        for x in args:
            if 'id' in x:
                continue
            new_args.append(x)
        res = super(rh_funcionario, self).search(cr, uid, new_args, offset, limit, order, context,
                                                 count)
        return res


    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        ids = []
        try:
            codigo = int(name)
        except:
            codigo = False
        if codigo:
            ids = self.search(cr, user, [('matricula', '=', codigo)], limit=limit, context=context)
        if not ids:
            ids = self.search(cr, user, [('name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return {}
        context = context and context.copy() or {}
        reads = self.read(cr, uid, ids, ['matricula', 'name'], context=context)
        res = []
        for record in reads:
            name = '%s - %s' % (formataString(record['matricula'], 4, '0', 'E'), record['name'])
            res.append((record['id'], name))
            if context.get('rhfun_so_codigo'):
                res.append((record['id'], record['matricula']))
            if context.get('rhfun_sem_codigo'):
                res.append((record['id'], record['name']))
        return res

