# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp import pooler
import time

class rhcurriculo_curriculo(osv.Model):
    """
    Tabela de curriculos
    """

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        foto = vals.get('foto')
        if foto:
            pessoa_id = vals.get('pessoa_id')
            self.pool.get('adm.pessoa').write(cr, uid, pessoa_id, {'foto': foto}, context)
        res = super(rhcurriculo_curriculo, self).create(cr, uid, vals, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        if 'foto' in vals:
            foto = vals.get('foto')
            if foto:
                pessoa_id = self.read(cr, uid, ids, ['pessoa_id'], context=context)
                if isinstance(pessoa_id, list):
                    pessoa_id = pessoa_id[0]
                    if isinstance(pessoa_id, dict):
                        pessoa_id = pessoa_id.get('pessoa_id')
                        if isinstance(pessoa_id, tuple):
                            pessoa_id = pessoa_id[0]
                            self.pool.get('adm.pessoa').write(cr, uid, pessoa_id, {'foto': foto}, context=context)
        res = super(rhcurriculo_curriculo, self).write(cr, uid, ids, vals, context=context)
        return res

    _name = 'rhcurriculo.curriculo'
    _inherits = {
        'adm.pessoa': 'pessoa_id',
    }
    _columns = {
        'filhos' : fields.boolean('Filhos'), # Informa se possui filhos
        'deficiencia' : fields.boolean('Deficiencia'), # Informa se possui deficiencia fisica
        'deficiencia_descricao' : fields.text('Tipo de deficiencia'), #Descreve o tipo de dificiencia fisica
        'outra_cidade' : fields.boolean('Considera trabalhar em outra cidade'), # Informe se o candidato aceita trabalhar em outra cidade
        'pretencao' : fields.selection([('A', 'Ate R$ 510,00'), ('B', 'R$ 511,00 - R$ 1000,00'),
                                                 ('C', 'R$ 1001,00 - R$ 2000,00'), ('D', 'R$ 2001,00 - R$ 3000,00'),
                                                 ('E', 'R$ 3001,00 - R$ 4000,00'), ('F', 'R$ 4001,00 - R$ 5000,00'),
                                                 ('G', 'R$ 5001,00 - R$ 6000,00'), ('H', 'R$ 6001,00 - R$ 10000,00'),
                                                ], 'Pretensao salarial', required=True, size=1), # Pretensao salarial do candidato
        'salario_atual' : fields.float('Salario atual', required=False), # Salario atual
        'disponibilidade' : fields.selection([('0','Imediata'), ('1', 'Uma semana'),('2', 'Quinze dias'),('3', 'Trinta dias'),('4', 'Mais de trinta dias') ], 'Disponibilidade', required=True, size=1), # Disponibilidade para inicio\n0=Imediata\n1=Uma semana\n2=15 dias\n3=30 dias\n4=Mais de 30 dias
        'trabalhando' : fields.selection([('S','Sim'), ('N', 'Nao'), ('A', 'Aviso previo')], 'Trabalhando', required=True, size=1), # Indica se o candidato esta trabalhando\nS=Sim,\nN=Nao\nA=Aviso previo
        'escolaridade' : fields.selection([('00','Sem Informacao'),('01', 'Nao sabe ler/escrever'),('02', 'Alfabetizado'),('03', 'Fundamental 1o grau incompleto'),('04', 'Fundamental 1o grau completo'),('05', 'Medio 2o grau incompleto'),('06', 'Medio 2o grau completo'),('07', 'Superior incompleto'),('08', 'Superior completo'),('09', 'Especializacao'),('10', 'Residencia'),('11', 'Mestrado'),('12', 'Doutorado')], 'Escolaridade', required=True, size=2), # Grau de escolaridade\n01=Nao sabe ler/escrever\n02=Alfabetizado\n03=Fundamental 1o grau incompleto\n04=Fundamental 1o grau completo\n05=Medio 2o grau incompleto\n06=Medio 2o grau completo\n07=Superior incompleto\n08=Superior completo\n09=Especializacao\n10=Residencia\n11=Mestrado\n12=Doutorado\n00=Sem informacao
        'habilitacao' : fields.selection([('S','Sim'), ('N', 'Nao')], 'Habilitacao', size=1), # Se possuir habilitacao
        'habilitacao_categoria' : fields.selection([('A','A'),('B','B'),('C','C'),('D','D'),('E','E'),('AB','AB'),('AC','AC'),('AD','AD'),('AE','AE')], 'Habilitacao categoria', size=2),
        'complemento' : fields.text('Complemento'), # Informacoes complementares
        'pessoa_id' : fields.many2one("adm.pessoa", 'Candidato', ondelete="cascade", required=True, select = True, domain="[('pessoa_tipo','=','F')]"), # Pessoa_id
        'active' : fields.integer('Ativo'), # Situacao de Ativo/Inativo
        'acesso_id' : fields.many2one("rhcurriculo.acesso", 'Acesso', ondelete="cascade", select = True),
        'create_date':fields.date('Data da inclusao', readonly=True), # Data da inclusao do curriculo
        'empresas_ids' : fields.one2many('rhcurriculo.empresas', 'curriculo_id','Empresas'),
        'cursos_ids' : fields.one2many('rhcurriculo.cursos', 'curriculo_id','Cursos'),
        'idiomas_ids' : fields.one2many('rhcurriculo.idiomas', 'curriculo_id','Idiomas'),
        'experiencia' : fields.boolean('Possui experiencia'),
        'experiencias_ids' : fields.one2many('rhcurriculo.experiencias', 'curriculo_id','Experiencia profissional'),
        'entrevistas_ids' : fields.one2many('rhcurriculo.entrevista', 'curriculo_id','Entrevistas'),
        'tecnologia_ids' : fields.one2many('rhcurriculo.tecnologia', 'curriculo_id','Tecnologia'),
        'rhccur_categoria_id': fields.many2one('rhcurriculo.categoria', 'Categoria', select=True),
    }
    _defaults = {
        'active': lambda *a: True,
        'filhos': lambda *a: False,
        'deficiencia': lambda *a: False,
        'outra_cidade': lambda *a: False,
    }
rhcurriculo_curriculo()

