# -*- coding: utf-8 -*-
from django import forms
from django.contrib.localflavor.br.br_states import STATE_CHOICES
from django.contrib.admin import widgets
import datetime
from django.contrib.admin.widgets import AdminDateWidget
import xmlrpclib, socket

dbname = 'erp'
username = 'rhexterno'
pwd = 'rhexterno'
sock_common = xmlrpclib.ServerProxy ('http://localhost:8069/xmlrpc/common')
socket.setdefaulttimeout(3)
uid = sock_common.login(dbname, username, pwd)
sock = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/object')

tipo_cursos = sock.execute(dbname, uid, pwd, 'rhcurriculo.cursos.tipos', 'search', [])
tipos_curso = [(False,'')]
for id in tipo_cursos:
    vals = sock.execute(dbname, uid, pwd, 'rhcurriculo.cursos.tipos', 'read', id, ['id', 'name'])
    tipos_curso.append((vals['id'],vals['name']))


MONTHS = [
    ('False', ''),
    ('01', '01 - JAN   '),
    ('02', '02 - FEV   '),
    ('03', '03 - MAR   '),
    ('04', '04 - ABR   '),
    ('05', '05 - MAI   '),
    ('06', '06 - JUN   '),
    ('07', '07 - JUL   '),
    ('08', '08 - AGO   '),
    ('09', '09 - SET   '),
    ('10', '10 - OUT   '),
    ('11', '11 - NOV   '),
    ('12', '12 - DEZ   ')
]
YEARS = range(1980,datetime.datetime.now().year +1)
years = [(False, '')]
for i in YEARS:
    years.append((i,i))
    
class DateDropdownWidget(forms.MultiWidget):
    def __init__(self,attrs=None,year_range=None,month_range=None,day_range=None):
        YEARS = year_range or range(2000,2021)
        MONTHES = month_range or range(1,13)
        DAYS = day_range or range(1,32)

        years = map( lambda x: (x,x), YEARS )
        months = map(lambda x:(x,x), MONTHES )
        days = map( lambda x: (x,x), DAYS )

        widgets = (
                forms.Select(choices=years),
                forms.Select(choices=months),
                forms.Select(choices=days),
                )
        super(DateDropdownWidget, self).__init__(widgets, attrs)

    def format_output(self,widgets):
        format = "Dia %s&nbsp;Mes %s&nbsp;Ano %s".decode('utf-8')
        return format%(widgets[2],widgets[1],widgets[0])

    def decompress(self,value):
        if value:
            return [value.year, value.month,value.day]
        return [None,None,None]

class DateField(forms.MultiValueField):
    widget = DateDropdownWidget
    def __init__(self,*args,**kwargs):
        fields = (
                forms.IntegerField( required=True),
                forms.IntegerField( required=True),
                forms.IntegerField( required=True ),
                )
        super(DateField, self).__init__(fields, *args,**kwargs )

    def compress(self,data_list):
        EMPTY_VALUES = [None, '']
        ERROR_EMPTY = "Fill the fields."
        ERROR_INVALID = "Enter a valid date."
        if data_list:
            if filter(lambda x: x in EMPTY_VALUES, data_list):
                raise forms.ValidationError(ERROR_EMPTY)

            try:
                return datetime.datetime(*map(lambda x:int(x),data_list))
            except ValueError:
                raise forms.ValidationError(ERROR_INVALID)
        return None

class MonthYearWidget(forms.MultiWidget):
    """
    A widget that splits a date into Month/Year with selects.
    """
    def __init__(self, attrs=None):
        months = (
            ('01', 'Jan (01)'),
            ('02', 'Feb (02)'),
            ('03', 'Mar (03)'),
            ('04', 'Apr (04)'),
            ('05', 'May (05)'),
            ('06', 'Jun (06)'),
            ('07', 'Jul (07)'),
            ('08', 'Aug (08)'),
            ('09', 'Sep (09)'),
            ('10', 'Oct (10)'),
            ('11', 'Nov (11)'),
            ('12', 'Dec (12)'),
            )


        year = int(datetime.date.today().year)
        year_digits = range(year, year+10)
        years = [(year, year) for year in year_digits]

        widgets = (forms.Select(attrs=attrs, choices=months), forms.Select(attrs=attrs, choices=years))
        super(MonthYearWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            return [value.month, value.year]
        return [None, None]


    def render(self, name, value, attrs=None):
        try:
            value = datetime.date(month=int(value[0]), year=int(value[1]), day=1)
        except:
            value = ''

        return super(MonthYearWidget, self).render(name, value, attrs)


class MonthYearField(forms.MultiValueField):
    def compress(self, data_list):
        if data_list:
            return datetime.date(year=int(data_list[1]), month=int(data_list[0]), day=1)
        return datetime.date.today()


class LoginForm(forms.Form):
    cpf = forms.CharField(max_length=11, widget=forms.TextInput(attrs={'size':'11', 'onBlur':'Verifica_campo_CPF(this)'}))
    senha = forms.CharField(max_length=45, widget=forms.PasswordInput(attrs={'size':'20'}))
    senha_confirm = forms.CharField(max_length=45, widget=forms.PasswordInput(attrs={'size':'20'}), required=False)

class LoggedForm(forms.Form):
    email = forms.EmailField(widget=forms.TextInput(attrs={'size':'50'}))
    email_confirm = forms.EmailField(widget=forms.TextInput(attrs={'size':'50'}))
    nome = forms.CharField(max_length=50, widget=forms.TextInput(attrs={'size':'50'}))
    cep = forms.CharField(max_length=8, widget=forms.TextInput(attrs={'size':'8', 'onBlur':'get_cep()'}))
    #ENDEREÇO
    cidade = forms.CharField(max_length=50, required=False, widget=forms.TextInput(attrs={'size':'50', 'readonly':'readonly'}))
    uf = forms.CharField(max_length=50, required=False, widget=forms.TextInput(attrs={'size':'50', 'readonly':'readonly'}))
    endereco = forms.CharField(max_length=50, required=False, widget=forms.TextInput(attrs={'size':'50', 'readonly':'readonly'}))
    bairro = forms.CharField(max_length=50, required=False, widget=forms.TextInput(attrs={'size':'50', 'readonly':'readonly'}))
    numero = forms.CharField(max_length=5, required=False, widget=forms.TextInput(attrs={'size':'5'}))
    complemento = forms.CharField(max_length=20, required=False, widget=forms.TextInput(attrs={'size':'20'}))
    #FONES
    ddd_cel = forms.DecimalField(max_digits=4, required=False, widget=forms.TextInput(attrs={'size':'4'}))
    fone_cel = forms.CharField(max_length=10, required=False, widget=forms.TextInput(attrs={'size':'10'}))
    ddd_com = forms.DecimalField(max_digits=4, required=False, widget=forms.TextInput(attrs={'size':'4'}))
    fone_com = forms.CharField(max_length=10, required=False, widget=forms.TextInput(attrs={'size':'10'}))
    ddd_res = forms.DecimalField(max_digits=4, required=False, widget=forms.TextInput(attrs={'size':'4'}))
    fone_res = forms.CharField(max_length=10, required=False, widget=forms.TextInput(attrs={'size':'10'}))
    # PESSOA FISICA
    nascimento =  forms.DateField(('%d/%m/%Y',), widget=forms.DateInput(format='%d/%m/%Y'), required=True, help_text=u'(Formatodd/mm/aaaa)')

    sexo = forms.ChoiceField(choices=[('',''), ('M','Masculino'),('F','Feminino')])
    estado_civil = forms.ChoiceField(choices=[('',''),('C','Casado'),('S','Solteiro'),('V','Viuvo'),('D','Divorciado'),('E','Desquitado')])
    filhos = forms.BooleanField(initial=False, required=False)
    # DEFICIENCIA
    deficiencia = forms.BooleanField(required=False)
    deficiencia_descricao = forms.CharField(widget=forms.widgets.Textarea(attrs={'cols':'70', 'rows': '2'}), required=False)
    # EXPERIENCIA
    experiencia = forms.BooleanField(required=False)
    trabalhando = forms.ChoiceField(choices=[('S','Sim'),
                                                ('N', 'Nao'),
                                                ('A', 'Aviso previo')], required=False)
    disponibilidade = forms.ChoiceField(choices=[('0','Imediata'),('1', 'Uma semana'),
                                              ('2', 'Quinze dias'),('3', 'Trinta dias'),
                                              ('04', 'Mais de trinta dias')], required=False)
        #VENDAS
    vendas_ope = forms.BooleanField(required=False)
    vendas_sup = forms.BooleanField(required=False)
    vendas_ger = forms.BooleanField(required=False)
    vendas_dir = forms.BooleanField(required=False)
        #OFFICE BOY
    boy_ope = forms.BooleanField(required=False)
    boy_sup = forms.BooleanField(required=False)
    boy_ger = forms.BooleanField(required=False)
    boy_dir = forms.BooleanField(required=False)
        #ADMINISTRATIVO
    admin_ope = forms.BooleanField(required=False)
    admin_sup = forms.BooleanField(required=False)
    admin_ger = forms.BooleanField(required=False)
    admin_dir = forms.BooleanField(required=False)
        #FINANCEIRO
    finan_ope = forms.BooleanField(required=False)
    finan_sup = forms.BooleanField(required=False)
    finan_ger = forms.BooleanField(required=False)
    finan_dir = forms.BooleanField(required=False)
        #COMPRAS
    comp_ope = forms.BooleanField(required=False)
    comp_sup = forms.BooleanField(required=False)
    comp_ger = forms.BooleanField(required=False)
    comp_dir = forms.BooleanField(required=False)
        #ESTOQUE
    est_ope = forms.BooleanField(required=False)
    est_sup = forms.BooleanField(required=False)
    est_ger = forms.BooleanField(required=False)
    est_dir = forms.BooleanField(required=False)
        #FISCAL
    fisc_ope = forms.BooleanField(required=False)
    fisc_sup = forms.BooleanField(required=False)
    fisc_ger = forms.BooleanField(required=False)
    fisc_dir = forms.BooleanField(required=False)
        #AMBIENTAL
    amb_ope = forms.BooleanField(required=False)
    amb_sup = forms.BooleanField(required=False)
    amb_ger = forms.BooleanField(required=False)
    amb_dir = forms.BooleanField(required=False)
        #ATENDIMENTO
    atend_ope = forms.BooleanField(required=False)
    atend_sup = forms.BooleanField(required=False)
    atend_ger = forms.BooleanField(required=False)
    atend_dir = forms.BooleanField(required=False)
        #COMUNICACAO
    com_ope = forms.BooleanField(required=False)
    com_sup = forms.BooleanField(required=False)
    com_ger = forms.BooleanField(required=False)
    com_dir = forms.BooleanField(required=False)
        #SUPORTE
    sup_ope = forms.BooleanField(required=False)
    sup_sup = forms.BooleanField(required=False)
    sup_ger = forms.BooleanField(required=False)
    sup_dir = forms.BooleanField(required=False)
        #QUALIDADE
    qual_ope = forms.BooleanField(required=False)
    qual_sup = forms.BooleanField(required=False)
    qual_ger = forms.BooleanField(required=False)
    qual_dir = forms.BooleanField(required=False)
        #CAIXA
    caixa_ope = forms.BooleanField(required=False)
    caixa_sup = forms.BooleanField(required=False)
    caixa_ger = forms.BooleanField(required=False)
    caixa_dir = forms.BooleanField(required=False)
        #NUTRICAO
    nut_ope = forms.BooleanField(required=False)
    nut_sup = forms.BooleanField(required=False)
    nut_ger = forms.BooleanField(required=False)
    nut_dir = forms.BooleanField(required=False)
        #ADVOCACIA
    adv_ope = forms.BooleanField(required=False)
    adv_sup = forms.BooleanField(required=False)
    adv_ger = forms.BooleanField(required=False)
    adv_dir = forms.BooleanField(required=False)
        #SECRETARIADO
    sec_ope = forms.BooleanField(required=False)
    sec_sup = forms.BooleanField(required=False)
    sec_ger = forms.BooleanField(required=False)
    sec_dir = forms.BooleanField(required=False)
        #SERVIÇOS GERAIS
    serv_ope = forms.BooleanField(required=False)
    serv_sup = forms.BooleanField(required=False)
    serv_ger = forms.BooleanField(required=False)
    serv_dir = forms.BooleanField(required=False)
        #SERVIÇOS TECNOLOGIA
    tec_ope = forms.BooleanField(required=False)
    tec_sup = forms.BooleanField(required=False)
    tec_ger = forms.BooleanField(required=False)
    tec_dir = forms.BooleanField(required=False)
        #OUVIDORIA
    ouv_ope = forms.BooleanField(required=False)
    ouv_sup = forms.BooleanField(required=False)
    ouv_ger = forms.BooleanField(required=False)
    ouv_dir = forms.BooleanField(required=False)
        #PLANEJAMENTO
    plan_ope = forms.BooleanField(required=False)
    plan_sup = forms.BooleanField(required=False)
    plan_ger = forms.BooleanField(required=False)
    plan_dir = forms.BooleanField(required=False)

    objetivos = forms.CharField(widget=forms.widgets.Textarea(attrs={'cols':'94', 'rows': '5'}), required=False)
    pretencao = forms.ChoiceField(choices=[('A', 'Ate R$ 510,00'), ('B', 'R$ 511,00 - R$ 1000,00'),
                                           ('C', 'R$ 1001,00 - R$ 2000,00'), ('D', 'R$ 2001,00 - R$ 3000,00'),
                                           ('E', 'R$ 3001,00 - R$ 4000,00'), ('F', 'R$ 4001,00 - R$ 5000,00'),
                                           ('G', 'R$ 5001,00 - R$ 6000,00'), ('H', 'R$ 6001,00 - R$ 10000,00'),
                                          ], required=False)
    outra_cidade = forms.ChoiceField(choices=[('', ''), (True, 'Sim'), (False, 'Nao')], required=False)
    # EMPRESAS 01
    empresa_01 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    segmento_01 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    cargo_01 = forms.CharField(max_length=30, required=False, widget=forms.TextInput(attrs={'size':'30'}))
    modalidade_01 = forms.ChoiceField(choices=[('', ''),('C', 'CONSULTOR'),
                                               ('E', 'EMPREGADO'),('P', 'EMPRESARIO'),
                                               ('A', 'AUTONOMO')], required=False)
    mes_entrada_01 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_entrada_01 = forms.ChoiceField(choices=years, required=False)
    mes_saida_01 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_saida_01 = forms.ChoiceField(choices=years, required=False)
    atribuicoes_01 = forms.CharField(widget=forms.widgets.Textarea(attrs={'cols':'60', 'rows': '2'}), required=False)
    # EMPRESAS 02
    empresa_02 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    segmento_02 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    cargo_02 = forms.CharField(max_length=30, required=False, widget=forms.TextInput(attrs={'size':'30'}))
    modalidade_02 = forms.ChoiceField(choices=[('', ''),('C', 'CONSULTOR'),
                                               ('E', 'EMPREGADO'),('P', 'EMPRESARIO'),
                                               ('A', 'AUTONOMO')], required=False)
    mes_entrada_02 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_entrada_02 = forms.ChoiceField(choices=years, required=False)
    mes_saida_02 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_saida_02 = forms.ChoiceField(choices=years, required=False)
    atribuicoes_02 = forms.CharField(widget=forms.widgets.Textarea(attrs={'cols':'60', 'rows': '2'}), required=False)
    # EMPRESAS 03
    empresa_03 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    segmento_03 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    cargo_03 = forms.CharField(max_length=30, required=False, widget=forms.TextInput(attrs={'size':'30'}))
    modalidade_03 = forms.ChoiceField(choices=[('', ''),('C', 'CONSULTOR'),
                                               ('E', 'EMPREGADO'),('P', 'EMPRESARIO'),
                                               ('A', 'AUTONOMO')], required=False)
    mes_entrada_03 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_entrada_03 = forms.ChoiceField(choices=years, required=False)
    mes_saida_03 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_saida_03 = forms.ChoiceField(choices=years, required=False)
    atribuicoes_03 = forms.CharField(widget=forms.widgets.Textarea(attrs={'cols':'60', 'rows': '2'}), required=False)
    # ESCOLARIDADE
    escolaridade = forms.ChoiceField(choices=[('00','Sem Informacao'),('01', 'Nao sabe ler/escrever'),
                                                ('02', 'Alfabetizado'),('03', 'Fundamental 1o grau incompleto'),
                                                ('04', 'Fundamental 1o grau completo'),('05', 'Medio 2o grau incompleto'),
                                                ('06', 'Medio 2o grau completo'),('07', 'Superior incompleto'),
                                                ('08', 'Superior completo'),('09', 'Especializacao'),
                                                ('10', 'Residencia'),('11', 'Mestrado'),
                                                ('12', 'Doutorado')], required=False)
    idioma_ingles = forms.ChoiceField(choices=[('N','Nao tem conhecimento'),('B','Basico'),('I','Intermediario'),('A','Avancado')], required=False)
    idioma_espanhol = forms.ChoiceField(choices=[('N','Nao tem conhecimento'),('B','Basico'),('I','Intermediario'),('A','Avancado')], required=False)
    idioma_frances = forms.ChoiceField(choices=[('N','Nao tem conhecimento'),('B','Basico'),('I','Intermediario'),('A','Avancado')], required=False)
    idioma_alemao = forms.ChoiceField(choices=[('N','Nao tem conhecimento'),('B','Basico'),('I','Intermediario'),('A','Avancado')], required=False)
    idioma_italiano = forms.ChoiceField(choices=[('N','Nao tem conhecimento'),('B','Basico'),('I','Intermediario'),('A','Avancado')], required=False)
    # CURSOS
    curso_office = forms.ChoiceField(choices=[('N','Nao tem conhecimento'), ('B', 'Baixo'),('I', 'Intermediario'), ('A', 'Avancado')], required=False)
    curso_cad = forms.ChoiceField(choices=[('N','Nao tem conhecimento'), ('B', 'Baixo'),('I', 'Intermediario'), ('A', 'Avancado')], required=False)
    curso_design = forms.ChoiceField(choices=[('N','Nao tem conhecimento'), ('B', 'Baixo'),('I', 'Intermediario'), ('A', 'Avancado')], required=False)
    curso_micros = forms.ChoiceField(choices=[('N','Nao tem conhecimento'), ('B', 'Baixo'),('I', 'Intermediario'), ('A', 'Avancado')], required=False)
    curso_redes = forms.ChoiceField(choices=[('N','Nao tem conhecimento'), ('B', 'Baixo'),('I', 'Intermediario'), ('A', 'Avancado')], required=False)
    curso_programacao = forms.ChoiceField(choices=[('N','Nao tem conhecimento'), ('B', 'Baixo'),('I', 'Intermediario'), ('A', 'Avancado')], required=False)
    # GRADUAÇÂO 01
    tipo_curso_01 = forms.ChoiceField(choices=tipos_curso, required=False)
    instituicao_01 = forms.CharField(max_length=60, required=False, widget=forms.TextInput(attrs={'size':'60'}))
    cidade_01 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    descricao_01 = forms.CharField(max_length=60, required=False, widget=forms.TextInput(attrs={'size':'60'}))
    mes_conclusao_01 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_conclusao_01 = forms.ChoiceField(choices=years, required=False)
    # GRADUAÇÂO 02
    tipo_curso_02 = forms.ChoiceField(choices=tipos_curso, required=False)
    instituicao_02 = forms.CharField(max_length=60, required=False, widget=forms.TextInput(attrs={'size':'60'}))
    cidade_02 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    descricao_02 = forms.CharField(max_length=60, required=False, widget=forms.TextInput(attrs={'size':'60'}))
    mes_conclusao_02 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_conclusao_02 = forms.ChoiceField(choices=years, required=False)
    # GRADUAÇÂO 03
    tipo_curso_03 = forms.ChoiceField(choices=tipos_curso, required=False)
    instituicao_03 = forms.CharField(max_length=60, required=False, widget=forms.TextInput(attrs={'size':'60'}))
    cidade_03 = forms.CharField(max_length=45, required=False, widget=forms.TextInput(attrs={'size':'45'}))
    descricao_03 = forms.CharField(max_length=60, required=False, widget=forms.TextInput(attrs={'size':'60'}))
    mes_conclusao_03 = forms.ChoiceField(choices=MONTHS, required=False)
    ano_conclusao_03 = forms.ChoiceField(choices=years, required=False)


#    class Survey(forms.Form):
#    name = forms.CharField(max_length=20)
#    age = forms.IntegerField()

#    def __init__(self, *args, **kwargs):
#        extra = [1,2,3,4,5]
#        super(LoggedForm, self).__init__(*args, **kwargs)
#        for i, question in enumerate(extra):
#            self.fields['custom_%s' % i] = forms.BooleanField(required=False)




#    multi = forms.MultipleChoiceField(choices=(
#                ('google', 'Google'), ('msn', 'MSN'), ('yahoo',
#        'Yahoo')),
#                required=False, initial=['google'])


#    nascimento = forms.CharField(widget=MonthYearWidget)
#
    


#    foto = forms.CharField(widget=forms.TextInput(attrs={'size':'50'}))
#    estado = forms.ChoiceField(choices=[('1','nome'),('2','idade')])
#    mensagem = forms.CharField(widget=forms.widgets.Textarea, required=False)

    def clean_email_confirm(self):
        if self.cleaned_data['email_confirm'] != self.cleaned_data['email']:
            raise forms.ValidationError('O email de verificacao nao confere.')
        else:
            return self.cleaned_data['email_confirm']
    def clean_fone_cel(self):
        vals = self.data
        if (self.cleaned_data['fone_cel']=='') and (vals['fone_com']=='') and (vals['fone_res'] == ''):
            raise forms.ValidationError('Informe ao menos um telefone para contato.')
        else:
            return self.cleaned_data['fone_cel']

    def clean_cep(self):
        cep = self.cleaned_data['cep']
        if not sock.execute(dbname, uid, pwd, 'cep.loggeral', 'search', [('cep_log','=',cep)]):
            raise forms.ValidationError('CEP Nao encontrado')
        else:
            return self.cleaned_data['cep']

