from django.shortcuts import render_to_response
from django.template import RequestContext
from django import forms
from django.core import validators
from django.http import HttpResponseRedirect
from django.contrib.auth.models import User, UserManager

from peergrab.peergrab.models import Bio

user_validator = validators.MatchesRegularExpression(r'^[a-zA-Z0-9]{3,15}$', "Must be 3-15 alphanumeric characters")

class RegisterManipulator(forms.Manipulator):
  def __init__(self, request):
    self.request = request
    self.fields = (
      forms.TextField(field_name="username", length=15, maxlength=15, is_required=True, validator_list=[user_validator, self.isValidUser]),
      forms.EmailField(field_name="email", length=30, is_required=True, validator_list=[self.isValidUserEmail]),
      forms.PasswordField(field_name="password1", length=30, maxlength=30, is_required=True, validator_list=[user_validator, validators.AlwaysMatchesOtherField('password2', "Password fields don't match")]),
      forms.PasswordField(field_name="password2", length=30, maxlength=30, is_required=True),
      forms.TextField(field_name='captcha', maxlength=10, is_required=True, validator_list=[self.isValidCaptcha])
    )

  def save(self, new_data):
    user = User.objects.create_user(username=new_data['username'],email=new_data['email'],password=new_data['password1'])
    bio = Bio()
    bio.user = user
    bio.save()

  def isValidUser(self, new_data, all_data):
    if (User.objects.filter(username__iexact=new_data)):
      raise validators.ValidationError, "Username exists"

  def isValidUserEmail(self, new_data, all_data):
    if (User.objects.filter(email__iexact=new_data)):
      raise validators.ValidationError, "Account exists with this address"

  def isValidCaptcha(self, new_data, all_data):
    captcha = self.request.session['captcha']
    self.request.session['captcha'] = ''
    if not (len(captcha) and captcha == new_data):
      raise validators.ValidationError, "You must type this field in correctly"

def run(request):
  errors= {}
  new_data = {}
  message = ''

  manipulator = RegisterManipulator(request)

  if request.POST:
    new_data = request.POST.copy()

    errors = manipulator.get_validation_errors(new_data)
    manipulator.do_html2python(new_data)

    if not errors:
      manipulator.save(new_data)
      request.session['message'] = 'Account created'
      new_data = {};
      return HttpResponseRedirect('/login/?message=Your account has been created please sign in.')
    else:
      new_data['captcha'] = ''

  form = forms.FormWrapper(manipulator, new_data, errors)
  return render_to_response('signup.html', {'form':form, 'message':message})
