# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from SigUtils.formatastring import formataString

class rhfolha_eventos(osv.Model):
    _inherit = 'rhfolha.eventos'

    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return {}
        context = context and context.copy() or {}
        reads = self.read(cr, uid, ids, ['rheven_codigo', 'rheven_name'], context=context)
        res = []
        for record in reads:
            name = '%s - %s' % (formataString(record['rheven_codigo'], 4, '0', 'E'), record['rheven_name'])
            res.append((record['id'], name))
            if context.get('rheven_so_codigo'):
                res.append((record['id'], record['rheven_codigo']))
            if context.get('rheven_sem_codigo'):
                res.append((record['id'], record['rheven_name']))
        return res

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        context = context and context.copy() or {}
        ids = []
        try:
            codigo = int(name)
        except:
            codigo = False
        if codigo:
            ids = self.search(cr, user, [('rheven_codigo', '=', codigo)], limit=limit, context=context)
        if not ids:
            ids = self.search(cr, user, [('rheven_name', operator, name)] + args, limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    def rheven_onchange_codigo(self, cr, uid, ids, codigo_evento):
        """
        Verifica o codigo do evento e determina os parametros relacionados conforme regra:
        - de 1 a 1000 - Vencimentos
            - 1 Evento automatico "Salario Base"
            - de 2 a 900 - Eventos manuais
            - de 901 a 1000 - Eventos automaticos
        - de 1001 a 2000 - Descontos
            - de 1001 a 1900 - Eventos manuais
            - de 1901 a 2000 - Eventos automaticos
        @param codigo_evento: {int} Codigo do evento.
        @return: {dict} automatico/manual e vencimento/descontos
        """
        value = {}
        msg = {}
        if codigo_evento < 1 or codigo_evento > 2000:
            msg['title'] = _('Atencao')
            msg['message'] = _('Codigo de evento invalido!   Informe um valor entre 1 e 2.000!')
            value['rheven_codigo'] = ''
            return {'value': value, 'warning': msg}
        elif codigo_evento < 1001:
            value['rheven_venc_desc'] = 'V'
        else:
            value['rheven_venc_desc'] = 'D'
        if codigo_evento < 901:
            value['rheven_processamento'] = 'V'
        elif codigo_evento < 1001:
            value['rheven_processamento'] = 'A'
        elif codigo_evento < 1901:
            value['rheven_processamento'] = 'V'
        else:
            value['rheven_processamento'] = 'A'
        if codigo_evento == 1:
            value['rheven_name'] = 'Salario base'
            value['rheven_processamento'] = 'R'
        return {'value': value}