# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class rhfolha_wiz_processar_folha(osv.TransientModel):
    _name = 'rhfolha.wiz.processar.folha'
    _columns = {
        'ano': fields.integer('Ano', required=True),
        'mes': fields.integer('Mes', required=True),
    }

    def button_processar(self, cr, uid, ids, context=None):
        """
        Funcao para calcular a folha de pagamento
        variaveis pre definidas para calculos em formulas
        SAL_BASE - Salario base do funcionario
        HORAS_SEMANAIS - Horas semanais do cadastro de funcionario
        REFER - Definido como referencia em cada evento
        SAL_HORA - Valor de uma hora trabalhada do funcionario
        QTD_ATE_14 - Quantidade de filhos ate 14 anos
        PERCENTUAL = percentual cadastrado no evento
        TOT_VENCIMENTOS = Total dos vencimentos do funcionario
        TOT_DESCONTOS = Total dos descontos do funcionario
        """

        def acumular_valores(liq, venc, desc):
            liq += val_vencimento - val_desconto
            desc += val_desconto
            venc += val_vencimento
            return liq, venc, desc

        def _get_qtd_filhos_ate_14():
            _qtd = 0
            for relacionamento in brw_lanc_capa.rhlanc_funcionario_id.relacionamento_ids:
                if relacionamento.relacionamento_tipo_id.id != rel_id:
                    continue
                if relacionamento.admpre_filho_idade < 15:
                    _qtd += 1
            return _qtd

        def _calcular_bases(base_inss, base_fgts, base_irrf):
            if brw_lanc_itens.rhlani_evento_id.rheven_irrf:
                base_irrf += val_vencimento
                base_irrf -= val_desconto
            if brw_lanc_itens.rhlani_evento_id.rheven_inss:
                base_inss += val_vencimento
                base_inss -= val_desconto
            if brw_lanc_itens.rhlani_evento_id.rheven_fgts:
                base_fgts += val_vencimento
                base_fgts -= val_desconto
            return (base_inss, base_fgts, base_irrf)

        def _proc_0001():
            """
            Calculo do salario base
            @return: float - O valor do salario
            """
            if val_referencia == 30:
                return SAL_BASE
            return round(SAL_BASE / 30 * REFER, 2)

        def _proc_0901(rendimentos):
            """
            Calculo do salario familia
            @return: float - O valor do salario familia
            """
            _valor = 0
            for dados_faixa in dados_sal_fam:
                if rendimentos > dados_faixa.get('rhsfam_limite'):
                    continue
                _valor = dados_faixa.get('rhsfam_valor')
                break
            _valor = _valor * QTD_ATE_14
            return _valor

        def _proc_1002(valor):
            """
            Calculo do vale transporte
            @return: float - O valor do vale transporte a ser deduzido
            """
            _limite = SAL_BASE * 6 / 100
            if _limite > valor:
                return _limite
            else:
                return valor

        def _calcular_imposto(imposto, base):
            _vl = 0.00
            if imposto == 'IRRF':
                for dados_faixa in dados_irrf:
                    if base > dados_faixa.get('rhirrf_limite'):
                        continue
                    _vl = (base * dados_faixa.get('rhirrf_aliquota') / 100) - dados_faixa.get(
                        'rhirrf_deducao')
                    _ded = deducao_dependente * brw_lanc_capa.rhlanc_funcionario_id.rhfunc_dependentes_irrf
                    if _vl > _ded:
                        _vl = _vl - _ded
                    else:
                        _vl = 0
                    break
            if imposto == 'INSS':
                for dados_faixa in dados_inss:
                    if base > dados_faixa.get('rhinss_limite'):
                        continue
                    _vl = (base * dados_faixa.get('rhinss_aliquota') / 100)
                    break
            if imposto == 'FGTS':
                _vl = base * aliquota_fgts / 100
            return _vl

        _lancar_financ = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'rh', [
            'rhpar_gerar_financeiro'], context=context).get(
            'rhpar_gerar_financeiro', False)
        context = context and context.copy() or {}
        _msg = ''
        # Validando dados da tela
        dados = self.read(cr, uid, ids, [], context=context)[0]
        if dados.get('ano') < 2014 or dados.get('ano') > 2100:
            raise osv.except_osv(_('Atencao'), _('Ano invalido!'))
        if dados.get('mes') < 1 or dados.get('mes') > 12:
            raise osv.except_osv(_('Atencao'), _('Mes invalido!'))
        #
        aliquota_fgts = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'rh', [
            'rhpar_aliquota_fgts'], context=context).get(
            'rhpar_aliquota_fgts', 0.0)
        irrf_ids = self.pool.get('rhfolha.irrf').search(cr, uid, [], context=context)
        rel_id = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'rh', [
            'rhpar_relacionamento_filhos_id'], context=context).get(
            'rhpar_relacionamento_filhos_id', 0)
        deducao_dependente = self.pool.get('adm.utils').admutl_get_parametros(cr, uid, 'rh', [
            'rhpar_deducao_dependente'], context=context).get(
            'rhpar_deducao_dependente', 0.0)
        dados_irrf = self.pool.get('rhfolha.irrf').read(cr, uid, irrf_ids, context=context)
        inss_ids = self.pool.get('rhfolha.inss').search(cr, uid, [], context=context)
        dados_inss = self.pool.get('rhfolha.inss').read(cr, uid, inss_ids, context=context)
        sal_fam_ids = self.pool.get('rhfolha.salario.familia').search(cr, uid, [], context=context)
        dados_sal_fam = self.pool.get('rhfolha.salario.familia').read(cr, uid, sal_fam_ids, context=context)
        _lanc_capa_ids = self.pool.get('rhfolha.lancamentos.capa').search(cr, uid, [], context=context)
        for brw_lanc_capa in self.pool.get('rhfolha.lancamentos.capa').browse(cr, uid,
                                                                              _lanc_capa_ids, context=context):
            SAL_BASE = brw_lanc_capa.rhlanc_funcionario_id.rhfunc_salario
            HORAS_SEMANAIS = brw_lanc_capa.rhlanc_funcionario_id.rhfunc_horas_semanais
            QTD_ATE_14 = _get_qtd_filhos_ate_14()
            SAL_HORA = SAL_BASE / (HORAS_SEMANAIS * 5)
            TOT_VENCIMENTOS = TOT_DESCONTOS = 0.0
            if not brw_lanc_capa.rhlanc_funcionario_id.active:
                continue  # funcionario inativo, nao calcular
            _obj_rhfolha_folha_capa = self.pool.get('rhfolha.folha.capa')
            _obj_rhfolha_folha_itens = self.pool.get('rhfolha.folha.itens')
            _args = []
            _args.append(('rhfolc_funcionario_id', '=', brw_lanc_capa.rhlanc_funcionario_id.id))
            _args.append(('rhfolc_ano', '=', dados.get('ano')))
            _args.append(('rhfolc_mes', '=', dados.get('mes')))
            base_inss = base_fgts = base_irrf = liquido = 0.0
            folha_ids = _obj_rhfolha_folha_capa.search(cr, uid, _args, context=context)
            if folha_ids:
                # folha ja calculada
                return True
            vals = {
                'rhfolc_funcionario_id': brw_lanc_capa.rhlanc_funcionario_id.id,
                'rhfolc_ano': dados.get('ano'),
                'rhfolc_mes': dados.get('mes'),
            }
            _capa_id = _obj_rhfolha_folha_capa.create(cr, uid, vals, context)
            fult_tit_custificacao_dados = []
            for brw_lanc_itens in brw_lanc_capa.rhlanc_lancamento_ids:
                val_evento = brw_lanc_itens.rhlani_evento_id.rheven_codigo
                REFER = val_referencia = brw_lanc_itens.rhlani_referencia
                PERCENTUAL = val_referencia = brw_lanc_itens.rhlani_evento_id.rheven_percentual
                val_vencimento = val_desconto = 0.0
                if val_evento == 1:  # Salario base
                    val_vencimento = _proc_0001()
                elif val_evento == 1002:  # Vale transporte
                    val_desconto = _proc_1002(brw_lanc_itens.rhlani_desconto)
                elif brw_lanc_itens.rhlani_evento_id.rheven_processamento == 'V':
                    val_vencimento = brw_lanc_itens.rhlani_vencimento
                    val_desconto = brw_lanc_itens.rhlani_desconto
                elif brw_lanc_itens.rhlani_evento_id.rheven_processamento == 'A':
                    pass  # implementar processamento automatico
                else:
                    try:
                        _vl = round(eval(brw_lanc_itens.rhlani_evento_id.rheven_formula), 2)
                        if brw_lanc_itens.rhlani_evento_id.rheven_venc_desc == 'V':
                            val_vencimento = _vl
                        else:
                            val_desconto = _vl
                    except:
                        _obj_rhfolha_folha_capa.unlink(cr, uid, _capa_id)
                        _msg += '\n Erro calculando o evento %s do funcionario %s' % (
                            val_evento, brw_lanc_capa.rhlanc_funcionario_id.id)
                base_inss, base_fgts, base_irrf = _calcular_bases(base_inss, base_fgts, base_irrf)
                # Gravar itens da folha
                vals = {
                    'rhfoli_capa_id': _capa_id,
                    'rhfoli_evento_id': brw_lanc_itens.rhlani_evento_id.id,
                    'rhfoli_evento_codigo': brw_lanc_itens.rhlani_evento_id.rheven_codigo,
                    'rhfoli_referencia': REFER,
                    'rhfoli_vencimento': val_vencimento,
                    'rhfoli_desconto': val_desconto,
                }
                _obj_rhfolha_folha_itens.create(cr, uid, vals, context=context)
                fult_tit_custificacao_dados.append({'centro_custo_id': brw_lanc_capa.rhlanc_funcionario_id.centro_custo,
                                                    'evento_id': brw_lanc_itens.rhlani_evento_id.rheven_evento_id.id,
                                                    'valor': val_vencimento or val_desconto,
                                                    'custf_historico': brw_lanc_itens.rhlani_evento_id.rheven_evento_id.name})
                liquido, TOT_VENCIMENTOS, TOT_DESCONTOS = acumular_valores(liquido, TOT_VENCIMENTOS, TOT_DESCONTOS)

            # Calcular salario familia
            val_vencimento = _proc_0901(base_inss)
            val_desconto = 0.0
            if val_vencimento:
                _evento_id = \
                self.pool.get('rhfolha.eventos').search(cr, uid, [('rheven_codigo', '=', 901)], context=context)[0]
                brw_evento = self.pool.get('rhfolha.eventos').browse(cr, uid, _evento_id)
                vals = {
                    'rhfoli_capa_id': _capa_id,
                    'rhfoli_evento_id': _evento_id,
                    'rhfoli_evento_codigo': 901,
                    'rhfoli_referencia': 0,
                    'rhfoli_vencimento': val_vencimento,
                    'rhfoli_desconto': val_desconto,
                }
                _obj_rhfolha_folha_itens.create(cr, uid, vals, context=context)
                fult_tit_custificacao_dados.append({'centro_custo_id': brw_lanc_capa.rhlanc_funcionario_id.centro_custo,
                                                    'evento_id': brw_evento.rheven_evento_id.id,
                                                    'valor': val_vencimento or val_desconto,
                                                    'custf_historico': brw_evento.rheven_evento_id.name})
                liquido, TOT_VENCIMENTOS, TOT_DESCONTOS = acumular_valores(liquido, TOT_VENCIMENTOS, TOT_DESCONTOS)

            # Calcular bases irrf/inss/fgts
            if base_irrf:
                _vl = _calcular_imposto('IRRF', base_irrf)
                if _vl:
                    _evento_id = \
                    self.pool.get('rhfolha.eventos').search(cr, uid, [('rheven_codigo', '=', 1903)], context=context)[0]
                    brw_evento = self.pool.get('rhfolha.eventos').browse(cr, uid, _evento_id)
                    vals = {
                        'rhfoli_capa_id': _capa_id,
                        'rhfoli_evento_id': _evento_id,
                        'rhfoli_evento_codigo': 1903,
                        'rhfoli_referencia': 0,
                        'rhfoli_desconto': _vl,
                        'rhfoli_vencimento': 0.0,
                    }
                    _obj_rhfolha_folha_itens.create(cr, uid, vals, context=context)
                    fult_tit_custificacao_dados.append(
                        {'centro_custo_id': brw_lanc_capa.rhlanc_funcionario_id.centro_custo,
                         'evento_id': brw_evento.rheven_evento_id.id,
                         'valor': val_vencimento or val_desconto,
                         'custf_historico': brw_evento.rheven_evento_id.name})
                    liquido -= _vl
            if base_inss:
                _vl = _calcular_imposto('INSS', base_inss)
                if _vl:
                    _evento_id = \
                    self.pool.get('rhfolha.eventos').search(cr, uid, [('rheven_codigo', '=', 1901)], context=context)[0]
                    brw_evento = self.pool.get('rhfolha.eventos').browse(cr, uid, _evento_id)
                    vals = {
                        'rhfoli_capa_id': _capa_id,
                        'rhfoli_evento_id': _evento_id,
                        'rhfoli_evento_codigo': 1901,
                        'rhfoli_referencia': 0,
                        'rhfoli_desconto': _vl,
                        'rhfoli_vencimento': 0.0,
                    }
                    _obj_rhfolha_folha_itens.create(cr, uid, vals, context=context)
                    fult_tit_custificacao_dados.append(
                        {'centro_custo_id': brw_lanc_capa.rhlanc_funcionario_id.centro_custo,
                         'evento_id': brw_evento.rheven_evento_id.id,
                         'valor': val_vencimento or val_desconto,
                         'custf_historico': brw_evento.rheven_evento_id.name})
                    liquido -= _vl
            if base_fgts:
                _vl = _calcular_imposto('FGTS', base_fgts)
                if _vl:
                    _evento_id = \
                    self.pool.get('rhfolha.eventos').search(cr, uid, [('rheven_codigo', '=', 1902)], context=context)[0]
                    brw_evento = self.pool.get('rhfolha.eventos').browse(cr, uid, _evento_id)
                    vals = {
                        'rhfoli_capa_id': _capa_id,
                        'rhfoli_evento_id': _evento_id,
                        'rhfoli_evento_codigo': 1902,
                        'rhfoli_referencia': 0,
                        'rhfoli_desconto': _vl,
                        'rhfoli_vencimento': 0.0,
                    }
                    _obj_rhfolha_folha_itens.create(cr, uid, vals, context=context)
                    fult_tit_custificacao_dados.append(
                        {'centro_custo_id': brw_lanc_capa.rhlanc_funcionario_id.centro_custo,
                         'evento_id': brw_evento.rheven_evento_id.id,
                         'valor': val_vencimento or val_desconto,
                         'custf_historico': brw_evento.rheven_evento_id.name})
                    liquido -= _vl
            vals = {
                'rhfolc_base_inss': base_inss,
                'rhfolc_base_fgts': base_fgts,
                'rhfolc_base_irrf': base_irrf,
                'rhfolc_liquido': liquido,
            }
            _obj_rhfolha_folha_capa.write(cr, uid, _capa_id, vals, context)
            if _lancar_financ:
                obj_financeiro_utils = self.pool.get('financeiro')
                _campos_adicionais = {'fintit_folha_id': _capa_id}
                res_id = obj_financeiro_utils.financ_titulo_entrada(cr, uid,
                                                                    fult_tit_pessoa_id=1,
                                                                    fult_tit_emissao=1,
                                                                    fult_tit_vencimento=1,
                                                                    fult_tit_documento=1,
                                                                    fult_tit_parcela=1,
                                                                    fult_tit_valor=1,
                                                                    fult_tit_tipo_lancamento=1,
                                                                    fult_tit_tipo_titulo='P',
                                                                    fult_tit_custificacao_dados=fult_tit_custificacao_dados,
                                                                    context=context,
                                                                    campos_adicionais=_campos_adicionais)

        return True
