#!/usr/bin/python
# -*- encoding: utf-8 -*-

import socket
import time
import binascii
from excecoes import ConnectTimeout, SendCommandTimeout



class SigArgosSocket:
    DEFAULT_PORT = 3000
    DEFAULT_TIMEOUT = 3  # socket timeout (connect included)
    DEFAULT_MAX_TRIES = 5  # how many times the operation should be tried
    DEFAULT_SLEEP_BETWEEN_TRIES = 0.5  # how much to wait between tries.
    BUFFER_SIZE = 2048
    TIMESTAMP_MASK = "%d/%m/%y %H:%M:%S"
    TIMESTAMP_MASK_EVENTS = (
        "%d/%m/%Y %H:%M:%S"
    )  # the protocol needs yyyy format just in events messages...

    def __init__(
        self,
        address,
        port=DEFAULT_PORT,
        timeout=DEFAULT_TIMEOUT,
        max_tries=DEFAULT_MAX_TRIES,
        sleep_between_tries=DEFAULT_SLEEP_BETWEEN_TRIES,
    ):
        self.address = address
        self.port = port
        self.timeout = timeout
        self.max_tries = max_tries
        self.sleep_between_tries = sleep_between_tries
        self.tries = 1
        self.socket = self.config_socket()

    def send_command(self, command, tries=DEFAULT_MAX_TRIES, timeout=DEFAULT_TIMEOUT):
        self.socket.settimeout(timeout)
        if tries == 0:
            raise SendCommandTimeout(self.address, command)
        try:
            tries -= 1
            self.socket.sendall(command.bytes())
            # print('Send:', command.bytes())
            return command.payload()
        except socket.timeout:
            time.sleep(self.sleep_between_tries)
            return self.send_command(command, tries)

    def config_socket(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(self.timeout)
        return sock

    def receive(self):
        try:
            raw_response = ""
            while True:
                partial_response = self.socket.recv(self.BUFFER_SIZE)
                raw_response += partial_response
                if self.check_response_integrity(raw_response):
                    return raw_response
        except socket.timeout:
            raise SendCommandTimeout(self.address, 'event')

    @staticmethod
    def check_response_integrity(raw_response):
        return binascii.hexlify(raw_response[0]) == '02' and binascii.hexlify(raw_response[-1]) == '03'

    def connect(self):
        self.socket = self.config_socket()
        try:
            self.socket.connect((self.address, self.port))
            self.tries = 1
        except socket.timeout:
            if self.tries < self.max_tries or self.max_tries == 0:
                self.tries += 1
                time.sleep(self.sleep_between_tries)
                self.connect()
            else:
                self.tries = 0
                raise ConnectTimeout(self.address, self.port, self.tries)

    def close(self):
        self.socket.close()
