#!/usr/bin/python
# -*- encoding: utf-8 -*-

from resposta import Resp
from argos import SigArgosSocket
import binascii
from excecoes import SendCommandTimeout
from SigUtils import datas
from openerp.tools.translate import _
import logging
logger = logging.getLogger('Rel.Henry')


class SigCommand:
    status = True
    erro = ''
    BYTE_INIT = "02"
    BYTE_END = "03"
    BYTE_START_MESSAGE = "00"

    # response = Resp

    def payload(self):  # pylint: disable=no-self-use
        return ""

    def bytes(self):
        message = self.message()
        return bytearray.fromhex(message)

    def __repr__(self):
        return self.__class__.__name__

    def message(self):
        """
        A mensagem vai no formato:
        INIT_BYTE + MESSAGE_SIZE + BYTE_START_MESSAGE + MESSAGE + CHECKSUM + END_BYTE

        INIT_BYTE = Identifica o início da transmissão, default 0x02
        MESSAGE_SIZE = Quantidade de bytes da payload, dois bytes. o menos significativo fica
        à esquerda, contrariando o padrão. Por ex uma mensagem com payload 421 bytes ficaria
        com o size A5 01.
        CHECKSUM = Representa a integridade da mensagem (ver método self.checksum())
        END_BYTE = Indifica o fim da transmissão, default 0x00

        Todos os valores são representados em uma string hexadecimal, sem separadores.
        A string é convertida para um bytearray ao final do processo, pelo socket.
        """
        payload = self.payload()
        logger.info("Comando enviado:  %s" % payload)
        payload_size = self.payload_size(
            payload
        )  # hexa com a garantia do "0" na frente
        if not isinstance(payload, bytes):
            payload = payload.encode()
        payload_hex = binascii.hexlify(payload)  # pylint: disable=no-member
        start_hex = "{BYTE_INIT}{payload_size}".format(
            BYTE_INIT=SigCommand.BYTE_INIT,
            payload_size=payload_size
        )
        checksum_hex = self.checksum(start_hex, payload_hex)
        end_hex = "{checksum_hex}{BYTE_END}".format(
            checksum_hex=checksum_hex,
            BYTE_END=SigCommand.BYTE_END
        )
        return "{start_hex}{payload_hex}{end_hex}".format(
            start_hex=start_hex,
            payload_hex=payload_hex,
            end_hex=end_hex
        )

    @staticmethod
    def checksum(start_hex, text_hex):
        byte_array = bytearray.fromhex(start_hex + text_hex)
        checksum_byte = byte_array[1]
        for byte in byte_array[2:]:
            checksum_byte = checksum_byte ^ byte
        checksum_byte = checksum_byte & 0xFF
        return "{checksum_byte:02X}".format(checksum_byte=checksum_byte)

    @staticmethod
    def payload_size(payload):
        hex_size = "{res:02X}".format(res=len(payload)).zfill(4)
        str_chunk_size = int(len(hex_size) / 2)
        firstbyte, secondbyte = (
            hex_size[:str_chunk_size],
            hex_size[str_chunk_size:],
        )  # the protocol says the most significant byte should come after (??)
        return secondbyte + firstbyte


class GetEvento():
    def __init__(self, sdk):
        self.sdk = sdk
        self.retorno = ''
        self.data = ''
        self.data_payload = {}
        self.index = False
        """
        Status:
            0 - Executado,
            1 - A executar,
            2 - Timeout,
            3 - Erro
        """
        self.status = 1
        try:
            self.retorno = self.sdk._socket.receive()
            self.status = 0
        except (
            SendCommandTimeout
        ) as e:
            self.status = 2
            return
        except Exception as e:
            print('Erro de except:', e)
            self.status = 3
            return
        r = Resp(self.retorno)
        self.data = r.data
        self.data_payload = r.data_payload
        self.index = self.data.get('index')
        self.comando = self.data.get('comando')
        if int(r.data.get('message_status')) != 0:
            self.status = 3
            self.erro = r.error_message()
        logger.info('Retorno: %s' % str(self.data.get('dados')))


class ComandaLiberarNegarResponse(Resp):
    response_mapping = {"payload": (14, -2, True), "type": (12, 13, False)}


class sigSetHorario(SigCommand):

    def __init__(self, timestamp):
        self.timestamp = timestamp

    def payload(self):
        timestamp_string = self.timestamp.strftime(SigArgosSocket.TIMESTAMP_MASK)
        message = "01+EH+00+{timestamp_string}]00/00/00]00/00/00".format(timestamp_string=timestamp_string)
        return message

class sigGetHorario(SigCommand):

    def payload(self):
        message = "01+RH+00"
        return message

class sigGetDigitais(SigCommand):

    def payload(self):
        message = "01+RD+00"
        return message

class sigGetUsuarios(SigCommand):

    def __init__(self, qtd_usuarios=1, indice_usuario=0):
        self.qtd_usuarios = qtd_usuarios
        self.indice_usuario = indice_usuario

    def payload(self):
        if self.indice_usuario == 0:
            _usu = self.indice_usuario
        else:
            _usu = '-1]%s' % self.indice_usuario
        message = "01+RU+00+%s]%s" % (self.qtd_usuarios, _usu)
        return message

class sigSetUsuarios(SigCommand):

    def __init__(self, operacao, nome=None, pis=None, cartao=None):
        self.operacao = operacao
        self.nome = nome
        self.pis = pis
        self.cartao = cartao
        if self.operacao not in ('I', 'A', 'E'):
            self.status = False
            self.erro =  _('Operacao invalida!')
        faltou = []
        if not self.nome:
            faltou.append(_('Nome'))
        if not self.pis:
            faltou.append(_('Pis'))
        if not self.cartao:
            faltou.append(_('Cartao'))
        if faltou:
            self.status = False
            self.erro = _('Faltou a seguinte informacao:')
            for x in faltou:
                self.erro += x + ',\n'
            self.erro = self.erro[:-2]  # Tirando ",\n" do final


    def payload(self):
        message = "01+EU+00+1+%s[%s[%s[0[1[%s" % (self.operacao, self.pis, self.nome, self.cartao)
        return message

class sigGetEventos(SigCommand):

    """
    Operacao:
    - T não funciona com indice, ao inves disto use 'N'
    - D data funciona

    """
    def __init__(self, operacao='T', qtd_eventos=50, indice_evento=1, dataInicial=None, dataFinal=None):
        self.operacao = operacao
        self.qtd_eventos = qtd_eventos
        self.indice_evento = indice_evento
        self.dataInicial = datas.getDataNoFormato(dataInicial, 'N') + ' 00:00:00'
        if dataFinal:
            self.dataFinal = datas.getDataNoFormato(dataFinal, 'N') + ' 23:59:59'
        else:
            self.dataFinal = None

    def payload(self):
        if self.operacao == 'D':
            if self.dataFinal:
                message = "01+RR+00+%s]%s]%s/%s" % (self.operacao, self.qtd_eventos, self.dataInicial, self.dataFinal)
            else:
                message = "01+RR+00+%s]%s]%s]" % (self.operacao, self.qtd_eventos, self.dataInicial)
        else:  # operacao = 'N'
            message = "01+RR+00+%s]%s]%s" % (self.operacao, self.qtd_eventos, self.indice_evento)
        return message
