#!/usr/bin/python
# -*- encoding: utf-8 -*-


class ConnectTimeout(Exception):
    def __init__(self, address, port=None, tries=None):
        self.address = address
        self.port = port
        self.tries = tries
        super(ConnectTimeout, self).__init__(address)

    def __repr__(self):
        return "failed to connect to {address}:{port}. Gave up after {tries} tries".format(
            address=self.address,
            port=self.port,
            tries=self.tries
        )

    def __str__(self):
        return self.__repr__()


class SendCommandTimeout(Exception):
    def __init__(self, address, command):
        self.address = address
        self.command = command
        super(SendCommandTimeout, self).__init__(address)

    def __repr__(self):
        return "failed to send command {command} to {address}. Timeout".format(
            command=self.command,
            address=self.address
        )

    def __str__(self):
        return self.__repr__()


class GenericErrorResponse(Exception):
    def __init__(self, message_id, error_message):
        self.message_id = message_id
        self.error_message = error_message
        super(SendCommandTimeout, self).__init__(error_message)

    def __repr__(self):
        return "The biometric reader returned an error: {message_id}: {error_message}".format(
            message_id=self.message_id,
            error_message=self.error_message
        )

    def __str__(self):
        return self.__repr__()

