#!/usr/bin/python
# -*- encoding: utf-8 -*-
import binascii
from openerp.tools.translate import _


class Resp:
    fields_payload = ()
    response_mapping = {}
    index = ''
    default_response_mapping = {
        "init": (0, 1, True),
        "size": (1, 2, True),
        "naoIdentificado": (2, 3, True),
        "dados": (3, -2, False),
        "checksum": (-2, -1, True),
    }
    fp = {
        'RH': ('data_hora', 'data_inicio_hor_verao', 'data_fim_hor_verao'),
        'REON': ('status_pedido', 'comanda', 'data_hora', 'sentido', 'situacao', 'tipo_entrada'),
        'RU': ('pis', 'nome', 'reservado', 'referencia', 'cartoes'),
        'RR2': ('NSR', 'tipo', 'data', 'horario', 'tipoempregador', 'cnpj', 'cei', 'razao', 'local'),
        'RR3': ('NSR', 'tipo', 'data', 'horario', 'pis'),
        'RR4': ('NSR', 'tipo', 'data', 'horario', 'dataajustada', 'horaajustada'),
        'RR5': ('NSR', 'tipo', 'data', 'horario', 'operacao', 'pis', 'nome'),
        'EU': ('message_status',),
    }
    fp_RR_pos = {
        '2': (0, 9, 10, 18, 22, 23, 37, 49, 199, -1),
        '3': (0, 9, 10, 18, 22, -1),
        '4': (0, 9, 10, 18, 22, 30, -1),
        '5': (0, 9, 10, 18, 22, 30, 42, -1),
    }

    error_messages = {
        "00": _("Nao ha erros"),
        "10": _("Comando desconhecido"),
        "11": _("Tamanho do pacote invalido"),
        "12": _("Parametros informados invalidos"),
        "13": _("Erro de Checksum"),
        "26": _("Index biometrico nao encontrado"),
        "29": _("Erro na manipulacao de biometrias"),
        "32": _("O IP é invalido"),
        "34": _("Identificador do empregado e invalido"),
        "37": _("Referencia de cartao de usuario e invalida"),
        "63": _("Opcao invalida"),
    }

    def __init__(self, raw_response):
        self.data_payload = {}
        self.fields_payload = ()
        self.raw_response = raw_response
        self.data = self.parse(raw_response)
        print('Resp', self.data)

    def parse(self, raw_response):
        return self._parse(raw_response)

    def _parse(self, raw_response):

        response_mapping = dict(self.default_response_mapping, **self.response_mapping)
        result = {}
        for field, (start, end, as_hex) in response_mapping.items():
            bytea = raw_response[start:end]
            if as_hex:
                value = binascii.hexlify(bytea)
            else:
                value = bytea.decode()
            result[field] = value
            if field == 'dados':
                _comandos = value.split('+')
                result['index'] = _comandos[0]
                result['comando'] = _comandos[1]
                result['message_status'] = '00'
                _dados = _comandos[-1].split(']')
                lista = False
                _newDados = []
                for x in _dados:
                    if '[' in x:
                        lista = True
                        _newDados.append(x.split('['))
                    if '\r\n' in x:
                        lista = True
                        _newDados = x.split('\r\n')
                if lista:
                    _dados = _newDados
                _comandos.pop(-1)
                self.fields_payload = self.fp.get(_comandos[1], ())
                if lista:
                    self.data_payload = []
                    if result.get('comando') == 'RR':
                        for x in _dados:
                            if not x:
                                continue
                            _registro = x[9:10]
                            _codigo = '%s%s' % (result.get('comando'), _registro)
                            d = {}
                            i = 0
                            for k in self.fp.get(_codigo):
                                _pos_i = self.fp_RR_pos.get(_registro)[i]
                                i += 1
                                _pos_f = self.fp_RR_pos.get(_registro)[i]
                                d[k] = x[_pos_i: _pos_f]
                            self.data_payload.append(d)
                    else:
                        for x in _dados:
                            d = dict(zip(self.fields_payload, x))
                            self.data_payload.append(d)
                else:
                    self.data_payload = dict(zip(self.fields_payload, _dados))
        if not isinstance(self.data_payload, list):
            if self.data_payload.get('message_status'):
                result['message_status'] = self.data_payload.get('message_status')
        return result

    def error_message(self):
        message_status = self.data.get("message_status")
        return self.error_messages.get(message_status, "Unknown")

    def __repr__(self):
        if self.data.get("payload") is not None:
            return repr(self.data["payload"])
        return repr(self.data)

    def __len__(self):
        return len(self.data)

    def __getitem__(self, key):
        if hasattr(self, key):
            return getattr(self, key)
        return self.data[key]

