#!/usr/bin/python
# -*- encoding: utf-8 -*-
from argos import SigArgosSocket
from comandos import SigCommand, sigSetHorario, sigGetHorario, sigGetDigitais, sigGetUsuarios
from comandos import sigGetEventos, sigSetUsuarios


class SDK:
    # codes to command get_quantity
    QT_USERS = "U"
    QT_CARDS = "C"
    QT_FINGERPRINTS = "D"
    QT_MAX_FINGERPRINTS = "TD"

    # to send cards:
    # Given the card_number is unique, UPDATE or INSERT not seems to change anything

    SEND_INSERT = "I"
    SEND_UPDATE = "A"
    SEND_DELETE = "E"  # complete wipe out, be careful
    MASTER_MODE = "6"  # 6 to set master mode.
    NORMAL_MODE = "1"  # 6 to set master mode.

    def __init__(self, address, porta, **args):
        self._socket = SigArgosSocket(address, port=porta, **args)

    def __enter__(self):
        self._socket.connect()
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self._socket.close()

    def send_command(self,
        command=SigCommand,
        tries=int(SigArgosSocket.DEFAULT_MAX_TRIES),
        timeout=int(SigArgosSocket.DEFAULT_TIMEOUT),
    ):
        """Sends a command to the socket, that will take care of transforming
        it into bytes and adding metadata adhering the protocol. This method
        should not be called by the end user, since probably there's a method in
        this class that prepare and send it to you."""
        resp = self._socket.send_command(command, tries, timeout)
        return resp, command

    def set_horario(self, timestamp, **args):
        dados = sigSetHorario(timestamp=timestamp)
        self.status = dados.status
        self.erro = dados.erro
        return self.send_command(dados, **args)
    def get_horario(self, **args):
        dados = sigGetHorario(**args)
        self.status = dados.status
        self.erro = dados.erro
        return self.send_command(dados)

    def get_digitais(self, **args):
        dados = sigGetDigitais(**args)
        self.status = dados.status
        self.erro = dados.erro
        return self.send_command(dados)

    def get_usuarios(self, **args):
        dados = sigGetUsuarios(**args)
        self.status = dados.status
        self.erro = dados.erro
        res = self.send_command(dados)
        return res

    def set_usuarios(self, **args):
        dados = sigSetUsuarios(**args)
        self.status = dados.status
        self.erro = dados.erro
        res = self.send_command(dados)
        return res

    def get_eventos(self, **args):
        dados = sigGetEventos(**args)
        self.status = dados.status
        self.erro = dados.erro
        return self.send_command(dados)
