# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from SigUtils.pis import Pis
from openerp.tools.translate import _
from ..comunicacao.sdk import SDK
from ..comunicacao.comandos import GetEvento
from SigUtils import datas
from datetime import datetime


class wiz_rhhenry_importar_marcacao(osv.Model):
    # prefixo: wizrhim

    _name = 'wiz.rhhenry.importar.marcacao'
    _columns = {
        'wizrhim_relogio_id': fields.many2one('rhponto.relogios', 'Relogio', required=True, ondelete='cascade'),
        'wizrhim_data_inicial': fields.date('Data inicial', required=True),
        'wizrhim_data_final': fields.date('Data final'),
        'wizrhim_linha_ids': fields.one2many('wiz.rhhenry.importar.marcacao.linhas', 'wizrhiml_capa_id', 'Linhas'),
    }
    _defaults = {
        'wizrhim_relogio_id': lambda s, cr, uid, c: s.pool.get('rhponto.relogios').rhprel_getRelogio(cr, uid,
                                                                                                      context=c)
    }

    def wizrhim_button_importar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        obj_rhponto_marcacao = self.pool.get('rhponto.marcacao')
        brwDados = self.browse(cr, uid, ids[0], context=context)
        linha_ids = self.pool.get('wiz.rhhenry.importar.marcacao.linhas').search(cr, uid, [('wizrhiml_capa_id', '=', ids[0])], context=context)
        if linha_ids:
            self.pool.get('wiz.rhhenry.importar.marcacao.linhas').unlink(cr, uid, linha_ids, context=context)
        ip = brwDados.wizrhim_relogio_id.rhprel_ip
        try:
            porta = int(brwDados.wizrhim_relogio_id.rhprel_porta)
        except:
            raise osv.except_osv(_('Atencao'), _('Porta nao e um valor numerico!'))
        timeout = 1
        with SDK(ip, porta, timeout=timeout) as s:
            s.get_eventos(operacao='D',
                          qtd_eventos=10000,
                          dataInicial=brwDados.wizrhim_data_inicial,
                          dataFinal=brwDados.wizrhim_data_final)
            relogio = GetEvento(s)

        for dado in relogio.data_payload:
            if dado.get('tipo') != '3':  # So interessa registros do tipo 3 Marcacao
                continue
            _dv = Pis().getDigito(dado.get('pis')+'0')
            if not _dv:
                continue
            _pis = Pis().verificarTamanho(dado.get('pis') + str(_dv))
            funcionario_ids = obj_rh_funcionario.search(cr, uid, [('rhfunc_pis', '=', _pis)], context=context)
            if not funcionario_ids:
                continue
            funcionario_id = funcionario_ids[0]
            data = datas.getDataNoFormato(dado.get('data'), 'I', '-')
            hora = '%s:%s' % (dado.get('horario')[0:2], dado.get('horario')[2:4])
            data_hora = '%s %s:00' % (data, hora)
            datatz =  self.pool.get('adm.utils').admutl_converte_tz(cr, uid,
                                                                    datetime.strptime(data_hora, '%Y-%m-%d %H:%M:%S'),
                                                                    tz_destino=context.get('tz'),
                                                                    tz_origem='UTC',
                                                                    context=context)
            data_hora1 = datatz.strftime('%Y-%m-%d %H:%M')
            data = data_hora1 [:10]
            hora = data_hora1[11:]
            horario = self.pool.get('adm.utils').admutl_converte_hora_str_to_float(hora)
            vals_linha = {
                'wizrhiml_capa_id': ids[0],
                'wizrhiml_funcionario_id':funcionario_id,
                'wizrhiml_data': data,
                'wizrhiml_hora': horario,
            }
            self.pool.get('wiz.rhhenry.importar.marcacao.linhas').create(cr, uid, vals_linha, context=context)
            rhponto_marcacao_ids = obj_rhponto_marcacao.search(
                cr, uid, [('rhpmar_funcionario_id', '=', funcionario_id),
                          ('rhpmar_data', '=', data),
                          ('rhpmar_horario', '=', horario)], context=context)
            if rhponto_marcacao_ids:
                continue
            vals_marcacao = {
                'rhpmar_funcionario_id': funcionario_id,
                'rhpmar_data': data,
                'rhpmar_horario': horario,
                'rhpmar_codigo': dado.get('NSR'),
                'rhpmar_tipo': 'A',
            }
            obj_rhponto_marcacao.create(cr, uid, vals_marcacao, context)
        return True

class wiz_rhhenry_importar_marcacao_linhas(osv.Model):
    # prefixo: wizrhiml

    _name = 'wiz.rhhenry.importar.marcacao.linhas'
    _columns = {
        'wizrhiml_capa_id': fields.many2one('wiz.rhhenry.importar.marcacao', 'Capa', required=True, ondelete='cascade'),
        'wizrhiml_funcionario_id': fields.many2one('rh.funcionario','Funcionario', ondelete='cascade'),
        'wizrhiml_data': fields.date('Data'),
        'wizrhiml_hora': fields.float('hora'),
    }
