# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
import time


class supervisor_log_importacao(osv.Model):
    # prefixo: suplog_

    _inherit = 'supervisor.log.importacao'

    def suplog_cria_log(self, cr, uid, nome, qtd_importada, qtd_total, erro='', finalizado=False, context=None):
        """
        O metodo cria logs dos registros dos arquivos que nao foram importados devido a cadastros
        que nao foram previamente realizados
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param qtd_importada: inteiro que representa a quantidade de registros ja importados
        :param erro: string que representa o erro que impediu a importaca do registro
        :param finalizado: booleano, com True para quando a importacao terminou e False para quando
        esta em execucao
        :param context: dicionario com valores contextuais
        :return: booleano True
        """
        context = context and context.copy() or {}
        obj_supervisor_log_importacao_item = self.pool.get('supervisor.log.importacao.item')
        vals = {
            'suplog_importacao': nome
        }
        if qtd_importada:
            vals.update({
                'suplog_qtd_importada': qtd_importada,
                'suplog_qtd_a_importar': qtd_total - qtd_importada})
        log_ids = self.search(cr, uid, [('suplog_importacao', '=', nome)],
                              context=context)
        if log_ids:
            log_id = log_ids[0]
            self.write(cr, uid, log_id, vals, context)
        else:
            log_id = self.create(cr, uid, vals, context)
        if erro:
            vals_item = {
                'suploi_capa_id': log_id,
                'suploi_erro': erro,
            }
            obj_supervisor_log_importacao_item.create(cr, uid, vals_item, context)
        cr.commit()
        if finalizado:
            log_item_ids = obj_supervisor_log_importacao_item.search(
                cr, uid, [('suploi_capa_id', '=', log_id)], context=context)
            if log_item_ids:
                self.write(cr, uid, log_id, {'state': 'E'}, context)
            else:
                self.write(cr, uid, log_id, {'state': 'P'}, context)
        return True

    def suplog_exclui_log(self, cr, uid, dias_para_exclusao=1, context=None):
        """
        Metodo chamado por registro no agendador (cron). Deleta registros da tabela
        supervisor_log_importacao de acordo com o periodo entre a data de hoje menos a quantidade
        de dias recebido no parametro dias_para_exclusao
        :param cr: cursor no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param dias_para_exclusao: inteiro que representa a quantidade de dias
        :param context: dicionario com valores contextuais
        :return: booleano True
        """
        context = context and context.copy() or {}
        data_hora = parse(time.strftime('%Y-%m-%d %H:%M:%S'))
        data_hora_inicial = data_hora - relativedelta(days=dias_para_exclusao)
        data_hora = data_hora.strftime('%Y-%m-%d %H:%M:%S')
        data_hora_inicial = data_hora_inicial.strftime('%Y-%m-%d %H:%M:%S')
        log_ids = self.search(cr, uid, [('create_date', '>=', data_hora_inicial),
                                        ('create_date', '<=', data_hora)], context=context)
        self.unlink(cr, uid, log_ids, context)
        return True
