# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import datetime, timedelta


class supervisor_mov_capa(osv.Model):
    """
    Tabela de Capa de cupons
    Prefixo: supcap

    """
    _name = 'supervisor.mov.capa'
    _inherit = 'supervisor.mov.capa'

    def name_get(self, cr, user, ids, context=None):
        context = context and context.copy() or {}
        res = []
        for id in ids:
            res.append((id, str(self.browse(cr, user, id).supcap_nro_cupom)))
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for i in self.read(cr, uid, ids, ['supcap_state'], context):
            if i.get('supcap_state'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Nota nao pode ser exluida, deve ser cancelada!'))

    def read_group(self, cr, uid, domain, fields, groupby, offset=0, limit=None, context=None,
                   orderby=False):
        for d in domain:
            if '30_dias' in d:
                d[2] = str((datetime.now() - timedelta(days=30)).date())

        res = super(supervisor_mov_capa, self).read_group(cr, uid, domain, fields, groupby, offset,
                                                          limit, context, orderby)
        for i in res:
            if 'supcap_caixa_count' in i:
                i['supcap_caixa'] = 'Caixa: %s' % i.get('supcap_caixa')
            else:
                del i['supcap_caixa']

            if 'supcap_loja_count' in i:
                i['supcap_loja'] = 'Loja: %s' % i.get('supcap_loja')
            else:
                del i['supcap_loja']

            if 'supcap_cliente_count' in i:
                aux = []
                for arg in i.get('__domain'):
                    if not arg[2] and 'supcap_cliente' in arg:
                        arg = ('supcap_cliente', 'in', [False, '', None])

                    aux.append(arg)

                if aux:
                    i['__domain'] = aux

                if i.get('supcap_cliente'):
                    obj_cliente = self.pool.get('supervisor.cad.clientes')
                    cli_ids = obj_cliente.search(cr, uid, [('supcli_cnpj_cpf', '=', i.get('supcap_cliente'))], context=context)
                    if cli_ids:
                        nome = obj_cliente.read(cr, uid, cli_ids[0], ['name'], context).get('name')
                        i['supcap_cliente'] = '%s - %s' % (nome, i.get('supcap_cliente'))
                    else:
                        i['supcap_cliente'] = 'Cliente sem cadastro - %s' % i.get('supcap_cliente')
                else:
                    i['supcap_cliente'] = 'Sem Cliente informado'

            if 'supcap_data_cupom_count' in i:
                i['supcap_data_cupom'] = 'Data: %s' % i.get('supcap_data_cupom')

            if 'supcap_operador_id_count' in i:
                if i.get('supcap_operador_id'):
                    nome = self.pool.get('supervisor.cad.operadores').read(cr, uid,
                                       i.get('supcap_operador_id')[0], ['name'], context).get('name')

                    i['supcap_operador_id'] = (i.get('supcap_operador_id')[0], 'Operador: %s' % nome)
                else:
                    i['supcap_operador_id'] = 'Sem operador'

            if 'supcap_nro_cupom' in i:
                del i['supcap_nro_cupom']

        return res

    def create(self, cr, user, vals, context=None):
        context = context and context.copy() or {}
        id = super(supervisor_mov_capa, self).create(cr, user, vals, context=context)
        if id: # cria o cupom servidor para o cupom
            self.pool.get('supervisor.mov.cupom.servidor').create(cr, user,
                                                  {'supcse_enviado': False,
                                                   'supcse_capa_id': id}, context=context)

        return id

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        if not isinstance(ids, list):
            ids = [ids]

        for cupom in  self.read(cr, user, ids, [], context=context):
            if context.get('supervisor_importacao_automatica') or cupom.get('supcap_state'):
                continue

            if cupom.get('supcap_tipo') != 'Z' and not context.get('importacao_cupom_via_mfd'):
                raise osv.except_osv(_('Atencao!'),
                                     _('E permitido alterar apenas a Reducao Z!'))

        cup_serv_ids = self.pool.get('supervisor.mov.cupom.servidor').search(cr, user,
                                                  [('supcse_capa_id','in', ids)], context=context)
        # atualiza a cupom servidor para os cupons alterados
        self.pool.get('supervisor.mov.cupom.servidor').write(cr, user, cup_serv_ids,
                                                         {'supcse_enviado': False}, context=context)
        return super(supervisor_mov_capa, self).write( cr, user, ids, vals, context=context)

    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
        for arg in args:
            if '30_dias' in arg:
                arg[2] = str((datetime.now() - timedelta(days=30)).date())
        res = super(supervisor_mov_capa, self).search(cr, user, args, offset, limit, order, context, count)
        return res
