# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from decimal import Decimal
import string
import locale
import pytz


class supervisor_util(osv.Model):
    # prefixo: suputi_

    _name = 'supervisor.util'

    # Metodos copiados do modulo adm (classe adm_utils) pois o supervisor deve ser um modulo sem
    # dependencia do sigaddons

    def supervisor_get_funcao(self, cr, uid, model, function, *args):
        try:
            _args = ','.join(map(lambda x: '%r' % x,  args))
            return eval("self.pool.get('{model}').{function}(cr, uid, {args})".format(
                model=model,
                function=function,
                args=_args))
        except Exception as e:
            txt = ''
            if e.__class__ in (osv.orm.except_orm,osv.except_osv) and len(e.args) > 1:
                txt = e.args[0].encode('utf-8') + ' ' + e.args[1].encode('utf-8')
            else:
                try:
                    txt = e.message or e.value
                except:
                    txt = str(e)

            txt = self.suputi_remover_acentuacao(txt)
            raise Exception('9999', txt)

    @staticmethod
    def suputi_remover_acentuacao(text, cod='utf-8'):
        """
        Funcao Wrapper para remover acentuacao
        :param text:texto onde sera feita a remocao da acentuacao
        :param cod:Tipo de codificacao
        return:string
        """
        from unicodedata import normalize, category

        try:
            unicode_string = str(text)
        except UnicodeEncodeError:
            return normalize('NFKD', text).encode('ASCII', 'ignore')

        return unicode(filter(lambda c: category(c) != 'Mn',
                              normalize('NFKD', unicode_string.decode(cod))))

    @staticmethod
    def suputi_remove_caracter(caracteres, letras=True, numeros=True, espaco=True, outros=None):
        """
        @Parametros: caracteres - Eh a palavra ou frase que deseja remover caracters
                                  especiais - Tipo: string
                     letras     - Se True filtra letras da sequencia informada - Tipo boolean
                     numeros    - Se True filtra os digitos da sequencia de caracteres
                                  informados - Tipo: boolean
                     espaco     - Se True filtra os espacos em branco - Tipo: boolean
                     outros     - uma lista de outros caracteres que deseja filtrar - Tipo: lista
        @Funcao: Remover caracters especiais de uma sequencia de caracteres
        @Retorno: Retorna uma string tratada de acordo com as informacoes passadas no
                  parametro - Tipo: string
        """
        if not caracteres:
            return ''
        if not isinstance(outros, list):
            outros = []
        char_validos = ''
        if letras:
            char_validos += string.letters
        if numeros:
            char_validos += string.digits
        if espaco:
            char_validos += ' '
        for i in outros:
            char_validos += i
        new_caracters = ''.join([char if char in char_validos else '' for char in caracteres])
        return new_caracters

    @staticmethod
    def suputi_datetime_to_timezone(cr, uid, data, formato=False):
        """
        Transforma a data do timezone UTC(Timezone padrão armazenado na base do open)
        para o timezone do usuário(formato de apresentação)
        @:param cr: cursor no banco de dados
        @:param uid: {integer} identificador do usuario, id da tabela res_users
        @:param data: {datetime}  data recebida direto de um campo datetime de uma classe do open
        @return: {string} data no formato YYYY-MM-DD HH:MM:SS
        """
        if not formato:
            formato = str(locale.nl_langinfo(locale.D_FMT).replace('%y', '%Y')) + ' ' + str(
                locale.nl_langinfo(locale.T_FMT))
        _dt = fields.datetime.context_timestamp(cr, uid, data).strftime(formato)
        return _dt

    def suputi_convert_2_decimal(self, value, places=2):
        res = value
        places = Decimal(10) ** (places * -1)
        if isinstance(value, (int, long, float)):
            valor = Decimal(str(value))
            res = valor.quantize(exp=places)
        elif isinstance(value, Decimal):
            res = value.quantize(exp=places)
        return res

    def efetuar_rateio(self, valor_total, elementos=[]):

        """
        Funcao generica para rateio.
        Passar lista de lista ex:[[1,10],[2,20],[3,30]], retornara a mesma lista mas com os valores do segundo indice rateados
        """
        float_round = fields.float_round
        if elementos:
            vr_total_elementos = sum(map(lambda x: x[1], elementos)) or 0
            if vr_total_elementos > 0 and valor_total:
                percentual = float(valor_total) / float(
                    vr_total_elementos)  # calcula o percentual sobre o valor informado no segundo elemento lista/tupla
                maior_elemento = [0, 0.0]
                for valor in elementos:
                    if isinstance(valor,
                                  list):  #verifica se os elementos contidos sao tuplas ou listas,
                        valor_rateado = float_round((valor[1] * percentual), 2)  #calcula o valor rateado
                        valor[1] = valor_rateado
                        if valor[1] > maior_elemento[
                            1]:  #pega maior elemento para fazer o tratamento de diferenca de valores
                            maior_elemento = valor
                vr_total_diferenca = sum(map(lambda x: x[1], elementos)) or 0
                if maior_elemento != [0, 0.0]:
                    indice = elementos.index(maior_elemento)
                else:
                    indice = elementos.index(elementos[0])
                if valor_total > vr_total_diferenca:  #verifica e ajusta diferencas
                    diff = float_round((valor_total - vr_total_diferenca), 2)
                    elementos[indice][1] = float_round(elementos[indice][1] + diff, 2)
                if valor_total < vr_total_diferenca:
                    diff = float_round((vr_total_diferenca - valor_total), 2)
                    elementos[indice][1] = float_round(elementos[indice][1] - diff, 2)
            # SE A QUANTIDADE PRESENTE NOS ELEMENTOS FOR ZERO, REALIZA O RATEIO DO VALOR TOTAL PARA CADA ELEMENTO
            elif vr_total_elementos == 0:
                len_elementos = len(elementos) or 1

                dec_valor_total = self.suputi_convert_2_decimal(valor_total, places=2)
                dec_valor_por_elemento = self.suputi_convert_2_decimal(
                    dec_valor_total / len_elementos, places=5)

                total_inserido = 0
                total = 0
                cont = 1

                for i in elementos:
                    total += dec_valor_por_elemento
                    if cont == len_elementos:
                        res = total - (total_inserido + dec_valor_por_elemento)
                        dec_valor_por_elemento += res

                    dec_total_por_elemento = self.suputi_convert_2_decimal(dec_valor_por_elemento,
                                                                           places=2)
                    i[1] = float_round(float(dec_total_por_elemento), 2)
                    total_inserido += dec_total_por_elemento
                    cont += 1

        return elementos

    def suputi_converte_tz(self, cr, uid, data, tz_origem=None, tz_destino=None, context=None):
        """
        Função para converter data de um timezone para outro
        @param data: data no formato datetime para ser convertida
        @param tz_origem: Timezone em que a data se encontra. Se None, vai buscar timezone do usuário
        @param tz_destino: Timezne em que a data será convertida. Se None, vai buscar timezone do usuário
        @return: data no formato datetime convertida para o novo timezone
        """

        context = context and context.copy() or {}
        def _buscar_tz():
            if context and context.get('tz'):
                tz_name = context['tz']
            else:
                tz_name = self.pool.get('res.users').read(cr, uid, uid, ['tz'], context=context)['tz']
            obj = pytz.timezone(tz_name)
            return obj

        context = context and context.copy() or {}
        if tz_origem:
            obj_origem = pytz.timezone(tz_origem)
        else:
            obj_origem = _buscar_tz()
        if tz_destino:
            obj_destino = pytz.timezone(tz_destino)
        else:
            obj_destino = _buscar_tz()
        x = obj_origem.localize(data, is_dst=False)
        return x.astimezone(obj_destino)

    def get_loja(self, cr, uid, context=None):
        """Retorna a loja do cadastro de configuracoes, se nao tiver loja retorna 0(todas as lojas) 
        e se houver mais de uma configuracao pega a loja da primeira"""
        context = context and context.copy() or {}
        obj_supcfg_configuracao = self.pool.get('supervisor.cfg.configuracao')
        configuracao_ids = obj_supcfg_configuracao.search(cr, uid, [], context=context)
        if not configuracao_ids:
            return 0

        return obj_supcfg_configuracao.read(cr, uid, configuracao_ids[0], ['supcfc_loja'],
                                            context=context).get('supcfc_loja') or 0

    def suputi_fetchall_to_dictfetchall(self, _cr):

        """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
         para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
         conversao."""

        # testa resultados armazenados
        results = _cr.fetchall()
        if results < 1:
            return results

        # substitui resultados das listas por dicionarios
        rownum = 0
        for row in results:
            dictrow = {}
            dictnum = 0
            for col in _cr.description:
                dictrow[col[0]] = row[dictnum]
                dictnum += 1

            results[rownum] = dictrow
            rownum += 1

        return results
