# -*- coding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
# #############################################################################

from openerp.osv import osv
from SigUtils import sigerp
from openerp.tools.translate import _
import logging
from openerp import SUPERUSER_ID
from unicodedata import normalize
from .....web_sig.control.utils import websig_get_my_ip
import xmlrpclib, socket
from re import sub


class valida_pendencias(osv.Model):
    _inherit = 'valida.pendencias'
    _logger = logging.getLogger(_inherit)

    def valpen_validar_erp(self, cr, uid, ids, context=None):
        """
        Método executado pelo Cron para comunicar com a Sig e pegar uma nova licenca
        @param cr:
        @param uid:
        @param ids:
        @param context:
        @return: True ou False dependendo se conseguiu ou não
        """
        # Se o modulo ADM estiver instalado, nao faz nada.
        context = context and context.copy() or {}
        super(valida_pendencias, self).valpen_validar_erp(cr, uid, ids, context)
        if self.valpen_verifica_rede_sig(cr, uid, context):
            self._logger.info(_('Nao validou, esta na rede interna da Sig!'))
            return True

        args = [('name', '=', 'adm'), ('state', 'in', ('installed', 'to upgrade'))]
        if self.pool.get('ir.module.module').search(cr, uid, args, context=context):
            return True

        self._logger.info(_('Validando o SigERP'))
        obj_config = self.pool.get('supervisor.cfg.configuracao')
        config_ids = obj_config.search(cr, uid, [], context=context)
        dados_conf = obj_config.read(cr, uid, config_ids[0], ['supcfc_cnpj'], context)
        cnpj_cadastro = sub('[^0-9]', '', dados_conf.get('supcfc_cnpj') or '')
        if not cnpj_cadastro:
            self._logger.info(_('Empresa sem cnpj cadastrado!'))
            return False

        servidor = self.pool.get('ir.config_parameter').get_sig_server(cr, uid).split(',')
        self._logger.info(_('Validando para o CNPJ: %s Servidor: %s') % (cnpj_cadastro, servidor))
        try:
            host = servidor[0]
            db = servidor[1]
            porta = servidor[2]
            user = servidor[3]
            senha = servidor[4]
            sock_common = xmlrpclib.ServerProxy('http://' + host + ':' + str(porta) + '/xmlrpc/common')
            socket.setdefaulttimeout(3)
            user_id = sock_common.login(db, user, senha)
            sock = xmlrpclib.ServerProxy('http://' + host + ':' + str(porta) + '/xmlrpc/object')
            ip_cliente = websig_get_my_ip()
            self._logger.info(_('Vou pedir geracao de licensa cnpj: %s IP: %s') % (cnpj_cadastro, ip_cliente))
            licenca = sock.execute(db, user_id, senha, 'financeiro.cliente', 'fincli_gerar_licensa',
                                   cnpj_cadastro, ip_cliente, context)
            if not licenca:
                int('A')  # forçar except
            self.pool.get('ir.config_parameter').set_param(cr, uid, 'licensa_sig_erp', licenca)
            self._logger.info(
                _('SigERP validado! Licenca:%s') % str(sigerp._crypto_sig('d', licenca)))
        except Exception as e:
            if isinstance(e, object) and not isinstance(e, ValueError) and 'faultCode' in e.__dict__:
                e = e.faultCode or ''
            else:
                e = str(e)
            try:
                string = normalize('NFKD', e.decode('utf-8')).encode('ASCII', 'ignore')
            except:
                string = normalize('NFKD', e).encode('ASCII', 'ignore')

            self._logger.info(('SigERP nao validado! Erro:%s') % string)
            # Não conseguindo a conecção, remover o servidor da configuração para que possar ser gerado novamente,
            # talvez ele esteja errado
            server_id = self.pool.get('ir.config_parameter').search(cr, SUPERUSER_ID,
                                                                    [('key', '=', 'server_sig')], context=context)
            self.pool.get('ir.config_parameter').unlink(cr, SUPERUSER_ID, server_id)

        return True
