# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class supervisor_cad_finalizadora(osv.Model):
    """
    Tabela de cadastro de finalizadoras
    Prefixo: supfin
    """
    _name = 'supervisor.cad.finalizadora'
    _gerar_logs_logs = 'N'
    _columns = {
        'supfin_codigo': fields.integer('Codigo', required=True, select=True),
        'name': fields.char('Descricao', size=100, required=True),
        'supfin_permite_troco': fields.boolean('Permite troco'),
        'supfin_troco_maximo': fields.float('Troco maximo', digits=(10, 2)),

        'supfin_tipo_finalizadora': fields.selection([('D', 'Dinheiro'),
                                                      ('H', 'Cheque'),
                                                      ('C', 'Cartao'),
                                                      ('T', 'Ticket'),
                                                      ('O', 'Outros')
                                                      ], 'Tipo de finalizadora',
                                                     oldname="supfin_tipo_tinalizadora",
                                                     required=True),

        'supfin_codigo_na_ecf': fields.many2one("supervisor.cad.finalizadora.na.ecf",
                                                'Codigo na ecf', ondelete="restrict",
                                                required=True, select=True),

        'supfin_avista_aprazo': fields.selection([('V', 'a Vista'), ('P', 'a Prazo')],
                                                 'a Vista/a Prazo', required=True),

        'supfin_usa_tef': fields.boolean('Usa tef'),
        'supfin_cartao_tipo': fields.selection([('N', 'Nao e cartao'), ('C', 'Credito'),
                                                ('D', 'Debito')], 'Tipo de cartao'),

        'supfin_cartao_parcelamento': fields.selection([('V', 'a Vista'),
                                                        ('A', 'Parcelado Adm'),
                                                        ('E', 'Parcelado p/Estabelecimento'),
                                                        ('P', 'Parcelado'),
                                                        ('R', 'Pre-datado')
                                                        ], 'Parcelamento'),

        'supfin_tipo_midia': fields.selection([('D', 'Digitado'), ('M', 'Magnetico')],
                                              'Tipo de midia'),

        'supfin_outros_qtd_parcelas': fields.integer('Outros qtd parcelas'),
        'supfin_outros_taxa_juros': fields.float('Outros taxa juros', digits=(5, 2)),
        'supfin_outros_prazo_primeira': fields.integer('Outros prazo primeira'),
        'supfin_outros_intervalo_parcelas': fields.integer('Outros intervalo parcelas'),
        'supfin_outros_periodo': fields.char('Outros periodo', size=1),
        'supfin_outros_semanal_dia': fields.char('Outros semanal dia', size=3),
        'supfin_outros_quinzenal_vencimento': fields.integer('Outros quinzenal vencimento'),
        'supfin_outros_quinzenal_tipo': fields.char('Outros quinzenal tipo', size=1),
        'supfin_outros_mensal_vencimento': fields.integer('Outros mensal vencimento'),
        'supfin_outros_mensal_tipo': fields.char('Outros mensal tipo', size=1),
        'supfin_outros_mensal_fecha_mes': fields.integer('Outros mensal fecha mes'),
        'supfin_conta_caixa_credito': fields.char('Conta caixa credito', size=15),
        'supfin_conta_caixa_debito': fields.char('Conta caixa debito', size=15),
        'supfin_conta_a_prazo': fields.char('Conta a prazo', size=15),
        'supfin_conta_taxa_administracao': fields.char('Conta taxa administracao', size=15),
        'supfin_cliente': fields.integer('Cliente'),
        'supfin_permite_desconto_a_vista': fields.boolean('Permite desconto a vista'),
        'supfin_aceita_contra_vale': fields.boolean('Aceita contra vale'),
        'supfin_contra_vale': fields.boolean('Contra vale'),
        'supfin_imp_promissoria': fields.boolean('Imprimir promissoria'),
        'supfin_conta_descontos': fields.char('Conta descontos', size=15),
        'supfin_conta_acrescimos': fields.char('Conta acrescimos', size=15),
        'supfin_conta_troco': fields.char('Conta troco', size=15),

        # Indica que a finalizadora e para uso em convênios
        'supfin_convenio': fields.boolean('Convenio'),

        # Indica que a finalizadora e apenas para DAV
        'supfin_somente_dav': fields.boolean('Somente dav'),

        'supfin_mod_pgto_grupo': fields.selection([('00', 'Cheque'),
                                                   ('01', 'Cartao de Debito'),
                                                   ('02', 'Cartao de Credito'),
                                                   ('03', 'Cartao tipo voucher'),
                                                   ('98', 'Dinheiro'),
                                                   ('99', 'Outro tipo de cartao')
                                                   ], 'Mod pgto grupo'),

        'supfin_mod_pgto_sub_grupo': fields.selection([('00', 'A vista'),
                                                       ('01', 'pre-datado'),
                                                       ('02', 'Parc.Fin.Estab.'),
                                                       ('03', 'Parc.Fin.Admin')
                                                       ], 'Mod pgto sub grupo'),

        'supfin_bandeira_cre': fields.many2one("supervisor.cad.bandeiras", "Mod bandeira credito",
                                               domain=[("supban_tipo", "in", ["C","V","O"])]),

        'supfin_bandeira_deb': fields.many2one("supervisor.cad.bandeiras", "Mod bandeira debito",
                                               domain=[("supban_tipo", "in", ["D", "V"])]),

        'supfin_cad_kit_ids': fields.one2many('supervisor.cad.kits', 'supkit_codigo_kit', 'Kits'),
        'supfin_itens_kit_ids': fields.one2many('supervisor.cad.kits', 'supkit_codigo_item',
                                                'Itens'),
        'supfin_operadora_id': fields.many2one("supervisor.cad.operadora.cartao",
                                               "Operadora de Cartao", select=True),
    }
    _sql_constraints = [
        ('supfin_codigo_uniq', 'unique (supfin_codigo)', 'O codigo precisa ser unico!'),
    ]
