# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _

MERCADORIA_NACIONAL = [
    ('0', '0 - Nacional'),
    ('3', '3 - Nacional, mercadoria ou bem com Conteudo de Importacao superior a 40% e inferior ou igual a 70%'),
    ('4', '4 - Nacional, conforme processo produtivo basico dec 288/67'),
    ('5', '5 - Nacional, mercadoria ou bem com Conteudo de Importacao inferior ou igual a 40% (quarenta por cento)'),
    ('8', '8 - Nacional, mercadoria ou bem com conteudo de importacao superior a 70%')
]
MERCADORIA_ESTRANGEIRA = [
    ('1', '1 - Estrangeira - Importacao Direta'),
    ('2', '2 - Estrangeira - Adquirida no mercado interno'),
    ('6', '6 - Estrangeira - Importacao direta, sem similar nacional, constante em lista de Resolucao CAMEX'),
    ('7', '7 - Estrangeira - Adquirida no mercado interno, sem similar nacional, constante em lista de Resolucao CAMEX.'),
]

ORIGEM_MERCADORIAS = MERCADORIA_NACIONAL + MERCADORIA_ESTRANGEIRA
ORIGEM_MERCADORIAS.sort(key=lambda i: i[0])


class supervisor_cad_itens(osv.Model):
    """
    Tabela de Cadastro de itens
    Prefixo: supite

    """
    _name = 'supervisor.cad.itens'
    _gerar_logs_logs = 'N'
    _columns = {
        'supite_loja': fields.integer('Loja', required=True),
        'supite_codigo_item': fields.char('Codigo item', size=14, required=True, select=True),
        'supite_descricao_completa':fields.char('Descricao Completa', size=60),
        'name': fields.char('Descricao', size=30, select=True),
        'supite_pesavel': fields.boolean('Pesavel'),
        'supite_fracionado': fields.boolean('Fracionado'),
        'write_date': fields.datetime('Data de atualizacao', select=True, readonly=True),
        'supite_preco': fields.float('Preco', digits=(10, 3)),
        'supite_unidade': fields.char('Unidade', size=3),
        'supite_peso_da_embalagem': fields.float('Peso da embalagem', digits=(4, 3)),
        'supite_peso_tara': fields.float('Peso da tara', digits=(1, 3)),
        'supite_codigo_retaguarda': fields.char('Codigo retaguarda', size=20, select=True),
        'supite_fator_multiplicador': fields.float('Fator multiplicador', digits=(11, 3)),
        'supite_producao': fields.selection([('P', 'Propria'), ('T', 'Terceiro')], 'Producao', required=True),
        'supite_iat': fields.selection([('A', 'Arredondar'), ('T', 'Tuncar')], 'IAT', required=True),
        'supite_ncm': fields.char('NCM', size=8),
        'supite_cest': fields.char('CEST', size=7),
        'active': fields.boolean('Ativo'),
        'supite_tributacao_id': fields.many2one("supervisor.cad.tributacao", 'Tributacao', ondelete="no action",
                                                required=True, select=True),
        'supite_tributacao_pis_id': fields.integer('Tributacao NFC-e PIS', select=True), # nao guarda o id, guarda o campo suptnf_codigo
        'supite_tributacao_cofins_id': fields.integer('Tributacao NFC-e COFINS', select=True), # nao guarda o id, guarda o campo suptnf_codigo
        'supite_tributacao_icms_id': fields.integer('Tributacao NFC-e ICMS/ISS', select=True), # nao guarda o id, guarda o campo suptnf_codigo
        'supite_aliquota_ibpt_fed': fields.float('Aliquota de Tributos Federais', digits=(3, 2),
                                                help='Aliquota aproximada de tributos Federais em cumprimento a lei 12.741/2012. Produtos adquiridos no mercado nacional'),
        'supite_aliquota_ibpt_est': fields.float('Aliquota de Tributos Estaduais', digits=(3, 2),
                                                help='Aliquota aproximada de tributos Estaduais em cumprimento a lei 12.741/2012.'),
        'supite_aliquota_ibpt_mun': fields.float('Aliquota de Tributos Municipais', digits=(3, 2),
                                                help='Aliquota aproximada de tributos Municipais em cumprimento a lei 12.741/2012.'),
        'supite_ibpt_chave': fields.char('Chave da tabela', size=10),
        'supite_codigo_anp': fields.integer('Combustivel/Lubrificante', size=9),
        'supite_adrem': fields.float('Aliquota adRemICMSRet', digits=(7, 4)),
        'supite_pglp':  fields.float('Percentual do GLP', digits=(7, 4),
                                     help='Percentual do GLP derivado do petróleo no '
                                          'produto GLP (cProdANP=210203001)(Valores de 0 a 100)'),
        'supite_pgnn': fields.float('Percentual de Gás Natural Nacional', digits=(7, 4),
                                     help='Percentual de Gás Natural Nacional - GLGNn para o '
                                          'produto GLP (cProdANP=210203001)(Valores de 0 a 100)'),
        'supite_pgni': fields.float('Percentual de Gás Natural Importado', digits=(7, 4),
                                     help='Percentual de Gás Natural Importado - GLGNi para o '
                                          'produto GLP (cProdANP=210203001)(Valores de 0 a 100)'),
        'supite_vpart': fields.float('Valor de partida (cProdANP=210203001)', digits=(13, 2),
                                     help='Deve ser informado neste campo o valor por quilograma'
                                          ' sem ICMS'),
        'supite_origem_mercadoria': fields.selection(ORIGEM_MERCADORIAS, "Origem da mercadoria", required=True, size=1),
        'supite_getin': fields.boolean("Tem GETIN"),
        'supite_med_codigo_anvisa': fields.char('Codigo anvisa', size=13, help="Codigo do produto na Anvisa."),
        'supite_med_motivo_isencao': fields.text('Motivo de isencao'),  # Codigo de excessao do NCM conforme TIPI!
        'supite_med_preco_maximo': fields.float('Preco maximo para consumidor', digits=(9, 2)),
    }
    _defaults = {
        'active': lambda *a: True,
        'supite_origem_mercadoria': '0',
        'supite_getin': True,
    }

    _sql_constraints = [
        ('item_loja_uniq', 'unique (supite_codigo_item, supite_loja)',
         _('O codigo do item precisa ser unico para a loja')),
    ]

