# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields

TIPO = [('A', 'Sangria'),
        ('b', 'Corr.Bancario'),
        ('B', 'Recibo Corr.Bancario'),
        ('c', 'Cancelamento Corr.Banc.'),
        ('C', 'Cancelamento'),
        ('D', 'DAV'),
        ('F', 'Fechamento de Turno'),
        ('G', 'Relatorio Gerencial'),
        ('M', 'Memoria Fiscal'),
        ('N', 'Nulo'),
        ('O', 'ContraVale'),
        ('P', 'Promissoria'),
        ('R', 'Recarga de Celular'),
        ('T', 'Tef'),
        ('U', 'Suprimento'),
        ('V', 'Venda'),
        ('I', 'Inutilizada'),
        ('X', 'Reducao X'),
        ('Z', 'Reducao Z'),
        ('W', 'Recebimentos Diversos'),
        ('w', 'Recibo dos Recebimentos')]
TIPO_COMPROVANTE = [('CM', 'Conferencia de mesa'),
                    ('RV', 'Registro de venda'),
                    ('CC', 'Comprovante de credito ou debito'),
                    ('CN', 'Comprovante nao fiscal'),
                    ('NC', 'Comprovante nao fiscal de cancelamento'),
                    ('RG', 'Relatorio Gerencial'),
                    ('NF', 'Nota fiscal')]
STATUS_NOTA_FISCAL = [
    ('1', 'Esboco'),
    ('2', 'Validada'),
    ('3', 'Faturada'),
    ('4', 'Contingencia'),
    ('5', 'Pendencia'),
    ('6', 'DAV'),
    ('7', 'DAV Cancelado'),
    ('C', 'Cancelada'),
    ('AI', 'Aguardando Inutilizacao'),
    ('2E', 'Validada com erro'),
    ('I', 'Inutilizada'),
    ('P', 'Em processamento'),
    ('D', 'Denegada')
]

class supervisor_mov_capa(osv.Model):
    """
    Tabela de Capa de cupons
    Prefixo: supcap

    """

    def supcap_get_dados_no_stored(self, cr, uid, ids, name, arg, context=None):
        context = context and context.copy() or {}
        res = {}
        for dados in self.read(cr, uid, ids, ['supcap_status', 'supcap_state'], context):
            faturada_sefaz = False
            # 501 = Prazo de cancelamento expirado, 919 = Nota referenciada difere da que deseja cancelar
            if dados.get('supcap_status') and dados.get('supcap_status').split()[0] in ('501', '919')\
                    and dados.get('supcap_state') == '5':
                faturada_sefaz = True

            res[dados.get('id')] = {'supcap_faturada_sefaz': faturada_sefaz}

        return res

    _name = 'supervisor.mov.capa'
    _inherit = ['mail.thread']
    _gerar_logs_logs = 'N'
    _rec_name = 'supcap_nro_cupom'
    _columns = {
        'id': fields.integer('ID'),
        'supcap_data_cupom': fields.date('data cupom', required=True, select=True),
        'supcap_loja': fields.integer('loja', required=True),
        'supcap_caixa': fields.integer('caixa', required=True),
        'supcap_nro_cupom': fields.integer('nro cupom', required=True, select=True),
        'supcap_operador_id': fields.many2one("supervisor.cad.operadores", 'operador', ondelete="restrict",
                                              required=True, select=True),
        'supcap_cliente': fields.char('cliente', size=15), # cpf/cnpj
        'supcap_data_inicio': fields.datetime('data inicio', select=True),
        'supcap_data_fim': fields.datetime('data fim', select=True),
        'supcap_tipo': fields.selection(TIPO, 'tipo'),  # A-Sangria, b-Corr.Bancario, B-Recibo Corr.Bancario,
                                                      # c-Cancelamento Corr.Banc., C-Cancelamento,
                                                      # F-Fechamento de Turno, G-Relatorio Gerencial, M-Memoria Fiscal,
                                                      # N-Nulo, O-ContraVale, P-Promissoria, R-Registro de Mesa, T-Tef,
                                                      # U-Suprimento, V-Venda, X-Reducao X, Z-Reducao Z,
                                                      # W-Recebimentos Diversos, w-Recibo dos Recebimentos,
        'supcap_alteracao': fields.selection([('A', 'Acrescimo'), ('D', 'Desconto')], 'alteracao'),  # A=Acrescimo,
                                                                                                     # D=Desconto
        'supcap_valor_desc_acres': fields.float('valor desc acres', digits=(10, 2)), # desc. ou acres. dado na capa
        'supcap_total_cupom': fields.float('total cupom', digits=(10, 2)),
        'supcap_combustivel': fields.char('combustivel', size=1),
        'supcap_nro_serie': fields.char('nro serie', size=20),
        'supcap_mf_adicional': fields.boolean('mf adicional'),
        'supcap_marca_ecf': fields.char('marca ecf', size=20),
        'supcap_modelo_ecf': fields.char('modelo ecf', size=20),
        'supcap_nro_proprietario': fields.integer('nro proprietario'),
        'supcap_ccf': fields.integer('ccf'),
        'supcap_gnf': fields.integer('gnf'),
        'supcap_crz': fields.integer('crz'),
        'supcap_grg': fields.integer('grg'),
        'supcap_cdc': fields.integer('cdc'),
        'supcap_tipo_comprovante': fields.selection(TIPO_COMPROVANTE, 'tipo comprovante'),
        'supcap_subtotal': fields.float('subtotal', digits=(10, 2)),
        'supcap_vr_imposto_fed': fields.float('vr imposto federal', digits=(10, 2)),
        'supcap_vr_imposto_est': fields.float('vr imposto estadual', digits=(10, 2)),
        'supcap_vr_imposto_mun': fields.float('vr imposto municipal', digits=(10, 2)),
        'supcap_desconto_subtotal': fields.float('desconto subtotal', digits=(10, 2)),
        'supcap_indicador_desconto': fields.selection([(' ', 'Sem desconto'),
                                                       ('V', 'Desconto em valor'),
                                                       ('P', 'Desconto em percentual')], 'indicador desconto'),
        'supcap_acrescimo_subtotal': fields.float('acrescimo subtotal', digits=(10, 2)),
        'supcap_indicador_acrescimo': fields.selection([(' ', 'Sem acrescimo'),
                                                        ('V', 'Acrescimo em valor'),
                                                        ('P', 'Acrescimo em percentual')], 'indicador acrescimo'),
        'supcap_indicador_cancelamento': fields.selection([('N', 'Nao'), ('S', 'Sim')], 'indicador cancelamento'),
        'supcap_cancelamento_acrescimo': fields.float('cancelamento acrescimo', digits=(10, 2)),
        'supcap_cancelado_posteriormente': fields.boolean('cancelado posteriormente'),
        'supcap_nome_cliente': fields.char('nome cliente', size=40),
        'supcap_nro_cupom_vinculado': fields.integer('nro cupom vinculado'),
        'supcap_nro_ultima_nfce': fields.integer('Ultima NFC-e'),
        'supcap_cod_vendedor': fields.integer('cod vendedor'),
        'supcap_autorizacoes_ids': fields.one2many('supervisor.autorizacoes', 'supaut_capa_id', 'Autorizacoes'),
        'supcap_mov_corpo_xz_ids': fields.one2many('supervisor.mov.corpo.xz', 'supcxz_capa_id', 'Xz'),
        'supcap_mov_corpos_ids': fields.one2many('supervisor.mov.corpos', 'supcor_capa_id', 'Corpos'),
        'supcap_mov_cupom_servidor_ids': fields.one2many('supervisor.mov.cupom.servidor', 'supcse_capa_id', 'Servidor'),
        'supcap_mov_fechamentos_ids': fields.one2many('supervisor.mov.fechamentos', 'supfec_capa_id', 'Fechamentos'),
        'supcap_mov_mesas_ids': fields.one2many('supervisor.mov.mesas', 'supmme_capa_id', 'Mesas'),
        'supcap_mov_recebimentos_ids': fields.one2many('supervisor.mov.recebimentos', 'suprec_capa_id', 'Recebimentos'),
        'supcap_mov_totalizadores_xz_ids': fields.one2many('supervisor.mov.totalizadores.xz', 'suptxz_capa_id', 'Xz'),
        'supcap_numero_pedido': fields.char('Numero do pedido', size=20),  # Numero do pedido.
        'supcap_cod_tot_nao_fiscal': fields.integer('Totalizador nao fiscal'),
        'supcap_lote': fields.char('Nfc-e lote', size=15),
        'supcap_status': fields.char('Status NFC-e', size=400),
        'supcap_xml_proc': fields.text('Nfc-e Assinada e autorizada'),
        'supcap_protocolo': fields.char('Nfc-e Protocolo', size=15),
        'supcap_protocolo_cancelamento': fields.char('Nfc-e Protocolo Cancelamento', size=15),
        'supcap_xml_gerado': fields.text('Xml Gerado'),
        'supcap_xml_cancelamento': fields.text('Xml de Cancelamento'),
        'supcap_chave_acesso': fields.char('Nfc-e chave acesso', size=44),
        'supcap_chave_cte': fields.char('Chave CT-e', size=44),
        'supcap_state': fields.selection(STATUS_NOTA_FISCAL, 'Status', size=2, track_visibility='onchange',
                                         select=True),
        'supcap_ambiente': fields.selection([('H', 'Homologacao'),
                                             ('P', 'Producao'),], 'Ambiente', size=1),

        'supcap_faturada_sefaz': fields.function(supcap_get_dados_no_stored, method=True,
                                                 string='Faturada Sefaz', type='boolean',
                                                 multi='dados_no_store',
                                                 store=False),
    }
    _sql_constraints = [
        ('supcap_cupom_uniq', 'unique (supcap_data_cupom, supcap_loja, supcap_caixa, supcap_nro_cupom, supcap_tipo)',
         'O cupom  precisa ser unico!'),
    ]
    _defaults = {
        'supcap_tipo': 'Z', # cria somente reducao Z manualmente.
    }
    _order = 'supcap_data_inicio DESC, supcap_nro_cupom DESC'
