# -*- encoding: utf-8 -*-
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from openerp.report.render import render
from openerp.osv import osv
import StringIO
import time
from openerp import pooler
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        def center_title(E, C, D, T=160):
            title_c = (T - len(C)) / 2
            title = E
            for i in range(title_c - len(E)):
                title += ' '
            title += C
            for i in range(title_c - len(D)):
                title += ' '
            title += D
            return title

        context = context and context.copy() or {}
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.informaNome(pdf_string, _('Listagem de Guias'))
        listagem.informaTamLinha(148)
        listagem.defineFormato(orientacao='0')
        listagem.defineMargem(0.5, 0.5, 1, 1)
        listagem.defineFonte(tam=6.1)
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')
        loja = datas.get('loja')
        dados_corpos = datas.get('dados_corpos')
        cupons = {}
        for corpo in dados_corpos:
            total_item = corpo.get('supcor_preco_total')
            desconto_item = corpo.get('supcor_desconto_item') or 0
            vals = {
                    'total_bruto_cupom': total_item,
                    'total_desconto': desconto_item,
                    'total_liq_cupom': total_item-desconto_item,
                    'cliente': corpo.get('cliente'),
                    'caixa': corpo.get('caixa'),
                    'tipo': corpo.get('tipo'),
                    'nome_autorizador': corpo.get('nome_autorizador'),
                    'limite_credito': corpo.get('supaut_liberou_limite_credito') or '',
                    'cancelou_cupom': corpo.get('cancelou_cupom') or '',
                    'liberou_desconto': corpo.get('liberou_desconto') or '',
                    'restricao_mensagem': corpo.get('supaut_liberou_restricao_mensagem') or '',
                    'liberou_cheque': corpo.get('supaut_liberou_cheque_a_vista') or ''}
            if not cupons.get(corpo.get('coo')):
                cupons[corpo.get('coo')] = [vals]
            else:
                cupons[corpo.get('coo')].append(vals)

        obj_supervisor_cfg_configuracao = pooler.get_pool(cr.dbname).get(
            'supervisor.cfg.configuracao')
        configuracao_ids = obj_supervisor_cfg_configuracao.search(cr, uid, [], context=context)
        nome_empresa = ''
        if configuracao_ids:
            nome_empresa = obj_supervisor_cfg_configuracao.browse(
                cr, uid, configuracao_ids[0], context=context).supcfc_cliente or ''
        data_hoje = time.strftime("%d/%m/%Y")
        title = center_title(_('Sistema de PDVs'), '%s', '') % nome_empresa
        docs = center_title(_('Data: %s') % data_hoje, _('Acompanhamento de Restricoes e Descontos'),
                            _('Pagina: {{ Page }} '))
        data_ini = time.strftime('%d/%m/%Y', time.strptime(data_ini, '%Y-%m-%d'))
        data_fim = time.strftime('%d/%m/%Y', time.strptime(data_fim, '%Y-%m-%d'))
        docs2 = center_title(_('Data Movimento: %s a %s') % (data_ini, data_fim),
                             _('Loja: %s') %
                             (loja or 'Todas'), '')

        listagem.informaCab('' + title)
        listagem.informaCab('')
        listagem.informaCab('' + docs)
        listagem.informaCab('  ')
        listagem.informaCab('' + docs2)
        listagem.informaCab('  ')
        linha_divisoria = '*%s*' % ''.ljust(153, '-')
        listagem.informaCab(linha_divisoria)
        listagem.informaCab(
          _('| CAIXA |   CUPOM   | TIPO |    CLIENTE    |           QUEM AUTORIZOU           | LC | RM | CA |  VALOR BRUTO  |  DESCONTO  | % DESCONTO |     VALOR      |'))
        listagem.informaCab(linha_divisoria)
        listagem.rodapePadrao()

        total_valor_bruto = total_desconto = total_valor_liquido = 0
        totais_listados = {}
        for coo in sorted(cupons.keys()):
            # retira os cupons cancelados e os de venda relacionados
            if (cupons.get(coo+1) and cupons.get(coo+1)[0].get('tipo') == 'C' \
                or cupons.get(coo)[0].get('tipo') == 'C') and not cupons.get(coo)[0].get('cancelou_cupom'):
                continue

            for i in cupons.get(coo):
                quem_autorizou = i.get('nome_autorizador')
                liberou_credito = 'S' if i.get('limite_credito') else ''
                restricao_mensagem = 'S' if i.get('restricao_mensagem') else ''
                restricao_cancelamento = 'S' if i.get('cancelou_cupom') else ''
                liberou_deconto = i.get('liberou_desconto')
                liberou_cheque = i.get('liberou_cheque')
                valor_desc = i.get('total_desconto')
                if not(liberou_deconto or liberou_credito or restricao_mensagem or liberou_cheque or restricao_cancelamento) \
                    or (liberou_deconto and valor_desc <= 0 and
                            not(liberou_credito or restricao_mensagem or liberou_cheque or restricao_cancelamento)):
                    continue

                valor_bruto = 0
                valor_liquido = 0
                percent_desconto = 0
                if coo not in totais_listados:
                    valor_bruto = i.get('total_bruto_cupom') or 0.0
                    valor_liquido = i.get('total_liq_cupom') or 0.0
                    if restricao_cancelamento:
                        valor_bruto = valor_liquido = 0

                    totais_listados[coo] = valor_bruto
                    if valor_bruto > 0:
                        percent_desconto = valor_desc / valor_bruto * 100
                else:
                    if totais_listados.get(coo) > 0:
                        percent_desconto = valor_desc / totais_listados.get(coo) * 100



                linha = [formataString(i.get('caixa') or '', 7, ' ', 'E', 'N', 'S'),
                         formataString(coo or '', 11, ' ', 'E', 'N', 'S'),
                         formataString(i.get('tipo') or '', 6, ' ', 'C', 'N', 'S'),
                         formataString(i.get('cliente') or '', 15, ' ', 'E', 'N', 'S'),
                         formataString(quem_autorizou, 36, ' ', 'D', 'N', 'S'),
                         formataString(liberou_credito, 4, ' ', 'C', 'N', 'S'),
                         formataString(restricao_mensagem, 4, ' ', 'C', 'N', 'S'),
                         formataString(restricao_cancelamento, 4, ' ', 'C', 'N', 'S'),
                         formataString(getNumeroEditado(valor_bruto, 2), 15, ' ', 'E', 'N', 'S'),
                         formataString(getNumeroEditado(valor_desc, 2), 12, ' ', 'E', 'N', 'S'),
                         formataString(getNumeroEditado(percent_desconto, 2), 12, ' ', 'E', 'N', 'S'),
                         formataString(getNumeroEditado(valor_liquido, 2), 16, ' ', 'E', 'N', 'S')]
                listagem.imprimeLinha('|%s|' % '|'.join(linha))
                total_valor_bruto += valor_bruto
                total_desconto += i.get('total_desconto') or 0
                total_valor_liquido += valor_liquido

        total_percent_desconto = 0
        if total_valor_bruto > 0:
            total_percent_desconto = total_desconto / total_valor_bruto * 100

        listagem.imprimeLinha(linha_divisoria)
        linha = [formataString(_('TOTAL'), 79, ' ', 'D', 'N', 'S'),
                 formataString(getNumeroEditado(total_valor_bruto, 2), 30, ' ', 'E', 'N', 'S'),
                 formataString(getNumeroEditado(total_desconto, 2), 12, ' ', 'E', 'N', 'S'),
                 formataString(getNumeroEditado(total_percent_desconto, 2), 12, ' ', 'E', 'N', 'S'),
                 formataString(getNumeroEditado(total_valor_liquido, 2), 16, ' ', 'E', 'N', 'S')]
        listagem.imprimeLinha('|%s|' % '|'.join(linha))
        listagem.imprimeLinha(linha_divisoria)
        listagem.imprimeLinha(
          _(' Legenda-> LC=LIMITE DE CREDITO, RM=RESTRICAO POR MENSAGEM, CA=CANCELAMENTO DE CUPOM                                  ** Sig Informatica LTDA **       '))

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return (self.obj.pdf, 'pdf')


report_custom('report.listagem_acompanhamento_desconto')
