# -*- encoding: utf-8 -*-
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
import locale
import time
# noinspection PyProtectedMember
from openerp.tools.translate import _


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        data_titulo = time.strftime('%d/%m/%Y')
        hora_titulo = time.strftime('%H:%M:%S')
        title1 = '{:<80}{:>82}'.decode('utf-8').format(
            _('Sistema de PDVs'),_('Pagina: {{ Page }}'))
        title2 = '{:<10}{:^139}{:>5}'.decode('utf-8').format(
            data_titulo, _('Detalhamento por Finalizadora'), hora_titulo[:5])
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Detalhamento por Finalizadora'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        data_inicial = datas.get('data_inicial')
        data_final = datas.get('data_final')
        loja = datas.get('loja')
        caixa = datas.get('caixa')
        turno_id = datas.get('turno_id')
        nome_turno = datas.get('nome_turno')
        finalizadora_id = datas.get('finalizadora_id')
        nome_finalizadora = datas.get('nome_finalizadora') or ''

        if not loja:
            nome_loja = _('Todas')
        else:
            nome_loja = loja
        if not caixa:
            nome_caixa = _('Todos')
        else:
            nome_caixa = caixa
        if not turno_id:
            turno = _('Todos')
        else:
            turno = nome_turno
        if not finalizadora_id:
            finalizadora = _('Todas')
        else:
            finalizadora = nome_finalizadora

        data_inicial_cabecalho = time.strftime("%d/%m/%Y", time.strptime(data_inicial, "%Y-%m-%d"))
        data_final_cabecalho = time.strftime("%d/%m/%Y", time.strptime(data_final, "%Y-%m-%d"))
        periodo = data_inicial, data_final
        dados = self._get_dados(cr, periodo, loja, caixa, turno_id, nome_finalizadora)
        det_format = '|{:<12}|{:<12}|{:^20}|{:<33}|{:<44}|{:>26}|'.decode('utf-8')

        listagem.informaCab(title1)
        listagem.informaCab('')
        listagem.informaCab(title2)
        listagem.informaCab('')
        listagem.informaCab(
            '{:<10} {:^10} {:^3} {:^10} {:>15} {:<40} {:>15} {:<40}'.decode('utf-8').format(
            _('Periodo:')[:10], data_inicial_cabecalho, _('a'), data_final_cabecalho,
            _('Loja:')[:5], nome_loja, _('Caixa:')[:6], nome_caixa))
        listagem.informaCab('{:<6} {:<37} {:>15} {:<20}'.decode('utf-8').format(
            _('Turno:')[:6], turno, _('Finalizadora:')[:15], finalizadora))
        listagem.informaCab('*' + '-' * 152 + '*')
        listagem.informaCab(det_format.format(
            _('LOJA')[:12], _('CAIXA')[:12], _('CUPOM')[:20], _('FINALIZADORA')[:33],
            _('OPERADOR')[:44], _('VALOR')[-26:]))
        listagem.informaCab('*' + '-' * 152 + '*')
        listagem.rodapePadrao()

        total_geral = sub_total = sub_total_fin = 0.0
        caixa = dados[0].get('caixa') if dados else None
        finalizadora = dados[0].get('finalizadora') if dados else None

        for dado in dados:
            # Se nao for o ultimo registro, recupera os dados do proximo registro
            proximo = dados[dados.index(dado) + 1] if dado != dados[-1] else dado

            # Verifica se o item seguinte da lista e um cancelamento de cupom. Se for, nao imprime
            # os valores e descosidera os valores no total geral
            if dado.get('tipo') != proximo.get('tipo') or dado.get('tipo') == 'C':
                continue

            # Subtotal por finalizadora
            if finalizadora != dado.get('finalizadora'):
                listagem.imprimeLinha('|{:>125}|{:>26}|'.decode('utf-8').format(
                    _('Subtotal - %s' % finalizadora)[:125], locale.format('%.2f', sub_total_fin, grouping=True)[-26:]))
                sub_total_fin = 0.0
                finalizadora = dado.get('finalizadora')

            # Subtotal por Caixa
            if caixa != dado.get('caixa'):
                listagem.imprimeLinha('|{:>125}|{:>26}|'.decode('utf-8').format(
                    _('Subtotal')[:125], locale.format('%.2f', sub_total, grouping=True)[-26:]))
                sub_total = 0.0
                caixa = dado.get('caixa')

            total_cupom = dado.get('total_cupom') or 0.0
            listagem.imprimeLinha(det_format.format(
                dado.get('loja'), dado.get('caixa'), dado.get('cupom'),
                dado.get('finalizadora')[:33], dado.get('operador')[:44],
                locale.format('%.2f', total_cupom, grouping=True)[-26:]))
            total_geral += total_cupom
            sub_total += total_cupom
            sub_total_fin += total_cupom

        # Subtotal por finalizadora
        listagem.imprimeLinha('|{:>125}|{:>26}|'.decode('utf-8').format(
            _('Subtotal - %s' % finalizadora)[:125],
            locale.format('%.2f', sub_total_fin, grouping=True)[-26:]))

        # Subtotal por Caixa (ultimo caixa)
        listagem.imprimeLinha('|{:>125}|{:>26}|'.decode('utf-8').format(
            _('Subtotal')[:125], locale.format('%.2f', sub_total, grouping=True)[-26:]))

        # Total geral
        listagem.imprimeLinha('*' + '-' * 152 + '*')
        listagem.imprimeLinha('|{:>125}|{:>26}|'.decode('utf-8').format(
            _('Total geral')[:125], locale.format('%.2f', total_geral, grouping=True)[-26:]))
        listagem.imprimeLinha('*' + '-' * 152 + '*')
        listagem.finalizaImpressao()

        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    # noinspection PyMethodMayBeStatic
    @staticmethod
    def _get_dados(cr, periodo, loja=False, caixa=False, turno_id=False, finalizadora=False):
        sql = """
            SELECT
                supcap_loja                         AS loja,
                supcap_caixa                        AS caixa,
                supcap_nro_cupom                    AS cupom,
                supcap_tipo                         AS tipo,
                supfec_nome_finalizadora            AS finalizadora,
                (
                    SELECT name
                    FROM supervisor_cad_operadores
                    WHERE id = supcap_operador_id
                )                                   AS operador,
                sum(supfec_valor - supfec_troco)    AS total_cupom
            FROM supervisor_mov_capa smc
            INNER JOIN supervisor_mov_fechamentos smf
                ON smc.id = smf.supfec_capa_id
            WHERE
                supcap_state IN ('3' ,'6')
                AND supcap_data_cupom BETWEEN '%s' AND '%s'
        """ % periodo

        if loja:
            sql += """
              AND supcap_loja = %s
            """ % loja

        if caixa:
            sql += """
              AND supcap_caixa = %s
            """ % caixa

        if turno_id:
            sql += """
              AND supcap_operador_id IN (SELECT supcax_operador_id
                                         FROM   supervisor_mov_caixa
                                         WHERE  id = %s)
            """ % turno_id

        if finalizadora:
            sql += """
                AND supfec_nome_finalizadora = '%s'
            """ % finalizadora

        sql += """
            GROUP BY supcap_data_cupom, supcap_loja, supcap_caixa, supcap_nro_cupom, supcap_tipo,
                     supfec_nome_finalizadora, supcap_operador_id
            ORDER BY loja, caixa, finalizadora, cupom
        """

        cr.execute(sql)

        return cr.dictfetchall()


report_custom('report.listagem_detalhamento_finalizadora')
