# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_acompanhamento_doacao_registros(osv.TransientModel):
    _name = 'wizard.acompanhamento.doacao.registros'
    _columns = {
        'doacao_id': fields.many2one("wizard.acompanhamento.doacao", 'Doacao', ondelete="cascade",
                                     required=True, select=True),
        'coo': fields.integer('Coo'),
        'data': fields.date('Data'),
        'loja': fields.integer('Loja'),
        'caixa': fields.integer('Caixa'),
        'valor': fields.float('Valor', digits=(14,2)),
    }


class wizard_acompanhamento_doacao(osv.TransientModel):
    _name = 'wizard.acompanhamento.doacao'
    _columns = {
        'data_ini': fields.date('Data Inicial', required=True),
        'data_fim': fields.date('Data Final', required=True),
        'loja': fields.integer('Loja'),
        'caixa': fields.integer('Caixa'),
        'registro_ids': fields.one2many('wizard.acompanhamento.doacao.registros', 'doacao_id',
                                        'Doacoes'),
    }

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_parametros = self.pool.get('supervisor.cfg.parametros')
        acompanhamento_doacao_dados = self.read(cr, uid, ids, [], context=context)
        for x in acompanhamento_doacao_dados:
            datas = {
                'data_ini': x.get('data_ini'),
                'data_fim': x.get('data_fim'),
                'loja': x.get('loja', False),
                'caixa': x.get('caixa', False),
            }

        par_ids = obj_parametros.search(cr, uid, [], context=context)
        tot = obj_parametros.read(cr, uid, par_ids[0], ['supcfp_tot_doacao'], context=context
                                  ).get('supcfp_tot_doacao')
        dados_doacao = self.get_dados_doacao(cr, datas, tot, context)
        if not dados_doacao:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha registros com estes filtros!\nVerifique o totalizador de'
                                   ' doacao no cadastro de parametros'))

        registros = []
        for i in dados_doacao:
            registros.append([0,0, i])

        obj_registros = self.pool.get('wizard.acompanhamento.doacao.registros')
        registro_ids = obj_registros.search(cr, uid, [], context=context)
        if registro_ids:
            obj_registros.unlink(cr, uid, registro_ids, context=context)

        vals = {'registro_ids': registros}
        self.write(cr, uid, ids, vals, context)
        return True

    def button_print_doacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_filtros = self.read(cr, uid, ids[0], context=context)
        obj_registros = self.pool.get('wizard.acompanhamento.doacao.registros')
        registro_ids = obj_registros.search(cr, uid, [('doacao_id','in',ids)], context=context)
        dados_registros = obj_registros.read(cr, uid, registro_ids, context=context)
        datas= {'dados': dados_registros,
                'data_ini': dados_filtros.get('data_ini'),
                'data_fim': dados_filtros.get('data_fim'),
                'caixa': dados_filtros.get('caixa'),
                'loja': dados_filtros.get('loja')}
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_acompanhamento_doacao',
            'datas': datas,
        }

    def get_dados_doacao(self, cr, datas, tot, context=None):
        context = context and context.copy() or {}
        caixa = datas.get('caixa')
        loja = datas.get('loja')
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')
        cond_caixa = ''
        cond_loja = ''
        if caixa:
            cond_caixa = 'AND smc.supcap_caixa = %s' % caixa

        if loja:
            cond_loja = 'AND smc.supcap_loja = %s' % loja

        sql = """
                        SELECT smc.supcap_total_cupom AS valor, 
                               smc.supcap_nro_cupom   AS coo,
                               smc.supcap_data_cupom  AS data,
                               smc.supcap_caixa       AS caixa,
                               smc.supcap_loja        AS loja
                        FROM supervisor_mov_capa  AS smc
                        WHERE smc.supcap_data_cupom >= '{data_ini}'
                          AND smc.supcap_data_cupom <= '{data_fim}'
                          AND smc.supcap_tipo = '{tipo}'
                          {cond_loja}
                          {cond_caixa}
                          AND smc.supcap_cod_tot_nao_fiscal = {tot}
                        ORDER BY smc.supcap_loja, smc.supcap_caixa, smc.supcap_nro_cupom
                """.format(tipo='W',
                           data_ini=data_ini,
                           data_fim=data_fim,
                           cond_loja=cond_loja,
                           cond_caixa=cond_caixa,
                           tot=tot)
        cr.execute(sql)
        return cr.dictfetchall()

