# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class wiz_supervisor_detalhamento_finalizadora(osv.TransientModel):
    # prefixo: wizsdf_
    _name = 'wiz.supervisor.detalhamento.finalizadora'

    _columns = {
        'wizsdf_data_inicial': fields.date('Data inicial', required=True),
        'wizsdf_data_final': fields.date('Data final', required=True),
        'wizsdf_loja': fields.integer('Loja'),
        'wizsdf_caixa': fields.integer('Caixa'),
        'wizsdf_turno_id': fields.many2one(
            'supervisor.mov.caixa', 'Turno',
            domain="[('supcax_loja', '=', wizsdf_loja), ('supcax_caixa', '=', wizsdf_caixa),"
                   "('supcax_data_hora_abertura', '>=', wizsdf_data_inicial),"
                   "('supcax_data_hora_fechamento', '<=', wizsdf_data_final)]"),
        'wizsdf_finalizadora_id': fields.many2one('supervisor.cad.finalizadora', 'Finalizadora'),
        'wizsdf_mesma_data': fields.boolean('Datas iguais'),
    }

    _defaults = {
        'wizsdf_mesma_data': False,
    }

    def wizsdf_onchange_data(self, cr, uid, ids, data_inicial, data_final, context=None):
        """
        Método chamado após inclusão/alteração nos campos wizsdf_data_inicial e wizsdf_data_final.
        Caso eles sejam iguais, seta o campo wizsdf_mesma_data para True, senão False.
        :param cr: cursor no banco de dados
        :param uid: identificador do usuario, id da tabela res_users
        :param ids: lista de id da wizard wiz_supervisor_detalhamento_finalizadora
        :param data_inicial: data de inicio para o periodo definido
        :param data_final: data fim para o periodo definido
        :param context: dicionario com valores contextuais
        :return: dicionario com os campos e seus valores definidos
        """
        context = context and context.copy() or {}
        res = {}
        if not data_final or not data_inicial:
            return res
        res['value'] = {'wizsdf_mesma_data': False}
        if data_inicial == data_final:
            res['value'] = {'wizsdf_mesma_data': True}
        return res

    def wizsdf_button_print(self, cr, uid, ids, context=None):
        """
        O metodo reune os dados necessarios para a impressao e os manda para a classe do relatorio
        :param cr: cursor no banco de dados
        :param uid: identificador do usuario, id da tabela res_users
        :param ids: lista de id da wizard wiz_supervisor_detalhamento_finalizadora
        :param context: dicionario com valores contextuais
        :return: dicionario com os dados do relatorio, este fara a impressao do arquivo pdf
        """
        context = context and context.copy() or {}
        brw_detalhamento_finalizadora = self.browse(cr, uid, ids, context)
        obj_supervisor_mov_caixa = self.pool.get('supervisor.mov.caixa')
        datas = {}
        for x in brw_detalhamento_finalizadora:
            turno_id = x.wizsdf_turno_id.id
            nome_turno = ''
            if turno_id:
                nome_turno = obj_supervisor_mov_caixa.name_get(cr, uid, [turno_id], context)
                nome_turno = nome_turno[0][1]
            loja = x.wizsdf_loja
            caixa = x.wizsdf_caixa
            if not loja:
                caixa = 0
            if x.wizsdf_data_inicial != x.wizsdf_data_final:
                turno_id = False
            datas.update({
                'data_inicial': x.wizsdf_data_inicial,
                'data_final': x.wizsdf_data_final,
                'loja': loja,
                'caixa': caixa,
                'turno_id': turno_id,
                'nome_turno': nome_turno,
                'finalizadora_id': x.wizsdf_finalizadora_id.id,
                'nome_finalizadora': x.wizsdf_finalizadora_id.name})
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_detalhamento_finalizadora',
            'datas': datas
        }
