# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
from pynfe.nfce.manual_600.NFCe import NFCe as nfc_e
from datetime import datetime
from openerp.addons.supervisor import print_log
import socket
import re


class supervisor_wizard_inutilizar_nota(osv.TransientModel):

    _name = 'supervisor.wizard.inutilizar.nota'

    _columns = {
        'wizino_loja': fields.integer('Loja', required=True),
        'wizino_caixa': fields.integer('Caixa', required=True),
        'wizino_inicio': fields.integer('Inicio do intervalo', required=True),
        'wizino_fim': fields.integer('Fim do intervalo', required=True),
        'wizino_justificativa': fields.char('Justificativa', size=100, required=True),
        'wizino_finalizado': fields.boolean('Finalizado')
    }

    _defaults = {
        'wizino_finalizado': False,
    }

    def wizcan_button_inutilizar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_filtro = self.browse(cr, uid, ids[0], context)
        msg = []
        if not brw_filtro.wizino_loja:
            msg.append(_('Loja'))

        if not brw_filtro.wizino_caixa or not self.pool.get('supervisor.cad.pdvs').search(cr, uid,
                                 [('suppdv_loja','=', brw_filtro.wizino_loja or 0),
                                  ('suppdv_caixa', '=', brw_filtro.wizino_caixa)], context=context):
            msg.append(_('Caixa'))

        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('%s invalido') % ' e '.join(msg))

        if not brw_filtro.wizino_inicio or brw_filtro.wizino_inicio > brw_filtro.wizino_fim:
            msg.append(_('Inicio'))

        if not brw_filtro.wizino_fim or brw_filtro.wizino_inicio > brw_filtro.wizino_fim:
            msg.append(_('Fim'))

        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('%s do intervalo invalido') % ' e '.join(msg))

        if len(brw_filtro.wizino_justificativa or '') < 15:
            raise osv.except_osv(_('Atencao!'),
                                 _('A justificativa deve ter mais de 15 caracters'))

        operador_ids = self.pool.get('supervisor.cad.operadores').search(cr, uid, [],
                                                                         context=context)
        if not operador_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum operador cadastrado!'))

        obj_conifg = self.pool.get('supervisor.cfg.configuracao')
        obj_capa = self.pool.get('supervisor.mov.capa')
        config_ids = obj_conifg.search(cr, uid, [('supcfc_loja', '=', brw_filtro.wizino_loja)],
                                       context=context)
        if not config_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Sem cadastro de configuracao!'))

        brw_config = obj_conifg.browse(cr, uid, config_ids[0], context=context)
        cnpj = re.sub('[^0-9]', '', brw_config.supcfc_cnpj) or ''
        if not cnpj:
            raise osv.except_osv(_('Atencao!'), _('Nao existe CNPJ cadastro para o emitente.'))

        ambiente = brw_config.supcfc_ambiente == 'P' and 1 or 2
        ini = brw_filtro.wizino_inicio
        fim = brw_filtro.wizino_fim
        capa_ids = obj_capa.search(cr, uid, [('supcap_nro_cupom', 'in', range(ini, fim+1)),
                                             ('supcap_caixa', '=', brw_filtro.wizino_caixa),
                                             ('supcap_loja', '=', brw_filtro.wizino_loja),
                                             ('supcap_tipo','!=', 'D')],
                                   context=context)
        dados_capa = obj_capa.read(cr, uid, capa_ids, ['supcap_nro_cupom', 'supcap_state'], context)
        nro_notas_f = []
        nro_notas_c = []
        nro_notas_i = []
        nro_notas_d = []
        for i in dados_capa:
            if i.get('supcap_state') in ('3','4'):
                nro_notas_f.append('%s' % i.get('supcap_nro_cupom'))
            elif i.get('supcap_state') == 'C':
                nro_notas_c.append('%s' % i.get('supcap_nro_cupom'))
            elif i.get('supcap_state') == 'I':
                nro_notas_i.append('%s' % i.get('supcap_nro_cupom'))
            elif i.get('supcap_state') == 'D':
                nro_notas_d.append('%s' % i.get('supcap_nro_cupom'))

        map(lambda x: x.sort(), [nro_notas_f,nro_notas_c,nro_notas_i,nro_notas_d])
        if nro_notas_f:
            msg.append('Faturadas: %s' % ', '.join(nro_notas_f))
        if nro_notas_c:
            msg.append('Canceladas: %s' % ', '.join(nro_notas_c))
        if nro_notas_i:
            msg.append('Inutilizadas: %s' % ', '.join(nro_notas_i))
        if nro_notas_d:
            msg.append('Denegadas: %s' % ', '.join(nro_notas_d))

        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('As notas a seguir ja estao:\n%s' % '\n'.join(msg)))

        justificativa = brw_filtro.wizino_justificativa
        serie = brw_filtro.wizino_caixa
        ano = datetime.now().strftime(u'%y')
        intervalor = '%s a %s' % (ini, fim)
        if ini == fim:
            intervalor = '%s' % ini
        print_log('Supevisor - Inutilizando NFC-e %s  serie %s' % (intervalor,
                                                                   brw_filtro.wizino_caixa), 'I')
        try:
            proc_nfce = nfc_e(brw_config.supcfc_certificado_conteudo,
                              brw_config.supcfc_chave_conteudo,
                              ambiente,
                              brw_config.supcfc_estado)
            processo, __ = proc_nfce.inutilizar_faixa_numeracao(ano, cnpj, serie, ini, fim,
                                                                justificativa)
        except socket.gaierror:
            raise osv.except_osv(_('Atencao!'),
                                 _('Falha no servidor da SEFAZ. '
                                   'Verifique se existe conexao com a Internet.'))
        except IOError:
            raise osv.except_osv(_('Atencao!'),
                                 _('Falha no servidor da SEFAZ. Tente novamente mais tarde.'))

        except Exception as err:
            raise osv.except_osv(_('Atencao!'),
                                 _('Erro de conexao com servidor. \n%s') % err)

        if processo.resposta.infInut.cStat.valor not in (u'102', u'563'):
            msg = '%s - %s.' % (processo.resposta.infInut.cStat.valor,
                                processo.resposta.infInut.xMotivo.valor)

            raise osv.except_osv(_('Atencao!'), msg)

        numero_protocolo = processo.resposta.infInut.nProt.valor
        vals = {'supcap_protocolo': numero_protocolo,
                'supcap_state': 'I',
                'supcap_tipo': 'I',
                'supcap_status': '102 - Inutilizacao de numero homologado',
                'supcap_caixa': brw_filtro.wizino_caixa,
                'supcap_loja': brw_filtro.wizino_loja,
                'supcap_data_cupom': datetime.now().date(),
                'supcap_data_inicio': datetime.now(),
                'supcap_data_fim': datetime.now(),
                'supcap_tipo_comprovante': 'RG',
                'supcap_ambiente': brw_config.supcfc_ambiente,
                'supcap_operador_id': operador_ids[0],

                }
        for i in range(ini, fim+1):
            vals['supcap_nro_cupom'] = i
            obj_capa.create(cr, uid, vals, context=context)

        self.write(cr, uid, ids, {'wizino_finalizado': True}, context)
        return True
