# -*- encoding: utf-8 -*-
from SigUtils.formatastring import formataString
from SigUtils.impressao import Impressora
from SigUtils.numero import getNumeroEditado
from openerp.osv import osv, fields
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
import time
import locale
from openerp import pooler

# noinspection PyProtectedMember
from openerp.tools.translate import _
from datetime import datetime

class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        data_titulo = time.strftime('%d/%m/%Y')
        hora_titulo = pooler.get_pool(cr.dbname).get('supervisor.util').suputi_datetime_to_timezone(
            cr, uid, datetime.now())[11:16]
        obj_supervisor_cfg_configuracao = pooler.get_pool(cr.dbname).get(
            'supervisor.cfg.configuracao')
        configuracao_ids = obj_supervisor_cfg_configuracao.search(cr, uid, [], context=context)
        nome_empresa = ''
        if configuracao_ids:
            nome_empresa = obj_supervisor_cfg_configuracao.browse(
                cr, uid, configuracao_ids[0], context=context).supcfc_cliente or ''
        title1 = ' {:<20}{:^61}{:>11}'.decode('utf-8').format(
            _('Sistema de PDVs')[:15], nome_empresa[:65], _('Pagina: ') + '{{ Page }}')
        title2 = ' {:<15}{:^60}{:>15}'.decode('utf-8').format(
            data_titulo, _('Itens mais vendidos'), hora_titulo)
        pdf_string = StringIO.StringIO()
        listagem = Impressora()

        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Itens mais vendidos'))
        listagem.informaTamLinha(82)
        listagem.defineMargem(1, 1, 0.5, 0.5)
        listagem.defineFonte(tam=9.7)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        data_inicial = datas.get('data_inicial')
        data_final = datas.get('data_final')
        loja = datas.get('loja')
        ordem = datas.get('ordem')
        nome_loja = _('Todas') if not loja else loja

        listagem.informaCab(title1)
        listagem.informaCab('')
        listagem.informaCab(title2)
        listagem.informaCab('')
        listagem.informaCab(' {:>21} {:^25} {:>10} {:<10}'.decode('utf-8').format(
            _('Periodo do movimento:')[:21],  _('%s a %s') % (data_inicial, data_final),
            _('Loja:'), nome_loja))
        listagem.informaCab('*' + '-' * 90 + '*')
        listagem.informaCab('|{:<14}|{:<45}|{:>12}|{:>16}|'.decode('utf-8').format(
            _('Codigo')[:14], _('Descricao do item')[:45], _('Quantidade')[:12],
            _('Valor da venda')[:16]))
        listagem.informaCab('*' + '-' * 90 + '*')
        listagem.rodapePadrao()

        if loja:
            args_loja = "AND supcap_loja = %s" % loja
        else:
            args_loja = ''

        if ordem == 'Q':
            order = "quantidade"
        else:
            order = "valor_total"

        vals_sql = dict(
            args_data_inicial=data_inicial, args_data_final=data_final, args_loja=args_loja,
            order=order)

        sql = """
            SELECT supcor_item            AS item,
                   supcor_nome_item       AS nome_item,
                   Sum(
                        CASE
                            WHEN mov_corpo.supcor_venda_cancelamento = 'V'
                                THEN COALESCE(supcor_quantidade, 0)
                            WHEN mov_corpo.supcor_venda_cancelamento = 'C'
                                AND mov_capa.supcap_tipo = 'C' AND mov_capa.supcap_state IS NULL
                                THEN COALESCE(supcor_quantidade, 0) * -1
                            ELSE 0
                        END
                   )                      AS quantidade,
                   Sum(
                        CASE
                            WHEN mov_corpo.supcor_venda_cancelamento = 'V'
                                THEN COALESCE(supcor_preco_total, 0) + 
                                     COALESCE(supcor_acrescimo_item, 0) -
                                     COALESCE(supcor_desconto_item, 0)
                            WHEN mov_corpo.supcor_venda_cancelamento = 'C'
                                AND mov_capa.supcap_tipo = 'C' AND mov_capa.supcap_state IS NULL
                                THEN (COALESCE(supcor_preco_total, 0) * -1) - 
                                     COALESCE(supcor_acrescimo_item, 0) +
                                     COALESCE(supcor_desconto_item, 0)
                            ELSE 0
		                END
                   )                      AS valor_total
            FROM   supervisor_mov_corpos mov_corpo
                   INNER JOIN supervisor_mov_capa mov_capa
                           ON mov_corpo.supcor_capa_id = mov_capa.id
            WHERE  mov_capa.supcap_data_cupom >= '%(args_data_inicial)s'
                   AND mov_capa.supcap_data_cupom <= '%(args_data_final)s'
                   AND mov_capa.supcap_tipo in ('V','C','D')
                    %(args_loja)s
            GROUP  BY supcor_item,
                      supcor_nome_item
            ORDER  BY %(order)s DESC
        """ % vals_sql

        # Quantidade maxima de registros
        if datas.get('limite_itens'):
            sql += ' LIMIT %s' % datas.get('limite_itens')

        cr.execute(sql)
        reg = cr.dictfetchall()
        reg = filter(lambda i: i.get('quantidade') > 0 and i.get('valor_total') > 0, reg)
        itens_quantidade_total = itens_valor_total = 0

        for i in reg:
            cod_item = i.get('item')
            nome_item = i.get('nome_item')
            quantidade = i.get('quantidade') or 0
            valor_total = i.get('valor_total') or 0
            linha = '|{:<14}|{:<45}|{:>12}|{:>16}|'.decode('utf-8').format(
                cod_item[:14], nome_item[:45],
                locale.format('%.3f', quantidade, grouping=True)[-12:],
                locale.format('%.2f', valor_total, grouping=True)[-16:])
            listagem.imprimeLinha(linha)

            # Somatorio dos totais
            itens_quantidade_total += quantidade
            itens_valor_total += valor_total

        listagem.imprimeLinha('|' + '-' * 90 + '|')
        linha = '|{:>60}|{:>12}|{:>16}|'.decode('utf-8').format(
            _('Total'), getNumeroEditado(itens_quantidade_total, 3),
            getNumeroEditado(itens_valor_total, 2))
        listagem.imprimeLinha(linha)
        listagem.imprimeLinha('*' + '-' * 90 + '*')
        listagem.finalizaImpressao()

        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

report_custom('report.wizard_relatorio_itens_mais_vendidos')


class wizard_relatorio_itens_mais_vendidos(osv.TransientModel):
    _name = 'wizard.relatorio.itens.mais.vendidos'
    _columns = {
        'data_inicial': fields.date('Data inicial', required=True),
        'data_final': fields.date('Data final', required=True),
        'loja': fields.integer('Loja'),
        'limite_itens': fields.integer('Numero registros'),
        'ordem': fields.selection([('Q', 'Quantidade'), ('V', 'Valor')], 'Ordem', required=True),
    }

    _defaults = {
        'limite_itens': 0,
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        itens_mais_vendidos_dados = self.read(cr, uid, ids, [], context=context)

        if isinstance(itens_mais_vendidos_dados, (list, tuple)):
            itens_mais_vendidos_dados = itens_mais_vendidos_dados[0]

        datas = {
            'data_inicial': itens_mais_vendidos_dados.get('data_inicial'),
            'data_final': itens_mais_vendidos_dados.get('data_final'),
            'loja': itens_mais_vendidos_dados.get('loja'),
            'limite_itens': itens_mais_vendidos_dados.get('limite_itens'),
            'ordem': itens_mais_vendidos_dados.get('ordem'),
        }

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'wizard_relatorio_itens_mais_vendidos',
            'datas': datas,
        }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
