# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields
import base64
from SigUtils.numero import getNumeroEditado
from openerp.tools.translate import _


class wiz_supervisor_exporta_busca_preco(osv.TransientModel):
    # prefixo: wizebp_

    _name = 'wiz.supervisor.exporta.busca.preco'

    _columns = {
        'wizebp_file_dados': fields.binary('Arquivo', readonly=True),
        'wizebp_file_desc': fields.char('Nome', size=50, readonly=True),
        'wizebp_tipo_layout': fields.selection([('G', 'Gertec TC505'),
                                                ('S', 'Sweda ECD1200'),], 'Modelo', required=True, size=1,
                                                 help="Tipo do layout do arquivo exportado"),
    }

    _defaults = {
        'wizebp_file_desc': 'pricetab.txt',
        'wizebp_tipo_layout': 'G',
    }

    def wizebp_button_exportar(self, cr, uid, ids, context=None):
        """
        O metodo gera para download um arquivo txt usado para a busca de preco.
        E realizada a busca dos codigos de barra ativos, a descricao e o preco de venda dos
        produtos relacionados. Posteriormente, esses dados sao concatenados e separados pelo
        pipe (|), sendo cada linha da string um conjunto desse tipo de dados.
        @param cr: cursor no banco de dados
        @param uid: id da tabela res_users, identificador do usuario
        @param ids: lista de ids da tabela desse modelo
        @param context: dicionario com valores contextuais
        @return: True
        """
        context = context and context.copy() or {}
        obj_supervisor_cfg_configuracao = self.pool.get('supervisor.cfg.configuracao')
        configuracao_ids = obj_supervisor_cfg_configuracao.search(cr, uid, [], context=context)
        loja = False
        if configuracao_ids:
            loja = obj_supervisor_cfg_configuracao.browse(
                cr, uid, configuracao_ids[0], context=context).supcfc_loja or False
        if not loja:
            raise osv.except_osv(_('Atencao'), _('Nao ha cadastro da configuracao do supervisor'))
        sql = """
            SELECT
                supite_codigo_item            AS codigo_barra,
                supite_descricao_completa     AS nome_produto,
                name                          AS descricao_resumida,
                supite_preco                  AS preco_venda
            FROM
                supervisor_cad_itens sci
            WHERE
                active = True
                AND supite_loja = %s
        """ % loja
        cr.execute(sql)
        dados = cr.dictfetchall()
        arq = ''
        modelo = self.browse(cr, uid, ids[0]).wizebp_tipo_layout
        if modelo == 'G':     # modelo GERTEC
            for x in dados:
                codigo_barra = x.get('codigo_barra')
                descricao_produto = x.get('descricao_resumida') or x.get('nome_produto') or ''
                valor_preco_venda = x.get('preco_venda') or 0.00
                preco_venda_produto = getNumeroEditado(valor_preco_venda or 0.0, 2)
                codigo_barra, descricao_produto, b = self.wizebp_formata_conteudo(
                    cr, uid, codigo_barra, descricao_produto, '')
                if codigo_barra.isdigit():
                    codigo_barra = str(int(codigo_barra))
                arq += '%s|%s|%s|\r\n' % (codigo_barra, descricao_produto, preco_venda_produto)
        elif modelo == 'S':     # modelo SWEDA
            for x in dados:
                codigo_barra = x.get('codigo_barra')
                descricao_produto = x.get('descricao_resumida') or x.get('nome_produto') or ''
                valor_preco_venda = x.get('preco_venda') or 0.00
                preco_venda_produto = getNumeroEditado(valor_preco_venda or 0.0, 2,
                                                       thousands_sep='')
                codigo_barra, descricao_produto, preco_venda_produto = self.wizebp_formata_conteudo(cr, uid, codigo_barra, descricao_produto, preco_venda_produto)
                arq += '%s%s%s\r\n' % (codigo_barra, descricao_produto, preco_venda_produto)
        if arq:
            file_dados = base64.encodestring(arq.encode('utf-8', 'strict'))
            self.write(cr, uid, ids, {'wizebp_file_dados': file_dados}, context)
        return True

    def wizebp_formata_conteudo(self, cr, uid, codigo_barra, descricao_produto, preco_venda_produto):
        if len(codigo_barra) > 13:
            codigo_barra = codigo_barra[-13:]
        descricao_produto = descricao_produto.replace(u"'",u' ')
        descricao_produto = descricao_produto.replace(u"´",u' ')
        if len(descricao_produto) < 20:
            dif = 20 - len(descricao_produto)
            complemento = ''
            for i in range(0,dif):
                complemento += ' '
            descricao_produto += complemento
        else:
            descricao_produto=descricao_produto[0:20]
        preco=''
        for i in preco_venda_produto:
            if i != ',':
                preco+= i
        preco_venda_produto = preco
        if len(preco_venda_produto) < 12:
            dif = 12 - len(preco_venda_produto)
            complemento = ''
            for i in range(0,dif):
                complemento += '0'
            complemento += preco_venda_produto
            preco_venda_produto = complemento
        else:
            preco_venda_produto=preco_venda_produto[0:12]
        return codigo_barra, descricao_produto, preco_venda_produto
