# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
from funcoes import get_permissao_exportar_reducao_z


class wizard_supervisor_exporta_para_ecf(osv.TransientModel):
    _name = "wizard.supervisor.exporta.para.ecf"

    _columns = {
        'itens': fields.boolean('Itens'),
        'resticao_itens': fields.boolean('Restricao de itens'),
        'tributacao': fields.boolean('Tributacao'),
        'tributacao_nfce': fields.boolean('Tributacao NFC-e'),
        'mensagem': fields.boolean('Mensagem'),
        'operadores': fields.boolean('Operadores'),
        'finalizadoras': fields.boolean('Finalizadoras'),
        'vendedor': fields.boolean('Vendedor'),
        'clientes': fields.boolean('Clientes'),
        'parametros': fields.boolean('Parametros'),
        'bandeiras': fields.boolean('Bandeiras'),
        'reducao_z': fields.boolean('Reducao Z'),
        'pdv_ids': fields.many2many('supervisor.cad.pdvs', 'supervisor_pdv_exporta_para_ecf', 'id',
                                    'pdv_id', 'PDVs'),
        'exportado': fields.boolean('Exportado'),
        'permite_exportar_reducao_z': fields.function(
            get_permissao_exportar_reducao_z, method=True, string='Permite exportar reducao Z',
            type='boolean', store=False),
    }
    _defaults = {
        'pdv_ids': lambda s, cr, uid, c: s.pool.get('supervisor.cad.pdvs').search(cr, uid, [], context=c),
    }
    def get_acao_promocao(self, cr, uid, ids, vals, args, dict_exportado, context=None):
        """
        Esta funcao eh defina por heranca no modulo supervisor_promocoes
        """
        pass

    def get_acao_sorteio(self, cr, uid, ids, vals, args, dict_exportado, context=None):
        """
        Esta funcao eh defina por heranca no modulo supervisor_promocoes
        """
        pass

    def default_get(self, cr, uid, fields_list, context=None):
        """
        O metodo inclui valores padrao aos campos recebebidos no parametro fields_list.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista com a descricao dos campos que receberao os valores.
        :type fields_list: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com a descricao dos campos e seus respectivos valores.
        :rtype: dict
        """
        context = context and context.copy() or {}
        res = super(wizard_supervisor_exporta_para_ecf, self).default_get(cr, uid, fields_list,
                                                                          context)
        res['permite_exportar_reducao_z'] = self.verifica_permissao_exportar_z(cr, uid, context)
        return res

    def verifica_permissao_exportar_z(self, cr, uid, context=None):
        """
        O metodo verifica se o usuario (uid) possui permissao no grupo 'Supervisor / Manager'.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso possua a permissao, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            SELECT
                gid
            FROM
                res_groups_users_rel rgur
            INNER JOIN
                res_groups rg ON rg.id = rgur.gid
            WHERE
                rg.name = 'Supervisor / Manager'
                AND rgur.uid = %s
        """ % uid
        cr.execute(sql)
        return any(cr.dictfetchall())

    def get_vals_acao_pendente(self, brw_pdv, context=None):
        context = context and context.copy() or {}
        return  {'supeape_loja': brw_pdv.suppdv_loja,
                 'supeape_pdv': brw_pdv.suppdv_caixa,
                 'supape_situacao': 'A',
                 }

    def get_args_acao_pendente(self, brw_pdv, context=None):
        context = context and context.copy() or {}
        args = [['supape_tabela', '=', ''],
                ['supeape_loja', '=', brw_pdv.suppdv_loja],
                ['supeape_pdv', '=', brw_pdv.suppdv_caixa],
                ['supape_situacao', '=', 'A']]
        return args

    def exporta_verificar_promocoes_tipo4(self, cr, uid, context):
        pass

    def exportar_para_ecf_acao_pendente(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.pool.get('valida.pendencias').valpen_verificar_licenca_ativa(cr, uid, context)
        exportado = False
        obj_mov_acao_pendente = self.pool.get('supervisor.mov.acao.pendente')
        acao_ids = obj_mov_acao_pendente.search(cr, uid, [('supape_tabela', '=', 'importando')],
                                                context=context)
        if acao_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Importacao de arquivos sendo feita! Aguarde por um instante.'))
        
        browse_exporta_para_ecf = self.browse(cr, uid, ids[0])
        ecf_ids = self.pool.get('supervisor.cfg.ecf.autorizada').search(cr, uid, [], context=context)
        lista_ecf = []
        for i in self.pool.get('supervisor.cfg.ecf.autorizada').read(cr, uid, ecf_ids,
                                         ['supcfa_nserie', 'supcfa_loja', 'supcfa_caixa'], context):
            if i.get('supcfa_nserie'):
                lista_ecf.append('%s-%s' % (i.get('supcfa_loja'), i.get('supcfa_caixa')))

        pdv_ids = browse_exporta_para_ecf.pdv_ids

        for i in pdv_ids:
            vals = self.get_vals_acao_pendente(i, context=context)
            args = self.get_args_acao_pendente(i, context=context)
            if browse_exporta_para_ecf.itens:
                browse_exporta_para_ecf.tributacao_nfce = True
                args[0][2] = 'Itens'
                self.exporta_verificar_promocoes_tipo4(cr, uid, context)
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Itens'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.resticao_itens:
                args[0][2] = 'Restricao'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Restricao'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            dict_exportado = {}
            self.get_acao_promocao(cr, uid, ids, vals, args, dict_exportado, context=context)
            self.get_acao_sorteio(cr, uid, ids, vals, args, dict_exportado, context=context)
            if dict_exportado:
                exportado = True

            if browse_exporta_para_ecf.tributacao:
                args[0][2] = 'Tributacao'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Tributacao'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.tributacao_nfce:
                chave = '%s-%s' % (i.suppdv_loja, i.suppdv_caixa)
                if chave not in lista_ecf:  # apenas pdvs com NFCE, nao cria para ECF
                    args[0][2] = 'Tributacao NFC-e'
                    acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                    exportado = True
                    if not acao_pendente_ids:
                        vals['supape_tabela'] = 'Tributacao NFC-e'
                        obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.mensagem:
                args[0][2] = 'Mensagem'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Mensagem'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.operadores:
                args[0][2] = 'Operadores'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Operadores'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.finalizadoras:
                args[0][2] = 'Finalizadoras'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Finalizadoras'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.vendedor:
                args[0][2] = 'Vendedor'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Vendedor'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.clientes:
                args[0][2] = 'Clientes'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Clientes'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.parametros:
                args[0][2] = 'Parametros'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Parametros'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.bandeiras:
                args[0][2] = 'Bandeiras'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Bandeiras'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

            if browse_exporta_para_ecf.reducao_z:
                args[0][2] = 'Reducao_Z'
                acao_pendente_ids = obj_mov_acao_pendente.search(cr, uid, args, context=context)
                exportado = True
                if not acao_pendente_ids:
                    vals['supape_tabela'] = 'Reducao_Z'
                    obj_mov_acao_pendente.create(cr, uid, vals, context=context)

        if exportado:
            obj_model_data = self.pool.get('ir.model.data')
            reference = obj_model_data.get_object_reference(
                cr, uid, 'supervisor', 'view_wizard_supervisor_exporta_para_ecf_log_form')
            view_id = reference and reference[1] or False
            return {'name': 'Exportacao para ECF',
                    'view_type': 'form',
                    'view_mode': 'form',
                    'res_model': 'wizard.supervisor.exporta.para.ecf',
                    'view_id': [view_id],
                    'target': 'new',
                    'type': 'ir.actions.act_window',
                    }
        return True

    def checking_all_boxes(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if ids:
            vals = {'itens': True,
                    'resticao_itens': True,
                    'tributacao': True,
                    'tributacao_nfce': True,
                    'mensagem': True,
                    'operadores': True,
                    'finalizadoras': True,
                    'vendedor': True,
                    'clientes': True,
                    'parametros': True,
                    'bandeiras': True
                    }
            self.write(cr, uid, ids, vals, context)
        return True

    def onchange_exportacao_itens(self, cr, uid, ids, check_item, context=None):
        context = context and context.copy() or {}
        res = {}
        if check_item:
            res['value'] = {'tributacao_nfce': True}

        return res