# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from os import path, listdir, remove
from SigUtils.registro import Registro
import os
import base64
from openerp.tools.translate import _
from datetime import datetime
from openerp.addons.supervisor import print_log


class wizard_supervisor_importa_arquivo(osv.Model):
    # _transient = False
    _name = "wizard.supervisor.importa.arquivo"

    _columns = {
        'arquivo': fields.binary('Arquivo', required=True,
                                 filters='itens.txt,kit.txt,vendedor.txt,mensag.txt,clientes.txt,nseries.txt'),
        'nome_arquivo': fields.char('Nome do arquivo', size=20, required=True,
                                    help="Nome completo do arquivo que esta importando, inclusive com a extensao '.txt'"),
        'arquivos_importados': fields.boolean('Arquivos importados'),
    }

    _defaults = {
        'arquivos_importados': False,
    }

    def onchange_arquivo(self, cr, uid, ids, arquivo, context=None):
        context = context and context.copy() or {}
        res = {}
        res['value'] = {'nome_arquivo': ''}
        return res

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        res = super(wizard_supervisor_importa_arquivo, self).create(cr, uid, vals, context)
        return res

    def check_modulo_instalado(self, cr, uid, modulo, context):
        args = [('name', '=', modulo), ('state', 'in', ('installed', 'to upgrade'))]
        modulo_ids = self.pool.get('ir.module.module').search(cr, uid, args, context=context)
        if modulo_ids:
            return True
        else:
            return False

    def button_importar_manual_arquivo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_config = self.pool.get('supervisor.cfg.configuracao')
        browse_arquivo = self.browse(cr, uid, ids[0])
        arquivo = browse_arquivo.arquivo or 'teste'
        nome_arquivo = browse_arquivo.nome_arquivo

        configuracao_id = obj_config.search(cr, uid, [], context=context)
        if not configuracao_id:
            raise osv.except_osv(_('Atencao'), _('Nao ha configuracao cadastrada'))

        diretorio_importacao = obj_config.browse(cr, uid, configuracao_id[0]).supcfc_diretorio_importacao or ''
        if not diretorio_importacao:
            raise osv.except_osv(_('Atencao'), _('Cadastre o diretorio para importacao na '
                                                     'configuracao'))

        vals = {
            'importa_arquivo_id': ids[0],
            'item_arquivo': arquivo,
            'file_name': nome_arquivo,
        }
        arquivo = base64.b64decode(arquivo).split('\n')
        if not arquivo[-1]:
            arquivo.pop()

        self.funcao_improtar_arquivos(cr, uid, ids, nome_arquivo, arquivo, context)
        return True

    def funcao_improtar_arquivos(self, cr, uid, ids, n_arquivo=False, dados=False, context=None):
        context = context and context.copy() or {}
        obj_sup_util = self.pool.get('supervisor.util')
        self.suputi_datetime_to_timezone = obj_sup_util.suputi_datetime_to_timezone
        self.efetuar_rateio = obj_sup_util.efetuar_rateio
        cwd = os.path.abspath(os.path.dirname(__file__))
        nome_arquivos = (fname for fname in os.listdir(cwd) \
                         if os.path.isdir(os.path.join(cwd, fname)))

        for nome_arquivo in nome_arquivos:
            module_importacao_path = os.path.join(cwd, '%s/arquivos' % nome_arquivo)
            registros = Registro(module_importacao_path)

        obj_config = self.pool.get('supervisor.cfg.configuracao')
        config_ids = obj_config.search(cr, uid, [], context=context)
        if not config_ids:
            if not context.get('supervisor_importacao_automatica'):
                raise osv.except_osv(_('Atencao'), _('Nao ha configuracao cadastrada'))
        # controle de exportacao para varias empresas da classes que nao possuem empresa, ou seja,
        # o registro eh o mesmos para todas as empresa.

        exportou_clientes = False
        exportou_mensagens = False
        exportou_vendedores = False
        exportou_funcionario = False
        importar = False
        del_arq_importacao = []
        arq_importacao = ['tributacao.txt', 'itens.txt', 'vendedor.txt', 'clientes.txt',
                          'mensag.txt', 'nserie.txt', 'kit.txt', 'promocao.txt', 'funcionario.txt']
        for configuracao in obj_config.browse(cr, uid, config_ids, context=context):
            cfg_loja = configuracao.supcfc_loja
            cfg_empresa = configuracao.supcfc_nro_empresa
            diretorio_importacao = configuracao.supcfc_diretorio_importacao or ''
            if not diretorio_importacao:
                if not context.get('supervisor_importacao_automatica'):
                    raise osv.except_osv(_('Atencao'), _('Cadastre o diretorio para importacao na '
                                                         'configuracao'))
            files_import = []
            files = diretorio_importacao and tuple(listdir(diretorio_importacao)) or []
            for i in arq_importacao:
                if i in files:
                    files_import.append(i)

            files_import.sort()
            if 'tributacao.txt' in files_import:
                files_import.remove('tributacao.txt')
                files_import = ['tributacao.txt'] + files_import

            for i in files_import:
                if n_arquivo and i != n_arquivo:
                    continue

                if not importar:
                    obj_mov_acao_pendente = self.pool.get('supervisor.mov.acao.pendente')
                    if obj_mov_acao_pendente.search(cr, uid, [('supape_situacao', '=', 'B')],
                                                    context=context):
                        if n_arquivo:
                            raise osv.except_osv(_('Atencao!'),
                                                 _('Importacao Bloqueada!\n'
                                                   'Delete a acao pendente de bloqueio!'))
                        else:
                            print_log(_('Importacao Bloqueada por acao pendente!'), 'I')
                            return True

                    importar = True
                    obj_mov_acao_pendente.create(cr, uid, {'supape_tabela': 'importando'},
                                                 context=context)
                    # cria a acao pendente para as outras secoes
                    cr.execute("""UPDATE supervisor_mov_acao_pendente SET supape_situacao = 'B';
                                          COMMIT;""")

                excluir = False
                try:
                    import_file = open(path.join(diretorio_importacao, i))
                except:
                    import_file = False

                if import_file or n_arquivo:
                    nome_arquivo = str(import_file) or n_arquivo
                    if not n_arquivo:
                        dados_arquivo = import_file.readlines()
                    else:
                        dados_arquivo = dados

                    if not exportou_mensagens and 'mensag.txt' in nome_arquivo:
                        self._importa_arquivo_mensagem(cr, uid, import_file, registros,
                                                       dados_arquivo, context=context)
                        excluir = True
                        exportou_mensagens = True
                    elif 'tributacao.txt' in nome_arquivo:
                        self._importa_arquivo_tributacao(cr, uid, registros, dados_arquivo,
                                                         cfg_loja, cfg_empresa, context=context)
                        excluir = True

                    elif 'itens.txt' in nome_arquivo:
                        self._importa_arquivo_itens(cr, uid, registros, dados_arquivo, cfg_loja,
                                                    cfg_empresa, context=context)
                        excluir = True

                    elif not exportou_vendedores and 'vendedor.txt' in nome_arquivo:

                        self._importa_arquivo_vendedor(cr, uid, import_file, registros,
                                                       dados_arquivo, context=context)
                        excluir = True
                        exportou_vendedores = True

                    elif not exportou_clientes and 'clientes.txt' in nome_arquivo:
                        self._importa_arquivo_clientes(cr, uid, import_file, registros,
                                                       dados_arquivo, context)
                        excluir = True
                        exportou_clientes = True

                    elif 'kit.txt' in nome_arquivo:
                        self._importa_arquivo_kit(cr, uid, registros, dados_arquivo, cfg_loja,
                                                  cfg_empresa, context)
                        excluir = True

                    elif 'promocao.txt' in nome_arquivo:
                        self._importa_arquivo_promocao(cr, uid, import_file, registros, cfg_empresa,
                                                       dados_arquivo, context)
                        excluir = True

                    elif not exportou_funcionario and 'funcionario.txt' in nome_arquivo:
                        self._importa_arquivo_funcionario(cr, uid, registros, dados_arquivo,
                                                          context)
                        excluir = True
                        exportou_funcionario = True

                if excluir:
                    del_arq_importacao.append(path.join(diretorio_importacao, i))

        del_arq_importacao = list(set(del_arq_importacao))
        for i in del_arq_importacao:
            remove(i)

        if ids:
            self.write(cr, uid, ids[0], {'arquivos_importados': True}, context)

        if importar:
            cr.execute("""DELETE FROM supervisor_mov_acao_pendente WHERE supape_tabela = 'importando';
                                  UPDATE supervisor_mov_acao_pendente SET supape_situacao = 'A';""")

        return True

    def button_importa_arquivo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.funcao_improtar_arquivos(cr, uid, ids, context=context)
        return True

    def _importa_arquivo_promocao(self ,cr, uid, registros, lines, context=None):
        'Implementado por heranca no modulo supervisor_promocoes'
        pass

    def _importa_arquivo_funcionario(self ,cr, uid, registros, lines, context=None):
        'Implementado por heranca no modulo supervisor_funcionario'
        pass

    def _importa_arquivo_mensagem(self, cr, uid, instancia_arquivo, registros, lines, context=None):
        context = context and context.copy() or {}
        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        res = False
        msg_erro = ''
        try:
            cr.execute("""DELETE FROM supervisor_cad_mensagens_clientes""")
            obj_supervisor_cad_mensagens_clientes = self.pool.get('supervisor.cad.mensagens.clientes')
            qtd_grava_log = qtd_importada = 0
            frequencia = (len(lines)/50) or 1
            parcelas = []
            inserts = []
            nome = '%s -  Mensagens' % self.suputi_datetime_to_timezone(cr, uid, datetime.now())
            for line in lines:
                qtd_importada += 1
                qtd_grava_log += 1
                arquivo_importacao_data = {}
                arquivo_importacao_data['mensagem'] = registros.Mensagem()
                arquivo_importacao_data['mensagem_str'] = line.strip('\r\n').strip('\n')
                try:
                    mensagem = registros.registro_decompoe(arquivo_importacao_data['mensagem'],
                                                           arquivo_importacao_data['mensagem_str'])
                except Exception as e:
                    log_importacao(cr, uid, nome, qtd_importada, len(lines),
                                   'Erro de Layout linha %s: %s' % (qtd_importada, str(e)),
                                   context=context)
                    continue

                if qtd_grava_log / frequencia > 0: # frenquencia em que grava o log
                    qtd_grava_log = 0
                    log_importacao(cr, uid, nome, qtd_importada, len(lines), '', context=context)

                inserts.append("""({sequencia}, '{cnpj_cpf}', '{msg}', '{tipo}')""".
                                 format(sequencia=1,
                                        cnpj_cpf=mensagem.supclm_cnpj_cpf,
                                        msg=mensagem.supclm_mensagem,
                                        tipo=mensagem.supclm_tipo))
                if len(inserts) >= 10000:
                    parcelas.append(inserts)
                    inserts = []

            if inserts:
                parcelas.append(inserts)

            sql = """INSERT INTO supervisor_cad_mensagens_clientes 
                                 (supclm_sequencia, supclm_cnpj_cpf, supclm_mensagem, supclm_tipo) 
                     VALUES %s;
                     COMMIT;"""
            for parcela in parcelas:
                try:
                    cr.execute(sql % ',\n'.join(parcela))
                except Exception as e:
                    cr.execute('ROLLBACK;')
                    log_importacao(cr, uid, nome, qtd_importada, len(lines), 'Erro: %s' % str(e),
                                   context=context)

        except Exception as e:
            msg_erro = '%s' % e

        log_importacao(cr, uid, nome, qtd_importada, len(lines), msg_erro, finalizado=True,
                       context=context) # final dos registro que sobraram

        return res

    def _verifica_atributos_item(self, item, context=None):
        '''
        Verifica se os atributos do item existem no aquivo de importacao
        Retorna um dict com as chaces 'res' e 'msg'. Se 'res' = True nao contem erro no arquivo,
        se 'res' = False, algum atributo nao existe e 'msg' contem a mensagem de erro
        '''
        context = context and context.copy() or {}
        res = False
        msg = ''
        try:
            item.supite_fracionado
            item.supite_codigo_item
            item.supite_tributacao
            item.name
            item.supite_preco
            item.supite_unidade
            item.supite_peso_da_embalagem
            item.supite_codigo_retaguarda
            item.supite_fator_multiplicador
            item.supite_producao
            item.supite_ncm
            item.supite_cest
            item.supite_aliquota_ibpt_fed
            item.supite_aliquota_ibpt_est
            item.supite_aliquota_ibpt_mun
            item.supite_ibpt_chave
            item.active
            item.supite_peso_tara
            item.supite_codigo_anp
            item.supite_origem_mercadoria
            item.supite_getin
            item.codigo_icms
            item.codigo_pis
            item.codigo_cofins
            item.app_comanda
            item.adrem
            if item.adrem > 1000000:
                raise Exception('Valor do campo Aliquota ADREM invalido: %s' % (item.adrem))

            item.local_producao
            res = True
        except Exception as e:
            msg += str(e)

        return {'res':res, 'msg':msg}

    def set_listas_itens(self, cr, uid, list_vals_update, list_vals_create, vals, cfg_loja, cfg_empresa,
                   item, truncar_tabela, context=None):
        'O parametro "cfg_empresa" eh usado por heranca no "supervisor_master" '
        context = context and context.copy() or {}
        context['active_test'] = False
        args = [('supite_loja', '=', cfg_loja),
                ('supite_codigo_item', '=', item.supite_codigo_item)]
        item_id = self.pool.get('supervisor.cad.itens').search(cr, uid, args, context=context) \
                                                                    if not truncar_tabela else False
        if item_id:
            list_vals_update.append((item_id, vals))
        else:
            vals.update({'write_date': str(datetime.now())})
            list_vals_create.append(vals)

    def truncar_tabelas(self, cr, list_vals_update, list_vals_create, truncar_tabela, context=None):
        "os parametros 'list_vals_update' e 'list_vals_create' sao usados na heranca do metodo"
        context = context and context.copy() or {}
        if not truncar_tabela:
            return

        cr.execute("""DELETE FROM supervisor_cad_kits;
                      DELETE FROM supervisor_cad_itens;""")

    def _importa_arquivo_tributacao(self, cr, uid, registros, lines, cfg_loja, cfg_empresa,
                                    context=None):
        context = context and context.copy() or {}
        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        truncar_tabela = False
        if lines and lines[0][0] == '*':  # * no inicio do arquivo indica que é para truncar a tabela
            truncar_tabela = True
            lines = lines[1:]

        qtd_importada = 0
        qtd_grava_log = 0
        qtd_linha = 0
        nome = '%s -  Tributacao' % self.suputi_datetime_to_timezone(cr, uid, datetime.now())
        frequencia = (len(lines) / 50) or 1
        if truncar_tabela:
            cr.execute("""TRUNCATE supervisor_cad_tributacao_nfce CASCADE""")

        codigos_trib = []
        tam_lines = len(lines)
        msg_erro = ''
        try:
            for line in lines:
                qtd_grava_log += 1
                qtd_linha += 1
                arquivo_importacao_data = {}
                arquivo_importacao_data['tributacao'] = registros.Tributacao()
                arquivo_importacao_data['tributacao_str'] = line.strip('\r\n').strip('\n')
                try:
                    trib = registros.registro_decompoe(arquivo_importacao_data['tributacao'],
                                                       arquivo_importacao_data['tributacao_str'])
                except Exception as e:
                    log_importacao(cr, uid, nome, qtd_linha, tam_lines,
                                   'Erro de Layout linha %s: %s' % (qtd_linha, str(e)), context=context)
                    break

                if trib.codigo_trib > 2000000000:
                    trib.codigo_trib = int(str(trib.codigo_trib)[1:])

                codigos_trib.append(trib.codigo_trib)
                if qtd_grava_log / frequencia > 0:  # frenquencia em que grava o log
                    qtd_grava_log = 0
                    log_importacao(cr, uid, nome, qtd_linha, tam_lines, '', context=context)


                str_vals = """ {suptnf_codigo}, {suptnf_aliquota}, {suptnf_base}, '{suptnf_encargo}', 
                              '{suptnf_codigo_cst}', '{suptnf_codigo_csosn}', 
                              '{suptnf_tipo_tributacao}', {suptnf_aliquota_fcp},
                              '{suptnf_codigo_beneficio}', '{suptnf_motivo_beneficio}' """.\
                                             format(suptnf_codigo=trib.codigo_trib,
                                                    suptnf_aliquota=trib.aliquota/100.,
                                                    suptnf_base=trib.base/100.,
                                                    suptnf_encargo=trib.encargo,
                                                    suptnf_codigo_cst=trib.codigo_cst,
                                                    suptnf_codigo_csosn=trib.csosn,
                                                    suptnf_tipo_tributacao=trib.tipo_tributacao,
                                                    suptnf_aliquota_fcp=trib.aliquota_fcp/100.,
                                                    suptnf_codigo_beneficio=trib.codigo_beneficio,
                                                    suptnf_motivo_beneficio=trib.motivo_beneficio)
                sql = """INSERT INTO supervisor_cad_tributacao_nfce
                                               (suptnf_codigo, suptnf_aliquota, suptnf_base, suptnf_encargo, 
                                                suptnf_codigo_cst, suptnf_codigo_csosn, suptnf_tipo_tributacao,
                                                suptnf_aliquota_fcp, suptnf_codigo_beneficio, 
                                                suptnf_motivo_beneficio)
                         VALUES (%s);
                         COMMIT;""" % (str_vals.replace(" u'", " '"))
                try:
                    if self.pool.get('supervisor.cad.tributacao.nfce').search(cr, uid,
                                                      [('suptnf_codigo', '=', trib.codigo_trib)]):
                        continue

                    cr.execute(sql)
                    qtd_importada += 1
                except Exception as e:
                    cr.execute('ROLLBACK;')
                    log_importacao(cr, uid, nome, qtd_linha, tam_lines, 'Erro linha %s: %s' % (qtd_linha, str(e)), context=context)

        except Exception as e:
            msg_erro = '%s' % e

        log_importacao(cr, uid, nome, qtd_importada, qtd_importada, msg_erro, finalizado=True,
                       context=context)
        return True

    def _importa_arquivo_itens(self, cr, uid, registros, lines, cfg_loja, cfg_empresa,
                               context=None):
        context = context and context.copy() or {}
        res = False
        obj_supervisor_cad_itens = self.pool.get('supervisor.cad.itens')
        obj_supervisor_cad_tributacao = self.pool.get('supervisor.cad.tributacao')
        obj_supervisor_util = self.pool.get('supervisor.util')
        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        truncar_tabela = False
        if lines and lines[0][0] == '*': # * no inicio do arquivo indica que é para truncar a tabela
            truncar_tabela = True
            lines = lines[1:]

        qtd_importada = 0
        qtd_grava_log = 0
        nome = '%s -  Itens' % self.suputi_datetime_to_timezone(cr, uid, datetime.now())
        frequencia = (len(lines)/50) or 1
        dict_tribs = {}
        cr.execute("""SELECT suptnf_codigo, suptnf_tipo_tributacao
                              FROM supervisor_cad_tributacao_nfce""")
        map(lambda x: dict_tribs.update({x.get('suptnf_codigo'): x.get('suptnf_tipo_tributacao')}),
            cr.dictfetchall())
        list_vals_update = []
        list_vals_create = []
        for line in lines:
            erro = False
            msg_erro = ''
            qtd_importada += 1
            qtd_grava_log += 1
            arquivo_importacao_data = {}
            arquivo_importacao_data['itens'] = registros.Itens()
            arquivo_importacao_data['itens_str'] = line.strip('\r\n').strip('\n')
            try:
                item = registros.registro_decompoe(arquivo_importacao_data['itens'],
                                                   arquivo_importacao_data['itens_str'])
            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines),
                               'Erro de layout linha %s: %s' % (qtd_importada, str(e)), context=context)
                continue

            verifica_item = self._verifica_atributos_item(item, context=context)
            if not verifica_item.get('res'):
                log_importacao(cr, uid, nome, qtd_importada, len(lines),
                               'Erro linha %s: %s' % (qtd_importada, verifica_item.get('msg')))
                continue

            fracionado = True if item.supite_fracionado == 'S' else False
            pesavel = True if item.supite_pesavel == 'S' else False
            tributacao = item.supite_tributacao
            supite_tributacao_id = obj_supervisor_cad_tributacao.search(
                cr, uid, [('suptri_codigo_trib_retag', '=', (int(tributacao)))], context=context)

            if not supite_tributacao_id:
                msg_erro = 'A tributacao %s do item %s nao esta registrada no cadastro de ' \
                           'tributacao' % (tributacao, item.supite_codigo_item)
                erro = True

            try:
                nome_item = obj_supervisor_util.suputi_remover_acentuacao(item.name)
                desc_completa = obj_supervisor_util.suputi_remover_acentuacao(
                                                        item.supite_descricao_completa)
            except:
                try:
                    nome_item = obj_supervisor_util.suputi_remove_caracter(item.name)
                    desc_completa = obj_supervisor_util.suputi_remove_caracter(
                                                            item.supite_descricao_completa)
                    nome_item = obj_supervisor_util.suputi_remover_acentuacao(nome_item)
                    desc_completa = obj_supervisor_util.suputi_remover_acentuacao(desc_completa)
                except Exception as e:
                    msg_erro = "Erro ao importar o item: %s Erro: %s" % (item.supite_codigo_item,
                                                                         str(e))
                    erro = True

            if qtd_grava_log / frequencia > 0 or erro: # frenquencia em que grava o log
                qtd_grava_log = 0
                log_importacao(cr, uid, nome, qtd_importada, len(lines), msg_erro, context=context)

            if erro:
                continue

            if not desc_completa:
                desc_completa = nome_item

            cb = item.supite_codigo_item
            icms = item.codigo_icms
            pis = item.codigo_pis
            cofins = item.codigo_cofins
            tipo_trib = dict_tribs.get(icms)
            msg = ''
            if not icms:
                msg += 'Item %s - %s: Sem ICMS cadastrado\n' % (cb, nome_item)
            if not pis:
                if not msg:
                    msg += 'Item %s - %s: Sem PIS cadastrado\n' % (cb, nome_item)
                else:
                    msg += '              Sem PIS cadastrado\n'
            if not cofins:
                if not msg:
                    msg += 'Item %s - %s: Sem COFINS cadastrado\n' % (cb, nome_item)
                else:
                    msg += '              Sem COFINS cadastrado\n'
            if not item.supite_ncm:
                if not msg:
                    msg += 'Item %s - %s: Sem NCM cadastrado\n' % (cb, nome_item)
                else:
                    msg += '              Sem NCM cadastrado\n'
            if tipo_trib in ('U', 'TU'):
                if not item.supite_cest: # qualquer tipo de substituicao
                    if not msg:
                        msg += 'Item %s - %s: Sem CEST cadastrado\n' % (cb, nome_item)
                    else:
                        msg += '              Sem CEST cadastrado\n'
                elif len(item.supite_cest.strip()) < 7:
                    if not msg:
                        msg += 'Item %s - %s: CEST invalido, menos de 7 caracters\n' % (cb, nome_item)
                    else:
                        msg += '              CEST invalido, menos de 7 caracters\n'

            if msg:
                log_importacao(cr, uid, nome, qtd_importada, len(lines), 'Erro linha %s: %s' % (qtd_importada ,msg), context=context)
                continue

            vals = {
                'supite_loja': cfg_loja,
                'supite_codigo_item': cb,
                'name': nome_item,
                'supite_descricao_completa': desc_completa,
                'supite_tributacao_id': supite_tributacao_id[0],
                'supite_pesavel': pesavel,
                'supite_fracionado': fracionado,
                'supite_preco': float(item.supite_preco) / 100.0,
                'supite_unidade': item.supite_unidade,
                'supite_peso_da_embalagem': item.supite_peso_da_embalagem / 1000,
                'supite_codigo_retaguarda': item.supite_codigo_retaguarda,
                'supite_fator_multiplicador': float(item.supite_fator_multiplicador)/1000.0,
                'supite_producao': item.supite_producao,
                'supite_iat': 'A',
                'supite_ncm': item.supite_ncm,
                'supite_cest': item.supite_cest,
                'supite_aliquota_ibpt_fed': item.supite_aliquota_ibpt_fed / 100.0,
                'supite_aliquota_ibpt_est': item.supite_aliquota_ibpt_est / 100.0,
                'supite_aliquota_ibpt_mun': item.supite_aliquota_ibpt_mun / 100.0,
                'supite_ibpt_chave': item.supite_ibpt_chave,
                'active': True if item.active == 'A' else False,
                'supite_peso_tara': float(item.supite_peso_tara) / 1000.0,
                'supite_tributacao_pis_id': pis,
                'supite_tributacao_cofins_id': cofins,
                'supite_tributacao_icms_id': icms,
                'supite_codigo_anp': int(item.supite_codigo_anp or 0),
                'supite_adrem': float(item.adrem or 0) / 10000.0,
                'supite_origem_mercadoria': item.supite_origem_mercadoria or '0',
                'supite_getin': item.supite_getin and True or False,
                'supite_med_codigo_anvisa': item.med_codigo_anvisa or '',
                'supite_med_preco_maximo': item.med_preco_maximo or 0,
                'supite_med_motivo_isencao': item.med_motivo_isencao or ''
            }
            if self.check_modulo_instalado(cr, uid, 'supervisor_producao_comanda', context):
                vals['supite_app_comanda'] = item.app_comanda and True or False

            if self.check_modulo_instalado(cr, uid, 'supervisor_producao', context):
                vals['supite_codigo_local'] = item.local_producao or 0

            self.set_listas_itens(cr, uid, list_vals_update, list_vals_create, vals, cfg_loja,
                            cfg_empresa, item, truncar_tabela, context)

        self.truncar_tabelas(cr, list_vals_update, list_vals_create, truncar_tabela, context)
        if list_vals_create:
            campos = ', '.join(list_vals_create[0].keys())
            list_vals = map(lambda x: '%s' % (tuple(x.values()),), list_vals_create)
            for i in range(0, 20):
                if not list_vals[i*10000:(i+1)*10000]:
                    break

                str_vals = ',\n'.join(list_vals[i*10000:(i+1)*10000])
                sql = """INSERT INTO supervisor_cad_itens
                               (%s)
                         VALUES %s;
                         COMMIT; 
                      """ % (campos, str_vals.replace(" u'", " '"))
                try:
                    cr.execute(sql)
                except Exception as e:
                    cr.execute('ROLLBACK;')
                    log_importacao(cr, uid, nome, qtd_importada, len(lines), 'Erro linha %s: %s' % (qtd_importada ,str(e)), context=context)

        for i in list_vals_update:
            obj_supervisor_cad_itens.write(cr, uid, i[0], i[1], context=context)


        log_importacao(cr, uid, nome, qtd_importada, len(lines), '', finalizado=True,
                       context=context)
        return res

    def _importa_arquivo_vendedor(self, cr, uid, instancia_arquivo, registros, lines, context=None):
        context = context and context.copy() or {}
        res = False
        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        obj_supervisor_cad_vendedor = self.pool.get('supervisor.cad.vendedor')
        nome = '%s -  Vendedores' % self.suputi_datetime_to_timezone(cr, uid, datetime.now())
        qtd_importada = qtd_grava_log = 0
        frequencia = (len(lines)/50) or 1
        for line in lines:
            qtd_importada += 1
            qtd_grava_log += 1
            arquivo_importacao_data = {}
            arquivo_importacao_data['vendedor'] = registros.Vendedor()
            arquivo_importacao_data['vendedor_str'] = line.strip('\r\n').strip('\n')
            vendedor = registros.registro_decompoe(arquivo_importacao_data['vendedor'],
                                                   arquivo_importacao_data['vendedor_str'])
            vendedor_id = obj_supervisor_cad_vendedor.search(cr, uid, [('supven_codigo', '=', vendedor.supven_codigo)],
                                                             context=context)

            if qtd_grava_log / frequencia > 0: # frenquencia em que grava o log
                qtd_grava_log = 0
                log_importacao(cr, uid, nome, qtd_importada, len(lines), '', context=context)

            if not vendedor_id:
                vals = {
                    'supven_codigo': vendedor.supven_codigo,
                    'name': vendedor.name,
                }
                try:
                    res = obj_supervisor_cad_vendedor.create(cr, uid, vals, context)
                    cr.execute('COMMIT;')
                except Exception as e:
                    try:
                        cr.execute('ROLLBACK;')
                    except:
                        pass
                    log_importacao(cr, uid, nome, qtd_importada, len(lines),
                                   'Erro linha %s: %s' % (qtd_importada, str(e)), context=context)


        log_importacao(cr, uid, nome, qtd_importada, len(lines), '', finalizado=True, context=context)
        return res

    def _importa_arquivo_clientes(self, cr, uid, instancia_arquivo, registros, lines, context=None):
        context = context and context.copy() or {}
        res = False
        obj_supervisor_util = self.pool.get('supervisor.util')
        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        nome = '%s -  Clientes' % self.suputi_datetime_to_timezone(cr, uid, datetime.now())
        qtd_importada = qtd_grava_log = 0
        frequencia = (len(lines)/50) or 1
        cr.execute('TRUNCATE supervisor_cad_clientes')
        insert_cliente = """
        INSERT INTO supervisor_cad_clientes
            (supcli_cnpj_cpf, name, supcli_status, supcli_endereco,
            supcli_limite_compras, supcli_total_em_aberto,
            supcli_codigo_interno, supcli_insc_estadual, supcli_telefone, supcli_email,
            supcli_bairro, supcli_cidade, supcli_estado, supcli_cep, supcli_tipo_pessoa,
            supcli_cod_municipio, supcli_endereco_numero,
            create_date, create_uid, write_date, write_uid)
        VALUES
        """
        parcelas_insert = []
        inserts = []
        for line in lines:
            qtd_importada += 1
            qtd_grava_log += 1
            arquivo_importacao_data = {}
            arquivo_importacao_data['clientes'] = registros.Clientes()
            arquivo_importacao_data['clientes_str'] = line.strip('\r\n').strip('\n')
            try:
                cliente = registros.registro_decompoe(arquivo_importacao_data['clientes'],
                                                      arquivo_importacao_data['clientes_str'])
            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines), 'Erro de layout linha %s: %s' %
                               (qtd_importada, str(e)), context=context)
                continue

            if qtd_grava_log / frequencia > 0: # frenquencia em que grava o log
                qtd_grava_log = 0
                log_importacao(cr, uid, nome, qtd_importada, len(lines), '', context=context)

            try:
                cliente_nome = obj_supervisor_util.suputi_remover_acentuacao(cliente.name)
                cliente_nome = obj_supervisor_util.suputi_remove_caracter(cliente_nome)
                cliente_endereco = obj_supervisor_util.suputi_remover_acentuacao(cliente.supcli_endereco)
                cliente_endereco = obj_supervisor_util.suputi_remove_caracter(cliente_endereco)
                cidade = obj_supervisor_util.suputi_remove_caracter(obj_supervisor_util.suputi_remover_acentuacao(cliente.supcli_cidade))
                bairro = obj_supervisor_util.suputi_remove_caracter(obj_supervisor_util.suputi_remover_acentuacao(cliente.supcli_bairro))
            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines),
                               'Erro na linha %s: %s' % (qtd_importada, str(e)), context=context)
                continue

            create_date = write_date = datetime.now()
            create_uid = uid
            write_uid = uid
            email = obj_supervisor_util.suputi_remover_acentuacao(cliente.supcli_email)
            sql = """
            ('{cpf_cnpj}', '{nome}', '{supcli_status}', '{endereco}', {limite_compras},
              {total_em_aberto}, {codigo_interno}, '{ie}', '{tel}', '{email}',
              '{bairro}', '{cidade}', '{estado}', '{cep}', '{tipo_pessoa}',
              '{cod_municipio}', '{numero}',
               '{create_date}', {create_uid}, '{write_date}', {write_uid})""".format(
                cpf_cnpj=cliente.supcli_cnpj_cpf, nome=cliente_nome, supcli_status=cliente.state,
                endereco=cliente_endereco, codigo_interno=cliente.supcli_codigo_interno,
                limite_compras=float(cliente.supcli_limite_compras) / 100,
                ie=cliente.supcli_insc_estadual, tel=cliente.supcli_telefone,
                email=email, total_em_aberto=0, create_date=create_date,
                bairro=bairro,
                cidade=cidade,
                estado=cliente.supcli_estado,
                cep=cliente.supcli_cep,
                tipo_pessoa=cliente.supcli_tipo_pessoa,
                cod_municipio=cliente.supcli_cod_municipio,
                numero=cliente.supcli_endereco_numero.replace(' ', ''),
                write_date=write_date, create_uid=create_uid, write_uid=write_uid
            )
            sql = sql.replace("'null'", 'null')
            inserts.append(sql)
            # divide o insert em blocos de 10.000 para nao sobrecarregar o postgres
            if len(inserts) >= 10000:
                parcelas_insert.append(inserts)
                inserts = []

        if inserts:
            parcelas_insert.append(inserts)

        for i in parcelas_insert:
            try:
                cr.execute(insert_cliente + ','.join(i) + '; COMMIT;')
            except Exception as e:
                cr.execute('ROLLBACK;')
                log_importacao(cr, uid, nome, qtd_importada, len(lines), 'Erro linha %s: %s' % (qtd_importada, str(e)), context=context)

        log_importacao(cr, uid, nome, qtd_importada, len(lines), '', finalizado=True, context=context)
        return res

    def set_listas_kits(self, cr, uid, vals, cfg_loja, cfg_empresa, kit, context=None):
        'Implementado por heranca no modulo supervisor_master'
        pass

    def ajusta_valor_total_itens(self, list_vals, dict_kits, context=None):
        context = context and context.copy() or {}
        if not dict_kits:
            map(lambda x: x.pop('cont'), list_vals)
            return

        totais = {}
        for i in dict_kits:
            valor_total = dict_kits.get(i).get('valor')
            itens = dict_kits.get(i).get('itens')
            self.pool.get('supervisor.util').efetuar_rateio(valor_total, elementos=itens)
            for j in itens:
                totais[j[0]] = j[1]

        for i in list_vals:
            total = totais.get(i.pop('cont'))
            if total:
                i['supkit_preco_venda'] = total

    def _importa_arquivo_kit(self, cr, uid, registros, lines, cfg_loja, cfg_empresa, context=None):
        context = context and context.copy() or {}
        res = False
        obj_supervisor_cad_kits = self.pool.get('supervisor.cad.kits')
        obj_supervisor_cad_itens = self.pool.get('supervisor.cad.itens')
        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        list_codigo_kit = [] # lista dos kits que ja foram deletados
        nome = '%s -  Kits' % self.suputi_datetime_to_timezone(cr, uid, datetime.now())
        qtd_importada = qtd_grava_log = 0
        frequencia = (len(lines)/50) or 1
        dict_kits = {}
        list_vals = []
        cont = 1
        for line in lines:
            erro = False
            msg_erro = ''
            qtd_importada += 1
            arquivo_importacao_data = {}
            arquivo_importacao_data['kit'] = registros.Kit()
            arquivo_importacao_data['kit_str'] = line.strip('\r\n').strip('\n')
            try:
                kit = registros.registro_decompoe(arquivo_importacao_data['kit'],
                                                  arquivo_importacao_data['kit_str'])
            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines),
                               'Erro de Layout linha %s: %s' % (qtd_importada, str(e)),
                               context=context)
                continue

            codigo_kit_id = obj_supervisor_cad_itens.search(
                cr, uid, [('supite_codigo_item', '=', kit.supkit_codigo_barra_kit)], context=context)
            if not codigo_kit_id:
                msg_erro = 'O kit %s nao esta registrado no cadastro de itens' % \
                           kit.supkit_codigo_barra_kit
                log_importacao(cr, uid, nome, qtd_importada, len(lines), msg_erro, context=context)
                continue

            codigo_kit_id = codigo_kit_id[0]

            codigo_item_kit_id = obj_supervisor_cad_itens.search(
                cr, uid, [('supite_codigo_item', '=', kit.supkit_codigo_barra_item)],
                context=context)
            if not codigo_item_kit_id:
                msg_erro = 'O item %s do kit %s nao esta registrado no cadastro de itens' % \
                           (kit.supkit_codigo_barra_item, kit.supkit_codigo_barra_kit)
                erro = True

            if codigo_item_kit_id and isinstance(codigo_item_kit_id, list):
                codigo_item_kit_id = codigo_item_kit_id[0]

            chave = str(cfg_loja) + str(kit.supkit_codigo_barra_kit)
            if not chave in list_codigo_kit:
                kit_ids = obj_supervisor_cad_kits.search(
                    cr, uid, [('supkit_loja', '=', cfg_loja),
                              ('supkit_codigo_barra_kit', '=', kit.supkit_codigo_barra_kit)],
                    context=context)
                obj_supervisor_cad_kits.unlink(cr, uid, kit_ids, context=context)
                list_codigo_kit.append(chave)

            if qtd_grava_log / frequencia > 0 or erro: # frenquencia em que grava o log
                qtd_grava_log = 0
                log_importacao(cr, uid, nome, qtd_importada, len(lines), msg_erro, context=context)

            if erro:
                continue

            vr_total_item = kit.supkit_preco_venda/100.0
            qtd_item = kit.supkit_quantidade / 1000.0
            # kits que nao sao gerados pelo ERP nao possuem valor total
            if not vr_total_item:
                if not dict_kits.get(kit.supkit_codigo_barra_kit):
                    dict_kits[kit.supkit_codigo_barra_kit] = {'itens': []}

                if not dict_kits.get(kit.supkit_codigo_barra_kit).get('valor'):
                    valor_kit = obj_supervisor_cad_itens.read(cr, uid, codigo_kit_id,
                                          ['supite_preco'], context=context).get('supite_preco')
                    dict_kits[kit.supkit_codigo_barra_kit]['valor'] = valor_kit

                valor_item = obj_supervisor_cad_itens.read(cr, uid, codigo_item_kit_id,
                                          ['supite_preco'], context=context).get('supite_preco')
                dict_kits[kit.supkit_codigo_barra_kit]['itens'].\
                    append([cont , valor_item * qtd_item])

            vals = {
                'cont': cont,
                'supkit_loja': cfg_loja,
                'supkit_codigo_kit': codigo_kit_id,
                'supkit_codigo_item': codigo_item_kit_id,
                'supkit_codigo_barra_kit': kit.supkit_codigo_barra_kit,
                'supkit_codigo_barra_item': kit.supkit_codigo_barra_item,
                'supkit_quantidade': qtd_item,
                'supkit_preco_venda': vr_total_item,
            }
            cont += 1
            self.set_listas_kits(cr, uid, vals, cfg_loja, cfg_empresa, kit, context)
            list_vals.append(vals)

        self.ajusta_valor_total_itens(list_vals, dict_kits, context=context)
        for vals in list_vals:
            try:
                res = obj_supervisor_cad_kits.create(cr, uid, vals, context=context)
                cr.execute('COMMIT;')
            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines), 'Erro: %s' % e,
                               context=context)

        log_importacao(cr, uid, nome, qtd_importada, len(lines), '', finalizado=True, context=context)
        return res
