# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields


class wizard_supervisor_reexporta_cupom(osv.TransientModel):
    # prefixo: wsurec_
    _name = 'wizard.supervisor.reexporta.cupom'
    _columns = {
        'wsurec_loja': fields.integer('Loja'),
        'wsurec_caixa': fields.integer('Caixa'),
        'wsurec_data_cupom': fields.date('Data cupom'),
        'wsurec_nro_cupom': fields.integer('Numero cupom'),
        'wsurec_item_ids': fields.one2many('wizard.supervisor.reexporta.cupom.item',
                                           'wsurei_reexporta_cupom_id', 'Itens'),
    }

    def wsurec_button_pesquisa(self, cr, uid, ids, context=None):
        """
        O metodo realiza a busca dos cupons que se enquadram de acordo com a data, loja, caixa,
        e numero do cupom informados.
        :param cr: cursor no banco de dados
        :param uid: identificador do usuario, id da tabela res_users
        :param ids: lista de registros da tabela dessa classe
        :param context: dicionario com valores contextuais
        :return: booleano, com True caso encontre algum cupom e False caso nao
        """
        context = context and context.copy() or {}
        obj_reexporta_cupom_item = self.pool.get('wizard.supervisor.reexporta.cupom.item')
        res = False
        brw_reexporta_cupom = self.browse(cr, uid, ids, context)
        cr.execute('DELETE FROM wizard_supervisor_reexporta_cupom_item')
        filtro_informado = False
        for x in brw_reexporta_cupom:
            reexporta_cupom_id = x.id
            loja = x.wsurec_loja
            caixa = x.wsurec_caixa
            data_cupom = x.wsurec_data_cupom
            nro_cupom = x.wsurec_nro_cupom
            sql = """
                SELECT
                    supcap_data_cupom    AS wsurei_data,
                    supcap_loja          AS wsurei_loja,
                    supcap_caixa         AS wsurei_caixa,
                    supcap_nro_cupom     AS wsurei_cupom,
                    supcap_total_cupom   AS wsurei_valor,
                    supcse_enviado       AS wsurei_enviado,
                    sms.id               AS wsurei_cupom_servidor
                FROM
                    supervisor_mov_capa smc
                INNER JOIN supervisor_mov_cupom_servidor sms
                    ON smc.id = sms.supcse_capa_id
            """
            if any([loja, caixa, data_cupom, nro_cupom]):
                filtro_informado = True
                sql += """
                WHERE
                    1 = 1
                """
            if loja:
                sql += """
                    AND supcap_loja = %s
                """ % loja
            if caixa:
                sql += """
                    AND supcap_caixa = %s
                """ % caixa
            if data_cupom:
                sql += """
                    AND supcap_data_cupom = '%s'
                """ % data_cupom
            if nro_cupom:
                sql += """
                    AND supcap_nro_cupom = %s
                """ % nro_cupom
            if not filtro_informado:
                sql += """
                LIMIT 10000
                """
            cr.execute(sql)
            dados = cr.dictfetchall()
            if dados:
                res = True
            for dado in dados:
                dado.update({'wsurei_reexporta_cupom_id': reexporta_cupom_id})
                obj_reexporta_cupom_item.create(cr, uid, dado, context)
        return res

    def wsurec_button_marca_nao_enviado(self, cr, uid, ids, context=None):
        """
        O metodo seta para False a coluna 'supcse_enviado' da tabela supevisor_mov_cupom_servidor,
        para os cupons previamente selecionados. Isso fara com que o ERP importe novamente esses
        cupons
        :param cr: cursor no banco de dados
        :param uid: identificador do usuario, id da tabela res_users
        :param ids: lista de registros da tabela dessa classe
        :param context: dicionario com valores contextuais
        :return: booleano, com True caso atualize algum cupom para reexportar e False caso nao
        """
        context = context and context.copy() or {}
        obj_supervisor_mov_cupom_servidor = self.pool.get('supervisor.mov.cupom.servidor')
        res = False
        brw_reexporta_cupom = self.browse(cr, uid, ids, context)
        for x in brw_reexporta_cupom:
            res = False
            mov_cupom_servidor_ids = map(lambda i: i.wsurei_cupom_servidor, x.wsurec_item_ids)
            if mov_cupom_servidor_ids:
                res = True
            obj_supervisor_mov_cupom_servidor.write(cr, uid, mov_cupom_servidor_ids,
                                                    {'supcse_enviado': False}, context)
        return res


class wizard_supervisor_reexporta_cupom_item(osv.TransientModel):
    # prefixo: wsurei_
    _name = 'wizard.supervisor.reexporta.cupom.item'
    _columns = {
        'wsurei_reexporta_cupom_id': fields.many2one('wizard.supervisor.reexporta.cupom', 'Capa'),
        'wsurei_data': fields.date('Data'),
        'wsurei_loja': fields.integer('Loja'),
        'wsurei_caixa': fields.integer('Caixa'),
        'wsurei_cupom': fields.integer('Cupom'),
        'wsurei_valor': fields.float('Valor', digits=(10, 2)),
        'wsurei_enviado': fields.boolean('Enviado'),
        'wsurei_cupom_servidor': fields.integer('Cupom servidor'),
    }
