# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report.render import render
from openerp.osv import osv, fields
from openerp.report.interface import report_int
import StringIO
import locale
from SigUtils.impressao import Impressora
# noinspection PyProtectedMember
from openerp.tools.translate import _
import time
from SigUtils.numero import getNumeroEditado
from openerp import pooler


# noinspection PyPep8Naming
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming,PyAbstractClass
class wizard_listagem_comandas(osv.TransientModel):
    """
    Gera os campos para filtro
    """

    _name = 'wizard.listagem.comandas'
    _columns = {
        'wiz_data_ini': fields.date('Data inicial', required=True),
        'wiz_data_fim': fields.date('Data final', required=True),
        'wiz_status': fields.selection([(-1, 'Cancelada'), (0, 'Aberta'), (2, 'Fechada'),
                                        (3, 'Todas')], 'Restricao'),
    }

    _defaults = {
        'wiz_status': 3,
    }

    def button_imprimir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        listagem_comandas = self.read(cr, uid, ids, [], context=context)

        if isinstance(listagem_comandas, (list, tuple)):
            listagem_comandas = listagem_comandas[0]
        datas = {
            'data_ini': listagem_comandas.get('wiz_data_ini'),
            'data_fim': listagem_comandas.get('wiz_data_fim'),
            'status': listagem_comandas.get('wiz_status')
        }

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'wizard.listagem.comandas',
            'datas': datas,
        }


# noinspection PyPep8Naming
class report_relatorio_listagem_comandas(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        # Dados informados
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')
        status = datas.get('status')

        # Cria relatorio de imprensao
        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Listagem de comandas'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)

        # Cabecalho do relatorio
        try:
            company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
                cr, uid, 'wizard.listagem.comandas', context=context)
            empresa_nome = pooler.get_pool(cr.dbname).get('res.company').browse(
                cr, uid, company_id).name
        except AttributeError:
            empresa_nome = None

        tam_linha = self.get_tam_linha(listagem)
        data_relatorio = time.strftime('%d/%m/%Y')
        det_format = self.get_det_format()
        listagem.informaCab('{:^150}'.decode('utf-8').format(empresa_nome))
        listagem.informaCab('')
        listagem.informaCab(' {:<10} {:^126} {:>10} '.decode('utf-8').format(
            data_relatorio, _('Listagem de comandas')[:126], _('Pagina: ')[:10] + '{{ Page }}'))
        listagem.informaCab('*' + '-' * tam_linha + '*')
        listagem.informaCab(eval("det_format.format('%s')" % "','".join(self.get_colunas()) ) )
        listagem.informaCab('*' + '-' * tam_linha + '*')

        # Rodape
        listagem.rodapePadrao()

        # Dados seleciondos do banco
        dados = self._gera_consulta(cr, data_ini, data_fim, status, datas, context)
        qt_comanda = 0
        valor_total = 0.0

        for i in dados:
            if i.get('corpo_venda_cancelamento') != 'C':
                nro_comanda = str(i.get('nro_comanda')).zfill(8)
                status = i.get('status')
                data_abertura = i.get('data_abertura_comanda')
                data_abertura = data_abertura.strftime('%d/%m/%Y %H:%M') if data_abertura else ''
                caixa = str(i.get('nro_caixa')).zfill(4) if i.get('nro_caixa') else ''
                cupom = '%s' % (i.get('nro_cupom') or '')
                data_fim_cupom = i.get('data_fim_cupom')
                data_fim_cupom = data_fim_cupom.strftime('%d/%m%Y %H:%M') if data_fim_cupom else ''
                total_comanda = locale.format('%.2f', i.get('total_comanda') or 0.0, grouping=True)
                qt_itens = locale.format('%.0f', i.get('quantidade_itens') or 0)
                val_conlunas = (nro_comanda, status, data_abertura, caixa, cupom, data_fim_cupom,
                                total_comanda, qt_itens)
                val_conlunas = self.get_val_empresa(val_conlunas, i)

                # Imprime a linha detalhe
                listagem.imprimeLinha(eval("det_format.format('%s')" % "','".join(val_conlunas)))

                # Calcula o total dos itens e valor total
                qt_comanda += 1 if status is not 'Cancelada' else 0
                valor_total += i.get('total_comanda') if status is not 'Cancelado' else 0.0

        media = valor_total / qt_comanda if qt_comanda else 0.0

        # Resumo
        valor_total = locale.format('%.2f', valor_total, grouping=True)
        media = locale.format('%.2f', media, grouping=True)
        listagem.imprimeLinha('*' + '-' * tam_linha + '*')
        listagem.imprimeLinha('')
        listagem.imprimeLinha(' Quantidade de comandas: {:>8}'.decode('utf-8').format(qt_comanda))
        listagem.imprimeLinha(' Valor Total...........: {:>8}'.decode('utf-8').format(valor_total))
        listagem.imprimeLinha(' Media.................: {:>8}'.decode('utf-8').format(media))

        # Fianaliza a impressao
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    def get_tam_linha(self, listagem):
        'listagem eh usado na heranca em supervisor_master_comanda'
        return 148

    def get_det_format(self):
        return '|{:^19}|{:^12}|{:^34}|{:^12}|{:^11}|{:^23}|{:>15}|{:>15}|'.decode('utf-8')

    def get_colunas(self):
        return _('Numero da comanda'), _('Status'), _('Data de abertura da comanda'), _('Caixa'),\
               _('Cupom'), _('Data fim do cupom'), _('Valor'), _('Qtde. de itens')

    def get_val_empresa(self, val_conlunas, dados):
        'Dados eh utilizado por heranca no modulo supervisor_master_comanda'
        return val_conlunas

    def get_cond(self, cr, datas):
        'Implementado por heranca no modulo supervisor_master_comanda'
        return ''

    def get_empresa(self):
        'Implementado por heranca no modulo supervisor_master_comanda'
        return ''

    def get_group_empresa(self):
        'Implementado por heranca no modulo supervisor_master_comanda'
        return ''

    def _gera_consulta(self, cr, data_ini, data_fim, status, datas, context=None):
        """
        Metodo para realizar a consulta no banco de dados
        recebendo como parametro o periodo e o status a filtrar
        """
        context = context and context.copy() or {}

        if status != 3:
            filtro = '  AND  capa.supcda_restricao = %d' % status
        else:
            filtro = ' --'

        cond = self.get_cond(cr, datas)
        empresa = self.get_empresa()
        group_empresa = self.get_group_empresa()
        _sql = """
            -- noinspection SqlDialectInspection
            SELECT capa.supcda_codigo_comanda                AS nro_comanda,
                   capa.supcda_data_hora AT TIME ZONE 'UTC'  AS data_abertura_comanda,
                   capa.supcda_caixa                         AS nro_caixa,
                   capa.supcda_nro_cupom                     AS nro_cupom,
                   capa.supcda_data_cupom                    AS data_fim_cupom,
                   capa.supcda_total_comanda                 AS total_comanda,
                   SUM(corpo.supcdo_quantidade)              AS quantidade_itens,
                   corpo.supcdo_venda_cancelamento           AS corpo_venda_cancelamento,
                   %(empresa)s
                   CASE WHEN supcda_restricao = -1 
                        THEN 'Cancelada'
                        WHEN supcda_restricao =  0 
                        THEN 'Aberta'
                        ELSE 'Fechada' 
                   END                                       AS status
            FROM   supervisor_cda_capa AS capa
            LEFT   JOIN supervisor_cda_corpo AS corpo
                   ON capa.id = corpo.supcdo_capa_id
            WHERE  capa.supcda_data_hora BETWEEN '%(dt_inicio)s 00:00:00' AND '%(dt_fim)s 23:59:59'
            %(filtro_status)s
            %(cond)s
            GROUP BY    corpo.supcdo_capa_id,
                        capa.id,
                        corpo.supcdo_venda_cancelamento
                        %(group_empresa)s
            ORDER BY capa.supcda_data_hora ASC %(group_empresa)s
        """ % dict(empresa=empresa, dt_inicio=data_ini, dt_fim=data_fim, filtro_status=filtro,
                   cond=cond,
                   group_empresa=group_empresa)
        cr.execute(_sql)
        reg = cr.dictfetchall()

        return reg

report_relatorio_listagem_comandas('report.wizard.listagem.comandas')
