# -*- coding: utf-8 -*-
from pytz import timezone
from SigUtils.impressao import Impressora
from openerp.osv import osv, fields
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
import time
from openerp import pooler
# noinspection PyProtectedMember
from openerp.tools.translate import _
from datetime import datetime
import locale


# noinspection PyPep8Naming
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        data_titulo = time.strftime('%d/%m/%Y')
        tz = context.get('tz') or 'Brazil/East'
        hora_titulo = datetime.now(timezone('UTC')).astimezone(timezone(tz)).strftime('%H:%M:%S')
        obj_supervisor_cfg_configuracao = pooler.get_pool(cr.dbname).get(
            'supervisor.cfg.configuracao')
        configuracao_ids = obj_supervisor_cfg_configuracao.search(cr, uid, [], context=context)

        if configuracao_ids:
            nome_empresa = obj_supervisor_cfg_configuracao.browse(
                cr, uid, configuracao_ids[0], context=context).supcfc_cliente or ''
        else:
            nome_empresa = ''

        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Flash de Comandas'))
        listagem.informaTamLinha(160)
        listagem.defineFonte(tam=8.6)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        # Cabecalho do relatorio
        data = datas.get('data')
        data_inicial_cabecalho = time.strftime("%d/%m/%Y", time.strptime(data, "%Y-%m-%d"))
        data_final_cabecalho = time.strftime("%d/%m/%Y", time.strptime(data, "%Y-%m-%d"))
        det_format = '|{:^19}|{:>18}|{:^14}|{:>13}|{:>18}|{:>20}|'.decode('utf-8')
        listagem.informaCab(' {:<15} {:^80} {:>10} '.decode('utf-8').format(
            _('Sistema de PDVs')[:15], nome_empresa[:71], _('Pagina: ')[:10] + '{{ Page }}'))
        listagem.informaCab('')
        listagem.informaCab(' {:<15} {:^80} {:>10} '.decode('utf-8').format(
           data_titulo, _('Flash de Comandas'), hora_titulo[:5]))
        listagem.informaCab('')
        listagem.informaCab(' {:<80}'.decode('utf-8').format(
            _('Periodo do movimento: %s a %s') % (data_inicial_cabecalho, data_final_cabecalho)))
        listagem.informaCab('*' + '-' * 107 + '*')
        listagem.informaCab(det_format.decode('utf-8').format(
            _('Horario'), _('Quantidade aberta'), _('Tempo medio'), _('Ticket medio'),
            _('Quantidade fechada'), _('Quantidade cancelada')))
        listagem.informaCab('*' + '-' * 107 + '*')

        # Rodape
        listagem.rodapePadrao()

        # Totais
        total_aberta = total_fechada = total_cancelada = total_ticket = 0
        total_medio = '0:00:00'

        # Recupera a menor hora de venda e maior hora de venda
        hr_min, hr_max = map(lambda m: int(m), self._get_periodo_comanda(cr, data))

        # Comandas hora-a-hora
        for hora in range(hr_min, hr_max + 1):
            dados = self._gera_consulta(cr, data, hora, datas)
            qtde_fechada = qtde_cancelada = qtde_aberta = tempo_medio = ticket_medio = 0

            if dados:
                qtde_aberta = dados[0].get('qtd_cmd_aberta') or 0
                tempo_medio = str(dados[0].get('tempo_medio')).split('.')[0]
                tempo_medio = tempo_medio if tempo_medio != 'None' else '0:00:00'
                ticket_medio = round(dados[0].get('ticket_medio') or 0, 2)
                total_medio = str(dados[0].get('total_medio')).split('.')[0]
                total_medio = total_medio if total_medio != 'None' else '0:00:00'

            for f in dados:
                if f.get('fechado'):
                    qtde_fechada = f.get('fechado')
                elif f.get('supcda_restricao') == -1:
                    qtde_cancelada += 1

            # Calcula os totais
            total_aberta += qtde_aberta
            total_fechada += qtde_fechada
            total_cancelada += qtde_cancelada
            total_ticket += ticket_medio


            # Hora da comanda
            fuso = datetime.now(timezone(tz)).time().hour - datetime.utcnow().hour
            horario = _('%02d:00 as %02d:59:59') % tuple([hora + fuso] * 2)

            # Imprime a linha detalhe
            listagem.imprimeLinha(det_format.decode('utf-8').format(
                horario, locale.format('%d', qtde_aberta, True), tempo_medio,
                locale.format('%.2f', ticket_medio, True),
                locale.format('%d', qtde_fechada, True),
                locale.format('%d', qtde_cancelada, True)))

        # Resumo (totais)
        tot_format = '|{:>19}|{:>18}|{:^14}|{:>13}|{:>18}|{:>20}|'.decode('utf-8')
        listagem.imprimeLinha('*' + '-' * 107 + '*')
        listagem.imprimeLinha(tot_format.decode('utf-8').format(
            _('Total')[:25], locale.format('%d', total_aberta, True), total_medio,
            locale.format('%.2f', total_ticket, True), locale.format('%d', total_fechada, True),
            locale.format('%d', total_cancelada, True)))
        listagem.imprimeLinha('*' + '-' * 107 + '*')

        # Finaliza a impressao
        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    @staticmethod
    def _get_periodo_comanda(cr, data):
        """
        Recupera a menor e a maior hora de venda nas comandas em funcao do dia (data) informada.

        :param cr: cursor do banco de dados
        :param data: data do dia utilizana consulta
        :param context: um dicionário padrão para valores contextuais,

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        _sql = """
            -- noinspection SqlDialectInspection
            SELECT DATE_PART('hour', MIN(supcda_data_hora)) AS supcda_hora_min
                 , DATE_PART('hour', MAX(supcda_data_hora)) AS supcda_hora_max
            FROM   supervisor_cda_capa
            WHERE  supcda_data_hora BETWEEN '%(data)s 00:00:00' AND '%(data)s 23:59:59'
        """

        cr.execute(_sql % dict(data=data))
        result = cr.fetchall()

        if result:
            return result[0]

        return 0, 0

    def get_cond(self, cr, datas):
        'Implementado por heranca no modulo supervisor_master_comanda'
        return ''


    def _gera_consulta(self, cr, data, hora, datas):
        """
        Realiza consulta no banco de dados e retorna os quantitativos de comandas.

        :param cr: cursor do banco de dados
        :param data: data do dia utilizana consulta
        :param hora: hora a ser utilizada na consulta: 1, 2, 3, 4, etc...
        :param context: um dicionário padrão para valores contextuais,
        :type cr: cursor
        :type data: date
        :type context: dict
        :return: lista de dicionario com os quantitativos de comandas
        :rtype: list[dict]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # SQL de consulta
        cond = self.get_cond(cr, datas)
        _sql = """
            -- noinspection SqlDialectInspection
            SELECT SUM(supcda_data_cupom - supcda_data_hora) / COUNT(*)        AS tempo_medio
                 , SUM(supcda_total_comanda) / COUNT(*)                        AS ticket_medio
                 , COUNT(*)                                                    AS qtd_cmd_aberta
                 , supcda_restricao
                 , (SELECT COUNT(*)
                    FROM   supervisor_cda_capa
                    WHERE  supcda_data_cupom BETWEEN '%(dt_flash_inicio)s'
                                                 AND '%(dt_flash_fim)s')       AS fechado
                 , (SELECT SUM(supcda_data_cupom -
                               supcda_data_hora) / COUNT(*)
                    FROM   supervisor_cda_capa
                    WHERE  supcda_data_cupom BETWEEN '%(dt_inicio)s 00:00:00'
                                                 AND '%(dt_fim)s 23:59:59')    AS total_medio
                 , (SELECT SUM(supcda_total_comanda) / COUNT(*)                AS ticket_medio
                    FROM   supervisor_cda_capa
                    WHERE  supcda_data_cupom BETWEEN '%(dt_inicio)s 00:00:00'
                                                 AND '%(dt_fim)s 23:59:59')    AS total_ticket
            FROM   supervisor_cda_capa
            WHERE  supcda_data_hora BETWEEN '%(dt_flash_inicio)s' AND '%(dt_flash_fim)s'
              %(cond)s
            GROUP  BY supcda_restricao
        """
        hora_inicio = data + ' %02d:00:00' % hora
        hora_fim = data + ' %02d:59:59' % hora
        cr.execute(_sql % dict(dt_flash_inicio=hora_inicio, dt_flash_fim=hora_fim, dt_inicio=data,
                               dt_fim=data, cond=cond))

        return cr.dictfetchall()

report_custom('report.relatorio_flash_comandas')


# noinspection PyAbstractClass,PyPep8Naming
class wizard_relatorio_flash_comanda(osv.TransientModel):
    _name = 'wizard.relatorio.flash.comanda'
    _columns = {
        'data': fields.date('Data', required=True),
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        flash_comanda_dados = self.read(cr, uid, ids, [], context=context)
        if isinstance(flash_comanda_dados, (list, tuple)):
            flash_comanda_dados = flash_comanda_dados[0]
        datas = {
            'data': flash_comanda_dados.get('data'),
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'relatorio_flash_comandas',
            'datas': datas,
        }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
