#!/usr/bin/python
# -*- encoding: utf-8 -*-
import binascii


class Resp:
    fields_payload = ()
    response_mapping = {}
    index = ''
    default_response_mapping = {
        "init": (0, 1, True),
        "size": (1, 2, True),
        "naoIdentificado": (2, 3, True),
        "dados": (3, -2, False),
        "checksum": (-2, -1, True),
    }
    fp = {
        'RH': ('data_hora', 'data_inicio_hor_verao', 'data_fim_hor_verao'),
        'REON': ('status_pedido', 'comanda', 'data_hora', 'sentido', 'situacao', 'tipo_entrada')
    }

    error_messages = {
        "00": "No errors",
        "10": "Unknown command",
        "11": "Invalid package size",
        "12": "Parametros informados invalidos",
        "13": "Erro de Checksum",
        "26": "Fingerprint index not found",
        "29": "Could not capture or understand a fingerprint",
        "32": " O IP é invalido",
        "37": "Invalid card reference",
        "63": "Invalid option",
    }

    def __init__(self, raw_response):
        self.data_payload = {}
        self.fields_payload = ()
        self.raw_response = raw_response
        self.data = self.parse(raw_response)
        # print('Resp', self.data)

    def parse(self, raw_response):
        return self._parse(raw_response)

    def _parse(self, raw_response):

        response_mapping = dict(self.default_response_mapping, **self.response_mapping)
        result = {}
        for field, (start, end, as_hex) in response_mapping.items():
            bytea = raw_response[start:end]
            if as_hex:
                value = binascii.hexlify(bytea)
            else:
                value = bytea.decode()
            result[field] = value
            if field == 'dados':
                _comandos = value.split('+')
                _dados = _comandos[-1].split(']')
                _comandos.pop(-1)
                result['index'] = _comandos[0]
                result['comando'] = _comandos[1]
                self.fields_payload = self.fp.get(_comandos[1], ())
                self.data_payload = dict(zip(self.fields_payload, _dados))
        return result

    def error_message(self):
        message_status = self.data.get("message_status")
        return self.error_messages.get(message_status, "Unknown")

    def __repr__(self):
        if self.data.get("payload") is not None:
            return repr(self.data["payload"])
        return repr(self.data)

    def __len__(self):
        return len(self.data)

    def __getitem__(self, key):
        if hasattr(self, key):
            return getattr(self, key)
        return self.data[key]

