# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import threading
import logging
from ...comunicacao.sdk import SDK
from datetime import datetime, timedelta
from ...comunicacao.comandos import ComandaLiberarNegar, GetEvento
from openerp import pooler
import pytz

ServerHenry = False
HenryEncerrar = False
_logger = logging.getLogger('CatracaHenry')

def _get_datetime():
    data = datetime.now()
    origem = pytz.timezone('UTC')
    destino = pytz.timezone('America/Sao_Paulo')
    x = origem.localize(data, is_dst=False)
    return x.astimezone(destino)

def Henry(cr, ip, porta, timeout):
    def _get_verificar_liberacao(cr, comanda):
        """
        Verifica se a comanda está aberta
        Return True - Comanda Fechada
        Return False - Comanda Aberta
        """
        _cr = pooler.get_db(cr.dbname).cursor()

        _sql = """
           select True from supervisor_cda_capa 
              where supcda_codigo_comanda = %s and supcda_restricao = 0""" % comanda
        _cr.execute(_sql)
        dados = _cr.dictfetchone()
        _cr.close()
        if dados:
            return False
        return True

    global HenryEncerrar
    _cr = pooler.get_db(cr.dbname).cursor()
    mens_livre_1 = mens_livre_2 = mens_negado_1 = mens_negado_2 = ''
    _sql = "select * from supervisor_cfg_configuracao limit 1"
    _cr.execute(_sql)
    dados = _cr.dictfetchone()
    _cr.close()
    if dados:
        mens_livre_1 = dados.get('supcfc_cat_henry_mens_livre_1')
        mens_livre_2 = dados.get('supcfc_cat_henry_mens_livre_2')
        mens_negado_1 = dados.get('supcfc_cat_henry_mens_negado_1')
        mens_negado_2 = dados.get('supcfc_cat_henry_mens_negado_2')
    ultimo_acesso = datetime.now()
    iniciar = True
    with SDK(ip, porta, timeout=timeout) as s:
        while True:
            if HenryEncerrar:
                break
            if iniciar:
                resp, command = s.set_horario(timestamp=_get_datetime())
                _logger.info("Send:" + command.payload())
                iniciar = False
            comanda = GetEvento(s)
            if comanda.data:
                _logger.info("Resp:" + comanda.data.get('dados'))
            if comanda.status != 0:
                # Ao passar de 1 minuto, ler o horário para manter a conexao
                if ultimo_acesso < (datetime.now() - timedelta(minutes=1)):
                    resp, command = s.get_horario()
                    _logger.info("Send:" + command.payload())
                    ultimo_acesso = datetime.now()
                continue
            ultimo_acesso = datetime.now()
            # print(comanda.retorno)
            # print(comanda.data)
            # print(comanda.data_payload)
            if comanda.data_payload.get('status_pedido') != '0':
                # print('====>', comanda.data_payload.get('status_pedido'))
                continue
            if comanda.comando != 'REON':
                continue
            if comanda.index:
                res = _get_verificar_liberacao(cr, comanda.data_payload.get('comanda'))
                if res:
                    resp, command = s.send_command(
                        ComandaLiberarNegar(comanda.index, liberaNega='L', mensagens=(mens_livre_1, mens_livre_2)))
                else:
                    resp, command = s.send_command(ComandaLiberarNegar(comanda.index, liberaNega='N',
                                                       mensagens=(mens_negado_1, mens_negado_2)))
                _logger.info("Send:" + command.payload())


class supervisor_cfg_configuracao(osv.Model):
    _inherit = 'supervisor.cfg.configuracao'

    @staticmethod
    def supcfc_catraca_ativar(cr, ip, porta, timeout):
        _logger.info("Ativando catraca ......")
        global ServerHenry, HenryEncerrar
        HenryEncerrar = False
        ServerHenry = threading.Thread(target=Henry, args=(cr, ip, porta, timeout))
        ServerHenry.start()
        return True

    def __init__(self, pool, cr):
        _sql = """
        SELECT 1 FROM information_schema.columns 
              WHERE table_name='supervisor_cfg_configuracao' AND column_name='supcfc_cat_henry_habilitada'"""
        cr.execute(_sql)
        dados = cr.dictfetchone()
        if dados:
            _sql = "select * from supervisor_cfg_configuracao where supcfc_cat_henry_habilitada = True"
            cr.execute(_sql)
            dados = cr.dictfetchone()
            if dados:
                self.supcfc_catraca_ativar(cr,
                                           ip=dados.get('supcfc_cat_henry_ip'),
                                           porta=dados.get('supcfc_cat_henry_porta'),
                                           timeout=dados.get('supcfc_cat_henry_timeout'))
        return super(supervisor_cfg_configuracao, self).__init__(pool, cr)

    def supcfc_btn_catraca_ativar(self, cr, uid, ids, context=None):
        dados = self.read(cr, uid, ids[0], [], context=context )
        self.supcfc_catraca_ativar(cr,
                                   ip=dados.get('supcfc_cat_henry_ip'),
                                   porta=dados.get('supcfc_cat_henry_porta'),
                                   timeout=dados.get('supcfc_cat_henry_timeout'))
        self.write(cr, uid, ids, {'supcfc_cat_henry_habilitada': True}, context=context)
        return True

    def supcfc_btn_catraca_desativar(self, cr, uid, ids, context=None):
        _logger.info("Desativando catraca ......")
        global ServerHenry, HenryEncerrar
        HenryEncerrar = True
        ServerHenry._Thread__stop()
        self.write(cr, uid, ids, {'supcfc_cat_henry_habilitada': False}, context=context)
        return True






