# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report.render import render
from openerp.osv import osv, fields
from openerp.report.interface import report_int
import StringIO
from SigUtils.impressao import Impressora
from openerp.tools.translate import _
import time
from SigUtils.numero import getNumeroEditado
from openerp import pooler
from datetime import date, datetime


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming,PyAbstractClass
class wizard_relatorio_fichas(osv.TransientModel):
    """
    Gera os campos para filtro
    """

    _name = 'wizard.relatorio.fichas'
    _columns = {
        'wizfic_inicio': fields.date('Data inicial', required=True),
        'wizfic_fim': fields.date('Data final', required=True),
        'wizfic_loja': fields.integer('Loja'),
        'wizfic_caixa': fields.integer('Caixa'),
        'wizfic_item': fields.char('Item', size=14),
        'wizfic_state': fields.selection([('1', 'Emitida'), ('2', 'Trocada'), ('3', 'Faturada')],
                                           'Estado'),
        'wizfic_ordem': fields.selection([('1', 'Loja/Caixa/Numero'), ('2', 'Data'), ('3', 'Item')],
                                           'Ordem a listar', required=True),
    }

    _defaults = {
        'wizfic_inicio': '%s' % date.today(),
        'wizfic_fim': '%s' % date.today(),
        'wizfic_ordem': '1',
    }

    def button_imprimir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        listagem_fichas = self.read(cr, uid, ids[0], [], context=context)
        data_ini = listagem_fichas.get('wizfic_inicio')
        data_fim = listagem_fichas.get('wizfic_fim')
        state = listagem_fichas.get('wizfic_state')
        loja = listagem_fichas.get('wizfic_loja')
        caixa = listagem_fichas.get('wizfic_caixa')
        item = listagem_fichas.get('wizfic_item')
        ordem = listagem_fichas.get('wizfic_ordem')
        # Dados seleciondos do banco
        dados = self._gera_consulta(cr, data_ini, data_fim, state, loja, caixa, item, ordem,
                                    context)
        if not dados:
            return osv.except_osv(_('Atencao!'),
                                  _('Nenhum dado encontrado com os filtros selecionados!'))

        datas = {
            'data_ini': data_ini,
            'data_fim': data_fim,
            'state': state,
            'loja': loja,
            'caixa': caixa,
            'item': listagem_fichas.get('wizfic_item'),
            'ordem': listagem_fichas.get('wizfic_ordem'),
            'dados': dados
        }
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'wizard.relatorio.fichas',
            'datas': datas,
        }

    def _gera_consulta(self, cr, data_ini, data_fim, state, loja, caixa, item, ordem,
                       context=None):
        """
        Metodo para realizar a consulta no banco de dados
        recebendo como parametro o periodo e o status a filtrar
        """
        context = context and context.copy() or {}
        cond_state = cond_loja = cond_caixa = cond_item = order_by = ''
        if state:
            cond_state = "AND state = '%s'" % state

        if loja:
            cond_loja = 'AND supfch_loja = %s' % loja

        if caixa:
            cond_caixa = 'AND supfch_caixa = %s' % caixa

        if item:
            cond_item = "AND supfch_item = '%s'" % item.zfill(14)

        if ordem:
            order_by = 'ORDER BY %s' % {'1': 'supfch_loja, supfch_caixa, supfch_codigo_ficha',
                                        '2': 'create_date, supfch_loja, supfch_caixa, supfch_codigo_ficha',
                                        '3': 'supfch_item, supfch_loja, supfch_caixa, supfch_codigo_ficha'}.get(ordem)

        _sql = """SELECT  id,
                          supfch_codigo_ficha   AS cod_ficha,
                          state                 AS state,
                          create_date           AS data_emissao,
                          supfch_valor          AS valor,
                          supfch_desconto       AS desconto,
                          supfch_loja           AS loja,
                          supfch_caixa          AS caixa,
                          supfch_caixa_faturado AS caixa_fat,
                          supfch_item           AS cod_item                                           
                  FROM supervisor_cad_ficha 
                  WHERE create_date AT TIME ZONE 'UTC' >= '{data_ini} 00:00:00' 
                    AND create_date AT TIME ZONE 'UTC' <= '{data_fim} 23:59:59'
                    {cond_item}
                    {cond_caixa}
                    {cond_state}
                    {cond_loja}
                  {order_by}
        """.format(data_ini=data_ini,
                   data_fim=data_fim,
                   cond_item=cond_item,
                   cond_caixa=cond_caixa,
                   cond_state=cond_state,
                   cond_loja=cond_loja,
                   order_by=order_by)
        cr.execute(_sql)
        reg = cr.dictfetchall()
        return reg

class report_relatorio_listagem_fichas(report_int):
    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')
        loja = datas.get('loja')
        caixa_filtro = datas.get('caixa')
        state = datas.get('state')
        item = datas.get('item')
        ordem =  datas.get('ordem')
        dados = datas.get('dados')

        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato(orientacao='0')
        listagem.informaNome(pdf_string, _('Listagem de fichas'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)
        listagem.defineFonte(tam=5.8)
        nome_status = {'1': 'Emitida',
                       '2': 'Trocada',
                       '3': 'Faturada'}

        pool = pooler.get_pool(cr.dbname)
        try:
            company_id = pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                                                          'wizard.listagem.fichas', context=context)
            empresa_nome = pool.get('res.company').browse(cr, uid, company_id).name
        except AttributeError:
            empresa_nome = None

        tam_linha = 160
        data_relatorio = time.strftime('%d/%m/%Y')
        listagem.informaCab('{:^150}'.decode('utf-8').format(empresa_nome))
        listagem.informaCab('')
        listagem.informaCab(' {:<10} {:^126} {:>10} '.decode('utf-8').format(
            data_relatorio, _('Listagem de fichas')[:126], _('Pagina: ')[:10] + '{{ Page }}'))

        filtros = _(' Filtros:   Data Emissao: %s a %s' ) % (datetime.strptime(data_ini, '%Y-%m-%d').strftime('%d/%m/%Y'),
                                                            datetime.strptime(data_fim, '%Y-%m-%d').strftime( '%d/%m/%Y'))
        if loja:
            filtros += _('    Loja: %02d') % loja

        if caixa_filtro:
            filtros += _('    Caixa: %02d') % caixa_filtro

        if state:
            filtros += _('    Estado: %s') % nome_status.get(state)

        if item:
            filtros += _('    Item: %s') % item.zfill(14)

        listagem.informaCab(filtros)
        if ordem:
            listagem.informaCab(_(' Ordem a listar: %s') % {'1': 'Loja/Caixa/Numero','2': 'Data','3': 'Item'}.get(ordem))

        listagem.informaCab('*%s*' % ''.ljust(tam_linha,'-'))

        listagem.informaCab(_('|  Numero  | Loja | Caixa |     Data Emissao    |   Valor  | Desconto |  Estado  |  Codigo Item   |                           Descricao                          |'))
        listagem.informaCab('*%s*' % ''.ljust(tam_linha,'-'))

        # Rodape
        listagem.rodapePadrao()
        ficha_ids = map(lambda x: x.get('id'), dados)
        item_names = pool.get('supervisor.cad.ficha').get_descricao(cr, uid, ficha_ids, '', [],
                                                                    context=context)
        total_valor = 0
        total_desconto = 0
        qtd_fichas = {}
        for i in dados:
            data = datetime.strptime(i.get('data_emissao')[:19], '%Y-%m-%d %H:%M:%S')
            data = pool.get('supervisor.util').suputi_converte_tz(cr, uid, data, tz_origem='UTC',
                                                                  context=context)
            data = data.strftime('%H:%M:%S %d/%m/%Y')
            loja = i.get('loja')
            caixa = i.get('caixa')
            valor =  i.get('valor')
            desconto = i.get('desconto')
            if i.get('state') == '2':
                valor *= -1
                desconto *= -1

            line = ['| {:0>8} |   {:0>2} |    {:0>2} | {:>19} | {:>8} | {:>8} | {:<8} | {:>14} |'
                    ' {:<60} |'.format(i.get('cod_ficha'),
                                       loja,
                                       caixa,
                                       data,
                                       getNumeroEditado(valor, 2),
                                       getNumeroEditado(desconto, 2),
                                       nome_status.get(i.get('state')),
                                       i.get('cod_item'),
                                       item_names.get(i.get('id')) and
                                                             item_names.get(i.get('id'))[:60] or '')
                    ]
            listagem.imprimeLinha(''.join(line))
            total_valor += valor
            total_desconto += desconto
            chave = '%s-%s' % (loja, caixa)
            if not qtd_fichas.get(chave):
                qtd_fichas[chave] = 1
            else:
                qtd_fichas[chave] += 1

        listagem.imprimeLinha('*%s*' % ''.ljust(tam_linha,'-'))
        listagem.imprimeLinha(_('                                    TOTAIS      |{:>9} |{:>9} |                                                                                           ')
                                                  .format(getNumeroEditado(total_valor, 2),
                                                          getNumeroEditado(total_desconto, 2)))
        listagem.imprimeLinha('*%s*' % ''.ljust(28,'-'))
        listagem.imprimeLinha('| Loja | Caixa |     Qtd.    |')
        total_qtd = 0
        for i in sorted(qtd_fichas, key=lambda x: x):
            loja, caixa = i.split('-')
            qtd = qtd_fichas.get(i)
            line = ['|   {:0>2} |    {:0>2} |  {:>10} |'.format(loja, caixa, qtd)]
            listagem.imprimeLinha(''.join(line))
            total_qtd += qtd

        listagem.imprimeLinha('*%s*' % ''.ljust(28,'-'))
        listagem.imprimeLinha('| TOTAL FICHAS |  {:>10} |'.format(total_qtd))
        listagem.imprimeLinha('*%s*' % ''.ljust(28,'-'))

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()
        return self.obj.pdf, 'pdf'

report_relatorio_listagem_fichas('report.wizard.relatorio.fichas')
