# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_acompanhamento_ficha_registros(osv.TransientModel):
    _name = 'wizard.acompanhamento.ficha.registros'
    _columns = {
        'ficha_id': fields.many2one("wizard.acompanhamento.ficha", 'Ficha', ondelete="cascade",
                                     required=True, select=True),
        'coo': fields.integer('Coo'),
        'data': fields.date('Data'),
        'loja': fields.integer('Loja'),
        'caixa': fields.integer('Caixa'),
        'valor': fields.float('Valor', digits=(14,2)),
    }


class wizard_acompanhamento_ficha(osv.TransientModel):
    _name = 'wizard.acompanhamento.ficha'
    _columns = {
        'data_ini': fields.date('Data Inicial', required=True),
        'data_fim': fields.date('Data Final', required=True),
        'loja': fields.integer('Loja'),
        'caixa': fields.integer('Caixa'),
        'totalizador': fields.selection([('1','Todos'),
                                         ('2','Ficha'),
                                         ('3','Devolucao de Ficha'),], 'Totalizador', size=1,
                                        required=True),
    }

    _defaults = {
        'totalizador': '1',
    }

    def button_print_ficha(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_parametros = self.pool.get('supervisor.cfg.parametros')
        filtros = self.read(cr, uid, ids[0], [], context=context)
        datas = {
            'data_ini': filtros.get('data_ini'),
            'data_fim': filtros.get('data_fim'),
            'loja': filtros.get('loja', False),
            'caixa': filtros.get('caixa', False),
            'totalizador': filtros.get('totalizador'),
        }

        par_ids = obj_parametros.search(cr, uid, [], context=context)
        tots = obj_parametros.read(cr, uid, par_ids[0], ['supcfp_tot_ficha',
                                                     'supcfp_tot_devolucao_ficha'], context=context)
        dados_ficha = self.get_dados_ficha(cr, datas, tots, context)
        if not dados_ficha:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha registros com estes filtros!\nVerifique o totalizador de'
                                   ' ficha no cadastro de parametros'))

        datas['dados'] = dados_ficha
        datas['tot_devolucao'] = tots.get('supcfp_tot_devolucao_ficha')
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_acompanhamento_ficha',
            'datas': datas,
        }

    def get_dados_ficha(self, cr, datas, tot, context=None):
        context = context and context.copy() or {}
        caixa = datas.get('caixa')
        loja = datas.get('loja')
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')
        if datas.get('totalizador') == '2':
            totalizador = '%s' % tot.get('supcfp_tot_ficha')
        elif datas.get('totalizador') == '3':
            totalizador = '%s' % tot.get('supcfp_tot_devolucao_ficha')
        else:
            totalizador = '%s, %s' % (tot.get('supcfp_tot_ficha'),
                                      tot.get('supcfp_tot_devolucao_ficha'))

        cond_caixa = ''
        cond_loja = ''
        if caixa:
            cond_caixa = 'AND smc.supcap_caixa = %s' % caixa

        if loja:
            cond_loja = 'AND smc.supcap_loja = %s' % loja

        sql = """
                        SELECT smc.supcap_total_cupom        AS valor, 
                               smc.supcap_nro_cupom          AS coo,
                               smc.supcap_data_cupom         AS data,
                               smc.supcap_caixa              AS caixa,
                               smc.supcap_loja               AS loja,
                               smf.supfec_tipo_finalizadora  AS finalizadora,
                               COALESCE(smf.supfec_valor, 0) AS fin_valor,
                               COALESCE(smf.supfec_troco, 0) AS fin_troco,
                               smc.supcap_cod_tot_nao_fiscal AS tot
                        FROM supervisor_mov_capa  AS smc
                        LEFT JOIN supervisor_mov_fechamentos AS smf ON smf.supfec_capa_id = smc.id
                        WHERE smc.supcap_data_cupom >= '{data_ini}'
                          AND smc.supcap_data_cupom <= '{data_fim}'
                          AND smc.supcap_tipo = '{tipo}'
                          {cond_loja}
                          {cond_caixa}
                          AND smc.supcap_cod_tot_nao_fiscal IN ({tot})
                        ORDER BY smc.supcap_loja, smc.supcap_caixa, smc.supcap_nro_cupom
                """.format(tipo='W',
                           data_ini=data_ini,
                           data_fim=data_fim,
                           cond_loja=cond_loja,
                           cond_caixa=cond_caixa,
                           tot=totalizador)
        cr.execute(sql)
        return cr.dictfetchall()

