# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from os import path, listdir, remove
from fnmatch import fnmatch
from SigUtils.registro import Registro
import os
import importlib
from datetime import datetime


class wizard_supervisor_importa_arquivo(osv.Model):

    _inherit = "wizard.supervisor.importa.arquivo"

    def _importa_arquivo_funcionario(self, cr, uid, registros, lines, context=None):
        """
        O metodo importa os dados do funcionario para a tabela 'supervisor_cad_funcionarios'.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param instancia_arquivo: obj do arquivo que sera importado.
        :type instancia_arquivo: object
        :param registros: objeto dos layouts dos arquivos de importacao.
        :type registros: object
        :param lines: dados do arquivo que sera importado.
        :type lines: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso algum funcionario seja incluido, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False
        cwd = os.path.abspath(os.path.dirname(__file__))
        nome_arquivos = (fname for fname in os.listdir(cwd) \
                         if os.path.isdir(os.path.join(cwd, fname)))

        for nome_arquivo in nome_arquivos:
            module_importacao_path = os.path.join(cwd, '%s/arquivos' % nome_arquivo)
            registros = Registro(module_importacao_path)

        log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
        nome = '%s -  Funcionario' % self.pool.get('supervisor.util').suputi_datetime_to_timezone(
            cr, uid, datetime.now())
        qtd_importada = qtd_grava_log = 0
        frequencia = (len(lines)/50) or 1
        cr.execute('TRUNCATE supervisor_cad_funcionarios CASCADE')
        insert_cliente = """
        INSERT INTO supervisor_cad_funcionarios
            (supfun_codigo, supfun_codigo_cliente, supfun_salario, supfun_cpf,
            supfun_percentual_limite_desconto, supfun_percentual_desconto, create_date, create_uid,
            write_date, write_uid)
        VALUES
        """
        codigo_clientes = []
        for i in lines:
            try:
                codigo_clientes.append(registros.registro_decompoe(
                                            registros.Funcionario(), i.strip('\r\n').strip('\n')
                                            ).supfun_codigo_cliente)
            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines),
                               'Erro de Layout linha %s: %s' % (qtd_importada, str(e)),
                               context=context)

        obj_cad_clientes = self.pool.get('supervisor.cad.clientes')
        cliente_ids = obj_cad_clientes.search(cr, uid,
                                  [('supcli_codigo_interno','in',codigo_clientes)], context=context)
        codigo_cpf_clientes = {}
        map(lambda x: codigo_cpf_clientes.update(
            {int(x.get('supcli_codigo_interno')): x.get('supcli_cnpj_cpf')}),
            obj_cad_clientes.read(cr, uid, cliente_ids, ['supcli_codigo_interno','supcli_cnpj_cpf'],
                                  context=context))

        inserts = []
        codigos = []
        cod_clientes = []
        for line in lines:
            qtd_importada += 1
            qtd_grava_log += 1
            try:
                funcionario = registros.registro_decompoe(registros.Funcionario(),
                                                          line.strip('\r\n').strip('\n'))
                if funcionario.supfun_codigo in codigos:
                    raise Exception("Funcionario duplicado: %s" % funcionario.supfun_codigo)

                codigos.append(funcionario.supfun_codigo)

                if funcionario.supfun_codigo_cliente in cod_clientes:
                    raise Exception("Codigo de cliente duplicado: %s" % funcionario.supfun_codigo_cliente)

                cod_clientes.append(funcionario.supfun_codigo_cliente)

            except Exception as e:
                log_importacao(cr, uid, nome, qtd_importada, len(lines),
                               'Erro de Layout linha %s: %s' % (qtd_importada, str(e)),
                               context=context)
                continue

            if qtd_grava_log / frequencia > 0:  # frenquencia em que grava o log
                qtd_grava_log = 0
                log_importacao(cr, uid, nome, qtd_importada, len(lines), '', context=context)

            create_date = write_date = datetime.now()
            create_uid = uid
            write_uid = uid
            codigo_clientes.append(funcionario.supfun_codigo_cliente)
            sql = """
            ('{supfun_codigo}', '{supfun_codigo_cliente}', '{supfun_salario}', '{supfun_cpf}',
            '{supfun_percentual_limite_desconto}', {supfun_percentual_desconto}, '{create_date}',
            {create_uid}, '{write_date}', {write_uid})""".format(
                supfun_codigo=funcionario.supfun_codigo,
                supfun_codigo_cliente=funcionario.supfun_codigo_cliente,
                supfun_salario=float(funcionario.supfun_salario) / 100,
                supfun_cpf=codigo_cpf_clientes.get(int(funcionario.supfun_codigo_cliente)) or '',
                supfun_percentual_limite_desconto=float(
                    funcionario.supfun_percentual_limite_desconto) / 100,
                supfun_percentual_desconto=float(funcionario.supfun_percentual_desconto) / 100,
                create_date=create_date,
                create_uid=create_uid,
                write_date=write_date,
                write_uid=write_uid
            )
            sql = sql.replace("'null'", 'null')
            inserts.append(sql)
            res = True

        insert_cliente += ','.join(inserts)
        msg_erro = ''
        try:
            cr.execute(insert_cliente + '; COMMIT;')
        except Exception as e:
            cr.execute('ROLLBACK;')
            msg_erro = 'Erro: %s' % e

        log_importacao(cr, uid, nome, qtd_importada, len(lines), msg_erro, finalizado=True,
                       context=context)
        return res
