#!/usr/bin/python
#  -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import datetime
from calendar import monthrange


class wizard_compras_funcionarios(osv.TransientModel):
    _name = 'wizard.compras.funcionarios'

    _columns = {
        'wizfun_consulta_id': fields.many2one('wizard.consulta.compras.funcionarios', 'Compras',
                                            required=True),
        'wizfun_nome': fields.char('Nome', size=40),
        'wizfun_cfp': fields.char('CPF', size=15),
        'wizfun_ano': fields.integer('Ano'),
        'wizfun_mes': fields.integer('Mes'),
        'wizfun_vr_compras': fields.float('Total de compras', digits=(14,2)),
        'wizfun_vr_compras_desconto': fields.float('Compras com desconto', digits=(14,2)),
        'wizfun_vr_desconto': fields.float('Desconto', digits=(14,2)),
        'wizfun_vr_limite_desconto': fields.float('Limite de desconto', digits=(10,2)),
        'wizfun_codigo': fields.float('Codigo do funcionario', digits=(14, 0), required=True,
                                      select=True),
        'wizfun_codigo_cliente': fields.float('Codigo no cadastro de clientes', digits=(14, 0),
                                              required=True, select=True),
        'wizfun_salario': fields.float('Salario', required=True),
        'active': fields.boolean('Ativo'),
    }

anos = range(datetime.now().year-15, datetime.now().year+1)
anos.reverse()
SELECTION_ANO = map(lambda x: ('%s' % x, '%s' % x), anos)
SELECTION_MES = [('1','Janeiro'),('2','Fevereiro'),('3','Marco'),('4','Abril'),('5','Maio'),
                 ('6','Junho'),('7','Julho'), ('8','Agosto'),('9','Setembro'),('10','Outubro'),
                 ('11','Novembro'),('12','Dezembro')]


class wizard_consulta_compras_funcionario(osv.TransientModel):

    _name = 'wizard.consulta.compras.funcionarios'
    _rec_name = 'id'
    _columns = {
        'id': fields.integer('ID'),
        'wizccf_ano': fields.selection(SELECTION_ANO,'Ano', required=True, size=4),
        'wizccf_mes': fields.selection(SELECTION_MES, 'Mes', required=True, size=4),
        'wizccf_compras_funcionarios_ids': fields.many2many('supervisor.cad.funcionarios',
                                              'consulta_compras_de_funcionarios', 'id',
                                              'funcionario_id',
                                              'Funcionarios'),
        'wizccf_compras_ids': fields.one2many('wizard.compras.funcionarios','wizfun_consulta_id',
                                              'Compras por funcionario'),
    }

    _defaults = {
        'wizccf_ano': '%s' % datetime.now().year,
        'wizccf_mes': '%s' % datetime.now().month
    }

    def get_total_compras(self, cr, uid, context=None):
        """
        Retorna o total de compras por funcionario no periodo selecionado
        """
        context = context and context.copy() or {}
        data_ini = '%s-%02d-01' % (self.ano,  int(self.mes))
        data_fim = '%s' % (datetime(int(self.ano), int(self.mes),
                           monthrange(int(self.ano), int(self.mes))[1]).date())
        funcionario_ids = ','.join(map(lambda x: '%s' % x, self.funcionarios_ids))
        cond_funcionario = ''
        if funcionario_ids:
            cond_funcionario = 'AND scf.id IN (%(funcionario_ids)s)' % locals()
        sql = """
            SELECT SUM(CASE WHEN smc.supcap_tipo = 'V'
                            THEN   smc.supcap_total_cupom
                            ELSE - smc.supcap_total_cupom
                       END)                AS total,
                   scf.id                  AS funcionario_id
            FROM supervisor_cad_funcionarios  AS scf
            LEFT JOIN supervisor_cad_clientes AS scc ON scc.supcli_codigo_interno = scf.supfun_codigo_cliente
            LEFT JOIN supervisor_mov_capa     AS smc ON smc.supcap_cliente = scc.supcli_cnpj_cpf
            WHERE smc.supcap_data_cupom >= '%(data_ini)s'
              AND smc.supcap_data_cupom <= '%(data_fim)s'
              %(cond_funcionario)s
              AND smc.supcap_tipo IN ('V', 'C')
            GROUP BY scf.id
              """ % locals()
        cr.execute(sql)
        res = {} # chave=id do funcionario, valor=soma das compras do funcionario
        map(lambda x: res.update({x.get('funcionario_id'): x.get('total')}), cr.dictfetchall())
        return res

    def get_total_compras_desconto(self, cr, uid, context=None):
        """
        Retorna o total de compras com desconto de funcionario por funcionario no periodo
        selecionado
        """
        context = context and context.copy() or {}
        data_ini = '%s-%02d-01' % (self.ano,  int(self.mes))
        data_fim = '%s' % (datetime(int(self.ano), int(self.mes),
                           monthrange(int(self.ano), int(self.mes))[1]).date())
        funcionario_ids = ','.join(map(lambda x: '%s' % x, self.funcionarios_ids))
        cond_funcionario = ''
        if funcionario_ids:
            cond_funcionario = 'AND scf.id IN (%(funcionario_ids)s)' % locals()
        sql = """
            SELECT SUM(CASE WHEN smc.supcap_tipo = 'V' AND smcor.supcor_venda_cancelamento = 'V'
                            THEN   smcor.supcor_preco_total - smcor.supcor_desconto_item_sem_rat
                            WHEN smc.supcap_tipo = 'C'
                            THEN - smcor.supcor_preco_total + smcor.supcor_desconto_item_sem_rat
                            ELSE 0
                       END)                AS total,
                   SUM(CASE WHEN smc.supcap_tipo = 'V' AND smcor.supcor_venda_cancelamento = 'V'
                            THEN   smcor.supcor_desconto_item_sem_rat
                            WHEN smc.supcap_tipo = 'C'
                            THEN - smcor.supcor_desconto_item_sem_rat
                            ELSE 0
                       END)                AS desconto,
                   scf.id                  AS funcionario_id
            FROM supervisor_cad_funcionarios  AS scf
            LEFT JOIN supervisor_cad_clientes AS scc   ON scc.supcli_codigo_interno = scf.supfun_codigo_cliente
            LEFT JOIN supervisor_mov_capa     AS smc   ON smc.supcap_cliente = scc.supcli_cnpj_cpf
            LEFT JOIN supervisor_mov_corpos   AS smcor ON smcor.supcor_capa_id = smc.id
            WHERE smc.supcap_data_cupom >= '%(data_ini)s'
              AND smc.supcap_data_cupom <= '%(data_fim)s'
              AND smcor.supcor_tipo_promo = '3'
              %(cond_funcionario)s
              AND smc.supcap_tipo IN ('V', 'C')
            GROUP BY scf.id
              """ % locals()
        cr.execute(sql)
        # chave=id do funcionario, valor= dict=> chaves=[total, desconto] valors=compras com desconto
        # e desconto
        res = {}
        map(lambda x: res.update({x.get('funcionario_id'): {'total': x.get('total'),
                                                            'desconto': x.get('desconto')}}),
            cr.dictfetchall())
        return res

    def get_nome_funcionarios(self, cr, uid, context=None):
        context = context and context.copy() or {}
        funcionario_ids = ','.join(map(lambda x: '%s' % x, self.funcionarios_ids))
        cond_funcionario = ''
        if funcionario_ids:
            cond_funcionario = 'WHERE scf.id IN (%(funcionario_ids)s)' % locals()

        sql = """
            SELECT scc.name AS nome,
                   scf.id   AS funcionario_id
            FROM supervisor_cad_funcionarios   AS scf
            LEFT JOIN supervisor_cad_clientes AS scc ON scc.supcli_codigo_interno = scf.supfun_codigo_cliente
            %(cond_funcionario)s
              """ % locals()
        cr.execute(sql)
        res = {} # chave=id do funcionario, valor=nome do funcionario
        map(lambda x: res.update({x.get('funcionario_id'): x.get('nome')}), cr.dictfetchall())
        return res

    def button_consultar_compras_funcionario(self, cr, uid, ids, context=None, ):
        context = context and context.copy() or {}
        obj_compras = self.pool.get('wizard.compras.funcionarios')
        compras_antigas_ids = obj_compras.search(cr, uid, [('wizfun_consulta_id','=', ids[0]),
                                                           ('active', '=', 'True')],context=context)
        # remove as contultas antigas
        if compras_antigas_ids:
            obj_compras.write(cr, uid, compras_antigas_ids, {'active': False}, context=context)

        brw_consulta = self.browse(cr, uid, ids[0], context=context)
        self.ano = brw_consulta.wizccf_ano
        self.mes = brw_consulta.wizccf_mes
        brw_funcionarios = brw_consulta.wizccf_compras_funcionarios_ids
        self.funcionarios_ids = map(lambda x: x.id,brw_funcionarios)
        total_compras = self.get_total_compras(cr, uid, context)
        total_compras_desconto = self.get_total_compras_desconto(cr, uid, context)
        nome_funcionarios = self.get_nome_funcionarios(cr, uid, context)
        if not brw_funcionarios:
            obj_funcionarios = self.pool.get('supervisor.cad.funcionarios')
            funcionarios_ids = obj_funcionarios.search(cr, uid, [], context=context)
            brw_funcionarios = obj_funcionarios.browse(cr, uid, funcionarios_ids, context=context)

        list_vals = []
        for funcionario in brw_funcionarios:
            vr_compras_desconto = 0
            desconto = 0
            if total_compras_desconto.get(funcionario.id):
                vr_compras_desconto = total_compras_desconto.get(funcionario.id).get('total')
                desconto = total_compras_desconto.get(funcionario.id).get('desconto')

            salario = funcionario.supfun_salario
            limite_desconto = (salario * funcionario.supfun_percentual_limite_desconto / 100)
            vals = {
            'wizfun_codigo': funcionario.supfun_codigo,
            'wizfun_codigo_cliente': funcionario.supfun_codigo_cliente,
            'wizfun_salario': salario,
            'wizfun_consulta_id': ids[0],
            'wizfun_nome': nome_funcionarios.get(funcionario.id),
            'wizfun_cfp': funcionario.supfun_cpf or '',
            'wizfun_ano': self.ano,
            'wizfun_mes': self.mes,
            'wizfun_vr_compras': total_compras.get(funcionario.id) or 0,
            'wizfun_vr_compras_desconto': vr_compras_desconto,
            'wizfun_vr_desconto': desconto,
            'wizfun_vr_limite_desconto': limite_desconto,
            'active': True,
            }
            list_vals.append(vals)

        for vals in sorted(list_vals, key=lambda x: x.get('wizfun_nome')):
            obj_compras.create(cr, uid, vals, context=context)

        return True