# -*- coding: utf-8 -*-
#
# OBS: para funcionar eh preciso que tenha a pasta 'static' dentro do modulo

import openerp.addons.web.http as http
from openerp.tools import config as condifERP
import psycopg2
import json
import os
import xmlrpclib, socket
from datetime import datetime
from decimal import Decimal, ROUND_DOWN
from time import sleep
import pytz
import ConfigParser

TWOPLACES = Decimal(10) ** -2

SEM_ERRO = '00'
ERRO_CONEXAO = '01'
USUARIO_SENHA = '02'
ERRO_CNPJ = '03'
ERRO_LICENCA = '04'
ERRO = '05'
ITEM_NAO_ENCONTRADO = '06'
COMANDA_NAO_ENCONTRADO = '07'
COMANDA_SEM_ITENS = '08'
COMANDA_SESATUALIZADA = '09'
NENHUM_ITEM_PARA_OPERADOR = '10'
ERRO_INTERNO = '99'


class Config(object):
    def __init__(self):
        self.dados = False

    def start(self):
        if self.dados:
            return
        file = condifERP.get('addons_path') + '/supervisor_producao_comanda/app.config'
        if not os.path.exists(file):
            raise Exception(u'Arquivo de configuração "app.config" não existe no módulo '
                            u'"supervisor_producao_comanda"')
        c = ConfigParser.ConfigParser()
        c.read(file)
        self.dados = c


config = Config()


def utl_converte_tz(data, tz_origem='UTC', tz_destino='America/Sao_Paulo'):
    """
    Função para converter data de um timezone para outro
    """
    obj_origem = pytz.timezone(tz_origem)
    obj_destino = pytz.timezone(tz_destino)
    x = obj_origem.localize(data, is_dst=False)
    return x.astimezone(obj_destino)


def remove_acentuacao(text, cod='utf8'):
    from unicodedata import normalize, category

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        res = normalize('NFKD', text).encode('ASCII', 'ignore')

    else:
        try:
            res = unicode(filter(lambda c: category(c) != 'Mn',
                                 normalize('NFKD', unicode_string.decode(cod, 'replace'))))
        except:
            res = normalize('NFKD', text.decode('utf8')).encode(cod, 'ignore')

    return res


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class MySock(object):
    def __init__(self, params):
        super(MySock, self).__init__()
        self.uid = int(params.get('uid') or 1)
        self.pwd = 'supervisor_remoto_77813'
        url = params.get('url') or ''
        self.host = url
        self.port = params.get('port') or ''
        if not self.port and ':' not in url:
            raise Exception(u'Porta não informada na URL!')
        elif ':' in url:
            self.host, self.port = url.split(':')

        cnpj = params.get('cnpj')
        config.start()
        self.db = config.dados.get(cnpj, 'database')
        self.sock = xmlrpclib.ServerProxy('http://{0:>}:{1:>}/xmlrpc/object'.format(self.host,
                                                                                    self.port))
        socket.setdefaulttimeout(300)

    def execute(self, model, function, *args):
        return self.sock.execute(self.db, self.uid, self.pwd, 'supervisor.util',
                                 'supervisor_get_funcao', model, function, *args)


class sc_supervisor(http.Controller):
    _cp_path = '/supervisor_producao_comanda'

    @http.httprequest
    def teste(self, req, s_action=None, **kw):
        ip = req.httprequest.environ.get('REMOTE_ADDR')
        print (ip)
        # return ip
        return json.dumps({'codigo': SEM_ERRO,
                    'mensagem': u'ok',
                    'retorno': [ip]})

    def _get_cursor(self, params):
        try:
            cnpj = params.get('cnpj')
            config.start()
            try:
                user = config.dados.get(cnpj, 'username')
            except Exception as e:
                if 'No section:' in e.message:
                    raise Exception(u'O CNPJ "%s" não está cadastrado no arquivo de configuração'
                                    u' "app.config"' % unicode(cnpj))
                else:
                    raise e

            pwd = config.dados.get(cnpj, 'password')
            if pwd == "''":
                pwd = ''
            host = config.dados.get(cnpj, 'hostname')
            port = config.dados.get(cnpj, 'port')
            db = config.dados.get(cnpj, 'database')
            conn = psycopg2.connect(database=db, password=pwd, user=user,
                                    host=host,
                                    port=port, connect_timeout=4)
            cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            return cr, False
        except Exception as e:
            return False, json.dumps({'codigo': ERRO_CONEXAO,
                                      'mensagem': u'Erro: %s' % e,
                                      'retorno': [],
                                      })

    @http.httprequest
    def check_licenca(self, req, s_action=None, **kw):
        try:
            sock = MySock(req.params)
            vals = {'serial': req.params.get('serial'),
                    'model': req.params.get('model'),
                    'brand': req.params.get('brand'),
                    'cnpj': req.params.get('cnpj'),
                    }
            if not req.params.get('cnpj'):
                return json.dumps({'codigo': ERRO_CNPJ,
                                   'mensagem': u'CNPJ não informado!',
                                   'retorno': []})

            if not sock.execute('supervisor.producao.comanda.dispositivo',
                                'spcdis_check_licenca', vals):
                return json.dumps({'codigo': ERRO_LICENCA,
                                   'mensagem': u'Licensa do dispositivo inválida!',
                                   'retorno': []})

            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': u'ok',
                               'retorno': []})

        except Exception as e:
            if type(e) == xmlrpclib.Fault:
                msg_erro = e.faultString
                if e.faultCode == 'AccessDenied':
                    msg_erro = 'Erro: %s' % "Acesso ao ERP negado para este Usuário e Senha!"
                elif '9999\n' in e.faultCode:
                    msg_erro = e.faultCode.split('9999\n')[1]
                return json.dumps({'codigo': ERRO_INTERNO,
                                   'mensagem': msg_erro,
                                   'retorno': []})

            msg_erro = ''
            if 'message' in dir(e):
                msg_erro = e.message
            else:
                msg_erro == str(e)
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % msg_erro,
                               'retorno': []})

    @http.httprequest
    def logar(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            login = req.params.get('username') or ''
            senha = req.params.get('password') or ''
            cr.execute("""SELECT * 
                          FROM supervisor_cad_operadores
                          WHERE supope_codigo = '{login}'
                            AND supope_senha = '{senha}'  
                        """.format(login=login,
                                   senha=senha))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'codigo': USUARIO_SENHA,
                                   'mensagem': u'Usuario ou senha incorretos!',
                                   'retorno': []})

            cr.execute("""SELECT id       AS uid,
                                 password AS pwd
                          FROM res_users
                          WHERE login = 'supervisor_remoto'
                        """.format())
            dados[0].update(dictfetchall(cr)[0])
            cr.close()
            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': u'ok',
                               'retorno': dados})

        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})
    @http.httprequest
    def get_comanda(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            codigo_comanda = int(req.params.get('codigo_comanda') or 0)
            if not codigo_comanda:
                cr.close()
                return json.dumps({'codigo': ERRO,
                                   'mensagem': u'Digitite um código válido!',
                                   'retorno': []})

            cr.execute("""SELECT *,
                                 COALESCE(0) AS sequencia
                          FROM supervisor_cda_capa
                          WHERE supcda_codigo_comanda = {codigo}
                            AND supcda_restricao = 0
                       """.format(codigo=codigo_comanda))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'retorno': [],
                                   'codigo': SEM_ERRO,
                                   'mensagem': u'ok'})

            cr.execute("""SELECT MAX(supcdo_sequencia_id) AS sequencia
                          FROM supervisor_cda_corpo 
                          WHERE supcdo_capa_id = {capa_id}
                       """.format(capa_id=dados[0].get('id')))
            res = dictfetchall(cr)
            if res:
                dados[0]['sequencia'] = res[0].get('sequencia')

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_itens(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            item = req.params.get('item') or ''
            where = "name ilike '%%%s%%'" % item
            if item.isdigit():
                where = "supite_codigo_item = '%s'" % item.zfill(14)

            cr.execute("""SELECT supcfp_produtos_app
                          FROM supervisor_cfg_parametros """)
            parametro = dictfetchall(cr)
            if parametro and parametro[0].get('supcfp_produtos_app'):
                where += "                AND supite_app_comanda = True"

            cr.execute("""SELECT name,
                                 supite_preco,
                                 supite_codigo_item,
                                 supite_producao
                          FROM supervisor_cad_itens 
                          WHERE {where} 
                            AND active = True
                          ORDER BY name
                       """.format(where=where))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'retorno': dados,
                                   'codigo': ITEM_NAO_ENCONTRADO,
                                   'mensagem': u'Nenhum item encontrado!'})

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})
    @http.httprequest
    def get_itens_favoritos(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            cr.execute("""
              SELECT sci.name,
                     sci.supite_preco,
                     sci.supite_codigo_item,
                     sci.supite_producao
              FROM supervisor_cad_itens AS sci
              INNER JOIN supervisor_cda_itens_favoritos AS sif ON sif.supifa_item_id = sci.id
              WHERE sci.active = True
              ORDER BY name
                       """)
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'retorno': dados,
                                   'codigo': ITEM_NAO_ENCONTRADO,
                                   'mensagem': u'Nenhum itens cadastrado como favorito!'})

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_comanda(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            sock = MySock(req.params)
            codigo_operador = int(req.params.get('codigo_operador') or 0)
            capa_id = int(req.params.get('comanda_id') or 0)
            total_comanda = Decimal(req.params.get('total_comanda') or 0)
            codigo_comanda = int(req.params.get('codigo_comanda') or 0)
            mesa = int(req.params.get('mesa') or 0)
            mesa = mesa and (', %s' % mesa) or ''
            camp_mesa = mesa and 'supcda_mesa,' or ''
            corpos = eval(req.params.get('corpos').replace("{", "{'").replace(": ", "': '").
                          replace("}", "'}").replace(", ", "', '").replace("'{", "{").
                          replace("}'", "}").replace(';', ',') or '[]')
            max_seq = 0
            if capa_id and not corpos and mesa:
                cr.execute("""UPDATE supervisor_cda_capa 
                              SET supcda_mesa = {mesa}
                              WHERE  id = {id};
                           """.format(mesa=mesa,
                                      id=capa_id))
            elif not capa_id:
                cr.execute("""SELECT id 
                              FROM supervisor_cda_capa 
                              WHERE supcda_codigo_comanda = {codigo}
                                AND supcda_restricao = 0 
                                 """.format(codigo=codigo_comanda))
                dados = dictfetchall(cr)
                if dados:
                    capa_id = dados[0].get('id')
                else:
                    cr.execute("""INSERT INTO supervisor_cda_capa
                                          (supcda_codigo_comanda, supcda_data_hora, supcda_operador,
                                           {camp_mesa} supcda_restricao,
                                           supcda_total_comanda) 
                                  VALUES ({codigo}, '{data}', {operador} {mesa}, 0, 0)
                                  RETURNING id;
                                  """.format(camp_mesa=camp_mesa,
                                             codigo=codigo_comanda,
                                             data=datetime.utcnow(),
                                             operador=codigo_operador,
                                             mesa=mesa))
                    dados = dictfetchall(cr)
                    capa_id = dados[0].get('id')

            cr.execute("""SELECT MAX(supcdo_sequencia_id) AS seq_max
                          FROM supervisor_cda_corpo 
                          WHERE supcdo_capa_id = {capa_id} 
                         """.format(capa_id=capa_id))
            dados = dictfetchall(cr)
            if dados:
                max_seq = dados[0].get('seq_max')

            sql = []
            corpos = sorted(corpos, key=lambda x: x.get('seq').strip())
            for i in corpos:
                seq = i.get('seq').strip()
                if int(seq) <= max_seq:
                    cr.close()
                    return json.dumps({'codigo': COMANDA_SESATUALIZADA,
                                       'mensagem': u'---* Atenção *---\nComanda desatualizada!'
                                                   u' Volte ao menu e lance os itens novamente!',
                                       'retorno': []})

                if max_seq:  # em casos onde esta inserindo itens na mesma comanda em 2 ou mais dispositivos
                    max_seq += 1
                    seq = max_seq

                total_item = Decimal(i.get('preco_total').replace(',', '.'))\
                             * Decimal(i.get('qtd').replace(',', '.')).quantize(TWOPLACES,
                                                                                rounding=ROUND_DOWN)
                total_comanda += total_item
                sql.append("""('{create_date}', {capa_id}, {seq}, '{cod_item}', {qtd}, {preco_total}, 
                                        {cod_vendedor}, '{nome_vendedor}', '{obs}', {seq_impressao}, 
                                        '{imprimir}', '{venda_cancel}', False)
                              """.format(create_date=datetime.utcnow(),
                                         capa_id=capa_id,
                                         seq=seq,
                                         cod_item=i.get('cod_item'),
                                         qtd=i.get('qtd').replace(',', '.'),
                                         preco_total=total_item,
                                         cod_vendedor=i.get('cod_vendedor'),
                                         nome_vendedor=i.get('nome_vendedor'),
                                         obs=i.get('obs'),
                                         seq_impressao=0,
                                         imprimir=i.get('imprimir'),
                                         venda_cancel='V'))
            campo_mesa = ''
            if mesa:
                campo_mesa = ', supcda_mesa = %s' % mesa.replace(', ', '')

            sql_insert = """INSERT INTO supervisor_cda_corpo 
                               (create_date, supcdo_capa_id, supcdo_sequencia_id, supcdo_item,
                                supcdo_quantidade, supcdo_preco_total, supcdo_cod_vendedor,
                                supcdo_nome_vendedor, supcdo_observacao, supcdo_seq_impressao,
                                supcdo_imprimir, supcdo_venda_cancelamento, supcdo_entregue)
                            VALUES {valores};
                            UPDATE supervisor_cda_capa 
                               SET supcda_total_comanda = {total} 
                                   {campo_mesa}
                            WHERE  id = {capa_id};
                            """.format(valores=',\n'.join(sql),
                                       total=total_comanda,
                                       campo_mesa=campo_mesa,
                                       capa_id=capa_id)
            cr.execute(sql_insert)
            self.imprimir_pedidos_cozinha(cr, sock, capa_id)
            cr.execute('COMMIT;')
            cr.close()
            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': u'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    def imprimir(self, sock, vals_impressora, arquivo):
        res = sock.execute('supervisor.producao.comanda.dispositivo',
                           'spcdis_imprimir', vals_impressora, arquivo, {})
        return res

    def imprimir_pedidos(self, sock, impressoras):
        for vals in impressoras.values():
            data = str(datetime.now().strftime('%Y-%m-%d_%H:%M:%S'))
            path = 'pedido_cozinha_%s.txt' % data
            try:
                res = self.imprimir(sock, vals, path)
                if not res[0]:
                    raise Exception(res[1])

            except Exception as e:
                if type(e) == xmlrpclib.Fault:
                    msg_erro = e.faultString
                    if e.faultCode == 'AccessDenied':
                        msg_erro = 'Erro: %s' % "Acesso ao ERP negado para este Usuário e Senha!"
                    elif '9999\n' in e.faultCode:
                        msg_erro = e.faultCode.split('9999\n')[1]
                    # cr.close()
                    # return json.dumps({'codigo': ERRO_INTERNO,
                    #                    'mensagem': msg_erro,
                    #                    'retorno': []})
                    raise Exception(msg_erro)
                # raise Exception(('Erro ao imprimir pedido de cozinha: %s' % e).decode('string_escape'))

            sleep(1)

    def imprimir_pedidos_cozinha(self, cr, sock, capa_id):
        cr.execute("""SELECT id, 
                             supcdo_seq_impressao, 
                             supcdo_observacao, 
                             supcdo_item, 
                             supcdo_quantidade,
                             supcdo_nome_vendedor, 
                             supcdo_capa_id, 
                             supcdo_sequencia_id, 
                             create_date,
                             supcdo_capa_id
                      FROM supervisor_cda_corpo 
                      WHERE supcdo_capa_id = {capa_id}
                        AND supcdo_imprimir = '3' 
                      ORDER BY id""".format(capa_id=capa_id))
        dados_corpo = dictfetchall(cr)
        if not dados_corpo:
            return True

        codigos = []
        capa_ids = []
        for i in dados_corpo:
            codigos.append(i.get('supcdo_item'))
            capa_ids.append(i.get('supcdo_capa_id'))

        # ------------ pega a relacao dos itens ---------------
        sql = """SELECT supite_codigo_local,
                        supite_descricao_completa,
                        supite_codigo_item
                  FROM supervisor_cad_itens 
                  WHERE supite_codigo_item in ('%s')
                    AND active = True
                  """ % "','".join(codigos)
        cr.execute(sql)
        dados_item = dictfetchall(cr)
        dict_itens = {}
        map(lambda x: dict_itens.update({x.get('supite_codigo_item'): x}), dados_item)
        # ------------ pega a relacao das capas ---------------
        dict_capa = {}
        sql = """SELECT id, 
                        supcda_nome_cliente, 
                        supcda_codigo_comanda, 
                        supcda_mesa, 
                        supcda_loja, 
                        supcda_caixa
                 FROM supervisor_cda_capa 
                 WHERE id = %s""" % capa_id
        cr.execute(sql)
        for i in dictfetchall(cr):
            cliente = i.get('supcda_nome_cliente') or ''
            comanda = i.get('supcda_codigo_comanda')
            mesa = i.get('supcda_mesa') or 0
            chave = '%s-%s' % (i.get('supcda_loja'), i.get('supcda_caixa'))
            dict_capa.update({i.get('id'): (chave, cliente, comanda, mesa)})

        impressoras = self._get_impressora_impressao(cr, dados_corpo, dict_itens, dict_capa)
        if not impressoras:
            raise Exception('Nenhuma impressora cadastrada no cadastro dos locais')

        self.imprimir_pedidos(sock, impressoras)
        return True

    def _get_realacao_locais(self, cr):
        locais = {}
        cr.execute("""SELECT spl.sucloc_impressora,
                             spl.sucloc_codigo,
                             spli.sulpim_porta AS porta_impressora,
                             spli.suplim_comando_id AS comando_id
                      FROM supervisor_producao_local AS spl
                      LEFT JOIN supervisor_producao_local_impressora AS spli 
                                ON spli.suplim_codigo = spl.sucloc_codigo                   
                      WHERE sucloc_impressora IS NOT NULL""")
        map(lambda x: locais.update({x.get('sucloc_codigo'): x}),
            dictfetchall(cr))
        return locais

    def _get_impressora_impressao(self, cr, dados_corpo, dict_itens, dict_capa):
        """Retorna um  dict com a chave sendo a impressora e o valor as impressoes"""
        dict_locais = self._get_realacao_locais(cr)
        impressoras = {}
        if not dict_locais:
            return impressoras

        cr.execute("""SELECT supcfc_cliente FROM supervisor_cfg_configuracao ORDER BY id""")
        nome_empresa = dictfetchall(cr)[0].get('supcfc_cliente') or ''
        data = utl_converte_tz(datetime.now(), tz_origem='UTC',
            tz_destino='America/Sao_Paulo').strftime('%d/%m/%Y %H:%M:%S')
        ultima_capa_id = False
        cabecalho = ['{b1}%s{b2}' % (nome_empresa.center(48)),
                     ('DATA IMPRESSAO: %s' % data).rjust(48),
                     '',
                     '{b1}--------------* PEDIDO DE COZINHA *-------------{b2}',
                     '']
        rodape = ['DESENVOLVIDO POR:',
                  '{b1}Sig Solucoes em Informatica Ltda. (31)2106-1700{b2}',
                  '{corta}',  # corta papel
                  ]
        impressao_comanda = []
        ultima_printer = False
        ultimo_local = False
        for corpo in dados_corpo:
            # itens cancelados nao imprime
            if corpo.get('supcdo_venda_cancelamento') == 'C':
                cr.execute("""UPDATE supervisor_cda_corpo 
                              SET supcdo_imprimir = '1'
                              WHERE id = %s;""" % corpo.get('id'))
                continue

            create_date = data
            if corpo.get('create_date'):
                create_date = utl_converte_tz(
                    datetime.strptime(('%s' % corpo.get('create_date'))[:19], '%Y-%m-%d %H:%M:%S'),
                    tz_origem='UTC', tz_destino='America/Sao_Paulo').strftime('%d/%m/%Y %H:%M:%S')
            item = dict_itens.get(corpo.get('supcdo_item'))
            local = item.get('supite_codigo_local')
            if not local:
                raise Exception(u'Item sem local de produção: %s' % item.get('supite_codigo_item'))

            if not dict_locais.get(local):
                raise Exception(u'Local de produção com código "%s" não cadastrado!' % local)

            printer = dict_locais.get(local).get('sucloc_impressora')
            porta = dict_locais.get(local).get('porta_impressora') or ''
            comando_id = dict_locais.get(local).get('comando_id')
            if not comando_id:
                raise Exception(u'Sem correlação de local e impressora para o local "%s"' % local)

            nome_cliente = False
            comanda = dict_capa.get(corpo.get('supcdo_capa_id'))[2]
            mesa = dict_capa.get(corpo.get('supcdo_capa_id'))[3]
            seq_item = corpo.get('supcdo_sequencia_id')
            descricao = remove_acentuacao(item.get('supite_descricao_completa'))
            qtd = corpo.get('supcdo_quantidade')
            if len(descricao) > 42:
                descricao = '\n      '.join([descricao[:42], descricao[42:]])

            obs = remove_acentuacao(corpo.get('supcdo_observacao') or '')
            if len(obs) > 48:
                obs = '\n'.join([obs[:48], obs[48:]])

            seq_impressao = (corpo.get('supcdo_seq_impressao') or 0) + 1
            if ultima_capa_id != corpo.get('supcdo_capa_id') or ultimo_local != local:
                if impressao_comanda:
                    if ultima_printer:
                        if not impressoras.get(ultimo_local):
                            impressoras[ultimo_local] = {'impressora': ultima_printer,
                                                         'texto': ['\n'.join(impressao_comanda)],
                                                         'porta': porta,
                                                         'comando_id': comando_id}

                        else:
                            impressoras[ultimo_local]['texto'][0] += '\n' + '\n'.join(impressao_comanda)
                    else:
                        raise Exception(u'Impressora não cadastrada no local do item: %s' %
                              item.get('supite_codigo_item'))

                ultima_capa_id = corpo.get('supcdo_capa_id')
                impressao_comanda = []
                if not impressoras.get(local):
                    impressao_comanda.extend(cabecalho)
                    if seq_impressao > 1:
                        impressao_comanda.append('{f1} Reimpressao de pedido{f2}')

                    if nome_cliente:
                        impressao_comanda.append('CLIENTE: %s' % nome_cliente)

                    impressao_comanda.extend([
                        'GARCOM: {b1}%s{b2}' % (corpo.get('supcdo_nome_vendedor') or ''),
                        'COMANDA: {b1}%06d{b2}' % comanda,
                        'MESA: {b1}%03d{b2}' % mesa,
                        'DATA: %s' % create_date,
                        '',
                        '{b1}------------------------------------------------{b2}',
                    ])

            impressao_comanda.extend([
                'Seq.Item: {b1}%03d{b2}' % seq_item,
                'SEQ.IMPRESSAO: %s' % seq_impressao,
                'ITEM: {f1}%s{f2}' % descricao,
                'QTD: {f1}{b1}%s{b2}{f2}' % qtd,
                '',
                'OBS:\n{f1}*%s{f2}' % obs,
                '', ])
            ultima_printer = printer
            ultimo_local = local
            if not printer:
                continue

            data = utl_converte_tz((datetime.strptime(data, '%d/%m/%Y %H:%M:%S')),
                                   tz_origem='America/Sao_Paulo', tz_destino='UTC').strftime(
                '%d/%m/%Y %H:%M:%S')
            cr.execute("""UPDATE supervisor_cda_corpo
                          SET supcdo_seq_impressao = %s,
                              supcdo_data_hora_impressao = '%s',
                              supcdo_imprimir = '4'
                          WHERE id = %s;""" % (seq_impressao, data, corpo.get('id')))

        if impressao_comanda:
            if ultima_printer:
                if not impressoras.get(ultimo_local):
                    impressoras[ultimo_local] = {'impressora': ultima_printer,
                                                 'texto': ['\n'.join(impressao_comanda)],
                                                 'porta': porta, 'comando_id': comando_id}
                else:
                    impressoras[ultimo_local]['texto'][0] += '\n' + '\n'.join(impressao_comanda)
            else:
                raise Exception(u'Impressora não cadastrada no local do item: %s' %
                                  item.get('supite_codigo_item'))

        for vals in impressoras.values():
            vals['texto'][0] += '\n' + '\n'.join(rodape)

        return impressoras

    @http.httprequest
    def get_itens_comanda(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            codigo_comanda = int(req.params.get('codigo_comanda')) or 0
            cr.execute("""
            SELECT scc.supcda_mesa           AS mesa,
                   sco.supcdo_sequencia_id   AS seq,
                   sco.supcdo_quantidade     AS qtd,
                   sco.id                    AS id,
                   sci.name                  AS nome,
                   sco.supcdo_capa_id        AS capa_id
            FROM supervisor_cda_capa        AS scc
            INNER JOIN supervisor_cda_corpo AS sco ON sco.supcdo_capa_id = scc.id
            INNER JOIN supervisor_cad_itens AS sci ON sci.supite_codigo_item = sco.supcdo_item
            WHERE scc.supcda_codigo_comanda = {comanda}
              AND sco.supcdo_venda_cancelamento = 'V'
              AND scc.supcda_restricao = 0
            ORDER BY sco.supcdo_sequencia_id
                       """.format(comanda=codigo_comanda))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'retorno': [],
                                   'codigo': COMANDA_NAO_ENCONTRADO,
                                   'mensagem': u'Comanda não encontrada!'})

            for i in dados:
                i['qtd'] = str(i.get('qtd')).replace('.', ',')
                i['seq' ] = str(i.get('seq')).zfill(2)
                i['mesa'] = str(i.get('mesa'))

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def cancelar_itens(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            capa_id = int(req.params.get('capa_id') or 0)
            corpo_ids = ','.join(map(lambda x: str(x), eval(req.params.get('corpo_ids') or '[]')))
            cr.execute("""SELECT supcda_total_comanda - (SELECT SUM(supcdo_preco_total)
                                                         FROM supervisor_cda_corpo
                                                         WHERE id IN ({corpo_ids})) AS total
                          FROM supervisor_cda_capa
                          WHERE id = {capa_id}

                       """.format(corpo_ids=corpo_ids,
                                  capa_id=capa_id))

            total = dictfetchall(cr)[0].get('total')
            sql = """UPDATE supervisor_cda_corpo
                     SET supcdo_venda_cancelamento = 'C'
                     WHERE id IN ({corpo_ids});
                     UPDATE supervisor_cda_capa SET supcda_total_comanda = {total}
                     WHERE id = {capa_id};
                     COMMIT;
                       """.format(corpo_ids=corpo_ids,
                                  capa_id=capa_id,
                                  total=total)
            cr.execute(sql)
            cr.close()
            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': u'Itens cancelados com sucesso!'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_itens_entrega(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            operador = req.params.get('operador') or ''
            cr.execute("""SELECT supcfp_itens_operador
                          FROM supervisor_cfg_parametros
                       """)
            parametro = dictfetchall(cr)[0]
            cond_operador = ''
            if parametro.get('supcfp_itens_operador') and operador:
                cond_operador = """
                          --(scc.supcda_operador = {operador}
                           --OR)
                           AND sco.supcdo_cod_vendedor = {operador}""".format(operador=operador)
            cr.execute("""
                    SELECT scc.supcda_mesa           AS mesa,
                           scc.supcda_codigo_comanda AS comanda,
                           sco.supcdo_sequencia_id   AS seq,
                           sco.supcdo_quantidade     AS qtd,
                           sco.id                    AS id,
                           sci.name                  AS nome
                    FROM supervisor_cda_capa        AS scc
                    INNER JOIN supervisor_cda_corpo AS sco ON sco.supcdo_capa_id = scc.id
                    INNER JOIN supervisor_cad_itens AS sci ON sci.supite_codigo_item = sco.supcdo_item
                    WHERE 
                       scc.supcda_restricao = 0
                       {cond_operador}
                       AND sco.supcdo_venda_cancelamento = 'V'
                       AND sco.supcdo_entregue = False
                    ORDER BY scc.supcda_codigo_comanda, sco.supcdo_sequencia_id                           
                       """.format(cond_operador=cond_operador))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'retorno': dados,
                               'codigo': NENHUM_ITEM_PARA_OPERADOR,
                               'mensagem': u'Nenhum item para este operador!'})

            for i in dados:
                i['qtd'] = str(i.get('qtd')).replace('.', ',')
                i['seq' ] = str(i.get('seq')).zfill(2)
                i['mesa'] = str(i.get('mesa'))

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_itens_entrega(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            corpo_ids = ','.join(map(lambda x: str(x), eval(req.params.get('corpo_ids') or '[]')))
            cr.execute("""UPDATE supervisor_cda_corpo
                          SET supcdo_entregue = True
                          WHERE id IN ({id});
                          COMMIT;
                       """.format(id=corpo_ids))
            cr.close()
            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': u'Itens entregues!'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_comanda_conferencia(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        nro_comanda = req.params.get('codigo_comanda') or ''
        if not nro_comanda:
            cr.close()
            return json.dumps({'codigo': COMANDA_NAO_ENCONTRADO,
                               'mensagem': u'Comanda não encontrada!',
                               'retorno': []})

        try:
            percentual = 0
            cr.execute("""SELECT supcfp_comissao
                          FROM supervisor_cfg_parametros""")
            parametro = dictfetchall(cr)
            if parametro:
                percentual = parametro[0].get('supcfp_comissao') or ''

            cr.execute("""SELECT scc.supcda_mesa             AS mesa,
                                 scc.supcda_codigo_comanda   AS comanda,
                                 scc.id                      AS capa_id,
                                 scc.supcda_total_comanda    AS vr_total_comanda,
                                 SUM(sco.supcdo_quantidade)  AS qtd,
                                 SUM(sco.supcdo_preco_total) AS preco,
                                 sci.name                      AS nome
                          FROM supervisor_cda_capa        AS scc
                          INNER JOIN supervisor_cda_corpo AS sco ON sco.supcdo_capa_id = scc.id
                          INNER JOIN supervisor_cad_itens AS sci ON sci.supite_codigo_item = sco.supcdo_item
                          WHERE scc.supcda_codigo_comanda = {comanda}
                            AND sco.supcdo_venda_cancelamento = 'V'
                            AND scc.supcda_restricao = 0
                          GROUP BY sci.name, scc.supcda_mesa, scc.supcda_codigo_comanda, scc.id, 
                                   scc.supcda_total_comanda
                          ORDER BY sci.name   
                               """.format(comanda=nro_comanda))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'codigo': COMANDA_SEM_ITENS,
                                   'mensagem': u'Comanda sem itens!',
                                   'retorno': []})

            cr.close()
            for i in dados:
                i['seq'] = str(i.get('seq') or '')
                i['qtd'] = str(i.get('qtd'))
                i['preco'] = str(i.get('preco'))
                i['mesa'] = str(i.get('mesa') or '')
                i['comanda'] = str(i.get('comanda'))
                i['vr_total_comanda'] = str(i['vr_total_comanda'])
                i['percentual'] = str(percentual or '0')

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u''})

        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_conferencia(self, req, s_action=None, **kw):
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            sock = MySock(req.params)
            impressora = req.params.get('impressora') or ''
            codigo_comanda = int(req.params.get('codigo_comanda') or 0)
            arquivo = 'conferencia_%s.txt' % str(datetime.now().strftime('%Y-%m-%d_%H:%M:%S'))
            if not impressora:
                cr.close()
                return json.dumps({'codigo': ERRO_INTERNO,
                                   'mensagem': u'Informar impressora de conferêmcia no APP',
                                   'retorno': []})

            texto = self.get_texto_comanda(cr, codigo_comanda)
            vals = {'impressora': impressora,
                    'texto': [texto],
                    'porta': '',
                    'comando_id': False}
            res = self.imprimir(sock, vals, arquivo)
            if not res[0]:
                raise Exception(res[1])

            dados = dictfetchall(cr)
            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': u'Impressão de Conferência enviada!'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': u'Erro: %s' % e,
                               'retorno': []})

    def get_texto_comanda(self, cr, nro_comanda):
        percentual = 0
        sql = """ SELECT supcfp_comissao
                  FROM supervisor_cfg_parametros"""
        cr.execute(sql)
        parametro = dictfetchall(cr)
        if parametro:
            percentual = parametro[0].get('supcfp_comissao')

        cr.execute("""SELECT id, 
                             supcda_data_hora,
                             supcda_restricao,
                             supcda_mesa,
                             supcda_total_comanda
                      FROM supervisor_cda_capa
                      WHERE supcda_codigo_comanda = %s
                        AND supcda_restricao = 0
                   """ % nro_comanda)
        comanda = dictfetchall(cr)
        if not comanda:
            raise Exception(u'Comanda não encontrada')

        comanda = comanda[0]
        cr.execute("""SELECT supcfc_cliente
                      FROM supervisor_cfg_configuracao
                      ORDER BY id""")
        condiguracao = dictfetchall(cr)[0]
        data = datetime.strptime(comanda.get('supcda_data_hora')[:19], '%Y-%m-%d %H:%M:%S')
        data = utl_converte_tz(data, 'UTC', 'America/Sao_Paulo').strftime('%d/%m/%Y %H:%M:%S')
        situacao = {-1: 'Cancelada',
                     0: 'Aberta',
                     1: 'Aguardando emissao no frente(Iniciou faturamento',
                     2: 'Faturada',
                     }.get(comanda.get('supcda_restricao'))

        data_impressao = utl_converte_tz(datetime.now(), 'UTC', 'America/Sao_Paulo'
                                       ).strftime('%d/%m/%Y %H:%M:%S')
        text = ['{b1}%s{b2}' % (condiguracao.get('supcfc_cliente')[:48].center(48)),
                ('DATA IMPRESSAO: %s' % data_impressao).rjust(48),
                '',
                '-----------* CONFERENCIA DE COMANDA *-----------',
                '',
                'NRO. COMANDA: {b1}%06d{b2}' % nro_comanda,
                'MESA: {b1}%03d{b2}' % (comanda.get('supcda_mesa') or 0),
                'DATA: %s' % data,
                'SITUACAO: %s' % situacao,
                '',
                'PRODUTO',
                ' QTD.                    VR.UNIT.        VALOR: ',
                '------------------------------------------------']
        cr.execute("""SELECT sci.supite_descricao_completa AS descricao,
                             SUM(scc.supcdo_preco_total)        AS preco,
                             SUM(scc.supcdo_quantidade)         AS qtd,
                             scc.supcdo_venda_cancelamento AS venda_cancelamento
                      FROM supervisor_cda_corpo       AS scc 
                      INNER JOIN supervisor_cad_itens AS sci ON sci.supite_codigo_item = scc.supcdo_item
                      WHERE scc.supcdo_capa_id = %s
                        AND scc.supcdo_venda_cancelamento = 'V' 
                      GROUP BY sci.supite_descricao_completa, scc.supcdo_venda_cancelamento
                      ORDER BY sci.supite_descricao_completa""" % comanda.get('id'))
        for i in dictfetchall(cr):
            if i.get('venda_cancelamento') == 'C':
                continue

            qtd = str(i.get('qtd')).rjust(3)
            vr_unit = ('%0.2f' % Decimal(i.get('preco') / i.get('qtd')).quantize(TWOPLACES,
                                                                     rounding=ROUND_DOWN)).rjust(6)
            preco = ('%0.2f' % Decimal(i.get('preco')).quantize(TWOPLACES, rounding=ROUND_DOWN)).rjust(7)
            text.extend([i.get('descricao'),
                         ' %sx                 R$ %s    R$ %s ' % (qtd.replace('.', ','),
                                                                   vr_unit.replace('.', ','),
                                                                   preco.replace('.', ','))])

        subtotal = str(comanda.get('supcda_total_comanda')).rjust(9).replace('.', ',')
        text.extend(['',
                     '------------------------------------------------',
                     '                         {b1}SUBTOTAL  R$ %s{b2}' %
                     subtotal])
        vr_percentual = 0
        if percentual:
            vr_percentual = Decimal(comanda.get('supcda_total_comanda') * percentual / 100
                                    ).quantize(TWOPLACES)
            str_percent = ('%.2f' % vr_percentual).rjust(9).replace('.', ',')
            text.extend(['                         {b1}+ %s%%  R$ %s{b2}' %
                         (str(percentual).replace('.', ','), str_percent)])
        else:
            text.extend([''])

        total = ('%.2f' % Decimal(Decimal(comanda.get('supcda_total_comanda')) + vr_percentual
                              ).quantize(TWOPLACES, rounding=ROUND_DOWN)).rjust(9).replace('.', ',')
        text.extend(['                         -----------------------',
                     '                            {b1}{s1}TOTAL  R$ %s{s2}{b2}' % (total),
                     '',
                     'DESENVOLVIDO POR:',
                     '{b1}Sig Solucoes em Informatica Ltda. (31)2106-1700{b2}',
                     '{corta}',  # corta papel
                     ])
        return '\n'.join(text)
