# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class cad_itens_view(osv.Model):
    '''
    View dos itens no supervisor para a API Avanco utilizar
    '''
    _name = "scanntech_itens_view"
    _auto = False
    _columns = {
        'codigo_barras': fields.char('Codigo de barras', size=14),
        'loja': fields.integer('Loja'),
        'codigo_interno': fields.char('Codigo retaguarda', size=20, select=True),
        'descricao_completa':fields.char('Descricao Completa', size=60),
        'descricao_reduzida': fields.char('Descricao', size=30, select=True),
        'fracionado': fields.boolean('Fracionado'),
        'pesavel': fields.boolean('Pesavel'),
        'ncm': fields.char('NCM', size=8),
        'preco': fields.float('Preco', digits=(10, 3)),
        'aliquota': fields.float('Aliquota', digits=(5, 3)),
        'tributacao': fields.char('Tipo de Tributacao', size=1),
        'incide_pis_cofins': fields.boolean('Incide pis/cofins'),
        'fator_multiplicador': fields.float('Falor multiplicador', digits=(11, 3)),
        'scanntech': fields.integer('Sacanntech', sequence=True),
        'log_erro': fields.text('Log'),
    }

    def init(self, cr):
        drop_view_if_exists(cr, "scanntech_itens_view")
        cr.execute("""
            CREATE OR REPLACE VIEW scanntech_itens_view AS(
                SELECT sci.supite_codigo_item                  AS codigo_barras,
                       (SELECT supcfc_loja
                        FROM supervisor_cfg_configuracao
                        LIMIT 1)                               AS loja,
                       sci.supite_codigo_retaguarda            AS codigo_interno,
                       sci.supite_descricao_completa           AS descricao_completa,
                       sci.name                                AS descricao_reduzida,
                       sci.supite_ncm                          AS ncm,
                       COALESCE(sci.supite_fracionado, False)  AS fracionado,
                       COALESCE(sci.supite_pesavel, False)     AS pesavel,
                       sci.supite_preco                        AS preco,
                       sct.suptri_aliquota                     AS aliquota,
                       sct.suptri_tipo_tributacao              AS tributacao,
                       False                                   AS incide_pis_cofins,
                       COALESCE(supite_fator_multiplicador, 1) AS fator_multiplicador,
                       sci.scanntech,
                       sci.log_erro
                FROM supervisor_cad_itens AS sci
                INNER JOIN supervisor_cad_tributacao AS sct ON sct.id = sci.supite_tributacao_id
            );
            CREATE RULE scanntech_itens_view_upd AS
            ON UPDATE TO scanntech_itens_view DO INSTEAD UPDATE supervisor_cad_itens
            SET scanntech = NEW.scanntech,
                log_erro = NEW.log_erro
            WHERE supite_codigo_item = NEW.codigo_barras and supite_loja = NEW.loja;
        """)
