# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from criar_triger import criar_triger
from openerp.addons.supervisor import print_log
from datetime import datetime, timedelta
import os
from decimal import Decimal
TWOPLACES = Decimal(10) ** -2
ZEROPLACES = Decimal(10) ** 0


class supervisor_mov_capa(osv.Model):
    """
    Tabela de Capa de cupons
    Prefixo: supcap

    """
    _inherit = 'supervisor.mov.capa'

    def _auto_init(self, cr, context=None):
        context = context and context.copy() or {}
        try:
            super(supervisor_mov_capa, self)._auto_init(cr, context)
            criar_triger(cr, 'mov_capa', context=context)
        except:
            pass

    def _formatar_numero(self, numero, decimal=2):
        return str(Decimal((numero or 0) * 10**decimal).quantize(ZEROPLACES))

    def cron_supcap_exporta_notas_pif(self, cr, uid, context=None):
        context = context and context.copy() or {}
        self.funcao_supcap_exporta_notas_pif(cr, uid, context)
        return True

    def funcao_supcap_exporta_notas_pif(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_par = self.pool.get('supervisor.cfg.parametros')
        par_id = obj_par.search(cr, uid, [], context=context)
        if not par_id:
            return True

        dados_par = obj_par.read(cr, uid, par_id[0], ['supcfp_dir_pif'], context)
        dir_pif = dados_par.get('supcfp_dir_pif')
        if not dir_pif:
            return True

        obj_export = self.pool.get('wizard.supervisor.reexportar.arq')
        export_id = context.get('supcap_exp_manual') and context.get('supcap_exp_manual').get('export_id') or None
        if not os.path.exists(dir_pif):
            msg = _("O diretorio '%s' nao existe.") % dir_pif
            print_log(msg)
            if context.get('supcap_exp_manual'):
                obj_export.write(cr, uid, export_id, {'wizexa_log': msg}, context)

            return True

        sql = """SELECT supcfc_cnpj
                 FROM supervisor_cfg_configuracao"""
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            msg = _("Sem configuracao castrada no Supervisor") % dir_pif
            print_log(msg)
            if context.get('supcap_exp_manual'):
                obj_export.write(cr, uid, export_id, {'wizexa_log': msg}, context)

            return True

        cnpj = dados[0].get('supcfc_cnpj')
        if not cnpj:
            msg = _("Sem CNPJ cadastrado nas configuracoes do Supervisor!") % dir_pif
            print_log(msg)
            if context.get('supcap_exp_manual'):
                obj_export.write(cr, uid, export_id, {'wizexa_log': msg}, context)

            return True

        partner_id = self.pool.get('res.users').read(cr, uid, uid, ['partner_id'], context,
                                                     '_classic_write').get('partner_id')
        context['tz'] = self.pool.get('res.partner').read(cr, uid, partner_id, ['tz'],
                                                          context).get('tz') or 'America/Sao_Paulo'
        self.exprotar_cupons_notas(cr, uid, dir_pif, cnpj, context)
        self.exportar_fechamento(cr, uid, dir_pif, context)
        return True

    def exprotar_cupons_notas(self, cr, uid, dir_pif, cnpj, context=None):
        context = context and context.copy() or {}
        obj_util = self.pool.get('supervisor.util')
        cond_data = 'supcap_data_cupom >= CURRENT_DATE - 15 AND'
        if context.get('supcap_exp_manual'):
            data_ini = datetime.strptime(context.get('supcap_exp_manual').get('ini'),
                                         "%Y-%m-%d").date()
            data_fim = datetime.strptime(context.get('supcap_exp_manual').get('ini'),
                                         "%Y-%m-%d").date()
            cond_data = "supcap_data_cupom >= '%s' AND supcap_data_cupom <= '%s' AND " % (data_ini, data_fim)

        sql = """SELECT id,
                        supcap_loja,
                        supcap_nro_cupom,
                        supcap_caixa,
                        supcap_tipo,         
                        supcap_total_cupom,
                        supcap_data_inicio,
                        supcap_state,      
                        supcap_chave_acesso,      
                        COALESCE(supcap_valor_desc_acres, 0) AS supcap_valor_desc_acres, 
                        supcap_alteracao 
                        
                 FROM supervisor_mov_capa
                 WHERE {cond_data} scanntech = 0
                   AND (supcap_state IN ('3', 'C')
                        OR (supcap_state IS NULL AND supcap_tipo IN ('V', 'C')))
                 ORDER BY supcap_nro_cupom, supcap_caixa
              """.format(cond_data=cond_data)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            return True

        print_log(_('Exportando NFCEs/Cupons para o PIF...'))
        capa_ids = ','.join(map(lambda x: '%s' % x.get('id'), dados))
        sql = """SELECT supfec_capa_id, 
                        COALESCE(supfec_valor, 0) AS supfec_valor, 
                        COALESCE(supfec_troco, 0) AS supfec_troco
                 FROM supervisor_mov_fechamentos 
                 WHERE supfec_capa_id IN (%s)
              """ % capa_ids
        cr.execute(sql)
        dict_fechamentos = {}
        for i in cr.dictfetchall():
            capa_id = i.get('supfec_capa_id')
            if not dict_fechamentos.get(capa_id):
                dict_fechamentos[capa_id] = [i]
            else:
                dict_fechamentos[capa_id].append(i)

        sql = """SELECT supcse_capa_id
                 FROM supervisor_mov_cupom_servidor
                 WHERE supcse_capa_id IN (%s)
              """ % capa_ids
        cr.execute(sql)
        dict_servidor = {}
        for i in cr.dictfetchall():
            capa_id = i.get('supcse_capa_id')
            if not dict_servidor.get(capa_id):
                dict_servidor[capa_id] = True

        sql = """SELECT supcor_capa_id,
                        supcor_item,
                        supcor_sequencia,
                        supcor_venda_cancelamento,
                        supcor_cancelado,
                        supcor_quantidade,
                        supcor_preco_unitario,
                        supcor_preco_total,
                        COALESCE(supcor_desconto_item, 0) AS supcor_desconto_item,
                        COALESCE(supcor_acrescimo_item, 0) AS supcor_acrescimo_item,
                        COALESCE(supcor_desconto_item_sem_rat, 0) AS supcor_desconto_item_sem_rat,
                        COALESCE(supcor_acrescimo_item_sem_rat, 0) AS supcor_acrescimo_item_sem_rat,
                        supcor_id_promo,
                        supcor_tipo_promo,
                        supcor_unidade,
                        supcor_nome_item,
                        supcor_tipo_tributacao_icms,
                        supcor_aliquota_icms
                        
                 FROM supervisor_mov_corpos
                 WHERE supcor_capa_id IN (%s)
                 ORDER BY supcor_capa_id, supcor_sequencia
              """ % capa_ids
        cr.execute(sql)
        dict_corpos = {}
        for i in cr.dictfetchall():
            capa_id = i.get('supcor_capa_id')
            if not dict_corpos.get(capa_id):
                dict_corpos[capa_id] = [i]
            else:
                dict_corpos[capa_id].append(i)

        for capa in dados:
            try:
                # se nao tem cupom servidor, ainda nao terminou de ser exportada
                if not dict_servidor.get(capa.get('id')):
                    continue

                loja = int(str(capa.get('supcap_loja'))[-2:])
                caixa = int(str(capa.get('supcap_caixa'))[-2:])
                fechamentos = dict_fechamentos.get(capa.get('id')) or []
                corpos = dict_corpos.get(capa.get('id')) or []
                nome_arq = 'OL%02d%06d.%02d' % (loja, capa.get('supcap_nro_cupom'), caixa)
                desc_subtotal = '0'
                acres_subtotal = '0'
                total_venda = self._formatar_numero(capa.get('supcap_total_cupom'))
                if capa.get('supcap_valor_desc_acres'):
                    if capa.get('supcap_alteracao') == 'D':
                        desc_subtotal = self._formatar_numero(capa.get('supcap_valor_desc_acres'))
                        total_venda = self._formatar_numero(capa.get('supcap_total_cupom') +
                                                            (capa.get('supcap_valor_desc_acres') or 0))
                    else:
                        acres_subtotal = self._formatar_numero(capa.get('supcap_valor_desc_acres'))
                        total_venda = self._formatar_numero(capa.get('supcap_total_cupom') -
                                                            (capa.get('supcap_valor_desc_acres') or 0))

                qtd_itens = 0
                desc_total_itens = 0
                for corpo in corpos:
                    if corpo.get('supcor_venda_cancelamento') == 'C' and capa.get('supcap_tipo') == 'V' \
                            or corpo.get('supcor_cancelado'):
                        continue

                    qtd_itens += 1
                    desc_total_itens += corpo.get('supcor_desconto_item_sem_rat')

                desc_total_itens = self._formatar_numero(desc_total_itens)
                valor_pago = 0
                troco = 0
                for i in fechamentos:
                    valor_pago += i.get('supfec_valor')
                    troco += i.get('supfec_troco')

                valor_pago = self._formatar_numero(valor_pago)
                troco = self._formatar_numero(troco)
                data_cupom = datetime.strptime(capa.get('supcap_data_inicio')[:19], '%Y-%m-%d %H:%M:%S')
                data_cupom = datetime.strptime(self.pool.get('supervisor.util').suputi_datetime_to_timezone(cr, uid,
                                                     data_cupom, formato='%Y-%m-%d %H:%M:%s')[:19], '%Y-%m-%d %H:%M:%S')
                hora_cupom = data_cupom.strftime('%H%M')
                data_cupom = data_cupom.strftime('%Y%m%d')
                if not os.path.exists(dir_pif + data_cupom):
                    os.mkdir(dir_pif + data_cupom)

                arquivo = ''
                if capa.get('supcap_tipo') == 'C':
                    arquivo += ('3').rjust(2, '0')  # tipo do registro
                    arquivo += ('EXCLUIDO').rjust(8, ' ')  # Texto
                    arquivo += (' ').rjust(6, ' ')  # Código Motivo
                    arquivo += (' ').rjust(2, ' ')  # Tipo Motivo
                    arquivo += (' ').rjust(50, ' ')  # Descrição Motivo
                    arquivo += (' ').rjust(10, ' ')  # Código de quem fez o cancelamento
                    arquivo += (' ').rjust(30, ' ')  # Nome de quem fez o cancelamento
                    arquivo += '\n'

                if capa.get('supcap_state'):
                    arquivo += ('51').rjust(2, '0')  # tipo do registro
                    arquivo += ('NFCE').rjust(4, ' ')  # NFCE
                    arquivo += (capa.get('supcap_chave_acesso') or '').rjust(44, ' ')  # Chave da NFCe
                    arquivo += ('_').rjust(1, ' ')  # undeline
                    arquivo += str(capa.get('supcap_caixa')).rjust(3, '0')  # Série Nota
                    arquivo += '\n'

                hora_geracao = obj_util.suputi_converte_tz(cr, uid, datetime.now(), tz_origem='UTC',
                                                           tz_destino=context.get('tz'),context=context)
                arquivo += ('0').rjust(2, '0')  # tipo do registro
                arquivo += ('0').rjust(14, '0')  # CPF/CNPJ do cliente
                arquivo += ('0').rjust(3, '0')  #  Vendedor
                arquivo += desc_subtotal.rjust(9, '0')  # Desconto no subtotal
                arquivo += total_venda.rjust(12, '0')  # Valor total da venda
                arquivo += str(qtd_itens).rjust(3, '0')  # Total de itens
                arquivo += desc_total_itens.rjust(9, '0')  # Desconto nos itens
                arquivo += troco.rjust(9, '0')  # Troco
                arquivo += ('0').rjust(6, '0')  # Livre
                arquivo += data_cupom.rjust(8, '0')  # Data do Cupom AAAAMMDD
                arquivo += hora_cupom.rjust(4, '0')  # Hora do Cupom HHMM
                arquivo += str(caixa).rjust(2, '0')  # Número do PDV
                arquivo += acres_subtotal.rjust(9, '0')  # Acréscimo subtotal
                arquivo += ('58').rjust(2, '0')  # Versão / Layout
                arquivo += ('0').rjust(19, '0')  # Número Cartão
                arquivo += (' ').rjust(2, ' ')  # Nro de parcelas
                arquivo += (' ').rjust(1, ' ')  # Contra-vale
                arquivo += (' ').rjust(15, ' ')  # Operador
                arquivo += (' ').rjust(13, ' ')  # Número do DAV
                arquivo += (' ').rjust(15, ' ')  # Marca do ECF
                arquivo += (' ').rjust(20, ' ')  # Modelo do ECF
                arquivo += ('0').rjust(9, '0')  # Valor Saque TEF
                arquivo += (' ').rjust(6, ' ')  # Versão PDV
                arquivo += (' ').rjust(15, ' ')  # Nome Operador
                arquivo += (' ').rjust(10, ' ')  # Número PV
                arquivo += (' ').rjust(10, ' ')  # Número Controle
                arquivo += (' ').rjust(40, ' ')  # Número do documento
                arquivo += data_cupom.rjust(8, '0')  # Data Movimento AAAAMMDD
                arquivo += (' ').rjust(14, ' ')  # Número Controle de conta atrelado ao Cupom
                arquivo += ('0').rjust(10, '0')  # Val. Aprox. dos Tributos
                arquivo += (' ').rjust(19, ' ')  # Consulta InforCard (Somente Linux)
                arquivo += (' ').rjust(12, ' ')  # Pontos Dotz total do Cupom
                arquivo += ('0').rjust(9, '0')  # Troco Solidário
                arquivo += ('N').rjust(1, '0')  # Desconto ScannPlus S/N .De desconto veio da escanntech ou nao
                arquivo += str(capa.get('supcap_nro_cupom')).rjust(6, '0')  # CCF do Cupom Fiscal/NFCE
                arquivo += hora_geracao.strftime('%H%M%S').rjust(6, '0')  # Hora Criação do Arquivo HHMMSS
                arquivo += cnpj[:14].rjust(14, '0')  # CNPJ da Loja
                arquivo += (' ').rjust(10, ' ')  # Código Operador Liberação Desconto/Acréscimo Subtotal
                arquivo += (' ').rjust(30, ' ')  # Nome Operador Liberação Desconto/Acréscimo Subtotal
                arquivo += ('0').rjust(11, '0')  # CPF Sócio Torcedor
                arquivo += (' ').rjust(6, ' ')  # Código Motivo
                arquivo += (' ').rjust(2, ' ')  # Tipo Motivo
                arquivo += (' ').rjust(50, ' ')  # Descrição Motivo
                arquivo += ('0').rjust(11, '0')  # CPF Beblue
                arquivo += ('0').rjust(9, '0')  # Troco Simples
                arquivo += ('0').rjust(11, '0')  # CPF Dotz
                arquivo += '\n'

                # --------- Fechamento -------------------
                arquivo += ('1').rjust(2, '0')  # Tipo do Registro
                arquivo += ('0').rjust(3, '0')  # Código Plano Pagamento
                arquivo += (' ').rjust(20, ' ')  # Descrição do Plano Pagamento
                arquivo += ('0001').rjust(4, '0')  # Código Forma Pagamento(0001) Dinheriro, outros cartao
                arquivo += (' ').rjust(30, ' ')  # Descrição da Forma Pagamento
                arquivo += ('0').rjust(8, '0')  # Data Vencimento
                arquivo += valor_pago.rjust(9, '0')  # Valor Pago
                arquivo += (' ').rjust(20, ' ')  # Documento
                arquivo += (' ').rjust(50, ' ')  # Número do Docto
                arquivo += (' ').rjust(2, ' ')  # Número de Parcelas
                arquivo += (' ').rjust(20, ' ')  # Bandeira
                arquivo += ('0').rjust(11, '0')  # Valor da Parcela
                arquivo += (' ').rjust(50, ' ')  # Texto real da Modalidade de Pgto.
                arquivo += (' ').rjust(6, ' ')  # NSU Sitef
                arquivo += (' ').rjust(15, ' ')  # NSU Host
                arquivo += (' ').rjust(6, ' ')  # BIN
                arquivo += (' ').rjust(15, ' ')  # Autorização Crédito
                arquivo += (' ').rjust(110, ' ')  # Produto Sitef
                arquivo += (' ').rjust(20, ' ')  # Documento Digitado
                arquivo += ('0').rjust(6, '0')  # Pontos Dotz da Forma de Pagamento
                arquivo += ('0').rjust(10, '0')  # Código Cliente Integral
                arquivo += (' ').rjust(10, ' ')  # Código Operador Liberação Impedimento
                arquivo += (' ').rjust(30, ' ')  # Nome Operador Liberação Impedimento
                arquivo += ('0').rjust(2, '0')  # Tipo Impedimento
                arquivo += ('0').rjust(12, '0')  # Valor Ultrapassado
                arquivo += '\n'

                # --------- Corpos -------------------
                for corpo in corpos:
                    if corpo.get('supcor_venda_cancelamento') == 'C' and capa.get('supcap_tipo') == 'V' or \
                            corpo.get('supcor_cancelado'):
                        continue

                    qtd = self._formatar_numero(corpo.get('supcor_quantidade'), 3)
                    vr_unitario = self._formatar_numero(corpo.get('supcor_preco_unitario'), 3)
                    vr_bruto_item = corpo.get('supcor_preco_total')
                    desc_item = corpo.get('supcor_desconto_item_sem_rat')
                    acres_item = corpo.get('supcor_acrescimo_item_sem_rat')
                    vr_liquido = self._formatar_numero(vr_bruto_item - desc_item + acres_item, 3)
                    vr_bruto_item = self._formatar_numero(vr_bruto_item)
                    desc_item_rat = self._formatar_numero(corpo.get('supcor_desconto_item') -
                                                          desc_item, 4)
                    acres_item_rat = self._formatar_numero(corpo.get('supcor_acrescimo_item') -
                                                           acres_item, 4)
                    desc_item = self._formatar_numero(desc_item)
                    acres_item = self._formatar_numero(acres_item)
                    id_promo = '0'
                    tipo_promo = corpo.get('supcor_tipo_promo') == 1 and 'S' or ' '
                    if tipo_promo == 'S':  # Scanntech
                        id_promo = str(corpo.get('supcor_id_promo') or 0)

                    tipo = '2' or '4'  # 4 usado apenas para ECF
                    sit_trib = (corpo.get('supcor_tipo_tributacao_icms') or '').split()[0]
                    if sit_trib == 'T':
                        sit_trib = 'T%02d' % int((corpo.get('supcor_aliquota_icms') or 0))
                    elif sit_trib == 'U':
                        sit_trib = 'F'
                    elif sit_trib == 'O':
                        sit_trib = 'N'

                    arquivo += tipo.rjust(2, '0')  # Tipo do Registro
                    arquivo += str(capa.get('supcap_caixa')).ljust(25, ' ')  # Nro série do ECF
                    arquivo += str(capa.get('supcap_nro_cupom')).rjust(6, '0')  # Nro do cupom fiscal/NFCE
                    arquivo += str(corpo.get('supcor_sequencia')).rjust(3, '0')  # Nro do item(sequencia)
                    arquivo += corpo.get('supcor_item').rjust(14, '0')  # Código do produto
                    arquivo += qtd.rjust(8, '0')  # Quantidade
                    arquivo += vr_unitario.rjust(10, '0')  # Preço unitário
                    arquivo += vr_bruto_item.rjust(13, '0')  # Valor total do item
                    arquivo += ('%s' % sit_trib).ljust(3, ' ')  # Situação Tributária
                    arquivo += data_cupom.rjust(8, '0')  # Data da Venda AAAAMMDD
                    arquivo += str(caixa).rjust(2, '0')  # Número do PDV(Caixa)
                    arquivo += ('P').rjust(1, ' ')  # Tipo do item
                    arquivo += (' ').rjust(20, ' ')  # Livre1
                    arquivo += (' ').rjust(20, ' ')  # Livre2
                    arquivo += ('0').rjust(4, '0')  # Alíquota Real
                    arquivo += ('0').rjust(4, '0')  # Alíquota Efetiva
                    arquivo += corpo.get('supcor_item')[-6:].rjust(6, '0')  # *Código Reduzido/Interno
                    arquivo += (' ').rjust(14, ' ')  # Código Alternativo
                    arquivo += vr_liquido.rjust(10, '0')  # Preço real do item
                    arquivo += desc_item.rjust(10, '0')  # Valor do desconto no item
                    arquivo += (' ').rjust(1, ' ')  # Venda no preço promocional
                    arquivo += (' ').rjust(7, ' ')  # Registrador da Alíquota no ECF
                    arquivo += (' ').rjust(4, ' ')  # Dado complementar
                    arquivo += acres_item.rjust(10, '0')  # Acréscimo Item
                    arquivo += (' ').rjust(10, ' ')  # Código do Usuário cancelamento
                    arquivo += (' ').rjust(30, ' ')  # Nome Usuário cancelamento
                    arquivo += ('0').rjust(4, '0')  # Alíquota IBPT
                    arquivo += ('0').rjust(10, '0')  # Valor Aproximado do Tributo
                    arquivo += (' ').rjust(1, ' ')  # Sócio Torcedor
                    arquivo += (' ').rjust(1, ' ')  # Promoção Especial
                    arquivo += (' ').rjust(6, ' ')  # Pontos Dotz por Item
                    arquivo += id_promo.rjust(6, '0')  # ID da Promoção Scanntech/Avanço
                    arquivo += corpo.get('supcor_unidade').ljust(3, ' ')  # Unidade de medida
                    arquivo += ('0').rjust(10, '0')  # Quantidade Promoção
                    arquivo += (corpo.get('supcor_nome_item')[:20] or '').ljust(20, ' ')  # Descrição resumida do produto
                    arquivo += ('0001').rjust(4, '0')  # Fator de Conversão
                    arquivo += tipo_promo.rjust(1, '0')  # Tipo Promoção 'S'-Scanntech ou ' '
                    arquivo += (' ').rjust(4, ' ')  # Código Tabela de Preços/Atacado
                    arquivo += (' ').rjust(10, ' ')  # Código Operador Liberação Desconto/Acréscimo Item
                    arquivo += (' ').rjust(30, ' ')  # Nome Operador Liberação Desconto/Acréscimo Item
                    arquivo += desc_item_rat.rjust(10, '0')  # Desconto Rateado
                    arquivo += acres_item_rat.rjust(10, '0')  # Acréscimo Rateado
                    arquivo += ('0').rjust(10, '0')  # Base Cálculo
                    arquivo += ('0').rjust(10, '0')  # Imposto Calculado
                    arquivo += vr_bruto_item.rjust(10, '0')  # Total Bruto
                    arquivo += ('0').rjust(4, '0')  # Alíquota % PIS
                    arquivo += ('0').rjust(10, '0')  # Imposto PIS
                    arquivo += ('0').rjust(4, '0')  # Alíquota % COFINS
                    arquivo += ('0').rjust(10, '0')  # Imposto COFINS
                    arquivo += (' ').rjust(4, ' ')  # CFOP
                    arquivo += (' ').rjust(2, ' ')  # CST do PIS
                    arquivo += (' ').rjust(2, ' ')  # CST do COFINS
                    arquivo += (' ').rjust(8, ' ')  # NCM
                    arquivo += (' ').rjust(7, ' ')  # CEST
                    arquivo += (' ').rjust(6, ' ')  # Código Motivo
                    arquivo += (' ').rjust(2, ' ')  # Tipo Motivo
                    arquivo += (' ').rjust(50, ' ')  # Descrição Motivo
                    arquivo += ('0').rjust(20, '0')  # ERP_Supervarejista
                    arquivo += '\n'

                arquivo += ('FIM').rjust(3, ' ')  # Tipo do Registro
                open(dir_pif + data_cupom + '/' + 'carregando_' + nome_arq, 'w').write(arquivo)
                # muda o nome do arquivo para nao correr o risco do PIF pegar o arquivo antes de
                # terminar a gracacao dos registros
                os.rename(dir_pif + data_cupom + '/' + 'carregando_' + nome_arq, dir_pif + data_cupom + '/' + nome_arq)
                sql = """UPDATE supervisor_mov_capa
                         SET scanntech = 1 
                         WHERE id = %s;
                         COMMIT;                         
                      """ % capa.get('id')
                cr.execute(sql)
            except Exception as e:
                try:
                    msg = _("Erro ao exportar a NFCE/ECF: %s Serie: %s\n%s") % \
                          (capa.get('supcap_nro_cupom'), capa.get('supcap_caixa'), e)
                except:
                    path = '/tmp/erro_%s_%s.txt' % (capa.get('supcap_nro_cupom'), capa.get('supcap_caixa'))
                    msg = _("Erro ao exportar a NFCE/ECF: %s Serie: %s gravado no arquivo '%s'") % \
                          (capa.get('supcap_nro_cupom'), capa.get('supcap_caixa'),
                           path)
                    open(path, 'w').write(e.message)

                print_log(msg)
                if context.get('supcap_exp_manual'):
                    self.pool.get('wizard.supervisor.reexportar.arq').write(cr, uid,
                                        context.get('supcap_exp_manual').get('export_id'), {'wizexa_log': msg}, context)

        return True

    def exportar_fechamento(self, cr, uid, dir_pif, context=None):
        """Exporta o fechamento do dia que esta informado no PDV 'suppdv_data_scanntech',
           mas nao no mesmo dia que o cron estiver rodando. Caso nao tenha data infromada,
           pega o dia anterior. O fechamento sera enviado apenas se todas a notas do dia ja tiverem
           sido enviadas"""
        context = context and context.copy() or {}
        obj_util = self.pool.get('supervisor.util')
        obj_ecf = self.pool.get('supervisor.cfg.ecf.autorizada')
        sql = """SELECT id, 
                        suppdv_loja,
                        suppdv_caixa,
                        suppdv_data_scanntech        
                 FROM supervisor_cad_pdvs
                 ORDER BY suppdv_loja, suppdv_caixa
              """
        cr.execute(sql)
        data_atual = obj_util.suputi_converte_tz(cr, uid, datetime.now(), tz_origem='UTC',
                                                       tz_destino=context.get('tz'),context=context).date()
        datas = []
        if context.get('supcap_exp_manual') and \
                context.get('supcap_exp_manual').get('fechamento'):
            data_aux = datetime.strptime(context.get('supcap_exp_manual').get('ini'),
                                         "%Y-%m-%d").date()
            while True:
                datas.append(str(data_aux))
                data_aux += timedelta(days=1)
                if data_aux >= data_atual \
                        or data_aux > datetime.strptime(
                    context.get('supcap_exp_manual').get('fim'), "%Y-%m-%d").date():
                    break

        for pdv in cr.dictfetchall():
            if not datas:
                if pdv.get('suppdv_data_scanntech') and \
                        pdv.get('suppdv_data_scanntech') < str(datetime.now().date() - timedelta(days=15)):
                    pdv['suppdv_data_scanntech'] = str(datetime.now().date() - timedelta(days=15))

                datas = [str(pdv.get('suppdv_data_scanntech') or (data_atual - timedelta(days=1)))]

            for data in datas:
                if data >= str(data_atual):
                    continue

                loja = pdv.get('suppdv_loja')
                caixa = pdv.get('suppdv_caixa')
                sql = """SELECT id                        
                         FROM supervisor_mov_capa
                         WHERE supcap_data_cupom = '{data}' 
                           AND supcap_caixa = {caixa}
                           AND scanntech = %s
                           AND (supcap_state IN ('3', '4', '5', 'C', '2E')
                                OR (supcap_state IS NULL AND supcap_tipo IN ('V', 'C')))
                           AND supcap_loja = {loja}
                         ORDER BY supcap_nro_cupom, supcap_caixa
                      """.format(data=data, loja=loja, caixa=caixa)
                cr.execute(sql % 0)
                # enquanto tiver cupom para ser exportado nao envia o fechamento
                if cr.dictfetchall():
                    continue
                else: # caso nao tenha movimento no dia, nao gera arquivo
                    cr.execute(sql % 1)
                    if not cr.dictfetchall():
                        data = (datetime.strptime(data, '%Y-%m-%d') + timedelta(days=1)).date()
                        self.pool.get('supervisor.cad.pdvs').write(cr, uid, pdv.get('id'),
                                                                   {'suppdv_data_scanntech': data}, context)
                        continue

                context['wizrte_nao_considera_dav'] = True
                dados = self.pool.get('wizard.relatorio.tesouraria').get_dados_fechamento(cr, uid,
                                                                     data, loja, caixa, [], context)

                data = (datetime.strptime(data, '%Y-%m-%d') + timedelta(days=1)).date()
                str_data = (data - timedelta(days=1)).strftime("%Y%m%d")
                if not os.path.exists(dir_pif + str_data):
                    os.mkdir(dir_pif + str_data)

                if not context.get('supcap_exp_manual'):
                    self.pool.get('supervisor.cad.pdvs').write(cr, uid, pdv.get('id'),
                                                           {'suppdv_data_scanntech': data}, context)
                dados = dados[0]
                n_serie = False
                ecf_ids = obj_ecf.search(cr, uid, [('supcfa_loja', '=', loja),
                                                   ('supcfa_caixa', '=', caixa)],context=context)
                gt_inicial = 0
                gt_final = dados.get('venda_liquida') + (dados.get('descontos') or 0) + (dados.get('cancelamentos') or 0)

                if ecf_ids:
                    n_serie = obj_ecf.read(cr, uid, ecf_ids[0], ['supcfa_nserie'],context
                                           ).get('supcfa_nserie') or False
                    sql = """SELECT corpo.supcxz_gt_atual,
                                    corpo.supcxz_gt_anterior
                             FROM supervisor_mov_capa AS capa
                             INNER JOIN supervisor_mov_corpo_xz AS corpo ON corpo.supcxz_capa_id = capa.id
                             WHERE capa.supcap_tipo = 'Z'
                               AND capa.supcap_data_cupom = '{data}'
                          """.format(data=str_data)
                    cr.execute(sql)
                    dados_z = cr.dictfetchall()
                    if not dados_z:
                        msg = _('Cupom "Z" faltante! Dia: %s Caixa: %s Loja: %s') % \
                              ((data-timedelta(days=1)).strftime('%d-%m-%Y'), caixa, loja)
                        print_log(msg)
                        if context.get('supcap_exp_manual'):
                            export_id = context.get('supcap_exp_manual').get('export_id')
                            obj_export = self.pool.get('wizard.supervisor.reexportar.arq')
                            dados = obj_export.read(cr, uid, export_id, ['wizexa_log'], context)
                            log = (dados[0].get('wizexa_log') or '') + '\n' + msg
                            obj_export.write(cr, uid, export_id, {'wizexa_log': log}, context)

                        continue

                    gt_final = dados_z[0].get('supcxz_gt_atual') or 0
                    gt_inicial = dados_z[0].get('supcxz_gt_anterior') or 0

                gt_inicial = self._formatar_numero(gt_inicial, 2)
                gt_final = self._formatar_numero(gt_final, 2)
                cancelamentos = self._formatar_numero(dados.get('cancelamentos'), 2)
                acrescimos = self._formatar_numero(dados.get('acrescimos'), 2)
                descontos = self._formatar_numero(dados.get('descontos'), 2)

                dia = (data - timedelta(days=1)).day
                mes = (data - timedelta(days=1)).month
                loja = int(str(loja)[-2:])
                nome_arq = 'OLZZ%02d%02d%02d.%02d' % (dia, mes, loja, int(str(caixa)[-2:]))
                arquivo = ''
                if not ecf_ids:
                    arquivo += ('90').rjust(2, '0')  # *Tipo do registro
                    arquivo += ('NFCE').ljust(7, ' ')  # SAT / NFCE
                    arquivo += str_data.rjust(8, '0')  # Data Movimento
                    arquivo += obj_util.suputi_converte_tz(cr, uid, datetime.now(), tz_origem='UTC',
                                                           tz_destino=context.get('tz'),context=context
                                                           ).strftime('%H%M%S').rjust(6, '0')  # Hora Geração
                    arquivo += '\n'

                arquivo += ('0').rjust(2, '0')  # *Tipo do registro
                arquivo += str(caixa)[-2:].rjust(2, '0')  # *Número do PDV
                arquivo += ('0').rjust(6, '0')  # *Cupom inicial
                arquivo += ('0').rjust(6, '0')  # Cupom final
                arquivo += gt_inicial.rjust(21, '0')  # *GT. Inicial
                arquivo += gt_final.rjust(21, '0')  # *GT. Final
                arquivo += cancelamentos.rjust(13, '0')  # *Cancelamentos
                arquivo += acrescimos.rjust(13, '0')  # *Acréscimos
                arquivo += descontos.rjust(13, '0')  # *Descontos
                arquivo += str(loja).rjust(2, '0')  # *Número da loja
                arquivo += ('0').rjust(6, '0')  # *Contador de Reduções
                arquivo += ('0').rjust(36, '0')  # *LIVRE
                arquivo += str(n_serie or caixa).ljust(25, ' ')  # *Número de Série
                arquivo += ('0').rjust(6, '0')  # Versão PDV
                arquivo += ('0').rjust(6, '0')  # *Contador de Reinicio de Operações
                arquivo += str_data.rjust(8, '0')  # *Data Movimento
                arquivo += '\n'

                arquivo += ('1').rjust(2, '0')  # *Tipo do Registro
                arquivo += ('I').rjust(1, '0')  # *Isento (Fixo=I)
                arquivo += ('0').rjust(13, '0')  # *Total Isento
                arquivo += ('N').rjust(1, '0')  # *Não Tributado(Fixo=N)
                arquivo += ('0').rjust(13, '0')  # *Total não Tributado
                arquivo += ('F').rjust(1, '0')  # *Substituição
                arquivo += ('0').rjust(13, '0')  # *Total Substituição
                arquivo += '\n'
                arquivo += ('FIM').rjust(3, ' ')  # Tipo do Registro
                open(dir_pif + str_data + '/' + 'carregando_' + nome_arq, 'w').write(arquivo)
                # muda o nome do arquivo para nao correr o risco do PIF pegar o arquivo antes de
                # terminar a gracacao dos registros
                os.rename(dir_pif + str_data + '/' + 'carregando_' + nome_arq, dir_pif + str_data + '/' + nome_arq)

        return True