# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class mov_capa_view(osv.Model):
    '''
    View das capas do cupom no supervisor para a API Avanco utilizar
    '''
    _name = "scanntech_capa_view"
    _auto = False
    _columns = {
        'id': fields.integer('Id'),
        'data_cupom':  fields.date('data cupom'),
        'hora_cupom':  fields.datetime('hora cupom'), #apenas a hora
        'quantidade_itens':fields.integer('Quantidade de itens'), # quantidade de linhas de itens no cupom
        'numero_serie':fields.char('nro serie', size=20),
        'loja': fields.integer('loja', required=True),
        'caixa': fields.integer('caixa', required=True),
        'total': fields.float('total cupom', digits=(10, 2)),
        'desconto_subtotal': fields.float('valor desc subtotal', digits=(10, 2)),
        'acrescimo_subtotal': fields.float('valor acres subtotal', digits=(10, 2)),
        'coo': fields.integer('coo'),
        'ccf': fields.integer('ccf'),
        'tipo': fields.selection([('C', 'Cancelamento'),
                                  ('V', 'Venda'),
                                  ('Z', 'Reducao Z'),
                                  ], 'tipo'),
        'scanntech': fields.integer('Sacanntech', sequence=True),
        'log_erro': fields.text('Log'),
    }

    def init(self, cr):
        drop_view_if_exists(cr, "scanntech_capa_view")
        cr.execute("""
            CREATE OR REPLACE VIEW scanntech_capa_view AS(
                SELECT capa.id,
                       capa.supcap_data_cupom                AS data_cupom,
                       cast((supcap_data_inicio -
                            (supcap_data_inicio - supcap_data_inicio at TIME zone 'UTC')) as time) AS hora_cupom,
                       (SELECT count(id)
                        FROM supervisor_mov_corpos
                        WHERE supcor_capa_id = capa.id)      AS quantidade_itens,
                       capa.supcap_nro_serie                 AS numero_serie,
                       capa.supcap_loja                      AS loja,
                       capa.supcap_caixa                     AS caixa,
                       capa.supcap_total_cupom               AS total,
                       capa.supcap_nro_cupom                 AS coo,
                       capa.supcap_ccf                       AS ccf,
                       capa.supcap_tipo                      AS tipo,
                       CASE WHEN capa.supcap_alteracao = 'D'
                            THEN capa.supcap_valor_desc_acres
                            ELSE 0
                       END                                   AS desconto_subtotal,
                       CASE WHEN capa.supcap_alteracao = 'A'
                            THEN capa.supcap_valor_desc_acres
                            ELSE 0
                       END                                   AS acrescimo_subtotal,
                       capa.scanntech,
                       capa.log_erro
                FROM supervisor_mov_capa AS capa
                WHERE capa.supcap_tipo IN ('C','V','Z')
            );

            CREATE RULE scanntech_capa_view_upd AS
            ON UPDATE TO scanntech_capa_view DO INSTEAD UPDATE supervisor_mov_capa
            SET scanntech = NEW.scanntech,
                log_erro = NEW.log_erro
            WHERE id = NEW.id;
        """)
