# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################


from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


class supervisor_mov_corpos_view(osv.Model):
    """
    View dos corpos no supervisor para a API Avanco utilizar
    """
    _name = "scanntech_corpos_view"
    _auto = False
    _columns = {
        'id': fields.integer('Id', required=True),
        'capa_id': fields.many2one("supervisor.mov.capa", 'Capa', ondelete="cascade",
                                          required=True, select=True),
        'id_scanntech': fields.integer('Id Promocao'),
        'codigo_barras': fields.char('Item', size=14),
        'descricao_reduzida': fields.char('Descricao reduzida', size=20),
        'codigo_interno': fields.char('Codigo retaguarda', size=20, select=True),
        'quantidade': fields.float('Quantidade', digits=(10, 3)),
        'quantidade_promocao': fields.float('Quantidade Promocao', digits=(10, 3)),
        'preco_unitario': fields.float('Preco unitario', digits=(10, 3)),
        'preco_total': fields.float('Preco total', digits=(10, 3)), # preco bruto do item sem desconto
                                                                           # ou acrescimo
        'desconto_item': fields.float('Desconto item', digits=(10, 3)),
        'acrescimo_item': fields.float('Acrescimo item', digits=(10, 3)),
        'venda_cancelamento': fields.selection([('V', 'Venda'), ('C', 'Cancelamento')],
                                                      'Venda cancelamento'),
        'fator_multiplicador': fields.float('Fator multiplicador', digits=(11, 3)),
        'scanntech': fields.integer('Sacanntech', sequence=True),
    }

    def init(self, cr):
        drop_view_if_exists(cr, "scanntech_corpos_view")
        cr.execute("""
            CREATE OR REPLACE VIEW scanntech_corpos_view AS(
                SELECT smc.supcor_sequencia                        AS id,
                       smc.supcor_capa_id                          AS capa_id,
                       smc.supcor_item                             AS codigo_barras,
                       cast(sci.name as varchar(20))               AS descricao_reduzida,
                       sci.supite_codigo_retaguarda                AS codigo_interno,
                       (CASE WHEN smc.supcor_tipo_promo = '1'
                             THEN smc.supcor_id_promo
                             ELSE NULL
                        END)                                       AS id_scanntech,
                       smc.supcor_quantidade                       AS quantidade,
                       (CASE WHEN smc.supcor_tipo_promo = '1'
                             THEN smc.supcor_quantidade_promo
                             ELSE 0.0
                       END)                                        AS quantidade_promocao,
                       smc.supcor_preco_unitario                   AS preco_unitario,
                       smc.supcor_preco_total                      AS preco_total,
                       smc.supcor_desconto_item_sem_rat            AS desconto_item,
                       smc.supcor_acrescimo_item_sem_rat           AS acrescimo_item,
                       smc.supcor_venda_cancelamento               AS venda_cancelamento,
                       COALESCE(sci.supite_fator_multiplicador, 1) AS fator_multiplicador,
                       smc.scanntech
                FROM supervisor_mov_corpos AS smc
                LEFT JOIN supervisor_cad_itens AS sci ON sci.supite_codigo_item = smc.supcor_item
                );

            CREATE RULE scanntech_corpos_view_upd AS
            ON UPDATE TO scanntech_corpos_view DO INSTEAD UPDATE supervisor_mov_corpos SET scanntech = NEW.scanntech
            WHERE supcor_capa_id = NEW.capa_id
              and supcor_sequencia = NEW.id
              and supcor_item = NEW.codigo_barras;
        """)
