#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from datetime import datetime


class supervisor_cad_sorteio(osv.Model):

    _inherit = 'supervisor.cad.sorteio'

    def create(self, cr, user, vals, context=None):
        self.verifica_valor(vals, context)
        return super(supervisor_cad_sorteio, self).create(cr, user, vals, context)

    def write(self, cr, user, ids, vals, context=None):
        if 'supsor_valor' in vals:
            self.verifica_valor(vals, context)

        return super(supervisor_cad_sorteio, self).write(cr, user, ids, vals, context)

    def button_supsor_esboco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': '1'})
        return True

    def button_supsor_ativar(self, cr, uid, ids, context=None):
        """
        Ativa o sorteio 
        """
        context = context and context.copy() or {}
        for dados in self.read(cr, uid, ids, ['supsor_inicio', 'supsor_data_fim', 'supsor_valor'],
                               context=context):
            self.verifica_datas(cr, uid, dados, context)
            self.verifica_valor(dados, context)
            self.write(cr, uid, dados.get('id'), {'state': '2'}, context=context)

        return True

    def verifica_datas(self, cr, uid, dados, onchange=False, context=None):
        context = context and context.copy() or {}
        obj_sup_util = self.pool.get('supervisor.util')
        data_atual = obj_sup_util.suputi_converte_tz(cr, uid,
                                                      datetime.now(), 'UTC', context=context).date()
        msg = []
        if dados.get('supsor_data_fim') and dados.get('supsor_data_inicio') and \
                        dados.get('supsor_data_fim') < dados.get('supsor_data_inicio'):
            msg.append(_('Data final maior que a data inicial!'))

        if dados.get('supsor_data_fim') and dados.get('supsor_data_fim') < str(data_atual):
            msg.append(_('Data final menor que a data atual!'))

        if msg:
            if onchange:
                return {'warning': {'title': _('Atencao!'),
                                    'message': '\n'.join(msg)}}

            raise osv.except_osv(_('Atencao!'), '\n'.join(msg))

        if onchange:
            return {}

        return True

    def onchange_supsor_datas(self, cr, uid, ids, data_inicial, data_final, context=None):
        context = context and context.copy() or {}
        dados = {'supsor_data_inicio': data_inicial,
                 'supsor_data_fim': data_final}
        return self.verifica_datas(cr, uid, dados, context)

    def verifica_valor(self, dados, context=None):
        context = context and context.copy() or {}
        if dados.get('supsor_valor') <= 0:
            raise osv.except_osv(_('Atencao!'),
                                 _('O campo Valor deve ser maior que 0(zero)!'))

        return True


    def finalizar_promocoes_vencidas(self, cr, uid, ids, context=None):
        '''
        Funcao ativada pelo cron 'cronjob_supervisor_excluir_sorteio'
        Deleta os sorteios vencidps
        '''
        obj_sup_util = self.pool.get('supervisor.util')
        data_atual = obj_sup_util.suputi_converte_tz(cr, uid,
                                                      datetime.now(), 'UTC', context=context).date()
        ids = self.search(cr, uid, [('supsor_data_fim', '<', str(data_atual))], context=context)
        if not ids:
            return True

        self.unlink(cr, uid, ids, context=context)
        return True