#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import datetime
from SigUtils.cnpj import Cnpj
import subprocess
import os


class supervisor_emitir_sorteio(osv.TransientModel):
    """
    Tabela de sorteios
    Prefixo: supeso

    """

    _name = 'supervisor.emitir.sorteio'
    _gerar_logs_logs = 'N'
    _columns = {
        'supsor_qtd': fields.integer('Quantide de Cupons', required=True),
    }

    _defaults = {
        'supsor_qtd': 1,
    }

    def _get_endereco(self, brw_config, context=None):
        context = context and context.copy() or {}
        remove_acentuacao = self.pool.get('supervisor.util').suputi_remover_acentuacao
        linha1 = '{rua}  {numero} - {bairro}'.format(rua=remove_acentuacao(brw_config.supcfc_endereco),
                                                     numero=brw_config.supcfc_numero,
                                                     bairro=remove_acentuacao(brw_config.supcfc_bairro))
        linha2 = 'CEP: {cep} - {cidade} - {uf}'.format(cep=brw_config.supcfc_cep or '',
                                                       cidade=remove_acentuacao(brw_config.supcfc_cidade),
                                                       uf=brw_config.supcfc_estado)
        aux = ''
        texto = ''
        for i in linha1.split(' '):
            if len(aux) + 1 + len(i) > 48:
                texto += aux.center(48) + '\n'
                aux = i + ' '
                continue

            aux += i + ' '

        texto += aux.center(48) + '\n'
        aux = ''
        for i in linha2.split(' '):
            if len(aux) + 1 + len(i) > 48:
                texto += aux.center(48) + '\n'
                aux = i + ' '
                continue

            aux += i + ' '

        texto += aux.center(48)
        return texto

    def imprimir_cupom_sorteio(self, cr, uid, qtd, context=None):
        context = context and context.copy() or {}
        obj_config = self.pool.get('supervisor.cfg.configuracao')
        config_ids = obj_config.search(cr, uid, [], context=context)
        if not config_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhuma configuracao cadastrada!'))

        obj_parametros = self.pool.get('supervisor.cfg.parametros')
        par_ids = obj_parametros.search(cr, uid, [], context=context)
        if not par_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum parametro cadastrado!'))

        dados_param = obj_parametros.read(cr, uid, par_ids[0], ['supcfp_impressora_sorteio', 'supcfp_porta_impressora',
                                                                'supcfp_senha'], context)
        impressora = dados_param.get('supcfp_impressora_sorteio')
        porta_impressora = dados_param.get('supcfp_porta_impressora')
        senha = dados_param.get('supcfp_senha')
        if not impressora:
            raise osv.except_osv(_('Atencao!'),
                                 _('Impressora nao esta cadastrada no parametro!'))

        obj_sorteio = self.pool.get('supervisor.cad.sorteio')
        sort_ids = obj_sorteio.search(cr, uid, [('state','=','2')], context=context)
        if not sort_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhuma sorteio Ativo!'))

        obj_comandos = self.pool.get('supervisor.cad.comandos.impressoras')
        comando_ids = obj_comandos.search(cr, uid, [('state', '=', '2')], context=context)
        if not comando_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum comando ativo em Supervisor->Cadastros->Comandos de '
                                   'impressoras'))

        brw_comandos = obj_comandos.browse(cr, uid, comando_ids[0], context)
        if brw_comandos.supcci_marca == 'elgin':
            if not porta_impressora:
                raise osv.except_osv(_('Atencao!'),
                                     _('Para impressora Elgin, deve cadastrar o parametro "Porta da impressora"!'
                                       ' no menu Supervisor->Parametros'))
            if not senha:
                raise osv.except_osv(_('Atencao!'),
                                     _('Para impressora Elgin, deve cadastrar o parametro "Senha"!'
                                       ' no menu Supervisor->Parametros'))


        remove_acentuacao = self.pool.get('supervisor.util').suputi_remover_acentuacao
        brw_config = obj_config.browse(cr, uid, config_ids[0], context)
        cnpj_emissor = Cnpj().format(brw_config.supcfc_cnpj or '')
        nome_empresa = remove_acentuacao(brw_config.supcfc_cliente or '')
        ie = brw_config.supcfc_inscricao_estadual or ''
        im = brw_config.supcfc_inscricao_municipal or ''
        loja = brw_config.supcfc_loja
        endereco_emp = self._get_endereco(brw_config, context)
        telefone = '(    )'
        data = datetime.now().strftime('%d/%m/%Y %H:%M:%S')

        espac_linha = brw_comandos.supcci_espacamento_linha or ''
        sub1, sub2 = brw_comandos.supcci_sub_liga, brw_comandos.supcci_sub_desliga
        neg1, neg2 = brw_comandos.supcci_negrito_liga, brw_comandos.supcci_negrito_desliga
        fonte_i = brw_comandos.supcci_fonte_liga or ''
        fonte_f = brw_comandos.supcci_fonte_desliga or ''
        corta_papel = brw_comandos.supcci_corta_papel or ''
        nao_doc_fiscal = 'NAO E DOCUMENTO FISCAL'
        desc_sorteio = obj_sorteio.read(cr, uid, sort_ids[0], ['supsor_descricao'],
                                        context).get('supsor_descricao')
        texto = ['{linha}{neg1}%s{neg2}' % nome_empresa[:48].center(48).upper(),
                 endereco_emp,
                 'CNPJ:%s' % cnpj_emissor,
                 'IE:%s' % ie,
                 '{sub1}IM:%s{sub2}' % im.ljust(45),
                 '%s   GNF:000000      COO:000000' % data,
                 'GRG:000000  CER:0001',
                 nao_doc_fiscal.center(48, ' '),
                 '%s%s%s' % (fonte_i, 'RELATORIO GERENCIAL'.center(24), fonte_f),
                 'Relatorio Gerencial'.center(48),
                 '',
                 '%s' % desc_sorteio,
                 '------------------------------------------------',
                 'Nome:{sub1}{neg1}%s{sub2}{neg2}' % ''.ljust(43),
                 '',
                 'Endereco:{sub1}{neg1}%s{sub2}{neg2}' % ''.ljust(39),
                 '',
                 '{sub1}{neg1}%s{sub2}{neg2}' % ''.ljust(48),
                 '',
                 '{sub1}{neg1}%s{sub2}{neg2}' % ''.ljust(48),
                 '',
                 ]

        texto.extend(
                ['Tel:{sub1}{neg1} %s{sub2}{neg2}' % telefone.ljust(43),
                 '',
                 'CPF/CNPJ:{sub1}{neg1} %s{sub2}{neg2}' % ''.ljust(38),
                 '',
                 'RG:{sub1}{neg1} %s{sub2}{neg2}' % ''.ljust(44),
                 '',
                 # nao_doc_fiscal.center(48, ' '),
                 'E-mail:{sub1}{neg1} %s{sub2}{neg2}' % ''.ljust(40),
                 nao_doc_fiscal.center(48, '-'),
                 '00000000 00000000 00000000 00000000 000000000000',
                 '%s ' % (brw_comandos.supcci_marca or '').upper(),
                 'VERSAO:01.01.01 ECF:000 LJ:%04d' % int(loja),
                 'XXXXXXXXXXXXXXXXXX %s' % data,
                 'FAB:00000000000000000000',])

        cont = -10
        aux = []
        for i in texto:
            aux.append(i)
            cont += 1
            if '\n' in i:
                cont += (i.count('\n') - 1) or 1

            if cont >= 10 and i != texto[-1]:
                aux.append(nao_doc_fiscal.center(48, ' '))
                cont = 0

        texto = aux
        text = str(remove_acentuacao('\n'.join(texto))).format(linha=espac_linha,
                                                               sub1=sub1,
                                                               sub2=sub2,
                                                               neg1=neg1,
                                                               neg2=neg2,
                                                               b1=neg1,
                                                               b2=neg2,
                                                               f1=fonte_i,
                                                               f2=fonte_f)
        textos = []
        for j in range(0, qtd):
            textos.append(text)
            if corta_papel:
                if brw_comandos.supcci_marca == 'elgin':
                    textos.append('\n\n')
                textos.append('\n\n%s' % corta_papel) # corta papel

        try:
            path = '/tmp/cupom_sorteio.txt'
            file = open(path, 'w')
            file.write((''.join(textos)).replace('\\x1b', '\x1b').replace('\\x1d', '\x1d').\
                        replace('\\x57', '\x57').replace('\\x12', '\x12').\
                        replace('\\x0f', '\x0f').replace('\\x01', '\x01').\
                        replace('\\x02', '\x02').replace('\\x03', '\x03').\
                        replace('\\x0c', '\x0c').replace('\\x08', '\x08').\
                        replace('\\x00', '\x00').replace('\\r', '\r').replace('\\x20', '\x20'))
            file.close()
            if brw_comandos.supcci_marca == 'elgin':
                os.system("echo %s | sudo -S chmod 776 %s; cat %s > %s" %
                          (senha or 'sig',
                           porta_impressora,
                           path,
                           porta_impressora))
            else:
                subprocess.call(["lp", "-o raw", "-n1", '-d%s' % impressora, path])

        except Exception as e:
            print e

        return True


    def button_supeso_imprimir(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_filtro = self.browse(cr, uid, ids[0], context)
        qtd = brw_filtro.supsor_qtd
        self.imprimir_cupom_sorteio(cr, uid, qtd, context)

        return True