# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
import xml.etree.ElementTree as ET
import base64
import time
from openerp.tools.translate import _


class wizard_importa_xml_retorno_taxpertise(osv.TransientModel):
    # prefixo: impret_
    _name = 'wizard.importa.xml.retorno.taxpertise'
    _columns = {
        'id': fields.integer('ID', readonly=True),
        'xml_file': fields.binary('Arquivo xml para importacao', required=True),
    }

    def impret_button_importa(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for x in self.browse(cr, uid, ids, context):
            dict_arquivo_dados = self.impret_converte_xml_dados(cr, uid, x.xml_file, context)
            self.impret_importa_retorno_revisao(cr, uid, dict_arquivo_dados, context)
        return True

    def impret_converte_xml_dados(self, cr, uid, xml_file_data, context=None):
        context = context and context.copy() or {}
        dados = list()
        file_data = base64.b64decode(xml_file_data)
        file_data = ET.XML(file_data)
        for segmento in file_data:
            xmldict = XmlDictConfig(segmento)
            if segmento.tag == 'erro':
                raise osv.except_osv(_('Atencao'), segmento.text)
            if (context.get('taxpertise_produto_id') or context.get('taxpertise_ncm_id')) and \
                    xmldict.get('saida'):
                dados.append(xmldict)
                continue
            if not xmldict.get('revisao'):
                continue
            dados.append(xmldict)
        return dados

    def impret_importa_retorno_revisao(self, cr, uid, dados_retorno, context):
        context = context and context.copy() or {}

        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_fiscal_ncm = self.pool.get('fiscal.ncm')
        obj_fiscal_cest = self.pool.get('fiscal.cest')

        list_vals_itens = list()
        if context.get('taxpertise_produto_id'):
            produto_ids = [context.get('taxpertise_produto_id')]
        elif context.get('taxpertise_ncm_id'):
            produto_ids = obj_estoque_produto.search(
                cr, uid, [('ncm_id', '=', context.get('taxpertise_ncm_id'))], context=context)
            dados_retorno = [dados_retorno[0]]
        else:
            codigo_interno_list = map(lambda p: p.get('revisao').get('codigoInterno'), dados_retorno)
            produto_ids = obj_estoque_produto.search(cr, uid, [('codigo', 'in', codigo_interno_list)],
                                                     context=context)
        cadastro_produto_dados = self.impret_get_produto_dados(cr, uid, produto_ids, context)
        for segmento in dados_retorno:
            revisao = segmento.get('revisao')
            detalhe = segmento.get('detalhes')
            saida = segmento.get('saida')
            federal = segmento.get('federal')

            if context.get('taxpertise_produto_id'):
                produto_dados = [cadastro_produto_dados[0]]
            elif context.get('taxpertise_ncm_id'):
                produto_dados = cadastro_produto_dados
            else:
                codigo_interno = revisao.get('codigoInterno')
                codigo_ean = revisao.get('codigoEan')
                codigo_ean = codigo_ean.zfill(14)
                produto_dados = filter(
                    lambda p: p.get('codigo_interno') == codigo_interno
                    and p.get('codigo_ean') == codigo_ean, cadastro_produto_dados)
                if not produto_dados:
                    continue
                produto_dados = [produto_dados[0]]

            novo_ncm = detalhe.get('ncm').replace('.', '')
            novo_ncm_id = obj_fiscal_ncm.search(cr, uid, [('codigo', '=', novo_ncm)], limit=1,
                                                context=context)
            novo_ncm_id = novo_ncm_id[0] if novo_ncm_id else False
            if not novo_ncm_id and novo_ncm:
                novo_ncm_id = obj_fiscal_ncm.create(cr, uid, {'codigo': novo_ncm}, context)

            cest_codigo = segmento.get('tabelaCest').get('cest').replace('.', '')
            novo_cest_id = obj_fiscal_cest.search(
                cr, uid, [('fiscest_codigo', '=', cest_codigo)], limit=1, context=context)
            novo_cest_id = novo_cest_id[0] if novo_cest_id else False
            if not novo_cest_id and cest_codigo:
                novo_cest_id = obj_fiscal_cest.create(cr, uid, {'fiscest_codigo': cest_codigo},
                                                      context)

            for dado in produto_dados:
                vals_itens = {
                    'estfit_produto_id': dado.get('produto_id'),
                    'estfit_cad_ncm': dado.get('ncm'),
                    'estfit_cad_cest': dado.get('cest'),
                    'estfit_cad_icms_cst': dado.get('cst_icms_saida'),
                    'estfit_cad_icms_aliquota': dado.get('aliquota_icms_saida'),
                    'estfit_cad_icms_base': dado.get('base_icms_saida'),
                    'estfit_cad_pis_cst_entrada': dado.get('cst_pis_cofins_entrada'),
                    'estfit_cad_pis_cst_saida': dado.get('cst_pis_cofins_saida'),
                    'estfit_cad_pis_aliquota_entrada': dado.get('aliquota_pis_entrada'),
                    'estfit_cad_pis_aliquota_saida': dado.get('aliquota_pis_saida'),
                    'estfit_cad_pis_nat_rec': dado.get('natureza_receita_pis_cofins'),
                    'estfit_cad_cofins_cst_entrada': dado.get('cst_pis_cofins_entrada'),
                    'estfit_cad_cofins_cst_saida': dado.get('cst_pis_cofins_saida'),
                    'estfit_cad_cofins_aliquota_entrada': dado.get('aliquota_cofins_entrada'),
                    'estfit_cad_cofins_aliquota_saida': dado.get('aliquota_cofins_saida'),
                    'estfit_cad_cofins_nat_rec': dado.get('natureza_receita_pis_cofins'),

                    'estfit_novo_ncm': detalhe.get('ncm').replace('.', ''),
                    'estfit_novo_cest': segmento.get('tabelaCest').get('cest').replace('.', ''),
                    'estfit_novo_icms_cst': (saida.get('icmsCstSaida') or '')[1:3],
                    'estfit_novo_icms_aliquota': saida.get('icmsSaida'),
                    'estfit_novo_icms_base': 0,
                    'estfit_novo_pis_cst_entrada': federal.get('piscofinsCstEntrada'),
                    'estfit_novo_pis_cst_saida': federal.get('piscofinsCstSaida'),
                    'estfit_novo_pis_aliquota_entrada': federal.get('pisEntrada'),
                    'estfit_novo_pis_aliquota_saida': federal.get('pisSaida'),
                    'estfit_novo_pis_nat_rec': federal.get('piscofinsNatRec'),
                    'estfit_novo_cofins_cst_entrada': federal.get('piscofinsCstEntrada'),
                    'estfit_novo_cofins_cst_saida': federal.get('piscofinsCstSaida'),
                    'estfit_novo_cofins_aliquota_entrada': federal.get('cofinsEntrada'),
                    'estfit_novo_cofins_aliquota_saida': federal.get('cofinsSaida'),
                    'estfit_novo_cofins_nat_rec': federal.get('piscofinsNatRec'),
                    'estfit_novo_icms_tribut_id': self.impret_get_nova_tributacao_id(
                        cr, uid, 'icms', saida.get('icmsCstSaida'), 0, saida.get('icmsSaida'), context),
                    'estfit_novo_pis_tribut_id': self.impret_get_nova_tributacao_id(
                        cr, uid, 'pis', federal.get('piscofinsCstSaida'), 0, federal.get('pisSaida'),
                        context),
                    'estfit_novo_cofins_tribut_id': self.impret_get_nova_tributacao_id(
                        cr, uid, 'cofins', federal.get('piscofinsCstSaida'), 0,
                        federal.get('cofinsSaida'), context),
                }
                novo_ncm_id and vals_itens.update({'estfit_novo_ncm_id': novo_ncm_id})
                novo_cest_id and vals_itens.update({'estfit_novo_cest_id': novo_cest_id})
                list_vals_itens.append((0, 0, vals_itens))
        if not list_vals_itens:
            return False
        vals = {'estfca_data': time.strftime('%Y-%m-%d'), 'estfca_origem': 'Taxpertise',
                'estfca_item_ids': list_vals_itens}
        return self.pool.get('estoque.fiscal.capa').create(cr, uid, vals, context)

    def impret_get_nova_tributacao_id(self, cr, uid, encargo, cst, base, aliquota, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT ft.id
            FROM fiscal_tributacao ft
            INNER JOIN fiscal_encargo fe
                ON fe.id = ft.encargo_id
            INNER JOIN fiscal_cst fc
                ON fc.id = ft.cst_id
            WHERE
                fe.sigla = '%s'
                AND fc.codigo = '%s'
                AND ft.aliquota = %s
        """ % (encargo.upper(), cst, aliquota)
        if base:
            sql += """
                AND ft.base = %s
            """ % base
        sql += """
            LIMIT 1
        """
        cr.execute(sql)
        tributacao_id = cr.fetchone()
        return tributacao_id or False

    def impret_get_produto_dados(self, cr, uid, produto_ids, context=None):
        """
        O metodo reune todos os dados necessarios do cadastro dos produtos para a importacao da
        revisao fiscal.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela dessa classe.
        :type ids: list
        :param context: valores contextuais.
        :type context: dict
        :return: dados do cadastro dos produtos.
        :rtype: list
        """
        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, 'estoque.produto',
                                                                    context=context)
        if not produto_ids:
            return list()
        if len(produto_ids) == 1:
            produto_ids = str(tuple(produto_ids)).replace(',', '')
        else:
            produto_ids = str(tuple(produto_ids))

        sql = """
            SELECT
                DISTINCT
                ep.id               AS produto_id,
                epg.id              AS grade_id,
                ep.codigo           AS codigo_interno,
                acb.codigo_barra    AS codigo_ean,
                CASE
                    WHEN substr(acb.codigo_barra, 1, 13)::bigint = ep.codigo::bigint
                        THEN 1
                    ELSE 0
                END                 AS codigo_ean_interno,
                CASE
                    WHEN (SELECT estpar_descricao_grade FROM estoque_parametro WHERE company_id = 1) = 'P'
                        THEN ep.name || '-' || epg.descricao
                    ELSE
                        epg.descricao
                END                 AS descricao_produto,
                fn.codigo           AS ncm,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'PIS')
                     )
                )                   AS aliquota_pis_saida,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'PIS')
                     )
                )                   AS aliquota_pis_entrada,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'COFINS')
                     )
                )                   AS aliquota_cofins_saida,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'COFINS')
                     )
                )                   AS aliquota_cofins_entrada,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla IN ('PIS', 'COFINS'))
                     )
                )                   AS cst_pis_cofins_saida,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'E'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla IN ('PIS', 'COFINS'))
                     )
                )                   AS cst_pis_cofins_entrada,
                CASE
                    WHEN ep.estpro_pis_nat_rec IS NOT NULL THEN ep.estpro_pis_nat_rec
                    WHEN ep.estpro_cofins_nat_rec IS NOT NULL THEN ep.estpro_cofins_nat_rec
                    ELSE ''
                END                 AS natureza_receita_pis_cofins,
                (SELECT aliquota
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS aliquota_icms_saida,
                (SELECT base
                 FROM fiscal_tributacao
                 WHERE
                     id IN (
                     SELECT min(tributacao_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS base_icms_saida,
                (SELECT codigo
                 FROM fiscal_cst
                 WHERE
                     id IN (
                     SELECT min(cst_id)
                     FROM fiscal_tributacao_encargos
                     WHERE
                         tipo_tributacao = 'T'
                         AND produto_id = ep.id
                         AND encargo_id IN (
                             SELECT id
                             FROM fiscal_encargo
                             WHERE
                                sigla = 'ICMS')
                     )
                )                   AS cst_icms_saida,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'U' THEN 1
                    ELSE 0
                END                 AS substituicao_tributaria,
                0                   AS cesta_basica,
                CASE
                    WHEN ep.origem_mercadoria IN ('1','2','6','7')
                        THEN 1
                    ELSE 0
                END                 AS importado,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'I' THEN 1
                    ELSE 0
                END                 AS isento,
                CASE WHEN
                    (SELECT tipo
                     FROM fiscal_cst
                     WHERE
                         id IN (
                         SELECT min(cst_id)
                         FROM fiscal_tributacao_encargos
                         WHERE
                             tipo_tributacao = 'T'
                             AND produto_id = ep.id
                             AND encargo_id IN (
                                 SELECT id
                                 FROM fiscal_encargo
                                 WHERE
                                    sigla = 'ICMS')
                         )
                    ) = 'D' THEN 1
                    ELSE 0
                END                 AS diferido,
                ''                  AS tipo_lista,
                (SELECT fiscest_codigo
                 FROM fiscal_cest
                 WHERE id = ep.estpro_cest_id)
                                    AS cest
            FROM estoque_produto ep
            INNER JOIN estoque_produto_grade epg
                ON ep.id = epg.produto_id
            INNER JOIN automacao_codigo_barra acb
                ON ep.id = acb.produto_id AND epg.id = acb.grade_id
            INNER JOIN fiscal_ncm fn
                ON fn.id = ep.ncm_id
            WHERE
                ep.id IN {produto_ids}
                AND ep.company_id = {company_id}
        """.format(produto_ids=produto_ids, company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()

        return dados


class XmlListConfig(list):
    def __init__(self, aList):
        for element in aList:
            if element:
                # treat like dict
                if len(element) == 1 or element[0].tag != element[1].tag:
                    self.append(XmlDictConfig(element))
                # treat like list
                elif element[0].tag == element[1].tag:
                    self.append(XmlListConfig(element))
            elif element.text:
                text = element.text.strip()
                if text:
                    self.append(text)


class XmlDictConfig(dict):
    '''
    Example usage:

    >>> tree = ElementTree.parse('your_file.xml')
    >>> root = tree.getroot()
    >>> xmldict = XmlDictConfig(root)

    Or, if you want to use an XML string:

    >>> root = ElementTree.XML(xml_string)
    >>> xmldict = XmlDictConfig(root)

    And then use xmldict for what it is... a dict.
    '''
    def __init__(self, parent_element):
        if parent_element.items():
            self.update(dict(parent_element.items()))
        for element in parent_element:
            if len(element):
                # treat like dict - we assume that if the first two tags
                # in a series are different, then they are all different.
                if len(element) == 1 or element[0].tag != element[1].tag:
                    aDict = XmlDictConfig(element)
                # treat like list - we assume that if the first two tags
                # in a series are the same, then the rest are the same.
                else:
                    # here, we put the list in dictionary; the key is the
                    # tag name the list elements all share in common, and
                    # the value is the list itself
                    aDict = {element[0].tag: XmlListConfig(element)}
                # if the tag has attributes, add those to the dict
                if element.items():
                    aDict.update(dict(element.items()))
                self.update({element.tag: aDict})
            # this assumes that if you've got an attribute in a tag,
            # you won't be having any text. This may or may not be a
            # good idea -- time will tell. It works for the way we are
            # currently doing XML configuration files...
            elif element.items():
                self.update({element.tag: dict(element.items())})
            # finally, if there are no child tags and no attributes, extract
            # the text
            else:
                self.update({element.tag: element.text})
