# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _


class trocas_capa(osv.Model):
    # prefixo: trccap_

    _inherit = 'trocas.capa'

    def trccap_button_esboco(self, cr, uid, ids, context=None):
        """
        O metodo altera o status das trocas para esboco (ES).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso as trocas tenham seus status atualizados, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = self.write(cr, uid, ids, {'state': 'ES'}, context)
        return res

    def trccap_button_valida(self, cr, uid, ids, context=None):
        """
        O metodo altera o status das trocas para validado (VA).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso as trocas tenham seus status atualizados, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_trocas_capa = self.browse(cr, uid, ids, context)
        if not all(map(lambda i: i.trccap_doc_origem_ids, brw_trocas_capa)) or not \
            all(map(lambda i: i.trccap_itens_entrada_ids, brw_trocas_capa)):
            raise osv.except_osv(_('Atencao'),
                                 _('Nenhum documento ou item para devolucao foi informado.'))
        res = self.write(cr, uid, ids, {'state': 'VA'}, context)
        return res

    def trccap_button_efetiva(self, cr, uid, ids, context=None):
        """
        O metodo altera o status das trocas para efetivado (EF), como tambem, cria as notas de
        entrada para os itens de devolucao e as notas de saida para os itens de venda.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso as trocas tenham seus status atualizados, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        self.trccap_gera_nf(cr, uid, ids, context)
        res = self.write(cr, uid, ids, {'state': 'EF'}, context)
        return res

    def trccap_button_cancela(self, cr, uid, ids, context=None):
        """
        O metodo altera o status das trocas para cancelado (CA), como tambem, deleta as notas de
        entrada e saida caso elas nao estejam com status de faturada ou em processamento.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso as trocas tenham seus status atualizados, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')

        for troca in self.browse(cr, uid, ids, context):
            msg = ''
            nf_ids = filter(lambda i: i.state in ('3', 'P'), troca.trccap_nf_ids)
            if nf_ids:
                for nf_id in nf_ids:
                    msg += nf_id.name + '\n'
            else:
                context.update({'cancelar': True})
                obj_fiscal_nf_capa.unlink(cr, uid, map(lambda i: i.id, troca.trccap_nf_ids),
                                          context)
            if msg:
                raise osv.except_osv(_('Atencao'),
                                     _('A troca nao pode ser cancelada. As notas fiscais a seguir '
                                       'se encontram faturadas ou em processamento:\n %s' % msg))

        res = self.write(cr, uid, ids, {'state': 'CA'}, context)
        return res

    def trccap_gera_nf(self, cr, uid, ids, context=None):
        """
        O metodo  cria as notas de entrada para os itens de devolucao e as notas de saida para os
        itens de venda.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: Verdadeiro, caso as notas sejam criadas, senao, Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = False

        obj_fiscal_utils = self.pool.get('fiscal.utils')
        obj_fiscal_cfop_natureza = self.pool.get('fiscal.cfop.natureza')
        obj_fiscal_cfop = self.pool.get('fiscal.cfop')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')

        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)

        uf_company = self.pool.get('res.company').browse(cr, uid, company_id, context).\
            rescon_pessoa_id.uf

        fiscal_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'fiscal', ['modelo_nf_series_id'], context=context)
        modelo_nf_id = fiscal_parametro_dados.get('modelo_nf_series_id')

        nf_ids = list()

        for troca in self.browse(cr, uid, ids, context):
            uf_destinatario = troca.trccap_pessoa_id.uf
            if uf_company == uf_destinatario:
                cfop = '1202'
                cfop_ids = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop)],
                                                  context=context)
                cfop_saida = '5102'
                cfop_saida_ids = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop_saida)],
                                                        context=context)
            else:
                cfop = '2202'
                cfop_ids = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop)],
                                                  context=context)
                cfop_saida = '6102'
                cfop_saida_ids = obj_fiscal_cfop.search(cr, uid, [('codigo', '=', cfop_saida)],
                                                        context=context)
            natureza_ids = obj_fiscal_cfop_natureza.search(
                cr, uid, [('cfop_id', 'in', cfop_ids)], context=context)
            if not natureza_ids:
                raise osv.except_osv(_('Atencao'),
                                     _('Nao ha registro de natureza para o CFOP %s' % cfop))

            # Cria a nota fiscal de entrada
            context.update({'ctx_nf_propria': True, 'nota_fiscal_entrada': True, 'proprio': True})
            nf_id = obj_fiscal_utils.gerar_nf(
                cr, uid,
                company_id=company_id,
                destinatario_id=troca.trccap_pessoa_id.id,
                natureza_id=natureza_ids[0],
                tipo='E',
                modelo_nf_id=modelo_nf_id,
                context=context
                )
            if not nf_id:
                raise osv.except_osv(_('Atencao'),
                                     _('Nao foi possivel criar a nota fiscal de entrada.'))
            obj_fiscal_nf_capa.write(cr, uid, [nf_id], {'finalidade_emissao': '4'}, context)
            nf_ids.append(nf_id)

            for item in troca.trccap_itens_entrada_ids:
                obj_fiscal_utils.gerar_nf_itens(
                    cr, uid,
                    nf_id=nf_id,
                    produto_id=item.produto_id.id,
                    tipo=item.produto_id.tipo,
                    local_id=item.local_id.id,
                    produto_embalagem_id=item.produto_embalagem_id.id,
                    grade_id=item.grade_id.id,
                    quantidade=item.trcien_quantidade_devolver,
                    vr_unitario=item.vr_unitario,
                    vr_total_item=item.vr_total_item,
                    context=context
                    )
            del context['nota_fiscal_entrada']

            cr.execute("""INSERT INTO trocas_capa_m2m_nf_capa (troca_id, nf_id) VALUES (%s, %s)""" %
                       (troca.id, nf_id))

            # Cria a nota fiscal de saida
            context.update({'nota_fiscal_saida': True})

            if not troca.trccap_itens_saida_ids:
                continue

            produto_ids = list(set(map(lambda i: i.trcisa_produto_id.id,
                                       troca.trccap_itens_saida_ids)))
            grade_ids = list(set(map(lambda i: i.trcisa_grade_id.id,
                                       troca.trccap_itens_saida_ids)))
            preco_venda_dados = obj_estoque_preco_venda.get_preco_venda(
                cr, uid, epc_produto_ids=produto_ids, epc_grade_ids=grade_ids, context=context)

            natureza_saida_ids = obj_fiscal_cfop_natureza.search(
                cr, uid, [('cfop_id', 'in', cfop_saida_ids)], context=context)
            if not natureza_ids:
                raise osv.except_osv(_('Atencao'),
                                     _('Nao ha registro de natureza para o CFOP %s' % cfop_saida))

            nf_id = obj_fiscal_utils.gerar_nf(
                cr, uid,
                company_id=company_id,
                destinatario_id=troca.trccap_pessoa_id.id,
                natureza_id=natureza_saida_ids[0],
                tipo='S',
                modelo_nf_id=modelo_nf_id,
                vr_desconto=troca.trccap_desconto,
                context=context
                )
            if not nf_id:
                raise osv.except_osv(_('Atencao'),
                                     _('Nao foi possivel criar a nota fiscal de saida.'))
            nf_ids.append(nf_id)

            for item in troca.trccap_itens_saida_ids:

                preco_venda_produto = preco_venda_dados.get(item.trcisa_produto_id.id) or {}
                vr_preco_venda = preco_venda_produto.get(item.trcisa_grade_id.id)
                if not vr_preco_venda:
                    vr_preco_venda = preco_venda_produto.get('preco_venda')

                locacao_ids = obj_estoque_local_locacao.search(
                    cr, uid, [('local_id', '=', item.trcisa_local_id.id),
                              ('produto_id', '=', item.trcisa_produto_id.id),
                              ('grade_id', '=', item.trcisa_grade_id.id),
                              ('company_id', '=', company_id)], context=context)
                produto_embalagem_id = False
                if locacao_ids:
                    produto_embalagem_id = obj_estoque_local_locacao.browse(
                        cr, uid, locacao_ids[0], context).estllc_produto_embalagem_id.id

                obj_fiscal_utils.gerar_nf_itens(
                    cr, uid,
                    nf_id=nf_id,
                    produto_id=item.trcisa_produto_id.id,
                    tipo=item.trcisa_produto_id.tipo,
                    local_id=item.trcisa_local_id.id,
                    produto_embalagem_id=produto_embalagem_id,
                    grade_id=item.trcisa_grade_id.id,
                    quantidade=item.trcisa_quantidade,
                    vr_unitario=item.trcisa_valor_total /
                    item.trcisa_quantidade,
                    vr_total_item=item.trcisa_valor_total,
                    item_preco_venda=vr_preco_venda,
                    context=context
                    )
            del context['nota_fiscal_saida']
            res = True

            cr.execute("""INSERT INTO trocas_capa_m2m_nf_capa (troca_id, nf_id) VALUES (%s, %s)""" %
                       (troca.id, nf_id))
        return res

    def trccap_button_print(self, cr, uid, ids, context=None):
        """
        O metodo faz a chamada da acao para a classe do relatorio, onde e processado e gerado o
        arquivo PDF.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: acao para a classe do relatorio
        :rtype: dict
        """
        context = context and context.copy() or {}
        if isinstance(ids, (list, tuple)):
            ids = ids[0]
        return {'type': 'ir.actions.report.xml',
                'report_name': 'trocas',
                'datas': {'troca_id': ids},
                'context': context
                }
