# -*- encoding: utf-8 -*-

from openerp.osv import osv, fields


class trocas_itens_saida(osv.Model):
    # prefixo: trcisa_

    _inherit = 'trocas.itens.saida'

    def default_get(self, cr, uid, fields_list, context=None):
        """
        O metodo atribui valor padrao para os atributos da classe.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param fields_list: lista com a descricao dos atributos que receberao os valores.
        :type fields_list: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com a descricao dos atributos e seus respectivos valores
        :rtype: bool
        """
        context = context and context.copy() or {}
        res = super(trocas_itens_saida, self).default_get(cr, uid, fields_list, context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'local_venda_padrao_id', context=context)
        res['trcisa_local_id'] = estoque_parametro_dados.get('local_venda_padrao_id')
        return res

    def trcisa_onchange_quantidade(self, cr, uid, ids, local_id, produto_id, grade_id, quantidade,
                                   context=None):
        """
        O metodo e chamado na saida do atributo 'trcisa_quantidade'. Ele calcula o valor para o
        atributo 'trcisa_valor_total' baseado no local_id, produto_id, grade_id e quantidade.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param local_id: id da tabela estoque_local.
        :type local_id: int
        :param produto_id: id da tabela estoque_produto.
        :type produto_id: int
        :param grade_id: id da tabela estoque_produto_grade.
        :type grade_id: int
        :param quantidade: numero de itens que serao vendidos.
        :type quantidade: float
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com os atributos e seus respectivos valores.
        :rtype: dict
        """
        res = {'value': {'trcisa_valor_total': 0.0}}
        if not all([local_id, produto_id, grade_id, quantidade]):
            return res
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
        locacao_ids = obj_estoque_local_locacao.search(
            cr, uid, [('produto_id', '=', produto_id), ('grade_id', '=', grade_id),
                      ('local_id', '=', local_id)], context=context)
        if not locacao_ids:
            return res
        qtd_embalagem = obj_estoque_local_locacao.browse(
            cr, uid, locacao_ids[0],
            context).estllc_produto_embalagem_id.embalagem_id.quantidade_na_embalagem
        preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(
            cr, uid, epc_produto_ids=produto_id, epc_grade_ids=grade_id, context=context)
        if preco_venda.get(produto_id).get(grade_id):
            preco_venda = preco_venda.get(produto_id).get(grade_id)
        else:
            preco_venda = preco_venda.get(produto_id).get('preco_venda')
        res['value']['trcisa_valor_total'] = preco_venda * (quantidade * qtd_embalagem)
        return res