# -*- coding: utf-8 -*-

from openerp.osv import osv, fields


class trocas_documento_origem(osv.Model):
    # prefixo: trcdor_

    _inherit = 'trocas.documento.origem'

    def trcdor_seleciona_item_nota_e_cupom(self, cr, uid, nf_id, cupom_id, context=None):
        """
        O metodo encontra na tabela estoque_movimento_itens todos os registros que se referem a
        esse cupom fiscal (trcdor_cupom_id). Os dados desses registro sao incluidos na tabela
        trocas_itens_entrada.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param cupom_id: id do cupom fiscal, tabela ecf_cupons_capa.
        :type cupom_id: int
        :param context: dicionario com valores contextuais.
        :type context:dict
        :return: Lista dos itens para serem incluidas no atributo trcdor_itens_ids.
        :rtype: list
        """
        context = context and context.copy() or {}
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_ecf_cupons_capa = self.pool.get('ecf.cupons.capa')
        movimento_itens_ids = list()
        if cupom_id:
            # Verifica se ha um cupom de cancelamento para o cupom recebido (cupom_id), que e de
            # venda e validado. Nesse caso, nao deve considerar os itens.
            brw_cupons_capa = obj_ecf_cupons_capa.browse(cr, uid, cupom_id, context)
            coo = brw_cupons_capa.coo + 1
            ecf_id = brw_cupons_capa.ecf_id.id
            cupom_cancelado_ids = obj_ecf_cupons_capa.search(
                cr, uid, [('tipo_cupom', '=', 'C'), ('ecf_id', '=', ecf_id), ('coo', '=', coo)],
                context=context)

            if not cupom_cancelado_ids:
                movimento_itens_ids = obj_estoque_movimento_itens.search(
                    cr, uid, [('cupom_id', '=', cupom_id), ('item_cancelado', '!=', 'True')],
                    context=context)
        brw_movimento_itens = obj_estoque_movimento_itens.browse(cr, uid, movimento_itens_ids,
                                                                 context)
        item_ids = self.trcdor_seleciona_item(cr, uid, nf_id, context)
        for movimento in brw_movimento_itens:
            item_ids.append((0, 0, {'trcien_movimento_item_id': movimento.id,
                                    'produto_id': movimento.produto_id.id,
                                    'grade_id': movimento.grade_id.id,
                                    'quantidade': movimento.quantidade,
                                    'trcien_quantidade_devolver': movimento.quantidade}))
        return item_ids

    def trcdor_onchange_nf_id_cupom_id(self, cr, uid, ids, nf_id, cupom_id, context=None):
        """
        O metodo insere os dados do movimento de itens no atributo 'trcdor_itens_ids' a partir do
        cupom e nota fiscal recebidos (cupom_id e nf_id).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param nf_id: id da nota fiscal, tabela fiscal_nf_capa.
        :type nf_id: int
        :param cupom_id: id do cupom fiscal, tabela ecf_cupons_capa
        :type cupom_id: int
        :param context: dicionario com valores contextuais.
        :type context:dict
        :return:
        """
        context = context and context.copy() or {}
        res = {'value': {'trcdor_itens_ids': self.trcdor_seleciona_item_nota_e_cupom(
            cr, uid, nf_id, cupom_id, context)}}
        return res
