# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class veiculos_motorista(osv.Model):
    """
    Classe veiculos_motorista

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    @since: 17.04.2013
    """
    _name = 'veiculos.motorista'                                                    # veimot
    _inherit = 'veiculos.motorista'

    def veimot_onchange_cpf_cnpj(self, cr, uid, ids, cpf_cnpj, context=None):
        """
        Critica e formatacao do CPF/CNPJ.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param cpf_cnpj: {char} o numero do cpf (pessoa fisica) ou do cnpj (pes-
        soa juridica),
        @return: {dict} valor do cpf/cnpj formatado.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        @since: 17.04.2013

        @change: 26.04.2013
        """

        from re import sub as re_sub
        from SigUtils.cpf import Cpf

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pessoa = self.pool.get('adm.pessoa')

        pessoa_id = False

        if cpf_cnpj:
            numeros = re_sub('[^0-9]', '', cpf_cnpj)
            cpf_cnpj = numeros.rjust(11, '0')

            if not Cpf().validar(cpf_cnpj):
                raise osv.except_osv(_('Aviso.'), _('CPF invalido.'))

            cpf_cnpj_formatado = '%s.%s.%s-%s' % (cpf_cnpj[:3],
                                                  cpf_cnpj[3:6],
                                                  cpf_cnpj[6:9],
                                                  cpf_cnpj[9:])
            args = [('cpf_cnpj', '=', cpf_cnpj_formatado)]

            pessoa_ids = obj_pessoa.search(cr, uid, args, context=context)

            if pessoa_ids:
                return {'value': {'pessoa_id': pessoa_ids[0],
                                  'cpf_cnpj': cpf_cnpj_formatado}}

        return {}

    def veimot_onchange_pessoa_id(self, cr, uid, ids, pessoa_id, context=None):
        """
        Critica e formatacao do CPF/CNPJ.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param pessoa_id: {int} o ID da pessoa,
        @return: {dict} dados da pessoa.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        @since: 17.04.2013

        @change: 26.04.2013
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pessoa = self.pool.get('adm.pessoa')

        # Dados do motorista
        args = [('id', '=', pessoa_id)]
        pessoa_ids = obj_pessoa.search(cr, uid, args, context=context)

        if pessoa_ids:
            pessoa_id = pessoa_ids[0]
            browse_pessoa = obj_pessoa.browse(cr, uid, pessoa_id)

            # Dados de endereco
            endereco_ids = []
            for dados_endereco in browse_pessoa.endereco_ids:
                enderecos = {
                    'cep': dados_endereco.cep,
                    'active': dados_endereco.active,
                    'uf_id': dados_endereco.uf_id and dados_endereco.uf_id.id or False,
                    'numero': dados_endereco.numero,
                    'pais_id': dados_endereco.pais_id and dados_endereco.pais_id.id or False,
                    'bairro_id': dados_endereco.bairro_id and dados_endereco.bairro_id.id,
                    'cidade_id': dados_endereco.cidade_id and dados_endereco.cidade_id.id or False,
                    'observacao': dados_endereco.observacao,
                    'complemento': dados_endereco.complemento,
                    'logradouro_id': dados_endereco.logradouro_id and dados_endereco.logradouro_id.id or False,
                    'endereco_tipo_id': dados_endereco.endereco_tipo_id.id
                }

                endereco_ids.append((1, dados_endereco.id, enderecos))

            # Dados de e-mail
            email_ids = []
            for dados_emails in browse_pessoa.email_ids:
                emails = {
                    'name': dados_emails.name,
                    'observacao': dados_emails.observacao,
                    'email_tipo_id': dados_emails.email_tipo_id and dados_emails.email_tipo_id.id or False
                }

                email_ids.append((1, dados_emails.id, emails))

            # Dados de telefone
            telefones_ids = []
            for dados_telefones in browse_pessoa.telefones_ids:
                telefones = {
                    "ddi": dados_telefones.ddi,
                    "ddd": dados_telefones.ddd,
                    "numero": dados_telefones.numero,
                    "observacoes": dados_telefones.observacoes,
                    "telefone_tipo_id": dados_telefones.telefone_tipo_id and dados_telefones.telefone_tipo_id.id or False
                }

                telefones_ids.append((1, dados_telefones.id, telefones))

            # Dados de documentos
            documento_ids = []
            for documento in browse_pessoa.documentos_ids:
                documentos = {
                    'tipo_id': documento.tipo_id and documento.tipo_id.id or False,
                    'numero': documento.numero,
                    'emitente': documento.emitente,
                    'data_emissao': documento.data_emissao
                }

                documento_ids.append((1, documento.id, documentos))

            # Mostra os dados do motorista
            res = {
                'name': browse_pessoa.name,
                "foto": browse_pessoa.foto,
                'pessoa_id': browse_pessoa.id,
                'admpes_fis_rg': browse_pessoa.admpes_fis_rg,
                'cpf_cnpj': browse_pessoa.cpf_cnpj,
                'admpes_fis_sexo': browse_pessoa.admpes_fis_sexo,
                'email_ids': email_ids,
                'endereco_ids': endereco_ids,
                'telefones_ids': telefones_ids,
                'documento_ids': documento_ids,
                'admpes_fis_estado_civil': browse_pessoa.admpes_fis_estado_civil,
                'fantasia_apelido': browse_pessoa.fantasia_apelido,
                'admpes_fis_cnh_nro': browse_pessoa.admpes_fis_cnh_nro,
                'admpes_fis_data_nascimento': browse_pessoa.admpes_fis_data_nascimento,
                'admpes_fis_cnh_validade': browse_pessoa.admpes_fis_cnh_validade,
                'admpes_fis_cnh_categoria': browse_pessoa.admpes_fis_cnh_categoria,
                'admpes_fis_rg_orgao_emissor': browse_pessoa.admpes_fis_rg_orgao_emissor,
                'admpes_fis_rg_data_emisssao': browse_pessoa.admpes_fis_rg_data_emisssao,
                'admpes_fis_cnh_orgao_emissor': browse_pessoa.admpes_fis_cnh_orgao_emissor
            }

            return {'value': res}

        return {}

    def create(self, cr, uid, values, context=None):
        """
        Verifica se o motorista já foi cadastrado.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        @param values: {dict} dicionário com o nome dos campos e seus respectivos
        valores,
        @param context: {dict} um dicionário padrao para valores contextuais,
        @return: {super} novo motorista cadastrado ou mensagem de erro.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        @since: 26.04.2013
        """

        context = context and context.copy() or {}

        # Verifica se motorista já existe
        args = [('pessoa_id', '=', values.get('pessoa_id'))]

        if self.search(cr, uid, args, context=context):
            raise osv.except_osv(_('Atencao'), _('Motorista ja cadastrado.'))

        return super(veiculos_motorista, self).create(cr, uid, values, context)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
