# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br   
#
##############################################################################

from openerp.osv import osv, fields


class veiculos_veiculo(osv.Model):
    """
    Tabela de veiculos
    """
    _name = 'veiculos.veiculo'
    _columns = {
        'placa': fields.char('Placa', size=7, required=True), # placa do veiculo
        'name': fields.char('Descricao', size=50, required=True), # nome do veiculo(descricao)
        'marca_id': fields.many2one('veiculos.marca', 'Marca', required=True, ondelete='restrict'), #Marca do veiculo
        'modelo_id': fields.many2one('veiculos.modelo', 'Modelo', required=True, ondelete='restrict',
                                     domain="[('marca_id','=',marca_id)]"), # Modelo do veiculo
        'ano_fabricacao': fields.char('Ano fabricacao', size=4), # ano de fabricacao
        'ano_modelo': fields.char('Ano modelo', size=4), # ano do modelo
        'carga_maxima_peso': fields.float('Carga maxima (Kg)', digits=(15, 3)), # Carga maxima suportada em kilograma.
        'carga_altura': fields.float('Altura (m)', digits=(5, 2), help="Altura da carroceria"),
        'carga_largura': fields.float('Largura (m)', digits=(5, 2), help="largura da carroceria"),
        'carga_comprimento': fields.float('Profundidade (m)', digits=(5, 2), help="Comprimento da carroceria"),
        'motorista_id': fields.many2one("veiculos.motorista", 'Motorista', select=True), # id do motorista
        'combustivel': fields.selection(
            [('A', 'Alcool'), ('G', 'Gasolina'), ('D', 'Diesel'), ('S', 'Gas'), ('E', 'Eletrico'), ('F', 'Flex')],
            'Combustivel', size=1, help="Tipo de combustivel do veiculo"),
        'centro_custo_id': fields.many2one("adm.empresa.centrocusto", 'Centro custo', ondelete="set null", select=True),
        'despesas_ids': fields.one2many('veiculos.despesas', 'veiculo_id', 'Despesas'),
        'veivei_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', select=True, help='Proprietario do veiculo'),
        'veivei_tipo_veiculo': fields.many2one('veiculos.veiculo.tipo', 'Tipo Veiculo'),
    }
    _sql_constraints = [
        ( 'placa_veic_uniq', 'unique (placa)', 'A placa do veiculo precisa ser unica' ),
    ]

