#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from openerp import SUPERUSER_ID


class vendas_utils(osv.Model):
    _name = 'vendas.utils'
    _auto = False

    _tem_erro = False
    _messagem_erro = ''

    def venutl_get_erro(self):
        return self._mensagem_erro

    def venutl_get_nf_pessoa_id(self, cr, uid, nf_ids, context=None):
        """
        RETORNA:
         - DICIONARIO COM ID DA NOTA COMO CHAVE E ID DA PESSOA COMO VALOR
        """
        context = context and context.copy() or {}
        res = {}
        for nota in self.pool.get('fiscal.nf.capa').read(cr, uid, nf_ids, ['destinatario_id'],
                                                         context=context,
                                                         load='_classic_write'):
            res[nota.get('id')] = nota.get('destinatario_id')

        return res

    def venutl_get_pedido_pessoa_id(self, cr, uid, pedido_ids, context=None):
        """
        RETORNA:
         - DICIONARIO COM ID DO PEDIDO COMO CHAVE E ID DA PESSOA COMO VALOR
        """
        context = context and context.copy() or {}
        res={}
        for pedido in self.pool.get('estoquepedidos.pedido.capa').read(cr, uid, pedido_ids,
                                                                       ['pessoa_id'],
                                                                        context=context,
                                                                       load='_classic_write'):
            res[pedido.get('id')] = pedido.get('pessoa_id')

        return res

    def venutl_get_orcamento_pessoa_id(self, cr, uid, orcamento_ids, context=None):
        """
        RETORNA:
         - DICIONARIO COM ID DO ORCAMENTO COMO CHAVE E ID DA PESSOA COMO VALOR
        """
        context = context and context.copy() or {}
        res = {}
        for orcamento in self.pool.get('orcamentos.orcamento.capa').read(cr, uid, orcamento_ids,
                                                                         ['pessoa_id'],
                                                                         context=context,
                                                                         load='_classic_write'):
            res[orcamento.get('id')] = orcamento.get('pessoa_id')

        return res

    def venutl_cliente_bloqueado_por_msg_ids(self, cr, uid, venutl_pessoa_ids, cliente_ids=False,
                                             context=None):
        """
        FUNÇÃO BUSCA AS O IDS DAS MENSAGENS QUE BLOQUEAM VENDA PARA O CLIENTE
        RETORNA MSG_IDS OU []
        """
        context = context and context.copy() or {}
        obj_financeiro = self.pool.get('financeiro')
        msg_nome_ids = {'pessoa_ids': {}, 'cliente_ids': {}}
        if not isinstance(venutl_pessoa_ids, list):
            venutl_pessoa_ids = [venutl_pessoa_ids]

        if not cliente_ids:
            cliente_ids = obj_financeiro.finutl_get_cliente_ids(cr, uid, venutl_pessoa_ids,
                                                                context=context)
            if not cliente_ids:
                return msg_nome_ids

        if not isinstance(cliente_ids, list):
            cliente_ids = [cliente_ids]

        obj_clientes_msg = self.pool.get('financeiro.clientes.mensagem')
        cliente_mensagem_ids = obj_clientes_msg.search(
            cr, uid, [('cliente_id', 'in', cliente_ids)], context=context)
        if not cliente_mensagem_ids:
            return msg_nome_ids

        brw_mensagem = obj_clientes_msg.browse(cr, uid, cliente_mensagem_ids, context)
        # PROCURA MENSAGENS QUE BLOQUEAM VENDA
        for mensagem in filter(lambda f: f.mensagens_id.tipo_mensagem == 'B', brw_mensagem):
            msg_nome_ids['pessoa_ids'][mensagem.cliente_id.pessoa_id.id] = mensagem.mensagens_id.id
            msg_nome_ids['cliente_ids'][mensagem.cliente_id.id] = mensagem.mensagens_id.id

        return msg_nome_ids

    def venutl_cliente_bloqueado_por_msg(self, cr, uid, venutl_pessoa_ids, cliente_ids=False,
                                         context=None):
        """
        RETORNA
            TRUE - EXISTE ALGUMA MENSAGEM QUE O TIPO BLOQUEIE VENDA
            FALSE - NÃO HÁ NENHUMA MENSAGEM QUE O TIPO BLOQUEIE VENDA
        """
        context = context and context.copy() or {}
        res = self.venutl_cliente_bloqueado_por_msg_ids(cr, uid, venutl_pessoa_ids,
                                                        cliente_ids=cliente_ids,
                                                        context=context)
        return res

    def venutl_get_limite_credito_cliente(self, cr, uid, pessoa_ids, cliente_ids=[],
                                          context=None):
        """
        Recupera um dicinario com chaves 'pessoa_ids', 'cliente_ids', e os valores sao outros
        dicinaios onde as chaves sao os ids das pessoas e dos clietens contendo o valor
        do limite de credito

        :param cr:  a linha atual do cursor do banco de dados,
        :param uid:  o ID do usuario utilizado para a verificacao de perfil,
        :param pessoa_ids:  lista com o ID da pessoa (cliente),
        :param cliente_ids: lista com o ID dos clientes
        :param context:  um dicionario padrao para valores contextuais.
        :return: valor do limite de credito do cliente.
        """
        context = context and context.copy() or {}
        obj_financeiro = self.pool.get('financeiro')
        if not isinstance(pessoa_ids, list):
            pessoa_ids = [pessoa_ids]
        if not isinstance(cliente_ids, list):
            cliente_ids = [cliente_ids]

        res = {'pessoa_ids': {}.fromkeys(pessoa_ids, 0),
               'cliente_ids': {}.fromkeys(cliente_ids, 0)}
        if not cliente_ids:
            cliente_ids = obj_financeiro.finutl_get_cliente_ids(cr, uid, pessoa_ids,
                                                                context=context)
            if not cliente_ids:
                return res
        campos = ['pessoa_id', 'limite_credito']
        dados_cliente = self.pool.get('financeiro.cliente').read(
            cr, uid, cliente_ids, campos, context, '_classic_write')
        for dado in dados_cliente:
            res['pessoa_ids'][dado.get('pessoa_id')] = dado.get('limite_credito', 0)
            res['cliente_ids'][dado.get('id')] = dado.get('limite_credito', 0)

        return res

    def _get_dict_con_forma(self, cr, uid, cod_forma_ids, context=None, load='_classic_write'):
        context = context and context.copy() or {}
        dict_res = {}
        dict_forma_cond = {}
        cond_ids = []
        for i in self.pool.get('adm.condicao.forma').read(cr, uid, cod_forma_ids, ['adm_condicao_id'],
                                                     context, load):
            dict_forma_cond.update({i.get('id'): i.get('adm_condicao_id')})
            cond_ids.append(i.get('adm_condicao_id'))

        dict_cond = {}
        map(lambda x: dict_cond.update({i.get('id'): i.get('tipo')}),
            self.pool.get('adm.condicao').read(cr, uid, cond_ids, ['tipo'], context))

        for i, j in dict_forma_cond.items():
            dict_res[i] = dict_cond.get(j)

        return dict_res

    def _get_dict_forma(self, cr, uid, forma_ids, context=None, load='_classic_write'):
        context = context and context.copy() or {}
        dict_res = {}
        for i in self.pool.get('adm.forma.pgto').read(cr, uid, forma_ids, ['cartao_credito_debito'],
                                             context, load):
            dict_res[i.get('id')] = i.get('cartao_credito_debito')

        return dict_res

    def venutl_get_nf_total_a_prazo(self, cr, uid, venutl_pessoa_ids, venutl_nf_id=False, venutl_exclude_nf_id=False,
                                    context=None):
        """
        FUNÇÃO BUSCA O DEBITO A PRAZO NA NOTA FISCAL
        COMO A BUSCA É FEITA:

        * venutl_nf_id INFORMADO:
            - BUSCA SOMENTE O DEBITO DESTA NOTA INFORMADA

        * venutl_nf_id NÃO INFORMADO:
            - BUSCA O VALOR DO DEBITO DE TODAS AS NOTAS EM DETERMINADOS STATUS
            * venutl_exclude_nf_id INFORMADO:
                - FAZ A PESQUISA CITADA A CIMA, POREM, EXCLUIDO A NOTA INFORMADA DA BUSCA.

        RETORNA DICIONARIO, SENDO A CHAVE O ID DA PESSOA E O VALOR DA CHAVE O TOTAL DO DEBITO
        A PRAZO NA(S) NOTA(S)
        """

        context = context and context.copy() or {}
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_financeiro_util = self.pool.get('financeiro')
        load = '_classic_write'
        dict_total_debito = {}
        total_debito_nf = 0
        if isinstance(venutl_pessoa_ids, (int, long)):
            venutl_pessoa_ids = [venutl_pessoa_ids]
        if venutl_pessoa_ids:
            obj_nf_capa = self.pool.get('fiscal.nf.capa')
            obj_natureza = self.pool.get('fiscal.cfop.natureza')
            obj_cfop = self.pool.get('fiscal.cfop')
            obj_condicoes = self.pool.get('fiscal.condicao.pagamento')
            if venutl_nf_id and isinstance(venutl_nf_id, (int, long)):
                nf_ids = [venutl_nf_id]
            else:
                args_nf = [
                    ('destinatario_id', 'in', venutl_pessoa_ids), ('tipo', '=', 'S'),
                    ('state', 'not in', ('1', '3', 'C', 'I', 'AI', 'D'))
                ]
                if venutl_exclude_nf_id:
                    if not isinstance(venutl_exclude_nf_id, list):
                        venutl_exclude_nf_id = [venutl_exclude_nf_id]

                    args_nf.append(('id', 'not in', venutl_exclude_nf_id))

                cliente_ids = list()
                cliente_ids.extend(obj_financeiro_util.finutl_get_cliente_ids(cr, uid,
                                                                        venutl_pessoa_ids, context))

                company_ids = list()
                map(lambda i: company_ids.append(i.get('company_id')),
                    obj_financeiro_cliente.read(cr, uid, cliente_ids, ['company_id'],
                                                context=context, load=load))
                company_ids = list(set(company_ids))
                args_nf.append(('company_id', 'in', company_ids))
                context['sig_force_company'] = True

                nf_ids = obj_nf_capa.search(cr, uid, args_nf, context=context)
            if nf_ids:
                dados_nfs = obj_nf_capa.read(cr, uid, nf_ids, ['natureza_id', 'destinatario_id',
                                                             'nf_condicao_ids'], context, load=load)
                natureza_ids = map(lambda x: x.get('natureza_id'), dados_nfs)
                dados_natureza = obj_natureza.read(cr, uid, natureza_ids, ['cfop_id'], context,
                                                   load=load)
                dict_nat_cfop = {}
                cfop_ids = []
                for i in dados_natureza:
                    dict_nat_cfop[i.get('id')] = i.get('cfop_id')
                    cfop_ids.append(i.get('cfop_id'))

                dict_cfop = {}
                map(lambda x: dict_cfop.update({x.get('id'): x}),
                    obj_cfop.read(cr, uid, cfop_ids, ['movimento_tipo'], context))
                dados_nf_nao_devolucao = filter(lambda i: i.get('natureza_id') and
                       dict_cfop.get(dict_nat_cfop.get(i.get('natureza_id'))).get('movimento_tipo') != 'D',
                                                 dados_nfs)
                condicoes_ids = []
                dict_nf_condicoes = {}
                for i in dados_nf_nao_devolucao:
                    condicoes_ids.extend(i.get('nf_condicao_ids'))
                    dict_nf_condicoes[i.get('id')] = i.get('nf_condicao_ids')

                dict_condicoes = {}
                cond_forma_ids = []
                forma_ids = []
                for i in obj_condicoes.read(cr, uid, condicoes_ids, ['valor',
                                        'condicao_forma_id', 'forma_pagto_id'], context, load=load):
                    dict_condicoes.update({i.get('id'): i})
                    cond_forma_ids.append(i.get('condicao_forma_id'))
                    forma_ids.append(i.get('forma_pagto_id'))

                dict_cond_forma = self._get_dict_con_forma(cr, uid, cond_forma_ids, context, load)
                dict_forma = self._get_dict_forma(cr, uid, forma_ids, context, load)
                for i in dados_nf_nao_devolucao:
                    pessoa_id = i.get('destinatario_id')
                    # FILTRA SOMENTE AS CONDIÇÕES QUE A CONDIÇÃO DE PAGAMENTO SEJA A PRAZO E
                    # A FORMA DE PAGAMENTO NÃO SEJA CARTAO DE CREDITO.
                    condicoes = []
                    for j in i.get('nf_condicao_ids'):
                        condicoes.append(dict_condicoes.get(j))
                    record_condicoes_a_prazo = filter(
                        lambda c: c.get('condicao_forma_id') and
                                  dict_cond_forma.get(c.get('condicao_forma_id')) != 'V' and
                                  not dict_forma.get(c.get('forma_pagto_id')), condicoes
                    )
                    total_debito_nf = sum(map(lambda c: c.get('valor'),
                                              record_condicoes_a_prazo or []))
                    if pessoa_id and not dict_total_debito.get(pessoa_id):
                        dict_total_debito[pessoa_id] = total_debito_nf
                    elif pessoa_id and dict_total_debito.get(pessoa_id):
                        dict_total_debito[pessoa_id] += total_debito_nf

        return dict_total_debito

    def venutl_get_pedido_total_a_prazo(self, cr, uid, venutl_pessoa_ids, venutl_pedido_id=False,
                                        venutl_exclude_pedido_id=False, context=None):
        """
        FUNÇÃO BUSCA O DEBITO A PRAZO NO PEDIDO
        COMO A BUSCA É FEITA:

        * venutl_pedido_id INFORMADO:
            - BUSCA SOMENTE O DEBITO DESTE PEDIDO INFORMADO

        * venutl_pedido_id NÃO INFORMADO:
            - BUSCA O VALOR DO DEBITO DE TODOS OS PEDIDOS EM DETERMINADOS STATUS
            * venutl_exclude_pedido_id INFORMADO:
                - FAZ A PESQUISA CITADA A CIMA, POREM, EXCLUIDO O PEDIDO INFORMADO DA BUSCA.

        RETORNA DICIONARIO, SENDO A CHAVE O ID DA PESSOA E O VALOR DA CHAVE O TOTAL DO DEBITO
        A PRAZO NO(S) PEDIDO(S)
        """

        context = context and context.copy() or {}
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_financeiro_util = self.pool.get('financeiro')
        load = '_classic_write'
        dict_total_debito = {}
        if isinstance(venutl_pessoa_ids, (int, long)):
            venutl_pessoa_ids = [venutl_pessoa_ids]
        total_debito_pedido = 0
        if venutl_pessoa_ids:
            obj_pedido = self.pool.get('estoquepedidos.pedido.capa')
            if venutl_pedido_id and isinstance(venutl_pedido_id, (int, long)):
                pedido_ids = [venutl_pedido_id]
            else:
                args_pedido = [('state', 'not in', ('R', 'F', 'C')),
                               ('pessoa_id', 'in', venutl_pessoa_ids),
                               ('tipo', '=', 'V'),
                ]
                if venutl_exclude_pedido_id:
                    if not isinstance(venutl_exclude_pedido_id, list):
                        venutl_exclude_pedido_id = [venutl_exclude_pedido_id]

                    args_pedido.append(('id', 'not in', venutl_exclude_pedido_id))

                cliente_ids = list()
                cliente_ids.extend(obj_financeiro_util.finutl_get_cliente_ids(cr, uid,
                                                                        venutl_pessoa_ids, context))

                company_ids = list()
                map(lambda i: company_ids.append(i.get('company_id')),
                    obj_financeiro_cliente.read(cr, uid, cliente_ids, ['company_id'], context,
                                                load=load))
                company_ids = list(set(company_ids))
                args_pedido.append(('company_id', 'in', company_ids))
                context['sig_force_company'] = True

                pedido_ids = obj_pedido.search(cr, uid, args_pedido, context=context)
            if pedido_ids:
                dados_pedido = obj_pedido.read(cr, uid, pedido_ids, ['pessoa_id',
                                                         'pedido_condicao_ids'], context, load=load)
                condicoes_ids = []
                for i in dados_pedido:
                    condicoes_ids.extend(i.get('pedido_condicao_ids'))

                dict_condicoes = {}
                cond_forma_ids = []
                forma_ids = []
                for i in self.pool.get('estoquepedidos.pedido.condicao').read(cr, uid,
                                    condicoes_ids, ['condicao_forma_id', 'forma_pagto_id', 'valor'],
                                                                        context=context, load=load):
                    dict_condicoes.update({i.get('id'): i})
                    cond_forma_ids.append(i.get('condicao_forma_id'))
                    forma_ids.append(i.get('forma_pagto_id'))

                dict_cond_forma = self._get_dict_con_forma(cr, uid, cond_forma_ids, context, load)
                dict_forma = self._get_dict_forma(cr, uid, forma_ids, context, load)

                for i in dados_pedido:
                    pessoa_id = i.get('pessoa_id')
                    # FILTRA SOMENTE AS CONDIÇÕES QUE A CONDIÇÃO DE PAGAMENTO SEJA A PRAZO E
                    # A FORMA DE PAGAMENTO NÃO SEJA CARTAO DE CREDITO.
                    condicoes = []
                    for j in i.get('pedido_condicao_ids'):
                        condicoes.append(dict_condicoes.get(j))

                    record_condicoes_a_prazo = filter(
                        lambda c: c.get('condicao_forma_id') and dict_cond_forma.get(c.get('condicao_forma_id')) != 'V' and
                                  not dict_forma.get(c.get('forma_pagto_id')), condicoes
                    )
                    total_debito_pedido = sum(map(lambda c: c.get('valor'), record_condicoes_a_prazo or []))
                    if pessoa_id and not dict_total_debito.get(pessoa_id):
                        dict_total_debito[pessoa_id] = total_debito_pedido
                    elif pessoa_id and dict_total_debito.get(pessoa_id):
                        dict_total_debito[pessoa_id] += total_debito_pedido

        return dict_total_debito

    def venutl_get_orcamento_total_a_prazo(self, cr, uid, venutl_pessoa_ids, venutl_orcamento_id=False,
                                           venutl_exclude_orcamento_id=False, context=None):
        """
        FUNÇÃO BUSCA O DEBITO A PRAZO NO ORÇAMENTO
        COMO A BUSCA É FEITA:

        * venutl_orcamento_id INFORMADO:
            - BUSCA SOMENTE O DEBITO DESTE ORÇAMENTO INFORMADO

        * venutl_orcamento_id NÃO INFORMADO:
            - BUSCA O VALOR DO DEBITO DE TODOS OS ORÇAMENTOS EM DETERMINADOS STATUS
            * venutl_exclude_orcamento_id INFORMADO:
                - FAZ A PESQUISA CITADA A CIMA, POREM, EXCLUIDO O ORÇAMENTO INFORMADO DA BUSCA.

        RETORNA DICIONARIO, SENDO A CHAVE O ID DA PESSOA E O VALOR DA CHAVE O TOTAL DO DEBITO
        A PRAZO NO(S) ORÇAMENTO(S)
        """
        context = context and context.copy() or {}
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_financeiro_util = self.pool.get('financeiro')
        load = '_classic_write'
        dict_total_debito = {}
        if isinstance(venutl_pessoa_ids, (int, long)):
            venutl_pessoa_ids = [venutl_pessoa_ids]
        total_debito_orcamento = 0
        if venutl_pessoa_ids:
            obj_orcamento = self.pool.get('orcamentos.orcamento.capa')
            if venutl_orcamento_id and isinstance(venutl_orcamento_id, (int, long)):
                orc_ids = [venutl_orcamento_id]
            else:
                args_orcamento = [
                    ('pessoa_id', 'in', venutl_pessoa_ids), ('state', 'not in', ('1E', '4G', '5C'))
                ]
                if venutl_exclude_orcamento_id:
                    if not isinstance(venutl_exclude_orcamento_id, list):
                        venutl_exclude_orcamento_id = [venutl_exclude_orcamento_id]

                    args_orcamento.append(('id', 'not in', venutl_exclude_orcamento_id))

                cliente_ids = list()
                cliente_ids.extend(obj_financeiro_util.finutl_get_cliente_ids(cr, uid,
                                                                        venutl_pessoa_ids, context))

                company_ids = list()
                map(lambda i: company_ids.append(i.get('company_id')),
                    obj_financeiro_cliente.read(cr, uid, cliente_ids, ['company_id'],
                                                context=context, load=load))
                company_ids = list(set(company_ids))
                args_orcamento.append(('company_id', 'in', company_ids))
                context['sig_force_company'] = True

                orc_ids = obj_orcamento.search(cr, uid, args_orcamento, context=context)
            if orc_ids:
                dados_orcamentos = obj_orcamento.read(cr, uid, orc_ids, ['orcamento_condicao_ids',
                                                      'pessoa_id'], context, load=load)
                condicoes_ids = []
                for i in dados_orcamentos:
                    condicoes_ids.extend(i.get('orcamento_condicao_ids'))

                dict_condicoes = {}
                map(lambda x: dict_condicoes.update({x.id: x}),
                    self.pool.get('orcamentos.orcamento.condicao').browse(cr, uid, condicoes_ids,
                                                                          context=context))
                for i in dados_orcamentos:

                    pessoa_id = i.get('pessoa_id')
                    condicoes = []
                    for j in i.get('orcamento_condicao_ids'):
                        condicoes.append(dict_condicoes.get(j))

                    record_condicoes_aprovadas = filter(lambda c: c.aprovado, condicoes)
                    if record_condicoes_aprovadas or len(condicoes or []) == 1:
                        # FILTRA SOMENTE AS CONDIÇÕES QUE A CONDIÇÃO DE PAGAMENTO SEJA A PRAZO E
                        # A FORMA DE PAGAMENTO NÃO SEJA CARTAO DE CREDITO.
                        record_condicoes_a_prazo_aprovadas = filter(
                            lambda c: c.condicao_id and c.condicao_id.adm_condicao_id.tipo != 'V' and
                                      not c.forma_pagto_id.cartao_credito_debito, record_condicoes_aprovadas
                        )
                        if record_condicoes_a_prazo_aprovadas:
                            total_debito_orcamento += sum(
                                map(lambda c: c.valor, record_condicoes_a_prazo_aprovadas or [])
                            )
                    else:
                        record_condicoes_a_prazo = filter(
                            lambda c: c.condicao_id and c.condicao_id.adm_condicao_id.tipo != 'V' \
                                      and not c.forma_pagto_id.cartao_credito_debito, condicoes
                        )
                        if record_condicoes_a_prazo:
                            total_debito_orcamento += max(map(lambda c: c.valor, record_condicoes_a_prazo or [0]))
                    if pessoa_id and not dict_total_debito.get(pessoa_id):
                        dict_total_debito[pessoa_id] = total_debito_orcamento
                    elif pessoa_id and dict_total_debito.get(pessoa_id):
                        dict_total_debito[pessoa_id] += total_debito_orcamento
        return dict_total_debito

    def venutl_get_nf_valor_liquido_bruto_desconto(self, cr, uid, venutl_nf_id, context=None):
        context = context and context.copy() or {}
        if isinstance(venutl_nf_id, list):
            venutl_nf_id = venutl_nf_id[0]
        modulo_madeireira_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        obj_fiscal_nf_capa = self.pool.get('fiscal.nf.capa')
        browse_nf = obj_fiscal_nf_capa.browse(cr, uid, venutl_nf_id, context)
        if browse_nf.tipo_frete == 'F':
            vr_frete = 0
        else:
            vr_frete = browse_nf.vr_frete or 0
        vr_seguro = browse_nf.vr_seguro or 0
        vr_desconto = browse_nf.vr_desconto or 0
        vr_despesas_acessorias = browse_nf.vr_despesas_acessorias or 0
        vr_icms_substituicao = browse_nf.vr_icms_substituicao or 0
        vr_icms_desonerado = browse_nf.fisnfc_valor_icms_desonerado or 0
        vr_ipi = browse_nf.vr_ipi or 0
        vr_total_deducao = sum(map(
            lambda deducoes: deducoes.valor, browse_nf.deducao_itens_ids or [])) or 0
        total_itens = total_desconto_itens = 0

        # O metodo ´read´ foi implementado porque o ´browse´ nao executa o metodo ´read´ que foi
        # sobrescrito na classe ´estoque.movimento.itens´
        for i in self.pool.get('estoque.movimento.itens').read(
                cr, uid, map(lambda m: m.id, browse_nf.estoque_movimento_itens_ids), [
                    'vr_desconto', 'vr_acrescimos', 'quantidade', 'estmvi_volume', 'vr_unitario'],
                context):
            quantidade = i.get('quantidade')
            if modulo_madeireira_instalado:
                quantidade = i.get('estmvi_volume') or quantidade
            total_itens += i.get('vr_acrescimos') or 0 + quantidade * i.get('vr_unitario')
            total_itens -= i.get('vr_desconto')
            total_itens = float(obj_estoque_movimento_itens._float_persist(
                'vr_total_item',  total_itens))
            total_desconto_itens += i.get('vr_desconto')

        nf_valor_liquido = total_itens + vr_seguro + vr_frete + vr_despesas_acessorias + vr_ipi + \
            vr_icms_substituicao - vr_total_deducao - vr_desconto - vr_icms_desonerado
        nf_valor_bruto = nf_valor_liquido + total_desconto_itens + vr_desconto
        nf_valor_desconto = vr_desconto + total_desconto_itens

        return nf_valor_liquido, nf_valor_bruto, nf_valor_desconto

    def venutl_get_pedido_valor_liquido_bruto_desconto(self, cr, uid, venutl_pedido_id, context=None):
        context = context and context.copy() or {}
        if isinstance(venutl_pedido_id, list):
            venutl_pedido_id = venutl_pedido_id[0]
        obj_adm_util = self.pool.get('adm.utils')
        modulo_fiscal_instalado = obj_adm_util.admutl_check_modulo_instalado(
            cr, uid, 'fiscal', context)
        modulo_madeireira_instalado = obj_adm_util.admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        total_itens = total_desconto_itens = 0
        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        browse_pedido = obj_estoquepedidos_pedido_capa.browse(cr, uid, venutl_pedido_id, context)
        pedcap_valor_ipi = 0
        if modulo_fiscal_instalado:
            pedcap_valor_ipi = browse_pedido.pedcap_valor_ipi
        vr_frete = browse_pedido.vr_frete or 0
        vr_seguro = browse_pedido.vr_seguro or 0
        vr_desconto = browse_pedido.vr_desconto or 0
        vr_despesas_acessorias = browse_pedido.vr_despesas_acessorias or 0
        orcamento_id = browse_pedido.orcamento_id.id or False

        for i in browse_pedido.pedido_item_ids:
            item_desconto = i.vr_desconto
            item_quantidade = i.qtde_atendida or i.qtde_pedida
            if modulo_madeireira_instalado:
                item_quantidade = i.pedite_volume or item_quantidade
            item_valor_unitario = i.vr_unitario

            total_itens += item_quantidade * item_valor_unitario - item_desconto
            total_itens = float(obj_estoquepedidos_pedido_capa._float_persist('valor_total',
                                                                              total_itens))
            total_desconto_itens += item_desconto

        if orcamento_id:
            pedido_valor_liquido = total_itens + vr_seguro + vr_frete + vr_despesas_acessorias + pedcap_valor_ipi - vr_desconto
            pedido_valor_bruto = pedido_valor_liquido + vr_desconto
            pedido_valor_desconto = vr_desconto
        else:
            pedido_valor_liquido = total_itens + vr_seguro + vr_frete + vr_despesas_acessorias + pedcap_valor_ipi - vr_desconto
            pedido_valor_bruto = pedido_valor_liquido + total_desconto_itens
            pedido_valor_desconto = total_desconto_itens + vr_desconto

        return pedido_valor_liquido, pedido_valor_bruto, pedido_valor_desconto

    def venutl_get_orcamento_valor_liquido_bruto_desconto(self, cr, uid, venutl_orcamento_id, context=None):
        context = context and context.copy() or {}
        if isinstance(venutl_orcamento_id, list):
            venutl_orcamento_id = venutl_orcamento_id[0]
        orc_valor_liquido = orc_valor_bruto = orc_valor_desconto = 0
        fields_to_read = ['valor_liquido', 'valor_desconto_total', 'orcamento_itens_ids']
        orcamento_dados = self.pool.get('orcamentos.orcamento.capa').read(cr, uid, venutl_orcamento_id, fields_to_read,
                                                                          context=context)
        if orcamento_dados and isinstance(orcamento_dados, list):
            orcamento_dados = orcamento_dados[0]
        if isinstance(orcamento_dados, dict):
            valor_liquido = orcamento_dados.get('valor_liquido') or 0
            # O resultado do desconto da capa do orcamento ja eh a soma do desconto dos itens mais
            # da capa
            desconto_capa = orcamento_dados.get('valor_desconto_total') or 0

            orc_valor_liquido = valor_liquido
            orc_valor_desconto = desconto_capa
            orc_valor_bruto = orc_valor_liquido + orc_valor_desconto

        return orc_valor_liquido, orc_valor_bruto, orc_valor_desconto

    def venutl_get_valor_liquido_bruto_desconto(self, cr, uid, DOCUMENTO_ID, DOCUMENTO_TIPO, context):
        """
        DOCUMENTO TIPO
        'ORC' - ORCAMENTO
        'PED' - PEDIDO
        'NFL' - NOTA FISCAL
        """
        ORCAMENTO = 'ORC'
        PEDIDO = 'PED'
        NF = 'NFL'

        if DOCUMENTO_TIPO == NF:
            valor_liquido, valor_bruto, valor_desconto = self.venutl_get_nf_valor_liquido_bruto_desconto(
                cr, uid, venutl_nf_id=DOCUMENTO_ID, context=context
            )
        elif DOCUMENTO_TIPO == PEDIDO:
            valor_liquido, valor_bruto, valor_desconto = self.venutl_get_pedido_valor_liquido_bruto_desconto(
                cr, uid, venutl_pedido_id=DOCUMENTO_ID, context=context
            )
        else:
            valor_liquido, valor_bruto, valor_desconto = self.venutl_get_orcamento_valor_liquido_bruto_desconto(
                cr, uid, venutl_orcamento_id=DOCUMENTO_ID, context=context
            )

        return valor_liquido, valor_bruto, valor_desconto

    # TODO: Separar para orcamento, pedido e nota
    def venutl_get_total_debito_cliente(self, cr, uid, venutl_pessoa_ids, venutl_doc_tipo='',
                                        venutl_doc_id=False, venutl_exclude_doc_id=False, context=None):
        """
        DOCUMENTO TIPO
        'ORC' - ORCAMENTO
        'PED' - PEDIDO
        'NFL' - NOTA FISCAL

        SE ALGUM DOS PARAMETROS (venutl_doc_id, venutl_exclude_doc_id) FOR PASSADO,
        O PARAMETRO (venutl_doc_tipo) DEVE SER INFORMADO.

        RETORNA DICIONARIO, SENDO A CHAVE O ID DA PESSOA E O VALOR DA CHAVE O TOTAL DO DEBITO
        """
        context = context and context.copy() or {}
        ORCAMENTO = 'ORC'
        PEDIDO = 'PED'
        NF = 'NFL'

        # VERIFICA SE O MODULO FISCAL ESTA INSTALADO
        modulo_fiscal_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'fiscal', context)
        # VERIFICA SE O MODULO ORCAMENTO ESTA INSTALADO
        modulo_orcamento_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid, 'orcamentos',
                                                                                              context)
        # VERIFICA SE O MODULO ESTOQUEPEDIDOS ESTA INSTALADO
        modulo_estoquepedido_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, uid,
                                                                                                  'estoquepedidos',
                                                                                                  context)
        dict_total_debito_nf = {}
        dict_total_debito_orcamento = {}
        dict_total_debito_pedido = {}

        tipo_documento = (venutl_doc_tipo or '').upper()
        if isinstance(venutl_pessoa_ids, (int, long)):
            venutl_pessoa_ids = [venutl_pessoa_ids]

        if venutl_doc_id:
            # BUSCA O VALOR A PRAZO DO SOMENTE DO DOCUMENTO (ORCAMENTO, PEDIDO OU NF) INFORMADO.
            # TODO: Mover para a classe ´fiscal´ como heranca
            if modulo_fiscal_instalado and tipo_documento == NF:
                dict_total_debito_nf = self.venutl_get_nf_total_a_prazo(
                    cr, uid, venutl_pessoa_ids, venutl_nf_id=venutl_doc_id, context=context
                )
            if modulo_orcamento_instalado and tipo_documento == ORCAMENTO:
                dict_total_debito_orcamento = self.venutl_get_orcamento_total_a_prazo(
                    cr, uid, venutl_pessoa_ids, venutl_orcamento_id=venutl_doc_id, context=context
                )
            if modulo_estoquepedido_instalado and tipo_documento == PEDIDO:
                dict_total_debito_pedido = self.venutl_get_pedido_total_a_prazo(
                    cr, uid, venutl_pessoa_ids, venutl_pedido_id=venutl_doc_id, context=context
                )

        else:
            # BUSCA O VALOR A PRAZO DE TODOS OS DOCUMENTOS (ORCAMENTO, PEDIDO E NF) PARA A PESSOA INFORMADA.
            # SE O PARAMETRO venutl_exclude_doc_id FOR INDICADO, ESTE DOCUMENTO NÃO ENTRARÁ NA BUSCA.
            if venutl_exclude_doc_id:
                if tipo_documento == NF:
                    exclude_nf_id = venutl_exclude_doc_id
                    exclude_pedido_id = False
                elif tipo_documento == PEDIDO:
                    exclude_nf_id = False
                    exclude_pedido_id = venutl_exclude_doc_id
                elif tipo_documento == ORCAMENTO:
                    exclude_nf_id = False
                    exclude_pedido_id = False
                else:
                    exclude_nf_id = False
                    exclude_pedido_id = False
            else:
                exclude_nf_id = False
                exclude_pedido_id = False

            if modulo_fiscal_instalado:
                dict_total_debito_nf = self.venutl_get_nf_total_a_prazo(
                    cr, uid, venutl_pessoa_ids, venutl_exclude_nf_id=exclude_nf_id, context=context
                )
            if modulo_estoquepedido_instalado:
                dict_total_debito_pedido = self.venutl_get_pedido_total_a_prazo(
                    cr, uid, venutl_pessoa_ids, venutl_exclude_pedido_id=exclude_pedido_id, context=context
                )

        dict_cliente_valor_total_debito = {}
        if any([dict_total_debito_nf, dict_total_debito_pedido, dict_total_debito_orcamento]):
            for pessoa_id in venutl_pessoa_ids:
                debito_nf = dict_total_debito_nf.get(pessoa_id) or 0
                debito_orcamento = dict_total_debito_orcamento.get(pessoa_id) or 0
                debito_pedido = dict_total_debito_pedido.get(pessoa_id) or 0
                valor_total_debito = debito_nf + debito_orcamento + debito_pedido
                dict_cliente_valor_total_debito[pessoa_id] = valor_total_debito
        return dict_cliente_valor_total_debito

    # TODO: Mover para a classe ´orcamento´ como heranca
    def venutl_get_total_debito_orcamento(self, cr, user, ids, context=None):
        """
        Retorna total do debito de orcamentos realizados.
        """
        context = context and context.copy() or {}
        result = dict()

        # Verifica se o modulo orcamento esta instalado
        modulo_orcamento_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, user, 'orcamentos', context)

        # Busca o valor a prazo do orcamento
        if modulo_orcamento_instalado:
            result = self.venutl_get_orcamento_total_a_prazo(cr, user, ids, context=context)

        return result

    # TODO: Mover para a classe ´estoquepedidos´ como heranca
    def venutl_get_total_debito_pedido(self, cr, user, ids, context=None):
        """
        Retorna total de debitos para pedidos efetuados.
        """
        context = context and context.copy() or {}
        result = dict()

        # Verifica se o modulo estoquepedidos esta instalado
        modulo_estoquepedido_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, user, 'estoquepedidos', context)

        # Busca o valor a prazo do pedido
        if modulo_estoquepedido_instalado:
            result = self.venutl_get_pedido_total_a_prazo(cr, user, ids, context=context)

        return result

    # TODO: Mover para a classe ´fiscal´ como heranca
    def venutl_get_total_debito_nota_fiscal(self, cr, user, ids, context=None):
        """
        Retorna total de debitos  em notas fiscais emitidas.
        """
        context = context and context.copy() or {}
        result = dict()

        # Verifica se o modulo orcamento esta instalado
        modulo_fiscal_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, user, 'fiscal', context)

        # Busca o valor a prazo do orcamento
        if modulo_fiscal_instalado:
            result = self.venutl_get_nf_total_a_prazo(cr, user, ids, context=context)

        return result

    def venutl_comparar_restricao_cadastrada_com_restricao_atual(self, cr, uid, restricao_ids, venutl_total_debito,
                                                                 venutl_limite_credito, venutl_total_titulo_atrasado,
                                                                 venutl_bloqueado_por_msg, context=None):

        context = context and context.copy() or {}
        dif = False
        fields_to_read = [
            'venlre_autorizado', 'venlre_valor_documento', 'venlre_limite_debito',
            'venlre_bloqueado_por_msg', 'venlre_valor_limite_credito', 'venlre_valor_titulos_atrasados',
        ]
        restricao_dados = self.pool.get('venlre.liberar.restricoes').read(cr, uid, restricao_ids, fields_to_read,
                                                                          context=context)
        if restricao_dados and isinstance(restricao_dados, list):
            restricao_dados = restricao_dados[0]
        if isinstance(restricao_dados, dict):
            rest_valor_documento = restricao_dados.get('venlre_valor_documento') or 0
            rest_bloqueado_por_msg = restricao_dados.get('venlre_bloqueado_por_msg') or False
            rest_valor_limite_debito = restricao_dados.get('venlre_limite_debito') or 0
            rest_valor_limite_credito = restricao_dados.get('venlre_valor_limite_credito') or 0
            rest_valor_titulos_atrasados = restricao_dados.get('venlre_valor_titulos_atrasados') or 0
            rest_autorizada = restricao_dados.get('venlre_autorizado') or False
            rest_total_debito = rest_valor_documento + rest_valor_limite_debito

            if not rest_autorizada:
                dif = True

            if venutl_limite_credito:
                if round(venutl_limite_credito, 2) != round(rest_valor_limite_credito, 2):
                    dif = True
                elif round(venutl_total_debito, 2) != round(rest_total_debito, 2):
                    if round((rest_total_debito - rest_valor_documento), 2) != round(rest_valor_limite_debito, 2):
                        dif = True

            if venutl_bloqueado_por_msg != rest_bloqueado_por_msg:
                dif = True
            elif round(venutl_total_titulo_atrasado, 2) != round(rest_valor_titulos_atrasados, 2):
                dif = True
        return dif

    def venutl_documento_restricao_venda(self, cr, uid, documento_id, pessoa_id, documento_state,
                                         documento_tipo, context=None):
        """
        * DOCUMENTO_ID - ID do orcamento, pedido ou nf
        * DOCUMENTO_STATE - State do orcamento, pedido ou nf
        """

        context = context and context.copy() or {}
        self._tem_erro = u''
        self._mensagem_erro = u''
        TIPO_DOCUMENTO = documento_tipo.upper()
        if TIPO_DOCUMENTO == 'NFL':
            ESBOCO = '1'
            MODULO_NAME = 'Notas Fiscais'
            DOC_FIELD_M2O = 'venlre_nf_id'
        elif TIPO_DOCUMENTO ==  'ORC':
            ESBOCO = '1E'
            MODULO_NAME = 'Orcamentos'
            DOC_FIELD_M2O = 'venlre_orcamento_id'
        else:  # 'PED'
            ESBOCO = 'R'
            MODULO_NAME = 'Pedidos'
            DOC_FIELD_M2O = 'venlre_pedido_id'

        # Objetos
        obj_liberar_restricoes = self.pool.get('venlre.liberar.restricoes')

        # Controles
        erro_restricao = u''
        erro_cliente_restricao_venda = u''
        erro_ped_venda_nao_autorizada = u''
        info_ped_liberacao_restricao_venda_criada = u''

        # Nome dos pedidos
        if TIPO_DOCUMENTO == 'NFL':
            dict_doc_names = self.pool.get('fiscal.utils').fisutl_get_nf_names(
                cr, uid, documento_id, context)
        elif TIPO_DOCUMENTO == 'ORC':
            dict_doc_names = self.pool.get('orcamento.utils').orcutl_get_orcamento_names(
                cr, uid, documento_id, context)
        else:  # 'PED'
            dict_doc_names = self.pool.get('pedido.utils').pedutl_get_pedido_names(
                cr, uid, documento_id, context)
        documento_name = \
            unicode(dict_doc_names.get(documento_id) or '') or '** Nao encontrado **'

        # Restricao de venda
        criar_restricao = True

        obj_financeiro_utils = self.pool.get('financeiro')

        # Titulos em atraso
        titulo_em_atraso = obj_financeiro_utils.finutl_get_valor_em_atraso(
            cr, uid, venutl_pessoa_ids=pessoa_id, context=context)
        titulo_em_atraso = titulo_em_atraso.get(pessoa_id) or 0

        # Bloqueado por mensagem
        bloqueado_por_mensagem = self.venutl_cliente_bloqueado_por_msg(
            cr, uid, venutl_pessoa_ids=pessoa_id, context=context).get(
            'pessoa_ids').get(pessoa_id) or False
        if bloqueado_por_mensagem:
            bloqueado_por_mensagem = True

        # Limite de credito
        cliente_limite_credito = self.venutl_get_limite_credito_cliente(
            cr, uid, pessoa_ids=pessoa_id, context=context).get(
            'pessoa_ids').get(pessoa_id)

        # Decresce o total do limite de credito em funcao dos valores em aberto
        novo_limite_credito = cliente_limite_credito
        adm_pessoa_dados = self.pool.get('adm.pessoa').read(
            cr, uid, pessoa_id, ['cliente_ids'], context)
        for financeiro_cliente_dado in self.pool.get('financeiro.cliente').read(
                cr, uid, adm_pessoa_dados and adm_pessoa_dados.get('cliente_ids', []) or [],
                ['fincli_total_em_aberto'], context):
            novo_limite_credito -= financeiro_cliente_dado.get('fincli_total_em_aberto', 0)

        # Bloqueado por limite de credito
        bloqueado_por_credito = False
        dados_parametro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', ['verificar_limite_credito'])

        # Debito anteior
        cliente_debito = self.venutl_get_total_debito_cliente(
            cr, uid, venutl_pessoa_ids=pessoa_id, venutl_doc_tipo=TIPO_DOCUMENTO,
            context=context)

        if documento_state == ESBOCO:
            # Orcamento em esboco nao tem o valor de debito calculado. sendo assim, eh necessario
            # chamar a funcao informando o documento para buscar o debito
            debito_nota_atual = self.venutl_get_total_debito_cliente(
                cr, uid, venutl_pessoa_ids=pessoa_id, venutl_doc_tipo=TIPO_DOCUMENTO,
                venutl_doc_id=documento_id, context=context)
            debito = cliente_debito.get(pessoa_id) or 0
            debito_nota_atual = debito_nota_atual.get(pessoa_id) or 0
            cliente_debito[pessoa_id] = debito + debito_nota_atual

        if dados_parametro.get('verificar_limite_credito', False):
            debito = cliente_debito.get(pessoa_id) or 0
            if debito > novo_limite_credito:
                bloqueado_por_credito = True
        if any([titulo_em_atraso, bloqueado_por_mensagem, bloqueado_por_credito]):
            # Verifica se jah existe uma liberacao de restricao para o documento
            restricao_pedido_ids = obj_liberar_restricoes.search(
                cr, uid, [(DOC_FIELD_M2O, '=', documento_id)], context=context)
            if restricao_pedido_ids:
                criar_restricao = False

                # Verifica se a situacao atual de restricao do cliente eh a mesma de quando a
                # liberacao foi autorizada
                dif = self.venutl_comparar_restricao_cadastrada_com_restricao_atual(
                    cr, uid, restricao_ids=restricao_pedido_ids,
                    venutl_total_debito=cliente_debito.get(pessoa_id) or 0,
                    venutl_limite_credito=cliente_limite_credito,
                    venutl_total_titulo_atrasado=titulo_em_atraso,
                    venutl_bloqueado_por_msg=bloqueado_por_mensagem,  context=context)
                if dif:
                    # Se a situacao for diferente, a autorizacao eh marcada com false
                    obj_liberar_restricoes.write(
                        cr, uid, restricao_pedido_ids, {'venlre_autorizado': False}, context)
                    erro_ped_venda_nao_autorizada += '%s, ' % documento_name
            else:
                erro_cliente_restricao_venda = '%s, ' % documento_name

            if criar_restricao:
                # Cria a restricao automaticamente
                vals_restricao_venda = {DOC_FIELD_M2O: documento_id}
                obj_liberar_restricoes.create(cr, uid, vals_restricao_venda, context)

                # O ´commit´ eh necessario pois se houver um ´raise´, a transacao para criar a
                # restricao de venda nao sera desfeita
                cr.commit()
                info_ped_liberacao_restricao_venda_criada += '%s, ' % documento_name
        if erro_cliente_restricao_venda:
            erro_restricao += _('* [CLIENTE COM RESTRICAO DE VENDA] %s: %s') % (
                MODULO_NAME, erro_cliente_restricao_venda)
        if erro_ped_venda_nao_autorizada:
            erro_restricao += _('* [RESTRICAO VENDA NAO AUTORIZADA] %s: %s') % (
                MODULO_NAME, erro_ped_venda_nao_autorizada)
        if info_ped_liberacao_restricao_venda_criada:
            erro_restricao += _('** [CRIADA] - Liberacao de restricao de venda criada para os '
                                '%s: %s') % (MODULO_NAME, info_ped_liberacao_restricao_venda_criada)
        if erro_restricao:
            self._mensagem_erro = erro_restricao
            self._tem_erro = True
        if self._tem_erro:
            return False
        else:
            return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
