# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


# noinspection PyAbstractClass,PyPep8Naming
class estoquepedidos_pedido_capa(osv.Model):
    _name = 'estoquepedidos.pedido.capa'
    _inherit = 'estoquepedidos.pedido.capa'

    def _get_estpar_pedido_separar_padrao(self, cr, uid, context=None):
        context = context and context.copy() or {}
        res = True
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        estoque_parametro_id = obj_estoque_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)
        estpar_pedido_separar = obj_estoque_parametro.read(
            cr, uid, estoque_parametro_id[0], ['estpar_pedido_separar'], context=context)

        if estpar_pedido_separar and isinstance(estpar_pedido_separar, list):
            estpar_pedido_separar = estpar_pedido_separar[0]

        if isinstance(estpar_pedido_separar, dict):
            res = estpar_pedido_separar.get('estpar_pedido_separar')

        return res

    def pedcap_button_registra_cancelamento(self, cr, uid, ids, context=None):
        """
        Efetiva o cancelamento do pedido de venda.

        :param cr: cursor no banco de dados
        :param uid: registro da tebale res_users, identificador do usuario
        :param ids: lista de registros da tabela estoque_inventario_capa
        :param context: dicionario com valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se o registro foi cancelado.
        :rtype: bool
        """

        context = context and context.copy() or {}

        # Cancela o pedido de venda
        # noinspection PyUnresolvedReferences
        result = self.pedcap_cancelar_pedido(cr, uid, ids, context)

        for pedido in self.browse(cr, uid, ids, context):
            self.message_post(cr, uid, ids,
                              _(pedido.pedcap_cancelamento_complemento or ''),
                              _(pedido.pedcap_cancelamento_id.name))

        return result

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).write(cr, user, ids, vals, context)
        if not vals.get('state') or vals.get('state') == 'R':
            return res

        if not isinstance(ids, list):
            ids = [ids]

        obj_restricaoes = self.pool.get('venlre.liberar.restricoes')
        restricoes_ids = obj_restricaoes.search(cr, user,[('venlre_pedido_id', 'in', ids),
                                                ('venlre_autorizado', '=', False)], context=context)
        if not restricoes_ids:
            return res

        # Marca como altoriza as restricoes dos pedidos que nao estao mais em esboco
        obj_restricaoes.write(cr, user, restricoes_ids, {'venlre_autorizado': True})

        return res

    def pedcap_get_dado_extra(self, cr, uid, ids, context=None):
        """
        O metodo encontra os dados que se encontram nesse modulo, necessarios para a impressao
        dos pedidos.
        :param cr: cursor no banco de dados.
        :type: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type: integer
        :param ids: lista de id da tabela desse objeto.
        :type: list
        :param context: dicionario com valores contextuais.
        :type: dict
        :return: dicionario. Cada chave representa um id de um pedido. Cada valor dessa chave e um
        dicionario com os campos e seus respectivos valores.
        :rtype: dict
        """
        if not isinstance(ids, list):
            ids = [ids]
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).pedcap_get_dado_extra(cr, uid, ids, context)
        brw_pedido_capa = self.browse(cr, uid, ids, context)
        for pedido in brw_pedido_capa:
            vendedor = ''
            if pedido.vendedor_id:
                vendedor = pedido.vendedor_id.nome or ''
            vendedor_externo = ''
            if pedido.vendedor_externo_id:
                vendedor_externo = pedido.vendedor_externo_id.nome or ''
            vals = {
                'vendedor': vendedor,
                'vendedor_externo': vendedor_externo,
            }
            if pedido.id in res.keys():
                res[pedido.id].update(vals)
            else:
                res[pedido.id] = vals
        return res

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).default_get(cr, uid, fields_list, context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_vendedor_eh_usuario', context)
        vendedor_id = self.pool.get('vendas.vendedor').venven_get_vendedor_id(cr, uid,
                                                                              context=context)
        res.update({'pedcap_vendedor_usuario': True if estoque_parametro_dados.get(
            'estpar_vendedor_eh_usuario') and vendedor_id else False,
                   'vendedor_id': estoque_parametro_dados.get(
                       'estpar_vendedor_eh_usuario') and vendedor_id})
        return res

    def onchange_cliente_pedido(self, cr, uid, ids, pessoa_id, context=None):
        """
        Metodo para buscar o vendedor cadastrado para o cliente, se houver.
        """
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).onchange_cliente_pedido(
            cr, uid, ids, pessoa_id, context)
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_vendedor_eh_usuario', context)
        vendedor_id = self.pool.get('vendas.vendedor').venven_get_vendedor_id(cr, uid,
                                                                              context=context)
        if estoque_parametro_dados.get('estpar_vendedor_eh_usuario') and vendedor_id:
            return res
        res['value'].update({'vendedor_id': None, 'vendedor_externo_id': None})
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        if not pessoa_id:
            return res
        cliente_id = obj_financeiro_cliente.search(cr, uid, [('pessoa_id', '=', pessoa_id)],
                                                   context=context)
        if not cliente_id:
            return res
        cliente_dados = obj_financeiro_cliente.read(
            cr, uid, cliente_id, ['vendedor_id', 'vendedor_externo_id', 'fincli_rota'], context=context)
        for i in cliente_dados:
            if i.get('vendedor_id'):
                vendedor_id = i.get('vendedor_id')
                if isinstance(vendedor_id, tuple):
                    vendedor_id = vendedor_id[0]
                res['value'].update({'vendedor_id': vendedor_id})
            if i.get('vendedor_externo_id'):
                vendedor_externo_id = i.get('vendedor_externo_id')
                if isinstance(vendedor_externo_id, tuple):
                    vendedor_externo_id = vendedor_externo_id[0]
                res['value'].update({'vendedor_externo_id': vendedor_externo_id})
            res['value'].update({'pedcap_rota_entrega': i.get('fincli_rota')})
        return res

    def pedcap_button_analise_venda(self, cr, uid, ids, context=None):
        """
        O metodo realiza a chamada do formulario de analise de venda para o pedido atual.
        :param cr: cursor no banco de dadados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: dados para chamada da analise de venda para o pedido.
        :rtype: dict
        """
        context = context and context.copy() or {}

        pedido_id = ids[0] if isinstance(ids, list) else ids
        estoquepedidos_pedido_capa_dados = self.read(cr, uid, pedido_id, ['pessoa_id'], context,
                                                     '_classic_write')
        if isinstance(estoquepedidos_pedido_capa_dados, list):
            estoquepedidos_pedido_capa_dados = estoquepedidos_pedido_capa_dados[0]
        vals = {'wizanv_documento': 'P', 'wizanv_pedido_id': pedido_id,
                'wizanv_pessoa_id': estoquepedidos_pedido_capa_dados.get('pessoa_id'),
                'wizanv_id_documento_informado': True}
        analise_venda_id = self.pool.get('wizard.analise.venda').create(cr, uid, vals, context)

        obj_model_data = self.pool.get('ir.model.data')
        reference = obj_model_data.get_object_reference(
            cr, uid, 'vendas', 'wizard_analise_venda_form')
        view_id = reference and reference[1] or False
        return {'res_id': analise_venda_id,
                'name': 'Analise de Venda',
                'view_mode': 'form',
                'res_model': 'wizard.analise.venda',
                'view_id': [view_id],
                'target': 'new',
                'type': 'ir.actions.act_window',
                'context': context
                }

    def button_validar(self, cr, uid, ids, context=None):
        """
        Efetua a validacao do pedido de venda.
        O metodo aqui sobrescrito tem o objetivo de inserir os vendedores nos itens dos pedidos,
        caso tenha sido informado na capa desses.

        :param cr:  a linha atual do cursor do banco de dados,
        :param uid:  ID do usuario utilizado para a verificacao de perfil,
        :param ids:  lista de IDs,
        :param context:  um dicionario padrao para valores contextuais,
        :return: verdadeiro se o pedido foi validado
        :raise: mensagem de erro se o pedido nao foi validado.
        """
        context = context and context.copy() or {}
        res = super(estoquepedidos_pedido_capa, self).button_validar(cr, uid, ids, context)
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        for pedido in self.browse(cr, uid, ids if isinstance(ids, (list, tuple)) else [ids],
                                  context):
            if any([pedido.tipo == 'C', not pedido.vendedor_id and not pedido.vendedor_externo_id,
                    not pedido.pedido_item_ids]):
                continue
            if pedido.vendedor_id:
                item_ids = []
                for item in pedido.pedido_item_ids:
                    if item.vendedor_id:
                        continue
                    item_ids.append(item.id)
                item_ids and obj_estoquepedidos_pedido_item.write(
                    cr, uid, item_ids, {'vendedor_id': pedido.vendedor_id.id}, context)
            if pedido.vendedor_externo_id:
                item_ids = []
                for item in pedido.pedido_item_ids:
                    if item.vendedor_externo_id:
                        continue
                    item_ids.append(item.id)
                item_ids and obj_estoquepedidos_pedido_item.write(
                    cr, uid, item_ids, {'vendedor_externo_id': pedido.vendedor_externo_id.id},
                    context)
        return res
