# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from datetime import datetime


class venlre_liberar_restricoes(osv.Model):
    from funcoes import venlre_restricao_functions

    _name = 'venlre.liberar.restricoes'
    _columns = {
        'create_date': fields.datetime(select=True),
        'company_id': fields.many2one("res.company", 'Empresa', ondelete="cascade", required=True,
                                      select=True),
        'venlre_user_id': fields.many2one('res.users', 'Autorizado por', ondelete='restrict',
                                         select=True),
        'venlre_pedido_id': fields.many2one(
            'estoquepedidos.pedido.capa', 'Pedido', ondelete='cascade', select=True),
        'venlre_data': fields.datetime('Data/hora da liberacao'),
        'venlre_autorizado': fields.boolean('Autorizado'),

        # __________________ fields function ________________________
        'venlre_bloqueado_por_msg': fields.function(
            venlre_restricao_functions, multi='restricao_functions', method=True,
            oldname='venlre_mensagem_bloqueio', type="boolean", string='Bloqueado por mensagem'),
        'venlre_bloqueado_titulo_atrasado': fields.function(
            venlre_restricao_functions, multi='restricao_functions', method=True,
            oldname='venlre_titulos_em_atrasado', type="boolean",
            string='Bloqueado por titulos atrasados'),
        'venlre_valor_titulos_atrasados': fields.function(
            venlre_restricao_functions, multi='restricao_functions', digits=(12, 2), method=True,
            oldname='venlre_valor_em_atrasados', string='Valor dos titulos em atraso'),
        'venlre_bloqueado_limite_credito': fields.function(
            venlre_restricao_functions, multi='restricao_functions', method=True, type="boolean",
            oldname='venlre_limite_credito', string='Bloqueado por limite de credito'),
        'venlre_limite_debito': fields.function(
            venlre_restricao_functions, multi='restricao_functions', digits=(12, 2), method=True,
            string='Outros debitos', help="Debitos em orcamentos, pedidos e notas fiscais."),
        'venlre_valor_limite_credito': fields.function(
            venlre_restricao_functions, multi='restricao_functions', digits=(12, 2), method=True,
            oldname='venlre_limite_valor', string='Valor do limite de credito'),
        'venlre_valor_documento': fields.function(
            venlre_restricao_functions, multi='restricao_functions', digits=(12, 2), method=True,
            string='Valor do documento a prazo'),
        'venlre_total_debito': fields.function(
            venlre_restricao_functions, multi='restricao_functions', digits=(12, 2), method=True,
            string='Total do debito'),
        'venlre_titulos_a_vencer': fields.function(
            venlre_restricao_functions, multi='restricao_functions', type='float', digits=(12, 2),
            method=True, string='Titulos a vencer'),
        'venlre_titulos_em_aberto': fields.function(
            venlre_restricao_functions, multi='restricao_functions', type='float', digits=(12, 2),
            method=True, string='Subtotal'),
    }
    _defaults = {
        'venlre_data': lambda *a: str(datetime.now()),
        'venlre_autorizado': False,
        'venlre_user_id': lambda slf, cr, uid, ctx: uid,
        'company_id': lambda slf, cr, uid, ctx: slf.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, slf._name, context=ctx),
    }
    _order = 'create_date desc'
    _sql_constraints = [('venlre_pedido_uniq', 'unique(venlre_pedido_id, company_id)',
                         'Pedido ja cadastrado em outra restricao.')]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
