# -*- encoding: utf-8 -*-
from SigUtils.impressao import Impressora
from openerp.osv import osv, fields
from openerp.report.interface import report_int
from openerp.report.render import render
import StringIO
import time
import locale
from openerp import pooler
from SigUtils.numero import getNumeroEditado
# noinspection PyProtectedMember
from openerp.tools.translate import _
from dateutil.parser import parse


# noinspection PyPep8Naming
class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming,PyMethodMayBeStatic
class report_custom(report_int):
    def _get_dados_movimento_ordenados(self, movimentos, dict_nota, dict_cupom):
        for movimento in movimentos:
            if not movimento:
                continue
            dados_nf = dict_nota.get(movimento.get('nf_id')) or {}
            dados_cupom = dict_cupom.get(movimento.get('cupom_id')) or {}
            data_movimento = (dados_nf.get('data_emissao') or
                              dados_cupom.get('date_cupom') or '')[:10]
            movimento['data_movimento'] = str(data_movimento)

        return sorted(movimentos, key=lambda x: x.get('data_movimento'))

    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_estoque_movimento_itens = pooler.get_pool(cr.dbname).get('estoque.movimento.itens')
        obj_fiscal_nf_capa = pooler.get_pool(cr.dbname).get('fiscal.nf.capa')
        obj_vendas_vendedor = pooler.get_pool(cr.dbname).get('vendas.vendedor')
        obj_estoque_categoria = pooler.get_pool(cr.dbname).get('estoque.categoria')
        obj_estoque_produto = pooler.get_pool(cr.dbname).get('estoque.produto')
        obj_adm_utils = pooler.get_pool(cr.dbname).get('adm.utils')
        obj_estoque_movimento_capa = pooler.get_pool(cr.dbname).get('estoque.movimento.capa')

        pdf_string = StringIO.StringIO()
        listagem = Impressora()
        listagem.defineFormato()
        listagem.informaNome(pdf_string, _('Comissao de Venda'))
        listagem.informaTamLinha(160)
        listagem.defineMargem(0.5, 0.5, 0.5, 0.5)
        listagem.defineFonte(tam=6.25)

        # Define a fomatacao dos numeros
        locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % context.get('lang')))

        check_modulo_ecf = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'ecf', context)
        check_modulo_fiscal = obj_adm_utils.admutl_check_modulo_instalado(cr, uid, 'fiscal',
                                                                          context)
        tipo_vendedor = datas.get('tipo_vendedor') or ['I', 'E']
        vendedor_ids = datas.get('vendedor_id')
        categoria_ids = datas.get('categoria_ids')
        tipo_relatorio = datas.get('tipo_relatorio')
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')

        # Vendedores por tipo (interno/externo)
        exibir_vendas_sem_vendedor = False
        if not vendedor_ids:
            context.update({'active_test': False})
            vendedor_ids = obj_vendas_vendedor.search(
                cr, uid, [('tipo', 'in' if isinstance(tipo_vendedor, list)
                else '=', tipo_vendedor)], context=context)
            del context['active_test']
            exibir_vendas_sem_vendedor = True
        else:
            vendedor_ids = [vendedor_ids[0]]

        # Parametros
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'vendas.apuracao.comissao.osv', context=context)
        res_company_dados = pooler.get_pool(cr.dbname).get('res.company').read(
            cr, uid, company_id if isinstance(company_id, list) else [company_id], ['name'],
            context=context)
        if res_company_dados and isinstance(res_company_dados, list):
            res_company_dados = res_company_dados[0]
        company = res_company_dados.get('name')
        cfop_ids = pooler.get_pool(cr.dbname).get('fiscal.cfop').search(
            cr, uid, [('movimento_tipo', '=', 'C'), ('codigo', 'not in', ['5929', '6929'])],
            context=context)
        natureza_ids = pooler.get_pool(cr.dbname).get('fiscal.cfop.natureza').search(
            cr, uid, [('cfop_id', 'in', cfop_ids)], context=context)
        ids_nf = obj_fiscal_nf_capa.search(
            cr, uid, [('data_emissao', '>=', '%s 00:00:00' % data_ini),
                      ('data_emissao', '<=', '%s 23:59:59' % data_fim),
                      ('state', '=', '3'),
                      ('tipo', '=', 'S'),
                      ('natureza_id', 'in', natureza_ids)], context=context)

        # Protutos por categoria
        args = [('data', '>=', data_ini), ('data', '<=', data_fim)]
        mov_capa_ids = obj_estoque_movimento_capa.search(cr, uid, args, context=context)

        selecionada_categoria_ids = categoria_ids
        if categoria_ids:
            categoria_ids = obj_estoque_categoria.search(
                cr, uid, [('parent_id', 'child_of', categoria_ids)], context=context)
            produto_ids = obj_estoque_produto.search(
                cr, uid, [('categoria_id', 'in', categoria_ids)], context=context)
            movimento_item_ids = obj_estoque_movimento_itens.search(
                cr, uid, [('capa_id', 'in', mov_capa_ids), ('nf_id', 'in', ids_nf),
                          ('produto_id', 'in', produto_ids)], context=context)
            if check_modulo_ecf:
                movimento_item_ids.extend(obj_estoque_movimento_itens.search(
                    cr, uid, [('capa_id', 'in', mov_capa_ids), ('cupom_id', '!=', None),
                              ('produto_id', 'in', produto_ids)], context=context))
        else:
            movimento_item_ids = obj_estoque_movimento_itens.search(
                cr, uid, [('nf_id', 'in', ids_nf)], context=context)
            if check_modulo_ecf:
                movimento_item_ids.extend(obj_estoque_movimento_itens.estmvi_get_item_venda_cupom(
                    cr, uid, mov_capa_ids, context))
        data = time.strftime("%d/%m/%Y")
        data_ini = time.strftime("%d/%m/%Y", time.strptime(data_ini, "%Y-%m-%d"))
        data_fim = time.strftime("%d/%m/%Y", time.strptime(data_fim, "%Y-%m-%d"))
        det_format = \
            '|{:>11}|{:>5}|{:^12}|{:^8}|{:<36}|{:<38}|{:>10}|{:>10}|{:>10}|'.decode('utf-8')
        listagem.informaCab('*' + '-' * 148 + '*')
        listagem.informaCab('| {:^146} |'.decode('utf-8').format(_('Listagem de comissoes vendas')))
        listagem.informaCab('|' + ' ' * 148 + '|')
        listagem.informaCab('| {:<10} {:^125} {:>5} |'.decode('utf-8').format(
            data, company, 'Pagina: {{ Page }}'))
        listagem.informaCab('|' + ' ' * 148 + '|')
        listagem.informaCab('| ' + _('Periodo: %s a %s') % (data_ini, data_fim) + ' ' * 114 + ' |')
        categoria = ', '.join(map(
            lambda m: m.get('nome_completo'), obj_estoque_categoria.read(
                cr, uid, selecionada_categoria_ids, context=context)))
        if not categoria:
            categoria = _('Todas')
        listagem.informaCab('| {:<146} |'.decode('utf-8').format(_('Categoria: %s') % categoria))

        listagem.informaCab('| {:<8}: {:<136} |'.decode('utf-8').format(_('Vendedor')[:8], ''))
        listagem.informaCab('*' + '-' * 148 + '*')
        listagem.informaCab(det_format.format(
            _('NOTA/CUPOM'), _('SERIE'), _('DATA'), _('CODIGO'), _('CLIENTE'), _('ITEM'),
            _('VALOR'), _('DESCONTO'), _('COMISSAO')))
        listagem.informaCab('*' + '-' * 148 + '*')
        listagem.rodapePadrao()

        # Relatorio
        # Foi retirado o Browse para melhorar a performance do relatorio
        load = '_classic_write'
        campos = ['nf_id', 'cupom_id', 'vr_total_item_rateado', 'vr_total_item', 'vr_desconto',
                  'vr_desconto_rateado', 'estmvi_vr_comissao', 'vendedor_id', 'vendedor_externo_id',
                  'grade_id']
        context['super_read_estoque_movimento_itens'] = True
        dados_movimento_itens = obj_estoque_movimento_itens.read(
            cr, uid, movimento_item_ids, campos, context=context, load=load)
        venda_sem_vendedor = False
        dados_sem_vendedor = []
        nf_ids = []
        cupom_ids = []
        grade_ids = []
        for i in dados_movimento_itens:
            grade_ids.append(i.get('grade_id'))
            if i.get('nf_id'):
                nf_ids.append(i.get('nf_id'))
            elif i.get('cupom_id'):
                cupom_ids.append(i.get('cupom_id'))

            if not i.get('vendedor_id') and not i.get('vendedor_externo_id'):
                venda_sem_vendedor = True
                dados_sem_vendedor.append(i)

        if tipo_relatorio == "A":
            list_notas_devolucao = self._get_dados_notas_devolucao_analitico(
                cr, uid, company_id, datas.get('data_ini'), datas.get('data_fim'), context)
            # todo: colocar o metodo por heranca no modulo ECF, mas precisa retirar a funcao do
            # todo: 'report' e colocar dentro da classe 'vendas.apuracao.comissao.osv'
            if check_modulo_ecf:
                list_notas_devolucao.extend(self._get_dados_notas_devolucao_analitico_ecf(cr, uid,
                                company_id, datas.get('data_ini'), datas.get('data_fim'), context))
        else:
            list_notas_devolucao = self._get_dados_notas_devolucao_sintetico(
                cr, uid, company_id, datas.get('data_ini'), datas.get('data_fim'), context)
            if check_modulo_ecf:
                list_notas_devolucao.extend(self._get_dados_notas_devolucao_sintetico_ecf(
                    cr, uid, company_id, datas.get('data_ini'), datas.get('data_fim'), context))

        if list_notas_devolucao:
            if filter(lambda i: not i.get('mov_vendedor_id') and not i.get(
                    'mov_vendedor_externo_id'), list_notas_devolucao):
                venda_sem_vendedor = True
                dados_sem_vendedor.append({})
        dict_notas = self._get_dados_notas(cr, uid, nf_ids, context=context, load=load)
        dict_cupons = self._get_dados_cupons(cr, uid, cupom_ids, context=context, load=load)
        dict_pessoas = self._get_dados_pessoas(cr, uid, dict_notas, dict_cupons, context=context)
        dict_grade = self._get_dados_grade(cr, uid, grade_ids, context=context)
        if exibir_vendas_sem_vendedor:
            if venda_sem_vendedor:
                vendedor_ids.append([])
        total_geral_venda = 0
        resumo = {}
        context['admpes_retorno_sem_codigo'] = True
        total_forma_pagamento_dados = []
        if tipo_relatorio == "A":
            total_geral_comissao = 0.0
            for vendedor_id in vendedor_ids:
                if vendedor_id:
                    vendedor_dados = obj_vendas_vendedor.read(
                        cr, uid, vendedor_id, ['pessoa_id'], context=context)
                    pessoa_id = vendedor_dados.get('pessoa_id')
                    linha = '| {:<8}: {:<136} |'.decode('utf-8').format(
                        _('Vendedor')[:8], pessoa_id[1][:133])
                else:
                    linha = '| {:<8}: {:<136} |'.decode('utf-8').format(
                        _('Vendedor')[:8], 'VENDAS SEM VENDEDOR')

                movimento_vendedor_externo = False
                # Filtra o vendedor
                if vendedor_id:
                    dados_movimento = filter(
                        lambda f: f.get('vendedor_id') == vendedor_id, dados_movimento_itens)
                    if not dados_movimento:
                        dados_movimento = filter(
                            lambda f: f.get('vendedor_externo_id') == vendedor_id,
                            dados_movimento_itens)
                        if dados_movimento:
                            movimento_vendedor_externo = True
                    # Nao deve listar vendedores que nao tiveram movimento
                    if not dados_movimento:
                        continue
                else:
                    dados_movimento = dados_sem_vendedor

                listagem.cabecalhos[7] = linha

                # Outro vendedor (outra pagina)
                if vendedor_id != vendedor_ids[0]:
                    listagem.imprimeLinha('{{ Page }}')

                l = []
                total_valor = 0.0
                total_desconto = 0.0
                total_comissao = 0.0
                possui_movimento = False
                dados_movimento = self._get_dados_movimento_ordenados(dados_movimento, dict_notas,
                                                                      dict_cupons)
                for movimento in dados_movimento:
                    if movimento:
                        dados_nf = dict_notas.get(movimento.get('nf_id')) or {}
                        dados_cupom = dict_cupons.get(movimento.get('cupom_id')) or {}
                        nota_coo = dados_nf.get('numero_nf') or dados_cupom.get('coo')
                        if dados_nf:
                            dados_pessoa = dict_pessoas.get(dados_nf.get('destinatario_id'))
                        else:
                            dados_pessoa = dict_pessoas.get(dados_cupom.get('cliente_id'))
                        l.append(nota_coo)
                        serie = dados_nf.get('serie_nf') or ''
                        data_movimento = (dados_nf.get('data_emissao') or
                                          dados_cupom.get('date_cupom') or '')[:10]

                        # Formatacao da data no relatorio
                        if data_movimento:
                            data_movimento = time.strftime(
                                '%d/%m/%Y', time.strptime(data_movimento, '%Y-%m-%d'))

                        item = dict_grade.get(movimento.get('grade_id'))[:38]
                        valor_rateado = movimento.get('vr_total_item_rateado') or 0.0
                        desconto = movimento.get('vr_desconto') or 0.0
                        desconto += movimento.get('vr_desconto_rateado') or 0
                        vr_comissao = not movimento_vendedor_externo and (
                            movimento.get('estmvi_vr_comissao') or 0.0) or 0
                        codigo = dados_pessoa and dados_pessoa.get('codigo') or ''
                        cliente = dados_pessoa and dados_pessoa.get('name')[:36] or \
                                  'Sem cliente informado'

                        # Imprime a linha detalhe
                        listagem.imprimeLinha(
                            det_format.format(nota_coo, serie, data_movimento, codigo, cliente,
                                              item,
                                              locale.format('%.2f', valor_rateado, grouping=True),
                                              locale.format('%.2f', desconto, grouping=True),
                                              locale.format('%.6f', vr_comissao, grouping=True)))
                        total_valor += valor_rateado
                        total_desconto += desconto
                        total_comissao += vr_comissao
                        total_geral_comissao += vr_comissao
                        total_geral_venda += valor_rateado
                        possui_movimento = True

                if vendedor_id:
                    vendedor_item_devolucao = filter(
                        lambda vid: vid.get('mov_vendedor_id') == vendedor_id
                        and datas.get('data_ini') <= vid.get('data_emissao')[:10]
                        and vid.get('data_emissao')[:10] <= datas.get('data_fim'),
                        list_notas_devolucao)
                    if not vendedor_item_devolucao:
                        vendedor_item_devolucao = filter(
                            lambda vid: vid.get('mov_vendedor_externo_id') == vendedor_id
                            and datas.get('data_ini') <= vid.get('data_emissao')[:10]
                            and vid.get('data_emissao')[:10] <= datas.get('data_fim'),
                            list_notas_devolucao)
                else:
                    vendedor_item_devolucao = filter(
                        lambda vid: not vid.get('mov_vendedor_id') and not vid.get(
                            'mov_vendedor_externo_id'),
                        list_notas_devolucao)
                if vendedor_item_devolucao:
                    for devolucao in vendedor_item_devolucao:
                        list_notas_devolucao.remove(devolucao)
                        l.append(devolucao.get('numero_nf'))
                        dev_vr_total_item_rateado = devolucao.get('vr_total_item_rateado') or 0
                        dev_desconto = devolucao.get('desconto') or 0
                        dev_comissao = devolucao.get('vr_comissao') or 0
                        listagem.imprimeLinha(det_format.format(
                            devolucao.get('numero_nf'),
                            devolucao.get('serie_nf'),
                            parse(devolucao.get('data_emissao')).strftime('%d/%m/%Y'),
                            devolucao.get('codigo'),
                            (devolucao.get('cliente') or '')[:36] or 'Sem cliente informado',
                            devolucao.get('item')[:38],
                            locale.format('%.2f', dev_vr_total_item_rateado * -1,
                                          grouping=True),
                            locale.format('%.2f', dev_desconto * -1 if dev_desconto else 0,
                                          grouping=True),
                            locale.format('%.6f', dev_comissao * -1 if dev_comissao else 0,
                                          grouping=True)))
                        total_valor -= dev_vr_total_item_rateado
                        total_desconto -= dev_desconto
                        total_comissao -= dev_comissao
                        total_geral_comissao -= dev_comissao
                        total_geral_venda -= dev_vr_total_item_rateado
                        possui_movimento = True

                if not possui_movimento:
                    listagem.imprimeLinha('|{:^148}|'.decode('utf-8').format(
                        'NENHUMA VENDA PARA ESTE PERIODO'))
                listagem.imprimeLinha('*' + '-' * 148 + '*')
                listagem.imprimeLinha('|{:>115}|{:>10}|{:>10}|{:>10}|'.decode('utf-8').format(
                    _('TOTAL (%s):') % len(list(set(l))),
                    locale.format('%.2f', total_valor, grouping=True),
                    locale.format('%.2f', total_desconto, grouping=True),
                    locale.format('%.6f', total_comissao, grouping=True)))
                listagem.imprimeLinha('*' + '-' * 148 + '*')
            listagem.imprimeLinha('|{:>137}|{:>10}|'.decode('utf-8').format(
                _('TOTAL GERAL COMISSAO:'), locale.format(
                    '%.6f', total_geral_comissao, grouping=True)))
            listagem.imprimeLinha('*' + '-' * 148 + '*')
            listagem.imprimeLinha('|{:>137}|{:>10}|'.decode('utf-8').format(
                _('TOTAL GERAL VENDA:'), locale.format('%.2f', total_geral_venda, grouping=True)))
            listagem.imprimeLinha('*' + '-' * 148 + '*')
        elif tipo_relatorio == "S":
            if check_modulo_fiscal:
                devolucao_nf_ids = self._get_dados_notas_devolucao_sintetico(
                    cr, uid, company_id, datas.get('data_ini'), datas.get('data_fim'),
                    context={'comissao_devolucao_nf_ids': True})
            if check_modulo_ecf:
                devolucao_nf_ids.extend(self._get_dados_notas_devolucao_sintetico_ecf(
                    cr, uid, company_id, datas.get('data_ini'), datas.get('data_fim'),
                    context={'comissao_devolucao_nf_ids': True}))
            query_total_forma_pagamento = ''
            if check_modulo_fiscal:
                query_total_forma_pagamento += self.get_total_vendedor_forma_pagamento_nf(
                    cr, uid, vendedor_ids, nf_ids, devolucao_nf_ids, context)
            if check_modulo_ecf:
                if check_modulo_fiscal:
                    query_total_forma_pagamento += '\n UNION ALL \n'
                query_total_forma_pagamento += self.get_total_vendedor_forma_pagamento_cupom(
                    cr, uid, vendedor_ids, cupom_ids, context)
            total_forma_pagamento_dados = self.get_total_vendedor_forma_pagamento(
                cr, uid, query_total_forma_pagamento, context)

            list_datas = list(set(map(lambda ld: ld.get('nf_data_emissao'), list_notas_devolucao)))
            # Indices
            total_geral_comissao = 0.0
            serie = codigo = cliente = item = ''
            posicao = 1
            for vendedor_id in vendedor_ids:
                if vendedor_id:
                    vendedor_dados = obj_vendas_vendedor.read(
                        cr, uid, vendedor_id, ['pessoa_id'], context=context)
                    pessoa_id = vendedor_dados.get('pessoa_id')
                    linha = '| {:<8}: {:<136} |'.decode('utf-8').format(
                        _('Vendedor')[:8], pessoa_id[1][:133])
                    resumo[pessoa_id[0]] = {'nome': pessoa_id[1][:133], 'pos': posicao}
                else:
                    linha = '| {:<8}: {:<136} |'.decode('utf-8').format(
                        _('Vendedor')[:8], 'VENDAS SEM VENDEDOR')
                    resumo[99999] = {'nome': 'VENDAS SEM VENDEDOR', 'pos': posicao}

                movimento_vendedor_externo = False

                # Filtra o vendedor
                if vendedor_id:
                    dados_movimento = filter(
                        lambda f: f.get('vendedor_id') == vendedor_id, dados_movimento_itens)
                    if not dados_movimento:
                        dados_movimento = filter(
                            lambda f: f.get('vendedor_externo_id') == vendedor_id,
                            dados_movimento_itens)
                        if dados_movimento:
                            movimento_vendedor_externo = True
                    # Nao deve listar vendedores que nao tiveram movimento
                    if not dados_movimento:
                        continue
                else:
                    dados_movimento = dados_sem_vendedor

                posicao += 1
                listagem.cabecalhos[7] = linha

                # Outro vendedor (outra pagina)
                if vendedor_id != vendedor_ids[0]:
                    listagem.imprimeLinha('{{ Page }}')

                total_valor = total_desconto = 0.0
                total_item = 0
                total_comissao = 0.0
                possui_movimento = False

                # Consolida as datas de movimento
                movimento_sintetico = dict()
                if filter(lambda i: i, dados_movimento):
                    for x in map(lambda r: (dict_notas.get(r.get('nf_id')).get('data_emissao')[:10]
                                            if dict_notas.get(r.get('nf_id'))
                                            else dict_cupons.get(r.get('cupom_id')).get('date_cupom'),
                                            r.get('vr_total_item_rateado'),
                                            r.get('vr_total_item'),
                                            r.get('vr_desconto') + r.get('vr_desconto_rateado'),
                                            r.get('estmvi_vr_comissao')) if r else (),
                                 dados_movimento):
                        if not x:
                            continue

                        desconto = x[3]
                        valor_rateado = x[1] + x[3]
                        percent = round(desconto / valor_rateado, 2) if valor_rateado else 0
                        if not vendedor_id:
                            if not resumo.get(99999).get('%'):
                                resumo[99999].update({'%': {percent: {'desconto': desconto,
                                                                      'bruto': valor_rateado}
                                                            }
                                                      })
                            else:
                                if not resumo.get(99999).get('%').get(percent):
                                    resumo[99999]['%'].update({percent: {'desconto': desconto,
                                                                         'bruto': valor_rateado}
                                                               })
                                else:
                                    resumo[99999]['%'][percent]['desconto'] += desconto
                                    resumo[99999]['%'][percent]['bruto'] += valor_rateado

                        else:
                            if not resumo.get(pessoa_id[0]).get('%'):
                                resumo[pessoa_id[0]].update(
                                    {'%': {percent: {'desconto': desconto,
                                                     'bruto': valor_rateado}
                                           }
                                     })
                            else:
                                if not resumo.get(pessoa_id[0]).get('%').get(percent):
                                    resumo[pessoa_id[0]]['%'].update(
                                        {percent: {'desconto': desconto,
                                                   'bruto': valor_rateado}
                                         })
                                else:
                                    resumo[pessoa_id[0]]['%'][percent]['desconto'] += desconto
                                    resumo[pessoa_id[0]]['%'][percent]['bruto'] += valor_rateado



                        if x[0] not in movimento_sintetico:
                            movimento_sintetico[x[0]] = dict(
                                valor_rateado=x[1], vr_total_item=x[2] + x[3], nota_coo=1,
                                vr_comissao= not movimento_vendedor_externo and x[4] or 0,
                                vr_desconto=x[3])
                        else:
                            movimento_sintetico[x[0]]['valor_rateado'] += x[1]
                            movimento_sintetico[x[0]]['vr_total_item'] += x[2] + x[3]
                            movimento_sintetico[x[0]]['nota_coo'] += 1
                            movimento_sintetico[x[0]]['vr_comissao'] += (
                                    not movimento_vendedor_externo and x[4] or 0)
                            movimento_sintetico[x[0]]['vr_desconto'] += x[3]

                for ld in list_datas:
                    if ld not in movimento_sintetico.keys():
                        movimento_sintetico.update({ld: {}})

                for movimento in sorted(movimento_sintetico, key=lambda d: d):
                    if movimento_sintetico.get(movimento):
                        valor_rateado = movimento_sintetico[movimento].get('valor_rateado')
                        nota_coo = movimento_sintetico[movimento].get('nota_coo')
                        desconto = movimento_sintetico[movimento].get('vr_desconto') or 0
                        vr_comissao = movimento_sintetico[movimento].get('vr_comissao') or 0.0

                        # Formata a data do movimento
                        data_movimento = time.strftime(
                            '%d/%m/%Y', time.strptime(movimento, '%Y-%m-%d'))

                        # Imprime a linha detalhe
                        listagem.imprimeLinha(
                            det_format.format(nota_coo, serie, data_movimento, codigo, cliente,
                                              item,
                                              locale.format('%.2f', valor_rateado, grouping=True),
                                              locale.format('%.2f', desconto, grouping=True),
                                              locale.format('%.6f', vr_comissao, grouping=True)))

                        total_valor += valor_rateado
                        total_desconto += desconto
                        total_item += nota_coo
                        total_comissao += vr_comissao
                        total_geral_comissao += vr_comissao
                        total_geral_venda += valor_rateado
                        possui_movimento = True

                    if vendedor_id:
                        vendedor_item_devolucao = filter(
                            lambda vid: vid.get('mov_vendedor_id') == vendedor_id
                            and vid.get('nf_data_emissao') == movimento, list_notas_devolucao)
                        if not vendedor_item_devolucao:
                            vendedor_item_devolucao = filter(
                                lambda vid: vid.get('mov_vendedor_externo_id') == vendedor_id
                                and vid.get('nf_data_emissao') == movimento,
                                list_notas_devolucao)
                    else:
                        vendedor_item_devolucao = filter(
                            lambda vid: not vid.get('mov_vendedor_id') and not vid.get(
                                'mov_vendedor_externo_id')
                            and vid.get('nf_data_emissao') == movimento, list_notas_devolucao)
                    if vendedor_item_devolucao:
                        qtd_nf = 0
                        dev_vr_total_item_rateado = 0
                        dev_desconto = 0
                        dev_comissao = 0
                        for devolucao in vendedor_item_devolucao:
                            qtd_nf += devolucao.get('qtd_nf') or 0
                            dev_vr_total_item_rateado += devolucao.get('vr_total_item_rateado') or 0
                            desconto = devolucao.get('vr_desconto_rateado') or 0.0
                            dev_desconto += desconto
                            comissao = devolucao.get('vr_comissao') or 0.0
                            dev_comissao += comissao
                            total_valor -= devolucao.get('vr_total_item_rateado')
                            total_desconto -= desconto
                            total_item += 1
                            total_comissao -= comissao
                            total_geral_comissao -= comissao
                            total_geral_venda -= devolucao.get('vr_total_item_rateado')
                            possui_movimento = True

                        listagem.imprimeLinha(det_format.format(qtd_nf, '',
                                                                parse(devolucao.get('nf_data_emissao')).strftime('%d/%m/%Y'),
                            codigo, cliente, item,
                            locale.format('%.2f', dev_vr_total_item_rateado * -1, grouping=True),
                            locale.format('%.2f', dev_desconto * -1 if dev_desconto else 0,
                                          grouping=True),
                            locale.format('%.6f', dev_comissao * -1 if dev_comissao else 0,
                                          grouping=True)))

                if not possui_movimento:
                    listagem.imprimeLinha('|{:^148}|'.decode('utf-8').format(
                        'NENHUMA VENDA PARA ESTE PERIODO'))
                listagem.imprimeLinha('*' + '-' * 148 + '*')
                listagem.imprimeLinha('|{:>115}|{:>10}|{:>10}|{:>10}|'.decode('utf-8').format(
                    _('TOTAL (%s):') % total_item,
                    locale.format('%.2f', total_valor, grouping=True),
                    locale.format('%.2f', total_desconto, grouping=True),
                    locale.format('%.6f', total_comissao if round(total_comissao, 6) else 0,
                                  grouping=True)))
                listagem.imprimeLinha('*' + '-' * 148 + '*')
            listagem.imprimeLinha('|{:>137}|{:>10}|'.decode('utf-8').format(
                _('TOTAL GERAL COMISSAO:'), locale.format(
                    '%.6f', total_geral_comissao if round(total_geral_comissao, 6) else 0,
                    grouping=True)))
            listagem.imprimeLinha('*' + '-' * 148 + '*')
            listagem.imprimeLinha('|{:>137}|{:>10}|'.decode('utf-8').format(
                _('TOTAL GERAL VENDA:'), locale.format('%.2f', total_geral_venda, grouping=True)))
            listagem.imprimeLinha('*' + '-' * 148 + '*')

        primeiro = True
        tam = len(resumo)
        cont = 0
        for i in sorted(resumo.values(), key=lambda x: x.get('pos')):
            if not i.get('%'):
                cont += 1
                continue

            if primeiro:
                listagem.cabecalhos = listagem.cabecalhos[:-4]
                listagem.informaCab('*' + '-' * 148 + '*')
                listagem.imprimeLinha('{{ Page }}')
                listagem.imprimeLinha('')
                listagem.imprimeLinha('*%s*' % 'Resumo por % de Desconto'.center(59, '-') + '     '
                                      '*%s*' % 'Resumo por Forma de Pagamento'.center(82, '-'))
                primeiro = False

            nome = i.get('nome')
            vendedor_forma_pagamento_dados = filter(lambda vf: vf.get('nome_vendedor') == nome,
                                                    total_forma_pagamento_dados)
            if not vendedor_forma_pagamento_dados and nome == 'VENDAS SEM VENDEDOR':
                vendedor_forma_pagamento_dados = filter(lambda vf: not vf.get('nome_vendedor'),
                                                        total_forma_pagamento_dados)
            listagem.imprimeLinha('|{:<148}|'.decode('utf-8').format(nome))
            linha = '| % Desconto | Valor Bruto | Valor Desconto | Valor liquido |' + '     '
            linha += '|{:<34}|{:>15}|{:>15}|{:>15}|'.format(
                'Forma de Pagamento', 'Valor Bruto', 'Valor Desc', 'Valor Liquido')
            listagem.imprimeLinha(linha)
            listagem.imprimeLinha(('*%s*' % ''.ljust(59, '-')) + '     '
                                  + ('*%s*' % ''.ljust(82, '-')))
            vendedor_desconto_dados = sorted(i.get('%').items(), key=lambda x: x[0])
            qtd_iteracoes = max([len(i.get('%').items()), len(vendedor_forma_pagamento_dados)])
            for itr in range(0, qtd_iteracoes):
                try:
                    bruto = vendedor_desconto_dados[itr][1].get('bruto')
                    desconto = vendedor_desconto_dados[itr][1].get('desconto')
                    liquido = bruto - desconto
                    linha = '|{:>12}|{:>13}|{:>16}|{:>15}|     '.format(
                        int(vendedor_desconto_dados[itr][0]*100),
                        getNumeroEditado(bruto),
                        getNumeroEditado(desconto),
                        getNumeroEditado(liquido))
                except:
                    linha = '|{:>12}|{:>13}|{:>16}|{:>15}|     '.format('', '', '', '')
                try:
                    linha += '|{:<34}|{:>15}|{:>15}|{:>15}|'.format(
                        vendedor_forma_pagamento_dados[itr].get('nome_forma'),
                        getNumeroEditado(vendedor_forma_pagamento_dados[itr].get('vr_bruto')),
                        getNumeroEditado(vendedor_forma_pagamento_dados[itr].get('vr_desconto')),
                        getNumeroEditado(vendedor_forma_pagamento_dados[itr].get('vr_liquido')))
                except:
                    linha += '|{:<34}|{:>15}|{:>15}|{:>15}|'.format('', '', '', '')
                listagem.imprimeLinha(linha)
            listagem.imprimeLinha('*%s*' % ''.ljust(148, '-'))
            cont += 1
            if cont < tam:
                listagem.imprimeLinha('|%s|' % ''.ljust(148, ' '))

        listagem.finalizaImpressao()
        self.obj = external_pdf(pdf_string.getvalue())
        self.obj.render()
        pdf_string.close()

        return self.obj.pdf, 'pdf'

    def _get_dados_notas_devolucao_analitico(self, cr, uid, company_id, data_ini, data_fim,
                                             context=None):
        context = context and context.copy() or {}

        sql = """ SELECT nf2.id AS id
                  FROM fiscal_nf_capa      AS nf1
                  LEFT JOIN fiscal_nf_referencia AS fnr ON fnr.nf_ref_id = nf1.id
                  LEFT JOIN fiscal_nf_capa AS nf2 ON nf2.id = fnr.id 
                  WHERE fnr.nf_ref_id IS NOT NULL
                    AND nf2.state = '3'
                    AND nf2.tipo = 'E'
                    AND CASE 
                            WHEN nf2.nf_propria IS TRUE 
                                THEN nf2.data_emissao BETWEEN '{data_ini} 00:00:00' AND 
                                    '{data_fim} 23:59:59'
                            ELSE nf2.data_saida_entrada BETWEEN '{data_ini} 00:00:00' AND 
                                '{data_fim} 23:59:59'
                        END
                    AND nf1.company_id = {company_id}
              """.format(data_ini=data_ini, data_fim=data_fim, company_id=company_id)
        cr.execute(sql)
        nf_ids = map(lambda x: '%s' % x.get('id'), cr.dictfetchall())
        if not nf_ids:
            return []

        sql = """
            SELECT nf.id AS nf_capa_id,
                   nf.numero_nf,
                   nf.serie_nf,
                   nf.data_emissao,
                   emi.vendedor_id AS mov_vendedor_id,
                   emi.vendedor_externo_id AS mov_vendedor_externo_id, 
                   emi.grade_id, 
                   SUM(vr_total_item_rateado) AS vr_total_item_rateado,
                   SUM(vr_desconto_rateado) AS desconto,
                   SUM(estmvi_vr_comissao) AS vr_comissao,
                   (SELECT codigo FROM adm_pessoa WHERE id = nf.destinatario_id) AS codigo,
                   (SELECT nome FROM adm_pessoa WHERE id = nf.destinatario_id) AS cliente,
                   (SELECT name FROM estoque_produto_grade WHERE id = emi.grade_id) AS item
            FROM estoque_movimento_itens   AS emi
            LEFT JOIN fiscal_nf_capa       AS nf ON nf.id = emi.nf_id
            WHERE emi.nf_id IN (%s)
                --AND emi.vendedor_id IS NOT NULL
            GROUP BY nf_capa_id, mov_vendedor_id, mov_vendedor_externo_id, grade_id, emi.id
            ORDER BY nf_capa_id
        """ % ','.join(nf_ids)
        cr.execute(sql)
        return cr.dictfetchall()

    def _get_dados_notas_devolucao_analitico_ecf(self, cr, uid, company_id, data_ini, data_fim,
                                             context=None):
        context = context and context.copy() or {}
        sql = """ SELECT cupom.fisnfc_nf_referencia_id AS id
                  FROM ecf_cupons_capa     AS cupom
                  LEFT JOIN fiscal_nf_capa AS nf ON nf.id = cupom.fisnfc_nf_referencia_id 
                  WHERE (cupom.fisnfc_nf_referencia_id IS NOT NULL OR cupom.ecfccp_cupom_cancelamento_id IS NOT NULL)
                    AND nf.state = '3'
                    AND nf.tipo = 'E'
                    AND CASE 
                            WHEN nf.nf_propria IS TRUE 
                                THEN nf.data_emissao BETWEEN '{data_ini} 00:00:00' AND 
                                    '{data_fim} 23:59:59'
                            ELSE nf.data_saida_entrada BETWEEN '{data_ini} 00:00:00' AND 
                                '{data_fim} 23:59:59'
                        END
                    AND cupom.company_id = {company_id}
              """.format(data_ini=data_ini, data_fim=data_fim, company_id=company_id)
        cr.execute(sql)
        nf_ids = map(lambda x: '%s' % x.get('id'), cr.dictfetchall())
        if not nf_ids:
            return []

        sql = """
            SELECT nf.id AS nf_capa_id,
                   nf.numero_nf AS numero_nf,
                   nf.serie_nf  AS serie_nf,
                   data_emissao AS data_emissao,
                   emi.vendedor_id AS mov_vendedor_id,
                   emi.vendedor_externo_id AS mov_vendedor_externo_id, 
                   emi.grade_id, 
                   SUM(vr_total_item_rateado) AS vr_total_item_rateado,
                   SUM(vr_desconto_rateado) AS desconto,
                   SUM(estmvi_vr_comissao) AS vr_comissao,
                   (SELECT codigo FROM adm_pessoa WHERE id = nf.destinatario_id) AS codigo,
                   (SELECT nome FROM adm_pessoa WHERE id = nf.destinatario_id) AS cliente,
                   (SELECT name FROM estoque_produto_grade WHERE id = emi.grade_id) AS item
            FROM estoque_movimento_itens AS emi
            LEFT JOIN fiscal_nf_capa     AS nf ON nf.id = emi.nf_id
            WHERE emi.nf_id IN (%s)
                --AND emi.vendedor_id IS NOT NULL
            GROUP BY nf_capa_id, mov_vendedor_id, mov_vendedor_externo_id, grade_id, emi.id
            ORDER BY nf_capa_id
        """ % ','.join(nf_ids)
        cr.execute(sql)
        return cr.dictfetchall()

    def _get_dados_notas_devolucao_sintetico(self, cr, uid, company_id, data_ini, data_fim,
                                             context=None):
        context = context and context.copy() or {}

        sql = """
        SELECT nf2.id AS id
        FROM fiscal_nf_capa            AS nf1
        LEFT JOIN fiscal_nf_referencia AS fnr ON fnr.nf_ref_id = nf1.id
        LEFT JOIN fiscal_nf_capa       AS nf2 ON nf2.id = fnr.id
        WHERE nf2.id IS NOT NULL
          AND nf2.state = '3'
          AND nf2.tipo = 'E'
          AND CASE 
                  WHEN nf2.nf_propria IS TRUE 
                      THEN nf2.data_emissao BETWEEN '{data_ini} 00:00:00' AND '{data_fim} 23:59:59'
                  ELSE nf2.data_saida_entrada BETWEEN '{data_ini} 00:00:00' AND 
                      '{data_fim} 23:59:59'
              END
          AND nf1.company_id = {company_id}
        """.format(data_ini=data_ini, data_fim=data_fim, company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if context.get('comissao_devolucao_nf_ids'):
            return map(lambda x: x.get('id'), dados) or []
        nf_ids = map(lambda x: '%s' % x.get('id'), dados)
        if not nf_ids:
            return []

        sql = """
            SELECT 
                count(DISTINCT emi.nf_id) AS qtd_nf,
                to_date(left(nf.data_emissao::VARCHAR, 10), 'YYYY-MM-DD') AS nf_data_emissao,
                emi.vendedor_id AS mov_vendedor_id,
                emi.vendedor_externo_id AS mov_vendedor_externo_id, 
                sum(vr_total_item_rateado) AS vr_total_item_rateado,
                sum(vr_desconto_rateado) AS vr_desconto_rateado,
                SUM(estmvi_vr_comissao) AS vr_comissao
            FROM estoque_movimento_itens   AS emi
            LEFT JOIN fiscal_nf_capa       AS nf ON nf.id = emi.nf_id
            WHERE emi.nf_id IN (%s)
            GROUP BY mov_vendedor_id, mov_vendedor_externo_id, nf_data_emissao
            ORDER BY nf_data_emissao
        """ % ','.join(nf_ids)
        cr.execute(sql)
        return cr.dictfetchall()

    def _get_dados_notas_devolucao_sintetico_ecf(self, cr, uid, company_id, data_ini, data_fim,
                                             context=None):
        context = context and context.copy() or {}

        sql = """
        SELECT cupom.fisnfc_nf_referencia_id AS id
        FROM ecf_cupons_capa      AS cupom
        LEFT JOIN fiscal_nf_capa AS nf ON nf.id = cupom.fisnfc_nf_referencia_id
        WHERE (cupom.fisnfc_nf_referencia_id IS NOT NULL OR cupom.ecfccp_cupom_cancelamento_id IS NOT NULL)
          AND nf.state = '3'
          AND nf.tipo = 'E'
          AND CASE 
                  WHEN nf.nf_propria IS TRUE 
                      THEN nf.data_emissao BETWEEN '{data_ini} 00:00:00' AND '{data_fim} 23:59:59'
                  ELSE nf.data_saida_entrada BETWEEN '{data_ini} 00:00:00' AND 
                      '{data_fim} 23:59:59'
              END
          AND cupom.company_id = {company_id}
        """.format(data_ini=data_ini, data_fim=data_fim, company_id=company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if context.get('comissao_devolucao_nf_ids'):
            return map(lambda x: x.get('id'), dados) or []
        nf_ids = map(lambda x: '%s' % x.get('id'), dados)
        if not nf_ids:
            return []

        sql = """
            SELECT 
                count(DISTINCT emi.nf_id) AS qtd_nf,
                to_date(left(nf.data_emissao::VARCHAR, 10), 'YYYY-MM-DD') AS nf_data_emissao,
                emi.vendedor_id AS mov_vendedor_id,
                emi.vendedor_externo_id AS mov_vendedor_externo_id, 
                sum(vr_total_item_rateado) AS vr_total_item_rateado,
                sum(vr_desconto_rateado) AS vr_desconto_rateado,
                SUM(estmvi_vr_comissao) AS vr_comissao
            FROM estoque_movimento_itens AS emi
            LEFT JOIN fiscal_nf_capa     AS nf ON nf.id = emi.nf_id
            WHERE emi.nf_id IN (%s)
            GROUP BY mov_vendedor_id, mov_vendedor_externo_id, nf_data_emissao
            ORDER BY nf_data_emissao
        """ % ','.join(nf_ids)
        cr.execute(sql)
        return cr.dictfetchall()

    def _get_dados_notas(self, cr, uid, nf_ids, context=None, load=''):
        context = context and context.copy() or {}
        res = {}
        if not nf_ids:
            return res
        campos = ['numero_nf', 'serie_nf', 'data_emissao', 'destinatario_id']
        map(lambda x: res.update({x.get('id'): x}),
            pooler.get_pool(cr.dbname).get('fiscal.nf.capa').read(
                cr, uid, nf_ids, campos, context=context, load=load))
        return res

    def _get_dados_cupons(self, cr, uid, cupom_ids, context=None, load=''):
        context = context and context.copy() or {}
        res = {}
        if not cupom_ids:
            return res
        campos = ['coo', 'date_cupom', 'cliente_id']
        map(lambda x: res.update({x.get('id'): x}),
            pooler.get_pool(cr.dbname).get('ecf.cupons.capa').read(
                cr, uid, cupom_ids, campos, context=context, load=load))
        return res

    def _get_dados_pessoas(self, cr, uid, dict_notas, dict_cupons, context=None):
        context = context and context.copy() or {}
        res = {}
        pessoa_ids = []
        if dict_notas:
            pessoa_ids.extend(map(lambda x: x.get('destinatario_id'), dict_notas.values()))
        if dict_cupons:
            pessoa_ids.extend(map(lambda x: x.get('cliente_id'), dict_cupons.values()))
        pessoa_ids = list(set(pessoa_ids))
        if False in pessoa_ids:
            pessoa_ids.remove(False)
        context['super_read_adm_pessoa'] = True
        campos = ['name', 'codigo']
        map(lambda x: res.update({x.get('id'): x}),
            pooler.get_pool(cr.dbname).get('adm.pessoa').read(
                cr, uid, pessoa_ids, campos, context=context))
        return res

    def _get_dados_grade(self,cr, uid, grade_ids, context=None):
        context = context and context.copy() or {}
        res = {}
        map(lambda x: res.update({x.get('id'): x.get('name')}),
            pooler.get_pool(cr.dbname).get('estoque.produto.grade').read(
                cr, uid, grade_ids, ['name'], context=context))
        return res

    def get_total_vendedor_forma_pagamento(self, cr, uid, query, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT 
                forma_vendedor_id, 
                sum(vr_desconto) AS vr_desconto, 
                sum(vr_bruto) AS vr_bruto, 
                sum(vr_liquido) AS vr_liquido, 
                nome_forma, 
                forma_pagamento_id, 
                nome_vendedor
            FROM (
                {query}
            ) AS tmp
            GROUP BY
                forma_vendedor_id,
                forma_pagamento_id,
                nome_forma,
                nome_vendedor
        """.format(query=query)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def get_total_vendedor_forma_pagamento_nf(self, cr, uid, vendedor_ids, nf_ids, devolucao_nf_ids,
                                              context=None):
        context = context and context.copy() or {}
        nf_vendedor_ids = list(vendedor_ids)
        sql_sem_vendedor = ''
        if [] in vendedor_ids:
            nf_vendedor_ids.remove([])
            sql_sem_vendedor = """
                OR movimento.vendedor_id IS NULL
                OR movimento.vendedor_externo_id IS NULL
            """
        sql = """
                SELECT
                    (CASE 
                        WHEN movimento.vendedor_id IS NOT NULL 
                            THEN movimento.vendedor_id 
                        ELSE movimento.vendedor_externo_id 
                    END) AS forma_vendedor_id,
                    (
                        SELECT nome 
                        FROM adm_pessoa 
                        WHERE 
                            id IN (
                                SELECT pessoa_id 
                                FROM vendas_vendedor 
                                WHERE 
                                    CASE 
                                        WHEN movimento.vendedor_id IS NOT NULL 
                                            THEN id = movimento.vendedor_id 
                                        ELSE id = movimento.vendedor_externo_id 
                                    END
                                )
                    ) AS nome_vendedor,
                    round((sum(COALESCE(movimento.vr_desconto + movimento.vr_desconto_rateado, 0)) 
                        / nf.vr_total_nf) * condicao.valor, 2) AS vr_desconto,
                    round(condicao.valor + 
                        ((sum(COALESCE(movimento.vr_desconto + movimento.vr_desconto_rateado, 0)) 
                        / nf.vr_total_nf) * condicao.valor), 2) AS vr_bruto,
                    condicao.valor AS vr_liquido,
                    forma.name AS nome_forma,
                    forma.id AS forma_pagamento_id
                FROM
                    fiscal_nf_capa nf
                INNER JOIN
                    estoque_movimento_itens movimento ON nf.id = movimento.nf_id
                INNER JOIN
                    fiscal_condicao_pagamento condicao ON nf.id = condicao.capa_id
                INNER JOIN
                    adm_forma_pgto forma ON forma.id = condicao.forma_pagto_id
                WHERE
                    nf.id IN ({nf_ids})
                    AND nf.vr_total_nf > 0
                    AND (movimento.vendedor_id IN ({vendedor_ids}) 
                         OR movimento.vendedor_externo_id IN ({vendedor_ids})
                         {sql_sem_vendedor}
                         )
                GROUP BY
                    forma_vendedor_id,
                    forma_pagamento_id, condicao.valor , nf.id, nome_vendedor
                UNION ALL
                -- DEVOLUCAO
                SELECT
                    (CASE 
                        WHEN movimento.vendedor_id IS NOT NULL 
                            THEN movimento.vendedor_id 
                        ELSE movimento.vendedor_externo_id 
                    END) AS forma_vendedor_id,
                    (
                        SELECT nome 
                        FROM adm_pessoa 
                        WHERE 
                            id IN (
                                SELECT pessoa_id 
                                FROM vendas_vendedor 
                                WHERE 
                                    CASE 
                                        WHEN movimento.vendedor_id IS NOT NULL 
                                            THEN id = movimento.vendedor_id 
                                        ELSE id = movimento.vendedor_externo_id 
                                    END
                                )
                    ) AS nome_vendedor,
                    round((sum(COALESCE(movimento.vr_desconto + movimento.vr_desconto_rateado, 0)) 
                        / nf.vr_total_nf) * condicao.valor, 2) * -1 AS vr_desconto,
                    round(condicao.valor + 
                        ((sum(COALESCE(movimento.vr_desconto + movimento.vr_desconto_rateado, 0)) 
                        / nf.vr_total_nf) * condicao.valor), 2) * -1 AS vr_bruto,
                    condicao.valor * -1 AS vr_liquido,
                    forma.name AS nome_forma,
                    forma.id AS forma_pagamento_id
                FROM
                    fiscal_nf_capa nf
                INNER JOIN
                    estoque_movimento_itens movimento ON nf.id = movimento.nf_id
                INNER JOIN
                    fiscal_condicao_pagamento condicao ON nf.id = condicao.capa_id
                INNER JOIN
                    adm_forma_pgto forma ON forma.id = condicao.forma_pagto_id
                WHERE
                    nf.id IN ({devolucao_nf_ids})
                    AND nf.vr_total_nf > 0
                    AND (movimento.vendedor_id IN ({vendedor_ids}) 
                         OR movimento.vendedor_externo_id IN ({vendedor_ids})
                         {sql_sem_vendedor}
                         )
                GROUP BY
                    forma_vendedor_id,
                    forma_pagamento_id, condicao.valor , nf.id, nome_vendedor	
        """.format(nf_ids=nf_ids and ', '.join(map(lambda n: str(n), nf_ids)) or '-1',
                   vendedor_ids=nf_vendedor_ids and ', '.join(map(lambda v: str(v),
                                                                  nf_vendedor_ids)) or '-1',
                   devolucao_nf_ids=devolucao_nf_ids and ', '.join(map(lambda d: str(d),
                                                                       devolucao_nf_ids)) or '-1',
                   sql_sem_vendedor=sql_sem_vendedor)
        return sql

    def get_total_vendedor_forma_pagamento_cupom(self, cr, uid, vendedor_ids, cupom_ids,
                                                 context=None):
        context = context and context.copy() or {}
        cupom_vendedor_ids = list(vendedor_ids)
        sql_sem_vendedor = ''
        if [] in vendedor_ids:
            cupom_vendedor_ids.remove([])
            sql_sem_vendedor = """
                OR movimento.vendedor_id IS NULL
                OR movimento.vendedor_externo_id IS NULL
            """
        sql = """
                SELECT
                    (CASE 
                        WHEN movimento.vendedor_id IS NOT NULL 
                            THEN movimento.vendedor_id 
                        ELSE movimento.vendedor_externo_id 
                    END) AS forma_vendedor_id,
                    (
                        SELECT nome 
                        FROM adm_pessoa 
                        WHERE 
                            id IN (
                                SELECT pessoa_id 
                                FROM vendas_vendedor 
                                WHERE 
                                    CASE 
                                        WHEN movimento.vendedor_id IS NOT NULL 
                                            THEN id = movimento.vendedor_id 
                                        ELSE id = movimento.vendedor_externo_id 
                                    END
                                )
                    ) AS nome_vendedor,
                    round((sum(COALESCE(movimento.vr_desconto + movimento.vr_desconto_rateado, 0)) 
                        / COALESCE(cupom.total_cupom, 1)) * finalizacao.valor, 2) AS vr_desconto,
                    round(finalizacao.valor + 
                        ((sum(COALESCE(movimento.vr_desconto + movimento.vr_desconto_rateado, 0)) 
                        / COALESCE(cupom.total_cupom, 1)) * finalizacao.valor), 2) AS vr_bruto,
                    finalizacao.valor AS vr_liquido,
                    forma.name AS nome_forma,
                    forma.id AS forma_pagamento_id
                FROM
                    ecf_cupons_capa cupom
                INNER JOIN
                    estoque_movimento_itens movimento ON cupom.id = movimento.cupom_id
                INNER JOIN
                    ecf_cupons_finalizacao finalizacao ON cupom.id = finalizacao.cupom_id
                INNER JOIN
                    adm_condicao_forma condicao_forma ON condicao_forma.id = finalizacao.finalizadora_id
                INNER JOIN
                    adm_forma_pgto forma ON forma.id = condicao_forma.forma_pagamento_id
                WHERE
                    cupom.id IN ({cupom_ids})
                    AND (movimento.vendedor_id IN ({vendedor_ids}) 
                         OR movimento.vendedor_externo_id IN ({vendedor_ids})
                         {sql_sem_vendedor}
                         )
                GROUP BY
                    forma_vendedor_id,
                    forma.id, finalizacao.valor , cupom.id, nome_vendedor        
        """.format(cupom_ids=cupom_ids and ', '.join(map(lambda c: str(c), cupom_ids)) or '-1',
                   vendedor_ids=cupom_vendedor_ids and ', '.join(map(lambda v: str(v),
                                                                     cupom_vendedor_ids)) or '-1',
                   sql_sem_vendedor=sql_sem_vendedor)
        return sql


report_custom('report.vendas_apuracao_comissao')


# noinspection PyAbstractClass
class vendas_apuracao_comissao_osv(osv.TransientModel):
    """
    Classe do formulario. Comissao de vendas
    """
    _name = 'vendas.apuracao.comissao.osv'
    _columns = {
        'tipo_vendedor': fields.selection([('E', 'Externo'), ('I', 'Interno')], 'Tipo Vendedor'),
        'vendedor_id': fields.many2one('vendas.vendedor', 'Vendedor'),
        'tipo_relatorio': fields.selection(
            [('A', 'Analitico'), ('S', 'Sintetico')], 'Tipo relatorio', required='1'),
        'data_ini': fields.date('Data inicial', required='1'),
        'data_fim': fields.date('Data final', required='1'),
        'categoria_ids': fields.many2many(
            'estoque.categoria', 'estoque_categoria_produto', 'categoria_produto_id',
            'categoria_id', 'Categoria'),
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_estoque_parametro = pooler.get_pool(cr.dbname).get('estoque.parametro')
        obj_vendas_vendedor = pooler.get_pool(cr.dbname).get('vendas.vendedor')
        company_id = pooler.get_pool(cr.dbname).get('adm.empresa.objeto').getEmpresa(
            cr, uid, 'vendas.apuracao.comissao.osv', context=context)
        estoque_parametro_id = obj_estoque_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)
        if not estoque_parametro_id:
            raise osv.except_osv(_('Atencao'),
                                 _('Parametro do estoque nao cadastrado para essa empresa'))
        context.update({'active_test': False})
        vendedor_id = obj_vendas_vendedor.search(cr, uid, [(1, '=', 1)], context=context)
        if not vendedor_id:
            raise osv.except_osv(_('Atencao'), _('Nao ha vendedor cadastrado'))
        relatorio_dados = self.read(
            cr, uid, ids[0] if isinstance(ids, list) else ids, context=context)
        if relatorio_dados.get('data_ini') > relatorio_dados.get('data_fim'):
            raise osv.except_osv(_('Atencao'),
                                 _('Data inicial nao pode ser maior que a data final'))
        datas = dict(tipo_vendedor=relatorio_dados.get('tipo_vendedor'),
                     vendedor_id=relatorio_dados.get('vendedor_id'),
                     tipo_relatorio=relatorio_dados.get('tipo_relatorio'),
                     data_ini=relatorio_dados.get('data_ini'),
                     data_fim=relatorio_dados.get('data_fim'),
                     categoria_ids=relatorio_dados.get('categoria_ids'))

        return dict(type='ir.actions.report.xml', report_name='vendas_apuracao_comissao',
                    datas=datas)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
