# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from funcoes import wizanv_get_total_geral
from openerp.osv.fields import float_round


class wizard_analise_venda(osv.TransientModel):

    def wizanv_get_tipo_documento(self, cr, uid, context):
        context = context and context.copy() or {}
        return [('P', 'Pedido')]

    _name = 'wizard.analise.venda'
    _rec_name = 'wizanv_pessoa_id'
    _columns = {
        'wizanv_documento': fields.selection(wizanv_get_tipo_documento, 'Documento', size=1,
                                             required=True),
        'wizanv_pessoa_id': fields.many2one('adm.pessoa', 'Cliente', ondelete='cascade'),
        'wizanv_pedido_id': fields.many2one('estoquepedidos.pedido.capa', 'Pedido',
                                            ondelete='cascade'),
        'wizanv_vr_total_item': fields.function(
            wizanv_get_total_geral, method=True, string='Valor total dos itens', digits=(15, 2),
            type='float', multi=True),
        'wizanv_vr_total_custo': fields.function(
            wizanv_get_total_geral, method=True, string='Valor total dos custos', digits=(15, 5),
            type='float', multi=True),
        'wizanv_vr_total_custo_fixo': fields.function(
            wizanv_get_total_geral, method=True, string='Valor total custo fixo', digits=(15, 2),
            type='float', multi=True),
        'wizanv_vr_total_lucro': fields.function(
            wizanv_get_total_geral, method=True, string='Valor total do lucro', digits=(15, 2),
            type='float', multi=True),
        'wizanv_percentual_total_lucro': fields.function(
            wizanv_get_total_geral, method=True, string='Percentual total do lucro', digits=(3, 2),
            type='float', multi=True),
        'wizanv_percentual_total_contribuicao': fields.function(
            wizanv_get_total_geral, method=True, string='Percentual total agregado',
            digits=(3, 2), type='float', multi=True),
        'wizanv_numero_documento': fields.char('Numero documento', size=15),
        'wizanv_item_ids': fields.one2many('wizard.analise.venda.item', 'wizani_capa_id', 'Itens'),
        'wizanv_id_documento_informado': fields.boolean('ID do documento informado'),
        'wizanv_bloqueado_desconto': fields.boolean('Bloqueado limite desconto'),
        'wizanv_vr_despesas': fields.float('Despesas', digits=(15, 2), readonly=True),
        'wizanv_vr_desconto': fields.float('Desconto', digits=(15, 2), readonly=True),
        'wizanv_vr_seguro': fields.float('Seguro', digits=(15, 2), readonly=True),
        'wizanv_vr_frete': fields.float('Frete', digits=(14, 2), readonly=True),
        'wizanv_vr_total_documento': fields.float('Total do documento', digits=(15, 2),
                                                  readonly=True),
    }

    _defaults = {
        'wizanv_id_documento_informado': lambda *a: False,
        'wizanv_bloqueado_desconto': lambda *a: False,
        'wizanv_vr_despesas': 0.0,
        'wizanv_vr_desconto': 0.0,
        'wizanv_vr_seguro': 0.0,
        'wizanv_vr_frete': 0.0,
    }

    def wizanv_onchange_pessoa(self, cr, uid, ids, pessoa_id, context=None):
        context = context and context.copy() or {}
        return {'value': {'wizanv_pedido_id': False}}

    def wizanv_onchange_tipo_documento(self, cr, uid, ids, tipo_documento, context=None):
        context = context and context.copy() or {}
        vals = {}
        if not tipo_documento:
            vals.update({'wizanv_pedido_id': False})
        return {'value': vals}

    def wizanv_onchange_id_documento(self, cr, uid, ids, pedido_id, context=None):
        context = context and context.copy() or {}
        vals = {'wizanv_id_documento_informado': False, 'wizanv_bloqueado_desconto': False}
        if pedido_id:
            vals.update({'wizanv_id_documento_informado': True})
            if self.pool.get('vendas.liberar.desconto').search(
                    cr, uid, [('venauv_pedido_id', '=', pedido_id),
                              ('venauv_autorizado', '=', False)], limit=1, context=context):
                vals.update({'wizanv_bloqueado_desconto': True})
        return {'value': vals}

    def wizanv_button_libera_desconto(self, cr, uid, ids, context=None):
        """
        O metodo realiza a liberacao de desconto para o documento.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro, caso o documento tenha sido liberado. Senão , Falso.
        :rtype: bool
        """
        context = context and context.copy() or {}
        brw_analise_venda = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        pedido_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
        if not pedido_id:
            return False
        obj_vendas_liberar_desconto = self.pool.get('vendas.liberar.desconto')
        liberar_desconto_ids = obj_vendas_liberar_desconto.search(
            cr, uid, [('venauv_pedido_id', '=', pedido_id), ('venauv_autorizado', '=', False)],
            limit=1, context=context)
        if liberar_desconto_ids:
            obj_vendas_liberar_desconto.write(cr, uid, liberar_desconto_ids,
                                              {'venauv_autorizado': True}, context)
        self.write(cr, uid, ids, {'wizanv_bloqueado_desconto': False}, context)
        return True

    def wizanv_get_id_documento(self, cr, uid, brw_analise_venda, context=None):
        """
        O metodo identifica o registro do documento que sera feita a analise.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param brw_analise_venda: browse desse objeto.
        :type brw_analise_venda: object
        :param context: dados contextuais.
        :type context: dict
        :return: Inteiro, que representa id do registro. Senao, Falso.
        :rtype: int
        """
        context = context and context.copy() or {}
        if brw_analise_venda.wizanv_pedido_id:
            return brw_analise_venda.wizanv_pedido_id.id
        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        numero_pedido = '%' + brw_analise_venda.wizanv_numero_documento.zfill(10)
        pedido_id = False
        pedido_ids = obj_estoquepedidos_pedido_capa.search(
            cr, uid, [('numero_pedido', 'like', numero_pedido), ('tipo', '=', 'V'),
                      ('state', '!=', 'C')], limit=1, context=context)
        if pedido_ids:
            pedido_id = pedido_ids[0]
        return pedido_id

    def wizanv_button_analisa_venda(self, cr, uid, ids, context=None):
        """
        O metodo cria os dados para a tabela wizard_analise_venda_item.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dados contextuais.
        :type context: dict
        :return: Verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            DELETE FROM wizard_analise_venda_item WHERE wizani_capa_id = %s
        """ % ids[0]
        cr.execute(sql)
        brw_analise_venda = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        list_vals = self.wizanv_trata_analise_pedido(cr, uid, brw_analise_venda, context)
        if list_vals:
            vals = {'wizanv_item_ids': list_vals}
            pedido_id = self.wizanv_get_id_documento(cr, uid, brw_analise_venda, context)
            if pedido_id:
                if self.pool.get('vendas.liberar.desconto').search(
                        cr, uid, [('venauv_pedido_id', '=', pedido_id),
                                  ('venauv_autorizado', '=', False)], limit=1, context=context):
                    vals.update({'wizanv_bloqueado_desconto': True})
                brw_pedido_capa = self.pool.get('estoquepedidos.pedido.capa').browse(
                    cr, uid, pedido_id, context)
                itens = brw_pedido_capa.pedido_item_ids
                vals.update({
                    'wizanv_vr_despesas': brw_pedido_capa.vr_despesas_acessorias or 0.0,
                    'wizanv_vr_desconto': (brw_pedido_capa.vr_desconto or 0.0) + sum(
                        map(lambda i: i.vr_desconto or 0.0, itens)),
                    'wizanv_vr_seguro': brw_pedido_capa.vr_seguro or 0.0,
                    'wizanv_vr_frete': brw_pedido_capa.vr_frete or 0.0,
                    'wizanv_vr_total_documento': brw_pedido_capa.valor_total or 0.0})
            self.write(cr, uid, ids, vals, context)
        if context.get('active_model') and context.get('active_id'):
            obj_model_data = self.pool.get('ir.model.data')
            reference = obj_model_data.get_object_reference(
                cr, uid, 'vendas', 'wizard_analise_venda_form')
            view_id = reference and reference[1] or False
            return {'res_id': ids[0] if isinstance(ids, list) else ids,
                    'name': 'Analise de Venda',
                    'view_mode': 'form',
                    'res_model': 'wizard.analise.venda',
                    'view_id': [view_id],
                    'target': 'new',
                    'type': 'ir.actions.act_window',
                    'context': context
                    }
        return True

    def wizanv_trata_analise_pedido(self, cr, uid, brw_analise_venda, context=None):
        """
        O metodo reune os dados referente ao pedido para que sejam incluidos na tabela
        wizard_analise_venda_item.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param brw_analise_venda: browse desse objeto.
        :type brw_analise_venda: object
        :param context: dados contextuais.
        :type context: dict
        :return: dados para insercao na tabela wizard_analise_venda_item.
        :rtype: list
        """
        context = context and context.copy() or {}
        list_vals = []
        obj_adm_utils = self.pool.get('adm.utils')
        brw_pedido = brw_analise_venda.wizanv_pedido_id
        if not brw_pedido:
            obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
            numero_pedido = '%' + brw_analise_venda.wizanv_numero_documento.zfill(10)
            pedido_ids = obj_estoquepedidos_pedido_capa.search(
                cr, uid, [('numero_pedido', 'like', numero_pedido), ('tipo', '=', 'V'),
                          ('state', '!=', 'C'),
                          ('pessoa_id', '=', brw_analise_venda.wizanv_pessoa_id.id)], limit=1,
                context=context)
            if not pedido_ids:
                raise osv.except_osv(_('Atencao'), _('Pedido nao encontrado.'))
            brw_pedido = obj_estoquepedidos_pedido_capa.browse(cr, uid, pedido_ids[0], context)
        item_ids = brw_pedido.pedido_item_ids
        total_pedido = brw_pedido.valor_total
        total_itens = sum(map(lambda x: x.valor_total, item_ids))
        dif = total_pedido - total_itens
        dict_rateio = {}
        if dif <= -0.01 or 0.01 < dif:
            lista_rateio = []
            for i in item_ids:
                lista_rateio.append([i.id, i.valor_total])

            dict_rateio = dict(obj_adm_utils.efetuar_rateio(dif, lista_rateio))

        produto_ids = list(set(map(lambda p: p.produto_id.id, item_ids)))
        custo_dados = self.wizanv_get_preco_custo(cr, uid, produto_ids, context)
        parametros = ['comissoes', 'custo_fixo']
        estoque_parametro_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', parametros, context=context)
        comissao = estoque_parametro_dados.get('comissoes') or 0.0
        custo_fixo = estoque_parametro_dados.get('custo_fixo') or 0.0
        for item in item_ids:
            produto_id = item.produto_id.id
            quantidade_item = item.qtde_atendida or item.qtde_pedida
            quantidade_item_total = quantidade_item * (
                    item.produto_embalagem_id.embalagem_id.quantidade_na_embalagem or 1.0)
            custo_item_total = custo_dados.get(produto_id) * quantidade_item_total
            vr_total_item = (item.valor_total or 0) + (dict_rateio.get(item.id) or 0)
            comissao_item_total = float_round(comissao * vr_total_item / 100, 2)
            custo_fixo_item_total = float_round(custo_fixo * vr_total_item / 100, 2)
            lucro_item_total = float_round(vr_total_item - custo_item_total - comissao_item_total -
                                           custo_fixo_item_total, 2)
            agregado_item_total = vr_total_item - custo_item_total
            vals = {'wizani_codigo': item.produto_id.codigo,
                    'wizani_grade': item.grade_id.grade,
                    'wizani_descricao': item.grade_id.name,
                    'wizani_embalagem': item.produto_embalagem_id.embalagem_id.sigla,
                    'wizani_quantidade': quantidade_item,
                    'wizani_vr_total': vr_total_item,
                    'wizani_custo_total': custo_item_total,
                    'wizani_vr_lucro': lucro_item_total,
                    'wizani_percentual_lucro': float_round(lucro_item_total * 100 / vr_total_item,
                                                           2),
                    'wizani_vr_contribuicao': agregado_item_total,
                    'wizani_percentual_contribuicao': agregado_item_total / vr_total_item * 100,
                    'wizani_custo_fixo': custo_fixo_item_total,
                    'wizani_vr_comissao': comissao_item_total,
                    'wizani_produto_id': produto_id,
                    'wizani_preco_venda_item_total': vr_total_item,
                    }
            list_vals.append((0, 0, vals))
        return list_vals

    def wizanv_get_preco_custo(self, cr, uid, produto_ids, context=None):
        """
        O metodo encontra o custo para preco de venda para os produtos recebidos (produto_ids).
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param produto_ids: lista de id da tabela estoque_produto, identificadores do produto.
        :type produto_ids: list
        :param context: dados contextuais.
        :type context: dict
        :return:
        """
        context = context and context.copy() or {}
        custo_preco_venda = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_custo_para_precovenda', context=context).get(
            'estpar_custo_para_precovenda')
        if not custo_preco_venda:
            raise osv.except_osv(_('Atencao'), _('"Custo p/preço de venda" nao cadastrado nos '
                                                 'parametros do estoque.'))
        if custo_preco_venda == 'R':
            custo_preco_venda = 'estprc_custo_reposicao'
        elif custo_preco_venda == 'M':
            custo_preco_venda = 'estprc_custo_medio_contabil'
        elif custo_preco_venda == 'C':
            custo_preco_venda = 'estprc_custo_corrigido'
        else:
            custo_preco_venda = 'estprc_ultimo_custo'
        res = {}
        custos = self.pool.get('estoque.preco.custo').get_custos_do_dia(cr, uid, produto_ids,
                                                                        context=context)
        for produto_id in produto_ids:
            custo = custos.get(produto_id) or {}
            res[produto_id] = custo.get(custo_preco_venda) or 0.0
        return res


class wizard_analise_venda_item(osv.TransientModel):

    _name = 'wizard.analise.venda.item'
    _columns = {
        'wizani_capa_id': fields.many2one('wizard.analise.venda', 'Capa', ondelete='cascade'),
        'wizani_codigo': fields.char('Cod', size=20),
        'wizani_grade': fields.char('Grade', size=2),
        'wizani_descricao': fields.char('Desricao', size=63),
        'wizani_embalagem': fields.char('Emb', size=7),
        'wizani_quantidade': fields.float('Qtde', digits=(15, 3)),
        'wizani_vr_total': fields.float('Venda', digits=(15, 2)),
        'wizani_custo_total': fields.float('Custo', digits=(15, 5)),
        'wizani_vr_lucro': fields.float('Lucro', digits=(15, 2)),
        'wizani_percentual_lucro': fields.float('% Lucro', digits=(3, 2)),
        'wizani_vr_contribuicao': fields.float('V. Agregado', digits=(15, 5)),
        'wizani_percentual_contribuicao': fields.float('% V. Agreg', digits=(3, 2)),
        'wizani_custo_fixo': fields.float('C. Fixo', digits=(15, 2)),
        'wizani_vr_comissao': fields.float('Comissao', digits=(15, 2)),
        'wizani_produto_id': fields.many2one('estoque.produto', 'Produto', ondelete='cascade'),
        'wizani_preco_venda_item_total': fields.float('Preco de venda total', digits=(15, 2)),
    }
