# *-* encoding: utf-8 *-*

from openerp.osv import osv, fields
from openerp.tools.translate import _
from datetime import datetime


class wizard_resumo_vendas(osv.TransientModel):
    _name = 'wizard.resumo.vendas'
    _description = u'Resumo de Vendas'
    _columns = {
        'data_inicial': fields.date('Data Inicial', required=True),
        'data_final': fields.date('Data Final', required=True),
        'resumo_modelo': fields.boolean('Modelo'),
        'resumo_forma_condicao': fields.boolean('Forma/Condicao'),
        'resumo_cfop_natureza': fields.boolean('CFOP/Natureza'),
        'devolucao_venda': fields.boolean('Devolucoes'),
    }

    _defaults = {
        'resumo_modelo': True,
        'resumo_forma_condicao': True,
        'resumo_cfop_natureza': True,
        'devolucao_venda': True
    }

    def button_print(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        vals = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name,
                                                                    context=context)
        company_ids = self.pool.get('res.company').search(
            cr, uid, [('parent_id', 'child_of', company_id)], context=context)

        if vals.data_final < vals.data_inicial:
            raise osv.except_osv(_('Atencao'), _('A data final deve ser maior ou igual a data '
                                                 'inicial'))
        dt_inicial = datetime.strptime("%s 00:00:00" % vals.data_inicial, "%Y-%m-%d %H:%M:%S")
        dt_inicial_utc = self.pool.get('adm.utils').admutl_converte_tz(cr, uid, dt_inicial, 'America/Sao_Paulo', 'UTC', context=None)
        dt_inicial_utc = dt_inicial_utc.strftime("%Y-%m-%d %H:%M:%S")
        dt_final = datetime.strptime("%s 23:59:59" % vals.data_final, "%Y-%m-%d %H:%M:%S")
        dt_final_utc = self.pool.get('adm.utils').admutl_converte_tz(cr, uid, dt_final, 'America/Sao_Paulo', 'UTC', context=None)
        dt_final_utc = dt_final_utc.strftime("%Y-%m-%d %H:%M:%S")

        if not any([vals.resumo_modelo, vals.resumo_forma_condicao, vals.resumo_cfop_natureza,
                    vals.devolucao_venda]):
            raise osv.except_osv(_('Atencao'), _('Selecione no minimo um dos tipos de resumo.'))

        quantidade_vendas = self.get_quantidade_venda(cr, uid, ids, company_ids, dt_inicial_utc,
                                                      dt_final_utc, str(dt_inicial), str(dt_final),  context)

        dados_modelo_serie_valor = []
        vr_total = 0.0
        if vals.resumo_modelo:
            dados_modelo_serie_valor = self.modelo_serie_valor(cr, uid, ids,
                                                               dt_inicial_utc, dt_final_utc,
                                                               str(dt_inicial), str(dt_final),
                                                               company_ids,
                                                               context)
            vr_total = sum(map(lambda i: i.get('valor'), dados_modelo_serie_valor))

        dados_pagamento_condicao_valor = []
        if vals.resumo_forma_condicao:
            dados_pagamento_condicao_valor = self.pagamento_condicao_valor(
                cr, uid, ids, dt_inicial_utc, dt_final_utc, str(dt_inicial), str(dt_final), company_ids, context)
            vr_total = sum(map(lambda i: i.get('valor_cond'), dados_pagamento_condicao_valor))

        dados_cfop_natureza_valor = []
        if vals.resumo_cfop_natureza:
            dados_cfop_natureza_valor = self.cfop_natureza_valor(cr, uid, ids, dt_inicial_utc,
                                                                 dt_final_utc, str(dt_inicial), str(dt_final),
                                                                 company_ids,
                                                                 context)
            vr_total = sum(map(lambda i: i.get('valor'), dados_cfop_natureza_valor))

        dados_devolucao_vendas_valor = []
        if vals.devolucao_venda:
            dados_devolucao_vendas_valor = self.devolucao_vendas_valor(
                cr, uid, ids, dt_inicial_utc, dt_final_utc, company_ids, context)

        datas = {
            'modelo_serie_valor': dados_modelo_serie_valor,
            'pagamento_condicao_valor': dados_pagamento_condicao_valor,
            'cfop_natureza_valor': dados_cfop_natureza_valor,
            'devolucao_vendas_valor': dados_devolucao_vendas_valor,
            'data_inicial': vals.data_inicial,
            'data_final': vals.data_final,
            'quantidade_vendas': quantidade_vendas,
            'vr_total': vr_total
        }

        return{
            'type': 'ir_actions_report_xml',
            'report_name': 'resumo_vendas',
            'datas': datas,
        }

    def get_quantidade_venda(self, cr, uid, ids, company_ids, data_inicial_utc, data_final_utc, data_inicial, data_final,
                             context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT 
                count(capa.id) AS quantidade_venda
            FROM fiscal_nf_capa capa
            INNER JOIN fiscal_cfop_natureza cfop_natureza ON cfop_natureza.id = capa.natureza_id
            INNER JOIN fiscal_cfop cfop ON cfop.id = cfop_natureza.cfop_id
            WHERE
                capa.company_id IN ({company_ids}) 
                AND capa.tipo = 'S'
                AND capa.state = '3'
                AND cfop.movimento_tipo IN ('C', 'V')
                AND cfop.codigo NOT IN ('5929', '6929')
                AND data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' 
                    AND '{data_final}'  
        """.format(data_inicial=data_inicial_utc, data_final=data_final_utc,
                   company_ids=', '.join(map(lambda c: str(c), company_ids)))
        cr.execute(sql)
        dados = cr.dictfetchall() or [{}]
        quantidade_venda = 0
        if dados:
            quantidade_venda = dados[0].get('quantidade_venda') or 0
        return quantidade_venda

    def modelo_serie_valor(self, cr, uid, ids, data_inicial_utc, data_final_utc, data_inicial, data_final, company_ids, context=None):
        context = context and context.copy() or {}
        sql = """
                SELECT fismnf_codigo AS modelo,
                       serie_nf AS serie,                       
                       Sum(vr_total_nf) AS valor,
                       min(numero_nf) as inicial,
                       max(numero_nf) as final
                FROM fiscal_nf_capa capa
                INNER JOIN fiscal_cfop_natureza cfop_natureza ON cfop_natureza.id = capa.natureza_id
                INNER JOIN fiscal_cfop cfop ON cfop.id = cfop_natureza.cfop_id
                INNER JOIN fiscal_modelo_nf modelo_nf ON modelo_nf.id = capa.modelo_nf_id
                WHERE capa.tipo = 'S'
                AND capa.state = '3'
                AND cfop.movimento_tipo IN ('C', 'V')
                AND data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'  
                AND capa.company_id IN ({company_ids})
                GROUP BY modelo,
                         serie
        """.format(data_inicial=data_inicial_utc, data_final=data_final_utc,
                   company_ids=', '.join(map(lambda c: str(c), company_ids)))
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def pagamento_condicao_valor(self, cr, uid, ids, data_inicial_utc, data_final_utc, data_inicial, data_final, company_ids,
                                 context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT forma_pagto.name AS forma_pagto,
                   condicao_forma.name AS condicao_forma,
                   Sum(condicao.valor) AS valor_cond
            FROM fiscal_nf_capa capa
            INNER JOIN fiscal_cfop_natureza cfop_natureza ON cfop_natureza.id = capa.natureza_id
            INNER JOIN fiscal_cfop cfop ON cfop.id = cfop_natureza.cfop_id
            INNER JOIN fiscal_condicao_pagamento condicao ON condicao.capa_id = capa.id
            INNER JOIN adm_forma_pgto forma_pagto ON forma_pagto.id = condicao.forma_pagto_id
            INNER JOIN adm_condicao_forma condicao_forma ON condicao_forma.id = condicao.condicao_forma_id
            WHERE capa.tipo = 'S'
            AND capa.state = '3'
            AND cfop.movimento_tipo IN ('C', 'V')
            AND data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
            AND capa.company_id IN ({company_ids})
            GROUP BY forma_pagto,
                     condicao_forma
            ORDER BY forma_pagto
        """.format(data_inicial=data_inicial_utc, data_final=data_final_utc,
                   company_ids=', '.join(map(lambda c: str(c), company_ids)))

        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def cfop_natureza_valor(self, cr, uid, ids, data_inicial_utc, data_final_utc, data_inicial, data_final, company_ids, context=None):
        context = context and context.copy() or {}
        sql = """
        SELECT cfop_natureza.fisnat_natureza_fiscal as natureza_fiscal,
               cfop.codigo AS codigo_cfop,
               Sum(vr_total_nf) AS valor
        FROM fiscal_nf_capa capa
        INNER JOIN fiscal_cfop_natureza cfop_natureza ON cfop_natureza.id = capa.natureza_id
        INNER JOIN fiscal_cfop cfop ON cfop.id = cfop_natureza.cfop_id
        WHERE capa.tipo = 'S'
          AND capa.state = '3'
          AND cfop.movimento_tipo IN ('C', 'V')
          AND data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
          AND capa.company_id IN ({company_ids})
        GROUP BY codigo_cfop,
                 natureza_fiscal
        """.format(data_inicial=data_inicial_utc, data_final=data_final_utc,
                   company_ids=', '.join(map(lambda c: str(c), company_ids)))

        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def devolucao_vendas_valor(self, cr, uid, ids, data_inicial, data_final, company_ids,
                               context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT
              (SELECT codigo || '-' || nome
               FROM adm_pessoa
               WHERE id = capa.destinatario_id) AS pessoa,
                   capa.numero_nf AS nota,
                   capa.serie_nf AS serie,
                   Sum(vr_total_nf) AS valor
            FROM fiscal_nf_capa capa
            INNER JOIN fiscal_cfop_natureza cfop_natureza ON cfop_natureza.id = capa.natureza_id
            INNER JOIN fiscal_cfop cfop ON cfop.id = cfop_natureza.cfop_id
            WHERE capa.tipo = 'E'
            AND capa.state = '3'
              AND cfop.movimento_tipo = 'D'
              AND data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
              AND CASE
                      WHEN capa.nf_propria = TRUE THEN data_emissao AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
                      ELSE data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '{data_inicial}' AND '{data_final}'
                  END
              AND capa.company_id IN ({company_ids})
            GROUP BY nota,
                     serie,
                     pessoa
        """.format(data_inicial=data_inicial, data_final=data_final,
                   company_ids=', '.join(map(lambda c: str(c), company_ids)))

        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados


