#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.tools.translate import _

NOME_FUNCOES = [
    ('0001', _('0001-Gerar tributacoes para encargos no movimento.')),
    ('0002', _('0002-Altera as margens dos produtos por categoria.')),
    ('0003', _('0003-Corrigir saldo dos itens')),
    ('0004', _('0004-Corrigir natureza da receita PIS/COFINS.')),
    ('0006', _('0006-Recriar movimento dos cupons.')),
    ('0007', _('0007-Acerta custo negativo do inventario')),
    ('0009', _('0009-Atualiza saldo do inventario'))
]
"""
chaves permitidas para o dicionario "campos"
    tipo - tipo de campo: date, many2one, many2many, selection - Obrigatorio
    label - String da descricao do campo - Obrigatorio
    seq - Sequencia do campo na tela (numerico, não string) - Opcional
    attrs - para definicao de atributo. Aceitos: required, readonly - Opcional
            para readonly é necessario  informar um string, com 4 argumentos separados por ","
            ex.: attrs: 'readonly,teste,=,T'
    help - String com o help para o campo - Opcional
    colspan - quantidade de colunas para usar no colspan
    relation - nome da relacao no caso de campos relacionais - Obrigatorio se campo relacional
    widget - widget a ser utilizado - Opcional
    default - Valor default para o campo - Opcional
    lista - lista de duplas para os campos selection - obrigatorio se selection
"""
dic_funcoes = {
    1: {'name': _('Gerar tributacoes para encargos no movimento'),
           'obs': _("""
Esta funcao apaga todas as tributacoes dos itens no movimento para o periodo selecionado e para os encargos selecionados.
Apos isto, gera as tributacoes novamente conforme o cadastro de cada item
"""),
           'campos': {
               'data_inicial': {'tipo': 'date', 'label': _('Data inicial'), 'seq': 1,
                     'attrs': ('required'), 'help':_('Data inicial para selecao')},
               'data_final': {'tipo': 'date', 'label': _('Data final'), 'seq': 2,
                     'attrs': 'required', 'help': _('Data final para selecao')},
               'encargos': {'tipo': 'many2many', 'label': _('Encargos'), 'seq': 3,
                     'widget': 'many2many_tags', 'relation': 'fiscal.encargo',
                     'attrs': ('required',), 'help': _('Encargos para atualizar')},
               'origem': {'tipo': 'selection', 'size': 1, 'label': _('Origem'), 'seq': 4,
                     'help': _('Origem do movimento'), 'default': 'T',
                     'lista':[('N', _('NF')), ('C', _('Cupom')), ('T', _('Todas'))]},
               'operacao': {'tipo': 'selection', 'size': 1, 'label': _('Operacao'), 'seq': 5,
                     'help': _('Entrada/Saida'), 'default': 'T', 'attrs': ('readonly,origem,=,C'),
                     'lista':[('E', _('Entrada')), ('S', _('Saida')), ('T', _('Todas'))]},
               }
           },
    2: dict(
        name=_('Altera as margens dos produtos por categoria'),
        obs=_('Esta funcao realiza a manutencao das margens de lucro (bruta, ideal e minima) dos '
              'produtos com a opcao de selecionar qual a categoria tera seus produtos coma as '
              'margens modificadas.'),
        campos=dict(
            fnc0002_categoria_id=dict(
                tipo='many2one', label=_('Categoria'), seq=1, relation='estoque.categoria',
                help=_('Encargos para atualizar')),
            fnc0002_margem_minima=dict(
                tipo='float', label=_('Margem minima (%)'), seq=2, default=0.0,
                help=_('Percentual da margem minima de lucro do produto.')),
            fnc0002_margem_ideal=dict(
                tipo='float', label=_('Margem ideal (%)'), seq=3, default=0.0,
                help=_('Percentual da margem ideal de lucro do produto.')),
            fnc0002_margem_bruta=dict(
                tipo='float', label=_('Margem bruta (%)'), seq=4, default=0.0,
                help=_('Percentual da margem bruta de lucro do produto.')),
            fnc0002_altera_categoria=dict(
                tipo='boolean', label=_('Alterar categoria'), seq=5, default=False,
                help=_('Se marcado, altera o valor das margens da categoria selecionada'))
        )
    ),
    3: dict(
        name=_('Corrigir saldo dos itens'),
        obs=_("Esta funcao apaga o saldo de todos os itens a partir da data selecionada, "
              "verificando se tem apenas pedidos e notas de venda nos estados faturado ou "
              "cancelado, assim corrige o saldo de acordo com a tabela 'estoque_movimento_itens'."),
        campos=dict(
            data_inicial=dict(tipo='date', label=_('Data inicial'), seq=1,
                              attrs=('required'),
                              help=_('Data inicial para exclusao')))
    ),
    4: dict(
        name=_('Corrigir natureza da receita PIS/COFINS'),
        obs=_('Esta funcao recupera todos os itens que estejam cadastrados na Categoria e/ou NCM e '
              'altera seus respectivos valores PIS/COFINS conforme informado.'),
        campos=dict(
            categoria_id=dict(
                tipo='many2one', label=_('Categoria'), seq=1, relation='estoque.categoria',
                help=_('Categoria a ser utilizada na atualizacao.')),
            ncm_id=dict(
                tipo='many2one', label=_('NCM'), seq=2, relation='fiscal.ncm',
                help=_('NCM a ser utilizado na atualizacao')),
            pis_natureza_receita=dict(
                tipo='char', size=3, label=_('Natureza da receita do PIS'), seq=3,
                help=_('Codigo da natureza da receita do PIS.'), attrs='required'),
            cofins_natureza_receita=dict(
                tipo='char', size=3, label=_('Natureza da receita do COFINS'), seq=4,
                help=_('Codigo da natureza da receita do PIS.'), attrs='required')
        )
    ),

    6: {'name': _('Recriar movimento dos cupons'),
           'obs': _("""
Esta funcao seleciona os cupons no intervalo de datas escolhido, apaga os movimentos destes cupons e gera
novamente seus movimentos.
"""),
           'campos': {
                       'data_inicial': {'tipo': 'date', 'label': _('Data inicial'), 'seq': 1,
                             'attrs': ('required'), 'help':_('Data inicial')
                                       },
                       'data_final': {'tipo': 'date', 'label': _('Data final'), 'seq': 1,
                             'attrs': ('required'), 'help':_('Data final')
                                       },
                     },
    },
    7: dict(name='Acerta custo negativo de itens do inventario',
               obs='''
Esta funcao ira buscar todos os itens que ficaram com o custo negativo ou zerado no inventario,
este custo podera ser o ultimo_custo ou o custo_medio_contabil de acordo com o parametro do estoque.
Apos filtrar todos custos menores que 0.01 centavo, o script ira pegar o valor de ultimo_custo
cadastrado para o item e alterar o custo que esta errado no inventario, lembrando que mesmo que
esteja cadastrado no parametro do estoque para utilizar o custo_medio_contabil,
seu valor sera alterado para o valor do ultimo_custo. Caso o ultimo_custo cadastrado para o item
seja 0, o script ira considerar o valor de 1 centavo no custo do item no inventario.
               ''',
               campos=dict(
                   inventario_id=dict(
                       tipo='many2one', label='Inventario', relation='estoque.inventario.capa',
                       seq=1, attrs='required'))),
    9: dict(name='Atualiza saldo do inventario',
            obs = "Este script pega os itens que tiveram o saldo anterior e o saldo contado"
                  "diferentes e atualiza o saldo dos produtos no dia que foi atualizado o "
                  "inventario com o saldo contado",
            campos=dict(
                inventario_id=dict(tipo='many2one', label='Inventario',
                                   relation='estoque.inventario.capa', seq=1,
                                   domain=[('atualizado', '=', True)], attrs='required'),
                contagem_com_diferenca=dict(tipo='boolean', label='So contagem com diferenca',
                                            default=True, seq=2)))
}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
