#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from openerp.osv import osv
from openerp.tools.translate import _


# noinspection PyPep8Naming
class websig_executor_tela_osv(osv.TransientModel):
    _name = 'websig.executor.tela.osv'
    _inherit = 'websig.executor.tela.osv'

    def websig_exec_0002(self, cr, uid, ids, context=None):
        """
        0002-Manutencao de margem de lucro.
        Esta funcao mantem as margens de lucro dos produtos por categoria. Se nenhuma categoria for
        informada na selecao, todas as categorias sofrerao alteracao das margens.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com os IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: integer
        :type ids: long|list[long]
        :type context: dict
        :return: lista com o ID dos produtos que sofreram alteracao das margens
        :rtype: list[long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Recupera as informacoes do formulario (f0002)
        executor_tela_id = ids[0] if isinstance(ids, list) else ids
        funcao_escolhida = eval(self.browse(cr, uid, executor_tela_id, context).dados)[2]

        # __________________________ Objects _________________________________
        obj_estoque_produto = self.pool.get('estoque.produto')

        categoria_id = funcao_escolhida.get('campos').get('fnc0002_categoria_id').get('value',
                                                                                      False)
        if categoria_id:
            categoria_ids = [categoria_id]
        else:
            categoria_ids = self.pool.get('estoque.categoria').search(cr, uid, [], context=context)

        margem_minima = funcao_escolhida.get('campos').get('fnc0002_margem_minima').get('value',
                                                                                        False)
        margem_ideal = funcao_escolhida.get('campos').get('fnc0002_margem_ideal').get('value',
                                                                                      False)
        margem_bruta = funcao_escolhida.get('campos').get('fnc0002_margem_bruta').get('value',
                                                                                      False)
        altera_categoria = funcao_escolhida.get('campos').get('fnc0002_altera_categoria').get(
            'value', False)

        # Realiza as alteracoes das margens por categoria
        args = [('categoria_id', 'in', categoria_ids)]
        produto_ids = obj_estoque_produto.search(cr, uid, args, context=context)

        # Altera as margens de lucro nos produtos
        if produto_ids:
            _sql = 'UPDATE estoque_produto ' \
                   'SET    margem_minima = %s, ' \
                   '       margem_ideal = %s, ' \
                   '       margem_bruta = %s ' \
                   'WHERE  id IN %s'
            _sql %= (margem_minima, margem_ideal, margem_bruta, tuple(produto_ids),)
            cr.execute(_sql)

        # Altera as margens de lucro nas categorias
        if categoria_ids and altera_categoria:
            _sql = 'UPDATE estoque_categoria ' \
                   'SET    margem_minima = %s, ' \
                   '       margem_ideal = %s, ' \
                   '       margem_bruta = %s ' \
                   'WHERE  id IN %s'
            t_categoria_ids = tuple(cr.split_for_in_conditions(categoria_ids))
            cr.execute(_sql, [margem_minima, margem_ideal, margem_bruta, t_categoria_ids])

        # Mostra mensagem
        msg = _('<strong>Concluido sem erro.</strong>'
                '<br>'
                'Quantidade de categorias alteradas: %s'
                '<br>'
                'Quantidade de produtos alterados: %s') % (
            categoria_ids.__len__() or _('Nenhuma'), produto_ids.__len__() or _('Nenhum'))
        self.write(cr, uid, ids, {'state': '3', 'mensagem': msg}, context)

        return produto_ids

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
