#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

# Variaveis
# Peca rolica
# Dap - diametro a altura do peito (duas variáveis em uma)
# ht - altura total da árvore (arvore)
# hi - comprimento da tora = comprimento da peça (parametro)
# hi_atual - começa com altura do toco (30 cm) (parametro)
# es - espessura peça (peca)
# di_casc - diâmetro com casca (calculado)
# di_casc2 - diâmetro com casca para calculo do volume da peça (calculado)
# di - diâmetro sem casca (usar o percentual de casca da especie)
# vr - volume peça roliça (calculado)
# vt - volume arvore (calculado usando coefientes das especies a0, a1, a2)
# narv - numero de árvores (inventario arvores)
# codpeca-codigo peça (usar o nome da peca)

# hc - altura comercial da árvore
# hi - comprimento da tora = comprimento da peça
# es - espessura da peça
# es_serra - espessura da serra (perda de madeira no corte)
# min_es - menor espessura da peça
# max_es - maior espessura da peça
# l - largura peça
# es_sobra - sobra peça
# ntora - numero da tora

# l_d, l_e - larguras direita e esquerda da peça
# l_s, l_i - larguras superior e inferior da peça
# l_d2 - largura direita da próxima peça
# l_s2 - largura superior da próxima peça
# cord_d, cord_e - cordas direita e esquerda da peça
# cord_s, cord_i - cordas superior e inferior da peça
# ang_d, ang_e - angulos direito e esquerdo formados c respectivas cordas
# ang_s, ang_i - angulos superior e inferior formados c respectivas cordas
# n-num.peças do 2o quadrante
# nes = n*es - numero de peças vezes sua espessura


from funcoes import *
from openerp.osv import osv, fields
from funcoes import calcmadeira_get_totais
from funcoes import calcmadeira_check_calculo_receita
from openerp.tools.translate import _
import base64


def calcmadeira_get_tipo_funcao(self, cr, uid, context=None):
    context = context and context.copy() or {}
    res = []
    if context.get('tipo_funcao') == '1':
        res = [('1', 'Calculo peca rolica')]
    elif context.get('tipo_funcao') in ('2', '3', '4', '5', '13', '15'):
        res = [('2', 'Calculo desdobro longitudinal base'), ('3', 'Calculo desdobro longitudinal'),
               ('4', 'Calculo desdobro quadrado circunscrito'),
               ('5', 'Calculo peca desdobro radial'), ('13', 'Calculo peca gira 180 90'),
               ('15', 'Calculo peca radial longitudinal')]
    elif context.get('tipo_funcao') in ('6', '7', '8', '9', '14', '16'):
        res = [('6', 'Calculo desdobro longitudinal base por tora'),
               ('7', 'Calculo desdobro longitudinal por tora'),
               ('8', 'Calculo desdobro quadrado circunscrito por tora'),
               ('9', 'Calculo desdobro radial por tora'), ('14', 'Calculo gira 180 90 por tora'),
               ('16', 'Calculo radial longitudinal por tora')]
    elif context.get('calculo_fator_forma'):
        res = [('12', 'Calculo Fator de Forma')]
    elif context.get('calculo_laminada_arvore'):
        res = [('10', 'Calculo laminacao')]
    elif context.get('calculo_laminada_tora'):
        res = [('11', 'Calculo laminacao por tora')]
    return res


class wizard_calcmadeira_calcula(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula'
    FUNCOES = [('', '')]
    _columns = {
        'company_id': fields.many2one('res.company', 'Empresa', ondelete='cascade', required=True,
                                      select=True),
        'peca_ids': fields.one2many('wizard.calcmadeira.calcula.itens', 'peca_id', 'Itens'),
        'result_ids': fields.one2many('wizard.calcmadeira.resultado.peca', 'result_id',
                                      'Resultado'),
        'pecas_rolicas': fields.many2many('calcmadeira.pecas.rolicas', 'calcmadeira_calcula_rolica',
                                          'id',
                                          'peca_id', 'Pecas Rolicas'),
        'pecas_serradas': fields.many2many('calcmadeira.pecas.serradas',
                                           'calcmadeira_calcula_serrada', 'id',
                                           'peca_id', 'Pecas Serradas'),
        'tipo_funcao': fields.selection(calcmadeira_get_tipo_funcao, 'Funcao', size=2,
                                        required=True),
        'vt1': fields.float('Volume total(m³)', digits=(3, 4), readonly=True),
        'narv': fields.integer('Arvores', readonly=True),
        'total_vt_vr': fields.float('Vt-Vr(m³)', digits=(3, 4), readonly=True),
        'total_vt_v': fields.float('Vt-v(m³)', digits=(3, 4), readonly=True),
        'vr_rendimento': fields.function(calcmadeira_get_totais, method=True,
                                         string='Rendimento', type='float', digits=(3, 2),
                                         store=False, multi='totais'),
        'preco_total': fields.function(calcmadeira_get_totais, method=True,
                                       string='Preco Total (R$/m³)',
                                       digits=(9, 2), type='float', multi='totais'),
        'preco_peca': fields.function(calcmadeira_get_totais, method=True,
                                      string='Preco Pecas (R$/m³)',
                                      digits=(9, 2), type='float', multi='totais'),
        'preco_tora': fields.function(calcmadeira_get_totais, method=True,
                                      string='Preco Toras (R$/m³)',
                                      digits=(9, 2), type='float', multi='totais'),
        'preco_arvore': fields.function(calcmadeira_get_totais, method=True,
                                        string='Preco Arvores (R$/m³)',
                                        digits=(9, 2), type='float', multi='totais'),
        'quantidade_tora': fields.function(calcmadeira_get_totais, method=True, string='Toras',
                                           type='integer', multi='totais'),
        'coeficiente_volume_id': fields.many2one(
            'calcmadeira.coeficiente.equacao', 'Coeficiente volume', ondelete='cascade',
            domain="[('cmacoe_tipo', '=', 'V')]", required=False),
        'coeficiente_forma_id': fields.many2one(
            'calcmadeira.coeficiente.equacao', 'Coeficiente forma', ondelete='cascade',
            domain="[('cmacoe_tipo', '=', 'F')]", required=False),
        'quantidade_tora_total_laminada': fields.integer('Toras totais', readonly=True,
                                                         oldname='quantidade_tora_total'),
        'volume_tora_total': fields.float('Volume tora total', digits=(3, 4), readonly=True),
        'comprimento_laminacao_total': fields.float('Comprimento total laminacao', digits=(3, 2),
                                                    readonly=True),
        'folha_laminacao_total': fields.integer('Número de folhas', readonly=True),
        'fator_forma_medio': fields.float('Fator de forma medio', digits=(3, 4), readonly=True),
        'fator_forma_desvio_padrao': fields.float('Desvio padrao fator forma', digits=(3, 4),
                                                  readonly=True),
        'altura_comercial_calculada': fields.boolean(
            'Calcular altura comercial?', help='Se marcado, o sistema nao vai usar a altura '
                                               'comercial do cadastro e sim a calculada a partir '
                                               'do diametro minimo escolhido.'),
        'altura_comercial_diametro_minimo': fields.float(
            'Diametro minimo de aproveitamento madeireiro (cm)', digits=(4, 2)),
        'calcula_receita': fields.boolean('Calcula receita'),
        'arvore_por_hectare': fields.integer('Numero de arvores por hectare'),
        'itens_hectare_ids': fields.one2many('wizard.calcmadeira.calcula.itens.hectare',
                                             'wcahec_capa_id', 'Itens por hectare'),
        'permite_calculo_receita': fields.function(
            calcmadeira_check_calculo_receita, method=True, string='Permite calculo receita',
            type='boolean'),
        'arquivo_tora': fields.binary('Arquivo'),
        'arquivo_tora_name': fields.char('Arquivo name', size=8),
    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(
            cr, uid, uid).company_id.id,
        'pecas_rolicas': False,
        'pecas_serradas': False,
        'altura_comercial_calculada': False,
        'calcula_receita': True,
        'arvore_por_hectare': 0,
        'arquivo_tora_name': 'Tora.txt',
    }

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(wizard_calcmadeira_calcula, self).default_get(cr, uid, fields_list, context)
        obj_calcmadeira_coeficiente_equacao = self.pool.get('calcmadeira.coeficiente.equacao')
        coeficiente_volume_padrao_ids = obj_calcmadeira_coeficiente_equacao.search(
            cr, uid, [('name', '=', 'Volume padrao')], order='id', context=context)
        coeficiente_volume_padrao_ids and res.update({
            'coeficiente_volume_id': coeficiente_volume_padrao_ids[0]})
        coeficiente_forma_padrao_ids = obj_calcmadeira_coeficiente_equacao.search(
            cr, uid, [('name', '=', 'Forma padrao')], order='id', context=context)
        coeficiente_forma_padrao_ids and res.update({
            'coeficiente_forma_id': coeficiente_forma_padrao_ids[0]})
        context.get('calculo_fator_forma') and res.update({'tipo_funcao': '12'})
        context.get('calculo_laminada_arvore') and res.update({'tipo_funcao': '10'})
        context.get('calculo_laminada_tora') and res.update({'tipo_funcao': '11'})
        res.update({'permite_calculo_receita': True})
        if any([context.get('calculo_rolica'), context.get('calculo_laminada_arvore'),
                context.get('calculo_laminada_tora'), context.get('calculo_fator_forma')]):
            res.update({'permite_calculo_receita': False, 'calcula_receita': False})
        if context.get('calculo_rolica'):
            res.update({'calcula_receita': True})
        return res

    def onchange_calcmadeira_pecas_ids(self, cr, uid, ids, tipo_funcao, vt1, narv, total_vt_vr,
                                       total_vt_v, peca_ids, result_ids, context=None):
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.rolicas')
        res = {'value': {}}
        listpeca = []
        listresult = []
        limpa_dados_tela = []
        vals = {'altura_comercial_calculada': False, 'altura_comercial_diametro_minimo': 0}
        if tipo_funcao == '1':
            pecas_ids = obj_dados_pecas.search(cr, uid, [], context=context)
            obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)
            for y in obj_dados_pecas_dados:
                listpeca.append(y.get('id'))
            vals.update({'pecas_rolicas': listpeca, 'peca_ids': [],
                         'result': listresult, 'result_ids': [],
                         'vt1': limpa_dados_tela, 'narv': 0,
                         'total_vt_vr': limpa_dados_tela,
                         'total_vt_v': limpa_dados_tela})
        else:
            obj_dados_pecas = self.pool.get('calcmadeira.pecas.serradas')
            pecas_ids = obj_dados_pecas.search(cr, uid, [], context=context)
            obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)
            for y in obj_dados_pecas_dados:
                listpeca.append(y.get('id'))
            vals.update({'pecas_serradas': listpeca, 'peca_ids': [],
                         'result': listresult, 'result_ids': [],
                         'vt1': limpa_dados_tela, 'narv': 0,
                         'total_vt_vr': limpa_dados_tela,
                         'total_vt_v': limpa_dados_tela, 'quantidade_tora_total_laminada': 0,
                         'volume_tora_total': 0, 'comprimento_laminacao_total': 0,
                         'fator_forma_medio': 0, 'fator_forma_desvio_padrao': 0,
                         'folha_laminacao_total': 0})

        return {'value': vals}

    def button_print(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_pecas',
        }

    def calcula_peca_rolica(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças roliças
        # limpa os dados da tree
        peca_rolica_id = ids[0] if isinstance(ids, list) else ids
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        vt1 = 0.0000
        vt = 0
        ntora1 = 0
        arv = 0
        di_casc = di_casc2 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.rolicas')

        arvores_ids = obj_dados_arvores.search(cr, uid, [], context=context)
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context=context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        brw_calcmadeira_pecas = brw_calcmadeira_calcula.pecas_rolicas
        pecas_rolicas = []
        for i in brw_calcmadeira_pecas:
            pecas_rolicas.append(i.id)
        esp_id = obj_dados_pecas.search(cr, uid, [('id', 'in', pecas_rolicas)],
                                        order='cmapro_diametro_min', limit=1, context=context)
        obj_dados_espessura = obj_dados_pecas.read(cr, uid, esp_id, context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        # Seleciona dentre todas as peças o menor valor do comprimento
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        # di = 100
        hi_min = 2.2
        hi = hi_min
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append([i.get('cmaarv_diametro'),
                          i.get('cmaarv_altura'),
                          altura_comercial_calculada_dict.get(i.get('id'))
                          if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial')
                          ])
            narv += 1

        falha = 0

        while arv < narv:

            try:
                dap = list1[arv][0]
                ht = list1[arv][1]
                hc = list1[arv][2]
                arv += 1
                # vr = 0
                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         context)
            except:
                falha += 1

            # numero da tora
            ntora = 0

            # calcula seções da árvore. 6 é o menor diametro possível
            while hi_atual + hi_min <= hc:
                di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                       coeficiente_forma=coeficiente_forma,
                                                                       hi_atual=hi_atual,
                                                                       hi=hi,
                                                                       ht=ht,
                                                                       dap=dap,
                                                                       di_casc=di_casc,
                                                                       di_casc2=di_casc2,
                                                                       context=context)
                di = di_casc2 * (1 - prop_casc)
                # menor valor denter todas as peças (diametro)
                for i in obj_dados_espessura:
                    di_min = i.get('cmapro_diametro_min')
                ntora += 1
                if di < di_min or hi_atual + hi_min > hc:
                    ntora = ntora - 1
                    break
                codigopeca, namepeca, voltora, numtora, hi, ntora, preco \
                    = self.seleciona_peca_rolica(cr, uid, ids, dap, ht, hc, hi_atual, di, ntora,
                                                 arv, di_casc, di_casc2,
                                                 coeficiente_forma, prop_casc, context=context)
                hi_atual += hi
                hi = hi_min
                numpeca = 0
                volpeca = 0
                # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora, numpeca, volpeca, preco, context=context)
            ntora1 = ntora1 + ntora
            ntora = 0
            vt1 += vt
            hi_atual = parametro_dados.get('cmapar_altura_corte')

        total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids,
                                                                            vt1, context=context)
        self.write(cr, uid, peca_rolica_id, {'narv': narv,
                                             'vt1': vt1,
                                             'total_vt_vr': total_vt_vr,
                                             'total_vt_v': total_vt_v}, context=context)
        self.calcula_madeira_rolica_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_rolica(self, cr, uid, ids, dap, ht, hc, hi_atual, di, ntora, arv, di_casc,
                              di_casc2, coeficiente_forma, prop_casc, context=None):

        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas = brw_calcmadeira_calcula[0].pecas_rolicas
        pecas_rolicas = []
        for i in brw_calcmadeira_pecas:
            pecas_rolicas.append(i.id)
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.rolicas')
        pecas_ids = obj_dados_pecas.search(cr, uid, [('id', 'in', pecas_rolicas)],
                                           order='cmapro_sequence',
                                           context=context)
        obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)
        voltora = 0
        numtora = 0
        vr_preco = 0
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapro_codigo_peca')
            namepeca = y.get('name')
            di_min = y.get('cmapro_diametro_min') * (1 - prop_casc)
            di_max = y.get('cmapro_diametro_max') * (1 - prop_casc)
            hi = y.get('cmapro_comprimento_tora')
            preco = (y.get('cmapro_preco') or 0)

            # Retirei restrição di < di_max. Estava exato, mas pulando peça.
            # Agora a peça, em último caso, enquadra com di > di_min e di_max.
            # Pode acontecer de ser uma peça bem mais grossa para seu comprimento, mas é melhor do que descartar a madeira.

            if di_min <= di and hi_atual + hi <= hc:

                # Seleciona equação de forma
                di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                       coeficiente_forma=coeficiente_forma,
                                                                       hi_atual=hi_atual,
                                                                       hi=hi,
                                                                       ht=ht,
                                                                       dap=dap,
                                                                       di_casc=di_casc,
                                                                       di_casc2=di_casc2,
                                                                       context=context)

                di = di_casc2 * (1 - prop_casc)

                # Retirei restrição di < di_max. Estava exato, mas pulando peça.
                # Agora a peça, em último caso, enquadra com di > di_min e di_max.
                # Pode acontecer de ser uma peça bem mais grossa para seu comprimento, mas é melhor do que descartar a madeira.

                if di_min <= di and hi_atual + hi <= hc:

                    # volume da tora usando formula de Smalian
                    vr = math.pi / 40000 * (di_casc ** 2 + di_casc2 ** 2) / 2 * hi

                    voltora += vr
                    numtora += 1
                    vr_preco += preco
                    self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, codigopeca, namepeca, vr,
                                                             di_casc, di_casc2, di, ntora, arv,
                                                             0, 0, 0, 0, 0, 0, 0, 0,
                                                             conicidade=0, comprimento=hi,
                                                             # (di_casc - di_casc2) / hi,
                                                             preco=preco, context=context)
                else:
                    p = obj_dados_pecas_dados.index(y)
                    if p < len(obj_dados_pecas_dados) - 1:
                        p = p + 1
                    else:
                        break

                    codigopeca = obj_dados_pecas_dados[p].get('cmapro_codigo_peca')
                    namepeca = obj_dados_pecas_dados[p].get('name')
                    di_min = obj_dados_pecas_dados[p].get('cmapro_diametro_min') * (1 - prop_casc)
                    di_max = obj_dados_pecas_dados[p].get('cmapro_diametro_max') * (1 - prop_casc)
                    hi = obj_dados_pecas_dados[p].get('cmapro_comprimento_tora')
                    preco = obj_dados_pecas_dados[p].get('cmapro_preco') or 0

                    vr_preco += preco

                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi
                    voltora += vr
                    numtora += 1

                    di = di_casc2 * (1 - prop_casc)

                    self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, codigopeca, namepeca, vr,
                                                             di_casc, di_casc2, di, ntora, arv,
                                                             0, 0, 0, 0, 0, 0, 0, 0,
                                                             conicidade=0, comprimento=hi,
                                                             # (di_casc - di_casc2) / hi,
                                                             preco=preco, context=context)

                if vr != 0:
                    break
        return (codigopeca, namepeca, voltora, numtora, hi, ntora, vr_preco)

    def calcula_peca_longitudinal_base(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro
        # de retirada de 1/4 do diâmetro para cada lado (retirada de duas costaneiras)
        # e a costaneira base
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora1 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        arvores_ids = obj_dados_arvores.search(cr, uid, [], context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)

        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        narv = 0
        di = 100
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial')
                 ])
            narv += 1

        falha = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                try:
                    dap = t[0]
                    ht = t[1]
                    hc = t[2]
                    vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                             context)
                except:
                    falha += 1

                arv += 1

                # numero de toras
                ntora = 0

                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    r_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt += vt
                    # angulo inicial para retirada da costaneira da base
                    ang_base = parametro_dados.get('cmapar_ang_base')
                    # comprimento da peça, do eixo x até o limite da costaneira da base (R-flecha)
                    r_flecha_base = di / 2 * math.cos(ang_base * math.pi / 180)
                    # angulo inicial para desconto no di para cada lado (flecha0)
                    ang_d = parametro_dados.get('cmapar_ang_direito')

                    flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

                    if brw_calcmadeira_calcula.tipo_funcao == '15':  # '8' - Radial Longitudinal
                        ang_base = 90
                        r_flecha_base = 0

                    flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))
                    # flecha = di / 2 * (1 - math.cos(ang * math.pi / 180))

                    # largura da peça (corda)
                    if ang_d < 90 - ang_base or ang_base == 0:
                        l = di * math.sin(ang_d * math.pi / 180)
                    else:
                        l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                    if di < l_min_pecas + es_serra:
                        break
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                    cord_d1, cord_d, r_flecha_topo, s = \
                        self.seleciona_peca_longitudinal_base(
                            cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                            r_flecha_base, l, ntora, arv, di_casc, di_casc2,
                            pecas_serradas_preco_dados, parametro_dados, ang_base, flecha_div_r,
                            cord_d1, cord_d, r_flecha_topo, s, context=context)
                    hi_atual = hi_atual + hi

                    if brw_calcmadeira_calcula.tipo_funcao == '15':  # '8' - Radial Longitudinal
                        numpeca *= 2
                        volpeca *= 2
                        preco *= 2

                    # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                    self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                   numtora,
                                                   numpeca, volpeca, preco, context=context)
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {'narv': narv, 'vt1': vt1,
                                                         'total_vt_vr': total_vt_vr,
                                                         'total_vt_v': total_vt_v}, context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_longitudinal_base(self, cr, uid, ids, l_min_pecas, es_min_pecas, es_serra,
                                         hi, vr, di, flecha0, r_flecha_base, l, ntora, arv, di_casc,
                                         di_casc2, pecas_serradas_preco_dados, parametro_dados,
                                         ang_base, flecha_div_r, cord_d1, cord_d,
                                         r_flecha_topo, s, context=None):
        brw_calcmadeira_calcula = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                              context=context)
        escolha = brw_calcmadeira_calcula.tipo_funcao
        brw_calcmadeira_pecas = brw_calcmadeira_calcula.pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)

        flecha = flecha0
        es = es_serra

        # espessura residual(eixo horizontal)
        if escolha in ('13', '14'):  # '7' - Gira 180 90
            es_sobra = di / 2 * flecha_div_r - flecha0
        else:
            es_sobra = di - 2 * flecha0

        preco1 = 0
        preco2 = 0
        preco_menor2_14D = 0

        vr_tree = 0
        # espessura residual (eixo vertical)
        l_sobra = l
        es_aux = 0
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_pecas.search(cr, uid, [('id', 'in', pecas_serradas)],
                                           order='cmapse_sequence',
                                           context=context)
        obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)

        selec = 0

        vp1 = 0.0000

        # '-----------------------------------------------
        ang_e, ang_d, cord_e, cord_d = formulas_costaneirabase(
            flecha, es, es_serra, di, ang_base, r_flecha_base, escolha, parametro_dados)
        # '-----------------------------------------------

        if flecha <= di / 2 or escolha in ('13', '14'):  # '7' - Gira 180 90
            l_sobra = cord_d
        else:
            l_sobra = cord_e

        cord_e1 = 0

        while selec == 0 and es_sobra >= 0:

            for y in obj_dados_pecas_dados:
                codigopeca = y.get('cmapse_codigo_peca')
                namepeca = y.get('name')
                min_es = int(y.get('cmapse_menor_espessura'))
                max_es = int(y.get('cmapse_maior_espessura'))
                l_min = int(y.get('cmapse_menor_largura'))
                l_max = int(y.get('cmapse_maior_largura'))
                voltora = 0
                numtora = 0
                numpeca = 0
                volpeca = 0
                vr_preco = 0
                n = 0
                v = 0.0000
                n_aux = 0
                v_aux = 0


                selec = 0
                # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
                while (l_sobra - es_serra) >= l_min and (es_sobra - es_serra) >= min_es:
                    if (l_sobra - es_serra) >= l_min or (es_sobra - es_serra) >= min_es:

                        # min_es não pode ser menor do que es+es_serra! não precisa no VBA, Exit
                        # Do no 1o laço
                        if selec == 1 and es < min_es + es_serra:
                            break

                        selec = 1

                        # desdobra da maior espessura da peça para a menor
                        for inv_es in range(0,(max_es-min_es)+1, 5):
                            # sobra de espessura não pode ser menor do que flecha0 + menor espessura
                            # das peças (1cm) + es_serra
                            if (es_sobra - es_serra) < min_es:
                                break
                            es = max_es - inv_es
                            # Considera espessura da serra (fita, corrente motosserra...)
                            es = es + es_serra
                            # Testa es
                            es = testa_espessura(es, es_sobra, es_serra, min_es)

                            # min_es não pode ser menor do que es+es_serra!
                            if es < min_es + es_serra:
                                break

                            ang_e, ang_d, cord_e, cord_d = formulas_costaneirabase(
                                flecha, es, es_serra, di, ang_base, r_flecha_base, escolha,
                                parametro_dados)

                            if flecha <= di / 2 or escolha in ('13', '14'):  # '7' - Gira 180 90
                                l_sobra = cord_d
                            else:
                                l_sobra = cord_e

                            n_h = math.trunc(es_sobra / es)

                            # Se l permite divisao no mínimo em 2 grupos de peças selecionadas (p) em l
                            if l_sobra / (l_min + es_serra) >= 2:
                                # entra função peças divididas na vertical
                                l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, \
                                cord_e, cord_d, r_flecha_base, dados_impressao, preco2, cord_e1 = \
                                    pecamenor_long_1costaneirabase(
                                        l_min, l_max, codigopeca, di, hi, es, es_serra, l_sobra, n_h,
                                        flecha0, flecha, ang_e, ang_d, cord_e, cord_d, r_flecha_base,
                                        pecas_serradas_preco_dados, parametro_dados,
                                        flecha_div_r, ang_base, escolha)

                                for imp in dados_impressao:
                                    self.calcmadeira_atualiza_tree_resultado(
                                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                                        di, ntora, arv, es_serra, imp.get('l_div'), imp.get('n_div'),
                                        imp.get('v'), 0, imp.get('l_sobra'), es_sobra - n_h * es,
                                        es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                        rendimento=vr and (vp1 + v1) / vr or 0,
                                        preco=imp.get('preco'), context=context)

                                n = 0
                                v = 0.0000
                                es_aux = es
                            else:
                                n = n1 = 0
                                v = v1 = 0.0000

                                # sobra da largura para tentar outra classificação em
                                if l_sobra - es_serra < l_max:
                                    l_div = l_max + es_serra
                                    # ---------------------------
                                    # Testa Largura
                                    l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
                                    # ---------------------------
                                    l = l_div
                                else:
                                    l = l_max + es_serra

                                l_sobra = l_sobra - l

                                # volume das peças
                                for x in range(1, n_h + 1):
                                    vp = ((es - es_serra) / 1000.0) * (l - es_serra)/ 1000.0 * hi

                                    es_tmp = es
                                    l_tmp = l
                                    n_tmp = 1
                                    v_tmp = vp

                                    preco_peca = calc_preco(pecas_serradas_preco_dados, codigopeca,
                                                            hi, v_tmp)

                                    preco1 = preco1 + preco_peca

                                    # preco = 0

                                    self.calcmadeira_atualiza_tree_resultado(
                                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                                        di, ntora,
                                        arv, es_serra, l_tmp, 1, vp, 0, l_sobra,
                                        es_sobra - n_h * es, es - es_serra,
                                        conicidade=(di_casc - di_casc2) / hi,
                                        rendimento=vr and (vp1 + v1) / vr or 0, preco=preco_peca,
                                        context=context)

                                    n_aux += n
                                    v_aux += v
                                    n += 1
                                    vp1 += vp
                                    vp = 0

                                    if l_sobra >= l_min + es_serra:
                                        l_div, l_sobra, n_div, n1, v1, impressao, preco_menor2 \
                                            = pecamenor2_14dcostan(n1, v1, l_min, l_max, codigopeca,
                                                                   hi, es, es_serra, l_sobra,
                                                                   pecas_serradas_preco_dados)

                                        self.calcmadeira_atualiza_tree_resultado(
                                            cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                            di_casc2,
                                            di, ntora, arv, es_serra, impressao.get('l_div'),
                                            impressao.get('n_div'), impressao.get('v'), 0,
                                            impressao.get('l_sobra'), es_sobra - n_h * es,
                                            es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                            rendimento=vr and (vp1 + v1) / vr or 0,
                                            preco=preco_menor2, context=context)


                                        v = 0.0000

                                        preco_menor2_14D = preco_menor2_14D + preco_menor2

                                    if x <= n_h:
                                        flecha = flecha + es

                                        if x == n_h:
                                            es1 = es
                                            es = es_serra

                                        if escolha in ('13', '14'):  # '7' - Gira 180 90
                                            cord_e1 = cord_e

                                        # '-----------------------------------------------
                                        ang_e, ang_d, cord_e, cord_d = formulas_costaneirabase(
                                            flecha, es, es_serra, di, ang_base, r_flecha_base,
                                            escolha, parametro_dados)
                                        # '-----------------------------------------------

                                        if flecha <= di / 2 or escolha in ('13', '14'):  # '7' - Gira 180 90
                                            l = math.trunc(cord_d / 10) * 10
                                        else:
                                            l = math.trunc(cord_e / 10) * 10

                                        if l > l_max + es_serra:
                                            l = l_max + es_serra

                                        if flecha <= di/2 or escolha in ('13', '14'):  # '7' - Gira 180 90
                                            l_sobra = cord_d - l
                                        else:
                                            l_sobra = cord_e - l

                                es_aux = es

                                es = es1


                            preco = preco1 + preco2 + preco_menor2_14D
                            vr_preco += preco

                            preco1 = preco2 = preco_menor2_14D = 0

                            # espessura restante (eixo horizontal) sobra da peça para novo
                            # aproveitamento (cm)
                            es_sobra = es_sobra - n_h * es

                            v = vp1

                            numpeca += n + n1
                            volpeca += v + v1
                            if vr > 0:
                                numtora += 1
                                voltora += vr
                                vr_tree = vr
                                vr = 0


                            # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                            if brw_calcmadeira_calcula.tipo_funcao in ('15', '16'):
                                numpeca *= 2
                                volpeca *= 2
                                vr_preco *= 2
                            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca,
                                                           namepeca, voltora, numtora,
                                                           numpeca, volpeca, vr_preco,
                                                           context=context)

                            numpeca = 0
                            numtora = 0
                            voltora = 0
                            volpeca = 0
                            vr_preco = 0

                            vp1 = 0.0000

            if selec == 0:
                flecha = flecha + 0.5
                es = es_min_pecas

                if flecha + es < di - flecha0:
                    # '-----------------------------------------------
                    ang_e, ang_d, cord_e, cord_d = \
                        formulas_costaneirabase(flecha, es, es_serra, di, ang_base, r_flecha_base,
                                                escolha, parametro_dados)
                    # '-----------------------------------------------
                    if flecha <= di / 2 or escolha in ('13', '14'):
                        l_sobra = cord_d
                    else:
                        l_sobra = cord_e

                    if escolha in ('13', '14'):
                        es_sobra = di / 2 * flecha_div_r - flecha
                    else:
                        es_sobra = di - flecha - flecha0
                else:
                    es_sobra = 0

            if es_sobra == 0:
                break

            if escolha in ('13', '14') and \
                    (selec == 1 or es_sobra < es_min_pecas or flecha / (di / 2) >= flecha_div_r):
                break

        if escolha in ('13', '14'):  # '7' - Gira 180 90
            if s == 0:
                # comprimento da peça, do eixo x até o limite de corte do topo (R-flecha)
                r_flecha_topo = di / 2 * math.cos(ang_d * math.pi / 180)

                cord_d1 = cord_e1
            else:
                r_flecha_base2 = di / 2 * math.cos(ang_d * math.pi / 180)

                cord_d2 = cord_e1

                l = r_flecha_topo + r_flecha_base2

                if cord_d2 >= cord_d1:
                    cord_d = cord_d1
                else:
                    cord_d = cord_d2

        return (codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, vr_preco,
                l, cord_d1, cord_d, r_flecha_topo, s)

    def calcula_peca_longitudinal(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro
        # de retirada de 1/4 do diâmetro para cada lado (retirada de duas costaneiras)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        arvores_ids = obj_dados_arvores.search(cr, uid, [], context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial')
                 ])
            narv += 1

        falha = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                try:
                    dap = t[0]
                    ht = t[1]
                    hc = t[2]
                    arv += 1
                    vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                             context)
                except:
                    falha += 1

                # numero da tora
                ntora = 0
                v = 0
                n = 0
                vr = 0
                es_sobra = 100
                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    r_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt = vrt + vr
                    # angulo inicial para desconto do di para cada lado (flecha0)
                    ang_d = parametro_dados.get('cmapar_ang_direito')
                    ang_base = 0
                    flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                    # largura da peça (corda)
                    if ang_d < 90 - ang_base or ang_base == 0:
                        l = di * math.sin(ang_d * math.pi / 180)
                    else:
                        l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                    if di < l_min_pecas + es_serra:
                        break

                    l_sobra = l
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l,\
                    cord_d1, cord_d, r_flecha_topo, s = \
                        self.seleciona_peca_longitudinal_base(
                        cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                        r_flecha_base, l, ntora, arv, di_casc, di_casc2, pecas_serradas_preco_dados,
                        parametro_dados, ang_base, flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
                        context=context)

                    hi_atual = hi_atual + hi
                    # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                    self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                   numtora,
                                                   numpeca, volpeca, preco, context=context)
                # atualiza hi_atual com o comprimento da peça extraída (hi)
                # hi_atual = hi_atual + hi
                # 'acumula numero de toras da arvore (ntora) para resultado total de
                # toras na amostra
                ntora1 = ntora1 + ntora
                ntora = 0
                # acumula vt arvore e zera vt
                vt1 = vt1 + vt
                vt = 0
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {'narv': narv, 'vt1': vt1,
                                                         'total_vt_vr': total_vt_vr,
                                                         'total_vt_v': total_vt_v}, context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def calcula_peca_quadrado_circunscrito(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro
        # do quadrado circunscrito
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')

        arvores_ids = obj_dados_arvores.search(cr, uid, [], context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial')
                 ])
            narv += 1

        falha = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                try:
                    dap = t[0]
                    ht = t[1]
                    hc = t[2]
                    # Seleciona equação de volume
                    vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                             context)
                except:
                    falha += 1

                arv = arv + 1
                # numero da tora
                ntora = 0
                es_sobra = 100

                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    R_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)
                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi
                    di = di_casc2 * (1 - prop_casc) * 10
                    ntora = ntora + 1
                    vrt = vrt + vr
                    # largura da peça
                    l = di / math.sqrt(2)
                    if l < l_min_pecas + es_serra:
                        ntora = ntora - 1
                        break
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                        = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids,
                                                                    l_min_pecas, es_min_pecas,
                                                                    es_serra, hi,
                                                                    vr,
                                                                    l, ntora, arv, di,
                                                                    di_casc, di_casc2,
                                                                    pecas_serradas_preco_dados,
                                                                    cord_d, es_medula, flecha0,
                                                                    context=context)
                    hi_atual = hi_atual + hi
                # atualiza hi_atual com o comprimento da tora calculada (hi)
                # hi_atual = hi_atual + hi
                # acumula ntora da arvore e zera ntora
                ntora1 = ntora1 + ntora
                ntora = 0
                # acumula vt da árvore e zera vt
                vt1 += vt
                vt = 0
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {'narv': narv, 'vt1': vt1,
                                                         'total_vt_vr': total_vt_vr,
                                                         'total_vt_v': total_vt_v}, context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_quadrado_circunscrito(self, cr, uid, ids, l_min_pecas, es_min_pecas,
                                             es_serra, hi, vr,
                                             l, ntora, arv, di, di_casc, di_casc2,
                                             pecas_serradas_preco_dados, cord_d, es_medula, flecha0,
                                             context=None):
        brw_calcmadeira_calcula = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                              context=context)
        escolha = brw_calcmadeira_calcula.tipo_funcao
        brw_calcmadeira_pecas = brw_calcmadeira_calcula.pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)
        # espessura residual (eixo horizontal do bloco)
        if escolha in ('13', '14'):  # '7' - Gira 180 90
            es_sobra = cord_d
        else:
            es_sobra = l

        cont_medula = 0
        v_medula = 0

        es = es_serra
        selec = 0
        # espessura residual (eixo vertical do bloco)
        l_sobra = l
        l1 = l
        # l2 = 0

        preco1 = 0
        preco2 = 0

        l_sobra_aux = 0
        es = 0
        vr_tree = 0
        obj_dados_peca = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_peca.search(cr, uid, [('id', 'in', pecas_serradas)],
                                          order='cmapse_sequence',
                                          context=context)
        obj_dados_pecas_dados = obj_dados_peca.read(cr, uid, pecas_ids, context=context)
        numtora = 0
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapse_codigo_peca')
            namepeca = y.get('name')
            min_es = y.get('cmapse_menor_espessura')
            max_es = y.get('cmapse_maior_espessura')
            l_min = y.get('cmapse_menor_largura')
            l_max = y.get('cmapse_maior_largura')
            voltora = 0
            numtora = 0
            numpeca = 0
            volpeca = 0
            vr_preco = 0
            n = 0
            v = 0

            selec = 0
            # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
            while (l_sobra - es_serra) >= l_min and (es_sobra - es_serra) >= min_es:

                # min_es não pode ser menor do que es+es_serra!
                if selec == 1 and es < min_es + es_serra:
                    break

                selec = 1

                # desdobra da maior espessura da peça para a menor
                for inv_es in range(0, ((max_es-min_es)+1), 5):
                    # sobra de espessura não pode ser menor do que flecha0 + menor
                    # espessura das peças (1cm) + es_serra
                    if (l_sobra - es_serra) < l_min or (es_sobra - es_serra) < min_es:
                        break
                    es = max_es - inv_es
                    # Considera espessura da serra (fita, corrente motosserra...)
                    es = es + es_serra
                    es = testa_espessura(es, es_sobra, es_serra, min_es)
                    # min_es não pode ser menor do que es+es_serra!
                    if (es_sobra - es_serra) < min_es or es < min_es + es_serra:
                        break

                    if es_sobra > es + es_medula and es_medula > es_serra and \
                            cont_medula == 0:
                        n_h = math.trunc((es_sobra - es_medula) / es)
                    else:
                        n_h = math.trunc(es_sobra / es)

                    # Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
                    if l_sobra / (l_min + es_serra) >= 2:
                        # entra função peças divididas na vertical
                        l_sobra, n1, v1, dados_impressao, preco2 \
                            = pecamenor_quadcircun(l_min, l_max, codigopeca, hi, es, es_serra,
                                                   es_sobra, l,
                                                   l_sobra, n_h, pecas_serradas_preco_dados,
                                                   es_medula, escolha, cont_medula)

                        for imp in dados_impressao:
                            self.calcmadeira_atualiza_tree_resultado(
                                cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                                di, ntora, arv, es_serra, imp.get('l_div'), imp.get('n_div'),
                                imp.get('v'), 0, imp.get('l_sobra'), es_sobra - n_h * es,
                                es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                rendimento=vr and (imp.get('v') + v1) / vr or 0,
                                preco=imp.get('preco'), context=context)

                        v = 0.0000
                        n = 0
                    else:
                        n = n_h
                        n1 = 0
                        v1 = 0.0000
                        # sobra da largura para tentar outra classificação em PecaMenor
                        if l_sobra - es_serra < l_max:
                            l_div = l_max + es_serra
                            # ---------------------------
                            # Testa Largura
                            l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
                            # ---------------------------
                            l = l_div
                        else:
                            l = l_max + es_serra

                        l_sobra = l_sobra - l

                        # volume de peças sem divisão vertical
                        v = (n_h * (es - es_serra) / 1000.0) * (l - es_serra) / 1000.0 * hi

                        # Relatorio por peça
                        print('codpeca:{}, es:{}, l:{}, n:{}, v:{}, l_sobra:{}, ntora:{}, '
                              'arv:{}'.format(codigopeca, es, l, n, v, l_sobra, ntora, arv))

                        es_tmp = es
                        l_tmp = l
                        n_tmp = n
                        v_tmp = v
                        # -----------------------------------------------------------------
                        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                        # referencia
                        preco_peca = calc_preco(pecas_serradas_preco_dados, codigopeca, hi, v_tmp)
                        # -----------------------------------------------------------------

                        preco1 = preco_peca

                        if es_sobra > es + es_medula and es_medula > es_serra \
                                and cont_medula == 0:
                            v_medula = ((es_medula - es_serra) / 1000) * (l - es_serra) / 1000 * hi
                            n_div_medula = 1
                            v = v + v_medula
                            n = n + n_div_medula

                            es_tmp = es_medula
                            l_tmp = l_div
                            n_tmp = n_div_medula
                            v_tmp = v_medula
                            # -----------------------------------------------------------------
                            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                            # referencia
                            preco_peca = calc_preco(pecas_serradas_preco_dados, codigopeca, hi,
                                                    v_tmp)
                            # -----------------------------------------------------------------

                            preco1 = preco1 + preco_peca

                            # Relatorio por peça
                            # print('peca:{}, es:{}, l:{}, n:{}, v:{}, l_sobra:{}, ntora:{}, arv:{}, '
                            #       'preco_peca:{}, preco:{}'.format(codpeca, es_medula, l_div, 1, v_medula,
                            #                                        l_sobra, ntora, arv, preco_peca,
                            #                                        preco1))

                        self.calcmadeira_atualiza_tree_resultado(
                            cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                            di, ntora, arv, es_serra, l_div, n, v, 0, l_sobra, es_sobra - n_h * es,
                            es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                            rendimento=vr and (v + v1) / vr or 0, preco=preco_peca,
                            context=context)

                        if l_sobra >= l_min + es_serra:
                            # entra função peças divididas na vertical
                            l_sobra, n1, v1, dados_impressao, preco2 \
                                = pecamenor_quadcircun(l_min, l_max, codigopeca, hi, es,
                                                       es_serra,
                                                       es_sobra, l, l_sobra, n_h,
                                                       pecas_serradas_preco_dados,
                                                       es_medula, escolha, cont_medula)

                            for imp in dados_impressao:
                                self.calcmadeira_atualiza_tree_resultado(
                                    cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                    di_casc2,
                                    di, ntora, arv, es_serra, imp.get('l_div'),
                                    imp.get('n_div'),
                                    imp.get('v'), 0, imp.get('l_sobra'),
                                    es_sobra - n_h * es,
                                    es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                    rendimento=vr and (imp.get('v') + v1) / vr or 0,
                                    preco=imp.get('preco'),
                                    context=context)

                            l_sobra = l
                            n = 0
                            v = 0

                        else:
                            break

                # min_es não pode ser menor do que es+es_serra!
                if (es_sobra - es_serra) < min_es or es < min_es + es_serra:
                    break

                preco = preco1 + preco2
                preco1 = preco2 = 0

                # espessura restante: eixo horizontal do bloco
                if es_sobra > es + es_medula and es_medula > es_serra \
                        and cont_medula == 0:
                    es_sobra = es_sobra - n_h * es - es_medula
                else:
                    es_sobra = es_sobra - n_h * es

                # if l_sobra >= l_min_pecas + es_serra:
                #     l2 = l1
                #     l1 = l_sobra
                #
                # if l2 > 0:
                #     l1 = l2
                #
                # if es_sobra >= es_min_pecas + es_serra:
                #     l_sobra = l1

                numpeca += n + n1
                volpeca += v + v1
                vr_preco += preco
                if vr > 0:
                    numtora += 1
                    voltora += vr
                    vr_tree = vr
                    vr = 0

            l_sobra = l1

            if selec == 1:
                cont_medula = 1

            # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                           numtora,
                                           numpeca, volpeca, vr_preco, context=context)

        return (codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, vr_preco)

    def calcula_peca_desdobro_radial(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro radial
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        di_casc = di_casc2 = 0
        nes1 = nes2 = es1 = es2 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')

        arvores_ids = obj_dados_arvores.search(cr, uid, [], context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_precos_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_precos_ids, [], context)
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # Executa por árvore
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial')
                 ])
            narv += 1

        falha = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                try:
                    dap = t[0]
                    ht = t[1]
                    hc = t[2]
                    # Seleciona equação de volume
                    vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                             context)
                except:
                    falha += 1

                arv = arv + 1
                # numero da tora
                ntora = 0
                v = 0
                n = 0
                vr = 0

                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    R_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt = vrt + vr
                    l = di / 2
                    if l < l_min_pecas + es_serra or di/2 < es_min_pecas + es_serra:
                        break
                    l_e = l_d = l
                    l_i = l_s = l
                    l_sobra = 0.0000
                    nes1 = nes2 = 0.0
                    es1 = es2 = 0.0
                    ang_d = 90.0000
                    ang_s = 0.0000
                    ang_e = ang_i = 0
                    fatia = 0
                    n1 = v1 = 0

                    # Verifica se pelo menos a menor peça será selecionada
                    es = es_min_pecas + es_serra
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------
                    es = 0

                    while l_d >= l_min_pecas + es_serra or l_s >= l_min_pecas + es_serra:
                        if fatia == 0:
                            codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                            fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, \
                            nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                                = self.seleciona_peca_desdobro_radial_f0(cr, uid, ids,
                                                                         l_min_pecas, es_min_pecas,
                                                                         ang_e, ang_d, ang_i, ang_s,
                                                                         vr, es_serra, es, es1, es2,
                                                                         nes1,
                                                                         nes2, di, hi, l_e, l_d,
                                                                         l_i, l_s,
                                                                         l_sobra, ntora, arv,
                                                                         di_casc, di_casc2,
                                                                         pecas_serradas_preco_dados,
                                                                         fatia, context)
                            fatia = 1
                            # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                            # atualiza-lo
                            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                           voltora, numtora,
                                                           numpeca * 4, volpeca * 4, preco * 4,
                                                           context=context)
                        if fatia == 1:
                            codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                            fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, \
                            nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                                = self.seleciona_peca_desdobro_radial_f1(cr, uid, ids,
                                                                         l_min_pecas, es_min_pecas,
                                                                         ang_e, ang_d, ang_i, ang_s,
                                                                         vr, es_serra, es, es1, es2,
                                                                         nes1, nes2, di, hi, l_e,
                                                                         l_d, l_i, l_s,
                                                                         l_sobra, ntora, arv,
                                                                         di_casc, di_casc2,
                                                                         pecas_serradas_preco_dados,
                                                                         fatia, context)
                            fatia = 0
                            # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                            # atualiza-lo
                            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                           voltora, numtora,
                                                           numpeca * 4, volpeca * 4, preco * 4,
                                                           context=context)
                            if l_d < es_min_pecas + es_serra and l_s < es_min_pecas + es_serra:
                                break
                    # atualiza hi_atual com o comprimento da peça extraída (hi)
                    hi_atual = hi_atual + hi
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')
                # acumula vt arvore e zera vt
                vt1 += vt
                vt = 0
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                ntora = 0
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {'narv': narv, 'vt1': vt1,
                                                         'total_vt_vr': total_vt_vr,
                                                         'total_vt_v': total_vt_v}, context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_desdobro_radial_f0(self, cr, uid, ids, l_min_pecas, es_min_pecas, ang_e,
                                          ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
                                          nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, arv,
                                          di_casc, di_casc2, pecas_serradas_preco_dados,
                                          fatia, context=None):
        es = 0
        selec = 0
        l_sobra = l_d
        es_dif = 0
        p = 0

        preco = 0

        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas = brw_calcmadeira_calcula[0].pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)
        es_sobra = 0
        vr_tree = 0
        numtora = 0
        obj_dados_peca = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_peca.search(cr, uid, [('id', 'in', pecas_serradas)],
                                          order='cmapse_sequence',
                                          context=context)
        npecas = len(pecas_ids)
        obj_dados_pecas_dados = obj_dados_peca.read(cr, uid, pecas_ids, context=context)
        v_aux = 0
        n_aux = 0
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapse_codigo_peca')
            namepeca = y.get('name')
            min_es = y.get('cmapse_menor_espessura')
            max_es = y.get('cmapse_maior_espessura')
            l_min = y.get('cmapse_menor_largura')
            l_max = y.get('cmapse_maior_largura')
            voltora = 0
            numtora = 0
            numpeca = 0
            volpeca = 0
            vr_preco = 0
            p += 1

            if ang_d >= 45 and l_d >= l_min + es_serra and \
                    (nes1 + min_es) <= float(di / 2 * math.cos(45 * math.pi / 180)):

                # Considera espessura da serra (fita, corrente motosserra...)
                es = max_es + es_serra

                # retorna valores quando l_d não atende largura mínima
                while (nes1 + es - es_serra) >= float(di / 2 * math.cos(45 * math.pi / 180)):

                    es = es - 5

                selec = 1

                while l_d < l_min + es_serra:
                    es = es - 5

                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    if es < min_es + es_serra:
                        es = 0
                        selec = 0
                        break

                if es < min_es + es_serra:
                    continue

                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                               l_e, l_d, l_i, l_s)
                # ---------------------------------------------
                if l_d < l_min + es_serra:
                    selec = 0

                if selec == 0:
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    while l_d < l_min + es_serra:
                        es = es - 5
                        # ---------------------------------------------
                        ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                            = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i,
                                       ang_s, l_e, l_d, l_i, l_s)
                        # ---------------------------------------------

                        if es < min_es + es_serra:
                            es = 0
                            selec = 0
                            break
                    if es < min_es + es_serra:
                        continue
                # if es < min_es + es_serra:
                #     continue

                if l_d / (l_min + es_serra) >= 2:
                    l_sobra = l_d
                    # entra função peças divididas na vertical
                    fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                        = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es, es_serra,
                                          l_sobra, pecas_serradas_preco_dados)

                    for imp in dados_impressao:
                        self.calcmadeira_atualiza_tree_resultado(
                            cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                            di_casc2,
                            di, ntora, arv, es_serra, imp.get('l_div'),
                            imp.get('n_div'),
                            imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                            es_sobra, es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                            rendimento=vr and (imp.get('v') + v1) / vr or 0,
                            preco=imp.get('preco'), context=context)

                    n = 0
                    v = 0.0000
                    vr_preco += preco
                else:
                    n1 = 0
                    v1 = 0.0000
                    l_sobra = l_d
                    # l_div
                    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
                    l_sobra = l_sobra - l_div
                    # volume da peça
                    # v = ((es - es_serra) / 100) * l_min / 100 * hi
                    v=((es-es_serra)/1000.0)*(l_div-es_serra)/1000.0*hi
                    n = 1

                    es_tmp = es
                    l_tmp = l_div
                    n_tmp = n
                    v_tmp = v
                    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                    # referencia
                    preco_peca = calc_preco(pecas_serradas_preco_dados, codigopeca,
                                            hi, v_tmp)
                    vr_preco += preco_peca

                    self.calcmadeira_atualiza_tree_resultado(
                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                        di, ntora,
                        arv, es_serra, l_div, n, v, fatia, l_sobra,
                        es_sobra, es - es_serra,
                        conicidade=(di_casc - di_casc2) / hi, rendimento=vr and (v + v1) / vr or 0,
                        preco=preco_peca, context=context)

                    v_aux = v
                    n_aux = n

                    if l_sobra >= l_min + es_serra:
                        # entra função peças divididas na vertical
                        fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                            = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es,
                                              es_serra, l_sobra, pecas_serradas_preco_dados)

                        for imp in dados_impressao:
                            self.calcmadeira_atualiza_tree_resultado(
                                cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                di_casc2,
                                di, ntora, arv, es_serra, imp.get('l_div'),
                                imp.get('n_div'),
                                imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                                es_sobra, es - es_serra,
                                conicidade=(di_casc - di_casc2) / hi,
                                rendimento=vr and (imp.get('v') + v1) / vr or 0,
                                preco=imp.get('preco'), context=context)

                        n = 0
                        v = 0.0000
                        vr_preco += preco
                        preco = 0
                # define espessura acumulada na Fatia0(das peças selecionadas)
                numpeca += n + n1
                volpeca += v + v1
                if vr > 0:
                    numtora += 1
                    voltora += vr
                    vr_tree = vr
                    vr = 0

                selec = 1

                # zera variáveis da seção da árvore
                n = 0
                n1 = 0
                v = 0
                v1 = 0
                # es = 0

                preco = 0

            if l_sobra < l_min_pecas + es_serra:
                break

            # Aproveitamento do l_sobra com outro tipo de peça no mesmo es
            elif p < npecas and l_sobra >= l_min_pecas + es_serra and es >= es_min_pecas + es_serra:
                continue

        # define espessura acumulada na Fatia0(das peças selecionadas)

        if es > 0 and selec == 1:
            nes1 = nes1 + es
            es1 = es
            l_s = l_s - es1
        else:
            l_s = 0

        es_sobra = l_s

        if context.get('funcoes_serrada_tora'):
            numpeca *= 4
            volpeca *= 4
            vr_preco *= 4

        return (codigopeca, namepeca, voltora, numtora, numpeca, volpeca,
                fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
                nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, vr_preco)

    def seleciona_peca_desdobro_radial_f1(self, cr, uid, ids, l_min_pecas, es_min_pecas, ang_e,
                                          ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
                                          nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, arv,
                                          di_casc, di_casc2, pecas_serradas_preco_dados,
                                          fatia, context=None):

        selec = 0
        es = 0
        l_sobra = l_s

        preco = 0

        p = 0
        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas = brw_calcmadeira_calcula[0].pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)
        es_sobra = 0
        vr_tree = 0
        numtora = 0
        n_aux = 0
        v_aux = 0
        obj_dados_peca = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_peca.search(cr, uid, [('id', 'in', pecas_serradas)],
                                          order='cmapse_sequence',
                                          context=context)
        npecas = len(pecas_ids)
        obj_dados_pecas_dados = obj_dados_peca.read(cr, uid, pecas_ids, context=context)
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapse_codigo_peca')
            namepeca = y.get('name')
            min_es = y.get('cmapse_menor_espessura')
            max_es = y.get('cmapse_maior_espessura')
            l_min = y.get('cmapse_menor_largura')
            l_max = y.get('cmapse_maior_largura')
            voltora = 0
            numtora = 0
            numpeca = 0
            volpeca = 0
            vr_preco = 0
            p += 1

            if ang_s <= 45 and l_s >= l_min + es_serra and \
                    (nes2 + min_es) <= float(di / 2 * math.cos(45 * math.pi / 180)):

                # Considera espessura da serra (fita, corrente motosserra...)
                es = max_es + es_serra

                # retorna valores quando l_d não atende largura mínima
                while (nes2 + es - es_serra) >= float(di / 2 * math.cos(45 * math.pi / 180)):
                    es = es - 5

                selec = 1

                while l_s < l_min + es_serra:
                    es = es - 5

                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    if es < min_es + es_serra:
                        es = 0
                        selec = 0
                        break

                if es < min_es + es_serra:
                    continue

                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                               l_e, l_d, l_i, l_s)
                # ---------------------------------------------
                if l_s < l_min + es_serra:
                    selec = 0

                if selec == 0:
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    while l_s < l_min + es_serra:
                        es = es - 5
                        # ---------------------------------------------
                        ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                            = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i,
                                       ang_s, l_e, l_d, l_i, l_s)
                        # ---------------------------------------------

                        if es < min_es + es_serra:
                            es = 0
                            selec = 0
                            break
                    if es < min_es + es_serra:
                        continue
                # if es < min_es + es_serra:
                #     continue

                # Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
                if l_s / (l_min + es_serra) >= 2:
                    l_sobra = l_s
                    # entra função peças divididas na vertical
                    fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                        = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es, es_serra,
                                          l_sobra, pecas_serradas_preco_dados)

                    for imp in dados_impressao:
                        self.calcmadeira_atualiza_tree_resultado(
                            cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                            di_casc2,
                            di, ntora, arv, es_serra, imp.get('l_div'),
                            imp.get('n_div'),
                            imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                            es_sobra, es - es_serra,
                            conicidade=(di_casc - di_casc2) / hi,
                            rendimento=vr and (imp.get('v') + v1) / vr or 0,
                            preco=imp.get('preco'), context=context)

                    n = 0
                    v = 0.0000
                    vr_preco += preco
                else:
                    n1 = 0
                    v1 = 0.0000
                    l_sobra = l_s
                    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
                    l_sobra = l_sobra - l_div
                    # volume da peça
                    # v = ((es - es_serra) / 100) * l_min / 100 * hi
                    v = ((es - es_serra) / 1000.0) * (l_div-es_serra)/ 1000.0 * hi
                    n = 1

                    es_tmp = es
                    l_tmp = l_div
                    n_tmp = n
                    v_tmp = v
                    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                    # referencia
                    preco_peca = calc_preco(pecas_serradas_preco_dados, codigopeca,
                                            hi, v_tmp)
                    vr_preco += preco_peca

                    self.calcmadeira_atualiza_tree_resultado(
                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                        di, ntora,
                        arv, es_serra, l_div, n, v, fatia, l_sobra,
                        es_sobra, es - es_serra,
                        conicidade=(di_casc - di_casc2) / hi, rendimento=vr and (v + v1) / vr or 0,
                        preco=preco_peca, context=context)

                    v_aux = v
                    n_aux = n
                    if l_sobra >= l_min + es_serra:
                        # entra função peças divididas na vertical
                        fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                            = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es, es_serra,
                                              l_sobra, pecas_serradas_preco_dados)

                        for imp in dados_impressao:
                            self.calcmadeira_atualiza_tree_resultado(
                                cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                di_casc2,
                                di, ntora, arv, es_serra, imp.get('l_div'),
                                imp.get('n_div'),
                                imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                                es_sobra, es - es_serra,
                                conicidade=(di_casc - di_casc2) / hi,
                                rendimento=vr and (imp.get('v') + v1) / vr or 0,
                                preco=imp.get('preco'), context=context)

                        n = 0
                        v = 0.0000
                        vr_preco += preco
                        preco1 = 0
                # aviso de troca de fatia (selecionou peça)
                # peca = 1
                # define espessura acumulada na Fatia1(das peças selecionadas)
                numpeca += n + n1
                volpeca += v + v1
                if vr > 0:
                    numtora += 1
                    voltora += vr
                    vr_tree = vr
                    vr = 0

                selec = 1

                # zera variáveis da seção da árvore
                n = 0
                n1 = 0
                v = 0.0000
                v1 = 0.0000

                preco = 0

            if l_sobra < l_min_pecas + es_serra:
                break

            # Aproveitamento do l_sobra com outro tipo de peça no mesmo es
            elif p < npecas \
                    and l_sobra >= l_min_pecas + es_serra \
                    and es >= es_min_pecas + es_serra:
                continue

        # define espessura acumulada na Fatia1(das peças selecionadas)
        if es > 0 and selec == 1:
            nes2 = nes2 + es
            es2 = es
            l_d = l_d - es2
        else:
            l_d = 0

        es_sobra = l_d

        if context.get('funcoes_serrada_tora'):
            numpeca *= 4
            volpeca *= 4
            vr_preco *= 4

        return (
            codigopeca, namepeca, voltora, numtora, numpeca, volpeca, fatia, ang_e, ang_d,
            ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s,
            l_sobra, ntora, vr_preco)

    def calcula_peca_laminacao(self, cr, uid, ids, parametro_dados, context=None):
        context = context and context.copy() or {}
        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        obj_calcmadeira_arvore = self.pool.get('calcmadeira.arvores')
        calcmadeira_cubagem_ids = obj_calcmadeira_arvore.search(cr, uid, [], context=context)
        calcmadeira_arvore_dados = obj_calcmadeira_arvore.read(
            cr, uid, calcmadeira_cubagem_ids, [], context)

        di_eixo = parametro_dados.get('cmapar_laminacao_diametro_minimo')
        self.calcmadeira_valida_parametro_laminacao(cr, uid, parametro_dados, context)
        comp_folha = parametro_dados.get('cmapar_laminacao_comp_folha')
        linarv, narv = calcmadeira_arvore_dados, len(calcmadeira_arvore_dados)
        di_casc = di_casc2 = 0

        # numero de toras por amostra nas primeiras peças selecionadas
        # a1 = b1 = c1 = d1 = e1 = f1 = g1 = h1 = k1 = 0

        # volume de toras por amostra nas primeiras peças selecionadas
        # vra1 = vrb1 = vrc1 = vrd1 = vre1 = vrf1 = vrg1 = vrh1 = vrk1 = 0

        # numero de peças por tipo por amostra
        # na1 = nb1 = nc1 = nd1 = ne1 = nf1 = ng1 = nh1 = nk1 = 0

        # volume de peças por tipo por amostra
        # va1 = vb1 = vc1 = vd1 = ve1 = vf1 = vg1 = vh1 = vk1 = 0.0000

        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        comp_lamin1 = 0.0000
        comp_lamin1_eixo = 0.0000
        comp_lamin_total = 0

        di = 100

        # -------------------------------------------
        es_serra = parametro_dados.get('cmapar_es_serra')
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        hi = parametro_dados.get('cmapar_altura')
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        # -------------------------------------------

        # numero da árvore
        arv = 0

        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, calcmadeira_arvore_dados, parametro_dados,
            context)

        falha = 0

        # Executa por árvore
        while arv < narv:
            v = 0
            n = 0
            vr = 0

            a = b = c = d = e = f = g = h = k = 0
            vra = vrb = vrc = vrd = vre = vrf = vrg = vrh = vrk = 0.0000
            na = nb = nc = nd = ne = nf = ng = nh = nk = 0
            va = vb = vc = vd = ve = vf = vg = vh = vk = 0.0000

            # Acessa arquivo texto, armazena dap e ht por árvore, e calcula seu volume (vt)
            dados = linarv[arv]

            try:
                dap = float(dados.get('cmaarv_diametro'))
                ht = float(dados.get('cmaarv_altura'))
                hc = float(altura_comercial_calculada_dict.get(dados.get('id'))
                           if altura_comercial_calculada_dict else dados.get('cmaarv_altura_comecial'))

                # -----------------------------
                # Seleciona equação de volume
                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         context)
                # -----------------------------
            except:
                falha += 1

            arv = arv + 1
            # numero da tora
            ntora = 0
            es_sobra = 100

            # Calcula peças por seção do tronco até completar altura da árvore
            while hi_atual + hi <= hc:

                flecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                R_flecha_base = 0
                s = 0
                es_medula = 0

                # -------------------------------------------------
                # Seleciona equação de forma
                di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                       coeficiente_forma=coeficiente_forma,
                                                                       hi_atual=hi_atual,
                                                                       hi=hi,
                                                                       ht=ht,
                                                                       dap=dap,
                                                                       di_casc=di_casc,
                                                                       di_casc2=di_casc2,
                                                                       context=context)
                # -------------------------------------------------

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                ntora = ntora + 1
                # print('vr:{}, di_casc2:{}, di_casc:{}, di:{}, ntora:{},arv:{}'. \
                #       format(vr, di_casc2, di_casc, di, ntora, arv))

                vrt = vrt + vr

                # 'perímetro com espessura es
                es_perim = parametro_dados.get('cmapar_laminacao_espessura')  # 0.3
                # '(3 mm)

                # 'numero de voltas totais
                n = int(di / 2 / es_perim)

                for t in range(0, n + 1):
                    # 'comprimento da lamina (cm)

                    comp_lamin = math.pi *(di-t*2*es_perim)

                    if (di - t * 2 * es_perim) < di_eixo:
                        break

                    comp_lamin1 = comp_lamin1 + comp_lamin

                comp_lamin_total += comp_lamin1

                # 'Relatório de dimensões da laminação

                # 'armazena a linha de gravação dos dados em Relat_Serrad
                # print(
                #     'compr. lamin.: {}, num. tora: {}, vol.tora: {}'.format(comp_lamin, ntora, vr))
                self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, 0, '', vr, di_casc, di_casc2,
                                                         di, ntora, arv, 0, 0, 0, 0, 0, 0, 0, 0,
                                                         comp_lamin1 / 1000,
                                                         context=context)
                comp_lamin1 = 0

                # atualiza hi_atual com o comprimento da tora calculada (hi)
                hi_atual = hi_atual + hi
                # print('hi_atual=', hi_atual)

            # acumula vt da árvore e zera vt
            vt1 = vt1 + vt
            vt = 0

            # 'acumula numero de toras da arvore (ntora) para resultado total de toras na amostra
            ntora1 = ntora1 + ntora
            ntora = 0

            # retorna altura do toco
            hi_atual = parametro_dados.get('cmapar_altura_corte')

            # Alterado em 25/09/19
            ntora = ntora + 1

        # print('Sarrafo:{}, {}, {}, {}'.format(a1,vra1, na1, va1))
        # print('Ripão:{}, {}, {}, {}'.format(b1,vrb1, nb1, vb1))
        # print('Caibro:{}, {}, {}, {}'.format(c1,vrc1, nc1, vc1))
        # print('Pontalete:{}, {}, {}, {}'.format(d1,vrd1, nd1, vd1))
        # print('Ripa:{}, {}, {}, {}'.format(e1,vre1, ne1, ve1))
        # print('Viga:{}, {}, {}, {}'.format(f1,vrf1, nf1, vf1))
        # print('Tabua:{}, {}, {}, {}'.format(g1,vrg1, ng1, vg1))
        # print('Prancha:{}, {}, {}, {}'.format(h1,vrh1, nh1, vh1))
        # print('Pranchão:{}, {}, {}, {}'.format(k1,vrk1, nk1, vk1))
        # print('Volume tora total:{}, Volume total:{}, total de toras:{}'.format(vrt, vt1, ntora1))
        # print('Arvores:{}'.format(narv))

        # varcolunas

        # codpeca=[1,2,3,4,5,6,7,8,9]
        #
        # nomepeca=['Sarrafo','Ripão','Caibro','Pontalete','Ripa','Viga','Tabua','Prancha','Pranchão']
        #
        # num_tora=[a1, b1, c1, d1, e1, f1, g1, h1, k1]
        #
        # vol_tora=[vra1, vrb1,vrc1,vrd1, vre1, vrf1, vrg1, vrh1, vrk1]
        #
        # if escolha=='4':
        #     num_peca = [na1 * 4, nb1 * 4, nc1 * 4, nd1 * 4, ne1 * 4, nf1 * 4, ng1 * 4, nh1 * 4, nk1 * 4]
        #
        #     vol_peca = [va1 * 4, vb1 * 4, vc1 * 4, vd1 * 4, ve1 * 4, vf1 * 4, vg1 * 4, vh1 * 4, vk1 * 4]
        # else:
        #     num_peca=[na1, nb1, nc1, nd1, ne1, nf1, ng1, nh1, nk1]
        #
        #     vol_peca=[va1, vb1,vc1,vd1, ve1, vf1, vg1, vh1, vk1]
        #
        # #create a dictionary
        # data={'codpeca':codpeca,'nomepeca':nomepeca, 'numtora': num_tora, 'vol_tora':vol_tora, 'num_peca': num_peca, 'vol_peca': vol_peca}
        #
        # #create dataframe
        # df=pd.DataFrame(data)

        # print('')
        # print('-'*70)
        # print(df)
        # print('-' * 70)
        # print('')
        # print('-' * 20)
        # print('Toras totais: %s' % ntora1)
        # print('-' * 20)
        # print('Volume tora total: %s' % vrt)
        # print('-' * 20)
        # print('Volume total: %s' % vt1)
        # print('-' * 20)
        # print('comprim. total de laminação: %s' % comp_lamin1)
        # print('-' * 20)
        # print('Arvores: %s' % narv)
        # print('-' * 20)
        total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                            context=context)
        self.write(cr, uid, ids, {
            'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr, 'total_vt_v': total_vt_v,
            'quantidade_tora_total_laminada': ntora1, 'volume_tora_total': vrt,
            'comprimento_laminacao_total': comp_lamin_total / 1000,
            'folha_laminacao_total': int(comp_lamin_total / 1000 / parametro_dados.get(
                'cmapar_laminacao_comp_folha')) if parametro_dados.get(
                'cmapar_laminacao_comp_folha') else 0
            }, context=context)
        return True

    def calcula_peca_fator_forma(self, cr, uid, ids, parametro_dados, context=None):

        def Calcula_Part_di2_hi_difer(lintaper, taper, cont, hi_atual, ht):

            i = 1
            V_tora = 0

            while i <= cont:

                taper_aux = taper - 1
                dados = lintaper[taper_aux]
                di2 = float(dados.get('cmatap_di_casc')) ** 2
                hi_acum = float(dados.get('cmatap_hi'))

                if i == 1:
                    V_toc = math.pi / 40000 * di2 * hi_acum

                if i > 1 and i <= cont:
                    dados_1 = lintaper[taper_aux - 1]
                    di2_1 = float(dados_1.get('cmatap_di_casc')) ** 2
                    hi = float(dados.get('cmatap_hi')) - float(dados_1.get('cmatap_hi'))

                    V_tora = V_tora + (math.pi / 40000 * (di2_1 + di2) / 2) * hi

                if float(hi_acum) == 1.3:
                    dap = di2 ** (1 / 2.0)
                    dap2 = di2

                if i == cont:
                    hi_pta = float(ht) - float(hi_acum)
                    V_pta = math.pi / 40000 * di2 * hi_pta * 1 / 3

                i = i + 1
                taper = taper + 1

            V_cub_arv = V_toc + V_tora + V_pta

            hi1 = 0
            hi2 = 1
            taper = taper - 1
            cont = 0

            return dap, dap2, hi1, hi2, taper, cont, V_cub_arv

        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        obj_calcmadeira_taper = self.pool.get('calcmadeira.taper')
        calcmadeira_taper_ids = obj_calcmadeira_taper.search(cr, uid, [], context=context)
        calcmadeira_taper_dados = obj_calcmadeira_taper.read(cr, uid, calcmadeira_taper_ids, [],
                                                             context)
        # -----------------------
        lintaper, ntaper = calcmadeira_taper_dados, len(calcmadeira_taper_dados)
        # -----------------------

        vt = 0.0000
        vt1 = 0.0000
        ntora1 = 0

        # declarando variáveis, não necessário no VBA
        di = 100

        # Parametros: começa pela altura do toco (pode ser alterada pelo usuário)
        # -----------------------------------------------------------------------
        es_serra = parametro_dados.get('cmapar_es_serra')
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        hi = parametro_dados.get('cmapar_altura')
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
        es_medula = parametro_dados.get('es_medula')
        # -----------------------------------------------------------------------

        # numero da árvore
        arv = 1

        # percorre todos os dados. taper é o 'r' no VBA, que começa na linha 2. r=2.
        # '1 limita o 1o dado que não tem com quem comparar
        taper = 0

        # limita cada árvore
        cont = 0

        # para iniciar sem os dados
        hi = 0
        hi1 = 0
        hi2 = 1
        FFarv1 = 0.00
        FFarv1_2 = 0.00
        
        cubagem_dados = []

        # Executa por árvore
        while taper < ntaper:

            while hi1 < hi2:
                cont = cont + 1
                taper = taper + 1
                taper_aux = taper - 1
                try:
                    dados = lintaper[taper_aux]
                    hi = float(dados.get('cmatap_hi'))

                    dados1 = lintaper[taper_aux + 1]
                    hi1 = float(dados1.get('cmatap_hi'))

                    dados2 = lintaper[taper_aux + 2]
                    hi2 = float(dados2.get('cmatap_hi'))

                    # print('hi={}, hi1={}, hi2={}'.format(hi, hi1, hi2))

                except:
                    break
                    # pass

            try:
                ht = float(dados1.get('cmatap_ht'))
            except:
                print('Faltou altura total na coluna Ht')

            taper = taper + 1 - cont
            cont = cont + 1

            # -----------------------------------------------------------------------------------
            dap, dap2, hi1, hi2, taper, cont, V_cub_arv = Calcula_Part_di2_hi_difer(
                lintaper, taper, cont, hi_atual, ht)
            # -----------------------------------------------------------------------------------

            FFarv = V_cub_arv / (math.pi / 40000 * dap2 * ht)

            vt = V_cub_arv

            # print('arv=', arv, 'V_cub_arv=', V_cub_arv, 'FFarv=', FFarv)
            self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, 0, '', V_cub_arv, 0, 0,
                                                     0, 0, arv, 0, 0, 0, 0, 0, 0, 0, 0,
                                                     0, FFarv, context=context)

            cubagem_dados.append({'cmacub_linha': dados1.get('cmatap_lin'),
                                  'cmacub_arvore': arv,
                                  'cmacub_diametro': dap,
                                  'cmacub_altura': dados1.get('cmatap_ht'),
                                  'cmacub_altura_comercial': dados1.get('cmatap_ht'),
                                  'cmacub_volume': V_cub_arv})

            FFarv1 = FFarv1 + FFarv
            FFarv1_2 = FFarv1_2 + FFarv ** 2

            arv = arv + 1

        FFarv1_med = FFarv1 / (arv - 1)

        # 'formula para dp =((Sx^2-((Sx)^2/n)/(n-1))^(1/2)
        # 'Uso arv-1 e arv-2, pq o acumulado de arv vem como arv+1, ou seja, se existe apenas 1 arv, arv=2
        # 'Para calculo de dp são necessários pelo menos 3 dados, então arv > 3
        if arv > 3:
            FFarv1_dp = ((FFarv1_2 - FFarv1 ** 2 / (arv - 1)) / (arv - 2)) ** (1.0 / 2)

        print('Fator de Forma Médio: {}'.format(FFarv1_med))
        print('Desvio Padrão Fator de Forma: {}'.format(FFarv1_dp))

        self.write(cr, uid, ids, {'narv': arv > 0 and arv - 1 or 0, 'fator_forma_medio': FFarv1_med,
                                  'fator_forma_desvio_padrao': FFarv1_dp}, context=context)

        obj_calcmadeira_cubagem = self.pool.get('calcmadeira.cubagem')
        cubagem_dados and obj_calcmadeira_cubagem.unlink(cr, uid, obj_calcmadeira_cubagem.search(
            cr, uid, [], context=context), context)
        for cubagem in cubagem_dados:
            obj_calcmadeira_cubagem.create(cr, uid, cubagem, context)

        return True

    def calcula_peca_gira_180_90(self, cr, uid, ids, parametro_dados, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        preco_medula = 0.0000
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        arvores_ids = obj_dados_arvores.search(cr, uid, [], context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
        es_medula = parametro_dados.get('es_medula')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial')
                 ])
            narv += 1

        falha = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                try:
                    dap = t[0]
                    ht = t[1]
                    hc = t[2]
                    vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                             context)
                except:
                    falha += 1

                arv += 1
                # numero da tora
                ntora = 0
                v = 0
                n = 0
                vr = 0
                es_sobra = 100
                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    r_flecha_base = 0
                    s = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt = vrt + vr
                    # angulo inicial para desconto do di para cada lado (flecha0)
                    ang_d = parametro_dados.get('cmapar_ang_direito')
                    ang_base = 0
                    flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                    # largura da peça (corda)
                    if ang_d < 90 - ang_base or ang_base == 0:
                        l = di * math.sin(ang_d * math.pi / 180)
                    else:
                        l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                    if di < l_min_pecas + es_serra:
                        break

                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    l_sobra = l

                    for s in range(0, 1 + 1):
                        codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, \
                        preco, l, cord_d1, cord_d, r_flecha_topo, s = \
                            self.seleciona_peca_longitudinal_base(
                                cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di,
                                flecha0,
                                r_flecha_base, l, ntora, arv, di_casc, di_casc2,
                                pecas_serradas_preco_dados, parametro_dados, ang_base, flecha_div_r,
                                cord_d1, cord_d, r_flecha_topo, s, context=context)
                        # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                        # atualiza-lo
                        self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                       numtora, numpeca, volpeca, preco,
                                                       context=context)
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                        = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids,
                                                                    l_min_pecas, es_min_pecas,
                                                                    es_serra, hi,
                                                                    vr,
                                                                    l, ntora, arv, di,
                                                                    di_casc, di_casc2,
                                                                    pecas_serradas_preco_dados,
                                                                    cord_d, es_medula, flecha0,
                                                                    context=context)

                    hi_atual = hi_atual + hi

                # atualiza hi_atual com o comprimento da peça extraída (hi)
                # hi_atual = hi_atual + hi
                # 'acumula numero de toras da arvore (ntora) para resultado total de
                # toras na amostra
                ntora1 = ntora1 + ntora
                ntora = 0
                # acumula vt arvore e zera vt
                vt1 = vt1 + vt
                vt = 0
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {'narv': narv, 'vt1': vt1,
                                                         'total_vt_vr': total_vt_vr,
                                                         'total_vt_v': total_vt_v}, context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def calcula_tora_longitudinal_base(self, cr, uid, ids, parametro_dados, context=None):
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')

        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1
        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')
                es_sobra = 100

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                flecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                # angulo inicial para retirada da costaneira da base
                ang_base = parametro_dados.get('cmapar_ang_base')
                # comprimento da peça, do eixo x até o limite da costaneira da base (R-flecha)
                r_flecha_base = di / 2 * math.cos(ang_base * math.pi / 180)
                # angulo inicial para desconto de 1/4 do di para cada lado (flecha = 1/4*R)
                ang_d = parametro_dados.get('cmapar_ang_direito')

                if brw_calcmadeira_calcula.tipo_funcao == '16':  # '8' - Radial Longitudinal
                    ang_base = 90
                    r_flecha_base = 0

                flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                # largura da peça (corda)
                if ang_d < (90 - ang_base) or ang_base == 0:
                    l = di * math.sin(ang_d * math.pi / 180)
                else:
                    l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                                    context=context)
                if di < l_min_pecas + es_serra:
                    continue
                # 'ALTERADO
                l_sobra = l
                flecha = 0

                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                    cord_d1, cord_d, r_flecha_topo, s \
                    = self.seleciona_peca_longitudinal_base(
                        cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                        r_flecha_base, l, ntora, arv, di_casc, di_casc2, pecas_serradas_preco_dados,
                        parametro_dados, ang_base, flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
                        context=context)

                if brw_calcmadeira_calcula.tipo_funcao == '16':  # '8' - Radial Longitudinal
                    numpeca *= 2
                    volpeca *= 2
                    preco *= 2
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora,
                                               numpeca, volpeca, preco, context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': 0,
                        'vt1': sum(map(
                            lambda it: it.voltora or 0, self.browse(cr, uid, calcmadeira_calcula_id,
                                                                    context).peca_ids)),
                        'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v
                        }, context=context)
            return True

    def calcula_tora_longitudinal(self, cr, uid, ids, parametro_dados, context=None):
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')

        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1

        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                # angulo inicial para desconto de 1/4 do di para cada lado (flecha = 1/4*R)
                ang_d = parametro_dados.get('cmapar_ang_direito')
                ang_base = 0
                flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                # largura da peça (corda)
                if ang_d < (90 - ang_base) or ang_base == 0:
                    l = di * math.sin(ang_d * math.pi / 180)
                else:
                    l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0

                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                                    context=context)
                if di < l_min_pecas + es_serra:
                    continue
                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                    cord_d1, cord_d, r_flecha_topo, s \
                    = self.seleciona_peca_longitudinal_base(
                        cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                        r_flecha_base, l, ntora, arv, di_casc, di_casc2, pecas_serradas_preco_dados,
                        parametro_dados, ang_base, flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
                        context=context)
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora,
                                               numpeca, volpeca, preco, context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {
                'narv': 0,
                'vt1': sum(map(lambda it: it.voltora or 0,
                               self.browse(cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                'total_vt_vr': total_vt_vr,
                'total_vt_v': total_vt_v}, context=context)
            return True

    def calcula_tora_quadrado_circunscrito(self, cr, uid, ids, parametro_dados, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        # vrt = 0.0000
        # ntora = 0
        # di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Executa por tora
        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1
        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')
                v = 0
                n = 0
                vr = 0
                es_sobra = 100

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                # largura da peça
                l = di / math.sqrt(2)
                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                if l < l_min_pecas+es_serra:
                    ntora = ntora - 1
                    continue
                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                    = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids, l_min_pecas,
                                                                es_min_pecas, es_serra, hi, vr, l,
                                                                ntora, arv, di, di_casc, di_casc2,
                                                                pecas_serradas_preco_dados,
                                                                cord_d, es_medula, flecha0,
                                                                context=context)

                # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora,
                                               numpeca, volpeca, preco, context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': 0,
                        'vt1': sum(map(lambda it: it.voltora or 0, self.browse(
                            cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                        'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v}, context=context)
            return True

    def calcula_tora_desdobro_radial(self, cr, uid, ids, parametro_dados, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        vrt = 0.0000
        ntora = 0
        codpeca = 0
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Executa por tora
        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1
        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')
                v = 0
                n = 0
                vr = 0

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                l = di / 2
                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                # l_min_pecas=2 # DimSerrada.txt todas as peças
                if l < l_min_pecas + es_serra or di/2 < es_min_pecas + es_serra:
                    continue

                l_e = l_d = l_i = l_s = l
                l_sobra = 0.0000
                nes1 = nes2 = 0.0
                es1 = es2 = 0.0
                ang_d = 90.0000
                ang_s = 0.0000
                ang_e = ang_i = 0
                fatia = 0
                n1 = v1 = 0

                # Verifica se pelo menos a menor peça será selecionada
                es = es_min_pecas + es_serra
                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e,
                               l_d, l_i, l_s)
                # ---------------------------------------------
                es = 0

                while l_d >= l_min_pecas + es_serra or l_s >= l_min_pecas + es_serra:
                    if fatia == 0:
                        codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                        fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, \
                        di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                            = self.seleciona_peca_desdobro_radial_f0(
                            cr, uid, ids, l_min_pecas, es_min_pecas, ang_e, ang_d, ang_i, ang_s,vr,
                            es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra,
                            ntora, arv, di_casc, di_casc2, pecas_serradas_preco_dados, fatia,
                            context)
                        fatia = 1
                        # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                        # atualiza-lo
                        self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                       voltora, numtora,
                                                       numpeca, volpeca, preco, context=context)
                    if fatia == 1:
                        codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                        fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, \
                        di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                            = self.seleciona_peca_desdobro_radial_f1(
                            cr, uid, ids, l_min_pecas, es_min_pecas, ang_e, ang_d, ang_i, ang_s, vr,
                            es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra,
                            ntora, arv, di_casc, di_casc2, pecas_serradas_preco_dados, fatia,
                            context)
                        fatia = 0
                        # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                        # atualiza-lo
                        self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                       voltora, numtora,
                                                       numpeca, volpeca, preco, context=context)
                        if l_d < es_min_pecas + es_serra and l_s < es_min_pecas + es_serra:
                            break
                    # atualiza hi_atual com o comprimento da peça extraída (hi)
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # ntora = 0
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': 0,
                        'vt1': sum(map(lambda it: it.voltora or 0, self.browse(
                            cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                        'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v}, context=context)
            return True

    def calcula_tora_laminacao(self, cr, uid, ids, parametro_dados, context=None):
        context = context and context.copy() or {}
        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context)
        obj_calcmadeira_toras = self.pool.get('calcmadeira.toras')
        tora_ids = obj_calcmadeira_toras.search(cr, uid, [], context=context)
        calcmadeira_toras_dados = obj_calcmadeira_toras.read(cr, uid, tora_ids, [], context)

        lintora, ntora1 = calcmadeira_toras_dados, len(calcmadeira_toras_dados)
        # -----------------------

        di_eixo = parametro_dados.get('cmapar_laminacao_diametro_minimo')
        self.calcmadeira_valida_parametro_laminacao(cr, uid, parametro_dados, context)

        # numero de toras por amostra nas primeiras peças selecionadas
        # a1 = b1 = c1 = d1 = e1 = f1 = g1 = h1 = k1 = 0

        # volume de toras por amostra nas primeiras peças selecionadas
        # vra1 = vrb1 = vrc1 = vrd1 = vre1 = vrf1 = vrg1 = vrh1 = vrk1 = 0

        vrt = 0.0000
        vr1 = 0.0000
        ntora = 0
        comp_lamin1 = 0.0000
        comp_lamin1_eixo = 0.0000
        comp_lamin_total = 0

        # declarando variáveis, não necessário no VBA
        di = 100

        # -------------------------------------------
        es_serra = parametro_dados.get('cmapar_es_serra')
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        hi = parametro_dados.get('cmapar_altura')
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        # -------------------------------------------
        hi_atual = hi = 0

        # Executa por tora
        while ntora < ntora1:
            v = 0
            n = 0
            vr = 0

            a = b = c = d = e = f = g = h = k = 0
            vra = vrb = vrc = vrd = vre = vrf = vrg = vrh = vrk = 0.0000
            na = nb = nc = nd = ne = nf = ng = nh = nk = 0
            va = vb = vc = vd = ve = vf = vg = vh = vk = 0.0000

            # Acessa arquivo texto, armazena dap e ht por árvore, e calcula seu volume (vt)
            dados = lintora[ntora]
            ntora = ntora + 1
            arv = float(dados.get('cmator_arvore'))
            tora = int(dados.get('cmator_tora'))
            di_casc2 = float(dados.get('cmator_diametro_min'))
            di_casc = float(dados.get('cmator_diametro_max'))
            hi = float(dados.get('cmator_comprimento_tora'))

            es_sobra = 100

            # volume da tora usando formula de Smalian
            vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

            # (100 - 6.8% de casca, medido no Eucal Maravilhas)
            # (100- 8.7% casca, medido no citriodora Arv 1)
            # (100- 4.8% casca, medido no citriodora Arv 2)
            # (100- 8.5% casca, medido no citriodora Arv 3)
            # (100 - 9.4% casca, medido na tora 2 da Arv 3)
            di = di_casc2 * (1 - prop_casc)*10

            lecha = flecha0 = 0
            cord_d1 = 0
            cord_d = 0
            r_flecha_topo = 0
            r_flecha_base = 0
            s = 0
            es_medula = 0

            # print('vr:{}, di_casc2:{}, di_casc:{}, di:{}, tora:{},arv:{}'. \
            #       format(vr, di_casc2, di_casc, di, tora, arv))

            # 'perímetro com espessura es
            es_perim = parametro_dados.get('cmapar_laminacao_espessura')  # 0.3
            # '(3 mm)

            # 'numero de voltas
            n = int(di / 2 / es_perim)

            for t in range(0, n + 1):
                # 'comprimento da lamina (cm)

                comp_lamin = math.pi * (di - t * 2 * es_perim)

                if (di - t * 2 * es_perim ) < di_eixo:
                    break

                comp_lamin1 = comp_lamin1 + comp_lamin

            comp_lamin_total += comp_lamin1

            vr1 = vr1 + vr

            # 'Relatório de dimensões da laminação

            # 'armazena a linha de gravação dos dados em Relat_Serrad
            # print('compr. lamin.: {}, num. tora: {}, vol.tora: {}'.format(comp_lamin, ntora, vr))

            # print(
            #     'compr. lamin.: {}, num. tora: {}, vol.tora: {}'.format(comp_lamin, ntora, vr))
            self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, 0, '', vr, di_casc, di_casc2,
                                                     di, ntora, arv, 0, 0, 0, 0, 0, 0, 0, 0,
                                                     comp_lamin1 / 1000,
                                                     context=context)

            comp_lamin1 = 0

        # print('Sarrafo:{}, {}, {}, {}'.format(a1,vra1, na1, va1))
        # print('Ripão:{}, {}, {}, {}'.format(b1,vrb1, nb1, vb1))
        # print('Caibro:{}, {}, {}, {}'.format(c1,vrc1, nc1, vc1))
        # print('Pontalete:{}, {}, {}, {}'.format(d1,vrd1, nd1, vd1))
        # print('Ripa:{}, {}, {}, {}'.format(e1,vre1, ne1, ve1))
        # print('Viga:{}, {}, {}, {}'.format(f1,vrf1, nf1, vf1))
        # print('Tabua:{}, {}, {}, {}'.format(g1,vrg1, ng1, vg1))
        # print('Prancha:{}, {}, {}, {}'.format(h1,vrh1, nh1, vh1))
        # print('Pranchão:{}, {}, {}, {}'.format(k1,vrk1, nk1, vk1))
        # print('Volume tora total:{}, Volume total:{}, total de toras:{}'.format(vrt, vt1, ntora1))
        # print('Arvores:{}'.format(narv))

        # varcolunas

        # codpeca = [1, 2, 3, 4, 5, 6, 7, 8, 9]
        #
        # nomepeca = ['Sarrafo', 'Ripão', 'Caibro', 'Pontalete', 'Ripa', 'Viga', 'Tabua', 'Prancha', 'Pranchão']
        #
        # num_tora = [a1, b1, c1, d1, e1, f1, g1, h1, k1]
        #
        # vol_tora = [vra1, vrb1, vrc1, vrd1, vre1, vrf1, vrg1, vrh1, vrk1]
        #
        # if escolha=='4':
        #     num_peca = [na1 * 4, nb1 * 4, nc1 * 4, nd1 * 4, ne1 * 4, nf1 * 4, ng1 * 4, nh1 * 4, nk1 * 4]
        #
        #     vol_peca = [va1 * 4, vb1 * 4, vc1 * 4, vd1 * 4, ve1 * 4, vf1 * 4, vg1 * 4, vh1 * 4, vk1 * 4]
        # else:
        #     num_peca=[na1, nb1, nc1, nd1, ne1, nf1, ng1, nh1, nk1]
        #
        #     vol_peca=[va1, vb1,vc1,vd1, ve1, vf1, vg1, vh1, vk1]
        #
        # # create a dictionary
        # data = {'codpeca': codpeca, 'nomepeca': nomepeca, 'numtora': num_tora, 'vol_tora': vol_tora, 'num_peca': num_peca,
        #         'vol_peca': vol_peca}
        #
        # # create dataframe
        # df = pd.DataFrame(data)

        # print('')
        # print('-' * 70)
        # print(df)
        # print('-' * 70)
        # print('')
        # print('-' * 20)
        print('número de toras: %s' % ntora)
        # print('-' * 20)
        # print('volume total de toras: %s' % vr1)
        # print('-' * 20)
        # print('comprim. total de laminação: %s' % comp_lamin1)
        # print('-' * 20)
        # # print(f'Arvores: {narv}')
        # print('-' * 20)

        self.write(cr, uid, ids,
                   {'narv': 0, 'vt1': 0, 'total_vt_vr': 0, 'total_vt_v': 0,
                    'quantidade_tora_total_laminada': ntora, 'volume_tora_total': vr1,
                    'comprimento_laminacao_total': comp_lamin_total / 1000,
                    'folha_laminacao_total': int(comp_lamin_total / 1000 / parametro_dados.get(
                        'cmapar_laminacao_comp_folha')) if parametro_dados.get(
                        'cmapar_laminacao_comp_folha') else 0
                    },
                   context=context)

        return True

    def calcula_tora_gira_180_90(self, cr, uid, ids, parametro_dados, context=None):
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
        es_medula = parametro_dados.get('es_medula')
        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1

        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0

                # angulo inicial para desconto de 1/4 do di para cada lado (flecha = 1/4*R)
                ang_d = parametro_dados.get('cmapar_ang_direito')
                ang_base = 0
                flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                # largura da peça (corda)
                if ang_d < (90 - ang_base) or ang_base == 0:
                    l = di * math.sin(ang_d * math.pi / 180)
                else:
                    l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                cord_d1 = 0
                cord_d = 0

                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                                    context=context)
                if di < l_min_pecas + es_serra:
                    continue

                l_sobra = l
                flecha = 0
                r_flecha_topo = 0

                for s in range(0, 1 + 1):
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                        cord_d1, cord_d, r_flecha_topo, s \
                        = self.seleciona_peca_longitudinal_base(
                            cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                            r_flecha_base, l, ntora, arv, di_casc, di_casc2,
                            pecas_serradas_preco_dados, parametro_dados, ang_base, flecha_div_r,
                            cord_d1, cord_d, r_flecha_topo, s, context=context)
                    # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                    # atualiza-lo
                    self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                   numtora, numpeca, volpeca, preco,
                                                   context=context)
                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                    = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids,
                                                                l_min_pecas, es_min_pecas,
                                                                es_serra, hi,
                                                                vr,
                                                                l, ntora, arv, di,
                                                                di_casc, di_casc2,
                                                                pecas_serradas_preco_dados,
                                                                cord_d, es_medula, flecha0,
                                                                context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {
                'narv': 0,
                'vt1': sum(map(lambda it: it.voltora or 0,
                               self.browse(cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                'total_vt_vr': total_vt_vr,
                'total_vt_v': total_vt_v}, context=context)
            return True

    def calcmadeira_atualiza_tree(self, cr, uid, ids, codigopeca, namepeca, voltora, numtora,
                                  numpeca, volpeca, preco_item=0, context=None):
        obj_wizard_calcula_peca_rolica_itens = self.pool.get('wizard.calcmadeira.calcula.itens')
        peca_rolica_id = ids[0] if isinstance(ids, list) else ids
        if preco_item and not self.read(cr, uid, peca_rolica_id, ['calcula_receita'], context).get(
                'calcula_receita'):
            preco_item = 0

        itens_ids = obj_wizard_calcula_peca_rolica_itens.search(cr, uid, [
            ('peca_id', '=', peca_rolica_id), ('codpeca', '=', codigopeca)],
                                                                context=context)
        # se a peca ja estiver na tree, apenas atualiza os dados somando a
        # quantidade e volume de cada tora e peca
        if itens_ids:
            peca_itens_dados = obj_wizard_calcula_peca_rolica_itens.read(
                cr, uid, itens_ids[0], ['numtora', 'voltora', 'numpeca', 'volpeca', 'preco_item'],
                context=context)
            obj_wizard_calcula_peca_rolica_itens.write(cr, uid, itens_ids[0], {
                'numtora': numtora + peca_itens_dados.get('numtora'),
                'voltora': voltora + peca_itens_dados.get('voltora'),
                'numpeca': numpeca + peca_itens_dados.get('numpeca'),
                'volpeca': volpeca + peca_itens_dados.get('volpeca'),
                'preco_item': preco_item + peca_itens_dados.get('preco_item'),
            })
        else:
            # insere na tree a quantidade e volume de cada tora e peca
            obj_wizard_calcula_peca_rolica_itens.create(cr, uid, {'peca_id': peca_rolica_id,
                                                                  'codpeca': codigopeca,
                                                                  'nomepeca': namepeca,
                                                                  'numtora': numtora,
                                                                  'voltora': voltora,
                                                                  'numpeca': numpeca,
                                                                  'volpeca': volpeca,
                                                                  'preco_item': preco_item},
                                                        context=context)

    def calcmadeira_atualiza_tree_resultado(self, cr, uid, ids, codigopeca, namepeca, vr_tree,
                                            di_casc, di_casc2, di, numtora, arv,
                                            es_serra, l, n, v, fatia, l_sobra, es_sobra, es,
                                            comp_lamin=0, ffarv=0, conicidade=0, rendimento=0,
                                            preco=0, comprimento=0, context=None):
        if not context.get('calcula_receita'):
            preco = 0
        resultado_peca_id = ids[0] if isinstance(ids, list) else ids
        obj_wizard_resultado_peca = self.pool.get('wizard.calcmadeira.resultado.peca')
        obj_wizard_resultado_peca.create(cr, uid, {'result_id': resultado_peca_id,
                                                   'codpeca': codigopeca,
                                                   'nomepeca': namepeca,
                                                   'vr': vr_tree,
                                                   'di_casc': di_casc,
                                                   'di_casc2': di_casc2,
                                                   'di': di,
                                                   'numtora': numtora,
                                                   'arv': arv,
                                                   'es_serra': es_serra + es,
                                                   'l': l,
                                                   'n': n,
                                                   'v': v,
                                                   'fatia': fatia,
                                                   'l_sobra': l_sobra,
                                                   'es_sobra': es_sobra,
                                                   'comp_laminacao': comp_lamin,
                                                   'ffarv': ffarv,
                                                   'conicidade': conicidade,
                                                   'preco': preco,
                                                   'comprimento': comprimento,
                                                   },
                                                   # 'rendimento': rendimento},
                                         context=context)

    def calcmadeira_limpa_tree(self, cr, uid, ids, context=None):
        obj_wizard_calcula_peca_rolica_itens = self.pool.get('wizard.calcmadeira.calcula.itens')
        obj_wizard_resultado_peca = self.pool.get('wizard.calcmadeira.resultado.peca')
        peca_rolica_id = ids[0] if isinstance(ids, list) else ids
        itens_ids = obj_wizard_calcula_peca_rolica_itens.search(cr, uid,
                                                                [('peca_id', '=', peca_rolica_id)],
                                                                context=context)
        resultado_ids = obj_wizard_resultado_peca.search(cr, uid,
                                                         [('result_id', '=', peca_rolica_id)],
                                                         context=context)
        if itens_ids:
            obj_wizard_calcula_peca_rolica_itens.unlink(cr, uid, itens_ids, context=context)
        if resultado_ids:
            obj_wizard_resultado_peca.unlink(cr, uid, resultado_ids, context=context)

    def calmadeira_menor_largura_espessura(self, cr, uid, ids, context=None):
        obj_dados_pecas_serradas = self.pool.get('calcmadeira.pecas.serradas')
        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas_serradas = brw_calcmadeira_calcula[0].pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas_serradas:
            pecas_serradas.append(i.id)
        l_min_id = obj_dados_pecas_serradas.search(cr, uid, [('id', 'in', pecas_serradas)],
                                                   order='cmapse_menor_largura', limit=1,
                                                   context=context)
        es_min_id = obj_dados_pecas_serradas.search(cr, uid, [('id', 'in', pecas_serradas)],
                                                    order='cmapse_menor_espessura', limit=1,
                                                    context=context)
        obj_dados_largura_min = obj_dados_pecas_serradas.read(cr, uid, l_min_id, context=context)
        obj_dados_espessura_min = obj_dados_pecas_serradas.read(cr, uid, es_min_id, context=context)

        # se largura < largura mínima entre todas as peças, resto da árvore não atende
        # dimensões, mesmo com comprimento suficiente da peça
        for z in obj_dados_largura_min:
            l_min_pecas = z.get('cmapse_menor_largura')
        for y in obj_dados_espessura_min:
            es_min_pecas = y.get('cmapse_menor_espessura')
        return (l_min_pecas, es_min_pecas)

    def calcmadeira_calcula_volume_peca_tora(self, cr, uid, ids, vt1, context=None):
        dados_itens = self.pool.get('wizard.calcmadeira.calcula')
        brw_dados_itens = dados_itens.browse(cr, uid, ids, context=context)
        total_voltora = 0
        total_volpeca = 0
        if context.get('funcoes_serrada_tora'):
            return 0, 0
        for x in brw_dados_itens:
            if x.peca_ids:
                for y in x.peca_ids:
                    total_voltora += y.voltora
                    total_volpeca += y.volpeca
            elif x.result_ids:
                for y in x.result_ids:
                    total_voltora += (y.vr or 0)
        total_vt_vr = vt1 - total_voltora
        total_vt_v = vt1 - total_volpeca
        if context.get('calculo_rolica') or context.get('calculo_laminada_arvore'):
            total_vt_v = 0
        return total_vt_vr, total_vt_v

    def button_calcmadeira_calcula(self, cr, uid, ids, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        calcmadeira_calcula_dados = self.read(cr, uid, calcmadeira_calcula_id,
                                              ['tipo_funcao', 'calcula_receita'], context)
        tipo_funcao = calcmadeira_calcula_dados.get('tipo_funcao')
        context.update({'calcula_receita': calcmadeira_calcula_dados.get(
            'calcula_receita') or False})

        obj_calcmadeira_parametro = self.pool.get('calcmadeira.parametro')
        calcmadeira_parametro_id = obj_calcmadeira_parametro.search(cr, uid, [], context=context)
        if not calcmadeira_parametro_id:
            raise osv.except_osv(_('Atencao'), _('Parametros nao cadastrados.'))
        parametro_dados = obj_calcmadeira_parametro.read(cr, uid, calcmadeira_parametro_id[0], [],
                                                         context, '_classic_write')
        cmapar_es_medula = 0
        if parametro_dados.get('cmapar_espessura_peca_central') > parametro_dados.get(
                'cmapar_es_serra'):
            cmapar_es_medula = (parametro_dados.get('cmapar_espessura_peca_central') or 0) + (
                parametro_dados.get('cmapar_es_serra') or 0)
        parametro_dados.update({'es_medula': cmapar_es_medula})

        if tipo_funcao == '1':
            self.calcula_peca_rolica(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao in ('2', '15'):
            self.calcula_peca_longitudinal_base(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '3':
            self.calcula_peca_longitudinal(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '4':
            self.calcula_peca_quadrado_circunscrito(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '5':
            self.calcula_peca_desdobro_radial(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao in ('6', '16'):
            self.calcula_tora_longitudinal_base(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '7':
            self.calcula_tora_longitudinal(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '8':
            self.calcula_tora_quadrado_circunscrito(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '9':
            self.calcula_tora_desdobro_radial(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '10':
            self.calcula_peca_laminacao(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '11':
            self.calcula_tora_laminacao(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '12':
            self.calcula_peca_fator_forma(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '13':
            self.calcula_peca_gira_180_90(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '14':
            self.calcula_tora_gira_180_90(cr, uid, ids, parametro_dados, context)

        if tipo_funcao not in ('1', '10', '11'):
            self.calcmadeira_gera_rendimento_resultado(cr, uid, ids, tipo_funcao, context)

        return True

    def calcmadeira_gera_rendimento_resultado(self, cr, uid, ids, tipo_funcao, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT 
                min(id) AS id, 
                round(sum(v) / max(vr), 4) AS rendimento, 
                conicidade
            FROM 
                wizard_calcmadeira_resultado_peca
            WHERE 
                result_id = {result_id}
            GROUP 
                BY conicidade, numtora
            --ORDER BY 
            --    id
        """.format(result_id=ids[0] if isinstance(ids, list) else ids)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            return False
        resultado_ids = []
        for dado in dados:
            rendimento = dado.get('rendimento') or 0
            if tipo_funcao in ('5', '9'):
                rendimento *= 4
            elif tipo_funcao in ('15', '16'):
                rendimento *= 2
            resultado_ids.append((1, dado.get('id'), {'rendimento': rendimento}))
        self.write(cr, uid, ids, {'result_ids': resultado_ids}, context)
        return True

    def calcmadeira_get_equacao_volume(self, cr, uid, brw_coeficiente, dap, ht, context=None):
        context = context and context.copy() or {}
        vt = 0
        if brw_coeficiente.name == 'Volume padrao' and dap > 0 and ht > 0:
            vt = equacao_volume(dap, ht, brw_coeficiente.cmacoe_b0,
                                brw_coeficiente.cmacoe_b1, brw_coeficiente.cmacoe_b2)
        elif dap > 0 and ht > 0:
            vt = Eqvt(brw_coeficiente.cmacoe_b0, brw_coeficiente.cmacoe_b1,
                      brw_coeficiente.cmacoe_b2, dap, ht)
            vt = vt.vt_result()
        return vt

    def calcmadeira_get_equacao_forma(self, cr, uid,
                                      coeficiente_forma=None,
                                      hi_atual=None,
                                      hi=None,
                                      ht=None,
                                      dap=None,
                                      di_casc=None,
                                      di_casc2=None,
                                      context=None):
        context = context and context.copy() or {}
        if coeficiente_forma.name == 'Forma padrao':
            di_casc, di_casc2 = equacao_forma(hi_atual, hi, ht, dap, di_casc, di_casc2,
                                              coeficiente_forma.cmacoe_b0 or 0,
                                              coeficiente_forma.cmacoe_b1 or 0,
                                              coeficiente_forma.cmacoe_b2 or 0)
        else:
            di_casc2 = Eqdi_casc(
                coeficiente_forma.cmacoe_b0 or 0, coeficiente_forma.cmacoe_b1 or 0,
                coeficiente_forma.cmacoe_b2, hi_atual, hi, ht, dap, di_casc, di_casc2)
            di_casc = Eqdi_casc(
                coeficiente_forma.cmacoe_b0 or 0, coeficiente_forma.cmacoe_b1 or 0,
                coeficiente_forma.cmacoe_b2 or 0, hi_atual, hi, ht, dap, di_casc,
                di_casc2)
            di_casc2 = di_casc2.di_casc2_result()
            di_casc = di_casc.di_casc_result()
        return di_casc, di_casc2

    def get_altura_comercial_calculada_dados(self, cr, uid, brw_calcmadeira_calcula, arvores_dados,
                                             parametro_dados, context=None):
        context = context and context.copy() or {}
        altura_comercial_calculada = brw_calcmadeira_calcula.altura_comercial_calculada
        if not altura_comercial_calculada:
            return {}
        sql = """SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id""" % \
              brw_calcmadeira_calcula.company_id.id
        cr.execute(sql)
        taper_dados = cr.dictfetchall()
        diametro_minimo = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
        altura_comercial_calculada_dict = calcula_altura_comercial(
            arvores_dados, diametro_minimo, parametro_dados, taper_dados)
        # Atualiza a altura comercial no cadastro de arvores
        obj_calcmadeira_arvores = self.pool.get('calcmadeira.arvores')
        map(lambda ca_id: obj_calcmadeira_arvores.write(
            cr, uid, ca_id, {'cmaarv_altura_comecial': altura_comercial_calculada_dict.get(ca_id)},
            context), altura_comercial_calculada_dict.keys())
        return altura_comercial_calculada_dict

    def calcula_madeira_serrada_por_hectare(self, cr, uid, brw_calcmadeira_calcula, qtd_arvores,
                                            context=None):
        context = context and context.copy() or {}
        cr.execute("""DELETE FROM wizard_calcmadeira_calcula_itens_hectare 
                      WHERE wcahec_capa_id = %s""" % brw_calcmadeira_calcula.id)
        if not brw_calcmadeira_calcula.calcula_receita:
            return False
        arvore_por_hectare = brw_calcmadeira_calcula.arvore_por_hectare
        if arvore_por_hectare <= 0:
            return False
        list_itens_hectare = []
        for item in brw_calcmadeira_calcula.peca_ids:
            num_tora_ha = int(item.numtora * arvore_por_hectare / qtd_arvores)
            vol_tora_ha = (item.voltora * arvore_por_hectare / qtd_arvores)
            num_peca_ha = int(item.numpeca * arvore_por_hectare / qtd_arvores)
            vol_peca_ha = item.volpeca * arvore_por_hectare / qtd_arvores
            preco_ha = int(item.preco_item * arvore_por_hectare / qtd_arvores)
            vals_itens_hectare = {'wcahec_codigo_peca': item.codpeca,
                                  'wcahec_nome_peca': item.nomepeca,
                                  'wcahec_numero_peca': num_peca_ha,
                                  'wcahec_numero_tora': num_tora_ha,
                                  'wcahec_preco': preco_ha,
                                  'wcahec_volume_peca': vol_peca_ha,
                                  'wcahec_volume_tora': vol_tora_ha}
            list_itens_hectare.append((0, 0, vals_itens_hectare))
        list_itens_hectare and self.write(
            cr, uid, brw_calcmadeira_calcula.id, {'itens_hectare_ids': list_itens_hectare}, context)
        return True

    def calcula_madeira_rolica_por_hectare(self, cr, uid, brw_calcmadeira_calcula, qtd_arvores,
                                           context=None):
        context = context and context.copy() or {}
        arvore_por_hectare = brw_calcmadeira_calcula.arvore_por_hectare
        if arvore_por_hectare <= 0:
            return False
        cr.execute("""DELETE FROM wizard_calcmadeira_calcula_itens_hectare 
                      WHERE wcahec_capa_id = %s""" % brw_calcmadeira_calcula.id)
        list_itens_hectare = []
        for item in brw_calcmadeira_calcula.peca_ids:
            num_tora_ha = int(item.numtora * arvore_por_hectare / qtd_arvores)
            vol_tora_ha = (item.voltora * arvore_por_hectare / qtd_arvores)
            # num_peca_ha = int(item.numpeca * arvore_por_hectare / qtd_arvores)
            # vol_peca_ha = item.volpeca * arvore_por_hectare / qtd_arvores
            preco_ha = int(item.preco_item * arvore_por_hectare / qtd_arvores)
            vals_itens_hectare = {'wcahec_codigo_peca': item.codpeca,
                                  'wcahec_nome_peca': item.nomepeca,
                                  # 'wcahec_numero_peca': num_peca_ha,
                                  'wcahec_numero_tora': num_tora_ha,
                                  'wcahec_preco': preco_ha,
                                  # 'wcahec_volume_peca': vol_peca_ha,
                                  'wcahec_volume_tora': vol_tora_ha}
            list_itens_hectare.append((0, 0, vals_itens_hectare))
        list_itens_hectare and self.write(
            cr, uid, brw_calcmadeira_calcula.id, {'itens_hectare_ids': list_itens_hectare}, context)
        return True

    def calcmadeira_valida_parametro_laminacao(self, cr, uid, parametro_dados, context=None):
        context = context and context.copy() or {}
        res = True
        msg = ''
        obj_calcmadeira_toras = self.pool.get('calcmadeira.toras')
        toras_ids = obj_calcmadeira_toras.search(cr, uid, [], order='cmator_diametro_min', limit=1,
                                                 context=context)
        menor_diametro_tora = toras_ids and obj_calcmadeira_toras.read(
            cr, uid, toras_ids[0], ['cmator_diametro_min'], context).get('cmator_diametro_min') or 0
        if parametro_dados.get('cmapar_laminacao_diametro_minimo') < 0 \
                or parametro_dados.get('cmapar_laminacao_diametro_minimo') > menor_diametro_tora:
            msg += _('"Parte do diametro nao aproveitavel (rolo resto)" nao pode ser '
                     'menor que zero ou maior que o "Diametro minimo" do cadastro de toras.\n')
        if parametro_dados.get('cmapar_laminacao_espessura') <= 0 or parametro_dados.get(
                'cmapar_laminacao_espessura') > 100:
            msg += _('"Espessura da folha" deve ser maior que zero e menor ou igual a 100.\n')
        if parametro_dados.get('cmapar_laminacao_comp_folha') <= 0 or parametro_dados.get(
                'cmapar_laminacao_comp_folha') > 10:
            msg += _('"Comprimento da folha" deve ser maior que zero e menor ou igual a 10.\n')
        if msg:
            raise osv.except_osv(_('Atencao'), 'Parametros:\n' + msg)
        return res

    def calcmadeira_exporta_dados_txt(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_resultado_peca = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                         context).result_ids
        if not brw_resultado_peca:
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para serem exportados!'))
        str_arquivo = 'Arvore\tTora\tdi_casc2\tdi_casc\thi\r\n'
        for det in brw_resultado_peca:
            str_arquivo += '%s\t%s\t%s\t%s\t%s\r\n' % (det.arv, det.numtora, det.di_casc2,
                                                       det.di_casc, det.comprimento)
        str_arquivo and self.write(cr, uid, ids, {'arquivo_tora': base64.encodestring(str_arquivo)},
                                   context)
        return True


class wizard_calcula_peca_rolica_itens(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula.itens'
    _columns = {
        'peca_id': fields.many2one('wizard.calcmadeira.calcula', 'Peca'),
        'codpeca': fields.integer('Codigo peca'),
        'nomepeca': fields.char('Nome peca', size=40),
        'numtora': fields.integer('Numero tora'),
        'voltora': fields.float('Volume tora', digits=(3, 4)),
        'numpeca': fields.integer('Numero peca'),
        'volpeca': fields.float('Volume peca', digits=(3, 4)),
        'preco_item': fields.float('Preco', digits=(5, 2)),
    }
    _order = 'codpeca'


class wizard_resultado_peca(osv.TransientModel):
    _name = 'wizard.calcmadeira.resultado.peca'
    _columns = {
        'result_id': fields.many2one('wizard.calcmadeira.calcula', 'Resultado'),
        'codpeca': fields.integer('Codigo peca'),
        'nomepeca': fields.char('Nome peca', size=40),
        'es_serra': fields.float('es', digits=(3, 2)),
        'l': fields.float('l', digits=(3, 2)),
        'n': fields.integer('n'),
        'v': fields.float('v', digits=(3, 4)),
        'numtora': fields.integer('numtora'),
        'vr': fields.float('Vr', digits=(3, 4)),
        'di_casc2': fields.float('di_casc2', digits=(3, 2)),
        'di_casc': fields.float('di_casc', digits=(3, 2)),
        'di': fields.float('di', digits=(3, 2)),
        'arv': fields.integer('arv'),
        'fatia': fields.integer('fatia'),
        'l_sobra': fields.float('l_sobra', digits=(3, 2)),
        'es_sobra': fields.float('es_sobra', digits=(3, 2)),
        'comp_laminacao': fields.float('comp_lamin', digits=(3, 2)),
        'ffarv': fields.float('FFarv', digits=(3, 4)),
        'conicidade': fields.float('Conicidade', digits=(3, 4)),
        'rendimento': fields.float('Rendimento', digits=(3, 4)),
        'preco': fields.float('Preco', digits=(5, 2)),
        'comprimento': fields.float('compr.', digits=(2, 1)),
    }


class wizard_calcmadeira_calcula_itens_hectare(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula.itens.hectare'
    _columns = {
        'wcahec_capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'wcahec_codigo_peca': fields.integer('Codigo peca'),
        'wcahec_nome_peca': fields.char('Nome peca', size=40),
        'wcahec_numero_peca': fields.integer('Numero peca'),
        'wcahec_numero_tora': fields.integer('Numero tora'),
        'wcahec_preco': fields.float('Preco', digits=(5, 2)),
        'wcahec_volume_peca': fields.float('Volume peca', digits=(3, 4)),
        'wcahec_volume_tora': fields.float('Volume tora', digits=(3, 4)),
    }
#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import math
from openerp.osv import osv, fields
import numpy as np
from numpy.linalg import inv


class Eqvt():
    def __init__(self, B0, B1, B2,dap,ht):
        self.B0 = B0
        self.B1 = B1
        self.B2 = B2
        self.dap = dap
        self.ht = ht

    def vt_result(self):
        vt=math.exp(self.B0 +self.B1 * math.log(self.dap) + self.B2 * math.log(self.ht))
        return vt


#  Classe para definir altura comercial a partir do menor diametro (que compensa serrar)
class Eqhi():
    def __init__(self, B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite):
        self.B0 = B0f_hi
        self.B1 = B1f_hi
        self.B2 = B2f_hi
        self.hi_atual=hi_atual
        self.hi=hi
        self.ht = ht
        self.dap = dap
        self.di_limite=di_limite

    def hi_result(self):

        if self.di_limite + 0.5 < self.dap:
            # calcula di com casca(di_casc) na altura atual (hi_atual)
            hi = math.sqrt(
                self.B0+self.B1 * (self.di_limite / self.dap) + self.B2 * (self.di_limite / self.dap) ** 2) * self.ht
        else:
            hi = 0

        return hi
# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------


# Formulas
def equacao_volume(dap, ht, b0=0, b1=0, b2=0):
    # equação volume por árvore ILPF GG100 2011
    vt = math.exp(b0 + b1 * math.log(dap) + b2 * math.log(ht))

    # equação volume por árvore ILPF MARAVILHAS 2009
    # vt = math.exp(-10.5312815270227 + 1.7700934601883 * math.log(dap) + 1.29205471199543 * math.log(ht))
    return vt


class Eqdi_casc():
    def __init__(self, B0, B1, B2, hi_atual, hi, ht, dap, di_casc, di_casc2):
        self.B0 = B0
        self.B1 = B1
        self.B2 = B2
        self.hi_atual=hi_atual
        self.hi=hi
        self.ht = ht
        self.dap = dap
        self.di_casc=di_casc
        self.di_casc2=di_casc2

    def di_casc2_result(self):
        # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
        di_casc2 = math.sqrt(self.B0 +self.B1* ((self.hi_atual + self.hi) / self.ht) + self.B2 * (
                    (self.hi_atual + self.hi) / self.ht) ** 2) * self.dap

        return di_casc2

    def di_casc_result(self):

        # calcula di com casca(di_casc) na altura atual (hi_atual)
        di_casc = math.sqrt(
            self.B0+self.B1 * (self.hi_atual / self.ht) + self.B2 * (self.hi_atual / self.ht) ** 2) * self.dap

        return di_casc


# ----------------------------------------------------------------------------------------------------------------

# Equacao de Taper
def equacao_forma(hi_atual, hi, ht, dap, di_casc, di_casc2, b0=0, b1=0, b2=0):
    # if eqUsuario == 1 and eqForma == 1:

    # 'EQUAÇÃO TAPER (KOZAK)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = Sqr(Range("n2") + Range("n3") * ((hi_atual + hi) / ht) + Range("n4") * ((hi_atual + hi) / ht) ^ 2) * Dap

    # 'calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (Range("n2") + Range("n3") * (hi_atual / ht) + Range("n4") * (hi_atual / ht) ^ 2) * Dap
    # Else

    # EQUAÇÃO TAPER (KOZAK) ILPF GG100 2011
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    try:
        di_casc2 = math.sqrt(
            b0 - b1 * ((hi_atual + hi) / ht) + b2 * (
                        (hi_atual + hi) / ht) ** 2) * dap

        # calcula di com casca(di_casc) na altura atual (hi_atual)
        di_casc = math.sqrt(
            b0 - b1 * (hi_atual / ht) + b2 * (
                        hi_atual / ht) ** 2) * dap
        return di_casc, di_casc2
    except Exception:
        raise osv.except_osv(('Atencao'),
                             ('Dados de entrada para calculo incorreto'))

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV1 (CITRIODORA GUARITA 2) Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)

    # if hi != 0:
    #     di_casc2 = (1.04317139756795 - 1.15251507113576 * ((hi_atual + hi) / ht) + 0.745608873884745 * math.pow((
    #     (hi_atual + hi) / ht), 2)) * dap
    # else:
    #     #calcula di com casca(di_casc) na altura atual (hi_atual)
    #     di_casc = (1.04317139756795 - 1.15251507113576 * (hi_atual / ht) + 0.745608873884745 * math.pow((hi_atual / ht), 2)) * dap

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV2 (CITRIODORA GUARITA 2) Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = (1.10041792322808 - 1.46182396963 * ((hi_atual + hi) / ht) + 0.77535958077 * ((hi_atual + hi) / ht) ^ 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (1.10041792322808 - 1.46182396963 * (hi_atual / ht) + 0.77535958077 * (hi_atual / ht) ^ 2) * dap

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV3 (CITRIODORA GUARITA 2) s/toco Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = (1.04934898366772 - 1.01123933349793 * ((hi_atual + hi) / ht) - 0.231797825742175 * ((hi_atual + hi) / ht) ^ 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (1.04934898366772 - 1.01123933349793 * (hi_atual / ht) - 0.231797825742175 * (hi_atual / ht) ^ 2) * dap

    # EQUAÇÃO TAPER (KOZAK) MARAVILHAS s/toco Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # if hi!=0:
    # di_casc2 = math.sqrt(1.03698591927493 - 1.45223087205717 * ((hi_atual + hi) / ht) + 0.428220947928028 * ((hi_atual + hi) / ht) ** 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # else:
    # di_casc = math.sqrt(1.03698591927493 - 1.45223087205717 * (hi_atual / ht) + 0.428220947928028 * (hi_atual / ht) ** 2) * dap

# ----------------------------------------------------------------------------------------------------------------

def formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s):
    # Formulas metodo Radial

    # angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # 1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)
    # angulos direito, esquerdo (Fatia0), inferior, superior (Fatia1)
    # cordas direita, esquerda (Fatia0), inferior, superior (Fatia1)
    # larguras esquerda, direita, inferior, superior da peça (atualiza com desconto da espessura da peça)

    if fatia == 0:

        ang_e = math.acos(nes1 / (di / 2)) * 180 / math.pi
        ang_d = math.acos((nes1 + es-es_serra) / (di / 2)) * 180 / math.pi

        cord_e = di * math.sin(ang_e * math.pi / 180)
        cord_d = di * math.sin(ang_d * math.pi / 180)

        l_e = (cord_e / 2) - nes2
        l_d = (cord_d / 2) - nes2

    elif fatia == 1:

        ang_i = math.asin(nes2 / (di / 2)) * 180 / math.pi
        ang_s = math.asin((nes2 + es-es_serra) / (di / 2)) * 180 / math.pi

        cord_i = di * math.cos(ang_i * math.pi / 180)
        cord_s = di * math.cos(ang_s * math.pi / 180)

        l_i = (cord_i / 2) - nes1
        l_s = (cord_s / 2) - nes1

    return ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s


# volume da peça
def calc_v(es_serra, es, l_d, l_e, l_s, l_i, hi):
    if l_s == 0:
        v = ((es - es_serra) / 100) * ((l_d + l_e) / 2) / 100 * hi
    else:
        v = ((es - es_serra) / 100) * ((l_s + l_i) / 2) / 100 * hi
    return v


# SubRotinas
def testa_espessura(es, es_sobra, es_serra, min_es):
    while es > es_sobra:
        es = es - 5
        if es < min_es + es_serra:
            break
    return es


def testa_largura(l_div, l_sobra, l_min, es_serra):
    while l_div > l_sobra:
        l_div = l_div - 5
        if l_div < l_min + es_serra:
            break
    return l_div


def testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra):
    for inv_l in (0, (l_max - l_min) + 1, 5):
        l_div = l_max + es_serra - inv_l

        while l_div > l_sobra:
            l_div = l_div - 5

            if l_div < l_min:
                break

        return l_div


def RelatorioPecaRolica(a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk,
                        codpeca, vr):
    if codpeca == 1:
        a = a + 1
        vra = vra + vr

    elif codpeca == 2:
        b = b + 1
        vrb = vrb + vr

    elif codpeca == 3:
        c = c + 1
        vrc = vrc + vr

    elif codpeca == 4:
        d = d + 1
        vrd = vrd + vr

    elif codpeca == 5:
        e = e + 1
        vre = vre + vr

    elif codpeca == 6:
        f = f + 1
        vrf = vrf + vr

    elif codpeca == 7:
        g = g + 1
        vrg = vrg + vr

    elif codpeca == 8:
        h = h + 1
        vrh = vrh + vr

    elif codpeca == 9:
        k = k + 1
        vrk = vrk + vr

    return a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk


# ----------------------------------------------------------------------------------------------------------------

def RelatorioPeca(a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                  e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk,
                  codpeca, vr, n, n1, v, v1):
    if codpeca == 1:
        na = na + n + n1
        va = va + v + v1

        if vr > 0:
            a = a + 1
            vra = vra + vr
            # relatório por tora
            print('a:{}, vr:{}'.format(a, vr))
            vr = 0

    elif codpeca == 2:
        nb = nb + n + n1
        vb = vb + v + v1

        if vr > 0:
            b = b + 1
            vrb = vrb + vr
            print('b:{}, vr:{}'.format(b, vr))
            vr = 0

    elif codpeca == 3:
        nc = nc + n + n1
        vc = vc + v + v1

        if vr > 0:
            c = c + 1
            vrc = vrc + vr
            print('c:{}, vr:{}'.format(c, vr))
            vr = 0

    elif codpeca == 4:
        nd = nd + n + n1
        vd = vd + v + v1

        if vr > 0:
            d = d + 1
            vrd = vrd + vr
            print('d:{}, vr:{}'.format(d, vr))
            vr = 0

    elif codpeca == 5:
        ne = ne + n + n1
        ve = ve + v + v1

        if vr > 0:
            e = e + 1
            vre = vre + vr
            print('e:{}, vr:{}'.format(e, vr))
            vr = 0

    elif codpeca == 6:
        nf = nf + n + n1
        vf = vf + v + v1

        if vr > 0:
            f = f + 1
            vrf = vrf + vr
            print('f:{}, vr:{}'.format(f, vr))
            vr = 0

    elif codpeca == 7:
        ng = ng + n + n1
        vg = vg + v + v1

        if vr > 0:
            g = g + 1
            vrg = vrg + vr
            print('g:{}, vr:{}'.format(g, vr))
            vr = 0

    elif codpeca == 8:
        nh = nh + n + n1
        vh = vh + v + v1

        if vr > 0:
            h = h + 1
            vrh = vrh + vr
            print('h:{}, vr:{}'.format(h, vrh))
            vr = 0

    elif codpeca == 9:
        nk = nk + n + n1
        vk = vk + v + v1

        if vr > 0:
            k = k + 1
            vrk = vrk + vr
            print('k:{}, vr:{}'.format(k, vr))
            vr = 0

    return a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd, \
           e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk, \
           codpeca, vr


# ----------------------------------------------------------------------------------------------------------------

def RelatorioFinalRolica(a1, vra1, b1, vrb1, c1, vrc1, d1, vrd1, e1, vre1, f1, vrf1, g1, vrg1, h1,
                         vrh1, k1, vrk1, \
                         a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk):
    # transfere as peças contabilizadas da arvore para a variável final,
    # que armazena peças de todas as árvores
    a1 = a1 + a
    vra1 = vra1 + vra
    b1 = b1 + b
    vrb1 = vrb1 + vrb
    c1 = c1 + c
    vrc1 = vrc1 + vrc
    d1 = d1 + d
    vrd1 = vrd1 + vrd
    e1 = e1 + e
    vre1 = vre1 + vre
    f1 = f1 + f
    vrf1 = vrf1 + vrf
    g1 = g1 + g
    vrg1 = vrg1 + vrg
    h1 = h1 + h
    vrh1 = vrh1 + vrh
    k1 = k1 + k
    vrk1 = vrk1 + vrk

    vrt = vra1 + vrb1 + vrc1 + vrd1 + vre1 + vrf1 + vrg1 + vrh1 + vrk1

    return a1, vra1, b1, vrb1, c1, vrc1, d1, vrd1, e1, vre1, f1, vrf1, g1, vrg1, h1, vrh1, k1, vrk1, vrt


# ----------------------------------------------------------------------------------------------------------------

def RelatorioFinal(a1, vra1, na1, va1, b1, vrb1, nb1, vb1, c1, vrc1, nc1, vc1, d1, vrd1, nd1, vd1,
                   e1, vre1, ne1, ve1, f1, vrf1, nf1, vf1, g1, vrg1, ng1, vg1, h1, vrh1, nh1, vh1,
                   k1, vrk1, nk1, vk1,
                   a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                   e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk):
    # transfere peças calculadas da arvore para a variável final, que armazena peças de todas as árvores da amostra
    a1 = a1 + a
    vra1 = vra1 + vra
    na1 = na1 + na
    va1 = va1 + va

    b1 = b1 + b
    vrb1 = vrb1 + vrb
    nb1 = nb1 + nb
    vb1 = vb1 + vb

    c1 = c1 + c
    vrc1 = vrc1 + vrc
    nc1 = nc1 + nc
    vc1 = vc1 + vc

    d1 = d1 + d
    vrd1 = vrd1 + vrd
    nd1 = nd1 + nd
    vd1 = vd1 + vd

    e1 = e1 + e
    vre1 = vre1 + vre
    ne1 = ne1 + ne
    ve1 = ve1 + ve

    f1 = f1 + f
    vrf1 = vrf1 + vrf
    nf1 = nf1 + nf
    vf1 = vf1 + vf

    g1 = g1 + g
    vrg1 = vrg1 + vrg
    ng1 = ng1 + ng
    vg1 = vg1 + vg

    h1 = h1 + h
    vrh1 = vrh1 + vrh
    nh1 = nh1 + nh
    vh1 = vh1 + vh

    k1 = k1 + k
    vrk1 = vrk1 + vrk
    nk1 = nk1 + nk
    vk1 = vk1 + vk

    vrt = vra1 + vrb1 + vrc1 + vrd1 + vre1 + vrf1 + vrg1 + vrh1 + vrk1

    return a1, vra1, na1, va1, b1, vrb1, nb1, vb1, c1, vrc1, nc1, vc1, d1, vrd1, nd1, vd1, \
           e1, vre1, ne1, ve1, f1, vrf1, nf1, vf1, g1, vrg1, ng1, vg1, h1, vrh1, nh1, vh1, k1, vrk1, nk1, vk1, vrt


# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------

def pecamenor_quadcircun(l_min, l_max, codpeca, hi, es, es_serra, es_sobra, l, l_sobra, n_h,
                         pecas_serradas_preco_dados, es_medula, escolha, cont_medula):
    n1 = 0
    v1 = 0.0000

    l_div = l_min+es_serra

    preco = 0.0000

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco2 = 0.0000
    preco_medula = 0.0000

    dados_impressao = []
    while l_sobra >= l_min + es_serra:

        # min_es não pode ser menor do que es_serra!
        if l_div > l_sobra or l_div < l_min + es_serra:
            break

        for inv_l in range(0, ((l_max - l_min)+1), 5):
            l_div = l_max + es_serra - inv_l

            # ---------------------------
            # Testa Largura
            l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
            # ---------------------------

            # min_es não pode ser menor do que es_serra!
            if l_div > l_sobra or l_div < l_min + es_serra:
                break

            # numero de peças na vertical
            n_div = math.trunc(l_sobra / l_div)

            # numero de peças total por laço
            n_div1 = n_h * n_div

            # numero de peças total
            n1 = n1 + n_div1

            # volume de peças na horizontal e vertical
            v = (n_h * (es - es_serra) / 1000.0) * (n_div * (l_div - es_serra) / 1000.0) * hi
            v1 = v1 + v

            es_tmp = es
            l_tmp = l_div
            n_tmp = n_div1
            v_tmp = v
            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
            preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                    hi, v_tmp)

            preco_menor = preco_menor + preco_peca

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
            #       format(codpeca, es, l_div, n_div1, v, l_sobra))

            if es_sobra > es + es_medula and es_medula > es_serra \
                    and cont_medula == 0:
                n_div_medula = 1
                v_medula = (es_medula - es_serra) / 1000 * (
                            n_div_medula * (l_div - es_serra) / 1000) * hi

                v1 = v1 + v_medula
                n1 = n1 + n_div_medula

                # Relatório por peça na MEDULA
                # print('peca_medul.:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
                #       format(codpeca, es_medula, l_div, 1, v_medula, l_sobra))

                es_tmp = es_medula
                l_tmp = l_div
                n_tmp = n_div_medula
                v_tmp = v_medula
                # -----------------------------------------------------------------
                # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                        hi, v_tmp)
                # -----------------------------------------------------------------

                preco_medula = preco_medula + preco_peca

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}, preco_peca:{}, preco:{}' \
            #       .format(codpeca, es, l_div, n_div1, v, l_sobra, preco_peca, preco_medula))

            # largura restante (eixo vertical do bloco)
            l_sobra = l_sobra - (l_div * n_div)

            dados_impressao.append({'l_div': l_div, 'n_div': n_div1, 'v': v, 'l_sobra': l_sobra,
                                    'preco': preco_peca})
            # v=0

            # L sobra
            # print('l_sobra:{}'.format(l_sobra))

    if escolha in ('13', '14'):  # '7' - Gira 180 90
        preco2 = preco_menor + preco_medula
    else:
        preco2 = preco_menor + preco_menor2

    return l_sobra, n1, v1, dados_impressao, preco2


# ---------------------------------------------------------------------------------------------

def pecamenor2_14dcostan(n1, v1, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                         pecas_serradas_preco_dados):

    # ---------------------------
    # Testa Largura
    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
    # ---------------------------

    # está embutido na função TestaLargura...
    if l_div > l_sobra or l_div < l_min + es_serra:
        n_div = 0
        v = 0
        preco_menor2 = 0
        dados_impressao = {'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                           'preco': preco_menor2}
        return l_div, l_sobra, n_div, n1, v1, dados_impressao, preco_menor2
        # return l_div == 0

    # numero de peças na vertical
    n_div = math.trunc(l_sobra / l_div)

    # numero de peças total
    n1 += n_div

    # volume da peça dividida em subpeças
    v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
    v1 += v

    # largura restante (eixo vertical)
    l_sobra = l_sobra - (l_div * n_div)

    es_tmp = es
    l_tmp = l_div
    n_tmp = n_div
    v_tmp = v
    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
    preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                            hi, v_tmp)

    preco_menor2 = preco_peca

    # Relatório por peça
    print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}'. \
          format(codpeca, es, l_div, n_div, v, l_sobra))
    dados_impressao = {'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                       'preco': preco_peca}
    v = 0

    return l_div, l_sobra, n_div, n1, v1, dados_impressao, preco_menor2

# -------------------------------------------------------------------------------------------------------


def pecamenor_long_1costaneirabase(l_min, l_max, codpeca, di, hi, es, es_serra,l_sobra, n_h,
                                   flecha0,flecha,
                                   ang_e, ang_d, cord_e, cord_d,r_flecha_base,
                                   pecas_serradas_preco_dados, parametro_dados,
                                   flecha_div_r, ang_base, escolha):
    n1 = 0
    v1 = 0.0000
    cord_e1 = 0.00
    dados_impressao = []

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco_menor2_14D = 0.0000
    es1 = 0

    # aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h + 1):
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div > l_sobra or l_div < l_min + es_serra:
            es1 = es
            break

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                hi, v_tmp)

        preco_menor = preco_menor + preco_peca

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}, preco_peca:{}, '
              'preco_menor:{}'.format(codpeca, es, l_div, n_div, v, l_sobra, preco_peca,
                                      preco_menor))
        dados_impressao.append({'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                                'preco': preco_peca})
        v = 0

        if l_sobra >= l_min + es_serra:
            l_div2 = l_div
            n_div2 = n_div

            # --------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1, impressao, preco_menor2 \
                = pecamenor2_14dcostan(n1, v1, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                                       pecas_serradas_preco_dados)
            dados_impressao.append(impressao)
            # --------------------------------------------------------
            l_div = l_div2
            n_div = n_div2

            preco_menor2_14D = preco_menor2_14D + preco_menor2


        if x <= n_h:

            # incremento da flecha com espessura da nova peça
            flecha = flecha + es

            if x == n_h:
                es1 = es
                es = es_serra

            if escolha in ('13', '14'):  # '7' - Gira 180 90
                cord_e1 = cord_e

            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = formulas_costaneirabase(
                flecha, es, es_serra, di, ang_base, r_flecha_base, escolha, parametro_dados)
            # -----------------------------------------------

            if flecha <= di / 2  or escolha in ('13', '14'):  # '7' - Gira 180 90
                l_sobra = cord_d
            else:
                l_sobra = cord_e

            # 'limite para corte a direita, para giro de 180 graus da tora
            if escolha == '7' and flecha / (di / 2) >= flecha_div_r:
                break

    preco2 = preco_menor + preco_menor2_14D

    es = es1

    return l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, cord_e, cord_d, \
           r_flecha_base, dados_impressao, preco2, cord_e1


# ---------------------------------------------------------------------------------------------------------

def pecamenor_long_2costaneirabase(l_min, l_max, codpeca, di, hi, es, es_serra,l_sobra, n_h,flecha0,flecha,
                        ang_e, ang_d, cord_e, cord_d,r_flecha_base):
    n1=v1=0
    l_div=0.0000
    n_div=0
    #aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h+1):
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div < l_min + es_serra:
            break

        #numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        #numero de peças total
        n1 += n_div

        #volume da peça dividida em subpeças
        v = ((es - es_serra) / 100) * n_div * (l_div - es_serra) / 100 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}'\
              .format(codpeca, es, l_div, n_div,  v, l_sobra))
        v = 0

        if l_sobra - es_serra >= l_min:
            l_div2 = l_div
            n_div2 = n_div

            #--------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1\
            = pecamenor2_14dcostan(n1,v1,l_min, l_max, codpeca, hi, es, es_serra, l_sobra)
            #--------------------------------------------------------
            l_div = l_div2
            n_div = n_div2

        if flecha+es<=di-flecha0:
            #incremento da flecha com espessura da nova peça
            flecha = flecha + es
        else:
            break

        if x <n_h:
            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = formulas14d(flecha, es, di)
            # -----------------------------------------------
            if flecha <= di / 2:
                l_sobra = cord_d
            else:
                l_sobra = cord_e

    return l_div, n_div, l_sobra ,n1, v1,flecha, ang_e, ang_d, cord_e, cord_d, r_flecha_base

def pecamenor_fatia(fatia, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                    pecas_serradas_preco_dados):
    n1 = 0
    v1 = 0.0000
    l_div = 0.0000
    n_div = 0

    preco1 = 0.0000

    dados_impressao = []
    while l_sobra >= l_min + es_serra:
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                hi, v_tmp)

        preco1 = preco1 + preco_peca

        # Relatório por peça
        # print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, fatia:{}, l_sobra:{}' \
        #       .format(codpeca, es, l_div, n_div, v, fatia, l_sobra))
        # v = 0
        #
        # # aviso de troca de fatia (selecionou peça)
        # peca = 1
        dados_impressao.append({'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                                'fatia': fatia, 'preco': preco_peca})

    return fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco1


# def formulas14d( flecha, es, es_serra, di):
#
#     # 'angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
#     # '1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)
#
#     # 'angulos direito e esquerdo com a nova peça (antes do diametro, usa ang_d, após, usa ang_e
#     ang_e = 180 - math.acos((es-es_serra + flecha - di / 2) / (di / 2)) * 180 / math.pi
#     ang_d = 180 - math.acos((flecha - di / 2) / (di / 2)) * 180 / math.pi
#
#     # 'Cordas esquerda e direita da peça
#     cord_e = di * math.sin(ang_e * math.pi / 180)
#     cord_d = di * math.sin(ang_d * math.pi / 180)
#
#     return  ang_e, ang_d, cord_e, cord_d


def formulas_costaneirabase(flecha, es, es_serra, di, ang_base, r_flecha_base, escolha,
                            parametro_dados):

    # 'angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # '1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)

    # 'cos(180-alfa) = (es + flecha - di / 2) / (di / 2)
    # 'acos((es + flecha - di / 2) / (di / 2))=180-alfa
    # 'alfa = 180 - acos((es + flecha - di / 2) / (di / 2))

    # 'angulos direito e esquerdo com a nova peça (antes do diametro, usa ang_d, após, usa ang_e
    ang_e = 180 - math.acos((es -es_serra + flecha - di / 2) / (di / 2)) * 180 / math.pi
    ang_d = 180 - math.acos((flecha - di / 2) / (di / 2)) * 180 / math.pi

     # 'Cordas esquerda e direita da peça
    if ang_d < (90 - ang_base) or ang_d > (90 + ang_base):
        cord_d = di * math.sin(ang_d * math.pi / 180)
    else:
        cord_d = di * math.sin(ang_d * math.pi / 180) / 2 + r_flecha_base

    if ang_e < (90 - ang_base) or ang_e > (90 + ang_base):
        cord_e = di * math.sin(ang_e * math.pi / 180)
    else:
        cord_e = di * math.sin(ang_e * math.pi / 180) / 2 + r_flecha_base

    return ang_e, ang_d, cord_e, cord_d


def calcmadeira_get_totais(self, cr, uid, ids, field_name, arg, context=None):
    """
    Apura o rendimento no calculo das madeiras.
    Para a peca rolica, o rendimento e calculado com o volume total da tora divido pelo volume
    total multiplicado por 100.
    Para as demais pecas, o rendimento e calculado com o volume total da peca divido pelo volume
    total da tora multiplicado por 100.
    :param cr: cursor no banco de dados.
    :type cr: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type uid: int
    :param ids: lista de id da tabela do objeto wizard_calcmadeira_calcula.
    :type ids: list
    :param field_name: descricao do atributo funcional.
    :type field_name: str
    :param arg: lista de argumentos de pesquisa.
    :type arg: list
    :param context: dados contextuais.
    :type context: dict
    :return: dados da identificacao do registro e seu respectivo valor.
    :rtype: dict
    """
    context = context and context.copy()
    res = {}
    for calc in self.browse(cr, uid, ids, context):
        res.update({calc.id: {'vr_rendimento': 0}})
        if context.get('tipo_funcao') == '1' and calc.vt1:
            res[calc.id].update({'vr_rendimento': (sum(map(
                lambda ii: ii.voltora or 0, calc.peca_ids)) / calc.vt1) * 100.0})
        if context.get('tipo_funcao') != '1' and (calc.vt1 or calc.total_vt_vr) \
                and sum(map(lambda ii: ii.voltora or 0, calc.peca_ids)):
            res[calc.id].update({'vr_rendimento': (
                sum(map(lambda ii: ii.volpeca or 0, calc.peca_ids)) /
                sum(map(lambda ii: ii.voltora or 0, calc.peca_ids))
                )
                * 100.0}
            )
        preco_total = sum(map(lambda i: i.preco_item or 0, calc.peca_ids))
        volume_peca_total = sum(map(lambda i: i.volpeca or 0, calc.peca_ids))
        preco_peca = volume_peca_total and preco_total / volume_peca_total or 0
        volume_tora_total = sum(map(lambda i: i.voltora or 0, calc.peca_ids))
        preco_tora = volume_tora_total and preco_total / volume_tora_total or 0
        if calc.tipo_funcao == '1':  # Rolica
            preco_tora = 0
        preco_arvore = calc.vt1 and preco_total / calc.vt1 or 0
        quantidade_tora = 0
        if context.get('funcoes_serrada_tora'):
            preco_arvore = 0
            quantidade_tora = sum(map(lambda i: i.numtora or 0, calc.peca_ids))
        res[calc.id].update({'preco_total': preco_total, 'preco_peca': preco_peca,
                             'preco_tora': preco_tora, 'preco_arvore': preco_arvore,
                             'quantidade_tora': quantidade_tora})
    return res


def calcmadeira_check_calculo_receita(self, cr, uid, ids, field_name, arg, context=None):
    context = context and context.copy()
    permite_calculo_receita = True
    if any([context.get('calculo_rolica'), context.get('calculo_laminada_arvore'),
            context.get('calculo_laminada_tora'), context.get('calculo_fator_forma')]):
        permite_calculo_receita = False
    res = {}.fromkeys(ids, permite_calculo_receita)
    return res


def forma(forma_dados):

    print('Ajusta Equação de Forma')

    # --------------------
    lintaper, ntaper = forma_dados, len(forma_dados)
    # --------------------

    # numero da árvore
    dado = 0
    dap = 0

    # a = vetor de '1'
    # 'x1= hi/ht
    # 'x2= (hi/ht)**2
    # 'y= (di_casc/dap)**2

    a = []
    x1 = []
    x2 = []
    y = []

    # Acessa arquivo texto, armazena dap e ht por arvore
    taper = 1

    # Executa por seção
    i = 1
    hi = 0
    hi1 = 1

    while taper < ntaper:

        while hi < hi1:
            dados = lintaper[taper - 1]
            hi = float(dados.get('cmatap_hi'))

            if taper == ntaper:
                pass
            else:
                try:
                    dados1 = lintaper[taper]
                    hi1 = float(dados1.get('cmatap_hi'))
                except:
                    print('fim arquivo')
                    hi1 = 0.3
            try:
                arv = int(dados.get('cmatap_arv'))
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))
                ht = float(dados.get('cmatap_ht'))

                print('hi={}, hi1={}/ ht={}'.format(hi, hi1, ht))

            except:
                print('ainda não leu altura total (ht)')

            if hi == 1.3:
                dap = di_casc

            taper = taper + 1

        for c in range(i - 1, taper - 1, 1):
            dados = lintaper[c]
            try:
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))

            except:
                print('arquivo Taper.txt com campos incompletos')

            try:
                x1.append(hi / ht)
                x2.append((hi / ht) ** 2)
                y.append((di_casc / dap) ** 2)

                print('hi/ht:{}'.format(hi / ht))
            except:
                print('falta algum dado no Taper.txt')

            hi = 0
            hi1 = 1
        i = taper

    dado = ntaper
    dado1 = dado

    a = dado1 * [1]
    print('a=', a)
    print('x1f=', x1)
    print('x2f=', x2)
    print('yf=', y)
    # --------------------------
    B, r2 = regressao(dado, a, x1, x2, y)
    # --------------------------

    B0 = B[0]
    B1 = B[1]
    B2 = B[2]

    print('b0={}, b1={}, b2={}'.format(B0, B1, B2))

    return B0, B1, B2, r2


def volume(volume_dados):
    print('Ajusta Equação de Volume')

    # --------------------
    linarv, narv = volume_dados, len(volume_dados)
    # --------------------

    # numero da árvore
    arv = 0
    n_falha = 0
    dado = 0

    # a = vetor de '1'
    # 'x1=ln(dap)
    # 'x2=ln(ht)
    # 'y=ln(vol)

    a = []
    x1 = []
    x2 = []
    y = []

    # Executa por árvore
    while arv < narv:
        try:
            # Acessa arquivo texto, armazena dap e ht por árvore
            dados = linarv[arv]
            dap = float(dados.get('cmacub_diametro'))
            ht = float(dados.get('cmacub_altura'))
            vol = float(dados.get('cmacub_volume'))
        except:
            print('arquivo Arvore.txt com campos incompletos. Vol?')

        if dap == "" or ht == "":
            n_falha = n_falha + 1
        else:
            x1.append(math.log(dap))
            x2.append(math.log(ht))
            y.append(math.log(vol))

        arv = arv + 1

    dado = narv - n_falha

    a = dado * [1]

    # --------------------------
    B, r2 = regressao(dado, a, x1, x2, y)
    # --------------------------

    B0 = B[0]
    B1 = B[1]
    B2 = B[2]

    print('b0={}, b1={}, b2={}'.format(B0, B1, B2))

    # print('x1:{}. x2:{}'.format(x1[1],x2[1]))

    # 'EQUAÇÃO Volume (Schumacher and Hall)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # volest = math.exp(B0 + B1 * x1[1] + B2 * x2[1])
    # print('Volume: {}'.format(volest))

    return B0, B1, B2, r2


def regressao(dado, a, x1, x2, y):
    Y = np.array(y)
    Xt = np.array([a, x1, x2])
    X = Xt.transpose()

    # dimensoes(matriz):
    tam_Xt = (len(Xt), len(Xt[0]))
    print('{} X {}'.format(tam_Xt[0], tam_Xt[1]))

    tam_X = (len(X), len(X[0]))
    print('{} X {}'.format(tam_X[0], tam_X[1]))

    XtX = Xt.dot(X)
    XtXinv = inv(XtX)
    XtY = Xt.dot(Y)
    B = XtXinv.dot(XtY)
    Bt = B.transpose()
    Yt = Y.transpose()
    YtY = Yt.dot(Y)
    BtXtY = Bt.dot(XtY)

    cte = sum(Y) ** 2 / dado

    SQRegres = BtXtY - cte
    SQTotal = YtY - cte
    SQRes = SQTotal - SQRegres
    print(sum(y))

    if SQTotal != 0:
        r2 = SQRegres / SQTotal
        print('r2: {}'.format(r2))
        # print('B0:{},B1:{}, B2:{}'.format(B[0],B[1],B[2]))

    else:
        r2 = 0
        print('r2: {}'.format(r2))

    # B,r2=Regressao(a,Xt,Y)
    # --------------------------

    return B, r2


def Forma_hi(taper_dados):

    print('Ajusta Equação de Forma com arquivo Taper.txt')

    # --------------------
    lintaper,ntaper=taper_dados, len(taper_dados)
    # --------------------

    # numero da árvore
    dado = 0
    dap=0

    # a = vetor de '1'
    # 'x1= di_casc/dap
    # 'x2= (di_casc/dap)**2
    # 'y= (hi/ht)**2

    a = []
    x1 = []
    x2 = []
    y = []

    # Acessa arquivo texto, armazena dap e ht por arvore
    taper = 1

    # Executa por seção
    i=1
    hi=0
    hi1=1

    while taper <= ntaper:

        while hi<hi1:
            dados = lintaper[taper - 1]
            hi = float(dados.get('cmatap_hi'))

            if taper==ntaper:
                pass
            else:
                dados1 = lintaper[taper]
                hi1 = float(dados1.get('cmatap_hi'))

            try:
                arv=int(dados.get('cmatap_arv'))
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))
                ht=float(dados.get('cmatap_ht'))

            except:
                # print('ainda não leu altura total (ht)')
                pass

            if hi == 1.3:
                dap = di_casc

            taper=taper+1


        for c in range(i,taper,1):
            dados = lintaper[c - 1]
            try:
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))

            except:
                print('arquivo Taper.txt com campos incompletos')

            try:
                x1.append(di_casc/dap)
                x2.append((di_casc/dap)**2)
                y.append((hi/ht)**2)

                # print('hi/ht:{}'.format(di_casc/dap))
            except:
                print('falta algum dado no Taper.txt')

            hi=0
            hi1=1
        i = taper

    dado = ntaper

    a = dado * [1]
    # print('a=',a)
    # print('x1=',x1)
    # print('x2=',x2)
    # print('y=',y)
    # --------------------------
    B, r2 =regressao(dado, a, x1, x2, y)
    # --------------------------

    B0f_hi = B[0]
    B1f_hi = B[1]
    B2f_hi = B[2]

    # print('b0f_hi={}, b1f_hi={}, b2f_hi={}'.format(B0f_hi, B1f_hi, B2f_hi))

    return B0f_hi,B1f_hi,B2f_hi


def calcula_altura_comercial(arvore_dados, diametro_minimo, parametro_dados, taper_dados):
    # ------------------------------------------------------------------
    # Calcula Altura comercial a partir da Ht do arquivo Arvore.txt
    altura_comercial_dict = {}
    if not diametro_minimo:
        return altura_comercial_dict
    di_limite = diametro_minimo
    falha = 0

    # -----------------------
    linarv, narv = arvore_dados, len(arvore_dados)
    # -----------------------

    # escolha = '1'
    # -------------------------------------------
    # es_serra, hi_atual, hi, hi_min, prop_casc, l_min_pecas, es_min_pecas, di_min, es_perim = parametros(
    #     escolha)
    hi_atual = parametro_dados.get('cmapar_altura_corte')
    hi = parametro_dados.get('cmapar_altura')
    flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
    es_medula = parametro_dados.get('es_medula')
    # -------------------------------------------

    parc1 = []
    arvore1 = []
    dap1 = []
    ht1 = []
    hc1 = []

    # --------------------------------------
    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
    # --------------------------------------

    for arv in arvore_dados:

        try:
            parc = float(arv.get('cmaarv_linha'))
            arvore = float(arv.get('cmaarv_arvore'))
            dap = float(arv.get('cmaarv_diametro'))
            ht = float(arv.get('cmaarv_altura'))

            if hi <= ht:
                # --------------------------------------
                hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                hi2 = hi1.hi_result()

                if hi2 < hi:
                    hi2 = 0

        except:
            falha += 1

            parc = float(arv.get('cmaarv_linha'))
            arvore = 0
            dap = 0
            ht = 0
            hi2 = 0

        parc1.append(parc)
        arvore1.append(arvore)
        dap1.append(dap)
        ht1.append(ht)
        hc1.append(hi2)

        altura_comercial_dict.update({arv.get('id'): hi2})
    return altura_comercial_dict


# Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia


def calc_preco(pecas_serradas_preco_dados, codigopeca, hi, v_tmp):
    preco_peca = 0
    w = 0
    if pecas_serradas_preco_dados:
        preco = pecas_serradas_preco_dados[w]
        while codigopeca != int(preco.get('cpspre_codigo_peca')):
            preco = pecas_serradas_preco_dados[w]
            w = w + 1

        es_ref = float(preco.get('cpspre_espessura'))
        l_ref = float(preco.get('cpspre_largura'))
        preco_ref = float(preco.get('cpspre_preco'))

        v_ref = es_ref / 1000 * l_ref / 1000 * hi
        preco_peca = (v_tmp / v_ref) * preco_ref

    return preco_peca
