#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

# Variaveis
# Peca rolica
# Dap - diametro a altura do peito (duas variáveis em uma)
# ht - altura total da árvore (arvore)
# hi - comprimento da tora = comprimento da peça (parametro)
# hi_atual - começa com altura do toco (30 cm) (parametro)
# es - espessura peça (peca)
# di_casc - diâmetro com casca (calculado)
# di_casc2 - diâmetro com casca para calculo do volume da peça (calculado)
# di - diâmetro sem casca (usar o percentual de casca da especie)
# vr - volume peça roliça (calculado)
# vt - volume arvore (calculado usando coefientes das especies a0, a1, a2)
# narv - numero de árvores (inventario arvores)
# codpeca-codigo peça (usar o nome da peca)

# hc - altura comercial da árvore
# hi - comprimento da tora = comprimento da peça
# es - espessura da peça
# es_serra - espessura da serra (perda de madeira no corte)
# min_es - menor espessura da peça
# max_es - maior espessura da peça
# l - largura peça
# es_sobra - sobra peça
# ntora - numero da tora

# l_d, l_e - larguras direita e esquerda da peça
# l_s, l_i - larguras superior e inferior da peça
# l_d2 - largura direita da próxima peça
# l_s2 - largura superior da próxima peça
# cord_d, cord_e - cordas direita e esquerda da peça
# cord_s, cord_i - cordas superior e inferior da peça
# ang_d, ang_e - angulos direito e esquerdo formados c respectivas cordas
# ang_s, ang_i - angulos superior e inferior formados c respectivas cordas
# n-num.peças do 2o quadrante
# nes = n*es - numero de peças vezes sua espessura


from funcoes import *
from openerp.osv import osv, fields
from funcoes import calcmadeira_get_totais, Formulas
from openerp.tools.translate import _
import base64
import subprocess
from time import sleep


def calcmadeira_get_tipo_funcao(self, cr, uid, context=None):
    context = context and context.copy() or {}
    res = []
    if context.get('tipo_funcao') == '1':
        res = [('1', _('Calculo peca rolica'))]
    elif context.get('tipo_funcao') in ('2', '3', '4', '5', '13', '15'):
        res = [('2', _('Calculo desdobro longitudinal base')),
               ('3', _('Calculo desdobro longitudinal')),
               ('4', _('Calculo desdobro quadrado circunscrito')),
               ('5', _('Calculo peca desdobro radial')),
               ('13', _('Calculo peca gira 180 90')),
               ('15', _('Calculo peca radial longitudinal'))]
    elif context.get('tipo_funcao') in ('6', '7', '8', '9', '14', '16'):
        res = [('6', _('Calculo desdobro longitudinal base por tora')),
               ('7', _('Calculo desdobro longitudinal por tora')),
               ('8', _('Calculo desdobro quadrado circunscrito por tora')),
               ('9', _('Calculo desdobro radial por tora')),
               ('14', _('Calculo gira 180 90 por tora')),
               ('16', _('Calculo radial longitudinal por tora'))]
    elif context.get('calculo_fator_forma'):
        res = [('12', _('Calculo Fator de Forma'))]
    elif context.get('calculo_laminada_arvore'):
        res = [('10', _('Calculo laminacao'))]
    elif context.get('calculo_laminada_tora'):
        res = [('11', _('Calculo laminacao por tora'))]
    return res


class wizard_calcmadeira_calcula(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula'
    FUNCOES = [('', '')]
    _columns = {
        'company_id': fields.many2one('res.company', _('Empresa'), ondelete='cascade',
                                      required=True, select=True),
        'peca_ids': fields.one2many('wizard.calcmadeira.calcula.itens', 'peca_id', _('Itens')),
        # itens do calculo de rolicas nas rotinas de serradas
        'peca_rolica_ids': fields.one2many('wizard.calcmadeira.calcula.rolica.itens', 'capa_id',
                                           _('Itens Rolica')),
        'result_ids': fields.one2many('wizard.calcmadeira.resultado.peca', 'result_id',
                                      _('Resultado')),
        'mostrar_pecas_rolicas': fields.boolean('Mostrar pecas rolicas'),
        'pecas_rolicas': fields.many2many('calcmadeira.pecas.rolicas', 'calcmadeira_calcula_rolica',
                                          'id',
                                          'peca_id', _('Pecas Rolicas')),
        'mostrar_pecas_serradas': fields.boolean('Mostrar pecas serradas'),
        'pecas_serradas': fields.many2many('calcmadeira.pecas.serradas',
                                           'calcmadeira_calcula_serrada', 'id',
                                           'peca_id', _('Pecas Serradas')),
        'tipo_funcao': fields.selection(calcmadeira_get_tipo_funcao, _('Funcao'), size=2,
                                        required=True),
        'vt1': fields.float(_('Volume total (m³)'), digits=(3, 4), readonly=True),
        'narv': fields.integer(_('Arvores'), readonly=True),
        'total_vt_vr': fields.float(_('Vt-Vr (m³)'), digits=(3, 4), readonly=True),
        'total_vt_v': fields.float(_('Vt-v (m³)'), digits=(3, 4), readonly=True),
        'vr_rendimento': fields.function(calcmadeira_get_totais, method=True,
                                         string=_('Rendimento'), type='float', digits=(3, 2),
                                         store=False, multi='totais'),
        'preco_total': fields.function(calcmadeira_get_totais, method=True,
                                       string=_('Preco da Amostra (R$)'),
                                       digits=(9, 2), type='float', multi='totais'),
        'preco_peca': fields.function(calcmadeira_get_totais, method=True,
                                      string=_('Preco Pecas (R$/m³)'),
                                      digits=(9, 2), type='float', multi='totais'),
        'preco_tora': fields.function(calcmadeira_get_totais, method=True,
                                      string=_('Preco Toras (R$/m³)'),
                                      digits=(9, 2), type='float', multi='totais'),
        'preco_arvore': fields.function(calcmadeira_get_totais, method=True,
                                        string=_('Preco Arvores (R$/m³)'),
                                        digits=(9, 2), type='float', multi='totais'),
        'quantidade_tora': fields.function(calcmadeira_get_totais, method=True, string=_('Toras'),
                                           type='integer', multi='totais'),
        'coeficiente_volume_id': fields.many2one(
            'calcmadeira.coeficiente.equacao', _('Coeficiente volume'), ondelete='cascade',
            domain="[('cmacoe_tipo', '=', 'V')]", required=False),
        'coeficiente_forma_id': fields.many2one(
            'calcmadeira.coeficiente.equacao', _('Coeficiente forma'), ondelete='cascade',
            domain="[('cmacoe_tipo', '=', 'F')]", required=False),
        'quantidade_tora_total_laminada': fields.integer(_('Toras totais'), readonly=True,
                                                         oldname='quantidade_tora_total'),
        'volume_tora_total': fields.float(_('Volume tora total'), digits=(3, 4), readonly=True),
        'comprimento_laminacao_total': fields.float(_('Comprimento total laminacao (m)'),
                                                    digits=(3, 2), readonly=True),
        'folha_laminacao_total': fields.integer(_('Numero de folhas'), readonly=True),
        'fator_forma_medio': fields.float(_('Fator de forma medio'), digits=(3, 4), readonly=True),
        'fator_forma_desvio_padrao': fields.float(_('Desvio padrao fator forma'), digits=(3, 4),
                                                  readonly=True),
        'altura_comercial_calculada': fields.boolean(
            _('Calcular altura comercial?'),
            help=_('Se marcado, o sistema nao vai usar a altura comercial do cadastro e sim a '
                   'calculada a partir do diametro minimo escolhido.')),
        'altura_comercial_diametro_minimo': fields.float(
            _('Diametro minimo de aproveitamento madeireiro (cm)'), digits=(4, 2)),
        'calcula_receita': fields.boolean(_('Calcula receita')),
        # 'arvore_por_hectare': fields.integer(_('Numero de arvores por hectare')),
        'itens_hectare_ids': fields.one2many('wizard.calcmadeira.calcula.itens.hectare',
                                             'wcahec_capa_id', _('Itens por hectare')),
        # itens do calculo de rolicas por hectare nas rotinas de serradas
        'itens_hectare_rolica_ids': fields.one2many(
            'wizard.calcmadeira.calcula.rolica.itens.hectare', 'capa_id',
            _('Itens rolica hectare')),
        # 'permite_calculo_receita': fields.function(
        #     calcmadeira_check_calculo_receita, method=True, string=_('Permite calculo receita'),
        #     type='boolean'),
        'arquivo_tora': fields.binary(_('Arquivo')),
        'arquivo_tora_name': fields.char(_('Arquivo nome'), size=8),
        'arvore_processada': fields.integer('Arvores Processadas', readonly=True),
        'falha': fields.integer('Falhas', readonly=True),
        'arvore_processada_hectare': fields.function(
            calcmadeira_get_totais, method=True, string='Arvore processada/ha', type='integer',
            multi='totais'),
        'falha_hectare': fields.function(
            calcmadeira_get_totais, method=True, string='Falhas/ha', type='integer', 
            multi='totais'),
        'valor_residual': fields.function(
            calcmadeira_get_totais, method=True, string=_('Residuo'), type='float', digits=(3, 4),
            multi='totais'),
        'valor_residual_percentual': fields.function(
            calcmadeira_get_totais, method=True,
            string=_('Residuo Volume tora total - Volume peca rolica (%)'), type='float',
            digits=(3, 4), multi='totais'),
        'total_volume_hectare': fields.function(
            calcmadeira_get_totais, method=True, string='Vol/Ha', type='float', digits=(3, 4),
            multi='totais'),
        'talhao_ids': fields.many2many(
            'calcmadeira.talhao', 'calcmadeira_calcula_calcmadeira_talhao',
            'calcmadeira_calcula_id', 'talhao_id', 'Talhao'),
        'talhao_todos': fields.boolean(
            _('Todos os talhoes'),
            help=_('Se marcado, automaticamente, todos os talhoes disponiveis serao '
                   'selecionados.')),
        'parcela_ids': fields.many2many(
            'calcmadeira.parcela', 'calcmadeira_calcula_calcmadeira_parcela',
            'calcmadeira_calcula_id', 'parcela_id', 'Parcela'),
        'parcela_todos': fields.boolean(
            'Todas as parcelas',
            help=_('Se marcado, automaticamente, todas as parcelas disponiveis serao '
                   'selecionadas.')),
        'qtd_talhao': fields.function(calcmadeira_get_totais, method=True,
                                      string='Quantidade talhoes', type='integer', multi='totais'),
        'espacamento_entre_arvore': fields.float('Espacamento entre arvores', digits=(3, 2)),
        'espacamento_entre_linha': fields.float('Espacamento entre linhas', digits=(3, 2)),
        'valor_floresta': fields.float('Valor da floresta', digits=(15, 2)),
        'frequencia_peca_parcela_talhao_ids': fields.one2many(
            'wizard.calcmadeira.frequencia.peca.parcela.talhao', 'capa_id', 
            'Frequencia Resultado Por Parcela Dentro de Talhao'),
        'frequencia_peca_parcela_talhao_rolica_ids': fields.one2many(
            'wizard.calcmadeira.frequencia.peca.parcela.talhao.rolica', 'capa_id',
            'Frequencia Resultado Por Parcela Dentro de Talhao - Rolica'),
        'frequencia_peca_talhao_ids': fields.one2many(
            'wizard.calcmadeira.frequencia.peca.talhao', 'capa_id',
            'Frequencia Resultado Por Talhao'),
        'frequencia_peca_talhao_rolica_ids': fields.one2many(
            'wizard.calcmadeira.frequencia.peca.talhao.rolica', 'capa_id',
            'Frequencia Resultado Por Talhao - Rolica'),
        'resultado_parcela_talhao_ids': fields.one2many(
            'wizard.calcmadeira.parcela.talhao', 'capa_id', 'Resultado Parcela Dentro de Talhao'),
        'resultado_parcela_talhao_rolica_ids': fields.one2many(
            'wizard.calcmadeira.parcela.talhao.rolica', 'capa_id',
            'Resultado Parcela Dentro de Talhao - Rolica'),
        'resultado_talhao_ids': fields.one2many('wizard.calcmadeira.talhao', 'capa_id',
                                                'Resultado Por Talhao'),
        'resultado_talhao_rolica_ids': fields.one2many(
            'wizard.calcmadeira.talhao.rolica', 'capa_id', 'Resultado Por Talhao - Rolica'),
        'resultado_csv_ids': fields.one2many(
            'wizard.calcmadeira.arquivo.csv', 'capa_id', 'CSV'),

    }
    _defaults = {
        'company_id': lambda s, cr, uid, c: s.pool.get('res.users').browse(
            cr, uid, uid).company_id.id,
        'pecas_rolicas': False,
        'pecas_serradas': False,
        'altura_comercial_calculada': False,
        'calcula_receita': True,
        # 'arvore_por_hectare': 0,
        'arquivo_tora_name': 'Tora.txt',
    }

    def default_get(self, cr, uid, fields_list, context=None):
        context = context and context.copy() or {}
        res = super(wizard_calcmadeira_calcula, self).default_get(cr, uid, fields_list, context)
        obj_calcmadeira_coeficiente_equacao = self.pool.get('calcmadeira.coeficiente.equacao')
        coeficiente_volume_padrao_ids = obj_calcmadeira_coeficiente_equacao.search(
            cr, uid, [('name', '=', 'Volume padrao')], order='id', context=context)
        coeficiente_volume_padrao_ids and res.update({
            'coeficiente_volume_id': coeficiente_volume_padrao_ids[0]})
        coeficiente_forma_padrao_ids = obj_calcmadeira_coeficiente_equacao.search(
            cr, uid, [('name', '=', 'Forma padrao')], order='id', context=context)
        coeficiente_forma_padrao_ids and res.update({
            'coeficiente_forma_id': coeficiente_forma_padrao_ids[0]})
        context.get('calculo_fator_forma') and res.update({'tipo_funcao': '12'})
        context.get('calculo_laminada_arvore') and res.update({'tipo_funcao': '10'})
        if context.get('funcoes_serrada_tora') and context.get('calculo_laminada_tora'):
            res.update({'tipo_funcao': '11'})
        elif context.get('funcoes_serrada_tora'):
            res.update({'tipo_funcao': '6'})
        # res.update({'permite_calculo_receita': True})
        if any([context.get('calculo_laminada_arvore'), context.get('calculo_laminada_tora'),
                context.get('calculo_fator_forma')]):
            # res.update({'permite_calculo_receita': False, 'calcula_receita': False})
            res.update({'calcula_receita': False})
        if any([context.get('calculo_rolica'), context.get('serrada_e_rolica'),
                context.get('funcoes_serrada_arvore')]):
            res.update({'calcula_receita': True})
        # if context.get('serrada_e_rolica'):
        #     # O Calculo de 'Madeira Serrada + Rolica' sempre realiza o calculo da receita.
        #     # No entanto, atribui o valor falso para 'permite_calculo_receita' para que a opcao de
        #     # selecao nao seja exibida ao usuario
        #     # res.update({'permite_calculo_receita': False, 'calcula_receita': True})
        #     res.update({'calcula_receita': True})

        obj_calcmadeira_talhao = self.pool.get('calcmadeira.talhao')
        obj_calcmadeira_arvores = self.pool.get('calcmadeira.arvores')
        arvore_ids = obj_calcmadeira_arvores.search(
            cr, uid, [('cmaarv_talhao', '!=', False)], order='cmaarv_talhao', context=context)
        talhao_arvore = list(set(map(lambda t: t.cmaarv_talhao, obj_calcmadeira_arvores.browse(
            cr, uid, arvore_ids, context))))
        talhao_ids = obj_calcmadeira_talhao.search(cr, uid, [('talhao', '!=', False)],
                                                   order='talhao', context=context)
        talhao_talhao = list(set(map(lambda t: t.talhao, obj_calcmadeira_talhao.browse(
            cr, uid, talhao_ids, context))))
        if talhao_arvore != talhao_talhao:
            obj_calcmadeira_talhao.unlink(cr, uid, talhao_ids, context)
            for talhao in talhao_arvore:
                obj_calcmadeira_talhao.create(cr, uid, {'talhao': talhao}, context)
            cr.commit()

        obj_calcmadeira_parcela = self.pool.get('calcmadeira.parcela')
        obj_calcmadeira_arvores = self.pool.get('calcmadeira.arvores')
        arvore_ids = obj_calcmadeira_arvores.search(
            cr, uid, [('cmaarv_linha', '!=', False)], order='cmaarv_linha', context=context)
        parcela_arvore = list(set(map(lambda t: t.cmaarv_linha, obj_calcmadeira_arvores.browse(
            cr, uid, arvore_ids, context))))
        parcela_ids = obj_calcmadeira_parcela.search(cr, uid, [('parcela', '!=', False)],
                                                     order='parcela', context=context)
        parcela_parcela = list(set(map(lambda t: t.parcela, obj_calcmadeira_parcela.browse(
            cr, uid, parcela_ids, context))))
        if parcela_arvore != parcela_parcela:
            obj_calcmadeira_parcela.unlink(cr, uid, parcela_ids, context)
            for parcela in parcela_arvore:
                obj_calcmadeira_parcela.create(cr, uid, {'parcela': parcela}, context)
            cr.commit()
        return res

    def calcmadeira_onchange_talhao_todos(self, cr, uid, ids, talhao_todos, context=None):
        context = context and context.copy() or {}
        res = {'value': {'talhao_ids': [], 'qtd_talhao': 0}}
        if not talhao_todos:
            return res
        talhao_ids = self.pool.get('calcmadeira.talhao').search(cr, uid, [], context=context)
        vals = {'talhao_ids': talhao_ids, 'qtd_talhao': len(talhao_ids)}
        if len(talhao_ids) > 1:
            vals.update({'parcela_todos': False, 'parcela_ids': []})
        else:
            vals.update({'parcela_todos': True})
        res.update({'value': vals})
        return res

    def calcmadeira_onchange_talhao_ids(self, cr, uid, ids, talhao_ids, context=None):
        context = context and context.copy() or {}
        res = {'value': {'qtd_talhao': 0}}
        if not talhao_ids:
            return res
        vals = {'qtd_talhao': len(talhao_ids[0][2])}
        if len(talhao_ids[0][2]) > 1:
            vals.update({'parcela_todos': False, 'parcela_ids': []})
        else:
            vals.update({'parcela_todos': True})
        res.update({'value': vals})
        return res

    def calcmadeira_onchange_parcela_todos(self, cr, uid, ids, parcela_todos, context=None):
        context = context and context.copy() or {}
        res = {'value': {'parcela_ids': []}}
        if not parcela_todos:
            return res
        res.update({'value': {'parcela_ids': self.pool.get('calcmadeira.parcela').search(
            cr, uid, [], context=context)}})
        return res

    def onchange_calcmadeira_pecas_ids(self, cr, uid, ids, tipo_funcao, vt1, narv, total_vt_vr,
                                       total_vt_v, result_ids, context=None):
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.rolicas')
        res = {'value': {}}
        listpeca = []
        listresult = []
        limpa_dados_tela = []
        vals = {'altura_comercial_calculada': False, 'altura_comercial_diametro_minimo': 0}
        if tipo_funcao == '1' or context.get('serrada_e_rolica'):
            pecas_ids = obj_dados_pecas.search(cr, uid, [], context=context)
            obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)
            for y in obj_dados_pecas_dados:
                listpeca.append(y.get('id'))
            vals.update({'pecas_rolicas': listpeca, 'peca_ids': [],
                         'result': listresult, 'result_ids': [],
                         'vt1': limpa_dados_tela, 'narv': 0,
                         'total_vt_vr': limpa_dados_tela,
                         'total_vt_v': limpa_dados_tela,
                         'mostrar_pecas_rolicas': True})
        listpeca = []
        if tipo_funcao != '1' and tipo_funcao != '10':
            obj_dados_pecas = self.pool.get('calcmadeira.pecas.serradas')
            pecas_ids = obj_dados_pecas.search(cr, uid, [], context=context)
            obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)
            for y in obj_dados_pecas_dados:
                listpeca.append(y.get('id'))
            vals.update({'pecas_serradas': listpeca, 'peca_ids': [],
                         'result': listresult, 'result_ids': [],
                         'vt1': limpa_dados_tela, 'narv': 0,
                         'total_vt_vr': limpa_dados_tela,
                         'total_vt_v': limpa_dados_tela, 'quantidade_tora_total_laminada': 0,
                         'volume_tora_total': 0, 'comprimento_laminacao_total': 0,
                         'fator_forma_medio': 0, 'fator_forma_desvio_padrao': 0,
                         'folha_laminacao_total': 0,
                         'mostrar_pecas_serradas': True})
        return {'value': vals}

    def button_print(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_pecas',
        }

    def calcula_peca_rolica(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças roliças
        # limpa os dados da tree
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        vt1 = 0.0000
        vt = 0
        ntora1 = 0
        arv = 0
        di_casc = di_casc2 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.rolicas')

        arvores_ids = obj_dados_arvores.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context=context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        brw_calcmadeira_pecas = brw_calcmadeira_calcula.pecas_rolicas
        pecas_rolicas = []
        for i in brw_calcmadeira_pecas:
            pecas_rolicas.append(i.id)
        esp_id = obj_dados_pecas.search(cr, uid, [('id', 'in', pecas_rolicas)],
                                        order='cmapro_diametro_min', limit=1, context=context)
        obj_dados_espessura = obj_dados_pecas.read(cr, uid, esp_id, context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        # Seleciona dentre todas as peças o menor valor do comprimento
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        # di = 100
        hi_min = 2.2
        hi = hi_min
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append([i.get('cmaarv_diametro'),
                          i.get('cmaarv_altura'),
                          altura_comercial_calculada_dict.get(i.get('id'))
                          if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial'),
                          i.get('cmaarv_arvore'),
                          i.get('cmaarv_linha')
                          ])
            narv += 1

        falha = 0
        arv_proces = 0

        while arv < narv:

            dap = list1[arv][0]
            ht = list1[arv][1]
            hc = list1[arv][2]
            cod_arv = list1[arv][3]
            parc = list1[arv][4]
            arv += 1
            # vr = 0

            # conta árvores processadas
            if hc > 0 and dap > 0 and ht > 0:
                arv_proces += 1

            vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht, hc,
                                                     context)

            if not dap or not ht:
                falha += 1

            # numero da tora
            ntora = 0

            # calcula seções da árvore. 6 é o menor diametro possível
            while hi_atual + hi_min <= hc:
                di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                       coeficiente_forma=coeficiente_forma,
                                                                       hi_atual=hi_atual,
                                                                       hi=hi,
                                                                       ht=ht,
                                                                       dap=dap,
                                                                       di_casc=di_casc,
                                                                       di_casc2=di_casc2,
                                                                       context=context)
                di = di_casc2 * (1 - prop_casc)
                # menor valor denter todas as peças (diametro)
                for i in obj_dados_espessura:
                    di_min = i.get('cmapro_diametro_min')
                ntora += 1
                if di < di_min or hi_atual + hi_min > hc:
                    ntora = ntora - 1
                    break
                codigopeca, namepeca, voltora, volpeca, numtora, hi, ntora, preco \
                    = self.seleciona_peca_rolica(cr, uid, ids, dap, ht, hc, hi_atual, di, ntora,
                                                 arv, cod_arv, di_casc, di_casc2,
                                                 coeficiente_forma, prop_casc, parc,
                                                 context=context)
                hi_atual += hi
                hi = hi_min
                numpeca = 0
                # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora, numpeca, volpeca, preco, context=context)
            ntora1 = ntora1 + ntora
            ntora = 0
            vt1 += vt
            hi_atual = parametro_dados.get('cmapar_altura_corte')

        total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids,
                                                                            vt1, context=context)
        self.write(cr, uid, calcmadeira_calcula_id,
                   {'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr, 'total_vt_v': total_vt_v,
                    'arvore_processada': arv_proces, 'falha': falha}, context=context)
        self.calcula_madeira_rolica_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_rolica(self, cr, uid, ids, dap, ht, hc, hi_atual, di, ntora, arv, cod_arv,
                              di_casc,
                              di_casc2, coeficiente_forma, prop_casc, parc, context=None):

        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas = brw_calcmadeira_calcula[0].pecas_rolicas
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.rolicas')
        pecas_rolicas = []
        for i in brw_calcmadeira_pecas:
            pecas_rolicas.append(i.id)
        if not pecas_rolicas and context.get('serrada_e_rolica'):
            pecas_rolicas = obj_dados_pecas.search(cr, uid, [], context=None)
        pecas_ids = obj_dados_pecas.search(cr, uid, [('id', 'in', pecas_rolicas)],
                                           order='cmapro_sequence',
                                           context=context)
        obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)
        voltora = 0
        numtora = 0
        vr_preco = 0
        volpeca = 0
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapro_codigo_peca')
            namepeca = y.get('name')
            di_min = y.get('cmapro_diametro_min') * (1 - prop_casc)
            di_max = y.get('cmapro_diametro_max') * (1 - prop_casc)
            hi = y.get('cmapro_comprimento_tora')
            preco = (y.get('cmapro_preco') or 0)

            # Retirei restrição di < di_max. Estava exato, mas pulando peça.
            # Agora a peça, em último caso, enquadra com di > di_min e di_max.
            # Pode acontecer de ser uma peça bem mais grossa para seu comprimento, mas é melhor do que descartar a madeira.

            if di_min <= di and hi_atual + hi <= hc:

                # Seleciona equação de forma
                di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                       coeficiente_forma=coeficiente_forma,
                                                                       hi_atual=hi_atual,
                                                                       hi=hi,
                                                                       ht=ht,
                                                                       dap=dap,
                                                                       di_casc=di_casc,
                                                                       di_casc2=di_casc2,
                                                                       context=context)

                di = di_casc2 * (1 - prop_casc)

                # Retirei restrição di < di_max. Estava exato, mas pulando peça.
                # Agora a peça, em último caso, enquadra com di > di_min e di_max.
                # Pode acontecer de ser uma peça bem mais grossa para seu comprimento, mas é melhor do que descartar a madeira.

                if di_min <= di and hi_atual + hi <= hc:

                    # volume da tora usando formula de Smalian
                    vr = math.pi / 40000 * (di_casc ** 2 + di_casc2 ** 2) / 2 * hi
                    voltora += vr

                    # volume da peça roliça selecionada
                    vp_rolic = math.pi / 40000 * (di_max ** 2 + di_min ** 2) / 2 * hi
                    volpeca += vp_rolic

                    numtora += 1
                    vr_preco += preco
                    self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, codigopeca, namepeca, vr,
                                                             di_casc, di_casc2, di, ntora, cod_arv,
                                                             0, 0, 0, 0, 0, 0, 0, 0, parc=parc,
                                                             conicidade=0, comprimento=hi,
                                                             # (di_casc - di_casc2) / hi,
                                                             preco=preco, context=context)
                else:
                    p = obj_dados_pecas_dados.index(y)
                    if p < len(obj_dados_pecas_dados) - 1:
                        p = p + 1
                    else:
                        break

                    codigopeca = obj_dados_pecas_dados[p].get('cmapro_codigo_peca')
                    namepeca = obj_dados_pecas_dados[p].get('name')
                    di_min = obj_dados_pecas_dados[p].get('cmapro_diametro_min') * (1 - prop_casc)
                    di_max = obj_dados_pecas_dados[p].get('cmapro_diametro_max') * (1 - prop_casc)
                    hi = obj_dados_pecas_dados[p].get('cmapro_comprimento_tora')
                    preco = obj_dados_pecas_dados[p].get('cmapro_preco') or 0

                    vr_preco += preco

                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)
                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi
                    voltora += vr

                    # volume da peça roliça selecionada
                    vp_rolic = math.pi / 40000 * (di_max ** 2 + di_min ** 2) / 2 * hi
                    volpeca += vp_rolic

                    numtora += 1

                    di = di_casc2 * (1 - prop_casc)

                    self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, codigopeca, namepeca, vr,
                                                             di_casc, di_casc2, di, ntora, cod_arv,
                                                             0, 0, 0, 0, 0, 0, 0, 0, parc=parc,
                                                             conicidade=0, comprimento=hi,
                                                             # (di_casc - di_casc2) / hi,
                                                             preco=preco, context=context)

                if vr != 0:
                    break
        return (codigopeca, namepeca, voltora, volpeca, numtora, hi, ntora, vr_preco)

    def calcula_peca_rolica_pta_serrad(self, cr, uid, ids, prop_casc, cod_arv, dap, ht, hi_atual,
                                       arv, parc, context=None):
        di_casc = di_casc2 = 0
        obj_pecas_rolicas = self.pool.get('calcmadeira.pecas.rolicas')
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context=context)
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        pecas_rolicas_ids = obj_pecas_rolicas.search(cr, uid, [], context=context)
        pecas_rolicas_dados = obj_pecas_rolicas.read(
            cr, uid, pecas_rolicas_ids, ['cmapro_comprimento_tora', 'cmapro_diametro_min'],
            context=context)
        hi_min = min(map(lambda x: x.get('cmapro_comprimento_tora'), pecas_rolicas_dados))
        di_min = min(map(lambda x: x.get('cmapro_diametro_min'), pecas_rolicas_dados))
        hi = hi_min
        hc = ht
        ntora = 0

        # calcula seções da árvore. 6 é o menor diametro possível
        while hi_atual + hi_min <= hc:
            di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                   coeficiente_forma=coeficiente_forma,
                                                                   hi_atual=hi_atual,
                                                                   hi=hi,
                                                                   ht=ht,
                                                                   dap=dap,
                                                                   di_casc=di_casc,
                                                                   di_casc2=di_casc2,
                                                                   context=context)
            di = di_casc2 * (1 - prop_casc)
            if di < di_min or hi_atual + hi_min > hc:
                break
            ntora += 1
            codigopeca, namepeca, voltora, volpeca, numtora, hi, ntora, preco \
                = self.seleciona_peca_rolica(cr, uid, ids, dap, ht, hc, hi_atual, di, ntora,
                                             arv, cod_arv, di_casc, di_casc2,
                                             coeficiente_forma, prop_casc, parc, context=context)
            hi_atual += hi
            hi = hi_min
            numpeca = 0
            # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
            context.update({'metodo_rolica_pta_serrada': True})
            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                           numtora, numpeca, volpeca, preco, context=context)
            del context['metodo_rolica_pta_serrada']
        return True

    def calcula_peca_longitudinal_base(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro
        # de retirada de 1/4 do diâmetro para cada lado (retirada de duas costaneiras)
        # e a costaneira base
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora1 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        arvores_ids = obj_dados_arvores.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)

        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        cod_arv = 0
        narv = 0
        di = 100
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial'),
                 i.get('cmaarv_arvore'),
                 i.get('cmaarv_linha')
                 ])
            narv += 1

        falha = 0
        arv_proces = 0
        conic_tora1 = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                dap = t[0]
                ht = t[1]
                hc = t[2]
                cod_arv = t[3]
                parc = t[4]

                if context.get('serrada_e_rolica'):
                    di_limite = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
                    sql = """
                    SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id
                    """ % brw_calcmadeira_calcula.company_id.id
                    cr.execute(sql)
                    taper_dados = cr.dictfetchall()
                    if di_limite > 0:
                        # --------------------------------------
                        B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
                        # --------------------------------------

                        # --------------------------------------
                        hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                        hc = hi1.hi_result()
                    else:
                        hi0 = hi
                        hi = hc
                        # --------------------------------------
                        resultado = Eqdi_casc(coeficiente_forma.cmacoe_b0 or 0, coeficiente_forma.cmacoe_b1 or 0,
                                                coeficiente_forma.cmacoe_b2 or 0, hi_atual, hi, ht, dap)
                        di_limite = resultado.di_casc2_result()
                        # --------------------------------------
                        hi = hi0

                # conta árvores processadas
                if hc > 0 and dap > 0 and ht > 0:
                    arv_proces += 1

                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         hc, context)

                if not dap or not ht:
                    falha += 1

                arv += 1

                # numero de toras
                ntora = 0

                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    r_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt += vt
                    # angulo inicial para retirada da costaneira da base
                    ang_base = parametro_dados.get('cmapar_ang_base')
                    # comprimento da peça, do eixo x até o limite da costaneira da base (R-flecha)
                    r_flecha_base = di / 2 * math.cos(ang_base * math.pi / 180)
                    # angulo inicial para desconto no di para cada lado (flecha0)
                    ang_d = parametro_dados.get('cmapar_ang_direito')

                    flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

                    if brw_calcmadeira_calcula.tipo_funcao == '15':  # '8' - Radial Longitudinal
                        ang_base = 90
                        r_flecha_base = 0

                    flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))
                    # flecha = di / 2 * (1 - math.cos(ang * math.pi / 180))

                    # largura da peça (corda)
                    if ang_d < 90 - ang_base or ang_base == 0:
                        l = di * math.sin(ang_d * math.pi / 180)
                    else:
                        l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                    if di < l_min_pecas + es_serra:
                        break
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                    cord_d1, cord_d, r_flecha_topo, s = \
                        self.seleciona_peca_longitudinal_base(
                            cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                            r_flecha_base, l, ntora, arv, cod_arv, di_casc, di_casc2,
                            pecas_serradas_preco_dados, parametro_dados, ang_base, flecha_div_r,
                            cord_d1, cord_d, r_flecha_topo, s, parc, context=context)
                    hi_atual = hi_atual + hi

                    if brw_calcmadeira_calcula.tipo_funcao == '15':  # '8' - Radial Longitudinal
                        numpeca *= 2
                        volpeca *= 2
                        preco *= 2

                    # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                    self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                   numtora,
                                                   numpeca, volpeca, preco, context=context)

                if context.get('serrada_e_rolica'):
                    self.calcula_peca_rolica_pta_serrad(
                        cr, uid, ids, prop_casc, cod_arv, dap, ht, hi_atual, arv, parc, context)

                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v, 'arvore_processada': arv_proces, 'falha': falha},
                       context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_longitudinal_base(self, cr, uid, ids, l_min_pecas, es_min_pecas, es_serra,
                                         hi, vr, di, flecha0, r_flecha_base, l, ntora, arv, cod_arv,
                                         di_casc,
                                         di_casc2, pecas_serradas_preco_dados, parametro_dados,
                                         ang_base, flecha_div_r, cord_d1, cord_d,
                                         r_flecha_topo, s, parc, context=None):
        brw_calcmadeira_calcula = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                              context=context)
        escolha = brw_calcmadeira_calcula.tipo_funcao
        brw_calcmadeira_pecas = brw_calcmadeira_calcula.pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)

        flecha = flecha0
        es = es_serra

        # espessura residual(eixo horizontal)
        if escolha in ('13', '14'):  # '7' - Gira 180 90
            es_sobra = di / 2 * flecha_div_r - flecha0
        else:
            es_sobra = di - 2 * flecha0

        preco1 = 0
        preco2 = 0
        preco_menor2_14D = 0

        vr_tree = 0
        # espessura residual (eixo vertical)
        l_sobra = l
        es_aux = 0
        obj_dados_pecas = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_pecas.search(cr, uid, [('id', 'in', pecas_serradas)],
                                           order='cmapse_sequence',
                                           context=context)
        obj_dados_pecas_dados = obj_dados_pecas.read(cr, uid, pecas_ids, context=context)

        selec = 0

        vp1 = 0.0000

        # '-----------------------------------------------
        ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(
            flecha, es, es_serra, di, ang_base, r_flecha_base)
        # '-----------------------------------------------

        if flecha <= di / 2 or escolha in ('13', '14'):  # '7' - Gira 180 90
            l_sobra = cord_d
        else:
            l_sobra = cord_e

        cord_e1 = 0

        while selec == 0 and es_sobra >= 0:

            for y in obj_dados_pecas_dados:
                codigopeca = y.get('cmapse_codigo_peca')
                namepeca = y.get('name')
                min_es = int(y.get('cmapse_menor_espessura'))
                max_es = int(y.get('cmapse_maior_espessura'))
                l_min = int(y.get('cmapse_menor_largura'))
                l_max = int(y.get('cmapse_maior_largura'))
                voltora = 0
                numtora = 0
                numpeca = 0
                volpeca = 0
                vr_preco = 0
                n = 0
                v = 0.0000
                n_aux = 0
                v_aux = 0


                selec = 0
                # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
                while (l_sobra - es_serra) >= l_min and (es_sobra - es_serra) >= min_es:
                    if (l_sobra - es_serra) >= l_min or (es_sobra - es_serra) >= min_es:

                        # min_es não pode ser menor do que es+es_serra! não precisa no VBA, Exit
                        # Do no 1o laço
                        if selec == 1 and es < min_es + es_serra:
                            break

                        selec = 1

                        # desdobra da maior espessura da peça para a menor
                        for inv_es in range(0,(max_es-min_es)+1, 5):
                            # sobra de espessura não pode ser menor do que flecha0 + menor espessura
                            # das peças (1cm) + es_serra
                            if (es_sobra - es_serra) < min_es:
                                break
                            es = max_es - inv_es
                            # Considera espessura da serra (fita, corrente motosserra...)
                            es = es + es_serra
                            # Testa es
                            es = testa_espessura(es, es_sobra, es_serra, min_es)

                            # min_es não pode ser menor do que es+es_serra!
                            if es < min_es + es_serra:
                                break

                            ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(
                                flecha, es, es_serra, di, ang_base, r_flecha_base)

                            if flecha <= di / 2 or escolha in ('13', '14'):  # '7' - Gira 180 90
                                l_sobra = cord_d
                            else:
                                l_sobra = cord_e

                            n_h = math.trunc(es_sobra / es)

                            # Se l permite divisao no mínimo em 2 grupos de peças selecionadas (p) em l
                            if l_sobra / (l_min + es_serra) >= 2:
                                # entra função peças divididas na vertical
                                l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, \
                                cord_e, cord_d, r_flecha_base, dados_impressao, preco2, cord_e1 = \
                                    pecamenor_long_1costaneirabase(
                                        l_min, l_max, codigopeca, di, hi, es, es_serra, l_sobra, n_h,
                                        flecha0, flecha, ang_e, ang_d, cord_e, cord_d, r_flecha_base,
                                        pecas_serradas_preco_dados, parametro_dados,
                                        flecha_div_r, ang_base, escolha)

                                for imp in dados_impressao:
                                    self.calcmadeira_atualiza_tree_resultado(
                                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                                        di, ntora, cod_arv, es_serra, imp.get('l_div'),
                                        imp.get('n_div'),
                                        imp.get('v'), 0, imp.get('l_sobra'), es_sobra - n_h * es,
                                        es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                        rendimento=vr and (vp1 + v1) / vr or 0, parc=parc,
                                        preco=imp.get('preco'), context=context)

                                n = 0
                                v = 0.0000
                                es_aux = es
                            else:
                                n = n1 = 0
                                v = v1 = 0.0000

                                # sobra da largura para tentar outra classificação em
                                if l_sobra - es_serra < l_max:
                                    l_div = l_max + es_serra
                                    # ---------------------------
                                    # Testa Largura
                                    l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
                                    # ---------------------------
                                    l = l_div
                                else:
                                    l = l_max + es_serra

                                l_sobra = l_sobra - l

                                # volume das peças
                                for x in range(1, n_h + 1):
                                    vp = ((es - es_serra) / 1000.0) * (l - es_serra)/ 1000.0 * hi

                                    es_tmp = es
                                    l_tmp = l
                                    n_tmp = 1
                                    v_tmp = vp

                                    preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codigopeca,
                                                            hi, v_tmp)

                                    preco1 = preco1 + preco_peca

                                    # preco = 0

                                    self.calcmadeira_atualiza_tree_resultado(
                                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                                        di, ntora,
                                        cod_arv, es_serra, l_tmp, 1, vp, 0, l_sobra,
                                        es_sobra - n_h * es, es - es_serra,
                                        conicidade=(di_casc - di_casc2) / hi, parc=parc,
                                        rendimento=vr and (vp1 + v1) / vr or 0, preco=preco_peca,
                                        context=context)

                                    n_aux += n
                                    v_aux += v
                                    n += 1
                                    vp1 += vp
                                    vp = 0

                                    if l_sobra >= l_min + es_serra:
                                        l_div, l_sobra, n_div, n1, v1, impressao, preco_menor2 \
                                            = pecamenor2_14dcostan(n1, v1, l_min, l_max, codigopeca,
                                                                   hi, es, es_serra, l_sobra,
                                                                   pecas_serradas_preco_dados)

                                        self.calcmadeira_atualiza_tree_resultado(
                                            cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                            di_casc2,
                                            di, ntora, cod_arv, es_serra, impressao.get('l_div'),
                                            impressao.get('n_div'), impressao.get('v'), 0,
                                            impressao.get('l_sobra'), es_sobra - n_h * es,
                                            es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                            rendimento=vr and (vp1 + v1) / vr or 0, parc=parc,
                                            preco=preco_menor2, context=context)


                                        v = 0.0000

                                        preco_menor2_14D = preco_menor2_14D + preco_menor2

                                    if x <= n_h:
                                        flecha = flecha + es

                                        if x == n_h:
                                            es1 = es
                                            es = es_serra

                                        if escolha in ('13', '14'):  # '7' - Gira 180 90
                                            cord_e1 = cord_e

                                        # '-----------------------------------------------
                                        ang_e, ang_d, cord_e, cord_d = Formulas_CostaneiraBase(
                                            flecha, es, es_serra, di, ang_base, r_flecha_base)
                                        # '-----------------------------------------------

                                        if flecha <= di / 2 or escolha in ('13', '14'):  # '7' - Gira 180 90
                                            l = math.trunc(cord_d / 10) * 10
                                        else:
                                            l = math.trunc(cord_e / 10) * 10

                                        if l > l_max + es_serra:
                                            l = l_max + es_serra

                                        if flecha <= di/2 or escolha in ('13', '14'):  # '7' - Gira 180 90
                                            l_sobra = cord_d - l
                                        else:
                                            l_sobra = cord_e - l

                                es_aux = es

                                es = es1


                            preco = preco1 + preco2 + preco_menor2_14D
                            vr_preco += preco

                            preco1 = preco2 = preco_menor2_14D = 0

                            # espessura restante (eixo horizontal) sobra da peça para novo
                            # aproveitamento (cm)
                            es_sobra = es_sobra - n_h * es

                            v = vp1

                            numpeca += n + n1
                            volpeca += v + v1
                            if vr > 0:
                                numtora += 1
                                voltora += vr
                                vr_tree = vr
                                vr = 0


                            # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                            if brw_calcmadeira_calcula.tipo_funcao in ('15', '16'):
                                numpeca *= 2
                                volpeca *= 2
                                vr_preco *= 2
                            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca,
                                                           namepeca, voltora, numtora,
                                                           numpeca, volpeca, vr_preco,
                                                           context=context)

                            numpeca = 0
                            numtora = 0
                            voltora = 0
                            volpeca = 0
                            vr_preco = 0

                            vp1 = 0.0000

            if selec == 0:
                flecha = flecha + 0.5
                es = es_min_pecas

                if flecha + es < di - flecha0:
                    # '-----------------------------------------------
                    ang_e, ang_d, cord_e, cord_d = \
                        Formulas_CostaneiraBase(flecha, es, es_serra, di, ang_base, r_flecha_base)
                    # '-----------------------------------------------
                    if flecha <= di / 2 or escolha in ('13', '14'):
                        l_sobra = cord_d
                    else:
                        l_sobra = cord_e

                    if escolha in ('13', '14'):
                        es_sobra = di / 2 * flecha_div_r - flecha
                    else:
                        es_sobra = di - flecha - flecha0
                else:
                    es_sobra = 0

            if es_sobra == 0:
                break

            if escolha in ('13', '14') and \
                    (selec == 1 or es_sobra < es_min_pecas or flecha / (di / 2) >= flecha_div_r):
                break

        if escolha in ('13', '14'):  # '7' - Gira 180 90
            if s == 0:
                # comprimento da peça, do eixo x até o limite de corte do topo (R-flecha)
                r_flecha_topo = di / 2 * math.cos(ang_d * math.pi / 180)

                cord_d1 = cord_e1
            else:
                r_flecha_base2 = di / 2 * math.cos(ang_d * math.pi / 180)

                cord_d2 = cord_e1

                l = r_flecha_topo + r_flecha_base2

                if cord_d2 >= cord_d1:
                    cord_d = cord_d1
                else:
                    cord_d = cord_d2

        return (codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, vr_preco,
                l, cord_d1, cord_d, r_flecha_topo, s)

    def calcula_peca_longitudinal(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro
        # de retirada de 1/4 do diâmetro para cada lado (retirada de duas costaneiras)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        arvores_ids = obj_dados_arvores.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        cod_arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial'),
                 i.get('cmaarv_arvore'),
                 i.get('cmaarv_linha'),
                 ])
            narv += 1

        falha = 0
        arv_proces = 0
        conic_tora1 = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                dap = t[0]
                ht = t[1]
                hc = t[2]
                cod_arv = t[3]
                parc = t[4]
                arv += 1

                if context.get('serrada_e_rolica'):
                    di_limite = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
                    sql = """
                    SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id
                    """ % brw_calcmadeira_calcula.company_id.id
                    cr.execute(sql)
                    taper_dados = cr.dictfetchall()
                    # --------------------------------------
                    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
                    # --------------------------------------

                    # --------------------------------------
                    hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                    hi2 = hi1.hi_result()
                    # ---------------------------------------

                    if hi2 < hi:
                        hi2 = 0

                    hc = hi2

                # conta árvores processadas
                if hc > 0 and dap > 0 and ht > 0:
                    arv_proces += 1

                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         hc, context)

                if not dap or not ht:
                    falha += 1

                # numero da tora
                ntora = 0
                v = 0
                n = 0
                vr = 0
                es_sobra = 100
                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    r_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt = vrt + vr
                    # angulo inicial para desconto do di para cada lado (flecha0)
                    ang_d = parametro_dados.get('cmapar_ang_direito')
                    ang_base = 0
                    flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                    # largura da peça (corda)
                    if ang_d < 90 - ang_base or ang_base == 0:
                        l = di * math.sin(ang_d * math.pi / 180)
                    else:
                        l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                    if di < l_min_pecas + es_serra:
                        break

                    l_sobra = l
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l,\
                    cord_d1, cord_d, r_flecha_topo, s = \
                        self.seleciona_peca_longitudinal_base(
                        cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                        r_flecha_base, l, ntora, arv, cod_arv, di_casc, di_casc2,
                        pecas_serradas_preco_dados,
                        parametro_dados, ang_base, flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
                        parc, context=context)

                    hi_atual = hi_atual + hi
                    # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                    self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                   numtora,
                                                   numpeca, volpeca, preco, context=context)

                if context.get('serrada_e_rolica'):
                    self.calcula_peca_rolica_pta_serrad(
                        cr, uid, ids, prop_casc, cod_arv, dap, ht, hi_atual, arv, parc, context)

                # atualiza hi_atual com o comprimento da peça extraída (hi)
                # hi_atual = hi_atual + hi
                # 'acumula numero de toras da arvore (ntora) para resultado total de
                # toras na amostra
                ntora1 = ntora1 + ntora
                ntora = 0
                # acumula vt arvore e zera vt
                vt1 = vt1 + vt
                vt = 0
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v, 'arvore_processada': arv_proces, 'falha': falha},
                       context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def calcula_peca_quadrado_circunscrito(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro
        # do quadrado circunscrito
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')

        arvores_ids = obj_dados_arvores.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        cod_arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial'),
                 i.get('cmaarv_arvore'),
                 i.get('cmaarv_linha')
                 ])
            narv += 1

        falha = 0
        arv_proces = 0
        conic_tora1 = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                dap = t[0]
                ht = t[1]
                hc = t[2]
                cod_arv = t[3]
                parc = t[4]

                if context.get('serrada_e_rolica'):
                    di_limite = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
                    sql = """
                    SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id
                    """ % brw_calcmadeira_calcula.company_id.id
                    cr.execute(sql)
                    taper_dados = cr.dictfetchall()
                    # --------------------------------------
                    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
                    # --------------------------------------

                    # --------------------------------------
                    hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                    hi2 = hi1.hi_result()
                    # ---------------------------------------

                    if hi2 < hi:
                        hi2 = 0

                    hc = hi2

                # conta árvores processadas
                if hc > 0 and dap > 0 and ht > 0:
                    arv_proces += 1

                # Seleciona equação de volume
                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         hc, context)

                if not dap or not ht:
                    falha += 1

                arv = arv + 1
                # numero da tora
                ntora = 0
                es_sobra = 100

                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    R_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)
                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi
                    di = di_casc2 * (1 - prop_casc) * 10
                    ntora = ntora + 1
                    vrt = vrt + vr
                    # largura da peça
                    l = di / math.sqrt(2)
                    if l < l_min_pecas + es_serra:
                        ntora = ntora - 1
                        break
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                        = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids,
                                                                    l_min_pecas, es_min_pecas,
                                                                    es_serra, hi,
                                                                    vr,
                                                                    l, ntora, arv, cod_arv, di,
                                                                    di_casc, di_casc2,
                                                                    pecas_serradas_preco_dados,
                                                                    cord_d, es_medula, flecha0,
                                                                    parc, context=context)
                    hi_atual = hi_atual + hi

                if context.get('serrada_e_rolica'):
                    self.calcula_peca_rolica_pta_serrad(
                        cr, uid, ids, prop_casc, cod_arv, dap, ht, hi_atual, arv, parc, context)

                # atualiza hi_atual com o comprimento da tora calculada (hi)
                # hi_atual = hi_atual + hi
                # acumula ntora da arvore e zera ntora
                ntora1 = ntora1 + ntora
                ntora = 0
                # acumula vt da árvore e zera vt
                vt1 += vt
                vt = 0
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v, 'arvore_processada': arv_proces, 'falha': falha},
                       context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_quadrado_circunscrito(self, cr, uid, ids, l_min_pecas, es_min_pecas,
                                             es_serra, hi, vr,
                                             l, ntora, arv, cod_arv, di, di_casc, di_casc2,
                                             pecas_serradas_preco_dados, cord_d, es_medula, flecha0,
                                             parc, context=None):
        brw_calcmadeira_calcula = self.browse(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                              context=context)
        escolha = brw_calcmadeira_calcula.tipo_funcao
        brw_calcmadeira_pecas = brw_calcmadeira_calcula.pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)
        # espessura residual (eixo horizontal do bloco)
        if escolha in ('13', '14'):  # '7' - Gira 180 90
            es_sobra = cord_d
        else:
            es_sobra = l

        cont_medula = 0
        v_medula = 0

        es = es_serra
        selec = 0
        # espessura residual (eixo vertical do bloco)
        l_sobra = l
        l1 = l
        # l2 = 0

        preco1 = 0
        preco2 = 0

        l_sobra_aux = 0
        es = 0
        vr_tree = 0
        obj_dados_peca = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_peca.search(cr, uid, [('id', 'in', pecas_serradas)],
                                          order='cmapse_sequence',
                                          context=context)
        obj_dados_pecas_dados = obj_dados_peca.read(cr, uid, pecas_ids, context=context)
        numtora = 0
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapse_codigo_peca')
            namepeca = y.get('name')
            min_es = y.get('cmapse_menor_espessura')
            max_es = y.get('cmapse_maior_espessura')
            l_min = y.get('cmapse_menor_largura')
            l_max = y.get('cmapse_maior_largura')
            voltora = 0
            numtora = 0
            numpeca = 0
            volpeca = 0
            vr_preco = 0
            n = 0
            v = 0

            selec = 0
            # Calcula peças na seção do tronco até limites mínimos (l<2 e min_es por tipo de peça)
            while (l_sobra - es_serra) >= l_min and (es_sobra - es_serra) >= min_es:

                # min_es não pode ser menor do que es+es_serra!
                if selec == 1 and es < min_es + es_serra:
                    break

                selec = 1

                # desdobra da maior espessura da peça para a menor
                for inv_es in range(0, ((max_es-min_es)+1), 5):
                    # sobra de espessura não pode ser menor do que flecha0 + menor
                    # espessura das peças (1cm) + es_serra
                    if (l_sobra - es_serra) < l_min or (es_sobra - es_serra) < min_es:
                        break
                    es = max_es - inv_es
                    # Considera espessura da serra (fita, corrente motosserra...)
                    es = es + es_serra
                    es = testa_espessura(es, es_sobra, es_serra, min_es)
                    # min_es não pode ser menor do que es+es_serra!
                    if (es_sobra - es_serra) < min_es or es < min_es + es_serra:
                        break

                    if es_sobra > es + es_medula and es_medula > es_serra and \
                            cont_medula == 0:
                        n_h = math.trunc((es_sobra - es_medula) / es)
                    else:
                        n_h = math.trunc(es_sobra / es)

                    # Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
                    if l_sobra / (l_min + es_serra) >= 2:
                        # entra função peças divididas na vertical
                        l_sobra, n1, v1, dados_impressao, preco2 \
                            = pecamenor_quadcircun(l_min, l_max, codigopeca, hi, es, es_serra,
                                                   es_sobra, l,
                                                   l_sobra, n_h, pecas_serradas_preco_dados,
                                                   es_medula, escolha, cont_medula)

                        for imp in dados_impressao:
                            self.calcmadeira_atualiza_tree_resultado(
                                cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                                di, ntora, cod_arv, es_serra, imp.get('l_div'), imp.get('n_div'),
                                imp.get('v'), 0, imp.get('l_sobra'), es_sobra - n_h * es,
                                es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                rendimento=vr and (imp.get('v') + v1) / vr or 0, parc=parc,
                                preco=imp.get('preco'), context=context)

                        v = 0.0000
                        n = 0
                    else:
                        n = n_h
                        n1 = 0
                        v1 = 0.0000
                        # sobra da largura para tentar outra classificação em PecaMenor
                        if l_sobra - es_serra < l_max:
                            l_div = l_max + es_serra
                            # ---------------------------
                            # Testa Largura
                            l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
                            # ---------------------------
                            l = l_div
                        else:
                            l = l_max + es_serra

                        l_sobra = l_sobra - l

                        # volume de peças sem divisão vertical
                        v = (n_h * (es - es_serra) / 1000.0) * (l - es_serra) / 1000.0 * hi

                        # Relatorio por peça
                        # print('codpeca:{}, es:{}, l:{}, n:{}, v:{}, l_sobra:{}, ntora:{}, '
                        #       'arv:{}'.format(codigopeca, es, l, n, v, l_sobra, ntora, arv))

                        es_tmp = es
                        l_tmp = l
                        n_tmp = n
                        v_tmp = v
                        # -----------------------------------------------------------------
                        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                        # referencia
                        preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codigopeca, hi, v_tmp)
                        # -----------------------------------------------------------------

                        preco1 = preco_peca

                        if es_sobra > es + es_medula and es_medula > es_serra \
                                and cont_medula == 0:
                            v_medula = ((es_medula - es_serra) / 1000) * (l - es_serra) / 1000 * hi
                            n_div_medula = 1
                            v = v + v_medula
                            n = n + n_div_medula

                            es_tmp = es_medula
                            l_tmp = l_div
                            n_tmp = n_div_medula
                            v_tmp = v_medula
                            # -----------------------------------------------------------------
                            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                            # referencia
                            preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codigopeca, hi,
                                                    v_tmp)
                            # -----------------------------------------------------------------

                            preco1 = preco1 + preco_peca

                            # Relatorio por peça
                            # print('peca:{}, es:{}, l:{}, n:{}, v:{}, l_sobra:{}, ntora:{}, arv:{}, '
                            #       'preco_peca:{}, preco:{}'.format(codpeca, es_medula, l_div, 1, v_medula,
                            #                                        l_sobra, ntora, arv, preco_peca,
                            #                                        preco1))

                        self.calcmadeira_atualiza_tree_resultado(
                            cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                            di, ntora, cod_arv, es_serra, l_div, n, v, 0,
                            l_sobra, es_sobra - n_h * es,
                            es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                            rendimento=vr and (v + v1) / vr or 0, preco=preco_peca, parc=parc,
                            context=context)

                        if l_sobra >= l_min + es_serra:
                            # entra função peças divididas na vertical
                            l_sobra, n1, v1, dados_impressao, preco2 \
                                = pecamenor_quadcircun(l_min, l_max, codigopeca, hi, es,
                                                       es_serra,
                                                       es_sobra, l, l_sobra, n_h,
                                                       pecas_serradas_preco_dados,
                                                       es_medula, escolha, cont_medula)

                            for imp in dados_impressao:
                                self.calcmadeira_atualiza_tree_resultado(
                                    cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                    di_casc2,
                                    di, ntora, cod_arv, es_serra, imp.get('l_div'),
                                    imp.get('n_div'),
                                    imp.get('v'), 0, imp.get('l_sobra'),
                                    es_sobra - n_h * es,
                                    es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                                    rendimento=vr and (imp.get('v') + v1) / vr or 0,
                                    preco=imp.get('preco'), parc=parc,
                                    context=context)

                            l_sobra = l
                            n = 0
                            v = 0

                        else:
                            break

                # min_es não pode ser menor do que es+es_serra!
                if (es_sobra - es_serra) < min_es or es < min_es + es_serra:
                    break

                preco = preco1 + preco2
                preco1 = preco2 = 0

                # espessura restante: eixo horizontal do bloco
                if es_sobra > es + es_medula and es_medula > es_serra \
                        and cont_medula == 0:
                    es_sobra = es_sobra - n_h * es - es_medula
                else:
                    es_sobra = es_sobra - n_h * es

                # if l_sobra >= l_min_pecas + es_serra:
                #     l2 = l1
                #     l1 = l_sobra
                #
                # if l2 > 0:
                #     l1 = l2
                #
                # if es_sobra >= es_min_pecas + es_serra:
                #     l_sobra = l1

                numpeca += n + n1
                volpeca += v + v1
                vr_preco += preco
                if vr > 0:
                    numtora += 1
                    voltora += vr
                    vr_tree = vr
                    vr = 0

            l_sobra = l1

            if selec == 1:
                cont_medula = 1

            # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                           numtora,
                                           numpeca, volpeca, vr_preco, context=context)

        return (codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, vr_preco)

    def calcula_peca_desdobro_radial(self, cr, uid, ids, parametro_dados, context=None):
        # Calcula valor bruto de peças de madeira serrada pelo método de desdobro radial
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        di_casc = di_casc2 = 0
        nes1 = nes2 = es1 = es2 = 0
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')

        arvores_ids = obj_dados_arvores.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_precos_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_precos_ids, [], context)
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        cod_arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # Executa por árvore
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial'),
                 i.get('cmaarv_arvore'),
                 i.get('cmaarv_linha')
                 ])
            narv += 1

        falha = 0
        arv_proces = 0
        conic_tora1 = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                dap = t[0]
                ht = t[1]
                hc = t[2]
                cod_arv = t[3]
                parc = t[4]

                if context.get('serrada_e_rolica'):
                    di_limite = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
                    sql = """
                    SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id
                    """ % brw_calcmadeira_calcula.company_id.id
                    cr.execute(sql)
                    taper_dados = cr.dictfetchall()
                    # --------------------------------------
                    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
                    # --------------------------------------

                    # --------------------------------------
                    hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                    hi2 = hi1.hi_result()
                    # ---------------------------------------

                    if hi2 < hi:
                        hi2 = 0

                    hc = hi2

                # conta árvores processadas
                if hc > 0 and dap > 0 and ht > 0:
                    arv_proces = arv_proces + 1

                # Seleciona equação de volume
                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         hc, context)

                if not dap or not ht:
                    falha += 1

                arv = arv + 1
                # numero da tora
                ntora = 0
                v = 0
                n = 0
                vr = 0

                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    R_flecha_base = 0
                    s = 0
                    es_medula = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt = vrt + vr
                    l = di / 2
                    if l < l_min_pecas + es_serra or di/2 < es_min_pecas + es_serra:
                        break
                    l_e = l_d = l
                    l_i = l_s = l
                    l_sobra = 0.0000
                    nes1 = nes2 = 0.0
                    es1 = es2 = 0.0
                    ang_d = 90.0000
                    ang_s = 0.0000
                    ang_e = ang_i = 0
                    fatia = 0
                    n1 = v1 = 0

                    # Verifica se pelo menos a menor peça será selecionada
                    es = es_min_pecas + es_serra
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------
                    es = 0

                    while l_d >= l_min_pecas + es_serra or l_s >= l_min_pecas + es_serra:
                        if fatia == 0:
                            codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                            fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, \
                            nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                                = self.seleciona_peca_desdobro_radial_f0(cr, uid, ids,
                                                                         l_min_pecas, es_min_pecas,
                                                                         ang_e, ang_d, ang_i, ang_s,
                                                                         vr, es_serra, es, es1, es2,
                                                                         nes1,
                                                                         nes2, di, hi, l_e, l_d,
                                                                         l_i, l_s,
                                                                         l_sobra, ntora, arv,
                                                                         cod_arv,
                                                                         di_casc, di_casc2,
                                                                         pecas_serradas_preco_dados,
                                                                         fatia, parc, context)
                            fatia = 1
                            # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                            # atualiza-lo
                            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                           voltora, numtora,
                                                           numpeca * 4, volpeca * 4, preco * 4,
                                                           context=context)
                        if fatia == 1:
                            codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                            fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, \
                            nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                                = self.seleciona_peca_desdobro_radial_f1(cr, uid, ids,
                                                                         l_min_pecas, es_min_pecas,
                                                                         ang_e, ang_d, ang_i, ang_s,
                                                                         vr, es_serra, es, es1, es2,
                                                                         nes1, nes2, di, hi, l_e,
                                                                         l_d, l_i, l_s,
                                                                         l_sobra, ntora, arv,
                                                                         cod_arv,
                                                                         di_casc, di_casc2,
                                                                         pecas_serradas_preco_dados,
                                                                         fatia, parc, context)
                            fatia = 0
                            # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                            # atualiza-lo
                            self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                           voltora, numtora,
                                                           numpeca * 4, volpeca * 4, preco * 4,
                                                           context=context)
                            if l_d < es_min_pecas + es_serra and l_s < es_min_pecas + es_serra:
                                break
                    # atualiza hi_atual com o comprimento da peça extraída (hi)
                    hi_atual = hi_atual + hi

                if context.get('serrada_e_rolica'):
                    self.calcula_peca_rolica_pta_serrad(
                        cr, uid, ids, prop_casc, cod_arv, dap, ht, hi_atual, arv, parc, context)

                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')
                # acumula vt arvore e zera vt
                vt1 += vt
                vt = 0
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                ntora = 0
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v, 'arvore_processada': arv_proces, 'falha': falha},
                       context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def seleciona_peca_desdobro_radial_f0(self, cr, uid, ids, l_min_pecas, es_min_pecas, ang_e,
                                          ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
                                          nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, arv,
                                          cod_arv,
                                          di_casc, di_casc2, pecas_serradas_preco_dados,
                                          fatia, parc, context=None):
        es = 0
        selec = 0
        l_sobra = l_d
        es_dif = 0
        p = 0

        preco = 0

        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas = brw_calcmadeira_calcula[0].pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)
        es_sobra = 0
        vr_tree = 0
        numtora = 0
        obj_dados_peca = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_peca.search(cr, uid, [('id', 'in', pecas_serradas)],
                                          order='cmapse_sequence',
                                          context=context)
        npecas = len(pecas_ids)
        obj_dados_pecas_dados = obj_dados_peca.read(cr, uid, pecas_ids, context=context)
        v_aux = 0
        n_aux = 0
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapse_codigo_peca')
            namepeca = y.get('name')
            min_es = y.get('cmapse_menor_espessura')
            max_es = y.get('cmapse_maior_espessura')
            l_min = y.get('cmapse_menor_largura')
            l_max = y.get('cmapse_maior_largura')
            voltora = 0
            numtora = 0
            numpeca = 0
            volpeca = 0
            vr_preco = 0
            p += 1

            if ang_d >= 45 and l_d >= l_min + es_serra and \
                    (nes1 + min_es) <= float(di / 2 * math.cos(45 * math.pi / 180)):

                # Considera espessura da serra (fita, corrente motosserra...)
                es = max_es + es_serra

                # retorna valores quando l_d não atende largura mínima
                while (nes1 + es - es_serra) >= float(di / 2 * math.cos(45 * math.pi / 180)):

                    es = es - 5

                selec = 1

                while l_d < l_min + es_serra:
                    es = es - 5

                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    if es < min_es + es_serra:
                        es = 0
                        selec = 0
                        break

                if es < min_es + es_serra:
                    continue

                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                               l_e, l_d, l_i, l_s)
                # ---------------------------------------------
                if l_d < l_min + es_serra:
                    selec = 0

                if selec == 0:
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    while l_d < l_min + es_serra:
                        es = es - 5
                        # ---------------------------------------------
                        ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                            = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i,
                                       ang_s, l_e, l_d, l_i, l_s)
                        # ---------------------------------------------

                        if es < min_es + es_serra:
                            es = 0
                            selec = 0
                            break
                    if es < min_es + es_serra:
                        continue
                # if es < min_es + es_serra:
                #     continue

                if l_d / (l_min + es_serra) >= 2:
                    l_sobra = l_d
                    # entra função peças divididas na vertical
                    fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                        = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es, es_serra,
                                          l_sobra, pecas_serradas_preco_dados)

                    for imp in dados_impressao:
                        self.calcmadeira_atualiza_tree_resultado(
                            cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                            di_casc2,
                            di, ntora, cod_arv, es_serra, imp.get('l_div'),
                            imp.get('n_div'),
                            imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                            es_sobra, es - es_serra, conicidade=(di_casc - di_casc2) / hi,
                            rendimento=vr and (imp.get('v') + v1) / vr or 0, parc=parc,
                            preco=imp.get('preco'), context=context)

                    n = 0
                    v = 0.0000
                    vr_preco += preco
                else:
                    n1 = 0
                    v1 = 0.0000
                    l_sobra = l_d
                    # l_div
                    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
                    l_sobra = l_sobra - l_div
                    # volume da peça
                    # v = ((es - es_serra) / 100) * l_min / 100 * hi
                    v=((es-es_serra)/1000.0)*(l_div-es_serra)/1000.0*hi
                    n = 1

                    es_tmp = es
                    l_tmp = l_div
                    n_tmp = n
                    v_tmp = v
                    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                    # referencia
                    preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codigopeca,
                                            hi, v_tmp)
                    vr_preco += preco_peca

                    self.calcmadeira_atualiza_tree_resultado(
                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                        di, ntora,
                        cod_arv, es_serra, l_div, n, v, fatia, l_sobra,
                        es_sobra, es - es_serra, parc=parc,
                        conicidade=(di_casc - di_casc2) / hi, rendimento=vr and (v + v1) / vr or 0,
                        preco=preco_peca, context=context)

                    v_aux = v
                    n_aux = n

                    if l_sobra >= l_min + es_serra:
                        # entra função peças divididas na vertical
                        fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                            = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es,
                                              es_serra, l_sobra, pecas_serradas_preco_dados)

                        for imp in dados_impressao:
                            self.calcmadeira_atualiza_tree_resultado(
                                cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                di_casc2,
                                di, ntora, cod_arv, es_serra, imp.get('l_div'),
                                imp.get('n_div'),
                                imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                                es_sobra, es - es_serra,
                                conicidade=(di_casc - di_casc2) / hi,
                                rendimento=vr and (imp.get('v') + v1) / vr or 0, parc=parc,
                                preco=imp.get('preco'), context=context)

                        n = 0
                        v = 0.0000
                        vr_preco += preco
                        preco = 0
                # define espessura acumulada na Fatia0(das peças selecionadas)
                numpeca += n + n1
                volpeca += v + v1
                if vr > 0:
                    numtora += 1
                    voltora += vr
                    vr_tree = vr
                    vr = 0

                selec = 1

                # zera variáveis da seção da árvore
                n = 0
                n1 = 0
                v = 0
                v1 = 0
                # es = 0

                preco = 0

            if l_sobra < l_min_pecas + es_serra:
                break

            # Aproveitamento do l_sobra com outro tipo de peça no mesmo es
            elif p < npecas and l_sobra >= l_min_pecas + es_serra and es >= es_min_pecas + es_serra:
                continue

        # define espessura acumulada na Fatia0(das peças selecionadas)

        if es > 0 and selec == 1:
            nes1 = nes1 + es
            es1 = es
            l_s = l_s - es1
        else:
            l_s = 0

        es_sobra = l_s

        if context.get('funcoes_serrada_tora'):
            numpeca *= 4
            volpeca *= 4
            vr_preco *= 4

        return (codigopeca, namepeca, voltora, numtora, numpeca, volpeca,
                fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
                nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, vr_preco)

    def seleciona_peca_desdobro_radial_f1(self, cr, uid, ids, l_min_pecas, es_min_pecas, ang_e,
                                          ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1,
                                          nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, arv,
                                          cod_arv,
                                          di_casc, di_casc2, pecas_serradas_preco_dados,
                                          fatia, parc, context=None):

        selec = 0
        es = 0
        l_sobra = l_s

        preco = 0

        p = 0
        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas = brw_calcmadeira_calcula[0].pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas:
            pecas_serradas.append(i.id)
        es_sobra = 0
        vr_tree = 0
        numtora = 0
        n_aux = 0
        v_aux = 0
        obj_dados_peca = self.pool.get('calcmadeira.pecas.serradas')
        pecas_ids = obj_dados_peca.search(cr, uid, [('id', 'in', pecas_serradas)],
                                          order='cmapse_sequence',
                                          context=context)
        npecas = len(pecas_ids)
        obj_dados_pecas_dados = obj_dados_peca.read(cr, uid, pecas_ids, context=context)
        for y in obj_dados_pecas_dados:
            codigopeca = y.get('cmapse_codigo_peca')
            namepeca = y.get('name')
            min_es = y.get('cmapse_menor_espessura')
            max_es = y.get('cmapse_maior_espessura')
            l_min = y.get('cmapse_menor_largura')
            l_max = y.get('cmapse_maior_largura')
            voltora = 0
            numtora = 0
            numpeca = 0
            volpeca = 0
            vr_preco = 0
            p += 1

            if ang_s <= 45 and l_s >= l_min + es_serra and \
                    (nes2 + min_es) <= float(di / 2 * math.cos(45 * math.pi / 180)):

                # Considera espessura da serra (fita, corrente motosserra...)
                es = max_es + es_serra

                # retorna valores quando l_d não atende largura mínima
                while (nes2 + es - es_serra) >= float(di / 2 * math.cos(45 * math.pi / 180)):
                    es = es - 5

                selec = 1

                while l_s < l_min + es_serra:
                    es = es - 5

                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    if es < min_es + es_serra:
                        es = 0
                        selec = 0
                        break

                if es < min_es + es_serra:
                    continue

                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                               l_e, l_d, l_i, l_s)
                # ---------------------------------------------
                if l_s < l_min + es_serra:
                    selec = 0

                if selec == 0:
                    # ---------------------------------------------
                    ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                        = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s,
                                   l_e, l_d, l_i, l_s)
                    # ---------------------------------------------

                    while l_s < l_min + es_serra:
                        es = es - 5
                        # ---------------------------------------------
                        ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                            = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i,
                                       ang_s, l_e, l_d, l_i, l_s)
                        # ---------------------------------------------

                        if es < min_es + es_serra:
                            es = 0
                            selec = 0
                            break
                    if es < min_es + es_serra:
                        continue
                # if es < min_es + es_serra:
                #     continue

                # Se l permite divisão no mínimo em 2 grupos de peças selecionadas (p) em l
                if l_s / (l_min + es_serra) >= 2:
                    l_sobra = l_s
                    # entra função peças divididas na vertical
                    fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                        = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es, es_serra,
                                          l_sobra, pecas_serradas_preco_dados)

                    for imp in dados_impressao:
                        self.calcmadeira_atualiza_tree_resultado(
                            cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                            di_casc2,
                            di, ntora, cod_arv, es_serra, imp.get('l_div'),
                            imp.get('n_div'),
                            imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                            es_sobra, es - es_serra,
                            conicidade=(di_casc - di_casc2) / hi,
                            rendimento=vr and (imp.get('v') + v1) / vr or 0, parc=parc,
                            preco=imp.get('preco'), context=context)

                    n = 0
                    v = 0.0000
                    vr_preco += preco
                else:
                    n1 = 0
                    v1 = 0.0000
                    l_sobra = l_s
                    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
                    l_sobra = l_sobra - l_div
                    # volume da peça
                    # v = ((es - es_serra) / 100) * l_min / 100 * hi
                    v = ((es - es_serra) / 1000.0) * (l_div-es_serra)/ 1000.0 * hi
                    n = 1

                    es_tmp = es
                    l_tmp = l_div
                    n_tmp = n
                    v_tmp = v
                    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de
                    # referencia
                    preco_peca = calc_preco_sig(pecas_serradas_preco_dados, codigopeca,
                                            hi, v_tmp)
                    vr_preco += preco_peca

                    self.calcmadeira_atualiza_tree_resultado(
                        cr, uid, ids, codigopeca, namepeca, vr, di_casc, di_casc2,
                        di, ntora,
                        cod_arv, es_serra, l_div, n, v, fatia, l_sobra,
                        es_sobra, es - es_serra, parc=parc,
                        conicidade=(di_casc - di_casc2) / hi, rendimento=vr and (v + v1) / vr or 0,
                        preco=preco_peca, context=context)

                    v_aux = v
                    n_aux = n
                    if l_sobra >= l_min + es_serra:
                        # entra função peças divididas na vertical
                        fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco \
                            = pecamenor_fatia(fatia, l_min, l_max, codigopeca, hi, es, es_serra,
                                              l_sobra, pecas_serradas_preco_dados)

                        for imp in dados_impressao:
                            self.calcmadeira_atualiza_tree_resultado(
                                cr, uid, ids, codigopeca, namepeca, vr, di_casc,
                                di_casc2,
                                di, ntora, cod_arv, es_serra, imp.get('l_div'),
                                imp.get('n_div'),
                                imp.get('v'), imp.get('fatia'), imp.get('l_sobra'),
                                es_sobra, es - es_serra,
                                conicidade=(di_casc - di_casc2) / hi,
                                rendimento=vr and (imp.get('v') + v1) / vr or 0, parc=parc,
                                preco=imp.get('preco'), context=context)

                        n = 0
                        v = 0.0000
                        vr_preco += preco
                        preco1 = 0
                # aviso de troca de fatia (selecionou peça)
                # peca = 1
                # define espessura acumulada na Fatia1(das peças selecionadas)
                numpeca += n + n1
                volpeca += v + v1
                if vr > 0:
                    numtora += 1
                    voltora += vr
                    vr_tree = vr
                    vr = 0

                selec = 1

                # zera variáveis da seção da árvore
                n = 0
                n1 = 0
                v = 0.0000
                v1 = 0.0000

                preco = 0

            if l_sobra < l_min_pecas + es_serra:
                break

            # Aproveitamento do l_sobra com outro tipo de peça no mesmo es
            elif p < npecas \
                    and l_sobra >= l_min_pecas + es_serra \
                    and es >= es_min_pecas + es_serra:
                continue

        # define espessura acumulada na Fatia1(das peças selecionadas)
        if es > 0 and selec == 1:
            nes2 = nes2 + es
            es2 = es
            l_d = l_d - es2
        else:
            l_d = 0

        es_sobra = l_d

        if context.get('funcoes_serrada_tora'):
            numpeca *= 4
            volpeca *= 4
            vr_preco *= 4

        return (
            codigopeca, namepeca, voltora, numtora, numpeca, volpeca, fatia, ang_e, ang_d,
            ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s,
            l_sobra, ntora, vr_preco)

    def calcula_peca_laminacao(self, cr, uid, ids, parametro_dados, context=None):
        context = context and context.copy() or {}
        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        obj_calcmadeira_arvore = self.pool.get('calcmadeira.arvores')
        calcmadeira_arvore_ids = obj_calcmadeira_arvore.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        calcmadeira_arvore_dados = obj_calcmadeira_arvore.read(
            cr, uid, calcmadeira_arvore_ids, [], context)

        di_eixo = parametro_dados.get('cmapar_laminacao_diametro_minimo')
        self.calcmadeira_valida_parametro_laminacao(cr, uid, parametro_dados, context)
        comp_folha = parametro_dados.get('cmapar_laminacao_comp_folha')
        linarv, narv = calcmadeira_arvore_dados, len(calcmadeira_arvore_dados)
        di_casc = di_casc2 = 0

        # numero de toras por amostra nas primeiras peças selecionadas
        # a1 = b1 = c1 = d1 = e1 = f1 = g1 = h1 = k1 = 0

        # volume de toras por amostra nas primeiras peças selecionadas
        # vra1 = vrb1 = vrc1 = vrd1 = vre1 = vrf1 = vrg1 = vrh1 = vrk1 = 0

        # numero de peças por tipo por amostra
        # na1 = nb1 = nc1 = nd1 = ne1 = nf1 = ng1 = nh1 = nk1 = 0

        # volume de peças por tipo por amostra
        # va1 = vb1 = vc1 = vd1 = ve1 = vf1 = vg1 = vh1 = vk1 = 0.0000

        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        comp_lamin1 = 0.0000
        comp_lamin1_eixo = 0.0000
        comp_lamin_total = 0

        di = 100

        # -------------------------------------------
        es_serra = parametro_dados.get('cmapar_es_serra')
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        hi = parametro_dados.get('cmapar_altura')
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        # -------------------------------------------

        # numero da árvore
        arv = 0
        cod_arv = 0

        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, calcmadeira_arvore_dados, parametro_dados,
            context)

        falha = 0
        arv_proces = 0

        # Executa por árvore
        while arv < narv:
            v = 0
            n = 0
            vr = 0

            a = b = c = d = e = f = g = h = k = 0
            vra = vrb = vrc = vrd = vre = vrf = vrg = vrh = vrk = 0.0000
            na = nb = nc = nd = ne = nf = ng = nh = nk = 0
            va = vb = vc = vd = ve = vf = vg = vh = vk = 0.0000

            # Acessa arquivo texto, armazena dap e ht por árvore, e calcula seu volume (vt)
            dados = linarv[arv]

            dap = float(dados.get('cmaarv_diametro'))
            ht = float(dados.get('cmaarv_altura'))
            hc = float(altura_comercial_calculada_dict.get(dados.get('id'))
                       if altura_comercial_calculada_dict else dados.get('cmaarv_altura_comecial'))
            cod_arv = dados.get('cmaarv_arvore')
            parc = dados.get('cmaarv_linha')

            # conta árvores processadas
            if hc > 0 and dap > 0 and ht > 0:
                arv_proces = arv_proces + 1

            # -----------------------------
            # Seleciona equação de volume
            vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                     hc, context)
            # -----------------------------

            if not dap or not ht:
                falha += 1

            arv = arv + 1
            # numero da tora
            ntora = 0
            es_sobra = 100

            # Calcula peças por seção do tronco até completar altura da árvore
            while hi_atual + hi <= hc:

                flecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                R_flecha_base = 0
                s = 0
                es_medula = 0

                # -------------------------------------------------
                # Seleciona equação de forma
                di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                       coeficiente_forma=coeficiente_forma,
                                                                       hi_atual=hi_atual,
                                                                       hi=hi,
                                                                       ht=ht,
                                                                       dap=dap,
                                                                       di_casc=di_casc,
                                                                       di_casc2=di_casc2,
                                                                       context=context)
                # -------------------------------------------------

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                ntora = ntora + 1
                # print('vr:{}, di_casc2:{}, di_casc:{}, di:{}, ntora:{},arv:{}'. \
                #       format(vr, di_casc2, di_casc, di, ntora, arv))

                vrt = vrt + vr

                # 'perímetro com espessura es
                es_perim = parametro_dados.get('cmapar_laminacao_espessura')  # 0.3
                # '(3 mm)

                # 'numero de voltas totais
                n = int(di / 2 / es_perim)

                for t in range(0, n + 1):
                    # 'comprimento da lamina (cm)

                    comp_lamin = math.pi *(di-t*2*es_perim)

                    if (di - t * 2 * es_perim) < di_eixo:
                        break

                    comp_lamin1 = comp_lamin1 + comp_lamin

                comp_lamin_total += comp_lamin1

                # 'Relatório de dimensões da laminação

                # 'armazena a linha de gravação dos dados em Relat_Serrad
                # print(
                #     'compr. lamin.: {}, num. tora: {}, vol.tora: {}'.format(comp_lamin, ntora, vr))
                self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, 0, '', vr, di_casc, di_casc2,
                                                         di, ntora, cod_arv, 0, 0, 0, 0, 0, 0, 0, 0,
                                                         comp_lamin1 / 1000, parc=parc,
                                                         context=context)
                comp_lamin1 = 0

                # atualiza hi_atual com o comprimento da tora calculada (hi)
                hi_atual = hi_atual + hi
                # print('hi_atual=', hi_atual)

            # acumula vt da árvore e zera vt
            vt1 = vt1 + vt
            vt = 0

            # 'acumula numero de toras da arvore (ntora) para resultado total de toras na amostra
            ntora1 = ntora1 + ntora
            ntora = 0

            # retorna altura do toco
            hi_atual = parametro_dados.get('cmapar_altura_corte')

            # Alterado em 25/09/19
            ntora = ntora + 1

        # print('Sarrafo:{}, {}, {}, {}'.format(a1,vra1, na1, va1))
        # print('Ripão:{}, {}, {}, {}'.format(b1,vrb1, nb1, vb1))
        # print('Caibro:{}, {}, {}, {}'.format(c1,vrc1, nc1, vc1))
        # print('Pontalete:{}, {}, {}, {}'.format(d1,vrd1, nd1, vd1))
        # print('Ripa:{}, {}, {}, {}'.format(e1,vre1, ne1, ve1))
        # print('Viga:{}, {}, {}, {}'.format(f1,vrf1, nf1, vf1))
        # print('Tabua:{}, {}, {}, {}'.format(g1,vrg1, ng1, vg1))
        # print('Prancha:{}, {}, {}, {}'.format(h1,vrh1, nh1, vh1))
        # print('Pranchão:{}, {}, {}, {}'.format(k1,vrk1, nk1, vk1))
        # print('Volume tora total:{}, Volume total:{}, total de toras:{}'.format(vrt, vt1, ntora1))
        # print('Arvores:{}'.format(narv))

        # varcolunas

        # codpeca=[1,2,3,4,5,6,7,8,9]
        #
        # nomepeca=['Sarrafo','Ripão','Caibro','Pontalete','Ripa','Viga','Tabua','Prancha','Pranchão']
        #
        # num_tora=[a1, b1, c1, d1, e1, f1, g1, h1, k1]
        #
        # vol_tora=[vra1, vrb1,vrc1,vrd1, vre1, vrf1, vrg1, vrh1, vrk1]
        #
        # if escolha=='4':
        #     num_peca = [na1 * 4, nb1 * 4, nc1 * 4, nd1 * 4, ne1 * 4, nf1 * 4, ng1 * 4, nh1 * 4, nk1 * 4]
        #
        #     vol_peca = [va1 * 4, vb1 * 4, vc1 * 4, vd1 * 4, ve1 * 4, vf1 * 4, vg1 * 4, vh1 * 4, vk1 * 4]
        # else:
        #     num_peca=[na1, nb1, nc1, nd1, ne1, nf1, ng1, nh1, nk1]
        #
        #     vol_peca=[va1, vb1,vc1,vd1, ve1, vf1, vg1, vh1, vk1]
        #
        # #create a dictionary
        # data={'codpeca':codpeca,'nomepeca':nomepeca, 'numtora': num_tora, 'vol_tora':vol_tora, 'num_peca': num_peca, 'vol_peca': vol_peca}
        #
        # #create dataframe
        # df=pd.DataFrame(data)

        # print('')
        # print('-'*70)
        # print(df)
        # print('-' * 70)
        # print('')
        # print('-' * 20)
        # print('Toras totais: %s' % ntora1)
        # print('-' * 20)
        # print('Volume tora total: %s' % vrt)
        # print('-' * 20)
        # print('Volume total: %s' % vt1)
        # print('-' * 20)
        # print('comprim. total de laminação: %s' % comp_lamin1)
        # print('-' * 20)
        # print('Arvores: %s' % narv)
        # print('-' * 20)
        total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                            context=context)
        self.write(cr, uid, ids, {
            'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr, 'total_vt_v': total_vt_v,
            'quantidade_tora_total_laminada': ntora1, 'volume_tora_total': vrt,
            'comprimento_laminacao_total': comp_lamin_total / 1000,
            'folha_laminacao_total': int(comp_lamin_total / 1000 / parametro_dados.get(
                'cmapar_laminacao_comp_folha')) if parametro_dados.get(
                'cmapar_laminacao_comp_folha') else 0
            }, context=context)
        return True

    def calcula_peca_fator_forma(self, cr, uid, ids, parametro_dados, context=None):

        def Calcula_Part_di2_hi_difer(lintaper, taper, cont, hi_atual, ht):

            i = 1
            V_tora = 0

            while i <= cont:

                taper_aux = taper - 1
                dados = lintaper[taper_aux]
                di2 = float(dados.get('cmatap_di_casc')) ** 2
                hi_acum = float(dados.get('cmatap_hi'))

                if i == 1:
                    V_toc = math.pi / 40000 * di2 * hi_acum

                if i > 1 and i <= cont:
                    dados_1 = lintaper[taper_aux - 1]
                    di2_1 = float(dados_1.get('cmatap_di_casc')) ** 2
                    hi = float(dados.get('cmatap_hi')) - float(dados_1.get('cmatap_hi'))

                    V_tora = V_tora + (math.pi / 40000 * (di2_1 + di2) / 2) * hi

                if float(hi_acum) == 1.3:
                    dap = di2 ** (1 / 2.0)
                    dap2 = di2

                if i == cont:
                    hi_pta = float(ht) - float(hi_acum)
                    V_pta = math.pi / 40000 * di2 * hi_pta * 1 / 3

                i = i + 1
                taper = taper + 1

            V_cub_arv = V_toc + V_tora + V_pta

            hi1 = 0
            hi2 = 1
            taper = taper - 1
            cont = 0

            return dap, dap2, hi1, hi2, taper, cont, V_cub_arv

        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        obj_calcmadeira_taper = self.pool.get('calcmadeira.taper')
        calcmadeira_taper_ids = obj_calcmadeira_taper.search(cr, uid, [], context=context)
        calcmadeira_taper_dados = obj_calcmadeira_taper.read(cr, uid, calcmadeira_taper_ids, [],
                                                             context)
        # -----------------------
        lintaper, ntaper = calcmadeira_taper_dados, len(calcmadeira_taper_dados)
        # -----------------------

        vt = 0.0000
        vt1 = 0.0000
        ntora1 = 0

        # declarando variáveis, não necessário no VBA
        di = 100

        # Parametros: começa pela altura do toco (pode ser alterada pelo usuário)
        # -----------------------------------------------------------------------
        es_serra = parametro_dados.get('cmapar_es_serra')
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        hi = parametro_dados.get('cmapar_altura')
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
        es_medula = parametro_dados.get('es_medula')
        # -----------------------------------------------------------------------

        # numero da árvore
        arv = 1

        # percorre todos os dados. taper é o 'r' no VBA, que começa na linha 2. r=2.
        # '1 limita o 1o dado que não tem com quem comparar
        taper = 0

        # limita cada árvore
        cont = 0

        # para iniciar sem os dados
        hi = 0
        hi1 = 0
        hi2 = 1
        FFarv1 = 0.00
        FFarv1_2 = 0.00
        
        cubagem_dados = []

        # Executa por árvore
        while taper < ntaper:

            while hi1 < hi2:
                cont = cont + 1
                taper = taper + 1
                taper_aux = taper - 1
                try:
                    dados = lintaper[taper_aux]
                    hi = float(dados.get('cmatap_hi'))

                    dados1 = lintaper[taper_aux + 1]
                    hi1 = float(dados1.get('cmatap_hi'))

                    dados2 = lintaper[taper_aux + 2]
                    hi2 = float(dados2.get('cmatap_hi'))

                    # print('hi={}, hi1={}, hi2={}'.format(hi, hi1, hi2))

                except:
                    break
                    # pass

            try:
                ht = float(dados1.get('cmatap_ht'))
            except:
                print('Faltou altura total na coluna Ht')

            taper = taper + 1 - cont
            cont = cont + 1

            # -----------------------------------------------------------------------------------
            dap, dap2, hi1, hi2, taper, cont, V_cub_arv = Calcula_Part_di2_hi_difer(
                lintaper, taper, cont, hi_atual, ht)
            # -----------------------------------------------------------------------------------

            FFarv = V_cub_arv / (math.pi / 40000 * dap2 * ht)

            vt = V_cub_arv

            # print('arv=', arv, 'V_cub_arv=', V_cub_arv, 'FFarv=', FFarv)
            self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, 0, '', V_cub_arv, 0, 0,
                                                     0, 0, arv, 0, 0, 0, 0, 0, 0, 0, 0,
                                                     0, FFarv, context=context)

            cubagem_dados.append({'cmacub_linha': dados1.get('cmatap_lin'),
                                  'cmacub_arvore': arv,
                                  'cmacub_diametro': dap,
                                  'cmacub_altura': dados1.get('cmatap_ht'),
                                  'cmacub_altura_comercial': dados1.get('cmatap_ht'),
                                  'cmacub_volume': V_cub_arv})

            FFarv1 = FFarv1 + FFarv
            FFarv1_2 = FFarv1_2 + FFarv ** 2

            arv = arv + 1

        FFarv1_med = FFarv1 / (arv - 1)

        # 'formula para dp =((Sx^2-((Sx)^2/n)/(n-1))^(1/2)
        # 'Uso arv-1 e arv-2, pq o acumulado de arv vem como arv+1, ou seja, se existe apenas 1 arv, arv=2
        # 'Para calculo de dp são necessários pelo menos 3 dados, então arv > 3
        if arv > 3:
            FFarv1_dp = ((FFarv1_2 - FFarv1 ** 2 / (arv - 1)) / (arv - 2)) ** (1.0 / 2)

        print('Fator de Forma Médio: {}'.format(FFarv1_med))
        print('Desvio Padrão Fator de Forma: {}'.format(FFarv1_dp))

        self.write(cr, uid, ids, {'narv': arv > 0 and arv - 1 or 0, 'fator_forma_medio': FFarv1_med,
                                  'fator_forma_desvio_padrao': FFarv1_dp}, context=context)

        obj_calcmadeira_cubagem = self.pool.get('calcmadeira.cubagem')
        cubagem_dados and obj_calcmadeira_cubagem.unlink(cr, uid, obj_calcmadeira_cubagem.search(
            cr, uid, [], context=context), context)
        for cubagem in cubagem_dados:
            obj_calcmadeira_cubagem.create(cr, uid, cubagem, context)

        return True

    def calcula_peca_gira_180_90(self, cr, uid, ids, parametro_dados, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        coeficiente_volume = brw_calcmadeira_calcula.coeficiente_volume_id
        coeficiente_forma = brw_calcmadeira_calcula.coeficiente_forma_id
        # limpa os dados da tree
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        di_casc = di_casc2 = 0
        vt = 0.0000
        vt1 = 0.0000
        vrt = 0.0000
        ntora = 0
        ntora1 = 0
        preco_medula = 0.0000
        obj_dados_arvores = self.pool.get('calcmadeira.arvores')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        arvores_ids = obj_dados_arvores.search(
            cr, uid, [] + self.calcmadeira_trata_talhao_peca(cr, uid, ids, context),
            context=context)
        obj_calcmadeira_arvores_dados = obj_dados_arvores.read(cr, uid, arvores_ids,
                                                               context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # altura do toco (m)
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
        es_medula = parametro_dados.get('es_medula')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Comprimento da peça(m)
        hi = parametro_dados.get('cmapar_altura')
        arv = 0
        cod_arv = 0
        narv = 0
        list1 = []
        altura_comercial_calculada_dict = self.get_altura_comercial_calculada_dados(
            cr, uid, brw_calcmadeira_calcula, obj_calcmadeira_arvores_dados, parametro_dados,
            context)
        # verifica quantidade de arvores
        for i in obj_calcmadeira_arvores_dados:
            list1.append(
                [i.get('cmaarv_diametro'),
                 i.get('cmaarv_altura'),
                 altura_comercial_calculada_dict.get(i.get('id'))
                 if altura_comercial_calculada_dict else i.get('cmaarv_altura_comecial'),
                 i.get('cmaarv_arvore'),
                 i.get('cmaarv_linha'),
                 ])
            narv += 1

        falha = 0
        arv_proces = 0
        conic_tora1 = 0

        # Executa por árvore
        while arv < narv:
            for t in list1:

                dap = t[0]
                ht = t[1]
                hc = t[2]
                cod_arv = t[3]
                parc = t[4]

                if context.get('serrada_e_rolica'):
                    di_limite = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
                    sql = """
                    SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id
                    """ % brw_calcmadeira_calcula.company_id.id
                    cr.execute(sql)
                    taper_dados = cr.dictfetchall()
                    # --------------------------------------
                    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
                    # --------------------------------------

                    # --------------------------------------
                    hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                    hi2 = hi1.hi_result()
                    # ---------------------------------------

                    if hi2 < hi:
                        hi2 = 0

                    hc = hi2

                # conta árvores processadas
                if hc > 0 and dap > 0 and ht > 0:
                    arv_proces = arv_proces + 1

                vt = self.calcmadeira_get_equacao_volume(cr, uid, coeficiente_volume, dap, ht,
                                                         hc, context)

                if not dap or not ht:
                    falha += 1

                arv += 1
                # numero da tora
                ntora = 0
                v = 0
                n = 0
                vr = 0
                es_sobra = 100
                # Calcula peças por seção do tronco até completar altura da árvore
                while hi_atual + hi <= hc:

                    flecha = flecha0 = 0
                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    r_flecha_base = 0
                    s = 0

                    # Seleciona equação de forma
                    di_casc, di_casc2 = self.calcmadeira_get_equacao_forma(cr, uid,
                                                                           coeficiente_forma=coeficiente_forma,
                                                                           hi_atual=hi_atual,
                                                                           hi=hi,
                                                                           ht=ht,
                                                                           dap=dap,
                                                                           di_casc=di_casc,
                                                                           di_casc2=di_casc2,
                                                                           context=context)

                    # volume da tora usando formula de Smalian
                    vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                    di = di_casc2 * (1 - prop_casc)*10
                    # armazena a linha de gravação dos dados para relatório por peça
                    ntora = ntora + 1
                    vrt = vrt + vr
                    # angulo inicial para desconto do di para cada lado (flecha0)
                    ang_d = parametro_dados.get('cmapar_ang_direito')
                    ang_base = 0
                    flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                    # largura da peça (corda)
                    if ang_d < 90 - ang_base or ang_base == 0:
                        l = di * math.sin(ang_d * math.pi / 180)
                    else:
                        l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                    if di < l_min_pecas + es_serra:
                        break

                    cord_d1 = 0
                    cord_d = 0
                    r_flecha_topo = 0
                    l_sobra = l

                    for s in range(0, 1 + 1):
                        codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, \
                        preco, l, cord_d1, cord_d, r_flecha_topo, s = \
                            self.seleciona_peca_longitudinal_base(
                                cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di,
                                flecha0,
                                r_flecha_base, l, ntora, arv, cod_arv, di_casc, di_casc2,
                                pecas_serradas_preco_dados, parametro_dados, ang_base, flecha_div_r,
                                cord_d1, cord_d, r_flecha_topo, s, parc, context=context)
                        # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                        # atualiza-lo
                        self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                       numtora, numpeca, volpeca, preco,
                                                       context=context)
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                        = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids,
                                                                    l_min_pecas, es_min_pecas,
                                                                    es_serra, hi,
                                                                    vr,
                                                                    l, ntora, arv, cod_arv, di,
                                                                    di_casc, di_casc2,
                                                                    pecas_serradas_preco_dados,
                                                                    cord_d, es_medula, flecha0,
                                                                    parc, context=context)

                    hi_atual = hi_atual + hi

                if context.get('serrada_e_rolica'):
                    self.calcula_peca_rolica_pta_serrad(
                        cr, uid, ids, prop_casc, cod_arv, dap, ht, hi_atual, arv, parc, context)

                # atualiza hi_atual com o comprimento da peça extraída (hi)
                # hi_atual = hi_atual + hi
                # 'acumula numero de toras da arvore (ntora) para resultado total de
                # toras na amostra
                ntora1 = ntora1 + ntora
                ntora = 0
                # acumula vt arvore e zera vt
                vt1 = vt1 + vt
                vt = 0
                # acumula vt arvore e zera vt
                vt1 += vt
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # retorna altura do toco
                hi_atual = parametro_dados.get('cmapar_altura_corte')

            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, vt1,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': narv, 'vt1': vt1, 'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v, 'arvore_processada': arv_proces, 'falha': falha},
                       context=context)
        self.calcula_madeira_serrada_por_hectare(cr, uid, brw_calcmadeira_calcula, narv, context)
        return True

    def calcula_tora_longitudinal_base(self, cr, uid, ids, parametro_dados, context=None):
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        brw_calcmadeira_calcula = self.browse(cr, uid, calcmadeira_calcula_id, context)
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')

        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1
        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                cod_arv = arv
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')
                es_sobra = 100

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                flecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                # angulo inicial para retirada da costaneira da base
                ang_base = parametro_dados.get('cmapar_ang_base')
                # comprimento da peça, do eixo x até o limite da costaneira da base (R-flecha)
                r_flecha_base = di / 2 * math.cos(ang_base * math.pi / 180)
                # angulo inicial para desconto de 1/4 do di para cada lado (flecha = 1/4*R)
                ang_d = parametro_dados.get('cmapar_ang_direito')

                if brw_calcmadeira_calcula.tipo_funcao == '16':  # '8' - Radial Longitudinal
                    ang_base = 90
                    r_flecha_base = 0

                flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                # largura da peça (corda)
                if ang_d < (90 - ang_base) or ang_base == 0:
                    l = di * math.sin(ang_d * math.pi / 180)
                else:
                    l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                                    context=context)
                if di < l_min_pecas + es_serra:
                    continue
                # 'ALTERADO
                l_sobra = l
                flecha = 0

                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                    cord_d1, cord_d, r_flecha_topo, s \
                    = self.seleciona_peca_longitudinal_base(
                        cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                        r_flecha_base, l, ntora, arv, cod_arv, di_casc, di_casc2,
                        pecas_serradas_preco_dados,
                        parametro_dados, ang_base, flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
                        0, context=context)

                if brw_calcmadeira_calcula.tipo_funcao == '16':  # '8' - Radial Longitudinal
                    numpeca *= 2
                    volpeca *= 2
                    preco *= 2
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora,
                                               numpeca, volpeca, preco, context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': 0,
                        'vt1': sum(map(
                            lambda it: it.voltora or 0, self.browse(cr, uid, calcmadeira_calcula_id,
                                                                    context).peca_ids)),
                        'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v
                        }, context=context)
            return True

    def calcula_tora_longitudinal(self, cr, uid, ids, parametro_dados, context=None):
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')

        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')

        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1

        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                cod_arv = arv
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                # angulo inicial para desconto de 1/4 do di para cada lado (flecha = 1/4*R)
                ang_d = parametro_dados.get('cmapar_ang_direito')
                ang_base = 0
                flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                # largura da peça (corda)
                if ang_d < (90 - ang_base) or ang_base == 0:
                    l = di * math.sin(ang_d * math.pi / 180)
                else:
                    l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0

                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                                    context=context)
                if di < l_min_pecas + es_serra:
                    continue
                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                    cord_d1, cord_d, r_flecha_topo, s \
                    = self.seleciona_peca_longitudinal_base(
                        cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                        r_flecha_base, l, ntora, arv, cod_arv, di_casc, di_casc2,
                        pecas_serradas_preco_dados,
                        parametro_dados, ang_base, flecha_div_r, cord_d1, cord_d, r_flecha_topo, s,
                        0, context=context)
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora,
                                               numpeca, volpeca, preco, context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {
                'narv': 0,
                'vt1': sum(map(lambda it: it.voltora or 0,
                               self.browse(cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                'total_vt_vr': total_vt_vr,
                'total_vt_v': total_vt_v}, context=context)
            return True

    def calcula_tora_quadrado_circunscrito(self, cr, uid, ids, parametro_dados, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        # vrt = 0.0000
        # ntora = 0
        # di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Executa por tora
        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1
        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                cod_arv = arv
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')
                v = 0
                n = 0
                vr = 0
                es_sobra = 100

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                # largura da peça
                l = di / math.sqrt(2)
                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                if l < l_min_pecas+es_serra:
                    ntora = ntora - 1
                    continue
                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                    = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids, l_min_pecas,
                                                                es_min_pecas, es_serra, hi, vr, l,
                                                                ntora, arv, cod_arv, di, di_casc,
                                                                di_casc2,
                                                                pecas_serradas_preco_dados,
                                                                cord_d, es_medula, flecha0, 0,
                                                                context=context)

                # Funcao para adicionar os dados na tree ou se o mesmo ja existir atualiza-lo
                self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                               numtora,
                                               numpeca, volpeca, preco, context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': 0,
                        'vt1': sum(map(lambda it: it.voltora or 0, self.browse(
                            cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                        'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v}, context=context)
            return True

    def calcula_tora_desdobro_radial(self, cr, uid, ids, parametro_dados, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        vrt = 0.0000
        ntora = 0
        codpeca = 0
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                            context=context)
        # Executa por tora
        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1
        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                cod_arv = arv
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')
                v = 0
                n = 0
                vr = 0

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                # (100 - 6.8% de casca, medido no Eucal Maravilhas)
                # (100- 8.7% casca, medido no citriodora Arv 1)
                # (100- 4.8% casca, medido no citriodora Arv 2)
                # (100- 8.5% casca, medido no citriodora Arv 3)
                # (100 - 9.4% casca, medido na tora 2 da Arv 3)
                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0
                es_medula = 0

                l = di / 2
                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                # l_min_pecas=2 # DimSerrada.txt todas as peças
                if l < l_min_pecas + es_serra or di/2 < es_min_pecas + es_serra:
                    continue

                l_e = l_d = l_i = l_s = l
                l_sobra = 0.0000
                nes1 = nes2 = 0.0
                es1 = es2 = 0.0
                ang_d = 90.0000
                ang_s = 0.0000
                ang_e = ang_i = 0
                fatia = 0
                n1 = v1 = 0

                # Verifica se pelo menos a menor peça será selecionada
                es = es_min_pecas + es_serra
                # ---------------------------------------------
                ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s \
                    = Formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e,
                               l_d, l_i, l_s)
                # ---------------------------------------------
                es = 0

                while l_d >= l_min_pecas + es_serra or l_s >= l_min_pecas + es_serra:
                    if fatia == 0:
                        codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                        fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, \
                        di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                            = self.seleciona_peca_desdobro_radial_f0(
                            cr, uid, ids, l_min_pecas, es_min_pecas, ang_e, ang_d, ang_i, ang_s,vr,
                            es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra,
                            ntora, arv, cod_arv, di_casc, di_casc2, pecas_serradas_preco_dados,
                            fatia, 0, context)
                        fatia = 1
                        # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                        # atualiza-lo
                        self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                       voltora, numtora,
                                                       numpeca, volpeca, preco, context=context)
                    if fatia == 1:
                        codigopeca, namepeca, voltora, numtora, numpeca, volpeca, \
                        fatia, ang_e, ang_d, ang_i, ang_s, vr, es_serra, es, es1, es2, nes1, nes2, \
                        di, hi, l_e, l_d, l_i, l_s, l_sobra, ntora, preco \
                            = self.seleciona_peca_desdobro_radial_f1(
                            cr, uid, ids, l_min_pecas, es_min_pecas, ang_e, ang_d, ang_i, ang_s, vr,
                            es_serra, es, es1, es2, nes1, nes2, di, hi, l_e, l_d, l_i, l_s, l_sobra,
                            ntora, arv, cod_arv, di_casc, di_casc2, pecas_serradas_preco_dados,
                            fatia, 0, context)
                        fatia = 0
                        # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                        # atualiza-lo
                        self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca,
                                                       voltora, numtora,
                                                       numpeca, volpeca, preco, context=context)
                        if l_d < es_min_pecas + es_serra and l_s < es_min_pecas + es_serra:
                            break
                    # atualiza hi_atual com o comprimento da peça extraída (hi)
                # acumula ntora da arvore e zera ntora
                ntora1 += ntora
                # ntora = 0
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id,
                       {'narv': 0,
                        'vt1': sum(map(lambda it: it.voltora or 0, self.browse(
                            cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                        'total_vt_vr': total_vt_vr,
                        'total_vt_v': total_vt_v}, context=context)
            return True

    def calcula_tora_laminacao(self, cr, uid, ids, parametro_dados, context=None):
        context = context and context.copy() or {}
        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        brw_calcmadeira_calcula = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                              context)
        obj_calcmadeira_toras = self.pool.get('calcmadeira.toras')
        tora_ids = obj_calcmadeira_toras.search(cr, uid, [], context=context)
        calcmadeira_toras_dados = obj_calcmadeira_toras.read(cr, uid, tora_ids, [], context)

        lintora, ntora1 = calcmadeira_toras_dados, len(calcmadeira_toras_dados)
        # -----------------------

        di_eixo = parametro_dados.get('cmapar_laminacao_diametro_minimo')
        self.calcmadeira_valida_parametro_laminacao(cr, uid, parametro_dados, context)

        # numero de toras por amostra nas primeiras peças selecionadas
        # a1 = b1 = c1 = d1 = e1 = f1 = g1 = h1 = k1 = 0

        # volume de toras por amostra nas primeiras peças selecionadas
        # vra1 = vrb1 = vrc1 = vrd1 = vre1 = vrf1 = vrg1 = vrh1 = vrk1 = 0

        vrt = 0.0000
        vr1 = 0.0000
        ntora = 0
        comp_lamin1 = 0.0000
        comp_lamin1_eixo = 0.0000
        comp_lamin_total = 0

        # declarando variáveis, não necessário no VBA
        di = 100

        # -------------------------------------------
        es_serra = parametro_dados.get('cmapar_es_serra')
        hi_atual = parametro_dados.get('cmapar_altura_corte')
        hi = parametro_dados.get('cmapar_altura')
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        # -------------------------------------------
        hi_atual = hi = 0

        # Executa por tora
        while ntora < ntora1:
            v = 0
            n = 0
            vr = 0

            a = b = c = d = e = f = g = h = k = 0
            vra = vrb = vrc = vrd = vre = vrf = vrg = vrh = vrk = 0.0000
            na = nb = nc = nd = ne = nf = ng = nh = nk = 0
            va = vb = vc = vd = ve = vf = vg = vh = vk = 0.0000

            # Acessa arquivo texto, armazena dap e ht por árvore, e calcula seu volume (vt)
            dados = lintora[ntora]
            ntora = ntora + 1
            arv = float(dados.get('cmator_arvore'))
            tora = int(dados.get('cmator_tora'))
            di_casc2 = float(dados.get('cmator_diametro_min'))
            di_casc = float(dados.get('cmator_diametro_max'))
            hi = float(dados.get('cmator_comprimento_tora'))

            es_sobra = 100

            # volume da tora usando formula de Smalian
            vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

            # (100 - 6.8% de casca, medido no Eucal Maravilhas)
            # (100- 8.7% casca, medido no citriodora Arv 1)
            # (100- 4.8% casca, medido no citriodora Arv 2)
            # (100- 8.5% casca, medido no citriodora Arv 3)
            # (100 - 9.4% casca, medido na tora 2 da Arv 3)
            di = di_casc2 * (1 - prop_casc)*10

            lecha = flecha0 = 0
            cord_d1 = 0
            cord_d = 0
            r_flecha_topo = 0
            r_flecha_base = 0
            s = 0
            es_medula = 0

            # print('vr:{}, di_casc2:{}, di_casc:{}, di:{}, tora:{},arv:{}'. \
            #       format(vr, di_casc2, di_casc, di, tora, arv))

            # 'perímetro com espessura es
            es_perim = parametro_dados.get('cmapar_laminacao_espessura')  # 0.3
            # '(3 mm)

            # 'numero de voltas
            n = int(di / 2 / es_perim)

            for t in range(0, n + 1):
                # 'comprimento da lamina (cm)

                comp_lamin = math.pi * (di - t * 2 * es_perim)

                if (di - t * 2 * es_perim ) < di_eixo:
                    break

                comp_lamin1 = comp_lamin1 + comp_lamin

            comp_lamin_total += comp_lamin1

            vr1 = vr1 + vr

            # 'Relatório de dimensões da laminação

            # 'armazena a linha de gravação dos dados em Relat_Serrad
            # print('compr. lamin.: {}, num. tora: {}, vol.tora: {}'.format(comp_lamin, ntora, vr))

            # print(
            #     'compr. lamin.: {}, num. tora: {}, vol.tora: {}'.format(comp_lamin, ntora, vr))
            self.calcmadeira_atualiza_tree_resultado(cr, uid, ids, 0, '', vr, di_casc, di_casc2,
                                                     di, ntora, arv, 0, 0, 0, 0, 0, 0, 0, 0,
                                                     comp_lamin1 / 1000,
                                                     context=context)

            comp_lamin1 = 0

        # print('Sarrafo:{}, {}, {}, {}'.format(a1,vra1, na1, va1))
        # print('Ripão:{}, {}, {}, {}'.format(b1,vrb1, nb1, vb1))
        # print('Caibro:{}, {}, {}, {}'.format(c1,vrc1, nc1, vc1))
        # print('Pontalete:{}, {}, {}, {}'.format(d1,vrd1, nd1, vd1))
        # print('Ripa:{}, {}, {}, {}'.format(e1,vre1, ne1, ve1))
        # print('Viga:{}, {}, {}, {}'.format(f1,vrf1, nf1, vf1))
        # print('Tabua:{}, {}, {}, {}'.format(g1,vrg1, ng1, vg1))
        # print('Prancha:{}, {}, {}, {}'.format(h1,vrh1, nh1, vh1))
        # print('Pranchão:{}, {}, {}, {}'.format(k1,vrk1, nk1, vk1))
        # print('Volume tora total:{}, Volume total:{}, total de toras:{}'.format(vrt, vt1, ntora1))
        # print('Arvores:{}'.format(narv))

        # varcolunas

        # codpeca = [1, 2, 3, 4, 5, 6, 7, 8, 9]
        #
        # nomepeca = ['Sarrafo', 'Ripão', 'Caibro', 'Pontalete', 'Ripa', 'Viga', 'Tabua', 'Prancha', 'Pranchão']
        #
        # num_tora = [a1, b1, c1, d1, e1, f1, g1, h1, k1]
        #
        # vol_tora = [vra1, vrb1, vrc1, vrd1, vre1, vrf1, vrg1, vrh1, vrk1]
        #
        # if escolha=='4':
        #     num_peca = [na1 * 4, nb1 * 4, nc1 * 4, nd1 * 4, ne1 * 4, nf1 * 4, ng1 * 4, nh1 * 4, nk1 * 4]
        #
        #     vol_peca = [va1 * 4, vb1 * 4, vc1 * 4, vd1 * 4, ve1 * 4, vf1 * 4, vg1 * 4, vh1 * 4, vk1 * 4]
        # else:
        #     num_peca=[na1, nb1, nc1, nd1, ne1, nf1, ng1, nh1, nk1]
        #
        #     vol_peca=[va1, vb1,vc1,vd1, ve1, vf1, vg1, vh1, vk1]
        #
        # # create a dictionary
        # data = {'codpeca': codpeca, 'nomepeca': nomepeca, 'numtora': num_tora, 'vol_tora': vol_tora, 'num_peca': num_peca,
        #         'vol_peca': vol_peca}
        #
        # # create dataframe
        # df = pd.DataFrame(data)

        # print('')
        # print('-' * 70)
        # print(df)
        # print('-' * 70)
        # print('')
        # print('-' * 20)
        print('número de toras: %s' % ntora)
        # print('-' * 20)
        # print('volume total de toras: %s' % vr1)
        # print('-' * 20)
        # print('comprim. total de laminação: %s' % comp_lamin1)
        # print('-' * 20)
        # # print(f'Arvores: {narv}')
        # print('-' * 20)

        self.write(cr, uid, ids,
                   {'narv': 0, 'vt1': 0, 'total_vt_vr': 0, 'total_vt_v': 0,
                    'quantidade_tora_total_laminada': ntora, 'volume_tora_total': vr1,
                    'comprimento_laminacao_total': comp_lamin_total / 1000,
                    'folha_laminacao_total': int(comp_lamin_total / 1000 / parametro_dados.get(
                        'cmapar_laminacao_comp_folha')) if parametro_dados.get(
                        'cmapar_laminacao_comp_folha') else 0
                    },
                   context=context)

        return True

    def calcula_tora_gira_180_90(self, cr, uid, ids, parametro_dados, context=None):
        self.calcmadeira_limpa_tree(cr, uid, ids, context=context)
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        obj_dados_toras = self.pool.get('calcmadeira.toras')
        obj_pecas_serradas_preco = self.pool.get('calcmadeira.pecas.serradas.preco')
        toras_ids = obj_dados_toras.search(cr, uid, [], order='cmator_arvore',
                                           context=context)
        obj_calcmadeira_toras_dados = obj_dados_toras.read(cr, uid, toras_ids,
                                                           context=context)
        pecas_serradas_preco_ids = obj_pecas_serradas_preco.search(cr, uid, [], context=context)
        pecas_serradas_preco_dados = obj_pecas_serradas_preco.read(
            cr, uid, pecas_serradas_preco_ids, [], context)
        di = 100
        # espessura serra (cm)
        es_serra = parametro_dados.get('cmapar_es_serra')
        # proporção de casca
        prop_casc = parametro_dados.get('cmapar_prop_casc')
        flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
        es_medula = parametro_dados.get('es_medula')
        ntora = 0
        ntora1 = 0
        for i in obj_calcmadeira_toras_dados:
            ntora1 += 1

        while ntora < ntora1:
            for t in obj_calcmadeira_toras_dados:
                ntora = ntora + 1
                arv = t.get('cmator_arvore')
                cod_arv = arv
                tora = t.get('cmator_tora')
                di_casc = t.get('cmator_diametro_max')
                di_casc2 = t.get('cmator_diametro_min')
                hi = t.get('cmator_comprimento_tora')

                # volume da tora usando formula de Smalian
                vr = math.pi/40000 * (di_casc**2 + di_casc2**2) / 2 * hi

                di = di_casc2 * (1 - prop_casc)*10

                lecha = flecha0 = 0
                cord_d1 = 0
                cord_d = 0
                r_flecha_topo = 0
                r_flecha_base = 0
                s = 0

                # angulo inicial para desconto de 1/4 do di para cada lado (flecha = 1/4*R)
                ang_d = parametro_dados.get('cmapar_ang_direito')
                ang_base = 0
                flecha0 = di / 2 * (1 - math.cos(ang_d * math.pi / 180))

                # largura da peça (corda)
                if ang_d < (90 - ang_base) or ang_base == 0:
                    l = di * math.sin(ang_d * math.pi / 180)
                else:
                    l = di * (math.sin(ang_d * math.pi / 180) / 2) + r_flecha_base

                cord_d1 = 0
                cord_d = 0

                # se largura < largura mínima entre todas as peças, resto da árvore não atende dimensões, mesmo com comprimento suficiente da peça
                l_min_pecas, es_min_pecas = self.calmadeira_menor_largura_espessura(cr, uid, ids,
                                                                                    context=context)
                if di < l_min_pecas + es_serra:
                    continue

                l_sobra = l
                flecha = 0
                r_flecha_topo = 0

                for s in range(0, 1 + 1):
                    codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco, l, \
                        cord_d1, cord_d, r_flecha_topo, s \
                        = self.seleciona_peca_longitudinal_base(
                            cr, uid, ids, l_min_pecas, es_min_pecas, es_serra, hi, vr, di, flecha0,
                            r_flecha_base, l, ntora, arv, cod_arv, di_casc, di_casc2,
                            pecas_serradas_preco_dados, parametro_dados, ang_base, flecha_div_r,
                            cord_d1, cord_d, r_flecha_topo, s, 0, context=context)
                    # Funcao para adicionar os dados na tree ou se o mesmo ja existir
                    # atualiza-lo
                    self.calcmadeira_atualiza_tree(cr, uid, ids, codigopeca, namepeca, voltora,
                                                   numtora, numpeca, volpeca, preco,
                                                   context=context)
                codigopeca, namepeca, voltora, numtora, numpeca, volpeca, vr, ntora, preco \
                    = self.seleciona_peca_quadrado_circunscrito(cr, uid, ids,
                                                                l_min_pecas, es_min_pecas,
                                                                es_serra, hi,
                                                                vr,
                                                                l, ntora, arv, cod_arv, di,
                                                                di_casc, di_casc2,
                                                                pecas_serradas_preco_dados,
                                                                cord_d, es_medula, flecha0, 0,
                                                                context=context)
            total_vt_vr, total_vt_v = self.calcmadeira_calcula_volume_peca_tora(cr, uid, ids, 0,
                                                                                context=context)
            self.write(cr, uid, calcmadeira_calcula_id, {
                'narv': 0,
                'vt1': sum(map(lambda it: it.voltora or 0,
                               self.browse(cr, uid, calcmadeira_calcula_id, context).peca_ids)),
                'total_vt_vr': total_vt_vr,
                'total_vt_v': total_vt_v}, context=context)
            return True

    def calcmadeira_atualiza_tree(self, cr, uid, ids, codigopeca, namepeca, voltora, numtora,
                                  numpeca, volpeca, preco_item=0, context=None):
        obj_wizard_calcula_peca_itens = self.pool.get('wizard.calcmadeira.calcula.itens')
        obj_wizard_calcula_rolica_itens = self.pool.get('wizard.calcmadeira.calcula.rolica.itens')
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        if preco_item and not self.read(cr, uid, calcmadeira_calcula_id, ['calcula_receita'],
                                        context).get('calcula_receita'):
            preco_item = 0
        if context.get('metodo_rolica_pta_serrada'):
            itens_ids = obj_wizard_calcula_rolica_itens.search(
                cr, uid, [('capa_id', '=', calcmadeira_calcula_id), ('codpeca', '=', codigopeca)],
                context=context)
            # se a peca ja estiver na tree, apenas atualiza os dados somando a
            # quantidade e volume de cada tora e peca
            if itens_ids:
                peca_itens_dados = obj_wizard_calcula_rolica_itens.read(
                    cr, uid, itens_ids[0],
                    ['numtora', 'voltora', 'numpeca', 'volpeca', 'preco_item'],
                    context=context)
                obj_wizard_calcula_rolica_itens.write(cr, uid, itens_ids[0], {
                    'numtora': numtora + peca_itens_dados.get('numtora'),
                    'voltora': voltora + peca_itens_dados.get('voltora'),
                    'numpeca': numpeca + peca_itens_dados.get('numpeca'),
                    'volpeca': volpeca + peca_itens_dados.get('volpeca'),
                    'preco_item': preco_item + peca_itens_dados.get('preco_item'),
                })
            else:
                # insere na tree a quantidade e volume de cada tora e peca
                obj_wizard_calcula_rolica_itens.create(
                    cr, uid, {'capa_id': calcmadeira_calcula_id,
                              'codpeca': codigopeca,
                              'nomepeca': namepeca,
                              'numtora': numtora,
                              'voltora': voltora,
                              'numpeca': numpeca,
                              'volpeca': volpeca,
                              'preco_item': preco_item},
                    context=context)
        else:
            itens_ids = obj_wizard_calcula_peca_itens.search(cr, uid, [
                ('peca_id', '=', calcmadeira_calcula_id), ('codpeca', '=', codigopeca)],
                                                                    context=context)
            # se a peca ja estiver na tree, apenas atualiza os dados somando a
            # quantidade e volume de cada tora e peca
            if itens_ids:
                peca_itens_dados = obj_wizard_calcula_peca_itens.read(
                    cr, uid, itens_ids[0],
                    ['numtora', 'voltora', 'numpeca', 'volpeca', 'preco_item'],
                    context=context)
                obj_wizard_calcula_peca_itens.write(cr, uid, itens_ids[0], {
                    'numtora': numtora + peca_itens_dados.get('numtora'),
                    'voltora': voltora + peca_itens_dados.get('voltora'),
                    'numpeca': numpeca + peca_itens_dados.get('numpeca'),
                    'volpeca': volpeca + peca_itens_dados.get('volpeca'),
                    'preco_item': preco_item + peca_itens_dados.get('preco_item'),
                })
            else:
                # insere na tree a quantidade e volume de cada tora e peca
                obj_wizard_calcula_peca_itens.create(
                    cr, uid, {'peca_id': calcmadeira_calcula_id,
                              'codpeca': codigopeca,
                              'nomepeca': namepeca,
                              'numtora': numtora,
                              'voltora': voltora,
                              'numpeca': numpeca,
                              'volpeca': volpeca,
                              'preco_item': preco_item},
                    context=context)

    def calcmadeira_atualiza_tree_resultado(self, cr, uid, ids, codigopeca, namepeca, vr_tree,
                                            di_casc, di_casc2, di, numtora, arv,
                                            es_serra, l, n, v, fatia, l_sobra, es_sobra, es,
                                            comp_lamin=0, ffarv=0, conicidade=0, rendimento=0,
                                            preco=0, comprimento=0, parc=0, context=None):
        if not context.get('calcula_receita'):
            preco = 0
        resultado_peca_id = ids[0] if isinstance(ids, list) else ids
        obj_wizard_resultado_peca = self.pool.get('wizard.calcmadeira.resultado.peca')
        obj_wizard_resultado_peca.create(cr, uid, {'result_id': resultado_peca_id,
                                                   'codpeca': codigopeca,
                                                   'nomepeca': namepeca,
                                                   'vr': vr_tree,
                                                   'di_casc': di_casc,
                                                   'di_casc2': di_casc2,
                                                   'di': di,
                                                   'numtora': numtora,
                                                   'arv': arv,
                                                   'es_serra': es_serra + es,
                                                   'l': l,
                                                   'n': n,
                                                   'v': v,
                                                   'fatia': fatia,
                                                   'l_sobra': l_sobra,
                                                   'es_sobra': es_sobra,
                                                   'comp_laminacao': comp_lamin,
                                                   'ffarv': ffarv,
                                                   'conicidade': conicidade,
                                                   'preco': preco,
                                                   'comprimento': comprimento,
                                                   'parc': parc,
                                                   },
                                                   # 'rendimento': rendimento},
                                         context=context)

    def calcmadeira_limpa_tree(self, cr, uid, ids, context=None):
        # obj_wizard_calcula_itens = self.pool.get('wizard.calcmadeira.calcula.itens')
        # obj_wizard_calcula_rolica_itens = self.pool.get('wizard.calcmadeira.calcula.rolica.itens')
        # obj_wizard_resultado_peca = self.pool.get('wizard.calcmadeira.resultado.peca')
        # calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        for brw_calcmadeira_calcula in self.browse(cr, uid, ids, context):
        # itens_ids = obj_wizard_calcula_itens.search(
        #     cr, uid, [('peca_id', '=', calcmadeira_calcula_id)], context=context)
        # rolica_itens_ids = obj_wizard_calcula_rolica_itens.search(
        #     cr, uid, [('capa_id', '=', calcmadeira_calcula_id)], context=context)
        # resultado_ids = obj_wizard_resultado_peca.search(
        #     cr, uid, [('result_id', '=', calcmadeira_calcula_id)], context=context)
        # if itens_ids:
        #     obj_wizard_calcula_itens.unlink(cr, uid, itens_ids, context=context)
        # if rolica_itens_ids:
        #     obj_wizard_calcula_rolica_itens.unlink(cr, uid, rolica_itens_ids, context)
        # if resultado_ids:
        #     obj_wizard_resultado_peca.unlink(cr, uid, resultado_ids, context=context)

            self.pool.get('wizard.calcmadeira.calcula.itens').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.peca_ids),
                context)
            self.pool.get('wizard.calcmadeira.calcula.rolica.itens').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.peca_rolica_ids),
                context)
            self.pool.get('wizard.calcmadeira.resultado.peca').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.result_ids),
                context)

            self.pool.get('wizard.calcmadeira.frequencia.peca.parcela.talhao').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.frequencia_peca_parcela_talhao_ids),
                context)
            self.pool.get('wizard.calcmadeira.frequencia.peca.parcela.talhao.rolica').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.frequencia_peca_parcela_talhao_rolica_ids),
                context)
            self.pool.get('wizard.calcmadeira.frequencia.peca.talhao').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.frequencia_peca_talhao_ids),
                context)
            self.pool.get('wizard.calcmadeira.frequencia.peca.talhao.rolica').unlink(
                cr, uid,
                map(lambda i: i.id, brw_calcmadeira_calcula.frequencia_peca_talhao_rolica_ids),
                context)
            self.pool.get('wizard.calcmadeira.parcela.talhao').unlink(
                cr, uid, map(lambda i: i.id, brw_calcmadeira_calcula.resultado_parcela_talhao_ids),
                context)
            self.pool.get('wizard.calcmadeira.parcela.talhao.rolica').unlink(
                cr, uid, map(lambda i: i.id,
                             brw_calcmadeira_calcula.resultado_parcela_talhao_rolica_ids), context)
            self.pool.get('wizard.calcmadeira.talhao').unlink(
                cr, uid, map(lambda i: i.id, brw_calcmadeira_calcula.resultado_talhao_ids),
                context)
            self.pool.get('wizard.calcmadeira.talhao.rolica').unlink(
                cr, uid, map(lambda i: i.id, brw_calcmadeira_calcula.resultado_talhao_rolica_ids),
                context)
            self.pool.get('wizard.calcmadeira.calcula.itens.hectare').unlink(
                cr, uid, brw_calcmadeira_calcula.itens_hectare_ids, context
            )
            self.pool.get('wizard.calcmadeira.calcula.rolica.itens.hectare').unlink(
                cr, uid, brw_calcmadeira_calcula.itens_hectare_rolica_ids, context
             )
            self.pool.get('wizard.calcmadeira.arquivo.csv').unlink(
                cr, uid, brw_calcmadeira_calcula.resultado_csv_ids, context
             )
        sleep(0.25)  # Não retirar o sleep, por algum motivo o javascrip quebra ao traocar talhaoes parcelas
        return True

    def calmadeira_menor_largura_espessura(self, cr, uid, ids, context=None):
        obj_dados_pecas_serradas = self.pool.get('calcmadeira.pecas.serradas')
        brw_calcmadeira_calcula = self.browse(cr, uid, ids, context=context)
        brw_calcmadeira_pecas_serradas = brw_calcmadeira_calcula[0].pecas_serradas
        pecas_serradas = []
        for i in brw_calcmadeira_pecas_serradas:
            pecas_serradas.append(i.id)
        l_min_id = obj_dados_pecas_serradas.search(cr, uid, [('id', 'in', pecas_serradas)],
                                                   order='cmapse_menor_largura', limit=1,
                                                   context=context)
        es_min_id = obj_dados_pecas_serradas.search(cr, uid, [('id', 'in', pecas_serradas)],
                                                    order='cmapse_menor_espessura', limit=1,
                                                    context=context)
        obj_dados_largura_min = obj_dados_pecas_serradas.read(cr, uid, l_min_id, context=context)
        obj_dados_espessura_min = obj_dados_pecas_serradas.read(cr, uid, es_min_id, context=context)

        # se largura < largura mínima entre todas as peças, resto da árvore não atende
        # dimensões, mesmo com comprimento suficiente da peça

        l_min_pecas = es_min_pecas = 0
        for z in obj_dados_largura_min:
            l_min_pecas = z.get('cmapse_menor_largura')
        for y in obj_dados_espessura_min:
            es_min_pecas = y.get('cmapse_menor_espessura')
        return (l_min_pecas, es_min_pecas)

    def calcmadeira_calcula_volume_peca_tora(self, cr, uid, ids, vt1, context=None):
        dados_itens = self.pool.get('wizard.calcmadeira.calcula')
        brw_dados_itens = dados_itens.browse(cr, uid, ids, context=context)
        total_voltora = 0
        total_volpeca = 0
        if context.get('funcoes_serrada_tora'):
            return 0, 0
        for x in brw_dados_itens:
            if x.peca_ids:
                for y in x.peca_ids:
                    total_voltora += y.voltora
                    total_volpeca += y.volpeca
            elif x.result_ids:
                for y in x.result_ids:
                    total_voltora += (y.vr or 0)
        total_vt_vr = vt1 - total_voltora
        total_vt_v = vt1 - total_volpeca
        if context.get('calculo_laminada_arvore'):
            total_vt_v = 0
        return total_vt_vr, total_vt_v

    def calcmadeira_trata_talhao_peca(self, cr, uid, ids, context=None):
        """
        Metodo responsavel por inserir os talhoes e parcelas selecionados pelo usuario na busca 
        pelas arvores
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario na transacao
        :type uid: int
        :param ids: ids dos registros da tabela desse objeto
        :type ids: list
        :param context: dados contextuais
        :type context: dict
        :return: argumentos de pesquisa envolvendo talhoes e parcelas
        :rtype: list
        """
        context = context and context.copy() or {}
        res = []
        brw_calcmadeira_calcula = self.browse(
            cr, uid, isinstance(ids, (list, tuple)) and ids[0] or ids, context)
        if brw_calcmadeira_calcula.talhao_ids:
            res.append(('cmaarv_talhao', 'in', map(lambda t: t.talhao,
                                                   brw_calcmadeira_calcula.talhao_ids)))
        if brw_calcmadeira_calcula.parcela_ids:
            res.append(('cmaarv_linha', 'in', map(lambda t: t.parcela,
                                                  brw_calcmadeira_calcula.parcela_ids)))
        return res

    def button_calcmadeira_calcula(self, cr, uid, ids, context=None):
        calcmadeira_calcula_id = ids[0] if isinstance(ids, list) else ids
        calcmadeira_calcula_dados = self.read(
            cr, uid, calcmadeira_calcula_id,
            ['tipo_funcao', 'calcula_receita', 'pecas_rolicas', 'pecas_serradas'],
            context)
        tipo_funcao = calcmadeira_calcula_dados.get('tipo_funcao')
        context.update({'calcula_receita': calcmadeira_calcula_dados.get(
            'calcula_receita') or False})
        if not context.get('calculo_laminada_arvore'):
            if not any([calcmadeira_calcula_dados.get('pecas_rolicas'),
                    calcmadeira_calcula_dados.get('pecas_serradas')]):
                raise osv.except_osv(_('Atencao'), _('Nao ha peca informada.'))
        obj_calcmadeira_parametro = self.pool.get('calcmadeira.parametro')
        calcmadeira_parametro_id = obj_calcmadeira_parametro.search(cr, uid, [], context=context)
        if not calcmadeira_parametro_id:
            raise osv.except_osv(_('Atencao'), _('Parametros nao cadastrados.'))
        parametro_dados = obj_calcmadeira_parametro.read(cr, uid, calcmadeira_parametro_id[0], [],
                                                         context, '_classic_write')
        cmapar_es_medula = 0
        if parametro_dados.get('cmapar_espessura_peca_central') > parametro_dados.get(
                'cmapar_es_serra'):
            cmapar_es_medula = (parametro_dados.get('cmapar_espessura_peca_central') or 0) + (
                parametro_dados.get('cmapar_es_serra') or 0)
        parametro_dados.update({'es_medula': cmapar_es_medula})

        self.calcmadeira_limpa_tree(cr, uid, ids, context)
        resultado_dados = {'detalhado': {}, 'sintetico': {}}
        if context.get('funcoes_serrada_arvore'):
            obj = self.pool.get('wizard.calcula.pecas.mad.serr')
            calcula_pecas_mad_serr_id = obj.create(
                cr, uid, {'calcmadeira_calcula_id': calcmadeira_calcula_id}, context)
            resultado_dados = obj.executa(cr, uid, calcula_pecas_mad_serr_id, context)
            print resultado_dados
        if context.get('serrada_e_rolica'):
            obj = self.pool.get('wizard.calcula.pecas.mad.pta.serrad.rolica')
            calcula_pecas_mad_pta_serrad_rolica_id = obj.create(
                cr, uid, {'calcmadeira_calcula_id': calcmadeira_calcula_id}, context)
            resultado_dados = obj.executa(cr, uid, calcula_pecas_mad_pta_serrad_rolica_id, context)
            print resultado_dados
        if context.get('calculo_rolica'):
            obj = self.pool.get('wizard.calcula.pecas.rolicas')
            calcula_pecas_rolicas_id = obj.create(
                cr, uid, {'calcmadeira_calcula_id': calcmadeira_calcula_id}, context)
            resultado_dados = obj.executa(cr, uid, calcula_pecas_rolicas_id, context)
            print resultado_dados
        vals_calcmadeira_calcula = {}

        detalhado_ids = []
        contador_detalhado = 0
        for det in resultado_dados.get('detalhado', {}).get('codpeca', []):
            posicao = contador_detalhado
            detalhado_ids.append((0, 0, {
                'codpeca': det,
                'numtora': resultado_dados.get('detalhado').get('tora')[posicao],
                'arv': resultado_dados.get('detalhado').get('arv')[posicao],
                'di': resultado_dados.get('detalhado').get('di')[posicao],
                'di_casc2': resultado_dados.get('detalhado').get('di_casc2')[posicao],
                'l': resultado_dados.get('detalhado').get('l') and resultado_dados.get(
                    'detalhado').get('l')[posicao] or 0,
                'n': any(resultado_dados.get('detalhado').get('n', [])) and resultado_dados.get(
                    'detalhado').get('n')[posicao] or 0,
                'nomepeca': resultado_dados.get('detalhado').get('peca')[posicao],
                'vr': resultado_dados.get('detalhado').get('vr')[posicao],
                'comprimento': resultado_dados.get('detalhado').get('hi')[posicao],
                'preco': resultado_dados.get('detalhado').get('preco')[posicao],
                'di_casc': resultado_dados.get('detalhado').get('di_casc')[posicao],
                'v': resultado_dados.get('detalhado').get('v')[posicao],
                'parc': resultado_dados.get('detalhado').get('parc')[posicao],
                'crv': resultado_dados.get('detalhado').get('CRV') and resultado_dados.get(
                    'detalhado').get('CRV')[posicao] or 0,
                'conicidade': resultado_dados.get('detalhado').get('conicidade')[posicao],
                'talhao': resultado_dados.get('detalhado').get('talh')[posicao],
                'es_serra': resultado_dados.get('detalhado').get('es') and resultado_dados.get(
                    'detalhado').get('es')[posicao] or 0,
                'rendimento': 0 if resultado_dados.get('detalhado').get('peca')[posicao] ==
                'NaoRolica' else (resultado_dados.get('detalhado').get('rend')
                                  and resultado_dados.get('detalhado').get('rend')[posicao] or 0)
            }))
            contador_detalhado += 1
        detalhado_ids and vals_calcmadeira_calcula.update({'result_ids': detalhado_ids})

        descricao_campo = resultado_dados.get('sintetico').get('frequencia_peca_talhao', {}).get(
            'codpeca_freq') and '_freq' or ''
        sintetico_frequencia_peca_talhao_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('frequencia_peca_talhao', {}).get(
                'codpeca'+descricao_campo, []):
            posicao = contador_sintetico
            sintetico_frequencia_peca_talhao_ids.append((0, 0, {
                'codpeca_freq': sin,
                'n_pecas_freq': resultado_dados.get('sintetico').get('frequencia_peca_talhao').get(
                    'n_pecas'+descricao_campo)[posicao],
                'peca': resultado_dados.get('sintetico').get('frequencia_peca_talhao').get(
                    'peca'+descricao_campo)[posicao],
                'preco_freq': resultado_dados.get('sintetico').get('frequencia_peca_talhao').get(
                    'preco'+descricao_campo)[posicao],
                'talh_freq': resultado_dados.get('sintetico').get('frequencia_peca_talhao').get(
                    'talh'+descricao_campo)[posicao],
                'v_freq': resultado_dados.get('sintetico').get('frequencia_peca_talhao').get(
                    'v'+descricao_campo)[posicao],
            }))
            contador_sintetico += 1
        sintetico_frequencia_peca_talhao_ids and vals_calcmadeira_calcula.update(
            {'frequencia_peca_talhao_ids': sintetico_frequencia_peca_talhao_ids})

        descricao_campo = resultado_dados.get('sintetico').get(
            'frequencia_peca_talhao_rolica', {}).get('codpeca_freq') and '_freq' or ''
        sintetico_frequencia_peca_talhao_rolica_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('frequencia_peca_talhao_rolica', {}).get(
                'codpeca'+descricao_campo, []):
            posicao = contador_sintetico
            sintetico_frequencia_peca_talhao_rolica_ids.append((0, 0, {
                'codpeca_freq': sin,
                'n_pecas_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_talhao_rolica').get('n_pecas'+descricao_campo)[posicao],
                'peca': resultado_dados.get('sintetico').get('frequencia_peca_talhao_rolica').get(
                    'peca'+descricao_campo)[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_talhao_rolica').get('preco'+descricao_campo)[posicao],
                'talh_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_talhao_rolica').get('talh'+descricao_campo)[posicao],
                'v_freq': resultado_dados.get('sintetico').get('frequencia_peca_talhao_rolica').get(
                    'v'+descricao_campo)[posicao],
            }))
            contador_sintetico += 1
        sintetico_frequencia_peca_talhao_rolica_ids and vals_calcmadeira_calcula.update(
            {'frequencia_peca_talhao_rolica_ids': sintetico_frequencia_peca_talhao_rolica_ids})

        descricao_campo = resultado_dados.get('sintetico').get(
            'frequencia_peca_parcela_talhao', {}).get('codpeca_freq') and '_freq' or ''
        sintetico_frequencia_peca_parcela_talhao_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('frequencia_peca_parcela_talhao', {}).get(
                'codpeca'+descricao_campo, []):
            posicao = contador_sintetico
            sintetico_frequencia_peca_parcela_talhao_ids.append((0, 0, {
                'codpeca_freq': sin,
                'n_pecas_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao').get('n_pecas'+descricao_campo)[posicao],
                'peca': resultado_dados.get('sintetico').get('frequencia_peca_parcela_talhao').get(
                    'peca')[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao').get('preco'+descricao_campo)[posicao],
                'talh_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao').get('talh'+descricao_campo)[posicao],
                'v_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao').get('v'+descricao_campo)[posicao],
                'parc_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao').get('parc'+descricao_campo)[posicao],
            }))
            contador_sintetico += 1
        sintetico_frequencia_peca_parcela_talhao_ids and vals_calcmadeira_calcula.update(
            {'frequencia_peca_parcela_talhao_ids': sintetico_frequencia_peca_parcela_talhao_ids})

        descricao_campo = resultado_dados.get('sintetico').get(
            'frequencia_peca_parcela_talhao_rolica', {}).get('codpeca_freq') and '_freq' or ''
        sintetico_frequencia_peca_parcela_talhao_rolica_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get(
                'frequencia_peca_parcela_talhao_rolica', {}).get('codpeca'+descricao_campo, []):
            posicao = contador_sintetico
            sintetico_frequencia_peca_parcela_talhao_rolica_ids.append((0, 0, {
                'codpeca_freq': sin,
                'n_pecas_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao_rolica').get(
                    'n_pecas'+descricao_campo)[posicao],
                'peca': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao_rolica').get('peca'+descricao_campo)[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao_rolica').get('preco'+descricao_campo)[posicao],
                'talh_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao_rolica').get('talh'+descricao_campo)[posicao],
                'v_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao_rolica').get('v'+descricao_campo)[posicao],
                'parc_freq': resultado_dados.get('sintetico').get(
                    'frequencia_peca_parcela_talhao_rolica').get('parc'+descricao_campo)[posicao],
            }))
            contador_sintetico += 1
        sintetico_frequencia_peca_parcela_talhao_rolica_ids and vals_calcmadeira_calcula.update(
            {'frequencia_peca_parcela_talhao_rolica_ids':
                sintetico_frequencia_peca_parcela_talhao_rolica_ids})

        descricao_campo = resultado_dados.get('sintetico').get('resultado_parcela_talhao', {}).get(
            'preco_freq') and '_freq' or ''
        sintetico_resultado_parcela_talhao_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('resultado_parcela_talhao', {}).get(
                'crv', []):
            posicao = contador_sintetico
            sintetico_resultado_parcela_talhao_ids.append((0, 0, {
                'crv': sin,
                'arv': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('arv')[posicao],
                'falha': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('falha')[posicao],
                'n_pecas': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('n_pecas')[posicao],
                'parc': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('parc')[posicao],
                'talh': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('talh')[posicao],
                'preco_arv': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('preco_arv')[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('preco'+descricao_campo)[posicao],
                'preco_inv': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('preco_inv')[posicao],
                'preco_pecas': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('preco_pecas')[posicao],
                'v': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('v')[posicao],
                'vr': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('vr')[posicao],
                'vr_v': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('vr_v')[posicao],
                'vt': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('vt')[posicao],
                'vt_vr': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao').get('vt_vr')[posicao],
            }))
            contador_sintetico += 1
        sintetico_resultado_parcela_talhao_ids and vals_calcmadeira_calcula.update(
            {'resultado_parcela_talhao_ids': sintetico_resultado_parcela_talhao_ids})

        descricao_campo = resultado_dados.get('sintetico').get(
            'resultado_parcela_talhao_rolica', {}).get('preco_freq') and '_freq' or ''
        sintetico_resultado_parcela_talhao_rolica_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('resultado_parcela_talhao_rolica', {}).get(
                'crv', []):
            posicao = contador_sintetico
            sintetico_resultado_parcela_talhao_rolica_ids.append((0, 0, {
                'crv': sin,
                'arv': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('arv')[posicao],
                'falha': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('falha')[posicao],
                'n_pecas': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('n_pecas')[posicao],
                'parc': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('parc')[posicao],
                'talh': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('talh')[posicao],
                'preco_arv': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('preco_arv')[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('preco'+descricao_campo)[posicao],
                'preco_inv': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('preco_inv')[posicao],
                'preco_pecas': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('preco_pecas')[posicao],
                'v': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('v')[posicao],
                'vr': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('vr')[posicao],
                'vr_v': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('vr_v')[posicao],
                'vt': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('vt')[posicao],
                'vt_vr': resultado_dados.get('sintetico').get(
                    'resultado_parcela_talhao_rolica').get('vt_vr')[posicao],
            }))
            contador_sintetico += 1
        sintetico_resultado_parcela_talhao_rolica_ids and vals_calcmadeira_calcula.update(
            {'resultado_parcela_talhao_rolica_ids': sintetico_resultado_parcela_talhao_rolica_ids})

        descricao_campo = resultado_dados.get('sintetico').get('resultado_talhao', {}).get(
            'preco_freq') and '_freq' or ''
        sintetico_resultado_talhao_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('resultado_talhao', {}).get(
                'crv', []):
            posicao = contador_sintetico
            sintetico_resultado_talhao_ids.append((0, 0, {
                'crv': sin,
                'arv': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('arv')[posicao],
                'falha': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('falha')[posicao],
                'n_pecas': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('n_pecas')[posicao],
                'talh': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('talh')[posicao],
                'preco_arv': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('preco_arv')[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('preco'+descricao_campo)[posicao],
                'preco_inv': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('preco_inv')[posicao],
                'preco_pecas': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('preco_pecas')[posicao],
                'v': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('v')[posicao],
                'vr': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('vr')[posicao],
                'vr_v': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('vr_v')[posicao],
                'vt': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('vt')[posicao],
                'vt_vr': resultado_dados.get('sintetico').get(
                    'resultado_talhao').get('vt_vr')[posicao],
            }))
            contador_sintetico += 1
        sintetico_resultado_talhao_ids and vals_calcmadeira_calcula.update(
            {'resultado_talhao_ids': sintetico_resultado_talhao_ids})

        descricao_campo = resultado_dados.get('sintetico').get('resultado_talhao_rolica', {}).get(
            'preco_freq') and '_freq' or ''
        sintetico_resultado_talhao_rolica_ids = []
        contador_sintetico = 0
        for sin in resultado_dados.get('sintetico').get('resultado_talhao_rolica', {}).get(
                'crv', []):
            posicao = contador_sintetico
            sintetico_resultado_talhao_rolica_ids.append((0, 0, {
                'crv': sin,
                'arv': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('arv')[posicao],
                'falha': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('falha')[posicao],
                'n_pecas': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('n_pecas')[posicao],
                'talh': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('talh')[posicao],
                'preco_arv': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('preco_arv')[posicao],
                'preco_freq': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('preco'+descricao_campo)[posicao],
                'preco_inv': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('preco_inv')[posicao],
                'preco_pecas': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('preco_pecas')[posicao],
                'v': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('v')[posicao],
                'vr': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('vr')[posicao],
                'vr_v': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('vr_v')[posicao],
                'vt': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('vt')[posicao],
                'vt_vr': resultado_dados.get('sintetico').get(
                    'resultado_talhao_rolica').get('vt_vr')[posicao],
            }))
            contador_sintetico += 1
        sintetico_resultado_talhao_rolica_ids and vals_calcmadeira_calcula.update(
            {'resultado_talhao_rolica_ids': sintetico_resultado_talhao_rolica_ids})

        vals_calcmadeira_calcula and self.write(cr, uid, ids, vals_calcmadeira_calcula, context)

        # if tipo_funcao == '1':
        #    self.calcula_peca_rolica(cr, uid, ids, parametro_dados, context=context)
        # elif tipo_funcao in ('2', '15'):
        #     self.calcula_peca_longitudinal_base(cr, uid, ids, parametro_dados, context=context)
        # elif tipo_funcao == '3':
        #    self.calcula_peca_longitudinal(cr, uid, ids, parametro_dados, context=context)
        # elif tipo_funcao == '4':
        #    self.calcula_peca_quadrado_circunscrito(cr, uid, ids, parametro_dados, context=context)
        # elif tipo_funcao == '5':
        #    self.calcula_peca_desdobro_radial(cr, uid, ids, parametro_dados, context=context)
        if tipo_funcao in ('6', '16'):
            self.calcula_tora_longitudinal_base(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '7':
            self.calcula_tora_longitudinal(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '8':
            self.calcula_tora_quadrado_circunscrito(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '9':
            self.calcula_tora_desdobro_radial(cr, uid, ids, parametro_dados, context=context)
        elif tipo_funcao == '10':
            self.calcula_peca_laminacao(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '11':
            self.calcula_tora_laminacao(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '12':
            self.calcula_peca_fator_forma(cr, uid, ids, parametro_dados, context)
        # elif tipo_funcao == '13':
        #    self.calcula_peca_gira_180_90(cr, uid, ids, parametro_dados, context)
        elif tipo_funcao == '14':
            self.calcula_tora_gira_180_90(cr, uid, ids, parametro_dados, context)

        if tipo_funcao not in ('1', '10', '11'):
            self.calcmadeira_gera_rendimento_resultado(cr, uid, ids, tipo_funcao, context)
        return True

    def calcmadeira_gera_rendimento_resultado(self, cr, uid, ids, tipo_funcao, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT 
                min(id) AS id, 
                round(sum(v) / max(vr), 4) AS rendimento, 
                conicidade
            FROM 
                wizard_calcmadeira_resultado_peca
            WHERE 
                result_id = {result_id}
            GROUP 
                BY conicidade, numtora
            --ORDER BY 
            --    id
        """.format(result_id=ids[0] if isinstance(ids, list) else ids)
        cr.execute(sql)
        dados = cr.dictfetchall()
        if not dados:
            return False
        resultado_ids = []
        for dado in dados:
            rendimento = dado.get('rendimento') or 0
            if tipo_funcao in ('5', '9'):
                rendimento *= 4
            elif tipo_funcao in ('15', '16'):
                rendimento *= 2
            resultado_ids.append((1, dado.get('id'), {'rendimento': rendimento}))
        self.write(cr, uid, ids, {'result_ids': resultado_ids}, context)
        return True

    def calcmadeira_get_equacao_volume(self, cr, uid, brw_coeficiente, dap, ht, hc, context=None):
        context = context and context.copy() or {}
        vt = 0
        if brw_coeficiente.name == 'Volume padrao' and dap > 0 and ht > 0 and hc > 0:
            vt = equacao_volume(dap, ht, brw_coeficiente.cmacoe_b0,
                                brw_coeficiente.cmacoe_b1, brw_coeficiente.cmacoe_b2)
        elif dap > 0 and ht > 0 and hc > 0:
            vt = Eqvt(brw_coeficiente.cmacoe_b0, brw_coeficiente.cmacoe_b1,
                      brw_coeficiente.cmacoe_b2, dap, ht)
            vt = vt.vt_result()
        return vt

    def calcmadeira_get_equacao_forma(self, cr, uid,
                                      coeficiente_forma=None,
                                      hi_atual=None,
                                      hi=None,
                                      ht=None,
                                      dap=None,
                                      di_casc=None,
                                      di_casc2=None,
                                      hi_min=None,
                                      context=None):
        context = context and context.copy() or {}
        if coeficiente_forma.name == 'Forma padrao':
            di_casc, di_casc2 = equacao_forma(hi_atual, hi, ht, dap, di_casc, di_casc2,
                                              coeficiente_forma.cmacoe_b0 or 0,
                                              coeficiente_forma.cmacoe_b1 or 0,
                                              coeficiente_forma.cmacoe_b2 or 0)
            di_casc_deriv = di_casc2_deriv = 0
        else:
            # di_casc2 = Eqdi_casc(
            #     coeficiente_forma.cmacoe_b0 or 0, coeficiente_forma.cmacoe_b1 or 0,
            #     # 08/09/2023 retiramos o hi_min novamente porque na funcao Eqdi_casc não recebe o hi_min
            #     coeficiente_forma.cmacoe_b2, hi_atual, hi, ht, dap, di_casc, di_casc2)
            #     # 08/09/2023
            #     # brw_coeficiente.cmacoe_b2, hi_atual, hi_min, ht, dap, di_casc, di_casc2)
            #     #
            resultado = Eqdi_casc(
                coeficiente_forma.cmacoe_b0 or 0, coeficiente_forma.cmacoe_b1 or 0,
                coeficiente_forma.cmacoe_b2 or 0, hi_atual, hi, ht, dap)
            di_casc2 = resultado.di_casc2_result()
            di_casc = resultado.di_casc_result()
            di_casc2_deriv = resultado.di_casc2_deriv_result()
            di_casc_deriv = resultado.di_casc_deriv_result()
        return di_casc, di_casc2, di_casc_deriv, di_casc2_deriv

    def get_altura_comercial_calculada_dados(self, cr, uid, brw_calcmadeira_calcula, arvores_dados,
                                             parametro_dados, context=None):
        context = context and context.copy() or {}
        altura_comercial_calculada = brw_calcmadeira_calcula.altura_comercial_calculada
        if not altura_comercial_calculada:
            return {}
        sql = """SELECT * FROM calcmadeira_taper WHERE company_id = %s ORDER BY id""" % \
              brw_calcmadeira_calcula.company_id.id
        cr.execute(sql)
        taper_dados = cr.dictfetchall()
        diametro_minimo = brw_calcmadeira_calcula.altura_comercial_diametro_minimo
        altura_comercial_calculada_dict = calcula_altura_comercial(
            arvores_dados, diametro_minimo, parametro_dados, taper_dados)
        # Atualiza a altura comercial no cadastro de arvores
        obj_calcmadeira_arvores = self.pool.get('calcmadeira.arvores')
        map(lambda ca_id: obj_calcmadeira_arvores.write(
            cr, uid, ca_id, {'cmaarv_altura_comecial': altura_comercial_calculada_dict.get(ca_id)},
            context), altura_comercial_calculada_dict.keys())
        return altura_comercial_calculada_dict

    def calcula_madeira_serrada_por_hectare(self, cr, uid, brw_calcmadeira_calcula, qtd_arvores,
                                            context=None):
        context = context and context.copy() or {}
        cr.execute("""DELETE FROM wizard_calcmadeira_calcula_itens_hectare 
                      WHERE wcahec_capa_id = %s""" % brw_calcmadeira_calcula.id)
        cr.execute("""DELETE FROM wizard_calcmadeira_calcula_rolica_itens_hectare 
                      WHERE capa_id = %s""" % brw_calcmadeira_calcula.id)
        if not brw_calcmadeira_calcula.calcula_receita:
            return False
        # arvore_por_hectare = brw_calcmadeira_calcula.arvore_por_hectare
        # if arvore_por_hectare <= 0:
        #     return False
        list_itens_hectare = []
        list_itens_rolica_hectare = []
        for item in brw_calcmadeira_calcula.peca_ids:
            num_tora_ha = 0 # int(item.numtora * arvore_por_hectare / qtd_arvores)
            vol_tora_ha = 0 # (item.voltora * arvore_por_hectare / qtd_arvores)
            num_peca_ha = 0 # int(item.numpeca * arvore_por_hectare / qtd_arvores)
            vol_peca_ha = 0 # item.volpeca * arvore_por_hectare / qtd_arvores
            preco_ha = 0 # int(item.preco_item * arvore_por_hectare / qtd_arvores)
            vals_itens_hectare = {'wcahec_codigo_peca': item.codpeca,
                                  'wcahec_nome_peca': item.nomepeca,
                                  'wcahec_numero_peca': num_peca_ha,
                                  'wcahec_numero_tora': num_tora_ha,
                                  'wcahec_preco': preco_ha,
                                  'wcahec_volume_peca': vol_peca_ha,
                                  'wcahec_volume_tora': vol_tora_ha}
            list_itens_hectare.append((0, 0, vals_itens_hectare))
        if context.get('serrada_e_rolica'):
            for item in brw_calcmadeira_calcula.peca_rolica_ids:
                num_tora_ha = 0  # int(item.numtora * arvore_por_hectare / qtd_arvores)
                vol_tora_ha = 0  # (item.voltora * arvore_por_hectare / qtd_arvores)
                num_peca_ha = 0  # int(item.numpeca * arvore_por_hectare / qtd_arvores)
                vol_peca_ha = 0  # item.volpeca * arvore_por_hectare / qtd_arvores
                preco_ha = 0  # int(item.preco_item * arvore_por_hectare / qtd_arvores)
                vals_itens_hectare = {'codigo_peca': item.codpeca,
                                      'nome_peca': item.nomepeca,
                                      'numero_peca': num_peca_ha,
                                      'numero_tora': num_tora_ha,
                                      'preco': preco_ha,
                                      'volume_peca': vol_peca_ha,
                                      'volume_tora': vol_tora_ha}
                list_itens_rolica_hectare.append((0, 0, vals_itens_hectare))
        vals = {}
        list_itens_hectare and vals.update({'itens_hectare_ids': list_itens_hectare})
        list_itens_rolica_hectare and vals.update(
            {'itens_hectare_rolica_ids': list_itens_rolica_hectare})
        vals and self.write(cr, uid, brw_calcmadeira_calcula.id, vals, context)
        return True

    def calcula_madeira_rolica_por_hectare(self, cr, uid, brw_calcmadeira_calcula, qtd_arvores,
                                           context=None):
        context = context and context.copy() or {}
        # arvore_por_hectare = brw_calcmadeira_calcula.arvore_por_hectare
        # if arvore_por_hectare <= 0:
        #     return False
        cr.execute("""DELETE FROM wizard_calcmadeira_calcula_itens_hectare 
                      WHERE wcahec_capa_id = %s""" % brw_calcmadeira_calcula.id)
        list_itens_hectare = []
        for item in brw_calcmadeira_calcula.peca_ids:
            num_tora_ha = 0  # int(item.numtora * arvore_por_hectare / qtd_arvores)
            vol_tora_ha = 0  # (item.voltora * arvore_por_hectare / qtd_arvores)
            # num_peca_ha = int(item.numpeca * arvore_por_hectare / qtd_arvores)
            # vol_peca_ha = item.volpeca * arvore_por_hectare / qtd_arvores
            preco_ha = 0  # int(item.preco_item * arvore_por_hectare / qtd_arvores)
            vals_itens_hectare = {'wcahec_codigo_peca': item.codpeca,
                                  'wcahec_nome_peca': item.nomepeca,
                                  # 'wcahec_numero_peca': num_peca_ha,
                                  'wcahec_numero_tora': num_tora_ha,
                                  'wcahec_preco': preco_ha,
                                  # 'wcahec_volume_peca': vol_peca_ha,
                                  'wcahec_volume_tora': vol_tora_ha}
            list_itens_hectare.append((0, 0, vals_itens_hectare))
        list_itens_hectare and self.write(
            cr, uid, brw_calcmadeira_calcula.id, {'itens_hectare_ids': list_itens_hectare}, context)
        return True

    def calcmadeira_valida_parametro_laminacao(self, cr, uid, parametro_dados, context=None):
        context = context and context.copy() or {}
        res = True
        msg = ''
        obj_calcmadeira_toras = self.pool.get('calcmadeira.toras')
        toras_ids = obj_calcmadeira_toras.search(cr, uid, [], order='cmator_diametro_min', limit=1,
                                                 context=context)
        menor_diametro_tora = toras_ids and obj_calcmadeira_toras.read(
            cr, uid, toras_ids[0], ['cmator_diametro_min'], context).get('cmator_diametro_min') or 0
        if parametro_dados.get('cmapar_laminacao_diametro_minimo') < 0:
            msg += _('"Parte do diametro nao aproveitavel (rolo resto)" nao pode ser '
                     'menor que zero.\n')
        if parametro_dados.get('cmapar_laminacao_espessura') <= 0 or parametro_dados.get(
                'cmapar_laminacao_espessura') > 100:
            msg += _('"Espessura da folha" deve ser maior que zero e menor ou igual a 100.\n')
        if parametro_dados.get('cmapar_laminacao_comp_folha') <= 0 or parametro_dados.get(
                'cmapar_laminacao_comp_folha') > 10:
            msg += _('"Comprimento da folha" deve ser maior que zero e menor ou igual a 10.\n')
        if msg:
            raise osv.except_osv(_('Atencao'), 'Parametros:\n' + msg)
        return res

    def calcmadeira_exporta_dados_txt(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        brw_resultado_peca = self.browse(cr, uid, isinstance(ids, list) and ids[0] or ids,
                                         context).result_ids
        if not brw_resultado_peca:
            raise osv.except_osv(_('Atencao'), _('Nao ha dados para serem exportados!'))
        str_arquivo = 'Arvore\tTora\tdi_casc2\tdi_casc\thi\r\n'
        for det in brw_resultado_peca:
            str_arquivo += '%s\t%s\t%s\t%s\t%s\r\n' % (det.arv, det.numtora, det.di_casc2,
                                                       det.di_casc, det.comprimento)
        str_arquivo and self.write(cr, uid, ids, {'arquivo_tora': base64.encodestring(str_arquivo)},
                                   context)
        return True

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        res = super(wizard_calcmadeira_calcula, self).fields_view_get(
            cr, uid, view_id, view_type, context, toolbar, submenu)
        # if context.get('tipo_funcao') == '1':
        #     res['fields']['arvore_processada_hectare']['string'] = _('Arvore p. Rolicas/ha')
        #     res['fields']['arvore_processada']['string'] = _('Arvore p. Rolicas')
        # elif context.get('tipo_funcao') == '2':
        #     res['fields']['arvore_processada_hectare']['string'] = _('Arvore p. Serrar/ha')
        #     res['fields']['arvore_processada']['string'] = _('Arvore p. Serrar')
        return res

    def calcmadeira_button_gera_arquivo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_calcmadeira_arquivo_csv = self.pool.get('wizard.calcmadeira.arquivo.csv')
        arquivo_csv_ids = obj_calcmadeira_arquivo_csv.search(cr, uid, [('capa_id', 'in', ids)],
                                                             context=context)
        arquivo_csv_ids and obj_calcmadeira_arquivo_csv.unlink(cr, uid, arquivo_csv_ids, context)
        arquivo_csv_id = obj_calcmadeira_arquivo_csv.create(cr, uid, {'capa_id': ids[0]}, context)
        obj_calcmadeira_arquivo_csv.gera_arquivo_csv(cr, uid, [arquivo_csv_id], context)
        reference = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'calcmadeira', 'view_wizard_calcmadeira_arquivo_csv_form')
        view_id = reference and reference[1] or False
        return {
            'name': 'Arquivos CSV',
            'type': 'ir.actions.act_window',
            'res_model': 'wizard.calcmadeira.arquivo.csv',
            'view_mode': 'form',
            'res_id': arquivo_csv_id,
            'target': 'new',
            'view_id': view_id,
        }


class wizard_calcula_peca_itens(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula.itens'
    _columns = {
        'peca_id': fields.many2one('wizard.calcmadeira.calcula', _('Peca')),
        'codpeca': fields.integer(_('Codigo peca')),
        'nomepeca': fields.char(_('Nome peca'), size=40),
        'numtora': fields.integer(_('Numero tora')),
        'voltora': fields.float(_('Volume tora'), digits=(3, 4)),
        'numpeca': fields.integer(_('Numero peca')),
        'volpeca': fields.float(_('Volume peca'), digits=(3, 4)),
        'preco_item': fields.float(_('Preco'), digits=(5, 2)),
    }
    _order = 'codpeca'


class wizard_calcmadeira_calcula_rolica_itens(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula.rolica.itens'
    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', _('Capa')),
        'codpeca': fields.integer(_('Codigo peca')),
        'nomepeca': fields.char(_('Nome peca'), size=40),
        'numtora': fields.integer(_('Numero tora')),
        'voltora': fields.float(_('Volume tora'), digits=(3, 4)),
        'numpeca': fields.integer(_('Numero peca')),
        'volpeca': fields.float(_('Volume peca'), digits=(3, 4)),
        'preco_item': fields.float(_('Preco'), digits=(5, 2)),
    }
    _order = 'codpeca'


class wizard_resultado_peca(osv.TransientModel):
    _name = 'wizard.calcmadeira.resultado.peca'
    _columns = {
        'result_id': fields.many2one('wizard.calcmadeira.calcula', _('Resultado')),
        'codpeca': fields.integer(_('Codigo peca')),
        'nomepeca': fields.char(_('Nome peca'), size=40),
        'es_serra': fields.float(_('es (mm)'), digits=(3, 2)),
        'l': fields.float(_('l (mm)'), digits=(3, 2)),
        'n': fields.integer(_('n')),
        'v': fields.float(_('v (m³)'), digits=(3, 4)),
        'numtora': fields.integer(_('numtora')),
        'vr': fields.float(_('Vr (m³)'), digits=(3, 4)),
        'di_casc2': fields.float(_('di_casc2'), digits=(3, 2)),
        'di_casc': fields.float(_('di_casc'), digits=(3, 2)),
        'di': fields.float(_('di (cm)'), digits=(3, 2)),
        'arv': fields.integer(_('arv')),
        'fatia': fields.integer(_('fatia')),
        'l_sobra': fields.float(_('l_sobra'), digits=(3, 2)),
        'es_sobra': fields.float(_('es_sobra'), digits=(3, 2)),
        'comp_laminacao': fields.float(_('comp_lamin'), digits=(3, 2)),
        'ffarv': fields.float(_('FFarv'), digits=(3, 4)),
        'conicidade': fields.float(_('Conicidade (cm/m)'), digits=(3, 4)),
        'rendimento': fields.float(_('Rendimento (v/vr)'), digits=(3, 4)),
        'preco': fields.float(_('Preco'), digits=(5, 2)),
        'comprimento': fields.float(_('compr. (m)'), digits=(2, 1)),
        'parc': fields.integer('parc'),
        'crv': fields.float(_('CRV'), digits=(3, 4)),
        'talhao': fields.integer('Talhao'),
    }


class wizard_calcmadeira_calcula_itens_hectare(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula.itens.hectare'
    _columns = {
        'wcahec_capa_id': fields.many2one('wizard.calcmadeira.calcula', _('Capa')),
        'wcahec_codigo_peca': fields.integer(_('Codigo peca')),
        'wcahec_nome_peca': fields.char(_('Nome peca'), size=40),
        'wcahec_numero_peca': fields.integer(_('Numero peca')),
        'wcahec_numero_tora': fields.integer(_('Numero tora')),
        'wcahec_preco': fields.float(_('Preco'), digits=(5, 2)),
        'wcahec_volume_peca': fields.float(_('Volume peca'), digits=(3, 4)),
        'wcahec_volume_tora': fields.float(_('Volume tora'), digits=(3, 4)),
    }


class wizard_calcmadeira_calcula_rolica_itens_hectare(osv.TransientModel):
    _name = 'wizard.calcmadeira.calcula.rolica.itens.hectare'
    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', _('Capa')),
        'codigo_peca': fields.integer(_('Codigo peca')),
        'nome_peca': fields.char(_('Nome peca'), size=40),
        'numero_peca': fields.integer(_('Numero peca')),
        'numero_tora': fields.integer(_('Numero tora')),
        'preco': fields.float(_('Preco'), digits=(5, 2)),
        'volume_peca': fields.float(_('Volume peca'), digits=(3, 4)),
        'volume_tora': fields.float(_('Volume tora'), digits=(3, 4)),
    }


class wizard_calcmadeira_arquivo_csv(osv.TransientModel):

    _name = 'wizard.calcmadeira.arquivo.csv'
    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'resumido_item_dados': fields.binary('Resumido itens'),
        'resumido_item_nome': fields.char('Descricao itens', size=50),
        'resumido_item_rolica_dados': fields.binary('Resumido itens rolica'),
        'resumido_item_rolica_nome': fields.char('Descricao itens rolica', size=50),
        'resumido_item_hectare_dados': fields.binary('Resumido itens hectare'),
        'resumido_item_hectare_nome': fields.char('Descricao itens hectare', size=50),
        'resumido_item_rolica_hectare_dados': fields.binary('Resumido itens rolica hectare'),
        'resumido_item_rolica_hectare_nome': fields.char('Descricao itens rolica nome', size=50),
        'analitico_item_dados': fields.binary('Analitico itens'),
        'analitico_item_nome': fields.char('Descricao analitico', size=50),
    }

    _defaults = {
        'resumido_item_nome': 'lista_itens_sintetico.csv',
        'resumido_item_rolica_nome': 'lista_itens_rolica_sintetico.csv',
        'resumido_item_hectare_nome': 'lista_itens_hectare_sintetico.csv',
        'resumido_item_rolica_hectare_nome': 'lista_itens_rolica_hectare_sintetico.csv',
        'analitico_item_nome': 'lista_itens_analitico.csv',
    }

    def gera_arquivo_csv(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        remove_acentuacao = self.pool.get('calcmadeira.utils').calutl_remover_acentuacao
        brw_calcmadeira_calcula = self.browse(cr, uid, ids[0], context).capa_id
        vals = {}
        dir_arq_temporario = '/tmp/arquivo.csv'
        if brw_calcmadeira_calcula.peca_ids:
            arquivo = open(dir_arq_temporario, 'wb')
            arquivo.write('codpeca,nomepeca,numtora,voltora,numpeca,volpeca,preco\r\n')
            for item in brw_calcmadeira_calcula.peca_ids:
                texto = '%s,%s,%s,%s,%s,%s,%s\r\n' % (
                    item.codpeca, remove_acentuacao(cr, uid, item.nomepeca), item.numtora,
                    item.voltora, item.numpeca, item.volpeca,
                    item.preco_item)
                arquivo.write(texto)
            arquivo.close()
            vals.update({'resumido_item_dados': base64.encodestring(
                open(dir_arq_temporario, 'r').read())})
            arquivo.close()
            subprocess.call(['rm', "/tmp/arquivo.csv"])
        if brw_calcmadeira_calcula.peca_rolica_ids:
            arquivo = open(dir_arq_temporario, 'wb')
            arquivo.write('codpeca,nomepeca,numtora,voltora,numpeca,volpeca,preco\r\n')
            for item in brw_calcmadeira_calcula.peca_rolica_ids:
                texto = '%s,%s,%s,%s,%s,%s,%s\r\n' % (
                    item.codpeca, remove_acentuacao(cr, uid, item.nomepeca), item.numtora,
                    item.voltora, item.numpeca, item.volpeca, item.preco_item)
                arquivo.write(texto)
            arquivo.close()
            vals.update({'resumido_item_rolica_dados': base64.encodestring(
                open(dir_arq_temporario, 'r').read())})
            arquivo.close()
            subprocess.call(['rm', "/tmp/arquivo.csv"])
        if brw_calcmadeira_calcula.itens_hectare_ids:
            arquivo = open(dir_arq_temporario, 'wb')
            arquivo.write('codpeca,nomepeca,numtora,voltora,numpeca,volpeca,preco\r\n')
            for item in brw_calcmadeira_calcula.itens_hectare_ids:
                texto = '%s,%s,%s,%s,%s,%s,%s\r\n' % (
                    item.wcahec_codigo_peca, remove_acentuacao(cr, uid, item.wcahec_nome_peca),
                    item.wcahec_numero_tora, item.wcahec_volume_tora, item.wcahec_numero_peca,
                    item.wcahec_volume_peca, item.wcahec_preco)
                arquivo.write(texto)
            arquivo.close()
            vals.update({'resumido_item_hectare_dados': base64.encodestring(
                open(dir_arq_temporario, 'r').read())})
            arquivo.close()
            subprocess.call(['rm', "/tmp/arquivo.csv"])
        if brw_calcmadeira_calcula.itens_hectare_rolica_ids:
            arquivo = open(dir_arq_temporario, 'wb')
            arquivo.write('codpeca,nomepeca,numtora,voltora,numpeca,volpeca,preco\r\n')
            for item in brw_calcmadeira_calcula.itens_hectare_rolica_ids:
                texto = '%s,%s,%s,%s,%s,%s,%s\r\n' % (
                    item.codigo_peca, remove_acentuacao(cr, uid, item.nome_peca), item.numero_tora,
                    item.volume_tora, item.numero_peca, item.volume_peca,
                    item.preco)
                arquivo.write(texto)
            arquivo.close()
            vals.update({'resumido_item_rolica_hectare_dados': base64.encodestring(
                open(dir_arq_temporario, 'r').read())})
            arquivo.close()
            subprocess.call(['rm', "/tmp/arquivo.csv"])
        if brw_calcmadeira_calcula.result_ids:
            arquivo = open(dir_arq_temporario, 'wb')
            arquivo.write('codpeca,nomepeca,es_serra,l,n,v,numtora,vr,di_casc2,di_casc,comprimento,'
                          'di,arv,preco,conicidade,rendimento,comp_laminacao,ffarv\r\n')
            for item in brw_calcmadeira_calcula.result_ids:
                texto = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\r\n' % (
                    item.codpeca, remove_acentuacao(cr, uid, item.nomepeca), item.es_serra, item.l,
                    item.n, item.v, item.numtora, item.vr, item.di_casc2, item.di_casc,
                    item.comprimento, item.di, item.arv, item.preco, item.conicidade,
                    item.rendimento, item.comp_laminacao, item.ffarv)
                arquivo.write(texto)
            arquivo.close()
            vals.update({'analitico_item_dados': base64.encodestring(
                open(dir_arq_temporario, 'r').read())})
            arquivo.close()
            subprocess.call(['rm', "/tmp/arquivo.csv"])
        vals and self.write(cr, uid, ids, vals, context)
        return True


class wizard_calcmadeira_frequencia_peca_parcela_talhao(osv.TransientModel):
    _name = 'wizard.calcmadeira.frequencia.peca.parcela.talhao'
    _description = 'FREQUENCIA DE PEÇAS - RESULTADO POR PARCELA DENTRO DE TALHAO'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'n_pecas_freq': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'v_freq': fields.float('Volume', digits=(15, 4)),
        'talh_freq': fields.integer(_('Talhao')),
        'preco_freq': fields.float(_('Preco'), digits=(15, 4)),
        'peca': fields.char(_('Peca'), size=40),
        'codpeca_freq': fields.integer(_('Codigo Peca')),
        'parc_freq': fields.integer('Parcela'),
    }


class wizard_calcmadeira_frequencia_peca_parcela_talhao_rolica(osv.TransientModel):
    _name = 'wizard.calcmadeira.frequencia.peca.parcela.talhao.rolica'
    _description = 'FREQUENCIA PEÇAS - RESULTADO PARCELA DENTRO DE TALHAO - ROLICA'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'n_pecas_freq': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'v_freq': fields.float('Volume', digits=(15, 4)),
        'talh_freq': fields.integer(_('Talhao')),
        'preco_freq': fields.float(_('Preco'), digits=(15, 4)),
        'peca': fields.char(_('Peca'), size=40),
        'codpeca_freq': fields.integer(_('Codigo Peca')),
        'parc_freq': fields.integer('Parcela'),
    }


class wizard_calcmadeira_frequencia_peca_talhao(osv.TransientModel):
    _name = 'wizard.calcmadeira.frequencia.peca.talhao'
    _description = 'FREQUENCIA DE PEÇAS - RESULTADO POR TALHAO'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'n_pecas_freq': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'v_freq': fields.float('Volume', digits=(15, 4)),
        'talh_freq': fields.integer(_('Talhao')),
        'preco_freq': fields.float(_('Preco'), digits=(15, 4)),
        'peca': fields.char(_('Peca'), size=40),
        'codpeca_freq': fields.integer(_('Codigo Peca')),
    }


class wizard_calcmadeira_frequencia_peca_talhao_rolica(osv.TransientModel):
    _name = 'wizard.calcmadeira.frequencia.peca.talhao.rolica'
    _description = 'FREQUENCIA DE PEÇAS - RESULTADO POR TALHAO - ROLICA'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'n_pecas_freq': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'v_freq': fields.float('Volume', digits=(15, 4)),
        'talh_freq': fields.integer(_('Talhao')),
        'preco_freq': fields.float(_('Preco'), digits=(15, 4)),
        'peca': fields.char(_('Peca'), size=40),
        'codpeca_freq': fields.integer(_('Codigo Peca')),
    }


class wizard_calcmadeira_parcela_talhao(osv.TransientModel):
    _name = 'wizard.calcmadeira.parcela.talhao'
    _description = 'RESULTADO TOTALIZADO POR PARCELA DENTRO DE TALHAO'
    
    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'falha': fields.float('Falha', digits=(15,4 )),
        'parc': fields.integer('Parcela'),
        'arv': fields.float(_('Arvore Frequencia'), digits=(15, 4)),
        'preco_arv': fields.float(_('Preco Arvore'), digits=(15, 4)),
        'vt_vr': fields.float(_('Volume Arvore - Volume Peca'), digits=(15, 4)),
        'crv': fields.float('CRV', digits=(5, 4)),
        'preco_freq': fields.float(_('Preco Frequencia'), digits=(15, 4)),
        'vr': fields.float('VR', digits=(15, 4)),
        'preco_pecas': fields.float(_('Preco Peca'), digits=(15, 4)),
        'vt': fields.float('VT', digits=(15, 4)),
        'v': fields.float('V', digits=(15, 4)),
        'preco_inv': fields.float(_('Preco Inv'), digits=(15, 4)),
        'n_pecas': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'talh': fields.integer(_('Talhao')),
        'vr_v': fields.float('VR-V', digits=(15, 4)),
    }


class wizard_calcmadeira_parcela_talhao_rolica(osv.TransientModel):
    _name = 'wizard.calcmadeira.parcela.talhao.rolica'
    _description = 'RESULTADO TOTALIZADO POR PARCELA DENTRO DE TALHAO - ROLICA'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'falha': fields.float('Falha', digits=(15,4 )),
        'parc': fields.integer('Parcela'),
        'arv': fields.float(_('Arvore Frequencia'), digits=(15, 4)),
        'preco_arv': fields.float(_('Preco Arvore'), digits=(15, 4)),
        'vt_vr': fields.float(_('Volume Arvore - Volume Peca'), digits=(15, 4)),
        'crv': fields.float('CRV', digits=(5, 4)),
        'preco_freq': fields.float(_('Preco Frequencia'), digits=(15, 4)),
        'vr': fields.float('VR', digits=(15, 4)),
        'preco_pecas': fields.float(_('Preco Peca'), digits=(15, 4)),
        'vt': fields.float('VT', digits=(15, 4)),
        'v': fields.float('V', digits=(15, 4)),
        'preco_inv': fields.float(_('Preco Inv'), digits=(15, 4)),
        'n_pecas': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'talh': fields.integer(_('Talhao')),
        'vr_v': fields.float('VR-V', digits=(15, 4)),
    }


class wizard_calcmadeira_talhao(osv.TransientModel):
    _name = 'wizard.calcmadeira.talhao'
    _description = 'RESULTADO TOTALIZADO POR TALHAO'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'falha': fields.float('Falha', digits=(15,4 )),
        'arv': fields.float(_('Arvore Frequencia'), digits=(15, 4)),
        'preco_arv': fields.float(_('Preco Arvore'), digits=(15, 4)),
        'vt_vr': fields.float(_('Volume Arvore - Volume Peca'), digits=(15, 4)),
        'crv': fields.float('CRV', digits=(5, 4)),
        'preco_freq': fields.float(_('Preco Frequencia'), digits=(15, 4)),
        'vr': fields.float('VR', digits=(15, 4)),
        'preco_pecas': fields.float(_('Preco Peca'), digits=(15, 4)),
        'vt': fields.float('VT', digits=(15, 4)),
        'v': fields.float('V', digits=(15, 4)),
        'preco_inv': fields.float(_('Preco Inv'), digits=(15, 4)),
        'n_pecas': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'talh': fields.integer(_('Talhao')),
        'vr_v': fields.float('VR-V', digits=(15, 4)),
    }


class wizard_calcmadeira_talhao_rolica(osv.TransientModel):
    _name = 'wizard.calcmadeira.talhao.rolica'
    _description = 'RESULTADO TOTALIZADO POR TALHAO - ROLICA'

    _columns = {
        'capa_id': fields.many2one('wizard.calcmadeira.calcula', 'Capa'),
        'falha': fields.float('Falha', digits=(15,4 )),
        'arv': fields.float(_('Arvore Frequencia'), digits=(15, 4)),
        'preco_arv': fields.float(_('Preco Arvore'), digits=(15, 4)),
        'vt_vr': fields.float(_('Volume Arvore - Volume Peca'), digits=(15, 4)),
        'crv': fields.float('CRV', digits=(5, 4)),
        'preco_freq': fields.float(_('Preco Frequencia'), digits=(15, 4)),
        'vr': fields.float('VR', digits=(15, 4)),
        'preco_pecas': fields.float(_('Preco Peca'), digits=(15, 4)),
        'vt': fields.float('VT', digits=(15, 4)),
        'v': fields.float('V', digits=(15, 4)),
        'preco_inv': fields.float(_('Preco Inv'), digits=(15, 4)),
        'n_pecas': fields.float(_('Numero de Pecas'), digits=(15, 4)),
        'talh': fields.integer(_('Talhao')),
        'vr_v': fields.float('VR-V', digits=(15, 4)),
    }
