# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.tools.translate import _
from pynfe.nfe.manual_600.consrecinfe import ProcNFe
from datetime import date, datetime
from dateutil.parser import parse
from openerp import SUPERUSER_ID
from pynfe.nfe.manual_600 import nfe_400
from utils import adjust_m2o
from openerp.osv import osv
from SigUtils import datas
import logging
import time
import re
import locale
from utils import log_debug
from ..inherit.adm.forma_data import MEIO_PAGAMENTO


class fiscal_nf_capa(osv.Model):
    _inherit = 'fiscal.nf.capa'
    _logger = logging.getLogger(_inherit)

    class ParamToObject(object):
        """
        Converte valores de dicionarios em objetos
        """
        def __init__(self, **entries):
            self.financeiro = None
            self.fiscal = None
            self.estoque = None
            self.adm = None
            self.rh = None
            self.__dict__.update(entries)

    def _parametros(self, cr, uid, company=False, context=None):
        context = context and context.copy() or {}
        company = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, None, context, company)
        parametros = dict([(mod, self.ParamToObject(
            **self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, mod, [], company))) for mod in ['financeiro', 'fiscal', 'estoque', 'adm',
                                                         'rh']])
        return self.ParamToObject(**parametros)

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de notas fiscais que ainda nao foram faturadas.'

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuário utilizado para a verificacao de perfil,
        :param context: um dicionário padrao para valores contextuais,
        :return: dominio utilizado para contar as Ordens de Servico com ´status´ igual a 'aberta'.
        @see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        @copyright: OpenERP SA.
        """
        context = context and context.copy() or {}

        return [('state', 'in', ['1', '2', 'P', '2E'])]

    def button_nf_abrir_pedido(self, cr, uid, ids, context=None):
        """Somente para notas fiscais de servico. Abre o formulario de pedido relacionado a nota
        fiscal."""
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        brw_nf_capa = self.browse(cr, uid, ids, context)
        pedido_id = brw_nf_capa.pedido_id.id
        if not pedido_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum pedido associado a nota fiscal selecionada.'))
        else:
            reference = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'estoquepedidos', 'view_estoquepedidos_pedido_capa_form')
            view_id = reference and reference[1] or False
            return {
                'name': _('Pedido'),
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view_id],
                'res_model': 'estoquepedidos.pedido.capa',
                'res_id': pedido_id,
                'type': 'ir.actions.act_window'}

    def sequence_criar(self, cr, uid, context=None):
        """Cria o sequencial de numero de notas fiscais de servico."""

        context = context and context.copy() or {}
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(
            cr, uid, self._name, context=context)
        __code = 'numero_rps.fiscal'
        __name = 'Numero do RPS para NFS-e'
        vals = {
            'name': __name,
            'code': __code,
            'padding': 10,
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence.type').search(
            cr, uid, [('code', '=', __code)], context=context)

        if not consulta:
            self.pool.get('ir.sequence.type').create(
                cr, uid, {'name': __name, 'code': __code}, context)

        self.pool.get('ir.sequence').create(cr, uid, vals, context)

        return True

    def fisnfc_button_print_promissoria(self, cr, uid, ids, context=None):
        """
        O metodo cria a promissoria em arquivo PDF, referente aos dados da nota fiscal. A
        promissoria so e disponivel para download caso a nota seja faturada com alguma forma de
        pagamento que esteja selacionada para tal.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de id da tabela desse objeto.
        :type ids: list
        :param context: dicionario com valores contextuais.
        :type context: dict
        :return: dicionario com acao para a classe do relatorio.
        :rtype: dict
        """
        fiscal_nf_capa_dados = self.read(cr, uid, ids[0], ['numero_nf'], context=context)
        nome_arquivo = 'Promissoria_%s' % fiscal_nf_capa_dados.get('numero_nf')
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'promissoria',
            'name': nome_arquivo
        }

    def fisnfc_onchange_frete_tipo_terceiro(self, cr, user, ids, itens, frete, tipo, nat_id,
                                            context=None):
        """
        Atualiza o valor rateado do frete e o valor base das tributacoes em funcao do tipo de frete.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: o valor do frete na capa da nota fiscal
        :param tipo: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type frete: float
        :type tipo: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        itens = self.fisnfc_rateio_frete(cr, user, itens, frete, tipo, nat_id, context)
        if not itens:
            frete = 0

        return {'value': {'estoque_movimento_itens_ids': itens}, 'vr_frete': frete}

    def fisnfc_onchange_frete_tipo(self, cr, user, ids, itens, frete, tipo_frete, valor, nat_id,
                                   context=None):
        """
        Atualiza o valor rateado do frete e o valor base das tributacoes em funcao do tipo de frete.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: tipo de frete
        :param tipo_frete: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param valor: o valor do frete na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type frete: str
        :type tipo_frete: str
        :type valor: float
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        vals = {}
        if frete and frete in '03':
            tipo_frete = 'D'
            vals = {}.fromkeys(
                ['modelo_nf_frete', 'serie_frete', 'data_vencimento_frete'], False)
        else:
            tipo_frete = 'F'
            valor = 0
        vals.update({'estoque_movimento_itens_ids': self.fisnfc_rateio_frete(
            cr, user, itens, valor, tipo_frete, nat_id, context), 'vr_frete': valor,
            'tipo_frete': tipo_frete})

        return {'value': vals}

    def onchange_data_emissao(self, cr, uid, ids, data_emissao, nf_propria, tipo, context=None):
        """ Se a nota fiscal for propria e de entrada, ao trocar a data de emissao, o valor sera
         atribuido igualmente para a data de entrada e saida."""

        context = context and context.copy() or {}

        if data_emissao and nf_propria and tipo == 'E':
            vals = {'data_saida_entrada': data_emissao}
        else:
            vals = {}

        return {'value': vals}

    def onchange_verifica_duplicacao(self, cr, uid, ids, company_id, numero_nf, serie_nf,
                                     data_emissao, destinatario_id, nf_propria):
        """ Verifica se exite uma nota fiscal de entrada propria que ja foi lancada com a mesma
        data de emissao."""
        if not nf_propria:
            if serie_nf and numero_nf and data_emissao and destinatario_id:
                res = self.verifica_duplicacao_nf(
                    cr, uid, tipo='E', serie_nf=serie_nf, numero_nf=numero_nf,
                    nf_propria=nf_propria, company_id=company_id, destinatario_id=destinatario_id,
                    data_emissao=data_emissao, context={'onchange_verifica': True})

                if 'numero_nf' in res and 'serie_nf' in res:
                    warning = {
                        'title': _('Atencao'),
                        'message': _('Nota fiscal ja lancada para esta data.')
                    }

                    return {'value': res, 'warning': warning}

        return {'value': {}}

    def verifica_executa_status(self, cr, uid, ids, state_nf, context=None):
        """
        Verificacao de validacao em cascata
        """
        context = context and context.copy() or {}
        state = self.browse(cr, uid, ids[0] if isinstance(ids, list) else id, context).state
        result = {}
        if state_nf == '1' and state == '2':  # Retorna a nota fiscal ao estado de esboco
            self.button_rascunho(cr, uid, ids, context)
        elif state_nf == '2' and state == '1':  # Verifica e valida a nota fiscal
            self.button_verificar_nf(cr, uid, ids[0], context)
        elif state_nf == '3' and state == '2':  # Fatura a nota fiscal
            result = self.button_faturar(cr, uid, ids, context)
        elif state_nf == '3' and state == '1':  # Verifica, valida e fatura a nota fiscal
            self.button_verificar_nf(cr, uid, ids[0], context)
            result = self.button_faturar(cr, uid, ids, context)
        elif state_nf == 'C' and state != 'C':  # Cancela a nota fiscal
            self.fisnfc_funcao_button_cancelar(cr, uid, ids, context)

        return result

    # TODO: Mover o metodo para a classe fiscal.cfop.natureza.encargos.
    def verificar_encargos_natureza(self, cr, uid, natureza_id, context=None):
        """
        Retorna os ids dos encargos que estao cadastrados para a natureza informada
        """

        context = context and context.copy() or {}
        encargo_ids = []
        cfop_natureza_dados = self.pool.get('fiscal.cfop.natureza').read(
            cr, uid, natureza_id, ['natureza_encargos_ids'], context)

        if cfop_natureza_dados and isinstance(cfop_natureza_dados, list):
            cfop_natureza_dados = cfop_natureza_dados[0]

        natureza_encargo_ids = cfop_natureza_dados.get('natureza_encargos_ids')
        if natureza_encargo_ids:
            cfop_natureza_encargo_dados = self.pool.get('fiscal.cfop.natureza.encargos').read(
                cr, uid, natureza_encargo_ids, ['encargo_id'], context, '_classic_write')

            # Separa somente encargos para a natureza da operacao
            nat_encargo_id = map(lambda x: x.get('encargo_id'), cfop_natureza_encargo_dados)
            encargo_ids.extend(nat_encargo_id)

        return encargo_ids

    @log_debug
    def verificar_cfop_devolucao_transferencia(self, cr, uid, dados_cfop_nat, dict_itens,
                                               dict_produtos, dict_cfop_itens, context=None):
        context = context and context.copy() or {}
        cfop_movimento_tipo = dados_cfop_nat.get('movimento_tipo')
        msg_cfop_devolucao = msg_cfop_transferencia = msg_cfop_outros = ''
        for item_id in dict_itens:
            item = dict_itens.get(item_id)
            produto = dict_produtos.get(item.get('produto_id'))
            cfop = dict_cfop_itens.get(item.get('cfop_id')) or {}
            if cfop_movimento_tipo in ('D', 'T'):
                if cfop_movimento_tipo != cfop.get('movimento_tipo'):
                    if cfop_movimento_tipo == 'D':
                        msg_cfop_devolucao += '* ' + produto.get('codigo') + ' - ' + \
                                              produto.get('name').decode('utf-8') + '\n'
                    elif cfop_movimento_tipo == 'T':
                        try:
                            msg_cfop_transferencia += '* ' + produto.get('codigo') + ' - ' + \
                                                  produto.get('name').decode('utf-8') + '\n'
                        except:
                            msg_cfop_transferencia += '* ' + produto.get('codigo') + ' - ' + \
                                                      produto.get('name') + '\n'
            else:
                if cfop.get('movimento_tipo') in ('D', 'T'):
                    msg_cfop_outros += '* ' + produto.get('codigo') + ' - ' + \
                                       produto.get('name').decode('utf-8') + '\n'
        if msg_cfop_devolucao:
            raise osv.except_osv(_('Atencao!'),
                                 _('Os itens: %s da nota devem possuir um CFOP de devolucao.') %
                                 msg_cfop_devolucao)
        if msg_cfop_transferencia:
            raise osv.except_osv(_('Atencao!'),
                                 _('Os itens: %s da nota devem possuir um CFOP de transferencia.') %
                                 msg_cfop_transferencia)
        if msg_cfop_outros:
            raise osv.except_osv(_('Atencao!'),
                                 _('O CFOP dos itens: %s sao invalidos para a natureza da operacao '
                                   'informada, permitido somente CFOPs que nao sejam de devolucao '
                                   'ou de transferencia.') % msg_cfop_outros)

        return True

    def verifica_cfop_fora_estado(self, cr, uid, company_id, natureza_id, destinatario_endereco_id,
                                  tipo, cfop_id=False, indicador_presenca=None, context=None):
        context = context and context.copy() or {}
        load = '_classic_write'
        brw_cfop_natureza = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, natureza_id, context)
        dados_company = self.pool.get('res.company').read(
            cr, uid, company_id, ['rescon_pessoa_id'], context, load)
        obj_adm_pessoa_endereco = self.pool.get('adm.pessoa.endereco')
        estado_destinatario = False
        if destinatario_endereco_id:
            endereco = obj_adm_pessoa_endereco.read(
                cr, uid, destinatario_endereco_id, ['uf_id'], context, load)
            if endereco.get('uf_id'):
                estado_destinatario = self.pool.get('adm.uf').read(
                    cr, uid, endereco.get('uf_id'), ['sigla'], context).get('sigla') or False
        nf_cfop_codigo = brw_cfop_natureza.cfop_id.codigo
        if nf_cfop_codigo[0] in '37':  # Se for exportacao, nao valida
            return True
        if cfop_id:
            nf_cfop_codigo = self.pool.get('fiscal.cfop').read(
                cr, uid, cfop_id, ['codigo'], context).get('codigo')
        msg = ''
        if destinatario_endereco_id and not estado_destinatario:
            msg += _('Nao existe estado informado para o endereco do destinatario.\n')
        enderecos = self.pool.get('adm.pessoa').admpes_endereco_fiscal(
            cr, uid, dados_company.get('rescon_pessoa_id'), context).get(
            dados_company.get('rescon_pessoa_id'))
        estado_emitente = enderecos and enderecos.get('uf_sigla') or False
        if not estado_emitente:
            msg += _('Nao existe estado informado para o endereco do emitente.\n')
        if not msg:
            msg_cfop = ''
            estados_emitente_destinatario = \
                _("UF Emitente: %s - UF Destinatario: %s. CFOP - %s") % \
                (estado_emitente, estado_destinatario, nf_cfop_codigo)
            if tipo == 'S':
                if estado_emitente == estado_destinatario:
                    if nf_cfop_codigo[0] != '5':
                        msg_cfop = _("Natureza da operacao invalida para operacoes no mesmo "
                                     "estado. %s") % estados_emitente_destinatario
                else:
                    # if self.pool.get('adm.pessoa').admpes_get_tipo_consumidor(
                    #     cr, uid, endereco.get('pessoa_id'), context=context) != 'F':
                        if nf_cfop_codigo[0] != '6':
                            # caso a venda seja presencial mas o destinatario eh de um UF diferente nao faz
                            # a verificacao
                            if indicador_presenca and indicador_presenca != '1':
                                msg_cfop = _("Natureza da operacao invalida para operacoes fora do "
                                             "estado. %s") % estados_emitente_destinatario
                                return msg_cfop

            if tipo == 'E':
                if estado_emitente == estado_destinatario:
                    if nf_cfop_codigo[0] == '2':
                        msg_cfop = _("Natureza da operacao invalida para operacoes no mesmo "
                                     "estado. %s") % estados_emitente_destinatario
                else:
                    if nf_cfop_codigo[0] == '1':
                        msg_cfop = _("Natureza da operacao invalida para operacoes fora do "
                                     "estado. %s") % estados_emitente_destinatario
            if msg_cfop:
                raise osv.except_osv(_('Atencao!'), msg_cfop)
        elif msg:
            raise osv.except_osv(_('Atencao!'), msg)

        return True

    @log_debug
    def fisnfc_verificar_parcelas(self, cr, user, dados_nf, dict_parcelas, dict_cfop_itens,
                                  context=None):
        """
        Verifica divergencias nas parcelas de uma nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao existir divergencia entre parcelas e condicao de pagamento.
        :rtype: bool
        """
        context = context and context.copy() or {}
        if dados_nf.get('nf_propria'):
            result = self._fisnfc_verificar_parcelas_propria(
                cr, user, dados_nf, dict_parcelas, context)
        else:
            result = self._fisnfc_verificar_parcelas_terceiros(
                cr, user, dados_nf, dict_parcelas, dict_cfop_itens, context)

        return result

    def _fisnfc_get_tipo_condicao_pagamento(self, cr, user, nf, context=None):
        """
        Retorna a condicao de pagamento de uma nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: 'V' para condicao "a vista" e 'P' para condicao a prazo.
        :rtype: str
        """
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, nf, context)
        tipo_condicao = 'V'
        if brw_nf.nf_propria:
            if filter(lambda f: ('C' in f) or ('P' in f),
                      map(lambda m: m.condicao_forma_id.adm_condicao_id.tipo,
                          brw_nf.nf_condicao_ids)):
                tipo_condicao = 'P'
            if brw_nf.tipo == 'S':  # Nota de saida (venda/devolucao de compra)
                # na nota de saida, cartao a vista e cheque a vista sao considerados a prazo
                if filter(lambda f: f.forma_pagto_id.cartao_credito_debito, brw_nf.nf_condicao_ids)\
                    or filter(lambda f: f.forma_pagto_id.admfpg_documento_tipo_id.sigla == 'CHQ',
                                  brw_nf.nf_condicao_ids):
                    tipo_condicao = 'P'
        else:  # Nota fiscal de terceiro
            if filter(lambda f: f.vencimento > brw_nf.data_emissao[:10],
                      brw_nf.fisnfc_parcelas_terceiros_ids):
                tipo_condicao = 'P'

        return tipo_condicao

    @log_debug
    def _fisnfc_verificar_parcelas_propria(self, cr, user, dados_nf, dict_parcelas, context=None):
        """
        Verifica divergencias nas parcelas de uma nota fiscal propria.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao existir divergencia entre parcelas e condicao de pagamento.
        :rtype: bool
        """
        context = context and context.copy() or {}
        if dados_nf.get('finalidade_emissao') == '2':
            return True
        if not dados_nf.get('nf_parcelas_ids'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhuma parcela inserida na nota fiscal.'))

        # Nao permite notas fiscais com parcelas vencidas
        data_emissao = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
            cr, user, datetime.strptime(dados_nf.get('data_emissao'), '%Y-%m-%d %H:%M:%S'), 'UTC',
            context=context), '%Y-%m-%d')
        if filter(lambda f: f.get('vencimento') < data_emissao, dict_parcelas.values()):
            raise osv.except_osv(_('Atencao!'),
                                 _('Data de vencimento da parcela inferior a data de emissao.'))

        # O somatorio dos valores das parcelas deve ser igual ao total da nota
        parcelas_total = sum(map(lambda m: m.get('valor'), dict_parcelas.values()))
        vr_deducao = dados_nf.get('vr_total_deducao') if dados_nf.get('tipo') == 'E' else 0
        if not round(parcelas_total, 2) == round(dados_nf.get('vr_total_nf') - vr_deducao, 2):
            raise osv.except_osv(_('Atencao!'),
                                 _('O valor total da nota deve ser igual ao valor total das '
                                   'parcelas.'))

        return True

    @log_debug
    def _fisnfc_verificar_parcelas_terceiros(self, cr, user, dados_nf, dict_parcelas,
                                             dict_cfop_itens, context=None):
        """
        Verifica divergencias nas parcelas de uma nota fiscal de terceiro.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao existir divergencia entre parcelas.
        :rtype: bool
        """
        context = context and context.copy() or {}

        # Se TODOS os itens forem de bonificaao, desconsidera verificacao das parcelas
        for i in dict_cfop_itens.values():
            if i.get('codigo') != '910' and not dados_nf.get('fisnfc_parcelas_terceiros_ids'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Nenhuma parcela inserida na nota fiscal.'))

        tot_parcelas = 0
        # Nao permite notas fiscais com parcelas vencidas
        for i in dict_parcelas.values():
            tot_parcelas += i.get('valor')
            if i.get('vencimento') < dados_nf.get('data_emissao')[:10]:
                raise osv.except_osv(_('Atencao!'),
                                     _('Data de vencimento da parcela inferior a data de emissao.'))

        parcelas_total = dados_nf.get('fisnfc_valor_bonificado') + tot_parcelas
        vr_deducao = dados_nf.get('vr_total_deducao')
        if not round(parcelas_total, 2) == round(dados_nf.get('vr_total_nf') - vr_deducao, 2):
            raise osv.except_osv(_('Atencao!'),
                                 _('O valor total da nota deve ser igual ao valor total das '
                                   'parcelas.'))

        return True

    @log_debug
    def fisnfc_verificar_condicao_pagamento(self, cr, user, dados_nf, dict_condicao, dict_forma_pgto,
                                            context=None):
        """
        Verifica divergencias nas condicoes de pagamento de uma nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao existir divergencia entre parcelas e condicao de pagamento.
        :rtype: bool
        """
        context = context and context.copy() or {}
        parametro_financeiro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'financeiro', [], context=context)
        if dados_nf.get('finalidade_emissao') == '2': # Nota de complemento
            return True
        if not dados_nf.get('nf_condicao_ids') and dados_nf.get('nf_propria'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhuma condicao de pagamento inserida na nota.'))
        if self._fisnfc_get_tipo_condicao_pagamento(cr, user, dados_nf.get('id'), context) == 'V':
            if not parametro_financeiro.get('local_cobranca_id'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Local de cobranca nao informado nos parametros do '
                                       'Financeiro.'))
            else:
                if not self.pool.get('financeiro.local.cobranca').browse(
                        cr, user, parametro_financeiro.get('local_cobranca_id'), context).finloc_conta_id:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Conta nao informada no local de cobranca em parametros '
                                           'do Financeiro.'))
        chq_doc_tipo_id = self.pool.get('documento.documento.tipo').search(
            cr, user, [('sigla', '=', 'CHQ')], context=context)
        chq_doc_tipo_id = chq_doc_tipo_id and chq_doc_tipo_id[0] or False
        for condicao in dict_condicao.values():
            forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
            if forma_pgto and not forma_pgto.get('admfpg_tipo'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Pagamento em "%s" nao tem forma de pagamento definida.') %
                                     forma_pgto.get('name'))
            if forma_pgto.get('admfpg_documento_tipo_id') == chq_doc_tipo_id:
                if not all((condicao.get('fiscpg_chq_banco'), condicao.get('fiscpg_chq_agencia'),
                            condicao.get('fiscpg_chq_numero'), condicao.get('fiscpg_chq_conta'))):
                    raise osv.except_osv(_('Atencao!'),
                                         _('Forma de pagamento Cheque com dados incompletos.\n'
                                           'Banco: %s\nAgencia: %s\nConta: %s\n Cheque nr.: %s.' % (
                                             condicao.get('fiscpg_chq_banco') or '* Falta *',
                                             condicao.get('fiscpg_chq_agencia') or '* Falta *',
                                             condicao.get('fiscpg_chq_conta') or '* Falta *',
                                             condicao.get('fiscpg_chq_numero') or '* Falta *')))

        return True

    @log_debug
    def fisnfc_criar_fatura_pvr(self, cr, user, dados_nf, dados_operacao, dict_condicao,
                                dict_forma_pgto, dict_cond_forma, dict_adm_cond, dict_parcelas,
                                context):
        obj_doc_doc = self.pool.get('documento.documento.tipo')
        nf_id = dados_nf.get('id')

        # Verifica se ja existe fatura para nota fiscal
        fatura_id = self.pool.get('financeiro.fatura').search(
            cr, user, [('nf_id', '=', nf_id), ('cancelada', '=', False)], context=context)

        # Verifica se gera movimento no financeiro
        if not dados_operacao.get('estnto_gerar_financeiro') or fatura_id:
            return True

        if dados_nf.get('nf_propria'):
            # Contexto passado para que o sequence seja gerado
            context.update({'lancamento_fatura_r': True})
        fatura_pessoas_ids = {}
        doc_tipo_id = obj_doc_doc.search(cr, user, [('sigla', '=', 'PRV')], context=context)[0]
        pessoa_id = dados_nf.get('destinatario_id')
        vals_fatura = {
            'company_id': dados_nf.get('company_id'),
            'pessoa_id': pessoa_id,
            'data_emissao': dados_nf.get('data_emissao'),
            'tipo_fatura': 'R' if dados_nf.get('tipo') == 'S' else 'P',
            'nf_id': nf_id,
            'documento': str(dados_nf.get('numero_nf') or '').zfill(9)
        }
        if doc_tipo_id:
            vals_fatura['finfat_documento_tipo_id'] = doc_tipo_id

        # Nota proria
        for condicao in dict_condicao.values():
            pessoa_id = dados_nf.get('destinatario_id')
            forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
            if forma_pgto.get('cartao_credito_debito'):
                pessoa_id = forma_pgto.get('admfpg_cartao_pessoa_id')

            if dados_nf.get('fisnfc_modelo_serie') == '02D' or dados_nf.get('fisnfc_consumidor')\
                    or dados_nf.get('fisnfc_modelo') >= '90':
                if not pessoa_id and forma_pgto.get('name') == 'Dinheiro':
                    continue

            # Cria uma fatura para cada condicao da nota onde a pessoa eh diferente
            fatura_id = fatura_pessoas_ids.get(pessoa_id)
            if not fatura_id:
                vals_fatura['pessoa_id'] = pessoa_id
                fatura_id = self.pool.get('financeiro.fatura').create(
                    cr, user, vals_fatura, context)
                fatura_pessoas_ids[pessoa_id] = fatura_id

            # Titulo deve ser criado para nota de saida e entrada em previsao fin.
            filter_dict_parcelas = {}
            parcela_ids = filter(
                lambda p: dict_parcelas.get(p).get('nf_condicao_id') == condicao.get('id'),
                dict_parcelas)
            for parcela_id in parcela_ids:
                filter_dict_parcelas.update({parcela_id: dict_parcelas.get(parcela_id)})
            self.fisnfc_cria_titulos_prv_financeiro(
                cr, user, dados_nf, dict_forma_pgto, dict_cond_forma, dict_adm_cond, fatura_id,
                pessoa_id, dict_parcelas=filter_dict_parcelas, dict_condicao=dict_condicao,
                dados_condicao=condicao, context=context)

        if not dados_nf.get('nf_propria'): # nota de terceiros
            fatura_id = self.pool.get('financeiro.fatura').create(
                cr, user, vals_fatura, context=context)
            self.fisnfc_cria_titulos_prv_financeiro(
                cr, user, dados_nf, dict_forma_pgto, dict_cond_forma, dict_adm_cond, fatura_id,
                pessoa_id, dict_condicao=dict_condicao, dict_parcelas=dict_parcelas,
                context=context)

        return True

    def fisnfc_cria_titulos_prv_financeiro(self, cr, uid, dados_nf, dict_forma_pgto,
                                           dict_cond_forma, dict_adm_cond, fatura, pessoa_id,
                                           dados_condicao=False, dict_condicao={}, dict_parcelas={},
                                           context=None):
        """
        Cria titulo em previsao no financeiro

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param fatura: ID da fatura
        :param dados_condicao: se tiver condicao eh nota propria
        :param dict_parcelas: se tiver parcela eh nota de terceiros
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type fatura: long
        :type context: dict
        :rtype: long

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        res = False
        obj_adm_utl = self.pool.get('adm.utils')
        obj_doc_doc = self.pool.get('documento.documento.tipo')
        fields_to_read_finpar = ['moeda_id', 'local_cobranca_id']
        parametros_dados = obj_adm_utl.admutl_get_parametros(
            cr, uid, 'financeiro', fields_to_read_finpar)
        moeda_id = parametros_dados.get('moeda_id', False)
        localcobranca_id = parametros_dados.get('local_cobranca_id', False)
        prv_doc_tipo_id = obj_doc_doc.search(cr, uid, [('sigla', '=', 'PRV')], context=context)[0]
        tipo_titulo = 'P' if dados_nf.get('tipo') == 'E' else 'R'
        vals_titulo = {'company_id': dados_nf.get('company_id'),
                       'local_cobranca_id': localcobranca_id,
                       'natureza_cobranca_id': '',
                       'moeda_id': moeda_id,
                       'state': 'A',
                       'pessoa_id': pessoa_id,
                       'tipo_lancamento': 'A',
                       'doc_tipo_id': prv_doc_tipo_id,
                       'data_emissao': obj_adm_utl.admutl_datetime_to_timezone(
                           cr, uid, parse(dados_nf.get('data_emissao')), '%Y-%m-%d'),
                       'tipo_titulo': tipo_titulo,
                       'documento': str(dados_nf.get('numero_nf') or ''),
                       'fatura_id': fatura}
        # Atualiza as parcelas da nota fiscal
        if dados_nf.get('tipo') == 'S':
            data_emissao = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
                cr, uid, datetime.strptime(dados_nf.get('data_emissao'), '%Y-%m-%d %H:%M:%S'),
                'UTC', context=context), '%Y-%m-%d')
            parcela_ids = []
            for parcela in dict_parcelas.values():
                condicao = dict_condicao.get(parcela.get('nf_condicao_id'))
                cond_forma = dict_cond_forma.get(condicao.get('condicao_forma_id'))
                adm_cond = dict_adm_cond.get(cond_forma.get('adm_condicao_id'))
                if adm_cond.get('tipo') in ['V', 'S']:
                    parcela_ids.append(parcela.get('id'))
            self.pool.get('fiscal.nf.parcelas').write(
                cr, uid, parcela_ids, {'vencimento': data_emissao}, context)
        num_parcela = 1
        chq_doc_tipo_id = self.pool.get('documento.documento.tipo').search(
            cr, uid, [('sigla', '=', 'CHQ')], context=context)
        chq_doc_tipo_id = chq_doc_tipo_id and chq_doc_tipo_id[0] or False
        for parcela in dict_parcelas.values():
            condicao = dict_condicao.get(parcela.get('nf_condicao_id'))
            if condicao:
                forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
                if forma_pgto.get('admfpg_documento_tipo_id') == chq_doc_tipo_id:
                    vals_titulo.update({'documento': condicao.get('fiscpg_chq_numero'),
                                        'chq_banco': condicao.get('fiscpg_chq_banco'),
                                        'chq_agencia': condicao.get('fiscpg_chq_agencia'),
                                        'chq_emitente': condicao.get('fiscpg_chq_emitente'),
                                        'chq_numero': condicao.get('fiscpg_chq_numero'),
                                        'chq_conta': condicao.get('fiscpg_chq_conta')})
            valor = parcela.get('valor')
            vals_titulo['data_vencimento'] = parcela.get('vencimento')
            vals_titulo['vr_original'] = valor
            vals_titulo['saldo'] = valor
            vals_titulo['parcela'] = num_parcela
            res = self.fisnfc_criar_titulo(cr, uid, dados_nf.get('id'), vals_titulo, context)
            num_parcela += 1

        # Para os titulos a receber se houver condicao cartao e a taxa for maoir que 0, cria
        # fatura e titulo para a pagar(Obs: apenas notas prorioas podem possuir condicao cartao)
        if tipo_titulo == 'R' and dados_nf.get('nf_propria'):
            dict_condicao = {dados_condicao.get('id'): dados_condicao}
            self.criar_titulo_fatura_cartao(
                cr, uid, moeda_id, localcobranca_id, dados_nf, dict_condicao, dict_forma_pgto,
                dict_cond_forma, dict_parcelas, previsao_doc_id=prv_doc_tipo_id, context=context)
        return res

    @log_debug
    def fisnfc_verifica_itens_saldo(self, cr, uid, dados_nf, dict_itens, dict_produtos, dict_locais,
                                    local_grades, context=None):
        context = context and context.copy() or {}
        obj_itens = self.pool.get('estoque.movimento.itens')
        log = ''
        saldos = {}
        data = str(date.today())
        for local_id, grade_ids in local_grades.items():
            saldos[local_id] = self.pool.get('estoque.produto.saldo').get_saldo_do_dia(
                            cr, uid, eps_grade_ids=grade_ids, eps_data=data,
                            eps_local_id=local_id, context=context)
        for item_id in dict_itens:
            item = dict_itens.get(item_id)
            produto = dict_produtos.get(item.get('produto_id'))
            if produto.get('tipo') == 'P':
                grade_id = item.get('grade_id')
                context['retorno_sem_codigo'] = True
                # Se for nota fiscal de saida
                if produto.get('tipo') == 'P' and dados_nf.get('tipo') == 'S':
                    local_id = item.get('local_id')
                    local = dict_locais.get(local_id)
                    local_name = local.get('name') or ''

                    # Verifica se o local aceita saldo negativo
                    if not local.get('aceita_saldo_negativo'):
                        # Recupera saldo do dia para o produto
                        saldo = saldos.get(local_id).get(grade_id).get('saldo')
                        # Verifica se a quantidade pedida eh maior que o saldo disponivel
                        if not obj_itens.estmvi_valida_disponibilidade_saldo(saldo, item):
                            log += obj_itens.estmvi_get_mensagem_saldo_indisp(
                                produto.get('name'), local_name, item, saldo)

            if log:
                raise osv.except_osv(_('Atencao!'), log)

        return True

    def adicionar_pedidos_itens(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        context.update({'adicionar_pedido_compras': True})
        brw_nf = self.browse(cr, uid, ids[0], context)
        dados_pessoa = {'id': brw_nf.destinatario_id.id,
                        'pessoa_tipo': brw_nf.destinatario_id.pessoa_tipo,
                        'admpes_jur_ie': brw_nf.destinatario_id.admpes_jur_ie,
                        'admpes_fis_produtor_rural': brw_nf.destinatario_id.admpes_fis_produtor_rural}
        values = {}
        brw_pedidos = brw_nf.fisnfc_pedido_ids
        if not brw_pedidos:
            raise osv.except_osv(_('Atencao!'), _('Nenhum pedido selecionado.'))

        # Atualiza os valores da capa da nota fiscal em funcao dos valores da capa dos pedidos
        transportador_id = brw_pedidos[0].transportador_id and brw_pedidos[0].transportador_id.id \
                           or False
        values.update({'company_id': brw_nf.company_id.id,
                       'destinatario_id': brw_nf.destinatario_id.id,
                       'destinatario_endereco_id': brw_nf.destinatario_endereco_id.id,
                       'natureza_id': brw_nf.natureza_id.id,
                       'vendedor_id': brw_nf.vendedor_id.id,
                       'vendedor_externo_id': brw_nf.vendedor_externo_id.id,
                       'tipo': brw_nf.tipo,
                       'fisnfc_nf_devolucao': brw_nf.fisnfc_nf_devolucao})
        values['transportador_id'] = transportador_id
        values['vr_frete'] = sum(map(lambda m: m.vr_frete or 0, brw_pedidos)) + brw_nf.vr_frete
        values['frete'] = '1' if values['vr_frete'] else '0'
        values['tipo_frete'] = 'F' if values['frete'] == 1 else 'D'
        values['vr_desconto'] = sum(map(lambda m: m.vr_desconto or 0, brw_pedidos)) + \
                                brw_nf.vr_desconto
        values['vr_despesas_acessorias'] = \
            sum(map(lambda m: m.vr_despesas_acessorias or 0, brw_pedidos)) + \
            brw_nf.vr_despesas_acessorias
        values['vr_seguro'] = sum(map(lambda m: m.vr_seguro or 0, brw_pedidos)) + brw_nf.vr_seguro

        # Acrescenta os itens dos pedidos na nota fiscal
        itens = list()
        dict_parcelas = {}
        for brw_pedido in brw_pedidos:
            itens.extend([
                [0, False, {'local_id': i.local_id.id, 'produto_id': i.produto_id.id,
                            'quantidade': i.qtde_pedida,
                            'produto_embalagem_id': i.produto_embalagem_id.id,
                            'grade_id': i.grade_id.id, 'vr_desconto': i.vr_desconto,
                            'vr_unitario': i.vr_unitario, 'vr_total_item': i.valor_total,
                            'vr_total_item_rateado': i.valor_total}] for i in
                brw_pedido.pedido_item_ids])

            # Realiza o rateio do valor do frete para os itens
            itens = self.fisnfc_rateio_frete(
                cr, uid, itens, values.get('vr_frete'), brw_nf.tipo, brw_nf.natureza_id.id, context)

            # Realiza o rateio do valor do desconto para os itens
            itens = self.fisnfc_rateio_desconto(cr, uid, itens, values.get('vr_desconto'),
                                                  brw_nf.tipo_frete, brw_nf.natureza_id.id, context)
            # Realiza o rateio do valor das despesas acessorias para os itens
            itens = self.fisnfc_rateio_despesas_acessorias(
                cr, uid, itens, values.get('vr_despesas_acessorias'), brw_nf.tipo_frete,
                brw_nf.natureza_id.id, context)

            # Realiza o rateio do valor do seguro para os itens
            itens = self.fisnfc_rateio_seguro(
                cr, uid, itens, values.get('vr_seguro'), brw_nf.tipo_frete, brw_nf.natureza_id.id,
                context)

            # Tributacoes dos itens
            for item in itens:
                item[2]['tributacoes_ids'] = \
                    self.pool.get('estoque.movimento.itens').estmvi_get_tributos_item(
                        cr, uid, item[2].get('produto_id'), brw_nf.natureza_id.id,
                        dados_pessoa, 'E', item[2].get('vr_total_item', 0),
                        item[2].get('vr_seguro_rateado', 0), item[2].get('vr_despesas_rateado', 0),
                        item[2].get('vr_frete_rateado', 0), item[2].get('vr_desconto_rateado', 0),
                        context=context)
            item[2] = adjust_m2o(item[2])

            # Ajusta o valor total dos itens rateados
            itens = self.fisnfc_set_valor_total_item_rateado(
                cr, uid, itens, brw_nf.tipo_frete, context)


            for i in brw_pedido.pedido_compra_parcela_ids:
                if not dict_parcelas.get(i.data_vencimento):
                    dict_parcelas[i.data_vencimento] = {'valor': i.vr_parcela,
                                                        'vencimento': i.data_vencimento}
                else:
                    dict_parcelas[i.data_vencimento]['valor'] += i.vr_parcela

        values['estoque_movimento_itens_ids'] = itens
        self.pool.get('estoque.movimento.itens')._estmvi_create_fiscal(cr, uid,
                                         values.get('estoque_movimento_itens_ids'), values, context)
        parcelas = []
        for i in dict_parcelas.values():
            parcelas.append([0, 0, i])

        values['fisnfc_parcelas_terceiros_ids'] = parcelas
        return self.write(cr, uid, ids, values, context)

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        args = [] if not args else args
        context = context and context.copy() or {}
        if name:
            try:
                ids = self.search(
                    cr, user, [('numero_nf', '=', name)] + args, limit=limit, context=context)
            except:
                ids = []
        else:
            ids = self.search(cr, user, args, limit=limit, context=context)

        return self.name_get(cr, user, ids, context)

    def _check_gerar_xml(self, cr, uid, destinatario_id, context=None):
        context = context and context.copy() or {}

        if not destinatario_id:
            return True

        id = self.pool.get('res.users').browse(cr, uid, uid, context).company_id.rescon_pessoa_id.id

        return id == destinatario_id

    @log_debug
    def verificar_incluir_cli_for(self, cr, uid, tipo, pessoa_id, company_id, acao,
                                  natureza_id=False, context=None):
        context = context and context.copy() or {}
        obj_financeiro_cliente = self.pool.get('financeiro.cliente')
        obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
        res = True
        if not natureza_id:
            return res
        cfop_id = self.pool.get('fiscal.cfop.natureza').read(
            cr, uid, natureza_id, ['cfop_id'], context, '_classic_write').get('cfop_id')
        dados_cfop = self.pool.get('fiscal.cfop').read(
            cr, uid, cfop_id, ['movimento_tipo'], context)

        # Nota de devolucao nao faz nada
        if natureza_id and dados_cfop.get('movimento_tipo') != 'D':
            if tipo == 'E':
                res = obj_financeiro_fornecedor.verificar_incluir_fornecedor(
                    cr, uid, pessoa_id, company_id, acao, context)
            elif tipo == 'S':
                res = obj_financeiro_cliente.verificar_incluir_cliente(
                    cr, uid, pessoa_id, company_id, acao, context)

        return res

    def verifica_endereco_destinatario(self, cr, uid, tipo, destinatario_id, company_id,
                                       natureza_id=False, context=None):
        context = context and context.copy() or {}
        id_end = False
        vendedor_id = False
        vendedor_externo_id = False
        gerar_xml = True
        if destinatario_id:
            verificar = self.verificar_incluir_cli_for(
                cr, uid, tipo, destinatario_id, company_id, 'verificar', natureza_id)
            if not verificar:
                criar = self.verificar_incluir_cli_for(
                    cr, uid, tipo, destinatario_id, company_id, 'incluir')
            endereco = self.pool.get('adm.pessoa').read(
                cr, uid, destinatario_id, ['endereco_nf_id', 'endereco_ids'], context,
                '_classic_write')
            if endereco:
                endereco_id = endereco.get('endereco_nf_id')
                if endereco_id:
                    id_end = endereco_id
                else:
                    endereco_id = endereco.get('endereco_ids')
                    if endereco_id:
                        id_end = endereco_id[0]
            if company_id:
                obj_financeiro_cliente = self.pool.get('financeiro.cliente')
                cliente_ids = obj_financeiro_cliente.search(
                    cr, uid, [('pessoa_id', '=', destinatario_id), ('company_id', '=', company_id)],
                    context=context)
                if cliente_ids:
                    cliente_id = cliente_ids[0]
                    cliente_dados = obj_financeiro_cliente.read(
                        cr, uid, cliente_id, ['vendedor_id', 'vendedor_externo_id'], context)
                    vendedor_id = cliente_dados.get('vendedor_id') and \
                                  cliente_dados['vendedor_id'] or False
                    vendedor_externo_id = cliente_dados.get('vendedor_externo_id') and \
                                          cliente_dados['vendedor_externo_id'] or False
            gerar_xml = self._check_gerar_xml(cr, uid, destinatario_id, None)

        return {
            'destinatario_endereco_id': id_end,
            'gerar_xml': gerar_xml,
            'vendedor_id': vendedor_id,
            'vendedor_externo_id': vendedor_externo_id}

    def fisnfc_identifica_nf_transferencia(self, cr, uid, company_id, natureza_id, destinatario_id,
                                           context=None):
        """
        O metodo identifica se a nota fiscal e de transferencia
        :param cr: cursor no banco de dados
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario
        :type uid: int
        :param company_id: id da tabela res_company, identificador da empresa
        :type company_id: int
        :param natureza_id: id da tabela fiscal_cfop_natureza, identificador da natureza da operacao
        :type natureza_id: int
        :param destinatario_id: id da tabela adm_pessoa, identificador do cliente/fornecedor
        :type destinatario_id: int
        :param context: dados contextuais
        :type context: dict
        :return: Verdadeiro para NF de transferencia, senao, falso
        :rtype: bool
        """
        context = context and context.copy() or {}
        sql = """
            SELECT
                CASE
                    WHEN fc.movimento_tipo = 'T' AND ape.cpf_cnpj != apd.cpf_cnpj 
                        AND left(ape.cpf_cnpj, 10) = left(apd.cpf_cnpj, 10)
                            THEN TRUE
                    ELSE
                        FALSE
                END AS nf_transferencia
            FROM
                fiscal_cfop_natureza fcn
            LEFT JOIN
                fiscal_cfop fc ON fcn.cfop_id = fc.id
            LEFT JOIN
                adm_pessoa apd ON apd.id = {destinatario_id}
            LEFT JOIN
                res_company rc ON rc.id = {company_id}
            LEFT JOIN
                adm_pessoa ape ON ape.id = rc.rescon_pessoa_id
            WHERE
                fcn.id = {natureza_id}
        """.format(company_id=company_id, destinatario_id=destinatario_id,
                   natureza_id=natureza_id)
        cr.execute(sql)
        nf_transferencia = cr.fetchone()
        nf_transferencia = nf_transferencia and nf_transferencia[0] or False
        return nf_transferencia

    def onchange_destinatario(self, cr, uid, ids, tipo, destinatario_id, company_id, numero_nf,
                              serie_nf, data_emissao, nf_propria, natureza_id,
                              destinatario_endereco_id, context=None, di_ids=None):
        context = context and context.copy() or {}
        warning = {}
        vals = self.verifica_endereco_destinatario(
            cr, uid, tipo, destinatario_id, company_id, natureza_id, context)
        vals['observacao_pessoa'] = None
        if not nf_propria:
            if all([destinatario_id, company_id, natureza_id]):
                vals.update({'fisnfc_nf_transferencia': self.fisnfc_identifica_nf_transferencia(
                    cr, uid, company_id, natureza_id, destinatario_id, context)})

            vals.update({'fisnfc_origem_uf_id': False, 'fisnfc_origem_cidade_id': False})
            if serie_nf and numero_nf and data_emissao and destinatario_id:
                context_verifica = context.copy()
                context_verifica.update({'onchange_verifica': True})
                if not context_verifica.get('active_id'):
                    context_verifica['active_id'] = ids and isinstance(ids, list) and ids[0] or ids
                res = self.verifica_duplicacao_nf(
                    cr, uid, tipo=tipo, serie_nf=serie_nf, numero_nf=numero_nf,
                    nf_propria=nf_propria, company_id=company_id, destinatario_id=destinatario_id,
                    data_emissao=data_emissao, context=context_verifica)
                if 'numero_nf' in res and 'serie_nf' in res:
                    warning = {
                        'title': _('Atencao!'),
                        'message': _('Nota fiscal ja lancada para esta data.')
                    }
        else:
            if tipo == 'E':
                # Declaracao de importacao
                brw_natureza = natureza_id and self.pool.get('fiscal.cfop.natureza').browse(
                    cr, uid, natureza_id, context)
                if brw_natureza and brw_natureza.cfop_id.codigo[0] == '3':
                    exportador = '%04d' % self.pool.get('adm.pessoa').read(
                            cr, uid, destinatario_id, ['codigo'], context).get('codigo')
                    for di in self.resolve_2many_commands(cr, uid, 'fisnfc_di_ids', di_ids):
                        di['fisdim_exportador'] = exportador
                        for adicao in self.pool.get('fiscal.declaracao.importacao').resolve_2many_commands(
                            cr, uid, 'fisdim_adicao_ids', di['fisdim_adicao_ids']):
                            adicao['fisdia_fabricante'] = exportador
                    vals['fisnfc_importacao'] = True
                    vals['fisnfc_di_ids'] = di_ids
        brw_cfop = self.pool.get('fiscal.cfop.natureza').browse(cr, uid, natureza_id, context)
        if tipo == 'S':
            if natureza_id:
                # Verifica restricao de clientes somente para natureza onde o CFOP tenha o tipo de
                # movimento "compra/venda"
                if brw_cfop.cfop_id.movimento_tipo == 'C' and destinatario_id:
                    is_cliente = self.verificar_incluir_cli_for(
                        cr, uid, tipo, destinatario_id, company_id, 'incluir', context=context)
                    if is_cliente:
                        pessoa_com_restricao = \
                            self.pool.get('financeiro').finutl_verificar_restricao_cliente(
                                cr, uid, finutl_pessoa_id=destinatario_id, context=context)
                        if pessoa_com_restricao:
                            msg = _(' Cliente possui restricoes de venda.')
                            if 'message' in warning:
                                warning['message'] += msg
                            else:
                                warning = {'title': _('Atencao'), 'message': msg}
                    else:
                        warning['title'] = _('Atencao')
                        nome_pessoa = self.pool.get('adm.pessoa').browse(
                            cr, uid, destinatario_id, context).nome
                        warning['message'] = _('%s nao e um cliente.' % nome_pessoa)
                        vals['destinatario_id'] = False
        if destinatario_id:
            brw_adm_pessoa = self.pool.get('adm.pessoa').browse(cr, uid, destinatario_id, context)
            if tipo == 'S':
                obj_financeiro = self.pool.get('financeiro')
                observacao_cliente = obj_financeiro. \
                    finutl_observacao_cliente(cr, uid, destinatario_id, context)
                vals['observacao_pessoa'] = observacao_cliente.get(destinatario_id)

            if natureza_id and brw_cfop.cfop_id.codigo[0] not in '37':
                if not brw_adm_pessoa.cpf_cnpj:
                    vals['destinatario_id'] = False
                    warning = {
                        'title': _('Atencao'),
                        'message': _('Destinatario nao possui CPF/CNPJ.')
                    }
            if not nf_propria and brw_adm_pessoa.endereco_ids:
                vals.update(
                    {'fisnfc_origem_uf_id': brw_adm_pessoa.endereco_ids[0].uf_id.id,
                     'fisnfc_origem_cidade_id': brw_adm_pessoa.endereco_ids[0].cidade_id.id})

        return {'value': vals, 'warning': warning}

    def dados_onchange_modelo(self, cr, uid, modelo_nf_series_id, nf_propria, context=None):
        context = context and context.copy() or {}
        vals = {}
        if not modelo_nf_series_id:
            return vals
        modelo_nf_serie = self.pool.get('fiscal.modelo.nf.series').read(
            cr, uid, modelo_nf_series_id, ['serie_nf', 'subserie_nf', 'versao_xml',
                                           'tipo_contingencia', 'modelo_nf_id', 'ambiente'],
            context, '_classic_write')
        if isinstance(modelo_nf_serie, list):
            modelo_nf_serie = modelo_nf_serie[0]
        try:
            modelo_id = modelo_nf_serie.get('modelo_nf_id')
            vals['modelo_nf_id'] = modelo_id
        except:
            return {'modelo_nf_id': []}
        vals['serie_nf'] = modelo_nf_serie['serie_nf']
        vals['subserie'] = modelo_nf_serie['subserie_nf']
        dados_modelo_nf = self.pool.get('fiscal.modelo.nf').read(
            cr, uid, modelo_id, ['fismnf_tipo_nfe', 'fismnf_codigo', 'fismnf_tipo'], context)
        vals['fisnfc_modelo_serie'] = dados_modelo_nf.get('fismnf_codigo') + vals.get('serie_nf')
        tipo_nfe = dados_modelo_nf.get('fismnf_tipo_nfe')
        vals['fisnfc_modelo_nf_tipo'] = dados_modelo_nf.get('fismnf_tipo')
        vals['modelo_55'] = False

        # Se nao for nota fiscal eletronica ja retorna os valores
        if not tipo_nfe:
            return vals

        vals['modelo_55'] = True
        vals['finalidade_emissao'] = '1'
        vals['nfe_processo_emissao'] = '0'
        vals['nfe_versao_processo_emissao'] = modelo_nf_serie.get('versao_xml')

        # Status configuracao da NF-E
        status_configuracao = ''
        if modelo_nf_serie['ambiente'] == 1:
            status_configuracao += 'Ambiente: PRODUCAO\n'
        elif modelo_nf_serie['ambiente'] == 2:
            status_configuracao += 'Ambiente: HOMOLOGACAO\n'

        if modelo_nf_serie.get('tipo_contingencia'):
            status_configuracao += '- %s\n' % modelo_nf_serie.get('tipo_contingencia')
            forma_emissao = '2'
        else:
            forma_emissao = '1'
            status_configuracao += 'Servidor: SEFAZ\n'
        validade_certificado = self.fisnfc_get_param(cr, uid, context).get('data_validade')
        if validade_certificado:
            status_configuracao += 'Venc. Certificado: ' + validade_certificado[:10]
        vals['nfe_forma_emissao'] = forma_emissao
        vals['nfe_forma_impressao_danfe'] = '1'
        vals['fisnfc_nfe_status_configuracao_homologacao'] = status_configuracao
        vals['fisnfc_nfe_status_configuracao_producao'] = ''
        if modelo_nf_serie['ambiente'] == 1:
            vals['fisnfc_nfe_status_configuracao_producao'] = status_configuracao
            vals['fisnfc_nfe_status_configuracao_homologacao'] = ''

        return vals

    def onchange_modelo_nf(self, cr, uid, ids, modelo_nf_series_id, tipo, nf_propria, context=None):
        context = context and context.copy() or {}
        vals = self.dados_onchange_modelo(cr, uid, modelo_nf_series_id, nf_propria, context=context)
        if vals.get('fisnfc_modelo_nf_tipo') != 'T':
            vals.update({'fisnfc_origem_uf_id': False, 'fisnfc_origem_cidade_id': False,
                         'fisnfc_destino_uf_id': False, 'fisnfc_destino_cidade_id': False})
        return {'value': vals}

    def onchange_modelo_id(self, cr, uid, ids, modelo_nf_id, nf_propria, context=None):
        context = context and context.copy() or {}
        modelo_nf_series_id = self.pool.get('fiscal.modelo.nf.series').search(cr, uid,
                                              [('modelo_nf_id','=', modelo_nf_id)], context=context)
        vals = self.dados_onchange_modelo(cr, uid, modelo_nf_series_id, nf_propria, context=context)
        tipo = ''
        if modelo_nf_id and not modelo_nf_series_id:
            tipo = self.pool.get('fiscal.modelo.nf').read(
                cr, uid, modelo_nf_id, ['fismnf_tipo'], context).get('fismnf_tipo')
            vals.update({'fisnfc_modelo_nf_tipo': tipo})
        if (vals.get('fisnfc_modelo_nf_tipo') or tipo) != 'T':
            vals.update({'fisnfc_origem_uf_id': False, 'fisnfc_origem_cidade_id': False,
                         'fisnfc_destino_uf_id': False, 'fisnfc_destino_cidade_id': False})
        return {'value': vals}

    def onchange_modelo_nf_terceiro(self, cr, uid, ids, modelo_terceiro, context=None):
        context = context and context.copy() or {}

        if modelo_terceiro:
            obj_fiscal_modelo_nf = self.pool.get('fiscal.modelo.nf')
            brw_fiscal_modelo_nf = obj_fiscal_modelo_nf.browse(cr, uid, modelo_terceiro, context)
            modelo_55 = brw_fiscal_modelo_nf.fismnf_tipo_nfe
            energia_eletrica = False
            agua_canalizada = False

            if brw_fiscal_modelo_nf.fismnf_tipo == 'E':
                energia_eletrica = True
            elif brw_fiscal_modelo_nf.fismnf_tipo == 'A':
                agua_canalizada = True
            modelo_nf_tipo = brw_fiscal_modelo_nf.fismnf_tipo
            res = {'value': {
                'modelo_55': modelo_55,
                'fisnfc_nf_energia_eletrica': energia_eletrica,
                'fisnfc_nf_agua_canalizada': agua_canalizada,
                'fisnfc_modelo_nf_tipo': modelo_nf_tipo,
                'fisnfc_origem_uf_id': False,
                'fisnfc_origem_cidade_id': False,
                'fisnfc_destino_uf_id': False,
                'fisnfc_destino_cidade_id': False
            }}
            if modelo_nf_tipo != 'T':
                res['value']['nfe_chave_cte'] = ''
            else:
                brw_res_company = self.pool.get('res.company').browse(cr, uid, self.pool.get(
                    'adm.empresa.objeto').getEmpresa(cr, uid, 'fiscal.nf.capa', context=context),
                    context)
                brw_pessoa_endereco = brw_res_company.rescon_pessoa_id.endereco_ids
                res['value'].update(
                    {'fisnfc_destino_uf_id': brw_pessoa_endereco[0].uf_id.id,
                     'fisnfc_destino_cidade_id': brw_pessoa_endereco[0].cidade_id.id})
            return res

        return {'value': {'modelo_55': False, 'fisnfc_origem_uf_id': False,
                          'fisnfc_origem_cidade_id': False, 'fisnfc_destino_uf_id': False,
                          'fisnfc_destino_cidade_id': False}}

    @staticmethod
    def get_vr_total_nf(total_itens, seguro, frete, despesas, desconto, icms_st, ipi, outros=0):
        """
        Calcula o valor total da nota de acordo com os paremetros passados.
        
        :param total_itens: valor total dos itens
        :param seguro: valor do seguro
        :param frete: valor do frete
        :param despesas: valor total das despesas
        :param desconto: valor do desconto
        :param icms_st: valor do ICMS/ST
        :param ipi: valor do IPI
        :param outros: outros valores que podem compor o total da nota fiscal
        :type total_itens: float
        :type seguro: float
        :type frete: float
        :type despesas: float
        :type desconto: float
        :type icms_st: float
        :type ipi: float
        :type outros: float
        :return: somatorio dos valores representando o total da nota fiscal
        :rtype: float
        """

        return total_itens + seguro + frete + despesas + ipi + icms_st - desconto + outros

    def _get_nf_resolve_many_commands(self, cr, uid, tabela, campos, elementos, context=None,
                                      load='_classic_write'):
        """Funcao criada para fazer em mais rapido o que faz 'resolve_2many_commands'"""
        context = context and context.copy() or {}
        dados = []
        ids = []
        comandos = {}  # Relacao entre o comando e o id do elemento
        dados_novos = {}
        for i in elementos:
            if not i[2] and i[1]:
                ids.append(i[1])
                comandos[i[1]] = i[0]
            elif i[2]:
                tem_todos_campos = True
                if i[1]:
                    dados_novos[i[1]] = i[2]
                for campo in campos:
                    if campo not in i[2]:
                        tem_todos_campos = False
                        break

                if tem_todos_campos:
                    dados.append(i)
                elif i[1]:
                    ids.append(i[1])
                    comandos[i[1]] = i[0]
        if not ids:
            return dados
        obj = self.pool.get(tabela)
        if ids:
            for i in obj.read(cr, uid, ids, campos, context, load):
                id = i.pop('id')
                i.update(dados_novos.get(id) or {})
                dados.append([comandos.get(id), id, i])

        return dados

    def fisnfc_onchange_movimento_itens(self, cr, user, ids, itens, seguro, frete, despesas,
                                        desconto, icms_st, ipi, context=None, natureza=None,
                                        tipo_frete=None):
        """
        Calcula e atribui o valor total da nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param seguro: valor do seguro
        :param frete: valor do frete
        :param despesas: valor das despesas acessorias
        :param desconto: valor do desconto
        :param icms_st: valor do ICMS/ST
        :param ipi: valor do IPI
        :param context: um dicionario padrao para valores contextuais
        :param natureza: o ID da natureza da operacao
        :param tipo_frete: o tipo do frete: por 'D'entro ou por 'F'ora
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list[list]
        :type seguro: float
        :type frete: float
        :type despesas: float
        :type desconto: float
        :type icms_st: float
        :type ipi: float
        :type context: dict
        :return: valores alterados da nota fiscal
        :rtype: dict
        """
        context = context and context.copy() or {}
        if not itens:
            return {'value': {'vr_total_nf': 0, 'vr_base_icms_substituicao': 0,
                              'vr_icms_substituicao': 0, 'vr_total_produtos': 0,
                              'vr_despesas_acessorias': 0, 'vr_ipi': 0,
                              'fisnfc_valor_icms_desonerado': 0}}
        if not natureza:
            return {'warning': {'title': _('Atencao!'),
                                'message': 'Informe a natureza na capa da nota antes de incluir os itens!'},
                    'value': {'estoque_movimento_itens_ids': []}}

        dict_encargos = {}
        obj_encargo = self.pool.get('fiscal.encargo')
        map(lambda x: dict_encargos.update({x.get('id'): x}), obj_encargo.read(
            cr, user, obj_encargo.search(cr, user, [], context=context), ['sigla'], context))
        vr_icms_substituicao = vr_base_icms_substituicao = vr_despesas_acessorias = 0
        itens_total = ipi = vr_desoneracao = 0
        context['super_read_estoque_movimento_itens'] = True
        importacao = self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, natureza, context).cfop_id.codigo[0] == '3'
        for item in filter(lambda f: f[0] in (0, 1, 4), itens):
            item[2] = item[2] or {}
            item[2].update(adjust_m2o(self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', [item], [
                    'vr_total_item', 'vr_seguro_rateado', 'vr_despesas_rateado',
                    'estmvi_vr_despesas_extras_rateado', 'estmvi_vr_icms_frete_rateado',
                    'vr_desconto_rateado', 'tributacoes_ids', 'vr_frete_rateado'])[0]))
            itens_total += item[2].get('vr_total_item') or 0
            vr_despesas_acessorias += item[2].get('vr_despesas_rateado') or 0
            item[2]['vr_total_item_rateado'] = \
                item[2].get('vr_total_item') + item[2].get('vr_seguro_rateado', 0) + \
                item[2].get('vr_despesas_rateado', 0) + \
                item[2].get('estmvi_vr_despesas_extras_rateado', 0) + \
                item[2].get('estmvi_vr_icms_frete_rateado', 0) - \
                item[2].get('vr_desconto_rateado', 0)
            if tipo_frete == 'D':  # Frete ´por dentro´
                item[2]['vr_total_item_rateado'] += item[2].get('vr_frete_rateado', 0)
            for dados_trib in self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', item[2].get('tributacoes_ids'),
                    ['encargo_id', 'vr_encargo', 'vr_icms_substituicao',
                     'vr_base_icms_substituicao', 'fisnit_vr_icms_desonerado'], context) or []:
                dados_trib = adjust_m2o(dados_trib)
                vr_icms_substituicao += round(dados_trib.get('vr_icms_substituicao'), 2) or 0
                vr_base_icms_substituicao += round(dados_trib.get('vr_base_icms_substituicao'), 2) or 0
                encargo_id = dados_trib.get('encargo_id')
                if dict_encargos.get(encargo_id).get('sigla').upper() == 'IPI':
                    ipi += dados_trib.get('vr_encargo') or 0
                    if importacao:
                        item[2]['vr_total_item_rateado'] += dados_trib.get('vr_encargo')
                elif dict_encargos.get(encargo_id).get('sigla').upper() == 'ICMS':
                    vr_desoneracao += dados_trib.get('fisnit_vr_icms_desonerado') or 0
                    if importacao:
                        item[2]['vr_total_item_rateado'] += dados_trib.get('vr_encargo')
                elif dict_encargos.get(encargo_id).get('sigla').upper() == 'II':
                    if importacao:
                        item[2]['vr_total_item_rateado'] += dados_trib.get('vr_encargo')

        # Acrescenta desoneracao de ICMS (pela natureza da operacao) ao desconto
        if natureza and self.pool.get('fiscal.cfop.natureza').browse(
                cr, user, natureza, context).fisnat_icms_desonerado_motivo == '8':

            # Desconsidera quando a empresa for Simples Nacional
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, user, context=context)
            pessoa_id = self.pool.get('res.company').read(
                cr, user, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                'rescon_pessoa_id')
            if self.pool.get('adm.pessoa').read(
                cr, user, pessoa_id, ['admpes_jur_regime_tributario'], context).get(
                'admpes_jur_regime_tributario') != 'S':
                desconto += vr_desoneracao
            else:
                vr_desoneracao = 0

        # Acrescenta a outros valores o somatorio dos encargos II, IPI e ICMS se for uma nota
        # fiscal de importacao
        outros = 0
        if natureza and self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, natureza, context).cfop_id.codigo[0] == '3':
            ii_ids = self.pool.get('fiscal.encargo').search(cr, user, [('sigla', 'ilike', 'II')])
            ii_id = ii_ids and ii_ids[0]
            icms_ids = self.pool.get('fiscal.encargo').search(cr, user, [('sigla', 'ilike', 'ICMS')])
            icms_id = icms_ids and icms_ids[0]
            for i in self.resolve_2many_commands(
                    cr, user, 'estoque_movimento_itens_ids', itens, ['tributacoes_ids'], context):
                for tributacao in self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                        cr, user, 'tributacoes_ids', i['tributacoes_ids'], [], context):
                    tributacao = adjust_m2o(tributacao)
                    if tributacao['encargo_id'] in (icms_id, ii_id) and tributacao['vr_encargo']:
                        outros += tributacao['vr_encargo']

        # Recupera os valores das tributacoes para a capa da nota fiscal
        vr_total = self.get_vr_total_nf(
            itens_total, seguro, frete, vr_despesas_acessorias, desconto, vr_icms_substituicao,
            ipi, outros)
        vals = {
            'vr_total_nf': vr_total, 'vr_base_icms_substituicao': vr_base_icms_substituicao,
            'vr_icms_substituicao': vr_icms_substituicao, 'vr_total_produtos': itens_total,
            'vr_despesas_acessorias': vr_despesas_acessorias, 'vr_ipi': ipi,
            'fisnfc_valor_icms_desonerado': vr_desoneracao, 'estoque_movimento_itens_ids': itens
        }

        return {'value': vals}

    def fisnfc_onchange_movimento_itens_terceiro(self, cr, user, ids, itens, frete, ipi,
                                                 context=None):
        """
        Ajusta diversos valores da nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param ipi: valor do IPI
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type frete: str
        :type ipi: float
        :type context: dict
        :return: valores alterados da nota fiscal
        :rtype: dict
        """
        context = context and context.copy() or {}
        value = {}
        itens = [[1 if i[0] == 4 else i[0],
                  i[1], dict(i[2] or {}, sequencia=itens.index(i) + 1)] for i in itens]
        # Recupera os valores das tributacoes para a capa da nota fiscal
        value.update({'vr_base_icms_substituicao': self.fisnfc_get_total_base_icms_substituicao(
            cr, user, itens, context)})
        value.update({'vr_icms_substituicao': self.fisnfc_get_total_icms_substituicao(
            cr, user, itens, context)})

        # Recupera o valor das despesas acessorias
        value.update({'vr_despesas_acessorias': self.fisnfc_get_total_despesas_acessorias(
            cr, user, itens, context)})

        # Recupera o valor das despesa extras (complemento)
        value.update({'fisnfc_vr_despesas_extras': self.fisnfc_get_total_despesas_extras(
            cr, user, itens, context)})

        # Recupera o valor dos produtos
        value.update({'vr_total_produtos': self.fisnfc_get_total_produtos(
            cr, user, itens, context)})

        # Recupera o valor dos servicos
        value.update({'vr_total_servicos': self.fisnfc_get_total_servicos(
            cr, user, itens, context)})

        # Recupera o valor do frete
        value.update({'vr_frete': self.fisnfc_get_total_frete(cr, user, itens, context)})

        # Recupera o valor do seguro
        value.update({'vr_seguro': self.fisnfc_get_total_seguro(cr, user, itens, context)})

        # Recupera o valor do desconto
        value.update({'vr_desconto': self.fisnfc_get_total_desconto(cr, user, itens, context)})

        # Ajusta o valor total dos itens rateados
        value.update({'estoque_movimento_itens_ids': self.fisnfc_set_valor_total_item_rateado(
            cr, user, itens, frete, context)})

        # Recupera o valor total da nota fiscal
        itens_total = icms_st_total = 0
        for item_dados in self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', value['estoque_movimento_itens_ids'],
                ['vr_total_item', 'tributacoes_ids'], context):
            itens_total += item_dados.get('vr_total_item') or 0
            tributacao_dados = self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                cr, user, 'tributacoes_ids', item_dados['tributacoes_ids'],
                ['vr_icms_substituicao'], context)
            icms_st_total += sum(map(
                lambda m: m.get('vr_icms_substituicao') or 0, tributacao_dados))
        value['vr_total_nf'] = self.get_vr_total_nf(
            itens_total, value.get('vr_seguro'), value.get('vr_frete'),
            value.get('vr_despesas_acessorias'), value.get('vr_desconto'), icms_st_total, ipi)

        return {'value': value}

    def fisnfc_set_valor_total_item_rateado(self, cr, user, itens, frete, context=None):
        """
        Calcula e atribui a soma dos valores rateados aos itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type frete: str
        :type context: dict
        :return: o valor total dos produtos
        :rtype: float
        """
        context = context and context.copy() or {}
        for item in itens:
            values = self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', [item],
                ['vr_total_item', 'vr_desconto_rateado', 'vr_frete_rateado', 'vr_seguro_rateado',
                 'vr_despesas_rateado'])
            value = values[0] if values and isinstance(values, list) else values
            total_item_rateado = value and \
                value.get('vr_total_item') - (value.get('vr_desconto_rateado') or 0) + \
                ((value.get('vr_frete_rateado') or 0) if frete == 'D' else 0) + \
                (value.get('vr_seguro_rateado') or 0) + (value.get('vr_despesas_rateado') or 0)
            if isinstance(item, list) and item[0] in (0, 1, 4):
                if item[2]:
                    item[2].update({'vr_total_item_rateado': total_item_rateado})
                else:
                    item[2] = {'vr_total_item_rateado': total_item_rateado}

        return itens

    def fisnfc_get_total_produtos(self, cr, user, itens, context=None):
        """
        Calcula o valor total dos produtos (itens).

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total dos produtos
        :rtype: float
        """
        context = context and context.copy() or {}
        itens = self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', itens, ['vr_total_item', 'produto_id'])
        produto_ids = []
        for i in itens:
            produto_id = i.get('produto_id')
            if not produto_id:
                continue
            if isinstance(produto_id, (list, tuple)):
                produto_id = produto_id[0]
            produto_ids.append(produto_id)
        dict_produtos = {}
        for i in self.pool.get('estoque.produto').read(cr, user, produto_ids, ['tipo'], context):
            dict_produtos[i.get('id')] = i
        total = 0
        for i in itens:
            produto_id = i.get('produto_id')
            if isinstance(produto_id, (list, tuple)):
                produto_id = produto_id[0]
            produto = dict_produtos.get(produto_id)
            if produto.get('tipo').upper() == 'P':
                total += i.get('vr_total_item') or 0

        return round(total, 2)

    def fisnfc_get_total_servicos(self, cr, user, itens, context=None):
        """
        Calcula o valor total dos servicos (itens).

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total dos servicos
        :rtype: float
        """
        context = context and context.copy() or {}
        itens = self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', itens, ['vr_total_item', 'produto_id'])
        produto_ids = []
        for i in itens:
            produto_id = i.get('produto_id')
            if not produto_id:
                continue
            if isinstance(produto_id, (list, tuple)):
                produto_id = produto_id[0]
            produto_ids.append(produto_id)
        dict_produtos = {}
        for i in self.pool.get('estoque.produto').read(cr, user, produto_ids, ['tipo'], context):
            dict_produtos[i.get('id')] = i
        total = 0
        for i in itens:
            produto_id = i.get('produto_id')
            if isinstance(produto_id, (list, tuple)):
                produto_id = produto_id[0]
            produto = dict_produtos.get(produto_id)
            if produto.get('tipo').upper() == 'S':
                total += i.get('vr_total_item') or 0

        return round(total, 2)

    def fisnfc_get_total_despesas_extras(self, cr, user, itens, context=None):
        """
        Calcula o valor total das despesas extras (complemento).

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total das depesas acessorias
        :rtype: float
        """
        context = context and context.copy() or {}

        return round(sum(map(lambda m: m.get('estmvi_vr_despesas_extras_rateado') or 0,
                             self.resolve_2many_commands(
                                 cr, user, 'estoque_movimento_itens_ids', itens,
                                 ['estmvi_vr_despesas_extras_rateado']))), 2)

    def fisnfc_get_total_despesas_acessorias(self, cr, user, itens, context=None):
        """
        Calcula o valor total das despesas acessorias.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total das depesas acessorias
        :rtype: float
        """
        context = context and context.copy() or {}

        return round(sum(map(lambda m: m.get('vr_despesas_rateado') or 0,
                             self.resolve_2many_commands(
                                 cr, user, 'estoque_movimento_itens_ids', itens,
                                 ['vr_despesas_rateado']))), 2)

    def fisnfc_get_total_icms_substituicao(self, cr, user, itens, context=None):
        """
        Calcula o valor total do ICMS/ST.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total do ICMS/ST
        :rtype: float
        """
        context = context and context.copy() or {}
        result = 0
        for item in self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', itens, ['tributacoes_ids']):
            for tributacao in self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', item.get('tributacoes_ids'),
                    ['vr_icms_substituicao']):
                result += tributacao.get('vr_icms_substituicao') or 0.

        return round(result, 2)

    def fisnfc_get_item_icms_st(self, cr, user, itens, context=None):
        """
        Retorna somente os itens que tenham tributacao com ICMS/ST.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: lista com os itens que contem tributacao com ICMS/ST
        :rtype: list
        """
        context = context and context.copy() or {}
        result = []
        for item in itens:
            value = self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', [item], ['tributacoes_ids'])
            if not value:
                continue
            for tributacao in self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', value[0].get('tributacoes_ids'),
                    ['cst_id']):
                cst = tributacao.get('cst_id')
                cst_id = cst[0] if isinstance(cst, (list, tuple)) else cst

                # Item com ICMS/ST
                if self.pool.get('fiscal.cst').read(
                        cr, user, cst_id, ['tipo'], context).get('tipo') in ('U', 'TU'):
                    result.append(item)
                    break

        return result

    def fisnfc_get_total_base_icms_substituicao(self, cr, user, itens, context=None):
        """
        Calcula o valor total da base do ICMS/ST.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total da base do ICMS/ST
        :rtype: float
        """
        context = context and context.copy() or {}
        result = 0
        for item in self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', itens, ['tributacoes_ids']):
            for tributacao in self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', item.get('tributacoes_ids'),
                    ['vr_base_icms_substituicao']):
                result += tributacao.get('vr_base_icms_substituicao') or 0.

        return round(result, 2)

    def fisnfc_onchange_placa_frete(self, cr, uid, ids, placa, context=None):
        """
        Passa as letras da placa para maiusculas
        """
        context = context and context.copy() or {}
        if not placa or placa == placa.upper():
            return {}

        return {'value': {'fisnfc_placa_frete': placa.upper()}}


    def fisnfc_onchange_nfe_chave_cte(self, cr, uid, ids, nfe_chave_cte, context=None):
        """
        O metodo ira inserir dados no campo 'nfe_chave_acesso' de acordo com o que for incluido no
        'nfe_chave_cte', recebido no parametro de mesmo nome.
        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param nfe_chave_cte: char que representa a chave de acesso do conhecimento de transporte
        eletronico.
        :param context: dicionario com valores contextuais.
        :return: dicionario com a chave (campo) 'nfe_chave_acesso' preenchido.
        """
        context = context and context.copy() or {}
        res = {}
        if not nfe_chave_cte:
            return res
        res['value'] = {'nfe_chave_acesso': nfe_chave_cte}

        return res

    def get_nf_item_ids(self, cr, uid, nf_id, context=None):
        context = context and context.copy() or {}
        item_ids = []
        nf_capa_dados = self.read(cr, uid, nf_id, ['estoque_movimento_itens_ids'], context)
        if nf_capa_dados and isinstance(nf_capa_dados, list):
            nf_capa_dados = nf_capa_dados[0]
        if isinstance(nf_capa_dados, dict):
            item_ids = nf_capa_dados.get('estoque_movimento_itens_ids') or []

        return item_ids

    def get_total_nf_itens(self, cr, user, nf, context=None):
        """
        Calcula o valor total de cada item da nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: o valor total do item: produto entre valor unitario e a quantidade.
        :rtype: float
        """
        context = context and context.copy() or {}
        item_ids = self.get_nf_item_ids(cr, user, nf_id=nf, context=context)
        item_dados = self.pool.get('estoque.movimento.itens').read(
            cr, user, item_ids, ['vr_unitario', 'quantidade'], context)

        return round(sum(map(lambda m: m.get('vr_unitario') * m.get('quantidade'), item_dados)), 2)

    # TODO: Mover o metodo para a classe estoque.movimento.itens.
    def get_peso_nf_itens(self, cr, uid, nf_id, context=None):
        context = context and context.copy() or {}
        peso_nf_itens = {}.fromkeys(['peso_liquido', 'peso_bruto'], 0)
        mov_item_ids = self.get_nf_item_ids(cr, uid, nf_id=nf_id, context=context)
        if not mov_item_ids:
            return peso_nf_itens
        mov_item_dados = self.pool.get('estoque.movimento.itens').read(
            cr, uid, mov_item_ids, ['quantidade', 'produto_embalagem_id'], context)
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        peso_bruto_total = 0
        peso_liquido_total = 0
        for i in mov_item_dados:
            quantidade = i.get('quantidade') or 0
            try:
                produto_embalagem_id = i['produto_embalagem_id'][0]
            except:
                produto_embalagem_id = 0
            if produto_embalagem_id:
                embalagem_peso = obj_estoque_produto_embalagem.read(
                    cr, uid, produto_embalagem_id, ['peso_liquido', 'peso_bruto'], context)
                peso_liquido_total += quantidade * embalagem_peso.get('peso_liquido') or 0
                peso_bruto_total += quantidade * embalagem_peso.get('peso_bruto') or 0
        peso_nf_itens['peso_liquido'] = peso_liquido_total
        peso_nf_itens['peso_bruto'] = peso_bruto_total

        return peso_nf_itens

    def onchange_vr_desconto_capa(self, cr, user, ids, itens, desconto, frete, nat_id,
                                  context=None):
        context = context and context.copy() or {}
        avisos = list()
        if desconto and not itens:
            avisos.append(_('* Nao existe item na nota fiscal para ratear o valor do desconto.'))
        vr_total_desconto = self.fisnfc_get_total_desconto(cr, user, itens, context)
        if vr_total_desconto == desconto:
            return {}
        if not itens:
            desconto = 0

        # Nao permite valor de desconto negativo
        if desconto < 0:
            desconto = 0
            avisos.append(_('* Desconto nao permite valor negativo.'))

        if avisos:
            aviso = {'title': _('Aviso.'), 'message': '\n'.join(avisos)}
        else:
            aviso = {}
        itens = self.fisnfc_rateio_desconto(cr, user, itens, desconto, frete, nat_id, context)

        return {'value': {'estoque_movimento_itens_ids': itens, 'vr_desconto': desconto},
                'warning': aviso}

    def onchange_endereco_destinatario(self, cr, uid, ids, company_id, natureza_id,
                                       destinatario_endereco_id, tipo, indicador_presenca=None, context=None):
        context = context and context.copy() or {}
        vals = {}
        if destinatario_endereco_id and natureza_id:
            brw_natureza = self.pool.get('fiscal.cfop.natureza').browse(
                cr, uid, natureza_id, context)
            val_gnre = False
            if brw_natureza.cfop_id:
                if brw_natureza.cfop_id.codigo[:1] != '2':
                    val_gnre = True
            vals['value'] = {'fisnfc_cfop_gnre': val_gnre}
            res = self.verifica_cfop_fora_estado(cr, uid, company_id, natureza_id,
                                           destinatario_endereco_id, tipo,
                                           indicador_presenca=indicador_presenca, context=context)
            if isinstance(res, (str, unicode)):
                vals['warning'] = {'title': _('Atencao!'),
                                   'message': res}

        return vals

    def onchange_natureza(self, cr, uid, ids, natureza_id, company_id, endereco_id, tipo, itens,
                          consumidor_id=False, destinatario_id=False, context=None):
        """
        Verifica natureza da operacao da nota fiscal para sua obrigatoriedade de encargos, servico,
        devolucao, exportacao e complemento de arroz.
        
        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal 
        :param natureza_id: 
        :param company_id: 
        :param tipo: tipo da nota fiscal
        :param endereco_id: o ID do endereco do destinatario da nota fiscal
        :param itens: lista com o ID dos itens da nota fiscal
        :param consumidor: indicativo de nota fiscal de consumidor
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list 
        :type natureza_id: long
        :type company_id: long
        :type tipo: str
        :type endereco_id: long
        :type itens: list
        :type consumidor: bool
        :type context: dict
        :return: valores ajustados na nota fiscal.
        :rtype: dict
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        dados_nf = ids and self.read(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                     ['finalidade_emissao'], context, load) or {}
        vals, aviso = {}, {}
        vals['fisnfc_di_ids'] = [(5, 0, 0)]  # O padrao eh sem declaracao de importacao!
        vals['fisnfc_importacao'] = False
        if natureza_id and endereco_id:
            self.verifica_cfop_fora_estado(
                cr, uid, company_id, natureza_id, endereco_id, tipo, context=context)

        # Verificacao de obrigatoriedade para natureza da operacao
        obj_cfop_natureza_encargos = self.pool.get('fiscal.cfop.natureza.encargos')
        natureza_encargos_ids = obj_cfop_natureza_encargos.search(
            cr, uid, [('natureza_id', '=', natureza_id)], context=context)
        encargo_ids = []
        if natureza_encargos_ids:
            brw_natureza_encargos = obj_cfop_natureza_encargos.browse(
                cr, uid, natureza_encargos_ids, context)
            for enc in brw_natureza_encargos:
                encargo_ids.append(enc.encargo_id.id)
        encargos_faltantes = natureza_id and \
            self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(
                cr, uid, encargo_ids, context) or []
        msg = ''
        for m in encargos_faltantes:
            msg += '* ' + m[1] + '\n'
        if msg:
            raise osv.except_osv(_('Atencao'),
                                 _('Verifique o cadastro de natureza da operacao informada na '
                                   'nota. Os seguintes encargos obrigatorios para esta empresa '
                                   'devem ser inseridos:\n') + msg)

        # Verifica natureza de devolucao e GNRE
        brw_natureza = natureza_id and self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, natureza_id, context)
        val_devolucao = False
        val_gnre = False
        if brw_natureza and brw_natureza.cfop_id:
            if brw_natureza.cfop_id.codigo[:1] != '2':
                val_gnre = True
            if brw_natureza.cfop_id.movimento_tipo == 'D':
                val_devolucao = True
        vals.update({'fisnfc_cfop_gnre': val_gnre, 'fisnfc_nf_devolucao': val_devolucao,
                     'finalidade_emissao': val_devolucao and '4' or '1'})

        # Nota fiscal com natureza de servico
        vals['fisnfc_tipo_servico'] = brw_natureza and brw_natureza.cfop_id.produto_servico_tipo == 'S'

        # Nota fiscal de exportacao
        vals['fisnfc_exportacao'] = brw_natureza and brw_natureza.cfop_id.codigo[0] == '7'

        # Nota fiscal de complemento de arroz (NF-e)
        if brw_natureza and brw_natureza.fisnat_complemento_arroz:
            if tipo == 'E':
                vals['fisnfc_infcomp_fisco'] = _(
                    'Antecipacao do imposto, nos termos do art. 4º da Parte 1 do Anexo XVI do '
                    'RICMS.\n')
                # Nota fiscal de entrada de terceiro nao pode ser complemento de arroz
                if not context.get('terceiro'):  # Entrada propria
                    # Remove todos os itens
                    movimento_itens = [[2, i[1], False] for i in filter(lambda f: f[0], itens)]

                    # Insere o novo item (complemento de arroz)
                    local_id = self.pool.get('adm.utils').admutl_get_parametros(
                        cr, uid, 'estoque', [], context).get('estpar_local_compra_padrao_id')
                    item_id = brw_natureza.fisnat_complemento_arroz_item_id.id
                    local_ids = self.pool.get('estoque.local.locacao').search(
                        cr, uid, [('produto_id', '=', item_id), ('local_id', '=', local_id)],
                        context=context)
                    if not local_ids:
                        local_ids = self.pool.get('estoque.local.locacao').search(cr, uid,
                                                    [('produto_id', '=', item_id)], context=context)
                    local_dados = local_ids and self.pool.get('estoque.local.locacao').read(
                        cr, uid, local_ids[0], ['grade_id', 'estllc_produto_embalagem_id',
                                                'local_id'], context, load) or {}
                    if local_dados and local_dados.get('local_id') != local_id:
                            local_id = local_dados.get('local_id')

                    grade_id = local_dados.get('grade_id') or False
                    produto_embalagem_id = local_dados.get('estllc_produto_embalagem_id') or False
                    destinatario_id = company_id and self.pool.get('res.company').read(
                        cr, uid, company_id, ['rescon_pessoa_id'], context, load).get(
                        'rescon_pessoa_id')
                    campos = ['origem_mercadoria']
                    dados_produto = self.pool.get('estoque.produto').read(
                        cr, uid, item_id, campos, context)
                    obj_pessoa = self.pool.get('adm.pessoa')
                    dados_destinatario = obj_pessoa.read(
                        cr, uid, destinatario_id, ['cpf_cnpj', 'endereco_ids',
                                                   'admpes_jur_tipo_atividade'], context)
                    campos = ['cfop_id', 'natureza_encargos_ids', 'tipo', 'fisnat_icms_desonerado']
                    dados_natureza = self.pool.get('fiscal.cfop.natureza').read(
                        cr, uid, natureza_id, campos, context, load)
                    campos = ['movimento_tipo', 'codigo', 'sigla']
                    dados_cfop_nat = self.pool.get('fiscal.cfop').read(
                        cr, uid, dados_natureza.get('cfop_id'), campos, context)
                    campos = ['encargo_id', 'sobrepor_tributacao_id',
                              'fiscne_sobrepoe_so_tributados']
                    dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(
                        cr, uid, dados_natureza.get('natureza_encargos_ids'), campos, context, load)
                    encargo_ids = map(lambda x: x.get('encargo_id'), dados_nat_encargo)
                    dict_encargos_nat = {}
                    map(lambda x: dict_encargos_nat.update({x.get('id'): x}),
                        self.pool.get('fiscal.encargo').read(
                            cr, uid, encargo_ids, ['sigla'], context))
                    uf_id = False
                    if endereco_id:
                        uf_id = self.pool.get('adm.pessoa.endereco').read(cr, uid, endereco_id,
                                                              ['uf_id'], context, load).get('uf_id')
                    tributacoes = destinatario_id and \
                                  self.pool.get('estoque.movimento.itens').calcular_tributo_itens(
                                       cr, uid, dados_produto, dados_natureza, dados_nat_encargo,
                                       dados_cfop_nat, encargo_ids, dados_destinatario, 'E', 0, [],
                                       uf_id, context)
                    movimento_itens.append([
                        0, False, {
                            'local_id': local_id, 'produto_id': item_id, 'grade_id': grade_id,
                            'produto_embalagem_id': produto_embalagem_id, 'quantidade': 1,
                            'cfop_id': brw_natureza.cfop_id.id, 'tributacoes_ids': tributacoes,
                            'vr_desconto_rateado': 0, 'vr_frete_rateado': 0, 'vr_seguro_rateado': 0,
                            'estmvi_permissao_altera_preco': True, 'vr_despesas_rateado': 0,
                            'vr_unitario': 0, 'vr_desconto': 0, 'vr_total_item': 0,
                            'natureza_id': natureza_id}])
                    vals.update({'destinatario_id': destinatario_id, 'finalidade_emissao': '3',
                                 'fisnfc_nfe_indicador_presenca': 0,
                                 'estoque_movimento_itens_ids': movimento_itens})
                else:
                    aviso = {'title': _('Aviso.'),
                             'message': _('Nota fiscal de entrada de terceiro nao pode ser '
                                          'complemento de arroz.')}
                    vals['natureza_id'] = False
            else:
                aviso = {'title': _('Aviso.'),
                         'message': _('Nota fiscal de saida nao pode ser complemento de arroz.')}
                vals['natureza_id'] = False
        else:
            # Atualiza a natureza nos itens da nota
            fields2read = ['vr_total_item', 'vr_despesas_rateado', 'estmvi_incide_despesa_bc',
                           'vr_frete_rateado', 'vr_seguro_rateado', 'vr_desconto_rateado',
                           'tributacoes_ids', 'natureza_id']
            itens = map(lambda m: [m.get('id') and 1 or 0, m.get('id') or False, adjust_m2o(m)],
                        self.resolve_2many_commands(
                            cr, uid, 'estoque_movimento_itens_ids', itens, fields2read))
            for item in itens:
                item[2]['natureza_id'] = natureza_id
                if dados_nf.get('finalidade_emissao') == '2':
                    continue
                item[2]['tributacoes_ids'] = \
                    self.pool.get('estoque.movimento.itens')._estmvi_ajusta_tributacao_propria(
                        cr, uid, item[2].get('vr_total_item'), item[2].get('vr_despesas_rateado'),
                        item[2].get('estmvi_incide_despesa_bc'), item[2].get('vr_frete_rateado'),
                        item[2].get('vr_seguro_rateado'), item[2].get('vr_desconto_rateado'),
                        item[2].get('tributacoes_ids'), context, natureza_id)

            # Atualiza os itens da nota
            vals.update(estoque_movimento_itens_ids=itens)

        # Desoneracao de ICMS (pela natureza da operacao). Desconsidera quando a empresa for
        # Simples Nacional
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
        pessoa_id = self.pool.get('res.company').read(
            cr, uid, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
            'rescon_pessoa_id')
        simples = self.pool.get('adm.pessoa').read(
            cr, uid, pessoa_id, ['admpes_jur_regime_tributario'], context).get(
            'admpes_jur_regime_tributario') == 'S'
        vals['fisnfc_icms_desonerado'] = \
            natureza_id and brw_natureza.fisnat_icms_desonerado_motivo == '8' and not simples

        if all([endereco_id, company_id, natureza_id, context.get('terceiro')]):
            destinatario_id = self.pool.get('adm.pessoa.endereco').read(
                cr, uid, endereco_id, ['pessoa_id'], context, '_classic_write').get('pessoa_id')
            vals.update({'fisnfc_nf_transferencia': self.fisnfc_identifica_nf_transferencia(
                cr, uid, company_id, natureza_id, destinatario_id, context)})

        # Declaracao de importacao
        if tipo == 'E' and not context.get('terceiro'):  # Entrada propria
            if brw_natureza and brw_natureza.cfop_id.codigo[0] == '3':
                # Alerta se o encargo II nao estiver cadastrado
                if not self.pool.get('fiscal.encargo').search(
                        cr, uid, [('sigla', 'ilike', 'II')], count=True):
                    aviso = {'title': _('Aviso.'),
                             'message': _('Encargo II nao esta cadastrado.')}
                    vals['natureza_id'] = False
                    vals['fisnfc_importacao'] = False
                else:
                    exportador = False
                    if destinatario_id:
                        exportador = '%04d' % self.pool.get('adm.pessoa').read(
                            cr, uid, destinatario_id, ['codigo'], context).get('codigo')
                    vals['fisnfc_di_ids'] += [(0, 0, {
                        'fisdim_exportador': exportador,
                        'fisdim_vr_despesa': 0,
                        'fisdim_vr_afrmm': 0,
                        'fisdim_adicao_ids': [(5, 0, 0), (0, 0, {
                            'sequence': 1,
                            'fisdia_vr_desconto': 0,
                            'fisdia_fabricante': exportador})]})]
                    vals['fisnfc_importacao'] = True

        return {'value': vals, 'warning': aviso}

    def _fisnfc_set_item_desoneracao(self, cr, user, itens, natureza, destinatario, tipo,
                                     context=None):
        """
        Ajusta o valor da desoneracao do ICMS para itens tributados em funcao da natureza da
        operacao.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens da nota (one2many)
        :param natureza: o ID da natureza da operacao
        :param destinatario: o ID do destinatario
        :param tipo: tipo da nota fiscal (entrada ou saida)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type itens: list
        :type natureza: long
        :type destinatario: long
        :type tipo: str
        :type context: dict
        :return: lista com valores ajustados para itens tributados com desoneracao de ICMS.
        :rtype: list
        """
        campos = ['fisnat_icms_desonerado', 'tipo', 'natureza_encargos_ids', 'cfop_id']
        dados_natureza = natureza and self.pool.get('fiscal.cfop.natureza').read(
            cr, user, natureza, campos, context, '_classic_write') or {}
        campos = ['encargo_id', 'sobrepor_tributacao_id', 'fiscne_sobrepoe_so_tributados']
        dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(
            cr, user, dados_natureza.get('natureza_encargos_ids'), campos, context,
            '_classic_write') or {}
        encargo_ids = map(lambda m: m.get('encargo_id'), dados_nat_encargo)
        campos = ['movimento_tipo', 'codigo', 'sigla']
        dados_cfop_nat = self.pool.get('fiscal.cfop').read(
            cr, user, dados_natureza.get('cfop_id'), campos, context) or {}
        campos = ['cpf_cnpj', 'endereco_ids', 'admpes_jur_tipo_atividade']
        dados_destinatario = destinatario and self.pool.get('adm.pessoa').read(
            cr, user, destinatario, campos, context) or {}
        for item in itens:
            if not item[0]:
                produto_id = item[2].get('produto_id')
                vr_total_item = item[2].get('vr_total_item')
            else:
                dados_item = self.pool.get('estoque.movimento.itens').read(
                    cr, user, item[1], ['produto_id', 'vr_total_item'], context, '_classic_write')
                produto_id = dados_item.get('produto_id')
                vr_total_item = dados_item.get('vr_total_item')
            dados_produto = self.pool.get('estoque.produto').read(
                cr, user, produto_id, ['origem_mercadoria'], context)
            item[0] = 1 if item[0] else item[0]
            item[2]['natureza_id'] = natureza
            trib_del = [[2, t[1], False] for t in item[2].get('tributacoes_ids', [])]
            tributacoes = self.pool.get('estoque.movimento.itens').calcular_tributo_itens(
                cr, user, dados_produto, dados_natureza, dados_nat_encargo, dados_cfop_nat,
                encargo_ids, dados_destinatario, tipo == 'S' and 'T' or 'E', vr_total_item,
                context=context)
            item[2].update(tributacoes_ids=trib_del + tributacoes)

        return itens

    def onchange_natureza_frete(self, cr, uid, ids, natureza, empresa, endereco, tipo,
                                context=None):
        context = context and context.copy() or {}
        vals = {}
        if not natureza:
            return vals
        if endereco:
            self.verifica_cfop_fora_estado(cr, uid, empresa, natureza, endereco, tipo,
                                           context=context)

        # Verificacao de obrigatoriedade para natureza da operacao
        obj_cfop_natureza_encargos = self.pool.get('fiscal.cfop.natureza.encargos')
        natureza_encargos_ids = obj_cfop_natureza_encargos.search(
            cr, uid, [('natureza_id', '=', natureza)], context=context)
        encargo_ids = []
        if natureza_encargos_ids:
            brw_natureza_encargos = obj_cfop_natureza_encargos.browse(
                cr, uid, natureza_encargos_ids, context)
            for enc in brw_natureza_encargos:
                encargo_ids.append(enc.encargo_id.id)

        encargos_faltantes = \
            self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(
                cr, uid, encargo_ids, context)
        msg = ''
        for m in encargos_faltantes:
            msg += '* ' + m[1] + '\n'
        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('Verifique o cadastro de natureza da operacao informada na '
                                   'nota. Os seguintes encargos obrigatorios para esta empresa '
                                   'devem ser inseridos:\n') + msg)
        brw_natureza = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid, natureza, context)
        val_devolucao = False
        val_gnre = False
        if brw_natureza.cfop_id:
            if brw_natureza.cfop_id.codigo[:1] != '2':
                val_gnre = True
            if brw_natureza.cfop_id.movimento_tipo == 'D':
                val_devolucao = True
        vals['fisnfc_cfop_gnre'] = val_gnre
        vals['fisnfc_nf_devolucao'] = val_devolucao
        vals['finalidade_emissao'] = '1'
        if val_devolucao:
            vals['finalidade_emissao'] = '4'
        vals['fisnfc_tipo_servico'] = False
        if brw_natureza.cfop_id.produto_servico_tipo == 'S':
            vals['fisnfc_tipo_servico'] = True

        # Nota fiscal de exportacao
        vals['fisnfc_exportacao'] = brw_natureza.cfop_id.codigo[0] == '7'

        return {'value': vals}

    def calcula_frete(self, cr, uid, tributacao_id, vr_frete, context=None):
        context = context and context.copy() or {}
        vals = {}.fromkeys(['vr_base_icms_frete', 'vr_aliquota_frete', 'vr_icms_frete'], 0)
        if not tributacao_id or not vr_frete:
            return vals
        obj_fiscal_tributacao = self.pool.get('fiscal.tributacao')
        base = obj_fiscal_tributacao.browse(cr, uid, tributacao_id, context).base
        vals['vr_base_icms_frete'] = vr_frete * base / 100  # base integral ou reduzida
        vals['vr_aliquota_frete'] = obj_fiscal_tributacao.browse(
            cr, uid, tributacao_id, context).aliquota
        vals['vr_icms_frete'] = vr_frete * vals['vr_aliquota_frete'] / 100

        return vals

    def onchange_tributacao(self, cr, uid, ids, tributacao_id, vr_frete):
        vals = {}.fromkeys(['vr_base_icms_frete', 'vr_aliquota_frete', 'vr_icms_frete'], 0)
        if tributacao_id:
            if vr_frete:
                vals = self.calcula_frete(cr, uid, tributacao_id, vr_frete)

        return {'value': vals}

    @log_debug
    def verifica_icms_substituicao(self, cr, uid, itens_ids, vr_icms_substituicao, itens_novos=None,
                                   context=None):
        """
        Realiza a verificacao dos itens com substituicao tributaria a partir do valor do ICMS ST
        destacado. (Somente nota fiscal de terceiros)

        :param itens_ids: lista com ids dos itens
        :param vr_icms_substituicao: valor do ICMS ST destacado na capa da NF
        :param itens_novos: lista com ID dos novos itens
        :type itens_ids: list[long]
        :type vr_icms_substituicao: float
        :type itens_novos: list[long]
        :return: DicionArio com chaves opcao_confirma e erro.
        :rtype: dict
        """

        context = context and context.copy() or {}
        # todo: refazer este metodo tirando os browses
        obj_estoque_movimento_itens = self.pool.get('estoque.movimento.itens')
        brw_est_mov_itens = obj_estoque_movimento_itens.browse(cr, uid, itens_ids, context)
        icms_st_venda_item = 0
        valor_base_icms = 0
        vals = {'erro': False, 'context': context, 'informacao_mva_pauta': []}
        msg = ''
        ids_itens = []
        itens_tributacao = []
        item_mva_pauta = []
        total_icms_st = 0
        itens_novos = [] if not itens_novos else itens_novos
        tributos_novos = filter(
            lambda i: (i[0] == 1) and (i[2].get('tributacoes_ids')), itens_novos)
        modbcst_sem_mva = False
        for a in brw_est_mov_itens:
            for x in a.tributacoes_ids:
                if (x.percentual_mva and x.aliquota_venda) or x.vr_icms_substituicao:
                    item_mva_pauta.append(a.id)
                if x.modbcst != '4':
                    modbcst_sem_mva = True
                tributo = filter(lambda i: (i[0] == 1) and (i[2].get('tributacoes_ids')) and
                                           (i[1] == a.id), tributos_novos)
                vr_icms = x.vr_icms_substituicao or 0
                vr_base_icms = x.vr_base_icms_substituicao or 0
                if tributo:
                    tributacoes_ids = tributo[0][2]['tributacoes_ids']
                    if tributacoes_ids[0][0] == 1 and tributacoes_ids[0][1] == x.id:
                        vr_icms = tributacoes_ids[0][2].get('vr_icms_substituicao') or 0
                total_icms_st += vr_icms
                valor_base_icms += vr_base_icms
        total_icms_itens = total_icms_st
        total_base_icms_itens = valor_base_icms
        vals['necessita_rateio'] = False
        if round(total_icms_st, 2) != round(vr_icms_substituicao, 2):
            vals['necessita_rateio'] = True
            total_icms_st = 0
            for n in brw_est_mov_itens:
                # Filtra todas as tributacoes (fiscal.nf.item.tributacao) do item com ICMS/ST
                brw_tributacao_itens_compra = filter(
                    lambda i: (i.encargo_id.sigla == 'ICMS') and
                              (i.cst_id.tipo in ('U', 'TU')) and
                              (i.cst_id.codigo in ('10', '30', '60', '70')),
                    n.tributacoes_ids)

                # Filtra todas as tributacoes do produto (fiscal.tributacao.encargos) ICMS/ST, MVA
                # e aliquota de venda.
                brw_tributacao_produto_venda = filter(
                    lambda i: (i.encargo_id.sigla == 'ICMS') and (i.tipo_tributacao == 'T') and
                              (i.cst_id.codigo in ('10', '30', '60', '70')) and (i.mva > 0) and
                              (i.fisten_aliquota_venda > 0), n.produto_id.tributacao_ids)

                if brw_tributacao_itens_compra:
                    vals['contem_icms_substituicao'] = True
                    icms_st_compra_item = \
                        brw_tributacao_itens_compra[0].vr_base_calculo * \
                        (brw_tributacao_itens_compra[0].aliquota / 100)
                    if not brw_tributacao_produto_venda:
                        if not brw_tributacao_itens_compra[0].percentual_mva > 0 or \
                                not brw_tributacao_itens_compra[0].aliquota_venda > 0:
                            if n.produto_id.id not in ids_itens and \
                                            brw_tributacao_itens_compra[0].modbcst == '4':
                                msg += _('* Informe o percentual MVA e aliquota para venda na '
                                         'tributacao do item - ') + n.produto_id.codigo + ' - ' + \
                                       n.produto_id.name + '\n'
                        else:
                            icms_st_venda_item = \
                                (brw_tributacao_itens_compra[0].vr_base_calculo +
                                 (brw_tributacao_itens_compra[0].vr_base_calculo *
                                  brw_tributacao_itens_compra[0].percentual_mva / 100)) * \
                                (brw_tributacao_itens_compra[0].aliquota_venda / 100)
                    else:
                        icms_st_venda_item = \
                            (brw_tributacao_itens_compra[0].vr_base_calculo +
                             (brw_tributacao_itens_compra[0].vr_base_calculo *
                              brw_tributacao_produto_venda[0].mva / 100)) * \
                            (brw_tributacao_produto_venda[0].fisten_aliquota_venda / 100)
                    ids_itens.append(n.produto_id.id)
                    total_icms_st_item = \
                        round(icms_st_venda_item, 2) - round(icms_st_compra_item, 2)
                    total_icms_st_item = \
                        total_icms_st_item * -1 if total_icms_st_item < 0 else total_icms_st_item
                    id_item_tributacao = brw_tributacao_itens_compra[0].id
                    itens_tributacao.append(
                        (id_item_tributacao, {'vr_icms_substituicao': total_icms_st_item}))
                    if total_icms_st_item < 0: total_icms_st_item *= -1
                    total_icms_st += total_icms_st_item
                    valor_base_icms += \
                        brw_tributacao_itens_compra[0].vr_base_calculo + \
                        (brw_tributacao_itens_compra[0].vr_base_calculo *
                         brw_tributacao_itens_compra[0].percentual_mva / 100)
            if msg:
                raise osv.except_osv(_('Atencao'), _(msg))
            else:
                if modbcst_sem_mva:
                    total_icms_st = total_icms_itens
                    valor_base_icms = total_base_icms_itens
                vals['total_icms_st'] = round(total_icms_st, 2)
                vals['total_base_icms_st'] = round(valor_base_icms, 2)
            if not ids_itens:
                vals['erro'] = _("* Nenhum dos itens informados contem tributacao de ICMS ST com "
                                 "CST's 10,30 ou 70. As informacoes da nota devem ser corrigidas "
                                 "para prosseguir.")
                vals['opcao_confirma'] = False
                return vals
        vals['itens_tributacao'] = itens_tributacao
        vals['informacao_mva_pauta'] = item_mva_pauta
        vals['opcao_confirma'] = True
        if msg:
            vals['erro'] = _('*Nao existe aliquota para venda, para o(s) item(ns): \n') + msg
            vals['erro'] += _("\n* Se deseja que o sistema realize o rateio sem informar a "
                              "aliquota de venda e o MVA para o(s) item(ns), clique em "
                              "'Confirmar'. Mas se desejar que o sistema realize o calculo clique "
                              "em Calcular e sair")

        return vals

    @log_debug
    def calcula_rateio_icms_substituicao_guia(self, cr, uid, total_icms_substituicao_guia,
                                              dict_itens, dict_tribs, context=None):
        """
        Rateio do ICMS da guia.
        """
        context = context and context.copy() or {}
        res = False
        if total_icms_substituicao_guia > 0 and dict_itens:
            obj_adm_utils = self.pool.get('adm.utils')
            lista_valores_itens_tributos = []
            for item in dict_itens.values():
                for t in item.get('tributacoes_ids'):
                    trib = dict_tribs.get(t)
                    if trib.get('tipo_substituicao'):
                        lista_valores_itens_tributos.append(
                            [trib.get('id'), item.get('vr_total_item')])
            valores_rateados_guia = obj_adm_utils.efetuar_rateio(
                total_icms_substituicao_guia, lista_valores_itens_tributos)
            obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
            for v in valores_rateados_guia:
                obj_fiscal_nf_item_tributacao.write(
                    cr, uid, v[0], {'vr_icms_substituicao_guia': v[1]}, context)
            res = True

        return res

    @log_debug
    def calcula_rateio_substituicao(self, cr, uid, vr_base_icms_substituicao, vr_icms_substituicao,
                                    itens_ids, context=None):
        """
        Realiza o cálculo do rateio de itens com substituicao.
        """
        context = context and context.copy() or {}
        res = False
        context.update({'calcula_rateio_substituicao': True})
        if vr_base_icms_substituicao > 0 or vr_icms_substituicao > 0 and itens_ids:
            obj_adm_utils = self.pool.get('adm.utils')
            obj_movimento_itens = self.pool.get('estoque.movimento.itens')
            brw_itens = obj_movimento_itens.browse(cr, uid, itens_ids, context)
            lista_valores_itens_tributos = []
            lista_valores_itens_tributos_base = []
            for item in brw_itens:
                for t in item.tributacoes_ids:
                    if t.tipo_substituicao or t.cst_id.codigo in ('10', '30', '60', '70'):
                        lista_valores_itens_tributos.append([t.id, item.vr_total_item])
                        lista_valores_itens_tributos_base.append([t.id, item.vr_total_item])
            valores_base_icms_substituicao_rateado = obj_adm_utils.efetuar_rateio(
                vr_base_icms_substituicao, lista_valores_itens_tributos_base)
            valores_icms_substituicao_rateado = obj_adm_utils.efetuar_rateio(
                vr_icms_substituicao, lista_valores_itens_tributos)
            obj_fiscal_nf_item_tributacao = self.pool.get('fiscal.nf.item.tributacao')
            if valores_base_icms_substituicao_rateado or valores_icms_substituicao_rateado:
                for v in valores_base_icms_substituicao_rateado:
                    obj_fiscal_nf_item_tributacao.write(
                        cr, uid, v[0], {'vr_base_icms_substituicao': v[1]}, context)
                for v in valores_icms_substituicao_rateado:
                    obj_fiscal_nf_item_tributacao.write(
                        cr, uid, v[0], {'vr_icms_substituicao': v[1]}, context)
                res = True

        return res

    @log_debug
    def calcula_rateio(self, cr, user, desconto, frete, seguro, despesas_acessorias,
                       despesas_extras, icms_frete, nat_id, gravados=None, novos=None,
                       tipo_frete=None, context=None):
        """
        Calculo dos valores de rateio de itens em relacao aos valores da capa.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param desconto: o valor do desconto na capa da nota fiscal
        :param frete: o valor do frete na capa da nota fiscal (aba frete)
        :param seguro: o valor do seguro na capa da nota fiscal
        :param despesas_acessorias: o valor das despesas acessorias na capa da nota fiscal
        :param despesas_extras: o valor das despesas extras na capa da nota fiscal
        :param icms_frete: o valor do icms do frete na capa da nota fiscal
        :param nat_id: id da natureza
        :param gravados: lisda com ID dos itens (movimento) persistidos
        :param novos: lista com ID dos itens (movimento) ainda nao persistidos (formato one2many)
        :param tipo_frete: o tipo do frete: ´D´=por dentro ou ´F´=por fora
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type desconto: float
        :type frete: float
        :type seguro: float
        :type despesas_acessorias: float
        :type despesas_extras: float
        :type icms_frete: float
        :type nat_id: int
        :type gravados: list
        :type novos: list
        :type tipo_frete: str
        :type context: dict
        :return: dados de rateio dos itens calculados
        :rtype: dict
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], ['vr_total_item'])
        o2m = lambda vals: [[1 if val[0] == 4 else val[0], val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]
        itens = o2m(novos or list() + [[4, _id, False] for _id in gravados or list()])
        total_itens = float(sum(map(lambda m: m[2] and m[2].get('vr_total_item') or 0, itens)))
        if not total_itens:
            return itens
        if context.get('calcular_despesa'):
            itens = self.fisnfc_rateio_despesas_acessorias(
                cr, user, itens, despesas_acessorias, tipo_frete, nat_id, context)
        itens = self.fisnfc_rateio_desconto(cr, user, itens, desconto, tipo_frete, nat_id, context)
        itens = self.fisnfc_rateio_seguro(cr, user, itens, seguro, tipo_frete, nat_id, context)
        itens = self.fisnfc_rateio_frete(cr, user, itens, frete, tipo_frete, nat_id, context)
        itens = self.fisnfc_rateio_despesas_extras(cr, user, itens, despesas_extras, context)
        itens = self.fisnfc_rateio_icms_frete(cr, user, itens, icms_frete, context)
        importacao = self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, nat_id, context).cfop_id.codigo[0] == '3'
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            item = adjust_m2o(item[2])
            item['vr_total_item_rateado'] = \
                item.get('vr_total_item') + item.get('vr_seguro_rateado', 0) + \
                item.get('vr_despesas_rateado', 0) + \
                item.get('estmvi_vr_despesas_extras_rateado', 0) + \
                item.get('estmvi_vr_icms_frete_rateado', 0) - item.get('vr_desconto_rateado', 0)
            if tipo_frete == 'D':  # Frete ´por dentro´
                item['vr_total_item_rateado'] += item.get('vr_frete_rateado', 0)
            for dados_trib in self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', item.get('tributacoes_ids'),
                    ['encargo_id', 'vr_encargo', 'vr_icms_substituicao',
                     'vr_base_icms_substituicao', 'fisnit_vr_icms_desonerado'], context) or []:
                dados_trib = adjust_m2o(dados_trib)
                encargo = self.pool.get('fiscal.encargo').read(
                    cr, user, dados_trib.get('encargo_id'), ['sigla'], context).get('sigla')
                if encargo.upper() == 'IPI'and importacao:
                    item['vr_total_item_rateado'] += dados_trib.get('vr_encargo')
                elif encargo.upper() == 'ICMS' and importacao:
                    item['vr_total_item_rateado'] += dados_trib.get('vr_encargo')
                elif encargo.upper() == 'II' and importacao:
                    item['vr_total_item_rateado'] += dados_trib.get('vr_encargo')

        return itens

    def fisnfc_get_param(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_parametro = self.pool.get('fiscal.parametro')
        context.update({'sig_search_sem_filhos': True})
        company_id = self.pool.get('res.company')._company_default_get(
            cr, uid, 'fiscal.nf.capa', context=context)
        parametro_modelo = obj_parametro.search(
            cr, uid, [('company_id', '=', company_id)], context=context)
        if parametro_modelo and len(parametro_modelo) == 1:
            try:
                dados_param = obj_parametro.read(
                    cr, uid, parametro_modelo, ['modelo_nf_series_id', 'endereco_entrega',
                                                'endereco_coleta','tipo_frete', 'data_validade'],
                    context=context)[0]
            except:
                dados_param = {}
        else:
            raise osv.except_osv(_('Atencao'),
                                 _('Nao existe parametro do Fiscal cadastrado para a empresa '
                                   'corrente.'))

        return dados_param

    def fields_view_get(self, cr, user, view_id=None, view_type=False, context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}
        if context.get('pcp_ordem_producao'):
            view_id = self.pool.get('ir.ui.view').search(
                cr, user, [('name', '=', 'fiscal.nf.capa.form')],  context=context)
            if view_id and isinstance(view_id, (list, tuple)):
                view_id = view_id[0]
        res = super(fiscal_nf_capa, self).fields_view_get(
            cr, user, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,
            submenu=submenu)
        if context.get('nota_fiscal_entrada'):
            if view_type == 'form' or view_type == 'tree':
                if context.get('proprio'):
                    res['fields']['destinatario_id']['string'] = 'Cliente / Fornecedor'
                else:
                    res['fields']['destinatario_id']['string'] = 'Fornecedor'
            if view_type == 'form':
                res['fields']['estoque_movimento_itens_ids']['views']['tree']['fields'][
                    'vr_unitario']['required'] = True
        else:
            if view_type == 'form':
                res['fields']['estoque_movimento_itens_ids']['views']['tree']['fields'][
                    'vr_unitario']['readonly'] = '1'
            if view_type == 'form' or view_type == 'tree':
                res['fields']['destinatario_id']['string'] = 'Cliente'

        return res

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        context = context and context.copy() or {}
        fields = fields if fields else []
        get_destinatario = 'destinatario_id' in fields or len(fields) == 0
        if 'gerar_xml' in fields and not get_destinatario:
            fields.append('destinatario_id')
        res = super(fiscal_nf_capa, self).read(cr, uid, ids, fields, context=context, load=load)
        if 'gerar_xml' in fields or len(fields) == 0:
            for r in res if isinstance(res, list) else [res]:
                r['gerar_xml'] = self._check_gerar_xml(
                    cr, uid, r['destinatario_id'], context=context)
                if not get_destinatario:
                    r.pop('destinatario_id')

        return res

    def ajusta_kits(self, cr, uid, vals, context=None):
        '''Impementado por heranca no kit'''
        pass

    def copy(self, cr, uid, nf_id, default=None, context=None):
        context = context and context.copy() or {}
        default = default and default.copy() or {}
        brw_nf = self.browse(cr, uid, nf_id, context)
        agrupada_ids = map(lambda m: m.id, brw_nf.tributacao_agrupar_ids)  # Ajuste da pageview
        if brw_nf.state in 'IC':
            raise osv.except_osv(_('Atencao'),
                                 _('Nao e permitido duplicar uma nota fiscal %s.') % {
                                     'I': 'inutilizada', 'C': 'cancelada'}.get(brw_nf.state))
        context['nf_capa_manutencao'] = True
        default.update({'numero_nf': 0, 'fisnfc_pedido_ids': [], 'fisnfc_titulos_ids': [],
                        'state': '1', 'fisnfc_nf_referenciada_ids': [[6, False, []]],
                        'fisnfc_cupons_referenciados_ids': [], 'fisnfc_nf_frete_ids': [],
                        'fisnfc_frete_tributacoes_ids': [], 'tributacao_agrupar_ids': [],
                        'message_ids': []})
        brw_nf.nf_propria and default.update({'data_emissao': time.strftime('%Y-%m-%d %H:%M:%S')})

        res_id = super(fiscal_nf_capa, self).copy(cr, uid, nf_id, default, context)

        # Ajuste do movimento em funcao da capa do movimento
        mov_capa_ids = self.pool.get('estoque.movimento.capa').search(cr, uid, [
            ('nf_capa_id', '=', res_id)])
        mov_capa_id = mov_capa_ids and mov_capa_ids[0]
        movimento_item_ids = self.pool.get('estoque.movimento.itens').search(cr, uid, [
            ('nf_id', '=', res_id)])
        self.pool.get('estoque.movimento.itens').write(cr, uid, movimento_item_ids, {
            'capa_id': mov_capa_id
        })

        # Ajuste da pageview (tributacao_agrupar_ids)
        agrupada_ids = self.pool.get('fiscal.nf.item.tributacao').search(
            cr, uid, [('fisnit_capa_id', '=', nf_id), ('id', 'not in', agrupada_ids)],
            context=context) or []
        if agrupada_ids:
            self.pool.get('fiscal.nf.item.tributacao').write(
                cr, uid, agrupada_ids, {'fisnit_capa_id': res_id}, context)

        return res_id

    def message_post(self, cr, uid, thread_id, body='', subject=None, type='notification',
                     subtype=None, parent_id=False, attachments=None, context=None,
                     content_subtype='html', **kwargs):
        context = context and context.copy() or {}

        return super(fiscal_nf_capa, self).message_post(
            cr, uid, thread_id, body=body, subject=subject, type=type, subtype=subtype,
            parent_id=parent_id, attachments=attachments, context=context,
            content_subtype=content_subtype, **kwargs)

    def _verifica_encargo_trib(self, sigla, encargo_id, nat_encargo_ids, context=None):
        context = context and context.copy() or {}
        if encargo_id in nat_encargo_ids or context.get('terceiro') or \
              context.get('importacao_nfe') or context.get('buscar_tributacao_devolucao'):
            return
        msg = _('Nao e permitido incluir o encargo %s para a natureza de '
                'operacao informada.' % sigla)
        if context.get('fisnfc_nf_tranferencia'):
            msg = _('A natureza de entrada de tranferencia nao possui o '
                    'encargo:%s' % sigla)
        raise osv.except_osv(_('Atencao!'), msg)

    def _set_dados_trib(self, cr, uid, vals, context=None):
        """Funcao para otimizar a nota. Este trecho foi retirado do create de 
           'fiscal.nf.item.tributacao' para ser feito em massa e nao uma a uma tributacao"""
        context = context and context.copy() or {}
        nat_encargo_ids = self.verificar_encargos_natureza(
            cr, uid, vals.get('natureza_id'), context=context)
        if not vals.get('estoque_movimento_itens_ids') or context.get('importacao_nfe'):
            return
        produto_ids = []
        encargo_ids = []
        for i in vals.get('estoque_movimento_itens_ids'):
            item = i[2]
            item.get('produto_id') and produto_ids.append(item.get('produto_id'))
            if not item.get('tributacoes_ids'):
                continue
            for j in item.get('tributacoes_ids'):
                trib = j[2]
                trib.get('encargo_id') and encargo_ids.append(trib.get('encargo_id'))
        if not produto_ids or not encargo_ids:
            return
        dict_produtos = {}
        campos = ['estpro_pis_nat_rec', 'estpro_cofins_nat_rec']
        map(lambda x: dict_produtos.update({x.get('id'): x}),
            self.pool.get('estoque.produto').read(cr, uid, produto_ids, campos, context))
        dict_encargos = {}
        map(lambda x: dict_encargos.update({x.get('id'): x.get('sigla')}),
            self.pool.get('fiscal.encargo').read(cr, uid, encargo_ids, ['sigla'], context=context))
        for i in vals.get('estoque_movimento_itens_ids'):
            item = i[2]
            dados_produto = dict_produtos.get(item.get('produto_id'))
            if not item.get('tributacoes_ids'):
                continue
            for j in item.get('tributacoes_ids'):
                trib = j[2]
                sigla_encargo = (dict_encargos.get(trib.get('encargo_id')) or '').upper()
                self._verifica_encargo_trib(
                    sigla_encargo, trib.get('encargo_id'), nat_encargo_ids, context)
                if sigla_encargo == 'PIS' and not trib.get('fisnit_nat_rec_pis_cofins'):
                    receita_pis = dados_produto.get('estpro_pis_nat_rec')
                    trib['fisnit_nat_rec_pis_cofins'] = receita_pis or ''
                elif sigla_encargo == 'COFINS' and not trib.get('fisnit_nat_rec_pis_cofins'):
                    receita_cofins = dados_produto.get('estpro_cofins_nat_rec')
                    trib['fisnit_nat_rec_pis_cofins'] = receita_cofins or ''

    @log_debug
    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}

        # O contexto nf_capa_manutencao e utilizado para criacao de nota fiscal no contexto
        # de manutencao
        # Este foi realizado desta forma pelo fato desta manutencao nao realizar nehuma
        # validacao na nota fiscal;
        if context.get('nf_capa_manutencao'):
            return super(fiscal_nf_capa, self).create(cr, uid, vals, context=context)

        self.fisnfc_verifica_data_conciliacao_create(
            cr, uid, vals.get('nf_propria') and vals.get('data_emissao') or vals.get(
                'data_saida_entrada'), context)

        obj_mov_itens = self.pool.get('estoque.movimento.itens')
        obj_itens_trib = self.pool.get('fiscal.nf.item.tributacao')
        obj_cfop_natureza = self.pool.get('fiscal.cfop.natureza')
        obj_adm_utils = self.pool.get('adm.utils')
        load = '_classic_write'
        context['super_read_adm_pessoa'] = True
        if vals.get('destinatario_id'):
            obj_adm_pessoa = self.pool.get('adm.pessoa')
            cpf_cnpj = obj_adm_pessoa.read(cr, uid, vals.get('destinatario_id'), ['cpf_cnpj'],
                                           context).get('cpf_cnpj')
            brw_cfop = vals.get('natureza_id') and obj_cfop_natureza.browse(
                cr, uid, vals.get('natureza_id'), context)
            if not brw_cfop:
                raise osv.except_osv(_('Atencao!'),
                                     _('Natureza fiscal nao cadastrada para o CFOP 1353 ou 2353.'))
            if not cpf_cnpj and brw_cfop.cfop_id.codigo[0] not in '37':
                raise osv.except_osv(_('Atencao!'),
                                     _('O destinatario nao possui CPF/CNPJ.'))
        if vals['tipo'] != 'E' and not context.get('nota_referenciada'):
            if vals.get('modelo_nf_series_id'):
                obj_fiscal_modelo_nf_series = self.pool.get('fiscal.modelo.nf.series')
                modelo = obj_fiscal_modelo_nf_series.read(
                    cr, uid, vals['modelo_nf_series_id'], ['serie_nf', 'subserie_nf',
                                                           'modelo_nf_id'], context=context)
                if isinstance(modelo, list):
                    modelo = modelo[0]
                if isinstance(modelo, dict):
                    vals['serie_nf'] = modelo.get('serie_nf')
                    vals['subserie'] = modelo.get('subserie_nf')
                if 'modelo_nf_id' not in vals:
                    if modelo.get('modelo_nf_id'):
                        vals['modelo_nf_id'] = modelo['modelo_nf_id'][0]
        if vals.get('tributacao_id', False) and vals.get('vr_frete', False):
            vals_frete = self.calcula_frete(cr, uid, vals['tributacao_id'], vals['vr_frete'])
            vals['vr_base_icms_frete'] = vals_frete['vr_base_icms_frete']
            vals['vr_aliquota_frete'] = vals_frete['vr_aliquota_frete']
            vals['vr_icms_frete'] = vals_frete['vr_icms_frete']
        if not context.get('inutilizacao'):
            self.verifica_duplicacao_nf(
                cr, uid, vals.get('tipo'), vals.get('serie_nf'), vals.get('nf_propria'),
                vals.get('company_id'), vals.get('destinatario_id'), vals.get('data_emissao') or '',
                vals.get('numero_nf'), vals.get('modelo_nf_series_id'), context)
        if context.get('terceiro'):
            if vals.get('tipo_frete') and vals.get('vr_frete'):
                if vals['tipo_frete'] == 'F' and not vals['vr_frete'] > 0:
                    raise osv.except_osv(_('Atencao'),
                                         _('O valor do frete deve ser maior que zero.'))
        if vals.get('tipo_frete') and vals.get('vr_frete'):
            if vals['tipo_frete'] == 'F' and not vals['vr_frete'] > 0:
                raise osv.except_osv(_('Atencao'),
                                     _('O valor do frete deve ser maior que zero.'))
        if vals.get('company_id') and vals.get('natureza_id') and \
                vals.get('destinatario_endereco_id') and vals.get('tipo'):
            if not context.get('inutilizacao') and not context.get('nota_referenciada'):
                res = self.verifica_cfop_fora_estado(cr, uid, vals.get('company_id'),
                                               vals.get('natureza_id'),
                                               vals.get('destinatario_endereco_id'),
                                               vals.get('tipo'),
                                               indicador_presenca=vals.get('fisnfc_nfe_indicador_presenca'),
                                               context=context)
                if isinstance(res, (str, unicode)):
                    vals['fisnfc_infcomp_fisco'] = (vals.get('fisnfc_infcomp_fisco') or '') + \
                    _('Operacao interna e presencial, conforme artigo 61, paragrafo 7º do RICMS')

        if not context.get('inutilizacao') and not context.get('nota_referenciada') and \
                vals.get('nf_propria'):
            context.update({'calcular_despesa': True})
            if vals.get('estoque_movimento_itens_ids'):
                produto_ids = []
                grade_ids = []
                for i in vals.get('estoque_movimento_itens_ids'):
                    if not isinstance(i[2], dict) or i[2].get('vr_unitario') or i[1]:
                        continue
                    produto_ids.append(i[2].get('produto_id'))
                    grade_ids.append(i[2].get('grade_id'))
                if produto_ids:
                    precos = obj_mov_itens.get_item_preco_venda(
                        cr, uid, produto_ids, grade_id=grade_ids, company_id=vals.get('company_id'),
                        pessoa_id=vals.get('destinatario_id'), context=context)
                    grade_qtd_emb = self.pool.get('estoque.utils').get_grade_qtd_embalagem(
                        cr, grade_ids, context=context)
                for itens in vals.get('estoque_movimento_itens_ids'):
                    if not isinstance(itens[2], dict) or itens[2].get('vr_unitario') or itens[1]:
                        continue
                    preco = precos.get(itens[2].get('produto_id')) or {}
                    produto_valor_unitario = preco.get(vals.get('grade_id')) \
                                             or preco.get('preco_venda') or 0
                    chave = '%s-%s' % (itens[2].get('grade_id'),
                                       itens[2].get('produto_embalagem_id'))
                    qtde_embalagem = grade_qtd_emb.get(chave).get('qtd')
                    produto_valor_unitario *= qtde_embalagem
                    itens[2]['vr_unitario'] = produto_valor_unitario
                vals['estoque_movimento_itens_ids'] = self.calcula_rateio(
                    cr, uid, vals.get('vr_desconto', 0), vals.get('vr_frete', 0),
                    vals.get('vr_seguro', 0), vals.get('vr_despesas_acessorias', 0),
                    vals.get('fisnfc_vr_despesas_extras', 0), vals.get('vr_icms_frete', 0),
                    vals.get('natureza_id'), [],
                    vals.get('estoque_movimento_itens_ids'), vals.get('tipo_frete'),
                    context=context)
        company_id = vals.get('company_id')
        data = str(vals.get('data_saida_entrada') or '')
        if not data:
            data = vals.get('data_emissao')
        if vals.get('nf_propria') and vals.get('tipo') == 'E' and not vals.get(
                'data_saida_entrada'):
            vals['data_saida_entrada'] = vals.get('data_emissao')

        # A serie da nota nao pode ser ´null´, por isto, se nao tiver estou acrescentando
        if not vals.get('serie_nf', False):
            vals['serie_nf'] = ''

        funcionario_id = self.pool.get('rh.funcionario').get_id_func(cr, uid)
        if not funcionario_id:
            raise osv.except_osv(_('Atencao'),
                                 _('Usuario nao esta cadastrado como funcionario.'))
        estmvc_operacao_id = obj_cfop_natureza.read(cr, uid, vals.get('natureza_id'),
                           ['fisnat_operacao_id'], context, load=load).get('fisnat_operacao_id')
        dict_mov_capa = {
            'funcionario_id': funcionario_id,
            'company_id': company_id,
            'data': data,
            'state': 'E',
            'estmvc_operacao_id': estmvc_operacao_id,
            'pessoa_id': vals.get('destinatario_id')}
        mov_capa_id = self.pool.get('estoque.movimento.capa').create(
            cr, uid, dict_mov_capa, context)
        if vals.get('estoque_movimento_itens_ids'):
            for i in vals.get('estoque_movimento_itens_ids'):
                i[2]['capa_id'] = mov_capa_id
            obj_mov_itens._estmvi_create_fiscal(
                cr, uid, vals.get('estoque_movimento_itens_ids'), vals, context)
            vr_icms_substituicao = 0.0
            vr_base_icms_substituicao = 0.0
            for i in vals.get('estoque_movimento_itens_ids'):
                if not i[2].get('tributacoes_ids'):
                    continue
                vr_icms_substituicao += sum(map(lambda t: t[2].get('vr_icms_substituicao') or 0,
                                                i[2].get('tributacoes_ids')))
                vr_base_icms_substituicao += sum(map(lambda t: t[2].get(
                    'vr_base_icms_substituicao') or 0, i[2].get('tributacoes_ids')))
            if not vals.get('vr_icms_substituicao'):
                if vals.get('nf_condicao_ids') and vr_icms_substituicao:
                    elementos_con = map(
                        lambda c: [vals.get('nf_condicao_ids').index(c), c[2].get('valor')],
                        vals.get('nf_condicao_ids'))
                    elementos_con = obj_adm_utils.efetuar_rateio(vr_icms_substituicao,
                                                                 elementos_con)
                    for condicao in vals.get('nf_condicao_ids'):
                        vr_icms_substituicao_cond = elementos_con[vals.get('nf_condicao_ids').index(
                            condicao)][1]
                        condicao[2]['valor'] += vr_icms_substituicao_cond
                        elementos_par = map(
                            lambda c: [condicao[2].get('nf_parcela_ids').index(c),
                                       c[2].get('valor')], condicao[2].get('nf_parcela_ids'))
                        elementos_par = obj_adm_utils.efetuar_rateio(vr_icms_substituicao_cond,
                                                                     elementos_par)
                        for parcela in condicao[2].get('nf_parcela_ids'):
                            vr_icms_substituicao_parc = elementos_par[condicao[2].get(
                                'nf_parcela_ids').index(parcela)][1]
                            parcela = list(parcela)
                            parcela[2]['valor'] += vr_icms_substituicao_parc
                vals['vr_icms_substituicao'] = vr_icms_substituicao
            if not vals.get('vr_base_icms_substituicao'):
                vals['vr_base_icms_substituicao'] = vr_base_icms_substituicao
        if vals.get('fisnfc_nf_devolucao') and vals.get('tipo') == 'S':
            context.update({'buscar_tributacao_devolucao': True})
        self._set_dados_trib(cr, uid, vals, context)
        res = super(fiscal_nf_capa, self).create(cr, uid, vals, context=context)
        campos = ['fisnfc_frete_tributacoes_ids', 'destinatario_id', 'vr_total_nf',
                  'nf_parcelas_ids']
        dados_nf = self.read(cr, uid, res, campos, context, load=load)
        tributacao_ids = dados_nf.get('fisnfc_frete_tributacoes_ids')
        if tributacao_ids:
            # Write para recalculo dos metodos funcionais que nao sao executados quando se passa
            # registro detalhe em 3 niveis no create(Bug do ERP, Felipe)
            context_trib = context.copy()
            context_trib['fisnit_super_write'] = True
            obj_itens_trib.write(cr, uid, tributacao_ids, {}, context_trib)
        data = self.pool.get('adm.utils').admutl_converte_tz(
            cr, uid, datetime.strptime(data[:19], "%Y-%m-%d %H:%M:%S"), tz_origem='UTC',
            tz_destino=context.get('tz'), context=context).date()
        if res:
            dict_mov_capa = {'pessoa_id': dados_nf.get('destinatario_id'), 'nf_capa_id': res}
            self.pool.get('estoque.movimento.capa').write(
                cr, uid, mov_capa_id, dict_mov_capa, context)
        self.pool.get('adm.transacao.capa').admtrc_criar_transacao(cr, uid, 'nota', res, context)

        # Verificacao de parcelas. O total das parcelas nao deve ser maior que o total da nota
        if 'nf_parcelas_ids' in vals:
            tot_nf = dados_nf.get('vr_total_nf')
            total = 0
            p = dados_nf.get('nf_parcelas_ids')
            for id in p:
                vr = self.pool.get('fiscal.nf.parcelas').read(
                    cr, uid, id, ['valor'], context=context)['valor']
                total += vr
            if round(total, 2) > round(tot_nf, 2):
                raise osv.except_osv(_('Atencao'),
                                     _('A soma das parcelas nao deve ser maior que o valor total '
                                       'da nota.'))
        self.atualiza_sequencia_itens(cr, uid, res, context=context)

        return res

    @log_debug
    def write(self, cr, uid, ids, values, context=None):
        context = context and context.copy() or {}
        obj_estoque_movimento_capa = self.pool.get('estoque.movimento.capa')
        # O contexto nf_capa_manutencao e utilizado para criacao de nota fiscal no contexto
        # de manutencao
        # Este foi realizado desta forma pelo fato desta manutencao nao realizar nehuma
        # validacao na nota fiscal;
        if context.get('super_write_fiscal_nf_capa') or context.get('nf_capa_manutencao'):
            self._delete_tributacoes_excluidas(cr, values.get('estoque_movimento_itens_ids'),
                                               context=context)
            res = super(fiscal_nf_capa, self).write(cr, uid, ids, values, context=context)
            if (values.get('data_saida_entrada') or values.get('data_emissao')) and context.get(
                    'nf_capa_manutencao'):
                obj_estoque_movimento_capa.estmvc_atualiza_data(cr, uid, ids, values, context)
            return res

        if values.get('fisnfc_nf_referenciada_ids'):
            context['nf_referenciada'] = True

        context['fintit_matem_prv'] = True
        if values.get('state') == 'C':
            return super(fiscal_nf_capa, self).write(cr, uid, ids, values, context=context)
        nf_capa_dados = self.read(
            cr, uid, ids, ['company_id', 'tipo', 'serie_nf', 'numero_nf', 'nf_propria',
                           'data_emissao', 'modelo_nf_series_id', 'state', 'destinatario_id',
                           'tributacao_id', 'vr_frete', 'vr_despesas_acessorias', 'vr_seguro',
                           'vr_icms_substituicao', 'vr_desconto', 'vr_total_produtos',
                           'vr_total_servicos', 'estoque_movimento_itens_ids', 'tipo_frete', 'frete',
                           'natureza_id', 'destinatario_endereco_id', 'vr_icms_frete',
                           'fisnfc_vr_despesas_extras', 'fisnfc_nf_frete_ids', 'modelo_nf_id',
                           'fisnfc_nf_devolucao', 'fisnfc_despesa_acessoria_capa',
                           'fisnfc_modelo_serie', 'vr_base_icms_substituicao',
                           'fisnfc_origem_uf_id', 'fisnfc_destino_uf_id',
                           'fisnfc_calcula_peso_automaticamente', 'fisnfc_valor_icms_desonerado',
                           'fisnfc_nfe_indicador_presenca', 'fisnfc_infcomp_fisco'],
            context, '_classic_write')
        if isinstance(nf_capa_dados, list):
            nf_capa_dados = nf_capa_dados[0]

        self.fisnfc_verifica_data_conciliacao_write(
            cr, uid, nf_capa_dados.get('nf_propria') and
            (values.get('data_emissao') or nf_capa_dados.get('data_emissao')) or
            (values.get('data_saida_entrada') or nf_capa_dados.get('data_saida_entrada')), context)

        if not context.get('wizard_selecao_nf'):
            if not nf_capa_dados.get('destinatario_endereco_id'):
                nf_capa_dados['destinatario_endereco_id'] = \
                    values.get('destinatario_endereco_id') or False
            self._fisnfc_valida_cfop_item(
                cr, uid, nf_capa_dados, values.get('estoque_movimento_itens_ids'),
                context=context)
        for x in values:
            if x not in ['nfe_chave_acesso', 'nfe_protocolo_cancelamento',
                         'nfe_protocolo', 'fisnfc_infcomp_contribuinte',
                         'fisnfc_infcomp_fisco', 'fisnfc_nfe_retorno_situacao_nfe',
                         'fisnfc_nfe_situacao_nfe_motivo',
                         'fisnfc_nfe_xml_situacao_cliente',
                         'fisnfc_nfe_situacao_motivo', 'fisnfc_nfe_situacao_cliente',
                         'fisnfc_nfe_situacao_regular', 'fisnfc_nfe_xml_correcao',
                         'fisnfc_nfe_protocolo_correcao', 'fisnfc_nfe_status_correcao',
                         'fisnfc_nf_referencia_ids', 'fisnfc_recebimento_conferido',
                         'fisnfc_recebimento_data_hora',
                         'fisnfc_recebimento_usuario_id']:
                if not values.get('state'):
                    if ('nfe_xml_cancelamento' not in values) or \
                            ('nfe_protocolo_cancelamento' not in values):
                        if nf_capa_dados.get('state') in ('3', 'C', 'I') and \
                                not context.get('nf_capa_manutencao'):

                            # Usado para obrigar alteracao em qualquer estado
                            if not (context.get('salvar_forcado')
                                    and context['salvar_forcado']) \
                                    and not context.get('view_name'):
                                if context.get('view_name') == 'contratos_contrato':
                                    raise osv.except_osv(
                                        _('Atencao'),
                                        _('O status da nota fiscal nao permite sua '
                                          'alteracao.'))
        tributacao_id = values.get('tributacao_id') or nf_capa_dados.get('tributacao_id')
        vr_frete = values.get('vr_frete') or nf_capa_dados.get('vr_frete') or 0

        # Tributacao do frete
        if tributacao_id:
            valores = self.calcula_frete(cr, uid, tributacao_id, vr_frete)
            values['vr_base_icms_frete'] = valores['vr_base_icms_frete']
            values['vr_aliquota_frete'] = valores['vr_aliquota_frete']
            values['vr_icms_frete'] = valores['vr_icms_frete']
        else:
            values['vr_base_icms_frete'] = 0
            values['vr_aliquota_frete'] = 0
            values['vr_icms_frete'] = 0

        if 'vr_icms_substituicao' in values:
            context.update({'alterou_icms_st_capa': True})
        vr_icms_substituicao = values.get('vr_icms_substituicao') or \
                               nf_capa_dados.get('vr_icms_substituicao') or 0
        if 'vr_base_icms_substituicao' in values:
            context.update({'alterou_base_icms_st_capa': True})
        tipo = values.get('tipo') or nf_capa_dados.get('tipo')
        serie_nf = values.get('serie_nf') or nf_capa_dados.get('serie_nf')
        numero_nf = values.get('numero_nf') or nf_capa_dados.get('numero_nf') or 0
        nf_propria = values.get('nf_propria') or nf_capa_dados.get('nf_propria') or False
        data_emissao = values.get('data_emissao') or nf_capa_dados.get('data_emissao')
        destinatario_id = values.get('destinatario_id') or nf_capa_dados.get('destinatario_id')
        destinatario_endereco_id = values.get('destinatario_endereco_id') or \
                                   nf_capa_dados.get('destinatario_endereco_id') or False
        modelo_nf_id = values.get('modelo_nf_series_id') or \
                       nf_capa_dados.get('modelo_nf_series_id') or False
        company_id = values.get('company_id') or nf_capa_dados.get('company_id') or False
        natureza_id = values.get('natureza_id') or nf_capa_dados.get('natureza_id') or False
        fisnfc_modelo_serie = values.get('fisnfc_modelo_serie') or \
                              nf_capa_dados.get('fisnfc_modelo_serie')
        tipo_frete = values.get('tipo_frete') or nf_capa_dados.get('tipo_frete')
        if nf_capa_dados.get('frete') != '9' and tipo_frete == 'F' and not vr_frete > 0 \
                and values.get('numero_nf') and \
                not context.get('cancelamento_terceiros') and not context.get('inutilizacao'):
            raise osv.except_osv(_('Atencao!'),
                                 _('O valor do frete deve ser maior que zero.'))
        obj_res_company = self.pool.get('res.company')
        company_pessoa_id = obj_res_company.read(
            cr, uid, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
            'rescon_pessoa_id')
        if destinatario_id:
            obj_adm_pessoa = self.pool.get('adm.pessoa')
            cpf_cnpj = obj_adm_pessoa.read(
                cr, uid, destinatario_id, ['cpf_cnpj'], context).get('cpf_cnpj')
            brw_cfop = natureza_id and self.pool.get('fiscal.cfop.natureza').browse(
                cr, uid, natureza_id, context)
            if (not brw_cfop and not cpf_cnpj) or \
                                    brw_cfop.cfop_id.codigo[0] not in '37' and not cpf_cnpj:
                raise osv.except_osv(_('Atencao!'),
                                     _('O destinatario nao possui CPF/CNPJ.'))
        itens_movimento =[[4, i, False] if not isinstance(i, (list, tuple)) else i for i in
                          values.get('estoque_movimento_itens_ids') or list()]
        if not context.get('inutilizacao') and not context.get('nota_referenciada'):
            if 'fisnfc_despesa_acessoria_capa' in values and nf_propria:
                fisnfc_despesa_acessoria_capa = values['fisnfc_despesa_acessoria_capa']
            else:
                fisnfc_despesa_acessoria_capa = \
                    nf_capa_dados.get('fisnfc_despesa_acessoria_capa')
            context.update({'calcular_despesa': fisnfc_despesa_acessoria_capa})
            if not fisnfc_despesa_acessoria_capa and nf_propria:
                sum_despesas = 0
                ids_ant = []
                for it in itens_movimento:
                    # Pega itens novos inseridos
                    if isinstance(it, (tuple, list)) and not it[1] and it[0] != 2:
                        if isinstance(it[2], dict) and it[2].get('vr_despesas_rateado'):
                            if it[2]['vr_despesas_rateado']:
                                sum_despesas += it[2]['vr_despesas_rateado']
                    if isinstance(it, (tuple, list)) and it[0] == 1:
                        ids_ant.append(it[1])
                        if isinstance(it[2], dict) and it[2].get('vr_despesas_rateado'):
                            sum_despesas += it[2]['vr_despesas_rateado']
                itens_movimento_ids = nf_capa_dados.get('estoque_movimento_itens_ids')
                for x in ids_ant:
                    itens_movimento_ids.remove(x)
                context_item = context.copy()
                context_item['super_read_estoque_movimento_itens'] = True
                dados_itens = self.pool.get('estoque.movimento.itens').read(
                    cr, uid, itens_movimento_ids, ['vr_despesas_rateado'], context_item)
                sum_despesas_ant = sum(map(
                    lambda d: d.get('vr_despesas_rateado') or 0, dados_itens))
                values['vr_despesas_acessorias'] = sum_despesas + sum_despesas_ant
        if any((('serie_nf' in values), ('numero_nf' in values), ('data_emissao' in values),
                ('destinatario_id' in values), ('modelo_series_nf_id' in values),
                ('company_id' in values))):
            if not context.get('nota_referenciada'):
                active_id = ids
                if isinstance(active_id, list):
                    active_id = active_id[0]
                context.update({'active_id': active_id})
                self.verifica_duplicacao_nf(cr, uid, tipo, serie_nf, nf_propria, company_id,
                                            destinatario_id, data_emissao, numero_nf, modelo_nf_id,
                                            context=context)
        if 'state' in values and values.get('state') not in ('1', 'I') and not context.get('inutilizacao'):
            if natureza_id and destinatario_endereco_id:
                if not context.get('nota_referenciada'):
                    if not destinatario_id and fisnfc_modelo_serie == '02D':
                        vals_end = self.verifica_endereco_destinatario(
                            cr, uid, tipo, company_pessoa_id, company_id, context=context)
                        destinatario_endereco_id = vals_end['destinatario_endereco_id']

                    indicador = values.get('fisnfc_nfe_indicador_presenca') or \
                                nf_capa_dados.get('fisnfc_nfe_indicador_presenca')
                    if tipo == 'E' and self.pool.get('fiscal.modelo.nf').read(
                            cr, uid, nf_capa_dados.get('modelo_nf_id'), ['fismnf_codigo'],
                            context).get('fismnf_codigo') == '57':
                        uf_origem_id = values.get('fisnfc_origem_uf_id') or nf_capa_dados.get(
                            'fisnfc_origem_uf_id')
                        uf_destino_id = values.get('fisnfc_destino_uf_id') or nf_capa_dados.get(
                            'fisnfc_destino_uf_id')
                        res = self.verifica_cfop_fora_estado_cte(
                            cr, uid, natureza_id, uf_origem_id, uf_destino_id, context)
                    else:
                        res = self.verifica_cfop_fora_estado(
                            cr, uid, company_id, natureza_id, destinatario_endereco_id, tipo,
                            indicador_presenca=indicador, context=context)
                    if isinstance(res, (str, unicode)):
                        texto = 'interna e presencial, conforme artigo 61,'
                        info = (nf_capa_dados.get('fisnfc_infcomp_fisco') or '') + \
                               (values.get('fisnfc_infcomp_fisco') or '')
                        if texto not in info:
                            values['fisnfc_infcomp_fisco'] = \
                                (values.get('fisnfc_infcomp_fisco') or
                                 nf_capa_dados.get('fisnfc_infcomp_fisco') or '') + '\n' + \
                                _('Operacao interna e presencial, conforme artigo 61, paragrafo 7º do RICMS')

        # Verifica nota de devolucao com ICMS ST
        if itens_movimento and not values.get('state') == '3':
            if tipo == 'E' and nf_propria:
                verifica_substituicao_dados = self.verifica_icms_substituicao(
                    cr, uid, map(lambda m: m[1], filter(lambda f: f[0] in (4, 1), itens_movimento)),
                    vr_icms_substituicao, filter(lambda f: not f[0], itens_movimento), context)
                if not verifica_substituicao_dados.get('erro') \
                        and vr_icms_substituicao <= 0 and not context.get('fatura_pedido'):
                    values['vr_icms_substituicao'] = \
                        verifica_substituicao_dados.get('total_icms_st')
                    values['vr_base_icms_substituicao'] = \
                        verifica_substituicao_dados.get('total_base_icms_st')
                if not nf_propria:
                    if not verifica_substituicao_dados['informacao_mva_pauta'] and \
                                    vr_icms_substituicao > 0 and \
                            not context.get('fisnit_calcular_vr_icms_st_capa') and \
                            context.get('alterou_icms_st_capa'):
                        self.calcula_rateio_substituicao(
                            cr, uid, verifica_substituicao_dados.get('total_base_icms_st'),
                            vr_icms_substituicao, map(lambda m: m[1], filter(
                                lambda f: f[0] in (4, 1), itens_movimento)), context)

        # Atualiza o ID da capa da nota nos itens de movimento
        mov_capa_ids = obj_estoque_movimento_capa.search(
            cr, uid, [('nf_capa_id', 'in' if isinstance(ids, list) else '=', ids)],
            context=context)
        for item in values.get('estoque_movimento_itens_ids') or list():
            if isinstance(item, tuple):
                item = list(item)
            if item[0] in [0, 1]:
                item[2].update({'capa_id': mov_capa_ids and mov_capa_ids[0]})
        self._delete_tributacoes_excluidas(
            cr, values.get('estoque_movimento_itens_ids'), context=context)
        obj_estoque_movimento_capa.estmvc_atualiza_data(cr, uid, ids, values, context)
        res = super(fiscal_nf_capa, self).write(cr, uid, ids, values, context=context)
        if values.get('estoque_movimento_itens_ids'):
            self.atualiza_sequencia_itens(cr, uid, ids, context)

        return res

    @log_debug
    def _delete_tributacoes_excluidas(self, cr, itens, context=None):
        #####
        # ** AVISO IMPORTANTE! **
        # Tributacoes que jah foram persistidas e que estao novamente sendo lancadas, estao sendo
        # criticadas pelo metodo _sql_constraints da classe fiscal_nf_item_tributacao e nao permitem
        # que os novos registros sejam lancados. Em ´values´ devera ser observado o elemento
        # ´tributacao_ids´ que ao trazer valores de sua lista como [[0, False, {...}],
        # [2, <id>, False]...] causam o problema uma vez que o OpenERP persiste primeiro os valores
        # da tributacao e somente depois os remove.
        # Para contornar o problema a solucao eh iterar os valores da lista do elemento
        # ´tributacao_ids´ que contenham o indice 2 (registro a ser excluido) e remove-los antes da
        # chamada final do metodo ´write´
        #                                      ------
        # O codigo pode ser removido se em futura versao do OpenERP a solucao for contemplada
        context = context and context.copy() or {}
        if not itens:
            return
        for item in filter(lambda f: f[2] and 'tributacoes_ids' in f[2], itens):
            _ids = [t[1] for t in filter(lambda f: f[0] == 2, item[2].get('tributacoes_ids'))]
            if _ids:
                cr.execute("""
                -- noinspection SqlDialectInspectionForFile
                DELETE
                FROM   fiscal_nf_item_tributacao
                WHERE  id IN (%s)""" % ', '.join([str(i) for i in _ids]))

    @log_debug
    def _fisnfc_valida_cfop_item(self, cr, uid, nf_capa_dados, estoque_movimento_item_ids,
                                 context=None):
        """
        O metodo reune os dados necessarios da capa da nota e os CFOPs dos itens para chamar o
        metodo 'verifica_cfop_fora_estado', que verifica de acordo com o emitente e destinatario
        da nota, os CFOPs pertinentes para a natureza da operacao.

        :param cr: object no banco de dados
        :param uid: id da tabela res_users, identificador do usuario
        :param nf_capa_dados: dicionario contendo os dados da capa da nota
        :param estoque_movimento_item_ids: lista de listas contendo os dados dos itens da nota
        :return: booleano True, caso o cfop informado nos itens seja compativel com a natureza da
                 operacao, senao, sera gerada excecao que eh exibida ao usuario
        """
        context = context and context.copy() or {}
        if not estoque_movimento_item_ids:
            return True
        nf_company_id = nf_capa_dados.get('company_id')
        if isinstance(nf_company_id, tuple):
            nf_company_id = nf_company_id[0]
        nf_natureza_id = nf_capa_dados.get('natureza_id')
        if isinstance(nf_natureza_id, tuple):
            nf_natureza_id = nf_natureza_id[0]
        nf_destinatario_endereco_id = nf_capa_dados.get('destinatario_endereco_id')
        if isinstance(nf_destinatario_endereco_id, tuple):
            nf_destinatario_endereco_id = nf_destinatario_endereco_id[0]
        nf_tipo = nf_capa_dados.get('tipo')
        list_cfop = []
        for movimento in estoque_movimento_item_ids:
            for x in movimento:
                if isinstance(x, dict):
                    cfop_id = x.get('cfop_id')
                    if cfop_id and not cfop_id in list_cfop:
                        list_cfop.append(cfop_id)
        if list_cfop:
            for cfop_id in list_cfop:
                self.verifica_cfop_fora_estado(
                    cr, uid, company_id=nf_company_id, natureza_id=nf_natureza_id,
                    destinatario_endereco_id=nf_destinatario_endereco_id, tipo=nf_tipo,
                    cfop_id=cfop_id,
                    indicador_presenca=nf_capa_dados.get('fisnfc_nfe_indicador_presenca'),
                    context=context)

        return True

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context and context.copy() or {}
        if context.get('selecao_destinatario_id'):
            tipo = 'E'
            if context.get('selecao_nf_tipo') == 'E':
                tipo = 'S'
            args.extend([('destinatario_id', 'in', (context.get('selecao_destinatario_id'), False)),
                         ('state', '=', '3'), ('tipo', '=', tipo)])
        res = super(fiscal_nf_capa, self).search(
            cr, uid, args, offset, limit, order, context, count)

        return res

    def fisnfc_tratar_exclusao(self, cr, uid, nf_ids, context=None):
        '''
        Funcao para excluir toda a movimentacao gerada pela NF.
        '''
        context = context and context.copy() or {}
        if not isinstance(nf_ids, list):
            nf_ids = [nf_ids]
        context.update({'fisnfc_tratar_exclusao': True})
        for nf_id in nf_ids:
            obj_financeiro_fatura = self.pool.get('financeiro.fatura')
            obj_financeiro_titulo = self.pool.get('financeiro.titulo')
            obj_financeiro_tesouraria_movimento = self.pool.get('financeiro.tesouraria.movimento')
            obj_financeiro_titulo_baixa = self.pool.get('financeiro.titulo.baixa')
            obj_financeiro_titulo_historico = self.pool.get('financeiro.titulo.historico')
            fatura_ids = obj_financeiro_fatura.search(
                cr, uid, [('nf_id', '=', nf_id)], context=context)
            if fatura_ids:
                titulo_ids = obj_financeiro_titulo. \
                    search(cr, uid, [('fatura_id', 'in', fatura_ids)], context=context)

                # Exclui baixa
                titulo_baixa_ids = obj_financeiro_titulo_baixa.search(
                    cr, uid, [('titulo_id', 'in', titulo_ids)],  context=context)
                obj_financeiro_titulo_baixa.unlink(cr, uid, titulo_baixa_ids, context=context)

                # Exclui historico
                titulo_historico_ids = obj_financeiro_titulo_historico. \
                    search(cr, uid, [('titulo_id', 'in', titulo_ids)], context=context)
                obj_financeiro_titulo_historico.unlink(cr, uid, titulo_historico_ids,
                                                       context=context)
                # Exclui movimento tesouraria
                tesouraria_ids = obj_financeiro_tesouraria_movimento. \
                    search(cr, uid, [('fintem_titulo_ids', 'in', titulo_ids)], context=context)
                obj_financeiro_tesouraria_movimento.unlink(cr, uid, tesouraria_ids, context=context)

                # Exclui titulo e fatura
                obj_financeiro_titulo.unlink(cr, uid, titulo_ids, context=context)
                obj_financeiro_fatura.unlink(cr, uid, fatura_ids, context=context)

            # Exclusao de titulos de pedidos
            obj_estoque_pedidos_capa = self.pool.get('estoquepedidos.pedido.capa')
            pedido_ids = obj_estoque_pedidos_capa.search(
                cr, uid, [('pedcap_nf_ids', '=', nf_id)], context=context)

            if pedido_ids:
                brw_pedidos = obj_estoque_pedidos_capa.browse(cr, uid, pedido_ids, context)
                for x in brw_pedidos:
                    titulo_pedido_ids = obj_financeiro_titulo.search(
                        cr, uid, [('documento', '=', x.numero_pedido)], context=context)

                    # Exclui baixa
                    titulo_baixa_ids = obj_financeiro_titulo_baixa.search(
                        cr, uid, [('titulo_id', 'in', titulo_pedido_ids)], context=context)
                    obj_financeiro_titulo_baixa.unlink(cr, uid, titulo_baixa_ids, context=context)

                    # Exclui historico
                    titulo_historico_ids = obj_financeiro_titulo_historico. \
                        search(cr, uid, [('titulo_id', 'in', titulo_pedido_ids)], context=context)
                    obj_financeiro_titulo_historico.unlink(
                        cr, uid, titulo_historico_ids, context=context)

                    # Exclui movimento tesouraria
                    tesouraria_ids = obj_financeiro_tesouraria_movimento. \
                        search(cr, uid, [('titulo_id', 'in', titulo_pedido_ids)], context=context)
                    obj_financeiro_tesouraria_movimento.unlink(
                        cr, uid, tesouraria_ids, context=context)
                    obj_financeiro_titulo.unlink(cr, uid, titulo_pedido_ids, context=context)

        return True

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        # O contexto nf_capa_manutencao e utilizado para criacao de nota fiscal no contexto
        # de manutencao
        # Este foi realizado desta forma pelo fato desta manutencao nao realizar nehuma
        # validacao na nota fiscal;
        if context.get('nf_capa_manutencao'):
            obj_tes_mov = self.pool.get('financeiro.tesouraria.movimento')
            tes_mov_ids = obj_tes_mov.search(cr, uid, [('fintem_nf_id', 'in', ids)],
                                             context=context)
            if tes_mov_ids:
                obj_baixa = self.pool.get('financeiro.titulo.baixa')
                baixa_ids = obj_baixa.search(cr, uid,
                                  [('tesouraria_movimento_id', 'in', tes_mov_ids)], context=context)
                if baixa_ids:
                    obj_baixa.unlink(cr, uid, baixa_ids, context)

            return super(fiscal_nf_capa, self).unlink(cr, uid, ids, context)

        nf = self.read(
            cr, uid, ids, ['nf_propria', 'modelo_55', 'fisnfc_consumidor', 'state', 'modelo_nf_id',
                           'data_emissao', 'data_saida_entrada'],
            context, load='_classic_write')
        for i in nf:
            self.fisnfc_verifica_data_conciliacao_unlink(
                cr, uid, i.get('nf_propria') and i.get('data_emissao') or i.get(
                    'data_saida_entrada'), context)

            cod_model = self.pool.get('fiscal.modelo.nf').read(cr, uid, i.get('modelo_nf_id'),
                                                    ['fismnf_codigo'], context).get('fismnf_codigo')
            if i['nf_propria'] and (not i['modelo_55'] or cod_model != '65') and i['state'] == 'C':
                # Exclui toda a movimentacao
                self.fisnfc_tratar_exclusao(cr, uid, ids, context={})
            elif not context.get('cancelar'):
                if context.get('nf_referenciada'):
                    if not i['fisnfc_consumidor']:
                        self.write(cr, uid, ids, {'fisnfc_nf_referencia_ids': [[6, False, []]]}, context=context)
                        return True
                    else:
                        cr.execute('DELETE FROM fiscal_nf_referencia WHERE nf_ref_id = %s' % i['id'])
                        return True
                raise osv.except_osv(_('Atencao'),
                                     _('A nota fiscal nao pode excluida. Somente cancelada.'))
        res = super(fiscal_nf_capa, self).unlink(cr, uid, ids, context)

        return res

    @log_debug
    def cadastrar_tributacao_produto(self, cr, uid, company_id, produto_id, list_vals,
                                     pessoa_id=False, context=None):
        """
        Verifica se existe tributacao de entrada para o produto informado se nao existir cadastra.
        Tambem verifica se existe tributacao para o fornecedor informado.
        :param cr: 
        :param uid: 
        :param company_id: id da empresa da nota
        :param produto_id: id do produto
        :param list_vals: lista com os valores das tributacoes usadas pelo item da nota
        :param pessoa_id: id da pessoa do fornecedor da nota
        :param context: 
        :type cr: object
        :type uid: int
        :type company_id: int
        :type produto_id: int
        :type list_vals: list
        :type pessoa_id: int
        :return: True
        """
        context = context and context.copy() or {}

        obj_trib_encargos = self.pool.get('fiscal.tributacao.encargos')
        # Se a tributacao de entrada informada no item da nota nao estiver cadastrada no cadastro
        # do produto, cria uma nova tributacao
        for vals in list_vals:
            args_trib_encargo = [('produto_id', '=', vals.get('produto_id')),
                                 ('encargo_id', '=', vals.get('encargo_id')),
                                 ('tipo_tributacao', '=', 'E')]

            if not obj_trib_encargos.search(cr, uid, args_trib_encargo, context=context):
                obj_trib_encargos.create(cr, uid, vals, context)

        # Se nao existir fornecedor entao sera criada um
        fornecedor_id = self.pool.get('financeiro.fornecedor').search(cr, uid,
                                              [('pessoa_id', '=', pessoa_id),
                                               ('company_id', '=', company_id)], context=context)[0]
        tributacao_entrada_ids = self.pool.get('fiscal.tributacao.entrada').search(cr, uid,
                           [('produto_id', '=', produto_id), ('fornecedor_id', '=', fornecedor_id)],
                           context=context)
        if not tributacao_entrada_ids:
            # Cria as tributacoes
            tributacoes = [[
                0, 0, {'encargo_id': v.get('encargo_id'), 'cst_id': v.get('cst_id'),
                       'tributacao_id': v.get('tributacao_id')}] for v in list_vals]
            self.pool.get('fiscal.tributacao.entrada').create(
                cr, uid, {'produto_id': produto_id, 'fornecedor_id': fornecedor_id,
                          'tributacao_entrada_ids': tributacoes}, context)
            return True

        # Ajusta as tributacoes do fornecedor
        tributacoes_anteriores = map(
            lambda m: m.encargo_id.id, self.pool.get('fiscal.tributacao.entrada').browse(
                cr, uid, tributacao_entrada_ids[0], context).tributacao_entrada_ids)
        tributacoes_novas = map(lambda m: m.get('encargo_id'), list_vals)

        # Exclui tributacoes
        tributacoes = [[2, t, False]
                       for t in self.pool.get('fiscal.tributacao.entrada.encargos').search(
                cr, uid, [('tributacao_entrada_id', '=', tributacao_entrada_ids[0]),
                          ('encargo_id', 'in', list(set(tributacoes_anteriores) -
                                                    set(tributacoes_novas)))])]

        # Inclui tributacoes
        tributacoes.extend([0, 0, {'encargo_id': v.get('encargo_id'), 'cst_id': v.get('cst_id'),
                                   'tributacao_id': v.get('tributacao_id')}] for v in filter(
            lambda f: f.get('encargo_id') in list(set(tributacoes_novas) -
                                                  set(tributacoes_anteriores)), list_vals))

        # Altera tributacoes
        tributacoes.extend([1, self.pool.get('fiscal.tributacao.entrada.encargos').search(
            cr, uid, [('tributacao_entrada_id', '=', tributacao_entrada_ids[0]),
                      ('encargo_id', '=', v.get('encargo_id'))], context=context)[0],
                            {'cst_id': v.get('cst_id'),
                             'tributacao_id': v.get('tributacao_id')}] for v in
                           filter(lambda f: f.get('encargo_id') in list(
                               set(tributacoes_anteriores) -
                               set(list(set(tributacoes_anteriores) -
                                        set(tributacoes_novas)))), list_vals))

        self.pool.get('fiscal.tributacao.entrada').write(
            cr, uid, tributacao_entrada_ids[0], {'tributacao_entrada_ids': tributacoes}, context)

        return True

    @log_debug
    def verifica_duplicacao_nf(self, cr, uid, tipo, serie_nf, nf_propria, company_id,
                               destinatario_id, data_emissao='', numero_nf=0,
                               modelo_nf_series_id=False, context=None):
        """
        Verifica a duplicidade da NF de acordo com a regra e negocio. Retorna um dicionario com
        o tipo e o numero da NF.
        """
        context = context and context.copy() or {}
        if 'active_id' in context:
            active_id = context.get('active_id')
        else:
            active_id = 0

        if tipo == 'E' and not nf_propria:
            if not context.get('criar'):
                if numero_nf == 0:
                    raise osv.except_osv(_('Atencao'), _('O Numero da nota fiscal nao pode ser '
                                                         'zero.'))
            args = [('numero_nf', '=', numero_nf),
                    ('data_emissao', '>=', data_emissao[:10] + ' 00:00:00'),
                    ('data_emissao', '<=', data_emissao[:10] + ' 23:59:59'),
                    ('company_id', '=', company_id),
                    ('destinatario_id', '=', destinatario_id),
                    ('serie_nf', '=', serie_nf),
                    ('state', '!=', 'C'),
                    ('id', '!=', active_id),
                    ('tipo', '=', 'E'),
                    ('nf_propria', '=', False)]
            busca_nf = self.search(cr, uid, args, context=context)
            if busca_nf:
                raise osv.except_osv(_('Atencao'), _('Numero da nota fiscal ja foi lancado para '
                                                     'esta data.'))

        return {'tipo': tipo, 'numero_nf': numero_nf}

    def button_rascunho(self, cr, uid, ids, context=None):
        """
        Retorna o estado da nota fiscal para esboco.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: verdadeiro se o estado foi retornado para esboco.
        :rtype: bool
        """
        context = context and context.copy() or {}
        if context.get('on_click'):
            if not self.fisnfc_permissao_para_voltar_esboco(cr, uid):
                msg = "Este usuario nao tem permissao para voltar notas fiscais para esboco"
                raise osv.except_osv(_('Atencao'), _(msg))
        context['acao_reservar_grade'] = 'Esboco'
        brw_nf = self.browse(cr, uid, ids[0], context)
        values = {'state': '1', 'fisnfc_infcomp_fisco': ''}
        self.write(cr, uid, ids, values, context=context)
        if brw_nf.fisnfc_nf_frete_ids and brw_nf.tipo_frete == 'F':
            context['validacao_nf_frete'] = True
            for f in brw_nf.fisnfc_nf_frete_ids:
                self.verifica_executa_status(cr, uid, [f.id], '1', context)
        self.fisnfc_deleta_fatura_prv_financeiro(cr, uid, ids, context)

        return True

    @log_debug
    def button_validar(self, cr, uid, nf, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, nf, {'state': '2'}, context=context)
        campos = ['tipo_frete', 'fisnfc_nf_frete_ids']
        if isinstance(nf, list):
            nf = nf[0]
        dados_nf = self.read(cr, uid, nf, campos, context=context, load='_classic_write')
        context['validacao_nf_frete'] = True
        for nf_frete_id in dados_nf.get('fisnfc_nf_frete_ids'):
            if dados_nf.get('tipo_frete') == 'F':
                self.verifica_executa_status(cr, uid, [nf_frete_id], '2', context)

        return True

    @log_debug
    def fisnfc_verifica_divergencia_frete(self, cr, user, dados_nf, dict_itens, dict_produtos,
                                          dict_trib, dict_cfop_itens, dados_cfop_nat,
                                          dados_nat_encargo, dict_encargos_nat, encargo_ids,
                                          dados_pessoa_user, dados_destinatario, tipo_atividade,
                                          context=None):
        """
        Verifica divergencias de frete na nota fiscal.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se nao ha divergencia no frete
        :rtype: bool
        """
        context = context and context.copy() or {}
        if not (dados_nf.get('fisnfc_nf_frete_ids') and dados_nf.get('tipo_frete') == 'F'):
            return True

        # Valor do frete e total das parcelas
        parcelas_total = 0

        campos = ['fisnfc_parcelas_terceiros_ids']
        for i in self.read(cr, user, dados_nf.get('fisnfc_nf_frete_ids'), campos, context):
            for parcela in self.pool.get('fiscal.nf.parcelas').read(
                    cr, user, i.get('fisnfc_parcelas_terceiros_ids'), ['valor'], context):
                parcelas_total += parcela.get('valor') or 0
        if dados_nf.get('tipo_frete') == 'F' and parcelas_total != dados_nf.get('vr_frete'):
            raise osv.except_osv(_('Atencao!'),
                                 _('O valor informado para o frete difere do somatorio das '
                                   'parcelas das notas fiscais de frete.'))
        msg_sem_tributos = list()
        msg_encargos = list()
        msg_cfop = list()
        for item in dict_itens.values():
            produto = dict_produtos.get(item.get('produto_id')) or {}
            nome_item = '%s - %s' % (produto.get('codigo'), produto.get('name'))

            # Verifica obrigatoridade de encargo dos itens
            if produto.get('tipo') == 'P':
                msg = ['* %s\n' % m[1] for m in
                       self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(
                           cr, user, map(lambda m: dict_trib.get(m).get('encargo_id'),
                                         item.get('tributacoes_ids')),
                           context)]
                if msg:
                    msg_encargos.append(_('Os seguintes encargos obrigatorios para esta empresa '
                                          'devem ser inseridos no item %s:\n%s') % (
                        nome_item, ''.join(msg)))

            # Item sem tributo
            if produto and not item.get('tributacoes_ids'):
                msg_sem_tributos.append('* %s\n' % nome_item)

            # Nota modelo ´02´ serie ´D´, terao o estado validado do proprio emitente
            if dados_nf.get('tipo') == 'S':
                dados_pessoa  = dados_destinatario
                if dados_nf.get('fisnfc_modelo_serie') == '02D' and not dados_nf.get('destinatario_id'):
                    dados_pessoa = dados_pessoa_user
                cfop_item_id = self.pool.get('estoque.movimento.itens').estmvi_get_cfop_id(
                    cr, user, produto.get('id'), produto.get('tipo'), dados_nf.get('tipo'),
                                 dados_cfop_nat, dados_nat_encargo, dict_encargos_nat, encargo_ids,
                                 dados_pessoa, tipo_atividade, context)
                if item.get('cfop_id'):
                    cfop = dict_cfop_itens.get(item.get('cfop_id'))
                    if cfop.get('id') != cfop_item_id and self.pool.get('fiscal.cfop').read(
                        cr, user, cfop.get('codigo'), ['codigo'], context).get('codigo') not in (
                            '1910', '2910', '1911', '2911', '5202', '5403', '5411', '5405', '5949',
                            '5910', '5911',
                            '5912', '2922', '5914', '5915', '5920', '6403', '6405', '6949', '6910',
                            '6911', '6912', '6914', '6915', '6920', '6949', '5927', '6411'):
                        msg_cfop.append('* %s\n' % nome_item)
        if msg_sem_tributos:
            raise osv.except_osv(_('Atencao!'),
                                 _('Os seguintes itens nao possuem tributos: %s\nSe nao existe '
                                   'tributacao do produto cadastrada para esta operacao, '
                                   'deve ser cadastrada. Apos cadastrada, excluir o item da '
                                   'nota e inserir novamente para que seja efetuado o '
                                   'recalculo.') % msg_sem_tributos)
        if msg_encargos:
            raise osv.except_osv(_('Atencao!'), ''.join(msg_encargos))
        if msg_cfop:
            raise osv.except_osv(_('Atencao!'),
                                 _('CFOP invalido para os seguintes itens:\n%s')
                                 % ''.join(msg_cfop))

        return True

    @log_debug
    def check_condicao_total_parcelas(self, cr, uid, dict_condicao, dict_cond_forma, dict_adm_cond,
                                      context=None):
        context = context and context.copy() or {}
        obj_parcela = self.pool.get('fiscal.nf.parcelas')
        divergente = False
        for i in dict_condicao.values():
            cond_forma = dict_cond_forma.get(i.get('condicao_forma_id'))
            adm_cond = dict_adm_cond.get(cond_forma.get('adm_condicao_id'))
            qtde_parcelas = adm_cond.get('qtde_parcelas')
            tipo = adm_cond.get('tipo')
            parcela_ids = obj_parcela.search(
                cr, uid, [('nf_condicao_id', '=', i.get('id'))], context=context)
            if tipo == 'V' and len(parcela_ids) != 1:
                divergente = True
            if len(parcela_ids) != qtde_parcelas:
                divergente = True
                break
        if divergente and not context.get('faturar_diversos_pedidos'):
            raise osv.except_osv(_('Atencao!'),
                                 _('Quantidade de parcelas encontra-se divergente para as '
                                   'condicoes de pagamento!'))
        return True

    @log_debug
    def fisnfc_validar_nota_fiscal(self, cr, uid, nf, context=None):
        """
        Valida as informacoes da nota fiscal antes de ser faturada.

        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota fiscal foi validada.
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['verificar_nf'] = True
        context['super_write_fiscal_nf_capa'] = True
        obj_natureza = self.pool.get('fiscal.cfop.natureza')
        obj_encargo = self.pool.get('fiscal.encargo')
        load = '_classic_write'
        campos = ['modelo_nf_series_id', 'natureza_id', 'fisnfc_nf_referencia_ids', 'guia_ids',
                  'vr_total_nf', 'nf_condicao_ids', 'company_id', 'numero_nf', 'data_emissao',
                  'destinatario_id', 'nf_propria', 'fisnfc_infcomp_fisco', 'fisnfc_modelo_serie',
                  'fisnfc_modelo', 'destinatario_endereco_id',
                  'estoque_movimento_itens_ids', 'vr_icms_substituicao', 'vendedor_id', 'tipo',
                  'vr_base_icms_substituicao', 'fisnfc_tipo_servico', 'data_saida_entrada',
                  'fisnfc_parcelas_terceiros_ids', 'fisnfc_consumidor', 'state',
                  'fisnfc_valor_icms_desonerado', 'modelo_nf_id']

        # Foi colocado 'read' para melhorar a performance para notas com muitos itens
        dados_nf = self.read(cr, uid, nf, campos, context, load)

        mod_serie_id = dados_nf.get('modelo_nf_series_id')
        encargo_ids = obj_encargo.search(cr, uid, [], context=context)
        dict_encargos = {}
        map(lambda x: dict_encargos.update({x.get('id'): x}),
            obj_encargo.read(cr, uid, encargo_ids, ['sigla'], context, load))
        natureza_id = dados_nf.get('natureza_id')
        dados_natureza = obj_natureza.read(
            cr, uid, natureza_id, ['fisnat_operacao_id', 'fisnat_calcula_difal', 'cfop_id',
                                   'fisnat_dispositivo_fiscal'],
            context, load)
        dados_operacao = self.pool.get('estoque.natureza.operacao').read(
            cr, uid, dados_natureza.get('fisnat_operacao_id'), ['estnto_gerar_financeiro'], context)
        trib_ids = []
        produto_ids = []
        dict_itens = {}
        campos = ['tributacoes_ids', 'produto_id', 'vendedor_id', 'vr_total_item']
        for i in self.pool.get('estoque.movimento.itens').read(
                cr, uid, dados_nf.get('estoque_movimento_itens_ids'), campos, context, load):
            dict_itens[i.get('id')] = i
            i.get('tributacoes_ids') and trib_ids.extend(i.get('tributacoes_ids'))
            produto_ids.append(i.get('produto_id'))
        dict_produtos = {}
        if produto_ids:
            map(lambda x: dict_produtos.update({x.get('id'): x}),
                self.pool.get('estoque.produto').read(cr, uid, produto_ids, ['name'], context))
        cst_ids = []
        tributacao_ids = []
        dict_tribs = {}
        campos = ['encargo_id', 'cst_id', 'vr_base_icms_substituicao', 'vr_icms_substituicao',
                  'tributacao_id', 'tipo_substituicao']
        for i in  self.pool.get('fiscal.nf.item.tributacao').read(
                cr, uid, trib_ids, campos, context, load):
            dict_tribs.update({i.get('id'): i})
            tributacao_ids.append(i.get('tributacao_id'))
            cst_ids.append(i.get('cst_id'))
        dict_tributacoes = {}
        campos = ['cst_id', 'encargo_id', 'dispositivo_fiscal']
        for i in self.pool.get('fiscal.tributacao').read(
                cr, uid, [_id for _id in tributacao_ids if _id], campos, context, load):
            dict_tributacoes.update({i.get('id'): i})
            cst_ids.append(i.get('cst_id'))

        dict_cst = {}
        map(lambda x: dict_cst.update({x.get('id'): x}),
            self.pool.get('fiscal.cst').read(cr, uid, cst_ids, ['tipo', 'codigo'], context))
        dict_condicao = {}
        dict_forma_pgto = {}
        dict_cond_forma = {}
        dict_adm_cond = {}
        dict_parcelas = {}
        if dados_nf.get('nf_condicao_ids') or dados_nf.get('fisnfc_parcelas_terceiros_ids'):
            condicao_ids = dados_nf.get('nf_condicao_ids') or []
            if not dados_nf.get('nf_condicao_ids'):
                parcela_ids = dados_nf.get('fisnfc_parcelas_terceiros_ids')
                if parcela_ids:
                    campos = ['nf_condicao_id', 'valor', 'vencimento']
                    for i in self.pool.get('fiscal.nf.parcelas').read(
                            cr, uid, parcela_ids, campos, context, load):
                        dict_parcelas.update({i.get('id'): i})
                        i.get('nf_condicao_id') and condicao_ids.append(i.get('nf_condicao_id'))
            cond_forma_ids = []
            forma_pgto_ids = []
            parcela_ids = []
            campos = ['condicao_forma_id', 'forma_pagto_id', 'fiscpg_chq_banco', 'nf_parcela_ids',
                      'fiscpg_chq_agencia', 'fiscpg_chq_numero', 'fiscpg_chq_conta',
                      'fiscpg_chq_emitente']
            if condicao_ids:
                for i in self.pool.get('fiscal.condicao.pagamento').read(
                        cr, uid, condicao_ids, campos, context, load):
                    dict_condicao[i.get('id')] = i
                    i.get('condicao_forma_id') and cond_forma_ids.append(i.get('condicao_forma_id'))
                    i.get('forma_pagto_id') and forma_pgto_ids.append(i.get('forma_pagto_id'))
                    i.get('nf_parcela_ids') and parcela_ids.extend(i.get('nf_parcela_ids'))
            if forma_pgto_ids:
                campos = ['cartao_credito_debito', 'admfpg_tipo', 'name',
                          'admfpg_documento_tipo_id', 'admfpg_cartao_pessoa_id']
                map(lambda x: dict_forma_pgto.update({x.get('id'): x}),
                    self.pool.get('adm.forma.pgto').read(cr, uid, forma_pgto_ids, campos, context,
                                                         load))
            adm_cond_ids = []
            if cond_forma_ids:
                campos = ['adm_condicao_id']
                for i in self.pool.get('adm.condicao.forma').read(
                        cr, uid, cond_forma_ids, campos, context, load):
                    dict_cond_forma.update({i.get('id'): i})
                    i.get('adm_condicao_id') and adm_cond_ids.append(i.get('adm_condicao_id'))
            if adm_cond_ids:
                campos = ['qtde_parcelas', 'tipo']
                map(lambda x: dict_adm_cond.update({x.get('id'): x}),
                    self.pool.get('adm.condicao').read(cr, uid, adm_cond_ids, campos, context))
            if not dict_parcelas:
                campos = ['nf_condicao_id', 'valor', 'vencimento']
                map(lambda x: dict_parcelas.update({x.get('id'): x}),
                    self.pool.get('fiscal.nf.parcelas').read(
                        cr, uid, parcela_ids, campos, context, load))
        values = {}  # Valores a serem atualizados na nota fiscal

        # Inclui nas informacoes complementares do fisco os dados que identificam que a NF possui
        # ICMS desonerado
        if dados_nf.get('fisnfc_valor_icms_desonerado') and dados_nf.get('tipo') == 'S':
            values.update({'fisnfc_infcomp_fisco': (dados_nf.get('fisnfc_infcomp_fisco') or '') + _(
                u'Operacao beneficada com isensao do ICMS nos termos do Convenio ICMS 26/03. Valor'
                u' dispensado de %s.') % locale.currency(dados_nf.get(
                    'fisnfc_valor_icms_desonerado') or 0, grouping=True)})

        # Cadastra o destinatario (cliente ou fornecedor) se necessario
        if dados_nf.get('fisnfc_modelo_serie') != '02D' and not dados_nf.get('fisnfc_consumidor') \
                and dados_nf.get('destinatario_id'):
            if not self.verificar_incluir_cli_for(
                    cr, uid, dados_nf.get('tipo'), dados_nf.get('destinatario_id'),
                    dados_nf.get('company_id'), 'incluir', natureza_id):
                raise osv.except_osv(_('Atencao'), _('E necessario cadastrar cliente/fornecedor'))

        nf_propria = dados_nf.get('nf_propria')
        data_emissao = dados_nf.get('data_emissao')

        # Efetua rateio para ICMS/ST da guia
        if dados_nf.get('tipo') == 'E':
            vr_guia = 0
            if dados_nf.get('guia_ids'):
                campos = ['guia_recomp_aliq', 'valor']
                for i in self.pool.get('fiscal.guia').read(
                        cr, uid, dados_nf.get('guia_ids'), campos, context):
                    if not i.get('guia_recomp_aliq'):
                        vr_guia += i.get('valor')
            self.calcula_rateio_icms_substituicao_guia(cr, uid, vr_guia, dict_itens, dict_tribs)

            # Data de entrada e saida
            if nf_propria:
                values['data_saida_entrada'] = data_emissao

                # Verifica codigo de pais no banco central para notas de importacao
                if self.pool.get('fiscal.cfop').read(
                        cr, uid, dados_natureza.get('cfop_id'), ['codigo'], context).get(
                    'codigo')[0] == '3':
                    endereco = self.pool.get('adm.pessoa.endereco').browse(
                        cr, uid, dados_nf.get('destinatario_endereco_id'), context)
                    if endereco and not endereco.pais_id:
                        raise osv.except_osv(
                            _('Atencao!'),
                            _('O pais no endereco do destinatario nao esta cadastrado.'))
                    elif endereco and not endereco.pais_id.codigo_bc:
                        raise osv.except_osv(
                            _('Atencao!'),
                            _('O codigo BC (banco central) para o pais no endereco do destinatario '
                              'nao esta cadastrado.'))
        else:
            if dados_nf.get('data_saida_entrada') and \
                    dados_nf.get('data_saida_entrada') < dados_nf.get('data_emissao'):
                values['data_saida_entrada'] = dados_nf.get('data_emissao')

        # Cria as faturas e os titulos
        if not dados_nf.get('fisnfc_nf_referencia_ids') and dados_nf.get('vr_total_nf'):
            if (any(map(lambda con: con.get('tipo') == 'C', dict_adm_cond.values())) or
                    any(map(lambda f: f.get('cartao_credito_debito'), dict_forma_pgto.values()))) \
                    or dados_nf.get('destinatario_id'):
                self.fisnfc_criar_fatura_pvr(cr, uid, dados_nf, dados_operacao, dict_condicao,
                                             dict_forma_pgto, dict_cond_forma, dict_adm_cond,
                                             dict_parcelas, context)

        # Atualiza informacoes complementares (fisco)
        encargos = []
        informacoes_complementares_fisco = dados_nf.get('fisnfc_infcomp_fisco') or ''
        if dados_nf.get('modelo_nf_series_id'):
            _modelo_nf_id = self.pool.get('fiscal.modelo.nf.series').read(cr, uid, dados_nf.get('modelo_nf_series_id'),['modelo_nf_id']).get('modelo_nf_id')[0]
        else:
            _modelo_nf_id = dados_nf.get('modelo_nf_id')
        _modelo_nf = self.pool.get('fiscal.modelo.nf').read(cr, uid, _modelo_nf_id, ['fismnf_codigo']).get('fismnf_codigo')
        if _modelo_nf < '90':
            for item in dict_itens.values():
                # Dispositivo fiscal
                produto = dict_produtos.get(item.get('produto_id'))
                for trib_id in item.get('tributacoes_ids'):
                    trib = dict_tribs.get(trib_id)
                    tributacao = dict_tributacoes.get(trib.get('tributacao_id'))

                    # Verificacao da tributacao do item
                    if not tributacao:
                        raise osv.except_osv(_('Atencao!'),
                                             _('O item %s esta sem tributacao definida ou contem uma '
                                               'tributacao invalida.') % produto.get('name'))

                    cst_it = dict_cst.get(trib.get('cst_id'))
                    cst_trib = dict_cst.get(tributacao.get('cst_id'))
                    encargo_it = dict_encargos.get(trib.get('encargo_id'))
                    encargo_trib = dict_encargos.get(tributacao.get('encargo_id'))

                    # Verifica encargo e CST da tributacao
                    if encargo_it.get('sigla') != encargo_trib.get('sigla'):
                        raise osv.except_osv(_('Atencao!'),
                                             _('O item %s apresenta divergencia entre os '
                                               'encargos. \n Encargo: %s \n Encargo da tributacao: '
                                               '%s.') % (produto.get('name'), encargo_it.get(
                                                 'sigla'), encargo_trib.get('sigla')))
                    if cst_it.get('codigo') != cst_trib.get('codigo'):
                        raise osv.except_osv(_('Atencao!'),
                                             _('O item %s apresenta divergencia entre CST. \n '
                                               'CST do encargo %s: %s \n CST da tributacao: '
                                               '%s.') % (produto.get('name'), encargo_it.get('sigla'),
                                                         cst_it.get('codigo'), cst_trib.get('codigo')))
                    if (encargo_it.get('id'), tributacao.get('id')) not in encargos:
                        if encargos and tributacao.get('dispositivo_fiscal') and \
                                tributacao.get('dispositivo_fiscal').strip():
                            informacoes_complementares_fisco += \
                                ' *%s' % tributacao.get('dispositivo_fiscal')
                    encargos.append((encargo_it.get('id'), tributacao.get('id')))
        if dados_nf.get('vr_icms_substituicao'):
            informacoes_complementares_fisco += _(' * B.Calc.Subst: %.2f, Valor ICMS Subst: %.2f' %
                                                  (dados_nf.get('vr_base_icms_substituicao'),
                                                   dados_nf.get('vr_icms_substituicao')))
        if dados_natureza.get('fisnat_dispositivo_fiscal'):
            informacoes_complementares_fisco += ' %s' % dados_natureza.get('fisnat_dispositivo_fiscal')
        values['fisnfc_infcomp_fisco'] = (values.get('fisnfc_infcomp_fisco') or '') + \
            informacoes_complementares_fisco

        # Vendedor do item igual ao vendedor da nota fiscal
        if dados_nf.get('vendedor_id'):
            movimento_itens_ids = []
            for i in dict_itens.values():
                if not i.get('vendedor_id'):
                    movimento_itens_ids.append(i.get('id'))
            if movimento_itens_ids:
                context_itens = context.copy()
                context_itens['super_write_estoque_movimento_itens'] = True
                self.pool.get('estoque.movimento.itens').write(
                    cr, uid, movimento_itens_ids, {'vendedor_id': dados_nf.get('vendedor_id')},
                    context)

        # Verifica totais de icms_st_itens x total_icms_st destacada
        # A parte de verificacao do ICMS ST nao foi separada em uma funcao porque os retornos para
        # a tela que interagem com o usuario tem de ter retorno dentro deste metodo, pois em uma
        # funcao separada a tela nao retorna.
        if not nf_propria:
            if not context.get('rateio_substituicao_conferido'):
                obj_fiscal_exibe_questao = self.pool.get('fiscal.exibe.questao.osv')

                # Calcula o valor total do ICMS ST dos itens..
                if dados_nf.get('vr_icms_substituicao'):
                    verifica_substituicao = self.verifica_icms_substituicao(
                        cr, uid, dados_nf.get('estoque_movimento_itens_ids'),
                        dados_nf.get('vr_icms_substituicao'), context=context)
                    if verifica_substituicao['erro']:
                        id_info = obj_fiscal_exibe_questao.create(cr, uid, {
                            'name': verifica_substituicao['erro'],
                            'opcao_confirma': verifica_substituicao.get(
                                'opcao_confirma')}, context)
                        return {
                            'name': _('Informacao'),
                            'view_type': 'form',
                            'view_mode': 'form',
                            'res_id': id_info,
                            'res_model': 'fiscal.exibe.questao.osv',
                            'view_id': False,
                            'context': context,
                            'target': 'new',
                            'type': 'ir.actions.act_window'}
                    elif not verifica_substituicao['erro']:
                        # Compara valor base ICMS/ST da capa com o total dos itens
                        compara_base_icms_st = \
                            verifica_substituicao.get('total_base_icms_st') != \
                            dados_nf.get('vr_base_icms_substituicao') \
                                if verifica_substituicao.get('total_base_icms_st') else False

                        # Troca a mensagem se a divergencia for no valor base ICMS ST
                        if compara_base_icms_st or verifica_substituicao.get('necessita_rateio'):
                            msg_substituicao = _(
                                'O valor destacado da BASE ICMS ST esta divergente em relacao ao '
                                'valor total da BASE ICMS ST dos itens (com CST 10, 30, 60 e 70). '
                                'Se desejar que os valores da BASE ICMS ST sejam considerados, '
                                'clique em "Confirmar". Se desejar que o sistema realize o calculo '
                                'e ajuste os valores, clique em "Calcular e sair".')
                            id_info = self.pool.get('fiscal.exibe.questao.osv').create(
                                cr, uid, {'name': msg_substituicao, 'opcao_confirma': True})
                            return {'name': _('Informacao'), 'view_type': 'form',
                                    'view_mode': 'form', 'res_id': id_info,
                                    'res_model': 'fiscal.exibe.questao.osv', 'view_id': False,
                                    'context': context, 'target': 'new',
                                    'type': 'ir.actions.act_window'}

        if nf_propria:
            # Atribui numero de serie e numero da nota
            dados_modelo_serie = self.pool.get('fiscal.modelo.nf.series').read(
                cr, uid, mod_serie_id, ['modelo_nf_id', 'serie_nf', 'numero_nf'], context, load)
            if not any((dados_nf.get('serie_nf'), dados_nf.get('numero_nf'))):
                values['serie_nf'] = dados_modelo_serie.get('serie_nf')
                numero_nf = dados_modelo_serie.get('numero_nf')
                values['numero_nf'] = numero_nf

                # Atualiza numeracao do modelo da nota fiscal
                self.pool.get('fiscal.modelo.nf.series').fismnfs_atualiza_modelo_numero_nf(
                    cr, uid, mod_serie_id, context)
            else:
                numero_nf = dados_nf.get('numero_nf')

            # Atualiza o numero do documento da fatura
            fatura_ids = self.pool.get('financeiro.fatura').search(
                cr, uid, [('nf_id', '=', dados_nf.get('id'))], context=context)
            if fatura_ids:
                self.pool.get('financeiro.fatura').write(
                    cr, uid, fatura_ids, {'documento': '%09d' % numero_nf})

            values['modelo_nf_id'] = dados_modelo_serie.get('modelo_nf_id')
            context['super_write_fiscal_nf_capa'] = True
            data_emissao = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
                cr, uid, datetime.strptime(data_emissao, '%Y-%m-%d %H:%M:%S'), 'UTC',
                context=context), '%Y-%m-%d')

            # Ajusta data de vencimento das parcelas
            values_cond = []
            for condicao in dict_condicao.values():
                values_parcelas = []
                for parcela_id in condicao.get('nf_parcela_ids'):
                    parcela = dict_parcelas.get(parcela_id)
                    vencimento = parcela.get('vencimento')
                    if parcela.get('vencimento') < data_emissao:
                        vencimento = data_emissao
                    values_parcelas.append([1, parcela.get('id'), {'vencimento': vencimento}])
                values_cond.append([1, condicao.get('id'), {'nf_parcela_ids': values_parcelas}])
            values['nf_condicao_ids'] = values_cond
        else:
            data_emissao = datetime.strftime(self.pool.get('adm.utils').admutl_converte_tz(
                cr, uid, datetime.strptime(data_emissao, '%Y-%m-%d %H:%M:%S'), 'UTC',
                context=context), '%Y-%m-%d')
            values_parcelas = []
            for parcela in dict_parcelas.values():
                vencimento = parcela.get('vencimento')
                if parcela.get('vencimento') < data_emissao:
                    vencimento = data_emissao
                values_parcelas.append([1, parcela.get('id'), {'vencimento': vencimento}])
            values['fisnfc_parcelas_terceiros_ids'] = values_parcelas

        # DIFAL
        values['data_emissao'] = values.get('data_emissao') or self.read(
            cr, uid, nf, ['data_emissao'], context).get('data_emissao')
        values['destinatario_endereco_id'] = values.get('destinatario_endereco_id') or self.read(
            cr, uid, nf, ['destinatario_endereco_id'], context, '_classic_write').get(
            'destinatario_endereco_id')
        item_ids = values.get('estoque_movimento_itens_ids') or self.read(
            cr, uid, nf, ['estoque_movimento_itens_ids'], context).get(
            'estoque_movimento_itens_ids')
        if 'estoque_movimento_itens_ids' not in values:
            values['estoque_movimento_itens_ids'] = item_ids
        context['fisnat_calcula_difal'] = dados_natureza.get('fisnat_calcula_difal')
        values['destinatario_id'] = dados_nf.get('destinatario_id')
        values['company_id'] = dados_nf.get('company_id')
        self.pool.get('fiscal.nf.item.tributacao').fisnit_gerar_difal(cr, uid, values, context)

        # Nota fiscal validada!
        values['state'] = '2'
        context['super_write_estoque_movimento_itens'] = True
        return self.write(cr, uid, nf, values, context)

    @log_debug
    def verifica_cst(self, cr, uid, dados_nf, context=None):
        '''Verifica o tipo da nota e os csts dos itens. Nota de entrada nao deve ter CST de saida e 
           nota de saida nao deve ter CST de entrada 
           
           OBS: nao utilizar browse para nao perder performance'''
        context = context and context.copy() or {}
        # Notas sem itens nao testa
        if not dados_nf.get('estoque_movimento_itens_ids'):
            return True

        campos = ['sequencia','tributacoes_ids']
        dict_tribs_item = {}
        trib_ids = []
        context['super_read_estoque_movimento_itens'] = True
        # nao utilizar browse para nao perder performance
        for i in self.pool.get('estoque.movimento.itens').read(
                cr, uid, dados_nf.get('estoque_movimento_itens_ids'), campos, context=context):
            trib_ids.extend(i.get('tributacoes_ids'))
            dict_tribs_item.update({}.fromkeys(i.get('tributacoes_ids'), i))
        trib_invalidas_ids = []
        dict_trib_cst = {}
        cst_ids = []
        for i in self.pool.get('fiscal.nf.item.tributacao').read(
                cr, uid, trib_ids, ['cst_id'], context=context, load='_classic_write'):
            if not dict_trib_cst.get((i.get('cst_id'))):
                dict_trib_cst[i.get('cst_id')] = [i.get('id')]
            else:
                dict_trib_cst[i.get('cst_id')].append(i.get('id'))
            cst_ids.append(i.get('cst_id'))
        for cst in self.pool.get('fiscal.cst').read(
                cr, uid, cst_ids, ['fiscst_tipo_movimento'], context=context):
            if (cst.get('fiscst_tipo_movimento') == 'E' and dados_nf.get('tipo') == 'S') or \
               (cst.get('fiscst_tipo_movimento') == 'S' and dados_nf.get('tipo') == 'E'):
                trib_invalidas_ids.extend(dict_trib_cst.get(cst.get('id')))
        if not trib_invalidas_ids:
            return True
        msg = []
        for i in trib_invalidas_ids:
            item = dict_tribs_item.get(i)
            if '%s' % item.get('sequencia') not in msg:
                msg.append('%s' % item.get('sequencia'))
        texto = dados_nf.get('tipo') == 'S' and _('Entrada') or _('Saida')
        raise osv.except_osv(_('Atencao!'),
                             _('Os itens de sequencia a seguir possuem cst de %s:\n%s') %
                             (texto, 'Sequencia: %s' % ', '.join(msg)) )

    @log_debug
    def button_verificar_nf(self, cr, uid, nf, context=None):
        """
        Verifica e valida uma nota fiscal.

        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota foi verificada e validada.
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['super_write_fiscal_nf_capa'] = True
        context['super_write_estoque_movimento_itens'] = True
        context['ctx_fisnfc_titulo_create_sem_custificacao'] = True
        nf = nf[0] if isinstance(nf, list) else nf
        self.pool.get('estoque.movimento.itens').estmvi_valida_quantidade(cr, uid, nf, context)
        dados_nf = self.read(cr, uid, nf, ['fisnfc_calcula_peso_automaticamente',
                                           'estoque_movimento_itens_ids', 'tipo', 'modelo_nf_series_id', 'modelo_nf_id'],
                             context=context)
        if dados_nf.get('modelo_nf_series_id'):
            _modelo_nf_id = self.pool.get('fiscal.modelo.nf.series').read(cr, uid, dados_nf.get('modelo_nf_series_id')[0],
                                                                          ['modelo_nf_id']).get('modelo_nf_id')[0]
            _modelo_nf = self.pool.get('fiscal.modelo.nf').read(cr, uid, _modelo_nf_id, ['fismnf_codigo']).get('fismnf_codigo')
        else:
            _modelo_nf = dados_nf.get('modelo_nf_id')[1][:2]
        if _modelo_nf < '90':
            self.verifica_cst(cr, uid, dados_nf, context)
        if dados_nf.get('fisnfc_calcula_peso_automaticamente'):
            self.fisnfc_atualiza_peso(cr, uid, nf, context=context)
        self.fisnfc_verificar_nota_fiscal(cr, uid, nf, context=context)
        result = self.fisnfc_validar_nota_fiscal(cr, uid, nf, context=context)
        if not isinstance(result, dict):  # Dicionario se houve erro nos totais de ICMS/ST
            result = self.atualiza_sequencia_itens(cr, uid, nf, context=context)

        return result

    def atualiza_sequencia_itens(self, cr, uid, id, context=None):
        context = context and context.copy() or {}
        obj_itens = self.pool.get('estoque.movimento.itens')
        context.update({'super_read_estoque_movimento_itens': True,
                        'super_write_estoque_movimento_itens':True})
        if isinstance(id, list):
            id = id[0]
        dados_nf = self.read(cr, uid, id, ['estoque_movimento_itens_ids'], context=context)
        item_ids = dados_nf.get('estoque_movimento_itens_ids')
        if not item_ids:
            return True

        item_ids = obj_itens.search(cr, uid, [('id', 'in', item_ids)], order='sequencia')
        qtd_itens = len(item_ids)
        sequencia = range(1, qtd_itens+1)
        dados_itens = sorted(obj_itens.read(cr, uid, item_ids, ['sequencia'], context=context),
                             key=lambda x: x.get('sequencia'))
        sequencia_errada = filter(lambda x: x.get('sequencia') not in sequencia, dados_itens)
        if not sequencia_errada:
            return True

        cont = 1
        context['super_write_estoque_movimento_itens'] = True
        for i in dados_itens:
            obj_itens.write(cr, uid, i.get('id'), {'sequencia': cont}, context=context)
            cont += 1

        return True

    def verifica_condicaos_sem_tipo_documento(self, cr, uid, dados_nf, context=None):
        '''
        Verifica se tem alguma condicao com forama de pagamento sem tipo de documento cadastrado
        '''
        context = context and context.copy() or {}
        load = '_classic_write'
        forma_pgto_ids = map(lambda x: x.get('forma_pagto_id'),
                             self.pool.get('fiscal.condicao.pagamento').read(
                                 cr, uid, dados_nf.get('nf_condicao_ids'), ['forma_pagto_id'],
                                 context, load))
        campos = ['name', 'admfpg_documento_tipo_id']
        dados_forma_pgto = self.pool.get('adm.forma.pgto').read(
            cr, uid, forma_pgto_ids, campos, context, load)
        formas_pgto = []
        for forma_pgto in dados_forma_pgto:
            if not forma_pgto.get('admfpg_documento_tipo_id'):
                formas_pgto.append(forma_pgto.get('name'))
        if formas_pgto:
            raise osv.except_osv(_('Atencao!'),
                 _('A(s) seguinte(s) forma(s) de pagamento(s) nao possui(em) tipo de documento:\n%s'
                   % ('\n'.join(formas_pgto))))

        return True

    @log_debug
    def verifica_fornecedor(self, cr, uid, pessoa_id, company_id, context=None):
        """
        Cria um fornecedor no financeiro se a pessoa informada na nota nao estiver cadastrada como
        fornecedor, e se o paramentro no financeiro for para cadastar automaticamente o fornecedor.
        :param cr:
        :param uid:
        :param pessoa_id: id da pessoa na capa da nota
        :param company_id: id da empresa da nota
        :param context:
        :return: 'True' se tem forcedor ou conseguiu cadastra-lo, do contrario 'False'
        """
        # Se nao existir fornecedor entao sera criada um
        if self.pool.get('financeiro.fornecedor').search(
                cr, uid, [('pessoa_id', '=', pessoa_id), ('company_id', '=', company_id)],
                context=context):
            return True

        # Cadastra ou nao o fornecedor conforme parametro do Financeiro
        parametro_financeiro = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'financeiro', [], context)
        if not parametro_financeiro.get('cadastro_automatico_fornecedor'):
            return False

        return self.pool.get('financeiro.fornecedor').verificar_incluir_fornecedor(
            cr, uid, pessoa_id, company_id, acao='incluir', context=context) and True or False

    @log_debug
    def verifica_cliente_nf_transferencia(self, dados_nf, dados_cfop_nat, dados_pessoa_user,
                                          dados_pessoa_cliente, context=None):
        context = context and context.copy() or {}
        if not dados_nf.get('natureza_id') or dados_cfop_nat.get('movimento_tipo') != 'T':
            return
        if not dados_pessoa_user:
            return
        if dados_pessoa_cliente.get('cpf_cnpj') == dados_pessoa_user.get('cpf_cnpj'):
            raise osv.except_osv(_('Atencao!'),
                                 _('O destinatario da nota de transferencia nao pode possuir o '
                                   'mesmo CPF/CNPJ do emitente.'))

        # Verifica se a raiz do CNPJ sao iguais
        if dados_pessoa_user.get('cpf_cnpj')[:10] != dados_pessoa_cliente.get('cpf_cnpj')[:10]:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nota de transferecia apenas entre empresas do mesmo grupo!'))

    def fisnfc_validar_valor(self, cr, uid, dados_nf, context=None):
        # Verifica NFC-e com valor acima de 3000 sem clientes, nao deixando realizar a venda
        if not context.get('nota_fiscal_consumidor') or dados_nf.get('vr_total_nf') < 3000 or \
                dados_nf.get('destinatario_id'):
            return True

        raise osv.except_osv(_('Atencao!'),
                             _('Para NFC-e com valor acima de R$3000,00 e obrigatorio informar '
                               'o cliente pelas regras da SEFAZ'))

    @log_debug
    def fisnfc_verificar_nota_fiscal(self, cr, uid, nf, context=None):
        """
        Realiza a verificacao de uma nota fiscal.

        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se a nota fiscal foi verificada sem erro ou inconsistencia.
        :rtype: bool
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        obj_encargo = self.pool.get('fiscal.encargo')
        obj_produto = self.pool.get('estoque.produto')
        obj_natureza = self.pool.get('fiscal.cfop.natureza')
        obj_cfop = self.pool.get('fiscal.cfop')
        obj_item_trib = self.pool.get('fiscal.nf.item.tributacao')
        obj_pessoa = self.pool.get('adm.pessoa')
        context['ctx_fisnfc_titulo_create_sem_custificacao'] = True
        campos = ['estoque_movimento_itens_ids', 'finalidade_emissao', 'natureza_id', 'vr_total_nf',
                  'nf_propria', 'modelo_nf_id', 'numero_nf', 'serie_nf', 'tipo', 'destinatario_id',
                  'company_id', 'data_saida_entrada', 'data_emissao', 'nf_condicao_ids',
                  'transportador_id', 'fisnfc_placa_frete', 'fisnfc_modelo_serie',
                  'fisnfc_modelo',
                  'destinatario_endereco_id', 'vr_desconto', 'fisnfc_nf_referenciada_ids',
                  'fisnfc_nf_referencia_ids', 'vendedor_id', 'guia_ids', 'state', 'tipo_frete',
                  'fisnfc_nf_frete_ids', 'vr_frete', 'contrato_id', 'vr_total_deducao',
                  'nf_parcelas_ids', 'fisnfc_parcelas_terceiros_ids', 'fisnfc_valor_bonificado',
                  'fisnfc_consumidor', 'fisnfc_icms_desonerado', 'nfe_processo_emissao',
                  'modelo_55', 'fisnfc_consumidor']
        # Foi colocado 'read' para melhorar a performance para notas com muitos itens
        dados_nf = self.read(cr, uid, nf, campos, context, load)
        self.fisnfc_validar_valor(cr, uid, dados_nf, context)
        destinatario_id = dados_nf.get('destinatario_id')
        company_id = dados_nf.get('company_id')
        if not dados_nf.get('nf_propria'):
            if not self.verifica_fornecedor(cr, uid, destinatario_id, company_id, context):
                raise osv.except_osv('Atencao!',
                                     'Fornecedor nao cadastrado no financeiro!')

            # Verifica se a data de saida/entrada eh maior que a data de emissao.
            self.fisnfc_verifica_data_saida_entrada(cr, uid, dados_nf, context)
        self.verifica_condicaos_sem_tipo_documento(cr, uid, dados_nf, context=context)

        # TODO: Separar verificacao em metodos distintos para notas fiscais propria ou terceiros e
        # de entrada e saida
        #  Verifica usuario logado
        if not self.pool.get('rh.funcionario').get_id_func(cr, uid):
            raise osv.except_osv(_('Atencao!'), _('Usuario nao esta cadastrado como funcionario.'))

        # Verifica a natureza da operacao
        natureza_id = dados_nf.get('natureza_id')
        campos = ['natureza_encargos_ids', 'tipo', 'fisnat_natureza_fiscal', 'cfop_id',
                  'fisnat_operacao_id', 'fisnat_complemento_arroz',
                  'fisnat_complemento_arroz_item_id']
        dados_natureza = obj_natureza.read(cr, uid, natureza_id, campos, context, load)
        campos = ['encargo_id', 'sobrepor_tributacao_id', 'fiscne_sobrepoe_so_tributados']
        dados_nat_encargo = self.pool.get('fiscal.cfop.natureza.encargos').read(
            cr, uid, dados_natureza.get('natureza_encargos_ids'), campos, context, load)
        nat_encargo_ids = map(lambda x: x.get('encargo_id'), dados_nat_encargo)
        dict_encargos_nat = {}
        map(lambda x: dict_encargos_nat.update({x.get('id'): x}),
            self.pool.get('fiscal.encargo').read(cr, uid, nat_encargo_ids, ['sigla'], context))
        dados_operacao = self.pool.get('estoque.natureza.operacao').read(
            cr, uid, dados_natureza.get('fisnat_operacao_id'), ['estnto_tipo_operacao'], context)
        if dados_natureza.get('tipo') != dados_operacao.get('estnto_tipo_operacao'):
            raise osv.except_osv(_('Atencao!'),
                                 _('As informacoes da natureza da operacao nao foram '
                                   'preenchidas corretamente. Na operacao selecionada dentro '
                                   'do cadastro de natureza, o tipo da operacao difere do tipo '
                                   'da natureza, corrija o cadastro por favor.'))
        campos = ['movimento_tipo', 'codigo', 'sigla']
        dados_cfop_nat = obj_cfop.read(cr, uid, dados_natureza.get('cfop_id'), campos, context)
        if dados_natureza and not dados_natureza.get('fisnat_natureza_fiscal'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A natureza fiscal deve ser informada para o CFOP %s.') %
                                 dados_cfop_nat.get('codigo'))
        dados_company = self.pool.get('res.company').read(
            cr, uid, company_id, ['rescon_pessoa_id'], context, load)
        pessoa_ids = destinatario_id and list(
            {destinatario_id, dados_company.get('rescon_pessoa_id')}) or []
        dados_pessoa_user = False
        dados_pessoa_cliente = False
        for i in pessoa_ids and obj_pessoa.read(
                cr, uid, pessoa_ids, ['cpf_cnpj', 'endereco_ids', 'admpes_jur_tipo_atividade'],
                context):
            if i.get('id') == destinatario_id:
                dados_pessoa_cliente = i.copy()
                if len(pessoa_ids) == 1:
                    dados_pessoa_user = i.copy()
            elif i.get('id') == dados_company.get('rescon_pessoa_id'):
                dados_pessoa_user = i.copy()
        tipo_atividade = dados_pessoa_user and dados_pessoa_user.get('admpes_jur_tipo_atividade')
        self.verifica_cliente_nf_transferencia(
            dados_nf, dados_cfop_nat, dados_pessoa_user, dados_pessoa_cliente, context=context)

        # Verifica encargos obrigatorios faltantes para Natureza da operacao
        msg = ['* %s\n' % m[1] for m in
               self.pool.get('fiscal.utils').verifica_encargos_obrigatorio_empresa(
                   cr, uid, nat_encargo_ids, context)]
        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('Verifique o cadastro de natureza da operacao informada na '
                                   'nota. Os seguintes encargos obrigatorios para esta empresa '
                                   'devem ser inseridos:\n%s') % ''.join(msg))

        itens_ids = dados_nf.get('estoque_movimento_itens_ids')
        dict_itens = {}
        produto_ids = []
        cfop_ids = []
        local_ids = []
        local_grades = {}
        tributacoes_ids = []
        campos = ['sequencia', 'tributacoes_ids', 'produto_id', 'cfop_id', 'grade_id', 'local_id',
                  'quantidade', 'estmvi_volume', 'vr_total_item', 'vendedor_id']
        for i in self.pool.get('estoque.movimento.itens').read(
                cr, uid, itens_ids, campos, context, load):
            dict_itens.update({i.get('id'): i})
            produto_ids.append(i.get('produto_id'))
            i.get('cfop_id')and cfop_ids.append(i.get('cfop_id'))
            i.get('tributacoes_ids') and tributacoes_ids.extend(i.get('tributacoes_ids'))
            local_id = i.get('local_id')
            local_ids.append(local_id)
            if not local_grades.get(local_id):
                local_grades[local_id] = [i.get('grade_id')]
            else:
                local_grades[local_id].append(i.get('grade_id'))
        dict_produtos = {}
        campos = ['name', 'tipo', 'codigo', 'ncm_id']
        map(lambda x: dict_produtos.update({x.get('id'): x}),
            obj_produto.read(cr, uid, produto_ids, campos, context, '_classic_write'))
        encargo_ids = obj_encargo.search(cr, uid, [], context=context)
        dict_encargos = {}
        map(lambda x: dict_encargos.update({x.get('id'): x}),
            obj_encargo.read(cr, uid, encargo_ids, ['sigla'], context))
        cst_ids = []
        dict_trib = {}
        campos = ['encargo_id', 'cst_id', 'vr_base_icms_substituicao', 'vr_icms_substituicao']
        for i in  obj_item_trib.read(cr, uid, tributacoes_ids, campos, context, load):
            dict_trib.update({i.get('id'): i})
            cst_ids.append(i.get('cst_id'))
        dict_cst = {}
        map(lambda x: dict_cst.update({x.get('id'): x}),
            self.pool.get('fiscal.cst').read(cr, uid, cst_ids, ['tipo', 'codigo'], context))
        dict_cfop_itens = {}
        campos = ['movimento_tipo', 'codigo']
        map(lambda x: dict_cfop_itens.update({x.get('id'): x}),
            obj_cfop.read(cr, uid, cfop_ids, campos, context))
        dict_locais = {}
        campos = ['name', 'aceita_saldo_negativo']
        map(lambda x: dict_locais.update({x.get('id'): x}),
            self.pool.get('estoque.local').read(cr, uid, local_ids, campos, context))
        campos = ['fismnf_tipo', 'fismnf_tipo_nfe', 'fismnf_tipo_nfe']
        dados_modelo = self.pool.get('fiscal.modelo.nf').read(
            cr, uid, dados_nf.get('modelo_nf_id'), campos, context)
        encago_empresa_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid,
                          'fiscal.encargo.empresa', context=context, user_company_id=company_id)
        if dados_nf.get('tipo') == 'S':
            # nota eletronica
            if dados_modelo.get('fismnf_tipo_nfe'):
                # transportadora com ANTT
                if dados_nf.get('transportador_id'):
                    dados_transp = self.pool.get('adm.transportador').read(
                        cr, uid, dados_nf.get('transportador_id'), ['admtra_annt'], context)
                    if dados_transp.get('admtra_annt'):
                        # Nao valida nota sem placa com transportadora com ANTT cadastrado para o
                        # frete
                        if not dados_nf.get('fisnfc_placa_frete'):
                            raise osv.except_osv(_('Atencao!'),
                                                 _('Para transportadora com ANTT deve ser '
                                                   'informada a placa do veiculo para o frete!'))

            # Verifica encargos obrigatorios faltantes para os itens da nota de saida
            if dados_nf.get('finalidade_emissao') != '2':  # Desconsidera notas de complemento
                for item_id in dados_nf.get('estoque_movimento_itens_ids'):
                    item = dict_itens.get(item_id)
                    produto = dict_produtos.get(item.get('produto_id'))
                    if dados_nf.get('fisnfc_modelo') < '90':
                        msg = self.pool.get('fiscal.utils').fisutl_encargo_faltante_item(cr, uid,
                                     item.get('tributacoes_ids'), produto, dict_encargos,
                                     dict_trib, encago_empresa_id, context)
                        if not msg:
                            msg = self.pool.get('fiscal.utils').fisutl_encargo_item_fora_natureza(cr, uid,
                                     item.get('tributacoes_ids'), nat_encargo_ids, dict_encargos,
                                     dict_trib, context)

                        if msg:
                            raise osv.except_osv(_('Atencao!'),
                                                 _('Produto: %s\n%s') % (produto.get('name'), msg))
        elif dados_nf.get('nf_propria'):
            # Verifica encargos obrigatorios faltantes para os itens da nota
            for item_id in dados_nf.get('estoque_movimento_itens_ids'):
                item = dict_itens.get(item_id)
                produto = dict_produtos.get(item.get('produto_id'))
                msg = self.pool.get('fiscal.utils').fisutl_encargo_faltante_item(cr, uid,
                                 item.get('tributacoes_ids'), produto, dict_encargos,
                                 dict_trib, encago_empresa_id, context)
                if not msg:
                    msg = self.pool.get('fiscal.utils').fisutl_encargo_item_fora_natureza(cr, uid,
                             item.get('tributacoes_ids'), nat_encargo_ids, dict_encargos,
                             dict_trib, context)
                if msg:
                    produto = dict_produtos.get(item.get('produto_id'))
                    raise osv.except_osv(_('Atencao!'),
                                         _('Produto: %s - %s:\n%s') % (item.get('sequencia'),
                                                                       produto.get('name'),
                                                                       msg))
            # Verifica se o encargo II (Imposto de Importacao) esta cadastrado para CFOP que
            # inicia com '3'
            if dados_nf.get('tipo') == 'E':
                if dados_cfop_nat.get('codigo')[0] == '3':
                    if not self.pool.get('fiscal.encargo').search(
                            cr, uid, [('sigla', 'ilike', 'II')], count=True):
                        raise osv.except_osv(_('Atencao!'),
                                             _('Encargo II-Impostro de importacao nao esta '
                                               'cadastrado.'))
        else:  # Entrada de terceiro
            if dados_nf.get('modelo_55') or dados_nf.get('fisnfc_consumidor'):
                nfe_valida = self.fisnfc_valida_chave_nfe(
                    cr, uid, dados_nf['id'], dados_nf.get('nfe_chave_acesso'),
                    dados_nf.get('nfe_processo_emissao'), context)
                if not nfe_valida['valida']:
                    raise osv.except_osv(_('Atencao!'),
                                         nfe_valida['msg'].decode('utf-8'))

        # Verifica se todas as formas de pagamento para as notas proprias possuem a conta da
        # tesouraria informada, senao, verifica se a conta caixa para recebimento a vista esta
        # cadastrada no Financeiro
        cond_pag_campos = ['condicao_forma_id', 'forma_pagto_id', 'fiscpg_chq_banco',
                           'fiscpg_chq_agencia', 'fiscpg_chq_numero', 'fiscpg_chq_conta', 'capa_id']
        fiscal_condicao_pagamento_dados = self.pool.get('fiscal.condicao.pagamento').read(
            cr, uid, dados_nf.get('nf_condicao_ids'), cond_pag_campos, context, load)
        forma_pagto_ids = list(set(map(
            lambda cp: cp.get('forma_pagto_id'), fiscal_condicao_pagamento_dados))) \
            if fiscal_condicao_pagamento_dados else []
        adm_forma_pgto_dados = self.pool.get('adm.forma.pgto').read(
            cr, uid, forma_pagto_ids, ['admfpg_conta_tesouraria_id'], context, load)
        admfpg_conta_tesouraria_ids = map(lambda ct: ct.get('admfpg_conta_tesouraria_id'),
                                          adm_forma_pgto_dados) if adm_forma_pgto_dados else []
        if dados_nf.get('nf_propria') and all(admfpg_conta_tesouraria_ids):
            pass
        elif not self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'financeiro', [], context=context).get('finpar_conta_tesouraria_avista_id'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A conta caixa para recebimento a vista deve ser cadastrada nos '
                                   'parametros do Financeiro.'))

        if not destinatario_id and dados_nf.get('fisnfc_modelo_serie') != '02D' and \
                dados_nf.get('fisnfc_modelo') < '90':
            if not dados_nf.get('fisnfc_consumidor'):
                raise osv.except_osv(_('Atencao!'), _('Fornecedor ou cliente nao foi informado.'))

        # Verifica se modelo de nota fiscal foi informado
        if not dados_nf.get('nf_propria') and not dados_nf.get('modelo_nf_id'):  # Nota fiscal de terceiro
            raise osv.except_osv(_('Atencao!'), _('Modelo de nota fiscal nao foi informado.'))

        # Verifica itens
        if not dados_nf.get('estoque_movimento_itens_ids'):
            if dados_modelo.get('fismnf_tipo') != 'T':
                raise osv.except_osv(_('Atencao!'), _('Nenhum item inserido na nota.'))

        # Verifica destinatario (cliente ou fornecedor)
        if dados_nf.get('tipo') and destinatario_id and company_id:
            if not self.verificar_incluir_cli_for(
                    cr, uid, dados_nf.get('tipo'), destinatario_id, company_id,
                    'incluir', natureza_id, context=context):
                raise osv.except_osv(_('Atencao!'), _('E necessario cadastrar cliente/fornecedor'))

        # Verifica o endereco do destinatario
        if not dados_nf.get('destinatario_endereco_id') and \
                dados_nf.get('fisnfc_modelo_serie') != '02D':
            if not self.verifica_endereco_destinatario(
                    cr, uid, dados_nf.get('tipo'), destinatario_id, company_id,
                    context=context):
                raise osv.except_osv(_('Atencao!'),
                                     _('Endereco do cliente ou fornecedor nao esta cadastrado.'))
        if destinatario_id and not dados_pessoa_cliente.get('endereco_ids'):
            if dados_cfop_nat.get('codigo')[0] != '7':
                raise osv.except_osv(_('Atencao!'), _('Destinatario sem endereco para nota fiscal'))

        # Verifica o valor do desconto
        if dados_nf.get('vr_desconto') < 0 or dados_nf.get('vr_total_nf') < 0:
            raise osv.except_osv(_('Atencao!'),
                                 _('Valor do desconto invalido. O desconto deve ser maior que zero '
                                   'e menor ou igual ao valor total da nota.'))

        # Verifica divergencia de condicao de pagamento X parcelas
        dict_condicao = {}
        dict_forma_pgto = {}
        dict_cond_forma = {}
        dict_adm_cond = {}
        if dados_nf.get('nf_condicao_ids'):
            cond_forma_ids = []
            forma_pgto_ids = []
            for i in fiscal_condicao_pagamento_dados:
                dict_condicao[i.get('id')] = i
                cond_forma_ids.append(i.get('condicao_forma_id'))
                forma_pgto_ids.append(i.get('forma_pagto_id'))
            campos = ['cartao_credito_debito', 'admfpg_tipo', 'name', 'admfpg_documento_tipo_id']
            map(lambda x: dict_forma_pgto.update({x.get('id'): x}),
                self.pool.get('adm.forma.pgto').read(cr, uid, forma_pgto_ids, campos, context))
            adm_cond_ids = []
            campos = ['adm_condicao_id', 'taxa']
            for i in self.pool.get('adm.condicao.forma').read(
                    cr, uid, cond_forma_ids, campos, context, load):
                dict_cond_forma.update({i.get('id'): i})
                adm_cond_ids.append(i.get('adm_condicao_id'))
            campos = ['qtde_parcelas', 'tipo']
            map(lambda x: dict_adm_cond.update({x.get('id'): x}),
                self.pool.get('adm.condicao').read(cr, uid, adm_cond_ids, campos, context))
            self.check_condicao_total_parcelas(cr, uid, dict_condicao, dict_cond_forma,
                                               dict_adm_cond, context)

        # Verifica chave de acesso para nota fiscal eletronica
        if dados_modelo.get('fismnf_tipo_nfe') and dados_nf.get('nfe_chave_acesso') \
                and dados_nf.get('numero_nf'):
            # Trado desta forma para notas fiscais serie U por exemplo, estas vao para a sefaz
            # com a serie 0
            # ex: no chamado 98895
            if not dados_nf.get('serie_nf').isdigit():
                raise osv.except_osv(_('Atencao'),
                                     _('Serie invalida para este modelo de nota fiscal.'))

        # Verifica divergencias de frete
        self.fisnfc_verifica_divergencia_frete(cr, uid, dados_nf, dict_itens, dict_produtos,
                                               dict_trib, dict_cfop_itens, dados_cfop_nat,
                                               dados_nat_encargo, dict_encargos_nat,
                                               nat_encargo_ids, dados_pessoa_user,
                                               dados_pessoa_cliente, tipo_atividade, context)

        # Verifica parcelas e condicoes de pagamento e o saldo dos itens se nao houverem notas
        # fiscais de referencia
        dict_parcelas = {}
        campos = ['vencimento', 'valor']
        parcela_ids = dados_nf.get('nf_parcelas_ids') or \
                      dados_nf.get('fisnfc_parcelas_terceiros_ids') or []
        if parcela_ids:
            map(lambda x: dict_parcelas.update({x.get('id'): x}),
                self.pool.get('fiscal.nf.parcelas').read(cr, uid, parcela_ids, campos, context))
        if not dados_nf.get('fisnfc_nf_referenciada_ids') or dados_nf.get('tipo') == 'S':
            self.fisnfc_verificar_parcelas(
                cr, uid, dados_nf, dict_parcelas, dict_cfop_itens, context)
            self.fisnfc_verificar_condicao_pagamento(
                cr, uid, dados_nf, dict_condicao, dict_forma_pgto, context)
            self.fisnfc_verifica_itens_saldo(
                cr, uid, dados_nf, dict_itens, dict_produtos, dict_locais, local_grades, context)
        else:  # Se houver...
            # Verifica se as notas referenciadas teem modelo informado
            campos = ['modelo_nf_id', 'modelo_nf_series_id', 'numero_nf', 'serie_nf']
            for nf_referenciada in self.read(
                    cr, uid, dados_nf.get('fisnfc_nf_referenciada_ids'), campos, context,
                    load):
                if not nf_referenciada.get('modelo_nf_series_id') and not \
                        nf_referenciada.get('modelo_nf_id'):
                    raise osv.except_osv(_('Atencao!'),
                                         _('O modelo da nota fiscal referenciada %s/%s nao foi '
                                           'informado.') % (nf_referenciada.get('numero_nf'),
                                                            nf_referenciada.get('serie_nf')))

            # Se for nota fiscal de devolucao (com NF referenciada) tambem verifica parcelas e
            # condicao de pagamento
            if dados_cfop_nat.get('movimento_tipo') == 'D':
                self.fisnfc_verificar_parcelas(cr, uid, dados_nf, dict_parcelas, dict_cfop_itens,
                                               context)
                self.fisnfc_verificar_condicao_pagamento(cr, uid, dados_nf, dict_condicao,
                                                         dict_forma_pgto, context)

        # Verifica CFOP de nota fiscal de devolucao
        self.verificar_cfop_devolucao_transferencia(cr, uid, dados_cfop_nat, dict_itens,
                                                    dict_produtos, dict_cfop_itens, context)

        # Verifica se CST de PIS e COFINS dos itens sao iguais
        if context.get('terceiro') or context.get('nota_fiscal_saida'):
            self.fisnfc_verifica_cst_pis_cofins(cr, uid, dados_nf, dict_itens, dict_produtos,
                                                dict_trib, dict_encargos, dict_cst, context)

        if dados_nf.get('tipo') == 'S':  # Nota fiscal de saida
            # valida uso custificacao
            if not self.pool.get('adm.utils').admutl_check_modulo_instalado(
                    cr, uid, 'contratos', context):
                self.pool.get('gerencial.movimento').fingmo_valida_usar_centro_custo_evento(
                    cr, uid, context)

            # Verifica restricao de clientes somente para natureza onde o CFOP tenha o tipo de
            # movimento "compra/venda"
            if dados_cfop_nat.get('movimento_tipo') == 'C':
                erro_nf = u''
                obj_fiscal_utils = self.pool.get('fiscal.utils')

                # Verifica liberacao de desconto
                if not obj_fiscal_utils.fisutl_venda_liberacao_desconto(cr, uid, dados_nf, context):
                    erro_nf += obj_fiscal_utils.fisutl_get_erro()

                # Teste de restricao de venda. Nao testa para vendas "a vista e sem condicoes"
                _nf_condicao_pagto = 'V'
                for _condicao_pagto in dict_condicao.values():
                    forma_pgtop = dict_forma_pgto.get(_condicao_pagto.get('forma_pagto_id'))
                    cod_forma = dict_cond_forma.get(_condicao_pagto.get('condicao_forma_id'))
                    adm_cond = dict_adm_cond.get(cod_forma.get('adm_condicao_id'))
                    if (not forma_pgtop.get('cartao_credito_debito')) and \
                            (adm_cond.get('tipo') not in ['V', 'S']):
                        _nf_condicao_pagto = 'P'

                if _nf_condicao_pagto == 'P':
                    if not obj_fiscal_utils.fisutl_restricao_venda(cr, uid, dados_nf, context):
                        erro_nf += obj_fiscal_utils.fisutl_get_erro()
                if erro_nf:
                    raise osv.except_osv(_('Erro na verificacao da nota fiscal.'), erro_nf)

            if not dados_nf.get('fisnfc_nf_referencia_ids'):
                itens_custo_maior_precovenda = ''

                # Devolucao e transferencia nao testar
                if not self.pool.get('adm.utils').admutl_get_parametros(
                        cr, uid, 'estoque', 'estpar_pvenda_menor_pcusto', context=context):
                    if dados_cfop_nat.get('movimento_tipo') not in ('D', 'T'):
                        for item in dict_itens.values():
                            preco_venda = item.get('vr_total_item') / item.get('quantidade') or 0.0
                            produto = dict_produtos.get(item.get('produto_id'))
                            produto_name = produto.get('name')
                            if self.pool.get('estoque.preco.venda').estpve_verificar_custo_maior_pvenda(
                                    cr, uid, preco_venda, produto.get('id'), [], context):
                                itens_custo_maior_precovenda += '\n %s, \n' % produto_name

                if itens_custo_maior_precovenda:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Os produtos a seguir possuem preco de venda menor que '
                                           'o de custo: %s') % itens_custo_maior_precovenda)
            # Vendedor obrigatorio
            if self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'estoque', 'estpar_vendedor_obrigatorio', context=context).get(
                    'estpar_vendedor_obrigatorio') and not dados_nf.get('vendedor_id'):
                itens_sem_vendedor = filter(lambda isv: not isv.get('vendedor_id'),
                                            dict_itens.values())
                if itens_sem_vendedor:
                    itens_sem_vendedor = \
                        sorted(itens_sem_vendedor, key=lambda s: s.get('sequencia'))
                    raise osv.except_osv(_('Atencao!'),
                                         _('O vendedor deve ser informado para os itens de '
                                           'sequencia: ' + ', '.join(
                                             map(lambda s: str(s.get('sequencia')),
                                                 itens_sem_vendedor))))

            # Verifica desoneracao ICMS. Nao ha desoneracao se empresa for Simples Nacional
            company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, context=context)
            pessoa_id = self.pool.get('res.company').read(
                cr, uid, company_id, ['rescon_pessoa_id'], context, '_classic_write').get(
                'rescon_pessoa_id')
            simples = self.pool.get('adm.pessoa').read(
                    cr, uid, pessoa_id, ['admpes_jur_regime_tributario'], context).get(
                'admpes_jur_regime_tributario') == 'S'
            if simples and dados_nf.get('fisnfc_icms_desonerado'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Venda com desoneracao de ICMS nao pode ocorrer para '
                                       'empresa cadastrada no Simples Nacional.'))
        else:  # Nota fiscal de entrada
            # Verifica se o CFOP eh de bonificacao e nao permite validar notas com itens de
            # bonificacao que possuem PIS e COFINS tributados
            msg = ''
            for item_id in dados_nf.get('estoque_movimento_itens_ids'):
                item = dict_itens.get(item_id)
                produto = dict_produtos.get(item.get('produto_id'))
                cfop = dict_cfop_itens.get(item.get('cfop_id'))
                if cfop and cfop.get('codigo') in ('1910', '2910', '1911', '2911'):
                    for trib_id in item.get('tributacoes_ids'):
                        trib = dict_trib.get(trib_id)
                        encargo = dict_encargos.get(trib.get('encargo_id'))
                        cst = dict_cst.get(trib.get('cst_id'))
                        if encargo.get('sigla') in ('PIS', 'COFINS') and cst.get('tipo') == 'T':
                            msg += '{codigo} - {produto} -- Encargo: {encargo}.\n'.format(
                                codigo=produto.get('codigo'), produto=produto.get('name'),
                                encargo=encargo.get('sigla'))
            if msg:
                raise osv.except_osv(_('Atencao!'),
                                     _('Itens bonificados nao podem aproveitar o '
                                       'credito de PIS/COFINS, '
                                       'verifique os itens abaixo:\n' + msg))

            # Consistencia da guia
            if dados_nf.get('guia_ids'):
                campos = ['valor', 'data_vencimento', 'guia_recomp_aliq']
                dados_guia = self.pool.get('fiscal.guia').read(
                    cr, uid, dados_nf.get('guia_ids'), campos, context)
                for guia in filter(lambda f: not f.get('guia_recomp_aliq'), dados_guia):
                    if guia.get('valor') <= 0:
                        raise osv.except_osv(_('Atencao!'),
                                             _('O valor da guia com vencimento para o dia: %s deve '
                                               'ser maior que zero.' % guia.get('data_vencimento')))

            for item_id in dados_nf.get('estoque_movimento_itens_ids'):
                item = dict_itens.get(item_id)
                for trib_id in item.get('tributacoes_ids'):
                    trib = dict_trib.get(trib_id)
                    cst = dict_cst.get(trib.get('cst_id'))
                    encargo = dict_encargos.get(trib.get('encargo_id'))

                    # Verifica se item com tributacao diferente de substituicao tem valor de ST
                    if (encargo.get('sigla') or '').upper() == 'ICMS' and \
                            cst.get('tipo') not in ('U', 'TU'):
                        if trib.get('vr_base_icms_substituicao') or \
                                trib.get('vr_icms_substituicao'):
                            produto = dict_produtos.get(item.get('produto_id'))
                            raise osv.except_osv(_('Atencao!'),
                                                 _('O item %s apresenta valores de ICMS '
                                                   'substituicao, porem o CST nao e de '
                                                   'substituicao.') % produto.get('name'))

            # Verifica numeracao da nota fiscal para entrada de terceiro
            if not dados_nf.get('nf_propria'):
                if self.search(cr, uid, [('modelo_nf_id', '=', dados_nf.get('modelo_nf_id')),
                                         ('numero_nf', '=', dados_nf.get('numero_nf')),
                                         ('state', '!=', 'C'),
                                         ('serie_nf', '=', dados_nf.get('serie_nf')),
                                         ('id', '!=', nf),
                                         ('destinatario_id', '=', destinatario_id),
                                         ('tipo', '=', 'E')],
                               context=context):
                    raise osv.except_osv(_('Atencao!'),
                                         _('Nota fiscal numero %s e serie %s ja cadastrada.') %
                                         (dados_nf.get('numero_nf'), dados_nf.get('serie_nf')))

            # Nota fiscal de complemento de arroz permite apenas um item com quantidade igual a um
            # e se for NF-e a finalidade de emissao deve ser de ajuste
            if dados_natureza.get('fisnat_complemento_arroz'):
                if len(dict_itens) != 1:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Nota fiscal de complemento de arroz deve conter apenas '
                                           'um item.'))
                elif dict_itens.values()[0].get('quantidade') != 1:
                    raise osv.except_osv(_('Atencao!'),
                                         _('O item da nota fiscal de complemento de arroz nao pode '
                                           'ter quantidade diferente de 1 (um).'))
                elif dict_itens.values()[0].get('produto_id') != \
                        dados_natureza.get('fisnat_complemento_arroz_item_id'):
                    raise osv.except_osv(_('Atencao!'),
                                         _('O item da nota fiscal de complemento de arroz nao e  '
                                           'igual ao cadastrado na natureza da operacao.'))
                if dados_modelo.get('fismnf_tipo_nfe') and dados_nf.get('finalidade_emissao') != '3':
                    raise osv.except_osv(_('Atencao!'),
                                         _('Nota fiscal de complemento de arroz deve ter a '
                                           'finalidade de emissao igual a "Ajuste".'))

                # Verifica o parametro de codigo de ajuste do ICMS no Fiscal.
                if not self._parametros(cr, uid).fiscal.fispar_codigo_ajuste_icms_nf_arroz:
                    raise osv.except_osv(_('Atencao!'),
                                         _('O codigo de ajuste ICMS nao foi informado no parametro '
                                           'do Fiscal.'))

        return True

    @log_debug
    def fisnfc_criar_titulo(self, cr, uid, nf_id, values, context=None):
        """
        Cria um novo titulo.
        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: o ID da nota fiscal
        :param values: valores utilizados na criacao no titulo novo
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type nf_id: long
        :type values: dict
        :type context: dict
        :return: o id do novo titulo
        :rtype: long
        """
        context = context and context.copy() or {}
        if not values.get('vr_original'):
            return 0  # Nao gera titulo se nao houver valor
        if 'fintit_manter_prv' not in context.keys():
            context['fintit_manter_prv'] = True
        if len(values['data_emissao']) > 10:
            # Esta conversao eh feita pois quando a nota eh emitida apos as 21h estava colocando a
            # data do proximo dia
            values['data_emissao'] = datetime.strftime(self.pool.get('adm.utils').
                   admutl_converte_tz(cr, uid, datetime.strptime(
                values['data_emissao'], '%Y-%m-%d %H:%M:%S'), 'UTC', context=context), '%Y-%m-%d')

        return self.pool.get('financeiro.titulo').create(cr, uid, values, context)

    @log_debug
    def fisnfc_baixar_titulo(self, cr, uid, titulo_id, forma_pgto_id,
                             local_cobranca_id, vr_pago=0, cheque_id=False, cheque_numero='',
                             emitente_id=False, agencia_id=False, cheque_conta='',
                             nf_terceiro=False, company_id=False, context=None):
        """
        Baixa somente os titulos "a vista".
        :warning: MODULO ECF UTILIZA ESTA FUNCAO.
        """
        context = context and context.copy() or {}
        brw_titulo = self.pool.get('financeiro.titulo').browse(cr, uid, titulo_id, context)
        titulo_data_emissao = brw_titulo.data_emissao
        titulo_data_vencimento = brw_titulo.data_vencimento
        if not vr_pago:
            vr_pago = brw_titulo.saldo or 0
        obj_financeiro_utils = self.pool.get('financeiro')
        date_emissao = obj_financeiro_utils.financ_convert_str_date(
            str_data=titulo_data_emissao, str_data_name=u'emissao', context=context)
        date_vencimento = obj_financeiro_utils.financ_convert_str_date(
            str_data=titulo_data_vencimento, str_data_name=u'vencimento', context=context)
        if not date_emissao or not date_vencimento:
            msg = obj_financeiro_utils.financ_get_erro()
            raise osv.except_osv(_('Atencao'), msg)

        # Caso a forma de pagamento nao seja passada (situacao: NF de 3o., guia da NF), a forma de
        # pagamento utilizada serah dinheiro
        if not forma_pgto_id and nf_terceiro:
            # A forma de pagamento dinheiro eh criada pelo sistema.
            forma_pgto_id = self.pool.get('adm.forma.pgto').search(
                cr, uid, [('sigla', '=', 'DN')], context=context)[0]

        # Nao baixa titulos a prazo apenas para notas de saida
        if brw_titulo.fatura_id.nf_id.tipo == 'S' and date_emissao != date_vencimento:
            return True

        # Conta da tesouraria
        obj_adm_utils = self.pool.get('adm.utils')
        financeiro_parametro = ['finpar_conta_tesouraria_avista_id']
        financeiro_parametro_dado = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'financeiro', financeiro_parametro, company_id=company_id,
            context=context)
        finpar_conta_tesouraria_avista_id = \
            financeiro_parametro_dado.get('finpar_conta_tesouraria_avista_id')
        adm_forma_pgto_dados = self.pool.get('adm.forma.pgto').read(
            cr, uid, forma_pgto_id, ['admfpg_conta_tesouraria_id'], context, '_classic_write')
        if not nf_terceiro and adm_forma_pgto_dados.get('admfpg_conta_tesouraria_id'):
            conta_id = adm_forma_pgto_dados.get('admfpg_conta_tesouraria_id')
        elif finpar_conta_tesouraria_avista_id:
            conta_id = finpar_conta_tesouraria_avista_id
        else:
            raise osv.except_osv(_('Atencao'), _('A conta caixa para recebimento a vista deve '
                                                 'ser cadastrada nos parametros do Financeiro.'))

        titulos = [{'titulo_id': titulo_id, 'valor_pago': vr_pago,
                    'saldo_baixado': vr_pago,
                    'valor_multa': 0, 'valor_juros': 0,
                    'valor_desconto': 0, 'tipo_titulo': brw_titulo.tipo_titulo,
                    'pessoa_id': brw_titulo.pessoa_id.id}]

        res_baixa = obj_financeiro_utils.financ_baixar_titulo(
            cr, uid, futl_bxti_titulos=titulos, ftul_bxti_forma_pgto_id=forma_pgto_id,
            futl_bxti_data_pagamento=titulo_data_vencimento, futl_bxti_tesouraria_conta_id=conta_id,
            futl_bxti_baixa_multipla=False, futl_bxti_cheque_banco_id=cheque_id,
            futl_bxti_cheque_numero=cheque_numero, futl_bxti_cheque_conta=cheque_conta,
            futl_bxti_cheque_emitente_id=emitente_id, futl_bxti_agencia_id=agencia_id,
            futl_bxti_company_id=company_id, context=context)

        if not res_baixa:
            msg = obj_financeiro_utils.financ_get_erro()
            raise osv.except_osv(_('Atencao'), msg)

        return True

    # TODO: Mover este metodo para a classe financeiro.fatura
    @log_debug
    def criar_titulo_fatura_cartao(self, cr, uid, moeda_id, local_cobranca_id, dados_nf,
                                   dict_condicao, dict_forma_pgto, dict_cond_forma, dict_parcelas,
                                   previsao_doc_id=False, context=None):
        """
        Cria a fatura e o titulo para taxa de cartao credito/debito.
        """
        context = context and context.copy() or {}
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')
        operadora_pessoa_ids = []
        for condicao in dict_condicao.values():
            taxa = condicao.get('condicao_forma_id') and self.pool.get('adm.condicao.forma').read(
                cr, uid, condicao.get('condicao_forma_id'), ['taxa'], context).get('taxa')
            forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
            if not forma_pgto.get('cartao_credito_debito') or taxa <= 0:
                continue
            operadora_pessoa_id = forma_pgto.get('admfpg_cartao_pessoa_id')
            documento_tipo_id = previsao_doc_id or forma_pgto.get('admfpg_documento_tipo_id')
            vals_fatura_cartao = {
                'company_id': dados_nf.get('company_id'),
                'documento': str(dados_nf.get('numero_nf') or '').zfill(9),
                'finfat_documento_tipo_id': documento_tipo_id,
                'pessoa_id': operadora_pessoa_id,
                'data_emissao': dados_nf.get('data_emissao'),
                'tipo_fatura': 'P',
                'nf_id': dados_nf.get('id')}
            context.update({'ctx_fisnfc_titulo_create_sem_custificacao': True})

            # Cria apenas uma fatrua por operadora de cartoes
            if operadora_pessoa_id not in operadora_pessoa_ids:
                fatura_id = obj_financeiro_fatura.create(cr, uid, vals_fatura_cartao, context)
                operadora_pessoa_ids.append(operadora_pessoa_id)

            for parcela_id in condicao.get('nf_parcela_ids'):
                parcela = dict_parcelas.get(parcela_id)
                valor = parcela.get('valor')
                vencimento = parcela.get('vencimento')
                if parcela.get('vencimento') and valor > 0:
                    vals_taxa_cartao = {
                        'company_id': dados_nf.get('company_id'),
                        'pessoa_id': operadora_pessoa_id,
                        'local_cobranca_id': local_cobranca_id,
                        'adm_moeda_id': moeda_id,
                        'tipo_titulo': 'P',
                        'state': 'A',
                        'data_emissao': dados_nf.get('data_emissao'),
                        'tipo_lancamento': 'A',
                        'fatura_id': fatura_id,
                        'data_vencimento': vencimento,
                        'data_prorrogacao': vencimento,
                        'vr_original': valor * (taxa / 100.0),
                        'saldo': valor * (taxa / 100.0),
                        'doc_tipo_id': documento_tipo_id,
                        'documento': str(dados_nf.get('numero_nf') or ''),
                        'parcela': 98,
                    }
                    self.fisnfc_criar_titulo(
                        cr, uid, dados_nf.get('id'), vals_taxa_cartao, context=context)

        return True

    @log_debug
    def button_faturar(self, cr, uid, ids, context=None):
        """
        Fatura uma nota fiscal.

        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a nota fiscal foi faturada.
        :rtype: bool
        """
        context = context and context.copy() or {}
        return self.fisnfc_funcao_button_faturar(cr, uid, ids, context)

    @log_debug
    def fisnfc_funcao_button_faturar(self, cr, uid, ids, context=None):
        """
        Fatura uma nota fiscal.

        :param cr: o cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: dicionario com valores da nota fiscal faturada.
        :rtype: dict
        """
        context = context and context.copy() or {}
        cr.execute("""BEGIN;
                      SAVEPOINT fisnfe_faturar_nota;""")
        vals = {}
        brw_nf = self.browse(cr, uid, ids[0], context)
        if not brw_nf.fisnfc_nfce_contingencia_offline: # nao fatura contingencia offline pois ja foi faturada
            vals = self.fisnfc_faturar_nf_erp(cr, uid, ids, context)
        brw_nf = self.browse(cr, uid, ids[0], context)

        if brw_nf.modelo_nf_id.fismnf_tipo_nfe:
            if not brw_nf.fisnfc_tipo_servico:
                # O ENVIDO DA NOTA PARA A SEFAZ DEVE SER A ULTIMA OPERACAO
                vals.update(self.fisnfc_faturar_nota_fiscal_eletronica(
                    cr, uid, brw_nf.id, context))
                vals['nfe_versao_processo_emissao'] = self._parametros(
                    cr, uid).fiscal.fispar_nfe_versao_xml
                if brw_nf.nf_propria and brw_nf.data_saida_entrada < vals.get('data_emissao'):
                    if brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_codigo not in ('55', '65'):
                        vals['data_saida_entrada'] = vals.get('data_emissao')

                    # quando for 'D' e 'P', o savepoint ja foi desfeito
                if vals.get('state') not in ('3', 'D', 'P') and (brw_nf.modelo_nf_id.fismnf_codigo == u'55' or vals.get('nfe_forma_emissao') != '2'):
                    cr.execute("""ROLLBACK TO SAVEPOINT fisnfe_faturar_nota;""")
                    # nao faz o commit quando vem da importacao de NFC-e
                    if not context.get('fisnfc_importacao_nfce_do_supervisor'):
                        cr.execute('COMMIT;')
                elif vals.get('state') == '3':
                    if not vals.get('fisnfc_nfce_contingencia_offline'):
                        cr.execute("""RELEASE SAVEPOINT fisnfe_faturar_nota""")
                    # nao faz o commit quando vem da importacao de NFC-e
                    if not context.get('fisnfc_importacao_nfce_do_supervisor'):
                        cr.execute('COMMIT;')
            else:
                # Todo: Colocar aqui a rotina da nota de servico eletronica
                vals['state'] = '3'

        context['super_write_fiscal_nf_capa'] = True
        dados_nf = self.read(cr, uid, ids, ['state'], context)
        # teste implementado devido ao chamado 149445, a nota mudou de esboco para faturada, nao gerando saldo nem custo
        if dados_nf[0].get('state') == '1':
            raise osv.except_osv(_('Atencao!'),
                                 _('A nota em Esboco, recarregue a tela!'))

        self.write(cr, uid, ids, vals, context)
        return vals

    def fisnfc_faturar_nf_erp(self, cr, uid, ids, context=None):
        """Fatura a nota apenas no ERP, deixando a parte da SEFAZ, se for eletronica,
           para outro momento"""
        context = context and context.copy() or {}
        self.pool.get('estoque.movimento.itens').estmvi_valida_quantidade(cr, uid, ids[0], context)
        self.pool.get('valida.pendencias').valpen_verificar_licenca_ativa(cr, uid, context)
        context['super_write_fiscal_nf_capa'] = True  # Para compatibilidade. Remover se nao usado
        values = {}
        brw_nf = self.browse(cr, uid, ids[0], context)

        # Verifica se o ´´status´´ da nota fiscal permite faturar
        args = [('id', '=', brw_nf.id), ('state', 'in', ['2', '2E'])]
        if not self.search(cr, uid, args, context=context) and \
                not context.get('fisnfc_importacao_nfce_do_supervisor'):
            # no modulo 'ml' esta esperando um 'dict' no retorno desta funcao
            return {'state': brw_nf.state}

        if not context.get('fisnfc_importacao_nfce_do_supervisor') :
            # Verifica e valida a nota fiscal antes de faturar
            self.fisnfc_verificar_nota_fiscal(cr, uid, brw_nf.id, context)

        # Esta parte de custificacao deve ser calculada antes de faturar no SEFAZ, pois se houver
        # algum problema com parametro nao cadastrado a aplicacao aborta neste ponto, mas o
        # movimento gerencial sera gravado depois
        gerencial_movimento = self.pool.get('gerencial.movimento').custificar_nota_fiscal(
            cr, uid, brw_nf, context)

        # Fatura NF-e se nao for de servico
        # Notas que nao sao eletronicas
        if brw_nf.tipo == 'E' and brw_nf.nf_propria:
            agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            if brw_nf.data_emissao < agora:
                values['data_emissao'] = agora

        values['state'] = '3'
        values.update(self._fisnfc_faturar_nota_fiscal(cr, uid, brw_nf.id, values,
                                                       gerencial_movimento, context) or {})
        self.write(cr, uid, brw_nf.id, values, context)
        if values.get('estoque_movimento_itens_ids'):
            for item in values.get('estoque_movimento_itens_ids'):
                if item[2].get('ajuste_fiscal_ids'):
                    # retira os ajuste fiscal pois ele ja foi criado no write acima, e nao deve
                    # ser criado novamente ao passar em outro write da nota
                    item[2].pop('ajuste_fiscal_ids')

        return values

    def _fisnfc_faturar_nota_fiscal(self, cr, user, nf_id, values, gerencial_movimento,
                                    context=None):
        """
        Fatura uma nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param values: dados da NF a serem salvos.
        :param gerencial_movimento: dados do gerencial movimento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type values: dict
        :type gerencial_movimento: list
        :type context: dict
        :return: dados da nota fiscal faturada
        :rtype: dict
        """
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, nf_id, context)
        obj_adm_utils = self.pool.get('adm.utils')
        obj_fiscal_apuracao_imposto = self.pool.get('fiscal.apuracao.imposto')
        fiscal_parametro = ['fispar_codigo_ajuste_icms_nf_arroz',
                            'fispar_codigo_ajuste_icms_simples_nac', 'fispar_codigo_ajuste_icms_st']
        fiscal_parametro_dados = obj_adm_utils.admutl_get_parametros(
                cr, user, 'fiscal', fiscal_parametro, context=context)

        # Notas fiscais de complemento sao ignoradas
        if brw_nf.tipo == 'S':
            if not brw_nf.vr_total_nf or brw_nf.finalidade_emissao == 'C':
                return values

        # Este contexto nao permite custificar os titulos, pois a nota eh que deve ser custificada
        context['context_nao_custificar_titulos'] = True
        # o movimento financeiro deve ser gerado independente de ter cliente ou nao
        values.update(self.fisnfc_gerar_movimento_financeiro(cr, user, brw_nf.id,
                                                             gerencial_movimento, context))

        # Atualiza a embalagem do fornecedor
        self._fisnfc_atualizar_embalagem_fornecedor(cr, user, brw_nf.id, context)

        # Altera o ´status´ das notas de frete para "Faturada"
        values['fisnfc_nf_frete_ids'] = [[1, brw_nf_frete.id, {'state': '3'}]
                                         for brw_nf_frete in brw_nf.fisnfc_nf_frete_ids]

        # Ajusta vencimento das parcelas
        data_emissao = parse(obj_adm_utils.admutl_datetime_to_timezone(
            cr, user, parse(brw_nf.data_emissao), formato='%Y-%m-%d')).strftime('%Y-%m-%d')
        if brw_nf.modelo_nf_series_id and brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe:
            data_emissao = brw_nf.data_emissao[:10] or data_emissao
        if brw_nf.nf_propria:
            values['nf_condicao_ids'] = [[1, c.id, {'nf_parcela_ids': [
                [1, p.id, {'vencimento': data_emissao if p.vencimento < data_emissao
                else p.vencimento}]
                for p in c.nf_parcela_ids]}] for c in brw_nf.nf_condicao_ids]
        else:
            values['fisnfc_parcelas_terceiros_ids'] = [
                [1, p.id, {'vencimento': data_emissao if p.vencimento < data_emissao
                else p.vencimento}] for p in brw_nf.fisnfc_parcelas_terceiros_ids]

        # Fatura os pedidos
        if brw_nf.fisnfc_pedido_ids:
            context['pedcap_write_state'] = True
            self.pool.get('estoquepedidos.pedido.capa').write(
                cr, user, map(lambda m: m.id, brw_nf.fisnfc_pedido_ids), {'state': 'F'}, context)

        # Se o CFOP da natureza da operacao tiver tipo de movimento transferencia, e o destinatario
        # for empresa com a mesma raiz de CNPJ, deve ser criada uma nota fiscal de entrada de 3o.
        # para a empresa destino com ´status´ igual a validado. Somente para notas fiscais de saida
        if brw_nf.tipo == 'S' and brw_nf.destinatario_id.admpes_empresa_ids:
            self._fisnfc_criar_nf_transferencia_entrada_terceiro(
                cr, user, brw_nf.id, values, context)

        apuracao_ids = obj_fiscal_apuracao_imposto.search(
            cr, user, [('tipo', '=', 'C'), ('mes', '=', int(time.strftime('%m'))),
                       ('ano', '=', time.strftime('%Y'))], context=context)
        ajuste_fiscal_ids = []
        infcomp_fisco = ''

        if brw_nf.fisnfc_valor_icms_desonerado and brw_nf.natureza_id.\
                fisnat_codigo_ajuste_desonerado and brw_nf.tipo == 'S':
            for item in brw_nf.estoque_movimento_itens_ids:
                tributacoes_icms_desonerado = filter(
                    lambda t: t.encargo_id.sigla.upper() == 'ICMS' and t.fisnit_vr_icms_desonerado,
                    item.tributacoes_ids)
                for trib in tributacoes_icms_desonerado:
                    ajuste_fiscal_ids.append([1, item.id, {'ajuste_fiscal_ids': [[
                        0, False,
                        {'fisaji_codigo_ajuste': brw_nf.natureza_id.fisnat_codigo_ajuste_desonerado,
                         'fisaji_aliquota_ajuste': trib.aliquota,
                         'fisaji_valor_ajuste': trib.fisnit_vr_icms_desonerado,
                         'fisaji_valor_base_ajuste': trib.vr_base_calculo,
                         'fisaji_valor_outros': 0.0}]]}])

        # Nota fiscal de complemento de arroz. Ajusta valor de informacoes complementares do fisco
        if brw_nf.natureza_id.fisnat_complemento_arroz:
            # Verifica o parametro de codigo de ajuste do ICMS no Fiscal.
            codigo_ajuste_icms = \
                self._parametros(cr, user).fiscal.fispar_codigo_ajuste_icms_nf_arroz
            if not codigo_ajuste_icms:
                raise osv.except_osv(_('Atencao!'),
                                     _('O codigo de ajuste ICMS nao foi informado no parametro do '
                                       'Fiscal.'))
            infcomp_fisco += _('Antecipacao do imposto, nos termos do art. 4º da Parte 1 do Anexo '
                               'XVI do RICMS.\n')

            # Ajuste fiscal do item de complemento
            brw_tributacao = filter(lambda f: f.encargo_id.sigla.upper() == 'ICMS',
                                    brw_nf.estoque_movimento_itens_ids[0].tributacoes_ids)
            brw_tributacao = brw_tributacao and brw_tributacao[0]
            if brw_tributacao:
                ajuste_fiscal_ids += [
                [1, brw_nf.estoque_movimento_itens_ids[0].id, {'ajuste_fiscal_ids': [[
                    0, False, {'fisaji_codigo_ajuste': 'MG10000004',
                               'fisaji_aliquota_ajuste': brw_tributacao.aliquota,
                               'fisaji_valor_ajuste': brw_nf.estoque_movimento_itens_ids[
                                   0].vr_total_item,
                               'fisaji_valor_base_ajuste': brw_tributacao.vr_base_icms_substituicao,
                               'fisaji_valor_outros': 0}]]}]]

            # Apuracao de ICMS
            if brw_tributacao:
                outros_creditos_ids = [[
                    0, False, {'descricao': _('Antecipacao de arroz'),
                               'fisapuoc_codigo_ajuste': codigo_ajuste_icms,
                               'fisapuoc_tipo_utilizacao': 'MG04',
                               'fisapuoc_numero_doc': brw_nf.numero_nf,
                               'fisapuoc_chave_doc_eletronico': brw_nf.nfe_chave_acesso,
                               'fisapuoc_nf_id': nf_id,
                               'valor': brw_nf.vr_total_nf}]]
                if apuracao_ids:
                    obj_fiscal_apuracao_imposto.write(
                        cr, user, apuracao_ids[0], {'outros_creditos_ids': outros_creditos_ids},
                        context)
                else:
                    data = obj_adm_utils.admutl_datetime_to_timezone(cr, user,
                                                  parse(brw_nf.data_saida_entrada), formato='%Y-%m')
                    ano = data.split('-')[0]
                    mes = data.split('-')[1]
                    obj_fiscal_apuracao_imposto.create(
                        cr, user, {'tipo': 'C', 'mes': int(mes),
                                   'ano': ano,
                                   'outros_creditos_ids': outros_creditos_ids}, context)
        if brw_nf.tipo == 'E' and brw_nf.nf_propria:
            for item in brw_nf.estoque_movimento_itens_ids:
                for trib in item.tributacoes_ids:
                    if trib.encargo_id.sigla.upper() != 'ICMS' or \
                            not trib.fisnit_vr_fcp_st:
                        continue
                    ajuste_fiscal_ids.append([1, item.id, {'ajuste_fiscal_ids': [[
                            0, False, {'fisaji_codigo_ajuste': 'MG91990018',
                                       'fisaji_aliquota_ajuste': trib.tributacao_id.fistri_aliquota_fcp,
                                       'fisaji_valor_ajuste': trib.fisnit_vr_fcp or trib.fisnit_vr_fcp_st,
                                       'fisaji_valor_base_ajuste': trib.vr_base_icms_substituicao,
                                       'fisaji_valor_outros': 0}
                                             ]]}])
        if brw_nf.fisnfc_parametro_e200:
            e200_grade_ids = []
            for item in filter(lambda it: it.estmvi_sped_fiscal_e200_manual,
                               brw_nf.estoque_movimento_itens_ids):
                if not fiscal_parametro_dados.get('fispar_codigo_ajuste_icms_st'):
                    raise osv.except_osv(
                        _('Atencao'), _('Campo "Codigo de ajuste do ICMS ST" nao encontra-se '
                                        'cadastrado nos parametros do fiscal.'))
                if item.grade_id.id in e200_grade_ids:
                    continue
                e200_grade_ids.append(item.grade_id.id)
                ajuste_fiscal_ids.append([1, item.id, {'ajuste_fiscal_ids': [[0, False, {
                    'fisaji_codigo_ajuste': fiscal_parametro_dados.get(
                        'fispar_codigo_ajuste_icms_st')}]]}])

        empresa_lucro_real = brw_nf.company_id.rescon_pessoa_id.admpes_jur_regime_tributario == 'R'
        fornecedor_simples_nacional = brw_nf.destinatario_id.admpes_jur_regime_tributario == 'S'
        tributacao_ids = list()
        map(lambda tri: tributacao_ids.extend(tri.tributacoes_ids),
            brw_nf.estoque_movimento_itens_ids)
        vr_credito_icms = sum(map(
            lambda tri: tri.vr_encargo or 0, filter(
                lambda e: e.encargo_id.sigla.upper() == 'ICMS', tributacao_ids)))
        if all([not brw_nf.nf_propria, empresa_lucro_real, fornecedor_simples_nacional,
                vr_credito_icms]):
            if not fiscal_parametro_dados.get('fispar_codigo_ajuste_icms_simples_nac'):
                raise osv.except_osv(
                    _('Atencao'),
                    _("- Empresa: Lucro Real.\n - Fornecedor: Simples Nacional.\n "
                      "- Nota Fiscal possui credito de ICMS.\n"
                      "Deve ser cadastrado nos parametros do fiscal o campo "
                      "'Codigo de ajuste do ICMS p/ compras Simples Nac.'"))

            infcomp_fisco += _('Permite aproveitamento de credito.\n')

            # Ajuste fiscal dos itens
            for movimento in brw_nf.estoque_movimento_itens_ids:
                brw_tributacao = filter(lambda f: f.encargo_id.sigla.upper() == 'ICMS',
                                        movimento.tributacoes_ids)
                if not brw_tributacao:
                    continue
                brw_tributacao = brw_tributacao and brw_tributacao[0]
                ajuste_fiscal_ids += [
                    [1, movimento.id, {'ajuste_fiscal_ids': [[
                        0, False, {'fisaji_codigo_ajuste': 'MG10990505',
                                   'fisaji_aliquota_ajuste': brw_tributacao.aliquota or 0.0,
                                   'fisaji_valor_ajuste': brw_tributacao.vr_encargo or 0.0,
                                   'fisaji_valor_base_ajuste': brw_tributacao.vr_base_calculo or 0.0,
                                   'fisaji_valor_outros': 0}]]}]]
            outros_creditos_ids = [[
                0, False, {'descricao': 'S.Nacional',
                           'fisapuoc_codigo_ajuste': fiscal_parametro_dados.get(
                               'fispar_codigo_ajuste_icms_simples_nac'),
                           'fisapuoc_tipo_utilizacao': 'MG04',
                           'fisapuoc_numero_doc': brw_nf.numero_nf,
                           'fisapuoc_chave_doc_eletronico': brw_nf.nfe_chave_acesso or '',
                           'fisapuoc_nf_id': nf_id, 'valor': vr_credito_icms}]]
            if apuracao_ids:
                obj_fiscal_apuracao_imposto.write(
                    cr, user, apuracao_ids[0], {'outros_creditos_ids': outros_creditos_ids},
                    context)
            else:
                data = obj_adm_utils.admutl_datetime_to_timezone(cr, user,
                                              parse(brw_nf.data_saida_entrada), formato='%Y-%m')
                ano = data.split('-')[0]
                mes = data.split('-')[1]
                obj_fiscal_apuracao_imposto.create(
                    cr, user, {'tipo': 'C', 'mes': int(mes),
                               'ano': ano,
                               'outros_creditos_ids': outros_creditos_ids}, context)
        if ajuste_fiscal_ids:
            values['estoque_movimento_itens_ids'] = ajuste_fiscal_ids
        if infcomp_fisco:
            values['fisnfc_infcomp_fisco'] = infcomp_fisco

        return values

    def _fisnfc_atualizar_embalagem_fornecedor(self, cr, user, nf, context=None):
        """
        Atualiza a embalagem do fornecedor em funcao das embalagens dos itens.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: verdadeiro se as embalagens forem atualizadas no vendedor.
        :rtype: bool
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        obj_est_fornecedor = self.pool.get('estoque.fornecedor')
        obj_prod_embalagem = self.pool.get('estoque.produto.embalagem')
        dados_nf = self.read(
            cr, user, nf, ['estoque_movimento_itens_ids', 'destinatario_id', 'tipo'], context, load)
        if dados_nf.get('tipo') == 'S': # atualiza embalagem apenas para notas de entrada
            return True
        pessoa_id = dados_nf.get('destinatario_id')
        dados_itens = self.pool.get('estoque.movimento.itens').read(
            cr, user, dados_nf.get('estoque_movimento_itens_ids'),
            ['produto_id', 'produto_embalagem_id'], context, load)
        fornecedor_ids = self.pool.get('financeiro.fornecedor').search(
            cr, user, [('pessoa_id', '=', pessoa_id)], context=context)
        produto_ids = map(lambda x: x.get('produto_id'), dados_itens)
        estoque_fornecedor_ids = obj_est_fornecedor.search(
            cr, user, [('fornecedor_id', '=', fornecedor_ids and fornecedor_ids[0]),
                       ('produto_id', '=', produto_ids)], context=context)
        ids_produto_forncecedor  = {}
        map(lambda x: ids_produto_forncecedor.update({x.get('produto_id'): x.get('id')}),
            obj_est_fornecedor.read(
                cr, user,estoque_fornecedor_ids, ['produto_id'], context, load))
        prod_emb_ids = map(lambda x: x.get('produto_embalagem_id'), dados_itens)
        ids_prod_emb_embalagem = {}
        map(lambda x: ids_prod_emb_embalagem.update({x.get('id'):x.get('embalagem_id')}),
            obj_prod_embalagem.read(cr, user, prod_emb_ids, ['embalagem_id'], context, load))
        for item in dados_itens:
            obj_est_fornecedor.write(
                cr, user, ids_produto_forncecedor.get(item.get('produto_id')), {
                    'embalagem_id': ids_prod_emb_embalagem.get(item.get('produto_embalagem_id'))},
                context=context)

        return True

    def _fisnfc_criar_nf_transferencia_entrada_terceiro(self, cr, user, nf_id, values_nf_origem,
                                                        context=None):
        """
        Cria uma nota fiscal de entrada de terceiro para a empresa destinatario se o CFOP da
        natureza da operacao da nota fiscal (origem) tiver tipo de movimento transferencia e o
        destinatario tiver a mesma raiz de CNPJ.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nota fiscal
        :param values_nf_origem: dados a serem salvos da NF de origem.
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type values_nf_origem: dict
        :type context: dict
        :return: o ID da nota fiscal que foi criada.
        :rtype: long
        """
        context = context and context.copy() or {}
        context['fisnfc_nf_tranferencia'] = True  # Usado ao criar uma tributacao
        obj_adm_utils = self.pool.get('adm.utils')
        fiscal_parametros = obj_adm_utils.admutl_get_parametros

        def _get_tributacao_entrada(encargo_id, cst_id, tributacao_id):
            """Retorna a tributacao de entrada em funcao do CST de saida"""
            # Dados da tributacao de saida
            brw_tributacao = self.pool.get('fiscal.tributacao').browse(
                cr, user, tributacao_id, context)

            cst_entrada_id = _get_cst_entrada(encargo_id, cst_id, natureza_id)
            tributacao_ids = self.pool.get('fiscal.tributacao').search(
                cr, user, [('encargo_id', '=', encargo_id), ('cst_id', '=', cst_entrada_id),
                           ('base', '=', brw_tributacao.base),
                           ('aliquota', '=', brw_tributacao.aliquota)], context=context)

            return tributacao_ids and tributacao_ids[0] or False

        def _get_cst_entrada(encargo_id, cst_id, natureza_id):
            """Retorna o CST de entrada baseado no encargo e CST de saida"""
            cst_ids = self.pool.get('fiscal.cst.correlacao').search(
                cr, user, [('encargo_id', '=', encargo_id), ('cst_saida_id', '=', cst_id)],
                context=context)
            cst_entrada_dados = cst_ids and self.pool.get('fiscal.cst.correlacao').read(
                cr, user, cst_ids[0], ['cst_entrada_id'], context, '_classic_write') or {}
            cst_entrada_id = cst_entrada_dados.get('cst_entrada_id') or cst_id

            # Verifica sobreposicao (natureza)
            natureza_ids = self.pool.get('fiscal.cfop.natureza.encargos').search(
                cr, user, [('natureza_id', '=', natureza_id), ('encargo_id', '=', encargo_id)],
                context=context)
            natureza_id = natureza_ids and natureza_ids[0]
            sobreposicao = natureza_id and self.pool.get('fiscal.cfop.natureza.encargos').read(
                cr, user, natureza_id, ['fiscne_sobrepoe_so_tributados', 'sobrepor_tributacao_id'],
                context, '_classic_write') or {}
            tributacao_id = sobreposicao and sobreposicao.get('sobrepor_tributacao_id')
            if tributacao_id:
                tributacao_dados = self.pool.get('fiscal.tributacao').read(
                    cr, user, tributacao_id, ['cst_id'], context, '_classic_write')
                cst_sobreposicao_id = tributacao_dados and tributacao_dados.get('cst_id')
                if sobreposicao.get('fiscne_sobrepoe_so_tributados'):
                    cst_dados = cst_sobreposicao_id and self.pool.get('fiscal.cst').read(
                        cr, user, cst_sobreposicao_id, ['tipo'], context)
                    if cst_dados.get('tipo') in ('T', 'TU'):
                        cst_entrada_id = cst_sobreposicao_id or cst_entrada_id
                else:
                    cst_entrada_id = cst_sobreposicao_id or cst_entrada_id

            return cst_entrada_id

        # Dados da nota fiscal de origem
        brw_nf = self.browse(cr, user, nf_id, context)

        # Verifica se o CFOP da natureza da operacao tem ou nao tipo de movimento transferencia
        if brw_nf.natureza_id.cfop_id.movimento_tipo != 'T':
            return 0

        # Define os valores da nota fiscal de entrada de terceiro
        # cfop do item de substituicao tributaria
        cfop_ids_subst = self.pool.get('fiscal.cfop').search(
            cr, user, [('codigo', '=', '1409')], context=context)

        company_ids = brw_nf.destinatario_id.admpes_empresa_ids
        context['sig_force_company'] = True
        natureza_id = fiscal_parametros(
            cr, user, 'fiscal', 'fispar_natureza_transferencia_entrada_id',
            company_ids and company_ids[0].id, context).get(
            'fispar_natureza_transferencia_entrada_id')
        del context['sig_force_company']
        if not natureza_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Natureza para entrada por transferencia nao se encontra '
                                   'cadastrada nos parametros do fiscal'))
        cfop_id = self.pool.get('fiscal.cfop.natureza').browse(
            cr, user, natureza_id, context).cfop_id.id

        # Dados da capa
        endereco_id = brw_nf.company_id.rescon_pessoa_id.endereco_nf_id.id or \
                      brw_nf.destinatario_id.endereco_ids[0]
        values = {'company_id': company_ids and company_ids[0].id,
                  'data_emissao': datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S'),
                  'data_saida_entrada': datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S'),
                  'tipo': 'E', 'state': '2', 'prazo': '0',
                  'destinatario_id': brw_nf.company_id.rescon_pessoa_id.id, 'prazo_tipo': 'DD',
                  'modelo_nf_id': brw_nf.modelo_nf_series_id.modelo_nf_id.id,
                  'destinatario_endereco_id': endereco_id,
                  'modelo_55': brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe,
                  'numero_nf': brw_nf.numero_nf, 'serie_nf': brw_nf.modelo_nf_series_id.serie_nf,
                  'natureza_id': natureza_id}

        # Se for nota fiscal de origem for modelo 55 (eletronica)
        if brw_nf.modelo_nf_series_id.modelo_nf_id.fismnf_tipo_nfe:
            values.update(
                {'nfe_chave_acesso': brw_nf.nfe_chave_acesso or values_nf_origem.get(
                    'nfe_chave_acesso'),
                 'nfe_processo_emissao': brw_nf.nfe_processo_emissao,
                 'nfe_versao_processo_emissao': brw_nf.nfe_versao_processo_emissao,
                 'nfe_forma_emissao': brw_nf.nfe_forma_emissao or values_nf_origem.get(
                     'nfe_forma_emissao'),
                 'finalidade_emissao': brw_nf.finalidade_emissao,
                 'nfe_lote': brw_nf.nfe_lote or values_nf_origem.get('nfe_lote'),
                 'nfe_forma_impressao_danfe': brw_nf.nfe_forma_impressao_danfe,
                 'nfe_protocolo': brw_nf.nfe_protocolo or values_nf_origem.get('nfe_protocolo'),
                 'nfe_status': brw_nf.nfe_status or values_nf_origem.get('nfe_status'),
                 'fisnfc_nfe_situacao_nfe_motivo': brw_nf.nfe_status or values_nf_origem.get(
                     'nfe_status'),
                 'nfe_xml_auto': brw_nf.nfe_xml_auto or values_nf_origem.get('nfe_xml_auto')
                 }
            )

        # Itens e respectivas tributacoes
        context_sig_force = context.copy()
        context_sig_force['sig_force_company'] = True
        estpar_ids = self.pool.get('estoque.parametro').search(
            cr, user, [('company_id', '=', company_ids[0].id)], context=context_sig_force)
        estoque_parametro_dados = self.pool.get('estoque.parametro').read(
            cr, user, estpar_ids[0],
            ['estpar_local_compra_padrao_id', 'estpar_local_entrada_transf_padrao_id'], context,
            '_classic_write')
        entrada_local_id = estoque_parametro_dados.get('estpar_local_entrada_transf_padrao_id') or \
            estoque_parametro_dados.get('estpar_local_compra_padrao_id')
        values.update({'estoque_movimento_itens_ids': [[
            0, False,
            {'sequencia': i.sequencia,
             'local_id': entrada_local_id or i.local_id.id,
             'produto_id': i.produto_id.id,
             'grade_id': i.grade_id.id, 'produto_embalagem_id': i.produto_embalagem_id.id,
             'quantidade': i.quantidade, 'vr_unitario': i.vr_unitario,
             'vr_desconto': i.vr_desconto,
             'cfop_id': cfop_id if i.cfop_id.codigo != '5409' else cfop_ids_subst and
                                                                   cfop_ids_subst[0],
             'capa_id': i.capa_id.id, 'vr_total_item': i.vr_total_item,
             'natureza_id': i.natureza_id.id, 'vr_seguro_rateado': i.vr_seguro_rateado,
             'vr_despesas_rateado': i.vr_despesas_rateado,
             'vr_total_item_rateado': i.vr_total_item_rateado,
             'vr_frete_rateado': i.vr_frete_rateado,
             'estmvi_vr_icms_frete_rateado': i.estmvi_vr_icms_frete_rateado,
             'vr_preco_venda': i.vr_preco_venda, 'vr_desconto_rateado': i.vr_desconto_rateado,
             'estmvi_vr_despesas_extras_rateado': i.estmvi_vr_despesas_extras_rateado,
             'tributacoes_ids': [[
                 0, False, {'encargo_id': t.encargo_id.id,
                            'cst_id': _get_cst_entrada(
                                t.encargo_id.id, t.cst_id.id, values.get('natureza_id')),
                            'tributacao_id': _get_tributacao_entrada(
                                t.encargo_id.id, t.cst_id.id, t.tributacao_id.id),
                            'aliquota': t.aliquota, 'vr_base_calculo': t.vr_base_calculo,
                            'vr_encargo': t.vr_encargo, 'vr_reducao_base': t.vr_reducao_base,
                            'modbcst': t.modbcst, 'vr_icms_substituicao': t.vr_icms_substituicao,
                            'percentual_mva': t.percentual_mva,
                            'vr_base_icms_substituicao': t.vr_base_icms_substituicao,
                            'aliquota_venda': t.aliquota_venda}] for t in i.tributacoes_ids]}]
                                                       for i in
                                                       brw_nf.estoque_movimento_itens_ids]})

        # Complemento
        values.update({'quantidade_volume': brw_nf.quantidade_volume, 'especie': brw_nf.especie,
                       'marca': brw_nf.marca, 'numero_volume': brw_nf.numero_volume,
                       'peso_bruto': brw_nf.peso_bruto, 'peso_liquido': brw_nf.peso_liquido})

        # Frete
        values.update(
            {'frete': brw_nf.frete, 'vr_frete': brw_nf.vr_frete, 'tipo_frete': brw_nf.tipo_frete})

        # Valores fiscais
        values.update({'vr_total_produtos': brw_nf.vr_total_produtos,
                       'vr_total_servicos': brw_nf.vr_total_servicos, 'vr_seguro': brw_nf.vr_seguro,
                       'vr_despesas_acessorias': brw_nf.vr_despesas_acessorias,
                       'vr_ipi': brw_nf.vr_ipi, 'vr_desconto': brw_nf.vr_desconto,
                       'vr_base_icms_substituicao': brw_nf.vr_base_icms_substituicao,
                       'vr_icms_substituicao': brw_nf.vr_icms_substituicao})

        # Parcelas
        values.update({'fisnfc_parcelas_terceiros_ids': [
            [0, False, {'vencimento': p.vencimento, 'valor': p.valor}]
            for p in brw_nf.nf_parcelas_ids]})

        # Garante nota fiscal de terceiro para a empresa destinatario
        context['proprio'] = False  # Para compatibilidade. Remover se nao utilizar
        context['terceiro'] = True  # Para compatibilidade. Remover se nao utilizar
        context['sig_force_company'] = True  # Forca a empresa a ser o destinatario

        return self.create(cr, user, values, context)

    @log_debug
    def fisnfc_gerar_movimento_financeiro(self, cr, user, nf_id, gerencial_movimento, context=None):
        """
        Realiza o movimento financeiro em funcao dos itens da nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nota fiscal
        :param gerencial_movimento: dados do movimento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type gerencial_movimento: list
        :type context: dict
        :return: valores ajustados da nota fiscal.
        :rtype: dict
        """
        context = context and context.copy() or {}
        parametro = self._parametros(cr, user)

        # Dados da nota fiscal
        load = '_classic_write'
        campos = ['fisnfc_modelo_serie', 'destinatario_id', 'tipo', 'company_id', 'data_emissao',
                  'natureza_id', 'fisnfc_contrato_id', 'data_saida_entrada', 'deducao_itens_ids',
                  'estoque_movimento_itens_ids', 'numero_nf', 'nf_condicao_ids', 'nf_propria',
                  'vendedor_id', 'vendedor_externo_id', 'guia_ids', 'tipo_frete',
                  'fisnfc_infcomp_contribuinte', 'fisnfc_parcelas_terceiros_ids', 'fisnfc_modelo',
                  'transportador_id', 'fisnfc_nf_devolucao', 'modelo_nf_id', 'serie_nf',
                  'nfe_forma_emissao']
        values = {}  # Valores a serem atribuidos aa nota fiscal
        dados_nf = self.read(cr, user, nf_id, campos, context, load)
        data_atualiza_saldo = time.strftime('%Y-%m-%d')
        data_emissao = dados_nf.get('data_emissao')
        eletronica = self.pool.get('fiscal.modelo.nf').read(cr, user, dados_nf.get('modelo_nf_id'), ['fismnf_tipo_nfe'],
                                                              context=context).get('fismnf_tipo_nfe')
        if (dados_nf.get('nfe_forma_emissao') != '2' and
                not context.get('fisnfc_importacao_nfce_do_supervisor') and
                dados_nf.get('tipo') == 'S' and eletronica and
                not context.get('ctx_fiscal_nao_alterar_emissao')):
            data_atualiza_saldo = time.strftime('%Y-%m-%d')
            dados_nf['data_emissao'] = time.strftime('%Y-%m-%d %H:%M:%S')
            values['data_emissao'] = time.strftime('%Y-%m-%d %H:%M:%S')
            self.write(cr, user, dados_nf.get('id'),
                       {'data_emissao': dados_nf['data_emissao']}, context)
            data_emissao = dados_nf['data_emissao']

        natureza_id = dados_nf.get('natureza_id')
        campos = ['fisnat_operacao_id']
        dados_natureza = self.pool.get('fiscal.cfop.natureza').read(
            cr, user, natureza_id, campos, context, load)
        operacao_id = dados_natureza.get('fisnat_operacao_id')
        if not operacao_id:
            raise osv.except_osv(_('Atencao!'),
                                 _('Cadastre a operacao para a natureza da nota.'))
        campos = ['estnto_gerar_financeiro', 'estnto_atualizar_saldo']
        dados_operacao = self.pool.get('estoque.natureza.operacao').read(
            cr, user, operacao_id, campos, context)
        dados_modelo = {}
        if dados_nf.get('modelo_nf_id'):
            dados_modelo = self.pool.get('fiscal.modelo.nf').read(
                cr, user, dados_nf.get('modelo_nf_id'), ['fismnf_tipo_nfe'], context)
        obj_encargo = self.pool.get('fiscal.encargo')
        encargo_ids = obj_encargo.search(cr, user, [], context=context)
        dict_encargos = {}
        map(lambda x: dict_encargos.update({x.get('id'): x}),
            obj_encargo.read(cr, user, encargo_ids, ['sigla'], context))
        dict_guias = {}
        campos = ['data_vencimento', 'valor', 'fisgui_fornecedor_reembolsar', 'local_cobranca_id',
                  'agencia_id', 'fisgui_fornecedor_vencimento']
        if dados_nf.get('guia_ids'):
            map(lambda x: dict_guias.update({x.get('id'): x}), self.pool.get('fiscal.guia').read(
                cr, user, dados_nf.get('guia_ids'), campos, context, load))
        dict_itens = {}
        tri_ids = []
        item_cfop_ids = []
        produto_ids = []
        grade_ids = []
        local_ids = []
        pro_embalagem_id = []
        campos = ['cfop_id', 'quantidade', 'grade_id', 'produto_id', 'estmvi_volume', 'local_id',
                  'tributacoes_ids', 'produto_embalagem_id', 'vr_total_item_rateado']
        for i in self.pool.get('estoque.movimento.itens').read(
                cr, user, dados_nf.get('estoque_movimento_itens_ids'), campos, context, load):
            dict_itens[i.get('id')] = i
            tri_ids.extend(i.get('tributacoes_ids'))
            item_cfop_ids.append(i.get('cfop_id'))
            produto_ids.append(i.get('produto_id'))
            grade_ids.append(i.get('grade_id'))
            local_ids.append(i.get('local_id'))
            pro_embalagem_id.append(i.get('produto_embalagem_id'))
        dict_tribs = {}
        campos = ['vr_icms_substituicao', 'encargo_id', 'vr_encargo', 'cst_id', 'tributacao_id',
                  'percentual_mva', 'valor_pauta', 'modbcst', 'aliquota_venda']
        map(lambda x: dict_tribs.update({x.get('id'): x}),
            self.pool.get('fiscal.nf.item.tributacao').read(
                cr, user, tri_ids, campos, context, load))
        dict_cfop_itens = {}
        campos = ['codigo']
        map(lambda x: dict_cfop_itens.update({x.get('id'): x}),
            self.pool.get('fiscal.cfop').read(cr, user, item_cfop_ids, campos, context))
        dict_condicao = {}
        dict_forma_pgto = {}
        dict_cond_forma = {}
        dict_parcelas = {}
        if dados_nf.get('nf_condicao_ids') or dados_nf.get('fisnfc_parcelas_terceiros_ids'):
            condicao_ids = dados_nf.get('nf_condicao_ids') or []
            if not dados_nf.get('nf_condicao_ids'):
                parcela_ids = dados_nf.get('fisnfc_parcelas_terceiros_ids')

                if parcela_ids:
                    campos = ['nf_condicao_id', 'valor', 'vencimento']
                    for i in self.pool.get('fiscal.nf.parcelas').read(
                            cr, user, parcela_ids, campos, context, load):
                        dict_parcelas.update({i.get('id'): i})
                        condicao_ids and condicao_ids.append(i.get('nf_condicao_id'))
            cond_forma_ids = []
            forma_pgto_ids = []
            parcela_ids = []
            campos = ['condicao_forma_id', 'forma_pagto_id', 'fiscpg_chq_banco', 'nf_parcela_ids',
                      'fiscpg_chq_agencia', 'fiscpg_chq_numero', 'fiscpg_chq_conta',
                      'fiscpg_chq_emitente', 'valor']
            for i in self.pool.get('fiscal.condicao.pagamento').read(cr, user, condicao_ids, campos,
                                                                     context, load):
                dict_condicao[i.get('id')] = i
                cond_forma_ids.append(i.get('condicao_forma_id'))
                forma_pgto_ids.append(i.get('forma_pagto_id'))
                i.get('nf_parcela_ids') and parcela_ids.extend(i.get('nf_parcela_ids'))
            campos = ['cartao_credito_debito', 'admfpg_tipo', 'name', 'admfpg_documento_tipo_id',
                      'admfpg_cartao_pessoa_id', 'admfpg_boleto', 'admfpg_local_cobranca_id',
                      'admfpg_natureza_id']
            map(lambda x: dict_forma_pgto.update({x.get('id'): x}),
                self.pool.get('adm.forma.pgto').read(
                    cr, user, forma_pgto_ids, campos, context, load))
            campos = ['adm_condicao_id']
            for i in self.pool.get('adm.condicao.forma').read(
                    cr, user, cond_forma_ids, campos, context, load):
                dict_cond_forma.update({i.get('id'): i})
            if not dict_parcelas:
                campos = ['nf_condicao_id', 'valor', 'vencimento']
                map(lambda x: dict_parcelas.update({x.get('id'): x}),
                    self.pool.get('fiscal.nf.parcelas').read(
                        cr, user, parcela_ids, campos, context, load))
        modelo_serie = dados_nf.get('fisnfc_modelo_serie')
        tipo_nf = dados_nf.get('tipo')
        pessoa_obrigatorio = not any((modelo_serie == '02D', context.get('nota_fiscal_consumidor')))
        obj_adm_utils = self.pool.get('adm.utils')
        obj_financeiro_fatura = self.pool.get('financeiro.fatura')

        # Gera movimento no financeiro
        context['lancamento_fatura_r'] = True
        contrato_id = False
        tipo_condicao = self._fisnfc_get_tipo_condicao_pagamento(cr, user, nf_id, context)
        if tipo_nf == 'E':  # Nota fiscal de entrada
            # Somente compatibilidade. Remover se nao usado
            context.update({'nota_fiscal_entrada': True})
        else:  # Nota fiscal de saida
            # Somente compatibilidade. Remover se nao usado
            context.update({'nota_fiscal_saida': True})

        # Nota modelo 02 e serie D a fatura sera gerada com o pessoa_id do emitente
        destinatario_id = dados_nf.get('destinatario_id')
        company_id = dados_nf.get('company_id')
        nf_propria = dados_nf.get('nf_propria')
        numero_nf = dados_nf.get('numero_nf')

        # Remove faturas e titulos em previsao
        self.fisnfc_deleta_fatura_prv_financeiro(cr, user, nf_id, context)

        local_cobranca_id = parametro.financeiro.local_cobranca_id
        natureza_cobranca_id = ''
        moeda = parametro.financeiro.moeda_id
        obj_documento_tipo = self.pool.get('documento.documento.tipo')
        documento_ids = obj_documento_tipo.search(cr, user, [('sigla', '=', 'NF')], context=context)
        documento_id = documento_ids and documento_ids[0]
        vals = {'company_id': company_id,
                'adm_moeda_id': parametro.financeiro.moeda_id,
                'tipo_titulo': 'P' if tipo_nf == 'E' else 'R', 'state': 'A',
                'data_emissao': data_emissao, 'tipo_lancamento': 'A'}
        if dados_operacao.get('estnto_gerar_financeiro'):
            context.update({'entrada_titulo_receber': True})
            modulo_contrato_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
                cr, user, 'contratos', context)
            if modulo_contrato_instalado:
                contrato_id = dados_nf.get('fisnfc_contrato_id') or False
                if contrato_id:
                    context.update({'custifica_contrato': contrato_id})

            # Condicao a vista (todas a condicoes devem ser a vista, cartao nao eh a vista)
            if tipo_condicao == 'V':
                # Nao gera financeiro se nota fiscal for totalmente bonificada
                if dict_cfop_itens and ((pessoa_obrigatorio and dados_nf.get('fisnfc_modelo') < '90')
                                        or destinatario_id) and \
                        filter(lambda f: f.get('codigo')[-3:] != '910', dict_cfop_itens.values()):
                    vals_fatura = {'company_id': company_id,
                                   'pessoa_id': destinatario_id,
                                   'tipo_fatura': 'P' if tipo_nf == 'E' else 'R',
                                   'finfat_documento_tipo_id': documento_id,
                                   'data_emissao': data_emissao,
                                   'nf_id': dados_nf.get('id'),
                                   'documento': str(numero_nf or '').zfill(9)}
                    fatura_id = obj_financeiro_fatura.create(cr, user, vals_fatura, context)
                    self._fisnfc_gerar_movimento_financeiro_avista(
                        cr, user, dados_nf, fatura_id, dados_modelo, dict_condicao, dict_forma_pgto,
                        dict_parcelas, dict_guias, context)
                elif modelo_serie == '02D' or context.get('nota_fiscal_consumidor') or \
                        dados_nf.get('fisnfc_modelo') >= '90':
                    for condicao in dict_condicao.values():
                        context['fintem_nao_verificar_custificacao'] = True
                        self.fisnfc_criar_movimento_financeiro_2D(
                            cr, user, dados_nf, condicao, dict_forma_pgto, 'C', context=context)
                        del context['fintem_nao_verificar_custificacao']
            else:  # Faturamento para condicoes a prazo
                vendedor_id = dados_nf.get('vendedor_id')
                vendedor_externo_id = dados_nf.get('vendedor_externo_id')
                if nf_propria:
                    cntx_cond_parc = {'ctx_fisnfc_titulo_create_sem_custificacao': True}
                    cntx_cond_parc.update(context)
                    forma_pagto_fatura = {}  # chave= id da forma de pagamento, valor= id da fatura
                    for condicao in dict_condicao.values():
                        # Limpa a pessoa para nao repetir a operadora de cartao
                        fatura_pessoa_id = destinatario_id
                        forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
                        doc_tipo_id = forma_pgto.get('admfpg_documento_tipo_id')

                        # Se a forma de pagamento for cartao usa a pessoa da administ. do cartao
                        if forma_pgto.get('cartao_credito_debito'):
                            fatura_pessoa_id = forma_pgto.get('admfpg_cartao_pessoa_id')

                        # cria fatura e titulo apenas se houver cliente informado ou operadora
                        # de cartao na condicao de cartao
                        if not fatura_pessoa_id:
                            # cria tesouraria caso a nota seja de consumidor, sem cliente e com
                            # condicoes a vista e a prazo(EX: dinheiro e cartao)
                            if context.get('nota_fiscal_consumidor') \
                                    or context.get('nota_fiscal_saida'):
                                context['fintem_nao_verificar_custificacao'] = True
                                self.fisnfc_criar_movimento_financeiro_2D(
                                    cr, user, dados_nf, condicao, dict_forma_pgto, 'C',
                                    context=context)
                                del context['fintem_nao_verificar_custificacao']
                            continue

                        fatura_id = forma_pagto_fatura.get(fatura_pessoa_id)

                        # Evita criar mais de uma fatura do mesmo tipo para uma mesma pessoa
                        if not fatura_id:
                            vals_fatura = {'company_id': company_id,
                                           'tipo_fatura': 'P' if tipo_nf == 'E' else 'R',
                                           'data_emissao': data_emissao, 'nf_id': nf_id,
                                           'documento': str(numero_nf or '').zfill(9),
                                           'pessoa_id': fatura_pessoa_id,
                                           'finfat_documento_tipo_id': doc_tipo_id}
                            fatura_id = obj_financeiro_fatura.create(cr, user,vals_fatura, context)
                            forma_pagto_fatura[fatura_pessoa_id] = fatura_id

                        num_parcela = 0
                        for parcela_id in condicao.get('nf_parcela_ids'):
                            parcela = dict_parcelas.get(parcela_id)
                            num_parcela += 1
                            if forma_pgto.get('admfpg_boleto'):
                                local_cobranca_id = forma_pgto.get('admfpg_local_cobranca_id')
                                natureza_cobranca_id = forma_pgto.get('admfpg_natureza_id')
                            saldo_titulo = parcela.get('valor')
                            vencimento = parcela.get('vencimento')
                            vals_titulo = dict(vals,
                                data_emissao=data_emissao, vr_original=saldo_titulo,
                                data_vencimento=vencimento, saldo=saldo_titulo,
                                data_prorrogacao=vencimento, pessoa_id=fatura_pessoa_id,
                                vendedor_id=vendedor_id, parcela=num_parcela,
                                local_cobranca_id=local_cobranca_id, doc_tipo_id=doc_tipo_id,
                                natureza_cobranca_id=natureza_cobranca_id, fatura_id=fatura_id,
                                vendedor_externo_id=vendedor_externo_id,
                                documento=str(numero_nf or ''),
                                observacoes=self.get_observacao_titulo(
                                    cr, user, parcela.get('id'), context))
                            infcomp_contribuinte = dados_nf.get('fisnfc_infcomp_contribuinte')
                            vals_titulo = self.get_propagar_inf(
                                cr, user, {'tipo': tipo_nf,
                                           'fisnfc_infcomp_contribuinte': infcomp_contribuinte},
                                vals_titulo)
                            if doc_tipo_id and self.pool.get('documento.documento.tipo').read(
                                    cr, user, doc_tipo_id, ['sigla'], context).get('sigla') == 'CHQ':
                                vals_titulo.update(
                                    {'documento': condicao.get('fiscpg_chq_numero'),
                                     'fintit_chq_banco': condicao.get('fiscpg_chq_banco'),
                                     'fintit_chq_agencia': condicao.get('fiscpg_chq_agencia'),
                                     'fintit_chq_emitente': condicao.get('fiscpg_chq_emitente'),
                                     'fintit_chq_numero': condicao.get('fiscpg_chq_numero'),
                                     'fintit_chq_conta': condicao.get('fiscpg_chq_conta')})
                                cntx_cond_parc['lancamento_cheque'] = True
                            else:
                                if cntx_cond_parc.get('lancamento_cheque'):
                                    del cntx_cond_parc['lancamento_cheque']
                            if contrato_id:
                                vals_titulo.update({'contrato_id': contrato_id})

                            # Cria e baixa o titulos ´a vista´
                            titulo_id = False
                            if modelo_serie == '02D':
                                if not vals_titulo.get('pessoa_id'):
                                    self.fisnfc_criar_movimento_financeiro_2D(
                                        cr, user, dados_nf, condicao, dict_forma_pgto, 'C', context)
                                else:
                                    titulo_id = self.fisnfc_criar_titulo(
                                        cr, user, dados_nf.get('id'), vals_titulo, cntx_cond_parc)
                            else:
                                titulo_id = self.fisnfc_criar_titulo(
                                    cr, user, dados_nf.get('id'), vals_titulo, cntx_cond_parc)

                            # nao baixa titulo se for cheque ou cartao na nota de saida e que
                            # nao sao a vista(data de emissao diferente de vencimento)
                            if (not tipo_nf == 'S' or
                                    not cntx_cond_parc.get('lancamento_cheque') and \
                                    not forma_pgto.get('cartao_credito_debito')) and \
                                            data_emissao[:10] == vencimento and titulo_id:
                                self.fisnfc_baixar_titulo(
                                    cr, user, titulo_id, forma_pgto.get('id'), local_cobranca_id,
                                    saldo_titulo, context=context)

                # TODO: Transformar em metodos separados para nota propria e de terceiro
                else:  # Notas fiscais de terceiros
                    # itens bonificados
                    total_itens_bonificacao = 0
                    trib_bonificacao_ids = []
                    for i in dict_itens.values():
                        cfop = dict_cfop_itens.get(i.get('cfop_id'))
                        if cfop.get('codigo') in ('1910', '2910', '1911', '2911'):
                            total_itens_bonificacao += i.get('vr_total_item_rateado')
                            trib_bonificacao_ids.extend(i.get('tributacoes_ids'))
                    vals_fatura = {'company_id': company_id,
                                   'pessoa_id': destinatario_id,
                                   'tipo_fatura': 'P' if tipo_nf == 'E' else 'R',
                                   'finfat_documento_tipo_id': documento_id,
                                   'data_emissao': data_emissao, 'nf_id': nf_id,
                                   'documento': str(numero_nf or '').zfill(9)}
                    fatura_id = obj_financeiro_fatura.create(cr, user, vals_fatura, context)
                    for trib_id in trib_bonificacao_ids:
                        trib = dict_tribs.get(trib_id)
                        encargo = dict_encargos.get(trib.get('encargo_id'))
                        if encargo.get('sigla') == 'ICMS':
                            total_itens_bonificacao += trib.get('vr_icms_substituicao') or 0.0
                        elif encargo.get('sigla') == 'IPI':
                            total_itens_bonificacao += trib.get('vr_encargo') or 0.0
                    num = 0
                    context_parcela = {'ctx_fisnfc_titulo_create_sem_custificacao': True}
                    context_parcela.update(context)
                    total_itens_bonificacao_parcela = total_itens_bonificacao / len(dict_parcelas)
                    for parcela in dict_parcelas.values():
                        num += 1
                        vencimento = parcela.get('vencimento')
                        vals.update({'data_emissao': obj_adm_utils.admutl_datetime_to_timezone(
                            cr, user, parse(data_emissao), '%Y-%m-%d'),
                            'pessoa_id': destinatario_id,
                            'data_vencimento': vencimento,
                            'data_prorrogacao': vencimento,
                            'vr_original': parcela.get('valor'),
                            'parcela': num,
                            'saldo': round(parcela.get('valor'), 2) -
                                     round(total_itens_bonificacao_parcela, 2),
                            'documento': str(numero_nf or ''),
                            'vendedor_id': vendedor_id,
                            'fatura_id': fatura_id,
                            'vendedor_externo_id': vendedor_externo_id})

                        # Gera titulos
                        if contrato_id:
                            vals.update({'contrato_id': contrato_id})
                        titulo_id = self.fisnfc_criar_titulo(cr, user, dados_nf.get('id'),
                                                             vals, context_parcela)

                        # Baixa apenas titulos a vista onde a data de emissao eh igual a
                        # de vencimento
                        if data_emissao[:10] == vencimento:
                            self.fisnfc_baixar_titulo(
                                cr, user, titulo_id, False, local_cobranca_id, nf_terceiro=True,
                                context=context)

                    # Lancamento da guia no Financeiro
                    documento_tipo_ids = obj_documento_tipo.search(
                        cr, user, [('sigla', '=', 'GUI')], context=context)
                    context_guia = {'ctx_fisnfc_titulo_create_sem_custificacao': True}
                    context_guia.update(context)
                    guia_titulo_id_saldo = {}
                    for guia in dict_guias.values():
                        if destinatario_id and guia.get('data_vencimento') \
                                and guia.get('valor') > 0 \
                                and guia.get('fisgui_fornecedor_reembolsar'):
                            vals_guia = {'company_id': company_id,
                                         'pessoa_id': destinatario_id,
                                         'finfat_documento_tipo_id': documento_tipo_ids[0],
                                         'data_emissao': data_emissao,
                                         'documento': str(numero_nf or '').zfill(9) + 'G',
                                         'tipo_fatura': 'P' if tipo_nf == 'E' else 'R',
                                         'nf_id': nf_id}
                            id_fatura_guia = obj_financeiro_fatura.create(
                                cr, user, vals_guia, context={'faturar_nf': True})
                            vals_tit_guia = {'company_id': company_id,
                                             'pessoa_id': destinatario_id,
                                             'local_cobranca_id': guia.get('local_cobranca_id'),
                                             'adm_moeda_id': moeda,
                                             'tipo_titulo': 'P' if tipo_nf == 'E' else 'R',
                                             'state': 'A',
                                             'data_emissao': data_emissao,
                                             'tipo_lancamento': 'A',
                                             'fatura_id': id_fatura_guia,
                                             'data_vencimento': guia.get('data_vencimento'),
                                             'data_prorrogacao': guia.get('data_vencimento'),
                                             'vr_original': guia.get('valor'),
                                             'saldo': guia.get('valor'),
                                             'documento': str(numero_nf or '') + 'G',}
                            if guia.get('agencia_id'):
                                vals_tit_guia['agencia_id'] = guia.get('agencia_id')
                            vals_tit_guia['doc_tipo_id'] = documento_tipo_ids[0]
                            titulo_id_guia = self.fisnfc_criar_titulo(
                                cr, user, dados_nf.get('id'), vals_tit_guia, context=context_guia)
                            guia_titulo_id_saldo[titulo_id_guia] = guia.get('valor')
                            self.pool.get('fiscal.guia').write(cr, user, guia.get('id'),
                                                             {'titulo_id': titulo_id_guia}, context)

                            # Baixa apenas titulos a vista(data de emissao = de vencimento)
                            if data_emissao == guia.get('data_vencimento'):
                                self.fisnfc_baixar_titulo(
                                    cr, user, titulo_id=titulo_id_guia, vr_pago=guia.get('valor'),
                                    forma_pgto_id=False, local_cobranca_id=local_cobranca_id,
                                    nf_terceiro=True, context=context)

            # Cria faturas e titulos para as condicoes de pagamento Cartao que possuem taxa de
            # administracao apenas para nostas de saida
            if tipo_nf == 'S':
                self.criar_titulo_fatura_cartao(
                    cr, user, moeda, local_cobranca_id, dados_nf, dict_condicao, dict_forma_pgto,
                    dict_cond_forma, dict_parcelas, context=context)

            # TODO: Transformar em metodo
            # Gerar fatura/titulos para as notas com deducoes
            if tipo_nf == 'E' and dados_nf.get('deducao_itens_ids'):
                # Deducoes na nota fiscal
                data_fiscal_deducoes_itens = self.pool.get('fiscal.deducoes.itens').read(
                    cr, user, dados_nf.get('deducao_itens_ids'), [], context=context)

                # Busca 'documento de compra'
                id_doc_tipo_deducao = obj_documento_tipo.search(
                    cr, user, [('sigla', '=', 'DEC')], context=context)

                # Se o documento nao existir, cria (apesar de o documento ter sido incluido
                # no documento_data .xml
                if not id_doc_tipo_deducao:
                    id_doc_tipo_deducao = obj_documento_tipo.create(
                        cr, user, {'descricao': 'Documento de Compra', 'sigla': 'DEC',
                                   'bloqueio': True, 'liquidavel': True}, context)
                else:
                    id_doc_tipo_deducao = id_doc_tipo_deducao[0]

                # Varre as deducoes da nota fiscal e gera fatura/titulo
                inc = 1
                deducao_fatura_ids = []
                deducao_titulo_ids = []
                context_deducao = {'ctx_fisnfc_titulo_create_sem_custificacao': True}
                if context:
                    context_deducao.update(context)
                for deducao in data_fiscal_deducoes_itens:
                    vals_fatura = {
                        'company_id': company_id,
                        'pessoa_id': deducao.get('pessoa_id')[0],
                        'data_emissao': data_emissao,
                        'tipo_fatura': 'P',
                        'nf_id': nf_id
                    }

                    # Cria a fatura
                    id_fatura_deducao = obj_financeiro_fatura.create(
                        cr, user, vals_fatura, context)
                    deducao_fatura_ids.append(id_fatura_deducao)
                    vals.update({
                        'pessoa_id': deducao.get('pessoa_id')[0],
                        'fatura_id': id_fatura_deducao,
                        'data_vencimento': deducao.get('data_vencimento'),
                        'data_prorrogacao': deducao.get('data_vencimento'),
                        'vr_original': deducao.get('valor'),
                        'saldo': deducao.get('valor'),
                        'parcela': inc,
                        'doc_tipo_id': id_doc_tipo_deducao,
                        'documento': str(nf_id or ''),
                    })

                    # Cria o titulo
                    if modulo_contrato_instalado:
                        vals_titulo.update({'contrato_id': contrato_id})
                    deducao_titulo_id = self.fisnfc_criar_titulo(
                        cr, user, dados_nf.get('id'), vals, context_deducao)
                    deducao_titulo_ids.append(deducao_titulo_id)

                    inc += 1

                # Baixa do titulo se condicao de pagamento for a vista
                for i in deducao_titulo_ids:
                    self.fisnfc_baixar_titulo(
                        cr, user, i, False, local_cobranca_id, nf_terceiro=True, context=context)

        # TODO: Transformar em metodo
        # Lancamento de fatura do Frete
        # frete por fora na nota de entrada sempre deve gerar titulo
        if not dados_nf.get('transportador_id') and dados_nf.get('tipo_frete') == 'F' \
                and tipo_nf == 'E':
            nf_frete_ids = self.search(
                cr, user,[('fisnfc_nf_frete_id', '=', dados_nf.get('id'))], context=context)
            campos = ['destinatario_id', 'fisnfc_parcelas_terceiros_ids', 'company_id',
                      'numero_nf', 'data_emissao', 'vr_total_nf']
            for nf_frete in self.read(
                    cr, user, nf_frete_ids, campos, context, '_classic_write'):
                if nf_frete.get('vr_total_nf') and \
                        nf_frete.get('fisnfc_parcelas_terceiros_ids'):
                    dados_parcela = self.pool.get('fiscal.nf.parcelas').read(
                        cr, user, nf_frete.get('fisnfc_parcelas_terceiros_ids'),
                        ['valor', 'vencimento'], context)
                    for parcela in dados_parcela:
                        vencimento_frete = parcela.get('vencimento')
                        vals_frete = {
                            'company_id': nf_frete.get('company_id'),
                            'pessoa_id': nf_frete.get('destinatario_id'),
                            'data_emissao': nf_frete.get('data_emissao'),
                            'tipo_fatura': 'P',  # Nota de frete eh sempre de entrada
                            'nf_id': nf_id,
                            'finfat_documento_tipo_id': documento_id,
                            'documento': str(nf_frete.get('numero_nf') or '').zfill(9)
                        }
                        id_fatura_frete = obj_financeiro_fatura.create(
                            cr, user, vals_frete, context)
                        n_documento_frete = obj_financeiro_fatura.read(
                            cr, user, id_fatura_frete, ['documento'], context).get('documento')
                        frete_titulo_id = False
                        context_frete = {'ctx_fisnfc_titulo_create_sem_custificacao': True}
                        context_frete.update(context)
                        if id_fatura_frete and n_documento_frete:
                            vals_tit_frete = {
                                'company_id': nf_frete.get('company_id'),
                                'doc_tipo_id': documento_id,
                                'pessoa_id': nf_frete.get('destinatario_id'),
                                'local_cobranca_id': local_cobranca_id,
                                'adm_moeda_id': moeda,
                                'tipo_titulo': 'P',  # Nota de frete eh somente de entrada
                                'state': 'A',
                                'data_emissao': nf_frete.get('data_emissao'),
                                'tipo_lancamento': 'A',
                                'fatura_id': id_fatura_frete,
                                'data_vencimento': vencimento_frete,
                                'data_prorrogacao': vencimento_frete,
                                'vr_original': parcela.get('valor'),
                                'saldo': parcela.get('valor'),
                                'documento': str(nf_frete.get('numero_nf') or ''),
                                'parcela': 97}

                            # Gera titulo para frete
                            frete_titulo_id = self.fisnfc_criar_titulo(
                                cr, user, dados_nf.get('id'), vals_tit_frete, context_frete)
                        elif id_fatura_frete and not n_documento_frete:
                            raise osv.except_osv(_('Atencao'),
                                                 _('Existe uma fatura criada referente a esta nota '
                                                   'fiscal mas nao contem o numero do documento, '
                                                   'por este motivo nao e possivel prosseguir.'))

                        # Baixa o titulo se for a vista(emissao = vencimento)
                        if frete_titulo_id and vencimento_frete == nf_frete.get('data_emissao')[:10]:
                            self.fisnfc_baixar_titulo(
                                cr, user, frete_titulo_id, False, local_cobranca_id,
                                nf_terceiro=True, context=context)

        # TODO: Transformar em metodo
        # Movimentacao de itens
        capa_ids = self.pool.get('estoque.movimento.capa').search(
            cr, user, [('nf_capa_id', '=', nf_id)], context=context)
        data_saida_entrada = dados_nf.get('data_saida_entrada')
        if dict_itens:
            # quando vem da importacao de notas do supervisor, ja tem funcionario no movimento
            funcionario_id = self.pool.get('rh.funcionario').get_id_func(cr, user) or \
                             self.pool.get('estoque.movimento.capa').read(cr, user, capa_ids[0],
                              ['funcionario_id'], context=context, load=load).get('funcionario_id')

            # Verifica a data de movimento de acordo com o tipo da nota
            if nf_propria:
                data_mov_capa = datetime.now().date()
            else:
                data_mov_capa = obj_adm_utils.admutl_datetime_to_timezone(
                    cr, user, parse(data_saida_entrada or time.strftime('%Y-%m-%d')),
                    '%Y-%m-%d')

            if tipo_nf == 'S':
                data_mov_capa = obj_adm_utils.admutl_datetime_to_timezone(
                    cr, user, parse(data_emissao), '%Y-%m-%d')
            self.pool.get('estoque.movimento.capa').write(
                cr, user, capa_ids, {'company_id': company_id, 'data': data_mov_capa,
                                     'estmvc_operacao_id': dados_operacao.get('id'),
                                     'funcionario_id': funcionario_id, 'nf_capa_id': nf_id,
                                     'state': 'F', 'pessoa_id': destinatario_id}, context)
            modulo_madeireira_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
                cr, user, 'madeireira', context)
            estnto_atualizar_saldo = dados_operacao.get('estnto_atualizar_saldo')
            seq = 0
            update_fornec_produto_ids = []
            obj_estoque_fornecedor = self.pool.get('estoque.fornecedor')
            obj_trib_entrada = self.pool.get('fiscal.tributacao.entrada')
            obj_trib_entrada_encargos = self.pool.get('fiscal.tributacao.entrada.encargos')
            locais_grade = {}
            grade_qtd_emb = self.pool.get('estoque.utils').get_grade_qtd_embalagem(
                cr, grade_ids, context=context)

            obj_financeiro_fornecedor = self.pool.get('financeiro.fornecedor')
            # Verifica se o destinatario esta presente na tabela de fornecedores
            fornecedor_ids = obj_financeiro_fornecedor.search(cr, user,
                                                              [('pessoa_id', '=', destinatario_id),
                                                               ('company_id', '=', company_id)],
                                                              context=context)
            custos_do_dia = self.pool.get('estoque.preco.custo').get_custos_do_dia(cr, user,
                                                           produto_ids, company_id, context=context)
            for item in dict_itens.values():
                seq += 1
                vals_mov_itens = {'sequencia': seq}
                produto_id = item.get('produto_id')
                custo_produto = self.pool.get('estoque.preco.custo').search(
                    cr, user, [('produto_id', '=', produto_id)], context=context)
                if custo_produto:
                    custo_dia = custos_do_dia.get(produto_id) or {}
                    chave = '%s-%s' % (item.get('grade_id'), item.get('produto_embalagem_id'))
                    qtd_or_volume = item.get('quantidade') * grade_qtd_emb.get(chave).get('qtd')
                    if modulo_madeireira_instalado and item.get('estmvi_volume'):
                        qtd_or_volume = item.get('estmvi_volume')
                    ultimo_custo = custo_dia.get('estprc_ultimo_custo', 0) * qtd_or_volume
                    custo_corrigido = \
                        custo_dia.get('estprc_custo_corrigido', 0) * qtd_or_volume
                    custo_medio = \
                        custo_dia.get('estprc_custo_medio_contabil', 0) * qtd_or_volume
                    custo_reposicao = \
                        custo_dia.get('estprc_custo_reposicao', 0) * qtd_or_volume
                    vals_mov_itens.update({
                        'estmvi_saidas_ultimo_custo_total': ultimo_custo,
                        'estmvi_saidas_custo_corrigido_total': custo_corrigido,
                        'estmvi_saidas_custo_medio_contabil_total': custo_medio,
                        'estmvi_saidas_custo_reposicao_total': custo_reposicao,
                        'natureza_id': natureza_id
                    })
                context_aux = context.copy()
                context_aux['super_write_estoque_movimento_itens'] = True
                self.pool.get('estoque.movimento.itens').write(
                    cr, user, item.get('id'), vals_mov_itens, context=context_aux)

                # Atualiza tributacao de entrada do produto para o fornecedor especifico da nota
                # fiscal terceiros com inf. de MVA
                cfop = dict_cfop_itens.get(item.get('cfop_id'))
                if not nf_propria and cfop.get('codigo')[-3:] != '910':
                    tributacao_encargos_produtos = []
                    for trib_id in item.get('tributacoes_ids'):
                        trib = dict_tribs.get(trib_id)
                        # Atualiza infomacao de tributacao para o produto.
                        encargo_id = trib.get('encargo_id')
                        cst_id = trib.get('cst_id')
                        tributacao_id = trib.get('tributacao_id')
                        encargo = dict_encargos.get(trib.get('encargo_id'))
                        tributacao_encargos_produtos.append(
                            {'encargo_id': encargo_id,
                             'cst_id': cst_id,
                             'tributacao_id': tributacao_id,
                             'mva': trib.get('percentual_mva'),
                             'valor_pauta': trib.get('valor_pauta'),
                             'modbcst': trib.get('modbcst'),
                             'tipo_tributacao': 'E',
                             'produto_id': produto_id,
                             'fisten_aliquota_venda': trib.get('aliquota_venda'),
                             'encargo_sigla': encargo.get('sigla')})
                        if encargo.get('sigla').upper() == 'ICMS':
                            args = [('fornecedor_id', '=', fornecedor_ids[0]),
                                    ('produto_id', '=', produto_id)]
                            trib_ent_ids = obj_trib_entrada.search(
                                cr, user, args, context=context)
                            if not trib_ent_ids:
                                trib_ent_id = obj_trib_entrada.create(
                                    cr, user, {'fornecedor_id': fornecedor_ids[0],
                                               'produto_id': produto_id}, context)
                            else:
                                trib_ent_id = trib_ent_ids[0]
                            args = [('tributacao_entrada_id', '=', trib_ent_id),
                                    ('encargo_id', '=', encargo_id)]
                            ent_encargo_ids = obj_trib_entrada_encargos.search(
                                cr, user, args, context=context)
                            vals = {'fistee_aliquota_venda': trib.get('aliquota_venda'),
                                    'fistee_modbcst': trib.get('modbcst'),
                                    'fistee_mva': trib.get('percentual_mva'),
                                    'fistee_valor_pauta': trib.get('valor_pauta')}
                            if ent_encargo_ids:
                                obj_trib_entrada_encargos.write(
                                    cr, user, ent_encargo_ids, vals, context=context)
                            else:
                                vals.update({'encargo_id': encargo_id,
                                             'tributacao_entrada_id': trib_ent_id,
                                             'tributacao_id': tributacao_id,
                                             'cst_id': cst_id,})
                                obj_trib_entrada_encargos.create(cr, user, vals, context)

                    # Verifica e atualiza as informacoes se necessario
                    if cfop.get('codigo')[-3:] != '910':
                        self.cadastrar_tributacao_produto(
                            cr, user, company_id, produto_id, tributacao_encargos_produtos,
                            destinatario_id)

                    # Verifica se o produto e o fornecedor existem na tabela 'estoque.fornecedor'
                    if fornecedor_ids:
                        fornecedor_produto_ids = obj_estoque_fornecedor.search(
                            cr, user, [('fornecedor_id', '=', fornecedor_ids[0]),
                                       ('produto_id', '=', produto_id)], context=context)

                        if fornecedor_produto_ids:
                            # Adiciona o ID do registro para realizar um write alterando a data
                            # da ultima compra
                            update_fornec_produto_ids += fornecedor_produto_ids
                        else:
                            # Cria um registro que associa o produto ao fornecedor
                            vals_produto_fornecedor = {
                                'produto_id': produto_id,
                                'fornecedor_id': fornecedor_ids[0],
                                'data_da_ultima_compra': data_emissao}
                            obj_estoque_fornecedor.create(
                                cr, user, vals_produto_fornecedor, context=context)

                grade_id = item.get('grade_id')
                try:
                    cahve = '%s-%s' % (grade_id, item.get('produto_embalagem_id'))
                    qd_embalagem = grade_qtd_emb.get(cahve).get('qtd')
                except:
                    qd_embalagem = 1
                qnt_real_item = item.get('quantidade') * qd_embalagem
                item_local_id = item.get('local_id')
                if item_local_id:
                    # Se houver o contexto implementado abaixo, a atualização do saldo
                    # será definida na heranca da nota fiscal no modulo
                    # madeireira, visto que, a quantidade deve ser atualizada do volume.
                    modulo_madeireira_instalado = self.pool.get('adm.utils'). \
                        admutl_check_modulo_instalado(cr, user, 'madeireira', context)
                    if modulo_madeireira_instalado:
                        qnt_real_item = item.get('estmvi_volume') or qnt_real_item
                if not locais_grade.get(item_local_id):
                    locais_grade[item_local_id] = {}
                if grade_id in locais_grade[item_local_id]:
                    locais_grade[item_local_id][grade_id] += qnt_real_item
                else:
                    locais_grade[item_local_id][grade_id] = qnt_real_item
            if tipo_nf == 'E':
                if data_saida_entrada:
                    data_atualiza_saldo = data_saida_entrada[:10]
            else:
                data_atualiza_saldo = data_emissao[:10]
            if estnto_atualizar_saldo and locais_grade:
                # Atualiza os itens por local, pode haver itens em locais diferentes na mesma nota
                for local_id in locais_grade:
                    grade_qtd = locais_grade.get(local_id)
                    self.pool.get('estoque.produto.saldo').atualizar_saldo_do_dia(
                        cr, user, company_id, grade_qtd, data_atualiza_saldo, tipo_nf, local_id,
                        context=context)
            if update_fornec_produto_ids:
                update_fornec_produto_ids = list(set(update_fornec_produto_ids))
                obj_estoque_fornecedor.write(
                    cr, user, update_fornec_produto_ids, {'data_da_ultima_compra': data_emissao},
                    context=context)

            # Atualiza custo do produto e saldo no estoque
            if tipo_nf == 'E' and estnto_atualizar_saldo and \
                    not dados_nf.get('fisnfc_nf_devolucao'):
                self.pool.get('estoque.preco.custo').estprc_set_custos_do_dia(
                    cr, user, nf_id, context=context)

        for i in gerencial_movimento:
            self.pool.get('gerencial.movimento').create(cr, user, i, context=context)

        return values

    def cancelar_registros_relacionados_nf(self, cr, uid, ids, context=None):
        """
        Cancela registros relacionados com a NF.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuário utilizado para a verificacao de perfil,
        :param ids: id da NF,
        :param context: um dicionário padrao para valores contextuais,
        :return: sempre verdadeiro.
        """
        context = context and context.copy() or {}
        dados_fiscal_nf_capa = self.read(cr, uid, ids[0] if isinstance(ids, list) else ids,
                                         ['fisnfc_pedido_ids', 'tipo'], context)
        pedido_ids = dados_fiscal_nf_capa.get('fisnfc_pedido_ids')
        estoque_parametro_dados = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_cancel_nf_pedido_validado', context=context)

        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        if estoque_parametro_dados.get('estpar_cancel_nf_pedido_validado') and \
                dados_fiscal_nf_capa.get('tipo') == 'S' and pedido_ids:
            # Volta pedido para o estado validado
            obj_estoquepedidos_pedido_capa.pedcap_cancela_nf_pedido_validado(
                cr, uid, pedido_ids, context)
        else:
            # Cancela pedidos
            for p in pedido_ids:
                obj_estoquepedidos_pedido_capa.button_cancelar(cr, uid, p, context)

        # Exclui registros do gerencial movimento
        gerecial_movimento_ids = self.pool.get('gerencial.movimento').search(
            cr, uid, [('fingmo_nf_id', '=', ids[0] if isinstance(ids, list) else ids)],
            context=context)
        self.pool.get('gerencial.movimento').unlink(cr, uid, gerecial_movimento_ids)

        return True

    def button_cancelar(self, cr, uid, ids, context=None):
        """
        :param cr: o cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a nota fiscal foi cancelada
        :rtype: bool
        """
        context = context and context.copy() or {}
        return self.fisnfc_funcao_button_cancelar(cr, uid, ids, context)

    def fisnfc_funcao_button_cancelar(self, cr, uid, ids, context=None):
        """
        :param cr: o cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se a nota fiscal foi cancelada
        :rtype: bool
        """
        context = context and context.copy() or {}
        context['cancelar_nota_fiscal'] = True
        context['super_write_fiscal_nf_capa'] = True
        nf_id = ids[0]
        load = '_classic_write'
        campos_nf = ['state', 'natureza_id', 'estoque_movimento_itens_ids', 'company_id', 'tipo',
                     'fisnfc_modelo_serie', 'nf_condicao_ids', 'nf_parcelas_ids', 'nf_propria',
                     'fisnfc_nf_referencia_ids', 'data_emissao', 'destinatario_id', 'numero_nf',
                     'serie_nf', 'modelo_55', 'fisnfc_consumidor', 'fisnfc_titulos_ids',
                     'fisnfc_nf_referenciada_ids', 'data_saida_entrada']
        dados_nf = self.read(cr, uid, nf_id, campos_nf, context=context, load=load)
        brw_natureza = self.pool.get('fiscal.cfop.natureza').browse(
            cr, uid,  dados_nf.get('natureza_id'), context=context)
        emi_ids = dados_nf.get('estoque_movimento_itens_ids')
        campos_emi = ['estmvi_entrada_preco_custo_id', 'local_id', 'quantidade', 'grade_id',
                      'produto_embalagem_id']
        modulo_madeireira_instalado = self.pool.get('adm.utils').admutl_check_modulo_instalado(
            cr, uid, 'madeireira', context)
        if modulo_madeireira_instalado:
            campos_emi.append('estmvi_volume')
        dados_emi = self.pool.get('estoque.movimento.itens').read(
            cr, uid, emi_ids, campos_emi, context, load)
        if dados_nf.get('state') == '3':  # Somente NF faturada realizou movimento financeiro
            if self.pool.get('financeiro.titulo.baixa').search(
                    cr, uid, [('titulo_id', 'in', dados_nf.get('fisnfc_titulos_ids'))],
                    context=context) and self.pool.get('financeiro.titulo.historico').search(
                    cr, uid, [('titulo_id', 'in', dados_nf.get('fisnfc_titulos_ids')),
                              ('operacao', '=', 'BP')], context=context):
                raise osv.except_osv(_('Atencao'), _('Efetue o estorno da baixa de titulos antes '
                                                     'de cancelar a NF.'))
            locais_grade = {}
            estnto_atualizar_saldo =  brw_natureza.fisnat_operacao_id.estnto_atualizar_saldo
            grade_ids = map(lambda x: x.get('grade_id'), dados_emi)
            grade_qtd_emb = self.pool.get('estoque.utils').get_grade_qtd_embalagem(
                cr, grade_ids, context=context)
            for emi in dados_emi:
                # Desabilita o custo
                self.pool.get('estoque.preco.custo').write(
                    cr, uid, emi.get('estmvi_entrada_preco_custo_id'), {'active': False}, context)
                chave = '%s-%s' % (emi.get('grade_id'), emi.get('produto_embalagem_id'))
                qd_embalagem = chave in grade_qtd_emb and grade_qtd_emb.get(chave).get('qtd')
                if qd_embalagem and emi.get('local_id'):
                    # Se houver o contexto implementado abaixo, a atualização do saldo
                    # será definida na heranca da nota fiscal no modulo
                    # madeireira, visto que, a quantidade deve ser atualizada do volume.
                    quantidade = emi.get('quantidade') * qd_embalagem
                    if modulo_madeireira_instalado:
                        quantidade = emi.get('estmvi_volume') or quantidade
                    if not locais_grade.get(emi.get('local_id')):
                        locais_grade[emi.get('local_id')] = {}
                    if not locais_grade[emi.get('local_id')].get(emi.get('grade_id')):
                        locais_grade[emi.get('local_id')][emi.get('grade_id')] = quantidade
                    else:
                        locais_grade[emi.get('local_id')][emi.get('grade_id')] += quantidade
            if estnto_atualizar_saldo and locais_grade:
                # Atualiza os itens por local, pode haver itens em locais diferentes na mesma nota
                data_saida_entrada = dados_nf.get('data_saida_entrada') or time.strftime('%Y-%m-%d')
                for local_id in locais_grade:
                    grade_qtd = locais_grade.get(local_id)
                    self.pool.get('estoque.produto.saldo').atualizar_saldo_do_dia(
                        cr, uid, dados_nf.get('company_id'), grade_qtd, data_saida_entrada[:10],
                        'E' if dados_nf.get('tipo') == 'S' else 'S', local_id,  context=context)

            self.fisnfc_cancelar_movimentacao_financeira(cr, uid, nf_ids=nf_id, context=context)

            # Cancelamento de tesouraria modelo 2D
            if dados_nf.get('fisnfc_modelo_serie') == '02D':
                self.fisnfc_estorna_movimento_tesouraria_mod2D(cr, uid, dados_nf, context)

        values = {}

        # Cancelamento de fatura em previsao
        self.fisnfc_deleta_fatura_prv_financeiro(cr, uid, nf_id, context)

        # Cancelamento de titulos de pedidos
        obj_estoque_pedidos_capa = self.pool.get('estoquepedidos.pedido.capa')
        pedido_ids = obj_estoque_pedidos_capa.search(
            cr, uid, [('pedcap_nf_ids', '=', nf_id)], context=context)
        for x in obj_estoque_pedidos_capa.browse(cr, uid, pedido_ids, context):
            titulo_ids = self.pool.get('financeiro.titulo').search(
                cr, uid, [('documento', '=', x.numero_pedido)], context=context)
            for t in titulo_ids:
                self.pool.get('financeiro.titulo').write(
                    cr, uid, t, {'state': 'C'}, context=context)

        # Exclui os dados da nota cancelada
        context['cancelamento_nf'] = True
        values['estoque_movimento_itens_ids'] = [[2, item_id, False] for item_id in emi_ids]
        self.pool.get('fiscal.condicao.pagamento').unlink(
            cr, uid, map(lambda m: m, dados_nf.get('nf_condicao_ids')), context)
        self.pool.get('fiscal.nf.parcelas').unlink(
            cr, uid, map(lambda m: m, dados_nf.get('nf_parcelas_ids')), context)

        # Cancela registros relacionados
        self.cancelar_registros_relacionados_nf(cr, uid, nf_id, context)

        # Zera valores dos campos da capa
        values.update({}.fromkeys(['vr_icms_substituicao', 'vr_base_icms_substituicao', 'vr_seguro',
                                   'vr_frete', 'vr_despesas_acessorias', 'vr_desconto'], 0))
        if not dados_nf.get('nf_propria'):
            context['cancelar'] = True
            context['cancelamento_terceiros'] = True

        # Nota fiscal de complemento de arroz e compra por empresa lucro real a fornecedor simples
        # nacional. Reajusta os valores complementares do fisco
        if brw_natureza.fisnat_complemento_arroz:
            values['fisnfc_infcomp_fisco'] = ''

        # Apuracao de ICMS
        apuracao_ids = self.pool.get('fiscal.apuracao.imposto.outros.creditos').search(
            cr, uid, [('fisapuoc_nf_id', '=', ids[0])], context=context)
        self.pool.get('fiscal.apuracao.imposto.outros.creditos').unlink(
            cr, uid, apuracao_ids, context)

        self.button_cancelado(cr, uid, ids, context)
        if dados_nf.get('fisnfc_nf_referencia_ids'):
            values['fisnfc_nf_referencia_ids'] = [[6, False, []]]
            # limpa a referencia da nota referenciada
            self.write(cr, uid, dados_nf.get('fisnfc_nf_referencia_ids'),
                              {'fisnfc_nf_referencia_ids': [[6, False, []]]}, context)

        # Notas referenciadas
        if dados_nf.get('fisnfc_nf_referenciada_ids'):
            cr.execute('DELETE FROM fiscal_nf_referencia WHERE nf_ref_id IN (%s)' % ', '.join([
                str(_id) for _id in dados_nf.get('fisnfc_nf_referenciada_ids')]))

        values['state'] = 'C'  # Efetiva o cancelamento da nota fiscal
        self.cancela_sugestao_preco(cr, uid, ids, context=context)

        return self.write(cr, uid, ids, values, context)

    def cancela_sugestao_preco(self, cr, uid, ids, context=None):
        '''
        Cancela a sugestao de compra ligada a nota que esta sendo cancelada
        '''
        context = context and context.copy() or {}
        sug_ids = self.pool.get('fiscal.sugestao.preco').search(
            cr, uid, [('fissup_nf_id','in', ids)], context=context)

        return self.pool.get('fiscal.sugestao.preco').unlink(cr, uid, sug_ids, context=context)

    def fisnfc_cancelar_movimentacao_financeira(self, cr, uid, nf_ids, context=None):
        """
        Funcao procura todos os titulos gerados por meio da nota fiscal para sem cancelados.
        Para os Titulos que tiveram baixa, serao criados novos titulos para realizar o estorno.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} o ID do usuário utilizado para a verificacao de perfil,
        :param nf_ids: {int or list} nota fiscal a ser cancelada,
        :param context: {dict} um dicionário padrao para valores contextuais,
        :return: True
        """
        context = context and context.copy() or {}
        if not nf_ids:
            return True
        if nf_ids and isinstance(nf_ids, (int, long)):
            nf_ids = [nf_ids]

        obj_ftm = self.pool.get('financeiro.tesouraria.movimento')
        obj_titulo = self.pool.get('financeiro.titulo')
        obj_local_cobranca = self.pool.get('financeiro.local.cobranca')
        obj_fatura = self.pool.get('financeiro.fatura')
        self.fisnfc_deleta_fatura_prv_financeiro(cr, uid, nf_ids, context)
        fatura_ids = obj_fatura.search(cr, uid, [('nf_id', 'in', nf_ids)], context=context)
        if fatura_ids:
            titulo_ids = obj_titulo.search(
                cr, uid, [('fatura_id', 'in', fatura_ids)], context=context)
            if titulo_ids:
                titulo_fields_to_read = ['state', 'saldo', 'documento', 'tipo_titulo',
                                         'vr_original', 'data_vencimento', 'data_emissao',
                                         'local_cobranca_id']
                titulo_dados = obj_titulo.read(cr, uid, titulo_ids, titulo_fields_to_read, context)
                erro_conta_nao_informada = u''
                for i in titulo_dados:
                    tit_state = i.get('state')
                    tit_saldo = i.get('saldo') or 0
                    tit_documento = \
                        (i.get('documento') or '') or _('** Titulo nao encontrado **')
                    tit_tipo_titulo = i.get('tipo_titulo')
                    tit_valor_original = i.get('vr_original') or 0
                    tit_local_cobranca_id = \
                        i.get('local_cobranca_id') and  i['local_cobranca_id'][0] or False
                    valor_saldo_restante = round(tit_saldo, 2)
                    valor_original = round(tit_valor_original, 2)
                    if valor_saldo_restante and valor_original != valor_saldo_restante or \
                                    tit_state == 'B':
                        conta_id = False
                        local_cobranca_descricao = _('** Local nao encontrado **')
                        local_cobranca_dados = obj_local_cobranca.read(
                            cr, uid, tit_local_cobranca_id, ['finloc_conta_id', 'descricao'], context)
                        if local_cobranca_dados and isinstance(local_cobranca_dados, list):
                            local_cobranca_dados = local_cobranca_dados[0]
                        if isinstance(local_cobranca_dados, dict):
                            conta = local_cobranca_dados.get('finloc_conta_id')
                            conta_id = conta and conta[0] or False
                            local_cobranca_descricao = \
                                (local_cobranca_dados.get('descricao') or '') \
                                or _('** Local nao encontrado **')
                        if not conta_id:
                            erro_conta_nao_informada += \
                                (_('Titulo: %s - Local Cobranca: %s') %
                                 (tit_documento, local_cobranca_descricao))
                        else:
                            if i.get('data_vencimento') == i.get('data_emissao'):
                                # Para movimento avista, e necesario alterar o titulo para
                                # cancelado, deletar as baixas do titulo e do movimento de
                                # tesouraria
                                self.fisnfc_cancelar_movimentacao_financeira_avista(
                                    cr, uid, i.get('id'), context)
                            else:
                                valor_titulo = valor_original - valor_saldo_restante
                                historico = \
                                    _('Movimento lancado automaticamente para estorno no titulo: '
                                      '%s, devido ao cancelamento de nota fiscal.') % tit_documento
                                tesouraria_cred_ou_deb = 'D' if tit_tipo_titulo == 'R' else 'C'
                                data_movimento = date.today()
                                vals_tesouraria = {
                                    'state': 'A',
                                    'valor': valor_titulo,
                                    'conta_id': conta_id,
                                    'historico': historico,
                                    'debito_credito': tesouraria_cred_ou_deb,
                                    'data_movimento': data_movimento.strftime('%Y-%m-%d'),
                                    'ano_competencia': data_movimento.year,
                                    'mes_competencia': data_movimento.month
                                }
                                obj_ftm.create(cr, uid, vals_tesouraria, context=context)
                if erro_conta_nao_informada:
                    erro = _('Conta nao informada para os locais de cobrancas: %s') % \
                           erro_conta_nao_informada
                    raise osv.except_osv(_('Estorno do movimento da tesouraria'), erro)
                obj_fatura.write(cr, uid, fatura_ids, {'cancelada': True}, context=context)
        else: # notas sem titulo
            ftm_ids = obj_ftm.search(cr, uid, [('fintem_nf_id', 'in', nf_ids)], context=context)
            if ftm_ids:
                for i in obj_ftm.read(cr, uid, ftm_ids, [], context, '_classic_write'):
                    historico = _('Movimento lancado automaticamente para estorno '
                                   ', devido ao cancelamento de nota fiscal .')
                    data_movimento = date.today()
                    vals_tesouraria = {
                        'state': 'A',
                        'valor': i.get('valor'),
                        'conta_id': i.get('conta_id'),
                        'historico': historico,
                        'debito_credito': i.get('debito_credito') == 'D' and 'C' or 'D',
                        'data_movimento': data_movimento.strftime('%Y-%m-%d'),
                        'ano_competencia': data_movimento.year,
                        'mes_competencia': '%02d' % data_movimento.month,
                        'fintem_nf_id': i.get('fintem_nf_id'),
                    }
                    obj_ftm.create(cr, uid, vals_tesouraria, context=context)
        return True

    def fisnfc_cancelar_movimentacao_financeira_avista(self, cr, uid, titulo_id, context=None):
        """
        Este exclui a movimentaca financeira para titulos avista
        Para movimento avista, e necesario alterar o titulo para cancelado, deletar as baixas
        do titulo e do movimento de tesouraria
        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param titulo_id: Id do titulo a ser pesquisado para deletar sua baixa e seu movimento
                          de tesouraria
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type titulo_id: int | long
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        fintit_baixa = self.pool.get('financeiro.titulo.baixa')
        titulo_baixado_ids = fintit_baixa.search(
            cr, uid, [('titulo_id', '=', titulo_id)], context=context)
        if titulo_baixado_ids:
            for titulo_baixado in fintit_baixa.browse(cr, uid, titulo_baixado_ids, context=context):
                tesouraria_movimento_id = titulo_baixado.tesouraria_movimento_id.id
                fintit_baixa.unlink(cr, uid, titulo_baixado.id, context=context)
                self.pool.get('financeiro.tesouraria.movimento').unlink(
                    cr, uid, tesouraria_movimento_id, context)

    def fisnfc_button_apagar_itens(self, cr, uid, ids, context=None):
        """
        Exclui todos os itens e recalcula o valor da nota fiscal.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: verdadeiro se os itens foram excluidos ou nao houver itens a excluir
        :rtype: bool
        """
        context = context and context.copy() or {}

        # Itens a excluir
        itens = [[2, item_id, False] for item_id in self.pool.get('estoque.movimento.itens').search(
            cr, uid, [('nf_id', '=', ids[0])], context=context)]
        if not itens:
            return True

        # Calcula o valor total da nota fiscal
        brw_nf = self.browse(cr, uid, ids[0], context)
        vr_total = brw_nf.vr_seguro + brw_nf.vr_frete + brw_nf.vr_despesas_acessorias + \
                   brw_nf.vr_ipi - brw_nf.vr_desconto

        return self.write(cr, uid, ids, {'estoque_movimento_itens_ids': itens,
                                         'vr_total_nf': vr_total}, context)

    def button_cancelado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, float)):
            ids = [ids]
        brw_capa = self.browse(cr, uid, ids[0], context)
        if brw_capa.fisnfc_nf_frete_ids and brw_capa.tipo_frete == 'F':
            context['validacao_nf_frete'] = True
            for f in brw_capa.fisnfc_nf_frete_ids:
                self.verifica_executa_status(cr, uid, [f.id], 'C', context)

        return True

    def inutilizado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'I'}, context=context)
        return True

    def aguarda_inutilizacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': 'AI'}, context)

        return True

    # Metodo mantido para compatibilidade.
    # TODO: Alterar as chamadas a este metodo que nao sao originadas de uma ´view´ (button) para
    # chamar diretamente o metodo ´gerar_parcelas´
    def button_parcelas(self, cr, uid, ids, context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: sempre verdadeiro.
        :rtype: bool
        """
        context = context and context.copy() or {}
        nf_id = ids[0] if isinstance(ids, list) else ids
        nf_capa_dados = self.read(cr, uid, nf_id, ['nf_propria'], context)

        # Desativa as parcelas antigas de notas fiscais proprias e de terceiros
        nf_parcela_ids = self.pool.get('fiscal.nf.parcelas').search(
            cr, uid, [('fisnfc_nf_terceiro_id', '=', nf_id)], context=context)
        nf_parcela_ids.extend(self.pool.get('fiscal.nf.parcelas').search(
            cr, uid, [('fisnfc_nf_terceiro_id', '=', nf_id)], context=context))
        self.pool.get('fiscal.nf.parcelas').desativar_parcelas(cr, uid, nf_parcela_ids, context)

        # TODO: Remover a condicao do contexto se o sistema nao permitir que o usuario crie parcelas
        context['criar'] = "parcelas"

        # Cria as novas parcelas
        for parcela in self.gerar_parcelas(cr, uid, nf_id, context):
            if nf_capa_dados.get('nf_propria'):
                vals = dict(parcela, nf_id=nf_id)
            else:
                vals = dict(parcela, fisnfc_nf_terceiro_id=nf_id)
            self.pool.get('fiscal.nf.parcelas').create(cr, uid, vals, context)

        return True

    def onchange_nf_condicao(self, cr, user, ids, condicoes, emissao, controle='', context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID da nota fiscal,
        :param condicoes: condicoes de pagamento
        :param emissao: data de referencia para o calculo do vencimento das parcelas
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type condicoes: list
        :type emissao: str
        :type context: dict
        :return: lista com os dados das parcelas
        :rtype: list
        """
        context = context and context.copy() or {}
        if controle == 'P':  # 'P'=vem do onchange das parcelas
            return {'value': {'fisnfc_controle_onchange_parcelas': ''}}

        nf_parcelas = parcelas = list()
        obj_utils = self.pool.get('adm.utils')

        # Se a nota fiscal for eletronica, a emissao serah sempre a data de hoje
        if ids and self.browse(cr, user, ids[0], context).modelo_nf_id.fismnf_tipo_nfe:
            emissao = datetime.strftime(obj_utils.admutl_converte_tz(
                cr, user, datetime.now(), 'UTC', context=context), '%Y-%m-%d')
        else:
            emissao = datetime.strftime(obj_utils.admutl_converte_tz(
                cr, user, datetime.strptime(emissao, '%Y-%m-%d %H:%M:%S'), 'UTC', context=context),
                '%Y-%m-%d')

        # Parcelas em funcao das condicoes de pagamento
        for condicao in condicoes:
            if not condicao[0]:  # Item novo
                condicao_id = self.pool.get('adm.condicao.forma').browse(
                    cr, user, condicao[2].get('condicao_forma_id'), context).adm_condicao_id.id

                # Inclui as parcelas para a condicao de pagamento
                parcelas = [[0, 0, p] for p in self.pool.get('adm.condicao').gerar_parcelas(
                    cr, user, condicao_id, condicao[2].get('valor'), emissao, context)]
                condicao[2]['nf_parcela_ids'] = parcelas
            elif condicao[0] == 1:  # Item alterado
                # Recupera as parcelas da condicao
                parcela_ids = self.pool.get('fiscal.nf.parcelas').search(
                    cr, user, [('nf_condicao_id', '=', condicao[1])])
                nf_parcela_dados = self.pool.get('fiscal.nf.parcelas').read(
                    cr, user, parcela_ids, ['vencimento', 'valor'], context)
                parcelas_antigas = [[1, p.get('id'), {'valor': p.get('valor'),
                                                      'vencimento': p.get('vencimento')}]
                                    for p in nf_parcela_dados]

                # Verifica se a condicao foi alterada
                if 'condicao_forma_id' in condicao[2]:
                    condicao_id = self.pool.get('adm.condicao.forma').browse(
                        cr, user, condicao[2]['condicao_forma_id'], context).adm_condicao_id.id
                else:
                    condicao_id = self.pool.get('fiscal.condicao.pagamento').browse(
                        cr, user, condicao[1], context).condicao_forma_id.adm_condicao_id.id

                # Verifica se o valor da condicao foi alterado
                if 'valor' in condicao[2]:
                    condicao_valor = condicao[2]['valor']
                else:
                    condicao_valor = self.pool.get('fiscal.condicao.pagamento').browse(
                        cr, user, condicao[1], context).valor

                parcelas = self.pool.get('adm.condicao').gerar_parcelas(
                    cr, user, condicao_id, condicao_valor, emissao, context)

                # Atualiza as parcelas
                qtde_parcelas, qtde_parcelas_antigas = len(parcelas), len(parcelas_antigas)
                diferenca = qtde_parcelas - qtde_parcelas_antigas
                if diferenca > 0:  # Numero de parcelas maior
                    parcela_extra = [[0, 0, p] for p in parcelas[diferenca * - 1:]]
                    parcelas = [[p1[0], p1[1], p1[2].update(p2) or p1[2]]
                                for p1, p2 in zip(parcelas_antigas, parcelas[:diferenca * - 1])]
                    parcelas.extend(parcela_extra)
                    condicao[2]['nf_parcela_ids'] = parcelas
                elif diferenca < 0:  # Numero de parcelas  menor
                    parcelas = [[p1[0], p1[1], p1[2].update(p2) or p1[2]]
                                for p1, p2 in zip(parcelas_antigas[:diferenca], parcelas)]
                    parcela_extra = [[2, p[1], False] for p in parcelas_antigas[diferenca:]]
                    condicao[2]['nf_parcela_ids'] = parcelas + parcela_extra
                else:  # O numero de parcelas eh igual
                    parcelas = [[p1[0], p1[1], p1[2].update(p2) or p1[2]]
                                for p1, p2 in zip(parcelas_antigas, parcelas)]
                    condicao[2]['nf_parcela_ids'] = parcelas
            elif condicao[0] == 2:  # Item excluido
                # Exclui as parcelas da condicao de pagamento
                parcela_ids = self.pool.get('fiscal.nf.parcelas').search(
                    cr, user, [('nf_condicao_id', '=', condicao[1])])
                parcelas = [[2, parcela_id, False] for parcela_id in parcela_ids]
            elif condicao[0] == 4:  # Item gravado
                parcelas = [[4, _id, False] for _id in self.pool.get(
                    'fiscal.nf.parcelas').search(cr, user, [('nf_condicao_id', '=', condicao[1])])]

            nf_parcelas.extend(parcelas)

        return {'value': {'nf_condicao_ids': condicoes, 'nf_parcelas_ids': nf_parcelas,
                          'fisnfc_controle_onchange_parcelas': 'C'}}

    def onchange_fisnfc_parcelas(self, cr, uid, ids, parcela_ids, cond_parcelas_ids, controle='', context=None):
        context = context and context.copy() or {}
        res = {}
        if controle == 'C':  # 'C'=vem do onchange da condicao
            res['value'] = {'fisnfc_controle_onchange_parcelas': ''}
            return res

        modificou = False
        for i in parcela_ids:
            if i[0] != 0:
                continue

            for j in cond_parcelas_ids:
                if modificou:
                    break

                if j[2] and j[2].get('nf_parcela_ids'):
                    for x in j[2].get('nf_parcela_ids'):
                        if modificou:
                            break

                        if x[2] and x[2].get('valor') == i[2].get('valor') and \
                                x[2].get('vencimento') != i[2].get('vencimento'):
                            x[2]['vencimento'] = i[2].get('vencimento')
                            modificou = True

        if modificou:
            res = {'value': {'nf_condicao_ids': cond_parcelas_ids,
                             'fisnfc_controle_onchange_parcelas': 'P'}}

        return res

    @log_debug
    def gerar_parcelas(self, cr, user, nf, context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param nf: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: lista com os dados das parcelas
        :rtype: list
        """
        context = context and context.copy() or {}
        nf_capa_dados = self.read(cr, user, nf, ['nf_propria'], context)
        if nf_capa_dados.get('nf_propria'):
            parcelas = self.gerar_parcelas_nf_propria(cr, user, nf, context)
        else:
            parcelas = self.gerar_parcelas_nf_terceiro(cr, user, nf, context)

        return parcelas

    @log_debug
    def gerar_parcelas_nf_terceiro(self, cr, user, nf, condicao_ids, context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento para um nota fiscal de
        terceiro.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param nf: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: lista com os dados das parcelas
        :rtype: list
        """
        from decimal import Decimal, ROUND_HALF_DOWN
        context = context and context.copy() or {}
        f_vencimento = lambda d, p, t: datetime.strptime(
            datas.sigDatas(d).getCalculaPrazo(p, t), '%Y%m%d').strftime('%Y-%m-%d')
        f_parcelas = lambda p, t, v, e: [{'prazo': p, 'valor': v, 'prazo_tipo': t,
                                          'vencimento': f_vencimento(e, p, t)} for p in p]
        f_round = lambda f: float(Decimal(f).quantize(Decimal(10) ** -2, ROUND_HALF_DOWN))

        # Recupera os dados da nota fiscal
        campos = ['vr_total_nf', 'fisnfc_valor_bonificado', 'prazo_tipo', 'prazo', 'data_emissao',
                  'vr_total_deducao']
        dados_nf = self.read(cr, user, nf, campos, context)
        vr_deducao = dados_nf.get('vr_total_deducao')
        vr_total_nf = dados_nf.get('vr_total_nf') - \
                      (vr_deducao + (dados_nf.get('fisnfc_valor_bonificado') or 0.0))
        dt_emissao = dados_nf.get('data_emissao')[:10]
        tp_prazo = dados_nf.get('prazo_tipo') or 'DD'
        prazos = map(lambda m: int(m), re.sub('[^0-9]', ' ', dados_nf.get('prazo') or '0').split())
        vr_parcela = f_round(vr_total_nf / len(prazos) if len(prazos) else 0.0)

        # Gera as parcelas
        result = f_parcelas(prazos, tp_prazo, vr_parcela, dt_emissao)

        # Ajusta o valor da ultima parcela
        diferenca = f_round(vr_total_nf - vr_parcela * len(prazos))

        if diferenca:
            result[len(prazos) - 1]['valor'] = vr_parcela + diferenca

        return result

    @log_debug
    def gerar_parcelas_nf_propria(self, cr, user, nf, context=None):
        """
        Gera as parcelas em funcao das formas e condicoes de pagamento para um nota fiscal propria.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: o ID do usuario utilizado para a verificacao de perfil,
        :param nf: ID da nota fiscal,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type nf: long
        :type context: dict
        :return: lista com os dados das parcelas
        :rtype: list
        """
        context = context and context.copy() or {}
        result = []

        # Recupera os dados da nota fiscal
        dados_nf = self.read(cr, user, nf, ['nf_condicao_ids', 'tipo', 'data_emissao'], context)
        if not dados_nf.get('nf_condicao_ids'):
            return result

        load = '_classic_write'
        vr_deducao = 0.0 # Nota fiscal de saida
        if dados_nf.get('tipo') == 'E':  # Nota fiscal de entrada:
            vr_deducao = self._fisnfc_get_total_deducao(cr, user, nf, '', ()).get(nf, 0.0)
        dict_condicao = {}
        cond_forma_ids = []
        campos = ['condicao_forma_id', 'valor']
        for i in self.pool.get('fiscal.condicao.pagamento').read(
                cr, user, dados_nf.get('nf_condicao_ids'), campos, context, load):
            dict_condicao[i.get('id')] = i
            cond_forma_ids.append(i.get('condicao_forma_id'))
        dict_cond_forma = {}
        map(lambda x: dict_cond_forma.update({x.get('id'): x}),
        self.pool.get('adm.condicao.forma').read(
            cr, user, cond_forma_ids, ['adm_condicao_id'], context, load))

        for condicao in dict_condicao.values():
            cond_forma = dict_cond_forma.get(condicao.get('condicao_forma_id'))
            valor_condicao_pagto = condicao.get('valor') - vr_deducao
            result = self.pool.get('adm.condicao').gerar_parcelas(
                cr, user, cond_forma.get('adm_condicao_id'), valor_condicao_pagto,
                dados_nf.get('data_emissao')[:10], context)

        return result

    def abrir_pedido(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, list):
            ids = ids[0]
        pedido_id = self.browse(cr, uid, ids, context).pedido_id.id
        if not pedido_id:
            raise osv.except_osv(_('Atencao'), _('Nao existe pedido para esta nota fiscal.'))

        return {'res_id': pedido_id, 'name': _('Pedidos'), 'view_type': 'form', 'view_mode': 'form',
                'res_model': 'estoquepedidos.pedido.capa', 'view_id': False,
                'type': 'ir.actions.act_window'}

    def abrir_cancelar_nfe(self, cr, uid, ids, context=None):
        """
        Realiza o cancelamento de uma nota fiscal.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: verdadeiro se a nota foi cancelada.
        :rtype: bool
        """
        context = context and context.copy() or {}

        # Verifica se o ´´status´´ da nota fiscal permite cancelar
        args = [('id', 'in' if isinstance(ids, list) else '=', ids),
                ('state', 'in', ['1', '2', '2E', '3', 'P'])]

        if not self.search(cr, uid, args, context=context):
            return False

        brw_nf = self.browse(cr, uid, ids[0], context)
        # NFC-e emitidas em contingencia devem estar faturadas para o seu cancelamento
        if brw_nf.nfe_forma_emissao == '2' and brw_nf.state != '3':
            raise osv.except_osv(_('Atencao!'),
                                 _('Para cancelar notas emitidas em contingência, '
                                   'primeiro elas devem estar autorizadas na SEFAZ!'))

        # Se nao houver numero de protocolo a nota nao pode ser cancelada
        if brw_nf.modelo_55 and brw_nf.nf_propria or brw_nf.fisnfc_consumidor:
            if brw_nf.state == '3' and not brw_nf.nfe_protocolo:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nota fiscal sem o numero de protocolo. Recupere o '
                                       'protocolo manualmente na aba "NF-e/NFC-e > Recuperar '
                                       'protocolo" (botao).'))

        # active_id e active_ids eh setado porque ao importar uma nota e tentar cancela-la o valor
        # deste atributos eh o id da tela de importacao e nao da nota.
        context['active_id'] = ids[0]
        context['active_ids'] = ids

        context['cancelamento_nf'] = True
        context['cancelamento_terceiros'] = True and context.get('terceiro') or False
        msg = _('Deseja realmente cancelar esta nota fiscal? Este processo e irreversivel.')
        # if not context.get('cancelamento_confirmado'):
        if not context.get('terceiro'):
            context['proprio'] = True
            if not context.get('registro_id'):
                context['registro_id'] = ids[0] if ids and isinstance(ids, list) else False
        id_info = self.pool.get('fiscal.exibe.questao.osv').create(
            cr, uid, {'name': msg, 'opcao_confirma': True}, context=context)

        return {'name': _('Informacao'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_id': id_info,
                'res_model': 'fiscal.exibe.questao.osv',
                'view_id': False,
                'context': context,
                'target': 'new',
                'type': 'ir.actions.act_window'}

    def fisnfc_button_salvar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}

        return True

    def fisnfc_button_rps_imprimir(self, cr, uid, ids, context=None):
        """
        Realiza a impressao do Recibo Provisorio de Servicos - RPS

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: relatorio do RPS.
        :rtype: object
        """
        context = context and context.copy() or {}
        for brw_nf in self.browse(cr, uid, ids, context=context):
            if not brw_nf.fisnfc_rps_numero:
                # Se ainda não tem numero de rps, gerar a numeração
                vals = {}
                vals['fisnfc_rps_numero'] = self.pool.get('adm.utils').get_sequence(
                    cr, uid, self._name, 'numero_rps.fiscal')
                vals['fisnfc_rps_data'] = datetime.now()
                self.write(cr, uid, ids, vals, context=context)
                self.message_post(
                    cr, uid, ids, 'NFS-e', 'Gerado RPS %s ' % vals.get('fisnfc_rps_numero'))
        result = {'type': 'ir.actions.report.xml', 'report_name': 'fiscal.nf.capa.rps',
                  'context': context}

        return result

    def get_propagar_inf(self, cr, uid, vals_nf_capa, vals_titulo):
        if not isinstance(vals_nf_capa, dict):
            vals_nf_capa = {}
        if not isinstance(vals_titulo, dict):
            vals_titulo = {}
        if not vals_nf_capa['tipo'] == 'S':
            return vals_titulo
        obj_adm_utils = self.pool.get('adm.utils')
        field_par_propaga_inf = ['fispar_propagar_informacoes_nf']
        propagar_inf = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'fiscal', field_par_propaga_inf, context=None)
        propagar_inf = propagar_inf.get('fispar_propagar_informacoes_nf')
        if not propagar_inf:
            return vals_titulo
        vals_titulo['observacoes'] = vals_nf_capa['fisnfc_infcomp_contribuinte']

        return vals_titulo

    def fisnfc_permissao_para_voltar_esboco(self, cr, uid, name=None):
        """
        Verifica se o usuario tem permissao para voltar a nota para esboco

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :type cr: object
        :type uid: int
        :rtype: boolean
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        # havera 'name' quando a funcao for chamada dos metodos funcionais de 'fiscal.nf.capa'
        name = name or []
        if name and 'fisnfc_permissao_voltar_esboco' not in name:
            return False

        tem_permissao = True
        obj_adm_utils = self.pool.get('adm.utils')
        fispar_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'fiscal.parametro', ['fispar_esboco_so_manager'])
        if isinstance(fispar_dados, dict):
            fispar_esboco_so_manager = fispar_dados.get('fispar_esboco_so_manager', False)
            if fispar_esboco_so_manager:
                grupos = "fiscal.group_fiscal_manager,prime.group_prime_manager"
                tem_permissao = self.user_has_groups(cr, uid, grupos)

        return tem_permissao

    @log_debug
    def fisnfc_deleta_fatura_prv_financeiro(self, cr, uid, ids, context):
        """
        Exclui fatura em previsao no financeiro

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das notas fiscais
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type context: dict
        :rtype: long

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if ids and not isinstance(ids, list):
            ids = [ids]
        obj_fin_fat = self.pool.get('financeiro.fatura')
        obj_doc_doc = self.pool.get('documento.documento.tipo')
        obj_estped_pedido = self.pool.get('estoquepedidos.pedido.capa')

        # Deletando previsoes de pedidos de compra
        dados_nota = self.read(cr, uid, ids, ['tipo', 'fisnfc_pedido_ids'], load='_classic_write',
                               context=context)
        for dado_nota in dados_nota:
            if dado_nota.get('fisnfc_pedido_ids') and dado_nota.get('tipo') == 'E':
                obj_estped_pedido.pedcap_exclui_titulo_prv_financeiro(cr, uid, dado_nota.get(
                    'fisnfc_pedido_ids'), context)
        doc_tip_prv_id = obj_doc_doc.search(cr, uid, [('sigla', '=', 'PRV')], context=context)
        args = [('nf_id', 'in', ids), ('finfat_documento_tipo_id', 'in', doc_tip_prv_id)]
        fatura_ids = obj_fin_fat.search(cr, uid, args, context=context)

        if fatura_ids:
            obj_fin_fat.unlink(cr, SUPERUSER_ID, fatura_ids, context)

        return True

    def fisnfc_verifica_data_saida_entrada(self, cr, uid, dados_nf, context=None):
        """
        Valida data de saida/entrada da nota fiscal.
        
        :param cr: o cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type nf: long
        :type context: dict 
        :return: verdadeiro se a data de saida/entrada for uma data valida
        :rtype: bool
        """
        context = context and context.copy() or {}

        def _format_to_datetime(data):
            """ Formata data (dia, mes, ano, hora, minuto e segundo) """
            if isinstance(data, str):
                data = datetime.strptime(data.replace('/', '-'), '%d-%m-%Y %H:%M:%S')

            return data

        obj_utils = self.pool.get('adm.utils')
        agora = datetime.now()
        data_saida_entrada = obj_utils.admutl_datetime_to_timezone(
            cr, uid, parse(dados_nf.get('data_saida_entrada')))
        agora = obj_utils.admutl_datetime_to_timezone(cr, uid, agora)
        if _format_to_datetime(data_saida_entrada) > _format_to_datetime(agora):
            msg = _("A data de saida/entrada nao pode ser maior que a data atual.\n" 
                    "Data informada: %s\n Data do servidor: %s") % (data_saida_entrada, agora)
            raise osv.except_osv(_('Atencao!'), msg)
        if dados_nf.get('data_saida_entrada') < dados_nf.get('data_emissao'):
            raise osv.except_osv(_('Atencao!'),
                                 _('A data de saida/entrada nao pode ser menor que a data de '
                                   'emissao.'))

        return True

    def fisnfc_onchange_despesas_extras_terceiro(self, cr, user, ids, despesa, itens, context=None):
        """
        Rateia o valor das depesas extras (complemento) entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param despesa: valor das despesas extras (complemento da nota)
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type despesa: float
        :type itens: list
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de despesas acessorias ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if despesa and not itens:
            return {'warning': {'title': _('Aviso.'),
                                'message': _('Nao existe item na nota fiscal para ratear o valor '
                                             'das despesas extras.')}}
        if self.fisnfc_get_total_despesas_extras(cr, user, itens, context) == despesa:
            return {}
        itens = self.fisnfc_rateio_despesas_extras(cr, user, itens, despesa, context)
        despesa = 0 if not itens else despesa

        return {'value': {'estoque_movimento_itens_ids': itens,
                          'fisnfc_vr_despesas_extras': despesa}}

    @log_debug
    def fisnfc_rateio_despesas_extras(self, cr, user, itens, despesa, context=None):
        """
        Calcula e rateia o valor das despesas extras entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param despesa: o valor das despesas extras na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type despesa: float
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de despesas extras ajustados
        :rtype: list
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], [
                'vr_total_item', 'vr_despesas_rateado', 'vr_desconto_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado', 'tributacoes_ids', 'estmvi_vr_despesas_extras_rateado'])
        o2m = lambda vals: [[1 if val[0] == 4 else val[0], val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]

        # Recupera os valores dos itens
        itens = o2m(itens)

        total = self.fisnfc_get_total_produtos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context)
        total += self.fisnfc_get_total_servicos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context) * 1.  # Sempre ´float´
        total_rateado = 0

        # Rateia o valor das despesas extras entre os itens nao rateados
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            item[2]['estmvi_vr_despesas_extras_rateado'] = \
                round(item[2].setdefault('vr_total_item', 0) / total * despesa if total else 0, 2)
            total_rateado += item[2].get('estmvi_vr_despesas_extras_rateado')
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if itens and total_rateado != despesa:
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item')))
            itens[index][2]['estmvi_vr_despesas_extras_rateado'] += despesa - total_rateado

        return itens

    def fisnfc_onchange_despesas_acessorias(self, cr, user, ids, despesa, itens, frete,
                                            nat_id, context=None):
        """
        Calcula e rateia o valor das depesas acessorias entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param despesa: valor da despesa acessoria
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param nat_id: id da natureza da nota
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type despesa: float
        :type itens: list
        :type frete: str
        :type nat_id: int
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de despesas acessorias ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if despesa and not itens:
            return {'warning': {'title': _('Aviso.'),
                                'message': _('Nao existe item na nota fiscal para ratear o valor '
                                             'da despesa.')}}
        if self.fisnfc_get_total_despesas_acessorias(cr, user, itens, context) == despesa:
            return {'value': {'fisnfc_despesa_acessoria_capa': True}}
        itens = self.fisnfc_rateio_despesas_acessorias(cr, user, itens, despesa, frete, nat_id,
                                                       context)
        despesa = 0 if not itens else despesa

        return {'value': {'estoque_movimento_itens_ids': itens, 'vr_despesas_ecessorias': despesa}}

    @log_debug
    def fisnfc_rateio_despesas_acessorias(self, cr, user, itens, despesa, frete, nat_id,
                                          context=None):
        """
        Calcula e rateia o valor das despesas acessorias entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param despesa: o valor das despesas acessorias na capa da nota fiscal
        :param frete: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param nat_id: id da natureza da nota
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type despesa: float
        :type frete: str
        :type nat_id: int
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de despesas acessorias ajustados
        :rtype: list
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], [
                'vr_total_item', 'vr_despesas_rateado', 'vr_desconto_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado', 'tributacoes_ids', 'estmvi_incide_despesa_bc'])
        o2m = lambda vals: [[1 if val[0] == 4 else val[0], val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]

        # Recupera os valores dos itens
        itens = o2m(itens)

        total = self.fisnfc_get_total_produtos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context)
        total += self.fisnfc_get_total_servicos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context) * 1.  # Sempre ´float´
        total_rateado = 0
        fisnit = self.pool.get('fiscal.nf.item.tributacao')

        # Rateia o valor da despesa acessoria entre os itens nao rateados
        total_despesas_rateado = sum(map(lambda s: s[2] and s[2].get('vr_despesas_rateado') or 0,
                                         itens))
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            if despesa != total_despesas_rateado:
                item[2]['vr_despesas_rateado'] = round(
                    item[2].setdefault('vr_total_item', 0) / total * despesa if total else 0, 2)
            total_rateado += item[2].get('vr_despesas_rateado') or 0

            # Ajusta o valor da base de calculo e valor do encargo
            tributacoes = [[tributacao[0], tributacao[1], tributacao[0] in [0, 1, 4] and adjust_m2o(
                            self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                                cr, user, 'tributacoes_ids', [tributacao])[0]) or False]
                           for tributacao in item[2].get('tributacoes_ids') or []]
            for tributacao in filter(lambda f: f[0] in [0, 1, 4], tributacoes):
                vr_frete = item[2].get('vr_frete_rateado') or 0 if frete == 'D' else 0
                incide_despesa = item[2].get('estmvi_incide_despesa_bc')
                tributacao_id = tributacao[2].get('tributacao_id')
                percentual_base = tributacao_id and self.pool.get('fiscal.tributacao').browse(
                    cr, user, tributacao_id).base
                percentual_base = percentual_base or 0  # False ou None => 0
                if percentual_base:
                    vr_base_calculo = (item[2].get('vr_total_item') or 0) + vr_frete \
                                      + (item[2].get('vr_seguro_rateado') or 0) \
                                      + (incide_despesa and (item[2].get('vr_despesas_rateado') or
                                                             0) or 0) \
                                      - (item[2].get('vr_desconto_rateado') or 0)
                else:
                    vr_base_calculo = tributacao[2]['vr_base_calculo']
                vr_despesa = item[2].get('vr_despesas_rateado') or 0
                tributacao[2].update(fisnit.fisnit_get_tributacao_valor(cr, user, tributacao_id,
                                 vr_base_calculo, context, vr_despesa, incide_despesa, nat_id))

            # Ajusta os valores das tributacoes do item
            item[2]['tributacoes_ids'] = tributacoes
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if itens and total_rateado != despesa:
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item', 0)))
            itens[index][2]['vr_despesas_rateado'] += despesa - total_rateado
            for tributo in filter(lambda f: f[0] in [0, 1, 4], itens[index][2]['tributacoes_ids']):
                tributo[2]['vr_base_calculo'] += despesa - total_rateado
                tributo[2]['vr_encargo'] = fisnit.fisnit_calcula_vr_encargo(
                    tributo[2]['vr_base_calculo'], tributo[2]['aliquota'])

                # Ajusta o valor do ICMS desonerado
                if tributacao[2].get('fisnit_vr_icms_desonerado'):
                    tributacao[2]['fisnit_vr_icms_desonerado'] = tributacao[2]['vr_encargo']

        return itens

    def fisnfc_onchange_icms_st_terceiro(self, cr, user, ids, itens, icms, context=None):
        """
        Rateia o valor do ICMS/ST entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param icms: o valor do ICMS/ST na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type icms: float
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de ICMS/ST ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}

        # Somente itens com ICMS/ST
        itens_icms_st = self.fisnfc_get_item_icms_st(cr, user, itens, context)

        # Os itens que nao tem ICMS/ST
        itens = [item for item in itens if item not in itens_icms_st]

        vr_total_icms_st = self.fisnfc_get_total_icms_substituicao(cr, user, itens_icms_st, context)
        if vr_total_icms_st == icms:
            return {}

        itens_icms_st = self.fisnfc_rateio_icms_st(cr, user, itens_icms_st, icms, context)
        warning = {}
        if not itens_icms_st:
            icms = 0
            warning = {'title': _('Aviso.'), 'message': _(
                'Nao existe item na nota fiscal com valor de substituicao tributaria '
                'para ratear o valor.')}

        return {'value': {'estoque_movimento_itens_ids': itens + itens_icms_st,
                          'vr_icms_substituicao': icms}, 'warning': warning}

    @log_debug
    def fisnfc_rateio_icms_st(self, cr, user, itens, icms, context=None):
        """
        Rateia o valor do ICMS/ST entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param icms: o valor do ICMS/ST na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type icms: float
        :type context: dict
        :return: lista com os itens da nota fiscal e os respectivos valores de ICMS/ST ajustados.
        :rtype: list
        """
        context = context and context.copy() or {}

        # Rateia o valor da base de ICMS/ST
        total = self.fisnfc_get_total_produtos(cr, user, itens, context)
        total += self.fisnfc_get_total_servicos(cr, user, itens, context) * 1.  # Sempre ´float´
        total_rateado = 0

        for item in itens:
            # Recupera os valores do item
            value = self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', [item],
                ['tributacoes_ids', 'vr_total_item'])[0]

            # Atualiza os valores do item
            if item[2]:
                value.update(item[2])

            item[2] = value
            tributacoes = []
            for tributacao in item[2].get('tributacoes_ids'):
                cst = self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', [tributacao], ['cst_id'])[0].get('cst_id')
                cst_id = cst[0] if isinstance(cst, (list, tuple)) else cst
                brw_cst = self.pool.get('fiscal.cst').browse(cr, user, cst_id, context)

                # Encargo ICMS/ST
                if brw_cst.encargo_id.sigla.upper() == 'ICMS' and brw_cst.tipo in ('U', 'TU'):
                    icms_st = round(value.get('vr_total_item', 0) / total * icms if total else 0, 2)
                    total_rateado += icms_st
                    item[0] = 1 if item[0] == 4 else item[0]
                    if isinstance(tributacao, (int, long)):
                        tributacao = [1, tributacao, {'vr_icms_substituicao': icms_st}]
                    else:
                        tributacao[0] = 1 if tributacao[0] == 4 else tributacao[0]
                        if not tributacao[2]:
                            tributacao[2] = {'vr_icms_substituicao': icms_st}
                        else:
                            tributacao[2].update({'vr_icms_substituicao': icms_st})
                else:
                    if isinstance(tributacao, (int, long)):
                        tributacao = [4, tributacao, False]

                tributacoes.append(tributacao)
            item[2]['tributacoes_ids'] = tributacoes
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if itens and total_rateado and total_rateado != icms:
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item')))

            # Recuperar a tributacao com maior valor de ICMS/ST (vr_icms_substituicao)
            tributacao = max(itens[index][2].get('tributacoes_ids'),
                             key=lambda k: k[2] and k[2].get('vr_icms_substituicao'))
            tributacao[2]['vr_icms_substituicao'] += icms - total_rateado

        return itens

    def fisnfc_onchange_base_icms_st_terceiro(self, cr, user, ids, itens, base_icms, context=None):
        """
        Calcula e rateia o valor da base de ICMS/ST entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param base_icms: o valor base de ICMS da substituicao na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type base_icms: float
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de base de ICMS/ST ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}

        # Somente itens com ICMS/ST
        itens_base_icms_st = self.fisnfc_get_item_icms_st(cr, user, itens, context)

        # Os itens que nao tem ICMS/ST
        itens = [item for item in itens if item not in itens_base_icms_st]

        vr_total_base_icms_st = self.fisnfc_get_total_base_icms_substituicao(
            cr, user, itens_base_icms_st, context)
        if vr_total_base_icms_st == base_icms:
            return {}
        itens_base_icms_st = self.fisnfc_rateio_base_icms_st(
            cr, user, itens_base_icms_st, base_icms, context)
        warning = {}
        if not itens_base_icms_st:
            base_icms = 0
            warning = {'title': _('Aviso.'),
                       'message': _('Nao existe item na nota fiscal com valor de substituicao '
                                    'tributaria para ratear o valor.')}

        return {'value': {'estoque_movimento_itens_ids': itens + itens_base_icms_st,
                          'vr_base_icms_substituicao': base_icms}, 'warning': warning}

    @log_debug
    def fisnfc_rateio_base_icms_st(self, cr, user, itens, base, context=None):
        """
        Rateia o valor da base do ICMS/ST entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param base: o valor da base do ICMS/ST na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type base: float
        :type context: dict
        :return: lista com os itens da nota fiscal e os respectivos valores de base de ICMS/ST
        ajustados.
        :rtype: list
        """
        context = context and context.copy() or {}

        # Rateia o valor da base de ICMS/ST
        total = self.fisnfc_get_total_produtos(cr, user, itens, context)
        total += self.fisnfc_get_total_servicos(cr, user, itens, context) * 1.  # Sempre ´float´
        total_rateado = 0

        for item in itens:
            # Recupera os valores do item
            value = self.resolve_2many_commands(
                cr, user, 'estoque_movimento_itens_ids', [item],
                ['tributacoes_ids', 'vr_total_item'])[0]

            # Atualiza os valores do item
            if item[2]:
                value.update(item[2])

            item[2] = value
            tributacoes = list()
            for tributacao in item[2].get('tributacoes_ids'):
                cst = self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', [tributacao], ['cst_id'])[0].get('cst_id')
                cst_id = cst[0] if isinstance(cst, (list, tuple)) else cst
                brw_cst = self.pool.get('fiscal.cst').browse(cr, user, cst_id, context)

                # Encargo ICMS/ST
                if brw_cst.encargo_id.sigla.upper() == 'ICMS' and brw_cst.tipo in ('U', 'TU'):
                    base_icms_st = \
                        round(value.get('vr_total_item', 0) / total * base if total else 0, 2)
                    total_rateado += base_icms_st
                    item[0] = 1 if item[0] == 4 else item[0]
                    if isinstance(tributacao, (int, long)):
                        tributacao = [1, tributacao, {'vr_base_icms_substituicao': base_icms_st}]
                    else:
                        tributacao[0] = 1 if tributacao[0] == 4 else tributacao[0]
                        if not tributacao[2]:
                            tributacao[2] = {'vr_base_icms_substituicao': base_icms_st}
                        else:
                            tributacao[2].update({'vr_base_icms_substituicao': base_icms_st})
                else:
                    if isinstance(tributacao, (int, long)):
                        tributacao = [4, tributacao, False]

                tributacoes.append(tributacao)
            item[2]['tributacoes_ids'] = tributacoes
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if itens and total_rateado and total_rateado != base:
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item')))

            # Recuperar a tributacao com maior valor de base de ICMS/ST (vr_base_icms_substituicao)
            tributacao = max(itens[index][2].get('tributacoes_ids'),
                             key=lambda k: k[2] and k[2].get('vr_base_icms_substituicao'))
            tributacao[2]['vr_base_icms_substituicao'] += base - total_rateado

        return itens

    @log_debug
    def fisnfc_rateio_icms_frete(self, cr, user, itens, icms, context=None):
        """
        Calcula e rateia o valor das despesas extras entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param icms: o valor do ICMS do frete na nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type icms: float
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de ICMS do frete ajustados
        :rtype: dict
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], [
                'vr_total_item', 'vr_despesas_rateado', 'vr_desconto_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado', 'tributacoes_ids', 'estmvi_vr_icms_frete_rateado'])
        o2m = lambda vals: [[1 if val[0] == 4 else val[0], val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]

        # Recupera os valores dos itens
        itens = o2m(itens)

        total = self.fisnfc_get_total_produtos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context)
        total += self.fisnfc_get_total_servicos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context) * 1.  # Sempre ´float´
        total_rateado = 0

        # Rateia o valor do ICMS do frete entre os itens nao rateados
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            item[2]['estmvi_vr_icms_frete_rateado'] = \
                round(item[2].setdefault('vr_total_item', 0) / total * icms if total else 0, 2)
            total_rateado += item[2].get('estmvi_vr_icms_frete_rateado')
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if itens and total_rateado != icms:
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item')))
            itens[index][2]['estmvi_vr_icms_frete_rateado'] += icms - total_rateado

        return itens

    def fisnfc_onchange_seguro(self, cr, user, ids, itens, seguro, frete, nat_id, context=None):
        """
        Rateia o valor do seguro entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param seguro: o valor do seguro na capa da nota fiscal
        :param frete: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type seguro: float
        :type frete: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if seguro and not itens:
            return {'warning': {'title': _('Aviso.'),
                                'message': _('Nao existe item na nota fiscal para ratear o valor '
                                             'do seguro.')}}
        vr_total_seguro = self.fisnfc_get_total_seguro(cr, user, itens, context)
        if vr_total_seguro == seguro:
            return {}
        itens = self.fisnfc_rateio_seguro(cr, user, itens, seguro, frete, nat_id, context)
        if not itens:
            seguro = 0

        return {'value': {'estoque_movimento_itens_ids': itens}, 'vr_seguro': seguro}

    @log_debug
    def fisnfc_rateio_seguro(self, cr, user, itens, seguro, frete, nat_id, context):
        """
        Rateia o valor do seguro entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param seguro: o valor do seguro na capa da nota fiscal
        :param frete: o tipo do frete: ´D´= "por dentro"  ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type seguro: float
        :type frete: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores do seguro ajustados.
        :rtype: list
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], [
                'vr_total_item', 'vr_despesas_rateado', 'vr_desconto_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado', 'tributacoes_ids', 'estmvi_incide_despesa_bc'])
        o2m = lambda vals: [[1 if val[0] == 4 else val[0], val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]

        # Recupera os valores dos itens
        itens = o2m(itens)

        total = self.fisnfc_get_total_produtos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context)
        total += self.fisnfc_get_total_servicos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context) * 1.  # Sempre ´float´
        total_rateado = 0
        fisnit = self.pool.get('fiscal.nf.item.tributacao')
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]

        # Rateia o valor do seguro (capa) entre os itens nao rateados
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            item[2]['vr_seguro_rateado'] = \
                round(item[2].setdefault('vr_total_item', 0) / total * seguro if total else 0, 2)
            total_rateado += item[2]['vr_seguro_rateado']

            # Ajusta o valor da base de calculo e valor do encargo
            tributacoes = [[tributacao[0], tributacao[1], tributacao[0] in [0, 1, 4] and adjust_m2o(
                self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', [tributacao])[0]) or False]
                           for tributacao in item[2].get('tributacoes_ids')]
            for tributacao in filter(lambda f: f[0] in [0, 1, 4], tributacoes):
                vr_frete = item[2].get('vr_frete_rateado') or 0 if frete == 'D' else 0
                incide_despesa = item[2].get('estmvi_incide_despesa_bc')
                tributacao_id = tributacao[2].get('tributacao_id')
                percentual_base = tributacao_id and self.pool.get('fiscal.tributacao').browse(
                    cr, user, tributacao_id).base or 0
                percentual_base = percentual_base or 0  # False ou None => 0x
                if percentual_base:
                    vr_base_calculo = (item[2].get('vr_total_item') or 0) + vr_frete \
                                      + (item[2].get('vr_seguro_rateado') or 0) \
                                      + (incide_despesa and (item[2].get('vr_despesas_rateado') or
                                                             0) or 0) \
                                      - (item[2].get('vr_desconto_rateado') or 0)
                else:
                    vr_base_calculo = tributacao[2]['vr_base_calculo']
                vr_despesa = item[2].get('vr_despesas_rateado') or 0
                tributacao[2].update(fisnit.fisnit_get_tributacao_valor(cr, user, tributacao_id,
                                                                        vr_base_calculo, context,
                                                                        vr_despesa, incide_despesa,
                                                                        nat_id))

            # Ajusta os valores das tributacoes do item
            item[2]['tributacoes_ids'] = tributacoes
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if itens and total_rateado != seguro:
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item', 0)))
            itens[index][2]['vr_seguro_rateado'] += seguro - total_rateado
            for tributo in filter(lambda f: f[0] in [0, 1, 4], itens[index][2]['tributacoes_ids']):
                tributo[2]['vr_base_calculo'] += seguro - total_rateado
                tributo[2]['vr_encargo'] = fisnit.fisnit_calcula_vr_encargo(
                    tributo[2]['vr_base_calculo'], tributo[2]['aliquota'])

                # Ajusta o valor do ICMS desonerado
                if tributacao[2].get('fisnit_vr_icms_desonerado'):
                    tributacao[2]['fisnit_vr_icms_desonerado'] = tributacao[2]['vr_encargo']

        return itens

    def fisnfc_get_total_seguro(self, cr, user, itens, context=None):
        """
        Calcula o valor total do seguro.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total do seguro.
        :rtype: float
        """

        context = context and context.copy() or {}

        return round(sum(map(lambda m: m.get('vr_seguro_rateado') or 0, self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', itens, ['vr_seguro_rateado']))), 2)

    def fisnfc_onchange_vr_frete(self, cr, user, ids, tipo_nf, modelo_nf_id, itens, vr_frete,
                                 tipo_frete, nat_id, context=None):
        """
        Rateia o valor do frete entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param tipo_nf: tipo da nota fiscal: ´E´= Entrada ou ´S´= Saida
        :param modelo_nf_id: o ID do modelo da nota
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param vr_frete: o valor do frete na capa da nota fiscal
        :param tipo_frete: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type tipo_nf: str
        :type modelo_nf_id: long
        :type itens: list
        :type vr_frete: float
        :type tipo_frete: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        result = {}
        if vr_frete and not itens:
            return dict(result, warning={'title': _('Aviso.'), 'message': _(
                'Nao existe item na nota fiscal para ratear o valor do frete.')})
        fismnf_tipo = modelo_nf_id and self.pool.get('fiscal.modelo.nf').read(
            cr, user, modelo_nf_id, ['fismnf_tipo'], context=context).get('fismnf_tipo') or ''
        if not (tipo_frete in ('', 'D') or fismnf_tipo == 'T' or tipo_nf == 'E'):
            vr_frete = 0
        itens = self.fisnfc_rateio_frete(cr, user, itens, vr_frete, tipo_frete, nat_id, context)
        result.update({'value': {'estoque_movimento_itens_ids': itens, 'vr_frete': vr_frete}})

        return result

    def fisnfc_get_total_frete(self, cr, user, itens, context=None):
        """
        Calcula o valor total do frete.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total do frete.
        :rtype: float
        """
        context = context and context.copy() or {}

        return round(sum(map(lambda m: m.get('vr_frete_rateado') or 0, self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', itens, ['vr_frete_rateado']))), 2)

    @log_debug
    def fisnfc_rateio_frete(self, cr, user, itens, frete, tipo, nat_id, context):
        """
        Rateia o valor do frete entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: o valor do frete na capa da nota fiscal
        :param tipo: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type frete: float
        :type tipo: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores do frete ajustados.
        :rtype: list
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], [
                'vr_total_item', 'vr_despesas_rateado', 'vr_desconto_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado', 'tributacoes_ids', 'estmvi_incide_despesa_bc'])
        o2m = lambda vals: [[1 if val[0] == 4 else val[0], val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]

        # Recupera os valores dos itens
        itens = o2m(itens)

        total = self.fisnfc_get_total_produtos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context)
        total += self.fisnfc_get_total_servicos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context) * 1.  # Sempre ´float´
        total_rateado = 0
        fisnit = self.pool.get('fiscal.nf.item.tributacao')
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]

        # Rateia o valor do frete (capa) entre os itens nao rateados
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            item[2]['vr_frete_rateado'] = \
                round(item[2].setdefault('vr_total_item', 0) / total * frete if total else 0, 2)
            total_rateado += item[2]['vr_frete_rateado']

            # Ajusta o valor da base de calculo e valor do encargo
            tributacoes = []
            # quando as tributacoes sao 'None' ocorre erro sem este 'if'
            if item[2] and item[2].get('tributacoes_ids'):
                for tributacao in item[2].get('tributacoes_ids'):
                    tributacoes.append([tributacao[0], tributacao[1], tributacao[0] in [0, 1, 4] and adjust_m2o(
                        self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                            cr, user, 'tributacoes_ids', [tributacao])[0]) or False])

            for tributacao in filter(lambda f: f[0] in [0, 1, 4], tributacoes):
                vr_frete = item[2].get('vr_frete_rateado') or 0 if tipo == 'D' else 0
                incide_despesa = item[2].get('estmvi_incide_despesa_bc')
                tributacao_id = tributacao[2].get('tributacao_id')
                percentual_base = tributacao_id and self.pool.get('fiscal.tributacao').browse(
                    cr, user, tributacao_id).base or 0
                percentual_base = percentual_base or 0  # False ou None => 0x
                if percentual_base:
                    vr_base_calculo = (item[2].get('vr_total_item') or 0) + vr_frete \
                                      + (item[2].get('vr_seguro_rateado') or 0) \
                                      + (incide_despesa and (item[2].get('vr_despesas_rateado') or
                                                             0) or 0) \
                                      - (item[2].get('vr_desconto_rateado') or 0)
                else:
                    vr_base_calculo = tributacao[2]['vr_base_calculo']
                vr_despesa = item[2].get('vr_despesas_rateado') or 0
                tributacao[2].update(fisnit.fisnit_get_tributacao_valor(cr, user, tributacao_id,
                                                                        vr_base_calculo, context,
                                                                        vr_despesa, incide_despesa,
                                                                        nat_id))
            # Ajusta os valores das tributacoes do item
            item[2]['tributacoes_ids'] = tributacoes
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        if filter(lambda f: f[0] in [0, 1, 4], itens):
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item', 0)))
            itens[index][2]['vr_frete_rateado'] += frete - total_rateado
            for tributo in filter(lambda f: f[0] in [0, 1, 4], itens[index][2]['tributacoes_ids']):
                tributo[2]['vr_base_calculo'] += frete - total_rateado
                tributo[2]['vr_encargo'] = fisnit.fisnit_calcula_vr_encargo(
                    tributo[2]['vr_base_calculo'], tributo[2]['aliquota'])

        return itens

    def fisnfc_onchange_desconto_terceiro(self, cr, user, ids, itens, desconto, frete, nat_id,
                                          context=None):
        """
        Rateia o valor do desconto informado na capa entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param desconto: o valor do desconto na capa da nota fiscal
        :param frete: o tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type desconto: float
        :type frete: str
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de desconto (capa) ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        avisos = list()
        if desconto and not itens:
            avisos.append(_('* Nao existe item na nota fiscal para ratear o valor do desconto.'))
        vr_total_desconto = self.fisnfc_get_total_desconto(cr, user, itens, context)
        if vr_total_desconto == desconto:
            return {}
        if not itens:
            desconto = 0

        # Nao permite desconto com valor negativo
        if desconto < 0:
            desconto = 0
            avisos.append(_('* Desconto nao permite valor negativo.'))

        itens = self.fisnfc_rateio_desconto(cr, user, itens, desconto, frete, nat_id, context)
        if avisos:
            aviso = {'title': _('Aviso.'), 'message': '\n'.join(avisos)}
        else:
            aviso = {}

        return {'value': {'estoque_movimento_itens_ids': itens, 'vr_desconto': desconto},
                'warning': aviso}

    def fisnfc_get_total_desconto(self, cr, user, itens, context=None):
        """
        Calcula o valor total do desconto rateado da capa.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type context: dict
        :return: o valor total do desconto.
        :rtype: float
        """
        context = context and context.copy() or {}

        return round(sum(map(lambda m: m.get('vr_desconto_rateado') or 0,
                             self.resolve_2many_commands(
                                 cr, user, 'estoque_movimento_itens_ids', itens,
                                 ['vr_desconto_rateado']))), 2)

    @log_debug
    def fisnfc_rateio_desconto(self, cr, user, itens, desconto, frete, nat_id, context):
        """
        Rateia o valor do desconto aplicado na capa entre os itens.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param desconto: o valor do desconto na capa da nota fiscal
        :param frete: o tipo do frete: ´D´= "por dentro"  ou ´F´= "por fora"
        :param nat_id: id da natureza da nota
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type itens: list
        :type desconto: float
        :type frete: str
        :type nat_id: int
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores do desconto (capa) ajustados.
        :rtype: list
        """
        context = context and context.copy() or {}
        r2mc = lambda cmd: self.resolve_2many_commands(
            cr, user, 'estoque_movimento_itens_ids', [cmd], [
                'vr_total_item', 'vr_despesas_rateado', 'vr_desconto_rateado', 'vr_frete_rateado',
                'vr_seguro_rateado', 'tributacoes_ids', 'estmvi_incide_despesa_bc'])
        o2m = lambda vals: [[1 if val[0] else 0, val[1],
                             r2mc(val) and adjust_m2o(r2mc(val)[0]) or False] for val in vals]

        # Recupera os valores dos itens
        itens = o2m(itens)

        total = self.fisnfc_get_total_produtos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context)
        total += self.fisnfc_get_total_servicos(
            cr, user, filter(lambda f: f[0] in [0, 1, 4], itens), context) * 1.  # Sempre ´float´
        total_rateado = 0
        fisnit = self.pool.get('fiscal.nf.item.tributacao')
        icms_ids = self.pool.get('fiscal.encargo').search(
            cr, user, [('sigla', 'ilike', 'ICMS')], context=context)
        icms_id = icms_ids and icms_ids[0]

        # Rateia o valor do desconto (capa) entre os itens nao rateados
        for item in filter(lambda f: f[0] in [0, 1, 4], itens):
            item[2]['vr_desconto_rateado'] = \
                round(item[2].setdefault('vr_total_item', 0) / total * desconto if total else 0, 2)
            total_rateado += item[2]['vr_desconto_rateado']

            # Ajusta o valor da base de calculo e valor do encargo
            tributacoes = [[tributacao[0], tributacao[1], tributacao[0] in [0, 1, 4] and adjust_m2o(
                self.pool.get('estoque.movimento.itens').resolve_2many_commands(
                    cr, user, 'tributacoes_ids', [tributacao])[0]) or False]
                           for tributacao in item[2].get('tributacoes_ids')]
            for tributacao in filter(lambda f: f[0] in [0, 1, 4], tributacoes):
                vr_frete = item[2].get('vr_frete_rateado') or 0 if frete == 'D' else 0
                incide_despesa = item[2].get('estmvi_incide_despesa_bc')
                tributacao_id = tributacao[2].get('tributacao_id')
                percentual_base = tributacao_id and self.pool.get('fiscal.tributacao').browse(
                    cr, user, tributacao_id).base or 0
                percentual_base = percentual_base or 0  # False ou None => 0
                if percentual_base:
                    vr_base_calculo = (item[2].get('vr_total_item') or 0) + vr_frete \
                                      + (item[2].get('vr_seguro_rateado') or 0) \
                                      + (incide_despesa and (item[2].get('vr_despesas_rateado') or
                                                             0) or 0) \
                                      - (item[2].get('vr_desconto_rateado') or 0)
                else:
                    vr_base_calculo = tributacao[2]['vr_base_calculo']
                vr_despesa = item[2].get('vr_despesas_rateado') or 0
                tributacao[2].update(fisnit.fisnit_get_tributacao_valor(
                    cr, user, tributacao_id, vr_base_calculo, context, vr_despesa, incide_despesa,
                    nat_id))
                tributacao[0] = tributacao[0] and 1 or 0
            # Ajusta os valores das tributacoes do item
            item[2]['tributacoes_ids'] = tributacoes
            item[2] = adjust_m2o(item[2])

        # Ajusta o valor rateado
        dif = desconto - total_rateado
        if itens and (dif <= -0.001 or 0.001 <= dif):
            # Recupera o item com o maior valor (vr_total_item)
            index = itens.index(max(itens, key=lambda k: k[2] and k[2].get('vr_total_item', 0)))
            itens[index][2]['vr_desconto_rateado'] += dif
            for tributo in filter(lambda f: f[0] in [0, 1, 4], itens[index][2]['tributacoes_ids']):
                tributo[2]['vr_base_calculo'] += dif * -1
                tributo[2]['vr_encargo'] = fisnit.fisnit_calcula_vr_encargo(
                    tributo[2]['vr_base_calculo'], tributo[2]['aliquota'])

                # Ajusta o valor do ICMS desonerado
                if tributo[2].get('fisnit_vr_icms_desonerado'):
                    tributo[2]['fisnit_vr_icms_desonerado'] = tributo[2]['vr_encargo']

        return itens

    @log_debug
    def fisnfc_verifica_cst_pis_cofins(self, cr, uid, dados_nf, dict_itens, dict_produtos,
                                       dict_trib, dict_encargos, dict_cst, context=None):
        context = context and context.copy() or {}
        if not dados_nf.get('nf_propria') or dados_nf.get('tipo') == 'S':
            obj_fisnit = self.pool.get('fiscal.nf.item.tributacao')
            natureza_pessoa_juridica = self.pool.get('adm.utils').admutl_get_parametros(
                cr, uid, 'fiscal', 'fispar_indicador_natureza_pessoa_juridica',
                context=context).get('fispar_indicador_natureza_pessoa_juridica')
            for item_id in dict_itens:
                item = dict_itens.get(item_id)
                produto = dict_produtos.get(item.get('produto_id'))
                valida_cst_pis_cofins = obj_fisnit.fisnit_valida_cst_pis_cofins(
                    cr, uid, item,  dict_trib, dict_encargos, dict_cst, context)
                if not valida_cst_pis_cofins and natureza_pessoa_juridica != '02':
                    msg = 'Item numero %s - %s possui CST de PIS diferente do CST de COFINS' % (
                        item.get('sequencia'), produto.get('name'))
                    raise osv.except_osv(_('Atencao'), _(msg))

        return True

    def fisnfc_get_documento_id(self, cr, uid, adm_forma_pgto_id, sigla_alternativa=None,
                                context=None):
        context = context and context.copy() or {}
        obj_doc_tip = self.pool.get('documento.documento.tipo')
        obj_adm_form_pgto = self.pool.get('adm.forma.pgto')
        res = False
        doc_tip = obj_adm_form_pgto.read(
            cr, uid, adm_forma_pgto_id, ['admfpg_documento_tipo_id'], context, '_classic_write')
        if doc_tip:
            res = doc_tip.get('admfpg_documento_tipo_id')
        else:
            if sigla_alternativa:
                args = [('sigla', '=', sigla_alternativa)]
                doc_tip = obj_doc_tip.seach(cr, uid, args)
                for doc_tip_id in doc_tip:
                    res = doc_tip_id
        return res

    def fisnfc_criar_movimento_financeiro_2D(self, cr, uid, dados_nf, dados_condicao,
                                             dict_forma_pgto, debito_credito, context=None):
        """
        Este metodo gera movimento direto na tesouraria para condicoes de pagamentos
        de notas fiscais modelo 2D que nao possuam cliente na condicao e nem cliente na capa
        da nota.
        Caso a nota seja modelo 2D, mais possua cliente em suas condicoes de pagamento ou na
        capa da nota, este metodo nao deve ser chamado, e sim os metodos para criacao de titulos
        no financeiro

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param condicao_id: ID da condicao de fiscal.condicao.pagamento para criar o movimento
        :param debito_credito: char contendo 'C' ou 'D' informando o tipo do movimento
               (credito ou debito)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type context: dict
        :rtype: boolean

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not dados_condicao:
            return False
        conta_avista = self._parametros(cr, uid).financeiro.finpar_conta_tesouraria_avista_id
        if not self.pool.get('fiscal.parametro').search(
                cr, uid, [('company_id', '=', dados_nf.get('company_id'))], context=context):
            raise osv.except_osv(_('Atencao'),
                                 _('As informacoes dos parametros do fiscal devem ser '
                                   'cadastradas.'))
        forma_pgto = dict_forma_pgto.get(dados_condicao.get('forma_pagto_id'))
        if not forma_pgto.get('admfpg_cartao_pessoa_id') and not dados_nf.get('destinatario_id'):
            data_emissao = dados_nf.get('data_emissao')
            msg = 'Recebimento' if debito_credito == 'C' else 'Cancelamento'
            historico = '%s da nota fiscal %s-%s' % (msg, dados_nf.get('numero_nf'),
                                                       dados_nf.get('serie_nf') or 0)
            self.pool.get('financeiro.tesouraria.movimento').create(
                cr, uid, {'state': 'A', 'conta_id': conta_avista,
                          'historico': historico, 'debito_credito': debito_credito,
                          'data_movimento': data_emissao,
                          'ano_competencia': data_emissao[:4],
                          'valor': dados_condicao.get('valor'),
                          'mes_competencia': data_emissao[5:7],
                          'fintem_nf_id': dados_nf.get('id')}, context)

        return True

    def fisnfc_estorna_movimento_tesouraria_mod2D(self, cr, uid, dados_nf, context=None):
        """
        Este metodo serve para gerar o estorno de movimentacao das condicoes pagamento
        de notas fiscais com o modelo 2D onde as condicoes nao possuam cliente cadastrado, e na
        capa da nota tenha destinatario_id

        :param cr: a Vinha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param nf: o ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type context: dict
        :rtype: boolean

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        load = '_classic_write'
        dict_condicao = {}
        dict_forma_pgto = {}
        if dados_nf.get('nf_condicao_ids'):
            condicao_ids = dados_nf.get('nf_condicao_ids') or []
            forma_pgto_ids = []
            campos = ['condicao_forma_id', 'forma_pagto_id', 'fiscpg_chq_banco', 'nf_parcela_ids',
                      'fiscpg_chq_agencia', 'fiscpg_chq_numero', 'fiscpg_chq_conta',
                      'fiscpg_chq_emitente', 'valor']
            for i in self.pool.get('fiscal.condicao.pagamento').read(
                    cr, uid, condicao_ids, campos, context, load):
                dict_condicao[i.get('id')] = i
                forma_pgto_ids.append(i.get('forma_pagto_id'))
            campos = ['cartao_credito_debito', 'admfpg_tipo', 'name', 'admfpg_documento_tipo_id',
                      'admfpg_cartao_pessoa_id', 'admfpg_boleto', 'admfpg_local_cobranca_id',
                      'admfpg_natureza_id']
            map(lambda x: dict_forma_pgto.update({x.get('id'): x}),
                self.pool.get('adm.forma.pgto').read(cr, uid, forma_pgto_ids, campos, context))
        for condicao in dict_condicao.values():
            forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id'))
            if not dados_nf.get('destinatario_id') and not \
                    forma_pgto.get('admfpg_cartao_pessoa_id'):
                self.fisnfc_criar_movimento_financeiro_2D(
                    cr, uid, dados_nf, condicao, dict_forma_pgto, 'D', context)

        return True

    def get_observacao_titulo(self, cr, uid, parcela_id, context=None):
        """
        Retorna o numero da autorizacao do cartao na observacao do titulo.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param parcela_id: ID da parcela
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type parcela_id: long
        :type context: dict
        :return: o numero da autorizacao do cartao.
        :rtype: str
        """
        context = context and context.copy() or {}
        result = ''
        brw_fiscal_nf_parcela = self.pool.get('fiscal.nf.parcelas').browse(
            cr, uid, parcela_id, context)
        if not brw_fiscal_nf_parcela.nf_condicao_id:
            return result
        brw_forma_pagamento = brw_fiscal_nf_parcela.nf_condicao_id.forma_pagto_id
        if brw_forma_pagamento.cartao_credito_debito:
            result = _('No. da autorizacao do cartao: %s') % \
                     brw_fiscal_nf_parcela.nf_condicao_id.fiscpg_nro_autorizacao_cartao

        return result

    def calcula_prazo_frete(self, cr, uid, ids, prazo, prazo_tipo, valor, data_emissao,
                            context=None):
        """
        Calcula o o prazo do frete para nota fiscal de frete lancada na
        entrada de terceiros com documento de frete vinculado.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param prazo: prazo em dias para calcular o vencimento
        :param prazo_tipo: tipo de prazo para calculo ex: DD - Dias direto
        :param valor: valor total do frete
        :param data_emissao: data de emissao do frete
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type ids: list
        :type prazo: str
        :type prazo_tipo: str
        :type valor: float
        :type data_emissao: str
        :type context: dict
        :return: criacao ou alteracao da parcela do frete
        :rtype: list[list[int, int, dict]] -- Formato persistencia o2m

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        obj_adm_util = self.pool.get('adm.utils')
        brw_nf = self.browse(cr, uid, ids, context=context)
        condicao_pagamento_ids = []
        if prazo_tipo and prazo:
            prazo = prazo.replace(' ', '').replace(',', ' ').replace('/', ' ')
            prazo = prazo.split(' ')
            qt_parcelas = len(prazo) * 1.  # Sempre float!
            valor_parcela = round(valor / qt_parcelas, 2)
            count = 0
            data_emissao = obj_adm_util.admutl_datetime_to_timezone(
                cr, uid, parse(data_emissao))
            for p in prazo:
                count += 1
                try:
                    dt = datas.sigDatas(data_emissao[:10]).getCalculaPrazo(p, prazo_tipo)
                    dt_venc = dt[0:4] + '-' + dt[4:6] + '-' + dt[6:8]
                except TypeError:
                    raise osv.except_osv(
                        _('Atencao'),
                        _("Verifique se o prazo foi informado no formato correto. O prazo "
                          "em dias deve ser separados por ',' ou '/' ."))
                if count == len(prazo):
                    valor_parcela = valor - (valor_parcela * (qt_parcelas - 1))
                fisnfc_nf_terceiro_id = False
                if ids and isinstance(ids, list):
                    fisnfc_nf_terceiro_id = ids[0]
                vals = {'active': True, 'fisnfc_nf_terceiro_id': fisnfc_nf_terceiro_id,
                        'prazo': int(p), 'valor': valor_parcela, 'prazo_tipo': prazo_tipo,
                        'vencimento': dt_venc}
                condicao_pagamento_ids.append([0, False, vals])
                for dado_nf in brw_nf:
                    if dado_nf.fisnfc_parcelas_terceiros_ids:
                        for parcela in dado_nf.fisnfc_parcelas_terceiros_ids:
                            condicao_pagamento_ids.append([2, parcela.id])

        return condicao_pagamento_ids

    def onchange_prazo_tipo_frete(self, cr, uid, ids, prazo, prazo_tipo, valor, data_emissao,
                                  context=None):
        """
        Ao alterar o prazo_tipo do frete recalcula vencimento da parcela do frete

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param prazo: prazo em dias para calcular o vencimento
        :param prazo_tipo: tipo de prazo para calculo ex: DD - Dias direto
        :param valor: valor total do frete
        :param data_emissao: data de emissao do frete
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type ids: list
        :type prazo: str
        :type prazo_tipo: str
        :type valor: float
        :type data_emissao: str
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        return {'value': {
            'fisnfc_parcelas_terceiros_ids': self.calcula_prazo_frete(
                cr, uid, ids, prazo, prazo_tipo, valor, data_emissao)}}

    def onchange_prazo_frete(self, cr, uid, ids, prazo, prazo_tipo, valor, data_emissao,
                             context=None):
        """
        Ao alterar o prazo do frete recalcula vencimento da parcela do frete

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param prazo: prazo em dias para calcular o vencimento
        :param prazo_tipo: tipo de prazo para calculo ex: DD - Dias direto
        :param valor: valor total do frete
        :param data_emissao: data de emissao do frete
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type ids: list
        :type prazo: str
        :type prazo_tipo: str
        :type valor: float
        :type data_emissao: str
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        return {'value': {'fisnfc_parcelas_terceiros_ids': self.calcula_prazo_frete(
            cr, uid, ids, prazo, prazo_tipo, valor, data_emissao)}}

    def onchange_valor_frete(self, cr, uid, ids, vr_frete, fisnfc_nf_frete_ids, context=None):
        """
        Ao alterar o prazo do frete recalcula vencimento da parcela do frete

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param fisnfc_nf_frete_ids: lista contendo as notas fiscais de frete relacionadas a
                                    nf de terceiro
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type ids: list
        :type vr_frete: float
        :type fisnfc_nf_frete_ids: list -- (Valores no padrao o2m)
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if isinstance(fisnfc_nf_frete_ids, (list, tuple)):
            for frete in fisnfc_nf_frete_ids:
                nf_frete_id = frete[1]
                if frete[0] in (1, 4):
                    frete[0] = 1
                    frete[2] = {'vr_frete': vr_frete, 'vr_total_nf': vr_frete}
                    fields_frete = ['fisnfc_parcelas_terceiros_ids', 'vr_frete', 'prazo',
                                    'prazo_tipo', 'data_emissao']
                    dados_nf_frete = self.read(
                        cr, uid, [nf_frete_id], fields_frete, context, '_classic_write')
                    for dado in dados_nf_frete:
                        parcelas = self.calcula_prazo_frete(
                            cr, uid, [nf_frete_id], dado.get('prazo'), dado.get('prazo_tipo'),
                            vr_frete, dado.get('data_emissao'))
                        if parcelas:
                            frete[2].update({'fisnfc_parcelas_terceiros_ids': parcelas})
                elif frete[0] == 0:
                    frete[2]['vr_frete'] = vr_frete
                    frete[2]['vr_total_nf'] = vr_frete
                    if isinstance(frete[2], dict):
                        prazo = frete[2].get('prazo')
                        prazo_tipo = frete[2].get('prazo_tipo')
                        data_emissao = frete[2].get('data_emissao')
                        parcelas = self.calcula_prazo_frete(
                            cr, uid, [nf_frete_id], prazo, prazo_tipo, vr_frete, data_emissao,
                            context=context)
                        frete[2]['fisnfc_parcelas_terceiros_ids'] = parcelas

        return {'value': {'fisnfc_nf_frete_ids': fisnfc_nf_frete_ids}}

    def onchange_data_emissao_frete(self, cr, uid, ids, prazo, prazo_tipo, valor, data_emissao,
                                    context=None):
        """
        Ao alterar a data de emissao do frete recalcula vencimento da parcela do frete

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs
        :param prazo: prazo em dias para calcular o vencimento
        :param prazo_tipo: tipo de prazo para calculo ex: DD - Dias direto
        :param valor: valor total do frete
        :param data_emissao: data de emissao do frete
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type ids: list
        :type prazo: str
        :type prazo_tipo: str
        :type valor: float
        :type data_emissao: str
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        return {'value': {
            'fisnfc_parcelas_terceiros_ids': self.calcula_prazo_frete(
                cr, uid, ids, prazo, prazo_tipo, valor, data_emissao)}}

    def default_get(self, cr, uid, fields_list, context=None):
        """
        Ao alterar o prazo do frete recalcula vencimento da parcela do frete

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param fields_list: lista de campos para tratar o valor default
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type fields_list: list
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        res = super(fiscal_nf_capa, self).default_get(cr, uid, fields_list, context)

        # Necessario quando a nota fiscal estiver no contexto de manutencao.
        # Nesta situacao a nota deve ser criada automaticamente como faturada e nao deve fazer
        # nenhuma validacao.
        if context.get('nf_capa_manutencao'):
            res['state'] = '3'
            return res
        if all([context.get('nf_frete'), context.get('terceiro')]):
            res['vr_total_nf'] = context.get('valor_frete') or 0.0
            res['vr_frete'] = context.get('valor_frete') or 0.0

        return res

    def fisnfc_button_atualizar_peso(self, cr, uid, ids, context=None):
        """
        Botao responsavel por chamar o metodo que ira atualizar os pesos liquido e bruto de
        forma automatica
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da nota fiscal
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        nf = ids[0] if ids and isinstance(ids, list) else ids

        return self.fisnfc_atualiza_peso(cr, uid, nf, context=context)

    def fisnfc_atualiza_peso(self, cr, uid, nf, context=None):
        """
        Metodo responsavel por atualizar os valores do peso liquido e peso bruto
        de acordo com o metodo get_peso_nf_itens, este metodo e chamado apenas
        quando o o usuario clicar no botao Calcular peso, ou se o parametro
        fispar_calcula_peso_item_automatico do parametro do fiscal estiver marcado
        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param context: Dicionario de contextos
        :type cr: object
        :type uid: int
        :type nf: int | long
        :type context: dict
        :rtype: dict

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        pesos = self.get_peso_nf_itens(cr, uid, nf, context=context)
        vals = {'peso_liquido': pesos.get('peso_liquido') or 0.0,
                'peso_bruto': pesos.get('peso_bruto') or 0.0}

        return self.write(cr, uid, nf, vals, context=context)

    def _fisnfc_gerar_movimento_financeiro_avista(self, cr, user, dados_nf, fatura_id, dados_modelo,
                           dict_condicao, dict_forma_pgto, dict_parcelas, dict_guias, context=None):
        """
        Gera movimento financeiro para condicao "a vista".

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf_id: ID da nota fiscal
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf_id: long
        :type context: dict
        :return: verdadeiro se o movimento foi criado.
        :rtype: bool

        """
        context = context and context.copy() or {}
        local_cobranca_id = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'financeiro', [], context=context).get('local_cobranca_id')
        moeda_id = self.pool.get('adm.utils').admutl_get_parametros(
            cr, user, 'financeiro', [], context=context).get('moeda_id')
        pessoa_id = dados_nf.get('destinatario_id')
        context['ctx_fisnfc_titulo_create_sem_custificacao'] = True
        vencimento = time.strftime('%Y-%m-%d')
        nf_propria = dados_nf.get('nf_propria')
        if dados_modelo.get('fismnf_tipo_nfe') and nf_propria \
                and not context.get('fisnfc_importacao_nfce_do_supervisor'):
            # Atualiza as parcelas de nota fiscal "a vista"
            # Extrai a data do recebimento da NF-e na SEFAZ e atualiza o vencimento
            vencimento = context.get('nfe_xml_dhRecbto', '')[:10] or time.strftime('%Y-%m-%d')
            self.pool.get('fiscal.nf.parcelas').write(
                cr, user, map(lambda m: m.get('id'), dict_parcelas.values()),
                {'vencimento': vencimento}, context)
        elif nf_propria: # proria nao eletronica
            vencimento = dict_parcelas.values()
            # Como a condicao e ´a vista´ acesso a posicao 0 diretamente se existir vencimento
            vencimento = vencimento[0].get('vencimento') if isinstance(vencimento, list) else \
                                                                                      vencimento
        else:  # Nota de terceiros
            if dados_nf.get('data_emissao'):
                vencimento = dados_nf.get('data_emissao')[0:10]

            # Atualiza a data de vencimento para a mesma data da nota(a vista)
            self.pool.get('fiscal.nf.parcelas').write(
                cr, user, map(lambda m: m.get('id'), dict_parcelas.values()),
                {'vencimento': vencimento}, context)

        info_contribuinte = dados_nf.get('fisnfc_infcomp_contribuinte')
        tipo_nf = dados_nf.get('tipo')
        vals_titulo = {'company_id': dados_nf.get('company_id'), 'pessoa_id': pessoa_id,
                       'local_cobranca_id': local_cobranca_id, 'adm_moeda_id': moeda_id,
                       'tipo_titulo': 'P' if tipo_nf == 'E' else 'R', 'state': 'A',
                       'data_emissao': dados_nf.get('data_emissao'), 'tipo_lancamento': 'A',
                       'fatura_id': fatura_id, 'data_vencimento': vencimento,
                       'data_prorrogacao': vencimento, 'vendedor_id': dados_nf.get('vendedor_id'),
                       'vendedor_externo_id': dados_nf.get('vendedor_externo_id'),
                       'documento': str(dados_nf.get('numero_nf') or '')}
        vals_titulo = self.get_propagar_inf(
            cr, user, {'tipo': tipo_nf, 'fisnfc_infcomp_contribuinte': info_contribuinte},
            vals_titulo)
        chq_doc_ids = self.pool.get('documento.documento.tipo').search(
            cr, user, [('sigla', '=', 'CHQ')], context=context)

        # Gera um titulo para cada parcela
        for parcela in dict_parcelas.values():
            if nf_propria:
                vals_titulo['observacoes'] = self.get_observacao_titulo(
                    cr, user, parcela.get('id'), context)
            condicao = dict_condicao.get(parcela.get('nf_condicao_id')) or {}
            forma_pgto = dict_forma_pgto.get(condicao.get('forma_pagto_id')) or {}
            local_cobranca_id = forma_pgto.get('admfpg_local_cobranca_id') or  local_cobranca_id
            if forma_pgto and forma_pgto.get('admfpg_documento_tipo_id') in chq_doc_ids:
                vals_titulo.update({'fintit_chq_banco': condicao.get('fiscpg_chq_banco'),
                                    'fintit_chq_agencia': condicao.get('fiscpg_chq_agencia'),
                                    'fintit_chq_emitente': condicao.get('fiscpg_chq_emitente'),
                                    'fintit_chq_conta': condicao.get('fiscpg_chq_conta'),
                                    'documento': str(condicao.get('fiscpg_chq_numero') or '')})
                # Se cheque, nao pode baixar o titulo
                context['lancamento_cheque'] = True

            vals_titulo.update(
                {'doc_tipo_id': forma_pgto and forma_pgto.get('admfpg_documento_tipo_id') or
                                self.pool.get('documento.documento.tipo').search(
                                    cr, user, [('sigla', '=', 'NF')], context=context)[0],
                 'natureza_cobranca_id': forma_pgto and forma_pgto.get('admfpg_natureza_id'),
                 'vr_original': parcela.get('valor'), 'saldo': parcela.get('valor')})
            titulo_id = self.fisnfc_criar_titulo(cr, user, dados_nf.get('id'), vals_titulo, context)

            # Nao baixa titulo se for cheque ou cartao na nota de saida
            if not tipo_nf == 'S' or not context.get('lancamento_cheque'):
                if titulo_id:
                    self.fisnfc_baixar_titulo(
                        cr, user, titulo_id, forma_pgto.get('id'), local_cobranca_id,
                        nf_terceiro=not nf_propria, context=context)

        # GNRE
        if tipo_nf == 'E':  # Somente notas fiscais de entrada (Propria e terceiros)
            self._fisnfc_gerar_gnre(cr, user, dados_nf, moeda_id, local_cobranca_id, dict_guias,
                                    context)

        return True

    @log_debug
    def _fisnfc_gerar_gnre(self, cr, user, dados_nf, moeda, local, dict_guias, context=None):
        """
        Gera guia de recolhimento (GNRE).

        :param cr: o cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: ID da nota fiscal
        :param pessoa: o ID da pessoa (cliente/fornecedor)
        :param moeda: ID da moeda utlizada nas guias
        :param local: o ID do local de cobranca
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: long
        :type pessoa: long
        :type moeda: long
        :type local: long
        :type context: dict
        :return: verdadeiro se a guia foi criadas.
        :rtype: bool
        """
        context = context and context.copy() or {}
        if not local or not moeda or not dados_nf.get('destinatario_id') or not dict_guias:
            return False

        context['total_liquido'] = True  # Para compatibilidade. Remover se nao utilizar
        context['ctx_fisnfc_titulo_create_sem_custificacao'] = True
        for guia in dict_guias.values():
            if guia.get('fisgui_fornecedor_reembolsar'):
                vals_tit_guia = {
                    'company_id': dados_nf.get('company_id'),
                    'pessoa_id': dados_nf.get('destinatario_id'),
                    'local_cobranca_id': guia.get('local_cobranca_id'),
                    'adm_moeda_id': moeda,
                    'tipo_titulo': 'P' if dados_nf.get('tipo') == 'E' else 'R',
                    'state': 'A',
                    'data_emissao': dados_nf.get('data_emissao'),
                    'tipo_lancamento': 'A',
                    'data_vencimento': guia.get('fisgui_fornecedor_vencimento'),
                    'data_prorrogacao': guia.get('fisgui_fornecedor_vencimento'),
                    'vr_original': guia.get('valor'),
                    'saldo': guia.get('valor'),
                    'doc_tipo_id': self.pool.get('documento.documento.tipo').search(
                        cr, user, [('sigla', '=', 'GUI')], context=context)[0],
                    'documento': '%06d/%02d' % (dados_nf.get('numero_nf'), guia.get('id'))
                }
                if guia.get('agencia_id'):
                    vals_tit_guia['agencia_id'] = guia.get('agencia_id')
                titulo_id_guia = self.fisnfc_criar_titulo(cr, user, dados_nf.get('id'),
                                                          vals_tit_guia, context)
                self.pool.get('fiscal.guia').write(
                    cr, user, guia.get('id'), {'titulo_id': titulo_id_guia},
                    context)

                # Baixa dos titulos "a vista"
                self.fisnfc_baixar_titulo(
                    cr, user, titulo_id_guia, False, local, nf_terceiro=True, context=context)

        return True

    @log_debug
    def _faturar_pedidos_compra(self, cr, uid, nf_id, context=None):
        """
        Metodo responsavel por fazer a chamada do metodo que ira faturar os pedidos de compra
        que tiverem com as quantidades atendidas.

        :param cr: Cursor do openerp
        :param uid: Usuario do openerp
        :param nf_id: ID da nota fiscal
        :param context: Dicionario contendo valores contextuais
        :type cr: Cursor
        :type uid: int
        :type nf_id: int | long
        :type context: dict
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        fisnfc_pedido_ids = self.browse(cr, uid, nf_id, context=context).fisnfc_pedido_ids
        self.pool.get('estoquepedidos.pedido.capa').faturar_pedido_compra(
            cr, uid, nf_id, fisnfc_pedido_ids, context=context)

        return True

    @log_debug
    def fisnfc_faturar_varias_notas(self, cr, uid, ids, context=None):
        """
        Fatura varias notas de uma vez.  Somente validadas.
        """
        context = context and context.copy() or {}
        _erros = []
        for brw_nf in self.browse(cr, uid, ids, context=context):
            if brw_nf.state != '2':  # Considera apenas pedidos validados
                continue
            try:
                res = self.button_faturar(cr, uid, [brw_nf.id], context=context)
            except Exception as e:
                try:
                    if e.value:
                        _erros.append([brw_nf.name, e.value])
                finally:
                    res = False
            if res:
                cr.execute(''' COMMIT; ''')
            else:
                cr.execute(''' ROLLBACK; ''')
        if _erros:
            _mensagens = ''
            for _erro in _erros:
                _mensagens += '%s %s\n' % (_erro[0], _erro[1])
            raise osv.except_osv('Foram encontrados os seguintes erros:', _mensagens)

        return True

    def fisnfc_onchange_frete_terceiro(self, cr, uid, ids, tipo_nf, frete, context=None):
        """
        Atribui o valor zero frete se tipo de frete nao for por conta do emitente.

        :param cr: cursor no banco de dados.
        :param uid: id da tabela res_users, identificador do usuario.
        :param ids: lista de id da tabela desse objeto.
        :param frete: caracter que indica a responsabilidade do frete.
        :param context: dicionario com valores contextuais.
        :type cr: object
        :type uid: int
        :type ids: list
        :type frete: str
        :type context: dict
        :return: dicionario com a descricao do campo e seu respectivo valor.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if not frete or frete != '0':
            return {'value': {'vr_frete': 0}}

        return {}

    def fisnfc_onchange_frete(self, cr, user, ids, itens, frete, tipo_frete, valor, nat_id,
                              context=None):
        """
        Atualiza o valor rateado do frete e o valor base das tributacoes em funcao do tipo de frete.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID da nota fiscal
        :param itens: lista com os dados dos itens de movimento (formato one2many)
        :param frete: tipo de frete
        :param tipo_frete: tipo do frete: ´D´= "por dentro" ou ´F´= "por fora"
        :param valor: o valor do frete na capa da nota fiscal
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type user: int
        :type ids: list
        :type itens: list
        :type frete: str
        :type tipo_frete: str
        :type valor: float
        :type context: dict
        :return: itens da nota fiscal com os respectivos valores de frete ajustados.
        :rtype: dict
        """
        context = context and context.copy() or {}
        if frete and frete in '03':
            tipo_frete = 'D'
        else:
            valor = 0
            tipo_frete = 'F'

        return {'value': {'estoque_movimento_itens_ids': self.fisnfc_rateio_frete(
            cr, user, itens, valor, tipo_frete, nat_id, context), 'vr_frete': valor,
            'tipo_frete': tipo_frete}}

    def fisnfc_onchange_estado_nf_frete(self, cr, uid, ids, origem_uf_id, destino_uf_id,
                                        origem_cidade_id, destino_cidade_id, context=None):
        """
        O metodo remove o dado da cidade origem caso o estado origem (origem_uf_id) seja removido.
        Acontece o mesmo para o dado da cidade destino. Serve para manter o dominio de cidade
        pertencente a aquele estado.
        :param cr: cursor no banco de dados.
        :type cr: object
        :param uid: id da tabela res_users, identificador do usuario.
        :type uid: int
        :param ids: lista de registros da tabela desse objeto.
        :type ids: list
        :param origem_uf_id: id da tabela adm_uf, identificador do estado.
        :type origem_uf_id: int
        :param destino_uf_id: id da tabela adm_uf, identificador do estado.
        :type destino_uf_id: int
        :param origem_cidade_id: id da tabela adm_cidade, identificador da cidade.
        :type origem_cidade_id: int
        :param destino_cidade_id: id da tabela adm_cidade, identificador da cidade.
        :type destino_cidade_id: int
        :param context: dados contextuais.
        :type context: dict
        :return: dicionario com a descricao e valor de cada campo.
        :rtype: dict
        """
        context = context and context.copy() or {}
        obj_adm_cidade = self.pool.get('adm.cidade')
        vals = {}
        if not origem_uf_id:
            vals.update({'fisnfc_origem_cidade_id': False})
        if not destino_uf_id:
            vals.update({'fisnfc_destino_cidade_id': False})
        if origem_uf_id and origem_cidade_id:
            if origem_uf_id != obj_adm_cidade.browse(cr, uid, origem_cidade_id, context).uf_id.id:
                vals.update({'fisnfc_origem_cidade_id': False})
        if destino_uf_id and destino_cidade_id:
            if destino_uf_id != obj_adm_cidade.browse(cr, uid, destino_cidade_id, context).uf_id.id:
                vals.update({'fisnfc_destino_cidade_id': False})
        return {'value': vals}

    def fisnfc_cronjob_ajustar_protocolo_autorizacao_nfe(self, cr, user, context=None):
        context = context and context.copy() or {}
        cr.execute("""SELECT * 
                      FROM fiscal_parametro""")
        dados_par = cr.dictfetchall()
        for par in dados_par:
            # PEGANDO DIRETO DO BANCO PARA PEGAR NOTAS DE TODAS AS EMPRESA
            cr.execute("""SELECT id
                          FROM fiscal_nf_capa
                          WHERE data_emissao > current_date - 100
                            AND nfe_chave_acesso IS NOT NULL
                            AND state = '3'
                            AND nfe_xml_auto IS NULL
                            AND modelo_55 = True
                            AND nf_propria = True
                            AND company_id = {company_id}
                          LIMIT 10  --limite de 10 para o cron nao demorar muito nesta rotina, 
                                    --continua na proxima interacao  
                       """.format(company_id=par.get('company_id')))
            for nf_id in cr.dictfetchall():
                self.fisnfc_ajustar_protocolo_autorizacao_nfe(cr, user, nf_id.get('id'), par,
                                                              context)
        return True

    def fisnfc_ajustar_protocolo_autorizacao_nfe(self, cr, user, nf_id, parametro, context=None):
        context = context and context.copy() or {}
        brw_nf = self.browse(cr, user, nf_id, context)
        company_id = parametro.get('company_id')
        pessoa_id = self.pool.get('res.company').read(
            cr, user, company_id, ['rescon_pessoa_id'], context=context)['rescon_pessoa_id'][0]
        enderecos = self.pool.get('adm.pessoa').admpes_endereco_fiscal(cr, user, pessoa_id, context)
        endereco = enderecos.get(pessoa_id)
        try:
            nf = nfe_400
            nf = nf(certify=parametro.get('certificado_conteudo'),
                    key=parametro.get('chave_conteudo'),
                    environment=parametro.get('fispar_nfe_ambiente'),
                    state=endereco.get('uf_sigla'),
                    contingency=parametro.get('fispar_nfe_contingencia'))
            if brw_nf.nfe_lote:
                processo, __ = nf.consultar_recibo(brw_nf.nfe_lote)
                if processo.resposta.cStat.valor != u'104':  # Lote nao encontrado
                    processo, __ = nf.consultar_nota(brw_nf.nfe_chave_acesso)
                    if processo.resposta.cStat.valor == u'100':
                        self.write(
                            cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe.xml},
                            context)
                else:  # Lote processado
                    self.write(
                        cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe[0].xml},
                        context)
            elif brw_nf.nfe_chave_acesso:
                processo, __ = nf.consultar_nota(brw_nf.nfe_chave_acesso)
                if processo.resposta.cStat.valor == u'100':
                    self.write(
                        cr, user, nf_id, {'nfe_xml_auto': processo.resposta.protNFe.xml}, context)

        except Exception as err:
            self._logger.error('CRON - Ajusta XML %s >> %s' % (brw_nf.nfe_chave_acesso, err))

    def fisnfc_verifica_data_conciliacao_create(self, cr, uid, data_nf, context=None):
        """
        Metodo invocado na criacao de uma nota fiscal. Baseado na data de lancamento 
        (emissao/saida_entrada), e verificado se está dentro do prazo para conciliacao.
        """
        context = context and context.copy() or {}
        if not self.fisnfc_verifica_data_conciliacao(cr, uid, data_nf, context):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel lancar a NF nesta data. Este dia '
                                                 'ja esta conciliado.'))

    def fisnfc_verifica_data_conciliacao_write(self, cr, uid, data_nf, context=None):
        """
        Metodo invocado na atualizacao de uma nota fiscal. Baseado na data de lancamento 
        (emissao/saida_entrada), e verificado se está dentro do prazo para conciliacao.
        """
        context = context and context.copy() or {}
        if not self.fisnfc_verifica_data_conciliacao(cr, uid, data_nf, context):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel alterar a NF nesta data. Este dia '
                                                 'ja esta conciliado.'))

    def fisnfc_verifica_data_conciliacao_unlink(self, cr, uid, data_nf, context=None):
        """
        Metodo invocado na exclusao de uma nota fiscal. Baseado na data de lancamento
        (emissao/saida_entrada), e verificado se está dentro do prazo para conciliacao.
        """
        context = context and context.copy() or {}
        if not self.fisnfc_verifica_data_conciliacao(cr, uid, data_nf, context):
            raise osv.except_osv(_('Atencao'), _('Nao e possivel excluir a NF nesta data. Este dia '
                                                 'ja esta conciliado.'))

    def fisnfc_verifica_data_conciliacao(self, cr, uid, data, context=None):
        """
        O metodo basea-se na data de lancamento (emissao/saida_entrada) das notas fiscais de
        entrada e saida para verificar se está dentro do prazo para conciliacao.
        """
        context = context and context.copy() or {}
        data_conciliacao = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'fiscal', 'fispar_data_conciliacao', context=context).get(
            'fispar_data_conciliacao')
        if not data_conciliacao or not data:
            return True
        if data[:10] <= data_conciliacao:
            return False
        return True

    def fisnfc_importar_xml_proprio(self, cr, uid, xml, cancelada=False, local_id=None, context=None):
        """
        Metodo para importar xml de nfe/nfce propria
        xml Aquivo xml a ser importado
        cancelada, boolean indicando nota cancelada
        loca_id, local para lancamento da nf, se nao informado, usar o local padrao
        return {'status: True/False, 'mensagem':'', nf_id: id_da_nf_criada}
        """
        context = context and context.copy() or {}
        _spg = self.pool.get
        nfe = ProcNFe()
        nfe.xml = xml
        # dados do Emitente
        _cnpj_emitente = nfe.NFe.infNFe.emit.CNPJ.valor
        _pessoa_emitente_ids = _spg('adm.pessoa').search(cr, uid, [('cpf_cnpj', '=', _cnpj_emitente)], context=context)
        if not _pessoa_emitente_ids:
            return {'status': False, 'mensagem': 'Emitente nao cadastrado no sistema: %s' % _cnpj_emitente}
        company_ids = _spg('res.company').search(cr, uid, [('rescon_pessoa_id', '=', _pessoa_emitente_ids[0])],
                                                 context=context)
        if not company_ids:
            return {'status': False, 'mensagem': _('Empresa nao cadastrada para o Emitente: %s') % _cnpj_emitente}
        company_id = company_ids[0]
        context['sig_force_company'] = company_id

        # dados da nf
        _campos = ['local_venda_padrao_id', 'estpar_local_compra_padrao_id']
        _dados_parametros = _spg('adm.utils').admutl_get_parametros(cr, uid, 'estoque.parametro', _campos,
                                                                    company_id=company_id, context=context)

        _local_id = local_id
        if not _local_id:
            if nfe.NFe.infNFe.ide.tpNF.valor == 1:
                _local_id =_dados_parametros.get('local_venda_padrao_id')
            else:
                _local_id = _dados_parametros.get('estpar_local_compra_padrao_id')
        _modelo_ids = _spg('fiscal.modelo.nf').search(cr, uid, [('fismnf_codigo', '=', nfe.NFe.infNFe.ide.mod.valor)],
                                                      context=context)
        if not _modelo_ids:
            return {'status': False, 'mensagem': _('Modelo de nf nao cadastrado: %s') % nfe.NFe.infNFe.ide.mod.valor}
        _modelo_id = _modelo_ids[0]
        _campos = ['id', 'condicao_pgto_transferencia_id','company_id','fispar_natureza_uf_dentro_venda_id',
                   'fispar_natureza_uf_fora_venda_id', 'fispar_natureza_uf_dentro_devolucao_id',
                   'fispar_natureza_uf_fora_devolucao_id', 'fispar_natureza_transferencia_saida_id']
        _dados_parametros = _spg('adm.utils').admutl_get_parametros(cr, uid, 'fiscal.parametro', _campos,
                                                                    company_id=company_id, context=context)
        _param_id = _dados_parametros.get('id')
        args = [('modelo_nf_id', '=', _modelo_id),
                ('serie_nf', '=', nfe.NFe.infNFe.ide.serie.valor),
                ('parametro_id', '=', _param_id)]
        _modelo_nf_series_ids = _spg('fiscal.modelo.nf.series').search(cr, uid, args, context=context)
        if not _modelo_nf_series_ids:
            return {'status': False,
                    'mensagem': _('Serie %s / Modelo %s nao cadastrada para a empresa_id: %s') %
                                 (nfe.NFe.infNFe.ide.serie.valor, nfe.NFe.infNFe.ide.mod.valor, company_id)}
        _modelo_nf_series_id = _modelo_nf_series_ids[0]
        _natureza = nfe.NFe.infNFe.ide.natOp.valor
        _natureza_ids = _spg('fiscal.cfop.natureza').search(cr, uid, [('name', '=', _natureza)], context=context,
                                                            limit=1)
        if not _natureza_ids:
            return {'status': False, 'mensagem': _('Natureza nao cadastrada: %s') % _natureza}
        brw_natureza = _spg('fiscal.cfop.natureza').browse(cr, uid, _natureza_ids[0], context=context)
        _est_operacao_id = brw_natureza.fisnat_operacao_id.id

        args = [('numero_nf', '=', nfe.NFe.infNFe.ide.nNF.valor),
                ('serie_nf', '=', nfe.NFe.infNFe.ide.serie.valor),
                ('modelo_nf_series_id', '=', _modelo_nf_series_id),
                ('company_id', '=', company_id),
                ('data_emissao', '=', nfe.NFe.infNFe.ide.dhEmi.txt.replace('T', ' ')[:19])]
        nf_ids = _spg('fiscal.nf.capa').search(cr, uid, args, context=context)
        if nf_ids:
            return {'status': True, 'nf_id': nf_ids[0]}

        _user_dados = _spg('res.users').read(cr, uid, uid, ['pessoa_id'], context=context)
        if not _user_dados:
            return {'status': False, 'mensagem': _('O Usuario %s nao tem uma pessoa_id!') % uid}
        _user_pessoa_id = _user_dados.get('pessoa_id')[0]

        _args = [('company_id', '=', company_id), ('pessoa', '=', _user_pessoa_id)]
        _func_ids = _spg('rh.funcionario').search(cr, uid, _args, context=context)
        if not _func_ids:
            _func_id = _spg('rh.funcionario').create(cr, uid, {'company_id': company_id, 'pessoa': _user_pessoa_id},
                                                   context=context)
        else:
            _func_id = _func_ids[0]

        # dados do destinatario
        _cpf_cnpj = nfe.NFe.infNFe.dest.CNPJ.valor or nfe.NFe.infNFe.dest.CPF.valor
        _pessoa_id = False
        if _cpf_cnpj and nfe.NFe.infNFe.dest.xNome.valor:
            _pessoa_ids = _spg('adm.pessoa').search(cr, uid, [('cpf_cnpj', '=', _cpf_cnpj)], context=context)
            if _pessoa_ids:
                _pessoa_id = _pessoa_ids[0]
            else:
                dados = {
                    'nome': nfe.NFe.infNFe.dest.xNome.valor,
                    'cpf_cnpj': _cpf_cnpj,
                }
                if nfe.NFe.infNFe.dest.IE.valor:
                    dados['admpes_jur_ie'] = nfe.NFe.infNFe.dest.IE.valor
                res = self.pool.get('adm.pessoa').admpes_incluir_pessoa(cr, uid, context=context, **dados)
                if not res.get('status'):
                    return res
                _pessoa_id = res.get('pessoa_id')

            _cep_destinatario = nfe.NFe.infNFe.dest.enderDest.CEP.valor
            _cep_ids = _spg('adm.cep').search(cr, uid, [('cep', '=', _cep_destinatario)], context=context, limit=1)
            if _cep_ids:
                _cep_id = _cep_ids[0]
            else:
                _pais_ids = _spg('adm.pais').search(cr, uid, [('name', '=', nfe.NFe.infNFe.dest.enderDest.xPais.valor)],
                                                    context=context, limit=1)
                if _pais_ids:
                    _pais_id = _pais_ids[0]
                else:
                    _pais_id = _spg('adm.pais').create(cr, uid, {'name': nfe.NFe.infNFe.dest.enderDest.xPais.valor},
                                                    context=context)

                _uf_ids = _spg('adm.uf').search(cr, uid, [('sigla', '=', nfe.NFe.infNFe.dest.enderDest.UF.valor)],
                                                    context=context, limit=1)
                if _uf_ids:
                    _uf_id = _uf_ids[0]
                else:
                    _uf_id = _spg('adm.uf').create(cr, uid, {'name': nfe.NFe.infNFe.dest.enderDest.UF.valor,
                                                              'sigla': nfe.NFe.infNFe.dest.enderDest.UF.valor,
                                                              'pais_id': _pais_id
                                                              }, context=context)

                _cidade_ids = _spg('adm.cidade').search(cr, uid,
                                                        [('codigo_ibge', '=', nfe.NFe.infNFe.dest.enderDest.cMun.valor)],
                                                        context=context, limit=1)
                if _cidade_ids:
                    _cidade_id = _cidade_ids[0]
                else:
                    _cidade_id = _spg('adm.cidade').create(cr, uid, {
                        'name': nfe.NFe.infNFe.dest.enderDest.xMun.valor,
                        'codigo_ibge': nfe.NFe.infNFe.dest.enderDest.cMun.valor,
                        'uf_id': _uf_id,
                        'pais_id': _pais_id
                        }, context=context)

                _bairro_ids = _spg('adm.bairro').search(cr, uid,
                                                        [('name', '=', nfe.NFe.infNFe.dest.enderDest.xBairro.valor)],
                                                        context=context, limit=1)
                if _bairro_ids:
                    _bairro_id = _uf_ids[0]
                else:
                    _bairro_id = _spg('adm.bairro').create(cr, uid, {'name': nfe.NFe.infNFe.dest.enderDest.xBairro.valor},
                                                           context=context)


                _logradouro = nfe.NFe.infNFe.dest.enderDest.xLgr.valor
                _logr_space = _logradouro.partition(' ')
                _logr_ponto = _logradouro.partition('.')
                if len(_logr_space[0]) < len(_logr_ponto[0]):
                    _titulo = _logr_space[0]
                    _descr = _logr_space[2]
                else:
                    _titulo = _logr_ponto[0]
                    _descr = _logr_ponto[2]

                _logradouro_ids = _spg('adm.logradouro').search(cr, uid,
                                                        [('titulo', '=', _titulo), ('name', '=', _descr)],
                                                        context=context, limit=1)
                if _logradouro_ids:
                    _logradouro_id = _logradouro_ids[0]
                else:
                    _logradouro_id = _spg('adm.logradouro').create(cr, uid, {'titulo': _titulo, 'name': _descr},
                                                           context=context)
                vals_cep = {
                    'cep': nfe.NFe.infNFe.dest.enderDest.CEP.valor,
                    'pessoa_pais_id': _pais_id,
                    'pessoa_uf_id': _uf_id,
                    'pessoa_cidade_id': _cidade_id,
                    'pessoa_bairro_id': _bairro_id,
                    'pessoa_logradouro_id': _logradouro_id
                }
                _cep_id = _spg('adm.cep').create(cr, uid, vals_cep, context=context)
            args = [
                ('pessoa_id', '=', _pessoa_id),
                ('cep', '=', _cep_destinatario),
                ('logradouro_id', '=', _logradouro_id),
                ('numero', '=', nfe.NFe.infNFe.dest.enderDest.nro.valor)
            ]
            _endereco_ids = _spg('adm.pessoa.endereco').search(cr, uid, args, context=context)
            if _endereco_ids:
                _endereco_id = _endereco_ids[0]
            else:
                vals_end = {
                    'pessoa_id': _pessoa_id,
                    'cep': _cep_destinatario,
                    'pais_id': _pais_id,
                    'uf_id': _uf_id,
                    'cidade_id': _cidade_id,
                    'bairro_id': _bairro_id,
                    'logradouro_id': _logradouro_id,
                    'numero': nfe.NFe.infNFe.dest.enderDest.nro.valor,
                    'complemento': nfe.NFe.infNFe.dest.enderDest.xCpl.valor
                }
                _endereco_id = _spg('adm.pessoa.endereco').create(cr, uid, vals_end, context=context)

        context['fiscal'] = True
        context['proprio'] = True
        context['fisnfc_nao_faturar_sefaz'] = True
        context['fisnfc_importacao_nfce_do_supervisor'] = True
        if cancelada:
            _state = 'C'
        else:
            _state = '1'

        vals = {'tipo': brw_natureza.tipo,
                'fisnfc_tipo_servico': False,
                'company_id': company_id,
                'numero_nf': nfe.NFe.infNFe.ide.nNF.valor,
                'serie_nf': nfe.NFe.infNFe.ide.serie.valor,
                'nf_propria': True,
                'natureza_id': brw_natureza.id,
                'modelo_nf_series_id': _modelo_nf_series_id,
                'modelo_nf_id': _modelo_id,
                'data_emissao': nfe.NFe.infNFe.ide.dhEmi.valor,
                'hora_emissao': nfe.NFe.infNFe.ide.dhEmi.valor,
                'data_saida_entrada': nfe.NFe.infNFe.ide.dhSaiEnt.valor or nfe.NFe.infNFe.ide.dhEmi.valor,
                'modelo_55': True,
                'gerar_xml': True,
                'especie': 'NF',
                'nfe_forma_impressao_danfe': str(nfe.NFe.infNFe.ide.tpImp.valor),
                'nfe_processo_emissao': nfe.NFe.infNFe.ide.procEmi.valor,
                'fisnfc_nfe_indicador_presenca': nfe.NFe.infNFe.ide.indPres.valor,
                'nfe_versao_processo_emissao': nfe.NFe.infNFe.versao.valor,
                'nfe_forma_emissao': str(nfe.NFe.infNFe.ide.tpEmis.valor),
                'finalidade_emissao': str(nfe.NFe.infNFe.ide.finNFe.valor),
                'nfe_protocolo': nfe.protNFe.infProt.nProt.valor,
                'nfe_status': '%s - %s' % (nfe.protNFe.infProt.cStat.valor, nfe.protNFe.infProt.xMotivo.valor),
                'nfe_chave_acesso': nfe.protNFe.infProt.chNFe.valor,
                'vr_desconto': nfe.NFe.infNFe.total.ICMSTot.vDesc.valor,
                'vr_icms_substituicao': nfe.NFe.infNFe.total.ICMSTot.vST.valor,
                'vr_base_icms_substituicao': nfe.NFe.infNFe.total.ICMSTot.vBCST.valor,
                'vr_seguro': nfe.NFe.infNFe.total.ICMSTot.vSeg.valor,
                'vr_despesas_acessorias': nfe.NFe.infNFe.total.ICMSTot.vOutro.valor,
                'vr_frete': nfe.NFe.infNFe.total.ICMSTot.vFrete.valor,
                'vr_total_nf': nfe.NFe.infNFe.total.ICMSTot.vNF.valor,
                'frete': '0' if nfe.NFe.infNFe.total.ICMSTot.vFrete.valor else '9',
                'fisnfc_infcomp_contribuinte': nfe.NFe.infNFe.infAdic.infCpl.valor,
                'fisnfc_infcomp_fisco': nfe.NFe.infNFe.infAdic.infAdFisco.valor,
                'state': _state,
                }
        if _pessoa_id:
            vals.update({
                'destinatario_id': _pessoa_id,
                'destinatario_endereco_id': _endereco_id,
            })
        if nfe.NFe.infNFe.ide.mod.valor == 65:
            vals.update({'fisnfc_consumidor': True})
        else:
            vals.update({'fisnfc_consumidor': False})

        if _state != 'C':
            vals.update({
                'nfe_xml_gerado': nfe.NFe.xml,
                'nfe_xml_auto': nfe.protNFe.xml,
            })
        if nfe.NFe.infNFe.total.ICMSTot.vFrete.valor > 0:
            vals['tipo_frete'] = 'D'
        context['nf_capa_manutencao'] = True
        context['nota_fiscal_consumidor'] = True
        try:
            nf_id = _spg('fiscal.nf.capa').create(cr, uid, vals, context=context)
        except Exception as e:
            return {'status': False, 'mensagem': '%s. %s' % (e.name, e.value)}
        context.pop('nf_capa_manutencao')
        if not nf_id:
            return {'status': False, 'mensagem': _('Nao consegui criar a nota fiscal')}

            #### Criar capa do movimento do estoque
        if cancelada:
            return {'status': True, 'nf_id': nf_id}

        vals_mov_capa = {
            'company_id': company_id,
            'nf_capa_id': nf_id,
            'funcionario_id': _func_id,
            'estmvc_operacao_id': _est_operacao_id,
            'data': nfe.NFe.infNFe.ide.dhEmi.valor,
            'state': 'C' if _state == 'C' else 'E',
        }
        mov_capa_id = _spg('estoque.movimento.capa').create(cr, uid, vals_mov_capa, context=context)

        _troco = nfe.NFe.infNFe.pag.vTroco.valor
        i = 0
        for pag in nfe.NFe.infNFe.pag.detPag:
            if _troco:
                if pag.vPag.valor == _troco:
                    _troco = 0
                    continue
                elif pag.vPag.valor < _troco:
                    _troco = _troco - pag.vPag.valor
                    continue
                else:
                    pag.vPag.valor = pag.vPag.valor - _troco
                    _troco = 0

            i += 1
            if pag.indPag.valor == '0':
                _tipo = 'V'
            elif pag.indPag.valor == '1':
                _tipo = 'C'
            else:
                _tipo = 'S'
            forma_ids = _spg('adm.forma.pgto').search(cr, uid, [('admfpg_tipo', '=', pag.tPag.valor)], context=context)
            if not forma_ids:
                return {'status': False, 'mensagem': _('Forma de pagamento nao encontrada! %s') % pag.tPag.valor}
            adm_condicao_forma_ids = _spg('adm.condicao.forma').search(cr, uid,
                                                                       [('forma_pagamento_id', '=', forma_ids[0])],
                                                                       context=context)
            _forma_id = False
            for acf_brw in _spg('adm.condicao.forma').browse(cr, uid, adm_condicao_forma_ids, context=context):
                if acf_brw.adm_condicao_id.tipo == _tipo:
                    _forma_id = acf_brw.forma_pagamento_id.id
                    break
            if not _forma_id:
                _desc = pag.tPag.valor
                for x in MEIO_PAGAMENTO:
                    if x[0] == pag.tPag.valor:
                        _desc = x[1]
                        break
                return {'status': False,
                        'mensagem': _('Nenhuma condicao de pagamento cadastrado para a forma "%s" do tipo "%s"') %
                                    (_desc, _tipo)}
            vals_pag = {
                'capa_id': nf_id,
                'condicao_forma_id': acf_brw.id,
                'forma_pagto_id': _forma_id,
                'valor': pag.vPag.valor
            }
            fcp_id = _spg('fiscal.condicao.pagamento').create(cr, uid, vals_pag, context=context)
            vals_par = {
                'nf_condicao_id': fcp_id,
                'prazo': 0,
                'prazo_tipo': 'DD',
                'sequencia': i,
                'sequencia_vencimento': 1,
                'valor': pag.vPag.valor,
                'vencimento': nfe.NFe.infNFe.ide.dhEmi.txt[:10]
            }
            _spg('fiscal.nf.parcelas').create(cr, uid, vals_par, context=context)

        for item in nfe.NFe.infNFe.det:
            acb_ids = _spg('automacao.codigo.barra').search(cr, uid, [('codigo_barra', '=', item.prod.cEAN.valor)],
                                                            context=context, limit=1)
            _produto_id = _grade_id = False
            if acb_ids:
                acb_brw = _spg('automacao.codigo.barra').browse(cr, uid, acb_ids[0], context=context)
                _grade_id = acb_brw.grade_id.id
                _produto_id = acb_brw.produto_id.id
                _qtd_embalagem = acb_brw.produto_embalagem_id.embalagem_id.quantidade_na_embalagem
                _estoque_produto_embalagem_id = acb_brw.produto_embalagem_id.id
                _produto_browse = acb_brw.produto_id
            if not _produto_id:
                args = [('name', '=', item.prod.xProd.valor)]
                _prod_ids = _spg('estoque.produto').search(cr, uid, args, context=context, limit=1)
                if _prod_ids:
                    _produto_id = _prod_ids[0]
                    _produto_browse = _spg('estoque.produto').browse(cr, uid, _produto_id, context=context)
                    _grade_id = _produto_browse.produto_grade_ids[0].id
                    _qtd_embalagem = 0
                    for x in _produto_browse.produto_grade_ids[0].local_locacao_ids:
                        if x.local_id.id == _local_id:
                            _qtd_embalagem = x.estllc_produto_embalagem_id.embalagem_id.quantidade_na_embalagem
                            _estoque_produto_embalagem_id = x.estllc_produto_embalagem_id.id
                            break
                    if _qtd_embalagem == 0:
                        _uCom = item.prod.uCom.valor
                        _embalagem_ids = _spg('estoque.embalagem').search(cr, uid, [('sigla', '=', _uCom)],
                                                                          context=context)
                        # if not _embalagem_ids
                        vals = {
                            'grade_id': _grade_id,
                            'local_id': _local_id,
                            'produto_id': _produto_id,
                            # 'estllc_produto_embalagem_id':
                        }
                        return {'status': False,
                                'mensagem': _('Produto nao encontrado no local! Cod:%s Ean:%s Nome:%s Local:%s') % (
                                    item.prod.cProd.valor,
                                    item.prod.cEAN.valor,
                                    item.prod.xProd.valor,
                                    _local_id
                                )}
                else:
                    return {'status': False,
                            'mensagem': _('Produto nao encontrado! Cod:%s Ean:%s Nome:%s') % (
                                item.prod.cProd.valor,
                                item.prod.cEAN.valor,
                                item.prod.xProd.valor,
                            )}
            custos = _spg('estoque.preco.custo').get_custos_do_dia(cr, uid, _produto_id, company_id,
                                                                   nfe.NFe.infNFe.ide.dhEmi.txt, context)[_produto_id]
            preco_venda = self.pool.get('estoque.preco.venda').get_preco_venda(cr, uid, [_produto_id],
                                                                context=context).get(_produto_id).get('preco_venda')
            _qtd = float(item.prod.qCom.valor)
            _vr_total_produto = item.prod.vProd.valor - item.prod.vDesc.valor + item.prod.vOutro.valor \
                                + item.prod.vSeg.valor
            vals_itens = {
                'capa_id': mov_capa_id,
                'cfop_id': brw_natureza.cfop_id.id,
                'estmvi_saidas_custo_corrigido_total': (custos.get('estprc_custo_corrigido') or 0) \
                                                       * _qtd * _qtd_embalagem,
                'estmvi_saidas_custo_medio_contabil_total': (custos.get('estprc_custo_medio_contabil') or 0) \
                                                            * _qtd * _qtd_embalagem,
                'estmvi_saidas_custo_reposicao_total': (custos.get('estprc_custo_reposicao') or 0) \
                                                       * _qtd * _qtd_embalagem,
                'estmvi_saidas_ultimo_custo_total': (custos.get('estprc_ultimo_custo') or 0) * _qtd * _qtd_embalagem,
                'fornecedor_id': _pessoa_id,
                'grade_id': _grade_id,
                'local_id': _local_id,
                'natureza_id': brw_natureza.id,
                'nf_id': nf_id,
                'produto_embalagem_id': _estoque_produto_embalagem_id,
                'produto_id': _produto_id,
                'quantidade': _qtd,
                'sequencia': item.nItem.valor,
                'vr_desconto_rateado': item.prod.vDesc.valor,
                'vr_despesas_rateado': item.prod.vOutro.valor,
                'vr_frete_rateado': item.prod.vFrete.valor,
                'vr_seguro_rateado': item.prod.vSeg.valor,
                'vr_total_item_rateado': _vr_total_produto,
                'vr_preco_venda': preco_venda,
                'vr_total_item':  item.prod.vProd.valor - item.prod.vDesc.valor,
                'vr_unitario': float(item.prod.vUnCom.valor)
            }
            _item_id = _spg('estoque.movimento.itens').create(cr, uid, vals_itens, context=context)

            # Tratamento do ICMS
            _encargo_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'ICMS')], context=context)
            if not _encargo_ids:
                return {'status': False, 'mensagem': _('Encargo ICMS nao cadastrado!')}
            _cst = _csosn = False
            if item.imposto.ICMS.CSOSN.valor:
                csosn_ids = _spg('fiscal.csosn').search(cr, uid, [('codigo', '=', item.imposto.ICMS.CSOSN.valor)],
                                                        context=context, limit=1)
                if not csosn_ids:
                    return {'status': False, 'mensagem': _('CSOSN %s nao cadastrado!') % item.imposto.ICMS.CSOSN.valor}
                cst_ids = _spg('fiscal.cst').search(cr, uid,
                                                    ['|',
                                                     ('fiscst_csosn_contribuinte_id', '=', csosn_ids[0]),
                                                     ('fiscst_csosn_nao_contribuinte_id', '=', csosn_ids[0])
                                                     ],  context=context)
                if not cst_ids:
                    return {'status': False,
                            'mensagem': _('CSOSN %s nao relacionado a nenhum CST!') % item.imposto.ICMS.CSOSN.valor}
                _csosn = True
            else:
                cst_ids = _spg('fiscal.cst').search(cr, uid,
                                                    [('codigo', '=', item.imposto.ICMS.CST.valor),
                                                     ('encargo_id', '=', _encargo_ids[0])],
                                                    context=context)
                _cst = True
                if not cst_ids:
                    return {'status': False,
                            'mensagem': _('CST %s nao cadastrado para o ICMS!') % item.imposto.ICMS.CST.valor}

            args = [('encargo_id', '=', _encargo_ids[0]), ('cst_id', 'in', cst_ids)]
            tributacao_ids = _spg('fiscal.tributacao').search(cr, uid, args, context=context)
            if not tributacao_ids:
                return {'status': False, 'mensagem': _('Nao encontrei tributacoes para o encargo "ICMS" e cst/csosn %s')
                                         % (item.imposto.ICMS.CSOSN.valor if _csosn else item.imposto.ICMS.CST.valor)}
            _tributacao = False
            for tributacao_brw in _spg('fiscal.tributacao').browse(cr, uid, tributacao_ids, context=context):
                if tributacao_brw.fistri_cst_tipo == 'T':
                    if item.imposto.ICMS.pICMS.valor:
                        if tributacao_brw.aliquota == float(item.imposto.ICMS.pICMS.valor):
                            _tributacao = True
                            break
                    else:
                        if tributacao_brw.aliquota == 18:
                            _tributacao = True
                            break
                    continue
                else:
                    _tributacao = True
                    break
            if not _tributacao:
                return {'status': False,
                        'mensagem': _('Nao encontrei a tributacao! Encargo: ICMS, CST/CSOSN: %s Tipo: %s') %
                                    ((item.imposto.ICMS.CSOSN.valor if _csosn else item.imposto.ICMS.CST.valor),
                                     tributacao_brw.fistri_cst_tipo)}
            vals = {
                'movimento_itens_id': _item_id,
                'encargo_id': _encargo_ids[0],
                'fisnit_capa_id': nf_id,
                'aliquota': item.imposto.ICMS.pICMS.valor,
                'vr_encargo': item.imposto.ICMS.vICMS.valor,
                'vr_base_calculo': item.imposto.ICMS.vBC.valor or _vr_total_produto,
                'vr_reducao_base': 0.0,
                'fisnit_csosn_id': csosn_ids[0] if _csosn else False,
                'cst_id': tributacao_brw.cst_id.id,
                'tributacao_id': tributacao_brw.id,
                'fisnit_vr_difal_uf_origem': item.imposto.ICMSUFDest.vICMSUFRemet.valor or 0.00,
                'fisnit_vr_difal_uf_destino': item.imposto.ICMSUFDest.vICMSUFDest.valor or 0.00,
                'fisnit_vr_fcp': item.imposto.ICMSUFDest.pFCPUFDest.valor or 0.00,
            }
            _nf_item_tributacao_id = _spg('fiscal.nf.item.tributacao').create(cr, uid, vals, context=context)

            # Tratamento do PIS
            _encargo_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'PIS')], context=context)
            if not _encargo_ids:
                return {'status': False, 'mensagem': _('Encargo PIS nao cadastrado!')}
            cst_ids = _spg('fiscal.cst').search(cr, uid, [('codigo', '=', item.imposto.PIS.CST.valor),
                                                          ('encargo_id', '=', _encargo_ids[0])],
                                                context=context)
            if not cst_ids:
                return {'status': False,
                        'mensagem': _('CST %s nao cadastrado para o PIS!') % item.imposto.PIS.CST.valor}
            args = [('encargo_id', '=', _encargo_ids[0]), ('cst_id', 'in', cst_ids)]
            tributacao_ids = _spg('fiscal.tributacao').search(cr, uid, args, context=context)
            if not tributacao_ids:
                return {'status': False,
                        'mensagem': _('Nao encontrei tributacoes para o encargo "PIS" e cst %s')
                                    % item.imposto.PIS.CST.valor}
            _tributacao = False
            for tributacao_brw in _spg('fiscal.tributacao').browse(cr, uid, tributacao_ids, context=context):
                if tributacao_brw.fistri_cst_tipo == 'T':
                    if tributacao_brw.aliquota == float(item.imposto.PIS.pPIS.valor):
                        _tributacao = True
                        break
                    continue
                else:
                    _tributacao = True
                    break
            if not _tributacao:
                return {'status': False,
                        'mensagem': _('Nao encontrei a tributacao! Encargo: PIS, CST: %s Tipo: %s, Aliquota: %s') %
                                    (item.imposto.PIS.CST.valor, tributacao_brw.fistri_cst_tipo, item.imposto.PIS.pPIS.valor)}
            vals = {
                'movimento_itens_id': _item_id,
                'encargo_id': _encargo_ids[0],
                'fisnit_capa_id': nf_id,
                'aliquota': item.imposto.PIS.pPIS.valor,
                'vr_encargo': item.imposto.PIS.vPIS.valor,
                'vr_base_calculo': item.imposto.PIS.vBC.valor or _vr_total_produto,
                'vr_reducao_base': 0.0,
                'fisnit_nat_rec_pis_cofins': _produto_browse.estpro_pis_nat_rec,
                'cst_id': tributacao_brw.cst_id.id,
                'tributacao_id': tributacao_brw.id,
            }
            _nf_item_tributacao_id = _spg('fiscal.nf.item.tributacao').create(cr, uid, vals, context=context)


            # Tratamento do COFINS
            _encargo_ids = _spg('fiscal.encargo').search(cr, uid, [('sigla', '=', 'COFINS')], context=context)
            if not _encargo_ids:
                return {'status': False, 'mensagem': _('Encargo COFINS nao cadastrado!')}
            cst_ids = _spg('fiscal.cst').search(cr, uid, [('codigo', '=', item.imposto.COFINS.CST.valor),
                                                          ('encargo_id', '=', _encargo_ids[0])],
                                                context=context)
            if not cst_ids:
                return {'status': False,
                        'mensagem': _('CST %s nao cadastrado para o COFINS!') % item.imposto.PIS.CST.valor}
            args = [('encargo_id', '=', _encargo_ids[0]), ('cst_id', 'in', cst_ids)]
            tributacao_ids = _spg('fiscal.tributacao').search(cr, uid, args, context=context)
            if not tributacao_ids:
                return {'status': False, 'mensagem': _('Nao encontrei tributacoes para o encargo "COFINS" e cst %s')
                                                     % item.imposto.COFINS.CST.valor}
            _tributacao = False
            for tributacao_brw in _spg('fiscal.tributacao').browse(cr, uid, tributacao_ids, context=context):
                if tributacao_brw.fistri_cst_tipo == 'T':
                    if tributacao_brw.aliquota == float(item.imposto.COFINS.pCOFINS.valor):
                        _tributacao = True
                        break
                    continue
                else:
                    _tributacao = True
                    break
            if not _tributacao:
                return {'status': False,
                        'mensagem': _('Nao encontrei a tributacao! Encargo: COFINS, CST: %s, Tipo: %s, Aliquota: %s') %
                                    (item.imposto.COFINS.CST.valor, tributacao_brw.fistri_cst_tipo, item.imposto.COFINS.pCOFINS.valor)}
            vals = {
                'movimento_itens_id': _item_id,
                'encargo_id': _encargo_ids[0],
                'fisnit_capa_id': nf_id,
                'aliquota': item.imposto.COFINS.pCOFINS.valor,
                'vr_encargo': item.imposto.COFINS.vCOFINS.valor,
                'vr_base_calculo': item.imposto.COFINS.vBC.valor or _vr_total_produto,
                'vr_reducao_base': 0.0,
                'fisnit_nat_rec_pis_cofins': _produto_browse.estpro_cofins_nat_rec,
                'cst_id': tributacao_brw.cst_id.id,
                'tributacao_id': tributacao_brw.id,
            }
            _nf_item_tributacao_id = _spg('fiscal.nf.item.tributacao').create(cr, uid, vals, context=context)

        self.write(cr, uid, nf_id, {}, context=context)  # Regravar apra recaulcular metoros funcionais estoriados
        self.button_verificar_nf(cr, uid, nf_id, context=context)
        context['fisnfc_nao_faturar_sefaz'] = True
        result = self.fisnfc_funcao_button_faturar(cr, uid, [nf_id], context=context)

        return {'status': True, 'mensagem': ''}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
