#!/usr/bin/python
# -*- encoding: utf-8 -*-
from traceback import format_exc as trace
import thread
import time
import psycopg2
from openerp.tools.config import config


class SqlSpedContribuicoes(object):
    def __init__(self, cr, company_ids, data_inicio, data_fim, nf_nao_fiscais):
        """
        Este construtor incia threads de consultas que sao muito demoradas
        para que no momento em a class fiscal.sped.contribuicoes, necessitar destes registros
        eles ja estejam prontos.
        Ao iniciar uma thread lembrar de colocar um uma verificacao no metodo
        da class fiscal.sped que necessite destes dados observando se a thread do registro
        que ele necessita ainda esta sendo executada
        """
        self.cr_open = cr
        self.data_inicio = data_inicio
        self.data_fim = data_fim
        self.company_ids = str(company_ids).replace('[', '(').replace(']', ')')
        self.nf_nao_fiscais = nf_nao_fiscais
        self.C481 = {'executando': True, 'dados': [], 'erro': False}
        self.C485 = {'executando': True, 'dados': [], 'erro': False}
        self.M100 = {'executando': True, 'dados': [], 'erro': False}
        self.M105 = {'executando': True, 'dados': [], 'erro': False}
        self.M500 = {'executando': True, 'dados': [], 'erro': False}
        self.M505 = {'executando': True, 'dados': [], 'erro': False}
        self.erro = False
        try:
            thread.start_new_thread(self.sql_C481_C485, ('PIS', 'C481'))
            time.sleep(1)
            thread.start_new_thread(self.sql_C481_C485, ('COFINS', 'C485'))
            time.sleep(1)
            thread.start_new_thread(self.sql_M100, ())
            time.sleep(0.4)
            thread.start_new_thread(self.sql_M105, ())
            time.sleep(0.4)
            thread.start_new_thread(self.sql_M500, ())
            time.sleep(0.4)
            thread.start_new_thread(self.sql_M505, ())
        except:
            self.erro = trace()

    def new_cursor(self):
        password = config.options.get('db_password') or ''
        conn = psycopg2.connect(dsn=self.cr_open._cnx.dsn, password=password)
        return conn.cursor()

    def _convert_company_id(self, filtro):
        tipo_sped = filtro.get('tipo_sped')
        if tipo_sped == 'F':
            company_id = filtro.get('company_id')
            query = 'AND company_id = %s' % company_id
        else:
            company_id = tuple(filtro.get('company_id'))
            if company_id.__len__() == 1:
                company_id = str(company_id).replace(',', '')
            company_id = str(company_id)
            query = 'AND company_id IN %s' % company_id
        dados = {
            'company_id': company_id,
            'query': query
        }
        return dados

    def sql_consulta_movimento_notas(self, cr, filtros):
        dt_ini = filtros.get('sped_data_inicial')
        dt_fin = filtros.get('sped_data_final')
        convert_company_id = self._convert_company_id(filtros)
        query = convert_company_id.get('query')

        sql_terceiros = """
            UNION
            SELECT nf.id
            FROM   fiscal_nf_capa nf
                   LEFT JOIN fiscal_modelo_nf modelo_nf
                          ON modelo_nf.id = nf.modelo_nf_id
                   LEFT JOIN fiscal_cfop_natureza fcn
                          ON fcn.id = nf.natureza_id
                   LEFT JOIN fiscal_cfop fc
                          ON fcn.cfop_id = fc.id
            WHERE  nf.tipo = 'E'
                   AND nf.data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                   %s
                   AND nf.nf_propria = False
                   AND nf.state = '3'
                   AND fc.fiscfop_sped_contribuicoes IS TRUE
            GROUP  BY nf.id,
                      modelo_nf.fismnf_tipo_nfe
            HAVING nf.state != 'C'
                    OR ( nf.state = 'C'
                         AND ( modelo_nf.fismnf_tipo_nfe = false
                                OR ( modelo_nf.fismnf_tipo_nfe = true
                                     AND nf.nfe_chave_acesso != '' ) ) )
            """ % (dt_ini, dt_fin, query)
        sql_saida = """
            UNION
            SELECT nf.id
            FROM   fiscal_nf_capa nf
                   LEFT JOIN fiscal_modelo_nf_series modelo_nf_serie
                          ON modelo_nf_serie.id = nf.modelo_nf_series_id
                   LEFT JOIN fiscal_cfop_natureza fcn
                          ON fcn.id = nf.natureza_id
                   LEFT JOIN fiscal_cfop fc
                          ON fcn.cfop_id = fc.id
            WHERE  nf.tipo = 'S'
                   AND nf.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                   %s
                   AND nf.nf_propria = true
                   AND nf.state IN ( '3', 'C', 'I', 'D' )
                   AND fc.fiscfop_sped_contribuicoes IS TRUE
            GROUP  BY nf.id,
                      modelo_nf_serie.flag_modelo_nfe
            HAVING nf.state != 'C'
                    OR ( nf.state = 'C'
                         AND ( modelo_nf_serie.flag_modelo_nfe = false
                                OR ( modelo_nf_serie.flag_modelo_nfe = true
                                     AND nf.nfe_chave_acesso != '' ) ) )
            """ % (dt_ini, dt_fin, query)

        sql = """
                SELECT nf.id
                FROM   fiscal_nf_capa nf
                       LEFT JOIN fiscal_modelo_nf_series modelo_nf_serie
                              ON modelo_nf_serie.id = nf.modelo_nf_series_id
                       LEFT JOIN fiscal_cfop_natureza fcn
                              ON fcn.id = nf.natureza_id
                       LEFT JOIN fiscal_cfop fc
                              ON fcn.cfop_id = fc.id
                WHERE  nf.tipo = 'E'
                       AND nf.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                       %s
                       AND nf.nf_propria = true
                       AND nf.state IN ( '3', 'C', 'I', 'D' )
                       AND fc.fiscfop_sped_contribuicoes IS TRUE
                GROUP  BY nf.id,
                          modelo_nf_serie.flag_modelo_nfe
                HAVING nf.state != 'C'
                        OR ( nf.state = 'C'
                             AND ( modelo_nf_serie.flag_modelo_nfe = false
                                    OR ( modelo_nf_serie.flag_modelo_nfe = true
                                         AND nf.nfe_chave_acesso != '' ) ) )
                %s
                %s
            """ % (dt_ini, dt_fin, query, sql_saida, sql_terceiros)
        cr.execute(sql)
        movimento_notas = filter(None, map(lambda x: x[0], cr.fetchall()))
        return movimento_notas

    def sql_consulta_nf_energia(self, cr, filtro):
        data_ini = filtro.get('data_ini')
        data_fin = filtro.get('data_fin')
        sql = """
                SELECT nf_capa.id
                FROM   fiscal_nf_capa nf_capa
                       INNER JOIN fiscal_cfop cfop
                               ON cfop.id = (SELECT cfop_id
                                             FROM   fiscal_cfop_natureza
                                             WHERE  id = nf_capa.natureza_id)
                       INNER JOIN fiscal_modelo_nf modelo_nf
                               ON modelo_nf.id = nf_capa.modelo_nf_id
                WHERE  cfop.produto_servico_tipo = 'E'
                       AND nf_capa.data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                       AND nf_capa.state = '3'
                        OR modelo_nf.fismnf_codigo IN( '66', '28', '29' )
                           AND nf_capa.data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                           AND nf_capa.state = '3'
                   """ % (data_ini, data_fin, data_ini, data_fin)
        cr.execute(sql)
        dados = filter(None, map(lambda x: x[0], cr.fetchall()))
        return dados

    def sql_consulta_nf_servico(self, cr, filtro):
        mov_notas = str(filtro.get('movimento_notas'))
        sql = """
                SELECT id
                FROM   fiscal_nf_capa
                WHERE  fisnfc_tipo_servico = true
                       AND id IN %s
              """ % mov_notas.replace('[', '(').replace(']', ')')
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_consulta_mov_cartao(self, cr, dt_ini, dt_fin, ecf_instalado, company_id):
        sql_nf = """
                --NF--
                SELECT pessoa_id,
                       pessoa_tipo,
                       operadora,
                       cpf_cnpj AS cpf_cnpj,
                       ie
                FROM   (SELECT afp.admfpg_cartao_pessoa_id AS pessoa_id,
                               ap.pessoa_tipo              AS pessoa_tipo,
                               ap.nome                     AS operadora,
                               ap.cpf_cnpj                 AS cpf_cnpj,
                               ap.admpes_jur_ie            AS ie
                        FROM   fiscal_condicao_pagamento fcp
                               INNER JOIN fiscal_nf_capa fnc
                                       ON fnc.id = fcp.capa_id
                               INNER JOIN adm_forma_pgto afp
                                       ON afp.id = fcp.forma_pagto_id
                                          AND afp.cartao_credito_debito = true
                               INNER JOIN adm_pessoa ap
                                       ON ap.id = afp.admfpg_cartao_pessoa_id
                        WHERE  fnc.data_emissao AT TIME ZONE 'UTC' BETWEEN '{dt_ini}' AND '{dt_fin}'
                               AND fnc.state = '3'
                               AND afp.tipo_cartao = 'C'
                               AND fnc.company_id = {company_id}
                        GROUP  BY pessoa_id,
                                  operadora,
                                  ap.pessoa_tipo,
                                  ap.cpf_cnpj,
                                  ap.admpes_jur_ie
                        UNION ALL
                        SELECT afp.admfpg_cartao_pessoa_id AS pessoa_id,
                               ap.pessoa_tipo              AS pessoa_tipo,
                               ap.nome                     AS operadora,
                               ap.cpf_cnpj                 AS cpf_cnpj,
                               ap.admpes_jur_ie            AS ie
                        FROM   fiscal_condicao_pagamento fcp
                               INNER JOIN fiscal_nf_capa fnc
                                       ON fnc.id = fcp.capa_id
                               INNER JOIN adm_forma_pgto afp
                                       ON afp.id = fcp.forma_pagto_id
                                          AND afp.cartao_credito_debito = true
                               INNER JOIN adm_pessoa ap
                                       ON ap.id = afp.admfpg_cartao_pessoa_id
                        WHERE  fnc.data_emissao AT TIME ZONE 'UTC' BETWEEN '{dt_ini}' AND '{dt_fin}'
                               AND fnc.state = '3'
                               AND afp.tipo_cartao = 'D'
                               AND fnc.company_id = {company_id}
                        GROUP  BY pessoa_id,
                                  operadora,
                                  ap.pessoa_tipo,
                                  ap.cpf_cnpj,
                                  ap.admpes_jur_ie) AS TMP
                GROUP  BY pessoa_id,
                          operadora,
                          pessoa_tipo,
                          cpf_cnpj,
                          ie

                """.format(dt_ini=dt_ini, dt_fin=dt_fin, company_id=company_id)

        if not ecf_instalado:
            sql_cupom = ''
        else:
            sql_cupom = """
                        --CUPOM--
                        UNION ALL
                        SELECT pessoa_id,
                               pessoa_tipo,
                               operadora,
                               cpf_cnpj AS cpf_cnpj,
                               ie
                        FROM   (SELECT afp.admfpg_cartao_pessoa_id AS pessoa_id,
                                       ap.pessoa_tipo              AS pessoa_tipo,
                                       ap.nome                     AS operadora,
                                       ap.cpf_cnpj                 AS cpf_cnpj,
                                       ap.admpes_jur_ie            AS ie
                                FROM   ecf_cupons_finalizacao ecf
                                       INNER JOIN ecf_cupons_capa ecc
                                               ON ecc.id = ecf.cupom_id
                                       INNER JOIN adm_condicao_forma acf
                                               ON acf.id = ecf.finalizadora_id
                                       INNER JOIN adm_forma_pgto afp
                                               ON afp.id = acf.forma_pagamento_id
                                                  AND afp.cartao_credito_debito = true
                                       INNER JOIN adm_pessoa ap
                                               ON ap.id = afp.admfpg_cartao_pessoa_id
                                WHERE  ecc.date_cupom BETWEEN '{dt_ini}' AND '{dt_fin}'
                                       AND ecc.state = 'V'
                                       AND ecc.tipo_cupom = 'V'
                                       AND ecf.tipo_finalizadora = 'C'
                                       AND afp.tipo_cartao = 'C'
                                       AND ecc.company_id = {company_id}
                                GROUP  BY pessoa_id,
                                          operadora,
                                          ap.pessoa_tipo,
                                          ap.cpf_cnpj,
                                          ap.admpes_jur_ie
                                UNION ALL
                                SELECT afp.admfpg_cartao_pessoa_id AS pessoa_id,
                                       ap.pessoa_tipo              AS pessoa_tipo,
                                       ap.nome                     AS operadora,
                                       ap.cpf_cnpj                 AS cpf_cnpj,
                                       ap.admpes_jur_ie            AS ie
                                FROM   ecf_cupons_finalizacao ecf
                                       INNER JOIN ecf_cupons_capa ecc
                                               ON ecc.id = ecf.cupom_id
                                       INNER JOIN adm_condicao_forma acf
                                               ON acf.id = ecf.finalizadora_id
                                       INNER JOIN adm_forma_pgto afp
                                               ON afp.id = acf.forma_pagamento_id
                                                  AND afp.cartao_credito_debito = true
                                       INNER JOIN adm_pessoa ap
                                               ON ap.id = afp.admfpg_cartao_pessoa_id
                                WHERE  ecc.date_cupom BETWEEN '{dt_ini}' AND '{dt_fin}'
                                       AND ecc.state = 'V'
                                       AND ecc.tipo_cupom = 'V'
                                       AND ecf.tipo_finalizadora = 'C'
                                       AND afp.tipo_cartao = 'D'
                                       AND ecc.company_id = {company_id}
                                GROUP  BY pessoa_id,
                                          operadora,
                                          ap.pessoa_tipo,
                                          ap.cpf_cnpj,
                                          ap.admpes_jur_ie) AS TMP
                        GROUP  BY pessoa_id,
                                  operadora,
                                  pessoa_tipo,
                                  cpf_cnpj,
                                  ie
            """.format(dt_ini=dt_ini, dt_fin=dt_fin, company_id=company_id)

        sql = """
            SELECT MOV_CARTAO.pessoa_id,
                   pessoa_tipo,
                   operadora,
                   (SELECT codigo_bc
                    FROM   adm_pais
                    WHERE  id = ape.pais_id
                    LIMIT  1)                  AS cod_pais,
                   cpf_cnpj                    AS cpf_cnpj,
                   ie,
                   (SELECT codigo_ibge
                    FROM   adm_cidade
                    WHERE  id = ape.cidade_id
                    LIMIT  1)                  AS cod_mun,
                   (SELECT admpes_jur_suframa
                    FROM   adm_pessoa
                    WHERE  id = MOV_CARTAO.pessoa_id
                    LIMIT  1)                  AS suframa,
                   (SELECT descricao
                    FROM   adm_logradouro
                    WHERE  id = ape.logradouro_id
                    LIMIT  1)                  AS endr,
                   ape.numero                  AS num,
                   ape.complemento             AS compl,
                   (SELECT name
                    FROM   adm_bairro
                    WHERE  id = ape.bairro_id) AS bairro
            FROM   ({sql_nf}
                    {sql_cupom}) AS MOV_CARTAO
                   LEFT JOIN adm_pessoa_endereco ape
                          ON ape.pessoa_id = MOV_CARTAO.pessoa_id
            GROUP  BY MOV_CARTAO.pessoa_id,
                      operadora,
                      pessoa_tipo,
                      ape.pais_id,
                      cpf_cnpj,
                      ie,
                      cod_mun,
                      endr,
                      num,
                      compl,
                      bairro
        """.format(sql_nf=sql_nf, sql_cupom=sql_cupom)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados


    def sql_consulta_movimento_cupons(self, cr, filtros):
        dt_ini = filtros.get('sped_data_inicial')
        dt_fin = filtros.get('sped_data_final')
        tipo_sped = filtros.get('tipo_sped')
        if tipo_sped == 'F':
            company_id = filtros.get('company_id')
            query = 'AND company_id = %s' % company_id
        else:
            company_id = tuple(filtros.get('company_id'))
            if company_id.__len__() == 1:
                company_id = str(company_id).replace(',', '')
            company_id = str(company_id)
            query = 'AND company_id IN %s' % company_id
        sql = """
                SELECT id
                FROM   ecf_cupons_capa
                WHERE  date_cupom >= '%s'
                       AND date_cupom <= '%s'
                       %s
                       AND state = 'V'
              """ % (dt_ini, dt_fin, query)
        cr.execute(sql)
        movimento_cupons = filter(None, map(lambda x: x[0], cr.fetchall()))
        return movimento_cupons

    def sql_consulta_mov_modelo_2d(self, cr, filtro):
        data_ini = filtro.get('data_ini')
        data_fin = filtro.get('data_fin')
        sql = """
                SELECT nf_capa.id,
                       nf_capa.modelo_nf_id
                FROM   fiscal_nf_capa nf_capa
                       INNER JOIN fiscal_modelo_nf modelo_nf
                               ON modelo_nf.id = nf_capa.modelo_nf_id
                WHERE  modelo_nf.fismnf_codigo = '02'
                       AND nf_capa.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                       AND nf_capa.tipo = 'S'
                       AND nf_capa.state = '3'
        """ % (data_ini, data_fin)
        cr.execute(sql)
        dados = filter(None, map(lambda x: x[0], cr.fetchall()))
        return dados

        #Sql do bloco 0
    def sql_reg0150(self, cr, filtro, nf_nao_fiscais):
        nf_capa_ids = filter(lambda i: i if i not in nf_nao_fiscais else False,
                             filtro.get('movimento_notas'))
        if not nf_capa_ids:
            return []
        else:
            nf_capa_ids = str(nf_capa_ids)
        company_id = filtro.get('company_id')
        if not company_id:
            company_id = -1
        sql_0150 = """
                   SELECT DISTINCT Cast(pessoa.codigo AS TEXT)
                            || (SELECT Cast(id AS TEXT)
                                FROM   adm_pessoa_endereco
                                WHERE  id = nf_capa.destinatario_endereco_id)            AS
                            cod_part,
                            pessoa.nome                                                  AS
                            nome,
                            (SELECT codigo_bc
                             FROM   adm_pais
                             WHERE  id = (SELECT pais_id
                                          FROM   adm_pessoa_endereco
                                          WHERE  id = nf_capa.destinatario_endereco_id)) AS
                            cod_pais,
                            (SELECT cpf_cnpj
                             FROM   adm_pessoa
                             WHERE  id = nf_capa.destinatario_id
                                    AND pessoa_tipo = 'J')                               AS
                            CNPJ,
                            (SELECT cpf_cnpj
                             FROM   adm_pessoa
                             WHERE  id = nf_capa.destinatario_id
                                    AND pessoa_tipo = 'F')                               AS
                            CPF,
                            pessoa.admpes_jur_ie                                                AS
                            ie,
                            (SELECT codigo_ibge
                             FROM   adm_cidade
                             WHERE  id = (SELECT cidade_id
                                          FROM   adm_pessoa_endereco
                                          WHERE  id = nf_capa.destinatario_endereco_id)) AS
                            cod_mun,
                            pessoa.admpes_jur_suframa                                           AS
                            suframa,
                            (SELECT descricao
                             FROM   adm_logradouro
                             WHERE  id = (SELECT logradouro_id
                                          FROM   adm_pessoa_endereco
                                          WHERE  id = nf_capa.destinatario_endereco_id)) AS
                            logradouro,
                            (SELECT numero
                             FROM   adm_pessoa_endereco
                             WHERE  id = nf_capa.destinatario_endereco_id)               AS
                            num,
                            (SELECT name
                             FROM   adm_bairro
                             WHERE  id = (SELECT bairro_id
                                          FROM   adm_pessoa_endereco
                                          WHERE  id = nf_capa.destinatario_endereco_id)) AS
                            bairro
                   FROM     fiscal_nf_capa nf_capa
                            LEFT JOIN adm_pessoa pessoa
                                ON pessoa.id = nf_capa.destinatario_id
                            LEFT JOIN fiscal_modelo_nf modelo
                                ON modelo.id = nf_capa.modelo_nf_id
                   WHERE    
                        nf_capa.company_id = %s
                        AND nf_capa.id IN %s
                        AND nf_capa.state NOT IN ('I', 'C', 'D')
                        AND modelo.fismnf_codigo NOT IN ('02', '65')
                               """ % (company_id, nf_capa_ids.replace('[', '(').replace(']', ')'))
        cr.execute(sql_0150)
        dados = cr.dictfetchall()
        return dados

    def sql_reg0150_opera_cart(self, cr, dt_ini, dt_fin, opcao, company_id):
        sql_cartao = """
                INNER JOIN ecf_cupons_capa cupom
                    ON cupom.id = cond_pgto.cupom_id
                WHERE  cond_pgto.tipo_finalizadora = 'C'
                    AND data_cupom >= '%s'
                    AND data_cupom <= '%s'
                    AND cupom.company_id = %s
                ORDER BY cond_pgto.pessoa_id

                    """ % (dt_ini, dt_fin, company_id)
        sql_nf = """
                INNER JOIN fiscal_nf_capa nf_capa
                    ON nf_capa.id = cond_pgto.nf_id
                LEFT JOIN fiscal_modelo_nf modelo
                    ON modelo.id = nf_capa.modelo_nf_id
                WHERE  cond_pgto.tipo_finalizadora = 'C'
                        AND data_nf >= '%s'
                        AND data_nf <= '%s'
                        AND nf_capa.company_id = %s
                        AND modelo.fismnf_codigo NOT IN ('02', '65')
                ORDER BY cond_pgto.pessoa_id
                    """ % (dt_ini, dt_fin, company_id)

        if opcao == 'C':
            query = sql_cartao
        else:
            query = sql_nf
        sql = """
                SELECT DISTINCT cond_pgto.pessoa_id                             AS cod_part,
                                pessoa.pessoa_tipo,
                                pessoa.nome                                     AS nome,
                                (SELECT codigo_bc
                                 FROM   adm_pais
                                 WHERE  id = pessoa_endereco.pais_id)           AS cod_pais,
                                pessoa.cpf_cnpj                                 AS cnpj,
                                pessoa.cpf_cnpj                                 AS cpf,
                                pessoa.admpes_jur_ie                                   AS ie,
                                (SELECT codigo_ibge
                                 FROM   adm_cidade
                                 WHERE  id = pessoa_endereco.cidade_id)         AS cod_mun,
                                pessoa.admpes_jur_suframa                              AS suframa,
                                (SELECT descricao
                                 FROM   adm_logradouro
                                 WHERE  id IN( pessoa_endereco.logradouro_id )) AS endr,
                                pessoa_endereco.numero                          AS num,
                                pessoa_endereco.complemento                     AS compl,
                                (SELECT name
                                 FROM   adm_bairro
                                 WHERE  id = pessoa_endereco.bairro_id)         AS bairro
                FROM   fiscal_condicao_pagamento_pgview cond_pgto
                       INNER JOIN adm_pessoa pessoa
                               ON pessoa.id = cond_pgto.pessoa_id
                       LEFT JOIN adm_pessoa_endereco pessoa_endereco
                              ON pessoa_endereco.pessoa_id = cond_pgto.pessoa_id
                       %s
                """ % query
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    @staticmethod
    def sql_reg0190(cr, company_id, nf_ids=[], cupom_ids=[], nf_nao_fiscais=False,
                    ecf_instalado=False):
        if not nf_nao_fiscais:
            nf_nao_fiscais = ''
        else:
            nf_nao_fiscais = str(nf_nao_fiscais).replace('[', '(').replace(']', ')')
            nf_nao_fiscais = 'AND fnc.id NOT IN {nf_nao_fiscais}'.format(
                nf_nao_fiscais=nf_nao_fiscais)
        sql_nf = """
                 SELECT produto.estpro_unidade_inventario AS unid,
                        (
                            SELECT NAME
                            FROM   estoque_embalagem_unidade
                            WHERE  unidade = produto.estpro_unidade_inventario) AS descr
                 FROM            estoque_movimento_itens mov_itens
                 INNER JOIN      estoque_produto produto
                 ON              produto.id = mov_itens.produto_id
                 LEFT JOIN       fiscal_nf_capa fnc
                 ON              fnc.id = mov_itens.nf_id
                 WHERE
                 fnc.company_id = {company_id}
                 AND fnc.id IN ({nf_ids})
                 AND             produto_embalagem_id IS NOT NULL
                 {nf_nao_fiscais}
        """.format(company_id=company_id,
                   nf_ids=', '.join(map(lambda n: str(n), nf_ids)) if nf_ids else '-1',
                   nf_nao_fiscais=nf_nao_fiscais)
        sql_cupom = """
                     SELECT produto.estpro_unidade_inventario AS unid,
                            (
                                SELECT NAME
                                FROM   estoque_embalagem_unidade
                                WHERE  unidade = produto.estpro_unidade_inventario) AS descr
                     FROM            estoque_movimento_itens mov_itens
                     INNER JOIN      estoque_produto produto
                     ON              produto.id = mov_itens.produto_id
                     LEFT JOIN       ecf_cupons_capa ecc
                     ON              ecc.id = mov_itens.cupom_id
                     WHERE           ecc.company_id = {company_id}
                     AND             ecc.id IN ({cupom_ids})
                     AND             produto_embalagem_id IS NOT NULL
                    """.format(company_id=company_id,
                               cupom_ids=', '.join(map(lambda c: str(c),
                                                       cupom_ids)) if cupom_ids else '-1')
        if ecf_instalado:
            sql_0190 = """
                {sql_nf}
                UNION ALL
                {sql_cupom}
            """.format(sql_nf=sql_nf, sql_cupom=sql_cupom)
        else:
            sql_0190 = sql_nf
        cr.execute(sql_0190)
        dados_0190 = []
        chaves = []
        dados = cr.dictfetchall()
        for dado in dados:
            if not dado.get('unid') or not dado.get('descr'):
                continue
            chave = dado.get('unid') + dado.get('descr')
            if chave in chaves:
                continue
            chaves.append(chave)
            dados_0190.append(dado)
        return dados_0190

    @staticmethod
    def sql_reg0200(cr, company_id, nf_ids=[], cupom_ids=[], nf_nao_fiscais=False,
                    ecf_instalado=False):
        if not nf_nao_fiscais:
            nf_nao_fiscais = ''
        else:
            nf_nao_fiscais = str(nf_nao_fiscais).replace('[', '(').replace(']', ')')
            nf_nao_fiscais = 'AND fnc.id NOT IN {nf_nao_fiscais}'.format(
                nf_nao_fiscais=nf_nao_fiscais)
        sql_nf = """
                    SELECT DISTINCT CASE
                                      WHEN produto.grade_usar = true THEN produto.codigo
                                                                                      || ' '
                                                                                      || grade.grade
                                       ELSE produto.codigo
                                    END                                  AS cod_item,
                                    grade.name   AS descr_item,
                                    tipo_item    AS tipo_item,
                                    (SELECT min(id) 
                                     FROM automacao_codigo_barra 
                                     WHERE active = TRUE 
                                           AND produto_id = produto.id 
                                           AND grade_id = grade.id)      AS cod_barra_id,
                                    (
                                           SELECT codigo
                                           FROM   fiscal_ncm
                                           WHERE  id = ncm_id) AS cod_ncm,
                                    codigo_ex_ncm              AS ex_ipi,
                                    codigo_genero              AS cod_gen,
                                    estpro_unidade_inventario  AS unid_inv,
                                    (
                                           SELECT codigo
                                           FROM   fiscal_tabela_servico
                                           WHERE  id = codigo_tab_servicos_id) AS cod_lst
                    FROM            estoque_produto produto
                    INNER JOIN      estoque_movimento_itens itens
                    ON              itens.produto_id = produto.id
                    INNER JOIN      estoque_produto_grade grade
                    ON              grade.id = itens.grade_id
                    LEFT JOIN       fiscal_nf_capa fnc
                    ON		        fnc.id = itens.nf_id
                    WHERE
                    fnc.company_id = {company_id}
                    AND fnc.id IN ({nf_ids})
                    {nf_nao_fiscais}
                    GROUP BY        produto.id,
                                    grade.id,
                                    itens.produto_embalagem_id
        """.format(nf_ids=', '.join(map(lambda n: str(n), nf_ids)) if nf_ids else '-1',
                   company_id=company_id, nf_nao_fiscais=nf_nao_fiscais)
        sql_cupom = """
                    SELECT DISTINCT CASE
                                      WHEN produto.grade_usar = true THEN produto.codigo
                                                                                      || ' '
                                                                                      || grade.grade
                                       ELSE produto.codigo
                                    END                                  AS cod_item,
                                    grade.name   AS descr_item,
                                    tipo_item    AS tipo_item,
                                    (SELECT min(id) 
                                     FROM automacao_codigo_barra 
                                     WHERE active = TRUE 
                                           AND produto_id = produto.id 
                                           AND grade_id = grade.id)      AS cod_barra_id,
                                    (
                                           SELECT codigo
                                           FROM   fiscal_ncm
                                           WHERE  id = ncm_id) AS cod_ncm,
                                    codigo_ex_ncm              AS ex_ipi,
                                    codigo_genero              AS cod_gen,
                                    estpro_unidade_inventario  AS unid_inv,
                                    (
                                           SELECT codigo
                                           FROM   fiscal_tabela_servico
                                           WHERE  id = codigo_tab_servicos_id) AS cod_lst
                    FROM            estoque_produto produto
                    INNER JOIN      estoque_movimento_itens itens
                    ON              itens.produto_id = produto.id
                    INNER JOIN      estoque_produto_grade grade
                    ON              grade.id = itens.grade_id
                    LEFT JOIN	    ecf_cupons_capa coo
                    ON  		    coo.id = itens.cupom_id
                    WHERE           
                    coo.company_id = {company_id}
                    AND coo.id IN ({cupom_ids})
                    GROUP BY        produto.id,
                                    grade.id,
                                    itens.produto_embalagem_id
        """.format(cupom_ids=', '.join(map(lambda n: str(n), cupom_ids)) if cupom_ids else '-1',
                   company_id=company_id)
        if ecf_instalado:
            sql_0200 = """
                        SELECT DISTINCT *
                        FROM
                        ( {sql_nf}
                          UNION ALL
                          {sql_cupom}
                        ) AS TMP
            """.format(sql_nf=sql_nf, sql_cupom=sql_cupom)
        else:
            sql_0200 = sql_nf
        cr.execute(sql_0200)
        return cr.dictfetchall()

    def sql_reg0400(self, cr, filtro, nf_nao_fiscais):
        nf_capa_ids = filter(lambda i: i if i not in nf_nao_fiscais else False,
                             filtro.get('movimento_notas'))
        if not nf_capa_ids:
            return []
        else:
            nf_capa_ids = str(nf_capa_ids)
        tipo_sped = filtro.get('tipo_sped')
        company_id = filtro.get('company_id')
        if tipo_sped == 'P':
            nf_propria = ''
        else:
            nf_propria = 'AND nf_propria = false'
        sql_0400 = """
                    SELECT id AS cod_nat,
                           name AS descr_nat
                    FROM fiscal_cfop_natureza
                    WHERE id IN
                        (SELECT natureza_id
                         FROM estoque_movimento_itens
                         WHERE nf_id IN
                             (SELECT id
                              FROM fiscal_nf_capa
                              WHERE id IN %s
                              %s
                              AND company_id = %s))

                    """ % (str(nf_capa_ids).replace('[', '(').replace(']', ')'), nf_propria,
                           company_id)
        cr.execute(sql_0400)
        dados = cr.dictfetchall()
        return dados

    def sql_reg0450(self, cr, filtro, nf_nao_fiscais):
        nf_capa_ids = filter(lambda i: i if i not in nf_nao_fiscais else False,
                             filtro.get('movimento_notas'))
        if not nf_capa_ids:
            return []
        sql_0450 = """
                    SELECT right(id::varchar, 6)       AS cod_inf,
                           fisnfc_infcomp_contribuinte AS txt
                    FROM   fiscal_nf_capa
                    WHERE  fisnfc_infcomp_contribuinte IS NOT NULL
                           AND id IN %s
                           AND company_id = {company_id}
                    """ % (str(nf_capa_ids).replace('[', '(').replace(']', ')'))
        cr.execute(sql_0450.format(company_id=filtro.get('company_id')))
        dados = cr.dictfetchall()
        return dados

    def sql_regC100(self, cr, filtro):
        nf_capa_ids = ','.join(map(lambda x: "%s" % x, filtro.get('movimento_notas')))
        company_id = filtro.get('company_id')
        sql_encargos = """SELECT id,
                                 sigla
                          FROM fiscal_encargo
                          WHERE sigla IN ('PIS', 'COFINS', 'ICMS', 'IPI')"""
        cr.execute(sql_encargos)
        for i in cr.dictfetchall():
            if i.get('sigla') == 'PIS':
                pis_encargo_id = i.get('id')
            elif i.get('sigla') == 'COFINS':
                cofins_encargo_id = i.get('id')
            elif i.get('sigla') == 'ICMS':
                icms_encargo_id = i.get('id')
            elif i.get('sigla') == 'IPI':
                ipi_encargo_id = i.get('id')

        sql_modelo_nf = """SELECT id
                           FROM fiscal_modelo_nf
                           WHERE fismnf_codigo IN ('01', '1B', '04', '55', '65')"""
        cr.execute(sql_modelo_nf)
        modelo_nf_ids = ','.join(map(lambda x: '%s' % x.get('id'), cr.dictfetchall()))
        sql_C100 = """
                    SELECT DISTINCT nf_capa.id AS id ,
                                    (Select codigo from fiscal_cfop
                                      where id = (Select cfop_id from fiscal_cfop_natureza
                                      where id = nf_capa.natureza_id)) AS cfop,
                                    CASE
                                        WHEN nf_capa.tipo = 'S' THEN '1'
                                        WHEN nf_capa.tipo = 'E' THEN '0'
                                    END AS IND_OPER,
                                    CASE
                                        WHEN nf_capa.state = '3' 
                                            AND nf_capa.nfe_processo_emissao IN ('1', '2') 
                                                THEN '08'
                                        WHEN nf_capa.state = '3' THEN '00'
                                        WHEN nf_capa.state = 'C' THEN '02'
                                        WHEN nf_capa.state = 'I' THEN '05'
                                        WHEN nf_capa.state = 'D' THEN '04'
                                    END AS COD_SIT,
                                    CASE
                                        WHEN nf_capa.nf_propria = TRUE THEN '0'
                                        WHEN nf_capa.nf_propria = FALSE THEN '1'
                                    END AS IND_EMIT,
                                    serie_nf AS SER,
                                    numero_nf AS NUM_DOC,
                                    nfe_chave_acesso AS CHV_NFE,
                                    tipo,
                      CASE 
                          WHEN modelo.fismnf_codigo = '65' THEN ''
                          ELSE
                              (SELECT CAST (codigo AS text) ||
                                 (SELECT CAST(id AS text)
                                  FROM adm_pessoa_endereco
                                  WHERE id = nf_capa.destinatario_endereco_id)
                               FROM adm_pessoa
                               WHERE id = nf_capa.destinatario_id) 
                      END   AS COD_PART,

                      (SELECT cpf_cnpj
                       FROM adm_pessoa
                       WHERE id = nf_capa.destinatario_id) AS CNPJ_CPF_PART,

                      (SELECT fismnf_codigo
                       FROM fiscal_modelo_nf
                       WHERE id = nf_capa.modelo_nf_id) AS COD_MOD,
                                    data_emissao AT TIME ZONE 'UTC' AS DT_DOC,
                                    data_saida_entrada AT TIME ZONE 'UTC' AS DT_E_S,
                                    vr_total_nf AS VL_DOC,
                                    COALESCE(vr_total_produtos, 0.0) +
                                        COALESCE(vr_total_servicos, 0.0) +
                                        COALESCE(nf_capa.vr_icms_substituicao, 0.0) +
                                        COALESCE(nf_capa.vr_despesas_acessorias, 0.0) +
                                        COALESCE(nf_capa.vr_frete, 0.0) +
                                        COALESCE(sum(
                                              (SELECT fnit.vr_encargo
                                               FROM fiscal_nf_item_tributacao fnit
                                               LEFT JOIN fiscal_cst fc
                                                    ON fc.id = fnit.cst_id
                                               WHERE fnit.encargo_id = {ipi_id}
                                                 AND fnit.id = tributacao.id)
                                        ), 0.0) +
                                        COALESCE(nf_capa.fisnfc_valor_fcp_st, 0.0)
                                        AS VL_MERC,
                                    vr_desconto AS VL_DESC,
                                    fisnfc_valor_icms_desonerado AS VL_ABAT_NT,
                                    frete AS IND_FRT,
                                    vr_frete AS VL_FRT,
                                    vr_seguro AS VL_SEG,
                                    nf_propria ,
                                    vr_despesas_acessorias AS VL_OUT_DA,
                                    sum(
                                          (SELECT vr_base_calculo
                                           FROM fiscal_nf_item_tributacao trib
                                           INNER JOIN fiscal_cst cst ON tributacao.cst_id = cst.id
                                           WHERE trib.encargo_id = {icms_id}
                                             AND cst.codigo NOT in('10', '30', '40', '51', '60', 
                                                                   '70', '90')
                                             AND trib.id = tributacao.id)) AS VL_BC_ICMS,
                                    sum(
                                          (SELECT vr_encargo
                                           FROM fiscal_nf_item_tributacao trib
                                           INNER JOIN fiscal_cst cst ON tributacao.cst_id = cst.id
                                           WHERE trib.encargo_id = {icms_id}
                                             AND cst.codigo NOT in('10', '30', '40', '51', '60', 
                                                                   '70', '90')
                                             AND trib.id = tributacao.id)) AS VL_ICMS,
                                    sum(
                                          (SELECT vr_base_icms_substituicao
                                           FROM fiscal_nf_item_tributacao
                                           WHERE tipo_substituicao = TRUE
                                             AND encargo_id = {icms_id}
                                             AND id = tributacao.id)) AS VL_BC_ICMS_ST,
                                    sum(
                                          (SELECT vr_icms_substituicao
                                           FROM fiscal_nf_item_tributacao
                                           WHERE tipo_substituicao = TRUE
                                             AND encargo_id = {icms_id}
                                             AND id = tributacao.id)) AS VL_ICMS_ST,
                                    sum(
                                          (SELECT fnit.vr_encargo
                                           FROM fiscal_nf_item_tributacao fnit
                                           LEFT JOIN fiscal_cst fc
                                                ON fc.id = fnit.cst_id
                                           WHERE fnit.encargo_id = {ipi_id}
                                             AND fnit.id = tributacao.id
                                             AND fc.tipo = 'T')) AS VL_IPI ,
                                    sum(
                                          (SELECT fnit.vr_encargo
                                           FROM fiscal_nf_item_tributacao fnit
                                           INNER JOIN fiscal_cst cst ON cst.id = fnit.cst_id 
                                           WHERE cst.codigo != '05'
                                             AND fnit.encargo_id = {pis_id}
                                             AND fnit.id = tributacao.id)) AS VL_PIS ,
                                    sum(
                                          (SELECT fnit.vr_encargo
                                           FROM fiscal_nf_item_tributacao fnit
                                           INNER JOIN fiscal_cst cst ON cst.id = fnit.cst_id
                                           WHERE cst.codigo != '05'
                                             AND fnit.encargo_id = {cofins_id}
                                             AND fnit.id = tributacao.id)) AS VL_COFINS ,
                                    sum(
                                          (SELECT vr_encargo
                                           FROM fiscal_nf_item_tributacao
                                           WHERE tipo_substituicao = TRUE
                                             AND encargo_id = {pis_id}
                                             AND id = tributacao.id)) AS VL_PIS_ST,
                                    sum(
                                          (SELECT vr_encargo
                                           FROM fiscal_nf_item_tributacao
                                           WHERE tipo_substituicao = TRUE
                                             AND encargo_id = {cofins_id}
                                             AND id = tributacao.id)) AS VL_COFINS_ST,
                                    --Verificando condicoes de pagamentos

                                    CASE
                                        WHEN
                                               (SELECT tipo
                                                FROM adm_condicao
                                                WHERE id =
                                                    (SELECT adm_condicao_id
                                                     FROM adm_condicao_forma
                                                     WHERE id IN
                                                         (SELECT condicao_forma_id
                                                          FROM fiscal_condicao_pagamento
                                                          WHERE capa_id = nf_capa.id)LIMIT 1)) IN ('C',
                                                                                                   'P') THEN '1'
                                        WHEN
                                               (SELECT tipo
                                                FROM adm_condicao
                                                WHERE id =
                                                    (SELECT adm_condicao_id
                                                     FROM adm_condicao_forma
                                                     WHERE id IN
                                                         (SELECT condicao_forma_id
                                                          FROM fiscal_condicao_pagamento
                                                          WHERE capa_id = nf_capa.id)LIMIT 1)) IN ('V') THEN '0'
                                        WHEN nf_capa.nf_propria = FALSE
                                             AND nf_capa.prazo = '0' THEN '0'
                                        WHEN nf_capa.nf_propria = FALSE
                                             AND nf_capa.prazo <> '0' THEN '1'
                                        ELSE '2'
                                    END AS IND_PGTO,
                                    fisnfc_infcomp_contribuinte AS txt
                    FROM fiscal_nf_capa                  AS nf_capa
                    LEFT  JOIN fiscal_nf_item_tributacao AS tributacao ON tributacao.fisnit_capa_id = nf_capa.id
                    LEFT  JOIN fiscal_modelo_nf          AS modelo ON modelo.id = nf_capa.modelo_nf_id
                    WHERE nf_capa.id IN ({nf_id})
                      AND nf_capa.company_id = {company_id}
                      AND nf_capa.fisnfc_tipo_servico = FALSE
                      AND nf_capa.modelo_nf_id IN ({modelo_nf_ids})
                    GROUP BY nf_capa.id,
                             nf_capa.tipo,
                             nf_capa.state,
                             nf_capa.serie_nf,
                             nf_capa.numero_nf,
                             nf_capa.destinatario_id,
                             nf_capa.data_emissao,
                             nf_capa.data_saida_entrada,
                             nf_capa.vr_total_produtos,
                             nf_capa.vr_total_servicos,
                             nf_capa.vr_desconto,
                             modelo.fismnf_codigo
                     """.format(pis_id=pis_encargo_id,
                                cofins_id=cofins_encargo_id,
                                ipi_id=ipi_encargo_id,
                                icms_id=icms_encargo_id,
                                nf_id=nf_capa_ids,
                                company_id=company_id,
                                modelo_nf_ids=modelo_nf_ids)
        cr.execute(sql_C100)
        dados = cr.dictfetchall()
        return dados

    def sql_reg_C100_descontos(self, cr, filtro):
        nf_id = filtro
        sql_descontos_C100 = """
                                SELECT SUM(vr_desconto) + SUM(vr_desconto_rateado) AS desc_item
                                FROM estoque_movimento_itens
                                WHERE nf_id = %s
                             """ % nf_id
        cr.execute(sql_descontos_C100)
        dados = cr.dictfetchall()
        return dados

    def sql_regC170(self, cr, filtro):
        nf_ids = filtro
        sql = """
        SELECT DISTINCT 
            ROW_NUMBER() OVER (ORDER BY movimento_item.id)         AS NUM_ITEM,
            movimento_item.nf_id                                   AS nf_id,
            movimento_item.id                                      AS MOVIMENTO_ITEM_ID,
            CASE WHEN produto.grade_usar = TRUE THEN
                 produto.codigo || ' ' || grade.grade
            ELSE produto.codigo END                                AS COD_ITEM,
            (SELECT name
             FROM   estoque_produto_grade
             WHERE  id = movimento_item.grade_id)                  AS DESCR_COMPL,
            quantidade                                             AS QTD,
            movimento_item.sequencia                               AS SEQUENCIA,
            produto.estpro_unidade_inventario                      AS UNID,
            (SELECT codigo
             FROM   fiscal_cfop
             WHERE  id = cfop_id)                                  AS CFOP,
            natureza_id                                            AS COD_NAT,
            vr_total_item_rateado + COALESCE(
            (SELECT SUM(vr_encargo)
             FROM   fiscal_nf_item_tributacao                      AS fnit
             LEFT JOIN fiscal_cst                                  AS fc
                    ON fc.id = fnit.cst_id
             WHERE  fnit.encargo_id = (
                    SELECT id
                    FROM   fiscal_encargo
                    WHERE  sigla = 'IPI')
               AND  movimento_itens_id = movimento_item.id
            ), 0) + COALESCE(
            (SELECT SUM(vr_icms_substituicao)          
             FROM   fiscal_nf_item_tributacao                      AS fnit      
             LEFT JOIN fiscal_cst                                  AS fc                    
                    ON fc.id = fnit.cst_id                 
             WHERE  fnit.encargo_id = (
                    SELECT id        
                    FROM   fiscal_encargo
                    WHERE  sigla = 'ICMS')
               AND  fnit.movimento_itens_id = movimento_item.id
               AND  fc.tipo != 'T'), 0) +
            (SELECT sum(COALESCE(fisnit_vr_fcp_st, 0.0))
                    FROM   fiscal_nf_item_tributacao
                    WHERE  movimento_itens_id = movimento_item.id) AS VL_ITEM,
            (SELECT sum(COALESCE(fisnit_vr_icms_desonerado, 0.0))
                    FROM   fiscal_nf_item_tributacao
                    WHERE  movimento_itens_id = movimento_item.id) AS vl_icms_desonerado,
            vr_desconto + vr_desconto_rateado                      AS VL_DESC,
            (SELECT codigo
             FROM   fiscal_ncm
             WHERE  id = (
                    SELECT ncm_id
                    FROM   estoque_produto
                    WHERE  id = movimento_item.produto_id))        AS COD_NCM,
            (SELECT codigo_ex_ncm
             FROM   estoque_produto
             WHERE  id = movimento_item.produto_id)                AS EX_IPI,
            (SELECT codigo
             FROM   fiscal_cst
             WHERE  id = (
                    SELECT cst_id
                    FROM   fiscal_nf_item_tributacao
                    WHERE  encargo_id = (
                           SELECT id
                           FROM   fiscal_encargo
                           WHERE  sigla = 'ICMS')
               AND  movimento_itens_id = movimento_item.id))       AS CST_ICMS,
            SUM((SELECT vr_base_calculo
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'ICMS')
                   AND  movimento_itens_id = movimento_item.id))   AS VL_BC_ICMS,
            SUM((SELECT vr_encargo
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'ICMS')
                   AND  movimento_itens_id = movimento_item.id))   AS VL_ICMS,
            SUM((SELECT vr_base_icms_substituicao
                 FROM   fiscal_nf_item_tributacao
                 WHERE  tipo_substituicao = true
                   AND  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'ICMS')
                   AND  movimento_itens_id = movimento_item.id))   AS VL_BC_ICMS_ST,
            SUM((SELECT vr_icms_substituicao
                 FROM   fiscal_nf_item_tributacao
                 WHERE  tipo_substituicao = true
                   AND  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'ICMS')
                   AND  movimento_itens_id = movimento_item.id))   AS VL_ICMS_ST,
            (SELECT aliquota
             FROM   fiscal_nf_item_tributacao
             WHERE  encargo_id = (
                    SELECT id
                    FROM   fiscal_encargo
                    WHERE  sigla = 'ICMS')
               AND  movimento_itens_id = movimento_item.id)        AS ALIQ_ICMS,
            (SELECT aliquota
             FROM   fiscal_nf_item_tributacao
             WHERE  tipo_substituicao = TRUE
               AND  encargo_id = (
                    SELECT id
                    FROM   fiscal_encargo
                    WHERE  sigla = 'ICMS')
               AND  movimento_itens_id = movimento_item.id)        AS ALIQ_ST,
            (SELECT codigo
             FROM   fiscal_cst
             WHERE  id = (
                    SELECT cst_id
                    FROM   fiscal_nf_item_tributacao
                    WHERE  encargo_id = (
                           SELECT id
                           FROM   fiscal_encargo
                           WHERE  sigla = 'PIS')
               AND  movimento_itens_id = movimento_item.id))       AS CST_PIS,
            SUM((SELECT vr_base_calculo
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'PIS')
                   AND  movimento_itens_id = movimento_item.id
                   AND  cst_id IN (SELECT cst.id 
                                   FROM fiscal_cst cst 
                                   WHERE cst.tipo = 'T' 
                                         AND cst.encargo_id = encargo_id)
                   AND  aliquota > 0))   
                                                                   AS VL_BC_PIS,
            SUM((SELECT aliquota
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'PIS')
                   AND  movimento_itens_id = movimento_item.id))   AS ALIQ_PIS,
            SUM((SELECT vr_encargo
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'PIS')
                   AND  movimento_itens_id = movimento_item.id))   AS VL_PIS,
            (SELECT codigo
             FROM   fiscal_cst
             WHERE  id = (
                    SELECT cst_id
                    FROM   fiscal_nf_item_tributacao
                    WHERE  encargo_id = (
                           SELECT id
                           FROM   fiscal_encargo
                           WHERE  sigla = 'IPI')
               AND  movimento_itens_id = movimento_item.id))       AS CST_IPI,
            SUM((SELECT vr_base_calculo
                 FROM   fiscal_nf_item_tributacao                  AS fnit
                 LEFT JOIN fiscal_cst                              AS fc
                        ON fc.id = fnit.cst_id
                 WHERE  fnit.encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'IPI')
                   AND  movimento_itens_id = movimento_item.id
                   AND fc.tipo = 'T'))                             AS VL_BC_IPI,
            (SELECT aliquota
             FROM   fiscal_nf_item_tributacao                      AS fnit
             LEFT JOIN fiscal_cst                                  AS fc
                    ON fc.id = fnit.cst_id
             WHERE  fnit.encargo_id = (
                    SELECT id
                    FROM   fiscal_encargo
                    WHERE  sigla = 'IPI')
               AND  movimento_itens_id = movimento_item.id
               AND  fc.tipo = 'T')                                 AS ALIQ_IPI,
            SUM((SELECT vr_encargo
                 FROM   fiscal_nf_item_tributacao                  AS fnit
                 LEFT JOIN fiscal_cst                              AS fc
                        ON fc.id = fnit.cst_id
                 WHERE fnit.encargo_id = (
                       SELECT id
                       FROM   fiscal_encargo
                       WHERE  sigla = 'IPI')
                   AND movimento_itens_id = movimento_item.id
                   AND fc.tipo = 'T'))                             AS VL_IPI,
            (SELECT codigo
             FROM   fiscal_cst
             WHERE  id = (
                    SELECT cst_id
                    FROM   fiscal_nf_item_tributacao
                    WHERE  encargo_id = (
                           SELECT id
                           FROM   fiscal_encargo
                           WHERE  sigla = 'COFINS')
               AND  movimento_itens_id = movimento_item.id))       AS CST_COFINS,
            SUM((SELECT vr_base_calculo
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'COFINS')
                   AND  movimento_itens_id = movimento_item.id
                   AND  cst_id IN (SELECT cst.id 
                                   FROM fiscal_cst cst 
                                   WHERE cst.tipo = 'T' 
                                         AND cst.encargo_id = encargo_id)
                   AND  aliquota > 0))   
                                                                   AS VL_BC_COFINS,
            (SELECT aliquota
             FROM   fiscal_nf_item_tributacao
             WHERE  encargo_id = (
                    SELECT id
                    FROM   fiscal_encargo
                    WHERE  sigla = 'COFINS')
               AND  movimento_itens_id = movimento_item.id)        AS ALIQ_COFINS,
            SUM((SELECT vr_encargo
                 FROM   fiscal_nf_item_tributacao
                 WHERE  encargo_id = (
                        SELECT id
                        FROM   fiscal_encargo
                        WHERE  sigla = 'COFINS')
                   AND  movimento_itens_id = movimento_item.id))   AS VL_COFINS
        FROM estoque_movimento_itens                               AS movimento_item
        INNER JOIN estoque_produto                                 AS produto
                ON produto.id = movimento_item.produto_id
        INNER JOIN estoque_produto_grade                           AS grade
                ON grade.id = movimento_item.grade_id
        WHERE movimento_item.nf_id IN ({nf_ids})
        GROUP BY movimento_item.nf_id, movimento_item.id, movimento_item.produto_id, 
                 produto.grade_usar, produto.codigo, 
                 grade.grade, cod_item, produto.estpro_unidade_inventario
        """.format(nf_ids=', '.join(map(lambda n: str(n), nf_ids)))
        cr.execute(sql)
        dados = cr.dictfetchall()
        dados_c170 = {}
        for dado in dados:
            nf_id = dado.get('nf_id')
            if nf_id in dados_c170:
                dados_c170[nf_id].append(dado)
            else:
                dados_c170[nf_id] = [dado]
        return dados_c170

    def sql_regC175(self, cr, filtro):
        ctb_fiscal = filtro.get('contabil_fiscal')
        sql_C175 = """
            SELECT 
                nf_id, cst, cfop, aliq_pis AS aliq_pis, aliq_cofins AS aliq_cofins, 
                sum(vl_opr) AS vl_opr, sum(vl_bc) AS vl_bc,	
                sum(vl_pis) AS vl_pis, 
                sum(vl_cofins) AS vl_cofins, info_compl
                {col_cod_cta}
            FROM (
                SELECT 
                  nf_capa.id AS nf_id,
                  cst.codigo AS cst,
            
                  fc.codigo AS cfop ,
            
                  CASE 
                    WHEN nf_item_tributacao.encargo_id IN (
                        SELECT id FROM fiscal_encargo WHERE sigla = 'PIS')
                    THEN nf_item_tributacao.aliquota 
                    ELSE 0.0
                  END AS aliq_pis,
        
                  (SELECT aliquota 
                  FROM fiscal_nf_item_tributacao 
                  WHERE 
                      encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'COFINS') 
                      AND movimento_itens_id = nf_item_tributacao.movimento_itens_id) 
                  AS aliq_cofins,
            
                   SUM(COALESCE((
                       SELECT COALESCE(vr_icms_substituicao, 0.0) + 
                                        COALESCE(fisnit_vr_fcp_st, 0.0) - 
                                        COALESCE(fisnit_vr_icms_desonerado, 0.0)
                       FROM   fiscal_nf_item_tributacao
                       WHERE  
                           tipo_substituicao = true
                           AND encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                           AND movimento_itens_id = nf_item_tributacao.movimento_itens_id), 0.0)) + 
                       sum(mov_item.vr_total_item_rateado) AS vl_opr,
                   
                   sum(
                       CASE
                            WHEN nf_capa.nf_propria IS FALSE
                                AND destinatario.admpes_jur_regime_tributario = 'S'
                                AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                                AND cst.tipo = 'T'
                                AND nf_item_tributacao.vr_encargo > 0
                                AND nf_item_tributacao.encargo_id IN (
                                    SELECT min(id) FROM fiscal_encargo WHERE sigla = 'PIS')
                            THEN 0.00
                            WHEN nf_item_tributacao.encargo_id IN (
                                    SELECT min(id) FROM fiscal_encargo WHERE sigla = 'PIS')
                            THEN nf_item_tributacao.vr_base_calculo
                            ELSE
                                0.0
                       END) AS vl_bc,
                       
                   sum(
                       CASE
                            WHEN nf_capa.nf_propria IS FALSE
                                AND destinatario.admpes_jur_regime_tributario = 'S'
                                AND pessoa_empresa.admpes_jur_regime_tributario = 'R'
                                AND cst.tipo = 'T'
                                AND nf_item_tributacao.vr_encargo > 0
                                AND nf_item_tributacao.encargo_id IN (
                                    SELECT min(id) FROM fiscal_encargo WHERE sigla = 'PIS')
                            THEN 0.00
                            WHEN nf_item_tributacao.encargo_id IN (
                                    SELECT min(id) FROM fiscal_encargo WHERE sigla = 'PIS')
                            THEN nf_item_tributacao.vr_encargo
                            ELSE 0.0
                       END) AS vl_pis,
                       
                   sum(
                       (SELECT vr_encargo 
                       FROM fiscal_nf_item_tributacao 
		               WHERE 
		                   encargo_id IN (SELECT id FROM fiscal_encargo WHERE sigla = 'COFINS') 
		                   AND movimento_itens_id = nf_item_tributacao.movimento_itens_id)
		               ) AS vl_cofins, 
                       
                   {cod_cta}
                   
                   '' AS info_compl
                FROM fiscal_nf_capa nf_capa
                LEFT JOIN fiscal_nf_item_tributacao nf_item_tributacao 
                    ON nf_item_tributacao.fisnit_capa_id = nf_capa.id
                LEFT JOIN estoque_movimento_itens mov_item 
                    ON mov_item.id = nf_item_tributacao.movimento_itens_id
                LEFT JOIN fiscal_cst cst
                    ON cst.id = nf_item_tributacao.cst_id
                LEFT JOIN adm_pessoa destinatario
                    ON nf_capa.destinatario_id = destinatario.id
                LEFT JOIN res_company empresa
                    ON empresa.id = nf_capa.company_id
                LEFT JOIN adm_pessoa pessoa_empresa
                    ON empresa.rescon_pessoa_id = pessoa_empresa.id
                LEFT JOIN fiscal_cfop fc
                    ON fc.id = mov_item.cfop_id
                WHERE nf_capa.id IN ({nf_ids})
                  AND nf_item_tributacao.encargo_id IN
                    (SELECT id
                     FROM fiscal_encargo
                     WHERE sigla = 'PIS')
                GROUP BY nf_capa.id, cst, fc.id , aliq_pis, aliq_cofins {group_cta}
            ) AS tmp
            GROUP BY nf_id, cst, cfop, info_compl, aliq_pis, aliq_cofins {group_cta} 
        """.format(nf_ids=', '.join(map(lambda n: str(n), filtro.get('nf_ids'))),
                   cod_cta="(SELECT ctbnfi_conta_debito_id FROM contabil_natureza_fiscal "
                           "WHERE company_id = %s "
                           "      AND ctbnfi_natureza_id = nf_capa.natureza_id LIMIT 1)  "
                           "AS cod_cta," % filtro.get('natureza_fiscal_company_id')
                   if ctb_fiscal else '',
                   group_cta=', cod_cta' if ctb_fiscal else '',
                   col_cod_cta=', cod_cta' if ctb_fiscal else ''
                   )
        cr.execute(sql_C175)
        dados = cr.dictfetchall()
        dados_c175 = {}
        for dado in dados:
            nf_id = dado.get('nf_id')
            if nf_id in dados_c175:
                dados_c175[nf_id].append(dado)
            else:
                dados_c175[nf_id] = [dado]
        return dados_c175

    def sql_regC400(self, cr, filtro):
        company_id = filtro.get('company_id')
        dt_ini = filtro.get('dt_ini')
        dt_fim = filtro.get('dt_fim')
        sql_C400 = """
                    SELECT CAST('C400' AS text) AS reg,
                    CAST('2D' AS text) AS cod_mod,
                    modelo AS ecf_mod,
                    nro_serie AS ecf_fab,
                        caixa AS ecf_cx,
                    id
                    FROM ecf_ecf
                    WHERE id IN (SELECT ecf_id
                                    FROM ecf_cupons_capa
                                    WHERE date_cupom >= '%s'
                                    AND date_cupom <= '%s'
                                    AND company_id = %s
                                    GROUP BY ecf_id)
                   """ % (dt_ini, dt_fim, company_id)
        cr.execute(sql_C400)
        dados = cr.dictfetchall()
        return dados

    def sql_regC405(self, cr, filtro):

        dt_ini = filtro.get('dt_ini')
        dt_fin = filtro.get('dt_fin')
        ecf_id = filtro.get('ecf_id')

        sql_C405 = """
                    SELECT  id,
                            CAST('C405' AS text) AS reg,
                            data AS dt_doc,
                            cro,
                            numero_z AS crz,
                            coo AS num_coo_fin,
                            gt_final AS gt_fin,
                            gt_final - gt_inicial AS vl_brt,
                            acrescimos,
                            descontos,
                            cancelamentos
                    FROM ecf_mapa_resumo_capa
                    WHERE data >= '%s'
                    AND data <= '%s'
                    AND ecf = %s
                    """ % (dt_ini, dt_fin, ecf_id)
        cr.execute(sql_C405)
        dados = cr.dictfetchall()
        return dados

    def sql_D100(self, cr, nf_capa_ids, company_id):
        if company_id:
            query_company_id = 'AND nf_capa.company_id = %s' % company_id
        else:
            query_company_id = ''
        sql_D100 = """
                        SELECT nf_capa.id                            AS nf_id,
                               Cast('D100' AS TEXT)                  AS reg,
                               CASE
                                 WHEN nf_capa.tipo = 'E' THEN '0'
                                 WHEN nf_capa.tipo = 'S' THEN '1'
                               END                                   AS ind_oper,
                               CASE
                                 WHEN nf_capa.nf_propria = true THEN '0'
                                 ELSE '1'
                               END                                   AS ind_emit,
                               (SELECT Cast (codigo AS TEXT)
                                       || (SELECT Cast(id AS TEXT)
                                           FROM   adm_pessoa_endereco
                                           WHERE  id = nf_capa.destinatario_endereco_id)
                                FROM   adm_pessoa
                                WHERE  id = nf_capa.destinatario_id) AS COD_PART,
                               modelo_nf.fismnf_codigo               AS cod_mod,
                               CASE
                                 WHEN nf_capa.state = '3' THEN '00'
                                 WHEN nf_capa.state = 'C' THEN '02'
                                 WHEN nf_capa.state = 'I' THEN '05'
                               END                                   AS COD_SIT,
                               nf_capa.serie_nf                      AS serie,
                               nf_capa.subserie                      AS subserie,
                               nf_capa.numero_nf                     AS num_doc,
                               nf_capa.nfe_chave_cte                 AS chv_cte,
                               nf_capa.data_emissao AT TIME ZONE 'UTC' AS dt_doc,
                               nf_capa.data_saida_entrada AT TIME ZONE 'UTC' AS dt_a_p,
                               ''                                    chv_cte_ref,
                               nf_capa.vr_frete                      AS vl_doc,
                               nf_capa.vr_desconto                   AS vl_desc,
                               nf_capa.frete                         AS ind_frt,
                               nf_capa.vr_total_nf                   AS vl_serv,
                               nf_capa.vr_icms                       AS vl_icms,
                               (SELECT item_tributacao.vr_base_calculo
                                FROM   fiscal_nf_item_tributacao item_tributacao
                                WHERE  item_tributacao.encargo_id = (SELECT id
                                                                     FROM   fiscal_encargo
                                                                     WHERE  sigla = 'ICMS')
                                       AND nf_capa.id = item_tributacao.fisnit_capa_id
                                GROUP  BY item_tributacao.fisnit_capa_id,
                                          vr_base_calculo
                                LIMIT 1)                             AS vl_bc_icms,
                               (SELECT item_tributacao.vr_encargo
                                FROM   fiscal_nf_item_tributacao item_tributacao
                                WHERE  item_tributacao.encargo_id = (SELECT id
                                                                     FROM   fiscal_encargo
                                                                     WHERE  sigla = 'ICMS')
                                       AND nf_capa.id = item_tributacao.fisnit_capa_id
                                GROUP  BY item_tributacao.fisnit_capa_id,
                                          vr_encargo
                                LIMIT 1)                             AS vl_icms,
                               0.00                                  AS vl_nt,
                               nf_capa.fisnfc_infcomp_contribuinte   AS txt,
                               ''                                    AS cod_cta
                        FROM   fiscal_nf_capa nf_capa
                               INNER JOIN fiscal_modelo_nf modelo_nf
                                       ON modelo_nf.id = nf_capa.modelo_nf_id
                        WHERE  nf_capa.id IN %s
                               %s
                               AND modelo_nf.fismnf_codigo IN( '07', '08', '8B', '09',
                                                               '10', '11', '26', '27', '57' )
                               AND nf_capa.state IN( '3', 'C', 'I' )
                        ORDER  BY nf_capa.id ASC
                    """ % (str(nf_capa_ids).replace('[', '(').replace(']', ')'), query_company_id)
        cr.execute(sql_D100)
        dados = cr.dictfetchall()
        return dados

    @staticmethod
    def sql_D101(cr, nf_id):
        sql = """
                       SELECT
                          CASE
                          WHEN fisnfc.fisnfc_nf_frete_id IS NULL
                          THEN fisnfc.vr_total_produtos
                          ELSE
                            (SELECT vr_frete
                             FROM fiscal_nf_capa
                             WHERE id = fisnfc.fisnfc_nf_frete_id)
                          END                    AS vl_item,
                       fiscst.codigo                           AS cst_pis,
                       fiscfop.codigo                          AS cfop,
                       CASE 
                           WHEN fiscst.tipo = 'T' AND fisnit.aliquota > 0 
                               THEN fisnit.vr_base_calculo
                           ELSE 0 
                       END                                     AS vl_bc_pis,
                       fisnit.aliquota                         AS aliq_pis,
                       fisnit.vr_encargo                       AS vl_pis,
                       CASE
                         WHEN fiscfop.codigo IN (
                                        '1102', '1113', '1117', '1118',
                                        '1121', '1251', '1403', '2102',
                                        '2113', '2117', '2118', '2121',
                                        '2251', '2403', '3102', '3251' ) THEN '01' :: text
                         WHEN fiscfop.codigo IN (
                                        '1101', '1111', '1116', '1120',
                                        '1122', '1126', '1128', '1401',
                                        '1407', '1556', '1651', '1653',
                                        '2101', '2111', '2116', '2120',
                                        '2122', '2126', '2401', '2556', '2651',
                                        '2653', '3101', '3126', '3651', '3653' ) THEN '02' ::
                         text
                         WHEN fiscfop.codigo IN(
                                        '1124', '1125', '1933', '2124',
                                        '2125', '2933' ) THEN '03' :: text
                         WHEN fiscfop.codigo IN(
                                        '1252', '1253', '1254', '1255',
                                        '1256', '1257', '2252', '2253',
                                        '2254', '2255', '2256', '2257' ) THEN '04' :: text
                         WHEN fiscfop.codigo IN(
                                        '1352', '1353', '1354', '1355',
                                        '1356', '1932', '2352', '2353',
                                        '2354', '2355', '2356', '2932',
                                        '3352', '3353', '3356' ) THEN '07' :: text
                         WHEN fiscfop.codigo IN(
                                        '1201', '1202', '1410', '1411',
                                        '2201', '2202', '2410' ) THEN '12' :: text
                         WHEN fiscfop.codigo IN( '1351', '2351', '3351' ) THEN '14' :: text
                         ELSE ''
                       END AS codigo_natureza_credito
                FROM   fiscal_nf_capa fisnfc
                       INNER JOIN fiscal_nf_item_tributacao fisnit
                               ON fisnit.fisnit_capa_id = fisnfc.id
                       INNER JOIN fiscal_encargo fisenc
                               ON fisenc.id = fisnit.encargo_id
                       INNER JOIN fiscal_cst fiscst
                               ON fiscst.id = fisnit.cst_id
                       INNER JOIN fiscal_cfop_natureza fiscnat
                               ON fiscnat.id = fisnfc.natureza_id
                       INNER JOIN fiscal_cfop fiscfop
                               ON fiscfop.id = fiscnat.cfop_id
                WHERE  fisnfc.id = {nf_id}
                       AND fisenc.sigla = 'PIS'
          """.format(nf_id=nf_id)
        cr.execute(sql)
        return cr.dictfetchall()

    @staticmethod
    def sql_D105(cr, nf_id):
        sql = """
                       SELECT
                          CASE
                          WHEN fisnfc.fisnfc_nf_frete_id IS NULL
                          THEN fisnfc.vr_total_produtos
                          ELSE
                            (SELECT fisnfc.vr_frete
                             FROM fiscal_nf_capa
                             WHERE id = fisnfc.fisnfc_nf_frete_id)
                          END                    AS vl_item,
                       fiscst.codigo                           AS cst_cofins,
                       fiscfop.codigo                          AS cfop,
                       CASE 
                           WHEN fiscst.tipo = 'T' AND fisnit.aliquota > 0 
                               THEN fisnit.vr_base_calculo
                           ELSE 0
                       END                                     AS vl_bc_cofins,
                       fisnit.aliquota                         AS aliq_cofins,
                       fisnit.vr_encargo                       AS vl_cofins,
                       CASE
                         WHEN fiscfop.codigo IN (
                                        '1102', '1113', '1117', '1118',
                                        '1121', '1251', '1403', '2102',
                                        '2113', '2117', '2118', '2121',
                                        '2251', '2403', '3102', '3251' ) THEN '01' :: text
                         WHEN fiscfop.codigo IN (
                                        '1101', '1111', '1116', '1120',
                                        '1122', '1126', '1128', '1401',
                                        '1407', '1556', '1651', '1653',
                                        '2101', '2111', '2116', '2120',
                                        '2122', '2126', '2401', '2556', '2651',
                                        '2653', '3101', '3126', '3651', '3653' ) THEN '02' ::
                         text
                         WHEN fiscfop.codigo IN(
                                        '1124', '1125', '1933', '2124',
                                        '2125', '2933' ) THEN '03' :: text
                         WHEN fiscfop.codigo IN(
                                        '1252', '1253', '1254', '1255',
                                        '1256', '1257', '2252', '2253',
                                        '2254', '2255', '2256', '2257' ) THEN '04' :: text
                         WHEN fiscfop.codigo IN(
                                        '1352', '1353', '1354', '1355',
                                        '1356', '1932', '2352', '2353',
                                        '2354', '2355', '2356', '2932',
                                        '3352', '3353', '3356' ) THEN '07' :: text
                         WHEN fiscfop.codigo IN(
                                        '1201', '1202', '1410', '1411',
                                        '2201', '2202', '2410' ) THEN '12' :: text
                         WHEN fiscfop.codigo IN( '1351', '2351', '3351' ) THEN '14' :: text
                         ELSE ''
                       END AS codigo_natureza_credito
                FROM   fiscal_nf_capa fisnfc
                       INNER JOIN fiscal_nf_item_tributacao fisnit
                               ON fisnit.fisnit_capa_id = fisnfc.id
                       INNER JOIN fiscal_encargo fisenc
                               ON fisenc.id = fisnit.encargo_id
                       INNER JOIN fiscal_cst fiscst
                               ON fiscst.id = fisnit.cst_id
                       INNER JOIN fiscal_cfop_natureza fiscnat
                               ON fiscnat.id = fisnfc.natureza_id
                       INNER JOIN fiscal_cfop fiscfop
                               ON fiscfop.id = fiscnat.cfop_id
                WHERE  fisnfc.id = {nf_id}
                       AND fisenc.sigla = 'COFINS'
          """.format(nf_id=nf_id)
        cr.execute(sql)
        return cr.dictfetchall()

    def sql_D500(self, cr, dt_ini, dt_fim, company_id):
        sql = """
            SELECT    nf_capa.id          AS nf_id,
                      'D500'              AS reg,
                       nf_capa.tipo       AS IND_OPER,
                       CASE
                                  WHEN nf_capa.nf_propria IS true THEN '0'
                                  ELSE '1'
                       END                AS ind_emit,
                       Cast(pessoa.codigo AS TEXT)
                                  ||
                       (
                              SELECT Cast(id AS TEXT)
                              FROM   adm_pessoa_endereco
                              WHERE  id = nf_capa.destinatario_endereco_id) AS cod_part,
                       modelo.fismnf_codigo                                 AS cod_mod,
                       CASE
                           WHEN nf_capa.state = '3' THEN '00'
                           WHEN nf_capa.state = 'C' THEN '02'
                           WHEN nf_capa.state = 'I' THEN '05'
                           WHEN nf_capa.state = 'D' THEN '04'
                       END AS COD_SIT,
                       nf_capa.serie_nf                                     AS ser,
                       nf_capa.subserie                                     AS sub,
                       nf_capa.numero_nf                                    AS num_doc,
                       nf_capa.data_emissao AT TIME ZONE 'UTC'              AS dt_doc,
                       nf_capa.data_saida_entrada AT TIME ZONE 'UTC'        AS dt_a_p,
                       nf_capa.vr_total_nf                                  AS vl_doc,
                       nf_capa.vr_desconto                                  AS vl_desc,
                       nf_capa.vr_total_nf                                  AS vl_serv,
                       nf_capa.vr_despesas_acessorias                       AS vl_da,
                       nf_capa.vr_base_icms                                 AS vl_bc_icms,
                       nf_capa.vr_icms                                      AS vl_icms,
                       (
                              SELECT Sum(vr_encargo)
                              FROM   fiscal_nf_item_tributacao
                              WHERE  encargo_id =
                                     (
                                            SELECT id
                                            FROM   fiscal_encargo
                                            WHERE  sigla = 'PIS')
                              AND    fisnit_capa_id = nf_capa.id) AS vl_pis,
                       (
                              SELECT Sum(vr_encargo)
                              FROM   fiscal_nf_item_tributacao
                              WHERE  encargo_id =
                                     (
                                            SELECT id
                                            FROM   fiscal_encargo
                                            WHERE  sigla = 'COFINS')
                              AND    fisnit_capa_id = nf_capa.id) AS vl_cofins,
                              nf_capa.fisnfc_infcomp_contribuinte AS txt
            FROM       fiscal_nf_capa nf_capa
            INNER JOIN fiscal_modelo_nf modelo
            ON         modelo.id = nf_capa.modelo_nf_id
            INNER JOIN adm_pessoa pessoa
            ON         pessoa.id = nf_capa.destinatario_id
            LEFT JOIN  fiscal_cfop_natureza natureza
            ON         natureza.id = nf_capa.natureza_id
            LEFT JOIN  fiscal_cfop cfop
            ON         natureza.cfop_id = cfop.id
            WHERE      modelo.fismnf_codigo IN( '21',
                                               '22' )
            AND        nf_capa.company_id = {company_id}
            AND
                       CASE
                                  WHEN nf_capa.tipo = 'E'
                                  THEN nf_capa.data_saida_entrada AT TIME ZONE 'UTC'
                                      BETWEEN '%s' AND '%s'
                                  ELSE nf_capa.data_emissao = '%s' AND nf_capa.data_emissao = '%s'
                       END
            AND NOT (nf_capa.tipo = 'E'
                     AND nf_capa.nf_propria IS NOT TRUE
                     AND nf_capa.state = 'C')
            AND cfop.fiscfop_sped_contribuicoes IS TRUE
        """ % (dt_ini, dt_fim, dt_ini, dt_fim)
        cr.execute(sql.format(company_id=company_id))
        dados = cr.dictfetchall()
        return dados

    def sql_D501_D505(self, cr, nf_id, encargo):
        sql = """
                SELECT cst.codigo                         AS cst,
                       fisnfc.vr_total_produtos,
                       fisnit.vr_base_calculo,
                       fisnit.aliquota,
                       fisnit.vr_encargo,
                       CASE
                         WHEN fiscfop.codigo IN (
                                        '1102', '1113', '1117', '1118',
                                        '1121', '1251', '1403', '2102',
                                        '2113', '2117', '2118', '2121',
                                        '2251', '2403', '3102', '3251' ) THEN '01' :: text
                         WHEN fiscfop.codigo IN (
                                        '1101', '1111', '1116', '1120',
                                        '1122', '1126', '1128', '1401',
                                        '1407', '1556', '1651', '1653',
                                        '2101', '2111', '2116', '2120',
                                        '2122', '2126', '2401', '2556', '2651',
                                        '2653', '3101', '3126', '3651', '3653' ) THEN '02' ::
                         text
                         WHEN fiscfop.codigo IN(
                                        '1124', '1125', '1933', '2124',
                                        '2125', '2933' ) THEN '03' :: text
                         WHEN fiscfop.codigo IN(
                                        '1252', '1253', '1254', '1255',
                                        '1256', '1257', '2252', '2253',
                                        '2254', '2255', '2256', '2257' ) THEN '04' :: text
                         WHEN fiscfop.codigo IN(
                                        '1352', '1353', '1354', '1355',
                                        '1356', '1932', '2352', '2353',
                                        '2354', '2355', '2356', '2932',
                                        '3352', '3353', '3356' ) THEN '07' :: text
                         WHEN fiscfop.codigo IN(
                                        '1201', '1202', '1410', '1411',
                                        '2201', '2202', '2410' ) THEN '12' :: text
                         WHEN fiscfop.codigo IN( '1351', '2351', '3351' ) THEN '14' :: text
                         ELSE ''
                       END AS codigo_natureza_credito
                FROM   fiscal_nf_item_tributacao fisnit
                       INNER JOIN fiscal_nf_capa fisnfc
                               ON fisnfc.id = fisnit.fisnit_capa_id
                       INNER JOIN estoque_movimento_itens estmvi
                               ON estmvi.id = fisnit.movimento_itens_id
                       INNER JOIN fiscal_cfop fiscfop
                               ON fiscfop.id = estmvi.cfop_id
                       INNER JOIN fiscal_cst cst
                               ON cst.id = fisnit.cst_id
                       INNER JOIN fiscal_encargo fe
                               ON fe.id = fisnit.encargo_id
                WHERE  fisnfc.id = {nf_id}
                       AND fe.sigla = '{encargo}'
            """.format(nf_id=nf_id, encargo=encargo)
        cr.execute(sql)
        return cr.dictfetchall()

    def gera_mensagem_M100_M500(self, cr, filtro):
        data_ini = filtro.get('data_ini')
        data_fim = filtro.get('data_fim')
        encargo = filtro.get('encargo')
        sql = """
                SELECT
                       cfop.codigo                              AS cfop,
                       natureza.id                              AS natureza,
                       item_trib.fisnit_nat_rec_pis_cofins      AS cod_rec
                FROM   estoque_movimento_itens mv_itens
                       LEFT JOIN fiscal_cfop cfop
                              ON cfop.id = mv_itens.cfop_id
                       LEFT JOIN fiscal_cfop_natureza natureza
                              ON natureza.id = mv_itens.natureza_id
                       LEFT JOIN fiscal_nf_capa nf_capa
                              ON nf_capa.id = capa_id
                       INNER JOIN fiscal_nf_item_tributacao on item_trib
                              ON item_trib.movimento_itens_id = mv_itens.id
                       INNER JOIN fiscal_encargo encargo
                              ON encargo.id = item_trib.encargo_id
                WHERE  data_saida_entrada AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                       AND nf_capa.state = '3'
                       AND nf_capa.tipo = 'E'
                       AND encargo.sigla = '%s'
                ORDER  BY nf_capa.numero_nf ASC
              """ % (data_ini, data_fim, encargo)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_0140(self, cr, empresa_ids):
        sql = """
                SELECT DISTINCT company.id                         AS cod_est,
                                company.name                       AS nome,
                                pessoa.cpf_cnpj                    AS cnpj,
                                uf.sigla                           AS uf,
                                pessoa.admpes_jur_ie                      AS ie,
                                cidade.codigo_ibge                 AS cod_mun,
                                (SELECT numero
                                 FROM   adm_pessoa_documento_dados
                                 WHERE  tipo_id = (SELECT id
                                                   FROM   adm_pessoa_documento_tipo
                                                   WHERE  sigla = 'IM'
                                                   LIMIT  1)
                                        AND pessoa_id = pessoa.id) AS im,
                                pessoa.admpes_jur_suframa                 suframa
                FROM   res_company company
                       INNER JOIN adm_pessoa pessoa
                               ON pessoa.id = company.rescon_pessoa_id
                       INNER JOIN adm_pessoa_endereco endereco
                               ON endereco.pessoa_id = pessoa.id
                       INNER JOIN adm_uf uf
                               ON uf.id = endereco.uf_id
                       INNER JOIN adm_cidade cidade
                               ON cidade.id = endereco.cidade_id
                       LEFT JOIN adm_pessoa_documento_dados doc_dados
                               ON doc_dados.pessoa_id = pessoa.id
                       LEFT JOIN adm_pessoa_documento_tipo doc_tipo
                               ON doc_tipo.id = doc_dados.tipo_id
                WHERE 
                       company.id IN ({empresa_ids})
                       AND endereco.active IS TRUE
        """.format(empresa_ids=', '.join(map(lambda e: str(e), empresa_ids)))
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_A100(self, cr, filtro):
        mov_notas = str(filtro.get('movimento_notas'))
        company_id = filtro.get('company_id')
        sql = """
            SELECT nf_capa.id AS nf_id,
                   CASE
                       WHEN nf_capa.tipo = 'S' THEN '1'
                       WHEN nf_capa.tipo = 'E' THEN '0'
                   END AS IND_OPER,
                   CASE
                       WHEN nf_capa.state = '3' THEN '00'
                       WHEN nf_capa.state = 'C' THEN '02'
                       WHEN nf_capa.state = 'I' THEN '05'
                   END AS COD_SIT,
                   serie_nf AS SER,
                   numero_nf AS NUM_DOC,
              (SELECT CAST (codigo AS text) ||
                  (SELECT CAST(id AS text)
                         FROM adm_pessoa_endereco
                         WHERE id = nf_capa.destinatario_endereco_id)
                  FROM adm_pessoa
                  WHERE id = nf_capa.destinatario_id) AS COD_PART,
                   data_emissao AT TIME ZONE 'UTC' AS DT_DOC,
                   data_saida_entrada AT TIME ZONE 'UTC' AS DT_E_S,
                   vr_total_produtos + vr_total_servicos AS VL_DOC,
                   vr_desconto AS VL_DESC,
                   CASE
                       WHEN nf_capa.nf_propria = TRUE THEN '0'
                       WHEN nf_capa.nf_propria = FALSE THEN '1'
                   END AS ind_emit,
                   --Busca todas as tributacoes pis vl_bc_pis e vl_pis

                   sum(
                         (SELECT vr_base_calculo
                          FROM fiscal_nf_item_tributacao
                          WHERE encargo_id =
                              (SELECT id
                               FROM fiscal_encargo
                               WHERE sigla='PIS')
                            AND id = tributacao.id)) AS VL_BC_PIS,
                   sum(
                         (SELECT vr_encargo
                          FROM fiscal_nf_item_tributacao
                          WHERE encargo_id =
                              (SELECT id
                               FROM fiscal_encargo
                               WHERE sigla='PIS')
                            AND id = tributacao.id)) AS VL_PIS ,
                   --Busca tributacoes bc_cofins e vl cofins

                   sum(
                         (SELECT vr_base_calculo
                          FROM fiscal_nf_item_tributacao
                          WHERE encargo_id =
                              (SELECT id
                               FROM fiscal_encargo
                               WHERE sigla='COFINS')
                            AND id = tributacao.id)) AS VL_BC_COFINS,
                   sum(
                         (SELECT vr_encargo
                          FROM fiscal_nf_item_tributacao
                          WHERE encargo_id =
                              (SELECT id
                               FROM fiscal_encargo
                               WHERE sigla='COFINS')
                            AND id = tributacao.id)) AS VL_COFINS ,
                   --Buscao tributacao para ISS

                   sum(
                         (SELECT vr_encargo
                          FROM fiscal_nf_item_tributacao
                          WHERE encargo_id =
                              (SELECT id
                               FROM fiscal_encargo
                               WHERE sigla='ISS')
                            AND id = tributacao.id))AS VL_ISS ,
                   --Verificando condicoes de pagamentos

                   CASE
                       WHEN
                              (SELECT tipo
                               FROM adm_condicao
                               WHERE id =
                                   (SELECT adm_condicao_id
                                    FROM adm_condicao_forma
                                    WHERE id IN
                                        (SELECT condicao_forma_id
                                         FROM fiscal_condicao_pagamento
                                         WHERE capa_id = nf_capa.id)LIMIT 1)) IN ('C',
                                                                                  'P') THEN '1'
                       WHEN
                              (SELECT tipo
                               FROM adm_condicao
                               WHERE id =
                                   (SELECT adm_condicao_id
                                    FROM adm_condicao_forma
                                    WHERE id IN
                                        (SELECT condicao_forma_id
                                         FROM fiscal_condicao_pagamento
                                         WHERE capa_id = nf_capa.id)LIMIT 1)) IN ('V') THEN '0'
                       ELSE '9'
                   END AS IND_PGTO,
                   fisnfc_infcomp_contribuinte AS txt
            FROM fiscal_nf_capa nf_capa
            LEFT JOIN fiscal_nf_item_tributacao tributacao ON tributacao.fisnit_capa_id = nf_capa.id
            WHERE nf_capa.company_id = %s
              AND nf_capa.fisnfc_tipo_servico = TRUE
              AND nf_capa.numero_nf > 0  -- Notas faturadas que foram canceladas
              AND nf_capa.id IN %s
            GROUP BY nf_capa.id,
                     nf_capa.tipo,
                     nf_capa.state,
                     nf_capa.serie_nf,
                     nf_capa.numero_nf,
                     nf_capa.destinatario_id,
                     nf_capa.data_emissao,
                     nf_capa.data_saida_entrada,
                     nf_capa.vr_total_produtos,
                     nf_capa.vr_total_servicos,
                     nf_capa.vr_desconto
        """ % (company_id, mov_notas.replace('[', '(').replace(']', ')'))
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_A170(self, cr, filtro):
        nf_id = filtro
        sql = """
                    SELECT Row_number()
                             OVER (
                               ORDER BY movimento_item.id)                                  AS
                           num_item,
                           movimento_item.id AS movimento_item_id,
                           (SELECT codigo
                            FROM   estoque_produto
                            WHERE  id = produto_id)                                         AS
                           cod_item,
                           (SELECT name
                            FROM   estoque_produto_grade
                            WHERE  id = movimento_item.grade_id)                            AS
                           DESCR_COMPL,
                           vr_total_item                                                    AS
                           vl_item,
                           vr_desconto                                                      AS
                           vl_desc,
                           (SELECT fisnat_codigo_natureza_credito
                            FROM fiscal_cfop_natureza
                            WHERE id IN (SELECT natureza_id
                                         FROM fiscal_nf_capa
                                         WHERE id = movimento_item.nf_id))                  AS
                            nat_bc_cred,
                           CASE
                             WHEN (SELECT origem_mercadoria
                                   FROM   estoque_produto
                                   WHERE  id = produto_id) = '0' THEN '0'
                             WHEN (SELECT origem_mercadoria
                                   FROM   estoque_produto
                                   WHERE  id = produto_id) <> '0' THEN '1'
                           END                                                              AS
                           ind_orig_cred,
                           --BUSCA INFORMACOES PIS
                           (SELECT codigo
                            FROM   fiscal_cst
                            WHERE  id = (SELECT cst_id
                                         FROM   fiscal_nf_item_tributacao
                                         WHERE  encargo_id = (SELECT id
                                                              FROM   fiscal_encargo
                                                              WHERE  sigla = 'PIS')
                                                AND movimento_itens_id = movimento_item.id))AS
                           CST_PIS,
                           Sum((SELECT vr_base_calculo
                                FROM   fiscal_nf_item_tributacao
                                WHERE  encargo_id = (SELECT id
                                                     FROM   fiscal_encargo
                                                     WHERE  sigla = 'PIS')
                                       AND id = tributacao.id))         AS
                           VL_BC_PIS,
                           (SELECT aliquota
                            FROM   fiscal_nf_item_tributacao
                            WHERE  encargo_id = (SELECT id
                                                 FROM   fiscal_encargo
                                                 WHERE  sigla = 'PIS')
                                   AND movimento_itens_id = movimento_item.id)              AS
                           ALIQ_PIS,
                           Sum((SELECT vr_encargo
                                FROM   fiscal_nf_item_tributacao
                                WHERE  encargo_id = (SELECT id
                                                     FROM   fiscal_encargo
                                                     WHERE  sigla = 'PIS')
                                       AND id = tributacao.id))                             AS
                           VL_PIS,
                           --BUSCA INFORMACOES COFINS
                           (SELECT codigo
                            FROM   fiscal_cst
                            WHERE  id = (SELECT cst_id
                                         FROM   fiscal_nf_item_tributacao
                                         WHERE  encargo_id = (SELECT id
                                                              FROM   fiscal_encargo
                                                              WHERE  sigla = 'COFINS')
                                                AND movimento_itens_id = movimento_item.id))AS
                           CST_COFINS,
                           Sum((SELECT vr_base_calculo
                                FROM   fiscal_nf_item_tributacao
                                WHERE  encargo_id = (SELECT id
                                                     FROM   fiscal_encargo
                                                     WHERE  sigla = 'COFINS')
                                       AND id = tributacao.id))                             AS
                           VL_BC_COFINS,
                           (SELECT aliquota
                            FROM   fiscal_nf_item_tributacao
                            WHERE  encargo_id = (SELECT id
                                                 FROM   fiscal_encargo
                                                 WHERE  sigla = 'COFINS')
                                   AND movimento_itens_id = movimento_item.id)              AS
                           ALIQ_COFINS,
                           Sum((SELECT vr_encargo
                                FROM   fiscal_nf_item_tributacao
                                WHERE  encargo_id = (SELECT id
                                                     FROM   fiscal_encargo
                                                     WHERE  sigla = 'COFINS')
                                       AND id = tributacao.id))                             AS
                           VL_COFINS
                    FROM   estoque_movimento_itens movimento_item
                           LEFT JOIN fiscal_nf_item_tributacao tributacao
                                  ON tributacao.movimento_itens_id = movimento_item.id
                    WHERE  movimento_item.nf_id = %d
                    GROUP  BY movimento_item.id,
                              movimento_item.produto_id
        """ % nf_id
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_C380(self, cr, filtro):
        mov_notas = filtro.get('movimento_notas')
        company_id = filtro.get('company_id')
        sql = """
                SELECT
                       (SELECT Sum(vr_total_produtos + vr_total_servicos)
                        FROM   fiscal_nf_capa
                        WHERE  id = id
                               AND state = '3'
                               AND modelo_nf_id = (SELECT id
                                                   FROM   fiscal_modelo_nf
                                                   WHERE  fismnf_codigo = '02')) AS vl_doc,
                       (SELECT Sum(vr_total_produtos + vr_total_servicos)
                        FROM   fiscal_nf_capa
                        WHERE  id = id
                               AND state = 'C'
                               AND modelo_nf_id = (SELECT id
                                                   FROM   fiscal_modelo_nf
                                                   WHERE  fismnf_codigo = '02')) AS vl_doc_canc,
                       Min(data_emissao AT TIME ZONE 'UTC')               AS DT_DOC_INI,
                       Max(data_emissao AT TIME ZONE 'UTC')               AS DT_DOC_FIN,
                       Min(numero_nf)                                     AS NUM_DOC_INI,
                       Max(numero_nf)                                     AS NUM_DOC_FIN
                FROM   fiscal_nf_capa
                WHERE  company_id = %s
                       AND tipo = 'S'
                       AND modelo_nf_id = (SELECT id
                                           FROM   fiscal_modelo_nf
                                           WHERE  fismnf_codigo = '02')
                       AND id IN %s
                        """ % (company_id, str(mov_notas).replace('[', '(').replace(']', ')'))
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_C381(self, cr, filtro):
        nf_ids = filtro
        sql = """
            SELECT (SELECT codigo
                    FROM   estoque_produto
                    WHERE  id = produto_id)        AS cod_item,
                   Sum(vr_total_item)              AS vl_item,
                   Sum(vr_desconto)                AS vl_desc,
                   (SELECT codigo
                    FROM   fiscal_cst
                    WHERE  id = tributacao.cst_id) AS CST_PIS,
                   Sum(tributacao.vr_base_calculo) AS VL_BC_PIS,
                   tributacao.aliquota        AS ALIQ_PIS,
                   Sum(tributacao.vr_encargo)      AS VL_PIS
            FROM   estoque_movimento_itens movimento_item
                   LEFT JOIN fiscal_nf_item_tributacao tributacao
                          ON tributacao.movimento_itens_id = movimento_item.id
            WHERE  movimento_item.nf_id IN %s
                   AND tributacao.encargo_id IN (SELECT id
                                                 FROM   fiscal_encargo
                                                 WHERE  sigla = 'PIS')
                   AND movimento_item.item_cancelado = false
            GROUP  BY tributacao.cst_id,
                      movimento_item.produto_id,
                      tributacao.aliquota
        """ % (str(nf_ids).replace('[', '(').replace(']', ')'))
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_C385(self, cr, filtro):
        nf_ids = filtro
        sql = """
            SELECT (SELECT codigo
                    FROM   estoque_produto
                    WHERE  id = produto_id)        AS cod_item,
                   Sum(vr_total_item)              AS vl_item,
                   Sum(vr_desconto)                AS vl_desc,
                   (SELECT codigo
                    FROM   fiscal_cst
                    WHERE  id = tributacao.cst_id) AS CST_COFINS,
                   Sum(tributacao.vr_base_calculo) AS VL_BC_COFINS,
                   tributacao.aliquota        AS ALIQ_COFINS,
                   Sum(tributacao.vr_encargo)      AS VL_COFINS
            FROM   estoque_movimento_itens movimento_item
                   LEFT JOIN fiscal_nf_item_tributacao tributacao
                          ON tributacao.movimento_itens_id = movimento_item.id
            WHERE  movimento_item.nf_id IN %s
                   AND tributacao.encargo_id IN (SELECT id
                                                 FROM   fiscal_encargo
                                                 WHERE  sigla = 'COFINS')
                   AND movimento_item.item_cancelado = false
            GROUP  BY tributacao.cst_id,
                      movimento_item.produto_id,
                      tributacao.aliquota
        """ % (str(nf_ids).replace('[', '(').replace(']', ')'))
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_C481_C485(self, encargo, registro):
        cr = self.new_cursor()
        try:
            sql = """
                    SELECT ecf_id,
                           data,
                           cod_item,
                           Sum(vl_item) AS vl_item,
                           Sum(vl_desc) AS vl_desc,
                           cst_pis_cofins,
                           Sum(vl_bc_pis_cofins) AS vl_bc_pis_cofins,
                           aliq_pis_cofins,
                           Sum(vl_pis_cofins)  AS vl_pis_cofins,
                           cfop_id
                    FROM  (SELECT ecf_id                                                          AS
                                        ecf_id
                                        ,
                                  date_cupom
                                        AS data,
                                  min(movimento_item.cfop_id) AS cfop_id,
                                  (SELECT CASE
                                             WHEN produto.grade_usar = true THEN produto.codigo
                                                                                 || ' '
                                                                                 || grade.grade
                                             ELSE produto.codigo
                                           END AS cod_item
                                    FROM   estoque_produto produto
                                           INNER JOIN estoque_produto_grade grade
                                                   ON grade.produto_id = produto.id
                                    WHERE  grade.id = movimento_item.grade_id)
                                   AS
                                           cod_item,
                                  Sum(movimento_item.vr_total_item_rateado)                       AS
                                         vl_item,
                                  Sum(movimento_item.vr_desconto)                                 AS
                                         vl_desc,
                                  (SELECT codigo
                                   FROM   fiscal_cst
                                   WHERE  id = (SELECT cst_id
                                                FROM   fiscal_nf_item_tributacao
                                                WHERE  movimento_itens_id = movimento_item.id
                                                       AND encargo_id IN (SELECT id
                                                                          FROM   fiscal_encargo
                                                                          WHERE  sigla = '{encargo}')
                                                LIMIT 1)) AS CST_pis_cofins,
                                  Sum((SELECT sum(vr_base_calculo)
                                       FROM   fiscal_nf_item_tributacao
                                       WHERE  movimento_itens_id = movimento_item.id
                                              AND encargo_id IN (SELECT id
                                                                 FROM   fiscal_encargo
                                                                 WHERE  sigla = '{encargo}')
                                              AND cst_id IN (SELECT cst.id 
                                                             FROM fiscal_cst cst 
                                                             WHERE 
                                                                 cst.tipo = 'T'
                                                                 AND cst.encargo_id = encargo_id
                                                             )
                                              AND aliquota > 0
                                       )) AS VL_BC_pis_cofins,
                                  (SELECT aliquota
                                   FROM   fiscal_nf_item_tributacao
                                   WHERE  movimento_itens_id = movimento_item.id
                                          AND encargo_id IN (SELECT id
                                                             FROM   fiscal_encargo
                                                             WHERE  sigla = '{encargo}')
                                   LIMIT 1) AS ALIQ_pis_cofins,
                                  Sum((SELECT sum(vr_encargo)
                                       FROM   fiscal_nf_item_tributacao
                                       WHERE  movimento_itens_id = movimento_item.id
                                              AND encargo_id IN (SELECT id
                                                                 FROM   fiscal_encargo
                                                                 WHERE  sigla = '{encargo}')))          AS
                                         VL_pis_cofins
                           FROM   estoque_movimento_itens movimento_item
                                  INNER JOIN ecf_cupons_capa cupom_capa
                                          ON movimento_item.cupom_id = cupom_capa.id
                                             AND movimento_item.nf_id IS NULL
                           WHERE  cupom_capa.date_cupom BETWEEN
                                  '{dt_ini}' AND '{dt_fim}'
                                  AND movimento_item.item_cancelado = false
                                  AND cupom_capa.tipo_cupom = 'V'
                                  AND cupom_capa.state = 'V'
                                  AND cupom_capa.company_id IN {company_ids}
                           GROUP  BY cod_item,
                                     aliq_pis_cofins,
                                     cst_pis_cofins,
                                     ecf_id,
                                     date_cupom
                           UNION ALL
                           SELECT ecf_id                                                          AS
                                  ecf_id,
                                  date_cupom                                                      AS
                                  data,
                                  min(movimento_item.cfop_id) AS cfop_id,
                                  (SELECT CASE
                                             WHEN produto.grade_usar = true THEN produto.codigo
                                                                                 || ' '
                                                                                 || grade.grade
                                             ELSE produto.codigo
                                           END AS cod_item
                                    FROM   estoque_produto produto
                                           INNER JOIN estoque_produto_grade grade
                                                   ON grade.produto_id = produto.id
                                    WHERE  grade.id = movimento_item.grade_id)
                                   AS
                                           cod_item,
                                  Sum(-movimento_item.vr_total_item_rateado)                      AS
                                  vl_item,
                                  Sum(-movimento_item.vr_desconto)                                AS
                                  vl_desc,
                                  (SELECT codigo
                                   FROM   fiscal_cst
                                   WHERE  id = (SELECT cst_id
                                                FROM   fiscal_nf_item_tributacao
                                                WHERE  movimento_itens_id = movimento_item.id
                                                       AND encargo_id IN (SELECT id
                                                                          FROM   fiscal_encargo
                                                                          WHERE  sigla = '{encargo}')
                                                LIMIT 1)) AS CST_pis_cofins,
                                  Sum(-(SELECT sum(vr_base_calculo)
                                        FROM   fiscal_nf_item_tributacao
                                        WHERE  movimento_itens_id = movimento_item.id
                                               AND encargo_id IN (SELECT id
                                                                  FROM   fiscal_encargo
                                                                  WHERE  sigla = '{encargo}')
                                               AND cst_id IN (SELECT cst.id 
                                                              FROM fiscal_cst cst 
                                                              WHERE 
                                                                 cst.tipo = 'T'
                                                                 AND cst.encargo_id = encargo_id
                                                             )
                                               AND aliquota > 0
                                  )) AS VL_BC_pis_cofins,
                                  (SELECT aliquota
                                   FROM   fiscal_nf_item_tributacao
                                   WHERE  movimento_itens_id = movimento_item.id
                                          AND encargo_id IN (SELECT id
                                                             FROM   fiscal_encargo
                                                             WHERE  sigla = '{encargo}')
                                   LIMIT 1) AS ALIQ_pis_cofins,
                                  Sum(-(SELECT sum(vr_encargo)
                                        FROM   fiscal_nf_item_tributacao
                                        WHERE  movimento_itens_id = movimento_item.id
                                               AND encargo_id IN (SELECT id
                                                                  FROM   fiscal_encargo
                                                                  WHERE  sigla = '{encargo}')))         AS
                                  VL_pis_cofins
                           FROM   estoque_movimento_itens movimento_item
                                  INNER JOIN ecf_cupons_capa cupom_capa
                                          ON movimento_item.cupom_id = cupom_capa.id
                                             AND movimento_item.nf_id IS NULL
                           WHERE  cupom_capa.date_cupom BETWEEN
                                  '{dt_ini}' AND '{dt_fim}'
                                  AND cupom_capa.tipo_cupom = 'C'
                                  AND cupom_capa.state = 'V'
                                  AND cupom_capa.company_id IN {company_ids}
                           GROUP  BY cod_item,
                                     aliq_pis_cofins,
                                     cst_pis_cofins,
                                     ecf_id,
                                     date_cupom) AS TMP
                    --ORDER BY ecf_id, data, cod_item
                    GROUP  BY ecf_id,
                              cfop_id,
                              data,
                              cod_item,
                              cst_pis_cofins,
                              aliq_pis_cofins
                """.format(dt_ini=self.data_inicio, dt_fim=self.data_fim, encargo=encargo,
                           company_ids=self.company_ids)
            cr.execute(sql)
            if registro == 'C481':
                self.C481 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
            else:
                self.C485 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
        except:
            msg_erro = trace()
            if registro == 'C481':
                self.C481 = {'executando': False, 'dados': [], 'erro': msg_erro}
            else:
                self.C485 = {'executando': False, 'dados': [], 'erro': msg_erro}
        finally:
            cr.connection.close()

    #Todo: Em manutencao
    # def sql_C481_C485(self, encargo, registro):
    #     cr = self.new_cursor()
    #     try:
    #         sql = """
    #                 SELECT chave,
    #                        cod_item       AS cod_item,
    #                        Sum(vl_item)   AS vl_item,
    #                        Sum(vl_desc)   AS vl_desc,
    #                        cst_pis        AS cst_pis,
    #                        Sum(vl_bc_pis) AS vl_bc_pis,
    #                        aliq_pis       AS aliq_pis,
    #                        Sum(vl_pis)    AS vl_pis
    #                 FROM   (SELECT ecf_id
    #                                || '-'
    #                                || date_cupom                     AS chave,
    #                                CASE
    #                                  WHEN produto.grade_usar = true THEN produto.codigo
    #                                                                      || ' '
    #                                                                      || grade.grade
    #                                  ELSE produto.codigo
    #                                END                               AS cod_item,
    #                                Sum(movimento_item.vr_total_item) AS vl_item,
    #                                Sum(movimento_item.vr_desconto)   AS vl_desc,
    #                                (SELECT codigo
    #                                 FROM   fiscal_cst
    #                                 WHERE  id = tributacao.cst_id)   AS CST_PIS,
    #                                Sum(tributacao.vr_base_calculo)   AS VL_BC_PIS,
    #                                tributacao.aliquota               AS ALIQ_PIS,
    #                                Sum(tributacao.vr_encargo)        AS VL_PIS
    #                         FROM   estoque_movimento_itens movimento_item
    #                                INNER JOIN estoque_produto produto
    #                                        ON produto.id = movimento_item.produto_id
    #                                INNER JOIN estoque_produto_grade grade
    #                                        ON grade.id = movimento_item.grade_id
    #                                INNER JOIN fiscal_nf_item_tributacao tributacao
    #                                        ON tributacao.movimento_itens_id = movimento_item.id
    #                                INNER JOIN ecf_cupons_capa cupom_capa
    #                                        ON movimento_item.cupom_id = cupom_capa.id
    #                                           AND movimento_item.nf_id IS NULL
    #                         WHERE  cupom_capa.date_cupom BETWEEN
    #                                '{dt_ini}' AND '{dt_fim}'
    #                                AND tributacao.encargo_id IN (SELECT id
    #                                                              FROM   fiscal_encargo
    #                                                              WHERE  sigla = '{encargo}')
    #                                AND movimento_item.item_cancelado = false
    #                                AND cupom_capa.company_id IN {company_ids}
    #                         GROUP  BY produto.id,
    #                                   tributacao.cst_id,
    #                                   grade.id,
    #                                   tributacao.aliquota,
    #                                   chave
    #                         UNION ALL
    #                         SELECT ecf_id
    #                                || '-'
    #                                || date_cupom                      AS chave,
    #                                CASE
    #                                  WHEN produto.grade_usar = true THEN produto.codigo
    #                                                                      || ' '
    #                                                                      || grade.grade
    #                                  ELSE produto.codigo
    #                                END                                AS cod_item,
    #                                Sum(-movimento_item.vr_total_item) AS vl_item,
    #                                Sum(-movimento_item.vr_desconto)   AS vl_desc,
    #                                (SELECT codigo
    #                                 FROM   fiscal_cst
    #                                 WHERE  id = tributacao.cst_id)    AS CST_PIS,
    #                                Sum(-tributacao.vr_base_calculo)   AS VL_BC_PIS,
    #                                tributacao.aliquota                AS ALIQ_PIS,
    #                                Sum(-tributacao.vr_encargo)        AS VL_PIS
    #                         FROM   estoque_movimento_itens movimento_item
    #                                INNER JOIN estoque_produto produto
    #                                        ON produto.id = movimento_item.produto_id
    #                                INNER JOIN estoque_produto_grade grade
    #                                        ON grade.id = movimento_item.grade_id
    #                                INNER JOIN fiscal_nf_item_tributacao tributacao
    #                                        ON tributacao.movimento_itens_id = movimento_item.id
    #                                INNER JOIN ecf_cupons_capa cupom_capa
    #                                        ON movimento_item.cupom_id = cupom_capa.id
    #                                           AND movimento_item.nf_id IS NULL
    #                         WHERE  cupom_capa.date_cupom BETWEEN
    #                                '{dt_ini}' AND '{dt_fim}'
    #                                AND tributacao.encargo_id IN (SELECT id
    #                                                              FROM   fiscal_encargo
    #                                                              WHERE  sigla = '{encargo}')
    #                                AND cupom_capa.tipo_cupom = 'C'
    #                                AND cupom_capa.company_id IN {company_ids}
    #                         GROUP  BY produto.id,
    #                                   tributacao.cst_id,
    #                                   grade.id,
    #                                   tributacao.aliquota,
    #                                   chave) AS TMP
    #                 GROUP  BY chave,
    #                           cod_item,
    #                           cst_pis,
    #                           aliq_pis
    #                 HAVING Sum(vl_item) > 0
    #             """.format(dt_ini=self.data_inicio, dt_fim=self.data_fim, encargo=encargo,
    #                        company_ids=self.company_ids)
    #         cr.execute(sql)
    #         if registro == 'C481':
    #             self.C481 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
    #         else:
    #             self.C485 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
    #     except:
    #         msg_erro = trace()
    #         if registro == 'C481':
    #             self.C481 = {'executando': False, 'dados': [], 'erro': msg_erro}
    #         else:
    #             self.C485 = {'executando': False, 'dados': [], 'erro': msg_erro}
    #     finally:
    #         cr.connection.close()


    def sql_C485(self, cr, filtro):
            dt_doc = filtro.get('dt_doc')
            ecf_id = filtro.get('ecf_id')
            sql = """
                        SELECT cod_item       AS cod_item,
                               Sum(vl_item)   AS vl_item,
                               Sum(vl_desc)   AS vl_desc,
                               cst_cofins        AS cst_cofins,
                               Sum(vl_bc_cofins) AS vl_bc_cofins,
                               aliq_cofins       AS aliq_cofins,
                               Sum(vl_cofins)    AS vl_cofins
                        FROM  (SELECT CASE
                                        WHEN produto.grade_usar = true THEN produto.codigo
                                                                            || ' '
                                                                            || grade.grade
                                        ELSE produto.codigo
                                      END                               AS cod_item,
                                      Sum(movimento_item.vr_total_item_rateado) 
                                                                        AS vl_item,
                                      Sum(movimento_item.vr_desconto)   AS vl_desc,
                                      (SELECT codigo
                                       FROM   fiscal_cst
                                       WHERE  id = tributacao.cst_id)   AS CST_COFINS,
                                      Sum(tributacao.vr_base_calculo)   AS VL_BC_COFINS,
                                      tributacao.aliquota               AS ALIQ_COFINS,
                                      Sum(tributacao.vr_encargo)        AS VL_COFINS
                               FROM   estoque_movimento_itens movimento_item
                                      INNER JOIN estoque_produto produto
                                              ON produto.id = movimento_item.produto_id
                                      INNER JOIN estoque_produto_grade grade
                                              ON grade.id = movimento_item.grade_id
                                      INNER JOIN fiscal_nf_item_tributacao tributacao
                                              ON tributacao.movimento_itens_id = movimento_item.id
                                      INNER JOIN ecf_cupons_capa cupom_capa
                                              ON movimento_item.cupom_id = cupom_capa.id
                                              AND movimento_item.nf_id  IS NULL
                               WHERE  cupom_capa.date_cupom = '%s'
                                      AND cupom_capa.ecf_id = %d
                                      AND tributacao.encargo_id IN (SELECT id
                                                                    FROM   fiscal_encargo
                                                                    WHERE  sigla = 'COFINS')
                                      AND movimento_item.item_cancelado = false
                               GROUP  BY produto.id,
                                         tributacao.cst_id,
                                         grade.id,
                                         tributacao.aliquota
                               UNION ALL
                               SELECT CASE
                                        WHEN produto.grade_usar = true THEN produto.codigo
                                                                            || ' '
                                                                            || grade.grade
                                        ELSE produto.codigo
                                      END                                AS cod_item,
                                      Sum(-movimento_item.vr_total_item_rateado) 
                                                                         AS vl_item,
                                      Sum(-movimento_item.vr_desconto)   AS vl_desc,
                                      (SELECT codigo
                                       FROM   fiscal_cst
                                       WHERE  id = tributacao.cst_id)    AS CST_COFINS,
                                      Sum(-tributacao.vr_base_calculo)   AS VL_BC_COFINS,
                                      tributacao.aliquota                AS ALIQ_COFINS,
                                      Sum(-tributacao.vr_encargo)        AS VL_COFINS
                               FROM   estoque_movimento_itens movimento_item
                                      INNER JOIN estoque_produto produto
                                              ON produto.id = movimento_item.produto_id
                                      INNER JOIN estoque_produto_grade grade
                                              ON grade.id = movimento_item.grade_id
                                      INNER JOIN fiscal_nf_item_tributacao tributacao
                                              ON tributacao.movimento_itens_id = movimento_item.id
                                      INNER JOIN ecf_cupons_capa cupom_capa
                                              ON movimento_item.cupom_id = cupom_capa.id
                                              AND movimento_item.nf_id  IS NULL
                               WHERE  cupom_capa.date_cupom = '%s'
                                      AND cupom_capa.ecf_id = %d
                                      AND tributacao.encargo_id IN (SELECT id
                                                                    FROM   fiscal_encargo
                                                                    WHERE  sigla = 'COFINS')
                                      AND cupom_capa.tipo_cupom = 'C'
                               GROUP  BY produto.id,
                                         tributacao.cst_id,
                                         grade.id,
                                         tributacao.aliquota) AS TMP
                        GROUP  BY cod_item,
                                  cst_cofins,
                                  aliq_cofins
                        HAVING Sum(vl_item) > 0
                """ % (dt_doc, ecf_id, dt_doc, ecf_id)
            cr.execute(sql)
            dados = cr.dictfetchall()
            return dados

    def sql_C500(self, cr, filtro):
        nf_capa_ids = filtro.get('movimento_notas_energia')
        company_id = filtro.get('company_id')
        sql = """
                SELECT DISTINCT nf_capa.id                                                    AS
                                id,
                                CASE
                                  WHEN nf_capa.state = '3' THEN '00'
                                  WHEN nf_capa.state = 'C' THEN '02'
                                  WHEN nf_capa.state = 'I' THEN '05'
                                END                                                           AS
                                COD_SIT,
                                CASE
                                  WHEN nf_capa.nf_propria = true THEN '0'
                                  WHEN nf_capa.nf_propria = false THEN '1'
                                END                                                           AS
                                IND_EMIT,
                                serie_nf                                                      AS
                                SER,
                                numero_nf                                                     AS
                                NUM_DOC,
                                (SELECT Cast (codigo AS TEXT)
                                        || (SELECT Cast(id AS TEXT)
                                            FROM   adm_pessoa_endereco
                                            WHERE  id = nf_capa.destinatario_endereco_id)
                                 FROM   adm_pessoa
                                 WHERE  id = nf_capa.destinatario_id)                         AS
                                COD_PART,
                                (SELECT fismnf_codigo
                                 FROM   fiscal_modelo_nf
                                 WHERE  id = nf_capa.modelo_nf_id)                            AS
                                COD_MOD,
                                data_emissao AT TIME ZONE 'UTC'                               AS
                                DT_DOC,
                                data_saida_entrada AT TIME ZONE 'UTC'                         AS
                                DT_E_S,
                                vr_total_produtos + vr_total_servicos
                                + vr_despesas_acessorias + vr_seguro + vr_frete - vr_desconto AS
                                VL_DOC,
                                Sum((SELECT vr_encargo
                                     FROM   fiscal_nf_item_tributacao
                                     WHERE  encargo_id = (SELECT id
                                                          FROM   fiscal_encargo
                                                          WHERE  sigla = 'ICMS')
                                            AND id = tributacao.id))                          AS
                                VL_ICMS,
                                Sum((SELECT vr_encargo
                                     FROM   fiscal_nf_item_tributacao
                                     WHERE  encargo_id = (SELECT id
                                                          FROM   fiscal_encargo
                                                          WHERE  sigla = 'PIS')
                                            AND id = tributacao.id))                          AS
                                VL_PIS,
                                Sum((SELECT vr_encargo
                                     FROM   fiscal_nf_item_tributacao
                                     WHERE  encargo_id = (SELECT id
                                                          FROM   fiscal_encargo
                                                          WHERE  sigla = 'COFINS')
                                            AND id = tributacao.id))                          AS
                                VL_COFINS,
                                fisnfc_infcomp_contribuinte                                   AS
                                TXT,
                                nf_capa.nfe_chave_acesso AS chv_doce
                FROM   fiscal_nf_capa nf_capa
                       LEFT JOIN fiscal_nf_item_tributacao tributacao
                              ON tributacao.fisnit_capa_id = nf_capa.id
                WHERE  nf_capa.id IN %s
                AND    nf_capa.company_id = '%s'
                AND nf_capa.modelo_nf_id IN (SELECT id
                                                    FROM   fiscal_modelo_nf
                                                    WHERE  fismnf_codigo IN ( '66', '29', '28' ))
                GROUP  BY nf_capa.id,
                          nf_capa.tipo,
                          nf_capa.state,
                          nf_capa.serie_nf,
                          nf_capa.numero_nf,
                          nf_capa.destinatario_id,
                          nf_capa.data_emissao,
                          nf_capa.data_saida_entrada,
                          nf_capa.vr_total_produtos,
                          nf_capa.vr_total_servicos,
                          nf_capa.vr_desconto
        """ % (str(nf_capa_ids).replace('[', '(').replace(']', ')'), company_id)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_C501(self, cr, filtro):
        nf_id = filtro
        sql = """
                SELECT 
                       Sum(vr_total_item)                      AS vl_item,
                       --BUSCA INFORMACOES PIS
                       (SELECT codigo
                        FROM   fiscal_cst
                        WHERE  id = tributacao.cst_id) 		   AS CST_PIS,
                       Sum((SELECT vr_base_calculo
                            FROM   fiscal_nf_item_tributacao
                            WHERE  encargo_id = (SELECT id
                                                 FROM   fiscal_encargo
                                                 WHERE  sigla = 'PIS')
                                   AND id = tributacao.id))    AS VL_BC_PIS,
                       tributacao.aliquota		               AS ALIQ_PIS,
                       Sum((SELECT vr_encargo
                            FROM   fiscal_nf_item_tributacao
                            WHERE  encargo_id = (SELECT id
                                                 FROM   fiscal_encargo
                                                 WHERE  sigla = 'PIS')
                                   AND id = tributacao.id))    AS VL_PIS
                FROM   estoque_movimento_itens movimento_item
                       INNER JOIN fiscal_nf_item_tributacao tributacao
                              ON tributacao.movimento_itens_id = movimento_item.id
                       INNER JOIN fiscal_encargo encargo
                              ON encargo.id = tributacao.encargo_id
                WHERE  movimento_item.nf_id = %s
                AND encargo.sigla = 'PIS'
                GROUP  BY tributacao.cst_id, tributacao.aliquota
        """ % nf_id
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_C505(self, cr, filtro):
        nf_id = filtro
        sql = """
            SELECT 
                   Sum(vr_total_item)                   AS vl_item,
                   --BUSCA INFORMACOES COFINS
                   (SELECT codigo
                    FROM   fiscal_cst
                    WHERE  id = tributacao.cst_id)		AS CST_COFINS,
                   tributacao.aliquota         			AS ALIQ_COFINS,
                   Sum((SELECT vr_encargo
                        FROM   fiscal_nf_item_tributacao
                        WHERE  encargo_id = (SELECT id
                                             FROM   fiscal_encargo
                                             WHERE  sigla = 'COFINS')
                               AND id = tributacao.id)) AS VL_COFINS,
                   Sum((SELECT vr_base_calculo
                        FROM   fiscal_nf_item_tributacao
                        WHERE  encargo_id = (SELECT id
                                             FROM   fiscal_encargo
                                             WHERE  sigla = 'COFINS')
                               AND id = tributacao.id)) AS VL_BC_COFINS
            FROM   estoque_movimento_itens movimento_item
                   INNER JOIN fiscal_nf_item_tributacao tributacao
                          ON tributacao.movimento_itens_id = movimento_item.id
                   INNER JOIN fiscal_encargo encargo
                          ON encargo.id = tributacao.encargo_id
            WHERE  movimento_item.nf_id = %s
            AND encargo.sigla = 'COFINS'
            GROUP  BY tributacao.cst_id, tributacao.aliquota
        """ % nf_id
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_D200(self, cr, filtro):
        mov_notas = filtro
        sql = """
                SELECT CASE
                         WHEN state = '3' THEN '00'
                         WHEN state = 'C' THEN '02'
                         WHEN state = 'I' THEN '05'
                       END                                        AS COD_SIT,
                       serie_nf                                   AS SER,
                       (SELECT codigo
                        FROM   fiscal_cfop
                        WHERE  id = (SELECT cfop_id
                                     FROM   fiscal_cfop_natureza
                                     WHERE  id = natureza_id))    AS CFOP,
                       natureza_id                                AS natureza_id,
                       (SELECT codigo
                        FROM   fiscal_modelo_nf
                        WHERE  id = modelo_nf_id)                 AS COD_MOD,
                       Sum(vr_total_produtos + vr_total_servicos) AS VL_DOC,
                       Sum(vr_desconto)                           AS VL_DESC,
                       CASE
                          WHEN tipo = 'E' THEN Min(data_saida_entrada AT TIME ZONE 'UTC')
                          ELSE Min(data_emissao AT TIME ZONE 'UTC')
                       END                                        AS DT_DOC_INI,
                       CASE
                          WHEN tipo = 'E' THEN Max(data_saida_entrada AT TIME ZONE 'UTC')
                          ELSE Max(data_emissao AT TIME ZONE 'UTC')
                       END                                        AS DT_DOC_FIN,
                       Min(numero_nf)                             AS NUM_DOC_INI,
                       Max(numero_nf)                             AS NUM_DOC_FIN,
                       data_emissao AT TIME ZONE 'UTC'            AS DT_REF
                FROM   fiscal_nf_capa
                WHERE  modelo_nf_id IN (SELECT id
                                        FROM   fiscal_modelo_nf
                                        WHERE  fismnf_codigo IN ( '07', '08', '8B', '09',
                                                                  '10', '11', '26', '27', '57' ))
                       AND tipo = 'S'
                       AND id IN %s
                GROUP  BY modelo_nf_id,
                          state,
                          natureza_id,
                          data_emissao,
                          serie_nf
        """ % mov_notas
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_D201(self, cr, filtro):
        sql = """
                SELECT (SELECT codigo
                        FROM   estoque_produto
                        WHERE  id = produto_id)        AS cod_item,
                       vr_total_item                   AS vl_item,
                       vr_desconto                     AS vl_desc,
                       (SELECT codigo
                        FROM   fiscal_cst
                        WHERE  id = tributacao.cst_id) AS CST_PIS,
                       Sum(tributacao.vr_base_calculo) AS VL_BC_PIS,
                       Sum(tributacao.aliquota)        AS ALIQ_PIS,
                       Sum(tributacao.vr_encargo)      AS VL_PIS,
                       movimento_item.id               AS movimento_item_id
                FROM   estoque_movimento_itens movimento_item
                       LEFT JOIN fiscal_nf_item_tributacao tributacao
                              ON tributacao.movimento_itens_id = movimento_item.id
                WHERE  movimento_item.nf_id IN ( 1 )
                       AND movimento_item.nf_id IN (SELECT id
                                                    FROM   fiscal_nf_capa
                                                    WHERE
                           modelo_nf_id IN (SELECT id
                                            FROM   fiscal_modelo_nf
                                            WHERE
                           fismnf_codigo IN ( '07', '08',
                                             '8B', '09',
                                             '10', '11',
                                             '26', '27',
                                             '57' )))
                       AND tributacao.encargo_id IN (SELECT id
                                                     FROM   fiscal_encargo
                                                     WHERE  sigla = 'PIS')
                GROUP  BY tributacao.cst_id,
                          movimento_item.id,
                          movimento_item.produto_id,
                          movimento_item.vr_total_item,
                          movimento_item.vr_desconto,
                          tributacao.id
        """
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_D205(self, cr, filtro):
        sql = """
                SELECT (SELECT codigo
                        FROM   estoque_produto
                        WHERE  id = produto_id)        AS cod_item,
                       vr_total_item                   AS vl_item,
                       vr_desconto                     AS vl_desc,
                       (SELECT codigo
                        FROM   fiscal_cst
                        WHERE  id = tributacao.cst_id) AS CST_COFINS,
                       Sum(tributacao.vr_base_calculo) AS VL_BC_COFINS,
                       Sum(tributacao.aliquota)        AS ALIQ_COFINS,
                       Sum(tributacao.vr_encargo)      AS VL_COFINS,
                       movimento_item.id               AS movimento_item_id
                FROM   estoque_movimento_itens movimento_item
                       LEFT JOIN fiscal_nf_item_tributacao tributacao
                              ON tributacao.movimento_itens_id = movimento_item.id
                WHERE  movimento_item.nf_id IN %s
                       AND movimento_item.nf_id IN (SELECT id
                                                    FROM   fiscal_nf_capa
                                                    WHERE
                           modelo_nf_id IN (SELECT id
                                            FROM   fiscal_modelo_nf
                                            WHERE
                           fismnf_codigo IN ( '07', '08',
                                             '8B', '09',
                                             '10', '11',
                                             '26', '27',
                                             '57' )))
                       AND tributacao.encargo_id IN (SELECT id
                                                     FROM   fiscal_encargo
                                                     WHERE  sigla = 'COFINS')
                GROUP  BY tributacao.cst_id,
                          movimento_item.id,
                          movimento_item.produto_id,
                          movimento_item.vr_total_item,
                          movimento_item.vr_desconto,
                          tributacao.id
        """
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_M100(self):
        cr = self.new_cursor()
        query_nf_nao_fiscais = ''
        if self.nf_nao_fiscais:
            query_nf_nao_fiscais = """AND tributacao.fisnit_capa_id NOT IN {nf_nao_fiscais}
            """.format(nf_nao_fiscais=str(self.nf_nao_fiscais).replace('[', '(').replace(']', ')'))
        try:
            sql = """
                    SELECT Sum(tributacao.vr_base_calculo)                     AS VL_BC_PIS,
                           tributacao.aliquota                                 AS ALIQ_PIS,
                           Sum(tributacao.vr_encargo)                          AS VL_CRED,
                           cst.codigo                                          AS CST_CODIGO
                    FROM   fiscal_nf_item_tributacao tributacao
                           INNER JOIN fiscal_cst cst
                                   ON cst.id = tributacao.cst_id
                    WHERE  tributacao.fisnit_capa_id IN (SELECT id
                                                         FROM   fiscal_nf_capa
                                                         WHERE  state = '3'
                                                                AND company_id IN {company_ids}
                                                                AND tipo = 'E'
                                                                AND data_saida_entrada AT TIME ZONE 'UTC'
                                                                 BETWEEN '{dt_ini}' AND '{dt_fim}'
                                                                AND modelo_nf_id NOT IN (Select id
                                                                from fiscal_modelo_nf  WHERE
                                                                fismnf_codigo >= '90'))
                           AND tributacao.encargo_id IN (SELECT id
                                                         FROM   fiscal_encargo
                                                         WHERE  sigla = 'PIS')
                           AND cst.codigo IN( '50', '51', '52', '53',
                                              '54', '55', '56', '60',
                                              '61', '62', '63', '64',
                                              '65', '66' )
                           {query_nf_nao_fiscais}
                    GROUP  BY tributacao.cst_id,
                              tributacao.aliquota,
                              cst.codigo
            """.format(dt_ini=self.data_inicio, dt_fim=self.data_fim,
                       query_nf_nao_fiscais=query_nf_nao_fiscais, company_ids=self.company_ids)
            cr.execute(sql)
            self.M100 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
        except:
            msg_erro = trace()
            self.M100 = {'executando': False, 'dados': [], 'erro': msg_erro}
        finally:
            cr.connection.close()

    def sql_M105(self):
        codigos = "'50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63', '64', '65', '66'"
        cr = self.new_cursor()
        query_nf_nao_fiscais = ''
        if self.nf_nao_fiscais:
            query_nf_nao_fiscais = """
            AND fisnfc.id NOT IN {nf_nao_fiscais}
            """.format(nf_nao_fiscais=str(self.nf_nao_fiscais).replace('[', '(').replace(']', ')'))
        try:
            sql = """
                    SELECT CASE
                             WHEN cfop IN ( '1102', '1113', '1117', '1118',
                                            '1121', '1251', '1403', '2102',
                                            '2113', '2117', '2118', '2121',
                                            '2251', '2403', '3102', '3251',
                                            '1652')                         THEN '01' :: text
                             WHEN cfop IN ( '1101', '1111', '1116', '1120',
                                            '1122', '1126', '1128', '1401',
                                            '1407', '1556', '1651', '1653',
                                            '2101', '2111', '2116', '2120',
                                            '2122', '2126', '2401', '2556', '2651',
                                            '2653', '3101', '3126', '3651', '3653' ) THEN '02' ::
                             text
                             WHEN cfop IN( '1124', '1125', '1933', '2124',
                                           '2125', '2933', '1303', '2303') THEN '03' :: text
                             WHEN cfop IN( '1252', '1253', '1254', '1255',
                                           '1256', '1257', '2252', '2253',
                                           '2254', '2255', '2256', '2257' ) THEN '04' :: text
                             WHEN cfop IN( '1352', '1353', '1354', '1355',
                                           '1356', '1932', '2352', '2353',
                                           '2354', '2355', '2356', '2932',
                                           '3352', '3353', '3356' ) THEN '07' :: text
                             WHEN cfop IN( '1201', '1202', '1410', '1411', '2411',
                                           '2201', '2202', '2410' ) THEN '12' :: text
                             WHEN cfop IN( '1351', '2351', '3351' ) THEN '14' :: text
                             ELSE ''
                           END                AS codigo_natureza_credito,
                           SUM(vl_bc_pis_tot) AS vl_bc_pis_tot,
                           aliquota,
                           cst_pis            AS cst_pis
                    FROM   (SELECT CASE
                                     WHEN fisnit.movimento_itens_id IS NOT NULL THEN cfop.codigo
                                     ELSE cfop_frete.codigo
                                   END                         AS cfop,
                                   SUM(fisnit.vr_base_calculo) AS vl_bc_pis_tot,
                                   fisnit.aliquota,
                                   fiscst.codigo               AS cst_pis
                            FROM   fiscal_nf_item_tributacao fisnit
                                   inner join fiscal_nf_capa fisnfc
                                           ON fisnfc.id = fisnit.fisnit_capa_id
                                   inner join fiscal_encargo fisenc
                                           ON fisenc.id = fisnit.encargo_id
                                   left join estoque_movimento_itens estmov
                                          ON estmov.id = fisnit.movimento_itens_id
                                   inner join fiscal_cst fiscst
                                           ON fiscst.id = fisnit.cst_id
                                   left join fiscal_cfop cfop
                                          ON cfop.id = estmov.cfop_id
                                   left join fiscal_cfop_natureza fisnat
                                          ON fisnat.id = fisnfc.natureza_id
                                   left join fiscal_cfop cfop_frete
                                          ON cfop_frete.id = fisnat.cfop_id
                            WHERE  fisnfc.data_saida_entrada AT TIME ZONE 'UTC'
                                BETWEEN '{dt_ini}' AND '{dt_fim}'
                                   AND fisnfc.tipo = 'E'
                                   AND fisenc.sigla = 'PIS'
                                   AND fisnfc.company_id in {company_ids}
                                   AND fisnfc.state = '3'
                                   AND fiscst.codigo IN ( {codigos} )
                                   {query_nf_nao_fiscais}
                            GROUP  BY cfop.codigo,
                                      fisnit.aliquota,
                                      fiscst.codigo,
                                      fisnit.movimento_itens_id,
                                      cfop_frete.codigo) AS TMP
                    GROUP  BY codigo_natureza_credito,
                              aliquota,
                              cst_pis
            """.format(dt_ini=self.data_inicio, dt_fim=self.data_fim, codigos=codigos,
                       query_nf_nao_fiscais=query_nf_nao_fiscais, company_ids=self.company_ids)
            cr.execute(sql)
            self.M105 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
        except:
            msg_erro = trace()
            self.M105 = {'executando': False, 'dados': [], 'erro': msg_erro}
        finally:
            cr.connection.close()

    def sql_total_debito(self, cr, filtro, nf_nao_fiscais):
        data_ini = filtro.get('data_ini')
        data_fim = filtro.get('data_fim')
        ecf_instalado = filtro.get('ecf_instalado')
        encargo = filtro.get('encargo')
        company_ids = str(filtro.get('company_ids')).replace('[', '(').replace(']', ')')
        if not nf_nao_fiscais:
            nf_nao_fiscais = ''
        else:
            nf_nao_fiscais = str(nf_nao_fiscais).replace('[', '(').replace(']', ')')
            nf_nao_fiscais = 'AND fn.id NOT IN {nf_nao_fiscais}'.format(
                nf_nao_fiscais=nf_nao_fiscais
            )
        sql_nf = """
        SELECT tr.vr_base_calculo,
               tr.aliquota,
               tr.vr_encargo
        FROM   fiscal_nf_item_tributacao tr
               INNER JOIN fiscal_encargo fe
                       ON fe.id = tr.encargo_id
               INNER JOIN estoque_movimento_itens emi
                       ON emi.id = tr.movimento_itens_id
               INNER JOIN fiscal_nf_capa fn
                       ON fn.id = emi.nf_id
               INNER JOIN fiscal_cfop cfop
                       ON cfop.id = emi.cfop_id
        WHERE  fe.sigla = '{encargo}'
               AND fn.data_emissao AT TIME ZONE 'UTC' BETWEEN
                   '{data_ini}' AND '{data_fim}'
               AND fn.company_id IN {company_ids}
               AND fn.state = '3'
               AND fn.tipo = 'S'
               AND cfop.codigo NOT IN ('5929', '6929')
               {nf_nao_fiscais}
        """
        sql_nf = sql_nf.format(data_ini=data_ini, data_fim=data_fim, encargo=encargo,
                               company_ids=company_ids, nf_nao_fiscais=nf_nao_fiscais)
        if ecf_instalado:
            sql_ecf = """
                    UNION ALL
                    SELECT tr.vr_base_calculo,
                           tr.aliquota,
                           tr.vr_encargo
                    FROM   fiscal_nf_item_tributacao tr
                           INNER JOIN fiscal_encargo fe
                                   ON fe.id = tr.encargo_id
                           INNER JOIN estoque_movimento_itens emi
                                   ON emi.id = tr.movimento_itens_id
                           INNER JOIN ecf_cupons_capa ecc
                                   ON ecc.id = emi.cupom_id
                                   AND emi.nf_id IS NULL
                    WHERE  fe.sigla = '{encargo}'
                           AND ecc.date_cupom BETWEEN '{data_ini}' AND '{data_fim}'
                           AND emi.item_cancelado = false
                           AND state = 'V'
                           AND tipo_cupom = 'V'
                           AND ecc.company_id IN {company_ids}
                    UNION ALL
                    SELECT -tr.vr_base_calculo,
                           tr.aliquota,
                           -tr.vr_encargo
                    FROM   fiscal_nf_item_tributacao tr
                           INNER JOIN fiscal_encargo fe
                                   ON fe.id = tr.encargo_id
                           INNER JOIN estoque_movimento_itens emi
                                   ON emi.id = tr.movimento_itens_id
                           INNER JOIN ecf_cupons_capa ecc
                                   ON ecc.id = emi.cupom_id
                                   AND emi.nf_id IS NULL
                    WHERE  fe.sigla = '{encargo}'
                           AND ecc.date_cupom BETWEEN '{data_ini}' AND '{data_fim}'
                           --AND emi.item_cancelado = False
                           AND state = 'V'
                           AND tipo_cupom = 'C'
                           AND ecc.company_id IN {company_ids}
            """
            sql_ecf = sql_ecf.format(data_ini=data_ini, data_fim=data_fim, encargo=encargo,
                                     company_ids=company_ids)
        else:
            sql_ecf = ""

        sql = """
        SELECT 
            Sum(vr_base_calculo) AS vr_base_calculo,
	        aliquota
        FROM(
            {sql_nf}
            {sql_ecf}
            ) AS TMP
        WHERE aliquota > 0
        GROUP BY aliquota
        """
        sql = sql.format(sql_nf=sql_nf, sql_ecf=sql_ecf)
        cr.execute(sql)
        dados = cr.dictfetchall()
        dados1 = []
        for dado in dados:
            dados1.append({
                'vl_tot_cont_nc_per': round(
                    dado.get('vr_base_calculo') * dado.get('aliquota') / 100, 2)})
        return dados1


    def sql_M200(self, cr, filtro):
        data_ini = filtro.get('data_ini')
        data_fim = filtro.get('data_fim')
        ecf_instalado = filtro.get('ecf_instalado')
        union_ecf = ') AS TMP'
        if ecf_instalado:
            union_ecf = """
                           UNION ALL
                           SELECT Sum(tributacao.vr_encargo)                             AS
                                  VL_TOT_CONT_NC_PER,
                                  Sum(tributacao.vr_reducao_base)                        AS
                                  VL_RET_NC,
                                  Sum(tributacao.vr_encargo -
                                  tributacao.vr_reducao_base)VL_CONT_NC_REC
                           FROM   fiscal_nf_item_tributacao tributacao
                                  INNER JOIN estoque_movimento_itens mov_item
                                          ON tributacao.movimento_itens_id = mov_item.id
                                  INNER JOIN ecf_cupons_capa cupom
                                          ON cupom.id = mov_item.cupom_id
                                          AND mov_item.nf_id IS NULL
                           WHERE  cupom.date_cupom >= '%s'
                                  AND cupom.date_cupom <= '%s'
                                  AND mov_item.cupom_id IN (SELECT id
                                                            FROM   ecf_cupons_capa
                                                            WHERE  date_cupom >= '%s'
                                                                   AND date_cupom <= '%s'
                                                                   AND tipo_cupom = 'V'
                                                                   AND state = 'V')
                                  AND mov_item.nf_id IS NULL
                                  AND mov_item.item_cancelado = false
                                  AND tributacao.encargo_id IN (SELECT id
                                                                FROM   fiscal_encargo
                                                                WHERE  sigla = 'PIS')) AS Tmp

            """ % (data_ini, data_fim, data_ini, data_fim)
        sql = """
                SELECT Sum (vl_tot_cont_nc_per2)                                      AS
                       vl_tot_cont_nc_per,
                       Sum (vl_ret_nc2)                                               AS
                       vl_ret_nc,
                       Sum (vl_cont_nc_rec2)                                          AS
                       vl_cont_nc_rec
                FROM  (SELECT Sum(tributacao.vr_encargo)                             AS
                                    VL_TOT_CONT_NC_PER2,
                              Sum(tributacao.vr_reducao_base)                        AS
                              VL_RET_NC2,
                              Sum(tributacao.vr_encargo -
                              tributacao.vr_reducao_base)VL_CONT_NC_REC2
                       FROM   fiscal_nf_item_tributacao tributacao
                              INNER JOIN estoque_movimento_itens mov_item
                                      ON tributacao.movimento_itens_id = mov_item.id
                              INNER JOIN fiscal_nf_capa nf_capa
                                      ON nf_capa.id = mov_item.nf_id
                       WHERE  nf_capa.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                              AND fisnit_capa_id IN (SELECT id
                                                     FROM   fiscal_nf_capa
                                                     WHERE  data_emissao BETWEEN '%s' AND '%s'
                                                            AND state = '3'
                                                            AND tipo = 'S')
                       %s
        """ % (data_ini, data_fim, data_ini, data_fim, union_ecf)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_M210(self, cr, filtro, nf_nao_fiscais):
        #TODO: verificar possibilidade de desmenbrar os cupons das notas de forma mais legivel
        data_ini = filtro.get('data_ini')
        data_fim = filtro.get('data_fim')
        ecf_instalado = filtro.get('ecf_instalado')
        company_ids = filtro.get('company_ids')
        if not nf_nao_fiscais:
            nf_nao_fiscais = ''
        else:
            nf_nao_fiscais = str(nf_nao_fiscais).replace('[', '(').replace(']', ')')
            nf_nao_fiscais = 'AND nf_capa.id NOT IN {nf_nao_fiscais}'.format(
                nf_nao_fiscais=nf_nao_fiscais
            )
        union = """
        )AS tmp
                GROUP  BY cst_pis2,
                          aliq_pis2
        """
        if ecf_instalado:
            union = """
                        UNION ALL
                        SELECT (SELECT codigo
                                FROM   fiscal_cst
                                WHERE  id = tributacao.cst_id)    AS cst_pis2,
                               Sum (movimento_item.vr_total_item_rateado)  + 
                               (SELECT Sum(COALESCE(vr_icms_substituicao, 0.0)) 
                                FROM fiscal_nf_item_tributacao 
                                WHERE movimento_itens_id = movimento_item.id)
                                                                  AS VL_REC_BRT2,
                               Sum (CASE WHEN cst.tipo = 'U' THEN 0 ELSE tributacao.vr_base_calculo 
                                    END)                          AS VL_BC_CONT2,
                               tributacao.aliquota                AS ALIQ_PIS2,
                               Sum(tributacao.vr_encargo)         AS VL_CONT_APUR2
                        FROM   estoque_movimento_itens movimento_item
                               INNER JOIN fiscal_nf_item_tributacao tributacao
                                       ON tributacao.movimento_itens_id = movimento_item.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                               INNER JOIN ecf_cupons_capa cupom
                                       ON cupom.id = movimento_item.cupom_id
                                       AND movimento_item.nf_id  IS NULL
                               LEFT JOIN fiscal_cst cst
                                       ON cst.id = tributacao.cst_id
                        WHERE  movimento_item.cupom_id IN (SELECT id
                                                           FROM   ecf_cupons_capa
                                                           WHERE  date_cupom >= '%s'
                                                                  AND date_cupom <= '%s'
                                                                  AND state = 'V'
                                                                  AND tipo_cupom = 'V')
                               AND cupom.date_cupom >= '%s'
                               AND cupom.date_cupom <= '%s'
                               AND encargo.sigla = 'PIS'
                               AND movimento_item.item_cancelado = false
                               AND cupom.company_id IN {company_ids}
                               AND (cst.tipo IN ('T', 'TU') 
                                    OR (cst.tipo = 'U' AND tributacao.aliquota > 0))
                        GROUP  BY tributacao.cst_id,
                                  tributacao.aliquota,
                                  movimento_item.id
                UNION ALL --Uniao da venda menos o cancelamento
                        SELECT (SELECT codigo
                                FROM   fiscal_cst
                                WHERE  id = tributacao.cst_id)    AS cst_pis2,
                               Sum (-movimento_item.vr_total_item_rateado)  + 
                               (SELECT Sum(COALESCE(-vr_icms_substituicao, 0.0)) 
                                FROM fiscal_nf_item_tributacao 
                                WHERE movimento_itens_id = movimento_item.id) 
                                                                  AS VL_REC_BRT2,
                               Sum (CASE WHEN cst.tipo = 'U' THEN 0 ELSE -tributacao.vr_base_calculo 
                                    END)                          AS VL_BC_CONT2,
                               tributacao.aliquota                AS ALIQ_PIS2,
                               Sum(-tributacao.vr_encargo)        AS VL_CONT_APUR2
                        FROM   estoque_movimento_itens movimento_item
                               INNER JOIN fiscal_nf_item_tributacao tributacao
                                       ON tributacao.movimento_itens_id = movimento_item.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                               INNER JOIN ecf_cupons_capa cupom
                                       ON cupom.id = movimento_item.cupom_id
                                       AND movimento_item.nf_id  IS NULL
                               LEFT JOIN fiscal_cst cst
                                       ON cst.id = tributacao.cst_id
                        WHERE  movimento_item.cupom_id IN (SELECT id
                                                           FROM   ecf_cupons_capa
                                                           WHERE  date_cupom >= '%s'
                                                                  AND date_cupom <= '%s'
                                                                  AND state = 'V'
                                                                  AND tipo_cupom = 'C')
                               AND cupom.date_cupom >= '%s'
                               AND cupom.date_cupom <= '%s'
                               AND encargo.sigla = 'PIS'
                               AND cupom.company_id IN {company_ids}
                               AND (cst.tipo IN ('T', 'TU') 
                                    OR (cst.tipo = 'U' AND tributacao.aliquota > 0))
                        GROUP  BY tributacao.cst_id,
                                  tributacao.aliquota,
                                  movimento_item.id) AS tmp
                GROUP  BY cst_pis2,
                          aliq_pis2
            """ % (data_ini, data_fim, data_ini, data_fim, data_ini, data_fim, data_ini, data_fim)
        sql = """
                SELECT cst_pis2           AS cst_pis,
                       Sum(vl_rec_brt2)   AS VL_REC_BRT,
                       Sum(vl_bc_cont2)   AS VL_BC_CONT,
                       aliq_pis2          AS ALIQ_PIS,
                       Sum(vl_cont_apur2) AS VL_CONT_APUR
                FROM   (SELECT (SELECT codigo
                                FROM   fiscal_cst
                                WHERE  id = tributacao.cst_id)    AS cst_pis2,
                               Sum(movimento_item.vr_total_item_rateado)  + 
                               (SELECT Sum(COALESCE(vr_icms_substituicao, 0.0)) 
                                FROM fiscal_nf_item_tributacao 
                                WHERE movimento_itens_id = movimento_item.id) 
                                                                  AS VL_REC_BRT2,
                               Sum (CASE WHEN cst.tipo = 'U' THEN 0 ELSE tributacao.vr_base_calculo 
                                    END)                          AS VL_BC_CONT2,
                               tributacao.aliquota                AS ALIQ_PIS2,
                               Sum(tributacao.vr_encargo)         AS VL_CONT_APUR2
                        FROM   estoque_movimento_itens movimento_item
                               INNER JOIN fiscal_nf_item_tributacao tributacao
                                       ON tributacao.movimento_itens_id = movimento_item.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                               INNER JOIN fiscal_nf_capa nf_capa
                                       ON nf_capa.id = movimento_item.nf_id
                               INNER JOIN fiscal_cfop cfop
                                       ON cfop.id = movimento_item.cfop_id
                               LEFT JOIN fiscal_cst cst
                                       ON cst.id = tributacao.cst_id
                        WHERE  movimento_item.nf_id IN (SELECT id
                                                        FROM   fiscal_nf_capa
                                                        WHERE
                               data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                               AND state = '3'
                               AND tipo = 'S')
                               AND nf_capa.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                               AND encargo.sigla = 'PIS'
                               AND cfop.codigo NOT IN ('5929', '6929')
                               AND cfop.fiscfop_sped_contribuicoes IS TRUE
                               AND nf_capa.company_id IN {company_ids}
                               AND (cst.tipo IN ('T', 'TU') 
                                    OR (cst.tipo = 'U' AND tributacao.aliquota > 0))
                               {nf_nao_fiscais}
                        GROUP  BY tributacao.cst_id,
                                  tributacao.aliquota,
                                  movimento_item.id
                %s --union de ecf_cupons_capa

        """ % (data_ini, data_fim, data_ini, data_fim, union)
        sql = sql.format(company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                         nf_nao_fiscais=nf_nao_fiscais)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_M400(self, cr, filtro):
        filtro['company_ids'] = str(filtro.get('company_ids')).replace('[', '(').replace(']', ')')
        sql = """
        SELECT SUM(VL_TOT_REC)                      AS VL_TOT_REC
             , CST_PIS
             , NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
             , COD_CTA
             """
        sql += """
        FROM (
            SELECT emi.vr_total_item_rateado +
                   COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                 AND movimento_itens_id = emi.id), 0) +
                   COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                 AND movimento_itens_id = emi.id), 0)                  
                                                    AS VL_TOT_REC
                 , (SELECT codigo 
                    FROM fiscal_cst 
                    WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))) 
                                                    AS CST_PIS
                  , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))
                       END                              AS NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
                 , cc.id                            AS COD_CTA"""
        sql += """
            FROM estoque_movimento_itens            AS emi
            INNER JOIN fiscal_cfop                  AS fcfop 
                    ON fcfop.id = emi.cfop_id
            INNER JOIN fiscal_nf_capa               AS fnc 
                    ON fnc.id = emi.nf_id 
            INNER JOIN fiscal_nf_item_tributacao    AS fnit 
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo               AS fe 
                    ON fe.id = fnit.encargo_id
            INNER JOIN fiscal_cst                   AS fcst 
                    ON fcst.id = fnit.cst_id"""
        if filtro.get('contabil_fiscal'):
            sql += """
            INNER JOIN fiscal_cfop_natureza         AS fcn 
                    ON fcn.id = fnc.natureza_id
            LEFT  JOIN contabil_natureza_fiscal     AS cnf 
                    ON cnf.ctbnfi_natureza_id = fcn.id 
                    AND cnf.company_id = %(natureza_fiscal_company_id)s
            LEFT  JOIN contabil_conta               AS cc 
                    ON cc.id = cnf.ctbnfi_conta_debito_id"""
        sql += """
            WHERE fcfop.codigo NOT IN ('5929', '6929')
              AND fcfop.fiscfop_sped_contribuicoes IS TRUE
              AND fnc.company_id IN %(company_ids)s
              AND fnc.data_emissao AT TIME ZONE 'UTC' BETWEEN '%(data_ini)s' AND '%(data_fim)s'
              AND fnc.state = '3' 
              AND fnc.tipo = 'S'
              AND fe.sigla = 'PIS'
              AND fcst.codigo IN ('04', '06', '07', '08', '09') 
            GROUP BY fcst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
        if filtro.get('contabil_fiscal'):
            sql += """, cc.id"""
        if filtro.get('ecf_instalado'):
            sql += """
            UNION ALL
            SELECT emi.vr_total_item_rateado +
                   COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                 AND movimento_itens_id = emi.id), 0) +
                   COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                 AND movimento_itens_id = emi.id), 0)                  
                                                    AS VL_TOT_REC
                 , (SELECT codigo 
                    FROM fiscal_cst 
                    WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))) 
                                                    AS CST_PIS
                 , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                          OR fnit.fisnit_nat_rec_pis_cofins = '00'
                          OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                          ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))
                      END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                 , cc.id                            AS COD_CTA"""
            sql += """
            FROM estoque_movimento_itens            AS emi
            INNER JOIN fiscal_nf_item_tributacao    AS fnit
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo               AS fe
                    ON fe.id = fnit.encargo_id
            INNER JOIN fiscal_cst                   AS fcst
                    ON fcst.id = fnit.cst_id
            INNER JOIN ecf_cupons_capa              AS coo
                    ON coo.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
            INNER JOIN fiscal_cfop_natureza         AS fcn 
                    ON fcn.id = %(coo_natureza)s
            LEFT  JOIN contabil_natureza_fiscal     AS cnf
                    ON cnf.ctbnfi_natureza_id = fcn.id  
                    AND cnf.company_id = %(natureza_fiscal_company_id)s
            LEFT  JOIN contabil_conta               AS cc
                    ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
            WHERE coo.state = 'V'
              AND coo.tipo_cupom = 'V'
              AND emi.nf_id IS NULL
              AND coo.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
              AND fe.sigla = 'PIS'
              AND fcst.codigo IN ('04', '06', '07', '08', '09') 
              AND fnit.aliquota = 0
              AND NOT emi.item_cancelado
              AND emi.nf_id IS NULL
              AND coo.company_id IN %(company_ids)s
            GROUP BY fcst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += """, cc.id"""
            sql += """
            UNION ALL
            SELECT (emi.vr_total_item_rateado * -1) -
                   COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                 AND movimento_itens_id = emi.id), 0) -
                   COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                 AND movimento_itens_id = emi.id), 0)                  
                                                    AS VL_TOT_REC
                 , (SELECT codigo 
                    FROM fiscal_cst 
                    WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))) 
                                                    AS CST_PIS
                 , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))
                       END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                 , cc.id                            AS COD_CTA"""
            sql += """
            FROM estoque_movimento_itens            AS emi
            INNER JOIN fiscal_nf_item_tributacao    AS fnit
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo               AS fe
                    ON fe.id = fnit.encargo_id
            INNER JOIN fiscal_cst                   AS fcst
                    ON fcst.id = fnit.cst_id
            INNER JOIN ecf_cupons_capa              AS coo
                    ON coo.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
            INNER JOIN fiscal_cfop_natureza         AS fcn 
                    ON fcn.id = %(coo_natureza)s
            LEFT  JOIN contabil_natureza_fiscal     AS cnf
                    ON cnf.ctbnfi_natureza_id = fcn.id  
                    AND cnf.company_id = %(natureza_fiscal_company_id)s
            LEFT  JOIN contabil_conta               AS cc
                    ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
            WHERE coo.state = 'V'
              AND coo.tipo_cupom = 'C'
              AND emi.nf_id IS NULL
              AND coo.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
              AND fe.sigla = 'PIS'
              AND fcst.codigo IN ('04', '06', '07', '08', '09') 
              AND fnit.aliquota = 0
              AND emi.nf_id IS NULL
              AND coo.company_id  IN %(company_ids)s
            GROUP BY fcst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += """, cc.id"""
        sql += """)                                 AS tmp
        GROUP BY CST_PIS, NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """, COD_CTA"""
        cr.execute(sql % filtro)

        return cr.dictfetchall()

    def sql_M410(self, cr, filtro):
        filtro['company_ids'] = str(filtro.get('company_ids')).replace('[', '(').replace(']', ')')
        sql = """
            SELECT CST_PIS
                 , SUM(VL_REC)                          AS VL_REC
                 , NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
                 , COD_CTA
                 , DESC_COMPL"""
        sql += """
            FROM (
                SELECT 
                     (SELECT codigo 
                     FROM fiscal_cst 
                     WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))) 
                                                       AS CST_PIS   
                     , emi.vr_total_item_rateado +
                       COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                     AND movimento_itens_id = emi.id), 0) +
                       COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                     AND movimento_itens_id = emi.id), 0)                     
                                                        AS VL_REC
                     , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))
                       END                              AS NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
                 , COALESCE(cc.conta, '')               AS COD_CTA
                 , cc.name                              AS DESC_COMPL"""
        sql += """
                FROM fiscal_nf_item_tributacao          AS fnit
                INNER JOIN fiscal_cst                   AS cst
                        ON fnit.cst_id = cst.id
                INNER JOIN estoque_movimento_itens      AS emi
                        ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo               AS fe
                        ON fe.id = fnit.encargo_id
                INNER JOIN fiscal_nf_capa               AS fnc
                        ON fnc.id = emi.nf_id
                INNER JOIN fiscal_cfop                  AS cfop
                        ON cfop.id = emi.cfop_id"""
        if filtro.get('contabil_fiscal'):
            sql += """
                INNER JOIN fiscal_cfop_natureza         AS fcn 
                        ON fcn.id = fnc.natureza_id
                LEFT  JOIN contabil_natureza_fiscal     AS cnf
                        ON cnf.ctbnfi_natureza_id = fcn.id
                LEFT  JOIN contabil_conta               AS cc
                        ON cc.id = cnf.ctbnfi_conta_debito_id"""
        sql += """
                WHERE fnc.data_emissao AT TIME ZONE 'UTC' BETWEEN '%(data_ini)s' AND '%(data_fim)s'
                  AND fnc.state = '3'
                  AND fe.sigla = 'PIS' 
                  AND cst.codigo = '%(cst_pis)s'
                  AND fnc.company_id IN %(company_ids)s
                  AND cfop.codigo NOT IN ('5929', '6929')
                  AND cfop.fiscfop_sped_contribuicoes IS TRUE
                GROUP BY cst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
        if filtro.get('contabil_fiscal'):
            sql += """, cc.conta, cc.name"""
        if filtro.get('ecf_instalado'):
            sql += """
                UNION ALL
                SELECT 
                     (SELECT codigo 
                     FROM fiscal_cst 
                     WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))) 
                                                       AS CST_PIS
                     , emi.vr_total_item_rateado +
                       COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                     AND movimento_itens_id = emi.id), 0) +
                       COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                     AND movimento_itens_id = emi.id), 0)                     
                                                        AS VL_REC
                     , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                          OR fnit.fisnit_nat_rec_pis_cofins = '00'
                          OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                          ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))
                      END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                     , COALESCE(cc.conta, '')          AS COD_CTA
                     , cc.name                         AS DESC_COMPL"""
            sql += """
                FROM fiscal_nf_item_tributacao         AS fnit
                INNER JOIN fiscal_cst                  AS cst
                        ON fnit.cst_id = cst.id
                INNER JOIN estoque_movimento_itens     AS emi
                        ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo              AS fe
                        ON fe.id = fnit.encargo_id
                INNER JOIN ecf_cupons_capa             AS ecc
                        ON ecc.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
                INNER JOIN fiscal_cfop_natureza        AS fcn 
                        ON fcn.id = %(coo_natureza)s
                LEFT  JOIN contabil_natureza_fiscal    AS cnf
                        ON cnf.ctbnfi_natureza_id = fcn.id
                LEFT  JOIN contabil_conta              AS cc
                        ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
                WHERE ecc.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
                  AND cst.codigo = '%(cst_pis)s'
                  AND fe.sigla = 'PIS'
                  AND emi.item_cancelado = false
                  AND ecc.tipo_cupom = 'V'
                  AND ecc.state = 'V'
                  AND ecc.company_id IN %(company_ids)s
                  AND emi.nf_id IS NULL
                GROUP BY cst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += ", cc.conta, cc.name"""
            sql += """
                UNION ALL
                SELECT 
                     (SELECT codigo 
                     FROM fiscal_cst 
                     WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))) 
                                                       AS CST_PIS
                     , (emi.vr_total_item_rateado * -1) -
                       COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                     AND movimento_itens_id = emi.id), 0) -
                       COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                     AND movimento_itens_id = emi.id), 0)                     
                                                        AS VL_REC
                     , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'PIS'))
                       END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                     , COALESCE(cc.conta, '')           AS COD_CTA
                     , cc.name                          AS DESC_COMPL"""
            sql += """
                FROM fiscal_nf_item_tributacao          AS fnit
                INNER JOIN fiscal_cst                   AS cst
                        ON fnit.cst_id = cst.id
                INNER JOIN estoque_movimento_itens      AS emi
                        ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo               AS fe
                        ON fe.id = fnit.encargo_id
                INNER JOIN ecf_cupons_capa              AS ecc
                        ON ecc.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
                INNER JOIN fiscal_cfop_natureza         AS fcn 
                        ON fcn.id = %(coo_natureza)s
                LEFT  JOIN contabil_natureza_fiscal     AS cnf
                        ON cnf.ctbnfi_natureza_id = fcn.id
                LEFT  JOIN contabil_conta               AS cc
                        ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
                WHERE ecc.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
                  AND cst.codigo = '%(cst_pis)s'
                  AND fe.sigla = 'PIS'
                  AND ecc.tipo_cupom = 'C'
                  AND ecc.state = 'V'
                  AND ecc.company_id IN %(company_ids)s
                  AND emi.nf_id IS NULL
                GROUP BY cst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += """, cc.conta, cc.name"""
        sql += """
                )                                       AS tmp"""
        if filtro.get('contabil_fiscal'):
            sql += """
            WHERE COD_CTA = '%(cod_cta)s'"""
        sql += """
            GROUP BY CST_PIS, NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """, COD_CTA, DESC_COMPL"""
        cr.execute(sql % filtro)

        return cr.dictfetchall()

    def sql_M500(self):
        cr = self.new_cursor()
        query_nf_nao_fiscais = ''
        if self.nf_nao_fiscais:
            query_nf_nao_fiscais = """
            AND tributacao.fisnit_capa_id NOT IN {nf_nao_fiscais}
            """.format(nf_nao_fiscais=str(self.nf_nao_fiscais).replace('[', '(').replace(']', ')'))
        try:
            sql = """
                    SELECT Sum(tributacao.vr_base_calculo)                     AS VL_BC_COFINS,
                           tributacao.aliquota                                 AS ALIQ_COFINS,
                           Sum(tributacao.vr_encargo)                          AS VL_CRED,
                           cst.codigo                                          AS CST_CODIGO
                    FROM   fiscal_nf_item_tributacao tributacao
                           INNER JOIN fiscal_cst cst
                                   ON cst.id = tributacao.cst_id
                    WHERE  tributacao.fisnit_capa_id IN (SELECT id
                                                         FROM   fiscal_nf_capa
                                                         WHERE  state = '3'
                                                                AND company_id IN {company_ids}
                                                                AND tipo = 'E'
                                                                AND data_saida_entrada AT TIME ZONE 'UTC'
                                                                BETWEEN '{dt_ini}' AND '{dt_fim}'
                                                                AND modelo_nf_id NOT IN (Select id
                                                                from fiscal_modelo_nf  WHERE
                                                                fismnf_codigo >= '90'))
                           AND tributacao.encargo_id IN (SELECT id
                                                         FROM   fiscal_encargo
                                                         WHERE  sigla = 'COFINS')
                           AND cst.codigo IN( '50', '51', '52', '53',
                                              '54', '55', '56', '60',
                                              '61', '62', '63', '64',
                                              '65', '66' )
                           {query_nf_nao_fiscais}
                    GROUP  BY tributacao.cst_id,
                              tributacao.aliquota,
                              cst.codigo
            """.format(dt_ini=self.data_inicio, dt_fim=self.data_fim,
                       query_nf_nao_fiscais=query_nf_nao_fiscais, company_ids=self.company_ids)
            cr.execute(sql)
            self.M500 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
        except:
            msg_erro = trace()
            self.M500 = {'executando': False, 'dados': [], 'erro': msg_erro}
        finally:
            cr.connection.close()

    def sql_M505(self):
        codigos = "'50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63', '64', '65', '66'"
        cr = self.new_cursor()
        query_nf_nao_fiscais = ''
        if self.nf_nao_fiscais:
            query_nf_nao_fiscais = """
            AND fisnfc.id NOT IN {nf_nao_fiscais}
        """.format(nf_nao_fiscais=str(self.nf_nao_fiscais).replace('[', '(').replace(']', ')'))
        try:
            sql = """
                    SELECT CASE
                             WHEN cfop IN ( '1102', '1113', '1117', '1118',
                                            '1121', '1251', '1403', '2102',
                                            '2113', '2117', '2118', '2121',
                                            '2251', '2403', '3102', '3251', 
                                            '1652' )                        THEN '01' :: text
                             WHEN cfop IN ( '1101', '1111', '1116', '1120',
                                            '1122', '1126', '1128', '1401',
                                            '1407', '1556', '1651', '1653',
                                            '2101', '2111', '2116', '2120',
                                            '2122', '2126', '2401', '2556', '2651',
                                            '2653', '3101', '3126', '3651', '3653' ) THEN '02' ::
                             text
                             WHEN cfop IN( '1124', '1125', '1933', '2124',
                                           '2125', '2933', '1303', '2303' ) THEN '03' :: text
                             WHEN cfop IN( '1252', '1253', '1254', '1255',
                                           '1256', '1257', '2252', '2253',
                                           '2254', '2255', '2256', '2257' ) THEN '04' :: text
                             WHEN cfop IN( '1352', '1353', '1354', '1355',
                                           '1356', '1932', '2352', '2353',
                                           '2354', '2355', '2356', '2932',
                                           '3352', '3353', '3356' ) THEN '07' :: text
                             WHEN cfop IN( '1201', '1202', '1410', '1411',
                                           '2201', '2202', '2410', '2411') THEN '12' :: text
                             WHEN cfop IN( '1351', '2351', '3351' ) THEN '14' :: text
                             ELSE ''
                           END                AS codigo_natureza_credito,
                           SUM(vl_bc_cofins_tot) AS vl_bc_cofins_tot,
                           aliquota,
                           cst_cofins            AS cst_cofins
                    FROM   (SELECT CASE
                                     WHEN fisnit.movimento_itens_id IS NOT NULL THEN cfop.codigo
                                     ELSE cfop_frete.codigo
                                   END                         AS cfop,
                                   SUM(fisnit.vr_base_calculo) AS vl_bc_cofins_tot,
                                   fisnit.aliquota,
                                   fiscst.codigo               AS cst_cofins
                            FROM   fiscal_nf_item_tributacao fisnit
                                   inner join fiscal_nf_capa fisnfc
                                           ON fisnfc.id = fisnit.fisnit_capa_id
                                   inner join fiscal_encargo fisenc
                                           ON fisenc.id = fisnit.encargo_id
                                   left join estoque_movimento_itens estmov
                                          ON estmov.id = fisnit.movimento_itens_id
                                   inner join fiscal_cst fiscst
                                           ON fiscst.id = fisnit.cst_id
                                   left join fiscal_cfop cfop
                                          ON cfop.id = estmov.cfop_id
                                   left join fiscal_cfop_natureza fisnat
                                          ON fisnat.id = fisnfc.natureza_id
                                   left join fiscal_cfop cfop_frete
                                          ON cfop_frete.id = fisnat.cfop_id
                            WHERE  fisnfc.data_saida_entrada AT TIME ZONE 'UTC'
                                BETWEEN '{dt_ini}' AND '{dt_fim}'
                                   AND fisnfc.company_id IN {company_ids}
                                   AND fisnfc.tipo = 'E'
                                   AND fisenc.sigla = 'COFINS'
                                   AND fisnfc.state = '3'
                                   AND fiscst.codigo IN ( {codigos} )
                                   {query_nf_nao_fiscais}
                            GROUP  BY cfop.codigo,
                                      fisnit.aliquota,
                                      fiscst.codigo,
                                      fisnit.movimento_itens_id,
                                      cfop_frete.codigo) AS TMP
                    GROUP  BY codigo_natureza_credito,
                              aliquota,
                              cst_cofins
            """.format(dt_ini=self.data_inicio, dt_fim=self.data_fim, codigos=codigos,
                       query_nf_nao_fiscais=query_nf_nao_fiscais, company_ids=self.company_ids)
            cr.execute(sql)
            self.M505 = {'executando': False, 'dados': self.dictfetchall(cr), 'erro': False}
        except:
            msg_erro = trace()
            self.M505 = {'executando': False, 'dados': [], 'erro': msg_erro}
        finally:
            cr.connection.close()

    def sql_M600(self, cr, filtro):
        data_ini = filtro.get('data_ini')
        data_fim = filtro.get('data_fim')
        ecf_instalado = filtro.get('ecf_instalado')
        union = """
        ) AS Tmp
        """
        if ecf_instalado:
            union = """
                       UNION ALL
                       SELECT Sum(tributacao.vr_encargo)                             AS
                              VL_TOT_CONT_NC_PER,
                              Sum(tributacao.vr_reducao_base)                        AS
                              VL_RET_NC,
                              Sum(tributacao.vr_encargo -
                              tributacao.vr_reducao_base)VL_CONT_NC_REC
                       FROM   fiscal_nf_item_tributacao tributacao
                              INNER JOIN estoque_movimento_itens mov_item
                                      ON tributacao.movimento_itens_id = mov_item.id
                              INNER JOIN ecf_cupons_capa cupom
                                      ON cupom.id = mov_item.cupom_id
                                      AND mov_item.nf_id IS NULL
                       WHERE  cupom.date_cupom >= '%s'
                              AND cupom.date_cupom <= '%s'
                              AND mov_item.cupom_id IN (SELECT id
                                                        FROM   ecf_cupons_capa
                                                        WHERE  date_cupom >= '%s'
                                                               AND date_cupom <= '%s'
                                                               AND tipo_cupom = 'V'
                                                               AND state = 'V')
                              AND mov_item.nf_id IS Null
                              AND mov_item.item_cancelado = false
                              AND tributacao.encargo_id IN (SELECT id
                                                            FROM   fiscal_encargo
                                                            WHERE  sigla = 'COFINS')) AS Tmp

            """ % (data_ini, data_fim, data_ini, data_fim)
        sql = """
                SELECT Sum (vl_tot_cont_nc_per2)                                      AS
                       vl_tot_cont_nc_per,
                       Sum (vl_ret_nc2)                                               AS
                       vl_ret_nc,
                       Sum (vl_cont_nc_rec2)                                          AS
                       vl_cont_nc_rec
                FROM  (SELECT Sum(tributacao.vr_encargo)                             AS
                                    VL_TOT_CONT_NC_PER2,
                              Sum(tributacao.vr_reducao_base)                        AS
                              VL_RET_NC2,
                              Sum(tributacao.vr_encargo -
                              tributacao.vr_reducao_base)VL_CONT_NC_REC2
                       FROM   fiscal_nf_item_tributacao tributacao
                              INNER JOIN estoque_movimento_itens mov_item
                                      ON tributacao.movimento_itens_id = mov_item.id
                              INNER JOIN fiscal_nf_capa nf_capa
                                      ON nf_capa.id = mov_item.nf_id
                       WHERE  nf_capa.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                              AND fisnit_capa_id IN (SELECT id
                                                     FROM   fiscal_nf_capa
                                                     WHERE  data_emissao AT TIME ZONE 'UTC'
                                                     BETWEEN '%s' AND '%s'
                                                            AND state = '3'
                                                            AND tipo = 'S')
                %s -- union de ecf_cupons_capa quando ecf instalado
        """ % (data_ini, data_fim, data_ini, data_fim, union)
        cr.execute(sql)
        dados = cr.dictfetchall()
        return dados

    def sql_M610(self, cr, filtro, nf_nao_fiscais):
        data_ini = filtro.get('data_ini')
        data_fim = filtro.get('data_fim')
        ecf_instalado = filtro.get('ecf_instalado')
        company_ids = filtro.get('company_ids')
        if not nf_nao_fiscais:
            nf_nao_fiscais = ''
        else:
            nf_nao_fiscais = str(nf_nao_fiscais).replace('[', '(').replace(']', ')')
            nf_nao_fiscais = 'AND nf_capa.id NOT IN {nf_nao_fiscais}'.format(
                nf_nao_fiscais=nf_nao_fiscais
            )
        union = """
            )AS tmp
                            GROUP  BY cst_cofins2,
                                      aliq_cofins2
        """
        if ecf_instalado:
            union = """
                        UNION ALL
                        SELECT (SELECT codigo
                                FROM   fiscal_cst
                                WHERE  id = tributacao.cst_id)    AS cst_cofins2,
                               Sum (movimento_item.vr_total_item_rateado)  + 
                               (SELECT Sum(COALESCE(vr_icms_substituicao, 0.0)) 
                                FROM fiscal_nf_item_tributacao 
                                WHERE movimento_itens_id = movimento_item.id) 
                                                                  AS VL_REC_BRT2,
                               Sum (CASE WHEN cst.tipo = 'U' THEN 0 ELSE tributacao.vr_base_calculo 
                                    END)                          AS VL_BC_CONT2,
                               tributacao.aliquota                AS ALIQ_COFINS2,
                               Sum(tributacao.vr_encargo)         AS VL_CONT_APUR2
                        FROM   estoque_movimento_itens movimento_item
                               INNER JOIN fiscal_nf_item_tributacao tributacao
                                       ON tributacao.movimento_itens_id = movimento_item.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                               INNER JOIN ecf_cupons_capa cupom
                                       ON cupom.id = movimento_item.cupom_id
                                       AND movimento_item.nf_id  IS NULL
                               LEFT JOIN fiscal_cst cst
                                       ON cst.id = tributacao.cst_id
                        WHERE  movimento_item.cupom_id IN (SELECT id
                                                           FROM   ecf_cupons_capa
                                                           WHERE  date_cupom >= '%s'
                                                                  AND date_cupom <= '%s'
                                                                  AND state = 'V'
                                                                  AND tipo_cupom = 'V')
                               AND cupom.date_cupom >= '%s'
                               AND cupom.date_cupom <= '%s'
                               AND encargo.sigla = 'COFINS'
                               AND movimento_item.item_cancelado = false
                               AND cupom.company_id IN {company_ids}
                               AND (cst.tipo IN ('T', 'TU') 
                                    OR (cst.tipo = 'U' AND tributacao.aliquota > 0))
                        GROUP  BY tributacao.cst_id,
                                  tributacao.aliquota,
                                  movimento_item.id
                        UNION ALL
                        SELECT (SELECT codigo
                                FROM   fiscal_cst
                                WHERE  id = tributacao.cst_id)     AS cst_cofins2,
                               Sum (-movimento_item.vr_total_item_rateado)  + 
                               (SELECT Sum(COALESCE(-vr_icms_substituicao, 0.0)) 
                                FROM fiscal_nf_item_tributacao 
                                WHERE movimento_itens_id = movimento_item.id) 
                                                                   AS VL_REC_BRT2,
                               Sum (CASE WHEN cst.tipo = 'U' THEN 0 ELSE -tributacao.vr_base_calculo 
                                    END)                          AS VL_BC_CONT2,
                               tributacao.aliquota                 AS ALIQ_COFINS2,
                               Sum(-tributacao.vr_encargo)         AS VL_CONT_APUR2
                        FROM   estoque_movimento_itens movimento_item
                               INNER JOIN fiscal_nf_item_tributacao tributacao
                                       ON tributacao.movimento_itens_id = movimento_item.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                               INNER JOIN ecf_cupons_capa cupom
                                       ON cupom.id = movimento_item.cupom_id
                                       AND movimento_item.nf_id  IS NULL
                               LEFT JOIN fiscal_cst cst
                                       ON cst.id = tributacao.cst_id
                        WHERE  movimento_item.cupom_id IN (SELECT id
                                                           FROM   ecf_cupons_capa
                                                           WHERE  date_cupom >= '%s'
                                                                  AND date_cupom <= '%s'
                                                                  AND state = 'V'
                                                                  AND tipo_cupom = 'C')
                               AND cupom.date_cupom >= '%s'
                               AND cupom.date_cupom <= '%s'
                               AND encargo.sigla = 'COFINS'
                               AND cupom.company_id IN {company_ids}
                               AND (cst.tipo IN ('T', 'TU') 
                                    OR (cst.tipo = 'U' AND tributacao.aliquota > 0))
                        GROUP  BY tributacao.cst_id,
                                  tributacao.aliquota,
                                  movimento_item.id
                                  ) AS tmp
                GROUP  BY cst_cofins2,
                          aliq_cofins2
            """ % (data_ini, data_fim, data_ini, data_fim, data_ini, data_fim, data_ini, data_fim)
        sql = """
                SELECT cst_cofins2           AS cst_cofins,
                       Sum(vl_rec_brt2)   AS VL_REC_BRT,
                       Sum(vl_bc_cont2)   AS VL_BC_CONT,
                       aliq_cofins2          AS ALIQ_COFINS,
                       Sum(vl_cont_apur2) AS VL_CONT_APUR
                FROM   (SELECT (SELECT codigo
                                FROM   fiscal_cst
                                WHERE  id = tributacao.cst_id)    AS cst_cofins2,
                               Sum (movimento_item.vr_total_item_rateado)  + 
                               (SELECT Sum(COALESCE(vr_icms_substituicao, 0.0)) 
                                FROM fiscal_nf_item_tributacao 
                                WHERE movimento_itens_id = movimento_item.id) 
                                                                  AS VL_REC_BRT2,
                               Sum (CASE WHEN cst.tipo = 'U' THEN 0 ELSE tributacao.vr_base_calculo 
                                    END)                          AS VL_BC_CONT2,
                               tributacao.aliquota                AS ALIQ_COFINS2,
                               Sum(tributacao.vr_encargo)         AS VL_CONT_APUR2
                        FROM   estoque_movimento_itens movimento_item
                               INNER JOIN fiscal_nf_item_tributacao tributacao
                                       ON tributacao.movimento_itens_id = movimento_item.id
                               INNER JOIN fiscal_encargo encargo
                                       ON encargo.id = tributacao.encargo_id
                               INNER JOIN fiscal_nf_capa nf_capa
                                       ON nf_capa.id = movimento_item.nf_id
                               INNER JOIN fiscal_cfop cfop
                                       ON cfop.id = movimento_item.cfop_id
                               LEFT JOIN fiscal_cst cst
                                       ON cst.id = tributacao.cst_id
                        WHERE  movimento_item.nf_id IN (SELECT id
                                                        FROM   fiscal_nf_capa
                                                        WHERE
                               data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                               AND state = '3'
                               AND tipo = 'S')
                               AND nf_capa.data_emissao AT TIME ZONE 'UTC' BETWEEN '%s' AND '%s'
                               AND encargo.sigla = 'COFINS'
                               AND cfop.codigo NOT IN ('5929', '6929')
                               AND cfop.fiscfop_sped_contribuicoes IS TRUE
                               AND nf_capa.company_id IN {company_ids}
                               AND (cst.tipo IN ('T', 'TU') 
                                    OR (cst.tipo = 'U' AND tributacao.aliquota > 0))
                               {nf_nao_fiscais}
                        GROUP  BY tributacao.cst_id,
                                  tributacao.aliquota,
                                  movimento_item.id
                %s --union de ecf_cupons_capa quando modulo ecf instalado
        """ % (data_ini, data_fim, data_ini, data_fim, union)
        cr.execute(sql.format(company_ids=str(company_ids).replace('[', '(').replace(']', ')'),
                              nf_nao_fiscais=nf_nao_fiscais))
        dados = cr.dictfetchall()
        return dados

    def sql_M800(self, cr, filtro):
        filtro['company_ids'] = str(filtro.get('company_ids')).replace('[', '(').replace(']', ')')
        sql = """
        SELECT SUM(VL_TOT_REC)                          AS VL_TOT_REC
             , CST_COFINS
             , NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
             , COD_CTA
             """
        sql += """
        FROM (
            SELECT emi.vr_total_item_rateado +
                   COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                 AND movimento_itens_id = emi.id), 0) +
                   COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                 AND movimento_itens_id = emi.id), 0)
                                                       AS VL_TOT_REC
                 , (SELECT codigo 
                    FROM fiscal_cst 
                    WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))) 
                                                    AS CST_COFINS
                 , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))
                       END                              AS NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
                 , cc.id                                AS COD_CTA"""
        sql += """
            FROM estoque_movimento_itens                AS emi
            INNER JOIN fiscal_cfop                      AS fcfop
                    ON fcfop.id = emi.cfop_id
            INNER JOIN fiscal_nf_capa                   AS fnc
                    ON fnc.id = emi.nf_id
            INNER JOIN fiscal_nf_item_tributacao        AS fnit
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo                   AS fe
                    ON fe.id = fnit.encargo_id
            INNER JOIN fiscal_cst                       AS fcst
                    ON fcst.id = fnit.cst_id"""
        if filtro.get('contabil_fiscal'):
            sql += """
            INNER JOIN fiscal_cfop_natureza             AS fcn 
                    ON fcn.id = fnc.natureza_id
            LEFT  JOIN contabil_natureza_fiscal         AS cnf
                    ON cnf.ctbnfi_natureza_id = fcn.id 
                    AND cnf.company_id = %(natureza_fiscal_company_id)s
            LEFT  JOIN contabil_conta                   AS cc
                    ON cc.id = cnf.ctbnfi_conta_debito_id"""
        sql += """
            WHERE fcfop.codigo NOT IN ('5929', '6929')
              AND fcfop.fiscfop_sped_contribuicoes IS TRUE
              AND fnc.company_id IN %(company_ids)s
              AND fnc.data_emissao AT TIME ZONE 'UTC' BETWEEN '%(data_ini)s' AND '%(data_fim)s'
              AND fnc.state = '3'
              AND fnc.tipo = 'S'
              AND fe.sigla = 'COFINS'
              AND fcst.codigo IN ('04', '06', '07', '08', '09') 
            GROUP BY fcst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
        if filtro.get('contabil_fiscal'):
            sql += """, cc.id"""
        if filtro.get('ecf_instalado'):
            sql += """
            UNION ALL
            SELECT emi.vr_total_item_rateado +
                   COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                 AND movimento_itens_id = emi.id), 0) +
                   COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                 AND movimento_itens_id = emi.id), 0)
                                                       AS VL_TOT_REC
                 , (SELECT codigo 
                    FROM fiscal_cst 
                    WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))) 
                                                    AS CST_COFINS
                 , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                          OR fnit.fisnit_nat_rec_pis_cofins = '00'
                          OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                          ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))
                      END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                 , cc.id                                AS COD_CTA"""
            sql += """
            FROM estoque_movimento_itens                AS emi
            INNER JOIN fiscal_nf_item_tributacao        AS fnit
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo                   AS fe
                    ON fe.id = fnit.encargo_id
            INNER JOIN fiscal_cst                       AS fcst
                    ON fcst.id = fnit.cst_id
            INNER JOIN ecf_cupons_capa                  AS coo
                    ON coo.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
            INNER JOIN fiscal_cfop_natureza             AS fcn 
                    ON fcn.id = %(coo_natureza)s
            LEFT  JOIN contabil_natureza_fiscal         AS cnf
                    ON cnf.ctbnfi_natureza_id = fcn.id 
                    AND cnf.company_id = %(natureza_fiscal_company_id)s
            LEFT  JOIN contabil_conta                   AS cc
                    ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
            WHERE coo.state = 'V'
              AND coo.tipo_cupom = 'V'
              AND emi.nf_id IS NULL
              AND coo.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
              AND fe.sigla = 'COFINS' 
              AND fcst.codigo IN ('04', '06', '07', '08', '09') 
              AND fnit.aliquota = 0
              AND NOT emi.item_cancelado
              AND emi.nf_id IS NULL
              AND coo.company_id IN %(company_ids)s
            GROUP BY fcst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += """, cc.id"""
            sql += """
            UNION ALL
            SELECT (emi.vr_total_item_rateado * -1) -
                   COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                 AND movimento_itens_id = emi.id), 0) -
                   COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                             FROM fiscal_nf_item_tributacao 
                             WHERE 
                                 encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                 AND movimento_itens_id = emi.id), 0)
                                                       AS VL_TOT_REC
                 , (SELECT codigo 
                    FROM fiscal_cst 
                    WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))) 
                                                    AS CST_COFINS
                 , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))
                       END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                 , cc.id                                AS COD_CTA"""
            sql += """
            FROM estoque_movimento_itens                AS emi
            INNER JOIN fiscal_nf_item_tributacao        AS fnit
                    ON fnit.movimento_itens_id = emi.id
            INNER JOIN fiscal_encargo                   AS fe
                    ON fe.id = fnit.encargo_id
            INNER JOIN fiscal_cst                       AS fcst
                    ON fcst.id = fnit.cst_id
            INNER JOIN ecf_cupons_capa                  AS coo
                    ON coo.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
            INNER JOIN fiscal_cfop_natureza             AS fcn 
                    ON fcn.id = %(coo_natureza)s
            LEFT  JOIN contabil_natureza_fiscal         AS cnf
                    ON cnf.ctbnfi_natureza_id = fcn.id 
                    AND cnf.company_id = %(natureza_fiscal_company_id)s
            LEFT  JOIN contabil_conta                   AS cc
                    ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
            WHERE coo.state = 'V'
              AND coo.tipo_cupom = 'C'
              AND emi.nf_id IS NULL
              AND coo.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
              AND fe.sigla = 'COFINS' 
              AND fcst.codigo IN ('04', '06', '07', '08', '09') 
              AND fnit.aliquota = 0
              AND emi.nf_id IS NULL
              AND coo.company_id  IN %(company_ids)s
            GROUP BY fcst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += """, cc.id"""
        sql += """)                                     AS tmp
        GROUP  BY CST_COFINS, NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """, COD_CTA"""
        cr.execute(sql % filtro)

        return cr.dictfetchall()

    def sql_M810(self, cr, filtro):
        filtro['company_ids'] = str(filtro.get('company_ids')).replace('[', '(').replace(']', ')')
        sql = """
            SELECT CST_COFINS
                 , SUM(VL_REC)                          AS VL_REC
                 , NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
                 , COD_CTA
                 , DESC_COMPL"""
        sql += """
            FROM (
                SELECT 
                     (SELECT codigo 
                     FROM fiscal_cst 
                     WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))) 
                                                       AS CST_COFINS
                     , emi.vr_total_item_rateado +
                       COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                     AND movimento_itens_id = emi.id), 0) +
                       COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                     AND movimento_itens_id = emi.id), 0)                     
                                                        AS VL_REC
                     , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))
                       END                              AS NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """
                     , COALESCE(cc.conta, '')           AS COD_CTA
                     , cc.name                          AS DESC_COMPL"""
        sql += """
                FROM fiscal_nf_item_tributacao          AS fnit
                INNER JOIN fiscal_cst                   AS cst
                        ON fnit.cst_id = cst.id
                INNER JOIN estoque_movimento_itens      AS emi
                        ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo               AS fe
                        ON fe.id = fnit.encargo_id
                INNER JOIN fiscal_nf_capa               AS fnc
                        ON fnc.id = emi.nf_id
                INNER JOIN fiscal_cfop                  AS cfop
                        ON cfop.id = emi.cfop_id"""
        if filtro.get('contabil_fiscal'):
            sql += """
                INNER JOIN fiscal_cfop_natureza         AS fcn 
                        ON fcn.id = fnc.natureza_id
                LEFT  JOIN contabil_natureza_fiscal     AS cnf
                        ON cnf.ctbnfi_natureza_id = fcn.id
                LEFT  JOIN contabil_conta               AS cc
                        ON cc.id = cnf.ctbnfi_conta_debito_id"""
        sql += """
                WHERE fnc.data_emissao AT TIME ZONE 'UTC' BETWEEN '%(data_ini)s' AND '%(data_fim)s'
                  AND fnc.state = '3'
                  AND cst.codigo = '%(cst_cofins)s'
                  AND fe.sigla = 'COFINS'
                  AND fnc.company_id IN %(company_ids)s
                  AND cfop.codigo NOT IN ('5929', '6929')
                  AND cfop.fiscfop_sped_contribuicoes IS TRUE
                GROUP BY cst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
        if filtro.get('contabil_fiscal'):
            sql += """, cc.conta, cc.name"""
        if filtro.get('ecf_instalado'):
            sql += """
                UNION ALL
                SELECT 
                     (SELECT codigo 
                     FROM fiscal_cst 
                     WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))) 
                                                       AS CST_COFINS
                     , emi.vr_total_item_rateado +
                       COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                     AND movimento_itens_id = emi.id), 0) +
                       COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                     AND movimento_itens_id = emi.id), 0)                     
                                                        AS VL_REC
                     , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))
                       END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                     , COALESCE(cc.conta, '')           AS COD_CTA
                     , cc.name                          AS DESC_COMPL"""
            sql += """
                FROM fiscal_nf_item_tributacao          AS fnit
                INNER JOIN fiscal_cst                   AS cst
                        ON fnit.cst_id = cst.id
                INNER JOIN estoque_movimento_itens      AS emi
                        ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo               AS fe
                        ON fe.id = fnit.encargo_id
                INNER JOIN ecf_cupons_capa              AS ecc
                        ON ecc.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
                INNER JOIN fiscal_cfop_natureza         AS fcn 
                        ON fcn.id = %(coo_natureza)s
                LEFT  JOIN contabil_natureza_fiscal     AS cnf
                        ON cnf.ctbnfi_natureza_id = fcn.id
                LEFT  JOIN contabil_conta               AS cc
                        ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
                WHERE ecc.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
                  AND cst.codigo = '%(cst_cofins)s'
                  AND fe.sigla = 'COFINS'
                  AND emi.item_cancelado = false
                  AND ecc.tipo_cupom = 'V'
                  AND ecc.state = 'V'
                  AND ecc.company_id IN %(company_ids)s
                  AND emi.nf_id IS NULL
                GROUP BY cst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += ", cc.conta, cc.name"""
            sql += """
                UNION ALL
                SELECT 
                     (SELECT codigo 
                     FROM fiscal_cst 
                     WHERE id IN (SELECT cst_id 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))) 
                                                       AS CST_COFINS
                     , (emi.vr_total_item_rateado * -1) -
                       COALESCE((SELECT sum(coalesce(vr_encargo, 0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'IPI')
                                     AND movimento_itens_id = emi.id), 0) -
                       COALESCE((SELECT sum(vr_icms_substituicao + COALESCE(fisnit_vr_fcp_st, 0.0)) 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE 
                                     encargo_id = (SELECT id FROM fiscal_encargo WHERE sigla = 'ICMS')
                                     AND movimento_itens_id = emi.id), 0)                     
                                                        AS VL_REC
                     , CASE WHEN fnit.fisnit_nat_rec_pis_cofins = '0'
                            OR fnit.fisnit_nat_rec_pis_cofins = '00'
                            OR fnit.fisnit_nat_rec_pis_cofins = '000' THEN ''
                            ELSE (SELECT lpad(min(fisnit_nat_rec_pis_cofins), 3, '0') 
                                 FROM fiscal_nf_item_tributacao 
                                 WHERE movimento_itens_id = emi.id 
                                       AND encargo_id IN (SELECT id 
                                                          FROM fiscal_encargo 
                                                          WHERE sigla = 'COFINS'))
                       END                              AS NAT_REC"""
            if filtro.get('contabil_fiscal'):
                sql += """
                     , COALESCE(cc.conta, '')           AS COD_CTA
                     , cc.name                          AS DESC_COMPL"""
            sql += """
                FROM fiscal_nf_item_tributacao          AS fnit
                INNER JOIN fiscal_cst                   AS cst
                        ON fnit.cst_id = cst.id
                INNER JOIN estoque_movimento_itens      AS emi
                        ON emi.id = fnit.movimento_itens_id
                INNER JOIN fiscal_encargo               AS fe
                        ON fe.id = fnit.encargo_id
                INNER JOIN ecf_cupons_capa              AS ecc
                        ON ecc.id = emi.cupom_id"""
            if filtro.get('contabil_fiscal'):
                sql += """
                INNER JOIN fiscal_cfop_natureza         AS fcn 
                        ON fcn.id = %(coo_natureza)s
                LEFT  JOIN contabil_natureza_fiscal     AS cnf
                        ON cnf.ctbnfi_natureza_id = fcn.id
                LEFT  JOIN contabil_conta               AS cc
                        ON cc.id = cnf.ctbnfi_conta_debito_id"""
            sql += """
                WHERE ecc.date_cupom BETWEEN '%(data_ini)s' AND '%(data_fim)s'
                  AND cst.codigo = '%(cst_cofins)s'
                  AND fe.sigla = 'COFINS'
                  AND ecc.tipo_cupom = 'C'
                  AND ecc.state = 'V'
                  AND ecc.company_id IN %(company_ids)s
                  AND emi.nf_id IS NULL
                GROUP BY cst.codigo, fnit.fisnit_nat_rec_pis_cofins, emi.id"""
            if filtro.get('contabil_fiscal'):
                sql += """, cc.conta, cc.name"""
        sql += """
                )                                       AS tmp"""
        if filtro.get('contabil_fiscal'):
            sql += """
            WHERE COD_CTA = '%(cod_cta)s'"""
        sql += """
            GROUP BY CST_COFINS, NAT_REC"""
        if filtro.get('contabil_fiscal'):
            sql += """, COD_CTA, DESC_COMPL"""
        cr.execute(sql % filtro)

        return cr.dictfetchall()

    @staticmethod
    def dictfetchall(cr):
        """Retorna todas as linhas de um cursor como um dicionario"""
        desc = cr.description
        return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
