# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
from geraldo import ReportBand, ObjectValue, Label, Image, Rect, Line
from reportlab.lib.pagesizes import A4, landscape
from reportlab.lib.colors import HexColor
from reportlab.lib.units import cm
from StringIO import StringIO
import qrcode
import os

DIRNAME = os.path.dirname(__file__)

# Margens e tamanhos padronizados
PAGE_SIZE = A4
PORTRAIT = PAGE_SIZE
LANDSCAPE = landscape(PAGE_SIZE)
MARGIN_TOP = .5*cm
MARGIN_BOTTOM = .5*cm
MARGIN_LEFT = .5*cm
MARGIN_RIGHT = .5*cm
WIDTH, HEIGHT = 0, 1
WIDTH_PORTRAIT = PORTRAIT[WIDTH] - MARGIN_LEFT - MARGIN_RIGHT
WIDTH_LANDSCAPE = LANDSCAPE[WIDTH] - MARGIN_LEFT - MARGIN_RIGHT
HEIGHT_PORTRAIT = PORTRAIT[HEIGHT] - MARGIN_TOP - MARGIN_BOTTOM
HEIGHT_LANDSCAPE = LANDSCAPE[HEIGHT] - MARGIN_TOP - MARGIN_BOTTOM

# Fontes
FONTS = {
    u'DejaVu Sans Mono': (
        (u'DejaVu Sans Mono', DIRNAME + u'/fonts/DejaVuSansMono.ttf', False, False),
        (u'DejaVu Sans Mono Bold', DIRNAME + u'/fonts/DejaVuSansMono-Bold.ttf', True, False),
        (u'DejaVu Sans Mono Italic', DIRNAME + u'/fonts/DejaVuSansMono-Oblique.ttf', False, True),
        (u'DejaVu Sans Mono Bold Italic', DIRNAME + u'/fonts/DejaVuSansMono-BoldOblique.ttf', True,
         True),
    ),
    u'Roboto Condensed': (
        (u'Roboto Condensed', DIRNAME + u'/fonts/RobotoCondensed-Regular.ttf', False, False),
        (u'Roboto Condensed Bold', DIRNAME + u'/fonts/RobotoCondensed-Bold.ttf', True, False),
        (u'Roboto Condensed Italic', DIRNAME + u'/fonts/RobotoCondensed-Italic.ttf', False, True),
        (u'Roboto Condensed Bold Italic', DIRNAME + u'/fonts/RobotoCondensed-BoldItalic.ttf', True,
         True)
    )
}

# Estilos
NORMAL = u'Roboto Condensed'
BOLD = NORMAL + u' Bold'
OBLIQUE = NORMAL + u' Italic'
BOLD_OBLIQUE = NORMAL + u' Bold Italic'
SIZE = 9
SIZE_1 = SIZE + 1
SIZE_2 = SIZE + 2
SIZE_3 = SIZE + 3
SIZE_4 = SIZE + 4
SIZE_4_5 = SIZE_4 + .5
SIZE_5 = SIZE_4 + 1
SIZE_12 = SIZE * 2
SIZE_13 = SIZE * 2 + 1
SIZE_18 = SIZE * 3
SIZE_40 = SIZE_2 * 5

# Cores
RED1 = HexColor(0xff9393)
GRAY1 = HexColor(0x939393)
SILVER = HexColor(0xC0C0C0)   # rgb(192, 192, 192)
GRAY = HexColor(0x808080)     # rgb(128, 128, 128)
BLACK = HexColor(0x000000)    # rgb(0, 0, 0)
RED = HexColor(0xff0000)      # rgb(255, 0, 0)
MAROON = HexColor(0x800000)   # rgb(128, 0, 0)
YELLOW = HexColor(0xffff00)   # rgb(255, 255, 0)
OLIVE = HexColor(0x808000)    # rgb(128, 128, 0)
LIME = HexColor(0x00ff00)     # rgb(0, 255, 0)
GREEN = HexColor(0x008000)    # rgb(0, 128, 0)
AQUA = HexColor(0x00ffff)     # rgb(0, 255, 255)
TEAL = HexColor(0x008080)     # rgb(0, 128, 128)
BLUE = HexColor(0x0000ff)     # rgb(0, 0, 255)
NAVY = HexColor(0x000080)     # rgb(0, 0, 128)
FUCHSIA = HexColor(0xff00ff)  # rgb(255, 0, 255)
PURPLE = HexColor(0x800080)   # rgb(128, 0, 128)
WHITE = HexColor(0xffffff)    # rgb(255, 255, 255)

# Padroes
DEFAULT = dict(fontName=NORMAL, fontSize=SIZE, alignment=TA_LEFT)
DEFAULT_WHITE = dict(fontName=NORMAL, fontSize=SIZE, alignment=TA_LEFT, textColor=WHITE)
DEFAULT_RIGHT = dict(fontName=NORMAL, fontSize=SIZE, alignment=TA_RIGHT)
DEFAULT_CENTER = dict(fontName=NORMAL, fontSize=SIZE, alignment=TA_CENTER)
DEFAULT_BOLD = dict(fontName=BOLD, fontSize=SIZE, alignment=TA_LEFT)
DEFAULT_BOLD_RIGHT = dict(fontName=BOLD, fontSize=SIZE, alignment=TA_RIGHT)
DEFAULT_BOLD_CENTER = dict(fontName=BOLD, fontSize=SIZE, alignment=TA_CENTER)
OCCURRENCE = dict(fontName=NORMAL, fontSize=SIZE, alignment=TA_LEFT, textColor=BLACK,
                  padding_left=2, padding_top=2)
OCCURRENCE_7 = dict(fontName=NORMAL, fontSize=SIZE_1, alignment=TA_LEFT, textColor=BLACK,
                    padding_left=2, padding_top=2)
OCCURRENCE_7B = dict(fontName=BOLD, fontSize=SIZE_1, alignment=TA_LEFT, textColor=BLACK,
                     padding_left=2, padding_top=2)
OCCURRENCE_8B = dict(fontName=BOLD, fontSize=SIZE_2, alignment=TA_LEFT, textColor=BLACK)
NOTE = dict(fontName=NORMAL, fontSize=SIZE - 1, alignment=TA_LEFT)
COMPANY_NAME = dict(fontName=BOLD, fontSize=SIZE_2, alignment=TA_LEFT)


class RectDamdfe(Rect):
    def __init__(self):
        super(RectDamdfe, self).__init__()


class LabelDamdfe(Label):
    def __init__(self):
        super(LabelDamdfe, self).__init__()
        self.height = .8*cm
        self.padding_top = 0
        self.padding_bottom = 0
        self.padding_left = 0
        self.padding_right = 0


class FieldDamdfe(ObjectValue):
    def __init__(self):
        super(FieldDamdfe, self).__init__()
        self.height = .8*cm
        self.padding_top = 0
        self.padding_bottom = 0
        self.padding_left = 0
        self.padding_right = 0


class ImageDamdfe(Image):
    def __init__(self):
        super(ImageDamdfe, self).__init__()
        self.width = 2.5*cm
        self.height = 2.5*cm
        self.left = 0
        self.top = 0


class BandDamdfe(ReportBand):
    def __init__(self):
        super(BandDamdfe, self).__init__()

    @staticmethod
    def _add_label(text, top, left, width, height=None, line=True):
        lbl = LabelDamdfe()
        lbl.text = text or u''
        lbl.top = top*cm
        lbl.left = left*cm
        lbl.width = width*cm
        lbl.height = height and height*cm or lbl.height

        return lbl

    @staticmethod
    def _add_field(attr_name, top, left, width, height=None):
        fld = FieldDamdfe()
        fld.attribute_name = attr_name
        fld.top = top*cm
        fld.left = left*cm
        fld.width = width*cm
        fld.height = height and height*cm or fld.height

        return fld

    @staticmethod
    def _add_image(filename, top, left, width, height=None):
        img = ImageDamdfe()
        img.filename = filename.len and filename or None
        img.top = top*cm
        img.left = left*cm
        img.width = width*cm
        img.height = height and height*cm or img.height

        return img

    def add_tag(self, text, attr_name, top, left, width, height=None, style=None, visible=True,
                border=None, fill=HexColor(0xFFFFFF), radius=1):
        lbl = self._add_label(text, top, left, width, height)
        lbl.padding_left = 2
        lbl.style = style.copy() or DEFAULT
        lbl.style['fontSize'] *= .9
        lbl.visible = bool(visible)
        fld = self._add_field(attr_name, top + .5, left, width, height)
        fld.padding_left = 2
        fld.style = style.copy() or DEFAULT
        fld.visible = bool(visible)
        height = height and height * cm or .1*cm
        rct = Rect(top=lbl.top, left=lbl.left, width=lbl.width, height=height, stroke_width=.3,
                   stroke_color=border or fill, fill=True, fill_color=fill, radius=radius)
        self.elements.append(rct)
        self.elements.append(lbl)
        self.elements.append(fld)

        return rct, lbl, fld

    def add_label(self, text, top, left, width, height=None, style=None, visible=True, border=None):
        lbl = self._add_label(text, top, left, width, height, border)
        lbl.style = style or DEFAULT
        lbl.visible = bool(visible)
        self.elements.append(lbl)

        return lbl

    def add_field(self, attr_name, top, left, width, height=None, style=None, visible=True):
        fld = self._add_field(attr_name, top, left, width, height)
        fld.style = style or DEFAULT
        fld.visible = bool(visible)
        self.elements.append(fld)

        return fld

    def add_image(self, filename, top, left, width, height=None, visible=True):
        img = self._add_image(filename, top, left, width, height)
        img.visible = bool(visible)
        self.elements.append(img)

        return img

    def add_qrcode(self, value, top, left, width=3, height=3):
        qr = qrcode.QRCode(version=1, error_correction=qrcode.constants.ERROR_CORRECT_L)
        qr.add_data(value)
        qr.make()
        img = qr.make_image()
        qr_img = StringIO()
        img.save(qr_img)
        qrc = self._add_image(qr_img, top, left, width, height)
        self.elements.append(qrc)


LOGO = """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"""

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
